!$Id: avhrr_l1b_calib_plot.f,v 1.3 2005/06/17 09:58:42 frna Exp $

       program avhrr_l1b_calib_plot

! Purpose: read AVHRR full resolution 1b file to display the results using GMT
!          including
!	   application of the calibration
!          calculation of reflectance factors (VIS channels 1,2,3A)
!          calculation of brightness temperatures (IR channels 3B,4,5)
!          calculation of geolocation for each pixel
!          Data are written into formatted file, 
!          which will be read by the GMT script
!-------------------------------------------------------------------------------------
! Author: Joerg Ackermann, July 2000
! updated: Rainer Schraidt July 2001
! updated: N Atkinson Nov 2004, modify includes assuming in-situ compilation
! updated: N Atkinson May 2005, correct use of band coefs, allowing for
!                               different 1b format definitions
! updated: N Atkinson Nov 2006, change i.and.j to iand(i,j), and use #include
! updated: N Atkinson Jul 2014, get it working when every line has navigation
!-------------------------------------------------------------------------------------

C******************************************************************************
C*****Parameters you may want to modify****************************************
      
! Includes

#include "avh1b.h"
#include "reclopt.h"
	
c******************************************************************************	
c******************************************************************************


! Declarations


c*****AVHRR L1B Input file, produced by AAPP
        character*80 L1b_file
c*****subsampling factor for AVHRR data plot
	integer sample
C*****First navigation block to be plotted
        integer iblou
C*****Last navigation block to be plotted
        integer ibloo

c--------name of output data file-----------------
      character*80 satellite_data
            parameter(satellite_data='satellite_data.txt')
c-----channel number---------------            
      integer*4 ch_num
c-----check channel status--------------
c------result=ibitresx (with x: channel number)--
c------for the selected channel, ibitresx should not be zero----
	integer*4 i3abit,i3bbit,ibitres3a,ibitres3b,
     $ i1bit,ibitres1,i2bit,ibitres2,i4bit,ibitres4,
     $ i5bit,ibitres5

c-------pixel index for each scanline----------
      integer*4 ipix

c-----------start and end of the scene--------
	integer*4 s_hour,s_min,s_sec,e_hour,e_min,e_sec
c----parameters----------------
c-------nn:  number of pixels per scanline----------
c-------ntie:number of tie points per scan line-----
c-------nval:grid spacing between tie points per scan line-----
c-------noffset:number of pixels at scan line edge, which are--
c---------------extrapolated and not interpolated--------------
c-------nmaxline: Determines the maximum of blocks (i.e. 20 
c                 consecutive scan lines; with nmax=500, maximum
c                 pixel number per scene is 500x20x2048.)----
	parameter(nn=2048,ntie=51,nval=40,noffset=25,nmaxline=500)
c-------work fields for the interpolation routine-----------
c--------input: ueb(ntie)-----
c--------output: sol(nn)------
	common /ueber/ ueb(ntie),sol(nn)
	real xint(nn),yint(nn)
c--------full resolution output field, either brightness temperatures
c        (channel 3b, 4, 5) or reflectance factors (channels 1, 2, 3a)
c----note: the second field dimension must be at least as large as the
c        pixels distance between two subsequent scan lines (i.e. 21)
 	real tempfield(nn,nval+1,5),critfield(nn,nval+1)
c--------geographical positions-----
c         full resolution (each pixel) along scan line and--------
c        (nominally) each 20th scan line along flight direction---
	real geofilat(nn,nmaxline)
	real geofilon(nn,nmaxline)
c--------geographical positions-----
c         full resolution (each pixel) across and along the scan line--
	real geolatint(nn,nval+1)
	real geolonint(nn,nval+1)
c-------work fields---------
	real temphilf(nn,5),crithilf(nn)
	integer icalfield(nmaxline)

! Code starts here
c****************************
c----constants for the calculation of Planck's function-----
	    c1=0.00001191062
	    c2=1.4387863
	    pi=4.*atan(1.)
c------initalisation of the integers for the channel quality check-
	    i1bit=8192
	    i2bit=4096
 	    i3abit=2048
	    i3bbit=1024
	    i4bit=512
	    i5bit=256
	    ibitres1=0
	    ibitres2=0 
	    ibitres3a=0
	    ibitres3b=0
	    ibitres4=0
	    ibitres5=0

c-----read plot parameters---------
      print*,'Enter AVHRR l1b file name:'
      read(*,111) L1b_file
 111  format(a80)
      write(*,111) L1b_file
      print*,'Enter channel/parameter number:'
      read*,ch_num
      print*, ch_num
      if(ch_num.lt.0.or.ch_num.gt.5) goto 888

      print*,'Enter subsampling factor:'
      read*,sample
      print*, sample
      print*,'Enter lower block number:'
      read*,iblou
      print*, iblou
      print*,'Enter upper block number:'
      read*,ibloo
      print*, ibloo

c--------Initialisation of the block number variable-----
	iblock=iblou
    
      open(unit=10,
     1     file=L1b_file,
     2     form='unformatted',
     3     access='direct',
     4     recl=reclavh/rldiv,
     5     status='old')


! Read Header Information 

      read(10,rec=1,err=999) avh_hd

	s_hour=avh_h_startdatatime/1.e3/60./60.
	s_min=(avh_h_startdatatime/1.e3-s_hour*60*60)/60
	s_sec=(avh_h_startdatatime/1.e3-s_hour*60*60-s_min*60)

	e_hour=avh_h_enddatatime/1.e3/60./60.
	e_min=(avh_h_enddatatime/1.e3-e_hour*60*60)/60
	e_sec=(avh_h_enddatatime/1.e3-e_hour*60*60-e_min*60)

        print*,'NOAA',avh_h_satid,' ',
     $   avh_h_startdatayr,avh_h_startdatady,
     $   s_hour,s_min,s_sec,e_hour,e_min,e_sec 
c***************************
	write(*,*)'avh_h_dataname ',avh_h_dataname
	write(*,*)'avh_h_prblkid ',avh_h_prblkid
	write(*,*)'avh_h_instid',avh_h_instid
	write(*,*)'avh_h_satid',avh_h_satid
	write(*,*)'avh_h_datatyp',avh_h_datatyp
	write(*,*)'avh_h_siteid ',avh_h_siteid

c----check for channel quality:
c     ibitresx .ne. 0 => enable	
c     ibitresx .eq. 0 => disable	
	ibitres1=iand(avh_h_inststat1,i1bit)
	ibitres2=iand(avh_h_inststat1,i2bit)
	ibitres3a=iand(avh_h_inststat1,i3abit)
	ibitres3b=iand(avh_h_inststat1,i3bbit)
	ibitres4=iand(avh_h_inststat1,i4bit)
	ibitres5=iand(avh_h_inststat1,i5bit)
	write(*,*)'status',avh_h_inststat1
	write(*,*)'status1',ibitres1
	write(*,*)'status2',ibitres2
	write(*,*)'status3a',ibitres3a
	write(*,*)'status3b',ibitres3b
	write(*,*)'status4',ibitres4
	write(*,*)'status5',ibitres5

! Read Data Records

      open(unit=30,
     1     file=satellite_data,
     2     form='formatted',
     3     status='unknown')
c-----initialisation of landmark pixel counter--------
	ijk=0
c-----initialisation of the block number variable-----
	ik1=0
c-------loop over the scan lines-------------
      do irec=2,avh_h_scnlin
c****Identification of calibration line******************
	if (avh_pos(1,1) .ne. 0. .and. avh_pos(2,1) .ne. 0.)then
	ik1=ik1+1
	icalfield(ik1)=irec
c*********latitude interpolation along scan line********
	do i=1,ntie
	 ueb(i)=avh_pos(1,i)
	enddo
	call interpol
	do i=1,nn
	 xint(i)=sol(i)
	 geofilat(i,ik1)=xint(i)
	enddo
c+++++++++++
c*********longitude interpolation along scan line********
	do i=1,ntie
	 ueb(i)=avh_pos(2,i)
	enddo
	call interpol
	do i=1,nn
	 yint(i)=sol(i)
	 geofilon(i,ik1)=yint(i)
	enddo
c+++++++++++
c+++++++calibration line endif++++++++++
	endif

c-------read AVHRR data from input file--
        read(10,rec=irec,err=889) avh_dt

!this program was written when AVHRR used tie points every 20 lines: simulate that here
!to avoid re-writing code
          if (mod(irec,20) .NE. 0) then
            avh_pos(1,1) = 0
            avh_pos(2,1) = 0
          endif
         
! Write data record and latitude,longitude information formatted into file

c--------compute brightness temperatures or reflectance factors--
c--------for each pixel of the considered scan line---------------
           do ipix=1,avh_mxpix
	if(avh_hrpt(ch_num,ipix) .gt. -9999) then
c+++++++++++++IR+++++++++++++++++++++++++++++
	  if(ch_num .gt. 3 .or. 
     $      (ch_num .eq. 3 .and. ibitres3b .ne. 0))then
c---------count-to-radiance-conversion-------------
	    pixelrad=avh_calir(3,1,ch_num-2)+
     $      avh_calir(2,1,ch_num-2)*
     $      avh_hrpt(ch_num,ipix)+
     $      avh_calir(1,1,ch_num-2)*1.e-03*
     $      avh_hrpt(ch_num,ipix)**2.
	    pixelrad=pixelrad*1.e-06
c---------radiance-to-temperature-conversion----------
	    rmue=avh_h_radtempcnv(1,ch_num-2)*1.e-03
	    if(ch_num-2 .eq. 1)rmue=rmue*10.
	    tast=c2*rmue/(alog(1.+(c1*rmue**3.)/pixelrad))
c---------NESDIS 1b format differs from AAPP(vers 1-4), so check the sign of
c---------the band correction coefficient
            if (avh_h_radtempcnv(2,ch_num-2).lt.0) then !rad->temp        
              tebb=avh_h_radtempcnv(2,ch_num-2)*1.e-05 +
     $        avh_h_radtempcnv(3,ch_num-2)*1.e-06*tast
            else                                 !avh_h_radtempcnv is temp->rad
              tebb=-avh_h_radtempcnv(2,ch_num-2)*1.e-05/
     $        (avh_h_radtempcnv(3,ch_num-2)*1.e-06) +
     $        1/(avh_h_radtempcnv(3,ch_num-2)*1.e-06)*tast
            endif
	    if(ik1 .ne. 0)then
	    if(ik1 .ne. iblock .and. ik1-1 .eq. iblock)then
	      temphilf(ipix,ch_num)=tebb
c	      write(30,*)temphilf(ipix,ch_num),ipix,ch_num
	    endif
   	    if(ik1 .eq. iblock)then
	      tempfield(ipix,irec-icalfield(ik1)+1,ch_num)=tebb
c	      write(30,*)tempfield(ipix,irec-icalfield(ik1)+1,ch_num),
c     $       ipix,ch_num
	    endif
          endif
c+++++++++++endif IR+++++++++++++++++++++
	endif
c+++++++++++++VIS++++++++++++++++++++++++++++
	if(ch_num .lt. 3 .or. 
     $   (ch_num .eq. 3 .and. ibitres3a .ne. 0))then
	 if(avh_hrpt(ch_num,ipix) 
     $     .lt. avh_calvis(5,3,ch_num))then
	 pixelrad=real(avh_calvis(1,3,ch_num))*
     $   real(avh_hrpt(ch_num,ipix))*1.e-10+
     $   real(avh_calvis(2,3,ch_num))*1.e-07
	endif
	 if(avh_hrpt(ch_num,ipix) 
     $     .ge. avh_calvis(5,3,ch_num))then
	 pixelrad=real(avh_calvis(3,3,ch_num))*
     $   real(avh_hrpt(ch_num,ipix))*1.e-10+
     $   real(avh_calvis(4,3,ch_num))*1.e-07
	endif
	 pixelrad=1000.*pi*(pixelrad/avh_h_albcnv(1,ch_num))
	if(ik1 .ne. 0)then
	if(ik1 .ne. iblock .and. ik1-1 .eq. iblock)then
	temphilf(ipix,ch_num)=pixelrad
c	write(30,*)temphilf(ipix,ch_num),ipix,ch_num
	endif
 	if(ik1 .eq. iblock)then
	tempfield(ipix,irec-icalfield(ik1)+1,ch_num)=pixelrad
c	write(30,*)tempfield(ipix,irec-icalfield(ik1)+1,ch_num),
c     $   ipix,ch_num
 	endif
	endif
c+++++++++++endif VIS++++++++++++++++++++
	endif
c+++++++++endif threshold++++++++++++++++
	endif
c+vegetation index and brightness temperature difference++++
	if(ik1 .ne. 0)then
	if(ik1 .ne. iblock .and. ik1-1 .eq. iblock)then
  	crithilf(ipix)=temphilf(ipix,ch_num)
	endif
 	if(ik1 .eq. iblock)then
	critfield(ipix,irec-icalfield(ik1)+1)=
     $	tempfield(ipix,irec-icalfield(ik1)+1,ch_num)
 	endif
	endif
c+++++++++enddo ipix++++++++++++++++
           enddo
	if(ik1 .ne. iblock .and. ik1-1 .eq. iblock)then
	do ii=1,nn
	 xdif=(geofilon(ii,iblock)-geofilon(ii,ik1))/
     1    (icalfield(iblock)-icalfield(ik1))
	 ydif=(geofilat(ii,iblock)-geofilat(ii,ik1))/
     1    (icalfield(iblock)-icalfield(ik1))
	  iextens=iabs(icalfield(iblock)-icalfield(ik1))
	 do jj=1,iextens+1
	  geolonint(ii,jj)=geofilon(ii,iblock)+(jj-1)*xdif
	  geolatint(ii,jj)=geofilat(ii,iblock)+(jj-1)*ydif
	enddo
	enddo
	 do jj=1,iextens
	  do ii=1,avh_mxpix
   	if(iblock .ge. iblou .and. iblock .le. ibloo)then
   	 if((jj+1)/sample .gt. jj/sample)then
   	 if((ii+1)/sample .gt. ii/sample)then
 	 write(30,*)geolonint(ii,jj)*1.e-04,
     $     geolatint(ii,jj)*1.e-04,critfield(ii,jj)
     	endif
     	endif
   	endif
	enddo
	enddo
 	 iblock=ik1
	  do ij=1,nn
	  critfield(ij,1)=crithilf(ij)
	  enddo
c+++++++++endif interpolation++++++++++++++++
        endif
c+++++++++enddo scanline+++++++++++++++
	 if(iblock .gt. ibloo)goto 884
      enddo

c*******visible cal coeff.*************
 884	write(*,*)'vis cal., F and w'
	do j=1,3
	 do i=1,2
c*	  write(*,*)'channel',j,'calibration',avh_h_albcnv(i,j)
         enddo
        enddo
	write(*,*)'vis cal., Gain and intercept'
	do k=1,3
	do j=1,3
	do i=1,5
c*	  write(*,*)'k',k,'channel',j,'calibration',avh_calvis(i,k,j)
         enddo
        enddo
        enddo
c*******infrared cal coeff.*************
	write(*,*)'ir cal., Temperature radiance conversion'
	do j=1,avh_mxirchn
	 do i=1,3
c*	  write(*,*)'channel',j,avh_h_radtempcnv(i,j)
         enddo
        enddo
c****************************************
c*******infrared cal coeff.*************
	write(*,*)'ir cal., Count radiance conversion'
	  do j=1,2
	 do k=1,avh_mxirchn
	   do i=1,3
c*	  write(*,*)'channel',k,'calibration',avh_calir(i,j,k)
          enddo
         enddo
        enddo
c****************************************

      print*,'End of satellite data reached, ',irec,' records were read'
      close(unit=30)
      close(unit=20)
      close(unit=10)
      stop
      
 888  print*,ch_num,' is no valid channel number for AVHRR'
      stop

 889  print*,'An error occurred during reading the satellite data !'
      close(unit=30)
      close(unit=20)
      close(unit=10)
      stop

 999  print*,'An error occurred during reading the header !'
      close(unit=20)
      close(unit=10)
      stop
      end
    
c************
	subroutine interpol
	parameter(nn=2048,ntie=51,nval=40,noffset=25)
	common /ueber/ ueb(ntie),sol(nn)
	ifa=-1
	do i=noffset,noffset+(ntie-1)*nval
	 ifa=ifa+1
	 if(ifa .gt. nval-1)ifa=0
	 ilo=((i-noffset)/nval)+1
	 iup=((i-noffset)/nval)+2
	 xdif = ueb(iup)-ueb(ilo)
	 if(xdif.lt.-1800000.) xdif=xdif+3600000.
	 if(xdif.gt. 1800000.) xdif=xdif-3600000.
	 xdif=xdif/real(nval)
	 if(i .eq. noffset)xdiftaildo=xdif
	 if(i .eq. noffset-1+(ntie-1)*nval)xdiftailup=xdif
	 sol(i)=ueb(ilo)+ifa*xdif
	enddo
	do i=1,noffset
	 ifa=noffset-i
	 sol(i)=ueb(1)-ifa*xdiftaildo
        enddo
	do i=noffset+(ntie-1)*nval,nn
	 ifa=i-(noffset+(ntie-1)*nval)
	  sol(i)=ueb(ntie)+ifa*xdiftailup
	enddo
	return
	end
