!$Id: l1d_plot_tovs.f,v 1.2 2005/04/15 13:39:47 frna Exp $

      subroutine l1d_plot_tovs(L1d_file,instrument,channel_number,
     &  n_scnlin)

! Purpose: read AAPP TOVS level 1d file to display the results with GMT
!          Data are written into formatted file, which will be read by the GMT 
!          script

!-------------------------------------------------------------------------------------
! Author: K. Dieter Klaes January 2000
! updated: Rainer Schraidt July 2001
! updated: N Atkinson Nov 2004, modify includes assuming in-situ compilation
! updated: N Atkinson Nov 2006, use #include (required for AAPP v6)
! updated: N Atkinson Jul 2014, remove float() for portability
!-------------------------------------------------------------------------------------
! Includes
      
#include "ppparms.h"
#include "hrs1dm.h"
#include "reclopt.h"

! Declarations

      character*80 L1d_file
      character*80 satellite_data
            parameter(satellite_data='satellite_data.txt')

      character*2 instrument
      integer*4 channel_number,n_scnlin
      integer*4 irec
      integer*4 ipix
      
      

! Code starts here
      
      open(unit=10,
     1     file=L1d_file,
     2     form='unformatted',
     3     access='direct',
     4     recl=hrs_1drecl/rldiv,
     5     status='old')

      open(unit=30,
     1     file=satellite_data,
     2     form='formatted',
     3     status='unknown')

! Read Data Records

      do irec=2,n_scnlin
      
         read(10,rec=irec,err=889) hrs1d_dt
         
! Write data record and latitude,longitude information formatted into file

         if(instrument .eq. 'HO') then           ! original HIRS data

           do ipix=1,Hnfovs
	      if(hrs1d_btemps(channel_number,ipix) .gt. -9999) then
                write(30,110) 
     1          hrs1d_latlon(2,ipix)/1.e4,
     2          hrs1d_latlon(1,ipix)/1.e4,
     2          hrs1d_btemps(channel_number,ipix)/1.e2
 	      endif
           enddo

         else if(instrument .eq. 'MH') then      ! MSU on HIRS

           do ipix=1,Hnfovs
	      if(hrs1d_btemps(channel_number+Hnchan,ipix).gt.-9999) then
                write(30,110)
     1          hrs1d_latlon(2,ipix)/1.e4,
     2          hrs1d_latlon(1,ipix)/1.e4,
     2          hrs1d_btemps(channel_number+Hnchan,ipix)/1.e2
	      endif
           enddo

         else if(instrument .eq. 'AV') then      ! AVHRR on HIRS

           do ipix=1,Hnfovs
	      if(hrs1d_avhrr(channel_number,ipix) .gt. -9999) then
	        if(channel_number .gt. 1) then
                    write(30,110)
     1              hrs1d_latlon(2,ipix)/1.e4,
     2              hrs1d_latlon(1,ipix)/1.e4,
     2	            hrs1d_avhrr(channel_number,ipix)/1.e2 
		else
                    write(30,110)
     1              hrs1d_latlon(2,ipix)/1.e4,
     2              hrs1d_latlon(1,ipix)/1.e4,
     2	            hrs1d_avhrr(channel_number,ipix)
		endif
              endif
           enddo

         else if(instrument .eq. 'ax') then       ! Auxiliary Data

           do ipix=1,Hnfovs
              if(hrs1d_surface(channel_number,ipix) .gt. -9999) then
                write(30,110)
     1          hrs1d_latlon(2,ipix)/1.e4,
     2          hrs1d_latlon(1,ipix)/1.e4,
     2	        hrs1d_surface(channel_number,ipix)
              endif
   	   enddo
   	   
   	 else
   	 go to 890

         endif

      enddo
      
      close(unit=30)
      close(unit=10)
      
      return

 110  format(1x,3(f10.4,4x))

 888  print*,'End of satellite data reached, ',irec,' records were read'
      close(unit=30)
      close(unit=10)
      stop

 889  print*,'An Error occurred during reading the satellite data !'
      close(unit=30)
      close(unit=10)
      stop

 890  print*,instrument,' is no valid TOVS instrument plot request !'
      close(unit=30)
      close(unit=10)
      stop

      end
