!+ Level 1D Data Set (AMSU on AMSU-A grid)
!
! Name:  ama1d.h
!
! Purpose:
!   'INCLUDE' file for common /ama1dhd/   (AMSU-A grid level 1D header record)
!                  and common /ama1ddt/   (AMSU-A grid level 1D data record)
!
! Some parts of the 1D format are intended to be flexible, allowing
! different centres to customise the format for their own requirements.
! These parts are marked as '[user-defined ...]' below.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0     27/11/96  Original code. Conforms to 1D format vn1.8 (R.J.Renshaw)
!  1.1     12/12/96  Updated to 1D format vn1.9
!  1.2     17/01/97  Updated to 1D format vn1.91
!  1.3     10/03/98  Updated to 1D format vn1.92
!  1.4     15/06/98  Extra variables stored in ama1d_prepro, vn 1.93
!  1.5     15/08/05  add wmosatid in place of spare1 (NCA)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
      integer       ama_1dvnnb     ! AMSU-A level 1D format version number
         parameter (ama_1dvnnb = 193)!  - this code is set up for vn no 1.93
      integer       ama_1dvnyr     ! format version year for 1D AMSU-A
         parameter (ama_1dvnyr = 1998)
      integer       ama_1dvndy     ! format version day for 1D AMSU-A
         parameter (ama_1dvndy = 166)  ! = 15th June
      integer       ama_1drecl     ! AMSU-A level 1D dataset record length
         parameter (ama_1drecl = 1152*4)!  - number of bytes 
!
!###########################################################################
!
!               HEADER RECORD
!
!###########################################################################
!
      character*3  ama1d_h_site       !1D dataset creation site ID
      character*1  ama1d_h_cfill1     !filler
      character*3  ama1d_h_1bsite     !creation site for original 1B data
      character*1  ama1d_h_cfill2     !filler
      integer*4    ama1d_h_versnb     !level 1d format version number
      integer*4    ama1d_h_versyr     !level 1d format version year
      integer*4    ama1d_h_versdy     !level 1d format version day of year
      integer*4    ama1d_h_hdrcnt     !count of header records in this data set
      integer*4    ama1d_h_satid      !satellite id (e.g. 14 for NOAA-14)
      integer*4    ama1d_h_grid       !code for instrument grid
!                                 (5=HIRS; 6=MSU; 10=AMSU-A; 11=AMSU-B)
      integer*4    ama1d_h_satht      !nominal satellite altitude, km*10
      integer*4    ama1d_h_period     !nominal orbit period (seconds)
      integer*4    ama1d_h_startorbit !orbit number (at start of dataset)
      integer*4    ama1d_h_startdatayr!start of data set year
      integer*4    ama1d_h_startdatady!start of data set day of the year
      integer*4    ama1d_h_startdatatime !start of data set UTC time of day (ms)
      integer*4    ama1d_h_endorbit   !orbit number (at end of dataset)
      integer*4    ama1d_h_enddatayr  !end of data set year
      integer*4    ama1d_h_enddatady  !end of data set day of the year
      integer*4    ama1d_h_enddatatime!end of data set UTC time of day (ms)
      integer*4    ama1d_h_scnlin     !count of scan lines in this data set
      integer*4    ama1d_h_misscnlin  !count of missing scan lines
      integer*4    ama1d_h_atovppvn   !ATOVPP version number (test vns = 9000+)
      integer*4    ama1d_h_instruments!instruments present
!                   (bit0=HIRS, bit1=MSU, bit3=AMSU-A, bit4=AMSU-B, bit5=AVHRR)
      integer*4    ama1d_h_Aantenna   ! vn no, AMSU-A antenna corrections
                                      ! (from ATOVIN)
      integer*4    ama1d_h_wmosatid   !WMO sat ID
!
! AMSU(-A&B) temperature-radiance conversion coefficients  (Planck function)
      integer*4    ama1d_h_amsPlanck(3,Anchan+Bnchan)
!                  order of channels = AMSU-A 1-15, AMSU-B 1-5
!                  (1,*) = 10^6 x (central wavenumber)
!                  (2,*) = 10^6 x (constant 1)
!                  (3,*) = 10^6 x (constant 2)
!
! Pre-processing information
![user-defined:
      integer*4    ama1d_h_dT  ! size of data batches processed within ATOVPP
!                                (seconds   - see ATOVPP documentation)
      integer*4    ama1d_h_mapBTmode! mapping method used for BTs
!      (1=nearest neighbour, 2=bilinear interpolation, 3=spatial averaging)
      integer*4    ama1d_h_Acorrvn    ! vn no AMSU-A limb/emiss correction
!                                         (=0 if data not corrected)
      integer*4    ama1d_h_Apcpvn     ! vn no AMSU-A precip/scatter detection
!                                         (=0 if not done)
      integer*4    ama1d_h_Bcorrvn    ! vn no AMSU-B limb/emiss correction
!                                         (=0 if data not corrected)
      integer*4    ama1d_h_Bpcpvn     ! vn no AMSU-B precip/scatter detection
!                                         (=0 if not done)
      integer*4    ama1d_h_Bantenna   ! vn no, AMSU-B antenna corrections
                                      ! (from ATOVIN)
      integer*4    ama1d_h_Ascat !threshold value for AMSU-A scattering index
      integer*4    ama1d_h_ACrosby !threshold value for AMSU-A logistic precip
      integer*4    ama1d_h_AB89 !threshold value for AMSU-A/B 89GHz differences
      integer*4    ama1d_h_spare2(546)  !spare
! ... ]
!
      integer*4 ama1d_hd(ama_1drecl/4)
      equivalence(ama1d_hd(1),ama1d_h_site)
      common /ama1dhd/ ama1d_h_site, ama1d_h_cfill1, ama1d_h_1bsite,
     & ama1d_h_cfill2, ama1d_h_versnb, ama1d_h_versyr,
     & ama1d_h_versdy, ama1d_h_hdrcnt, ama1d_h_satid, ama1d_h_grid,
     & ama1d_h_satht, ama1d_h_period, ama1d_h_startorbit,
     & ama1d_h_startdatayr, ama1d_h_startdatady, ama1d_h_startdatatime,
     & ama1d_h_endorbit,ama1d_h_enddatayr, ama1d_h_enddatady,
     & ama1d_h_enddatatime,ama1d_h_scnlin, ama1d_h_misscnlin,
     & ama1d_h_atovppvn, ama1d_h_instruments, ama1d_h_Aantenna,
     & ama1d_h_wmosatid, ama1d_h_amsplanck, ama1d_h_dT, 
     & ama1d_h_mapBTmode, ama1d_h_Acorrvn, ama1d_h_Apcpvn, 
     & ama1d_h_Bcorrvn, ama1d_h_Bpcpvn, ama1d_h_Bantenna, ama1d_h_Ascat, 
     & ama1d_h_ACrosby, ama1d_h_AB89,
     & ama1d_h_spare2
!
!##############################################################################
!
!               DATA RECORD
!
!##############################################################################
!
      integer*4   ama1d_scnlin     !scan line number
      integer*4   ama1d_scnlinyr   !scan line year
      integer*4   ama1d_scnlindy   !scan line day of year
      integer*4   ama1d_scnlintime !scan line UTC time of day in milliseconds
      integer*4   ama1d_qualind    !quality indicator bit field
!
!                 In all of the following, if the bit is on (= is set to 1)
!                 then the statement is true. Otherwise it is false.
!
!                 general
!
!                 bit 31: do not use scan for product generation
!                 bit 30: time sequence error detected with this scan
!                 bit 29: data gap precedes this scan
!                 bit 28: no AMSU-A calibration
!                 bit 27: no earth location
!                 bit 26: first good time following a clock update
!                 bit 25: instrument status changed with this scan
!                 bit 24-0: spare  <zero fill>
!
      integer*4   ama1d_scanqual   !scan line quality flags
!                 time problem code
!                 (all bits off implies the scan time is as expected)
!                 bit 31-24: spare   <zero fill>
!                 bit 23: time field is bad but can probably be inferred
!                         from the previous good time
!                 bit 22: time field is bad and can't be inferred
!                         from the previous good time
!                 bit 21: this record starts a sequence that is inconsistent
!                         with previous times (i.e., there is a time
!                         discontinuity). This may or may not be associated
!                         with a spacecraft clock update. (see bit 26 above)
!                 bit 20: start of a sequence that apparently repeats scan
!                         times that have been previously accepted.
!                 bits 19-16: spare
!
!                 calibration problem code
!
!                 (all bits set to 0 indicates normal calibration)
!                  bit 15: scan line was not calibrated because of bad time
!                  bit 14: scan line was calibrated using fewer than the
!                          preferred number of scan lines because of
!                          proximity to start or end of data set or
!                          to a data gap.
!                  bit 13: scan line was not calibrated because of bad or
!                          insufficient PRT data
!                  bit 12: scan line was calibrated but with marginal PRT data
!                  bit 11: some uncalibrated channels of this scan.
!                          See channel indicators.
!                  bit 10: uncalibrated due to instrument mode
!                  bits 9 and 8: spare
!
!                  Earth location problem code
!
!                  (all bits set to 0 implies the Earth location was normal)
!                  bit 7: not Earth located because of bad time
!                  bit 6: Earth location questionable because of questionable
!                         time code (see time problem flags above)
!                  bit 5: Earth location questionable. Only marginal agreement
!                         with reasonableness check.
!                  bit 4: Earth location questionable. Fails reasonableness
!                         check
!                  bit 3-0: spare
!
      integer*4   ama1d_instrtemp1        !AMSU-A1 RF shelf temp (K*100)
      integer*4   ama1d_instrtemp2        !AMSU-A2 RF shelf temp (K*100)
      integer*4   ama1d_instrtempb        !AMSU-B Mixer ch 18-20 temp (K*100)
!
      integer*4   ama1d_spare1(2)             !spare
!
      integer*4   ama1d_latlon(2,Anfovs)  !lat/lon in degrees for Anfovs
!                     first   : 10^4 x (latitude)
!                     second  : 10^4 x (longitude)
      integer*4   ama1d_surface(2,Anfovs) !height/type for Anfovs
!                     first   : surface height (metres)
!                     second  : surface type (0=sea, 1=mixed, 2=land)
      integer*4   ama1d_angles(4,Anfovs) !scan angles for Anfovs
!                     first:    10^2 x (local zenith angle)
!                     second:   10^2 x (local azimuth angle)
!                     third:    10^2 x (solar zenith angle)
!                     fourth:   10^2 x (solar azimuth angle)
      integer*4   ama1d_scalti !sat altitude above reference ellipsoid, km*10
      integer*4   ama1d_spare2(2)             !spare
!
      integer*4   ama1d_btemps(Anchan+Bnchan,Anfovs)
!                 10^2 x scene brightness temperature (K) for channels
!                 AMSU-A 1-15, AMSU-B 1-5
!
! Pre-processing output
![user-defined:
      integer*4   ama1d_prepro(7,Anfovs) ! values from pre-processing
!                  (1,*):   pre-processing quality control word
!                            (all bits off implies acceptable data)
!                  bit 31: set if AMSU BTs considered contaminated
!                          due e.g. to precip or surface type
!                  bit 30: set if AMSU-A used secondary calibration
!                  bit 29: set if AMSU-B used secondary calibration
!                  bit 28: set if AMSU-B data missing due to insufficient
!                          good data available for the mapping.
!                  bit 27: maximum probability scheme cloud flag
!                  bit 26: scattering test (only set over the sea)
!                  bit 25: logistic precipitation probability test
!                  bit 24: Grody light rainfall test
!                  bit 23: mismatch between AMSU-A/B 89GHz values
!                  bit 22: mismatch between surface type from topography
!                          dataset and from pre-processing
!                  bit 21-1: spare   <zero fill>
!                  bit 0: set if AMSU-A data missing
!
!                  (2,*)    estimated surface type from the pre-processing:
!            1 = Bare young ice (i.e. new ice, no snow)
!            2 = Dry land (i.e. dry with or without significant vegetation)
!            3 = Dry snow (i.e. snow with water less than 2%, over land)
!            4 = Multi-year ice (i.e. old ice with snow [assumed dry] cover)
!            5 = Sea (i.e. open water, no islands, ice-free, WS=0 to 14m/s)
!            6 = Wet forest (i.e. established forest with wet canopy)
!            7 = Wet land (i.e. non-forested land with a wet surface)
!            8 = Wet snow (i.e. snow with water content > 2%, over land or ice)
!            9 = Desert
!
!                  (3,*) cost fn from PPASURF surface identification
!                  (4,*) scattering index
!                  (5,*) logistic precipitation probability
!                  (6,*)   spare
!                  (7,*)   spare
!
      integer*4   ama1d_filler(88)           !filler
! ... ]
!
      integer*4 ama1d_dt(ama_1drecl/4)
      equivalence(ama1d_dt(1),ama1d_scnlin)
      common /ama1ddt/ ama1d_scnlin, ama1d_scnlinyr, ama1d_scnlindy,
     & ama1d_scnlintime, ama1d_qualind, ama1d_scanqual,
     & ama1d_instrtemp1, ama1d_instrtemp2, ama1d_instrtempb,
     & ama1d_spare1, ama1d_latlon, ama1d_surface, ama1d_angles,
     & ama1d_scalti, ama1d_spare2, ama1d_btemps, ama1d_prepro,
     & ama1d_filler
!-
