!$Id: amb1b.h,v 1.2 2005/04/15 13:39:47 frna Exp $
!###########################################################################
!
!               AMSU-B Level 1B Data Set
!    Updated 5 March 1998 (based on NOAA revision dated 11 July 97)
!    Updated and checked on 11 May 1998 (Marcel Derrien)
!            (based on NOAA revision dated 08 April 1998)
!    Updated 14 Dec 1998 (include bias and anomalous bias corrections)
!    Minor update April 2004 - amb_moonang replaces amb_filler5
!    Updated 18 Oct 2004, based on NOAA update of Sept 27 2004. N C Atkinson
!
!###########################################################################
!                                                                          !
!                      AMSU-B  PARAMETERS                                  !
!                                                                          !
!###########################################################################

       INTEGER reclamb         !length in byte of one AMSU-B record
       parameter(reclamb=3072)
       INTEGER camb_size       !length in INTEGER*2 of one AMSU-B record
       parameter(camb_size=1536)

!!###########################################################################
!
!              AMSU-B HEADER RECORD
!
!##############################################################################
C!
C!..AMSU-B header words----------------------------------------------------
C!
C!.. General information
C!
       CHARACTER*3  amb_h_siteid        !Data set creation site ID
C!                            CMS = Centre de Meteorologie Spatiale/France
C!                            NSS = NESDIS/USA
C!                            UKM = UK Met Office
       CHARACTER*1  amb_h_blank
       INTEGER*2    amb_h_l1bversnb     !level 1b format version number
       INTEGER*2    amb_h_l1bversyr     !level 1b format version year
       INTEGER*2    amb_h_l1bversdy     !level 1b format version day of year
       INTEGER*2    amb_h_reclg         ! logical record length
       INTEGER*2    amb_h_blksz         ! blocksize
       INTEGER*2    amb_h_hdrcnt        !count of header records in data set
       INTEGER*2    amb_h_filler1(3)
       CHARACTER*42 amb_h_dataname      !data set name
       CHARACTER*8  amb_h_prblkid       !processing block identification
       INTEGER*2    amb_h_satid         !NOAA spacecraft identification code
       INTEGER*2    amb_h_instid        !AMSU-B instrument identification
       INTEGER*2    amb_h_datatyp       !data type code
C!                  (1 = LAC; 2 = GAC; 3 = HRPT; 4 = TIP; 5 = HIRS; 6 = MSU;
C!                   7 = SSU; 8 = DCS; 9 = SEM; 10 = AMSU-A; 11 = AMSU-B)
       INTEGER*2    amb_h_tipsrc        !TIP source code
C!        (0 = N/A; 1 = embedded; 2 = stored; 3 = third CDA; 4 = HRPT embedded)
       INTEGER*4    amb_h_startdatajd   !start of data set julian day
       INTEGER*2    amb_h_startdatayr   !start of data set year
       INTEGER*2    amb_h_startdatady   !start of data set day of year
       INTEGER*4    amb_h_startdatatime !start of data set UTC time of day in
C!                                       milliseconds
       INTEGER*4    amb_h_enddatajd     !end of data set julian day
       INTEGER*2    amb_h_enddatayr     !end of data set year
       INTEGER*2    amb_h_enddatady     !end of data set day of year
       INTEGER*4    amb_h_enddatatime   !end of data set UTC time of day in
C!                                       milliseconds
       INTEGER*2    amb_h_cpidsupdyr    ! year of last CPIDS update
       INTEGER*2    amb_h_cpidsupddy    ! day of year of last cpids update
       INTEGER*2    amb_h_fov1offset    !time offset for FOV 1 (ms)
C!       CHARACTER*2  amb_h_calparid     !Redundant, but retain for compatibility
       INTEGER*2    amb_h_filler2(5)   !filler
C!
C!              DATA SET QUALITY INDICATORS
C!
       INTEGER*4   amb_h_inststat1       !AMSU-B instrument status
       INTEGER*2   amb_h_filler3
       INTEGER*2   amb_h_statchrecnb     !record number of status change
C!                 ( if 0, none occurred)
       INTEGER*4   amb_h_inststat2       !AMSU-B second instrument status
C!                 ( if previous word is 0, no change)
       INTEGER*2   amb_h_scnlin         !count of scan lines in this data set
       INTEGER*2   amb_h_callocsclin    !count of calibrated, earth located
C!                                       scan lines in this data set
       INTEGER*2   amb_h_misscnlin      !count of missing scan lines
       INTEGER*2   amb_h_datagaps       !count of data gaps in this data set
       INTEGER*2   amb_h_okdatafr       !count of data frames without frame
C!                                       sync word errors
       INTEGER*2   amb_h_pacsparityerr  !count of PACS detected TIP parity
C!                                       errors
       INTEGER*2   amb_h_auxsyncerrsum  !sum of all auxiliary sync errors
C!                                       detected in the input data
       INTEGER*2   amb_h_timeseqerr     !time sequence error
C!               (0 =none; otherwise the record number of the first occurrence)
       INTEGER*2   amb_h_timeseqerrcode !time sequence error code
       INTEGER*2   amb_h_socclockupind  !socc clock update indicator
C!                 ( 0 = none during this orbit;
C!                   otherwise the record number of the first occurrence)
       INTEGER*2   amb_h_locerrind      !earth location error indicator
C!                 ( 0 = none during this orbit;
C!                   otherwise the record number of the first occurrence)
       INTEGER*2   amb_h_locerrcode     !earth location error code
       INTEGER*2   amb_h_pacsstatfield
       INTEGER*2   amb_h_pacsdatasrc
       CHARACTER*8 amb_h_spare2         ! reserved for ingester
       CHARACTER*8 amb_h_spare3         ! reserved for decommutation
       INTEGER*4   amb_h_filler4(4)
C!
C! Calibration parameters
C!
      INTEGER*2 amb_h_tsensid      !Instrument temperature sensor ID
C!                 (0 = Mixer Ch 18-20; 1 = Mixer Ch 16)
      INTEGER*2 amb_h_filler5
      INTEGER*2 amb_h_reftemp(3)   ! Instrument reference temps (Mixer 18-20)
      INTEGER*2 amb_h_reftemp2(3)   ! Instrument reference temps (Mixer 16)
      INTEGER*2 amb_h_bias(20)     ! Calibration bias corrections
      INTEGER*4 amb_h_nonlin(15)   ! Nonlinearity corrections
      INTEGER*4 amb_h_filler6(4)
C!
C! temp to radiance conversion factors
C!
      INTEGER*4 amb_h_temrad(15)  !10^6*(Central wavenumber, const1 and const2)
      INTEGER*4 amb_h_filler7(4)
C!
C!              EARTH LOCATION
C!
      CHARACTER*8 amb_h_modelid    !Reference elipsoid model ID
      INTEGER*2   amb_h_nadloctol  !10 x (nadir earth location tolerance in
C!                                         kilometers)
      INTEGER*2   amb_h_locbit     !earth location bit field
C!          bit 0 : constant attitude error correction (0 = not performed)
C!          bit 1 : reasonableness test (0 = inactive, 1=active)
C!          bit 2 : dynamic attitude error correction (0 = not performed)
      INTEGER*2   amb_h_filler8
      INTEGER*2   amb_h_rollerr    !10^3 x (constant roll attitude error in
C!                                           degrees)
      INTEGER*2   amb_h_pitcherr   !10^3 x (constant pitch attitude error in
C!                                           degrees)
      INTEGER*2   amb_h_yawerr     !10^3 x (constant yaw attitude error in
C!                                           degrees)
      INTEGER*2   amb_h_epoyr      !epoch year for orbit vector
      INTEGER*2   amb_h_epody      !day of epoch year for orbit vector
      INTEGER*4   amb_h_epotime    !epoc UTC time of day in milliseconds for
C!                                   orbit vector
      INTEGER*4   amb_h_smaxis     !10^6 x (semi-major axis in kilometers)
      INTEGER*4   amb_h_eccen      !10^8 x (eccentricity)
      INTEGER*4   amb_h_incli      !10^5 x (inclination in degrees)
      INTEGER*4   amb_h_argper     !10^5 x (argument of perigee in degrees)
      INTEGER*4   amb_h_rascnod    !10^5 x (right ascension of the ascending
C!                                           node in degrees)
      INTEGER*4   amb_h_manom      !10^5 x (mean anomaly in degrees)
      INTEGER*4   amb_h_xpos       !10^6 x (position vector x component in
C!                                           kilometers)
      INTEGER*4   amb_h_ypos       !10^6 x (position vector y component in
C!                                           kilometers)
      INTEGER*4   amb_h_zpos       !10^6 x (position vector z component in
C!                                           kilometers)
      INTEGER*4   amb_h_xvel       !10^8 x (velocity vector x-dot component
C!                                           in kilometers/second)
      INTEGER*4   amb_h_yvel       !10^8 x (velocity vector y-dot component
C!                                           in kilometers/second)
      INTEGER*4   amb_h_zvel       !10^8 x (velocity vector z-dot component
C!                                           in kilometers/second)
      INTEGER*4   amb_h_earthsun   !10^6 x (earth/sun distance ratio)
      INTEGER*4   amb_h_filler9(4)
C!
C! digital A and analogue telemetry conversion coeffs
C!
      INTEGER*2 amb_h_digacoef(96) !AMSU-B digital A coeffs
      INTEGER*4 amb_h_filler10(4)
      INTEGER*4 amb_h_analcoef(72) !AMSU-B analogue coeffs
C!
C! Corrections for S-band transmitter interference
C!      
      INTEGER*4 amb_h_filler11(2)
      INTEGER*2 amb_h_CountsCorr(5,21,4)  !Normal bias
      INTEGER*4 amb_h_filler12(2)
      INTEGER*2 amb_h_txpow(4)
      INTEGER*4 amb_h_filler13(2)
      INTEGER*2 amb_h_NewCorr(5,33,3)    !Anomalous bias
C!
C! Lunar contamination
C!
      INTEGER*2 amb_h_moonscnlin     !count of scans with moon in space view
                                     ! (-1 = detection algorithm turned off)
      INTEGER*2 amb_h_moonthresh     !100 x lunar angle threshold in degrees 
C!
C! Spare words
C!
      INTEGER*2 amb_h_filler14(107)
C!
      INTEGER*2 amb_hd(camb_size)
      EQUIVALENCE(amb_hd(1),amb_h_siteid)
C!
C!-------------------------------------------------------------------------
      COMMON/AMB1BHD/amb_h_siteid,amb_h_blank,
     &       amb_h_l1bversnb,amb_h_l1bversyr,amb_h_l1bversdy,
     &       amb_h_reclg,amb_h_blksz,amb_h_hdrcnt,
     &       amb_h_filler1,amb_h_dataname,amb_h_prblkid,amb_h_satid,
     &       amb_h_instid,amb_h_datatyp,amb_h_tipsrc,
     &       amb_h_startdatajd,amb_h_startdatayr,amb_h_startdatady,
     &       amb_h_startdatatime,amb_h_enddatajd,amb_h_enddatayr,
     &       amb_h_enddatady,amb_h_enddatatime,amb_h_cpidsupdyr,
     &       amb_h_cpidsupddy,amb_h_fov1offset,amb_h_filler2,
     &       amb_h_inststat1,amb_h_filler3,amb_h_statchrecnb,
     &       amb_h_inststat2,amb_h_scnlin,amb_h_callocsclin,
     &       amb_h_misscnlin,amb_h_datagaps,amb_h_okdatafr,
     &       amb_h_pacsparityerr,amb_h_auxsyncerrsum,amb_h_timeseqerr,
     &       amb_h_timeseqerrcode,amb_h_socclockupind,amb_h_locerrind,
     &       amb_h_locerrcode,amb_h_pacsstatfield,amb_h_pacsdatasrc,
     &       amb_h_spare2,amb_h_spare3,amb_h_filler4,amb_h_tsensid,
     &       amb_h_filler5,amb_h_reftemp,amb_h_reftemp2,amb_h_bias,
     &       amb_h_nonlin,amb_h_filler6,amb_h_temrad,amb_h_filler7,
     &       amb_h_modelid,amb_h_nadloctol,amb_h_locbit,amb_h_filler8,
     &       amb_h_rollerr,amb_h_pitcherr,amb_h_yawerr,amb_h_epoyr,
     &       amb_h_epody,amb_h_epotime,amb_h_smaxis,
     &       amb_h_eccen,amb_h_incli,amb_h_argper,amb_h_rascnod,
     &       amb_h_manom,amb_h_xpos, amb_h_ypos,amb_h_zpos,
     &       amb_h_xvel,amb_h_yvel,amb_h_zvel,amb_h_earthsun,
     &       amb_h_filler9,amb_h_digacoef,amb_h_filler10,
     &       amb_h_analcoef,amb_h_filler11,amb_h_countscorr,
     &       amb_h_filler12,amb_h_txpow,amb_h_filler13,
     &       amb_h_NewCorr,amb_h_moonscnlin,amb_h_moonthresh,
     &       amb_h_filler14
!
!##############################################################################
!
!            SINGLE  AMSU-B DATA RECORD
!
!##############################################################################
C!
C!......scan line information
C!
      INTEGER*2 amb_scnlin,amb_scnlinyr,amb_scnlindy,amb_clockdrift
      INTEGER*4 amb_scnlintime
      INTEGER*2 amb_scnlinbit
      INTEGER*2 amb_tipmf
      INTEGER*4 amb_filler1(2)
      INTEGER*4 amb_qualind,amb_linqualflgs
      INTEGER*2 amb_chqualflg(5)
      INTEGER*2 amb_filler2(9)
C!
C!......calibration coeffs
C!
      INTEGER*4 amb_pmcal(15),amb_sdcal(15)
      INTEGER*2 amb_filler3(5)
C!
C!.......navigation
C!
      INTEGER*2  amb_totattcorr(3)  !Total applied attitude correction
                                    !1000* Yaw, pitch, roll (degrees)
      INTEGER*4  amb_navstat        !navigation status bit field
!         bits17:    Nadir Earth location OK (see amb_h_nadloctol) (1=true)
!         bit 16:    earth location corrected for Euler angles (1=true)
!         bits15-12: earth location indicator
!                    0=earth location available
!                    1=user ephemerics files greater than 24hours old
!                    2=no earth location available
!         bits11-08: spacecraft attitude control
!                    0=operating in YGC or nominal mode
!                    1=operating in another mode
!                    2=attitude exceeds nominal tolerance
!                    3=both 1 and 2
!         bits07-04: attitude SMODE
!                    0=nominal mode
!                    1=rate nulling mode
!                    2=YGC mode
!                    3=search mode
!                    4=coast mode
!         bits03-00: attitude PWT1$AC
!                    0=nominal mode/no test
!                    1=yaw axis test in progress
!                    2=roll axis test in progress
!                    3=pitch axis test in progress
      INTEGER*4  amb_attangtime     !time associated angles (seconds)
      INTEGER*2  amb_rollang        !10^3*Roll angle in degrees
      INTEGER*2  amb_pitchang       !10^3*Pitch angle in degrees
      INTEGER*2  amb_yawang         !10^3*Yaw angle in degrees
      INTEGER*2  amb_scalti         !10*spacecraft altitude (MSL) in km
      INTEGER*2  amb_ang(270) !scan angles in degrees
!                     word1: 100*solar zenith angle, point 1
!                     word2: 100*satellite zenith angle, point 1
!                     word3: 100*satellite azimuth angle, point 1
!                     word4: 100*solar zenith angle, point 2
!                     word5: 100*satellite zenith angle, point 2
!                     word6: 100*satellite azimuth angle, point 2
!                     ......
!                     word268: 100*solar zenith angle, point 90
!                     word269: 100*satellite zenith angle, point 90
!                     word270: 100*satellite azimuth angle, point 90
      INTEGER*4  amb_pos(180) !Earth location
!                     (north latitude and east longitude are positive)
!                     word1: 10^4*latitude in degrees, point 1
!                     word2: 10^4*longitude in degrees, point 1
!                     word3: 10^4*latitude in degrees, point 2
!                     word4: 10^4*longitude in degrees, point 2
!                     ......
!                     word180: 10^4*longitude in degrees, point 90
      INTEGER*2  amb_moonang(4)  !Was zero fill. Use to store moon angles
C!
C!.......AMSU-B sensor data
C!
      INTEGER*2 amb_earth_data(540)
      INTEGER*4 amb_filler5(2)
C!
C!.......AMSU-B calibration data
C!
      INTEGER*2 amb_coldcal(24),amb_warmcal(24)
      INTEGER*4 amb_filler6(2)
C!
C!.......AMSU-B digital A telemetry 
C!
      INTEGER*4 amb_digaval             !Invalid data bit flags
      INTEGER*4 amb_filler7
      INTEGER*2 amb_diga(2)             ! digital data words A01 and A02
      INTEGER*2 amb_temps(24)           ! digital data words A03 through A26
      INTEGER*4 amb_filler8(3)
C!
C!.......AMSU-B analog telemetry
C!
      INTEGER*4 amb_ahkval  !Invalid analogue data bit flags
      INTEGER*2 amb_ahk(27)     !Added STX and SARR information
C!
C!.......fillers
C!
      INTEGER*2 amb_filler9
      INTEGER*4 amb_filler10(67)
C!
      INTEGER*2 amb_dt(camb_size)
      EQUIVALENCE(amb_dt(1),amb_scnlin)
C!
      COMMON/AMB1BDT/amb_scnlin,amb_scnlinyr,amb_scnlindy,
     &        amb_clockdrift,amb_scnlintime,amb_scnlinbit,amb_tipmf,
     &        amb_filler1,amb_qualind,amb_linqualflgs,
     &        amb_chqualflg,amb_filler2,
     &        amb_pmcal,amb_sdcal,amb_filler3,amb_totattcorr,
     &        amb_navstat,amb_attangtime,amb_rollang,amb_pitchang,
     &        amb_yawang,
     &        amb_scalti,amb_ang,amb_pos,amb_moonang,
     &        amb_earth_data,amb_filler5,
     &        amb_coldcal,amb_warmcal,amb_filler6,amb_digaval,
     &        amb_filler7,amb_diga,amb_temps,amb_filler8,
     &        amb_ahkval,amb_ahk,amb_filler9,amb_filler10
!-------------------------------------------------------------------------------
!                         -- CVS MODIFICATION HISTORY --
!-------------------------------------------------------------------------------
!$Log: amb1b.h,v $
!Revision 1.2  2005/04/15 13:39:47  frna
!Merge AAPP5Dev with main trunk
!
!Revision 1.1.1.1.4.4  2004/11/26 11:38:57  frna
!Add CVS identifiers
!
