!+ Level 1D Data Set (AMSU-B on AMSU-B grid)
!
! Name:  amb1d.h
!
! Purpose:
!   'INCLUDE' file for common /amb1dhd/   (AMSU-B grid level 1D header record)
!                  and common /amb1ddt/   (AMSU-B grid level 1D data record)
!
! Some parts of the 1D format are intended to be flexible, allowing
! different centres to customise the format for their own requirements.
! These parts are marked as '[user-defined ...]' below.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0     27/11/96  Original code. Conforms to 1D format vn1.8 (R.J.Renshaw)
!  1.1     12/12/96  Updated to 1D format vn1.9
!  1.2     17/01/97  Updated to 1D format vn1.91
!  1.3     10/03/98  Updated to 1D format vn1.92
!  1.4     22/06/98  Updated to 1D format vn1.93
!                    (several new variables  - record length doubled!)
!  1.5     16/03/00  Quality control parameters added. 1D format vn1.94
!  1.6     15/08/05  WMO sat id instead of spare1
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
      integer       amb_1dvnnb     ! AMSU-B level 1D format version number
         parameter (amb_1dvnnb = 194)!  - this code is set up for vn no 1.94
      integer       amb_1dvnyr     ! format version year for 1D AMSU-B
         parameter (amb_1dvnyr = 2000)
      integer       amb_1dvndy     ! format version day for 1D AMSU-B
         parameter (amb_1dvndy = 76)  ! = 16th March
      integer       array_size_ab1d     ! Length of array
          parameter (array_size_ab1d = 3072) !  Length of array
      integer       amb_1drecl     ! AMSU-B level 1D dataset record length
         parameter (amb_1drecl = array_size_ab1d*4)!  - number of bytes 
!
!###########################################################################
!
!               HEADER RECORD
!
!###########################################################################
!
      character*3  amb1d_h_site       !1D dataset creation site ID
      character*1  amb1d_h_cfill1     !filler
      character*3  amb1d_h_1bsite     !creation site for original 1B data
      character*1  amb1d_h_cfill2     !filler
      integer*4    amb1d_h_versnb     !level 1d format version number
      integer*4    amb1d_h_versyr     !level 1d format version year
      integer*4    amb1d_h_versdy     !level 1d format version day of year
      integer*4    amb1d_h_hdrcnt     !count of header records in this data set
      integer*4    amb1d_h_satid      !satellite id (e.g. 14 for NOAA-14)
      integer*4    amb1d_h_grid       !code for instrument grid
!                                 (5=HIRS; 6=MSU; 10=AMSU-A; 11=AMSU-B)
      integer*4    amb1d_h_satht      !nominal satellite altitude, km*10
      integer*4    amb1d_h_period     !nominal orbit period (seconds)
      integer*4    amb1d_h_startorbit !orbit number (at start of dataset)
      integer*4    amb1d_h_startdatayr!start of data set year
      integer*4    amb1d_h_startdatady!start of data set day of the year
      integer*4    amb1d_h_startdatatime !start of data set UTC time of day (ms)
      integer*4    amb1d_h_endorbit   !orbit number (at end of dataset)
      integer*4    amb1d_h_enddatayr  !end of data set year
      integer*4    amb1d_h_enddatady  !end of data set day of the year
      integer*4    amb1d_h_enddatatime!end of data set UTC time of day (ms)
      integer*4    amb1d_h_scnlin     !count of scan lines in this data set
      integer*4    amb1d_h_misscnlin  !count of missing scan lines
      integer*4    amb1d_h_atovppvn   !ATOVPP version number (test vns = 9000+)
      integer*4    amb1d_h_instruments!instruments present
!                   (bit0=HIRS, bit1=MSU, bit3=AMSU-A, bit4=AMSU-B, bit5=AVHRR)
      integer*4    amb1d_h_Bantenna   ! vn no, AMSU-B antenna corrections
                                      ! (from ATOVIN)
      integer*4    amb1d_h_wmosatid   ! WMO sat ID
!
! AMSU-A and B temperature-radiance conversion coefficients  (Planck function)
      integer*4    amb1d_h_amsPlanck(3,Anchan+Bnchan)
!                  order of channels = AMSU-A 1-15, AMSU-B 1-5
!                  (1,*) = 10^6 x (central wavenumber)
!                  (2,*) = 10^6 x (constant 1)
!                  (3,*) = 10^6 x (constant 2)
!
! Pre-processing information
![user-defined:
      integer*4    amb1d_h_Bcorrvn    ! vn no AMSU-B limb/emiss correction
!                                         (=0 if data not corrected)
      integer*4    amb1d_h_Bpcpvn     ! vn no AMSU-B precip/scatter detection
!                                         (=0 if not done)
      integer*4    amb1d_h_spare2(2218)  !spare
! ... ]
!
      integer*4 amb1d_hd(amb_1drecl/4)
      equivalence(amb1d_hd(1),amb1d_h_site)
      common /amb1dhd/ amb1d_h_site, amb1d_h_cfill1, amb1d_h_1bsite,
     & amb1d_h_cfill2, amb1d_h_versnb, amb1d_h_versyr,
     & amb1d_h_versdy, amb1d_h_hdrcnt, amb1d_h_satid, amb1d_h_grid,
     & amb1d_h_satht, amb1d_h_period, amb1d_h_startorbit,
     & amb1d_h_startdatayr, amb1d_h_startdatady, amb1d_h_startdatatime,
     & amb1d_h_endorbit,amb1d_h_enddatayr, amb1d_h_enddatady,
     & amb1d_h_enddatatime,amb1d_h_scnlin, amb1d_h_misscnlin,
     & amb1d_h_atovppvn, amb1d_h_instruments, amb1d_h_Bantenna,
     & amb1d_h_wmosatid, amb1d_h_amsplanck, amb1d_h_Bcorrvn,
     & amb1d_h_Bpcpvn, amb1d_h_spare2
!
!##############################################################################
!
!               DATA RECORD
!
!##############################################################################
!
      integer*4   amb1d_scnlin     !scan line number
      integer*4   amb1d_scnlinyr   !scan line year
      integer*4   amb1d_scnlindy   !scan line day of year
      integer*4   amb1d_scnlintime !scan line UTC time of day in milliseconds
      integer*4   amb1d_qualind    !quality indicator bit field
!
!                 In all of the following, if the bit is on (= is set to 1)
!                 then the statement is true. Otherwise it is false.
!
!                 general
!
!                 bit 31: do not use scan for product generation
!                 bit 30: time sequence error detected with this scan
!                 bit 29: data gap precedes this scan
!                 bit 28: no AMSU-B calibration
!                 bit 27: no earth location
!                 bit 26: first good time following a clock update
!                 bit 25: instrument status changed with this scan
!                 bit 24-0: spare  <zero fill>
!
      integer*4   amb1d_scanqual   !scan line quality flags
!                 time problem code
!                 (all bits off implies the scan time is as expected)
!                 bit 31-24: spare   <zero fill>
!                 bit 23: time field is bad but can probably be inferred
!                         from the previous good time
!                 bit 22: time field is bad and can't be inferred
!                         from the previous good time
!                 bit 21: this record starts a sequence that is inconsistent
!                         with previous times (i.e., there is a time
!                         discontinuity). This may or may not be associated
!                         with a spacecraft clock update. (see bit 26 above)
!                 bit 20: start of a sequence that apparently repeats scan
!                         times that have been previously accepted.
!                 bits 19-16: spare
!
!                 calibration problem code
!
!                 (all bits set to 0 indicates normal calibration)
!                  bit 15: scan line was not calibrated because of bad time
!                  bit 14: scan line was calibrated using fewer than the
!                          preferred number of scan lines because of
!                          proximity to start or end of data set or
!                          to a data gap.
!                  bit 13: scan line was not calibrated because of bad or
!                          insufficient PRT data
!                  bit 12: scan line was calibrated but with marginal PRT data
!                  bit 11: some uncalibrated channels of this scan.
!                          See channel indicators.
!                  bit 10: uncalibrated due to instrument mode
!                  bits 9 and 8: spare
!
!                  Earth location problem code
!
!                  (all bits set to 0 implies the Earth location was normal)
!                  bit 7: not Earth located because of bad time
!                  bit 6: Earth location questionable because of questionable
!                         time code (see time problem flags above)
!                  bit 5: Earth location questionable. Only marginal agreement
!                         with reasonableness check.
!                  bit 4: Earth location questionable. Fails reasonableness
!                         check
!                  bit 3-0: spare
!
      integer*4   amb1d_instrtemp         !AMSU-B Mixer chan 18-20 temp (K*100)

      integer*4   amb1d_spare1(2)         !spare
!
      integer*4   amb1d_latlon(2,Bnfovs)  !lat/lon in degrees for Bnfovs
!                     first   : 10^4 x (latitude)
!                     second  : 10^4 x (longitude)
      integer*4   amb1d_surface(2,Bnfovs) !height/type for Bnfovs
!                     first   : surface height (metres)
!                     second  : surface type (0=sea, 1=mixed, 2=land)
      integer*4   amb1d_angles(4,Bnfovs) !scan angles for Bnfovs
!                     first:    10^2 x (local zenith angle)
!                     second:   10^2 x (local azimuth angle)
!                     third:    10^2 x (solar zenith angle)
!                     fourth:   10^2 x (solar azimuth angle)
      integer*4   amb1d_scalti !sat altitude above reference ellipsoid, km*10
      integer*4   amb1d_spare2(2)             !spare
!
      integer*4   amb1d_btemps(Anchan+Bnchan,Bnfovs)
!                 10^2 x scene brightness temperature (K) for channels
!                 AMSU-A 1-15, AMSU-B 1-5
!
! Pre-processing output
![user-defined:
      integer*4   amb1d_prepro(6,Bnfovs) ! values from nearest AMSU-A:
!                  (1,*):   pre-processing quality control word
!                            (all bits off implies acceptable data)
!                  bit 31: spare   <zero fill>
!                  bit 30: set if AMSU-B used secondary calibration
!                  bit 29: set if AMSU-A used secondary calibration
!                  bit 28: set if AMSU-A data missing
!                 From nearest neighbour AMSU-A fov:
!                  bit 27: maximum probability scheme cloud flag
!                  bit 26: scattering test (only set over the sea)
!                  bit 25: logistic precipitation probability test
!                  bit 24: Grody light rainfall test
!                  bit 23: mismatch between AMSU-A/B 89GHz values
!                  bit 22: mismatch between surface type from topography
!                          dataset and from pre-processing
!                  bit 21:   spare   <zero fill>
!                 From AMSU-B fov:
!                  bit 20: set if 89GHz channel greatly different from
!                          that in surrounding fovs (PPMEDIAN)
!                  bit 19: scattering test (only set over the sea)  - using
!                                                       AMSU-B 89GHz channel
!                  bit 18: mismatch between AMSU-A/B 89GHz values
!                  bit 17: AMSU-B quality control flag 1
!                  bit 16: AMSU-B quality control flag 2
!                  bit 15: AMSU-B quality control flag 3
!                  bit 14: AMSU-B quality control flag 4
!                  bit 13-1: spare   <zero fill>
!                  bit 0:    bad or missing data (in any or all AMSU-B channels)

!                  (2,*)    estimated AMSU-A surface type:
!            1 = Bare young ice (i.e. new ice, no snow)
!            2 = Dry land (i.e. dry with or without significant vegetation)
!            3 = Dry snow (i.e. snow with water less than 2%, over land)
!            4 = Multi-year ice (i.e. old ice with snow [assumed dry] cover)
!            5 = Sea (i.e. open water, no islands, ice-free, WS=0 to 14m/s)
!            6 = Wet forest (i.e. established forest with wet canopy)
!            7 = Wet land (i.e. non-forested land with a wet surface)
!            8 = Wet snow (i.e. snow with water content > 2%, over land or ice)
!            9 = Desert
!
!                  (3,*) cost fn from PPASURF surface identification
!                  (4,*) scattering index (recalculated with AMSU-B 89GHz)
!                  (5,*) (microwave cirrus index)*1000 (range is +/-200)
!                  (6,*) spare
! ... ]
!
      integer*4 amb1d_dt(amb_1drecl/4)
      equivalence(amb1d_dt(1),amb1d_scnlin)
      common /amb1ddt/ amb1d_scnlin, amb1d_scnlinyr, amb1d_scnlindy,
     & amb1d_scnlintime, amb1d_qualind, amb1d_scanqual,
     & amb1d_instrtemp, amb1d_spare1,
     & amb1d_latlon, amb1d_surface, amb1d_angles, amb1d_scalti,
     & amb1d_spare2, amb1d_btemps, amb1d_prepro
!-
