!+ ATMS Level 1C Data Set
!
! Name:  atms1c.h
!
! Purpose:
!   'INCLUDE' file for common /atms1chd/   (ATMS level 1C header record)
!                  and common /atms1cdt/   (ATMS level 1C data record)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  0.1    20/07/2010  Original code. Nigel Atkinson.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
!
! Declarations:
!
      integer       atms_1cnchan
         parameter (atms_1cnchan = 22)
      integer       atms_1cnfov
         parameter (atms_1cnfov = 96)
      integer       atms_1cvnnb
         parameter (atms_1cvnnb = 1)
      integer       atms_1cvnyr
         parameter (atms_1cvnyr = 2010)
      integer       atms_1cvndy
         parameter (atms_1cvndy = 193)
      integer       array_size_atms1c            ! Length of array
          parameter (array_size_atms1c = 
     &         8 + 3*atms_1cnchan + 7*atms_1cnfov + 
     &         2*atms_1cnchan*atms_1cnfov )

      integer       array_size_atms1cspare   ! Length of spare header array
         parameter (array_size_atms1cspare = 
     &         array_size_atms1c - 30 - 8*atms_1cnchan )

      integer       atms_1crecl                  ! Record length in bytes
         parameter (atms_1crecl = array_size_atms1c*4)
!
!###########################################################################
!
!               HEADER RECORD
!
!###########################################################################
!
      character*3  atms1c_h_site       !dataset creation site ID
      character*1  atms1c_h_cfill1     !filler
      character*3  atms1c_h_1bsite     !creation site for original 1B data
      character*1  atms1c_h_cfill2     !filler
      integer*4    atms1c_h_versnb     !level 1c format version number
      integer*4    atms1c_h_versyr     !level 1c format version year
      integer*4    atms1c_h_versdy     !level 1c format version day of year
      integer*4    atms1c_h_hdrcnt     !count of header records in this data set
      integer*4    atms1c_h_satid      !satellite id (1 for NPP)
      integer*4    atms1c_h_instrument !WMO instrument identifier: 621=ATMS
      integer*4    atms1c_h_satht      !nominal satellite altitude, km*10
      integer*4    atms1c_h_period     !nominal orbit period (seconds)
      integer*4    atms1c_h_startorbit !orbit number (at start of file)
      integer*4    atms1c_h_startdatayr!start of data set year
      integer*4    atms1c_h_startdatady!start of data set day of the year
      integer*4    atms1c_h_startdatatime !start of data set UTC time of day (ms)
      integer*4    atms1c_h_endorbit   !orbit number (at end of file)
      integer*4    atms1c_h_enddatayr  !end of data set year
      integer*4    atms1c_h_enddatady  !end of data set day of the year
      integer*4    atms1c_h_enddatatime!end of data set UTC time of day (ms)
      integer*4    atms1c_h_scnlin     !count of scan lines in this data set
      integer*4    atms1c_h_misscnlin  !count of missing scan lines
      integer*4    atms1c_h_vnantennacorr  !version number, antenna corrections
      integer*4    atms1c_h_spare      !Spare

! temperature-radiance conversion coefficients
      integer*4    atms1c_h_tempradcnv(3,atms_1cnchan)
!                  order of channels = 1,2,3,4,...,22
!                  for nochannel = 1 to 22 :
!                      10^6 x (nochannel central wavenumber)
!                      10^6 x (nochannel constant 1)
!                      10^6 x (nochannel constant 2)
      integer*4    atms1c_h_bandwidth(atms_1cnchan)
      integer*4    atms1c_h_nchan
      integer*4    atms1c_h_nfov
      integer*4    atms1c_h_nchanused
      integer*4    atms1c_h_recwords   !Words in record
      integer*4    atms1c_h_wmosatid   !WMO satellite id (224 for NPP)
      integer*4    atms1c_h_centre     !WMO id of reception centre
      integer*4    atms1c_h_subcentre  !WMO id of reception subcentre

! The following variables characterise any image processing
      integer*4    atms1c_h_beamversion   !User defined (see atms_beamwidth.dat)
! FFT manipulation:
      integer*4    atms1c_h_beamwidth(atms_1cnchan)   !deg*1E3
      integer*4    atms1c_h_cutoff(atms_1cnchan)      !fraction*1E3
! Simple averaging:
      integer*4    atms1c_h_nxaverage(atms_1cnchan)
      integer*4    atms1c_h_nyaverage(atms_1cnchan)
      
      integer*4    atms1c_h_filler(array_size_atms1cspare)  !filler
!
      integer*4 atms1c_hd(array_size_atms1c)
      equivalence(atms1c_hd(1),atms1c_h_site)
      common /atms1chd/ atms1c_h_site, atms1c_h_cfill1,
     & atms1c_h_1bsite, atms1c_h_cfill2, atms1c_h_versnb,
     & atms1c_h_versyr, atms1c_h_versdy,
     & atms1c_h_hdrcnt, atms1c_h_satid, atms1c_h_instrument,
     & atms1c_h_satht, atms1c_h_period, atms1c_h_startorbit,
     & atms1c_h_startdatayr, atms1c_h_startdatady,
     & atms1c_h_startdatatime, atms1c_h_endorbit,
     & atms1c_h_enddatayr, atms1c_h_enddatady, atms1c_h_enddatatime,
     & atms1c_h_scnlin, atms1c_h_misscnlin, atms1c_h_vnantennacorr,
     & atms1c_h_spare, atms1c_h_tempradcnv, atms1c_h_bandwidth,
     & atms1c_h_nchan, atms1c_h_nfov, atms1c_h_nchanused,
     & atms1c_h_recwords, atms1c_h_wmosatid, 
     & atms1c_h_centre, atms1c_h_subcentre, atms1c_h_beamversion, 
     & atms1c_h_beamwidth, atms1c_h_cutoff, atms1c_h_nxaverage,
     & atms1c_h_nyaverage, atms1c_h_filler

      character*16 atms1c_h_software_version
      equivalence(atms1c_h_filler(1),atms1c_h_software_version)

!
!##############################################################################
!
!               DATA RECORD
!
!##############################################################################
!
      integer*4   atms1c_scnlin     !scan line number
      integer*4   atms1c_scnlinyr   !scan line year
      integer*4   atms1c_scnlindy   !scan line day of year
      integer*4   atms1c_scnlintime(atms_1cnfov)
                                    !scan line UTC time of day in milliseconds
      integer*4   atms1c_orbit

      integer*4   atms1c_granulequal  !granule quality flags
!Bit(1=MS of 16 bits)
!1-5	Reserved
!6	The No. 1-No.7 health checks failed
!7	The No. 8-No.15 health checks failed
!8	The No. 16-No.23 health checks failed
!9	The No. 24-No.31 health checks failed
!10	The No. 32-No.39 health checks failed
!11	The No. 40-No.47 health checks failed
!12	The No. 48-No.55 health checks failed
!13	The No. 56-No.63 health checks failed
!14	The No. 64-No.70 health checks failed
!15	Quadratic correction applied to the radiometric transfer function for
!         non-linearity correction
!All 16	Missing value

      integer*4   atms1c_scanqual     !scan line quality flags
!Bit(1=MS of 20 bits)
!1-6	Reserved
!7	Divide-by-zero condition or computation loop failed to converge in the
!         K/Ka and V (KAV) Band PRT
!8	Divide-by-zero condition or computation loop failed to converge in the
!         WG Band PRT
!9	Divide-by-zero condition or computation loop failed to converge in the
!         K/Ka, V, W, G Band Receiver Shelf PRT K temperature computation
!10	Out of range condition for the K/Ka and V Band PRT
!11	Out of range condition for the WG Band PRT
!12	KAV PRT temperature inconsistency
!13	WG PRT temperature inconsistency  
!14	Time Sequence Error
!15	Data Gap - Missing scan(s) preceding the current scan
!16	KAV PRT Sufficiency - Insufficient KAV PRT data are available
!17	WG PRT Sufficiency - Insufficient WG PRT data are available
!18	Space View antenna position error
!19	Blackbody antenna position error
!All 20	Missing value

      integer*4   atms1c_geolocqual   !geolocation quality
!Value
!0	Nominal - altitude and Ephemeris data available
!1	Missing at most a small gap of altitude and Ephemeris data
!2	Missing more than a small gap of altitude and Ephemeris data, but no
!         more than a granule boundary
!3	Missing more than a granule boundary of altitude and Ephemeris data
!4-14	Reserved
!15	Missing

      integer*4   atms1c_chanqual(atms_1cnchan)  !quality flag for each channel
!Bit(1=MS of 12 bits)
!1-2	Reserved
!3	Moon in Space View
!4	Gain Error - The lowest blackbody count is smaller than or equal to the
!         highest space view count in a scan
!5	Calibration With Fewer Than Preferred Samples
!6	Space View Data Sufficiency Check - Insufficient space view samples are
!         available
!7	Blackbody View Data Sufficiency Check -  Insufficient blackbody view
!         samples are available
!8	Out of range condition for the Space View 
!9	Out of range condition for the BlackBody View
!10	Space view inconsistency  
!11	BlackBody view inconsistency  
!All 12	Missing value

      integer*4   atms1c_latlon(2,atms_1cnfov)!lat/lon in degrees for atms_1cnfov
!                     first   : 10^4 x (latitude)
!                     second  : 10^4 x (longitude)
      integer*4   atms1c_angles(4,atms_1cnfov) !scan angles for atms_1cnfov
!                     first:    10^2 x (local zenith angle)
!                     second:   10^2 x (local azimuth angle)
!                     third:    10^2 x (solar zenith angle)
!                     fourth:   10^2 x (solar azimuth angle)
      integer*4   atms1c_scalti !sat altitude above reference ellipsoid, km*10

      integer*4   atms1c_nedtcold(atms_1cnchan)  !Cold view NEDT K*1E4
      integer*4   atms1c_nedtwarm(atms_1cnchan)  !Warm view NEDT K*1E4
      
      integer*4   atms1c_anttemps(atms_1cnchan,atms_1cnfov) !Antenna temps *100

      integer*4   atms1c_btemps(atms_1cnchan,atms_1cnfov)   !BT *100

      integer*4 atms1c_dt(array_size_atms1c)
      equivalence(atms1c_dt(1),atms1c_scnlin)
      common /atms1cdt/ atms1c_scnlin, atms1c_scnlinyr, atms1c_scnlindy,
     & atms1c_scnlintime, atms1c_orbit,
     & atms1c_granulequal, atms1c_scanqual, 
     & atms1c_geolocqual, atms1c_chanqual,
     & atms1c_latlon, atms1c_angles,
     & atms1c_scalti, atms1c_nedtcold, atms1c_nedtwarm, 
     & atms1c_anttemps, atms1c_btemps
!-
