!+ Level 1D Data Set (ATMS on ATMS grid)
!
! Name:  atms1d.h
!
! Purpose:
!   'INCLUDE' file for common /atms1dhd/   (ATMS grid level 1D header record)
!                  and common /atms1ddt/   (ATMS grid level 1D data record)
!
! Some parts of the 1D format are intended to be flexible, allowing
! different centres to customise the format for their own requirements.
! These parts are marked as '[user-defined ...]' below.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0   16/08/2010  Original code
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
      integer       atms_1dnchan
         parameter (atms_1dnchan = 22)
      integer       atms_1dnfov
         parameter (atms_1dnfov = 96)
      integer       atms_1dvnnb     ! ATMS level 1D format version number
         parameter (atms_1dvnnb = 1)
      integer       atms_1dvnyr     ! format version year for 1D ATMS
         parameter (atms_1dvnyr = 2010)
      integer       atms_1dvndy     ! format version day for 1D ATMS
         parameter (atms_1dvndy = 228)
      integer       array_size_atms1d     ! Length of array
         parameter (array_size_atms1d = 
     &         8 + 3*atms_1dnchan + atms_1dnfov*(20 + atms_1dnchan))

      integer       array_size_atms1dspare   ! Length of spare header array
         parameter (array_size_atms1dspare = 
     &      array_size_atms1d - 37 - 8*atms_1dnchan)

      integer       atms_1drecl     ! ATMS level 1D dataset record length
         parameter (atms_1drecl = array_size_atms1d*4)!  - number of bytes 
!
!###########################################################################
!
!               HEADER RECORD
!
!###########################################################################
!
      character*3  atms1d_h_site       !1D dataset creation site ID
      character*1  atms1d_h_cfill1     !filler
      character*3  atms1d_h_1bsite     !creation site for original 1B data
      character*1  atms1d_h_cfill2     !filler
      integer*4    atms1d_h_versnb     !level 1d format version number
      integer*4    atms1d_h_versyr     !level 1d format version year
      integer*4    atms1d_h_versdy     !level 1d format version day of year
      integer*4    atms1d_h_hdrcnt     !count of header records in this data set
      integer*4    atms1d_h_satid      !satellite id (e.g. 14 for NOAA-14)
      integer*4    atms1d_h_grid       !code for instrument grid
!                                 (5=HIRS; 6=MSU; 10=AMSU-A; 11=ATMS)
      integer*4    atms1d_h_satht      !nominal satellite altitude, km*10
      integer*4    atms1d_h_period     !nominal orbit period (seconds)
      integer*4    atms1d_h_startorbit !orbit number (at start of dataset)
      integer*4    atms1d_h_startdatayr!start of data set year
      integer*4    atms1d_h_startdatady!start of data set day of the year
      integer*4    atms1d_h_startdatatime !start of data set UTC time of day (ms)
      integer*4    atms1d_h_endorbit   !orbit number (at end of dataset)
      integer*4    atms1d_h_enddatayr  !end of data set year
      integer*4    atms1d_h_enddatady  !end of data set day of the year
      integer*4    atms1d_h_enddatatime!end of data set UTC time of day (ms)
      integer*4    atms1d_h_scnlin     !count of scan lines in this data set
      integer*4    atms1d_h_misscnlin  !count of missing scan lines
      integer*4    atms1d_h_atovppvn   !ATOVPP version number (test vns = 9000+)
      integer*4    atms1d_h_instruments!instruments present
!                   (bit0=HIRS, bit1=MSU, bit3=AMSU-A, bit4=ATMS, bit5=AVHRR,
!                    bit6=IASI, bit7=ATMS, bit8=CrIS)
      integer*4    atms1d_h_Aantenna   ! vn no, ATMS antenna corrections

! ATMS temperature-radiance conversion coefficients  (Planck function)
      integer*4    atms1d_h_planck(3,atms_1dnchan)
!                  (1,*) = 10^6 x (central wavenumber)
!                  (2,*) = 10^6 x (constant 1)
!                  (3,*) = 10^6 x (constant 2)
!
      integer*4    atms1d_h_bandwidth(atms_1dnchan)
      integer*4    atms1d_h_nchan
      integer*4    atms1d_h_nfov
      integer*4    atms1d_h_nchanused
      integer*4    atms1d_h_recwords   !Words in record
      integer*4    atms1d_h_wmosatid   !WMO sat ID (224 for NPP)
      integer*4    atms1d_h_centre     !WMO id of reception centre
      integer*4    atms1d_h_subcentre  !WMO id of reception subcentre
!
! The following variables characterise any image processing (see atms_beamwidth)
      integer*4    atms1d_h_beamversion   !User defined (see atms_beamwidth.dat)
! FFT manipulation:
      integer*4    atms1d_h_beamwidth(atms_1dnchan)   !deg*1E3
      integer*4    atms1d_h_cutoff(atms_1dnchan)      !fraction*1E3
! or simple averaging:
      integer*4    atms1d_h_nxaverage(atms_1dnchan)
      integer*4    atms1d_h_nyaverage(atms_1dnchan)
      
! Pre-processing information
![user-defined:
      integer*4    atms1d_h_dT  ! size of data batches processed within ATOVPP
!                                (seconds   - see ATOVPP documentation)
      integer*4    atms1d_h_Acorrvn   ! vn no ATMS limb/emiss correction
!                                         (=0 if data not corrected)
      integer*4    atms1d_h_Apcpvn    ! vn no ATMS precip/scatter detection
!                                         (=0 if not done)
      integer*4    atms1d_h_Ascat     !threshold value for ATMS scattering index
      integer*4    atms1d_h_Acirr     !threshold value for cirrus index
      integer*4    atms1d_h_ACrosby   !threshold value for ATMS logistic precip
      integer*4    atms1d_h_spare2(array_size_atms1dspare)  !spare
! ... ]
!
      integer*4 atms1d_hd(atms_1drecl/4)
      equivalence(atms1d_hd(1),atms1d_h_site)
      common /atms1dhd/ atms1d_h_site, atms1d_h_cfill1, atms1d_h_1bsite,
     & atms1d_h_cfill2, atms1d_h_versnb, atms1d_h_versyr,
     & atms1d_h_versdy, atms1d_h_hdrcnt, atms1d_h_satid, atms1d_h_grid,
     & atms1d_h_satht, atms1d_h_period, atms1d_h_startorbit,
     & atms1d_h_startdatayr, atms1d_h_startdatady, 
     & atms1d_h_startdatatime,
     & atms1d_h_endorbit,atms1d_h_enddatayr, atms1d_h_enddatady,
     & atms1d_h_enddatatime,atms1d_h_scnlin, atms1d_h_misscnlin,
     & atms1d_h_atovppvn, atms1d_h_instruments, atms1d_h_Aantenna,
     & atms1d_h_planck, atms1d_h_bandwidth, atms1d_h_nchan, 
     & atms1d_h_nfov, atms1d_h_nchanused, atms1d_h_recwords,
     & atms1d_h_wmosatid, atms1d_h_centre, atms1d_h_subcentre, 
     & atms1d_h_beamversion, atms1d_h_beamwidth, atms1d_h_cutoff,
     & atms1d_h_nxaverage, atms1d_h_nyaverage, atms1d_h_dT,
     & atms1d_h_Acorrvn, atms1d_h_Apcpvn, atms1d_h_Ascat, 
     & atms1d_h_Acirr, atms1d_h_ACrosby, atms1d_h_spare2
!
!##############################################################################
!
!               DATA RECORD
!
!##############################################################################
!
      integer*4   atms1d_scnlin     !scan line number
      integer*4   atms1d_scnlinyr   !scan line year
      integer*4   atms1d_scnlindy   !scan line day of year
      integer*4   atms1d_scnlintime(atms_1dnfov) !UTC time of day in milliseconds
      integer*4   atms1d_orbit

      integer*4   atms1d_granulequal  !granule quality flags

      integer*4   atms1d_scanqual     !scan line quality flags

      integer*4   atms1d_geolocqual   !geolocation quality

      integer*4   atms1d_chanqual(atms_1dnchan)  !quality flag for each channel

      integer*4   atms1d_latlon(2,atms_1dnfov)  !lat/lon in degrees
!                     first   : 10^4 x (latitude)
!                     second  : 10^4 x (longitude)
      integer*4   atms1d_surface(2,atms_1dnfov) !height/type
!                     first   : surface height (metres)
!                     second  : surface type (0=sea, 1=mixed, 2=land)
      integer*4   atms1d_landfrac(2,atms_1dnfov)
!                     first   : AMSU-A-like channels (percent)
!                     second  : AMSU-B-like channels (percent)
      integer*4   atms1d_angles(4,atms_1dnfov) !scan angles
!                     first:    10^2 x (local zenith angle)
!                     second:   10^2 x (local azimuth angle)
!                     third:    10^2 x (solar zenith angle)
!                     fourth:   10^2 x (solar azimuth angle)
      integer*4   atms1d_scalti !sat altitude above reference ellipsoid, km*10

      integer*4   atms1d_nedtcold(atms_1dnchan)  !Cold view NEDT K*100
      integer*4   atms1d_nedtwarm(atms_1dnchan)  !Warm view NEDT K*100
!
      integer*4   atms1d_btemps(atms_1dnchan,atms_1dnfov)
!                 10^2 x scene brightness temperature (K)
!
      integer*4   atms1d_atmsqual(atms_1dnfov)
!                 Quality control word for each field of view:
!                  (all bits off implies acceptable data)
!                  bit 27-4: bit n set to 1 if brightness temperature in
!                            ATMS channel n-3 is physically unreasonable or has
!                            not been calculated due to calibration  
!                            problems.
!                  bit 3-1:  reserved
!                  bit 0:    bad or missing data (in any or all channels)

! Pre-processing output
      integer*4   atms1d_prepro(8,atms_1dnfov)
!                  (1,*):   pre-processing quality control word, atmsqc
!                            (all bits off implies acceptable data)
!                  bit 31: set if ATMS BTs considered contaminated
!                          due e.g. to precip or surface type
!                  bit 30: spare
!                  bit 29: spare
!                  bit 28: spare
!                  bit 27: maximum probability scheme cloud flag
!                  bit 26: scattering test (only set over the sea)
!                  bit 25: Crosby logistic precipitation probability test
!                  bit 24: Grody light rainfall test
!                  bit 23: spare
!                  bit 22: mismatch between surface type from topography
!                          dataset and from pre-processing
!                  bit 21:   spare   <zero fill>
!                  bit 20:   spare
!                  bit 19: scattering test (only set over the sea)  - using
!                                                       ATMS 89GHz channel
!                  bit 18: spare
!                  bit 17: spare
!                  bit 16: spare
!                  bit 15: spare
!                  bit 14: spare
!                  bit 13-1: spare   <zero fill>
!                  bit 0:    bad or missing data (in any or all ATMS channels)

!                  (2,*)    estimated AMSU-A surface type:
!            1 = Bare young ice (i.e. new ice, no snow)
!            2 = Dry land (i.e. dry with or without significant vegetation)
!            3 = Dry snow (i.e. snow with water less than 2%, over land)
!            4 = Multi-year ice (i.e. old ice with snow [assumed dry] cover)
!            5 = Sea (i.e. open water, no islands, ice-free, WS=0 to 14m/s)
!            6 = Wet forest (i.e. established forest with wet canopy)
!            7 = Wet land (i.e. non-forested land with a wet surface)
!            8 = Wet snow (i.e. snow with water content > 2%, over land or ice)
!            9 = Desert
!
!                  (3,*) cost fn from PPASURF surface identification
!                  (4,*) scattering index (89GHz)
!                  (5,*) cirrus index (*100)
!                  (6,*) logistic precipitation probability (*100)
!                  (7,*) NWC-SAF (Bennartz) scattering index (K)
!                  (8,*) NWC-SAF (Bennartz) precipitation probabilities
! ... ]
!
      integer*4 atms1d_dt(atms_1drecl/4)
      equivalence(atms1d_dt(1),atms1d_scnlin)
      common /atms1ddt/ atms1d_scnlin, atms1d_scnlinyr, atms1d_scnlindy,
     & atms1d_scnlintime, atms1d_orbit, atms1d_granulequal, 
     & atms1d_scanqual, atms1d_geolocqual, atms1d_chanqual, 
     & atms1d_latlon, atms1d_surface, atms1d_landfrac,
     & atms1d_angles, atms1d_scalti,
     & atms1d_nedtcold, atms1d_nedtwarm, atms1d_btemps, 
     & atms1d_atmsqual, atms1d_prepro
!-
