!+
!NOM/NAME  %M% version %I% on %G% 
! 
!DESCRIPTION 
!      include file for avhrr/hrpt 1b format handling
!
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
! Marcel Derrien
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
!  1.0     03/95     initial            Marcel Derrien
!  2.0     07/11/95  new release from NOAA available 30/10/95
!                    still problems in integer boudaries alignments
!                    and undefined scaling factors
!  3.0     11/05/98  modified to conform to NOAA release dated 
!                    01 April 1998 (gpg # 13 for header record)
!  3.1     06/12/01  Add comments about the scalling factor of 
!                    avh_h_irttcoef(6,4)  (T.Labrot)
!  3.2     02/04/04  Correct dimension of avh_filler7 
!
!           Please note that :
!            the format is NOAA one from byte 1 to 21752 )
!              the telemetry part does not conform to NOAA version
!              ( bytes 21753 to 21882 in our format)
!              it does not include the clouds from AVHRR part
! 
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!###########################################################################
!
!               AVHRR/HRPT 1B Data Set 
!
!###########################################################################
!
!  parameters definitions
!
       integer avh_mxirchn       !IR channels of AVHRR
       integer avh_mxvischn      !visible channels of AVHRR
       integer avh_mxhrpchn      !count of AVHRR channels in HRPT stream
       parameter(avh_mxirchn=3,
     >           avh_mxvischn=3,
     >           avh_mxhrpchn=5)
       integer avh_mxpix          !maximun pixels number
       parameter(avh_mxpix=2048) 
       integer reclavh            ! 1B record length ( bytes)
!  does not respect 1B NOAA size (see las part of the data record)
       parameter (reclavh = 22016)
       integer cavh_size           ! 1B record length ( number of integer*4)
       parameter ( cavh_size = reclavh/4)
!
!###########################################################################
!
!               HEADER RECORD
!
!###########################################################################
!
!                GENERAL INFORMATION
!
       character*3  avh_h_siteid        !Data set creation site ID
       character*1  avh_h_blank         !ASCII blank
       integer*2    avh_h_l1bversnb     !level 1b format version number
       integer*2    avh_h_l1bversyr     !level 1b format version year
       integer*2    avh_h_l1bversdy     !level 1b format version day of year
       integer*2    avh_h_reclg         !reserved for logical record length
       integer*2    avh_h_blksz         !reserved for block size
       integer*2    avh_h_hdrcnt        !count of header records in data set
       integer*2    avh_h_filler0(3)
       character*42 avh_h_dataname      !data set name
       character*8  avh_h_prblkid       !processing block identification
       integer*2    avh_h_satid         !NOAA spacecraft identification code
       integer*2    avh_h_instid        !instrument identification
       integer*2    avh_h_datatyp       !data type code
!                   (1 = LAC; 2 = GAC; 3 = HRPT; 4 = TIP; 5 = HIRS; 6 = MSU;
!                    7 = SSU; 8 = DCS; 9 = SEM; 10 = AMSU-A; 11 = AMSU-B)
       integer*2    avh_h_tipsrc        !TIP source code
!                   (0 = not applicable; 1 = GAC embedded; 2 = stored;
!                    3 = third CDA; 4 = HRPT embedded)
       integer    avh_h_startdatajd   !start of data set day count 
!                                        from 00h,1 Jan 1950
       integer*2    avh_h_startdatayr   !start of data set year
       integer*2    avh_h_startdatady   !start of data set day of year
       integer    avh_h_startdatatime !start of data set UTC time of day in
!                                        milliseconds
       integer    avh_h_enddatajd     !end of data set day count 
!                                        from 00h,1 Jan 1950
       integer*2    avh_h_enddatayr     !end of data set year
       integer*2    avh_h_enddatady     !end of data set day of year
       integer      avh_h_enddatatime   !end of data set UTC time of day in
!                                        milliseconds
       integer*2    avh_h_cpidsyr       !year of Last CPIDS Update
       integer*2    avh_h_cpidsdy       !Day of Year of Last CPIDS Update

       integer*2    avh_h_filler1(4)    !filler
!
!                DATA SET QUALITY INDICATORS
!
       integer*4   avh_h_inststat1         !first instrument status
!                  bit 15 : motor/telemetry (0 = off; 1 = on) 
!                  bit 14 : electronics/telemetry (0 = off; 1 = on)
!                  bit 13 : channel 1  status (0 = disable; 1 = enable)
!                  bit 12 : channel 2  status (0 = disable; 1 = enable)
!                  bit 11 : channel 3A status (0 = disable; 1 = enable)
!                  bit 10 : channel 3B status (0 = disable; 1 = enable)
!                  bit  9 : channel 4  status (0 = disable; 1 = enable)
!                  bit  8 : channel 5  status (0 = disable; 1 = enable)
       integer*2   avh_h_filler2         !filler
       integer*2   avh_h_statchrecnb       !record number of status change 
!                  ( if 0, none occurred)
       integer*4   avh_h_inststat2         !second instrument status 
!                  ( if previous word is 0, no change)
       integer*2   avh_h_scnlin            !count of scan lines in this data set
       integer*2   avh_h_callocscnlin      !count of calibrated, earth located
!                                           scan lines in this data set
       integer*2   avh_h_misscnlin         !count of missing scan lines
       integer*2   avh_h_datagaps          !count of data gaps in this data set
       integer*2   avh_h_okdatafr          !count of data frames without frame 
!                                           sync word errors
       integer*2   avh_h_pacsparityerr     !count of PACS detected TIP parity
!                                           errors
       integer*2   avh_h_auxsyncerrsum     !sum of all auxiliary sync errors  
!                                           detected in the input data
       integer*2   avh_h_timeseqerr        !time sequence error
!                (0 =none; otherwise the record number of the first occurrence)
       integer*2   avh_h_timeseqerrcode    !time sequence error code 
       integer*2   avh_h_socclockupind     !socc clock update indicator
!                  ( 0 = none during this orbit;
!                    otherwise the record number of the first occurrence)
       integer*2   avh_h_locerrind         !earth location error indicator
!                  ( 0 = none during this orbit;
!                    otherwise the record number of the first occurrence)
       integer*2   avh_h_locerrcode        !earth location error code
       integer*2   avh_h_pacsstatfield     !PACS status bit field
!                  bit 15-3 : spare
!                  bit  2   : pseudo noise (0 = normal data; 1 = P/N data)
!                  bit  1   : tape direction (0 = time decrementing)
!                  bit  0   : data mode (o = test data; 1 = flight data)
       integer*2   avh_h_pacsdatasrc       !PACS data source 
!                  (0 = unused; 1 = Gilmore; 2 = Wallops; 3 = SOCC)
       integer     avh_h_filler3
       character*8 avh_h_spare1            !spare (reserved for ingester)
       character*8 avh_h_spare2            !spare (reserved of the
!                                           decommutation)
       integer*2 avh_h_filler4(5)          !filler


!
!                CALIBRATION
!
       integer*2 avh_h_racalind    !ramp/auto calibration indicators bit field
!                bit 5:  ramp non-linearity for GAC, LAC, and HRPT channel 5
!                bit 4:  ramp non-linearity for GAC, LAC, and HRPT channel 4
!                bit 3:  ramp non-linearity for GAC, LAC, and HRPT channel 3B
!                bit 2:  ramp non-linearity for GAC, LAC, and HRPT channel 3A
!                bit 1:  ramp non-linearity for GAC, LAC, and HRPT channel 2
!                bit 0:  ramp non-linearity for GAC, LAC, and HRPT channel 1
       integer*2 avh_h_solarcalyr    !year of most recent solar channel
!                                     calibration
       integer*2 avh_h_solarcaldy    !day of year of most recent solar channel
!                                     calibration
       integer*2 avh_h_pcalalgind    !primary calibration algorithm
!                                    identification 
       integer*2 avh_h_pcalalgopt    !primary calibration algorithm option
!                                     selected 
       integer*2 avh_h_scalalgind    !secundary calibration algorithm
!                                     identification 
       integer*2 avh_h_scalalgopt    !secundary calibration algorithm option
!                                     selected 
       integer*2 avh_h_irttcoef(6,4) ! IR Target Temperature Conversion Coefficients
                                     ! scalling factor of
                                     !    avh_h_irttcoef(1,*) = 10^2
                                     !    avh_h_irttcoef(2,*) = 10^5
                                     !    avh_h_irttcoef(3,*) = 10^8
                                     !    avh_h_irttcoef(4,*) = 10^11
                                     !    avh_h_irttcoef(5,*) = 10^14
                                     !    avh_h_irttcoef(6,*) = 10^17              
       integer*4 avh_h_filler5(2)
!
!                RADIANCE TO TEMPERATURE CONVERSION
!
       integer avh_h_albcnv(2,avh_mxvischn)
!                albedo conversion 
!                order of channels : 1, 2, 3A.
!                for nochannel = 1 to 3A :
!                  10^1 x (albedo-radiance nochannel solar filtered irradiance 
!                          in wavelength)
!                  10^3 x (albedo-radiance nochannel equivalent filter width 
!                          in wavelength)
       integer avh_h_radtempcnv(3,avh_mxirchn)
!                radiance to temperature conversion 
!                order of channels : 3B, 4, 5.
!                for nochannel = 3B to 5:
!            10^2 x (temperature-radiance nochannel central wavenumber Ch3b)
!        ou  10^3 x (temperature-radiance nochannel central wavenumber Ch4,5)
!                  10^5 x (temperature-radiance nochannel constant 1)
!                  10^6 x (temperature-radiance nochannel constant 2)
       integer*4 avh_h_filler6(3)         ! Filler <Zero Fill>

!                NAVIGATION
!
       character*8 avh_h_modelid
       integer*2   avh_h_nadloctol  !10 x (nadir earth location tolerance in
!                                          kilometers)
       integer*2   avh_h_locbit     !earth location bit field
!                  bit 0 : attitude error correction (0 = not corrected)
       integer*2   avh_h_filler7

       integer*2   avh_h_rollerr    !10^3 x (constant roll attitude error in
!                                            degrees)
       integer*2   avh_h_pitcherr   !10^3 x (constant pitch attitude error in
!                                            degrees)
       integer*2   avh_h_yawerr     !10^3 x (constant yaw attitude error in
!                                            degrees)
       integer*2   avh_h_epoyr      !epoch year for orbit vector
       integer*2   avh_h_epody      !day of epoch year for orbit vector

       integer   avh_h_epotime    !Epoch UTC Time of day in milliseconds for Orbit 
!                                    Vector

       integer   avh_h_smaxis     !10^5 x (semi-major axis in kilometers)
       integer   avh_h_eccen      !10^8 x (eccentricity)
       integer   avh_h_incli      !10^5 x (inclination in degrees)
       integer   avh_h_argper     !10^5 x (argument of perigee in degrees)
       integer   avh_h_rascnod    !10^5 x (right ascension of the ascending 
!                                            node in degrees)
       integer   avh_h_manom      !10^5 x (mean anomaly in degrees)
       integer   avh_h_xpos       !10^5 x (position vector x component in
!                                            kilometers)
       integer   avh_h_ypos       !10^5 x (position vector y component in
!                                            kilometers)
       integer   avh_h_zpos       !10^5 x (position vector z component in
!                                            kilometers)
       integer   avh_h_xvel       !10^8 x (velocity vector x-dot component 
!                                            in kilometers/second)
       integer   avh_h_yvel       !10^8 x (velocity vector y-dot component
!                                            in kilometers/second)
       integer   avh_h_zvel       !10^8 x (velocity vector z-dot component 
!                                            in kilometers/second)
       integer   avh_h_earthsun   !10^6 x (earth/sun distance ratio)
       integer*4   avh_h_filler8(4)  ! Filler <Zero Fill>
!
!                ANALOG TELEMETRY CONVERSION
!
!      conversion coefficients
!
! 
       integer*2 avh_h_pchtemp(5)     !patch temperature coef.
       integer*2 avh_h_reserved1
       integer*2 avh_h_pchtempext(5)  !patch temperature extended coef.
       integer*2 avh_h_reserved2       
       integer*2 avh_h_pchpow(5)      !patch power coef.
       integer*2 avh_h_reserved3       
       integer*2 avh_h_rdtemp(5)      !radiator temperature coef.
       integer*2 avh_h_reserved4        
       integer*2 avh_h_bbtemp1(5)     !black body temperature 1 coef.
       integer*2 avh_h_reserved5        
       integer*2 avh_h_bbtemp2(5)     !black body temperature 2 coef.   
       integer*2 avh_h_reserved6        
       integer*2 avh_h_bbtemp3(5)     !black body temperature 3 coef. 
       integer*2 avh_h_reserved7        
       integer*2 avh_h_bbtemp4(5)     !black body temperature 4 coef.
       integer*2 avh_h_reserved8        
       integer*2 avh_h_eleccur(5)     !electronics current  coef.
       integer*2 avh_h_reserved9        
       integer*2 avh_h_motorcur(5)    !motor current  coef.
       integer*2 avh_h_reserved10        
       integer*2 avh_h_earthpos(5)    !earth shield position coef.
       integer*2 avh_h_reserved11       
       integer*2 avh_h_electemp(5)    !electronics temperature  coef.
       integer*2 avh_h_reserved12       
       integer*2 avh_h_chtemp(5)      !cooler housing  temperature coef. 
       integer*2 avh_h_reserved13       
       integer*2 avh_h_bptemp(5)      !baseplate temperature coef.
       integer*2 avh_h_reserved14       
       integer*2 avh_h_mhtemp(5)      !motor housing temperature  coef.
       integer*2 avh_h_reserved15        
       integer*2 avh_h_adcontemp(5)   !A/D converter temperature  coef.
       integer*2 avh_h_reserved16        
       integer*2 avh_h_d4bvolt(5)     !detector #4 bias voltage  coef.
       integer*2 avh_h_reserved17        
       integer*2 avh_h_d5bvolt(5)     !detector #5 bias voltage  coef.
       integer*2 avh_h_reserved18        
       integer*2 avh_h_bbtempchn3B(5) !black body temperature channel 3B coef.
       integer*2 avh_h_reserved19        
       integer*2 avh_h_bbtempchn4(5)  !black body temperature channel 4  coef.
       integer*2 avh_h_reserved20        
       integer*2 avh_h_bbtempchn5(5)  !black body temperature channel 5 coef.
       integer*2 avh_h_reserved21        
       integer*2 avh_h_refvolt(5)     !reference voltage coef.
       integer*2 avh_h_reserved22        

!
!                FILLER
!
       integer*2 avh_h_filler9(10664) !filler bytes to equal data record length
       integer avh_hd(cavh_size)
       
       equivalence ( avh_hd , avh_h_siteid )
       
        common/avh1bhd/ avh_h_siteid , avh_h_blank ,
     &  avh_h_l1bversnb ,avh_h_l1bversyr , avh_h_l1bversdy ,
     &  avh_h_reclg , avh_h_blksz ,
     &  avh_h_hdrcnt , avh_h_filler0 , avh_h_dataname , avh_h_prblkid ,  
     &  avh_h_satid , 
     &  avh_h_instid , avh_h_datatyp , avh_h_tipsrc , 
     &  avh_h_startdatajd , avh_h_startdatayr , avh_h_startdatady , 
     &  avh_h_startdatatime , avh_h_enddatajd , avh_h_enddatayr , 
     &  avh_h_enddatady , avh_h_enddatatime ,
     &  avh_h_cpidsyr , avh_h_cpidsdy ,  
     &  avh_h_filler1 , avh_h_inststat1 , avh_h_filler2 , 
     &  avh_h_statchrecnb ,  
     &  avh_h_inststat2 , avh_h_scnlin , avh_h_callocscnlin , 
     &  avh_h_misscnlin , avh_h_datagaps , avh_h_okdatafr , 
     &  avh_h_pacsparityerr , avh_h_auxsyncerrsum , avh_h_timeseqerr , 
     &  avh_h_timeseqerrcode , avh_h_socclockupind , avh_h_locerrind , 
     &  avh_h_locerrcode , avh_h_pacsstatfield  , avh_h_pacsdatasrc , 
     &  avh_h_filler3 , avh_h_spare1 , avh_h_spare2 , avh_h_filler4 ,
     &  avh_h_racalind , avh_h_solarcalyr , avh_h_solarcaldy ,
     &  avh_h_pcalalgind, avh_h_pcalalgopt , avh_h_scalalgind ,  
     &  avh_h_scalalgopt , avh_h_irttcoef , avh_h_filler5 ,
     &  avh_h_albcnv , avh_h_radtempcnv , avh_h_filler6 ,
     &  avh_h_modelid , avh_h_nadloctol , avh_h_locbit, avh_h_filler7 ,
     &  avh_h_rollerr , avh_h_pitcherr , avh_h_yawerr ,
     &  avh_h_epoyr , avh_h_epody ,
     &  avh_h_epotime , 
     &  avh_h_smaxis , avh_h_eccen , avh_h_incli , avh_h_argper , 
     &  avh_h_rascnod , avh_h_manom , 
     &  avh_h_xpos , avh_h_ypos ,avh_h_zpos , 
     &  avh_h_xvel , avh_h_yvel , avh_h_zvel , avh_h_earthsun , 
     &  avh_h_filler8 ,
     &  avh_h_pchtemp , avh_h_reserved1 , avh_h_pchtempext , 
     &  avh_h_reserved2 ,
     &  avh_h_pchpow , avh_h_reserved3 , avh_h_rdtemp , 
     &  avh_h_reserved4 , 
     &  avh_h_bbtemp1 , avh_h_reserved5 , avh_h_bbtemp2 , 
     &  avh_h_reserved6 ,
     &  avh_h_bbtemp3 , avh_h_reserved7 , avh_h_bbtemp4 , 
     &  avh_h_reserved8 ,
     &  avh_h_eleccur , avh_h_reserved9 , avh_h_motorcur , 
     &  avh_h_reserved10 ,
     &  avh_h_earthpos , avh_h_reserved11 , avh_h_electemp , 
     &  avh_h_reserved12 ,
     &  avh_h_chtemp , avh_h_reserved13  , avh_h_bptemp , 
     &  avh_h_reserved14 , 
     &  avh_h_mhtemp , avh_h_reserved15 , avh_h_adcontemp , 
     &  avh_h_reserved16 ,
     &  avh_h_d4bvolt , avh_h_reserved17 , avh_h_d5bvolt , 
     &  avh_h_reserved18 , 
     &  avh_h_bbtempchn3B , avh_h_reserved19 , avh_h_bbtempchn4 ,
     &  avh_h_reserved20 ,
     &  avh_h_bbtempchn5 , avh_h_reserved21 , avh_h_refvolt , 
     &  avh_h_reserved22 ,  avh_h_filler9 

!
!##############################################################################
!
!                DATA RECORD 
!
!##############################################################################
!
!                SCAN LINE INFORMATION 
!
       integer*2 avh_scnlin        !scan line number
       integer*2 avh_scnlinyr      !scan line year
       integer*2 avh_scnlindy      !scan line day of year
       integer*2 avh_clockdrift    !satellite clock drift delta in milliseconds
       integer   avh_scnlintime    !scan line UTC time of day in milliseconds
       integer*2 avh_scnlinbit     !scan line bit field
!                bit 15 : (0 = ascending data; 1 = descending data)
!                bit 14 : (1 = scan time corrected for clock drift)
!                bit 13 : (1 = earth location corrected for TIP attitude)
!                bit  0 : channel 3 select (0 = 3A; 1 = 3B)
       integer*2 avh_filler0(5)  !<Zero Fill>
!
!                QUALITY INDICATORS
!    
       integer avh_qualind         !quality indicator bit field
!                bit 31 : (1 = do not use data for product generation)
!                bit 30 : (1 = time sequence error detected in this frame)
!                bit 29 : (1 = data gap precedes this frame)
!                bit 28 : (1 = insufficient data for calibration)
!                bit 27 : (1 = earth location data not available)
!                bit 26 : (1 = sync lock dropped during this frame)
!                bit 25 : (1 = frame sync word error greater than zero)
!                bit 24 : (1 = frame sync previously dropped lock)
!                bit 23 : (1 = flywheeling detected during this frame)
!                bit 22 : (1 = bit slippage detected during this frame)
!                bit  9 : tip parity in first  minor frame
!                bit  8 : tip parity in second minor frame
!                bit  7 : tip parity in third  minor frame
!                bit  6 : tip parity in fourth minor frame
!                bit  5 : tip parity in fifth  minor frame
!                bit  4 : (1 = reflected sunlight detected channel 3B)
!                bit  3 : (1 = reflected sunlight detected channel 4)
!                bit  2 : (1 = reflected sunlight detected channel 5)
!                bit  1 : (1 = resync occured on this frame)
!                bit  0 : (1 = pseudo noise occured on this frame)
       integer avh_scnlinqual    !scan line quality flags  
!
!                Time Problem Code
!
!                bit 31-24 : spare
!                bit 23 : time field is bad but can probably be inferred from 
!                         the previous good time
!                bit 22 : time filed is bad and can't be inferred from
!                         the previous good time
!                bit 21 : this record starts a sequence that is inconsistent 
!                         with previous time (i.e., there is a time
!                         discontinuity).this may or may not associated
!                         with a spacecraft clock update.(see bit 26 above)
!                bit 20 : start of a sequence that apparently repeats scan times
!                         that have been previously accepted.
!                bit 19-16 : spare
!
!                Calibration Problem Code
!
!                (all bits set to 0 indicates normal calibration)
!                bit 15 : scan line was not calibred because of bad time
!                bit 14 : scan line was calibred using fewer than the 
!                         preferred number of scan lines because of proximity
!                         to start or end of data set or to a data gap
!                bit 13 : scan line was not calibred because of bad or
!                         insufficient PRT data
!                bit 12 : scan line was calibred but with marginal PRT data
!                bit 11 : some uncalibred channels on this scan 
!                         (see channel indicators)
!                bit 10 : spare
!                bit 09 : spare
!                bit 08 : spare
!
!                Earth location Problem Code
!
!                (all bits set to 0 implies the earth location was normal)
!                bit 07 : not earth located because of bad time
!                bit 06 : earth location questionable because of 
!                         questionable time code 
!                         (see time problem flags above)
!                bit 05 : earth location questionable -- only marginal
!                          agreement with reasonableness check    
!                bit 04 : earth location questionable -- fails 
!                         reasonnableness check
!                bit 03-00 : spare 
       integer*2 avh_calqual(avh_mxirchn)    !calibration quality flags 
!                order of channels : 3B, 4, 5.
!                (all bits off implies a good calibration)
!                bit 7 : this channel is not calibred
!                bit 6 : this channel is calibred but questionable
!                bit 5 : not good blackboody count for scan line
!                bit 4 : not good space view counts for scan line
!                bit 3 : insufficient PRT data
!                bit 2 : some bad blackboody view counts for this line
!                bit 1 : some bad space view counts for this line
!                bit 0 : some bad PRT data

       integer*2 avh_cbiterr      !count of bit errors in frame sync
       integer*4 avh_filler1(2)   !filler
!
!                CALIBRATION COEFFICIENTS
!       
       integer avh_calvis(5,3,avh_mxvischn) 
!        first index
!                1: operational set
!                2: test set
!                3: prelaunch set
!        second index
!                1   10^10 x (solar calibration nochannel slope 1)
!                2   10^7 x (solar calibration nochannel intercept 1)
!                3   10^10 x (solar calibration nochannel slope 2, active)
!                4   10^7 x (solar calibration nochannel intercept 2)
!                5   solar calibration nochannel  intersection
!       third index 
!                order of channels : 1, 2, 3A.
!                for nochannel = 1 to 3A :

       integer avh_calir(3,2,avh_mxirchn) !IR calibration
!                order of channels : 3B, 4, 5.
!     first index:
! 
!               1 coefficient 1
!               2 coefficient 2
!               3 coefficient 3
!     second index:
!               1 operational
!               2 prelaunch 
!     third index 
!               channel number 
!          we get              
!                for nochannel = 3B to 5 :
!                   10^6 x (IR calibration nochannel coefficient 1, operational)
!                   10^6 x (IR calibration nochannel coefficient 2, operational)
!                   10^6 x (IR calibration nochannel coefficient 3, operational)
!                   10^6 x (IR calibration nochannel coefficient 1, prelaunch)
!                   10^6 x (IR calibration nochannel coefficient 2, prelaunch)
!                   10^6 x (IR calibration nochannel coefficient 3, prelaunch)
!
           integer*4 avh_filler2(3) ! Used to store IR channel calibration coef scaling factors
!
!                NAVIGATION
!
       integer*4 avh_navstat      ! Navigation Status Bit Field
!             bits 31-17: zero fill
!             bit 16: earth location corrected for TIP Euler Angles
!             bits 15 - 12: earth location indicator 
!                     (0 = earth location available; 
!                      1 = user ephemeris files greater than 24 hours old;
!                      2 = no earth location available)

!             bits 11 - 8: spacecraft attitude control 
!                     (0 = operating inYGC or NOMINAL mode;
!                      1 = operating in another mode;
!                      2 = attitude exceeds nominal tolerance;
!                      3 = both 1 and 2)

!            bits 7 - 4: attitude SMODE 
!                     (0 = NOMINAL mode;
!                      1 = rate nulling mode;
!                      2 = YGC mode; 
!                      3 = search mode;
!                      4 = coast mode)
!            bits 3 - 0: attitude PWT1P$AC 
!                     (0 = NOMINAL mode/no test; 
!                      1 = yaw axis test in progress;
!                      2 = roll axis test in progress; 
!                      3 = pitch axis test in progress)       
       integer avh_attangtime   !Time associated with TIP Euler angles(seconds)
       integer*2 avh_rollang    !10^3xRoll Angle in Degrees
       integer*2 avh_pitchang   !10^3xPitch Angle in Degrees
       integer*2 avh_yawang     !10^3xYaw Angle in Degrees

!
       integer*2 avh_scalti      ! 10 x Spacecraft Altitude (MSL) in km     
!                                  in this scan line)
       integer*2 avh_ang(3,51)    !set of 3 angles in degrees for point 25
!                                  to point 2025 every 40 points
!                first angle  : 10^2 x (solar zenith angle )
!                second angle : 10^2 x (satellite zenith angle )
!                third angle  : 10^2 x (relative azimuth angle )
       integer*2 avh_filler3(3)   !Used to store PFS version of avh_calqual,
                                  !containing NEdT in bits 8-15
       integer*4 avh_pos(2,51)    !lat/lon pair in degrees for point 25
!                                  to point 2025 every 40 points
!                first  : 10^4 x (latitude )
!                second : 10^4 x (longitude )
!            (North latitude and East longitude are positive)

       integer*4 avh_filler4(2)      !filler
!
!                HRPT MINOR FRAME TELEMETRY (103 integer*2 words)
!

       integer*2 avh_telem(103)  !telemetry data (HRPT minor frame format)
!                                it corresponds to the 103 first 10 bit words 
!                                from HRPT. They are packed three (10 bit) words
!                                in four bytes, right justified. 
!
      integer*2 avh_filler5    ! <Zero Fill>

!                AVHRR SENSOR DATA
!                      Sensor Data, Band Interleaved by Pixel (BIP)
       integer*2 avh_hrpt(avh_mxhrpchn,avh_mxpix) !avh_mxhrpchn = 5
!                                                  avh_mxpix = 2048
!                       set of 5 channels every point
       integer*2 avh_video(avh_mxhrpchn*avh_mxpix) ! equivalent writing
       integer*4 avh_filler6(2)      !filler

!  THIS PART DOES NOT RESPECT THE NOAA 1B FORMAT SPECIFICATIO
!  BUT IT KEEPS THE POSSIBILITY TO DERIVE THE ANALOG AND DIGITAL
!   TELEMETRY SRTRUCTURE
!
!                TIP MINOR FRAME HEADER
!
       integer*2 avh_tipmfhd(7,5) !the following sequence is repeated 5 times :
!                words 1 to 3 :
!                  bit 47-28 : TIP minor frame sync
!                  bit 27-24 : satellite address
!                  bit 23    : command verification status
!                  bit 22-21 : TIP status
!                  bit 20-18 : TIP major frame counter
!                  bit 17-09 : dwell mode address
!                  bit 08-00 : TIP minor frame counter
!                word  4      : command verification
!                words 5 to 7 :
!                  bit 47-40 :  3.2 second digital B subcom1
!                  bit 39-32 : 32   second analog subcom
!                  bit 31-24 : 16   second analog subcom
!                  bit 23-16 :  1   second analog subcom
!                  bit 15-08 :  3.2 second digital B subcom 2
!                  bit 07-00 : 16   second analog subcom 2
!
!                CPU TELEMETRY
!
       character*6 avh_cputel(2,5) !the following sequence is repeated 5 times :
       integer*2 avh_icputel(6,5)
!                first  : CPU-A telemetry (words TIP 46-51)
!                second : CPU-B telemetry (words TIP 96-101)
!
!                FILLER
       integer*2 avh_filler7(67)    !was 191 prior to v3.2

       integer avh_dt(cavh_size)

       common/avh1bdt/ avh_scnlin , avh_scnlinyr , avh_scnlindy , 
     & avh_clockdrift , avh_scnlintime , avh_scnlinbit ,
     & avh_filler0 , 
     & avh_qualind , avh_scnlinqual , avh_calqual ,
     & avh_cbiterr , avh_filler1 ,
     & avh_calvis , avh_calir , avh_filler2 ,
     & avh_navstat , avh_attangtime , avh_rollang , avh_pitchang , 
     & avh_yawang ,
     & avh_scalti , avh_ang , avh_filler3 ,  avh_pos , avh_filler4 ,
     & avh_telem , avh_filler5 ,
     & avh_video , avh_filler6 ,
     & avh_tipmfhd,  avh_icputel ,
     & avh_filler7

       equivalence (avh_scnlin,avh_dt(1))
       equivalence (avh_hrpt,avh_video)
       equivalence (avh_cputel,avh_icputel)

