!------------------------------------------------------------------------------
!  %Z%%M% version %I% %G%  (C) NWP SAF
!------------------------------------------------------------------------------             
!------------------------------------------------------------------------------
!NAME  
! avh1c_F90.h 
! 
!PURPOSE 
! include file for avhrr 1c format handling
!  
!CODE OWNER  
! NWP SAF
!
!HISTORY (CHANGES) 
!(version   date      comment           <author>)
!  1.0     May 09      initial         T. Labrot (MF)
!
!COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!------------------------------------------------------------------------------


!###########################################################################
!                     AVHRR L1C version
!###########################################################################

!Number of header record in the data set 
        integer nb_avh1c_hd
        parameter ( nb_avh1c_hd = 1 )
        
!Identify the version of the avhrr l1c format        
        integer avh1c_vers
        parameter (avh1c_vers = 1)
        integer avh1c_versyr
        parameter (avh1c_versyr = 2009)
        integer avh1c_versdy
        parameter (avh1c_versdy = 91)


!###########################################################################
!                     PARAMETERS DEFINITIONS
!###########################################################################

       integer avh1c_mxirchn            !IR channels of AVHRR
       integer avh1c_mxvischn           !visible channels of AVHRR
       integer avh1c_mxhrpchn           !count of AVHRR channels in HRPT stream
       parameter(avh1c_mxirchn=3,  &
                 avh1c_mxvischn=3, &
                 avh1c_mxhrpchn=5)
       integer avh1c_mxpix              !maximun pixels number
       parameter(avh1c_mxpix=2048) 
       
       integer avh1c_record_size_byte   !1C record length ( in bytes)
       parameter (avh1c_record_size_byte = 29808)
       integer avh1c_record_size_i4     !1C record length ( in integer*4)
       parameter (avh1c_record_size_i4 = avh1c_record_size_byte/4)
       
       
!###########################################################################
!
!                     AVHRR/HRPT 1C Data Set 
!
!###########################################################################       

!###########################################################################
!
!                         HEADER RECORD
!
!###########################################################################

!		        GENERAL INFORMATION


      character*3  avh1c_h_site       !dataset creation site ID
      character*1  avh1c_h_cfill1     !filler
      character*3  avh1c_h_1bsite     !creation site for original 1B data
      character*1  avh1c_h_cfill2     !filler
      integer*4    avh1c_h_versnb     !level 1c format version number
      integer*4    avh1c_h_versyr     !level 1c format version year
      integer*4    avh1c_h_versdy     !level 1c format version day of year
      integer*4    avh1c_h_hdrcnt     !count of header records in this data set
!      integer*4    avh1c_h_satid      !satellite id (4=noaa15;2=noaa16;6=noaa17;7=noaa18;
!                                     !8=noaa19;11=metop1;12=metopA;...)
      integer*4    avh1c_h_satid      !satellite id (e.g. 14 for NOAA-14)
      integer*4    avh1c_h_instrument !instrument identification
!                                     !(1=AVHRR)
      integer*4    avh1c_h_satht      !nominal satellite altitude, km*10
      integer*4    avh1c_h_period     !nominal orbit period (seconds)
      integer*4    avh1c_h_startorbit !orbit number (at start of file)
      integer*4    avh1c_h_startdatayr!start of data set year
      integer*4    avh1c_h_startdatady!start of data set day of the year
      integer*4    avh1c_h_startdatatime !start of data set UTC time of day (ms)
      integer*4    avh1c_h_endorbit   !orbit number (at end of file)
      integer*4    avh1c_h_enddatayr  !end of data set year
      integer*4    avh1c_h_enddatady  !end of data set day of the year
      integer*4    avh1c_h_enddatatime!end of data set UTC time of day (ms)
      integer*4    avh1c_h_scnlin     !count of scan lines in this data set
      integer*4    avh1c_h_misscnlin  !count of missing scan lines 
      integer*4    avh1c_h_callocscnlin !count of calibrated, earth located
!                                     !scan lines in this data set
      integer*4    avh1c_h_datagaps   !count of data gaps in this data set 
      integer*4    avh1c_h_datatyp    !data type code
!       			      !(1 = LAC; 2 = GAC; 3 = HRPT)      
      
      character*68 avh1c_h_infilename !input file name (allow pfs name)
      
      integer*4    avh1c_h_wmosatid   !WMO satellite id (3 for METOP-1)            

!                CALIBRATION INFORMATION

      integer*4    avh1c_h_cpidsyr    !year of Last CPIDS Update
      integer*4    avh1c_h_cpidsdy    !Day of Year of Last CPIDS Update

!                NAVIGATION INFORMATION               

      integer*4   avh1c_h_rollerr    !10^3 x (constant roll attitude error in
!                                            degrees)
      integer*4   avh1c_h_pitcherr   !10^3 x (constant pitch attitude error in
!                                            degrees)
      integer*4   avh1c_h_yawerr     !10^3 x (constant yaw attitude error in
!                                            degrees)
 
           
!                RADIANCE TO TEMPERATURE CONVERSION

      integer*4 avh1c_h_albcnv(2,avh1c_mxvischn)
!                albedo conversion 
!                order of channels : 1, 2, 3A.
!                for nochannel = 1 to 3A :
!                  10^1 x (albedo-radiance nochannel solar filtered irradiance 
!                          in wavelength)
!                  10^3 x (albedo-radiance nochannel equivalent filter width 
!                          in wavelength)

       integer*4 avh1c_h_radtempcnv(3,avh1c_mxirchn)
!                radiance to temperature conversion 
!                order of channels : 3B, 4, 5.
!                for nochannel = 3B to 5:
!            10^2 x (temperature-radiance nochannel central wavenumber Ch3b)
!        or  10^3 x (temperature-radiance nochannel central wavenumber Ch4,5)
!                  10^5 x (temperature-radiance nochannel constant 1)
!                  10^6 x (temperature-radiance nochannel constant 2)

       integer*4 avh1c_h_filler(7388)         ! Filler <Zero Fill>

       integer*4 avh1c_hd(avh1c_record_size_i4)
       
       equivalence ( avh1c_hd(1) , avh1c_h_site )


       common/avh1chd/avh1c_h_site , avh1c_h_cfill1 ,	&
     & avh1c_h_1bsite , avh1c_h_cfill2 ,		&
     & avh1c_h_versnb , avh1c_h_versyr , avh1c_h_versdy ,  &
     & avh1c_h_hdrcnt , 				&   
     & avh1c_h_satid , avh1c_h_instrument ,		&    
     & avh1c_h_satht , avh1c_h_period , 		&
     & avh1c_h_startorbit ,				&
     & avh1c_h_startdatayr , avh1c_h_startdatady , 	&
     & avh1c_h_startdatatime ,				&
     & avh1c_h_endorbit , 				&  
     & avh1c_h_enddatayr , avh1c_h_enddatady, avh1c_h_enddatatime ,  &
     & avh1c_h_scnlin , avh1c_h_misscnlin , avh1c_h_callocscnlin ,   & 
     & avh1c_h_datagaps ,				&
     & avh1c_h_datatyp ,				&   
     & avh1c_h_infilename ,				&  
     & avh1c_h_wmosatid ,  				&
     & avh1c_h_cpidsyr , avh1c_h_cpidsdy  ,		& 
     & avh1c_h_rollerr , avh1c_h_pitcherr , avh1c_h_yawerr ,  &
     & avh1c_h_albcnv , avh1c_h_radtempcnv ,		&
     & avh1c_h_filler     
     
!###########################################################################
!
!                         DATA RECORD
!
!###########################################################################

!                SCAN LINE INFORMATION 

       integer*4 avh1c_scnlin        !scan line number
       integer*4 avh1c_scnlinyr      !scan line year
       integer*4 avh1c_scnlindy      !scan line day of year
       integer*4 avh1c_scnlintime    !scan line UTC time of day in milliseconds
       integer*4 avh1c_clockdrift    !satellite clock drift delta in milliseconds
       integer*4 avh1c_scnlinbit     !scan line bit field
!                    bit 15 : (0 = ascending data; 1 = descending data)
!                    bit 14 : (1 = scan time corrected for clock drift)
!                    bit 13 : (1 = earth location corrected for TIP attitude)
!                    bit  0 : channel 3 select (0 = 3A; 1 = 3B)

       integer*4 avh1c_spare0(2)     !<Zero Fill>

!                QUALITY INDICATORS
    
       integer*4 avh1c_qualind         !quality indicator bit field
!                    bit 31 : (1 = do not use data for product generation)
!                    bit 30 : (1 = time sequence error detected in this frame)
!                    bit 29 : (1 = data gap precedes this frame)
!                    bit 28 : (1 = insufficient data for calibration)
!                    bit 27 : (1 = earth location data not available)
!                    bit 26 : (1 = sync lock dropped during this frame)
!                    bit 25 : (1 = frame sync word error greater than zero)
!                    bit 24 : (1 = frame sync previously dropped lock)
!                    bit 23 : (1 = flywheeling detected during this frame)
!                    bit 22 : (1 = bit slippage detected during this frame)
!                    bit  9 : tip parity in first  minor frame
!                    bit  8 : tip parity in second minor frame
!                    bit  7 : tip parity in third  minor frame
!                    bit  6 : tip parity in fourth minor frame
!                    bit  5 : tip parity in fifth  minor frame
!                    bit  4 : (1 = reflected sunlight detected channel 3B)
!                    bit  3 : (1 = reflected sunlight detected channel 4)
!                    bit  2 : (1 = reflected sunlight detected channel 5)
!                    bit  1 : (1 = resync occured on this frame)
!                    bit  0 : (1 = pseudo noise occured on this frame)

       integer*4 avh1c_scnlinqual    !scan line quality flags  

!                Time Problem Code

!                bit 31-24 : spare
!                bit 23 : time field is bad but can probably be inferred from 
!                         the previous good time
!                bit 22 : time filed is bad and can't be inferred from
!                         the previous good time
!                bit 21 : this record starts a sequence that is inconsistent 
!                         with previous time (i.e., there is a time
!                         discontinuity).this may or may not associated
!                         with a spacecraft clock update.(see bit 26 above)
!                bit 20 : start of a sequence that apparently repeats scan times
!                         that have been previously accepted.
!                bit 19-16 : spare

!                Calibration Problem Code

!                (all bits set to 0 indicates normal calibration)
!                bit 15 : scan line was not calibred because of bad time
!                bit 14 : scan line was calibred using fewer than the 
!                         preferred number of scan lines because of proximity
!                         to start or end of data set or to a data gap
!                bit 13 : scan line was not calibred because of bad or
!                         insufficient PRT data
!                bit 12 : scan line was calibred but with marginal PRT data
!                bit 11 : some uncalibred channels on this scan 
!                         (see channel indicators)
!                bit 10 : spare
!                bit 09 : spare
!                bit 08 : spare

!                Earth location Problem Code

!                (all bits set to 0 implies the earth location was normal)
!                bit 07 : not earth located because of bad time
!                bit 06 : earth location questionable because of 
!                         questionable time code 
!                         (see time problem flags above)
!                bit 05 : earth location questionable -- only marginal
!                          agreement with reasonableness check    
!                bit 04 : earth location questionable -- fails 
!                         reasonnableness check
!                bit 03-00 : spare 

       integer*4 avh1c_calqual(avh1c_mxirchn)    !calibration quality flags
        
!                order of channels : 3B, 4, 5.

!                (all bits off implies a good calibration)
!                bit 7 : this channel is not calibred
!                bit 6 : this channel is calibred but questionable
!                bit 5 : not good blackboody count for scan line
!                bit 4 : not good space view counts for scan line
!                bit 3 : insufficient PRT data
!                bit 2 : some bad blackboody view counts for this line
!                bit 1 : some bad space view counts for this line
!                bit 0 : some bad PRT data

       integer*4 avh1c_spare1(2)   !<Zero Fill>

!                NAVIGATION

       integer*4 avh1c_scalti        ! 10 x Spacecraft Altitude (MSL) in km     
!                                    !in this scan line)
       integer*4 avh1c_ang(3,51)     !set of 3 angles in degrees for point 25
!                                    !to point 2025 every 40 points
!                         first angle  : 10^2 x (solar zenith angle )
!                         second angle : 10^2 x (satellite zenith angle )
!                         third angle  : 10^2 x (relative azimuth angle )

       integer*4 avh1c_pos(2,51)     !lat/lon pair in degrees for point 25
!                                    !to point 2025 every 40 points
!                              first  : 10^4 x (latitude )
!                              second : 10^4 x (longitude )
!                              (North latitude and East longitude are positive)

       integer*4 avh1c_spare2(2)      !<Zero Fill>
       
!
!                 AVHRR SENSOR DATA

       integer*2 avh1c_btemps(avh1c_mxhrpchn,avh1c_mxpix)
					! avh1c_mxpix = 2048 for pixel in line
					! avh1c_mxhrpchn = 5 for channels every pixel
					! 100*% albedo
					! 100*K IR channels
					
!                AVHRR CLOUD MASK
      
       integer avh1c_maskind   	!cloud mask done =1 

       integer*2 avh1c_mask(avh1c_mxpix)
        ! avh1c_mxpix = 2048 for pixel in line
	
	! bits 15-14:	0	Night
	!		1	Twilight
	!		2	Day
	!		3	Sunglint
	! bit 13 : 	1	sea
	!		0	land or coast
	! bit 12 : 	1	cloudy
	!		0	clear
	! bit 11 : 	1	blackbody (only filled if bit12 =1)
	! bits 10-6:	0	non-processed: containing no data or corrupted data
	!	   	1	cloud free land	no contamination by snow/ice covered surface, 
	!				no contamination by clouds ; 
	!				but contamination by thin dust/volcanic clouds not checked
	!	   	2	cloud free sea	no contamination by snow/ice covered surface, 
	!				no contamination by clouds ; 
	!				but contamination by thin dust/volcanic clouds not checked
	!	   	3	land contaminated by snow
	!	   	4	sea contaminated by snow/ice
	!	   	5	very low and cumuliform clouds
	!	   	6	very low and stratiform clouds
	!	   	7	low and cumuliform clouds
	!	   	8	low and stratiform clouds
	!	   	9	medium and cumuliform clouds
	!	   	10	medium and stratiform clouds
	!	   	11	high opaque and cumuliform clouds
	!	   	12	high opaque and stratiform clouds
	!	   	13	very high opaque and cumuliform clouds
	!	   	14	very high opaque and stratiform clouds
	!	   	15	high semitransparent thin clouds
	!	   	16	high semitransparent meanly thick clouds
	!	   	17	high semitransparent thick clouds
	!	   	18	high semitransparent above low or medium clouds
	!	   	19	fractional clouds (sub-pixel water clouds)
	!	   	20	undefined (undefined by CMa)
	! bit 5:	0	Ts background : climatology used
	!		1	Ts background : forecast used 
	! bits 4-3:	0	WV content : AMSU used
	!		1	WV content : forecast used
	!		2	WV content : climatology used
	! bit 2-1:	quality flag:	0= same CMa, 1= bad data, 
	!				2= different CMa, 3= coast
	! bit 0 :	for marine cloudy  =1 , clear=0
  
				
       integer*2  avh1c_tsurf(avh1c_mxpix) !retrieved temperature in C*100 
				!if clear: retrieved SST (sea) ot Tsurf(land)
				!if cloudy and blackbody: cloud top temperature
				!0=missing values
				
       integer*4 avh1c_spare3(10)      !<Zero Fill>
       

       integer*4 avh1c_dt(avh1c_record_size_i4)
       
       equivalence (avh1c_dt(1),avh1c_scnlin)

       common/avh1cdt/ avh1c_scnlin , avh1c_scnlinyr , avh1c_scnlindy ,  & 
     & avh1c_scnlintime , avh1c_clockdrift ,  avh1c_scnlinbit ,	&
     & avh1c_spare0 ,						&
     & avh1c_qualind , avh1c_scnlinqual , avh1c_calqual ,	&
     & avh1c_spare1 ,						&
     & avh1c_scalti , avh1c_ang ,  avh1c_pos ,			&
     & avh1c_spare2 ,						&
     & avh1c_btemps , avh1c_maskind , avh1c_mask ,  avh1c_tsurf ,	&
     & avh1c_spare3
