!+ Fundamental and physical constants
!
! Name:  constants_preproc.h
!
! Purpose:
!   'INCLUDE' file containing fundamental and geophysical constants for use
!   within atovin and atovpp code.
!
! Current Code Owner: N.C.Atkinson
!
! History:
! Version    Date     Comment
!
!  1.0       15/9/04  Original code. (N.C.Atkinson)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!

      REAL          PI                      ! PI
         PARAMETER (PI=3.141592654)
      REAL          deg2rad                 ! convert degrees to radians
         PARAMETER (deg2rad=PI/180.)
      REAL          rad2deg                 ! convert radians to degrees
         PARAMETER (rad2deg=1./deg2rad)

! The following values were used up to AAPP version 4:
! Constants from Cohen & Taylor "Physics Today" pp 9-13 Aug 1992
! Constants from the "Dictionary of Physics", Penguin Press, ed. V.H.Pitt.
!      REAL          C_1                     ! first radiation constant
!         PARAMETER (C_1=1.191044E-5)        !   mW/m2/ster/cm^-4
!      REAL          C_2                     ! second radiation constant
!         PARAMETER (C_2=1.438769)           !   K/cm^-1
!      REAL          C_Planck                ! Planck constant
!         PARAMETER (C_Planck=6.6260755E-34) !   J s
!      REAL          C_Boltzman              ! Boltzman constant
!         PARAMETER (C_Boltzman=1.380658E-23)!   J/K
!      REAL          C_light                 ! Speed of light
!         PARAMETER (C_light=299792458.0)    !   m/s

! For AAPP version 5, standardise on the values currently used by NESDIS. These
! are the 1998 CODATA constants and can be found at
! http://www2.ncdc.noaa.gov/docs/klm/html/p/app-p.htm
! The 2002 CODATA update is not significantly different

      REAL          C_1                      ! first radiation constant
         PARAMETER (C_1=1.1910427E-5)        !   mW/m2/ster/cm^-4
      REAL          C_2                      ! second radiation constant
         PARAMETER (C_2=1.4387752)           !   K/cm^-1
      REAL          C_Planck                 ! Planck constant
         PARAMETER (C_Planck=6.62606876E-34) !   J s
      REAL          C_Boltzman               ! Boltzman constant
         PARAMETER (C_Boltzman=1.3806503E-23)!   J/K
      REAL          C_light                  ! Speed of light
         PARAMETER (C_light=299792458.0)     !   m/s

!
! Earth radius as used in AAPP version 4
      REAL          rEarth                  ! mean Earth radius
         PARAMETER (rEarth=6371.03)         !   km

! Earth semi-major axis and difference between semi-major and semi-minor
      REAL          semiMajor, semiDiff
         PARAMETER (semiMajor=6378137.0)    !   m
         PARAMETER (semiDiff=21384.7)       !   m
