!+ CrIS level 1C Data Setn - holds data for one scan
!
! Name:  cris1c.h
!
! Purpose:
!   'INCLUDE' file for common /cris1chd/   (cris level 1C header record)
!                  and common /cris1cdt/   (cris level 1C data record)
!
! Can be used for full channel dataset, channel selection or PC score
! representation if needed.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  0.1    16/07/2010  Original code. Nigel Atkinson.
!  1.1    05/05/2016  Full spectral resolution
!  1.2    15/06/2017  Add cris1c_extraradiance (4 channels per band)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!   User adjustable parameters:
      integer       cris_1cnchan        ! Max number of channels in format
         parameter (cris_1cnchan = 2211)!  2211 for FSR, 1305 for standard res
      integer       cris_1cnpc          ! Number of PCs in format (at least 1)
         parameter (cris_1cnpc = 1)

!   Normally fixed parameters:
      integer       cris_1cnfov         ! Number of fields of view
         parameter (cris_1cnfov = 9)
      integer       cris_1cnfor         ! Number of fields of regard (scan pos)
         parameter (cris_1cnfor = 30)
      integer       cris_1cnband        ! Number of bands
         parameter (cris_1cnband = 3)
      integer       cris_1cvnnb         ! format version number
         parameter (cris_1cvnnb = 3)
      integer       cris_1cvnyr         ! format version year
         parameter (cris_1cvnyr = 2017)
      integer       cris_1cvndy         ! format version day
         parameter (cris_1cvndy = 166)

      integer       array_size_cris1c   ! Length of array
         parameter (array_size_cris1c = 
     &         11 + cris_1cnfor + cris_1cnband*cris_1cnfov + 
     &         (13 + 2*cris_1cnband + cris_1cnchan +12 + cris_1cnpc)
     &              *cris_1cnfov*cris_1cnfor)

      integer       array_size_cris1cspare   ! Length of spare header array
         parameter (array_size_cris1cspare = 
     &         array_size_cris1c - 36 - cris_1cnchan - 4*cris_1cnband)

      integer       cris_1crecl         ! Record length in bytes
         parameter (cris_1crecl = array_size_cris1c*4)
!
!###########################################################################
!
!               HEADER RECORD
!
!###########################################################################

      character*3  cris1c_h_site       !dataset creation site ID
      character*1  cris1c_h_cfill1     !filler
      character*3  cris1c_h_1bsite     !creation site for original 1B data
      character*1  cris1c_h_cfill2     !filler
      integer*4    cris1c_h_versnb     !level 1c format version number
      integer*4    cris1c_h_versyr     !level 1c format version year
      integer*4    cris1c_h_versdy     !level 1c format version day of year
      integer*4    cris1c_h_hdrcnt     !count of header records in this data set
      integer*4    cris1c_h_satid      !satellite id in series (1 for NPP)
      integer*4    cris1c_h_instrument !WMO instrument identifier: 620=CrIS
      integer*4    cris1c_h_satht      !nominal satellite altitude, km*10
      integer*4    cris1c_h_period     !nominal orbit period (seconds)
      integer*4    cris1c_h_startorbit !orbit number (at start of file)
      integer*4    cris1c_h_startdatayr!start of data set year
      integer*4    cris1c_h_startdatady!start of data set day of the year
      integer*4    cris1c_h_startdatatime !start of data set UTC time of day(ms)
      integer*4    cris1c_h_endorbit   !orbit number (at end of file)
      integer*4    cris1c_h_enddatayr  !end of data set year
      integer*4    cris1c_h_enddatady  !end of data set day of the year
      integer*4    cris1c_h_enddatatime!end of data set UTC time of day (ms)
      integer*4    cris1c_h_scnlin     !count of scan lines in this data set
      integer*4    cris1c_h_misscnlin  !count of missing scan lines
      integer*4    cris1c_h_spare(2)   !Spare
      integer*4    cris1c_h_wmosatid   !WMO satellite id (224 for NPP)
      integer*4    cris1c_h_spare1     !spare
      integer*4    cris1c_h_centre     !WMO id of reception centre
      integer*4    cris1c_h_subcentre  !WMO id of reception subcentre

! CrIS variables

      integer*4    cris1c_h_recwords       !Size of record in integer*4 words
      integer*4    cris1c_h_nfov           !Number of FOVs
      integer*4    cris1c_h_nfor           !Number of FORs
      integer*4    cris1c_h_nband          !Number of bands
      integer*4    cris1c_h_startw(cris_1cnband)    !First wavenumber *1E2
      integer*4    cris1c_h_endw(cris_1cnband)      !Last wavenumber *1E2
      integer*4    cris1c_h_startchan(cris_1cnband) !First channel in band
      integer*4    cris1c_h_endchan(cris_1cnband)   !Last channel in band
      integer*4    cris1c_h_eigvn          !Version number of eigenvectors
      integer*4    cris1c_h_npc            !Number of PCs in format
      integer*4    cris1c_h_npcused        !Number of PCs used
      integer*4    cris1c_h_chanvn         !Version number of channel selection
      integer*4    cris1c_h_nchan          !Number of channels in format
      integer*4    cris1c_h_nchanused               !Number of channels used
      integer*4    cris1c_h_channels(cris_1cnchan)  !List of CrIS channels
                                                    ! range 1 to 1317
      integer*4    cris1c_h_spare2(array_size_cris1cspare)  !spare

      integer*4 cris1c_hd(array_size_cris1c)
      equivalence(cris1c_hd(1),cris1c_h_site)
      common /cris1chd/ cris1c_h_site, cris1c_h_cfill1, cris1c_h_1bsite,
     & cris1c_h_cfill2, cris1c_h_versnb, cris1c_h_versyr, 
     & cris1c_h_versdy, cris1c_h_hdrcnt, cris1c_h_satid, 
     & cris1c_h_instrument, cris1c_h_satht,
     & cris1c_h_period, cris1c_h_startorbit, cris1c_h_startdatayr,
     & cris1c_h_startdatady, cris1c_h_startdatatime, cris1c_h_endorbit,
     & cris1c_h_enddatayr, cris1c_h_enddatady, cris1c_h_enddatatime,
     & cris1c_h_scnlin, cris1c_h_misscnlin, cris1c_h_spare,
     & cris1c_h_wmosatid, cris1c_h_spare1,cris1c_h_centre,
     & cris1c_h_subcentre, cris1c_h_recwords,cris1c_h_nfov,
     & cris1c_h_nfor,cris1c_h_nband, cris1c_h_startw,cris1c_h_endw,
     & cris1c_h_startchan,cris1c_h_endchan,
     & cris1c_h_eigvn, cris1c_h_npc, cris1c_h_npcused, cris1c_h_chanvn,
     & cris1c_h_nchan, cris1c_h_nchanused, cris1c_h_channels,
     & cris1c_h_spare2
     
      character*16 cris1c_h_software_version
      equivalence(cris1c_h_spare2(1),cris1c_h_software_version)

!##############################################################################
!
!               DATA RECORD
!
!##############################################################################

      integer*4   cris1c_scnlin       !scan line number
      integer*4   cris1c_scnlinyr     !scan line year
      integer*4   cris1c_scnlindy     !scan line day of year
      integer*4   cris1c_scnlintime(cris_1cnfor) !scan line UTC time of day (ms)

      integer*4   cris1c_latlon(2,cris_1cnfov,cris_1cnfor)
!                     first   : 10^4 x (latitude)
!                     second  : 10^4 x (longitude)
      integer*4   cris1c_angles(4,cris_1cnfov,cris_1cnfor)
!                     first:    10^2 x (local zenith angle)
!                     second:   10^2 x (local azimuth angle)
!                     third:    10^2 x (solar zenith angle)
!                     fourth:   10^2 x (solar azimuth angle)
      integer*4   cris1c_satxyz(3)    ! x,y,z of satellite at mid-scan
      integer*4   cris1c_ascdesc      ! ascending / descending orbit qualifier
      integer*4   cris1c_orbit        ! orbit number
      integer*4   cris1c_scalti       ! satellite altitude
      integer*4   cris1c_surfht(cris_1cnfov,cris_1cnfor)       ! surface height
      integer*4   cris1c_landfrac(cris_1cnfov,cris_1cnfor)     ! land fraction
      integer*4   cris1c_landsea(cris_1cnfov,cris_1cnfor)      ! land/sea qualifier
      integer*4   cris1c_cloudcover(cris_1cnfov,cris_1cnfor)   ! cloud cover (percent)
      integer*4   cris1c_cloudtopht(cris_1cnfov,cris_1cnfor)   ! height of top of cloud (m)

      integer*4   cris1c_radianceflgs(cris_1cnfov,cris_1cnfor) ! radiance flags
!Bit(1=MS)
!1	Clear path
!2	Partly cloudy path
!3	Cloudy path
!4	Apodized
!5	Unapodized
!6	Reconstructed
!7	Cloud cleared
!8-14	Reserved
!All 15	Missing value

      integer*4 cris1c_scanqualflgs              ! scan quality flags
!Bit(1=MS)
!1	Gap in Raw Data Record (RDR) data detected (i.e., missing scan(s)
!       preceding the current scan)
!2	Recorded time is not in sequence (i.e., the scan start time is out of
!       sequence)
!3	Lamda monitored calculation cannot be updated
!4	The measured temperatures of any instrument components (e.g., 
!       beam-splitter, scan mirror, scan baffle, etc.) are outside the
!       allowable ranges
!5	At least one of the monitored instrument temperatures has drifted more
!       than a specified tolerance value
!6-12	Reserved
!All 13	Missing value

      integer*4   cris1c_calqualflgs(cris_1cnband,cris_1cnfov)   !calibration quality
!Bit(1=MS)
!1	Lunar intrusion on first deep space view (see Note)
!2	Lunar intrusion on second deep space view (see Note)
!3-8	Reserved
!All 9	Missing value

      integer*4 cris1c_fovqualflgs(cris_1cnband,cris_1cnfov,
     & cris_1cnfor) !FOV quality
!Bit(1=MS)
!1	Degraded SDR quality
!2	Invalid SDR quality
!3	Invalid SDR geolocation information
!4	Degraded radiometric calibration
!5	Invalid radiometric calibration
!6	Degraded spectral calibration
!7	Invalid spectral calibration
!8	Fringe count error detected and corrected
!9	Day/night indicator
!10	Invalid RDR data
!11	Significant fringe count error detected
!12	Bit trim failed
!13-18	Reserved
!All 19	Missing value

      integer*4   cris1c_geolocqual                !Geolocation quality
!0  Nominal
!1  Missing at most a small gap of altitude and Ephemeris data
!2  Missing more than a small gap of altitude and Ephemeris data, but no more
!   than a granule boundary
!3  Missing more than a granule boundary of altitude and Ephemeris data
!4-14 Reserved
!15 Missing

      integer*4   cris1c_quality(cris_1cnfov,cris_1cnfor)  !Quality information
!0  Not suspect                                                                  
!1  Suspect                                                                      
!2  Very suspect                                                                 
!3  Unusable                                                                     
      
      integer*4   cris1c_scores(cris_1cnpc,cris_1cnfov,cris_1cnfor)    !PC scores
      integer*4   cris1c_logfit(cris_1cnband,cris_1cnfov,cris_1cnfor)  !Log Reconstruction Scores

!For radiance, put the channel index last, to allow shorter record length 
!to be used in the case of a channel subset

      integer*4  cris1c_extraradiance(cris_1cnfov,cris_1cnfor,12)      !12 guard channels
      integer*4  cris1c_radiance(cris_1cnfov,cris_1cnfor,cris_1cnchan) !Radiance
                                                                       !Wm-2sr-1cm *1E7

      integer*4 cris1c_dt(array_size_cris1c)
      equivalence(cris1c_dt(1),cris1c_scnlin)
      common /cris1cdt/ cris1c_scnlin,cris1c_scnlinyr,cris1c_scnlindy,
     & cris1c_scnlintime,cris1c_latlon,cris1c_angles,
     & cris1c_satxyz,cris1c_ascdesc,cris1c_orbit,cris1c_scalti,
     & cris1c_surfht,cris1c_landfrac,cris1c_landsea,cris1c_cloudcover,
     & cris1c_cloudtopht,cris1c_radianceflgs,cris1c_scanqualflgs,
     & cris1c_calqualflgs,cris1c_fovqualflgs,cris1c_geolocqual,
     & cris1c_quality,cris1c_scores,cris1c_logfit,cris1c_extraradiance,
     & cris1c_radiance

