!+ Level 1D Data Set (ATMS on CrIS grid)
!
! Name:  cris1d.h
!
! Purpose:
!   'INCLUDE' file for common /cris1dhd/   (CrIS level 1D header record)
!                  and common /cris1ddt/   (CrIS level 1D data record)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    13/08/2010  Original code
!  1.1    23/11/2017  399 -> 431 channels to accommodate FSR channel selection
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!   User adjustable parameters:
      integer       cris_1dnchan        ! Number of channel slots present in 1d
         parameter (cris_1dnchan = 431) !  Must be at least 1
      integer       cris_1dnpc          ! Number of Principal Components present
         parameter (cris_1dnpc = 1)     !  Must be at least 1
!
!   Normally fixed parameters:
      integer       cris_1dnchanatms    ! Number of ATMS channels
         parameter (cris_1dnchanatms = 22)
      integer       cris_1dnfor         ! Number of FORs present in 1d
         parameter (cris_1dnfor = 30)   !  Full resolution requires 9 records
      integer       cris_1dnband        ! Number of bands
         parameter (cris_1dnband = 3)
      integer       cris_1dvnnb         ! cris level 1D format version number
         parameter (cris_1dvnnb = 1)
      integer       cris_1dvnyr         ! format version year
         parameter (cris_1dvnyr = 2010)
      integer       cris_1dvndy         ! format version day
         parameter (cris_1dvndy = 225)
      integer       cris_1dnchanviirs   ! Number of VIIRS channels in format
         parameter (cris_1dnchanviirs = 22)

      integer       array_size_cris1d   ! Length of array
         parameter (array_size_cris1d = 
     &         11 + cris_1dnfor*(53 + cris_1dnchanatms + cris_1dnchan + 
     &              cris_1dnpc + 2*cris_1dnband + 2*cris_1dnchanviirs) 
     &            + 2*cris_1dnchanatms)              

      integer       array_size_cris1dspare   ! Length of spare header array
         parameter (array_size_cris1dspare = 
     &      array_size_cris1d - 49 - cris_1dnchan - 7*cris_1dnchanatms
     &      - 4*cris_1dnband - 2*cris_1dnchanviirs)

      integer       cris_1drecl         ! Record length in bytes
         parameter (cris_1drecl = array_size_cris1d*4)
!
!###########################################################################
!
!               HEADER RECORD
!
!###########################################################################
!
      character*3  cris1d_h_site       !dataset creation site ID
      character*1  cris1d_h_cfill1     !filler
      character*3  cris1d_h_1bsite     !creation site for original 1B data
      character*1  cris1d_h_cfill2     !filler
      integer*4    cris1d_h_versnb     !level 1d format version number
      integer*4    cris1d_h_versyr     !level 1d format version year
      integer*4    cris1d_h_versdy     !level 1d format version day of year
      integer*4    cris1d_h_hdrcnt     !count of header records in this data set
      integer*4    cris1d_h_satid      !satellite id in series (1 for NPP)
      integer*4    cris1d_h_grid       !code for instrument grid
!            (5=HIRS; 6=MSU; 10=AMSU-A; 11=AMSU-B; 12=IASI; 13=ATMS; 14=CrIS)
      integer*4    cris1d_h_satht      !nominal satellite altitude, km*10
      integer*4    cris1d_h_period     !nominal orbit period (seconds)
      integer*4    cris1d_h_startorbit !orbit number (at start of file)
      integer*4    cris1d_h_startdatayr!start of data set year
      integer*4    cris1d_h_startdatady!start of data set day of the year
      integer*4    cris1d_h_startdatatime !start of data set UTC time of day (ms)
      integer*4    cris1d_h_endorbit   !orbit number (at end of file)
      integer*4    cris1d_h_enddatayr  !end of data set year
      integer*4    cris1d_h_enddatady  !end of data set day of the year
      integer*4    cris1d_h_enddatatime!end of data set UTC time of day (ms)
      integer*4    cris1d_h_scnlin     !count of scan lines in this data set
      integer*4    cris1d_h_misscnlin  !count of missing scan lines
      integer*4    cris1d_h_atovppvn   !ATOVPP version number (test vns = 9000+)
      integer*4    cris1d_h_instruments!instruments present
!       (bit0=HIRS, bit1=MSU, bit3=AMSU-A, bit4=AMSU-B, bit5=AVHRR, bit 6=IASI,
!        bit7=ATMS, bit8=CrIS)
      integer*4    cris1d_h_wmosatid   !WMO satellite id (224 for NPP)
      integer*4    cris1d_h_spare1     !spare
      integer*4    cris1d_h_centre     !WMO id of reception centre
      integer*4    cris1d_h_subcentre  !WMO id of reception subcentre

! CrIS variables

      integer*4    cris1d_h_recwords       !Size of record in integer*4 words
      integer*4    cris1d_h_nfov           !Number of FOVs
      integer*4    cris1d_h_nfor           !Number of FORs
      integer*4    cris1d_h_nband          !Number of bands
      integer*4    cris1d_h_startw(cris_1dnband)    !First wavenumber *1E2
      integer*4    cris1d_h_endw(cris_1dnband)      !Last wavenumber *1E2
      integer*4    cris1d_h_startchan(cris_1dnband) !First channel in band
      integer*4    cris1d_h_endchan(cris_1dnband)   !Last channel in band
      integer*4    cris1d_h_eigvn          !Version number of eigenvectors
      integer*4    cris1d_h_npc            !Number of PCs in format
      integer*4    cris1d_h_npcused        !Number of PCs used
      integer*4    cris1d_h_chanvn         !Version number of channel selection
      integer*4    cris1d_h_nchan          !Number of channels in format
      integer*4    cris1d_h_nchanused               !Number of channels used
      integer*4    cris1d_h_channels(cris_1dnchan)  !List of cris channels
                                                    ! range 1 to 8461

! ATMS variables

! The following variables characterise any image processing for ATMS
      integer*4    cris1d_h_atmsbeamversion   !User defined (see atms_beamwidth.dat)
! FFT manipulation:
      integer*4    cris1d_h_atmsbeamwidth(cris_1dnchanatms)   !deg*1E3
      integer*4    cris1d_h_atmscutoff(cris_1dnchanatms)      !fraction*1E3
! Simple averaging:
      integer*4    cris1d_h_atmsnxaverage(cris_1dnchanatms)
      integer*4    cris1d_h_atmsnyaverage(cris_1dnchanatms)
      
! ATMS temperature-radiance conversion coefficients  (Planck function)
      integer*4    cris1d_h_atmsPlanck(3,cris_1dnchanatms)
!                  (1,*) = 10^6 x (central wavenumber)
!                  (2,*) = 10^6 x (constant 1)
!                  (3,*) = 10^6 x (constant 2)


! Pre-processing information

      integer*4    cris1d_h_dT  ! size of data batches processed within ATOVPP
!                                (seconds   - see ATOVPP documentation)
      integer*4    cris1d_h_mapBTmode  ! mapping method used for BTs
!      (1=nearest neighbour, 2=bilinear interpolation)
      integer*4    cris1d_h_Acorrvn    ! vn no ATMS limb/emiss correction
!                                         (=0 if data not corrected)
      integer*4    cris1d_h_Apcpvn     ! vn no ATMS precip/scatter detection
!                                         (=0 if not done)
      integer*4    cris1d_h_Aantenna   ! vn no, ATMS antenna corrections

      integer*4    cris1d_h_Ascat !threshold value for ATMS scattering index
      integer*4    cris1d_h_ACrosby !threshold value for ATMS logistic precip
      integer*4    cris1d_h_cloudtests  !Cloud tests used (0 if none)

      integer*4    cris1d_h_recperscan !Records per scan: 1 to 9
      integer*4    cris1d_h_fovmode    !0 = full resolution
                                       !Others TBD
      
! VIIRS

      integer*4    cris1d_h_nviirschan       !number of VIIRS channels in format
      integer*4    cris1d_h_nviirschanused   !number of channels used
      integer*4    cris1d_h_viirschans(cris_1dnchanviirs)  !Channel numbers
      integer*4    cris1d_h_viirsbands(cris_1dnchanviirs)  !Band numbers
                                                           ! 5=M, 6=I, 7-DN

      integer*4    cris1d_h_spare2(array_size_cris1dspare)  !spare

      integer*4 cris1d_hd(array_size_cris1d)
      equivalence(cris1d_hd(1),cris1d_h_site)
      common /cris1dhd/ cris1d_h_site, cris1d_h_cfill1, cris1d_h_1bsite,
     & cris1d_h_cfill2, cris1d_h_versnb, cris1d_h_versyr, 
     & cris1d_h_versdy,
     & cris1d_h_hdrcnt, cris1d_h_satid, cris1d_h_grid, cris1d_h_satht,
     & cris1d_h_period, cris1d_h_startorbit, cris1d_h_startdatayr,
     & cris1d_h_startdatady, cris1d_h_startdatatime, cris1d_h_endorbit,
     & cris1d_h_enddatayr, cris1d_h_enddatady, cris1d_h_enddatatime,
     & cris1d_h_scnlin, cris1d_h_misscnlin, cris1d_h_atovppvn,
     & cris1d_h_instruments, cris1d_h_wmosatid, cris1d_h_spare1,
     & cris1d_h_centre, cris1d_h_subcentre,
     & cris1d_h_recwords,cris1d_h_nfov,cris1d_h_nfor,cris1d_h_nband,
     & cris1d_h_startw,cris1d_h_endw,
     & cris1d_h_startchan,cris1d_h_endchan,
     & cris1d_h_eigvn, cris1d_h_npc, cris1d_h_npcused, cris1d_h_chanvn,
     & cris1d_h_nchan, cris1d_h_nchanused, cris1d_h_atmsbeamversion, 
     & cris1d_h_atmsbeamwidth, cris1d_h_atmscutoff,
     & cris1d_h_atmsnxaverage, cris1d_h_atmsnyaverage,
     & cris1d_h_atmsplanck, cris1d_h_dT,
     & cris1d_h_mapBTmode, cris1d_h_Acorrvn,
     & cris1d_h_Apcpvn, cris1d_h_Aantenna, cris1d_h_Ascat,
     & cris1d_h_ACrosby, cris1d_h_cloudtests,
     & cris1d_h_recperscan, cris1d_h_fovmode,
     & cris1d_h_nviirschan, cris1d_h_nviirschanused,
     & cris1d_h_viirschans, cris1d_h_viirsbands,
     & cris1d_h_channels, cris1d_h_spare2

!##############################################################################
!
!               DATA RECORD
!
!##############################################################################

!General information

      integer*4   cris1d_scnlin            !scan line number
      integer*4   cris1d_scnlinyr          !scan line year
      integer*4   cris1d_scnlindy          !scan line day of year
      integer*4   cris1d_scnlintime(cris_1dnfor) !scan line UTC time of day (ms)

      integer*4   cris1d_latlon(2,cris_1dnfor)           !lat/lon in degrees
!                     first   : 10^4 x (latitude)
!                     second  : 10^4 x (longitude)
      integer*4   cris1d_angles(4,cris_1dnfor)           !scan angles
!                     first:    10^2 x (local zenith angle)
!                     second:   10^2 x (local azimuth angle)
!                     third:    10^2 x (solar zenith angle)
!                     fourth:   10^2 x (solar azimuth angle)

      integer*4   cris1d_satxyz(3)    ! x,y,z of satellite at mid-scan
      integer*4   cris1d_ascdesc      ! ascending / descending orbit qualifier
      integer*4   cris1d_orbit        ! orbit number
      integer*4   cris1d_scalti       ! satellite altitude
      integer*4   cris1d_surfht(cris_1dnfor)       ! surface height
      integer*4   cris1d_landfrac(cris_1dnfor)     ! land fraction
      integer*4   cris1d_landsea(cris_1dnfor)      ! land/sea qualifier
      integer*4   cris1d_cloudcover(cris_1dnfor)   ! cloud cover (percent)
      integer*4   cris1d_cloudtopht(cris_1dnfor)   ! height of top of cloud (m)

!Surfelev output (duplicates info in 1c but useful as a check)

      integer*4   cris1d_surface(3,cris_1dnfor)
!                     first   : surface height (metres)
!                     second  : surface type (0=sea, 1=mixed, 2=land)
!                     third   : land fraction (percent)

!ATMS mapped to CrIS

      integer*4   cris1d_nedtcoldatms(cris_1dnchanatms)
      integer*4   cris1d_nedtwarmatms(cris_1dnchanatms)
      
      integer*4   cris1d_btemps(cris_1dnchanatms,cris_1dnfor)
!                 10^2 x scene brightness temperature (K) for ATMS channels

      integer*4   cris1d_crisqual(cris_1dnfor)
!                 Quality control word for each field of view:
!                  (all bits off implies acceptable data)
!                  bit 27-4: bit n set to 1 if brightness temperature in
!                            ATMS channel n-3 is physically unreasonable or has
!                            not been calculated due to calibration  
!                            problems.
!                  bit 3-1:  bit n set to 1 if data for CrIS detector n are bad
!                  bit 0:    bad or missing data (in any or all channels)

      integer*4   cris1d_prepro(8,cris_1dnfor) ! values from pre-processing
!                  (1,*):   pre-processing quality control word with flags
!                            (all bits off implies acceptable data)
!                  bit 31: set if ATMS surface types not all the same
!                  bit 30-28: spare
!                  bit 27: flag for cloud cost set for any mapped ATMS
!                  bit 26: scattering flag set for any mapped ATMS
!                          (only set over the sea)
!                  bit 25: logistic precipitation probability test
!                          calculated from ATMS data mapped to CrIS grid
!                  bit 24: Grody light rainfall test calculated on CrIS grid
!                  bit 23: spare
!                  bit 22: mismatch between surface type from topography
!                          dataset and from pre-processing (any ATMS)
!                  bit 21: fewer ATMS mapping co-locations than expected
!                  bit 20-4: spare   <zero fill>
!                  bit 3: spare
!                  bit 2: flag for cloud cost (recalculated on CrIS grid)
!                  bit 1: flag for scattering index (recalculated on CrIS grid)
!                  bit 0: set if ATMS data missing
!
!                  (2,*)    estimated surface type from the pre-processing:
!            1 = Bare young ice (i.e. new ice, no snow)
!            2 = Dry land (i.e. dry with or without significant vegetation)
!            3 = Dry snow (i.e. snow with water less than 2%, over land)
!            4 = Multi-year ice (i.e. old ice with snow [assumed dry] cover)
!            5 = Sea (i.e. open water, no islands, ice-free, WS=0 to 14m/s)
!            6 = Wet forest (i.e. established forest with wet canopy)
!            7 = Wet land (i.e. non-forested land with a wet surface)
!            8 = Wet snow (i.e. snow with water content > 2%, over land or ice)
!            9 = Desert
!
!                  (3,*) cost fn from PPASURF surface identification (*100)
!                  (4,*) scattering index (*100)
!                  (5,*) cirrus index (*100)
!                  (6,*) logistic precipitation probability (*100)
!                  (7,*) NWC-SAF (Bennartz) scattering index (K)
!                  (8,*) NWC-SAF (Bennartz) precipitation probabilities

      integer*4   cris1d_emiss(13,cris_1dnfor) !13 emissivity parameters

!CrIS data

      integer*4   cris1d_radianceflgs(cris_1dnfor) ! radiance flags
!Bit(1=MS)
!1	Clear path
!2	Partly cloudy path
!3	Cloudy path
!4	Apodized
!5	Unapodized
!6	Reconstructed
!7	Cloud cleared
!8-14	Reserved
!All 15	Missing value

      integer*4 cris1d_scanqualflgs              ! scan quality flags
!Bit(1=MS)
!1	Gap in Raw Data Record (RDR) data detected (i.e., missing scan(s)
!       preceding the current scan)
!2	Recorded time is not in sequence (i.e., the scan start time is out of
!       sequence)
!3	Lamda monitored calculation cannot be updated
!4	The measured temperatures of any instrument components (e.g., 
!       beam-splitter, scan mirror, scan baffle, etc.) are outside the
!       allowable ranges
!5	At least one of the monitored instrument temperatures has drifted more
!       than a specified tolerance value
!6-12	Reserved
!All 13	Missing value

      integer*4   cris1d_calqualflgs(cris_1dnband,cris_1dnfor)  !cal. quality
                      !Same for all FOVS 1 to 9, but thinned data may have 
                      !different FOVs within a record
!Bit(1=MS of 9 bits)
!1	Lunar intrusion on first deep space view
!2	Lunar intrusion on second deep space view
!3-8	Reserved
!All 9	Missing value

      integer*4 cris1d_fovqualflgs(cris_1dnband,cris_1dnfor) !FOV quality
!Bit(1=MS of 19 bits)
!1	Degraded SDR quality
!2	Invalid SDR quality
!3	Invalid SDR geolocation information
!4	Degraded radiometric calibration
!5	Invalid radiometric calibration
!6	Degraded spectral calibration
!7	Invalid spectral calibration
!8	Fringe count error detected and corrected
!9	Day/night indicator
!10	Invalid RDR data
!11	Significant fringe count error detected
!12	Bit trim failed
!13-18	Reserved
!All 19	Missing value

      integer*4   cris1d_geolocqual                !Geolocation quality
!Value
!0  Nominal
!1  Missing at most a small gap of altitude and Ephemeris data
!2  Missing more than a small gap of altitude and Ephemeris data, but no more
!   than a granule boundary
!3  Missing more than a granule boundary of altitude and Ephemeris data
!4-14 Reserved
!15 Missing

      integer*4   cris1d_quality(cris_1dnfor)  !Quality information
!Value
!0  Not suspect                                                                  
!1  Suspect                                                                      
!2  Very suspect                                                                 
!3  Unusable                                                                     
      
!CrIS cloud flag (user defined)

      integer*4   cris1d_cloudflag(10,cris_1dnfor)
      
!CrIS radiance and PC scores

      integer*4   cris1d_FOV(cris_1dnfor) !30 values in range 1 to 270
      integer*4   cris1d_scores(cris_1dnpc,cris_1dnfor) !PC scores
      integer*4   cris1d_logfit(3,cris_1dnfor)!Reconstruction Scores for 3 bands

!For radiance, put the channel index last, to allow shorter record length 
!to be used in the case of a channel subset

      integer*4   cris1d_radiance(cris_1dnfor,cris_1dnchan)  !Radiance
                                                             !Wm-2sr-1cm *1E7

!VIIRS

      integer*4   cris1d_viirsradiance(cris_1dnchanviirs,cris_1dnfor)
      integer*4   cris1d_viirsreflectance(cris_1dnchanviirs,cris_1dnfor)

      integer*4 cris1d_dt(array_size_cris1d)
      equivalence(cris1d_dt(1),cris1d_scnlin)
      common /cris1ddt/ cris1d_scnlin,cris1d_scnlinyr,cris1d_scnlindy,
     &   cris1d_scnlintime,cris1d_latlon,cris1d_angles,
     &   cris1d_satxyz,cris1d_ascdesc,cris1d_orbit,
     &   cris1d_scalti,cris1d_surfht,cris1d_landfrac,cris1d_landsea,
     &   cris1d_cloudcover,cris1d_cloudtopht,cris1d_surface,
     &   cris1d_nedtcoldatms,cris1d_nedtwarmatms,
     &   cris1d_btemps,cris1d_crisqual,cris1d_prepro,cris1d_emiss,
     &   cris1d_radianceflgs,cris1d_scanqualflgs,cris1d_calqualflgs,
     &   cris1d_fovqualflgs,cris1d_geolocqual,cris1d_quality,
     &   cris1d_cloudflag,cris1d_FOV,cris1d_scores,cris1d_logfit,
     &   cris1d_viirsradiance,cris1d_viirsreflectance,
     &   cris1d_radiance

