! 
!DESCRIPTION 
!      include file for dcs 1b format handling
!
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
! Marcel Derrien
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
!  1.0     07/95     initial            Marcel Derrien
!  2.0     07/11/95  new release from NOAA available 30/10/95
!                    still problems in integer boudaries alignments
!                    and undefined scaling factors
!  3.0     11/05/98  modified to conform to NOAA release dated 
!                    27 March 1998 (gpg # 7 for header record)
!          reviewed 12/05/98 by Robert Gouriou
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!###########################################################################
!
!               DCS 1B Data Set 
!
!###########################################################################
!
!   parameters section
       integer recldcs 
       parameter (recldcs = 10752 ) ! 1B dcs record length (bytes)

       integer cdcs_size 
       parameter ( cdcs_size = recldcs/4) ! 1B dcs record length (4 bytes words)

!
!###########################################################################
!
!               HEADER RECORD
!
!###########################################################################
!
!                GENERAL INFORMATION
!
       character*3  dcs_h_siteid        !Data set creation site ID
       character*1  dcs_h_blank         !ASCII blank
       integer*2    dcs_h_l1bversnb     !level 1b format version number
       integer*2    dcs_h_l1bversyr     !level 1b format version year
       integer*2    dcs_h_l1bversdy     !level 1b format version day of year
       integer*2    dcs_h_reclg         !reserved for logical record length
       integer*2    dcs_h_blksz         !reserved for block size
       integer*2    dcs_h_hdrcnt        !count of header records in data set
       integer*2    dcs_h_filler0(3)
       character*42 dcs_h_dataname      !data set name
       character*8  dcs_h_prblkid       !processing block identification
       integer*2    dcs_h_satid         !NOAA spacecraft identification code
       integer*2    dcs_h_instid        !instrument identification
       integer*2    dcs_h_datatyp       !data type code
!                   (1 = LAC; 2 = GAC; 3 = HRPT; 4 = TIP; 5 = HIRS; 6 = MSU;
!                    7 = SSU; 8 = DCS; 9 = SEM; 10 = AMSU-A; 11 = AMSU-B)
       integer*2    dcs_h_tipsrc        !TIP source code
!                   (0 = not applicable; 1 = GAC embedded; 2 = stored;
!                    3 = third CDA; 4 = HRPT embedded)
       integer    dcs_h_startdatajd   !start of data set day count 
!                                        from 00h,1 Jan 1950
       integer*2    dcs_h_startdatayr   !start of data set year
       integer*2    dcs_h_startdatady   !start of data set day of year
       integer    dcs_h_startdatatime !start of data set UTC time of day in
!                                        milliseconds
       integer    dcs_h_enddatajd     !end of data set day count 
!                                        from 00h,1 Jan 1950
       integer*2    dcs_h_enddatayr     !end of data set year
       integer*2    dcs_h_enddatady     !end of data set day of year
       integer      dcs_h_enddatatime   !end of data set UTC time of day in
!                                        milliseconds
       integer*2    dcs_h_cpidsyr       !year of Last CPIDS Update
       integer*2    dcs_h_cpidsdy       !Day of Year of Last CPIDS Update

       integer*4    dcs_h_filler1(2)    !filler
!
!                DATA SET QUALITY INDICATORS
!
       integer*4   dcs_h_inststat          !instrument status
       integer*2   dcs_h_statchrecnb       !record number of status change
!                 ( if 0, none occurred)     
       integer*2   dcs_h_scnlin            !count of scan lines in this data set
       integer*2   dcs_h_datagaps          !count of data gaps in this data set
       integer*2   dcs_h_okdatafr          !count of data frames without frame 
!                                           sync word errors
       integer*2   dcs_h_pacsparityerr     !count of PACS detected TIP parity
!                                           errors
       integer*2   dcs_h_auxsyncerrsum     !sum of all auxiliary sync errors  
!                                           detected in the input data
       integer*2   dcs_h_pacsstatfield     !PACS status bit field
!                  bit 15-3 : spare
!                  bit  2   : pseudo noise (0 = normal data; 1 = P/N data)
!                  bit  1   : tape direction (0 = time decrementing)
!                  bit  0   : data mode (0 = test data; 1 = flight data)
       integer*2   dcs_h_pacsdatasrc       !PACS data source 
!                  (0 = unused; 1 = Gilmore; 2 = Wallops; 3 = SOCC)
       character*8 dcs_h_spare1            !spare (reserved for ingester)
       character*8 dcs_h_spare2            !spare (reserved of the
!                                           decommutation)
       integer*4   dcs_h_filler2(4)        !filler <zero fill>




!                        ANALOG TELEMETRY CONVERSION
!
!
!                                        channel calibration
!
       integer*4   dcs_h_ancnv(5,9)     !conversion coefficients for analog 
!                                        housekeeping telemetry
!                  (five consecutive 16-bit words per telemetry field)
!                  word  1 -  5: type 1 telemetry transfer coefficients
!                  word  6 - 10: type 2 telemetry transfer coefficients
!                  word 11 - 15: type 3 telemetry transfer coefficients
!                  word 16 - 20: type 4 telemetry transfer coefficients
!                  word 21 - 25: type 5 telemetry transfer coefficients
!                  word 26 - 30: type 6 telemetry transfer coefficients
!                  word 31 - 35: type 7 telemetry transfer coefficients
!                  word 36 - 40: type 8 telemetry transfer coefficients
!                  word 41 - 45: type 9 telemetry transfer coefficients
!
       integer*2   dcs_h_filler3(5202)     ! filler <zero fill>
        
      integer dcs_hd(cdcs_size)
      equivalence(dcs_hd(1) , dcs_h_siteid)
      common/dcs1bhd/ dcs_h_siteid , dcs_h_blank ,
     &   dcs_h_l1bversnb , dcs_h_l1bversyr , dcs_h_l1bversdy ,
     &   dcs_h_reclg , dcs_h_blksz ,
     &   dcs_h_hdrcnt , dcs_h_filler0 , dcs_h_dataname , dcs_h_prblkid ,
     &   dcs_h_satid , dcs_h_instid , dcs_h_datatyp , dcs_h_tipsrc ,
     &   dcs_h_startdatajd , dcs_h_startdatayr , dcs_h_startdatady ,
     &   dcs_h_startdatatime , dcs_h_enddatajd , dcs_h_enddatayr ,
     &   dcs_h_enddatady , dcs_h_enddatatime ,
     &   dcs_h_cpidsyr , dcs_h_cpidsdy , 
     &   dcs_h_filler1 ,
     &   dcs_h_inststat , dcs_h_statchrecnb , dcs_h_scnlin , 
     &   dcs_h_datagaps , dcs_h_okdatafr , dcs_h_pacsparityerr ,
     &   dcs_h_auxsyncerrsum , dcs_h_pacsstatfield ,
     &   dcs_h_pacsdatasrc , dcs_h_spare1 , dcs_h_spare2 ,
     &   dcs_h_filler2 , 
     &   dcs_h_ancnv , dcs_h_filler3
!
!##############################################################################
!
!                DATA RECORD 
!
!##############################################################################
!
!                FRAME INFORMATION STRUCTURE 
!
       integer*2 dcs_mjfrcnt       !major frame count 
       integer*2 dcs_mjfryr        !major frame year
       integer*2 dcs_mjfrdy        !major frame day of year
       integer*2 dcs_clockdrift    !satellite clock drift delta in milliseconds
       integer   dcs_mjfrtime      !scan line UTC time of day in milliseconds
       integer*2 dcs_mjfbf         !major frame bit field
       integer*2 dcs_filler1(5)    !filler <zero fill>
!
!                Quality Indicators
!    
       integer dcs_qualind         !quality indicator bit field
       integer dcs_mjfqf           !major frame quality flags
       integer*4 dcs_filler2(2)    !<Zero Fill>
!
!                 DCS MINOR FRAME DATA 
!       
       integer dcs_data(320*8)
       integer dcs_filler3

!
!                 DIGITAL B TELEMETRY
!
       integer*2 dcs_digbinvwbf        ! invalid word bit flags
!
       integer*2 dcs_digbtlm       !digital B telemetry
!
!                bit 15: relay A status 
!                bit 14: relay B status 
!                bit 13: DRU 1 
!                bit 12: DRU 2 
!                bit 11: DRU 3 
!                bit 10: DRU 4 
!                bit  9: DRU 5 
!                bit  8: DRU 6 
!                bit  7: DRU 7 
!                bit  6: DRU 8 
!                bit  5: Memory Overflow 
!                bit  4: DCS Time Code (MSB) 
!                bit  3: Pseudo message status 
!                bit  2: spare
!                bit  1: spare
!                bit  0: spare
       integer dcs_filler4
!
!                 ANALOG TELEMETRY
!
       integer*2 dcs_aninvwbf        ! invalid word bit flags
       integer*2 dcs_analog(8)      ! analog telemetry  
!      (16 8-bit words)
!        word  1: RPU temperature (use type 1 transfer coefficients)
!        word  2: SPU-A temperature (use type 1 transfer coefficients)
!        word  3: SPU-B temperature (use type 1 transfer coefficients)
!        word  4: converter voltage +5V (use type 2 transfer
!                 coefficients)
!        word  5: converter voltage +12V (use type 3 transfer
!                 coefficients)
!        word  6: converter voltage -5V (use type 4 transfer
!                 coefficients)
!        word  7: converter voltage -12V (use type 5 transfer
!                 coefficients)
!        word  8: converter temperature (use type 1 transfer
!                 coefficients)
!        word  9: USO oven temperature (use type 6 transfer
!                 coefficients)
!        word 10: USO thermal regulation (use type 7 transfer
!                 coefficients)
                 

        integer*2 dcs_filler5(221)  ! filler  
        
        integer dcs_dt(cdcs_size) 
        equivalence(dcs_dt(1) , dcs_mjfrcnt)   
        common/dcs1bdt/
     &    dcs_mjfrcnt , 
     &    dcs_mjfryr , dcs_mjfrdy , dcs_clockdrift , 
     &    dcs_mjfrtime , dcs_mjfbf , dcs_filler1 ,
     &    dcs_qualind , dcs_mjfqf , 
     &    dcs_filler2 , dcs_data , dcs_filler3 , 
     &    dcs_digbinvwbf , dcs_digbtlm , dcs_filler4 ,
     &    dcs_aninvwbf , dcs_analog , dcs_filler5
