!------------------------------------------------------------------------------
!NAME
!  hcalibcycleinfo_algoV4.h
!  
!  Include to process with program hirscl_algoV4
!
!HISTORY (CHANGES) 
!(version   date         comment           <author>)
! 1.0     January 05    Creation          T. Labrot (MF)
! 1.1     Feb 05        Double precision variables first in common block. NCA.
! 1.2     14/06/05      Add spcntmed,spnedn,iwtcntmed,iwtnedn 
!                       Change dimension of FWT   T.Labrot/P.Brunel (MF)
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
!------------------------------------------------------------------------------

      INTEGER calib     ! Total number of cabration cycles in the orbit
                        ! (computed in h_scanpos_algoV4.F)
                        
      INTEGER goodcalib,badcalib ! Total number of good/bad calib cycle                 

      LOGICAL qualified(hrs_mxchn,hrs_mxcalib) !=true if the calib cycle is usable
      
      INTEGER splintab(hrs_mxcalib)     
                                ! Array of the numbers of space lines 
                                ! (computed in h_scanpos_algoV4.F)
      INTEGER iwtlintab(hrs_mxcalib)    
                                ! Array of the numbers of IWT lines
                                ! (computed in h_scanpos_algoV4.F)
      LOGICAL mooncontaflag(hrs_mxcalib) 
                                !Moon contamination flag of space line

      REAL PTT(hrs_mxlin)       ! Primary Telescope Temperature                  
      REAL stt(hrs_mxlin)       ! Secondary Telescope Temperature
                                ! Computed with hrs_elem(5,62,splintab(icalib))
      REAL TTT(hrs_mxlin)       ! Tertiary Telescope Temperature                                                 
      REAL FWT(4,hrs_mxlin)     ! Filter Wheel Temperature
      REAL BBT(hrs_mxlin)       ! Base Plate Temperature
      
      DOUBLE PRECISION calibcoef0(hrs_mxchn,3,hrs_mxcalib)
                        ! Black Body(BB) third coefficient,BB slope, BB intercept
                        ! = raw third coefficient,raw slope, raw intercept
                        ! for each of the 20 channels
                        ! (h_BBinterslop_algoV4.F)

      DOUBLE PRECISION spcntmn(hrs_mxirchn,hrs_mxcalib)
                        ! Mean of the space counts for the 19 IR channels
                        ! (computed in  h_cntmn_algoV4.F)
      DOUBLE PRECISION spcntmed(hrs_mxirchn,hrs_mxcalib)
                        ! Median of the space counts for the 19 IR channels
                        ! (computed in  h_cntmn_algoV4.F)
      DOUBLE PRECISION spcntstd(hrs_mxirchn,hrs_mxcalib)
                        ! Standard deviation of the space counts for the 19 IR channels
                        ! ( computed in h_cntmn_algoV4.F)
      DOUBLE PRECISION spnedn(hrs_mxirchn,hrs_mxcalib)
                        ! Observed NEDN corresponding to the space view for the 19 IR channels
                        ! ( computed in h_cntmn_algoV4.F)
      INTEGER spgoodcnt(hrs_mxirchn,hrs_mxcalib)
                        ! Number of space counts used to compute the mean
                        ! ( computed in h_cntmn_algoV4.F)
      INTEGER spmisscnt(hrs_mxirchn,hrs_mxcalib)
                        ! Number of missing space counts 
                        ! (computed in h_cntmn_algoV4.F)
      INTEGER spinfcnt(hrs_mxirchn,hrs_mxcalib)
                        ! Number of space counts inferiors to the limit 
                        ! (computed in h_cntmn_algoV4.F)
      INTEGER spsupcnt(hrs_mxirchn,hrs_mxcalib)
                        ! Number of space counts superiors to the limit 
                        ! (computed in h_cntmn_algoV4.F)
      INTEGER spejeccnt(hrs_mxirchn,hrs_mxcalib)
                        ! Number of rejected space counts
                        ! =missing value if median value is used for the mean
                        ! (computed in h_cntmn_algoV4.F)
                        
      DOUBLE PRECISION iwtcntmn(hrs_mxirchn,hrs_mxcalib)
                        ! Mean of the IWT counts for the 19 IR channels
                        ! (computed in h_cntmn_algoV4.F)
      DOUBLE PRECISION iwtcntmed(hrs_mxirchn,hrs_mxcalib)
                        ! Median of the IWT counts for the 19 IR channels
                        ! (computed in h_cntmn_algoV4.F)
      DOUBLE PRECISION iwtcntstd(hrs_mxirchn,hrs_mxcalib)
                        ! Standard deviation of the IWT counts for the 19 IR channels
                        ! (computed in h_cntmn_algoV4.F)
      DOUBLE PRECISION iwtnedn(hrs_mxirchn,hrs_mxcalib)
                        ! Observed NEDN corresponding to IWT view for the 19 IR channels
                        ! (computed in h_cntmn_algoV4.F)
      INTEGER iwtgoodcnt(hrs_mxirchn,hrs_mxcalib)
                        ! Number of IWT counts used to compute the mean
                        ! (computed in h_cntmn_algoV4.F)
      INTEGER iwtmisscnt(hrs_mxirchn,hrs_mxcalib)
                        ! Number of missing IWT counts 
                        ! (computed in h_cntmn_algoV4.F)
      INTEGER iwtinfcnt(hrs_mxirchn,hrs_mxcalib)
                        ! Number of IWT counts inferiors to the limit 
                        ! (computed in h_cntmn_algoV4.F)
      INTEGER iwtsupcnt(hrs_mxirchn,hrs_mxcalib)
                        ! Number of IWT counts superiors to the limit 
                        ! (computed in h_cntmn_algoV4.F)
      INTEGER iwtejeccnt(hrs_mxirchn,hrs_mxcalib)
                        ! Number of rejected IWT counts
                        ! =missing value if median value is used for the mean
                        ! (computed in h_cntmn_algoV4.F)
                        
      DOUBLE PRECISION tmp(hrs_mxprtnb,hrs_mxcalib)
                        ! Temperature of mxprtnb PRTs
                        ! (computed in h_iwttmp_algoV4.F)
      DOUBLE PRECISION prtmean(hrs_mxprtnb,hrs_mxcalib)
                        ! Mean of the PRT readings
                        ! (computed in h_iwttmp_algoV4.F)
      DOUBLE PRECISION prtstd(hrs_mxprtnb,hrs_mxcalib)
                        ! Standard deviation of the PRT readings
                        ! (computed in h_iwttmp_algoV4.F)
      INTEGER datanb(hrs_mxprtnb,hrs_mxcalib)
                        ! Initial number of PRT readings
                        ! (computed in h_iwttmp_algoV4.F)       
      INTEGER goodprt(hrs_mxprtnb,hrs_mxcalib)
                        ! Number of good PRT readings
                        ! (computed in h_iwttmp_algoV4.F)
      INTEGER ejecprt(hrs_mxprtnb,hrs_mxcalib)
                        ! Number of rejected PRT readings
                        ! (computed in h_iwttmp_algoV4.F)
                        
      COMMON/ calibinfo / goodcalib,badcalib
      
      COMMON / hirscalibelts / 
     &   calibcoef0,spcntmn,spcntmed,spcntstd,spnedn,
     &   iwtcntmn,iwtcntmed,iwtcntstd,iwtnedn,
     &   tmp,prtmean,prtstd,calib,qualified,
     &   splintab,iwtlintab,mooncontaflag,
     &   stt,PTT,TTT,FWT,BBT,spgoodcnt,spmisscnt,spinfcnt,
     &   spsupcnt,spejeccnt,iwtgoodcnt,iwtmisscnt,
     &   iwtinfcnt,iwtsupcnt,iwtejeccnt,
     &   datanb,goodprt,ejecprt

                                
