!------------------------------------------------------------------------------
!NAME
!  hcalibhisto_algoV4.h
!  
!  Include to process with program hirscl_algoV4
!
!HISTORY (CHANGES) 
!(version   date         comment           <author>)
! 1.0     January 05    Creation          P. Brunel (MF)
! 1.1     14/06/05      Add spcntmed,spnedn,iwtcntmed,iwtnedn 
!                       Change dimension of FWT   T.Labrot/P.Brunel (MF)
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
!------------------------------------------------------------------------------

      INTEGER hrs_mxcalibhisto
      PARAMETER (hrs_mxcalibhisto=10000)

      CHARACTER*6 scname(hrs_mxcalibhisto)
      DOUBLE PRECISION ijsp(hrs_mxcalibhisto) ! julian date of space view
      INTEGER orbnum(hrs_mxcalibhisto)    ! orbit number
      INTEGER ascdsc(hrs_mxcalibhisto)    !  ascending descending (0/1)
      REAL solzen(hrs_mxcalibhisto)       ! solar zenith angle (deg)
      REAL latitude(hrs_mxcalibhisto)     ! latitude at nadir (deg)
      INTEGER kcal(hrs_mxcalibhisto)      ! calib cycle number in pass
      INTEGER totcal(hrs_mxcalibhisto)    ! total calibration cycles in pass
      INTEGER nlines(hrs_mxcalibhisto)    ! total number of lines in pass
      LOGICAL qualified(hrs_mxchn,hrs_mxcalibhisto)
      INTEGER splintab(hrs_mxcalibhisto)        ! number of space line 
      INTEGER iwtlintab (hrs_mxcalibhisto)! number of IWT line
      LOGICAL mooncontaflag(hrs_mxcalibhisto) !Moon contamination flag of space line

                                 
      REAL PTT(hrs_mxcalibhisto)        ! Primary Telescope Temperature
      REAL STT(hrs_mxcalibhisto)        ! Secondary Telescope Temperature
      REAL TTT(hrs_mxcalibhisto)        ! Tertiary Telescope Temperature
      REAL FWT(4,hrs_mxcalibhisto)      ! Filter Wheel Temperature
      REAL BBT(hrs_mxcalibhisto)        ! Base Plate Temperature

      DOUBLE PRECISION calibcoef0(hrs_mxchn,3,hrs_mxcalibhisto)
                        ! Black Body(BB) third coefficient,BB slope, BB intercept
                        ! = raw third coefficient,raw slope, raw intercept
                        ! for each of the 20 channels

      DOUBLE PRECISION spcntmn(hrs_mxirchn,hrs_mxcalibhisto)
                        ! Mean of the space counts for the 19 IR channels
      DOUBLE PRECISION spcntmed(hrs_mxirchn,hrs_mxcalibhisto)
                        ! Median of the space counts for the 19 IR channels
      DOUBLE PRECISION spcntstd(hrs_mxirchn,hrs_mxcalibhisto)
                        ! Standard deviation of the space counts for the 19 IR channels
      DOUBLE PRECISION spnedn(hrs_mxirchn,hrs_mxcalibhisto)
                        ! Observed NEDN corresponding to the space view for the 19 IR channels
      INTEGER spgoodcnt(hrs_mxirchn,hrs_mxcalibhisto)
                        ! Number of space counts used to compute the mean
      INTEGER spmisscnt(hrs_mxirchn,hrs_mxcalibhisto)
                        ! Number of missing space counts 
      INTEGER spinfcnt(hrs_mxirchn,hrs_mxcalibhisto)
                        ! Number of space counts inferiors to the limit 
      INTEGER spsupcnt(hrs_mxirchn,hrs_mxcalibhisto)
                        ! Number of space counts superiors to the limit 
      INTEGER spejeccnt(hrs_mxirchn,hrs_mxcalibhisto)
                        ! Number of rejected space counts
                        
      DOUBLE PRECISION iwtcntmn(hrs_mxirchn,hrs_mxcalibhisto)
                        ! Mean of the IWT counts for the 19 IR channels
      DOUBLE PRECISION iwtcntmed(hrs_mxirchn,hrs_mxcalibhisto)
                        ! Median of the IWT counts for the 19 IR channels
                        ! (computed in h_cntmn_algoV4.F)
      DOUBLE PRECISION iwtcntstd(hrs_mxirchn,hrs_mxcalibhisto)
                        ! Standard deviation of the IWT counts for the 19 IR channels
      DOUBLE PRECISION iwtnedn(hrs_mxirchn,hrs_mxcalibhisto)
                        ! Observed NEDN corresponding to IWT view for the 19 IR channels
      INTEGER iwtgoodcnt(hrs_mxirchn,hrs_mxcalibhisto)
                        ! Number of IWT counts used to compute the mean
      INTEGER iwtmisscnt(hrs_mxirchn,hrs_mxcalibhisto)
                        ! Number of missing IWT counts 
      INTEGER iwtinfcnt(hrs_mxirchn,hrs_mxcalibhisto)
                        ! Number of IWT counts inferiors to the limit 
      INTEGER iwtsupcnt(hrs_mxirchn,hrs_mxcalibhisto)
                        ! Number of IWT counts superiors to the limit 
      INTEGER iwtejeccnt(hrs_mxirchn,hrs_mxcalibhisto)
                        ! Number of rejected IWT counts
                        
      DOUBLE PRECISION tmp(hrs_mxprtnb,hrs_mxcalibhisto)
                        ! Temperature of hrs_mxprtnb PRTs
      DOUBLE PRECISION prtmean(hrs_mxprtnb,hrs_mxcalibhisto)
                        ! Mean of the PRT readings
      DOUBLE PRECISION prtstd(hrs_mxprtnb,hrs_mxcalibhisto)
                        ! Standard deviation of the PRT readings
      INTEGER datanb(hrs_mxprtnb,hrs_mxcalibhisto)
                        ! Initial number of PRT readings
      INTEGER goodprt(hrs_mxprtnb,hrs_mxcalibhisto)
                        ! Number of good PRT readings
      INTEGER ejecprt(hrs_mxprtnb,hrs_mxcalibhisto)
                        ! Number of rejected PRT readings
                                                
      
      COMMON /calibhistov4/
     &   scname, ijsp, orbnum,
     &   ascdsc, solzen, latitude,
     &   kcal, totcal, nlines, qualified,
     &   splintab, iwtlintab, mooncontaflag,
     &   PTT, STT, TTT, FWT, BBT,
     &   calibcoef0,
     &   spcntmn, spcntmed, spcntstd, spnedn, spgoodcnt, spmisscnt,
     &   spinfcnt, spsupcnt, spejeccnt,
     &   iwtcntmn, iwtcntmed, iwtcntstd, iwtnedn, iwtgoodcnt,
     &   iwtmisscnt, iwtinfcnt, iwtsupcnt, iwtejeccnt,
     &   tmp, prtmean, prtstd, datanb, goodprt, ejecprt
