!------------------------------------------------------------------------------
!NAME
!  hcinit_algoV4.h
!  
!  Include to process with program hirscl_algoV4
!
!HISTORY (CHANGES) 
!(version   date         comment           <author>)
! 1.0     January 05     creation          T. Labrot (MF)
! 1.1     March 05       Double precision variables first in common block. NCA.
! 1.2     March 05       Add numprtlm and redefine prtminmax. TL.
! 1.3     25 April 07    Added the new parameter cntmax  T. Labrot (MF)
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
!------------------------------------------------------------------------------


      INTEGER versnb           !number of the version of the calib. coef. file
      INTEGER versyr           !year of the version of the calib. coef. file
      INTEGER versday          !day of the year of the calib. coef. file 
      INTEGER ch20year         !year of most recent solar channel calibration
      INTEGER ch20day          !day of year of most recent solar channel calibration
      INTEGER prtnb            !number of PRTs =4 for HIRS3, =5 for HIRS4
      INTEGER prtwght(5)       !weight coefficients for each PRT
      INTEGER spcntlm(hrs_mxirchn,2)    !output counts limits of space view scan
      INTEGER iwtcntlm(hrs_mxirchn,2)   !output counts limits of iwt

      REAL nedn(hrs_mxirchn)            !noise Equivalent Delta Radiance (NEDN)
      
      DOUBLE PRECISION iwtprtcnttmp(5,6) 
!                              !IWT PRT count to temp. conversion coefficients
      DOUBLE PRECISION ictprtcnttmp(4,6)
!                              !ICT PRT count to temp. conversion coefficients      
      DOUBLE PRECISION telescopcnttmp(3,6)
!                              !Telescope count to temperature conversion coefficients      
      DOUBLE PRECISION fwcnttmp(4,6)
!                              !Filter wheel count to temperature conversion coefficients 
      DOUBLE PRECISION bpcnttmp(6)
!                              !Base plate count to temperature conversion coefficients     
      DOUBLE PRECISION patchexpcnttmp(6)
!                              !Patch (expd) count to temperature conversion coefficients
      DOUBLE PRECISION radcnttmp(6)
!                              !Radiator count to temperature conversion coefficients
      DOUBLE PRECISION scmircnttmp(6)
!                              !Scan Mirror count to temperature conversion coefficients   
      DOUBLE PRECISION electcnttmp(6)
!                              !Electronics count to temperature conversion coefficients     
      DOUBLE PRECISION patchfcnttmp(6)
!                              !Patch (full) count to temperature conversion coefficients
      DOUBLE PRECISION scmotcnttmp(6)
!                              !Scan Motor count to temperature conversion coefficients     
      DOUBLE PRECISION fwmcnttmp(6)
!                              !Filter Wheel Motor count to temperature conversion coefficients 
      DOUBLE PRECISION chsgcnttmp(6)
!                              !Coller HSG count to temperature conversion coefficients
      DOUBLE PRECISION cwave(hrs_mxirchn)       
!                              !hrs_mxirchn central waves
      DOUBLE PRECISION bdcorr(hrs_mxirchn,2) 
!                              !hrs_mxirchn*2 band-correction coefficients    
      DOUBLE PRECISION plck(2) !Planck constants
      DOUBLE PRECISION visibcoef(2)
!                              !calib coefficients for the visible channel



      INTEGER sp1fv           !the first field of view in the space scan that
!                              is not contaminated.
      INTEGER minvw           !the minimum number of space or iwt data  
!                              to compute an average
      INTEGER stdevcf         !coefficient xx for the test +/- xx*standard
!                              deviation to eliminate some extreme output counts
      INTEGER prtminmax       !maximum value of the difference between the
!                              maximum and minimum values of PRT output counts
      INTEGER prtminnb        !the minimum of PRT output values to compute an 
!                              average
      INTEGER prtbfore        !specified number of scan lines before the IWT
!                              (internal warm target) scan line
      INTEGER prtafter        !specified number of scan lines after the IWT
!                              (internal warm target) scan line 
      INTEGER numprtlm        !minimum number of PRT temperatures to compute 
!                              the IWT temperature
      INTEGER nbslval         !minimum number of successive scan lines with 
!                              agreed line counters                                                           
      INTEGER BBslopethreshold  !threshold (in percent): The BBslope values are within this threshold
!                                from their mean value
      INTEGER avgslopethreshold !threshold (in percent): The mean of the BBslopes
!                                are within this threshold from the average slope
      INTEGER cntmax          !max number of the same value of outpout counts
!                              for one IFOV       
 
      COMMON/hrs_cldate/versnb,versyr,versday,ch20year,ch20day
      
      COMMON/hrs_clcf/iwtprtcnttmp,ictprtcnttmp,telescopcnttmp,
     &                fwcnttmp,bpcnttmp,patchexpcnttmp,radcnttmp,
     &                scmircnttmp,electcnttmp,patchfcnttmp,
     &                scmotcnttmp,fwmcnttmp,chsgcnttmp,
     &                cwave,bdcorr,plck,visibcoef,
     &                prtnb,prtwght,spcntlm,iwtcntlm,nedn
              
      COMMON/hrs_tstcf/sp1fv,minvw,stdevcf,prtminmax,prtminnb,prtbfore,
     &                 prtafter,numprtlm,nbslval,
     &                 BBslopethreshold,avgslopethreshold,
     &                 cntmax
