!+ HIRAS level 1C Data Setn - holds data for one scan
!
! Name:  hiras1c.h
!
! Purpose:
!   'INCLUDE' file for common /hiras1chd/   (hiras level 1C header record)
!                  and common /hiras1cdt/   (hiras level 1C data record)
!
! Can be used for full channel dataset, channel selection or PC score
! representation if needed.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    04/02/2019  Initial. Nigel Atkinson.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!   User adjustable parameters:
      integer       hiras_1cnchan        ! Max number of channels in format
         parameter (hiras_1cnchan = 2275)! At FSR
      integer       hiras_1cnpc          ! Number of PCs in format (at least 1)
         parameter (hiras_1cnpc = 1)

!   Normally fixed parameters:
      integer       hiras_1cnfov         ! Number of fields of view
         parameter (hiras_1cnfov = 4)
      integer       hiras_1cnfor         ! Number of fields of regard (scan pos)
         parameter (hiras_1cnfor = 29)
      integer       hiras_1cnband        ! Number of bands
         parameter (hiras_1cnband = 3)
      integer       hiras_1cvnnb         ! format version number
         parameter (hiras_1cvnnb = 1)
      integer       hiras_1cvnyr         ! format version year
         parameter (hiras_1cvnyr = 2019)
      integer       hiras_1cvndy         ! format version day
         parameter (hiras_1cvndy = 35)

      integer       array_size_hiras1c   ! Length of array
         parameter (array_size_hiras1c = 
     &         11 + hiras_1cnfor + hiras_1cnband*hiras_1cnfov + 
     &         (13 + 2*hiras_1cnband + hiras_1cnchan +12 + hiras_1cnpc)
     &              *hiras_1cnfov*hiras_1cnfor)

      integer       array_size_hiras1cspare   ! Length of spare header array
         parameter (array_size_hiras1cspare = 
     &         array_size_hiras1c - 36 - hiras_1cnchan -4*hiras_1cnband)

      integer       hiras_1crecl         ! Record length in bytes
         parameter (hiras_1crecl = array_size_hiras1c*4)
!
!###########################################################################
!
!               HEADER RECORD
!
!###########################################################################

      character*3  hiras1c_h_site       !dataset creation site ID
      character*1  hiras1c_h_cfill1     !filler
      character*3  hiras1c_h_1bsite     !creation site for original 1B data
      character*1  hiras1c_h_cfill2     !filler
      integer*4    hiras1c_h_versnb     !level 1c format version number
      integer*4    hiras1c_h_versyr     !level 1c format version year
      integer*4    hiras1c_h_versdy     !level 1c format version day of year
      integer*4    hiras1c_h_hdrcnt     !count of header records in this data set
      integer*4    hiras1c_h_satid      !satellite id in series (4 for FY-3D)
      integer*4    hiras1c_h_instrument !WMO instrument identifier: 955=HIRAS
      integer*4    hiras1c_h_satht      !nominal satellite altitude, km*10
      integer*4    hiras1c_h_period     !nominal orbit period (seconds)
      integer*4    hiras1c_h_startorbit !orbit number (at start of file)
      integer*4    hiras1c_h_startdatayr!start of data set year
      integer*4    hiras1c_h_startdatady!start of data set day of the year
      integer*4    hiras1c_h_startdatatime !start of data set UTC time of day(ms)
      integer*4    hiras1c_h_endorbit   !orbit number (at end of file)
      integer*4    hiras1c_h_enddatayr  !end of data set year
      integer*4    hiras1c_h_enddatady  !end of data set day of the year
      integer*4    hiras1c_h_enddatatime!end of data set UTC time of day (ms)
      integer*4    hiras1c_h_scnlin     !count of scan lines in this data set
      integer*4    hiras1c_h_misscnlin  !count of missing scan lines
      integer*4    hiras1c_h_spare(2)   !Spare
      integer*4    hiras1c_h_wmosatid   !WMO satellite id (523 for FY-3D)
      integer*4    hiras1c_h_spare1     !spare
      integer*4    hiras1c_h_centre     !WMO id of reception centre
      integer*4    hiras1c_h_subcentre  !WMO id of reception subcentre

! HIRAS variables

      integer*4    hiras1c_h_recwords       !Size of record in integer*4 words
      integer*4    hiras1c_h_nfov           !Number of FOVs
      integer*4    hiras1c_h_nfor           !Number of FORs
      integer*4    hiras1c_h_nband          !Number of bands
      integer*4    hiras1c_h_startw(hiras_1cnband)    !First wavenumber *1E2
      integer*4    hiras1c_h_endw(hiras_1cnband)      !Last wavenumber *1E2
      integer*4    hiras1c_h_startchan(hiras_1cnband) !First channel in band
      integer*4    hiras1c_h_endchan(hiras_1cnband)   !Last channel in band
      integer*4    hiras1c_h_eigvn          !Version number of eigenvectors
      integer*4    hiras1c_h_npc            !Number of PCs in format
      integer*4    hiras1c_h_npcused        !Number of PCs used
      integer*4    hiras1c_h_chanvn         !Version number of channel selection
      integer*4    hiras1c_h_nchan          !Number of channels in format
      integer*4    hiras1c_h_nchanused               !Number of channels used
      integer*4    hiras1c_h_channels(hiras_1cnchan)  !List of HIRAS channels
                                                    ! range 1 to 1317
      integer*4    hiras1c_h_spare2(array_size_hiras1cspare)  !spare

      integer*4 hiras1c_hd(array_size_hiras1c)
      equivalence(hiras1c_hd(1),hiras1c_h_site)
      common /hiras1chd/ hiras1c_h_site, hiras1c_h_cfill1,
     & hiras1c_h_1bsite,
     & hiras1c_h_cfill2, hiras1c_h_versnb, hiras1c_h_versyr, 
     & hiras1c_h_versdy, hiras1c_h_hdrcnt, hiras1c_h_satid, 
     & hiras1c_h_instrument, hiras1c_h_satht,
     & hiras1c_h_period, hiras1c_h_startorbit, hiras1c_h_startdatayr,
     & hiras1c_h_startdatady, hiras1c_h_startdatatime,
     & hiras1c_h_endorbit,
     & hiras1c_h_enddatayr, hiras1c_h_enddatady, hiras1c_h_enddatatime,
     & hiras1c_h_scnlin, hiras1c_h_misscnlin, hiras1c_h_spare,
     & hiras1c_h_wmosatid, hiras1c_h_spare1,hiras1c_h_centre,
     & hiras1c_h_subcentre, hiras1c_h_recwords,hiras1c_h_nfov,
     & hiras1c_h_nfor,hiras1c_h_nband, hiras1c_h_startw,hiras1c_h_endw,
     & hiras1c_h_startchan,hiras1c_h_endchan,
     & hiras1c_h_eigvn, hiras1c_h_npc, hiras1c_h_npcused,
     & hiras1c_h_chanvn,
     & hiras1c_h_nchan, hiras1c_h_nchanused, hiras1c_h_channels,
     & hiras1c_h_spare2
     
      character*16 hiras1c_h_software_version
      equivalence(hiras1c_h_spare2(1),hiras1c_h_software_version)

!##############################################################################
!
!               DATA RECORD
!
!##############################################################################

      integer*4   hiras1c_scnlin       !scan line number
      integer*4   hiras1c_scnlinyr     !scan line year
      integer*4   hiras1c_scnlindy     !scan line day of year
      integer*4   hiras1c_scnlintime(hiras_1cnfor) !scan line UTC time of day (ms)

      integer*4   hiras1c_latlon(2,hiras_1cnfov,hiras_1cnfor)
!                     first   : 10^4 x (latitude)
!                     second  : 10^4 x (longitude)
      integer*4   hiras1c_angles(4,hiras_1cnfov,hiras_1cnfor)
!                     first:    10^2 x (local zenith angle)
!                     second:   10^2 x (local azimuth angle)
!                     third:    10^2 x (solar zenith angle)
!                     fourth:   10^2 x (solar azimuth angle)
      integer*4   hiras1c_satxyz(3)    ! x,y,z of satellite at mid-scan
      integer*4   hiras1c_ascdesc      ! ascending / descending orbit qualifier
      integer*4   hiras1c_orbit        ! orbit number
      integer*4   hiras1c_scalti       ! satellite altitude
      integer*4   hiras1c_surfht(hiras_1cnfov,hiras_1cnfor)       ! surface height
      integer*4   hiras1c_landfrac(hiras_1cnfov,hiras_1cnfor)     ! land fraction
      integer*4   hiras1c_landsea(hiras_1cnfov,hiras_1cnfor)      ! land/sea qualifier
      integer*4   hiras1c_cloudcover(hiras_1cnfov,hiras_1cnfor)   ! cloud cover (percent)
      integer*4   hiras1c_cloudtopht(hiras_1cnfov,hiras_1cnfor)   ! height of top of cloud (m)

      integer*4   hiras1c_radianceflgs(hiras_1cnfov,hiras_1cnfor) ! radiance flags (as CrIS)
      integer*4   hiras1c_scanqualflgs                            ! scan quality flags (as CrIS)
      integer*4   hiras1c_calqualflgs(hiras_1cnband,hiras_1cnfov) ! calibration quality
      integer*4   hiras1c_fovqualflgs(hiras_1cnband,hiras_1cnfov,
     & hiras_1cnfor)                                              !FOV quality
      integer*4   hiras1c_geolocqual                              !Geolocation quality
      integer*4   hiras1c_quality(hiras_1cnfov,hiras_1cnfor)      !Quality information

      integer*4  hiras1c_scores(hiras_1cnpc,hiras_1cnfov,hiras_1cnfor)    !PC scores
      integer*4  hiras1c_logfit(hiras_1cnband,hiras_1cnfov,hiras_1cnfor)  !Log Reconstruction Scores

!For radiance, put the channel index last, to allow shorter record length 
!to be used in the case of a channel subset

      integer*4  hiras1c_extraradiance(hiras_1cnfov,hiras_1cnfor,12)      !12 guard channels
      integer*4  hiras1c_radiance(hiras_1cnfov,hiras_1cnfor,
     &               hiras_1cnchan) !Radiance
                                                                       !Wm-2sr-1cm *1E7

      integer*4 hiras1c_dt(array_size_hiras1c)
      equivalence(hiras1c_dt(1),hiras1c_scnlin)
      common /hiras1cdt/ hiras1c_scnlin,hiras1c_scnlinyr,
     & hiras1c_scnlindy,
     & hiras1c_scnlintime,hiras1c_latlon,hiras1c_angles,
     & hiras1c_satxyz,hiras1c_ascdesc,hiras1c_orbit,hiras1c_scalti,
     & hiras1c_surfht,hiras1c_landfrac,hiras1c_landsea,
     & hiras1c_cloudcover,
     & hiras1c_cloudtopht,hiras1c_radianceflgs,hiras1c_scanqualflgs,
     & hiras1c_calqualflgs,hiras1c_fovqualflgs,hiras1c_geolocqual,
     & hiras1c_quality,hiras1c_scores,hiras1c_logfit,
     & hiras1c_extraradiance,hiras1c_radiance

