!+ Level 1D Data Set (HIRAS grid)
!
! Name:  hiras1d.h
!
! Purpose:
!   'INCLUDE' file for common /hiras1dhd/   (HIRAS level 1D header record)
!                  and common /hiras1ddt/   (HIRAS level 1D data record)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    07/02/2019  Initial, based on CrIS. This version only has
!                     dummy slots for MWTS and MERSI.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!   User adjustable parameters:
      integer       hiras_1dnchan        ! Number of channel slots present in 1d
         parameter (hiras_1dnchan = 431) !  Must be at least 1
      integer       hiras_1dnpc          ! Number of Principal Components present
         parameter (hiras_1dnpc = 1)     !  Must be at least 1
!
!   Normally fixed parameters:
      integer       hiras_1dnchanmwts    ! Number of MWTS/MWHS channels
         parameter (hiras_1dnchanmwts = 22)   !make it the same as ATMS
      integer       hiras_1dnfor         ! Number of FORs present in 1d
         parameter (hiras_1dnfor = 29)   !  Full resolution requires 9 records
      integer       hiras_1dnband        ! Number of bands
         parameter (hiras_1dnband = 3)
      integer       hiras_1dvnnb         ! hiras level 1D format version number
         parameter (hiras_1dvnnb = 1)
      integer       hiras_1dvnyr         ! format version year
         parameter (hiras_1dvnyr = 2019)
      integer       hiras_1dvndy         ! format version day
         parameter (hiras_1dvndy = 35)
      integer       hiras_1dnchanmersi   ! Number of MERSI channels in format
         parameter (hiras_1dnchanmersi = 1)

      integer       array_size_hiras1d   ! Length of array
         parameter (array_size_hiras1d = 
     &      11 + hiras_1dnfor*(53 + hiras_1dnchanmwts + hiras_1dnchan + 
     &           hiras_1dnpc + 2*hiras_1dnband + 2*hiras_1dnchanmersi) 
     &         + 2*hiras_1dnchanmwts)              

      integer       array_size_hiras1dspare   ! Length of spare header array
         parameter (array_size_hiras1dspare = 
     &      array_size_hiras1d - 49 - hiras_1dnchan -7*hiras_1dnchanmwts
     &      - 4*hiras_1dnband - 2*hiras_1dnchanmersi)

      integer       hiras_1drecl         ! Record length in bytes
         parameter (hiras_1drecl = array_size_hiras1d*4)
!
!###########################################################################
!
!               HEADER RECORD
!
!###########################################################################
!
      character*3  hiras1d_h_site       !dataset creation site ID
      character*1  hiras1d_h_cfill1     !filler
      character*3  hiras1d_h_1bsite     !creation site for original 1B data
      character*1  hiras1d_h_cfill2     !filler
      integer*4    hiras1d_h_versnb     !level 1d format version number
      integer*4    hiras1d_h_versyr     !level 1d format version year
      integer*4    hiras1d_h_versdy     !level 1d format version day of year
      integer*4    hiras1d_h_hdrcnt     !count of header records in this data set
      integer*4    hiras1d_h_satid      !satellite id in series (1 for NPP)
      integer*4    hiras1d_h_grid       !code for instrument grid
!            (5=HIRS; 6=MSU; 10=AMSU-A; 11=AMSU-B; 12=IASI; 13=MWTS; 14=HIRAS)
      integer*4    hiras1d_h_satht      !nominal satellite altitude, km*10
      integer*4    hiras1d_h_period     !nominal orbit period (seconds)
      integer*4    hiras1d_h_startorbit !orbit number (at start of file)
      integer*4    hiras1d_h_startdatayr!start of data set year
      integer*4    hiras1d_h_startdatady!start of data set day of the year
      integer*4    hiras1d_h_startdatatime !start of data set UTC time of day (ms)
      integer*4    hiras1d_h_endorbit   !orbit number (at end of file)
      integer*4    hiras1d_h_enddatayr  !end of data set year
      integer*4    hiras1d_h_enddatady  !end of data set day of the year
      integer*4    hiras1d_h_enddatatime!end of data set UTC time of day (ms)
      integer*4    hiras1d_h_scnlin     !count of scan lines in this data set
      integer*4    hiras1d_h_misscnlin  !count of missing scan lines
      integer*4    hiras1d_h_atovppvn   !ATOVPP version number (test vns = 9000+)
      integer*4    hiras1d_h_instruments!instruments present
!       (bit0=HIRS, bit1=MSU, bit3=AMSU-A, bit4=AMSU-B, bit5=AVHRR, bit 6=IASI,
!        bit7=MWTS, bit8=HIRAS)
      integer*4    hiras1d_h_wmosatid   !WMO satellite id (224 for NPP)
      integer*4    hiras1d_h_spare1     !spare
      integer*4    hiras1d_h_centre     !WMO id of reception centre
      integer*4    hiras1d_h_subcentre  !WMO id of reception subcentre

! HIRAS variables

      integer*4    hiras1d_h_recwords       !Size of record in integer*4 words
      integer*4    hiras1d_h_nfov           !Number of FOVs
      integer*4    hiras1d_h_nfor           !Number of FORs
      integer*4    hiras1d_h_nband          !Number of bands
      integer*4    hiras1d_h_startw(hiras_1dnband)    !First wavenumber *1E2
      integer*4    hiras1d_h_endw(hiras_1dnband)      !Last wavenumber *1E2
      integer*4    hiras1d_h_startchan(hiras_1dnband) !First channel in band
      integer*4    hiras1d_h_endchan(hiras_1dnband)   !Last channel in band
      integer*4    hiras1d_h_eigvn          !Version number of eigenvectors
      integer*4    hiras1d_h_npc            !Number of PCs in format
      integer*4    hiras1d_h_npcused        !Number of PCs used
      integer*4    hiras1d_h_chanvn         !Version number of channel selection
      integer*4    hiras1d_h_nchan          !Number of channels in format
      integer*4    hiras1d_h_nchanused               !Number of channels used
      integer*4    hiras1d_h_channels(hiras_1dnchan)  !List of hiras channels
                                                    ! range 1 to 8461

! MWTS variables

! The following variables characterise any image processing for MWTS
      integer*4    hiras1d_h_mwtsbeamversion   !User defined (see mwts_beamwidth.dat)
! FFT manipulation:
      integer*4    hiras1d_h_mwtsbeamwidth(hiras_1dnchanmwts)   !deg*1E3
      integer*4    hiras1d_h_mwtscutoff(hiras_1dnchanmwts)      !fraction*1E3
! Simple averaging:
      integer*4    hiras1d_h_mwtsnxaverage(hiras_1dnchanmwts)
      integer*4    hiras1d_h_mwtsnyaverage(hiras_1dnchanmwts)
      
! MWTS temperature-radiance conversion coefficients  (Planck function)
      integer*4    hiras1d_h_mwtsPlanck(3,hiras_1dnchanmwts)
!                  (1,*) = 10^6 x (central wavenumber)
!                  (2,*) = 10^6 x (constant 1)
!                  (3,*) = 10^6 x (constant 2)


! Pre-processing information

      integer*4    hiras1d_h_dT  ! size of data batches processed within ATOVPP
!                                (seconds   - see ATOVPP documentation)
      integer*4    hiras1d_h_mapBTmode  ! mapping method used for BTs
!      (1=nearest neighbour, 2=bilinear interpolation)
      integer*4    hiras1d_h_Acorrvn    ! vn no MWTS limb/emiss correction
!                                         (=0 if data not corrected)
      integer*4    hiras1d_h_Apcpvn     ! vn no MWTS precip/scatter detection
!                                         (=0 if not done)
      integer*4    hiras1d_h_Aantenna   ! vn no, MWTS antenna corrections

      integer*4    hiras1d_h_Ascat !threshold value for MWTS scattering index
      integer*4    hiras1d_h_ACrosby !threshold value for MWTS logistic precip
      integer*4    hiras1d_h_cloudtests  !Cloud tests used (0 if none)

      integer*4    hiras1d_h_recperscan !Records per scan: 1 to 9
      integer*4    hiras1d_h_fovmode    !0 = full resolution
                                       !Others TBD
      
! MERSI

      integer*4    hiras1d_h_nmersichan       !number of VIIRS channels in format
      integer*4    hiras1d_h_nmersichanused   !number of channels used
      integer*4    hiras1d_h_mersichans(hiras_1dnchanmersi)  !Channel numbers
      integer*4    hiras1d_h_mersibands(hiras_1dnchanmersi)  !Band numbers
                                                           ! 5=M, 6=I, 7-DN

      integer*4    hiras1d_h_spare2(array_size_hiras1dspare)  !spare

      integer*4 hiras1d_hd(array_size_hiras1d)
      equivalence(hiras1d_hd(1),hiras1d_h_site)
      common /hiras1dhd/ hiras1d_h_site, hiras1d_h_cfill1,
     & hiras1d_h_1bsite,
     & hiras1d_h_cfill2, hiras1d_h_versnb, hiras1d_h_versyr, 
     & hiras1d_h_versdy,
     & hiras1d_h_hdrcnt, hiras1d_h_satid, hiras1d_h_grid,
     & hiras1d_h_satht,
     & hiras1d_h_period, hiras1d_h_startorbit, hiras1d_h_startdatayr,
     & hiras1d_h_startdatady, hiras1d_h_startdatatime,
     & hiras1d_h_endorbit,
     & hiras1d_h_enddatayr, hiras1d_h_enddatady, hiras1d_h_enddatatime,
     & hiras1d_h_scnlin, hiras1d_h_misscnlin, hiras1d_h_atovppvn,
     & hiras1d_h_instruments, hiras1d_h_wmosatid, hiras1d_h_spare1,
     & hiras1d_h_centre, hiras1d_h_subcentre,
     & hiras1d_h_recwords,hiras1d_h_nfov,hiras1d_h_nfor,hiras1d_h_nband,
     & hiras1d_h_startw,hiras1d_h_endw,
     & hiras1d_h_startchan,hiras1d_h_endchan,
     & hiras1d_h_eigvn, hiras1d_h_npc, hiras1d_h_npcused,
     & hiras1d_h_chanvn,
     & hiras1d_h_nchan, hiras1d_h_nchanused, hiras1d_h_mwtsbeamversion, 
     & hiras1d_h_mwtsbeamwidth, hiras1d_h_mwtscutoff,
     & hiras1d_h_mwtsnxaverage, hiras1d_h_mwtsnyaverage,
     & hiras1d_h_mwtsplanck, hiras1d_h_dT,
     & hiras1d_h_mapBTmode, hiras1d_h_Acorrvn,
     & hiras1d_h_Apcpvn, hiras1d_h_Aantenna, hiras1d_h_Ascat,
     & hiras1d_h_ACrosby, hiras1d_h_cloudtests,
     & hiras1d_h_recperscan, hiras1d_h_fovmode,
     & hiras1d_h_nmersichan, hiras1d_h_nmersichanused,
     & hiras1d_h_mersichans, hiras1d_h_mersibands,
     & hiras1d_h_channels, hiras1d_h_spare2

!##############################################################################
!
!               DATA RECORD
!
!##############################################################################

!General information

      integer*4   hiras1d_scnlin            !scan line number
      integer*4   hiras1d_scnlinyr          !scan line year
      integer*4   hiras1d_scnlindy          !scan line day of year
      integer*4   hiras1d_scnlintime(hiras_1dnfor) !scan line UTC time of day (ms)

      integer*4   hiras1d_latlon(2,hiras_1dnfor)           !lat/lon in degrees
!                     first   : 10^4 x (latitude)
!                     second  : 10^4 x (longitude)
      integer*4   hiras1d_angles(4,hiras_1dnfor)           !scan angles
!                     first:    10^2 x (local zenith angle)
!                     second:   10^2 x (local azimuth angle)
!                     third:    10^2 x (solar zenith angle)
!                     fourth:   10^2 x (solar azimuth angle)

      integer*4   hiras1d_satxyz(3)    ! x,y,z of satellite at mid-scan
      integer*4   hiras1d_ascdesc      ! ascending / descending orbit qualifier
      integer*4   hiras1d_orbit        ! orbit number
      integer*4   hiras1d_scalti       ! satellite altitude
      integer*4   hiras1d_surfht(hiras_1dnfor)       ! surface height
      integer*4   hiras1d_landfrac(hiras_1dnfor)     ! land fraction
      integer*4   hiras1d_landsea(hiras_1dnfor)      ! land/sea qualifier
      integer*4   hiras1d_cloudcover(hiras_1dnfor)   ! cloud cover (percent)
      integer*4   hiras1d_cloudtopht(hiras_1dnfor)   ! height of top of cloud (m)

!Surfelev output (duplicates info in 1c but useful as a check)

      integer*4   hiras1d_surface(3,hiras_1dnfor)
!                     first   : surface height (metres)
!                     second  : surface type (0=sea, 1=mixed, 2=land)
!                     third   : land fraction (percent)

!MWTS mapped to HIRAS

      integer*4   hiras1d_nedtcoldmwts(hiras_1dnchanmwts)
      integer*4   hiras1d_nedtwarmmwts(hiras_1dnchanmwts)
      
      integer*4   hiras1d_btemps(hiras_1dnchanmwts,hiras_1dnfor)
!                 10^2 x scene brightness temperature (K) for MWTS channels

      integer*4   hiras1d_hirasqual(hiras_1dnfor)
!                 Quality control word for each field of view:
!                  (all bits off implies acceptable data)
!                  bit 27-4: bit n set to 1 if brightness temperature in
!                            MWTS channel n-3 is physically unreasonable or has
!                            not been calculated due to calibration  
!                            problems.
!                  bit 3-1:  bit n set to 1 if data for HIRAS detector n are bad
!                  bit 0:    bad or missing data (in any or all channels)

      integer*4   hiras1d_prepro(8,hiras_1dnfor) ! values from pre-processing
!                  (1,*):   pre-processing quality control word with flags
!                            (all bits off implies acceptable data)
!                  bit 31: set if MWTS surface types not all the same
!                  bit 30-28: spare
!                  bit 27: flag for cloud cost set for any mapped MWTS
!                  bit 26: scattering flag set for any mapped MWTS
!                          (only set over the sea)
!                  bit 25: logistic precipitation probability test
!                          calculated from MWTS data mapped to HIRAS grid
!                  bit 24: Grody light rainfall test calculated on HIRAS grid
!                  bit 23: spare
!                  bit 22: mismatch between surface type from topography
!                          dataset and from pre-processing (any MWTS)
!                  bit 21: fewer MWTS mapping co-locations than expected
!                  bit 20-4: spare   <zero fill>
!                  bit 3: spare
!                  bit 2: flag for cloud cost (recalculated on HIRAS grid)
!                  bit 1: flag for scattering index (recalculated on HIRAS grid)
!                  bit 0: set if MWTS data missing
!
!                  (2,*)    estimated surface type from the pre-processing:
!            1 = Bare young ice (i.e. new ice, no snow)
!            2 = Dry land (i.e. dry with or without significant vegetation)
!            3 = Dry snow (i.e. snow with water less than 2%, over land)
!            4 = Multi-year ice (i.e. old ice with snow [assumed dry] cover)
!            5 = Sea (i.e. open water, no islands, ice-free, WS=0 to 14m/s)
!            6 = Wet forest (i.e. established forest with wet canopy)
!            7 = Wet land (i.e. non-forested land with a wet surface)
!            8 = Wet snow (i.e. snow with water content > 2%, over land or ice)
!            9 = Desert
!
!                  (3,*) cost fn from PPASURF surface identification (*100)
!                  (4,*) scattering index (*100)
!                  (5,*) cirrus index (*100)
!                  (6,*) logistic precipitation probability (*100)
!                  (7,*) NWC-SAF (Bennartz) scattering index (K)
!                  (8,*) NWC-SAF (Bennartz) precipitation probabilities

      integer*4   hiras1d_emiss(13,hiras_1dnfor) !13 emissivity parameters

!HIRAS data

      integer*4   hiras1d_radianceflgs(hiras_1dnfor) ! radiance flags
!Bit(1=MS)
!1	Clear path
!2	Partly cloudy path
!3	Cloudy path
!4	Apodized
!5	Unapodized
!6	Reconstructed
!7	Cloud cleared
!8-14	Reserved
!All 15	Missing value

      integer*4 hiras1d_scanqualflgs              ! scan quality flags
!Bit(1=MS)
!1	Gap in Raw Data Record (RDR) data detected (i.e., missing scan(s)
!       preceding the current scan)
!2	Recorded time is not in sequence (i.e., the scan start time is out of
!       sequence)
!3	Lamda monitored calculation cannot be updated
!4	The measured temperatures of any instrument components (e.g., 
!       beam-splitter, scan mirror, scan baffle, etc.) are outside the
!       allowable ranges
!5	At least one of the monitored instrument temperatures has drifted more
!       than a specified tolerance value
!6-12	Reserved
!All 13	Missing value

      integer*4   hiras1d_calqualflgs(hiras_1dnband,hiras_1dnfor)  !cal. quality
                      !Same for all FOVS 1 to 9, but thinned data may have 
                      !different FOVs within a record
!Bit(1=MS of 9 bits)
!1	Lunar intrusion on first deep space view
!2	Lunar intrusion on second deep space view
!3-8	Reserved
!All 9	Missing value

      integer*4 hiras1d_fovqualflgs(hiras_1dnband,hiras_1dnfor) !FOV quality
!Bit(1=MS of 19 bits)
!1	Degraded SDR quality
!2	Invalid SDR quality
!3	Invalid SDR geolocation information
!4	Degraded radiometric calibration
!5	Invalid radiometric calibration
!6	Degraded spectral calibration
!7	Invalid spectral calibration
!8	Fringe count error detected and corrected
!9	Day/night indicator
!10	Invalid RDR data
!11	Significant fringe count error detected
!12	Bit trim failed
!13-18	Reserved
!All 19	Missing value

      integer*4   hiras1d_geolocqual                !Geolocation quality
!Value
!0  Nominal
!1  Missing at most a small gap of altitude and Ephemeris data
!2  Missing more than a small gap of altitude and Ephemeris data, but no more
!   than a granule boundary
!3  Missing more than a granule boundary of altitude and Ephemeris data
!4-14 Reserved
!15 Missing

      integer*4   hiras1d_quality(hiras_1dnfor)  !Quality information
!Value
!0  Not suspect                                                                  
!1  Suspect                                                                      
!2  Very suspect                                                                 
!3  Unusable                                                                     
      
!HIRAS cloud flag (user defined)

      integer*4   hiras1d_cloudflag(10,hiras_1dnfor)
      
!HIRAS radiance and PC scores

      integer*4   hiras1d_FOV(hiras_1dnfor) !30 values in range 1 to 270
      integer*4   hiras1d_scores(hiras_1dnpc,hiras_1dnfor) !PC scores
      integer*4   hiras1d_logfit(3,hiras_1dnfor)!Reconstruction Scores for 3 bands

!For radiance, put the channel index last, to allow shorter record length 
!to be used in the case of a channel subset

      integer*4   hiras1d_radiance(hiras_1dnfor,hiras_1dnchan)  !Radiance
                                                             !Wm-2sr-1cm *1E7

!MERSI

      integer*4   hiras1d_mersiradiance(hiras_1dnchanmersi,hiras_1dnfor)
      integer*4   hiras1d_mersireflectance(hiras_1dnchanmersi,
     &            hiras_1dnfor)

      integer*4 hiras1d_dt(array_size_hiras1d)
      equivalence(hiras1d_dt(1),hiras1d_scnlin)
      common /hiras1ddt/ hiras1d_scnlin,hiras1d_scnlinyr,
     &   hiras1d_scnlindy,
     &   hiras1d_scnlintime,hiras1d_latlon,hiras1d_angles,
     &   hiras1d_satxyz,hiras1d_ascdesc,hiras1d_orbit,
     &   hiras1d_scalti,hiras1d_surfht,hiras1d_landfrac,hiras1d_landsea,
     &   hiras1d_cloudcover,hiras1d_cloudtopht,hiras1d_surface,
     &   hiras1d_nedtcoldmwts,hiras1d_nedtwarmmwts,
     &   hiras1d_btemps,hiras1d_hirasqual,hiras1d_prepro,hiras1d_emiss,
     &   hiras1d_radianceflgs,hiras1d_scanqualflgs,hiras1d_calqualflgs,
     &   hiras1d_fovqualflgs,hiras1d_geolocqual,hiras1d_quality,
     &   hiras1d_cloudflag,hiras1d_FOV,hiras1d_scores,hiras1d_logfit,
     &   hiras1d_mersiradiance,hiras1d_mersireflectance,
     &   hiras1d_radiance

