!+
!$Id: hrs1b.h,v 1.3 2005/04/15 13:39:48 frna Exp $
! 
!DESCRIPTION 
!      include file for hirs 1b format handling
!
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
! Tiphaine Labrot
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
!
!  1.0     11/95  new release from NOAA available 30/10/95
!                    still problems in integer boudaries alignments
!                    and undefined scaling factors
!  2.0     11/05/98  modified to conform to NOAA release dated 
!  2.1     19/11/03  include NOAA v2 header format, for use following launch
!                    of NOAA-N (and also applied to NOAA-KLM).
!                    Update comments in data record for HIRS/4. N C Atkinson 
!  2.2     March 04  full orbit parametrisation P. Brunel
!  2.3     5/11/04   based on NOAA update of Sept 27 2004. N C Atkinson
!  2.4     22/11/04  remove second header format, since latest NOAA release
!                    is backward-compatible
!  2.5     28/01/05  update maximum HIRS lines number to 1240 T Labrot
!                    modified to conform to NOAA v2 format and comments T Labrot
!  2.6     02/03/05  change byte to character*1 for portability. NCA
!  2.6     17/08/12  add comment giving more info on hrs_qualind. NCA
!                    

!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!#######################################################################
!                                                                          !
!                      HIRS  PARAMETERS                                    !
!                                                                          !
!###########################################################################
       integer hrs_mxirchn     !IR channels of the HIRS
       parameter(hrs_mxirchn=19)
       integer hrs_mxchn       !channels of the HIRS (19 IR + 1 visible channel)
       parameter(hrs_mxchn=20) 
       integer hrs_mxpix       !maximum spot number
       parameter(hrs_mxpix=56)
       integer hrs_mxlin       !maximum HIRS lines number
       parameter(hrs_mxlin=1240)

       integer reclhrs         !length in byte of one HIRS record
       parameter(reclhrs=4608)
       integer chrs_size       !length in integer of one HIRS record
       parameter(chrs_size=1152)

!###########################################################################
!
!               HEADER RECORD Sept 2004 - for NOAA-N/IJPS era
!               Formulated for HIRS/4 but also used for HIRS/3
!
!###########################################################################
! FILE IDENTIFICATION
!--------------------
       character*3  hrs_h_siteid      !data set creation site ID
!                                     !CMS=Centre de Meteorologie Spatiale/France
!                                     !DSS=Dundee Satellite Receiving Station/UK
!                                     !NSS=National Environmental Satellite,Data and 
!                                     !    Information Service/USA
!                                     !UKM=United Kingdom Meteorological Office/UK
       character*1  hrs_h_blank       !ASCII blank      
       integer*2    hrs_h_l1bversnb   !level 1b format version number
       integer*2    hrs_h_l1bversyr   !level 1b format version year
       integer*2    hrs_h_l1bversdy   !level 1b format version day of year
       integer*2    hrs_h_reclg       !<reserved for logical record length>
       integer*2    hrs_h_blksz       !<reserved for block size>
       integer*2    hrs_h_hdrcnt      !count of header records in this data set
       integer*2    hrs_h_filler0(3)  !<zero fill>
       character*42 hrs_h_dataname    !data set name
       character*8  hrs_h_prblkid     !Processing block identification
       integer*2    hrs_h_satid       !Spacecraft identification code
!                                     !2=NOAA-L
!                                     !4=NOAA-K
!                                     !6=NOAA-M
!                                     !7=NOAA-N
!                                     !8=NOAA-N'
!                                     !11=Metop-1 (TBC)
!                                     !12=Metop-2 (TBC)
       integer*2    hrs_h_instid      !instrument ID
!                                     !301=s/n H301 (NOAA-L)
!                                     !302=s/n H302 (NOAA-K)
!                                     !303=s/n H303 (NOAA-M)
!                                     !304=s/n H304 (NOAA-N')
!                                     !305=s/n H305 (NOAA-N)
!                                     !306=s/n H306 (Metop-2)
!                                     !307=s/n H307 (Metop-1)
       integer*2    hrs_h_datatyp     !data type code 5=HIRS
       integer*2    hrs_h_tipsrc      !TIP source code
!                                     !(NOAA: values defined below 
!                                     ! Metop: zero fill)
!                                     !0=unused, i.e.,GAC/HRPT/LAC data
!                                     !1=GAC-embedded AMSU and TIP
!                                     !2=stored TIP (STIP)
!                                     !3=HRPT/LAC-embedded AMSU and TIP
!                                     !4=stored AIP (SAIP)
       integer      hrs_h_startdatajd !start of data set Julian Day(00h,1jan50)
       integer*2    hrs_h_startdatayr !start of data set year
       integer*2    hrs_h_startdatady !start of data set day of the year
       integer      hrs_h_startdatatime !start of data set UTC time of day
!                                       !in milliseconds
       integer      hrs_h_enddatajd   !end of data set Julian Day(00h,1jan50) 
       integer*2    hrs_h_enddatayr   !end of data set year
       integer*2    hrs_h_enddatady   !end of data set day of the year
       integer      hrs_h_enddatatime !end of data set UTC time of day 
!                                     !in milliseconds
       integer*2    hrs_h_cpidsyr     !year of last CPIDS Update
       integer*2    hrs_h_cpidsdy     !day of year of last CPIDS Update
       integer*2    hrs_h_fov1offset  !offset between start of scan and center 
!                                     !of first FOV
       character*6  hrs_h_instrtype   !instrument type 'HIRS/3' or 'HIRS/4'                                  

! DATA SET QUALITY INDICATORS
!----------------------------

       integer      hrs_h_inststat1   !instrument status
!                                     !bits 31-16: <zero fill>
!                                     !bits 15: Instrument power (0=off;1=on)
!                                     !bits 14: Electronics power (0=off;1=on)
!                                     !bits 13: Filter motor power (0=off;1=on)
!                                     !bits 12: Scan motor power (0=off;1=on)
!                                     !bits 11: Cooler heater (0=off;1=on)
!                                     !bits 10: Filter housing heater (0=off;1=on)
!                                     !bits  9: Cooler door release (0=disabled;1=enabled)
!                                     !bits  8: Cooler window heater (0=on;1=off)
!                                     !bits  7: Go to nadir position (0=no;1=yes/initiated)
!                                     !bits  6: Calibration sequence (0=disabled;1=enabled)
!                                     !bits  5: Cooler door closed (0=yes;1=no)
!                                     !bits  4: Cooler door fully open (0=yes;1=no)
!                                     !bits  3: Filter motor power level (0=normal;1=high)
!                                     !bits  2: Patch temperature controller (0=off;1=on)
!                                     !bits 1-0: <zero fill>
       integer*2    hrs_h_filler1     !<zero fill>
       integer*2    hrs_h_statchrecnb !record number of status change 
!                                     !(if 0, none occured)
       integer      hrs_h_inststat2   !second instrument status 
!                                     !(if previous word is 0, no change)
       integer*2    hrs_h_scnlin      !count of data records in this data set
       integer*2    hrs_h_callocsclin !count of calibrated, Earth located scan 
!                                     !lines in this data set
       integer*2    hrs_h_misscnlin   !count of missing scan lines
       integer*2    hrs_h_datagaps    !count of data gaps in this data set
       integer*2    hrs_h_okdatafr    !count of data frames without frame sync
!                                     !word errors(NOAA)
!                                     !<zero fill> (Metop)
       integer*2    hrs_h_pacsparityerr !count of PACS detected TIP parity errors(NOAA)
!                                     !<zero fill> (Metop) 
       integer*2    hrs_h_auxsyncerrsum !sum of all auxiliary sync errors 
!                                     !detected in the input data(NOAA)
!                                     !<zero fill> (Metop)
       integer*2    hrs_h_timeseqerr  !time sequence error
!                                     !  0=none
!                                     !  otherwise the record number of the first occurence
       integer*2    hrs_h_timeseqerrcode !time sequence error code
!                                     !if a bit is on(=1) then the statement is true
!                                     !bits 15-8: <zero fill>
!                                     !bits 7: time field is bad but can probably be 
!                                     !        inferred from the previous good time
!                                     !bits 6: time field is bad and can't be 
!                                     !        inferred from the previous good time
!                                     !bits 5: this record starts a sequence that is 
!                                     !        inconsistent with previous times
!                                     !        (i.e., there is a time discontinuity);
!                                     !        may be associated with a spacecraft clock 
!                                     !        update
!                                     !bits 4: start of a sequence that apparently 
!                                     !        repeats scan times that have been 
!                                     !        previously accepted
!                                     !bits3-0: <zero fill>
       integer*2    hrs_h_socclockupind !SOCC clock update indicator
!                                     ! 0=none during this orbit
!                                     ! otherwise the record number of the first occurence
       integer*2    hrs_h_locerrind   !Earth location error indicator  
!                                     ! 0=none during this orbit
!                                     ! otherwise the record number of the first occurence
       integer*2    hrs_h_locerrcode  !Earth location error code
!                                     !if a bit is on(=1) then the statement is true
!                                     !bits 15-8: <zero fill>
!                                     !bits 7: not Earth located because of bad time; 
!                                     !        Earth location fields zero-filled
!                                     !bits 6: Earth location questionable: 
!                                     !        questionable time code 
!                                     !bits 5: Earth location questionable: Marginal 
!                                     !        agreement with reasonableness check
!                                     !bits 4: Earth location questionable: Fails 
!                                     !        reasonableness check
!                                     !bits 3-2: <zero fill>
!                                     !bits 1: not Earth located because of satellite 
!                                     !        in-plane maneuver(Metop) 
!                                     !        <zero fill>(NOAA)
!                                     !bits 0: not Earth located because of satellite 
!                                     !        out-of-plane maneuver(Metop) 
!                                     !        <zero fill>(NOAA) 
       integer*2    hrs_h_pacsstatfield !PACS status bit field(NOAA: values defined below or
!                                     !Metop: <zero fill>)
!                                     !bit 15-3: <zero fill>
!                                     !bit 2: pseudonoise (0=normal data;1=pseudonoise data)
!                                     !bit 1: tape direction (0=reverse playback, 
!                                     !                         time decrementing)
!                                     !bit 0: data mode (0=test data;1=flight data)
       integer*2    hrs_h_pacsdatasrc !data source    
!                                     !  0=unused
!                                     !  1=Fairbanks, AK
!                                     !  2=Wallops Is., VA
!                                     !  3=SOCC
!                                     !  4=Svalbard, Norway
!                                     !  5=Monterey, CA
       integer      hrs_h_filler2     !<zero fill>
       character*8  hrs_h_spare1      !reserved for the ingester
       character*8  hrs_h_spare2      !reserved for the decommutation
       integer*2    hrs_h_filler3(5)  !<zero fill>

! CALIBRATION
!------------

       integer*2    hrs_h_autocalind  !auto calibration indicators bit field
!                                     !bit 15-1:<zero fill>
!                                     !bit 0: auto calibration override switch for HIRS/3-HIRS/4
!                                     !       0=normal calibration sequence enabled
!                                     !         during entire time period of this data set
!                                     !       1=calibration sequence was desabled at some 
!                                     !         point during time period of this data set 
       integer*2    hrs_h_solarcalyr  !year of most recent solar channel calibration
       integer*2    hrs_h_solarcaldy  !day of year of most recent solar channel calibration
       integer      hrs_h_calinf(4,hrs_mxchn)  !calibration information
!                                     !order of the channels : 
!                                     !1, 17, 2, 3, 13, 4, 18, 11, 19, 7, 8,
!                                     !20, 10, 14, 6, 5, 15, 12, 16, 9.
!                                     !10^6*(mean calibration slope of channel nochannel)
!                                     !10^6*(stantard deviation of calibration slope)
!                                     !10^6*(B1 for channel nochannel)
!                                     !10^6*(stantard deviation of linear regression
!                                     !      for B1 for channel nochannel)
       integer     hrs_h_filler4(2)   !<zero fill>

! TEMPERATURE-RADIANCE CONVERSION
!--------------------------------

       integer      hrs_h_tempradcnv(3,hrs_mxirchn) !temperature-radiance conversion                      
!                                     !order of channels = 1,2,3,4,...,18,19
!                                     !for nochannel = 1 to 19 : 
!                                     !10^6 x (nochannel central wavenumber for ch1 to ch 12)
!                                     !10^5 x (nochannel central wavenumber for ch13 to ch 19)
!                                     !10^6 x (nochannel constant 1)
!                                     !10^6 x (nochannel constant 2)
       integer*2    hrs_h_20solfiltirrad
!                                     !10^2 x (albedo-radiance ch20 solar filtered irradiance in W/m^2 )
!                                     !Note scale factor different from pre-2004 format
       integer*2    hrs_h_20equifiltwidth  
!                                     !10^4 x (albedo-radiance ch 20 equivalent filter width)
!                                     !Note scale factor different from pre-2004 format
       integer      hrs_h_filler5(2)  !<zero fill>

! NAVIGATION
!-----------

       character*8  hrs_h_modelid     !WGS-72=World Geodetic Survey 1972
       integer*2    hrs_h_nadloctol   !10*(Nadir Earth location tolerance in kilometers
       integer*2    hrs_h_locbit      !Earth location bit field
!                                     !bit 15-3: <zero fill>
!                                     !bit 2: dynamic attitude error correction
!                                     !       (0=not performed;1=performed)
!                                     !bit 1: reasonableness test 
!                                     !       (0=inactive; 1=active)
!                                     !bit 0: constant attitude error correction 
!                                     !       (0=not performed;1=performed)
       integer*2    hrs_h_filler6     !<zero fill>
       integer*2    hrs_h_rollerr     !10^3 x (constant roll attitude error(dg))
       integer*2    hrs_h_pitcherr    !10^3 x (constant pitch attitude error(dg))
       integer*2    hrs_h_yawerr      !10^3 x (constant yaw attitude error(dg))
       integer*2    hrs_h_epoyr       !epoch year for orbit vector
       integer*2    hrs_h_epody       !day of epoch year for orbit vector
       integer      hrs_h_epotime     !epoch UTC time of day for orbit vector in milliseconds
       integer      hrs_h_smaxis      !10^5 x (semi-major axis in kilometers)
       integer      hrs_h_eccen       !10^8 x (eccentricity)
       integer      hrs_h_incli       !10^5 x (inclination in degrees) 
       integer      hrs_h_argper      !10^5 x (argument of perigee in degrees)
       integer      hrs_h_rascnod     !10^5 x (right ascension of the
!                                     !        ascending node in degrees) 
       integer      hrs_h_manom       !10^5 x (mean anomaly in degrees)
       integer      hrs_h_xpos        !10^5 x (position vector X component
!                                     !          in kilometers)
       integer      hrs_h_ypos        !10^5 x (position vector Y component
!                                     !        in kilometers) 
       integer      hrs_h_zpos        !10^5 x (position vector Z component
!                                     !        in kilometers)
       integer      hrs_h_xvel        !10^8 x (velocity vector X-dot component
!                                     !        in kilometers/second)
       integer      hrs_h_yvel        !10^8 x (velocity vector Y-dot component 
!                                     !        in kilometers/second)
       integer      hrs_h_zvel        !10^8 x (velocity vector Z-dot component
!                                     !        in kilometers/second) 
       integer      hrs_h_earthsun    !10^6 x (Earth/sun distance ratio)
       integer      hrs_h_filler7(4)  !<zero fill>

! ANALOG TELEMETRY CONVERSION
!----------------------------
! Scaling factors 10**(2, 2, 3, 3, 3, 5)
! Temperatures in degres Celsius

       integer*4    hrs_h_rdtemp(6)    !radiator temp.conversion coefficients(6)
       integer*4    hrs_h_bptemp(6)    !base plate temp. conv. coef. (6)
       integer*4    hrs_h_eltemp(6)    !electronics temp. conv. coef. (6)
       integer*4    hrs_h_pchtemp(6)   !patch temp. conv. coef. (6)
       integer*4    hrs_h_fhcc(6)      !filter housing controller current conv. coef. (6)
       integer*4    hrs_h_scnmtemp(6)  !motor temperature conv. coef.(6)
       integer*4    hrs_h_fwmtemp(6)   !filter wheel motor temp. conv. coef.(6) 
       integer*4    hrs_h_p5v(6)       !+5 VDC monitor conv. coef.(6)
       integer*4    hrs_h_p10v(6)      !+10 VDC TLM/DC/DC conv. coef.(6) 
       integer*4    hrs_h_p75v(6)      !+7.5 VDC TLM/DC/DC conv. coef.(6)
       integer*4    hrs_h_m75v(6)      !-7.5 VDC TLM/DC/DC conv. coef.(6)
       integer*4    hrs_h_p15v(6)      !+15 VDC monitor conv. coef.(6)
       integer*4    hrs_h_m15v(6)      !-15 VDC monitor conv. coef.(6)
       integer*4    hrs_h_fwmcur(6)    !filter wheel motor current conv.coef. (6)
       integer*4    hrs_h_scmcur(6)    !scan motor current conv. coef.(6)
       integer*4    hrs_h_pchcpow(6)   !patch controller power conv. coef.(6)

! DIGITAL A TELEMETRY CONVERSION
!-------------------------------
! Coefficients are given to get temperatures in degrees Celcius
! Scaling factors for coefficients:10**(6, 9, 14, 17, 21, 25)
      
       integer*4    hrs_h_iwtcnttmp(6,5) !internal Warm Target, Temperature sensors
!                                        !4 sensors for NOAAKLM
!                                        !5 sensors for NOAANN'and Metop
       integer*4    hrs_h_ictcnttmp(6,4) !internal Cold Target, Temperature sensors 
!                                        !4 sensors for NOAAKLM
!                                        !1 sensor  for NOAANN' and Metop      
       integer*4    hrs_h_tttcnttmp(6)   !tertiary Telescope, Temperature sensor
!                                        !0 sensor for NOAAKLM
!                                        !1 sensor  for NOAANN'and Metop
       integer*4    hrs_h_fwcnttmp(6,4)  !filter wheel housing,temperature sensors
!                                        !4 sensors.
       integer*4    hrs_h_patchexpcnttmp(6) !patch temperature (expanded Scale)
       integer*4    hrs_h_fsradcnttmp(6) !first stage radiator temperature sensor
       integer*4    hrs_h_scmircnttmp(6) !scan mirror temperature
       integer*4    hrs_h_pttcnttmp(6)   !primary telescope, temperature sensor
       integer*4    hrs_h_sttcnttmp(6)   !secondary telescope, temperature sensor
       integer*4    hrs_h_bpcnttmp(6)    !baseplate temperature
       integer*4    hrs_h_electcnttmp(6) !electronics temperature
       integer*4    hrs_h_patchfcnttmp(6) !patch temperature full range
       integer*4    hrs_h_scmotcnttmp(6) !scan motor temperature
       integer*4    hrs_h_fwmcnttmp(6)   !filter wheel motor temperature
       integer*4    hrs_h_chsgcnttmp(6)  !cooler housing temperature

       integer*4    hrs_h_conversions(11) !conversion constants
!                                         !word1:10^9 x filter wheel housing heater current 
!                                         !word2:10 x Eelectronic calibration digital 
!                                         !           to analog converter  
!                                         !word3: 10^17 x patch control power
!                                         !word4: 10^9 x scan motor current
!                                         !word5: 10^9 x filter motor current
!                                         !word6: 10^8 x +15 VDC
!                                         !word7: 10^8 x -15 VDC
!                                         !word8: 10^8 x +7.5 VDC
!                                         !word9: 10^8 x -7.5 VDC
!                                         !word10: 10^8 x +10 VDC
!                                         !word11: 10^8 x +5 VDC

! LUNAR CONTAMINATION
!--------------------
       integer*2    hrs_h_moonscnlin 
!                         !count of space view scans containing lunar-contaminated
!                         !=-1 if the detection algorithm for lunar contamination 
!                         !    is turned off
!                         !=0  if the detection algorithm for lunar contamination 
!                         !    is turned on and no lunar-contaminated space view 
!                         !    scans were found.
!                         !=>0 if the detection algorithm for lunar contamination 
!                         !    is turned on and the value in this field represents
!                         !    the number of lunar-contaminated space view scans.
                         
      integer*2    hrs_h_moonthresh
!                         !100 x lunar angle thresold in degrees
!                         !any space view whose lunar angle is less than this 
!                         !value is flagged as being "lunar contaminated" and 
!                         !is not used in the calibration.

! 24-HOUR AVERAGE SPACE VIEW COUNTS
!----------------------------------
      integer*4    hrs_h_avspcounts(hrs_mxchn)
!                         !24-hour average space view counts for each channel
!                         !Order of the channels: 1,17,2,3,13,4,18,11,19,7,
!                         !8,20,10,14,6,5,15,12,16,9
                         
! METOP MANOEUVRES IDENTIFICATION
!--------------------------------
! For NOAA, these fields are zero-fill
      integer*2    hrs_h_startmanyr   !start of maneuver Year
      integer*2    hrs_h_startmandy   !start of maneuver day of Year
      integer*4    hrs_h_startmantime !start of maneuver UTC time of day (milliseconds)
      integer*2    hrs_h_endmanyr     !end of maneuver Year
      integer*2    hrs_h_endmandy     !end of maneuver day of Year
      integer*4    hrs_h_endmantime   !end of maneuver UTC time of day (milliseconds)
      integer*4    hrs_h_deltav(3)    !change in spacecraft velocity
      integer*4    hrs_h_mass(2)      !spacecraft mass
                                                     
! LAST FILLER
!------------
      integer*2    hrs_h_filler8(1302) !<zero fill>

      integer hrs_hd(chrs_size)
      equivalence(hrs_hd(1),hrs_h_siteid)
      common/hrs1bhd/hrs_h_siteid,hrs_h_blank,
     & hrs_h_l1bversnb,hrs_h_l1bversyr,hrs_h_l1bversdy,
     & hrs_h_reclg,hrs_h_blksz,hrs_h_hdrcnt,hrs_h_filler0,
     & hrs_h_dataname,
     & hrs_h_prblkid,hrs_h_satid,hrs_h_instid,hrs_h_datatyp,
     & hrs_h_tipsrc,
     & hrs_h_startdatajd,hrs_h_startdatayr,hrs_h_startdatady,
     & hrs_h_startdatatime,hrs_h_enddatajd,hrs_h_enddatayr,
     & hrs_h_enddatady,
     & hrs_h_enddatatime,hrs_h_cpidsyr,hrs_h_cpidsdy,
     & hrs_h_fov1offset,hrs_h_instrtype,
     & hrs_h_inststat1,hrs_h_filler1,
     & hrs_h_statchrecnb,hrs_h_inststat2,
     & hrs_h_scnlin,hrs_h_callocsclin,hrs_h_misscnlin,
     & hrs_h_datagaps,
     & hrs_h_okdatafr,hrs_h_pacsparityerr,hrs_h_auxsyncerrsum,
     & hrs_h_timeseqerr,hrs_h_timeseqerrcode,hrs_h_socclockupind,
     & hrs_h_locerrind,
     & hrs_h_locerrcode,
     & hrs_h_pacsstatfield,hrs_h_pacsdatasrc,hrs_h_filler2,
     & hrs_h_spare1,hrs_h_spare2,
     & hrs_h_filler3,
     & hrs_h_autocalind,hrs_h_solarcalyr,
     & hrs_h_solarcaldy,hrs_h_calinf,
     & hrs_h_filler4,
     & hrs_h_tempradcnv,hrs_h_20solfiltirrad,
     & hrs_h_20equifiltwidth,hrs_h_filler5,
     & hrs_h_modelid,hrs_h_nadloctol,hrs_h_locbit,hrs_h_filler6,
     & hrs_h_rollerr,hrs_h_pitcherr,hrs_h_yawerr,hrs_h_epoyr,
     & hrs_h_epody,
     & hrs_h_epotime,hrs_h_smaxis,hrs_h_eccen,hrs_h_incli,
     & hrs_h_argper,
     & hrs_h_rascnod,hrs_h_manom,hrs_h_xpos,hrs_h_ypos,hrs_h_zpos,
     & hrs_h_xvel,
     & hrs_h_yvel,hrs_h_zvel,hrs_h_earthsun,hrs_h_filler7,
     & hrs_h_rdtemp,
     & hrs_h_bptemp,hrs_h_eltemp,hrs_h_pchtemp,hrs_h_fhcc,
     & hrs_h_scnmtemp,hrs_h_fwmtemp,hrs_h_p5v,hrs_h_p10v,
     & hrs_h_p75v,hrs_h_m75v,hrs_h_p15v,hrs_h_m15v,
     & hrs_h_fwmcur,hrs_h_scmcur,hrs_h_pchcpow,
     & hrs_h_iwtcnttmp,hrs_h_ictcnttmp,hrs_h_tttcnttmp,
     & hrs_h_fwcnttmp,hrs_h_patchexpcnttmp,hrs_h_fsradcnttmp,
     & hrs_h_scmircnttmp,hrs_h_pttcnttmp,hrs_h_sttcnttmp,
     & hrs_h_bpcnttmp,hrs_h_electcnttmp,hrs_h_patchfcnttmp,
     & hrs_h_scmotcnttmp,hrs_h_fwmcnttmp,hrs_h_chsgcnttmp,
     & hrs_h_conversions,hrs_h_moonscnlin,hrs_h_moonthresh,
     & hrs_h_avspcounts,
     & hrs_h_startmanyr,hrs_h_startmandy,hrs_h_startmantime,
     & hrs_h_endmanyr,hrs_h_endmandy,hrs_h_endmantime,
     & hrs_h_deltav,hrs_h_mass,hrs_h_filler8
     
!#######################################################################
!
!               DATA RECORD Sept 2004 - for NOAA-N/IJPS era
!               Formulated for HIRS/4 but also used for HIRS/3 
!
!#######################################################################
! SCAN LINE INFORMATION
!----------------------

      integer*2   hrs_scnlin       !scan line number
!                                  !(cumulative, starting with 1)      
      integer*2   hrs_scnlinyr     !scan line year
      integer*2   hrs_scnlindy     !scan line day of year
      integer*2   hrs_clockdrift   !satellite clock drift delta in milliseconds
      integer     hrs_scnlintime   !scan line UTC time of day in milliseconds
      integer*2   hrs_scnlinf      !scan line bit field
!                                  !bit 15: satellite direction
!                                  !        0=northbound
!                                  !        1=southbound
!                                  !bit 14: clock drift correction
!                                  !        0=not corrected
!                                  !        1=scan time corrected for clock drift
!                                  !bit 13-0: <zero fill>
      integer*2   hrs_mjfrcnt      !major frame count for NOAA
!                                  !(cumulative, starting with 1)
!                                  !<zero fill> for Metop
      integer*2   hrs_scnpos       !scan position number in 32 seconds cycle
!                                  !for NOAA; <zero fill> for Metop
      integer*2   hrs_scntyp       !scan type code
!                                  ! 0 : Earth view
!                                  ! 1 : space view
!                                  ! 2 : cold black body view  
!                                  ! 3 : main black body view  
      integer*4   hrs_filler1(2)   !<zero fill>

! QUALITY INDICATORS
!-------------------

      integer     hrs_qualind      !quality indicator bit field
!                 (if a bit is on(=1) then the statement is true.)
!                   bit 31: do not use scan for product generation
!                   bit 30: time sequence error detected with this scan 
!                   bit 29: data gap precedes this scan
!                   bit 28: insufficient data for calibration
!                   bit 27: Earth location data not available
!                   bit 26: first good time following a clock update
!                           (nominally 0)
!                   bit 25: instrument status changed with this scan
!        NOAA:      bit 24-0: <zero fill>
!        AAPP:      bit 23: time field is bad but can probably be inferred from
!                           the previous good time.
!                   bit 19: tip synch error
!                   bit 18: tip minor or major frame sequence error
!                   bit 17: tip status error
!                   bit 16: tip parity error
!                   bit 15-0: <zero fill>

      integer     hrs_linqualflgs   !scan line quality flags
!           if a bit is on(=1) then the statement is true.
!
!           Additional calibration problem code:
!                 bit 31: not calibrated because of satellite manoeuvre (METOP)
!                         or <zero fill> (NOAA)
!                 bit 30-24: <zero fill>
!
!           time problem code :
!           (all bits off implies the scan time is as expected)
!                 bit 23: time field is bad but can probably be inferred
!                         from the previous good time
!                 bit 22: time field is bad and can't be inferred 
!                         from the previous good time
!                 bit 21: this record starts a sequence that is inconsistent  
!                         with previous times (i.e., there is a time
!                         discontinuity). This may or may not be associated
!                         with a spacecraft clock update. (see bit 26 above)
!                 bit 20: start of a sequence that apparently repeats scan
!                         times that have been previously accepted.
!                 bits 19-16: <zero fill>
!
!            calibration problem code :
!            (Note these bits complement the channel indicators; all bits set
!             to 0 indicates normal calibration)
!                  bit 15: scan line was not calibrated because of bad time
!                  bit 14: scan line was calibrated using fewer than the
!                          preferred number of scan lines because of 
!                          proximity to start or end of data set or
!                          to a data gap.
!                  bit 13: scan line was not calibrated because of bad or
!                          insufficient PRT data
!                  bit 12: scan line was calibrated but with marginal PRT data
!                  bit 11: some uncalibrated channels of this scan.
!                          See channel indicators.
!                  bit 10: uncalibrated due to instrument mode
!                  bit  9: space view scan is lunar contaminated
!                  bit  8: lunar-contaminated space view scan was corrected
!                          (only applicable if bit 9 is set)
!
!             Earth location problem code
!             (all bits set to 0 implies the Earth location was normal)
!                  bit 7: not Earth located because of bad time
!                  bit 6: Earth location questionable because of questionable
!                         time code (see time problem flags above)
!                  bit 5: Earth location questionable. Only marginal agreement 
!                         with reasonableness check.
!                  bit 4: Earth location questionable. Fails reasonableness
!                         check
!                  bit 3-2: <zero fill>
!                  bit 1: <zero fill> (NOAA)
!                       Not calibrated because of in-plane manoeuvre (Metop)
!                  bit 0: <zero fill> (NOAA)
!                       Not calibrated because of out-of-plane manoeuvre (Metop)
!
      integer*2   hrs_chqualflg(hrs_mxchn) !quality flag for each channel
!                 order of the channels: 
!                 1, 17, 2, 3, 13, 4, 18, 11, 19, 7, 8,
!                 20, 10, 14, 6, 5, 15, 12, 16, 9.
!                 (all bits off implies a good calibration)
!                      bit 15-6: <zero fill>
!                      bit  5: all bad blackbody counts for scan line
!                      bit  4: all bad space view counts for scan line
!                      bit  3: all bad PRTs counts for scan line
!                      bit  2: marginal blackbody view counts for this line
!                      bit  1: marginal space view counts for this line
!                      bit  0: marginal PRTs counts for scan line
!
      character*1  hrs_mnfrqual(0:63)     !minor frame quality indicators
!                                    for nel 0 to 63
!       (for bits 7 through 1,if bit is on(=1) then statement is true)    
!               bit  7 : 1 = this frame suspect due to a time error (NOAA)
!               bit  6 : 1 = this frame contains data gap data fill (NOAA)
!               bit  5 : 1 = this frame contains TIP dwell data fill (NOAA)
!               bit  4 : 1 = data suspect due to PACS QC error (NOAA)
!               bit  3 : 1 = mirror locked during this frame
!               bit  2 : 1 = mirror position error during this frame
!               bit  1 : 1 = mirror was moving during this frame
!               bit  0 : minor word odd parity bit 
      integer*4  hrs_filler2(4)     ! <zero fill> 

! CALIBRATION COEFFICIENTS
!-------------------------

      integer     hrs_calcof(3,hrs_mxchn)   !primary calibration coefficients
!                 order of the channels:
!                 1, 17, 2, 3, 13, 4, 18, 11, 19, 7, 8,
!                 20, 10, 14, 6, 5, 15, 12, 16, 9.
!                    - 10^12*the second one
!                    - 10^9*the first one
!                    - 10^6*the zeroth one
      integer     hrs_scalcof(3,hrs_mxchn)  !spare calibration coefficients
!                 order of the channels:
!                 1, 17, 2, 3, 13, 4, 18, 11, 19, 7, 8,
!                 20, 10, 14, 6, 5, 15, 12, 16, 9.
!                    - 10^12*the second one
!                    - 10^9*the first one
!                    - 10^6*the zeroth one
    
! NAVIGATION
!-----------

      INTEGER*2  hrs_yawsteering(3) !computed yaw steering angles for METOP
                                    !Yaw, pitch, roll (degrees)
      INTEGER*2  hrs_totattcorr(3)  !Total applied attitude correction for METOP
                                    !1000* Yaw, pitch, roll (degrees)
      integer    hrs_navstat    !navigation status bit field
!          (bits20-18 are Metop specific and will contain zero fill for NOAA)
!          (bits11-0 are NOAA specific and will contain zero fill for Metop)
!               bits31-21: <zero fill>
!               bits20-19: yaw steering parameters usage indicator
!                          0=no yaw steering correction
!                          1=computed parameters from Metop data stream
!                          2=measured parameters from Metop data stream
!                          3=computed parameters from AELDS
!               bit 18   : Metop maneuver indicator 
!                          0=scan does not occur during a Metop in-plane 
!                            or out-of-plane maneuver
!                          1=scan, or some part of it, occurs during
!                            a maneuver
!               bit 17   : Earth location at the satellite subpoint is 
!                          accurate and reasonable, i.e., is within 
!                          tolerance defined by "nadir Earth location
!                          tolerance" in header
!                          0=out of tolerance
!                          1=in tolerance
!               bit 16    :Earth location corrected for Euler angles
!                          0=false
!                          1=true
!               bits15-12: Earth location indicator
!                          0=Earth location available
!                          1=user ephemerics files greater than 24hours old
!                          2=no Earth location available
!               bits11-08: spacecraft attitude control 
!                          0=operating in YGC or nominal mode
!                          1=operating in another mode
!                          2=attitude exceeds nominal tolerance
!                          3=both 1 and 2
!               bits07-04: attitude SMODE 
!                          0=nominal mode
!                          1=rate nulling mode
!                          2=YGC mode
!                          3=search mode
!                          4=coast mode
!               bits03-00: attitude PWTIP$AC
!                          0=nominal mode/no test
!                          1=yaw axis test in progress
!                          2=roll axis test in progress
!                          3=pitch axis test in progress
      integer    hrs_attangtime     !time associated angles (seconds)
      integer*2  hrs_rollang         !10^3xRoll angle in degrees
      integer*2  hrs_pitchang        !10^3xPitch angle in degrees
      integer*2  hrs_yawang          !10^3xYaw angle in degrees
      integer*2  hrs_scalti        !10*spacecraft altitude (MSL) in km 
      integer*2  hrs_ang(168) !scan angles in degrees
!                     word1: 100*solar zenith angle, point 1
!                     word2: 100*satellite zenith angle, point 1
!                     word3: 100*local azimuth angle, point 1
!                     word4: 100*solar zenith angle, point 2
!                     word5: 100*satellite zenith angle, point 2
!                     word6: 100*local azimuth angle, point 2
!                     ......
!                     word166: 100*solar zenith angle, point 56
!                     word167: 100*satellite zenith angle, point 56
!                     word168: 100*local azimuth angle, point 56
      integer*4  hrs_pos(112) !Earth location
!                     (north latitude and east longitude are positive)
!                     word1: 10^4*latitude in degrees, point 1
!                     word2: 10^4*longitude in degrees, point 1
!                     word3: 10^4*latitude in degrees, point 2
!                     word4: 10^4*longitude in degrees, point 2
!                     ......
!                     word112: 10^4*longitude in degrees, point 56
      integer*2  hrs_moonang     !100*angle between moon and space view
      integer*2  hrs_filler3(3)  ! <zero fill>

! HIRS DATA ELEMENTS
!-------------------

      integer*2   hrs_elem(24,0:63)        !element data
!             for nel 0 to 63
!               word 1     bits 15-8 : scan encoder position
!                          bits  7-3 : electronic cal level indicator
!                          bits  2-0 : spare
!               word 2     bits 15-13: spare
!                          bits 12-7 : channel 1 period monitor
!                          bits  6-1 : element number
!                          bit     0 : filter sync designator
!               word 23    bits flag
!                          bit    15 : valid data flag
!                          bit    14 : odd bit parity
!                          bits 13-0 : spare
!               word 24    zero fill
!
!             for nel 0 to 55
!               word 3-22: radiometric channel no. 1, 17, 2, 3, 13, 4, 
!                          18, 11, 19, 7, 8, 20, 10, 14, 6, 5, 15, 12, 16, 9
!
!             for nel 56
!               word 3-22: positive calibration channel no. 1, 17, 2, 3, 13, 4, 
!                          18, 11, 19, 7, 8, 20, 10, 14, 6, 5, 15, 12, 16, 9
!
!             for nel 57
!               word 3-22: negative calibration channel no. 1, 17, 2, 3, 13, 4,
!                          18, 11, 19, 7, 8, 20, 10, 14, 6, 5, 15, 12, 16, 9
!
!             for nel 58
!               word 3-7:   internal warm target, temp sensor #1 (5 readings)
!               word 8-12:  internal warm target, temp sensor #2
!               word 13-17: internal warm target, temp sensor #3
!               word 18-22: internal warm target, temp sensor #4
!
!
!             for nel 59
!               word 3-7: internal cold target, temp sensor #1 (5 readings)
!               word 8-12: HIRS/4: Analog ground 3
!                          HIRS/3: internal cold target, temp sensor #2
!               word 13-17 HIRS/4: Internal Warm Target, temp Sensor #5
!                          HIRS/3: internal cold target, temp sensor #3
!               word 18-22 HIRS/4: Tertiary Telescope Temperature Sensor
!                          HIRS/3: internal cold target, temp sensor #4
!
!             for nel 60
!               word 3-7: internal filter wheel housing, temp sensor #1
!               word 8-12: internal filter wheel housing, temp sensor #2 
!               word 13-17: internal filter wheel housing, temp sensor #3
!               word 18-22: internal filter wheel housing, temp sensor #4
!
!             for nel 61
!               word 3-7: patch temp (5 readings)
!               word 8-12: 1st stage radiator 
!               word 13-17: filter wheel housing heater current
!               word 18-22: elec.cal.digital to analog converter 
!
!             for nel 62
!               word 3      scan mirror temperature
!               word 4      primary telescope temperature
!               word 5      secondary telescope temperature
!               word 6      HIRS baseplate temperature
!               word 7      HIRS electronics temperature
!               word 8      patch temperature full range
!               word 9      scan motor temperature
!               word 10     filter whell motor temperature
!               word 11     cooler housing temperature
!               word 12     patch control power
!               word 13     scan motor current
!               word 14     filter motor current
!               word 15     +15 VDC
!               word 16     -15 VDC
!               word 17     +7.5 VDC
!               word 18     -7.5 VDC
!               word 19     +10 VDC
!               word 20     + 5 VDC
!               word 21     analog ground
!               word 22     analog ground
!
!             for nel 63
!               word 3      line counter
!               word 4      first status word
!                                bits 15-8 : <zero fill>
!                                bit     7 : instrument on/off
!                                bit     6 : scan motor on/off
!                                bit     5 : filter wheel on/off
!                                bit     4 : electronics on/off
!                                bit     3 : cooler heat on/off
!                                bit     2 : internal warm target pos.
!                                bit     1 : internal cold target pos.
!                                bit     0 : space position
!               word 5      second status word
!                                bits 15-8 : <zero fill>
!                                bit     7 : nadir pos.
!                                bit     6 : calibration enable/disable
!                                bit     5 : cooler door release enable/disable
!                                bit     4 : cooler door open
!                                bit     3 : cooler door closed
!                                bit     2 : filter housing heat on/off
!                                bit     1 : patch temp. control on/off
!                                bit     0 : filter motor power high
!               word 6      Data verification binary code
!               word 7-22:  spare

        integer*2  hrs_filler4(6)    !<zero fill>

! DIGITAL B HOUSEKEEPING TELEMETRY
!---------------------------------
      integer*2   hrs_digitbupdatefg   !digital "B" telemetry update flags
!         (if bit=0, associated telemetry item is up-to-date.
!          if bit=1, associated telemetry item was not updated during most recent
!                    telemetry cycle-possibly due to lost frame)
      
      integer*2   hrs_digitbwrd  !digital "B" data from TIP
!                        bit 15: instrument power (0=off,1=on)
!                        bit 14: electronics power (0=off,1=on)
!                        bit 13: filter motor power (0=off,1=on)
!                        bit 12: scan motor power (0=off,1=on)
!                        bit 11: cooler heater (0=off,1=on)
!                        bit 10: filter housing heater (0=off,1=on)
!                        bit 9 : cooler door release (0=disabled,1=enabled)
!                        bit 8 : cooler window heater (0=on,1=off)
!                        bit 7 : go to nadir position (0=no,1=yes/initiated)
!                        bit 6 : calibration sequence (0=disabled,1=enabled)
!                        bit 5 : cooler door closed (0=yes,1=no)
!                        bit 4 : cooler door fully open (0=yes,1=no)
!                        bit 3 : filter motor power level (0=normal,1=yes)
!                        bit 2 : patch temperature controller (0=off,1=on)
!                        bit 1-0: <zero fill>

! ANALOG HOUSEKEEPING TELEMETRY
!------------------------------
      integer*4   hrs_analogupdatefg  !analog telemetry update flags
!         (if bit=0, associated telemetry item is up-to-date.
!          if bit=1, associated telemetry item was not updated during most recent
!                    telemetry cycle-possibly due to lost frame)
!                        bits 31-17: <zero fill> 
!                        bit 16: patch controller power
!                        bit 15: scan motor current
!                        bit 14: filter wheel motor current
!                        bit 13: -15VDC monitor
!                        bit 12: +15VDC monitor
!                        bit 11: -7.5 VDC TLM/DC/DC conv.
!                        bit 10: +7.5 VDC TLM/DC/DC conv.
!                        bit  9: +10V VDC TLM/DC/DC conv.
!                        bit  8: +5 VDC monitor
!                        bit  7: filter wheel motor temperature
!                        bit  6: scan motor temperature
!                        bit  5: filter housing controller current
!                        bit  4: patch temperature
!                        bit  3: electronics temperature
!                        bit  2: base plate temperature     
!                        bit  1: radiator temperature
!                        bit  0: <zero fill>

      character*1   hrs_anwrd(16)
!                       word 1: radiator temperature (range 0-255)
!                       word 2: base plate temperature  (range 0-255)
!                       word 3: electronics temperature (range 0-255)
!                       word 4: patch temperature (range 0-255)
!                       word 5: filter housing controller (range 0-255)
!                       word 6: scan motor temperature (range 0-255)
!                       word 7: filter wheel motor temperature (range 0-255)
!                       word 8: +5 VDC monitor (range 0-255)
!                       word 9: +10 VDC TLM/DC/DC conv. (range 0-255)
!                       word 10: +7.5 VDC TLM/DC/DC conv. (range 0-255)
!                       word 11: -7.5 VDC TLM/DC/DC conv. (range 0-255)
!                       word 12: +15 VDC monitor (range 0-255)
!                       word 13: +15 VDC monitor (range 0-255)
!                       word 14: filter wheel motor current (range 0-255)
!                       word 15: scan motor current (range 0-255)
!                       word 16: patch controller power (range 0-255)
! FILLER
!-------
      integer*4   hrs_filler5(11)     !<zero fill>

      integer hrs_dt(chrs_size)
      equivalence(hrs_dt(1),hrs_scnlin)
      common/hrs1bdt/hrs_scnlin,hrs_scnlinyr,hrs_scnlindy,
     &               hrs_clockdrift,hrs_scnlintime,
     &               hrs_scnlinf,hrs_mjfrcnt,
     &               hrs_scnpos,
     &               hrs_scntyp,hrs_filler1,hrs_qualind,
     &               hrs_linqualflgs,
     &               hrs_chqualflg,hrs_mnfrqual,
     &               hrs_filler2,hrs_calcof,hrs_scalcof,
     &               hrs_yawsteering,hrs_totattcorr,hrs_navstat,
     &               hrs_attangtime,hrs_rollang,hrs_pitchang,
     &               hrs_yawang,hrs_scalti,hrs_ang,
     &               hrs_pos,hrs_moonang,hrs_filler3,
     &               hrs_elem,hrs_filler4,
     &               hrs_digitbupdatefg,hrs_digitbwrd,
     &               hrs_analogupdatefg,hrs_anwrd,hrs_filler5
!-------------------------------------------------------------------------------
!                         -- CVS MODIFICATION HISTORY --
!-------------------------------------------------------------------------------
!$Log: hrs1b.h,v $
!Revision 1.3  2005/04/15 13:39:48  frna
!Merge AAPP5Dev with main trunk
!
!Revision 1.2.2.9  2005/03/14 15:58:27  frna
!Reverse order of indices for hrs_h_iwtcnttmp, hrs_h_ictcnttmp and hrs_h_fwcnttmp
!
!Revision 1.2.2.8  2005/03/03 12:51:43  frna
!change byte to character*1 for portability
!
!Revision 1.2.2.7  2005/03/02 08:57:36  frna
!Updates for AAPP-5 from Tiphaine Labrot
!
!Revision 1.2.2.6  2004/11/26 11:39:42  frna
!Correction
!
!Revision 1.2.2.5  2004/11/22 09:01:14  frna
!Remove second header format, since latest NOAA release is backward-compatible
!
!Revision 1.2.2.4  2004/11/22 08:48:16  frna
!Correct number of digital A sensors
!
!Revision 1.2.2.3  2004/11/15 15:12:14  frna
!make comments more compact
!
!Revision 1.2.2.2  2004/11/10 09:43:49  frna
!Conform to NOAA 1b release dated Sept 27 2004

