!+
!$Id: hrs1bdt.h,v 1.3 2005/04/15 13:39:48 frna Exp $
! 
!DESCRIPTION 
!      include file for hirs 1b format handling
!
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
! Tiphaine Labrot
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
!
!  1.0     11/95     new release from NOAA available 30/10/95
!                    still problems in integer boudaries alignments
!                    and undefined scaling factors
!  2.0     20/05/98  modified to conform to NOAA release dated 
!  2.1     19/11/03  Update comments in data record for HIRS/4. N C Atkinson 
!  2.2     28/01/05  modified to conform to NOAA v2 format and comments T Labrot                
!  2.3     02/03/05  change byte to character*1 for portability. NCA
!                    

!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!###########################################################################
!
!               HIRS Level 1B Data Set   
!
!###########################################################################
!
!
!##############################################################################
!
!               DATA RECORD Sept 2004 - for NOAA-N/IJPS era
!               Formulated for HIRS/4 but also used for HIRS/3 
!
!##############################################################################
! SCAN LINE INFORMATION
!----------------------

      integer*2   hrs_scnlin0      !scan line number
!                                  !(cumulative, starting with 1)      
      integer*2   hrs_scnlinyr0    !scan line year
      integer*2   hrs_scnlindy0    !scan line day of year
      integer*2   hrs_clockdrift0  !satellite clock drift delta in milliseconds
      integer     hrs_scnlintime0  !scan line UTC time of day in milliseconds
      integer*2   hrs_scnlinf0     !scan line bit field
!                                  !bit 15: satellite direction
!                                  !        0=northbound
!                                  !        1=southbound
!                                  !bit 14: clock drift correction
!                                  !        0=not corrected
!                                  !        1=scan time corrected for clock drift
!                                  !bit 13-0: <zero fill>
      integer*2   hrs_mjfrcnt0     !major frame count for NOAA
!                                  !(cumulative, starting with 1)
!                                  !<zero fill> for Metop
      integer*2   hrs_scnpos0      !scan position number in 32 seconds cycle
!                                  !for NOAA; <zero fill> for Metop
      integer*2   hrs_scntyp0      !scan type code
!                                  ! 0 : Earth view
!                                  ! 1 : space view
!                                  ! 2 : cold black body view  
!                                  ! 3 : main black body view  
      integer*4   hrs_filler10(2)  !<zero fill>

! QUALITY INDICATORS
!-------------------

      integer     hrs_qualind0    !quality indicator bit field
!                 (if a bit is on(=1) then the statement is true.)
!                   bit 31: do not use scan for product generation
!                   bit 30: time sequence error detected with this scan 
!                   bit 29: data gap precedes this scan
!                   bit 28: insufficient data for calibration
!                   bit 27: Earth location data not available
!                   bit 26: first good time following a clock update
!                           (nominally 0)
!                   bit 25: instrument status changed with this scan
!                   bit 24-0: <zero fill>

      integer     hrs_linqualflgs0  !scan line quality flags
!           (if a bit is on(=1) then the statement is true.)
!
!           Additional calibration problem code
!                   bit 31: scan line was not calibrated because 
!                           of satellite maneuver (Metop)
!                           <zero fill> (NOAA)
!                   bit 30-24: <zero fill>
!           Time problem code :
!           (all bits off implies the scan time is as expected)
!                   bit 23: time field is bad but can probably be inferred
!                           from the previous good time
!                   bit 22: time field is bad and can't be inferred 
!                           from the previous good time
!                   bit 21: this record starts a sequence that is inconsistent  
!                           with previous times (i.e., there is a time
!                           discontinuity). This may be associated
!                           with a spacecraft clock update. 
!                           (see bit 26, Quality indicator bit field)
!                   bit 20: start of a sequence that apparently repeats scan
!                           times that have been previously accepted.
!                   bits 19-16: <zero fill>
!
!            Calibration problem code :
!            (these bits, along with those in "scan line quality flags",
!             complement the channel indicators; all bits set
!             to 0 indicates normal calibration)
!                   bit 15: scan was not calibrated
!                   bit 14: scan contains marginal calibration in some 
!                           of the IR channels
!                   bit 13: PRT quality test failed
!                   bit 12: PRT data marginal, some readings were rejected
!                   bit 11: scan contains some uncalibrated channels
!                   bit 10: scan indicates that the normal HIRS calibration
!                           sequence is disabled
!                   bit  9: space view scan is lunar contaminated
!                   bit  8: <zero fill> 
!
!             Earth location problem code
!             (all bits set to 0 implies the Earth location was normal)
!                   bit 7: not Earth located because of bad time; Earth
!                          location fields zero-filled
!                   bit 6: Earth location questionable because of questionable
!                          time code (see time problem flags above)
!                   bit 5: Earth location questionable. Marginal agreement 
!                          with reasonableness check.
!                   bit 4: Earth location questionable. Fails reasonableness
!                          check
!                   bit 3-2: <zero fill>
!                   bit 1: <zero fill> (NOAA)
!                          Not Earth located because of satellite
!                          in-plane maneuver (Metop)
!                   bit 0: <zero fill> (NOAA)
!                          Not Earth located because of satellite
!                          out-of-plane maneuver (Metop)
!
      integer*2   hrs_chqualflg0(hrs_mxchn) 
!                 Calibration quality flags for each channel
!                 order of the channels: 
!                 1, 17, 2, 3, 13, 4, 18, 11, 19, 7, 8,
!                 20, 10, 14, 6, 5, 15, 12, 16, 9.
!                 (all bits off implies a good calibration)
!                   bit 15-6: <zero fill>
!                   bit  5: calibration failed
!                   bit  4: calibration marginal
!                   bit  3: calibration slope value came from the HCF
!                   bit  2: BB views failed NEDC test
!                   bit  1: space views failed NEDC test
!                   bit  0: some QC tests could not be applied
!
      character*1  hrs_mnfrqual0(0:63)     
!                  Minor frame quality indicator bit fields for 
!                  minor frame 0 to 63 (in order)
!       (for bits 7 through 1,if bit is on(=1) then statement is true)    
!               bit  7 : 1 = this frame suspect due to a time error (NOAA)
!                        <zero fill> (Metop)
!               bit  6 : 1 = this frame contains data gap data fill (NOAA)
!                        <zero fill> (Metop)
!               bit  5 : 1 = this frame contains TIP dwell data fill (NOAA)
!                        <zero fill> (Metop)
!               bit  4 : 1 = data suspect due to PACS QC error (NOAA)
!                        <zero fill> (Metop)
!               bit  3 : 1 = mirror locked during this frame
!               bit  2 : 1 = mirror position error during this frame
!               bit  1 : 1 = mirror was moving during this frame
!               bit  0 : minor word odd parity bit
 
      integer*4  hrs_filler20(4)     ! <zero fill> 

! CALIBRATION COEFFICIENTS
!-------------------------

      integer     hrs_calcof0(3,hrs_mxchn)  
!              Primary calibration coefficients
!                 order of the channels:
!                 1, 17, 2, 3, 13, 4, 18, 11, 19, 7, 8,
!                 20, 10, 14, 6, 5, 15, 12, 16, 9.
!                    - 10^12*the second order term
!                    - 10^9*the first order term
!                    - 10^6*the zeroth order term

      integer     hrs_scalcof0(3,hrs_mxchn)  
!              Spare calibration coefficients
!                 order of the channels:
!                 1, 17, 2, 3, 13, 4, 18, 11, 19, 7, 8,
!                 20, 10, 14, 6, 5, 15, 12, 16, 9.
!                    - 10^12*the second order term
!                    - 10^9*the first order term
!                    - 10^6*the zeroth order term

     
! NAVIGATION
!-----------
      
      integer*2  hrs_yawsteering0(3) 
!                                  !(Metop: content defined below)
!                                  !<zero fill> (NOAA)
!                                  !word1: Computed roll angle (degrees)
!                                  !word2: Computed pitch angle (degrees)
!                                  !word3: Computed yaw angle (degrees)
      integer*2  hrs_totattcorr0(3) 
!                                  !Total applied attitude correction
!                                  !word1: Roll (degrees)
!                                  !word2: Pitch (degrees)
!                                  !word3: Yaw (degrees)                        
      integer    hrs_navstat0        
!                navigation status bit field
!          (bits20-18 are Metop specific and will contain zero fill for NOAA)
!          (bits11-0 are NOAA specific and will contain zero fill for Metop)
!               bits31-21: <zero fill>
!               bits20-19: yaw steering parameters usage indicator
!                          0=no yaw steering correction
!                          1=computed parameters from Metop data stream
!                          2=measured parameters from Metop data stream
!                          3=computed parameters from AELDS
!               bit 18   : Metop maneuver indicator 
!                          0=scan does not occur during a Metop in-plane 
!                            or out-of-plane maneuver
!                          1=scan, or some part of it, occurs during
!                            a maneuver
!               bit 17   : Earth location at the satellite subpoint is 
!                          accurate and reasonable, i.e., is within 
!                          tolerance defined by "nadir Earth location
!                          tolerance" in header
!                          0=out of tolerance
!                          1=in tolerance
!               bit 16    :Earth location corrected for Euler angles
!                          0=false
!                          1=true
!               bits15-12: Earth location indicator
!                          0=Earth location available
!                          1=user ephemerics files greater than 24hours old
!                          2=no Earth location available
!               bits11-08: spacecraft attitude control 
!                          0=operating in YGC or nominal mode
!                          1=operating in another mode
!                          2=attitude exceeds nominal tolerance
!                          3=both 1 and 2
!               bits07-04: attitude SMODE 
!                          0=nominal mode
!                          1=rate nulling mode
!                          2=YGC mode
!                          3=search mode
!                          4=coast mode
!               bits03-00: attitude PWTIP$AC
!                          0=nominal mode/no test
!                          1=yaw axis test in progress
!                          2=roll axis test in progress
!                          3=pitch axis test in progress
      integer    hrs_attangtime0  !time associated with Euler angles (seconds)
      integer*2  hrs_rollang0     !10^3xRoll angle in degrees
      integer*2  hrs_pitchang0    !10^3xPitch angle in degrees
      integer*2  hrs_yawang0      !10^3xYaw angle in degrees
      integer*2  hrs_scalti0      !10*spacecraft altitude above 
!                                 !reference ellipsoid (kilometers)
      integer*2  hrs_ang0(168)    !scan angles in degrees
!                     word1: 100*solar zenith angle, point 1
!                     word2: 100*satellite zenith angle, point 1
!                     word3: 100*local azimuth angle, point 1
!                     word4: 100*solar zenith angle, point 2
!                     word5: 100*satellite zenith angle, point 2
!                     word6: 100*local azimuth angle, point 2
!                     ......
!                     word166: 100*solar zenith angle, point 56
!                     word167: 100*satellite zenith angle, point 56
!                     word168: 100*local azimuth angle, point 56

      integer*4  hrs_pos0(112) !Earth location
!                     (north latitude and east longitude are positive)
!                     word1: 10^4*latitude in degrees, point 1
!                     word2: 10^4*longitude in degrees, point 1
!                     word3: 10^4*latitude in degrees, point 2
!                     word4: 10^4*longitude in degrees, point 2
!                     ......
!                     word112: 10^4*longitude in degrees, point 56

      integer*2  hrs_moonang0 !10^2*lunar angle in degrees
!                     (angle between moon and space view; only
!                     applicable for the space view scan, otherwise, 
!                     undefined; range 0 to 180 degrees) 
     
      integer*2  hrs_filler30(3)  ! <zero fill>


! HIRS DATA ELEMENTS
!-------------------

      integer*2   hrs_elem0(24,0:63)        !element data
!             for nel 0 to 63
!               word 1     bits 15-8 : scan encoder position
!                          bits  7-3 : electronic cal level indicator
!                          bits  2-0 : spare
!               word 2     bits 15-13: spare
!                          bits 12-7 : channel 1 period monitor
!                          bits  6-1 : element number
!                          bit     0 : filter sync designator
!               word 23    bits flag
!                          bit    15 : valid data flag
!                          bit    14 : odd bit parity
!                          bits 13-0 : spare
!               word 24    zero fill
!
!             for nel 0 to 55
!               word 3-22: radiometric channel no. 1, 17, 2, 3, 13, 4, 
!                          18, 11, 19, 7, 8, 20, 10, 14, 6, 5, 15, 12, 16, 9
!
!             for nel 56
!               word 3-22: positive calibration channel no. 1, 17, 2, 3, 13, 4, 
!                          18, 11, 19, 7, 8, 20, 10, 14, 6, 5, 15, 12, 16, 9
!
!             for nel 57
!               word 3-22: negative calibration channel no. 1, 17, 2, 3, 13, 4,
!                          18, 11, 19, 7, 8, 20, 10, 14, 6, 5, 15, 12, 16, 9
!
!             for nel 58
!               word 3-7:   internal warm target, temp sensor #1 (5 readings)
!               word 8-12:  internal warm target, temp sensor #2
!               word 13-17: internal warm target, temp sensor #3
!               word 18-22: internal warm target, temp sensor #4
!
!
!             for nel 59
!               word 3-7: internal cold target, temp sensor #1 (5 readings)
!               word 8-12: HIRS/4: Analog ground 3
!                          HIRS/3: internal cold target, temp sensor #2
!               word 13-17 HIRS/4: Internal Warm Target, temp Sensor #5
!                          HIRS/3: internal cold target, temp sensor #3
!               word 18-22 HIRS/4: Tertiary Telescope Temperature Sensor
!                          HIRS/3: internal cold target, temp sensor #4
!
!             for nel 60
!               word 3-7: internal filter wheel housing, temp sensor #1
!               word 8-12: internal filter wheel housing, temp sensor #2 
!               word 13-17: internal filter wheel housing, temp sensor #3
!               word 18-22: internal filter wheel housing, temp sensor #4
!
!             for nel 61
!               word 3-7: patch temp (5 readings)
!               word 8-12: 1st stage radiator 
!               word 13-17: filter wheel housing heater current
!               word 18-22: elec.cal.digital to analog converter 
!
!             for nel 62
!               word 3      scan mirror temperature
!               word 4      primary telescope temperature
!               word 5      secondary telescope temperature
!               word 6      HIRS baseplate temperature
!               word 7      HIRS electronics temperature
!               word 8      patch temperature full range
!               word 9      scan motor temperature
!               word 10     filter whell motor temperature
!               word 11     cooler housing temperature
!               word 12     patch control power
!               word 13     scan motor current
!               word 14     filter motor current
!               word 15     +15 VDC
!               word 16     -15 VDC
!               word 17     +7.5 VDC
!               word 18     -7.5 VDC
!               word 19     +10 VDC
!               word 20     + 5 VDC
!               word 21     analog ground
!               word 22     analog ground
!
!             for nel 63
!               word 3      line counter
!               word 4      first status word
!                                bits 15-8 : <zero fill>
!                                bit     7 : instrument on/off
!                                bit     6 : scan motor on/off
!                                bit     5 : filter wheel on/off
!                                bit     4 : electronics on/off
!                                bit     3 : cooler heat on/off
!                                bit     2 : internal warm target pos.
!                                bit     1 : internal cold target pos.
!                                bit     0 : space position
!               word 5      second status word
!                                bits 15-8 : <zero fill>
!                                bit     7 : nadir pos.
!                                bit     6 : calibration enable/disable
!                                bit     5 : cooler door release enable/disable
!                                bit     4 : cooler door open
!                                bit     3 : cooler door closed
!                                bit     2 : filter housing heat on/off
!                                bit     1 : patch temp. control on/off
!                                bit     0 : filter motor power high
!               word 6      Data verification binary code
!               word 7-22:  spare

        integer*2  hrs_filler40(6)    !<zero fill>

! DIGITAL B HOUSEKEEPING TELEMETRY
!---------------------------------
     
      integer*2   hrs_digitbupdatefg0 !digital "B" telemetry update flags
!         (if bit=0, associated telemetry item is up-to-date.
!          if bit=1, associated telemetry item was not updated during most recent
!                    telemetry cycle-possibly due to lost frame)

      integer*2   hrs_digitbwrd0  !digital "B" data from TIP
!                        bit 15: instrument power (0=off,1=on)
!                        bit 14: electronics power (0=off,1=on)
!                        bit 13: filter motor power (0=off,1=on)
!                        bit 12: scan motor power (0=off,1=on)
!                        bit 11: cooler heater (0=off,1=on)
!                        bit 10: filter housing heater (0=off,1=on)
!                        bit 9 : cooler door release (0=disabled,1=enabled)
!                        bit 8 : cooler window heater (0=on,1=off)
!                        bit 7 : go to nadir position (0=no,1=yes/initiated)
!                        bit 6 : calibration sequence (0=disabled,1=enabled)
!                        bit 5 : cooler door closed (0=yes,1=no)
!                        bit 4 : cooler door fully open (0=yes,1=no)
!                        bit 3 : filter motor power level (0=normal,1=yes)
!                        bit 2 : patch temperature controller (0=off,1=on)
!                        bit 1-0: <zero fill>

! ANALOG HOUSEKEEPING TELEMETRY
!------------------------------
      integer*4   hrs_analogupdatefg0 !analog telemetry update flags
!         (if bit=0, associated telemetry item is up-to-date.
!          if bit=1, associated telemetry item was not updated during most recent
!                    telemetry cycle-possibly due to lost frame)
!                        bits 31-17: <zero fill> 
!                        bit 16: patch controller power
!                        bit 15: scan motor current
!                        bit 14: filter wheel motor current
!                        bit 13: -15VDC monitor
!                        bit 12: +15VDC monitor
!                        bit 11: -7.5 VDC TLM/DC/DC conv.
!                        bit 10: +7.5 VDC TLM/DC/DC conv.
!                        bit  9: +10V VDC TLM/DC/DC conv.
!                        bit  8: +5 VDC monitor
!                        bit  7: filter wheel motor temperature
!                        bit  6: scan motor temperature
!                        bit  5: filter housing controller current
!                        bit  4: patch temperature
!                        bit  3: electronics temperature
!                        bit  2: base plate temperature     
!                        bit  1: radiator temperature
!                        bit  0: <zero fill>

      character*1   hrs_anwrd0(16)
!                       word 1: radiator temperature (range 0-255)
!                       word 2: base plate temperature  (range 0-255)
!                       word 3: electronics temperature (range 0-255)
!                       word 4: patch temperature (range 0-255)
!                       word 5: filter housing controller (range 0-255)
!                       word 6: scan motor temperature (range 0-255)
!                       word 7: filter wheel motor temperature (range 0-255)
!                       word 8: +5 VDC monitor (range 0-255)
!                       word 9: +10 VDC TLM/DC/DC conv. (range 0-255)
!                       word 10: +7.5 VDC TLM/DC/DC conv. (range 0-255)
!                       word 11: -7.5 VDC TLM/DC/DC conv. (range 0-255)
!                       word 12: +15 VDC monitor (range 0-255)
!                       word 13: +15 VDC monitor (range 0-255)
!                       word 14: filter wheel motor current (range 0-255)
!                       word 15: scan motor current (range 0-255)
!                       word 16: patch controller power (range 0-255)
      
! FILLER
!-------
      integer*4   hrs_filler50(11)     !<zero fill>
 
      integer hrs_dt(chrs_size)
      equivalence(hrs_dt(1),hrs_scnlin0)
      common/hrs1bdt/hrs_scnlin0,hrs_scnlinyr0,hrs_scnlindy0,
     &               hrs_clockdrift0,hrs_scnlintime0,
     &               hrs_scnlinf0,hrs_mjfrcnt0,
     &               hrs_scnpos0,hrs_scntyp0,hrs_filler10,
     &               hrs_qualind0,
     &               hrs_linqualflgs0,hrs_chqualflg0,
     &               hrs_mnfrqual0,hrs_filler20,
     &               hrs_calcof0,hrs_scalcof0,
     &               hrs_yawsteering0,hrs_totattcorr0,hrs_navstat0,
     &               hrs_attangtime0,hrs_rollang0,hrs_pitchang0,
     &               hrs_yawang0,hrs_scalti0,hrs_ang0,
     &               hrs_pos0,hrs_moonang0,hrs_filler30,
     &               hrs_elem0,hrs_filler40,
     &               hrs_digitbupdatefg0,hrs_digitbwrd0,
     &               hrs_analogupdatefg0,hrs_anwrd0,hrs_filler50
!-------------------------------------------------------------------------------
!                         -- CVS MODIFICATION HISTORY --
!-------------------------------------------------------------------------------
!$Log: hrs1bdt.h,v $
!Revision 1.3  2005/04/15 13:39:48  frna
!Merge AAPP5Dev with main trunk
!
!Revision 1.2.2.3  2005/03/03 12:51:43  frna
!change byte to character*1 for portability
!
!Revision 1.2.2.2  2005/03/02 08:57:36  frna
!Updates for AAPP-5 from Tiphaine Labrot
!
!Revision 1.2.2.1  2004/11/26 11:42:24  frna
!Update for NOAA 2004 formats
!
