!------------------------------------------------------------------------------
!NAME
!  hrs1belem_algoV4.h
!  
!  Include to process with program hirscl_algoV4
!
!HISTORY (CHANGES) 
!(version   date         comment           <author>)
! 1.0     January 05     creation          T. Labrot (MF)
! 1.1     March 05       4 byte variables first in common block. NCA.
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
!------------------------------------------------------------------------------
! Inputs (// header l1b)
      CHARACTER*3  hsiteid        !Data set creation site ID
      INTEGER*2    hhdrcnt        !Count of header records in this data set
      CHARACTER*8  hprblkid       !Processing block identification
      INTEGER*2    hsatid         !NOAA spacecraft identification code
      INTEGER      hstartdatajd   !start of data set Julian Day(00h,1jan50)
      INTEGER*2    hstatchrecnb   !record number of status change 
!                                  (if 0, none occured)
      INTEGER      hinststat1     !instrument status
      INTEGER*2    hscnlin        !count of scan lines in this data set
      
! Inputs (// data record l1b)      
      INTEGER*2    hscnlinyr(hrs_mxlin)     !scan line year
      INTEGER*2    hscnlindy(hrs_mxlin)     !scan line day of year
      INTEGER      hscnlintime(hrs_mxlin)   !scan line UTC time of day in milliseconds
      INTEGER*2    hscnlinf(hrs_mxlin)      !scan line bit field
      INTEGER*2    hang(168,hrs_mxlin)      !scan angles in degrees
      INTEGER*4    hpos(112,hrs_mxlin)      !Earth location
      INTEGER*2    helem(24,0:63,hrs_mxlin) !element data
      
! Inputs/outputs (// data record l1b)
      INTEGER      hqualind(hrs_mxlin)      !quality indicator bit field        

! Outputs (// header l1b)
      INTEGER*2    hcpidsyr           !year of last CPIDS Update
      INTEGER*2    hcpidsdy           !day of year of last CPIDS Update
      CHARACTER*6  hinstrtype         !instrument type'HIRS/3' or 'HIRS/4'
      INTEGER*2    hcallocsclin       !count of calibrated, Earth located scan 
!                                     !lines in this data set
      INTEGER*2    hautocalind        !auto calibration indicators bit field
      INTEGER*2    hsolarcalyr        !year of most recent solar channel calibration
      INTEGER*2    hsolarcaldy        !day of year of most recent solar channel calibration
      INTEGER      hcalinf(4,hrs_mxchn) !calibration information
      INTEGER      htempradcnv(3,hrs_mxirchn) !temperature-radiance conversion
      INTEGER*2    h20solfiltirrad    !albedo-radiance ch20 solar filtered irradiance in W/m^2
      INTEGER*2    h20equifiltwidth   !albedo-radiance ch 20 equivalent filter width

      INTEGER*4    hrdtemp(6)    !radiator temp.conversion coefficients
      INTEGER*4    hbptemp(6)    !base plate temp. conv. coef.
      INTEGER*4    heltemp(6)    !electronics temp. conv. coef.
      INTEGER*4    hpchtemp(6)   !patch temp. conv. coef.
      INTEGER*4    hfhcc(6)      !filter housing controller current conv. coef.
      INTEGER*4    hscnmtemp(6)  !scan motor temperature conv. coef.
      INTEGER*4    hfwmtemp(6)   !filter wheel motor temp. conv. coef.
      INTEGER*4    hp5v(6)       !+5 VDC monitor conv. coef.
      INTEGER*4    hp10v(6)      !+10 VDC TLM/DC/DC conv. conv. coef
      INTEGER*4    hp75v(6)      !+7.5 VDC TLM/DC/DC conv. conv. coef.
      INTEGER*4    hm75v(6)      !-7.5 VDC TLM/DC/DC conv. conv. coef.
      INTEGER*4    hp15v(6)      !+15 VDC monitor conv. coef.
      INTEGER*4    hm15v(6)      !-15 VDC monitor conv. coef
      INTEGER*4    hfwmcur(6)    !filter wheel motor current conv. coef.
      INTEGER*4    hscmcur(6)    !scan motor current conv. coef.
      INTEGER*4    hpchcpow(6)   !patch controller power conv. coef.   
      INTEGER*4    hiwtcnttmp(5,6)  !internal Warm Target, Temperature sensors     
      INTEGER*4    hictcnttmp(4,6) !internal Cold Target, Temperature sensors 
      INTEGER*4    htttcnttmp(6)  !tertiary Telescope, Temperature sensor
      INTEGER*4    hfwcnttmp(4,6) !filter wheel housing,temperature sensors
      INTEGER*4    hpatchexpcnttmp(6) !patch temperature (expanded Scale)
      INTEGER*4    hfsradcnttmp(6) !first stage radiator temperature sensor
      INTEGER*4    hscmircnttmp(6) !scan mirror temperature
      INTEGER*4    hpttcnttmp(6)   !primary telescope, temperature sensor
      INTEGER*4    hsttcnttmp(6)   !secondary telescope, temperature sensor
      INTEGER*4    hbpcnttmp(6)    !baseplate temperature
      INTEGER*4    helectcnttmp(6) !electronics temperature
      INTEGER*4    hpatchfcnttmp(6) !patch temperature full range
      INTEGER*4    hscmotcnttmp(6) !scan motor temperature
      INTEGER*4    hfwmcnttmp(6)   !filter wheel motor temperature
      INTEGER*4    hchsgcnttmp(6)  !cooler housing temperature
      INTEGER*4    hconversions(11) !conversion constants
      INTEGER*2    hmoonscnlin     !count of space view scans containing lunar-contaminated
      INTEGER*2    hmoonthresh     !lunar angle thresold in degrees
      INTEGER*4    havspcounts(hrs_mxchn)  !24-hour average space view counts for each channel

! outputs (//data record l1b)
      INTEGER*2    hscntyp(hrs_mxlin)      !scan type code
      INTEGER*4    hlinqualflgs(hrs_mxlin) !scan line quality flags
      INTEGER*2    hchqualflg(hrs_mxchn,hrs_mxlin)  !calibration quality flags for each channel
      INTEGER*4    hcalcof(3,hrs_mxchn,hrs_mxlin)  !Primary calibration coefficients
      INTEGER*4    hscalcof(3,hrs_mxchn,hrs_mxlin) !Spare calibration coefficients
      
      
      COMMON/hrs1belem_i/hstartdatajd,hinststat1,hscnlintime,
     & hpos,hprblkid,hhdrcnt,hsatid,hstatchrecnb,hscnlin,
     & hscnlinyr,hscnlindy,hscnlinf,hqualind,hang,helem,hsiteid
     
      COMMON/hrs1belem_o/hcalinf,htempradcnv,hrdtemp,    
     & hbptemp,heltemp,hpchtemp,hfhcc,hscnmtemp,hfwmtemp,
     & hp5v,hp10v,hp75v,hm75v,hp15v,hm15v,hfwmcur,hscmcur,
     & hpchcpow,hiwtcnttmp,hictcnttmp,htttcnttmp,hfwcnttmp,
     & hpatchexpcnttmp,hfsradcnttmp,hscmircnttmp,
     & hpttcnttmp,hsttcnttmp,hbpcnttmp,helectcnttmp,
     & hpatchfcnttmp,hscmotcnttmp,hfwmcnttmp,hchsgcnttmp,
     & hconversions,hmoonscnlin,hmoonthresh,
     & havspcounts,hlinqualflgs,hcalcof,hscalcof,
     & hcpidsyr,hcpidsdy,hinstrtype,hcallocsclin,      
     & hautocalind,hsolarcalyr,hsolarcaldy,
     & h20solfiltirrad,h20equifiltwidth,hscntyp,hchqualflg 
