!+
!NOM/NAME  %M% version %I% on %G% 
! 
!DESCRIPTION 
!      include file for hirs 1b format handling
!
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
! Tiphaine Labrot
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
!
!  1.0     11/95  new release from NOAA available 30/10/95
!                    still problems in integer boudaries alignments
!                    and undefined scaling factors
!  2.0     20/05/98  modified to conform to NOAA release dated 
!  2.1     19/11/03  include NOAA v2 header format, for use following launch
!                    of NOAA-N (and also applied to NOAA-KLM). N C Atkinson 
!  2.3     5/11/04   based on NOAA update of Sept 27 2004. N C Atkinson
!  2.4     22/11/04  remove second header format, since latest NOAA release
!                    is backward-compatible
!                    

!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!!###########################################################################
!
!               HIRS Level 1B Data Set  MAY 1998
!
!###########################################################################
!
!
!###########################################################################
!
!               HEADER RECORD Sept 2004 - for NOAA-N/IJPS era
!               Includes both HIRS/4 and HIRS/3
!
!###########################################################################
!
      character*3  hrs_h_siteid        !Data set creation site ID
      character*1  hrs_h_blank         !ASCII blank      
      integer*2    hrs_h_l1bversnb     !level 1b format version number
      integer*2    hrs_h_l1bversyr     !level 1b format version year
      integer*2    hrs_h_l1bversdy     !level 1b format version day of year
      integer*2    hrs_h_reclg         !<reserved for logical record length>
      integer*2    hrs_h_blksz         !<reserved for block size>
      integer*2    hrs_h_hdrcnt        !count of header records in this data set
      integer*2    hrs_h_filler0(3)
      character*42 hrs_h_dataname      !data set name
      character*8  hrs_h_prblkid       !processing block identification
      integer*2    hrs_h_satid         !NOAA spacecraft identification code
!                  2=NOAA-L; 4=NOAA-K; 6=NOAA-M; 7=NOAA-N; 8=NOAA-N';
!                  11=Metop-1(TBC); 12=Metop-2(TBC)
      integer*2    hrs_h_instid        !instrument ID
!                  301=NOAA-L; 302=NOAA-K; 303=NOAA-M; 304=NOAA-N'; 305=NOAA-N;
!                  306=Metop-2; 307=Metop-1
      integer*2    hrs_h_datatyp       !data type code
!                  1=LAC ; 2=GAC ; 3=HRPT ; 4=TIP ; 5=HIRS ; 6=MSU 
!                  7=SSU ; 8=DCS ; 9=SEM  ; 10=AMSU-A ; 11=AMSU-B ; 12=MHS
      integer*2    hrs_h_tipsrc        !TIP source code (zero fill for Metop)
!                  0=not applicable; 1=GAC embedded ; 2=stored TIP ;
!                  3=HRPT embedded ; 4=stored AIP
      integer      hrs_h_startdatajd   !start of data set Julian Day(00h,1jan50)
      integer*2    hrs_h_startdatayr   !start of data set year
      integer*2    hrs_h_startdatady   !start of data set day of the year
      integer      hrs_h_startdatatime !start of data set UTC time of day
!                                       in milliseconds
      integer      hrs_h_enddatajd     !end of data set Julian Day(00h,1jan50) 
      integer*2    hrs_h_enddatayr     !end of data set year
      integer*2    hrs_h_enddatady     !end of data set day of the year
      integer      hrs_h_enddatatime   !end of data set UTC time of day 
!                                       in milliseconds
      integer*2    hrs_h_cpidsyr    !year of last CPIDS Update
      integer*2    hrs_h_cpidsdy    !day of year of last CPIDS Update
      integer*2    hrs_h_fov1offset    !time offset for FOV 1 (ms)
      character*6  hrs_h_instrtype     !'HIRS/3' or 'HIRS/4'

! DATA SET QUALITY INDICATORS

      integer      hrs_h_inststat1     !instrument status for first record
      integer*2    hrs_h_filler2
      integer*2    hrs_h_statchrecnb   !record number of status change 
!                                       (if 0, none occured)
      integer      hrs_h_inststat2     !second instrument status 
!                                       (if previous word is 0, no change)
      integer*2    hrs_h_scnlin        !count of scan lines in this data set
      integer*2    hrs_h_callocsclin   !count of calibrated, earth located scan 
!                                       lines in this data set
      integer*2    hrs_h_misscnlin     !count of missing scan lines
      integer*2    hrs_h_datagaps      !count of data gaps in this data set
      integer*2    hrs_h_okdatafr      !count of data frames without frame sync 
!                                       word errors (zero fill for Metop)
      integer*2    hrs_h_pacsparityerr !count of PACS detected TIP parity errors
!                                       (zero fill for Metop)
      integer*2    hrs_h_auxsyncerrsum !sum of all auxiliary sync errors 
!                                       in the input data (zero fill Metop)
      integer*2    hrs_h_timeseqerr    !time sequence error
!                      0=none
!                      otherwise the record number of the first occurance
      integer*2    hrs_h_timeseqerrcode !time sequence error code
      integer*2    hrs_h_socclockupind  !socc clock update indicator
!                      0=none during this orbit
!                      otherwise the record number of the first occurance
      integer*2    hrs_h_locerrind      !earth location error indicator  
!                      0=none during this orbit
!                      otherwise the record number of the first occurance
      integer*2    hrs_h_locerrcode     !earth location error code
      integer*2    hrs_h_pacsstatfield  !PACS status bit field
!                  bit 15-3 : spare (zero fill)
!                  bit    2 : pseudo noise    0=normal data
!                                             1=P/N data
!                  bit    1 : tape direction  0=time decrementing
!                  bit    0 : data mode       0=test data
!                                             1=flight data
      integer*2    hrs_h_pacsdatasrc    !Pacs data source    0=unused
!                                                            1=Gilmore
!                                                            2=Wallops
!                                                            3=SOCC
      integer      hrs_h_filler3
      character*8  hrs_h_spare1         !spare <reserved for the ingester>
      character*8  hrs_h_spare2         !spare <reserved of the decommutation>
      integer*2    hrs_h_filler4(5)     !<zero fill>

! CALIBRATION

      integer*2    hrs_h_autocalind     !auto calibration indicators
!                  bit 0 : auto calibration override switch for HIRS/4
!                        0=normal calib, 1=disabled at some point during period
      integer*2    hrs_h_solarcalyr     !year of most recent solar ch. calib.
      integer*2    hrs_h_solarcaldy     !day of year of most recent solar
!                                        channel calibration
      integer      hrs_h_calinf(4,hrs_mxchn)    !calibration information
!                  order of the channels : 
!                  1, 17, 2, 3, 13, 4, 18, 11, 19, 7, 8,
!                  20, 10, 14, 6, 5, 15, 12, 16, 9.
!                      10^6*(mean calibration slope of channel nochannel)
!                      10^6*(stantard deviation of calibration slope)
!                      10^6*(b-sub-1 for channel nochannel)
!                      10^6*(stantard deviation of linear regression
!                          for b-sub-1 for channel nochannel)
      integer     hrs_h_filler5(2)     !<zero fill>

! TEMPERATURE-RADIANCE CONVERSION

      integer      hrs_h_tempradcnv(3,hrs_mxirchn)
!                  temperature-radiance conversion                       
!                  order of channels = 1,2,3,4,...,18,19
!                  for nochannel = 1 to 19 : 
!                      10^6 x (nochannel central wavenumber for ch1 to ch 12)
!                      10^5 x (nochannel central wavenumber for ch13 to ch 19)
!                      10^6 x (nochannel constant 1)
!                      10^6 x (nochannel constant 2)
      integer*2    hrs_h_20solfiltirrad
!               10^2 x (albedo-radiance ch20 solar filtered irradiance in W/m^2)
!               Note scale factor different from pre-2004 format
      integer*2    hrs_h_20equifiltwidth  
!               10^4 x (albedo-radiance ch 20 equivalent filter width)
!               Note scale factor different from pre-2004 format
      integer     hrs_h_filler6(2)     !<zero fill>

! NAVIGATION

      character*8  hrs_h_modelid

      integer*2    hrs_h_nadloctol      !10*(nadir earth location tolerance
!                                       in kilometers)
      integer*2    hrs_h_locbit         !earth location bit field
C!          bit 0 : constant attitude error correction (0 = not performed)
C!          bit 1 : reasonableness test (0 = inactive, 1=active)
C!          bit 2 : dynamic attitude error correction (0 = not performed)
      integer*2    hrs_h_filler7
      integer*2    hrs_h_rollerr        !10^3 x (constant roll attitude error
!                                                (dg)) 
      integer*2    hrs_h_pitcherr       !10^3 x (constant pitch attitude error
!                                                (dg))  
      integer*2    hrs_h_yawerr         !10^3 x (constant yaw attitude error
!                                                (dg))
      integer*2    hrs_h_epoyr          !epoch year for orbit vector
      integer*2    hrs_h_epody          !day of epoch year for orbit vector
      integer      hrs_h_epotime        !epoch UTC time of day in milliseconds
!                                        for orbit vector
      integer      hrs_h_smaxis         !10^5 x (semi-major axis in kilometers)
      integer      hrs_h_eccen          !10^8 x (eccentricity)
      integer      hrs_h_incli          !10^5 x (inclination in degrees) 
      integer      hrs_h_argper         !10^5 x (argument of perigee in degrees)
      integer      hrs_h_rascnod        !10^5 x (right ascension of the
!                                                ascending node in degrees) 
      integer      hrs_h_manom          !10^5 x (mean anomaly in degrees)
      integer      hrs_h_xpos           !10^5 x (position vector X component
!                                              in km)
      integer      hrs_h_ypos           !10^5 x (position vector Y component
!                                              in km) 
      integer      hrs_h_zpos           !10^5 x (position vector Z component
!                                              in km)
      integer      hrs_h_xvel           !10^8 x (velocity vector X-dot component
!                                                in kilometers/seconds)
      integer      hrs_h_yvel           !10^8 x (velocity vector Y-dot component !                                                in kilometers/seconds)
      integer      hrs_h_zvel           !10^8 x (velocity vector Z-dot component
!                                                in kilometers/seconds) 
      integer      hrs_h_earthsun       !10^6 x (earth/sun distance ratio)
      integer      hrs_h_filler8(4)     !<zero fill>

! ANALOG TELEMETRY CONVERSION
! Scaling factors 10**(2, 2, 3, 3, 3, 5)

      integer*4    hrs_h_rdtemp(6)     !radiator temp.conversion coefficients(6)
      integer*4    hrs_h_bptemp(6)     !base plate temp. conv. coef. (6)
      integer*4    hrs_h_eltemp(6)     !electronics temp. conv. coef. (6)
      integer*4    hrs_h_pchtemp(6)    !patch temp. conv. coef. (6)
      integer*4    hrs_h_fhcc(6)       !filter housing controller current conv.
!                                       coef. (6)
      integer*4    hrs_h_scnmtemp(6)   !scan motor temperature conv. coef.(6)
      integer*4    hrs_h_fwmtemp(6)    !filter wheel motor temp. conv. coef.(6) 
      integer*4    hrs_h_p5v(6)        !+5 VDC monitor conv. coef.(6)
      integer*4    hrs_h_p10v(6)       !+10 VDC TLM/DC/DC conv. conv. coef.(6) 
      integer*4    hrs_h_p75v(6)       !+7.5 VDC TLM/DC/DC conv. conv. coef.(6)
      integer*4    hrs_h_m75v(6)       !-7.5 VDC TLM/DC/DC conv. conv. coef.(6)
      integer*4    hrs_h_p15v(6)       !+15 VDC monitor conv. coef.(6)
      integer*4    hrs_h_m15v(6)       !-15 VDC monitor conv. coef.(6)
      integer*4    hrs_h_fwmcur(6)     !filter wheel motor current conv.
!                                       coef.(5)
      integer*4    hrs_h_scmcur(6)     !scan motor current conv. coef.(6)
      integer*4    hrs_h_pchcpow(6)    !patch controller power conv. coef.(6)

! DIGITAL A TELEMETRY CONVERSION
!-------------------------------
! Coefficients are given to get temperatures in degrees Celcius
! Scaling factors for coefficients:10**(6, 9, 14, 17, 21, 25)
      
      integer*4    hrs_h_iwtcnttmp(6,5) !internal Warm Target, Temperature sensors
!                                       !4 sensors for NOAAKLM
!                                       !5 sensors for NOAANN'and Metop
      integer*4    hrs_h_ictcnttmp(6,4) !internal Cold Target, Temperature sensors 
!                                       !4 sensors for NOAAKLM
!                                       !1 sensor  for NOAANN' and Metop      
      integer*4    hrs_h_tttcnttmp(6)   !tertiary Telescope, Temperature sensor
!                                       !0 sensor for NOAAKLM
!                                       !1 sensor  for NOAANN'and Metop
      integer*4    hrs_h_fwcnttmp(6,4)  !filter wheel housing,temperature sensors
!                                       !4 sensors.
      integer*4    hrs_h_patchexpcnttmp(6) !patch temperature (expanded Scale)
      integer*4    hrs_h_fsradcnttmp(6) !first stage radiator temperature sensor
      integer*4    hrs_h_scmircnttmp(6) !scan mirror temperature
      integer*4    hrs_h_pttcnttmp(6)   !primary telescope, temperature sensor
      integer*4    hrs_h_sttcnttmp(6)   !secondary telescope, temperature sensor
      integer*4    hrs_h_bpcnttmp(6)    !baseplate temperature
      integer*4    hrs_h_electcnttmp(6) !electronics temperature
      integer*4    hrs_h_patchfcnttmp(6)!patch temperature full range
      integer*4    hrs_h_scmotcnttmp(6) !scan motor temperature
      integer*4    hrs_h_fwmcnttmp(6)   !filter wheel motor temperature
      integer*4    hrs_h_chsgcnttmp(6)  !cooler housing temperature

      integer*4    hrs_h_conversions(11) !conversion constants
!                                        !word1:10^9 x filter wheel housing heater current 
!                                        !word2:10 x Eelectronic calibration digital 
!                                        !           to analog converter  
!                                        !word3: 10^17 x patch control power
!                                        !word4: 10^9 x scan motor current
!                                        !word5: 10^9 x filter motor current
!                                        !word6: 10^8 x +15 VDC
!                                        !word7: 10^8 x -15 VDC
!                                        !word8: 10^8 x +7.5 VDC
!                                        !word9: 10^8 x -7.5 VDC
!                                        !word10: 10^8 x +10 VDC
!                                        !word11: 10^8 x +5 VDC

! LUNAR CONTAMINATION

      INTEGER*2 hrs_h_moonscnlin     !count of scans with moon in space view
                                      ! (-1 = detection algorithm turned off)
      INTEGER*2 hrs_h_moonthresh     !100 x lunar angle threshold in degrees 

! 24-HOUR AVERAGE SPACE VIEW COUNTS

      integer*4   hrs_h_avspcounts(hrs_mxchn)

! METOP MANOEUVRES IDENTIFICATION

      INTEGER*2   hrs_h_startmanyr      !4 digit year
      INTEGER*2   hrs_h_startmandy      !Day of year
      INTEGER*4   hrs_h_startmantime    !Time of day in milliseconds
      INTEGER*2   hrs_h_endmanyr
      INTEGER*2   hrs_h_endmandy
      INTEGER*4   hrs_h_endmantime
      INTEGER*4   hrs_h_deltav(3)       !Change in velocity
      INTEGER*4   hrs_h_mass(2)         !Spacecraft mass before and after

! FILLER
      integer     hrs_h_filler9(651)     !<zero fill>

      integer hrs_hd(chrs_size)
      equivalence(hrs_hd(1),hrs_h_siteid)
      common/hrs1bhd/hrs_h_siteid,hrs_h_blank,
     & hrs_h_l1bversnb,hrs_h_l1bversyr,hrs_h_l1bversdy,
     & hrs_h_reclg,hrs_h_blksz,hrs_h_hdrcnt,hrs_h_filler0,
     & hrs_h_dataname,
     & hrs_h_prblkid,hrs_h_satid,hrs_h_instid,hrs_h_datatyp,
     & hrs_h_tipsrc,
     & hrs_h_startdatajd,hrs_h_startdatayr,hrs_h_startdatady,
     & hrs_h_startdatatime,hrs_h_enddatajd,hrs_h_enddatayr,
     & hrs_h_enddatady,
     & hrs_h_enddatatime,hrs_h_cpidsyr,hrs_h_cpidsdy,
     & hrs_h_fov1offset,hrs_h_instrtype,hrs_h_inststat1,hrs_h_filler2,
     & hrs_h_statchrecnb,hrs_h_inststat2,
     & hrs_h_scnlin,hrs_h_callocsclin,hrs_h_misscnlin,
     & hrs_h_datagaps,
     & hrs_h_okdatafr,hrs_h_pacsparityerr,hrs_h_auxsyncerrsum,
     & hrs_h_timeseqerr,hrs_h_timeseqerrcode,hrs_h_socclockupind,
     & hrs_h_locerrind,
     & hrs_h_locerrcode,
     & hrs_h_pacsstatfield,hrs_h_pacsdatasrc,hrs_h_filler3,
     & hrs_h_spare1,hrs_h_spare2,
     & hrs_h_filler4,hrs_h_autocalind,hrs_h_solarcalyr,
     & hrs_h_solarcaldy,
     & hrs_h_calinf,hrs_h_filler5,hrs_h_tempradcnv,
     & hrs_h_20solfiltirrad,hrs_h_20equifiltwidth,hrs_h_filler6,
     & hrs_h_modelid,hrs_h_nadloctol,hrs_h_locbit,hrs_h_filler7,
     & hrs_h_rollerr,hrs_h_pitcherr,hrs_h_yawerr,hrs_h_epoyr,
     & hrs_h_epody,
     & hrs_h_epotime,hrs_h_smaxis,hrs_h_eccen,hrs_h_incli,
     & hrs_h_argper,
     & hrs_h_rascnod,hrs_h_manom,hrs_h_xpos,hrs_h_ypos,hrs_h_zpos,
     & hrs_h_xvel,
     & hrs_h_yvel,hrs_h_zvel,hrs_h_earthsun,hrs_h_filler8,
     & hrs_h_rdtemp,
     & hrs_h_bptemp,hrs_h_eltemp,hrs_h_pchtemp,hrs_h_fhcc,
     & hrs_h_scnmtemp,hrs_h_fwmtemp,hrs_h_p5v,hrs_h_p10v,
     & hrs_h_p75v,hrs_h_m75v,hrs_h_p15v,hrs_h_m15v,
     & hrs_h_fwmcur,hrs_h_scmcur,hrs_h_pchcpow,
     & hrs_h_iwtcnttmp,hrs_h_ictcnttmp,hrs_h_tttcnttmp,
     & hrs_h_fwcnttmp,hrs_h_patchexpcnttmp,hrs_h_fsradcnttmp,
     & hrs_h_scmircnttmp,hrs_h_pttcnttmp,hrs_h_sttcnttmp,
     & hrs_h_bpcnttmp,hrs_h_electcnttmp,hrs_h_patchfcnttmp,
     & hrs_h_scmotcnttmp,hrs_h_fwmcnttmp,hrs_h_chsgcnttmp,
     & hrs_h_conversions,hrs_h_moonscnlin,
     & hrs_h_moonthresh,hrs_h_avspcounts,hrs_h_startmanyr,
     & hrs_h_startmandy,hrs_h_startmantime,hrs_h_endmanyr,
     & hrs_h_endmandy,hrs_h_endmantime,hrs_h_deltav,hrs_h_mass,
     & hrs_h_filler9
