!+ Level 1D Data Set (ATOVS on HIRS grid)
!
! Name:  hrs1d.h
!
! Purpose:
!   'INCLUDE' file for common /hrs1dhd/   (HIRS grid level 1D header record)
!                  and common /hrs1ddt/   (HIRS grid level 1D data record)
!
! Some parts of the 1D format are intended to be flexible, allowing
! different centres to customise the format for their own requirements.
! These parts are marked as '[user-defined ...]' below.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0     27/11/96  Original code. Conforms to 1D format vn1.8 (R.J.Renshaw)
!  1.1     12/12/96  Updated to 1D format vn1.9
!  1.2     17/01/97  Updated to 1D format vn1.91
!  1.3     19/11/97  AVHRR slots finalised, vn1.92
!  1.4     10/03/98  updated to vn1.93 (instrument temperatures added)
!  1.5     20/05/98  include extra pre-processing variables
!  1.6     07/09/98  bits changed in hrs1d_prepro(1,*)
!  1.7     30/09/98  Size of hrs_1drecl, hrs1d_h_spare3, hrs1d_filler corrected
!  1.8     07/04/04  New flag in hrs1d_prepro(1,*)
!  1.9     24/01/05  Add WMO Sat Id
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
      integer       hrs_1dvnnb     ! HIRS level 1D format version number
         parameter (hrs_1dvnnb = 195)!  - this code is set up for vn no 1.95
      integer       hrs_1dvnyr     ! format version year for 1D HIRS
         parameter (hrs_1dvnyr = 1998)
      integer       hrs_1dvndy     ! format version day for 1D HIRS
         parameter (hrs_1dvndy = 250)  ! = 7th September
      integer       hrs_1drecl     ! HIRS level 1D dataset record length
         parameter (hrs_1drecl = 3968*4)!  - number of bytes 
!
!###########################################################################
!
!               HEADER RECORD
!
!###########################################################################
!
      character*3  hrs1d_h_site       !1D dataset creation site ID
      character*1  hrs1d_h_cfill1     !filler
      character*3  hrs1d_h_1bsite     !creation site for original 1B data
      character*1  hrs1d_h_cfill2     !filler
      integer*4    hrs1d_h_versnb     !level 1d format version number
      integer*4    hrs1d_h_versyr     !level 1d format version year
      integer*4    hrs1d_h_versdy     !level 1d format version day of year
      integer*4    hrs1d_h_hdrcnt     !count of header records in this data set
      integer*4    hrs1d_h_satid      !satellite id (e.g. 14 for NOAA-14)
      integer*4    hrs1d_h_grid       !code for instrument grid
!                                 (5=HIRS; 6=MSU; 10=AMSU-A; 11=AMSU-B)
      integer*4    hrs1d_h_satht      !nominal satellite altitude, km*10
      integer*4    hrs1d_h_period     !nominal orbit period (seconds)
      integer*4    hrs1d_h_startorbit !orbit number (at start of dataset)
      integer*4    hrs1d_h_startdatayr!start of data set year
      integer*4    hrs1d_h_startdatady!start of data set day of the year
      integer*4    hrs1d_h_startdatatime !start of data set UTC time of day (ms)
      integer*4    hrs1d_h_endorbit   !orbit number (at end of dataset)
      integer*4    hrs1d_h_enddatayr  !end of data set year
      integer*4    hrs1d_h_enddatady  !end of data set day of the year
      integer*4    hrs1d_h_enddatatime!end of data set UTC time of day (ms)
      integer*4    hrs1d_h_scnlin     !count of scan lines in this data set
      integer*4    hrs1d_h_misscnlin  !count of missing scan lines
      integer*4    hrs1d_h_atovppvn   !ATOVPP version number (test vns = 9000+)
      integer*4    hrs1d_h_instruments!instruments present
!                   (bit0=HIRS, bit1=MSU, bit3=AMSU-A, bit4=AMSU-B, bit5=AVHRR)
      integer*4    hrs1d_h_wmosatid   !WMO satellite id (3 for METOP-1)
      integer*4    hrs1d_h_spare1     !spare
!
! HIRS temperature-radiance conversion coefficients  (Planck function)
      integer*4    hrs1d_h_hrsPlanck(3,Hnchan-1)
!                  order of channels = HIRS 1,2,3,4,...,18,19
!                  (1,*) = 10^6 x (central wavenumber)
!  (scaling factor for wavenumber changes to 10^5 for channels 13-19 inclusive)
!                  (2,*) = 10^6 x (constant 1)
!                  (3,*) = 10^6 x (constant 2)
      integer*4    hrs1d_h_20solfiltirrad
!                  10^6 x (albedo-radiance ch20 solar filtered irradiance W/m2)
      integer*4    hrs1d_h_20equifiltwidth
!                  10^6 x (albedo-radiance ch 20 equivalent filter width cm^-1)
      integer*4    hrs1d_h_spare2  !spare
! AMSU(-A&B) temperature-radiance conversion coefficients  (Planck function)
      integer*4    hrs1d_h_amsPlanck(3,Anchan+Bnchan)
!                  order of channels = AMSU-A 1-15, AMSU-B 1-5
!                  (1,*) = 10^6 x (central wavenumber)
!                  (2,*) = 10^6 x (constant 1)
!                  (3,*) = 10^6 x (constant 2)
!
! Pre-processing information
![user-defined:
      integer*4    hrs1d_h_dT  ! size of data batches processed within ATOVPP
!                                (seconds   - see ATOVPP documentation)
      integer*4    hrs1d_h_mapBTmode! mapping method used for BTs
!      (1=nearest neighbour, 2=bilinear interpolation, 3=spatial averaging)
      integer*4    hrs1d_h_Hcorrvn    ! vn no HIRS limb/emiss correction
!                                         (=0 if data not corrected)
      integer*4    hrs1d_h_Hcloudvn   ! vn no HIRS cloud detection/correction
!                                         (=0 if not done)
      integer*4    hrs1d_h_Acorrvn    ! vn no AMSU-A limb/emiss correction
!                                         (=0 if data not corrected)
      integer*4    hrs1d_h_Apcpvn     ! vn no AMSU-A precip/scatter detection
!                                         (=0 if not done)
      integer*4    hrs1d_h_Aantenna   ! vn no, AMSU-A antenna corrections
                                      ! (from ATOVIN)
      integer*4    hrs1d_h_Bcorrvn    ! vn no AMSU-B limb/emiss correction
!                                         (=0 if data not corrected)
      integer*4    hrs1d_h_Bpcpvn     ! vn no AMSU-B precip/scatter detection
!                                         (=0 if not done)
      integer*4    hrs1d_h_Bantenna   ! vn no, AMSU-B antenna corrections
                                      ! (from ATOVIN)
      integer*4    hrs1d_h_Ascat !threshold value for AMSU-A scattering index
      integer*4    hrs1d_h_ACrosby !threshold value for AMSU-A logistic precip
      integer*4    hrs1d_h_AB89 !threshold value for AMSU-A/B 89GHz differences
      integer*4    hrs1d_h_spare3(3809)  !spare
! ... ]
!
      integer*4 hrs1d_hd(hrs_1drecl/4)
      equivalence(hrs1d_hd(1),hrs1d_h_site)
      common /hrs1dhd/ hrs1d_h_site, hrs1d_h_cfill1, hrs1d_h_1bsite,
     & hrs1d_h_cfill2, hrs1d_h_versnb, hrs1d_h_versyr, hrs1d_h_versdy,
     & hrs1d_h_hdrcnt, hrs1d_h_satid, hrs1d_h_grid, hrs1d_h_satht,
     & hrs1d_h_period, hrs1d_h_startorbit, hrs1d_h_startdatayr,
     & hrs1d_h_startdatady, hrs1d_h_startdatatime, hrs1d_h_endorbit,
     & hrs1d_h_enddatayr, hrs1d_h_enddatady, hrs1d_h_enddatatime,
     & hrs1d_h_scnlin, hrs1d_h_misscnlin, hrs1d_h_atovppvn,
     & hrs1d_h_instruments, hrs1d_h_wmosatid, hrs1d_h_spare1, 
     & hrs1d_h_hrsplanck,
     & hrs1d_h_20solfiltirrad, hrs1d_h_20equifiltwidth, hrs1d_h_spare2,
     & hrs1d_h_amsplanck, hrs1d_h_dT, hrs1d_h_mapBTmode,
     & hrs1d_h_Hcorrvn, hrs1d_h_Hcloudvn, hrs1d_h_Acorrvn,
     & hrs1d_h_Apcpvn, hrs1d_h_Aantenna, hrs1d_h_Bcorrvn,
     & hrs1d_h_Bpcpvn, hrs1d_h_Bantenna, hrs1d_h_Ascat,
     & hrs1d_h_ACrosby, hrs1d_h_AB89, hrs1d_h_spare3
!
!##############################################################################
!
!               DATA RECORD
!
!##############################################################################
!
      integer*4   hrs1d_scnlin     !scan line number
      integer*4   hrs1d_scnlinyr   !scan line year
      integer*4   hrs1d_scnlindy   !scan line day of year
      integer*4   hrs1d_scnlintime !scan line UTC time of day in milliseconds
      integer*4   hrs1d_qualind    !quality indicator bit field
!
!                 In all of the following, if the bit is on (= is set to 1)
!                 then the statement is true. Otherwise it is false.
!
!                 general
!
!                 bit 31: do not use scan for product generation
!                 bit 30: time sequence error detected with this scan
!                 bit 29: data gap precedes this scan
!                 bit 28: no HIRS calibration
!                 bit 27: no earth location
!                 bit 26: first good time following a clock update
!                 bit 25: instrument status changed with this scan
!                 bit 24-0: spare  <zero fill>
!
      integer*4   hrs1d_scanqual   !scan line quality flags
!                 time problem code
!                 (all bits off implies the scan time is as expected)
!                 bit 31-24: spare   <zero fill>
!                 bit 23: time field is bad but can probably be inferred
!                         from the previous good time
!                 bit 22: time field is bad and can't be inferred
!                         from the previous good time
!                 bit 21: this record starts a sequence that is inconsistent
!                         with previous times (i.e., there is a time
!                         discontinuity). This may or may not be associated
!                         with a spacecraft clock update. (see bit 26 above)
!                 bit 20: start of a sequence that apparently repeats scan
!                         times that have been previously accepted.
!                 bits 19-16: spare
!
!                 calibration problem code
!
!                 (all bits set to 0 indicates normal calibration)
!                  bit 15: scan line was not calibrated because of bad time
!                  bit 14: scan line was calibrated using fewer than the
!                          preferred number of scan lines because of
!                          proximity to start or end of data set or
!                          to a data gap.
!                  bit 13: scan line was not calibrated because of bad or
!                          insufficient PRT data
!                  bit 12: scan line was calibrated but with marginal PRT data
!                  bit 11: some uncalibrated channels of this scan.
!                          See channel indicators.
!                  bit 10: uncalibrated due to instrument mode
!                  bits 9 and 8: spare
!
!                  Earth location problem code
!
!                  (all bits set to 0 implies the Earth location was normal)
!                  bit 7: not Earth located because of bad time
!                  bit 6: Earth location questionable because of questionable
!                         time code (see time problem flags above)
!                  bit 5: Earth location questionable. Only marginal agreement
!                         with reasonableness check.
!                  bit 4: Earth location questionable. Fails reasonableness
!                         check
!                  bit 3-0: spare
!
      integer*4   hrs1d_instrtemp         !HIRS baseplate temperature (K*100)
      integer*4   hrs1d_instrtempa1       !AMSU-A1 RF shelf temp (K*100)
      integer*4   hrs1d_instrtempa2       !AMSU-A2 RF shelf temp (K*100)
      integer*4   hrs1d_instrtempb        !AMSU-B mixer ch 18-20 temp (K*100)
                                          !(all from the nearest scanlines)

      integer*4   hrs1d_spare1(2)         !spare
!
      integer*4   hrs1d_latlon(2,Hnfovs)  !lat/lon in degrees for Hnfovs
!                     first   : 10^4 x (latitude)
!                     second  : 10^4 x (longitude)
      integer*4   hrs1d_surface(2,Hnfovs) !height/type for Hnfovs
!                     first   : surface height (metres)
!                     second  : surface type (0=sea, 1=mixed, 2=land)
      integer*4   hrs1d_angles(4,Hnfovs) !scan angles for Hnfovs
!                     first:    10^2 x (local zenith angle)
!                     second:   10^2 x (local azimuth angle)
!                     third:    10^2 x (solar zenith angle)
!                     fourth:   10^2 x (solar azimuth angle)
      integer*4   hrs1d_scalti !sat altitude above reference ellipsoid, km*10
      integer*4   hrs1d_spare2(2)             !spare
!
      integer*4   hrs1d_btemps(Hnchan+Anchan+Bnchan,Hnfovs)
!                 10^2 x scene brightness temperature (K) for channels
!                 HIRS 1-20, AMSU-A 1-15, AMSU-B 1-5
!                 10^2 x radiance Wm-2sr-1(cm-1)-1 for channel HIRS 20
!
![user-defined:
      integer*4   hrs1d_avhrr(13,Hnfovs)
!                 AVHRR parameters
!     fraction of clear AVHRR pixels in HIRS fov
!     mean albedo (%) or brightness temperature (K), AVHRR channels 1-5
!     mean clear albedo (%) or brightness temperature (K), channels 1-5
!     std dev chan 4 (K), all pixels
!     std dev chan 4 (K), clear pixels
! ... ]
!
! Pre-processing output
      integer*4   hrs1d_hirsqual(Hnfovs)
!                 HIRS quality control word for each field of view:
!                  (all bits off implies acceptable data)
![user-defined:
!                  bit 31: spare   <zero fill>
!                  bit 30: set if secondary calibration used (..see calflg)
!                  bit 29-22: spare   <zero fill>
!                  bit 21: HIRS cloud test (TBD)
! ... ]
!                  bit 20-1: bit n set to 1 if brightness temperature in
!                            HIRS channel n is physically unreasonable or has
!                            not been calculated due to calibration  
!                            problems.
!                  bit 0:    bad or missing data (in any or all channels)
!
![user-defined:
      integer*4   hrs1d_prepro(7,Hnfovs) ! values from pre-processing
!                  (1,*):   pre-processing quality control word with flags
!                            (all bits off implies acceptable data)
!                  bit 31: set if AMSU-A surface types not all the same
!                  bit 30: set if AMSU-A used secondary calibration
!                  bit 29: set if AMSU-B used secondary calibration
!                  bit 28: set if AMSU-B data missing
!                  bit 27: flag for cloud cost set for any mapped AMSU-A
!                  bit 26: scattering flag set for any mapped AMSU-A
!                          (only set over the sea)
!                  bit 25: logistic precipitation probability test
!                          calculated from AMSU-A data mapped to HIRS grid
!                  bit 24: Grody light rainfall test calculated on HIRS grid
!                  bit 23: mismatch between AMSU-A/B 89GHz values (any AMSU-A)
!                  bit 22: mismatch between surface type from topography
!                          dataset and from pre-processing (any AMSU-A)
!                  bit 21: fewer AMSU mapping co-locations than expected
!                  bit 20-4: spare   <zero fill>
!                  bit 3: set when AVHRR channel 3 is albedo not brightness temp
!                  bit 2: flag for cloud cost (recalculated on HIRS grid)
!                  bit 1: flag for scattering index (recalculated on HIRS grid)
!                  bit 0: set if AMSU-A & B data missing
!
!                  (2,*)    estimated surface type from the pre-processing:
!            1 = Bare young ice (i.e. new ice, no snow)
!            2 = Dry land (i.e. dry with or without significant vegetation)
!            3 = Dry snow (i.e. snow with water less than 2%, over land)
!            4 = Multi-year ice (i.e. old ice with snow [assumed dry] cover)
!            5 = Sea (i.e. open water, no islands, ice-free, WS=0 to 14m/s)
!            6 = Wet forest (i.e. established forest with wet canopy)
!            7 = Wet land (i.e. non-forested land with a wet surface)
!            8 = Wet snow (i.e. snow with water content > 2%, over land or ice)
!            9 = Desert
!
!                  (3,*) cost fn from PPASURF surface identification (*100)
!                  (4,*) scattering index (*100)
!                  (5,*) logistic precipitation probability (*100)
!                  (6,*)   spare
!                  (7,*)   spare
!
      integer*4   hrs1d_filler(89)          !filler
! ... ]
!
      integer*4 hrs1d_dt(hrs_1drecl/4)
      equivalence(hrs1d_dt(1),hrs1d_scnlin)
      common /hrs1ddt/ hrs1d_scnlin, hrs1d_scnlinyr, hrs1d_scnlindy,
     & hrs1d_scnlintime, hrs1d_qualind, hrs1d_scanqual,
     & hrs1d_instrtemp, hrs1d_instrtempa1, hrs1d_instrtempa2,
     & hrs1d_instrtempb, hrs1d_spare1, hrs1d_latlon, hrs1d_surface,
     & hrs1d_angles, hrs1d_scalti, hrs1d_spare2, hrs1d_btemps,
     & hrs1d_avhrr, hrs1d_hirsqual, hrs1d_prepro, hrs1d_filler
!-
