!+ IASI Level 1C Data Set
!
!$Id: iasi1c.h,v 1.1.2.3 2005/11/23 16:32:25 frna Exp $
!
! Purpose:
!   'INCLUDE' file for common /iasi1chd/   (IASI level 1C header record)
!                  and common /iasi1cdt/   (IASI level 1C data record)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0      02/11/05  Original code
!  1.1      22/02/06  Move iasi1c_h_wmosatid. NCA
!  2.0      31/07/09  Version 2 - add new variables from EPS format v8.0
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
!
! Declarations:
!
      integer       iasi_1cvnnb     ! IASI level 1C format version number
         parameter (iasi_1cvnnb = 2)!  - this code is set up for vn no 1
      integer       iasi_1cvnyr     ! format version year for 1C IASI
         parameter (iasi_1cvnyr = 2009)
      integer       iasi_1cvndy     ! format version day for 1C IASI
         parameter (iasi_1cvndy = 212)
      integer       iasi_1cnchan    ! Number of channel slots present in 1c
         parameter (iasi_1cnchan = 8700)
      integer       iasi_1cnfov    ! Number of FOVs present in 1c
         parameter (iasi_1cnfov = 30)
      integer       iasi_1cndet    ! Number of Detectors per FOV
         parameter (iasi_1cndet = 4)
      integer       array_size_iasi1c     ! Length of array
!         parameter (array_size_iasi1c = 545561) !  Length of array (v1)
         parameter (array_size_iasi1c = 545981) !  Length of array (v2)
      integer       iasi_1crecl     ! IASI level 1C file record length
         parameter (iasi_1crecl = array_size_iasi1c*4)!  - number of bytes
!
!###########################################################################
!
!               HEADER RECORD
!
!###########################################################################
!
      character*3  iasi1c_h_site       !dataset creation site ID
      character*1  iasi1c_h_cfill1     !filler
      character*3  iasi1c_h_1bsite     !creation site for original 1B data
      character*1  iasi1c_h_cfill2     !filler
      integer*4    iasi1c_h_versnb     !level 1c format version number
      integer*4    iasi1c_h_versyr     !level 1c format version year
      integer*4    iasi1c_h_versdy     !level 1c format version day of year
      integer*4    iasi1c_h_hdrcnt     !count of header records in this data set
      integer*4    iasi1c_h_satid      !satellite id in series (1 for METOP-1)
      integer*4    iasi1c_h_instrument !instrument code
!                       (5=HIRS; 6=MSU; 10=AMSU-A; 11=AMSU-B; 12=MHS; 13=IASI)
      integer*4    iasi1c_h_satht      !nominal satellite altitude, km*10
      integer*4    iasi1c_h_period     !nominal orbit period (seconds)
      integer*4    iasi1c_h_startorbit !orbit number (at start of file)
      integer*4    iasi1c_h_startdatayr!start of data set year
      integer*4    iasi1c_h_startdatady!start of data set day of the year
      integer*4    iasi1c_h_startdatatime !start of data set UTC time of day (ms)
      integer*4    iasi1c_h_endorbit   !orbit number (at end of file)
      integer*4    iasi1c_h_enddatayr  !end of data set year
      integer*4    iasi1c_h_enddatady  !end of data set day of the year
      integer*4    iasi1c_h_enddatatime!end of data set UTC time of day (ms)
      integer*4    iasi1c_h_scnlin     !count of scan lines in this data set
      integer*4    iasi1c_h_misscnlin  !count of missing scan lines
      integer*4    iasi1c_h_spare(2)   !Spare
      integer*4    iasi1c_h_wmosatid   !WMO satellite id (3 for METOP-1)
      integer*4    iasi1c_h_startchan(10)  !Start channel for scaling definition
      integer*4    iasi1c_h_endchan(10)    !End channel for scaling definition
      integer*4    iasi1c_h_scalefactor(10)!Radiance = scalrad*10^-scalefactor

      integer*4    iasi1c_h_filler(array_size_iasi1c - 53)
!
      integer*4 iasi1c_hd(array_size_iasi1c)
      equivalence(iasi1c_hd(1),iasi1c_h_site)
      common /iasi1chd/ iasi1c_h_site,iasi1c_h_cfill1,iasi1c_h_1bsite,
     &   iasi1c_h_cfill2,iasi1c_h_versnb,iasi1c_h_versyr,
     &   iasi1c_h_versdy,iasi1c_h_hdrcnt,iasi1c_h_satid,
     &   iasi1c_h_instrument,iasi1c_h_satht,iasi1c_h_period,
     &   iasi1c_h_startorbit,iasi1c_h_startdatayr,iasi1c_h_startdatady,
     &   iasi1c_h_startdatatime,iasi1c_h_endorbit,iasi1c_h_enddatayr,
     &   iasi1c_h_enddatady,iasi1c_h_enddatatime,iasi1c_h_scnlin,
     &   iasi1c_h_misscnlin,iasi1c_h_spare,iasi1c_h_wmosatid,
     &   iasi1c_h_startchan,iasi1c_h_endchan,iasi1c_h_scalefactor,
     &   iasi1c_h_filler
      
!##############################################################################
!
!               DATA RECORD
!
!##############################################################################
!
      integer*4   iasi1c_scnlin       !scan line number
      integer*4   iasi1c_granule      !granule number
      integer*4   iasi1c_scnlinyr     !scan line year
      integer*4   iasi1c_scnlindy     !scan line day of year
      integer*4   iasi1c_scnlintime(iasi_1cnfov) !scan line UTC time of day (ms)
      integer*4   iasi1c_GQisFlagQual(iasi_1cndet,iasi_1cnfov)

!The above flag combines GQisFlagQual and GQisFlagQualDetailed in the PFS format
! Bit 0: GQisFlagQual summary flag for all bands (1=anomaly)
! Bit 1: anomaly in band 1
! Bit 2: anomaly in band 2
! Bit 3: anomaly in band 3
! Bit 4: hardware
! Bit 5: Band 1 affected by spikes
! Bit 6: Band 2 affected by spikes
! Bit 7: Band 3 affected by spikes
! Bit 8: NZPD and complex calibration error
! Bit 9: On-board general quality flag
! Bit 10: Overflow/underflow
! Bit 11: Spectral calibration error
! Bit 12: Radiometric post-calibration error
! Bit 13: GQisFlagQual summary flag for all bands (same as bit 0)
! Bit 14: Missing sounder data
! Bit 15: Missing IIS data
! Bit 16: Missing AVHRR data

      integer*4   iasi1c_GQisQualIndex    !NEDT estimated/expected
      integer*4   iasi1c_GQisQualIndexLoc !IIS/AVHRR co-registration uncertainty
      integer*4   iasi1c_GQisQualIndexRad !NEDT estimated/expected radiometric
      integer*4   iasi1c_GQisQualIndexSpect !NEDT estimated/expected spectral
      integer*4   iasi1c_GQisSysTecSondQual

      integer*4   iasi1c_latlon(2,iasi_1cndet,iasi_1cnfov)
!                     lat/lon in degrees for iasi_1cnfov
!                     first   : 10^4 x (latitude)
!                     second  : 10^4 x (longitude)
      integer*4   iasi1c_angles(4,iasi_1cndet,iasi_1cnfov) 
!                     scan angles in degrees for iasi_1cnfov
!                     first:    10^2 x (local zenith angle)
!                     second:   10^2 x (local azimuth angle)
!                     third:    10^2 x (solar zenith angle)
!                     fourth:   10^2 x (solar azimuth angle)
      integer*4   iasi1c_scalti !sat altitude above reference ellipsoid, km*10

      integer*2   iasi1c_scalrad(iasi_1cnchan,iasi_1cndet,iasi_1cnfov) 
!                             Scaled radiance
      integer*4   iasi1c_avhcombination
!                             1 bit for each channel
      integer*4   iasi1c_avhrrmean(6,7,iasi_1cndet,iasi_1cnfov) 
!                             Mean radiance for 6 channels, 7 classes
      integer*4   iasi1c_avhmeanscalefactor(6,7,iasi_1cndet,iasi_1cnfov)
      integer*4   iasi1c_avhrrsdev(6,7,iasi_1cndet,iasi_1cnfov) 
!                             Std dev radiance for 6 channels, 7 classes
      integer*4   iasi1c_avhsdevscalefactor(6,7,iasi_1cndet,iasi_1cnfov)
      integer*4   iasi1c_avhrrfrac(7,iasi_1cndet,iasi_1cnfov)
!                             10^6 x Fraction in each class
      integer*4   iasi1c_avhrrCGy(7,iasi_1cndet,iasi_1cnfov)
!                             10^6 x Y angular position of centre of gravity
      integer*4   iasi1c_avhrrCGz(7,iasi_1cndet,iasi_1cnfov)
!                             10^6 x Z angular position of centre of gravity

!v2.0: New variables from IASI 1C EPS format version 8.0 (420 words):-

      integer*4   iasi1c_GIacVarImagIIS(iasi_1cnfov)
!                             10^6 x Variance of IIS image W/(m^2 sr m^-1)
      integer*4   iasi1c_GIacAvgImagIIS(iasi_1cnfov)
!                             10^6 x Average of IIS image W/(m^2 sr m^-1)
      integer*4   iasi1c_GEUMAvhrr1BCldFrac(iasi_1cndet,iasi_1cnfov)
!                             Cloud fraction
      integer*4   iasi1c_GEUMAvhrr1BLandFrac(iasi_1cndet,iasi_1cnfov)
!                             Land fraction
      integer*4   iasi1c_GEUMAvhrr1BQual(iasi_1cndet,iasi_1cnfov)
!                             Bits 0-6: percentage snow/ice
!                             Bit 7: 0 if AVHRR is OK
!                             If bit 7 is 1 then bits 0-6 show the number of bad
!                               AVHRR pixels (0-126)
                  

      integer*4 iasi1c_dt(array_size_iasi1c)
      equivalence(iasi1c_dt(1),iasi1c_scnlin)
      common /iasi1cdt/ iasi1c_scnlin,iasi1c_granule,iasi1c_scnlinyr,
     &  iasi1c_scnlindy,
     &  iasi1c_scnlintime,iasi1c_GQisFlagQual,iasi1c_GQisQualIndex,
     &  iasi1c_GQisQualIndexLoc,iasi1c_GQisQualIndexRad,
     &  iasi1c_GQisQualIndexSpect,iasi1c_GQisSysTecSondQual,
     &  iasi1c_latlon,iasi1c_angles,iasi1c_scalti,iasi1c_scalrad,
     &  iasi1c_avhcombination,iasi1c_avhrrmean,
     &  iasi1c_avhmeanscalefactor,
     &  iasi1c_avhrrsdev,iasi1c_avhsdevscalefactor,
     &  iasi1c_avhrrfrac,iasi1c_avhrrCGy,iasi1c_avhrrCGz,
     &  iasi1c_GIacVarImagIIS,iasi1c_GIacAvgImagIIS,
     &  iasi1c_GEUMAvhrr1BCldFrac,iasi1c_GEUMAvhrr1BLandFrac,
     &  iasi1c_GEUMAvhrr1BQual
!-
