!+ Level 1D Data Set (ATOVS on IASI grid)
!
! Name:  iasi1d.h
!
! Purpose:
!   'INCLUDE' file for common /iasi1dhd/   (IASI level 1D header record)
!                  and common /iasi1ddt/   (IASI level 1D data record)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0      12/08/05  Original code
!  1.1      21/12/06  increase to 314 channels. NCA
!  1.2      28/02/07  Error in iasi1d_cloudflag: should be defined for every
!                     spot. Increase iasi_1dvnnb to 2. NCA
!  1.3      12/03/09  Increase to 366 channels
!  1.4      25/07/14  Increase to 500 channels
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!   User adjustable parameters:
      integer       iasi_1dnchan        ! Number of channel slots present in 1d
         parameter (iasi_1dnchan = 500) !  Must be at least 1
      integer       iasi_1dnpc          ! Number of Principal Components present
         parameter (iasi_1dnpc = 300)   !  Must be at least 1

!   Normally fixed parameters:
      integer       iasi_1dnfov         ! Number of FOVs present in 1d
         parameter (iasi_1dnfov = 30)   !  Full resolution requires 4 records
      integer       iasi_1dvnnb         ! IASI level 1D format version number
         parameter (iasi_1dvnnb = 2)
      integer       iasi_1dvnyr         ! format version year for 1D IASI
         parameter (iasi_1dvnyr = 2007)
      integer       iasi_1dvndy         ! format version day for 1D IASI
         parameter (iasi_1dvndy = 59)
      integer       array_size_iasi1d   ! Length of array
         parameter (array_size_iasi1d = 
     &         6377 + 15*iasi_1dnchan + 30*iasi_1dnpc)
C1.2     &         6087 + 15*iasi_1dnchan + 30*iasi_1dnpc)
      integer       array_size_iasi1dspare   ! Length of spare header array
         parameter (array_size_iasi1dspare = 
     &         array_size_iasi1d - 134 - iasi_1dnchan)
      integer       iasi_1drecl         ! Record length in bytes
         parameter (iasi_1drecl = array_size_iasi1d*4)
!
!###########################################################################
!
!               HEADER RECORD
!
!###########################################################################
!
      character*3  iasi1d_h_site       !dataset creation site ID
      character*1  iasi1d_h_cfill1     !filler
      character*3  iasi1d_h_1bsite     !creation site for original 1B data
      character*1  iasi1d_h_cfill2     !filler
      integer*4    iasi1d_h_versnb     !level 1d format version number
      integer*4    iasi1d_h_versyr     !level 1d format version year
      integer*4    iasi1d_h_versdy     !level 1d format version day of year
      integer*4    iasi1d_h_hdrcnt     !count of header records in this data set
      integer*4    iasi1d_h_satid      !satellite id in series (1 for METOP-1)
      integer*4    iasi1d_h_grid       !code for instrument grid
!                                 (5=HIRS; 6=MSU; 10=AMSU-A; 11=AMSU-B; 12=IASI)
      integer*4    iasi1d_h_satht      !nominal satellite altitude, km*10
      integer*4    iasi1d_h_period     !nominal orbit period (seconds)
      integer*4    iasi1d_h_startorbit !orbit number (at start of file)
      integer*4    iasi1d_h_startdatayr!start of data set year
      integer*4    iasi1d_h_startdatady!start of data set day of the year
      integer*4    iasi1d_h_startdatatime !start of data set UTC time of day (ms)
      integer*4    iasi1d_h_endorbit   !orbit number (at end of file)
      integer*4    iasi1d_h_enddatayr  !end of data set year
      integer*4    iasi1d_h_enddatady  !end of data set day of the year
      integer*4    iasi1d_h_enddatatime!end of data set UTC time of day (ms)
      integer*4    iasi1d_h_scnlin     !count of scan lines in this data set
      integer*4    iasi1d_h_misscnlin  !count of missing scan lines
      integer*4    iasi1d_h_atovppvn   !ATOVPP version number (test vns = 9000+)
      integer*4    iasi1d_h_instruments!instruments present
!       (bit0=HIRS, bit1=MSU, bit3=AMSU-A, bit4=AMSU-B, bit5=AVHRR, bit 6=IASI)
      integer*4    iasi1d_h_wmosatid   !WMO satellite id
!       (3 for METOP-1, 4 for METOP-2, 5 for METOP-3)
      integer*4    iasi1d_h_spare1     !spare

! IASI variables

      integer*4    iasi1d_h_startchan(10)  !Start channel for scaling definition
      integer*4    iasi1d_h_endchan(10)    !End channel for scaling definition
      integer*4    iasi1d_h_scalefactor(10)!Radiance = scalrad*10^-scalefactor
      integer*4    iasi1d_h_eigvn          !Version number of eigenvectors
      integer*4    iasi1d_h_npc            !Number of PCs in format
      integer*4    iasi1d_h_npcused        !Number of PCs used
      integer*4    iasi1d_h_chanvn         !Version number of channel selection
      integer*4    iasi1d_h_nchan          !Number of channels in format
      integer*4    iasi1d_h_nchanused               !Number of channels used
      integer*4    iasi1d_h_channels(iasi_1dnchan)  !List of IASI channels
                                                    ! range 1 to 8461

! AMSU(-A&B) temperature-radiance conversion coefficients  (Planck function)
      integer*4    iasi1d_h_amsPlanck(3,Anchan+Bnchan)
!                  order of channels = AMSU-A 1-15, MHS 1-5
!                  (1,*) = 10^6 x (central wavenumber)
!                  (2,*) = 10^6 x (constant 1)
!                  (3,*) = 10^6 x (constant 2)

! Pre-processing information

      integer*4    iasi1d_h_dT  ! size of data batches processed within ATOVPP
!                                (seconds   - see ATOVPP documentation)
      integer*4    iasi1d_h_mapBTmode  ! mapping method used for BTs
!      (1=nearest neighbour, 2=bilinear interpolation, 3=spatial averaging)
      integer*4    iasi1d_h_Acorrvn    ! vn no AMSU-A limb/emiss correction
!                                         (=0 if data not corrected)
      integer*4    iasi1d_h_Apcpvn     ! vn no AMSU-A precip/scatter detection
!                                         (=0 if not done)
      integer*4    iasi1d_h_Aantenna   ! vn no, AMSU-A antenna corrections
                                      ! (from ATOVIN)
      integer*4    iasi1d_h_Bcorrvn    ! vn no MHS limb/emiss correction
!                                         (=0 if data not corrected)
      integer*4    iasi1d_h_Bpcpvn     ! vn no MHS precip/scatter detection
!                                         (=0 if not done)
      integer*4    iasi1d_h_Bantenna   ! vn no, MHS antenna corrections
                                      ! (from ATOVIN)
      integer*4    iasi1d_h_Ascat !threshold value for AMSU-A scattering index
      integer*4    iasi1d_h_ACrosby !threshold value for AMSU-A logistic precip
      integer*4    iasi1d_h_AB89 !threshold value for AMSU-A/B 89GHz differences
      integer*4    iasi1d_h_cloudtests  !Cloud tests used (0 if none)

      integer*4    iasi1d_h_recperscan !Records per scan: 1 or 4
      integer*4    iasi1d_h_fovmode    !0 = full resolution
                                       !1 to 4 = fixed detector
                                       !5+ = other methods (TBD)
      integer*4    iasi1d_h_spare2(array_size_iasi1dspare)  !spare

      integer*4 iasi1d_hd(array_size_iasi1d)
      equivalence(iasi1d_hd(1),iasi1d_h_site)
      common /iasi1dhd/ iasi1d_h_site, iasi1d_h_cfill1, iasi1d_h_1bsite,
     & iasi1d_h_cfill2, iasi1d_h_versnb, iasi1d_h_versyr, 
     & iasi1d_h_versdy,
     & iasi1d_h_hdrcnt, iasi1d_h_satid, iasi1d_h_grid, iasi1d_h_satht,
     & iasi1d_h_period, iasi1d_h_startorbit, iasi1d_h_startdatayr,
     & iasi1d_h_startdatady, iasi1d_h_startdatatime, iasi1d_h_endorbit,
     & iasi1d_h_enddatayr, iasi1d_h_enddatady, iasi1d_h_enddatatime,
     & iasi1d_h_scnlin, iasi1d_h_misscnlin, iasi1d_h_atovppvn,
     & iasi1d_h_instruments, iasi1d_h_wmosatid, iasi1d_h_spare1,
     & iasi1d_h_startchan, iasi1d_h_endchan, iasi1d_h_scalefactor,
     & iasi1d_h_eigvn, iasi1d_h_npc, iasi1d_h_npcused, iasi1d_h_chanvn,
     & iasi1d_h_nchan, iasi1d_h_nchanused, iasi1d_h_channels,
     & iasi1d_h_amsplanck, iasi1d_h_dT,
     & iasi1d_h_mapBTmode, iasi1d_h_Acorrvn,
     & iasi1d_h_Apcpvn, iasi1d_h_Aantenna, iasi1d_h_Bcorrvn,
     & iasi1d_h_Bpcpvn, iasi1d_h_Bantenna, iasi1d_h_Ascat,
     & iasi1d_h_ACrosby, iasi1d_h_AB89, iasi1d_h_cloudtests,
     & iasi1d_h_recperscan, iasi1d_h_fovmode,iasi1d_h_spare2

!##############################################################################
!
!               DATA RECORD
!
!##############################################################################

!General information

      integer*4   iasi1d_scnlin            !scan line number
      integer*4   iasi1d_scnlinyr          !scan line year
      integer*4   iasi1d_scnlindy          !scan line day of year
      integer*4   iasi1d_scnlintime(iasi_1dnfov) !scan line UTC time of day (ms)

      integer*4   iasi1d_instrtempa1       !AMSU-A1 RF shelf temp (K*100)
      integer*4   iasi1d_instrtempa2       !AMSU-A2 RF shelf temp (K*100)
      integer*4   iasi1d_instrtempb        !AMSU-B mixer ch 18-20 temp (K*100)
                                          !(all from the nearest scanlines)

      integer*4   iasi1d_spare1(2)         !spare

      integer*4   iasi1d_latlon(2,iasi_1dnfov)           !lat/lon in degrees
!                     first   : 10^4 x (latitude)
!                     second  : 10^4 x (longitude)
      integer*4   iasi1d_surface(2,iasi_1dnfov)          !height/type
!                     first   : surface height (metres)
!                     second  : surface type (0=sea, 1=mixed, 2=land)
      integer*4   iasi1d_angles(4,iasi_1dnfov)           !scan angles
!                     first:    10^2 x (local zenith angle)
!                     second:   10^2 x (local azimuth angle)
!                     third:    10^2 x (solar zenith angle)
!                     fourth:   10^2 x (solar azimuth angle)
      integer*4   iasi1d_scalti !sat altitude above reference ellipsoid, km*10
      integer*4   iasi1d_spare2(2)             !spare

!AMSU mapped to IASI

      integer*4   iasi1d_btemps(Anchan+Bnchan,iasi_1dnfov)
!                 10^2 x scene brightness temperature (K) for channels
!                 AMSU-A 1-15, MHS 1-5
!
      integer*4   iasi1d_iasiqual(iasi_1dnfov)
!                 Quality control word for each field of view:
!                  (all bits off implies acceptable data)
!                  bit 23-4: bit n set to 1 if brightness temperature in
!                            AMSU channel n-3 is physically unreasonable or has
!                            not been calculated due to calibration  
!                            problems.
!                  bit 3-1:  bit n set to 1 if data for IASI detector n are bad
!                  bit 0:    bad or missing data (in any or all channels)

      integer*4   iasi1d_prepro(7,iasi_1dnfov) ! values from pre-processing
!                  (1,*):   pre-processing quality control word with flags
!                            (all bits off implies acceptable data)
!                  bit 31: set if AMSU-A surface types not all the same
!                  bit 30: set if AMSU-A used secondary calibration
!                  bit 29: set if AMSU-B used secondary calibration
!                  bit 28: set if AMSU-B data missing
!                  bit 27: flag for cloud cost set for any mapped AMSU-A
!                  bit 26: scattering flag set for any mapped AMSU-A
!                          (only set over the sea)
!                  bit 25: logistic precipitation probability test
!                          calculated from AMSU-A data mapped to HIRS grid
!                  bit 24: Grody light rainfall test calculated on HIRS grid
!                  bit 23: mismatch between AMSU-A/B 89GHz values (any AMSU-A)
!                  bit 22: mismatch between surface type from topography
!                          dataset and from pre-processing (any AMSU-A)
!                  bit 21: fewer AMSU mapping co-locations than expected
!                  bit 20-4: spare   <zero fill>
!                  bit 3: set when AVHRR channel 3 is albedo not brightness temp
!                  bit 2: flag for cloud cost (recalculated on HIRS grid)
!                  bit 1: flag for scattering index (recalculated on HIRS grid)
!                  bit 0: set if AMSU-A & B data missing
!
!                  (2,*)    estimated surface type from the pre-processing:
!            1 = Bare young ice (i.e. new ice, no snow)
!            2 = Dry land (i.e. dry with or without significant vegetation)
!            3 = Dry snow (i.e. snow with water less than 2%, over land)
!            4 = Multi-year ice (i.e. old ice with snow [assumed dry] cover)
!            5 = Sea (i.e. open water, no islands, ice-free, WS=0 to 14m/s)
!            6 = Wet forest (i.e. established forest with wet canopy)
!            7 = Wet land (i.e. non-forested land with a wet surface)
!            8 = Wet snow (i.e. snow with water content > 2%, over land or ice)
!            9 = Desert
!
!                  (3,*) cost fn from PPASURF surface identification (*100)
!                  (4,*) scattering index (*100)
!                  (5,*) logistic precipitation probability (*100)
!                  (6,*)   spare
!                  (7,*)   spare

      integer*4   iasi1d_emiss(13,iasi_1dnfov) !13 emissivity parameters

!IASI data

      integer*4   iasi1d_GQisFlagQual(iasi_1dnfov)    !1=some anomaly
      integer*4   iasi1d_GQisQualIndex    !NEDT estimated/expected
      integer*4   iasi1d_GQisQualIndexLoc !IIS/AVHRR co-registration uncertainty
      integer*4   iasi1d_GQisQualIndexRad !NEDT estimated/expected radiometric
      integer*4   iasi1d_GQisQualIndexSpect !NEDT estimated/expected spectral
      integer*4   iasi1d_GQisSysTecSondQual

      integer*2   iasi1d_scalrad(iasi_1dnchan,iasi_1dnfov) !Scaled radiance

      integer*4   iasi1d_scores(iasi_1dnpc,iasi_1dnfov) !PC scores
      integer*4   iasi1d_logfit(3,iasi_1dnfov)!Reconstruction Scores for 3 bands
      integer*4   iasi1d_FOV(iasi_1dnfov) !30 values in range 1 to 120

!AVHRR data

      integer*4   iasi1d_avhcombination             !1 bit for each channel
      integer*4   iasi1d_avhrrmean(6,7,iasi_1dnfov) !Scaled mean radiance for 6
                                                    ! channels, 7 classes
      integer*4   iasi1d_avhrrsdev(6,7,iasi_1dnfov) !Scaled Std Dev for 6
                                                    ! channels, 7 classes
      integer*2   iasi1d_avhmeanscalefactor(6,7,iasi_1dnfov)
      integer*2   iasi1d_avhsdevscalefactor(6,7,iasi_1dnfov)
      integer*4   iasi1d_avhrrfrac(7,iasi_1dnfov) !10^6 x Fraction in each class
      integer*4   iasi1d_avhrrCGy(7,iasi_1dnfov)  !10^6 x Y angular position of 
                                                  ! centre of gravity (deg)
      integer*4   iasi1d_avhrrCGz(7,iasi_1dnfov)  !10^6 x Z angular position of
                                                  ! centre of gravity (deg)
!Cloud flag (user defined)

      integer*4   iasi1d_cloudflag(10,iasi_1dnfov)
C1.2      integer*4   iasi1d_cloudflag(10)
      
      integer*4 iasi1d_dt(array_size_iasi1d)
      equivalence(iasi1d_dt(1),iasi1d_scnlin)
      common /iasi1ddt/ iasi1d_scnlin,iasi1d_scnlinyr,iasi1d_scnlindy,
     & iasi1d_scnlintime,iasi1d_instrtempa1,iasi1d_instrtempa2,
     & iasi1d_instrtempb,iasi1d_spare1,iasi1d_latlon,iasi1d_surface,
     & iasi1d_angles,iasi1d_scalti,iasi1d_spare2,iasi1d_btemps,
     & iasi1d_iasiqual,iasi1d_prepro,iasi1d_emiss,iasi1d_GQisFlagQual,
     & iasi1d_GQisQualIndex,iasi1d_GQisQualIndexLoc,
     & iasi1d_GQisQualIndexRad,iasi1d_GQisQualIndexSpect,
     & iasi1d_GQisSysTecSondQual,iasi1d_scalrad,iasi1d_scores,
     & iasi1d_logfit,iasi1d_FOV,iasi1d_avhcombination,iasi1d_avhrrmean,
     & iasi1d_avhrrsdev,iasi1d_avhmeanscalefactor,
     & iasi1d_avhsdevscalefactor,iasi1d_avhrrfrac,iasi1d_avhrrCGy,
     & iasi1d_avhrrCGz,iasi1d_cloudflag
