!+ IASI PC Level 1C Data Set
!
!$Id$
!
! Purpose:
!   'INCLUDE' file for common /iasipchd/   (IASI PC level 1C header record)
!                  and common /iasipcdt/   (IASI PC level 1C data record)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2008, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0      11/09/08  Created from iasi1c.h. NCA
!  1.1      09/08/12  Version 2 - PC scores from integer*2 to integer*4, to
!                     avoid overflow.
!  1.2      25/07/14  Increase iasi_pcnchan and iasi_pcnrecon to 500 channels.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
!
! Declarations:
!
      integer       iasi_pcvnnb     ! iasipc level 1C format version number
         parameter (iasi_pcvnnb = 2)!  - this code is set up for vn no 2
      integer       iasi_pcvnyr     ! format version year for 1C iasipc
         parameter (iasi_pcvnyr = 2012)
      integer       iasi_pcvndy     ! format version day for 1C iasipc
         parameter (iasi_pcvndy = 222)
      integer       iasi_pcnchan    ! Number of channels present in format
         parameter (iasi_pcnchan = 500)
      integer       iasi_pcnrecon   ! Number of reconstructed radiances in format
         parameter (iasi_pcnrecon = 500)
      integer       iasi_pcnpc     ! Number of PCs in format
         parameter (iasi_pcnpc = 300)
      integer       iasi_pcnfov    ! Number of FOVs present in 1c
         parameter (iasi_pcnfov = 30)
      integer       iasi_pcndet    ! Number of Detectors per FOV
         parameter (iasi_pcndet = 4)
      integer       array_size_iasipc     ! Length of array
         parameter (array_size_iasipc = 24341
     &                        + iasi_pcnpc*120 
     &                        + iasi_pcnchan*60
     &                        + iasi_pcnrecon*60) !  Length of array
      integer       iasi_pcrecl     ! iasipc level 1C file record length
         parameter (iasi_pcrecl = array_size_iasipc*4)!  - number of bytes
!
!###########################################################################
!
!               HEADER RECORD
!
!###########################################################################
!
      character*3  iasipc_h_site       !dataset creation site ID
      character*1  iasipc_h_cfill1     !filler
      character*3  iasipc_h_1bsite     !creation site for original 1B data
      character*1  iasipc_h_cfill2     !filler
      integer*4    iasipc_h_versnb     !level 1c format version number
      integer*4    iasipc_h_versyr     !level 1c format version year
      integer*4    iasipc_h_versdy     !level 1c format version day of year
      integer*4    iasipc_h_hdrcnt     !count of header records in this data set
      integer*4    iasipc_h_satid      !satellite id in series (1 for METOP-1)
      integer*4    iasipc_h_instrument !instrument code
!                       (5=HIRS; 6=MSU; 10=AMSU-A; 11=AMSU-B; 12=MHS; 13=iasi;
!                        14=iasipc)
      integer*4    iasipc_h_satht      !nominal satellite altitude, km*10
      integer*4    iasipc_h_period     !nominal orbit period (seconds)
      integer*4    iasipc_h_startorbit !orbit number (at start of file)
      integer*4    iasipc_h_startdatayr!start of data set year
      integer*4    iasipc_h_startdatady!start of data set day of the year
      integer*4    iasipc_h_startdatatime !start of data set UTC time of day (ms)
      integer*4    iasipc_h_endorbit   !orbit number (at end of file)
      integer*4    iasipc_h_enddatayr  !end of data set year
      integer*4    iasipc_h_enddatady  !end of data set day of the year
      integer*4    iasipc_h_enddatatime!end of data set UTC time of day (ms)
      integer*4    iasipc_h_scnlin     !count of scan lines in this data set
      integer*4    iasipc_h_misscnlin  !count of missing scan lines
      integer*4    iasipc_h_spare(2)   !Spare
      integer*4    iasipc_h_wmosatid   !WMO satellite id (3 for METOP-1)
      integer*4    iasipc_h_startchan(10)  !Start channel for scaling definition
      integer*4    iasipc_h_endchan(10)    !End channel for scaling definition
      integer*4    iasipc_h_scalefactor(10)!Radiance = scalrad*10^-scalefactor
      integer*4    iasipc_h_recl       !record length in bytes
      integer*4    iasipc_h_nchan      !number of channels in format
      integer*4    iasipc_h_nrchan     !number of recon channels in format
      integer*4    iasipc_h_npc        !number of PCs in format
      integer*4    iasipc_h_nchanused  !number of channels used
      integer*4    iasipc_h_nrchanused !number of recon channels used
      integer*4    iasipc_h_npcused    !number of PCs used
      integer*4    iasipc_h_npcbands       !number of pc bands
      integer*4    iasipc_h_startpcchan(3) !start channel number for pc bands
      integer*4    iasipc_h_endpcchan(3)   !end channel number for pc bands
      integer*4    iasipc_h_startpc(3)     !start pc for pc bands
      integer*4    iasipc_h_endpc(3)       !end pc for pc bands
      integer*4    iasipc_h_quantization(3)!100*Quantization factor for PC scores
      integer*4    iasipc_h_chanvn     !Version number of original channel selection
      integer*4    iasipc_h_rchanvn    !Version number of reconstructed channels
      integer*4    iasipc_h_eigvn      !Version number of eigenvectors (from spec file)
      character*80 iasipc_h_evfile(3)  !eigenvector file names
      integer*4    iasipc_h_evfilevn(3)!version numbers derived from ev files
      integer*4    iasipc_h_channels(iasi_pcnchan)  !list of original channels
      integer*4    iasipc_h_rchannels(iasi_pcnrecon) !list of reconstructed chans
      
      integer*4    iasipc_h_filler(array_size_iasipc - 142 
     &                - iasi_pcnchan - iasi_pcnrecon)
!
      integer*4 iasipc_hd(array_size_iasipc)
      equivalence(iasipc_hd(1),iasipc_h_site)
      common /iasipchd/ iasipc_h_site,iasipc_h_cfill1,iasipc_h_1bsite,
     &   iasipc_h_cfill2,iasipc_h_versnb,iasipc_h_versyr,
     &   iasipc_h_versdy,iasipc_h_hdrcnt,iasipc_h_satid,
     &   iasipc_h_instrument,iasipc_h_satht,iasipc_h_period,
     &   iasipc_h_startorbit,iasipc_h_startdatayr,iasipc_h_startdatady,
     &   iasipc_h_startdatatime,iasipc_h_endorbit,iasipc_h_enddatayr,
     &   iasipc_h_enddatady,iasipc_h_enddatatime,iasipc_h_scnlin,
     &   iasipc_h_misscnlin,iasipc_h_spare,iasipc_h_wmosatid,
     &   iasipc_h_startchan,iasipc_h_endchan,iasipc_h_scalefactor,
     &   iasipc_h_recl,iasipc_h_nchan,iasipc_h_nrchan,iasipc_h_npc,
     &   iasipc_h_nchanused,iasipc_h_nrchanused,
     &   iasipc_h_npcused,iasipc_h_npcbands,iasipc_h_startpcchan,
     &   iasipc_h_endpcchan,iasipc_h_startpc,iasipc_h_endpc,
     &   iasipc_h_quantization,iasipc_h_chanvn,iasipc_h_rchanvn,
     &   iasipc_h_eigvn,iasipc_h_evfile,iasipc_h_evfilevn,
     &   iasipc_h_channels,iasipc_h_rchannels,iasipc_h_filler
      
!##############################################################################
!
!               DATA RECORD
!
!##############################################################################
!
      integer*4   iasipc_scnlin       !scan line number
      integer*4   iasipc_granule      !granule number
      integer*4   iasipc_scnlinyr     !scan line year
      integer*4   iasipc_scnlindy     !scan line day of year
      integer*4   iasipc_scnlintime(iasi_pcnfov) !scan line UTC time of day (ms)
      integer*4   iasipc_GQisFlagQual(iasi_pcndet,iasi_pcnfov)  !1=some anomaly
      integer*4   iasipc_GQisQualIndex    !NEDT estimated/expected
      integer*4   iasipc_GQisQualIndexLoc !IIS/AVHRR co-registration uncertainty
      integer*4   iasipc_GQisQualIndexRad !NEDT estimated/expected radiometric
      integer*4   iasipc_GQisQualIndexSpect !NEDT estimated/expected spectral
      integer*4   iasipc_GQisSysTecSondQual

      integer*4   iasipc_latlon(2,iasi_pcndet,iasi_pcnfov)
!                     lat/lon in degrees for iasi_pcnfov
!                     first   : 10^4 x (latitude)
!                     second  : 10^4 x (longitude)
      integer*4   iasipc_angles(4,iasi_pcndet,iasi_pcnfov) 
!                     scan angles in degrees for iasi_pcnfov
!                     first:    10^2 x (local zenith angle)
!                     second:   10^2 x (local azimuth angle)
!                     third:    10^2 x (solar zenith angle)
!                     fourth:   10^2 x (solar azimuth angle)
      integer*4   iasipc_scalti !sat altitude above reference ellipsoid, km*10

      integer*2   iasipc_scalrad(iasi_pcnchan,iasi_pcndet,iasi_pcnfov) 
!                             Scaled radiance
      integer*2   iasipc_reconrad(iasi_pcnrecon,iasi_pcndet,iasi_pcnfov) 
!                             Reconstructed scaled radiance
      integer*4   iasipc_scores(iasi_pcnpc,iasi_pcndet,iasi_pcnfov) 
!                             PC scores
      integer*4   iasipc_logfit(3,iasi_pcndet,iasi_pcnfov)
!                             Reconstruction Scores for 3 bands
      integer*4   iasipc_avhcombination
!                             1 bit for each channel
      integer*4   iasipc_avhrrmean(6,7,iasi_pcndet,iasi_pcnfov) 
!                             Mean radiance for 6 channels, 7 classes
      integer*4   iasipc_avhmeanscalefactor(6,7,iasi_pcndet,iasi_pcnfov)
      integer*4   iasipc_avhrrsdev(6,7,iasi_pcndet,iasi_pcnfov) 
!                             Std dev radiance for 6 channels, 7 classes
      integer*4   iasipc_avhsdevscalefactor(6,7,iasi_pcndet,iasi_pcnfov)
      integer*4   iasipc_avhrrfrac(7,iasi_pcndet,iasi_pcnfov)
!                             10^6 x Fraction in each class
      integer*4   iasipc_avhrrCGy(7,iasi_pcndet,iasi_pcnfov)
!                             10^6 x Y angular position of centre of gravity
      integer*4   iasipc_avhrrCGz(7,iasi_pcndet,iasi_pcnfov)
!                             10^6 x Z angular position of centre of gravity
      integer*4   iasipc_GIacVarImagIIS(iasi_pcnfov)
!                             10^6 x Variance of IIS image W/(m^2 sr m^-1)
      integer*4   iasipc_GIacAvgImagIIS(iasi_pcnfov)
!                             10^6 x Average of IIS image W/(m^2 sr m^-1)
      integer*4   iasipc_GEUMAvhrr1BCldFrac(iasi_pcndet,iasi_pcnfov)
!                             Cloud fraction
      integer*4   iasipc_GEUMAvhrr1BLandFrac(iasi_pcndet,iasi_pcnfov)
!                             Land fraction
      integer*4   iasipc_GEUMAvhrr1BQual(iasi_pcndet,iasi_pcnfov)
!                             Bits 0-6: percentage snow/ice
!                             Bit 7: 0 if AVHRR is OK
!                             If bit 7 is 1 then bits 0-6 show the number of bad
!                               AVHRR pixels (0-126)

      integer*4 iasipc_dt(array_size_iasipc)
      equivalence(iasipc_dt(1),iasipc_scnlin)
      common /iasipcdt/ iasipc_scnlin,iasipc_granule,iasipc_scnlinyr,
     &  iasipc_scnlindy,
     &  iasipc_scnlintime,iasipc_GQisFlagQual,iasipc_GQisQualIndex,
     &  iasipc_GQisQualIndexLoc,iasipc_GQisQualIndexRad,
     &  iasipc_GQisQualIndexSpect,iasipc_GQisSysTecSondQual,
     &  iasipc_latlon,iasipc_angles,iasipc_scalti,iasipc_scalrad,
     &  iasipc_reconrad,iasipc_scores,iasipc_logfit,
     &  iasipc_avhcombination,iasipc_avhrrmean,
     &  iasipc_avhmeanscalefactor,
     &  iasipc_avhrrsdev,iasipc_avhsdevscalefactor,
     &  iasipc_avhrrfrac,iasipc_avhrrCGy,iasipc_avhrrCGz,
     &  iasipc_GIacVarImagIIS,iasipc_GIacAvgImagIIS,
     &  iasipc_GEUMAvhrr1BCldFrac,iasipc_GEUMAvhrr1BLandFrac,
     &  iasipc_GEUMAvhrr1BQual
!-
