!+ AMSU-A Data Variables for processing within ATOVIN
!
! Name:  inastore.h
!
! Purpose:
!   'INCLUDE' file for COMMON / inastore /
!   containing AMSU-A data for use within ATOVIN code.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version    Date     Comment
!
!  1.0      2/2/96  Original code. (R.J.Renshaw)
!  1.1    18/06/98  ASatId added
!  1.2      6/6/01  Add moon position information (N.C.Atkinson)
!  1.3     10/9/15  Add information for computing NEDT
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
      INTEGER       ASatId         ! e.g. 15 for NOAA-15
!
      REAL          Acalib(2,3,Anchan)     ! calibration coefficients:
                                           !   primary & secondary,
                                           !   1st,2nd,0th coefficients,
                                           !   for each channel.
      INTEGER       Acounts(Anchan,Anfovs) ! instrument counts for each chan,
                                           ! for each field of view
      REAL          APlanck(4,Anchan)      ! 'inverse Planck function'
!                   temperature-radiance conversion
!                   order of channels = 1,2,3,4,...,15
!                      (1,*) central wavenumber
!                      (2,*) constant 1
!                      (3,*) constant 2

! 'antenna efficiencies'
!   ( reference: 'Measurements of the AMSU-B Antenna Pattern',
!   T.J.Hewison & R.Saunders, IEEE Transactions on Geoscience
!   and Remote Sensing, Vol 34, No 2, March 1996 )
      REAL          Aeff_Earth(Anfovs,Anchan)   ! Earth efficiencies
      REAL          Aeff_Platf(Anfovs,Anchan)   ! Platform
      REAL          Aeff_Space(Anfovs,Anchan)   ! Space

! Moon position information and flags
      INTEGER       mxlin_moon
      PARAMETER (mxlin_moon=1000)   
      INTEGER       mxlin_nedt_ama
      PARAMETER (mxlin_nedt_ama=1000)   
!
!Set mxlin_moon to maximum number of 1B scan lines (1000) in order to invoke
!routines to interpolate gain during moon events.
!Or set to 1 to just flag the event
!
      REAL          dummyastore   !To fix alignment
      REAL*8        tdays(2)      !Start/end time in days from 12Z, 1 Jan 2000
      REAL          ra(2)         !Start/end right ascension of moon (radians)
      REAL          sindec(2)     !Start/end sine of moon declination
      REAL          cosdec(2)     !Start/end cosine of moon declination
      REAL          sinamsual(2)  !A1/A2 sine of AMSU space view altitude
      REAL          cosamsual(2)  !A1/A2 cosine of AMSU space view altitude
      REAL          angle_diff(2) !A1/A2 Angle between moon and sp view (deg)
      INTEGER       MFlag         !Moon flag
      INTEGER       Moonflag(mxlin_moon) !Moon flag per scan

      INTEGER       Acoldcals(2,Anchan,mxlin_nedt_ama)
      INTEGER       Awarmcals(2,Anchan,mxlin_nedt_ama)
      REAL          Acalibs(3,Anchan,mxlin_nedt_ama)      !radiance calibration coefs 
      REAL          AcoldNEDT(Anchan,mxlin_nedt_ama) 
      REAL          AwarmNEDT(Anchan,mxlin_nedt_ama)
      REAL          AcoldNEDT_rms(Anchan)
      REAL          AwarmNEDT_rms(Anchan)

      COMMON / inastore / ASatId, Acalib, Acounts, APlanck,
     &                    Aeff_Earth, Aeff_Platf, Aeff_Space, 
     &                    dummyastore,
     &                    tdays, ra, sindec, cosdec, sinamsual, 
     &                    cosamsual, angle_diff, MFlag, Moonflag,
     &                    Acoldcals, Awarmcals, Acalibs,
     &                    AcoldNEDT, AwarmNEDT,
     &                    AcoldNEDT_rms, AwarmNEDT_rms 
