!+ AMSU-B Data Variables for processing within ATOVIN
!
! Name:  inbstore.h
!
! Purpose:
!   'INCLUDE' file for COMMON / inbstore /
!   containing AMSU-B data for use within ATOVIN code.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version    Date     Comment
!
!  1.0      2/2/96  Original code. (R.J.Renshaw)
!  1.1      04/01/99 Add bias correction tables (N.C.Atkinson)
!  1.2      10/09/15 Add information for computing NEDT
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
      REAL          Bcalib(2,3,Bnchan)     ! calibration coefficients:
                                           !   primary & secondary,
                                           !   1st,2nd,0th coefficients,
                                           !   for each channel.
      INTEGER       Bcounts(Bnchan,Bnfovs) ! instrument counts for each chan,
                                           ! for each field of view
      REAL          BPlanck(4,Bnchan)      ! 'inverse Planck function'
!                   temperature-radiance conversion
!                   order of channels = 1,2,3,4,5
!                      (1,*) central wavenumber
!                      (2,*) constant 1
!                      (3,*) constant 2

! 'antenna efficiencies'
!   ( reference: 'Measurements of the AMSU-B Antenna Pattern',
!   T.J.Hewison & R.Saunders, IEEE Transactions on Geoscience
!   and Remote Sensing, Vol 34, No 2, March 1996 )
      REAL          Beff_Earth(Bnfovs,Bnchan)   ! Earth efficiencies
      REAL          Beff_Platf(Bnfovs,Bnchan)   ! Platform
      REAL          Beff_Space(Bnfovs,Bnchan)   ! Space

      INTEGER       mxlin_nedt_mhs
      PARAMETER (mxlin_nedt_mhs=3000)   

      INTEGER       Bcoldcals(4,Bnchan,mxlin_nedt_mhs)
      INTEGER       Bwarmcals(4,Bnchan,mxlin_nedt_mhs)
      REAL          Bcalibs(3,Bnchan,mxlin_nedt_mhs)      !radiance calibration coefs 
      REAL          BcoldNEDT(Bnchan,mxlin_nedt_mhs) 
      REAL          BwarmNEDT(Bnchan,mxlin_nedt_mhs)
      REAL          BcoldNEDT_rms(Bnchan)
      REAL          BwarmNEDT_rms(Bnchan)

      INTEGER*2     CountsCorr(5,90,4) !Bias corrections for 4 transmitters
      INTEGER*2     NewCorr(5,90,3)    !Anomalous bias corrections for 3 scans
      INTEGER*2     STXPowTmp(5)       !Transmit power temporary storage

      COMMON / inbstore / Bcalib, Bcounts, BPlanck,
     &                    Beff_Earth, Beff_Platf, Beff_Space,
     &                    Bcoldcals, Bwarmcals, Bcalibs,
     &                    BcoldNEDT, BwarmNEDT,
     &                    BcoldNEDT_rms, BwarmNEDT_rms, 
     &                    CountsCorr, NewCorr, STXPowTmp
