!+ HIRS Data Variables for processing within ATOVIN
!
! Name:  inhstore.h
!
! Purpose:
!   'INCLUDE' file for COMMON / inhstore /
!   containing HIRS data for use within ATOVIN code.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version    Date     Comment
!
!  1.0      2/2/96  Original code. (R.J.Renshaw)
!  1.1     20/1/97  Hcaline added. (R.J.Renshaw)
!  1.2     15/9/15  Add information for computing NEDT
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
      REAL          Hcalib(2,3,Hnchan)     ! calibration coefficients:
                                           !   primary & secondary,
                                           !   2nd,1st,0th coefficients,
                                           !   for each channel.
      INTEGER       Hcounts(Hnchan,Hnfovs) ! instrument counts for each chan,
                                           ! for each field of view
      REAL          HPlanck(4,Hnchan)      ! 'inverse Planck function'
!                   temperature-radiance conversion
!                   order of channels = 1,2,3,4,...,18,19
!                   for channels 1 to 19 :
!                      (1,*) First radiation constant * (central wavenumber)^3
!                      (2,*) Second radiation constant * central wavenumber
!                      (3,*) bandwidth correction coefficient 'a'
!                      (4,*) bandwidth correction coefficient 'b'
!                            such that T* = a + bT  where radiance=B(T)
!                   and for channel 20 :
!                      (1,20) = albedo-radiance ch20 solar filtered irradiance
!                      (2,20) = albedo-radiance ch 20 equivalent filter width

      LOGICAL       Hcaline                ! .TRUE. for calibration lines

      INTEGER       mxlin_nedt_hrs
      PARAMETER (mxlin_nedt_hrs=30)   !cal scans only

      INTEGER       Hcoldline(mxlin_nedt_hrs)
      INTEGER       Hwarmline(mxlin_nedt_hrs)
      REAL          HcoldNEDT(Hnchan-1,mxlin_nedt_hrs) 
      REAL          HwarmNEDT(Hnchan-1,mxlin_nedt_hrs)
      REAL          HcoldNEDT_rms(Hnchan-1)
      REAL          HwarmNEDT_rms(Hnchan-1)
      REAL          Hdbdt(Hnchan-1)

      COMMON / inhstore / Hcalib, Hcounts, HPlanck, Hcaline,
     &  Hcoldline, Hwarmline,
     &  HcoldNEDT, HwarmNEDT, HcoldNEDT_rms, HwarmNEDT_rms, Hdbdt
