!+ Parameters for ATOVIN Processing
!
! Name:  inparms.h
!
! Purpose:
!   'INCLUDE' file containing parameters for use within ATOVIN code.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version    Date     Comment
!
!  1.0     22/2/96  Original code. (R.J.Renshaw)
!  1.1     19/3/96  Process ATOVS and TOVS.  RJR
!  1.2     12/04/96 'angles' deleted, BT limits changed.  RJR
!  1.3     12/12/96 WMO satid deleted for new 1C format vn 1.8
!  1.4     09/06/98 missing data indicator changed from 0
!                   *cycle REAL not INTEGER
!  1.5     27/06/02 MaxSats changed to 5 to cope with NOAA-17
!  1.6     24/09/14 MaxSats changed to 30 (was 15) - same as satid.h
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
      CHARACTER*3   SiteID        ! file creation site ID
!       - NESDIS maintain a list of 'recognised' site ids
! *********************************************
! *** Please edit this line as appropriate: ***
         PARAMETER (SiteID= AAPP_SITE_ID ) ! e.g. 'UKM' for UKMO
! *********************************************
!
      INTEGER       MDI           ! general purpose missing data indicator
         PARAMETER (MDI=-999999)
!
! Satellite information: (from Fixed Data File)
      INTEGER       MaxSats       ! maximum number of satellites available
         PARAMETER (MaxSats=30)   ! in Fixed Data File
      INTEGER       FDFvnno(MaxSats) ! version number from FDF
      INTEGER       NOAAid(MaxSats)  ! NOAA satellite ids (e.g. 15 for NOAA-15)
      INTEGER       NESDISid(MaxSats)! NESDIS (e.g. 4 for NOAA-15)
      INTEGER       WMOid(MaxSats)   ! WMO (e.g. 206 for NOAA-15)
      REAL          SatHt(MaxSats)   ! nominal satellite height (km)
      REAL          OrbitPd(MaxSats) ! nominal satellite orbit period (seconds)
!
      INTEGER       Nsat          ! number of satellites actually in FDF
!
      COMMON / FDFinfo /
     &         FDFvnno, NOAAid, NESDISid, WMOid, SatHt, OrbitPd, Nsat
!
! Instrument dependent parameters:
!
! HIRS
! ----
      INTEGER       Hnchan        ! number of channels (19 IR + 1 visible)
         PARAMETER (Hnchan=20)
      INTEGER       Hnfovs        ! number of fields-of-view in 1 scanline
         PARAMETER (Hnfovs=56)
      REAL          Hcycle        ! time between successive scans (millisec)
         PARAMETER (Hcycle=6400.)
      REAL          HmaxBT        ! maximum brightness temperature acceptable
         PARAMETER (HmaxBT=340.)  ! (Kelvin)
      REAL          HminBT        ! minimum brightness temperature acceptable
         PARAMETER (HminBT=150.)  ! (Kelvin)
!
! AMSU-A
! ------
      INTEGER       Anchan        ! number of channels
         PARAMETER (Anchan=15)
      INTEGER       Anfovs        ! number of fields-of-view in 1 scanline
         PARAMETER (Anfovs=30)
      REAL          Acycle        ! time between successive scans (millisec)
         PARAMETER (Acycle=8000.)
      REAL          AmaxBT        ! maximum brightness temperature acceptable
         PARAMETER (AmaxBT=340.)  ! (Kelvin)
      REAL          AminBT        ! minimum brightness temperature acceptable
         PARAMETER (AminBT=90.)   ! (Kelvin)
!
! AMSU-B
! ------
      INTEGER       Bnchan        ! number of channels
         PARAMETER (Bnchan=5)
      INTEGER       Bnfovs        ! number of fields-of-view in 1 scanline
         PARAMETER (Bnfovs=90)
      REAL          Bcycle        ! time between successive scans (millisec)
         PARAMETER (Bcycle=Acycle/3.)
      REAL          BmaxBT        ! maximum brightness temperature acceptable
         PARAMETER (BmaxBT=340.)  ! (Kelvin)
      REAL          BminBT        ! minimum brightness temperature acceptable
         PARAMETER (BminBT=90.)   ! (Kelvin)
!
! MSU
! ---
      INTEGER       Mnchan        ! number of channels
         PARAMETER (Mnchan=4)
      INTEGER       Mnfovs        ! number of fields-of-view in 1 scanline
         PARAMETER (Mnfovs=11)
      REAL          Mcycle        ! time between successive scans (millisec)
         PARAMETER (Mcycle=25600.)
      REAL          MmaxBT        ! maximum brightness temperature acceptable
         PARAMETER (MmaxBT=340.)  ! (Kelvin)
      REAL          MminBT        ! minimum brightness temperature acceptable
         PARAMETER (MminBT=90.)   ! (Kelvin)
