!+ IRAS Level 1C Data Set
!
! Name:  iras1c.h
!
! Purpose:
!   'INCLUDE' file for common /iras1chd/   (MWTS level 1C header record)
!                  and common /iras1cdt/   (MWTS level 1C data record)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2013, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    31/01/2013  From mwts1c.
!  1.1    05/06/2014  Increase number of mapped channels to 28 (for FY-3C) and
!                     add header information. Size of record increased from 2540
!                     to 3611 words
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
!
! Declarations:
!
      integer       iras_1cvnnb     ! IRAS level 1C format version number
         parameter (iras_1cvnnb = 101)
      integer       iras_1cvnyr     ! format version year for 1C IRAS
         parameter (iras_1cvnyr = 2014)
      integer       iras_1cvndy     ! format version day for 1C IRAS
         parameter (iras_1cvndy = 156)
      integer       iras_nchan
         parameter (iras_nchan = 26)
      integer       iras_nmapchan
         parameter (iras_nmapchan = 28) !Space for MWTS/MWHS (was 9)
      integer       iras_nfovs
         parameter (iras_nfovs = 56)
      integer       array_size_iras1c     ! Length of array
          parameter (array_size_iras1c = 3611) !  Length of array (was 2540)
      integer       iras_1crecl     ! IRAS level 1C file record length
         parameter (iras_1crecl = array_size_iras1c*4)!  - number of bytes
!
!###########################################################################
!
!               HEADER RECORD
!
!###########################################################################
!
      character*3  iras1c_h_site       !dataset creation site ID
      character*1  iras1c_h_cfill1     !filler
      character*3  iras1c_h_1bsite     !creation site for original 1B data
      character*1  iras1c_h_cfill2     !filler
      integer*4    iras1c_h_versnb     !level 1c format version number
      integer*4    iras1c_h_versyr     !level 1c format version year
      integer*4    iras1c_h_versdy     !level 1c format version day of year
      integer*4    iras1c_h_hdrcnt     !count of header records in this data set
      integer*4    iras1c_h_satid      !satellite id (e.g. 1 for FY-3A)
      integer*4    iras1c_h_instrument !instrument code
!                                      (15=MWTS, 16=MWHS, 17=IRAS)
      integer*4    iras1c_h_satht      !nominal satellite altitude, km*10
      integer*4    iras1c_h_period     !nominal orbit period (seconds)
      integer*4    iras1c_h_startorbit !orbit number (at start of file)
      integer*4    iras1c_h_startdatayr!start of data set year
      integer*4    iras1c_h_startdatady!start of data set day of the year
      integer*4    iras1c_h_startdatatime !start of data set UTC time of day (ms)
      integer*4    iras1c_h_endorbit   !orbit number (at end of file)
      integer*4    iras1c_h_enddatayr  !end of data set year
      integer*4    iras1c_h_enddatady  !end of data set day of the year
      integer*4    iras1c_h_enddatatime!end of data set UTC time of day (ms)
      integer*4    iras1c_h_scnlin     !count of scan lines in this data set
      integer*4    iras1c_h_misscnlin  !count of missing scan lines
      integer*4    iras1c_h_spare(2)   !Spare

! temperature-radiance conversion coefficients
      integer*4    iras1c_h_tempradcnv(3,iras_nchan)
!                  order of channels = 1,2,3,4...
!                  for nochannel = 1 to 26 :
!                      10^4 x (nochannel central wavenumber)
!                      10^6 x (nochannel constant 1)
!                      10^6 x (nochannel constant 2)
      integer*4    iras1c_h_wmosatid   !WMO satellite id, 520=FY-3A

      integer*4    iras1c_h_nchan         !number of IRAS channels in format
      integer*4    iras1c_h_nmapchan      !number of mapped channels in format
      integer*4    iras1c_h_nmapchanused_mwts  !number of MWTS channels used
      integer*4    iras1c_h_nmapchanused_mwhs  !number of MWHS channels used

      integer*4    iras1c_h_filler(3506)  !filler 
!
      integer*4 iras1c_hd(array_size_iras1c)
      equivalence(iras1c_hd(1),iras1c_h_site)
      common /iras1chd/ iras1c_h_site, iras1c_h_cfill1, iras1c_h_1bsite,
     & iras1c_h_cfill2, iras1c_h_versnb, iras1c_h_versyr, 
     & iras1c_h_versdy,
     & iras1c_h_hdrcnt, iras1c_h_satid, iras1c_h_instrument,
     & iras1c_h_satht,
     & iras1c_h_period, iras1c_h_startorbit, iras1c_h_startdatayr,
     & iras1c_h_startdatady, iras1c_h_startdatatime,
     & iras1c_h_endorbit,
     & iras1c_h_enddatayr, iras1c_h_enddatady, iras1c_h_enddatatime,
     & iras1c_h_scnlin, iras1c_h_misscnlin,
     & iras1c_h_spare, iras1c_h_tempradcnv, iras1c_h_wmosatid, 
     & iras1c_h_nchan, iras1c_h_nmapchan, iras1c_h_nmapchanused_mwts,
     & iras1c_h_nmapchanused_mwhs, iras1c_h_filler
!
!##############################################################################
!
!               DATA RECORD
!
!##############################################################################
!
      integer*4   iras1c_scnlin     !scan line number
      integer*4   iras1c_scnlinyr   !scan line year
      integer*4   iras1c_scnlindy   !scan line day of year
      integer*4   iras1c_scnlintime !scan line UTC time of day in milliseconds
      integer*4   iras1c_geo_qc     !quality indicators
      integer*4   iras1c_scnlin_qc
      integer*4   iras1c_cal_qc
      integer*4   iras1c_pixel_qc(iras_nfovs)
      integer*4   iras1c_instrtemp(9)          !Instrument temps (K*100)

      integer*4   iras1c_latlon(2,iras_nfovs)!lat/lon in degrees for iras_nfovs
!                     first   : 10^4 x (latitude)
!                     second  : 10^4 x (longitude)
      integer*4   iras1c_surface(2,iras_nfovs) !height/type for iras_fovs
!                     first   : surface height (metres)
!                     second  : surface type (0=sea, 1=mixed, 2=land, 3=inland water)
      integer*4   iras1c_angles(4,iras_nfovs) !scan angles for iras_nfovs
!                     first:    10^2 x (local zenith angle)
!                     second:   10^2 x (local azimuth angle)
!                     third:    10^2 x (solar zenith angle)
!                     fourth:   10^2 x (solar azimuth angle)
      integer*4   iras1c_scalti !sat altitude above reference ellipsoid, km*10
      integer*4   iras1c_satposition(3)   !x,y,z in metres
      integer*4   iras1c_satvelocity(3)   !vx,vy,vz in m/s
      integer*4   iras1c_orbitangle       !angle round orbit relative to solar
                                           !plane, in deg*100

      integer*4   iras1c_btemps(iras_nchan,iras_nfovs) !BT data for iras_nfovs
!                     10^2 x scene brightness temperature (K)

      integer*4   iras1c_btmap(iras_nmapchan,iras_nfovs) !mapped MWTS/MWHS BTs
!                     10^2 x scene brightness temperature (K)

      integer*4   iras1c_dataqual(iras_nfovs)
!                 quality control word for the data in each field of view:
!                  (all bits off implies acceptable data)
!                  bit 1:    set if some channels missing
!                  bit 0:    set if all channels missing

      integer*4   iras1c_filler(3)            !filler

      integer*4 iras1c_dt(array_size_iras1c)
      equivalence(iras1c_dt(1),iras1c_scnlin)
      common /iras1cdt/ iras1c_scnlin, iras1c_scnlinyr, iras1c_scnlindy,
     & iras1c_scnlintime, iras1c_geo_qc, iras1c_scnlin_qc, 
     & iras1c_cal_qc, iras1c_pixel_qc, iras1c_instrtemp,
     & iras1c_latlon, iras1c_surface, iras1c_angles,
     & iras1c_scalti, iras1c_satposition, iras1c_satvelocity,
     & iras1c_orbitangle, iras1c_btemps, iras1c_btmap,
     & iras1c_dataqual, iras1c_filler
!-
