!$Id: mhs1b.h,v 1.3 2005/04/15 13:39:48 frna Exp $
!###########################################################################
!
!               MHS Level 1B Data Set
!
! Created 19 June 2003, based on NOAA Version 1.1. N C Atkinson
! Updated 18 Oct 2004, based on NOAA update of Sept 27 2004. N C Atkinson
!
!###########################################################################
!                                                                          !
!                      MHS  PARAMETERS                                  !
!                                                                          !
!###########################################################################

       INTEGER reclmhs         !length in byte of one MHS record
       parameter(reclmhs=3072)
       INTEGER cmhs_size       !length in INTEGER*2 of one MHS record
       parameter(cmhs_size=1536)

!!###########################################################################
!
!              MHS HEADER RECORD
!
!##############################################################################
C!
C!......General information
C!
       CHARACTER*3  mhs_h_siteid        !Data set creation site ID
C!                            CMS = Centre de Meteorologie Spatiale/France
C!                            DSS = Dundee Satellite Receiving Station/UK
C!                            NSS = NESDIS/USA
C!                            UKM = Met Office/UK
       CHARACTER*1  mhs_h_blank
       INTEGER*2    mhs_h_l1bversnb     !level 1b format version number
       INTEGER*2    mhs_h_l1bversyr     !level 1b format version year
       INTEGER*2    mhs_h_l1bversdy     !level 1b format version day of year
       INTEGER*2    mhs_h_reclg         ! logical record length
       INTEGER*2    mhs_h_blksz         ! blocksize
       INTEGER*2    mhs_h_hdrcnt        !count of header records in data set
       INTEGER*2    mhs_h_filler1(3)
       CHARACTER*42 mhs_h_dataname      !data set name
       CHARACTER*8  mhs_h_prblkid       !processing block identification
       INTEGER*2    mhs_h_satid         !NOAA spacecraft identification code
C!                            7=NOAA-N
C!                            8=NOAA-N'
C!                            11=Metop-1
C!                            12=Metop-2
C!                            13=Metop-3
C!                            14=Metop simulator
       INTEGER*2    mhs_h_instid        !MHS instrument identification
C!                            0=EM
C!                            1=PFM (NOAA-N)
C!                            2=FM2 (NOAA-N')
C!                            3=FM3 (Metop-1) <TBC>
C!                            4=FM4 (Metop-2) <TBC>
C!                            5=FM5 (Metop-3) <TBC>
       INTEGER*2    mhs_h_datatyp       !data type code
C!                            12 = MHS
       INTEGER*2    mhs_h_tipsrc        !TIP source code
C!                            0 = unused, i.e. GAC/HRPT/LAC data
C!                            1 = GAC-embedded AMSU and TIP
C!                            2 = stored TIP
C!                            3 = HRPT-embedded AMSU and TIP
C!                            4 = stored AIP (SAIP)
       INTEGER*4    mhs_h_startdatajd   !start of data set julian day
C!                                       starting from 0 at 00h, 1 Jan 1950
       INTEGER*2    mhs_h_startdatayr   !start of data set year, 4 digits
       INTEGER*2    mhs_h_startdatady   !start of data set day of year
       INTEGER*4    mhs_h_startdatatime !start of data set UTC time of day in
C!                                       milliseconds
       INTEGER*4    mhs_h_enddatajd     !end of data set julian day
       INTEGER*2    mhs_h_enddatayr     !end of data set year
       INTEGER*2    mhs_h_enddatady     !end of data set day of year
       INTEGER*4    mhs_h_enddatatime   !end of data set UTC time of day in
C!                                       milliseconds
       INTEGER*2    mhs_h_cpidsupdyr    !year of last CPIDS update
       INTEGER*2    mhs_h_cpidsupddy    !day of year of last cpids update
       INTEGER*2    mhs_h_fov1offset    !time offset for FOV 1 (ms)
       INTEGER*2    mhs_h_filler2(5)    !filler
C!
C!......Data set quality indicators
C!
       INTEGER*4   mhs_h_inststat1       !MHS instrument status (first reported)
C!                              Bits 31-24: as for mhs_mode
C!                              Bits 23-0:  as for mhs_switchstat 
       INTEGER*2   mhs_h_filler3
       INTEGER*2   mhs_h_statchrecnb     !record number of status change
C!                 ( if 0, none occurred)
       INTEGER*4   mhs_h_inststat2       !MHS second instrument status
C!                 ( if previous word is 0, no change)
       INTEGER*2   mhs_h_scnlin         !count of scan lines in this data set
       INTEGER*2   mhs_h_callocsclin    !count of calibrated, earth located
C!                                       scan lines in this data set
       INTEGER*2   mhs_h_misscnlin      !count of missing scan lines
       INTEGER*2   mhs_h_datagaps       !count of data gaps in this data set
       INTEGER*2   mhs_h_moonscnlin     !count of scans with moon in space view
C!                                       (-1 = detection algorithm turned off)
       INTEGER*2   mhs_h_okdatafr       !count of data frames without frame
C!                                       sync word errors
       INTEGER*2   mhs_h_pacsparityerr  !count of PACS detected TIP parity
C!                                       errors
       INTEGER*2   mhs_h_auxsyncerrsum  !sum of all auxiliary sync errors
C!                                       detected in the input data
       INTEGER*2   mhs_h_timeseqerr     !time sequence error
C!               (0 =none; otherwise the record number of the first occurrence)
       INTEGER*2   mhs_h_timeseqerrcode !time sequence error code
       INTEGER*2   mhs_h_socclockupind  !SOCC clock update indicator
C!                 ( 0 = none during this orbit;
C!                   otherwise the record number of the first occurrence)
       INTEGER*2   mhs_h_locerrind      !earth location error indicator
C!                 ( 0 = none during this orbit;
C!                   otherwise the record number of the first occurrence)
       INTEGER*2   mhs_h_locerrcode     !earth location error code
       INTEGER*2   mhs_h_pacsstatfield
       INTEGER*2   mhs_h_pacsdatasrc
       CHARACTER*8 mhs_h_spare2         ! reserved for ingester
       CHARACTER*8 mhs_h_spare3         ! reserved for decommutation
       INTEGER*2   mhs_h_filler4(8)
C!
C!......Calibration parameters
C!
      INTEGER*2 mhs_h_tsensid      !Instrument temperature sensor ID
C!                            0 = primary (LO H5 temperature [QBS5])
C!                            1 = secondary (LO H1 temperature [QBS1])
      INTEGER*2 mhs_h_filler5
      INTEGER*2 mhs_h_reftemp(6) !Instrument reference temps 
C!                               Minimum, nominal, maximum (Primary & secondary)
      INTEGER*2 mhs_h_bias(35)     ! Calibration bias corrections
C!                            For each of 5 channels:
C!                              Warm load temp correction (at 3 instr temps)
C!                              Cold space temp correction (for 4 profiles)
C!                                   
      INTEGER*4 mhs_h_nonlin(30)   ! Nonlinearity corrections
C!                        For each LO (LO A and LO B):
C!                          For each of 5 channels:
C!                            Nonlin correction at 3 instr temps (min,nom,max)
      INTEGER*4 mhs_h_filler6(4)
C!
C!......Temp to radiance conversion factors
C!
      INTEGER*4 mhs_h_temrad(15)  !10^6*(Central wavenumber, const1 and const2)
      INTEGER*4 mhs_h_filler7(4)
C!
C!......Earth location
C!
      CHARACTER*8 mhs_h_modelid    !Reference elipsoid model ID
      INTEGER*2   mhs_h_nadloctol  !10 x (nadir earth location tolerance in
C!                                         kilometers)
      INTEGER*2   mhs_h_locbit     !earth location bit field
C!      bit 2: dynamic attitude error correction (0=not performed, 1=performed)
C!      bit 1: reasonableness test (0=inactive, 1=active)
C!      bit 0: constant attitude error correction (0=not performed, 1=performed)
      INTEGER*2   mhs_h_filler8
      INTEGER*2   mhs_h_rollerr    !10^3 x (constant roll attitude error in
C!                                           degrees)
      INTEGER*2   mhs_h_pitcherr   !10^3 x (constant pitch attitude error in
C!                                           degrees)
      INTEGER*2   mhs_h_yawerr     !10^3 x (constant yaw attitude error in
C!                                           degrees)
      INTEGER*2   mhs_h_epoyr      !epoch year for orbit vector
      INTEGER*2   mhs_h_epody      !day of epoch year for orbit vector
      INTEGER*4   mhs_h_epotime    !epoch UTC time of day in milliseconds for
C!                                   orbit vector
C!                  The following are all at the orbit vector epoch time
      INTEGER*4   mhs_h_smaxis     !10^6 x (semi-major axis in kilometers)
      INTEGER*4   mhs_h_eccen      !10^8 x (eccentricity)
      INTEGER*4   mhs_h_incli      !10^5 x (inclination in degrees)
      INTEGER*4   mhs_h_argper     !10^5 x (argument of perigee in degrees)
      INTEGER*4   mhs_h_rascnod    !10^5 x (right ascension of the ascending
C!                                           node in degrees)
      INTEGER*4   mhs_h_manom      !10^5 x (mean anomaly in degrees)
      INTEGER*4   mhs_h_xpos       !10^6 x (position vector x component in
C!                                           kilometers)
      INTEGER*4   mhs_h_ypos       !10^6 x (position vector y component in
C!                                           kilometers)
      INTEGER*4   mhs_h_zpos       !10^6 x (position vector z component in
C!                                           kilometers)
      INTEGER*4   mhs_h_xvel       !10^8 x (velocity vector x-dot component
C!                                           in kilometers/second)
      INTEGER*4   mhs_h_yvel       !10^8 x (velocity vector y-dot component
C!                                           in kilometers/second)
      INTEGER*4   mhs_h_zvel       !10^8 x (velocity vector z-dot component
C!                                           in kilometers/second)
      INTEGER*4   mhs_h_earthsun   !10^6 x (earth/sun distance ratio
C!                                  relative to the mean distance of 1AU)
      INTEGER*4   mhs_h_filler9(4)
C!
C!......Temperature, current and resistance conversion coeffs
C!
      INTEGER*4 mhs_h_therm(5)     !5 coefficients for the 24 thermistors
      INTEGER*4 mhs_h_filler10(4)
      INTEGER*4 mhs_h_currcoef(12) !Current sensors
C!                            2 coefficients for each of 6 sensors
      INTEGER*4 mhs_h_prtcoef(40)  !Cal target PRTs: resistance to temperature
C!                            For PIE-A and PIE-B,
C!                              4 coefficients for each of 5 PRTs
      INTEGER*4 mhs_h_survival(6)  !Survival volts to temperature conversion
C!                            Same coefficients for all 3 sensors
C!                            Note 1 volt = 0.02 counts
      INTEGER*4 mhs_h_antpos       !Antenna degrees per count
C!                            Value = 10^8 x 0.00703125
      INTEGER*4 mhs_h_refres(6)    !10^4 x reference resistance in ohms
C!                                 PIE-A 3 channels
C!                                 PIE-B 3 channels
C!......Thresholds
C!
      INTEGER*2 mhs_h_moonthresh   !100 x lunar angle threshold in degrees
C!
C!......RFI Correction, as for AMSU-B, or zero fill if not needed
C!      
      INTEGER*2 mhs_h_CountsCorr(5,21,4)  !Normal bias, ordered by
C!                                         channel, FOV and transmitter
      INTEGER*2 mhs_h_filler11(4)
      INTEGER*2 mhs_h_txpow(4)           !10*Mean power in counts
      INTEGER*2 mhs_h_filler12(5)
C!
C!......METOP manoeuvres identification
C!
      INTEGER*2 mhs_h_startmanyr        !4 digit year
      INTEGER*2 mhs_h_startmandy        !Day of year
      INTEGER*4 mhs_h_startmantime      !Time of day in milliseconds
      INTEGER*2 mhs_h_endmanyr
      INTEGER*2 mhs_h_endmandy
      INTEGER*4 mhs_h_endmantime
      INTEGER*4 mhs_h_deltav(3)         !Change in velocity
      INTEGER*4 mhs_h_mass(2)           !Spacecraft mass before and after
      
      INTEGER*2 mhs_h_filler13(642)
C!
      INTEGER*2 mhs_hd(cmhs_size)
      EQUIVALENCE(mhs_hd(1),mhs_h_siteid)
C!
C!-------------------------------------------------------------------------
      COMMON/mhs1BHD/mhs_h_siteid,mhs_h_blank,
     &       mhs_h_l1bversnb,mhs_h_l1bversyr,mhs_h_l1bversdy,
     &       mhs_h_reclg,mhs_h_blksz,mhs_h_hdrcnt,
     &       mhs_h_filler1,mhs_h_dataname,mhs_h_prblkid,mhs_h_satid,
     &       mhs_h_instid,mhs_h_datatyp,mhs_h_tipsrc,
     &       mhs_h_startdatajd,mhs_h_startdatayr,mhs_h_startdatady,
     &       mhs_h_startdatatime,mhs_h_enddatajd,mhs_h_enddatayr,
     &       mhs_h_enddatady,mhs_h_enddatatime,mhs_h_cpidsupdyr,
     &       mhs_h_cpidsupddy,mhs_h_fov1offset,mhs_h_filler2,
     &       mhs_h_inststat1,mhs_h_filler3,mhs_h_statchrecnb,
     &       mhs_h_inststat2,mhs_h_scnlin,mhs_h_callocsclin,
     &       mhs_h_misscnlin,mhs_h_datagaps,mhs_h_moonscnlin,
     &       mhs_h_okdatafr,
     &       mhs_h_pacsparityerr,mhs_h_auxsyncerrsum,mhs_h_timeseqerr,
     &       mhs_h_timeseqerrcode,mhs_h_socclockupind,mhs_h_locerrind,
     &       mhs_h_locerrcode,mhs_h_pacsstatfield,mhs_h_pacsdatasrc,
     &       mhs_h_spare2,mhs_h_spare3,mhs_h_filler4,mhs_h_tsensid,
     &       mhs_h_filler5,mhs_h_reftemp,mhs_h_bias,
     &       mhs_h_nonlin,mhs_h_filler6,mhs_h_temrad,mhs_h_filler7,
     &       mhs_h_modelid,mhs_h_nadloctol,mhs_h_locbit,mhs_h_filler8,
     &       mhs_h_rollerr,mhs_h_pitcherr,mhs_h_yawerr,mhs_h_epoyr,
     &       mhs_h_epody,mhs_h_epotime,mhs_h_smaxis,
     &       mhs_h_eccen,mhs_h_incli,mhs_h_argper,mhs_h_rascnod,
     &       mhs_h_manom,mhs_h_xpos, mhs_h_ypos,mhs_h_zpos,
     &       mhs_h_xvel,mhs_h_yvel,mhs_h_zvel,mhs_h_earthsun,
     &       mhs_h_filler9,mhs_h_therm,mhs_h_filler10,
     &       mhs_h_currcoef,mhs_h_prtcoef,mhs_h_survival,mhs_h_antpos,
     &       mhs_h_refres,mhs_h_moonthresh,
     &       mhs_h_countscorr,mhs_h_filler11,mhs_h_txpow,mhs_h_filler12,
     &       mhs_h_startmanyr,mhs_h_startmandy,mhs_h_startmantime,
     &       mhs_h_endmanyr,mhs_h_endmandy,mhs_h_endmantime,
     &       mhs_h_deltav,mhs_h_mass,mhs_h_filler13
!
!##############################################################################
!
!            SINGLE  MHS DATA RECORD
!
!##############################################################################
C!
C!......Scan line information
C!
      INTEGER*2 mhs_scnlin,mhs_scnlinyr,mhs_scnlindy,mhs_clockdrift
      INTEGER*4 mhs_scnlintime
      INTEGER*2 mhs_scnlinbit
C!                bit 15: 0=northbound, 1=southbound
C!                bit 14: 0=not corrected, 1=scan time corrected for clock drift
      INTEGER*2 mhs_tipmf         !major frame count
      INTEGER*4 mhs_coarsetime    !Coarse MHS on-board time (OBT)
C!                                 Time since last reset to zero.
      INTEGER*2 mhs_finetime      !Fine MHS OBT (fraction of second since last
C!                                 increment of coarse time.
C!                                 Res 2^-16 sec, ramge 0-65535
      CHARACTER*1 mhs_modeflg     !MHS Mode Flag
C!                        3=scan, 4=fixed view. Rest are empty science data.     
      CHARACTER*1 mhs_filler1
C!
C!......Quality Indicators
C! 
      INTEGER*4 mhs_qualind
C!         bit 31: do not use for product generation
C!         bit 30: time sequence error detected within this scan
C!         bit 29: data gap precedes this scan
C!         bit 28: insufficient data for calibration
C!         bit 27: earth location data not available
C!         bit 26: first good time following clock update
C!         bit 25: instrument status changed with this scan
C!         bit 24-5: <zero fill>
C!         bit 4: transmitter status change occurred (probably irrelevant)
C!         bit 3: AMSU sync error detected
C!         bit 2: AMSU minor frame error detected
C!         bit 1: AMSU major frame error detected
C!         bit 0: AMSU parity error detected
      INTEGER*4 mhs_linqualflgs
C!         bit 31: time field bad; probably inferred from previous good time
C!         bit 30: time field bad; can't be inferred from previous good time
C!         bit 29: time discontinuity
C!         bit 28: repeats scan times previously accepted
C!         bit 27-19: <zero fill>
C!         bit 18: not calibrated because of satellite manoeuvre (Metop)
C!         bit 17: One of more space views moon contaminated
C!         bit 16: moon-contaminated scan able to be calibrated
C!         bit 15: Not calibrated because of bad time
C!         bit 14: Calibrated using fewer than preferred no of scan lines
C!         bit 13: Not calibrated because of bad or insufficient PRT data
C!         bit 12: Calibrated with marginal PRT data
C!         bit 11: Some uncalibrated channels
C!         bit 10: Uncalibrated due to instrument mode
C!         bit 9: Questionable calibration: antenna position error space view
C!         bit 8: Questionable calibration: antenna position error OBCT view
C!         bit 7: Not earth located because of bad time
C!         bit 6: Questionable earth location: questionable time code
C!         bit 5: Questionable earth location: marginal reasonableness check
C!         bit 4: Questionable earth location: fails reasonableness check
C!         bit 3: Questionable earth location: antenna position check
C!         bit 2: <zero fill>
C!         bit 1: Not earth located: satellite in-plane manoeuvre (Metop)
C!         bit 0: Not earth located: satellite out-of-plane manoeuvre (Metop)
      INTEGER*2 mhs_chqualflg(5)
C!         bits 15-7: <zero fill>
C!         bit 6: last scan before, or first after, calibration jump 
C!         bit 5: all bad OBCT view counts 
C!         bit 4: all bad space view counts 
C!         bit 3: all bad PRTs 
C!         bit 2: some bad OBCT view counts 
C!         bit 1: some bad space view counts 
C!         bit 0: some bad PRT temps 
      INTEGER*2 mhs_filler2(9)
C!
C!......calibration coeffs
C!
      INTEGER*4 mhs_pmcal(15)   !a2, a1, a0 for 5 channels
C!                               scaling 10^16, 10^10, 10^6
      INTEGER*4 mhs_sdcal(15)
      INTEGER*2 mhs_filler3(2)
C!
C!.......navigation
C!
      INTEGER*2  mhs_yawsteering(3) !computed yaw steering angles for METOP
                                    !Yaw, pitch, roll (degrees)
      INTEGER*2  mhs_totattcorr(3)  !Total applied attitude correction for METOP
                                    !1000* Yaw, pitch, roll (degrees)
      INTEGER*4  mhs_navstat        !navigation status bit field
!         bit 31-21: <zero fill>
!         bit 20-19: METOP yaw steering usage indicator 
!                    0=no corr
!                    1=computed parameters from METOP data stream
!                    2=measured parameters from METOP data stream 
!                    3=computed parameters from AELDS
!         bit 18:    METOP manoeuvre occuring (1=true)
!         bit 17:    Nadir Earth location OK (see mhs_h_nadloctol) (1=true) 
!         bit 16:    earth location corrected for Euler angles (1=true)
!         bits15-12: earth location indicator
!                    0=earth location available
!                    1=user ephemerics files greater than 24 hours old
!                    2=no earth location available
!         bits11-08: spacecraft attitude control
!                    0=operating in YGC or nominal mode
!                    1=operating in another mode
!                    2=attitude exceeds nominal tolerance
!                    3=both 1 and 2
!         bits07-04: attitude SMODE
!                    0=nominal mode
!                    1=rate nulling mode
!                    2=YGC mode
!                    3=search mode
!                    4=coast mode
!         bits03-00: attitude PWTIP$AC
!                    0=nominal mode/no test
!                    1=yaw axis test in progress
!                    2=roll axis test in progress
!                    3=pitch axis test in progress
      INTEGER*4  mhs_attangtime     !time associated with Euler angles (seconds)
      INTEGER*2  mhs_rollang        !10^3*Roll angle in degrees
      INTEGER*2  mhs_pitchang       !10^3*Pitch angle in degrees
      INTEGER*2  mhs_yawang         !10^3*Yaw angle in degrees
      INTEGER*2  mhs_scalti         !10*spacecraft altitude (MSL) in km
      INTEGER*2  mhs_ang(270) !scan angles in degrees
!                     word1: 100*solar zenith angle, point 1
!                     word2: 100*satellite zenith angle, point 1
!                     word3: 100*satellite azimuth angle, point 1
!                     word4: 100*solar zenith angle, point 2
!                     word5: 100*satellite zenith angle, point 2
!                     word6: 100*satellite azimuth angle, point 2
!                     ......
!                     word268: 100*solar zenith angle, point 90
!                     word269: 100*satellite zenith angle, point 90
!                     word270: 100*satellite azimuth angle, point 90
      INTEGER*4  mhs_pos(180) !Earth location
!                     (north latitude and east longitude are positive)
!                     word1: 10^4*latitude in degrees, point 1
!                     word2: 10^4*longitude in degrees, point 1
!                     word3: 10^4*latitude in degrees, point 2
!                     word4: 10^4*longitude in degrees, point 2
!                     ......
!                     word180: 10^4*longitude in degrees, point 90
      INTEGER*2  mhs_moonang(4)  !100*angle between moon and each space view
C!      
C!.......MHS sensor data
C!
      INTEGER*2 mhs_earth_data(540)  !90 x [position, counts(H1-H5)]
      INTEGER*4 mhs_filler4(2)
C!      
C!.......MHS calibration data
C!      
      INTEGER*2 mhs_coldcal(24)   !4 x [position, counts(H1-H5)]
      INTEGER*2 mhs_warmcal(24)   !4 x [position, counts(H1-H5)]
      INTEGER*4 mhs_filler5(2)
C!
C!.......Position validity flags
C!
      CHARACTER*1 mhs_earthval(12)   !0= position OK. 1 bit per FOV  
C!                                     FOV1-8   = Word 1 bits 0-7, etc.
C!                                     FOV89-90 = Word 12 bits 0-1
      CHARACTER*1 mhs_spaceval       !bits 0-3
      CHARACTER*1 mhs_targval        !bits 0-3
C!      
C!.......Full housekeeping data 
C!      
      CHARACTER*1 mhs_mode
C!                     bits 7-4: 3=scan, 4=fixed view
C!                     bit 3: 0=PIE-A, 1=PIE-B
C!                     bits 2-0: Subcomm code (for telemetry packet data)
      CHARACTER*1 mhs_fault(5)
C!        Word 1-4 Telecommand acknowledgement
C!        Word 5 Fault code:
C!          bits 7-3: Fault in PSU current, thermistor, switch, processor, motor
C!          bit 2: DC offset error (change in DC offset required)
C!          bit 1: scan control error
C!          bit 0: ref clock error
      CHARACTER*1 mhs_switchstat(3)
C!        Word 1:
C!          bit 7: H4 backend (0=off, 1=on)
C!          bit 6: H3 backend (0=off, 1=on)
C!          bit 5: H3/H4 LO selected (0=A, 1=B)
C!          bit 4: H3/H4 front-end (0=off, 1=on)
C!          bit 3: H2 LO selected (0=A, 1=B)
C!          bit 2: H2 front-end (0=off, 1=on)
C!          bit 1: H1 LO selected (0=A, 1=B)
C!          bit 0: H1 front-end (0=off, 1=on)
C!        Word 2:
C!          bit 7: PROM (1=failed ON)
C!          bit 6: sig proc / scan electronics (0=off, 1=on)
C!          bit 5: Aux heaters (0=off, 1=on)
C!          bit 4: Scan mech operational heaters (0=off, 1=on)
C!          bit 3: Receiver operational heaters (0=off, 1=on)
C!          bit 2: Rx CV (0=off, 1=on)
C!          bit 1: H5 LO selected (0=A, 1=B)
C!          bit 0: H5 receiver (0=off, 1=on)
C!        Word 3: Flywheel Drive Mechanism (FDM) and Reflector Drive Mech (RDM)
C!          bit 7/6: FDM/RDM motor current trip status (0=enabled, 1=disabled)
C!          bit 5/4: FDM/RDM motor supply (0=off, 1=on)
C!          bit 3/2: FDM/RDM motor sensors selected (0=A, 1=B)
C!          bit 1/0: FDM/RDM zero position sensors (0=A, 1=B)
!
      CHARACTER*1 mhs_temps(24)  !Range 0-255 for 24 sensors
C!        Word 1-4: LO temps H1, H2, H3/H4, H5
C!        Word 5-8: Mixer temps H1, H2, H3/H4, H5
C!        Word 9-10: Quasi optic baseplate #1, #2
C!        Word 11-12: IF baseplate #1, #2
C!        Word 13-14: Scan mechanism core, housing
C!        Word 15-16: RDM, FDM Scan Sensor Head Module
C!        Word 17-19: Structure 1, 2, 3
C!        Word 20: Processor module
C!        Word 21: Main DC/DC convertor
C!        Word 22-23: Scan Control Electronics RDM, FDM
C!        Word 24: RF DC/DC convertor

      CHARACTER*1 mhs_current(6)
      CHARACTER*1 mhs_filler6
C!      
C!.......Status word 
C!      
      CHARACTER*1 mhs_stat
C!          bit 7: DC offset valid (1=all channels OK)
C!          bit 6: Scan control valid (1=all views OK)
C!          bit 5-4: Scan profile (0=nominal, 1/2=alternate, 3=manual)
C!          bit 3-0: <unused>
      CHARACTER*1 mhs_filler7(7)
C!      
C!.......Signal processing status 
C!      
      CHARACTER*1 mhs_dcoffset(5)   !Channels H1 to H5
      CHARACTER*1 mhs_chvalflgs
C!          bits 7-3: H1-H5 valid (all samples within ADC dynamic range)
C!          bits 2-0: SPE MUX code
      CHARACTER*1 mhs_chgains(3)     !0 to 3dB
C!        Word 1: H1 & H2 (bits 7-5 & 4-2)
C!        Word 2: H3 & H4 (bits 7-5 & 4-2)
C!        Word 3: H5 (bits 7-5)
      CHARACTER*1 mhs_filler8(7)
C!      
C!.......OBCT temperature data 
C!      
      INTEGER*2 mhs_obctprt(5)    !PRT counts
      INTEGER*2 mhs_refrescts(3)  !Reference resistor counts (upper, mid, lower)
      INTEGER*2 mhs_filler9
      INTEGER*4 mhs_obcttemp(5)   !Computed temperatures (mK)
      CHARACTER*1 mhs_filler10(46)
C!      
C!.......MHS discrete telemetry (digb/analog)
C!
      CHARACTER*1 mhs_discrete(6)!6 status words, 1 or 0
      INTEGER*2 mhs_survtemp(3)  !3 Survival temps (receiver, electronics, scan)
      INTEGER*2 mhs_stx(9)       !STX 1-4 status, STX1-3 power, SARR A&B power
C!      
      INTEGER*4 mhs_ahkval      !bits 0-17: 1 if item is up to date
      INTEGER*2 mhs_filler11(102)
C!      
      INTEGER*2 mhs_dt(cmhs_size)
      EQUIVALENCE(mhs_dt(1),mhs_scnlin)
C!
      COMMON/mhs1BDT/mhs_scnlin,mhs_scnlinyr,mhs_scnlindy,
     &        mhs_clockdrift,mhs_scnlintime,mhs_scnlinbit,mhs_tipmf,
     &        mhs_coarsetime,mhs_finetime,mhs_modeflg,
     &        mhs_filler1,mhs_qualind,mhs_linqualflgs,
     &        mhs_chqualflg,mhs_filler2,mhs_pmcal,mhs_sdcal,mhs_filler3,
     &        mhs_yawsteering,mhs_totattcorr,
     &        mhs_navstat,mhs_attangtime,mhs_rollang,mhs_pitchang,
     &        mhs_yawang,mhs_scalti,mhs_ang,mhs_pos,mhs_moonang,
     &        mhs_earth_data,mhs_filler4,mhs_coldcal,mhs_warmcal,
     &        mhs_filler5,mhs_earthval,mhs_spaceval,mhs_targval,
     &        mhs_mode,mhs_fault,mhs_switchstat,mhs_temps,mhs_current,
     &        mhs_filler6,mhs_stat,mhs_filler7,mhs_dcoffset,
     &        mhs_chvalflgs,mhs_chgains,mhs_filler8,mhs_obctprt,
     &        mhs_refrescts,mhs_filler9,mhs_obcttemp,mhs_filler10,
     &        mhs_discrete,mhs_survtemp,mhs_stx,mhs_ahkval,mhs_filler11
!-------------------------------------------------------------------------------
!                         -- CVS MODIFICATION HISTORY --
!-------------------------------------------------------------------------------
!$Log: mhs1b.h,v $
!Revision 1.3  2005/04/15 13:39:48  frna
!Merge AAPP5Dev with main trunk
!
!Revision 1.2.2.3  2004/11/26 11:43:26  frna
!Correct typo
!
!Revision 1.2.2.2  2004/11/10 09:43:50  frna
!Conform to NOAA 1b release dated Sept 27 2004

