!+
!$Id: msu1b.h,v 1.2 2005/04/15 13:39:48 frna Exp $
! 
!DESCRIPTION 
!      include file for msu 1b format handling
!
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
! Marcel Derrien
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
!  1.0     03/95     initial            Marcel Derrien
!  2.0     07/11/95  new release from NOAA available 30/10/95
!                    still problems in integer boudaries alignments
!                    and undefined scaling factors
!  3.0     11/05/98  modified to conform to new NOAA structures
!                    general information for header
!                    navigation for header
!                    general information for data record
!                    navigation for data record
!  3.1     06/12/01  modified comments for msu_h_tipsrc to be coherent 
!                    with the AAPP code and with the other xxx_h_tipsrc
! 
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!!###########################################################################
!
!               MSU Level 1B Data Set 
!
!###########################################################################
!
       integer   msu_mxchn              !count of MSU channels
       parameter(msu_mxchn=4)
       integer reclmsu            ! 1B record length ( bytes)
       parameter (reclmsu = 1024)
       integer cmsu_size           ! 1B record length ( number of integer  )
       parameter ( cmsu_size = reclmsu/4)
!
!###########################################################################
!
!               HEADER RECORD
!
!###########################################################################
!
!               GENERAL INFORMATION
!
       character*3  msu_h_siteid        !Data set creation site ID
       character*1  msu_h_blank         !ASCII blank
       integer*2    msu_h_l1bversnb     !level 1b format version number
       integer*2    msu_h_l1bversyr     !level 1b format version year
       integer*2    msu_h_l1bversdy     !level 1b format version day of year
       integer*2    msu_h_reclg         !reserved for logical record length
       integer*2    msu_h_blksz         !reserved for block size
       integer*2    msu_h_hdrcnt        !count of header records in data set
       integer*2    msu_h_filler0(3)
       character*42 msu_h_dataname      !data set name
       character*8  msu_h_prblkid       !processing block identification
       integer*2    msu_h_satid         !NOAA spacecraft identification code
       integer*2    msu_h_instid        !MSU instrument identification
       integer*2    msu_h_datatyp       !data type code
!                   (1 = LAC; 2 = GAC; 3 = HRPT; 4 = TIP; 5 = HIRS; 6 = MSU;
!                    7 = SSU; 8 = DCS; 9 = SEM; 10 = AMSU-A; 11 = AMSU-B)
       integer*2    msu_h_tipsrc        !TIP source code
!                   (0 = not applicable; 1 = GAC embedded; 2 = stored;
!                    3 = third CDA; 4 = HRPT embedded)
       integer      msu_h_startdatajd   !start of data set julian day
       integer*2    msu_h_startdatayr   !start of data set year
       integer*2    msu_h_startdatady   !start of data set day of year
       integer      msu_h_startdatatime !start of data set UTC time of day in
!                                        milliseconds
       integer      msu_h_enddatajd     !end of data set julian day
       integer*2    msu_h_enddatayr     !end of data set year
       integer*2    msu_h_enddatady     !end of data set day of year
       integer      msu_h_enddatatime   !end of data set UTC time of day in
!                                        milliseconds
       integer*2    msu_h_cpidsyr       !year of Last CPIDS Update
       integer*2    msu_h_cpidsdy       !Day of Year of Last CPIDS Update
       integer*2    msu_h_filler1(4)    !filler

!
!               DATA SET QUALITY INDICATORS
!                         
       integer*4   msu_h_inststat1      !instrument status (TBD)
       integer*2   msu_h_filler2
       integer*2   msu_h_statchrecnb    !record number of status change 
!                  ( if 0, none occurred)
       integer*4   msu_h_inststat2      !second instrument status 
!                  ( if previous word is 0, no change)
       integer*2   msu_h_scnlin         !count of scan lines in this data set
       integer*2   msu_h_callocsclin    !count of calibrated, earth located
!                                        scan lines in this data set
       integer*2   msu_h_misscnlin      !count of missing scan lines
       integer*2   msu_h_datagaps       !count of data gaps in this data set
       integer*2   msu_h_okdatafr       !count of data frames without frame 
!                                        sync word errors
       integer*2   msu_h_pacsparityerr  !count of PACS detected TIP parity
!                                        errors
       integer*2   msu_h_auxsyncerrsum  !sum of all auxiliary sync errors  
!                                        detected in the input data
       integer*2   msu_h_timeseqerr     !time sequence error
!                (0 =none; otherwise the record number of the first occurrence)
       integer*2   msu_h_timeseqerrcode !time sequence error code 
       integer*2   msu_h_socclockupind  !socc clock update indicator
!                  ( 0 = none during this orbit;
!                    otherwise the record number of the first occurrence)
       integer*2   msu_h_locerrind      !earth location error indicator
!                  ( 0 = none during this orbit;
!                    otherwise the record number of the first occurrence)
       integer*2   msu_h_locerrcode     !earth location error code
       integer*2   msu_h_pacsstatfield     !PACS status bit field
!                  bit 15-3 : spare
!                  bit  2   : pseudo noise (0 = normal data; 1 = P/N data)
!                  bit  1   : tape direction (0 = time decrementing)
!                  bit  0   : data mode (o = test data; 1 = flight data)
       integer*2   msu_h_pacsdatasrc       !PACS data source 
!                  (0 = unused; 1 = Gilmore; 2 = Wallops; 3 = SOCC)
       integer      msu_h_filler3
       character*8 msu_h_spare1            !spare (reserved for ingester)
       character*8 msu_h_spare2            !spare (reserved of the
!                                           decommutation)
       integer*2 msu_h_filler4(5)          !filler

!               CALIBRATION 
!
       integer*2 msu_h_racalind    !ramp/auto calibration indicators bit field
       integer   msu_h_calinf(4,msu_mxchn) !Calibration information
!                for nochannel 1 to 4 :
!                    ?? x  (mean calibration slope nochannel)
!                    ?? x  (standard deviation calibration slope nochannel)
!                    ?? x  (b-sub-1 nochannel)  
!                    ?? x  (standard deviation of linear regression 
!                           for b-sub-1 nochannel)
 

!                 RADIANCE TO TEMPERATURE CONVERSION

!               corrections for calibration (filled after MSUCL)
!
       integer   msu_h_nolincor(3,msu_mxchn) !nonlinearity correction
!                                             coefficients
!                for nochannel 1 to 4 :
!                    10^12 x (second order term nochannel (zero in 1A data))
!                    10^6  x ( first order term nochannel (zero in 1A data))
!                    10^6  x (zeroth order term nochannel (zero in 1A data))
       integer   msu_h_prttores(2)          !PRT count to resistance
!                                            coefficients
!                ?? x (first one  K0 (Ohm) (zero in 1A data))
!                ?? x (second one K1 (Ohm) (zero in 1A data)) 
       integer   msu_h_iwttotemp(3,4)       !Internal Warm Target PRT Resistance !                                            to Temperature (K) coefficients
!                for term in PRT 1A, PRT 2A, PRT 1B,PRT 2B :
!                    10^12 x (second order term (zero in 1A data))
!                    10^6  x ( first order term (zero in 1A data))
!                    10^6  x (zeroth order term (zero in 1A data)       
       integer   msu_h_censperes(msu_mxchn)      !centroid of spectral response
!                                                 functions
!                for nochannel 1 to 4 :
!                    10^4 x (nochannel (cm-1)(zero in 1A data))         
       integer   msu_h_radspa(msu_mxchn)   !Radiance of space 
!                for nochannel 1 to 4 :
!                    10^6 x (radiance of space nochannel(cm-1)(zero in 1A data))
!
       integer*4   msu_h_filler5(3)    !filler

!
!               NAVIGATION
!
       character*8 msu_h_modelid
       integer*2   msu_h_nadloctol  !10 x (nadir earth location tolerance in
!                                          kilometers)
       integer*2   msu_h_locbit     !earth location bit field
!                  bit 0 : attitude error correction (0 = not corrected)
       integer*2   msu_h_filler6
       integer*2   msu_h_rollerr    !10^3 x (constant roll attitude error in
!                                            degrees)
       integer*2   msu_h_pitcherr   !10^3 x (constant pitch attitude error in
!                                            degrees)
       integer*2   msu_h_yawerr     !10^3 x (constant yaw attitude error in
!                                            degrees)
       integer*2   msu_h_epoyr      !epoch year for orbit vector
       integer*2   msu_h_epody      !day of epoch year for orbit vector
       integer     msu_h_epotime    !epoc UTC time of day in milliseconds for
!                                    orbit vector
       integer     msu_h_smaxis     !10^5 x (semi-major axis in kilometers)
       integer     msu_h_eccen      !10^8 x (eccentricity)
       integer     msu_h_incli      !10^5 x (inclination in degrees)
       integer     msu_h_argper     !10^5 x (argument of perigee in degrees)
       integer     msu_h_rascnod    !10^5 x (right ascension of the ascending 
!                                            node in degrees)
       integer     msu_h_manom      !10^5 x (mean anomaly in degrees)
       integer     msu_h_xpos       !10^5 x (position vector x component in
!                                            kilometers)
       integer     msu_h_ypos       !10^5 x (position vector y component in
!                                            kilometers)
       integer     msu_h_zpos       !10^5 x (position vector z component in
!                                            kilometers)
       integer     msu_h_xvel       !10^8 x (velocity vector x-dot component 
!                                           in kilometers/second)
       integer     msu_h_yvel       !10^8 x (velocity vector y-dot component
!                                            in kilometers/second)
       integer     msu_h_zvel       !10^8 x (velocity vector z-dot component 
!                                            in kilometers/second)
       integer     msu_h_earthsun   !10^6 x (earth/sun distance ratio)
       integer*2 msu_h_filler7(272)
!
!               DIGITAL A CONVERSION
!
!      conversion coeffs for MSU PRTs/volts etc ...
!      lannion to specify (zero in 1A data)           TBD
!

       integer   msu_hd(cmsu_size)
       
       equivalence(msu_hd(1),msu_h_siteid)
       
       common/msu1bhd/ msu_h_siteid , msu_h_blank ,
     &   msu_h_l1bversnb , msu_h_l1bversyr , 
     &   msu_h_l1bversdy , msu_h_reclg , msu_h_blksz ,
     &   msu_h_hdrcnt , msu_h_filler0 , msu_h_dataname , 
     &   msu_h_prblkid , msu_h_satid , msu_h_instid , msu_h_datatyp , 
     &   msu_h_tipsrc  , msu_h_startdatajd , 
     &   msu_h_startdatayr , msu_h_startdatady , msu_h_startdatatime , 
     &   msu_h_enddatajd , msu_h_enddatayr , msu_h_enddatady , 
     &   msu_h_enddatatime  ,  msu_h_cpidsyr  ,  msu_h_cpidsdy  , 
     &   msu_h_filler1 , 
     &   msu_h_inststat1 , msu_h_filler2 , 
     &   msu_h_statchrecnb , msu_h_inststat2 , 
     &   msu_h_scnlin , msu_h_callocsclin , msu_h_misscnlin , 
     &   msu_h_datagaps , msu_h_okdatafr , msu_h_pacsparityerr , 
     &   msu_h_auxsyncerrsum , msu_h_timeseqerr , msu_h_timeseqerrcode , 
     &   msu_h_socclockupind , msu_h_locerrind , msu_h_locerrcode , 
     &   msu_h_pacsstatfield , msu_h_pacsdatasrc ,  msu_h_filler3 ,
     &   msu_h_spare1 , msu_h_spare2 ,
     &   msu_h_filler4 ,
     &   msu_h_racalind , msu_h_calinf ,
     &   msu_h_nolincor , msu_h_prttores , msu_h_iwttotemp ,
     &   msu_h_censperes , msu_h_radspa , msu_h_filler5 ,
     &   msu_h_modelid ,  
     &   msu_h_nadloctol , msu_h_locbit , msu_h_filler6 ,
     &   msu_h_rollerr ,  msu_h_pitcherr , msu_h_yawerr , msu_h_epoyr , 
     &   msu_h_epody  , msu_h_epotime , msu_h_smaxis , 
     &   msu_h_eccen , msu_h_incli , msu_h_argper , msu_h_rascnod , 
     &   msu_h_manom , msu_h_xpos ,  msu_h_ypos , msu_h_zpos , 
     &   msu_h_xvel , msu_h_yvel , msu_h_zvel , msu_h_earthsun , 
     &   msu_h_filler7 
!
!##############################################################################
!
!               DATA RECORD 
!
!##############################################################################
!
       integer*2 msu_scnlin                !scan line number
       integer*2 msu_scnlinyr              !scan line year
       integer*2 msu_scnlindy              !scan line day of year
       integer*2 msu_clockdrift            !satellite clock drift delta in ms
       integer   msu_scnlintime            !scan line UTC time of day in ms
       integer*2 msu_scnlinbit               !scan line bit field
!              bit 15: (0 = ascending data; 1 = descending data)
!              bit 14: (1 = scan time corrected for clock drift)
!              bit 13: (1 = earth location corrected for TIP attitude)
      integer*2   msu_mjfrcnt      !major frame count
      integer*2   msu_scnpos       !scan position number in 32 seconds 
      integer*2   msu_filler1(5)
!
!                QUALITY INDICATORS
!  

       integer   msu_qualind               ! quality indicator bit field  
!                in all of the following if the bit is on (i.e., if it is set
!                to 1) then the statement is true. Otherwise it is false.          !
!                General
!
!                bit 31: do not use scan for product generation
!                bit 30: time sequence error detected with this scan (see below)
!                bit 29: data gap precedes this scan
!                bit 28: no calibration (see below)
!                bit 27: no earth location (see below)
!                bit 26: first good time following a clock update
!                bit 25: instrument status changed with this scan
!                bit 24: spare
!
!                Time Problem Code
!
!                (All bits off implies the scan time is as expected.)
!                bit 23: time field is bad but can probably be inferred from
!                        the previous good time.
!                bit 22: time field is bad and can't be inferred from the
!                        previous good time.
!                bit 21: this record starts a sequence that is inconsistent
!                        with previous times (i.e., there is a time
!                        discontinuity).  This may or may not be associated
!                        with a spacecraft clock update. (See bit 26 above)
!                bit 20: start of a sequence that apparently repeats scan 
!                        times that have been previously accepted.
!
!                TIP Problem Code
! 
!                bit 19: TIP synch error
!                bit 18: TIP minor or major frame sequence error
!                bit 17: TIP status error 
!                bit 16: TIP parity error 
!
!                Calibration Problem Code
!
!                (Note these bits compliment the channel indicators; all bits
!                set to 0 indicates normal calibration. ) 
!                bit 15: Scan line was not calibrated because of bad time.
!                bit 14: Scan line was calibrated using fewer than the
!                        preferred number of scan lines because of proximity 
!                        to start or end of data set or to a data gap.
!                bit 13: Scan line was not calibrated because of bad or
!                        insufficient PRT data.
!                bit 12: Scan line was calibrated but with marginal PRT data.
!                bit 11: Some uncalibrated channels on this scan. (See channel
!                        indicators.)
!                bit 10: Uncalibrated due to instrument mode.
!                bit 09: spare
!                bit 08: spare
!
!                Earth Location Problem Code
!
!               (all bits set to 0 implies the earth location was normal)
!               bit 07: Not earth located because of bad time.
!               bit 06: Earth location questionable because of questionable
!                       time code. (See time problem flags above.)
!               bit 05: Earth location questionable -- only marginal agreement
!                       with reasonableness check.
!               bit 04: Earth location questionable -- fails reasonableness
!                       check.
!               bit 03-00: spare
!
       integer*2 msu_scnlinqual(msu_mxchn)   !quality flag
!              for nochannel 1 to 4 :
!                  (all bits off implies a good calibration)
!                  bit 7: spare
!                  bit 6: spare
!                  bit 5: No good blackbody counts for scan line
!                  bit 4: No good space view counts for scan line 
!                  bit 3: No good PRTs for this line
!                  bit 2: Some bad blackbody view counts for this line
!                  bit 1: Some bad space view counts for this line
!                  bit 0: Some bad PRT temps on this line
       integer*2 msu_filler2(8) !
!               CALIBRATION COEFFICIENTS
!
       integer   msu_pmcal(3,msu_mxchn)       !primary calibration
!              for nochannel 1 to 4 :
!                  ?? x (chn nochannel second order term, a2)
!                  ?? x (chn nochannel  first order term, a1)
!                  ?? x (chn nochannel zeroth order term, a0)
       integer   msu_sdcal(3,msu_mxchn)       !secondary calibration 
!              for nochannel 1 to 4 :
!                  ?? x (chn nochannel second order term, a2)
!                  ?? x (chn nochannel  first order term, a1)
!                  ?? x (chn nochannel zeroth order term, a0)

!
!               NAVIGATION
!
       integer  msu_navstat      ! Navigation Status Bit Field
       integer msu_attangtime   !Time associated with attitude angles(seconds)
       integer*2 msu_rollang    !10^3 xRoll Angle in Degrees
       integer*2 msu_pitchang   !10^3 xPitch Angle in Degrees
       integer*2 msu_yawang     !10^3 xYaw Angle in Degrees
!

!             bits 15 - 12: earth location indicator 
!                     (0 = earth location available; 
!                      1 = user ephemeris files greater than 24 hours old;
!                      2 = no earth location available)

!             bits 11 - 8: spacecraft attitude control 
!                     (0 = operating inYGC or NOMINAL mode;
!                      1 = operating in another mode;
!                      2 = attitude exceeds nominal tolerance;
!                      3 = both 1 and 2)

!            bits 7 - 4: attitude SMODE 
!                     (0 = NOMINAL mode;
!                      1 = rate nulling mode;
!                      2 = YGC mode; 
!                      3 = search mode;
!                      4 = coast mode)
!            bits 3 - 0: attitude PWT1P$AC 
!                     (0 = NOMINAL mode/no test; 
!                      1 = yaw axis test in progress;
!                      2 = roll axis test in progress; 
!                      3 = pitch axis test in progress)
!
       integer*2 msu_scalti      ! Spacecraft Altitude (MSL) in km     
!                                  in this scan line)
       integer*2  msu_ang(3,11)    !set of 3 angles in degrees 
!                first angle  : 10^2 x (solar zenith angle )
!                second angle : 10^2 x (satellite zenith angle )
!                third angle  : 10^2 x (relative azimuth angle )
       integer*2 msu_filler3
       integer msu_pos(2,11)    !lat/lon pair in degrees for point 1
!                                  to point 11 every point
!                first  : 10^4 x (latitude )
!                second : 10^4 x (longitude )
!            (North latitude and East longitude are positive)

       integer*2 msu_filler4(8)      !filler
!
!               MSU DATA (as from TIP minor frame)
!
       integer*2 msu_data(112)   !First to last MSU words for scan line
!
!               FILLER
!
       integer*2 msu_filler5(230) !filler

       integer   msu_dt(cmsu_size)
       equivalence(msu_dt(1),msu_scnlin)

       common/msu1bdt/
     &  msu_scnlin , msu_scnlinyr , msu_scnlindy , msu_clockdrift ,
     &  msu_scnlintime , msu_scnlinbit ,
     &  msu_mjfrcnt , msu_scnpos , 
     &  msu_filler1 ,
     &  msu_qualind , msu_scnlinqual , msu_filler2 , 
     &  msu_pmcal , msu_sdcal , msu_navstat ,
     &  msu_attangtime , msu_rollang , msu_pitchang , msu_yawang ,
     &  msu_scalti , msu_ang , msu_filler3 , msu_pos , msu_filler4 ,
     &  msu_data , msu_filler5
!-------------------------------------------------------------------------------
!                         -- CVS MODIFICATION HISTORY --
!-------------------------------------------------------------------------------
!$Log: msu1b.h,v $
!Revision 1.2  2005/04/15 13:39:48  frna
!Merge AAPP5Dev with main trunk
!
!Revision 1.1.1.1.4.1  2004/11/26 10:47:33  frna
!Add CVS identifiers
!

