! 
!DESCRIPTION 
!     include file for msu 1b format handling
!
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
!     Tiphaine Labrot
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
!  1.0     11/95     take into consideration the     T.Labrot
!                    release from NOAA available 
!                    30/10/95.Still problems in 
!                    integer boudaries alignments
!                    and undefined scaling factors
!  2.0     25/05/98  modified to conform to NOAA release dated
! 
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!###########################################################################
!
!               MSU Level 1B Data Set 
!
!###########################################################################
!
!##############################################################################
!
!               DATA RECORD 
!
!##############################################################################
!
       integer*2 msu_scnlin0               !scan line number
       integer*2 msu_scnlinyr0             !scan line year
       integer*2 msu_scnlindy0             !scan line day of year
       integer*2 msu_clockdrift0           !satellite clock drift delta in ms
       integer   msu_scnlintime0           !scan line UTC time of day in ms
       integer*2 msu_scnlinbit0            !scan line bit field
!              bit 15: (0 = ascending data; 1 = descending data)
!              bit 14: (1 = scan time corrected for clock drift)
!              bit 13-0: <zero fill>
      integer*2  msu_mjfrcnt0              !major frame count
      integer*2  msu_scnpos0               !scan position number in 32 seconds 
      integer*2  msu_filler10(5)
!
!                QUALITY INDICATORS
!  
      integer    msu_qualind0              ! quality indicator bit field 
! 
!                in all of the following if the bit is on (i.e., if it is set
!                to 1) then the statement is true. Otherwise it is false.
!          
!                General
!
!                bit 31: do not use scan for product generation
!                bit 30: time sequence error detected with this scan (see below)
!                bit 29: data gap precedes this scan
!                bit 28: no calibration (see below)
!                bit 27: no earth location (see below)
!                bit 26: first good time following a clock update
!                bit 25: instrument status changed with this scan
!                bit 24: spare
!
!                Time Problem Code
!
!                (All bits off implies the scan time is as expected.)
!                bit 23: time field is bad but can probably be inferred from
!                        the previous good time.
!                bit 22: time field is bad and can't be inferred from the
!                        previous good time.
!                bit 21: this record starts a sequence that is inconsistent
!                        with previous times (i.e., there is a time
!                        discontinuity).  This may or may not be associated
!                        with a spacecraft clock update. (See bit 26 above)
!                bit 20: start of a sequence that apparently repeats scan 
!                        times that have been previously accepted.
!
!                TIP Problem Code
! 
!                bit 19: TIP synch error
!                bit 18: TIP minor or major frame sequence error
!                bit 17: TIP status error 
!                bit 16: TIP parity error 
!
!                Calibration Problem Code
!
!                (Note these bits compliment the channel indicators; all bits
!                set to 0 indicates normal calibration. ) 
!                bit 15: Scan line was not calibrated because of bad time.
!                bit 14: Scan line was calibrated using fewer than the
!                        preferred number of scan lines because of proximity 
!                        to start or end of data set or to a data gap.
!                bit 13: Scan line was not calibrated because of bad or
!                        insufficient PRT data.
!                bit 12: Scan line was calibrated but with marginal PRT data.
!                bit 11: Some uncalibrated channels on this scan. (See channel
!                        indicators.)
!                bit 10: Uncalibrated due to instrument mode.
!                bit 09: spare
!                bit 08: spare
!
!                Earth Location Problem Code
!
!                (all bits set to 0 implies the earth location was normal)
!                bit 07: Not earth located because of bad time.
!                bit 06: Earth location questionable because of questionable
!                        time code. (See time problem flags above.)
!                bit 05: Earth location questionable -- only marginal agreement
!                        with reasonableness check.
!                bit 04: Earth location questionable -- fails reasonableness
!                        check.
!                bit 03-00: spare
!
       integer*2 msu_scnlinqual0(msu_mxchn)       !quality flag
!              for nochannel 1 to 4 :
!                  (all bits off implies a good calibration)
!                  bit 7: spare
!                  bit 6: spare
!                  bit 5: No good blackbody counts for scan line
!                  bit 4: No good space view counts for scan line 
!                  bit 3: No good PRTs for this line
!                  bit 2: Some bad blackbody view counts for this line
!                  bit 1: Some bad space view counts for this line
!                  bit 0: Some bad PRT temps on this line
       integer*2 msu_filler20(8) 
!
!               CALIBRATION COEFFICIENTS
!
       integer   msu_pmcal0(3,msu_mxchn)         !primary calibration
!              for nochannel 1 to 4 :
!                   0   x (chn nochannel second order term, a2)
!                 10^12 x (chn nochannel  first order term, a1)
!                 10^8  x (chn nochannel zeroth order term, a0)
       integer   msu_sdcal0(3,msu_mxchn)         !secondary calibration 
!              for nochannel 1 to 4 :
!                   0 x (chn nochannel second order term, a2)
!               10^12 x (chn nochannel  first order term, a1)
!               10^8  x (chn nochannel zeroth order term, a0)
!
!               NAVIGATION
!
       integer msu_navstat0      !Navigation Status Bit Field
!             bits 31 - 16: <zero fill>       
!             bits 15 - 12: earth location indicator 
!                     (0 = earth location available; 
!                      1 = user ephemeris files greater than 24 hours old;
!                      2 = no earth location available)
!
!             bits 11 - 8: spacecraft attitude control 
!                     (0 = operating inYGC or NOMINAL mode;
!                      1 = operating in another mode;
!                      2 = attitude exceeds nominal tolerance;
!                      3 = both 1 and 2)
!
!            bits 7 - 4: attitude SMODE 
!                     (0 = NOMINAL mode;
!                      1 = rate nulling mode;
!                      2 = YGC mode; 
!                      3 = search mode;
!                      4 = coast mode)
!            bits 3 - 0: attitude PWT1P$AC 
!                     (0 = NOMINAL mode/no test; 
!                      1 = yaw axis test in progress;
!                      2 = roll axis test in progress; 
!                      3 = pitch axis test in progress)       
       integer msu_attangtime0   !Time associated with attitude angles(seconds)
       integer*2 msu_rollang0    !Roll Angle in Degrees
       integer*2 msu_pitchang0   !Pitch Angle in Degrees
       integer*2 msu_yawang0     !Yaw Angle in Degrees
!
       integer*2 msu_scalti0        !Spacecraft Altitude (MSL) in km     
!                                    in this scan line)
       integer*2 msu_ang0(3,11)     !set of 3 angles in degrees for point 25
!                                    to point 2025 every 40 points
!                first angle  : 10^2 x (solar zenith angle )
!                second angle : 10^2 x (satellite zenith angle )
!                third angle  : 10^2 x (relative azimuth angle )
       integer*2 msu_filler30
       integer   msu_pos0(2,11)     !lat/lon pair in degrees for point 25
!                                    to point 2025 every 40 points
!                first  : 10^4 x (latitude )
!                second : 10^4 x (longitude )
!            (North latitude and East longitude are positive)

       integer*2 msu_filler40(8)    !filler
!
!               MSU DATA (as from TIP minor frame)
!
       integer*2 msu_data0(112)    !First to last MSU words for scan line
!
!               FILLER
!
       integer*2 msu_filler50(230)  !filler

       integer   msu_dt(cmsu_size)
       equivalence(msu_dt(1),msu_scnlin0)

       common/msu1bdt/
     &  msu_scnlin0 , msu_scnlinyr0 , msu_scnlindy0 ,
     &  msu_clockdrift0 , msu_scnlintime0 , msu_scnlinbit0 ,
     &  msu_mjfrcnt0,msu_scnpos0, 
     &  msu_filler10 , 
     &  msu_qualind0 , msu_scnlinqual0 , msu_filler20 , 
     &  msu_pmcal0 , msu_sdcal0 , msu_navstat0 ,
     &  msu_attangtime0 , msu_rollang0 , msu_pitchang0 , msu_yawang0 ,
     &  msu_scalti0 , msu_ang0 , msu_filler30 , msu_pos0 ,
     &  msu_filler40 ,
     &  msu_data0 , msu_filler50
