!+ MSU Level 1C Data Set
!
! Name:  msu1c.h
!
! Purpose:
!   'INCLUDE' file for common /msu1chd/   (MSU level 1C header record)
!                  and common /msu1cdt/   (MSU level 1C data record)
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version    Date     Comment
!
!  1.0     22/2/96  Original code. Conforms to 1C format vn1.0 (R.J.Renshaw)
!  1.1     12/12/96 Updated to 1C format vn1.8
!                   'WMO satellite id' now just NOAA sat id
!                   bit 0 (msu1c_dataqual) now set if all channels missing
!                   Calibration flagword deleted
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
      integer       msu_1cvnnb     ! MSU level 1C format version number
         parameter (msu_1cvnnb = 180)!  - this code is set up for vn no 1.8
      integer       msu_1cvnyr     ! format version year for 1C MSU
         parameter (msu_1cvnyr = 1996)
      integer       msu_1cvndy     ! format version day for 1C MSU
         parameter (msu_1cvndy = 347) ! = 12th December
      integer       array_size_m1c     ! Length of array
          parameter (array_size_m1c = 128) !  Length of array
      integer       msu_1crecl     ! MSU level 1C file record length
         parameter (msu_1crecl = array_size_m1c*4)!  - number of bytes
!
!###########################################################################
!
!               HEADER RECORD
!
!###########################################################################
!
      character*3  msu1c_h_site       !dataset creation site ID
      character*1  msu1c_h_cfill1     !filler
      character*3  msu1c_h_1bsite     !creation site for original 1B data
      character*1  msu1c_h_cfill2     !filler
      integer*4    msu1c_h_versnb     !level 1c format version number
      integer*4    msu1c_h_versyr     !level 1c format version year
      integer*4    msu1c_h_versdy     !level 1c format version day of year
      integer*4    msu1c_h_hdrcnt     !count of header records in this data set
      integer*4    msu1c_h_satid      !satellite id (e.g. 14 for NOAA-14)
      integer*4    msu1c_h_instrument !instrument code
!                                      (5=HIRS; 6=MSU; 10=AMSU-A; 11=AMSU-B)
      integer*4    msu1c_h_satht      !nominal satellite altitude, km*10
      integer*4    msu1c_h_period     !nominal orbit period (seconds)
      integer*4    msu1c_h_startorbit !orbit number (at start of file)
      integer*4    msu1c_h_startdatayr!start of data set year
      integer*4    msu1c_h_startdatady!start of data set day of the year
      integer*4    msu1c_h_startdatatime !start of data set UTC time of day (ms)
      integer*4    msu1c_h_endorbit   !orbit number (at end of file)
      integer*4    msu1c_h_enddatayr  !end of data set year
      integer*4    msu1c_h_enddatady  !end of data set day of the year
      integer*4    msu1c_h_enddatatime!end of data set UTC time of day (ms)
      integer*4    msu1c_h_scnlin     !count of scan lines in this data set
      integer*4    msu1c_h_misscnlin  !count of missing scan lines
      integer*4    msu1c_h_spare1     !spare
      integer*4    msu1c_h_spare2     !spare

! temperature-radiance conversion coefficients
      integer*4    msu1c_h_tempradcnv(3,Mnchan)
!                  order of channels = 1,2,3,4
!                  for nochannel = 1 to 4 :
!                      10^6 x (nochannel central wavenumber)
!                      10^6 x (nochannel constant 1)
!                      10^6 x (nochannel constant 2)
      integer*4    msu1c_h_filler(94)  !filler
!
      integer*4 msu1c_hd(array_size_m1c)
      equivalence(msu1c_hd(1),msu1c_h_site)
      common /msu1chd/ msu1c_h_site, msu1c_h_cfill1, msu1c_h_1bsite,
     & msu1c_h_cfill2, msu1c_h_versnb, msu1c_h_versyr, msu1c_h_versdy,
     & msu1c_h_hdrcnt, msu1c_h_satid, msu1c_h_instrument, msu1c_h_satht,
     & msu1c_h_period, msu1c_h_startorbit, msu1c_h_startdatayr,
     & msu1c_h_startdatady, msu1c_h_startdatatime, msu1c_h_endorbit,
     & msu1c_h_enddatayr, msu1c_h_enddatady, msu1c_h_enddatatime,
     & msu1c_h_scnlin, msu1c_h_misscnlin, msu1c_h_spare1,
     & msu1c_h_spare2, msu1c_h_tempradcnv, msu1c_h_filler
!
!##############################################################################
!
!               DATA RECORD
!
!##############################################################################
!
      integer*4   msu1c_scnlin     !scan line number
      integer*4   msu1c_scnlinyr   !scan line year
      integer*4   msu1c_scnlindy   !scan line day of year
      integer*4   msu1c_scnlintime !scan line UTC time of day in milliseconds
      integer*4   msu1c_mjfrcnt    !major frame count
      integer*4   msu1c_qualind    !quality indicator bit field

!                 In all of the following, if the bit is on (= is set to 1)
!                 then the statement is true. Otherwise it is false.
!
!                 general
!
!                 bit 31: do not use scan for product generation
!                 bit 30: time sequence error detected with this scan
!                 bit 29: data gap precedes this scan
!                 bit 28: no calibration
!                 bit 27: no earth location
!                 bit 26: first good time following a clock update
!                 bit 25: instrument status changed with this scan
!                 bit 24-0: spare  <zero fill>
!
      integer*4   msu1c_scanqual   !scan line quality flags
!                 time problem code
!                 (all bits off implies the scan time is as expected)
!                 bit 31-24: spare   <zero fill>
!                 bit 23: time field is bad but can probably be inferred
!                         from the previous good time
!                 bit 22: time field is bad and can't be inferred
!                         from the previous good time
!                 bit 21: this record starts a sequence that is inconsistent
!                         with previous times (i.e., there is a time
!                         discontinuity). This may or may not be associated
!                         with a spacecraft clock update. (see bit 26 above)
!                 bit 20: start of a sequence that apparently repeats scan
!                         times that have been previously accepted.
!
!                TIP Problem Code
!                (All bits off implies no errors.)
!                 bit 19:   TIP synch error
!                 bit 18:   TIP minor or major frame sequence error
!                 bit 17:   TIP status error
!                 bit 16:   TIP parity error
!
!                 calibration problem code
!
!                 (all bits set to 0 indicates normal calibration)
!                  bit 15: scan line was not calibrated because of bad time
!                  bit 14: scan line was calibrated using fewer than the
!                          preferred number of scan lines because of
!                          proximity to start or end of data set or
!                          to a data gap.
!                  bit 13: scan line was not calibrated because of bad or
!                          insufficient PRT data
!                  bit 12: scan line was calibrated but with marginal PRT data
!                  bit 11: some uncalibrated channels of this scan.
!                          See channel indicators.
!                  bit 10: uncalibrated due to instrument mode
!                  bits 9 and 8: spare
!
!                  Earth location problem code
!
!                  (all bits set to 0 implies the Earth location was normal)
!                  bit 7: not Earth located because of bad time
!                  bit 6: Earth location questionable because of questionable
!                         time code (see time problem flags above)
!                  bit 5: Earth location questionable. Only marginal agreement
!                         with reasonableness check.
!                  bit 4: Earth location questionable. Fails reasonableness
!                         check
!                  bit 3-0: spare
!
      integer*4   msu1c_chanqual(Mnchan)  !quality flag for each channel
!                 with the channels in the order 1,2,3,4
!                 (all bits off implies a good calibration)
!                      bit 31-6: spare
!                      bit 5:    No good blackbody counts for scan line
!                      bit 4:    No good space view counts for scan line 
!                      bit 3:    No good PRTs for this line
!                      bit 2:    Some bad blackbody view counts for this line
!                      bit 1:    Some bad space view counts for this line
!                      bit 0:    Some bad PRT temps on this line

      integer*4   msu1c_spare1(3)             !spare

      integer*4   msu1c_latlon(2,Mnfovs)!lat/lon in degrees for Mnfovs
!                     first   : 10^4 x (latitude)
!                     second  : 10^4 x (longitude)
      integer*4   msu1c_angles(2,Mnfovs) !scan angles for Mnfovs
!                     first:    10^2 x (local zenith angle)
!                     second:   10^2 x (local azimuth angle)
      integer*4   msu1c_scalti !sat altitude above reference ellipsoid, km*10
      integer*4   msu1c_spare2                !spare

      integer*4   msu1c_btemps(Mnchan,Mnfovs) !BT data for Mnfovs
!                     10^2 x scene brightness temperature (K) for channels 1-4

      integer*4   msu1c_dataqual(Mnfovs)
!                 quality control word for the data in each field of view:
!                  (all bits off implies acceptable data)
!                  bit 31: spare   <zero fill>
!                  bit 30: set if secondary calibration used
!                  bit 29-5: spare   <zero fill>
!                  bit 4-1:  bit n set to 1 if brightness temperature in
!                            channel n is physically unreasonable or has
!                            not been calculated due to calibration  
!                            problems.
!                  bit 0:    set if all channels missing

      integer*4   msu1c_filler(34)            !filler

      integer*4 msu1c_dt(array_size_m1c)
      equivalence(msu1c_dt(1),msu1c_scnlin)
      common /msu1cdt/ msu1c_scnlin, msu1c_scnlinyr, msu1c_scnlindy,
     & msu1c_scnlintime, msu1c_mjfrcnt, msu1c_qualind, msu1c_scanqual, 
     & msu1c_chanqual, msu1c_spare1, msu1c_latlon, msu1c_angles,
     & msu1c_scalti, msu1c_spare2, msu1c_btemps, msu1c_dataqual,
     & msu1c_filler
!-
