!+ MWHS Level 1C Data Set
!
! Name:  mwhs1c.h
!
! Purpose:
!   'INCLUDE' file for common /mwhs1chd/   (MWHS level 1C header record)
!                  and common /mwhs1cdt/   (MWHS level 1C data record)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    30/11/2010  From amb1c.h
!  1.01   03/10/2012  Add mwhs1c_surface (as for mwts)
!  1.01   27/04/2015  Add beam averaging variables at end of header record
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
!
! Declarations:
!
      integer       mwhs_1cvnnb     ! MWHS level 1C format version number
         parameter (mwhs_1cvnnb = 101)!  - this code is set up for vn no 1.00
      integer       mwhs_1cvnyr     ! format version year for 1C MWHS
         parameter (mwhs_1cvnyr = 2012)
      integer       mwhs_1cvndy     ! format version day for 1C MWHS
         parameter (mwhs_1cvndy = 277)  ! = 10th March
      integer       mwhs_nchan
         parameter (mwhs_nchan = 5)
      integer       mwhs_nfovs
         parameter (mwhs_nfovs = 98)
      integer       array_size_mwhs1c         ! Length of array
         parameter (array_size_mwhs1c = 1396)
      integer       mwhs_1crecl     ! MWHS level 1C file record length
         parameter (mwhs_1crecl = array_size_mwhs1c*4)!  - number of bytes
!
!###########################################################################
!
!               HEADER RECORD
!
!###########################################################################
!
      character*3  mwhs1c_h_site       !dataset creation site ID
      character*1  mwhs1c_h_cfill1     !filler
      character*3  mwhs1c_h_1bsite     !creation site for original 1B data
      character*1  mwhs1c_h_cfill2     !filler
      integer*4    mwhs1c_h_versnb     !level 1c format version number
      integer*4    mwhs1c_h_versyr     !level 1c format version year
      integer*4    mwhs1c_h_versdy     !level 1c format version day of year
      integer*4    mwhs1c_h_hdrcnt     !count of header records in this data set
      integer*4    mwhs1c_h_satid      !satellite id (e.g. 1 for FY-3A)
      integer*4    mwhs1c_h_instrument !instrument code
!                                      (15=MWTS, 16=MWHS)
      integer*4    mwhs1c_h_satht      !nominal satellite altitude, km*10
      integer*4    mwhs1c_h_period     !nominal orbit period (seconds)
      integer*4    mwhs1c_h_startorbit !orbit number (at start of file)
      integer*4    mwhs1c_h_startdatayr!start of data set year
      integer*4    mwhs1c_h_startdatady!start of data set day of the year
      integer*4    mwhs1c_h_startdatatime !start of data set UTC time of day (ms)
      integer*4    mwhs1c_h_endorbit   !orbit number (at end of file)
      integer*4    mwhs1c_h_enddatayr  !end of data set year
      integer*4    mwhs1c_h_enddatady  !end of data set day of the year
      integer*4    mwhs1c_h_enddatatime!end of data set UTC time of day (ms)
      integer*4    mwhs1c_h_scnlin     !count of scan lines in this data set
      integer*4    mwhs1c_h_misscnlin  !count of missing scan lines
      integer*4    mwhs1c_h_vnantennacorr  !version number, antenna corrections
      integer*4    mwhs1c_h_spare      !spare

! temperature-radiance conversion coefficients
      integer*4    mwhs1c_h_tempradcnv(3,mwhs_nchan)
!                  order of channels = 1,2,3,4,5
!                  for nochannel = 1 to 5 :
!                      10^6 x (nochannel central wavenumber)
!                      10^6 x (nochannel constant 1)
!                      10^6 x (nochannel constant 2)
      integer*4    mwhs1c_h_wmosatid   !WMO satellite id, 520=FY-3A
      integer*4    mwhs1c_h_bandwidth(mwhs_nchan)    !GHz*1E6
!
! The following variables characterise any image processing (same as ATMS)
      integer*4    mwhs1c_h_beamversion   !User defined (see mwhs_beamwidth.dat)
! FFT manipulation:
      integer*4    mwhs1c_h_beamwidth(mwhs_nchan)   !deg*1E3
      integer*4    mwhs1c_h_cutoff(mwhs_nchan)      !fraction*1E3
! or simple averaging:
      integer*4    mwhs1c_h_nxaverage(mwhs_nchan)  !number of spots averaged in x 
      integer*4    mwhs1c_h_nyaverage(mwhs_nchan)  !number of spots averaged in y
      integer*4    mwhs1c_h_filler(1332)  !filler
!
      integer*4 mwhs1c_hd(array_size_mwhs1c)
      equivalence(mwhs1c_hd(1),mwhs1c_h_site)
      common /mwhs1chd/ mwhs1c_h_site, mwhs1c_h_cfill1, mwhs1c_h_1bsite,
     & mwhs1c_h_cfill2, mwhs1c_h_versnb, mwhs1c_h_versyr, 
     & mwhs1c_h_versdy,
     & mwhs1c_h_hdrcnt, mwhs1c_h_satid, mwhs1c_h_instrument, 
     & mwhs1c_h_satht,
     & mwhs1c_h_period, mwhs1c_h_startorbit, mwhs1c_h_startdatayr,
     & mwhs1c_h_startdatady, mwhs1c_h_startdatatime, mwhs1c_h_endorbit,
     & mwhs1c_h_enddatayr, mwhs1c_h_enddatady, mwhs1c_h_enddatatime,
     & mwhs1c_h_scnlin, mwhs1c_h_misscnlin, mwhs1c_h_vnantennacorr,
     & mwhs1c_h_spare, mwhs1c_h_tempradcnv, mwhs1c_h_wmosatid,
     & mwhs1c_h_bandwidth, mwhs1c_h_beamversion, mwhs1c_h_beamwidth,
     & mwhs1c_h_cutoff, mwhs1c_h_nxaverage, mwhs1c_h_nyaverage,
     & mwhs1c_h_filler
!
!##############################################################################
!
!               DATA RECORD
!
!##############################################################################
!
      integer*4   mwhs1c_scnlin     !scan line number
      integer*4   mwhs1c_scnlinyr   !scan line year
      integer*4   mwhs1c_scnlindy   !scan line day of year
      integer*4   mwhs1c_scnlintime !scan line UTC time of day in milliseconds
      integer*4   mwhs1c_qualind    !quality indicator bit field

!                 In all of the following, if the bit is on (= is set to 1)
!                 then the statement is true. Otherwise it is false.
!
!                 general
!
!                 bit 31: do not use scan for product generation
!                 bit 30: time sequence error detected with this scan
!                 bit 29: data gap precedes this scan
!                 bit 28: no calibration
!                 bit 27: no earth location
!                 bit 26: first good time following a clock update
!                 bit 25: instrument status changed with this scan
!                 bit 24-0: spare  <zero fill>
!
      integer*4   mwhs1c_scanqual   !scan line quality flags
!                 time problem code
!                 (all bits off implies the scan time is as expected)
!                 bit 31-24: spare   <zero fill>
!                 bit 23: time field is bad but can probably be inferred
!                         from the previous good time
!                 bit 22: time field is bad and can't be inferred
!                         from the previous good time
!                 bit 21: this record starts a sequence that is inconsistent
!                         with previous times (i.e., there is a time
!                         discontinuity). This may or may not be associated
!                         with a spacecraft clock update. (see bit 26 above)
!                 bit 20: start of a sequence that apparently repeats scan
!                         times that have been previously accepted.
!                 bits 19-16: spare
!
!                 calibration problem code
!
!                 (all bits set to 0 indicates normal calibration)
!                  bit 15: scan line was not calibrated because of bad time
!                  bit 14: scan line was calibrated using fewer than the
!                          preferred number of scan lines because of
!                          proximity to start or end of data set or
!                          to a data gap.
!                  bit 13: scan line was not calibrated because of bad or
!                          insufficient PRT data
!                  bit 12: scan line was calibrated but with marginal PRT data
!                  bit 11: some uncalibrated channels of this scan.
!                          See channel indicators.
!                  bit 10: uncalibrated due to instrument mode
!                  bits 9 and 8: spare
!
!                  Earth location problem code
!
!                  (all bits set to 0 implies the Earth location was normal)
!                  bit 7: not Earth located because of bad time
!                  bit 6: Earth location questionable because of questionable
!                         time code (see time problem flags above)
!                  bit 5: Earth location questionable. Only marginal agreement
!                         with reasonableness check.
!                  bit 4: Earth location questionable. Fails reasonableness
!                         check
!                  bit 3: Earth location questionable because of antenna
!                         position check [rs060794.doc & rs062094.do1]
!                  bit 2-0: spare
!
      integer*4   mwhs1c_chanqual(mwhs_nchan)  !quality flag for each channel
!                 with the channels in the order 1,2,3,4,5.
!                 (all bits off implies a good calibration)
!                      bit 31-6: spare
!                      bit 5:    No good blackbody counts for scan line
!                      bit 4:    No good space view counts for scan line 
!                      bit 3:    No good PRTs for this line
!                      bit 2:    Some bad blackbody view counts for this line
!                      bit 1:    Some bad space view counts for this line
!                      bit 0:    Some bad PRT temps on this line
      integer*4   mwhs1c_instrtemp         !MWHS instrument temp (K*100)

      integer*4   mwhs1c_spare1(2)         !spare

      integer*4   mwhs1c_latlon(2,mwhs_nfovs)!lat/lon in degrees for mwhs_nfovs
!                     first   : 10^4 x (latitude)
!                     second  : 10^4 x (longitude)
      integer*4   mwhs1c_surface(2,mwhs_nfovs) !height/type for mwhs_fovs
!                     first   : surface height (metres)
!                     second  : surface type (0=sea, 1=mixed, 2=land, 3=inland water)
      integer*4   mwhs1c_angles(4,mwhs_nfovs) !scan angles for mwhs_nfovs
!                     first:    10^2 x (local zenith angle)
!                     second:   10^2 x (local azimuth angle)
!                     third:    10^2 x (solar zenith angle)
!                     fourth:   10^2 x (solar azimuth angle)
      integer*4   mwhs1c_scalti !sat altitude above reference ellipsoid, km*10
      integer*4   mwhs1c_spare2(2)             !spare

      integer*4   mwhs1c_btemps(mwhs_nchan,mwhs_nfovs) !BT data for mwhs_nfovs
!                     10^2 x scene brightness temperature (K), channels 1-5

      integer*4   mwhs1c_dataqual(mwhs_nfovs)
!                 quality control word for the data in each field of view:
!                  (all bits off implies acceptable data)
!                  bit 31: spare   <zero fill>
!                  bit 30: set if secondary calibration used
!                  bit 29-6: spare   <zero fill>
!                  bit 5-1: bit n set to 1 if brightness temperature in
!                            channel n is physically unreasonable or has
!                            not been calculated due to calibration  
!                            problems.
!                  bit 0:    set if all channels missing

      integer*4   mwhs1c_filler(7)           !filler

      integer*4 mwhs1c_dt(array_size_mwhs1c)
      equivalence(mwhs1c_dt(1),mwhs1c_scnlin)
      common /mwhs1cdt/ mwhs1c_scnlin, mwhs1c_scnlinyr, mwhs1c_scnlindy,
     & mwhs1c_scnlintime, mwhs1c_qualind, mwhs1c_scanqual, 
     & mwhs1c_chanqual, mwhs1c_instrtemp, mwhs1c_spare1, mwhs1c_latlon,
     & mwhs1c_surface,
     & mwhs1c_angles, mwhs1c_scalti, mwhs1c_spare2, mwhs1c_btemps,
     & mwhs1c_dataqual, mwhs1c_filler
!-
