!+ MWHS2 Level 1C Data Set
!
! Name:  mwhs21c.h
!
! Purpose:
!   'INCLUDE' file for common /mwhs21chd/   (mwhs2 level 1C header record)
!                  and common /mwhs21cdt/   (mwhs2 level 1C data record)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0     20/05/2014 Create from mwts21c.h
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
!
! Declarations:
!
      integer       mwhs2_1cvnnb     ! mwhs2 level 1C format version number
         parameter (mwhs2_1cvnnb = 100)!  - this code is set up for vn no 1.9
      integer       mwhs2_1cvnyr     ! format version year for 1C mwhs2
         parameter (mwhs2_1cvnyr = 2014)
      integer       mwhs2_1cvndy     ! format version day for 1C mwhs2
         parameter (mwhs2_1cvndy = 98)
      integer       mwhs2_nchan
         parameter (mwhs2_nchan = 15)
      integer       mwhs2_nmapchan
         parameter (mwhs2_nmapchan = 13)
      integer       mwhs2_nfovs
         parameter (mwhs2_nfovs = 98)
      integer       array_size_mwhs21c     ! Length of array
          parameter (array_size_mwhs21c = 3668) !  Length of array
      integer       mwhs2_1crecl     ! mwhs2 level 1C file record length
         parameter (mwhs2_1crecl = array_size_mwhs21c*4)!  - number of bytes
!
!###########################################################################
!
!               HEADER RECORD
!
!###########################################################################
!
      character*3  mwhs21c_h_site       !dataset creation site ID
      character*1  mwhs21c_h_cfill1     !filler
      character*3  mwhs21c_h_1bsite     !creation site for original 1B data
      character*1  mwhs21c_h_cfill2     !filler
      integer*4    mwhs21c_h_versnb     !level 1c format version number
      integer*4    mwhs21c_h_versyr     !level 1c format version year
      integer*4    mwhs21c_h_versdy     !level 1c format version day of year
      integer*4    mwhs21c_h_hdrcnt     !count of header records in this data set
      integer*4    mwhs21c_h_satid      !satellite id (e.g. 3 for FY-3C)
      integer*4    mwhs21c_h_instrument !instrument code
!                                      (18=MWTS2, 19=MWHS2)
      integer*4    mwhs21c_h_satht      !nominal satellite altitude, km*10
      integer*4    mwhs21c_h_period     !nominal orbit period (seconds)
      integer*4    mwhs21c_h_startorbit !orbit number (at start of file)
      integer*4    mwhs21c_h_startdatayr!start of data set year
      integer*4    mwhs21c_h_startdatady!start of data set day of the year
      integer*4    mwhs21c_h_startdatatime !start of data set UTC time of day (ms)
      integer*4    mwhs21c_h_endorbit   !orbit number (at end of file)
      integer*4    mwhs21c_h_enddatayr  !end of data set year
      integer*4    mwhs21c_h_enddatady  !end of data set day of the year
      integer*4    mwhs21c_h_enddatatime!end of data set UTC time of day (ms)
      integer*4    mwhs21c_h_scnlin     !count of scan lines in this data set
      integer*4    mwhs21c_h_misscnlin  !count of missing scan lines
      integer*4    mwhs21c_h_vnantennacorr  !version number, antenna corrections
      integer*4    mwhs21c_h_spare      !Spare

! temperature-radiance conversion coefficients
      integer*4    mwhs21c_h_tempradcnv(3,mwhs2_nchan)
!                  order of channels = 1,2,3,4...15
!                  for nochannel = 1 to 15 :
!                      10^6 x (nochannel central wavenumber)
!                      10^6 x (nochannel constant 1)
!                      10^6 x (nochannel constant 2)
      integer*4    mwhs21c_h_wmosatid   !WMO satellite id, 522=FY-3C
      integer*4    mwhs21c_h_bandwidth(mwhs2_nchan)    !GHz*1E6
      integer*4    mwhs21c_h_nchan         !number of MWHS channels in format
      integer*4    mwhs21c_h_nmapchan      !number of mapped channels in format
      integer*4    mwhs21c_h_nmapchanused  !number of mapped channels used
!
! The following variables characterise any image processing (same as ATMS)
      integer*4    mwhs21c_h_beamversion   !User defined (see mwhs2_beamwidth.dat)
! FFT manipulation:
      integer*4    mwhs21c_h_beamwidth(mwhs2_nchan)   !deg*1E3
      integer*4    mwhs21c_h_cutoff(mwhs2_nchan)      !fraction*1E3
! or simple averaging:
      integer*4    mwhs21c_h_nxaverage(mwhs2_nchan)  !number of spots averaged in x 
      integer*4    mwhs21c_h_nyaverage(mwhs2_nchan)  !number of spots averaged in y
      integer*4    mwhs21c_h_filler(3521)  !filler
!
      integer*4 mwhs21c_hd(array_size_mwhs21c)
      equivalence(mwhs21c_hd(1),mwhs21c_h_site)
      common /mwhs21chd/ mwhs21c_h_site, mwhs21c_h_cfill1, 
     & mwhs21c_h_1bsite,
     & mwhs21c_h_cfill2, mwhs21c_h_versnb, mwhs21c_h_versyr, 
     & mwhs21c_h_versdy,
     & mwhs21c_h_hdrcnt, mwhs21c_h_satid, mwhs21c_h_instrument,
     & mwhs21c_h_satht,
     & mwhs21c_h_period, mwhs21c_h_startorbit, mwhs21c_h_startdatayr,
     & mwhs21c_h_startdatady, mwhs21c_h_startdatatime,
     & mwhs21c_h_endorbit,
     & mwhs21c_h_enddatayr, mwhs21c_h_enddatady, mwhs21c_h_enddatatime,
     & mwhs21c_h_scnlin, mwhs21c_h_misscnlin, mwhs21c_h_vnantennacorr,
     & mwhs21c_h_spare, mwhs21c_h_tempradcnv, mwhs21c_h_wmosatid, 
     & mwhs21c_h_bandwidth, mwhs21c_h_nchan, mwhs21c_h_nmapchan, 
     & mwhs21c_h_nmapchanused, mwhs21c_h_beamversion, 
     & mwhs21c_h_beamwidth, mwhs21c_h_cutoff, 
     & mwhs21c_h_nxaverage, mwhs21c_h_nyaverage,
     & mwhs21c_h_filler
!
!##############################################################################
!
!               DATA RECORD
!
!##############################################################################
!
      integer*4   mwhs21c_scnlin     !scan line number
      integer*4   mwhs21c_scnlinyr   !scan line year
      integer*4   mwhs21c_scnlindy   !scan line day of year
      integer*4   mwhs21c_scnlintime !scan line UTC time of day in milliseconds
      integer*4   mwhs21c_qualind    !quality indicator bit field
      integer*4   mwhs21c_scanqual   !scan line quality flags
!
      integer*4   mwhs21c_chanqual(mwhs2_nchan)  !quality flag for each channel
!                 with the channels in the order 1,2,3,..,15.
!                 (all bits off implies a good calibration)
!                      bit 31-6: spare
!                      bit 5:    No good blackbody counts for scan line
!                      bit 4:    No good space view counts for scan line 
!                      bit 3:    No good PRTs for this line
!                      bit 2:    Some bad blackbody view counts for this line
!                      bit 1:    Some bad space view counts for this line
!                      bit 0:    Some bad PRT temps on this line
      integer*4   mwhs21c_instrtemp1            !Instr temp (K*100)
      integer*4   mwhs21c_instrtemp2            !Instr temp (K*100)

      integer*4   mwhs21c_spare1(2)             !spare

      integer*4   mwhs21c_latlon(2,mwhs2_nfovs)!lat/lon in degrees for mwhs2_nfovs
!                     first   : 10^4 x (latitude)
!                     second  : 10^4 x (longitude)
      integer*4   mwhs21c_surface(2,mwhs2_nfovs) !height/type for mwhs2_fovs
!                     first   : surface height (metres)
!                     second  : surface type (0=sea, 1=mixed, 2=land, 3=inland water)
      integer*4   mwhs21c_angles(4,mwhs2_nfovs) !scan angles for mwhs2_nfovs
!                     first:    10^2 x (local zenith angle)
!                     second:   10^2 x (local azimuth angle)
!                     third:    10^2 x (solar zenith angle)
!                     fourth:   10^2 x (solar azimuth angle)
      integer*4   mwhs21c_scalti !sat altitude above reference ellipsoid, km*10
      integer*4   mwhs21c_satposition(3)   !x,y,z in metres
      integer*4   mwhs21c_satvelocity(3)   !vx,vy,vz in m/s
      integer*4   mwhs21c_orbitangle       !angle round orbit relative to solar
                                           !plane, in deg*100

      integer*4   mwhs21c_spare2(5)             !spare

      integer*4   mwhs21c_btemps(mwhs2_nchan,mwhs2_nfovs) !BT data for mwhs2_nfovs
!                     10^2 x scene brightness temperature (K)

      integer*4   mwhs21c_btmap(mwhs2_nmapchan,mwhs2_nfovs) !mapped MWHS BT data
!                     10^2 x scene brightness temperature (K)

      integer*4   mwhs21c_dataqual(mwhs2_nfovs)
!                 quality control word for the data in each field of view:
!                  (all bits off implies acceptable data)
!                  bit 31: spare   <zero fill>
!                  bit 30: set if secondary calibration used
!                  bit 29-14: spare   <zero fill>
!                  bit 13-1: bit n set to 1 if brightness temperature in
!                            channel n is physically unreasonable or has
!                            not been calculated due to calibration  
!                            problems.
!                  bit 0:    set if all channels missing

      integer*4   mwhs21c_filler(4)            !filler

      integer*4 mwhs21c_dt(array_size_mwhs21c)
      equivalence(mwhs21c_dt(1),mwhs21c_scnlin)
      common /mwhs21cdt/ mwhs21c_scnlin, mwhs21c_scnlinyr, 
     & mwhs21c_scnlindy,
     & mwhs21c_scnlintime, mwhs21c_qualind, mwhs21c_scanqual, 
     & mwhs21c_chanqual, mwhs21c_instrtemp1, mwhs21c_instrtemp2,
     & mwhs21c_spare1, mwhs21c_latlon, mwhs21c_surface, mwhs21c_angles,
     & mwhs21c_scalti, mwhs21c_satposition, mwhs21c_satvelocity,
     & mwhs21c_orbitangle,
     & mwhs21c_spare2, mwhs21c_btemps, mwhs21c_btmap,
     & mwhs21c_dataqual, mwhs21c_filler
!-
