/* Variables to hold the MWHS SDR */

/* MWHS_SCANS is the max number of scans that we need to accommodate (1 orbit) */

#define MWHS_SPOTS 98
#define MWHS_CHANS 5
#define MWHS_SCANS 2700
#define RMDI -9999.99
#define MDI -999999

/* Compound data types */

typedef struct t_time_vdata {
  short int Package_Counter;
  int RawScanLine_dayTime_for_1th;
  int RawScanLine_dayTime_for_49th;
  int RawScanLine_dayTime_for_50th;
  int RawScanLine_dayTime_for_98th;
  int RawScanLine_msTime_for_1th;
  int RawScanLine_msTime_for_49th;
  int RawScanLine_msTime_for_50th;
  int RawScanLine_msTime_for_98th;
  short Quality_Flag_for_Time_Data;
} t_time_vdata;

typedef struct t_instr_vdata {
  short int Application_Flag;
  short int Data_Package_Type;
  short int Package_Counter;
  short int Package_Length;
  short int AD_Flag;
  short int AD_Busy;
  short int Software_Malfunction_Record;
  short int Software_Working_Mode_Flag;
  unsigned char Cell_Status;
  unsigned char AGC_Flag;
  short int Scan_Mode;
  short int Scan_Bias_or_Pointing_Angle;
  short int Voltage_Detection_5V;
  short int Tele_Measure_27V;
  unsigned char The_Last_Inner_Command;
  unsigned char Count_of_Reset_Signal;
  short int AGC_Data_for_Channel_1;
  short int AGC_Data_for_Channel_2;
  short int AGC_Data_for_Channel_3;
  short int AGC_Data_for_Channel_4;
  short int AGC_Data_for_Channel_5;
  float Digital_Contrlo_Unit;
  float Cell_Control_Unit;
  float Motor_Temperature_1;
  float Motor_Temperature_2;
  float Antenna_Mask_Temperature_1;
  float Antenna_Mask_Temperature_2;
  float FET_150GHz_Amplifier_Temperature;
  float Mixer_150GHz_Temperature;
  float FET_183GHz_Amplifier_Temperature;
  float Mixer_183GHz_Temperature;
} t_instr_vdata;

/* Variables */

int          Count_scnlines;
int          Orbit_Number;
int          Orbit_Period;      /* minutes */

float        Earth_Obs_BT[MWHS_CHANS*MWHS_SCANS*MWHS_SPOTS];
float        Latitude[MWHS_SCANS][MWHS_SPOTS];
float        Longitude[MWHS_SCANS][MWHS_SPOTS];
short int    SensorZenith[MWHS_SCANS][MWHS_SPOTS];     /* deg *100 */
short int    SensorAzimuth[MWHS_SCANS][MWHS_SPOTS];
short int    SolarZenith[MWHS_SCANS][MWHS_SPOTS];
short int    SolarAzimuth[MWHS_SCANS][MWHS_SPOTS];
short int    Height[MWHS_SCANS][MWHS_SPOTS];           /* metres */
short int    LandSeaMask[MWHS_SCANS][MWHS_SPOTS];
int          GPS_Position_Time[MWHS_SCANS];            /* milliseconds */
int          Raw_Time[MWHS_SCANS];                     /* milliseconds */
short int    Package_Counter[MWHS_SCANS];
short int    Black_Body_View_Ang[MWHS_SCANS][3];
short int    Space_View_Ang[MWHS_SCANS][3];
float        Mixer_Temp[MWHS_SCANS];
int          GPS_Position_Data[MWHS_SCANS][3];         /* units of 10cm? */

char         Beginning_Date[11];     /* e.g. "2010-11-30" */
char         Beginning_Time[13];     /* e.g. "04:00:28.000" */
char         Ending_Date[11];
char         Ending_Time[13];
char         Satellite_Name[6];      /* e.g. "FY-3A" */

t_time_vdata time_vdata[MWHS_SCANS];
t_instr_vdata instr_vdata[MWHS_SCANS];
