!+ mwri Level 1C Data Set
!
! Name:  mwri1c.h
!
! Purpose:
!   'INCLUDE' file for common /mwri1chd/   (mwri level 1C header record)
!                  and common /mwri1cdt/   (mwri level 1C data record)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0     20/05/2014 Create from mwts21c.h
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
!
! Declarations:
!
      integer       mwri_1cvnnb     ! mwri level 1C format version number
         parameter (mwri_1cvnnb = 100)!  - this code is set up for vn no 1.9
      integer       mwri_1cvnyr     ! format version year for 1C mwri
         parameter (mwri_1cvnyr = 2014)
      integer       mwri_1cvndy     ! format version day for 1C mwri
         parameter (mwri_1cvndy = 98)
      integer       mwri_nchan
         parameter (mwri_nchan = 10)
      integer       mwri_nmapchan
         parameter (mwri_nmapchan = 10)
      integer       mwri_nfovs
         parameter (mwri_nfovs = 254)
      integer       array_size_mwri1c     ! Length of array
          parameter (array_size_mwri1c = 9000) !  Length of array
      integer       mwri_1crecl     ! mwri level 1C file record length
         parameter (mwri_1crecl = array_size_mwri1c*4)!  - number of bytes
!
!###########################################################################
!
!               HEADER RECORD
!
!###########################################################################
!
      character*3  mwri1c_h_site       !dataset creation site ID
      character*1  mwri1c_h_cfill1     !filler
      character*3  mwri1c_h_1bsite     !creation site for original 1B data
      character*1  mwri1c_h_cfill2     !filler
      integer*4    mwri1c_h_versnb     !level 1c format version number
      integer*4    mwri1c_h_versyr     !level 1c format version year
      integer*4    mwri1c_h_versdy     !level 1c format version day of year
      integer*4    mwri1c_h_hdrcnt     !count of header records in this data set
      integer*4    mwri1c_h_satid      !satellite id (e.g. 3 for FY-3C)
      integer*4    mwri1c_h_instrument !instrument code
!                                              (20=mwri)
      integer*4    mwri1c_h_satht      !nominal satellite altitude, km*10
      integer*4    mwri1c_h_period     !nominal orbit period (seconds)
      integer*4    mwri1c_h_startorbit !orbit number (at start of file)
      integer*4    mwri1c_h_startdatayr!start of data set year
      integer*4    mwri1c_h_startdatady!start of data set day of the year
      integer*4    mwri1c_h_startdatatime !start of data set UTC time of day (ms)
      integer*4    mwri1c_h_endorbit   !orbit number (at end of file)
      integer*4    mwri1c_h_enddatayr  !end of data set year
      integer*4    mwri1c_h_enddatady  !end of data set day of the year
      integer*4    mwri1c_h_enddatatime!end of data set UTC time of day (ms)
      integer*4    mwri1c_h_scnlin     !count of scan lines in this data set
      integer*4    mwri1c_h_misscnlin  !count of missing scan lines
      integer*4    mwri1c_h_vnantennacorr  !version number, antenna corrections
      integer*4    mwri1c_h_spare      !Spare

! temperature-radiance conversion coefficients
      integer*4    mwri1c_h_tempradcnv(3,mwri_nchan)
!                  order of channels = 1,2,3,4...15
!                  for nochannel = 1 to 15 :
!                      10^6 x (nochannel central wavenumber)
!                      10^6 x (nochannel constant 1)
!                      10^6 x (nochannel constant 2)
      integer*4    mwri1c_h_wmosatid   !WMO satellite id, 522=FY-3C
      integer*4    mwri1c_h_bandwidth(mwri_nchan)    !GHz*1E6
      integer*4    mwri1c_h_nchan         !number of MWHS channels in format
      integer*4    mwri1c_h_nmapchan      !number of mapped channels in format
      integer*4    mwri1c_h_nmapchanused  !number of mapped channels used
!
! The following variables characterise any image processing (same as ATMS)
      integer*4    mwri1c_h_beamversion   !User defined (see mwri_beamwidth.dat)
! FFT manipulation:
      integer*4    mwri1c_h_beamwidth(mwri_nchan)   !deg*1E3
      integer*4    mwri1c_h_cutoff(mwri_nchan)      !fraction*1E3
! or simple averaging:
      integer*4    mwri1c_h_nxaverage(mwri_nchan)  !number of spots averaged in x 
      integer*4    mwri1c_h_nyaverage(mwri_nchan)  !number of spots averaged in y
      integer*4    mwri1c_h_filler(200)  !filler
!
      integer*4 mwri1c_hd(array_size_mwri1c)
      equivalence(mwri1c_hd(1),mwri1c_h_site)
      common /mwri1chd/ mwri1c_h_site, mwri1c_h_cfill1, 
     & mwri1c_h_1bsite,
     & mwri1c_h_cfill2, mwri1c_h_versnb, mwri1c_h_versyr, 
     & mwri1c_h_versdy,
     & mwri1c_h_hdrcnt, mwri1c_h_satid, mwri1c_h_instrument,
     & mwri1c_h_satht,
     & mwri1c_h_period, mwri1c_h_startorbit, mwri1c_h_startdatayr,
     & mwri1c_h_startdatady, mwri1c_h_startdatatime,
     & mwri1c_h_endorbit,
     & mwri1c_h_enddatayr, mwri1c_h_enddatady, mwri1c_h_enddatatime,
     & mwri1c_h_scnlin, mwri1c_h_misscnlin, mwri1c_h_vnantennacorr,
     & mwri1c_h_spare, mwri1c_h_tempradcnv, mwri1c_h_wmosatid, 
     & mwri1c_h_bandwidth, mwri1c_h_nchan, mwri1c_h_nmapchan, 
     & mwri1c_h_nmapchanused, mwri1c_h_beamversion, 
     & mwri1c_h_beamwidth, mwri1c_h_cutoff, 
     & mwri1c_h_nxaverage, mwri1c_h_nyaverage,
     & mwri1c_h_filler
!
!##############################################################################
!
!               DATA RECORD
!
!##############################################################################
!
      integer*4   mwri1c_scnlin     !scan line number
      integer*4   mwri1c_scnlinyr   !scan line year
      integer*4   mwri1c_scnlindy   !scan line day of year
      integer*4   mwri1c_scnlintime !scan line UTC time of day in milliseconds
      integer*4   mwri1c_qualind    !quality indicator bit field
      integer*4   mwri1c_scanqual   !scan line quality flags
!
      integer*4   mwri1c_chanqual(mwri_nchan)  !quality flag for each channel
!                 with the channels in the order 1,2,3,..,15.
!                 (all bits off implies a good calibration)
!                      bit 31-6: spare
!                      bit 5:    No good blackbody counts for scan line
!                      bit 4:    No good space view counts for scan line 
!                      bit 3:    No good PRTs for this line
!                      bit 2:    Some bad blackbody view counts for this line
!                      bit 1:    Some bad space view counts for this line
!                      bit 0:    Some bad PRT temps on this line
      integer*4   mwri1c_instrtemp1            !Instr temp (K*100)
      integer*4   mwri1c_instrtemp2            !Instr temp (K*100)

      integer*4   mwri1c_spare1(2)             !spare

      integer*4   mwri1c_latlon(2,mwri_nfovs)!lat/lon in degrees for mwri_nfovs
!                     first   : 10^4 x (latitude)
!                     second  : 10^4 x (longitude)
      integer*4   mwri1c_surface(2,mwri_nfovs) !height/type for mwri_fovs
!                     first   : surface height (metres)
!                     second  : surface type (0=sea, 1=mixed, 2=land, 3=inland water)
      integer*4   mwri1c_angles(4,mwri_nfovs) !scan angles for mwri_nfovs
!                     first:    10^2 x (local zenith angle)
!                     second:   10^2 x (local azimuth angle)
!                     third:    10^2 x (solar zenith angle)
!                     fourth:   10^2 x (solar azimuth angle)
      integer*4   mwri1c_scalti !sat altitude above reference ellipsoid, km*10
      integer*4   mwri1c_satposition(3)   !x,y,z in metres
      integer*4   mwri1c_satvelocity(3)   !vx,vy,vz in m/s
      integer*4   mwri1c_orbitangle       !angle round orbit relative to solar
                                           !plane, in deg*100

      integer*4   mwri1c_spare2(5)             !spare

      integer*4   mwri1c_btemps(mwri_nchan,mwri_nfovs) !BT data for mwri_nfovs
!                     10^2 x scene brightness temperature (K)

      integer*4   mwri1c_btmap(mwri_nmapchan,mwri_nfovs) !mapped MWHS BT data
!                     10^2 x scene brightness temperature (K)

      integer*4   mwri1c_dataqual(mwri_nfovs)
!                 quality control word for the data in each field of view:
!                  (all bits off implies acceptable data)
!                  bit 31: spare   <zero fill>
!                  bit 30: set if secondary calibration used
!                  bit 29-14: spare   <zero fill>
!                  bit 13-1: bit n set to 1 if brightness temperature in
!                            channel n is physically unreasonable or has
!                            not been calculated due to calibration  
!                            problems.
!                  bit 0:    set if all channels missing

      integer*4   mwri1c_filler(4)            !filler

      integer*4 mwri1c_dt(array_size_mwri1c)
      equivalence(mwri1c_dt(1),mwri1c_scnlin)
      common /mwri1cdt/ mwri1c_scnlin, mwri1c_scnlinyr, 
     & mwri1c_scnlindy,
     & mwri1c_scnlintime, mwri1c_qualind, mwri1c_scanqual, 
     & mwri1c_chanqual, mwri1c_instrtemp1, mwri1c_instrtemp2,
     & mwri1c_spare1, mwri1c_latlon, mwri1c_surface, mwri1c_angles,
     & mwri1c_scalti, mwri1c_satposition, mwri1c_satvelocity,
     & mwri1c_orbitangle,
     & mwri1c_spare2, mwri1c_btemps, mwri1c_btmap,
     & mwri1c_dataqual, mwri1c_filler
!-
