!+ MWTS Level 1C Data Set
!
! Name:  mwts1c.h
!
! Purpose:
!   'INCLUDE' file for common /mwts1chd/   (MWTS level 1C header record)
!                  and common /mwts1cdt/   (MWTS level 1C data record)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0      2/2/96  Original code. Conforms to 1C format vn1.7 (R.J.Renshaw)
!  1.1     12/12/96 Updated to 1C format vn1.8
!                   'WMO satellite id' now just NOAA sat id
!                   bit 0 (mwts1c_dataqual) now set if all channels missing
!                   Calibration flagword deleted
!  1.2     10/03/98 1C format vn1.9  - add instrument temperature & solar angles
!  1.3     30/04/03 mwts1c_scanqual bits 16-19 used for moon contamination flags
!  1.4     12/08/05 add wmosatid in place of filler (NCA)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
!
! Declarations:
!
      integer       mwts_1cvnnb     ! MWTS level 1C format version number
         parameter (mwts_1cvnnb = 100)!  - this code is set up for vn no 1.9
      integer       mwts_1cvnyr     ! format version year for 1C MWTS
         parameter (mwts_1cvnyr = 2010)
      integer       mwts_1cvndy     ! format version day for 1C MWTS
         parameter (mwts_1cvndy = 334)
      integer       mwts_nchan
         parameter (mwts_nchan = 4)
      integer       mwts_nmapchan
         parameter (mwts_nmapchan = 5)
      integer       mwts_nfovs
         parameter (mwts_nfovs = 15)
      integer       array_size_mwts1c     ! Length of array
          parameter (array_size_mwts1c = 300) !  Length of array
      integer       mwts_1crecl     ! MWTS level 1C file record length
         parameter (mwts_1crecl = array_size_mwts1c*4)!  - number of bytes
!
!###########################################################################
!
!               HEADER RECORD
!
!###########################################################################
!
      character*3  mwts1c_h_site       !dataset creation site ID
      character*1  mwts1c_h_cfill1     !filler
      character*3  mwts1c_h_1bsite     !creation site for original 1B data
      character*1  mwts1c_h_cfill2     !filler
      integer*4    mwts1c_h_versnb     !level 1c format version number
      integer*4    mwts1c_h_versyr     !level 1c format version year
      integer*4    mwts1c_h_versdy     !level 1c format version day of year
      integer*4    mwts1c_h_hdrcnt     !count of header records in this data set
      integer*4    mwts1c_h_satid      !satellite id (e.g. 1 for FY-3A)
      integer*4    mwts1c_h_instrument !instrument code
!                                      (15=MWTS, 16=MWHS)
      integer*4    mwts1c_h_satht      !nominal satellite altitude, km*10
      integer*4    mwts1c_h_period     !nominal orbit period (seconds)
      integer*4    mwts1c_h_startorbit !orbit number (at start of file)
      integer*4    mwts1c_h_startdatayr!start of data set year
      integer*4    mwts1c_h_startdatady!start of data set day of the year
      integer*4    mwts1c_h_startdatatime !start of data set UTC time of day (ms)
      integer*4    mwts1c_h_endorbit   !orbit number (at end of file)
      integer*4    mwts1c_h_enddatayr  !end of data set year
      integer*4    mwts1c_h_enddatady  !end of data set day of the year
      integer*4    mwts1c_h_enddatatime!end of data set UTC time of day (ms)
      integer*4    mwts1c_h_scnlin     !count of scan lines in this data set
      integer*4    mwts1c_h_misscnlin  !count of missing scan lines
      integer*4    mwts1c_h_vnantennacorr  !version number, antenna corrections
      integer*4    mwts1c_h_spare      !Spare

! temperature-radiance conversion coefficients
      integer*4    mwts1c_h_tempradcnv(3,mwts_nchan)
!                  order of channels = 1,2,3,4
!                  for nochannel = 1 to 4 :
!                      10^6 x (nochannel central wavenumber)
!                      10^6 x (nochannel constant 1)
!                      10^6 x (nochannel constant 2)
      integer*4    mwts1c_h_wmosatid   !WMO satellite id, 520=FY-3A
      integer*4    mwts1c_h_bandwidth(mwts_nchan)    !GHz*1E6
      integer*4    mwts1c_h_filler(261)  !filler
!
      integer*4 mwts1c_hd(array_size_mwts1c)
      equivalence(mwts1c_hd(1),mwts1c_h_site)
      common /mwts1chd/ mwts1c_h_site, mwts1c_h_cfill1, mwts1c_h_1bsite,
     & mwts1c_h_cfill2, mwts1c_h_versnb, mwts1c_h_versyr, 
     & mwts1c_h_versdy,
     & mwts1c_h_hdrcnt, mwts1c_h_satid, mwts1c_h_instrument,
     & mwts1c_h_satht,
     & mwts1c_h_period, mwts1c_h_startorbit, mwts1c_h_startdatayr,
     & mwts1c_h_startdatady, mwts1c_h_startdatatime,
     & mwts1c_h_endorbit,
     & mwts1c_h_enddatayr, mwts1c_h_enddatady, mwts1c_h_enddatatime,
     & mwts1c_h_scnlin, mwts1c_h_misscnlin, mwts1c_h_vnantennacorr,
     & mwts1c_h_spare, mwts1c_h_tempradcnv, mwts1c_h_wmosatid, 
     & mwts1c_h_bandwidth, mwts1c_h_filler
!
!##############################################################################
!
!               DATA RECORD
!
!##############################################################################
!
      integer*4   mwts1c_scnlin     !scan line number
      integer*4   mwts1c_scnlinyr   !scan line year
      integer*4   mwts1c_scnlindy   !scan line day of year
      integer*4   mwts1c_scnlintime !scan line UTC time of day in milliseconds
      integer*4   mwts1c_qualind    !quality indicator bit field

!                 In all of the following, if the bit is on (= is set to 1)
!                 then the statement is true. Otherwise it is false.
!
!                 general
!
!                 bit 31: do not use scan for product generation
!                 bit 30: time sequence error detected with this scan
!                 bit 29: data gap precedes this scan
!                 bit 28: no calibration
!                 bit 27: no earth location
!                 bit 26: first good time following a clock update
!                 bit 25: instrument status changed with this scan
!                 bit 24-0: spare  <zero fill>
!
      integer*4   mwts1c_scanqual   !scan line quality flags
!                 time problem code
!                 (all bits off implies the scan time is as expected)
!                 bit 31-24: spare   <zero fill>
!                 bit 23: time field is bad but can probably be inferred
!                         from the previous good time
!                 bit 22: time field is bad and can't be inferred
!                         from the previous good time
!                 bit 21: this record starts a sequence that is inconsistent
!                         with previous times (i.e., there is a time
!                         discontinuity). This may or may not be associated
!                         with a spacecraft clock update. (see bit 26 above)
!                 bit 20: start of a sequence that apparently repeats scan
!                         times that have been previously accepted.
!
!                 moon problem code
!
!                 bit 19: spare
!                 bit 18: Moon in MWTS space view, corrected
!                 bit 17: spare
!                 bit 16: Moon in MWTS space view, uncorrected
!
!                 calibration problem code
!
!                 (all bits set to 0 indicates normal calibration)
!                  bit 15: scan line was not calibrated because of bad time
!                  bit 14: scan line was calibrated using fewer than the
!                          preferred number of scan lines because of
!                          proximity to start or end of data set or
!                          to a data gap.
!                  bit 13: scan line was not calibrated because of bad or
!                          insufficient PRT data
!                  bit 12: scan line was calibrated but with marginal PRT data
!                  bit 11: some uncalibrated channels of this scan.
!                          See channel indicators.
!                  bit 10: uncalibrated due to instrument mode
!                  bits 9 and 8: spare
!
!                  Earth location problem code
!
!                  (all bits set to 0 implies the Earth location was normal)
!                  bit 7: not Earth located because of bad time
!                  bit 6: Earth location questionable because of questionable
!                         time code (see time problem flags above)
!                  bit 5: Earth location questionable. Only marginal agreement
!                         with reasonableness check.
!                  bit 4: Earth location questionable. Fails reasonableness
!                         check
!                  bit 3: Earth location questionable because of antenna
!                         position check [rs060794.doc & rs062094.do1]
!                  bit 2-0: spare
!
      integer*4   mwts1c_chanqual(mwts_nchan)  !quality flag for each channel
!                 with the channels in the order 1,2,3,..,15.
!                 (all bits off implies a good calibration)
!                      bit 31-6: spare
!                      bit 5:    No good blackbody counts for scan line
!                      bit 4:    No good space view counts for scan line 
!                      bit 3:    No good PRTs for this line
!                      bit 2:    Some bad blackbody view counts for this line
!                      bit 1:    Some bad space view counts for this line
!                      bit 0:    Some bad PRT temps on this line
      integer*4   mwts1c_instrtemp1            !MWTS1 RF shelf temp (K*100)
      integer*4   mwts1c_instrtemp2            !MWTS2 RF shelf temp (K*100)

      integer*4   mwts1c_spare1(2)             !spare

      integer*4   mwts1c_latlon(2,mwts_nfovs)!lat/lon in degrees for mwts_nfovs
!                     first   : 10^4 x (latitude)
!                     second  : 10^4 x (longitude)
      integer*4   mwts1c_surface(2,mwts_nfovs) !height/type for mwts_fovs
!                     first   : surface height (metres)
!                     second  : surface type (0=sea, 1=mixed, 2=land)
      integer*4   mwts1c_angles(4,mwts_nfovs) !scan angles for mwts_nfovs
!                     first:    10^2 x (local zenith angle)
!                     second:   10^2 x (local azimuth angle)
!                     third:    10^2 x (solar zenith angle)
!                     fourth:   10^2 x (solar azimuth angle)
      integer*4   mwts1c_scalti !sat altitude above reference ellipsoid, km*10
      integer*4   mwts1c_spare2(2)             !spare

      integer*4   mwts1c_btemps(mwts_nchan,mwts_nfovs) !BT data for mwts_nfovs
!                     10^2 x scene brightness temperature (K)

      integer*4   mwts1c_btmap(mwts_nmapchan,mwts_nfovs) !mapped MWHS BT data
!                     10^2 x scene brightness temperature (K)

      integer*4   mwts1c_dataqual(mwts_nfovs)
!                 quality control word for the data in each field of view:
!                  (all bits off implies acceptable data)
!                  bit 31: spare   <zero fill>
!                  bit 30: set if secondary calibration used
!                  bit 29-5: spare   <zero fill>
!                  bit 4-1: bit n set to 1 if brightness temperature in
!                            channel n is physically unreasonable or has
!                            not been calculated due to calibration  
!                            problems.
!                  bit 0:    set if all channels missing

      integer*4   mwts1c_filler(13)            !filler

      integer*4 mwts1c_dt(array_size_mwts1c)
      equivalence(mwts1c_dt(1),mwts1c_scnlin)
      common /mwts1cdt/ mwts1c_scnlin, mwts1c_scnlinyr, mwts1c_scnlindy,
     & mwts1c_scnlintime, mwts1c_qualind, mwts1c_scanqual, 
     & mwts1c_chanqual, mwts1c_instrtemp1, mwts1c_instrtemp2,
     & mwts1c_spare1, mwts1c_latlon, mwts1c_surface, mwts1c_angles,
     & mwts1c_scalti, mwts1c_spare2, mwts1c_btemps, mwts1c_btmap,
     & mwts1c_dataqual, mwts1c_filler
!-
