!+ Numbers for processing AMSU-A within ATOVPP
!
! Name:  ppafdata.h
!
! Purpose:
!   'INCLUDE' file containing for processing AMSU-A within ATOVPP.
!
! History:
! Version    Date     Comment
!
!  1.0        12/04/96  Original code. (R.J.Renshaw)
!  1.1        10/12/96  Numbers added for Crosby & Grody tests
!  1.2        17/12/96  ASecAng initialised in PPAFDF, not here
!  1.3        10/01/97  AXcost new (multiplier for cost function without CLW)
!  1.4        14/08/00  ABcirr and ABmaxcirr new (coefficients for cirrus index) (D.C.Jones)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
      INTEGER     AboxSize        ! size of grid-box used in SURFELEV
      PARAMETER ( AboxSize = 100 )! km
!
! version numbers for the pre-processing
      INTEGER       Acorrvn       ! limb/emissivity correction (0=not done)
      INTEGER       Apcpvn        ! precip/scattering detection (0=not done)
      INTEGER       Asurfvn       ! surface identification (0=not done)
!
! bias corrections
      REAL          Abiases(Anchan,Anfovs)  ! deg K, added to the obs
!
! numbers for scattering test PPASCAT
      REAL          Amaxscat        ! threshold for scattering index (deg K)
      REAL          Ascat(4,Anfovs) ! predict AMSU-A channel 15 from 1,2,3

! numbers for microwave cirrus index PPACIR test
      REAL          ABmaxcirr        ! threshold for cirrus index (deg K)
      REAL          ABcirr(4,Anfovs) ! predict AMSU-B channel 20 from AMSU-A
                                     ! channel 1 and AMSU-B channels 16,17

!     The equation for each fov is T15 =A1 + T1*A2 + T2*A3 + T3*A4
!
! numbers for logistic precipitation probability PPCROSBY
      REAL          Acrosby1, Acrosby2, Acrosby3   ! coefficients
      REAL          Amaxcrosby                     ! threshold
!
! numbers for Grody light rainfall test PPGRODY
      REAL          Agrody1, Agrody2
!
! threshold for difference in AMSU-A/AMSUB 89GHz channels
      REAL          ABmax89
!
! numbers for surface identification PPASURF
      INTEGER       NASrfTyp        ! number of surface types
         PARAMETER (NASrfTyp=8)
      INTEGER       NASecAng        ! number of angles
         PARAMETER (NASecAng=5)
      INTEGER       NASrfChMx       ! max number of channels
         PARAMETER (NASrfChMx=10)
      INTEGER       NASrfCh         ! number of channels
      INTEGER       ASrfChn(NASrfChMx) ! channels to use in PPASURF
      REAL          ASrfThresh      ! threshold to detect cloud in cost function
      REAL          AmeanBT(NASrfTyp,NASecAng,NASrfChMx)  ! mean BT vectors
      REAL          AcovBT(NASrfTyp,NASecAng,NASrfChMx,NASrfChMx) ! BT cov. mx.
      REAL          ASecAng(NASecAng) ! secant(angles) for the above
!
      COMMON / PPAFDATA / Acorrvn, Apcpvn, Asurfvn,
     &                    Abiases, Amaxscat, Ascat,
     &                    ABmaxcirr, ABcirr,
     &                    Acrosby1, Acrosby2, Acrosby3, Amaxcrosby,
     &                    Agrody1, Agrody2, ABmax89,
     &         AmeanBT, AcovBT, ASecAng, NASrfCh, ASrfChn, ASrfThresh
