!+ AMSU-A Data Variables for processing within ATOVPP
!
! Name:  ppastore.h
!
! Purpose:
!   'INCLUDE' file for COMMON / ppastore /
!   containing AMSU-A data for use within ATOVIN code.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0        12/04/96  Original code. (R.J.Renshaw)
!  1.1        12/12/96  Calibration flagword deleted for 1D format vn 1.8
!  1.2        10/03/98  instrument temperatures and solar angles stored
!  1.3        20/05/98  extra pre-processing variables included
!  1.4        15/06/98  ASatId added
!  1.5        29/06/98  AmapBT deleted (not used)
!  1.6        02/07/98  Aangles,Asurfcost,Ascatindx,Acrosby REAL not INTEGER
!  1.7        15/03/05  Add ALandFrac
!  1.8        12/08/05  Add Ayear1, Aday1, Atime1, Aline1 (NCA)
!
!  Size of common block is 450 4-byte words / AMSU-A scanline    + 6*4 bytes
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
      INTEGER       ama_stvnnb     ! format version number of this file
         PARAMETER (ama_stvnnb = 130) !  - this code is set up for vn no 1.30
!
      INTEGER       ASatId         ! e.g. 15 for NOAA-15
!
      INTEGER       Arecdno(Anlines)               ! record number (1C & 1D)
!            note that Arecdno() = MDI indicates that the scanline is missing
      INTEGER       Alineno(Anlines)               ! scanline number
      INTEGER       Ayear(Anlines)                 ! scanline year
      INTEGER       Aday(Anlines)                  ! scanline day of year
      INTEGER       Atime(Anlines)                 ! scanline time of day (ms)
      INTEGER       Aqualind(Anlines)              !quality indicator bit field
!                 bit 31: do not use scan for product generation
!                 bit 30: time sequence error detected with this scan
!                 bit 29: data gap precedes this scan
!                 bit 28: no AMSU-A calibration
!                 bit 27: no earth location
!                 bit 26: first good time following a clock update
!                 bit 25: instrument status changed with this scan
!                 bit 24-0: spare  <zero fill>
!
      INTEGER       Ascanqual(Anlines)             ! scan line quality flags
!                 time problem code
!                 (all bits off implies the scan time is as expected)
!                 bit 31-24: spare   <zero fill>
!                 bit 23: time field is bad but can probably be inferred
!                         from the previous good time
!                 bit 22: time field is bad and can't be inferred
!                         from the previous good time
!                 bit 21: this record starts a sequence that is inconsistent
!                         with previous times (i.e., there is a time
!                         discontinuity). This may or may not be associated
!                         with a spacecraft clock update. (see bit 26 above)
!                 bit 20: start of a sequence that apparently repeats scan
!                         times that have been previously accepted.
!                 bits 19-16: spare
!
!                 calibration problem code
!
!                 (all bits set to 0 indicates normal calibration)
!                  bit 15: scan line was not calibrated because of bad time
!                  bit 14: scan line was calibrated using fewer than the
!                          preferred number of scan lines because of
!                          proximity to start or end of data set or
!                          to a data gap.
!                  bit 13: scan line was not calibrated because of bad or
!                          insufficient PRT data
!                  bit 12: scan line was calibrated but with marginal PRT data
!                  bit 11: some uncalibrated channels of this scan.
!                          See channel indicators.
!                  bit 10: uncalibrated due to instrument mode
!                  bits 9 and 8: spare
!
!                  Earth location problem code
!
!                  (all bits set to 0 implies the Earth location was normal)
!                  bit 7: not Earth located because of bad time
!                  bit 6: Earth location questionable because of questionable
!                         time code (see time problem flags above)
!                  bit 5: Earth location questionable. Only marginal agreement
!                         with reasonableness check.
!                  bit 4: Earth location questionable. Fails reasonableness
!                         check
!                  bit 3: Earth location questionable because of antenna
!                         position check [rs060794.doc & rs062094.do1]
!                  bit 2-0: spare
!
      INTEGER       Ainstrtemp1(Anlines)           ! AMSU-A1 RF shelf temp
      INTEGER       Ainstrtemp2(Anlines)           ! AMSU-A1 RF shelf temp
      INTEGER       Ainstrtempb(Anlines)           ! nearest AMSU-B temp
      REAL          Alats(Anfovs,Anlines)          ! latitudes (deg, N +ve)
      REAL          Alons(Anfovs,Anlines)          ! longitudes (deg, E +ve)
      INTEGER       Aelev(Anfovs,Anlines)          ! surface height (m)
      INTEGER       ALandFrac(Anfovs,Anlines)      ! Land fraction (percent)
      INTEGER       Asurftype(Anfovs,Anlines)!surface type(0=sea,1=mixed,2=land)
      REAL          Aangles(4,Anfovs,Anlines)      !scan angles for Anfovs
!                                          (1,*,*) local zenith angle (deg)
!                                          (2,*,*) local azimuth angle (deg)
!                                          (3,*,*) solar zenith angle (deg)
!                                          (4,*,*) solar azimuth angle (deg)
      INTEGER       Aaltitude(Anlines)             ! satellite altitude (km*10)
                                                   ! above reference ellipsoid
      REAL          Abtemps(Anchan+Bnchan,Anfovs,Anlines) ! AMSU-A&B BTs (K)
      INTEGER       Aqc(Anfovs,Anlines)            ! ATOVPP QC word for AMSU
!                            (all bits off implies acceptable data)
!                  bit 31: set if AMSU-A BTs considered contaminated
!                         for the mapping, due e.g. to precip or surface type
!                  bit 30: set if AMSU-A used secondary calibration
!                  bit 29: set if AMSU-B used secondary calibration
!                  bit 28: set if AMSU-B data missing due to insufficient
!                      good data available for the mapping.
!                  bit 27: maximum probability scheme cloud flag
!                  bit 26: scattering test (only set over the sea)
!                  bit 25: logistic precipitation probability test
!                  bit 24: Grody light rainfall test
!                  bit 23: mismatch between AMSU-A/B 89GHz values
!                  bit 22: mismatch between surface type from topography
!                          dataset and from pre-processing
!                  bit 21-1: spare   <zero fill>
!                  bit 0:    bad or missing data (in any or all channels)
!
      INTEGER       Appsurf(Anfovs,Anlines)       ! estimated surface type
!                  1 = Bare young ice (i.e. new ice, no snow)
!                  2 = Dry land (i.e. dry with/without significant vegetation)
!                  3 = Dry snow (i.e. snow with water less than 2%, over land)
!                  4 = Multi-year ice (i.e. old ice & snow [assumed dry] cover)
!                  5 = Sea (i.e. open water, ice-free, WS=0 to 14m/s)
!                  6 = Wet forest (i.e. established forest with wet canopy)
!                  7 = Wet land (i.e. non-forested land with a wet surface)
!                  8 = Wet snow (i.e. snow, water content>2%, over land or ice)
!                  9 = Desert
!
      REAL          Asurfcost(Anfovs,Anlines) ! max prob cloud test (ppasurf)
      REAL          Ascatindx(Anfovs,Anlines) ! scattering index (ppascat)
      REAL          Acrosby(Anfovs,Anlines)   ! logistic precip test (ppcrosby)
      REAL          Asatht               ! nominal sat height
      REAL          Asatperiod           ! nominal orbit period (mins)
      INTEGER       Ahdrcnt              !count of header records in 1C/1D file
      INTEGER       Ascnlin              !count of scanlines in 1C/1D file
      INTEGER       Ainstruments         ! instruments present
!                   (bit0=HIRS, bit1=MSU, bit3=AMSU-A, bit4=AMSU-B, bit5=AVHRR)
      INTEGER       Ayear1        ! year of first "good" AMSU-A scanline
      INTEGER       Aday1         ! day
      INTEGER       Atime1        ! time
      INTEGER       Aline1        ! scanline no.

      COMMON / ppastore / ASatId, Arecdno, Alineno, Ayear, Aday, Atime,
     & Aqualind, Ascanqual, Ainstrtemp1, Ainstrtemp2, Ainstrtempb,
     & Alats, Alons, Aelev, ALandFrac, Asurftype, Aangles, Aaltitude,
     & Abtemps, Aqc, Appsurf, Asurfcost, Ascatindx, Acrosby, Asatht,
     & Asatperiod, Ahdrcnt, Ascnlin, Ainstruments,
     & Ayear1, Aday1, Atime1, Aline1
