!+ Numbers for processing ATMS within ATOVPP
!
! Name:  ppafdata.h
!
! Purpose:
!   'INCLUDE' file containing for processing ATMS within ATOVPP.
!
! History:
! Version    Date     Comment
!
!  1.0      16/08/2010 From ppafdata.h
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
      INTEGER     atmsboxSize        ! size of grid-box used in SURFELEV
      PARAMETER ( atmsboxSize = 80 ) ! km
!
! version numbers for the pre-processing
      INTEGER       atmscorrvn       ! limb/emissivity correction (0=not done)
      INTEGER       atmspcpvn        ! precip/scattering detection (0=not done)
      INTEGER       atmssurfvn       ! surface identification (0=not done)
!
! bias corrections
      REAL          atmsbiases(atmsnchan,atmsnfovs)  ! deg K, added to the obs
!
! numbers for scattering test PPASCAT
      REAL          atmsmaxscat        ! threshold for scattering index (deg K)
      REAL          atmsscat(4,atmsnfovs) ! predict ATMS 16 from 1,2,3

! numbers for microwave cirrus index PPACIR test
      REAL          atmsmaxcirr           ! threshold for cirrus index (deg K)
      REAL          atmscirr(4,atmsnfovs) ! predict channel 18 from 
                                          ! channels 1,16,17

!     The equation for each fov is T16 =A1 + T1*A2 + T2*A3 + T3*A4
!
! numbers for logistic precipitation probability PPCROSBY
      REAL          atmscrosby1, atmscrosby2, atmscrosby3   ! coefficients
      REAL          atmsmaxcrosby                           ! threshold
!
! numbers for Grody light rainfall test PPGRODY
      REAL          atmsgrody1, atmsgrody2
!
! numbers for surface identification PPASURF
      INTEGER       NatmsSrfTyp        ! number of surface types
         PARAMETER (NatmsSrfTyp=8)
      INTEGER       NatmsSecAng        ! number of angles
         PARAMETER (NatmsSecAng=5)
      INTEGER       NatmsSrfChMx       ! max number of channels
         PARAMETER (NatmsSrfChMx=10)
      INTEGER       NatmsSrfCh         ! number of channels
      INTEGER       atmsSrfChn(NatmsSrfChMx) ! channels to use in PPASURF
      REAL          atmsSrfThresh      ! threshold to detect cloud in cost function
      REAL          atmsmeanBT(NatmsSrfTyp,NatmsSecAng,NatmsSrfChMx)  ! mean BT vectors
      REAL          atmscovBT(NatmsSrfTyp,NatmsSecAng,NatmsSrfChMx,
     &                   NatmssrfChMx) ! BT cov. mx.
      REAL          atmsSecAng(NatmsSecAng) ! secant(angles) for the above
!
! channel ignore status
      LOGICAL       atms_ignore_channel(atmsnchan)

      COMMON / PPATMSFDATA / atmscorrvn, atmspcpvn, atmssurfvn,
     &                    atmsbiases, atmsmaxscat, atmsscat,
     &                    atmsmaxcirr, atmscirr,
     &                    atmscrosby1, atmscrosby2, atmscrosby3, 
     &                    atmsmaxcrosby,
     &                    atmsgrody1, atmsgrody2,
     &                    atmsmeanBT, atmscovBT, atmsSecAng, 
     &                    NatmsSrfCh, atmsSrfChn, atmsSrfThresh,
     &                    atms_ignore_channel
