!+ ATMS Data Variables for processing within ATOVPP
!
! Name:  ppatmsstore.h
!
! Purpose:
!   'INCLUDE' file for COMMON / ppatmsstore /
!   containing ATMS data for use within ATOVIN code.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    13/08/2010  Created from ppastore.h (N.C.Atkinson)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
      INTEGER       atms_stvnnb     ! format version number of this file
         PARAMETER (atms_stvnnb = 1) !  - this code is set up for vn no 1
!
      INTEGER       atmsSatId         ! e.g. 15 for NOAA-15
!
      INTEGER       atmsrecdno(atmsnlines)               ! record number (1C & 1D)
!            note that atmsrecdno() = MDI indicates that the scanline is missing

      INTEGER       atmslineno(atmsnlines)               ! scanline number
      INTEGER       atmsyear(atmsnlines)                 ! scanline year
      INTEGER       atmsday(atmsnlines)                  ! scanline day of year
      INTEGER       atmstime(atmsnfovs,atmsnlines)       ! scanline time of day (ms)
      INTEGER       atmsgranulequal(atmsnlines)          !quality indicator bit field
      INTEGER       atmsscanqual(atmsnlines)             ! scan line quality flags
      INTEGER       atmschanqual(atmsnchan,atmsnlines)! channel quality flags
      INTEGER       atmsgeolocqual(atmsnlines)           ! geolocation quality flags

      REAL          atmslats(atmsnfovs,atmsnlines)          ! latitudes (deg, N +ve)
      REAL          atmslons(atmsnfovs,atmsnlines)          ! longitudes (deg, E +ve)
      INTEGER       atmselev(atmsnfovs,atmsnlines)          ! surface height (m)
      INTEGER       atmsLandFraca(atmsnfovs,atmsnlines)     ! Land fraction (percent)
      INTEGER       atmsLandFracb(atmsnfovs,atmsnlines)     ! Land fraction (percent)
      INTEGER       atmssurftype(atmsnfovs,atmsnlines)      !surface type(0=sea,1=mixed,2=land)
      REAL          atmsangles(4,atmsnfovs,atmsnlines)      !scan angles for atmsnfovs

      INTEGER       atmsaltitude(atmsnlines)             ! satellite altitude (km*10)
                                                   ! above reference ellipsoid

      REAL          atmsnedtcold(atmsnchan,atmsnlines)! Cold view NEDT
      REAL          atmsnedtwarm(atmsnchan,atmsnlines)! Warm view NEDT

      REAL          atmsbtemps(atmsnchan,atmsnfovs,atmsnlines) ! ATMS&B BTs (K)

      INTEGER       atmsqc(atmsnfovs,atmsnlines)            ! ATOVPP QC word for AMSU

      INTEGER       atmsppsurf(atmsnfovs,atmsnlines)       ! estimated surface type
!
      REAL          atmssurfcost(atmsnfovs,atmsnlines) ! max prob cloud test (ppatmssurf)
      REAL          atmsscatindx(atmsnfovs,atmsnlines) ! scattering index (ppatmsscat)
      REAL          atmscirrindx(atmsnfovs,atmsnlines) ! cirrus index (ppatmscirr)
      REAL          atmscrosby(atmsnfovs,atmsnlines)   ! logistic precip test (ppatmscrosby)
      REAL          atmssatht               ! nominal sat height
      REAL          atmssatperiod           ! nominal orbit period (mins)
      INTEGER       atmshdrcnt              !count of header records in 1C/1D file
      INTEGER       atmsscnlin              !count of scatmsnlines in 1C/1D file
      INTEGER       atmsinstruments         ! instruments present
!                   (bit0=HIRS, bit1=MSU, bit3=AMSU-A, bit4=AMSU-B, bit5=AVHRR,
!                    bit6=IASI, bit7=ATMS, bit8=CRIS)
      INTEGER       atmsyear1        ! year of first "good" ATMS scanline
      INTEGER       atmsday1         ! day
      INTEGER       atmstime1        ! time
      INTEGER       atmsline1        ! scanline no.
      REAL          atmsm0(atmsnfovs,atmsnlines)         !Precipitation prob - none
      REAL          atmsm1(atmsnfovs,atmsnlines)         !Precipitation prob - light
      REAL          atmsm2(atmsnfovs,atmsnlines)         !Precipitation prob - moderate
      REAL          atmsm3(atmsnfovs,atmsnlines)         !Precipitation prob - intense
      REAL          atmssi_nwcsaf(atmsnfovs,atmsnlines)      !Scattering index (K)
      INTEGER       atmssi_nwcsaf_flag(atmsnfovs,atmsnlines) !Scattering index flag

      COMMON / ppatmsstore / atmsSatId, atmsrecdno, atmslineno, 
     & atmsyear, atmsday, atmstime,
     & atmsgranulequal, atmsscanqual, atmsgeolocqual, atmschanqual, 
     & atmslats, atmslons, atmselev, atmsLandFraca, atmsLandFracb,
     & atmssurftype, 
     & atmsangles, atmsaltitude, atmsnedtcold, atmsnedtwarm,
     & atmsbtemps, atmsqc, atmsppsurf, atmssurfcost, atmsscatindx, 
     & atmscirrindx, atmscrosby, atmssatht,
     & atmssatperiod, atmshdrcnt, atmsscnlin, atmsinstruments,
     & atmsyear1, atmsday1, atmstime1, atmsline1,
     & atmsm0, atmsm1, atmsm2, atmsm3, atmssi_nwcsaf, atmssi_nwcsaf_flag
