!+ AMSU-B Data Variables for processing within ATOVPP
!
! Name:  ppastore.h
!
! Purpose:
!   'INCLUDE' file for COMMON / ppastore /
!   containing AMSU-B data for use within ATOVIN code.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0        12/04/96  Original code. (R.J.Renshaw)
!  1.1        12/12/96  Calibration flagword deleted for 1D format vn 1.8
!  1.2        10/03/98  Instrument temperature and solar angles added
!  1.3        15/06/98  BSatId added
!  1.4        29/06/98  Variables added for mapping AMSU-A to AMSU-B grid
!  1.5        02/07/98  Bangles,Bsurfcost,Bscatindx REAL not INTEGER
!  1.6        14/08/00  Bcirrindx new (D.C.Jones)
!  1.7        15/03/05  Add BLandFrac and nwcsaf scattering variables (NCA)
!  1.8        12/08/05  Add Byear1, Bday1, Btime1, Bline1 (NCA)
!
!  Size of common block is 1029 4-byte words / AMSU-B scanline    + 6*4 bytes
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
      INTEGER       amb_stvnnb     ! format version number of this file
         PARAMETER (amb_stvnnb = 140) !  - this code is set up for vn no 1.30
!
      INTEGER       BSatId         ! e.g. 15 for NOAA-15
!
      INTEGER       Brecdno(Bnlines)               ! record number (1C & 1D)
!            note that Brecdno() = MDI indicates that the scanline is missing
      INTEGER       Blineno(Bnlines)               ! scanline number
      INTEGER       Byear(Bnlines)                 ! scanline year
      INTEGER       Bday(Bnlines)                  ! scanline day of year
      INTEGER       Btime(Bnlines)                 ! scanline time of day (ms)
      INTEGER       Bqualind(Bnlines)              ! quality indicator bit field
!                 bit 31: do not use scan for product generation
!                 bit 30: time sequence error detected with this scan
!                 bit 29: data gap precedes this scan
!                 bit 28: no calibration
!                 bit 27: no earth location
!                 bit 26: first good time following a clock update
!                 bit 25: instrument status changed with this scan
!                 bit 24-0: spare  <zero fill>
!
      INTEGER       Bscanqual(Bnlines)             ! scan line quality flags
!                 time problem code
!                 (all bits off implies the scan time is as expected)
!                 bit 31-24: spare   <zero fill>
!                 bit 23: time field is bad but can probably be inferred
!                         from the previous good time
!                 bit 22: time field is bad and can't be inferred
!                         from the previous good time
!                 bit 21: this record starts a sequence that is inconsistent
!                         with previous times (i.e., there is a time
!                         discontinuity). This may or may not be associated
!                         with a spacecraft clock update. (see bit 26 above)
!                 bit 20: start of a sequence that apparently repeats scan
!                         times that have been previously accepted.
!                 bits 19-16: spare
!
!                 calibration problem code
!
!                 (all bits set to 0 indicates normal calibration)
!                  bit 15: scan line was not calibrated because of bad time
!                  bit 14: scan line was calibrated using fewer than the
!                          preferred number of scan lines because of
!                          proximity to start or end of data set or
!                          to a data gap.
!                  bit 13: scan line was not calibrated because of bad or
!                          insufficient PRT data
!                  bit 12: scan line was calibrated but with marginal PRT data
!                  bit 11: some uncalibrated channels of this scan.
!                          See channel indicators.
!                  bit 10: uncalibrated due to instrument mode
!                  bits 9 and 8: spare
!
!                  Earth location problem code
!
!                  (all bits set to 0 implies the Earth location was normal)
!                  bit 7: not Earth located because of bad time
!                  bit 6: Earth location questionable because of questionable
!                         time code (see time problem flags above)
!                  bit 5: Earth location questionable. Only marginal agreement
!                         with reasonableness check.
!                  bit 4: Earth location questionable. Fails reasonableness
!                         check
!                  bit 3: Earth location questionable because of antenna
!                         position check [rs060794.doc & rs062094.do1]
!                  bit 2-0: spare
!
      INTEGER       Binstrtemp(Bnlines)            ! AMSU-B Mixer ch18-20 temp
      REAL          Blats(Bnfovs,Bnlines)          ! latitudes (deg, N +ve)
      REAL          Blons(Bnfovs,Bnlines)          ! longitudes (deg, E +ve)
      INTEGER       Belev(Bnfovs,Bnlines)          ! surface height (m)
      INTEGER       BLandFrac(Bnfovs,Bnlines)      ! Land fraction (percent)
      INTEGER       BMapLandFrac(Bnfovs,Bnlines)   ! Mapped Land fraction
      INTEGER       Bsurftype(Bnfovs,Bnlines)!surface type(0=sea,1=mixed,2=land)
      REAL          Bangles(4,Bnfovs,Bnlines)      !scan angles for Bnfovs
!                                          (1,*,*) local zenith angle (deg)
!                                          (2,*,*) local azimuth angle (deg)
!                                          (3,*,*) solar zenith angle (deg)
!                                          (4,*,*) solar azimuth angle (deg)
      INTEGER       Baltitude(Bnlines)             ! satellite altitude (km*10)
                                                   ! above reference ellipsoid
      REAL          Bbtemps(Bnchan,Bnfovs,Bnlines) ! AMSU-B BTs (K)
      REAL          BmapBT(Anchan,Bnfovs,Bnlines)  ! AMSU-A mapped BTs (K)
      INTEGER       Bqc(Bnfovs,Bnlines)            ! ATOVPP QC word for AMSU
!                            (all bits off implies acceptable data)
!                  bit 31: spare   <zero fill>
!                  bit 30: set if AMSU-B used secondary calibration
!                  bit 29: set if AMSU-A used secondary calibration
!                  bit 28: set if AMSU-A data missing
!                 From nearest neighbour AMSU-A fov:
!                  bit 27: maximum probability scheme cloud flag
!                  bit 26: scattering test (only set over the sea)
!                  bit 25: logistic precipitation probability test
!                  bit 24: Grody light rainfall test
!                  bit 23: mismatch between AMSU-A/B 89GHz values
!                  bit 22: mismatch between surface type from topography
!                          dataset and from pre-processing
!                  bit 21:   spare   <zero fill>
!                 From AMSU-B fov:
!                  bit 20: set if 89GHz channel greatly different from
!                          that in surrounding fovs (PPMEDIAN)
!                  bit 19: scattering test (only set over the sea)  - using
!                                                       AMSU-B 89GHz channel
!                  bit 18: mismatch between AMSU-A/B 89GHz values
!                  bit 17-1: cirrus test (only over sea and if SI test passed.
!                  bit 16-1: spare   <zero fill>
!                  bit 0:    bad or missing data (in any or all channels)
!
      INTEGER       Bmapfov(Bnfovs,Bnlines)        ! nearest AMSU-A fov
      INTEGER       Bmapsurf(Bnfovs,Bnlines)       ! estimated surface type
                                                   !  (word 2 in amb1d_prepro)
      REAL          Bsurfcost(Bnfovs,Bnlines)      ! AMSU-A max prob cloud test
                                                   !  (word 3 in amb1d_prepro)
      REAL          Bscatindx(Bnfovs,Bnlines)      ! AMSU-A scattering index
                                                   !  (word 4 in amb1d_prepro)
      REAL          Bcirrindx(Bnfovs,Bnlines)      ! AMSU-AB cirrus index
                                                   !  (word 5 in amb1d_prepro)
      REAL          Bsatht                         ! nominal sat height
      REAL          Bsatperiod                     ! nominal orbit period (mins)
      INTEGER       Bhdrcnt              !count of header records in 1C/1D file
      INTEGER       Bscnlin              !count of scanlines in 1C/1D file
      INTEGER       Binstruments                   ! instruments present
!                   (bit0=HIRS, bit1=MSU, bit3=AMSU-A, bit4=AMSU-B, bit5=AVHRR)
      REAL          Bm0(Bnfovs,Bnlines)         !Precipitation prob - none
      REAL          Bm1(Bnfovs,Bnlines)         !Precipitation prob - light
      REAL          Bm2(Bnfovs,Bnlines)         !Precipitation prob - moderate
      REAL          Bm3(Bnfovs,Bnlines)         !Precipitation prob - intense
      REAL          Bsi_nwcsaf(Bnfovs,Bnlines)      !Scattering index (K)
      INTEGER       Bsi_nwcsaf_flag(Bnfovs,Bnlines) !Scattering index flag
      INTEGER       Byear1        ! year of first "good" AMSU-B scanline
      INTEGER       Bday1         ! day
      INTEGER       Btime1        ! time
      INTEGER       Bline1        ! scanline no.

      COMMON / ppbstore / BSatId, Brecdno, Blineno, Byear, Bday,
     & Btime, Bqualind, Bscanqual, Binstrtemp, Blats, Blons, Belev,
     & BLandFrac, BMapLandFrac, Bsurftype, Bangles, Baltitude, Bbtemps,
     & BmapBT, Bqc, Bmapfov, Bmapsurf, Bsurfcost, Bscatindx, Bcirrindx,
     & Bsatht, Bsatperiod, Bhdrcnt, Bscnlin, Binstruments,
     & Bm0, Bm1, Bm2, Bm3, Bsi_nwcsaf, Bsi_nwcsaf_flag,
     & Byear1, Bday1, Btime1, Bline1
