!+ Numbers for processing CRIS within ATOVPP
!
! Name:  ppcrisfdata.h
!
! Purpose:
!   'INCLUDE' file containing for processing CRIS within ATOVPP.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    16/08/2010 from ppifdata.h
!  1.1    20/06/2012 Shorten name of "Use_reconstructed_radiances_cris" to less
!                    than 32 characters. NCA
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
      INTEGER     crisboxSize        ! size of grid-box used in SURFELEV
      PARAMETER ( crisboxSize = 20 )! km

      INTEGER thinmode_cris !Thinning mode
      INTEGER nrec_cris     !Number of 1d records to output per scan: 1 or 9
      INTEGER FirstFOV_cris !First FOV to process (1 to 9)
      INTEGER LastFOV_cris  !Last FOV to process (1 to 9)
      INTEGER npc_cris      !Number of PC scores to output in 1d
      INTEGER nchan_cris    !Number of channels to output in 1d
      INTEGER default_cris  !Default detector for thinning
      INTEGER crischan      !cris channel to use in thinning tests
      INTEGER OKdetector_cris(9) !Status of each detector (1=OK, 0=not OK)
      INTEGER detector_use_cris(9) !List of detectors to output

      INTEGER channels_cris(crisnchan)     !List of channels
      INTEGER pccount_cris                 !Number of PCs in eigenvector file
      INTEGER eigformat_cris               !TBD
      INTEGER chanvn_cris                  !Channel selection version number
      INTEGER eigvn_cris                   !PC version number
      INTEGER startpcchan_cris(3)          !Start channel for each PC band
      INTEGER endpcchan_cris(3)            !End channel for each PC band
      INTEGER startpc_cris(3)              !Start PC for each PC band
      INTEGER endpc_cris(3)                !End PC for each PC band
      INTEGER npcbands_cris                !Number of PC bands
      CHARACTER*4 evlevel_cris             !'1B' or '1C'
      CHARACTER*80 evfile_cris             !eigenvector file names
      
      REAL    mean_encode_cris(crisnchan)     !Mean normalised radiance
      REAL    mean_decode_cris(crisnchan)     !Mean radiance for decoding
      REAL    eig_encode_cris(crisnchan,crismaxnpc)   !eigenvectors for encoding PCs
      REAL    eig_decode_cris(crisnchan,crismaxnpc)   !eigenvectors for decoding PCs
      REAL    nedr_cris(crisnchan)                    !Noise profile
      REAL    latlimits_cris(2)                       !Lat/lon limits for hybrid thinning
      LOGICAL Use_reconstructed_rad_cris      !Used by atovpp with PC input data 
!
      COMMON / PPCRISFDATA / thinmode_cris, nrec_cris, 
     & FirstFOV_cris, LastFOV_cris, npc_cris, nchan_cris, default_cris, 
     & crischan, OKdetector_cris, detector_use_cris, channels_cris,
     & pccount_cris, eigformat_cris, chanvn_cris, eigvn_cris,
     & startpcchan_cris, endpcchan_cris, startpc_cris, endpc_cris,
     & npcbands_cris, mean_encode_cris, mean_decode_cris,
     & eig_encode_cris, eig_decode_cris, nedr_cris, latlimits_cris,
     & Use_reconstructed_rad_cris, evfile_cris
