!+ CrIS Data Variables for processing within ATOVPP
!
! Name:  ppcrisstore.h
!
! Purpose:
!   'INCLUDE' file for COMMON / ppcrisstore /
!   containing IASI data for use within ATOVPP code.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    13/08/2010  Created from ppistore.h (N.C.Atkinson)
!  1.1    03/07/2012  Correct crisnedtcoldatms and crisnedtwarmatms
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
      INTEGER       cris_stvnnb     ! format version number of this file
         PARAMETER (cris_stvnnb = 1) !  - this code is set up for vn no 1
!
      INTEGER       crisSatId         ! e.g. 15 for NOAA-15
      INTEGER       criswmoSatId      ! WMO identifier
!
      INTEGER       crisrecdno(crisnlines)               ! record number (1C)
!            note that Irecdno() = MDI indicates that the scanline is missing
      INTEGER       crislineno(crisnlines)               ! scanline number
      INTEGER       crisyear(crisnlines)                 ! scanline year
      INTEGER       crisday(crisnlines)                  ! scanline day of year
      INTEGER       cristime(crisnfor,crisnlines)        ! scanline time of day (ms)
!
      REAL          crislats(crisnfovs,crisnlines)     ! latitudes (deg, N +ve)
      REAL          crislons(crisnfovs,crisnlines)     ! longitudes (deg, E +ve)
      REAL          crisangles(4,crisnfovs,crisnlines) !scan angles
!                                          (1,*,*) local zenith angle (deg)
!                                          (2,*,*) local azimuth angle (deg)
!                                          (3,*,*) solar zenith angle (deg)
!                                          (4,*,*) solar azimuth angle (deg)
      
! The following are generated by atovpp
      INTEGER       crissurftype(crisnfovs,crisnlines) !surface type(0=sea,1=mixed,2=land)
      INTEGER       criselev(crisnfovs,crisnlines)     !elevation in m (surfelev)
      INTEGER       crislandfrac(crisnfovs,crisnlines) !land fraction in % (surfelev)

      REAL          crismapbt(atmsnchan,crisnfovs,crisnlines) ! ATMS BTs (K)
      REAL          crisnedtcoldatms(atmsnchan,crisnlines)
      REAL          crisnedtwarmatms(atmsnchan,crisnlines)

      INTEGER       crisqc(crisnfovs,crisnlines)       ! ATOVPP QC word for CrIS
!                                                (becomes cris1d_crisqual in 1D)

      INTEGER       crismapqc(crisnfovs,crisnlines)   ! ATOVPP QC word for ATMS
                                                     !  (word 1 in cris1d_prepro)
      INTEGER       crismapsurf(crisnfovs,crisnlines) ! estimated surface type
                                                     !  (word 2 in cris1d_prepro)
      REAL          crissurfcost(crisnfovs,crisnlines)! ATMS max prob cloud test
                                                     !  (word 3 in cris1d_prepro)
      REAL          crisscatindx(crisnfovs,crisnlines)! ATMS scattering index
                                                     !  (word 4 in cris1d_prepro)
      REAL          criscirrindx(crisnfovs,crisnlines)! ATMS cirrus index
                                                     !  (word 5 in cris1d_prepro)
      REAL          criscrosby(crisnfovs,crisnlines)  ! ATMS logistic precip test
                                                     !  (word 6 in cris1d_prepro)
      REAL          crissatht               ! nominal sat height
      REAL          crissatperiod           ! nominal orbit period (mins)
      INTEGER       crishdrcnt              !count of header records in 1C/1D file
      INTEGER       crisscnlin              !count of scanlines in 1C/1D file
      INTEGER       crisinstruments                   ! instruments present

      INTEGER       crisbestFOV(crisnfovs,crisnlines)  !For use with CrIS thinning
      INTEGER       crisCloudflag(10,crisnfovs,crisnlines)  !For use with cloud tests
      INTEGER       crisyear1        ! year of first "good" HIRS scanline
      INTEGER       crisday1         ! day
      INTEGER       cristime1        ! time
      INTEGER       crisline1        ! scanline no.
      REAL          crism0(crisnfovs,crisnlines)         !Precipitation prob - none
      REAL          crism1(crisnfovs,crisnlines)         !Precipitation prob - light
      REAL          crism2(crisnfovs,crisnlines)         !Precipitation prob - moderate
      REAL          crism3(crisnfovs,crisnlines)         !Precipitation prob - intense
      REAL          crissi_nwcsaf(crisnfovs,crisnlines)      !Scattering index (K)
      INTEGER       crissi_nwcsaf_flag(crisnfovs,crisnlines) !Scattering index flag

      REAL    crisCloudW
      INTEGER crisCloudindex     !Position of cloud chan in the channel array

      COMMON / ppcstore / crisSatId, criswmoSatId, crisrecdno, 
     & crislineno, crisyear, crisday, cristime, crislats, crislons,
     & crisangles, crissurftype, criselev, crislandfrac,
     & crismapbt, crisnedtcoldatms, crisnedtwarmatms,
     & crisqc, crismapqc, crismapsurf, crissurfcost,
     & crisscatindx, criscirrindx, criscrosby,
     & crissatht, crissatperiod,crishdrcnt,
     & crisscnlin, crisinstruments, crisbestFOV, crisCloudflag,
     & crisyear1, crisday1, crisTime1, crisline1,
     & crism0, crism1, crism2, crism3, crissi_nwcsaf,
     & crissi_nwcsaf_flag, crisCloudW, crisCloudindex
