!+ Numbers for processing HIRAS within ATOVPP
!
! Name:  pphirasfdata.h
!
! Purpose:
!   'INCLUDE' file containing for processing HIRAS within ATOVPP.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    11/02/2019 from ppcfdata.h
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
      INTEGER     hirasboxSize        ! size of grid-box used in SURFELEV
      PARAMETER ( hirasboxSize = 20 )! km

      INTEGER thinmode_hiras !Thinning mode
      INTEGER nrec_hiras     !Number of 1d records to output per scan: 1 or 9
      INTEGER FirstFOV_hiras !First FOV to process (1 to 9)
      INTEGER LastFOV_hiras  !Last FOV to process (1 to 9)
      INTEGER npc_hiras      !Number of PC scores to output in 1d
      INTEGER nchan_hiras    !Number of channels to output in 1d
      INTEGER default_hiras  !Default detector for thinning
      INTEGER hiraschan      !hiras channel to use in thinning tests
      INTEGER OKdetector_hiras(4) !Status of each detector (1=OK, 0=not OK)
      INTEGER detector_use_hiras(4) !List of detectors to output

      INTEGER channels_hiras(hirasnchan)     !List of channels
      INTEGER pccount_hiras                 !Number of PCs in eigenvector file
      INTEGER eigformat_hiras               !TBD
      INTEGER chanvn_hiras                  !Channel selection version number
      INTEGER eigvn_hiras                   !PC version number
      INTEGER startpcchan_hiras(3)          !Start channel for each PC band
      INTEGER endpcchan_hiras(3)            !End channel for each PC band
      INTEGER startpc_hiras(3)              !Start PC for each PC band
      INTEGER endpc_hiras(3)                !End PC for each PC band
      INTEGER npcbands_hiras                !Number of PC bands
      CHARACTER*4 evlevel_hiras             !'1B' or '1C'
      CHARACTER*80 evfile_hiras             !eigenvector file names
      
      REAL    mean_encode_hiras(hirasnchan)     !Mean normalised radiance
      REAL    mean_decode_hiras(hirasnchan)     !Mean radiance for decoding
      REAL    eig_encode_hiras(hirasnchan,hirasmaxnpc)   !eigenvectors for encoding PCs
      REAL    eig_decode_hiras(hirasnchan,hirasmaxnpc)   !eigenvectors for decoding PCs
      REAL    nedr_hiras(hirasnchan)                    !Noise profile
      REAL    latlimits_hiras(2)                       !Lat/lon limits for hybrid thinning
      LOGICAL Use_reconstructed_rad_hiras      !Used by atovpp with PC input data 
!
      COMMON / PPHIRASFDATA / thinmode_hiras, nrec_hiras, 
     & FirstFOV_hiras, LastFOV_hiras, npc_hiras, nchan_hiras, 
     & default_hiras, 
     & hiraschan, OKdetector_hiras, detector_use_hiras, channels_hiras,
     & pccount_hiras, eigformat_hiras, chanvn_hiras, eigvn_hiras,
     & startpcchan_hiras, endpcchan_hiras, startpc_hiras, endpc_hiras,
     & npcbands_hiras, mean_encode_hiras, mean_decode_hiras,
     & eig_encode_hiras, eig_decode_hiras, nedr_hiras, latlimits_hiras,
     & Use_reconstructed_rad_hiras, evfile_hiras
