!+ HIRAS Data Variables for processing within ATOVPP
!
! Name:  pphirasstore.h
!
! Purpose:
!   'INCLUDE' file for COMMON / pphirasstore /
!   containing HIRAS data for use within ATOVPP code.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    07/02/2019  Created from ppcrisstore.h (N.C.Atkinson)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
      INTEGER       hiras_stvnnb     ! format version number of this file
         PARAMETER (hiras_stvnnb = 1) !  - this code is set up for vn no 1
!
      INTEGER       hirasSatId         ! e.g. 4 for FY-3D
      INTEGER       hiraswmoSatId      ! WMO identifier
!
      INTEGER       hirasrecdno(hirasnlines)               ! record number (1C)
!            note that Irecdno() = MDI indicates that the scanline is missing
      INTEGER       hiraslineno(hirasnlines)               ! scanline number
      INTEGER       hirasyear(hirasnlines)                 ! scanline year
      INTEGER       hirasday(hirasnlines)                  ! scanline day of year
      INTEGER       hirastime(hirasnfor,hirasnlines)        ! scanline time of day (ms)
!
      REAL          hiraslats(hirasnfovs,hirasnlines)     ! latitudes (deg, N +ve)
      REAL          hiraslons(hirasnfovs,hirasnlines)     ! longitudes (deg, E +ve)
      REAL          hirasangles(4,hirasnfovs,hirasnlines) !scan angles
!                                          (1,*,*) local zenith angle (deg)
!                                          (2,*,*) local azimuth angle (deg)
!                                          (3,*,*) solar zenith angle (deg)
!                                          (4,*,*) solar azimuth angle (deg)
      
! The following are generated by atovpp
      INTEGER       hirassurftype(hirasnfovs,hirasnlines) !surface type(0=sea,1=mixed,2=land)
      INTEGER       hiraselev(hirasnfovs,hirasnlines)     !elevation in m (surfelev)
      INTEGER       hiraslandfrac(hirasnfovs,hirasnlines) !land fraction in % (surfelev)

      REAL          hirasmapbt(mwtsnchan,hirasnfovs,hirasnlines) ! ATMS BTs (K)
      REAL          hirasnedtcoldmwts(mwtsnchan,hirasnlines)
      REAL          hirasnedtwarmmwts(mwtsnchan,hirasnlines)

      INTEGER       hirasqc(hirasnfovs,hirasnlines)       ! ATOVPP QC word for HIRAS
!                                                (becomes hiras1d_hirasqual in 1D)

      INTEGER       hirasmapqc(hirasnfovs,hirasnlines)   ! ATOVPP QC word for ATMS
                                                     !  (word 1 in hiras1d_prepro)
      INTEGER       hirasmapsurf(hirasnfovs,hirasnlines) ! estimated surface type
                                                     !  (word 2 in hiras1d_prepro)
      REAL          hirassurfcost(hirasnfovs,hirasnlines)! ATMS max prob cloud test
                                                     !  (word 3 in hiras1d_prepro)
      REAL          hirasscatindx(hirasnfovs,hirasnlines)! ATMS scattering index
                                                     !  (word 4 in hiras1d_prepro)
      REAL          hirascirrindx(hirasnfovs,hirasnlines)! ATMS cirrus index
                                                     !  (word 5 in hiras1d_prepro)
      REAL          hirascrosby(hirasnfovs,hirasnlines)  ! ATMS logistic precip test
                                                     !  (word 6 in hiras1d_prepro)
      REAL          hirassatht               ! nominal sat height
      REAL          hirassatperiod           ! nominal orbit period (mins)
      INTEGER       hirashdrcnt              !count of header records in 1C/1D file
      INTEGER       hirasscnlin              !count of scanlines in 1C/1D file
      INTEGER       hirasinstruments                   ! instruments present

      INTEGER       hirasbestFOV(hirasnfovs,hirasnlines)  !For use with HIRAS thinning
      INTEGER       hirasCloudflag(10,hirasnfovs,hirasnlines)  !For use with cloud tests
      INTEGER       hirasyear1        ! year of first "good" HIRS scanline
      INTEGER       hirasday1         ! day
      INTEGER       hirastime1        ! time
      INTEGER       hirasline1        ! scanline no.
      REAL          hirasm0(hirasnfovs,hirasnlines)         !Precipitation prob - none
      REAL          hirasm1(hirasnfovs,hirasnlines)         !Precipitation prob - light
      REAL          hirasm2(hirasnfovs,hirasnlines)         !Precipitation prob - moderate
      REAL          hirasm3(hirasnfovs,hirasnlines)         !Precipitation prob - intense
      REAL          hirassi_nwcsaf(hirasnfovs,hirasnlines)      !Scattering index (K)
      INTEGER       hirassi_nwcsaf_flag(hirasnfovs,hirasnlines) !Scattering index flag

      REAL    hirasCloudW
      INTEGER hirasCloudindex     !Position of cloud chan in the channel array

      COMMON / pphirasstore / hirasSatId, hiraswmoSatId, hirasrecdno, 
     & hiraslineno, hirasyear, hirasday, hirastime, hiraslats,hiraslons,
     & hirasangles, hirassurftype, hiraselev, hiraslandfrac,
     & hirasmapbt, hirasnedtcoldmwts, hirasnedtwarmmwts,
     & hirasqc, hirasmapqc, hirasmapsurf, hirassurfcost,
     & hirasscatindx, hirascirrindx, hirascrosby,
     & hirassatht, hirassatperiod,hirashdrcnt,
     & hirasscnlin, hirasinstruments, hirasbestFOV, hirasCloudflag,
     & hirasyear1, hirasday1, hirasTime1, hirasline1,
     & hirasm0, hirasm1, hirasm2, hirasm3, hirassi_nwcsaf,
     & hirassi_nwcsaf_flag, hirasCloudW, hirasCloudindex
