!+ Numbers for processing IASI within ATOVPP
!
! Name:  ppifdata.h
!
! Purpose:
!   'INCLUDE' file containing for processing IASI within ATOVPP.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0      10/08/05  Original code. (N.C.Atkinson)
!  1.1      16/01/06  Add extra variables.
!  1.2      20/03/07  Add OKdetector
!  1.3      22/01/08  Add pccount, eigformat, evlevel, mean_encode_IASI,
!                     mean_decode_IASI
!  1.4      15/09/08  Use Imaxnchan/Imaxnpc instead of iasi_1dnchan/iasi_1dnpc.
!                     Add chanvn, eigvn.
!  1.5      04/03/09  Add startpcchan,endpcchan,startpc,endpc,npcbands
!  1.6      01/09/09  Add evfile
!  1.7      05/09/17  Change Iboxsize to 50km. Used at Met Office for several
!                     years and found to be beneficial in allowing more IASI
!                     observations to be used in NWP.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
      INTEGER     IboxSize        ! size of grid-box used in SURFELEV
      PARAMETER ( IboxSize = 50 )! km

!Band definitions. The following are nominal values taken from the
!IASI Level 1 Products Guide, EUM/OPS-EPS/MAN/04/0033. Actual values will differ
!and will be optimised post-launch.
!Band 1 = 645.0-1210.0 cm^-1
!Band 2 = 1210.0-2000.0 cm^-1
!Band 3 = 2000.0-2760.0 cm^-1
!
!04/03/2009: Change to using the lower end of band overlap, as recommended by
!EUMETSAT in the context of the PC compression product, based on Table of
!Parameters (TOP) set 11. 
      INTEGER     Firstchan(3)
      DATA        Firstchan/1,1998,5117/       !First channel for each detector
      INTEGER     Lastchan(3)
      DATA        Lastchan/1997,5116,8461/     !Last channel for each detector

      INTEGER thinmode_IASI !Thinning mode
      INTEGER thinmode_backup !Backup thinning mode in case AVHRR is missing
      INTEGER nrec_IASI     !Number of 1d records to output per scan: 1 or 4
      INTEGER FirstFOV_IASI !First FOV to process (1 to 4)
      INTEGER LastFOV_IASI  !Last FOV to process (1 to 4)
      INTEGER npc_IASI      !Number of PC scores to output in 1d
      INTEGER nchan_IASI    !Number of channels to output in 1d
      INTEGER default_IASI  !Default detector for thinning
      INTEGER AVHRRchan_IASI !AVHRR channel to use in thinning tests (1 to 6)
      INTEGER IASIchan      !IASI channel to use in thinning tests (1 to 8461)
      INTEGER OKdetector(4) !Status of each detector (1=OK, 0=not OK)

      INTEGER channels_IASI(Inchan)        !List of channels
      INTEGER pccount                      !Number of PCs in eigenvector file
      INTEGER eigformat                    !0=AAPP v6.1, 1=ECMWF
      INTEGER chanvn                       !Channel selection version number
      INTEGER eigvn                        !PC version number
      INTEGER startpcchan(3)               !Start channel for each PC band
      INTEGER endpcchan(3)                 !End channel for each PC band
      INTEGER startpc(3)                   !Start PC for each PC band
      INTEGER endpc(3)                     !End PC for each PC band
      INTEGER npcbands                     !Number of PC bands
      CHARACTER*4 evlevel                  !'1B' or '1C'
      CHARACTER*80 evfile(3)               !eigenvector file names
      
      REAL    mean_encode_IASI(Inchan)     !Mean normalised radiance
      REAL    mean_decode_IASI(Inchan)     !Mean radiance for decoding
      REAL    eig_encode_IASI(Inchan,Imaxnpc)    !eigenvectors for encoding PCs
      REAL    eig_decode_IASI(Inchan,Imaxnpc)    !eigenvectors for decoding PCs
      REAL    nedr_IASI(Inchan)                  !Noise profile (Gauss apodised)
      REAL    mtf_IASI(45,512)                   !MTF correction for 45 bands
      REAL    latlimits(2)                       !Lat/lon limits for hybrid thinning
      LOGICAL Use_reconstructed_radiances  !Used by atovpp with PC input data 
!
      COMMON / PPIFDATA / thinmode_IASI, thinmode_backup, nrec_IASI, 
     & FirstFOV_IASI, LastFOV_IASI, npc_IASI, nchan_IASI, default_IASI, 
     & AVHRRchan_IASI, IASIchan, OKdetector, channels_IASI,
     & pccount, eigformat, chanvn, eigvn,
     & startpcchan, endpcchan, startpc, endpc, npcbands,
     & evlevel, mean_encode_IASI,
     & mean_decode_IASI,
     & eig_encode_IASI, eig_decode_IASI, nedr_IASI, mtf_IASI, latlimits,
     & Use_reconstructed_radiances,evfile
