!+ IASI Data Variables for processing within ATOVPP
!
! Name:  ppistore.h
!
! Purpose:
!   'INCLUDE' file for COMMON / ppistore /
!   containing IASI data for use within ATOVPP code.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0      26/01/05  Created from pphstore.h (N.C.Atkinson)
!  1.1      26/03/07  bug fix: add IwmoSatId, Iangles, Ialtitude to common block
!                     Update dimension of ICloudflag.
!
!  Size of common block is .....
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
      INTEGER       iasi_stvnnb     ! format version number of this file
         PARAMETER (iasi_stvnnb = 1) !  - this code is set up for vn no 1
!
      INTEGER       ISatId         ! e.g. 15 for NOAA-15
      INTEGER       IwmoSatId      ! WMO identifier
!
      INTEGER       Irecdno(Inlines)               ! record number (1C)
!            note that Irecdno() = MDI indicates that the scanline is missing
      INTEGER       Ilineno(Inlines)               ! scanline number
      INTEGER       Iyear(Inlines)                 ! scanline year
      INTEGER       Iday(Inlines)                  ! scanline day of year
      INTEGER       Itime(Inlines)                 ! scanline time of day (ms)
!
      INTEGER       Iinstrtempa1(Inlines)          ! AMSU-A1 temperature
      INTEGER       Iinstrtempa2(Inlines)          ! AMSU-A2 temperature
      INTEGER       Iinstrtempb(Inlines)           ! AMSU-B temperature
      REAL          Ilats(Infovs,Inlines)          ! latitudes (deg, N +ve)
      REAL          Ilons(Infovs,Inlines)          ! longitudes (deg, E +ve)
      INTEGER       Ielev(Infovs,Inlines)          ! surface height (m)
      INTEGER       Isurftype(Infovs,Inlines)!surface type(0=sea,1=mixed,2=land)
      REAL          Iangles(4,Infovs,Inlines)      !scan angles for Hnfovs
!                                          (1,*,*) local zenith angle (deg)
!                                          (2,*,*) local azimuth angle (deg)
!                                          (3,*,*) solar zenith angle (deg)
!                                          (4,*,*) solar azimuth angle (deg)
      INTEGER       Ialtitude(Inlines)             ! satellite altitude (km*10)
                                                   ! above reference ellipsoid
      REAL          Imapbt(Anchan+Bnchan,Infovs,Inlines) ! AMSU-A&B BTs (K)
      INTEGER       Iqc(Infovs,Inlines)            ! ATOVPP QC word for IASI
!                                                (becomes iasi1d_iasiqual in 1D)
!                   (all bits off implies acceptable data)
!                  bit 23-4: bit n set to 1 if brightness temperature in
!                            AMSU channel n-3 is physically unreasonable or has
!                            not been calculated due to calibration  
!                            problems.
!                  bit 3-1:  bit n set to 1 if data for IASI detector n are bad
!                  bit 0:    bad or missing data (in any or all channels)

      INTEGER       Imapqc(Infovs,Inlines)         ! ATOVPP QC word for AMSU
                                                   !  (word 1 in iasi1d_prepro)
!                            (all bits off implies acceptable data)
!                  bit 31: set if AMSU-A surface types not all the same
!                  bit 30: set if AMSU-A used secondary calibration
!                  bit 29: set if AMSU-B used secondary calibration
!                  bit 28: set if AMSU-B data missing
!                  bit 27: flag for cloud cost set for any mapped AMSU-A
!                  bit 26: scattering flag set for any mapped AMSU-A
!                          (only set over the sea)
!                  bit 25: logistic precipitation probability test
!                          calculated from AMSU-A data mapped to IASI grid
!                  bit 24: Grody light rainfall test calculated on IASI grid
!                  bit 23: mismatch between AMSU-A/B 89GHz values (any AMSU-A)
!                  bit 22: mismatch between surface type from topography
!                          dataset and from pre-processing (any AMSU-A)
!                  bit 21-4: spare   <zero fill>
!                  bit 3: set when AVHRR channel 3 is albedo not brightness temp
!                  bit 2: flag for cloud cost (recalculated on IASI grid)
!                  bit 1: flag for scattering index (recalculated on IASI grid)
!                  bit 0: set if AMSU-A & B data missing
!
      INTEGER       Imapsurf(Infovs,Inlines)       ! estimated surface type
                                                   !  (word 2 in iasi1d_prepro)
      REAL          Isurfcost(Infovs,Inlines)      ! AMSU-A max prob cloud test
                                                   !  (word 3 in iasi1d_prepro)
      REAL          Iscatindx(Infovs,Inlines)      ! AMSU-A scattering index
                                                   !  (word 4 in iasi1d_prepro)
      REAL          Icrosby(Infovs,Inlines)        ! AMSU-A logistic precip test
                                                   !  (word 5 in iasi1d_prepro)
      REAL          Isatht                         ! nominal sat height
      REAL          Isatperiod                     ! nominal orbit period (mins)
      INTEGER       Ihdrcnt              !count of header records in 1C/1D file
      INTEGER       Iscnlin              !count of scanlines in 1C/1D file
      INTEGER       Iinstruments                   ! instruments present
!   (bit0=HIRS, bit1=MSU, bit3=AMSU-A, bit4=AMSU-B, bit5=AVHRR, bit 6=IASI)
!
      INTEGER       IbestFOV(30,Inlines)           !For use with IASI thinning
      INTEGER       ICloudflag(10,Infovs,Inlines)  !For use with cloud tests
      INTEGER       Iyear1        ! year of first "good" HIRS scanline
      INTEGER       Iday1         ! day
      INTEGER       Itime1        ! time
      INTEGER       Iline1        ! scanline no.
      INTEGER       ICloudchan                     !Cloud test channel index
      REAL          Scalefactors(Inchan)

      COMMON / ppistore / ISatId, IwmoSatId, Irecdno, Ilineno, Iyear,
     & Iday, Itime,
     & Iinstrtempa1, Iinstrtempa2,
     & Iinstrtempb, Ilats, Ilons, Ielev, Isurftype, Iangles, Ialtitude,
     & Imapbt, Iqc, Imapqc, Imapsurf, Isurfcost, Iscatindx,
     & Icrosby, Isatht, Isatperiod, Ihdrcnt, Iscnlin, Iinstruments,
     & IbestFOV, ICloudflag,
     & Iyear1, Iday1, ITime1, Iline1, ICloudchan, Scalefactors
