!+ Parameters for the mapping routines within ATOVPP
!
! Name:  ppluts.h
!
! Purpose:
!   'INCLUDE' file containing parameters for use within ATOVPP mapping.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0        12/04/96  Original code. (R.J.Renshaw)
!  1.1        29/06/98  Added A2BLUT, deleted LUTwtmode
!  1.2        23/06/05  Change A2Hncolocs from 15 to 20, to avoid error. NCA
!  1.3        12/08/05  Add IASI
!
! Note that many of the mappings are commented out.
! Their inclusion would require other modifications to ATOVPP,
! such as the coding of appropriate pre-processing tests and of
! the necessary FOV-selection routines PPH2B,..etc.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Instrument-independent parameters:
      INTEGER LUTelements     ! number of elements of information per colocation
        PARAMETER (LUTelements=10)! ok for everything except AVHRR to HIRS
      INTEGER LUTmultfact         ! mult factor (*100) applied to ellipse scan
        PARAMETER (LUTmultfact=20)
!
! Mapping-specific parameters:
!
! 1.  Mapping to HIRS grid
!     ------- -- ---- ----
!
! 1.1 AMSU-A to HIRS
      INTEGER A2Hnlines           ! number of target lines in duration (HIRS)
         PARAMETER (A2Hnlines=5)  ! implies LUT of 32 secs
      INTEGER A2Hncolocs          ! max number of colocations
        PARAMETER (A2Hncolocs=20) ! was 15 in AAPP-4
      INTEGER A2Hmult             ! mult factor for target CONE_ANGLE (*10)
         PARAMETER (A2Hmult=70)
      INTEGER A2Hmode             ! mapping AMSU-A to HIRS: 1=nearest neighbour
         PARAMETER (A2Hmode=2)    ! 1=nearest neighbour, 2=bilinear interp,
                                  ! 3=spatial average with gaussian weighting
                                  ! 4=spatial average with linear weighting
      INTEGER A2Hstart            ! a `reasonable' target start line for LUT
                                  ! - will be adjusted if necessary.
      REAL    A2Htime_cor(Hnfovs) ! time correction for each scan position 
      REAL    A2Hscan_cor(Hnfovs) ! scan angle corrections
      REAL    A2HLUT (LUTelements, A2Hncolocs, Hnfovs, A2Hnlines)
                                  ! buffer to hold AMSU-A to HIRS mapping LUT
!
! 1.2 AMSU-B to HIRS
C      INTEGER B2Hnlines           ! number of target lines in duration (HIRS)
C         PARAMETER (B2Hnlines=5)  ! implies LUT of 32 secs
C      INTEGER B2Hncolocs          ! max number of colocations
C         PARAMETER (B2Hncolocs=25)
C      INTEGER B2Hmult             ! mult factor for target CONE_ANGLE (*10)
C         PARAMETER (B2Hmult=10)
C      INTEGER B2Hmode             ! mapping AMSU-B to HIRS: 1=nearest neighbour
C         PARAMETER (B2Hmode=3)    ! 1=nearest neighbour, 2=bilinear interp,
C                                  ! 3=spatial average with gaussian weighting
C                                  ! 4=spatial average with linear weighting
C      INTEGER B2Hstart            ! a `reasonable' target start line for LUT
C                                  ! - will be adjusted if necessary.
C      REAL    B2Htime_cor(Hnfovs) ! time correction for each scan position 
C      REAL    B2Hscan_cor(Hnfovs) ! scan angle corrections
C      REAL    B2HLUT (LUTelements, B2Hncolocs, Hnfovs, B2Hnlines)
C                                  ! buffer to hold AMSU-B to HIRS mapping LUT
!
! 1.3 MSU to HIRS
      INTEGER M2Hnlines           ! number of target lines in duration (HIRS)
         PARAMETER (M2Hnlines=20) ! implies LUT of 128 secs
      INTEGER M2Hncolocs          ! max number of colocations
        PARAMETER (M2Hncolocs=30)
      INTEGER M2Hmult             ! mult factor for target CONE_ANGLE (*10)
         PARAMETER (M2Hmult=220)
      INTEGER M2Hmode             ! mapping MSU to HIRS: 1=nearest neighbour
         PARAMETER (M2Hmode=2)    ! 1=nearest neighbour, 2=bilinear interp,
                                  ! 3=spatial average with gaussian weighting
                                  ! 4=spatial average with linear weighting
      INTEGER M2Hstart            ! a `reasonable' target start line for LUT
                                  ! - will be adjusted if necessary.
      REAL    M2Htime_cor(Hnfovs) ! time correction for each scan position 
      REAL    M2Hscan_cor(Hnfovs) ! scan angle corrections
      REAL    M2HLUT (LUTelements, M2Hncolocs, Hnfovs, M2Hnlines)
                                  ! buffer to hold MSU to HIRS mapping LUT
!
!
! 2.  Mapping to AMSU-A grid
!     ------- -- ------ ----
!
C! 2.1 HIRS to AMSU-A
C      INTEGER H2Anlines           ! number of target lines in duration (AMSU-A)
C         PARAMETER (H2Anlines=4)  ! implies LUT of 32 secs
C      INTEGER H2Ancolocs          ! max number of colocations
C        PARAMETER (H2Ancolocs=15)
C      INTEGER H2Amult             ! mult factor for target CONE_ANGLE (*10)
C         PARAMETER (H2Amult=70)
C      INTEGER H2Amode             ! mapping HIRS to AMSU-A: 1=nearest neighbour
C         PARAMETER (H2Amode=3)    ! 1=nearest neighbour, 2=bilinear interp,
C                                  ! 3=spatial average with gaussian weighting
C                                  ! 4=spatial average with linear weighting
C      INTEGER H2Astart            ! a `reasonable' target start line for LUT
C                                  ! - will be adjusted if necessary.
C      REAL    H2Atime_cor(Anfovs) ! time correction for each scan position 
C      REAL    H2Ascan_cor(Anfovs) ! scan angle corrections
C      REAL    H2ALUT (LUTelements, H2Ancolocs, Anfovs, H2Anlines)
C                                  ! buffer to hold HIRS to AMSU-A mapping LUT
!
! 2.2 AMSU-B to AMSU-A
      INTEGER B2Anlines           ! number of target lines in duration (AMSU-A)
      PARAMETER (B2Anlines=4)     ! implies LUT of 32 secs
      INTEGER B2Ancolocs          ! max number of colocations
        PARAMETER (B2Ancolocs=15)
      INTEGER B2Amult             ! mult factor for target CONE_ANGLE (*10)
      PARAMETER (B2Amult=10)
      INTEGER B2Amode             ! mapping AMSU-B to A: 1=nearest neighbour
         PARAMETER (B2Amode=3)    ! 1=nearest neighbour, 2=bilinear interp,
                                  ! 3=spatial averaging
                                  ! 3=spatial average with gaussian weighting
                                  ! 4=spatial average with linear weighting
                                  ! 5=BG-convolution
      INTEGER B2Astart            ! a `reasonable' target start line for LUT
                                  ! - will be adjusted if necessary.
      REAL    B2Atime_cor(Anfovs) ! time correction for each scan position 
      REAL    B2Ascan_cor(Anfovs) ! scan angle corrections
      REAL    B2ALUT (LUTelements, B2Ancolocs, Anfovs, B2Anlines)
                                  ! buffer to hold AMSU-B to AMSU-A mapping LUT
!
! 3.  Mapping to AMSU-B grid
!     ------- -- ------ ----
!
C! 3.1 HIRS to AMSU-B
C      INTEGER H2Bnlines           ! number of target lines in duration (AMSU-B)
C         PARAMETER (H2Bnlines=6)  ! implies LUT of 16 secs
C      INTEGER H2Bncolocs          ! max number of colocations
C         PARAMETER (H2Bncolocs=4)
C      INTEGER H2Bmult             ! mult factor for target CONE_ANGLE (*10)
C         PARAMETER (H2Bmult=100)
C      INTEGER H2Bmode             ! mapping HIRS to AMSU-B: 1=nearest neighbour
C         PARAMETER (H2Bmode=2)    ! 1=nearest neighbour, 2=bilinear interp,
C                                  ! 3=spatial average with gaussian weighting
C                                  ! 4=spatial average with linear weighting
C      INTEGER H2Bstart            ! a `reasonable' target start line for LUT
C                                  ! - will be adjusted if necessary.
C      REAL    H2Btime_cor(Bnfovs) ! time correction for each scan position 
C      REAL    H2Bscan_cor(Bnfovs) ! scan angle corrections
C      REAL    H2BLUT (LUTelements, H2Bncolocs, Bnfovs, H2Bnlines)
C                                  ! buffer to hold HIRS to AMSU-B mapping LUT
!
! 3.2 AMSU-A to AMSU-B
      INTEGER A2Bnlines           ! number of target lines in duration (AMSU-A)
      PARAMETER (A2Bnlines=6)     ! implies LUT of 16 secs
      INTEGER A2Bncolocs          ! max number of colocations
        PARAMETER (A2Bncolocs=4)
      INTEGER A2Bmult             ! mult factor for target CONE_ANGLE (*10)
      PARAMETER (A2Bmult=40)
      INTEGER A2Bmode             ! mapping AMSU-A to B: 1=nearest neighbour
         PARAMETER (A2Bmode=1)    ! 1=nearest neighbour, 2=bilinear interp,
                                  ! 3=spatial average with gaussian weighting
                                  ! 4=spatial average with linear weighting
      INTEGER A2Bstart            ! a `reasonable' target start line for LUT
                                  ! - will be adjusted if necessary.
      REAL    A2Btime_cor(Bnfovs) ! time correction for each scan position 
      REAL    A2Bscan_cor(Bnfovs) ! scan angle corrections
      REAL    A2BLUT (LUTelements, A2Bncolocs, Bnfovs, A2Bnlines)
                                  ! buffer to hold AMSU-A to AMSU-B mapping LUT

! 4.  Mapping to IASI grid
!     ------- -- ---- ----
!
! 4.1 AMSU-A to IASI
      INTEGER A2Inlines           ! number of target lines in duration (IASI)
         PARAMETER (A2Inlines=4)  ! implies LUT of 32 secs
      INTEGER A2Incolocs          ! max number of colocations
        PARAMETER (A2Incolocs=20) ! 
      INTEGER A2Imult             ! mult factor for target CONE_ANGLE (*10)
         PARAMETER (A2Imult=245)  ! 245 gives same cone angle as used for HIRS
      INTEGER A2Imode             ! mapping AMSU-A to IASI: 1=nearest neighbour
         PARAMETER (A2Imode=2)    ! 1=nearest neighbour, 2=bilinear interp,
                                  ! 3=spatial average with gaussian weighting
                                  ! 4=spatial average with linear weighting
      INTEGER A2Istart            ! a `reasonable' target start line for LUT
                                  ! - will be adjusted if necessary.
      REAL    A2Itime_cor(Inscanpos) ! time correction for each scan position 
      REAL    A2Iscan_cor(Inscanpos) ! scan angle corrections
      REAL    A2ILUT (LUTelements, A2Incolocs, Inscanpos, A2Inlines)
                                  ! buffer to hold AMSU-A to IASI mapping LUT
!
! 5.  Mapping to CRIS & HIRAS grid
!
      INTEGER atms2crismode
         PARAMETER (atms2crismode=2)  ! 1=nearest neighbour, 2=bilinear interp

      INTEGER mwts2hirasmode
         PARAMETER (mwts2hirasmode=2)  ! 1=nearest neighbour, 2=bilinear interp

! 6.  Choice of mappings
!     ------ -- --------
!
      INTEGER     Nmappings       ! number of mappings requested by user
      CHARACTER*6 MapInstr(10)    ! mapping instruments requested (e.g. "MSU")
      CHARACTER*6 TargInstr(10)   ! target instruments requested (e.g. "HIRS")

      COMMON / PPLUTS / A2HLUT, A2Hstart, A2Htime_cor, A2Hscan_cor
C     &                , B2HLUT, B2Hstart, B2Htime_cor, B2Hscan_cor
     &                , M2HLUT, M2Hstart, M2Htime_cor, M2Hscan_cor
C     &                , H2ALUT, H2Astart, H2Atime_cor, H2Ascan_cor
     &                , B2ALUT, B2Astart, B2Atime_cor, B2Ascan_cor
C     &                , H2BLUT, H2Bstart, H2Btime_cor, H2Bscan_cor
     &                , A2BLUT, A2Bstart, A2Btime_cor, A2Bscan_cor
     &                , A2ILUT, A2Istart, A2Itime_cor, A2Iscan_cor
     &                , Nmappings, MapInstr, TargInstr
