!+ MSU Data Variables for processing within ATOVPP
!
! Name:  ppmstore.h
!
! Purpose:
!   'INCLUDE' file for COMMON / ppmstore /
!   containing MSU data for use within ATOVIN code.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0        12/04/96  Original code. (R.J.Renshaw)
!  1.1        15/06/98  MSatId added
!  1.2        02/07/98  Mangles REAL not INTEGER
!  1.3        12/08/05  Add Myear1, Mday1, Mtime1, Mline1 (NCA)
!
!  Size of common block is 130 4-byte words / MSU scanline    + 4*4 bytes
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
      INTEGER       msu_stvnnb     ! format version number of this file
         PARAMETER (msu_stvnnb = 100) !  - this code is set up for vn no 1.00
!
      INTEGER       MSatId         ! e.g. 15 for NOAA-15
!
      INTEGER       Mrecdno(Mnlines)               ! record number (1C & 1D)
!            note that Mrecdno() = MDI indicates that the scanline is missing
      INTEGER       Mlineno(Mnlines)               ! scanline number
      INTEGER       Myear(Mnlines)                 ! scanline year
      INTEGER       Mday(Mnlines)                  ! scanline day of year
      INTEGER       Mtime(Mnlines)                 ! scanline time of day (ms)
      INTEGER       Mqualind(Mnlines)              ! quality indicator bit field
                                                   !  (msu1c_qualind in 1C)
      INTEGER       Mscanqual(Mnlines)             ! scan line quality flags
                                                   !  (msu1c_scanqual in 1C)
      REAL          Mlats(Mnfovs,Mnlines)          ! latitudes (deg, N +ve)
      REAL          Mlons(Mnfovs,Mnlines)          ! longitudes (deg, E +ve)
      INTEGER       Melev(Mnfovs,Mnlines)          ! surface height (m)
      INTEGER       Msurftype(Mnfovs,Mnlines)!surface type(0=sea,1=mixed,2=land)
      REAL          Mangles(2,Mnfovs,Mnlines)      !scan angles for Mnfovs
!                                          (1,*,*) local zenith angle (deg)
!                                          (2,*,*) local azimuth angle (deg)
      INTEGER       Maltitude(Mnlines)             ! satellite altitude (km*10)
                                                   ! above reference ellipsoid
      REAL          Mbtemps(Mnchan,Mnfovs,Mnlines) ! MSU brightness temps (K)

      INTEGER       Mqc(Mnfovs,Mnlines)          ! ATOVPP quality control word
!                                                  bit 31: bad fov
!                                                  bit 30: 2ndary calib used
!                                                  bits 29-5: spare (zero)
!                                                  bits 4-1:set if chan4-1 bad
!                                                  bit 0:set with any of above
      REAL          Msatht                         ! nominal sat height
      REAL          Msatperiod                     ! nominal orbit period (mins)
      INTEGER       Mhdrcnt              !count of header records in 1C/1D file
      INTEGER       Mscnlin              !count of scanlines in 1C/1D file
      INTEGER       Myear1        ! year of first "good" MSU scanline
      INTEGER       Mday1         ! day
      INTEGER       Mtime1        ! time
      INTEGER       Mline1        ! scanline no.

      COMMON / ppmstore / MSatId, Mrecdno, Mlineno, Myear, Mday, Mtime,
     & Mqualind, Mscanqual, Mlats, Mlons, Melev, Msurftype,
     & Mangles, Maltitude, Mbtemps, Mqc,
     & Msatht, Msatperiod, Mhdrcnt, Mscnlin,
     & Myear1, Mday1, Mtime1, Mline1
