!+ Numbers for processing MWTS within ATOVPP
!
! Name:  ppafdata.h
!
! Purpose:
!   'INCLUDE' file containing for processing MWTS within ATOVPP.
!
! History:
! Version    Date     Comment
!
!  1.0      11/02/2019 From ppmwhsfdata.h
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
      INTEGER     mwtsboxSize        ! size of grid-box used in SURFELEV
      PARAMETER ( mwtsboxSize = 80 ) ! km
!
! version numbers for the pre-processing
      INTEGER       mwtscorrvn       ! limb/emissivity correction (0=not done)
      INTEGER       mwtspcpvn        ! precip/scattering detection (0=not done)
      INTEGER       mwtssurfvn       ! surface identification (0=not done)
!
! bias corrections
      REAL          mwtsbiases(mwtsnchan,mwtsnfovs)  ! deg K, added to the obs
!
! numbers for scattering test PPASCAT
      REAL          mwtsmaxscat        ! threshold for scattering index (deg K)
      REAL          mwtsscat(4,mwtsnfovs) ! predict MWTS 16 from 1,2,3

! numbers for microwave cirrus index PPACIR test
      REAL          mwtsmaxcirr           ! threshold for cirrus index (deg K)
      REAL          mwtscirr(4,mwtsnfovs) ! predict channel 18 from 
                                          ! channels 1,16,17

!     The equation for each fov is T16 =A1 + T1*A2 + T2*A3 + T3*A4
!
! numbers for logistic precipitation probability PPCROSBY
      REAL          mwtscrosby1, mwtscrosby2, mwtscrosby3   ! coefficients
      REAL          mwtsmaxcrosby                           ! threshold
!
! numbers for Grody light rainfall test PPGRODY
      REAL          mwtsgrody1, mwtsgrody2
!
! numbers for surface identification PPASURF
      INTEGER       NmwtsSrfTyp        ! number of surface types
         PARAMETER (NmwtsSrfTyp=8)
      INTEGER       NmwtsSecAng        ! number of angles
         PARAMETER (NmwtsSecAng=5)
      INTEGER       NmwtsSrfChMx       ! max number of channels
         PARAMETER (NmwtsSrfChMx=10)
      INTEGER       NmwtsSrfCh         ! number of channels
      INTEGER       mwtsSrfChn(NmwtsSrfChMx) ! channels to use in PPASURF
      REAL          mwtsSrfThresh      ! threshold to detect cloud in cost function
      REAL          mwtsmeanBT(NmwtsSrfTyp,NmwtsSecAng,NmwtsSrfChMx)  ! mean BT vectors
      REAL          mwtscovBT(NmwtsSrfTyp,NmwtsSecAng,NmwtsSrfChMx,
     &                   NmwtssrfChMx) ! BT cov. mx.
      REAL          mwtsSecAng(NmwtsSecAng) ! secant(angles) for the above
!
! channel ignore status
      LOGICAL       mwts_ignore_channel(mwtsnchan)

      COMMON / PPMWTSFDATA / mwtscorrvn, mwtspcpvn, mwtssurfvn,
     &                    mwtsbiases, mwtsmaxscat, mwtsscat,
     &                    mwtsmaxcirr, mwtscirr,
     &                    mwtscrosby1, mwtscrosby2, mwtscrosby3, 
     &                    mwtsmaxcrosby,
     &                    mwtsgrody1, mwtsgrody2,
     &                    mwtsmeanBT, mwtscovBT, mwtsSecAng, 
     &                    NmwtsSrfCh, mwtsSrfChn, mwtsSrfThresh,
     &                    mwts_ignore_channel
