!+ Parameters for ATOVPP Processing
!
! Name:  ppparms.h
!
! Purpose:
!   'INCLUDE' file containing parameters for use within ATOVPP code.
!
!   dT may be tuned according to available memory on the
!   host machine   (- see AAPP documentation APP104).
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0       27/2/96  Original code. (R.J.Renshaw)
!  1.1        2/1/97  RMDI now REAL, not INTEGER
!  1.2       11/6/98  *cycle also REAL, not INTEGER
!  1.3       04/03/99 RMDI set to 9999.99 rather than 999999. (div by 100 gave
!                     rounding problem which upset a bufr encoding process)
!  1.4       21/04/99 change to size of mapping blocks following change in
!                     'border' (ppbget.F etc)
!  1.5       26/01/05 Add IASI (N.C.Atkinson)
!  1.6       15/09/08 Add parameter Imaxnpc (allows iasi1d.h and
!                     iasipc.h to specify smaller numbers if required)
!  1.7       16/08/10 Add ATMS and CRIS
!  1.8       23/11/17 crisnchan ->2211
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
      CHARACTER*3   SiteID        ! file creation site ID
!       - NESDIS maintain a list of 'recognised' site ids
! *********************************************
! *** Please edit this line as appropriate: ***
         PARAMETER (SiteID= AAPP_SITE_ID ) ! e.g. 'UKM' for UKMO
! *********************************************
!
!     dT may be tuned according to available memory:
      INTEGER       dT            ! max length of HIRS block stored in memory
      PARAMETER   ( dT = 500000 )!   (milliseconds)
!
      INTEGER       ATOVPPvn      ! ATOVPP version number
         PARAMETER (ATOVPPvn=1)   ! as at 15/06/98
!
      REAL          RMDI          ! general purpose real missing data indicator
         PARAMETER (RMDI=-9999.99)
      INTEGER       MDI           ! general purpose integer mdi
         PARAMETER (MDI=-999999)
!
! Instrument dependent parameters:
!
! HIRS
! ----
      INTEGER       Hnchan        ! number of channels (19 IR + 1 visible)
         PARAMETER (Hnchan=20)
      INTEGER       Hnfovs        ! number of fields-of-view in 1 scanline
         PARAMETER (Hnfovs=56)
      REAL          Hangle        ! viewing angle between fov's (degs)
         PARAMETER (Hangle=1.8)
      REAL          Hcycle        ! time between successive scans (millisec)
         PARAMETER (Hcycle=6400.)
      REAL          HmaxBT        ! maximum brightness temperature acceptable
         PARAMETER (HmaxBT=340.)  ! (Kelvin)
      REAL          HminBT        ! minimum brightness temperature acceptable
         PARAMETER (HminBT=150.)  ! (Kelvin)
!
! AMSU-A
! ------
      INTEGER       Anchan        ! number of channels
         PARAMETER (Anchan=15)
      INTEGER       Anfovs        ! number of fields-of-view in 1 scanline
         PARAMETER (Anfovs=30)
      REAL          Aangle        ! viewing angle between fov's (degs)
         PARAMETER (Aangle=3.333)
      REAL          Acycle        ! time between successive scans (millisec)
         PARAMETER (Acycle=8000.)
      REAL          AmaxBT        ! maximum brightness temperature acceptable
         PARAMETER (AmaxBT=340.)  ! (Kelvin)
      REAL          AminBT        ! minimum brightness temperature acceptable
         PARAMETER (AminBT=90.)   ! (Kelvin)
!
! AMSU-B
! ------
      INTEGER       Bnchan        ! number of channels
         PARAMETER (Bnchan=5)
      INTEGER       Bnfovs        ! number of fields-of-view in 1 scanline
         PARAMETER (Bnfovs=90)
      REAL          Bangle        ! viewing angle between fov's (degs)
         PARAMETER (Bangle=1.1)
      REAL          Bcycle        ! time between successive scans (millisec)
         PARAMETER (Bcycle=Acycle/3.)
      REAL          BmaxBT        ! maximum brightness temperature acceptable
         PARAMETER (BmaxBT=340.)  ! (Kelvin)
      REAL          BminBT        ! minimum brightness temperature acceptable
         PARAMETER (BminBT=90.)   ! (Kelvin)
!
! MSU
! ---
      INTEGER       Mnchan        ! number of channels
         PARAMETER (Mnchan=4)
      INTEGER       Mnfovs        ! number of fields-of-view in 1 scanline
         PARAMETER (Mnfovs=11)
      REAL          Mangle        ! viewing angle between fov's (degs)
         PARAMETER (Mangle=360./38.)
      REAL          Mcycle        ! time between successive scans (millisec)
         PARAMETER (Mcycle=25600.)
      REAL          MmaxBT        ! maximum brightness temperature acceptable
         PARAMETER (MmaxBT=340.)  ! (Kelvin)
      REAL          MminBT        ! minimum brightness temperature acceptable
         PARAMETER (MminBT=150.)  ! (Kelvin)
!
! IASI
! ----
      INTEGER       Inchan        ! number of channels
         PARAMETER (Inchan=8461)
      INTEGER       Indet         ! number of detectors
         PARAMETER (Indet=3)
      INTEGER       Infovs        ! number of fields-of-view in 1 scanline
         PARAMETER (Infovs=120)
      INTEGER       Inscanpos     ! number of scan positions in 1 scanline
         PARAMETER (Inscanpos=30)
      REAL          Iangle        ! viewing angle between fov's (degs)
         PARAMETER (Iangle=3.333)
      REAL          Icycle        ! time between successive scans (millisec)
         PARAMETER (Icycle=8000.)
      INTEGER       Imaxnpc       ! max number of PCs to handle
         PARAMETER (Imaxnpc=300)  !   - various applications, not just atovpp.
!
! ATMS
! ----
      INTEGER       atmsnchan        ! number of channels
         PARAMETER (atmsnchan=22)
      INTEGER       atmsnfovs        ! number of fields-of-view in 1 scanline
         PARAMETER (atmsnfovs=96)
      REAL          atmsangle        ! viewing angle between fov's (degs)
         PARAMETER (atmsangle=1.111)
      REAL          atmscycle        ! time between successive scans (millisec)
         PARAMETER (atmscycle=8000./3)
      REAL          atmsmaxBT        ! maximum brightness temperature acceptable
         PARAMETER (atmsmaxBT=340.)  ! (Kelvin)
      REAL          atmsminBT        ! minimum brightness temperature acceptable
         PARAMETER (atmsminBT=90.)   ! (Kelvin)
!
! CRIS
! ----
      INTEGER       crisnchan        ! number of channels
         PARAMETER (crisnchan=2211)
      INTEGER       crisndet         ! number of detectors
         PARAMETER (crisndet=3)
      INTEGER       crisnfovs        ! number of fields-of-view in 1 scanline
         PARAMETER (crisnfovs=270)
      INTEGER       crisnfor         ! number of fields-of-regard in 1 scanline
         PARAMETER (crisnfor=30)
      INTEGER       crisnscanpos     ! number of scan positions in 1 scanline
         PARAMETER (crisnscanpos=30)
      REAL          crisangle        ! viewing angle between scan positions (degs)
         PARAMETER (crisangle=3.333)
      REAL          criscycle        ! time between successive scans (millisec)
         PARAMETER (criscycle=8000.)
      INTEGER       crismaxnpc       ! max number of PCs to handle
         PARAMETER (crismaxnpc=300)  !   - various applications, not just atovpp.
!
! MWTS2
! ----
      INTEGER       mwtsnchan        ! number of channels
         PARAMETER (mwtsnchan=13)
      INTEGER       mwtsnfovs        ! number of fields-of-view in 1 scanline
         PARAMETER (mwtsnfovs=90)
      REAL          mwtsangle        ! viewing angle between fov's (degs)
         PARAMETER (mwtsangle=1.111)
      REAL          mwtscycle        ! time between successive scans (millisec)
         PARAMETER (mwtscycle=8000./3)
      REAL          mwtsmaxBT        ! maximum brightness temperature acceptable
         PARAMETER (mwtsmaxBT=340.)  ! (Kelvin)
      REAL          mwtsminBT        ! minimum brightness temperature acceptable
         PARAMETER (mwtsminBT=90.)   ! (Kelvin)
!
! HIRAS
! ----
      INTEGER       hirasnchan        ! number of channels
         PARAMETER (hirasnchan=2275)
      INTEGER       hirasndet         ! number of detectors
         PARAMETER (hirasndet=3)
      INTEGER       hirasnfovs        ! number of fields-of-view in 1 scanline
         PARAMETER (hirasnfovs=116)
      INTEGER       hirasnfor         ! number of fields-of-regard in 1 scanline
         PARAMETER (hirasnfor=29)
      INTEGER       hirasnscanpos     ! number of scan positions in 1 scanline
         PARAMETER (hirasnscanpos=29)
      REAL          hirasangle        ! viewing angle between scan positions (degs)
         PARAMETER (hirasangle=3.333)
      REAL          hirascycle        ! time between successive scans (millisec)
         PARAMETER (hirascycle=10000.)
      INTEGER       hirasmaxnpc       ! max number of PCs to handle
         PARAMETER (hirasmaxnpc=1)    ! (not used)
!
!
! Size of processing blocks
      INTEGER       Hnlines      ! max number HIRS scanlines in block
        PARAMETER ( Hnlines = dT/Hcycle + 5)
      INTEGER       Anlines      ! max number AMSU-A scanlines in block
        PARAMETER ( Anlines = dT/Acycle + 15)  ! from 5 990418
      INTEGER       Bnlines      ! max number AMSU-B scanlines in block
        PARAMETER ( Bnlines = dT/Bcycle + 25)  ! from 5 990418
      INTEGER       Mnlines      ! max number MSU scanlines in block
        PARAMETER ( Mnlines = dT/Mcycle + 20)
      INTEGER       Inlines      ! max number IASI scanlines in block
        PARAMETER ( Inlines = dT/Icycle + 15)  ! Same as AMSU-A
      INTEGER       atmsnlines   ! max number ATMS scanlines in block
        PARAMETER ( atmsnlines = dT/atmscycle + 25) !Same as AMSU-B
      INTEGER       crisnlines   ! max number CRIS scanlines in block
        PARAMETER ( crisnlines = dT/criscycle + 15) ! Same as AMSU-A
      INTEGER       hirasnlines   ! max number hiras scanlines in block
        PARAMETER ( hirasnlines = dT/hirascycle + 15) ! Same as AMSU-A

