C LIBS = amsuacl cal navnoaa f7nl1b satid tbus navtool f7ml f7tp f7cp f7gp auxaapp
      program amsuacl
!+
!NOM/NAME $Id: amsuacl.F 47 2006-08-21 08:44:41Z frna $
!
!----------------------------------------------------------------------------
!BUT/PURPOSE
!    This main program converts an AMSU-A 1A file to a 1B file by applying
!    the navigation and the calibration routines for AMSU-A. It also
!    performs quality control checks on the data and flags the scan lines
!    if necessary. If no calibration coeffs can be computed from the Earth
!    and Space view data, secondary calibration coefficients are still
!    written to the file. These coefficients are based on mean values around
!    an orbit. Logical flags have the following meanings:
!      lstat(iscan) -- if false do not navigate or calibrate this line
!      nvstat(iscan) -- if false scan line was not navigated successfully
!      prtstat(3,iscan) -- if false bad prts temps for this line (A1-1,A1-2,A2)
!      sampok(ichan,iscan) -- if false calib view counts for this channel
!                             bad for this scan line
!      calok(ichan,iscan) -- if false scan line not successfully calibrated
!
!DESCRIPTION
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions
!
!PROPRIETAIRE/OWNER
!     Nigel Atkinson
!     - based on AMSU-B code by Roger Saunders (AMSUBCL)
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES)
!(version   date      comment           <author>)
! 0.0                 AMSU-B          R. Saunders
! 1.0      June 96                    N. Atkinson
! 1.1      July 96   add navigation   M. Derrien
!
!FUNCTIONS AND SUBROUTINES CALLS
! amaclin , amasetu, ama_initcl, ama_antpos, ama_status, ama_smpmn,
! ama_iwttmp, ama_avg, ama_cal, ama_updt, amaclexit
! ama_loc, hd1bnav
!
!--------------------------------------------------------------------------
!--DECLARATIONS------------------------------------------------------------
      implicit none
!includes/commons
#include "amaclin.h"
#include "amascn.h"
#include "ama1b.h"
! local variables
      integer ama_cloptn(13),ichan
      integer nbnavcal    ! count of navigated/calibrated scans
      integer noanum,iscan,ispvw(3,mxlin),pllo(mxlin)
      real itmean(15,mxlin),spmean(15,mxlin),targ_temp(3,mxlin)
      real inst_temp(3,mxlin),itavg(15),spavg(15)
      logical*1 lstat(mxlin),prtstat(3,mxlin),calok(15,mxlin)
      logical*1 sampok(15,mxlin),nvstat(mxlin)
!
!--END OF DECLARATIONS--------------------------------------------------

      call amaclin(ama_cloptn)
      call amasetu(noanum,lstat,nvstat,calok)
      if(noanum.eq.0)go to 999
      if(ama_clflg.eq.1.or.ama_lflg.eq.1)then
!
!------- call AMSU-A navigation routines---------------------------------
!
!  n.b. for each successfully navigated line set nvstat array to .true.
!
         call ml_wt('info','navigation start')
!
!      earth locate 1B scans
         call ama_loc
         call ml_wt('info','ama_loc has been doen')
         do  iscan = 1, ama_h_scnlin
           nvstat(iscan)=.true.
         enddo
!
!      update navigation header variables
!
         call hd1bnav(ama_h_modelid,
     & ama_h_nadloctol,ama_h_locbit,ama_h_yawerr,
     & ama_h_rollerr,ama_h_pitcherr,ama_h_epoyr,ama_h_epody,
     & ama_h_epotime,ama_h_smaxis,ama_h_eccen,ama_h_incli,
     & ama_h_argper,ama_h_rascnod,ama_h_manom,ama_h_xpos,
     & ama_h_ypos,ama_h_zpos,ama_h_xvel,ama_h_yvel,ama_h_zvel,
     & ama_h_earthsun)
        call ml_wt('info','navigation end')
      endif
      if(ama_clflg.eq.1.or.ama_cflg.eq.1) then
!
!--------call AMSU-A calibration routines---------------------------------
!
! read calibration files
       call ama_initcl(noanum)      ! read in user options
       if (noanum.ne.0) then
       call ama_status(lstat,ispvw,sampok,pllo)    ! setup files and check ok
       call ama_antpos(nvstat,sampok,lstat)        ! check antenna pointing
       call ama_smpmn(lstat,ispvw,pllo,itmean,spmean,sampok)
       call ama_iwttmp(lstat,prtstat,targ_temp,inst_temp)
       do 200 ichan = 1 , 15 !------------start channel loop-------------
       do 100 iscan = 1 , ama_h_scnlin !-start scan line loop-----------
         call ama_avg(iscan,ichan,lstat,sampok,itmean,spmean,itavg,
     &    spavg,calok)
         call ama_cal(iscan,ichan,calok,targ_temp,inst_temp,
     &    itavg,spavg,ispvw,pllo)
 100   continue !---------------end of scan line loop-----------------
 200   continue !---------------end of channel loop-------------------
       endif
       endif
!
!-------Update 1B header and write out 1B records to disk------------
!
       call ama_updt(nvstat,calok,nbnavcal,ispvw)
!
! tidy up
!
 999   call amaclexit(nbnavcal,nvstat,lstat,calok,sampok,prtstat)
       stop
       end
