C LIBS = amsubcl cal navnoaa f7nl1b satid tbus navtool f7ml f7tp f7cp f7gp auxaapp
      program amsubcl
!+
!NOM/NAME $Id: amsubcl.F 47 2006-08-21 08:44:41Z frna $
!
!----------------------------------------------------------------------------
!BUT/PURPOSE 
!    This main program converts an AMSU-B 1A file to a 1B file by applying
!    the navigation and the calibration routines for AMSU-B. It also 
!    performs quality control checks on the data and flags the scan lines
!    if necessary. If no calibration coeffs can be computed from the Earth 
!    and Space view data, secondary calibration coefficients are still
!    written to the file. These coefficients are based on mean values around 
!    an orbit. Logical flags have the following meanings:
!      lstat(iscan) -- if false do not navigate or calibrate this line
!      nvstat(iscan) -- if false scan line was not navigated successfully
!      prtstat(iscan) -- if false bad prts temps for this line
!      sampok(ichan,iscan) -- if false calib view counts for this channel 
!                             bad for this scan line
!      calok(ichan,iscan) -- if false scan line not successfully calibrated
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!
!PROPRIETAIRE/OWNER  
!     Roger Saunders  
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES)
!(version   date      comment           <author>)
! 1.0      June 95       no             R. SAUNDERS
! 1.1      31 May  96     add navigation   M. Derrien
! 1.2      May 98      update hd1bnav arguments ( 1B modif related)
!                                     M. Derrien
! 1.3      Dec 98      Add bias correction input to amb_cloptn  N.Atkinson
! 1.4      July 2003   Add call to amb_moon                     N.Atkinson
!
!FUNCTIONS AND SUBROUTINES CALLS
! ambclin , ambsetu, amb_initcl, amb_antpos, amb_status, amb_smpmn,
! amb_iwttmp, amb_avg, amb_cal, amb_updt, ambclexit
! amb_loc, hd1bnav
!
!--------------------------------------------------------------------------
!--DECLARATIONS------------------------------------------------------------
      implicit none                                                      
!includes/commons
#include "ambclin.h"
#include "ambscn.h"
#include "amb1b.h"
! local variables
      integer amb_cloptn(12),ichan
      integer nbnavcal    ! count of navigated/calibrated scans
      integer noanum,iscan,ispvw(mxlin)
      real itmean(5,mxlin),spmean(5,mxlin),targ_temp(mxlin)
      real inst_temp(mxlin),itavg(5),spavg(5)
      logical*1 lstat(mxlin),prtstat(mxlin),calok(5,mxlin)
      logical*1 sampok(5,mxlin),nvstat(mxlin)
!
!--END OF DECLARATIONS--------------------------------------------------
      call ambclin(amb_cloptn)
      call ambsetu(noanum,lstat,nvstat,calok)
      if(noanum.eq.0)go to 999
!
!------- call AMSU-B navigation routines---------------------------------
!
      if(amb_clflg.eq.1.or.amb_lflg.eq.1)then
!
!      earth locate 1B scans
        call amb_loc
!
!      update navigation header variables
!
         call hd1bnav(amb_h_modelid,
     & amb_h_nadloctol,amb_h_locbit,amb_h_yawerr,
     & amb_h_rollerr,amb_h_pitcherr,amb_h_epoyr,amb_h_epody,
     & amb_h_epotime,amb_h_smaxis,amb_h_eccen,amb_h_incli,
     & amb_h_argper,amb_h_rascnod,amb_h_manom,amb_h_xpos,
     & amb_h_ypos,amb_h_zpos,amb_h_xvel,amb_h_yvel,amb_h_zvel,
     & amb_h_earthsun)
      endif
!       
!--------call AMSU-B calibration routines---------------------------------
!
      if(amb_clflg.eq.1.or.amb_cflg.eq.1)then
!
! read calibration files
        call amb_initcl(noanum)      ! read in user options

        if (noanum.ne.0) then
          call amb_status(lstat,ispvw,sampok)    ! setup files and check ok
          call amb_antpos(nvstat)               ! check antenna pointing

! Calculate angles between moon and space views for all scans
          call amb_moon

! Calculate mean calibration counts for all scans
          call amb_smpmn(lstat,ispvw,itmean,spmean,sampok) 

! Calculate mean target temperature for all scans
          call amb_iwttmp(lstat,prtstat,targ_temp,inst_temp)

! Loop over channels and scan lines
          do ichan = 1 , 5
            do iscan = 1 , amb_h_scnlin
              call amb_avg(iscan,ichan,sampok,itmean,spmean,itavg,
     &          spavg,calok)
              call amb_cal(iscan,ichan,prtstat,calok,targ_temp,
     &          inst_temp,itavg,spavg,ispvw)
            enddo   !Scan line loop
          enddo   !Channel loop
        endif
      endif
!  
!-------Update 1B header and write out 1B records to disk------------
!
      call amb_updt(nvstat,calok,nbnavcal,ispvw)
!
! tidy up
!
 999   call ambclexit(nbnavcal,nvstat,lstat,calok,sampok,prtstat)
       stop
       end
