C LIBS = avhrcl f7nl1b f7ml f7cp f7tp f7gp auxaapp 
!+
!NOM/NAME $Id: fy1cl.F 112 2008-02-01 13:37:40Z frna $
!
!BUT/PURPOSE 
!     Calibrate AVHRR-like channels of FY1 imager
!
!DESCRIPTION 
!     FORTRAN 77 with AAPP accepted extensions   
!  
!PROPRIETAIRE/OWNER  
! NWP SAF
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES)
!
!1.0 20/12/2006 NCA
!1.1 09/05/2007 Make band corrections consistent with standard AAPP
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2008, EUMETSAT, All Rights Reserved.
!

      implicit none                                                     

!commons
#include "avh1b.h"
#include "cavh_cpi.h"
#include "reclopt.h"    

!local variables
      integer ios,loc_rl,luavh,lupar,I,scn,chn
      character*80 txt

      data luavh/10/
      data lupar/11/
!-
!-------------------------------

!-------------------
! open avhrr 1B file (containing fy3 data)
!-------------------
      loc_rl=reclavh/rldiv
      open (10,form='unformatted',access='direct',
     &     recl=loc_rl,status='old',iostat=ios)

      if(ios.ne.0)
     &  open(luavh,form='unformatted',access='direct',
     &     recl=loc_rl,status='unknown',iostat=ios)

      if(ios.ne.0) then
        call ml_wti('info',' avhsetu iostat = ',ios)
        write(txt,*)'avhrr 1B File could not be opened ',ios,
     &              ' on unit',luavh
        call ml_wt('error',txt)
        call aappexit(2)
      endif

!-------------------
! read header
!-------------------

      call avhhdr(luavh,ios) 
      if(ios.ne.0) then
        call ml_wti('error','avhsetu reading 1B avhrr header',ios)
        call aappexit(2)
      endif

!----------------------------------------------
! read calibration parameters and update header
!----------------------------------------------

      call fy1_cpar(lupar)
      avh_h_l1bversnb = versnb
      avh_h_l1bversyr = versyr
      avh_h_l1bversdy = versdy
      do chn=1,2
        avh_h_albcnv(1,chn) = 10*vis_isi(chn)
        avh_h_albcnv(2,chn) = 1000*vis_eqwi(chn)
      enddo
      do chn=1,3
        avh_h_radtempcnv(1,chn) = nint(avh_bandcor(1,chn) * 1.e+03)
        avh_h_radtempcnv(2,chn) = 
     &     nint( (-avh_bandcor(2,chn)/avh_bandcor(3,chn)) * 1.e+05 )
        avh_h_radtempcnv(3,chn) = nint((1./avh_bandcor(3,chn)) * 1.e+06)
      enddo
      avh_h_radtempcnv(1,1) = nint(avh_bandcor(1,1) * 1.e+02)

!-------------------
! read data records
!-------------------

      do scn=1,avh_h_scnlin
        call avhdtr(luavh, scn, ios)
        if(ios.ne.0) then
          call ml_wti('error','avhsetu reading 1B data record',ios)
          call aappexit(2)
        endif

        call fy1_cal

        call avhdtw(luavh, scn, ios)
      enddo

!-------------------
! update header
!-------------------

      call avhhdw(luavh,ios)
      close(luavh)
      
      END 
