#!/bin/ksh
#
# Korn Shell to run the main program calcb1_algoV4
# Create the hirs_b1aslope.txt file useful to the 
# algorithm version 4 of the HIRS calibration
#
#
# PROPRIETAIRE/OWNER
#       Tiphaine Labrot
#
# HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
# (version   date      comment           <author>)
#  1.0    January 05   initial           Tiphaine Labrot
#  1.2    February 06  change config file P. Marguinaud

. "$AAPP_PREFIX/ATOVS_CONF"
. liblog.sh

# usage
#--------
usage(){
	echo 'usage : hcalcb1_algoV4 [-s satellite] [-y year] [-m month][-d day] [-h hour] [-n minutes] '
	echo '   -s	     : satellite (ex noaa17) '
	echo '   -y year     : year of the reference date (ex date of the orbit)'
	echo '   -m month    : month of the reference date (ex date of the orbit)'
	echo '   -d day      : day of the reference date (ex date of the orbit)'
	echo '   -h hour     : hour of the reference date (ex date of the orbit)'
	echo '   -n minutes  : minutes of the reference date (ex date of the orbit)'
	}

#  get the arguments
#-------------------
while getopts s:y:m:d:h:n: c
do
  case $c in
    s) SAT=$OPTARG;;
    y) YYYY=$OPTARG;;
    m) MM=$OPTARG;;
    d) DD=$OPTARG;;
    h) HH=$OPTARG;;
    n) MN=$OPTARG;;
    *)       usage
       exit 2;;
  esac
done

if [ -z "${SAT}" ]    ; then usage ; exit 1 ;fi
if [ -z "${YYYY}" ]   ; then usage ; exit 1 ;fi
if [ -z "${MM}" ]     ; then usage ; exit 1 ;fi
if [ -z "${DD}" ]     ; then usage ; exit 1 ;fi
if [ -z "${HH}" ]     ; then usage ; exit 1 ;fi
if [ -z "${MN}" ]     ; then usage ; exit 1 ;fi

log_info "Satellite = $SAT"
log_info "Year      = $YYYY"
log_info "Month     = $MM"
log_info "Day       = $DD"
log_info "Hour      = $HH"
log_info "Minut     = $MN"


# the name of the  historic file (=input file)
#---------------------------------------------
HIRS_HISTORIC_FILE=${PAR_CALIBRATION_MONITOR}/${SAT}/hirs_historic.txt

if [ ! -d ${PAR_CALIBRATION_MONITOR}/${SAT} ]; then
   mkdir ${PAR_CALIBRATION_MONITOR}/${SAT}
fi
if [ ! -f ${HIRS_HISTORIC_FILE} ]; then
  touch ${HIRS_HISTORIC_FILE}
fi

# test the option (see ATOVS_ENV) to take into account or not the hours,
#-----------------------------------------------------------------------
# minutes in the reference date to compute b1 and the average slope
#------------------------------------------------------------------

if [ ${HCALIB_B1ASLOP_FLAG} -eq 0 ]; then
  HH=0
  MN=0
  log_info "Hours/minutes are not taken into account to compute avg slope/b1"
fi

log_info "after test Hour    = $HH"
log_info "           Minut   = $MN"

# Links
#------ 
ln -s ${HIRS_HISTORIC_FILE} hirs_historic.txt

# Execute
#--------

rm -f hcalcb1.log
rm -f hirs_b1aslope.txt 

#hcalcb1_algoV4.exe << EOF >hcalcb1.log 2>&1
hcalcb1_algoV4.exe << EOF
${IPRINT},${YYYY},${MM},${DD},${HH},${MN},${HCALIB_B1ASLOP_NBHOUR}
EOF

exit=$?
if [ "${exit}" -ne 0 ]; then
   log_error hcalcb1_algoV4.exe error
   exit 1
fi

# Cut links
#----------

rm hirs_historic.txt

exit ${exit} 
