C LIBS = hirscl navnoaa tbus satid navtool f7nl1b f7ml f7tp f7cp f7gp auxaapp
      program hirscl
!+
!NOM/NAME $Id: hirscl.F 47 2006-08-21 08:44:41Z frna $
! hirscl
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!    This main program pilots the navigation, the localisation and the
!    calibration of the HIRS/2 or 3 or 4.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment                       <author>)
! 1.0      June 95       no                         T. LABROT
! 1.1      April 96  Navigation made before calibration to add "typorb" 
!                    (ascending/descending orbit) in the calib stat file.(TL)
! 1.2      May  98   Modified to conform to NOAA format l1b release (TL)
! 2.0      August00  Added an exit command when calib=0 (TL)
! 2.1      16 Feb 06 Use STOP instead of exit(0) - more portable. NCA
!         
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------
!--DECLARATIONS----------------------------------------------------------------

      implicit none
                                                      

!includes/commons
#include "hclin.h"
#include "hrsparam.h"
#include "hrs1bhd.h"
#include "hstatist.h"

! variables
      integer noanum
      integer splintab(hrs_mxcalib),iwtlintab(hrs_mxcalib) 
      integer quality(7,hrs_mxlin)

      double precision iwttmp(hrs_mxcalib)
      double precision iwtrad(hrs_mxirchn,hrs_mxcalib)
      double precision calibcoef0(hrs_mxchn,3,hrs_mxcalib)
      double precision calibcoef(hrs_mxchn,3,hrs_mxlin)
   
!--END OF DECLARATIONS----------------------------------------------------------

!-----------------------------
! init and open global files 
!-----------------------------

       call hclin
       call hclsetu

!----------------------
! Earth location part
!----------------------

       if(hrs_lflg.eq.1) then
         call ml_wt('info','hirs navigation start')

! Earth locate 1B scans
         call h_loc

! Update navigation header variables
         call hd1bnav(hrs_h_modelid,hrs_h_nadloctol,hrs_h_locbit,
     & hrs_h_yawerr,
     & hrs_h_rollerr,hrs_h_pitcherr,hrs_h_epoyr,hrs_h_epody,
     & hrs_h_epotime,hrs_h_smaxis,hrs_h_eccen,hrs_h_incli,
     & hrs_h_argper,hrs_h_rascnod,hrs_h_manom,hrs_h_xpos,
     & hrs_h_ypos,hrs_h_zpos,hrs_h_xvel,hrs_h_yvel,hrs_h_zvel,
     & hrs_h_earthsun)
         call ml_wt('info','navigation end')

       endif
       
! if no calibration, write and close the level 1b file
      if(hrs_cflg.eq.0) then       
       call hl1bwrt
       call hclexit
       call ml_wt('info','hirs navigation normal end')
       call aappexit(0)
      endif 
       
!----------------------
! calibration part
!----------------------

       if(hrs_cflg.eq.1) then
        
         call ml_wt('info','calibration start')

! init and open calibration specific files

         call h_cinit(noanum,quality)
         call h_instrtest(quality)

! check calibration lines and cycles

         call h_scanpos(noanum, splintab,iwtlintab)

! if good calibration cycles : work 

         if (calib.ne.0) then
           call h_cntmn(splintab,iwtlintab)
           call h_iwttmp(calib,iwtlintab,quality, iwttmp)
           call h_iwtrad(calib,iwtlintab,iwttmp, iwtrad)
           call h_interslop(calib,spcntmn,iwtcntmn,iwtrad,calibcoef0)
           call h_linlin(calib,splintab,calibcoef0,quality, calibcoef)
           call h_gtmean(calibcoef)
           call h_upcommon1(calibcoef,quality)
           if (hrs_stflg.eq.1) call h_stat
           call ml_wt('info','hirs calibration end')
! write level 1b file and exit           
           call hl1bwrt
           call hclexit
           call ml_wt('info','NORMAL END')
           STOP
           
         else 

! if no calibration cycle, only update the commons l1b

           call h_upcommon2(quality)
! write level 1b file and exit error          
           call hl1bwrt
           call hclexit
           call ml_wt('error','No calib cycle found')
           call ml_wt('error','No calibration possible')
           call aappexit(10)           
   
         endif

       endif

       end

