#! /bin/ksh
#$Id: hirscl.ksh,v 1.2 2005/04/15 13:39:50 frna Exp $       
#
# Korn Shell to calibrate and/or earth locate an hirs 1B file
#
#
# PROPRIETAIRE/OWNER
#       Marcel Derrien
#
# HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
# (version   date   comment                                           <author>)
#  1.0     11/12/95 initial                                            Marcel Derrien
#  1.1     FEB 97   suppression d'un parametre lu                      T.Labrot-M.M.Pinaud
#  1.2     01/04/04   ajout des du modele d'orbite Two Lines           Pascal Brunel
#  1.3     Feb 05   remove IPRINT=2 because initialised in ATOVS_ENV   T.Labrot 
#  1.4     19/01/05 use ${DIR_ERRCLK} rather than hard code tbus_db  N Atkinson
#  1.5     09/01/06   Create clock error file if it doesn't exist. NCA
#  1.6     01/02/06   change config file P. Marguinaud
#  1.7     07/06/06   add -p option to create satpos file. NCA + spm support PM
#
# COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
#

. "$AAPP_PREFIX/ATOVS_CONF"

 
# function to get attitude parameters
#------------------------------------

function det_att {
#recherche des valeurs d'attitude pour l'orbite $1 ou la precedente
#  dans le fichier $2 de type estatt (estimation d'attitude
#  on cherche le pattern  'ooooo  0' qui indique orbite ooooo OK
#  le numero d'orbite apparait en colonne 4 du fichier
#  et le code 0=OK en colonne 5
ORB0=`expr $1 - 0 `
ORBM1=`expr $1 - 1 `
ATT=`grep ${ORB0} $2 | awk '{if($4 == '${ORB0}' && $5 == 0) print $0;}' | tail -1l`
if [ -z "${ATT}" ] ; then
  ATT=`grep ${ORBM1} $2 | awk '{if($4 == '${ORBM1}' && $5 == 0) print $0;}' | tail -1l`
  if [ -z "${ATT}" ] ; then
    ATT='-99.99 -99.99 -99.99'
  fi
fi
}

# usage
#------- 
usage(){
	echo
	echo ' hirscl calibration and earth location of an hirs 1B file'
	echo
	echo 'usage : hirscl -c -l -p -s satimg -d yyyymmdd -h hhmm -n nnnnn l1b_file';
	echo
	echo ' -c calibration flag'
	echo ' -l earth location flag'
	echo ' -p to create satpos file if not already present'
	echo ' -s satimg	satellite '
	echo ' -d yyyymmdd	date'
	echo ' -h hhmm	hour-minute'
	echo ' -n nnnnn	orbite'
	echo ' l1b_file	1B file to process (calibate/locate or both)'
	echo
	echo 'environment variables:'
	echo '	DIR_SATID    path for the satellite identification file'
	echo '	SATID_FILE    filename for the satellite identification file'
        echo '  DIR_NAVIGATION  generic path for the tle dir tle_db'
        echo '                                 and the tbus dir tbus_db'
	}


rm -f hirscl.log


# get the arguments
#-------------------
	while getopts s:d:h:n:clp c
	do
               case $c in
               s)	SATIMG=$OPTARG;;
               d)	YYYYMMDD=$OPTARG;;
               h)	HHMM=$OPTARG;;
               n)	ORB=$OPTARG;;
               c)       CAL=1;;
               l)       LOC=1;;
               p)       POS=1;;
               \?)      usage
                        exit 2;;
               esac
	done
	shift `expr $OPTIND - 1`

	if [ "$#" -le 0 ] ; then
	  echo 'the name of 1b file is missing'
	  usage
	  exit 1
	fi

L1B_FILE=$1
if [ -z "${SATIMG}" ]   ; then usage ; exit 1 ;fi
if [ -z "${YYYYMMDD}" ] ; then usage ; exit 1 ;fi
if [ -z "${HHMM}" ]     ; then usage ; exit 1 ;fi
if [ -z "${ORB}" ]      ; then usage ; exit 1 ;fi
CAL=${CAL:=0}
LOC=${LOC:=0}
POS=${POS:=0}

# year month and day variables 
#------------------------------
YY=`echo ${YYYYMMDD} | awk '{print substr($0,3,2)}'`
MM=`echo ${YYYYMMDD} | awk '{print substr($0,5,2)}'`
DD=`echo ${YYYYMMDD} | awk '{print substr($0,7,2)}'`


# which type of bulletin for the requested satellite
# this determines which satpos program to run
#---------------------------------------------------
i=0
set -A ABUL ${PAR_NAVIGATION_DEFAULT_LISTEBUL}
for SAT in ${PAR_NAVIGATION_DEFAULT_LISTESAT}
do
if [ "${SAT}" = "${SATIMG}" ] ; then
  BUL=${ABUL[i]}
fi
i=`expr ${i} + 1`
done
BUL=${BUL=argos}
case ${BUL} in
	tle)  SATPOS_COM='satpostle';;
	tbus) SATPOS_COM='satpost';;
	spm)  SATPOS_COM='satposspm';;
	*) SATPOS_COM='satposa';;
esac


# satpos file is created if it does not exist
#--------------------------------------------
DIR_SATPOS=${DIR_NAVIGATION}/satpos
FILE_SATPOS=${DIR_SATPOS}/satpos_${SATIMG}_${YYYYMMDD}.txt
if [ ! -s ${FILE_SATPOS} ] && [ $POS = 1 ]; then
  ${SATPOS_COM} -c p -s ${SATIMG} -d ${DD}/${MM}/${YY} -n 1.2 > ${FILE_SATPOS}
  exit=$?
  if [ ! ${exit} -eq 0 ]; then
    echo "error for command ${SATPOS_COM}"
    rm -f ${FILE_SATPOS}
    exit ${exit} 
  elif [ ! -s ${FILE_SATPOS} ]; then
    echo 'satpos file does not exist'
    exit 1
  fi
fi

# getting previous or current orbit attitude values
#--------------------------------------------------
FILE_ESTATT=${DIR_NAVIGATION}/ana/estatt_${BUL}_${SATIMG}.txt
if [ -s ${FILE_ESTATT} ] && [ $LOC = 1 ]; then
  det_att ${ORB} ${FILE_ESTATT}
else
  ATT='-99.99 -99.99 -99.99'
fi 

# defining calibration and errorclock related files
#--------------------------------------------------
FILE_COEF=${PAR_CALIBRATION_COEF}/hirs/calcoef.dat
FILE_TEST_COEF=${PAR_CALIBRATION_COEF}/hirs/testcoef.dat
FILE_STAT=${PAR_CALIBRATION_MONITOR}/${SATIMG}/monhirs.txt
if [ ! -d ${PAR_CALIBRATION_MONITOR}/${SATIMG} ]; then
   mkdir ${PAR_CALIBRATION_MONITOR}/${SATIMG}
fi
case ${BUL} in
    tbus)  DIR_ERRCLK='tbus_db';;
    tle)   DIR_ERRCLK='tle_db';;
    spm)   DIR_ERRCLK='spm';;
    *) DIR_ERRCLK='tbus_db';;
esac
if [ ! -d ${DIR_NAVIGATION}/${DIR_ERRCLK} ]; then
  DIR_ERRCLK=orb_elem
fi
FILE_ERRCLK=${DIR_NAVIGATION}/${DIR_ERRCLK}/clkerr_${SATIMG}.txt
if [ ! -f ${FILE_ERRCLK} ]; then
  echo "creating clock error file $FILE_ERRCLK"
  touch $FILE_ERRCLK
fi

STAT=1
LULOG=10
LUHIRS=11
LUCOEF=12
LUTESTCOEF=13
LUSTAT=14
LUSATPOS=15
LUCLOCK=16

#?? on many of the lines below fort. has been replace by ${FORT}
rm -f ${FORT}*
ln -s ${FILE_COEF} ${FORT}${LUCOEF}
ln -s ${FILE_TEST_COEF} ${FORT}${LUTESTCOEF}
ln -s ${L1B_FILE} ${FORT}${LUHIRS}
ln -s ${FILE_SATPOS} ${FORT}${LUSATPOS}
ln -s ${FILE_ERRCLK} ${FORT}${LUCLOCK}

# executing hirs 1B calibration and earth location program
#----------------------------------------------------------
#hirscl.exe << EOF >hirscl.log 2>&1
hirscl.exe << EOF
${LOC},${CAL},${STAT},${LULOG},${LUSTAT},${LUHIRS},${LUSATPOS},${LUCLOCK},${LUCOEF},${LUTESTCOEF},${IPRINT}
${ATT}
EOF
exit=$?
if [ "${exit}" -eq 0 ]; then
  if [ "${CAL}" -eq 1 ]; then
    cat ${FORT}${LUSTAT} >> ${FILE_STAT}
  fi
fi

mv ${FORT}${LULOG} hirscl.log
rm -f ${FORT}*
exit ${exit} 
