C LIBS = mhscl cal navnoaa f7nl1b satid tbus navtool f7ml f7tp f7cp f7gp auxaapp
      program mhscl
!+
!NOM/NAME $Id: mhscl.F 47 2006-08-21 08:44:41Z frna $
!
!----------------------------------------------------------------------------
!BUT/PURPOSE 
!    This main program converts an MHS 1A file to a 1B file by applying
!    the navigation and the calibration routines for MHS. It also 
!    performs quality control checks on the data and flags the scan lines
!    if necessary. If no calibration coeffs can be computed from the Earth 
!    and Space view data, secondary calibration coefficients are still
!    written to the file. These coefficients are based on mean values around 
!    an orbit. Logical flags have the following meanings:
!      lstat(iscan) -- if false do not navigate or calibrate this line
!      nvstat(iscan) -- if false scan line was not navigated successfully
!      prtstat(iscan) -- if false bad prts temps for this line
!      sampok(ichan,iscan) -- if false calib view counts for this channel 
!                             bad for this scan line
!      calok(ichan,iscan) -- if false scan line not successfully calibrated
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!
!PROPRIETAIRE/OWNER  
!     Nigel Atkinson  
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES)
!(version   date      comment                         <author>)
! 1.0      July 2003  Created from amsubcl.F         N C Atkinson
!
!FUNCTIONS AND SUBROUTINES CALLS
! mhsclin , mhssetu, mhs_initcl, mhs_antpos, mhs_status, mhs_smpmn,
! mhs_iwttmp, mhs_avg, mhs_cal, mhs_updt, mhsclexit
! mhs_loc, hd1bnav
!
!--------------------------------------------------------------------------
!--DECLARATIONS------------------------------------------------------------
      implicit none                                                      
!includes/commons
#include "mhsclin.h"
#include "mhsscn.h"
#include "mhs1b.h"
! local variables
      integer mhs_cloptn(12),ichan
      integer nbnavcal    ! count of navigated/calibrated scans
      integer noanum,iscan,ispvw(mxlin)
      real itmean(5,mxlin),spmean(5,mxlin),targ_temp(mxlin)
      real inst_temp(mxlin),itavg(5),spavg(5)
      logical*1 lstat(mxlin),prtstat(mxlin),calok(5,mxlin)
      logical*1 sampok(5,mxlin),nvstat(mxlin)
!
!--END OF DECLARATIONS--------------------------------------------------

! Read in user options
      call mhsclin(mhs_cloptn)

! Read in all the data
      call mhssetu(noanum,lstat,nvstat,calok)
      if(noanum.eq.0)go to 999
!
!------- call MHS navigation routines---------------------------------
!
      if(mhs_clflg.eq.1.or.mhs_lflg.eq.1)then
!
! Earth locate 1B scans
        call mhs_loc

! Update navigation header variables
!
         call hd1bnav(mhs_h_modelid,
     & mhs_h_nadloctol,mhs_h_locbit,mhs_h_yawerr,
     & mhs_h_rollerr,mhs_h_pitcherr,mhs_h_epoyr,mhs_h_epody,
     & mhs_h_epotime,mhs_h_smaxis,mhs_h_eccen,mhs_h_incli,
     & mhs_h_argper,mhs_h_rascnod,mhs_h_manom,mhs_h_xpos,
     & mhs_h_ypos,mhs_h_zpos,mhs_h_xvel,mhs_h_yvel,mhs_h_zvel,
     & mhs_h_earthsun)
      endif
!       
!--------call MHS calibration routines---------------------------------
!
      if(mhs_clflg.eq.1.or.mhs_cflg.eq.1)then
!
! read calibration files
        call mhs_initcl(noanum)      ! read in user options

        if (noanum.ne.0) then
          call mhs_status(lstat,ispvw,sampok)    ! setup files and check ok
          call mhs_antpos(nvstat)               ! check antenna pointing

! Calculate angles between moon and space views for all scans
          call mhs_moon

! Calculate mean calibration counts for all scans
          call mhs_smpmn(lstat,ispvw,itmean,spmean,sampok) 

! Calculate mean target temperature for all scans
          call mhs_iwttmp(lstat,prtstat,targ_temp,inst_temp)

! Loop over channels and scan lines
          do ichan = 1 , 5
            do iscan = 1 , mhs_h_scnlin

! Average cal counts over scan lines
              call mhs_avg(iscan,ichan,sampok,itmean,spmean,itavg,
     &           spavg,calok)

! Compute calibration coefficients
              call mhs_cal(iscan,ichan,prtstat,calok,targ_temp,
     &           inst_temp,itavg,spavg,ispvw)

            enddo   !Scan line loop
          enddo   !Channel loop
        endif
      endif
!  
!-------Update 1B header and write out 1B records to disk------------
!
      call mhs_updt(nvstat,calok,nbnavcal,ispvw)
!
! tidy up
!
 999  call mhsclexit(nbnavcal,nvstat,lstat,calok,sampok,prtstat)
      stop
      end
