C LIBS = msucl navnoaa f7nl1b satid tbus navtool f7ml f7tp f7cp f7gp auxaapp
      program msucl
!+
!NOM/NAME $Id: msucl.F 47 2006-08-21 08:44:41Z frna $
! msucl
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!    This main program pilots the navigation, the localisation and the
!    calibration of the MSU.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Nov. 95      no              T. LABROT
! 1.1      April 96  Navigation made before calibration to add "typorb" 
!                    (ascending/descending orbit) in the calib stat file.(TL)
! 1.2     25/05/98  modified to conform to NOAA release        (TL)
!         
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------
!--DECLARATIONS----------------------------------------------------------------

      implicit none
                                                            
     
! includes/commons
#include "mclin.h"
#include "msuparam.h"
#include "msu1bhd.h"
      
! variables
      integer noanum   ! order satellite number
      integer quality1(7,msu_mxlin),quality2(6,msu_mxchn,msu_mxlin)
!                      ! arrays of quality information to update the l1b file
      
      double precision tg1(msu_mxlin),tg2(msu_mxlin)
!       tg1():temperature of the target 1 for each scan line 
!       tg2():temperature of the target 2 for each scan line

      double precision tgrad(msu_mxchn,msu_mxlin) 
!       tgrad(,):target's radiances for each channel and for each scan line

      double precision spcount(msu_mxchn,msu_mxlin)
!           For each scan line, radiant signal output counts (after applying 
!           the non-linearity coefficients) for each 
!           channel and when the radiometer is looking to space.

      double precision tgcount(msu_mxchn,msu_mxlin)
!           For each scan line, radiant signal output counts (after applying
!           the non-linearity coefficients) for each 
!           channel and when the radiometer is looking to target.

      double precision slope(msu_mxchn)
!           Slopes (calibration coef.1) for each channel

      double precision intcept(msu_mxchn)
!           Intercepts (calibration coef.2) for each channel

!--END OF DECLARATIONS----------------------------------------------------------

!-----------------------------
! init and open global files
!-----------------------------

       call mclin
       call mclsetu

!-----------------------       
! Earth location part
!----------------------- 

       if (msu_lflg.eq.1) then
        
         call ml_wt('info','MSU localisation start')
         
! Earth locate 1B scans
         call m_loc
       
! Update navigation header variables
         call hd1bnav(msu_h_modelid,msu_h_nadloctol,msu_h_locbit,
     & msu_h_yawerr,
     & msu_h_rollerr,msu_h_pitcherr,msu_h_epoyr,msu_h_epody,
     & msu_h_epotime,msu_h_smaxis,msu_h_eccen,msu_h_incli,
     & msu_h_argper,msu_h_rascnod,msu_h_manom,msu_h_xpos,
     & msu_h_ypos,msu_h_zpos,msu_h_xvel,msu_h_yvel,msu_h_zvel,
     & msu_h_earthsun)
         
         call ml_wt('info','MSU localisation end')
       
       endif        

!--------------------
! calibration part
!--------------------

       if (msu_cflg.eq.1) then
       
         call ml_wt('info','MSU calibration start')
   
! init and open calibration specific files       
        
         call m_cinit(noanum,quality1,quality2)
         call m_tgtmp(quality1,quality2,tg1,tg2)
         call m_tgrad(tg1,tg2,tgrad)
         call m_gfcounts(spcount,tgcount,quality2)

         call m_interslop(spcount,tgcount,tgrad,quality1, slope,intcept)
         call m_upcommon(slope,intcept,quality1,quality2)
         
         if (msu_stflg.eq.1) call m_finalstat
         
         call ml_wt('info','MSU calibration end')
         
       endif 

!-------------------------------      
! write level 1b file and exit
!-------------------------------

       call ml1bwrt
       call mclexit
       
       call ml_wt('info','NORMAL END')
       
       call aappexit(0)

       end
