#! /bin/ksh
#$Id: msucl.ksh,v 1.2 2005/04/15 13:39:53 frna Exp $
#
# Korn Shell to calibrate an earth locate an msu 1B file
#
#
# PROPRIETAIRE/OWNER
#       Tiphaine Labrot
#
# HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
# (version   date      comment                     <author>)
#  1.0      DEC 95                                 Tiphaine Labrot
#  1.1      FEB 97   suppresion d'un parametre lu  T. Labrot et M.-M. Pinaud
#  1.2      Mar 05   remove IPRINT=2 because initialised in ATOVS_ENV. NCA
#  1.3      Feb 06   change config file P. Marguinaud
#  1.4     07/06/06  add -p option to create satpos file. NCA
#
# COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
#

. "$AAPP_PREFIX/ATOVS_CONF"


# 
# function to get attitude parameters
#

function det_att {
#recherche des valeurs d'attitude pour l'orbite $1 ou la precedente
#  dans le fichier $2 de type estatt (estimation d'attitude
#  on cherche le pattern  'ooooo  0' qui indique orbite ooooo OK
#  le numero d'orbite apparait en colonne 4 du fichier
#  et le code 0=OK en colonne 5
ORB0=`expr $1 - 0 `
ORBM1=`expr $1 - 1 `
ATT=`grep ${ORB0} $2 | awk '{if($4 == '${ORB0}' && $5 == 0) print $0;}' | tail -1l`
if [ -z "${ATT}" ] ; then
  ATT=`grep ${ORBM1} $2 | awk '{if($4 == '${ORBM1}' && $5 == 0) print $0;}' | tail -1l`
  if [ -z "${ATT}" ] ; then
    ATT='-99.99 -99.99 -99.99'
  fi
fi
}

# 1 usage
#-------- 
usage(){
	echo
	echo ' msucl calibration and earth location of an msu 1B file'
	echo
	echo 'usage : msucl -c -l -p -s satimg -d yyyymmdd -h hhmm -n nnnnn l1b_file';
	echo
	echo ' -c calibration flag'
	echo ' -l earth location flag'
	echo ' -p to create satpos file if not already present'
	echo ' -s satimg	satellite '
	echo ' -d yyyymmdd	date'
	echo ' -h hhmm	hour-minute'
	echo ' -n nnnnn	orbite'
	echo ' l1b_file	1B file to process (calibate/locate or both)'
	echo
	echo 'environment variables:'
	echo '	DIR_SATID    path for the satellite identification file'
        echo '  DIR_NAVIGATION  generic path for the argos dir argos_db'
        echo '                                 and the tbus dir tbus_db'
	echo '	PATH_NAVIGATION  path for the navigation binaries'
	}


rm -f msucl.log


# 2 get the arguments
#--------------------
	while getopts s:d:h:n:clp c
	do
               case $c in
               s)	SATIMG=$OPTARG;;
               d)	YYYYMMDD=$OPTARG;;
               h)	HHMM=$OPTARG;;
               n)	ORB=$OPTARG;;
               c)       CAL=1;;
               l)       LOC=1;;
               p)       POS=1;;
               \?)      usage
                        exit 2;;
               esac
	done
	shift `expr $OPTIND - 1`

	if [ "$#" -le 0 ] ; then
	  echo 'the name of 1b file is missing'
	  usage
	  exit 1
	fi

L1B_FILE=$1
if [ -z "${SATIMG}" ]   ; then usage ; exit 1 ;fi
if [ -z "${YYYYMMDD}" ] ; then usage ; exit 1 ;fi
if [ -z "${HHMM}" ]     ; then usage ; exit 1 ;fi
if [ -z "${ORB}" ]      ; then usage ; exit 1 ;fi
CAL=${CAL:=0}
LOC=${LOC:=0}
POS=${POS:=0}

# 3 year month and day variables 
#-------------------------------
YY=`echo ${YYYYMMDD} | awk '{print substr($0,3,2)}'`
MM=`echo ${YYYYMMDD} | awk '{print substr($0,5,2)}'`
DD=`echo ${YYYYMMDD} | awk '{print substr($0,7,2)}'`



# which type of bulletin for the requested satellite
# this determines which satpos program to run
#---------------------------------------------------
i=0
set -A ABUL ${PAR_NAVIGATION_DEFAULT_LISTEBUL}
for SAT in ${PAR_NAVIGATION_DEFAULT_LISTESAT}
do
if [ "${SAT}" = "${SATIMG}" ] ; then
  BUL=${ABUL[i]}
fi
i=`expr ${i} + 1`
done
BUL=${BUL=argos}
case ${BUL} in
	argos) SATPOS_COM='satposa';;
	tbus) SATPOS_COM='satpost';;
	*) SATPOS_COM='satposa';;
esac


# 5 satpos file is created if it does not exists
#------------------------------------------------
DIR_SATPOS=${DIR_NAVIGATION}/satpos
FILE_SATPOS=${DIR_SATPOS}/satpos_${SATIMG}_${YYYYMMDD}.txt
if [ ! -s ${FILE_SATPOS} ] && [ $POS = 1 ]; then
  ${PATH_NAVIGATION}/${SATPOS_COM} -c p -s ${SATIMG} -d ${DD}/${MM}/${YY} -n 1.2 > ${FILE_SATPOS}
  exit=$?
  if [ ! ${exit} -eq 0 ]; then
    echo "error for command ${SATPOS_COM}"
    rm -f ${FILE_SATPOS}
    exit ${exit} 
  elif [ ! -s ${FILE_SATPOS} ]; then
    echo 'satpos file does not exist'
    exit 1
  fi
fi



# 6 getting previous or current orbit attitude values
#----------------------------------------------------
FILE_ESTATT=${DIR_NAVIGATION}/ana/estatt_${BUL}_${SATIMG}.txt
if [ -s ${FILE_ESTATT} ]; then
  det_att ${ORB} ${FILE_ESTATT}
else
  ATT='-99.99 -99.99 -99.99'
fi  

# 7 defining calibration and errorclock related files
#----------------------------------------------------
FILE_COEF=${PAR_CALIBRATION_COEF}/msu/calcoef.dat
FILE_TEST_COEF=${PAR_CALIBRATION_COEF}/msu/testcoef.dat
FILE_STAT=${PAR_CALIBRATION_MONITOR}/${SATIMG}/monmsu.txt
FILE_ERRCLK=${DIR_NAVIGATION}/tbus_db/clkerr_${SATIMG}.txt
if [ ! -f ${FILE_ERRCLK} ]; then
  FILE_ERRCLK=${DIR_NAVIGATION}/orb_elem/clkerr_${SATIMG}.txt
fi

STAT=1
LULOG=10
LUMSU=11
LUCOEF=12
LUTESTCOEF=13
LUSTAT=14
LUSATPOS=15
LUCLOCK=16

rm -f ${FORT}*
ln -s ${FILE_COEF} ${FORT}${LUCOEF}
ln -s ${FILE_TEST_COEF} ${FORT}${LUTESTCOEF}
ln -s ${L1B_FILE} ${FORT}${LUMSU}
ln -s ${FILE_SATPOS} ${FORT}${LUSATPOS}
ln -s ${FILE_ERRCLK} ${FORT}${LUCLOCK}

# 8 executing hirs 1B calibration and earth location program
#------------------------------------------------------------
msucl.exe << EOF >msucl.log 2>&1
${LOC},${CAL},${STAT},${LULOG},${LUSTAT},${LUMSU},${LUSATPOS},${LUCLOCK},${LUCOEF},${LUTESTCOEF},${IPRINT}
${ATT}
EOF
exit=$?
if [ "${exit}" -eq 0 ]; then
  if [ "${CAL}" -eq 1 ]; then
    cat ${FORT}${LUSTAT} >> ${FILE_STAT}
  fi
fi
rm -f ${FORT}*
exit ${exit} 
