      subroutine ama_antpos(nvstat,sampok,lstat)
!+
!NOM/NAME $Id: ama_antpos.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!    This module checks the antenna pointing of the AMSU-A Earth views.
!    if it is outside the threshold the navigation flag nvstat is set to
!    false and also the bit in the 1b scan line flag. Only set flags and 
!    print warning messages if in scan normal ('sampok' true and 'lstat' true).
!    This allows for the possibility of one instrument being off but good data
!    from the other.
!
!DESCRIPTION
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions
!
!PROPRIETAIRE/OWNER
!     Nigel Atkinson
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES)
!(version   date      comment           <author>)
! 0.0                 AMSU-B          R. Saunders
! 1.0      June 96                    N. Atkinson
! 1.1      May 98     Update 1B format  N Atkinson
! 1.2      March 03   f90  test       T.Labrot
!            I.OR.J not correct and modified by IOR(I,J)
! 1.3      Dec 03     Ignore instruments that are off. N Atkinson
!  
!ARGUMENTS
!      nvstat(mxlin) L*1 in/out  If .TRUE. then pointing ok
!      sampok(15,mxlin) L*1 in   Set by ama_status
!      lstat(mxlin)     L*1 in    "  "    "
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################

!--DECLARATIONS----------------------------------------------------------------
      implicit none
c-----------------------------------------
c
c
!
! includes/commons
#include "amaclin.h"
#include "ama1b.h"
#include "amascn.h"
#include "ama_cinit.h"
!
! local variables
      real op1,op2,op3,nom_pos
      integer*4 iscan,i
      logical*1 nvstat(mxlin),sampok(15,mxlin),lstat(mxlin)
!
!--END OF DECLARATIONS---------------------------------------------------------

      if (iprint.ge.1) write(ama_cllgu,*)
      if (iprint.ge.1) write(ama_cllgu,*) '***************************'
      if (iprint.ge.1) write(ama_cllgu,*) '**   SUBROUTINE AMA_ANTPOS*'
      if (iprint.ge.1) write(ama_cllgu,*) '***************************'

!..Loop in scan line and Earth View for AMSU-A1
      do 20 iscan = 1 , ama_h_scnlin
       if (sampok(3,iscan).and.lstat(iscan)) then     !Use chan 3 sampok flag
        do 10 i = 1 , mxpix
!n.b. bit0 is always zero, bit 1 is error bit, so divide by 4
!To conform to NESDIS product, the division is now done in decommutation (NCA)
         OP1 = ant_off_a11 + (a1_earth_data(1,i,iscan)  )*ant_coef_a11
         OP2 = ant_off_a12 + (a1_earth_data(2,i,iscan)  )*ant_coef_a12
         OP1 = AMOD(OP1+360+135, 360.0)-135.0   ! -135 to +225 degrees
         OP2 = AMOD(OP2+360+135, 360.0)-135.0   ! -135 to +225 degrees
         NOM_POS = 48.333 - FLOAT(I-1)*3.333333
         IF((ABS(NOM_POS-OP1).GT.pos_erre_a11).OR.
     &      (ABS(NOM_POS-OP2).GT.pos_erre_a12)) then
          nvstat(iscan) = .false.
          linqualflgs(iscan)=IOR(linqualflgs(iscan),8)  !Earth loc questionable
          if(iprint.ge.1) WRITE(ama_cllgu,200)iscan,i,NOM_POS-OP1,
     &      NOM_POS-OP2
 200      FORMAT(1X,' scan ',I4,', pix ',I4,
     &    ' A1: Nominal-measured earth ant posns=',F7.3,',',F7.3,
     &    ' deg')
          go to 20
         endif
 10     continue   ! .....end of pixel loop ..........
       else
! v1.3: Comment out next line to ignore instrument that is off or bad
!        nvstat(iscan) = .false.     !Not scan normal
       endif
 20   continue   ! ......end of scan line loop
!
!..Loop in scan line and Earth view for AMSU-A2
      do 40 iscan = 1 , ama_h_scnlin
       if (sampok(1,iscan).and.lstat(iscan)) then      !Use chan 1 sampok flag
        do 30 i = 1 , mxpix
!n.b. bit0 is always zero, bit 1 is error bit, so divide by 4
!To conform to NESDIS product, the division is now done in decommutation (NCA)
         OP3 = ant_off_a2 + (a2_earth_data(1,i,iscan)  )*ant_coef_a2
         OP3 = AMOD(OP3+360+135, 360.0)-135.0   ! -135 to +225 degrees
         NOM_POS = 48.333 - FLOAT(I-1)*3.333333
         IF(ABS(NOM_POS-OP3).GT.pos_erre_a2) then
          nvstat(iscan) = .false.
          linqualflgs(iscan)=IOR(linqualflgs(iscan),8)  !Earth loc questionable
          if(iprint.ge.1) WRITE(ama_cllgu,210)iscan,i,NOM_POS-OP3
 210  FORMAT(1X,' scan ',I4,', pix ',I4,
     &    ' A2: Nominal-measured earth ant posn=',F7.3,' deg')
          go to 40
         endif
 30     continue   ! .....end of pixel loop ..........
       else
! v1.3: Comment out next line to ignore instrument that is off or bad
!        nvstat(iscan) = .false.     !Not scan normal
       endif
 40   continue   ! ......end of scan line loop
!
      return
      end
