      subroutine ama_avg(iscan,ichan,lstat,sampok,itmean,spmean,itavg,
     & spavg,calok)
!+
!NOM/NAME $Id: ama_avg.F 46 2006-08-18 14:39:27Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!     To average over consecutive scan lines to give mean internal target
!     and space view counts for the AMSU-A channels. A triangular weighting 
!     function is applied as recommended in the Mo & Saunders specification. 
!     If there are bad scan lines within the averaging window then increased
!     weight is given to the good scan lines. If there are bad scan lines or 
!     no good scan lines then the 1b quality flags are set.
!
!DESCRIPTION
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions
!
!PROPRIETAIRE/OWNER
!     Nigel Atkinson
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES)
!(version   date      comment           <author>)
! 0.0                 AMSU-B          R. Saunders
! 1.0      Jan 96                     N. ATKINSON
! 1.1      March 03    f90 test       T.Labrot
!      I.OR.J not correct and modified by IOR(I,J)
!      I and J must have the same type ==> created i2word6 and i2word30x
!                                          local variables
! 1.2      17/01/2006 : Philippe Marguinaud : change hex constants from
!                       '1234'X to Z'1234'
! 1.3      17/08/2006 use i4word00000800 and i4word00004000. NCA
!                    
!
!ARGUMENTS
!      iscan i*4 in scan line number
!      ichan i*4 in Channel number (1-15)
!      lstat(mxlin) L*1 in  If false don't use scan line
!      sampok(15,mxlin) L*1 in cal view counts ok flag
!      itmean(15,mxlin) r*4 in mean target counts for each channel
!      spmean(15,mxlin) r*4 in mean space view counts for each channel
!      itavg(15) r*4 out line averaged target counts for each channel
!      spavg(15) r*4 out line averaged space view counts for each channel
!      calok(15,mxlin) out calib ok flag
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################
!
!
!--DECLARATIONS----------------------------------------------------------------
      implicit none
!includes/commons
#include "amaclin.h"
#include "ama1b.h"
#include "amascn.h"
#include "ama_cinit.h"
! local variables
      integer*2 i2word6, i2word30x
      parameter (i2word6=6)
      parameter (i2word30x=Z'30')
      integer iscan,ichan,is,iscan2
      real itmean(15,mxlin),spmean(15,mxlin),anorm,bnorm
      real itavg(15),spavg(15),wt
      logical*1 lstat(mxlin),calok(15,mxlin),sampok(15,mxlin)

      integer*4 i4word00000800, i4word00004000
      parameter (i4word00000800=Z'00000800')
      parameter (i4word00004000=Z'00004000')
!
!--END OF DECLARATIONS--------------------------------------------------------
!
! Compute weighted mean values from valid counts. If invalid counts are present
! (e.g. at start and end of run) the valid counts are all given a higher 
! weighting.
!
      anorm = FLOAT(nscan2+1)
      spavg(ichan) = 0.0       ! zeroise arrays
      itavg(ichan) = 0.0
      bnorm = 0.0
      do 600 is = -nscan2 , nscan2
        iscan2 = iscan+is
        if(iscan2.gt.0.and.iscan2.le.ama_h_scnlin)then
          if(sampok(ichan,iscan2).and.lstat(iscan2))then
            wt = (1.- float(abs(is))/anorm)/anorm
            spavg(ichan) = spavg(ichan) + wt*spmean(ichan,iscan2)
            itavg(ichan) = itavg(ichan) + wt*itmean(ichan,iscan2)
            bnorm=bnorm+wt
          endif
        endif
 600  continue
!
      if(bnorm.eq.0.)then        ! set no counts flags
        chqualflg(ichan,iscan)=IOR(chqualflg(ichan,iscan),i2word30x)
        calok(ichan,iscan) = .false.
        linqualflgs(iscan) = IOR(linqualflgs(iscan),i4word00000800)
      else
        itavg(ichan) = itavg(ichan)/bnorm
        spavg(ichan) = spavg(ichan)/bnorm
        calok(ichan,iscan) = .true.
        if(bnorm.lt.1.0)then   ! set some missing counts flags
          chqualflg(ichan,iscan)=IOR(chqualflg(ichan,iscan),i2word6)
          linqualflgs(iscan) = IOR(linqualflgs(iscan),i4word00004000)
        endif
      endif
      RETURN
      END
