      subroutine ama_initcl(noanum)
!+
!NOM/NAME $Id: ama_initcl.F 25 2006-02-03 15:43:53Z frna $
!
!---------------------------------------------------------------------------
!BUT/PURPOSE
!     This module opens and reads the calibration coefficients file.
!     Data are ranged in common ama_clcoef (include ama_cinit.h).
!     This module also reads the test coefficients and puts data in
!     common ama_tstcf (include ama_cinit.h).
!
!DESCRIPTION
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions
!
!PROPRIETAIRE/OWNER
!     Nigel Atkinson
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES)
!(version   date      comment           <author>)
! 0.0                 AMSU-B          R. Saunders
! 1.0      June 96                    N. Atkinson
! 1.1      Sept 96                    N. Atkinson
! 1.2      May 98    Update 1B definition  N Atkinson
! 1.3      March 03    f90 test       T.Labrot
!            I.AND.J not correct and modified by IAND(I,J)
!            I and J must have the same type ==> created i2w00ff and i2wff00
!                                          local variables
!            Changed  type XXX/yyy/
!                 to  type XXX
!                     data XXX/yyy/
! 1.4      Dec 03   Continue if one instr ID is not matched.  N. Atkinson
! 1.5      Mar 04   Modify for linux. N Atkinson / A Korpela
! 1.6      Nov 04   Remove obsolete calparid. NCA
! 1.7      17/01/2006 : Philippe Marguinaud : change hex constants from
!                       '1234'X to Z'1234'
!
!ARGUMENTS
!      integer  in  noanum    ! satellite number (12-14)
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################
!
!--DECLARATIONS-------------------------------------------------------------
      implicit none
! argument variables
      integer noanum
! local variables
      real tab0(4),tab1(4)        ! intermediate array
      integer i,j                 !index for loops
      integer versnb              !number of version of the coef file
      integer versyr              !year of the version of the coef file
      integer versday             !day of the version of the coef file (0/365)
      integer ios                 !variable that receives the error status
      integer*2 i2w00ff, i2wff00
      parameter(i2w00ff=Z'00FF')
      parameter(i2wff00=Z'FF00')
      character*80 charact        !buffer storing one line of the file
!      character*2 calparid        ! character variable of version number
!   added by M. Derrien May 98      
!  end of modification
      logical*1 a1found,a2found
      data a1found/.false./
      data a2found/.false./ 


! includes/commons
#include "ama_cinit.h"
#include "amaclin.h"
#include "amascn.h"
#include "ama1b.h"
!
!--END OF DECLARATIONS---------------------------------------------------------

      if (iprint.ge.1) write(ama_cllgu,*)
      if (iprint.ge.1) write(ama_cllgu,*) '***************************'
      if (iprint.ge.1) write(ama_cllgu,*) '**  SUBROUTINE AMA_INITCL**'
      if (iprint.ge.1) write(ama_cllgu,*) '***************************'
!
!Open and read calibration coefficients file
       open(ama_coefu,access='sequential',status='old',iostat=ios)

!      If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
       IF ( ios.ne.0 )
     &   open(ama_coefu,err=900,access='sequential',status='unknown')

!Skips the header lines
       read(ama_coefu,'(a80)',err=901) charact
       do while (charact(1:1) .eq. '#')
        read(ama_coefu,'(a80)',err=901) charact
       enddo
       backspace ama_coefu

       read(ama_coefu,*,err=901)versnb
       read(ama_coefu,*,err=901)versyr
       read(ama_coefu,*,err=901)versday
       ama_h_cpidsupdyr = versyr
       ama_h_cpidsupddy = versday
!       write(calparid,'(a2)')versnb
!       ama_h_calparid = calparid
!
       if (iprint.ge.1) then
        write(ama_cllgu,*)
        write(ama_cllgu,*) 'calibration coefficients file'
        write(ama_cllgu,520)versnb,versyr,versday
       endif

! Read fundamental constants
       read(ama_coefu,'(a80)',err=901) charact
       do while (charact(4:9) .ne. 'Speed ')
        read(ama_coefu,'(a15)',err=901) charact
       enddo
       read(ama_coefu,*,err=901) c
       if (iprint.ge.1) then
         write(ama_cllgu,*)'Speed of light m/s',c
       endif 
       read(ama_coefu,'(a80)',err=901) charact
       do while (charact(4:9) .ne. 'First ')
        read(ama_coefu,'(a15)',err=901) charact
       enddo
       read(ama_coefu,*,err=901) plck(1),plck(2)
       if (iprint.ge.1) then
         write(ama_cllgu,*)
     &'First & second radiation constants ',plck(1),plck(2)
       endif        
       do while (charact(4:9) .ne. 'Bright')
        read(ama_coefu,'(a15)',err=901) charact
       enddo
       read(ama_coefu,*,err=901) tspace
       if (iprint.ge.1) then
         write(ama_cllgu,*)
     &'Brightness temperature of space ',tspace
       endif     
! get center frquencies
       read(ama_coefu,'(a15)',err=901) charact
       read(ama_coefu,*,err=901) freq
       if (iprint.ge.1) then
         write(ama_cllgu,*)
     &'Central wavenumbers ',freq
       endif       
! skip 3 records
       do i = 1 , 3
        read(ama_coefu,*,err=901)
       enddo
! get band correction coeffs
       do i = 1 , 15
         read(ama_coefu,*,err=901)b(i),a(i)
         if (iprint.ge.1) then
           write(ama_cllgu,*)
     &   'Band Correction Coefficients (b,a) ',b(i),a(i)
        endif
       enddo
!
!       read(ama_coefu,*,err=901)          ! NB now a parameter
!       read(ama_coefu,*,err=901)nscan     ! no. of scan lines to avg over
       nscan2 = nscan/2
! nominal antenna positions for cal views. Same for A1 and A2.
       read(ama_coefu,*,err=901)
       do i = 1 , 4
         read(ama_coefu,*,err=901)(scanpos(j,i),j=1,2)  !Space and Internal
         if (iprint.ge.1) then
           write(ama_cllgu,*)
     &   'Nominal space & internal target viewing angles',
     &   (scanpos(j,i),j=1,2)
         endif
       enddo
       if (iprint.ge.1) write(ama_cllgu,*)'Cal Header read OK'
!
! Get the data for the correct instruments
!
  10   read(ama_coefu,'(a80)',err=901) charact
       do while (charact(4:5) .ne. 'ID')
        read(ama_coefu,'(a15)',err=901) charact
       enddo
       read(ama_coefu,*,err=901) modid
       if(iprint.ge.1) then
         write(ama_cllgu,*) 'instrument id. ',modid
       endif  
       if (modid.eq.99)then
         if (.not.a1found.and..not.a2found) then
           write(ama_cllgu,*) 'neither instrument id. matched:'
           write(ama_cllgu,*) 'no calibration possible '
           noanum=0
         else
           if (.not.a1found) then
             write(ama_cllgu,*) 'A1 id. not matched. Continuing'
           else
             write(ama_cllgu,*) 'A2 id. not matched. Continuing'
           endif
         endif
         return
       endif
       
! Least sig 2 bits are 10 for AMSU-A2, 01 for AMSU-A1, 00 for AMSU-B
       if (IAND(modid,2).eq.2) then
         modid=modid*256                !Use bits 8-15 if AMSU-A2
         i = IAND(ama_h_instid,i2wff00)
       else
         i = IAND(ama_h_instid,i2w00ff)
       endif
       if (modid.ne.i) go to 10
       if (modid.ge.256) then 
        a2found=.true.
       else
        a1found=.true.
       endif
       if (modid.lt.256) then
!
! AMSU-A1 parameters:
!
! Space viewing direction
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)spview_a1 
       if(iprint.ge.1) then
         write(ama_cllgu,*)
     &   '  Selected position of space view for calibration', spview_a1
       endif      
! Conversion to antenna position in degrees for both antennae
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)ant_coef_a11,ant_off_a11,
     &                          ant_coef_a12,ant_off_a12
       if(iprint.ge.1) then
         write(ama_cllgu,*)
     &   'Slope and offset for counts to antenna posn', 
     &   ant_coef_a11,ant_off_a11,ant_coef_a12,ant_off_a12
       endif     
! max error in antenna position in degrees for cal and earth views
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)pos_errc_a11,pos_erre_a11,
     &                          pos_errc_a12,pos_erre_a12
       if(iprint.ge.1) then
         write(ama_cllgu,*)
     &   'Antenna Pos error allowed in deg for cal and Earth views', 
     &   pos_errc_a11,pos_erre_a11,pos_errc_a12,pos_erre_a12
       endif      
! IWT PRT count to temperature conversion coefficients (5 PRTs,4coefs)
       read(ama_coefu,*,err=901)
       do i=1,5
        read(ama_coefu,*,err=901)tab0
        do j=1,4
          prtcnttmp_a11(j,i) = tab0(j)
        enddo
       enddo
       do i=1,5
        read(ama_coefu,*,err=901)tab0
        do j=1,4
          prtcnttmp_a12(j,i) = tab0(j)
        enddo
       enddo
! Weight coefficients for each PRT
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)prtwght_a11
       read(ama_coefu,*,err=901)prtwght_a12
! Reasonable PRT temp limits  (min,max)
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)prtmin_a11,prtmax_a11,
     &                          prtmin_a12,prtmax_a12
! Max PRT temp change allowed before rejecting
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)prtdt_a11,prtdt_a12
! Minimum number of PRT readings acceptable
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)nprtmin_a11,nprtmin_a12
! Number of lines to fill in PRT data
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)nprtgap_a11,nprtgap_a12
! Number of lines to use in consistency checks of cal views
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)ncalgap_a11,ncalgap_a12
! Temperature sensor ID, 0=RF shelf, 1=RF Mux
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)tsensid_a11,tsensid_a12
! 3 instrument RF shelf temps for A1-1 and A1-2 and A1-1 PLLO 2
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)(shelftemp(1,i),i=1,3)   !A1-1
       read(ama_coefu,*,err=901)(shelftemp(2,i),i=1,3)   !A1-2
       read(ama_coefu,*,err=901)(shelftemp(4,i),i=1,3)   !A1-1 PLLO 2
! 3 instrument RF mux temps for A1-1 and A1-2 and A1-1 PLLO 2
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)(muxtemp(1,i),i=1,3)   !A1-1
       read(ama_coefu,*,err=901)(muxtemp(2,i),i=1,3)   !A1-2
       read(ama_coefu,*,err=901)(muxtemp(4,i),i=1,3)   !A1-1 PLLO 2
! RF Shelf temp PRT count to temp conv coeffs
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)prtcnttmpi_a11
       read(ama_coefu,*,err=901)prtcnttmpi_a12
! RF Mux temp PRT count to temp conv coeffs
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)tab0  !Temporary storage
       read(ama_coefu,*,err=901)tab1
! Store the temperature of the required reference in 'reftemp', and the
! coefficients in 'prtcnttmpi'
       IF (tsensid_a11.eq.1) THEN
         DO i=1,3
           reftemp(1,i)=muxtemp(1,i)  !Use RF Mux for A1-1
           reftemp(4,i)=muxtemp(4,i)
         ENDDO
         DO i=1,4
           prtcnttmpi_a11(i)=tab0(i)
         ENDDO
       ELSE
         DO i=1,3
           reftemp(1,i)=shelftemp(1,i)  !Use RF Shelf for A1-1
           reftemp(4,i)=shelftemp(4,i)
         ENDDO
       ENDIF

       IF (tsensid_a12.eq.1) THEN
         DO i=1,3
           reftemp(2,i)=muxtemp(2,i)  !Use RF Mux for A1-2
         ENDDO
         DO i=1,4
           prtcnttmpi_a12(i)=tab1(i)
         ENDDO
       ELSE
         DO i=1,3
           reftemp(2,i)=shelftemp(2,i)  !Use RF Shelf for A1-2
         ENDDO
       ENDIF
! Internal target correction factors by channel for 3 instrument temperatures
       read(ama_coefu,*,err=901)
       do i = 1 ,3
         read(ama_coefu,*,err=901)(intcorr(j,i),j=3,15)
         read(ama_coefu,*,err=901)(intcorr(j,i),j=16,21)  !PLLO 2
       enddo
! Cold space correction factors  by channel for 4 space views
! N.B. Header record only allows for 1 correction factor for each channel
       read(ama_coefu,*,err=901)
       do i = 1 , 4
         read(ama_coefu,*,err=901)(spcorr(j,i),j=3,15)
         read(ama_coefu,*,err=901)(spcorr(j,i),j=16,21)  !PLLO 2
       enddo
! Reasonable count limits by channel (min,max) for internal target
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)(intcntmin(j),j=3,15)
       read(ama_coefu,*,err=901)(intcntmin(j),j=16,21)  !PLLO 2
       read(ama_coefu,*,err=901)(intcntmax(j),j=3,15)
       read(ama_coefu,*,err=901)(intcntmax(j),j=16,21)  !PLLO 2
! Reasonable count limits by channel (min,max) for space view
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)(spcntmin(j),j=3,15)
       read(ama_coefu,*,err=901)(spcntmin(j),j=16,21)  !PLLO 2
       read(ama_coefu,*,err=901)(spcntmax(j),j=3,15)
       read(ama_coefu,*,err=901)(spcntmax(j),j=16,21)  !PLLO 2
! max change in mean counts from previous scan allowed before rejecting
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)(dcount(j),j=3,15)
       read(ama_coefu,*,err=901)(dcount(j),j=16,21)   !PLLO 2
! non linearity correction terms by channel at 3 instrument temperatures
       read(ama_coefu,*,err=901)
       do i = 1, 3
         read(ama_coefu,*,err=901)(nonlin(j,i),j=3,15)
         read(ama_coefu,*,err=901)(nonlin(j,i),j=16,21)  !PLLO 2
       enddo
! Digital A coeffs
       read(ama_coefu,*,err=901)
       do i=1,45
         read(ama_coefu,*,err=901)(digacoef_a1(j,i),j=1,4)
       enddo
! Analogue coeffs
       read(ama_coefu,*,err=901)
       do i=1,27
         read(ama_coefu,*,err=901)(analcoef_a1(j,i),j=1,2)
       enddo
       if (iprint.ge.1) 
     &   write(ama_cllgu,*)'Cal Coefs read for instrument',modid
       else
!
! AMSU-A2 parameters:
!
! space viewing direction
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)spview_a2    
! conversion to antenna position
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)ant_coef_a2,ant_off_a2  
! max error in antenna position
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)pos_errc_a2,pos_erre_a2 
! IWT PRT count to temperature conversion coefficients (7 PRTs,4coefs)
       read(ama_coefu,*,err=901)
       do i=1,7
        read(ama_coefu,*,err=901)tab0
        do j=1,4
          prtcnttmp_a2(j,i) = tab0(j)
        enddo
       enddo
! Weight coefficients for each PRT
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)prtwght_a2
! Reasonable PRT temp limits  (min,max)
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)prtmin_a2,prtmax_a2
! Max PRT temp change allowed before rejecting
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)prtdt_a2
! Minimum number of PRT readings acceptable
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)nprtmin_a2
! Number of lines to fill in PRT data
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)nprtgap_a2
! Number of lines to use in consistency checks of cal views
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)ncalgap_a2
! Temperature sensor ID, 0=RF shelf, 1=RF Mux
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)tsensid_a2
! 3 instrument RF shelf temps
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)(shelftemp(3,i),i=1,3)   !A2
! 3 instrument RF mux temps
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)(muxtemp(3,i),i=1,3)   !A2
! RF shelf temp PRT count to temp conv coeffs
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)prtcnttmpi_a2
! RF mux temp PRT count to temp conv coeffs
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)tab0
! Store the temperature of the required reference in 'reftemp', and the 
! coefficients in prtcnttmpi_a2
       IF (tsensid_a2.eq.1) THEN
         DO i=1,3
           reftemp(3,i)=muxtemp(3,i)  !Use RF Mux for A2
         ENDDO
         DO i=1,4
           prtcnttmpi_a2(i)=tab0(i)
         ENDDO
       ELSE
         DO i=1,3
           reftemp(3,i)=shelftemp(3,i)  !Use RF Shelf for A2
         ENDDO
       ENDIF
! Internal target correction factors by channel for 3 instrument temperatures
       read(ama_coefu,*,err=901)
       do i = 1 ,3
         read(ama_coefu,*,err=901)(intcorr(j,i),j=1,2)
       enddo
! Cold space correction factors by channel for 4 space views
       read(ama_coefu,*,err=901)
       do i = 1 , 4
         read(ama_coefu,*,err=901)(spcorr(j,i),j=1,2)
       enddo
! Reasonable count limits by channel (min,max) for internal target
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)(intcntmin(j),j=1,2)
       read(ama_coefu,*,err=901)(intcntmax(j),j=1,2)
! Reasonable count limits by channel (min,max) for space view
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)(spcntmin(j),j=1,2)
       read(ama_coefu,*,err=901)(spcntmax(j),j=1,2)
! max change in mean counts from previous scan allowed before rejecting
       read(ama_coefu,*,err=901)
       read(ama_coefu,*,err=901)(dcount(j),j=1,2)
! non linearity correction terms by channel at 3 instrument temperatures
       read(ama_coefu,*,err=901)
       do i = 1, 3
         read(ama_coefu,*,err=901)(nonlin(j,i),j=1,2)
       enddo
! Digital A coeffs
       read(ama_coefu,*,err=901)
       do i=1,19
         read(ama_coefu,*,err=901)(digacoef_a2(j,i),j=1,4)
       enddo
! Analogue coeffs
       read(ama_coefu,*,err=901)
       do i=1,15
         read(ama_coefu,*,err=901)(analcoef_a2(j,i),j=1,2)
       enddo
       if (iprint.ge.1) 
     &   write(ama_cllgu,*)'Cal Coefs read for instrument',modid/256

       endif
       if (.not.(a1found.and.a2found)) goto 10
!
! Cal parameters found for both instruments
!
       close(ama_coefu,err=902)
       if(iprint.ge.1)write(ama_cllgu,*)
     &'Calibration parameters read ok'
!
! Open and read secondary calibration coefficients file
!
       open(ama_tstcoefu,access='sequential',status='old',iostat=ios)

!      If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
       IF ( ios.ne.0 )
     &  open(ama_tstcoefu,err=900,access='sequential',status='unknown')

       a1found=.false.     !Re-initialize flags
       a2found=.false.

! Skip the header lines
       read(ama_tstcoefu,'(a80)',err=901) charact
       do while (charact(1:1) .eq. '#')
        read(ama_tstcoefu,'(a80)',err=901) charact
       enddo
       backspace ama_tstcoefu

       read(ama_tstcoefu,*,err=901)versnb
       read(ama_tstcoefu,*,err=901)versyr
       read(ama_tstcoefu,*,err=901)versday

       if (iprint.ge.1) then
        write(ama_cllgu,*) 'test coefficients file'
        write(ama_cllgu,520)versnb,versyr,versday
       endif

110    read(ama_tstcoefu,'(a15)',err=901) charact
       do while (charact(4:5) .ne. 'ID')
         read(ama_tstcoefu,'(a15)',err=901) charact
       enddo
       read(ama_tstcoefu,*,err=901) modid
       if (modid.eq.99)then
         write(ama_cllgu,*)' instrument id. not matched no secondary'
         write(ama_cllgu,*)' calibration coeffs available '
         return
       endif

! Least sig 2 bits are 10 for AMSU-A2, 01 for AMSU-A1, 00 for AMSU-B
       if (IAND(modid,2).eq.2) then
         modid=modid*256               !Use bits 8-15 if AMSU-A2
         i = IAND(ama_h_instid,i2wff00)
       else
         i = IAND(ama_h_instid,i2w00ff)
       endif
       if (modid.ne.i) go to 110
       if (modid.ge.256) then 
        a2found=.true.
       else
        a1found=.true.
       endif
       if (modid.lt.256) then                ! AMSU-A1
         read(ama_tstcoefu,*,err=901)
         do i = 1 , 3
           read(ama_tstcoefu,'(a80)',err=901)
           read(ama_tstcoefu,*,err=901)(u0(j,i),j=3,15)
           read(ama_tstcoefu,*,err=901)(u1(j,i),j=3,15)
           read(ama_tstcoefu,*,err=901)(u2(j,i),j=3,15)
         enddo
       else                                  ! AMSU-A2
         read(ama_tstcoefu,*,err=901)
         do i = 1 , 3
           read(ama_tstcoefu,'(a80)',err=901)
           read(ama_tstcoefu,*,err=901)(u0(j,i),j=1,2)
           read(ama_tstcoefu,*,err=901)(u1(j,i),j=1,2)
           read(ama_tstcoefu,*,err=901)(u2(j,i),j=1,2)
         enddo
       endif
!
       close(ama_tstcoefu,err=902)
       if(iprint.ge.1)write(ama_cllgu,*)' Secondary cal coeffs read ok'
!
! scale secondary coeffs so they can be stored in 1B
!
      do 120 i = 1 , 3
      do 120 j = 1 , 15
        u0(j,i) = u0(j,i)*1E9
        u1(j,i) = u1(j,i)*1E13
        u2(j,i) = u2(j,i)*1E19
 120  continue
      go to 200
!
!
 520  format(2X,' version number: ',I3/'   version year: ',I4/
     &       '   version day of year: ',I3)
 540  format(3(3X,I3),3(6X,I3))

 900  write(ama_cllgu,*)'Error opening cal coefficients file'
      stop
 901  write(ama_cllgu,*)'Error reading cal coefficients file'
      stop
 902  write(ama_cllgu,*)'Error closing cal coefficients file'
      stop
!
 200  return
      end
