      subroutine ama_iwttmp(lstat,prtstat,targ_temp,inst_temp)
!+
!NOM/NAME $Id: ama_iwttmp.F 361 2013-06-10 09:37:26Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!     This module extracts the AMSU-A target and instrument PRT data
!     necessary for the calibration and performs quality control checks
!     and sets flags for bad PRT data. The target temperature data
!     extracted from good lines is the averaged internal target PRT
!     temperature (there are 5 PRTs in the AMSU-A1 targets and 7 PRTs in the
!     AMSU-A2 target). The instrument temperature for AMSU-A is the RF Shelf
!     temperature for each scan line. If there is no good PRT data prtstat
!     is set to false for that scan line.
!
!DESCRIPTION
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions
!
!PROPRIETAIRE/OWNER
!     Nigel Atkinson
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES)
!(version   date      comment                                <author>)
! 0.0                 AMSU-B                                 R. Saunders
! 1.0     June 96                                            N. Atkinson
! 1.1     16 July 96  Allow for non-integer PRT weights      N. Atkinson
! 1.2     May 98      Update 1B format                       N. Atkinson
! 1.3     July 98     correct t2 for a2_temp                 M. Derrien
!                     according to Nigel's mail on 17 Jul 1998
! 1.4     March 03    f90 test                               T.Labrot
!          I.AND.J not correct and modified by IAND(I,J)
!            I.OR.J not correct and modified by IOR(I,J)
!            I and J must have the same type
!            ==> created local variables : i2word1,i2word8
!          if (prtstat(j,iscan).eq..true.) then   
!                changed to   if (prtstat(j,iscan)) then
! 1.5      17/01/2006 : Philippe Marguinaud : change hex constants from
!                       '1234'X to Z'1234'
! 1.6      17/08/2006 use i4word00002000,i4word00000800,i4word00001000. NCA
! 1.7      06/06/2013 Bug fix: change tsensid_a11 to tsensid_a12
!
!ARGUMENTS
!      lstat(mxlin)   L*1 in  If false don't use scan line
!      prtstat(3,mxlin) L*1 out  If .TRUE. then PRT data ok
!      targ_temp(3,mxlin) r*4 out avg internal target temp in K
!      inst_temp(3,mxlin) r*4 out instrument temperatures A11,A12,A2 in K
!
!SUBROUTINE CALLS
!      prtchk
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################

!--DECLARATIONS---------------------------------------------------------------
      implicit none
c-----------------------------------------
c
c
!
! includes/commons
#include "amaclin.h"
#include "ama1b.h"
#include "amascn.h"
#include "ama_cinit.h"
!
! local variables
      real*4 inst_temp(3,mxlin),targ_temp(3,mxlin)
      real*4 prtavg_a11,prtavg_a12,prtavg_a2
      real*4 prt_a11(5),prt_a12(5),prt_a2(7)
      integer*4 ngoodweight_a11,ngoodweight_a12,ngoodweight_a2
      integer*4 iscan,iold,iprt,nprt_a11,nprt_a12,nprt_a2,cindex(15),i,j
      integer*4 nprtgap(3),instmin(3),instmax(3)
      real*4 t11,t12,t2,d(3),difft,diffi
      logical*1 lstat(mxlin),prtstat(3,mxlin),validfound
      
      integer*2 i2word1,i2word8
      parameter (i2word1=1)
      parameter (i2word8=8)

      integer*4 i4word00002000, i4word00000800, i4word00001000
      parameter (i4word00002000=Z'00002000')
      parameter (i4word00000800=Z'00000800')
      parameter (i4word00001000=Z'00001000')

      data cindex/6,7,9,10,11,12,13,14,15,3,4,5,8,1,2/  !A1-1,A1-2,A2
      data instmin/1,10,14/, instmax/9,13,15/
!
!--END OF DECLARATIONS--------------------------------------------------------

      if (iprint.ge.1) write(ama_cllgu,*)
      if (iprint.ge.1) write(ama_cllgu,*) '***************************'
      if (iprint.ge.1) write(ama_cllgu,*) '**   SUBROUTINE AMA_IWTTMP*'
      if (iprint.ge.1) write(ama_cllgu,*) '***************************'
!..Loop in scan line................................................
      do 100 iscan = 1 , ama_h_scnlin

        if(lstat(iscan))then
!
! ...get instrument temperatures....
!Note: division by 2 is now carried out in decommutation (NCA, May 98)
          IF (tsensid_a11.eq.0) THEN
            t11 = float(a1_temps(33,iscan)  )   !RF Shelf A1-1
          ELSE
            t11 = float(a1_temps(5,iscan)  )   !RF Mux A1-1
          ENDIF
          IF (tsensid_a12.eq.0) THEN
            t12 = float(a1_temps(34,iscan)  )   !RF Shelf A1-2
          ELSE
            t12 = float(a1_temps(6,iscan)  )   !RF Mux A1-2
          ENDIF
          IF (tsensid_a2.eq.0) THEN
            t2 = float(a2_temps(11,iscan)  )    !RF Shelf A2
          ELSE
            t2 = float(a2_temps(3,iscan)  )    !RF Mux A2
          ENDIF

          inst_temp(1,iscan) = prtcnttmpi_a11(1) 
     &      + prtcnttmpi_a11(2)*t11 + prtcnttmpi_a11(3)*t11*t11 
     &      + prtcnttmpi_a11(4)*t11*t11*t11
          inst_temp(2,iscan) = prtcnttmpi_a12(1) 
     &      + prtcnttmpi_a12(2)*t12 + prtcnttmpi_a12(3)*t12*t12 
     &      + prtcnttmpi_a12(4)*t12*t12*t12
          inst_temp(3,iscan) = prtcnttmpi_a2(1) 
     &      + prtcnttmpi_a2(2)*t2 + prtcnttmpi_a2(3)*t2*t2 
     &      + prtcnttmpi_a2(4)*t2*t2*t2
!
! ...get target temperatures....
          do 50 iprt = 1 , 7      !AMSU-A1 has 5 prts, A2 has 7          
           if (iprt.le.5) then
             t11 = float(a1_temps(iprt+35,iscan)  )
             t12 = float(a1_temps(iprt+40,iscan)  )
             prt_a11(iprt) = prtcnttmp_a11(1,iprt) 
     &         + prtcnttmp_a11(2,iprt)*t11 
     &         + prtcnttmp_a11(3,iprt)*t11*t11 
     &         + prtcnttmp_a11(4,iprt)*t11*t11*t11
             prt_a12(iprt) = prtcnttmp_a12(1,iprt) 
     &         + prtcnttmp_a12(2,iprt)*t12 
     &         + prtcnttmp_a12(3,iprt)*t12*t12 
     &         + prtcnttmp_a12(4,iprt)*t12*t12*t12
           endif
           t2 = float(a2_temps(iprt+12,iscan)  )
           prt_a2(iprt) = prtcnttmp_a2(1,iprt) 
     &       + prtcnttmp_a2(2,iprt)*t2 + prtcnttmp_a2(3,iprt)*t2*t2
     &       + prtcnttmp_a2(4,iprt)*t2*t2*t2
  50      continue
!
! ...carry out a check of PRT consistency on scan line and compute mean
!
          call prtchk(5,prt_a11,prtwght_a11,prtmax_a11,prtmin_a11,
     &      prtdt_a11,prtavg_a11,nprt_a11,ngoodweight_a11)
          call prtchk(5,prt_a12,prtwght_a12,prtmax_a12,prtmin_a12,
     &      prtdt_a12,prtavg_a12,nprt_a12,ngoodweight_a12)
          call prtchk(7,prt_a2,prtwght_a2,prtmax_a2,prtmin_a2,
     &      prtdt_a2,prtavg_a2,nprt_a2,ngoodweight_a2)
!
! ...store avg prt temp for target
! ...A11
!          write(6,*)'prt_a11=',(prt_a11(j),j=1,5)
!          write(6,*)'Raw=',(a1_temps(35+j,iscan)  ,j=1,5)
          if ( nprt_a11 .ge. nprtmin_a11)then
            targ_temp(1,iscan) = prtavg_a11
            prtstat(1,iscan) = .true.
            if (nprt_a11.lt.ngoodweight_a11) then
              do j=1,9
                chqualflg(cindex(j),iscan) = 
     &                 IOR(chqualflg(cindex(j),iscan),i2word1)
              enddo               !Some bad PRTs on this line
            endif
          else
            if (iprint.ge.1)
     &       write(ama_cllgu,*)'scan ',iscan,' nprt_a11=',nprt_a11
            targ_temp(1,iscan) = 0.0
            prtstat(1,iscan) = .false.
          endif
! ...A12
          if ( nprt_a12 .ge. nprtmin_a12)then
            targ_temp(2,iscan) = prtavg_a12
            prtstat(2,iscan) = .true.
            if (nprt_a12.lt.ngoodweight_a12) then 
              do j=10,13
                chqualflg(cindex(j),iscan) = 
     &                 IOR(chqualflg(cindex(j),iscan),i2word1)
              enddo               !Some bad PRTs on this line
            endif
          else
            if (iprint.ge.1)
     &       write(ama_cllgu,*)'scan ',iscan,' nprt_a12=',nprt_a12
            targ_temp(2,iscan) = 0.0
            prtstat(2,iscan) = .false.
          endif
! ...A2
          if ( nprt_a2 .ge. nprtmin_a2)then
            targ_temp(3,iscan) = prtavg_a2
            prtstat(3,iscan) = .true.
            if (nprt_a2.lt.ngoodweight_a2) then 
              do j=14,15
                chqualflg(cindex(j),iscan) = 
     &                 IOR(chqualflg(cindex(j),iscan),i2word1)
              enddo               !Some bad PRTs on this line
            endif
          else
            if (iprint.ge.1)
     &       write(ama_cllgu,*)'scan ',iscan,' nprt_a2=',nprt_a2
            targ_temp(3,iscan) = 0.0
            prtstat(3,iscan) = .false.
          endif
        else                         ! if lstat(iscan) false
          targ_temp(1,iscan) = 0.0
          inst_temp(1,iscan) = 0.0
          prtstat(1,iscan) = .false.
          targ_temp(2,iscan) = 0.0
          inst_temp(2,iscan) = 0.0
          prtstat(2,iscan) = .false.
          targ_temp(3,iscan) = 0.0
          inst_temp(3,iscan) = 0.0
          prtstat(3,iscan) = .false.
        endif
!
 100  Continue   ! ......end of scan line loop
!
! Now rescan through PRT temps to do interline test and fill in gaps
! by allowing up to nprtgap scans to have the last good PRT value or the next
! good PRT value. The difference between successive 'good' values must be 
! less than the 'prtdt' value from the calibration data file.
! Logic is similar to the interline test on Space/Internal target counts, 
! except that here we correct any bad values.
!
      d(1)=prtdt_a11
      d(2)=prtdt_a12
      d(3)=prtdt_a2
      nprtgap(1)=nprtgap_a11
      nprtgap(2)=nprtgap_a12
      nprtgap(3)=nprtgap_a2
      do 410 j=1,3       !Loop over the 3 instruments
      validfound=.false.
      iold = -999
      do 400 iscan = 1 , ama_h_scnlin
        if (.not.validfound) then   !Start of sequence - no good data yet
          if (prtstat(j,iscan)) then
            if ((iscan-iold).gt.nprtgap(j)) then
              if (iold.gt.0) then
                prtstat(j,iold)=.false.        !Data gap: old point is invalid
                targ_temp(j,iold)=0.0
              endif
            else
              if(abs(targ_temp(j,iold)-targ_temp(j,iscan)).le.d(j).and.
     &           abs(inst_temp(j,iold)-inst_temp(j,iscan)).le.d(j))then
                validfound=.true.    !Diff good - both points accepted

! See whether there are any prior points which can be assigned this temperature
                do 200 i=iold-1,iold-nprtgap(j),-1
                  if (i.lt.1) goto 210
                  if (prtstat(j,i)) goto 210   !Quit loop at first good point
! Replace faulty target or instrument temps with old value, but leave prtstat
! as false
                  difft = abs(targ_temp(j,i)-targ_temp(j,iold))
                  diffi = abs(inst_temp(j,i)-inst_temp(j,iold))
                  if(difft.gt.d(j)) targ_temp(j,i)=targ_temp(j,iold)
                  if(diffi.gt.d(j)) inst_temp(j,i)=inst_temp(j,iold)
                  if (iprint.ge.1)
     &             write(ama_cllgu,*)'Scan ',i,' Inst ',j,
     &              ': PRT substituted by later value'
  200           continue
  210           continue  

              else
                prtstat(j,iold)=.false.  !Diff bad - reject old point
                targ_temp(j,iold)=0.0
              endif
            endif
            iold=iscan
          endif
        else                         !Valid data already present
          if ((iscan-iold).gt.nprtgap(j)) then
            validfound=.false.   !Too long ago - start again
            if (prtstat(j,iscan)) then
              iold=iscan  
            else
              iold=-999
            endif
          else                        !Test for consistency
            difft = abs(targ_temp(j,iold)-targ_temp(j,iscan))
            diffi = abs(inst_temp(j,iold)-inst_temp(j,iscan))
            if (difft.le.d(j).and.diffi.le.d(j)) then
              iold=iscan              !New point valid - update iold
            else
! Replace faulty target or instrument temps with old value, but set prtstat 
! false
              if(difft.gt.d(j)) targ_temp(j,iscan)=targ_temp(j,iold)
              if(diffi.gt.d(j)) inst_temp(j,iscan)=inst_temp(j,iold) 
              prtstat(j,iscan)=.false.
              if (iprint.ge.1)
     &         write(ama_cllgu,*)'Scan ',iscan,' Inst ',j,
     &            ': PRT substituted by prior value'
            endif
          endif
        endif
 400  continue  ! end of scan line loop---------------------
      if (.not.validfound.and.iold.gt.0) then
        prtstat(j,iold)=.false. !Set last point if necessary
        targ_temp(j,iold)=0.0
      endif
 410  continue  ! end of instrument loop
!
! Set flags
!
      do 500 iscan = 1 , ama_h_scnlin
        if (targ_temp(1,iscan).eq.0.0.and.
     &      targ_temp(2,iscan).eq.0.0.and.
     &      targ_temp(3,iscan).eq.0.0) then 
          linqualflgs(iscan) = IOR(linqualflgs(iscan),i4word00002000) 
                  !Not calibrated due to bad PRT data
          do i=1,15
            chqualflg(i,iscan)=IOR(chqualflg(i,iscan),i2word8)  !No good PRTs
          enddo
        else
          do j=1,3         !Loop round 3 instruments
            if (targ_temp(j,iscan).eq.0.0) then       !Flag bad PRT data
             linqualflgs(iscan) = IOR(linqualflgs(iscan),i4word00000800) 
                  !Some uncalibrated channels
             do i=instmin(j),instmax(j)
              chqualflg(cindex(i),iscan)=
     &                       IOR(chqualflg(cindex(i),iscan),i2word8)
             enddo         !No good PRTs for this channel and scan line
            endif
                           !Flag PRT data which has been corrected
            if (targ_temp(j,iscan).gt.0.0.and..not.prtstat(j,iscan))then
             linqualflgs(iscan)=IOR(linqualflgs(iscan),i4word00001000) 
                                                          !Marginal PRT data
             do i=instmin(j),instmax(j)              
              chqualflg(cindex(i),iscan)=
     &                          IOR(chqualflg(cindex(i),iscan),i2word8)
             enddo         !No good PRTs for this channel and scan line
            endif
          enddo
        endif
 500  continue
      return
      end
