      subroutine ama_smpmn(lstat,ispvw,pllo,itmean,spmean,sampok)
!+
!NOM/NAME $Id: ama_smpmn.F 553 2017-09-29 12:34:44Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!     This module computes the average AMSU-A internal target and
!     space view counts for a single scan line.
!     Also performs quality control on the space view and internal
!     target view counts and updates the scan line flags if necessary.
!
!DESCRIPTION
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions
!
!PROPRIETAIRE/OWNER
!     Nigel Atkinson
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES)
!(version   date      comment           <author>)
! 0.0                 AMSU-B          R. Saunders
! 1.0      June 96                    N. Atkinson
! 1.1      May 98   Update 1B format  N. Atkinson
! 1.2      July 98  relax one of the quality control checks
!         according to Nigel' mail on 17 July   M. Derrien 
! 1.3      March 03  f90 test         T.Labrot
!            I.OR.J not correct and modified by IOR(I,J)
!            I and J must have the same type
!            ==> created local variables : i2word48,i2word16,i2word32
!            Changed  type XXX/yyy/
!                 to  type XXX
!                     data XXX/yyy/
! 1.4      Feb 15  Bug fix: Line-to-line check was using pllo value for
!                  invalid scan.
! 1.5      Sep 17  Improve diagnostic when there is an angle discrepancy.
!
!ARGUMENTS  lstat(mxlin) - in status of scan
!           ispvw(3,mxlin) - in space view selected (0-3)
!           pllo(mxlin) - in PLLO for chan 9-14
!           itmean(15,mxlin) - out scan mean internal target counts
!           spmean(15,mxlin) - out scan mean space view counts
!           sampok(15,mxlin)- in/out cal view counts flag
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!           none
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################

!--DECLARATIONS----------------------------------------------------------------
      implicit none
!
!
! includes/commons
#include "ama_cinit.h"
#include "amaclin.h"
#include "ama1b.h"
#include "amascn.h"
!
! local variables
      integer ichan,ichan1,iscan        !index of loops
      integer ispvw(3,mxlin),pllo(mxlin),ncalgap(3)
      integer w1,w2,c1,c2,inst,instarr(15),chanoff1(15),iold
      data instarr/3,3,2,2,2,1,1,2,1,1,1,1,1,1,1/  !1=A1-1,2=A1-2,3=A2
! chanoff1 is used to index intcntmin,intcntmax,spcntmin,spcntmax:
      data chanoff1/0,0,0,0,0,0,0,0,7,7,7,7,7,7,0/ !Chan offset for PLLO2
      real pc1,pc2,pw1,pw2,e,s1,s2,i1,i2,d,err1,err2
      real itmean(15,mxlin),spmean(15,mxlin)
      logical*1 sampok(15,mxlin),validfound
      logical*1 lstat(mxlin),warnwarm(3),warnspace(3)
      
      integer*2 i2word48,i2word16,i2word32
      parameter (i2word48=48)
      parameter (i2word16=16)
      parameter (i2word32=32)
!
!--END OF DECLARATIONS---------------------------------------------------------
!
!----start scan line loop----------------------------------------------------
      do 200 iscan = 1 , ama_h_scnlin
      warnwarm(1)=.true.
      warnwarm(2)=.true.
      warnwarm(3)=.true.
      warnspace(1)=.true.
      warnspace(2)=.true.
      warnspace(3)=.true.
      if (lstat(iscan))then             !scan line ok
!----start channel loop------------------------------------------------------
      do 100 ichan = 1 , 15
       inst=instarr(ichan)
       spmean(ichan,iscan) = 0.0
       itmean(ichan,iscan) = 0.0
       if (pllo(iscan).eq.1) then
         ichan1 = ichan
       else
         ichan1 = ichan + chanoff1(ichan)
       endif

       if(sampok(ichan,iscan))then
!...perform quality checks on counts from both readings
!Note: division by 2 is now performed in decommutation (NCA, May 98)

       if (ichan.le.2) then   
         w1 = a2_warmcal(ichan + 2, iscan)     !First reading, chan 1-2
         w2 = a2_warmcal(ichan + 4, iscan)     !Second reading, chan 1-2
         c1 = a2_coldcal(ichan + 2, iscan)     !First reading, chan 1-2
         c2 = a2_coldcal(ichan + 4, iscan)     !Second reading, chan 1-2
       else                   
         w1 = a1_warmcal(ichan-2 + 4, iscan)     !First reading, chan 3-15
         w2 = a1_warmcal(ichan-2 +17, iscan)     !Second reading, chan 3-15
         c1 = a1_coldcal(ichan-2 + 4, iscan)     !First reading, chan 3-15
         c2 = a1_coldcal(ichan-2 +17, iscan)     !Second reading, chan 3-15
       endif
 
!...do gross limit check

       if(w1.gt.intcntmax(ichan1) .or. w1.lt.intcntmin(ichan1)
     &   .or.w2.gt.intcntmax(ichan1) .or. w2.lt.intcntmin(ichan1))then
         sampok(ichan,iscan)=.false.
         if (iprint.ge.1)
     &    write(ama_cllgu,*)'Warm view gross limit err: scan',iscan,
     &        ',chan',ichan
       endif
       if(c1.gt.spcntmax(ichan1) .or. c1.lt.spcntmin(ichan1)
     &   .or.c2.gt.spcntmax(ichan1) .or. c2.lt.spcntmin(ichan1))then
         sampok(ichan,iscan)=.false.
         if (iprint.ge.1)
     &    write(ama_cllgu,*)'Cold view gross limit err: scan',iscan,
     &        ',chan',ichan
       endif

!...do antenna pointing check on space view
!Note: division by 2 is now performed in decommutation (NCA, May 98)

       if (instarr(ichan).eq.3) then   !A2
         pc1 = ant_off_a2 + (a2_coldcal(1,iscan)  )*ant_coef_a2 
         pc2 = ant_off_a2 + (a2_coldcal(2,iscan)  )*ant_coef_a2 
         pw1 = ant_off_a2 + (a2_warmcal(1,iscan)  )*ant_coef_a2
         pw2 = ant_off_a2 + (a2_warmcal(2,iscan)  )*ant_coef_a2
         e = pos_errc_a2
       else
         if (instarr(ichan).eq.1) then    !A1:Antenna 1 
           pc1 = ant_off_a11 + (a1_coldcal(1,iscan)  )*ant_coef_a11 
           pc2 = ant_off_a11 + (a1_coldcal(3,iscan)  )*ant_coef_a11 
           pw1 = ant_off_a11 + (a1_warmcal(1,iscan)  )*ant_coef_a11 
           pw2 = ant_off_a11 + (a1_warmcal(3,iscan)  )*ant_coef_a11 
           e = pos_errc_a11
         else                             !A1:Antenna 2
           pc1 = ant_off_a12 + (a1_coldcal(2,iscan)  )*ant_coef_a12 
           pc2 = ant_off_a12 + (a1_coldcal(4,iscan)  )*ant_coef_a12 
           pw1 = ant_off_a12 + (a1_warmcal(2,iscan)  )*ant_coef_a12 
           pw2 = ant_off_a12 + (a1_warmcal(4,iscan)  )*ant_coef_a12 
           e = pos_errc_a12
         endif
       endif
       pc1 = AMOD(pc1+360+135, 360.0)-135.0   ! -135 to +225 degrees
       pc2 = AMOD(pc2+360+135, 360.0)-135.0   ! -135 to +225 degrees
       pw1 = AMOD(pw1+360+135, 360.0)-135.0   ! -135 to +225 degrees
       pw2 = AMOD(pw2+360+135, 360.0)-135.0   ! -135 to +225 degrees
       err1 = pc1-scanpos(1,ispvw(inst,iscan)+1)
       err2 = pc2-scanpos(1,ispvw(inst,iscan)+1)
       if (abs(err1) .gt. e .or. abs(err2) .gt. e) then
         if ( iprint.ge.1 .and. warnspace(inst)) then
           write(ama_cllgu,10) iscan,inst,err1,err2
   10      format(' **Warning Sp view select & ',
     1     'ant pos disagree: scan=',I4,' antenna=',
     2     I1,' Error=',2F6.1)
           warnspace(inst) = .false.
         endif
         sampok(ichan,iscan)=.false.
       endif

!...do antenna pointing check on internal target view

       err1 = pw1-scanpos(2,ispvw(inst,iscan)+1)
       err2 = pw2-scanpos(2,ispvw(inst,iscan)+1)
       if (abs(err1) .gt. e .or. abs(err2) .gt. e) then
         if ( iprint.ge.1 .and. warnwarm(inst))then
           write(ama_cllgu,20) iscan,inst,err1,err2
   20      format(' **Warning bad antenna pointing ',
     1     'on cal target: scan=',I4,' antenna=',
     2     I1,' Error=',2F6.1)
           warnwarm(inst) = .false.
         endif
         sampok(ichan,iscan) = .false.
       endif  

!...check difference between samples
       if (abs(w1-w2).gt.2*dcount(ichan1)) then
         sampok(ichan,iscan) = .false.
         if (iprint.ge.1)
     &    write(ama_cllgu,*)'Warm view difference err: scan',iscan,
     &     ',chan',ichan
       endif
       if (abs(c1-c2).gt.dcount(ichan1)) then
         sampok(ichan,iscan) = .false.
         if (iprint.ge.1)
     &    write(ama_cllgu,*)'Cold view difference err: scan',iscan,
     &     ',chan',ichan
       endif
!
!..compute mean sample
!
       if (sampok(ichan,iscan)) then
         spmean(ichan,iscan) = (c1+c2)/2.0
         itmean(ichan,iscan) = (w1+w2)/2.0
       endif
!
       endif     !end of 'if sampok(...
  100  continue   !-------end of channel loop------------
      endif      !end of 'if lstat(...
!
 200  continue     !-------end of scan line loop----------
!
! Now look at line-to-line consistency of the Space and Internal target counts.
! The difference between successive 'good' values must be less than the 
! 'dcount' values for each channel from the calibration data file. 
! A pair of consistent readings is necessary to define the start of a 'good'
! sequence, after which inconsistent readings are flagged as bad. If
! more than 'ncalgap' scans elapses since the last good point, the sequence is
! restarted. Isolated valid readings (i.e. more than 'ncalgap' scans between
! each one) are considered unreliable and are flagged as bad.
!
      ncalgap(1)=ncalgap_a11
      ncalgap(2)=ncalgap_a12
      ncalgap(3)=ncalgap_a2 
      do 500 ichan = 1 , 15
        validfound=.false.
        iold = -999

        do 400 iscan = 1 , ama_h_scnlin
          if (pllo(iscan).eq.1) then
            ichan1 = ichan
          else
            ichan1 = ichan + chanoff1(ichan)
          endif
          d = dcount(ichan1)
          if (.not.validfound) then   !Start of sequence - no good data yet
            if(lstat(iscan).and.sampok(ichan,iscan))then
              if ((iscan-iold).gt.ncalgap(instarr(ichan))) then
                if (iold.gt.0) then
                  sampok(ichan,iold)=.false. !Data gap: old channel is invalid
                  linqualflgs(iold)=IOR(linqualflgs(iold),2048) !Flag bad channel
                      !This point could be OK, but there are no other good 
                      !points in the vicinity, so flag both internal and space 
                      !counts as bad.
                  chqualflg(ichan,iold)=
     &                      IOR(chqualflg(ichan,iold),i2word48)
                endif
              else
                s1=spmean(ichan,iold)   !Test space and internal counts for
                s2=spmean(ichan,iscan)  !consistency.
                i1=itmean(ichan,iold)
                i2=itmean(ichan,iscan)
                if(abs(s1-s2).le.d.and.abs(i1-i2).le.d)then
                  validfound=.true.    !Diff good - both points accepted
                  if (iold.gt.1.and.iprint.ge.1) write(ama_cllgu,*)
     & 'Start of good space/target data for chan',ichan,', Scan',iold 
                else
                  sampok(ichan,iold)=.false.  !Diff bad - reject old point
                  linqualflgs(iold)=IOR(linqualflgs(iold),2048) !Flag bad channel
                  if (abs(s1-s2).gt.d)
     &             chqualflg(ichan,iold)=
     &                       IOR(chqualflg(ichan,iold),i2word16)
                  if (abs(i1-i2).gt.d)
     &             chqualflg(ichan,iold)=
     &                       IOR(chqualflg(ichan,iold),i2word32)
                endif
              endif
              iold=iscan
            endif
          else          !Valid data already present
            if ((iscan-iold).gt.ncalgap(instarr(ichan))) then 
              validfound=.false.     !Too long ago - start again
              if (lstat(iscan).and.sampok(ichan,iscan))then
                iold=iscan  
              else
                iold=-999
              endif
            else
              if (lstat(iscan).and.sampok(ichan,iscan))then
                s1=spmean(ichan,iold)
                s2=spmean(ichan,iscan)
                i1=itmean(ichan,iold)
                i2=itmean(ichan,iscan)
                if(abs(s1-s2).le.d.and.abs(i1-i2).le.d)then
                  iold=iscan             !New point valid - update iold
                else
                  sampok(ichan,iscan)=.false.  !New point is inconsistent
                  linqualflgs(iscan)=IOR(linqualflgs(iscan),2048) !Bad channel
                  if (abs(s1-s2).gt.d) then
                   chqualflg(ichan,iscan)=
     &                   IOR(chqualflg(ichan,iscan),i2word16)
                   if (iprint.ge.1) write(ama_cllgu,*)
     & 'Space view inconsistent with previous line: Chan',ichan,
     & ', Scan',iscan
                  endif
                  if (abs(i1-i2).gt.d) then
                   chqualflg(ichan,iscan)=
     &                      IOR(chqualflg(ichan,iscan),i2word32)
                   if (iprint.ge.1) write(ama_cllgu,*)
     & 'Target view inconsistent with previous line: Chan',ichan,
     & 'Scan',iscan
                  endif
                endif
              endif  
            endif
          endif
 400    continue  ! end of scan line loop---------------------
        if (.not.validfound.and.iold.gt.0) then
          sampok(ichan,iold)=.false.  !Set last point flags
          linqualflgs(iold)=IOR(linqualflgs(iold),2048)    !Bad channel
          chqualflg(ichan,iold)=IOR(chqualflg(ichan,iold),i2word48)
        endif
 500  continue    ! end of channel loop------------------------
      return
      end
