      subroutine ama_status(lstat,ispvw,sampok,pllo)
!+
!NOM/NAME $Id: ama_status.F 46 2006-08-18 14:39:27Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!     This module checks the quality indicators on each scan line are OK
!     Also checks status of instrument (ie antenna scanning and channels are
!     switched on. If not OK for calibration then status flags returned
!     as false.  Also the scan line quality indicators are updated.
!
!
!DESCRIPTION
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions
!
!PROPRIETAIRE/OWNER
!     Nigel Atkinson
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES)
!(version   date      comment           <author>)
! 0.0                 AMSU-B          R. Saunders
! 1.0      June 96        no          N. Atkinson
! 1.1      May 98     Update 1B format N. Atkinson
! 1.2      March 03   f90  test        T.Labrot
!            I.AND.J not correct and modified by IAND(I,J)
!            I.OR.J not correct and modified by IOR(I,J)
!      I and J must have the same type ==> created i2word30x local variable
!            Changed  type XXX/yyy/
!                 to  type XXX
!                     data XXX/yyy/
! 1.7      17/01/2006 : Philippe Marguinaud : change hex constants from
!                       '1234'X to Z'1234'
! 1.8      17/08/2006 use i4word00001e1b, etc. NCA
!                
!ARGUMENTS
!      lstat(mxlin) L*1 in/out  If .TRUE. then data ok if false don't use
!      ispvw(3,mxlin) I*4 out  space viewing direction (0-3)
!      sampok(15,mxlin) l*1 out calibrate status for each channel
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################

!--DECLARATIONS----------------------------------------------------------------
      implicit none
c-----------------------------------------
c
c
!
! includes/commons
#include "amaclin.h"
#include "ama1b.h"
#include "amascn.h"
#include "ama_cinit.h"
!
! local variables
      integer*2 i2word30x
      parameter (i2word30x=Z'30')
      integer*2 a1_imode,a2_imode,znormal_a1,znormal_a2
      integer*4 i,iscan
      integer*4 ispvw(3,mxlin),pllo(mxlin)
      logical*1 lstat(mxlin),sampok(15,mxlin)
      logical*1 warn1,warn2
      data warn1/.true./
      data warn2/.true./

      integer*4 i4word00001e16,i4word90000000,i4word00000400,
     & i4word00000800,i4word00000008,i4word00006000,i4word00002000
      parameter (i4word00001e16=Z'00001e16')
      parameter (i4word90000000=Z'90000000')
      parameter (i4word00000400=Z'00000400')
      parameter (i4word00000800=Z'00000800')
      parameter (i4word00000008=Z'00000008')
      parameter (i4word00006000=Z'00006000')
      parameter (i4word00002000=Z'00002000')
      
!
!--END OF DECLARATIONS---------------------------------------------------------

      if (iprint.ge.1) write(ama_cllgu,*)
      if (iprint.ge.1) write(ama_cllgu,*) '***************************'
      if (iprint.ge.1) write(ama_cllgu,*) '**   SUBROUTINE AMA_STATUS*'
      if (iprint.ge.1) write(ama_cllgu,*) '***************************'
!..Loop in scan line................................................
      do 100 iscan = 1 , ama_h_scnlin
      do 10 i = 1 , 15
       sampok(i,iscan) = .true.
 10   continue

! check scan line quality indicator
!
      if ( qualind(iscan) .lt. 0 )then
        lstat(iscan) = .FALSE.        ! bad scan line detected from ingest
        do i=1,15
          sampok(i,iscan)=.FALSE.
        enddo
        go to 100
      endif
!
! Check Digital A housekeeping to see whether both instruments are scanning OK. 
! Use characters 1 and 2 only. 1st character is MS byte of integer*2,
! 2nd character as LS byte. See UIIS table 8. Expect:
! A1: Scanner power on (bit 1  & 2 set), Full scan mode (bit 9 set),
! A2: Scanner power on (bit 1  & 2 set), Full scan mode (bit 9 set),
!
      znormal_a1 = Z'0206' 
      znormal_a2 = Z'0206' 
      
      a1_imode = IAND(a1_dhk(iscan),i4word00001e16)   !Ignore irrelevant bits
      a2_imode = IAND(a2_dhk(iscan),i4word00001e16)
      if ((a1_imode.ne.znormal_a1).and.(a2_imode.ne.znormal_a2)) then
! Both instruments bad
        do i=1,15
          sampok(i,iscan)=.FALSE.
        enddo
        lstat(iscan) = .FALSE.    
        qualind(iscan) = IOR(qualind(iscan),i4word90000000) !set bits 28,31
        linqualflgs(iscan) = IOR(linqualflgs(iscan),i4word00000400) !set bit 10
        if (iprint.ge.1)
     &   write(ama_cllgu,*)
     &   'Line ',iscan,': Both instruments power off or not scanning'
        go to 100
      endif

      if (a1_imode.ne.znormal_a1) then  
! A1 is bad
        linqualflgs(iscan) = IOR(linqualflgs(iscan),i4word00000800)  !set bit 11
        do 15 i=3,15
          chqualflg(i,iscan)=IOR(chqualflg(i,iscan),i2word30x) !set bits 4 and 5
          sampok(i,iscan)=.false.                        !of chan 3-15
  15    continue                                        
        if (iprint.ge.1)
     &   write(ama_cllgu,*)
     &   'Line ',iscan,': A1 power off or not scanning'
      endif

      if (a2_imode.ne.znormal_a2) then  
! A2 is bad
        linqualflgs(iscan) = IOR(linqualflgs(iscan),i4word00000800)  !set bit 11
        do 20 i=1,2
          chqualflg(i,iscan)=IOR(chqualflg(i,iscan),i2word30x) !set bits 4 and 5
          sampok(i,iscan)=.false.                         !of chan 1-2
  20    continue
        if (iprint.ge.1)
     &   write(ama_cllgu,*)
     &   'Line ',iscan,': A2 power off or not scanning'
      endif
!
!  get PLLO for A1-1
!
      if (IAND(a1_dhk(iscan),i4word00000008).eq.0) then
        pllo(iscan)=2  !Redundant
      else
        pllo(iscan)=1  !Primary
      endif
!      ama_h_plloid = pllo(iscan)   !Store in header (obsolete M.Derrien)
!
!  get space view position (0->3) for both instruments
!
!     ispvw(1,iscan) = IAND(a1_dhk(iscan),'6000'X)/'2000'X
      ispvw(1,iscan) = IAND(a1_dhk(iscan),i4word00006000)/i4word00002000
      ispvw(2,iscan) = ispvw(1,iscan)   !A1-1 and A1-2 have same position
      if (ispvw(1,iscan).ne.spview_a1 .and. warn1) then
       if(iprint.ge.1) write(ama_cllgu,'(A,I1,A,I1,A,I5)')
     &'  Warning A1 space view ',ispvw(1,iscan),
     &' selected, ',spview_a1,' is recommended, from scan',iscan
       warn1 = .false. 
      endif
      if (ispvw(1,iscan).eq.spview_a1.and..not.warn1) then
       if(iprint.ge.1) write(ama_cllgu,'(A,I5)')
     & '  A1 space view select OK, from scan',iscan
       warn1 = .true.
      endif
!
!     ispvw(3,iscan) = IAND(a2_dhk(iscan),'6000'X)/'2000'X
      ispvw(3,iscan) = IAND(a2_dhk(iscan),i4word00006000)/i4word00002000
      if (ispvw(3,iscan).ne.spview_a2 .and. warn2) then
       if(iprint.ge.1) write(ama_cllgu,'(A,I1,A,I1,A,I5)')
     &' Warning A2 space view ',ispvw(3,iscan),
     &' selected, ',spview_a2,' is recommended, from scan',iscan
       warn2 = .false.
      endif
      if (ispvw(3,iscan).eq.spview_a2.and..not.warn2) then
       if(iprint.ge.1) write(ama_cllgu,'(A,I5)')
     & ' A2 space view select OK, from scan',iscan
       warn2 = .true.
      endif
!
 100  Continue   ! ......end of scan line loop
!
      return
      end
