      subroutine ama_updt(nvstat,calok,nbnavcal,ispvw)
!+
!NOM/NAME $Id: ama_updt.F 410 2014-08-08 14:16:37Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!    This module updates the AMSU-A 1B header array and writes out
!    header and navigated, calibrated scan lines to the 1B file.
!
!DESCRIPTION
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions
!
!PROPRIETAIRE/OWNER
!     Nigel Atkinson
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES)
!(version   date      comment           <author>)
! 0.0                 AMSU-B          R. Saunders
! 1.0      June 96                    N. Atkinson
! 1.1      July 96    change pos/earthl  M. Derrien
! 1.2      May 98     Update 1B format   N. Atkinson
! 1.3      June  98   invert ama_h_temrad a & b positions to conform
!                                       M. Derrien
! 1.4      Dec 03     increment 'nbnavcal' is some rather than all channels are
!                     calibrated.        N. Atkinson
! 1.5      April 04   Change statement for test on noaa klm serie
!                                         P. Brunel
!
!ARGUMENTS
! nvstat(mxlin) l*1 in array containing navigation flags
! calok(15,mxlin) l*1 in array containing calibration flags
! nbnavcal i*4 out number of navigated and calibrated scans
! ispvw(3,mxlin) in Space view readings
!------------------------------------------------------------------------------
!--DECLARATIONS----------------------------------------------------------------
      implicit none

!includes/commons
#include "amaclin.h"
#include "ama1b.h"
#include "amascn.h"
#include "ama_cinit.h"
!
!local  variables
      integer iscan,ios,j,i,nh
      integer nbnavcal
      logical*1 nvstat(mxlin),calok(15,mxlin),somegood
      integer ispvw(3,mxlin),instarr(15)
      real scala(4)
      integer satid
      integer noanum
      integer*2 work2
      integer id_in             ! noaa id corresponding to satid variable
      integer id_noaak          ! noaa id corresponding to Noaa-K == 15
      logical switch_7_15
      data instarr/3,3,2,2,2,1,1,2,1,1,1,1,1,1,1/  !1=A1-1,2=A1-2,3=A2
      data scala/1E4,1E9,1E16,1E20/   !Scaling for digital A coefs
      data id_noaak/15/
! functions
      integer ord1bid
      integer noascid
!
!--END OF DECLARATIONS-------------------------------------------------
      if(iprint.ge.1)then
       write(ama_cllgu,*)' ******************************'
       write(ama_cllgu,*)' *          AMA_UPDT'
       write(ama_cllgu,*)' ******************************'
      endif
!
      satid = ama_h_satid
      noanum = ord1bid(satid,ama_h_startdatajd)
      id_in = noascid(noanum)

      switch_7_15=.false.
      !if ( noanum.eq.12 .and. ama_h_callocsclin.eq.0
      if ( id_in.eq.id_noaak .and. ama_h_callocsclin.eq.0
     &     .and. (ama_clflg.ne.0.or.ama_cflg.ne.0)) then
        switch_7_15=.true.
        call ml_wt('info','channels 7/15 are switched when reading')
        write(ama_cllgu,*)
     &       'Info:channels 7/15 are switched when reading' 
      endif
      nbnavcal = 0
      nh = ama_h_hdrcnt
!..determine nbnavcal
      do 10 iscan = 1 , ama_h_scnlin
!  If ama_lflg=0 (i.e. not running navigation), assume navigation is OK
        if(nvstat(iscan).or.(ama_lflg.eq.0))then
!        if(nvstat(iscan))then
!          allgood=.true.
!          do i=1,15
!            if (.not.calok(i,iscan)) allgood=.false.
!          enddo
          somegood=.false.
          do i=1,15
            if (calok(i,iscan)) somegood=.true.
          enddo
          if (somegood) nbnavcal = nbnavcal+1
        endif
 10   continue
!..update header (nb with correct scaling factors)
      ama_h_callocsclin = nbnavcal
      ama_h_tsensid(1) = tsensid_a11
      ama_h_tsensid(2) = tsensid_a12
      ama_h_tsensid(3) = tsensid_a2
      do i=1,4
        do j=1,3
          ama_h_reftemp(3*(i-1)+j) = nint(shelftemp(i,j)*100)
          ama_h_muxtemp(3*(i-1)+j) = nint(muxtemp(i,j)*100)
        enddo
      enddo  
      do i=1,15   !Channel count
        do j=1,3    !3 temperatures
          ama_h_bias(4*(i-1)+j) = nint(intcorr(i,j)*1000)
        enddo
        ama_h_bias(4*(i-1)+4) = nint(spcorr(i,
     &     ispvw(instarr(i),ama_h_scnlin)+1)*1000) 
              !Use final space view reading to set space correction header
      enddo
      do i=16,21  !PLLO 2 channels
        do j=1,3    !3 temperatures
          ama_h_bias(3*(i-16)+60+j) = nint(intcorr(i,j)*1000)
        enddo
      enddo
      do i=1,21  !All channels
        do j=1,3   !3 temperatures
C          ama_h_nonlin(3*(i-1)+j) = nint(nonlin(i,j)*1000)
          ama_h_nonlin(3*(i-1)+j) = nint(nonlin(i,j)*1E6)
        enddo
      enddo
      do i=1,15  !Temperature to radiance
        ama_h_temrad(1+3*(i-1)) = nint(freq(i)*1E6)
        ama_h_temrad(2+3*(i-1)) = nint(b(i)*1E6)
        ama_h_temrad(3+3*(i-1)) = nint(a(i)*1E6)
      enddo

!..add digital A telemetry conversion coeffs
      do i=1,45
        do j=1,4
          ama1_h_digacoef((i-1)*4+j) = nint(digacoef_a1(j,i)*scala(j))
        enddo
      enddo
      do i=1,19
        do j=1,4
          ama2_h_digacoef((i-1)*4+j) = nint(digacoef_a2(j,i)*scala(j))
        enddo
      enddo

!..add analogue telemetry conversion coeffs
      do i=1,27
        do j=1,2
          ama1_h_analcoef((i-1)*2+j) = nint(analcoef_a1(j,i)*1000.0)
        enddo
      enddo
      do i=1,15
        do j=1,2
          ama2_h_analcoef((i-1)*2+j) = nint(analcoef_a2(j,i)*1000.0)
        enddo
      enddo

!### N.B. Navigation header words were updated by 'hd1bnav' ###

!..write out header to disk
      call amahdw(ama_l1bu,ios)
      if(ios.ne.0)then
       write(ama_cllgu,*)' AMSU-A 1B header could not be written'
       stop
      endif
!
!..write out all data records to disk
!
      do 30 iscan = 1 , ama_h_scnlin  !-----start scan loop-------
!..read current 1B record before updating
       call amadtr(ama_l1bu,iscan,ios)
       if(ios.ne.0)then
        write(ama_cllgu,*)' AMSU-A 1B data record could not be read'
        stop
       endif
       ama_clockdrift = clockdrift(iscan)
       ama_qualind = qualind(iscan)
       ama_scnlinbit = scnlinbit(iscan)
       ama_linqualflgs = linqualflgs(iscan)

       ama_scnlinyr = scnlinyr(iscan)
       ama_scnlindy = scnlindy(iscan)
       ama_scnlintime = scnlintime(iscan)
       ama_attangtime = attangtime(iscan)
       ama_rollang = rollang(iscan)
       ama_pitchang = pitchang(iscan)
       ama_yawang = yawang(iscan)
       ama_scalti = scalti(iscan)
       ama_navstat = navstat(iscan)

       do 15 i = 1, mxpix*3
        ama_ang(i) = ang(i,iscan)
 15    continue
       do 25 i = 1, mxpix*2
        ama_pos(i) = earthl(i,iscan)
 25    continue
!.. switch 7/15 discussion for earth_data coldcal warmcal temps contents
        if(switch_7_15) then
          do i = 1, mxpix
            work2=ama1_earth_data(9,i)
            ama1_earth_data(9,i)=ama1_earth_data(17,i)
            ama1_earth_data(17,i)=work2
          enddo
          work2=ama1_temps(11)
          ama1_temps(11)=ama1_temps(13)
          ama1_temps(13)=work2
          work2=ama1_temps(21)
          ama1_temps(21)=ama1_temps(24)
          ama1_temps(24)=work2
          work2=ama1_coldcal(9)
          ama1_coldcal(9)=ama1_coldcal(17)
          ama1_coldcal(17)=work2
          work2=ama1_coldcal(22)
          ama1_coldcal(22)=ama1_coldcal(30)
          ama1_coldcal(30)=work2          
          work2=ama1_warmcal(9)
          ama1_warmcal(9)=ama1_warmcal(17)
          ama1_warmcal(17)=work2
          work2=ama1_warmcal(22)
          ama1_warmcal(22)=ama1_warmcal(30)
          ama1_warmcal(30)=work2
       endif
!.. end of discussion on switch 7/15 variables
       do 35 i = 1 , 15
         ama_chqualflg(i) = chqualflg(i,iscan)
 35    continue
       do 40 i=1,45
         ama_pmcal(i) = pmcal(i,iscan)
         ama_sdcal(i) = sdcal(i,iscan)
 40    continue


!..write current updated 1B record
       call amadtw(ama_l1bu,iscan,ios)
       if(ios.ne.0)then
       write(ama_cllgu,*)' AMSU-A 1B data record could not be written'
       stop
       endif
 30   continue  !------end scan loop-------------
!
      return
      end
