      subroutine amaclexit(nbnavcal,nvstat,lstat,calok,sampok,prtstat)
!+
!NOM/NAME $Id: amaclexit.F 47 2006-08-21 08:44:41Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!    This module closes the amsuacl program and provides a summary
!    of the job if iprint is greater than or equal to 1.
!
!DESCRIPTION
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions
!
!PROPRIETAIRE/OWNER
!     Nigel Atkinson
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES)
!(version   date      comment           <author>)
! 0.0                 AMSU-B          R. Saunders
! 1.0      June 96                    N. ATKINSON
! 1.1      Aug 2000   exit 10 when bad cal/loc P. Brunel
! 1.2      May 2004   Use ncalreq to allow for bad channels. N C Atkinson
!
!ARGUMENTS
! nbnavcal i*4 in number of navigated and calibrated scans
! lstat(mxlin) l*1 in scan line quality flag (.true. is ok)
! nvstat(mxlin) l*1 in navigation quality flag
! calok(15,mxlin) l*1 in calibration ok flag
! sampok(15,mxlin) l*1 in cal view radiances quality flag
! prtstat(3,mxlin) l*1 in target and mixer prt quality flag
!------------------------------------------------------------------------------
!--DECLARATIONS----------------------------------------------------------------
      implicit none

!includes/commons
#include "amaclin.h"
#include "ama1b.h"
#include "amascn.h"
#include "ama_cinit.h"
!
!local  variables
      integer iscan,ic
      integer nbnavcal,nnav,ncal(15),nprt(3),nsamp(15),ngood
      logical*1 nvstat(mxlin),lstat(mxlin),calok(15,mxlin)
      logical*1 sampok(15,mxlin),prtstat(3,mxlin)
      integer ierr        ! 0 = OK
      integer ncaltot     ! total calibration number max is lines*channels
      integer ncalreq     ! required total calibration number
!
!--END OF DECLARATIONS-------------------------------------------------
      if(iprint.ge.1)then
       write(ama_cllgu,*)' ******************************'
       write(ama_cllgu,*)' *          AMACLEXIT         *'
       write(ama_cllgu,*)' ******************************'
!..print summary
!
      write(ama_cllgu,*)' '
      write(ama_cllgu,*)'     AMSUACL summary log'
      write(ama_cllgu,*)'     -------------------'
      nnav = 0
      nprt(1) = 0
      nprt(2) = 0
      nprt(3) = 0
      ngood = 0
!
      do 5 ic = 1 , 15
      nsamp(ic) = 0
      ncal(ic) = 0
 5    continue
      do 100 iscan = 1 , ama_h_scnlin
       if(nvstat(iscan))nnav = nnav + 1
       if(lstat(iscan))ngood = ngood + 1
       if (prtstat(1,iscan)) nprt(1)=nprt(1)+1
       if (prtstat(2,iscan)) nprt(2)=nprt(2)+1
       if (prtstat(3,iscan)) nprt(3)=nprt(3)+1
       do 99 ic = 1 , 15
        if(calok(ic,iscan))ncal(ic) = ncal(ic) + 1
        if(sampok(ic,iscan))nsamp(ic) = nsamp(ic) + 1
  99   continue
 100  continue
       
      write(ama_cllgu,10)nbnavcal
 10   format(1x,i5,' scan lines were calibrated and navigated')
      write(ama_cllgu,20)ama_h_scnlin-ngood
 20   format(1x,i5,' scan lines were flagged as corrupt data')
      write(ama_cllgu,30)nnav
 30   format(1x,i5,' scan lines were navigated')
      write(ama_cllgu,40)nprt(1),nprt(2),nprt(3)
 40   format(1x,i5,' scan lines had good prt data for A1-1',/,
     &       1x,i5,' scan lines had good prt data for A1-2',/,
     &       1x,i5,' scan lines had good prt data for A2')
       write(ama_cllgu,60)(nsamp(ic),ic=1,15)
 60   format(1x,'Chan 1=',i5,' scans of good cal views',/,
     &' Chan 2=',i5,' scans of good cal views',/,
     &' Chan 3=',i5,' scans of good cal views',/,
     &' Chan 4=',i5,' scans of good cal views',/,
     &' Chan 5=',i5,' scans of good cal views',/,
     &' Chan 6=',i5,' scans of good cal views',/,
     &' Chan 7=',i5,' scans of good cal views',/,
     &' Chan 8=',i5,' scans of good cal views',/,
     &' Chan 9=',i5,' scans of good cal views',/,
     &' Chan 10=',i5,' scans of good cal views',/,
     &' Chan 11=',i5,' scans of good cal views',/,
     &' Chan 12=',i5,' scans of good cal views',/,
     &' Chan 13=',i5,' scans of good cal views',/,
     &' Chan 14=',i5,' scans of good cal views',/,
     &' Chan 15=',i5,' scans of good cal views',/)
      write(ama_cllgu,50)(ncal(ic),ic=1,15)
 50   format(1x,'Chan 1=',i5,' scans calibrated',/,
     &' Chan 2=',i5,' scans calibrated',/,
     &' Chan 3=',i5,' scans calibrated',/,
     &' Chan 4=',i5,' scans calibrated',/,
     &' Chan 5=',i5,' scans calibrated',/,
     &' Chan 6=',i5,' scans calibrated',/,
     &' Chan 7=',i5,' scans calibrated',/,
     &' Chan 8=',i5,' scans calibrated',/,
     &' Chan 9=',i5,' scans calibrated',/,
     &' Chan 10=',i5,' scans calibrated',/,
     &' Chan 11=',i5,' scans calibrated',/,
     &' Chan 12=',i5,' scans calibrated',/,
     &' Chan 13=',i5,' scans calibrated',/,
     &' Chan 14=',i5,' scans calibrated',/,
     &' Chan 15=',i5,' scans calibrated',/)
      endif

!..close files
!
      write (*,*) 'ama_l1bu,ama_cllgu,ama_stu ',
     &   ama_l1bu,ama_cllgu,ama_stu
      close(ama_l1bu)
      close(ama_stu)
!
!.. final quality test for end of execution return code
      ierr = 0
      if(ama_lflg.eq.1 .and. nnav.le.10) then
         write(ama_cllgu,*) 'Error not enough navigated lines'
         ierr = 1
      endif

      ncaltot = 0
      ncalreq = 0
      do ic=1,15
        if (ncal(ic).ne.0) then
          ncaltot = ncaltot+ncal(ic)
          ncalreq = ncalreq + 10         !Minimum 10 lines per good channel
        endif
      end do
      if(ama_cflg.eq.1 .and. ncaltot.le.ncalreq) then
         write(ama_cllgu,*) 'Error not enough calibrated lines'
         ierr = 1
      endif

      close(ama_cllgu)

      if(ierr.ne.0) then
         ! all files are closed, exit 10 will be returned to user 
         ! through amsua.ksh script and enables to stop further processing
         call aappexit(10)
      endif

      return
      end
