      subroutine amaclin(ama_cloptn)
!+
!NOM/NAME $Id: amaclin.F 47 2006-08-21 08:44:41Z frna $
!
!----------------------------------------------------------------------------
!BUT/PURPOSE
!     This module initializes the AMSU-A navigation and calibration tasks.
!     It reads the input options (ama_cloptn)
!
!DESCRIPTION
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions
!
!PROPRIETAIRE/OWNER
!     Nigel Atkinson
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES)
!(version   date      comment           <author>)
! 0.0                 AMSU-B          R. Saunders
! 1.0      June 96                    N. Atkinson
! 1.1      July 96   add navigation   M. Derrien
! 1.2      May 98            replace read(7,*   by read(*.*
!                            suppress useless option 1 
!                            suppress useless option 10
! 1.3    July 2002  Correct test that fails some compilers   K Whyte
! 1.4  24 March 2003 Correct ios test on read of options     K Whyte
!
!ARGUMENTS
!     output: ama_cloptn(11)   (see after for the meaning)
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!     none
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################
!
!--DECLARATIONS--------------------------------------------------------------
      implicit none
! local variables
      integer i         !index of loop
      integer ios       !input/output status
      integer ama_cloptn(11)
!    ama_cloptn(1) -- 1 for only AMSU-A navigation/localisation (0 if not for)
!    ama_cloptn(2) -- 1 for only AMSU-A calibration (0 if not for)
!    ama_cloptn(3) -- 1 for statistics file (0 if not for)
!    ama_cloptn(4) -- logical unit for AMSU-A navig/loc/calib log file
!    ama_cloptn(5) -- logical unit for statistics file
!    ama_cloptn(6) -- logical unit AMSU-A level1b file
!    ama_cloptn(7) -- logical unit for SATPOS file
!    ama_cloptn(8) -- logical unit for the clock drift data file
!    ama_cloptn(9)-- logical unit for AMSU-A calibration coefficients file
!    ama_cloptn(10)-- logical unit for AMSU-A calib/test coefficients file
!    ama_cloptn(11)-- print option in the log.file
!                       = 0, only warning messages are printed
!                       = 1, entering routines and few values of the main
!                            parameters are printed
!                       = 2, computed values that would help in debugging
!                            are printed

! includes/commons
#include "amaclin.h"
#include "c_1bnav.h"
!
!--END OF DECLARATIONS---------------------------------------------------------

      read(*,*,iostat=ios) ama_cloptn
      if(ios.ne.0) then
         call ml_wti('error','amaclin reading options',ios)
         call aappexit(2)
       endif
! Check ama_cloptn values
      do i = 1,3
        if(ama_cloptn(i).gt.1 .or. ama_cloptn(i).lt.0) then
            write(ama_cloptn(4),*)
     &      ' **Invalid option number--correct input**' ,ama_cloptn(i)
            stop
        endif
      enddo

      if((ama_cloptn(11).ne.0).and.(ama_cloptn(11).ne.1).and.
     & (ama_cloptn(11).ne.2)) then
            write(ama_cloptn(4),*)
     &      ' **Invalid print option value--correct input**'
           stop
      endif


      ama_lflg    = ama_cloptn(1)
      ama_cflg    = ama_cloptn(2)
      if(ama_cloptn(1).eq.1.and.ama_cloptn(2).eq.1) then
        ama_clflg   = 1
      endif
      ama_stflg   = ama_cloptn(3)

      ama_cllgu   = ama_cloptn(4)
      ama_l1bu    = ama_cloptn(6)
      iprint      = ama_cloptn(11)

      if (ama_stflg.eq.1) then
        ama_stu    = ama_cloptn(5)
      else
        ama_stu   = 0
      endif

      if (ama_clflg.eq.1.or.ama_lflg.eq.1) then
        satposu     = ama_cloptn(7)
        ckdriftu    = ama_cloptn(8)
!??        ama_geou    = ama_cloptn(9)
        ama_coefu   = ama_cloptn(9)       !??  from 10
        ama_tstcoefu= ama_cloptn(10)      !?? from 11
! read attitude parameters
        read(*,*,iostat=ios) yaw, roll, pitch
        if(ios.ne.0) then
!         call ml_wti('error',
!     &       'while reading attitude values I/O status',ios)
!         call ml_wt('error','amsubcl aborted')
          write(ama_cloptn(4),*) 
     &      ' **Error while reading attitude values--correct input**'
          call aappexit(1)
        endif
! convert the values in radians
        yaw = yaw*1.d-3
        roll = roll*1.d-3
        pitch = pitch*1.d-3
      endif
      ama_geou=0
      if (ama_lflg.eq.0) then
        ama_coefu   = 0
        ama_tstcoefu= 0
      endif

      if (ama_cflg.eq.1) then
        ama_coefu   = ama_cloptn(9)
        ama_tstcoefu= ama_cloptn(10)
      endif
!
      return
      end
