        subroutine amasetu(noanum,lstat,nvstat,calok)
!+
!NOM/NAME $Id: amasetu.F 554 2017-09-29 13:03:06Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!     This module opens  the log file, the statistics file (if requested).
!     It also opens the AMSU-A ama_.level1b and reads the header,
!     all the records. It also checks data type.
!     The 1b data are ranged in commons ama1b.h
!
!DESCRIPTION
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions
!
!PROPRIETAIRE/OWNER
!     Nigel Atkinson
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES)
!(version   date      comment           <author>)
! 0.0                 AMSU-B          R. Saunders
! 1.0      June 96                    N. Atkinson
! 1.1      July 96   add navigation part M. Derrien
! 1.3       May 98      update variable names for housekeeping from 1B record
!                       accept an error when getting clock error, but apply 0
!                                        M. Derrien
! 1.4      June 98  add swicth 7/15 actions for noaa15 M. Derrien P. Brunel
! 1.5      July 98  removed swicth 7/15 on a1_temps M. Derrien
! 2.0      August00 Add a test on ierr, output of sp_read (TL)
! 2.1      Dec 00   Display attitude values  P. Brunel
! 2.1      Dec 00   Change position in the code of the display of
!                     the clock error    P. Brunel
! 2.2      March 03   f90  test          T.Labrot
!                    I.AND.J not correct and modified by IAND(I,J)
!                    jmod not correct and modified by mod
! 2.3      March 04   Modify for linux   N Atkinson / A Korpela
! 2.4      June 04   Allow noaa18 and noaa19  N Atkinson
! 2.5      July 04   Change statement for test on noaa klm serie
!                                         P. Brunel
! 2.6      July 04  Upgrade to navigation code version 2.
!                   def_att call changed for arguments and position in
!                   code (after sp_read)     P. Brunel
! 2.7      April 05 Correct status masks and array indexing in diagnostics. NCA
! 2.8      May 06   Remove tests on n < noaak ( PM/PB )
! 2.9      April 13 Initialise chqualflg to zero (for re-calibration). NCA
! 2.10     Oct 14   Initialise switched 7/15 chqualflg to zero. NCA
! 2.11     Sep 17   Initialise qualind to zero if we are re-calibrating NESDIS
!                   global data.
!
!ARGUMENTS
! noanum i*4 out  satellite id number (see satid file)
! lstat(mxlin) l*1 out scan line status flag
! nvstat(mxlin) l*1 out navigation status flag
! calok(15,mxlin) l*1 out calibration status flag
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
! ord1bid  amahdr amadtr ml_wti ml_wt ml_wtdp ml_wtr def_att calatt
! clkerr_get sp_read
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE MET OFFICE
!                  AT FITZROY RD., EXETER, EX1 3PB, U.K.
!#########################################################################

!--DECLARATIONS----------------------------------------------------------------
      implicit none
!
!
#include "amaclin.h"
#include "amascn.h"
#include "ama1b.h"
#include "c_1bnav.h"
#include "reclopt.h"
#include "p_earth.h"
!
! local variables
      integer i,j  !index of loops
      integer ios,ama_h_satid0,noanum
      integer scn
      integer loc_rl     ! local record length for opening AMSU-A 1B file
      integer att_mode          ! attitude mode
                                ! 0 = NOAA POES operational mode
                                ! 1 = METOP Yaw steering mode
                                ! 2 = Geocentric mode
! status flags
      logical*1 lstat(mxlin),calok(15,mxlin),nvstat(mxlin)
      logical switch_7_15 
      integer ierr,val
      double precision ijstrt,ijend
      character*6 scname
      integer id_in             ! noaa id corresponding to satid variable
      integer id_noaak          ! noaa id corresponding to Noaa-K == 15
!
! functions
      integer ord1bid
      character*6 noascnam
      integer noascid

      data id_noaak/15/

!    the following were for ukmo tests
!      character*6 noascnam(14)
!      data noascnam(12)/'NOAA-K'/
!      data noascnam(13)/'NOAA-L'/
!      data noascnam(14)/'NOAA-M'/
!
! parameters
      integer mask1E00, mask0016, mask0200, mask0006
      parameter (mask1E00 = Z'00001E00')
      parameter (mask0016 = Z'00000016')
      parameter (mask0200 = Z'00000200')
      parameter (mask0006 = Z'00000006')

! includes/commons
!--END OF DECLARATIONS---------------------------------------------------------

!open the log file
      open(ama_cllgu,form='formatted',access='sequential',
     &     status='new',iostat=ios)

!     If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
      if (ios.ne.0)
     &  open(ama_cllgu,form='formatted',access='sequential',
     &     status='unknown',iostat=ios)
!
      if (iprint.ge.1) write(ama_cllgu,*) '**************************'
      if (iprint.ge.1) write(ama_cllgu,*) '** Starting AMSUACL     **'
      if (iprint.ge.1) write(ama_cllgu,*) '**************************'
      if (iprint.ge.1) write(ama_cllgu,*)
      if (iprint.ge.1) write(ama_cllgu,*)
      if (iprint.ge.1) write(ama_cllgu,*) '***************************'
      if (iprint.ge.1) write(ama_cllgu,*) '**   SUBROUTINE AMASETU  **'
      if (iprint.ge.1) write(ama_cllgu,*) '***************************'


!open the statistics file
      if(ama_stflg.eq.1) then
        open(ama_stu,form='formatted',access='sequential',
     &       status='new',iostat=ios)

!     If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
        if (ios.ne.0)
     &    open(ama_stu,form='formatted',access='sequential',
     &       status='unknown')
      endif


!open and read the AMSU-A-level1b file
      loc_rl = reclama / rldiv
      open(ama_l1bu,form='unformatted',access='direct',recl=loc_rl,
     &     status='old',iostat=ios)

!     If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
      if (ios.ne.0)
     &  open(ama_l1bu,form='unformatted',access='direct',recl=loc_rl,
     &     status='unknown',iostat=ios)

      if(ios.ne.0) then
       write(ama_cllgu,*)'AMSU_A 1B file could not be opened,ios:',ios
       call ml_wti('error','AMSU_A 1B file could not be opened',ios)
       call aappexit(10)
!       stop
      else
       if(iprint.ge.1)
     & write(ama_cllgu,*)'AMSU-A 1B file has been opened,ios:',ios
      endif

      call amahdr(ama_l1bu,ios)
      if(ios.ne.0) then
       write(ama_cllgu,*)' AMSU-A 1B header could not be read,ios:',ios
       call ml_wti('error','AMSU_A 1B header could not be read',ios)
       call aappexit(10)
!       stop
      else
       if(ama_h_datatyp.ne.10)then
         write(ama_cllgu,*)' Not AMSU-A data '
         val=ama_h_datatyp
         call ml_wti('error','Not AMSU-A data',val)
         call aappexit(10)
!         stop
       endif
      endif
      if(IAND(ama1_h_inststat1,mask1E00).ne.mask0200) then
        write(ama_cllgu,*)' Warning A1 status: not scan mode'
      endif
      if(IAND(ama1_h_inststat1,mask0016).ne.mask0006) then
        write(ama_cllgu,*)' Warning A1 status: scan power off'
      endif
      if (ama1_h_statchrecnb.gt.0) then
        if(IAND(ama1_h_inststat2,mask1E00).ne.mask0200) then
          write(ama_cllgu,*)' Warning A1 2nd status: not scan mode'
        endif
        if(IAND(ama1_h_inststat2,mask0016).ne.mask0006) then
          write(ama_cllgu,*)' Warning A1 2nd status: scan power off'
        endif
      endif

      if(IAND(ama2_h_inststat1,mask1E00).ne.mask0200) then
        write(ama_cllgu,*)' Warning A2 status not scanning'
      endif

      if(IAND(ama2_h_inststat1,mask0016).ne.mask0006) then
        write(ama_cllgu,*)' Warning A2 status scan power off'
      endif
      if (ama2_h_statchrecnb.gt.0) then
        if(IAND(ama2_h_inststat2,mask1E00).ne.mask0200) then
          write(ama_cllgu,*)' Warning A2 2nd status: not scan mode'
        endif
        if(IAND(ama2_h_inststat2,mask0016).ne.mask0006) then
          write(ama_cllgu,*)' Warning A2 2nd status: scan power off'
        endif
      endif
!
!Deliver the order satellite number for the l1b satellite number and
!the julian date of the data
      ama_h_satid0 = ama_h_satid
      noanum = ord1bid(ama_h_satid0,ama_h_startdatajd)
      id_in  = noascid(noanum)
      switch_7_15=.false.
      !if ( noanum.eq.12 .and. ama_h_callocsclin.eq.0
      if ( id_in.eq.id_noaak .and. ama_h_callocsclin.eq.0
     &     .and. (ama_clflg.ne.0.or.ama_cflg.ne.0)) then
        switch_7_15=.true.
        call ml_wt('info','channels 7/15 are switched when reading')
        write(ama_cllgu,*)
     &       'Info:channels 7/15 are switched when reading' 
      endif

      do 100 scn=1,ama_h_scnlin
        call amadtr(ama_l1bu,scn,ios)
        if(ios.ne.0) then
         write(ama_cllgu,*)
     &   ' AMSU-A 1B record could not be read, ios:',ios
              call ml_wti('error',
     &         'AMSU-A 1B record could not be read ios',ios)
         call aappexit(10)
!         stop
        endif
!
        lstat(scn) = .true.
        nvstat(scn) = .false.                  ! init nav status flag
        scnlin(scn)     = ama_scnlin
        scnlinyr(scn)   = ama_scnlinyr
        scnlindy(scn)   = ama_scnlindy
        scnlintime(scn) = ama_scnlintime
        clockdrift(scn) = ama_clockdrift
        qualind(scn)    = ama_qualind
!If this is NESDIS 1b data, not generated with AAPP, ignore the initial qualind
        IF (ama_h_siteid.EQ.'NSS' .AND. 
     &      ama_h_modelid.NE.earth_modelid) THEN
          qualind(scn)    = 0
        ENDIF
        scnlinbit(scn)  = ama_scnlinbit

        attangtime(scn) = ama_attangtime
        rollang(scn)    = ama_rollang
        pitchang(scn)   = ama_pitchang
        yawang(scn)     = ama_yawang
        scalti(scn)     = ama_scalti
        navstat(scn)    = ama_navstat

        do 5 i = 1,15
          calok(i,scn) = .false.                 ! init cal status flag
          chqualflg(i,scn) = 0                   !was ama_chqualflg(i) (04/2013)
 5      continue
!
!...  switch discussion for calibration flags
!...  not applicable if chqualflg is initialised to 0 (08/14)
!
!        if(switch_7_15) then
!          chqualflg(7,scn)=ama_chqualflg(15)
!          chqualflg(15,scn)=ama_chqualflg(7)
!        endif
!
!...  end switch discussion for calibration flags
!
        scalti(scn)      = ama_scalti
        do 10 i = 1 , mxpix*3   !Solar zenith, sat zenith, local azimuth
          ang(i,scn) = ama_ang(i)
 10     continue
        do 15 i = 1 , mxpix*2
          earthl(i,scn) = ama_pos(i)
 15     continue
        do 30 i =  1 , mxpix
          do 20 j = 1, 17
            a1_earth_data(j,i,scn) =
     &        mod(ama1_earth_data(j,i)+65536,65536) 

 20       continue
!
!...      switch discussion for scene telemetry
!
          if(switch_7_15) then
            a1_earth_data(9,i,scn) =
     &          mod(ama1_earth_data(17,i)+65536,65536)
            a1_earth_data(17,i,scn) =
     &          mod(ama1_earth_data(9,i)+65536,65536)
          endif
!
!...      end switch discussion scene telemetry 
!
          do 25 j = 1, 4
            a2_earth_data(j,i,scn) = 
     &        mod(ama2_earth_data(j,i)+65536,65536)
 25       continue
! 
 30      continue
         a1_dhk(scn) = ama1_dhk(1)
         a2_dhk(scn) = ama2_dhk(1)
         a1_digb(scn) = mod(ama1_digb+65536,65536)
         a2_digb(scn) = mod(ama2_digb+65536,65536)
         do 35 i = 1 , 46
           a1_temps(i,scn) =  mod(ama1_temps(i)+65536,65536)
 35      continue
!
!...     switch discussion for temperature sensor telemetry
!
!!!         if(switch_7_15) then
!!!           a1_temps(11,scn) = jmod(ama1_temps(13)+65536,65536)
!!!           a1_temps(13,scn) = jmod(ama1_temps(11)+65536,65536)
!!!           a1_temps(21,scn) = jmod(ama1_temps(24)+65536,65536)
!!!           a1_temps(24,scn) = jmod(ama1_temps(21)+65536,65536)
!!!         endif
!          
!...     end switch discussion temperature sensor telemetry 
!
         do 40 i = 1 , 20
           a2_temps(i,scn) =  mod(ama2_temps(i)+65536,65536)
 40      continue
         do 50 i = 1 , 30
           a1_coldcal(i,scn) =  mod(ama1_coldcal(i)+65536,65536)
           a1_warmcal(i,scn) =  mod(ama1_warmcal(i)+65536,65536)
 50      continue
         do 51 i = 1 , 6
           a2_coldcal(i,scn) =  mod(ama2_coldcal(i)+65536,65536)
           a2_warmcal(i,scn) =  mod(ama2_warmcal(i)+65536,65536)
 51      continue
!
!...  switch discussion for cold and warm telemetry
!
         if(switch_7_15) then
           

           a1_coldcal(9,scn)  = mod(ama1_coldcal(17)+65536,65536)
           a1_coldcal(22,scn) = mod(ama1_coldcal(30)+65536,65536)
           a1_coldcal(17,scn) = mod(ama1_coldcal(9)+65536,65536)
           a1_coldcal(30,scn) = mod(ama1_coldcal(22)+65536,65536)
           
           a1_warmcal(9,scn)  = mod(ama1_warmcal(17)+65536,65536)
           a1_warmcal(22,scn) = mod(ama1_warmcal(30)+65536,65536)
           a1_warmcal(17,scn) = mod(ama1_warmcal(9)+65536,65536)
           a1_warmcal(30,scn) = mod(ama1_warmcal(22)+65536,65536)

         endif
!         
!...  end switch discussion for cold and warm telemetry
!
 100   continue

!
      if (iprint.ge.1) then
       write(ama_cllgu,fmt=504) scnlintime(1)
       write(ama_cllgu,fmt=505) scnlindy(1)
       write(ama_cllgu,fmt=506) scnlinyr(1)
       write(ama_cllgu,fmt=507) scnlintime(ama_h_scnlin)
       write(ama_cllgu,fmt=508) scnlindy(ama_h_scnlin)
       write(ama_cllgu,fmt=509) scnlinyr(ama_h_scnlin)
       write(ama_cllgu,fmt=501) ama_h_scnlin
      endif
!
!
       scname=noascnam(noanum)
       if (iprint.ge.1)write(ama_cllgu,fmt=500) ama_h_satid0
       if (iprint.ge.1)write(ama_cllgu,fmt=511) noanum
       if (iprint.ge.1)write(ama_cllgu,510) scname
!
      
!
! Open localisation files if requested
! i.e. according tor the options read in amaclin module
!
      if(ama_lflg.eq.1.or.ama_clflg.eq.1) then
! Opens and reads the SATPOS file
! -----------------------------------
        open (unit=satposu,status='old',iostat=ios)

!       If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
        if ( ios.ne.0 )
     &    open (unit=satposu,status='unknown',iostat=ios)

        if( ios.ne.0 ) then
          call ml_wti('error',
     &       'while opening the satpos file I/O status',ios)
          call ml_wt('error','amasetu aborted')
          call aappexit(2)
        endif

! Read the satpos file and close
! ------------------------------
        ijstrt=ama_h_startdatajd + 
     &      (ama_h_startdatatime-errclock*1000.)/86400.d3
        ijend=ama_h_enddatajd + 
     &       (ama_h_enddatatime-errclock*1000.)/86400.d3
        call sp_read(ijstrt,ijend,satposu,scname,ierr)
        close(unit=satposu,iostat=ios)
        
        if(ierr.ne.0) then
          call ml_wti('error','return code of sp_read differ from 0'
     &                ,ierr)
          call ml_wt('error','amasetu aborted')
          call ml_wt('error','navigation aborted')
          call ml_wt('error','but calibration can work')
          call ml_wt('error','try calibration alone')
          call aappexit(2)
        endif

        if(yaw.lt.-99.d-03) then
          call def_att(scname,att_mode,yaw,roll,pitch)
          call ml_wt('info','take default attitude values')
          call ml_wti('info','default attitude mode ',att_mode)
          call ml_wtdp('info','default attitude yaw ',yaw)
          call ml_wtdp('info','default attitude roll ',roll)
          call ml_wtdp('info','default attitude pitch ',pitch)
        else
          call ml_wtdp('info','attitude yaw ',yaw)
          call ml_wtdp('info','attitude roll ',roll)
          call ml_wtdp('info','attitude pitch ',pitch)
        endif
        call calatt(yaw,roll,pitch,att)

! If the file has already been located we do not correct
! again from the clock error ( this is known from the header
! epoyr test
! --------------------------
        if(ama_h_epoyr.le.1950) then
          open (unit=ckdriftu,status='old',iostat=ios)

!         If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
          if ( ios.ne.0 )
     &      open (unit=ckdriftu,status='unknown',iostat=ios)

          if(ios.ne.0) then
            call ml_wti('error',
     &       'opening clock error file I/O status',ios)
            call aappexit(2)
          endif
          ijstrt=ama_h_startdatajd + 
     &      (ama_h_startdatatime)/86400.d3
          call clkerr_get(ckdriftu,ijstrt,errclock,ierr)
          close(unit=ckdriftu,iostat=ios)
          if(ierr.ne.0.or.abs(errclock).gt.2.) then
            call ml_wti('warning',
     &       'reading clock error value from clkerr_get ',ierr)
            call ml_wtr('warning','clock error from file',
     &                 errclock)
            call ml_wt('warning','clock error forced to 0')
            errclock=0 
          else
            call ml_wtr('info','clock error value (sec)',errclock)
          endif
        else
          errclock=0.
          call ml_wt('info',
     &     '1B file already corrected from clock error')
        endif

        
! Init earth location parameters
! ------------------------------
        first_pnav = 1 ! first pixel to locate
        first_lnav = 1 ! first scan to process
        step_pnav = 1  ! step between pixels to locate
        step_lnav = 1  ! step between lines to locate
        nb_pnav = 30 ! 30 pixels for AMSU-A
      endif      
!
 500  format(1X,'NOAA 1B spacecraft identification code: ',I2)
 501  format(1X,' Number of AMSU-A scan lines in this dataset ',I4)
 504  format(1X,'start of data set time (msecs): ',I8)
 505  format(1X,'start of data set Julian day: ',I7)
 506  format(1X,'start of data set Year: ',I7)
 507  format(1X,'end of data set time (msecs): ',I8)
 508  format(1X,'end of data set Julian day: ',I7)
 509  format(1X,'end of data set Year: ',I7)
 510  format(1X,'NOAA satellite name: ',A6)
 511  format(1X,'NOAA satellite order number: ',I2)
      return
      end
