      subroutine amb_avg(iscan,ichan,sampok,itmean,spmean,itavg,spavg
     &,calok)
!+
!NOM/NAME $Id: amb_avg.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!     To average over consecutive scan lines to give mean internal target
!     and space view counts for the AMSU-A channels. A triangular weighting 
!     function is applied as recommended in the Mo & Saunders specification. 
!     If there are bad scan lines within the averaging window then increased
!     weight is given to the good scan lines. If there are bad scan lines or 
!     no good scan lines then the 1b quality flags are set.
!
!DESCRIPTION
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions
!
!PROPRIETAIRE/OWNER
!     Nigel Atkinson
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES)
!(version   date      comment                      <author>)
! 0.0                 AMSU-B                       R. Saunders
! 1.0      Jan 96                                  N. ATKINSON
! 1.1      March 03   f90 test                     T.Labrot
!      I.OR.J not correct and modified by IOR(I,J)
!      I and J must have the same type 
!       ==> created i2word48, i2word6 local variables 
! 1.2      July 03    Addition to log file         N Atkinson
!
!ARGUMENTS
!      iscan i*4 in scan line number
!      ichan i*4 in Channel number (1-5)
!      sampok(5,mxlin) L*1 in cal view counts ok flag
!      itmean(5,mxlin) r*4 in mean target counts for each channel
!      spmean(5,mxlin) r*4 in mean space view counts for each channel
!      itavg(5) r*4 out line averaged target counts for each channel
!      spavg(5) r*4 out line averaged space view counts for each channel
!      calok(5,mxlin) out calib ok flag
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################
!
!
!--DECLARATIONS----------------------------------------------------------------
      implicit none
!includes/commons
#include "ambclin.h"
#include "amb1b.h"
#include "ambscn.h"
#include "amb_cinit.h"
! local variables
      integer iscan,ichan,is,iscan2
      real itmean(5,mxlin),spmean(5,mxlin),anorm,bnorm
      real itavg(5),spavg(5),wt
      logical*1 calok(5,mxlin),sampok(5,mxlin)
      
      integer*2 i2word48,i2word6
      parameter (i2word48=48)
      parameter (i2word6=6)
!
!--END OF DECLARATIONS--------------------------------------------------------

      if (iprint.ge.1.and.iscan.eq.1.and.ichan.eq.1) then
        write(amb_cllgu,*) '*************************************'
        write(amb_cllgu,*) '**   SUBROUTINE AMB_AVG / AMB_CAL  **'
        write(amb_cllgu,*) '*************************************'
      endif
!
! Compute weighted mean values from valid counts. If invalid counts are present
! (e.g. at start and end of run) the valid counts are all given a higher 
! weighting.
!
      anorm = FLOAT(nscan2+1)
      spavg(ichan) = 0.0       ! zeroise arrays
      itavg(ichan) = 0.0
      bnorm = 0.0
      do 600 is = -nscan2 , nscan2
        iscan2 = iscan+is
        if(iscan2.gt.0.and.iscan2.le.amb_h_scnlin)then
          if(sampok(ichan,iscan2))then
            wt = (1.- float(abs(is))/anorm)/anorm
            spavg(ichan) = spavg(ichan) + wt*spmean(ichan,iscan2)
            itavg(ichan) = itavg(ichan) + wt*itmean(ichan,iscan2)
            bnorm=bnorm+wt
          endif
        endif
 600  continue
!
      if(bnorm.eq.0.)then        ! set no counts flags
        chqualflg(ichan,iscan)=IOR(chqualflg(ichan,iscan),i2word48) 
        calok(ichan,iscan) = .false.
        linqualflgs(iscan) = IOR(linqualflgs(iscan),2048) !Some uncal channels
      else
        itavg(ichan) = itavg(ichan)/bnorm
        spavg(ichan) = spavg(ichan)/bnorm
        calok(ichan,iscan) = .true.
        if(bnorm.lt.1.0)then   ! set some missing counts flags
          chqualflg(ichan,iscan)=IOR(chqualflg(ichan,iscan),i2word6)
          linqualflgs(iscan) = IOR(linqualflgs(iscan),16384)
        endif
      endif
      RETURN
      END





