      subroutine amb_cal(iscan,ichan,prtstat,calok,targ_temp,inst_temp,
     &    itavg,spavg,ispvw)
!+
!NOM/NAME $Id: amb_cal.F 313 2012-02-28 14:51:43Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module computes the AMSU-B calibration coefficients given
!     the mean space view and internal target counts and the internal target
!     temperature. it computes the gain (in units of counts/mW/sqm.ster.cm-1).
!     It then computes the primary calibration coeffs a0, a1 & a2 in units
!     of mW/(sqm.ster.cm-1), mW/(sqm.ster.cm-1)/count & 
!     mW/(sqm.ster.cm-1)/count^2 and then scaled by 1E6, 1E10 & 1E16 to
!     allow them to be represented as integers in the 1B file. 
!     The parameters determined from the thermal vac data supplied
!     in calibration parameter file: uo, u1 and u2 are used to compute
!     these coefficients as defined in the noaa/ukmo calibration spec.
!     see UK Met Office RSI working paper 66 (1994) for overall philosophy.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
!     Roger SAUNDERS
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Jan 96       no             R.Saunders
! 1.1      March 03  f90 test           T.Labrot
!       I.OR.J not correct and modified by IOR(I,J)
!       Changed  type  XXX/yyy/
!            to  type XXX
!                data XXX/yyy/              
! 1.2      July 03   Modify diagnostics     N Atkinson
! 1.3      Jan 06    Fix order of calok indices in diagnostic statement. NCA
! 1.4      Jan 2010  Set c2=0 if the quadratic coefficient overflows. NCA.
! 1.5      Jul 2011  Use "save" instead of common blocks. NCA.
!         
!ARGUMENTS iscan - in scan line number
!          ichan - in AMSU-B channel number
!          prtstat(mxlin) - in status of PRT temp data
!          calok(5,mxlin) - in status of scan line for calibration
!          targ_temp(mxlin) - in mean target temp for scan line from PRT data
!          inst_temp(mxlin) - in mean instrument temp for scan from PRT data
!          itavg(5) - in mean internal target counts for each channel
!          spavg(5) - in mean space view counts for each channel
!          ispvw(mxlin) - in space viewing direction
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!           
!            plank
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################

!--DECLARATIONS----------------------------------------------------------------
      implicit none                                                     
!
!
! includes/commons
#include "amb_cinit.h"
#include "ambclin.h"
#include "amb1b.h"
#include "ambscn.h"
!
! local variables
      integer*4 iscan,ichan,i,ispvw(mxlin)
      real itavg(5),spavg(5)       ! internal target & space view counts
      real targ_temp(mxlin),inst_temp(mxlin) ! int target temp & inst temp
      real c2,cc,cw,cwg,recg,sdc,recg2,tmp
      logical*1 prtstat(mxlin),calok(5,mxlin)  ! prt and cal status 
      logical*4 first
      real btemp,tsp(5),radsp,radbb,gain,rdiff,r32,r21,n32(5),n21(5)
      real u021(5),u121(5),u221(5),u032(5),u132(5),u232(5)
! variables to save
      save tsp,r32,r21,n32,n21,first
      save u021,u121,u221,u032,u132,u232
      data first/.true./
!
      gain = 0.0
      if(targ_temp(iscan).eq.0. .or. inst_temp(iscan).eq.0.)
     & calok(ichan,iscan)=.false.
      if (first)then
        r32 = reftemp(3)-reftemp(2)
        r21 = reftemp(2)-reftemp(1)
        do 10 i = 1 , 5
         tsp(i) = tspace + spcorr(i,ispvw(iscan)+1)  ! apply antenna corrn
         n21(i) = (nonlin(i,2)-nonlin(i,1))/r21      ! to rad temp space
         n32(i) = (nonlin(i,3)-nonlin(i,2))/r32   
         u021(i) = (u0(i,2)-u0(i,1))/r21        
         u121(i) = (u1(i,2)-u1(i,1))/r21        
         u221(i) = (u2(i,2)-u2(i,1))/r21        
         u032(i) = (u0(i,3)-u0(i,2))/r32        
         u132(i) = (u1(i,3)-u1(i,2))/r32        
         u232(i) = (u2(i,3)-u2(i,2))/r32        
 10     continue                                     
        first = .false.
      endif
!
C.. Correct target temp to reduce warm biases for channel (ignore inst temp)
C..  this correction is zero for AMSU-B except for FM3 when it is small
C.. for ch 20 and zero for other channels) 
      if(calok(ichan,iscan))then
       btemp = targ_temp(iscan) + intcorr(ichan,1)
C.. convert physical temp to effective temp for AMSU-B channel
       btemp = A(ichan)*btemp + B(ichan)
!.. convert  temps to radiance in mW/cm-1/ster/sqm
       call plank(freq(ichan),btemp,radbb)
       call plank(freq(ichan),tsp(ichan),radsp)
!...Compute gain                         
       rdiff = radbb-radsp
       if(rdiff.gt.0.00002)then
        gain = (itavg(ichan)-spavg(ichan))/rdiff
       else
        gain = 0.0
       endif
!...compute non-linear coefficient for instrument temp by interpolation
       if ( inst_temp(iscan) .lt. reftemp(2))then
       c2 = nonlin(ichan,1)+(inst_temp(iscan)-reftemp(1))*
     &  n21(ichan)
       else
       c2 = nonlin(ichan,2)+(inst_temp(iscan)-reftemp(2))*
     &  n32(ichan)
      endif
      endif
!
!...Compute calibration coefficients and scale for 1B
!
      i = 1 + (ichan-1)*3
      if (gain.gt.0.0.and.calok(ichan,iscan))then
        cw = itavg(ichan)
        cc = spavg(ichan)
        cwg = cw/gain
        recg = 1./gain
        recg2 = recg*recg
        tmp = c2*recg2*1E16
        if(abs(tmp).le.2.147483647E9)then
          pmcal(i,iscan) = nint(tmp)
        else
          pmcal(i,iscan) = 0
          c2 = 0.0       !Try a linear calibration
        endif
        tmp = (recg - c2*(cc+cw)*recg2)*1E10
        if(abs(tmp).le.2.147483647E9)then
          pmcal(i+1,iscan) = nint(tmp)
        else
          pmcal(i+1,iscan) = 0
        endif
        tmp = (radbb-cwg + c2*cw*cc*recg2)*1E6
        if(abs(tmp).le.2.147483647E9)then
          pmcal(i+2,iscan) = nint(tmp)
        else
          pmcal(i+2,iscan) = 0
        endif

!v1.2 remove this diagnostic
!        if (iprint.eq.2)then
!          write(amb_cllgu,*)iscan,ichan,gain,pmcal(i,iscan),
!     &      pmcal(i+1,iscan),pmcal(i+2,iscan) 
!        endif

      else

!v1.2 add this diagnostic
        if (gain.lt.0.and.iprint.ge.1)
     &       write(amb_cllgu,*) 'scan',iscan,' chan',ichan,
     &         'no cal: gain=',gain
        if (.not.calok(ichan,iscan).and.iprint.ge.1)
     &       write(amb_cllgu,*) 'scan',iscan,' chan',ichan,' no cal'
        pmcal(i,iscan) = 0
        pmcal(i+1,iscan) = 0
        pmcal(i+2,iscan) = 0
        linqualflgs(iscan) = IOR(linqualflgs(iscan),2048)
      endif
!
!...Put secondary coefficients in 1B common also interpolate for inst temp
!
       if ( inst_temp(iscan) .lt. reftemp(2))then
       sdc = u2(ichan,1)+(inst_temp(iscan)-reftemp(1))*
     1 u221(ichan) 
       sdcal(i,iscan) = nint(sdc)
       sdc = u1(ichan,1)+(inst_temp(iscan)-reftemp(1))* 
     1 u121(ichan)
       sdcal(i+1,iscan) = nint(sdc)
       sdc = u0(ichan,1)+(inst_temp(iscan)-reftemp(1)) * 
     1 u021(ichan)
       sdcal(i+2,iscan) = nint(sdc)
       else
       sdc = u2(ichan,2)+(inst_temp(iscan)-reftemp(2)) * 
     1 u232(ichan)
       sdcal(i,iscan) = nint(sdc)
       sdc = u1(ichan,2)+(inst_temp(iscan)-reftemp(2)) * 
     1 u132(ichan)
       sdcal(i+1,iscan) = nint(sdc)
       sdc = u0(ichan,2)+(inst_temp(iscan)-reftemp(2)) * 
     1 u032(ichan)
       sdcal(i+2,iscan) = nint(sdc)
      endif
!
      RETURN
      END

