!+
!NOM/NAME $Id: amb_calcorrect.F 25 2006-02-03 15:43:53Z frna $
!
!ROUTINE    AMB_CALCORRECT
!
!PURPOSE    Correct AMSU-B space and target counts for bias errors. Use 
!           transmitter powers from amb_ahk for the scan line being read. Use
!           space and target view corrections from 1B header. Assumes call to
!           'AMB_TESTNEWBIAS' has already been made.
!
!VERSION    1.1      14/12/98     N C Atkinson, UKMO
!
!ARGUMENTS  
!   Integer*4    ColdCal(24),WarmCal(24)  from instrument telemetry
!   Integer*2    BiasFlag    =1 if correction for anomalous bias is required
!##############################################################################

      SUBROUTINE AMB_CALCORRECT(ColdCal,WarmCal,BiasFlag)


#include "amb1b.h"
      INTEGER*4 ColdCal(24),WarmCal(24)
      INTEGER*2 BiasFlag
      INTEGER*2 CalCorr(5,2)           !5 channels, 2 views (space/warm)
      INTEGER Chan,Sample,I,Tx
      REAL F
!
! Initialise
!
      DO Chan=1,5
        CalCorr(Chan,1) = 0
        CalCorr(Chan,2) = 0
      ENDDO
!
! Calculate current corrections
!
      DO Tx=1,4
        IF (Tx.LT.4) THEN
          F = amb_ahk(22+Tx)/(0.1*amb_h_txpow(Tx))
        ELSE                       !Use sum of SARR-A and SARR-B powers
          F = (amb_ahk(22+Tx)+amb_ahk(23+Tx))/(0.1*amb_h_txpow(Tx))
        ENDIF
        IF (F.GT.0.01) THEN
          DO Chan=1,5
            CalCorr(Chan,1) = CalCorr(Chan,1) + 
     &                  amb_h_CountsCorr(Chan,20,Tx)*F
            CalCorr(Chan,2) = CalCorr(Chan,2) + 
     &                  amb_h_CountsCorr(Chan,21,Tx)*F
          ENDDO
        ENDIF
      ENDDO
!
! Add anomalous bias correction if necessary
!
      IF (BiasFlag.EQ.1) THEN
        I = mod(amb_scnlin-1,3)+1     !1, 2 or 3
        DO Chan=1,5
          CalCorr(Chan,1) = CalCorr(Chan,1) + amb_h_NewCorr(Chan,32,I)
          CalCorr(Chan,2) = CalCorr(Chan,2) + amb_h_NewCorr(Chan,33,I)
        ENDDO
      ENDIF
!
! Apply current corrections
!
      DO Chan=1,5
        Do Sample=1,4
          I = (Sample-1)*6 + Chan + 1
          ColdCal(I) = ColdCal(I) + CalCorr(Chan,1)
          WarmCal(I) = WarmCal(I) + CalCorr(Chan,2)
        ENDDO
      ENDDO
      RETURN
      END

!##############################################################################
