        subroutine amb_initcl(noanum)
!+
!NOM/NAME $Id: amb_initcl.F 25 2006-02-03 15:43:53Z frna $
!
!---------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module opens and reads the calibration coefficients file. 
!     Data are ranged in common amb_clcoef (include amb_cinit.h).
!     This module also reads the test coefficients and puts data in 
!     common amb_tstcf (include amb_cinit.h).
!
!DESCRIPTION 
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER  
!     Roger SAUNDERS
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      August 95     no             R.SAUNDERS
! 1.2      October00   updated line with 110 to run with NOAA16 data (TL)
! 1.3      March 04    Modify for linux  N Atkinson / A Korpela
! 1.4      April 04    Add moon parameters (optional)  N C Atkinson
! 1.5      Nov 04      Remove obsolete calparid.  N C Atkinson
!         
!ARGUMENTS 
!      integer  in  noanum    ! satellite number (12-14)
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
! None
! 
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################
!
!--DECLARATIONS-------------------------------------------------------------
      implicit none                                                      
     
! argument variables 
      integer noanum
! local variables
      real tab0(4)                ! intermediate array
      integer i,j                 !index for loops
      integer versnb              !number of version of the coef file
      integer versyr              !year of the version of the coef file
      integer versday             !day of the version of the coef file (0/365)
      integer ios                 !error status
      character*80 charact        !buffer storing one line of the file
!      character*2 calparid        ! character variable of version number
! includes/commons
#include "amb_cinit.h"
#include "ambclin.h"
#include "ambscn.h"
#include "amb1b.h"
!
!--END OF DECLARATIONS---------------------------------------------------------

      if (iprint.ge.1) write(amb_cllgu,*)
      if (iprint.ge.1) write(amb_cllgu,*) '***************************'
      if (iprint.ge.1) write(amb_cllgu,*) '**  SUBROUTINE AMB_INITCL**'
      if (iprint.ge.1) write(amb_cllgu,*) '***************************'
!
!Open and read calibration coefficients file
       open(amb_coefu,access='sequential',status='old',iostat=ios)

!      If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
       if (ios.ne.0)
     & open(amb_coefu,err=900,access='sequential',status='unknown')
!
!Skips the header lines
       read(amb_coefu,'(a80)',err=901) charact
       do while (charact(1:1) .eq. '#')
        read(amb_coefu,'(a80)',err=901) charact
       enddo
       backspace amb_coefu

       read(amb_coefu,*,err=901)versnb
       read(amb_coefu,*,err=901)versyr
       read(amb_coefu,*,err=901)versday
       amb_h_cpidsupdyr = versyr
       amb_h_cpidsupddy = versday
!       write(calparid,'(a2)')versnb
!       amb_h_calparid = calparid
! 
       if (iprint.ge.1) then
        write(amb_cllgu,*)
        write(amb_cllgu,*) 'calibration coefficients file' 
        write(amb_cllgu,520)versnb,versyr,versday
       endif
        
! Read fundamental constants
       read(amb_coefu,'(a80)',err=901) charact
       do while (charact(4:9) .ne. 'Speed ')
        read(amb_coefu,'(a15)',err=901) charact
       enddo
       read(amb_coefu,*,err=901) c
       read(amb_coefu,'(a80)',err=901) charact
       do while (charact(4:9) .ne. 'First ')
        read(amb_coefu,'(a15)',err=901) charact
       enddo
       read(amb_coefu,*,err=901) plck(1),plck(2)
       do while (charact(4:9) .ne. 'Bright')
        read(amb_coefu,'(a15)',err=901) charact
       enddo
       read(amb_coefu,*,err=901) tspace
!
! Get the data for the correct instrument 
!
  10   read(amb_coefu,'(a80)',err=901) charact
       do while (charact(4:5) .ne. 'ID')
        read(amb_coefu,'(a15)',err=901) charact
       enddo
       read(amb_coefu,*,err=901) modid
       if (modid.eq.99)then
         write(amb_cllgu,*) 'instrument id. not matched no calibration'
         write(amb_cllgu,*) 'possible '
        noanum=0 
        return
       endif
       if (modid.ne.amb_h_instid)go to 10
       read(amb_coefu,*,err=901)
! get center frquencies
       do i = 1 , 5
        read(amb_coefu,*,err=901)freq(i)
       enddo
! skip 3 records
       do i = 1 , 3
        read(amb_coefu,*,err=901)
       enddo 
! get band correction coeffs
       do i = 1 , 5
       read(amb_coefu,*,err=901)b(i),a(i)
       enddo
!
!       read(amb_coefu,*,err=901)          ! NB now a parameter
!       read(amb_coefu,*,err=901)nscan     ! no. of scan lines to avg over
       nscan2 = nscan/2  
!                                          
       read(amb_coefu,*,err=901)
       read(amb_coefu,*,err=901)nview     ! no. of cal views  
!
       read(amb_coefu,*,err=901)
       read(amb_coefu,*,err=901)spview    ! space viewing direction   
!
       read(amb_coefu,*,err=901)
       read(amb_coefu,*,err=901)ant_coef  ! conversion to antenna position
!
       read(amb_coefu,*,err=901)
       read(amb_coefu,*,err=901)pos_errc,pos_erre ! max error in antenna
!                                                    position   
! IWT PRT count to temperature conversion coefficients (7 PRTs,4coefs)
       read(amb_coefu,*,err=901)
       do i=1,7
        read(amb_coefu,*,err=901)tab0
        do j=1,4
          prtcnttmp(j,i) = tab0(j)
        enddo
       enddo
!
! Weight coefficients for each PRT
       read(amb_coefu,*,err=901)
       read(amb_coefu,*,err=901)prtwght

! Reasonable PRT temp limits  (min,max)
       read(amb_coefu,*,err=901)
       read(amb_coefu,*,err=901)prtmin,prtmax
! Max PRT temp change allowed before rejecting
       read(amb_coefu,*,err=901)
       read(amb_coefu,*,err=901)prtdt
! Minimum number of PRT readings acceptable
       read(amb_coefu,*,err=901)
       read(amb_coefu,*,err=901)nprtmin
! Number of lines to fill in PRT data
       read(amb_coefu,*,err=901)  
       read(amb_coefu,*,err=901)nprtgap
! 3 instrument reference temps
       read(amb_coefu,*,err=901)
       read(amb_coefu,*,err=901)reftemp
! Instrument temp PRT count to temp conv coeffs
       read(amb_coefu,*,err=901)
       read(amb_coefu,*,err=901)prtcnttmpi
! Internal target correction factors
       read(amb_coefu,*,err=901)
       do i = 1 ,3
       read(amb_coefu,*,err=901)(intcorr(j,i),j=1,5)
       enddo
! Cold space correction factors
       read(amb_coefu,*,err=901)
       do i = 1 , 4
       read(amb_coefu,*,err=901)(spcorr(j,i),j=1,5)
       enddo
!
! Reasonable count limits by channel (min,max) for internal target
       read(amb_coefu,*,err=901)
       read(amb_coefu,*,err=901)intcntmin
       read(amb_coefu,*,err=901)intcntmax
! Reasonable count limits by channel (min,max) for space view
       read(amb_coefu,*,err=901)
       read(amb_coefu,*,err=901)spcntmin
       read(amb_coefu,*,err=901)spcntmax
! max change in mean counts from previous scan allowed before rejecting
       read(amb_coefu,*,err=901)
       read(amb_coefu,*,err=901)dcount
! max number of lines before resetting interline check last value
       read(amb_coefu,*,err=901)
       read(amb_coefu,*,err=901)ngap
! non linearity correction terms
       read(amb_coefu,*,err=901)
       do i = 1, 3
       read(amb_coefu,*,err=901)(nonlin(j,i),j=1,5)
       enddo
! nominal antenna positions for cal views
       read(amb_coefu,*,err=901)
       do i = 1 , 4
       read(amb_coefu,*,err=901)(scanpos(j,i),j=1,8)
       enddo
! read digital A coefs
       read(amb_coefu,*,err=901)
       do i = 1 , 24
       read(amb_coefu,*,err=901)(diga_coef(j,i),j=1,4)
       enddo      
! read analog telemetry coefs
       read(amb_coefu,*,err=901)
       do i = 1 , 18
       read(amb_coefu,*,err=901)(anal_coef(j,i),j=1,4)
       enddo
! read moon parameters if available
       read(amb_coefu,'(a80)',err=901) charact
       if (charact(1:6) .eq. '# Moon') then
         read(amb_coefu,*,err=901)moonthresh,moon_azcorr,moon_elevcorr
       else
         moonthresh = 1.5
         moon_azcorr = 0.0     !Defaults
         moon_elevcorr = 0.0
       endif
            
       close(amb_coefu,err=902)
       if(iprint.ge.1)write(amb_cllgu,*)'Calibration parameters read ok'
!
! Open and read secondary calibration coefficients file
       open(amb_tstcoefu,access='sequential',status='old',iostat=ios)
       if (ios.ne.0)
     &  open(amb_tstcoefu,err=900,access='sequential',status='unknown')

! Skips the header lines
       read(amb_tstcoefu,'(a80)',err=901) charact
       do while (charact(1:1) .eq. '#')
        read(amb_tstcoefu,'(a80)',err=901) charact
      enddo
       backspace amb_tstcoefu

       read(amb_tstcoefu,*,err=901)versnb
       read(amb_tstcoefu,*,err=901)versyr
       read(amb_tstcoefu,*,err=901)versday
 
       if (iprint.ge.1) then
        write(amb_cllgu,*) 'test coefficients file' 
        write(amb_cllgu,520)versnb,versyr,versday
       endif
 110   read(amb_tstcoefu,'(a15)',err=901) charact   
       do while (charact(4:5) .ne. 'ID')
        read(amb_tstcoefu,'(a15)',err=901) charact
       enddo
       read(amb_tstcoefu,*,err=901) modid
       if (modid.eq.99)then
         write(amb_cllgu,*)' instrument id. not matched no secondary'
         write(amb_cllgu,*)' calibration coeffs available '
        return
       endif
       if (modid.ne.amb_h_instid)go to 110
       read(amb_tstcoefu,*,err=901)
       do i = 1 , 3
       read(amb_tstcoefu,'(a80)',err=901) 
       read(amb_tstcoefu,'(a80)',err=901) 
       read(amb_tstcoefu,*,err=901)(u0(j,i),j=1,5)
       read(amb_tstcoefu,'(a80)',err=901) 
       read(amb_tstcoefu,*,err=901)(u1(j,i),j=1,5)
       read(amb_tstcoefu,'(a80)',err=901) 
       read(amb_tstcoefu,*,err=901)(u2(j,i),j=1,5)
       enddo
!
       close(amb_tstcoefu,err=902)
       if(iprint.ge.1)write(amb_cllgu,*)' Secondary cal coeffs read ok'
!       
! scale secondary coeffs so they can be stored in 1B
!
      do 120 i = 1 , 3
      do 120 j = 1 , 5
        u0(j,i) = u0(j,i)*1E6
        u1(j,i) = u1(j,i)*1E10
        u2(j,i) = u2(j,i)*1E16
 120  continue
      go to 200
!
!
 520  format(2X,' version number: ',I3/'   version year: ',I4/
     &       '   version day of year: ',I3)
 540  format(3(3X,I3),3(6X,I3))

 900  write(amb_cllgu,*)'Error opening cal coefficients file'
      stop
 901  write(amb_cllgu,*)'Error reading cal coefficients file'
      stop
 902  write(amb_cllgu,*)'Error closing cal coefficients file'
      stop
!
 200  return
      end
