      subroutine amb_iwttmp(lstat,prtstat,targ_temp,inst_temp)
!+
!NOM/NAME $Id: amb_iwttmp.F 46 2006-08-18 14:39:27Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!     This module extracts the AMSU-B target and instrument PRT data
!     necessary for the calibration and performs quality control checks
!     and sets flags for bad PRT data. The target temperature data
!     extracted from good lines is the averaged internal target PRT
!     temperature (there are 7 PRTs in the AMSU-B target)
!     and instrument temperature which for AMSU-B is the 18/19/20 mixer
!     temperature for each scan line. If there is no good PRT data prtstat
!     is set to false for that scan line.
!
!DESCRIPTION
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions
!
!PROPRIETAIRE/OWNER
!     Roger SAUNDERS
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES)
!(version   date      comment                                 <author>)
! 1.0     December 95     no                                   R.Saunders
! 1.1     May 96      Allowance made for non unit PRT weights  R.Saunders
! 1.2     March 03    f90 test            T.Labrot
!           I.AND.J not correct and modified by IAND(I,J)
!             I.OR.J not correct and modified by IOR(I,J)
!             I and J must have the same type
!             ==> created local variables : i2word1,i2word8
!           if (prtstat(j,iscan).eq..true.) then   
!                    modified by if (prtstat(j,iscan)) then
! 1.3      17/01/2006 : Philippe Marguinaud : change hex constants from
!                       '1234'X to Z'1234'
! 1.4      17/08/2006 use i4word00002000 and i4word00001000. NCA
!
!ARGUMENTS
!      lstat(mxlin)   L*1 in  If false don't use scan line
!      prtstat(mxlin) L*1 out  If .TRUE. then PRT data ok
!      targ_temp(mxlin) r*4 out avg internal target temp deg K
!      inst_temp(mxlin) r*4 out instrument temperature deg K
!
! EXTERNALS
! prtchk
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################

!--DECLARATIONS---------------------------------------------------------------
      implicit none
!
! includes/commons
#include "ambclin.h"
#include "amb1b.h"
#include "ambscn.h"
#include "amb_cinit.h"
!
! local variables
      real*4 inst_temp(mxlin),targ_temp(mxlin),prt(7),prtavg
      integer*4 iscan,iprt,iprt1,nprt,ichan,ngoodweight,iold,i
      real*4 t,d,difft,diffi
      logical*1 lstat(mxlin),prtstat(mxlin),validfound
      
      integer*2 i2word1,i2word8
      parameter (i2word1=1)
      parameter (i2word8=8)

      integer*4 i4word00002000, i4word00001000
      parameter (i4word00002000=Z'00002000')
      parameter (i4word00001000=Z'00001000')
!
!--END OF DECLARATIONS--------------------------------------------------------

      if (iprint.ge.1) write(amb_cllgu,*)
      if (iprint.ge.1) write(amb_cllgu,*) '***************************'
      if (iprint.ge.1) write(amb_cllgu,*) '**   SUBROUTINE AMB_IWTTMP*'
      if (iprint.ge.1) write(amb_cllgu,*) '***************************'
!
!..Loop in scan line................................................
      do 100 iscan = 1 , amb_h_scnlin
        if(lstat(iscan))then
! ...get instrument temperature....
          t = float(temps(3,iscan))
          inst_temp(iscan) = prtcnttmpi(1) + prtcnttmpi(2)*t +
     @    prtcnttmpi(3)*t*t + prtcnttmpi(4)*t*t*t
! ...get target temperatures....
          do 50 iprt = 1 , 7
           iprt1 = iprt + 8
           t = float(temps(iprt1,iscan))
           prt(iprt) = prtcnttmp(1,iprt) + prtcnttmp(2,iprt)*t
     @  +  prtcnttmp(3,iprt)*t*t + prtcnttmp(4,iprt)*t*t*t
  50      continue
!
! ...carry out a check of PRT consistency on scan line and compute mean
          call prtchk(7,prt,prtwght,prtmax,prtmin,prtdt,prtavg,nprt,
     @     ngoodweight)
! ...compute avg prt temp for target
          if ( nprt .ge. nprtmin)then
            targ_temp(iscan) = prtavg
            prtstat(iscan) = .true.
            if (nprt .lt. ngoodweight)then
              do ichan=1,5
                chqualflg(ichan,iscan) = 
     &                   IOR(chqualflg(ichan,iscan),i2word1)
              enddo               !Some bad PRTs
            endif
          else
            targ_temp(iscan) = 0.0
            prtstat(iscan) = .false.
!...set flags after interline test
          endif
        else              !if lstat false (i.e. problem with instrument status)
          targ_temp(iscan) = 0.0
          inst_temp(iscan) = 0.0
          prtstat(iscan) = .false.
        endif
!
 100  Continue   ! ......end of scan line loop
!
! Now rescan through PRT temps to do interline test and fill in gaps
! by allowing up to nprtgap scans to have the last good PRT value or the next
! good PRT value. The difference between successive 'good' values must be
! less than the 'prtdt' value from the calibration data file.
! Logic is similar to the interline test on Space/Internal target counts,
! except that here we correct any bad values.
!
      d=prtdt
      validfound=.false.
      iold = -999
      do 400 iscan = 1 , amb_h_scnlin
        if (.not.validfound) then   !Start of sequence - no good data yet
          if (prtstat(iscan)) then
            if ((iscan-iold).gt.nprtgap) then
              if (iold.gt.0) then
!                write(amb_cllgu,*)'scan ',iold,': Data gap'
                prtstat(iold)=.false.        !Data gap: old point is invalid
                targ_temp(iold)=0.0
              endif
            else
              if(abs(targ_temp(iold)-targ_temp(iscan)).le.d.and.
     &           abs(inst_temp(iold)-inst_temp(iscan)).le.d)then
                validfound=.true.    !Diff good - both points accepted

! See whether there are any prior points which can be assigned this temperature
                do 200 i=iold-1,iold-nprtgap-1,-1
                  if (i.lt.1) goto 210
                  if (prtstat(i)) goto 210   !Quit loop at first good point
! Replace faulty target or instrument temps with old value
                  difft = abs(targ_temp(i)-targ_temp(iold))
                  diffi = abs(inst_temp(i)-inst_temp(iold))
                  if(difft.gt.d) targ_temp(i)=targ_temp(iold)
                  if(diffi.gt.d) inst_temp(i)=inst_temp(iold)
                  if (iprint.ge.1)
     &             write(amb_cllgu,*)'Scan ',i,
     &              ': PRT substituted by later value'
  200           continue
  210           continue

              else
                prtstat(iold)=.false.  !Diff bad - reject old point
                targ_temp(iold)=0.0
              endif
            endif
            iold=iscan
          endif
        else                         !Valid data already present
          if ((iscan-iold).gt.nprtgap) then
            validfound=.false.   !Too long ago - start again
!            if (prtstat(iscan).eq..true.) then
            if (prtstat(iscan)) then
              iold=iscan
            else
              iold=-999
            endif
          else                         !Test for consistency
            difft = abs(targ_temp(iold)-targ_temp(iscan))
            diffi = abs(inst_temp(iold)-inst_temp(iscan))
            if (difft.le.d.and.diffi.le.d) then
              iold=iscan              !New point valid - update iold
            else
! Replace faulty target or instrument temps with old value, but set prtstat 
! false so that final statistics show the bad data
              if(difft.gt.d) targ_temp(iscan)=targ_temp(iold)
              if(diffi.gt.d) inst_temp(iscan)=inst_temp(iold)
              prtstat(iscan)=.false.
              if (iprint.ge.1)
     &           write(amb_cllgu,*)'Scan ',iscan,
     &              ': PRT substituted by prior value'
            endif
          endif
        endif
 400  continue  ! end of scan line loop---------------------
      if (.not.validfound.and.iold.gt.0) then 
        prtstat(iold)=.false.      !Set last point
        targ_temp(iold)=0.0
      endif
!
! Set flags
!
      do 450 iscan=1,amb_h_scnlin
        if (targ_temp(iscan).eq.0.0) then
          linqualflgs(iscan)=IOR(linqualflgs(iscan),i4word00002000)  !Bad PRT data
          do ichan=1,5
            chqualflg(ichan,iscan)=IOR(chqualflg(ichan,iscan),i2word8)
          enddo
        else
          if (.not.prtstat(iscan)) then
            linqualflgs(iscan)=IOR(linqualflgs(iscan),i4word00001000)  !Marginal PRT
          do ichan=1,5
            chqualflg(ichan,iscan)=IOR(chqualflg(ichan,iscan),i2word8)
          enddo
          endif
        endif
 450  continue

      if(iprint.gt.1)then
        do 500 iscan = 1 , amb_h_scnlin
           write(amb_cllgu,*)'Scan',iscan,
     &   ' Target prt temp=',targ_temp(iscan)
!           write(amb_cllgu,*)'Scan',iscan,
!     &   ' Target prt temp=',targ_temp(iscan),prtstat(iscan)
500    continue
      endif
!
      return
      end

