        subroutine amb_loc
!+
!NOM/NAME $Id: amb_loc.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module performs eath location for AMSU-B.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      31 May 96       no              Marcel Derrien
! 1.1      May 98     Update 1B definition  N Atkinson 
! 1.1      May 98     modify  nav_1blin arguments Marcel Derrien      
! 1.2      Dec 00        change message for clock error (PB)
! 1.3      March 03   f90 test          Tiphaine Labrot
!                     shift the comment lines at the end of #include 
!                     directive lines
!
!ARGUMENTS 
! None
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
! xiqj, gp_gb16, xijq, gp_sb16, nav_1blin, xdqj
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none                                                     

 
! includes/commons
! amsu-b 1b header and record description
#include "amb1b.h" 
! amsu-b common arrays for 1b record variables whole pass
!                      (scan indexation)
#include "ambscn.h" 
! navigation common 
#include "c_1bnav.h" 
! general common
#include "ambclin.h" 

! local variables
       integer scn
       integer iqscn(3)
       integer jdate
       double precision ijscn ! scan julian time 
       double precision ijstrt ! julian start time 
       logical clock_cor_ind ! .true. if clock error has to be applied
!

!functions
       integer*2 gp_gb16 ! from libf7gp
!-
      if (iprint.ge.1) write(amb_cllgu,*)
      if (iprint.ge.1) write(amb_cllgu,*) '***************************'
      if (iprint.ge.1) write(amb_cllgu,*) '**   SUBROUTINE AMB_LOC  **'
      if (iprint.ge.1) write(amb_cllgu,*) '***************************'

       iqscn(1)=scnlinyr(1)
       iqscn(2)=scnlindy(1)
       iqscn(3)=scnlintime(1)
       call xiqj(iqscn,ijstrt) ! converts julian start time
!
!      check if clock error not already applied 
!      if not we must correct
!      the current scan line times when computing earth location
!      ---------------------------------------------------------
       if(gp_gb16(scnlinbit(1),14).eq.0) then
         ijstrt=ijstrt-errclock/86400. ! apply clock error to the start time
         clock_cor_ind=.true.
         call ml_wt('info','clock error correction is valid')
         call ml_wt('info','datation will be corrected in this run')
       else
         clock_cor_ind=.false.
         call ml_wt('info','times are already corrected')
       endif


           do scn = 1,amb_h_scnlin

           if(clock_cor_ind) then
!          -------------------------------
!          clock error correction to apply
!          and flag bit 14 in scnlinbit
!          -------------------------------
             iqscn(1)=scnlinyr(scn)
             iqscn(2)=scnlindy(scn)
             iqscn(3)=scnlintime(scn)
             call xiqj(iqscn,ijscn)
             ijscn=ijscn-errclock/86400. ! apply clock error to the scan
             call xijq(ijscn,iqscn)
             scnlinyr(scn) = iqscn(1) ! feed back the correction to 1b common
             scnlindy(scn) = iqscn(2)
             scnlintime(scn) = iqscn(3)
             call gp_sb16(scnlinbit(scn),14,1)
             clockdrift(scn)=1000.*errclock ! feed back the correction to 1b common
          endif
!
!         compute navigation for AMSU-B ( data type code = 11 )
!         the scans are navigated according to the first scan time
!         because we are sure to get no missing records in the file
!         the earth location is computed according to the scanning geometry
!         arguments of the  nav_1blin module directly address the 
!         ambscn.h variables
          call nav_1blin(ijstrt,11, scn,
     &     scnlinbit, qualind,
     &     attangtime, rollang , pitchang ,
     &     yawang ,  navstat , scalti ,
     &     ang(1,scn) ,earthl(1,scn))
        enddo
!      -------------------------------------------------------
!      update header times according to the clock_cor_ind flag
!      -------------------------------------------------------
       if(clock_cor_ind) then
         iqscn(1)=scnlinyr(1)
         iqscn(2)=scnlindy(1)
         call xdqj(iqscn,jdate)
         amb_h_startdatajd = jdate 
         amb_h_startdatayr = scnlinyr(1)
         amb_h_startdatady = scnlindy(1)
         amb_h_startdatatime = scnlintime(1)
         iqscn(1)=scnlinyr(amb_h_scnlin)
         iqscn(2)=scnlindy(amb_h_scnlin)
         call xdqj(iqscn,jdate)
         amb_h_enddatajd = jdate 
             amb_h_enddatayr = scnlinyr(amb_h_scnlin)
             amb_h_enddatady = scnlindy(amb_h_scnlin)
         amb_h_enddatatime = scnlintime(amb_h_scnlin)
       endif

       return
       end

