!+
!ROUTINE   AMB_READCORR
!
!PURPOSE   Read AMSU-B bias correction tables from external file and store
!          in 1B header.
!
!VERSION   1.0   18/12/98   N C Atkinson, UKMO
!          1.1   09/03/04   Modify for linux (N Atkinson / A Korpela)
!
!ARGUMENTS None
!
!##############################################################################

      SUBROUTINE AMB_READCORR

      implicit none


#include "ambclin.h"
#include "amb1b.h"
#include "ambnewbias.h"

      INTEGER modid_corr
      INTEGER*2 dummy(6),I,Scan,Pix,Tx
      INTEGER ios
      REAL*4 TxPow
      CHARACTER*80 charact

      OPEN(amb_corcoefu,form='formatted',access='sequential',
     &  status='old',iostat=ios)

!     If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
      if (ios.ne.0)
     &  OPEN(amb_corcoefu,form='formatted',access='sequential',
     &  status='unknown',iostat=ios)
!
! Get the data for the correct instrument
!
  10  read(amb_corcoefu,'(a80)',err=901) charact
      do while (charact(2:3) .ne. 'ID')
        read(amb_corcoefu,'(a15)',err=901,end=902) charact
      enddo

      read(charact(5:10),*) modid_corr

      if (modid_corr.ne.amb_h_instid)go to 10
!
! Read the correction values for each channel, pixel and transmitter
!
      DO Tx=1,4
        read(amb_corcoefu,'(A80)',err=901) charact !Transmitter string (ignore)
        DO Pix=1,21
          read(amb_corcoefu,*,err=901) (Dummy(I),I=1,6)
          DO I=1,5
            amb_h_CountsCorr(I,Pix,Tx) = Dummy(I+1)
          ENDDO
        ENDDO
        read(amb_corcoefu,*,err=901) TxPow
        amb_h_TxPow(Tx) = TxPow*10
      ENDDO
!
! See if an anomalous bias correction is provided. If so, store correction
! table in 1B header and detection algorithm parameters in common block.
! Table has 31 earth views and 2 calibration views.
!
      do while (charact(2:5) .ne. 'Anom')
        read(amb_corcoefu,'(a15)',err=901,end=903) charact
        if (charact(2:3).eq.'ID') go to 903  !Terminate if new instrument found
      enddo

      DO Scan=1,3
        read(amb_corcoefu,'(A80)',err=901) charact  !Scan string (ignore)
        DO Pix=1,33
          read(amb_corcoefu,*,err=901) (Dummy(I),I=1,6)
          DO I=1,5
            amb_h_NewCorr(I,Pix,Scan) = Dummy(I+1)
          ENDDO
        ENDDO
      ENDDO
      read(amb_corcoefu,*,err=901) (newbias_pix(I),I=1,4)
      read(amb_corcoefu,*,err=901) (newbias_mult(I),I=1,5)
      read(amb_corcoefu,*,err=901) threshold
      RETURN

  901 write(amb_cllgu,*) 'Error reading bias correction file'
      amb_h_TxPow(1)=0
      return
  902 write(amb_cllgu,*) 'No bias correction applied' 
      amb_h_TxPow(1)=0
      return
  903 write(amb_cllgu,*) 'No anomalous bias correction provided'
      return
      end
!##############################################################################
