      subroutine amb_status(lstat,ispvw,sampok)
!+
!NOM/NAME $Id: amb_status.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module checks the quality indicators on each scan line are OK
!     Also checks status of instrument (ie antenna scanning and channels are
!     switched on. If not OK for calibration then status flags returned 
!     as false.  Also the scan line quality indicators are updated.
!    
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
!     Roger SAUNDERS
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      October 95     no             R.Saunders
! 1.1      July  2002  Correct bit order in ispvw(iscan)  N. Atkinson
! 1.2      March 03   f90  test        T.Labrot
!            I.AND.J not correct and modified by IAND(I,J)
!            I.OR.J not correct and modified by IOR(I,J)
!            Changed  type XXX/yyy/
!                 to  type XXX
!                     data XXX/yyy/
!         
!ARGUMENTS 
!      lstat(mxlin) L*1 in/out  If .TRUE. then data ok if false don't use 
!      ispvw(mxlin) I*4 out  space viewing direction (0-3)
!      sampok(5,mxlin) l*1 out calibrate status for each channel 
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################

!--DECLARATIONS----------------------------------------------------------------
      implicit none                                                     
c-----------------------------------------
c
c
!      
! includes/commons
#include "ambclin.h"
#include "amb1b.h"
#include "ambscn.h"
#include "amb_cinit.h"
!
! local variables
      integer*2 imode,znormal,zbit
      integer*4 zmask,zmask1,ichan,zmask2,isp
      integer*4 zqual,zqual1,zqual2,i,ispvw(mxlin),iscan
      logical*1 lstat(mxlin),sampok(5,mxlin),warn
      data warn/.true./
! 
      data znormal/5/,zmask/255/,zmask1/7/,zqual/-1879048192/
      data zbit/48/,zqual1/1024/,zqual2/2048/,zmask2/6144/
!
!--END OF DECLARATIONS---------------------------------------------------------

      if (iprint.ge.1) write(amb_cllgu,*)
      if (iprint.ge.1) write(amb_cllgu,*) '***************************'
      if (iprint.ge.1) write(amb_cllgu,*) '**   SUBROUTINE AMB_STATUS*'
      if (iprint.ge.1) write(amb_cllgu,*) '***************************'
!..Loop in scan line................................................
      do 100 iscan = 1 , amb_h_scnlin
      ispvw(iscan) = 2                  ! initialize with default 
      do 10 i = 1 , 5
       sampok(i,iscan) = .true.
 10   continue 
         
! check scan line quality indicator
!
      if ( qualind(iscan) .lt. 0 )then
        lstat(iscan) = .FALSE.        ! bad scan line detected from ingest
        do 12 i = 1 , 5
          sampok(i,iscan) = .false.
 12     continue       
        go to 100
      endif
!      
! check instrument scanning OK for calibration and power is on
!       
      imode = IAND(diga(iscan),zmask)
      if (imode.ne.znormal)then            
        lstat(iscan) = .FALSE. 
        do 13 i = 1 , 5
          sampok(i,iscan) = .false.
 13     continue       
        qualind(iscan) = IOR(qualind(iscan),zqual) !set bits 28,31
        linqualflgs(iscan) = IOR(linqualflgs(iscan),zqual1) !set bit 10
        if(iscan.eq.1)
     &   write(amb_cllgu,*) ' Instrument power off or not scanning'
        go to 100
      endif      
!      
! check at least one channel is on
!     
      ichan = diga(iscan)/256
       ichan = IAND(ichan,zmask1)
      if (ichan.eq.0)then            
        lstat(iscan) = .FALSE.         ! all channels off
        do 14 i = 1 , 5
          sampok(i,iscan) = .false.
 14     continue       
        qualind(iscan) = IOR(qualind(iscan),zqual) !set bits 28,31
        linqualflgs(iscan) = IOR(linqualflgs(iscan),zqual1) !set bit 10
        if(iscan.eq.1)
     &   write(amb_cllgu,*) ' all channels off'
        go to 100
      endif
! 
!  get space view position (0->3)
!
      isp = IAND(diga(iscan),zmask2)
      ispvw(iscan) = isp/2048
      ispvw(iscan) = ispvw(iscan)/2 + mod(ispvw(iscan),2)*2  !Reverse bit order
      if (ispvw(iscan).ne.spview .and. warn)then
       if(iprint.ge.1)
     &  write(amb_cllgu,*)' Warning space view ',ispvw(iscan),
     &' selected ',spview,' is recommended'
       warn = .false.
      endif
! check if all channels are on and if not set flags
!     
      if (IAND(ichan,4).eq.0)then           ! chans 18/19/20 off
         linqualflgs(iscan) = IOR(linqualflgs(iscan),zqual2) !set bit 11
         chqualflg(3,iscan)=IOR(chqualflg(3,iscan),zbit)     !set bit 7
         chqualflg(4,iscan)=IOR(chqualflg(4,iscan),zbit)
         chqualflg(5,iscan)=IOR(chqualflg(5,iscan),zbit)
         sampok(3,iscan) = .false.
         sampok(4,iscan) = .false.
         sampok(5,iscan) = .false.
         if(iscan.eq.1)write(amb_cllgu,*) ' channels 18/19/20 off'
      endif
      
      if (IAND(ichan,2).eq.0)then  ! chan 17 off
         linqualflgs(iscan) = IOR(linqualflgs(iscan),zqual2) !set bit 11
         chqualflg(2,iscan)=IOR(chqualflg(2,iscan),zbit)     ! set bit 7
         sampok(2,iscan) = .false.
         if(iscan.eq.1)write(amb_cllgu,*) ' channel 17 off'
      endif
      if (IAND(ichan,1).eq.0)then  ! chan 16 off
         linqualflgs(iscan) = IOR(linqualflgs(iscan),zqual2) !set bit 11
         chqualflg(1,iscan)=IOR(chqualflg(1,iscan),zbit)     ! Set bit 7
         sampok(1,iscan) = .false.
         if(iscan.eq.1)write(amb_cllgu,*) ' channel 16 off'
      endif
!
 100  Continue   ! ......end of scan line loop
!
      return
      end
