      subroutine amb_updt(nvstat,calok,nbnavcal,ispvw)
!+
!NOM/NAME $Id: amb_updt.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!    This module updates the AMSU-B 1B header array and writes out
!    header and navigated, calibrated scan lines to the 1B file.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!
!PROPRIETAIRE/OWNER  
!     Roger Saunders  
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment               <author>)
! 1.0      February 96     no                R. SAUNDERS 
! 1.1      May 98       Update 1B definition N Atkinson 
! 1.2      June 98      fix bugs         M Derrien  
!                         invert amb_h_temrad a & b   
! 1.3      April 04     Add moonang      N Atkinson  
!ARGUMENTS
! nvstat(mxlin) l*1 in array containing navigation flags
! calok(5,mxlin) l*1 in array containing calibration flags
! nbnavcal i*4 out number of navigated and calibrated scans
! ispvw(mxlin) I*4 in  space viewing direction (0-3)
!
! EXTERNALS
! ambhdw, ambdtr, ambdtw
!------------------------------------------------------------------------------
!--DECLARATIONS----------------------------------------------------------------
      implicit none                                                      

!includes/commons
#include "ambclin.h"
#include "amb1b.h"
#include "ambscn.h"
#include "amb_cinit.h"
!
!local  variables
      integer iscan,ios,ic,j,i,nh,k,kk
      integer nbnavcal,ispvw(mxlin)
      real scala(4,4)
      logical*1 nvstat(mxlin),calok(5,mxlin)
      data scala/1E2,1E7,1E12,1E18,1E3,1E5,1,1,1,1E5,1,1,
     & 1E1,1E6,1E10,1E15/
!
!--END OF DECLARATIONS-------------------------------------------------
      if(iprint.ge.1)then
       write(amb_cllgu,*)' ******************************'
       write(amb_cllgu,*)' *          AMB_UPDT'
       write(amb_cllgu,*)' ******************************'
      endif
!
      nbnavcal = 0
      nh = amb_h_hdrcnt
!..determine nbnavcal
      do 10 iscan = 1 , amb_h_scnlin
       if(nvstat(iscan))then
        if(calok(1,iscan).and.calok(2,iscan).and.calok(3,iscan).and.
     & calok(4,iscan).and.calok(5,iscan))nbnavcal = nbnavcal+1
       endif
 10   continue
!..update header with calibration params (nb with correct scaling factors)
      amb_h_callocsclin = nbnavcal
      do 12 i = 1 , 3
       amb_h_reftemp(i) = nint(reftemp(i)*100)
      do 12 ic = 1 , 5
       j = (ic-1)*4 + i
       amb_h_bias(j) = nint(intcorr(ic,i)*1000)
       j = (ic-1)*4 + 4  
       amb_h_bias(j) = nint(spcorr(ic,ispvw(1)+1)*1000) ! sp view selected
       j = (ic-1)*3 + i
       amb_h_nonlin(j) = nint(nonlin(ic,i)*1000)
       amb_h_temrad(1+3*(ic-1)) = nint(freq(ic)*1E6)
       amb_h_temrad(2+3*(ic-1)) = nint(b(ic)*1E6)
       amb_h_temrad(3+3*(ic-1)) = nint(a(ic)*1E6)
 12   continue
!
!..add digital A telemetry conversion coeffs
      do 20 i = 1 , 24
      do 20 j = 1 , 4
       k = (i-1)*4 + j
       if(i.le.16 .or. (i.ge.20.and.i.le.22))then
         kk = 1
       elseif (i.ge.17.and.i.le.19)then
         kk = 2
       elseif (i.eq.23 )then
         kk = 3
       elseif (i.eq.24)then
         kk = 4
       endif
      amb_h_digacoef(k) = nint(diga_coef(j,i)*scala(j,kk))
  20  continue
!
!..add analogue telemetry conversion coeffs
      do 25 i = 1 , 18
      do 25 j = 1 , 4
       k = (i-1)*4 + j
      amb_h_analcoef(k) = nint(anal_coef(j,i)*1E6)
  25  continue
!..write out header to disk
      call ambhdw(amb_l1bu,ios)
      if(ios.ne.0)then
       write(amb_cllgu,*)' AMSU-B 1B header could not be written'
       stop
      endif
!
!..write out all data records to disk
!
      do 30 iscan = 1 , amb_h_scnlin  !-----start scan loop-------
!..read current 1B record before updating
       call ambdtr(amb_l1bu,iscan,ios)
       if(ios.ne.0)then
        write(amb_cllgu,*)' AMSU-B 1B data record could not be read'
        stop
       endif
       amb_scnlinyr = scnlinyr(iscan)
       amb_scnlindy = scnlindy(iscan)
       amb_scnlintime = scnlintime(iscan)
       amb_clockdrift = clockdrift(iscan)
       amb_qualind = qualind(iscan)
       amb_scnlinbit = scnlinbit(iscan)
       amb_linqualflgs = linqualflgs(iscan)
       do 5 i = 1 , 5
        amb_chqualflg(i) = chqualflg(i,iscan)
 5     continue
       amb_attangtime = attangtime(iscan)
       amb_rollang = rollang(iscan)
       amb_pitchang = pitchang(iscan)
       amb_yawang = yawang(iscan)
!       amb_rollcor = rollcor(iscan)     !These are obsolete (NCA, May 98)
!       amb_pitchcor = pitchcor(iscan)
!       amb_yawcor = yawcor(iscan)
       amb_navstat = navstat(iscan)
       amb_scalti = scalti(iscan)
       do 15 i = 1,mxpix*3
        amb_ang(i) = ang(i,iscan)
 15    continue
       do 16 i = 1 , mxpix*2
        amb_pos(i) = earthl(i,iscan)
 16    continue
       do i=1,4
         amb_moonang(i) = moonang(i,iscan)   !Added April 2004
       enddo
       do 35 i = 1 , 15
       amb_pmcal(i) = pmcal(i,iscan)
       amb_sdcal(i) = sdcal(i,iscan)
 35    continue
!..write current updated 1B record 
       call ambdtw(amb_l1bu,iscan,ios)
       if(ios.ne.0)then
       write(amb_cllgu,*)' AMSU-B 1B data record could not be written'
       stop
       endif
 30   continue  !------end scan loop------------- 
!
      return
      end

