      subroutine ambclexit(nbnavcal,nvstat,lstat,calok,sampok,prtstat)
!+
!NOM/NAME $Id: ambclexit.F 47 2006-08-21 08:44:41Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!    This module closes the amsubcl program and provides a summary
!    of the job if iprint is greater than or equal to 1.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!
!PROPRIETAIRE/OWNER  
!     Roger Saunders  
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      February 96     no             R. SAUNDERS 
! 1.1      Aug 2000   exit 10 when bad cal/loc P. Brunel
! 1.2   12 Oct 2001 loop limit for ncaltot calculation corrected   K Whyte
! 1.3      May 2004   Use ncalreq to allow for bad channels. N C Atkinson
!         
!ARGUMENTS
! nbnavcal i*4 in number of navigated and calibrated scans
! lstat(mxlin) l*1 in scan line quality flag (.true. is ok)
! nvstat(mxlin) l*1 in navigation quality flag
! calok(5,mxlin) l*1 in calibration ok flag
! sampok(5,mxlin) l*1 in cal view radiances quality flag
! prtstat(mxlin) l*1 in target and mixer prt quality flag
!------------------------------------------------------------------------------
!--DECLARATIONS----------------------------------------------------------------
      implicit none                                                      

!includes/commons
#include "ambclin.h"
#include "amb1b.h"
#include "ambscn.h"
#include "amb_cinit.h"
!
!local  variables
      integer iscan,ic
      integer nbnavcal,nnav,ncal(5),nprt,nsamp(5),ngood
      logical*1 nvstat(mxlin),lstat(mxlin),calok(5,mxlin)
      logical*1 sampok(5,mxlin),prtstat(mxlin)
      integer ierr        ! 0 = OK
      integer ncaltot     ! total number of calibrated scan lines*channels 
      integer ncalreq     ! required total calibration number
!
!--END OF DECLARATIONS-------------------------------------------------
      if(iprint.ge.1)then
       write(amb_cllgu,*)' ******************************'
       write(amb_cllgu,*)' *          AMBCLEXIT         *'
       write(amb_cllgu,*)' ******************************'
!..print summary
!
      write(amb_cllgu,*)' '
      write(amb_cllgu,*)'     AMSUBCL summary log'
      write(amb_cllgu,*)'     -------------------'
      nnav = 0
      nprt = 0
      ngood = 0
!      
      do 5 ic = 1 , 5
      nsamp(ic) = 0
      ncal(ic) = 0
 5    continue
      do 100 iscan = 1 , amb_h_scnlin
       if(nvstat(iscan))nnav = nnav + 1
       if(lstat(iscan))ngood = ngood + 1
       if(prtstat(iscan))nprt = nprt + 1
       do 100 ic = 1 , 5
        if(calok(ic,iscan))ncal(ic) = ncal(ic) + 1
        if(sampok(ic,iscan))nsamp(ic) = nsamp(ic) + 1
 100   continue
      write(amb_cllgu,7)amb_h_scnlin
 7    format(1x,i5,' scan lines in this dataset')
      write(amb_cllgu,10)nbnavcal
 10   format(1x,i5,' scan lines were calibrated and navigated')
      write(amb_cllgu,20)amb_h_scnlin-ngood
 20   format(1x,i5,' scan lines were flagged as corrupt data')
      write(amb_cllgu,30)nnav
 30   format(1x,i5,' scan lines had good antenna pointing')
      write(amb_cllgu,40)nprt
 40   format(1x,i5,' scan lines had good prt data',/)
      write(amb_cllgu,50)(nsamp(ic),ic=1,5)
 50   format(1x,'Chan 16=',i5,' scans of good cal views',/,
     &' Chan 17=',i5,' scans of good cal views',/,
     &' Chan 18=',i5,' scans of good cal views',/,
     &' Chan 19=',i5,' scans of good cal views',/,
     &' Chan 20=',i5,' scans of good cal views',/)
      write(amb_cllgu,60)(ncal(ic),ic=1,5)
 60   format(1x,'Chan 16=',i5,' scans calibrated',/,
     &' Chan 17=',i5,' scans calibrated',/,
     &' Chan 18=',i5,' scans calibrated',/,
     &' Chan 19=',i5,' scans calibrated',/,
     &' Chan 20=',i5,' scans calibrated',/)
      endif

!..close files
!
      close(amb_l1bu)
!      close(amb_cllgu)
      close(amb_stu)
!   
!
!.. final quality test for end of execution return code
      ierr = 0
      if(amb_lflg.eq.1 .and. nnav.le.10) then
         write(amb_cllgu,*) 'Error not enough navigated lines'
         ierr = 1
      endif

      ncaltot = 0
      ncalreq = 0
      do ic=1,5
        if (ncal(ic).ne.0) then
          ncaltot = ncaltot+ncal(ic)
          ncalreq = ncalreq + 10         !Minimum 10 lines per good channel
        endif
      end do
      if(amb_cflg.eq.1 .and. ncaltot.le.ncalreq) then
         write(amb_cllgu,*) 'Error not enough calibrated lines'
         ierr = 1
      endif

      close(amb_cllgu)

      if(ierr.ne.0) then
         ! all files are closed, exit 10 will be returned to user 
         ! through amsub.ksh script and enables to stop further processing
         call aappexit(10)
      endif

      return
      end


