      subroutine ambsetu(noanum,lstat,nvstat,calok)
!+
!NOM/NAME $Id: ambsetu.F 47 2006-08-21 08:44:41Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module opens  the log file, the statistics file (if requested).
!     It also opens the AMSU-B amb_.level1b and reads the header,
!     all the records. It also checks data type.
!     The 1b data are ranged in commons amb1b.h
!     The navigation files are also opened.
!
!DESCRIPTION 
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
!     Roger SAUNDERS
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      August 95      no             R.Saunders
! 1.1       31 May   96   add navigation part M. Derrien
! 1.2       Sept 96     recl options     M. Derrien
! 1.3       May 98      update variable names for housekeeping from 1B record
!                       accept an error when getting clock error, but apply 0
!                                        M. Derrien
! 1.4       Jan 99     Add bias correction  N Atkinson
! 2.0      August00    Add a test on ierr, output of sp_read (TL)
! 2.1      Dec 00   Display attitude values  P. Brunel
! 2.1      Dec 00   Change position in the code of the display of
!                   the clock error  P. Brunel
! 2.2      March 03   f90  test          T.Labrot
!                    I.AND.J not correct and modified by IAND(I,J) 
!                    I.OR.J not correct and modified by IOR(I,J)            
! 2.3      March 04   Modify for linux   N Atkinson / A Korpela
! 2.4      April 04   Add warning for too many scan lines   N Atkinson
! 2.5      July 04   Change statement for test on noaa klm serie
!                                         P. Brunel
! 2.6      July 04  Upgrade to navigation code version 2.
!                   def_att call changed for arguments and position in
!                   code (after sp_read)     P. Brunel
!
!ARGUMENTS
! noanum i*4 out  satellite id number (15-17 for NOAA-K to M)
!    comment (M.Derrien) as coded it is the order number ( 12 to 14)
! lstat(mxlin) l*1 out scan line status flag
! nvstat(mxlin) l*1 out navigation status flag
! calok(5,mxlin) l*1 out calibration status flag
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
! ord1bid  ambhdr ambdtr ml_wti ml_wt ml_wtdp def_att calatt 
! clkerr_get ml_wtr  sp_read 
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################

!--DECLARATIONS----------------------------------------------------------------
      implicit none                                                     
!
!
#include "ambclin.h"
#include "amb1b.h"
#include "ambscn.h"
#include "c_1bnav.h"
#include "reclopt.h"
!
! local variables
      integer i,j  !index of loops
      integer ios,amb_h_satid0,noanum
      integer irec,irec1
      integer recn       !record number in the 1B file
      logical*1 lstat(mxlin),calok(5,mxlin),nvstat(mxlin) ! status flags
      integer ierr
      double precision ijstrt,ijend
      character*6 scname
      integer loc_rl     ! local record length for opening AMSU-B 1B file
      integer*2 OldTxPow(5)  !Transmitter powers
      integer*2 Flag,OldFlag   !Anomalous bias flags
      integer att_mode          ! attitude mode
                                ! 0 = NOAA POES operational mode
                                ! 1 = METOP Yaw steering mode
                                ! 2 = Geocentric mode
      integer id_in             ! noaa id corresponding to satid variable
      integer id_noaak          ! noaa id corresponding to Noaa-K == 15
!      
! functions
      integer ord1bid 
      character*6 noascnam
      integer noascid

      data id_noaak/15/

!
!--END OF DECLARATIONS---------------------------------------------------------

!open the log file
      open(amb_cllgu,form='formatted',access='sequential',
     &     status='new',iostat=ios)

!     If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
      if ( ios.ne.0 )
     &  open(amb_cllgu,form='formatted',access='sequential',
     &     status='unknown',iostat=ios)
!     
      if (iprint.ge.1) write(amb_cllgu,*) '**************************'
      if (iprint.ge.1) write(amb_cllgu,*) '** Starting AMSUBCL     **'
      if (iprint.ge.1) write(amb_cllgu,*) '**************************'
      if (iprint.ge.1) write(amb_cllgu,*)
      if (iprint.ge.1) write(amb_cllgu,*)
      if (iprint.ge.1) write(amb_cllgu,*) '***************************'
      if (iprint.ge.1) write(amb_cllgu,*) '**   SUBROUTINE AMBSETU  **'
      if (iprint.ge.1) write(amb_cllgu,*) '***************************'


!open the statistics file
      if(amb_stflg.eq.1) then
        open(amb_stu,form='formatted',access='sequential',
     &       status='new',iostat=ios)

!       If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
        if ( ios.ne.0 )
     &  open(amb_stu,form='formatted',access='sequential',
     &       status='unknown',iostat=ios)
      endif

!open and read the AMSU-B-level1b file
      loc_rl = reclamb / rldiv
      open(amb_l1bu,form='unformatted',access='direct',recl=loc_rl,
     &     status='old',iostat=ios)

!     If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
      if ( ios.ne.0 )
     &  open(amb_l1bu,form='unformatted',access='direct',recl=loc_rl,
     &     status='unknown',iostat=ios)

      if(ios.ne.0) then
       write(amb_cllgu,*)'AMSU_B 1B file could not be opened,ios:',ios
       stop
      else
       if(iprint.ge.1)
     & write(amb_cllgu,*)'AMSU-B 1B file has been opened,ios:',ios
      endif

      call ambhdr(amb_l1bu,ios)
      if(ios.ne.0) then
       write(amb_cllgu,*)' AMSU-B 1B header could not be read,ios:',ios
       stop
      else
       if(amb_h_datatyp.ne.11)then
         write(amb_cllgu,*)' Not AMSU-B data '
         stop
       endif
       if(amb_h_inststat1.ne.0.and.amb_h_inststat2.ne.0)then
         write(amb_cllgu,*)' Warning abnormal instrument status'
       endif  

       CALL AMB_READCORR     !Read bias correction tables and store in header
       IF (amb_h_TxPow(1).NE.0) THEN   !Only if the ref powers are non zero
        IF (amb_h_scnlin.GE.6) THEN
         CALL ambdtr(amb_l1bu,6,ios)   !Read transmitter powers from line 6
         DO J=1,5
           OldTxPow(J) = amb_ahk(22+J)
         ENDDO
        ENDIF         
       ENDIF

       OldFlag=-2  !Initialise
       Flag=-2     !Initialise

       IF (amb_h_scnlin .gt. mxlin) THEN
         write(amb_cllgu,*)
     &    'WARNING: too many scan lines',amb_h_scnlin,
     &    '. Just process to line',mxlin
         amb_h_scnlin = mxlin
       ENDIF
       
       irec = 0
       do 100 recn= 1 , amb_h_scnlin
        irec = irec + 1
        call ambdtr(amb_l1bu,irec,ios)
        if(ios.ne.0) then
         write(amb_cllgu,*)
     &         ' AMSU-B 1B record could not be read, ios:',ios
         stop
        else
!
         lstat(irec) = .true.
         nvstat(irec) = .false.               ! init nav status flag
         scnlin(irec)     = amb_scnlin 
         scnlinyr(irec)   = amb_scnlinyr 
         scnlindy(irec)   = amb_scnlindy 
         scnlintime(irec) = amb_scnlintime 
         clockdrift(irec) = amb_clockdrift          
         qualind(irec)    = amb_qualind 
         scnlinbit(irec)  = amb_scnlinbit 
         linqualflgs(irec)  = amb_linqualflgs 
         do 5 i = 1,5
          calok(i,irec) = .false.                 ! init cal status flag
          chqualflg(i,irec) = amb_chqualflg(i)
 5       continue
         tipmf(irec)       = amb_tipmf 
         attangtime(irec)  = amb_attangtime 
         rollang(irec)     = amb_rollang
         pitchang(irec)    = amb_pitchang
         yawang(irec)      = amb_yawang
!         rollcor(irec)     = amb_rollcor    !These are obsolete
!         pitchcor(irec)    = amb_pitchcor
!         yawcor(irec)      = amb_yawcor
         navstat(irec)     = amb_navstat
         scalti(irec)      = amb_scalti
         do 10 i = 1 , mxpix*3
          ang(i,irec) = amb_ang(i)
 10      continue
         do 15 i = 1 , mxpix*2
          earthl(i,irec) = amb_pos(i)
 15      continue
         do 20 i =  1 , 540
          if(amb_earth_data(i).ge.0)then
           earth_data(i,irec) = amb_earth_data(i)
          else
           earth_data(i,irec) = amb_earth_data(i)+65536
          endif
 20      continue
         if(amb_diga(1).ge.0)then
          stat(irec)       = amb_diga(1)
         else 
          stat(irec)       = amb_diga(1)+65536
         endif
         if(amb_diga(2).ge.0)then
          diga(irec)       = amb_diga(2)
         else 
          diga(irec)       = amb_diga(2)+65536
         endif
         do 25 i = 1 , 24
          if(amb_temps(i).ge.0)then
           temps(i,irec)   = amb_temps(i)
          else
           temps(i,irec)   = amb_temps(i)+65536
          endif
          if(amb_coldcal(i).ge.0)then
           coldcal(i,irec) = amb_coldcal(i)
          else
           coldcal(i,irec) = amb_coldcal(i)+65536
          endif
          if(amb_warmcal(i).ge.0)then
           warmcal(i,irec) = amb_warmcal(i)
          else
           warmcal(i,irec) = amb_warmcal(i)+65536
          endif
 25      continue
!
! For scans 1-5, use the transmitter powers from scan 6. The SARR power is only
! updated every 16 seconds.
!
         IF (amb_h_TxPow(1).NE.0) THEN   !Only if the ref powers are non zero
          IF (irec.LT.6) THEN
           DO J=1,5
             amb_ahk(22+J)=OldTxPow(J)
           ENDDO
          ENDIF
!
! Correct for normal and anomalous bias effects on space and target view.
! Only do the corrections and set flags if the quality indicator is +ve: the 
! decommutation sets bit 31 if not scan normal or there is a sync error.
!
          IF (qualind(irec).GE.0) THEN
           CALL AMB_TestNewBias(earth_data(1,irec),Flag)
           CALL AMB_CALCORRECT(coldcal(1,irec),warmcal(1,irec),Flag)
          IF (Flag.EQ.1) qualind(irec)=IOR(qualind(irec),32) !Flag anomalous
           IF (Flag-OldFlag.EQ.1) THEN                      ! bias
             write(amb_cllgu,*) 'Anomalous bias on at scan ',irec
           ELSE IF (Flag-OldFlag.EQ.-1) THEN
             write(amb_cllgu,*) 'Anomalous bias off at scan ',irec
           ENDIF
           IF (ABS(Flag-OldFlag).EQ.1) THEN           !Use bit 6 to flag bias
              qualind(irec) = IOR(qualind(irec),64)      !change: 2 scans are 
              qualind(irec-1) = IOR(qualind(irec-1),64)  !unreliable
           ENDIF
          ENDIF                                                 
!
! Flag transmitter power changes
!
          DO J=1,5
           IF ((ABS(amb_ahk(22+J)-OldTxPow(J)).GT.50).OR.
     &         (ABS(Flag-OldFlag).EQ.1)) THEN
             irec1 = irec-3
             IF (irec1.LT.1) irec1=1
             DO I=irec,irec1,-1
               qualind(I) = IOR(qualind(I),16)   !Flag lines within convolution
             ENDDO                             ! interval
             IF ((amb_ahk(22+J)-OldTxPow(J)).GT.50) THEN
               write(amb_cllgu,*) 'Transmitter ',J,' on at scan ',irec
             ELSE IF ((amb_ahk(22+J)-OldTxPow(J)).LT.-50) THEN
               write(amb_cllgu,*) 'Transmitter ',J,' off at scan ',irec
             ENDIF
           ENDIF
          ENDDO
          DO J=1,5
            OldTxPow(J) = amb_ahk(22+J)   !Store transmitter powers
          ENDDO
          OldFlag = Flag
         ENDIF
!
        endif
 100   continue
      endif
!
! Go backwards through 'qualind' array, flagging any lines for which the bias
! or transmitters change within convolution interval
!
      do recn = amb_h_scnlin-1,1,-1
        IF ( IAND(qualind(recn),16) .GT.0) THEN
          DO irec=recn+1,recn+3
            IF (irec.LE.amb_h_scnlin) THEN
              qualind(irec) = IOR(qualind(irec),16)    !Set bit 4
            ENDIF
          ENDDO
        ENDIF
      ENDDO
!
!Deliver the order satellite number for the l1b satellite number and
!the julian date of the data
      amb_h_satid0 = amb_h_satid
      noanum = ord1bid(amb_h_satid0,amb_h_startdatajd)  
      id_in  = noascid(noanum)
!  
      if (iprint.ge.1) then
       write(amb_cllgu,fmt=504) scnlintime(1)
       write(amb_cllgu,fmt=505) scnlindy(1)
       write(amb_cllgu,fmt=506) scnlinyr(1)
       write(amb_cllgu,fmt=507) scnlintime(irec)
       write(amb_cllgu,fmt=508) scnlindy(irec)
       write(amb_cllgu,fmt=509) scnlinyr(irec)
       write(amb_cllgu,fmt=501) amb_h_scnlin
      endif
!
      if (id_in .lt. id_noaak .or. id_in .gt. id_noaak+2) then
      !if (noanum.lt.12.or.noanum.gt.14) then

!
        write(amb_cllgu,*) ' sat. id. invalid or an error occured',
     &                     ' during the sat. id. file reading'
        write(amb_cllgu,*) 'noanum ',noanum
        write(amb_cllgu,*) 'amb_h_satid ',amb_h_satid
        
        noanum = 0
        return
! 
       else
!       
       if (iprint.ge.1)write(amb_cllgu,fmt=500) amb_h_satid0
       if (iprint.ge.1)write(amb_cllgu,510) noascnam(noanum)
      endif
      
      scname=noascnam(noanum)
      write(amb_cllgu,*) 'scname ',scname

!
! Open localisation files if requested
! i.e. according to the options read in ambclin module
!
      if(amb_lflg.eq.1.or.amb_clflg.eq.1) then
! Opens and reads the SATPOS file
! -----------------------------------
          open (unit=satposu,status='old',iostat=ios)

!         If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
          if ( ios.ne.0 )
     &      open (unit=satposu,status='unknown',iostat=ios)

          if( ios.ne.0 ) then
            call ml_wti('error',
     &       'while opening the satpos file I/O status',ios)
            call ml_wt('error','ambsetu aborted')
            call aappexit(2)
          endif

! Read the satpos file
! --------------------
          ijstrt=amb_h_startdatajd + 
     &      (amb_h_startdatatime-errclock*1000.)/86400.d3
          ijend=amb_h_enddatajd + 
     &       (amb_h_enddatatime-errclock*1000.)/86400.d3
          call sp_read(ijstrt,ijend,satposu,scname,ierr)
          close(unit=satposu,iostat=ios)
        
          if(ierr.ne.0) then
             call ml_wti('error','return code of sp_read differ from 0'
     &                   ,ierr)
             call ml_wt('error','ambsetu aborted')
             call ml_wt('error','navigation aborted')
             call ml_wt('error','but calibration can work')
             call ml_wt('error','try calibration alone')
             call aappexit(2)
          endif

          if(yaw.lt.-99.d-03) then
             call def_att(scname,att_mode,yaw,roll,pitch)
             call ml_wt('info','take default attitude values')
             call ml_wti('info','default attitude mode ',att_mode)
             call ml_wtdp('info','default attitude yaw ',yaw)
             call ml_wtdp('info','default attitude roll ',roll)
             call ml_wtdp('info','default attitude pitch ',pitch)
          else
             call ml_wtdp('info','attitude yaw ',yaw)
             call ml_wtdp('info','attitude roll ',roll)
             call ml_wtdp('info','attitude pitch ',pitch)
          endif
          call calatt(yaw,roll,pitch,att)

! If the file has already been located we do not correct
! again from the clock error ( this is known from the header
! epoyr test
! --------------------------
          if(amb_h_epoyr.le.1950) then
          open (unit=ckdriftu,status='old',iostat=ios)

!         If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
          if ( ios.ne.0 )
     &      open (unit=ckdriftu,status='unknown',iostat=ios)

          if(ios.ne.0) then
            call ml_wti('error',
     &       'opening clock error file I/O status',ios)
            call aappexit(2)
          endif
          ijstrt=amb_h_startdatajd + 
     &      (amb_h_startdatatime)/86400.d3
          call clkerr_get(ckdriftu,ijstrt,errclock,ierr)
          close(unit=ckdriftu,iostat=ios)
          if(ierr.ne.0.or.abs(errclock).gt.2.) then
            call ml_wti('warning',
     &       'reading clock error value from clkerr_get ',ierr)
            call ml_wtr('warning','clock error from file',
     &                 errclock)
            call ml_wt('warning','clock error forced to 0')
            errclock=0 
          else
            call ml_wtr('info','clock error value (sec)',errclock)
          endif
         else
          errclock=0.
          call ml_wt('info',
     &     '1B file already corrected from clock error')
         endif
        
! Init earth location parameters
! ------------------------------
        first_pnav = 1 ! first pixel to locate
        first_lnav = 1 ! first scan to process
        step_pnav = 1  ! step between pixels to locate
        step_lnav = 1  ! ste betwwen lines to locate
        nb_pnav = 90 ! 90 pixels for AMSU-B
      endif
!
 500  format(1X,'NOAA 1B spacecraft identification code: ',I2)
 501  format(1X,' Number of AMSU-B scan lines in this dataset ',I4)
 504  format(1X,'start of data set time (msecs): ',I8)
 505  format(1X,'start of data set Julian day: ',I7)
 506  format(1X,'start of data set Year: ',I7)
 507  format(1X,'end of data set time (msecs): ',I8)
 508  format(1X,'end of data set Julian day: ',I7)
 509  format(1X,'end of data set Year: ',I7)
 510  format(1X,'NOAA satellite name: ',A6)
 
      return
      end
