      subroutine avh_cal
!+
!NOM/NAME $Id: avh_cal.F 38 2006-06-16 13:04:03Z frna $
! avh_cal
!
!BUT/PURPOSE 
! manage the main loop for avhrr calibration
! initialize thermistors arrays before starting
! get target and space counts
! pass gross and 2 sigma filters
! compute coefficients for the sub-blocks
! linearize the coefficients for each scan line
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER 
! Marcel Derrien
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Sept. 95      no             Marcel Derrien
! 1.1      04/03/99  incorrect arrays used   P C Dibben
!                    sb_k2...=sb_k1... (should be sb_k2)
!                    sb_k3...=sb_k1... (should be sb_k3)
! 1.2      June  06  change number of superblocks P. Marguinaud
!
!         
!ARGUMENTS
!
!       
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!DECLARATIONS
!
!    
     
!commons
#include "avhrcl.h"
#include "avh1b.h"
#include "cavh_wrk.h"
!local variables
       integer strt_sblock
       integer end_sblock
       integer sblock,last_sblock
       integer i
!_
!----------------------------------------------
! initialize thermistors arrays before starting
!----------------------------------------------
        call avh_iprt
!-------------------------------------------
!       main loop on the "normal" sub-blocks
!-------------------------------------------
        strt_sblock = 1
        end_sblock = (avh_h_scnlin - start_cal_seq  + 1 ) / 5
        last_sblock = 1     
        do sblock = strt_sblock , end_sblock
          if(sblock.gt.6. and. sblock.lt. (end_sblock-6)) 
     &      call avh_gprt(sblock)
!------------------------------------
!         get target and space counts
!------------------------------------
          call avh_gvie(sblock)
!----------------------------------------
!         pass gross and 2 sigma filters
!----------------------------------------
          call avh_filt(flag_coe(sblock))
          if(flag_coe(sblock)) then
!-----------------------------------------------
!           compute coefficients for the sub-block
!-----------------------------------------------
            call avh_ccof(sblock)
            last_sblock=sblock
          else
            call ml_wti('warning','avh_cal pb flag_coe sblock',
     &      sblock)
            do i = 1,3
              sb_k1(i,sblock)=sb_k1(i,last_sblock)
              sb_k2(i,sblock)=sb_k2(i,last_sblock)
              sb_k3(i,sblock)=sb_k3(i,last_sblock)
            enddo
          endif
          call avh_sum(sblock)
        enddo 
!---------------------------------------------------
!      linearize the coefficients for each scan line
!---------------------------------------------------
        call avh_lico   
        return
        end    
