      subroutine avh_ccof(sblock)
!+
!NOM/NAME $Id: avh_ccof.F 346 2013-02-11 14:08:03Z frna $
! avh_ccof
!
!BUT/PURPOSE 
!
! calculate calibration coefficients for the current sub-block 
!        apply radiance non-linearity corrections method
!        according to NESS 107 specifications (see NOAA-KLMNN' Users' Guide)
!        with the following equations:
!
!        rlin-rsp = gain*(x-xsp)
!        rlin = gain*x + I          I = rsp - gain*xsp
!
!        rad = a*rlin + b*rlin**2 + c   (a,b,c specified in avhcal.txt)
!            = a*gain*x + a*I + b*[gain*x + I]**2 + c
!            = x**2*(b*gain**2) + x*(a*gain + 2*b*I*gain) + (a*I + b*I**2 + c)
!
!        rad = k1*x**2 + k2* x + k3
!        k1 = b * gain**2
!        k2 = a*gain + 2*b*I*gain
!           = a*gain + 2*b*rsp*gain - 2*xsp*k1
!        k3 = a*I + b*I**2 + c

!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER 
! Marcel Derrien
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Nov. 95      no             Marcel Derrien
! 1.1      Feb 2013   Update the equations in the "PURPOSE" section, so that
!                     they are consistent with Scientific Description doc. NCA.
!
!         
!ARGUMENTS
!
!       
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!DECLARATIONS
!
!    
!arguments
      integer sblock     
!commons
#include "avhrcl.h"
#include "avh1b.h"
#include "cavh_m1b.h"
#include "cavh_wrk.h"
#include "cavh_cpi.h"
!local variables
       integer i
       integer ichstrt
       double precision target_temp
       real gain
       real intercept
       real k1,k2,k3
       real target_radiance    
! functions
        real avh_bcor
!- 
!
!      convert prt counts to temperature
!
       call avh_poly(cth_mean,target_temp) 
       if(nb_obsirchn.eq.3) then
         ichstrt=1
       else
         ichstrt=2
       endif
!
!      compute the calibration coefficients for the sub-block
!
       do i = ichstrt , 3         
         target_radiance = avh_bcor (target_temp,i)
         gain = (target_radiance-rsp(i))/
     &            (bb_mean(i)-sp_mean(i+3))
         intercept = rsp(i) - gain * sp_mean(i+3)
!
!        apply non-linearity corrections 
!        according to NESS 107 specifications with the following
!        equations
!        rad = a*(rlin-rsp)+b*(rlin-rsp)**2 + c
!        rlin-rsp = gain*(x-xsp)
!        rad=a*gain*(x-xsp)+b*[gain*(x-xsp)]**2 +c
!        rad = k1*x**2 + k2* x +k3
!        k1 = b * gain**2
!        k2 = a*gain - 2*k1*xsp
!        k3 = c + k1*xsp**2 -a*gain*xsp
!
!         k1 = b_radcor(i)*gain*gain
!         k2 = a_radcor(i)*gain - 2. * k1 * sp_mean(i+3)
!         k3 = c_radcor(i) 
!     &      + k1 * sp_mean(i+3) * sp_mean(i+3)
!     &      - a_radcor(i) * gain *  sp_mean(i+3) 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! with  r= a*rlin + b*rlin**2 +c
!       k1 = b * gain**2
!
!       k2 = a*gain - 2*k1*xsp + 2*b*gain*rsp
!
!       k3 = c + a*rsp -a*gain*xsp +b*(rsp -gain* xsp)**2 
! 
        k1 = b_radcor(i)*gain*gain

        k2 = a_radcor(i) * gain - 2. * k1 * sp_mean(i+3)
     &  + 2. * b_radcor(i) * gain * rsp(i)

        k3 = c_radcor(i) + b_radcor(i) * intercept * intercept
     &    + a_radcor(i) * intercept
!  
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!        save k1,k2,k3 for this sub-block
!
         sb_k1(i,sblock) = k1
         sb_k2(i,sblock) = k2
         sb_k3(i,sblock) = k3
         if (i.eq.1) then
!         type *,'canal ',i,k1,k2,k3
!         type *,'      alpha,beta,rsp,spmean,tcible,rcible ', 
!     &     gain,intercept,rsp(i),sp_mean(i+3),target_radiance
!     &     ,target_temp
         endif
       enddo
!       type *,target_temp,sb_k1(i,sblock),sb_k2(i,sblock)
!     &     ,sb_k3(i,sblock)
       return
       end
