      subroutine avh_clst
!+
!NOM/NAME $Id: avh_clst.F 59 2006-11-10 11:08:39Z frna $
! avh_clst 
!
!BUT/PURPOSE
! calculate statistics from the AVHRR 1B calibration
! write summaty to stderr
! write statistic record on lustat
! close lustat
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER 
! Marcel Derrien
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Nov. 95      no             Marcel Derrien
! 1.1      March 96    add typorb to stat file (Marcel Derrien)
! 1.2      March 96    replace output avh_h_startdatajd by double precision
!                      instant
! 1.3      March 96    correct calculation of pspsu
! 1.4      July 96      bad kimeans when corrupted sblocks 
! 1.5      Nov 2006    check statc not zero (Nigel Atkinson)
!         
!ARGUMENTS
!
!       input : integer avh_cloptn(2)  ! options for avhrcl
!               avh_cloptn(1) -- 1  for AVHRR earth location option
!                                0  no earth location requested
!               avh_cloptn(2) -- 1  for AVHRR calibration option
!                                0  no calibration requested 
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!DECLARATIONS
!
!    
!commons
#include "avh1b.h"
#include "avhrcl.h"
#include "cavh_wrk.h"
!local variables
       integer i,ios
       real cthmean(4),pcthinf(4),pcthsup(4),pcthrej(4)
       real bbmean(3),bbsig(3),pbbinf(3),pbbsup(3),pbbrej(3)
       real spmean(6),spsig(6),pspinf(6),pspsup(6),psprej(6)
       character*5 orbnum
       character*80 text
       integer iq(3) ! gregorian start instant
       double precision ij ! julian start instant for output
       integer statc ! number of correct sub-blocks
       integer statc3b ! number of correct sub-blocks for ch3b
!functions
!-

!-------------------
! compute statistics
!-------------------
       if (stat_cnt .ne.0 ) then
!---------------------------------
! mean coefficients for sub-blocks
!---------------------------------
         statc = 0
         statc3b = 0
         do i = 1 , nb_sblock
           if(flag_coe(i)) statc = statc +1
           if(flag_coe(i).and.ch3_val(i).eq.ch3b) then
             statc3b = statc3b + 1
           endif
         enddo 
         do i = 2 , 3
           if(statc.ne.0) then 
             k1mean(i) = sumk1(i)/statc
             k2mean(i) = sumk2(i)/statc
             k3mean(i) = sumk3(i)/statc
           else
             k1mean(i) = 0.
             k2mean(i) = 0.
             k3mean(i) = 0.
           endif
         enddo
         i=1
         if(statc3b.ne.0) then
           k1mean(i) = sumk1(i)/statc3b
           k2mean(i) = sumk2(i)/statc3b
           k3mean(i) = sumk3(i)/statc3b
         else
           k1mean(i) = 0.
           k2mean(i) = 0.
           k3mean(i) = 0.
         endif         
!------------------------
!mean thermistors counts
!------------------------
         do  i=1,4
           cthmean(i)=sumcthmean(i)/stat_cnt
           if( sumcth(i).ne.0) then
             pcthinf(i)=(sumcthinf(i)/sumcth(i))*100.
             pcthsup(i)=(sumcthsup(i)/sumcth(i))*100.
             pcthrej(i)=(sumcthrej(i)/sumcth(i))*100.
           endif
         enddo
!------------------------
! mean target view counts
!------------------------
         i = 1
         if(stat_ch3b.ne.0) then
           bbmean(i)=sumbbmean(i)/stat_ch3b
           bbsig(i)=sumbbsig(i)/stat_ch3b
           if(sumbb(i).ne.0) then
             pbbinf(i)=(sumbbinf(i)/sumbb(i))*100.
             pbbsup(i)=(sumbbsup(i)/sumbb(i))*100.
             pbbrej(i)=(sumbbrej(i)/sumbb(i))*100.
           endif
         else
           bbmean(i)=0.
           bbsig(i)=0.
           pbbinf(i)=0.
           pbbsup(i)=0.
           pbbrej(i)=0.
         endif
         do i=2,3
           bbmean(i)=sumbbmean(i)/stat_cnt
           bbsig(i)=sumbbsig(i)/stat_cnt
           if(sumbb(i).ne.0) then
             pbbinf(i)=(sumbbinf(i)/sumbb(i))*100.
             pbbsup(i)=(sumbbsup(i)/sumbb(i))*100.
             pbbrej(i)=(sumbbrej(i)/sumbb(i))*100.
           endif
         enddo
!-----------------------
! mean space view counts
!-----------------------

         do i=1,6
           spmean(i) = 0.
           spsig(i)  = 0.
           pspinf(i) = 0.
           pspsup(i) = 0.
           psprej(i) = 0.
           if (i.eq.3.and.stat_ch3a.ne.0) then
             spmean(i)=sumspmean(i)/stat_ch3a
             spsig(i)=sumspsig(i)/stat_ch3a
           else if (i.eq.4.and.stat_ch3b.ne.0) then
             spmean(i)=sumspmean(i)/stat_ch3b
             spsig(i)=sumspsig(i)/stat_ch3b
           else
             spmean(i)=sumspmean(i)/stat_cnt
             spsig(i)=sumspsig(i)/stat_cnt
           endif
           if( sumsp(i).ne.0) then
             pspinf(i)=(sumspinf(i)/sumsp(i))*100.
             pspsup(i)=(sumspsup(i)/sumsp(i))*100.
             psprej(i)=(sumsprej(i)/sumsp(i))*100.
           endif
         enddo
       endif
!
       orbnum = avh_h_prblkid(2:6)
!
! convert year day and time to julian instant for output
       iq(1)=avh_h_startdatayr
       iq(2)=avh_h_startdatady
       iq(3)=avh_h_startdatatime
       call xiqj(iq,ij)
!--------------------------------------
! output information on statistics file
!--------------------------------------
       
       write(lustat,9000,iostat=ios) orbnum,typorb,ij,
     &   stat_cnt,stat_ch3a,stat_ch3b,
     &   k1mean,k2mean,k3mean,
     &   cthmean,pcthinf,pcthsup,pcthrej,
     &   bbmean,bbsig,pbbinf,pbbsup,pbbrej,
     &   spmean,spsig,pspinf,pspsup,psprej
9000   format(a5,1x,a1,f11.4,3i5,7f11.7,2f9.2,
     & 4f6.1,4f5.1,4f5.1,4f5.1,
     & 3f6.1,3f5.1,3f5.1,3f5.1,3f5.1,
     & 6f6.1,6f5.1,6f5.1,6f5.1,6f5.1)
       if(ios.ne.0) then
         call ml_wti('warning',
     &   'avh_clst problem when writing statistics',ios)
       endif
       close(lustat,iostat=ios)
       if(ios.ne.0) then
         call ml_wti('warning',
     &   'avh_clst problem when closing statistics',ios)
       endif
       write(text,9100) k1mean
9100   format('k1 mean',3f11.7)
       call ml_wt('info',text)
       write(text,9200) k2mean
9200   format('k2 mean',3f11.7)
       call ml_wt('info',text)
       write(text,9300) k3mean
9300   format('k3 mean',f11.7,2f9.2)
       call ml_wt('info',text)
       return
       end      
