      subroutine avh_cpar(lu,satid)
!+
!NOM/NAME $Id: avh_cpar.F 88 2007-05-30 13:52:09Z frna $
! avh_cpar
!
!BUT/PURPOSE 
! avh_cpar opens and reads the calibration parameters file
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER 
! Marcel Derrien
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Sept. 95      no             Marcel Derrien
! 1.1      Nov.00    Updated to accept NOAA16 data  TL
! 1.2      Mar 04    Modify for linux  N Atkinson / A Korpela
! 1.3      Jul 04    Change statement for test on noaa klm serie
!                                         P. Brunel
! 1.4    10/06/05    Remove format 9004 to read AVHRR normalized 
!                    response functions   T.Labrot
! 1.5      May 06    Extend test on noaak for metop PM/PB
! 1.6      May 07    Change open for g95 P. Marguinaud
!
!         
!ARGUMENTS
!
!       input : integer lu  !logical unit for calib parameters file
!       input : integer satid ! spacecraft internal number (see libsatid)
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!DECLARATIONS
!
!    
!arguments
      integer lu ! logical unit for calibration parameter file     
      integer satid ! spacecraft number
!commons
#include "cavh_cpi.h"
!local variables
      integer ios           ! i/o status
      integer i,j,k
      integer irchn,chn
      integer nbp
      character*80 text     ! to write message log
      character*80 charact  ! to read
      character*6 scname
      integer id_in             ! noaa id corresponding to satid variable
      integer id_noaak          ! noaa id corresponding to Noaa-K == 15
!functions 
!    from libsatid
      !integer genscid  
      character*6 noascnam
      integer noascid

      data id_noaak/15/
!-
!open calibration parameter file
!
#ifdef FORTRAN_OPEN_RW
       open(lu,access='sequential',status='old',iostat=ios,
     > action='READ')
#else
       open(lu,access='sequential',status='old',iostat=ios)

!      If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
       if(ios.ne.0)
     &   open(lu,access='sequential',status='unknown',iostat=ios)

       if(ios.ne.0) then
         text='avh_cpar: avhcal.txt file could not be opened'
         call ml_wti('error',text,ios)
         call aappexit(2)
       endif
#endif
!
!  skip the # commented first lines
!
       read(lu,'(a80)',err=100) charact
       do while (charact(1:1) .eq. '#')
        read(lu,'(a80)',err=100) charact
       enddo
       backspace lu
!
!  get the version num,year and day
!
       read(lu,*,err=100)versnb
       read(lu,*,err=100)versyr
       read(lu,*,err=100)versdy
       write(text,9000) versnb,versyr,versdy
       call ml_wt('info',text)
!       
! skip the # commented lines
!
       read(lu,'(a80)',err=100) charact
       do while (charact(1:1) .eq. '#')
         read(lu,'(a80)',err=100) charact
       enddo
       backspace lu
!
! read the physical constants c1 , c2 Planck function
!
       read(lu,*,err=100) c1,c2
!
! go to the specified satellite section
!
       do while (charact(4:9) .ne. noascnam(satid))
        read(lu,'(a15)',err=100) charact
       enddo
!
! get PRT count to temperature  conversion coefficients
! skip the # commented lines
!
       read(lu,'(a80)',err=100) charact
       do while (charact(1:1) .eq. '#')
        read(lu,'(a80)',err=100) charact
       enddo
       backspace lu
       
!       write(*,*) 'PRT count to temperature  conversion coefficients'
       do i=1,4 
         read(lu,*) (conv_prt(j,i),j=1,5)
!         write(*,*)(conv_prt(j,i),j=1,5)
        enddo
!
! get PRT weighting factors
!
       read(lu,'(a80)',err=100) charact
       do while (charact(1:1) .eq. '#')
        read(lu,'(a80)',err=100) charact
       enddo
       backspace lu
       read(lu,*) (weight_prt(i),i=1,4)
!       write(*,*)  'weight_prt'
!       write(*,*) (weight_prt(i),i=1,4)

! get visible coefficients
       read(lu,'(a80)',err=100) charact
       do while (charact(1:1) .eq. '#')
        read(lu,'(a80)',err=100) charact
       enddo
       backspace lu
       id_in  = noascid(satid)
       scname = noascnam(satid)
       !if(satid.ge.genscid('noaa-k')) then
       if( ( id_in .ge. id_noaak ) .or.
     &   ( scname(1:4) .eq. 'meto' ) ) then
         nbvischn=3
       else
         nbvischn=2
       endif
!       write(*,*) 'nbvischn=',nbvischn
!       write(*,*) 'visible coefficients'
       do i=1,nbvischn
        read(lu,*) vis_slope(i,1),vis_slope(i,2),
     &      vis_int(i,1),vis_int(i,2)
!        write(*,*) vis_slope(i,1),vis_slope(i,2),
!     &      vis_int(i,1),vis_int(i,2)

       enddo
!
! get equivalent widths, integrated solar irradiance,
! and effective central wavelength
!
       read(lu,'(a80)',err=100) charact
       do while (charact(1:1) .eq. '#')
        read(lu,'(a80)',err=100) charact
       enddo
       backspace lu
       
!      write(*,*) 'equivalent widths, integrated solar irradiance ...'
      do i=1,nbvischn
        read(lu,*,err=100) vis_eqwi(i),vis_isi(i),vis_ecwa(i)
!        write(*,*) vis_eqwi(i),vis_isi(i),vis_ecwa(i)
      enddo
!
! get AVHRR normalized response functions
! channel number
! starting wave
! increment
! number of points
!
       read(lu,'(a80)',err=100) charact
       do while (charact(1:1) .eq. '#')
        read(lu,'(a80)',err=100) charact
       enddo
       backspace lu
       do chn=1,3
         read(lu,9001,err=100) irchn
!         write(*,9001) irchn         
         read(lu,9002,err=100) ir_startwa(irchn-2)
!         write(*,9002) ir_startwa(irchn-2)
         read(lu,9002,err=100)ir_wavinc(irchn-2)
!         write(*,9002)ir_wavinc(irchn-2)
         read(lu,9003,err=100) nbp
!         write(*,9003)nbp
         read(lu,*,err=100) (ir_nresp(k,irchn-2),k=1,nbp)
!         write(*,*) (ir_nresp(k,irchn-2),k=1,nbp)
       enddo
       
! get central wave numbers
!       write(*,*) 'central wave numbers'
       read(lu,'(a80)',err=100) charact
       do while (charact(1:1) .eq. '#')
        read(lu,'(a80)',err=100) charact
       enddo
       backspace lu
       do i=1,4
         read(lu,9005,err=100) it1(i),it2(i),(ir_centwa(k,i),k=1,3)
!         write(*,9005) it1(i),it2(i),(ir_centwa(k,i),k=1,3)
       enddo
! get space view counts limits min,max
       read(lu,'(a80)',err=100) charact
       do while (charact(1:1) .eq. '#')
        read(lu,'(a80)',err=100) charact
       enddo
       backspace lu
       do i=1,nbvischn
         read (lu,*,err=100) sp_min(i),sp_max(i)
!         write(*,*)sp_min(i),sp_max(i)
       enddo
       do i=4,6
         read (lu,*,err=100) sp_min(i),sp_max(i)
!         write(*,*)sp_min(i),sp_max(i)
       enddo
! get  black body view counts limits min,max
       read(lu,'(a80)',err=100) charact
       do while (charact(1:1) .eq. '#')
        read(lu,'(a80)',err=100) charact
       enddo
       backspace lu
       do i=1,3
         read (lu,*,err=100) bb_min(i),bb_max(i)
!         write (*,*,err=100) bb_min(i),bb_max(i)
       enddo
! get black body thermistor counts min,max
       read(lu,'(a80)',err=100) charact
       do while (charact(1:1) .eq. '#')
        read(lu,'(a80)',err=100) charact
       enddo
       backspace lu
       do i=1,4
         read (lu,*,err=100) cth_min(i),cth_max(i)
!         write (*,*,err=100) cth_min(i),cth_max(i)
       enddo
! get visible count where apply the second coefficient set to visible channels
! (cross-over point)
       read(lu,'(a80)',err=100) charact
       do while (charact(1:1) .eq. '#')
        read(lu,'(a80)',err=100) charact
       enddo
       backspace lu
       do i=1,nbvischn
         read (lu,*,err=100) vis_cntlim(i)
!         write(*,*) vis_cntlim(i)
       enddo
! get radiance correction tables
!
       read(lu,'(a80)',err=100) charact
       do while (charact(1:1) .eq. '#')
        read(lu,'(a80)',err=100) charact
       enddo
       backspace lu
         read(lu,9006,err=100) (a_radcor(chn),chn=1,3)
         read(lu,9006,err=100) (b_radcor(chn),chn=1,3)
         read(lu,9006,err=100) (c_radcor(chn),chn=1,3)
!         write(*,9006) (a_radcor(chn),chn=1,3)
!         write(*,9006) (b_radcor(chn),chn=1,3)
!         write(*,9006) (c_radcor(chn),chn=1,3)
! get space radiance
       read(lu,9007,err=100) (rsp(chn),chn=1,3)
!       write(*,9007) (rsp(chn),chn=1,3)
       read(lu,'(a80)',err=100) charact
       do while (charact(1:1) .eq. '#')
        read(lu,'(a80)',err=100) charact
       enddo
       backspace lu
       do j=1,3
         read(lu,*,err=100)  (avh_bandcor(i,j),i=1,3)
!         write(*,*) (avh_bandcor(i,j),i=1,3) 
         planck1(j)=c1*avh_bandcor(1,j)*avh_bandcor(1,j)
     &        *avh_bandcor(1,j)
         planck2(j)=c2*avh_bandcor(1,j)
       enddo    
! end of getting calibration parameters file
       close(lu)
       return
100    continue
!
!      here we meet a fatal error for calibration parameter file read
!      the return code is 2 pointing an i/o error
!
       text='avh_cpar when reading cal parameter file'
       call ml_wti('error',text,ios)
       call aappexit(2)
9000   format('avh_cpar calib. par. file version: ',i2,' year',i5,
     &        ' day',i4)
9001   format(15x,i1)
9002   format(16x,f11.5)
9003   format(19x,i2)
9005   format(i3,3x,i3,5x,f8.3,6x,f8.4,6x,f8.4)
9006   format(20x,f10.7,5x,f10.7,5x,f10.7)
9007   format(20x,f7.4,9x,f7.4,8x,f8.4)
       end
