!+
!NOM/NAME $Id: avh_hdu.F 318 2012-07-05 08:38:01Z frna $
!
        subroutine avh_hdu(avh_cloptn)
!BUT/PURPOSE 
! update header avhrr 1b variables according to the 
! navigation and calibration options
! 
!
!DESCRIPTION 
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
! 
!PROPRIETAIRE/OWNER
!  
!Marcel Derrien
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0        Nov.  95  none          Marcel Derrien
! 1.1        May   98  adaptation to last NOAA 1B format (AAPP verson 3)  
!                      suppress  avh_h_calparid = 'HI'
! 1.2      06/12/01   Correct bug about avh_h_irttcoef(*,*): 
!                     - before only avh_h_irttcoef(1,*) was defined
!                     - modified the scalling factor to be in agreement of
!                       NOAA hrpt.l1b format (note that this modification has
!                       no influence for the other tasks of AAPP 
!                       (atovin, atovpp, ....).         (T.Labrot)
! 1.3      12/5/05    Change 1b definition such that avh_h_radtempcnv contains
!                     the radiance to temp conversions constants, not temp to
!                     radiance. Now consistent with NESDIS 1b.  (N.C.Atkinson)
! 1.4      May 2011   Test avh_h_solarcalyr was not initialised before updating
!  
!ARGUMENTS 
!
!       input : integer avh_cloptn(2)  ! options for avhrcl
!               avh_cloptn(1) -- 1  for AVHRR earth location option
!                                0  no earth location requested
!               avh_cloptn(2) -- 1  for AVHRR calibration option
!                                0  no calibration requested 
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
! 
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!DECLARATIONS
!
! 
!    
!
!
!arguments
       integer avh_cloptn(2)
!commons
#include "avh1b.h"
#include "cavh_cpi.h"
!local variables
       integer chn,i
!functions
!-
!----------------------------------------------
!       
!  to be updated if modification in 1B format
!
       if(avh_cloptn(2).eq.1) then
         avh_h_cpidsyr = versyr
         avh_h_cpidsdy = versdy
         avh_h_callocscnlin = avh_h_scnlin
         
         if (avh_h_solarcalyr.eq.0) then
          avh_h_solarcalyr = avh_h_cpidsyr
          avh_h_solarcaldy = avh_h_cpidsdy
         endif
         
         avh_h_pcalalgind = 0
         avh_h_pcalalgopt = 0
         avh_h_scalalgind = 0
         avh_h_scalalgopt = 0
         do chn = 1, nbvischn 
           avh_h_albcnv ( 1, chn) = nint(vis_isi (chn) * 1.e+01)
           avh_h_albcnv ( 2, chn) = nint(vis_eqwi(chn) * 1.e+03)
         enddo
! channel 3B central wave number    1
!            intercept              2
!            slope                  3
!
! For AAPP v5, store the radiance to temperature conversions in level 1b, for
! consistency with NESDIS. i.e. T = a+bT*, where a=-A/B, b=1/B and A and B are
! the band correction coefs for temp to radiance (T* = A+BT).       

         avh_h_radtempcnv ( 1,1) = nint(avh_bandcor(1,1) * 1.e+02)
         avh_h_radtempcnv ( 2,1) = 
     &         nint( (-avh_bandcor(2,1)/avh_bandcor(3,1)) * 1.e+05 )
         avh_h_radtempcnv ( 3,1) = nint((1./avh_bandcor(3,1)) * 1.e+06)

! channel 4 and 5
         do i = 2,3
           avh_h_radtempcnv ( 1,i) = nint(avh_bandcor(1,i) * 1.e+03)
           avh_h_radtempcnv ( 2,i) = 
     &         nint( (-avh_bandcor(2,i)/avh_bandcor(3,i)) * 1.e+05 )
           avh_h_radtempcnv ( 3,i) = nint((1./avh_bandcor(3,i)) 
     &                               * 1.e+06)
         enddo
! IR target temperature conversion coefficients
         do i=1,4
           avh_h_irttcoef(1,i) = nint(conv_prt(1,i) * 1.e+02)
           avh_h_irttcoef(2,i) = nint(conv_prt(2,i) * 1.e+05)
           avh_h_irttcoef(3,i) = nint(conv_prt(3,i) * 1.e+08)
           avh_h_irttcoef(4,i) = nint(conv_prt(4,i) * 1.e+11)
           avh_h_irttcoef(5,i) = nint(conv_prt(5,i) * 1.e+14)
           avh_h_irttcoef(6,i) = nint(0.* 1.e+17)
         enddo           
       endif
       if(avh_cloptn(1).ne.0) then
         call hd1bnav(avh_h_modelid,
     & avh_h_nadloctol,avh_h_locbit,avh_h_yawerr,
     & avh_h_rollerr,avh_h_pitcherr,avh_h_epoyr,avh_h_epody,
     & avh_h_epotime,avh_h_smaxis,avh_h_eccen,avh_h_incli,
     & avh_h_argper,avh_h_rascnod,avh_h_manom,avh_h_xpos,
     & avh_h_ypos,avh_h_zpos,avh_h_xvel,avh_h_yvel,avh_h_zvel,
     & avh_h_earthsun)
       endif

        return
        end
           

