      subroutine avh_lico
!+
!NOM/NAME $Id: avh_lico.F 25 2006-02-03 15:43:53Z frna $
! avh_lico
!
!BUT/PURPOSE 
! complete thecalibration coefficients for start and end of orbit  
! linearize the calibration coefficients between two successive sub-blocks
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER 
! Marcel Derrien
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Sept. 95      no             Marcel Derrien
! 2.0    13 Nov. 01  All the cases of ch3 changes between 2 successive sub-blocks 
!                    were not specified. The local variable ind_sb was not initialised.
!                    So with some compilaters, sb_k1(2 or 3)(1,ind_sb) could cause 
!                    an error          (Tiphaine Labrot) 
! 2.1    14 Apr 05   check last_sb.ge.1 before using it as array index. NCA
!
!         
!ARGUMENTS
!
!       
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!DECLARATIONS
!
!    
     
!commons
#include "avhrcl.h"
#include "avh1b.h"
#include "cavh_wrk.h"
#include "cavh_m1b.h"
!local variables
      integer sblock
      integer first_sb
      integer last_sb
      integer ind_sb
      integer i,k,istrt
      integer scn,scn_beg
      real sumk
      logical found,ended
      logical ch3a_to_ch3b,unknown_to_ch3b
      logical ch3b_to_ch3a,ch3b_to_unknown
      logical unknown_to_unknown,ch3a_to_ch3a
      logical unknown_to_ch3a,ch3a_to_unknown
!-

!
! complete sub-blocks arrays when data are missing 
! for starting and ending sub-blocks only
!
       sblock = 0
       first_sb = 0
       found=.false.
       ind_sb=0
100    continue
         sblock = sblock + 1
         sumk=0.
         do i=1 , 3
           sumk = abs(sb_k1(i,sblock)) + sumk
           sumk = abs(sb_k2(i,sblock)) + sumk
           sumk = abs(sb_k3(i,sblock)) + sumk
         enddo
         if(sumk.gt.1.e-05) then
! data is not missing 
           first_sb = sblock
           found = .true.
         endif
         ended = sblock .eq. nb_sblock
       if(.not.found.and..not.ended) go to 100

       sblock =  nb_sblock + 1
       last_sb = 0
       found = .false.
200    continue
         sblock = sblock - 1
         sumk=0.
         do i=1 , 3
           sumk = abs(sb_k1(i,sblock)) + sumk
           sumk = abs(sb_k2(i,sblock)) + sumk
           sumk = abs(sb_k3(i,sblock)) + sumk
         enddo
         if(sumk.gt.1.e-05) then
! data is not missing 
           found = .true.
           last_sb = sblock
         endif
         ended = sblock .eq. 1
       if(.not.found.and..not.ended) go to 200
!
!      complete the arrays for all coefficients if 
!      ch3 does not change
!
         if(first_sb.gt.1) then
           do sblock = 1, first_sb - 1
             if(ch3_val(sblock).eq.ch3_val(first_sb).and.
     &         ch3_val(sblock).eq.ch3b) then 
                istrt=1
             else
                istrt=2
             endif      
             do i = istrt,3
               sb_k1(i,sblock)= sb_k1(i,first_sb)
               sb_k2(i,sblock)= sb_k2(i,first_sb)
               sb_k3(i,sblock)= sb_k3(i,first_sb)
             enddo
           enddo
         endif

         if(last_sb.ge.1.and.last_sb.lt.nb_sblock) then
           do sblock = last_sb, nb_sblock
             if(ch3_val(sblock).eq.ch3_val(last_sb).and.
     &         ch3_val(sblock).eq.ch3b) then 
                istrt=1
             else
                istrt=2
             endif
             do i = istrt,3
               sb_k1(i,sblock)= sb_k1(i,first_sb)
               sb_k2(i,sblock)= sb_k2(i,first_sb)
               sb_k3(i,sblock)= sb_k3(i,first_sb)
             enddo
           enddo
         endif
!
!        performs linearization of the coefficients
!
         do scn = 1 , start_cal_seq + 2
           if(ch3_ind(scn).eq.ch3b.and.
     &         ch3_val(1).eq.ch3b) then
              istrt=1
            else 
               istrt=2
            endif
           do i = istrt,3
             calir(1,i,scn)=sb_k1(i,1)
             calir(2,i,scn)=sb_k2(i,1)
             calir(3,i,scn)=sb_k3(i,1)
           enddo
         enddo
!
!        main loop for central sub-blocks
!
         do sblock = 2 , nb_sblock
           if(ch3_val(sblock).eq.ch3_val(sblock-1).and.
     &         ch3_val(sblock).eq.ch3b) then 
! istrt=1 <==> 3B in sblock-1 and  in sblock  <==> 3B -> 3B   
             istrt=1
           else
! istrt=2 could be: 3A -> 3A       unknown -> unknown 
!                   3A -> 3B       unknown -> 3B 
!                   3B -> 3A       unknown -> 3A 
!                   3B -> unknown  3A -> unknown        
             istrt=2
           endif
!
!            performs linearization between two 
!            successive sub-blocks
! 
           scn_beg = start_cal_seq + (sblock - 2)*5 +3
           k = 0
           do scn = scn_beg ,scn_beg + 4 
             do i = istrt,3
               calir(1,i,scn) = sb_k1(i,sblock-1)
     &          + k*(sb_k1(i,sblock)-sb_k1(i,sblock-1))/5.
               calir(2,i,scn) = sb_k2(i,sblock-1)
     &          + k*(sb_k2(i,sblock)-sb_k2(i,sblock-1))/5.
               calir(3,i,scn) = sb_k3(i,sblock-1)
     &          + k*(sb_k3(i,sblock)-sb_k3(i,sblock-1))/5.
             enddo
             k = k + 1
           enddo
           if(istrt.eq.2) then
!             special look to channel 3
             ch3a_to_ch3b = ch3_val(sblock).eq.ch3b.and.
     &                    ch3_val(sblock-1).eq.ch3a
             unknown_to_ch3b = ch3_val(sblock).eq.ch3b.and.
     &                    ch3_val(sblock-1).eq.0 
             ch3b_to_ch3a = ch3_val(sblock).eq.ch3a.and.
     &                    ch3_val(sblock-1).eq.ch3b
             ch3b_to_unknown = ch3_val(sblock).eq.0.and.
     &                    ch3_val(sblock-1).eq.ch3b
             unknown_to_unknown = ch3_val(sblock).eq.0.and.
     &                    ch3_val(sblock-1).eq.0
             ch3a_to_ch3a = ch3_val(sblock).eq.ch3a.and.
     &                    ch3_val(sblock-1).eq.ch3a
             ch3a_to_unknown = ch3_val(sblock).eq.0.and.
     &                    ch3_val(sblock-1).eq.ch3a
             unknown_to_ch3a = ch3_val(sblock).eq.ch3a.and.
     &                    ch3_val(sblock-1).eq.0
     

! If ch3_val(sblock).eq.ch3a, that means that all the lines of the block
! sblock contain channel 3a data ==> no  ch3b coef of calib  ==> ind_sb = 0
             if(unknown_to_ch3a.or.ch3a_to_ch3a.or.ch3b_to_ch3a) then
              ind_sb = 0
             endif
! If ch3_val(sblock).eq.0, that means that some lines of the block
! sblock can contain channel 3b data ==> ind_sb = sblock
! But when ch3_val(sblock).eq.0, no ch3b calib coef have been conputed
! ==> sb_k1,k2,k3 =0            
             if(unknown_to_unknown.or.ch3a_to_unknown) ind_sb = sblock 
!
             if(unknown_to_ch3b.or.ch3a_to_ch3b) then
              ind_sb = sblock
             endif
             
             if(ch3b_to_unknown) ind_sb = sblock-1 


             do scn = scn_beg ,scn_beg + 4 
              if(ind_sb.ne.0) then
               if(ch3_ind(scn).eq.ch3b) then
                 calir(1,1,scn)= sb_k1(1,ind_sb)                  
                 calir(2,1,scn)= sb_k2(1,ind_sb)                  
                 calir(3,1,scn)= sb_k3(1,ind_sb) 
               endif
              endif 
             enddo 
           endif                
         enddo
!
!        terminating scans
!
         scn_beg = start_cal_seq + (nb_sblock - 1)*5 + 3
                  
         do scn = scn_beg , avh_h_scnlin 
           if(ch3_ind(scn).eq.ch3b.and.
     &         ch3_val(1).eq.ch3b) then
              istrt=1
            else 
               istrt=2
            endif
           do i = istrt,3
             calir(1,i,scn)=sb_k1(i,nb_sblock)
             calir(2,i,scn)=sb_k2(i,nb_sblock)
             calir(3,i,scn)=sb_k3(i,nb_sblock)
           enddo
         enddo
         return
         end
                        

            
         
         
 

