      subroutine avh_put(avh_cloptn,ios)
!+
!NOM/NAME $Id: avh_put.F 477 2016-02-08 14:57:45Z frna $
! avh_put
!
!BUT/PURPOSE 
! avh_get rewrites the hrpt 1B file (only a part of each record )
! this module depends on the 1B format structure and has to be updated if
! the format changes
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER 
! Marcel Derrien
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Sept. 95      no             Marcel Derrien
! 1.1      March 96      add typorb     Marcel Derrien
! 1.2      December 96   rewrite full record (MD)
! 1.3      May 98        new scaling factors for 1B cal coefficients
!                        arguments of nav_1blin modified
! 1.4      Jan 99        change scale3 from 10**6 to 10**9 and add nint   
! 1.5      Apr 99        stores the exponent of scaling coefs in avh_filler2
! 1.6      Oct 99        corection of bug. misplacement of avh_filler2
! 1.7      Dec 00        change message for clock error (PB)
! 1.8      17/08/2006    Change integer Z'xxxx' to Z'0000xxxx'. NCA 
! 1.9      08/02/2016    Add NEdT calculation.
!
!ARGUMENTS
!       input : integer avh_cloptn(2)
!       input : integer ios ! io status
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!DECLARATIONS
!
!    
!arguments
        integer avh_cloptn(2)
	integer ios ! io status 
!commons
#include "avhrcl.h"
#include "avh1b.h"
#include "cavh_m1b.h"
#include "cavh_cpi.h"
#include "c_1bnav.h"
#include "cavh_wrk.h"
!local variables
       integer expon3 ! exponent of scaling factor for 1st IR cal. coeff
       integer expon4 ! exponent of scaling factor for 2nd IR cal. coeff
       integer expon5 ! exponent of scaling factor for 3rd IR cal. coeff
       integer pix
       integer scn
       integer i
       integer recn
       integer chn
       integer iset
       integer icoef
       integer firstir
       integer c, nok, nedt_int 
       real scale1,scale2,scale3,scale4,scale5
       integer iqscn(3)
       integer jdate
       integer*2 eight
       parameter (eight=8)
       double precision ijscn
       double precision ijstrt
       real wsum, wmean, wsumsq, wsd, radsd, expn, dbdt(3)
       logical clock_cor_ind
       logical first_chn_change
       logical first_chn
       save
!functions
       integer*2 gp_gb16       
!-
!---------------------------
! loop for all scans writing
!---------------------------
       ios =0  
       first_chn_change=.true.
       first_chn=.true.
!
!      update the 1B variables before rewriting the data
!
!      visible coefficients only prelaunch coefficients are 
!      set and used for all the AVHRR scans of the data set
!
       scale1 = 1.e+10
       scale2 = 1.e+07
!
!     modification to correct lack of accuracy of scaling factors 
!     proposed by NOAA   ( M. Derrien 26/01/1999)
!     must be considered only for a transition, because we
!     do not change the l1bversnb number due to CMS consideration
!     and we don't have the new NESDIS scaling factors 
!     this will change scaling factors only for 1b files 
!     whose data are newer than 27 January 1999,12UTC date 
!     where changed at CMS (Keep old ones for running archive)
!  
!     Now avh_filler2(3) stores the 3 exponents of the scaling
!     coefficients. Subroutine avh_lbc will be able to read the
!     new exponents and will suppose exponent 6 for all archived
!     files. Beware of reading these new files without taking into
!     account the filler2 array
!     The fillers will be updated after reading the line (correction
!     of bug Oct. 12th 1999.
!     
      if (avh_h_siteid.eq.'CMS' .and. avh_h_startdatajd.lt.17923.5)then
         !keep old exponent for archive at CMS
         !new run will give same result as archive
         expon3=6
         expon4=6
         expon5=6
      else
         !new exponent values
         expon3=9
         expon4=6
         expon5=6
      endif

      scale3 = 10. ** expon3
      scale4 = 10. ** expon4
      scale5 = 10. ** expon5
       
      call ml_wti('info',
     &    'IR calibration exponent of scaling coefficient 1', expon3)
      call ml_wti('info',
     &    'IR calibration exponent of scaling coefficient 2', expon4)
      call ml_wti('info',
     &    'IR calibration exponent of scaling coefficient 3', expon5)


       if(avh_cloptn(2).ne.0) then
         do chn = 1 , nbvischn
           do iset = 1 , 2
             do icoef = 1 , 5
               avh_calvis(icoef,iset,chn)=0
             enddo
           enddo
           avh_calvis(1,3,chn)=nint(vis_slope(chn,1)*scale1)
           avh_calvis(2,3,chn)=nint(vis_int(chn,1)*scale2)
           avh_calvis(3,3,chn)=nint(vis_slope(chn,2)*scale1)
           avh_calvis(4,3,chn)=nint(vis_int(chn,2)*scale2)
           avh_calvis(5,3,chn)=vis_cntlim(chn)
         enddo
       endif
       iqscn(1)=scnlinyr(1)
       iqscn(2)=scnlindy(1)
       iqscn(3)=scnlintime(1)
       call xiqj(iqscn,ijstrt)
!
!      check if clock error not already applied 
!      if not we must correct
!      the current scan line times when computing earth location
!      ---------------------------------------------------------
       if(gp_gb16(scnlinbit(1),14).eq.0) then
         ijstrt=ijstrt-errclock/86400.
         clock_cor_ind=.true.
         call ml_wt('info','clock error correction is valid')
         call ml_wt('info','datation will be corrected in this run')
       else
         clock_cor_ind=.false.
         call ml_wt('info','times are already corrected')
       endif

       do chn=1,3   !compute gradient of the Planck function at 300K
                    !planck1 is C1 * w**3, planck2 is C2 * w
         expn = exp(planck2(chn)/300.0)
         dbdt(chn) = planck1(chn)*expn*planck2(chn)/300.0**2/(expn-1)**2
       enddo

!      -----------------------------           
!      loop on the avhrr scan lines  
!      -----------------------------           
       do scn = 1 , avh_h_scnlin
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
         call avhdtr(luavh,scn,ios)
         if(avh_cloptn(2).ne.0) then
           do chn = 1 , nbvischn
             do iset = 1 , 2
               do icoef = 1 , 5
                 avh_calvis(icoef,iset,chn)=0
               enddo
             enddo
             avh_calvis(1,3,chn)=nint(vis_slope(chn,1)*scale1)
             avh_calvis(2,3,chn)=nint(vis_int(chn,1)*scale2)
             avh_calvis(3,3,chn)=nint(vis_slope(chn,2)*scale1)
             avh_calvis(4,3,chn)=nint(vis_int(chn,2)*scale2)
             avh_calvis(5,3,chn)=vis_cntlim(chn)
           enddo
         endif
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
         recn = scn + avh_h_hdrcnt
!
!        navigation option
!        -----------------
         if(avh_cloptn(1).ne.0) then

           if(clock_cor_ind) then
!          -------------------------------
!          clock error correction to apply
!          and flag bit 14 in scnlinbit
!          -------------------------------
             iqscn(1)=scnlinyr(scn)
             iqscn(2)=scnlindy(scn)
             iqscn(3)=scnlintime(scn)
             call xiqj(iqscn,ijscn)
             ijscn=ijscn-errclock/86400.
             call xijq(ijscn,iqscn)
             scnlinyr(scn) = iqscn(1)
             scnlindy(scn) = iqscn(2)
             scnlintime(scn) = iqscn(3)
             call gp_sb16(scnlinbit(scn),14,1)
             clockdrift(scn)=1000.*errclock
          endif
         
           if(mod(scn-first_lnav,step_lnav).eq.0) then
!            --------------------------------
!            the scan line is to earth locate 
!            --------------------------------
	     call nav_1blin(ijstrt, 3, scn,
     &		scnlinbit, qualind,
     &		attangtime , rollang , pitchang , yawang,
     &		navstat ,
     &		scalti , avh_ang , avh_pos )
            else
!             ---------------------------------------------
!             this scan is not located
!             we repeat the value for acs. desc. indicator
!             of the last scan line
!             ---------------------------------------------
              if(vel(3).ge.0.d0) then
                call gp_sb16(scnlinbit(scn),15,0)
                typorb='a'
              else
                call gp_sb16(scnlinbit(scn),15,1)
                typorb='d'
              endif  
!             ----------------------------------------------------------
!             set earth location bit to 1 in quality indicator bit field
!             ----------------------------------------------------------
              call gp_sb32(qualind(scn),27,1)
              attangtime(scn) = 0        
              rollang(scn) = 0
              pitchang(scn)= 0
              yawang(scn) = 0
              navstat(scn) = 0
!             --------------------------------------------------------
!             set no earth location bit in navigation status bit field
!             --------------------------------------------------------
              call gp_sb32(navstat(scn),13,1) 
              scalti(scn) = 0
              do pix = 1, 51
                 avh_ang(1,pix) = 0
                 avh_ang(2,pix) = 0
                 avh_ang(3,pix) = 0
                 avh_pos(1,pix) = 0
                 avh_pos(2,pix) = 0
              enddo
            endif
          endif
!
!          calibration option
!          ------------------
          if(avh_cloptn(2).ne.0) then          
!            avh_filler number 2 contains the exponent
!            values for the calibration coefficients
!            -----------------------------------------
             avh_filler2(1) = expon3
             avh_filler2(2) = expon4
             avh_filler2(3) = expon5
!          update IR coefficients only for the operational set
!             we do not maintain prelaunch set today
!          ----------------------------------------------------
           do chn = 1 , 3
!            operational set
!            ---------------
              iset = 1 
               avh_calir(1,iset,chn) = nint(calir (1,chn,scn) * scale3)
               avh_calir(2,iset,chn) = nint(calir (2,chn,scn) * scale4)
               avh_calir(3,iset,chn) = nint(calir (3,chn,scn) * scale5)

!          second set coefficients
!          -------------------------
              iset = 2 
               avh_calir(1,iset,chn) = nint(k1mean(chn) * scale3)
               avh_calir(2,iset,chn) = nint(k2mean(chn) * scale4)
               avh_calir(3,iset,chn) = nint(k3mean(chn) * scale5)
           enddo
!
!         set ch3 bit indicators
!
           if(ch3_ind(scn).eq.ch3a) then
!            -------------
!            flag ch3a bit
!            -------------
             call gp_sb16(scnlinbit(scn),0,0)
             call gp_sb16(scnlinbit(scn),1,0)
             if(first_chn) then
               call ml_wt('info',
     &                'avhrr scene starts with ch3a')
               avh_h_inststat1=z'0000fb00'
               first_chn=.false.
             endif
           else if(ch3_ind(scn).eq.ch3b) then
!            -------------
!            flag ch3b bit
!            -------------
             call gp_sb16(scnlinbit(scn),0,1)
             if(first_chn) then
               call ml_wt('info',
     &                'avhrr scene starts with ch3b')
               avh_h_inststat1=z'0000f700'
               first_chn=.false.
             endif             
           endif
           if(ch3_change) then
             do i = 1, nb_ch3
               if (scn.eq.scn_new_chn(i)) then
!
!              flag transition bit
!              -------------------
                 call gp_sb16(scnlinbit(scn),0,1)
                 call gp_sb16(scnlinbit(scn),1,1)
                 if(first_chn_change) then
                   if(avh_h_inststat1.eq.z'0000f700') then
                     avh_h_inststat2=z'0000fb00'
                     call ml_wti('info',
     &                'avhrr ch3b becomes ch3a at',scn) 
                   else
                     call ml_wti('info',
     &                'avhrr ch3a becomes ch3b at',scn) 
                     avh_h_inststat2=z'0000f700'
                   endif
                   avh_h_statchrecnb=scn
                   first_chn_change=.false.
                 else
                   call ml_wti('warning',
     &                'avhrr ch3 changes again at',scn)                    
                 endif               
               endif
             enddo
           endif
         endif
!
!        output to 1B file
!        -------------------
         avh_scnlin = scnlin(scn)
         avh_scnlinyr = scnlinyr(scn)
         avh_scnlindy = scnlindy(scn)
         avh_scnlintime = scnlintime(scn) 
         avh_clockdrift = clockdrift(scn)
         avh_scnlinbit = scnlinbit(scn)
         avh_qualind = qualind (scn)
         avh_scnlinqual = scnlinqual(scn)
         do i = 1,avh_mxirchn
           avh_calqual(i)=calqual(i,scn)
         enddo
         avh_cbiterr = cbiterr(scn)
         if(avh_cloptn(1).ne.0) then
!           type 9000,scnlinbit(scn),qualind(scn),navstat(scn),
!     &       avh_h_locbit,scn
!9000    format('scbit,qualind,navst,locb ',z4.4,1x,z8,2(1x,z4,1x)
!     & ,'scn line ',i5)
           avh_attangtime = attangtime(scn)
           avh_rollang = rollang(scn)
           avh_pitchang = pitchang(scn)
           avh_yawang = yawang(scn)
           avh_navstat = navstat (scn) 
           avh_scalti = scalti(scn)
         endif 
!
!        Compute NEdT for IR channels
!        ----------------------------
!        Uses standard deviation of warm counts for each line, as computed by
!        EUMETSAT in the EPS PFS product. Precision is limited by the
!        10-bit resolution of AVHRR. Stored in avh_filler3 which is defined as
!        in the PFS CALIBRATION_QUALITY flag.
!
         if (ch3_ind(scn).eq.ch3b) then
           firstir = 1
         else
           firstir = 2
         endif
         do chn=firstir,3
           if (calir(2,chn,scn) .lt. 0) then   !expect negative slope
             wsum = 0.0
             nok = 0
             do i=1,10
               c = avh_telem(3*i+chn+19)
               if (c .ge. bb_min(chn) .and. c .le. bb_max(chn)) then
                 wsum = wsum + avh_telem(3*i+chn+19)
                 nok = nok + 1
               endif
             enddo
             if (nok .gt. 1) then
               wmean = wsum / nok
               wsumsq = 0.0
               do i=1,10
                 c = avh_telem(3*i+chn+19)
                 if (c .ge. bb_min(chn) .and. c .le. bb_max(chn)) then
                   wsumsq = wsumsq + (avh_telem(3*i+chn+19)-wmean)**2
                 endif
               enddo
               wsd = sqrt(wsumsq / (nok-1))
               radsd = wsd*abs(calir(2,chn,scn) + 
     &                         2*wmean*calir(1,chn,scn))
               nedt_int = (radsd/dbdt(chn) * 100.0) + 0.5         !scale 100
             else
               nedt_int = 255    !no good readings
             endif
             avh_filler3(chn) = avh_calqual(chn) + nedt_int*256 !set bits 8-15
             if (nedt_int .gt. 12) then   !Spec is 0.12K for all channels
               avh_filler3(chn) = iand(avh_filler3(chn),eight)
             endif
           endif
         enddo
!
!        Write to output file
!        --------------------
         call avhdtw(luavh,scn,ios)
         if(ios.ne.0) then
           return
         endif
       enddo	
!      --------------------------------------------------
!      update header times according to the clock_cor_ind
!      --------------------------------------------------
       if(clock_cor_ind) then
         iqscn(1)=scnlinyr(1)
         iqscn(2)=scnlindy(1)
         call xdqj(iqscn,jdate)
         avh_h_startdatajd = jdate 
         avh_h_startdatayr = scnlinyr(1)
	 avh_h_startdatady = scnlindy(1)
         avh_h_startdatatime = scnlintime(1)
         iqscn(1)=scnlinyr(avh_h_scnlin)
         iqscn(2)=scnlindy(avh_h_scnlin)
         call xdqj(iqscn,jdate)
         avh_h_enddatajd = jdate 
	 avh_h_enddatayr = scnlinyr(avh_h_scnlin)
	 avh_h_enddatady = scnlindy(avh_h_scnlin)
         avh_h_enddatatime = scnlintime(avh_h_scnlin)
       endif
       return
       end
     
