      subroutine avh_qc
!+
!NOM/NAME $Id: avh_qc.F 428 2014-12-02 13:33:56Z frna $
! avh_qc 
!
!BUT/PURPOSE 
! checks the quality of each AVHRR scan line from cavh_m1b.h
! flags the line having bad scan numbers inside 1B file
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER 
! Marcel Derrien
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment                  <author>)
! 1.0      Nov. 95      no                     Marcel Derrien
! 1.1      Oct. 97    allow noisy thermistors  Marcel Derrien 
! 1.2      Jun. 06    change nb_sblock         P. Marguinaud
! 1.3      Dec. 14    To accommodate noisy data at start, require 4
!                     consecutive scans with PRT readings before
!                     setting start_cal_seq. N. Atkinson 
!
!         
!ARGUMENTS
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!DECLARATIONS
!
!    
!arguments
!commons
#include "avhrcl.h"
#include "avh1b.h"
#include "cavh_m1b.h"
#include "cavh_wrk.h"
!local variables
      integer scn
      integer i
      integer th
      character*80 text
      integer nbgoodscans
      integer count_notcal

      logical found
!-
      found = .false.
      nbgoodscans = 0
      count_notcal = 0
      do scn=1,avh_h_scnlin
        if(scnlin(scn).ne.scn) then
!-------------------------------
! check the scan line number
! then set bits indicating this scan is not usable
! correct the scan line number
!-------------------------------
          call gp_bse(qualind(scn),31)
          scnlin(scn)=scn
        else
!---------------------------------------------
! find the first calibration sequence position
!---------------------------------------------
          nbgoodscans = nbgoodscans + 1
          th=0
          do i=1,3
            th=thcnt(i,scn)+th
          enddo
          
!Dec 2014: require 4 consecutive scans with PRT readings

          if (.not. found) then
            if (th.gt.20) then
              count_notcal = count_notcal + 1
            else
              if (count_notcal.ge.4) then
                found=.true.
                text=' avh_qc first calib sequence flag on'
                call ml_wti('info',text,scn)
                start_cal_seq = MOD(scn-1,5)+1
              else
                count_notcal = 0
              endif
            endif
          endif
        endif
      enddo
      nb_sblock = (avh_h_scnlin - start_cal_seq + 1)/5
      if(.not.found) then
        call ml_wt('error',
     &       'avh_qc no start condition to calibrate')

        call ml_wti('error',
     &       'avh_qc nb good scans',nbgoodscans) 
        call aappexit(10) 
      endif     
!---------------------
! check ch3 transitions
!---------------------
      call chkch3
!-
      return
      end
