      subroutine avhsetu(avh_cloptn)
!+
!NOM/NAME $Id: avhsetu.F 109 2008-01-30 10:14:34Z frna $
! avhsetu 
!
!BUT/PURPOSE 
!  open files for avhrcl
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER 
! Marcel Derrien
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Sept. 95      no             Marcel Derrien
! 1.1      July 98  add nblin variable for ml_wti compatibility
! 2.0      August00 Add a test on ierr, output of sp_read (TL)
! 2.1      Dec 00   Display attitude values  P. Brunel
! 2.1      Dec 00   Display correct value of clock error when file
!                    already been located.  P. Brunel
! 2.2      Mar 04   Modify for linux. N Atkinson / A Korpela
! 2.3      Jul 04   Upgrade to navigation code version 2.
!                   def_att call changed for arguments and position in
!                   code (after sp_read)     P. Brunel
! 2.4      Jun 05   Change step_lnav from 20 to 1 at CMS recommendation. NCA
! 2.5      Jan 08   Modify text of info message for satid. NCA
!         
!ARGUMENTS
!
!       output : integer avh_cloptn(2)  ! options for avhrcl
!               avh_cloptn(1) -- 1  for AVHRR earth location
!               avh_cloptn(2) -- 1  for AVHRR calibration
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!DECLARATIONS
!
!    
!arguments
        integer avh_cloptn(2)       ! options for avhrcl
!commons
#include "avh1b.h"
#include "avhrcl.h"
#include "cavh_wrk.h"
#include "c_1bnav.h"
#include "reclopt.h"    
!local variables
        integer ios
        integer ierr
        character*80 txt
        character*6 scname
        double precision ijstrt,ijend
        integer loc_rl
        integer nblin
        integer   i4_avh_clockdrift      
        integer att_mode        ! attitude mode
                                ! 0 = NOAA POES operational mode
                                ! 1 = METOP Yaw steering mode
                                ! 2 = Geocentric mode
!functions
       integer ord1bid
       character*6 noascnam
!-------------------
! open avhrr 1B file
!-------------------
      loc_rl=reclavh/rldiv
      open(luavh,form='unformatted',access='direct',
     &     recl=loc_rl,status='old',iostat=ios)

!     If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
      if(ios.ne.0)
     &  open(luavh,form='unformatted',access='direct',
     &     recl=loc_rl,status='unknown',iostat=ios)

      if(ios.ne.0) then
        call ml_wti('info',' avhsetu iostat = ',ios)
        write(txt,*)'avhrr 1B File could not be opened ',ios,
     &              ' on unit',luavh
        call ml_wt('error',txt)
        call aappexit(2)
      endif
!-------------------
! read 1B header
!-------------------
      ios=0 
      call avhhdr(luavh,ios) 
      if(ios.ne.0) then
        call ml_wti('error','avhsetu reading 1B avhrr header',ios)
        call aappexit(2)
      endif
      if(avh_h_scnlin.gt.avh_mxscn) then
        nblin=avh_h_scnlin
        call ml_wti('info','too many avhrr scan lines',nblin)
        call ml_wti('error','modify maximum from avhrcl.h',avh_mxscn)
        call aappexit(2)
      endif
      satid = avh_h_satid
      satid = ord1bid (satid,avh_h_startdatajd)
      scname=noascnam(satid)
      call ml_wti('info','order number',satid)
!----------------------------------
! calibration option
! -read  parameter calibration file
! -open statistics file
!----------------------------------
      if(avh_cloptn(2).eq.1) then
        call avh_cpar(lupar,satid)
        open(lustat,form='formatted',access='sequential',
     &     status='new',iostat=ios)

!       If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
        if(ios.ne.0)
     &  open(lustat,form='formatted',access='sequential',
     &     status='unknown',iostat=ios)

        if(ios.ne.0) then 
          call ml_wti('error','avhsetu opening calib. stat file'
     &           ,ios)
          call aappexit(2)
        endif 
      endif
!----------------------
! earth location option
!----------------------
      if(avh_cloptn(1).eq.1) then

!       opens and reads the SATPOS file
!       -----------------------------------
        open (unit=lusatpos,status='old',iostat=ios)

!       If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
        if(ios.ne.0) open (unit=lusatpos,status='unknown',iostat=ios)

        if( ios.ne.0 ) then
          call ml_wti('error',
     &       'while opening the satpos file I/O status',ios)
          call ml_wt('error','avhsetu aborted')
          call aappexit(2)
        endif

!       Read the satpos file
!       --------------------
        ijstrt=avh_h_startdatajd + 
     &      (avh_h_startdatatime-errclock*1000.)/86400.d3
        ijend=avh_h_enddatajd + 
     &       (avh_h_enddatatime-errclock*1000.)/86400.d3
        call sp_read(ijstrt,ijend,lusatpos,scname,ierr)
        close(unit=lusatpos,iostat=ios)
        
        if(ierr.ne.0) then
          call ml_wti('error',
     &    'return code of sp_read differ from 0',ierr)
          call ml_wt('error','avhsetu aborted')
          call ml_wt('error','navigation aborted')
          call ml_wt('error','but calibration can work')
          call ml_wt('error','try calibration alone')
          call aappexit(2)
        endif

        if(yaw.lt.-99.d-03) then
          call def_att(scname,att_mode,yaw,roll,pitch)
          call ml_wt('info','take default attitude values')
          call ml_wti('info','default attitude mode ',att_mode)
          call ml_wtdp('info','default attitude yaw ',yaw)
          call ml_wtdp('info','default attitude roll ',roll)
          call ml_wtdp('info','default attitude pitch ',pitch)
        else
          call ml_wtdp('info','attitude yaw ',yaw)
          call ml_wtdp('info','attitude roll ',roll)
          call ml_wtdp('info','attitude pitch ',pitch)
        endif
        call calatt(yaw,roll,pitch,att)

!       if the file has already been located we do not correct
!       again from the clock error
!       --------------------------
        errclock = 0
        if(avh_h_epoyr.le.1950) then
          open (unit=luclock,status='old',iostat=ios)

!         If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
          if(ios.ne.0) open (unit=luclock,status='unknown',iostat=ios)

          if(ios.ne.0) then
            call ml_wti('error',
     &       'opening clock error file I/O status',ios)
            call aappexit(2)
          endif
          ijstrt=avh_h_startdatajd + 
     &      (avh_h_startdatatime)/86400.d3
          call clkerr_get(luclock,ijstrt,errclock,ierr)
          close(unit=luclock,iostat=ios)
          if(ierr.ne.0.or.abs(errclock).gt.2.) then
            call ml_wti('warning',
     &       'reading clock error value from clkerr_get ',ierr)
            call ml_wtr('warning','clock error from file',
     &                 errclock)
            call ml_wt('warning','clock error forced to 0')
            errclock=0 
          else
             call ml_wtr('info','clock error value (sec)',errclock)
          endif
        else
          errclock=0.
          call ml_wt('info',
     &     '1B file already corrected from clock error')
          ! read first line to get the value of clock drift
          ! which was applied
          Call avhdtr(luavh, 1, ios)
          i4_avh_clockdrift = avh_clockdrift
          Call ml_wti('info',
     &       'scan time correction for clock drift  (msec) ',
     &       i4_avh_clockdrift) 
        endif
        !call ml_wtr('info','clock error value (sec)',errclock)

!       init earth location parameters
!       ------------------------------
        first_pnav = 25
        first_lnav = 1
        step_pnav = 40
        step_lnav = 1      !was 20 in AAPP-4
        nb_pnav = 51
      endif
      return
      end
