      subroutine chkch3
!+
!NOM/NAME $Id: chkch3.F 47 2006-08-21 08:44:41Z frna $
! chkch3
!BUT/PURPOSE
! determines which of channels 3a or 3b is active for every 
! AVHRR scan line of the 1B file
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER 
! Marcel Derrien
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Nov. 95      no             Marcel Derrien
! initial algortihm from NOAA preprocessor code
! 1.1      May 98      correct bug with test of indicator_bit
! 1.2      Apr 05      Accept NESDIS data. Nigel Atkinson
! 1.3      May 06      Change test for noaak and extend for metop PM/PB
!         
!ARGUMENTS
!
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none
!  
!commons
#include "avhrcl.h"
#include "avh1b.h"
#include "cavh_m1b.h"
#include "cavh_wrk.h"
!local variables
      integer  ch3
      integer  scn

      integer indicator_bit    ! channel 3 indicator value
!                                           ch3a --> channel 3a
!                                           ch3b --> channel 3b
      integer frst_known_ch3  ! first scan line with a known ch3 indicator
      integer frst_ch3_ind     ! first known ch3 indicator
      integer i ,k             ! do loop index.
      integer sp_ch3a,sp_ch3b  ! counter for space counts indicating 
!                                channel 3A or channel 3B
      integer start_scn
      logical bit_flag         ! bit test flag for channel 3 status
      logical ch3a_flag        ! flag set true if space observation
!                              indicates that data is from channel 3a.
      logical ch3b_flag        ! flag set true if space observation
!                              indicates that data is from channel 3b.
      logical ended            ! boolean to stop searhing 
      character*80 text
      integer id_noaak          ! noaa id corresponding to Noaa-K == 15
      data id_noaak/15/
      character*6 scname
      integer id_in             ! noaa id corresponding to satid variable
!  
!functions
      character*6 noascnam
      integer noascid
      logical gp_btn
!-
!---------------------------------------
!     main loop on every avhrr scan line
!---------------------------------------
      frst_ch3_ind = 0
      do scn = 1 , avh_h_scnlin
        ch3 = 0
        ch3a_flag = .false.
        ch3b_flag = .false.
        if ( .not. gp_btn( qualind(scn),31) ) then
!--------------------------------------------------------------
!     analyze space counts to determine if data is from channel
!     3a or 3b.  channel 3a data has counts less than 100.
!     data from channel 3b has counts more than 900.
!--------------------------------------------------------------
          sp_ch3b = 0
          sp_ch3a = 0
          do  i = 1, 10
            if (ispcnt (3, i, scn ) .gt. 900) then
              sp_ch3b = sp_ch3b + 1
            else if (ispcnt (3, i, scn ) .lt. 100) then
              sp_ch3a = sp_ch3a + 1
            endif
          enddo
          if( sp_ch3b .ge. 5) ch3b_flag = .true.
          if( sp_ch3a .ge. 5) ch3a_flag = .true.
          if (ch3a_flag .and. ch3b_flag) then
!           channel 3 indicators unknwown because
!           5 space counts indicate both 3a and 3b.
            ch3 = 0
!
          else if (ch3a_flag) then
!
!           space counts indicate channel 3a data
            ch3 = ch3a
!
          else if (ch3b_flag) then
!
!           space counts indicate channel 3b data
            ch3 = ch3b
!
          endif
!-------------------------------------------------------------------
!         if noaa-k era spacecraft (satid.ge.12)
!         compare channel indicator in scan line
!         with ch3 computed above
!         channel indicator is located in the least significant bit
!         of variable idavh(2,scn). if bit is set , channel is
!         infra-red (3b), otherwise it is visible (3a).
!         v1.2: According to the NOAA-KLM User Guide the indicator bit 
!         should be in idavh(1,scn). AAPP appears to have it in both (1,scn)
!         and (2,scn), but NESDIS only in (1,scn). So use (1,scn).
!------------------------------------------------------------------
          id_in  = noascid(satid)
          scname = noascnam(satid)
          if( ( id_in .ge. id_noaak ) .or.
     &        ( scname(1:4) .eq. 'meto' ) ) then
            indicator_bit = idavh(1,scn)
            bit_flag = gp_btn ( indicator_bit , 0 )
            if (bit_flag) then
              indicator_bit = ch3a
            else
              indicator_bit = ch3b
            endif
!-------------------------------------------------------------------
!           if ch3 doesn't agree with the indicator flag set ch3 = 0.
!--------------------------------------------------------------------
            if (indicator_bit .ne. ch3 ) then
              ch3 = 0
            endif
!            type *,'scn ',scn,' indicator_bit ',indicator_bit,
!     &             ' ch3 ',ch3 
          endif
        endif
        ch3_ind(scn) = ch3
        if(frst_ch3_ind.eq.0.and.ch3_ind(scn).ne.0) then
          frst_ch3_ind = ch3
          frst_known_ch3 = scn
        endif    
      enddo
!-----------------------------
!     check if ch3_ind changes 
!-----------------------------    
      ch3 = frst_ch3_ind
      nb_ch3 = 0
      do i = 1,100
        scn_new_chn(i) = 0 
      enddo
      if (frst_known_ch3 .eq. 0) then
        text = 'chkch3: no good start for ch3' 
        call ml_wt('error',text)
        call aappexit(10)
      endif

      do scn = frst_known_ch3 , avh_h_scnlin
        if(ch3_ind(scn) .ne. 0 .and. 
     &     ch3_ind(scn).ne. ch3 ) then
          nb_ch3 = nb_ch3 + 1
          scn_new_chn (nb_ch3) = scn
          ch3=ch3_ind(scn)
          if(nb_ch3.eq.100) then
!-------------------------------------------------------
!           the program can stop here because input data
!           are too much corrupted 
!--------------------------------------------------------
            text = 'chkch3 too many changes in ch3 indicator'
            call ml_wt('error',text)
            call aappexit(10)
!
          endif
        endif
      enddo
      ch3_change = nb_ch3 .ne. 0
      if(.not.ch3_change) then
        do scn = 1 ,  avh_h_scnlin
          ch3_ind(scn) = frst_ch3_ind
        enddo
      else 
        i = 1
        start_scn = 1
        ch3 = frst_ch3_ind
 100    continue
        if( ch3_ind(scn_new_chn(i)-1) .ne. 0) then
          do k =  start_scn, scn_new_chn(i)-1  
           ch3_ind(k) = ch3
          enddo 
        endif       
        start_scn = scn_new_chn(i)
        ch3 = ch3_ind(scn_new_chn(i))
        i = i + 1
        if ( i .gt. nb_ch3) ended = .true. 
        if(.not.ended) go to 100
        do scn = scn_new_chn(nb_ch3) , avh_h_scnlin
          ch3_ind(scn)= ch3_ind(scn_new_chn(nb_ch3))
        enddo
      endif
      return
      end
