      subroutine filter(x,nbx,xmin,xmax,xmean,sigma,
     &   nbxnul,nbxinf,nbxsup,nbreject,flag)
!+
!NOM/NAME $Id: filter.F 25 2006-02-03 15:43:53Z frna $
! filter the xdata array related to xmin,xmax values
!   (gross filter) if more than one value is kept
!  compute the xmean value and sigma then
!  performs the 2 sigma filter
!  and computes again xmean and sigma
!  flag indicates if xmean has been calculated (.true.) or not
!  (.false.)
!BUT/PURPOSE 
!  
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER 
! Marcel Derrien
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Nov. 95      no             Marcel Derrien
!
!         
!ARGUMENTS
!
!       integer sblock : sub-block we are computing cal coefficients    
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!DECLARATIONS
!
!    
!arguments
       integer  x        ! array to filter
       integer xmin,xmax ! min and max value for gross filter
       real xmean,sigma  ! mean and sigma
       integer  nbx      ! x array size 
       integer nbxnul    ! number of x elements zero
       integer nbxinf    ! number of x below xmin
       integer nbxsup    ! number of x element above xmax
       integer nbreject  ! number of x elements rejected
       logical flag      ! .true. if ok
       dimension x(nbx)
!commons
#include "avhrcl.h"
#include "avh1b.h"
#include "cavh_m1b.h"
#include "cavh_wrk.h"
!local variables
       integer i,j,k,xbinf,xbsup
       double precision xwork(50)
       double precision sumx,sumx2,xavg
       real sig2
!-
       nbxnul   = 0
       nbxinf = 0
       nbxsup = 0
       nbreject = 0
       i = 0
       j = 0
       k = 0
       sumx=0.
       sumx2=0.
       flag = .true.
!
!      gross filter
!
       do 10 i = 1 , nbx
         if (x(i).eq.0) then
           nbxnul=nbxnul+1
         else
           if(x(i).lt.xmin) then
             nbxinf = nbxinf + 1
           else
             if(x(i).gt.xmax) then
               nbxsup = nbxsup + 1
             else
               j = j + 1
               xwork(j) = x(i)
               sumx=sumx + xwork(j)
               sumx2=sumx2 + xwork(j) * xwork(j)
             endif
           endif
         endif
10     continue
!
!      if gross filter keeps data compute mean and sigma
!
       if(j.ne.0) then

         xavg = sumx/j
         sig2=sumx2/j-xavg*xavg
         sigma = sqrt( sig2 )
         xmean=xavg
         if(sigma.gt.1.0) then
!         
!      if sigma greater than 1 perform 2 sigma filter
!
           k = 0
           sumx = 0
           sumx2 = 0
           xbinf = nint(xmean - 2. * sigma)
           xbsup = nint( xmean + 2. * sigma)
           do 20 i = 1 , j
             if(xwork(i).ge.xbinf.and.xwork(i).le.xbsup) then
               k = k + 1
               sumx = sumx + xwork(i)
               sumx2 = sumx2 + xwork(i) * xwork(i)
             else
               nbreject = nbreject + 1
             endif
20         continue
           if(k.ne.0) then
!
!      2 sigma filter of compute mean and sigma
!
             xavg = sumx / k
             sig2=sumx2/k-xavg*xavg
             sigma = sqrt( sig2 )
             xmean=xavg
           else
             flag= .false.
           endif
         endif
       else
         flag= .false.
       endif
       end

