      subroutine fy1_cal

c-----------------------------------------
! Name: $Id: fy1_cal.F 112 2008-02-01 13:37:40Z frna $
!
! Calibrate MVISR data from FY1, for the AVHRR-like channels.
! Assume the three IR channels are linear
!
! History:
! Version    Date     Comment
!  1.0    20/12/2006  Initial (Nigel Atkinson)
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2008, EUMETSAT, All Rights Reserved.
!

      implicit none
 
!commons
#include "cavh_cpi.h"
#include "avh1b.h"

!functions
      real avh_bcor

!parameters
      integer*2 one
      parameter (one=1)

!local variables
      integer i, j, chn
      real prtcounts,spacecounts(5),targcounts(5),prtrad(3)
      real slope,intercept
      double precision prttemp

!initialise
      prtcounts = 0.0
      do chn=1,5
        spacecounts(chn) = 0.0
        targcounts(chn) = 0.0
      enddo

!PRT temperatures
      do i=1,4
        prtcounts = prtcounts + avh_telem(12+i)
      enddo
      prtcounts = prtcounts/4.0
      prttemp = conv_prt(1,1) + conv_prt(2,1)*prtcounts
C      write(*,*)'prttemp=',prttemp,prtcounts !!!!!!!!
      
!Warm counts
      j = 23         !targ counts are stored in backscan slots
      do i=1,6
        do chn=1,5
          targcounts(chn) = targcounts(chn) + avh_telem(j)
          j = j + 1
        enddo
      enddo

!Space counts
      j = 53         !space counts
      do i=1,10
        do chn=1,5
          spacecounts(chn) = spacecounts(chn) + avh_telem(j)
          j = j + 1
        enddo
      enddo
      
      do chn=1,5
        targcounts(chn) = targcounts(chn)/6.0
        spacecounts(chn) = spacecounts(chn)/10.0
      enddo

C      write(*,'(A,5F8.1)')'targ: ',(targcounts(chn),chn=1,5)    !!!!!!
C      write(*,'(A,5F8.1)')'space:',(spacecounts(chn),chn=1,5)      !!!!!

!Warm target radiance
      prtrad(1) = avh_bcor(prttemp,1)
      prtrad(2) = avh_bcor(prttemp,2)
      prtrad(3) = avh_bcor(prttemp,3)
      
!IR calibration coefs
      do chn=1,3
        slope = prtrad(chn)/(targcounts(chn+2) - spacecounts(chn+2))
        intercept = -spacecounts(chn+2)*slope
C        write(*,*)chn,slope,intercept !!!!!!!!
        avh_calir(1,1,chn) = 0               !operational
        avh_calir(2,1,chn) = slope*1E6
        avh_calir(3,1,chn) = intercept*1E6
        avh_calir(1,2,chn) = 0               !pre-launch
        avh_calir(2,2,chn) = slope*1E6
        avh_calir(3,2,chn) = intercept*1E6
      enddo

!Vis calibration coefs
      do chn=1,2
        do i=1,3
          avh_calvis(1,i,chn) = vis_slope(chn,1)*1E10
          avh_calvis(2,i,chn) = vis_int(chn,1)*1E7
          avh_calvis(3,i,chn) = vis_slope(chn,1)*1E10
          avh_calvis(4,i,chn) = vis_int(chn,1)*1E7
          avh_calvis(5,i,chn) = 0
        enddo
      enddo

!Set the ch3a/3b indicator
      avh_scnlinbit = ior(avh_scnlinbit,iand(avh_telem(7),one))
C      write(*,*)'scnlinbit=',avh_scnlinbit   !!!!!!!

!Increment count of calibrated and located lines. Check qualind bit 27 & 28
      if (iand(avh_qualind,402653184).eq.0)
     & avh_h_callocscnlin = avh_h_callocscnlin + 1

      return
      end
