      subroutine fy1_cpar(lu)

c-----------------------------------------
! Name: $Id: fy1_cpar.F 112 2008-02-01 13:37:40Z frna $
!
! History:
! Version    Date     Comment
!  1.0    20/12/2006  Initial (Nigel Atkinson)
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2008, EUMETSAT, All Rights Reserved.
!
      implicit none                                                     

!arguments
      integer lu ! logical unit for calibration parameter file     

!commons
#include "cavh_cpi.h"

!local variables
      integer ios, i, chn
      character*80 text     ! to write message log
      character*80 charact  ! to read
!-

!open calibration parameter file
!
       open(lu,access='sequential',status='old',iostat=ios)

!      If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
       if(ios.ne.0)
     &   open(lu,access='sequential',status='unknown',iostat=ios)

       if(ios.ne.0) then
         text='avh_cpar: fy1cal.txt file could not be opened'
         call ml_wti('error',text,ios)
         call aappexit(2)
       endif
!
!  skip the # commented first lines
!
       read(lu,'(a80)',err=100) charact
       do while (charact(1:1) .eq. '#')
        read(lu,'(a80)',err=100) charact
       enddo
       backspace lu
!
!  get the version num,year and day
!
       read(lu,*,err=100)versnb
       read(lu,*,err=100)versyr
       read(lu,*,err=100)versdy
!
! read the physical constants c1 , c2 Planck function
!
!       
       read(lu,'(a80)',err=100) charact
       do while (charact(1:1) .eq. '#')
         read(lu,'(a80)',err=100) charact
       enddo
       backspace lu
       read(lu,*,err=100) c1,c2
!
! PRT count to temperature  conversion coefficients
!
       read(lu,'(a80)',err=100) charact
       do while (charact(1:1) .eq. '#')
         read(lu,'(a80)',err=100) charact
       enddo
       backspace lu
       read(lu,*) conv_prt(1,1),conv_prt(2,1)  !just 2 coefs
!
! VISIBLE COEFFICIENTS slope and intercept
!
       read(lu,'(a80)',err=100) charact
       do while (charact(1:1) .eq. '#')
         read(lu,'(a80)',err=100) charact
       enddo
       backspace lu
       do chn=1,2
         read(lu,*) vis_slope(chn,1),vis_int(chn,1)     !just one set
       enddo
!
! equivalent widths, integrated solar irradiance,  effective central wavelength
!
       read(lu,'(a80)',err=100) charact
       do while (charact(1:1) .eq. '#')
         read(lu,'(a80)',err=100) charact
       enddo
       backspace lu
       do chn=1,2
         read(lu,*,err=100) vis_eqwi(chn), vis_isi(chn), vis_ecwa(chn)
       enddo
!
! wavenumber and band correction for IR channels
!
       read(lu,'(a80)',err=100) charact
       do while (charact(1:1) .eq. '#')
         read(lu,'(a80)',err=100) charact
       enddo
       backspace lu
       do chn=1,3
         read(lu,*,err=100) avh_bandcor(1,chn),avh_bandcor(2,chn),
     &    avh_bandcor(3,chn)
         planck1(chn) = c1*avh_bandcor(1,chn)*avh_bandcor(1,chn)
     &        *avh_bandcor(1,chn)
         planck2(chn) = c2*avh_bandcor(1,chn)
       enddo
  100  close (lu)
       return
       end

