!+ Calculate moon right ascension and declination for a given time
!
      SUBROUTINE MOON_POSITION ( jday, ra, dec )
      IMPLICIT NONE
!
! Purpose:
!  Calculate position of moon to an accuracy of approximately 0.3 degrees.
!  Reference: http://www.stargazing.net/kepler/moon.html
!
!  Current code owner: N.C.Atkinson
!
!  History:
!  Version  Date       Comment
!  1.0    06/06/01  Original code.
!  1.1    22/05/03  Web reference updated.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################

! Scalar arguments:
      REAL*8 jday       !Time in days from 1 Jan 2000, 12Z
      REAL   ra,dec     !Right ascension and declination in radians

! Local scalars:
      REAL*8 t,l,bm,gp,twopi
      REAL   rm,xg,yg,zg,ecl,xe,ye,ze

      twopi = 6.283185307
      t = jday/36525.0   !Time in centuries

! Calculate the geocentric ecliptic longitude (radians)
      l = MOD((3.8104 + 8399.70914*t), twopi)   !Repeats every 27 days
      l = l + 0.1098*sin(2.354 +  8328.691*t)
      l = l - 0.0222*sin(4.524 -  7214.063*t)
      l = l + 0.0115*sin(4.114 + 15542.754*t)
      l = l + 0.0037*sin(4.711 + 16657.382*t)
      l = l - 0.0033*sin(6.240 +   628.302*t)
      l = l - 0.0019*sin(3.257 + 16866.933*t)

! Calculate the geocentric ecliptic latitude
      bm =      0.0895*sin(1.628 +  8433.466*t)
      bm = bm + 0.0049*sin(3.983 + 16762.157*t)
      bm = bm - 0.0049*sin(5.555 +   104.775*t)
      bm = bm - 0.0030*sin(3.798 -  7109.289*t)

! Calculate the parallax
      gp = 0.016595 
      gp = gp + 0.000904*cos(2.354 +  8328.691*t)
      gp = gp + 0.000166*cos(4.524 -  7214.063*t)
      gp = gp + 0.000136*cos(4.114 + 15542.754*t)
      gp = gp + 0.000049*cos(4.711 + 16657.382*t)

! Radius vector
      rm = 1.0/(sin(gp))
      xg = rm * cos(l)*cos(bm)
      yg = rm * sin(l)*cos(bm)
      zg = rm * sin(bm)

! Rotate to equatorial coords
      ecl = 0.409093 - 0.0002271*t
      xe = xg
      ye = yg*cos(ecl) - zg*sin(ecl)
      ze = yg*sin(ecl) + zg*cos(ecl)

! Geocentric RA and Dec (radians)
      ra = atan2(ye,xe)
      if (ra .lt. 0) ra=ra+twopi
      dec = atan(ze/sqrt(xe*xe+ye*ye))

      RETURN
      END
