C
C ROUTINE          PLANK SUBROUTINE FORTVAX 
C
C PURPOSE          TO COMPUTE RADIANCE GIVEN TEMP AND FREQ IN CM-1
C
C DESCRIPTION      THIS ROUTINE COMPUTES A RADIANCE AT A FREQUENCY 
C                  SUPPLIED FOR A DEFINED BRIGHTNESS TEMP USING THE
C                  PLANCK FN.
C
C VERSION          1.00   210492  R.W.SAUNDERS
C                  2.0    March 03 T.Labrot
C                     f90 test : Changed  type XXX/yyy/
C                                     to  type XXX
C                                         parameter (XXX=yyy)
C
C ARGUMENTS        FGHZ R*4 FREQUENCY IN Cm-1
C                  T    R*4 TEMP IN DEGK
C                  BBRAD R*4 RADIANCE IN mW/(sqm/ster/cm-1)
C
C REFERENCE        CONSTANTS TAKEN FROM "ADJUSTMENTS TO FUNDAMENTAL
C                  PHYSICAL CONSTANTS  CODATA BULLETIN NO. 63 NOV 1986"
C
C CHANGES          01 NEW 
C
      SUBROUTINE PLANK(F,T,BBRAD)
      IMPLICIT NONE
c-----------------------------------------
c

      REAL BB,F,BBRAD,D,F1,T
      REAL C1,C2                    ! planck constants
      PARAMETER (C1=1.1910439E-5)
      PARAMETER (C2=1.4387687) 
C      
      BB=C1*F*F*F
      F1=C2*F/T
      D = EXP(F1)-1.
      BBRAD=BB/D
C
      RETURN
      END
