      subroutine prtchk(n,tarray,weightarray,maxt,mint,delta,meant,
     & nmean,ngoodweight)
!+
!NOM/NAME $Id: prtchk.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!     Quality control of PRT temperatures for a target, and calculation of
!     the mean temperature.
!
!DESCRIPTION
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions
!
!PROPRIETAIRE/OWNER
!     Nigel ATKINSON
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES)
!(version   date      comment                                 <author>)
! 1.0     June 96                                            N. Atkinson
! 1.1     16 July 96  Allow for non-integer PRT weights      N. Atkinson
!
!ARGUMENTS
!      n                  Integer    Number of PRTs
!      tarray(*)          Real       Array of temperatures
!      weightarray(*)     Real       PRT weights
!      maxt,mint          Real       Temperature limits
!      delta              Real       Tolerance
!      meant              Real       Mean temperature (returned by routine)
!      nmean              Integer    Number of PRTs used to calculate mean
!      ngoodweight        Integer    Number of PRTs with non-zero weight
!
!SUBROUTINE CALLS
!      amedian
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################

!--DECLARATIONS---------------------------------------------------------------
      implicit none
!
      real tarray(*),weightarray(*),maxt,mint,delta,meant,norm
      integer n,ngoodweight,nmean
!
! local variables
      real tgood(7),wgood(7),median
      integer i,ngood
!
! Select PRTs which have non-zero weight and are within gross limits
!
      ngoodweight=0        !Number of PRTs with weight>0
      ngood=0              !Number of PRTs within gross limits and weight>0
      nmean=0              !Number of PRTs used in calculation of mean
      meant=0.0
      norm=0.0
      do 10 i=1,n
        if (weightarray(i).gt.0) then
          ngoodweight=ngoodweight+1
          if (tarray(i).ge.mint.and.tarray(i).le.maxt) then
            ngood = ngood+1
            tgood(ngood) = tarray(i)       !Store selected points
            wgood(ngood) = weightarray(i)  !Store selected weights
          endif
        endif
   10 continue
!
! Calculate mean using only those PRTs which are within `delta' of the median
!
      if (ngood.gt.0) then
        call amedian(tgood,ngood,median)
        do 20 i=1,ngood
          if (abs(tgood(i)-median).le.delta) then
            nmean = nmean + 1
            norm = norm + wgood(i)
            meant = meant + tgood(i)*wgood(i)
          endif
   20   continue
        if (norm.gt.0) meant=meant/norm
      endif
      return
      end

