        subroutine h_cinit(satid,quality)
!+
!NOM/NAME $Id: h_cinit.F 47 2006-08-21 08:44:41Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module opens/closes,reads the calibration coefficients file. 
!     Data are ranged in common hrs_clcf (include cinit.h).
!     This module opens/closes,reads the test coefficients file and 
!     ranges data in common hrs_tstcf (include cinit.h). 
!     This module opens the statistics file (if requested). 
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      June 95       no             T. LABROT
! 1.1      May  98    modified to conform to NOAA format l1b release (TL)
! 2.0      August00   move call aappexit(9) in call aappexit(2)
! 2.1      Mar 04     Modify for linux  N Atkinson / A Korpela
!         
!ARGUMENTS 
!     input:
!     output:
!      integer   satid                  !order satellite number
!      integer   quality(7,hrs_mxlin)   !quality information array
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
! 
!     integer function   ord1bid :
!                  delivers the order satellite number for a given 
!                  level 1b id number (PACS number) and a julian date
!     character*6 function  noascnam :  
!                  delivers the NOAA satellite name for a given order number
!     subroutine  h_calibcoeffile to read calib. coef. file
!     subroutine  h_testcoeffile  to read test   coef. file
! 
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none 
                                                     

! includes/commons
#include "hclin.h"
#include "hrsparam.h"
#include "hrs1bhd.h"

! argument variables 
      integer satid,quality(7,*)

! local variables
      character*80 text        !variable text
      integer ios              !variable that receives the error status
      integer i,ilin           !index for loops
      integer hrs_h_satid0     !satellite id from a 1B file
      integer versnb           !number of version of the coef file
      integer versyr           !year of the version of the coef file
      integer versday          !day of the version of the coef file (0/365)
      integer ch20year         !year of most recent solar channel calibration
      integer ch20day          !day of year of recent solar channel calibration
      double precision f2      !albedo-radiance ch20 solar filtered irradiance
      double precision lambda  !albedo-radiance ch20 equivalent filter width 

! functions
      integer ord1bid 
      character*6 noascnam

!--END OF DECLARATIONS---------------------------------------------------------

      if (iprint.ge.1) write(hrs_cllgu,*)
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'
      if (iprint.ge.1) write(hrs_cllgu,*) '**  SUBROUTINE H_CINIT   **'
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'
      if (iprint.ge.1) write(hrs_cllgu,*)

! Deliver the order satellite number for the l1b satellite number and
! the julian date of the data (arguments must be integer)
      hrs_h_satid0 = hrs_h_satid

      satid = ord1bid(hrs_h_satid0,hrs_h_startdatajd)
 
      if (iprint.ge.1) then
       write(hrs_cllgu,fmt=500) hrs_h_satid0
       write(hrs_cllgu,fmt=505) hrs_h_startdatajd
      endif

      if (satid.eq.0) then

      text='Sat. id. not found or an error occured during the sat. id.' 
     & //  ' file reading'
       call ml_wt('error',text)
       if (iprint.ge.1) then
         write(hrs_cllgu,*) ' sat. id. not found or an error',
     &   ' occured during the sat. id. file reading'
       endif
       call aappexit(2)

      else

       text='Satellite :'//noascnam(satid)
       call ml_wt('info',text)
       if (iprint.ge.1)  write(hrs_cllgu,510) noascnam(satid)
       text='order number:'
       call ml_wti('info',text,satid)
       if (iprint.ge.1)  write(hrs_cllgu,515) satid
       
     
! Open and read the calibration coefficients file
       call h_calibcoeffile(hrs_coefu,satid, versnb,versyr,
     &                      versday,ch20year,ch20day,f2,lambda)


! Initialize words in the common hrs1bhd (header of the l1b)
! with the number of the version of the coefficients file. 
       hrs_h_cpidsyr = versyr 
       hrs_h_cpidsdy = versday 

! Initialize hrs_h_solarcalyr and hrs_h_solarcaldy in the common hrs1bhd
! (header of the l1b) with the date of the coefficients file. 
       hrs_h_solarcalyr = ch20year 
       hrs_h_solarcaldy = ch20day

! Initialize hrs_h_20solfiltirrad and hrs_h_20equifiltwidth in the common 
! hrs1bhd (header of the l1b).
       hrs_h_20solfiltirrad = idint(f2*(1.d+06))
       hrs_h_20equifiltwidth= idint(lambda*(1.d+06))


      endif

! Open and read test coefficients file
      call h_testcoeffile(hrs_tstcoefu)  


! Open the statistics file (if requested)
      if(hrs_stflg.eq.1) then

        open(hrs_stu,form='formatted',access='sequential',
     &       status='new',iostat=ios)

!       If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
        if(ios.ne.0)
     &  open(hrs_stu,form='formatted',access='sequential',
     &       status='unknown',iostat=ios)

        if(ios.ne.0) then
         text='The statistics file could not be opened, ios:'
         call ml_wti('error',text,ios)
         if (iprint.ge.1) write(hrs_cllgu,*)
     &   '  the statistics file could not be opened,  ios:',ios
         call aappexit(2)
        else
         text='The statistics file has been opened'
         call ml_wt('info',text)
         if (iprint.eq.1) write(hrs_cllgu,*)
     &   '  the statistics file has been opened'
        endif
        
      endif    

! Initialize the array which gives informations about the quality of the
! calibration scan line by scan line
      do i=1,7
        do ilin=i,hrs_h_scnlin
          quality(i,ilin) = 0
        enddo
      enddo 
   
 500  format(1X,'NOAA spacecraft identification code: ',I2)
 505  format(1X,'start of data set (Julian day): ',I7)
 510  format(1X,'NOAA satellite name: ',A6)
 515  format(1X,'NOAA order number: ',I2)

      return
      end
