        subroutine h_cntmn(splintab,iwtlintab)
!+
!NOM/NAME $Id: h_cntmn.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     For each superswath, compute the mean of filtered radiant signal outpout
!     counts for the space and IWT (internal warm target) lines.
!     HIRS instruments have contamination in the first xx FOVs of the space
!     scan (8 for the HIRS/2, unknown for the moment for the HIRS/3). The 
!     variable sp1fv (include cinit.h) indicates the first field of view
!     in the space scan that is not contaminated. The same FOVS will be
!     eliminated from the scan line of the warm target.
!     Tests: -outpout counts included in limits (sub. h_limit)
!            -minimum number of filtered data to compute the mean 
!             (sub. h_cntstat)
!            -rejection test: +/- coef*standard deviation for the counts.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!             
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      June 95       no             T. LABROT
! 1.1      June 96    modif. of h_limit and some "infos" have changed (TL)
! 1.2      Febr 05    modify write formats to write number of line > 1000 (TL)
!         
!ARGUMENTS 
!     input :
!      integer splintab(hrs_mxcalib)     !numbers of space views lines 
!      integer iwtlintab(hrs_mxcalib)    !numbers of IWT scan lines 
!
!     output:
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!     subroutine h_orderch(inptab,outtab,dim1,dim2,xx) :
!                It moves the order of the HIRS channels.
!
!     subroutine h_limit(sp1fv,counts,cntlm,infcnt,supcnt,misscnt)
!                It checks if counts are outside limits or not.
!
!     subroutine h_cntstat(sp1fv,minvw,counts,dtmn,dtstd,goodcnt) :
!                For one scan line (space or iwt one), it computes the mean and
!                the standard deviation of radiant signal outpout counts.
!    
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------


!--DECLARATIONS----------------------------------------------------------------

      implicit none
                                                      
 
!includes/commons
#include "hclin.h"
#include "cinit.h"
#include "hrsparam.h"
#include "hrs1bdts.h"
#include "hstatist.h"
#include "missingval.h"

! argument variables
      integer splintab(*),iwtlintab(*)

! local variables
      character*80 text             !variable text
      integer number                !parameter for initializing
      integer icalib,ich0,ich,ifv   !index of loops
      integer ilin                  !scan line number in the orbit
      integer infcnt            !counter of output count values
!                                lower than min limit
      integer supcnt            !counter of output count values
!                                greater than max limit
      integer misscnt           !counter of missing output count values
      integer goodcnt           !counter of valid outputcount values
      integer badcount          !output counts values exceeding 
!                                stdevcf*standard deviation.
      integer inputch(hrs_mxchn,hrs_mxpix)   
!                      output counts for all channels and hrs_mxpix FOVs
      integer chcounts(hrs_mxchn,hrs_mxpix)   
!                      output counts for all channels and hrs_mxpix FOVs
                                !but the channels has been ordered
      integer counts(hrs_mxpix) !output counts for one channel and 56 FOVs 
      double precision dtmn,dtstd   
                                !mean and standard deviation for one scan line

!--END OF DECLARATIONS----------------------------------------------------------

      parameter (number=2*hrs_mxchn*hrs_mxpix)
      data inputch,chcounts/number*0/
      data counts/hrs_mxpix*0/
      dtmn  = 0.d+0
      dtstd = 0.d+0

      if (iprint.ge.1) write(hrs_cllgu,*)
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'
      if (iprint.ge.1) write(hrs_cllgu,*) '**  SUBROUTINE H_CNTMN   **'
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'
      
      do icalib=1,calib
 

! ------------------------------------------------
! Processing of each space scan line in the orbit. 
! ------------------------------------------------
        ilin=splintab(icalib)

        if(iprint.ge.1) write(hrs_cllgu,fmt=500) icalib,ilin
        write(text,'("Compute mean of outpout counts for space line "
     &               ,I4)') ilin
        call ml_wt('info',text)
      
! Put HIRS channels output counts in order 1,2,3,...
        do ifv=0,hrs_mxpix-1
          do ich0=3,3+hrs_mxchn-1
            inputch(ich0-2,ifv+1)=hrs_elem(ich0,ifv,ilin)
          enddo
        enddo

       if (iprint.ge.2) write(hrs_cllgu,*)
       if (iprint.ge.2) write(hrs_cllgu,*) '   *call sub. H_ORDERCH*'
 
        call h_orderch(inputch,chcounts,hrs_mxchn,hrs_mxpix,0)

!        if (iprint.ge.2) then
!          do ifv=0,hrs_mxpix-1
!            write(hrs_cllgu,fmt=520) 
!            do ich=1,hrs_mxchn
!              write(hrs_cllgu,fmt=530) ifv+1,ich,chcounts(ich,ifv+1)
!            enddo
!          enddo
!        endif


! For each IR channel,compute the mean of filtered radiant signal outpout counts
!-------------------------------------------------------------------------------
 
         do ich=1,hrs_mxirchn

          if (iprint.ge.1) write(hrs_cllgu,fmt=540) ich 

! Pick up the appropriate counts for the channel i

          do ifv=1,hrs_mxpix
            counts(ifv)=chcounts(ich,ifv)
          enddo
      
! Test: outpout counts included in limits
                     
          call h_limit(ich,sp1fv,counts,spcntlm,infcnt,supcnt,misscnt)

! Compute the mean and  the standard deviation of output counts.
! (test:minimum of filtered data to compute the mean)

          if(iprint.ge.2) write(hrs_cllgu,fmt=550)
          call h_cntstat(sp1fv,minvw,counts,dtmn,dtstd,goodcnt)

! Eliminate output counts values exceeding stdevcf*standard deviation.

          if (dtmn.ne.missdp) then

            badcount=0
    
            do ifv=sp1fv,hrs_mxpix

              if (counts(ifv).lt.missint) then
                if (abs(counts(ifv)-dtmn).gt.(stdevcf*dtstd)) then

                  if(iprint.ge.1) write(hrs_cllgu,fmt=570) 
     &                                  ich,ifv,counts(ifv)
 
                  counts(ifv)=missint
                  badcount   = badcount+1
       
                endif
              endif
          
            enddo

           if (badcount.eq.0.and.iprint.ge.2) write(hrs_cllgu,fmt=580)


! Recompute the statistics if necessary
            if (badcount.ne.0) then

              if(iprint.ge.2) write(hrs_cllgu,fmt=560)

              call h_cntstat(sp1fv,minvw,counts,dtmn,dtstd,goodcnt)

            endif

          endif

          spcntmn(ich,icalib)  = dtmn
          spcntstd(ich,icalib) = dtstd
          spgoodcnt(ich,icalib)= goodcnt
          spinfcnt(ich,icalib) = infcnt
          spsupcnt(ich,icalib) = supcnt
          spmisscnt(ich,icalib)= misscnt
          spejeccnt(ich,icalib)= badcount

          if (dtmn.eq.missdp) then
            write(text,'("Space view scan line nb",I4,
     &      " = no mean for channel nb ",I2)')ilin,ich
            call ml_wt('warning',text)
          endif
        
! End of the loop "for each channel"
        enddo

! Reinitialize arrays and parameters for the processing of a IWT scan line
        do ifv=1,hrs_mxpix
          counts(ifv)=0
          do ich=1,hrs_mxchn
            chcounts(ich,ifv)=0
            inputch(ich,ifv) =0
          enddo
        enddo

        dtmn = 0.d+0
        dtstd = 0.d+0
 
!-------------------------------------------------
! Processing of each IWT scan line in the orbit. 
!-------------------------------------------------
        ilin=iwtlintab(icalib)

        if(iprint.ge.1) write(hrs_cllgu,fmt=510) icalib,ilin
        write(text,'("Compute mean of outpout counts for IWT line ",
     &        I4)') ilin
        call ml_wt('info',text)
        
! Put HIRS channels output counts in order 1,2,3,...
        do ifv=0,hrs_mxpix-1
          do ich0=3,3+hrs_mxchn-1
            inputch(ich0-2,ifv+1)=hrs_elem(ich0,ifv,ilin)
          enddo
        enddo

        if(iprint.ge.2) write(hrs_cllgu,*)
        if(iprint.ge.2) write(hrs_cllgu,*) '   *call sub. H_ORDERCH*'

        call h_orderch(inputch,chcounts,hrs_mxchn,hrs_mxpix,0)

!        if (iprint.ge.2) then
!          do ifv=0,hrs_mxpix-1
!            write(hrs_cllgu,fmt=520) 
!            do ich=1,hrs_mxchn
!              write(hrs_cllgu,fmt=530) ifv+1,ich,chcounts(ich,ifv+1)
!            enddo
!          enddo
!        endif

! For each IR channel,compute the mean of filtered radiant signal outpout counts
!-------------------------------------------------------------------------------
        do ich=1,hrs_mxirchn

          if (iprint.ge.1) write(hrs_cllgu,fmt=540) ich

! Pick up the appropriate counts for the channel ich
          do ifv=1,hrs_mxpix
            counts(ifv)=chcounts(ich,ifv)
          enddo

! Test: outpout counts included in limits
          
          call h_limit(ich,sp1fv,counts,iwtcntlm,infcnt,supcnt,misscnt)

! Compute the mean and  the standard deviation of output counts.
! (test:minimum of filtered data to compute the mean)

          if(iprint.ge.2) write(hrs_cllgu,fmt=550)
          call h_cntstat(sp1fv,minvw,counts,dtmn,dtstd,goodcnt)

! Eliminate output counts values exceeding stdevcf*standard deviation.

          if (dtmn.ne.missdp) then

            badcount=0

            do ifv=sp1fv,hrs_mxpix

              if (counts(ifv).lt.missint) then
                if(abs(counts(ifv)-dtmn).gt.(stdevcf*dtstd)) then

                  if(iprint.ge.1) write(hrs_cllgu,fmt=570) 
     &                                  ich,ifv,counts(ifv) 

                  counts(ifv)=missint
                  badcount   = badcount+1

                endif
              endif

            enddo

            if(badcount.eq.0.and.iprint.ge.2) write(hrs_cllgu,fmt=580)

! Recompute the statistics if necessary
            if (badcount.ne.0) then

              if (iprint.ge.2) write(hrs_cllgu,fmt=560)


              call h_cntstat(sp1fv,minvw,counts,dtmn,dtstd,goodcnt)

            endif

          endif

          iwtcntmn(ich,icalib)  = dtmn
          iwtcntstd(ich,icalib) = dtstd
          iwtgoodcnt(ich,icalib)= goodcnt
          iwtinfcnt(ich,icalib) = infcnt
          iwtsupcnt(ich,icalib) = supcnt
          iwtmisscnt(ich,icalib)= misscnt
          iwtejeccnt(ich,icalib)= badcount

          if (dtmn.eq.missdp) then
            write(text,'("IWT scan line nb",I4,
     &      " = no mean for channel nb ",I2)')ilin,ich
            call ml_wt('warning',text)
          endif

        enddo

       enddo 

      
 500   format(/2X,'CALIB CYCLE NUMBER:',I2,4X,' SPACE LINE NUMBER:',I4)
 510   format(/2X,'CALIB CYCLE NUMBER:',I2,4X,' IWT LINE NUMBER:',I4)
! 520   format(/2X,'ordered channels: ifov   ich    chcounts')
! 530   format(20X,I2,5X,I2,7X,I5)
 540   format(2X,'CHANNEL',I2,': ')
 550   format(4X,'initial values:')
 560   format(4X,'filtered values:')
 570   format(4X,'eliminated values: ch:',I2,'  ifov:',I2,
     &        '  count:',I6)
 580   format(4X,'eliminated values = none')

       return
       end

