        subroutine h_cntstat(sp1fv,minvw,counts, dtmn,dtstd,goodcnt)
!+
!NOM/NAME $Id: h_cntstat.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     For one scan line (space or IWT one), this module computes the mean and
!     the standard deviation of output counts.
!     HIRS instruments have contamination in the first xx FOVs of the space
!     scan (8 for the HIRS/2, unknown for the moment for the HIRS/3). The 
!     variable sp1fov (include cinit.h) indicates the first field of view
!     in the space scan that is not contaminated. The same FOVS will be
!     eliminated from the scan of the warm target.
!     Tests: minimum of filtered data to compute the mean
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!
            
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      dec. 95       no             T. LABROT
!         
!ARGUMENTS 
!    input :
!     integer sp1fv          !the first field of view in the space scan that
!                             is not contaminated.
!     integer minvw          !the minimum number of space or IWT data  
!                             to compute an average
!     integer counts(*)      !for one channel and hrs_mxpix FOVs output counts
!
!    output:
!     integer goodcnt        !number of values used to compute the mean
!     double precision dtmn  !output counts mean for one channel
!     double precision dtstd !output counts standard deviation for one channel
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!      
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------


!--DECLARATIONS----------------------------------------------------------------

      implicit none                                                     
 
 
!includes/commons
#include "hclin.h"
#include "hrsparam.h"
#include "missingval.h"

! argument variables
      integer sp1fv,minvw,counts(*),goodcnt
      double precision dtmn,dtstd

! local variables
      integer i               !index of loop

!--END OF DECLARATIONS---------------------------------------------------------

      dtmn = 0.d+0
      dtstd = 0.d+0
      goodcnt = 0
   
      if (iprint.ge.2) write(hrs_cllgu,*)'      *subroutine H_CNTSTAT*'

      do i=sp1fv,hrs_mxpix

        if (counts(i).lt.missint) then
          dtmn =dtmn + counts(i)
          dtstd = dtstd + counts(i)*counts(i)
          goodcnt = goodcnt + 1
        endif

      enddo

! Test: enough values to compute the mean. 
      if (goodcnt.ne.0.and.goodcnt.ge.minvw) then

        dtmn = dtmn/goodcnt
        dtstd =(dtstd/goodcnt)-(dtmn**2)
        if (dtstd .ge. 0.d+0) dtstd =dsqrt(dtstd)
 
      else

        dtmn = missdp
        dtstd = missdp

        if (iprint.ge.2) write(hrs_cllgu,fmt=500)

      endif

      if (iprint.ge.2) then
        write(hrs_cllgu,fmt=510) dtmn
        write(hrs_cllgu,fmt=520) dtstd
        write(hrs_cllgu,fmt=530) goodcnt
      endif

      if (iprint.ge.2) write(hrs_cllgu,*)
     & '      *end subroutine H_CNTSTAT*'

 500   format(12X,'not enough data to compute statistics')
 510   format(12X,'output counts mean = ',F12.5)
 520   format(12X,'output counts std  = ',F12.5)
 530   format(12X,'good counts =',I3)

       return
       end
