      subroutine h_gtmean(calibcoef)
!+
!NOM/NAME $Id: h_gtmean.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!     For each channel, this module computes the mean and the standard  
!     deviation of the slopes, of the intercepts and of the third coef.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0     June 1995       no              T. LABROT
! 1.1      May  98    modified to conform to NOAA format l1b release (TL)
!                     (scaling factor)
!                     added tests for the variable "calib"    (TL)
! 1.2     March 04  full orbit parametrisation P. Brunel
!         
!ARGUMENTS 
!   input :
!    dble prec calibcoef(hrs_mxchn,3,hrs_mxlin) 
!         third auto coefs,intercepts,slopes, array for each scan line, 
!         for each channel
!
!   output
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!     subroutine h_orderch(inptab,outtab,dim1,dim2,xx) :
!                It moves the order of the HIRS channels.
!    
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none                                                     
 
 
!includes/commons
#include "hclin.h"
#include "hrsparam.h"
#include "hrs1bhd.h"
#include "usablelines.h"
#include "hstatist.h"
#include "missingval.h"

! argument variables
      double precision calibcoef(hrs_mxchn,3,hrs_mxlin)

! local variables
      integer i,ich,ilin     !index of loops
      integer calinf0(hrs_mxchn,4)
      integer calinf1(hrs_mxchn,4)
      integer*2 goodlin      !counter od lines with good data
      integer*2 goodlinmx    !maximum of goodlin

!--END OF DECLARATIONS----------------------------------------------------------

      if (iprint.ge.1) write(hrs_cllgu,*)
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'
      if (iprint.ge.1) write(hrs_cllgu,*) '**  SUBROUTINE H_GTMEAN  **'
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'

      goodlinmx=0
      do ich=1,hrs_mxchn
        do i=1,3
          calibcoefmn(ich,i) = 0.d+0
          calibcoefstd(ich,i)= 0.d+0
        enddo
      enddo
          
      
      do ich=1,hrs_mxchn

       do i=1,3
       
        if (calib.eq.0) then
          calibcoefmn(ich,i) = missdp
          calibcoefstd(ich,i)= missdp
        endif
        if (calib.eq.1) then
          calibcoefmn(ich,i) = calibcoef(ich,i,lin0)
          calibcoefstd(ich,i)= 0.d+0
        endif  
        
        
        if (calib.gt.1) then
        
         goodlin=0

         do ilin=lin0,lin1
          if (calibcoef(ich,i,ilin).ne.missdp) then
           calibcoefmn(ich,i) = calibcoefmn(ich,i)+calibcoef(ich,i,ilin)
           calibcoefstd(ich,i)= calibcoefstd(ich,i)
     &                    + calibcoef(ich,i,ilin)*calibcoef(ich,i,ilin)
           goodlin=goodlin+1
          endif
         enddo

         if (goodlin.ne.0) then
          calibcoefmn(ich,i)=calibcoefmn(ich,i)/goodlin
          calibcoefstd(ich,i)=(calibcoefstd(ich,i)/goodlin)
     &                                   - (calibcoefmn(ich,i)**2)
          if (calibcoefstd(ich,i).ge.0.d+0) then
             calibcoefstd(ich,i)=dsqrt(calibcoefstd(ich,i))
          else
             calibcoefstd(ich,i)=missdp
          endif
         else
          calibcoefmn(ich,i) =missdp
          calibcoefstd(ich,i)=missdp
         endif

         if (goodlin.gt.goodlinmx) goodlinmx=goodlin
        
         calibcoefstd(20,i)=0.d+0
         
        endif
        
       enddo

       if (iprint.eq.2) then
         write(hrs_cllgu,fmt=500) ich
         write(hrs_cllgu,fmt=510)
         do i=1,3
           write(hrs_cllgu,fmt=520)i,calibcoefmn(ich,i),
     &                             calibcoefstd(ich,i)
         enddo
       endif

      enddo


! Update the common hrs1bhd (header du level 1b) 
!  - calibration information: mean of calibration slopes *(10**6)
!                             stantard deviation of calibration slopes*(10**6)
!                             mean of calibration intercepts *(10**6)
!                             standard deviation of intercepts *(10**6)
!  - count of calibrated scan lines 
! Range the data in integer arrays 

      do ich=1,hrs_mxchn

          if (calibcoefmn(ich,2).lt.missdp) then
             calinf0(ich,1) =idint(calibcoefmn(ich,2)*1.d+06)
          else
             calinf0(ich,1) = 0
          endif

          if (calibcoefstd(ich,2).lt.missdp) then 
             calinf0(ich,2) =idint(calibcoefstd(ich,2)*1.d+06)
          else
             calinf0(ich,2) = 0
          endif

          if (calibcoefmn(ich,3).lt.missdp) then
             calinf0(ich,3) =idint(calibcoefmn(ich,3)*1.d+06)
          else
             calinf0(ich,3) = 0
          endif

          if (calibcoefstd(ich,3).lt.missdp) then
             calinf0(ich,4) =idint(calibcoefstd(ich,3)*1.d+06)
          else
             calinf0(ich,4) = 0
          endif

      enddo

! Put the coefficients in the good order(=order of the channels in the l1b).
      call h_orderch(calinf0,calinf1,hrs_mxchn,4,1)

      do ich=1,hrs_mxchn
        hrs_h_calinf(1,ich) = calinf1(ich,1)
        hrs_h_calinf(2,ich) = calinf1(ich,2)
        hrs_h_calinf(3,ich) = calinf1(ich,3)
        hrs_h_calinf(4,ich) = calinf1(ich,4)
      enddo

!
      hrs_h_callocsclin=goodlinmx


 500  format(4X,'PROCESSING CHANNEL: ',I2)
 510  format(6X,'mean,std deviation for calibration coefficients:')
 520  format(8X,I1,3X,F20.10,3X,F20.10)

      return
      end
