      subroutine h_instrtest(quality)
!+
!NOM/NAME $Id: h_instrtest.F 47 2006-08-21 08:44:41Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!     Check the instrument status to define which scan lines are usable:
!     The first one = lin0  ;  The last one = lin1
!     At the present time, we impose lin0=1 and lin1=hrs_h_scnlin .
!     
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0     June 1995     no              T. LABROT
! 1.1     Dec. 96    suppression of a local variable which was never used (TL)
! 1.2     June 1998  test value for the variable h_inststat1 was changed (TL)
! 1.3     Mar  2005  Allow for NESDIS, as in h_instrtest_algoV4.F (NCA)
!         
!ARGUMENTS 
!  input/output :
! integer quality(7,hrs_mxlin) !quality information array about the calibration
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!    
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none                                                     
 
 
!includes/commons
#include "hclin.h"
#include "hrsparam.h"
#include "hrs1bhd.h"
#include "usablelines.h"

! argument variables
      integer quality(7,*)
  
! local variables
      character*80 text  !variable text 
      INTEGER inststatus !normal value of 'instrument status' field of the level1B
      INTEGER hinststat1 !copied from hrs_h_instat1
 
!--END OF DECLARATIONS----------------------------------------------------------

      if (iprint.ge.1) write(hrs_cllgu,*)
      if (iprint.ge.2) write(hrs_cllgu,*)
     &  '*****************************'
      if (iprint.ge.1) write(hrs_cllgu,*)
     &  '**  SUBROUTINE H_INSTRTEST **'
      if (iprint.ge.2) write(hrs_cllgu,*)
     &  '*****************************'
      if (iprint.ge.1) write(hrs_cllgu,*)

      hinststat1 = hrs_h_inststat1

      IF (hrs_h_siteid.EQ.'NSS') THEN
        inststatus=61556
        hinststat1 = IAND(hinststat1,65527) ! Allow high filter motor power
      ELSE
        inststatus=49152
      ENDIF 
      IF (iprint.GE.2) WRITE(hrs_cllgu,*) 'inststatus:',inststatus       

! The instrument status doesn't change and the instrument is OFF

      if (hrs_h_statchrecnb.eq.0.and. 
     &    hinststat1.ne.inststatus) then
        lin0=1
        lin1=hrs_h_scnlin
        text='Instrument is OFF and the status does not change'
        call ml_wt('warning',text) 
        if (iprint.ge.1) write(hrs_cllgu,*)
     &  '  instrument is OFF and the status does not change'
        return
      endif

! The instrument status doesn't change and the instrument is ON
      if (hrs_h_statchrecnb.eq.0 .and. 
     &    hinststat1.eq.inststatus) then
        lin0=1
        lin1=hrs_h_scnlin
        text='First line ='
        call ml_wti('info',text,lin0)
        text='Last line ='
        call ml_wti('info',text,lin1)
        if (iprint.ge.1) write(hrs_cllgu,*)
     &  '  first line =',lin0,'    last line =',lin1
        return
      endif
      
! The instrument status changes and the instrument is OFF first.
      if (hrs_h_statchrecnb.gt.0 .and. 
     &    hinststat1.ne.inststatus) then
        lin0=hrs_h_statchrecnb
        lin1=hrs_h_scnlin
        text='Instrument status is OFF first and changes at scan line'
        call ml_wti('warning',text,lin0)
        text='last line ='
        call ml_wti('info',text,lin1)
        if (iprint.ge.1) write(hrs_cllgu,*)
     &  '  instrument is OFF first and changes at line ',lin0
        if (iprint.ge.1) write(hrs_cllgu,*)
     &  ' first line =',lin0,'    last line =',lin1
        lin0=1
        lin1=hrs_h_scnlin
        return
      endif

! The instrument status changes and the instrument is ON first.
      if (hrs_h_statchrecnb.ne.0 .and. 
     &    hinststat1.eq.inststatus) then
        lin0=1
        lin1=hrs_h_statchrecnb-1
        text='first line ='
        call ml_wti('info',text,lin0)
        text='last line ='
        call ml_wti('info',text,lin1)
        text='Instrument status is ON first and changes at scan line'
        call ml_wti('warning',text,lin1+1) 
        if (iprint.ge.1) write(hrs_cllgu,*)
     &  '  the instrument is ON first and changes at line',lin1+1
        if (iprint.ge.1) write(hrs_cllgu,*)
     &  ' first line =',lin0,'    last line =',lin1
        lin0=1
        lin1=hrs_h_scnlin
        return
      endif

! Error in  the value of hrs_h_statchrecnb or hrs_h_inststat1
      lin0=0
      lin1=0
      text='Value error in the instrument status or the record number'
     & // 'of status change in the header of the level1b file'
      call ml_wt('error',text) 
      if (iprint.ge.1) write(hrs_cllgu,*)
     & '  value error in the instrument status or the record number'
     & // 'of status change in the header of the level1b file'

      call aappexit(10)
      end
