	subroutine h_iwtrad(calib,iwtlintab,iwttmp, iwtrad)
!+
!NOM/NAME $Id: h_iwtrad.F 24 2006-02-03 11:51:40Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     The IWT temperature is converted into radiance for each HIRS channel.
!     The used method is described in the NESS107 appendix A. 
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      June 95      no              T. LABROT
! 1.1      Febr 05    Modify the format of writing to write 
!                     number of line > 1000 (TL)
!         
!ARGUMENTS 
!     input :
!      integer calib                  !total number of calib cycle for the orbit
!      integer iwtlintab(hrs_mxcalib) !numbers of IWT lines 
!      dble prec iwttmp(hrs_mxcalib)  !IWT temperatures  
!
!     output:
!      double precision iwtrad(hrs_mxirchn,hrs_mxcalib)   
!                                     !IR channel radiances of the IWT
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!    
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none                                                     
 
 
!includes/commons
#include "hclin.h"
#include "cinit.h"
#include "hrsparam.h"
#include "missingval.h"

! argument variables
      integer calib,iwtlintab(hrs_mxcalib)
      double precision iwttmp(hrs_mxcalib)
      double precision iwtrad(hrs_mxirchn,hrs_mxcalib)

! local variables
      integer icalib,ich       !index of loops
      integer lin              !number of one IWT scan line in the orbit
      double precision iwtefftmp(hrs_mxirchn,hrs_mxcalib) 
                               !"effective" IWT temperature for the IR channels
      double precision planck1,planck2 
                               !intermediary variables for the Planck function

!--END OF DECLARATIONS----------------------------------------------------------

      if (iprint.ge.1) write(hrs_cllgu,*)
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'
      if (iprint.ge.1) write(hrs_cllgu,*) '**  SUBROUTINE H_IWTRAD  **'
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'

      do icalib=1,calib

       lin=iwtlintab(icalib)

       if (iprint.ge.1) write(hrs_cllgu,fmt=500) lin
       if (iprint.ge.1) write(hrs_cllgu,fmt=510) iwttmp(icalib)


! For each IR channel
       do ich=1,hrs_mxirchn

         if (iwttmp(icalib) .lt. missdp) then

! The IWT temperature is replaced by an "effective" temperature
           iwtefftmp(ich,icalib)=bdcorr(ich,1)+bdcorr(ich,2)
     &                           *iwttmp(icalib)


! Apply Planck function
           planck1 = plck(1)*(cwave(ich)**3)
           planck2 = dexp( plck(2)*cwave(ich)/iwtefftmp(ich,icalib))-1
           iwtrad(ich,icalib) = planck1 / planck2
                  
         else
          
           iwtrad(ich,icalib) = missdp

         endif

         if(iprint.ge.1) write(hrs_cllgu,fmt=520) ich,iwtrad(ich,icalib)      


       enddo

      enddo

 500  format(/2X,'IWT scan line number: ',I4)
 510  format(3X,'IWT temperature: ',F12.5) 
 520  format(3X,'IR channel ',I2,3X,'IWT rad: ',F15.5)

      return
      end
       
         
