        subroutine h_linecount(lnct,filin)
!+
!------------------------------------------------------------------------------
!NOM/NAME $Id: h_linecount.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!     For each scan line, this module checks the line count. 
!     The difference between the HIRS/2 and the HIRS/3 is taken into account.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Jan. 96       no             T. LABROT
! 1.1      June 96    bug inside do while ....       T. LABROT
! 1.2      March 03   F90 test: Delete the last "," in
!                write(text,'("Scan line",I3,": line counter is false")')
!            &        ,ilin                          T. LABROT
! 1.3      Febr 05    Modify formats of writing to write line number > 1000 
!                     Remove the hard-coding nbslval and ranged it in
!                     testcoeffile.dat (TL)
!  
!ARGUMENTS 
!     input/output: 
!       integer lnct(*)     !array of line counters.In output the values 
!                            have been corrected if necessary 
!       integer filin       !first line of first paquet of n lines 
!                            with correct line counters
!      
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------
      implicit none 

!includes/commons
#include "hclin.h"
#include "cinit.h"
#include "hrsparam.h"
#include "usablelines.h"
#include "missingval.h"

! argument variables
      integer lnct(*),filin
 
! local variables 
      character*80 text         !variable text
      integer ilin,j            !index for loops
      integer count             !value of the line counter
      integer mlnct(hrs_mxlin)  !line count that the line must have
      integer linnb,ilinnb      !numbers of lines between two scan lines
      logical error0,error1     !=.false. if no error, =.true. if error 
      integer loop_counter      ! loop counter

!--END OF DECLARATIONS---------------------------------------------------------
      filin=0

      if (iprint.ge.1) write(hrs_cllgu,*) 
      if (iprint.ge.1) write(hrs_cllgu,*) 
     &'FIRST PAQUET OF CONSECUTIVE LINES WITH CORRECT LINE COUNTERS'    
      
! Research the first packet of nbslval lines with line counts that are agree
      ilin=lin0
      error0=.true.
      loop_counter=0
      do while 
     &  (ilin.le.(lin1-nbslval+1).and.error0.and.loop_counter.lt.lin1)      
        loop_counter = loop_counter +1
        if (lnct(ilin).lt.0.or.lnct(ilin).gt.39) then
          write(text,'("Scan line",I4,":false or missing line",
     &         " count")')ilin
          call ml_wt('notice',text)
          if(iprint.ge.1)write(hrs_cllgu,fmt=500)
          ilin=ilin + 1
        else 
          if(iprint.ge.2)write(hrs_cllgu,fmt=510)ilin,lnct(ilin)  
          j=1
          error1=.false.
         
          
          do while ((j.le.nbslval-1).and.(.not.error1))
            count=lnct(ilin)+j
            if(count.gt.39) count=count-40
! We suppose that nbslval will be always inferior to 40 ==> (count-40)<40
            if(lnct(ilin+j).ne.count) error1=.true.
            j=j+1
          enddo
          if(error1) then
            error0=.true.
            ilin=ilin+1
          else
            error0=.false.
          endif
        endif
      enddo
      

      if (ilin.gt.(lin1-nbslval+1).and.error0) then
        write(text,'("No packet of ",I2,
     &   "correct consecutive line counters")')nbslval
        call ml_wt('warning',text)
        call ml_wt('info','encoder position is taken')
        if(iprint.ge.1) write(hrs_cllgu,fmt=520)
      else
        filin=ilin
        if(iprint.ge.1) write(hrs_cllgu,fmt=530) filin
        text='First paquet with correct line counts: line'
        call ml_wti('info',text,filin)
      endif
      
      
! Check the line counter by comparing with the line counters of the first paquet
!-------------------------------------------------------------------------------
      if(filin.gt.0) then
      
       if (iprint.ge.1) write(hrs_cllgu,*)
       if (iprint.ge.1) write(hrs_cllgu,*) 'LINE COUNTER'
      
       do ilin=lin0,lin1
! Compute the number of lines between the line ilin and filin
         linnb=filin-ilin
         ilinnb=linnb/40
! Compute the line count that the scan line must have
         mlnct(ilin)=lnct(filin)-linnb+40*ilinnb
         if(mlnct(ilin).lt.0) mlnct(ilin)=mlnct(ilin)+40
         if(mlnct(ilin).gt.39) mlnct(ilin)=mlnct(ilin)-40
!Check
         if(lnct(ilin).ne.mlnct(ilin)) then
           if(iprint.ge.1) then
             write(hrs_cllgu,fmt=540) ilin,lnct(ilin),mlnct(ilin)
           endif
 
        write(text,'("Scan line",I4,": line counter is false")')
     &        ilin
           call ml_wt('notice',text)
        write(text,'(3X,"initial value:",I4,"  corrected value:",I3)')
     &       lnct(ilin),mlnct(ilin)
           call ml_wt('notice',text)
        
           lnct(ilin)=mlnct(ilin)
! We have chosen to not correct the values of data coming from
! the TIP ==> we don't correct hrs_elem(3,63,ilin)
         endif
       enddo
      endif

 500  format(5X,'Scan line',I4,' initial line count false or',
     &  'missing')
 510  format(3X,'Scan line:',I4,'   initial line count ',I3) 
 520  format(3X,'No paquet of consecutive lines with correct line')
 530  format(5X,'the first paquet begins at scan line ',I4)        
 540  format(3X,'Scan line:',I4,'  line count:  initial',I4,
     &       '  corrected ',I2)
      
      return
      end            
