      subroutine h_linlin(calib,splintab,calibcoef0,quality, calibcoef)
!+
!NOM/NAME $Id: h_linlin.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!     For each channel and each scan line, this module computes slopes and
!     intercepts by a linear interpolating between the coefficients of two
!     adjacent superswaths.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0     June 1995       no              T. LABROT
! 1.1     March 04  full orbit parametrisation P. Brunel
! 1.2     Febr  05  Modify formats of writing to write line number> 1000
!                                                      calib cycle> 10  (TL)
!         
!ARGUMENTS 
!   input :
!    integer calib                  !total number of calib cycle for the orbit
!    integer splintab(hrs_mxcalib)  !numbers of space view lines 
!    dble prec calibcoef0(hrs_mcchn,3,hrs_mxcalib) 
!                                   !third auto coefs,slopes,intercepts array
!
!   input/output :
!    integer quality(7,hrs_mxlin)   !quality  information array
!
!   output
!    dble prec calibcoef(hrs_mxchn,3,hrs_mxlin)
!                   !third auto coefs,slopes,intercepts array for each scan line
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!    
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none                                                     
 
!includes/commons
#include "hclin.h"
#include "cinit.h"
#include "hrsparam.h"
#include "hrs1bhd.h"
#include "usablelines.h"
#include "missingval.h"

! argument variables
      integer calib,splintab(*)
      integer quality(7,*)
      double precision calibcoef0(hrs_mxchn,3,*)
      double precision calibcoef(hrs_mxchn,3,*)

! local variables
      character*80 text             !variable text
      integer icalib                !index of the calibration cycles
      integer ich                   !index of the IR channels
      integer ilin                  !index of the scan lines
      integer gdcal                 !number of correct calibration cycle
      integer gdcaltab(hrs_mxcalib) !index number table of good calib cycle 
      integer kcal1,kcal2           !index numbers of good calibration cycle
      integer kcal(hrs_mxcalib)     !index number table of good calib cycle
      integer k,kk                  !index
      integer badch                 !number of uncalibrated channel in one scan
      double precision w1,w2        !proportional coefficients

!--END OF DECLARATIONS----------------------------------------------------------

! Initialize variables
      data gdcal/0/
      data gdcaltab/hrs_mxcalib*0/
      data kcal1,kcal2,kcal/99,99,hrs_mxcalib*99/
      do ich=1,hrs_mxchn
        do ilin=1,hrs_h_scnlin
          calibcoef(ich,1,ilin) = missdp
          calibcoef(ich,2,ilin) = missdp
          calibcoef(ich,3,ilin) = missdp
        enddo
      enddo

       
      if (iprint.ge.1) write(hrs_cllgu,*)
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'
      if (iprint.ge.1) write(hrs_cllgu,*) '**  SUBROUTINE H_LINLIN  **'
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'

      text='Compute line by line calibration coefficients'
      call ml_wt('info',text)

! For each channel
      do ich = 1,hrs_mxchn

        gdcal=0

! How many calibration cycles are usable?. Which ones?.
        do icalib=1,calib
          if (calibcoef0(ich,3,icalib) .ne. missdp) then
            gdcal = gdcal + 1
            gdcaltab(gdcal) = icalib
          endif
        enddo

        if (iprint.ge.1) write(hrs_cllgu,fmt=500) ich
        if (gdcal.lt.calib) then
        write(text,'("  Channel ",I2," usable calib cycle only: ",I2)') 
     &        ich,gdcal
        call ml_wt('warning',text)
        endif

! No calibration cycle is usable
        if (gdcal .eq. 0) then
          if (iprint.ge.1) write(hrs_cllgu,fmt=510)
           
          do ilin=lin0,lin1
            calibcoef(ich,1,ilin) = missdp
            calibcoef(ich,2,ilin) = missdp
            calibcoef(ich,3,ilin) = missdp
            if (ich.eq.20) calibcoef(ich,2,ilin)=visibcoef(1)
          enddo
        endif


! Only one calibration cycle is usable
        if (gdcal .eq. 1) then
          if (iprint.ge.1) then
            write(hrs_cllgu,fmt=520) 
            write(hrs_cllgu,fmt=530) (calib-gdcal)
          endif

          kcal1 = gdcaltab(gdcal)
          if (iprint.ge.1.and.(calib-gdcal).ne.0) then
            write(hrs_cllgu,fmt=540) kcal1
          endif

          do ilin=lin0,lin1
            calibcoef(ich,1,ilin) = calibcoef0(ich,1,kcal1)
            calibcoef(ich,2,ilin) = calibcoef0(ich,2,kcal1)
            calibcoef(ich,3,ilin) = calibcoef0(ich,3,kcal1)
          enddo
        endif


! Two,three or four calibration cycles are usable
        if (gdcal .ge. 2) then

          if (iprint.ge.1) then
            write(hrs_cllgu,fmt=550) gdcal
            write(hrs_cllgu,fmt=530) (calib-gdcal)
          endif

          if(iprint.ge.1.and.(calib-gdcal).ne.0) then
            write(hrs_cllgu,fmt=560) 
          endif

          do  k=1,gdcal
            kcal(k) = gdcaltab(k)
            if (iprint.ge.1.and.(calib-gdcal).ne.0) then
              write(hrs_cllgu,*) '            ',kcal(k)
            endif
          enddo

! For each scan line in the orbit
          do ilin=lin0,lin1
         
           if (ilin .le. splintab(kcal(1))) then
! This scan line precedes the first good calibration or is the first
! space scan line
            calibcoef(ich,1,ilin) = calibcoef0(ich,1,kcal(1))
            calibcoef(ich,2,ilin) = calibcoef0(ich,2,kcal(1))
            calibcoef(ich,3,ilin) = calibcoef0(ich,3,kcal(1))

! Update quality(3,ilin) = 'scan line was calibrated using fewer than the
! preferred number of scan lines because the proximity to start and end"            
            quality(3,ilin) = 1

           else

            if (ilin .ge. splintab(kcal(gdcal))) then
! This scan line follows the last good calibration or is the last space scan 
! line
              calibcoef(ich,1,ilin) = calibcoef0(ich,1,kcal(gdcal))
              calibcoef(ich,2,ilin) = calibcoef0(ich,2,kcal(gdcal))
              calibcoef(ich,3,ilin) = calibcoef0(ich,3,kcal(gdcal))

              quality(3,ilin) = 1

            else

! This scan line is between two good calibration cycles. Which ones ?
             kk=1
             do while ( ilin .le. splintab(kcal(gdcal-kk)) ) 
              kk=kk+1
             enddo


! ilin is between calib number (gdcal-kk) and calib number (gdcal-kk+1) 
             kcal1 = kcal(gdcal-kk)
             kcal2 = kcal(gdcal-kk+1)

             w1 = dfloat(splintab(kcal2)-ilin) /
     &                   dfloat(splintab(kcal2)-splintab(kcal1))
             w2 = 1. - w1

             calibcoef(ich,1,ilin) = w1*calibcoef0(ich,1,kcal1)
     &                               +  w2*calibcoef0(ich,1,kcal2)
             calibcoef(ich,2,ilin) = w1*calibcoef0(ich,2,kcal1)
     &                               +  w2*calibcoef0(ich,2,kcal2)
             calibcoef(ich,3,ilin) = w1*calibcoef0(ich,3,kcal1)
     &                               +  w2*calibcoef0(ich,3,kcal2)


            endif

           endif

          enddo

         endif

         if (iprint.ge.2) then
           write(hrs_cllgu,fmt=570) 
           do ilin=lin0,lin1
             write(hrs_cllgu,fmt=580) calibcoef(ich,1,ilin),
     &       calibcoef(ich,2,ilin),calibcoef(ich,3,ilin),ilin
           enddo
         endif

! End of the loop over the 20 channels
       enddo

!! Update the quality array !! 

! quality(6,ilin) =0 if all the channels are calibrated for this scan line 
!                 =1 if some uncalibrated channels of this scan line 

! Update quality(1,ilin) = 'no calibration"
!                        = 1 if no calibration for all the channels 
!                        = 0 if only one or more channel has been calibrated 

       do ilin=lin0,lin1

          badch=0
          do ich=1,hrs_mxchn
            if ( calibcoef(ich,1,ilin).eq.missdp .or.
     &           calibcoef(ich,2,ilin).eq.missdp .or.
     &           calibcoef(ich,3,ilin).eq.missdp ) then
               badch=badch+1
            endif
          enddo
          if (badch.eq.0) quality(6,ilin)=0
          if (badch.ne.0) quality(6,ilin)=1
          if (badch.lt.hrs_mxchn) quality(1,ilin)=0 
          if (badch.ge.hrs_mxchn) quality(1,ilin)=1 

       enddo
       

 500   format(2X,'PROCESSING CHANNEL:',I2)
 510   format(/4X, 'No calibration cycle is usable')
 520   format(/4X,'Only one calibration cycle is usable')
 530   format(4X,I2,' not usable calibration cycle')
 540   format(6X,'the good one is the number:',I2)
 550   format(4X,I2,' usable calibration cycles')
 560   format(6X,'the good ones are the numbers:')
 570   format(/6X,'3 calibration coefficients',39X,'line nb.')
 580   format(3(f20.10,2X),6X,I4)


       return 
       end
