        subroutine h_orderch(inputab,outtab,dim1,dim2,xx)
!+
!NOM/NAME $Id: h_orderch.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!     This module moves the order of the HIRS channels.
!
!     If xx = 0, the moving is :
!       from the order of the level 1b
!          (1,17,2,3,13,4,18,11,19,7,8,20,10,14,6,5,15,12,16,9) 
!       to (1,2,3, ..... ,19,20) 
! 
!     If xx = 1, the moving is :
!       from (1,2,3, ..... ,19,20)
!            (1,17,2,3,13,4,18,11,19,7,8,20,10,14,6,5,15,12,16,9) 
!       to the order of the level 1 b
!
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0     June 95       no              T. LABROT
!         
!ARGUMENTS 
!    input :
!     integer xx              !parameter to determinate the order of the moving
!     integer dim1,dim2       !dimensions of the input array
!     integer inputab(dim1,dim2)  !input array 
!     
!    output :
!     integer outtab(dim1,dim2)   !output array
!                                  the channels has been ordered
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!      
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none                                                     
 
 
!includes/commons
#include "hrsparam.h"

! variables
      integer xx,dim1,dim2
      integer inputab(hrs_mxchn,*),outtab(hrs_mxchn,*)

! local variables
      integer i,j                     !index of the loops
      integer l1border(hrs_mxchn)     !channels in the l1b order
      integer interm(hrs_mxchn)       !intermediate array

!--END OF DECLARATIONS---------------------------------------------------------

       data l1border/ 1,17,2,3,13,4,18,11,19,7,8,20,
     &          10,14,6,5,15,12,16,9 /
       data interm/ 20*0 /

       do i=1,dim2

         if (xx.eq.0) then
           do j=1,dim1
             interm(l1border(j))=inputab(j,i)
           enddo

           do j=1,dim1
             outtab(j,i)=interm(j)
           enddo
         endif


         if (xx.eq.1) then
           do j=1,dim1
             outtab(j,i)=inputab(l1border(j),i)
           enddo
         endif

       enddo

       return
       end
     

            
