        subroutine h_prtstat(iprt,prtread,prtdtnb, dtmean,dtstd,goodprt,
     &                       dtmin,dtmax,idtmin,idtmax)
!+
!NOM/NAME $Id: h_prtstat.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!    This module computes the mean, the maximum and minimum values of a sample
!    PRT readings. 
!
!DESCRIPTION 
!    FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions           
!             
!PROPRIETAIRE/OWNER  
!    Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment                            <author>)
! 1.0      June 95       no                             T. Labrot (MF)
! 1.1      Feb 05     Remove hard-coding parameters          
!                     Add cinit.h include               T. Labrot (MF)
!
!ARGUMENTS 
!   input :
!     integer iprt            !number of the IPRT
!     integer prtdtnb(prtnb)      !total numbers of PRT readings
!     interger*2 prtread(prtnb,*) !array of PRT readings  
!
!   output:
!     integer idtmin,idtmax   !index of the min,max values of PRT readings
!     integer goodprt         !total number of not missing PRT data 
!     double precision dtmean !mean of PRT readings
!     double precision dtstd  !standard deviation of PRT readings 
!     integer*2  dtmin        !minimum value of PRT readings
!     integer*2  dtmax        !maximum value of PRT readings
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!    
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------


!--DECLARATIONS----------------------------------------------------------------

      implicit none 

!includes/commons
#include "hclin.h"
#include "cinit.h"
#include "missingval.h"

! argument variables
      integer iprt,prtdtnb(prtnb),idtmin,idtmax,goodprt
      double precision dtmean,dtstd
      integer*2 prtread(prtnb,*),dtmin,dtmax

! local variables
      integer i         !index of loop
      real val

!--END OF DECLARATIONS----------------------------------------------------------

      dtmean=0.d+0
      dtstd =0.d+0
      goodprt=0
      dtmin=9999
      dtmax=-9999
      idtmin = 0
      idtmax = 0

      if (iprint.ge.2) write(hrs_cllgu,*) 
     &                  '       *subroutine H_PRTSTAT*'

      do i=1,prtdtnb(iprt)

        if (prtread(iprt,i) .ne. missint2) then
        
          val=prtread(iprt,i)
          dtmean = dtmean + val
          dtstd  = dtstd + val*val
          goodprt  = goodprt + 1  

          if (prtread(iprt,i).lt.dtmin) then
            dtmin = prtread(iprt,i)
            idtmin=i
          endif

          if (prtread(iprt,i).gt.dtmax) then
            dtmax = prtread(iprt,i)
            idtmax=i
          endif

        endif

      enddo


      if (goodprt .gt. 0) then

        dtmean = dtmean/goodprt
        dtstd =(dtstd/goodprt)-(dtmean**2)
        if (dtstd .ge. 0.d+0) dtstd =dsqrt(dtstd)

      else

        write(hrs_cllgu,*)'   every PRT data is missing-mean=missing'
        dtmean = missdp
        dtstd  = missdp
        dtmin  = missint2
        dtmax  = missint2
        idtmin = 0
        idtmax = 0

      endif

      if (iprint.ge.2) write(hrs_cllgu,*)
     &    '       *end subroutine H_PRTSTAT*'

      return
      end
