      subroutine h_prtsum(icalib,mean,std,dim,gooddt,ejecdt)
!+
!NOM/NAME $Id: h_prtsum.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!     This module computes cumulative sum of valid PRT readings.
!     This module maintains counters in order to calculate final statistical
!     values.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER 
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      dec. 95      no              T. LABROT
!
!         
!ARGUMENTS
!     input:
!       integer icalib       !number of the current calib cycle
!       dble prec mean(*)    !means array of readings of the 4 PRTs
!       dble prec prtstd(*)  !standard deviations array of 4 PRTs readings
!       integer dim          !dimension of mean and prtstd arrays
!
!     output:
!       integer gooddt(*)    !numbers of readings used to compute the mean
!       integer ejecdt(*)    !numbers of rejected PRT readings  
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------
!--DECLARATIONS----------------------------------------------------------------

      implicit none                                                     


!commons
#include "hclin.h"
#include "hrsparam.h"
#include "missingval.h"
#include "hstatist.h"

! argument variables 
       integer icalib,dim
       integer gooddt(dim),ejecdt(dim)
       double precision mean(dim),std(dim)

! local variables
       integer iprt

! functions

!--END OF DECLARATIONS---------------------------------------------------------

      if (iprint.ge.2) write(hrs_cllgu,*) 
      if (iprint.ge.2) write(hrs_cllgu,*) 
     &                  '  *subroutine H_SUM*'

!-------------------------------
! cumulative variables for PRTs
!-------------------------------


        if (icalib.eq.1) then
          do iprt=1,4
            prtmnsum(iprt)  = 0.d+0
            prtstdsum(iprt) = 0.d+0
            prtgoodsum(iprt)= 0
            prtejecsum(iprt)= 0
            prtstatcnt(iprt)= 0
          enddo
        endif

        do iprt=1,4

          if (mean(iprt).lt.missdp .and. std(iprt).lt.missdp) then
            prtmnsum(iprt)  = prtmnsum(iprt) + mean(iprt)
            prtstdsum(iprt) = prtstdsum(iprt) + std(iprt)
            prtgoodsum(iprt)= prtgoodsum(iprt) + gooddt(iprt)
            prtejecsum(iprt)= prtejecsum(iprt) + ejecdt(iprt)
            prtstatcnt(iprt)= prtstatcnt(iprt) + 1
          endif

        enddo


      if (iprint.ge.2) write(hrs_cllgu,*)
     &    '  *end subroutine H_SUM*'


      return
      end
