        subroutine h_scanpos(noanum, splintab,iwtlintab)
!+
!------------------------------------------------------------------------------
!NOM/NAME $Id: h_scanpos.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!     For each scan line, this module checks the 56 encoder positions and 
!     the line count. It keeps the numbers of lines when the 
!     radiometer is looking to the space and the IWT (internal warm target).
!     It checks if the calibration cycle is full or not.
!     The difference between the HIRS/2 and the HIRS/3 is taken into account.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      June 95       no             T. LABROT
! 1.1      Oct 96    portability        M. Derrien
! 1.2      Dec 96    bug if spind or iwtind=0  T. Labrot
! 1.3      June 98   error in the NOAA order number (TL and MD)
! 1.4      March 03                     T. LABROT 
!  f90 test: warning message for line:if (gp_btn(hrs_qualind(ilin),31).eq..true.) then 
!            Correction: if (gp_btn(hrs_qualind(ilin),31)) then  
! 1.5      Febr 05   remove hard-coding parameters
!                    update the formats for long orbits with nb of lines > 1000
!                    and nb of calib cycles > 10   
!               
!ARGUMENTS 
!     input : 
!      integer noanum                    !order satellite number
!     output: 
!      integer   splintab(hrs_mxcalib)   !number array of space views lines 
!      integer   iwtlintab(hrs_mxcalib)  !number array of IWT scan lines
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!     function  gp_btn !give true or false according to the value of the bit
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none 

!includes/commons
#include "hclin.h"
#include "hrsparam.h"
#include "hrs1bhd.h"
#include "hrs1bdts.h"
#include "usablelines.h"
#include "hstatist.h"
#include "missingval.h"

! argument variables
      integer noanum
      integer splintab(hrs_mxcalib),iwtlintab(hrs_mxcalib)
 
! local variables 
      character*80 text                !variable text
      integer ilin,i,j,k,kk            !index for loops
      integer poscount(4)              !counter of Space,ICT,IWT,Earth lines 
      integer maxposct                 !max value of poscount(4)  
      integer imaxposct(hrs_mxlin)     !index of the max value
      integer splintab0(hrs_mxcalib),iwtlintab0(hrs_mxcalib) 
                 ! initial number arrays of space view lines and IWT scan lines
      integer spind                    !total number of space view line
      integer iwtind                   !total number of IWT scan line
      integer iwtk                     !index of IWT calib cycle
      integer lnct(hrs_mxlin)          !array of line counters
      integer*2 mask                   !mask
      integer*2 spaceval,ictval,iwtval !space,ICT,IWT encoder position values
      integer*2 encod(hrs_mxpix)       !encoder position values
      integer*2 spaceln,ictln,iwtln,earthln0,earthln1
                                    !line number values for space,ICT,IWT,earth
      integer filin                 !first line of first paquet of n lines 
                                    !with correct line counters
      integer*2 dim                 ! array dimension  
      integer*2 nbit                ! bits to extract  
      integer*2 of                  ! offset to apply
      
      logical gp_btn                !function to extract bit(s) and put
                                    !it(s) at right in a 16bits word
      
!--END OF DECLARATIONS---------------------------------------------------------

! Encoder position values for space,ict,iwt (according to the NESS107)
      data spaceval,ictval,iwtval/68,105,156/

      data poscount/4*0/
      data spind,iwtind,iwtk/0,0,0/

      calib=0

! Line count values for space,ICT(not for HIRS/3),IWT and earth: 
!  0,1,2,3 to 39 for the HIRS/2
!  0,1,2 to 39   for the HIRS/3
      spaceln=0

      if (noanum.le.11.and.noanum.ne.0) then
        ictln=1
        iwtln=2
        earthln0=3
        earthln1=39
      endif
      if (noanum.ge.12) then
        ictln=999
        iwtln=1
        earthln0=2
        earthln1=39
      endif

      do i=1,hrs_mxcalib
       splintab0(i) = missint
       iwtlintab0(i)= missint
       splintab(i)  = missint
       iwtlintab(i) = missint
      enddo

      mask=Z'00FF'

      if (iprint.ge.1) write(hrs_cllgu,*) 
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'
      if (iprint.ge.1) write(hrs_cllgu,*) '**  SUBROUTINE H_SCANPOS **'
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'
      
      if(iprint.ge.1)write(hrs_cllgu,*)
      if(iprint.ge.1)write(hrs_cllgu,*) 'FOR INFORMATION, CHECK BIT31'         

      do ilin=lin0,lin1 
! Check bit 31 of the quality indicator bit field.
        if (gp_btn(hrs_qualind(ilin),31)) then 
          text='BIT31=1 ==> No usable scan line number:'
          call ml_wti('notice',text,ilin)
          if (iprint.ge.1) then
            write(hrs_cllgu,fmt=500)ilin
            write(hrs_cllgu,fmt=510)
            write(hrs_cllgu,fmt=520)
          endif
        endif
      enddo

! Check the scan encoder position in each 56 elements of one scan line.
      if(iprint.ge.1)write(hrs_cllgu,*)
      if(iprint.ge.1)write(hrs_cllgu,*) 'ENCODER POSITION'        
      
      do ilin=lin0,lin1
 
        do i=1,4
          poscount(i)=0
        enddo
              
        do j=1,hrs_mxpix
          encod(j)=missint2
          
          dim=1
          nbit=8
          of=0
          
          if (hrs_elem(1,(j-1),ilin) .ne. missint) then
            call gp_x16(hrs_elem(1,(j-1),ilin),dim,nbit,of,encod(j))
          endif

! Built a counter for each position (space,ICT,IWT,earth)
! to find the good position (maximum of the counter).

          if (encod(j) .lt. missint2) then
            if(encod(j).eq.spaceval) poscount(1)=poscount(1)+1
            if(encod(j).eq.ictval) poscount(2)  =poscount(2)+1
            if(encod(j).eq.iwtval) poscount(3)  =poscount(3)+1
            if(encod(j).ge.1 .and. encod(j).le.56) 
     &        poscount(4) = poscount(4) + 1
          endif
        enddo
      
        maxposct=0
        imaxposct(ilin)=0
        
        if(iprint.ge.2) write(hrs_cllgu,fmt=500) ilin
        do j=1,4
          if (iprint.ge.2)  
     &        write(hrs_cllgu,fmt=530) j,poscount(j) 
          if (poscount(j) .gt. maxposct) then
            maxposct = poscount(j) 
            imaxposct(ilin) = j
          endif
        enddo
        
      enddo
      
! Check the line counter.
      do ilin=lin0,lin1
        lnct(ilin)=hrs_elem(3,63,ilin)
      enddo 
      call h_linecount(lnct,filin)
                
! Check the encoder position with line count.
      if(iprint.ge.1) write(hrs_cllgu,*)
      if(iprint.ge.1) write(hrs_cllgu,*)
     &'ENCODER POSITION AND LINE COUNT'

      do ilin=lin0,lin1
      
        if(filin.gt.0) then
          if( (imaxposct(ilin).eq.1.and.lnct(ilin).ne.spaceln)
     &      .or.
     &      ((imaxposct(ilin).eq.2.or.imaxposct(ilin).eq.3) 
     &      .and.(imaxposct(ilin).ne.(lnct(ilin)+(3-iwtln))))
     &      .or.
     &      ((imaxposct(ilin).eq.4).and.
     &      (lnct(ilin).lt.earthln0.or.lnct(ilin).gt.earthln1)) )
     &      then
     
            if(iprint.ge.1) then
            
              write(hrs_cllgu,fmt=500) ilin
              write(hrs_cllgu,fmt=640)
              write(hrs_cllgu,fmt=650)
              
              write(hrs_cllgu,fmt=580)
              if(imaxposct(ilin).eq.1) write(hrs_cllgu,fmt=600)    
              if(imaxposct(ilin).eq.2) write(hrs_cllgu,fmt=610)     
              if(imaxposct(ilin).eq.3) write(hrs_cllgu,fmt=620)
              if(imaxposct(ilin).eq.4) write(hrs_cllgu,fmt=630)

              write(hrs_cllgu,fmt=590)
              if(lnct(ilin).eq.spaceln) write(hrs_cllgu,fmt=600)
              if(lnct(ilin).eq.ictln) write(hrs_cllgu,fmt=610)
              if(lnct(ilin).eq.iwtln) write(hrs_cllgu,fmt=620) 
              if(lnct(ilin).ge.earthln0.and.lnct(ilin).le.earthln1) 
     &          write(hrs_cllgu,fmt=630)
     
            endif
            
          write(text,'("line:",I4," Encod. pos. not agree"
     &    ," with line count. Take the encod. pos.")')ilin
            call ml_wt('notice',text)         
          endif
        endif
          
! If the radiometer is looking to the space (or to the IWT), ranges the 
! number of the line in an array.

        if (imaxposct(ilin).eq.1) then
          spind = spind + 1 
          splintab0(spind)=ilin 
        endif
        if (imaxposct(ilin).eq.3) then
          iwtind = iwtind + 1 
          iwtlintab0(iwtind)=ilin 
        endif
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Reinitialized the value of hrs_scntyp (common hrs1bdts=level1b records)  
! which has been initialized by the decommutation.

       if(imaxposct(ilin).le.3) hrs_scntyp(ilin)=imaxposct(ilin)
       if(imaxposct(ilin).eq.4) hrs_scntyp(ilin) = 0


      enddo

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Initialize hrs_h_autocalind (common hrs1bhd = level1b header) 
! If no calibration lines (no space and no IWT lines), bit0=1
! If one calibration line exists (space one or IWT one), bit0=0
      if (spind.ne.0 .or. iwtind.ne.0) then
       hrs_h_autocalind = 0
      else
       hrs_h_autocalind = 1
      endif

      if (iprint.ge.1) write(hrs_cllgu,fmt=540) spind,iwtind 
      if (iprint.ge.1) write(hrs_cllgu,fmt=560) splintab0
      if (iprint.ge.1) write(hrs_cllgu,fmt=570) iwtlintab0

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Check if the space view line number splintab0(k) belows at the same  
! calibration cycle that the IWT scan line number iwtlintab0(k).
      kk=0
                        
      if (spind.ne.0.and.iwtind.ne.0) then
       do k=1,spind

        if (splintab0(k) .eq. (iwtlintab0(k)-iwtln)) then
! splintab0(k) and iwtlintab0(k) below to the same calibration cycle

          kk = kk + 1     
          splintab(kk) =splintab0(k)
          iwtlintab(kk)=iwtlintab0(k)

        else
! splintab0(k) and iwtlintab0(k) don't below to the same calibration cycle
! research of the index of IWT scan line according to the index k of the 
! space view line.


          iwtk=1
          do while ( (iwtk .lt. iwtind) .and.
     &    (splintab0(k) .ne. (iwtlintab0(iwtk)-iwtln)))
            iwtk=iwtk+1
          enddo
! iwtk is the first correct index of IWT scan line according to the index
! of the space view line

          if ((iwtk.ne.iwtind).or.((iwtk.eq.iwtind) .and.
     &    splintab0(k).eq.(iwtlintab0(iwtk)-iwtln))) then
            
            kk = kk+ 1
            splintab(kk) =splintab0(k)
            iwtlintab(kk)=iwtlintab0(iwtk)
          endif

         endif

        enddo
       endif
       
! Final number of correct calibration cycle
       calib=kk

       text='Number of full calib cycle :'
       call ml_wti('info',text,calib)
       text='First space views line'
       call ml_wti('info',text,splintab(1))
       text='First IWT scan line'
       call ml_wti('info',text,iwtlintab(1))
       if (iprint.ge.1) write(hrs_cllgu,fmt=550) calib
       if (iprint.ge.1) write(hrs_cllgu,fmt=560) splintab
       if (iprint.ge.1) write(hrs_cllgu,fmt=570) iwtlintab
 
 500  format(3X,'scan line number in the orbit: ',I4)    
 510  format(5X,'bit31=1 in the quality indicator bit field')
 520  format(5X,'-- do not use scan for product generation')
 530  format(5X,'pos:',I1,3X,'position counter:',I2)
 540  format(/1X,'nb of space and IWT lines:',32(2X,I2))
 550  format(/1X,'nb of full calib cycle : ',I2)
 560  format(5X,'numbers of the space view lines:',32(2X,I4))
 570  format(5X,'numbers of the IWT scan lines  :',32(2X,I4))
 580  format(7X,'With the encoder position parameter: ')
 590  format(7X,'With the line count parameter: ')
 600  format(24X,'radiometer is looking SPACE')  
 610  format(24X,'radiometer is looking ICT')  
 620  format(24X,'radiometer is looking IWT')  
 630  format(24X,'radiometer is looking EARTH') 
 640  format(7X,'Encoder position not agree with line counter')
 650  format(7X,'We take the encoder position')

      return
      end
