      subroutine h_stat
!+
!NOM/NAME $Id: h_stat.F 47 2006-08-21 08:44:41Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!     This module computes final statistics of the HIRS calibration.
!     It writes statistic record on logical unit hrs_stu 
!     It closes the file 
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER 
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date     comment                 <author>)
! 1.0     Dece. 95   No                      T. LABROT
! 1.1     April 96   Added "typorb" to stat file (TL)
! 1.2     April 96   Replaced output hrs_h_startdatajd by double prec.instant(TL)
! 1.3     Febr. 05   Remove hard-coding parameters
!                    Add cinit.h include for prtnb value    (TL)
!        
!ARGUMENTS
!     input:
!
!     output:
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------
!--DECLARATIONS----------------------------------------------------------------

      implicit none                                                     

!commons
#include "hclin.h"
#include "cinit.h"
#include "hrsparam.h"
#include "hrs1bhd.h"
#include "hstatist.h"
#include "missingval.h"

! argument variables 

! local variables
       integer iprt,ich,icalib       !index for loops
       integer ios                   !variable that receives the error status
       integer statcnt               !number of calib cycles to compute statistics
       integer spdatanb,iwtdatanb    !number of values to compute the percentages 
       integer spgoodstat(hrs_mxirchn),iwtgoodstat(hrs_mxirchn)
!                        number of space and IWT values to compute the percentages 
       integer iq(3)                 !gregorian start instant
       double precision ij           !julian start instant for output
       character*5 orbnum            !orbit number
       character*1 typorb            !ascending 'a' descending 'd'
       character*80 text             !variable text
! functions

!--END OF DECLARATIONS---------------------------------------------------------

      if (iprint.ge.1) write(hrs_cllgu,*)
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'
      if (iprint.ge.1) write(hrs_cllgu,*) '**   SUBROUTINE HSTAT    **'
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'

!------------------------
!  counts of the  PRTs
!------------------------
       do iprt=1,prtnb

         if (prtstatcnt(iprt).gt.0) then

           prtmnstat(iprt)  = prtmnsum(iprt) / prtstatcnt(iprt)
           prtstdstat(iprt)  = prtstdsum(iprt) / prtstatcnt(iprt)
           
           prtejecstat(iprt)= 100.*prtejecsum(iprt) /
     &                        (prtejecsum(iprt)+prtgoodsum(iprt))
     
          endif

       enddo

!-----------------------
!  space view counts
!-----------------------

       do ich=1,hrs_mxirchn

         statcnt = 0
         spdatanb= 0

         do icalib=1,calib
           if (spcntmn(ich,icalib).lt.missdp .and.
     &         spcntstd(ich,icalib).lt.missdp) then

             statcnt = statcnt + 1

             spmnstat(ich)  = spmnstat(ich) + spcntmn(ich,icalib) 
             spstdstat(ich) =spstdstat(ich) + spcntstd(ich,icalib)
             spinfstat(ich) =spinfstat(ich) + spinfcnt(ich,icalib)
             spsupstat(ich) =spsupstat(ich) + spsupcnt(ich,icalib)
             spmisstat(ich) =spmisstat(ich) + spmisscnt(ich,icalib)
             spejecstat(ich)=spejecstat(ich)+ spejeccnt(ich,icalib)
             spgoodstat(ich)=spgoodstat(ich)+ spgoodcnt(ich,icalib)

           endif
         enddo
         
         spmnstat(ich) = spmnstat(ich)  / statcnt
         spstdstat(ich)= spstdstat(ich) / statcnt

         spdatanb = spgoodstat(ich) + spinfstat(ich) + spsupstat(ich)
     &              + spmisstat(ich) + spejecstat(ich)

         spmisstat(ich) =100.*spmisstat(ich)/spdatanb
         spinfstat(ich) =100.*spinfstat(ich)/spdatanb
         spsupstat(ich) =100.*spsupstat(ich)/spdatanb
         spejecstat(ich)=100.*spejecstat(ich)/spdatanb

       enddo


!-----------------------------------
!  IWT radiant signal output counts
!-----------------------------------

       do ich=1,hrs_mxirchn

         statcnt = 0
         iwtdatanb= 0


         do icalib=1,calib
           if (iwtcntmn(ich,icalib).lt.missdp .and.
     &         iwtcntstd(ich,icalib).lt.missdp) then

             statcnt = statcnt + 1

             iwtmnstat(ich) = iwtmnstat(ich) + iwtcntmn(ich,icalib) 
             iwtstdstat(ich) =iwtstdstat(ich) + iwtcntstd(ich,icalib)
             iwtinfstat(ich) =iwtinfstat(ich) + iwtinfcnt(ich,icalib)
             iwtsupstat(ich) =iwtsupstat(ich) + iwtsupcnt(ich,icalib)
             iwtmisstat(ich) =iwtmisstat(ich) + iwtmisscnt(ich,icalib)
             iwtejecstat(ich)=iwtejecstat(ich)+ iwtejeccnt(ich,icalib)
             iwtgoodstat(ich)=iwtgoodstat(ich)+ iwtgoodcnt(ich,icalib)

           endif
         enddo
         
         iwtmnstat(ich) = iwtmnstat(ich)  / statcnt
         iwtstdstat(ich)= iwtstdstat(ich) / statcnt

         iwtdatanb = iwtgoodstat(ich) + iwtinfstat(ich) + 
     &               iwtsupstat(ich) + iwtmisstat(ich) + 
     &               iwtejecstat(ich)

         iwtmisstat(ich) =100.*iwtmisstat(ich)/iwtdatanb
         iwtinfstat(ich) =100.*iwtinfstat(ich)/iwtdatanb
         iwtsupstat(ich) =100.*iwtsupstat(ich)/iwtdatanb
         iwtejecstat(ich)=100.*iwtejecstat(ich)/iwtdatanb

       enddo
       
!-----------------------------------------
! other parameters for the statistics file
!-----------------------------------------
! convert year day and time to julian instant for output
       iq(1)=hrs_h_startdatayr
       iq(2)=hrs_h_startdatady
       iq(3)=hrs_h_startdatatime
       call xiqj(iq,ij)
       
       orbnum = hrs_h_prblkid(2:6)
       
       if (hrs_h_zvel.ge.0.d+0) then
         typorb='a'
       else
         typorb='d'
       endif  
!--------------------------------------
! output information on statistics file
!--------------------------------------
        
       write(hrs_stu,fmt=500,iostat=ios) 
     &   orbnum,typorb,ij,
     &   calib,
     &   calibcoefmn,calibcoefstd,
     &   prtmnstat,prtstdstat,prtejecstat,
     &   spmnstat,spstdstat,spmisstat,spinfstat,
     &   spsupstat,spejecstat,
     &   iwtmnstat,iwtstdstat,iwtmisstat,iwtinfstat,
     &   iwtsupstat,iwtejecstat

  500  format(a5,1x,a1,f11.4,
     &        i2,
     &        20f5.2,20f14.10,20f14.6,20f5.2,20f13.10,20f13.10,
     &        5f9.2,5f9.6,5f5.2,
     &        19f9.2,19f6.3,19f5.2,19f5.2,
     &        19f5.2,19f5.2,
     &        19f9.2,19f6.3,19f5.2,19f5.2,
     &        19f5.2,19f5.2)       


       if(ios.ne.0) then
         call ml_wti('warning',
     &   'problem when writing statistics',ios)
       endif

!--------------------------------------
! close the statistics file
!--------------------------------------
      close(hrs_stu,err=900)
      if(iprint.ge.1) write(hrs_cllgu,*) 'the stat.file has been closed'
      text='the statistics file has been closed'
      call ml_wt('info',text)
      return
       
 900  write(hrs_cllgu,*)'Error closing statistics file'
      text='Error closing statistics file'
      call ml_wt('error',text)
      call aappexit(2)

      end 
