        subroutine h_testcoeffile(lu)
!+
!NOM/NAME $Id: h_testcoeffile.F 114 2008-02-04 15:46:22Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module opens/closes and reads the test coefficients file and  
!     ranges data in common hrs_tstcf (include cinit.h).
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment                    <author>)
! 1.0      Sept. 95      no                     T. LABROT
! 1.1      Mar 04     Modify for linux          N Atkinson / A Korpela
! 1.1      Feb 05     Adde nbslval to be read   T.Labrot
! 1.2      May 07     Change open for g95       P.Marguinaud
! 1.4      July 07    Add a test on ios         T. Labrot (MF)
!         
!ARGUMENTS 
!     input:
!      integer lu        !logical unit for test coefficients file 
!     output:
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
! 
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------
      implicit none 

! includes/commons
#include "cinit.h"
#include "hclin.h"

! argument variables 
      integer lu        !logical unit for test coefficients file

! local variables
      character*80 text           !variable text
      character*80 charact        !buffer storing one line of the file
      integer ios                 !variable that receives the error status
      integer versnb              !number of version of the coef file
      integer versyr              !year of the version of the coef file
      integer versday             !day of the version of the coef file (0/365)

! functions

!--END OF DECLARATIONS---------------------------------------------------------

! Open and read test coefficients file
#ifdef FORTRAN_OPEN_RW
       open(lu,access='sequential',status='old',iostat=ios,
     >   action = 'READ' )
#else
       open(lu,access='sequential',status='old',iostat=ios)

!      If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
       if(ios.ne.0)
     &   open(lu,err=900,access='sequential',
     &      status='unknown',iostat=ios)
#endif

       if(ios.eq.0) then
         text='The test coef file has been opened'
         call ml_wt('info',text)
       endif
       
! Skips the header lines
       read(lu,'(a80)',err=901) charact
       do while (charact(1:1) .eq. '#')
        read(lu,'(a80)',err=901) charact
       enddo
       backspace lu

       read(lu,*,err=901)versnb
       read(lu,*,err=901)versyr
       read(lu,*,err=901)versday
 
       if (iprint.ge.1) then
        write(hrs_cllgu,*) 'test coefficients file' 
        write(hrs_cllgu,500)versnb,versyr,versday
       endif

      write(text,'(2X,"Version nb, year, day of year :",
     &     I2,1X,I4,1X,I3)')versnb,versyr,versday
       call ml_wt('info',text)

       read(lu,'(a80)',err=901) charact
       do while (charact(1:1) .eq. '#')
         read(lu,'(a80)',err=901) charact
       enddo
       backspace lu

       read(lu,*,err=901)sp1fv
       read(lu,*,err=901)minvw
       read(lu,*,err=901)stdevcf
       read(lu,*,err=901)prtminnb
       read(lu,*,err=901)prtbfore
       read(lu,*,err=901)prtafter
       read(hrs_tstcoefu,*,err=901)nbslval

       if (iprint.ge.2) then
         write(hrs_cllgu,fmt=510)
         write(hrs_cllgu,fmt=520)sp1fv,minvw,stdevcf,
     &             prtminnb,prtbfore,prtafter,nbslval
       endif

       close(lu,err=902)

       go to 200


 500  format(2X,' version number: ',I3/'   version year: ',I4/
     &       '   version day of year: ',I3)
 510  format(3X,'sp1fv,minvw,stdevcf,prtminnb,prtbfore',
     &       'prtafter,nbslval')
 520  format(3(3X,I3),4(6X,I3))

 900  write(hrs_cllgu,*)'Error opening test coefficients file'
      text='Test coefficients file could not be opened'
      call ml_wt('error',text)
      call aappexit(2)

 901  write(hrs_cllgu,*)'Error reading test coefficients file'
      text='Test coefficients file could not be read'
      call ml_wt('error',text)
      call aappexit(2)

 902  write(hrs_cllgu,*)'Error closing test coefficients file'
      text='Test coefficients file could not be closed' 
      call ml_wt('error',text)
      call aappexit(2)

 200  return
      end

