      subroutine h_upcommon1(calibcoef,quality)
!+
!NOM/NAME $Id: h_upcommon1.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!     This module finishes to update the parameters in the commons hrs1bhd,  
!     hrs1bdt and hrs1bdts (includes hrs1bhd.h, hrs1bdt.h and hrs1bdts.h).
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0     June 1995       no             T. LABROT
! 1.1      May  98    modified to conform to NOAA format l1b release (TL)
!                 
!         
!ARGUMENTS 
!   input :
!    integer quality(7,hrs_mxlin)
!                 quality information array about the calibration
!    dble prec calibcoef(hrs_mxchn,3,hrs_mxlin) 
!                 third coefs,slopes,intercepts array for each scan line
!
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!   sub.  gp_bse(word,bitnum) to set bitnum in word
!   sub.  gp_bcl(word,bitnum) to clear bitnum in word
!   sub.  h_orderch(inptab,outtab,dim1,dim2,xx) :
!                It moves the order of the HIRS channels.
!    
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none                                                     
 
 
!includes/commons
#include "hclin.h"
#include "cinit.h"
#include "missingval.h"
#include "hrsparam.h"
#include "hrs1bhd.h"
#include "hrs1bdts.h"
#include "usablelines.h"

! argument variables
      integer quality(7,*)
      double precision calibcoef(hrs_mxchn,3,*)

! local variables
      integer i,ich,ilin        !index of loops
      integer coef0(hrs_mxchn,3),coef1(hrs_mxchn,3)
!         third coefs,slopes,intercepts array for each scan line in  
!         different orders 
      integer word              !word where we put quality bits

!--END OF DECLARATIONS----------------------------------------------------------

      if (iprint.ge.1) write(hrs_cllgu,*)
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'
      if (iprint.ge.1) write(hrs_cllgu,*) '*  SUBROUTINE H_UPCOMMON1 *'
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'

!!!   COMMON HRS1BHD FOR THE HEADER   !!!

!    integer*4    hrs_h_tempradcnv(3,hrs_mxirchn)
!                  temperature-radiance conversion                       
!                  order of channels = 1,2,3,4,...,18,19
!                  for nochannel = 1 to 19 :
!                      10^6 x (nochannel central wavenumber for ch1 to ch12) 
!                      10^5 x (nochannel central wavenumber for ch13 to ch19)
!                      10^6 x (nochannel constant 1)
!                      10^6 x (nochannel constant 2)

      do ich=1,hrs_mxirchn
       if (ich.le.12)
     &       hrs_h_tempradcnv(1,ich) =idint(cwave(ich)*(10**6))
       if (ich.ge.13)
     &       hrs_h_tempradcnv(1,ich) =idint(cwave(ich)*(10**5))
       hrs_h_tempradcnv(2,ich) =idint(bdcorr(ich,1)*(10**6))
       hrs_h_tempradcnv(3,ich) =idint(bdcorr(ich,2)*(10**6))
      enddo



!!!   COMMON HRS1BDTS FOR THE RECORDS   !!!

! Update the calibration part in the integer hrs_qualind(hrs_mxlin) 
!    if a bit is on(=1) then the statement is true.
!    bit 28: no calibration (==> =1 at DC level)

! AND

! Update the calibration part in the integer hrs_linqualflgs(hrs_mxlin) 
!            (Note these bits complement the channel indicators; all bits set
!             to 0 indicates normal calibration)
!      bit 15: scan line was not calibrated because of bad time
!              always =0 because no time test in the calibration method
!      bit 14: scan line was calibrated using fewer than the preferred
!              number of scan lines because of proximity to start or 
!              end of data set or to a data gap.
!              =0 if a minimum of two calib cycles have been used to compute
!               the calib coef of this scan line
!              =1 if only one calib cycle has been used.
!      bit 13: scan line was not calibrated because of bad or insufficient
!              PRT data
!              =0 if sufficient PRT data
!              =1 if insufficient PRT data for all calib cycles
!      bit 12: scan line was calibrated but with marginal PRT data
!              always =0 because we don't compute calib coef with marginal data 
!      bit 11: some uncalibrated channels of this scan.See channel indicators
!              =0 if all the channels are calibrated
!              =1 if some uncalibrated channels of this scan
!      bit 10: uncalibrated due to instrument mode
!              =0 if no problem due to instrument mode
!              =1 if uncalibrated due to instrument mode
!      bits 9 and 8: spare

      do ilin=lin0,lin1

        word = hrs_qualind(ilin)

        if (quality(1,ilin).eq.1) then
          call gp_bse(word,28)
        else
          call gp_bcl(word,28)
        endif

        hrs_qualind(ilin) = word


        quality(2,ilin) = 0
        quality(5,ilin) = 0
        word = hrs_linqualflgs(ilin)

        if (quality(2,ilin).eq.1) then
          call gp_bse(word,15)
        else
          call gp_bcl(word,15)
        endif

        if (quality(3,ilin).eq.1) then
          call gp_bse(word,14)
        else
          call gp_bcl(word,14)
        endif

        if (quality(4,ilin).eq.1) then
          call gp_bse(word,13)
        else
          call gp_bcl(word,13)
        endif

        if (quality(5,ilin).eq.1) then
          call gp_bse(word,12)
        else
          call gp_bcl(word,12)
        endif

        if (quality(6,ilin).eq.1) then
          call gp_bse(word,11)
        else
          call gp_bcl(word,11)
        endif

        if (quality(7,ilin).eq.1) then
          call gp_bse(word,10)
        else
          call gp_bcl(word,10)
        endif

        hrs_linqualflgs(ilin) = word

      enddo


! integer hrs_calcof(3,hrs_mxchn,hrs_mxlin) : calibration coefficients
!   order of the channels:  1, 17, 2, 3, 13, 4, 18, 11, 19, 7, 8, 20, 10,
!                           14, 6, 5, 15, 12, 16, 9.
! For the moment we don't update hrs_spcalcof(3,hrs_mxchn,hrs_mxlin)
! (spare calibration coefficients).
! Range the data in integer array

      do ilin=lin0,lin1

        do ich=1,hrs_mxchn

          if (calibcoef(ich,1,ilin).lt.missdp) then
             coef0(ich,1)=idint(calibcoef(ich,1,ilin)*1.d+12)
          else
             coef0(ich,1)=0
          endif

          if (calibcoef(ich,2,ilin).lt.missdp) then
             coef0(ich,2)=idint(calibcoef(ich,2,ilin)*1.d+09)
          else
             coef0(ich,2)=0 
          endif

          if (calibcoef(ich,3,ilin).lt.missdp) then
             coef0(ich,3)=idint(calibcoef(ich,3,ilin)*1.d+06)
          else
             coef0(ich,3)=0 
          endif

        enddo

! Put the coefficients in the good order(=order of the channels in the l1b).

        call h_orderch(coef0,coef1,hrs_mxchn,3,1)

        do ich=1,hrs_mxchn
          do i=1,3
            hrs_calcof(i,ich,ilin)=coef1(ich,i)
          enddo
        enddo
      
      enddo

      return
      end
