      subroutine h_upcommon2(quality)
!+
!NOM/NAME $Id: h_upcommon2.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!     This module updates the parameters in the commons hrs1bhd,  
!     hrs1bdt and hrs1bdts when there is no calibration.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0    october 1995     no             T. LABROT
! 1.1      May  98    modified to conform to NOAA format l1b release (TL)
!         
!ARGUMENTS 
!   input :
!    integer quality(7,hrs_mxlin)
!                 quality information array about the calibration
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!   sub.  gp_bse(word,bitnum) to set bitnum in word
!   sub.  gp_bcl(word,bitnum) to clear bitnum in word
!    
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none                                                     
 
 
!includes/commons
#include "hclin.h"
#include "cinit.h"
#include "hrsparam.h"
#include "hrs1bhd.h"
#include "hrs1bdts.h"
#include "usablelines.h"

! argument variables
      integer quality(7,*)

! local variables
      integer ich,ilin        !index of loops
      integer word            !word where we put quality bits
 
!--END OF DECLARATIONS----------------------------------------------------------

      if (iprint.ge.1) write(hrs_cllgu,*)
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'
      if (iprint.ge.1) write(hrs_cllgu,*) '*  SUBROUTINE H_UPCOMMON2 *'
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'

!!!   COMMON HRS1BHD FOR THE HEADER   !!!

!    integer*4    hrs_h_tempradcnv(3,hrs_mxirchn)
!                  temperature-radiance conversion                       
!                  order of channels = 1,2,3,4,...,18,19
!                  for 10^6 x (nochannel central wavenumber for ch1 to ch12) 
!                      10^5 x (nochannel central wavenumber for ch13 to ch19) 
!                      10^6 x (nochannel constant 1)
!                      10^6 x (nochannel constant 2)

      do ich=1,hrs_mxirchn
       if (ich.le.12)
     &       hrs_h_tempradcnv(1,ich) =idint(cwave(ich)*(10**6))
       if (ich.ge.13)
     &       hrs_h_tempradcnv(1,ich) =idint(cwave(ich)*(10**5))
       hrs_h_tempradcnv(2,ich) =idint(bdcorr(ich,1)*(10**6))
       hrs_h_tempradcnv(3,ich) =idint(bdcorr(ich,2)*(10**6))
      enddo



!!!   COMMON HRS1BDTS FOR THE RECORDS   !!!

! Update the calibration part in the integer hrs_qualind(hrs_mxlin) 
!    if a bit is on(=1) then the statement is true.
!    bit 28: no calibration (==> =1 at DC level)

! AND

! Update the calibration part in the integer hrs_linqualflgs(hrs_mxlin)
! Because there is no calibration cycle, we update only the bit10 
!      (all bits set to 0 indicates normal calibration)
!      bit 15: scan line was not calibrated because of bad time
!              always =0 because no time test in the calibration method
!      bit 14: scan line was calibrated using fewer than the preferred
!              number of scan lines because of proximity to start or 
!              end of data set or to a data gap.
!              =0 if a minimum of two calib cycles have been used to compute
!               the calib coef of this scan line
!              =1 if only one calib cycle has been used.
!      bit 13: scan line was not calibrated because of bad or insufficient
!              PRT data
!              =0 if sufficient PRT data
!              =1 if insufficient PRT data for all calib cycles
!      bit 12: scan line was calibrated but with marginal PRT data
!              always =0 because we don't compute calib coef with marginal data 
!      bit 11: some uncalibrated channels of this scan.See channel indicators
!              =0 if all the channels are calibrated
!              =1 if some uncalibrated channels of this scan
!      bit 10: uncalibrated due to instrument mode
!              =0 if no problem due to instrument mode
!              =1 if uncalibrated due to instrument mode
!      bits 9 and 8: spare

      do ilin=lin0,lin1

        quality(1,ilin)=1 
        word = hrs_qualind(ilin)
        call gp_bse(word,28)
        hrs_qualind(ilin) = word

        word = hrs_linqualflgs(ilin)
        if (quality(7,ilin).eq.1) then
          call gp_bse(word,10)
        else
          call gp_bcl(word,10)
        endif
        hrs_linqualflgs(ilin) = word

      enddo

      return
      end
