        subroutine hclsetu
!+
!NOM/NAME $Id: hclsetu.F 47 2006-08-21 08:44:41Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module opens  the log/debug file (if requested).
!     It opens the HIRS.level1b, reads the header, all the records.
!     The data are ranged in commons hrs1bhd, hrs1bdts
!     (includes hrs1bhd.h, hrs1bdts.h).
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      June 95      no              T. LABROT
! 1.1      Febt 97    Some changes when "ios" not equal to 0  (TL)
! 1.2      May  98    modified to conform to NOAA format l1b release (TL)
! 1.3      Jul 98  force errclock to 0 when problem on file (Marcel Derrien)
! 2.0      August00 Add a test on ierr, output of sp_read (TL)
! 2.1      Dec 00   Display attitude values  P. Brunel
! 2.1      Dec 00   Change position in the code of the display of
!                     the clock error  P. Brunel
! 2.2      Mar 04   Modify for Linux (N Atkinson / A Korpela)
! 2.3      Jun 04   Allow for new 1b header format
! 2.4      Jul 04   Upgrade to navigation code version 2.
!                   def_att call changed for arguments and position in
!                   code (after sp_read)     P. Brunel
! 2.5      Nov 04   Comment out 2.3 change since 2004 NOAA format is backward
!                   compatible.
!                   yawsteering and totattcorr replace filler3. N C Atkinson
! 2.6      Feb 05   Merge with TL's updated code. NCA
! 2.7      Mar 05   Allow for NESDIS data: subtract 4096 from 1b counts. NCA
! 2.8      May 06   Remove tests on n < noaak ( PM/PB )
!         
!ARGUMENTS 
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none                                                     


! includes/commons
#include "hrsparam.h"
#include "hrs1bhd.h"
#include "hrs1bdt.h"
#include "hrs1bdts.h"
#include "hclin.h"
#include "c_1bnav.h"
#include "reclopt.h"

! local variables
      integer i,j        !index of loops
      integer ios        !variable that receives the error status
      integer recn       !record number in the HIRS 1B file
      integer recn0      !first record number in the HIRS 1B file
      integer recn1      !last record number in the HIRS 1B file
      integer ierr
      integer satid
      character*80 text  !variable text 
      character*6 scname
      double precision ijstrt,ijend
      integer loc_rl
      integer att_mode          ! attitude mode
                                ! 0 = NOAA POES operational mode
                                ! 1 = METOP Yaw steering mode
                                ! 2 = Geocentric mode
!functions
      integer ord1bid
      character*6 noascnam
                
!--END OF DECLARATIONS---------------------------------------------------------


! Open the log/debug file (if requested)
      if (iprint.gt.0) then

        open(hrs_cllgu,form='formatted',access='sequential',
     &       status='new',iostat=ios)

!       If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
        if(ios.ne.0)
     &  open(hrs_cllgu,form='formatted',access='sequential',
     &       status='unknown',iostat=ios)

        if(ios.ne.0) then
          text='The log/debug file could not be opened, ios:'
          call ml_wti('error',text,ios)
          call aappexit(2)
        else
          text='The log/debug file has been opened'
          call ml_wt('info',text)

       if (iprint.ge.1) write(hrs_cllgu,*)
       if (iprint.ge.2) write(hrs_cllgu,*) '***************************'
       if (iprint.ge.1) write(hrs_cllgu,*) '**   SUBROUTINE HCLSETU  **'
       if (iprint.ge.2) write(hrs_cllgu,*) '***************************'
       if (iprint.ge.1) write(hrs_cllgu,*)

         if (iprint.ge.1) write(hrs_cllgu,*)
     &   '  the log/debug file has been opened'
        endif

      endif

! Open the HIRS-level1b file
!---------------------------
      loc_rl=reclhrs/rldiv
      open(hrs_l1bu,recl=loc_rl,form='unformatted',access='direct',
     &     status='old',iostat=ios)

!     If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
      if(ios.ne.0)
     &  open(hrs_l1bu,recl=loc_rl,form='unformatted',access='direct',
     &     status='unknown',iostat=ios)

      if(ios.ne.0) then
         text='HIRS1B file could not be opened,  ios:'
         call ml_wti('error',text,ios)
         if (iprint.ge.1) write(hrs_cllgu,*)
     &   '  HIRS1B file could not be opened,ios:',ios
         call aappexit(2)
      else
         text='HIRS1B file has been opened'
         call ml_wt('info',text)
         if (iprint.ge.1) write(hrs_cllgu,*)
     &   '  HIRS1B file has been opened'
      endif

! Read the HIRS-level1b file
!---------------------------

      read(hrs_l1bu,rec=1,iostat=ios) hrs_hd
      
      if(ios.ne.0) then

         text='HIRS1B header could not be read,  ios:'
         call ml_wti('error',text,ios)
         if (iprint.ge.1) write(hrs_cllgu,*)
     &   '  HIRS1B header could not be read,ios:',ios
         call aappexit(2)

      else

       call byteswap1b("HRS1BHD")

! v2.3 Read in new 1b header format if needed. If we are re-calibrating existing
! new-format 1B files then this preserves the information. 
C       IF (hrs_h_l1bversyr .GE. 2003) THEN
C         read(hrs_l1bu,rec=1,iostat=ios) hrs_hd2
C         call byteswap1b("HRS1BHD2")
C       ENDIF

       recn0=hrs_h_hdrcnt+1
       recn1=hrs_h_hdrcnt+hrs_h_scnlin
    
       do recn=recn0,recn1
        read(hrs_l1bu,rec=recn,iostat=ios) hrs_dt
        if(ios.ne.0) then

         text='HIRS1B record could not be read,  ios:'
         call ml_wti('error',text,ios)
         if (iprint.ge.1) write(hrs_cllgu,*)
     &   '  HIRS1B record could not be read, ios:',ios
         call aappexit(2)

        else

         call byteswap1b("HRS1BDT")
         hrs_scnlin(recn - hrs_h_hdrcnt)     = hrs_scnlin0
         hrs_scnlinyr(recn - hrs_h_hdrcnt)   = hrs_scnlinyr0
         hrs_scnlindy(recn - hrs_h_hdrcnt)   = hrs_scnlindy0
         hrs_clockdrift(recn - hrs_h_hdrcnt) = hrs_clockdrift0
         hrs_scnlintime(recn - hrs_h_hdrcnt) = hrs_scnlintime0
         hrs_scnlinf(recn - hrs_h_hdrcnt)    = hrs_scnlinf0
         hrs_mjfrcnt(recn - hrs_h_hdrcnt)    = hrs_mjfrcnt0
         hrs_scnpos(recn - hrs_h_hdrcnt)     = hrs_scnpos0
         hrs_scntyp(recn - hrs_h_hdrcnt)     = hrs_scntyp0
         hrs_qualind(recn - hrs_h_hdrcnt)    = hrs_qualind0
         hrs_linqualflgs(recn - hrs_h_hdrcnt)= hrs_linqualflgs0
         hrs_navstat(recn - hrs_h_hdrcnt)    = hrs_navstat0
         hrs_attangtime(recn - hrs_h_hdrcnt) = hrs_attangtime0
         hrs_rollang(recn - hrs_h_hdrcnt)    = hrs_rollang0
         hrs_pitchang(recn - hrs_h_hdrcnt)   = hrs_pitchang0
         hrs_yawang(recn - hrs_h_hdrcnt)     = hrs_yawang0
         hrs_scalti(recn - hrs_h_hdrcnt)     = hrs_scalti0
         hrs_moonang(recn - hrs_h_hdrcnt)    = hrs_moonang0
         hrs_digitbupdatefg(recn - hrs_h_hdrcnt)=hrs_digitbupdatefg0
         hrs_digitbwrd(recn - hrs_h_hdrcnt)  = hrs_digitbwrd0
         hrs_analogupdatefg(recn - hrs_h_hdrcnt)=hrs_analogupdatefg0

         do i=1,2
           hrs_filler1(i,recn - hrs_h_hdrcnt)= hrs_filler10(i)
         enddo
         do i=1,4
           hrs_filler2(i,recn - hrs_h_hdrcnt)= hrs_filler20(i)
         enddo
         do i=1,3
           hrs_filler3(i,recn - hrs_h_hdrcnt)= hrs_filler30(i)
         enddo
         do i=1,6
           hrs_filler4(i,recn - hrs_h_hdrcnt)= hrs_filler40(i)
         enddo
         do i=1,11
           hrs_filler5(i,recn - hrs_h_hdrcnt)= hrs_filler50(i)
         enddo

         do i=1,3
           hrs_yawsteering(i,recn - hrs_h_hdrcnt) = hrs_yawsteering0(i)
           hrs_totattcorr(i,recn - hrs_h_hdrcnt)  = hrs_totattcorr0(i)
         enddo
           
         do i=1,168
           hrs_ang(i,recn - hrs_h_hdrcnt)    = hrs_ang0(i)
         enddo

         do i=1,112
           hrs_pos(i,recn - hrs_h_hdrcnt)    = hrs_pos0(i)
         enddo

         do i=1,hrs_mxchn
          hrs_chqualflg(i,recn-hrs_h_hdrcnt) = hrs_chqualflg0(i)
         enddo  

         do j=1,3
          do i=1,hrs_mxchn
           hrs_calcof(j,i,recn-hrs_h_hdrcnt)=hrs_calcof0(j,i)
           hrs_scalcof(j,i,recn-hrs_h_hdrcnt)=hrs_scalcof0(j,i)
          enddo
         enddo

         do j=1,24
          do i=0,63 
            hrs_elem(j,i,recn - hrs_h_hdrcnt) = hrs_elem0(j,i)
          enddo
         enddo

!     If this is NESDIS, subtract 4096 from radiometric counts
         IF (hrs_h_siteid .EQ. "NSS") THEN
           do j=3,22
             do i=0,57 
               hrs_elem(j,i,recn - hrs_h_hdrcnt) = 
     &           hrs_elem(j,i,recn - hrs_h_hdrcnt) - 4096
             enddo
           enddo
         ENDIF

         do i=0,63
          hrs_mnfrqual(i,recn-hrs_h_hdrcnt)  = hrs_mnfrqual0(i)
         enddo
         
         do i=1,16
          hrs_anwrd(i,recn-hrs_h_hdrcnt)  = hrs_anwrd0(i)
         enddo 

        endif

       enddo
  
      endif
      
      satid = hrs_h_satid
      satid = ord1bid (satid,hrs_h_startdatajd)
      scname=noascnam(satid)

!
! Open localisation files if requested
!
      if(hrs_lflg.eq.1) then
! Opens and reads the SATPOS file
! -----------------------------------
        open (unit=satposu,status='old',iostat=ios)

!       If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
        if( ios.ne.0 )
     &    open (unit=satposu,status='unknown',iostat=ios)

        if( ios.ne.0 ) then
          call ml_wti('error',
     &       'while opening the satpos file I/O status',ios)
          call ml_wt('error','hclsetu aborted')
          call aappexit(2)
        endif
        
! Read the satpos file
! --------------------
        ijstrt=hrs_h_startdatajd + 
     &      (hrs_h_startdatatime-errclock*1000.)/86400.d3
        ijend=hrs_h_enddatajd + 
     &       (hrs_h_enddatatime-errclock*1000.)/86400.d3
        call sp_read(ijstrt,ijend,satposu,scname,ierr)
        close(unit=satposu,iostat=ios)

        if(ierr.ne.0) then
          call ml_wti('error','return code of sp_read differ from 0',
     &                ierr)
          call ml_wt('error','hclsetu aborted')
          call ml_wt('error','navigation aborted')
          call ml_wt('error','but calibration can work')
          call ml_wt('error','try calibration alone')
          call aappexit(2)
        endif

        if(yaw.lt.-99.d-03) then
          call def_att(scname,att_mode,yaw,roll,pitch)
          call ml_wt('info','take default attitude values')
          call ml_wti('info','default attitude mode ',att_mode)
          call ml_wtdp('info','default attitude yaw ',yaw)
          call ml_wtdp('info','default attitude roll ',roll)
          call ml_wtdp('info','default attitude pitch ',pitch)
        else
          call ml_wtdp('info','attitude yaw ',yaw)
          call ml_wtdp('info','attitude roll ',roll)
          call ml_wtdp('info','attitude pitch ',pitch)
        endif
        
        call calatt(yaw,roll,pitch,att)
        
! If the file has already been located we do not correct
! again from the clock error
! --------------------------

        if(hrs_h_epoyr.le.1950) then
        
          open (unit=ckdriftu,status='old',iostat=ios)
          if(ios.ne.0)
     &      open (unit=ckdriftu,status='unknown',iostat=ios)
          if(ios.ne.0) then
            call ml_wti('error',
     &       'opening clock error file I/O status',ios)
            call aappexit(2)
          endif
          
          ijstrt=hrs_h_startdatajd + 
     &      (hrs_h_startdatatime)/86400.d3
          call clkerr_get(ckdriftu,ijstrt,errclock,ierr)          
          close(unit=ckdriftu,iostat=ios)

          if(ierr.ne.0.or.abs(errclock).gt.2.) then
            call ml_wti('warning',
     &       'reading clock error value from clkerr_get ',ierr)
            call ml_wtr('warning','clock error from file',
     &                 errclock)
            call ml_wt('warning','clock error forced to 0')
            errclock=0. 
          else
            call ml_wtr('info','clock error value (sec)',errclock)
          endif
          
        else
        
          errclock=0.
          call ml_wt('info',
     &     '1B file already corrected from clock error')
     
        endif
        
! Init earth location parameters
! ------------------------------
        first_pnav = 1
        first_lnav = 1
        step_pnav = 1
        step_lnav = 1
        nb_pnav = 56
      endif      
 
      return
      end

