        subroutine hl1bwrt
!+
!NOM/NAME $Id: hl1bwrt.F 47 2006-08-21 08:44:41Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module writes in the level1b file, header and records.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      June 95      no              T. LABROT
! 1.1      Febr 97    Some changes when "ios" not equal to 0  (TL)
! 1.2      May  98    modified to conform to NOAA format l1b release (TL) 
! 1.3      Nov  03    Accommodate new NOAA format (N.C.Atkinson) 
! 1.4      Nov  04    Comment out 1.3 change as 2004 NOAA format is backward
!                     compatible.
!                     yawsteering and totattcorr replace filler3. N C Atkinson
! 1.5      Feb  05    Merge with TL's update. NCA
! 1.6      Mar  05    Allow for NESDIS data: add 4096 to 1b counts. NCA
!         
!ARGUMENTS 
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none                                                     


! includes/commons
#include "hrsparam.h"
#include "hrs1bhd.h"
#include "hrs1bdt.h"
#include "hrs1bdts.h"
#include "hclin.h"

! local variables
      integer i,ii,j,jj  !index of loops
      integer ios        !variable that receives the error status
      integer recn       !record number in the HIRS 1B file
      integer recn0      !first record number in the HIRS 1B file
      integer recn1      !last record number in the HIRS 1B file
      character*80 text  !variable text
      
!--END OF DECLARATIONS---------------------------------------------------------

      if (iprint.ge.1) write(hrs_cllgu,*)
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'
      if (iprint.ge.1) write(hrs_cllgu,*) '**   SUBROUTINE HL1BWRT  **'
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'


      write(hrs_l1bu,rec=1,iostat=ios) hrs_hd
      
      if(ios.ne.0) then
       text='HIRS1B header could not be written,  ios:'
       call ml_wti('error',text,ios)
       if (iprint.ge.1) write(hrs_cllgu,*)
     &   '  HIRS1B header could not be written,ios:',ios
       call aappexit(2)
      endif

! Write in the records of the level 1b
      recn0=hrs_h_hdrcnt+1
      recn1=hrs_h_hdrcnt+hrs_h_scnlin

      do recn=recn0,recn1
        hrs_scnlin0     = hrs_scnlin(recn - hrs_h_hdrcnt)
        hrs_scnlinyr0   = hrs_scnlinyr(recn - hrs_h_hdrcnt)
        hrs_scnlindy0   = hrs_scnlindy(recn - hrs_h_hdrcnt)
        hrs_clockdrift0 = hrs_clockdrift(recn - hrs_h_hdrcnt)
        hrs_scnlintime0 = hrs_scnlintime(recn - hrs_h_hdrcnt)
        hrs_scnlinf0    = hrs_scnlinf(recn - hrs_h_hdrcnt)
        hrs_mjfrcnt0    = hrs_mjfrcnt(recn - hrs_h_hdrcnt)
        hrs_scnpos0     = hrs_scnpos(recn - hrs_h_hdrcnt)
        hrs_scntyp0     = hrs_scntyp(recn - hrs_h_hdrcnt)
        hrs_qualind0    = hrs_qualind(recn - hrs_h_hdrcnt)
        hrs_linqualflgs0= hrs_linqualflgs(recn - hrs_h_hdrcnt)
        hrs_navstat0    = hrs_navstat(recn - hrs_h_hdrcnt)
        hrs_attangtime0 = hrs_attangtime(recn - hrs_h_hdrcnt)
        hrs_rollang0    = hrs_rollang(recn - hrs_h_hdrcnt)
        hrs_pitchang0   = hrs_pitchang(recn - hrs_h_hdrcnt)
        hrs_yawang0     = hrs_yawang(recn - hrs_h_hdrcnt)
        hrs_scalti0     = hrs_scalti(recn - hrs_h_hdrcnt)
        hrs_moonang0    = hrs_moonang(recn - hrs_h_hdrcnt)
        hrs_digitbupdatefg0 = hrs_digitbupdatefg(recn - hrs_h_hdrcnt)
        hrs_digitbwrd0  = hrs_digitbwrd(recn - hrs_h_hdrcnt)
        hrs_analogupdatefg0 = hrs_analogupdatefg(recn - hrs_h_hdrcnt)

        do i=1,2
          hrs_filler10(i) = hrs_filler1(i,recn - hrs_h_hdrcnt)
        enddo
        do i=1,4
          hrs_filler20(i) = hrs_filler2(i,recn - hrs_h_hdrcnt)
        enddo
        do i=1,3
          hrs_filler30(i) = hrs_filler3(i,recn - hrs_h_hdrcnt)
        enddo
        do i=1,6
          hrs_filler40(i) = hrs_filler4(i,recn - hrs_h_hdrcnt)
        enddo
        do i=1,11
          hrs_filler50(i) = hrs_filler5(i,recn - hrs_h_hdrcnt)
        enddo

        do i=1,3
         hrs_yawsteering0(i) = hrs_yawsteering(i,recn - hrs_h_hdrcnt)
         hrs_totattcorr0(i)  = hrs_totattcorr(i,recn - hrs_h_hdrcnt)
        enddo

        do i=1,168
          hrs_ang0(i) = hrs_ang(i,recn - hrs_h_hdrcnt)
        enddo

        do i=1,112
          hrs_pos0(i) = hrs_pos(i,recn - hrs_h_hdrcnt)
        enddo

        do i=1,hrs_mxchn
          hrs_chqualflg0(i) = hrs_chqualflg(i,(recn-hrs_h_hdrcnt))
        enddo

        do j=1,3
         do i=1,hrs_mxchn
           hrs_calcof0(j,i)  = hrs_calcof(j,i,recn-hrs_h_hdrcnt)
           hrs_scalcof0(j,i) = hrs_scalcof(j,i,recn-hrs_h_hdrcnt)
         enddo
        enddo

        do ii=1,24
         do jj=0,63 
           hrs_elem0(ii,jj) = hrs_elem(ii,jj,recn - hrs_h_hdrcnt)
         enddo
        enddo

!     If this is NESDIS, add 4096 to radiometric counts
         IF (hrs_h_siteid .EQ. "NSS") THEN
           do ii=3,22
             do jj=0,57
               hrs_elem0(ii,jj) = hrs_elem0(ii,jj) + 4096
             enddo
           enddo
         ENDIF

        do jj=0,63
          hrs_mnfrqual0(jj) =  hrs_mnfrqual(jj,recn - hrs_h_hdrcnt)  
        enddo
        
        do i=1,16
          hrs_anwrd0(i) =  hrs_anwrd(i,recn - hrs_h_hdrcnt)  
        enddo
        


        write(hrs_l1bu,rec=recn,iostat=ios) hrs_dt
        if(ios.ne.0) then
         text='HIRS1B record could not be written,  ios:'
         call ml_wti('error',text,ios)
         if (iprint.ge.1) then
           write(hrs_cllgu,*)'  HIRS1B record could not be written'
           write(hrs_cllgu,*)'        rec nb: ',recn,'   ios: ',ios
         endif
         call aappexit(2)
        endif

      enddo

      return
      end
