        SUBROUTINE h_cinit_algoV4(satid)

!------------------------------------------------------------------------------
!NAME $Id: h_cinit_algoV4.F 55 2006-09-21 08:35:34Z frna $
!
!PURPOSE 
!    This module opens/closes,reads the calibration coefficients file. 
!    Data are ranged in the commons hrs_cldate, hrs_clcf (include hcinit_algoV4.h).
!    This module opens/closes,reads the test coefficients file and 
!    ranges data in the commons hrs_cldate, hrs_tstcf (include hcinit_algoV4.h). 
!    It initializes the elements in the includes that will be updated 
!    during the calibration task (includes hcaliblineinfo.h,hcalibcycleinfo.h)
!
!DESCRIPTION
!    This module is very similar to the module h_cinit.
!    It was created to manage the HIRS calibration algorithm version4.0. 
!
!HISTORY (CHANGES) 
!   (version   date          comment           <author>)
!     1.0     January 05     creation          T. Labrot (MF)
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!                       
!ARGUMENTS 
!     inputs :
!     outputs :
!      INTEGER   satid    !order satellite number
!
!UPDATED ELEMENTS OF INCLUDES 
!    All the elements of the include hcinit_algoV4.h
!    All the elements of the includes hcaliblineinfo.h, hcalibcycleinfo.h
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
! 
!     INTEGER FUNCTION   ord1bid :
!                  delivers the order satellite number for a given 
!                  level 1b id number (PACS number) and a julian date
!     CHARACTER*6 FUNCTION  noascnam :  
!                  delivers the NOAA satellite name for a given order number
!     SUBROUTINE XDJG(DJ,DG) :
!                  removes a Julien date to a Gregorien date
!     SUBROUTINE  h_calibcoeffile_algoV4 to read calib. coef. file
!     SUBROUTINE  h_testcoeffile_algoV4  to read test   coef. file
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE 
                                                     
! includes/commons
#include "hmissingval_algoV4.h"
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hcinit_algoV4.h"
#include "hrs1belem_algoV4.h"
#include "hcalibcycleinfo_algoV4.h"
#include "hcaliblineinfo_algoV4.h"

! argument variables 
      INTEGER satid     !order satellite number
      
! functions
      CHARACTER*6 noascnam  
             ! delivers the order satellite number for a given 
             ! level 1b id number (PACS number) and a julian date
      INTEGER ord1bid        
             ! delivers the NOAA satellite name for a given order number
             
! local variables
      CHARACTER*80 text              !variable text
      INTEGER i,ilin,icalib,iprt,ich !index for loops
      INTEGER hsatid0                !satellite id from a 1B file
      INTEGER startdate(3)

!--END OF DECLARATIONS---------------------------------------------------------

      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      IF (iprint.GE.2) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '**  SUB. H_CINIT_ALGOV4  **'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)

! Delivers the order satellite number for the l1b satellite number and
! the julian date of the data (arguments must be integer)

      hsatid0 = hsatid
           
      satid = ord1bid(hsatid0,hstartdatajd)
      CALL xdjg(hstartdatajd,startdate)
      IF (iprint.GE.1) THEN
       WRITE(hrs_cllgu,fmt=500) hsatid0
       WRITE(hrs_cllgu,fmt=510) hstartdatajd
       WRITE(hrs_cllgu,fmt=520) startdate 
      ENDIF
      WRITE (text,'("start of data set (Julian day): ",I6)')
     &hstartdatajd
      CALL ml_wt('info',text)
      WRITE (text,'("start of data set :",I4,2X,I2,2X,I2)')
     &startdate (1),startdate(2),startdate(3)
      CALL ml_wt('info',text)
       
      IF (satid.EQ.0) THEN

       IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=530)  
 
       text='Sat.id. not found or an error occured during the sat.id.' 
     &  //  ' file reading'
       CALL ml_wt('error',text)
       CALL ml_wt('error','h_cinit_algoV4 aborted')
       CALL aappexit(2)


      ELSE

       IF (iprint.GE.1)  WRITE(hrs_cllgu,540) satid
       IF (iprint.GE.1)  WRITE(hrs_cllgu,550) noascnam(satid)
       
       text='order number:'
       CALL ml_wti('info',text,satid)
       text='Satellite :'//noascnam(satid)
       CALL ml_wt('info',text)
     
! Opens/closes and reads the calibration coefficients file
       CALL h_calibcoeffile_algoV4(satid)

      ENDIF     ! IF (satid.EQ.0) THEN

! Opens/closes and reads the test coefficients file
      CALL h_testcoeffile_algoV4  

! Initializes the elements of the include hcalibcycleinfo.h
      calib=0
      goodcalib=0
      badcalib=0
      
      DO icalib=1,hrs_mxcalib
        
        mooncontaflag(icalib)=.FALSE.
        splintab(icalib)=missint
        iwtlintab(icalib)=missint
        
        DO ich=1,hrs_mxchn
          qualified(ich,icalib)=.FALSE.
          DO i=1,3
            calibcoef0(ich,i,icalib)=missdp
          ENDDO  
        ENDDO 
        
        DO ich=1,hrs_mxirchn
         spcntmn(ich,icalib)=missdp
         spcntstd(ich,icalib)=missdp
         spgoodcnt(ich,icalib)=missint
         spmisscnt(ich,icalib)=missint
         spinfcnt(ich,icalib)=missint
         spsupcnt(ich,icalib)=missint
         spejeccnt(ich,icalib)=missint 
         iwtcntmn(ich,icalib)=missdp
         iwtcntstd(ich,icalib)=missdp
         iwtgoodcnt(ich,icalib)=missint
         iwtmisscnt(ich,icalib)=missint
         iwtinfcnt(ich,icalib)=missint
         iwtsupcnt(ich,icalib)=missint
         iwtejeccnt(ich,icalib)=missint
        ENDDO 
          
        DO iprt=1,hrs_mxprtnb
          tmp(iprt,icalib)=0.d+0
          prtmean(iprt,icalib)=0.d+0
          prtstd(iprt,icalib)=0.d+0
          datanb(iprt,icalib)=0
          goodprt(iprt,icalib)=0
          ejecprt(iprt,icalib)=0
        ENDDO
        
        DO iprt=1, hrs_mxprtnb
          tmp(iprt,icalib)=missdp
          prtmean(iprt,icalib)=missdp
          prtstd(iprt,icalib)=missdp
          datanb(iprt,icalib)=missint
          goodprt(iprt,icalib)=missint
          ejecprt(iprt,icalib)=missint
        ENDDO 
         
       ENDDO    ! DO icalib=1,hrs_mxcalib
       
       DO ilin=1,hrs_mxlin
         stt(ilin) = missdp
       ENDDO  

         
! Initializes the elements of the include hcaliblineinfo.h
      lin0=missint
      lin1=missint

      DO ilin=1,hrs_mxlin
        DO i=1,10
          linequality(i,ilin) = missint
        ENDDO  
          
        DO ich=1,hrs_mxchn
         DO i=1,6
            linechquality(i,ich,ilin) = missint
         ENDDO  
        ENDDO
      ENDDO 
   
 500  FORMAT(1X,'NOAA spacecraft identification code: ',I2)
 510  FORMAT(1X,'start of data set (Julian day): ',I6)
 520  FORMAT(1X,'start of data set : ',I4,2X,I2,2X,I2)
 530  FORMAT(1X,'sat.id. not found or an error',
     &   ' occured during the sat.id. file reading')
 540  FORMAT(1X,'NOAA order number: ',I2)   
 550  FORMAT(1X,'NOAA satellite name: ',A6)

      RETURN
      END
     
