      SUBROUTINE h_cntstat_algoV4(sp1fv,minvw,counts, dtmn,dtstd,
     &goodcnt)

!------------------------------------------------------------------------------
!NAME $Id: h_cntstat_algoV4.F 25 2006-02-03 15:43:53Z frna $
!
!PURPOSE 
!    For one scan line (space or IWT one), this module computes the mean and
!    the standard deviation of output counts.
!    HIRS instruments have contamination in the first xx FOVs of the space
!    scan. The variable sp1fov (h_testcoef_algoV4.dat file and after in the
!    include hcinit_algoV4.h) indicates the first field of view in the
!    space scan that is not contaminated. The same FOVS are eliminated
!    from the scan of the warm target.
!    Tests: minimum of filtered data to compute the mean
!
!DESCRIPTION 
!    This module is very similar to the module h_cntstat.
!    It was created to manage the HIRS calibration algorithm version4.0
!
!HISTORY (CHANGES) 
!    (version    date          comment           <author>)
!       1.0     January 05     creation          T. Labrot (MF)
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.          
!         
!ARGUMENTS           
!    inputs :
!     INTEGER sp1fv          !the first field of view in the space scan that
!                             is not contaminated.
!     INTEGER minvw          !the minimum number of space or IWT data  
!                             to compute an average
!     INTEGER counts(hrs_mxpix)      !for one channel and hrs_mxpix FOVs output counts
!
!    outputs :
!     INTEGER goodcnt        !number of values used to compute the mean
!     DOUBLE PRECISION dtmn  !output counts mean for one channel
!     DOUBLE PRECISION dtstd !output counts standard deviation for one channel
!
!UPDATED ELEMENTS OF INCLUDES 
!  
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!------------------------------------------------------------------------------


!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE
 
!includes/commons
#include "hmissingval_algoV4.h"
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"

! argument variables
      INTEGER sp1fv,minvw,counts(hrs_mxpix),goodcnt
      DOUBLE PRECISION dtmn,dtstd

! local variables
      INTEGER i               !index of loop

!--END OF DECLARATIONS---------------------------------------------------------

      dtmn = 0.d+0
      dtstd = 0.d+0
      goodcnt = 0
   
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     &      '   *sub H_CNTSTAT_ALGOV4*'

      DO i=sp1fv,hrs_mxpix
        IF (counts(i).NE.missint) THEN
          dtmn =dtmn + counts(i)
          goodcnt = goodcnt + 1
        ENDIF
      ENDDO

! Test: enough values to compute the mean. 
      IF (goodcnt.GE.minvw) THEN
        dtmn = dtmn/goodcnt
      ELSE
        dtmn = missdp
        IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=500)
      ENDIF
      
      goodcnt = 0
      
      IF (dtmn.NE.missdp) THEN
        
        DO i=sp1fv,hrs_mxpix
          IF (counts(i).NE.missint) THEN
            dtstd = dtstd + (counts(i)-dtmn)**2
            goodcnt = goodcnt + 1
          ENDIF
        ENDDO
      
        IF (goodcnt.GE.minvw) THEN
           dtstd=dsqrt(dtstd/goodcnt)
        ELSE
           dtstd=missdp 
           IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=500) 
        ENDIF
     
      ENDIF      

      IF (iprint.GE.2) THEN
        WRITE(hrs_cllgu,fmt=510) dtmn
        WRITE(hrs_cllgu,fmt=520) dtstd
        WRITE(hrs_cllgu,fmt=530) goodcnt
      ENDIF

      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     & '   *end sub H_CNTSTAT_ALGOV4*'

 500   FORMAT(12X,'not enough data to compute statistics')
 510   FORMAT(12X,'output counts mean = ',F12.5)
 520   FORMAT(12X,'output counts std  = ',F12.5)
 530   FORMAT(12X,'good counts =',I3)

       RETURN
       END
