      SUBROUTINE h_gtb1mean_algoV4(
     & b11,
     & b1mn,
     & b1std)

!------------------------------------------------------------------------------
!NAME $Id: h_gtb1mean_algoV4.F 25 2006-02-03 15:43:53Z frna $
!
!PURPOSE
!    For each channel, this module computes the means and the standard  
!    deviations of the b1 coefficients of all the lines to update 
!    some elements of the l1b file.
!    b1 is the correlation coefficient between the telescope temperature
!    and calibration intercept for a given channel. Details can be found in 
!    NOAA KLM User's guide Section 7.2. 
!    Note that b1mn is different from b1 only if hrs_ttflg.EQ.2 and if the 
!    light conditions change during the orbit pass.
!
!DESCRIPTION 
! 
!HISTORY (CHANGES) 
!    (version     date         comment           <author>)
!       1.0     January 05     creation          T. Labrot (MF)
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!              
!         
!ARGUMENTS     
!
!   inputs :
!    REAL b11(hrs_mxchn,hrs_mxlin) 
!         Correlation coefficient between the telescope temperature
!         and calibration intercept used in the equations
!                 
!   outputs :
!    DBLE PREC b1mn(hrs_mxchn)     ! mean of the b1 coefficients
!    DBLE PREC b1std(hrs_mxchn)    ! standard deviation of the b1 coefficients
!
!UPDATED ELEMENTS OF INCLUDES
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE                                                     
 
!includes/commons
#include "hmissingval_algoV4.h"
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hcaliblineinfo_algoV4.h"

! argument variables
! we work with the dimension hrs_mxchn though b11 has no sence for the visible
! channel
      REAL b11(hrs_mxchn,hrs_mxlin)
      DOUBLE PRECISION b1mn(hrs_mxchn)
      DOUBLE PRECISION b1std(hrs_mxchn)

! local variables
      INTEGER ich,ilin     !index of loops
      INTEGER goodb1       !counter of lines with good data
      
!--END OF DECLARATIONS----------------------------------------------------------

      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      IF (iprint.GE.2) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '**  SUB. H_GTMEAN_ALGOV4 **'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)

      DO ich=1,hrs_mxchn
        b1mn(ich) = 0.d+0
        b1std(ich) = 0.d+0
      ENDDO
      
      IF (iprint.GE.2) 
     &     WRITE(hrs_cllgu,fmt=500)    
      
      DO ich=1,hrs_mxchn
        
        goodb1=0

        DO ilin=lin0,lin1
          IF (b11(ich,ilin).NE.missdp) THEN
           b1mn(ich) = b1mn(ich)+b11(ich,ilin)
           goodb1=goodb1+1
          ENDIF
        ENDDO
        
        IF (goodb1.NE.0) THEN
          b1mn(ich)=b1mn(ich)/goodb1
        ELSE
          b1mn(ich) =missdp
        ENDIF
         
        goodb1=0
      
        IF (b1mn(ich).NE.missdp) THEN
        
          DO ilin=lin0,lin1
            IF (b11(ich,ilin).NE.missdp) THEN
              b1std(ich) = b1std(ich)
     &       + (b11(ich,ilin)-b1mn(ich))**2
              goodb1 = goodb1 + 1
            ENDIF
          ENDDO
      
          IF (goodb1.NE.0) THEN
            b1std(ich)=dsqrt(b1std(ich)/goodb1)
          ELSE
             b1std(ich)=missdp 
          ENDIF
     
        ELSE
         
          b1std(ich)=missdp
         
        ENDIF  
        
        IF (iprint.GE.2) 
     &     WRITE(hrs_cllgu,fmt=510)ich,b1mn(ich),b1std(ich)
       
      ENDDO     !do ich=1,hrs_mxchn


 500  FORMAT(2X,'Mean,std of b1:')
 510  FORMAT('ch: ',I2,2X,2(2X,F13.9))

      RETURN
      END
