      SUBROUTINE h_gtmean_algoV4(
     & calibcoef,b11,
     & slopemn,slopestd,
     & b1mn,b1std)

!------------------------------------------------------------------------------
!NAME $Id: h_gtmean_algoV4.F 25 2006-02-03 15:43:53Z frna $
!
!PURPOSE
!    For each channel, this module computes the means and the standard  
!    deviations of the slopes of all the lines, of the b1 coefficients
!    of all the lines to update some elements of the l1b file.
!    b1 is the correlation coefficient between the telescope temperature
!    and calibration intercept for a given channel. Details can be found in 
!    NOAA KLM User's guide Section 7.2. 
!
!DESCRIPTION 
! 
!HISTORY (CHANGES) 
!    (version     date         comment           <author>)
!       1.0     January 05     creation          T. Labrot (MF)
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!              
!         
!ARGUMENTS     
!
!   inputs :
!    DBLE PREC calibcoef(hrs_mxchn,3,hrs_mxlin) 
!         third auto coefs,intercepts,slopes, array for each scan line, 
!         for each channel
!    REAL b11(hrs_mxchn,hrs_mxlin) 
!         Correlation coefficient between the telescope temperature
!         and calibration intercept used in the equations
!                 
!   outputs :
!    DBLE PREC slopemn(hrs_mxchn)  ! mean of the slopes
!    DBLE PREC slopestd(hrs_mxchn) ! standard deviation of the slopes
!    DBLE PREC b1mn(hrs_mxchn)     ! mean of the b1 coefficients
!    DBLE PREC b1std(hrs_mxchn)    ! standard deviation of the b1 coefficients
!
!UPDATED ELEMENTS OF INCLUDES
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!     SUBROUTINE h_orderch(inptab,outtab,dim1,dim2,xx) :
!                It moves the order of the HIRS channels.
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE                                                     
 
!includes/commons
#include "hmissingval_algoV4.h"
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hcaliblineinfo_algoV4.h"

! argument variables
      REAL b11(hrs_mxchn,hrs_mxlin)
      DOUBLE PRECISION calibcoef(hrs_mxchn,3,hrs_mxlin)
      DOUBLE PRECISION slopemn(hrs_mxchn)
      DOUBLE PRECISION slopestd(hrs_mxchn)
      DOUBLE PRECISION b1mn(hrs_mxchn)
      DOUBLE PRECISION b1std(hrs_mxchn)

! local variables
      INTEGER ich,ilin     !index of loops
      INTEGER goodslope    !counter of lines with good data
      INTEGER goodb1
      
!--END OF DECLARATIONS----------------------------------------------------------

      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      IF (iprint.GE.2) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '**  SUB. H_GTMEAN_ALGOV4 **'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)

      DO ich=1,hrs_mxchn
        slopemn(ich) = 0.d+0
        slopestd(ich)= 0.d+0
        b1mn(ich) = 0.d+0
        b1std(ich) = 0.d+0
      ENDDO
      
      IF (iprint.GE.2) 
     &     WRITE(hrs_cllgu,fmt=500)    
      
      DO ich=1,hrs_mxchn
        
        goodslope=0
        goodb1=0

        DO ilin=lin0,lin1
          IF (calibcoef(ich,2,ilin).NE.missdp) THEN
           slopemn(ich) = slopemn(ich)+calibcoef(ich,2,ilin)
           goodslope=goodslope+1
          ENDIF
          IF (b11(ich,ilin).NE.missdp) THEN
           b1mn(ich) = b1mn(ich)+b11(ich,ilin)
           goodb1=goodb1+1
          ENDIF
        ENDDO

        IF (goodslope.NE.0) THEN
          slopemn(ich)=slopemn(ich)/goodslope
        ELSE
          slopemn(ich) =missdp
        ENDIF
        
        IF (goodb1.NE.0) THEN
          b1mn(ich)=b1mn(ich)/goodb1
        ELSE
          b1mn(ich) =missdp
        ENDIF
         
        goodslope=0
        goodb1=0
     
        IF (slopemn(ich).NE.missdp) THEN
        
          DO ilin=lin0,lin1
            IF (calibcoef(ich,2,ilin).NE.missdp) THEN
              slopestd(ich) = slopestd(ich)
     &       + (calibcoef(ich,2,ilin)-slopemn(ich))**2
              goodslope = goodslope + 1
            ENDIF
          ENDDO
      
          IF (goodslope.NE.0) THEN
            slopestd(ich)=dsqrt(slopestd(ich)/goodslope)
          ELSE
             slopestd(ich)=missdp 
          ENDIF
     
        ELSE
         
          slopestd(ich)=missdp
         
        ENDIF 
      
        IF (b1mn(ich).NE.missdp) THEN
        
          DO ilin=lin0,lin1
            IF (b11(ich,ilin).NE.missdp) THEN
              b1std(ich) = b1std(ich)
     &       + (b11(ich,ilin)-b1mn(ich))**2
              goodb1 = goodb1 + 1
            ENDIF
          ENDDO
      
          IF (goodb1.NE.0) THEN
            b1std(ich)=dsqrt(b1std(ich)/goodb1)
          ELSE
             b1std(ich)=missdp 
          ENDIF
     
        ELSE
         
          b1std(ich)=missdp
         
        ENDIF  
        
        IF (iprint.GE.2) 
     &     WRITE(hrs_cllgu,fmt=510)ich,slopemn(ich),slopestd(ich),
     &                            b1mn(ich),b1std(ich)

       
      ENDDO     !do ich=1,hrs_mxchn


 500  FORMAT(2X,'Mean,std deviation of the slope, of b1:')
 510  FORMAT('ch: ',I2,2X,F13.9,3(2X,F13.9))

      RETURN
      END
