      SUBROUTINE h_instrtest_algoV4

!------------------------------------------------------------------------------
!NAME $Id: h_instrtest_algoV4.F 55 2006-09-21 08:35:34Z frna $
!
!PURPOSE
!     Checks the instrument status to define which scan lines are usable:
!     The first one = lin0  ;  The last one = lin1
!     Updates the quality array 'linequality'.
!     All data are defined in the include hcaliblineinfo_algoV4.h
!
!DESCRIPTION     
!     This module is very similar to the module h_instrtest.
!     It was created to manage the HIRS calibration algorithm version4.0
!
!HISTORY (CHANGES) 
!    (version   date         comment           <author>)
!     1.0     January 05     creation          T. Labrot (MF)
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!              
!         
!ARGUMENTS 
!
!UPDATED ELEMENTS OF INCLUDES 
!    In hcaliblineinfo_algoV4.h : 
!      integer lin0          !first scan line with usable data
!      integer lin1          !last scan line with usable data
!      integer linequality(10,hrs_mxlin) 
!                            !quality array for each scan line to update
!                            !quality flags in the HIRS level 1b
!                            !1: scan line no usable because the instrument
!                            !   status is OFF
!                            !2: scan line no usable because the bit 31 (do not
!                            !   use scan for product generation) is ON
!                            !3: insufficient data for calibration
!                            !4: scan was not calibrated
!                            !5: scan contains marginal calibration in some 
!                            !   of the IR channels
!                            !6: PRT quality test failed
!                            !7: PRT data marginal, some readings were rejected
!                            !8: scan contains some uncalibrated channels
!                            !9: scan indicates that the normal HIRS calibration 
!                            !   sequence is disabled 
!                            !10: space view scan is lunar contaminated
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!         
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE                                                     
 
!includes/commons
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hrs1belem_algoV4.h"
#include "hcaliblineinfo_algoV4.h"

! argument variables
  
! local variables
      CHARACTER*110 text !variable text 
      INTEGER inststatus !normal value of 'instrument status' field of the level1B
      INTEGER ilin       !index of loop
 
!--END OF DECLARATIONS----------------------------------------------------------

      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      IF (iprint.GE.2) WRITE(hrs_cllgu,*)
     &  '******************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     &  '**  SUB. H_INSTRTEST_ALGOV4 **'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     &  '******************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      
      IF (hsiteid.EQ.'NSS') THEN
        inststatus=61556
        hinststat1 = IAND(hinststat1,65527) ! Allow high filter motor power
      ELSE
        inststatus=49152
      ENDIF 
      IF (iprint.GE.2) WRITE(hrs_cllgu,*) 'inststatus:',inststatus       

! The instrument status doesn't change and the instrument is OFF


      IF (hstatchrecnb.EQ.0 .AND.       
     &    hinststat1.NE.inststatus) THEN
        lin0=0
        lin1=0
        IF (iprint.GE.1) write(hrs_cllgu,*)
     &  '  instrument is OFF and the status does not change'
        text='Instrument is OFF and the status does not change'
        CALL ml_wt('warning',text) 
        DO ilin=1,hscnlin
          linequality(1,ilin)=1
          linequality(4,ilin)=1
        ENDDO   
        RETURN
      ENDIF

! The instrument status doesn't change and the instrument is ON
      IF (hstatchrecnb.EQ.0 .AND. 
     &    hinststat1.EQ.inststatus) THEN
        lin0=1
        lin1=hscnlin
        IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     &  '  first line =',lin0,'    last line =',lin1
        text='First line ='
        CALL ml_wti('info',text,lin0)
        text='Last line ='
        CALL ml_wti('info',text,lin1)
        DO ilin=1,hscnlin
          linequality(1,ilin)=0
        ENDDO  
        RETURN
      ENDIF
      
! The instrument status changes and the instrument is OFF first.
      IF (hstatchrecnb.GT.0 .AND. 
     &    hinststat1.NE.inststatus) THEN
        lin0=hstatchrecnb
        lin1=hscnlin
        IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     &  '  instrument is OFF first and changes at line ',lin0
        IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     &  ' first line =',lin0,'    last line =',lin1
        text='Instrument status is OFF first and changes at scan line'
        CALL ml_wti('warning',text,lin0)
        text='last line ='
        CALL ml_wti('info',text,lin1)
        DO ilin=1,hstatchrecnb-1
          linequality(1,ilin)=1
          linequality(4,ilin)=1
        ENDDO  
        DO ilin= hstatchrecnb,hscnlin
          linequality(1,ilin)=0
        ENDDO  
        RETURN
      ENDIF

! The instrument status changes and the instrument is ON first.
      IF (hstatchrecnb.NE.0 .AND. 
     &    hinststat1.eq.inststatus) THEN
        lin0=1
        lin1=hstatchrecnb-1
        IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     &  '  the instrument is ON first and changes at line',lin1+1
        IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     &  ' first line =',lin0,'    last line =',lin1
        text='first line ='
        CALL ml_wti('info',text,lin0)
        text='last line ='
        CALL ml_wti('info',text,lin1)
        text='Instrument status is ON first and changes at scan line'
        CALL ml_wti('warning',text,lin1+1) 
        DO ilin=1,hstatchrecnb-1
          linequality(1,ilin)=0
        ENDDO
        DO ilin=hstatchrecnb,hscnlin
          linequality(1,ilin)=1
          linequality(4,ilin)=1
        ENDDO   
        RETURN
      ENDIF

! Error in  the value of hstatchrecnb or hinststat1
      lin0=0
      lin1=0
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     & '  value error in the instrument status or the record number'
     & // 'of status change in the header of the level1b file'
      text='Value error in the instrument status or the record number'
     & // 'of status change in the header of the level1b file'
      CALL ml_wt('warning',text) 
      CALL aappexit(10)
      
      END
