        SUBROUTINE h_linecount_algoV4(lnct,filin)

!------------------------------------------------------------------------------
!NAME $Id: h_linecount_algoV4.F 25 2006-02-03 15:43:53Z frna $
!
!PURPOSE 
!    For each scan line, this module checks the line count. 
!
!DESCRIPTION 
!    This module is very similar to the module h_linecount.
!    It was created to manage the HIRS calibration algorithm version4.0
!
!HISTORY (CHANGES) 
!    (version    date         comment           <author>)
!      1.0     January 05     creation          T. Labrot (MF)
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!  
!ARGUMENTS 
!     inputs/outputs : 
!       INTEGER lnct(*)     !array of line counters.In output the values 
!                            have been corrected if necessary 
!     outputs: 
!       INTEGER filin       !first line of first paquet of n lines 
!                            with correct line counters
!
!UPDATED ELEMENTS OF INCLUDES
!      
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE

!includes/commons
#include "hmissingval_algoV4.h"
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hcinit_algoV4.h"
#include "hcaliblineinfo_algoV4.h"


! argument variables
      INTEGER lnct(*),filin
 
! local variables 
      CHARACTER*80 text         !variable text
      INTEGER ilin,j            !index 
      INTEGER count             !value of the line counter
      INTEGER mlnct(hrs_mxlin)  !line count that the line must have
      INTEGER linnb,ilinnb      !numbers of lines between two scan lines     
      INTEGER loop_counter      !loop counter
      LOGICAL error0,error1     !=.false. if no error, =.true. if error 
      
!--END OF DECLARATIONS---------------------------------------------------------

      IF (iprint.GE.1) WRITE(hrs_cllgu,*) 
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) 
     &'   *sub. H_LINECOUNT_ALGOV4 *' 
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) 
      
! Initializes some local variables 
      ilin=lin0
      error0=.TRUE.
      loop_counter=0      
      count=missint
      DO j=1,hrs_mxlin
        mlnct(j) = missint
      ENDDO
        
! Researchs the first packet of nbslval lines with line counts that are agree
      
      DO WHILE 
     &  (ilin.LE.(lin1-nbslval+1).AND.error0.AND.loop_counter.LT.lin1)      
        loop_counter = loop_counter +1
        
        IF (lnct(ilin).LT.0 .OR. lnct(ilin).GT.39) THEN
        
          ilin=ilin + 1
         
          IF(iprint.GE.1)WRITE(hrs_cllgu,fmt=500) ilin,lnct(ilin)
          WRITE(text,'("Scan line",I3,":false or missing line",
     &         " count")')ilin
          CALL ml_wt('notice',text)
          
        ELSE 
 
          IF(iprint.GE.2)WRITE(hrs_cllgu,fmt=510)ilin,lnct(ilin)  
          j=1
          error1=.FALSE.
                   
          DO WHILE ((j.LE.nbslval-1).AND.(.not.error1))
            count=lnct(ilin)+j
            IF(count.GT.39) count=count-40
! We suppose that nbslval will be always inferior to 40 ==> (count-40)<40
            IF(lnct(ilin+j).NE.count) error1=.TRUE.
            j=j+1
          ENDDO
 
          IF (error1) THEN
            error0=.TRUE.
            ilin=ilin+1
          ELSE
            error0=.FALSE.
          ENDIF
          
        ENDIF   !IF (lnct(ilin).LT.0.OR.lnct(ilin).GT.39) THEN 
        
      ENDDO     !DO WHILE

      IF (ilin.GT.(lin1-nbslval+1).AND.error0) THEN

        IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=520)
        
        WRITE(text,'("No packet of ",I2,
     &  "correct consecutive line counters")')nbslval
        CALL ml_wt('warning',text)
        WRITE(text,'("==>encoder position is taken")')
        CALL ml_wt('info',text)
       
      ELSE

        filin=ilin
        
        IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=530) filin
        
        WRITE(text,'("First paquet with correct line counts: ",
     &  "line ",I4)') filin
        CALL ml_wt('info',text)
        
      ENDIF
      
      
      
! Checks the line counter by comparing with the line counters of the first paquet
!--------------------------------------------------------------------------------
      
      IF (filin.GT.0) THEN
       DO ilin=lin0,lin1
       
! Computes the number of lines between the line ilin and filin
         linnb=filin-ilin
         ilinnb=linnb/40
         
! Computes the line count that the scan line must have
         mlnct(ilin)=lnct(filin)-linnb+40*ilinnb
         IF (mlnct(ilin).LT.0) mlnct(ilin)=mlnct(ilin)+40
         IF (mlnct(ilin).GT.39) mlnct(ilin)=mlnct(ilin)-40
! Checks
         IF(lnct(ilin).NE.mlnct(ilin)) THEN
         
           IF(iprint.GE.1) 
     &     WRITE (hrs_cllgu,fmt=540) ilin,lnct(ilin),mlnct(ilin)
  
           WRITE(text,
     &     '("Scan line ",I4,": line counter is false")')ilin
           CALL ml_wt('notice',text)
           WRITE(text,
     &     '(3X,"initial value:",I4,"  corrected value:",I3)')
     &     lnct(ilin),mlnct(ilin)
           CALL ml_wt('notice',text)
        
           lnct(ilin)=mlnct(ilin)
           
! We have chosen to not correct the values of data coming from
! the TIP ==> we don't correct hrs_elem(3,63,ilin)

         ENDIF  !IF(lnct(ilin).NE.mlnct(ilin)) THEN 
         
       ENDDO    !DO ilin=lin0,lin1
      ENDIF     !IF (filin.GT.0) THEN  
      
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     &  '   *end sub H_LINECOUNT_ALGOV4*'
     
 500  FORMAT(5X,'Scan line ',I4,' initial line count false or ',
     &  'missing ',I2)
 510  FORMAT(3X,'Scan line:',I4,'   initial line count ',I2) 
 520  FORMAT(3X,'No paquet of consecutive lines with correct line')
 530  FORMAT(5X,'the first paquet begins at scan line ',I4)        
 540  FORMAT(3X,'Scan line: ',I4,'  line count:  initial',I4,
     &       '  corrected ',I2)
      
      RETURN
      END
  
      

       
