        SUBROUTINE h_median_algoV4(sp1fv,dim,counts,medcount)

!------------------------------------------------------------------------------
!NAME $Id: h_median_algoV4.F 72 2007-04-27 08:21:59Z frna $
!
!PURPOSE 
!    For one scan line (space or IWT one), this module computes the median
!    value of output counts.
!     
!    HIRS instruments have contamination in the first xx FOVs of the space
!    scan. The variable sp1fv (h_testcoef_algoV4.dat file and after in the
!    include hcinit_algoV4.h) indicates the first field of view in the 
!    space scan that is not contaminated. The same FOVS are eliminated
!    from the scan line of the warm target.
!
!DESCRIPTION 
!
!HISTORY (CHANGES) 
!   (version    date          comment           <author>)
!      1.0     January 05     creation          T. Labrot (MF)
!      1.1     25 April 07    Added a test on the count, missing or not
!                             Modified the way of computed the median
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.          
!             
!ARGUMENTS 
!    inputs :
!     INTEGER sp1fv        !the first field of view in the space scan that
!                           is not contaminated.
!     INTEGER dim
!     INTEGER counts(*)    !for one channel and hrs_mxpix FOVs output counts
!
!    outputs :
!     INTEGER medcount     !median of the output counts
!
!UPDATED ELEMENTS OF INCLUDES
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!      
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE                                                    

!includes/commons
#include "hmissingval_algoV4.h"
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"

! argument variables
      INTEGER sp1fv,dim,counts(dim),medcount

! local variables
      INTEGER i,j               !Index of loop
      INTEGER ii                !Total number of good counts. 
                                !Max value of ii is  dim-sp1fv+1
      INTEGER counttab(dim-sp1fv+1)
      INTEGER value		!value of an outpout count for the sorting

!--END OF DECLARATIONS---------------------------------------------------------

      DO i=1,dim-sp1fv+1
        counttab(i)=missint
      ENDDO
       
      value =  missint
      medcount = missint
      ii=0  
      
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     &      '   *sub H_MEDIAN_ALGOV4*'

      DO i=sp1fv,dim 
       
        IF (counts(i).NE.missint) THEN
          ii=ii+1
          counttab(ii)=counts(i)
        ENDIF
            
      ENDDO
      
      WRITE(hrs_cllgu,fmt='(12X,"total number of good counts: ",I2)')
     & ii 
      
      DO i=1,ii-1
      
        DO j=i+1,ii
        
          value=missint         
         
          IF ( counttab(j).LT.counttab(i) ) THEN
          
            value=counttab(i)
            counttab(i)=counttab(j)
            counttab(j)=value
          
          ENDIF
          
        ENDDO
        
      ENDDO 
      
!      DO i=1,ii 
!      WRITE(hrs_cllgu,*)i,'  ',counttab(i)
!      ENDDO 
      
            
      IF ( MOD(ii,2).EQ.1) THEN
! Number of values is odd (uneven)      
        medcount= counttab( (ii+1)/2 )
        
      ELSE
! Number of values is even. The median is the mean of the 2 central values.
! Must be an integer.      
        medcount= counttab( ii/2 ) +  counttab( (ii/2) + 1 )
        medcount= medcount/2
        
      ENDIF
      
      WRITE(hrs_cllgu,fmt='(12X,"median : ",I8)') medcount  
       
       
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     & '   *end sub H_MEDIAN_ALGOV4*'
     
      RETURN
      END
     
            
