        SUBROUTINE h_orderch_algoV4(inputab,outtab,dim1,dim2,xx)

!------------------------------------------------------------------------------
!NAME $Id: h_orderch_algoV4.F 25 2006-02-03 15:43:53Z frna $
!
!PURPOSE
!     This module moves the order of the HIRS channels.
!
!     If xx = 0, the moving is :
!       from the order of the level 1b
!          (1,17,2,3,13,4,18,11,19,7,8,20,10,14,6,5,15,12,16,9) 
!       to (1,2,3, ..... ,19,20) 
! 
!     If xx = 1, the moving is :
!       from (1,2,3, ..... ,19,20)
!            (1,17,2,3,13,4,18,11,19,7,8,20,10,14,6,5,15,12,16,9) 
!       to the order of the level 1 b
!
!DESCRIPTION
!     This module is very similar to the module h_orderch
!     It was created to manage the HIRS calibration algorithm version4.0
!
!HISTORY (CHANGES) 
!    (version     date         comment           <author>)
!       1.0     January 05     creation          T. Labrot (MF)
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!                       
!ARGUMENTS 
!    inputs :
!     INTEGER xx              !parameter to determinate the order of the moving
!     INTEGER dim1,dim2       !dimensions of the input array
!     INTEGER inputab(dim1,dim2)  !input array 
!     
!    outputs :
!     INTEGER outtab(dim1,dim2)   !output array
!                                  the channels has been ordered
!
!UPDATED ELEMENTS OF INCLUDES
! 
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!      
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE                                                    
     
!includes/commons
#include "hrsparam_algoV4.h"

! variables
      INTEGER xx,dim1,dim2
      INTEGER inputab(hrs_mxchn,*),outtab(hrs_mxchn,*)

! local variables
      INTEGER i,j                     !index of the loops
      INTEGER l1border(hrs_mxchn)     !channels in the l1b order
      INTEGER interm(hrs_mxchn)       !intermediate array

!--END OF DECLARATIONS---------------------------------------------------------

       DATA l1border/ 1,17,2,3,13,4,18,11,19,7,8,20,
     &          10,14,6,5,15,12,16,9 /
       DO i=1,hrs_mxchn
         interm(i)=0
       ENDDO  

       DO i=1,dim2

         IF (xx.EQ.0) THEN
           DO j=1,dim1
             interm(l1border(j))=inputab(j,i)
           ENDDO

           DO j=1,dim1
             outtab(j,i)=interm(j)
           ENDDO
         ENDIF


         IF (xx.EQ.1) THEN
           DO j=1,dim1
             outtab(j,i)=inputab(l1border(j),i)
           ENDDO
         ENDIF

       ENDDO

       RETURN
       END

     

            
