      SUBROUTINE h_partial_superswath_algoV4( 
     &     partialsw,
     &     nselectedslopes,
     &     selectedslopes)

!------------------------------------------------------------------------------      !
!NAME $Id: h_partial_superswath_algoV4.F 72 2007-04-27 08:21:59Z frna $
!
!PURPOSE
!   This module determines the slope cycles which will be involved in
!   the calculation of average slope for each superswath or
!   partial superswath
!
!DESCRIPTION 
!   Reference
!   HIRS calibration Algorithm Version 4.0 (Nov 3, 2003)
!   C. Cao NESDIS
!
!!HISTORY (CHANGES) 
!    (version    date         comment           <author>)
!      1.0     January 05     creation          P. Brunel (MF)
!      1.1     25 April 07     Modified the 'write' command into the log file  
!                             to help for debugging.       T.Labrot (MF)
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT 
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
!ARGUMENTS
!      LOGICAL partialsw(hrs_mxirchn, hrs_mxcalib+1) 
!         ! true if partial superswath for cycle for the slopes
!      INTEGER nselectedslopes(hrs_mxirchn, hrs_mxcalib+1)
!         ! number of selected slopes (1 to 3) for each cycle
!         ! 0 is not possible
!      INTEGER selectedslopes(3,hrs_mxirchn,hrs_mxcalib+1)
!         ! cycle number (1to3) selected slopes
!
!UPDATED ELEMENTS OF INCLUDES 
!    
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!    
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE
 
!includes/commons 
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hcalibcycleinfo_algoV4.h"
      
! argument variables      

! Add artificially 1 to hrs_mxcalib to compute the calibration coefficient 
! after the last calibration cycle.
      LOGICAL partialsw(hrs_mxirchn, hrs_mxcalib+1) 
                     ! true if partial superswath for cycle for the slopes
      INTEGER nselectedslopes(hrs_mxirchn, hrs_mxcalib+1)
                     ! number of selected slopes (1 to 3) for each cycle
                     ! 0 is not possible
      INTEGER selectedslopes(3,hrs_mxirchn,hrs_mxcalib+1)
                     ! cycle number (1to3) selected slopes
      
! local variables
      INTEGER k         ! current cycle number
      INTEGER km1       ! k minus 1
      INTEGER km2       ! k minus 2
      INTEGER kp1       ! k plus 1
      INTEGER kp2       ! k plus 2
      INTEGER ic        ! channel loop indice
      INTEGER icalib    ! calib cycle loop indice
      INTEGER i, i1, i2 ! calib cycle loop indice

      LOGICAL qkm2, qkm1, qk, qkp1, qkp2 ! qualified cycles in interval [-2,2]
      LOGICAL fullsw   ! true if normal superswath 
                       !   with 2 preceeding good cycles
!
!--END OF DECLARATIONS---------------------------------------------------------- 

      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      IF (iprint.GE.2) WRITE(hrs_cllgu,*) 
     &'************************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) 
     &'* SUB H_PARTIAL_SUPERSWATH_ALGOV4  *'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     & '***********************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)

      ! Global loop on channels
      !  qualified array may be different by channels
      !  if some decoding/calculation/filtering problem occured
      !
      DO ic = 1, hrs_mxirchn
      
      !Add artificially 1 to calib to compute the calibration coefficient 
      !after the last calibration cycle.
         DO k = 1, calib+1
            km1 = k - 1
            km2 = k - 2
            kp1 = k + 1
            kp2 = k + 2
            
            IF(km1 .GT. 0) THEN
               qkm1 = qualified(ic, km1) 
            ELSE
               qkm1 = .False.
            ENDIF
            IF(km2 .GT. 0) THEN
               qkm2 = qualified(ic, km2) 
            ELSE
               qkm2 = .False.
            ENDIF
            IF(k .GT. 0) THEN
               qk = qualified(ic, k) 
            ELSE
               qk = .False.
            ENDIF
            IF(kp1 .LE. calib+1) THEN
               qkp1 = qualified(ic, kp1) 
            ELSE
               qkp1 = .False.
            ENDIF
            IF(kp2 .LE. calib+1) THEN
               qkp2 = qualified(ic, kp2) 
            ELSE
               qkp2 = .FALSE.
            ENDIF


            ! Test Full or Partial superswath
            fullsw = qk .AND. qkm1 .AND. qkm2       
            partialsw(ic, k) = .NOT. fullsw

            IF( fullsw ) THEN
               nselectedslopes(  ic, k) = 3
               selectedslopes(1, ic, k) = km2
               selectedslopes(2, ic, k) = km1
               selectedslopes(3, ic, k) = k
            ELSE
               IF ( qk .AND. qkm1 ) THEN
                  ! A     k-1 , k
                  nselectedslopes(  ic, k) = 2
                  selectedslopes(1, ic, k) = km1
                  selectedslopes(2, ic, k) = k
                  selectedslopes(3, ic, k) = 0
               ELSE IF( qkm2 .AND. qk ) THEN
                  ! B     k-2, k
                  nselectedslopes(  ic, k) = 2
                  selectedslopes(1, ic, k) = km2
                  selectedslopes(2, ic, k) = k
                  selectedslopes(3, ic, k) = 0
               ELSE IF( qk .AND. qkp1 ) THEN
                  ! C     k, k+1
                  nselectedslopes(  ic, k) = 2
                  selectedslopes(1, ic, k) = k
                  selectedslopes(2, ic, k) = kp1
                  selectedslopes(3, ic, k) = 0
               ELSE IF( qkm2 .AND. qkm1 ) THEN
                  ! D     k-2, k-1
                  nselectedslopes(  ic, k) = 2
                  selectedslopes(1, ic, k) = km2
                  selectedslopes(2, ic, k) = km1
                  selectedslopes(3, ic, k) = 0
               ELSE IF( qkm1 .AND. qkp1 ) THEN
                  ! E     k-1, k+1
                  nselectedslopes(  ic, k) = 2
                  selectedslopes(1, ic, k) = km1
                  selectedslopes(2, ic, k) = kp1
                  selectedslopes(3, ic, k) = 0
               ELSE IF( qk .AND. qkp2 ) THEN
                  ! F     k, k+2
                  nselectedslopes(  ic, k) = 2
                  selectedslopes(1, ic, k) = k
                  selectedslopes(2, ic, k) = kp2
                  selectedslopes(3, ic, k) = 0
               ELSE IF( qkm2 .AND. qkp1 ) THEN
                  ! G     k-2, k+1
                  nselectedslopes(  ic, k) = 2
                  selectedslopes(1, ic, k) = km2
                  selectedslopes(2, ic, k) = kp1
                  selectedslopes(3, ic, k) = 0
               ELSE IF( qkm2 .AND. qkp2 ) THEN
                  ! H     k-2, k+2
                  nselectedslopes(  ic, k) = 2
                  selectedslopes(1, ic, k) = km2
                  selectedslopes(2, ic, k) = kp2
                  selectedslopes(3, ic, k) = 0
               ELSE IF( qkm1 .AND. qkp2 ) THEN
                  ! I     k-1, k+2
                  nselectedslopes(  ic, k) = 2
                  selectedslopes(1, ic, k) = km1
                  selectedslopes(2, ic, k) = kp2
                  selectedslopes(3, ic, k) = 0
               ELSE IF( qkp1 .AND. qkp2 ) THEN
                  ! J     k+1, k+2
                  nselectedslopes(  ic, k) = 2
                  selectedslopes(1, ic, k) = kp1
                  selectedslopes(2, ic, k) = kp2
                  selectedslopes(3, ic, k) = 0
               ELSE IF( qk ) THEN
                  ! K     k
                  nselectedslopes(  ic, k) = 1
                  selectedslopes(1, ic, k) = k
                  selectedslopes(2, ic, k) = 0
                  selectedslopes(3, ic, k) = 0
               ELSE IF( qkm1 ) THEN
                  ! L     k-1
                  nselectedslopes(  ic, k) = 1
                  selectedslopes(1, ic, k) = km1
                  selectedslopes(2, ic, k) = 0
                  selectedslopes(3, ic, k) = 0
               ELSE IF( qkp1 ) THEN
                  ! M     k+1
                  nselectedslopes(  ic, k) = 1
                  selectedslopes(1, ic, k) = kp1
                  selectedslopes(2, ic, k) = 0
                  selectedslopes(3, ic, k) = 0
               ELSE
                  nselectedslopes(  ic, k) = 0
                  selectedslopes(1, ic, k) = 0
                  selectedslopes(2, ic, k) = 0
                  selectedslopes(3, ic, k) = 0
               ENDIF
            ENDIF    ! if (fullsw)
         ENDDO       ! do calib+1
      ENDDO          ! do channels

      IF (iprint.GE.1) THEN

         DO icalib = 1, calib+1, 30
! Choose 30 for the step to get a printing that is easy to read (if a lot of calib cycles)          
            i1 = icalib
            i2 = icalib + 30 -1
            IF ( i2 .GT. calib+1 ) then
               i2 = calib+1
            ENDIF
            
            DO ic=1,hrs_mxirchn
            
            WRITE(hrs_cllgu,*)
            WRITE(hrs_cllgu,"(a12,30i3)") 'CHANNEL: ',ic 
            WRITE(hrs_cllgu,"(a12,30i3)")      
     &            'cycle       ',(i,i=i1,i2)
            WRITE(hrs_cllgu,"(a12,30(2x,l1))") 
     &            'qualified   ',(qualified(ic,i),i=i1,i2)
            WRITE(hrs_cllgu,"(a12,30(2x,l1))") 
     &            'partial SW  ',(partialsw(ic,i),i=i1,i2)
            WRITE(hrs_cllgu,"(a12,30i3)")      
     &            'Nb selected ',(nselectedslopes(ic,i),i=i1,i2)
            WRITE(hrs_cllgu,"(a12,30i3)")      
     &            'Selected 1  ',(selectedslopes(1,ic,i),i=i1,i2)
            WRITE(hrs_cllgu,"(a12,30i3)")      
     &            'Selected 2  ',(selectedslopes(2,ic,i),i=i1,i2)
            WRITE(hrs_cllgu,"(a12,30i3)")      
     &            'Selected 3  ',(selectedslopes(3,ic,i),i=i1,i2)
            
            ENDDO 
         ENDDO
      ENDIF

      RETURN
      END
