        SUBROUTINE h_prtstat_algoV4(iprt,prtnb,prtread,prtdtnb, dtmean,
     &                      dtstd,goodprt,dtmin,dtmax,idtmin,idtmax)

!------------------------------------------------------------------------------
!NAME $Id: h_prtstat_algoV4.F 46 2006-08-18 14:39:27Z frna $
!
!PURPOSE 
!    This module computes the mean, the maximum and minimum values of a sample
!    PRT readings. 
!
!DESCRIPTION 
!    This module is very similar to the module h_prtstat.
!    It was created to manage the HIRS calibration algorithm version 4.0 
!       
!HISTORY (CHANGES) 
!   (version   date          comment           <author>)
!     1.0     January 05     creation          T. Labrot (MF)
!     1.1     Aug 2006     check prtread not zero  NCA (reported by H Murata)
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!                                    
!         
!ARGUMENTS 
!   inputs :
!     INTEGER iprt                !number of the IPRT
!     INTEGER prtnb               !total number of PRT
!     INTEGER prtdtnb(prtnb)      !total numbers of PRT readings
!     INTEGER*2 prtread(prtnb,*) !array of PRT readings  
!
!   outputs :
!     INTEGER idtmin,idtmax   !index of the min,max values of PRT readings
!     INTEGER goodprt         !total number of not missing PRT data 
!     DBLE PREC dtmean !mean of PRT readings
!     DBLE PREC dtstd  !standard deviation of PRT readings 
!     INTEGER*2  dtmin        !minimum value of PRT readings
!     INTEGER*2  dtmax        !maximum value of PRT readings
!
!UPDATED ELEMENTS OF INCLUDES
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!    
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE 
 
!includes/commons
#include "hclin_algoV4.h"
#include "hmissingval_algoV4.h"

! argument variables
      INTEGER iprt,prtnb,prtdtnb(*),idtmin,idtmax,goodprt
      DOUBLE PRECISION dtmean,dtstd
      INTEGER*2 prtread(prtnb,*),dtmin,dtmax

! local variables
      INTEGER i         !index of loop
      REAL val

!--END OF DECLARATIONS----------------------------------------------------------

! Initializes the outputs
      dtmean=0.d+0
      dtstd =0.d+0
      goodprt=0
      dtmin=9999
      dtmax=-9999
      idtmin = 0
      idtmax = 0

      IF (iprint.GE.1) WRITE(hrs_cllgu,*) 
     &      '   *sub H_PRTSTAT_ALGOV4*'
     
! Computes the mean

      DO i=1,prtdtnb(iprt)

        IF ((prtread(iprt,i) .NE. missint2) .and.
     &      (prtread(iprt,i) .NE. 0)) THEN
        
          val=0.
          val=prtread(iprt,i)
          dtmean = dtmean + val
          goodprt  = goodprt + 1  

          IF (prtread(iprt,i).LT.dtmin) THEN
            dtmin = prtread(iprt,i)
            idtmin=i
          ENDIF

          IF (prtread(iprt,i).GT.dtmax) THEN
            dtmax = prtread(iprt,i)
            idtmax=i
          ENDIF

        ENDIF   !IF (prtread(iprt,i) .NE. missint2) THEN

      ENDDO     !DO i=1,prtdtnb(iprt)


      IF (goodprt .GT. 0) THEN
        dtmean = dtmean/goodprt
      ELSE
       IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     &       '   every PRT data is missing ; mean=missing'
        dtmean = missdp
        dtmin  = missint2
        dtmax  = missint2
        idtmin = 0
        idtmax = 0
      ENDIF

! Computes the standard deviation
      
      goodprt=0
      
      IF (dtmean.NE.missdp) THEN
      
        DO i=1,prtdtnb(iprt)

          IF (prtread(iprt,i) .NE. missint2) THEN
            val=0.
            val=prtread(iprt,i)
            dtstd = dtstd + (val-dtmean)**2
            goodprt  = goodprt + 1 
          ENDIF
          
        ENDDO
        
        IF ( goodprt .NE. 0 ) THEN
          dtstd=dsqrt(dtstd/goodprt)
        ELSE
          dtstd=missdp
          IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     &     '   every PRT data is missing ; stand dev=missing'
        ENDIF
      ELSE      !IF (dtmean.NE.missdp) THEN 
      
        dtstd=missdp 
         
        IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     &     '   mean is missing ; stand dev=missing'
      
      ENDIF     !IF (dtmean.NE.missdp) THEN  
             
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     &    '   *end sub H_PRTSTAT_ALGOV4*'

      RETURN
      END



