      Subroutine h_read_histo_algoV4(
     &   iprint,
     &   file_lu,
     &   log_lu,
     &   ical,
     &   errStatus)

! ------------------------------------------------------------------------------
!NAME $Id: h_read_histo_algoV4.F 25 2006-02-03 15:43:53Z frna $
!
!PURPOSE     
!   Reads next calibration information cycle stored in a
!   file opened on lu logical unit and stores it in the
!   common calibinfo_rec.h with the index ical
!
!DESCRIPTION
!
!HISTORY (CHANGES) 
!    (version    date         comment           <author>)
!      1.0     January 05    Creation          P. Brunel (MF)
!      1.1     13/06/05      Add spcntmed,iwtcntmed,spnedn,iwtnedn 
!                            Change read format of input_lgline
!                                              T.Labrot and P.Brunel (MF)
!      1.2     01/07/05      Use errstatus1 for non-fatal errors. NCA
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT 
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
!ARGUMENTS 
!     inputs :
!        Integer iprint     ! Print option 
!        Integer file_lu    ! logical unit 
!        Integer log_lu     ! logical unit 
!        Integer ical       ! index in common
!     outputs : 
!        Integer errStatus  ! I/O return code
!
!UPDATED ELEMENTS OF INCLUDES 
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)  
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      Implicit none    

! includes/commons      
#include "hrsparam_algoV4.h"
#include "hcalibhisto_algoV4.h"

! Subroutine arguments 
      Integer iprint    ! Print option 
      Integer file_lu   ! logical unit 
      Integer log_lu    ! logical unit 
      Integer ical      ! index in common 
      Integer errStatus   ! I/O return code

! Local variables
      Integer errStatus1  ! non-fatal I/O return code
      Integer c        ! channel number
      Integer iq(3)    ! date
      
      Logical com      ! true if comment line
                       ! (starting either '!' or ' !')

      Character*80  input_line    ! line read from unit 
      Character*150 input_lgline  ! line read from unit 

!--END OF DECLARATIONS---------------------------------------------------------
    

      If (iprint.Ge.1) Then
         Write(log_lu,*) '**  h_read_histo   ** for k ',ical
      End If

      If( ical .Gt. hrs_mxcalibhisto) Then
         Call ml_wt('warning',
     &     'index for common calibhisto reaches maximum')
         errStatus = 999
         Return
      End If

      ! search next start of calibration cycle information
      Read(file_lu,"(A80)",IOSTAT=errStatus) input_line
      Do While( input_line(1:15) .Ne. 'HIRS CALIB INFO' 
     &     .And. errStatus.Eq.0) 
         Read(file_lu,"(A80)",IOSTAT=errStatus) input_line
      End Do


      ! Now Read the data, ignoring the comment lines
      ! starting with a '!' or ' !'

      scname(ical) = 'xxxxxx'
      com = .True.
      Do While ( com .And. errStatus.Eq.0 )
         Read(File_Lu,"(A80)",IOSTAT=errStatus) input_line
         com = input_line(1:1).Eq."!" .Or. input_line(1:2).Eq." !"
      Enddo
      If ( errStatus.Eq.0 ) Then
         scname(ical) = input_line(1:6)
      End If

      ijsp(ical) = 0.
      com = .True.
      Do While ( com .And. errStatus.Eq.0 )
         Read(File_Lu,"(A80)",IOSTAT=errStatus) input_line
         com = input_line(1:1).Eq."!" .Or. input_line(1:2).Eq." !"
      Enddo
      Read(input_line,'(1x,2i4,i10)',IOSTAT=errStatus)iq(1),iq(2),iq(3)
      If ( errStatus.Eq.0 ) Then
         Call xiqj(iq, ijsp(ical))
      End If

      orbnum(ical) = 0
      com = .True.
      Do While ( com .And. errStatus.Eq.0 )
         Read(File_Lu,"(A80)",IOSTAT=errStatus) input_line
         com = input_line(1:1).Eq."!" .Or. input_line(1:2).Eq." !"
      Enddo
      If ( errStatus.Eq.0 ) Then
         Read(input_line,*,IOSTAT=errStatus) orbnum(ical)
      End If

      ascdsc(ical) = 0
      com = .True.
      Do While ( com .And. errStatus.Eq.0 )
         Read(File_Lu,"(A80)",IOSTAT=errStatus) input_line
         com = input_line(1:1).Eq."!" .Or. input_line(1:2).Eq." !"
      Enddo
      If ( errStatus.Eq.0 ) Then
         Read(input_line,*,IOSTAT=errStatus) ascdsc(ical)
      End If

      solzen(ical) = 0.
      com = .True.
      Do While ( com .And. errStatus.Eq.0 )
         Read(File_Lu,"(A80)",IOSTAT=errStatus) input_line
         com = input_line(1:1).Eq."!" .Or. input_line(1:2).Eq." !"
      Enddo
      If ( errStatus.Eq.0 ) Then
         Read(input_line,*,IOSTAT=errStatus) solzen(ical)
      End If

      latitude(ical) = 0.
      com = .True.
      Do While ( com .And. errStatus.Eq.0 )
         Read(File_Lu,"(A80)",IOSTAT=errStatus) input_line
         com = input_line(1:1).Eq."!" .Or. input_line(1:2).Eq." !"
      Enddo
      If ( errStatus.Eq.0 ) Then
         Read(input_line,*,IOSTAT=errStatus) latitude(ical)
      End If

      kcal(ical)= 0
      totcal(ical) = 0
      com = .True.
      Do While ( com .And. errStatus.Eq.0 )
         Read(File_Lu,"(A80)",IOSTAT=errStatus) input_line
         com = input_line(1:1).Eq."!" .Or. input_line(1:2).Eq." !"
      Enddo
      If ( errStatus.Eq.0 ) Then
         Read(input_line,*,IOSTAT=errStatus) kcal(ical), totcal(ical)
      End If

      nlines(ical) = 0
      com = .True.
      Do While ( com .And. errStatus.Eq.0 )
         Read(File_Lu,"(A80)",IOSTAT=errStatus) input_line
         com = input_line(1:1).Eq."!" .Or. input_line(1:2).Eq." !"
      Enddo
      If ( errStatus.Eq.0 ) Then
         Read(input_line,*,IOSTAT=errStatus) nlines(ical)
      End If
      
      Do c = 1, hrs_mxchn
         qualified(c,ical) = .False.
      End Do
      com = .True.
      Do While ( com .And. errStatus.Eq.0 )
         Read(File_Lu,"(A80)",IOSTAT=errStatus) input_line
         com = input_line(1:1).Eq."!" .Or. input_line(1:2).Eq." !"
      Enddo
      If ( errStatus.Eq.0 ) Then
         Read(input_line,'(20l2)',IOSTAT=errStatus) 
     &        (qualified(c,ical),c = 1, hrs_mxchn)
      End If

      splintab(ical) = 0
      com = .True.
      Do While ( com .And. errStatus.Eq.0 )
         Read(File_Lu,"(A80)",IOSTAT=errStatus) input_line
         com = input_line(1:1).Eq."!" .Or. input_line(1:2).Eq." !"
      Enddo
      If ( errStatus.Eq.0 ) Then
         Read(input_line,*,IOSTAT=errStatus) splintab(ical)
      End If

      iwtlintab(ical) = 0
      com = .True.
      Do While ( com .And. errStatus.Eq.0 )
         Read(File_Lu,"(A80)",IOSTAT=errStatus) input_line
         com = input_line(1:1).Eq."!" .Or. input_line(1:2).Eq." !"
      Enddo
      If ( errStatus.Eq.0 ) Then
         Read(input_line,*,IOSTAT=errStatus) iwtlintab(ical)
      End If

      mooncontaflag(ical) = .False.
      com = .True.
      Do While ( com .And. errStatus.Eq.0 )
         Read(File_Lu,"(A80)",IOSTAT=errStatus) input_line
         com = input_line(1:1).Eq."!" .Or. input_line(1:2).Eq." !"
      Enddo
      If ( errStatus.Eq.0 ) Then
         Read(input_line,*,IOSTAT=errStatus) mooncontaflag(ical)
      End If

      ptt(ical) = 0
      stt(ical) = 0
      ttt(ical) = 0
      com = .True.
      Do While ( com .And. errStatus.Eq.0 )
         Read(File_Lu,"(A80)",IOSTAT=errStatus) input_line
         com = input_line(1:1).Eq."!" .Or. input_line(1:2).Eq." !"
      Enddo
      If ( errStatus.Eq.0 ) Then
         Read(input_line,*,IOSTAT=errStatus)
     &        ptt(ical), stt(ical), ttt(ical)
      End If

      fwt(1,ical) = 0
      fwt(2,ical) = 0
      fwt(3,ical) = 0
      fwt(4,ical) = 0
      
      com = .True.
      Do While ( com .And. errStatus.Eq.0 )
         Read(File_Lu,"(A80)",IOSTAT=errStatus) input_line
         com = input_line(1:1).Eq."!" .Or. input_line(1:2).Eq." !"
      Enddo
      If ( errStatus.Eq.0 ) Then
         Read(input_line,*,IOSTAT=errStatus) (fwt(c,ical),c=1,4)
      End If

      bbt(ical) = 0
      com = .True.
      Do While ( com .And. errStatus.Eq.0 )
         Read(File_Lu,"(A80)",IOSTAT=errStatus) input_line
         com = input_line(1:1).Eq."!" .Or. input_line(1:2).Eq." !"
      Enddo
      If ( errStatus.Eq.0 ) Then
         Read(input_line,*,IOSTAT=errStatus) bbt(ical)
      End If

      Do c = 1, hrs_mxchn
         calibcoef0(c,3,ical) = 0
      End Do
      com = .True.
      Do While ( com .And. errStatus.Eq.0 )
         Read(File_Lu,"(A150)",IOSTAT=errStatus) input_lgline
         com = input_lgline(1:1).Eq."!" .Or. input_lgline(1:2).Eq." !"
      Enddo
      If ( errStatus.Eq.0 ) Then
         Backspace File_lu
         Read(File_lu,'(10e15.6)',IOSTAT=errStatus) 
     &        (calibcoef0(c,3,ical), c=1,hrs_mxchn)
      End If

      Do c = 1, hrs_mxchn
         calibcoef0(c,2,ical) = 0
      End Do
      com = .True.
      Do While ( com .And. errStatus.Eq.0 )
         Read(File_Lu,"(A150)",IOSTAT=errStatus) input_lgline
         com = input_lgline(1:1).Eq."!" .Or. input_lgline(1:2).Eq." !"
      Enddo
      If ( errStatus.Eq.0 ) Then
         Backspace File_lu
         Read(File_lu,'(10e15.6)',IOSTAT=errStatus) 
     &        (calibcoef0(c,2,ical), c=1,hrs_mxchn)
      End If

      Do c = 1, hrs_mxchn
         calibcoef0(c,1,ical) = 0
      End Do
      com = .True.
      Do While ( com .And. errStatus.Eq.0 )
         Read(File_Lu,"(A150)",IOSTAT=errStatus) input_lgline
         com = input_lgline(1:1).Eq."!" .Or. input_lgline(1:2).Eq." !"
      Enddo
      If ( errStatus.Eq.0 ) Then
         Backspace File_lu
         Read(File_lu,'(10e15.6)',IOSTAT=errStatus) 
     &        (calibcoef0(c,1,ical), c=1,hrs_mxchn)
      End If

      Do c = 1, hrs_mxirchn
         spcntmn(c,ical)   = 0.
         spcntmed(c,ical)  = 0.
         spcntstd(c,ical)  = 0.
         spnedn(c,ical)    = 0.
         spgoodcnt(c,ical) = 0
         spmisscnt(c,ical) = 0
         spinfcnt(c,ical)  = 0
         spsupcnt(c,ical)  = 0
         spejeccnt(c,ical) = 0
      End Do
      com = .True.
      Do While ( com .And. errStatus.Eq.0 )
         Read(File_Lu,"(A150)",IOSTAT=errStatus) input_lgline
         com = input_lgline(1:1).Eq."!" .Or. input_lgline(1:2).Eq." !"
      Enddo
      If ( errStatus.Eq.0 ) Then
         Read(input_lgline,'(19f7.1)',IOSTAT=errStatus) 
     &        (spcntmn(c,ical),c=1,hrs_mxirchn)
      End If
      If ( errStatus.Eq.0 ) Then
         Read(file_lu,'(19f7.1)',IOSTAT=errStatus) 
     &        (spcntmed(c,ical),c=1,hrs_mxirchn)
      End If
      If ( errStatus.Eq.0 ) Then
         Read(file_lu,'(19f7.2)',IOSTAT=errStatus) 
     &        (spcntstd(c,ical),c=1,hrs_mxirchn)
      End If      
      If ( errStatus.Eq.0 ) Then
         Read(file_lu,'(12f7.3,7(1x,f6.5))',IOSTAT=errStatus1) 
     &        (spnedn(c,ical),c=1,hrs_mxirchn)
      End If
      If ( errStatus.Eq.0 ) Then
         Read(file_lu,'(19i7)',IOSTAT=errStatus) 
     &        (spgoodcnt(c,ical),c=1,hrs_mxirchn)
      End If
      If ( errStatus.Eq.0 ) Then
         Read(file_lu,'(19i7)',IOSTAT=errStatus)
     &        (spmisscnt(c,ical),c=1,hrs_mxirchn)
      End If
      If ( errStatus.Eq.0 ) Then
         Read(file_lu,'(19i7)',IOSTAT=errStatus)
     &        (spinfcnt(c,ical),c=1,hrs_mxirchn)
      End If
      If ( errStatus.Eq.0 ) Then
         Read(file_lu,'(19i7)',IOSTAT=errStatus)
     &        (spsupcnt(c,ical),c=1,hrs_mxirchn)
      End If
      If ( errStatus.Eq.0 ) Then
         Read(file_lu,'(19i7)',IOSTAT=errStatus)
     &        (spejeccnt(c,ical),c=1,hrs_mxirchn)
      End If

      Do c = 1, hrs_mxirchn
         iwtcntmn(c,ical)   = 0.
         iwtcntmed(c,ical)  = 0.
         iwtcntstd(c,ical)  = 0.
         iwtnedn(c,ical)    = 0.
         iwtgoodcnt(c,ical) = 0
         iwtmisscnt(c,ical) = 0
         iwtinfcnt(c,ical)  = 0
         iwtsupcnt(c,ical)  = 0
         iwtejeccnt(c,ical) = 0
      End Do
      com = .True.
      Do While ( com .And. errStatus.Eq.0 )
         Read(File_Lu,"(A150)",IOSTAT=errStatus) input_lgline
         com = input_lgline(1:1).Eq."!" .Or. input_lgline(1:2).Eq." !"
      Enddo
      If ( errStatus.Eq.0 ) Then
         Read(input_lgline,'(19f7.1)',IOSTAT=errStatus) 
     &        (iwtcntmn(c,ical),c=1,hrs_mxirchn)
      End If
      If ( errStatus.Eq.0 ) Then
         Read(file_lu,'(19f7.1)',IOSTAT=errStatus) 
     &        (iwtcntmed(c,ical),c=1,hrs_mxirchn)
      End If
      If ( errStatus.Eq.0 ) Then
         Read(file_lu,'(19f7.2)',IOSTAT=errStatus) 
     &        (iwtcntstd(c,ical),c=1,hrs_mxirchn)
      End If      
      If ( errStatus.Eq.0 ) Then
         Read(file_lu,'(12f7.3,7(1x,f6.5))',IOSTAT=errStatus1) 
     &        (iwtnedn(c,ical),c=1,hrs_mxirchn)
      End If
      If ( errStatus.Eq.0 ) Then
         Read(file_lu,'(19i7)',IOSTAT=errStatus)
     &        (iwtgoodcnt(c,ical),c=1,hrs_mxirchn)
      End If
      If ( errStatus.Eq.0 ) Then
         Read(file_lu,'(19i7)',IOSTAT=errStatus) 
     &        (iwtmisscnt(c,ical),c=1,hrs_mxirchn)
      End If
      If ( errStatus.Eq.0 ) Then
         Read(file_lu,'(19i7)',IOSTAT=errStatus) 
     &        (iwtinfcnt(c,ical),c=1,hrs_mxirchn)
      End If
      If ( errStatus.Eq.0 ) Then
         Read(file_lu,'(19i7)',IOSTAT=errStatus) 
     &        (iwtsupcnt(c,ical),c=1,hrs_mxirchn)
      End If
      If ( errStatus.Eq.0 ) Then
         Read(file_lu,'(19i7)',IOSTAT=errStatus) 
     &        (iwtejeccnt(c,ical),c=1,hrs_mxirchn)
      End If

      Do c = 1, hrs_mxprtnb
         tmp(c,ical)     = 0
         prtmean(c,ical) = 0
         prtstd(c,ical)  = 0
         datanb(c,ical)  = 0
         goodprt(c,ical) = 0
         ejecprt(c,ical) = 0
      End Do
      com = .True.
      Do While ( com .And. errStatus.Eq.0 )
         Read(File_Lu,"(A80)",IOSTAT=errStatus) input_line
         com = input_line(1:1).Eq."!" .Or. input_line(1:2).Eq." !"
      Enddo
      If ( errStatus.Eq.0 ) Then
      Read(input_line,'(5f9.3)',IOSTAT=errStatus) 
     &     (tmp(c,ical),c=1,hrs_mxprtnb)
      End If
      If ( errStatus.Eq.0 ) Then
         Read(file_lu,'(5f9.2)',IOSTAT=errStatus) 
     &        (prtmean(c,ical),c=1,hrs_mxprtnb)
      End If
      If ( errStatus.Eq.0 ) Then
         Read(file_lu,'(5f9.2)',IOSTAT=errStatus) 
     &        (prtstd(c,ical),c=1,hrs_mxprtnb)
      End If
      If ( errStatus.Eq.0 ) Then
         Read(file_lu,'(5i9)',IOSTAT=errStatus)
     &        (datanb(c,ical),c=1,hrs_mxprtnb)
      End If
      If ( errStatus.Eq.0 ) Then
         Read(file_lu,'(5i9)',IOSTAT=errStatus) 
     &        (goodprt(c,ical),c=1,hrs_mxprtnb)
      End If
      If ( errStatus.Eq.0 ) Then
         Read(file_lu,'(5i9)',IOSTAT=errStatus) 
     &        (ejecprt(c,ical),c=1,hrs_mxprtnb)
      End If



      Return
      End
