       SUBROUTINE h_readb1slope_algoV4

!------------------------------------------------------------------------------
!NAME $Id: h_readb1slope_algoV4.F 55 2006-09-21 08:35:34Z frna $
!
!PURPOSE
!    This module opens/closes and reads the 'b1/average slope' file to get 
!    the b1 value and the average slope that will be used in this process.
!    b1 is the correlation coefficient between the telescope temperature
!    and calibration intercept for a given channel.
!
!    If the reference date of the file doesn't correspond to the date of 
!    the current orbit, the values for the average slopes and the B1 
!    parameters are not usable. Those values are set to zero for the ouput.   
!
!DESCRIPTION
!
!HISTORY (CHANGES) 
!    (version    date          comment           <author>)
!       1.0     January 05     creation          T. Labrot (MF)
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!              
!         
!ARGUMENTS   
!
!UPDATED ELEMENTS OF INCLUDES 
!   All the elements of the include hb1averageslope_algoV4.h
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!  SUBROUTINE  xdjg : Convert Julian date into gregorien date
!    
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE                                                    
 
!includes/commons
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hb1averageslope_algoV4.h"
#include "hrs1belem_algoV4.h"

! argument variables
      
! local variables
      CHARACTER*80 text     !variable text
      INTEGER ios           !I/O return code
      INTEGER ich           !index of loop
      INTEGER ch            !one channel
      INTEGER yyyy, mm, dd  !reference date
      INTEGER hh, mn        !reference hour
      INTEGER nbhour        !number of considered hours preceeding data date
      INTEGER dg(3)         !gregorien date

!--END OF DECLARATIONS----------------------------------------------------------

      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      IF (iprint.GE.2) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '*SUB H_READB1SLOPE_ALGOV4 *'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      
! Opens and reads the file hirs_b1aslope.txt
      OPEN(hrs_b1aslopu,access='sequential',status='old',iostat=ios)

!     If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
      IF (ios.NE.0)
     &  OPEN(hrs_b1aslopu,err=900,access='sequential',
     &       status='unknown',iostat=ios)
     
      IF (iprint.GE.1) WRITE(hrs_cllgu,
     &'("The b1/avg slope file has been opened")')
      text='The b1/avg_slope file has been opened'
      CALL ml_wt('info',text)      
     
      READ(hrs_b1aslopu,'(1x,i4,1x,i2.2,1x,i2.2)',err=901) 
     &       yyyy, mm, dd     
      READ(hrs_b1aslopu,'(1x,i2.2,1x,i2.2)',err=901) hh,mn
      READ(hrs_b1aslopu,'(1x,i2.2)',err=901) nbhour
      DO ich=1,hrs_mxirchn 
        READ(hrs_b1aslopu,*,err=901)
     &  ch,avgslope(ich),stdevavgslope(ich),b1(ich),b1n(ich),b1d(ich)
      ENDDO
      
      IF (iprint.GE.2) THEN
       WRITE(hrs_cllgu,*)
       WRITE(hrs_cllgu,'("b1slope file:")') 
       WRITE(hrs_cllgu,'(" reference date: ",i4,i4,i4)')yyyy,mm,dd
       WRITE(hrs_cllgu,'(" reference hour: ",i4,i4)') hh, mn
       WRITE(hrs_cllgu,
     &   '(" number of considered hours preceeding data date:",I4)')
     &   nbhour
       WRITE(hrs_cllgu,fmt=500)
       DO ich=1,hrs_mxirchn
         WRITE(hrs_cllgu,fmt=510)
     &   ich,avgslope(ich),stdevavgslope(ich),b1(ich),b1n(ich),b1d(ich) 
       ENDDO  
      ENDIF
      
      WRITE(text,'("b1slope file: Reference date/hour :",
     &  I4,1X,I2,1X,I2,1X,I2,1X,I2)')yyyy,mm,dd,hh,mn
      CALL ml_wt('info',text)
      WRITE(text,'("b1slope file: nb of hours preceeding date  :",
     &  1X,I2)')nbhour
      CALL ml_wt('info',text)      
      
      CALL xdjg(hstartdatajd,dg)
       
      WRITE(hrs_cllgu,fmt=520)dg(1),dg(2),dg(3)
      
      IF ( dg(1).NE.yyyy .OR. 
     &     dg(2).NE.mm   .OR.  
     &     dg(3).NE.dd       ) THEN
        
        DO ich=1,hrs_mxirchn
           avgslope(ich)=0.
           stdevavgslope(ich)=0.
           b1(ich)      =0.
           b1n(ich)     =0.
           b1d(ich)     =0.
        ENDDO 
          
        WRITE(hrs_cllgu,fmt=530)
        WRITE(hrs_cllgu,fmt=540)
        
        WRITE(text,'("Reference date of b1slope file ",
     &  "different from the one of the current orbit")')
        CALL ml_wt('warning',text)
        WRITE(text,'("avgslope,b1,b1n,b1d are set to zero")')
        CALL ml_wt('warning',text)
        
      ENDIF  
        
        
! Closes the b1slope file
      CLOSE(hrs_b1aslopu,err=902)
      
      GOTO 200
 
 500  FORMAT(1X,'ich',3X,'avgslope',4X,'stdevavgslope',
     &6X,'b1 ',11X,'b1n',11X,'b1d')
 510  FORMAT(1X,I2,5(1X,F13.9)) 
 520  FORMAT(/2X,'Date of the current orbit :',
     &1X,I4,2X,I2,2X,I2) 
 530  FORMAT(/2X,'Reference date of b1slope file ',
     &  'different from the one of the current orbit')
 540  FORMAT(2X,'avgslope,b1,b1n,b1d are set to zero')       

 900  WRITE(hrs_cllgu,*)'Error opening b1slope file'
      text='b1slope file could not be open'
      CALL ml_wt('error',text)
      text='h_readb1slope_algoV4 aborted'
      CALL ml_wt('error',text)
      CALL aappexit(2)

 901  WRITE(hrs_cllgu,*)'Error reading b1slope file'
      text='b1slope file could not be read'
      CALL ml_wt('error',text)
      text='h_readb1slope_algoV4 aborted'
      CALL ml_wt('error',text)
      CALL aappexit(2)

 902  WRITE(hrs_cllgu,*)'Error closing b1slope file'
      text='b1slope file could not be closed' 
      CALL ml_wt('error',text)
      text='h_readb1slope_algoV4 aborted'
      CALL ml_wt('error',text)
      CALL aappexit(2)

 200  RETURN
      END
