        SUBROUTINE h_scanpos_algoV4(noanum)

!------------------------------------------------------------------------------
!NAME $Id: h_scanpos_algoV4.F 58 2006-11-08 10:31:36Z frna $
!
!PURPOSE 
!    For each scan line, this module checks the 56 encoder positions and 
!    the line count. It keeps the numbers of lines when the 
!    radiometer is looking to the space and the IWT (internal warm target).
!    It checks if the calibration cycle is full or not.
!    It checks if one calibration cycle and the previous one are well 
!    separated by 40 scan lines.
!    The difference between the HIRS/3 or 4 is taken into account.
!
!DESCRIPTION
!    This module is very similar to the module h_scanpos.
!    It was created to manage the HIRS calibration algorithm version4.0
!
!  
!HISTORY (CHANGES) 
!    (version    date         comment           <author>)
!      1.0     January 05     creation          T. Labrot (MF)
!      1.1     30/10/06       Don't set splintab/iwtlintab to -999. NCA
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!                       
!ARGUMENTS 
!    inputs : 
!      INTEGER noanum    !order satellite number
!
!    outputs :
!
!UPDATED ELEMENTS OF INCLUDES
!
!  in include h_calibcycleinfo_algoV4.h: 
!    calib               !total number of cabration cycles in the orbit
!    goodcalib,badcalib  !total number of good/bad calib cycles
!    splintab(hrs_mxcalib)  !array of the numbers of space lines
!    iwtlintab(hrs_mxcalib) !array of the numbers of IWT lines
!    qualified(hrs_mxcalib) !=true if the calib cycle is usable
!
!  in include hrs1belem_algoV4.h
!       hscntyp(hrs_mxlin)  !scan type code 
! 
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!     LOGICAL FUNCTION  gp_btn !give true or false according to the value of the bit
!     SUBROUTINE gp_x16        !extract a sample and put it into an integer*2
!     SUBROUTINE h_linecount_algoV4  !check the line count
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE 
                                                     
!includes/commons
#include "hmissingval_algoV4.h"
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hrs1belem_algoV4.h"
#include "hcalibcycleinfo_algoV4.h"
#include "hcaliblineinfo_algoV4.h"

! argument variables
      INTEGER noanum
      
! function     
      LOGICAL gp_btn
      
! local variables 
      CHARACTER*80 text                !variable text
      INTEGER*2 mask                   !mask
      INTEGER*2 spaceval,ictval,iwtval !space,ICT,IWT encoder position values
      INTEGER*2 spaceln,iwtln,earthln0,earthln1
                                       !line values for space,IWT,Earth lines
      INTEGER*2 encod(hrs_mxpix)       !encoder position values                              
      INTEGER*2 dim                    !array dimension for input of gp_x16  
      INTEGER*2 nbit                   !bits to extract for input of gp_x16 
      INTEGER*2 of                     !offset to apply for input of gp_x16                             
      INTEGER ilin,icalib,iicalib,i,j,k,kk,ki  !index 
      INTEGER poscount(3)              !counter of space,IWT,Earth lines 
      INTEGER maxposct                 !max value of poscount(4)  
      INTEGER imaxposct(hrs_mxlin)     !index of the max value
      INTEGER splintab0(hrs_mxcalib),iwtlintab0(hrs_mxcalib) 
                           !initial arrays of the number of space view  
                           !lines and IWT scan lines
      INTEGER k40(hrs_mxcalib)         !number of missing calib cycle         
      INTEGER calib0                   !number of calib cycles 
                                       !(not the final one) 
      INTEGER spind                    !total number of space view line
      INTEGER iwtind                   !total number of IWT scan line
      INTEGER iwtk                     !index of IWT calib cycle
      INTEGER lnct(hrs_mxlin)          !array of line counters
      INTEGER filin                    !first line of first paquet of n  
                                       !lines with correct line counters 
      
      LOGICAL qualified0(hrs_mxcalib)  !=true if the calib cycle is usable
      
!--END OF DECLARATIONS---------------------------------------------------------

      IF (iprint.GE.1) WRITE(hrs_cllgu,*) 
      IF (iprint.GE.2) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '** SUB. H_SCANPOS_ALGOV4 **'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)


! Initializes some local variables
!---------------------------------
! Encoder position values for space,ict,iwt (NESS107)
      DATA spaceval,ictval,iwtval/68,105,156/
      
! Line count values for space,IWT and Earth: 
!  0,1,2 to 39   for the HIRS/3 and 4 (NESS107)
      DATA spaceln/0/
      DATA iwtln/1/
      DATA earthln0/2/
      DATA earthln1/39/
      
      mask=Z'00FF' 
      
      spind = 0
      iwtind= 0
      iwtk  = 0
      calib0= 0
      filin = 0
      
      DO icalib=1,hrs_mxcalib
       splintab0(icalib) = missint
       iwtlintab0(icalib)= missint
       k40(icalib)=0
       qualified0(icalib)=.FALSE.
      ENDDO 
      
      DO ilin=1,hrs_mxlin
       lnct(ilin)=missint
      ENDDO
      
               
! Checks bit 31 of the quality indicator bit field
!-------------------------------------------------
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) 
     &      'FOR INFORMATION and EACH SCAN LINE, CHECK BIT31'
     
      DO ilin=lin0,lin1 
        IF (gp_btn(hqualind(ilin),31)) THEN 
          IF (iprint.GE.1) THEN
            write(hrs_cllgu,fmt=500)ilin
            write(hrs_cllgu,fmt=510)
            write(hrs_cllgu,fmt=520)
          ENDIF
          text='BIT31=1 ==> No usable scan line number:'
          CALL ml_wti('notice',text,ilin)
        ENDIF
      ENDDO

! Checks the scan encoder position in each 56 elements of one scan line
!----------------------------------------------------------------------
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) 
     & 'ENCODER POSITION FOR EACH SCAN LINE'        
       
      
      DO ilin=lin0,lin1
 
! Reinitializes some local variables
        DO i=1,3
          poscount(i)=0
        ENDDO
              
        DO j=1,hrs_mxpix
          
          encod(j)=missint2
          
          dim=1
          nbit=8
          of=0          
                    
! Gets the encod element
          IF (helem(1,(j-1),ilin) .NE. missint) THEN
            CALL gp_x16(helem(1,(j-1),ilin),dim,nbit,of,encod(j))
          ENDIF

! Builts a counter for each position (space,IWT,Earth)
! to find the good position (maximum of the counter).
          IF (encod(j) .NE. missint2) THEN
            IF(encod(j).EQ.spaceval) poscount(1)=poscount(1)+1
            IF(encod(j).EQ.iwtval) poscount(2)  =poscount(2)+1
            IF(encod(j).GE.1 .AND. encod(j).LE.56) 
     &        poscount(3) = poscount(3) + 1
          ENDIF
          
        ENDDO   !DO j=1,hrs_mxpix
      
        maxposct=0
        imaxposct(ilin)=0
        
        IF (iprint.GE.2) WRITE(hrs_cllgu,fmt=500) ilin
        
        DO j=1,3
        
          IF (iprint.GE.2)  
     &        WRITE(hrs_cllgu,fmt=530) j,poscount(j) 
     
          IF (poscount(j) .GT. maxposct) THEN
            maxposct = poscount(j) 
            imaxposct(ilin) = j
          ENDIF
          
        ENDDO   !DO j=1,3
        
      ENDDO     !DO ilin=lin0,lin1
      
! Checks the line counter
!------------------------
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) 'LINE COUNTER' 
      DO ilin=lin0,lin1
        lnct(ilin)=helem(3,63,ilin)
      ENDDO 
      CALL h_linecount_algoV4(lnct,filin)
                
! Checks the encoder position with line count
!--------------------------------------------
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     &'CHECK ENCODER POSITION AND LINE COUNT'

      DO ilin=lin0,lin1
      
        IF (filin.GT.0) THEN
          IF ( (imaxposct(ilin).EQ.1 .AND. lnct(ilin).NE.spaceln)
     &      .OR.
     &      (imaxposct(ilin).EQ.2 .AND. lnct(ilin).NE.iwtln)
     &      .OR.
     &      ((imaxposct(ilin).EQ.3) .AND.
     &      (lnct(ilin).LT.earthln0.OR.lnct(ilin).GT.earthln1)) )
     &      THEN
     
            IF (iprint.GE.1) THEN
            
              WRITE(hrs_cllgu,fmt=500) ilin
              WRITE(hrs_cllgu,fmt=540)
              WRITE(hrs_cllgu,fmt=550)
              
              WRITE(hrs_cllgu,fmt=560)
              if(imaxposct(ilin).eq.1) WRITE(hrs_cllgu,fmt=580)    
              if(imaxposct(ilin).eq.2) WRITE(hrs_cllgu,fmt=590)     
              if(imaxposct(ilin).eq.3) WRITE(hrs_cllgu,fmt=600)

              WRITE(hrs_cllgu,fmt=570)
              IF (lnct(ilin).EQ.spaceln) WRITE(hrs_cllgu,fmt=580)
              IF (lnct(ilin).EQ.iwtln) WRITE(hrs_cllgu,fmt=590) 
              IF (lnct(ilin).GE.earthln0.AND.lnct(ilin).LE.earthln1) 
     &          WRITE(hrs_cllgu,fmt=600)
     
            ENDIF       !IF (iprint.GE.1) THEN
            
            WRITE(text,'("line:",I4," Encod. pos. not agree"
     &      ," with line count. Take the encod. pos.")')ilin
            CALL ml_wt('notice',text) 
                    
         ENDIF
         
        ELSE    !IF (filin.GT.0) THEN 
         
          IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=610)ilin
          WRITE(text,'("line:",I4,"No good line of first paquet " 
     &    ," of n lines. Take the encod. pos.")')ilin
          CALL ml_wt('notice',text)
            
        ENDIF   !IF (filin.GT.0) THEN 
          
! If the radiometer is looking to the space (or to the IWT), ranges the 
!----------------------------------------------------------------------
! number of the lines in arrays
!------------------------------

        IF (imaxposct(ilin).EQ.1) THEN
          spind = spind + 1 
          splintab0(spind)=ilin 
        ENDIF
        IF (imaxposct(ilin).EQ.2) THEN
          iwtind = iwtind + 1 
          iwtlintab0(iwtind)=ilin 
        ENDIF

! Reinitializes the value of hscntyp 
!------------------------------------ 
! (which has been initialized by the decommutation)

       IF (imaxposct(ilin).LE.2) hscntyp(ilin)=imaxposct(ilin)
       IF (imaxposct(ilin).EQ.3) hscntyp(ilin) = 0


      ENDDO     !DO ilin=lin0,lin1

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Print
!------
      IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=620)
      IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=630) spind,iwtind 
      IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=640) 
      IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=650) splintab0
      IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=660)
      IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=650) iwtlintab0

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Checks if the space view line number splintab0(k) belows at the same 
!--------------------------------------------------------------------- 
! calibration cycle that the IWT scan line number iwtlintab0(k)
!--------------------------------------------------------------
      kk=0
                        
      IF (spind.NE.0 .AND. iwtind.ne.0) THEN
       
       DO k=1,spind

        IF (splintab0(k) .EQ. (iwtlintab0(k)-iwtln)) THEN
! splintab0(k) and iwtlintab0(k) below to the same calibration cycle

          kk = kk + 1     
          splintab(kk) =splintab0(k)
          iwtlintab(kk)=iwtlintab0(k)

        ELSE
! splintab0(k) and iwtlintab0(k) don't below to the same calibration cycle
! research of the index of IWT scan line according to the index k of the 
! space view line.

          iwtk=1
          DO WHILE ( (iwtk .LT. iwtind) .AND.
     &    (splintab0(k) .NE. (iwtlintab0(iwtk)-iwtln)))
            iwtk=iwtk+1
          ENDDO
! iwtk is the first correct index of IWT scan line according to the index
! of the space view line

          IF ((iwtk.NE.iwtind).OR.((iwtk.EQ.iwtind) .AND.
     &    splintab0(k).EQ.(iwtlintab0(iwtk)-iwtln))) THEN
            kk = kk+ 1
            splintab(kk) =splintab0(k)
            iwtlintab(kk)=iwtlintab0(iwtk)
          ENDIF

         ENDIF  !IF (splintab0(k) .EQ. (iwtlintab0(k)-iwtln)) THEN

        ENDDO   !DO k=1,spind
        
       ENDIF    !IF (spind.NE.0 .AND. iwtind.NE.0)  THEN
       
! Number of correct calibration cycle after this step
       calib0=kk
       
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Print
!------       
       IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=670)
       IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=630) spind,iwtind 
       IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=640) 
       IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=650) splintab0
       IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=660)
       IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=650) iwtlintab0       
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

! Checks if calibration cycle k-1 and calibration cycle k are well
!----------------------------------------------------------------- 
! separated by 40 scan lines
!---------------------------

      IF (calib0.GT.0) THEN

! Note that for calib cycles 1 we have qualified0(1)=. TRUE.
      qualified0(1)=.TRUE.
      goodcalib=1
      DO icalib=2,calib0
        IF (iwtlintab(icalib)-40 .EQ. iwtlintab(icalib-1)) THEN
          qualified0(icalib)=.TRUE.
          goodcalib=goodcalib+1
        ELSE
! How many calibration cycles are missing between k and k-1 ?
          k40(icalib)=((iwtlintab(icalib)-iwtlintab(icalib-1))/40)-1 
          badcalib=badcalib+1             
          IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=680) icalib
          WRITE(text,'(" No 40 scan lines between calib cycle:",I2, 
     &       "  and the previous one")') icalib
          CALL ml_wt('warning',text)
        ENDIF
      ENDDO     !DO icalib=2,calib0
      
! Modifies  splintab and iwtlintab to introduce the k40 calib cycles
! which are missing.
      calib=calib0
      DO icalib=2,calib0 
        IF (k40(icalib).ne.0) THEN
          calib=calib0+k40(icalib)
          DO ki=1,k40(icalib)
            DO iicalib=calib0,icalib,-1
              splintab(iicalib+ki)=splintab(icalib)
              iwtlintab(iicalib+ki)=iwtlintab(icalib)
              qualified0(iicalib+ki)=qualified0(icalib)
            ENDDO  
! Next 2 lines changed to avoid problems with array indices 
C            splintab(icalib+ki-1)=-999
C            iwtlintab(icalib+ki-1)=-999 
            splintab(icalib+ki-1)=splintab(icalib+ki-2)+40
            iwtlintab(icalib+ki-1)=iwtlintab(icalib+ki-2)+40 
            qualified0(icalib+ki-1)=.FALSE.
          ENDDO
        ENDIF   !IF (k40(icalib).NE.0) THEN
      ENDDO     !DO icalib=2,calib0
      
      ENDIF     !IF (calib0.GT.0) THEN
               
       
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Print
!------          
       IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=690) calib
       IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=700) goodcalib
       IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=710) badcalib
       IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=640)
       IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=650) splintab
       IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=660)
       IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=650) iwtlintab
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
         
       text='Final number of full calib cycles:'
       CALL ml_wti('info',text,calib)
       text='Final number of full good calib cycles:'
       CALL ml_wti('info',text,goodcalib)
       text='Final number of bad calib cycles:'
       CALL ml_wti('info',text,badcalib)
       text='First space views line'
       CALL ml_wti('info',text,splintab(1))
       text='First IWT scan line'
       CALL ml_wti('info',text,iwtlintab(1))
       
! Initialises qualified(i,k) with qualified0(k)
       DO i=1,hrs_mxchn
         DO icalib=1,calib
            qualified(i,icalib)=qualified0(icalib)
         ENDDO
       ENDDO            
 
 500  FORMAT(3X,'scan line number in the orbit:',I3)    
 510  FORMAT(5X,'bit31=1 in the quality indicator bit field')
 520  FORMAT(5X,'-- do not use scan for product generation')
 530  FORMAT(5X,'pos:',I1,3X,'position counter:',I2)
 540  FORMAT(7X,'Encoder position not agree with line counter')
 550  FORMAT(7X,'We take the encoder position')
 560  FORMAT(7X,'With the encoder position parameter: ')
 570  FORMAT(7X,'With the line count parameter: ')
 580  FORMAT(24X,'radiometer is looking SPACE')
 590  FORMAT(24X,'radiometer is looking IWT')
 600  FORMAT(24X,'radiometer is looking EARTH')
 610  FORMAT(1X,'line:',I3,'No good line of first paquet ' 
     &    ,' of n lines. Take the encod. pos.')  
 620  FORMAT(/1X,'After the first step')
 630  FORMAT(/1X,'  nb of space and IWT lines:',2(2X,I2))
 640  FORMAT(5X,'numbers of the space view lines:')
 650  FORMAT(1X,32(2X,I4))
 660  FORMAT(5X,'numbers of the IWT scan lines  :')
 670  FORMAT(/1X,'After the second step')
 680  FORMAT(2X,'No 40 scan lines between calib cycle:',I2, 
     &        '  and the previous one')
 690  FORMAT(/1X,'Final nb of full calib cycles :',I2)
 700  FORMAT(1X,'Final nb of full good calib cycles :',I2)
 710  FORMAT(1X,'Final nb of bad calib cycles :',I2)


      
      RETURN
      END

