        SUBROUTINE h_testcoeffile_algoV4

!------------------------------------------------------------------------------
!NAME $Id: h_testcoeffile_algoV4.F 120 2008-06-27 09:32:19Z frna $
!
!PURPOSE 
!    This module opens/closes and reads the test coefficients file and  
!    ranges data in common hrs_tstcf (include hcinit_algoV4.h).
!
!DESCRIPTION 
!    This module is very similar to the module h_testcoeffile.
!    It was created to manage the HIRS calibration algorithm version4.0
!  
!HISTORY (CHANGES) 
!  (version    date          comment           <author>)
!     1.0     January 05   creation          T. Labrot (MF)
!     1.1     March 05   rename conflicting variables: versnb,versyr,versdy. NCA
!     1.2     25 April 07 Added the reading of the new parameter cntmax
!     1.3     30 May 2007 Change open for g95
!     1.4     July 07     Add a test on ios   T. Labrot (MF)
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!                       
!ARGUMENTS  
!     inputs :
!     outputs :
!
!UPDATED ELEMENTS OF INCLUDES 
!     In hcinit_algoV4.h : sp1fv,minvw,stdevcf,prtminmax,prtminnb,prtbfore,prtafter
!                          numprtlm,nbslval,BBslopethreshold,avgslopethreshold,
!                          cntmax
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------
       
      IMPLICIT NONE                                                      

! includes/commons(hrs_tstcoefu)
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hcinit_algoV4.h"

! argument variables 

! local variables
      CHARACTER*80 text           !variable text
      CHARACTER*80 charact        !buffer storing one line of the file
      INTEGER ios                 !variable that receives the error status
      INTEGER versnb_test         !number of version of the coef file
      INTEGER versyr_test         !year of the version of the coef file
      INTEGER versday_test        !day of the version of the coef file (0/365)

!--END OF DECLARATIONS---------------------------------------------------------

! Opens and reads test coefficients file
#ifdef FORTRAN_OPEN_RW
       OPEN(hrs_tstcoefu,access='sequential',status='old',iostat=ios,
     >   action = 'READ' )
#else
       OPEN(hrs_tstcoefu,access='sequential',status='old',iostat=ios)

!      If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
       IF (ios.NE.0)
     &   OPEN(hrs_tstcoefu,err=900,access='sequential',
     &      status='unknown',iostat=ios)
#endif

       if(ios.ne.0) then
         text='h_testcoeffile_algoV4: coef file could not be opened'
         call ml_wti('error',text,ios)
         call aappexit(2)
       endif

! Skips the header lines
       READ(hrs_tstcoefu,'(a80)',err=901) charact
       DO WHILE (charact(1:1) .EQ. '#')
        READ(hrs_tstcoefu,'(a80)',err=901) charact
       ENDDO
       BACKSPACE hrs_tstcoefu

       READ(hrs_tstcoefu,*,err=901)versnb_test
       READ(hrs_tstcoefu,*,err=901)versyr_test
       READ(hrs_tstcoefu,*,err=901)versday_test
 
       IF (iprint.GE.1) THEN
        WRITE(hrs_cllgu,*) 'test coefficients file' 
        WRITE(hrs_cllgu,500)versnb_test,versyr_test,versday_test
       ENDIF

      WRITE(text,'("Test file: Version nb, year, day of year :",
     &     I2,1X,I4,1X,I3)')versnb_test,versyr_test,versday_test
       CALL ml_wt('info',text)

       READ(hrs_tstcoefu,'(a80)',err=901) charact
       DO WHILE (charact(1:1) .EQ. '#')
         READ(hrs_tstcoefu,'(a80)',err=901) charact
       ENDDO
       BACKSPACE hrs_tstcoefu

       READ(hrs_tstcoefu,*,err=901)sp1fv
       READ(hrs_tstcoefu,*,err=901)minvw
       READ(hrs_tstcoefu,*,err=901)stdevcf
       READ(hrs_tstcoefu,*,err=901)prtminmax
       READ(hrs_tstcoefu,*,err=901)
       READ(hrs_tstcoefu,*,err=901)prtminnb
       READ(hrs_tstcoefu,*,err=901)prtbfore
       READ(hrs_tstcoefu,*,err=901)prtafter
       READ(hrs_tstcoefu,*,err=901)numprtlm
       READ(hrs_tstcoefu,*,err=901)nbslval
       READ(hrs_tstcoefu,*,err=901)BBslopethreshold
       READ(hrs_tstcoefu,*,err=901)
       READ(hrs_tstcoefu,*,err=901)avgslopethreshold
       READ(hrs_tstcoefu,*,err=901)
       READ(hrs_tstcoefu,*,err=901)cntmax


       CLOSE(hrs_tstcoefu,err=902)

       GOTO 200

 500  FORMAT(2X,' version number: ',I3/'   version year: ',I4/
     &       '   version day of year: ',I3)

 900  WRITE(hrs_cllgu,*)'Error opening test coefficients file'
      text='Test coefficients file could not be opened'
      CALL ml_wt('error',text)
      text='h_testcoeffile_algoV4 aborted'
      CALL ml_wt('error',text)
      CALL aappexit(2)

 901  WRITE(hrs_cllgu,*)'Error reading test coefficients file'
      text='Test coefficients file could not be read'
      CALL ml_wt('error',text)
      text='h_testcoeffile_algoV4 aborted'
      CALL ml_wt('error',text)
      CALL aappexit(2)

 902  WRITE(hrs_cllgu,*)'Error closing test coefficients file'
      text='Test coefficients file could not be closed' 
      CALL ml_wt('error',text)
      text='h_testcoeffile_algoV4 aborted'
      CALL ml_wt('error',text)
      CALL aappexit(2)

 200  RETURN
      END

