      SUBROUTINE h_uphrs1bcommon1_algoV4(
     & b1mn,
     & b1std, 
     & calibcoef)

!------------------------------------------------------------------------------
!NAME $Id: h_uphrs1bcommon1_algoV4.F 55 2006-09-21 08:35:34Z frna $
!
!PURPOSE
!    This module finishes to compute all the outputs of the HIRS 
!    calibration task (see include hrs1belem_algoV4.h). 
!    The writing in the HIRS level 1b will be done by another routine.
!
!DESCRIPTION 
!     
!HISTORY (CHANGES) 
!    (version    date         comment           <author>)
!      1.0     January 05     creation          T. Labrot (MF)
!      1.1     May     06     Change NOAA to noaa PM/PB
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!              
!         
!ARGUMENTS 
!   inputs :
!    DBLE PREC b1mn0(hrs_mxchn),b1std0(hrs_mxchn)
!                 mean and standard deviation of B1
!    DBLE PREC calibcoef(hrs_mxchn,3,hrs_mxlin) 
!                 third coefs,slopes,intercepts array for each scan line
!
!UPDATED ELEMENTS OF INCLUDES 
!    In include hcaliblineinfo_algoV4.h:
!      INTEGER linequality(10,hrs_mxlin) 
!                            !quality array for each scan line to update
!                            !quality flags in the HIRS level 1b
!                            !1: scan line no usable because the instrument
!                            !   status is OFF
!                            !2: scan line no usable because the bit 31 (do not
!                            !   use scan for product generation) is ON
!                            !3: insufficient data for calibration
!                            !4: scan was not calibrated
!                            !5: scan contains marginal calibration in some 
!                            !   of the IR channels
!                            !6: PRT quality test failed
!                            !7: PRT data marginal, some readings were rejected
!                            !8: scan contains some uncalibrated channels
!                            !9: scan indicates that the normal HIRS calibration 
!                            !   sequence is disabled 
!                            !10: space view scan is lunar contaminated
!       INTEGER linechquality(6,hrs_mxchn,hrs_mxlin)                       
!                            !1: calibration failed
!                            !2: calibration marginal
!                            !3: calibration slope came from the HCF
!                            !4: BB views failed NEDC test
!                            !5: space views failed NEDC test
!                            !6: some QC (quality control) tests could not be 
!                            !   applied 
!
!    All the output elements of the include hrs1belem_algoV4.h
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!    INTEGER FUNCTION   ord1bid :
!                  delivers the order satellite number for a given 
!                  level 1b id number (PACS number) and a julian date
!    CHARACTER*6 FUNCTION noascnam :  
!                  delivers the NOAA satellite name for a given order number
!    SUB.  gp_bse(word,bitnum) to set bitnum in word
!    sub.  gp_bcl(word,bitnum) to clear bitnum in word
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE                                                     
 
!includes/commons
#include "hmissingval_algoV4.h"
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hcinit_algoV4.h"
#include "hrs1belem_algoV4.h"
#include "hb1averageslope_algoV4.h"
#include "hcaliblineinfo_algoV4.h"

! argument variables
      DOUBLE PRECISION  b1mn(hrs_mxchn),b1std(hrs_mxchn)
      DOUBLE PRECISION  calibcoef(hrs_mxchn,3,hrs_mxlin)
! functions
      CHARACTER*6 noascnam 
      INTEGER     ord1bid     
! local variables
      CHARACTER*80 text !variable text
      INTEGER l1border(hrs_mxchn)     !channels in the l1b order
      data l1border/ 1,17,2,3,13,4,18,11,19,7,8,20,
     &          10,14,6,5,15,12,16,9 /
      INTEGER i,j,ich,ilin       !index of loops
      INTEGER hsatid0            !satellite id from a 1B file
      INTEGER satid              !order satellite number
      INTEGER word1,word2        !words where we put quality bits
      DOUBLE PRECISION avgslope0(hrs_mxchn),stdevavgslope0(hrs_mxchn)
      DOUBLE PRECISION avgslope00(hrs_mxchn),stdevavgslope00(hrs_mxchn)
!        The average slopes and its standard deviation in an other order
      DOUBLE PRECISION  b1mn0(hrs_mxchn),b1std0(hrs_mxchn)
!        mean and standard deviation of B1 in an other order

!--END OF DECLARATIONS----------------------------------------------------------

      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      IF (iprint.GE.2) WRITE(hrs_cllgu,*)'*****************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)'*SUB H_UPHRS1BCOMMON1_ALGOV4*'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)'*****************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)

!!!   OUTPUTS FOR THE HEADER OF THE LEVEL1B   !!!
!!!-------------------------------------------!!!

!INTEGER*2    hcpidsyr    !year of last CPIDS Update
!INTEGER*2    hcpidsdy    !day of year of last CPIDS Update
       
       hcpidsyr = versyr 
       hcpidsdy = versday
       
!CHARACTER*6    hinstrtype  !'HIRS/3' or 'HIRS/4'
 
       hsatid0 = hsatid     
       satid = ord1bid(hsatid0,hstartdatajd)
       
       IF (satid.EQ.0) THEN

        hinstrtype='     '
        
        IF (iprint.GE.1) 
     &  WRITE(hrs_cllgu,*) ' sat. id. not found or an error',
     &  ' occured during the sat. id. file reading'  
        
      text='Sat. id. not found or an error occured during the sat. id.' 
     & //  ' file reading'
       CALL ml_wt('error',text)       
       text='h_uphrs1bcommon1_algoV4 aborted'
       CALL ml_wt('error',text)
       CALL aappexit(2)

      ELSE

        IF (noascnam(satid).EQ.'noaa15' .OR.
     &      noascnam(satid).EQ.'noaa16' .OR.
     &      noascnam(satid).EQ.'noaa17') THEN
               hinstrtype='HIRS/3'
        ELSE
               hinstrtype='HIRS/4'
        ENDIF
        
      ENDIF 
      
!INTEGER*2    hallocsclin   !count of calibrated, earth located scan 
!                           !lines in this data set
      hcallocsclin=missint2
      
!INTEGER*2  hautocalind     !ramp/auto calibration indicators
!               bit 0 : auto calibration override switch for HIRS/3
 
      hautocalind=missint2      
       
!INTEGER*2    hsolarcalyr !year of most recent solar ch. calib.
!INTEGER*2    hsolarcaldy !day of year of most recent solar
!                         !channel calibration      
       
      hsolarcalyr = ch20year 
      hsolarcaldy = ch20day

!INTEGER      hcalinf(4,hrs_mxchn)    !calibration information
!                   order of the channels : 
!                   1, 17, 2, 3, 13, 4, 18, 11, 19, 7, 8,
!                   20, 10, 14, 6, 5, 15, 12, 16, 9.
!                      10^6*(mean calibration slope of channel nochannel)
!                      10^6*(stantard deviation of calibration slope)
!                      10^6*(b-sub-1 for channel nochannel)
!                      10^6*(stantard deviation of linear regression
!                          for b-sub-1 for channel nochannel)
      DO j=1,hrs_mxirchn 
        avgslope0(j)     = avgslope(j)
        stdevavgslope0(j)= stdevavgslope(j)
      ENDDO
      avgslope0(20)     = 0. 
      stdevavgslope0(20)= 0.
 
!     Changes the order of the channels
      DO j=1,hrs_mxchn
         avgslope00(j)=avgslope0(l1border(j))
         stdevavgslope00(j)=stdevavgslope0(l1border(j))
         b1mn0(j)=b1mn(l1border(j))
         b1std0(j)=b1std(l1border(j))
      ENDDO
       
      DO j=1,hrs_mxchn
         hcalinf(1,j) =IDINT(avgslope00(j)*1.d+06)
         hcalinf(2,j) =IDINT(stdevavgslope00(j)*1.d+06)
         hcalinf(3,j) =IDINT(b1mn0(j)*1.d+06)
         hcalinf(4,j) =IDINT(b1std0(j)*1.d+06)  
      ENDDO
       
                        
! INTEGER*4    htempradcnv(3,hrs_mxirchn)
!                  temperature-radiance conversion                       
!                  order of channels = 1,2,3,4,...,18,19
!                  for nochannel = 1 to 19 :
!                      10^6 x (nochannel central wavenumber for ch1 to ch12) 
!                      10^5 x (nochannel central wavenumber for ch13 to ch19)
!                      10^6 x (nochannel constant 1)
!                      10^6 x (nochannel constant 2)

      DO ich=1,hrs_mxirchn
       IF (ich.LE.12)
     &       htempradcnv(1,ich) =IDINT(cwave(ich)*1.d+06)
       IF (ich.GE.13)
     &       htempradcnv(1,ich) =IDINT(cwave(ich)*1.d+05)
       htempradcnv(2,ich) =IDINT(bdcorr(ich,1)*1.d+06)
       htempradcnv(3,ich) =IDINT(bdcorr(ich,2)*1.d+06)
      ENDDO
        
! INTEGER*2    h20solfiltirrad
!                      10^2 x (albedo-radiance ch20 solar filtered irradiance)       
! INTEGER*2    h20equifiltwidth  
!                      10^4 x (albedo-radiance ch 20 equivalent filter width)
       
       h20solfiltirrad = missint2
       h20equifiltwidth= missint2
       
! Analog telemetry conversion
! Scaling factors 10**(2, 2, 3, 3, 3, 5) 
!   INTEGER*4    hrdtemp(6)     !radiator temp.conversion coefficients(6)
!   INTEGER*4    hbptemp(6)     !base plate temp. conv. coef. (6)
!   INTEGER*4    heltemp(6)     !electronics temp. conv. coef. (6)
!   INTEGER*4    hpchtemp(6)    !patch temp. conv. coef. (6)
!   INTEGER*4    hfhcc(6)       !filter housing controller current conv.
!                                      coef. (6)
!   INTEGER*4    hscnmtemp(6)   !scan motor temperature conv. coef.(6)
!   INTEGER*4    hfwmtemp(6)    !filter wheel motor temp. conv. coef.(6) 
!   INTEGER*4    hp5v(6)        !+5 VDC monitor conv. coef.(6)
!   INTEGER*4    hp10v(6)       !+10 VDC TLM/DC/DC conv. conv. coef.(6) 
!   INTEGER*4    hp75v(6)       !+7.5 VDC TLM/DC/DC conv. conv. coef.(6)
!   INTEGER*4    hm75v(6)       !-7.5 VDC TLM/DC/DC conv. conv. coef.(6)
!   INTEGER*4    hp15v(6)       !+15 VDC monitor conv. coef.(6)
!   INTEGER*4    hm15v(6)       !-15 VDC monitor conv. coef.(6)
!   INTEGER*4    hfwmcur(6)     !filter wheel motor current conv.
!                                         coef.(5)
!   INTEGER*4    hscmcur(6)     !scan motor current conv. coef.(6)
!   INTEGER*4    hpchcpow(6)    !patch controller power conv. coef.(6)             
      DO j=1,6
        hrdtemp(j)=missint
        hbptemp(j)=missint
        heltemp(j)=missint
        hpchtemp(j)=missint
        hfhcc(j)=missint
        hscnmtemp(j)=missint
        hfwmtemp(j)=missint
        hp5v(j)=missint
        hp10v(j)=missint
        hp75v(j)=missint
        hm75v(j)=missint
        hp15v(j)=missint
        hm15v(j)=missint
        hfwmcur(j)=missint
        hscmcur(j)=missint
        hpchcpow(j)=missint
      ENDDO 
       
! Digital A Telemetry Conversion (6 coeffients for each)

!  INTEGER*4    hiwtcnttmp(5,6)  !Internal Warm Target, Temperature sensors
!                                !4 sensors for NOAAKLM
!                                !5 sensors for NOAANN'
!    scaling factors for coefficients:10**(6, 9, 14, 17, 21, 25)

      DO i=1,5        
        hiwtcnttmp(i,1)=IDINT((iwtprtcnttmp(i,1)+273.15)*1.d+06)
        hiwtcnttmp(i,2)=IDINT(iwtprtcnttmp(i,2)*1.d+09)  
        hiwtcnttmp(i,3)=IDINT(iwtprtcnttmp(i,3)*1.d+14)  
        hiwtcnttmp(i,4)=IDINT(iwtprtcnttmp(i,4)*1.d+17) 
        hiwtcnttmp(i,5)=IDINT(iwtprtcnttmp(i,5)*1.d+21)
        hiwtcnttmp(i,6)=IDINT(iwtprtcnttmp(i,6)*1.d+25)
      ENDDO
      
! INTEGER*4    hictcnttmp(4,6)  !Internal Cold Target, Temperature sensors 
!                               !4 sensors for NOAAKLM
!                               !1 sensor  for NOAANN'

      DO i=1,4
        hictcnttmp(i,1)=IDINT((ictprtcnttmp(i,1)+273.15)*1.d+06)
        hictcnttmp(i,2)=IDINT(ictprtcnttmp(i,2)*1.d+09)
        hictcnttmp(i,3)=IDINT(ictprtcnttmp(i,3)*1.d+14)
        hictcnttmp(i,4)=IDINT(ictprtcnttmp(i,4)*1.d+17)
        hictcnttmp(i,5)=IDINT(ictprtcnttmp(i,5)*1.d+21)
        hictcnttmp(i,6)=IDINT(ictprtcnttmp(i,6)*1.d+25)
      ENDDO  
      
! INTEGER*4    htttcnttmp(6)    !Tertiary Telescope, Temperature sensor
!                               !0 sensor for NOAAKLM
!                               !1 sensor  for NOAANN'         
      htttcnttmp(1)=IDINT((telescopcnttmp(3,1)+273.15)*1.d+06)
      htttcnttmp(2)=IDINT(telescopcnttmp(3,2)*1.d+09)
      htttcnttmp(3)=IDINT(telescopcnttmp(3,3)*1.d+14)
      htttcnttmp(4)=IDINT(telescopcnttmp(3,4)*1.d+17)
      htttcnttmp(5)=IDINT(telescopcnttmp(3,5)*1.d+21)
      htttcnttmp(6)=IDINT(telescopcnttmp(3,6)*1.d+25)
      
! INTEGER*4    hfwcnttmp(4,6)   !Filter Wheel Housing,Temperature sensors
!                               !4 sensors. 
      DO i=1,4
        hfwcnttmp(i,1)=IDINT((fwcnttmp(i,1)+273.15)*1.d+06)
        hfwcnttmp(i,2)=IDINT(fwcnttmp(i,2)*1.d+09)
        hfwcnttmp(i,3)=IDINT(fwcnttmp(i,3)*1.d+14)
        hfwcnttmp(i,4)=IDINT(fwcnttmp(i,4)*1.d+17)
        hfwcnttmp(i,5)=IDINT(fwcnttmp(i,5)*1.d+21)
        hfwcnttmp(i,6)=IDINT(fwcnttmp(i,6)*1.d+25)
      ENDDO 
      
! INTEGER*4    hpatchexpcnttmp(6) !Patch Temperature (expanded Scale)

      hpatchexpcnttmp(1)=IDINT((patchexpcnttmp(1)+273.15)*1.d+06)
      hpatchexpcnttmp(2)=IDINT(patchexpcnttmp(2)*1.d+09)
      hpatchexpcnttmp(3)=IDINT(patchexpcnttmp(3)*1.d+14)
      hpatchexpcnttmp(4)=IDINT(patchexpcnttmp(4)*1.d+17)
      hpatchexpcnttmp(5)=IDINT(patchexpcnttmp(5)*1.d+21)
      hpatchexpcnttmp(6)=IDINT(patchexpcnttmp(6)*1.d+25)
      
! INTEGER*4    hfsradcnttmp(6)  !First Stage Radiator Temperature sensor

      hfsradcnttmp(1)=IDINT((radcnttmp(1)+273.15)*1.d+06)
      hfsradcnttmp(2)=IDINT(radcnttmp(2)*1.d+09)
      hfsradcnttmp(3)=IDINT(radcnttmp(3)*1.d+14)
      hfsradcnttmp(4)=IDINT(radcnttmp(4)*1.d+17)
      hfsradcnttmp(5)=IDINT(radcnttmp(5)*1.d+21)
      hfsradcnttmp(6)=IDINT(radcnttmp(6)*1.d+25)
      
! INTEGER*4    hscmircnttmp(6)  !Scan Mirror Temperature

      hscmircnttmp(1)=IDINT((scmircnttmp(1)+273.15)*1.d+06)
      hscmircnttmp(2)=IDINT(scmircnttmp(2)*1.d+09)
      hscmircnttmp(3)=IDINT(scmircnttmp(3)*1.d+14)
      hscmircnttmp(4)=IDINT(scmircnttmp(4)*1.d+17)
      hscmircnttmp(5)=IDINT(scmircnttmp(5)*1.d+21)
      hscmircnttmp(6)=IDINT(scmircnttmp(6)*1.d+25)
      
! INTEGER*4    hpttcnttmp(6)    !Primary Telescope, Temperature sensor
 
      hpttcnttmp(1)=IDINT((telescopcnttmp(1,1)+273.15)*1.d+06)
      hpttcnttmp(2)=IDINT(telescopcnttmp(1,2)*1.d+09)
      hpttcnttmp(3)=IDINT(telescopcnttmp(1,3)*1.d+14)
      hpttcnttmp(4)=IDINT(telescopcnttmp(1,4)*1.d+17)
      hpttcnttmp(5)=IDINT(telescopcnttmp(1,5)*1.d+21)
      hpttcnttmp(6)=IDINT(telescopcnttmp(1,6)*1.d+25)
      
! INTEGER*4    hsttcnttmp(6)    !Secondary Telescope, Temperature sensor

      hsttcnttmp(1)=IDINT((telescopcnttmp(2,1)+273.15)*1.d+06)
      hsttcnttmp(2)=IDINT(telescopcnttmp(2,2)*1.d+09) 
      hsttcnttmp(3)=IDINT(telescopcnttmp(2,3)*1.d+14) 
      hsttcnttmp(4)=IDINT(telescopcnttmp(2,4)*1.d+17) 
      hsttcnttmp(5)=IDINT(telescopcnttmp(2,5)*1.d+21) 
      hsttcnttmp(6)=IDINT(telescopcnttmp(2,6)*1.d+25) 
        
! INTEGER*4    hbpcnttmp(6)     !Baseplate Temperature

      hbpcnttmp(1)=IDINT((bpcnttmp(1)+273.15)*1.d+06)
      hbpcnttmp(2)=IDINT(bpcnttmp(2)*1.d+09)
      hbpcnttmp(3)=IDINT(bpcnttmp(3)*1.d+14)
      hbpcnttmp(4)=IDINT(bpcnttmp(4)*1.d+17)
      hbpcnttmp(5)=IDINT(bpcnttmp(5)*1.d+21)
      hbpcnttmp(6)=IDINT(bpcnttmp(6)*1.d+25)

! INTEGER*4    helectcnttmp(6)  !Electronics Temperature

      helectcnttmp(1)=IDINT((electcnttmp(1)+273.15)*1.d+06)
      helectcnttmp(2)=IDINT(electcnttmp(2)*1.d+09)
      helectcnttmp(3)=IDINT(electcnttmp(3)*1.d+14)
      helectcnttmp(4)=IDINT(electcnttmp(4)*1.d+17)
      helectcnttmp(5)=IDINT(electcnttmp(5)*1.d+21)
      helectcnttmp(6)=IDINT(electcnttmp(6)*1.d+25)

! INTEGER*4    hpatchfcnttmp(6) !Patch Temperature Full Range

      hpatchfcnttmp(1)=IDINT((patchfcnttmp(1)+273.15)*1.d+06)
      hpatchfcnttmp(2)=IDINT(patchfcnttmp(2)*1.d+09)
      hpatchfcnttmp(3)=IDINT(patchfcnttmp(3)*1.d+14)
      hpatchfcnttmp(4)=IDINT(patchfcnttmp(4)*1.d+17)
      hpatchfcnttmp(5)=IDINT(patchfcnttmp(5)*1.d+21)
      hpatchfcnttmp(6)=IDINT(patchfcnttmp(6)*1.d+25)

! INTEGER*4    hscmotcnttmp(6)  !Scan Motor Temperature

      hscmotcnttmp(1)=IDINT((scmotcnttmp(1)+273.15)*1.d+06)
      hscmotcnttmp(2)=IDINT(scmotcnttmp(2)*1.d+09)
      hscmotcnttmp(3)=IDINT(scmotcnttmp(3)*1.d+14)
      hscmotcnttmp(4)=IDINT(scmotcnttmp(4)*1.d+17)
      hscmotcnttmp(5)=IDINT(scmotcnttmp(5)*1.d+21)
      hscmotcnttmp(6)=IDINT(scmotcnttmp(6)*1.d+25)

! INTEGER*4    hfwmcnttmp(6)    !Filter Wheel Motor Temperature

      hfwmcnttmp(1)=IDINT((fwmcnttmp(1)+273.15)*1.d+06)
      hfwmcnttmp(2)=IDINT(fwmcnttmp(2)*1.d+09)
      hfwmcnttmp(3)=IDINT(fwmcnttmp(3)*1.d+14)
      hfwmcnttmp(4)=IDINT(fwmcnttmp(4)*1.d+17)
      hfwmcnttmp(5)=IDINT(fwmcnttmp(5)*1.d+21)
      hfwmcnttmp(6)=IDINT(fwmcnttmp(6)*1.d+25)

! INTEGER*4    hchsgcnttmp(6)   !Cooler Housing Temperature

      hchsgcnttmp(1)=IDINT((chsgcnttmp(1)+273.15)*1.d+06) 
      hchsgcnttmp(2)=IDINT(chsgcnttmp(2)*1.d+09) 
      hchsgcnttmp(3)=IDINT(chsgcnttmp(3)*1.d+14)
      hchsgcnttmp(4)=IDINT(chsgcnttmp(4)*1.d+17)
      hchsgcnttmp(5)=IDINT(chsgcnttmp(5)*1.d+21)
      hchsgcnttmp(6)=IDINT(chsgcnttmp(6)*1.d+25) 
       
!  Digital A Telemetry Conversion
!  INTEGER*4    hconversions(11)  !Conversion constants 
       DO j=1,11
         hconversions(j) =missint
       ENDDO  
                   
! Lunar Contamination
! INTEGER*2    hmoonscnlin 
                         !Count of space view scans containing lunar contaminated
                         !=-1 if the detection algorithm for lunar contamination 
                         !    is turned off
                         !=0  if the detection algorithm for lunar contamination 
                         !    is turned on and no lunar-contaminated space view 
                         !    scans were found.
                         !=>0 if the detection algorithm for lunar contamination 
                         !    is turned on and the valu of this field represents
                         !    the number of lunar-contaminated space view scans.                         
! INTEGER*2    hmoonthresh
                         !Lunar angle thresold
                         !Any space view whose lunar angle is less than this 
                         !value is flagged as being "lunar contaminated" and 
                         !is not used in the calibration.
       hmoonscnlin = -1 
       hmoonthresh = missint2                         
                         
!24-hour average space view counts
! INTEGER*4    havspcounts(20)
                         !24-Hour Average Space View Counts
                         !Order of the channels (1,17,2,3,13,4,18,11,19,7,
                         !8,20,10,14,6,5,15,12,16,9

       DO j=1,20 
         havspcounts(j)=missint
       ENDDO                      


!!!   OUTPUTS FOR THE RECORDS OF THE LEVEL1B  !!!
!!!-------------------------------------------!!!

      DO ilin=lin0,lin1
      
! Updates the calibration part in the integer hqualind
!    if a bit is on(=1) then the statement is true.
!    bit 28: insufficient data for calibration

! And

! Updates the calibration part in the integer hlinqualflgs 
!            (these bits, along with those in "scan line quality flags",
!             complement the channel indicators; all bits set
!             to 0 indicates normal calibration)
!                  bit 15: scan was not calibrated
!                  bit 14: scan contains marginal calibration in some 
!                           of the IR channels
!                  bit 13: PRT quality test failed
!                  bit 12: PRT data marginal, some readings were rejected
!                  bit 11: scan contains some uncalibrated channels
!                  bit 10: scan indicates that the normal HIRS calibration
!                          sequence is disabled
!                  bit  9: space view scan is lunar contaminated
!                  bit  8: <zero fill>

! bit28 and bit15 =1 if for all the IR channels there is no intercept
! if only one IR channel is good, bit28 and bit15 =0
      word1 = hqualind(ilin)
      word2 = hlinqualflgs(ilin)
      linequality(3,ilin)=1
      linequality(4,ilin)=1
      CALL gp_bse(word1,28)
      CALL gp_bse(word2,15)
      DO ich=1,hrs_mxirchn
        IF (linechquality(1,ich,ilin).EQ.0) THEN
            linequality(3,ilin)=0
            linequality(4,ilin)=0
            CALL gp_bcl(word1,28)
            CALL gp_bcl(word2,15)
        ENDIF   
      ENDDO  
      hqualind(ilin) = word1
      hlinqualflgs(ilin) = word2

! bit14 and bit11 =1 if one IR channel is bad, but all the IR must be no bad
      CALL gp_bcl(word2,14)
      CALL gp_bcl(word2,11)
      DO ich=1,hrs_mxirchn
        IF (linechquality(1,ich,ilin).EQ.1 .AND. 
     &                linequality(4,ilin).EQ.0) THEN
           linequality(5,ilin)=1
           linequality(8,ilin)=1
           CALL gp_bse(word2,14)
           CALL gp_bse(word2,11)
         ENDIF
      ENDDO
      hlinqualflgs(ilin) = word2     

! bit13 and 12
      CALL gp_bcl(word2,13)
      CALL gp_bcl(word2,12)
      IF (linequality(6,ilin).EQ.1) CALL gp_bse(word2,13)
      IF (linequality(7,ilin).EQ.1) CALL gp_bse(word2,12)
      hlinqualflgs(ilin) = word2     
       
! bit10=0 if all the IR channels have an intercept       
      linequality(9,ilin)=0
      CALL gp_bcl(word2,10)
      DO ich=1,hrs_mxirchn
        IF (linechquality(1,ich,ilin).EQ.1) THEN
            linequality(9,ilin)=1
            CALL gp_bse(word2,10)
        ENDIF   
      ENDDO  
      hlinqualflgs(ilin) = word2

! bit9 =0 because no lunar contamination test is this version
      linequality(10,ilin)=0
      CALL gp_bcl(word2,9)
      hlinqualflgs(ilin) = word2

! bit8 =0 always
      CALL gp_bcl(word2,8)
      hlinqualflgs(ilin) = word2       
       
! Updates hchqualflg  
!                 Calibration quality flags for each channel
!                 order of the channels: 
!                 1, 17, 2, 3, 13, 4, 18, 11, 19, 7, 8,
!                 20, 10, 14, 6, 5, 15, 12, 16, 9.
!                 (all bits off implies a good calibration)
!                      bit 15-6: <zero fill>
!                      bit  5: calibration failed 
!                      bit  4: calibration marginal
!                      bit  3: calibration slope value came from the HCF
!                      bit  2: BB views failed NEDC test
!                      bit  1: space views failed NEDC test
!                      bit  0: some QC tests could not be applied

      DO ich=1,hrs_mxchn
       
         word1=hchqualflg(ich,ilin)
         CALL gp_bcl(word1,15)
         CALL gp_bcl(word1,14)
         CALL gp_bcl(word1,13)
         CALL gp_bcl(word1,12)
         CALL gp_bcl(word1,11)
         CALL gp_bcl(word1,10)
         CALL gp_bcl(word1,9)
         CALL gp_bcl(word1,8)
         CALL gp_bcl(word1,7)
         CALL gp_bcl(word1,6)
         IF (linechquality(1,ich,ilin).EQ.1) THEN
           CALL gp_bse(word1,5)
         ELSE
           CALL gp_bcl(word1,5)
         ENDIF
         IF (linechquality(2,ich,ilin).EQ.1) THEN
           CALL gp_bse(word1,4)
         ELSE
           CALL gp_bcl(word1,4)
         ENDIF 
         IF (linechquality(3,ich,ilin).EQ.1) THEN
           CALL gp_bse(word1,3)
         ELSE
           CALL gp_bcl(word1,3)
         ENDIF
         IF (linechquality(4,ich,ilin).EQ.1) THEN
           CALL gp_bse(word1,2)
         ELSE
           CALL gp_bcl(word1,2)
         ENDIF 
         IF (linechquality(5,ich,ilin).EQ.1) THEN
           CALL gp_bse(word1,1)
         ELSE
           CALL gp_bcl(word1,1)
         ENDIF 
         CALL gp_bcl(word1,0)
          
         hchqualflg(ich,ilin)=word1
         
      ENDDO     !DO ich=1,hrs_mxirchn   


! INTEGER hcalcof(3,hrs_mxchn,hrs_mxlin) : calibration coefficients
!   order of the channels:  1, 17, 2, 3, 13, 4, 18, 11, 19, 7, 8, 20, 10,
!                           14, 6, 5, 15, 12, 16, 9.
! only the first coefficient that is the coef of the second order term


       DO ich=1,hrs_mxchn
         hcalcof(1,ich,ilin)  = 0
         hscalcof(1,ich,ilin) = 0
       ENDDO
      
      ENDDO     !DO ilin=lin0,lin1

      RETURN
      END

