      Subroutine h_write_histo_algoV4(
     &     ordersatid)

!------------------------------------------------------------------------------
!NAME $Id: h_write_histo_algoV4.F 55 2006-09-21 08:35:34Z frna $
!
!PURPOSE
!   Store calibration information of all calibration
!   cycles in a historical ASCII file
!  
!   The routine writes on a new file which may be appended
!   to the historical file.
!
!DESCRIPTION 
!     
!HISTORY (CHANGES) 
!   (version    date         comment           <author>)
!     1.0     January 05    Creation          P. Brunel (MF)
!     1.1      13/06/05     Add spcntmed,spnedn,iwtcntmed,iwtnedn
!                                             T.Labrot and P.Brunel (MF)  
!     1.2      01/07/05     Check nedn values within format range. NCA
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
!ARGUMENTS
!  Inputs: 
!    Integer ordersatid  !order satellite number
! 
!UPDATED ELEMENTS OF INCLUDES
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!    Character*6 function  noascnam :  
!                  delivers the NOAA satellite name for a given order number
!    Integer*2 function gp_gb16(       
!                 returns the value of the bit n from an integer*2 
!
!------------------------------------------------------------------------------

!--DECLARATIONS---------------------------------------------------------------- 
      
      Implicit None
     
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hrs1belem_algoV4.h"
#include "hcalibcycleinfo_algoV4.h"

! argument variables      

      Integer ordersatid  !order satellite number
      
! functions      
      Character*6 noascnam
      Integer*2 gp_gb16 
        
! local variables
     
      Integer k          ! current cycle number
      Integer c          ! channel number
      Integer spl        ! space line number
      Integer ascdsc     ! ascending descending (0/1)
      Integer ios

      Character*80 text  ! text variable
      Character*6 scname ! satellite name
      Character*5 orbnum ! orbit number
      
      Logical qualif     ! true if at least a qualified channel

!--END OF DECLARATIONS----------------------------------------------------------

      scname = noascnam(ordersatid)
      orbnum = hprblkid(2:6)

      If (iprint.Ge.1) write(hrs_cllgu,*) 
      If (iprint.Ge.2) write(hrs_cllgu,*) '***************************'
      If (iprint.Ge.1) write(hrs_cllgu,*) '*SUB.H_WRITE_HISTO_ALGOV4 *'
      If (iprint.Ge.1) write(hrs_cllgu,*) '***************************'
      If (iprint.Ge.1) write(hrs_cllgu,*)
  
        Open(hrs_histou,form='formatted',access='sequential',
     &       status='new',iostat=ios)

!       If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
        If(ios.Ne.0)
     &  Open(hrs_histou,form='formatted',access='sequential',
     &       status='unknown',iostat=ios)

        If(ios.Ne.0) Then
         text='The historical file could not be opened, ios:'
         Call ml_wti('error',text,ios)
         If (iprint.Ge.1) Write(hrs_cllgu,*)
     &   'the historical file could not be opened,  ios:',ios
         Call aappexit(2)
        Else
         text='The historical file has been opened'
         Call ml_wt('info',text)
         If (iprint.Eq.1) Write(hrs_cllgu,*)
     &   'the historical file has been opened'
        Endif
        
      
      If( calib .Ge. 1 ) Then

         ! loop on calib cycles
         Call ml_wt('info','Writes  historical file for calib cycles')
         
         Do k = 1, calib

            ! tests if some qualified info
            qualif = .False.
            Do c = 1, hrs_mxchn
               If( qualified(c,k)) Then
                  qualif = .True.
               End If
            End Do
            If( qualif) Then

               spl = splintab(k)

                ! tests bit 15 of hscnlinf 
               ascdsc = gp_gb16(hscnlinf(spl),15)
               
               Write(hrs_histou,'(a15)') 'HIRS CALIB INFO'
               Write(hrs_histou,*) '!'
               Write(hrs_histou,*) '! satellite, date, orbit number' 
               Write(hrs_histou,*) scname
               Write(hrs_histou,'(1x,2i4,i10)') 
     &         hscnlinyr(spl), hscnlindy(spl), hscnlintime(spl)
               Write(hrs_histou,*) orbnum
               Write(hrs_histou,*) '! ascending descending (0/1)'
               Write(hrs_histou,*) ascdsc
               Write(hrs_histou,*) '! solar zenith angle'
               Write(hrs_histou,*) 
     &              (hang(82,spl) + hang(85,spl))/2/100.
               Write(hrs_histou,*) '! latitude at nadir'
               Write(hrs_histou,*) 
     &              (hpos(55,spl) + hpos(57,spl))/2./10000.


               Write(hrs_histou,*) '!'
               Write(hrs_histou,*) '! calib cycle number, total cycles'
               Write(hrs_histou,*) k, calib
               Write(hrs_histou,*) '! total path scan lines '
               Write(hrs_histou,*) hscnlin
               Write(hrs_histou,*) '! qualified channels'
               Write(hrs_histou,'(20l2)') 
     &                    (qualified(c,k),c=1,hrs_mxchn)
               Write(hrs_histou,*) '! space line number '
               Write(hrs_histou,*) splintab(k)
               Write(hrs_histou,*) '! warm target line number'
               Write(hrs_histou,*) iwtlintab(k)
               Write(hrs_histou,*) '! moon contamination flag'
               Write(hrs_histou,'(l2)') mooncontaflag(k)
               Write(hrs_histou,*) '! telescope temps (prim, sec, ter)'
               Write(hrs_histou,*) ptt(spl), stt(spl), ttt(spl)
               Write(hrs_histou,*) '! filter wheel temp'
               Write(hrs_histou,*) (fwt(c,spl),c=1,4)
               Write(hrs_histou,*) '! base plate temp'
               Write(hrs_histou,*) bbt(spl)

               Write(hrs_histou,*) '!'
               Write(hrs_histou,*) '! calibration intercept '
               Write(hrs_histou,'(10e15.6)') 
     &                         (calibcoef0(c,3,k),c=1,hrs_mxchn)
               Write(hrs_histou,*) '! calibration slope '
               Write(hrs_histou,'(10e15.6)') 
     &                         (calibcoef0(c,2,k),c=1,hrs_mxchn)
               Write(hrs_histou,*) '! calibration 2snd order '
               Write(hrs_histou,'(10e15.6)') 
     &                         (calibcoef0(c,1,k),c=1,hrs_mxchn)

               Write(hrs_histou,*) '!'
               Write(hrs_histou,*) '! space target '
               Write(hrs_histou,*) '! mean, med, stdev, nedn,',
     &          ' good, missing, lower than limit, higher, rejected'
               Write(hrs_histou,'(19f7.1)')
     &                          (spcntmn(c,k),c=1,hrs_mxirchn)
               Write(hrs_histou,'(19f7.1)')
     &                          (spcntmed(c,k),c=1,hrs_mxirchn)
               Write(hrs_histou,'(19f7.2)')
     &                          (spcntstd(c,k),c=1,hrs_mxirchn)
               Do c=1, 12
                 If( spnedn(c,k) .gt. 999 ) spnedn(c,k) = 0.
               End Do
               Do c=12, 19
                 If( spnedn(c,k) .gt. 0.99999 ) spnedn(c,k) = 0.
               End Do
               Write(hrs_histou,'(12f7.3,7(1x,f6.5))')
     &                          (spnedn(c,k),c=1,hrs_mxirchn)
               Write(hrs_histou,'(19i7)')  
     &                          (spgoodcnt(c,k),c=1,hrs_mxirchn)
               Write(hrs_histou,'(19i7)') 
     &                          (spmisscnt(c,k),c=1,hrs_mxirchn)
               Write(hrs_histou,'(19i7)')
     &                          (spinfcnt(c,k),c=1,hrs_mxirchn)
               Write(hrs_histou,'(19i7)')
     &                          (spsupcnt(c,k),c=1,hrs_mxirchn)
               Write(hrs_histou,'(19i7)') 
     &                          (spejeccnt(c,k),c=1,hrs_mxirchn)

               Write(hrs_histou,*) '!'
               Write(hrs_histou,*) '! warm target '
               Write(hrs_histou,*) '! mean, med, stdev, nedn,',
     &          ' good, missing, lower than limit, higher, rejected'
               Write(hrs_histou,'(19f7.1)')
     &                          (iwtcntmn(c,k),c=1,hrs_mxirchn)
               Write(hrs_histou,'(19f7.1)')
     &                          (iwtcntmed(c,k),c=1,hrs_mxirchn)
               Write(hrs_histou,'(19f7.2)')
     &                          (iwtcntstd(c,k),c=1,hrs_mxirchn)
               Do c=1, 12
                 If( iwtnedn(c,k) .gt. 999 ) iwtnedn(c,k) = 0.
               End Do
               Do c=12, 19
                 If( iwtnedn(c,k) .gt. 0.99999 ) iwtnedn(c,k) = 0.
               End Do
               Write(hrs_histou,'(12f7.3,7(1x,f6.5))')
     &                          (iwtnedn(c,k),c=1,hrs_mxirchn)
               Write(hrs_histou,'(19i7)')
     &                          (iwtgoodcnt(c,k),c=1,hrs_mxirchn)
               Write(hrs_histou,'(19i7)') 
     &                          (iwtmisscnt(c,k),c=1,hrs_mxirchn)
               Write(hrs_histou,'(19i7)') 
     &                          (iwtinfcnt(c,k),c=1,hrs_mxirchn)
               Write(hrs_histou,'(19i7)')
     &                          (iwtsupcnt(c,k),c=1,hrs_mxirchn)
               Write(hrs_histou,'(19i7)') 
     &                          (iwtejeccnt(c,k),c=1,hrs_mxirchn)

               Write(hrs_histou,*) '!'
               Write(hrs_histou,*) '! PRTs '
               Write(hrs_histou,*) '! temp, mean, stdev, initial, ',
     &                             'good, rejected'
               Write(hrs_histou,'(5f9.3)') (tmp(c,k),c=1,hrs_mxprtnb)
               Write(hrs_histou,'(5f9.2)') 
     &                          (prtmean(c,k),c=1,hrs_mxprtnb)
               Write(hrs_histou,'(5f9.2)')
     &                          (prtstd(c,k),c=1,hrs_mxprtnb)
               Write(hrs_histou,'(5i9)')
     &                          (datanb(c,k),c=1,hrs_mxprtnb)
               Write(hrs_histou,'(5i9)')
     &                          (goodprt(c,k),c=1,hrs_mxprtnb)
               Write(hrs_histou,'(5i9)')
     &                          (ejecprt(c,k),c=1,hrs_mxprtnb)

               Write(hrs_histou,*)  '!'
            End If ! qualified
         End Do ! calib cycle
      Else
         Call ml_wt('info','nothing written to calib historical file')
      End If

!--------------------------------------
! closes the historic file
!--------------------------------------
      Close(hrs_histou,iostat=ios)
      If( ios .Eq. 0 ) Then
         If(iprint.Ge.1) Then
            Write(hrs_cllgu,*) 'the historic file has been closed'
         End If
         text='The historical file has been closed'
         Call ml_wt('info',text)
      Else
         If(iprint.Ge.1) Then
            Write(hrs_cllgu,*) 
     &        'Error closing the historic file ios ',ios
         End If
         text='Error closing the historic file, ios'
         Call ml_wti('error',text,ios)
      End If

      Return
      End
