        SUBROUTINE hclin_algoV4

!------------------------------------------------------------------------------
!NAME $Id: hclin_algoV4.F 163 2009-09-21 12:20:33Z frna $
!
!PURPOSE 
!    This module initializes the HIRS navigation and calibration tasks. 
!    It reads the input options (hrs_cloptn)
!
!DESCRIPTION
!    This module is very similar to the module hclin.
!    It was created to manage the HIRS calibration algorithm version4.0
!
!HISTORY (CHANGES) 
!    (version   date         comment           <author>)
!      1.0     January 05     creation          T. Labrot (MF)
!      1.1     July 2009   Always initialise hrs_cllgu. N Atkinson
!  
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!              
!ARGUMENTS 
!
!UPDATED ELEMENTS OF INCLUDES
!     All the elements of hclin_algoV4.h
!
!FUNCTIONS AND SUBROUTINES CALLS (RETURN CODES FOR FUNCTIONS)
! 
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE 
                                                     
! includes/commons
#include "hclin_algoV4.h"      
#include "c_1bnav.h"

! argument variables 

! local variables
       INTEGER hrs_cloptn(12) 
!      hrs_cloptn(1) -- switch: 1 for HIRS navigation/localisation (0 if not for)
!      hrs_cloptn(2) -- switch: 1 for HIRS calibration (0 if not for)
!      hrs_cloptn(3) -- option to take into account the telescope temperature to compute the intercept
! =0 the telescope temperature is not taken into account
! =1 the telescope temperature is taken into account with b1 values for all conditions
! =2 the telescope temperature is taken into account with b1 values for night/day light conditions
!      hrs_cloptn(4) -- logical unit for HIRS calibration coefficients file
!      hrs_cloptn(5) -- logical unit for HIRS calib test coefficients file
!      hrs_cloptn(6) -- logical unit for HIRS calib B1 slope file
!      hrs_cloptn(7) -- logical unit for HIRS log/debug file
!      hrs_cloptn(8) -- logical unit for calib historic file
!      hrs_cloptn(9) -- logical unit HIRS level1b file
!      hrs_cloptn(10)-- logical unit for SATPOS file
!      hrs_cloptn(11)-- logical unit for the clock drift data file
!      hrs_cloptn(12)-- print option in the log.file
!                       = 0, nothing is printed
!                       = 1, entering routines and few values of the main
!                            parameters are printed
!                       = 2, computed values that would help in debugging 
!                            are printed

      INTEGER i         !index of loop
      INTEGER ios       !input/output status
      CHARACTER*80 text !variable text

!--END OF DECLARATIONS---------------------------------------------------------

      READ(*,*) hrs_cloptn

! Checks hrs_cloptn values
      DO i = 1,2
        IF (hrs_cloptn(i).GT.1 .OR. hrs_cloptn(i).LT.0) THEN
          text='**Invalid switch option number--correct input**'
          CALL ml_wti('error',text,hrs_cloptn(i))
          text='hclin_algoV4 aborted'
          CALL ml_wt('error',text)
          CALL aappexit(1)
        ENDIF
      ENDDO
      
      IF((hrs_cloptn(3).NE.0).AND.(hrs_cloptn(3).NE.1).AND.
     & (hrs_cloptn(3).NE.2)) THEN
         text='Invalid telescope temp option value--correct input'
         CALL ml_wt('error',text)
         text='hclin_algoV4 aborted'
         CALL ml_wt('error',text)
         CALL aappexit(1)
      ENDIF
      

      IF((hrs_cloptn(12).NE.0).AND.(hrs_cloptn(12).NE.1).AND.
     & (hrs_cloptn(12).NE.2)) THEN
         text='Invalid print option value--correct input'
         CALL ml_wt('error',text)
         text='hclin_algoV4 aborted'
         CALL ml_wt('error',text)
         CALL aappexit(1)
      ENDIF


      hrs_lflg    = hrs_cloptn(1)
      hrs_cflg    = hrs_cloptn(2)

      hrs_l1bu    = hrs_cloptn(9)
      iprint      = hrs_cloptn(12)
     
! Navigation/localisation
      IF (hrs_lflg.EQ.1) THEN
        satposu     = hrs_cloptn(10)
        ckdriftu    = hrs_cloptn(11)

        READ(*,*,iostat=ios) yaw, roll, pitch
        IF (ios.NE.0) THEN
          CALL ml_wti('error',
     &       'while reading attitude values I/O status',ios)
          CALL ml_wt('error','hclin_algoV4 aborted')
          CALL aappexit(1)
        ENDIF
!   Converts the values in radians
        yaw = yaw*1.d-3
        roll = roll*1.d-3
        pitch = pitch*1.d-3
      ENDIF

! Calibration
      IF (hrs_cflg.EQ.1) THEN
        hrs_ttflg   = hrs_cloptn(3)
        hrs_coefu   = hrs_cloptn(4)
        hrs_tstcoefu= hrs_cloptn(5)
        hrs_b1aslopu= hrs_cloptn(6)
        hrs_histou  = hrs_cloptn(8)
      ENDIF
      hrs_cllgu   = hrs_cloptn(7)

      RETURN
      END








  
