        SUBROUTINE hl1bwrt_algoV4

!------------------------------------------------------------------------------
!NAME $Id: hl1bwrt_algoV4.F 55 2006-09-21 08:35:34Z frna $
!
!PURPOSE 
!    This module writes in the HIRS level1b file, header and records.
!
!    The format of the output HIRS level1b file corresponds to the format
!    version 2. 
!    Note that this format will become operational at NOAA in March 2005.
!
!DESCRIPTION    
!  
!HISTORY (CHANGES) 
! (version     date         comment           <author>)
! 1.0     January 05     creation          T. Labrot (MF)
! 1.1     Mar  05    Allow for NESDIS data: add 4096 to 1b counts. NCA
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!                      
!ARGUMENTS 
!
!ELEMENTS OF INCLUDES UPDATED
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE                                                     

! includes/commons
#include "hmissingval_algoV4.h"
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hrs1belem_algoV4.h"
#include "hrs1bhd.h"
#include "hrs1bdt.h"
#include "hrs1bdts.h"

! local variables
      INTEGER i,ii,j,jj  !index of loops
      INTEGER ios        !variable that receives the error status
      INTEGER recn       !record number in the HIRS 1B file
      INTEGER recn0      !first record number in the HIRS 1B file
      INTEGER recn1      !last record number in the HIRS 1B file
      CHARACTER*80 text  !variable text
      
!--END OF DECLARATIONS---------------------------------------------------------

      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      IF (iprint.GE.2) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '**  SUB. HL1BWRT_ALGOV4  **'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)

! Header elements updated during the calibration task
      hrs_h_cpidsyr = hcpidsyr
      hrs_h_cpidsdy = hcpidsdy
      hrs_h_instrtype = hinstrtype 
      hrs_h_callocsclin = hcallocsclin
      hrs_h_autocalind = hautocalind
      hrs_h_solarcalyr = hsolarcalyr
      hrs_h_solarcaldy = hsolarcaldy
      DO j=1,4
        DO i=1,hrs_mxchn
          hrs_h_calinf(j,i) = hcalinf(j,i)
        ENDDO
      ENDDO
      DO j=1,3
        DO i=1,hrs_mxirchn    
          hrs_h_tempradcnv(j,i) = htempradcnv(j,i)
        ENDDO
      ENDDO    
      hrs_h_20solfiltirrad = h20solfiltirrad
      hrs_h_20equifiltwidth= h20equifiltwidth
      DO i=1,6
        hrs_h_rdtemp(i)  = hrdtemp(i)
        hrs_h_bptemp(i)  = hbptemp(i)
        hrs_h_eltemp(i)  = heltemp(i)
        hrs_h_pchtemp(i) = hpchtemp(i)
        hrs_h_fhcc(i)    = hfhcc(i)
        hrs_h_scnmtemp(i)= hscnmtemp(i)
        hrs_h_fwmtemp(i) = hfwmtemp(i)
        hrs_h_p5v(i)  = hp5v(i)
        hrs_h_p10v(i) = hp10v(i)
        hrs_h_p75v(i) = hp75v(i)
        hrs_h_m75v(i) = hm75v(i)
        hrs_h_p15v(i) = hp15v(i)
        hrs_h_m15v(i) = hm15v(i)
        hrs_h_fwmcur(i) = hfwmcur(i)
        hrs_h_scmcur(i) = hscmcur(i)
        hrs_h_pchcpow(i)= hpchcpow(i)
        DO j=1,5
          hrs_h_iwtcnttmp(i,j) = hiwtcnttmp(j,i)     !Transpose i,j for 1b
        ENDDO  
        hrs_h_tttcnttmp(i) = htttcnttmp(i)
        DO j=1,4
          hrs_h_ictcnttmp(i,j) = hictcnttmp(j,i) 
          hrs_h_fwcnttmp(i,j)  = hfwcnttmp(j,i)
        ENDDO  
        hrs_h_patchexpcnttmp(i)= hpatchexpcnttmp(i)
        hrs_h_fsradcnttmp(i) = hfsradcnttmp(i)
        hrs_h_scmircnttmp(i) = hscmircnttmp(i)
        hrs_h_pttcnttmp(i)   = hpttcnttmp(i)
        hrs_h_sttcnttmp(i)   = hsttcnttmp(i)
        hrs_h_bpcnttmp(i)    = hbpcnttmp(i)
        hrs_h_electcnttmp(i) = helectcnttmp(i)
        hrs_h_patchfcnttmp(i)= hpatchfcnttmp(i)
        hrs_h_scmotcnttmp(i) = hscmotcnttmp(i)
        hrs_h_fwmcnttmp(i)   = hfwmcnttmp(i)
        hrs_h_chsgcnttmp(i)  = hchsgcnttmp(i)
      
      ENDDO
      DO i=1,11 
        hrs_h_conversions(i) = hconversions(i)
      ENDDO  
      hrs_h_moonscnlin = hmoonscnlin
      hrs_h_moonthresh = hmoonthresh
      DO i=1,hrs_mxchn
        hrs_h_avspcounts(i) = havspcounts(i)
      ENDDO
        
! Writes in the header of the level 1b    
      WRITE(hrs_l1bu,rec=1,iostat=ios) hrs_hd 

      IF (ios.NE.0) THEN
        IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     &   '  HIRS1B header could not be written,ios:',ios
        text='HIRS1B header could not be written,  ios:'
        CALL ml_wti('error',text,ios)
        text='hl1bwrt_algoV4 aborted'
        CALL ml_wt('error',text)
        CALL aappexit(2)
      ENDIF 
        
! Elements for each data record  
      recn0=hhdrcnt+1 
      recn1=hhdrcnt+hscnlin 
        
      DO recn=recn0,recn1

! Elements updated during the calibration task              
        hrs_scntyp0  = hscntyp(recn-hhdrcnt)
        hrs_qualind0 = hqualind(recn-hhdrcnt)
        hrs_linqualflgs0 = hlinqualflgs(recn-hhdrcnt)
        DO i=1,hrs_mxchn
          hrs_chqualflg0(i) = hchqualflg(i,recn-hhdrcnt)
          DO j=1,3
            hrs_calcof0(j,i) = hcalcof(j,i,recn-hhdrcnt)
            hrs_scalcof0(j,i)= hscalcof(j,i,recn-hhdrcnt)
          ENDDO
        ENDDO
        
! Elements with no change during the calibration task
        hrs_scnlin0     = hrs_scnlin(recn - hhdrcnt)
        hrs_scnlinyr0   = hrs_scnlinyr(recn - hhdrcnt)
        hrs_scnlindy0   = hrs_scnlindy(recn - hhdrcnt)
        hrs_clockdrift0 = hrs_clockdrift(recn - hhdrcnt)
        hrs_scnlintime0 = hrs_scnlintime(recn - hhdrcnt)
        hrs_scnlinf0    = hrs_scnlinf(recn - hhdrcnt)
        hrs_mjfrcnt0    = hrs_mjfrcnt(recn - hhdrcnt)
        hrs_scnpos0     = hrs_scnpos(recn - hhdrcnt)
        hrs_navstat0    = hrs_navstat(recn - hhdrcnt)
        hrs_attangtime0 = hrs_attangtime(recn - hhdrcnt)
        hrs_rollang0    = hrs_rollang(recn - hhdrcnt)
        hrs_pitchang0   = hrs_pitchang(recn - hhdrcnt)
        hrs_yawang0     = hrs_yawang(recn - hhdrcnt)
        hrs_scalti0     = hrs_scalti(recn - hhdrcnt)
        hrs_moonang0    = hrs_moonang(recn - hhdrcnt)
        hrs_digitbupdatefg0 = hrs_digitbupdatefg(recn - hhdrcnt)
        hrs_digitbwrd0  = hrs_digitbwrd(recn - hhdrcnt)
        hrs_analogupdatefg0 = hrs_analogupdatefg(recn - hhdrcnt) 

        DO i=1,2
          hrs_filler10(i) = hrs_filler1(i,recn - hhdrcnt)
        ENDDO
        DO i=1,4
          hrs_filler20(i) = hrs_filler2(i,recn - hhdrcnt)
        ENDDO
        DO i=1,3
          hrs_filler30(i) = hrs_filler3(i,recn - hhdrcnt)
        ENDDO
        DO i=1,6
          hrs_filler40(i) = hrs_filler4(i,recn - hhdrcnt)
        ENDDO  
        DO i=1,11
          hrs_filler50(i) = hrs_filler5(i,recn - hhdrcnt)
        ENDDO
        
        DO i=1,3
          hrs_yawsteering0(i) = hrs_yawsteering(i,recn - hhdrcnt)
          hrs_totattcorr0(i)  = hrs_totattcorr(i,recn - hhdrcnt)
        ENDDO  

        DO i=1,168
          hrs_ang0(i) = hrs_ang(i,recn - hhdrcnt)
        ENDDO

        DO i=1,112
          hrs_pos0(i) = hrs_pos(i,recn - hhdrcnt)  
        ENDDO
 
        DO ii=1,24
         DO jj=0,63 
           hrs_elem0(ii,jj) = hrs_elem(ii,jj,recn - hhdrcnt)
         ENDDO
        ENDDO

!     If this is NESDIS, add 4096 to radiometric counts
        IF (hrs_h_siteid .EQ. "NSS") THEN
          do ii=3,22
            do jj=0,57 
              hrs_elem0(ii,jj) = hrs_elem0(ii,jj) + 4096
            enddo
          enddo
        ENDIF

        DO jj=0,63
          hrs_mnfrqual0(jj) =  hrs_mnfrqual(jj,recn - hhdrcnt)  
        ENDDO
        
        DO i=1,16
          hrs_anwrd0(i) =  hrs_anwrd(i,recn - hhdrcnt)  
        ENDDO   
       
! Writes in the data records of the level 1b       
        WRITE(hrs_l1bu,rec=recn,iostat=ios) hrs_dt
        
        IF (ios.NE.0) THEN
         IF (iprint.GE.1) THEN
         write(hrs_cllgu,*)'  HIRS1B record could not be written'
         write(hrs_cllgu,*)'        rec nb: ',recn,'   ios: ',ios
         ENDIF
         text='HIRS1B record could not be written,  ios:'
         CALL ml_wti('error',text,ios)
         text='hl1bwrt_algoV4 aborted'
         CALL ml_wt('error',text)
         CALL aappexit(2)
        ENDIF
         
      ENDDO     !DO recn=recn0,recn1
        
      RETURN
      END

