      subroutine mhs_antpos(nvstat)
!+
!NOM/NAME $Id: mhs_antpos.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!    This module checks the antenna pointing of the AMSU-B Earth views.
!    if it is outside the threshold the navigation flag nvstat is set to
!    false and also the bit in the 1b scan line flag.     
!
!DESCRIPTION 
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
!     Roger SAUNDERS
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      February 96    no             R.Saunders
!         
!ARGUMENTS 
!      nvtat(mxlin) L*1 in/out  If .TRUE. then pointing ok 
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################

!--DECLARATIONS----------------------------------------------------------------
      implicit none                                                     
c-----------------------------------------
c
!      
! includes/commons
#include "mhsclin.h"
#include "mhs1b.h"
#include "mhsscn.h"
#include "mhs_cinit.h"
!
! Arguments
      logical*1 nvstat(mxlin)

! local variables
      integer*4 iscan,i,iv,flagword,errword,errval(8)
      data errval/1,2,4,8,16,32,64,128/  !Values for position error flag
      logical first
      real OP,NOM_POS
!
!--END OF DECLARATIONS---------------------------------------------------------

      if (iprint.ge.1) write(mhs_cllgu,*)
      if (iprint.ge.1) write(mhs_cllgu,*) '***************************'
      if (iprint.ge.1) write(mhs_cllgu,*) '**   SUBROUTINE MHS_ANTPOS*'
      if (iprint.ge.1) write(mhs_cllgu,*) '***************************'
!..Loop in scan line................................................
      do 100 iscan = 1 , mhs_h_scnlin
      nvstat(iscan) = .true.
      first = .true.
      do i=1,8
        earthval(i,iscan) = char(0)
      enddo

!..Loop in Earth View...............................................
      do 10 i = 1 , mxpix
      flagword = (i-1)/8 + 1         !8 FOVS per word
      errword = mod((i-1),8) + 1
      iv = (i-1)*6 + 1
      OP = -90 + earth_data(iv,iscan)*ant_coef   ! Posn decode
      NOM_POS = 40.5555 + FLOAT(I-1)*1.1111111    

      IF(ABS(NOM_POS-OP).GT.pos_erre)then
        nvstat(iscan) = .false.
        earthval(flagword,iscan) = CHAR(ICHAR(earthval(flagword,iscan)) 
     &    + errval(errword))
        if (iprint.ge.1.and.first) then
          WRITE (mhs_cllgu,200) iscan,NOM_POS-OP
          first = .false.        !One warning per scan
        endif
 200    FORMAT(1X,' scan ',I4,
     &    ' Nominal - measured earth antenna posn=',F9.3,' deg')
      ENDIF

 10   continue   ! .....end of pixel loop ..........
!
 100  Continue   ! ......end of scan line loop
!
      return
      end
