      subroutine mhs_avg(iscan,ichan,sampok,itmean,spmean,itavg,spavg
     &,calok)
!+
!NOM/NAME $Id: mhs_avg.F 315 2012-02-29 13:32:26Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!     To average over consecutive scan lines to give mean internal target
!     and space view counts for the MHS channels. A triangular weighting 
!     function is applied as recommended in the Mo & Saunders specification. 
!     If there are bad scan lines within the averaging window then increased
!     weight is given to the good scan lines. If there are bad scan lines or 
!     no good scan lines then the 1b quality flags are set.
!
!DESCRIPTION
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions
!
!PROPRIETAIRE/OWNER
!     Nigel Atkinson
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES)
!(version   date      comment                            <author>)
! 1.0      July 2003  From amb_avg.F                    NCAtkinson
! 1.1      Feb 2011   Comment out diagnostic line due to crash
!
!ARGUMENTS
!      iscan i*4 in scan line number
!      ichan i*4 in Channel number (1-5)
!      sampok(5,mxlin) L*1 in cal view counts ok flag
!      itmean(5,mxlin) r*4 in mean target counts for each channel
!      spmean(5,mxlin) r*4 in mean space view counts for each channel
!      itavg(5) r*4 out line averaged target counts for each channel
!      spavg(5) r*4 out line averaged space view counts for each channel
!      calok(5,mxlin) out calib ok flag
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################
!
!
!--DECLARATIONS----------------------------------------------------------------
      implicit none

!includes/commons
#include "mhsclin.h"
#include "mhs1b.h"
#include "mhsscn.h"
#include "mhs_cinit.h"

! local variables
      integer iscan,ichan,is,iscan2
      real itmean(5,mxlin),spmean(5,mxlin),anorm,bnorm
      real itavg(5),spavg(5),wt
      logical*1 calok(5,mxlin),sampok(5,mxlin)
      
      integer*2 i2word48,i2word6
      parameter (i2word48=48)
      parameter (i2word6=6)
!
!--END OF DECLARATIONS--------------------------------------------------------
!
      if (iprint.ge.1.and.iscan.eq.1.and.ichan.eq.1) then
        write(mhs_cllgu,*) '*************************************'
        write(mhs_cllgu,*) '**   SUBROUTINE MHS_AVG / MHS_CAL  **'
        write(mhs_cllgu,*) '*************************************'
      endif

! Compute weighted mean values from valid counts. If invalid counts are present
! (e.g. at start and end of run) the valid counts are all given a higher 
! weighting.
!
! For MHS, also check that the gain and DC offset settings have not changed
!
      anorm = FLOAT(nscan2+1)  ! nscan2 is half no of scans to average over
      spavg(ichan) = 0.0       ! zeroise arrays
      itavg(ichan) = 0.0
      bnorm = 0.0

      do 600 is = -nscan2 , nscan2
        iscan2 = iscan+is
        if(iscan2.gt.0.and.iscan2.le.mhs_h_scnlin)then
          if (sampok(ichan,iscan2) .and.
     &       (dcoffset(ichan,iscan2).eq.dcoffset(ichan,iscan)).and.
     &       (chgains(ichan,iscan2).eq.chgains(ichan,iscan))) then
            wt = (1.- float(abs(is))/anorm)/anorm
            spavg(ichan) = spavg(ichan) + wt*spmean(ichan,iscan2)
            itavg(ichan) = itavg(ichan) + wt*itmean(ichan,iscan2)
            bnorm=bnorm+wt
          endif
        endif
 600  continue
!
      if(bnorm.eq.0.)then        ! set no counts flags
        chqualflg(ichan,iscan) = IOR(chqualflg(ichan,iscan),i2word48)
        if (iprint.ge.1) then 
          write(mhs_cllgu,*)'Scan',iscan,' chan',ichan,
     &     ' No good counts in mhs_avg'
!Comment out next line as it caused a crash on a g77 system:
!          write(mhs_cllgu,*)'  DC Offset, gain, sampok:',
!     &       ICHAR(dcoffset(ichan,iscan)),ICHAR(chgains(ichan,iscan)),
!     &       sampok(ichan,iscan)
        endif
        calok(ichan,iscan) = .false.
        linqualflgs(iscan) = IOR(linqualflgs(iscan),2048) !Some uncal channels
      else
        itavg(ichan) = itavg(ichan)/bnorm
        spavg(ichan) = spavg(ichan)/bnorm
        calok(ichan,iscan) = .true.
        if(bnorm.lt.1.0)then   ! set some missing counts flags
          chqualflg(ichan,iscan) = IOR(chqualflg(ichan,iscan),i2word6)
          linqualflgs(iscan) = IOR(linqualflgs(iscan),16384)
        endif
      endif
      RETURN
      END





