        subroutine mhs_initcl(noanum)
!+
!NOM/NAME $Id: mhs_initcl.F 25 2006-02-03 15:43:53Z frna $
!
!---------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module opens and reads the calibration coefficients file. 
!     Data are ranged in common mhs_clcoef (include mhs_cinit.h).
!     This module also reads the test coefficients and puts data in 
!     common mhs_tstcf (include mhs_cinit.h).
!
!DESCRIPTION 
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER  
!     Nigel Atkinson
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment                         <author>)
! 1.0      June 2003  Based on amb_initcl.F           NCAtkinson
!         
!ARGUMENTS 
!      integer  in  noanum    ! satellite number (15-16 for NOAA-N, N')
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
! None
! 
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################
!
!--DECLARATIONS-------------------------------------------------------------
      implicit none                                                      
     
! argument variables 
      integer noanum
! local variables
      real tab0(4)                ! intermediate array
      integer i,j                 !index for loops
      integer versnb              !number of version of the coef file
      integer versyr              !year of the version of the coef file
      integer versday             !day of the version of the coef file (0/365)
      integer ios                 !error status
      character*80 charact        !buffer storing one line of the file
!      character*2 calparid        ! character variable of version number
! includes/commons
#include "mhs_cinit.h"
#include "mhsclin.h"
#include "mhsscn.h"
#include "mhs1b.h"
!
!--END OF DECLARATIONS---------------------------------------------------------

      if (iprint.ge.1) write(mhs_cllgu,*)
      if (iprint.ge.1) write(mhs_cllgu,*) '***************************'
      if (iprint.ge.1) write(mhs_cllgu,*) '**  SUBROUTINE MHS_INITCL**'
      if (iprint.ge.1) write(mhs_cllgu,*) '***************************'
!
!Open and read calibration coefficients file
       open(mhs_coefu,access='sequential',status='old',iostat=ios)

!      If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
       if (ios.ne.0)
     & open(mhs_coefu,err=900,access='sequential',status='unknown')
!
!Skips the header lines
       read(mhs_coefu,'(a80)',err=901) charact
       do while (charact(1:1) .eq. '#')
        read(mhs_coefu,'(a80)',err=901) charact
       enddo
       backspace mhs_coefu

       read(mhs_coefu,*,err=901)versnb
       read(mhs_coefu,*,err=901)versyr
       read(mhs_coefu,*,err=901)versday
       mhs_h_cpidsupdyr = versyr
       mhs_h_cpidsupddy = versday
!       write(calparid,'(a2)')versnb
!       mhs_h_calparid = calparid       !calparid no longer used
! 
       if (iprint.ge.1) then
        write(mhs_cllgu,*)
        write(mhs_cllgu,*) 'calibration coefficients file' 
        write(mhs_cllgu,520)versnb,versyr,versday
       endif
        
! Read fundamental constants
       read(mhs_coefu,'(a80)',err=901) charact
       do while (charact(4:9) .ne. 'Speed ')
        read(mhs_coefu,'(a15)',err=901) charact
       enddo
       read(mhs_coefu,*,err=901) c
       read(mhs_coefu,'(a80)',err=901) charact
       do while (charact(4:9) .ne. 'First ')
        read(mhs_coefu,'(a15)',err=901) charact
       enddo
       read(mhs_coefu,*,err=901) plck(1),plck(2)
       do while (charact(4:9) .ne. 'Bright')
        read(mhs_coefu,'(a15)',err=901) charact
       enddo
       read(mhs_coefu,*,err=901) tspace
!
! Get the data for the correct instrument 
!
  10   read(mhs_coefu,'(a80)',err=901) charact
       do while (charact(4:5) .ne. 'ID')
        read(mhs_coefu,'(a15)',err=901) charact
       enddo
       read(mhs_coefu,*,err=901) modid
       if (modid.eq.99)then
         write(mhs_cllgu,*) 'instrument id. not matched no calibration'
         write(mhs_cllgu,*) 'possible '
        noanum=0 
        return
       endif
       if (modid.ne.mhs_h_instid)go to 10
       read(mhs_coefu,*,err=901)
! get center frquencies
       do i = 1 , 5
        read(mhs_coefu,*,err=901)freq(i)
       enddo
! skip 3 records
       do i = 1 , 3
        read(mhs_coefu,*,err=901)
       enddo 
! get band correction coeffs
       do i = 1 , 5
       read(mhs_coefu,*,err=901)b(i),a(i)
       enddo
!
!       read(mhs_coefu,*,err=901)          ! NB now a parameter
!       read(mhs_coefu,*,err=901)nscan     ! no. of scan lines to avg over
       nscan2 = nscan/2  
!                                          
       read(mhs_coefu,*,err=901)
       read(mhs_coefu,*,err=901)nview     ! no. of cal views  
!
       read(mhs_coefu,*,err=901)
       read(mhs_coefu,*,err=901)spview    ! space viewing direction   
!
       read(mhs_coefu,*,err=901)
       read(mhs_coefu,*,err=901)ant_coef  ! conversion to antenna position
!
       read(mhs_coefu,*,err=901)
       read(mhs_coefu,*,err=901)pos_errc,pos_erre ! max error in antenna
!                                                    position   
! IWT PRT resistance to temperature conversion coefficients (10 PRTs,4coefs)
       read(mhs_coefu,*,err=901)
       do i=1,10
        read(mhs_coefu,*,err=901)tab0
        do j=1,4
          prtrestmp(j,i) = tab0(j)
        enddo
       enddo
!
! Weight coefficients for each PRT
       read(mhs_coefu,*,err=901)
       read(mhs_coefu,*,err=901)(prtwght(j),j=1,10)
       
! Reasonable PRT temp limits  (min,max)
       read(mhs_coefu,*,err=901)
       read(mhs_coefu,*,err=901)prtmin,prtmax

! Max PRT temp change allowed before rejecting
       read(mhs_coefu,*,err=901)
       read(mhs_coefu,*,err=901)prtdt
! Minimum number of PRT readings acceptable
       read(mhs_coefu,*,err=901)
       read(mhs_coefu,*,err=901)nprtmin
! Number of lines to fill in PRT data
       read(mhs_coefu,*,err=901)  
       read(mhs_coefu,*,err=901)nprtgap
! 6 instrument reference temps (primary and backup)
       read(mhs_coefu,*,err=901)
       read(mhs_coefu,*,err=901)reftemp
! Thermistor count to temp conv coeffs
       read(mhs_coefu,*,err=901)
       read(mhs_coefu,*,err=901)thermcnttmp
! Internal target correction factors (5 channels, 3 temps)
       read(mhs_coefu,*,err=901)
       do i = 1 ,3
         read(mhs_coefu,*,err=901)(intcorr(j,i),j=1,5)
       enddo
! Cold space correction factors (5 channels, 4 views) 
       read(mhs_coefu,*,err=901)
       do i = 1 , 4
         read(mhs_coefu,*,err=901)(spcorr(j,i),j=1,5)
       enddo
!
! Reasonable count limits by channel (min,max) for internal target
       read(mhs_coefu,*,err=901)
       read(mhs_coefu,*,err=901)intcntmin
       read(mhs_coefu,*,err=901)intcntmax
! Reasonable count limits by channel (min,max) for space view
       read(mhs_coefu,*,err=901)
       read(mhs_coefu,*,err=901)spcntmin
       read(mhs_coefu,*,err=901)spcntmax
! max change in mean counts from previous scan allowed before rejecting
       read(mhs_coefu,*,err=901)
       read(mhs_coefu,*,err=901)dcount
! max number of lines before resetting interline check last value
       read(mhs_coefu,*,err=901)
       read(mhs_coefu,*,err=901)ngap
! non linearity correction terms
       read(mhs_coefu,*,err=901)
       do i = 1, 3
       read(mhs_coefu,*,err=901)(nonlin(j,i),j=1,5)   !LO A
       enddo
       do i = 1, 3
       read(mhs_coefu,*,err=901)(nonlin(j,i),j=6,10)  !LO B
       enddo
! nominal antenna positions for cal views
       read(mhs_coefu,*,err=901)
       do i = 1 , 4
       read(mhs_coefu,*,err=901)(scanpos(j,i),j=1,8)
       enddo

!The following are new for MHS (as compared with AMSU-B):
!
! Reference resistor values
       read(mhs_coefu,*,err=901)
       read(mhs_coefu,*,err=901)(refres(j),j=1,6)
 
! Current coefs
       read(mhs_coefu,*,err=901)
       do i = 1 , 6
       read(mhs_coefu,*,err=901)(curr_coef(j,i),j=1,2)
       enddo      

! Survival coefs
       read(mhs_coefu,*,err=901)
       read(mhs_coefu,*,err=901)(survival_coef(j),j=1,6)

! read moon parameters 
       read(mhs_coefu,*,err=901)
       read(mhs_coefu,*,err=901)moonthresh,moon_azcorr,moon_elevcorr
!
! Close file
       close(mhs_coefu,err=902)
       if(iprint.ge.1)write(mhs_cllgu,*)'Calibration parameters read ok'
!
! Open and read secondary calibration coefficients file
       open(mhs_tstcoefu,access='sequential',status='old',iostat=ios)
       if (ios.ne.0)
     &  open(mhs_tstcoefu,err=900,access='sequential',status='unknown')

! Skips the header lines
       read(mhs_tstcoefu,'(a80)',err=901) charact
       do while (charact(1:1) .eq. '#')
        read(mhs_tstcoefu,'(a80)',err=901) charact
       enddo
       backspace mhs_tstcoefu

       read(mhs_tstcoefu,*,err=901)versnb
       read(mhs_tstcoefu,*,err=901)versyr
       read(mhs_tstcoefu,*,err=901)versday
 
       if (iprint.ge.1) then
        write(mhs_cllgu,*) 'test coefficients file' 
        write(mhs_cllgu,520)versnb,versyr,versday
       endif
 110   read(mhs_tstcoefu,'(a15)',err=901) charact   
       do while (charact(4:5) .ne. 'ID')
        read(mhs_tstcoefu,'(a15)',err=901) charact
       enddo
       read(mhs_tstcoefu,*,err=901) modid
       if (modid.eq.99)then
         write(mhs_cllgu,*)' instrument id. not matched no secondary'
         write(mhs_cllgu,*)' calibration coeffs available '
        return
       endif
       if (modid.ne.mhs_h_instid)go to 110
       read(mhs_tstcoefu,*,err=901)
       do i = 1 , 3
       read(mhs_tstcoefu,'(a80)',err=901) 
       read(mhs_tstcoefu,'(a80)',err=901) 
       read(mhs_tstcoefu,*,err=901)(u0(j,i),j=1,5)
       read(mhs_tstcoefu,'(a80)',err=901) 
       read(mhs_tstcoefu,*,err=901)(u1(j,i),j=1,5)
       read(mhs_tstcoefu,'(a80)',err=901) 
       read(mhs_tstcoefu,*,err=901)(u2(j,i),j=1,5)
       enddo
!
       close(mhs_tstcoefu,err=902)
       if(iprint.ge.1)write(mhs_cllgu,*)' Secondary cal coeffs read ok'
!       
! scale secondary coeffs so they can be stored in 1B
!
      do 120 i = 1 , 3
      do 120 j = 1 , 5
        u0(j,i) = u0(j,i)*1E6
        u1(j,i) = u1(j,i)*1E10
        u2(j,i) = u2(j,i)*1E16
 120  continue
      go to 200
!
!
 520  format(2X,' version number: ',I3/'   version year: ',I4/
     &       '   version day of year: ',I3)
 540  format(3(3X,I3),3(6X,I3))

 900  write(mhs_cllgu,*)'Error opening cal coefficients file'
      stop
 901  write(mhs_cllgu,*)'Error reading cal coefficients file'
      stop
 902  write(mhs_cllgu,*)'Error closing cal coefficients file'
      stop
!
 200  return
      end
