      subroutine mhs_status(lstat,ispvw,sampok)
!+
!NOM/NAME $Id: mhs_status.F 401 2014-07-08 13:47:31Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module checks the quality indicators on each scan line are OK
!     Also checks status of instrument (ie antenna scanning and channels are
!     switched on. If not OK for calibration then status flags returned 
!     as false.  Also the scan line quality indicators are updated.
!    
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
!     Nigel Atkinson
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment                           <author>)
! 1.0      July 2003  Based on amb_status.F v1.2       N C Atkinson
! 1.1      03/07/14   Separate the ICHAR and IAND functions for portability.
!                     Remove some unused variables. NCA
!         
!ARGUMENTS 
!      lstat(mxlin) L*1 in/out  If .TRUE. then data ok if false don't use 
!      ispvw(mxlin) I*4 out  space viewing direction (0-3)
!      sampok(5,mxlin) l*1 out calibrate status for each channel 
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################

!--DECLARATIONS----------------------------------------------------------------
      implicit none                                                     
c-----------------------------------------
!      
! includes/commons
#include "mhsclin.h"
#include "mhs1b.h"
#include "mhsscn.h"
#include "mhs_cinit.h"
!
! Arguments
      logical*1 lstat(mxlin),sampok(5,mxlin)
      integer*4 ispvw(mxlin)

! local variables
      integer*2 zbit
      integer   imode,ichan1,ichan2,i,iscan
      integer*4 zqual,zqual1,zqual2
      logical*1 warn
      data warn/.true./
! 
      data zqual/-1879048192/
      data zbit/48/,zqual1/1024/,zqual2/2048/
!
!--END OF DECLARATIONS---------------------------------------------------------

      if (iprint.ge.1) write(mhs_cllgu,*)
      if (iprint.ge.1) write(mhs_cllgu,*) '***************************'
      if (iprint.ge.1) write(mhs_cllgu,*) '**   SUBROUTINE MHS_STATUS*'
      if (iprint.ge.1) write(mhs_cllgu,*) '***************************'

!..Loop in scan line................................................
      do 100 iscan = 1 , mhs_h_scnlin
      ispvw(iscan) = 0                  ! initialize with default 
      do i = 1 , 5
        sampok(i,iscan) = .true.
      enddo 
!         
! check scan line quality indicator
!
      if ( qualind(iscan) .lt. 0 )then
        lstat(iscan) = .FALSE.        ! bad scan line detected from ingest
        do i = 1 , 5
          sampok(i,iscan) = .false.
        enddo       
        go to 100
      endif
!      
! check instrument scanning OK for calibration and power is on
!       
      imode = ICHAR(mode(iscan))/16    !mode bits 7-4
      if (imode.ne.3) then        !3=scan mode          
        lstat(iscan) = .FALSE. 
        do i = 1 , 5
          sampok(i,iscan) = .false.
        enddo       
        qualind(iscan) = IOR(qualind(iscan),zqual) !set bits 28,31
        linqualflgs(iscan) = IOR(linqualflgs(iscan),zqual1) !set bit 10
        if(iscan.eq.1)
     &   write(mhs_cllgu,*) ' Instrument power off or not scanning'
        go to 100
      endif      
!      
! check at least one channel is on (front end and back end both on).
! If not set qualind bits 28,31 (no calibration, do not use)
!        and linqualflgs bit 10 (uncalibrated due to instrument mode)
!     
      ichan1 = ICHAR(switchstat(1,iscan))
      ichan2 = ICHAR(switchstat(2,iscan))
      
      if (IAND(ichan1,1).ne.1.and.IAND(ichan1,4).ne.4.and.       !H1, H2
     &    IAND(ichan1,80).ne.80.and.IAND(ichan1,192).ne.192.and. !H3, H4
     &    IAND(ichan2,1).ne.1) then                              !H5
        lstat(iscan) = .FALSE.         ! all channels off
        do i = 1 , 5
          sampok(i,iscan) = .false.
        enddo      
        qualind(iscan) = IOR(qualind(iscan),zqual)          !set bits 28,31
        linqualflgs(iscan) = IOR(linqualflgs(iscan),zqual1) !set bit 10
        if(iscan.eq.1)
     &   write(mhs_cllgu,*) ' all channels off'
        go to 100
      endif
! 
!  get space view position (0->3)
!
      i = ICHAR(status(iscan))
      ispvw(iscan) = IAND(i,48)/16
      if (ispvw(iscan).ne.spview .and. warn)then
       if(iprint.ge.1)
     &  write(mhs_cllgu,*)' Warning space view ',ispvw(iscan),
     &' selected ',spview,' is recommended'
       warn = .false.
      endif
!
! check which channels are on and set flags for any that are off.
! linqualflgs: bit 11 = some uncalibrated channels
! chqualflgs: bits 4 & 5 = all bad space & target counts
!     
      if (IAND(ichan1,1).ne.1) then
        linqualflgs(iscan) = IOR(linqualflgs(iscan),zqual2) 
        chqualflg(1,iscan) = IOR(chqualflg(1,iscan),zbit)
        sampok(1,iscan) = .false.
      endif
      if (IAND(ichan1,4).ne.4) then
        linqualflgs(iscan) = IOR(linqualflgs(iscan),zqual2) 
        chqualflg(2,iscan) = IOR(chqualflg(2,iscan),zbit)
        sampok(2,iscan) = .false.
      endif
      if (IAND(ichan1,80).ne.80) then
        linqualflgs(iscan) = IOR(linqualflgs(iscan),zqual2) 
        chqualflg(3,iscan) = IOR(chqualflg(3,iscan),zbit)
        sampok(3,iscan) = .false.
      endif
      if (IAND(ichan1,192).ne.192) then
        linqualflgs(iscan) = IOR(linqualflgs(iscan),zqual2) 
        chqualflg(4,iscan) = IOR(chqualflg(4,iscan),zbit)
        sampok(4,iscan) = .false.
      endif
      if (IAND(ichan2,1).ne.1) then
        linqualflgs(iscan) = IOR(linqualflgs(iscan),zqual2) 
        chqualflg(1,iscan) = IOR(chqualflg(1,iscan),zbit)
        sampok(1,iscan) = .false.
      endif
!
 100  Continue   ! ......end of scan line loop
!
      return
      end
