      subroutine mhs_updt(nvstat,calok,nbnavcal,ispvw)
!+
!NOM/NAME $Id: mhs_updt.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!    This module updates the MHS 1B header array and writes out
!    header and navigated, calibrated scan lines to the 1B file.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!
!PROPRIETAIRE/OWNER  
!     Nigel Atkinson  
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment                            <author>)
! 1.0      June 2003  Based on amb_updt.F               NCAtkinson
! 1.1      19 Oct 2004 mhs_obcttemp now I*4             NCA
! 1.2      13 May 2005 add mhs_h_nonlin and mhs_h_temrad    NCA
! 1.3      25 Oct 2005 bug fix in indexing of mhs_h_nonlin  NCA
!
!ARGUMENTS
! nvstat(mxlin) l*1 in array containing navigation flags
! calok(5,mxlin) l*1 in array containing calibration flags
! nbnavcal i*4 out number of navigated and calibrated scans
! ispvw(mxlin) I*4 in  space viewing direction (0-3)
!
! EXTERNALS
! mhshdw, mhsdtr, mhsdtw
!------------------------------------------------------------------------------
!--DECLARATIONS----------------------------------------------------------------
      implicit none                                                      

!includes/commons
#include "mhsclin.h"
#include "mhs1b.h"
#include "mhsscn.h"
#include "mhs_cinit.h"
!
!local  variables
      integer iscan,ios,ic,j,i,nh,k
      integer nbnavcal,ispvw(mxlin)
      logical*1 nvstat(mxlin),calok(5,mxlin)
      real scala(5)
      data scala/1E4,1E7,1E10,1E12,1E15/   !Scaling for thermistor coefs
!
!--END OF DECLARATIONS-------------------------------------------------
      if(iprint.ge.1)then
       write(mhs_cllgu,*)' ******************************'
       write(mhs_cllgu,*)' *          MHS_UPDT'
       write(mhs_cllgu,*)' ******************************'
      endif
!
      nbnavcal = 0
      nh = mhs_h_hdrcnt

!..determine nbnavcal
      do iscan = 1 , mhs_h_scnlin
        if(nvstat(iscan))then
          if(calok(1,iscan).and.calok(2,iscan).and.calok(3,iscan).and.
     &      calok(4,iscan).and.calok(5,iscan)) nbnavcal = nbnavcal+1
        endif
      enddo

!..update header with calibration params (nb with correct scaling factors)
      mhs_h_callocsclin = nbnavcal
      do i=1,6           !3 temps, 2 sensors
        mhs_h_reftemp(i) = nint(reftemp(i)*100)
      enddo

      do ic=1,5        !5 channels
        j = (ic-1)*7
        mhs_h_bias(j+1) = nint(intcorr(ic,1)*1000)   !warm corr at 3 temps
        mhs_h_bias(j+2) = nint(intcorr(ic,2)*1000)
        mhs_h_bias(j+3) = nint(intcorr(ic,3)*1000)
        mhs_h_bias(j+4) = nint(spcorr(ic,1)*1000)    !Cold corr for 4 views
        mhs_h_bias(j+5) = nint(spcorr(ic,2)*1000)
        mhs_h_bias(j+6) = nint(spcorr(ic,3)*1000)
        mhs_h_bias(j+7) = nint(spcorr(ic,4)*1000)
        j = (ic-1)*3
        do i=1,3       !3 instrument temps
          mhs_h_nonlin(j+i) = nint(nonlin(ic,i)*1.0E8)    !Nonlinearity for LO-A
          mhs_h_nonlin(j+i+15) = nint(nonlin(ic+5,i)*1.0E8) !- and for LO-B
        enddo
        mhs_h_temrad(j+1) = nint(freq(ic)*1E6)        !Central wavenumber
        mhs_h_temrad(j+2) = nint(b(ic)*1E6)           !Band coefficient B
        mhs_h_temrad(j+3) = nint(a(ic)*1E6)           !Band coefficient A
      enddo
!
!..Thermistor coeffs
      do i=1,5
        mhs_h_therm(i) = nint(thermcnttmp(i)*scala(i))
      enddo

!..Current conversion coeffs
      do i = 1 , 6
        k = (i-1)*2 + 1
        mhs_h_currcoef(k) = nint(curr_coef(1,i)*1E6)
        mhs_h_currcoef(k+1) = nint(curr_coef(2,i)*1E6)
      enddo

!..Survival conversion coeffs
      do i=1,6
        mhs_h_survival(i) = nint(survival_coef(i)*1E6)
      enddo

!..Reference resistances
      do i=1,6
        mhs_h_refres(i) = nint(refres(i)*1E4)
      enddo

!..Moon threshold
      mhs_h_moonthresh = nint(moonthresh*100)

!..write out header to disk
      call mhshdw(mhs_l1bu,ios)
      if(ios.ne.0)then
       write(mhs_cllgu,*)' MHS 1B header could not be written'
       stop
      endif
!
!..write out all data records to disk
!
      do 30 iscan = 1 , mhs_h_scnlin  !-----start scan loop-------
!..read current 1B record before updating items that may have changed
       call mhsdtr(mhs_l1bu,iscan,ios)
       if(ios.ne.0)then
        write(mhs_cllgu,*)' MHS 1B data record could not be read'
        stop
       endif
       mhs_scnlinyr = scnlinyr(iscan)
       mhs_scnlindy = scnlindy(iscan)
       mhs_scnlintime = scnlintime(iscan)
       mhs_clockdrift = clockdrift(iscan)
       mhs_qualind = qualind(iscan)
       mhs_scnlinbit = scnlinbit(iscan)
       mhs_linqualflgs = linqualflgs(iscan)
       do i = 1 , 5
         mhs_chqualflg(i) = chqualflg(i,iscan)
       enddo
       mhs_attangtime = attangtime(iscan)
       mhs_rollang = rollang(iscan)
       mhs_pitchang = pitchang(iscan)
       mhs_yawang = yawang(iscan)
       mhs_navstat = navstat(iscan)
       mhs_scalti = scalti(iscan)
       do i = 1,mxpix*3
         mhs_ang(i) = ang(i,iscan)
       enddo
       do i = 1 , mxpix*2
         mhs_pos(i) = earthl(i,iscan)
       enddo
       do i=1,4
         mhs_moonang(i) = moonang(i,iscan)
       enddo
       do i = 1 , 15
         mhs_pmcal(i) = pmcal(i,iscan)
         mhs_sdcal(i) = sdcal(i,iscan)
       enddo
       do i=1,5
         mhs_obcttemp(i) = obcttemp(i,iscan)
       enddo
       do i=1,8
         mhs_earthval(i) = earthval(i,iscan)
       enddo
       mhs_spaceval = spaceval(iscan)
       mhs_targval = targval(iscan)

!..write current updated 1B record 
       call mhsdtw(mhs_l1bu,iscan,ios)
       if(ios.ne.0)then
       write(mhs_cllgu,*)' MHS 1B data record could not be written'
       stop
       endif
 30   continue  !------end scan loop------------- 
!
      return
      end

