      subroutine mhsclexit(nbnavcal,nvstat,lstat,calok,sampok,prtstat)
!+
!NOM/NAME $Id: mhsclexit.F 47 2006-08-21 08:44:41Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!    This module closes the amsubcl program and provides a summary
!    of the job if iprint is greater than or equal to 1.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!
!PROPRIETAIRE/OWNER  
!     Roger Saunders  
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment                            <author>)
! 1.0      July 2003  Created from ambclexit            N C Atkinson
!         
!ARGUMENTS
! nbnavcal i*4 in number of navigated and calibrated scans
! lstat(mxlin) l*1 in scan line quality flag (.true. is ok)
! nvstat(mxlin) l*1 in navigation quality flag
! calok(5,mxlin) l*1 in calibration ok flag
! sampok(5,mxlin) l*1 in cal view radiances quality flag
! prtstat(mxlin) l*1 in target and mixer prt quality flag
!------------------------------------------------------------------------------
!--DECLARATIONS----------------------------------------------------------------
      implicit none                                                      

!includes/commons
#include "mhsclin.h"
#include "mhs1b.h"
#include "mhsscn.h"
#include "mhs_cinit.h"
!
!local  variables
      integer iscan,ic
      integer nbnavcal,nnav,ncal(5),nprt,nsamp(5),ngood
      logical*1 nvstat(mxlin),lstat(mxlin),calok(5,mxlin)
      logical*1 sampok(5,mxlin),prtstat(mxlin)
      integer ierr        ! 0 = OK
      integer ncaltot     ! total number of calibrated scan lines*channels 
!
!--END OF DECLARATIONS-------------------------------------------------
      if(iprint.ge.1)then
       write(mhs_cllgu,*)' ******************************'
       write(mhs_cllgu,*)' *          MHSCLEXIT         *'
       write(mhs_cllgu,*)' ******************************'
!..print summary
!
      write(mhs_cllgu,*)' '
      write(mhs_cllgu,*)'     MHSCL summary log'
      write(mhs_cllgu,*)'     -----------------'
      nnav = 0
      nprt = 0
      ngood = 0
!      
      do 5 ic = 1 , 5
      nsamp(ic) = 0
      ncal(ic) = 0
 5    continue
      do 100 iscan = 1 , mhs_h_scnlin
       if(nvstat(iscan))nnav = nnav + 1
       if(lstat(iscan))ngood = ngood + 1
       if(prtstat(iscan))nprt = nprt + 1
       do 100 ic = 1 , 5
        if(calok(ic,iscan))ncal(ic) = ncal(ic) + 1
        if(sampok(ic,iscan))nsamp(ic) = nsamp(ic) + 1
 100   continue
      write(mhs_cllgu,7)mhs_h_scnlin
 7    format(1x,i5,' scan lines in this dataset')
      write(mhs_cllgu,10)nbnavcal
 10   format(1x,i5,' scan lines were calibrated and navigated')
      write(mhs_cllgu,20)mhs_h_scnlin-ngood
 20   format(1x,i5,' scan lines were flagged as corrupt data')
      write(mhs_cllgu,30)nnav
 30   format(1x,i5,' scan lines had good antenna pointing')
      write(mhs_cllgu,40)nprt
 40   format(1x,i5,' scan lines had good prt data',/)
      write(mhs_cllgu,50)(nsamp(ic),ic=1,5)
 50   format(1x,'Chan 16=',i5,' scans of good cal views',/,
     &' Chan 17=',i5,' scans of good cal views',/,
     &' Chan 18=',i5,' scans of good cal views',/,
     &' Chan 19=',i5,' scans of good cal views',/,
     &' Chan 20=',i5,' scans of good cal views',/)
      write(mhs_cllgu,60)(ncal(ic),ic=1,5)
 60   format(1x,'Chan 16=',i5,' scans calibrated',/,
     &' Chan 17=',i5,' scans calibrated',/,
     &' Chan 18=',i5,' scans calibrated',/,
     &' Chan 19=',i5,' scans calibrated',/,
     &' Chan 20=',i5,' scans calibrated',/)
      endif

!..close files
!
      close(mhs_l1bu)
!      close(mhs_cllgu)
      close(mhs_stu)
!   
!
!.. final quality test for end of execution return code
      ierr = 0
      if(mhs_lflg.eq.1 .and. nnav.le.10) then
         write(mhs_cllgu,*) 'Error not enough navigated lines'
         ierr = 1
      endif

      ncaltot = 0
      do ic=1,5
         ncaltot = ncaltot+ncal(ic)
      end do
      if(mhs_cflg.eq.1 .and. ncaltot.le.50) then
         write(mhs_cllgu,*) 'Error not enough calibrated lines'
         ierr = 1
      endif

      close(mhs_cllgu)

      if(ierr.ne.0) then
         ! all files are closed, exit 10 will be returned to user 
         ! through amsub.ksh script and enables to stop further processing
         call aappexit(10)
      endif

      return
      end


