      subroutine mhsclin(mhs_cloptn)
!+
!NOM/NAME $Id: mhsclin.F 47 2006-08-21 08:44:41Z frna $
!
!----------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module initializes the MHS navigation and calibration tasks. 
!     It reads the input options (mhs_cloptn)
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER  
!     Nigel Atkinson
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment                          <author>)
! 1.0      July 2003  From mhsclin.F                 N C Atkinson
!         
!ARGUMENTS 
!     output: mhs_cloptn(12)   (see after for the meaning)
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!     none
! 
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################
!
!--DECLARATIONS--------------------------------------------------------------
      implicit none                                                      
! local variables
      integer i         !index of loop
      integer ios
      integer mhs_cloptn(12) 
!    mhs_cloptn(1) -- 1 for only MHS navigation/localisation (0 if not for)
!    mhs_cloptn(2) -- 1 for only MHS calibration (0 if not for)
!    mhs_cloptn(3) -- 1 for statistics file (0 if not for)
!    mhs_cloptn(4) -- logical unit for MHS navig/loc/calib log file
!    mhs_cloptn(5) -- logical unit for statistics file
!    mhs_cloptn(6) -- logical unit MHS level1b file
!    mhs_cloptn(7) -- logical unit for SATPOS file
!    mhs_cloptn(8) -- logical unit for the clock drift data file
!    mhs_cloptn(9) -- logical unit for MHS calibration coefficients file
!    mhs_cloptn(10)-- logical unit for MHS calib/test coefficients file
!    mhs_cloptn(11)-- print option in the log.file
!                       = 0, only warning messages are printed
!                       = 1, entering routines and few values of the main
!                            parameters are printed
!                       = 2, computed values that would help in debugging 
!                            are printed
!    mhs_cloptn(12)-- logical unit for MHS bias correction tables
!    yaw,roll,pitch are the observed attitude parameters bias milliradians
!    which have to be read when localisation is required
!    if one does not know the values put
!               -99.99 -99.99 -99.99
!    the attitude bias come from an attitude estimation program 

! includes/commons
#include "mhsclin.h"     
#include "c_1bnav.h"     
!
!--END OF DECLARATIONS---------------------------------------------------------

      read(*,*,iostat=ios) mhs_cloptn
      if(ios.ne.0) then
         call ml_wti('error','mhsclin reading options',ios)
         call aappexit(2)
       endif
! Check mhs_cloptn values
      do i = 1,3
        if(mhs_cloptn(i).gt.1 .or. mhs_cloptn(i).lt.0) then
            write(mhs_cloptn(4),*)
     &      ' **Invalid option number--correct input**' ,mhs_cloptn(i)
            stop
        endif
      enddo

      if((mhs_cloptn(11).ne.0).and.(mhs_cloptn(11).ne.1).and.
     & (mhs_cloptn(11).ne.2)) then
            write(mhs_cloptn(4),*)
     &      ' **Invalid print option value--correct input**' 
           stop
      endif


      mhs_lflg    = mhs_cloptn(1)
      mhs_cflg    = mhs_cloptn(2)
      if(mhs_cloptn(1).eq.1.and.mhs_cloptn(2).eq.1) then
        mhs_clflg   = 1
      endif
      mhs_stflg   = mhs_cloptn(3)
      mhs_cllgu   = mhs_cloptn(4)
      mhs_l1bu    = mhs_cloptn(6)
      iprint      = mhs_cloptn(11)

      if (mhs_stflg.eq.1) then
        mhs_stu    = mhs_cloptn(5)
      else
        mhs_stu   = 0 
      endif
     
      if (mhs_clflg.eq.1.or.mhs_lflg.eq.1) then 
        satposu     = mhs_cloptn(7)
        ckdriftu    = mhs_cloptn(8)
        mhs_coefu   = mhs_cloptn(9)
        mhs_tstcoefu= mhs_cloptn(10)
        mhs_corcoefu= mhs_cloptn(12)
! read attitude parameters
        read(*,*,iostat=ios) yaw, roll, pitch
            if(ios.ne.0) then
!             call ml_wti('error',
!     &       'while reading attitude values I/O status',ios)
!             call ml_wt('error','amsubcl aborted')
          write(mhs_cloptn(4),*) 
     &      ' **Error while reading attitude values--correct input**'
              call aappexit(1)
             endif
! convert the values in radians
             yaw = yaw*1.d-3
             roll = roll*1.d-3
         pitch = pitch*1.d-3
      endif
      
      mhs_geou    = 0
      if(mhs_lflg.eq.0) then
        satposu     = 0
        ckdriftu    = 0
      endif
      if (mhs_cflg.eq.1) then
        mhs_coefu   = mhs_cloptn(9)
        mhs_tstcoefu= mhs_cloptn(10)
        mhs_corcoefu= mhs_cloptn(12)
      endif
!
      return
      end
