      subroutine mhssetu(noanum,lstat,nvstat,calok)
!+
!NOM/NAME $Id: mhssetu.F 406 2014-07-28 09:08:47Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module opens  the log file, the statistics file (if requested).
!     It opens the MHS level1b, reads the header, checks data type and 
!     reads all the records. The 1b data are arranged in common mhs1b.h
!     The navigation files are also opened.
!
!DESCRIPTION 
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
!     Nigel Atkinson
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment                       <author>)
! 1.0      April 2004  based on ambsetu.F.          N C Atkinson
! 1.1      July 2004   Upgrade to navigation code version 2.  N C Atkinson
! 1.2      19 Oct 2004 mhs_obcttemp now I*4         NCA
! 1.3      23 Dec 2005 Allow for different NESDIS instrument ID. NCA
! 1.4      May 06   Remove tests on n < noaak ( PM/PB )
! 1.5      July 14  Separate the ICHAR and IAND functions for portability.
!                   If instrument id is >100, subtract 100. NCA
!         
!ARGUMENTS
! noanum i*4 out  satellite id number (see satid file)
! lstat(mxlin) l*1 out scan line status flag
! nvstat(mxlin) l*1 out navigation status flag
! calok(5,mxlin) l*1 out calibration status flag
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
! ord1bid  mhshdr mhsdtr ml_wti ml_wt ml_wtdp def_att calatt 
! clkerr_get ml_wtr  sp_read 
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################

!--DECLARATIONS----------------------------------------------------------------
      implicit none                                                     
!
!
#include "mhsclin.h"
#include "mhs1b.h"
#include "mhsscn.h"
#include "c_1bnav.h"
#include "reclopt.h"
!
! local variables
!      integer j,irec1    !Only needed if bias corrections implemented
      integer i,ios,mhs_h_satid0,noanum
      integer irec
      integer recn       !record number in the 1B file
      logical*1 lstat(mxlin),calok(5,mxlin),nvstat(mxlin) ! status flags
      integer ierr
      double precision ijstrt,ijend
      character*6 scname
      integer loc_rl     ! local record length for opening MHS 1B file

C The following are only needed if STX transmitters interfere
C      integer*2 OldTxPow(5)  !Transmitter powers
C      integer*2 Flag,OldFlag   !Anomalous bias flags
      integer att_mode          ! attitude mode
                                ! 0 = NOAA POES operational mode
                                ! 1 = METOP Yaw steering mode
                                ! 2 = Geocentric mode
!      
! functions
      integer ord1bid 
      character*6 noascnam


!
!--END OF DECLARATIONS---------------------------------------------------------

!open the log file
      open(mhs_cllgu,form='formatted',access='sequential',
     &     status='new',iostat=ios)

!     If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
      if ( ios.ne.0 )
     &  open(mhs_cllgu,form='formatted',access='sequential',
     &     status='unknown',iostat=ios)
!     
      if (iprint.ge.1) write(mhs_cllgu,*) '**************************'
      if (iprint.ge.1) write(mhs_cllgu,*) '** Starting MHSCL       **'
      if (iprint.ge.1) write(mhs_cllgu,*) '**************************'
      if (iprint.ge.1) write(mhs_cllgu,*)
      if (iprint.ge.1) write(mhs_cllgu,*)
      if (iprint.ge.1) write(mhs_cllgu,*) '***************************'
      if (iprint.ge.1) write(mhs_cllgu,*) '**   SUBROUTINE MHSSETU  **'
      if (iprint.ge.1) write(mhs_cllgu,*) '***************************'


!open the statistics file
      if(mhs_stflg.eq.1) then
        open(mhs_stu,form='formatted',access='sequential',
     &       status='new',iostat=ios)

!       If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
        if ( ios.ne.0 )
     &  open(mhs_stu,form='formatted',access='sequential',
     &       status='unknown',iostat=ios)
      endif

!open and read the MHS-level1b file
      loc_rl = reclmhs / rldiv
      open(mhs_l1bu,form='unformatted',access='direct',recl=loc_rl,
     &     status='old',iostat=ios)

!     If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
      if ( ios.ne.0 )
     &  open(mhs_l1bu,form='unformatted',access='direct',recl=loc_rl,
     &     status='unknown',iostat=ios)

      if(ios.ne.0) then
       write(mhs_cllgu,*)'MHS 1B file could not be opened,ios:',ios
       stop
      else
       if(iprint.ge.1)
     & write(mhs_cllgu,*)'MHS 1B file has been opened,ios:',ios
      endif

      call mhshdr(mhs_l1bu,ios)
      if(ios.ne.0) then
       write(mhs_cllgu,*)' MHS 1B header could not be read,ios:',ios
       stop
      else
       if(mhs_h_datatyp.ne.12)then
         write(mhs_cllgu,*)' Not MHS data '
         stop
       endif
       if (mhs_h_instid.ge.100) mhs_h_instid = mhs_h_instid-100  !NOAA 1b for Metop-B
       if (mhs_h_instid.eq.0) mhs_h_instid=1 !NOAA 1b files have ID=0 for NOAA-N
 
C Expect non-zero status so comment out the following
C       if(mhs_h_inststat1.ne.0.and.mhs_h_inststat2.ne.0)then
C         write(mhs_cllgu,*)' Warning abnormal instrument status'
C       endif  

C Do not expect to need bias corrections but keep code in case
C       CALL MHS_READCORR     !Read bias correction tables and store in header
C       IF (mhs_h_TxPow(1).NE.0) THEN   !Only if the ref powers are non zero
C        IF (mhs_h_scnlin.GE.6) THEN
C         CALL mhsdtr(mhs_l1bu,6,ios)   !Read transmitter powers from line 6
C         DO J=1,5
C           OldTxPow(J) = mhs_stx(4+J)
C         ENDDO
C        ENDIF         
C       ENDIF
C
C       OldFlag=-2  !Initialise
C       Flag=-2     !Initialise

       IF (mhs_h_scnlin .gt. mxlin) THEN
         write(mhs_cllgu,*)
     &    'WARNING: too many scan lines',mhs_h_scnlin,
     &    '. Just process to line',mxlin
         mhs_h_scnlin = mxlin
       ENDIF
       
       irec = 0
       do 100 recn= 1 , mhs_h_scnlin
        irec = irec + 1
        call mhsdtr(mhs_l1bu,irec,ios)
        if(ios.ne.0) then
         write(mhs_cllgu,*)
     &         ' MHS 1B record could not be read, ios:',ios
         stop
        else
!
         lstat(irec) = .true.
         nvstat(irec) = .false.               ! init nav status flag
         scnlin(irec)     = mhs_scnlin 
         scnlinyr(irec)   = mhs_scnlinyr 
         scnlindy(irec)   = mhs_scnlindy 
         scnlintime(irec) = mhs_scnlintime 
         clockdrift(irec) = mhs_clockdrift          
         qualind(irec)    = mhs_qualind 
         scnlinbit(irec)  = mhs_scnlinbit 
         linqualflgs(irec)  = mhs_linqualflgs 
         do i = 1,5
          calok(i,irec) = .false.                 ! init cal status flag
          chqualflg(i,irec) = mhs_chqualflg(i)
         enddo
         tipmf(irec)       = mhs_tipmf 
         modeflag(irec)    = mhs_modeflg 
         attangtime(irec)  = mhs_attangtime 
         rollang(irec)     = mhs_rollang
         pitchang(irec)    = mhs_pitchang
         yawang(irec)      = mhs_yawang
         navstat(irec)     = mhs_navstat
         scalti(irec)      = mhs_scalti

         do i = 1 , mxpix*3
          ang(i,irec) = mhs_ang(i)
         enddo
         do i = 1 , mxpix*2
          earthl(i,irec) = mhs_pos(i)
         enddo

         do i =  1 , 540
           if (mhs_earth_data(i).ge.0) then
             earth_data(i,irec) = mhs_earth_data(i)
           else
             earth_data(i,irec) = mhs_earth_data(i)+65536
           endif
         enddo

         do i = 1 , 24
           temps(i,irec) = mhs_temps(i)
           if (mhs_coldcal(i).ge.0) then
             coldcal(i,irec) = mhs_coldcal(i)
           else
             coldcal(i,irec) = mhs_coldcal(i)+65536
           endif
           if (mhs_warmcal(i).ge.0) then
             warmcal(i,irec) = mhs_warmcal(i)
           else
             warmcal(i,irec) = mhs_warmcal(i)+65536
           endif
         enddo

         do i=1,12
           earthval(i,irec) = mhs_earthval(i)
         enddo
         spaceval(irec) = mhs_spaceval
         targval(irec) = mhs_targval
         mode(irec) = mhs_mode
         do i=1,5
           fault(i,irec) = mhs_fault(i)
           obctprt(i,irec) = mhs_obctprt(i)
           obcttemp(i,irec) = mhs_obcttemp(i)
           dcoffset(i,irec) = mhs_dcoffset(i)
         enddo  
         do i=1,3
           switchstat(i,irec) = mhs_switchstat(i)
           refrescts(i,irec) = mhs_refrescts(i)
           survtemp(i,irec) = mhs_survtemp(i)
         enddo  
         do i=1,6
           current(i,irec) = mhs_current(i)
           discrete(i,irec) = mhs_discrete(i)
         enddo  
         status(irec) = mhs_stat
         chvalflgs(irec) = mhs_chvalflgs
         do i=1,9
           stx(i,irec) = mhs_stx(i)
         enddo  
         ahkval(irec) = mhs_ahkval
C For chgains, separate into channels
         i = ICHAR(mhs_chgains(1))
         chgains(1,irec) = CHAR(i/32)           !Bits 7-5
         chgains(2,irec) = CHAR(IAND(i,28)/4)   !Bits 4-2
         i = ICHAR(mhs_chgains(2))
         chgains(3,irec) = CHAR(i/32)
         chgains(4,irec) = CHAR(IAND(i,28)/4)
         i = ICHAR(mhs_chgains(3))
         chgains(5,irec) = CHAR(i/32)

C Only needed with bias corrections:
! For scans 1-5, use the transmitter powers from scan 6. The SARR power is only
! updated every 16 seconds.
!
C         IF (mhs_h_TxPow(1).NE.0) THEN   !Only if the ref powers are non zero
C          IF (irec.LT.6) THEN
C           DO J=1,5
C             mhs_stx(4+J)=OldTxPow(J)
C           ENDDO
C          ENDIF
!
! Correct for normal and anomalous bias effects on space and target view.
! Only do the corrections and set flags if the quality indicator is +ve: the 
! decommutation sets bit 31 if not scan normal or there is a sync error.
!
C          IF (qualind(irec).GE.0) THEN
C           CALL MHS_TestNewBias(earth_data(1,irec),Flag)
C           CALL MHS_CALCORRECT(coldcal(1,irec),warmcal(1,irec),Flag)
C          IF (Flag.EQ.1) qualind(irec)=IOR(qualind(irec),32) !Flag anomalous
C           IF (Flag-OldFlag.EQ.1) THEN                       ! bias
C             write(mhs_cllgu,*) 'Anomalous bias on at scan ',irec
C           ELSE IF (Flag-OldFlag.EQ.-1) THEN
C             write(mhs_cllgu,*) 'Anomalous bias off at scan ',irec
C           ENDIF
C           IF (ABS(Flag-OldFlag).EQ.1) THEN            !Use bit 6 to flag bias
C              qualind(irec) = IOR(qualind(irec),64)    ! change: 2 scans are 
C              qualind(irec-1) = IOR(qualind(irec-1),64)! unreliable
C           ENDIF
C          ENDIF                                                 
!
! Flag transmitter power changes
!
C          DO J=1,5
C           IF ((ABS(mhs_stx(4+J)-OldTxPow(J)).GT.50).OR.
C     &         (ABS(Flag-OldFlag).EQ.1)) THEN
C             irec1 = irec-3
C             IF (irec1.LT.1) irec1=1
C             DO I=irec,irec1,-1
C               qualind(I) = IOR(qualind(I),16)   !Flag lines within convolution
C             ENDDO                             ! interval
C             IF ((mhs_stx(4+J)-OldTxPow(J)).GT.50) THEN
C               write(mhs_cllgu,*) 'Transmitter ',J,' on at scan ',irec
C             ELSE IF ((mhs_stx(4+J)-OldTxPow(J)).LT.-50) THEN
C               write(mhs_cllgu,*) 'Transmitter ',J,' off at scan ',irec
C             ENDIF
C           ENDIF
C          ENDDO
C          DO J=1,5
C            OldTxPow(J) = mhs_stx(4+J)   !Store transmitter powers
C          ENDDO
C          OldFlag = Flag
C         ENDIF
!
        endif    !MHS record read OK
 100   continue  !Scan line loop
      endif
!
! Go backwards through 'qualind' array, flagging any lines for which the bias
! or transmitters change within convolution interval
!
C      DO recn = mhs_h_scnlin-1,1,-1
C        IF ( IAND(qualind(recn),16) .GT.0) THEN
C          DO irec=recn+1,recn+3
C            IF (irec.LE.mhs_h_scnlin) THEN
C              qualind(irec) = IOR(qualind(irec),16)    !Set bit 4
C            ENDIF
C          ENDDO
C        ENDIF
C      ENDDO

!Deliver the order satellite number for the l1b satellite number and
!the julian date of the data
      mhs_h_satid0 = mhs_h_satid
      noanum = ord1bid(mhs_h_satid0,mhs_h_startdatajd)  
!  
      if (iprint.ge.1) then
       write(mhs_cllgu,fmt=504) scnlintime(1)
       write(mhs_cllgu,fmt=505) scnlindy(1)
       write(mhs_cllgu,fmt=506) scnlinyr(1)
       write(mhs_cllgu,fmt=507) scnlintime(irec)
       write(mhs_cllgu,fmt=508) scnlindy(irec)
       write(mhs_cllgu,fmt=509) scnlinyr(irec)
       write(mhs_cllgu,fmt=501) mhs_h_scnlin
      endif
!
!       
       if (iprint.ge.1)write(mhs_cllgu,fmt=500) mhs_h_satid0
       if (iprint.ge.1)write(mhs_cllgu,510) noascnam(noanum)
      
      scname=noascnam(noanum)
      write(mhs_cllgu,*) 'scname ',scname
!
! Open localisation files if requested
! i.e. according to the options read in mhsclin module
!
      if(mhs_lflg.eq.1.or.mhs_clflg.eq.1) then
! Opens and reads the SATPOS file
! -----------------------------------
          open (unit=satposu,status='old',iostat=ios)

!         If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
          if ( ios.ne.0 )
     &      open (unit=satposu,status='unknown',iostat=ios)

          if( ios.ne.0 ) then
          call ml_wti('error',
     &       'while opening the satpos file I/O status',ios)
          call ml_wt('error','mhssetu aborted')
          call aappexit(2)
          endif

! Read the satpos file
! --------------------
          ijstrt=mhs_h_startdatajd + 
     &      (mhs_h_startdatatime-errclock*1000.)/86400.d3
          ijend=mhs_h_enddatajd + 
     &       (mhs_h_enddatatime-errclock*1000.)/86400.d3
          call sp_read(ijstrt,ijend,satposu,scname,ierr)
          close(unit=satposu,iostat=ios)
        
          if(ierr.ne.0) then
             call ml_wti('error','return code of sp_read differ from 0'
     &                   ,ierr)
             call ml_wt('error','mhssetu aborted')
             call ml_wt('error','navigation aborted')
             call ml_wt('error','but calibration can work')
             call ml_wt('error','try calibration alone')
             call aappexit(2)
          endif

          if(yaw.lt.-99.d-03) then
             call def_att(scname,att_mode,yaw,roll,pitch)
             call ml_wt('info','take default attitude values')
             call ml_wti('info','default attitude mode ',att_mode)
             call ml_wtdp('info','default attitude yaw ',yaw)
             call ml_wtdp('info','default attitude roll ',roll)
             call ml_wtdp('info','default attitude pitch ',pitch)
          else
             call ml_wtdp('info','attitude yaw ',yaw)
             call ml_wtdp('info','attitude roll ',roll)
             call ml_wtdp('info','attitude pitch ',pitch)
          endif
          call calatt(yaw,roll,pitch,att)

! If the file has already been located we do not correct
! again from the clock error ( this is known from the header
! epoyr test
! --------------------------
          if(mhs_h_epoyr.le.1950) then
          open (unit=ckdriftu,status='old',iostat=ios)
          if(ios.ne.0)
     &    open (unit=ckdriftu,status='unknown',iostat=ios)
          if(ios.ne.0) then
            call ml_wti('error',
     &       'opening clock error file I/O status',ios)
            call aappexit(2)
          endif
          ijstrt=mhs_h_startdatajd + 
     &      (mhs_h_startdatatime)/86400.d3
          call clkerr_get(ckdriftu,ijstrt,errclock,ierr)
          close(unit=ckdriftu,iostat=ios)
          if(ierr.ne.0.or.abs(errclock).gt.2.) then
            call ml_wti('warning',
     &       'reading clock error value from clkerr_get ',ierr)
            call ml_wtr('warning','clock error from file',
     &                 errclock)
            call ml_wt('warning','clock error forced to 0')
            errclock=0 
          else
            call ml_wtr('info','clock error value (sec)',errclock)
          endif
         else
          errclock=0.
          call ml_wt('info',
     &     '1B file already corrected from clock error')
         endif
        
! Init earth location parameters
! ------------------------------
        first_pnav = 1 ! first pixel to locate
        first_lnav = 1 ! first scan to process
        step_pnav = 1  ! step between pixels to locate
        step_lnav = 1  ! step between lines to locate
        nb_pnav = 90 ! 90 pixels for MHS
      endif
!
 500  format(1X,'NOAA 1B spacecraft identification code: ',I2)
 501  format(1X,' Number of MHS scan lines in this dataset ',I4)
 504  format(1X,'start of data set time (msecs): ',I8)
 505  format(1X,'start of data set Julian day: ',I7)
 506  format(1X,'start of data set Year: ',I7)
 507  format(1X,'end of data set time (msecs): ',I8)
 508  format(1X,'end of data set Julian day: ',I7)
 509  format(1X,'end of data set Year: ',I7)
 510  format(1X,'NOAA satellite name: ',A6)
 
      return
      end
