        subroutine m_calibcoeffile(lu,noanum, versnb,versyr,versday)
!+
!NOM/NAME $Id: m_calibcoeffile.F 47 2006-08-21 08:44:41Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module opens/closes and reads the calibration coefficients file. 
!     Data are ranged in common msu_clcf (include mcinit.h).
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Nov. 95      no              T. LABROT
! 1.1      Mar  04   Modify for Linux    N.Atkinson
!         
!ARGUMENTS 
!     input:
!      integer lu        !logical unit for calib coefficients file 
!      integer noanum    !spacecraft number (order)
!     output:
!      integer versnb    !number of the version of the calib coef file
!      integer versyr    !year of the version of the calib coef file
!      integer versday   !day of the year of the calib coef file 
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!     character*6 function  noascnam :  
!                  delivers the NOAA satellite name for a given order number
! 
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none  
                                                          
     
! includes/commons
#include "msuparam.h"
#include "mclin.h" 
#include "mcinit.h"
 
! argument variables 
      integer lu,noanum
      integer versnb,versyr,versday
       
! local variables
      character*80 text           !variable text
      character*80 charact        !buffer storing one line of the file
      integer ios                 !variable that receives the error status
      integer i,j                 !index for loops
      double precision tab0(3)    !array for the reading of the file
      
! functions
      character*6 noascnam

!--END OF DECLARATIONS---------------------------------------------------------

! Open and read calibration coefficients file
       open(lu,access='sequential',status='old',iostat=ios)
       if(ios.ne.0)
     & open(lu,err=900,access='sequential',status='unknown',iostat=ios)
       if(ios.eq.0) then
         text='The calib coef file has been opened'
         call ml_wt('info',text)
       endif

! Skip the header lines
       read(lu,'(a80)',err=901) charact
       do while (charact(1:1) .eq. '#')
        read(lu,'(a80)',err=901) charact
       enddo
       backspace lu

       read(lu,*,err=901)versnb
       read(lu,*,err=901)versyr
       read(lu,*,err=901)versday

       if (iprint.ge.1) then
        write(msu_cllgu,*)
        write(msu_cllgu,*) 'calibration coefficients file' 
        write(msu_cllgu,500)versnb,versyr,versday
       endif
       
       write(text,'(3X,"Version nb, year, day of year :",
     & I2,1X,I4,1X,I3)')versnb,versyr,versday
       call ml_wt('info',text)
              
! First & second radiation constants
       read(lu,'(a80)',err=901) charact
       do while (charact(4:9) .ne. 'First ')
        read(lu,'(a15)',err=901) charact
       enddo
       read(lu,*,err=901) plck
     
! To get the data of the good satellite
       read(lu,'(a80)',err=901) charact
       do while (charact(4:9) .ne. noascnam(noanum))
         read(lu,'(a15)',err=901) charact
       enddo
       
! Non-linearity correction coefficients
       read(lu,*,err=901)
       do i=1,msu_mxchn
        read(lu,*,err=901)tab0
        do j=1,3
          nonlin(i,j) = tab0(j)
        enddo
       enddo
       
! Weight coefficients for each PRT
       read(lu,*,err=901)
       read(lu,*,err=901)prtwght
       
! Central waves
       read(lu,*,err=901)
       do i=1,msu_mxchn
        read(lu,*,err=901) cwave(i)
       enddo
       
! Radiance of space
       read(lu,*,err=901)
       do i=1,msu_mxchn
        read(lu,*,err=901) sprad(i)
       enddo
! PRT count to resistance coefficients
       read(lu,*,err=901)
       read(lu,*,err=901) k0,k1

! PRT resistance to temperature coefficients
       read(lu,*,err=901)
       do i=1,msu_mxchn
        read(lu,*,err=901) tab0
        do j=1,3
          restmpcf(i,j) = tab0(j)
        enddo
       enddo
! High and low calibration reference points limits 
       read(lu,*,err=901)
       read(lu,*,err=901)
       read(lu,*,err=901) tcallomin,tcallomax
       read(lu,*,err=901) tcalhimin,tcalhimax

! To filter computed target temperature :
!      Reference temperature, two reference temperature differences
       read(lu,*,err=901)
       read(lu,*,err=901)
       read(lu,*,err=901) reftmp,refdiff0,refdiff1
! To filter the instrument output counts of the space :
!      one averaged output count and one standard deviation
       read(lu,*,err=901)
       read(lu,*,err=901)
       do i=1,msu_mxchn
         read(lu,*,err=901) spcavg(i),spcsigma(i)
       enddo

! To filter the instrument output counts of the target views :
!      one averaged output count and one standard deviation
       read(lu,*,err=901)
       read(lu,*,err=901)             
       do i=1,msu_mxchn
         read(lu,*,err=901) tgcavg(i),tgcsigma(i)
       enddo
      
       close(lu,err=902)       
 
       go to 200

 500  format(2X,' version number: ',I3/'   version year: ',I4/
     &       '   version day of year: ',I3)

 900  write(msu_cllgu,*)'Error opening calib coefficients file'
      text='Calib coef file could not be opened'
      call ml_wt('error',text)
      call aappexit(2)

 901  write(msu_cllgu,*)'Error reading calib coefficients file'
      text='Calib coefficients file could not be read'
      call ml_wt('error',text)
      call aappexit(2)

 902  write(msu_cllgu,*)'Error closing calib coefficients file'
      text='Calib coefficients file could not be closed' 
      call ml_wt('error',text)
      call aappexit(2)

 200  return
      end
