        subroutine m_cinit(satid,quality1,quality2)
!+
!NOM/NAME $Id: m_cinit.F 47 2006-08-21 08:44:41Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module opens/closes,reads the calibration coefficients files. 
!     Data are ranged in common msu_clcf (include cinit.h).
!     This module opens,reads the test coefficients and ranges data in 
!     common msu_tstcf (include cinit.h).
!     This module opens the MSU calib statistics file if requested.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Nov. 95      no              T. LABROT
! 1.1      Febr 97   Error in a "do" loop : do ilin=i,msu_mxlin
!                                           --> do ilin=1,msu_mxlin (TL)
! 1.2     25/05/98  modified to conform to NOAA release  (TL)
! 1.3     22/03/04  Update for Linux (N Atkinson / A Korpela)
!         
!ARGUMENTS 
!     input:
!     output:
!      integer   satid                           !order satellite number
!      integer   quality1(7,msu_mxlin)           !quality information array
!      integer   quality2(6,msu_mxchn,msu_mxlin) !quality information array
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
! 
!     integer function   ord1bid :
!                  delivers the order satellite number for a given 
!                  level 1b id number (PACS number) and a julian date
!     character*6 function  noascnam :  
!                  delivers the NOAA satellite name for a given order number
!     subroutine  m_calibcoeffile to read calib coef file
!     subroutine  m_testcoeffile  to read test  coef file
! 
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none 
                                                           
     
! includes/commons
#include "mclin.h"
#include "msuparam.h"
#include "msu1bhd.h"

! argument variables 
      integer satid,quality1(7,*),quality2(6,msu_mxchn,*)

! local variables
      character*80 text        !variable text
      integer ios              !variable that receives the error status
      integer i,ich,ilin       !index for loops
      integer msu_h_satid0     !satellite id from a 1B file
      integer versnb           !number of version of the coef file
      integer versyr           !year of the version of the coef file
      integer versday          !day of the version of the coef file (0/365)

! functions
      integer ord1bid 
      character*6 noascnam

!--END OF DECLARATIONS---------------------------------------------------------

      if (iprint.ge.1) write(msu_cllgu,*)
      if (iprint.ge.2) write(msu_cllgu,*) '***************************'
      if (iprint.ge.1) write(msu_cllgu,*) '**  SUBROUTINE M_CINIT   **'
      if (iprint.ge.2) write(msu_cllgu,*) '***************************'
      if (iprint.ge.1) write(msu_cllgu,*)

!Deliver the order satellite number for the l1b satellite number and
!the julian date of the data (The arguments must be integer)
      msu_h_satid0 = msu_h_satid

      satid = ord1bid(msu_h_satid0,msu_h_startdatajd)
  
      if (iprint.ge.1) then
       write(msu_cllgu,fmt=500) msu_h_satid0
       write(msu_cllgu,fmt=505) msu_h_startdatajd
      endif

      if (satid.eq.0) then

      text='Sat. id. not found or an error occured during the sat. id.' 
     & //  ' file reading'
       call ml_wt('error',text)
       if (iprint.ge.1) then
        write(msu_cllgu,*) 'sat. id. not found or an error',
     &  ' occured during the sat. id. file reading'
       endif
       call aappexit(9)

      else

       text='Satellite :'//noascnam(satid)
       call ml_wt('info',text)
       if (iprint.ge.1)  write(msu_cllgu,510) noascnam(satid)
     
! Open and read the calibration coefficients file
       call m_calibcoeffile(msu_coefu,satid, versnb,versyr,versday)

! Initialize words in the common hrs1bhd (header of the l1b)
! with the number of the version of the coefficients file.
       msu_h_cpidsyr = versyr
       msu_h_cpidsdy = versday

      endif

! Open and read test coefficients file
      call m_testcoeffile(msu_tstcoefu)      

! Open the statistics file (if requested)
      if(msu_stflg.eq.1) then

        open(msu_stu,form='formatted',access='sequential',
     &       status='new',iostat=ios)
        if(ios.ne.0)
     &  open(msu_stu,form='formatted',access='sequential',
     &       status='unknown',iostat=ios)

        if(ios.ne.0) then
         text='The statistics file could not be opened, ios:'
         call ml_wti('error',text,ios)
         if (iprint.ge.1) write(msu_cllgu,*)
     &   '  the statistics file could not be opened,  ios:',ios
         call aappexit(2)
        else
         text='The statistics file has been opened'
         call ml_wt('info',text)
         if (iprint.eq.1) write(msu_cllgu,*)
     &   '  the statistics file has been opened'
        endif
        
      endif    

! Initialize the array which gives informations about the quality of the
! calibration scan line by scan line

      do ilin=1,msu_mxlin
        do i=1,7
          quality1(i,ilin) = 0
        enddo
        do ich=1,msu_mxchn
          do i=1,6
            quality2(i,ich,ilin) = 0
          enddo 
        enddo
      enddo
   
 500  format(1X,'NOAA spacecraft identification code: ',I2)
 505  format(1X,'start of data set (Julian day): ',I7)
 510  format(1X,'NOAA satellite name: ',A6)

      return
      end
