        subroutine m_cntres(lin,prtidx,prtx,txcallox,txcalhix, res,flgx)
!+
!NOM/NAME $Id: m_cntres.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     Convert one PRT count to resistance
!     Test: Check txcallox,txcalhix 
!     Update the parameters of include mstat.h 
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!             
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Nov. 95       no             T. LABROT
!         
!ARGUMENTS 
!     input :
!       integer prtidx               !PRT index
!       integer prtx                 !PRT output count
!       integer txcallox,txcalhix    !low and high calib reference points 
!     output:
!       integer flgx                 !flag
!       dble prec. res               !resistance
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!     sub. m_statsum: accumulate the values for final statistics.
!    
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------


!--DECLARATIONS----------------------------------------------------------------

      implicit none 
                                                           
     
!includes/commons
#include "msuparam.h"
#include "mclin.h"
#include "mcinit.h"
#include "missingval.h"
#include "mstat.h"

! argument variables
      integer lin,flgx,prtidx,prtx,txcallox,txcalhix
      double precision res
      
! local variables
      character*80 text             

!--END OF DECLARATIONS----------------------------------------------------------

      if (iprint.ge.2) write(msu_cllgu,*) '  *subroutine M_CNTRES*'
 
      res = missdp
      flgx= 0
      
! Initialize common 'prtlineinfo' in include mstat.h
      prtid=prtidx
      prt=prtx
      txcallo=txcallox
      txcalhi=txcalhix
      infloflg=0
      suploflg=0
      infhiflg=0
      suphiflg=0
      zeroflg=0
      misprtflg=0
      flg=0
      
! Check txcallo,txcalhi with limits
! The limits don't depend on the PRT number
 
      if (txcallox.lt.tcallomin) then
        infloflg=1
        flgx=1
        write(text,'("line",I3," PRT",I2,": t.callo=",I6,
     &  "  inf to limit=",I6)') lin,prtidx,txcallox,tcallomin 
        call ml_wt('warning',text)
        if(iprint.ge.1)write(msu_cllgu,fmt=500)txcallox,tcallomin
      endif
      
      if (txcallox.gt.tcallomax) then  
        suploflg=1 
        flgx=1 
        write(text,'("line",I3," PRT",I2,": t.callo=",I6,
     &  "  sup to limit=",I6)') lin,prtidx,txcallox,tcallomax 
        call ml_wt('warning',text)        
        if(iprint.ge.1)write(msu_cllgu,fmt=510)txcallox,tcallomax
      endif
           
      if (txcalhix.lt.tcalhimin) then
        infhiflg=1
        flgx=1
        write(text,'("line",I3," PRT",I2,": t.calhi=",I6,
     &  "  inf to limit=",I6)') lin,prtidx,txcalhix,tcalhimin 
        call ml_wt('warning',text)   
        if(iprint.ge.1)write(msu_cllgu,fmt=520)
     &                 txcalhix,tcalhimin
      endif
      
      if (txcalhix.gt.tcalhimax) then 
        suphiflg=1
        flgx=1  
        write(text,'("line",I3," PRT",I2,": t.calhi=",I6,
     &  "  sup to limit=",I6)') lin,prtidx,txcalhix,tcalhimax 
        call ml_wt('warning',text)           
        if(iprint.ge.1)write(msu_cllgu,fmt=530)
     &                 txcalhix,tcalhimax
      endif
      
      if (flgx.eq.0.and.(txcalhix-txcallox).le.0.) then
        zeroflg=1
        flgx=1
        write(text,'("line",I3," PRT",I2,": t.calhi <= t.calho")')
     &  lin,prtidx,txcalhix,txcallox
        call ml_wt('warning',text)          
        if(iprint.ge.1)write(msu_cllgu,fmt=540)txcalhix,txcallox
      endif
      
      if (flgx.eq.0) then
          
        if (prtx.lt.missdp) then
! Every thing is ok. Compute resistance
          res = dfloat(prtx-txcallox)/dfloat(txcalhix-txcallox)
          res = k0 + k1*res
        else
          misprtflg=1
          flgx=1
          write(text,'("      missing data for PRT",I2)') prtidx
          call ml_wt('warning',text)                
          write(msu_cllgu,fmt=550)prtidx      
        endif        

      endif
      
! finish to update include "mstat.h"
      flg=flgx
      
! for final statistics,accumulate the values of data 
      call m_statsum 
      
! In common 'prtlineinfo' in include mstat.h
      prtid=0
      prt=0
      txcallo=0
      txcalhi=0
      infloflg=0
      suploflg=0
      infhiflg=0
      suphiflg=0
      zeroflg=0
      misprtflg=0  
 
 500   format(8X,'t.callo=',I6,' inf to limit ',I6)
 510   format(8X,'t.callo=',I6,' sup to limit ',I6) 
 520   format(8X,'t.calhi=',I6,' inf to limit ',I6)
 530   format(8X,'t.calhi=',I6,' sup to limit ',I6) 
 540   format(8X,'(t.calhi-t.callo)<= 0, t.calhi=',I6,
     &        't.callo=',I6)
 550   format(8X,'missing PRT data --> no temperature for PRT',I1)

       return
       end

