      subroutine m_finalstat
!+
!NOM/NAME $Id: m_finalstat.F 47 2006-08-21 08:44:41Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!     This module computes final statistics of the MSU calibration.
!     It writes statistic record on logical unit msu_stu 
!     It closes the file 
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER 
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Dec. 95      no              T. LABROT
! 1.1     April 96   Add "typorb" to stat file (TL)
! 1.2     April 96   Replace output msu_h_startdatajd by double prec.instant(TL)
!         
!ARGUMENTS
!     input:
!
!     output:
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------
!--DECLARATIONS----------------------------------------------------------------

      implicit none                                                     


!commons
#include "mclin.h"
#include "msuparam.h"
#include "msu1bhd.h"
#include "mstat.h"
#include "missingval.h"

! argument variables 

! local variables
      integer ios
      integer ich
      character*5 orbnum
      character*80 text             !variable text      
      integer iq(3)                 !gregorian start instant
      double precision ij           !julian start instant for output      
      character*1 typorb            !ascending 'a' descending 'd'

! functions

!--END OF DECLARATIONS---------------------------------------------------------

      if (iprint.ge.1) write(msu_cllgu,*)
      if (iprint.ge.2) write(msu_cllgu,*) '***************************'
      if (iprint.ge.1) write(msu_cllgu,*) '** SUBROUTINE M_FINALSTAT **'
      if (iprint.ge.2) write(msu_cllgu,*) '***************************'

!-----------------------------------------------------------------------
! For high and low calibration reference points for electronic system A 
!-----------------------------------------------------------------------
      if(gtalonb.gt.0) then
        talomn=talosum/gtalonb
        talodev=(talodevsum/gtalonb)-(talomn**2)
        if(talodev.ge.0.d+0) talodev=dsqrt(talodev)
      endif
      if(gtahinb.gt.0) then
        tahimn=tahisum/gtahinb
        tahidev=(tahidevsum/gtahinb)-(tahimn**2)
        if(tahidev.ge.0.d+0) tahidev=dsqrt(tahidev)
      endif
      talonb=gtalonb+inftalonb+suptalonb+talohinb
      gtalopct=100.*gtalonb/talonb
      inftalopct=100.*inftalonb/talonb
      suptalopct=100.*suptalonb/talonb
      tahinb=gtahinb+inftahinb+suptahinb+talohinb
      gtahipct=100.*gtahinb/tahinb
      inftahipct=100.*inftahinb/tahinb
      suptahipct=100.*suptahinb/tahinb
      talohipct=100.*2.*talohinb/(talonb+tahinb) 
!-----------------------------------------------------------------------
! For high and low calibration reference points for electronic system B 
!-----------------------------------------------------------------------
      if(gtblonb.gt.0) then
        tblomn=tblosum/gtblonb
        tblodev=(tblodevsum/gtblonb)-(tblomn**2)
        if(tblodev.ge.0.d+0) tblodev=dsqrt(tblodev)
      endif
      if(gtbhinb.gt.0) then
        tbhimn=tbhisum/gtbhinb
        tbhidev=(tbhidevsum/gtbhinb)-(tbhimn**2)
        if(tbhidev.ge.0.d+0) tbhidev=dsqrt(tbhidev)
      endif
      tblonb=gtblonb+inftblonb+suptblonb+tblohinb
      gtblopct=100.*gtblonb/tblonb
      inftblopct=100.*inftblonb/tblonb
      suptblopct=100.*suptblonb/tblonb
      tbhinb=gtbhinb+inftbhinb+suptbhinb+tblohinb 
      gtbhipct=100.*gtbhinb/tbhinb
      inftbhipct=100.*inftbhinb/tbhinb
      suptbhipct=100.*suptbhinb/tbhinb 
      tblohipct=100.*2.*tblohinb/(tblonb+tbhinb)    
!----------------
! For PRT1A data
!----------------
      if(gprt1anb.gt.0)then
        prt1amn=prt1asum/gprt1anb
        prt1adev=(prt1adevsum/gprt1anb)-(prt1amn**2)
        if(prt1adev.ge.0.) prt1adev=dsqrt(prt1adev)
      endif
      prt1anb=misprt1anb+gprt1anb
      gprt1apct=100.*gprt1anb/prt1anb
      misprt1apct=100.*misprt1anb/prt1anb 
!----------------
! For PRT1B data
!----------------
      if(gprt1bnb.gt.0)then
        prt1bmn=prt1bsum/gprt1bnb
        prt1bdev=(prt1bdevsum/gprt1bnb)-(prt1bmn**2)
        if(prt1bdev.ge.0.) prt1bdev=dsqrt(prt1bdev)
      endif
      prt1bnb=misprt1bnb+gprt1bnb
      gprt1bpct=100.*gprt1bnb/prt1bnb
      misprt1bpct=100.*misprt1bnb/prt1bnb      
!----------------
! For PRT2A data
!----------------
      if(gprt2anb.gt.0)then
        prt2amn=prt2asum/gprt2anb
        prt2adev=(prt2adevsum/gprt2anb)-(prt2amn**2)
        if(prt2adev.ge.0.) prt2adev=dsqrt(prt2adev)
      endif
      prt2anb=misprt2anb+gprt2anb
      gprt2apct=100.*gprt2anb/prt2anb
      misprt2apct=100.*misprt2anb/prt2anb 
!----------------
! For PRT2B data
!----------------
      if(gprt2bnb.gt.0)then
        prt2bmn=prt2bsum/gprt2bnb
        prt2bdev=(prt2bdevsum/gprt2bnb)-(prt2bmn**2)
        if(prt2bdev.ge.0.) prt2bdev=dsqrt(prt2bdev)
      endif
      prt2bnb=misprt2bnb+gprt2bnb
      gprt2bpct=100.*gprt2bnb/prt2bnb
      misprt2bpct=100.*misprt2bnb/prt2bnb 
!-----------------------
!  space view counts
!-----------------------
      do ich=1,msu_mxchn
        if(gspnb(ich).gt.0) then
           spcntmn(ich)=spcntsum(ich)/gspnb(ich)
           spcntdev(ich)=(spcntdevsum(ich)/gspnb(ich))
     &                   -(spcntmn(ich)**2)
           if(spcntdev(ich).ge.0.)spcntdev(ich)=dsqrt(spcntdev(ich))
        endif

        spnb(ich)=misspnb(ich)+ejectspnb(ich)+gspnb(ich)
        missppct(ich)=100.*float(misspnb(ich))/float(spnb(ich))
        ejectsppct(ich)=100.*float(ejectspnb(ich))/float(spnb(ich))
        gsppct(ich)=100.*float(gspnb(ich))/float(spnb(ich))
      enddo
!-----------------------------------
!  IWT radiant signal output counts
!-----------------------------------
      do ich=1,msu_mxchn
        if(gtgnb(ich).gt.0) then
           tgcntmn(ich)=tgcntsum(ich)/gtgnb(ich)
           tgcntdev(ich)=(tgcntdevsum(ich)/gtgnb(ich))
     &                   -(tgcntmn(ich)**2)
           if(tgcntdev(ich).ge.0.)tgcntdev(ich)=dsqrt(tgcntdev(ich))
         endif
     
        tgnb(ich)=mistgnb(ich)+ejecttgnb(ich)+gtgnb(ich)
        mistgpct(ich)=100.*float(mistgnb(ich))/float(tgnb(ich))
        ejecttgpct(ich)=100.*float(ejecttgnb(ich))/float(tgnb(ich))
        gtgpct(ich)=100.*float(gtgnb(ich))/float(tgnb(ich))
      enddo 
!-----------------------------------------
! other parameters for the statistics file
!-----------------------------------------
! convert year day and time to julian instant for output
       iq(1)=msu_h_startdatayr
       iq(2)=msu_h_startdatady
       iq(3)=msu_h_startdatatime
       call xiqj(iq,ij)
       
       orbnum = msu_h_prblkid(2:6)
       
       if (msu_h_zvel.ge.0.d+0) then
         typorb='a'
       else
         typorb='d'
       endif   
            
!--------------------------------------
! output information on statistics file
!--------------------------------------       
       write(msu_stu,fmt=500,iostat=ios) 
     & orbnum,typorb,ij,
     & talomn,talodev,talonb,gtalopct,inftalopct,suptalopct,
     & tahimn,tahidev,tahinb,gtahipct,inftahipct,suptahipct,
     & talohipct,
     & tblomn,tblodev,tblonb,gtblopct,inftblopct,suptblopct,
     & tbhimn,tbhidev,tbhinb,gtbhipct,inftbhipct,suptbhipct,
     & tblohipct,
     & prt1amn,prt1adev,prt1anb,gprt1apct,misprt1apct,
     & prt1bmn,prt1bdev,prt1bnb,gprt1bpct,misprt1bpct,
     & prt2amn,prt2adev,prt2anb,gprt2apct,misprt2apct,
     & prt2bmn,prt2bdev,prt2bnb,gprt2bpct,misprt2bpct,
     & spcntmn(1),spcntdev(1),spnb(1),gsppct(1),
     & missppct(1),ejectsppct(1),
     & spcntmn(2),spcntdev(2),spnb(2),gsppct(2),
     & missppct(2),ejectsppct(2), 
     & spcntmn(3),spcntdev(3),spnb(3),gsppct(3),
     & missppct(3),ejectsppct(3),      
     & spcntmn(4),spcntdev(4),spnb(4),gsppct(4),
     & missppct(4),ejectsppct(4),       
     & tgcntmn(1),tgcntdev(1),tgnb(1),gtgpct(1),
     & mistgpct(1),ejecttgpct(1),
     & tgcntmn(2),tgcntdev(2),tgnb(2),gtgpct(2),
     & mistgpct(2),ejecttgpct(2), 
     & tgcntmn(3),tgcntdev(3),tgnb(3),gtgpct(3),
     & mistgpct(3),ejecttgpct(3),      
     & tgcntmn(4),tgcntdev(4),tgnb(4),gtgpct(4),
     & mistgpct(4),ejecttgpct(4) 
     
                  
  500  format(a5,1x,a1,f11.4,
     & f10.4,f9.6,i3,3f7.2,
     & f10.4,f9.6,i3,3f7.2,
     & f7.2,
     & f10.4,f9.6,i3,3f7.2,
     & f10.4,f9.6,i3,3f7.2,
     & f7.2,
     & f10.4,f9.6,i3,2f7.2,
     & f10.4,f9.6,i3,2f7.2,
     & f10.4,f9.6,i3,2f7.2,
     & f10.4,f9.6,i3,2f7.2,
     & f10.4,f9.6,i3,3f7.2,
     & f10.4,f9.6,i3,3f7.2,
     & f10.4,f9.6,i3,3f7.2,
     & f10.4,f9.6,i3,3f7.2,
     & f10.4,f9.6,i3,3f7.2,
     & f10.4,f9.6,i3,3f7.2,
     & f10.4,f9.6,i3,3f7.2,
     & f10.4,f9.6,i3,3f7.2)



       if(ios.ne.0) then
         call ml_wti('warning',
     &   'problem when writing statistics',ios)
       endif

!--------------------------------------
! close the statistics file
!--------------------------------------
       close(msu_stu,err=900)
       if(iprint.ge.1) write(msu_cllgu,*)
     &  '  the statictics file has been closed'
       text='The statictics file has been closed'
       call ml_wt('info',text)

       return
 900  write(msu_cllgu,*)'Error closing statistics file'
      text='Error closing statistics file'
      call ml_wt('error',text)
      call aappexit(2)

      end 
