        subroutine m_gfcounts(spcount,tgcount,quality2)
!+
!NOM/NAME $Id: m_gfcounts.F 192 2010-01-27 15:06:55Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     For each scan line, this module gets one space view output count and 
!     one target output count. It applies the non_linearity coefficients and
!     the filter ( abs(count - averaged count) <or= 2*standard deviation ).
!     It accumulates the different values for statistics.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!             
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Nov. 95       no             T. LABROT
!         
!ARGUMENTS 
!   input :
!   output:
!     dble prec. spcount(msu_mxchn,msu_mxlin) !output count values of the space
!     dble prec. tgcount(msu_mxchn,msu_mxlin) !output count values of the target views
!     integer quality2(6,msu_mxchn,msu_mxlin)
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!    
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------


!--DECLARATIONS----------------------------------------------------------------

      implicit none   
                                                         
     
!includes/commons
#include "msuparam.h"
#include "mclin.h"
#include "mcinit.h"
#include "msu1bhd.h"
#include "msu1bdts.h"
#include "missingval.h"
#include "mstat.h"

! argument variables
      double precision spcount(msu_mxchn,*),tgcount(msu_mxchn,*)
      integer quality2(6,msu_mxchn,*)
      
! local variables
      character*80 text             !variable text
      integer lin,ich               !index of loop
      integer ind0                  !=91 for space, =99 for target
      integer outcount              !output count value
      double precision countbis     !count value after applying the 
!                                    non-linearity correction
      double precision cavg,csigma  !averaged output count and standard 
!                                    deviation to filter the counts

!--END OF DECLARATIONS---------------------------------------------------------

      text='Get and filter Space/IWT radiant signal outpout counts '
      call ml_wt('info',text)

      if (iprint.ge.1) write(msu_cllgu,*)
      if (iprint.ge.2) write(msu_cllgu,*) '***************************'
      if (iprint.ge.1) write(msu_cllgu,*) '** SUBROUTINE M_GFCOUNTS **'
      if (iprint.ge.2) write(msu_cllgu,*) '***************************'
      
      do lin=1,msu_mxlin
        do ich=1,msu_mxchn
          spcount(ich,lin)=missdp
          tgcount(ich,lin)=missdp
        enddo
      enddo
      
      do lin=1,msu_h_scnlin      

       do ich=1,msu_mxchn
! initialize or reinitialize line parameters

        do ind0=91,99,8
! ind=91 for the space data
! ind=99 for the target data
         if(iprint.ge.1.and.ind0.eq.91)write(msu_cllgu,fmt=500)lin,ich 
         if(iprint.ge.1.and.ind0.eq.99)write(msu_cllgu,fmt=510)lin,ich
                    
          ind=ind0
          ch=ich
          flag=0
          miscntflg=0
          ejectcntflg=0
     
          outcount=msu_data(ind+ich,lin)
          if(iprint.ge.2)write(msu_cllgu,fmt=520)outcount
          
          if(outcount.ge.missint2) then
          
            flag=1
            miscntflg=1
            
            if(iprint.ge.1) write(msu_cllgu,fmt=530) 
            if(ind.eq.91) then
              write(text,'("line ",I2,": missing space count for"
     &        ," channel ",I2)') lin,ich
              call ml_wt('notice',text)
            endif
            if(ind.eq.99) then
              write(text,'("line ",I2,": missing target count for"
     &        ," channel ",I2)') lin,ich
              call ml_wt('notice',text) 
            endif
            write(text,'(" ==> line ",I2," not used to compute",
     &            " slope and intercept")')lin
            call ml_wt('warning',text)
            
          else
          
! apply non-linearity coefficients
            countbis=nonlin(ich,1)+nonlin(ich,2)*outcount
     &               +nonlin(ich,3)*outcount*outcount    

                    
! apply the filter
            if(ind.eq.91) cavg=spcavg(ich)
            if(ind.eq.91) csigma=spcsigma(ich)
            if(ind.eq.99) cavg=tgcavg(ich)
            if(ind.eq.99) csigma=tgcsigma(ich)
            
            if(iprint.ge.2) then
              write(msu_cllgu,fmt=540) countbis 
              write(msu_cllgu,fmt=550) cavg
              write(msu_cllgu,fmt=560) csigma
            endif
            
            if(abs(countbis-cavg).le.2.*csigma) then
              if(ind.eq.91) spcount(ich,lin)=countbis
              if(ind.eq.99) tgcount(ich,lin)=countbis
            else
              ejectcntflg=1
             
             if(iprint.ge.1) write(msu_cllgu,fmt=570) 
             if(ind.eq.91) 
     &         write(text,'("line ",I2," ch ",I1,": Space countbis",
     &         " differs from mean by more than 2 sigma")')lin,ich
              
             if(ind.eq.99) 
     &         write(text,'("line ",I2," ch ",I1,": Target countbis",
     &         " differs from mean by more than 2 sigma")')lin,ich 
     
              call ml_wt('warning',text)
              write(text,'(4X,"countbis:",F11.5,4X,"mean:",F8.2,4X,
     &        "sigma:",F7.2)')countbis,cavg,csigma
              call ml_wt('warning',text)
                 
            endif
          endif
          
! accumulate the different values for statistics
          call m_statsum
          
        enddo
      
! Update quality2(1,ich,lin)= 'no good blackbody count for scan line'
!                           = 1 if true , =0 if false
! Update quality2(2,ich,lin)= 'no good space view count for scan line'
!                           = 1 if true , =0 if false 
! Update quality2(4,ich,lin)= 'some bad blackbody counts for this line'
! Update quality2(5,ich,lin)= 'some bad space view counts for this line'
! quality2(4,ich,lin)= quality2(1,ich,lin) and
! quality2(5,ich,lin)= quality2(2,ich,lin) because there is only one
! space/blackbody count by channel and by line. 
                     
         if(tgcount(ich,lin).ge.missdp) quality2(1,ich,lin)=1
         if(spcount(ich,lin).ge.missdp) quality2(2,ich,lin)=1
         quality2(4,ich,lin)= quality2(1,ich,lin)
         quality2(5,ich,lin)= quality2(2,ich,lin)     
        enddo
      enddo

      
!      do lin=1,msu_h_scnlin
!     do ich=1,msu_mxchn
!        write(*,*) 'lin ich spcount(ich,lin)  ',lin,ich,spcount(ich,lin)
!        write(*,*) 'lin ich tgcount(ich,lin)  ',lin,ich,tgcount(ich,lin) 
!      enddo
!      enddo
          
 500  format(/4X,'SCAN LINE:',I2,'  CHANNEL:',I2,'  SPACE')     
 510  format(/4X,'SCAN LINE:',I2,'  CHANNEL:',I2,'  TARGET')
 520  format(6X,'output count   :',I9) 
 530  format(6X,'count is missing')
 540  format(6X,'countbis:',F16.6)
 550  format(6X,'cavg    :',F16.4)
 560  format(6X,'csigma  :',F16.4)
 570  format(6X,'countbis differs from mean by more than two sigma')
 
      return
      end
 
      
