      subroutine m_interslop(spcount,tgcount,tgrad,quality1, 
     &slope,intcept)
!+
!NOM/NAME $Id: m_interslop.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module computes the calibration coefficients: slope,intercept.
!     For one channel, the final slope is the mean of slopes computed line
!     by line.
!     Here the mean is a "true" mean, different from the mean computed
!     in ITPP4 and ITPP5.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!             
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Nov. 95       no             T. LABROT
!         
!ARGUMENTS 
!  input :
!    dble prec. spcount(msu_mxchn,msu_mxlin) !output count values of the space
!    dble prec. tgcount(msu_mxchn,msu_mxlin) !output count values of the target views
!    dble prec. tgrad(msu_mxchn)             !radiances array 
!
!  output:
!    dble prec. slope(msu_mxchn)     !slopes array
!    dble prec. intcept(msu_mxchn)   !intercepts array
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!    
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------


!--DECLARATIONS----------------------------------------------------------------

      implicit none 
                                                           
     
!includes/commons
#include "msuparam.h"
#include "mclin.h"
#include "mcinit.h"
#include "msu1bhd.h"
#include "missingval.h"

! argument variables
      double precision spcount(msu_mxchn,*)
      double precision tgcount(msu_mxchn,*)
      double precision tgrad(msu_mxchn,*)
      double precision slope(msu_mxchn)
      double precision intcept(msu_mxchn)
      integer quality1(7,*)
            
! local variables
      character*80 text             !variable text
      integer lin,ich              !index of loop
      integer nb                    !counter of valid slopes or intercepts
      integer badch                 !number of uncalibrated channel
      double precision sl,int       !slope and intercept computed with data
                                    !of one scan line
      double precision slsum,intsum !accumulate the values of slopes and
                                    !intercepts
      
!--END OF DECLARATIONS----------------------------------------------------------

      text='Compute the 2 calibration coefficients'
      call ml_wt('info',text)

      if (iprint.ge.1) write(msu_cllgu,*)
      if (iprint.ge.2) write(msu_cllgu,*) '***************************'
      if (iprint.ge.1) write(msu_cllgu,*) '*  SUBROUTINE M_INTERSLOP *'
      if (iprint.ge.2) write(msu_cllgu,*) '***************************'
      
      do ich=1,msu_mxchn
        slope(ich)=missdp
        intcept(ich)=missdp
      enddo 
           
      do ich=1,msu_mxchn

        if (iprint.ge.1) write(msu_cllgu,'("CHANNEL  ",I2)') ich
      
        nb=0
        slsum=0.d+0
        intsum=0.d+0
      
        do lin=1,msu_h_scnlin
        
          sl=missdp
          int=missdp
        
!         write(*,*) 'lin spcount(ich,lin) ',lin,spcount(ich,lin)
!         write(*,*) 'lin tgcount(ich,lin) ',lin,tgcount(ich,lin)
!         write(*,*) 'lin tgrad(ich,lin)   ',lin,tgrad(ich,lin)
         
         if(spcount(ich,lin).lt.missdp.and.tgcount(ich,lin).lt.missdp
     &      .and.tgrad(ich,lin).lt.missdp ) then

           if (spcount(ich,lin).ne.tgcount(ich,lin)) then

             sl = (sprad(ich) - tgrad(ich,lin))
     &                        /(spcount(ich,lin) - tgcount(ich,lin))
             int = sprad(ich) - sl*spcount(ich,lin)
             
             slsum  = slsum  + sl
             intsum = intsum + int
             nb = nb + 1
             
           endif

         endif
         
           if(iprint.ge.2) write(msu_cllgu,fmt=500)lin,sl,int 
                
        enddo
         
        if (nb.ge.minnb) then
          slope(ich)   = slsum/nb
          intcept(ich) = intsum/nb
                   
          write(text,'("channel ",I1,": Slope ",D14.7,
     &          "   Intercept ",D14.7)')ich,slope(ich),intcept(ich)
          call ml_wt('info',text)
          write(text,'(11X,"nb of good lines ",I2,3X,"min nb ",I2)')
     &    nb,minnb
          call ml_wt('info',text)
          
        else
        
          slope(ich)   = missdp
          intcept(ich) = missdp
          
          if(iprint.ge.1) write(msu_cllgu,fmt=510)
          if(iprint.ge.1) write(msu_cllgu,fmt=520)
          write(text,'("channel ",I1,": No enough lines to compute",
     &    " mean slope and intercept")')ich
          call ml_wt('warning',text)
          write(text,'(11X,"nb of good lines: ",I2,3X,
     &    "minimum nb: ",I2)')nb,minnb
          call ml_wt('warning',text)
        endif
      enddo
       
! Update quality(1,lin)= 'no calibration"
!                      = 1 if no calibration for all the channels 
!                      = 0 if one (or more channel) has been calibrated
! Update quality(6,lin)= 'some uncalibrated channels on this scan'
!                      = 1 if true
!                      = 0 if all channels has been calibrated
      badch=0
      do ich=1,msu_mxchn
        if(slope(ich).ge.missdp.or.intcept(ich).ge.missdp) badch=badch+1
      enddo
      if(badch.eq.msu_mxchn) then
        do lin=1,msu_h_scnlin
          quality1(1,lin) = 1
        enddo
      endif
      if(badch.ge.1) then
        do lin=1,msu_h_scnlin
          quality1(6,lin) = 1
        enddo
      endif       
       
       
 500   format(3X,'scan line',I3,'   slope: ',D14.7,
     &        '   intercept: ',D14.7)
 510   format(3X,'no enough lines to compute mean slope and intercept')
 520   format(3X,' ==> NO CALIBRATION')
 
       return
       end




