        subroutine m_loc
!+
!NOM/NAME $Id: m_loc.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module performs eath location for MSU.
!     This module writes in the level1b file, header and records.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Dec. 95      no              T. LABROT
! 1.1      25/05/98  modified to conform to NOAA release  (TL)
! 1.2      Dec 00        change message for clock error (PB)
!         
!ARGUMENTS 
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none                                                     

 
! includes/commons
#include "msuparam.h"
#include "msu1bhd.h"
#include "msu1bdt.h"
#include "msu1bdts.h"
#include "mclin.h"
#include "c_1bnav.h"

! local variables
       integer scn
       integer iqscn(3)
       integer jdate
       double precision ijscn
       double precision ijstrt
       logical clock_cor_ind

!functions
       integer*2 gp_gb16
!-
       iqscn(1)=msu_scnlinyr(1)
       iqscn(2)=msu_scnlindy(1)
       iqscn(3)=msu_scnlintime(1)
       call xiqj(iqscn,ijstrt)
!
!      check if clock error not already applied 
!      if not we must correct
!      the current scan line times when computing earth location
!      ---------------------------------------------------------

       if(gp_gb16(msu_scnlinbit(1),14).eq.0) then
         ijstrt=ijstrt-errclock/86400.
         clock_cor_ind=.true.
         call ml_wt('info','clock error correction is valid')
         call ml_wt('info','datation will be corrected in this run')
       else
         clock_cor_ind=.false.
         call ml_wt('info','times are already corrected')
       endif


        do scn = 1,msu_h_scnlin

           if(clock_cor_ind) then
!          -------------------------------
!          clock error correction to apply
!          and flag bit 14 in scnlinbit
!          -------------------------------

             iqscn(1)=msu_scnlinyr(scn)
             iqscn(2)=msu_scnlindy(scn)
             iqscn(3)=msu_scnlintime(scn)
             call xiqj(iqscn,ijscn)
             ijscn=ijscn-errclock/86400.
             call xijq(ijscn,iqscn)
             msu_scnlinyr(scn) = iqscn(1)
             msu_scnlindy(scn) = iqscn(2)
             msu_scnlintime(scn) = iqscn(3)
             call gp_sb16(msu_scnlinbit(scn),14,1)
             msu_clockdrift(scn)=1000.*errclock
             
          endif
          call nav_1blin(ijstrt, 6, scn,
     &     msu_scnlinbit, msu_qualind,
     &     msu_attangtime, msu_rollang , msu_pitchang ,
     &     msu_yawang , 
     &     msu_navstat , 
     &     msu_scalti ,msu_ang(1,1,scn) , msu_pos(1,1,scn) )
        enddo
        
!      --------------------------------------------------
!      update header times according to the clock_cor_ind
!      --------------------------------------------------

       if(clock_cor_ind) then
         iqscn(1)=msu_scnlinyr(1)
         iqscn(2)=msu_scnlindy(1)
         call xdqj(iqscn,jdate)
         msu_h_startdatajd = jdate 
         msu_h_startdatayr = msu_scnlinyr(1)
         msu_h_startdatady = msu_scnlindy(1)
         msu_h_startdatatime = msu_scnlintime(1)
         iqscn(1)=msu_scnlinyr(msu_h_scnlin)
         iqscn(2)=msu_scnlindy(msu_h_scnlin)
         call xdqj(iqscn,jdate)
         msu_h_enddatajd = jdate 
         msu_h_enddatayr = msu_scnlinyr(msu_h_scnlin)
         msu_h_enddatady = msu_scnlindy(msu_h_scnlin)
         msu_h_enddatatime = msu_scnlintime(msu_h_scnlin)
       endif

       return
       end
