        subroutine m_restmp(ilin,iprtnb,res,tmp)
!+
!NOM/NAME $Id: m_restmp.F 25 2006-02-03 15:43:53Z frna $
!    
!------------------------------------------------------------------------------
!BUT/PURPOSE
!     Convert resistance to temperature. 
!     Test:     
!       Check the computed temperature with a reference temperature.
!       If the difference between those two temperatures is bigger than
!       a max difference, the temperature used later on will be the 
!       reference temperature.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!             
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Nov. 95       no             T. LABROT
! 1.1      July 96    Change "error" in "warning" for "ref. target temp ..."(TL)
!         
!ARGUMENTS 
!     input :
!      integer ilin    !number of the scan line
!      integer iprtnb  !=1 for PRT1A, =2 for PRT1B, =3 for PRT2A, =4 for PRT2B
!      dble prec. res  !resistance computed from one PRT data
!
!     output:
!      dble prec. tmp  !temperature computed from one PRT data
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!    
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------


!--DECLARATIONS----------------------------------------------------------------

      implicit none 
                                                           
     
!includes/commons
#include "msuparam.h"
#include "mclin.h"
#include "mcinit.h"
#include "missingval.h"

! argument variables
      integer ilin,iprtnb
      double precision res,tmp 
      
! local variables
      character*80 text             !variable text
      double precision c0,c1,c2     !PRT resistance to temperature coefficients
      double precision diff         !difference between computed temp. and 
!                                    reference temp. 

!--END OF DECLARATIONS----------------------------------------------------------

      if(iprint.ge.2) write(msu_cllgu,*) '  *subroutine M_RESTMP*'
      if(iprint.ge.2) write(msu_cllgu,fmt=500)reftmp
      if(iprint.ge.2) write(msu_cllgu,fmt=510)refdiff0,refdiff1
      

      c0 = restmpcf(iprtnb,1)
      c1 = restmpcf(iprtnb,2)
      c2 = restmpcf(iprtnb,3)
      
      tmp = c0 + c1*res + c2*res*res
     
! Check tmp 
      diff = abs(tmp-reftmp)
      
      if (iprint.ge.2) write(msu_cllgu,fmt=520)
     &                 tmp,diff
     
      if (diff.gt.refdiff0) then
        write(text,'("line ",I2," PRT",I2," temp. ",
     &               F8.3,"  too big diff/reftemp",F8.3,
     &               "/",F6.2)')
     &               ilin,iprtnb,tmp,diff,reftmp
        call ml_wt('warning',text)
      endif
      
      if (diff.gt.refdiff1) then
        if (iprint.ge.1) write(msu_cllgu,fmt=530)
        write(text,'("    reference target temp. has been ",
     &  "retained : ",F8.2)') reftmp
        call ml_wt('warning',text)
        tmp=reftmp
      endif
 
 500  format(6X,'tempe. of reference: ',F6.2) 
 510  format(6X,'difference of reference: ',F6.2,3X,F6.2)        
 520  format(6X,'tgtemp:',F10.4,'  diff:',F6.2)
 530  format(8X,'--> reference temp. will be retain for target temp.')

      return
      end
