        subroutine m_testcoeffile(lu)
!+
!NOM/NAME $Id: m_testcoeffile.F 47 2006-08-21 08:44:41Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module opens/closes and reads the test coefficients file and  
!     ranges data in common msu_tstcf (include mcinit.h).
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Nov. 95       no             T. LABROT
! 1.1      Mar 04   Modify for Linux    N.Atkinson
!         
!ARGUMENTS 
!     input:
!      integer lu        !logical unit for test coefficients file
!     output: 
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
! 
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none 
                                                           
     
! includes/commons
#include "msuparam.h"
#include "mclin.h"
#include "mcinit.h"

! argument variables 
      integer lu

! local variables
      character*80 text           !variable text
      character*80 charact        !buffer storing one line of the file
      integer ios                 !variable that receives the error status
      integer versnb              !number of version of the coef file
      integer versyr              !year of the version of the coef file
      integer versday             !day of the version of the coef file (0/365)

! functions

!--END OF DECLARATIONS---------------------------------------------------------

! Open and read test coefficients file
      open(lu,access='sequential',status='old',iostat=ios)
      if(ios.ne.0)
     &  open(lu,err=900,access='sequential',status='unknown',iostat=ios)
      if(ios.eq.0) then
        text='The test coef file has been opened'
        call ml_wt('info',text)
      endif

! Skips the header lines
      read(lu,'(a80)',err=901) charact
      do while (charact(1:1) .eq. '#')
        read(lu,'(a80)',err=901) charact
      enddo
      backspace lu

      read(lu,*,err=901)versnb
      read(lu,*,err=901)versyr
      read(lu,*,err=901)versday
 
      if (iprint.ge.1) then
        write(msu_cllgu,*) 'test coefficients file' 
        write(msu_cllgu,500)versnb,versyr,versday
      endif
       
      write(text,'(3X,"Version nb, year, day of year :",
     &     I2,1X,I4,1X,I3)')versnb,versyr,versday
      call ml_wt('info',text)


      read(lu,'(a80)',err=901) charact
      do while (charact(1:1) .eq. '#')
        read(lu,'(a80)',err=901) charact
      enddo
      backspace lu

      read(lu,*,err=901) minnb

      if (iprint.ge.2) write(msu_cllgu,fmt=510) minnb

      close(lu,err=902)

      go to 200

 500  format(2X,' version number: ',I3/'   version year: ',I4/
     &       '   version day of year: ',I3)
 510  format(3X,'min nb of correct slopes/intercepts to ',
     &       ' compute an mean slope/intercept:',I2)

 900  write(msu_cllgu,*)'Error opening test coefficients file'
      text='Test coef file could not be opened'
      call ml_wt('error',text)
      call aappexit(2)

 901  write(msu_cllgu,*)'Error reading test coefficients file'
      text='Test coefficients file could not be read'
      call ml_wt('error',text)
      call aappexit(2)

 902  write(msu_cllgu,*)'Error closing test coefficients file'
      text='Test coefficients file could not be closed' 
      call ml_wt('error',text)
      call aappexit(2)

 200  return
      end

