        subroutine m_tgrad(tg1,tg2, tgrad)
!+
!NOM/NAME $Id: m_tgrad.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     For each scan line, this module converts the target 1 temperature 
!     into radiance for MSU channels 1 and 2.
!     Idem with the target 2 temperature but for the MSU channel 3 and 4.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!             
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Nov. 95       no             T. LABROT
!         
!ARGUMENTS 
!   input :
!      dble prec tg1(msu_mxlin)
!           temperature of the target 1 for each scan line 
!      dble prec tg2(msu_mxlin)
!           temperature of the target 2 for each scan line 
!
!   output:
!      dble prec tgrad(msu_mxchn,msu_mxlin)  !radiances array
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!   subroutine m_planck(ich,temp,rad)
!    
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------


!--DECLARATIONS----------------------------------------------------------------

      implicit none
                                                            
      
!includes/commons
#include "msuparam.h"
#include "mclin.h"
#include "msu1bhd.h"
#include "missingval.h" 
    
! argument variables
      double precision tg1(*),tg2(*)
      double precision tgrad(4,*)
      
! local variables
      character*80 text             !variable text
      integer ilin,ich              !index of loop
      double precision temp         !target temperature 
      double precision rad          !radiance

!--END OF DECLARATIONS----------------------------------------------------------

      write(text,'("Convert temp.of the 2 targets into radiances",
     &     " for each scan line")')
      call ml_wt('info',text)

      if (iprint.ge.1) write(msu_cllgu,*)
      if (iprint.ge.2) write(msu_cllgu,*) '***************************'
      if (iprint.ge.1) write(msu_cllgu,*) '**  SUBROUTINE M_TGRAD   **'
      if (iprint.ge.2) write(msu_cllgu,*) '***************************'

      do ilin=1,msu_mxlin
        do ich=1,msu_mxchn
          tgrad(ich,ilin)=missdp
        enddo
      enddo
      
      do ilin=1,msu_h_scnlin
        do ich=1,msu_mxchn

          temp = tg1(ilin)
          if (ich.eq.3.or.ich.eq.4) temp = tg2(ilin)

          if (temp.lt.missdp) then
            call m_planck(ich,temp,rad)
            tgrad(ich,ilin) = rad
          endif
          
          if(iprint.ge.1) then
            write(msu_cllgu,fmt=500)ilin,ich,temp,tgrad(ich,ilin)
          endif
     
        enddo
      enddo

 500  format(3X,'line  channel  temp.  rad  ',I2,3X,I1,3X,F8.3
     &       ,2X,F12.10) 
      return
      end
