        subroutine m_tgtmp(quality1,quality2,tg1,tg2)
!+
!NOM/NAME $Id: m_tgtmp.F 192 2010-01-27 15:06:55Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!   For each scan line, this module computes target 1 temperature and
!   target 2 temperature, which are derived respectively from PRT 1A,1B 
!   counts and PRT 2A,2B counts. Target 1 is viewed by channels 1 and 2.
!   Target 2 is viewed by channels 3 and 4.
!   To convert PRT count to temperature requires two steps:
!   - Convert count to resistance (call m_cntres)
!   - Convert resistance to temperature (call m_restmp)
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!             
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Nov. 95       no             T. LABROT
! 1.1      July 96   one test more to discern when ref temp. must be changed(TL)
! 1.2      Dec. 96   bug((1-flg1a)*t1a+(1-flg1a)*t1b)-->((1-flg1a)*t1a + (1-flg1b)*t1b)
!                       ((1-flg2a)*t2a+(1-flg2a)*t2b)-->((1-flg2a)*t2a + (1-flg2b)*t2b)
!         
!ARGUMENTS 
!     input :
!       integer quality1(7,msu_mxlin)
!       integer quality2(6,msu_mxchn,msu_mxlin)
!     output:
!      dble prec tg1(msu_mxlin),tg2(msu_mxlin)
!           temperatures of the target 1 for each scan line 
!           temperatures of the target 2 for each scan line 
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!      sub. m_cntres  (see above)
!      sub. m_restmp  (see above)
!    
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none  
                                                          
      
!includes/commons
#include "msuparam.h"
#include "mclin.h"
#include "msu1bhd.h"
#include "msu1bdts.h"
#include "missingval.h"
#include "mcinit.h"

! argument variables
      integer quality1(7,*),quality2(6,msu_mxchn,*)
      double precision tg1(msu_mxlin),tg2(msu_mxlin)
      
! local variables
      character*80 text                     !variable text
      integer ilin                          !index of scan line number 
      integer iprt                          !index for the 4 PRTs
      integer ind                           !index to get MSU data
      integer prt                           !PRT value
      integer flg,flg1a,flg2a,flg1b,flg2b   !flags
      integer tcallo,tcalhi
!             High and low calibration reference points for electronic 
!             systems A and B respectively
      integer badtg1lin,badtg2lin
      integer refnb                         !counter of PRT temp. equal ref temp.
      double precision tmp,t1a,t1b,t2a,t2b  !temperatures
      double precision res ! resistance value
      integer refmax                        !limit to know if the target temp
      parameter ( refmax = 18 )             !is deriving or not.

!--END OF DECLARATIONS----------------------------------------------------------

      do ilin=1,msu_mxlin
        tg1(ilin)= missdp
        tg2(ilin)= missdp
      enddo
      refnb=0

      text='Compute temperatures of targets 1 and 2 for each scan line:'
      call ml_wt('info',text)

      if (iprint.ge.1) write(msu_cllgu,*)
      if (iprint.ge.2) write(msu_cllgu,*) '***************************'
      if (iprint.ge.1) write(msu_cllgu,*) '**  SUBROUTINE M_TGTMP   **'
      if (iprint.ge.2) write(msu_cllgu,*) '***************************'
      
      do ilin=1,msu_h_scnlin
      
        tmp  = missdp
        t1a  = missdp
        t2a  = missdp
        t1b  = missdp
        t2b  = missdp      

        do iprt=1,4 
        
          if(iprint.ge.1)write(msu_cllgu,fmt=700) ilin,iprt 
                    
          if (prtwght(iprt) .eq. 1) then
            if(iprt.eq.1.or.iprt.eq.2) ind=57
            if(iprt.eq.3.or.iprt.eq.4) ind=63
            prt = msu_data(ind+iprt,ilin)
            
            if(iprt.eq.1.or.iprt.eq.3) then
              tcallo=msu_data(2,ilin)
              tcalhi=msu_data(10,ilin)
            endif
            
            if(iprt.eq.2.or.iprt.eq.4) then
              tcallo=msu_data(3,ilin)
              tcalhi=msu_data(11,ilin)
            endif
            
            if(iprint.ge.2) 
     &         write(msu_cllgu,fmt=710) tcallo,tcalhi,prt
                      
            call m_cntres(ilin,iprt,prt,tcallo,tcalhi, res,flg)
            if (flg.eq.0) then
              call m_restmp(ilin,iprt,res, tmp)
              if(tmp.eq.reftmp) refnb=refnb+1
            else
              tmp=missdp
            endif
          else
            flg=1
            write(text,'("PRT",I1,"has been turned off")') iprt
            call ml_wt('notice',text) 
            if(iprint.ge.1) write(msu_cllgu,fmt=500) iprt
          endif
          
          if(iprt.eq.1) flg1a=flg 
          if(iprt.eq.1) t1a=tmp
          if(iprt.eq.2) flg1b=flg 
          if(iprt.eq.2) t1b=tmp
          if(iprt.eq.3) flg2a=flg 
          if(iprt.eq.3) t2a=tmp
          if(iprt.eq.4) flg2b=flg 
          if(iprt.eq.4) t2b=tmp          
     
        enddo
    
        if (flg1a.ne.0.and.flg1b.ne.0) then
          tg1(ilin) = missdp
        else
           tg1(ilin) = ((1-flg1a)*t1a + (1-flg1b)*t1b)
     &               / (((1-flg1a)+(1-flg1b))*1.d+0)
        endif

        if (flg2a.ne.0.and.flg2b.ne.0) then
          tg2(ilin) = missdp
        else
          tg2(ilin) = ((1-flg2a)*t2a + (1-flg2b)*t2b)
     &               / (((1-flg2a)+(1-flg2b))*1.d+0)
        endif
        
        write(text,'("line PRT 1a,1b,2a,2b temp.:",I3,2X,
     &        4(F8.3,1X))') ilin,t1a,t1b,t2a,t2b 
        call ml_wt('debug',text)
        
        if(iprint.ge.1) then
          write(msu_cllgu,fmt=510) ilin
          write(msu_cllgu,fmt=520) 
          write(msu_cllgu,fmt=530) flg1a,flg1b,flg2a,flg2b
          write(msu_cllgu,fmt=540) t1a,t1b,t2a,t2b
          write(msu_cllgu,fmt=550) tg1(ilin)
          write(msu_cllgu,fmt=560) tg2(ilin)
        endif
        
! Update quality2(3,ich,ilin) = 'no good PRTs for this line'
!                             = 1 if for ch1 and ch2, PRT1a and PRT1b are bad
!                                    for ch3 anf ch4, PRT2a and PRT2b are bad
!                             = 0 if for ch1 and ch2, PRT1a or PRT1b is good 
!                                    for ch3 and ch4, PRT2a or PRT2b is good 
        if (flg1a.ne.0.and.flg1b.ne.0) then 
          quality2(3,1,ilin) = 1    
          quality2(3,2,ilin) = 1      
        else        
          quality2(3,1,ilin) = 0    
          quality2(3,2,ilin) = 0 
        endif
        if (flg2a.ne.0.and.flg2b.ne.0) then 
          quality2(3,3,ilin) = 1    
          quality2(3,4,ilin) = 1 
        else        
          quality2(3,1,ilin) = 0    
          quality2(3,2,ilin) = 0 
        endif
   
! Update quality2(6,ich,ilin) = 'some bad PRT temp. for this line'
!                             = 1 if for ch1 and ch2, t1a or t1b is bad
!                             = 1 if for ch3 and ch4, t2a or t2b is bad
        if(t1a.ge.missdp.or.t1b.ge.missdp) then
          quality2(6,1,ilin) = 1    
          quality2(6,2,ilin) = 1 
        endif          
        if(t2a.ge.missdp.or.t2b.ge.missdp) then
          quality2(6,3,ilin) = 1    
          quality2(6,4,ilin) = 1 
        endif

      enddo
 
!       
      if (refnb.ge.refmax) then
        write(text,'(I3," PRT temp. are equal to the reference temp."
     &         )')refnb
        call ml_wt('error',text)
        write(text,'("Perhaps the target temp. is deriving and ",
     &               "must be changed in calcoeffile")')
        call ml_wt('error',text) 
      endif
         
! Update quality1(4,ilin)= bit13 of calibration problem code
!                        = 1 if scan line was not calibrated because of bad
!                            or insufficient PRT data.
!(in our code, "bad or insufficient PRT data" to calibrate means not enough 
!scan lines with good target temperatures.
!The minimum number = minnb (testcoeffile)
!                        = 0 if normal calibration
! 
      badtg1lin = 0
      badtg2lin = 0      
      do ilin=1,msu_h_scnlin   
        if(tg1(ilin).ge.missdp) badtg1lin=badtg1lin+1
        if(tg2(ilin).ge.missdp) badtg2lin=badtg2lin+1
      enddo 
      if((msu_h_scnlin-badtg1lin).lt.minnb.and.
     &    (msu_h_scnlin-badtg2lin).lt.minnb) then
        do ilin=1,msu_h_scnlin
          quality1(4,ilin) = 1
        enddo
      endif
      

 500  format(/2X,'PRT nb ',I1,' has been turned off')
 510  format(/2X,'SCAN LINE :',I3)
 520  format(/4X,'PRT1a      PRT1b      PRT2a      PRT2b')
 530  format(6X,4(I1,10X))
 540  format(4(3X,F8.3))
 550  format(/4X,'target 1 temp.:',f8.3)
 560  format(4X,'target 2 temp.:',f8.3)
 700  format(/2X,'SCAN LINE: ',I2,'    IPRT: ',I1)
 710  format(3X,'t.callo   t.calhi : ',2(I6,2X),'  PRT: ',I6)
 
      return
      end
