      subroutine m_upcommon(slope,intcept,quality1,quality2)
!+
!NOM/NAME $Id: m_upcommon.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!     This module finishes to update the parameters in the commons msu1bhd,  
!     msu1bdt and msu1bdts (includes msu1bhd.h, msu1bdt.h and msu1bdts.h).
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0     Nov 1995       no             T. LABROT
! 1.1     July 1998  change word2 to word4 from integer*2 to integer
!         
!ARGUMENTS 
!   input :
!    integer quality1(7,msu_mxlin)
!                 quality information array about calibration
!    integer quality2(6,msu_mxchn,msu_mxlin)
!                 quality information array about calibration
!    dble prec slope(msu_mxchn),intcept(msu_mxchn) 
!                 slopes and intercepts arrays for each scan line
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!   sub.  gp_bse(word,bitnum) to set bitnum in word
!   sub.  gp_bcl(word,bitnum) to clear bitnum in word
!    
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none                                                     
 
 
!includes/commons
#include "msuparam.h"
#include "mclin.h"
#include "mcinit.h"
#include "missingval.h"
#include "msu1bhd.h"
#include "msu1bdts.h"

! argument variables
      integer quality1(7,*),quality2(6,msu_mxchn,*)
      double precision slope(msu_mxchn)
      double precision intcept(msu_mxchn)

! local variables
      integer i,ich,ilin        !index of loops
      integer word1             !word where we put quality bits
      integer  word4           !word where we put quality bits
      
!--END OF DECLARATIONS----------------------------------------------------------

      if (iprint.ge.1) write(msu_cllgu,*)
      if (iprint.ge.2) write(msu_cllgu,*) '***************************'
      if (iprint.ge.1) write(msu_cllgu,*) '*  SUBROUTINE M_UPCOMMON  *'
      if (iprint.ge.2) write(msu_cllgu,*) '***************************'

!!!   COMMON MSU1BHD FOR THE HEADER   !!!

      do ich=1,msu_mxchn
! integer*2 msu_h_callocsclin: count of calibrated, earth located scan 
! lines in this data set
        if(slope(ich).lt.missdp) then
          msu_h_callocsclin = msu_h_scnlin
        endif     
      
! integer   msu_h_calinf(4,msu_mxchn)   
!    calibration information:  10^12*mean of slopes
!                              0*standard deviation of slopes
!                              10^8*mean of intercepts
!                              0*standard deviation of intercepts

        msu_h_calinf(1,ich) = idint(slope(ich) * 1.d+12)
        msu_h_calinf(2,ich) = 0
        msu_h_calinf(3,ich) = idint(intcept(ich) * 1.d+08)
        msu_h_calinf(4,ich) = 0
      
! integer msu_h_nolincor(3,msu_mxlin)
!    nonlinearity correction coefficients
!      10^12 * second order term
!      10^6  * first order term
!      10^6  * zeroth order term

        msu_h_nolincor(1,ich)=idint(nonlin(ich,3) * 1.d+12)
        msu_h_nolincor(2,ich)=idint(nonlin(ich,2) * 1.d+06)
        msu_h_nolincor(3,ich)=idint(nonlin(ich,1) * 1.d+06)
      
! integer msu_h_censperes(msu_mxchn)
!    10^4 * centroid of spectral response functions

        msu_h_censperes(ich)=idint(cwave(ich) * 1.d+04)
       
! integer msu_h_radspa(msu_mxchn)
!    10^6 * radiance of space for each MSU channel

        msu_h_radspa(ich)=idint(sprad(ich) * 1.d+06)
              
      enddo 
      
! integer msu_h_prttores(2)
!    PRT counts to resistance coefficients
!      10^2 * first one K0
!      10^2 * second one K1
       
      msu_h_prttores(1)=idint(k0 * 1.d+02)
      msu_h_prttores(2)=idint(k1 * 1.d+02)
      
! integer msu_h_iwttotemp(3,4) 
!    internal warm target PRT resistance to temperature(K) coefficients
!    (PRT 1A, PRT 2A, PRT 1B, PRT 2B) 

      do i=1,4
        msu_h_iwttotemp(1,i)=idint(restmpcf(i,3) * 1.d+12)
        msu_h_iwttotemp(2,i)=idint(restmpcf(i,2) * 1.d+07)
        msu_h_iwttotemp(3,i)=idint(restmpcf(i,1) * 1.d+06)
      enddo     



!!!   COMMON MSU1BDTS FOR THE RECORDS   !!!

! Update the calibration part in the integer msu_qualind(msu_mxlin) 
!    if a bit is on(=1) then the statement is true.
!      bit 28: no calibration (==> =1 at DC level)
!    all bits set to 0 indicates normal calibration
!      bit 15: scan line was not calibrated because of bad time
!              always =0 because no time test in the calibration method
!      bit 14: scan line was calibrated using fewer than the preferred
!              number of scan lines because of proximity to start or 
!              end of data set or to a data gap.
!      bit 13: scan line was not calibrated because of bad or insufficient
!              PRT data
!              =0 if sufficient PRT data
!              =1 if insufficient PRT data for all calib cycles
!      bit 12: scan line was calibrated but with marginal PRT data
!              always =0 because we don't compute calib coef with marginal data 
!      bit 11: some uncalibrated channels of this scan.See channel indicators
!              =0 if all the channels are calibrated
!              =1 if some uncalibrated channels of this scan
!      bit 10: uncalibrated due to instrument mode
!              =0 if no problem due to instrument mode
!              =1 if uncalibrated due to instrument mode

      do ilin=1,msu_mxlin

        quality1(2,ilin) = 0
        quality1(3,ilin) = 0
        quality1(5,ilin) = 0
        quality1(7,ilin) = 0
        
        word1 = msu_qualind(ilin)

        if (quality1(1,ilin).eq.1) then
          call gp_bse(word1,28)
        else
          call gp_bcl(word1,28)
        endif
        
        if (quality1(2,ilin).eq.1) then
          call gp_bse(word1,15)
        else
          call gp_bcl(word1,15)
        endif

        if (quality1(3,ilin).eq.1) then
          call gp_bse(word1,14)
        else
          call gp_bcl(word1,14)
        endif

        if (quality1(4,ilin).eq.1) then
          call gp_bse(word1,13)
        else
          call gp_bcl(word1,13)
        endif

        if (quality1(5,ilin).eq.1) then
          call gp_bse(word1,12)
        else
          call gp_bcl(word1,12)
        endif

        if (quality1(6,ilin).eq.1) then
          call gp_bse(word1,11)
        else
          call gp_bcl(word1,11)
        endif

        if (quality1(7,ilin).eq.1) then
          call gp_bse(word1,10)
        else
          call gp_bcl(word1,10)
        endif

        msu_qualind(ilin) = word1


      
! integer*2 msu_scnlinqual(msu_mxchn,msu_mxlin)
!    quality flag

        do ich=1,msu_mxchn
        
          quality2(1,ich,ilin) = 0
        
          word4 = msu_scnlinqual(ich,ilin)

          if (quality2(1,ich,ilin).eq.1) then
            call gp_bse(word4,5)
          else
            call gp_bcl(word4,5)
          endif
        
          if (quality2(2,ich,ilin).eq.1) then
            call gp_bse(word4,4)
          else
            call gp_bcl(word4,4)
          endif

          if (quality2(3,ich,ilin).eq.1) then
            call gp_bse(word4,3)
          else
            call gp_bcl(word4,3)
          endif

          if (quality2(4,ich,ilin).eq.1) then
            call gp_bse(word4,2)
          else
            call gp_bcl(word4,2)
          endif

          if (quality2(5,ich,ilin).eq.1) then
            call gp_bse(word4,1)
          else
            call gp_bcl(word4,1)
          endif

          if (quality2(6,ich,ilin).eq.1) then
            call gp_bse(word4,0)
          else
            call gp_bcl(word4,0)
          endif

          msu_scnlinqual(ich,ilin) = word4
        enddo
      enddo
      
! integer msu_pmcal(3,msu_mxchn,msu_mxlin) 
!    primary calibration coefficients
!      0     * second order term (always = 0)
!      10^12 * first order term
!      10^8  * zeroth order term
!
! For the moment we don't update msu_sdcal(3,msu_mxchn,msu_mxlin)
!    secondary calibration coefficients).

      do ich=1,msu_mxchn
        do ilin=1,msu_mxlin
          msu_pmcal(1,ich,ilin)=0
          msu_pmcal(2,ich,ilin)=idint(slope(ich) * 1.d+12)
          msu_pmcal(3,ich,ilin)=idint(intcept(ich) * 1.d+08) 
        enddo
      enddo
      
!      write(*,*) 'quality1(1,1)  ',quality1(1,1)
!      write(*,*) 'quality1(2,1)  ',quality1(2,1)      
!      write(*,*) 'quality1(3,1)  ',quality1(3,1)      
!      write(*,*) 'quality1(4,1)  ',quality1(4,1)      
!      write(*,*) 'quality1(5,1)  ',quality1(5,1)      
!      write(*,*) 'quality1(6,1)  ',quality1(6,1)      
!      write(*,*) 'quality1(7,1)  ',quality1(7,1)      
!      write(*,*) 
!      write(*,*) 'quality2(1,1,1)  ',quality2(1,1,1)     
!      write(*,*) 'quality2(2,1,1)  ',quality2(2,1,1)      
!      write(*,*) 'quality2(3,1,1)  ',quality2(3,1,1)      
!      write(*,*) 'quality2(4,1,1)  ',quality2(4,1,1)      
!      write(*,*) 'quality2(5,1,1)  ',quality2(5,1,1)      
!      write(*,*) 'quality2(6,1,1)  ',quality2(6,1,1)  
!      write(*,*) 
!      write(*,*) 'quality2(1,2,1)  ',quality2(1,2,1)     
!      write(*,*) 'quality2(2,2,1)  ',quality2(2,2,1)      
!      write(*,*) 'quality2(3,2,1)  ',quality2(3,2,1)      
!      write(*,*) 'quality2(4,2,1)  ',quality2(4,2,1)      
!      write(*,*) 'quality2(5,2,1)  ',quality2(5,2,1)      
!      write(*,*) 'quality2(6,2,1)  ',quality2(6,2,1) 
             
      return
      end
