        subroutine mclin
!+
!NOM/NAME $Id: mclin.F 47 2006-08-21 08:44:41Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module initializes the navigation and calibration tasks. 
!     It reads the input options (msu_cloptn)
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Nov. 95      no              T. LABROT
! 1.1      Febr 97   A never used option has been suppressed  (TL)
!         
!ARGUMENTS 
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
! 
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none  
                                                          
     
! includes/commons
#include "mclin.h"
#include "c_1bnav.h"

! argument variables
      
! local variables
      integer i         !index of loop
      integer ios       !input/output status
      character*80 text !variable text
      integer msu_cloptn(11) 
!      msu_cloptn(1) -- 1 for MSU navigation/localisation (0 if not for)
!      msu_cloptn(2) -- 1 for MSU calibration (0 if not for)
!      msu_cloptn(3) -- 1 for calib statistics file (0 if not for)
!      msu_cloptn(4) -- logical unit for MSU log/debug file
!      msu_cloptn(5) -- logical unit for calib statistics file
!      msu_cloptn(6) -- logical unit MSU level1b file
!      msu_cloptn(7) -- logical unit for SATPOS file
!      msu_cloptn(8) -- logical unit for the clock drift data file
!      msu_cloptn(9) -- logical unit for MSU calibration coefficients file
!      msu_cloptn(10)-- logical unit for MSU calib/test coefficients file
!      msu_cloptn(11)-- print option in the log.file
!                       = 0, nothing is printed, log file not open
!                       = 1, entering routines and few values of the main
!                            parameters are printed
!                       = 2, computed values that would help in debugging 
!                            are printed

!--END OF DECLARATIONS---------------------------------------------------------

      read(*,*) msu_cloptn

! Check msu_cloptn values
      do i = 1,3
        if(msu_cloptn(i).gt.1 .or. msu_cloptn(i).lt.0) then
          write(text,'("Invalid option number--correct input",I2)')
     &    msu_cloptn(i)
          call ml_wt('error',text)
          stop
        endif
      enddo

      if((msu_cloptn(11).ne.0).and.(msu_cloptn(11).ne.1).and.
     & (msu_cloptn(11).ne.2)) then
         write(text,'("Invalid print option value--correct input")')
         call ml_wt('error',text)
         stop
      endif

      msu_lflg    = msu_cloptn(1)
      msu_cflg    = msu_cloptn(2)
      msu_stflg   = msu_cloptn(3)
      msu_cllgu   = msu_cloptn(4)
      msu_l1bu    = msu_cloptn(6)
      iprint      = msu_cloptn(11)

      if (msu_stflg.eq.1) msu_stu = msu_cloptn(5)

     
      if (msu_lflg.eq.1) then 
        satposu     = msu_cloptn(7)
        ckdriftu    = msu_cloptn(8)
        
        read(*,*,iostat=ios) yaw, roll, pitch
        if(ios.ne.0) then
          call ml_wti('error',
     &       'while reading attitude values I/O status',ios)
          call ml_wt('error','msucl aborted')
          call aappexit(1)
        endif
        
! convert the values in radians
        yaw = yaw*1.d-3
        roll = roll*1.d-3
        pitch = pitch*1.d-3
      endif


      if (msu_cflg.eq.1) then
        msu_coefu   = msu_cloptn(9)
        msu_tstcoefu= msu_cloptn(10)
      endif

      return
      end
