        subroutine mclsetu
!+
!NOM/NAME $Id: mclsetu.F 47 2006-08-21 08:44:41Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module opens  the log/debug file (if requested).
!     It opens the MSU.level1b file, reads the header, all the records.
!     The data are ranged in commons msu1bhd, msu1bdt, msu1bdts
!     (includes msu1bhd.h, msu1bdt.h, msu1bdts.h).
!     It opens the localisation files if requested.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Nov. 95      no              T. LABROT
! 1.1      25/05/98  modified to conform to NOAA release  (TL)
! 1.2      Jul 98  force errclock to 0 when problem on file (Marcel Derrien)
! 2.0      August00 Add a test on ierr, output of sp_read (TL)
! 2.1      Dec 00   Display attitude values  P. Brunel
! 2.1      Dec 00   Change position in the code of the display of
!                     the clock error  P. Brunel
! 2.2      Mar 04   Update for Linux (N Atkinson / A Korpela)
! 2.3      Jul 04   Upgrade to navigation code version 2.
!                   def_att call changed for arguments and position in
!                   code (after sp_read)     P. Brunel
!         
!ARGUMENTS 
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!      sub. def_att(scname,att_mode,yaw,roll,pitch)
!      sub. calatt(yaw,roll,pitch,att)
!      sub. clkerr_get(ckdriftu,ijstrt,errclock,ierr)
!      sub. sp_read(ijstrt,ijend,satposu,scname,ierr)
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none                                                     


! includes/commons
#include "msuparam.h"
#include "msu1bhd.h"
#include "msu1bdt.h"
#include "msu1bdts.h"
#include "mclin.h"
#include "c_1bnav.h"
#include "reclopt.h"

! argument variables

! local variables
      integer i,j        !index of loops
      integer ios        !variable that receives the error status
      integer recn       !record number in the MSU 1B file
      integer recn0      !first record number in the MSU 1B file
      integer recn1      !last record number in the MSU 1B file
      integer ierr
      integer satid
      character*80 text  !variable text 
      character*6 scname
      double precision ijstrt,ijend
      integer loc_rl
      integer att_mode        ! attitude mode
                                ! 0 = NOAA POES operational mode
                                ! 1 = METOP Yaw steering mode
                                ! 2 = Geocentric mode
!functions
      integer ord1bid
      character*6 noascnam
      
!--END OF DECLARATIONS---------------------------------------------------------

!open the log file (if requested)
      if (iprint.gt.0) then
        open(msu_cllgu,form='formatted',access='sequential',
     &       status='new',iostat=ios)
        if(ios.ne.0)
     &  open(msu_cllgu,form='formatted',access='sequential',
     &       status='unknown',iostat=ios)

        if(ios.ne.0) then
          write(text,'("The log/debug file could not be opened,ios:",
     &    I3)')ios
          call ml_wt('error',text)
          call aappexit(2)
        else
          text='The log/debug file has been opened'
          call ml_wt('info',text)

       if (iprint.ge.1) write(msu_cllgu,*)
       if (iprint.ge.2) write(msu_cllgu,*) '***************************'
       if (iprint.ge.1) write(msu_cllgu,*) '**   SUBROUTINE MCLSETU  **'
       if (iprint.ge.2) write(msu_cllgu,*) '***************************'
       if (iprint.ge.1) write(msu_cllgu,*)

         if (iprint.ge.1) write(msu_cllgu,*)
     &   '  The log/debug file has been opened'
        endif
      endif

!open and read the MSU-level1b file
      loc_rl = reclmsu / rldiv
      open(msu_l1bu,recl=loc_rl,form='unformatted',access='direct',
     &     status='old',iostat=ios)
      if(ios.ne.0)
     &  open(msu_l1bu,recl=loc_rl,form='unformatted',access='direct',
     &     status='unknown',iostat=ios)
      if(ios.ne.0) then
         write(text,'("MSU1B file could not be opened,ios:",I3)')ios
         call ml_wt('error',text)
         if (iprint.ge.1) write(msu_cllgu,*)
     &   '  MSU1B file could not be opened,ios:',ios
         call aappexit(2)
      else
         text='MSU1B file has been opened'
         call ml_wt('info',text)
         if (iprint.ge.1) write(msu_cllgu,*)
     &   '  MSU1B file has been opened'
      endif

      read(msu_l1bu,rec=1,iostat=ios) msu_hd

      if(ios.ne.0) then

        write(text,'("MSU1B header could not be read,ios:",I3)')ios
        call ml_wt('error',text)
        if(iprint.ge.1) write(msu_cllgu,*)
     &   '  MSU1B header could not be read,ios:',ios
        call aappexit(2)

      else
      
       write(text,'("Number of  MSU scan lines: ",I2)')msu_h_scnlin
       call ml_wt('info',text)
       if(iprint.ge.1) write(msu_cllgu,*)
     &  '  Number of MSU scan lines: ',msu_h_scnlin

       recn0=msu_h_hdrcnt+1
       recn1=msu_h_hdrcnt+msu_h_scnlin
    
       do recn=recn0,recn1
        read(msu_l1bu,rec=recn,iostat=ios) msu_dt
        
        if(ios.ne.0) then
        
         write(text,'("MSU1B record could not be read,ios:",I3)')ios
         call ml_wt('error',text)
         if (iprint.ge.1) write(msu_cllgu,*)
     &   '  MSU1B record could not be read, ios:',ios
         call aappexit(2)

        else

         msu_scnlin(recn - msu_h_hdrcnt)     = msu_scnlin0
         msu_scnlinyr(recn - msu_h_hdrcnt)   = msu_scnlinyr0
         msu_scnlindy(recn - msu_h_hdrcnt)   = msu_scnlindy0
         msu_clockdrift(recn - msu_h_hdrcnt) = msu_clockdrift0
         msu_scnlintime(recn - msu_h_hdrcnt) = msu_scnlintime0
         msu_scnlinbit(recn - msu_h_hdrcnt)  = msu_scnlinbit0
         msu_mjfrcnt(recn - msu_h_hdrcnt)    = msu_mjfrcnt0
         msu_scnpos(recn - msu_h_hdrcnt)     = msu_scnpos0
         msu_qualind(recn - msu_h_hdrcnt)    = msu_qualind0
         msu_navstat(recn - msu_h_hdrcnt)    = msu_navstat0
         msu_attangtime(recn - msu_h_hdrcnt) = msu_attangtime0
         msu_rollang(recn - msu_h_hdrcnt)    = msu_rollang0
         msu_pitchang(recn - msu_h_hdrcnt)   = msu_pitchang0
         msu_yawang(recn - msu_h_hdrcnt)     = msu_yawang0
         msu_scalti(recn - msu_h_hdrcnt)     = msu_scalti0
         msu_filler3(recn - msu_h_hdrcnt)    = msu_filler30
 

         do i=1,5
          msu_filler1(i,(recn-msu_h_hdrcnt)) = msu_filler10(i)
         enddo

         do i=1,8
          msu_filler2(i,(recn-msu_h_hdrcnt)) = msu_filler20(i)
          msu_filler4(i,(recn-msu_h_hdrcnt)) = msu_filler40(i)
         enddo

         do i=1,230
          msu_filler5(i,(recn-msu_h_hdrcnt)) = msu_filler50(i)
         enddo        

         do i=1,msu_mxchn
          msu_scnlinqual(i,(recn-msu_h_hdrcnt))=msu_scnlinqual0(i)
         enddo  

         do j=1,3
           do i=1,msu_mxchn
             msu_pmcal(j,i,recn-msu_h_hdrcnt)=msu_pmcal0(j,i)
             msu_sdcal(j,i,recn-msu_h_hdrcnt)=msu_sdcal0(j,i)
           enddo
          
           do i=1,11
             msu_ang(j,i,recn-msu_h_hdrcnt) = msu_ang0(j,i)
           enddo
         enddo

         do j=1,2
           do i=1,11
             msu_pos(j,i,recn-msu_h_hdrcnt) = msu_pos0(j,i)
           enddo
         enddo

         do i=1,112     
           msu_data(i,recn - msu_h_hdrcnt)= msu_data0(i)
         enddo
 
        endif

       enddo
  
      endif

      satid = msu_h_satid
      satid = ord1bid (satid,msu_h_startdatajd)
      scname=noascnam(satid)

!
! Open localisation files if requested
!
      if(msu_lflg.eq.1) then
! Opens and reads the SATPOS file
! -----------------------------------
        open (unit=satposu,status='old',iostat=ios)
        if( ios.ne.0 )
     &    open (unit=satposu,status='unknown',iostat=ios)
        if( ios.ne.0 ) then
          call ml_wti('error',
     &       'while opening the satpos file I/O status',ios)
          call ml_wt('error','mclsetu aborted')
          call aappexit(2)
        endif
        
! Read the satpos file
! --------------------

        ijstrt=msu_h_startdatajd + 
     &      (msu_h_startdatatime-errclock*1000.)/86400.d3
        ijend=msu_h_enddatajd + 
     &       (msu_h_enddatatime-errclock*1000.)/86400.d3
        call sp_read(ijstrt,ijend,satposu,scname,ierr)
        close(unit=satposu,iostat=ios)
        
        if(ierr.ne.0) then
          call ml_wti('error','return code of sp_read differ from 0',
     &                ierr)
          call ml_wt('error','mclsetu aborted')
          call ml_wt('error','navigation aborted')
          call ml_wt('error','but calibration can work')
          call ml_wt('error','try calibration alone')
          call aappexit(2)
        endif

        if(yaw.lt.-99.d-03) then
          call def_att(scname,att_mode,yaw,roll,pitch)
          call ml_wt('info','take default attitude values')
          call ml_wti('info','default attitude mode ',att_mode)
          call ml_wtdp('info','default attitude yaw ',yaw)
          call ml_wtdp('info','default attitude roll ',roll)
          call ml_wtdp('info','default attitude pitch ',pitch)
        else
          call ml_wtdp('info','attitude yaw ',yaw)
          call ml_wtdp('info','attitude roll ',roll)
          call ml_wtdp('info','attitude pitch ',pitch)
        endif
        
        call calatt(yaw,roll,pitch,att)
        
! If the file has already been located we do not correct
! again from the clock error
! --------------------------
        if(msu_h_epoyr.le.1950) then
        
          open (unit=ckdriftu,status='old',iostat=ios)
          if(ios.ne.0)
     &      open (unit=ckdriftu,status='unknown',iostat=ios)
          if(ios.ne.0) then
            call ml_wti('error',
     &       'opening clock error file I/O status',ios)
            call aappexit(2)
          endif
          
          ijstrt=msu_h_startdatajd + 
     &      (msu_h_startdatatime)/86400.d3
          call clkerr_get(ckdriftu,ijstrt,errclock,ierr)
          
          close(unit=ckdriftu,iostat=ios)
          if(ierr.ne.0.or.abs(errclock).gt.2.) then
            call ml_wti('warning',
     &       'reading clock error value from clkerr_get ',ierr)
            call ml_wtr('warning','clock error from file',
     &                 errclock)
            call ml_wt('warning','clock error forced to 0')
            errclock=0. 
          else
            call ml_wtr('info','clock error value (sec)',errclock)
          endif
          
        else
        
          errclock=0.
          call ml_wt('info',
     &     '1B file already corrected from clock error')
     
        endif
       
        
! Init earth location parameters
! ------------------------------

        first_pnav = 1
        first_lnav = 1
        step_pnav = 1
        step_lnav = 1
        nb_pnav = msu_mxpix
        
      endif      
 
      return
      end

