        subroutine ml1bwrt
!+
!NOM/NAME $Id: ml1bwrt.F 47 2006-08-21 08:44:41Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module writes in the MSU level1b file, header and records.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      Nov. 95      no              T. LABROT
! 1.1      Febr 97    Some changes when "ios" not equal to 0  (TL)
! 1.2      25/05/98  modified to conform to NOAA release      (TL)
!         
!ARGUMENTS 
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none                                                     


! includes/commons
#include "msuparam.h"
#include "msu1bhd.h"
#include "msu1bdt.h"
#include "msu1bdts.h"
#include "mclin.h"

! local variables
      integer i,j        !index of loops
      integer ios        !variable that receives the error status
      integer recn       !record number in the HIRS 1B file
      integer recn0      !first record number in the HIRS 1B file
      integer recn1      !last record number in the HIRS 1B file
      character*80 text  !variable text
!--END OF DECLARATIONS---------------------------------------------------------

      if (iprint.ge.1) write(msu_cllgu,*)
      if (iprint.ge.2) write(msu_cllgu,*) '***************************'
      if (iprint.ge.1) write(msu_cllgu,*) '**   SUBROUTINE ML1BWRT  **'
      if (iprint.ge.2) write(msu_cllgu,*) '***************************'


! Write in the header of the level 1b
      write(msu_l1bu,rec=1,iostat=ios) msu_hd
      if(ios.ne.0) then
       text='MSU1B header could not be written,  ios:'
       call ml_wti('error',text,ios)
       if (iprint.ge.1) write(msu_cllgu,*)
     &   '  MSU1B header could not be written,ios:',ios
       call aappexit(2)
      endif

! Write in the records of the level 1b
      recn0=msu_h_hdrcnt+1
      recn1=msu_h_hdrcnt+msu_h_scnlin

      do recn=recn0,recn1
        msu_scnlin0     = msu_scnlin(recn - msu_h_hdrcnt)
        msu_scnlinyr0   = msu_scnlinyr(recn - msu_h_hdrcnt)
        msu_scnlindy0   = msu_scnlindy(recn - msu_h_hdrcnt)
        msu_clockdrift0 = msu_clockdrift(recn - msu_h_hdrcnt)
        msu_scnlintime0 = msu_scnlintime(recn - msu_h_hdrcnt)
        msu_scnlinbit0  = msu_scnlinbit(recn - msu_h_hdrcnt)
        msu_mjfrcnt0    = msu_mjfrcnt(recn - msu_h_hdrcnt)
        msu_scnpos0     = msu_scnpos(recn - msu_h_hdrcnt)
        msu_qualind0    = msu_qualind(recn - msu_h_hdrcnt)
        msu_navstat0    = msu_navstat(recn - msu_h_hdrcnt)
        msu_attangtime0 = msu_attangtime(recn - msu_h_hdrcnt)
        msu_rollang0    = msu_rollang(recn - msu_h_hdrcnt)
        msu_pitchang0   = msu_pitchang(recn - msu_h_hdrcnt)
        msu_yawang0     = msu_yawang(recn - msu_h_hdrcnt)
        msu_scalti0     = msu_scalti(recn - msu_h_hdrcnt)
        msu_filler30    = msu_filler3(recn - msu_h_hdrcnt) 


        do i=1,5
          msu_filler10(i) = msu_filler1(i,recn - msu_h_hdrcnt)
        enddo

        do i=1,8
          msu_filler20(i) = msu_filler2(i,recn - msu_h_hdrcnt)
          msu_filler40(i) = msu_filler4(i,recn - msu_h_hdrcnt)
        enddo
        
        do i=1,230
          msu_filler50(i) = msu_filler5(i,recn - msu_h_hdrcnt)
        enddo

        do i=1,11
          do j=1,3
            msu_ang0(j,i) = msu_ang(j,i,recn - msu_h_hdrcnt)
          enddo
          do j=1,2
            msu_pos0(j,i) = msu_pos(j,i,recn - msu_h_hdrcnt)
          enddo
        enddo

        do i=1,msu_mxchn
          msu_scnlinqual0(i) = msu_scnlinqual(i,(recn-msu_h_hdrcnt))
        enddo

        do j=1,3
         do i=1,msu_mxchn
           msu_pmcal0(j,i)  = msu_pmcal(j,i,recn-msu_h_hdrcnt)
           msu_sdcal0(j,i)  = msu_sdcal(j,i,recn-msu_h_hdrcnt)
         enddo
        enddo

        do i=1,112
          msu_data0(i) = msu_data(i,recn - msu_h_hdrcnt)
        enddo

        write(msu_l1bu,rec=recn,iostat=ios) msu_dt
        if(ios.ne.0) then
         text='MSU1B record could not be written,  ios:'
         call ml_wti('error',text,ios)
         if (iprint.ge.1) write(msu_cllgu,*)
     &   '  MSU1B record could not be written'
         call aappexit(2)
        endif
 
      enddo

      return
      end
