C LIBS = f7nl1b satid f7ml f7tp f7cp f7gp auxaapp
!+
!NOM/NAME $Id: chk1btime.F 64 2007-01-17 15:05:08Z frna $
!  program chk1btime
! 
!
!BUT/PURPOSE 
! check & correct the scan line datation for a given 1B file
! that has been processed by aapp atovdc module
!
! It is a program to apply to hirs,msu,amsua and amsub files
! that may present bad time information when crossing midnight.
!
! Please never apply to nesdis files (they have missing records
! & aapp ones don't have because we have filled records when 
! scan lines are missing)
!
! The error in atovdc had been to use the same date information
! for all instuments. I preferred to correct the files than
! fixing the bug in the decommutation step.
!
! This program is dependant on 1B format structure.
! It trusts the time indicated in the 1st scan line of the file.
!
! 01/09/2006: At the recommendation of JMA, the method has been changed.
! Instead of trusting the first scan line, we now read in all the scan times
! and look for the first scan line that is in agreement with at least 90% of
! the other scans.
!
!
!DESCRIPTION 
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
! NWP SAF
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0     10 July 98  initial          Marcel Derrien
! 1.1     August 2000 add tests on header julian days   (PB)
! 1.2     30 Jan 2001 removed p_timang.h and timang.h and initialized
!                     the array steptl(.)   (TL et PB)
! 1.3     24-May-2004  Time correction with 0.01s accuracy, was 0.1s (A.Korpela) 
! 1.4     25-Jun-2004  Add MHS
! 1.5     17-Aug-2006  separate division of I2 by I4 parameter (reclg). NCA
! 1.6     01-Sep-2006  Use more robust method proposed by JMA 
! 1.7     06-Nov-2006  Relax to 0.04s in the section that searches for
!                      trustworthy scan lines. Prevents rejection of entire pass
!                      if a small time shift occurs. NCA
! 1.8     16-Nov-2006  Fix header end time when last scan has been corrected. NCA
! 1.9     02-Jan-2007  Correct the number of scans (was 1 too large). NCA
!
!EXIT CODES
!
!COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2006, EUMETSAT, All Rights Reserved.
!
      implicit none                                                     
!
!DECLARATIONS
!
!    
     
!commons
#include "avh1b.h"
#include "dcs1b.h"
#include "hrs1b.h"
#include "msu1b.h"
#include "ama1b.h"
#include "amb1b.h"
#include "mhs1b.h"
#include "reclopt.h"
!local variables
        integer lul1b                ! logical unit for 1B file
        integer ier0,ier1,ier2       ! i/o return codes
        integer ios,ios1,ios2        ! i/o return codes
        integer reclg ,data_type     ! record length, 1B data type code
        integer scn,recn             ! scan & record index 
        integer*2 scnlin             
        integer*2 scnlinyr
        integer*2 scnlindy
        integer*2 clockdrift
        integer   scnlintime
        integer iq(3)                ! instant year, day of year , ms
        character*256 filename       ! 1B filename
        double precision time_line_file,time_line_exp ! instant julian
        double precision time_line_min,time_line_max  ! 
        character*22 ic
        character*512 txt
        logical start_cor,end_cor !flag indicating that header must be corrected      
!
!
       character*3  h_siteid        ! site identification
       character*1  h_blank         ! 
       integer*2    h_l1bversnb     !level 1b format version number
       integer*2    h_l1bversyr     !level 1b format version year
       integer*2    h_l1bversdy     !level 1b format version day of year
       integer*2    h_reclg         !reserved for logical record length
       integer*2    h_blksz         !reserved for block size
       integer*2    h_hdrcnt        !count of header records in data set
       character*42 h_dataname      !data set name
       character*8  h_prblkid       !processing block identification
       integer*2    h_satid         !NOAA spacecraft identification code
       integer*2    h_instid        !instrument identification
       integer*2    h_datatyp       !data type code
!              (1 = LAC; 2 = GAC; 3 = HRPT; 4 = TIP; 5 = HIRS; 6 = MSU;
!               7 = SSU; 8 = DCS; 9 = SEM; 10 = AMSU-A; 11 = AMSU-B; 12 = MHS)
       integer*2    h_tipsrc        !TIP source code
       integer      h_startdatajd   !start of data set day count 
!                                        from 00h,1 Jan 1950
       integer*2    h_startdatayr   !start of data set year
       integer*2    h_startdatady   !start of data set day of year
       integer      h_startdatatime !start of data set UTC time of day in
!                                        milliseconds  
       integer      h_enddatajd   !end of data set day count 
!                                        from 00h,1 Jan 1950
       integer*2    h_enddatayr   !end of data set year
       integer*2    h_enddatady   !end of data set day of year
       integer      h_enddatatime !end of data set UTC time of day in
!                                        milliseconds  
       integer*2 h_filler0(3) 
       double precision steptl(12)  ! step time between two lines (sec)   
!
       integer jdate             ! julian day for header comparison
       double precision reftime1 ! julian instant of 1st line from header
       double precision reftime2 ! julian instant of 1st line from scan info
       double precision endtime  ! julian instant of end of data set from header
       data lul1b/10/
       character*8 datatyp(13)
       data datatyp/'  ','lac','gac','hrpt',
     &              'tip','hirs','msu','ssu',
     &              'dcs','sem','amsua','amsub','mhs'/

! ==================== JMA declarations from here ====================
       integer*4    maxscans
       parameter (maxscans=1000)  ! max number of scan times to store
       integer*4    iscn, jscn    ! counter for serching correct scan line time
       integer*4    kscn, nscn    ! number of scans
       double precision time_line_file_org(1000) ! scan line time of each scan
       real*4       rate          ! fraction of time consistent scans
! ==================== JMA declarations to here ====================


!functions
!
!     for dcs time increment is 32.0 msecs
!
      steptl(1) = 1.d0/6.d0
      steptl(2) = 0.d0  
      steptl(3) = 1.d0/6.d0
      steptl(4) = 0.d0
      steptl(5) = 6.4d0
      steptl(6) = 25.6d0
      steptl(7) = 32.0d0
      steptl(8) = 32.0d0
      steptl(9) = 0.d0
      steptl(10) = 8.d0 
      steptl(11) = (8.d0/3.d0)
      steptl(12) = (8.d0/3.d0)
      ier1=0
      ier2=1
!      
!     get l1b file name to check
!
      read (*,'(a)',iostat=ier0) filename
      write(txt,*) 'input file is ',filename
      call ml_wt('info',txt)
! 
!     open 1B with record length = 512      
! 
      if(ier0.ne.0) then
        call ml_wti('error','problem reading filename ios',ier0)
        call aappexit(2) 
      endif        
      open(lul1b,form='unformatted',access='direct',
     &    recl=512,status='old',iostat=ier1,file=filename)
      if(ier1.ne.0) then
        call ml_wti('error','problem opening file ios',ier1)
        call aappexit(2) 
      endif
! 
!     read the driving data
!         
      read(lul1b,rec=1,iostat=ier2)
     &  h_siteid,h_blank,
     &  h_l1bversnb,h_l1bversyr,h_l1bversdy,
     &  h_reclg,h_blksz,
     &  h_hdrcnt,h_filler0,
     &  h_dataname,
     &  h_prblkid,
     &  h_satid,
     &  h_instid,
     &  h_datatyp,
     &  h_tipsrc,
     &  h_startdatajd,
     &  h_startdatayr,
     &  h_startdatady,
     &  h_startdatatime,
     &  h_enddatajd,
     &  h_enddatayr,
     &  h_enddatady,
     &  h_enddatatime
       if(ier2.ne.0) then
         call ml_wti('error','problem reading header ios',ier2)
         call aappexit(2)
       endif
!
!      close the file, re-open it with the correct record length
!      read the scan lines & check time information
!      rewrite the scan if an error is detected
!      write a message to the log when a correction is applied
!
          
       close(lul1b,iostat=ios)
C       reclg=h_reclg/rldiv     !h_reclg is integer*2, rldiv is integer*4
       reclg=h_reclg
       reclg=reclg/rldiv
       data_type=h_datatyp
       if(data_type.eq.3) then
         call ml_wt(
     &     'info','data type indicates HRPT: no correction')
         stop
       endif
       open(lul1b,form='unformatted',access='direct',
     &     recl=reclg,status='old',iostat=ier1,file=filename)
       if(ier1.ne.0) then
         call ml_wti('error','problem 2nd open ios',ier1)
         call aappexit(2)
       endif          
       if (data_type.eq.5) then        ! hirs
         call ml_wt(
     &     'info','data type indicates HIRS')         
         call hrshdr(lul1b,ios)
       elseif(data_type.eq.6) then   ! msu
         call ml_wt(
     &     'info','data type indicates MSU')     
         call msuhdr(lul1b,ios)
       elseif(data_type.eq.8) then   ! dcs
         call ml_wt(
     &     'info','data type indicates DCS')     
         call dcshdr(lul1b,ios)
       elseif(data_type.eq.10) then  ! amsua
         call ml_wt(
     &     'info','data type indicates AMSU-A')
          call amahdr(lul1b,ios)
       elseif(data_type.eq.11) then  ! amsub
         call ml_wt(
     &     'info','data type indicates AMSU-B')
         call ambhdr(lul1b,ios)
       elseif(data_type.eq.12) then  ! mhs
         call ml_wt(
     &     'info','data type indicates MHS')
         call mhshdr(lul1b,ios)
       else
         call ml_wti('error','unknown 1B data_type',data_type)
         call aappexit(2)
       endif

!
! ========== JMA addition to search for a trustworthy scan time ===========
!
! read in all the scan line times
       ios = 0
       scn = 0
       do while (ios.eq.0.and.scn.lt.maxscans)
         scn = scn + 1
         recn = scn + h_hdrcnt
         read (lul1b, rec=recn,iostat=ios) scnlin , scnlinyr ,
     &            scnlindy , clockdrift , scnlintime
         if( ios.eq.0) then
           iq(1) = scnlinyr
           iq(2) = scnlindy
           iq(3) = scnlintime
           call xiqj(iq,time_line_file)
           time_line_file_org(scn) = time_line_file
         else
           if (scn.eq.1) then
             call ml_wti('error','problem reading 1st scan',ios)
             call aappexit(2)
           endif
         endif  ! end of reading data 
       enddo

! loop round scans, finding the first scan that is in agreement with
! at least 90% of the rest

       nscn = scn - 1
       reftime2 = 0
       iscn=0
      
       do while (iscn.lt.nscn.and.reftime2.eq.0)
         iscn = iscn + 1
         kscn = 0
         do jscn=1, nscn
           time_line_exp = time_line_file_org(iscn) + 
     &                     (jscn-iscn) * steptl(data_type)/86400.d0
           time_line_min = time_line_file_org(jscn) - 0.04 / 86400.d0
           time_line_max = time_line_file_org(jscn) + 0.04 / 86400.d0
           if ( time_line_exp .ge. time_line_min .and. 
     &          time_line_exp .le. time_line_max ) then
             kscn = kscn + 1
           endif
         end do
         rate = float(kscn)/float(nscn)
         if ( rate >= 0.9 ) then

! work out the 1st line scan time from this trustworthy scan line time
           reftime2 = time_line_file_org(iscn) - 
     &                 (iscn-1) * steptl(data_type)/86400.d0
           if (iscn.ne.1) then
             call xijc(time_line_file_org(iscn),ic)
             write(txt,*)'time of scan',iscn,' is ',ic
             call ml_wt('warning',txt)
             write(txt,*)'time of 1st scan is corrected using scan',iscn
             call ml_wt('warning',txt)
           endif
         else

! not yet found a trustworthy scan
           call xijc(time_line_file_org(iscn),ic)
           write(txt,*)'time of scan',iscn,' is ',ic,
     &                 ', must be corrected '
           call ml_wt('warning',txt)
         endif
       end do

       if ( reftime2.eq.0 ) then
         call ml_wt('error','NO SCAN LINE TIME IS TRUSTWORTHY!')
         call aappexit(99)
       endif
! ==================== end of JMA addition ====================

!
!      check start of data time in header           
!      ----------------------------------

       iq(1) = h_startdatayr
       iq(2) = h_startdatady
       iq(3) = h_startdatatime
       call xiqj(iq,reftime1)
       call xijc(reftime1,ic)
       write(txt,*) 'start time in header ',ic
       call ml_wt('info',txt) 

C Prior to JMA modification, the first scan was used to obtain reftime2
C       recn = 1 + h_hdrcnt
C       read (lul1b, rec=recn,iostat=ios) scnlin , scnlinyr ,
C     &            scnlindy , clockdrift , scnlintime
C       if(ios.ne.0) then
C         call ml_wti('error','problem reading 1st scan',ios)
C         call aappexit(2)
C       endif              
C       iq(1) = scnlinyr
C       iq(2) = scnlindy
C       iq(3) = scnlintime 
C       call xiqj(iq,reftime2)

       call xijc(reftime2,ic)
       write(txt,*)'time of 1st scan is  ',ic
       call ml_wt('info',txt)
       ios = 0
       scn = 0
       time_line_min =   reftime2 -  0.01 / 86400.d0      
       time_line_max =   reftime2 +  0.01 / 86400.d0                
       start_cor=.false.
       if(reftime1.lt.time_line_min.or.
     &       reftime1.gt.time_line_max) then
         call ml_wt('warning',
     &             'start data time header/scan don''t agree')
         start_cor=.true.
       endif
!
!      tests the julian day of start time 
!
       jdate = int(reftime2)
       if(jdate .ne. h_startdatajd) then
          call ml_wt('warning',
     &              'start data julian day header/scan don''t agree')
          start_cor = .true.
       endif

!
!      read in each scan line and modify times if required
!      --------------------------------------------------- 
       do while (ios.eq.0)
         scn = scn + 1
         recn = scn + h_hdrcnt
         read (lul1b, rec=recn,iostat=ios) scnlin , scnlinyr ,
     &            scnlindy , clockdrift , scnlintime
         if( ios.eq.0) then
           iq(1) = scnlinyr
           iq(2) = scnlindy
           iq(3) = scnlintime
           call xiqj(iq,time_line_file)
           time_line_exp =   reftime2 + 
     &        (scn-1) * steptl(data_type)/86400.d0
           time_line_min =   time_line_exp -  0.01 / 86400.d0      
           time_line_max =   time_line_exp +  0.01 / 86400.d0                
           if(time_line_file.lt.time_line_min.or.
     &       time_line_file.gt.time_line_max) then
!
!            compute good time information from expected
!            correct the file
!            write a message to the log
!
             call xijq(time_line_exp,iq)
             ios1=0
             ios2=0
             if (data_type.eq.5) then      ! hirs
               call hrsdtr(lul1b,scn,ios1)
               hrs_scnlinyr = iq(1)
               hrs_scnlindy = iq(2)
               hrs_scnlintime = iq(3)              
               call hrsdtw(lul1b,scn,ios2)
             elseif(data_type.eq.6) then   ! msu
               call msudtr(lul1b,scn,ios1)
               msu_scnlinyr = iq(1)
               msu_scnlindy = iq(2)
               msu_scnlintime = iq(3)              
               call msudtw(lul1b,scn,ios2)
             elseif(data_type.eq.8) then   ! dcs
               call dcsdtr(lul1b,scn,ios1)
               dcs_mjfryr = iq(1)
               dcs_mjfrdy = iq(2)
               dcs_mjfrtime = iq(3)              
               call dcsdtw(lul1b,scn,ios2)
             elseif(data_type.eq.10) then  ! amsua
               call amadtr(lul1b,scn,ios1)
               ama_scnlinyr = iq(1)
               ama_scnlindy = iq(2)
               ama_scnlintime = iq(3)              
               call amadtw(lul1b,scn,ios2)              
             elseif(data_type.eq.11) then  ! amsub
               call ambdtr(lul1b,scn,ios1)            
               amb_scnlinyr = iq(1)
               amb_scnlindy = iq(2)
               amb_scnlintime = iq(3)              
               call ambdtw(lul1b,scn,ios2)
             elseif(data_type.eq.12) then  ! mhs
               call mhsdtr(lul1b,scn,ios1)            
               mhs_scnlinyr = iq(1)
               mhs_scnlindy = iq(2)
               mhs_scnlintime = iq(3)              
               call mhsdtw(lul1b,scn,ios2)
             endif
             if(ios1.ne.0.) then
                call aappexit(2)
             elseif(ios2.ne.0) then
               call ml_wti('error','re-writing 1B file',ios2)
               call aappexit(2)
             endif
!
!            information to the log
!            
             call xijc(time_line_file,ic)
             write(txt,*)'scan ',scn,' time in the file',ic
             call ml_wt('info',txt)
             call xijc(time_line_exp,ic)
             write(txt,*)'time expected is ',ic
             call ml_wt('info',txt)
             call ml_wt('warning','TIME CORRECTION HAS BEEN DONE')
             time_line_file = time_line_exp   !Used for end time in header
          endif  !  end of bad time case
        endif  ! end of reading data 
      enddo
!
!     check end of data time in header
!     --------------------------------
 
      call xijc(time_line_file,ic)
      write(txt,*)'time of last scan is ',ic
      call ml_wt('info',txt)       
      iq(1) = h_enddatayr
      iq(2) = h_enddatady
      iq(3) = h_enddatatime
      call xiqj(iq,endtime)
      call xijc(endtime,ic)
      write(txt,*)'end   time in header ',ic
      call ml_wt('info',txt)                                            
      end_cor=.false.
      if(endtime.lt.time_line_min.or.
     &     endtime.gt.time_line_max) then 
        call xijc(time_line_file,ic)
        call ml_wt('warning','end data time header/scan don''t agree')
        end_cor = .true.
      endif
!
!     tests the julian day of end time 
!
      jdate = int(time_line_file)
      if(jdate .ne. h_enddatajd) then
         call ml_wt('warning',
     &              'end data julian day header/scan don''t agree')
         end_cor = .true.
      endif
!
      if (start_cor) then
        call xijq(reftime2,iq)
        h_startdatajd = int ( reftime2 )
        h_startdatayr = iq(1)
        h_startdatady = iq(2)
        h_startdatatime = iq(3)
        if(data_type.eq.5) then     ! hirs
          hrs_h_startdatajd = h_startdatajd
          hrs_h_startdatayr = h_startdatayr
          hrs_h_startdatady = h_startdatady
          hrs_h_startdatatime = h_startdatatime
          call hrshdw(lul1b,ios)
        elseif(data_type.eq.6) then  ! msu
          msu_h_startdatajd = h_startdatajd
          msu_h_startdatayr = h_startdatayr
          msu_h_startdatady = h_startdatady
          msu_h_startdatatime = h_startdatatime
          call msuhdw(lul1b,ios)
        elseif(data_type.eq.8) then   ! dcs
          dcs_h_startdatajd = h_startdatajd
          dcs_h_startdatayr = h_startdatayr
          dcs_h_startdatady = h_startdatady
          dcs_h_startdatatime = h_startdatatime
          call dcshdw(lul1b,ios)
        elseif(data_type.eq.10) then   ! amsua
          ama_h_startdatajd = h_startdatajd
          ama_h_startdatayr = h_startdatayr
          ama_h_startdatady = h_startdatady
          ama_h_startdatatime = h_startdatatime
          call amahdw(lul1b,ios)
        elseif(data_type.eq.11) then   ! amsub
          amb_h_startdatajd = h_startdatajd
          amb_h_startdatayr = h_startdatayr
          amb_h_startdatady = h_startdatady
          amb_h_startdatatime = h_startdatatime
          call ambhdw(lul1b,ios)
        elseif(data_type.eq.12) then   ! mhs
          mhs_h_startdatajd = h_startdatajd
          mhs_h_startdatayr = h_startdatayr
          mhs_h_startdatady = h_startdatady
          mhs_h_startdatatime = h_startdatatime
          call mhshdw(lul1b,ios)
        endif
        call ml_wt('info','header start datation has been corrected')
      endif
      if(end_cor) then
        call xijq(time_line_file,iq)
        h_enddatajd = int ( time_line_file )
        h_enddatayr = iq(1)
        h_enddatady = iq(2)
        h_enddatatime = iq(3)        
        if(data_type.eq.5) then     ! hirs
          hrs_h_enddatajd = h_enddatajd
          hrs_h_enddatayr = h_enddatayr
          hrs_h_enddatady = h_enddatady
          hrs_h_enddatatime = h_enddatatime
          call hrshdw(lul1b,ios)
        elseif(data_type.eq.6) then   ! msu
          msu_h_enddatajd = h_enddatajd
          msu_h_enddatayr = h_enddatayr
          msu_h_enddatady = h_enddatady
          msu_h_enddatatime = h_enddatatime
          call msuhdw(lul1b,ios)
        elseif(data_type.eq.8) then   ! dcs
          dcs_h_enddatajd = h_enddatajd
          dcs_h_enddatayr = h_enddatayr
          dcs_h_enddatady = h_enddatady
          dcs_h_enddatatime = h_enddatatime
          call dcshdw(lul1b,ios)
        elseif(data_type.eq.10) then   ! amsua
          ama_h_enddatajd = h_enddatajd
          ama_h_enddatayr = h_enddatayr
          ama_h_startdatady = h_enddatady
          ama_h_enddatatime = h_enddatatime
          call amahdw(lul1b,ios)
        elseif(data_type.eq.11) then   ! amsub
          amb_h_enddatajd = h_enddatajd
          amb_h_enddatayr = h_enddatayr
          amb_h_enddatady = h_enddatady
          amb_h_enddatatime = h_enddatatime
          call ambhdw(lul1b,ios)
        elseif(data_type.eq.12) then   ! mhs
          mhs_h_enddatajd = h_enddatajd
          mhs_h_enddatayr = h_enddatayr
          mhs_h_enddatady = h_enddatady
          mhs_h_enddatatime = h_enddatatime
          call mhshdw(lul1b,ios)
        endif
        call ml_wt('info','header end datation has been corrected')
      endif
      stop
      end               
