C LIBS = decom f7nl1b satid f7ml f7tp f7cp f7gp auxaapp
!+
!NOM/NAME $Id: decommutation.F 47 2006-08-21 08:44:41Z frna $
!
	program decommutation
!
!BUT/PURPOSE 
! read local hrpt data on a file and call hrptdc
!
!DESCRIPTION 
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
! Marcel Derrien
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0       06/95     initial          Marcel Derrien
! 1.1       12/3/04   Byte swap if needed. Nigel Atkinson
! 1.2       23/11/04  Remove unused variables and add UKMO mods as comments
! 1.3       17/08/05  Ignore duplicate HRPT minor frames. NCA
! 1.4       05/09/05  Allow 256 character file names (was 80). NCA
!
!EXIT CODES
!
!  0 if OK
!  2 if I/O problem
! 11 if invalid option read by dcin module
! 12 if bad satid number passed to dcsget
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!DECLARATIONS
!
!    
!commons
#include "hrptdc.h"
#include "reclopt.h"

!local variables
        integer*2 hrptab(11090)        !The HRPT minor frame
        integer*2 hrptab_read(11090)   !Data read in. Change to 11095 for UKMO
        equivalence (hrptab,hrptab_read(1))   !hrptab_read(6) for UKMO

        integer ios1,ios,nblines,nrec,k
        integer nbmf, hrptmf, prev_timecode(3)
        data prev_timecode/-1,-1,-1/
        
        character*256 fic
!       character*80 fic
        integer loc_rl

        character*1 hrptabc(22180),ctmp
        equivalence (hrptabc(1),hrptab(1))
        integer*2 sync1swap
        data sync1swap/Z'8402'/     !normal sync is Z'0284'
        integer*2 tenbits
        data tenbits/Z'03FF'/       !mask for required bits
        integer*2 tenbitswap
        data tenbitswap/Z'FF03'/    !byte swapped bit mask
!-
        read(*,'(a)') fic
        call ml_wt('info',fic)
        loc_rl=22180/rldiv
       
	open(unit=20,iostat=ios1,status='old',form='unformatted',
     &    access ='direct',recl=loc_rl,file=fic)
        if(ios1.eq.0) then
          nblines=100
	  ios=0
	  nrec=0
!	  do j=1,nblines
        
          do while (ios.eq.0)
	    nrec=nrec+1
            read(20,iostat=ios,rec=nrec) hrptab_read

!Byte swap if needed
            if (IAND(hrptab(1),tenbitswap).eq.sync1swap) then
              do k=1,22180,2
                ctmp=hrptabc(k)
                hrptabc(k)=hrptabc(k+1)                
                hrptabc(k+1)=ctmp                
              enddo
            endif

!Set leading bits to zero if not already
            if (IAND(hrptab(1),tenbits).ne.hrptab(1)) THEN
              do k=1,11090
                hrptab(k) = IAND(hrptab(k),tenbits)
              enddo
            endif

            if(ios.eq.0) then

!Check for duplicate HRPT minor frames with the same time stamp as previous one
              hrptmf = iand(hrptab(7)/128,3)
              IF (hrptmf.GE.1.AND.hrptmf.LE.3) THEN
                IF (hrptab(12).NE. prev_timecode(hrptmf)) THEN
                  prev_timecode(hrptmf) = hrptab(12)
                  nbmf = 1
                  call hrptdc(hrptab,nbmf)   !OK - process this frame
                ELSE
                  call ml_wt('info','skip duplicate frame')
                ENDIF
              ENDIF

            else

	      nbmf = 0
              call hrptdc(hrptab,nbmf)
            endif
          enddo 
       else
          call ml_wti('error','when opening hrpt file iostat '
     &         ,ios1)
	  call aappexit(2)
     
       endif	    
       stop
       end
