#!/bin/ksh
#$Id: decommutation.ksh,v 1.2 2005/04/15 13:39:53 frna Exp $
#
#modified 23/11/04: At end, only re-name files that exist. This allows more
#               flexibility in which instruments to process. Also, don't
#               explicitly move to $WRK directory (AAPP_RUN is already in $WRK).
#modified 01/02/06: Change config file P. Marguinaud
#         18/08/06: Add $STATION_ID to end of decommutation.par   NCA


. "$AAPP_PREFIX/ATOVS_CONF"

chk1btime()
{
# tool to correct scan line datation for level 1 b files

FILE=$1
chk1btime.exe << EOF >>decommutation.log 2>&1
${FILE}
EOF

status=$?
if [ ${status} != 0 ]; then
     echo "chk1btime failed on ${FILE}: status = ${status}"
     exit 1
fi     
}


if [ $# -ne 3 ]; then
  echo " Three Arguments expected!!"
  echo " Arguments = A_TOVS; parameter file; path of HRPT file"
  exit 1
else
  dec_inp=${3}
fi

rm -f *.l1b decommutation.log decommutation.inp ${FORT}*
echo ${dec_inp} > decommutation.inp
echo parameter file = ${2}
cat ${2} >> decommutation.inp
echo $STATION_ID >> decommutation.inp        #2-character identifier

FILE_COEF=${PAR_CALIBRATION_COEF}/amsua/amsua_clparams.dat
ln -s ${FILE_COEF} ${FORT}50
decommutation.exe < decommutation.inp >decommutation.log 2>&1

status=$?
if [ ${status} != 0 ]; then
  rm -f ${FORT}*
  exit 1
fi  

if [ -f ${FORT}11 ]; then
  mv ${FORT}11 hrsn.l1b
  chk1btime hrsn.l1b
fi
if [ -f ${FORT}12 ]; then
  mv ${FORT}12 msun.l1b
  chk1btime msun.l1b
fi
if [ -f ${FORT}13 ]; then
  mv ${FORT}13 dcsn.l1b
  chk1btime dcsn.l1b
fi
if [ -f ${FORT}14 ]; then
  mv ${FORT}14 hrpt.l1b  #hrpt.l1b does not require chk1btime correction
fi
if [ -f ${FORT}15 ]; then
  mv ${FORT}15 aman.l1b
  chk1btime aman.l1b
fi
if [ -f ${FORT}16 ]; then
  mv ${FORT}16 ambn.l1b
  chk1btime ambn.l1b
fi

#if [ ${1} = ATOVS ]; then
#  mv ${FORT}15 ${WRK}/aman.l1b
#  chk1btime ${WRK}/aman.l1b
#  mv ${FORT}16 ${WRK}/ambn.l1b
#  chk1btime ${WRK}/ambn.l1b
#else
#  mv ${FORT}12 ${WRK}/msun.l1b
#  chk1btime ${WRK}/msun.l1b
#fi  
#  mv ${FORT}11 ${WRK}/hrsn.l1b
#  chk1btime ${WRK}/hrsn.l1b
#  mv ${FORT}13 ${WRK}/dcsn.l1b
#  chk1btime ${WRK}/dcsn.l1b
#  mv ${FORT}14 ${WRK}/hrpt.l1b
## hrpt.l1b does not require chk1btime correction  

rm -f ${FORT}* decommutation.inp

#-------------------------------------------------------------------------------
#                         -- CVS MODIFICATION HISTORY --
#-------------------------------------------------------------------------------
#$Log: decommutation.ksh,v $
#Revision 1.2  2005/04/15 13:39:53  frna
#Merge AAPP5Dev with main trunk
#
#Revision 1.1.1.1.4.2  2004/12/02 08:57:15  frna
#Don't move output files to $WRK
#
#Revision 1.1.1.1.4.1  2004/11/23 11:28:52  frna
#At end, only re-name files that exist
#
