!#############################################################################
!+
!ROUTINE     AMANAG_OUT   SUBROUTINE  FORT77
!
!PURPOSE     Copy AMSU analogue telemetry to 1B variables
!
!VERSION     1.0      10/01/97    N. Atkinson
!            1.1      6/5/98      N. Atkinson Enable AMSU-B analog validity
!                                             flag.
!            1.2      18/12/98    N. Atkinson Increase size of AMSU-B analogs
!            1.3      31/7/03     N. Atkinson Add MHS
!            1.4      14/4/04     N. Atkinson Bug fix in mhs_survtemp & mhs_stx
!
!ARGUMENTS
!   integer*2  amanag(71)     -- AMSU analogue data (A1, A2 and B)
!   integer*2  amvalid(71)    -- AMSU analogue validity, 1 if invalid
!   logical*1  mhspresent     -- Set to TRUE for NOAA-N,N'
!#############################################################################

      subroutine amanag_out(amanag,amvalid,mhspresent)
!
      implicit none
c-----------------------------------------
c
c
!
!arguments
      integer*2 amanag(71),amvalid(71)
      logical*1 mhspresent
      
#include "ama1b.h"
#include "amb1b.h"
#include "mhs1b.h"

!local variables
      integer*2 i
      integer*4 j
!
      ama1_ahkval = 0   !Initialize
      ama2_ahkval = 0   !Initialize
      amb_ahkval = 0   !Initialize
!
!AMSU-A1
      j = 1
      do i=1,28 
        j = j*2                          !Bit 0 of ama1_ahkval is zero-fill
        ama1_ahk(i) = char(amanag(i))
        ama1_ahkval = ama1_ahkval + amvalid(i)*j
      enddo
!
!AMSU-A2
      j = 1
      do i=1,16 
        j = j*2
        ama2_ahk(i) = char(amanag(i+28))
        ama2_ahkval = ama2_ahkval + amvalid(i+28)*j
      enddo
!
!AMSU-B
      if (.not. mhspresent) then
        j = 1
        do i=1,27
          j = j*2
          amb_ahk(i) = amanag(i+44)
          amb_ahkval = amb_ahkval + amvalid(i+44)*j
        enddo
      else
!
!MHS
! The 3 analog channels for MHS are in the same location as AMSU-B analog 
! channels 11-13. Fill bits 6-8 in the validity flag (bits 0-5 being for
! Dig B). Don't bother with Dig B for AAPP.
!
        j = 32
        do i=11,13
          j = j*2
          mhs_survtemp(i-10) = amanag(i+44)
          mhs_ahkval = mhs_ahkval + amvalid(i+44)*j
        enddo
!
! Now do the transmitter status words (AMSU-B analog words 19-27)
!
        do i=19,27
          j = j*2
          mhs_stx(i-18) = amanag(i+44)
          mhs_ahkval = mhs_ahkval + amvalid(i+44)*j
        enddo
      endif

      return
      end
