C!
C!ROUTINE          AMAST  SUBROUTINE  FORT77
C!
C!PURPOSE          GET AMSU-A INSTRUMENT STATUS
C!
C!DESCRIPTION      This routine extracts the AMSU-A1 & A2 instrument
C!                 status for the current scan line and updates the status
C!                 array for the 1A header.
C!
C!VERSION          1.01   250895  R.W.SAUNDERS
C!                 1.02   180996  N.C.ATKINSON
C!                 1.03   060598  N.C.ATKINSON
C!                 1.04   March 03 T.LABROT
C!      f90 test : I.AND.J not correct and modified by IAND(I,J) 
C!                 I.OR.J not correct and modified by IOR(I,J)
C!                 Changed  type XXX/yyy/
C!                      to  type XXX
C!                          parameter(XXX=yyy)
C!                 1.7    17/01/2006 : Philippe Marguinaud : change hex constants from
C!                                    '1234'X to Z'1234'
C!                 1.8    17/08/2006 Change integer Z'xxxx' to Z'0000xxxx'. NCA 
C!
C!ARGUMENTS
C!      INTEGER*2 AMSUA1(620) -- AMSU-A1 scan line
C!      INTEGER*2 AMSUA2(156) -- AMSU-A2 scan line
C!      INTEGER A1STAT       -- AMSU-A1 Instrument status
C!      INTEGER A2STAT       -- AMSU-A2 Instrument status
C!      INTEGER AERR         -- AMSU-A Error flag  0 = OK
C!
C!REFERENCES       ATOVDC design document version 1.5 30 May 95
C!
C!CHANGES          01 Modified to reflect change in 1B dataset 25/8/95 rws
C!                 02 Only set error flag if A1 and A2 both bad        NCA
C!                 03 Mod to header status
C!
C!#########################################################################
C! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
C!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
C!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
C!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
C!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
C!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
C!#########################################################################
C!
      SUBROUTINE AMAST(AMSUA1,AMSUA2,A1STAT,A2STAT,AERR)
C!
      IMPLICIT NONE
c-----------------------------------------
c
c
C!
      INTEGER*2 AMSUA1(620),AMSUA2(156)     ! AMSU-A scan line
      INTEGER*4 A1STAT,A2STAT               ! Instrument status
      INTEGER*4 AERR                        ! Error codes (0=OK)
      INTEGER*4 A1NORM,A2NORM      ! AMSU-A normal status,excluding space view,
      PARAMETER(A1NORM=Z'00000206')    ! PLLO (for A2) and unused bits
      PARAMETER(A2NORM=Z'00000206')
                      
      INTEGER*4 ZA                        ! HEX Mask to zero space view
      PARAMETER(ZA=Z'1E16')
        
      INTEGER*4 ZAMSU                     ! Bit mask: do not use scan
      PARAMETER(ZAMSU=Z'80000000')
      
      INTEGER*4 A1TMP,A2TMP
C!
      A1STAT = AMSUA1(3)
      A1TMP = IAND(A1STAT,ZA)
      A2STAT = AMSUA2(3)
      A2TMP = IAND(A2STAT,ZA)
      IF (A1TMP .NE. A1NORM. AND. A2TMP .NE. A2NORM)THEN
        AERR = IOR(AERR,ZAMSU)
      ENDIF
C!
!      A1STAT = A1STAT*65536   !Remove these lines 6/5/98
!      A2STAT = A2STAT*65536
C!
      RETURN
      END
