C!
C!ROUTINE          AMBST  SUBROUTINE  FORT77
C!
C!PURPOSE          GET AMSU-B INSTRUMENT STATUS
C!
C!DESCRIPTION      This routine extracts the AMSU-B instrument status for
C!                 all three scan lines and updates the status array for the
C!                 1A header. The header bits are defined as follows:
C!                        Bits 15-6    zero fill
C!                        Bit  5       Ch 18-20     0=on,     1=off
C!                        Bit  4       Ch 17        0=on,     1=off
C!                        Bit  3       Ch 16        0=on,     1=off
C!                        Bit  2       Survival htr 0=off,    1=on
C!                        Bit  1       Inst Power   0=on,     1=off
C!                        Bit  0       Scan mode    0=normal, 1=not normal
C!
C!VERSION          1.01   151195  R.W.SAUNDERS
C!                 1.02   180996  N.C.Atkinson
C!                 1.03   March 03 T.LABROT 
C!   f90 test: I.AND.J not correct and modified by IAND(I,J)
C!             I.OR.J not correct and modified by IOR(I,J)
C!             I and J must have the same type.
C!             add working variable for IAND/IOR tests
C!             Changed  type XXX/yyy/
C!                   to type XXX
C!                      parameter(XXX=yyy) or data/yyy/
C!                 1.04  17/01/2006 : Philippe Marguinaud : change hex constants from
C!                                   '1234'X to Z'1234'
C!
C!ARGUMENTS
C!      INTEGER*2 AMSUB(1950) -- AMSU-B scan line
C!      INTEGER INSTST(3)    -- AMSU-B Instrument status 0 = OK
C!      INTEGER BERR(3)      -- AMSU-B Error flag  0 = OK
C!
C!REFERENCES       ATOVDC design document version 1.9 Nov 95
C!
C!CHANGES          01 None
C!                 02 Corrections, and only set error flag if all channels off
C!
C!#########################################################################
C! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
C!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
C!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
C!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
C!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
C!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
C!#########################################################################
C!
      SUBROUTINE AMBST(AMSUB,INSTST,BERR)
C!
      IMPLICIT NONE
c-----------------------------------------
c
c
C!
      INTEGER*2 AMSUB(1950)             ! 3 AMSU-B scan lines
      INTEGER*4 INSTST(3)               ! Instrument status
      INTEGER*2 JINST                   ! Instrument status
      INTEGER*4 BERR(3)                 ! Error codes (0=OK)
      INTEGER*2 BNORM                   ! AMSU-B normal status
      PARAMETER(BNORM=Z'705')
                
      INTEGER*2 B707                   ! HEX Mask
      PARAMETER(B707=Z'707')
      
      INTEGER*4 ZAMSU                  ! Bit mask: do not use scan
      PARAMETER(ZAMSU=Z'80000000')
      
      INTEGER*4 ISCAN,J,BIT

      INTEGER*2 I2WORD  ! working variable for IAND/IOR tests
      INTEGER*2 J2WORD  ! working variable for IAND/IOR tests
      INTEGER*4 I4WORD  ! working variable for IAND/IOR tests
      INTEGER*4 J4WORD  ! working variable for IAND/IOR tests
      

      INTEGER*2 IOFF(6)            ! AMSU-B status offsets
      DATA IOFF/1,2,4,256,512,1024/
      INTEGER*4 JOFF(6)            ! 1A header status offsets
      DATA JOFF/2,4,1,8,16,32/     
C!
      DO 10 ISCAN = 1 , 3
        INSTST(ISCAN)=0
        J = 576 + (ISCAN-1)*650 + 2                     ! A02
        JINST = IAND(AMSUB(J),B707)
        IF (JINST .NE. BNORM)THEN
          DO 5 BIT = 1 , 6
           I2WORD = IAND(JINST,IOFF(BIT))
           J2WORD = IAND(BNORM,IOFF(BIT))
           IF( I2WORD .NE. J2WORD ) THEN
            INSTST(ISCAN) = IOR(INSTST(ISCAN),JOFF(BIT))
           ENDIF
   5      CONTINUE
C! Set the error flag if all channels are off or instrument power off or not
C! scan normal
          I4WORD = IAND(INSTST(ISCAN),56)
          J4WORD = IAND(INSTST(ISCAN),3)
          IF( I4WORD .EQ. 56 .OR. J4WORD .NE. 0) 
     &       BERR(ISCAN)= IOR(BERR(ISCAN),ZAMSU)
        ELSE
          INSTST(ISCAN) = 0
        ENDIF
  10  CONTINUE

      RETURN
      END
