C!
C!ROUTINE          AMSADC  SUBROUTINE  FORT77
C!
C!PURPOSE          TO DECOMMUTATE AMSU-A DATA AND PUT INTO 1A ARRAYS
C!
C!DESCRIPTION      This routine takes the AMSU-A1 and AMSU-A2 minor
C!                 frames packed into the arrays AMSUA1 and AMSUA2 and
C!                 checks the synch bytes are OK and then extracts the
C!                 data and places it in the 1A data array. In addition
C!                 the scan line times, quality indicator are added.
C!                 Provision is made to also include the AMSU-A analogue
C!                 data from the TIP minor frames but this is not yet
C!                 implemented.
C!
C!VERSION          1.01   151195  R.W.SAUNDERS
C!                 1.02   180996  R.W.SAUNDERS
C!                 1.03   060598  N.C.ATKINSON
C!                 1.04   March 03 T.LABROT and P.BRUNEL
C!   f90 test: I.AND.J not correct and modified by IAND(I,J)
C!             I and J must have the same type.
C!             add working variable for IAND/IOR tests
C!             Changed  type XXX/yyy/
C!                  to  type XXX
C!                      parameter(XXX=yyy)
C!                 1.05   Nov 04  N.C.ATKINSON Remove redundant arguments
C!                 1.06   17/01/2006 : Philippe Marguinaud : change hex constants from
C!                                     '1234'X to Z'1234'
C!
C!ARGUMENTS
C!      INTEGER*4 OPTN(12)   -- OPtion numbers
C!      INTEGER*2 AMSUA1(620)  -- AMSU-A1 scan line
C!      INTEGER*2 AMSUA2(156) -- AMSU-A2 scan line
C!      INTEGER MSECS     -- TIP msecs from midnight
C!      INTEGER AERR      -- AMSU-A Error flag (0=OK)
C!
C!REFERENCES       ATOVDC design document version 1.9 Nov 95
C!
C!CHANGES          01 None
C!                 02 Correct array sizes
C!                 03 Remove trailing zero from digital-A data, and also
C!                    the antenna position error bit. This is to conform to 
C!                    the NESDIS 1B product.
C!
C!#########################################################################
C! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
C!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
C!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
C!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
C!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
C!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
C!#########################################################################
C!
      SUBROUTINE AMSADC(OPTN,AMSUA1,AMSUA2,MSECS,AERR)
C!
      IMPLICIT NONE
c-----------------------------------------
c
c
C!
      INTEGER*2 AMSUA1(620),AMSUA2(156) ! AMSU-A data for 1 scan line
      INTEGER*4 MSECS                   ! TIP msecs from midnight
      INTEGER*4 AERR            ! Error code (0=OK)
      INTEGER*4 OPTN(12)         ! Option numbers
      INTEGER*4 I,IPOS,ICH,IVEC
      INTEGER*2 SYNC1              ! Sync words at start of frame
      PARAMETER(SYNC1=Z'FFFF')
      INTEGER*2 SYNC2              ! Sync words at start of frame
      PARAMETER(SYNC2=Z'FF00')       

      CHARACTER*80 TXT
      INTEGER*2 I2WORD  ! working variable for IAND/IOR tests
      INTEGER*2 J2WORD  ! working variable for IAND/IOR tests
C!
#include "ama1b.h"
#include "hrptdc.h"
C!...................................................................
C!          AMSU-A1 data extraction
      IF ( OPTN(3) .EQ. 1 )THEN
C!
C! First check sync bytes are OK
C!
         I2WORD = IAND(AMSUA1(1),SYNC1)
         J2WORD = IAND(AMSUA1(2),SYNC2)
      IF(I2WORD.NE.SYNC1.OR.
     &   J2WORD.NE.SYNC2) THEN
!        AERR = AERR.OR.(-2147483648)      !Set bit 31
         call gp_sb32(aerr,31,1)
C!        WRITE(LULOG,101)MSECS
         write(txt,101)msecs
         call ml_wt('info',txt)
        ama_qualind = AERR
      ENDIF
C!
      ama1_sync = AMSUA1(1)
      ama1_id = AMSUA1(2)
      ama1_dhk(1) = AMSUA1(3)
      ama1_dhk(2) = AMSUA1(4)
      DO 11 IPOS = 1 , 30
      DO 10 ICH = 1 , 4   !Antenna positions (2 antennae, 2 measurements)
       IVEC = 4 + ICH + (IPOS-1)*17
       ama1_earth_data(ICH,IPOS) = MOD(AMSUA1(IVEC)+65536,65536)/4
  10  CONTINUE
      DO 11 ICH = 5 , 17  !Channels 3 to 15
       IVEC = 4 + ICH + (IPOS-1)*17
       ama1_earth_data(ICH,IPOS) = MOD(AMSUA1(IVEC)+65536,65536)/2
  11  CONTINUE
C!
      DO 20 I = 1 , 4     !Positions (2 antennae, 2 measurements)
       ama1_coldcal(I) = MOD(AMSUA1(I+514)+65536,65536)/4
       ama1_warmcal(I) = MOD(AMSUA1(I+590)+65536,65536)/4
  20  CONTINUE
      DO 21 I = 5 , 30    !Channels 3 to 15 (2 measurements)
       ama1_coldcal(I) = MOD(AMSUA1(I+514)+65536,65536)/2
       ama1_warmcal(I) = MOD(AMSUA1(I+590)+65536,65536)/2
  21  CONTINUE
C!
      DO 30 I = 1 , 46
       ama1_temps(i) = MOD(AMSUA1(I+544)+65536,65536)/2
  30  CONTINUE
      ENDIF
C!...................................................................
C!          AMSU-A2 data extraction
      IF ( OPTN(4) .EQ. 1)THEN
C!
C! First check sync bytes are OK
C!
         I2WORD = IAND(AMSUA2(1),SYNC1)
         J2WORD = IAND(AMSUA2(2),SYNC2)
      IF(I2WORD.NE.SYNC1.OR.
     &   J2WORD.NE.SYNC2) THEN
!        AERR = AERR.OR.(-2147483648)     !Set bit 31
         call gp_sb32(aerr,31,1)
C!        WRITE(LULOG,102)MSECS
         write(txt,102)msecs
         call ml_wt('info',txt)
        ama_qualind = AERR
      ENDIF
C!
      ama2_sync = AMSUA2(1)
      ama2_id = AMSUA2(2)
      ama2_dhk(1) = AMSUA2(3)
      ama2_dhk(2) = AMSUA2(4)
      DO 41 IPOS = 1 , 30
      DO 40 ICH = 1 , 2       !Position (1 antenna, 2 readings)
       IVEC = 4 + ICH + (IPOS-1)*4
       ama2_earth_data(ICH,IPOS) = MOD(AMSUA2(IVEC)+65536,65536)/4
  40  CONTINUE
      DO 41 ICH = 3 , 4       !Channels 1 and 2
       IVEC = 4 + ICH + (IPOS-1)*4
       ama2_earth_data(ICH,IPOS) = MOD(AMSUA2(IVEC)+65536,65536)/2
  41  CONTINUE
C!
      DO 50 I = 1 , 2         !Position (1 antenna, 2 readings)
       ama2_coldcal(I) = MOD(AMSUA2(I+124)+65536,65536)/4
       ama2_warmcal(I) = MOD(AMSUA2(I+150)+65536,65536)/4
  50  CONTINUE
      DO 51 I = 3 , 6         !Channels 1 and 2 (2 readings)
       ama2_coldcal(I) = MOD(AMSUA2(I+124)+65536,65536)/2
       ama2_warmcal(I) = MOD(AMSUA2(I+150)+65536,65536)/2
  51  CONTINUE
C!
      DO 60 I = 1 , 20
       ama2_temps(I) = MOD(AMSUA2(I+130)+65536,65536)/2
  60  CONTINUE
      ENDIF
C!...................................................................
C!
  101 FORMAT(1X,'***A1 sync bytes not encountered at ',I7,' msecs')
  102 FORMAT(1X,'***A2 sync bytes not encountered at ',I7,' msecs')
      RETURN
      END
