C!
C!ROUTINE          AMSBDC  SUBROUTINE  FORT77
C!
C!PURPOSE          TO DECOMMUTATE AMSU-B DATA AND PUT INTO 1A ARRAYS
C!
C!DESCRIPTION      This routine takes the AMSU-B minor
C!                 frames packed into the array AMSUB and
C!                 checks the synch bytes are OK and then extracts the
C!                 data and places it in the 1A data array. In addition
C!                 the scan line times, quality indicator are added.
C!                 Provision is made to also include the AMSU-B analogue
C!                 data from the TIP minor frames but this is not yet
C!                 implemented.
C!
C!VERSION          1.01   151195  R.W.SAUNDERS
C!                 1.02   180996  N.C.Atkinson
C!                 1.03   March 03  T.Labrot
C!        f90 test: Changed  type XXX/yyy/
C!                       to  type XXX
C!                           parameter(XXX=yyy) or data/yyy/
C!                 1.04   July 03  Remove redundant arguments - 
C!                                  AMANAG, YR, DAY     N C Atkinson
C!
C!ARGUMENTS
C!      INTEGER*2 AMSUB(1950) -- AMSU-B scan line
C!      INTEGER MSECS     -- TIP msecs from midnight
C!      INTEGER BERR(3)   -- AMSU-B Error flag (0=OK)
C!      INTEGER ISCAN     -- AMSU-B scan (1->3)
C!
C!REFERENCES       ATOVDC design document version 1.9 Nov 95
C!
C!CHANGES          01 None
C!                 02 Correction to sync word checking
C!
C!#########################################################################
C! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
C!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
C!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
C!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
C!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
C!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
C!#########################################################################
C!
      SUBROUTINE AMSBDC(AMSUB,MSECS,BERR,ISCAN)
C!
      IMPLICIT NONE
C!
      INTEGER*2 AMSUB(1950)             ! AMSU-B data for 3 scan lines
      INTEGER*4 MSECS                   ! msecs from midnight
      INTEGER*4 BERR(3)                 ! Error code (0=OK)
      INTEGER*4 SYNC                    ! Sync word
      PARAMETER(SYNC=21845)
      INTEGER*4 J                       ! record number
      INTEGER*4 I,ISCAN                 ! Number of sync words
      INTEGER*4 NSYNC(3)                ! Number of sync words
      DATA NSYNC/33,17,17/
            
      INTEGER*4 IOFF
      CHARACTER*80 TXT
C!
#include "amb1b.h"
#include "hrptdc.h"
C!
C!...................................................................
C!          AMSU-B data extraction
C!
C! First check sync bytes are OK
C!
      IOFF = (ISCAN-1)*650
C!
      DO 10 I = 1, NSYNC(ISCAN)
      IF(AMSUB(IOFF+I).NE.SYNC.AND.(IOFF+I).NE.1)THEN  !word 1 problem
!        BERR(ISCAN) = BERR(ISCAN).OR.(-2147483648)   !Set bit 31
        call gp_sb32(berr(iscan),31,1)
C!        WRITE(LULOG,101)MSECS
        write(txt,101)msecs
        call ml_wt('info',txt)
        go to 11
      ENDIF
  10  CONTINUE
  11  CONTINUE
C!
      DO 20 I = 1 , 540
       J = 36 + (ISCAN-1)*650 + I
       amb_earth_data(I) = AMSUB(J)
 20   CONTINUE
C!
      amb_diga(1) = AMSUB(577+IOFF)        ! A0
      amb_diga(2) = AMSUB(578+IOFF)        ! A1
C!
      DO 30 I = 1 , 24
      amb_temps(I) = AMSUB(I+578+IOFF)  ! Temps and voltages
 30   CONTINUE
C!
      DO 40 I = 1 , 24
       amb_coldcal(I) = AMSUB(I+602+IOFF)   ! space views
       amb_warmcal(I) = AMSUB(I+626+IOFF)   ! internal target views
  40  CONTINUE
C!...................................................................
C!
  101 FORMAT(1X,'***AMSU-B sync bytes not encountered at ',I7,' ms')
      RETURN
      END
