C!
C!ROUTINE          AMSGET   SUBROUTINE  FORT77
C!
C!PURPOSE          TO EXTRACT AMSU DATA FROM NOAA-K AMSU MINOR FRAME
C!
C!DESCRIPTION      Each AMSU minor frame contains 26 bytes of AMSU-A1 data,
C!                 14 bytes of AMSU-A2 data and 50 bytes of AMSU-B or MHS data.
C!                 These words are extracted and placed in AMSU-A1, A2 and
C!                 AMSU-B scan line buffers. When one AMSU-A scan line or
C!                 three AMSU-B scan lines are full a flag is set.
C!
C!VERSION          1.01   141195  R.W.SAUNDERS
C!                 1.02   180996  N.C.ATKINSON
C!                 1.03   031299  A.V.KORPELA
C!                 1.04   March 03  T.LABROT and P.Brunel
C!   f90 test: I.AND.J not correct and modified by IAND(I,J)
C!             Cut lines exceeding 72 characters.
C!             Changed type XXX/yyy/
C!                  to type XXX
C!                     parameter(XXX=yyy)   or data XXX/yyy/
C!                 1.05   July 03  N.C.ATKINSON Add MHS
C!                 1.06   030106  Test for bad MIUMF. NCA
C!                 1.07   190810  Only test for bad MIUMF if MHS is requested.
C!                        (when MHS is in fault mode this test can upset AMSU-A
C!                        processing)
C!                 1.08   210711  Don't abort frame if MIUMF is bad, just skip
C!                        over the MHS processing.
C!
C!
C!ARGUMENTS
C!      INTEGER*4 OPTN(12)  -- HRPTDC options
C!      INTEGER*4 IMF       -- 1-5 to identify which minor frame
C!      INTEGER*4 AMF       -- AMSU minor frame no. (0->79)
C!      INTEGER*2 HRPAIP(104,5) -- current TIP array
C!      INTEGER*2 AMSUA1R(620) -- AMSU-A1 scan line
C!      INTEGER*2 AMSUA2R(156) -- AMSU-A2 scan line
C!      INTEGER*2 AMSUB(1950) -- 3 AMSU-B scan lines
C!      CHARACTER*1 MHS(3900) -- 3 MHS scan lines
C!      LOGICAL*1 FILLER  -- True when input minor frame is empty filler
C!      LOGICAL*1 FULLAM  -- Set to true when AMSU-A scan line filled
C!      LOGICAL*1 MHSPRESENT -- True for NOAA-N onwards
C!
C!
C!REFERENCES       ATOVDC design document version 1.9 Nov 95
C!
C! Definition of MHS/MIU output format:
C!                 MHS_18_draft_tlmy_extract.doc (NOAA/NASA), para 18.5.6.7
C!
C!CHANGES          01 None
C!                 02 Various corrections, and include first AMSU-B scan
C!                 03 Parity error checking and AMSU-A scan line data recovery 
C!
C!#########################################################################
C! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
C!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
C!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
C!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
C!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
C!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
C!#########################################################################
C!
      SUBROUTINE AMSGET(OPTN,IMF,AMF,HRPAIP,AMSUA1R,AMSUA2R,AMSUB,MHS,
     &  FILLER,FULLAM,MHSPRESENT)
C!
      IMPLICIT NONE
C!
      INTEGER*2 HRPAIP(104,5)                !5 TIP/AMSU/Backfill minor frames
      INTEGER*2 AMSUA1R(620),AMSUA2R(156)    ! AMSU-A data for 1 scan line
      INTEGER   A1MAX,A2MAX
      PARAMETER (A1MAX=1040)                 ! We may have additional words from noise, allow twice the size
      PARAMETER (A2MAX=312)                  !
      INTEGER*2 AMSUA1(A1MAX),AMSUA2(A2MAX)  ! AMSU-A temporary buffers
      INTEGER*2 AMSUB(1950)                  ! AMSU-B data for 3 scan lines
      CHARACTER*1 MHS(3900)                  ! MHS data for 3 complete scans
      CHARACTER*1 MHSBUFFER(3900)            ! Local MHS data buffer
      SAVE MHSBUFFER
      INTEGER*4 AMF                          ! AIP minor frame number
      INTEGER*4 IMF                          ! TIP minor frame counter
      INTEGER*4 MIUMF                        ! MIU minor frame counter (for MHS)
      INTEGER*4 A1OFF,A2OFF,BOFF             ! Word offsets
      INTEGER*4 IW
      INTEGER*4 OPTN(12)                     ! Options for A1/A2 extraction
      INTEGER*4 MSB,LSB,ZFF
      PARAMETER(ZFF=255)
      INTEGER*2 ZFF2
      PARAMETER(ZFF2=255)
      
      INTEGER*4 FIRSTMF,FIRSTBSCAN,FIRSTMIUMF
      LOGICAL*1 FULLAM               ! Logical flag for full lines
      LOGICAL*1 FILLER               ! Logical flag for blank amsu frames
      LOGICAL*1 MHSPRESENT
      INTEGER*2 GAP                  ! Mark for gaps during amsu scanline reception
      PARAMETER(GAP=1)
      
      LOGICAL*1 A1PROBLEMS           ! Flag for parity errors or gaps during AMSU-A1 scanline
      DATA A1PROBLEMS/.FALSE./       
      
      LOGICAL*1 A2PROBLEMS           ! Flag for parity errors or gaps during AMSU-A2 scanline
      DATA A2PROBLEMS/.FALSE./

      LOGICAL*1 DBGA1                ! Flag for AMSU-A1 debug printing 
      DATA DBGA1/.FALSE./
      
      LOGICAL*1 DBGA2                ! Flag for AMSU-A2 debug printing
      DATA DBGA2/.FALSE./

      LOGICAL*1 DBGB                 ! Flag for AMSU-B/MHS debug printing 
      DATA DBGB/.FALSE./

      INTEGER*4 BIDX,BLINE,BFOV,BCH  ! Misc.
      LOGICAL*1 AMSUPARITY(104,5),AMSUA_PERROR(5),AMSUB_PERROR(5)
      COMMON /QCAMSU/ AMSUPARITY,AMSUA_PERROR,AMSUB_PERROR
      
      LOGICAL*1 FIRST                ! Logical flag for start AMSU
      DATA FIRST/.TRUE./

      COMMON /FIRSTCOM/ FIRSTMF,FIRSTBSCAN
      SAVE FIRSTMIUMF
      
      INTEGER*2 MHS_AIPSTART(80) !MHS start position in AIP minor frame (from 1)
      INTEGER*2 MHS_AIPEND(80)   !MHS end position (from 1)
      
      LOGICAL BadMIUMF

C! Start and end words for MHS data, for each MIU frame
C! (Note: in the NOAA document the start and end positions are counted from 0)
      DATA MHS_AIPSTART/91,25*49,49,75,25*49,49,57,25*49/
      DATA MHS_AIPEND/  98,25*98,82,98,25*98,66,98,25*98/

C! MHS byte position for each minor frame. Array has 1300 bytes per scan.
C! Leave 6 bytes spare at start of each scan (where the CCSDS Primary Header
C! would be), and 2 bytes spare at end of each scan. Used 1292 bytes/scan.
C! See Figure 18.5-32 of NOAA document. The MIU minor frame lags the AIP minor
C! frame (by 2 frames experimentally) and the MIU itself delays the data by 1
C! scan. So we set the total delay to 2 scans and put them in the following
C! locations:
C!   MIU frame 0-26:  1301-2600 (last scan of previous cycle).
C!   MIU frame 27-53: 2601-3900 (scan 0)
C!   MIU frame 55-79: 1-1300 (scan 1)
 
      INTEGER*2 MHS_OFFSETS(80)  
      DATA MHS_OFFSETS/
C! MIU frames 0 to 26 (scan 2)
     &  1307,1315,1365,1415,1465,1515,1565,1615,1665,1715,
     &  1765,1815,1865,1915,1965,2015,2065,2115,2165,2215,2265,2315,
     &  2365,2415,2465,2515,2565,
C! MIU frames 27 to 53 (scan 0)
     &  2607,2631,2681,2731,2781,2831,2881,2931,2981,3031,3081,3131,
     &  3181,3231,3281,3331,3381,3431,3481,3531,3581,3631,3681,3731,
     &  3781,3831,3881,
C! MIU frames 54 to 79 (scan 1)
     &  7,49,99,149,199,249,299,349,399,449,499,549,599,649,699,749,799,
     &  849,899,949,999,1049,1099,1149,1199,1249/

      character*80 txt
C!
#include "hrptdc.h"
C!
C!                         Ignore if bad minor frame number
      IF (AMF.GT.79 .OR. AMF.LT.0)THEN
        FULLAM=.FALSE.
        RETURN
      ENDIF
C!
C!...Get the MIU minor frame number and check whether it's valid
C!
      MIUMF = HRPAIP(8,IMF)
      IF (MHSPRESENT .AND. (MIUMF.LT.0.OR.MIUMF.GT.79)) THEN
        BadMIUMF = .TRUE.
        write(txt,
     &      '("bad MIU minor frame",I5," at mf",I3)') MIUMF,AMF
        call ml_wt('info',txt)
      ELSE
        BadMIUMF = .FALSE.
      ENDIF
C!
C!...Get all complete AMSU-B scans including the first
C!
      IF (FIRST)THEN
        FIRST=.FALSE.
        FULLAM = .FALSE.
        FIRSTMF = AMF                !Store first minor frame number
        IF (BadMIUMF) THEN
          FIRSTMIUMF = 79
        ELSE
          FIRSTMIUMF = MIUMF           !Store first MIU frame number
        ENDIF
        A1OFF = 0
        A2OFF = 0
        BOFF = 0

        IF (.NOT.MHSPRESENT) THEN

C! Calculate first AMSU-B scan to use: 1 to 4 where 4 indicates no scans

          FIRSTBSCAN = (AMF+23)/26 + 1 

        ELSE

          DO IW=1,3900
            MHSBUFFER(IW) = CHAR(0)   !Initialise
          ENDDO

          IF (BadMIUMF) THEN
            FIRSTBSCAN = 4
          ELSE IF ((MIUMF.GT.54.OR.MIUMF.EQ.0).AND.AMF.LE.26) THEN
            FIRSTBSCAN = 2
          ELSE IF (MIUMF.LE.27) THEN
            FIRSTBSCAN = 3
          ELSE
            FIRSTBSCAN = 4   !Illegal value, i.e. no complete scans
          ENDIF
C! Note we can never get all 3 scans first time round 
          
C          write(*,*)'FIRSTMIUMF, FIRSTMF:',FIRSTMIUMF, FIRSTMF
        ENDIF   !MHS present

      ELSE    !Not first time round

        IF (AMF.EQ.0) THEN     !i.e. after the data has been sent
          IF (MHSPRESENT.AND.FIRSTMIUMF.GT.54) THEN
            FIRSTBSCAN=2       !Still not got a complete scan 1
          ELSE
            FIRSTBSCAN=1       !This is the normal situation
          ENDIF
          FIRSTMF=0
          FIRSTMIUMF=0
        ENDIF

      ENDIF
C!
C!################  Extract AMSU data  ####################
C!
C! If ATOVDC sends here filler frames because of missing hrpt lines, we must not
C! fill in AMSUA1 and AMSUA2 arrays, because we do not know how much valid data
C! those missing frames contained.  Instead, mark the gap and continue filling
C! only if more real data comes, and then at the end of scanline analyze and
C! re-position the data.

      IF (FILLER) THEN               ! Don't do much with filler AMSU-A data
         A1OFF = A1OFF+1
         A1OFF = MIN(A1MAX,A1OFF)
         AMSUA1(A1OFF) = GAP
         A2OFF = A2OFF+1
         A2OFF = MIN(A2MAX,A2OFF)
         AMSUA2(A2OFF) = GAP
         A1PROBLEMS = .TRUE.
         A2PROBLEMS = .TRUE.
         IF (DBGA1) WRITE(6,*) 'MISSING FRAME',AMF,IMF,A1OFF
         IF (DBGA2) WRITE(6,*) 'MISSING FRAME',AMF,IMF,A2OFF
         IF (DBGB)  WRITE(6,*) 'MISSING FRAME',AMF,IMF,BOFF
      ELSE                              ! Here we have real AMSU data
C!
C!################  Extract AMSU-A1 words  ################
C!
      IF (OPTN(3).EQ.1.AND.FIRSTMF.EQ.0)THEN
      DO 10 IW = 9, 34 , 2
!      Replace parity error words with 0.
!      These transformed words may have been either data words or word=1 filler values, 
!      we don't know, linecheck is required.
       IF (.NOT.(AMSUPARITY(IW,IMF).AND.AMSUPARITY(IW+1,IMF))) THEN
          IF (DBGA1) WRITE(6,*) 'AMSU-A1 PARITYERROR AT',A1OFF+1
          HRPAIP(IW,IMF)=0
          HRPAIP(IW+1,IMF)=0
          A1PROBLEMS = .TRUE.
       ENDIF
       MSB = HRPAIP(IW,IMF)
       MSB = IAND(MSB,ZFF)
       MSB = MSB*256
       IF(MSB.GT.32767)MSB=MSB-65536
       LSB = HRPAIP(IW+1,IMF)
       LSB = IAND(LSB,ZFF)
       IF ((MSB+LSB).NE.1)THEN
        A1OFF = A1OFF + 1
        A1OFF = MIN(A1MAX,A1OFF)
        AMSUA1(A1OFF) = MSB + LSB
       ENDIF
  10  CONTINUE
      ENDIF
C!
C!###############  Extract AMSU-A2 words  ####################
C!
      IF (OPTN(4).EQ.1.AND.FIRSTMF.EQ.0)THEN
      DO 20 IW = 35, 48 , 2
!      Replace parity error words with 0.
!      These transformed words may have been either data words or word=1 filler values,
!      we don't know, linecheck is required.
       IF (.NOT.(AMSUPARITY(IW,IMF).AND.AMSUPARITY(IW+1,IMF))) THEN
          IF (DBGA2) WRITE(6,*) 'AMSU-A2 PARITYERROR AT',A2OFF+1
          HRPAIP(IW,IMF)=0
          HRPAIP(IW+1,IMF)=0
          A2PROBLEMS = .TRUE.
          ENDIF
       MSB = HRPAIP(IW,IMF)
       MSB = IAND(MSB,ZFF)
       MSB = MSB*256
       IF(MSB.GT.32767)MSB=MSB-65536
       LSB = HRPAIP(IW+1,IMF)
       LSB = IAND(LSB,ZFF)
       IF ((MSB+LSB).NE.1)THEN
        A2OFF = A2OFF + 1
        A2OFF = MIN(A2MAX,A2OFF)
        AMSUA2(A2OFF) = MSB + LSB
       ENDIF
  20  CONTINUE
      ENDIF
      ENDIF             ! for AMSU  Filler/Data  distinction
C!
C!############# AMSU-B EXTRACTION ########################
C!
      IF(OPTN(5).EQ.1.AND.FIRSTBSCAN.LE.3.AND..NOT.MHSPRESENT)THEN
      IF(AMF.GT.1)THEN
C!
C!  Compute word offsets
C!
      BOFF = (AMF-2)*25
C!
C!  Extract AMSU-B words
C!
      DO 30 IW = 49, 98 , 2
!      Replace parity error words with 0.
       IF (.NOT.(AMSUPARITY(IW,IMF).AND.AMSUPARITY(IW+1,IMF))) THEN
          BLINE=1+AMF/27
          BIDX=BOFF+1-(BLINE-1)*650
          BFOV=0
          BCH=0
          IF (BIDX.GT.36) BFOV=1+(BIDX-37)/6
          IF (BFOV.GT.0)  BCH=BIDX-37-(BFOV-1)*6
          IF (BFOV.GT.90) THEN
             BFOV=91
             BCH=0
          ENDIF
          IF (DBGB) WRITE(6,*) 'AMSU-B PARITYERROR AT',
     &          BOFF+1,BLINE,BFOV,BCH
          HRPAIP(IW,IMF)=0
          HRPAIP(IW+1,IMF)=0
       ENDIF
       BOFF = BOFF + 1
       MSB = HRPAIP(IW,IMF)
       MSB = IAND(MSB,ZFF)
       MSB = MSB*256
       IF(MSB.GT.32767)MSB=MSB-65536
       LSB = HRPAIP(IW+1,IMF)
       LSB = IAND(LSB,ZFF)
       AMSUB(BOFF) = MSB + LSB
  30  CONTINUE
      ENDIF
      ENDIF
C!
C!############# MHS EXTRACTION ########################
C!
      IF(OPTN(5).EQ.1.AND.MHSPRESENT.AND..NOT.BadMIUMF) THEN  !Get data regardless of FIRSTBSCAN

        BOFF = MHS_OFFSETS(MIUMF+1)   !MIUMF runs from 0 to 79
        
        DO IW = MHS_AIPSTART(MIUMF+1), MHS_AIPEND(MIUMF+1)

          IF (.NOT.(AMSUPARITY(IW,IMF))) THEN
            IF (DBGB) WRITE(6,*) 'MHS PARITY ERROR AT',BOFF
            HRPAIP(IW,IMF)=0
          ENDIF

C! For MHS we store a character*1 array. This is because (1) many of the
C! housekeeping parameters are 1 byte, and (2) to get the alignment right 
C! (see mhsdc.F).

          MHSBUFFER(BOFF) = CHAR(IAND(HRPAIP(IW,IMF),ZFF2))
          BOFF = BOFF + 1
          
        ENDDO
C!
C! At MIU frame 53, copy the MHS buffer to the output array. This is 
C! necessary because the scan in frames 54-79 is not complete when AMF
C! reaches 79.
C!
        IF (MIUMF.EQ.53.AND.FIRSTBSCAN.LE.3) THEN
          DO IW = 1,3900
            MHS(IW) = MHSBUFFER(IW)
          ENDDO
        ENDIF
      ENDIF 
C!
C!################  CHECK FOR END OF SCANLINE  ############
C!
C!
C!   Is output AMSU array full?
C!
      IF( AMF .EQ. 79 )THEN
       CALL AMSUA_LINECHECK('AMSUA1',A1PROBLEMS,AMSUA1,AMSUA1R,A1OFF,
     &   DBGA1)
       CALL AMSUA_LINECHECK('AMSUA2',A2PROBLEMS,AMSUA2,AMSUA2R,A2OFF,
     &   DBGA2)
        A1PROBLEMS = .FALSE.
        A2PROBLEMS = .FALSE.
        FULLAM =.TRUE.
        A1OFF=0
        A2OFF=0
        BOFF=0
      ENDIF
C!
      RETURN
      END

