C!NOM/NAME $Id: amshdu.F 337 2012-10-17 10:41:36Z frna $
C!
C!ROUTINE          AMSHDU  SUBROUTINE  FORT77
C!
C!PURPOSE          TO UPDATE THE AMSU-A/B 1A HEADERS
C!
C!DESCRIPTION      This routine updates the AMSU-A & B 1A header arrays.
C!
C!VERSION          1.01   161195  R.W.SAUNDERS
C!                 1.02   231195  M. Derrien suppress day_ref_noaa
C!                       change AMB_H_PRBLKID and AMA_H_PRBLKID content
C!                 1.03   180996  N.C.Atkinson
C!                 1.04   120598  M.Derrien
C!                         add siteid, blank, change version
C!                         correct record lengths in bytes
C!                 1.05   100798  M.Derrien
C!                         correct end data time
C!                 1.06   March 2003 T.LABROT and P.BRUNEL
C!  f90 test : I.AND.J not correct and modified by IAND(I,J)
C!             I.OR.J not correct and modified by IOR(I,J)
C!             I and J must have the same type.
C!             Add working variable for IAND/IOR tests
C!             Changed  type XXX/yyy/
C!                  to  type XXX
C!                      parameter(XXX=yyy)  or   data XXX/yyy/
C!                 1.07   July 2004 A.Korpela - mod updating of AMSU-B instr I/D
C!                 1.08   July 2004 N.C.Atkinson.   Add MHS
C!                 1.09   Aug 2004 N.C.Atkinson. Use new satid method
C!                 1.10   Nov 2004 NCA Remove obsolete ama/amb_h_calparid
C!                 1.11   Nov 2005 PM, support for Metop
C!                 1.12   Feb 2006 PM, swap AMSUA1 & AMSUA2 ids to enforce
C!                                 AMSUA1 odd & AMSUA2 even
C!                 1.13   Jun 2006 PM, character*16 l1bscnam + MA, MB, MC, MD in
C!                                 datasetname
C!                 1.14   Aug 2006 NCA Use station_id instead of fixed 'WE'
C!                 1.15   Nov 2006 NCA Correct instrument IDs for METOP MHS
C!                 1.16   Jan 2007 NCA Only update AMSUA instr IDs if non-zero
C!                 1.17   Apr 2007 NCA Improved detection of swapped IDs
C!                 1.18   Sep 2008 NCA Only update AMSUB instr ID if non-zero
C!                 1.19   Nov 2008 NCA Re-compute JDAY for *_H_ENDDATAJD
C!                 1.20   Oct 2012 NCA MHS FM5 used for Metop1(B), FM4 for Metop3(C)
C!
C!ARGUMENTS
C!      INTEGER*2 AMSUA1(620) -- AMSU-A1 scan line
C!      INTEGER*2 AMSUA2(156) -- AMSU-A2 scan line
C!      INTEGER*2 AMSUB(1950) -- AMSU-B scan line
C!      CHARACTER*1 MHS(3900) -- MHS scan line
C!      INTEGER YR        -- Year of data (e.g. 1997).
C!      INTEGER DAY       -- TIP day number
C!      INTEGER MSECS     -- TIP msecs from midnight
C!      INTEGER SYSTIM(3) -- Current year, daynumber, secs from midnight
C!      INTEGER SATID     -- NOAA Satellite id
C!      INTEGER AERR      -- AMSU-A Error flag  0 = OK
C!      INTEGER BERR(3)   -- AMSU-B Error flag  0 = OK
C!      INTEGER AREC      -- Current AMSU-A data record number
C!      INTEGER BREC      -- Current AMSU-B data record number
C!      INTEGER MISREC    -- Number of missing lines
C!      LOGICAL*1 GOTIME  -- Set to TRUE when first TIP time read
C!      INTEGER STARTORB  -- Start orbit number
C!      INTEGER ENDORB    -- End orbit number
C!      LOGICAL*1 MHSPRESENT  -- Set to TRUE for NOAA-N,N'
C!
C!ROUTINE          XDQJ , AMBST , AMAST
C!
C!FILES            FORnnn LOG FILE OUTPUT
C!                 FORmmm HIRS 1A file OUTPUT
C!
C!REFERENCES       ATOVDC design document version 1.9 Nov 95
C!
C!CHANGES          01 None
C!                 02 Minor corrections
C!
C!#########################################################################
C! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
C!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
C!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
C!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
C!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
C!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
C!#########################################################################
C!
      SUBROUTINE AMSHDU(AMSUA1,AMSUA2,AMSUB,MHS,YR,DAY,MSECS,SYSTIM,
     &SATID,AERR,BERR,namsua,namsub,MISRECA,MISRECB,GOTIME,
     &STARTORB,ENDORB,MHSPRESENT)
C!
      IMPLICIT NONE
!                         necessary for creation of the files
#include "nl1b.h"
#include "hrptdc.h"
#include "ama1b.h"
#include "amb1b.h"
#include "mhs1b.h"
#include "satid.h"
#include "stdio_unit.h"
C!
      INTEGER*2 AMSUA1(620),AMSUA2(156) ! One AMSU-A scan line
      INTEGER*2 AMSUB(1950)             ! 3 AMSU-B scan lines
      CHARACTER*1 MHS(3900)             ! 3 MHS scan lines
      INTEGER*4 SYSTIM(3)               ! System yr, dayno, secs from midnight
      INTEGER*4 DAY,MSECS               ! TIP day no and msecs from midnight
      INTEGER*4 YR,NAMSUA,NAMSUB        ! Year of data, no of scan lines
      INTEGER*4 JDAY                    ! Julian day no (1=1 JAN 1950)
      INTEGER*4 DQ(2),IQ(3),IG(7)       ! Time arrays
      INTEGER*4 AMSAID,AMSBID,AMS1ID,AMS2ID  ! AMSU instrument ids
      INTEGER*4 SATID                   ! Satellite id
      INTEGER*4 INSTA1,INSTA2,LASTA1,LASTA2,INSTB(3),LASTB !Instrument status
      INTEGER*4 AERR,BERR(3)            ! Error codes (0=OK)
      INTEGER*4 ERRTIM,ERRPAR,ERRSYN    ! Error counters
      INTEGER*4 MISRECA,MISRECB,MISDAT  ! number of missing lines/data gaps
      INTEGER*2 ZFF                     ! Hex mask
      PARAMETER (ZFF=255)
      INTEGER*2 ZFF00                   ! Hex mask
      PARAMETER (ZFF00=-256)
      
      INTEGER*4 ZTIM1                   ! Bit mask
      PARAMETER (ZTIM1=1082130432)
      INTEGER*4 ZPAR,ZST                ! BIT Masks
      PARAMETER (ZPAR=1)
      PARAMETER (ZST=33554432)

      INTEGER*4 ZSYNC,ZGAP            ! BIT Masks
      PARAMETER (ZSYNC=8)
      PARAMETER (ZGAP=536870912)
      
      INTEGER*4 ZFRAM                 ! Missing data gap bit set
      PARAMETER (ZFRAM=6)
       
      INTEGER*4 STARTORB,ENDORB       ! start orbit no. end orbit no.
      INTEGER*4 l1bscid
      INTEGER*4 FIRSTMF,FIRSTBSCAN
      COMMON /FIRSTCOM/ FIRSTMF,FIRSTBSCAN
      CHARACTER*4 STR1
      CHARACTER*3 STR2
      CHARACTER*5 STR3,STR4
      CHARACTER*16 SNAME,L1BSCNAM
      INTEGER CP_LSC, SNAME_LEN
      CHARACTER*2 RNAME
      CHARACTER*22 IC
      LOGICAL*1 FIRST,GOTIME,MHSPRESENT
      DATA FIRST/.TRUE./
      LOGICAL*1 SWAPPED
      DATA SWAPPED/.FALSE./
      
      INTEGER*2 I2WORD         ! working variable for IAND/IOR tests
      INTEGER*4 IWORD          ! working variable for IAND/IOR tests
      
      INTEGER*2 INSTID_MHS_NOAA(2)
      DATA INSTID_MHS_NOAA/1,2/   !Instrument IDs for NOAA-N and N'. 1=PFM, 2=FM2
      INTEGER*2 INSTID_MHS_METOP(4)
      DATA INSTID_MHS_METOP/5,3,4,77/     !Instrument IDs for METOP

!MOD/NIWA begin
      INTEGER*4 UPDATECNT      ! Counter to make last AMSUB intrument id update to occur
      DATA UPDATECNT /0/       ! closer to the midpoint of local reception (to avoid noise)

      UPDATECNT = UPDATECNT + 1
!MOD/NIWA end
C!
C! .. Get AMSU-A instrument status
C!
        CALL AMAST(AMSUA1,AMSUA2,INSTA1,INSTA2,AERR)
C!
C! .. Get AMSU-B instrument status
C!
        IF (.NOT.MHSPRESENT) THEN
          CALL AMBST(AMSUB,INSTB,BERR)
        ELSE
          CALL MHSST(MHS,INSTB,BERR)
        ENDIF
C!
      IF(FIRST)THEN
C!...Initialise error counters
        ERRTIM = 0
        ERRPAR = 0
        ERRSYN = 0
        MISDAT = 0
        LASTA1 = INSTA1
        LASTA2 = INSTA2
        LASTB = INSTB(3)
C!
C! .. Get AMSU-A instrument id
C!
        I2WORD = IAND(AMSUA1(2),ZFF)
        AMS1ID = I2WORD
        I2WORD = IAND(AMSUA2(2),ZFF)
        AMS2ID = I2WORD

        IF (AMS1ID.NE.0 .AND. AMS2ID.NE.0) THEN   !both must be non-zero
          IF ( MOD(AMS1ID,2).EQ.0 .AND. MOD(AMS2ID,2).NE.0) THEN
            SWAPPED = .TRUE.
          ELSE
            SWAPPED = .FALSE.
          ENDIF
        ENDIF

	IF (SWAPPED) THEN   ! eumetsat has inverted AMSUA1 & AMSUA2 ids
          AMSAID = AMS2ID
          AMS2ID = AMS1ID
          AMS1ID = AMSAID
          SWAPPED = .TRUE.
          WRITE(STDERRU,*) "Exchanged AMSUA1 & AMSUA2 ids !!"
        ENDIF

        AMS2ID = AMS2ID*256
        AMSAID = IOR(AMS1ID,AMS2ID)       ! Combine ids from A1 & A2
C!
C! .. Get AMSU-B instrument id
C!
        IF (.NOT.MHSPRESENT) THEN
          I2WORD = IAND(AMSUB(577),ZFF)
          AMSBID = I2WORD
        ELSE
          IF( ( SAT_SERIE( SATID ) .EQ. 'noaa' )
     >  .AND. ( ( NOA_SC_NUM( SATID ) .EQ. 18 )
     >     .OR. ( NOA_SC_NUM( SATID ) .EQ. 19 ) )
     >   ) THEN
            AMSBID = INSTID_MHS_NOAA( NOA_SC_NUM( SATID ) - 18 + 1 )
          ELSE IF( SAT_SERIE( SATID ) .EQ. 'metop' ) THEN
            AMSBID = INSTID_MHS_METOP( NOA_SC_NUM( SATID ) )
          ELSE
            AMSBID = 0
          ENDIF
        ENDIF     !MHS ID is not available in the MHS telemetry

C!
        IF(GOTIME.AND.FIRSTMF.EQ.0)FIRST = .FALSE.
C! ..compute Julian day Jan 1 1950
        DQ(1) = YR
        DQ(2) = DAY
        CALL XDQJ(DQ,JDAY)
C!
C!..Create header record 1 for both AMSU-A&B (not all elements filled)
C!
C!..General information
C!...AMSU-A
        AMA_H_L1BVERSNB = libl1bvers    ! Format version number
        AMA_H_L1BVERSYR = libl1bversyr  ! Format version year
        AMA_H_L1BVERSDY = libl1bversdy  ! Format version daynumber
        AMA_H_RECLG = reclama           ! Logical record length
        AMA_H_BLKSZ = cama_size         ! Block size
        AMA_H_HDRCNT = nb1bhd           ! No. of header records
        AMA_H_SATID = L1BSCID(SATID)    ! NOAA s/c id
        AMA_H_INSTID = AMSAID           ! AMSU-A instrument id
        AMA_H_DATATYP = 10              ! AMSU-A data type code
        AMA_H_TIPSRC = 1                ! TIP source code
        AMA_H_STARTDATAJD = JDAY        ! Start day no.(1=1 JAN 1950)
        AMA_H_STARTDATAYR = YR          ! Start of data year
        AMA_H_STARTDATADY = DAY         ! Start of data day of year
C        AMA_H_STARTDATATIME = MSECS-(namsua-1)*8000 !Data start time msecs
C1.07 Subtract 8 sec to make header consistent with data record
        AMA_H_STARTDATATIME = MSECS-(namsua)*8000 !Data start time msecs
        IF (AMA_H_STARTDATATIME.LT.0)THEN
         AMA_H_STARTDATADY = DAY-1
         AMA_H_STARTDATATIME = AMA_H_STARTDATATIME+86400000
        ENDIF
C        AMA_H_CALPARID = '0'            ! CPIDS id
        AMA2_H_INSTSTAT1 = INSTA2       ! AMSU-A2 instrument status
        AMA1_H_INSTSTAT1 = INSTA1       ! AMSU-A1 instrument status
        AMA2_H_STATCHRECNB  = 0         ! rec number of status change
        AMA1_H_STATCHRECNB  = 0         ! rec number of status change
        AMA_H_PRBLKID = loccentre
        AMA_H_SITEID = loccentre
        AMA_H_BLANK=' '

        IF (.NOT.MHSPRESENT) THEN
C!...AMSU-B
        AMB_H_L1BVERSNB = libl1bvers    ! Format version number
        AMB_H_L1BVERSYR = libl1bversyr  ! Format version year
        AMB_H_L1BVERSDY = libl1bversdy  ! Format version daynumber
        AMB_H_RECLG = reclamb           ! Logical record length
        AMB_H_BLKSZ = camb_size         ! Block size
        AMB_H_HDRCNT = nb1bhd           ! No. of header records
        AMB_H_SATID = L1BSCID(SATID)    ! NOAA s/c id
!MOD/NIWA
        IF (UPDATECNT.LT.50.OR.AMB_H_INSTID.EQ.0) AMB_H_INSTID = AMSBID ! AMSU-B instrument id
        AMB_H_DATATYP = 11              ! AMSU-B data type code
        AMB_H_TIPSRC = 1                ! TIP source code
        AMB_H_STARTDATAJD = JDAY        !Start Julian day (1=1JAN 1950)
        AMB_H_STARTDATAYR = YR          ! Start of data year
        AMB_H_STARTDATADY = DAY         ! Start of data day of year
C        AMB_H_STARTDATATIME = MSECS-(namsub-3)*2667 ! Data start time msecs
C1.07 Subtract 8 sec to make header consistent with data record
        AMB_H_STARTDATATIME = MSECS-namsub*2667 ! Data start time msecs
        IF (AMB_H_STARTDATATIME.LT.0)THEN
         AMB_H_STARTDATADY = DAY-1
         AMB_H_STARTDATATIME = AMB_H_STARTDATATIME+86400000
        ENDIF
C        AMB_H_CALPARID = '0'            ! CPIDS id
        AMB_H_INSTSTAT1 = INSTB(1)      ! AMSU-B instrument status
        AMB_H_STATCHRECNB  = 0          ! rec number of status change
        AMB_H_SITEID = loccentre
        AMB_H_BLANK=' '

        ELSE
C!...MHS
        MHS_H_L1BVERSNB = libl1bvers    ! Format version number
        MHS_H_L1BVERSYR = libl1bversyr  ! Format version year
        MHS_H_L1BVERSDY = libl1bversdy  ! Format version daynumber
        MHS_H_RECLG = reclamb           ! Logical record length
        MHS_H_BLKSZ = camb_size         ! Block size
        MHS_H_HDRCNT = nb1bhd           ! No. of header records
        MHS_H_SATID = L1BSCID(SATID)    ! NOAA s/c id
        MHS_H_INSTID = AMSBID           ! MHS instrument id
        MHS_H_DATATYP = 12              ! MHS data type code
        MHS_H_TIPSRC = 1                ! TIP source code
        MHS_H_STARTDATAJD = JDAY        !Start Julian day (1=1JAN 1950)
        MHS_H_STARTDATAYR = YR          ! Start of data year
        MHS_H_STARTDATADY = DAY         ! Start of data day of year
C1.07 MHS is 2 scans behind the equivalent AMSU-B
        MHS_H_STARTDATATIME = MSECS-(namsub+2)*2667 ! 2 scans behind
        IF (MHS_H_STARTDATATIME.LT.0)THEN
         MHS_H_STARTDATADY = DAY-1
         MHS_H_STARTDATATIME = MHS_H_STARTDATATIME+86400000
        ENDIF
C obsolete        MHS_H_CALPARID = '0'            ! CPIDS id
        MHS_H_INSTSTAT1 = INSTB(1)      ! AMSU-B instrument status
        MHS_H_STATCHRECNB  = 0          ! rec number of status change
        MHS_H_SITEID = loccentre
        MHS_H_BLANK=' '

        ENDIF
        
C..prepare time strings
        IQ(1) = YR
        IQ(2) = DAY
        IQ(3) = AMA_H_STARTDATATIME
        WRITE(STR1,1000)YR
 1000   FORMAT(I4.4)
        WRITE(STR2,1001)DAY
 1001   FORMAT(I3.3)
        WRITE(STR3,1002)STARTORB
 1002   FORMAT(I5.5)
        WRITE(STR4,1002)ENDORB
        SNAME = L1BSCNAM(SATID)
        SNAME_LEN = CP_LSC( SNAME )
        RNAME = SNAME(1:1)//SNAME(SNAME_LEN:SNAME_LEN)
        CALL CP_UPS(RNAME)
        CALL XIQG(IQ,IG)
        CALL XIGC(IG,IC)
        AMA_H_DATANAME=LOCCENTRE//'.AMAX.'//RNAME//
     &  '.D'//STR1(3:4)//STR2//'.S'//IC(11:12)//IC(14:15)//
     &  '.E    .B'//STR3//STR4(4:5)//'.'//station_id
        AMB_H_DATANAME=LOCCENTRE//'.AMBX.'//RNAME//
     &  '.D'//STR1(3:4)//STR2//'.S'//IC(11:12)//IC(14:15)//
     &  '.E    .B'//STR3//STR4(4:5)//'.'//station_id
        MHS_H_DATANAME=LOCCENTRE//'.MHSX.'//RNAME//
     &  '.D'//STR1(3:4)//STR2//'.S'//IC(11:12)//IC(14:15)//
     &  '.E    .B'//STR3//STR4(4:5)//'.'//station_id
        AMA_H_PRBLKID = 'B'//STR3//STR4(4:5)
        AMB_H_PRBLKID = 'B'//STR3//STR4(4:5)
        MHS_H_PRBLKID = 'B'//STR3//STR4(4:5)

C!
      ENDIF
!
! Update header with instrument ids when they are reliable
!
      IF (AERR.EQ.0) THEN                       ! AMSU-A instrument id
         I2WORD = IAND(AMSUA1(2),ZFF)
         AMS1ID = I2WORD
         I2WORD = IAND(AMSUA2(2),ZFF)
         AMS2ID = I2WORD
	 
         IF (AMS1ID.NE.0 .AND. AMS2ID.NE.0) THEN   !both must be non-zero
           IF ( MOD(AMS1ID,2).EQ.0 .AND. MOD(AMS2ID,2).NE.0) THEN
             SWAPPED = .TRUE.
           ELSE
             SWAPPED = .FALSE.
           ENDIF
         ENDIF

	 IF (SWAPPED) THEN   ! eumetsat has inverted AMSUA1 & AMSUA2 ids
           AMSAID = AMS2ID
           AMS2ID = AMS1ID
           AMS1ID = AMSAID
           SWAPPED = .TRUE.
           WRITE(STDERRU,*) "Exchanged AMSUA1 & AMSUA2 ids !!"
         ENDIF 
	 
         AMS2ID = AMS2ID*256

!Only update the instrument IDs if they are non-zero

         IF (AMS2ID .EQ. 0) AMS2ID = IAND(AMA_H_INSTID,ZFF00)
         IF (AMS1ID .EQ. 0) AMS1ID = IAND(AMA_H_INSTID,ZFF)

         AMSAID = IOR(AMS1ID,AMS2ID)
         AMA_H_INSTID = AMSAID

      ENDIF

      IF (.NOT.MHSPRESENT) THEN
       IF ((BERR(1) .EQ. 0) .AND.
     &    (BERR(2) .EQ. 0) .AND.
     &    (BERR(3) .EQ. 0)) THEN                ! AMSU-B instrument id
        I2WORD = IAND(AMSUB(577),ZFF)
        AMSBID = I2WORD
!MOD/NIWA
        IF (AMSBID.NE.0) THEN
         IF (UPDATECNT.LT.50.OR.AMB_H_INSTID.EQ.0) AMB_H_INSTID = AMSBID
        ENDIF
       ENDIF
      ENDIF
C!
C! Add end data date and time
C!
      IQ(1) = YR
      IQ(2) = DAY
      IQ(3) = MSECS
      CALL XIQG(IQ,IG)
      CALL XIGC(IG,IC)
      DQ(1) = YR
      DQ(2) = DAY
      CALL XDQJ(DQ,JDAY)             ! Compute Julian day
      AMA_H_ENDDATAJD = JDAY         ! End Julian day no.(1=1 JAN 1950)
      AMA_H_ENDDATAYR = YR          ! End of data year
      AMA_H_ENDDATADY = DAY         ! End of data day of year
      AMA_H_ENDDATATIME = MSECS-8000 ! End data time of day msecs
      IF (AMA_H_ENDDATATIME.LT.0)THEN
       AMA_H_ENDDATADY = DAY-1
       AMA_H_ENDDATATIME = AMA_H_ENDDATATIME+86400000
      ENDIF
      AMA_H_DATANAME(27:30) = IC(11:12)//IC(14:15)
      AMB_H_ENDDATAJD = JDAY        ! End Julian day no.(1=1 JAN 1950)
      AMB_H_ENDDATAYR = YR          ! End of data year
      AMB_H_ENDDATADY = DAY         ! End of data day of year
      AMB_H_ENDDATATIME = MSECS-2666! End data time of day msecs
      IF (AMB_H_ENDDATATIME.LT.0)THEN
       AMB_H_ENDDATADY = DAY-1
       AMB_H_ENDDATATIME = AMB_H_ENDDATATIME+86400000
      ENDIF
      AMB_H_DATANAME(27:30) = IC(11:12)//IC(14:15)
      MHS_H_ENDDATAJD = JDAY        ! End Julian day no.(1=1 JAN 1950)
      MHS_H_ENDDATAYR = YR          ! End of data year
      MHS_H_ENDDATADY = DAY         ! End of data day of year
      MHS_H_ENDDATATIME = MSECS-8000! End data time of day msecs. 2 scans behind
      IF (MHS_H_ENDDATATIME.LT.0)THEN
       MHS_H_ENDDATADY = DAY-1
       MHS_H_ENDDATATIME = MHS_H_ENDDATATIME+86400000
      ENDIF
      MHS_H_DATANAME(27:30) = IC(11:12)//IC(14:15)

C!..Update instrument status if changed
      IF (LASTA1 .NE. INSTA1)THEN
        AMA1_H_STATCHRECNB = namsua+NB1BHD !Record of AMSU-A1 status change
        AMA1_H_INSTSTAT2 = INSTA1    ! AMSU-A1 new status
        AERR = IOR(AERR,ZST)
      ENDIF
      IF (LASTA2 .NE. INSTA2)THEN
        AMA2_H_STATCHRECNB = namsua+NB1BHD !Record of AMSU-A2 status change
        AMA2_H_INSTSTAT2 = INSTA2    ! AMSU-A2 new status
        AERR = IOR(AERR,ZST)
      ENDIF
      LASTA1 = INSTA1
      LASTA2 = INSTA2
      IF (LASTB .NE. INSTB(3))THEN
        AMB_H_STATCHRECNB = namsub+NB1BHD !Record of AMSU-B status change
        AMB_H_INSTSTAT2 = INSTB(3)  ! AMSU-B new status
        MHS_H_STATCHRECNB = namsub+NB1BHD !Record of AMSU-B status change
        MHS_H_INSTSTAT2 = INSTB(3)  ! MHS new status
        BERR(1) = IOR(BERR(1),ZST)
      ENDIF
      LASTB = INSTB(3)
C!
C!  Increment error counters if necessary
C!
      IWORD = IAND(AERR,ZTIM1)
      IF (IWORD.GT.0)ERRTIM=ERRTIM+1

      IWORD = IAND(AERR,ZPAR)
      IF (IWORD.GT.0)ERRPAR=ERRPAR+1

      IWORD = IAND(AERR,ZSYNC)
      IF (IWORD.GT.0)ERRSYN=ERRSYN+1

      IWORD = IAND(AERR,ZFRAM)
      IF (IWORD.GT.0)THEN
        MISDAT = MISDAT + 1
        AERR = IOR(AERR,ZGAP)
        BERR(1) = IOR(BERR(1),ZGAP)
      ENDIF
C!
      AMA_H_SCNLIN = namsua              ! Number of AMSU-A scan lines in file
      AMB_H_SCNLIN = namsub              ! Number of AMSU-B scan lines in file
      MHS_H_SCNLIN = namsub              ! Number of MHS scan lines in file
      ama_h_misscnlin = MISRECA          ! Number of missing AMSU-A lines
      amb_h_misscnlin = MISRECB         ! Number of missing AMSU-B lines
      mhs_h_misscnlin = MISRECB         ! Number of missing MHS lines
      ama_h_datagaps = MISDAT           ! Number of data gaps in AMSU-A data
      amb_h_datagaps = MISDAT           ! Number of data gaps in AMSU-B data
      mhs_h_datagaps = MISDAT           ! Number of data gaps in MHS data
      AMA_H_OKDATAFR = namsua-ERRSYN    ! Count of frames with no synch errors
      AMB_H_OKDATAFR = (namsua-ERRSYN)*3 !Count of frames with no synch errors
      MHS_H_OKDATAFR = (namsua-ERRSYN)*3 !Count of frames with no synch errors
      AMA_H_PACSPARITYERR = ERRPAR      ! Count of TIP parity errors
      AMB_H_PACSPARITYERR = ERRPAR*3    ! Count of TIP parity errors
      MHS_H_PACSPARITYERR = ERRPAR*3    ! Count of TIP parity errors
      AMA_H_AUXSYNCERRSUM = ERRSYN      ! Count of all synch errors
      AMB_H_AUXSYNCERRSUM = ERRSYN*3    ! Count of all synch errors
      MHS_H_AUXSYNCERRSUM = ERRSYN*3    ! Count of all synch errors
      AMA_H_TIMESEQERR = ERRTIM         ! Count of TIP timimg errors
      AMB_H_TIMESEQERR = ERRTIM*3       ! Count of TIP timimg errors
      MHS_H_TIMESEQERR = ERRTIM*3       ! Count of TIP timimg errors
C!
      RETURN
      END
