C!NOM/NAME $Id: amsout.F 212 2010-04-28 09:29:11Z frna $
C!
C!ROUTINE          AMSOUT  SUBROUTINE  FORT77
C!
C!PURPOSE          UPDATE HEADER AND WRITE AMSU A&B SCAN LINES TO DISK
C!
C!DESCRIPTION      This routine updates both the AMSU-A&B 1A headers and data
C!                 records (one data record per scan line) and writes out
C!                 one AMSU-A and three AMSU-B scan lines.
C!                 Called by atovdc.F when minor frame is 79.
C!
C!VERSION          1.01   151195  R.W.SAUNDERS
C!                 1.02   180996  N.C.Atkinson
C!                 1.03   211298  N.C.Atkinson
C!                 1.04   March 03  T.Labrot
C!    f90 test: Change TYPE to PRINT
C!              Changed type XXX/yyy/
C!                   to type XXX
C!                      data XXX/yyy/
C!                 1.05   050304  N C Atkinson change initialisation of MISSING
C!                 1.06   Apr 04  N.C.Atkinson. Add MHS
C!                 1.07   Sept 05 Set MHS time stamps to 1 scan line earlier,
C!                                as recommended by NESDIS. NCA
C!                 1.08   Nov 08  Bug fix: 86400000 was mis-typed as 864000000
C!                 1.09   Apr 10  For portability, re-write the currect scan 
C!                                when correcting times of first few scan lines.
C!
C!ARGUMENTS
C!      INTEGER*4 OPTN(12)  ---  Option numbers
C!      INTEGER*2 AMSUA1(620)  -- AMSU-A1 scan line
C!      INTEGER*2 AMSUA2(156)  -- AMSU-A2 scan line
C!      INTEGER*2 AMSUB(1950)  -- AMSU-B scan line
C!      CHARACTER*1 MHS(3900)  -- MHS scan line
C!      INTEGER*2 AMANAG(71)   -- AMSU Analogue data
C!      INTEGER YR        -- Year of data (e.g. 1997).
C!      INTEGER MSECA     -- Time of data
C!      INTEGER SYSTIM(3) -- System year, dayno, msecs from midnight
C!      INTEGER SATID     -- NOAA Satellite id
C!      INTEGER AERR      -- AMSU-A Error flag (0=OK)
C!      INTEGER BERR(3)   -- AMSU-B Error flag (0=OK)
C!      LOGICAL*1 FULLAM  -- Set to false when AMSU scan line written
C!      LOGICAL*1 GOTIME  -- Set to TRUE when first TIP time read
C!      INTEGER STARTORB  -- start orbit number
C!      INTEGER ENDORB    -- end orbit number
C!      LOGICAL*1 MHSPRESENT  -- Set to TRUE for NOAA-N,N'
C!
C!COMMON /LOG/ MSECS,DAY,NMF,NHIRS,NMSU,NAMSUA,NAMSUB,NDCS
C!
C!SUBPROGRAMS      AMSHDU, AMSDTR , AMSDTW , AMSADC , AMSBDC
C!
C!FILES            FORnnn LOG FILE OUTPUT
C!                 FORmmm AMSU-A 1A file OUTPUT
C!                 FORmmm AMSU-B 1A file OUTPUT
C!
C!REFERENCES       ATOVDC design document version 1.9  Nov 95
C!
C!CHANGES          01 None
C!                 02 Corrections, and allow first AMSU-B scan to be written
C!                 03 Increase size of AMSU-B analogs
C!
C!#########################################################################
C! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
C!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
C!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
C!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
C!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
C!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
C!#########################################################################
C!
      SUBROUTINE AMSOUT(OPTN,AMSUA1,AMSUA2,AMSUB,MHS,AMANAG,AMVALID,YR,
     &  MSECA,SYSTIM,SATID,AERR,BERR,FULLAM,GOTIME,STARTORB,ENDORB,
     &  MHSPRESENT)
C!
      IMPLICIT NONE
C!
      INTEGER*2 AMSUA1(620),AMSUA2(156) ! AMSU-A data for 1 scan line
      INTEGER*2 AMSUB(1950)             ! AMSU-B data for 3 scan lines
      CHARACTER*1 MHS(3900)             ! 3 MHS scan lines
      INTEGER*2 AMANAG(71)              ! AMSU analog telemetry
      INTEGER*2 AMVALID(71)             ! AMSU analog telemetry validity
      INTEGER*4 SYSTIM(3)               !System year, dayno, msecs frm midnit
      INTEGER*4 DAY,MSECS,MSECA,MSECB   ! TIP day no and msecs from midnight
      INTEGER*4 YR ,IND                 ! Year of data
      INTEGER*4 NLIN                    ! line number
      INTEGER*4 MISRECA,MISRECB         ! Number of missing scan lines
      INTEGER*4 SATID                   ! Satellite id
      INTEGER*4 AERR,BERR(3)            ! Error code (0=OK)
      INTEGER*4 NMF                     ! No. of TIP maj/minor frames read
      INTEGER*4 OPTN(12)                ! Option numbers
      INTEGER*4 STARTORB,ENDORB
      INTEGER*4 J,LMSECS(2),MMSECS,I
      INTEGER*4 MISSING,IOS
      INTEGER*4 MT(2)                   ! Thresholds for missing scan
      DATA MT/9000,3000/
      INTEGER*4 NHIRS,NMSU,NAMSUA,NAMSUB,NDCS !No scan lines
      INTEGER*4 NAMSUANEW,NAMSUBNEW
      INTEGER*4 DAYA
      INTEGER*4 FIRSTMF,FIRSTBSCAN
      LOGICAL*1 MHSPRESENT,FULLAM,START  ! START set to false when line written
      DATA START/.TRUE./

      LOGICAL*1 FIRST,GOTIME
      DATA FIRST/.TRUE./
      
!      DATA MISSING/-2147483648/         ! missing data flag
      SAVE MISSING
C!
#include "nl1b.h"
#include "hrptdc.h"
#include "ama1b.h"
#include "amb1b.h"
#include "mhs1b.h"
      COMMON /LOG/ MSECS,DAY,NMF,NHIRS,NMSU,NAMSUA,NAMSUB,NDCS
      COMMON /FIRSTCOM/ FIRSTMF,FIRSTBSCAN
C!
      IF(FIRST)THEN
       MISSING = -2147483647-1       !Some compilers don't accept -2147483648
       FIRST =.FALSE.
       lmsecs(1) = mseca
       lmsecs(2) = mseca
       namsua = 0
       namsub = 0
       misreca = 0
       misrecb = 0
      ENDIF
C!
C!.....Loop for AMSU-A and AMSU-B/MHS
C!
      DO 50 IND=1,2
C!
C!   Check for and fill in missing lines if necessary

C! AMSU-A

      IF(.not.gotime.and.(MSECA-LMSECS(IND)) .GT. MT(IND) )THEN
        MMSECS = LMSECS(IND)
        IF(IND.EQ.1 .AND. (OPTN(3).EQ.1.OR.OPTN(4).EQ.1))THEN
         print *,'Missing amsu-a lines in amsout'
         print *,'namsua= ',namsua
         nlin = NINT(FLOAT(MSECA-LMSECS(IND))/8000.0)
         print *,' mseca lmsec ',mseca,lmsecs(ind)
         do 3 i = 1,CAMA_SIZE
          ama_dt(i) = 0
 3       continue
         ama_scnlinyr = yr
         ama_scnlindy = day
         ama_qualind = missing
C!
         DO 5 I = 1 , nlin-1
          MMSECS = MMSECS + 8000
C1.05 Change to make the following line consistent with the case where the
C1.05 data is not missing. 
C1.05      ama_scnlintime = MMSECS
          ama_scnlintime = MMSECS - 8000
          namsua = namsua + 1
          ama_scnlin = namsua
          call amadtw(luamsa1,namsua,ios)
          MISRECA = MISRECA + 1
  5      CONTINUE
        ENDIF

C! AMSU-B

        IF(IND.EQ.2 .AND. OPTN(5).EQ.1 .AND. .NOT.MHSPRESENT )THEN
         print *,'Missing amsu-b lines in amsout'
         print *,'namsub= ',namsub
         nlin = NINT(FLOAT(MSECA-LMSECS(IND))/2667.0)
         print *,' mseca lmsec ',mseca,lmsecs(ind)
         do i = 1,camb_size
           amb_dt(i) = 0
         enddo
         amb_scnlinyr = yr
         amb_scnlindy = day
         amb_qualind = missing
C
         DO J = 1 , nlin-1
          MMSECS = MMSECS + 2667
C1.05 Change to make the following line consistent with the case where the
C1.05 data is not missing. 
C1.05      amb_scnlintime = MMSECS
          amb_scnlintime = MMSECS - 8000
          namsub = namsub + 1
          amb_scnlin = namsub
          call ambdtw(luamsb,namsub,ios)
          misrecb = misrecb + 1
         ENDDO
        ENDIF

C! MHS

        IF(IND.EQ.2 .AND. OPTN(5).EQ.1 .AND. MHSPRESENT )THEN
         print *,'Missing MHS lines in amsout'
         print *,'namsub= ',namsub
         nlin = NINT(FLOAT(MSECA-LMSECS(IND))/2667.0)
         print *,' mseca lmsec ',mseca,lmsecs(ind)
         do i = 1,camb_size
           mhs_dt(i) = 0
         enddo
         mhs_scnlinyr = yr
         mhs_scnlindy = day
         mhs_qualind = missing
C
         DO J = 1 , nlin-1
          MMSECS = MMSECS + 2667
C          mhs_scnlintime = MMSECS - 8000 - 5333   !2 scans in arrears
          mhs_scnlintime = MMSECS - 8000 - 8000   !3 scans in arrears (v1.07)
          namsub = namsub + 1
          mhs_scnlin = namsub
          call mhsdtw(luamsb,namsub,ios)
          misrecb = misrecb + 1
         ENDDO
        ENDIF

      ENDIF
C      LMSECS(IND) = MSECA
 50   CONTINUE
      LMSECS(1) = MSECA
      LMSECS(2) = MSECA+5333  !Change to avoid unnecessary MHS fill lines at 
                              !start
      DAYA=DAY
      IF ((MSECA-MSECS).GT.43200000) DAYA=DAY-1   !If MSECA is before midnight

C!
C! ...Update header records. Note: namsua and namsub not yet incremented
C!
      namsuanew = namsua
      if (firstmf.eq.0) namsuanew = namsua+1
      namsubnew = namsub + 4 - firstbscan
      CALL AMSHDU(AMSUA1,AMSUA2,AMSUB,MHS,YR,DAYA,MSECA,SYSTIM,SATID,
     & AERR,BERR,namsuanew,namsubnew,MISRECA,MISRECB,
     & GOTIME,STARTORB,ENDORB,MHSPRESENT)
C!
C! ...Update analogue telemetry
C!
      CALL AMANAG_OUT(AMANAG,AMVALID,MHSPRESENT)
C!
C!  Decommutate AMSU-A data
C!
      IF ((OPTN(3).EQ.1.OR.OPTN(4).EQ.1).AND.FIRSTMF.EQ.0) THEN
        CALL AMSADC(OPTN,AMSUA1,AMSUA2,MSECA,AERR)
C! add data date and time
        ama_scnlinyr = yr
        ama_scnlindy = daya
        ama_scnlintime = mseca - 8000
        IF (ama_scnlintime.lt.0) THEN
          ama_scnlintime = ama_scnlintime + 86400000
          ama_scnlindy = daya - 1
        ENDIF
        ama_clockdrift = 0
        ama_qualind = aerr
        namsua = namsua+1
        ama_scnlin = namsua
        ama_linqualflgs = 0
        do i=1,15
          ama_chqualflg(i) = 0
        enddo
C!
C! ... Write AMSU-A 1A data record
C!
C@        write(6,*) 'Writing AMSU-A data frame ',namsua, 'mseca=',mseca
        CALL AMADTW(luamsa1,namsua,ios)
        aerr = 0
      ENDIF
C!
C! ... Decommutate AMSU-B data
C!
      IF ( OPTN(5) .EQ. 1 .AND. FIRSTBSCAN .LE. 3 .AND.
     &                 .NOT.MHSPRESENT) THEN
       DO 65 J = FIRSTBSCAN , 3
        MSECB = MSECA + (J-1)*2667
        LMSECS(2) = MSECB
        CALL AMSBDC(AMSUB,MSECB,BERR,J)
C! add data date and time
        amb_scnlinyr = yr
        amb_scnlindy = daya
        amb_scnlintime = msecb - 8000
        IF (amb_scnlintime.lt.0) THEN
          amb_scnlintime = amb_scnlintime + 86400000
          amb_scnlindy = daya - 1
        ENDIF
        amb_clockdrift = 0
        amb_qualind = berr(j)
        namsub = namsub + 1
        amb_scnlin = namsub
        amb_linqualflgs = 0
        do i=1,5
          amb_chqualflg(i) = 0
        enddo
C!
C! ... Write AMSU-B 1A data record
C!
C@        write(6,*) 'Writing AMSU-B data frame ',namsub, 'msecb=',msecb
        CALL AMBDTW(luamsb,namsub,ios)
        berr(j) = 0
 65    CONTINUE
      ENDIF
C!
C! ... Decommutate MHS data
C!
      IF ( OPTN(5) .EQ. 1 .AND. FIRSTBSCAN .LE. 3 .AND.
     &                 MHSPRESENT) THEN
       DO 70 J = FIRSTBSCAN , 3
C        MSECB = MSECA + (J-2)*2667 !MHS runs 1 scan behind compared with AMSU-B
        MSECB = MSECA + (J-3)*2667 !MHS runs 2 scans behind compared with AMSU-B
        LMSECS(2) = MSECB + 5333   !Keep LMSECS(2) as for AMSU-B
        CALL MHSDC(MHS,MSECB,BERR,J)
C! add data date and time
        mhs_scnlinyr = yr
        mhs_scnlindy = daya
C        mhs_scnlintime = msecb - 8000
        mhs_scnlintime = msecb - 8000 - 2667     !v1.07
        IF (mhs_scnlintime.lt.0) THEN
          mhs_scnlintime = mhs_scnlintime + 86400000
          mhs_scnlindy = daya - 1
        ENDIF
        mhs_clockdrift = 0
        mhs_qualind = berr(j)
        namsub = namsub + 1
        mhs_scnlin = namsub
        mhs_linqualflgs = 0
        do i=1,5
          mhs_chqualflg(i) = 0
        enddo
C!
C! ... Write MHS 1A data record
C!
C@        write(6,*) 'Writing MHS data frame ',namsub, 'msecb=',msecb
        CALL MHSDTW(luamsb,namsub,ios)
        berr(j) = 0
 70    CONTINUE
      ENDIF
C!
C! Correct time on first few scan lines if necessary
C!
C1.05 Adjust logic to treat AMSU-A and AMSU-B separately, in case AMSU-A has
C1.05 not been requested, or there are extra AMSU-B scans at start. 
C1.05      IF (START.AND.GOTIME.AND.namsua.GT.1) THEN

      IF (START .AND. GOTIME) THEN
       START = .FALSE.
       IF (namsua.GT.1) THEN
        call ml_wti('info','amsu-a time correction before scan ',namsua)
C1.09        DO 25 I = 1 , namsua-1
        DO 25 I = 1 , namsua
        IF(OPTN(3).EQ.1 .OR. OPTN(4).EQ.1)THEN
         call amadtr(luamsa1,i,ios)
         ama_scnlintime = MSECA-(namsua-i)*8000 - 8000  ! corr TIP time msecs
         IF (ama_scnlintime.lt.0) THEN
          ama_scnlintime = ama_scnlintime + 86400000
          ama_scnlindy = daya - 1
         ENDIF
         call amadtw(luamsa1,i,ios)
        ENDIF
 25     continue
       ENDIF

C Only correct scans processed before this run
       IF (namsub.GT.3 .AND. OPTN(5).EQ.1) THEN
        IF (.NOT.MHSPRESENT) THEN        !AMSU-B
         call ml_wti('info','amsu-b time correction before scan ',
     &               namsub-2)
C1.09         DO 35 I=1,namsub-3
         DO 35 I=1,namsub
          call ambdtr(luamsb,i,ios)
C1.05 The following is correct, but re-write to make it clearer:
C1.05      amb_scnlintime = MSECA-(namsub-2-i)*2667 - 8000  ! cor TIP time msecs
          amb_scnlintime = MSECB-(namsub-i)*2667 - 8000    ! cor TIP time msecs
          IF (amb_scnlintime.lt.0) THEN
           amb_scnlintime = amb_scnlintime + 86400000
           amb_scnlindy = daya - 1
          ENDIF
          call ambdtw(luamsb,i,ios)
 35      CONTINUE
        ELSE                             !MHS
         call ml_wti('info','mhs time correction before scan ',namsub-2)
C1.09         DO 40 I=1,namsub-1
         DO 40 I=1,namsub
          call mhsdtr(luamsb,i,ios)
C          mhs_scnlintime = MSECB-(namsub-i)*2667-8000  ! cor TIP time  msecs
          mhs_scnlintime = MSECB-(namsub-i)*2667-8000-2667  ! v1.07
          IF (mhs_scnlintime.lt.0) THEN
           mhs_scnlintime = mhs_scnlintime + 86400000
           mhs_scnlindy = daya - 1
          ENDIF
          call mhsdtw(luamsb,i,ios)
 40      CONTINUE
        ENDIF
       ENDIF
      ENDIF

C!..reset full flag
      FULLAM = .FALSE.

C!..Update time
      MSECA = MSECA+8000
      if (mseca.gt.86400000)then
        mseca = mseca - 86400000
      endif

C!..zero arrays
      DO 10 J = 1 , 620
        AMSUA1(J) = 0
 10   CONTINUE
      DO 20 J = 1 , 156
        AMSUA2(J) = 0
 20   CONTINUE
      DO 30 J = 1 , 1950
        AMSUB(J) = 0
 30   CONTINUE
C!
      RETURN
      END
