C!
C!ROUTINE          AMSUQC    SUBROUTINE  FORT77
C!
C!PURPOSE          TO PERFORM QUALITY CONTROL CHECKS ON AMSU MINOR FRAMES
C!
C!DESCRIPTION      Check some AMSU minor frame parameters and flag if error:
C!                   -- Check AMSU frame synch bytes OK
C!                   -- Check AMSU minor frame no. OK
C!                   -- Check AMSU major frame no. OK
C!                   -- If AMSU parity error flag
C!
C!VERSION          1.01   140395  R.W.SAUNDERS
C!                 1.02   180996  N.C.Atkinson
C!                 1.03   March 03 T.LABROT and P.BRUNEL
C!   f90 test: I.AND.J not correct and modified by IAND(I,J)
C!             I.OR.J not correct and modified by IOR(I,J)
C!             Cut lines exceeding 72 characters.
C!             Changed type XXX/yyy/
C!                  to type XXX
C!                     parameter(XXX=yyy)   or data XXX/yyy/
C!
C!ARGUMENTS
C!      INTEGER MF         -- Current AMSU Minor frame number (1-5)
C!      INTEGER AMF        -- AMSU Minor frame number (0->79)
C!      INTEGER AERR       -- AMSU-A Error flag  0 = OK
C!      INTEGER BERR(3)    -- AMSU-B Error flag  0 = OK
C!                         -- for AERR and BERR if the
C!        bit is on (i.e., if it is set to 1) then the statement is
C!        true. Otherwise it is false.
C!                        General
C!        bit 31: do not use scan for product generation
C!        bit 30: time sequence error detected with this scan (see
C!                below)
C!        bit 29: data gap precedes this scan
C!        bit 28: no calibration (see below)
C!        bit 27: no earth location (see below)
C!        bit 26: first good time following a clock update
C!        bit 25: instrument status changed with this scan
C!        bit 24-4: spare
C!
C!                        TIP/AMSU Problem Code
C!        bit 3: AMSU synch error
C!        bit 2: AMSU minor frame sequence error
C!        bit 1: AMSU major frame sequence error
C!        bit 0: TIP parity error
C!        remaining bits are set by calibration and navigation routines
C!
C!      INTEGER*2 HRPAIP(104,5) -- current TIP array of 5 minor frames
C!      LOGICAL*1 FILLER   -- True when input minor frame is empty filler
C!
C! COMMON/gqc/timok,timco,partip,err_frsync,nb_frsync,err_auxsync,
C!     & nb_auxsync
C! COMMON /LOG/MSECS,DAY,NMF,NHIRS,NMSU,NAMSUA,NAMSUB,NDCS
C!
C!FILES            FORnnn LOG FILE OUTPUT
C!
C!REFERENCES       ATOVDC design document version 1.9 Nov 95
C!
C!CHANGES          01 None
C!                 02 Update ama_tipmf and amb_tipmf
C!
C!#########################################################################
C! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
C!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
C!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
C!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
C!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
C!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
C!#########################################################################
      SUBROUTINE AMSUQC(MF,AMF,AERR,BERR,HRPAIP,FILLER)
C!
      IMPLICIT NONE
C!
#include "hrptdc.h"
#include "ama1b.h"
#include "amb1b.h" 
C!
      LOGICAL*1 FILLER                  ! Logical flag for blank amsu frames
      INTEGER*2 HRPAIP(104,5)           ! 5 TIP/AMSU/Backfill minor frames
      INTEGER*4 WORD1,WORD2,WORD3       ! Temporary words
      INTEGER*4 AMF,AJMF,LAJMF,LASTMF   ! AMSU minor & major frame numbers
      INTEGER*4 MF                      ! Counter 1-5 of current frames
      INTEGER*4 AERR,BERR(3)            ! Error code (0=OK)
      INTEGER*4 Z3,ZFRAM                ! Bit masks
      PARAMETER (Z3=3)
      PARAMETER (ZFRAM=4)
      INTEGER*4 ZPAR,ZFRAM1             ! Bit masks
      PARAMETER (ZPAR=1)
      PARAMETER (ZFRAM1=2)
      INTEGER*4 ZSYNC                   ! Bit masks
      PARAMETER (ZSYNC=8)

      INTEGER*4 SYNC1,SYNC2             ! Note these are based on test dataset
      PARAMETER (SYNC1=243)
      PARAMETER (SYNC2=106)
      INTEGER*4 NMF,MSECS,DAY           ! No. of TIP maj/minor frames read
      INTEGER*4 NHIRS,NMSU,NAMSUA,NAMSUB,NDCS !No scan lines
      LOGICAL*1 FIRST
      DATA FIRST/.TRUE./
      CHARACTER*80 TXT
C!
      logical timok,err_frsync,partip(5)
      integer*4 nb_frsync,nb_auxsync
      logical err_auxsync
C!
      INTEGER*4 bline  
      LOGICAL*1 amsuparity(104,5),amsua_perror(5),amsub_perror(5)
C!
      COMMON/gqc/timok,partip,err_frsync,nb_frsync,
     & err_auxsync,nb_auxsync
      COMMON /LOG/MSECS,DAY,NMF,NHIRS,NMSU,NAMSUA,NAMSUB,NDCS
      COMMON /qcamsu/ amsuparity,amsua_perror,amsub_perror
C!
C!  Check 22 frame synch bits
C!
      WORD1 = HRPAIP(1,MF)
      WORD2 = HRPAIP(2,MF)
      WORD3 = HRPAIP(3,MF)
      IF (.NOT.FILLER .AND. (WORD1.NE.SYNC1.OR.WORD2.NE.SYNC2
     &    .OR.WORD3.NE.0)) THEN
C!        WRITE(LULOG,102)
C!        WRITE(LULOG,106)NMF,MSECS
        CALL ML_WT('info','AMSU Frame sync byte invalid')
        write(txt,106)NMF,MSECS
        call ml_wt('info',txt)
        AERR = IOR(AERR,ZSYNC)
        BERR(1) = IOR(BERR(1),ZSYNC)
        BERR(2) = IOR(BERR(2),ZSYNC)
        BERR(3) = IOR(BERR(3),ZSYNC)
      ENDIF
C!
C! ..Extract minor frame number
C!
        AMF = HRPAIP(5,MF)
C! ..Perform some quality control
        IF (AMF.GT.79 .OR. AMF.LT.0)THEN
C!          WRITE(LULOG,101)AMF
C!          WRITE(LULOG,106)NMF,MSECS
          write(txt,101)AMF
          call ml_wt('info',txt)
          write(txt,106)NMF,MSECS
          call ml_wt('info',txt)
          AERR = IOR(AERR,ZFRAM)
          BERR(1) = IOR(BERR(1),ZFRAM)
          BERR(2) = IOR(BERR(2),ZFRAM)
          BERR(3) = IOR(BERR(3),ZFRAM)
          AMF = -1
          GO TO 10
        ENDIF
C!  ..Reset scan line error flag if start of AMSU-A scan line
        IF( AMF.EQ.0 )THEN
          AERR = 0
          BERR(1) = 0
          BERR(2) = 0
          BERR(3) = 0
        ENDIF
C! ..Perform some more quality control
        IF (FIRST) THEN
          LASTMF = AMF
        ELSE
          IF (AMF .NE. LASTMF+1)THEN
C!            WRITE(LULOG,103)AMF,LASTMF
C!            WRITE(LULOG,106)NMF,MSECS
            write(txt,103)AMF,LASTMF
            call ml_wt('info',txt)
            write(txt,106)NMF,MSECS
            call ml_wt('info',txt)
            AERR = IOR(AERR,ZFRAM)
            BERR(1) = IOR(BERR(1),ZFRAM)
            BERR(2) = IOR(BERR(2),ZFRAM)
            BERR(3) = IOR(BERR(3),ZFRAM)
          ENDIF
          LASTMF = AMF
          IF(LASTMF.EQ.79)LASTMF=-1
        ENDIF
C!...Extract major frame number and check OK
        WORD1 = HRPAIP(6,MF)
        AJMF = IAND(WORD1,Z3)
        IF (FIRST) LAJMF = AJMF
        IF (.NOT.(FILLER .OR. amsuparity(6,mf))) AJMF=LAJMF     ! Replace unreliable value
        IF (FIRST .OR. (AMF.EQ.0))THEN
          amb_tipmf = AJMF
          ama_tipmf = AJMF
          LAJMF = AJMF
          FIRST = .FALSE.
        ELSE
          IF (AJMF .NE. LAJMF)THEN
C!            WRITE(LULOG,104)AJMF,LAJMF
C!            WRITE(LULOG,106)NMF,MSECS
            write(txt,104)AJMF,LAJMF
            call ml_wt('info',txt)
            write(txt,106)NMF,MSECS
            call ml_wt('info',txt)
            AERR = IOR(AERR,ZFRAM1)
            BERR(1) = IOR(BERR(1),ZFRAM1)
            BERR(2) = IOR(BERR(2),ZFRAM1)
            BERR(3) = IOR(BERR(3),ZFRAM1)
            LAJMF = AJMF
          ENDIF
        ENDIF
C!
C!  Parity check
C!
      IF (FILLER .OR. amsua_perror(MF)) AERR = IOR(AERR,ZPAR)   ! Parity errors (or gaps) on AMSU-A scanline
      BLINE=3
      IF (AMF.LE.53) BLINE=2
      IF (AMF.LE.27) BLINE=1
      IF (FILLER .OR. amsub_perror(MF)) 
     &   BERR(BLINE) = IOR(BERR(BLINE),ZPAR)     ! Parity errors (or gaps) on AMSU-B scanline
      IF (.NOT.FILLER .AND. (amsua_perror(MF).OR.amsub_perror(MF))) THEN
         call ml_wt('info','AMSU minor frame parity error')
         write(txt,106)NMF,MSECS
         call ml_wt('info',txt)
         ENDIF
C!
 10   RETURN
 102  FORMAT(1X,'***AMSU Frame sync byte invalid at ***')
 101  FORMAT(1X,'***Bad AMSU minor frame number=',I3,' at ***')
 103  FORMAT(1X,'***AMSU minor frame sequence error current=',
     & I3,' last=',I3,' at ***')
 104  FORMAT(1X,'***AMSU major frame sequence error current=',
     & I3,' last=',I3,' at***')
 105  FORMAT(1X,'***AMSU minor frame parity error at ***')
 106  FORMAT(1X,'  TIP major frame ',I4,' time ',I9,' msecs from 0Z')
      END
