!#############################################################################
!+
!ROUTINE     ANAGET   SUBROUTINE  FORT77
!
!PURPOSE     Read HIRS and AMSU analogue telemetry from TIP data
!
!VERSION     1.0      10/01/97    N. Atkinson
!            1.1      18/12/98    N. Atkinson
!
!ARGUMENTS
!   integer*4  imf            -- minor frame count, 1 to 5
!   integer*4  tipmf          -- TIP minor frame number, 0 to 319
!   integer*2  hrpaip(104,5)  -- 5 minor frames of 8-bit TIP data
!   integer*2  hianag(1)      -- HIRS analogue data (not yet implemented)
!   integer*2  amanag(71)     -- AMSU analogue data (A1, A2 and B)
!   integer*2  hivalid(1)     -- HIRS analogue validity, 1 if invalid
!   integer*2  amvalid(71)    -- AMSU analogue validity, 1 if invalid
!#############################################################################

      subroutine anaget(imf,tipmf,hrpaip,hianag,amanag,
     &    hivalid,amvalid)
!
      implicit none
c-----------------------------------------
c
c
!
!arguments
      integer*4 imf,tipmf
      integer*2 hrpaip(104,5),hianag(1),amanag(71)
      integer*2 hivalid(1),amvalid(71)
!local variables
      integer*2 amsu_subcom(62),amsu_mf(62),
     &  stx_subcom(15),stx_key(15),stx_mf(15),i
      integer*4 tipmfmod

!Analogue telemetry minor frame and subcom numbers, as given in UIIS.
!See also Table 2 of PS-2629774 'Performance Specification AMSU-B Processing 
!Requirements': note parameter 14 should be minor frame 8/168 (wrong in
!revision E and earlier).

! Define one to one relationship between parameter and minor frame. This
! updates each parameter every 16 seconds (except for minor frame 160).
! Note that minor frames 161 to 319 have the same parameters as 1 to 159.
!
      data amsu_mf/4,5,12,13,15,19,20,22,23,27,28,31,35,36,37,39,43,
     & 44,45,47,51,52,53,55,59,60,61,999,                       !AMSU-A1
     & 67,68,69,71,75,76,77,79,82,84,85,87,90,92,93,999,        !AMSU-A2
     & 160,109,112,116,117,120,123,124,125,16,17,80,81,8,63,99,
     & 100,101/                                                 !AMSU-B
!
      data amsu_subcom/28*2,     !AMSU-A1
     & 16*2,                     !AMSU-A2
     & 1,8*2,5*1,4*2/            !AMSU-B
!
! Add STX-1,2,3,4 status, STX-1,2,3 power, SARR-A power, SARR-B power.
! Some of these are updated every 8 seconds, some every 16 seconds, so use
! stx_key to identify which parameter corresponds to which minor frame.
! We need to update these parameters as frequently as possible since they may
! be used for bias correction.
!
      data stx_mf/49,129,39,119,41,121,111,48,128,50,130,40,120,114,2/
      data stx_key/1,1,2,2,3,3,4,5,5,6,6,7,7,8,9/
      data stx_subcom/13*1,2,2/

      tipmfmod = mod(tipmf-1,160)+1    !0 to 160 (0 is used for time)
      do i=1,62
        if (amsu_mf(i).eq.tipmfmod) then
          if (amsu_subcom(i).eq.1) then
            amanag(i) = hrpaip(11,imf)       !Subcom 1
          else
            amanag(i) = hrpaip(14,imf)       !Subcom 2
          endif
          amvalid(i) = 0
        endif
      enddo
!
      do i=1,15                        !Look at STX and SARR parameters
        if (stx_mf(i).eq.tipmfmod) then
          if (stx_subcom(i).eq.1) then
            amanag(62+stx_key(i)) = hrpaip(11,imf)       !Subcom 1
          else
            amanag(62+stx_key(i)) = hrpaip(14,imf)       !Subcom 2
          endif
          amvalid(62+stx_key(i)) = 0   !Invalid data flag set to zero
        endif
      enddo
      return
      end
