!+
!NOM/NAME $Id: avtipg.F 25 2006-02-03 15:43:53Z frna $
!
        subroutine avtipg(hrptmf)
!BUT/PURPOSE 
! fill the TIP header data part in a record of the LAC/HRPT 1B format 
! and the CPU A and B telem part from TIP data
! with the hrpt minor frame, it  updates the common avh1bdt
!
!DESCRIPTION 
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!  
!    fills the integer*2 avh_tipmfhd(7,5)  5 times :
!                words 1 to 3 :
!                  bit 47-28 : TIP minor frame sync
!                  bit 27-24 : satellite address
!                  bit 23    : command verification status
!                  bit 22-21 : TIP status
!                  bit 20-18 : TIP major frame counter
!                  bit 17-09 : dwell mode address
!                  bit 08-00 : TIP minor frame counter
!                word  4      : command verification
!                words 5 to 7 :
!                  bit 47-40 :  3.2 second digital B subcom1
!                  bit 39-32 : 32   second analog subcom
!                  bit 31-24 : 16   second analog subcom
!                  bit 23-16 :  1   second analog subcom
!                  bit 15-08 :  3.2 second digital B subcom 2
!                  bit 07-00 : 16   second analog subcom 2

!      use ISHFT FORTRAN extension
!PROPRIETAIRE/OWNER
!  
!Marcel Derrien
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0     28/03/95     none            Marcel Derrien
!         
!ARGUMENTS 
!       input : integer*2 hrptmf(11090)  ! hrpt minor frame to process  
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
! 
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!DECLARATIONS
!
! 
!    
!
!
!arguments
        integer*2 hrptmf(11090)
!commons

#include "avh1b.h"

!local variables
        integer i,j
        integer tipdat(14)
        integer cpua(6)
        integer word1,word2
!functions

!-
!  extract the useful TIP bytes from HRPT minor frame
        do i=1,5
          do j=1,14
            tipdat(j) = hrptmf(103+(i-1)*104+j)/4
          enddo
! updates tip mf header data
          do j=1,7
            word1 = ishft(tipdat((j-1)*2+1),8)
            word2 = tipdat((j-1)*2+2)
            word1=word1+word2
            avh_tipmfhd(j,i)=word1
          enddo
! extract the useful TIP bytes from HRPT minor frame
          do j= 46,51
            cpua(j-46+1)=hrptmf(103+(i-1)*104+j)/4
          enddo
          do j=96,101
            cpua(j-96+1)=hrptmf(103+(i-1)*104+j)/4
          enddo
! updates cpu a part
          do j=1,3
            word1=ishft(cpua((j-1)*2+1),8)
            word2=cpua((j-1)*2+2)
            word1=word1+word2
            avh_icputel(j,i)=word1
          enddo
! updates cpu b part
          do j=4,6
            word1=ishft(cpua((j-4)*2+1),8)
            word2=cpua((j-4)*2+2)
            word1=word1+word2
            avh_icputel(j,i)=word1
          enddo
        enddo
        return
        end
