!+
!NOM/NAME $Id: cktime.F 58 2006-11-08 10:31:36Z frna $
!
        subroutine cktime(iqcur,iqref,goodtime,nbbadmf,nbmismf)
!BUT/PURPOSE 
! check time consistency between two successive hrpt mf
! or between the current hrpt mf and the last with a synchro. time (iqref)
! increments nbbadmf and computes nbmismf the number of missing mf
!DESCRIPTION 
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
!       Marcel Derrien
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0    16/02/95       no              M. Derrien
! 2.0    29/05/01   When the number of missing lines is negative, the HRPT minor frame
!                   is defined "bad" and nbmismf is put to 0. A message is written.  (T.Labrot)
! 2.1    March 03                         T. LABROT/K. Whyte
!   f90 test :  Cut lines exceeding 72 characters.
! 2.2    24-May-2004  New logic for missing/bad line detection (A.Korpela)
! 2.3    17/12/04     Fix run-time error when there is noisy data. N.C.Atkinson
! 2.4    16/10/06     Reduce max permissible gap to 2 minutes. This improves
!                     performance with noisy data at end of pass. NCA
!         
!ARGUMENTS
 
!       input : integer iqcur(3) ! current instant
!       input : integer iqref(3) ! reference instant
!       output: logical goodtime ! flag for time consistency
!       output: integer nbbadmf  ! number of bad mf
!       output: integer nbmismf  ! number of missing mf
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
! 
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!DECLARATIONS
!
! 
!    
!
!
!arguments
        integer iqcur(3) 
        integer iqref(3)  
        logical goodtime   
        integer nbbadmf
        integer nbmismf  
!commons

!local variables
!parameters
        double precision dij_min, dij_moy, dij_max
        parameter (dij_moy= (1000./6.) / 86400.d+03 )
        parameter (dij_min= (1000./6.-2.) / 86400.d+03 )
        parameter (dij_max= (1000./6.+2.) / 86400.d+03 )
        logical frst,flagprev,flagref
        data frst /.true./
        integer iqprev(3)
        save frst,iqprev       
!
        double precision ijref,ijcur,ijprev,dijref,dijprev
        double precision dij
!MOD/NIWA begin
        integer sinceref
!MOD/NIWA end
!functions
        if (frst) then
          iqprev(1)=iqref(1)
          iqprev(2)=iqref(2)
          iqprev(3)=iqref(3)
          nbbadmf=0
          frst=.false.
        endif
! verify with reference time        
        call xiqj(iqref,ijref)
        call xiqj(iqcur,ijcur)
        dijref=ijcur-ijref
        flagref=
     &    dijref.ge.dij_min*(nbbadmf+1)
     &    .and.dijref.le.dij_max*(nbbadmf+1)
!        print *,' dijref ',dijref,' flagref ',flagref
        if(.not.flagref) then

! verify with previous time
          call xiqj(iqprev,ijprev)
          dijprev=ijcur-ijprev
          flagprev=
     &    dijprev.ge.dij_min
     &    .and.dijprev.le.dij_max
                 
        endif
!        print *,' dijprev ',dijprev,' flagprev ',flagprev

!last chance    
        if(.not.flagprev.and..not.flagref) then
! verify changing year if 1rst day of year
          if(iqcur(2).eq.1.and.iqcur(3).lt.900000) then
            iqcur(1)=iqref(1)+1
            call xiqj(iqcur,ijcur)
            dij=ijcur-ijref
            goodtime=
     &       dij.ge.dij_min*(nbbadmf+1).and.dij.le.dij_max*(nbbadmf+1)
            if(goodtime) then
              flagref=.true.
              dijref=dij
            else
              iqcur(1)=iqref(1)
            endif
          endif
        endif
        goodtime=flagprev.or.flagref
!        print *,' iqref(3)',iqref(3),' iqprev(3) ',iqprev(3),
!     & ' iqcur(3) ',iqcur(3)
!MOD/NIWA begin replace
! Using this HRPT minor frame with
!   dijref     seconds elapsed from the last good HRPT minor frame
!   dij_moy    duration of one HRPT minor frame (1/6 sec)
!   nbbadmf    number of bad frames already processed since the reference
!   goodtime   flag if this is good HRPT minor frame
!   flagref    flag if this is goodtime after consistent number of bad HRPT minor frames
!   flagprev   flag if this is goodtime after one previous HRPT minor frame
! determine
!   nbmismf    number of missing frames since the reference
!   nbbadmf    number of bad frames since the reference
!   
!        if (dijref.LE.0 .OR. dijref.GT.3000*dij_moy) then      ! give up
        if (dijref.LE.0 .OR. dijref.GT.720*dij_moy) then !give up if 2 min(v2.4)
           goodtime=.FALSE.
           flagref=.FALSE.
           flagprev=.FALSE.
        else
           sinceref=nint(dijref/dij_moy)  ! relative count for this HRPT minor frame after the reference
        endif                                                                                                                             

        if (flagref) then               ! goodtime at this minor frame, after consistent number of bad HRPT minor frames
           nbmismf=0                    ! ==> no missing frames
             
        elseif (flagprev) then          ! goodtime at this minor frame, after missing frames + consistent previous frame
           nbbadmf=nbbadmf-1            ! ==> the previous frame was OK after all
           nbmismf=sinceref-nbbadmf-2   ! ==> we have nbbadmf bad frames and 2 good frames, the rest are missing frames
             
        else                            ! not goodtime at this minor frame
           nbbadmf=nbbadmf+1            ! ==> this is (another) bad frame
        endif
             
        if (goodtime) then              ! set the new reference HRPT minor frame
           iqref(1)=iqcur(1)
           iqref(2)=iqcur(2)
           iqref(3)=iqcur(3)
        endif
        goto 100
!MOD/NIWA end
        if(.not.goodtime) then
          nbbadmf=nbbadmf+1       
        else
          if(flagref) then
           nbmismf=nint(dijref/dij_moy) - 1
           if(nbbadmf.gt.0) nbmismf = nbmismf - 1
          else
           nbmismf=nint((ijprev-ijref)/dij_moy) -1
          endif
          if(nbmismf.lt.0) then
      call ml_wti('warning','Number of missing HRPT minor frames <0'
     &            ,nbmismf)
      call ml_wt('warning',
     &           'this minor frame is bad, nbmismf is forced to 0')
           goodtime=.false.
           nbbadmf=nbbadmf+1
           nbmismf=0
          else
            iqref(1)=iqcur(1)
            iqref(2)=iqcur(2)
            iqref(3)=iqcur(3)
          endif
        endif
!MOD/NIWA add continue
  100   continue
! save current time
        iqprev(1)=iqcur(1)
        iqprev(2)=iqcur(2)
        iqprev(3)=iqcur(3)                                   
!-
        return

        end
