!+
!NOM/NAME $Id: dcin.F 47 2006-08-21 08:44:41Z frna $
!
        subroutine dcin(optn,year,opmode,ninst,startorb,endorb)
!BUT/PURPOSE 
!
!       reads the control cards for HRPTDC 
!       card 1 optn (11 values)
!                  OPTN(1) -- 0 or 1 for level of error logging
!                  OPTN(2) -- 0 or 1 for HIRS/3 (1 indicates extract data)
!                  OPTN(3) -- 0 or 1 for AMSU-A1
!                  OPTN(4) -- 0 or 1 for AMSU-A2
!                  OPTN(5) -- 0 or 1 for AMSU-B
!                  OPTN(6) -- 0 or 1 for HIRS/2 
!                  OPTN(7) -- 0 or 1 for MSU
!                  OPTN(8) -- 0 or 1 for DCS
!                  OPTN(9) -- 0 or 1 for SEM
!                  OPTN(10)-- 0 or 1 for SBUV
!                  OPTN(11)-- 0 or 1 for SAR
!                  OPTN(12)-- 0 or 1 for AVHRR
!       card 2 fortran I/O stream for general log and output files
!       card 3 year of data
!       card 4 operational indicator (0 = off line, 1=real time)
!       card 5 orbit number start, orbit number end (to fill 1b Headers)
!DESCRIPTION 
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
! Marcel Derrien
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0    16/02/95       no              M. Derrien
! 1.1    18/08/06    also read station_id   N.Atkinson
!
!         
!ARGUMENTS 
!       output : integer optn(12)
!       output : integer year
!       output : integer opmode
!       output : integer ninst
!       output : integer startorb
!       output : integer endorb
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
! 
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!DECLARATIONS
!
! 
!    
!
!
!arguments
        integer optn(12)
        integer year
        integer opmode
        integer ninst
        integer startorb
        integer endorb
!commons
#include "hrptdc.h"
!local variables
        integer i,ios
        integer stream(12)
        equivalence(stream,lulog)
!functions

!-
        ninst=0
        read(*,*) optn
        read(*,*) lulog,luhirs3,luamsa1,luamsa2,luamsb,luhirs2,lumsu,
     &   ludcs,lusem,lusbuv,lusar,luavh
        read(*,*) year
        read(*,*) opmode
        read(*,*) startorb,endorb
        read(*,*,iostat=ios) station_id
        if (ios.ne.0) station_id='WE'           !default Western Europe
! open log file
!       open(lulog,
!     &         file='decommutation.log',recl=180,
!     &         status='unknown',form='formatted',
!     &         iostat=ios)
!       if(ios.ne.0) then
!          print *,' **Log file could not be opened**'
!          stop
!        endif
! check optn values
        do i = 1,12
          if(optn(i).gt.1 .or. optn(i).lt.0) then
!            write(lulog,*)
!     &      ' **Invalid option number--correct input**' ,optn(i)
            call ml_wti('error','Invalid option number '
     &          ,optn(i))
            call aappexit(11)
          endif
          if(optn(i).eq.1) ninst=ninst+1
        enddo
C! Allow for fact AMSU-A1 and A2 are treated as same dataset
        if(optn(3).eq.1.and.optn(4).eq.1) ninst=ninst-1
C! Check year
        if((year.ge.0.and.year.lt.1978).or.year.gt.2100) then
!         write(lulog,*) ' **Invalid year **' ,year
            call ml_wti('error','Invalid year ',year)
          call aappexit(11)
        endif       
        do i = 2 , 12
          if(stream(i).lt.0.or.stream(i).gt.255) then
!            write(lulog,*) ' **Invalid stream number** ',stream(i)
            call ml_wti('error','Invalid stream number',stream(i))
            call aappexit(11)
          endif
        enddo
        return
        end
