!+
!NOM/NAME $Id: dcsget.F 47 2006-08-21 08:44:41Z frna $
!
        subroutine dcsget
     &  (satid,imf,tipnum,tiptab,dcswrd4,dcsan,dcs)
!
!BUT/PURPOSE
!
! extraction of the dcs data, digitalB and housekeeping data
!  from a TIP minor frame stored in an integer*2 array
!   containing samples without parity bits 
!   
!DESCRIPTION 
!
!     fills the dcsdat buffer according to the minor frame number
!     for the convenient TIP minor frames catch the digitalB bits
!     analog housekeeping data according to the NOAAKLM telemetry list doc.
!     into dcsan and dcswrd4
!     with a discussion on satid to keep the possibility
!     for pre NOAAK satellites which need more analog data
!
!PROPRIETAIRE/OWNER  
!    Marcel Derrien
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
!  1.0     10/05/95    none              Marcel Derrien
!  1.1     28/02/97    corrections to digital B extraction (Marcel Derrien)
!  1.2      June 98    corrections for noaa-k  (Marcel Derrien)
!  1.3      May 2000   corrections filling analog Tsat and 28V  (MD-TL)
!  1.4      March 03                       T. LABROT
!   f90 test : I.and.J not correct and modified by iand(I,J)
!               I and J must have the same type
!               Added a new local parameter i2word1=1
!  1.5     sept 2003   Change statement for test on noaa klm serie
!                                         P. Brunel
!       
!ARGUMENTS
!
!       input : integer satid
!       input : integer imf ( 1 to 5 index in tiptab)
!       input : integer tipnum ! number of the TIP minor frame (0-319)
!       input : integer*2 tiptab(104,5)
!       output :integer dcswrd4 from digital B
!       output : integer*2 dcsan(16)
!       output : integer*2 dcs(10240)
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!       nbdcswrd =0 if the satid is not OK (should never occur)
!       uses gp_bse to set bit in dig B wrd (integer word)
!
! 
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!DECLARATIONS
!
! 
!    
!
!arguments
        integer satid
        integer imf
        integer*2 tiptab(104,5)
        integer dcswrd4
        integer*2 dcsan(16)
        integer*2 dcs(10240)
!commons

!local variables
        integer id_in    ! noaa id corresponding to satid variable
        integer id_noaak ! noaa id corresponding to Noaa-K == 15
        logical frst
        integer bufad !index to store data in dcs array
        integer i,j ! work index
        integer tipnum ! tip minor frame number
        integer dcsad(32) ! array of dcs word locations in TIP minor frame
        integer nbdcswrd ! number of dcs words to get from TIP minor frame
        integer*2 dru1_st,dru2_st,dru3_st,dru4_st !analog
        integer*2 dru5_st,dru6_st,dru7_st,dru8_st !analog
        integer*2 pseud_mes_st,relay_a_st,relay_b_st !analog
        integer*2 mem_ov_st !analog
        integer*2 time_code!analog
        integer*2 i2word1
        parameter(i2word1=1)
        data frst/.true./
        data id_noaak/15/
        save

!functions
        integer noascid

!-
!     section 0     
!     init  section for the first call
!
        if(frst)then
           call dcsin(satid,dcsad,nbdcswrd)
           call ml_wti('info','number of dcs wrd to get',nbdcswrd)

           do i=1,10240
             dcs(i) = 0
           enddo

           do i=1,16
             dcsan(i) = 0
           enddo

           dcswrd4 = 0

           if(nbdcswrd.eq.0) then
            call ml_wt('error','bad satid passed to dcsget abort')
            call aappexit(12)
           endif
           frst = .false.
        endif   
!    section 1
!       extraction of DCS data
!       (take account word location from word 1) 
!
        j=imf
        bufad = nbdcswrd * tipnum + 1
        do i = 1 , nbdcswrd
           dcs( bufad ) = tiptab(dcsad(i) + 1,j)
           bufad = bufad + 1
        enddo
        

!    section 2      
!       extraction of digital B telemetry
!
            if(tipnum.eq.25) then
              dru7_st=iand( tiptab(9,j)/4 ,1 )
              if(dru7_st.eq.1) call gp_bse(dcswrd4,7)
              dru8_st=iand( tiptab(9,j)/2 ,1 )
              if(dru8_st.eq.1) call gp_bse(dcswrd4,6)
            elseif(tipnum.eq.26) then
              pseud_mes_st=iand( tiptab(9,j)/4 ,1)
              if(pseud_mes_st.eq.1) call gp_bse(dcswrd4,3)
            elseif(tipnum.eq.27) then
              relay_a_st = iand(tiptab(9,j),i2word1)
              if(relay_a_st.eq.1) call gp_bse(dcswrd4,15)
            elseif(tipnum.eq.28) then
              relay_b_st=iand( tiptab(9,j)/128 , 1)
              if(relay_b_st.eq.1) call gp_bse(dcswrd4,14)
              dru1_st=iand( tiptab(9,j)/64 , 1)
              if(dru1_st.eq.1) call gp_bse(dcswrd4,13)
              dru2_st=iand(tiptab(9,j)/32,1)
              if(dru2_st.eq.1) call gp_bse(dcswrd4,12)
              dru3_st=iand(tiptab(9,j)/16,1)
              if(dru3_st.eq.1) call gp_bse(dcswrd4,11)
              dru4_st=iand(tiptab(9,j)/8,1)
              if(dru4_st.eq.1) call gp_bse(dcswrd4,10)
              dru5_st=iand(tiptab(9,j)/4,1)
              if(dru5_st.eq.1) call gp_bse(dcswrd4,9)
              dru6_st=iand(tiptab(9,j)/2,1)
              if(dru6_st.eq.1) call gp_bse(dcswrd4,8)
              mem_ov_st=iand(tiptab(9,j),i2word1)
              if(mem_ov_st.eq.1) call gp_bse(dcswrd4,5)
            elseif(tipnum.eq.29) then
              time_code=iand( tiptab(9,j)/128 , 1)
              if(time_code.eq.1) call gp_bse(dcswrd4,4)
            endif
!    section 3
!           extraction of analog housekeeping (32 sec subcom) telemetry
!
          if(tipnum.eq.175)  dcsan(1) = tiptab(10,j) 
          if(tipnum.eq.167)  dcsan(2) = tiptab(10,j)
          if(tipnum.eq.47)  dcsan(3)  = tiptab(10,j)
          if(tipnum.eq.238)  dcsan(4) = tiptab(10,j)
          if(tipnum.eq.15)  dcsan(5)  = tiptab(10,j)
          if(tipnum.eq.31)  dcsan(6)  = tiptab(10,j)
          if(tipnum.eq.23)  dcsan(7)  = tiptab(10,j)
          if(tipnum.eq.39)  dcsan(8)  = tiptab(10,j)
          if(tipnum.eq.55)  dcsan(9)  = tiptab(10,j)
          if(tipnum.eq.63)  dcsan(10) = tiptab(10,j)
          if(tipnum.eq.115) dcsan(15) = tiptab(11,j)
          if(tipnum.eq.153) dcsan(16) = tiptab(10,j)
          
          ! test the noaa sc id (12 for noaa12) to
          ! determine if it is a klm serie or not
          id_in = noascid(satid)
          if(id_in .lt. id_noaak) then
          !if(satid.lt.12) then       
!
!         for pre noaak only  
!         extract the 4 supplementary 
!         analog houskeeping words
!
            if(tipnum.eq.71) dcsan(11) = tiptab(10,j)
            if(tipnum.eq.79) dcsan(12) = tiptab(10,j)
            if(tipnum.eq.87) dcsan(13) = tiptab(10,j)
            if(tipnum.eq.95) dcsan(14) = tiptab(10,j)
        else
          dcsan(11) = 0
          dcsan(12) = 0
          dcsan(13) = 0
          dcsan(14) = 0
        endif
!
        return
        end
               
