!+
!NOM/NAME $Id: dcsin.F 25 2006-02-03 15:43:53Z frna $
!
        subroutine dcsin(satid,dcsad,nbdcswrd)
!BUT/PURPOSE 
! initialization of working variables for DCS decommutation
!    ( satellite dependent tables and variables)
!
!
!DESCRIPTION 
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
! with satid (spacecraft identification in the serie order)
!      delivers the DCS data adresses dcsad in the TIP minor frame
!      and the number of these adresses
!      according to the NOAA K,L,M system design report and the 
!      TIP minor frame format from NESS107 for the pre-NOAAK satellites
!     the word locations in TIP minor frames are counted from word 0
!      according to the documentation from NOAA
!
!PROPRIETAIRE/OWNER  
!    Marcel Derrien
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
!  1.0     10/05/95    none              Marcel Derrien
!  1.1     sept 2003  Change statement for test on noaa klm serie
!                                         P. Brunel
!       
!ARGUMENTS 
!
!       input  : integer satid
!       output : integer array dcsad
!       output : nbdcswrd 
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!       nbdcswrd =0 if the satid is not OK
!
! 
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!DECLARATIONS
!
! 
!    
!
!
        integer nbdcm
        parameter(nbdcm=32)
!arguments
        integer satid
        integer dcsad(nbdcm)
        integer nbdcswrd
!commons

!local variables
        integer id_in    ! noaa id corresponding to satid variable
!functions
        integer noascid

!-
        ! use the noaa sc id (12 for noaa12) to
        ! determine the correct satellite number
        id_in = noascid(satid)

        ! tiros-n Noaa6,7,8,9,10 and 12
        if(id_in.ge.0 .and. id_in.le.12 .and. id_in.ne.11) then
        !if(satid.ge.0.and.satid.le.8) then
          nbdcswrd=9
          dcsad(1) = 56 
          dcsad(2) = 57
          dcsad(3) = 64
          dcsad(4) = 65
          dcsad(5) = 90
          dcsad(6) = 91
          dcsad(7) = 94
          dcsad(8) = 95
          dcsad(9) = 102

        ! Noaa11
        elseif(id_in.eq.11) then
        !elseif(satid.eq.9) then
        
          nbdcswrd=12
          dcsad(1) = 28
          dcsad(2) = 29
          dcsad(3) = 56
          dcsad(4) = 57
          dcsad(5) = 64
          dcsad(6) = 65
          dcsad(7) = 72
          dcsad(8) = 73
          dcsad(9) = 90
          dcsad(10) = 91
          dcsad(11) = 94
          dcsad(12) = 95
          
        ! Noaa13 et noaa14
        elseif(id_in.eq.13 .or. id_in.eq.14) then
        !elseif(satid.eq.10.or.satid.eq.11) then
          nbdcswrd=15
          dcsad(1) = 28
          dcsad(2) = 29
          dcsad(3) = 52
          dcsad(4) = 53
          dcsad(5) = 56
          dcsad(6) = 57
          dcsad(7) = 60
          dcsad(8) = 64
          dcsad(9) = 65
          dcsad(10) = 72
          dcsad(11) = 73
          dcsad(12) = 90
          dcsad(13) = 91
          dcsad(14) = 94
          dcsad(15) = 95
          
        ! Noaa15 16 17....20
        elseif(id_in.ge.15 .and. id_in.le.20) then
        !elseif(satid.ge.12.and.satid.le.14) then
          nbdcswrd=32
          dcsad(1) = 18
          dcsad(2) = 19
          dcsad(3) = 24
          dcsad(4) = 25
          dcsad(5) = 28
          dcsad(6) = 29
          dcsad(7) = 32
          dcsad(8) = 33
          dcsad(9) = 40
          dcsad(10) = 41
          dcsad(11) = 44
          dcsad(12) = 45
          dcsad(13) = 52
          dcsad(14) = 53
          dcsad(15) = 56
          dcsad(16) = 57
          dcsad(17) = 60
          dcsad(18) = 61
          dcsad(19) = 64
          dcsad(20) = 65
          dcsad(21) = 68
          dcsad(22) = 69
          dcsad(23) = 72
          dcsad(24) = 73
          dcsad(25) = 76
          dcsad(26) = 77
          dcsad(27) = 86
          dcsad(28) = 87
          dcsad(29) = 90
          dcsad(30) = 91
          dcsad(31) = 94
          dcsad(32) = 95
        else
          nbdcswrd=0
        endif   
        return

        end
