!+
!NOM/NAME $Id: genqc.F 47 2006-08-21 08:44:41Z frna $
!
        subroutine genqc(hrptmf,iqcur,flagtime,nbmiscn,satid,orbmf,
     &                   curmfn,err_tip)
!BUT/PURPOSE 
! This module receives one hrpt minor frame as input 
!  It performs the general control quality for this minor frame
!  it checks the scan line number, date, time, minor frame number
!  it checks the parity bits for each TIP word of the five TIP minor frame
!  
!DESCRIPTION 
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
!   Marcel Derrien
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
!   1.0      07/03/95    no          Marcel Derrien
!   1.1      March 03                    T. LABROT
!   f90 test : I.and.J not correct and modified by iand(I,J)
!              I and J must have the same type
!              Added a new local parameter i2word120=120
!   1.2     August 03 tsatid(i) for tsatid  Keith Whyte
!   1.3     April 04 Change "parity" to "parityOK" for portability   A Korpela
!   1.4     July 04  Change statement for test on noaa klm serie
!                                         P. Brunel
!         
!ARGUMENTS 
!       input  : integer*2 hrptmf(11090) ! one HRPT minor frame
!       input  : integer iqcur(3) year,day,ms for current hrpt minor frame 
!       input  : logical flagtime flag for time code
!       input  : integer nbmiscn ! nb of missing scans
!       output : integer satid  ! sat. number in prelaunch order 
!       output : integer orbmf  ! absolute scan line number in the orbit
!       output : integer curmfn ! current hrpt minor frame number
!       output : integer err_tip(5) ! contains the nb. of TIP word parity errors
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
! 
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!DECLARATIONS
!
! 
!    
!
!
!arguments
        integer*2 hrptmf(11090)
        integer iqcur(3)
        logical flagtime
        integer nbmiscn
        integer satid
        integer orbmf
        integer curmfn
        integer err_tip(5)
!parameters
        double precision dij_min, dij_moy, dij_max
        parameter (dij_moy= (10000./6.) / 86400.d+03 )
        parameter (dij_min= (dij_moy-2.) / 86400.d+03 )
        parameter (dij_max= (dij_moy+2.) / 86400.d+03 )


!commons
#include "avh1b.h"
#include "hrptdc.h"
        integer id_in    ! noaa id corresponding to satid variable
        integer id_noaak ! noaa id corresponding to Noaa-K == 15
!       timok is true if hrpt time is ok false if not
!       err_frsync flags errors in frame sync
!       partip(5) flags parity errors in TIP minor frames
!       nb_frsync counts the bit errors in frame sync.
!       err_auxsync flags errors in auxiliarysync.
!       nb_auxsync counts bit errors in aux. sync.
        logical timok,err_frsync,partip(5)
        integer nb_frsync,nb_auxsync
        logical err_auxsync
        common/gqc/timok,partip,err_frsync,nb_frsync,err_auxsync,
     &             nb_auxsync
        logical*1 amsuparity(104,5),amsua_perror(5),amsub_perror(5)
        common /qcamsu/ amsuparity,amsua_perror,amsub_perror
        
!local variables
        character*80 txt
        logical frst       ! true if the first call
        character*6 scname
        integer prescn     ! previous scan line number
        integer hsatid     ! hrpt derived satid
        integer iq(3)
        integer jdate
        integer premfn     ! previous hrpt minor frame counter
        integer i,j
        integer tsatid(5),tipnum(5)
        integer tipsat,tipnum1
        integer sh,st
        integer*4 bytw
        integer sumperr
        double precision jul1
        logical ended
        logical ltsat,ltipnum,found
        logical prenoaak
!     just added for a test
!       integer*2 seb(5)
!     end test
        integer status
        integer*2 frsync(6)  ! expected frame sync from hrpt (1 to 6)
        data frsync /
     &  Z'0284',Z'016F',Z'035C',Z'019d',Z'020F',Z'0095'/

        integer*2 sp127(127) ! expected spare words in hrpt (624 to 750)
      data sp127 /
     & Z'0109',Z'01EE',Z'03AA',Z'00EE',Z'00B0',Z'02FB',Z'007D',Z'0124',
     & Z'0087',Z'008E',Z'0198',Z'0036',Z'01B1',Z'01C7',Z'025B',Z'0304',
     & Z'0273',Z'02D3',Z'01AA',Z'02FD',Z'00A9',Z'018D',Z'02E8',Z'0277',
     & Z'024B',Z'0164',Z'0148',Z'0007',Z'0310',Z'008B',Z'0126',Z'00CB',
     & Z'00E9',Z'0042',Z'0261',Z'00FF',Z'02F6',Z'01F3',Z'0044',Z'02B5',
     & Z'0056',Z'0099',Z'030A',Z'0397',Z'03E7',Z'00AF',Z'0159',Z'0241',
     & Z'0018',Z'00B2',Z'02B7',Z'001A',Z'00FE',Z'02D0',Z'01C0',Z'02A9',
     & Z'039E',Z'02F1',Z'0101',Z'00DE',Z'0237',Z'0384',Z'01ED',Z'03C0',
     & Z'00BA',Z'0387',Z'0187',Z'0394',Z'038D',Z'00FB',Z'026E',Z'013D',
     & Z'03F1',Z'021B',Z'02CB',Z'02FA',Z'005B',Z'0117',Z'026A',Z'01A5',
     & Z'033F',Z'01AE',Z'0265',Z'0067',Z'0238',Z'0246',Z'00EA',Z'0028',
     & Z'0235',Z'03C8',Z'018A',Z'021A',Z'02ED',Z'02C9',Z'02B6',Z'003C',
     & Z'00CD',Z'003D',Z'00EB',Z'000E',Z'0206',Z'0125',Z'00A1',Z'00BD',
     & Z'0375',Z'011D',Z'0316',Z'005F',Z'018F',Z'02A4',Z'0210',Z'0391',
     & Z'0333',Z'0006',Z'0336',Z'00B8',Z'03CB',Z'01E0',Z'024E',Z'01DA',
     & Z'01B5',Z'015F',Z'0295',Z'00B1',Z'02DD',Z'004E',Z'03C9'/
     
       integer*2 auxsync(100) ! expected auxiliary sync (hrpt 10991 to 11090)
        data auxsync /
     & Z'03E2',Z'03F3',Z'01B5',Z'02BD',Z'0284',Z'0115',Z'01C4',Z'01D3',
     & Z'0341',Z'00E0',Z'02B6',Z'03DE',Z'00DC',Z'0199',Z'03F2',Z'0193',
     & Z'028E',Z'0069',Z'003E',Z'0363',Z'004B',Z'0095',Z'0140',Z'02D5',
     & Z'029C',Z'0245',Z'0362',Z'006D',Z'00A6',Z'03AD',Z'03FE',Z'003B',
     & Z'03DD',Z'00B6',Z'01CD',Z'00C5',Z'02EF',Z'0167',Z'02C0',Z'0042',
     & Z'0183',Z'00EE',Z'0352',Z'02EA',Z'01D9',Z'023D',Z'011A',Z'0006',
     & Z'00D4',Z'00A9',Z'026F',Z'02F9',Z'03D3',Z'0152',Z'00F9',Z'01C0',
     & Z'014B',Z'038F',Z'0355',Z'0218',Z'0143',Z'02BF',Z'02C8',Z'0172',
     & Z'001E',Z'0384',Z'020F',Z'03D1',Z'011E',Z'009E',Z'001A',Z'031C',
     & Z'02C1',Z'0064',Z'01B0',Z'0203',Z'0279',Z'004D',Z'0041',Z'01E9',
     & Z'00BA',Z'0065',Z'0196',Z'0230',Z'0094',Z'0166',Z'02E6',Z'0071',
     & Z'036E',Z'01C5',Z'01F5',Z'0372',Z'020D',Z'039D',Z'0179',Z'0144',
     & Z'024D',Z'0252',Z'01F0',Z'03CC'/
     
        data frst /.true./
        integer*4 MSB,LSB,ZFF
        parameter(MSB=255)
        parameter(LSB=255)
        parameter(ZFF=255)
        integer*2 i2word120
        parameter (i2word120=120)
        data id_noaak/15/
!functions
        integer ordavid,ordtiid,noascid
        character*6 noascnam
        integer gp_fb2
        logical parityOK
!-
!---------  it is the first call  ----------------------
        if (frst) then
          orbmf = 0                   
          prescn =  0                       
          iq(1) =  iqcur(1)
          iq(2) =  iqcur(2)
          iq(3) =  iqcur(3)
          call xdqj(iq,jdate) 
          call xiqj(iq,jul1)  
          premfn = iand( (hrptmf(7)/128),3 ) -1
          if(premfn.lt.0) premfn = 0 ! previous hrpt minor frame counter
          hsatid = ( iand(hrptmf(7),i2word120) )/8
          
!--------------------------------------------
!         get tip satid and tip minor frame counter
!         delivers tipsat, and tipnum1 tip minor counter for
!         the first minor frame
!         check tipsat against hrpt satid
!-------------------------------------------
          i = 0
          ltsat = .false.
          ltipnum = .false.
          found = .false.
          ended =.false.
          do while (.not.ended)
            ended=i.eq.4
            i = i+1
            tipnum(i) = (hrptmf(108+(i-1)*104)/4)*256 + 
     &                 hrptmf(109+(i-1)*104)/4
            tsatid(i) = iand((hrptmf(106+(i-1)*104)/4),15)
            
            if(i.gt.1) then
              if(tsatid(i).eq.tsatid(i-1)) then
                tipsat = tsatid(i)
                ltsat = .true.
              endif
              if(tipnum(i).eq.(tipnum(i-1)+1)) then
                tipnum1 =  tipnum(i)-i+1
                if(tipnum1.lt.0) tipnum1 = 320 + tipnum1
                ltipnum = .true.
              endif
            endif
            found=ltipnum.and.ltsat
            if(found) ended=.true.
          enddo 
          if(found) then
            write(txt,*) 'hrpt satid',hsatid,' tip satid',tipsat 
            call ml_wt('info',txt)
            sh = ordavid(hsatid,jdate)
            st = ordtiid(tipsat,jdate)
            if(sh.eq.st.and.sh.ne.0.and.st.ne.0) then
              satid = sh
              ! test the noaa sc id (12 for noaa12) to
              ! determine if it is a klm serie or not
              id_in = noascid(satid)
              if(id_in .lt. id_noaak) then
                prenoaak=.true.
              else
                 prenoaak=.false.
              endif
              scname=noascnam(sh)
              call ml_wt('info',scname)
              if(prenoaak) call ml_wt('info','pre noaak satellite')
            else
!             fatal error abort process
!             hrpt satid and tip satid are not consistent with the tables  
              call ml_wt('error','satids not consistent with the tables'
     &        //' or without operational status')
              status=12
              call aappexit(status)
            endif 
            frst=.false.
          endif         
        endif 
!----------------- end of first call considerations-------------   
!        
! get a good tip minor frame counter for current scan
!
        i = 0
        ltipnum = .false.
        ended =.false.
        do while (.not.ended)
          ended=i.eq.4
          i = i+1
        tipnum(i) = iand( hrptmf(108+(i-1)*104)/4 , 1)
        tipnum(i) = tipnum(i)*256 +
     &                 hrptmf(109+(i-1)*104)/4
        
          if(i.gt.1) then
            if(tipnum(i).eq.(tipnum(i-1)+1)) then
                tipnum1 =  tipnum(i)-i+1
                if(tipnum1.lt.0) tipnum1 = 320 + tipnum1
                ltipnum = .true.
            endif
          endif
          if(ltipnum) ended=.true.
        enddo
!
        curmfn = iand((hrptmf(7)/128),3)

!
        orbmf=orbmf+1+nbmiscn
!      
        timok = flagtime
!
        if (.not.prenoaak.and.curmfn.eq.3) then
           do i = 1,5
              do j = 1,104      ! Save parity check for each word in the AMSU minor frame
                 amsuparity(j,i)=parityOK(hrptmf(103+(i-1)*104+j),bytw)
                 enddo
              amsua_perror(i)=.FALSE.
              amsub_perror(i)=.FALSE.
              do j = 9, 48      ! Are the parity errors affecting AMSU-A data in this minor frame ?
                 IF (.not.amsuparity(j,i)) amsua_perror(i)=.TRUE.
                 enddo
              do j = 49, 98     ! Are the parity errors affecting AMSU-B data in this minor frame ?
                 IF (.not.amsuparity(j,i)) amsub_perror(i)=.TRUE.
                 enddo
              enddo
           endif
        if(prenoaak.or.curmfn.eq.1) then        
          sumperr=0
          do i = 1,5
            err_tip(i) = 0         
!check parity bits for each TIP minor frame word
            do j = 1,104          
              if(.not.parityOK(hrptmf(103+(i-1)*104+j),bytw))
     &                              err_tip(i) = err_tip(i)+1
            enddo
           if(err_tip(i).ne.0) then
             partip(i)=.true.
           else
             partip(i)=.false.
           endif
!------------------------------------------------------------------
!check group parity with word 104 not retained for the moment
!this check is more complex is to be introduced here if necessary
!------------------------------------------------------------------
            sumperr=err_tip(i)+sumperr
          enddo
!         if(sumperr.ne.0) then
!            write(txt,*)sumperr,' TIP parity errors for scan line ',
!     &       orbmf
!            call ml_wt('info',txt)
!          endif
        endif
!------------------------------------------
!check frame sync in the first 6 HRPT words
!------------------------------------------
        err_frsync = .false.
        nb_frsync = 0
        do i = 1,6
          if(frsync(i).ne.hrptmf(i)) then
            err_frsync = .true.
            nb_frsync = nb_frsync + 
     &                  gp_fb2(frsync(i),hrptmf(i),10)
          endif
        enddo
                
!------------------
!check spare words ( not retained )
!------------------
!        err_sp127 = .false.
!        nb_sp127 = 0
!        do i = 1,127
!          if(sp127(i).ne.hrptmf(623+i)) then
!            err_sp127 = .true.
!            nb_sp127 = nb_sp127 +
!     &                 gp_fb2(sp127(i),hrptmf(623+i),10)
!          endif
!        enddo
!--------------
!check aux sync
!--------------
        err_auxsync = .false.
        nb_auxsync = 0
        do i = 1 , 100
          if(auxsync(i).ne.hrptmf(10990+i)) then 
            err_auxsync = .true.
            nb_auxsync = nb_auxsync +
     &                   gp_fb2(hrptmf(10990+i),auxsync(i),10)
          endif
        enddo 
!--------------------------------
! end of general quality controls
!-------------------------------- 
!--------------------------------
! add test for debugging
!--------------------------------
!       do i=1,5
!          tipnum(i) = (hrptmf(108+(i-1)*104)/4.and.1)*256 + 
!     &                 hrptmf(109+(i-1)*104)/4
!          seb(i) = (hrptmf(104 + 14 + (i-1)*104)/4).and.255
!          type *,tipnum(i),seb(i)
!        enddo
                   
        return
        end
