c!
c!routine          getmf    subroutine  fort77
c!
c!purpose          extract tip minor & major frame numbers from 5 minor frames
c!
c!description      the tip words 4-5 in the minor frame contain the
c!                 tip minor frame counter. this is extracted and placed
c!                 in tipmf for the input tip minor frame.
c!                 some quality control is carried out on the major and
c!                 minor frame numbers. also the scan line error flag (err)
c!                 is reset to zero if a start of a hirs scan line is
c!                 encountered at minor frames 1,65,129,193 & 257.
c!                 the minor frame number is packed as follows:
c!   word offset             5        6
c!   (1 relative)       -------------------
c!                      |-------n|nnnnnnnn|
c!                      -------------------
c!                      9 bit minor frame no
c!
c!                 the major frame number is packed as follows:
c!   word offset             4
c!   (1 relative)       ----------       **warning this does not agree with
c!                      |---nnn--|         table 3.11 of ness 107 but is
c!                      ----------         consistent with preing
c!                  3 bit major frame no
c!
c!version          1.01   170195  r.w.saunders
c!                 1.02   130598  marcel derrien 
c!                        replace commn ama1bdt & amb1bdt by include files
c!                        to remove reference to address in amadat & ambdat
c!                 1.03   March 03  T.Labrot
c!    f90 test: I.and.J not correct and modified by iand(I,J)
c!              I.or.J not correct and modified by ior(I,J)
c!              Changed type XXX/yyy/
c!                   to type XXX
c!                      parameter(XXX=yyy)   or data XXX/yyy/
c!                 1.04   Sep 03  N.C.Atkinson, add mhs_tipmf
c!
c!arguments
c!      integer imf             -- minor frame 1->5 in hrpt array
c!      integer tipmf           -- tip minor frame number (0->319)
c!      integer tipjmf          -- tip major frame counter (0-->7)
c!      integer err             -- error flag (0=ok)
c!      integer*2 hrpaip(104,5) -- current tip array of 5 minor frames
c!
c! common/iocom/ lulog,luhirs3,luamsa1,luamsa2,luamsb,luhirs2,lumsu,
c!               ludcs,lusem,lusbuv,lusar,luavh
c! common /log/msecs,day,nmf,nhirs,nmsu,namsua,namsub,ndcs
c! common /ama1bdt/ amadat
c! common /amb1bdt/ ambdat
c!
c!files            fornnn log file output
c!
c!references       atovdc design document version 1.5 30 may 95
c!
c!changes          01 none
c!
c!#########################################################################
c! copyright        the contents of this file are crown copyright. the
c!                  software, data and other proprietary information
c!                  contained therein is supplied for bonafide research
c!                  purposes. commercial or business use is not permitted
c!                  without the written authority of the u.k. met. office.
c!                  at london rd., bracknell, berks, rg12 2sz, u.k.
c!#########################################################################
      subroutine getmf(imf,tipmf,tipjmf,err,hrpaip)
c!
      implicit none
c!
c!    integer*2 amadat(1280),ambdat(1280) !1a rec
      integer*2 hrpaip(104,5)           !5 tip/amsu/backfill minor frames
      integer*4 imf                     ! minor frame counter 1->5
      integer*4 word1,word2             ! temporary words
      integer*4 err                     ! error code (0=ok)
      integer*4 tipjmf                  ! major frame in tip data
      integer*4 tipmf                   ! array to contain tip minor frame no.s
      integer*4 z1,z7                   ! hexadecimal masks
      parameter(z1=1)
      parameter(z7=7)
      integer*4 zfram                   ! hexadecimal masks
      parameter(zfram=262144)
      
      integer*4 lastmf,lasjmf           ! last minor,major frame numbers 
                                        ! for checking
      integer*4 nmf,msecs,day           ! no. of tip major frames read
      integer*4 nhirs,nmsu,namsua,namsub,ndcs !no scan lines
      character*80 txt

      logical*1 first
      data first/.true./
c!
#include "hrptdc.h"
#include "hrs1b.h"
#include "msu1b.h"
#include "ama1b.h"
#include "amb1b.h"
#include "mhs1b.h"
!      common /ama1bdt/ amadat
!      common /amb1bdt/ ambdat
      common /log/msecs,day,nmf,nhirs,nmsu,namsua,namsub,ndcs
c!
c! ..extract minor frame number
        word1 = hrpaip(5,imf)
        word1 = IAND(word1,z1)
        word1 = word1*256                 ! shift left 8 bits
        word2 = hrpaip(6,imf)
        tipmf = word1 + word2
c! ..perform some quality control
        if (tipmf.gt.319 .or. tipmf.lt.0)then
c!          write(lulog,101)tipmf
          call ml_wti('info','bad tip minor frame number=',tipmf)
c!          write(lulog,104)nmf,msecs
          write(txt,104)nmf,msecs
          call ml_wt('info',txt)
          tipmf = -1
          err = IOR(err,zfram)
          go to 10
        endif
c!  ..reset scan line error flag if start of hirs scan line
        if(tipmf.eq.1 .or. tipmf.eq.65 .or. tipmf.eq.129
     &  .or. tipmf.eq.193 .or. tipmf.eq.257 )err = 0
c! ..perform some more quality control
        if (first) then
          lastmf = tipmf
        else
          if (tipmf .ne. (lastmf+1))then
c!            write(lulog,102)tipmf,lastmf

            write(txt,102) tipmf,lastmf

            call ml_wt('info',txt)

c!            write(lulog,104)nmf,msecs
            write(txt,104)nmf,msecs
            call ml_wt('info',txt)

            err = IOR(err,zfram)
          endif
          lastmf = tipmf
          if(lastmf.eq.319)lastmf=-1
        endif
c!...update major frame counter if necessary
        if(tipmf .eq. 0)then
          lasjmf = lasjmf+1
          if ( lasjmf .gt. 7)lasjmf = 0
        endif
c!...extract major frame number and check ok
        word1 = hrpaip(4,imf)
        word1 = word1/4                 ! shift right 3 bits
        tipjmf = IAND(word1,z7)
        if (first)then
          lasjmf = tipjmf
          first = .false.
        else
          if (tipjmf .ne. lasjmf)then
c!           write(lulog,103)tipjmf,lasjmf

            write(txt,103) tipjmf,lasjmf
            call ml_wt('info',txt)

c!            write(lulog,104)nmf,msecs

            write(txt,104)nmf,msecs
            call ml_wt('info',txt)
            err = IOR(err,zfram)
            lasjmf = tipjmf
          endif
        endif
c!
c! update data records variables with tip major frame counter
c!
        hrs_mjfrcnt = tipjmf
        hrs_scnpos = int(tipmf/64)
        msu_mjfrcnt = tipjmf
        ama_tipmf = tipjmf
        amb_tipmf = tipjmf
        mhs_tipmf = tipjmf
!        amadat(26) = tipjmf
!        ambdat(15) = tipjmf
c!
  10  return
 101  format(1x,'bad tip minor frame number=',i3,'*** at')
 102  format(1x,'tip mf number seq err current=',
     & i3,' last=',i3,' at ')
 103  format(1x,'tip mjf number seq err current=',
     & i3,' last=',i3,' at ')
 104  format(1x,'   tip major frame ',i4,' at ',i9,' msecs')
      end
