c!
c!routine          hirget   subroutine  fort77
c!
c!purpose          to extract hirs-3 data from noaa-k tip minor frame
c!
c!description      each tip minor frame contains 36 words of hirs/3 data
c!                 note the position of the first word is different to
c!                 that for pre-noaa-k tip.
c!                 these words are extracted and placed in a hirs scan line
c!                 buffer. when each scan line is full a flag is set.
c!
c!version          1.01   140395  r.w.saunders
c!                 1.02           m. derrien hirs array size to 1152
c!                 1.03   March 03  T.Labrot
c!   f90 test: I.and.J not correct and modified by iand(I,J)
c!             Changed type XXX/yyy/
c!                  to type XXX
c!                     parameter(XXX=yyy)   or data XXX/yyy/
c!
c!arguments
c!      integer imf       -- 1-5 to identify which minor frame
c!      integer tipmf     -- tip minor frame no.
c!      integer*2 hrpaip(104,5) -- current tip array
c!      integer*2 hirs(1152) -- hirs scan line
c!      logical*1 fullhi  -- set to true when hirs scan line filled
c!
c!files            fornnn log file output
c!
c!references       atovdc design document version 1.9 nov 95
c!
c!changes          01 none
c!
c!#########################################################################
c! copyright        the contents of this file are crown copyright. the
c!                  software, data and other proprietary information
c!                  contained therein is supplied for bonafide research
c!                  purposes. commercial or business use is not permitted
c!                  without the written authority of the u.k. met. office.
c!                  at london rd., bracknell, berks, rg12 2sz, u.k.
c!#########################################################################
c!
      subroutine hirget(imf,tipmf,hrpaip,hirs,fullhi)
c!
      implicit none
c-----------------------------------------
c
c
c!
      integer*2 hrpaip(104,5)           !5 tip/amsu/backfill minor frames
      integer*4 tipmf                   ! tip minor frame number
      integer*4 imf                     ! tip minor frame counter
      integer*2 hirs(1152)              ! one hirs scan line
      integer*4 ivh                     ! output array word offsets
      integer*4 hioff(18)               ! input array word offsets
      integer*4 iw                      ! counters etc
      integer*4 zff                     ! 8 bit mask
      parameter (zff=255)
      integer*4 msb,lsb                 ! temporary space
      logical*1 fullhi                  ! logical flag for full lines
      logical*1 first
      data first/.true./
      
c! ...define word offsets note first word different from pre noaa-k
      data hioff/17,23,27,31,35,39,43,55,59,63,67,71,75,79,83,85,89,93/
c!
c!...don't try getting any data until at start of first hirs scan line
c!
      if (first)then
        fullhi = .false.
        if (mod(tipmf,64).eq.1)then                  ! start of hirs line
          first=.false.
        else
          return
        endif
      endif
c!
c!  compute word offset from tip minor frame number
c!
      ivh = mod(tipmf-1,64)
      if(ivh.lt.0)ivh=63
      ivh = 18*ivh
c!
c!  extract hirs/2 words
c!
      do 10 iw = 1 , 18
      msb = hrpaip(hioff(iw),imf)
      msb = IAND(msb , zff)
      msb = msb*256                         ! shift left 8 bits
      if(msb.gt.32767)msb=msb-65536
      lsb = hrpaip(hioff(iw)+1,imf)
      lsb = IAND(lsb , zff)
      hirs(iw+ivh) = msb + lsb
  10  continue
c!
c!   is output hirs array full?
c!
      if( tipmf .eq. 0 .or. tipmf.eq.64 .or. tipmf.eq.128
     &    .or. tipmf.eq.192 .or. tipmf.eq. 256)fullhi=.true.
c!
      return
      end
