c!
c!routine          hirhdu  subroutine  fort77
c!
c!purpose          to update a hirs 1a header
c!
c!description      this routine updates the hirs 1a header array.
c!
c!version          1.01   250195  r.w.saunders
c!                 1.02   12/5/98  Marcel Derrien
c!                                 last 1B format effects siteid, version...
c!                                 changed instrument status bits  
c!                 1.03  20/11/98 correction of TIP source code 
c!                               ( 4 instead of 1)    T. Labrot
c!                 1.04  March 03    T.Labrot
c!    f90 test: I.and.J not correct and modified by iand(I,J)
c!              I.or.J not correct and modified by ior(I,J)
c!              Changed type XXX/yyy/
c!                   to type XXX
c!                      parameter(XXX=yyy)   or data XXX/yyy/
c!                 1.05  Jun 2006 PM, character*16 l1bscnam + MA, MB, MC, MD in
c!                                datasetname
C!                 1.06  Aug 2006 NCA Use station_id instead of fixed 'WE'
C!                 1.07  Nov 2008 NCA Re-compute jday for *_h_enddatajd
c! 
c!arguments
c!      integer*2 hirs(1152) -- hirs scan line
c!      integer yr        -- year of data (e.g. 1997).
c!      integer day       -- tip day number
c!      integer msecs     -- tip msecs from midnight
c!      integer systim(3) -- current year, daynumber, secs from midnight
c!      integer satid     -- noaa satellite id
c!      integer err        -- error flag  0 = ok
c!      integer nhirs       -- current data scan line
c!      logical*1 gotime  -- set to true when first valid tip time available
c!      integer*4 startorb,endorb         ! orbit number (start and end)
c!
c! common/iocom/ lulog,luhirs3,luamsa1,luamsa2,luamsb,luhirs2,lumsu,
c!               ludcs,lusem,lusbuv,lusar,luavh
c! common/hrs1bhd/hrs_h_l1bversnb,..............
c!
c!routine          xdqj,l1bscid
c!
c!files            fornnn log file output
c!                 formmm hirs 1a file output
c!
c!references       atovdc design document version 1.3 11 jan 95
c!
c!changes          01 none
c!
c!#########################################################################
c! copyright        the contents of this file are crown copyright. the
c!                  software, data and other proprietary information
c!                  contained therein is supplied for bonafide research
c!                  purposes. commercial or business use is not permitted
c!                  without the written authority of the u.k. met. office.
c!                  at london rd., bracknell, berks, rg12 2sz, u.k.
c!#########################################################################
c!
      subroutine hirhdu(hirs,yr,day,msecs,systim,satid,err,nhirs,gotime,
     & startorb,endorb)
c!
      implicit none
c!                         
#include "nl1b.h"
#include "hrptdc.h"
#include "hrs1b.h"
c!
c!
      integer*2 hirs(1152)              ! one hirs scan line
      integer*4 nhirs                   ! record no
      integer*4 systim(3)               ! system yr, dayno, secs from midnight
      integer*4 day,msecs               ! tip day no and msecs from midnight
      integer*4 yr                      ! year of data
      integer*4 jday,dq(2)              ! julian day no (1=1 jan 1950)
      integer*4 word1                   ! temp storage
      integer*4 hirsid                  ! hirs instrument id
      integer*4 satid                   ! satellite id
      integer*4 startorb,endorb         ! orbit number (start and end)
      integer*4 instst,lastst           ! hirs status
      integer*4 err                     ! error code (0=ok)
      integer*4 errtim,errpar,errsyn    ! error counters
      integer*4 normal                  ! hirs normal status
      parameter (normal=49152)
      integer*4 z7,zf,z1                ! hex mask
      parameter (z7=7)
      parameter (zf=15)
      parameter (z1=1)
      
      integer*4 zhirs                   ! bit mask
      parameter (zhirs=33554432)
      
      integer*4 ztim1                   ! bit mask
      parameter (ztim1=1082130432)
      
      integer*4 zpar                    ! bit masks
      parameter (zpar=65536)
      
      integer*4 zsync                   ! bit masks
      parameter (zsync=524288)
      
      character*80 txt

      logical*1 first,gotime
      data first/.true./
      
      logical gp_btn
c!
      character*4 str1
      character*3 str2
      character*5 str3,str4
      character*16 sname
      integer cp_lsc, sname_len
      character*2 rname
      integer iq(3)
      integer ig(7)
        
      character*22 ic

      integer l1bscid
      character*16 l1bscnam
c!
c! .. get hirs instrument status
c!
      instst = 0
      word1 = hirs(18*63+3)
      word1 = IAND(word1,zf)
      if(gp_btn(word1,0)) then
         call gp_sb32(instst,14,1)
      else
         call gp_sb32(instst,14,0)
      endif    ! Electronics on/off
      if(gp_btn(word1,3)) then
        call gp_sb32(instst,15,1) 
      else
        call gp_sb32(instst,15,0) 
       endif    ! Instrument on/off
      if(gp_btn(word1,2)) then
        call gp_sb32(instst,12,1)
      else
        call gp_sb32(instst,12,0)       
      endif! Scan Motor on/off
      if(gp_btn(word1,1)) then
        call gp_sb32(instst,13,1)
      else
        call gp_sb32(instst,13,0)
      endif    ! Filter wheel on/off            
      if (instst .ne. normal) err=IOR(err,zhirs)
c!    force to normal if missing data ( instst = 0 in this case )
      if (instst .eq. 0) instst = normal
c!
      if(first)then
c!...initialise error counters
        errtim = 0
        errpar = 0
        errsyn = 0
        lastst = normal
c!
        if(gotime)first = .false.
c! .. get instrument id
        word1 = hirs(18*63+3)
        word1 = word1/16                ! shift right 4 bits
        hirsid = IAND(word1,z7)           ! mask off 3 ls bits
c! ..compute julian day
       dq(1)= yr
       dq(2)= day
       call xdqj(dq,jday)
c!
c!..create header record 1 (not all elements filled)
c!
c!..general information

        hrs_h_l1bversnb = libl1bvers    ! format version number
        hrs_h_l1bversyr = libl1bversyr  ! format version  year
        hrs_h_l1bversdy = libl1bversdy  ! format version daynumber
        hrs_h_hdrcnt = nb1bhd           ! no. of header records
        hrs_h_satid = l1bscid(satid)    ! noaa s/c id
        hrs_h_instid = hirsid           ! instrument id?
        hrs_h_datatyp = 5               ! hirs data type code
        hrs_h_tipsrc = 4                ! tip source code
        hrs_h_startdatajd = jday  
! start julian day no.(1=1 jan 1950)
        hrs_h_startdatayr = yr          ! start of data year
        hrs_h_startdatady = day          ! start of data day of year
        hrs_h_startdatatime = msecs-(nhirs-1)*6400 !corrected time of day msecs
        hrs_h_inststat1 = normal        ! hirs status
        hrs_h_statchrecnb = 0           ! record number of hirs status change

         iq(1)=yr
         iq(2)=day
         iq(3)=hrs_h_startdatatime
         write(str1,1000) yr
1000     format(i4.4)
         write(str2,1001) day
1001     format(i3.3)
         write(str3,1002) startorb
1002     format(i5.5)

         write(str4,1002) endorb
         sname=l1bscnam(satid)
         sname_len = cp_lsc( sname )
         rname=sname(1:1)//sname(sname_len:sname_len)
         call cp_ups(rname)
         call xiqg(iq,ig)
         call xigc(ig,ic)
         hrs_h_dataname=loccentre//'.HIRX.'//rname//
     &   '.D'//str1(3:4)//str2//'.S'//ic(11:12)//ic(14:15)//
     &   '.E    .B'//str3//str4(4:5)//'.'//station_id
        hrs_h_prblkid = 'B'//str3//str4(4:5)
        hrs_h_siteid=loccentre
        hrs_h_blank=' '

      endif
c! add end data date and time
      iq(1) = yr
      iq(2) = day
      iq(3) = msecs
      call xiqg(iq,ig)
      call xigc(ig,ic)
      dq(1)= yr
      dq(2)= day
      call xdqj(dq,jday)                     ! Compute Julian day
      hrs_h_enddatajd = jday                 ! end julian day no.
      hrs_h_enddatayr = yr                   ! end of data year
      hrs_h_enddatady = day                  ! end of data day of year
      hrs_h_enddatatime = msecs              ! end of data time of day msecs
      hrs_h_dataname(27:30)=ic(11:12)//ic(14:15)
c!
c!...dataset quality indicators
      if (lastst .ne. instst)then
! don't apply because noisy data may introduce changes 
! and only one change can be indicated
! as hirscl uses this value and assume ON/OFF or OFF/ON only
! the log will see the change
!        hrs_h_statchrecnb = nhirs + nb1bhd  ! record number of hirs status change
!        hrs_h_inststat2 = instst        ! hirs new status
         write(txt,100) nhirs,instst,lastst
100      format('hirs scan ',i4,' instr.status changes',2i4)
         call ml_wt('warning',txt)
      endif
      lastst = instst
c!
      hrs_h_scnlin = nhirs         ! number of scan lines in file
      hrs_h_callocsclin = 0             ! number of calib/navig lines
      hrs_h_misscnlin = 0               ! count of missing scan lines
      hrs_h_datagaps = 0                ! count of data gaps
      hrs_h_okdatafr = nhirs-errsyn     ! count of frames with no synch errors
      hrs_h_pacsparityerr = errpar      ! count of tip parity errors
      hrs_h_auxsyncerrsum = errsyn      ! count of all synch errors
      hrs_h_timeseqerr = errtim         ! count of tip timimg errors
      hrs_h_timeseqerrcode = 0          ! time sequence error code (more info?)
c!
c!  increment error counters if necessary
c!
      if (IAND(err,ztim1).gt.0)errtim=errtim+1
      if (IAND(err,zpar).gt.0)errpar=errpar+1
      if (IAND(err,zsync).gt.0)errsyn=errsyn+1
c!
      return
      end
