c!
c!routine          hirout  subroutine  fort77
c!
c!purpose          to update header & write hirs scan line to disk
c!
c!description      this routine updates both the hirs 1a header and data
c!                 records (one data record per scan line) and writes out
c!                 a 1a data record.
c!
c!version          1.01   250195  r.w.saunders
c!                 1.02   March 03  T.Labrot
c!       f90 test: Changed type XXX/yyy/
c!                      to type XXX
c!                         data XXX/yyy/
C!                 1.03   050304  N C Atkinson delete redundant variable MISSING
C!                 1.09   Apr 10  For portability, re-write the currect scan 
C!                                when correcting times of first few scan lines.
c! 
c!arguments
c!      integer*2 hirs(1152) -- hirs scan line
c!      integer*2 hianag(1)  -- hirs analogue data
c!      integer yr        -- year of data (e.g. 1997).
c!      integer msech     -- tip msecs from midnight
c!      integer systim(3) -- current year, dayno, msecs from midnight
c!      integer satid     -- noaa satellite id
c!      integer err       -- error flag (0=ok)
c!      logical*1 fullhi  -- set to true when hirs scan line filled
c!      logical*1 gotime  -- set to true when first valid tip time available
c!      integer*4 startorb,endorb         ! orbit number (start and end)
c!      integer hirs3     -- if set to 1 assumes hirs 3 otherwise hirs 2
c!
c! common/iocom/ lulog,luhirs3,luamsa1,luamsa2,luamsb,luhirs2,lumsu,
c!               ludcs,lusem,lusbuv,lusar,luavh
c! common /log/msecs,day,nmf,nhirs,nmsu,namsua,namsub,ndcs
c!
c!subprograms      hirhdu, hrsdtw, hirsdc
c!
c!files            fornnn log file output
c!                 formmm hirs 1a file output
c!
c!references       atovdc design document version 1.3 11 jan 95
c!
c!changes          01 none
c!
c!#########################################################################
c! copyright        the contents of this file are crown copyright. the
c!                  software, data and other proprietary information
c!                  contained therein is supplied for bonafide research
c!                  purposes. commercial or business use is not permitted
c!                  without the written authority of the u.k. met. office.
c!                  at london rd., bracknell, berks, rg12 2sz, u.k.
c!#########################################################################
c!
      subroutine hirout(hirs,hianag,yr,msech,systim,satid,err,
     &fullhi,gotime,startorb,endorb,hirs3)
c!
      implicit none
c!-----------------------------------------
c!
c!
#include "nl1b.h"
#include "hrs1b.h"
c!
      integer*2 hirs(1152)              ! one hirs scan line
      integer*4 systim(3)               !system year, dayno, msecs from midnt
      integer*4 day,msecs,msech,lmsecs  ! tip day no and msecs from midnight
      integer*4 ios
      integer*4 yr                      ! year of data
      integer*4 satid                   ! satellite id
      integer*4 err,j                   ! error code (0=ok)
      integer*4 hirs3                   ! hirs 3 indicator
      integer*4 startorb,endorb         ! orbit number (start and end)
      integer*2 hianag(1)               ! hirs 3 analog data
      integer*4 nmf                     ! no. of tip maj/minor frames read
      integer*4 nhirs,nmsu,namsua,namsub,ndcs !no scan lines
      integer*4 i
      integer s
      integer*4 lulog,luhirs3,luamsa1,luamsa2,luamsb,luhirs2,lumsu
      integer*4 ludcs,lusem,lusbuv,lusar,luavh
      logical*1 fullhi                  ! set to false when line written
      logical*1 first,gotime
      data first/.true./
      
      logical*1 start
      data start/.true./
c!
      common/iocom/ lulog,luhirs3,luamsa1,luamsa2,luamsb,luhirs2,lumsu,
     &          ludcs,lusem,lusbuv,lusar,luavh
      common /log/msecs,day,nmf,nhirs,nmsu,namsua,namsub,ndcs

c!
c!
      if(first)then
       first =.false.
       lmsecs = msech
       nhirs=0
      endif
c!
      nhirs = nhirs + 1
c!
c! ...update header record
c!
      call hirhdu(hirs,yr,day,msech,systim,satid,err,nhirs,gotime,
     &    startorb,endorb)

      lmsecs = msech
c!
      hrs_qualind = err                ! quality indicator
      s = 0
      do i = 1,1152
         s = s + hirs(i)
      enddo
      if ( s .eq. 0 ) call gp_sb32(hrs_qualind,31,1)
c!
c! ...decommutate hirs data
c!
      call hirsdc(hirs,hianag)
c!
c! add data date and time
c!

      hrs_scnlinyr = yr                 ! year
      hrs_scnlindy = day                ! day of year
      hrs_scnlintime = msech                ! msecs from 0z
      hrs_clockdrift = 0                  ! clock drift
      hrs_scnlin = nhirs         ! scan line number
      
!      write(lulog,*)' hirout scan hirs ',nhirs,' msech ',msech
c!
      hrs_qualind = err                ! quality indicator
      s = 0
      do i = 1,1152
         s = s + hirs(i)
      enddo
      if ( s .eq. 0 ) call gp_sb32(hrs_qualind,31,1)
c!
c! ...write 1a data record

      if(hirs3.eq.1) then
        call hrsdtw(luhirs3,nhirs,ios)
      else
        call hrsdtw(luhirs2,nhirs,ios)
      endif
c!
c! correct time on first few scan lines if necessary
c!
      if(start.and.gotime.and.nhirs.gt.1)then
        start = .false.
c!        write(lulog,9000) rec
        call ml_wti('info','hirs time correction before scan',nhirs)
        do 25 i = 1 , nhirs
         if(hirs3.eq.1)then
          call hrsdtr(luhirs3,i,ios)
          hrs_scnlintime = msech-(nhirs-i)*6400 ! corrected tip time of day msecs from 0z
          call hrsdtw(luhirs3,i,ios)
         else
          call hrsdtr(luhirs2,i,ios)
          hrs_scnlintime = msech-(nhirs-i)*6400 ! corrected tip time of day msecs from 0z
          call hrsdtw(luhirs2,i,ios)
         endif
 25     continue
      endif
       
c!..reset full flag
      fullhi = .false.
c!..increment time
      msech = msech + 6400
        if(msech.gt.86400000)then
          msech = msech - 86400000
          day = day+1
        endif
c!..zero array
      do  j = 1 , 1152
        hirs(j) = 0
      enddo
c!
      return
      end
