!+
!NOM/NAME $Id: hirsdc.F 313 2012-02-28 14:51:43Z frna $
 
      subroutine hirsdc(hirs,hianag)
!-----------------------------------------------------------------------
!
! BUT/PURPOSE: 
!  converts TIP HIRS data and analog housekeeping into NOAAK 
!             level 1b structure for one HIRS scan line
!
! DESCRIPTION: FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions
!       see documentations     
!
! PROPRIETAIRE/OWNER: PAUL FLAMENT
!
! HISTORIQUE/HISTORY  (MODIFICATIONS/CHANGES)
!
! version       date            comment                 author
! 1.0           27/04/95                               PAUL FLAMENT
! 1.1                          corrections             Marcel Derrien
! 1.2        March 03                                  Tiphaine Labrot
!         f90 test: With MOD(X,Y) X and Y must have the same type
!                   Defined integer*2 i2word2  and changed
!                   MOD(nboct(i),2) to MOD(nboct(i),i2word2)
!                   IAND(X,Y) X and Y must have the same type
!                   Defined integer*2 cst1
! 1.3        19/07/2011   ISHFT arguments should be same type
!                           
! ARGUMENTS: 
!
!   input:      integer*2 hirs(1152)
!
!               array containing the hirs data from 64 consecutive 
!               TIP minor frames for one HIRS scan line
!               (2304 bytes, i.e. 1152 integer*2, 64x36 bytes)
!
!   input:       integer*2 hianag(32)
!
!       64 digital B (32 sec)  data for one scan line
!           (two bytes in one integer*2)
!
!   output the hirs1bdt common : hrs_elem() 
!                                and hrs_digitbwrd
!           
!
!
! FUNCTIONS AND SUBROUTINES CALLS (RETURN CODES FOR FUNCTIONS):
!
! gp_x16  ! extracts one sample of nb bits in an integer*2 array 
!       and put it right justified in an integer*2
!
! gp_sb16 ! set one bit to 0 or 1 in an integer*2
!
! gp_gb16 ! delivers the status of a given bit in an integer*2
!
!
! COPYRIGHT(c) Meteo-France ( EVOLUNIX - ATOVS - EUMETSAT project 1994 )
!
!-----------------------------------------------------------------------
 
      implicit none
c-----------------------------------------
c
c

!---- DECLARATIONS -----------------------------------------------------

      integer*2 nboct(2:15) ! bytes numbers in the 32 last bytes from hianag 
                            ! hianag, containing the interesting bits for
                            ! hirs

      data nboct/23,29,5,29,22,32,29,22,31,4,29,22,30,22/

      integer*2 nbbit(0:15) ! bit number in the bytes for hirs digitbwrd

      data nbbit/0,0,7,6,6,5,5,5,4,4,3,3,2,2,1,1/

!---- ARGUMENTS --------------------------------------------------------

      integer*2 hirs(1152)
      integer*2 hianag(64)

!---- COMMONS ----------------------------------------------------------

#include "hrs1b.h"

!---- LOCAL VARIABLES --------------------------------------------------

      integer*2 t2(2)   ! array containing the two first words of one minor frame
      integer*2 of      ! offset first bit for the sample to extract
      integer*2 tmf(18) ! arry for one minor frame
      integer*2 nbmf    ! minor frame number
      integer*2 ideb_mf ! position of the first word of the minor frame

      integer*2 n_word  ! word number in hianag
      integer   n_bit   ! bit number in hianag word
      integer   bit     ! bit value from  hianag word

      integer*2 sp1,sp2,sp3,sp4,sp5,sp6 ! 
      integer i
      integer*2 dim     ! array dimension  
      integer*2 nbit    ! bits to extract  
      integer*2 count   ! to receive HIRS counts from TIP before sign test 
      integer word 
      integer*2 i2word1
      parameter(i2word1=1)
      integer*2 i2word2
      parameter(i2word2=2)
      integer*2 i2word7
      parameter(i2word7=7)
      integer*2 i2word14
      parameter(i2word14=14)

      integer*2 cst1
      parameter (cst1=Z'FFF4')
      
      integer*2 cst2
      parameter (cst2=Z'1F80')
      
      integer*2 cst3
      parameter (cst3=Z'007E')
      
      integer*2 cst4
      parameter (cst4=Z'1FFE')
      
      integer*2 cst5
      parameter (cst5=Z'0001')
      
      integer*2 cst6
      parameter (cst6=Z'1FFF')
      
      integer*2 cst7
      parameter (cst7=Z'0FFF')
      
      integer*2 cst8
      parameter (cst8=Z'C000')
      
!---- FUNCTIONS --------------------------------------------------------

      integer*2 gp_gb16 ! get bit from an integer*2
      logical gp_btn

!---- CODE ------------------------------------------------------------- 

!     hirs data decommutation 
!     -----------------------

      do nbmf=1,64                ! 64 minor frames ( elements )

         ideb_mf=((nbmf-1)*18)+1  ! first minor frame word 

!        elements 0 to 63: hrs_elem word 1
!        -------------------------------

         sp1=hirs(ideb_mf)       
         sp2=IAND(sp1,cst1)       ! mask for ( s.e.p. and  e.c.l.i. )
         hrs_elem(1,(nbmf-1))=sp2

!        elements 0 to 63: hrs_elem word 2
!        -------------------------------

         t2(1)=hirs(ideb_mf)     
         t2(2)=hirs(ideb_mf+1)   
         of=13                    ! offset source channel1
         dim=2
         nbit=6
         call gp_x16(t2,dim,nbit,of,sp1) ! channel1 in sp1

         dim=2
         nbit=6
         call gp_x16(t2,dim,nbit,of,sp2) ! elem_number in sp2

         dim=2
         nbit=1
         call gp_x16(t2,dim,nbit,of,sp3) ! filter in sp3

         sp4=ISHFT(sp1,i2word7)  ! left shift for sp1
         sp1=IAND(sp4,cst2)      ! mask channel1
         
         sp4=ISHFT(sp2,i2word1)  ! left shift for sp2
         sp2=IAND(sp4,cst3)      ! mask elem_number
         
         sp4=IOR(sp1,sp2)        ! channel1 + elem_number
         
         sp5=IAND(sp4,cst4)      ! mask ( channel1 + elem_number )
         
         sp6=IAND(sp3,cst5)      ! mask filter
         
         sp4=IOR(sp6,sp5)        ! channel1 + elem_number + filter
         
         sp5=IAND(sp4,cst6)      ! mask ( c + e + f )
         
         hrs_elem(2,(nbmf-1))=sp5

!        minor frame put into tmf
!        ------------------------

         do i=1,18
            tmf(i)=hirs(ideb_mf + i - 1 ) 
         end do                            

!        elements 0 to 62: hrs_elem words 3 to 22
!        ----------------------------------------

         if((( nbmf-1).ge.0).and.((nbmf-1).le.62))then
            of=26  !  offset source bit 27
            dim=18
            nbit=13
            do i=3,22
               call gp_x16(tmf,dim,nbit,of,count)
!
!       check sign and perform complementation to integer*2
!
               word=count
               if(.not.gp_btn(word,12)) then
                 hrs_elem(i,nbmf-1)=-count
               else
                 hrs_elem(i,nbmf-1)= IAND(count,cst7)
               endif
            end do
         endif

!        element 63: hrs_elem words 3 to 22
!        ----------------------------------

         if((nbmf-1).eq.63)then
            of=26    !  offset source bit 27 ( line count )
            dim=18
            nbit=13
            call gp_x16(tmf,dim,nbit,of,hrs_elem(3,nbmf-1)) ! word3

            of=44    !  offset source bit 45 ( first status word )
            dim=18
            nbit=8
            call gp_x16(tmf,dim,nbit,of,hrs_elem(4,nbmf-1))  ! word4

            of=57    !  offset source bit 58 ( second status word )
            dim=18
            nbit=8
            call gp_x16(tmf,dim,nbit,of,hrs_elem(5,nbmf-1))  ! word5

            of=65  !  offset source bit 66
            dim=18
            nbit=13
            call gp_x16(tmf,dim,nbit,of,hrs_elem(6,nbmf-1))  ! word6

            do i=7,22
              hrs_elem(i,nbmf-1)=0 ! words 7 to 22
            end do
         endif

!        elements 0 to 62: hrs_elem word 23
!        --------------------------------

         of=286  !  offset source bit 287 ( valid data bit et odd parity )
         dim=18
         nbit=2
         call gp_x16(tmf,dim,nbit,of,sp6)  ! v.d.b. and o.p. in sp6
         sp5=ISHFT(sp6,i2word14)           ! left shift sp6
         
         sp6=IAND(sp5,cst8)                ! mask ( v.d.b. + o.p. )
         
         hrs_elem(23,(nbmf-1))=sp6

      end do

! update hrs_scntyp with line count found in hrs_elem(3,63)
       if(hrs_elem(3,63).eq.0) hrs_scntyp=1
       if(hrs_elem(3,63).eq.1) hrs_scntyp=2
       if(hrs_elem(3,63).eq.2) hrs_scntyp=3
       if(hrs_elem(3,63).ge.3) hrs_scntyp=0
       


!     decommutation hianag
!     --------------------

      do i=2,15 ! loop for  bits 2 to 15 to hrs_digitbwrd

        n_word= ((nboct(i)+1)/2)+16               ! word number hianag
        n_bit= (MOD(nboct(i),i2word2)*8)+nbbit(i) !  bit number in hianag word
        bit  = gp_gb16(n_word,n_bit)              ! get bit value from hianag word
        call gp_sb16(hrs_digitbwrd,i,bit)         ! set bit in hrs_digitbwrd         
      end do

      call gp_sb16(hrs_digitbwrd,0,0) ! set bit 0 to 0 in hrs_digitbwrd
      call gp_sb16(hrs_digitbwrd,1,0) ! set bit 1 to 0 in hrs_digitbwrd

      return
      end
