        subroutine hrptdc(hrptab,nbmf)
!+
!NOM/NAME $Id: hrptdc.F 361 2013-06-10 09:37:26Z frna $
!
!BUT/PURPOSE 
!
! This module receives as an input the unpacked HRPT minor frame(s)
! coming from the centre specific module  closely connected to the hardware.
! The HRPT minor frame is an array of 11090 words, 
! made of the 10 bit HRPT words placed right justified in 16 bit words.
! detects the end of HRPT steram if nbmf=0
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!Its role is to :
!-initialize of the decommutation tasks
!       read the input options, (generalisation of DCIN)
!-open the various files
!-determine which serie (current or KLM) with sat. addr 
!-check good start conditions
!-perfom the general quality controls GENQC
!-compute the number of possible missing HRPT minor frames
!-extract the data for ATOVDC
!-call AVHRDC
!-call conclusion routine to perform statistics and write them to the summary 
! log file, update headers, close files.
!  
!PROPRIETAIRE/OWNER  
!       Marcel Derrien
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0    16/02/95       no              M. Derrien
! 2.0    31/07/00     Added a counter for bad or missing HRPT minor frames 
!                     to avoid to loop endless after. (TL)
! 3.0    29/05/01     Put away the message when nbmismf is <0
!                     Because it is tested before in chktime.F  (TL)
! 3.1    11/06/01     Move the declaration of nbmf before the declaration of
!                     the array hrptab(11090,nbmf)  (TL)
! 3.2    March 03                         T. LABROT
!    f90 test: I.and.J not correct and modified by iand(I,J)
!              I and J must have the same type.
!              Added a new local parameter i2word120
!              Cut lines exceeding 72 characters.
! 3.3    24-May-2004  Improvements to recover data from noisy reception (A.Korpela)
!                     - New testing for good data start (including modified chksatid.F)
!                     - New bad line processing (including modified cktime.F)
! 3.4    10-Aug-2004  Set maxbad=1 (A Korpela)         
! 3.5    17-Jun-2005  Cut lines exceeding 72 characters (NCA)
! 3.6    19-Sep-2006  Change "6000" to "mx_hrpscn" (NCA)
! 3.7    05-Jun-2013  Add SKIP_DECOM_CHECK option (NCA)
!
!ARGUMENTS 
!
!       integer*2 hrptab(11090,nbmf) ! buffer of nbmf HRPT minor frames
!       integer nbmf                 !  number of hrpt minor frames to process
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
! 
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none  
!arguments
        integer nbmf                 !  number of hrpt minor frames to process                                                  
        integer*2 hrptab(11090,nbmf) !  buffer of nbmf HRPT minor frames
!
!DECLARATIONS
!
! 
!    
!local variables
        integer curhr   ! current hour of hrpt minor frame
        integer day1  ! day count from hrpt 
        integer isystq(3) ! system quant. time (tp library)
        logical endhrpt ! true if nbmf =0
        integer err(5)  ! TIP minor frame parity errors
        logical found   ! 
        logical frst ! true for the first call
        integer frstmf ! number of the first good minor frame
        integer hmfn ! hrpt minor frame number
        integer i
        integer mf
        integer ninst ! (number of instruments to process including avhrr)
        integer opmode  !( 0 = off-line, 1= real time)
        integer optn(12) 
!                  optn(1) -- 0 or 1 for level of error logging
!                  optn(2) -- 0 or 1 for HIRS/3 (1 indicates extract data)
!                  optn(3) -- 0 or 1 for AMSU-A1
!                  optn(4) -- 0 or 1 for AMSU-A2
!                  optn(5) -- 0 or 1 for AMSU-B
!                  optn(6) -- 0 or 1 for HIRS/2 
!                  optn(7) -- 0 or 1 for MSU
!                  optn(8) -- 0 or 1 for DCS
!                  optn(9) -- 0 or 1 for SEM
!                  optn(10)-- 0 or 1 for SBUV
!                  optn(11)-- 0 or 1 for SAR
!                  optn(12)-- 0 or 1 for AVHRR
        integer orbmf     ! orbit minor frame number i.e. AVHRR scan line number
        integer prehr     ! previous mf hour
        integer satid     ! satellite id. i.e. order number from tiros-n (1)
        logical start     ! true if condition to be checked, false if found
        logical ended     ! true if searching conditions reached
        integer iqcur(3),iqref(3)
        double precision ijprev, ijcur
        integer year
        integer nbmiscn   ! number of missing scan lines
        integer ttnbmiscn ! total number of missing scan lines
!/NIWA begin
        double precision ijprv,ijpp
        integer iqpp(3)
        double precision dij_moy
        parameter (dij_moy= (1000./6.) / 86400.d+03 )
        integer*4  tday,tmsec
        integer good9,good10,good11,good12
        integer ibad
        data ibad /0/
        integer ibad1,ibad2,nofix,imis,imis1,imis2,imis_avhrr
        integer MaxBad
        parameter (MaxBad=1)
        integer*2 hrpbadf(11090,MaxBad)
        integer prvday
        data prvday /99999/
        integer prvsat
        data prvsat /99999/
        integer numOK
        data numOK /0/
        logical dayok,difok
!/NIWA end
        integer startorb,endorb
        integer nbbadmf   ! number of minor frames with bad time
        integer ttnbbadmf ! total number of minor frames with bad time
        logical goodtime,flagtime
        integer*2 hrpbad(11090)
        integer yr,iday,itime
        integer k
        integer hrpsat,tipsat ! satid from hrpt and tip
        integer difmnf,tipmnf,lastmnf
        integer word1,word2
        integer z1
        logical idok
        integer*2 i2word120
        parameter(i2word120=120)
        integer*2 i2word1
        parameter(i2word1=1)

        character*1 SKIP_DECOM_CHECK
        
!commons
#include "hrptdc.h"
#include "avh1b.h"

! functions
        integer hrpdat,hrptim
        logical chksatid
!
        data frst/.true./
        data found /.false./
        data prehr/0/
        data start/.true./
        data z1/1/
        data lastmnf /3000/
        
        save ttnbbadmf
        save ttnbmiscn
        
!-
        if(frst) then
!-------------------------------
!       initialisation sequence
!             read options
!             open files
!-------------------------------
          call dcin(optn,year,opmode,ninst,startorb,endorb)
          call dcsetu(optn) 
          ttnbbadmf = 0
          ttnbmiscn = 0
          call getenv('SKIP_DECOM_CHECK',SKIP_DECOM_CHECK)
          if (SKIP_DECOM_CHECK(1:1).eq.'y') SKIP_DECOM_CHECK(1:1)='Y'
          if (SKIP_DECOM_CHECK(1:1).eq.'Y')
     &      call ml_wt('info','Skip initial QC checks')
          frst=.false.
        endif
!---------------------------------------------
!       determine if good start conditions
!         minor frame number =1 and 
!         acceptable time differences 
!         between two consecutive minor frames
!---------------------------------------------
        endhrpt=nbmf.eq.0
        if(.not.endhrpt) then

          if (start) then
!         get system date in isystq array 1=year, 2=day , 3=msecs
            call giq(isystq)
            i = 0
!--------------------------------------------------------------------
            found=.false.
            ended=.false.
            do while (.not.ended)
              ended=i.eq.(nbmf-1)
              i = i + 1
              hmfn=iand((hrptab(7,i)/128),3)   !hrpt minor frame number
              
              curhr=hrptim(hrptab(10,i))       !current hour
              if (SKIP_DECOM_CHECK(1:1).eq.'Y'.and.prehr .eq. 0)
     &          prehr = curhr - 166

              if(hmfn.eq.1.and.(curhr-prehr).gt.165
     &          .and.(curhr-prehr).le.168) then
                word1 = hrptab(108,i)/4
                word1 = iand(word1,z1)
                word1 = word1*256                 ! shift left 8 bits
                word2 = hrptab(109,i)/4      
                tipmnf= word1 + word2
                hrpsat=iand(hrptab(7,i),i2word120)/8
                tipsat=iand((hrptab(106,i)/4),15)
!/NIWA          idok=chksatid(hrpsat,tipsat)
                idok=chksatid(hrptab(1,i))
                difmnf=tipmnf-lastmnf
!/NIWA begin    if(idok.and.(difmnf.eq.5.or.difmnf.eq.-315)) found=.true.
                idok = idok .AND. hrpsat.EQ.prvsat          ! consistent satid
                dayok = hrpdat(hrptab(9,i)).EQ.prvday       ! consistent day
                difok = (difmnf.eq.5).OR.(difmnf.eq.-315)   ! consistent HRPT minor frame number
                found = idok.AND.dayok.AND.difok            ! require all 
                IF (.NOT.found) then
                   numOK = 0
                   frstmf = i                               ! new candidate for a good start
                ENDIF
                IF (found) numOK = numOK + 1
                IF (numOK.LT.2) found=.FALSE.               ! require two frames (after candidate) to pass the test
C               write(6,*) 'CHECK    ',hrpsat,hrpdat(hrptab(9,i)),tipmnf,difmnf,numOK,' ',idok,dayok,difok,found
                prvsat = hrpsat
                prvday = hrpdat(hrptab(9,i))
!/NIWA end
                lastmnf = tipmnf

                if (SKIP_DECOM_CHECK(1:1).eq.'Y') found = .true.
              endif
              lastmnf = tipmnf
              if (found) then
!/NIWA          day1=hrpdat(hrptab(9,i))
                day1=hrpdat(hrptab(9,frstmf))
                ended=.true.
                if (opmode.eq.1) then
!
!                 check date against system date if opmode = 1
!
                  if (day1.ne.isystq(2)) found=.false.
                endif
              endif
              prehr=curhr            
            enddo
!--------------------------------------------------------------------
            if(found) then
!/NIWA        frstmf= i
              start=.false.
              iqcur(1)=year
              iqref(1)=year
              iqref(2)=day1
!/NIWA        iqref(3)=hrptim(hrptab(10,i))-167
              iqref(3)=hrptim(hrptab(10,frstmf))-167
            else ! return to ingest because no good conditions found
              return
            endif
          endif
!--------------------------------------
!         good start conditions reached
!         work is possible
!--------------------------------------

          do mf=frstmf,nbmf
!
! check time consistency with the previous or the last good one
!

            iqcur(2)=hrpdat(hrptab(9,mf))  
            iqcur(3)=hrptim(hrptab(10,mf))
C/NIWA start replacement
            call xiqj(iqref,ijprv)
            call cktime(iqcur,iqref,goodtime,nbbadmf,nbmiscn)
            if (orbmf+nbbadmf+nbmiscn+2.GE.mx_hrpscn) THEN
               goodtime=.FALSE.
               call ml_wti
     & ('warning','Limit scanlines reached, increase limit ?',mx_hrpscn)
            endif
            if (.not.goodtime) then             ! bad time sequence for this minor frame (line)
               ibad=ibad+1
               ibad2=min(ibad,MaxBad)
               do k = 1,11090                   ! we store this minor frame in hrpbad
                  hrpbadf(k,ibad2)=hrptab(k,mf)
               enddo
c              write(6,*) 'STOREBAD',ibad
               call ml_wti
     &         ('info','number of bad HRPT minor frames',nbbadmf)
            endif
            if (goodtime .and. ibad.gt.0) then    ! There are ibad bad lines stored in (1-ibad2) before this good line
               imis_avhrr = nbmiscn+nbbadmf             ! gap initially before the last postprocess-line (for AVHRR)
               if (nbmiscn.eq.0) then             ! No missing lines, so we should have all the bad lines stored
                  ibad1=1                               ! postprocess all starting from the first
                  nofix=0                               ! fix the time for all
                  imis1=0                               ! gap before the first postprocess-line
                  imis2=ibad-1                          ! gap initially before the last postprocess-line
               else                               ! Missing lines, so the last stored line was ok
                  ibad1=ibad2                           ! postprocess just that last line
                  nofix=ibad2                           ! no need to fix it
                  imis1=nbmiscn+nbbadmf                 ! gap before the postprocess-line
                  imis2=nbmiscn+nbbadmf                 ! gap before the postprocess-line
               endif
               do ibad=ibad1,ibad2                ! Post-process
                  imis=imis1                            ! gap before the first line
                  if (ibad.gt.ibad1) imis=0             ! no gap during the post-processing, except for
                  if (ibad.eq.ibad2) imis=imis2         ! possible gap before the last line (if hrpbadf became full)
                  if (ibad.ne.nofix) then         ! Must fix the bad time
                     ijpp=ijprv+(imis+1)*dij_moy        ! current correct time 
                     call xijq(ijpp,iqpp)
                     tday=iqpp(2)
                     tmsec=iqpp(3)
                     good9=IAND(hrpbadf(9,ibad),i2word1)     ! Insert tday in word good9
                     good9=1024*(hrpbadf(9,ibad)/1024)+good9 !
                     good9=IOR(good9,2*tday)      !
                     good10=tmsec/1048576         ! Insert tmsec into words good10,good11,good12
                     tmsec=tmsec-good10*1048576   !
                     good11=tmsec/1024            !
                     good12=tmsec-good11*1024     !
                     good10=IOR((128*(hrpbadf(10,ibad)/128)),good10)
                     hrpbadf(9,ibad)=good9
                     hrpbadf(10,ibad)=good10
                     hrpbadf(11,ibad)=good11
                     hrpbadf(12,ibad)=good12
                     ijprv=ijpp
                  endif
                  iqcur(2)=hrpdat(hrpbadf(9,ibad))    ! OK to process the corrected bad frame
                  iqcur(3)=hrptim(hrpbadf(10,ibad))
c                 write(6,*) 'PP_MS',iqcur(1),iqcur(2),iqcur(3)
                  call genqc(hrpbadf(1,ibad),iqcur,goodtime,imis,satid,
     &              orbmf,hmfn,err)
                  call atovdc(iqcur(1),optn,ninst,opmode,satid,iqcur(2),
     &                        orbmf,iqcur(3),isystq,startorb,endorb,
     &                        hmfn,hrpbadf(104,ibad),imis)
                  if (ibad.eq.ibad2 .and. optn(12).eq.1)   ! AVHRR post-processing - currently only the last line
     &               call avhrdc(opmode,startorb,endorb,isystq,satid,
     &                iqcur,orbmf,imis_avhrr,hmfn,hrpbadf(1,ibad))
                  imis2=imis2-1     ! the remaining gap to the last post-process frame
               enddo
               nbmiscn=0        ! all frames processed before a good frame
            endif
            if (goodtime) then                       ! now process the good frame
               iqcur(2)=hrpdat(hrptab(9,mf))
               iqcur(3)=hrptim(hrptab(10,mf))
c              write(6,*) 'OK_MS',iqcur(1),iqcur(2),iqcur(3)
c              write(6,*) (hrptab(108+(i-1)*104,mf)/4)*256 + hrptab(109+(i-1)*104,mf)/4
c              write(6,*) iand(1,hrptab(108+(i-1)*104,mf)/4)*256 + hrptab(109+(i-1)*104,mf)/4
               call genqc(hrptab(1,mf),iqcur,goodtime,nbmiscn,satid,
     &              orbmf,hmfn,err)
               call atovdc(iqcur(1),optn,ninst,opmode,satid,iqcur(2),
     &              orbmf,iqcur(3),isystq,startorb,endorb,hmfn,
     &              hrptab(104,mf),nbmiscn)
               if (optn(12).eq.1) then
                  call avhrdc(opmode,startorb,endorb,isystq,satid,
     &             iqcur,orbmf,nbmiscn,hmfn,hrptab(1,mf))
               endif
               nbbadmf=0
               ibad=0
             endif
        goto 100
!/NIWA end replacement (skip to 100)
            call cktime(iqcur,iqref,goodtime,nbbadmf,nbmiscn)
            ttnbbadmf = ttnbbadmf + nbbadmf
            ttnbmiscn = ttnbmiscn + nbmiscn
            if(ttnbbadmf.ge.mx_hrpscn) then
             call ml_wti('error','too much bad HRPT minor frames'
     &            ,nbbadmf)
            endif 
            if(ttnbmiscn.ge.mx_hrpscn) then
             call ml_wti('error','too much missing HRPT minor frames'
     &            ,nbmiscn)
            endif
            if(nbbadmf.lt.0) then
             call ml_wti('error',
     &           'number of bad HRPT minor frames negative',nbbadmf)
             call aappexit(10)
            endif
!TL            if(nbmiscn.lt.0) then
!TL             call ml_wti('error','number of missing HRPT minor frames negative'
!TL     &            ,nbmiscn)
!TL             call aappexit(10)
!TL            endif 
            
           if(.not.goodtime) then
! bad time sequence for this minor frame
! we store this minor frame in hrpbad (last bad minor frame)
                do k = 1,11090
                  hrpbad(k)=hrptab(k,mf)
                enddo
                call ml_wti('info','number of bad HRPT minor frames'
     &            ,nbbadmf)

            else if(goodtime.and.nbbadmf.eq.0) then
! this minor frame has good time 
!
              call genqc(hrptab(1,mf),iqcur,goodtime,nbmiscn,satid,
     &                   orbmf,hmfn,err)
              yr=iqcur(1)
              iday=iqcur(2)
              itime=iqcur(3)
              call atovdc(yr,optn,ninst,opmode,satid,iday,
     &  orbmf,itime,isystq,startorb,endorb,hmfn,hrptab(104,mf),nbmiscn)
              if(optn(12).eq.1) then
                call avhrdc(opmode,startorb,endorb,isystq,satid,
     &          iqcur,orbmf,nbmiscn,hmfn,hrptab(1,mf))
              endif

            else if(goodtime.and.nbbadmf.gt.0) then
! pass the last minor frame before time resync
! supposing its time is the time of the preceeding one 
! 
              call xiqj(iqcur,ijcur)
              ijprev=ijcur-.1666/86400.d0
              call xijq(ijprev,iqcur)
              flagtime=.false.
              call genqc(hrpbad,iqcur,flagtime,nbmiscn,satid,orbmf,
     &              hmfn,err)
              call atovdc(iqcur(1),optn,ninst,opmode,satid,iqcur(2),
     &  orbmf,iqcur(3),isystq,startorb,endorb,hmfn,hrpbad(104),nbmiscn)
              if(optn(12).eq.1) then
                call avhrdc(opmode,startorb,endorb,isystq,satid,
     &          iqcur,orbmf,nbmiscn,hmfn,hrpbad(1))
              endif
! reset bad to zero
              nbmiscn=0
              nbbadmf=0
! and the minor frame with good time
            iqcur(2)=hrpdat(hrptab(9,mf))  
            iqcur(3)=hrptim(hrptab(10,mf))
              call genqc(hrptab(1,mf),iqcur,goodtime,nbmiscn,satid,
     &              orbmf,hmfn,err)
              call atovdc(iqcur(1),optn,ninst,opmode,satid,iqcur(2),
     &           orbmf,iqcur(3),isystq,startorb,endorb,hmfn,
     &           hrptab(104,mf),nbmiscn)
              if(optn(12).eq.1) then
                call avhrdc(opmode,startorb,endorb,isystq,satid,
     &          iqcur,orbmf,nbmiscn,hmfn,hrptab(1,mf))
              endif
            endif      
!/NIWA add continue
 100    continue
          enddo
        else
!---------------------------------------------
!       end of hrpt frames detected
!        because no more hrpt frames from ingest
!       conclude : update headers, close files
!---------------------------------------------
          call dcexit(optn)
        endif
        return
        end 
          
                 
            
