C!$Id: mhsdc.F 125 2008-09-19 10:44:35Z frna $
C!
C!ROUTINE          MHSDC  SUBROUTINE  FORT77
C!
C!PURPOSE          TO DECOMMUTATE MHS DATA AND PUT INTO 1A ARRAYS
C!
C!DESCRIPTION      This routine takes the MHS minor
C!                 frames packed into the array MHS, extracts the
C!                 data and places it in the 1A data array. 
C!
C!VERSION         
C  1.01   30/7/03  based on amsbdc.F                  N.C.Atkinson
C  1.02   22/6/05  Remove unused variables. NCA
C  1.03   23/1/06  Allow for the systems that have ICHAR as signed. NCA
C  1.04   19/9/08  Correct mhs_current. It was incorrectly using "MHS" array
C                  elements 47-52; should be 46-51. Also the current
C                  coefficients in mhs_clparams.dat were in the wrong order.
C!
C!ARGUMENTS
C!      CHARACTER*1 MHS(3900) -- MHS scan line
C!      INTEGER MSECS     -- TIP msecs from midnight
C!      INTEGER BERR(3)   -- MHS Error flag (0=OK)
C!      INTEGER ISCAN     -- MHS scan (1->3)
C!
C!REFERENCES       ATOVDC design document version 1.9 Nov 95
C!
C! Definition of MHS/MIU output format:
C!                 MHS_18_draft_tlmy_extract.doc (NOAA/NASA), para 18.5.6.7
C!      also in section 4.1.4.4 of NOAA-KLM User Guide
C!
C!
C!#########################################################################
C! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
C!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
C!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
C!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
C!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
C!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
C!#########################################################################
C!
      SUBROUTINE MHSDC(MHS,MSECS,BERR,ISCAN)

      IMPLICIT NONE

      CHARACTER*1 MHS(3900)             ! MHS data for 3 scan lines
      INTEGER*4 MSECS                   ! msecs from midnight
      INTEGER*4 BERR(3)                 ! Error code (0=OK)
      INTEGER*4 ISCAN                   ! Scan number (1, 2 or 3)
      INTEGER*4 I
      INTEGER*4 IOFF
      INTEGER*4 MSB,LSB,ZFF
      PARAMETER(ZFF=255)

#include "mhs1b.h"
#include "hrptdc.h"

C!
C!          MHS data extraction
C!
      IOFF = (ISCAN-1)*1300
      BERR(ISCAN) = 0       !There are no errors to check for at this level

C! Mode Code / PIE ID / Subcom code (byte 13)
      mhs_mode = MHS(13 + IOFF)
      mhs_modeflg = CHAR(ICHAR(mhs_mode)/16)  !4 mode bits only

C! Telecommand Acknowledgement Field (byte 14-17) and fault code (18)
      DO I = 1, 5
        mhs_fault(I) = MHS(13 + I + IOFF)
      ENDDO

C! Switch status
      DO I = 1, 3
        mhs_switchstat(I) = MHS(18 + I + IOFF)
      ENDDO

C! Thermistors
      DO I = 1, 24
        mhs_temps(I) = MHS(21 + I + IOFF)
      ENDDO

C! Currents, order is +5V, +8V, +15V, -15V, RDM Motor, FDM Motor.
      DO I = 1, 6
        mhs_current(I) = MHS(45 + I + IOFF)
      ENDDO
      
C! Status word
      mhs_stat = MHS(52 + IOFF)

C! Signal processing status: DC offsets, validity flags, channel gains
      DO I = 1, 5
        mhs_dcoffset(I) = MHS(52 + I + IOFF)
      ENDDO
      mhs_chvalflgs = MHS(58 + IOFF)
      DO I = 1, 3
        mhs_chgains(I) = MHS(58 + I + IOFF)
      ENDDO
      
C! Earth view data (byte 62 to 1141, I*2 words)
      DO I = 1, 540
        MSB = ICHAR(MHS(I*2 + 60 + IOFF))
        LSB = ICHAR(MHS(I*2 + 61 + IOFF))
        MSB = MSB*256
        IF (MSB.GT.32767) MSB=MSB-65536
        IF (LSB.LT.0) LSB=LSB+256         !LSB is unsigned 
        mhs_earth_data(I) = MSB + LSB
      ENDDO

C! Space view (byte 1142 to 1189) and target view (byte 1190 to 1237) 
      DO I = 1, 24
        MSB = ICHAR(MHS(I*2 + 1140 + IOFF))   ! space views
        LSB = ICHAR(MHS(I*2 + 1141 + IOFF))
        MSB = MSB*256
        IF (MSB.GT.32767) MSB=MSB-65536
        IF (LSB.LT.0) LSB=LSB+256         !LSB is unsigned 
        mhs_coldcal(I) = MSB + LSB   

        MSB = ICHAR(MHS(I*2 + 1188 + IOFF))   ! internal target views
        LSB = ICHAR(MHS(I*2 + 1189 + IOFF))
        MSB = MSB*256
        IF (MSB.GT.32767) MSB=MSB-65536
        IF (LSB.LT.0) LSB=LSB+256         !LSB is unsigned 
        mhs_warmcal(I) = MSB + LSB   
      ENDDO

C! OBCT and ref resistors (byte 1238 to 1253)
      DO I=1,5
        MSB = ICHAR(MHS(I*2 + 1236 + IOFF))   ! obct
        LSB = ICHAR(MHS(I*2 + 1237 + IOFF))
        MSB = MSB*256
        IF (MSB.GT.32767) MSB=MSB-65536
        IF (LSB.LT.0) LSB=LSB+256         !LSB is unsigned
        mhs_obctprt(I) = MSB + LSB
      ENDDO
      DO I=1,3
        MSB = ICHAR(MHS(I*2 + 1246 + IOFF))   ! ref resistors
        LSB = ICHAR(MHS(I*2 + 1247 + IOFF))
        MSB = MSB*256
        IF (MSB.GT.32767) MSB=MSB-65536
        IF (LSB.LT.0) LSB=LSB+256         !LSB is unsigned 
        mhs_refrescts(I) = MSB + LSB
      ENDDO
      
      RETURN
      END
