C!$Id: mhsst.F 26 2006-03-01 12:28:20Z frna $
C!
C!ROUTINE          MHSST  SUBROUTINE  FORT77
C!
C!PURPOSE          GET MHS INSTRUMENT STATUS
C!
C!DESCRIPTION      This routine extracts the MHS instrument status for
C!                 all three scan lines and updates the status array for the
C!                 1A header. The header bits are defined as follows:
C!                        Bits 31-24   As for mhs_mode
C!                        Bit  23-0    As for mhs_switchstat
C!
C!VERSION          
C!  1.01   July 2003 N.C.Atkinson
C!  1.02   17/01/06  Philippe Marguinaud : change hex constants from
C!                                    '1234'X to Z'1234'
C!  1.03   23/01/06  Allow for the systems that have ICHAR as signed. NCA
C!                 
C!
C!ARGUMENTS
C!      INTEGER*2 MHS(3900)  -- MHS scan line
C!      INTEGER*4 INSTST(3)  -- MHS Instrument status
C!      INTEGER*4 BERR(3)    -- MHS Error flag  0 = OK
C!
C!REFERENCES       ATOVDC design document version 1.9 Nov 95
C!
C!#########################################################################
C! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
C!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
C!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
C!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
C!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
C!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
C!#########################################################################
C!
      SUBROUTINE MHSST(MHS,INSTST,BERR)
C!
      IMPLICIT NONE
c-----------------------------------------
c
c
C!
      CHARACTER*1 MHS(3900)             ! 3 MHS scan lines
      INTEGER*4 INSTST(3)               ! Instrument status
      INTEGER*4 BERR(3)                 ! Error codes (0=OK)
                
      INTEGER*4 BCHANNELS               ! HEX Mask. Channel on/off switches
      PARAMETER(BCHANNELS=Z'D50100')
      
      INTEGER*4 ZAMSU                  ! Bit mask: do not use scan
      PARAMETER(ZAMSU=Z'80000000')
      
      INTEGER*4 ISCAN,J,MODE,SW1,SW2,SW3

C!
      DO 10 ISCAN = 1 , 3

        J = 13 + (ISCAN-1)*650     !point to Mode/PIE ID/Subcom code
        MODE = ICHAR(MHS(J))
        IF (MODE.GT.128) MODE = MODE-256   !Signed

        J = 19 + (ISCAN-1)*650     !point to switch status
        SW1 = ICHAR(MHS(J))
        IF (SW1.LT.0) SW1 = SW1 + 256  !Unsigned
        SW2 = ICHAR(MHS(J+1))
        IF (SW2.LT.0) SW1 = SW1 + 256  !Unsigned
        SW3 = ICHAR(MHS(J+2))
        IF (SW3.LT.0) SW1 = SW1 + 256  !Unsigned
        
        INSTST(ISCAN) = MODE*16777216 + SW1*65536 + SW2*256 + SW3    
        

C! Check for scan mode and that at least 1 channel is on. If not, set the 'do
C! not use' flag.

        IF (MODE/16 .NE. 3. OR. IAND(INSTST(ISCAN),BCHANNELS).EQ.0) THEN
          BERR(ISCAN) = IOR(BERR(ISCAN),ZAMSU)
        ENDIF

  10  CONTINUE

      RETURN
      END
