c!
c!routine          msuhdu  subroutine  fort77
c!
c!purpose          to update a msu  1a header record
c!
c!description      this routine updates the  msu 1a header
c!
c!version          1.01   300195  r.w.saunders
c!                 1.02  12/5/98  Marcel Derrien
c!                                last 1B format effects siteid, version...
c!                 1.03  20/11/98 correction of TIP source code 
c!                                    ( 4 instead of 1)   T.Labrot
c!                 1.04  March 03 T. Labrot
c!     f90 test: I.and.J not correct and modified by iand(I,J)
c!               I.or.J not correct and modified by ior(I,J)
c!               I and J must have the same type
c!               ==> Changed some vriables integer*4 to integer*2
c!               Defined integer*2 i2word4095  and changed
c!                msern=msu(1).and.4095 to msern=IAND(msu(1),i2word4095)               
c!               Changed   type XXX/yyy/ 
c!                     to  type XXX
c!                         parameter (XXX=yyy)  or data XXX/yyy/
c!                 1.05  10/12/03 ml_wti needs i*4 not i*2. 
c!                                change nserno(11) to 3823. N.Atkinson
C!                 1.06  Jun 2006 PM, character*16 l1bscnam + MA, MB, MC, MD in
C!                                datasetname
C!                 1.07  Aug 2006 NCA, re-insert missing assignment of sname
C!                 1.08  Aug 2006 NCA Use station_id instead of fixed 'WE'
C!                 1.09  Nov 2008 NCA Re-compute jday for *_h_enddatajd
c!
c!arguments
c!      integer*2 msu(112) -- msu scan line
c!      integer yr        -- year of data (e.g. 1997).
c!      integer day       -- tip day number
c!      integer msecs     -- tip msecs from midnight
c!      integer systim(3) -- current year, dayno,  msecs from midnight
c!      integer satid     -- noaa satellite id
c!      integer err        -- error flag  0 = ok
c!      integer nmsu       -- current data scan line
c!      logical*1 gotime  -- set to true when first valid tip time available
c!      integer*4 startorb,endorb         ! orbit number (start and end)
c!
c! common/iocom/ lulog,luhirs3,luamsa1,luamsa2,luamsb,luhirs2,lumsu,
c!               ludcs,lusem,lusbuv,lusar,luavh
c! common/msu1bhd/msu_h_l1bversnb,................
c!
c!routine          xdqj
c!
c!files            fornnn log file output
c!                 formmm hirs 1a file output
c!
c!references       atovdc design document version 1.3 11 jan 95
c!
c!changes          01 none
c!
c!#########################################################################
c! copyright        the contents of this file are crown copyright. the
c!                  software, data and other proprietary information
c!                  contained therein is supplied for bonafide research
c!                  purposes. commercial or business use is not permitted
c!                  without the written authority of the u.k. met. office.
c!                  at london rd., bracknell, berks, rg12 2sz, u.k.
c!#########################################################################
c!
      subroutine msuhdu(msu,yr,day,msecs,systim,satid,err,nmsu,gotime,
     &startorb,endorb)
c!
      implicit none
#include "nl1b.h"
#include "hrptdc.h"
#include "msu1b.h"
      integer iq(3)
      integer*2 msu(112)                ! one msu scan line
      integer*4 nmsu                    ! current data scan line
      integer*4 systim(3)               ! year, dayno,  msecs from midnight
      integer*4 day,msecs               ! tip day no and msecs from midnight
      integer*4 yr                      ! year of data
      integer*4 jday                    ! julian day
      integer*4 word1,dq(2)             ! temp storage
      integer*2 msuid                   ! msu instrument id
      integer*4 satid                   ! satellite id
      integer*4 startorb,endorb         ! orbit number (start and end)
      integer*4 instst,lastst           ! inst status
      integer*4 err                     ! error code (0=ok)
      integer*2 nserno(11)              ! msu serial number
      integer*4 errtim,errpar,errsyn    ! error counters
      integer*4 z800                    ! hex mask
      parameter (z800=2048)

      integer*4 zmsu                    ! bit mask
      parameter (zmsu=33554432)
      
      integer*4 ztim1,zpar,zsync        !bit masks
      parameter (ztim1=1082130432)
      parameter (zpar=65536)
      parameter (zsync=524288)
      
      integer*2 i2word4095
      parameter (i2word4095=4095)
 
      logical*1 first,gotime
      data first/.true./
      
        character*4 str1
        character*3 str2
        character*5 str3,str4
        character*16 sname
        integer cp_lsc, sname_len
        character*2 rname
        integer ig(7)
        character*22 ic
        integer*2 msern
        integer imsern
        
c!  ***  msu serial numbers
      data nserno(1) /1340/      ! tiros-n
      data nserno(2) /543/       ! noaa6
      data nserno(3) /0/         
      data nserno(4) /1752/      !noaa7
      data nserno(5) /3927/      !noaa12
      data nserno(6) /3377/      !noaa8
      data nserno(7) /2962/      !noaa9
      data nserno(8) /2554/      !noaa10
      data nserno(9) /2161/      !noaa11
      data nserno(10) /0/        !noaa13
!      data nserno(11)/3822/      !noaa14
      data nserno(11)/3823/      !noaa14


c!
        character*16 l1bscnam
        integer l1bscid
c!
c! .. get msu instrument status
       word1 = msu(8)
       instst = IAND(word1,z800)
       if (instst .ne. 0)err= IOR(err,zmsu)
c!
      if(first)then
c!...initialise error counters
        errtim = 0
        errpar = 0
        errsyn = 0
        lastst = instst
c!
        if(gotime)first = .false.
c! .. get instrument id
        msuid = nserno(satid)
c!    check with instrument id from tip
        msern=IAND(msu(1),i2word4095)
        imsern=msern            !Convert to I4 for call to ml_wti
        if(msern.ne.msuid) then
          call ml_wti('warning','msu serial number not expected',imsern)
        endif
c! ..compute julian day
        dq(1) = yr
        dq(2) = day
        call xdqj(dq,jday)
c!
c!..create header record 1 (not all elements filled)
c!
c!..general information
        msu_h_l1bversnb = libl1bvers          ! format version number
        msu_h_l1bversyr = libl1bversyr        ! format version year
        msu_h_l1bversdy = libl1bversdy        ! format version daynumber
        msu_h_hdrcnt = nb1bhd                 ! no. of header records
        msu_h_satid = l1bscid(satid)          ! noaa s/c id
        msu_h_instid = msuid                  ! instrument id
        msu_h_datatyp = 6                     ! msu data type code
        msu_h_tipsrc = 4                      ! tip source code
        msu_h_startdatajd = jday              ! start julian day no.
        msu_h_startdatayr = yr                ! start of data year
        msu_h_startdatady = day               ! start of data day of year
        msu_h_startdatatime = msecs-(nmsu-1)*25600 ! corrected time of day msecs
        msu_h_inststat1 = instst              ! msu status
        msu_h_statchrecnb = 0                 ! record number of msu status change

         iq(1)=yr
         iq(2)=day
         iq(3)=msu_h_startdatatime
         write(str1,1000) yr
1000     format(i4.4)
         write(str2,1001) day
1001     format(i3.3)
         write(str3,1002) startorb
1002     format(i5.5)

        write(str4,1002) endorb
         sname=l1bscnam(satid)
         sname_len = cp_lsc( sname )
         rname=sname(1:1)//sname(sname_len:sname_len)
         call cp_ups(rname)
         call xiqg(iq,ig)
         call xigc(ig,ic)
         msu_h_dataname=loccentre//'.MSUX.'//rname//
     &   '.D'//str1(3:4)//str2//'.S'//ic(11:12)//ic(14:15)//
     &   '.E    .B'//str3//str4(4:5)//'.'//station_id
         msu_h_prblkid='B'//str3//str4(4:5)
         msu_h_siteid=loccentre
         msu_h_blank=' '
      endif
c! add end data date and time
c! ..compute julian day
      iq(1) = yr
      iq(2) = day
      iq(3) = msecs
      call xiqg(iq,ig)
      call xigc(ig,ic)
      dq(1) = yr
      dq(2) = day
      call xdqj(dq,jday)                ! Compute Julian day
      msu_h_enddatajd = jday            ! end julian day no.
      msu_h_enddatayr = yr              ! end of data year
      msu_h_enddatady = day             ! end of data day of year
      msu_h_enddatatime = msecs         ! end of data time of day msecs
      msu_h_dataname(27:30)=ic(11:12)//ic(14:15)
c!
c!...dataset quality indicators
      if (lastst .ne. instst)then
        msu_h_statchrecnb = nmsu + nb1bhd         ! record number of status change
        msu_h_inststat2 = instst        ! msu new status
      endif
      lastst = instst
c!
      msu_h_scnlin = nmsu                 ! number of scan lines in file
      msu_h_callocsclin = 0               ! number of calib/navig lines
      msu_h_datagaps = 0                  ! count of data gaps
      msu_h_okdatafr = msu_h_misscnlin    ! count of frames with no synch errors
      msu_h_pacsparityerr = errpar        ! count of tip parity errors
      msu_h_auxsyncerrsum = errsyn        ! count of all synch errors
      msu_h_timeseqerr = errtim           ! count of tip timimg errors
      msu_h_timeseqerrcode = 0          ! time sequence error code (more info?)
c!
c!  increment error counters if necessary
c!
      if (IAND(err,ztim1).gt.0)errtim=errtim+1
      if (IAND(err,zpar).gt.0)errpar=errpar+1
      if (IAND(err,zsync).gt.0)errsyn=errsyn+1
c!
      return
      end
