
c!
c!routine          msuout  subroutine  fort77
c!
c!purpose          to update msu header & write scan line
c!
c!description      this routine updates the msu  1a header and
c!                 writes out one msu scan line record (one data record
c!                 per scan line).
c!
c!version          1.01   300195  r.w.saunders
c!                        281195 180996 131296 m. derrien
c!                        adaptation to noisy data 
c!                        replaced reclmsu by cmsu_size as noted
c!                            by paul dibben 
c!                 1.02   March 03     T.Labrot
c!   f90 test: Changed  type XXX/yyy/
c!                  to  type XXX
c!                      data XXX/yyy/
C!                 1.03   050304  N C Atkinson change initialisation of MISSING
C!                 1.09   Apr 10  For portability, re-write the currect scan 
C!                                when correcting times of first few scan lines.
c!
c!arguments
c!      integer*2 msu(112)-- msu  scan line
c!      integer yr        -- year of data (e.g. 1997).
c!      integer msecm     -- tip msecs from midnight
c!      integer systim(3) -- system yr, dayno, secs from midnight
c!      integer satid     -- noaa satellite id
c!      integer msuerr    -- error flag (0=ok)
c!      logical*1 fullms  -- set to true when msu  scan line filled
c!      logical*1 gotime  -- set to true when first valid tip time available
c!      integer*4 startorb,endorb         ! orbit number (start and end)
c!
c! common/iocom/ lulog,luhirs3,luamsa1,luamsa2,luamsb,luhirs2,lumsu,
c!               ludcs,lusem,lusbuv,lusar,luavh
c! common /log/msecs,day,nmf,nhirs,nmsu,namsua,namsub,ndcs
c!
c!subprograms      msuhdu, msudtw, msudc

c!files            fornnn log file output
c!                 formmm msu  1a file output
c!
c!references       atovdc design document version 1.3 11 jan 95
c!
c!changes          01 none
c!
c!#########################################################################
c! copyright        the contents of this file are crown copyright. the
c!                  software, data and other proprietary information
c!                  contained therein is supplied for bonafide research
c!                  purposes. commercial or business use is not permitted
c!                  without the written authority of the u.k. met. office.
c!                  at london rd., bracknell, berks, rg12 2sz, u.k.
c!#########################################################################
c!
      subroutine msuout(msu,yr,msecm,systim,satid,msuerr,fullms,
     &gotime,startorb,endorb)
c!
      implicit none
c-----------------------------------------
c
c
c!
c!
      integer*2 msu(112)                ! one msu scan line
      integer*4 systim(3)               ! yr, daynum, msecs from midnight
      integer*4 day,msecs,msecm         ! tip day no and msecs from midnight
      integer*4 yr                      ! year of data
      integer*4 satid                   ! satellite id
      integer*4 startorb,endorb         ! orbit number (start and end)
      integer*4 msuerr                  ! error code (0=ok)
      integer*4 i                       ! counter
      integer*4 nmf                     ! no. of tip maj/minor frames read
      integer*4 lmsecs,mmsecs
      integer*4 ios
      integer*4 nlin,tipjmf,missing,s
      integer*4 nhirs,nmsu,namsua,namsub,ndcs !no scan lines
      logical*1 fullms                  ! set to false when line written
      logical*1 first,gotime,start
      data first/.true./
      data start/.true./
      
c!
      common /log/msecs,day,nmf,nhirs,nmsu,namsua,namsub,ndcs
#include "nl1b.h"
#include "hrptdc.h"
#include "msu1b.h"
C      data missing/-2147483648/     ! missing data flag
      SAVE missing
c!
      if(first)then
       missing = -2147483647-1       !Some compilers don't accept -2147483648
       first =.false.
       lmsecs = msecm
       nmsu = 0
       msu_h_misscnlin = 0                 ! count of missing scan lines
      endif
c!
      nmsu = nmsu + 1
c!
c! ...update header record
c!
      call msuhdu(msu,yr,day,msecm,systim,satid,msuerr,nmsu,gotime,
     &startorb,endorb)
c!
c! ...write data record
c!
c!
c!   check for and fill in missing lines if necessary
c!
      if(.not.gotime.and.(msecm-lmsecs) .gt. 27000)then
        nlin = nint(float(msecm-lmsecs)/25600.0)
        tipjmf=msu_mjfrcnt
        do i =1,cmsu_size
          msu_dt(i)=0
        enddo
        msu_scnlinyr = yr
        msu_scnlindy = day
        mmsecs = lmsecs
        msu_qualind = missing
        do  i = 1 , nlin-1
          mmsecs = mmsecs + 25600
          msu_scnlintime = mmsecs
          msu_scnlin = nmsu
          call msudtw(lumsu,nmsu,ios)
          nmsu = nmsu +1
        enddo
        msu_mjfrcnt=tipjmf
      endif
      lmsecs = msecm
c!
c! add data date and time
      msu_scnlin = nmsu
      msu_scnlinyr = yr                 ! time year
      msu_scnlindy = day                ! time day of year
      msu_scnlintime = msecm              ! time of day msecs from 0z
      msu_clockdrift = 0                ! clock drift
c!
      msu_qualind = msuerr                ! error flags; zero is good data
      s = 0
      do i = 1,112
         s = s + msu(i)
      enddo
      if ( s .eq. 0 ) then 
         call gp_sb32(msu_qualind,31,1)
         msu_h_misscnlin =  msu_h_misscnlin + 1 ! count of missing scan lines
      endif
c!
c!   decommutate msu data into 1a record
c!
      call msudc(msu)

      call msudtw(lumsu,nmsu,ios)
!      write(lulog,*)' msuout scan msu ',nmsu,' msecm ',msecm
c!
c! correct time on first few scan lines if necessary
c!
      if(start.and.gotime.and.nmsu.gt.1)then
        start = .false.
        do 25 i = 1 , nmsu
          call msudtr(lumsu,i,ios)
          msu_scnlintime = msecm-(nmsu-i)*25600 ! corrected tip time  msecs from 0z
          call msudtw(lumsu,i,ios)
25     continue
      endif

c!..reset full flag
      fullms = .false.
c!..reset error flag
      msuerr = 0
c!..zero array
      do 20 i = 1, 112
        msu(i) = 0
 20   continue
c!
      return
      end
