c!
c!routine          otiget   subroutine  fort77
c!
c!purpose          to extract hirs-2,msu from pre noaa-k tip minor frame
c!
c!description      each tip minor frame contains 36 words of hirs/2 data as
c!                 defined in fig 3-6 of ness 107 rev1. these words are
c!                 extracted and placed in a hirs scan line buffer. when each
c!                 scan line is full a flag is set. similarly there are 4
c!                 msu words in a minor frame which are extracted and placed
c!                 in an msu scan line array.
c!
c!version          1.01   200195  r.w.saunders
c!                 2      170996  m. derrien
c!                        modifications to take care of noise 
c!                        and data loss 
c!                 2.01   March 03  T.Labrot
c!    f90 test: I.and.J not correct and modified by iand(I,J)
c!              Changed  type XXX/yyy/ 
c!                   to  type XXX
c!                       parameter (XXX=yyy)  or data XXX/yyy/
c!
c!arguments
c!      integer*4 optn(12) -- hrptdc options
c!      integer imf       -- 1-5 to identify which minor frame
c!      integer tipmf     -- tip minor frame no.
c!      integer*2 hrpaip(104,5) -- current tip array
c!      integer*2 hirs(1152) -- hirs scan line
c!      integer*2 msu(112) -- msu scan line
c!      logical*1 fullhi  -- set to true when hirs scan line filled
c!      logical*1 fullms  -- set to true when msu scan line filled
c!      integer*4 msecs -- tip minor frame time
c!      integer*4 msecm -- msu scan line time
c!      integer*4 msuerr -- msu error flag
c!
c! common/iocom/ lulog,luhirs3,luamsa1,luamsa2,luamsb,luhirs2,lumsu,
c!               ludcs,lusem,lusbuv,lusar,luavh
c!
c!files            fornnn log file output
c!
c!references       atovdc design document version 1.3 11 jan 95
c!                 ness 107 rev 1 1988
c!
c!changes          01 none
c!
c!#########################################################################
c! copyright        the contents of this file are crown copyright. the
c!                  software, data and other proprietary information
c!                  contained therein is supplied for bonafide research
c!                  purposes. commercial or business use is not permitted
c!                  without the written authority of the u.k. met. office.
c!                  at london rd., bracknell, berks, rg12 2sz, u.k.
c!#########################################################################
c!
      subroutine otiget(optn,imf,tipmf,hrpaip,hirs,msu,
     &    fullhi,fullms,msecs,msecm,msuerr)
c!
      implicit none
c-----------------------------------------
c
c
c!
#include "hrptdc.h"
      integer*2 hrpaip(104,5)           ! 5 tip/amsu/backfill minor frames
      integer*4 tipmf                   ! tip minor frame number
      integer*4 optn(12)                ! option numbers
      integer*4 imf                     ! tip minor frame counter
      integer*4 msecs,msecm             ! tip and msu times
      integer*2 hirs(1152)              ! one hirs scan line
      integer*2 msu(112)                ! one msu scan line
      integer*4 msuerr                  ! msu error flag
      integer*4 ivh                     ! output array word offsets
      integer*4 hioff(18),msuoff(2)     ! input array word offsets
      integer*4 iw,imh,nmsu,msucount    ! counters etc
      integer*4 msb,lsb,bit15,bit16,bit14 ! temporary space
      integer*4 z2000                   ! 14 bit mask
      parameter (z2000=8192)

      integer*4 z4000,zff               ! 15 bit mask
      parameter (z4000=16384)
      parameter (zff=255)
      
      integer*4 z8000                   ! 16 bit mask
      parameter (z8000=-32768)
      
      integer*4 zfram                   ! hexadecimal mask
      parameter (zfram=262144)
      
      logical*1 fullhi,fullms           ! logical flags for full lines
      logical*1 firsthirs,msubeg        ! logical flags for start hirs/msu
      data firsthirs/.true./
      data msubeg/.true./

      logical*1 firstmsu
      data firstmsu/.true./

      logical*1 msu_start  ! true when an acceptable msu start is found
      integer exp_tipmf(6) ! array of possible tip minor frames when msu starts
      integer k,ii,i
      integer ifov(256) ! array of assumed ifov numbers ( 1 to 14 , o if not)
                        ! according to msucount+1
       character*80 txt
c! ...define word offsets in tip array
      data hioff/15,23,27,31,35,39,43,55,59,63,67,71,75,79,83,85,89,93/
      data msuoff/25,41/
      data exp_tipmf/19,83,147,211,257,275/
      data ii/1/
      save
c!
c!########################################################################
c!                         hirs data extraction
c!
c!...don't try getting any data until at start of first hirs scan line
c!
      if (firsthirs) then
        fullms =.false.
        fullhi = .false. 
        if (mod(tipmf,64).eq.1)then                  ! start of hirs line
          firsthirs=.false.
        else
          return
        endif
      endif

c!
      if ( optn(6) .eq. 1)then
c!
c!  compute hirs word offset from tip minor frame number
c!
        ivh = mod(tipmf-1,64)
        if(ivh.lt.0) ivh=63

        ivh = 18*ivh
c!
c!  extract hirs/2 words
c!
      do   10 iw = 1 , 18
          msb = hrpaip(hioff(iw),imf)
          msb = IAND(msb,zff)
          msb = msb*256                         ! shift left 8 bits
          if(msb.gt.32767)msb=msb-65536
          lsb = hrpaip(hioff(iw)+1,imf)
          lsb = IAND(lsb , zff)
          hirs(iw+ivh) = msb + lsb
  10    continue
c!
c!   is output hirs array full?
c!
        if( tipmf .eq. 0 .or. tipmf.eq. 64 .or. tipmf.eq.128
     &    .or. tipmf.eq.192 .or. tipmf.eq. 256)fullhi=.true.
       endif
c!
c!
c!########################################################################
c!                         msu data extraction
      if ( optn(7) .eq. 1)then
c!
c!   apply rough test for start of an msu scan line
c!
      if(imh.eq.0.and.hrpaip(msuoff(1),imf).eq.0 .and.
     &    hrpaip(msuoff(1)+1,imf).eq.0 .and.
     &    hrpaip(msuoff(2),imf).eq.0 .and.
     &    hrpaip(msuoff(2)+1,imf).eq.0.and.firstmsu) then
        return
      endif
c!
c!   extract msu words
c!
      do 20 iw = 1 , 2
         msb = hrpaip(msuoff(iw),imf)
         msb = msb*256                         ! shift left 8 bits
         if(msb.gt.32767)msb=msb-65536
         lsb = hrpaip(msuoff(iw)+1,imf)
         bit15 = IAND((msb+lsb),z4000)
         bit16 = IAND((msb+lsb),z8000)
c!
c!   if not started scan look more closely
c!
         if(firstmsu) then
!
! a msu scan line has not already been found
!
           msubeg=.true.
           fullms = .false.
           imh = 0  
!
!        initialise ifov array with (msucount+1) pointer
!         ifov contains the ifov number ( 1 to 14 )
!         if the expected tip minor frame number can contain
!         real msu data
!           
           do i = 1,256
             ifov(i)=0
             if(i.le.7) ifov(i) = 1
             if(i.ge.21.and.i.le.26) ifov(i) = 2
             if(i.ge.40.and.i.le.45) ifov(i) = 3
             if(i.ge.59.and.i.le.64) ifov(i) = 4
             if(i.ge.78.and.i.le.83) ifov(i) = 5
             if(i.ge.97.and.i.le.101) ifov(i) = 6
             if(i.ge.116.and.i.le.121) ifov(i) = 7
             if(i.ge.134.and.i.le.140) ifov(i) = 8
             if(i.ge.153.and.i.le.159) ifov(i) = 9
             if(i.ge.172.and.i.le.179) ifov(i) = 10
             if(i.ge.191.and.i.le.197) ifov(i) = 11
             if(i.ge.211.and.i.le.218) ifov(i) = 12
             if(i.ge.232.and.i.le.237) ifov(i) = 13
             if(i.ge.239.and.i.le.243) ifov(i) = 14
           enddo
!
!  we search the first msu scan line start
!

           if(bit15.gt.0.and.bit16.lt.0) then
             msu_start=.false.
!
! check noisy data with acceptable tip minor frame
! according to NESS 107  table 4-5 page 36-rev
!
             do k=1,6
               if(iabs(tipmf-exp_tipmf(k)).le.3) then
                 msu_start=.true.
               endif
             enddo
!            
! msu_start true indicates that this msu start is retained
!
             if(msu_start) then  
               msubeg=.false.
               firstmsu=.false. 
!
! initialise msucount to 2 ( will allow variability in msu starts if any ) 
!               
               msucount=2
               imh = 0
               nmsu = tipmf
               msecm = msecs ! set msu scan line time
               call ml_wti('info','first MSU scan at tipmf',tipmf)
             endif          
           endif         
         endif
         if (msubeg) then
           if(bit15.gt.0.and.bit16.lt.0.and.msucount.lt.5) then
             msubeg=.false.                         !msu line start             
             imh = 0
             nmsu = tipmf
             msecm = msecs                         ! set msu scan line time
             call ml_wti('info','start of a MSU scan at tipmf',tipmf)
           else if(msucount.eq.5) then
!
! we assume msu line start has not been detected with bit15 and bit 16
! due to noise or loss 
! it should have occured within msucount =0 to 5
! it is assumed to be at msucount=2  
! rhen we force imh = 6 
!         
             msubeg=.false.                         !msu line start             
             imh = 6
             nmsu = mod(tipmf-3,320)
             if(nmsu.lt.0) nmsu=320+nmsu
             msecm = msecs - 300  ! assumed msucount 2 set msu scan line time
             if (msecm.lt.0) msecm= msecm + 8640000
             call ml_wti('info','force MSU scan start at tipmf',tipmf)
           endif
         endif

         if(.not.msubeg) then
           imh=(ifov(msucount+1)-1)*8 + ii
           if(bit16.lt.0)then
!             write(lulog,*)
!     &     'iw',iw,'real msu word tipmf',tipmf,imh,msucount,ii,
!     &     ifov(msucount+1)
             if(ifov(msucount+1).ne.0) then
!
!  these minor frames may have real msu words 
!  we jump to the next ifov if loss
!
               bit14 = IAND((msb+lsb),z2000)
               if(bit14.gt.0.and.ii.ne.8) then
                  write(txt,*)'msu desync. at tipmf',tipmf,
     &                  ' for ifov',ifov(msucount+1)
                  call ml_wt('info',txt)
               endif 
               msu(imh) = msb+lsb
               ii = ii+1 
             endif       
           endif
         endif
 20   continue
!         write(lulog,*)' msucount ', msucount

c!
c!   is output msu array full or should it be full ?
c!
      if(imh.ge.112.or.(msucount.gt.250).and.(.not.msubeg))then
!             write(lulog,1002) tipmf,nmsu,msucount
! 1002        format ('end MSU scan line tipmf',i7,
!     &     ' nmsu ',i4,' msu count',i4)                  
        fullms=.true. 
        msubeg=.true.
        imh = 0
      endif
      endif
      if(.not.firstmsu) msucount=mod(msucount+1,256) ! increment msu counter
      if(ifov(msucount+1).eq.0) ii=1 ! reset msu word 1 to 8 counter
c!
      return
      end
