c!
c!routine          tipqc    subroutine  fort77
c!
c!purpose          to perform quality control checks on various parameters
c!
c!description      check some tip parameters:
c!                   -- tip s/c id matches hrpt s/c id
c!                   -- check tip frame synch bytes ok
c!                   -- check tip status ok
c!                   -- flag tip parity errors
c!
c!version          1.01   170195  r.w.saunders
c!                 1.02   March 03  T.Labrot
c!    f90 test: I.and.J not correct and modified by iand(I,J)
c!              I.or.J not correct and modified by ior(I,J)
c!              Changed   type XXX/yyy/ 
c!                   to   type XXX
c!                        parameter (XXX=yyy)
c!
c!arguments
c!      integer mf         -- minor frame number (0->319)
c!      integer satid      -- satellite number from hrpt s/c address (with
c!                              noaa level 1b convention)
c!      integer idat       -- hrpt day number
c!      integer yr         -- year of data (e.g. 1995)
c!      integer err        -- error flag  0 = ok
c!                         -- in all of the following, if the
c!        bit is on (i.e., if it is set to 1) then the statement is
c!        true. otherwise it is false.
c!                        general
c!        bit 31: do not use scan for product generation
c!        bit 30: time sequence error detected with this scan (see
c!                below)
c!        bit 29: data gap precedes this scan
c!        bit 28: no calibration (see below)
c!        bit 27: no earth location (see below)
c!        bit 26: first good time following a clock update
c!        bit 25: instrument status changed with this scan
c!        bit 24: spare
c!                        time problem code
c!
c!        (all bits off implies the scan time is as expected.)
c!        bit 23: time field is bad but can probably be inferred from
c!                the previous good time.
c!        bit 22: time field is bad and can't be inferred from the
c!                previous good time.
c!        bit 21: this record starts a sequence that is inconsistent
c!                with previous times (i.e., there is a time
c!                discontinuity).  this may or may not be associated
c!                with a spacecraft clock update. (see bit 26 above)
c!        bit 20: start of a sequence that apparently repeats scan
c!                times that have been previously accepted.
c!
c!                        tip problem code
c!        bit 19: tip synch error
c!        bit 18: tip minor or major frame sequence error
c!        bit 17: tip status error
c!        bit 16: tip parity error
c!        remaining bits are set by calibration and navigation routines
c!      integer*2 hrpaip(104,5) -- current tip array of 5 minor frames
c!
c! common/iocom/ lulog,luhirs3,luamsa1,luamsa2,luamsb,luhirs2,lumsu,
c!               ludcs,lusem,lusbuv,lusar,luavh
c!common/gqc/timok,partip,err_frsync,nb_frsync,err_auxsync,nb_auxsync
c!common/log/msecs,day,nmf,nhirs,nmsu,namsua,namsub,ndcs
c!
c!function         ordtiid

c!routines         xdqj
c!
c!files            fornnn log file output
c!
c!references       atovdc design document version 1.3 11 jan 95
c!
c!changes          01 none
c!
c!#########################################################################
c! copyright        the contents of this file are crown copyright. the
c!                  software, data and other proprietary information
c!                  contained therein is supplied for bonafide research
c!                  purposes. commercial or business use is not permitted
c!                  without the written authority of the u.k. met. office.
c!                  at london rd., bracknell, berks, rg12 2sz, u.k.
c!#########################################################################
        subroutine tipqc(mf,satid,idat,yr,err,hrpaip)
c!
      implicit none
c-----------------------------------------
c
c
c!
      integer*2 hrpaip(104,5)           ! 5 tip/amsu/backfill minor frames
      integer*4 word1,word2,dq(2)       ! temporary words
      integer*4 idat,jday               ! hrpt day number, jul day from 1950
      integer*4 yr                      ! year
      integer*4 msecs,day,nmf,nhirs,nmsu,namsua,namsub,ndcs
      integer*4 mf                      ! tip minor frame number
      integer*4 err                     ! error code (0=ok)
      integer*4 satid                   ! hrpt s/c address
      integer*4 sync1,sync2             ! in hex ed,e2
      parameter (sync1=237)
      parameter (sync2=226)

      integer*4 z3                      ! bit masks
      parameter (z3=3)
      
      integer*4 z0f,zsync,zstat,zpar          ! bit masks
      parameter (z0f=15)
      parameter (zsync=524288)
      parameter (zstat=131072)
      parameter (zpar=65536)

      character*80 txt
c!
      integer*4 ordtiid     ! function to do tip to internal sat id conversion
c!
      logical timok,err_frsync,partip(5)
      integer*4 nb_frsync,nb_auxsync
      logical err_auxsync
c!
#include "hrptdc.h"
      common/gqc/timok,partip,err_frsync,nb_frsync,err_auxsync,
     & nb_auxsync
      common /log/ msecs,day,nmf,nhirs,nmsu,namsua,namsub,ndcs
c!
c!  check sat id
c!
      err = 0
      word1 = hrpaip(3,mf)
      word1 = IAND(word1,z0f)
      dq(1)= yr
      dq(2)= idat
      call xdqj(dq,jday)
      word2 = ordtiid(word1,jday)             ! convert tip id to ord id
      if ( word2 .ne. satid )then
        write(txt,101) word2,satid,msecs
        call ml_wt('info',txt)
      endif
c!
c!  check frame synch bytes
c!
      word1 = hrpaip(1,mf)
      word2 = hrpaip(2,mf)
      if (word1.ne.sync1.or.word2.ne.sync2)then
        err = IOR(err,zsync)
      endif
c!
c!  check tip status
c!
      word1 = hrpaip(4,mf)
      word1 = word1/32                  ! shift left 5 bits
      word1 = IAND(word1,z3)            ! remove all but 2 ls bits
      if (word1.ne.0)then
        write(txt,103)word1,nmf,msecs
        call ml_wt('info',txt)
        err = IOR(err,zstat)
      endif
c!
c!  parity check
c!
      if (partip(mf))then
        err = IOR(err,zpar)   ! genqc check on hrpt 2 lsb
      endif
        
      if (err.ne.0) then
        write(txt,105) err,nmf,msecs
        call ml_wt('info',txt)
      endif
c!
      return
 101  format(1x,'tip sat id ',i2,' not same as hrpt sat id ',i2,
     & ' at ',i8,' msecs')
 102  format(1x,'tip fr. sync byte invalid ***')
 103  format(1x,'tip status ',i2,' majfr ',i1,
     & ' at ',i8,' msecs')
 104  format(1x,'tip parity error')
 105  format('tip quality flag ',z8,' majfr ',i1,
     & ' at ',i8,' msecs')
      end
