c!
c!routine          tiptim   subroutine  fort77
c!
c!purpose          to extract time from tip word in minor frame 0.
c!
c!description      the tip words 8-12 in the minor frame 0 contains the
c!                 tip time code. this is extracted and output as msecs
c!                 and day. the time is referenced to the generation of the
c!                 first bit of the minor frame synch word in minor frame zero.
c!                 some quality control is applied to the time.
c!                 the time is packed as follows:
c!   word offset       9        10       11       12      13
c!   (1 relative)  ----------------------------------------------
c!                 |dddddddd|d0101sss|ssssssss|ssssssss|ssssssss|
c!                 ----------------------------------------------
c!                 daynumber  spare          msecs of day
c!
c!version          1.01   021294  r.w.saunders
! vesion           1.02   180396  m. derrien
!                          force day to hrpt day if not equal 
!                          at the first time
!                  1.03   020596 m. derrien 
!                          force day to hrpt day if not equal in any case
!                  1.04   200996 m. derrien
!                         add check first time and keep gotime false
!                         if not ok
!                  1.05   200197 m. derrien
!                         return checked msecs in any case 
!                         previous release returned bad msecs values
!                  1.06   March 03 T.Labrot and P.Brunel
!  f90 test : I.AND.J not correct and modified by IAND(I,J)
!             I.OR.J not correct and modified by IOR(I,J)
!             changed  type XXX/yyy/ 
!                  to  type XXX
!                      parameter (XXX=yyy)  or data XXX/yyy/ 
!
c!arguments
c!      integer imf       -- 1-5 to identify which minor frame is zero
c!      integer opmode    --  0 = off-line, 1= real time
c!      integer systim(3) -- current year, daynumber, msecs from midnight
c!      integer idat      -- extracted and checked hrpt day number
c!      integer itime     -- extracted and checked hrpt msecs from midnight
c!      integer err       -- error flag (0=ok)
c!      integer*2 hrpaip(104,5) -- current tip array
c!      logical*1 gotime  -- flag set to true when first tip time read
c!
c! common/iocom/ lulog,luhirs3,luamsa1,luamsa2,luamsb,luhirs2,lumsu,
c!               ludcs,lusem,lusbuv,lusar,luavh
c!common /log/msecs,day,nmf,nhirs,nmsu,namsua,namsub,ndcs
c!common /gqc/timok,partip,err_frsync,nb_frsync,err_auxsync,nb_auxsync
c!
c!files            fornnn log file output
c!
c!references       atovdc design document version 1.3 11 jan 95
c!
c!changes          01 none
c!
c!#########################################################################
c! copyright        the contents of this file are crown copyright. the
c!                  software, data and other proprietary information
c!                  contained therein is supplied for bonafide research
c!                  purposes. commercial or business use is not permitted
c!                  without the written authority of the u.k. met. office.
c!                  at london rd., bracknell, berks, rg12 2sz, u.k.
c!#########################################################################
      subroutine tiptim(imf,opmode,systim,idat,
     &itime,err,hrpaip,gotime)
c!
      implicit none
c!
      character*80 txt
      integer*2 hrpaip(104,5)           !5 tip/amsu/backfill minor frames
      integer*4 systim(3)               !system year, dayno, msecs from midnt
      integer*4 day,msecs,lasday,lasms  ! tip day no and msecs from midnight
      integer*4 idat                    ! hrpt day number
      integer*4 itime                   ! hrpt time msecs from midnight
      integer*4 imf                     ! tip minor frame counter
      integer*4 opmode                  ! real time (1) or off-line (0)
      integer*4 word1,word2,word3,word4 ! temporary words
      integer*4 err                     ! error code (0=ok)
      integer*4 ztim1                   ! bit mask
      parameter (ztim1=1082130432)
      integer*4 z7                      ! bit mask
      parameter (z7=7)
      integer*4 nmf                     ! no. of tip maj/minor frames read
      integer*4 nhirs,nmsu,namsua,namsub,ndcs !no scan lines
      logical*1 first,gotime
      data first/.true./
      logical timok,err_frsync,partip(5)
      integer*4 nb_frsync,nb_auxsync
      integer*4 cur_msecs! local variable for tip time 
      integer*4 cur_err
      logical err_auxsync  
c!
      common/gqc/timok,partip,err_frsync,nb_frsync,err_auxsync,
     & nb_auxsync
      common /log/msecs,day,nmf,nhirs,nmsu,namsua,namsub,ndcs
#include "hrptdc.h"
c!
c!  extract day number and time
c!
      word1 = hrpaip(9,imf)
      word1 = word1*2                 ! shift left 1 bit
      word2 = hrpaip(10,imf)
      word2 = word2/128               ! shift right 7 bits
      day = word1 + word2
      word1 = hrpaip(10,imf)
      word1 = IAND(word1,z7)          ! remove all but 3 lsb bits
      word1 = word1*16777216          ! shift left 24 bits
      word2 = hrpaip(11,imf)
      word2 = word2*65536             ! shift left 16 bits
      word3 = hrpaip(12,imf)
      word3 = word3*256               ! shift left 8 bits
      word4 = hrpaip(13,imf)
      cur_msecs = word1 + word2 + word3 + word4  ! combine 4 words
      cur_err=0
c!
c!  do some quality control on tip time
c!
      if (first) then
!
!     to avoid bad start date 
!     force day to hrpt day not equal
!
        if (idat .ne. day)then
          write(txt,103)idat,day,nmf
          call ml_wt('info',txt)
          day = idat
        endif
        lasday = day
        if(iabs(itime-cur_msecs).gt.1000) then
          write(txt,104)itime,cur_msecs,nmf
          call ml_wt('info',txt)
          err = IOR(err,ztim1)
          msecs=itime
          gotime = .false.
          return
        else
          msecs = cur_msecs
          lasms = cur_msecs
          first=.false.
        endif 
      else
        if( (day-lasday) .gt. 1 .or. (day-lasday).lt.0)then
c!          write(lulog,101)day,nmf
          write(txt,101)day,nmf
          call ml_wt('info',txt)
          cur_err = ztim1
        else
          lasday = day
        endif
        if((cur_msecs-lasms) .gt. 32100 .or. (msecs-lasms) .lt.1)then
c!          write(lulog,102)msecs,nmf
          write(txt,102) msecs,nmf
          call ml_wt('info',txt)
          cur_err = ztim1
        endif
        lasms = cur_msecs
      endif
c!
c!  compare with hrpt date & time
c!  modification M. Derrien ( 02 May 96)
c!  correct tip day from hrpt day when not equal
c!
      if (idat .ne. day)then
c!          write(lulog,103)idat,day,nmf
        write(txt,103)idat,day,nmf
        call ml_wt('info',txt)
        cur_err = ztim1
        day=idat
      endif
      if (timok)then
        if (iabs(itime-cur_msecs).gt.1000)then
c!          write(lulog,104)itime,cur_msecs,nmf
          write(txt,104)itime,cur_msecs,nmf
          call ml_wt('info',txt)
          cur_err = ztim1
          cur_msecs=itime
        endif
      endif
c!
c!  if real time ingest check tip date consistent with system date
c!
      if ( opmode .eq. 1 .and. systim(2).ne. day) then
        write(txt,105)systim(2),day,nmf
        call ml_wt('info',txt)
      endif
      if(cur_err.eq.0) then
        gotime = .true.              ! set flag
        msecs = cur_msecs
        lasms=msecs
      else 
        msecs=msecs+100
        err=IOR(err,ztim1)
      endif
         

        
c!
c!
      return
 101  format('bad tip day number ',i5,' at mjf ',
     &i4)
 102  format('bad tip time ',i8,' at mjf ',i4)
 103  format('hrpt day ',i3,' neq to tip day ',i3,
     &' at major frame ',i4)
 104  format('hrpt msec ',i8,' neq tip msec ',
     &i8,' at major frame ',i4)
 105  format('syst day is ',i3,' tip day is ',
     &i3,' at major frame ',i4)
      end
