#!/bin/ksh
#
# Korn Shell to run MAIA4 for VIIRS SDR level data 
#
# AAPP_PREFIX environment variable must be defined
#
# PROPRIETAIRE/OWNER
#       NWP SAF
#
# COPYRIGHT
#    This software was developed within the context of the EUMETSAT Satellite
#    Application Facility on Numerical Weather Prediction (NWP SAF), under the
#    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
#    Met Office, UK, by one or more partners within the NWP SAF. The partners
#    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
#
#    Copyright 2006, EUMETSAT, All Rights Reserved.
#
# HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
# (version   date         comment                       <author>)
#  1.0    October 2002   initial                        P.ROQUET (MF)
#  1.1    December 2017  update for NOAA20              P.ROQUET (MF)
#-------------------------------
# Parameters
#-------------------------------
if [ "$AAPP_PREFIX" = "" ]; then
  echo 'Error: AAPP_PREFIX not defined'
  exit 2
fi
if [ ! -d "$AAPP_PREFIX" ]; then
  echo 'Error: AAPP_PREFIX is not a directory:' $AAPP_PREFIX
  exit 2
fi

. "$AAPP_PREFIX/ATOVS_CONF"

#-------------------------------
# External functions
#-------------------------------
. liblog.sh

#-------------------------------
# Usage
#-------------------------------

usage(){
	echo 'usage : MAIA4_RUN path_name '
	echo '  path_name : pathname of the input VIIRS SDR files'
	}

#-------------------------------	
#  Get the argument
#-------------------------------
	
if [ $# -ne 1 ] ; then
  usage
  exit 2
fi
DIR=$1

log_info "Input  file in: ${DIR} directory"	
work_dir=`pwd`

#------------------------
# Verify the input files 
#------------------------
for GMTCO in ${DIR}/GMTCO_*.h5
do
namegeo=`basename $GMTCO`
YYYYMMDD=`echo $namegeo | cut -b 12-19`
HHMNSS=`echo $namegeo | cut -b 22-28`
NNNNN=`echo $namegeo | cut -b 40-44`
SATIMG=`echo $namegeo | awk -F_ '{print $2}'`
echo $PATH
maia4 -S ${SATIMG} -d ${YYYYMMDD} -h ${HHMNSS} -n ${NNNNN} ${GMTCO}

return_code=$?
if [ $return_code != 0 ]
then
 log_info "pb while running maia4"
 exit 1
fi




done

#------------------------------------------------
# Take the fundamental informations from the file
#------------------------------------------------

#-------------------------
# Link and Run
#-------------------------
