#!/bin/ksh
#
# $ Id $  
#
# run maia4 on avhrr
#
# PROPRIETAIRE/OWNER
#       NWP SAF
#
# COPYRIGHT
#    This software was developed within the context of the EUMETSAT Satellite
#    Application Facility on Numerical Weather Prediction (NWP SAF), under the
#    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
#    Met Office, UK, by one or more partners within the NWP SAF. The partners
#    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
#
#    Copyright 2006, EUMETSAT, All Rights Reserved.
#
# HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
# (version   date         comment                       <author>)
#  1.0       June 2017    initial                        P.ROQUET (MF)
#
#
#
#-------------------------------
# Parameters
#-------------------------------


if [ "$AAPP_PREFIX" = "" ]; then
  echo 'Error: AAPP_PREFIX not defined'
  exit 2
fi
if [ ! -d "$AAPP_PREFIX" ]; then
  echo 'Error: AAPP_PREFIX is not a directory:' $AAPP_PREFIX
  exit 2
fi

. "$AAPP_PREFIX/ATOVS_CONF"
#-------------------------------
# External functions
#-------------------------------
. liblog.sh
#-------------------------------
# Usage
#-------------------------------

usage(){
	echo 'usage : MAIA4_RUN_AVHRR filename(pfs00 or 1B) or hrpt.l1b file '
	}

#-------------------------------	
#  Get the argument
#-------------------------------
	
if [ $# -ne 1 ] ; then
  usage
  exit 2
fi
srcfile=$1

curdir=`pwd`
# take in input l1b or pfs file
inputformat="X"
# first look if it is an hrpt l1b files (same method as in convert_to_hdf5)

set $(head -c 78 $srcfile | od -v -An -t u2)
if [ $9 = 0 ] && [ ${10} = 0 ] && [ ${11} = 0 ]; then
      level=1b
      datatyp=${39}
      ins=""
      case $datatyp in 
         3 |  768 ) ins=hrpt 
                    inputformat=l1b  ;;
      esac
else
  # look if it is a PFS file and check his level 
  # PFS1B : PROCESSING_LEVEL              = 1B
  # PFS00:  PROCESSING_LEVEL              = 00
  inputformat=`dd if=${srcfile} bs=1 skip=20 | head -n 10 | grep "PROCESSING_LEVEL" | awk -F"=" '{print $2}'`
fi
log_info "inputformat=$inputformat"
case $inputformat in
*00)
    pfs0=$srcfile
    log_info "-------- decommutation ----------"
    flags="-ignore_degraded_inst_mdr -ignore_degraded_proc_mdr"
    decom-avhrr-metop $flags ${pfs0} hrpt.l1b > decom-avhrr-metop.log \
        || { log_error "decom-avhrr-metop failed"; exit 1; }
    l1b=hrpt.l1b
    SATIMG=`l1bidf -s $l1b`
    YYYYMMDD=`l1bidf -d $l1b`
    HHMN=`l1bidf -h $l1b`
    NNNNN=`l1bidf -n $l1b`

    log_info "---------- avhrcl ---------"

    ls -l hrpt.l1b
    log_info "DIR_NAVIGATION=${DIR_NAVIGATION}"
    log_info "DIR_CALIBRATION=${DIR_CALIBRATION}"
    avhrcl -c -l -s ${SATIMG} -d ${YYYYMMDD} -h ${HHMN}  -n ${NNNNN}  hrpt.l1b \
       || { log_error "avhrcl failed"; cat avhrcl.log ;rm -f hrpt.l1b;  exit 1; }
    cat avhrcl.log
    
    log_info "---------- avh_get_vis_coefs ---------"

    avh_get_vis_coefs -s ${SATIMG} -d ${YYYYMMDD} hrpt.l1b  \
       || { log_error "avh_get_vis_coefs failed"; exit 1; }

    l1b=${curdir}/hrpt.l1b

  
;;
*1B)
  pfsfile=$srcfile
  l1b=`basename ${pfsfile}`
  l1b=${l1b}.1b

  # convert pfs file to l1b
  convert_avh1b ${pfsfile} ${l1b}
  l1b=${curdir}/${l1b}  
  if [ ! -f ${l1b} ]
  then
    log_info "error cannot create 1b file"
    exit 1
  fi
;;
l1b)
  l1b=${srcfile}
;;
*) log_error "input format not recognized"
   exit 1;;
esac 
echo "l1bfile :  $l1b"
SATIMG=`l1bidf -s $l1b`
YYYYMMDD=`l1bidf -d $l1b`
HHMNSS=`l1bidf -h $l1b`
NNNNN=`l1bidf -n $l1b`
echo "${SATIMG} ${YYYYMMDD} ${HHMNSS} ${NNNNN}"
if [ -z "$NNNNN" ]
then
  NNNNN=00000
fi
echo "SATIMG=$SATIMG"
# avhrrin

l1c=`basename ${l1b}`
l1c=${curdir}/${l1c}.l1c

echo "avhrrin"
avhrrin -i ${l1b} -o ${l1c} || { log_error "avhrrin failed"; exit 1; }

# convert  1c to h5
avh=${l1c}.h5
echo "convert to h5 file"
avh1c_to_hdf5.exe ${l1c} ${avh}

if [ ! -f ${avh} ]
then
  echo "hdf5 file not present"
  exit 1
fi
# run maia
echo "maia4_Avhrr -s $SATIMG -d ${YYYYMMDD} -h ${HHMNSS}  -n ${NNNNN} ${avh}"
maia4_Avhrr -s $SATIMG -d ${YYYYMMDD} -h ${HHMNSS} -n ${NNNNN} ${avh} \
         || { log_error "maia4_Avhrr failed"; exit 1; }

exit 0


