#! /usr/bin/env python
#
# Python tool for merging MAIA4 HDF5 files
#
# Author: P. Roquet 2014
#
# usage: naggmaia4 list_of_maia4_files
#
# result file is named maia4_agg.h5 in current directory
#
import numpy
import h5py
import sys
import glob

if __name__ == '__main__':
    """ aggregate viiCT granules
    """
    if len(sys.argv) < 2:
        print("syntaxe:", sys.argv[0], "list of viiCT maia files")
        sys.exit(1)
    fileList = sys.argv[1:]
    h5File = "maia4_agg.h5"
    datasetList = {}
    datasetList1D = {}
    datasetListStatic = {}
    datasetListSpecific = {}
    datasetList['DATA'] = ["Alt_surface", "CloudMask", "CloudTopPres",
                           "CloudTopTemp", "CloudType", "Latitude",
                           "Longitude", "Mask_ind", "Sat_zenith", "Tsurf",
                           "fov_qual"]
    datasetList1D['DATA'] = ["scnlin_day", "scnlin_time", "scnlin_year",
                             "scnline_qual"]
    datasetListStatic['DATA'] = ["Nb_IChan_Used", "Nb_MChan_Used",
                                 "MChan_Used", "IChan_Used"]
    datasetListSpecific['DATA'] = [None]
    datasetList['HEADER'] = [None]
    datasetList1D['HEADER'] = ["ModScan"]
    datasetListStatic['HEADER'] = ["CMa_Version", "N_Granule_ID", "Nb_pixels",
                                   "dataname", "dataname_gmodo", "datatyp",
                                   "inst_id", "sat_id", "site1b", "site1c",
                                   "start_day", "start_orbit", "start_time",
                                   "start_year"]
    datasetListSpecific['HEADER'] = ["Nb_Granules", "Nb_lines", "end_day",
                                     "end_orbit", "end_time", "end_year"]
    datasetSum = ["Nb_Granules", "Nb_lines"]
    datasetLast = ["end_day", "end_orbit", "end_time", "end_year"]
    datasetDic = {}
    arrayDic = {}
    arrayDicAttr = {}
    fileList.sort()
    fileName = fileList[0]
    print(fileName)
    viiCT0 = h5py.File(fileName, 'r')
    for mds in ['DATA', 'HEADER']:
        datasetDic[mds] = {}
        arrayDic[mds] = {}
        arrayDicAttr[mds] = {}
        group = viiCT0[mds]
        for dataset in (datasetList[mds] + datasetList1D[mds]
                        + datasetListSpecific[mds]):
            if dataset:
                datasetDic[mds][dataset] = group[dataset]
                arrayDic[mds][dataset] = numpy.array(datasetDic[mds][dataset])
                arrayDicAttr[mds][dataset] = group[dataset].attrs.items()

    finalshape = {}
    for fileName in fileList[1:]:
        print(fileName)
        viiCT = h5py.File(fileName, 'r')
        for mds in ['DATA', 'HEADER']:
            group = viiCT[mds]
            for dataset in (datasetList[mds] + datasetList1D[mds]
                            + datasetListSpecific[mds]):
                if dataset:
                    data = group[dataset]
                    array = numpy.array(data)
                    if dataset in datasetSum:
                        arrayDic[mds][dataset] +=  array
                    elif dataset in datasetLast:
                        arrayDic[mds][dataset] = array
                    else:
                        if dataset in datasetList[mds]:
                            arrayDic[mds][dataset] = numpy.vstack(
                                (arrayDic[mds][dataset], array))
                        elif dataset in datasetList1D[mds]:
                            arrayDic[mds][dataset] = numpy.hstack(
                                (arrayDic[mds][dataset], array))
                        finalshape[dataset] = arrayDic[mds][dataset].shape
        viiCT.close()

    print("create output file", h5File)
    viiCT = h5py.File(h5File, "w")
    for mds in ['DATA', 'HEADER']:
        viiCT.create_group(mds)
        group = viiCT[mds]
        for dataset in (datasetList[mds] +
                        datasetList1D[mds] + datasetListSpecific[mds]):
            if dataset:
                group.create_dataset(dataset, arrayDic[mds][dataset].shape,
                                     arrayDic[mds][dataset].dtype,
                                     arrayDic[mds][dataset])
                for name, value in arrayDicAttr[mds][dataset]:
                    if name not in ["LBOUND", "UBOUND"]:
                        group[dataset].attrs[name] = value

        for dataset in datasetListStatic[mds]:
            header = viiCT0[mds + '/' + dataset]
            viiCT.copy(header, mds + '/' + dataset, mds + '/' + dataset)

    # Root attributes
    for name, value in viiCT0['/'].attrs.items():
        viiCT['/'].attrs[name] = value

    viiCT.close()
    viiCT0.close()
    print("end")
    sys.exit(0)
