! Name: $Id: viirs_edr_img.F90 530 2017-06-27 11:07:32Z frna $
Program viirs_edr_imgpp
! name:
!
! Purpose: Read a VIIRS EDR IMG file and geolocation file
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    19/05/2011  Original code. (P. Brunel)
!  1.1    05/01/2017  Clean code (P. Roquet)
! Code Description: F90
!
! LIBS = aapp_viirs atov auxaapp
!-------------------------------------------------------------------------------


#ifdef HAS_LIBHDF5
use hdf5
USE H5LT
USE aapp_hdf_mod
USE viirs_edr_img_def
USE viirs_edr_img_mem
USE viirs_edr_img_io

USE viirs_edr_img_mod
#include "aapp_throw.h"
implicit none

  character(len=256) :: tmpfilename
  character(len=256) :: filename(1)
  character(len=256) :: filenames(6)
  character(len=256) :: geolocfile

  character(len=3) :: band_name
  integer :: ios
  integer :: channels(6)
  integer :: nchannels, npixels, nlines, nscans, ngranules

  type(viirs_edr_img)  :: x
  INTEGER :: HDFERR
  INTEGER :: ERR
  
  integer ::  i, iopt

TRY
  channels  = 0
  filenames = ''
  
  CALL OPEN_HDF( HDFERR )
  THROWM(HDFERR.ne.0,"Cannot open HDF library")
  
  Read(*,*,iostat=ios) iopt

  If ( iopt .EQ. 1 ) THEN
  ! One Band, One file, List of channels
    Read(*,'(a)',iostat=ios) band_name

    Read(*,'(a)',iostat=ios) filename
  
    Read(*,*,iostat=ios) channels
    write(*,*) 'channels ',channels
    write(*,*) 'band_name ',band_name

    call viirs_edr_img_load(band_name, x, filename, err, channels=channels)
      
  ELSE IF ( iopt .EQ. 2 ) THEN
  ! One band, One file, All channels
    Read(*,'(a)',iostat=ios) band_name

    Read(*,'(a)',iostat=ios) filename
  
    write(*,*) 'band_name ',band_name

    call viirs_edr_img_load(band_name, x, filename, err)
    

  ELSE IF ( iopt .EQ. 3 ) THEN
  ! One band, list of [file, channel]
    Read(*,'(a)',iostat=ios) band_name
    i = 0
    Do
      Read(*,'(a)',iostat=ios) tmpfilename
      if ( ios .NE. 0 ) exit
      i = i +1
      filenames(i) = tmpfilename
      Read(*,*,iostat=ios) channels(i)
    EndDo
    
    write(*,*) 'channels ',channels
    write(*,*) 'band_name ',band_name

    call viirs_edr_img_load(band_name, x, filenames, err, channels=channels, clean=.true.)

  ELSE IF ( iopt .EQ. 4 ) THEN
  ! One band, One file, One GeoFile, list of channels
    Read(*,'(a)',iostat=ios) band_name

    Read(*,'(a)',iostat=ios) filename
    Read(*,'(a)',iostat=ios) geolocfile

    Read(*,*,iostat=ios) channels
    write(*,*) 'channels ',channels
    write(*,*) 'band_name ',band_name

    call viirs_edr_img_load(band_name, x, filename, err, channels=channels, geolocfile=geolocfile)

  END IF
  
  CALL viirs_edr_img_info( x, nchannels, npixels, nlines, nscans, ngranules, err )
  write(*,*) "nchannels ", nchannels
  write(*,*) "npixels   ", npixels
  write(*,*) "nlines    ", nlines
  write(*,*) "nscans    ", nscans
  write(*,*) "ngranules ", ngranules

  CALL viirs_edr_img_writea(x,20,err) 
  THROWM(ERR.ne.0,"Cannot write ascii file")

  CALL viirs_edr_img_free(x,err) 
  THROWM(HDFERR.ne.0,"Cannot free VIIRS EDR structure")

  CALL CLOSE_HDF( HDFERR )
  THROWM(HDFERR.ne.0,"Cannot close HDF library")

PCATCH

#else

  write(0,'("viirs_edr_img.exe has not been built with HDF-5 Fortran interface")')
  write(0,'("Please run configure with --external-libs=-Lxxx/lib -lhdf5hl_fortran")')
  write(0,'(" and --external-includes=-Lxxx/include")')
  write(0,'(" where xxx is the location of the HDF-5 library")')
  
#endif

End Program viirs_edr_imgpp
