MODULE AAPP_HDF_MOD
!
! Copyright:
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
#ifdef HAS_LIBHDF5

#include "aapp_throw.h"

USE HDF5

USE H5LT

IMPLICIT NONE


CHARACTER(LEN=*), PARAMETER :: H5_EXT = ".H5"

INTEGER(HID_T) :: H5T_JPRB_REAL, H5T_FILE_REAL, H5T_INTEGER, H5T_FILE_INTEGER, H5T_STD_I64

PUBLIC :: H5_EXT, H5T_JPRB_REAL, H5T_FILE_REAL, H5T_INTEGER, H5T_FILE_INTEGER, H5T_STD_I64
PUBLIC :: OPEN_HDF
PUBLIC :: CLOSE_HDF
PUBLIC :: WRITE_ARRAY_HDF
PUBLIC :: READ_ARRAY_HDF

PUBLIC :: EXGRP, MKDSR, MKPAR


PUBLIC :: PRINT_ARRAY_INTEGER
PUBLIC :: PRINT_ARRAY_REAL
PUBLIC :: PRINT_ARRAY_CHARACTER
PUBLIC :: PRINT_ARRAY_LONG

PUBLIC :: CLEAN_STRING

PRIVATE

CONTAINS

      SUBROUTINE OPEN_HDF(  ERR )
!
        INTEGER, INTENT(OUT) :: ERR
!
        INTEGER :: HDFERR
TRY
        CALL H5OPEN_F( HDFERR )
        ERR = HDFERR
        THROW(ERR.NE.0)

        H5T_JPRB_REAL    = H5T_NATIVE_REAL
        H5T_FILE_REAL    = H5T_NATIVE_REAL

        H5T_INTEGER = H5T_NATIVE_INTEGER
        H5T_FILE_INTEGER = H5T_NATIVE_INTEGER

        H5T_STD_I64 = H5T_STD_I64LE

#ifndef AAPP_BIG_ENDIAN
#define AAPP_BIG_ENDIAN 4321
#define AAPP_LITTLE_ENDIAN 1234
#endif
#if defined( AAPP_BYTE_ORDER ) && AAPP_BYTE_ORDER == AAPP_BIG_ENDIAN
        H5T_STD_I64 = H5T_STD_I64BE
#endif

CATCH
      END SUBROUTINE

      SUBROUTINE CLOSE_HDF( ERR )
!
        INTEGER, INTENT(OUT) :: ERR
!
        INTEGER :: HDFERR

TRY
        CALL H5CLOSE_F( HDFERR )
        ERR = HDFERR
        THROW(ERR.NE.0)
CATCH
      END SUBROUTINE


      SUBROUTINE WRITE_ARRAY_HDF( LUN, NAME, COMMENT, ERR, R0, R1, R2, R3, I0, I1, &
        I2, I3, C0, C1, C2, C3 ,IL1, L0, L1, UNITS, COMPRESS, COMPDIMS)
!
      INTEGER(HID_T), INTENT(IN) :: LUN
      CHARACTER(LEN=*),  INTENT(IN) :: NAME
      CHARACTER(LEN=*),  INTENT(IN) :: COMMENT
      INTEGER, INTENT(OUT) :: ERR
      REAL, INTENT(IN), OPTIONAL :: R0
      REAL, INTENT(IN), OPTIONAL :: R1(:)
      REAL, INTENT(IN), OPTIONAL :: R2(:,:)
      REAL, INTENT(IN), OPTIONAL :: R3(:,:,:)
      INTEGER, INTENT(IN), OPTIONAL :: I0
      INTEGER, INTENT(IN), OPTIONAL :: I1(:)
      INTEGER, INTENT(IN), OPTIONAL :: I2(:,:)
      INTEGER, INTENT(IN), OPTIONAL :: I3(:,:,:)
      CHARACTER(LEN=*), INTENT(IN), OPTIONAL :: C0
      CHARACTER(LEN=*), INTENT(IN), target, OPTIONAL :: C1(:)
      CHARACTER(LEN=*), INTENT(IN), OPTIONAL :: C2(:,:)
      CHARACTER(LEN=*), INTENT(IN), OPTIONAL :: C3(:,:,:)
      INTEGER*8, INTENT(IN), OPTIONAL :: IL1(:)
      LOGICAL, INTENT(IN), OPTIONAL :: L0
      LOGICAL, INTENT(IN), OPTIONAL :: L1(:)
      CHARACTER(LEN=*),  INTENT(IN) , OPTIONAL :: UNITS
      LOGICAL, INTENT(IN), OPTIONAL :: COMPRESS
      INTEGER, INTENT(IN), OPTIONAL :: COMPDIMS(:)

!
      INTEGER(HSIZE_T) :: DIMS(7), CHUNK_DIMS(7)
      INTEGER(SIZE_T) ::  RANK8
      INTEGER :: RANK, CHUNK_RANK
      INTEGER(HID_T) :: TYPE_F_ID, SPCE_ID, DSET_ID, CRP_LIST
      INTEGER :: UB(7), LB(7)
      INTEGER :: I
      INTEGER :: IL0
      INTEGER, ALLOCATABLE:: LL1(:)
      LOGICAL :: SCAL, LEXT
            integer*4, allocatable :: WI4(:)
      INTEGER(SIZE_T)  :: SDIM
      LOGICAL :: LCOMPRESS
      LOGICAL :: GZIP_FLAG
      INTEGER :: CONFIG_FLAG, config_flag_both
      INTEGER :: CLEVEL
!
TRY
      CLEVEL = 6 ! COMPRESSION LEVEL
      DIMS = 0
      LB = 1
      UB = -1
      SCAL = .FALSE.
      TYPE_F_ID = -1

      IF( PRESENT(COMPRESS) ) THEN
#ifdef DEBUG
write(0,*) "compress present"
#endif
         LCOMPRESS = COMPRESS
      ELSE
         LCOMPRESS = .FALSE.
      ENDIF
#ifdef DEBUG
write(0,*) "lcompress ", lcompress
#endif
      IF( PRESENT( C0 ) ) THEN
        SCAL = .TRUE.
      ELSE IF( PRESENT( L0 ) ) THEN
        LB(1) = 1
        UB(1) = 1
        TYPE_F_ID = H5T_FILE_INTEGER
        SCAL = .TRUE.
      ELSE IF( PRESENT( R0 ) ) THEN
        LB(1) = 1
        UB(1) = 1
        TYPE_F_ID = H5T_FILE_REAL
        SCAL = .TRUE.
      ELSE IF( PRESENT( I0 ) ) THEN
        LB(1) = 1
        UB(1) = 1
        TYPE_F_ID = H5T_FILE_INTEGER
        SCAL = .TRUE.
      ELSE IF( PRESENT( C1 ) ) THEN
        LB(1:1) = LBOUND( C1 )
        UB(1:1) = UBOUND( C1 )
        !TYPE_F_ID = H5T_FILE_CHARACTER
        SDIM = LEN( C1(1) )
        CALL H5Tcopy_f(H5T_FORTRAN_S1, TYPE_F_ID, ERR)
        CALL H5Tset_size_f(TYPE_F_ID, sdim, ERR)

      ELSE IF( PRESENT( R1 ) ) THEN
        LB(1:1) = LBOUND( R1 )
        UB(1:1) = UBOUND( R1 )
        TYPE_F_ID = H5T_FILE_REAL
      ELSE IF( PRESENT( I1 ) ) THEN
        LB(1:1) = LBOUND( I1 )
        UB(1:1) = UBOUND( I1 )
        TYPE_F_ID = H5T_FILE_INTEGER
      ELSE IF( PRESENT( L1 ) ) THEN
        LB(1:1) = LBOUND( L1 )
        UB(1:1) = UBOUND( L1 )
        TYPE_F_ID = H5T_FILE_INTEGER
        DIMS(1) = SIZE( L1, 1 )
        ALLOCATE( LL1(DIMS(1)), STAT=ERR)
        THROWM(ERR.NE.0,"COULD NOT ALLOCATE LL1 ")
        LL1(:) = 0
        WHERE ( L1 ) 
          LL1 = 1
        END WHERE
      ELSE IF( PRESENT( R2 ) ) THEN
        LB(1:2) = LBOUND( R2 )
        UB(1:2) = UBOUND( R2 )
        TYPE_F_ID = H5T_FILE_REAL
      ELSE IF( PRESENT( I2 ) ) THEN
        LB(1:2) = LBOUND( I2 )
        UB(1:2) = UBOUND( I2 )
        TYPE_F_ID = H5T_FILE_INTEGER
      ELSE IF( PRESENT( R3 ) ) THEN
        LB(1:3) = LBOUND( R3 )
        UB(1:3) = UBOUND( R3 )
        TYPE_F_ID = H5T_FILE_REAL
      ELSE IF( PRESENT( I3 ) ) THEN
        LB(1:3) = LBOUND( I3 )
        UB(1:3) = UBOUND( I3 )
        TYPE_F_ID = H5T_FILE_INTEGER
      ELSE IF( PRESENT( IL1 ) ) THEN
        LB(1:1) = LBOUND( IL1 )
        UB(1:1) = UBOUND( IL1 )
        TYPE_F_ID = H5T_STD_I64
      ELSE
        ERR = 1
      ENDIF
      THROWM(ERR.NE.0,"COULD NOT CREATE "//TRIM(NAME))

      DIMS = UB - LB + 1
      DO I = 7, 1, -1
        IF( DIMS(I) .GT. 0 ) EXIT
      ENDDO
      RANK = I

      if(lcompress) then
        chunk_dims = dims

        if(  present(compdims) ) then
          do i = 1, size(compdims)
            if( compdims(i) .GT. 0) then
              chunk_dims(i) = compdims(i)
            endif
          enddo
        endif
      
        !where( COMPRESS .LE. 0 ) 
        !  chunk_dims = dims
        !elsewhere
        !  chunk_dims = compress
        !end where
        chunk_rank = rank

        do i=1,7
          if( chunk_dims(i) .gt. dims(i) ) then
            chunk_dims(i) = dims(i)
          endif
        enddo
#ifdef DEBUG
        write(0,*) "chunk_dims ",chunk_dims
        write(0,*) "chunk_rank ",chunk_rank
#endif
        CALL h5zfilter_avail_f(H5Z_FILTER_DEFLATE_F, gzip_flag, err)
        THROWM(ERR.NE.0,"h5zfilter_avail_f")

        if (.NOT. gzip_flag) then
          lcompress = .false.
        endif

        CALL h5zget_filter_info_f(H5Z_FILTER_DEFLATE_F, config_flag, err)
        THROWM(ERR.NE.0,"h5zget_filter_info_f")

          !
          ! Make sure h5zget_filter_info_f returns the right flag
          !
          config_flag_both=IOR(H5Z_FILTER_ENCODE_ENABLED_F,H5Z_FILTER_DECODE_ENABLED_F)
          if( gzip_flag ) then
              if (config_flag .NE. config_flag_both) then
                  if(config_flag .NE. H5Z_FILTER_DECODE_ENABLED_F)  then
                     lcompress = .false.
                  endif
              endif
          endif

          ! Continue only when encoder is available
          if ( IAND(config_flag,  H5Z_FILTER_ENCODE_ENABLED_F) .EQ. 0 ) then
                     lcompress = .false.
          endif
        endif
#ifdef DEBUG
if( Present(compress) ) then
  write(0,*) 'logical lcompress ',lcompress
endif
#endif

      IF( TYPE_F_ID .GE. 0 ) THEN
        CALL H5LEXISTS_F( LUN, NAME, LEXT, ERR )
        THROWM(ERR.NE.0,"COULD NOT TEST EXISTANCE "//TRIM(NAME))
        IF( LEXT ) THEN
          CALL H5LDELETE_F( LUN, NAME, ERR )
          THROWM(ERR.NE.0,"COULD NOT DELETE "//TRIM(NAME))
        ENDIF
        CALL H5SCREATE_SIMPLE_F(RANK, DIMS, SPCE_ID, ERR)
        THROWM(ERR.NE.0,"COULD NOT CREATE "//TRIM(NAME))

        IF( LCOMPRESS ) THEN
          CALL h5pcreate_f(H5P_DATASET_CREATE_F, crp_list, err)
          THROWM(ERR.NE.0,"COULD NOT CREATE PROPERTY LIST FOR "//TRIM(NAME))
          CALL h5pset_chunk_f(crp_list, chunk_rank, chunk_dims, err)
          THROWM(ERR.NE.0,"COULD NOT CREATE CHUNK FOR "//TRIM(NAME))
          CALL h5pset_deflate_f(crp_list, clevel, err)
          THROWM(ERR.NE.0,"COULD NOT SET DEFLATE FOR "//TRIM(NAME))
          CALL H5DCREATE_F(LUN, NAME, TYPE_F_ID, SPCE_ID, DSET_ID, ERR, CRP_LIST)
          THROWM(ERR.NE.0,"COULD NOT CREATE "//TRIM(NAME))
          CALL H5LTSET_ATTRIBUTE_STRING_F( LUN, NAME, "COMPRESSED", "DEFLATE FILTER"//CHAR(0), ERR)
          THROWM(ERR.NE.0,"COULD NOT WRITE COMPRESSED ATTRIBUTE IN "//TRIM(NAME))
        ELSE
          CALL H5DCREATE_F(LUN, NAME, TYPE_F_ID, SPCE_ID, DSET_ID, ERR)
          THROWM(ERR.NE.0,"COULD NOT CREATE "//TRIM(NAME))
        ENDIF

        CALL H5SCLOSE_F(SPCE_ID, ERR)
        THROWM(ERR.NE.0,"COULD NOT CLOSE "//TRIM(NAME))

        IF( LCOMPRESS ) THEN
          CALL h5pclose_f(crp_list, err)
          THROWM(ERR.NE.0,"COULD NOT CLOSE PROPERTY LIST FOR "//TRIM(NAME))
        ENDIF
      ENDIF

      IF( PRESENT( C0 ) ) THEN
        CALL H5LEXISTS_F( LUN, NAME, LEXT, ERR )
        THROWM(ERR.NE.0,"COULD NOT TEST EXISTANCE "//TRIM(NAME))
        IF( LEXT ) THEN
          CALL H5LDELETE_F( LUN, NAME, ERR )
          THROWM(ERR.NE.0,"COULD NOT DELETE "//TRIM(NAME))
        ENDIF
        
        CALL H5LTMAKE_DATASET_STRING_F( LUN, NAME, TRIM(C0)//CHAR(0), ERR)
      ELSE IF( PRESENT( R0 ) ) THEN
        CALL H5DWRITE_F(DSET_ID, H5T_JPRB_REAL,    R0, DIMS, ERR)
      ELSE IF( PRESENT( I0 ) ) THEN
        CALL H5DWRITE_F(DSET_ID, H5T_INTEGER, I0, DIMS, ERR)
      ELSE IF( PRESENT( L0 ) ) THEN
        IF( L0 ) THEN 
          IL0 = 1
        ELSE
          IL0 = 0
        ENDIF
        CALL H5DWRITE_F(DSET_ID, H5T_INTEGER, IL0, DIMS, ERR)
          
      ELSE IF( PRESENT( C1 ) ) THEN
        CALL H5Dwrite_f(dset_id, TYPE_F_ID, C1(1)(1:1), DIMS, err)

      ELSE IF( PRESENT( R1 ) ) THEN
        CALL H5DWRITE_F(DSET_ID, H5T_JPRB_REAL,    R1, DIMS, ERR)
      ELSE IF( PRESENT( I1 ) ) THEN
        CALL H5DWRITE_F(DSET_ID, H5T_INTEGER, I1, DIMS, ERR)
      ELSE IF( PRESENT( L1 ) ) THEN
        CALL H5DWRITE_F(DSET_ID, H5T_INTEGER, LL1, DIMS, ERR)
      ELSE IF( PRESENT( R2 ) ) THEN
        CALL H5DWRITE_F(DSET_ID, H5T_JPRB_REAL,    R2, DIMS, ERR)
      ELSE IF( PRESENT( I2 ) ) THEN
        CALL H5DWRITE_F(DSET_ID, H5T_INTEGER, I2, DIMS, ERR)
      ELSE IF( PRESENT( R3 ) ) THEN
        CALL H5DWRITE_F(DSET_ID, H5T_JPRB_REAL,    R3, DIMS, ERR)
      ELSE IF( PRESENT( I3 ) ) THEN
        CALL H5DWRITE_F(DSET_ID, H5T_INTEGER, I3, DIMS, ERR)
        
      ELSE IF( PRESENT( IL1 ) ) THEN

        ALLOCATE( WI4(1:(UB(1)-LB(1)+1)*2), STAT = ERR )
        THROWM(ERR.NE.0,"CANNOT ALLOCATE "//TRIM(NAME))
        
        WI4  = TRANSFER(IL1, WI4 )
        CALL H5DWRITE_F(DSET_ID, H5T_STD_I64, WI4, DIMS, ERR)

        DEALLOCATE( WI4, STAT = ERR )
        THROWM(ERR.NE.0,"CANNOT DEALLOCATE "//TRIM(NAME))


        
        
        
      ELSE
        ERR = 1
      ENDIF
      THROWM(ERR.NE.0,"COULD NOT WRITE "//TRIM(NAME))

      IF( PRESENT( L1 ) ) THEN
        DEALLOCATE(LL1, STAT=ERR)
        THROWM(ERR.NE.0,"COULD NOT DEALLOCATE LL1")
      ENDIF
      
      CALL H5LTSET_ATTRIBUTE_STRING_F( LUN, NAME, "COMMENT", COMMENT//CHAR(0), ERR)
      THROWM(ERR.NE.0,"COULD NOT WRITE COMMENT IN "//TRIM(NAME))
      
      IF( PRESENT (UNITS) ) THEN
        CALL H5LTSET_ATTRIBUTE_STRING_F( LUN, NAME, "UNITS", UNITS//CHAR(0), ERR)
        THROWM(ERR.NE.0,"COULD NOT WRITE UNITS IN "//TRIM(NAME))
      ENDIF
      
      IF( .NOT. SCAL ) THEN
        RANK8 = RANK
        CALL H5LTSET_ATTRIBUTE_INT_F( LUN, NAME, "LBOUND", LB, RANK8, ERR)
        CALL H5LTSET_ATTRIBUTE_INT_F( LUN, NAME, "UBOUND", UB, RANK8, ERR)
      ENDIF

      IF( TYPE_F_ID .GE. 0 ) THEN
        CALL H5DCLOSE_F(DSET_ID, ERR)
        THROWM(ERR.NE.0,"COULD NOT CLOSE "//TRIM(NAME))
      ENDIF

CATCH

      END SUBROUTINE


      SUBROUTINE READ_ARRAY_HDF( LUN, NAME, ERR, R0, R1, R2, R3, I0, I1, &
        I2, I3, IL1, C0, C1, L0, L1, PR1, PR2, PR3, PZ1, PZ2, PZ3, PI1, PIL1, PI2, PI3, PL1, PC1 )
!
      INTEGER(HID_T), INTENT(IN) :: LUN
      CHARACTER(LEN=*),  INTENT(IN) :: NAME
      INTEGER, INTENT(OUT) :: ERR
      REAL, INTENT(OUT), OPTIONAL :: R0
      REAL, INTENT(OUT), OPTIONAL :: R1(:)
      REAL, INTENT(OUT), OPTIONAL :: R2(:,:)
      REAL, INTENT(OUT), OPTIONAL :: R3(:,:,:)
      INTEGER, INTENT(OUT), OPTIONAL :: I0
      INTEGER, INTENT(OUT), OPTIONAL :: I1(:)
      INTEGER*8, INTENT(OUT), OPTIONAL :: IL1(:)
      INTEGER, INTENT(OUT), OPTIONAL :: I2(:,:)
      INTEGER, INTENT(OUT), OPTIONAL :: I3(:,:,:)
      LOGICAL, INTENT(OUT), OPTIONAL :: L0
      LOGICAL, INTENT(OUT), OPTIONAL :: L1(:)
      REAL, POINTER, OPTIONAL :: PR1(:)
      REAL, POINTER, OPTIONAL :: PR2(:,:)
      REAL, POINTER, OPTIONAL :: PR3(:,:,:)
      COMPLEX, POINTER, OPTIONAL :: PZ1(:)
      COMPLEX, POINTER, OPTIONAL :: PZ2(:,:)
      COMPLEX, POINTER, OPTIONAL :: PZ3(:,:,:)
      INTEGER, POINTER, OPTIONAL :: PI1(:)
      INTEGER, POINTER, OPTIONAL :: PI2(:,:)
      INTEGER, POINTER, OPTIONAL :: PI3(:,:,:)
      LOGICAL, POINTER, OPTIONAL :: PL1(:)
      INTEGER*8, POINTER, OPTIONAL :: PIL1(:)
      CHARACTER(LEN=*), INTENT(OUT), OPTIONAL :: C0
      CHARACTER(LEN=*), INTENT(OUT), OPTIONAL :: C1(:)
      CHARACTER(LEN=*), POINTER, OPTIONAL :: PC1(:)
!
      INTEGER(HSIZE_T) :: DIMS(7), MAXDIMS(7)
      INTEGER :: UB(7), LB(7)
      INTEGER :: I0X(1)
      REAL :: R0X(1)
      CHARACTER(LEN=256) :: FF
      INTEGER(HID_T) :: DSET_ID, SPCE_ID, MEMTYPE
      LOGICAL :: HASATT
      INTEGER :: I
      integer*4, allocatable :: WI4(:)
      INTEGER, ALLOCATABLE:: LL1(:)
      INTEGER(SIZE_T)  :: SDIM

TRY

      ERR = 0

      DIMS = 0

      LB = 0
      UB = 0

      IF( PRESENT( C0 ) ) THEN
        FF = ""
        CALL H5LTREAD_DATASET_STRING_F( LUN, NAME, FF, ERR)
        C0 = FF
        DO I = 1, LEN( C0 )
          IF( C0(I:I) .EQ. CHAR(0)) C0(I:I) = ' '
        ENDDO
      ELSE IF( PRESENT( R0 ) ) THEN
        DIMS(1) = 1
        CALL H5LTREAD_DATASET_F( LUN, NAME, H5T_JPRB_REAL, R0X, DIMS(1:1), ERR)
        R0 = R0X(1)
      ELSE IF( PRESENT( I0 ) ) THEN
        DIMS(1) = 1
        CALL H5LTREAD_DATASET_F( LUN, NAME, H5T_FILE_INTEGER, I0X, DIMS(1:1), ERR)
        I0 = I0X(1) 
      ELSE IF( PRESENT( L0 ) ) THEN
        DIMS(1) = 1
        CALL H5LTREAD_DATASET_F( LUN, NAME, H5T_FILE_INTEGER, I0X, DIMS(1:1), ERR)
        L0 = I0X(1) .EQ. 1
                
      ELSE IF( PRESENT( C1 ) ) THEN

!        
        CALL H5DOPEN_F( LUN, NAME, DSET_ID, ERR )
        THROWM(ERR.NE.0,"CANNOT OPEN "//TRIM(NAME))
        CALL H5DGET_SPACE_F( DSET_ID, SPCE_ID, ERR ) 
        THROWM(ERR.NE.0,"CANNOT GET SPACE OF "//TRIM(NAME))

        DIMS(1) = SIZE( C1, 1 )
        SDIM = LEN( C1(1) )

        CALL H5TCOPY_F(H5T_FORTRAN_S1, MEMTYPE, ERR)
        THROWM(ERR.NE.0,"CANNOT H5TCOPY "//TRIM(NAME))

        CALL H5TSET_SIZE_F(MEMTYPE, SDIM, ERR)
        THROWM(ERR.NE.0,"CANNOT H5TSET_SIZE "//TRIM(NAME))

        CALL H5DREAD_F(DSET_ID, MEMTYPE, C1(1)(1:1), DIMS, ERR, SPCE_ID)
        THROWM(ERR.NE.0,"CANNOT READ "//TRIM(NAME))

        CALL H5SCLOSE_F( SPCE_ID, ERR )
        THROWM(ERR.NE.0,"CANNOT CLOSE "//TRIM(NAME))
        CALL H5DCLOSE_F( DSET_ID, ERR )
        THROWM(ERR.NE.0,"CANNOT CLOSE "//TRIM(NAME))

      ELSE IF( PRESENT( R1 ) ) THEN
        DIMS(1) = SIZE( R1, 1 )
        CALL H5LTREAD_DATASET_F( LUN, NAME, H5T_JPRB_REAL, R1, DIMS(1:1), ERR)
      ELSE IF( PRESENT( I1 ) ) THEN
        DIMS(1) = SIZE( I1, 1 )
        CALL H5LTREAD_DATASET_F( LUN, NAME, H5T_FILE_INTEGER, I1, DIMS(1:1), ERR)
      ELSE IF( PRESENT( L1 ) ) THEN
        DIMS(1) = SIZE( L1, 1 )
        ALLOCATE( LL1(DIMS(1)), STAT=ERR)
        THROWM(ERR.NE.0,"COULD NOT ALLOCATE LL1 ")
        CALL H5LTREAD_DATASET_F( LUN, NAME, H5T_FILE_INTEGER, LL1, DIMS(1:1), ERR)
        WHERE( LL1 .EQ. 1 )
          L1 = .TRUE.
        ELSEWHERE
          L1 = .FALSE.
        END WHERE
        DEALLOCATE( LL1, STAT=ERR)
        THROWM(ERR.NE.0,"COULD NOT DEALLOCATE LL1 ")
      ELSE IF( PRESENT( R2 ) ) THEN
        DIMS(1) = SIZE( R2, 1 )
        DIMS(2) = SIZE( R2, 2 )
        CALL H5LTREAD_DATASET_F( LUN, NAME, H5T_JPRB_REAL, R2, DIMS(1:2), ERR)
      ELSE IF( PRESENT( I2 ) ) THEN
        DIMS(1) = SIZE( I2, 1 )
        DIMS(2) = SIZE( I2, 2 )
        CALL H5LTREAD_DATASET_F( LUN, NAME, H5T_FILE_INTEGER, I2, DIMS(1:2), ERR)
      ELSE IF( PRESENT( R3 ) ) THEN
        DIMS(1) = SIZE( R3, 1 )
        DIMS(2) = SIZE( R3, 2 )
        DIMS(3) = SIZE( R3, 3 )
        CALL H5LTREAD_DATASET_F( LUN, NAME, H5T_JPRB_REAL, R3, DIMS(1:3), ERR)
      ELSE IF( PRESENT( I3 ) ) THEN
        DIMS(1) = SIZE( I3, 1 )
        DIMS(2) = SIZE( I3, 2 )
        DIMS(3) = SIZE( I3, 3 )
        CALL H5LTREAD_DATASET_F( LUN, NAME, H5T_FILE_INTEGER, I3, DIMS(1:3), ERR)
      ELSE IF( PRESENT( PR1 ) ) THEN
      ELSE IF( PRESENT( PZ1 ) ) THEN
      ELSE IF( PRESENT( PI1 ) ) THEN
      ELSE IF( PRESENT( PL1 ) ) THEN
      ELSE IF( PRESENT( PC1 ) ) THEN
      ELSE IF( PRESENT( PR2 ) ) THEN
      ELSE IF( PRESENT( PZ2 ) ) THEN
      ELSE IF( PRESENT( PI2 ) ) THEN
      ELSE IF( PRESENT( PR3 ) ) THEN
      ELSE IF( PRESENT( PZ3 ) ) THEN
      ELSE IF( PRESENT( PI3 ) ) THEN
      ELSE IF( PRESENT( PIL1) ) THEN
      ELSE
        ERR = -1
      ENDIF

      IF( ERR .GE. 0 ) ERR = 0
      THROWM(ERR.NE.0,"COULD NOT READ "//TRIM(NAME))


!     CALL H5ESET_AUTO_F( 0, SAERR )

      CALL H5DOPEN_F( LUN, NAME, DSET_ID, ERR )
      THROWM(ERR.NE.0,"COULD NOT OPEN "//TRIM(NAME))
      CALL H5AEXISTS_F( DSET_ID, "LBOUND", HASATT, ERR )
      THROWM(ERR.NE.0,"COULD NOT READ "//TRIM(NAME))
      CALL H5DCLOSE_F( DSET_ID, ERR )
      THROWM(ERR.NE.0,"COULD NOT CLOSE "//TRIM(NAME))

      IF( HASATT ) THEN
        CALL H5LTGET_ATTRIBUTE_INT_F( LUN, NAME, "LBOUND", LB, ERR )
        THROWM(ERR.NE.0,"CANNOT READ LBOUND/"//TRIM(NAME))
        CALL H5LTGET_ATTRIBUTE_INT_F( LUN, NAME, "UBOUND", UB, ERR )
        THROWM(ERR.NE.0,"CANNOT READ UBOUND/"//TRIM(NAME))
      ELSE
        ERR = 0
        LB = 1

        CALL H5DOPEN_F( LUN, NAME, DSET_ID, ERR )
        THROWM(ERR.NE.0,"CANNOT OPEN "//TRIM(NAME))
        CALL H5DGET_SPACE_F( DSET_ID, SPCE_ID, ERR ) 
        THROWM(ERR.NE.0,"CANNOT GET SPACE OF "//TRIM(NAME))
        CALL H5SGET_SIMPLE_EXTENT_DIMS_F( SPCE_ID, DIMS, MAXDIMS, ERR ) 
        IF( ERR .GE. 0 ) ERR = 0
        THROWM(ERR.NE.0,"CANNOT GET DIMS OF "//TRIM(NAME))
        CALL H5SCLOSE_F( SPCE_ID, ERR )
        THROWM(ERR.NE.0,"CANNOT CLOSE "//TRIM(NAME))
        CALL H5DCLOSE_F( DSET_ID, ERR )
        THROWM(ERR.NE.0,"CANNOT CLOSE "//TRIM(NAME))

        UB = INT( DIMS )

      ENDIF

      IF( PRESENT( PR1 ) ) THEN
        ALLOCATE( PR1(LB(1):UB(1)), STAT = ERR )
        THROWM(ERR.NE.0,"CANNOT ALLOCATE "//TRIM(NAME))
        DIMS(1) = SIZE( PR1, 1 )
        CALL H5LTREAD_DATASET_F( LUN, NAME, H5T_JPRB_REAL, PR1, DIMS(1:1), ERR)
      ELSE IF( PRESENT( PI1 ) ) THEN
        ALLOCATE( PI1(LB(1):UB(1)), STAT = ERR )
        THROWM(ERR.NE.0,"CANNOT ALLOCATE "//TRIM(NAME))
        DIMS(1) = SIZE( PI1, 1 )
        CALL H5LTREAD_DATASET_F( LUN, NAME, H5T_FILE_INTEGER, PI1, DIMS(1:1), ERR)
        
      ELSE IF( PRESENT( PL1 ) ) THEN
        ALLOCATE( PL1(LB(1):UB(1)), LL1(LB(1):UB(1)), STAT = ERR )
        THROWM(ERR.NE.0,"CANNOT ALLOCATE "//TRIM(NAME))
        DIMS(1) = SIZE( PL1, 1 )
        CALL H5LTREAD_DATASET_F( LUN, NAME, H5T_FILE_INTEGER, LL1, DIMS(1:1), ERR)
        WHERE( LL1 .EQ. 1 )
          PL1 = .TRUE.
        ELSEWHERE
          PL1 = .FALSE.
        END WHERE
        DEALLOCATE( LL1, STAT=ERR)
        THROWM(ERR.NE.0,"COULD NOT DEALLOCATE LL1 ")
      ELSE IF( PRESENT( PC1 ) ) THEN
        ALLOCATE( PC1(LB(1):UB(1)), STAT = ERR )
        THROWM(ERR.NE.0,"CANNOT ALLOCATE "//TRIM(NAME))
        DIMS(1) = SIZE( PC1, 1 )

        CALL H5DOPEN_F( LUN, NAME, DSET_ID, ERR )
        THROWM(ERR.NE.0,"CANNOT OPEN "//TRIM(NAME))
        CALL H5DGET_SPACE_F( DSET_ID, SPCE_ID, ERR ) 
        THROWM(ERR.NE.0,"CANNOT GET SPACE OF "//TRIM(NAME))

        SDIM = LEN( PC1(1) )

        CALL H5TCOPY_F(H5T_FORTRAN_S1, MEMTYPE, ERR)
        THROWM(ERR.NE.0,"CANNOT H5TCOPY "//TRIM(NAME))

        CALL H5TSET_SIZE_F(MEMTYPE, SDIM, ERR)
        THROWM(ERR.NE.0,"CANNOT H5TSET_SIZE "//TRIM(NAME))

        CALL H5DREAD_F(DSET_ID, MEMTYPE, PC1(1)(1:1), DIMS, ERR, SPCE_ID)
        THROWM(ERR.NE.0,"CANNOT READ "//TRIM(NAME))

        CALL H5SCLOSE_F( SPCE_ID, ERR )
        THROWM(ERR.NE.0,"CANNOT CLOSE "//TRIM(NAME))
        CALL H5DCLOSE_F( DSET_ID, ERR )
        THROWM(ERR.NE.0,"CANNOT CLOSE "//TRIM(NAME))

      ELSE IF( PRESENT( PR2 ) ) THEN
        ALLOCATE( PR2(LB(1):UB(1),LB(2):UB(2)), STAT = ERR )
        THROWM(ERR.NE.0,"CANNOT ALLOCATE "//TRIM(NAME))
        DIMS(1) = SIZE( PR2, 1 )
        DIMS(2) = SIZE( PR2, 2 )
        CALL H5LTREAD_DATASET_F( LUN, NAME, H5T_JPRB_REAL, PR2, DIMS(1:2), ERR)
      ELSE IF( PRESENT( PI2 ) ) THEN
        ALLOCATE( PI2(LB(1):UB(1),LB(2):UB(2)), STAT = ERR )
        THROWM(ERR.NE.0,"CANNOT ALLOCATE "//TRIM(NAME))
        DIMS(1) = SIZE( PI2, 1 )
        DIMS(2) = SIZE( PI2, 2 )
        CALL H5LTREAD_DATASET_F( LUN, NAME, H5T_FILE_INTEGER, PI2, DIMS(1:2), ERR)
      ELSE IF( PRESENT( PR3 ) ) THEN
        ALLOCATE( PR3(LB(1):UB(1),LB(2):UB(2),LB(3):UB(3)), STAT = ERR )
        THROWM(ERR.NE.0,"CANNOT ALLOCATE "//TRIM(NAME))
        DIMS(1) = SIZE( PR3, 1 )
        DIMS(2) = SIZE( PR3, 2 )
        DIMS(3) = SIZE( PR3, 3 )
        CALL H5LTREAD_DATASET_F( LUN, NAME, H5T_JPRB_REAL, PR3, DIMS(1:3), ERR)
      ELSE IF( PRESENT( PI3 ) ) THEN
        ALLOCATE( PI3(LB(1):UB(1),LB(2):UB(2),LB(3):UB(3)), STAT = ERR )
        THROWM(ERR.NE.0,"CANNOT ALLOCATE "//TRIM(NAME))
        DIMS(1) = SIZE( PI3, 1 )
        DIMS(2) = SIZE( PI3, 2 )
        DIMS(3) = SIZE( PI3, 3 )
        CALL H5LTREAD_DATASET_F( LUN, NAME, H5T_FILE_INTEGER, PI3, DIMS(1:3), ERR)
      ELSE IF( PRESENT( PIL1 ) ) THEN
        ALLOCATE( PIL1(LB(1):UB(1)), STAT = ERR )
        THROWM(ERR.NE.0,"CANNOT ALLOCATE "//TRIM(NAME))
        ALLOCATE( WI4(1:(UB(1)-LB(1)+1)*2), STAT = ERR )
        THROWM(ERR.NE.0,"CANNOT ALLOCATE "//TRIM(NAME))
        DIMS(1) = SIZE( PIL1, 1 )
        
        CALL H5DOPEN_F( LUN, NAME, DSET_ID, ERR )
        THROWM(ERR.NE.0,"CANNOT OPEN "//TRIM(NAME))
        CALL H5DGET_SPACE_F( DSET_ID, SPCE_ID, ERR ) 
        THROWM(ERR.NE.0,"CANNOT GET SPACE OF "//TRIM(NAME))
        
        CALL h5dread_f(dset_id, H5T_STD_I64, WI4, dims, err)
        THROWM(ERR.NE.0,"CANNOT h5dread_f "//TRIM(NAME))

        PIL1  = TRANSFER(WI4, PIL1, DIMS(1) )

        CALL H5SCLOSE_F( SPCE_ID, ERR )
        THROWM(ERR.NE.0,"CANNOT CLOSE "//TRIM(NAME))
        CALL H5DCLOSE_F( DSET_ID, ERR )
        THROWM(ERR.NE.0,"CANNOT CLOSE "//TRIM(NAME))
        DEALLOCATE( WI4, STAT = ERR )
        THROWM(ERR.NE.0,"CANNOT DEALLOCATE "//TRIM(NAME))


      ELSE
        ERR = 1
      ENDIF
      IF( ERR .GE. 0 ) ERR = 0
      THROWM(ERR.NE.0,"COULD NOT READ "//TRIM(NAME))

      
CATCH

      END SUBROUTINE


RECURSIVE SUBROUTINE PRINT_ARRAY_REAL( LUN, FORMAT, A0, A1, A2, A3, A4, A5, A6, A7 )
INTEGER, INTENT(IN)           :: LUN
CHARACTER(LEN=*),   INTENT(IN)           :: FORMAT
REAL,    INTENT(IN), OPTIONAL :: A0 
REAL,    INTENT(IN), OPTIONAL :: A1(:) 
REAL,    INTENT(IN), OPTIONAL :: A2(:,:)
REAL,    INTENT(IN), OPTIONAL :: A3(:,:,:)
REAL,    INTENT(IN), OPTIONAL :: A4(:,:,:,:)
REAL,    INTENT(IN), OPTIONAL :: A5(:,:,:,:,:)
REAL,    INTENT(IN), OPTIONAL :: A6(:,:,:,:,:,:) 
REAL,    INTENT(IN), OPTIONAL :: A7(:,:,:,:,:,:,:)
INTEGER :: I, J, L1, U1
INTEGER, PARAMETER :: N = 10

IF( PRESENT( A0 ) ) THEN
  WRITE( LUN, FMT = FORMAT ) A0
ELSE IF( PRESENT( A1 ) ) THEN
  L1 = LBOUND( A1, 1 )
  U1 = UBOUND( A1, 1 )
  DO I = L1, U1, N
    J = I + N - 1
    IF( J .GT. U1 ) J = U1
    WRITE( LUN, FMT = FORMAT ) A1(I:J)
  ENDDO
ELSE IF( PRESENT( A2 ) ) THEN
  DO I = 1, SIZE( A2, 1 )
    CALL PRINT_ARRAY_REAL( LUN, FORMAT, A1 = A2(I,:) )
    WRITE( LUN, * )
  ENDDO
ELSE IF( PRESENT( A3 ) ) THEN
  DO I = 1, SIZE( A3, 1 )
    CALL PRINT_ARRAY_REAL( LUN, FORMAT, A2 = A3(I,:,:) )
    WRITE( LUN, * )
  ENDDO
ELSE IF( PRESENT( A4 ) ) THEN
  DO I = 1, SIZE( A4, 1 )
    CALL PRINT_ARRAY_REAL( LUN, FORMAT, A3 = A4(I,:,:,:) )
    WRITE( LUN, * )
  ENDDO
ELSE IF( PRESENT( A5 ) ) THEN
  DO I = 1, SIZE( A5, 1 )
    CALL PRINT_ARRAY_REAL( LUN, FORMAT, A4 = A5(I,:,:,:,:) )
    WRITE( LUN, * )
  ENDDO
ELSE IF( PRESENT( A6 ) ) THEN
  DO I = 1, SIZE( A6, 1 )
    CALL PRINT_ARRAY_REAL( LUN, FORMAT, A5 = A6(I,:,:,:,:,:) )
    WRITE( LUN, * )
  ENDDO
ELSE IF( PRESENT( A7 ) ) THEN
  DO I = 1, SIZE( A7, 1 )
    CALL PRINT_ARRAY_REAL( LUN, FORMAT, A6 = A7(I,:,:,:,:,:,:) )
    WRITE( LUN, * )
  ENDDO
ENDIF

END SUBROUTINE
RECURSIVE SUBROUTINE PRINT_ARRAY_INTEGER( LUN, FORMAT, A0, A1, A2, A3, A4, A5, A6, A7 )
INTEGER, INTENT(IN)           :: LUN
CHARACTER(LEN=*),   INTENT(IN)           :: FORMAT
INTEGER,    INTENT(IN), OPTIONAL :: A0 
INTEGER,    INTENT(IN), OPTIONAL :: A1(:) 
INTEGER,    INTENT(IN), OPTIONAL :: A2(:,:)
INTEGER,    INTENT(IN), OPTIONAL :: A3(:,:,:)
INTEGER,    INTENT(IN), OPTIONAL :: A4(:,:,:,:)
INTEGER,    INTENT(IN), OPTIONAL :: A5(:,:,:,:,:)
INTEGER,    INTENT(IN), OPTIONAL :: A6(:,:,:,:,:,:) 
INTEGER,    INTENT(IN), OPTIONAL :: A7(:,:,:,:,:,:,:)
INTEGER :: I, J, L1, U1
INTEGER, PARAMETER :: N = 10

IF( PRESENT( A0 ) ) THEN
  WRITE( LUN, FMT = FORMAT ) A0
ELSE IF( PRESENT( A1 ) ) THEN
  L1 = LBOUND( A1, 1 )
  U1 = UBOUND( A1, 1 )
  DO I = L1, U1, N
    J = I + N - 1
    IF( J .GT. U1 ) J = U1
    WRITE( LUN, FMT = FORMAT ) A1(I:J)
  ENDDO
ELSE IF( PRESENT( A2 ) ) THEN
  DO I = 1, SIZE( A2, 1 )
    CALL PRINT_ARRAY_INTEGER( LUN, FORMAT, A1 = A2(I,:) )
    WRITE( LUN, * )
  ENDDO
ELSE IF( PRESENT( A3 ) ) THEN
  DO I = 1, SIZE( A3, 1 )
    CALL PRINT_ARRAY_INTEGER( LUN, FORMAT, A2 = A3(I,:,:) )
    WRITE( LUN, * )
  ENDDO
ELSE IF( PRESENT( A4 ) ) THEN
  DO I = 1, SIZE( A4, 1 )
    CALL PRINT_ARRAY_INTEGER( LUN, FORMAT, A3 = A4(I,:,:,:) )
    WRITE( LUN, * )
  ENDDO
ELSE IF( PRESENT( A5 ) ) THEN
  DO I = 1, SIZE( A5, 1 )
    CALL PRINT_ARRAY_INTEGER( LUN, FORMAT, A4 = A5(I,:,:,:,:) )
    WRITE( LUN, * )
  ENDDO
ELSE IF( PRESENT( A6 ) ) THEN
  DO I = 1, SIZE( A6, 1 )
    CALL PRINT_ARRAY_INTEGER( LUN, FORMAT, A5 = A6(I,:,:,:,:,:) )
    WRITE( LUN, * )
  ENDDO
ELSE IF( PRESENT( A7 ) ) THEN
  DO I = 1, SIZE( A7, 1 )
    CALL PRINT_ARRAY_INTEGER( LUN, FORMAT, A6 = A7(I,:,:,:,:,:,:) )
    WRITE( LUN, * )
  ENDDO
ENDIF
END SUBROUTINE

RECURSIVE SUBROUTINE PRINT_ARRAY_LONG( LUN, FORMAT, A0, A1, A2, A3, A4, A5, A6, A7 )
INTEGER, INTENT(IN)           :: LUN
CHARACTER(LEN=*),   INTENT(IN)           :: FORMAT
INTEGER*8,    INTENT(IN), OPTIONAL :: A0 
INTEGER*8,    INTENT(IN), OPTIONAL :: A1(:) 
INTEGER*8,    INTENT(IN), OPTIONAL :: A2(:,:)
INTEGER*8,    INTENT(IN), OPTIONAL :: A3(:,:,:)
INTEGER*8,    INTENT(IN), OPTIONAL :: A4(:,:,:,:)
INTEGER*8,    INTENT(IN), OPTIONAL :: A5(:,:,:,:,:)
INTEGER*8,    INTENT(IN), OPTIONAL :: A6(:,:,:,:,:,:) 
INTEGER*8,    INTENT(IN), OPTIONAL :: A7(:,:,:,:,:,:,:)
INTEGER :: I, J, L1, U1
INTEGER, PARAMETER :: N = 10

IF( PRESENT( A0 ) ) THEN
  WRITE( LUN, FMT = FORMAT ) A0
ELSE IF( PRESENT( A1 ) ) THEN
  L1 = LBOUND( A1, 1 )
  U1 = UBOUND( A1, 1 )
  DO I = L1, U1, N
    J = I + N - 1
    IF( J .GT. U1 ) J = U1
    WRITE( LUN, FMT = FORMAT ) A1(I:J)
  ENDDO
ELSE IF( PRESENT( A2 ) ) THEN
  DO I = 1, SIZE( A2, 1 )
    CALL PRINT_ARRAY_LONG( LUN, FORMAT, A1 = A2(I,:) )
    WRITE( LUN, * )
  ENDDO
ELSE IF( PRESENT( A3 ) ) THEN
  DO I = 1, SIZE( A3, 1 )
    CALL PRINT_ARRAY_LONG( LUN, FORMAT, A2 = A3(I,:,:) )
    WRITE( LUN, * )
  ENDDO
ELSE IF( PRESENT( A4 ) ) THEN
  DO I = 1, SIZE( A4, 1 )
    CALL PRINT_ARRAY_LONG( LUN, FORMAT, A3 = A4(I,:,:,:) )
    WRITE( LUN, * )
  ENDDO
ELSE IF( PRESENT( A5 ) ) THEN
  DO I = 1, SIZE( A5, 1 )
    CALL PRINT_ARRAY_LONG( LUN, FORMAT, A4 = A5(I,:,:,:,:) )
    WRITE( LUN, * )
  ENDDO
ELSE IF( PRESENT( A6 ) ) THEN
  DO I = 1, SIZE( A6, 1 )
    CALL PRINT_ARRAY_LONG( LUN, FORMAT, A5 = A6(I,:,:,:,:,:) )
    WRITE( LUN, * )
  ENDDO
ELSE IF( PRESENT( A7 ) ) THEN
  DO I = 1, SIZE( A7, 1 )
    CALL PRINT_ARRAY_LONG( LUN, FORMAT, A6 = A7(I,:,:,:,:,:,:) )
    WRITE( LUN, * )
  ENDDO
ENDIF

END SUBROUTINE

RECURSIVE SUBROUTINE PRINT_ARRAY_CHARACTER( LUN, FORMAT, A0, A1, A2, A3, A4, A5, A6, A7 )
INTEGER, INTENT(IN)           :: LUN
CHARACTER(LEN=*),   INTENT(IN)           :: FORMAT
CHARACTER(LEN=*),    INTENT(IN), OPTIONAL :: A0 
CHARACTER(LEN=*),    INTENT(IN), OPTIONAL :: A1(:) 
CHARACTER(LEN=*),    INTENT(IN), OPTIONAL :: A2(:,:)
CHARACTER(LEN=*),    INTENT(IN), OPTIONAL :: A3(:,:,:)
CHARACTER(LEN=*),    INTENT(IN), OPTIONAL :: A4(:,:,:,:)
CHARACTER(LEN=*),    INTENT(IN), OPTIONAL :: A5(:,:,:,:,:)
CHARACTER(LEN=*),    INTENT(IN), OPTIONAL :: A6(:,:,:,:,:,:) 
CHARACTER(LEN=*),    INTENT(IN), OPTIONAL :: A7(:,:,:,:,:,:,:)
INTEGER :: I, J, L1, U1
INTEGER, PARAMETER :: N = 10

IF( PRESENT( A0 ) ) THEN
  WRITE( LUN, FMT = FORMAT ) TRIM(A0)
ELSE IF( PRESENT( A1 ) ) THEN
  L1 = LBOUND( A1, 1 )
  U1 = UBOUND( A1, 1 )
  DO I = L1, U1, N
    J = I + N - 1
    IF( J .GT. U1 ) J = U1
    WRITE( LUN, FMT = FORMAT ) A1(I:J)
  ENDDO
ELSE IF( PRESENT( A2 ) ) THEN
  DO I = 1, SIZE( A2, 1 )
    CALL PRINT_ARRAY_CHARACTER( LUN, FORMAT, A1 = A2(I,:) )
    WRITE( LUN, * )
  ENDDO
ELSE IF( PRESENT( A3 ) ) THEN
  DO I = 1, SIZE( A3, 1 )
    CALL PRINT_ARRAY_CHARACTER( LUN, FORMAT, A2 = A3(I,:,:) )
    WRITE( LUN, * )
  ENDDO
ELSE IF( PRESENT( A4 ) ) THEN
  DO I = 1, SIZE( A4, 1 )
    CALL PRINT_ARRAY_CHARACTER( LUN, FORMAT, A3 = A4(I,:,:,:) )
    WRITE( LUN, * )
  ENDDO
ELSE IF( PRESENT( A5 ) ) THEN
  DO I = 1, SIZE( A5, 1 )
    CALL PRINT_ARRAY_CHARACTER( LUN, FORMAT, A4 = A5(I,:,:,:,:) )
    WRITE( LUN, * )
  ENDDO
ELSE IF( PRESENT( A6 ) ) THEN
  DO I = 1, SIZE( A6, 1 )
    CALL PRINT_ARRAY_CHARACTER( LUN, FORMAT, A5 = A6(I,:,:,:,:,:) )
    WRITE( LUN, * )
  ENDDO
ELSE IF( PRESENT( A7 ) ) THEN
  DO I = 1, SIZE( A7, 1 )
    CALL PRINT_ARRAY_CHARACTER( LUN, FORMAT, A6 = A7(I,:,:,:,:,:,:) )
    WRITE( LUN, * )
  ENDDO
ENDIF

END SUBROUTINE
        SUBROUTINE wordSwap8(rec,n)        !For 4-byte words

        INTEGER*8   n
        INTEGER*8   rec(n)
        INTEGER*8   word
        CHARACTER*1 bytes(8),ctmp
        INTEGER*8   iW

        EQUIVALENCE (word,bytes)

        DO iW = 1,n
           word = rec(iW)
           ctmp = bytes(1)
           bytes(1) = bytes(8)
           bytes(8) = ctmp
           ctmp = bytes(2)
           bytes(2) = bytes(7)
           bytes(7) = ctmp
           ctmp = bytes(3)
           bytes(3) = bytes(6)
           bytes(6) = ctmp
           ctmp = bytes(4)
           bytes(4) = bytes(5)
           bytes(5) = ctmp
           rec(iW) = word
        ENDDO

        
        END SUBROUTINE
        
  SUBROUTINE CLEAN_STRING(str)
  CHARACTER(LEN=*),    INTENT(INOUT):: str
  integer :: i
  DO i = 1, LEN(str)
    IF( ICHAR(str(i:i)) .EQ. 0 ) str(i:i)=' '
  END DO
  END SUBROUTINE

      SUBROUTINE EXGRP( L_ID, NAME, HDFERR )
!
      INTEGER(HID_T),   INTENT(IN)  :: L_ID
      CHARACTER(LEN=*),    INTENT(IN)  :: NAME
      INTEGER,          INTENT(OUT) :: HDFERR
!
      INTEGER :: I1, I2
      INTEGER :: SAERR, HDFERR1
      INTEGER(HID_T) :: L1_ID, L2_ID
      INTEGER :: N
!

      HDFERR = 0

      L1_ID = L_ID
      N = LEN( TRIM(NAME) )

      I1 = 1
      DO1: DO 
        DO2: DO WHILE( NAME(I1:I1) .EQ. '/' ) 
           I1 = I1 + 1
           IF( I1 .GT. N ) EXIT DO1
        ENDDO DO2
        I2 = I1 + 1
        DO3: DO WHILE( NAME(I2:I2) .NE. '/' ) 
           I2 = I2 + 1
           IF( I2 .GT. N ) EXIT DO3
        ENDDO DO3
        I2 = I2 - 1
        CALL H5ESET_AUTO_F( 0, SAERR )

        CALL H5GOPEN_F( L1_ID, NAME(I1:I2), L2_ID, HDFERR )

        IF( L1_ID .NE. L_ID ) &
          CALL H5GCLOSE_F( L1_ID, HDFERR1 )

        IF( HDFERR .NE. 0 ) THEN 
          CALL H5ESET_AUTO_F( 1, SAERR )
          RETURN
        ENDIF
          
        I1    = I2 + 1
        L1_ID = L2_ID
        IF( I1 .GT. N ) EXIT DO1
      ENDDO DO1


      END SUBROUTINE


      SUBROUTINE MKPAR( L_ID, NAME, G_ID, HDFERR )
      ! Returns a data group Id 
      ! Creates it if does not exist
!
      INTEGER(HID_T),   INTENT(IN)  :: L_ID
      CHARACTER(LEN=*),    INTENT(IN)  :: NAME
      INTEGER(HID_T),   INTENT(OUT) :: G_ID  
      INTEGER,          INTENT(OUT) :: HDFERR
!
      INTEGER :: I1, I2
      INTEGER :: SAERR
      INTEGER(HID_T) :: L1_ID, L2_ID
      INTEGER :: N
!

      L1_ID = L_ID
      N = LEN( TRIM(NAME) )

!     PRINT *, " NAME = ", TRIM(NAME)

      I1 = 1
      DO1: DO 
        DO2: DO WHILE( NAME(I1:I1) .EQ. '/' ) 
           I1 = I1 + 1
           IF( I1 .GT. N ) EXIT DO1
        ENDDO DO2
        I2 = I1 + 1
        DO3: DO WHILE( NAME(I2:I2) .NE. '/' ) 
           I2 = I2 + 1
           IF( I2 .GT. N ) EXIT DO3
        ENDDO DO3
        I2 = I2 - 1
        CALL H5ESET_AUTO_F( 0, SAERR )

!       PRINT *, " P = ", NAME(I1:I2)
        CALL H5GOPEN_F( L1_ID, NAME(I1:I2), L2_ID, HDFERR )
!       PRINT *, " HDFERR = ", HDFERR
        IF( HDFERR .NE. 0 ) &
          CALL H5GCREATE_F( L1_ID, NAME(I1:I2), L2_ID, HDFERR )
!       PRINT *, " HDFERR = ", HDFERR
        CALL H5ESET_AUTO_F( 1, SAERR )
        IF( HDFERR .NE. 0 ) RETURN
        IF( L1_ID .NE. L_ID ) &
          CALL H5GCLOSE_F( L1_ID, HDFERR )
        I1    = I2 + 1
        L1_ID = L2_ID
        IF( I1 .GT. N ) EXIT DO1
      ENDDO DO1

      G_ID = L2_ID


      END SUBROUTINE

      RECURSIVE SUBROUTINE MKDSR( LOC_ID, NAME, DSET_ID, HDFERR, DIMS )
!
      INTEGER(HID_T),   INTENT(IN)  :: LOC_ID
      CHARACTER(LEN=*),    INTENT(IN)  :: NAME
      INTEGER(HID_T),   INTENT(OUT) :: DSET_ID  
      INTEGER,          INTENT(OUT) :: HDFERR
      INTEGER(HSIZE_T), INTENT(IN), OPTIONAL :: DIMS(:)
!

      INTEGER :: I1
      INTEGER :: SAERR
      INTEGER(HID_T) :: SPCE_ID
      INTEGER(HID_T) :: LOC1_ID

      IF( NAME(1:1) .EQ. '/' ) THEN
        CALL H5GOPEN_F( LOC_ID, '/', LOC1_ID, HDFERR )
        IF( HDFERR .NE. 0 ) RETURN
        CALL MKDSR( LOC1_ID, TRIM(NAME(2:)), DSET_ID, HDFERR, DIMS )
        RETURN
      ENDIF

      DO I1 = 1, LEN( TRIM( NAME ) )
        IF( NAME(I1:I1) .EQ. '/' ) EXIT
      ENDDO

      IF( I1 .LT. LEN( TRIM( NAME ) ) ) THEN
        CALL H5ESET_AUTO_F( 0, SAERR )
        CALL H5GOPEN_F( LOC_ID, NAME(1:I1-1), LOC1_ID, HDFERR )
        IF( HDFERR .NE. 0 ) &
          CALL H5GCREATE_F( LOC_ID, NAME(1:I1-1), LOC1_ID, HDFERR )
        CALL H5ESET_AUTO_F( 1, SAERR )
        IF( HDFERR .NE. 0 ) RETURN
        CALL MKDSR( LOC1_ID, TRIM(NAME(I1+1:)), DSET_ID, HDFERR, DIMS )
      ELSE
        CALL H5ESET_AUTO_F( 0, SAERR )
        CALL H5DOPEN_F( LOC_ID, NAME(1:I1-1), LOC1_ID, HDFERR )
        CALL H5ESET_AUTO_F( 1, SAERR )
        IF( HDFERR .NE. 0 ) THEN
          IF( PRESENT( DIMS ) ) THEN
            CALL H5SCREATE_SIMPLE_F( SIZE(DIMS), DIMS, SPCE_ID, HDFERR )
            IF( HDFERR .NE. 0 ) RETURN
            CALL H5DCREATE_F( LOC_ID, NAME(1:I1-1), H5T_FILE_REAL, SPCE_ID, LOC1_ID, HDFERR )
          ELSE
            HDFERR = 1
          ENDIF
        ENDIF
        IF( HDFERR .NE. 0 ) RETURN
        DSET_ID = LOC1_ID
      ENDIF

      END SUBROUTINE

#endif


END MODULE
