! Name: $Id: viirs_sdr_mod.F90
!
! Purpose: I/O routines for multi chanel/multi granule VIIRS SDR files in HDF5 format
!
! Method:
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!   1.0   05/2012       Initial version (Pascal Brunel)
!   1.1   10/12/2012    Release for AAPP Pascale Roquet)
!
!
!
module viirs_sdr_io
#include "aapp_throw.h"

use viirs_sdr_def
use viirs_sdr_mem
Use jpss_meta_root_io
Use jpss_meta_aggregate_io
Use jpss_meta_granule_io
Use jpss_meta_dataproducts_io
Use viirs_sdr_data_io
Use viirs_sdr_geo_io


implicit none
private

public :: viirs_sdr_writea


contains


subroutine viirs_sdr_writea(x,lun,err)
type(viirs_sdr),intent(in)::x
integer,intent(in)::lun
integer,intent(out)::err
integer::lb(7),ub(7)
character(len=80)::sname
integer :: ichan, nchan

TRY
sname='viirs_sdr'
write(lun,'("SECTION = ",A)',err=999)sname
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("COMMENT = ",A)',err=999)'VIIRS SDR file'
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

Call jpss_meta_root_writea(x%root_att,lun,err)
THROWM(err.ne.0,"CANNOT WRITE Root Attibutes")
  
Call jpss_meta_aggregate_writea(x%agg_att,lun,err)
THROWM(err.ne.0,"CANNOT WRITE Aggregate Attributes")

nchan = size( x%band )
Do ichan=1,nchan
  If ( x%band(ichan)%channel .LE. 0 ) Cycle

  Call jpss_meta_dataproducts_writea(x%datap_att(ichan),lun,err)
  THROWM(err.ne.0,"CANNOT WRITE DataProducts Attributes")

  Call jpss_meta_granule_writea(x%gran0_att(ichan),lun,err)
  THROWM(err.ne.0,"CANNOT WRITE Granule_0 Attributes")

  Call viirs_sdr_data_writea(x%band(ichan),lun,err)
  THROWM(err.ne.0,"CANNOT WRITE BAND Data")
End Do

Call viirs_sdr_geo_writea(x%geo,lun,err)
THROWM(err.ne.0,"CANNOT WRITE Geolocation")

write(lun,'("SECTION = EOF")',err=999)
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
err=0
CATCH
end subroutine



end module
