
        subroutine avh1bdtp(printunit,iscn)
        
!------------------------------------------------------------------------------
!  $Id: avh1bdtp.F 530 2017-06-27 11:07:32Z frna $
!  (C) NWP SAF
!------------------------------------------------------------------------------          
!-----------------------------------------------------------------------
!
!NAME
!   avh1bdtp.F
!
!PURPOSE
!   It prints the requested records of level 1B AVHRR
!   file for verification. 
!   Level 1B format of the AAPP Software package.
!   Print on the unit file printunit.
!
!   It is very close to avhddp.F (libf7nl1b)
!
!CODE OWNER
!   NWP SAF
!
!HISTORY (CHANGES) 
!(version      date            comment                  <author>)       
!  1.0         05/09         Creation               T.Labrot (MF)
!
!ARGUMENTS
!  printunit: unit file where AVHRR data are written
!  iscn: Scan line number
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!  gb_wb32 , gp_gb32 (libf7gp), xhqc , xdqc (libf7tp)
!
!COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!-----------------------------------------------------------------------------

!----DECLARATIONS-------------------------------------------------------------


      IMPLICIT NONE
                                                       
#include "avh1b.h"

      INTEGER printunit
      INTEGER pu
      INTEGER iscn      ! Scan line number (Input)
      
      character*12 scnlintime
      integer i,k
      
!----END OF DECLARATIONS------------------------------------------------------- 

!---- CODE -------------------------------------------------------------------- 
     
       pu=printunit
       
       write(pu,*) ' '
       write(pu,*) '==============================================='
       write(pu,*) '             AVHRR/HRPT 1B  DATA RECORD',iscn
       write(pu,*) '==============================================='

       write(pu,*) ' '

       write(pu,*) 'scan line number --------------: ',avh_scnlin
       write(pu,*) 'scan line year ----------------: ',avh_scnlinyr
       write(pu,*) 'scan line day of year ---------: ',avh_scnlindy
       call xhqc(avh_scnlintime,scnlintime) 
       write(pu,*) 'scan line UTC time of day -----: ',avh_scnlintime,
     &                             '  (',scnlintime,')'

       write(pu,*) 'satellite clock drift delta ---: ',avh_clockdrift
       write(pu,'('' scan line bit field -----------: '',$)')
!TL       call gp_wb16( avh_scnlinbit )
       call write16bit(pu,avh_scnlinbit )
       write(pu,'('' quality indicator bit field ---: '',$)') 
!TL       call gp_wb32( avh_qualind )
       call write32bit(pu,avh_qualind )
       write(pu,'('' scan line quality flags -------: '',$)') 
!TL       call gp_wb32( avh_scnlinqual )
       call write32bit(pu,avh_scnlinqual )
       do k = 1 , avh_mxirchn
       write(pu,'('' calibration quality flags -----: '',$)') 
!TL       call gp_wb16( avh_calqual(k) )
       call write16bit(pu,avh_calqual(k) )
       enddo      
       write(pu,*) 'count of bit errors in frame sync --: ',
     &           avh_cbiterr




       write(pu,*) ' '
       write(pu,*) ' operational calibration coefficients'
       write(pu,*) '====================================='
       write(pu,*) ' '
       do i=1,avh_mxvischn
        write(pu,'(''solar cal. slope 1 ['',i2.2,'']: '',$)')i
        write(pu,*) avh_calvis(1,1,i)
        write(pu,'(''solar cal. intercept 1['',i2.2,'']: '',$)')i
        write(pu,*) avh_calvis(2,1,i)
        write(pu,'(''solar cal. slope 2 ['',i2.2,'']: '',$)')i
        write(pu,*) avh_calvis(3,1,i)
        write(pu,'(''solar cal. intercept 2['',i2.2,'']: '',$)')i
        write(pu,*) avh_calvis(4,1,i)
        write(pu,'(''intersection ['',i2.2,'']: '',$)')i
        write(pu,*) avh_calvis(5,1,i)
       enddo
       write(pu,*) ' '
       write(pu,*) ' test calibration coefficients'
       write(pu,*) '=============================='
       write(pu,*) ' '
       do i=1,avh_mxvischn
        write(pu,'(''solar cal. slope 1 ['',i2.2,'']: '',$)')i
        write(pu,*) avh_calvis(1,2,i)
        write(pu,'(''solar cal. intercept 1['',i2.2,'']: '',$)')i
        write(pu,*) avh_calvis(2,2,i)
        write(pu,'(''solar cal. slope 2 ['',i2.2,'']: '',$)')i
        write(pu,*) avh_calvis(3,2,i)
        write(pu,'(''solar cal. intercept 2['',i2.2,'']: '',$)')i
        write(pu,*) avh_calvis(4,2,i)
        write(pu,'(''the zeroth one ['',i2.2,'']: '',$)')i
        write(pu,*) avh_calvis(5,2,i)
       enddo
       write(pu,*) ' '
       write(pu,*) ' prelaunch calibration coefficients'
       write(pu,*) '==================================='
       write(pu,*) ' '
       do i=1,avh_mxvischn
        write(pu,'(''solar cal. slope 1 ['',i2.2,'']: '',$)')i
        write(pu,*) avh_calvis(1,3,i)
        write(pu,'(''solar cal. intercept 1['',i2.2,'']: '',$)')i
        write(pu,*) avh_calvis(2,3,i)
        write(pu,'(''solar cal. slope 2 ['',i2.2,'']: '',$)')i
        write(pu,*) avh_calvis(3,3,i)
        write(pu,'(''solar cal. intercept 2['',i2.2,'']: '',$)')i
        write(pu,*) avh_calvis(4,3,i)
        write(pu,'(''the zeroth one ['',i2.2,'']: '',$)')i
        write(pu,*) avh_calvis(5,3,i)
       enddo
       write(pu,*) ' '
       write(pu,*) ' IR operational calibration coefficients'
       write(pu,*) '========================================'
       write(pu,*) ' '
       do i=1,avh_mxirchn
        write(pu,'(''IR sec. ord. coef. ch['',i2.2,'']: '',$)')i
        write(pu,*) avh_calir(1,1,i)
        write(pu,'(''IR first ord. ch['',i2.2,'']: '',$)')i
        write(pu,*) avh_calir(2,1,i)
        write(pu,'(''IR zeroth ord. ch. ['',i2.2,'']: '',$)')i
        write(pu,*) avh_calir(3,1,i)
       enddo
        

       write(pu,*) ' '
       write(pu,*) 'time associated angles : ',avh_attangtime
       write(pu,*) 'Roll angle in degrees  : ',avh_rollang
       write(pu,*) 'pitch angle in degrees : ',avh_pitchang
       write(pu,*) 'yaw  angle in degrees  : ',avh_yawang
!!       write(pu,*) 'Roll errors correction  : ',avh_rollcor
!!       write(pu,*) 'pitch errors correction : ',avh_pitchcor
!!       write(pu,*) 'yaw  errors correction  : ',avh_yawcor
       write(pu,'(''navigation status bit field : '',$)')
!TL       call gp_wb32( avh_navstat )
       call write32bit(pu,avh_navstat )
       write(pu,*) '10*spacecraft altitude (MSL) in km  : ',avh_scalti


       write(pu,*) ' '
       write(pu,*) 'scan angles in degrees'
       write(pu,*) '======================'
       write(pu,*) ' '
       do i=1,51
        write(pu,*) 'solar zen: ',avh_ang(1,i),' sat zen: ',
     &  avh_ang(2,i),' local azimuth: ',avh_ang(3,i)
       enddo
 

       write(pu,*) ' '
       write(pu,*) 'lat/lon in degrees'
       write(pu,*) '=================='
       write(pu,*) ' '
       do i=1,51
        write(pu,*) 'lat: ',avh_pos(1,i),' lon: ',avh_pos(2,i)
       enddo
 
       write(pu,*) ' '
       write(pu,*) 'AVHRR SENSOR DATA'
       write(pu,*) '================='
       write(pu,*) ' ' 

       write(pu,'(15i5)') avh_hrpt 
       return
       end
       
       
      subroutine write16bit(printunit,word16)
      
! BUT/PURPOSE: Output 16 bits from an integer*2 word 
!              Write into file unit  printunit

      implicit none 
      
      integer printunit
      integer*2 word16 
      
      integer*2 resu 
      integer n

      integer*2 gp_gb16
      
      do n=15,0,-1
          resu=gp_gb16(word16,n)
          write(printunit,'(i1,$)') resu
          if(MOD(n,4).eq.0)write(printunit,'('' '',$)')
      end do
      write(printunit,*)
      return
      end
      
      subroutine write32bit(printunit,word32)
               
! BUT/PURPOSE: Output on stdout bit format for an integer
!              Write into file unit  printunit 

      implicit none
      
      integer printunit
      integer*4 word32 ! word de 32 bits 
      
      integer*4 n,resu 
      
      integer*4 gp_gb32
      
      do n=31,0,-1
          resu=gp_gb32(word32,n)
          write(printunit,'(i1,$)') resu
          if(MOD(n,4).eq.0)write(printunit,'('' '',$)')
      end do
      write(printunit,*)
      return
      end
