      subroutine avh1bhdp(printunit)

!------------------------------------------------------------------------------
!  $Id: avh1bhdp.F 530 2017-06-27 11:07:32Z frna $
!  (C) NWP SAF
!------------------------------------------------------------------------------             
!------------------------------------------------------------------------------
!NAME
!   avh1bhdp.F
!
!PURPOSE
!   It prints the header of level 1b AVHRR file for verification. 
!   Level 1C format of the AAPP Software package.
!   Print on the unit file printunit.
!
!   It is very close to avhhdp.F (libf7nl1b) 
!
!CODE OWNER
!   NWP SAF
!
!HISTORY (CHANGES) 
!(version      date            comment                  <author>)       
!  1.0         May 09         Creation               T.Labrot (MF)
!
!ARGUMENTS
!  printunit: unit file where AVHRR data are written
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!  gb_wb32 , gp_gb32 (libf7gp), xhqc , xdqc (libf7tp)
!
!COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!------------------------------------------------------------------------------

!----DECLARATIONS--------------------------------------------------------------
        
      IMPLICIT NONE

#include "avh1b.h"

              
       INTEGER printunit
       INTEGER pu
       character*8 datatyp(12)
       data datatyp/'(  )','(LAC)','(GAC)','(HRPT)',
     &              '(TIP)','(HIRS)','(MSU)','(SSU)',
     &              '(DCS)','(SEM)','(AMSU-A)','(AMSU-B)'/
       character*16 tipsrc(5)
       data tipsrc/'(not applicable)','(GAC embedded)','(stored)',
     &             '(third CDA)','(HRPT embedded)'/
       character*9 pacsdatasrc(4)
       data pacsdatasrc/'(unused)','(Gilmore)',
     &                  '(Wallops)','(SOCC)'/
       character*8 startdatac,enddatac
       character*12 startdatatime,enddatatime,epotime
       integer*4 inststat14,pacsstatfield4,locbit4,racalind4
       integer   i,j

!----END OF DECLARATIONS-------------------------------------------------            
        
       pu=printunit 


       write(pu,*) ' '
       write(pu,*) '             AVHRR/HRPT 1B Data Set HEADER RECORD'
       write(pu,*) ' '
       write(pu,*) '             GENERAL INFORMATION'
       write(pu,*) ' '
       write(pu,*) 'site creation = ',avh_h_siteid
       write(pu,*) 'l1bversnb     = ',avh_h_l1bversnb 
       write(pu,*) 'l1bversyr     = ',avh_h_l1bversyr
       write(pu,*) 'l1bversdy     = ',avh_h_l1bversdy
       write(pu,*) 'record length = ',avh_h_reclg
       write(pu,*) 'block size    = ',avh_h_blksz
       write(pu,*) 'hdrcnt        = ',avh_h_hdrcnt
       write(pu,*) 'dataname      = ',avh_h_dataname
       write(pu,*) 'prblkid       = ',avh_h_prblkid
       write(pu,*) 'satid         = ',avh_h_satid
       write(pu,*) 'instid        = ',avh_h_instid 
       write(pu,*) 'datatyp       = ',avh_h_datatyp,'  ',
     &             datatyp(avh_h_datatyp+1)(1:8) 
       write(pu,*) 'tipsrc        = ',avh_h_tipsrc,'  ',
     &             tipsrc(avh_h_tipsrc+1)(1:16)
       call xdjc(avh_h_startdatajd,startdatac) 
       write(pu,*) 'startdatajd   = ',avh_h_startdatajd,
     &                             '  (',startdatac,')' 
       write(pu,*) 'startdatayr   = ',avh_h_startdatayr
       write(pu,*) 'startdatady   = ',avh_h_startdatady
       call xhqc(avh_h_startdatatime,startdatatime) 
       write(pu,*) 'startdatatime = ',avh_h_startdatatime,
     &                             '  (',startdatatime,')'
       call xdjc(avh_h_enddatajd,enddatac) 
       write(pu,*) 'enddatajd     = ',avh_h_enddatajd,
     &                             '  (',enddatac,')' 
       write(pu,*) 'enddatayr     = ',avh_h_enddatayr
       write(pu,*) 'enddatady     = ',avh_h_enddatady
       call xhqc(avh_h_enddatatime,enddatatime) 
       write(pu,*) 'enddatatime   = ',avh_h_enddatatime,
     &                             '  (',enddatatime,')'
       write(pu,*) 'cpidsyr       = ',avh_h_cpidsyr
       write(pu,*) 'cpidsdy       = ',avh_h_cpidsdy

       write(pu,*) ' '
       write(pu,*) '              DATA SET QUALITY INDICATORS'
       write(pu,*) ' '
       write(pu,1) avh_h_inststat1
1      format(' inststat1      = ',i5,$)
       inststat14  = avh_h_inststat1
       call writebin(pu,inststat14,16,31)  
       write(pu,*) 'statchrecnb    = ',avh_h_statchrecnb 
       write(pu,*) 'inststat2      = ',avh_h_inststat2
       write(pu,*) 'scnlin         = ',avh_h_scnlin 
       write(pu,*) 'callocscnlin   = ',avh_h_callocscnlin 
       write(pu,*) 'misscnlin      = ',avh_h_misscnlin 
       write(pu,*) 'datagaps       = ',avh_h_datagaps
       write(pu,*) 'okdatafr       = ',avh_h_okdatafr 
       write(pu,*) 'pacsparityerr  = ',avh_h_pacsparityerr 
       write(pu,*) 'auxsyncerrsum  = ',avh_h_auxsyncerrsum 
       write(pu,*) 'timeseqerr     = ',avh_h_timeseqerr 
       write(pu,*) 'timeseqerrcode = ',avh_h_timeseqerrcode 
       write(pu,*) 'socclockupind  = ',avh_h_socclockupind 
       write(pu,*) 'locerrind      = ',avh_h_locerrind 
       write(pu,*) 'locerrcode     = ',avh_h_locerrcode 
       write(pu,2) avh_h_pacsstatfield
2      format(' pacsstatfield  = ',i5,$)
       pacsstatfield4 = avh_h_pacsstatfield
       call writebin(pu,pacsstatfield4,16,31)  
       write(pu,*) 'pacsdatasrc    = ',avh_h_pacsdatasrc,'  ',
     &             pacsdatasrc(avh_h_pacsdatasrc+1)(1:9) 
       write(pu,*) ' '
       write(pu,*) '              CALIBRATION'
       write(pu,*) ' '
       write(pu,4) avh_h_racalind
4      format(' racalind     = ',i5,$)
       racalind4 = avh_h_racalind
       call writebin(pu,racalind4,16,31)  
       write(pu,*) 'solarcalyr   = ',avh_h_solarcalyr 
       write(pu,*) 'solarcaldy   = ',avh_h_solarcaldy
       write(pu,*) 'pcalalgind   = ',avh_h_pcalalgind 
       write(pu,*) 'pcalalgopt   = ',avh_h_pcalalgopt
       write(pu,*) 'scalalgind   = ',avh_h_scalalgind 
       write(pu,*) 'scalalgopt   = ',avh_h_scalalgopt
       do i=1,4
       write(pu,*) 'i,irttcoef   = ',i,(avh_h_irttcoef(j,i),j=1,6)
       enddo
       write(pu,*) ' '
       write(pu,*) ' '
       write(pu,*) '              ALBEDO and RADIANCE TO TEMPERATURE CONV
     &ERSION'
       write(pu,*) ' '
       write(pu,*) 'albcnv       = ',avh_h_albcnv
       write(pu,*) 'radtempcnv   = ',avh_h_radtempcnv 
       write(pu,*) ' '
       write(pu,*) '              NAVIGATION'
       write(pu,*) ' '
       write(pu,*) 'modelid      = ',avh_h_modelid
       write(pu,*) 'nadloctol    = ',avh_h_nadloctol 
       write(pu,3) avh_h_locbit
3      format(' locbit       = ',i5,$)
       locbit4 = avh_h_locbit
       call writebin(pu,locbit4,16,31)  
        write(pu,*) 'yawerr       = ',avh_h_yawerr 
       write(pu,*) 'rollerr      = ',avh_h_rollerr 
       write(pu,*) 'pitcherr     = ',avh_h_pitcherr
       write(pu,*) 'epoyr        = ',avh_h_epoyr
       write(pu,*) 'epody        = ',avh_h_epody 
       call xhqc(avh_h_epotime,epotime) 
       write(pu,*) 'epotime      = ',avh_h_epotime,
     &                            '  (',epotime,')'
       write(pu,*) 'smaxis       = ',avh_h_smaxis
       write(pu,*) 'eccen        = ',avh_h_eccen
       write(pu,*) 'incli        = ',avh_h_incli 
       write(pu,*) 'argper       = ',avh_h_argper 
       write(pu,*) 'rascnod      = ',avh_h_rascnod 
       write(pu,*) 'manom        = ',avh_h_manom 
       write(pu,*) 'xpos         = ',avh_h_xpos 
       write(pu,*) 'ypos         = ',avh_h_ypos
       write(pu,*) 'zpos         = ',avh_h_zpos 
       write(pu,*) 'xvel         = ',avh_h_xvel 
       write(pu,*) 'yvel         = ',avh_h_yvel 
       write(pu,*) 'zvel         = ',avh_h_zvel 
       write(pu,*) 'earthsun     = ',avh_h_earthsun 
       write(pu,*) ' '
       write(pu,*) '              ANALOG TELEMETRY CONVERSION'
       write(pu,*) ' '
       write(pu,*) '      conversion coefficients'
       write(pu,*) 'pchtemp      = ',avh_h_pchtemp
       write(pu,*) 'pchtempext   = ',avh_h_pchtempext 
       write(pu,*) 'pchpow       = ',avh_h_pchpow 
       write(pu,*) 'rdtemp       = ',avh_h_rdtemp
       write(pu,*) 'bbtemp1      = ',avh_h_bbtemp1 
       write(pu,*) 'bbtemp2      = ',avh_h_bbtemp2
       write(pu,*) 'bbtemp3      = ',avh_h_bbtemp3 
       write(pu,*) 'bbtemp4      = ',avh_h_bbtemp4
       write(pu,*) 'eleccur      = ',avh_h_eleccur 
       write(pu,*) 'motorcur     = ',avh_h_motorcur 
       write(pu,*) 'earthpos     = ',avh_h_earthpos
       write(pu,*) 'electemp     = ',avh_h_electemp 
       write(pu,*) 'chtemp       = ',avh_h_chtemp 
       write(pu,*) 'bptemp       = ',avh_h_bptemp 
       write(pu,*) 'mhtemp       = ',avh_h_mhtemp 
       write(pu,*) 'adcontemp    = ',avh_h_adcontemp
       write(pu,*) 'd4bvolt      = ',avh_h_d4bvolt
       write(pu,*) 'd5bvolt      = ',avh_h_d5bvolt  
       write(pu,*) 'bbtempchn3B  = ',avh_h_bbtempchn3B
       write(pu,*) 'bbtempchn4   = ',avh_h_bbtempchn4 
       write(pu,*) 'bbtempchn5   = ',avh_h_bbtempchn5 
       write(pu,*) 'refvolt      = ',avh_h_refvolt 
       write(pu,*) ' '

        return
        end
        
        subroutine writebin(printunit,word,bitdeb,bitfin)

!BUT/PURPOSE 
!     print word from bitdeb to bitfin in binary

      implicit none                                                     

      integer printunit
      integer*4 word
      integer*4 bitdeb
      integer*4 bitfin

      integer*4 bitnum
      logical b

      logical gp_btn

       write(printunit,1)
  1    format('  (',$)

       do bitnum=31-bitdeb,31-bitfin,-1

         b=gp_btn(word,bitnum)
        
         if (b) then
           write(printunit,2)
  2        format('1',$)
         else
           write(printunit,3)
  3        format('0',$)
         endif
       end do

       write(printunit,4)
  4    format(')')

       return
       end
