      subroutine avh1cdtp(printunit,iscn,pix1,pix2)

!------------------------------------------------------------------------------
!  $Id: avh1cdtp.F 530 2017-06-27 11:07:32Z frna $
!  (C) NWP SAF
!------------------------------------------------------------------------------             
!------------------------------------------------------------------------------
!NAME
!   avh1cdtp.F
!
!PURPOSE
!   It prints the requested records of level 1c AVHRR
!   file for verification. 
!   Level 1C format of the AAPP Software package.
!   Print on the unit file printunit. 
!
!CODE OWNER
!   NWP SAF
!
!HISTORY (CHANGES) 
!(version      date            comment                  <author>)       
!  1.0         05/09         Creation               T.Labrot (MF)
!
!ARGUMENTS
!  printunit: unit file where AVHRR data are written
!  iscn: Scan line number
!  pix1: First pixel for which data will be write
!  pix2: Last pixel for which data will be write
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!  gb_wb32 , gp_gb32 (libf7gp), xhqc , xdqc (libf7tp)
!
!COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!------------------------------------------------------------------------------

!----DECLARATIONS--------------------------------------------------------------
        
      IMPLICIT NONE
      
#include "avh1c.h"  
      
      INTEGER j
      INTEGER printunit
      INTEGER pu
      INTEGER iscn      ! Scan line number (Input)
      INTEGER pix1      ! First pixel for which data will be write
      INTEGER pix2      ! Last pixel for which data will be write
      INTEGER ich,ifov  ! Channel counter, Field of View counter
      INTEGER bitvalue  ! Value of one bit in a word 
      INTEGER mask_10_6 
      INTEGER mask_4_3
      INTEGER mask_2_1    
      INTEGER gp_gb32   ! Get bit from an integer
      INTEGER gp_gb16   ! Get bit from an integer
      INTEGER dq(2)     ! Date in a table
      CHARACTER*8 dc    ! Date in a chain of characters
      CHARACTER*12 hc   ! Hour in a chain of characters
        
!----END OF DECLARATIONS-------------------------------------------------------             

      bitvalue=0 
      
      pu=printunit
      
      WRITE(pu,*) ' '
      WRITE(pu,*) '========================================= '
      WRITE(pu,*) '     LEVEL 1C1D AVHRR  SCAN LINE ',iscn
      WRITE(pu,*) '========================================= '
      WRITE(pu,*) ' '
      WRITE(pu,*) ' ' 
      WRITE(pu,*) '          GENERAL INFORMATION'
      WRITE(pu,*) '          -------------------'
      WRITE(pu,*) ' ' 
      WRITE(pu,'(" Scan line number",15X,"=",I8)') avh1c_scnlin
      WRITE(pu,'(" Scan line year  ",15X,"=",I8)') avh1c_scnlinyr
      WRITE(pu,'(" Scan line day of year",10X,"=",I8)')avh1c_scnlindy
      
      dq(1)=avh1c_scnlinyr
      dq(2)=avh1c_scnlindy
      CALL xdqc(dq,dc)
      WRITE(pu,'(14X,"(date in a chain) = ",A8)') dc
      
      CALL xhqc(avh1c_scnlintime,hc)
      WRITE(pu,'(" Scan line UTC time of day (ms) = ",I8,"  ",A12)')
     & avh1c_scnlintime,hc

      WRITE(pu,'(" Satellite clock drift delta (ms)",2X,"=",I8)')
     & avh1c_clockdrift
      
      WRITE(pu,*) ' '
      WRITE(pu,*) '              SCAN LINE BIT FIELD'
      WRITE(pu,*) '              -------------------'
      WRITE(pu,*) ' '
!     Output on bit format for an integer   
      CALL gp_wb32(avh1c_scnlinbit)      
      CALL write32bit(pu,avh1c_scnlinbit)
      
      WRITE(pu,*)
      bitvalue=gp_gb32(avh1c_scnlinbit,15)
      WRITE(pu,
     &'(I2,2X,"Bit 15: 0=ascending data , 1=descending data",I1)') 
     &bitvalue 
      
      bitvalue=gp_gb32(avh1c_scnlinbit,14)
      WRITE(pu,
     &'(I2,2X,"Bit 14: 1=scan time corrected for clock drift",I1)') 
     &bitvalue
      
      bitvalue=gp_gb32(avh1c_scnlinbit,13)
      WRITE(pu,
     &'(I2,2X,
     &"Bit 13: 1=earth location corrected for TIP attitude",I1)')
     &bitvalue
      
      bitvalue=gp_gb32(avh1c_scnlinbit,0)
      WRITE(pu,
     &'(I2,2X,"Bit  0: channel 3 select (0=3A ; 1=3B) ",I1)') 
     &bitvalue
      


      

      WRITE(pu,*) ' '
      WRITE(pu,*) '          QUALITY INDICATOR BIT FIELD'
      WRITE(pu,*) '          ---------------------------'
      WRITE(pu,*) ' '
      WRITE(pu,'(" Quality indicator bit field = ",$)')
!     Output on bit format for an integer   
      CALL gp_wb32(avh1c_qualind)
      CALL write32bit(pu,avh1c_qualind)
      
      WRITE(pu,*)
 
      bitvalue=gp_gb32(avh1c_qualind,31)
      WRITE(pu,'(I2,2X,"Bit 31: (1 = Do not use data for product ",
     & "generation.)")')
     & bitvalue 
     
      bitvalue=gp_gb32(avh1c_qualind,30)
      WRITE(pu,'(I2,2X,"Bit 30: (1 = Time sequence error detected in ",
     &"this frame.)")')
     & bitvalue 
     
      bitvalue=gp_gb32(avh1c_qualind,29)
      WRITE(pu,'(I2,2X,"Bit 29: (1 = Data gap precedes this frame.)")')
     & bitvalue   
      bitvalue=gp_gb32(avh1c_qualind,28)
      WRITE(pu,'(I2,2X,
     &"Bit 28: (1 = Insufficient data for calibration.)")')
     & bitvalue 
      bitvalue=gp_gb32(avh1c_qualind,27)
      WRITE(pu,'(I2,2X,
     &"Bit 27: (1 = Earth location data not available.)")')
     & bitvalue 
      bitvalue=gp_gb32(avh1c_qualind,26)
      WRITE(pu,'(I2,2X,
     &"Bit 26: (1 = Sync lock dropped during this frame.)")')
     & bitvalue 
      bitvalue=gp_gb32(avh1c_qualind,25)
      WRITE(pu,'(I2,2X,
     &"Bit 25: (1 = Frame sync word error greater than zero.)")')
     & bitvalue 
      bitvalue=gp_gb32(avh1c_qualind,24)
      WRITE(pu,'(I2,2X,
     &"Bit 24: (1 = Frame sync previously dropping lock.)")')
     & bitvalue
      bitvalue=gp_gb32(avh1c_qualind,23)
      WRITE(pu,'(I2,2X,
     &"Bit 23: (1 = Flywheeling detected during this frame.)")')
     & bitvalue
      bitvalue=gp_gb32(avh1c_qualind,22)
      WRITE(pu,'(I2,2X,
     &"Bit 22: (1 = bit slippage detected during this frame.)")')
     & bitvalue
      bitvalue=gp_gb32(avh1c_qualind,9)
      WRITE(pu,'(I2,2X,
     &"Bit  9: tip parity in first minor frame.")')
     & bitvalue
      bitvalue=gp_gb32(avh1c_qualind,8)
      WRITE(pu,'(I2,2X,"Bit  8: tip parity in second minor frame.")')
     & bitvalue
      bitvalue=gp_gb32(avh1c_qualind,7)
      WRITE(pu,'(I2,2X,"Bit  7: tip parity in third minor frame.")')
     & bitvalue
      bitvalue=gp_gb32(avh1c_qualind,6)
      WRITE(pu,'(I2,2X,"Bit  6: tip parity in fourth minor frame.")')
     & bitvalue
      bitvalue=gp_gb32(avh1c_qualind,5)
      WRITE(pu,'(I2,2X,"Bit  5: tip parity in fifth minor frame.")')
     & bitvalue
      bitvalue=gp_gb32(avh1c_qualind,4)
      WRITE(pu,'(I2,2X,
     &"Bit  4: (1 = reflected sunlight detected channel 3B.)")')
     & bitvalue
      bitvalue=gp_gb32(avh1c_qualind,3)
      WRITE(pu,'(I2,2X,
     &"Bit  3: (1 = reflected sunlight detected channel 4.)")')
     & bitvalue
      bitvalue=gp_gb32(avh1c_qualind,2)
      WRITE(pu,'(I2,2X,
     &"Bit  2: (1 = reflected sunlight detected channel 5.)")')
     & bitvalue
      bitvalue=gp_gb32(avh1c_qualind,1)
      WRITE(pu,'(I2,2X,"Bit  1: (1 = resync occured on this frame.)")')
     & bitvalue
      bitvalue=gp_gb32(avh1c_qualind,0)
      WRITE(pu,'(I2,2X,
     &"Bit  0: (1 = pseudo noise occured on this frame.)")')
     & bitvalue
      
      
      WRITE(pu,*) ' '
      WRITE(pu,*) '            SCAN LINE QUALITY FLAG'
      WRITE(pu,*) '            ----------------------'
      WRITE(pu,*) ' '
      WRITE(pu,'(" Scan line quality flag = ",$)')
!     Output on bit format for an integer   
      CALL gp_wb32(avh1c_scnlinqual)
      CALL write32bit(pu,avh1c_scnlinqual)
      
      WRITE(pu,*) ' '
      WRITE(pu,*) '               Time Problem Code'
      WRITE(pu,*) '   All bits off implies the scan time is as ',
     &'expected'
      WRITE(pu,*) ' '
      
      bitvalue=gp_gb32(avh1c_scnlinqual,23)
      WRITE(pu,'(I2,2X,"Bit 23: Time field is bad but can probably be ",
     &"inferred from the previous good time.")')
     &bitvalue
      bitvalue=gp_gb32(avh1c_scnlinqual,22)
      WRITE(pu,'(I2,2X,"Bit 22: Time field is bad and can t be ",
     &"inferred from the previous good time.")')
     &bitvalue
      bitvalue=gp_gb32(avh1c_scnlinqual,21)
      WRITE(pu,'(I2,2X,"Bit 21: This record starts a sequence that is ",
     &"inconsistent with previous time.")')
     &bitvalue
      WRITE(pu,'(12X,"There is a time discontinuity.")')
      WRITE(pu,'(12X,"This may or may not be associated with ",
     &"a spacecraft clock update.")')
      WRITE(pu,'(12X,"See bit 26 above.")')
      bitvalue=gp_gb32(avh1c_scnlinqual,20)
      WRITE(pu,'(I2,2X,"Bit 20: Start of a sequence that apparently ",
     &"repeats scan times that have been ")')
     &bitvalue 
      WRITE(pu,'(12X,"previously accepted.")')
      WRITE(pu,*) ' '  
      WRITE(pu,*) '               Calibration Problem Code'
      WRITE(pu,*) '     All bits set to 0 indicates normal calibration'
      WRITE(pu,*)
      bitvalue=gp_gb32(avh1c_scnlinqual,15)
      WRITE(pu,'(I2,2X,"Bit 15: Scan line was not calibrated because ",
     &"of bad time.")')
     &bitvalue
      bitvalue=gp_gb32(avh1c_scnlinqual,14)
      WRITE(pu,'(I2,2X,"Bit 14: Scan line was calibrated using ",
     &"fewer than the preferred number of")')
     &bitvalue 
      WRITE(pu,'(12X,"scan lines because of proximity to start ",
     &"or end")')
      WRITE(pu,'(12X,"of data set or to a data gap.")')
      bitvalue=gp_gb32(avh1c_scnlinqual,13)
      WRITE(pu,'(I2,2X,"Bit 13: Scan line was not calibrated because ",
     &"of bad or insufficient PRT data.")')
     &bitvalue
      bitvalue=gp_gb32(avh1c_scnlinqual,12)
      WRITE(pu,'(I2,2X,"Bit 12: Scan line was calibrated but with ",
     &" marginal PRT data.")')
     &bitvalue
      bitvalue=gp_gb32(avh1c_scnlinqual,11)
      WRITE(pu,'(I2,2X,"Bit 11: Some uncalibrated channels of this ",
     &"scan. See channel indicators.")')
     &bitvalue
      
      WRITE(pu,*)' '
      WRITE(pu,*) '             Earth Location  Problem Code'
      WRITE(pu,*) 
     &'      All bits set to 0 indicates normal Earth location'
      WRITE(pu,*)' ' 
      bitvalue=gp_gb32(avh1c_scnlinqual,7)
      WRITE(pu,'(I2,2X,"Bit 7: Not Earth located because of bad ",
     &"time.")')
     &bitvalue
      bitvalue=gp_gb32(avh1c_scnlinqual,6)
      WRITE(pu,'(I2,2X,"Bit 6: Earth location questionable because of ",
     &"questionable time code.")')
     &bitvalue
      WRITE(pu,'(10X,"See time problem flags above.")')
      bitvalue=gp_gb32(avh1c_scnlinqual,5)
      WRITE(pu,'(I2,2X,"Bit 5: Earth location questionable. Only ",
     &"marginal agreement with reasonableness check.")')
     &bitvalue
      bitvalue=gp_gb32(avh1c_scnlinqual,4)
      WRITE(pu,'(I2,2X,"Bit 4: Earth location questionable. Fails ",
     &"reasonableness check.")')
     &bitvalue

      DO ich=1,avh1c_mxirchn
        WRITE(pu,*) ' '
        WRITE(pu,*) '          QUALITY FLAG FOR CHANNEL ',ich
        WRITE(pu,*) '          ---------------------------'
        WRITE(pu,*) ' '
        WRITE(pu,*) '     All bits off implies a good calibration'
        WRITE(pu,'(" Quality flag = ",$)')
!       Output on bit format for an integer   
        CALL gp_wb32(avh1c_calqual(ich))
        CALL write32bit(pu,avh1c_calqual(ich))
        
        WRITE(pu,*) ' '
        bitvalue=gp_gb32(avh1c_calqual(ich),7)
        WRITE(pu,'(I2,2X,"Bit 7: This channel is not calibrated.")')
     &  bitvalue
        bitvalue=gp_gb32(avh1c_calqual(ich),6)
        WRITE(pu,'(I2,2X,
     &  "Bit 6: This channel is calibrated but questionnable.")')
     &  bitvalue
        bitvalue=gp_gb32(avh1c_calqual(ich),5)
        WRITE(pu,'(I2,2X,"Bit 5: No good blackbody counts for ",
     &  "scan line.")')
     &  bitvalue
        bitvalue=gp_gb32(avh1c_calqual(ich),4)
        WRITE(pu,'(I2,2X,"Bit 4: No good space view counts for "
     &  "scan line.")')
     &  bitvalue
        bitvalue=gp_gb32(avh1c_calqual(ich),3)
        WRITE(pu,'(I2,2X,"Bit 3: Insufficient PRT data.")')
     &  bitvalue
        bitvalue=gp_gb32(avh1c_calqual(ich),2)
        WRITE(pu,'(I2,2X,"Bit 2: Some bad blackbody view counts for ",
     &  "this line.")')
     &  bitvalue
        bitvalue=gp_gb32(avh1c_calqual(ich),1)
        WRITE(pu,'(I2,2X,"Bit 1: Some bad space view counts for this ",
     &  " line.")')
     &  bitvalue
        bitvalue=gp_gb32(avh1c_calqual(ich),0)
        WRITE(pu,'(I2,2X, "Bit 0: Some bad PRT data.")')
     &  bitvalue
      ENDDO
      
      WRITE(pu,*) ' '
      WRITE(pu,*) '                NAVIGATION'
      WRITE(pu,*) '                ----------'
      WRITE(pu,*) ' '
      
      WRITE(pu,'(" 10*Spacecraft altitude in km",15X,"=",I8)')
     & avh1c_scalti
        
      WRITE(pu,*) ' '
      WRITE(pu,*) '                 SCAN ANGLES'
      WRITE(pu,*) '                 -----------'
      WRITE(pu,*) '    First = 10^2 x (Solar zenith angle)'
      WRITE(pu,*) '    Second= 10^2 x (Satellite zenith angle)'
      WRITE(pu,*) '    Third = 10^2 x (Relative azimuth angle)'
      WRITE(pu,*) ' '
          
      DO j=1,51
       
         ifov=25+40*(j-1)
         
         WRITE(pu,'(" IFOV:",I4,2X,":",I7,"  ",I7,"  ",I7,"  ",I7)')
     &  ifov,avh1c_ang(1,j),avh1c_ang(2,j),avh1c_ang(3,j)
       
     
      ENDDO 
      
      
           
      WRITE(pu,*) ' '
      WRITE(pu,*) '            LATTITUDE-LONGITUDE (10^4 degrees)'
      WRITE(pu,*) '            ----------------------------------'
      WRITE(pu,*) ' '
      
      DO j=1,51
       
        ifov=25+40*(j-1)
      
        WRITE(pu,'(" IFOV:",I4,2X,"  LAT:",I8,"  LON:",I8)') 
     &  ifov,avh1c_pos(1,j),avh1c_pos(2,j)
     
      ENDDO
       

   
  
      
      DO ich=1,avh1c_mxhrpchn
        WRITE(pu,*) ' '
        WRITE(pu,*) '    CHANNEL ',ich,
     &   ': AVHRR SENSOR DATA (*10^2)'
        WRITE(pu,*) '    -----------------------------------------'
        WRITE(pu,*) ' '
        
        DO ifov=1,avh1c_mxpix
         IF (ifov.ge.pix1 .and. ifov.le.pix2 ) THEN
         
          WRITE(pu,'(" IFOV: ",I2,2X,"  Albedo/Temp: ",I8)') 
     &               ifov,avh1c_btemps(ich,ifov) 
         ENDIF
        ENDDO   !DO ifov=1,avh1c_mxpix
      ENDDO     !DO ich=1,avh1c_mxhrpchn
      
      WRITE(pu,*)
      WRITE(pu,'(" Cloud mask flag",15X,"=",I2)') avh1c_maskind
      WRITE(pu,'("      (=1 cloud mask done) ")') 
      
      DO ifov=1,avh1c_mxpix
      
       IF (ifov.ge.pix1 .and. ifov.le.pix2 ) THEN
       

        WRITE(pu,*) ' '
        WRITE(pu,*) '     AVHRR CLOUD MASK: IFOV',ifov
        WRITE(pu,*) '   ------------------------------'
        WRITE(pu,*) ' '
      
!       Output on bit format for an integer   
        CALL gp_wb16(avh1c_mask(ifov))      
        CALL write16bit(pu,avh1c_mask(ifov))

        WRITE(pu,*)
        WRITE(pu,*) '    Bit 15: 0=ascending data , 1=descending data' 
        bitvalue=gp_gb16(avh1c_mask(ifov),15)
        WRITE(pu,'(5X,"Bit 15:",I2)') bitvalue 
      
        WRITE(pu,*) '    Bit 13: 0= land or coast  1=sea' 
        bitvalue=gp_gb16(avh1c_mask(ifov),13)
        WRITE(pu,'(5X,"Bit 13:",I2)') bitvalue
      
        WRITE(pu,*) '    Bit 12: 0= clear   1=cloudy'
        bitvalue=gp_gb16(avh1c_mask(ifov),12)
        WRITE(pu,'(5X,"Bit 12:",I2)') bitvalue
        
        WRITE(pu,*) 
     &  '    Bit 11: 1= blackbody (only filled if bit12 =1)'
        bitvalue=gp_gb16(avh1c_mask(ifov),11)
        WRITE(pu,'(5X,"Bit 11:",I2)') bitvalue
        
        WRITE(pu,*)
        WRITE(pu,*) 
     &  '    Bits 10-6:  0= non-processed: ',
     &  ' containing no data or corrupted data'
        WRITE(pu,*)
     &  '             :  1= cloud free land'
        WRITE(pu,*)
     &  '                    no contamination by snow/ice ',
     &  ' covered surface,'
        WRITE(pu,*)
     &  '                    no contamination by clouds, '
        WRITE(pu,*)
     &  '                    but contamination by thin dust/volcanic ',
     &  ' clouds not checked.'       
        WRITE(pu,*)
     &  '             :  2= cloud free sea'
        WRITE(pu,*)
     &  '                    no contamination by snow/ice ',
     &  ' covered surface,'
        WRITE(pu,*)
     &  '                    no contamination by clouds, '
        WRITE(pu,*)
     &  '                    but contamination by thin dust/volcanic ',
     &  ' clouds not checked.' 
        WRITE(pu,*)
     &  '             :  3= land contaminated by snow'
        WRITE(pu,*)
     &  '             :  4= sea contaminated by snow/ice'
        WRITE(pu,*)
     &  '             :  5= very low and cumuliform clouds'
        WRITE(pu,*)
     &  '             :  6= very low and stratiform clouds'
        WRITE(pu,*)
     &  '             :  7= low and cumuliform clouds'
        WRITE(pu,*)
     &  '             :  8= low and stratiform clouds'
        WRITE(pu,*)
     &  '             :  9= medium and cumuliform clouds'
        WRITE(pu,*)
     &  '             : 10= medium and stratiform clouds'
        WRITE(pu,*)
     &  '             : 11= high opaque and cumuliform clouds'
        WRITE(pu,*)
     &  '             : 12= high opaque and stratiform clouds'
        WRITE(pu,*)
     &  '             : 13= very high opaque and cumuliform clouds'
        WRITE(pu,*)
     &  '             : 14= very high opaque and stratiform clouds'
        WRITE(pu,*)
     &  '             : 15= high semitransparent thin clouds'
        WRITE(pu,*)
     &  '             : 16= high semitransparent meanly thick clouds'
        WRITE(pu,*)
     &  '             : 17= high semitransparent thick clouds'
        WRITE(pu,*)
     &  '             : 18= high semitransparent above low or ',
     &  'medium clouds'
        WRITE(pu,*)
     &  '             : 19= fractional clouds (sub-pixel water ',
     &  'clouds)'
        WRITE(pu,*)
     &  '             : 20= undefined (undefined by CMa)'
     
        mask_10_6=IBITS(avh1c_mask(ifov),6,5)
        WRITE(pu,*) '    Value bits10-6 = ',mask_10_6
        
        WRITE(pu,*)
        WRITE(pu,*) '    Bit 5: 0= Ts background : climatology used'
        WRITE(pu,*) '           1= Ts background : forecast used'
        bitvalue=gp_gb16(avh1c_mask(ifov),5)
        WRITE(pu,'(5X,"Bit 5:",I2)') bitvalue
        
        WRITE(pu,*)
        WRITE(pu,*)  '    Bits 4-3:   0= WV content : AMSU used'
        WRITE(pu,*)  '                1= WV content : forecast used'
        WRITE(pu,*)  '                2= WV content : climatology used'
        
        mask_4_3=IBITS(avh1c_mask(ifov),3,2)
        WRITE(pu,*) '    Value bits4-3 = ',mask_4_3
        
        WRITE(pu,*)
        WRITE(pu,*)  '    Bits 2-1:   0= same CMa'
        WRITE(pu,*)  '                1= bad data'
        WRITE(pu,*)  '                2= different CMa'
        WRITE(pu,*)  '                3= coast'
        mask_2_1=IBITS(avh1c_mask(ifov),1,2)
        WRITE(pu,*) '    Value bits2-1 = ',mask_2_1
                
        WRITE(pu,*)
        WRITE(pu,*) '    Bit 0: for marine cloudy  =1 , clear=0' 
        bitvalue=gp_gb16(avh1c_mask(ifov),0)
        WRITE(pu,'(5X,"Bit 0:",I2)') bitvalue
        
        WRITE(pu,*)
        WRITE(pu,'(4X," Retrieved temperature = ",I6)') 
     & avh1c_tsurf(ifov)
        
       ENDIF 
      
      ENDDO     
 
      
      
      RETURN
      END
