      subroutine avh1chdp(printunit)

!------------------------------------------------------------------------------
!  $Id: avh1chdp.F 530 2017-06-27 11:07:32Z frna $
!  (C) NWP SAF
!------------------------------------------------------------------------------             
!------------------------------------------------------------------------------
!NAME
!   avh1chdp.F
!
!PURPOSE
!   It prints the header of level 1c AVHRR file for verification. 
!   Level 1C format of the AAPP Software package.
!   Print on the unit file printunit. 
!
!CODE OWNER
!   NWP SAF
!
!HISTORY (CHANGES) 
!(version      date            comment                  <author>)       
!  1.0         May 09         Creation               T.Labrot (MF)
!
!ARGUMENTS
!  printunit: unit file where AVHRR data are written
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!  gb_wb32 , gp_gb32 (libf7gp), xhqc , xdqc (libf7tp)
!
!COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!------------------------------------------------------------------------------

!----DECLARATIONS--------------------------------------------------------------
        
      IMPLICIT NONE

!Parameters for AVHRR   
#include "avh1c.h"      
       
      INTEGER printunit
      INTEGER pu
      INTEGER ich       ! Channel counter
      INTEGER dq(2)     ! Date in a table
      CHARACTER*8 dc    ! Date in a chain of characters
      CHARACTER*12 hc   ! Hour in a chain of characters
       
!----END OF DECLARATIONS-------------------------------------------------            
        
      pu=printunit 
       
      WRITE(pu,*) ' '
      WRITE(pu,*) '======================================= '
      WRITE(pu,*) '      AVHRR LEVEL 1C1D HEADER RECORD    '
      WRITE(pu,*) '======================================= '
      WRITE(pu,*) ' '
      WRITE(pu,*) ' ' 
      WRITE(pu,*) '          GENERAL INFORMATION'
      WRITE(pu,*) '          -------------------'
      WRITE(pu,*) ' '
      WRITE(pu,*) 'Dataset creation site ID            =',avh1c_h_site
      WRITE(pu,*) 'Creation site for original 1B data  =',avh1c_h_1bsite
      WRITE(pu,*) 'Level 1c format version number      =',avh1c_h_versnb
      WRITE(pu,*) 'Level 1c format version year        =',avh1c_h_versyr
      WRITE(pu,*) 'level 1c format version day of year =',avh1c_h_versdy
      WRITE(pu,*) 'Count of header records in data set =',avh1c_h_hdrcnt
      WRITE(pu,*) 'Satellite id (e.g. 14 for NOAA-14)  =',avh1c_h_satid
      WRITE(pu,*) 'Instrument  identification          =',
     &            avh1c_h_instrument 
      WRITE(pu,*) '(1=AVHRR)'
      WRITE(pu,*) 'Nominal satellite altitude, km*10   =',avh1c_h_satht
      WRITE(pu,*) 'Nominal orbit period (seconds)      =',avh1c_h_period
      WRITE(pu,*) 'Orbit number (at start of file)     =',
     &            avh1c_h_startorbit
      WRITE(pu,*) 'Start of data set year              =',
     &            avh1c_h_startdatayr
      WRITE(pu,*) 'Start of data set day of the year   =',
     &            avh1c_h_startdatady
     
      dq(1)=avh1c_h_startdatayr
      dq(2)=avh1c_h_startdatady
      CALL xdqc(dq,dc)
      WRITE(pu,*) '                (date in a chain)   = ',dc
    
      CALL xhqc(avh1c_h_startdatatime,hc)
      WRITE(pu,*) 'Start of data set UTC time (in ms)  =',
     &            avh1c_h_startdatatime
      WRITE(pu,*) '                      (in a chain)  =',hc
      WRITE(pu,*) 'Orbit number (at end of file)       =',
     &            avh1c_h_endorbit
      WRITE(pu,*) 'End of data set year                =',
     &            avh1c_h_enddatayr
      WRITE(pu,*) 'End of data set day of the year     =',
     &            avh1c_h_enddatady
     
      dq(1)=avh1c_h_enddatayr
      dq(2)=avh1c_h_enddatady
      CALL xdqc(dq,dc)
      WRITE(pu,*) '                 (date in a chain)  = ',dc
      
      
      CALL xhqc(avh1c_h_enddatatime,hc)
      WRITE(pu,*) 'End of data set UTC time (in ms)    =',
     &            avh1c_h_enddatatime
      WRITE(pu,*) '                    (in a chain)    =',hc
      WRITE(pu,*) 'Count of scan lines in this data set=',
     &            avh1c_h_scnlin
      WRITE(pu,*) 'Count of missing scan lines         =',
     &            avh1c_h_misscnlin
      WRITE(pu,*) 
     &   'Count  of calibrated, earth located scan lines ',
     &   'in this data set =',
     &            avh1c_h_callocscnlin
      WRITE(pu,*) 'Count of data gaps in this data set  =',
     &            avh1c_h_datagaps
      WRITE(pu,*) 'Data type code (1:LAC; 2:GAC; 3:HRPT)=',
     &            avh1c_h_datatyp
      WRITE(pu,*) 'Input file name (allow pfs name)     =',
     &            avh1c_h_infilename     
      WRITE(pu,*) 'WMO satellite id (3 for METOP-1)     =',
     &            avh1c_h_wmosatid
     
      WRITE(pu,*) ' '
      WRITE(pu,*) '            CALIBRATION INFORMATION'
      WRITE(pu,*) '            -----------------------'
      WRITE(pu,*) ' '
      WRITE(pu,*) 'Year of Last CPIDS Update        =',avh1c_h_cpidsyr
      WRITE(pu,*) 'Day of Year of Last CPIDS Update =',avh1c_h_cpidsdy
      
      WRITE(pu,*) ' '
      WRITE(pu,*) '             NAVIGATION INFORMATION'
      WRITE(pu,*) '             ----------------------'
      WRITE(pu,*) ' '
      WRITE(pu,*) '10^3 x (constant roll attitude error in degrees)  =',
     &            avh1c_h_rollerr
      WRITE(pu,*) '10^3 x (constant pitch attitude error in degrees) =',
     &            avh1c_h_pitcherr
      WRITE(pu,*) '10^3 x (constant yaw attitude error in degrees)   =',
     &            avh1c_h_yawerr
     
      WRITE(pu,*) ' '
      WRITE(pu,*) '     TEMPERATURE-RADIANCE CONVERSION COEFFICIENTS'
      WRITE(pu,*) '     --------------------------------------------'
      WRITE(pu,*) ' '
      DO ich=1,avh1c_mxvischn
        WRITE(pu,*) 'Channel : ',ich 
        WRITE(pu,*) 
     &  '  10^1 x (albedo-radiance nochannel solar filtered ', 
     &  'irradiance in wavelength) :',
     &      avh1c_h_albcnv(1,ich)
        WRITE(pu,*) 
     &  '  10^3 x (albedo-radiance nochannel equivalent filter width ', 
     &  'in wavelength) :', avh1c_h_albcnv(2,ich)
      ENDDO
      
      WRITE(pu,*) ' '
      WRITE(pu,*) '     TEMPERATURE-RADIANCE CONVERSION COEFFICIENTS'
      WRITE(pu,*) '     --------------------------------------------'
      WRITE(pu,*) ' '
      DO ich=1,avh1c_mxirchn
        WRITE(pu,*) 'Channel : ',ich 
        WRITE(pu,*) 
     &  '  10^2 (ch3b) or 10^3 (ch4,5)  x central wavenumber :',
     &                                     avh1c_h_radtempcnv(1,ich)
        WRITE(pu,*) '  10^5 x constant 1 :',avh1c_h_radtempcnv(2,ich)
        WRITE(pu,*) '  10^6 x constant 2 :',avh1c_h_radtempcnv(3,ich)
      ENDDO
      
      RETURN
      END       
