! Name: $Id: maia_BoxEnv_reset.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_BoxEnv_reset 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_BoxEnv_reset ( idbg, topo, box)

! Authors       : lydie lavanant

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

  type( debug ), INTENT(in)     :: idbg ! 
  type( topo_field),INTENT(inout)       :: topo ! surface topography landsea and elev
  type( box_id ),INTENT(inout)  :: box  ! box information

  real                          :: cozsat, cozsol, sizsat, sizsol, cofi
  INTEGER                       :: landsea, alt
  LOGICAL, SAVE                 :: info_norm=.true.
!===================================================================
  ! Longitude normalisation between -180 and +180 degrees
  ! -----------------------------------------------------
  if(info_norm) then
     call maia_Lon_Norm (.true., 'box   ', box % lon)
     info_norm=.false.
  else
     call maia_Lon_Norm (.false., 'box   ', box % lon)
  endif

  ! relative azimuth angle  (deg)
  ! -----------------------------
  if(box % difazi< -360.) then
     box % difazi = ABS(box % satazi - box % sunazi)
     if( box % difazi >  180. ) box % difazi = box % difazi - 360.
     if( box % difazi < -180. ) box % difazi = box % difazi + 360.
  endif

  ! secant of local zenith angle
  ! -----------------------------
  box % satsec = 1./cos(box % satzen * cdr)

  ! cos of scattering angle
  ! -----------------------------
  cozsat=cos( cdr* box % satzen)
  cozsol=cos( cdr* box % sunzen)
  sizsat=sin( cdr* box % satzen)
  sizsol=sin( cdr* box % sunzen)
  cofi=  cos( cdr* box % difazi)
  box % cos_scat_ang=cozsat*cozsol+sizsat*sizsol*cofi

  ! cosol
  ! -----------------------------
  box % cosol= cos( cdr * box % sunzen )
  if(box % cosol < cosolmax) box % cosol = cosolmax 

  ! glint conditions
  !-----------------
  call maia_GetGlint (idbg, box)

  ! 0: NIGHT /1: TWILIGHT /2: DAY /3: SUNGLINT
  ! -----------------------------
  box % day      =.false.
  box % night    =.false.
  box % glint    =.false.
  box % twilight =.false.

  if(box % dcj == 0 .and. box % sunzen < 83) then               
        box % day_time =2                       ! DAY           
        box % day      =.true.
  else if (box % dcj == 1 .and. box % sunzen < 83) then 
        box % day_time =3                       ! SUNGLINT              
        box % glint    =.true.
  else if (box % sunzen > 90) then                      
!  else if (box % sunzen > 85) then                     
        box % day_time =0                       ! NIGHT
        box % night    =.true.
  else                          
        box % day_time =1                       ! TWILIGHT
        box % twilight =.true.
  endif 

  ! topography at box resolution
  ! -----------------------------
  box % sea =           .false.  
  box % coast =         .false.  
  box % land =          .false.  
  box % desert =        .false. 
  box % snow_always =   .false. 

!   landsea=0   ! mer
!   landsea=1   ! terre
!   landsea=2   ! desert
!   landsea=3   ! neige perenne
!   landsea=4   ! cote
!   box% tm     ! 0=sea, 1=coast, 2=land, 3=desert, 4=long-lasting snow
  call maia_GetTopo ( idbg, topo, box%lat, box%lon, box%psize ,box%lsize, landsea, alt )

  box%alt=alt

  if (landsea ==0 ) then
     box % sea  = .true. 
     box%tm = 0
  elseif (landsea ==1 ) then
     box % land = .true. 
     box%tm = 2
  elseif (landsea ==2 ) then
     box % desert = .true.
     box%tm = 3
  elseif (landsea ==3 ) then
     box % snow_always = .true.
     box%tm = 4
  elseif (landsea ==4 ) then
     box % coast = .true. 
     box%tm = 1
  else
     write(*,*) 'Pb in TOPO', box%lat, box%lon, box%psize ,box%lsize, landsea, alt
     stop 2
  endif

  if(idbg % debug) then
      write(*,*) 'BoxEnv_reset: tm, alt, sea, coast, land, day, night:'
      write(*,*) box%tm, box%alt,box%sea, box%coast, box%land, box%day, box%night
  end if


  ! some specific areas
  ! -------------------
  box % Aladin=.false.
  if ( box % lat >= aladin_limit_S .and. box % lat <= aladin_limit_N .and. &
       box % lon >= aladin_limit_W .and. box % lon <= aladin_limit_E ) then
       box % Aladin=.true.
  endif
  box % Europe=.false.
  if ( box % lat >= europe_limit_S .and. box % lat <= europe_limit_N .and. &
       box % lon >= europe_limit_W .and. box % lon <= europe_limit_E ) then
       box % Europe=.true.
  endif
  box % Africa=.false.
  if ( box % lat >= africa_limit_S .and. box % lat <= africa_limit_N .and. &
       box % lon >= africa_limit_W .and. box % lon <= africa_limit_E ) then
       box % Africa=.true.
  endif

  box % ts_prev = vmanq         ! Temp surf from forecast at 0m altitude (K*100)
  box % tsurf= vmanq            ! surface temperature at surface altitude               

  box % t2m_prev= vmanq         ! Temp at 2m (forecast) (K*100) at 0m altitude
  box % sst_clim = vmanq        ! SST minimum from climatology  (K*100)
  box % sstmoy_clim = vmanq     ! SST moy from  climatology (K*100)

  box % albedo_clim = vmanq     ! Albedo  from climatology (%*100)

  box % cwv_prev= vmanq         ! total water vap content (forecast) (g/cm2*100)
  box % cwv= vmanq              ! total water vapour content g/cm2*100

  box % altnwp  = vmanq         ! altitude of the forecast (m)
  box % t500 = vmanq            ! forecast temperature at different pressure
  box % t700 = vmanq            ! forecast temperature at different pressure
  box % t850 = vmanq            ! forecast temperature at different pressure
  box % ttropo = vmanq          ! forecast temperature at tropopause
  
END SUBROUTINE  maia_BoxEnv_reset
      
