! Name: $Id: maia_Box_GetTopo.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_Box_GetTopo 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/01/2014   Lydie Lavanant
!  1.1    01/01/2014   Lydie Lavanant Add control for missing channels; use 0.01 degres landsea atlas; update definition of CloudMask (bits 15-18)
!  1.2    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.3    01/05/2015   Lydie Lavanant/Pascale Roquet bug fixes
!  1.4    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_Box_GetTopo ( idbg, pp, ll, field_id, field_1b, landsea, elev, box)

! Authors       : lydie lavanant

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

  type( debug ), INTENT(in)     :: idbg ! 
  integer, intent(in)           :: pp,ll
  Type (field_info)             :: field_id
  Type (field),intent(in)       :: field_1b
  type( topo_field),INTENT(out) :: landsea, elev        ! surface topography ls and elev
  type( box_id ),INTENT(inout)  :: box  ! box information

  CHARACTER (LEN=16)            :: chm          ! character string for messages
  real                          :: cozsat, cozsol, sizsat, sizsol, cofi
  LOGICAL, SAVE                 :: info_norm
  
  integer, allocatable          :: ls(:), alt(:)
  integer                       :: n, pix, lig, near
  real                          :: dist, distmin
  real                          :: lat, lon, latmoy, lonmoy, altmoy
  real                          :: satzenmoy, satazimoy, sunzenmoy, sunazimoy,difazimoy
  real(kind=8)                  :: ijmoy
  real(kind=8)                  :: lat_rd, lon_rd, pos_viirsmoy(3)
  real(kind=8),allocatable      :: pos_viirs(:,:)
  logical                       :: difazimoy_exist
  real                          :: dist_seuil
!===================================================================
  allocate ( ls(field_id % nb_fov(pp,ll)), alt(field_id % nb_fov(pp,ll)) )
  allocate ( pos_viirs(3,field_id % nb_fov(pp,ll)) )
  
  latmoy = 0. ;  lonmoy = 0. ;  altmoy = 0. 
  satzenmoy=0 ; satazimoy=0. ; sunzenmoy=0. ; sunazimoy=0. ; difazimoy=0 ;ijmoy=0
  difazimoy_exist=.false.
  do n=1, field_id % nb_fov(pp,ll)
     pix        = field_id % pix_box(n, pp, ll)
     lig        = field_id % lig_box(n, pp, ll)          
     lon        = field_1b % lon (pix,lig)      ! longitude (degrees)
     lat        = field_1b % lat (pix,lig)      ! latitude (degrees)
     lat_rd = lat * cdr
     lon_rd = lon * cdr     
     call geogcart(lat_rd,lon_rd, pos_viirs(:,n))

     if(info_norm) then
        info_norm=.false.
        if(idbg%debug)  info_norm=.true.
        call maia_Lon_Norm (.true., 'box   ', lon)
        info_norm=.false.
     else
        call maia_Lon_Norm (.false., 'box   ', lon)
     endif
 
     !   ls=0 mer ! ls=1 terre ! ls=2 desert ! ls=3 neige perenne ! ls=4 cote ! ls=5 inland wate
     call maia_GetTopo ( idbg, landsea, elev, lat, lon, 5 ,5, ls(n), alt(n) )
     
     if (field_id%isOver180 )then
      if(Lon  >=  360.) Lon = Lon - 360.
      if(Lon  <  0.)    Lon = Lon + 360. 
     endif    

     latmoy = latmoy+lat
     lonmoy = lonmoy+lon
     altmoy = altmoy+alt(n)

     satzenmoy = satzenmoy + field_1b % satzen (pix,lig)
     sunzenmoy = sunzenmoy + field_1b % sunzen (pix,lig)
     if( field_1b %difazi(pix,lig)>=-360.) then
        difazimoy = difazimoy + field_1b %difazi(pix,lig)
        difazimoy_exist=.true.
     else
        satazimoy = satazimoy + field_1b % satazi (pix,lig)
        sunazimoy = sunazimoy + field_1b % sunazi (pix,lig)
     endif
     ijmoy     = ijmoy     + field_1b % ij (lig)
   enddo
  
  latmoy = latmoy/field_id % nb_fov(pp,ll)
  lonmoy = lonmoy/field_id % nb_fov(pp,ll)    

  if (field_id%isOver180 ) then
    call maia_Lon_Norm(.false., "     ",lonmoy)
  endif
 

  ! recherche du pixel le plus proche
  lat_rd = latmoy * cdr
  lon_rd = lonmoy * cdr     
  call geogcart(lat_rd,lon_rd, pos_viirsmoy)
  distmin = 9999. ; near=0
  do n=1, field_id % nb_fov(pp,ll)
     dist =dsqrt( (pos_viirs(1,n)-pos_viirsmoy(1))**2 + &
                  (pos_viirs(2,n)-pos_viirsmoy(2))**2 + &
                  (pos_viirs(3,n)-pos_viirsmoy(3))**2) 
     if(dist<distmin) then
        near=n
        distmin = dist
     endif
  end do
  
  dist_seuil=4.
  if(latmoy>75. .or. latmoy<-75.) dist_seuil=6.
  
  if(distmin<dist_seuil) then
     pix                = field_id % pix_box(near, pp, ll)
     lig                = field_id % lig_box(near, pp, ll)          
     box % lon          = field_1b % lon (pix,lig)      ! longitude (degrees)
     box % lat          = field_1b % lat (pix,lig)      ! latitude (degrees)
     box % satzen       = field_1b % satzen(pix,lig)    ! satellite  zenithal angle(deg)
     box % sunzen       = field_1b % sunzen(pix,lig)    ! solar zenithal angle(deg)
     box % ij           = field_1b % ij(lig)    ! julian instant of the central pixel
     if(difazimoy_exist) then 
        box % difazi    = field_1b % difazi(pix,lig)
     else
     box % satazi       = field_1b % satazi(pix,lig)    ! satellite azimutal angle (deg)
     box % sunazi       = field_1b % sunazi(pix,lig)    ! solar azimutal angle(deg)
     endif
  else
     write(chm,'(2f8.2)') latmoy, lonmoy
     call ml_wt('warning','No nearest pixel at center of box for '//chm)
     call ml_wtr('warning','distmin: ',distmin)
     box % lon          = lonmoy        ! longitude (degrees)
     box % lat          = latmoy        ! latitude (degrees)
     box % satzen       = satzenmoy/field_id % nb_fov(pp,ll)    ! satellite  zenithal angle(deg)
     box % sunzen       = sunzenmoy/field_id % nb_fov(pp,ll)    ! solar zenithal angle(deg)
     box % ij           = ijmoy/field_id % nb_fov(pp,ll)        ! julian instant of the central pixel
     if(difazimoy_exist) then 
        box % difazi    = difazimoy/field_id % nb_fov(pp,ll)
     else
        box % satazi    = satazimoy/field_id % nb_fov(pp,ll)    ! satellite azimutal angle (deg)
        box % sunazi    = sunazimoy/field_id % nb_fov(pp,ll)    ! solar azimutal angle(deg)
     endif
  endif
  
  
  box % alt = altmoy/field_id % nb_fov(pp,ll)
  
  ! topography at box resolution
  ! -----------------------------
  box % sea =           .false.  
  box % coast =         .false.  
  box % land =          .false.  
  box % desert =        .false. 
  box % snow_always =   .false. 
 
  ! box% tm ! landsea at 0.01° : 0=sea, 1=land, 2=desert, 3=long-lasting snow,  4=coast, 5=inland water
  if (all(ls ==0) .or. all(ls ==5) ) then
     box % sea  = .true. 
     box%tm = 0
  elseif (all(ls ==1) ) then
     box % land = .true. 
     box%tm = 2
  elseif (all(ls ==2) ) then
     box % desert = .true.
     box%tm = 3
  elseif (all(ls ==3) ) then
     box % snow_always = .true.
     box%tm = 4
  elseif (any(ls ==4) .or. any(ls /= ls(1)) )then
     box % coast = .true. 
     box%tm = 1
  else
     write(*,*) 'Pb in TOPO', box%lat, box%lon, box%psize ,box%lsize, ls, alt
     stop 2
  endif
  if(idbg % debug) then
      write(*,'(a20,30i2)') 'BoxEnv_GetTopo: ls:',ls(1: field_id % nb_fov(pp,ll))
  end if
 
  ! relative azimuth angle  (deg)
  ! -----------------------------
  if(box % difazi< -360.) then
     box % difazi = ABS(box % satazi - box % sunazi)
     if( box % difazi >  180. ) box % difazi = box % difazi - 360.
     if( box % difazi < -180. ) box % difazi = box % difazi + 360.
  endif

  ! secant of local zenith angle
  ! -----------------------------
  box % satsec = 1./cos(box % satzen * cdr)

  ! cos of scattering angle
  ! -----------------------------
  cozsat=cos( cdr* box % satzen)
  cozsol=cos( cdr* box % sunzen)
  sizsat=sin( cdr* box % satzen)
  sizsol=sin( cdr* box % sunzen)
  cofi=  cos( cdr* box % difazi)
  box % cos_scat_ang=cozsat*cozsol+sizsat*sizsol*cofi

  ! cosol
  ! -----------------------------
  box % cosol= cos( cdr * box % sunzen )
  if(box % cosol < cosolmax) box % cosol = cosolmax 

  ! glint conditions
  !-----------------
  call maia_GetGlint (idbg, box)

  ! 0: NIGHT /1: TWILIGHT /2: DAY /3: SUNGLINT
  ! -----------------------------
  box % day      =.false.
  box % night    =.false.
  box % glint    =.false.
  box % twilight =.false.

  if(box % dcj == 0 .and. box % sunzen < cst_sunzenmax_day) then               
        box % day_time =2                       ! DAY           
        box % day      =.true.
  else if (box % dcj == 1 .and. box % sunzen < cst_sunzenmax_day ) then        
        box % day_time =3                       ! SUNGLINT              
        box % glint    =.true.
  else if (box % sunzen > cst_sunzenmin_night) then                      
        box % day_time =0                       ! NIGHT
        box % night    =.true.
  else                          
        box % day_time =1                       ! TWILIGHT
        box % twilight =.true.
  endif 
  
  if(idbg % debug) then
      write(*,*) 'BoxEnv_reset: tm, alt, sea, coast, land, day, night:'
      write(*,*) box%tm, box%alt,box%sea, box%coast, box%land, box%day, box%night
  end if

  ! some specific areas
  ! -------------------
  box % Aladin=.false.
  if ( box % lat >= aladin_limit_S .and. box % lat <= aladin_limit_N .and. &
       box % lon >= aladin_limit_W .and. box % lon <= aladin_limit_E ) then
       box % Aladin=.true.
  endif
  box % Europe=.false.
  if ( box % lat >= europe_limit_S .and. box % lat <= europe_limit_N .and. &
       box % lon >= europe_limit_W .and. box % lon <= europe_limit_E ) then
       box % Europe=.true.
  endif
  box % Africa=.false.
  if ( box % lat >= africa_limit_S .and. box % lat <= africa_limit_N .and. &
       box % lon >= africa_limit_W .and. box % lon <= africa_limit_E ) then
       box % Africa=.true.
  endif

  box % ts_prev = vmanq         ! Temp surf from forecast at 0m altitude (K*100)
  box % tsurf= vmanq            ! surface temperature at surface altitude               

  box % t2m_prev= vmanq         ! Temp at 2m (forecast) (K*100) at 0m altitude
  box % sst_clim = vmanq        ! SST minimum from climatology  (K*100)
  box % sstmoy_clim = vmanq     ! SST moy from  climatology (K*100)

  box % albedo_clim = vmanq     ! Albedo  from climatology (%*100)

  box % cwv_prev= vmanq         ! total water vap content (forecast) (g/cm2*100)
  box % cwv= vmanq              ! total water vapour content g/cm2*100

  box % altnwp  = vmanq         ! altitude of the forecast (m)
  box % t500 = vmanq            ! forecast temperature at different pressure
  box % t700 = vmanq            ! forecast temperature at different pressure
  box % t850 = vmanq            ! forecast temperature at different pressure
  box % ttropo = vmanq          ! forecast temperature at tropopause
  
  deallocate (ls, alt, pos_viirs)
  
END SUBROUTINE  maia_Box_GetTopo
