! Name: $Id: maia_Box_reset.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_Box_reset 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
!=====================================================================================
subroutine maia_Box_reset ( idbg, box ) 
!=====================================================================================

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

  type( debug ), INTENT(in)     :: idbg ! 
  type( box_id ),INTENT(inout)  :: box  ! box information
  if(idbg%debug) then
    write(*,*) "maia_Box_Reset"  
  endif
  box % ij              =vmanq  ! julian instant of the central pixel
  box % lon             =vmanq  ! longitude (degrees)
  box % lat             =vmanq  ! latitude (degrees)
  box % satzen          =vmanq  ! satellite  zenithal angle(deg)
  box % satazi          =vmanq  ! satellite azimutal angle (deg)
  box % sunzen          =vmanq  ! solar zenithal angle(deg)
  box % sunazi          =vmanq  ! solar azimutal angle(deg)
  box % psize           =imanq  ! size of the box in Moderate resolution pixels
  box % lsize           =imanq  ! size of the box in Moderate resolution lines
  box % tm              =imanq  ! 0=sea, 1=coast, 2=land, 3=desert, 4=long-lasting snow
  box % sea             =.false.        ! 
  box % land            =.false.        ! 
  box % coast           =.false.        ! 
  box % desert          =.false.        !
  box % snow_always     =.false.        ! from igbp atlas
  box % alt             =imanq  ! surface altitude (m)
  box % day_time        =imanq  ! 0: NIGHT /1: TWILIGHT /2: DAY /3: SUNGLINT
  box % day             =.false.
  box % night           =.false.
  box % twilight        =.false.
  box % glint           =.false.
  box % dcj             =imanq  ! specular reflexion 0=no / 1=yes
  box % satsec          =vmanq  ! secant of satellite zenith angle
  box % difazi          =vmanq  ! relative azimuth angle  (deg)
  box % cos_scat_ang    =vmanq  ! cosinus of the scattering angle 
  box % cosol           =vmanq
  box % cwv             =vmanq  ! total water vapour content g/cm2
  box % tsurf           =vmanq  ! surface temperature at surface altitude
  box % ps_prev         =vmanq  ! pressure   (forecast)  at surface altitude    
  box % ts_prev         =vmanq  ! Temp surf  (forecast) (K) at 0m altitude
  box % t2m_prev        =vmanq  ! Temp at 2m (forecast) (K) at 0m altitude
  box % cwv_prev        =vmanq  ! total water vap content (forecast) (g/cm2)
  box % sst_clim        =vmanq  ! SST minimum from atlas (K)
  box % sstmoy_clim     =vmanq  ! SST moy from atlas  (K)
  box % albedo_clim     =vmanq  ! Albedo  from atlas (%)
  box % altnwp          =vmanq  ! altitude of the forecast (m)
  box % t500            =vmanq  ! forecast temperature at different pressure
  box % t700            =vmanq  ! forecast temperature at different pressure
  box % t850            =vmanq  ! forecast temperature at different pressure
  box % ttropo          =vmanq  ! forecast temperature at tropopause
  box % inversion_NWP   =.false.!
  box % Aladin          =.false.
  box % Europe          =.false.
  box % Africa          =.false.
  box % Lake            =.false.

  box % psize   = 10
  box % lsize   = 10

end subroutine maia_Box_reset
