! Name: $Id: maia_CMa_CG.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_CMa_CG 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_CMa_CG  (idbg, pix_id, pix, box, thres, CMa)

! Authors       : lydie lavanant

! Coast - day

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ),      INTENT(in)  :: idbg       ! 
  type( pix_info),  INTENT(inout)  :: pix_id    ! 
  type( pix_data),  INTENT(in)  :: pix          ! 
  type( box_id ),     INTENT(in)  :: box        ! 
  type( maia_thres), INTENT(in)  :: thres       ! 
  type( maia_CMa ),  INTENT(out) :: CMa
  
! local
  real ::  idv, rivthr
!======================================================================
!    Vegetation threshold
!    --------------------------
  if(box % albedo_clim >= cst_desertalbmin) then
     rivthr = cst_rlivthr
  else
     rivthr = cst_rsivthr
  endif

!    --------------------------
!    Vegetation test
!    --------------------------
    if(pix % Obs_06 + pix % Obs_08 > 0) then
       idv=(pix % Obs_08 - pix % Obs_06) / (pix % Obs_06 + pix % Obs_08)
       if(idbg%debug )  write(*,*) 'Vegetation test: ', pix % Obs_08, pix % Obs_06, idv
    else
       idv=rivthr
    endif
  
    if (idv < rivthr) then      
       ! pixel over sea 
       ! --------------------------
       CMa % test_name   = 'CMa_CGS'        
       if(idbg%debug) write(*,'(a10,f10.2,a3,f10.2)') CMa % test_name, idv, '<',rivthr
          
       call maia_CMa_SG (idbg, pix_id,pix, box, thres, CMa)                       
    else
       ! pixel over land or coast
       ! --------------------------
       CMa % test_name   = 'CMa_CGL'        
       if(idbg%debug) write(*,'(a10,f10.2,a3,f10.2)') CMa % test_name, idv, '>=',rivthr
       call maia_CMa_LD (idbg, pix_id,pix, box, thres, CMa)                       
    endif

END SUBROUTINE maia_CMa_CG
