! Name: $Id: maia_CMa_ConfClear.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_CMa_ConfClear 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_CMa_ConfClear (idbg, ngroup, cc_group, CMa)

! Authors       : lydie lavanant

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE
! cc_group: individual clear confidence level from 1. (clear) to 0.(cloudy)

!I      Emission Threshold      BTM15
!II     Emission Difference     BTM12-BTM13
!                               BTM15-BTM12
!                               BTM14-BTM15 
!III    Reflectance Threshold   RefM1
!                               RefM5
!                               RefM7
!                               RefM7/RefM5
!IV     Reflectance Thin Cirrus RefM9
!V      Emission Thin Cirrus    BTM15-BTM16
!                               BTM12-BTM16

! input/output
  logical, INTENT(in)           :: idbg         ! 
  INTEGER, INTENT(inout)        :: ngroup(ngroups_max)
  real, INTENT(in)              :: cc_group(ngroups_max,ngroups_max)
  type( maia_CMa),INTENT(inout) :: CMa

  INTEGER               :: fl_clear !=3 confident clear
                                            !=2 probably clear
                                            !=1 probably cloudy
                                            !=0 confident cloudy    
! local
  REAL          :: confident_clear, conf_clear
  REAL          :: minimum_confidence(ngroups_max)
  INTEGER       :: n, ng, nv, ng_small
!=======================================================
   minimum_confidence(:) = vmanq
   ng_small = 0
   confident_clear=-99

   do n=1,ngroups_max
      if(ngroup(n)==1) then
         minimum_confidence(n) = cc_group(n,1)
      elseif(ngroup(n)>1) then
         nv=ngroup(n)
         minimum_confidence(n) = minval(cc_group(n,1:nv))
      else
         minimum_confidence(n) = vmanq
      endif
   end do
   if(idbg) write(*,'(a30,6i3,a20,6f9.2)') 'maia_CMa_ConfClear, ngroup:' ,ngroup, 'minimum_confidence:' ,minimum_confidence
   

   conf_clear = 1.
   ng=0
   do n= 1,  ngroups_max  
      if(minimum_confidence(n) > vmanq) then
         conf_clear= conf_clear * minimum_confidence(n)
         ng = ng+1
         if(minimum_confidence(n) <0.8) ng_small=ng_small+1
      endif
   end do
   
   ! ng should be always greater than 0
   if (ng ==0) then
     write(*,*) "PB of number of tests =0"
     write(*,'(a30,6i3,a20,6f8.1)') 'maia_CMa_ConfClear, ngroup:' ,ngroup, 'minimum_confidence:' ,minimum_confidence
     write(*,*) "Exit"
     stop 2
   endif

   confident_clear = conf_clear **(1./ng)

   if(confident_clear >= 0.99) then
      fl_clear=3                        ! confident clear
   elseif(confident_clear >= 0.50) then
      fl_clear=2                        ! probably clear
   elseif(confident_clear > 0.0) then
      fl_clear=1                        ! probably cloudy
   else
      fl_clear=0                        ! confident cloudy
   endif
   if(confident_clear<0.75 .and. fl_clear==2 .and. ng_small>=2) fl_clear=1

   if(idbg) write(*,'(a45,f6.2, i5)') 'maia_CMa_ConfClear: confident_clear, fl_clear:', confident_clear, fl_clear
   
   CMa % conf_clear = fl_clear
   CMa % conf_value = confident_clear
      
END SUBROUTINE maia_CMa_ConfClear
