! Name: $Id: maia_CMa_IceD.F90 514 2017-02-06 11:28:14Z frna $
!
! Purpose: 
!          Subroutines maia_CMa_IceD 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia4 version 4.3
!  1.3    23/12/2016   Lydie Lavanant enhancement for quality flags
SUBROUTINE maia_CMa_IceD (idbg, pix_id, pix, box, thres, CMa )

! Authors       : lydie lavanant

! cloud detection over snow/ice surface - day

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ),      INTENT(in)  :: idbg       ! 
  type( pix_info), INTENT(in)   :: pix_id       ! 
  type( pix_data),  INTENT(in)  :: pix          ! 
  type( box_id ),     INTENT(in)  :: box        ! 
  type( maia_thres), INTENT(in)  :: thres       ! 
  type( maia_CMa ),  INTENT(out)  :: CMa 
  
! local
  real  :: tsurf, ndvi0616min_snow, ndvi0616, seuil_06, seuil_37108, seuil_108min, seuil_108max
  real  :: s_ir, ndvi0616min_ice, cosol_ice, seuil_08, seuil_ir, qr16r06, s_06
  integer :: max_num_groups(ngroups_max), ngroup(ngroups_max), num_chan_ok
!======================================================================  
  max_num_groups(:)=0 ; ngroup(:)=0 ; num_chan_ok=0
  
  CMa % snowice_fl=0
  tsurf=box % ts_prev - (0.65* pix_id % alt)/100.
  cosol_ice=box%cosol
  if(cosol_ice < cosolmax_snowice) cosol_ice=cosolmax_snowice

  ! Land
  ! ----
  if( pix_id % land .or. pix_id % snow_always .or. pix_id % SnowIce_surf==1 .or. pix_id % coast) then

     seuil_06= snow_maxmin_offset + snow_maxmin_slope_ang*(box % cos_scat_ang + snow_maxmin_offset_ang)**2

     s_06 = thres % v_CMa_06_land
     if(pix_id % SnowIce_surf==1) s_06= 25. * cosol_ice
     seuil_06=MIN(s_06, seuil_06)       
       
     IF (pix % Qual_16 .and. pix % Obs_16>vmanq .and. pix % Qual_37 .and. pix % Obs_37>vmanq) THEN

        ndvi0616min_snow = ndvi0616_snow_offset + ndvi0616_snow_slope_ang*      &
                          (box % cos_scat_ang  + ndvi0616_snow_offset_ang)**2
   
        if( pix % Obs_06 >0 .and. pix % Obs_16 >0 ) then
          ndvi0616= (pix % Obs_06- pix % Obs_16) / (pix % Obs_06+ pix % Obs_16) 
        else
          ndvi0616=0
        endif 
        seuil_37108= 14. 

        if( idbg%debug ) then
          write(*,*) 'TEST_SNOW land done if ', box % sunzen,'<=', cst_sunzenmax_With16
          write(*,*) '   16       ',pix % Obs_16,'<', thres % v_CMa_16_ice + cst_ice16
          write(*,*) '   ndvi0616 ',ndvi0616,'>', ndvi0616min_snow
          write(*,*) '   108      ',pix % Obs_108, '>', tsurf- thres% v_CMa_108_land-cst_ice108
          write(*,*) '   37-108   ',pix % Obs_37-pix % Obs_108,'<', seuil_37108 
          write(*,*) '   06       ',pix % Obs_06, '>',seuil_06
          write(*,*) '   08       ',pix % Obs_08,'>', cst_08shadow * cos( cdr * box%sunzen )
        end if

        if( box % sunzen   <= cst_sunzenmax_With16                      .and. &
            pix % Obs_16   <  thres % v_CMa_16_ice + cst_ice16          .and. &
            ndvi0616       >  ndvi0616min_snow                          .and. &
            pix % Obs_108  <  286.15                                    .and. &
            pix % Obs_108  > (tsurf -thres% v_CMa_108_land - cst_ice108).and. & 
           (pix % Obs_37 - pix % Obs_108)  <  seuil_37108               .and. &
            pix % Obs_06   >  seuil_06                          ) then 
              CMa % snowice_fl=1
              if(pix % Qual_08 .and. pix % Obs_08 <= cst_08shadow * cos(cdr*box%sunzen)) CMa % snowice_fl=0
        endif 
        if(pix % Qual_16) num_chan_ok= num_chan_ok+1
        if(pix % Qual_06) num_chan_ok= num_chan_ok+1
        if(pix % Qual_108) num_chan_ok= num_chan_ok+1
        if(pix % Qual_37) num_chan_ok= num_chan_ok+1
        call maia_CMa_quality (idbg, 4, num_chan_ok, max_num_groups, ngroup, CMa % qual)
       

     ELSEIF (pix % Qual_16 .and. pix % Obs_16>vmanq .and. .not. pix % Qual_37 ) THEN

        ndvi0616min_snow = ndvi0616_snow_offset + ndvi0616_snow_slope_ang*      &
                          (box % cos_scat_ang  + ndvi0616_snow_offset_ang)**2
   
        if( pix % Obs_06 >0 .and. pix % Obs_16 >0 ) then
          ndvi0616= (pix % Obs_06- pix % Obs_16) / (pix % Obs_06+ pix % Obs_16) 
        else
          ndvi0616=0
        endif 

        if( idbg%debug ) then
          write(*,*) 'TEST_SNOW land done if ', box % sunzen, '<=', cst_sunzenmax_With16
          write(*,*) '   16       ',pix % Obs_16,'<', thres % v_CMa_16_ice + cst_ice16
          write(*,*) '   ndvi0616 ',ndvi0616,'>', ndvi0616min_snow
          write(*,*) '   108      ',pix % Obs_108, '>', tsurf- thres% v_CMa_108_land-cst_ice108
          write(*,*) '   108-120  ',pix % Obs_108 -pix % Obs_120,'<', cst_ice108120
          write(*,*) '   06       ',pix % Obs_06, '>',seuil_06
          write(*,*) '   08       ',pix % Obs_08,'>', cst_08shadow * cos( cdr * box%sunzen )
        end if

        if( box % sunzen   <= cst_sunzenmax_With16                      .and. &
            pix % Obs_16   <  thres % v_CMa_16_ice + cst_ice16          .and. &
            ndvi0616       >  ndvi0616min_snow                          .and. &
            pix % Obs_108  <  286.15                                    .and. &
            pix % Obs_108  > (tsurf -thres% v_CMa_108_land - cst_ice108).and. & 
           (pix % Obs_108 - pix % Obs_120)  <  cst_ice108120            .and. &
            pix % Obs_06   >  seuil_06                          ) then 
              CMa % snowice_fl=1
              if(pix % Qual_08 .and. pix % Obs_08 <= cst_08shadow * cos(cdr*box%sunzen)) CMa % snowice_fl=0
        endif      
        if(pix % Qual_16) num_chan_ok= num_chan_ok+1
        if(pix % Qual_06) num_chan_ok= num_chan_ok+1
        if(pix % Qual_108) num_chan_ok= num_chan_ok+1
        if(pix % Qual_120) num_chan_ok= num_chan_ok+1
        call maia_CMa_quality (idbg, 4, num_chan_ok, max_num_groups, ngroup, CMa % qual)

     ELSEIF (.not. pix % Qual_16 .and. pix % Qual_37 .and. pix % Obs_37>vmanq) THEN

        seuil_08= cst_08shadow * box%cosol *0.6
        seuil_37108= 15. * box%cosol + 0.3* thres % v_CMa_06_land
        seuil_108min= tsurf -thres% v_CMa_108_land- cst_ice108 -5.
        seuil_108max= 286.15
        if(box % glint) seuil_37108= cst_ice37108* box%cosol + 0.45* thres % v_CMa_06_land
        if(box % Aladin) then
          seuil_37108= 10. * box%cosol
          seuil_08= cst_08shadow * box%cosol
          seuil_108min= tsurf -thres% v_CMa_108_land -5.
          seuil_108max= 277.15
        endif

        if( idbg%debug ) then
          write(*,*) 'TEST_SNOW land done if ', box % sunzen, '<=', cst_sunzenmax_No16
          write(*,*) '   37-108  ',pix % Obs_37 - pix % Obs_108,'<', seuil_37108
          write(*,*) '   108     ',pix % Obs_108, '>', seuil_108min
          write(*,*) '   06      ',pix % Obs_06, '>',seuil_06
          write(*,*) '   08      ',pix % Obs_08,'>', seuil_08
        end if

        if( box % sunzen   <= cst_sunzenmax_No16                .and. &
               (pix % Obs_37 - pix % Obs_108) < seuil_37108     .and. &
                pix % Obs_108  <  seuil_108max                  .and. &
                pix % Obs_108  >  seuil_108min                  .and. & 
                pix % Obs_06  >  seuil_06                       .and. &
                pix % Obs_08  >  seuil_08) then
              CMa % snowice_fl=1
        endif      
        if(pix % Qual_08) num_chan_ok= num_chan_ok+1
        if(pix % Qual_06) num_chan_ok= num_chan_ok+1
        if(pix % Qual_108) num_chan_ok= num_chan_ok+1
        if(pix % Qual_37) num_chan_ok= num_chan_ok+1
        call maia_CMa_quality (idbg, 4, num_chan_ok, max_num_groups, ngroup, CMa % qual)

     ELSE
        if( idbg%debug ) then
          write(*,*) 'TEST_ICE land PB of Pixel Quality', pix_id%line, pix_id%pixel
          !call maia_Pr_InfoPix (pix_id, pix)
        end if
     ENDIF

  ! Sea
  ! ----
  elseif( pix_id % sea) then

     seuil_08= cst_08shadow * box%cosol *0.6
     seuil_ir=cst_ice108 + 5.

     if (box%t2m_prev > vmanq) then     ! used for sea ice detection
        s_ir=thres% v_CMa_108_sea       ! when no sst_clim available
     else
        s_ir=cst_irs
     endif     

     IF (pix % Qual_16 .and. pix % Obs_16>vmanq .and. pix % Qual_37 .and. pix % Obs_37>vmanq) THEN

        ndvi0616min_ice = ndvi0616_ice_offset + ndvi0616_ice_slope_ang* &
                         (box % cos_scat_ang  + ndvi0616_ice_offset_ang)**2
        
        if( pix % Obs_06 >0 .and. pix % Obs_16 >0 ) then
           ndvi0616= (pix % Obs_06- pix % Obs_16) / (pix % Obs_06+ pix % Obs_16) 
           qr16r06=pix % Obs_16/pix % Obs_06
        else
           ndvi0616=0
           qr16r06=0
        endif        
       
        if( idbg%debug ) then
          write(*,*) 'TEST_ICE sea ', box % sunzen
          write(*,*) '   sst_clim ',box % sst_clim ,'<=', 278.
          write(*,*) '   tsurf    ',tsurf,'<=', 278.
          write(*,*) '   16       ',pix % Obs_16,'<', thres % v_CMa_16_ice
          write(*,*) '   ndvi0616 ',ndvi0616,'>', ndvi0616min_ice
          write(*,*) '   qr16r06  ',qr16r06, '<', .32
          write(*,*) '   108      ',pix % Obs_108, '<', 278.
          write(*,*) '   108      ',pix % Obs_108, '>', tsurf- s_ir -seuil_ir
          write(*,*) '   37-108   ',pix % Obs_37-pix % Obs_108,'<', 11.2*cosol_ice
          write(*,*) '   06       ',pix % Obs_06, '>',thres % v_CMa_108_sea
          write(*,*) '   08       ',pix % Obs_08, '>',seuil_08
        end if

        if( box % sunzen  <  86. ) then
           if( (box % sst_clim <=  278. .or.  tsurf<= 278.)     .and. &
                pix % Obs_16   <  thres % v_CMa_16_ice          .and. &
                ndvi0616       >  ndvi0616min_ice               .and. &
                qr16r06        <  0.32                          .and. &
                pix % Obs_108  <  278.                          .and. &
                pix % Obs_108  >  (tsurf -s_ir -seuil_ir)       .and. &
               (pix % Obs_108 - pix % Obs_120) < cst_ice108120  .and. &
                pix % Obs_06   >  thres % v_CMa_06_sea          .and. &
                pix % Obs_08   >  seuil_08      ) then
                   CMa % snowice_fl=1
           endif
           if(pix % Qual_08) num_chan_ok= num_chan_ok+1
           if(pix % Qual_06) num_chan_ok= num_chan_ok+1
           if(pix % Qual_108) num_chan_ok= num_chan_ok+1
           if(pix % Qual_120) num_chan_ok= num_chan_ok+1
           if(pix % Qual_16) num_chan_ok= num_chan_ok+1
           call maia_CMa_quality (idbg, 5, num_chan_ok, max_num_groups, ngroup, CMa % qual)
        endif      

     ELSEIF (pix % Qual_16 .and. pix % Obs_16>vmanq .and. .not. pix % Qual_37 ) THEN

        ndvi0616min_ice = ndvi0616_ice_offset + ndvi0616_ice_slope_ang* &
                         (box % cos_scat_ang  + ndvi0616_ice_offset_ang)**2
        
        if( pix % Obs_06 >0 .and. pix % Obs_16 >0 ) then
           ndvi0616= (pix % Obs_06- pix % Obs_16) / (pix % Obs_06+ pix % Obs_16) 
           qr16r06=pix % Obs_16/pix % Obs_06
        else
           ndvi0616=0
           qr16r06=0
        endif        
       
        if( idbg%debug ) then
          write(*,*) 'TEST_ICE sea ', box % sunzen
          write(*,*) '   sst_clim ',box % sst_clim ,'<=', 278.
          write(*,*) '   tsurf    ',tsurf,'<=', 278.
          write(*,*) '   16       ',pix % Obs_16,'<', thres % v_CMa_16_ice
          write(*,*) '   ndvi0616 ',ndvi0616,'>', ndvi0616min_ice
          write(*,*) '   qr16r06  ',qr16r06, '<', .32
          write(*,*) '   108      ',pix % Obs_108, '<', 278.
          write(*,*) '   108      ',pix % Obs_108, '>', tsurf- s_ir -seuil_ir
          write(*,*) '   37-108   ',pix % Obs_37 -pix % Obs_108,'<', 14
          write(*,*) '   06       ',pix % Obs_06, '>',thres % v_CMa_108_sea
          write(*,*) '   08       ',pix % Obs_08, '>',seuil_08
        end if

        if( box % sunzen  <  86. ) then
           if( (box % sst_clim <=  278. .or.  tsurf<= 278.)     .and. &
                pix % Obs_16   <  thres % v_CMa_16_ice          .and. &
                ndvi0616       >  ndvi0616min_ice               .and. &
                qr16r06        <  0.32                          .and. &
                pix % Obs_108  <  278.                          .and. &
                pix % Obs_108  >  (tsurf -s_ir -seuil_ir)       .and. &
               (pix % Obs_37-pix % Obs_108) < 14.               .and. &
                pix % Obs_06   >  thres % v_CMa_06_sea          .and. &
                pix % Obs_08   >  seuil_08      ) then
                   CMa % snowice_fl=1
           endif
           if(pix % Qual_08) num_chan_ok= num_chan_ok+1
           if(pix % Qual_06) num_chan_ok= num_chan_ok+1
           if(pix % Qual_108) num_chan_ok= num_chan_ok+1
           if(pix % Qual_37) num_chan_ok= num_chan_ok+1
           if(pix % Qual_16) num_chan_ok= num_chan_ok+1
           call maia_CMa_quality (idbg, 5, num_chan_ok, max_num_groups, ngroup, CMa % qual)
        endif    
        
     ELSEIF (.not. pix % Qual_16 .and. pix % Qual_37 .and. pix % Obs_37>vmanq) THEN
       seuil_37108= cst_ice37108 * cosol_ice

        if( idbg%debug ) then
          write(*,*) 'TEST_ICE sea ', box % sunzen
          write(*,*) '   sst_clim ',box % sst_clim ,'<=', 278.
          write(*,*) '   tsurf    ',tsurf,'<=', 278.
          write(*,*) '   37-108   ',pix % Obs_37 - pix % Obs_108,'<', seuil_37108
          write(*,*) '   108      ',pix % Obs_108, '<', 278.
          write(*,*) '   108      ',pix % Obs_108, '>', tsurf- s_ir -seuil_ir
          write(*,*) '   06       ',pix % Obs_06, '>',thres % v_CMa_06_sea
          write(*,*) '   08       ',pix % Obs_08, '>',seuil_08
        end if

        if( box % sunzen  <  86. ) then
           if( (box % sst_clim <=  278. .or.  tsurf<= 278.)     .and. &
                (pix % Obs_37 - pix % Obs_108) < seuil_37108    .and. &
                pix % Obs_108  <  278.                          .and. &
                pix % Obs_108  >  (tsurf -s_ir -seuil_ir)       .and. &
                pix % Obs_06   >  thres % v_CMa_06_sea          .and. &
                pix % Obs_08   >  seuil_08      ) then
                   CMa % snowice_fl=1
           endif
           if(pix % Qual_08) num_chan_ok= num_chan_ok+1
           if(pix % Qual_06) num_chan_ok= num_chan_ok+1
           if(pix % Qual_108) num_chan_ok= num_chan_ok+1
           if(pix % Qual_37) num_chan_ok= num_chan_ok+1
           call maia_CMa_quality (idbg, 4, num_chan_ok, max_num_groups, ngroup, CMa % qual)
        endif      

     ELSE
        if( idbg%debug ) then
          write(*,*) 'TEST_ICE sea PB of Pixel Quality', pix_id%line, pix_id%pixel
          call maia_Pr_InfoPix (pix_id, pix)
          stop 2
        end if
     ENDIF

  endif      

  if( idbg%debug ) write(*,*) '   snowice_fl : ', CMa % snowice_fl

END SUBROUTINE maia_CMa_IceD
