! Name: $Id: maia_CMa_quality.F90 514 2017-02-06 11:28:14Z frna $
!
! Purpose: 
!          Subroutines maia_CMa_quality 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.2    23/12/2016   Lydie Lavanant enhancement for quality flags
SUBROUTINE maia_CMa_quality (idbg, max_num_tests, num_tests_done, max_num_groups, ngroup_done, qual_fl)

! Authors       : lydie lavanant

USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ), INTENT(in)     :: idbg         ! 
  INTEGER, INTENT(in)           :: max_num_tests, num_tests_done
  INTEGER, INTENT(in)           :: max_num_groups(ngroups_max), ngroup_done(ngroups_max)
  INTEGER, INTENT(out)          :: qual_fl  !=3 high
                                            !=2 Medium
                                            !=1 poor
                                            !=0 bad  
! local
  REAL    :: q_val
  INTEGER :: n
  LOGICAL :: group_ok
!=======================================================
  IF (max_num_tests == 0 .or. num_tests_done==0) THEN
     qual_fl = 0        !bad
  ELSE IF (num_tests_done >= max_num_tests) THEN
     qual_fl = 3        !high
  ELSE
     q_val = (1.* num_tests_done)/(1.*max_num_tests)
     IF (q_val > 0.5) THEN
        qual_fl = 2     !Medium
     ELSE
        qual_fl = 1     !poor
     END IF
  END IF
  if(qual_fl ==1 .or. qual_fl ==2) then
     group_ok=.true.
     do n=1,ngroups_max
    
         if(max_num_groups(n)>0 .and. ngroup_done(n)==0) then
            group_ok=.false.         
         endif
     end do
     if(group_ok) qual_fl = qual_fl +1
  endif

  if(idbg % debug) write(*,'(a47,i2, a1, i2, a35, i2)') 'maia_CMa_quality: num_tests_done/max_num_tests ', &
                               num_tests_done,'/', max_num_tests, '(0=bad, 1=poor, 2=medium, 3=high)', qual_fl

END SUBROUTINE maia_CMa_quality
