! Name: $Id: maia_CMa_texture.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_CMa_texture 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_CMa_texture (idbg, box, pix_id, pix, CMa)

! Authors       : lydie lavanant

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

  type( debug ),      INTENT(in)        :: idbg         ! 
  type( box_id ),     INTENT(in)        :: box          ! 
  type( pix_info),  INTENT(in)          :: pix_id       ! 
  type( pix_data),  INTENT(in)          :: pix          ! 
  type( maia_CMa ),  INTENT(inout)      :: CMa

  INTEGER, PARAMETER :: Conf_Clear = 3
  INTEGER, PARAMETER :: Prob_Clear = 2
  INTEGER, PARAMETER :: Prob_Cloudy = 1
  INTEGER, PARAMETER :: Conf_Cloudy = 0

  real :: seuil_sd108, seuil_sd10837, val_dt, s_dt10806, s_sd08, s_sd108
  real :: thr_sd08, thr_sd108s, thr_sd10837sd, thr_sd10837sn, thr_sd108l, thr_sd10837l
  real :: thr_valdt(3), thr_10806(3), thr_slope10806(2)

!=======================================================
  if(idbg % debug) write(*,*) 'maia_CMa_texture: ', CMa % Conf_clear
  CMa % texture=.false.
        
  if(pix_id % texture_fromImager)then
     thr_sd08           = cstI_sd08 
     thr_sd108s         = cstI_sd108s
     thr_sd10837sd      = cstI_sd10837sd
     thr_sd10837sn      = cstI_sd10837sn
     thr_sd108l         = cstI_sd108l
     thr_sd10837l       = cstI_sd10837l
     thr_valdt(:)       = cstI_valdt(:)
     thr_10806(:)       = cstI_10806(:)
     thr_slope10806(:)  = cstI_slope10806(:)
  elseif (pix_id % texture_fromNeighbors) then
     thr_sd08           = cstA_sd08 
     thr_sd108s         = cstA_sd108s
     thr_sd10837sd      = cstA_sd10837sd
     thr_sd10837sn      = cstA_sd10837sn
     thr_sd108l         = cstA_sd108l
     thr_sd10837l       = cstA_sd10837l
     thr_valdt(:)       = cstA_valdt(:)
     thr_10806(:)       = cstA_10806(:)
     thr_slope10806(:)  = cstA_slope10806(:)
  else
     call ml_wt('ERROR','Pb in source of texture')
     stop 2
  endif

  IF ( CMa % Conf_clear == Conf_Clear .or. CMa % Conf_clear == Prob_Clear ) THEN

     if(pix_id % sea  ) THEN

        IF (box % day .and. pix_id% sd_108>vmanq .and. pix_id % sd_08>vmanq) THEN
           s_sd08= thr_sd08 + 0.03*pix % Obs_08
           if ((pix_id% sd_108 - thr_sd108s)>0.1 .and. (pix_id% sd_10837 - thr_sd10837sd)>0.1 .and. &
               (pix_id % sd_08 - s_sd08)>0.1 ) then 
              CMa % texture = .true.
              IF ( CMa % Conf_clear == Prob_Clear) then
                        CMa % Conf_clear = Prob_Cloudy      
              ELSE IF (CMa % Conf_clear == Conf_Clear) THEN
                        CMa % Conf_clear = Prob_Cloudy
              ENDIF
           elseif ((pix_id% sd_108 > thr_sd108s .and. pix_id% sd_10837 > thr_sd10837sd) .or. pix_id % sd_08 > s_sd08 ) then 
              CMa % texture = .true.
              IF ( CMa % Conf_clear == Prob_Clear) then
                        CMa % Conf_clear = Prob_Cloudy      
              ELSE IF (CMa % Conf_clear == Conf_Clear) THEN
                        CMa % Conf_clear = Prob_Clear
              ENDIF
           endif
           
           if(idbg % debug) then
              write(*,*) 'maia_CMa_texture: ', pix_id% sd_108,'>', thr_sd108s , pix_id% sd_10837, '>', &
                                               thr_sd10837sd , pix_id % sd_08,'>', s_sd08
           endif
           
        elseif (box % glint .and. pix_id% sd_108>vmanq .and. pix_id % sd_08>vmanq) THEN
           s_sd108= thr_sd108s +0.35
           s_sd08 = thr_sd08   +0.50 + 0.06*pix % Obs_08
           if((pix_id% sd_108 - s_sd108) >0.1 .and. (pix_id % sd_08 - s_sd08)>0.1) then
              CMa % texture = .true.
              IF ( CMa % Conf_clear == Prob_Clear) then
                        CMa % Conf_clear = Prob_Cloudy      
              ELSE IF (CMa % Conf_clear == Conf_Clear) THEN
                        CMa % Conf_clear = Prob_Cloudy
              ENDIF
           
           elseif (pix_id% sd_108 > s_sd108 .or. pix_id % sd_08 > s_sd08) then 
              CMa % texture = .true.
              IF ( CMa % Conf_clear == Prob_Clear) then
                        CMa % Conf_clear = Prob_Cloudy      
              ELSE IF (CMa % Conf_clear == Conf_Clear) THEN
                        CMa % Conf_clear = Prob_Clear
              ENDIF
           endif    
           
           if(idbg % info) then
              write(*,*) 'maia_CMa_texture: ', pix_id% sd_108,'>', s_sd108 , pix_id % sd_08,'>', s_sd08
           endif
           
        elseif (box % night .and. pix_id% sd_108>vmanq .and. pix_id% sd_10837>vmanq) THEN
           if ((pix_id% sd_108 - thr_sd108s) >0.1 .or. (pix_id% sd_10837 - thr_sd10837sn)>0.1)  then 
              CMa % texture = .true.
              IF ( CMa % Conf_clear == Prob_Clear) then
                        CMa % Conf_clear = Prob_Cloudy      
              ELSE IF (CMa % Conf_clear == Conf_Clear) THEN
                        CMa % Conf_clear = Prob_Cloudy
              ENDIF
           elseif (pix_id% sd_108 > thr_sd108s .or. pix_id% sd_10837 > thr_sd10837sn)  then
              CMa % texture = .true.
              IF ( CMa % Conf_clear == Prob_Clear) then
                        CMa % Conf_clear = Prob_Cloudy      
              ELSE IF (CMa % Conf_clear == Conf_Clear) THEN
                        CMa % Conf_clear = Prob_Clear
              ENDIF
           endif
           
           if(idbg % debug) then
              write(*,*) 'maia_CMa_texture: ', pix_id% sd_108,'>', thr_sd108s , pix_id % sd_10837,'>', thr_sd10837sd
           endif
           
        elseif (box % twilight .and. pix_id% sd_108>vmanq .and. pix_id% sd_10837>vmanq) THEN
           if (pix_id% sd_108 > thr_sd108s .and. pix_id% sd_10837 > thr_sd10837sn)  then 
              CMa % texture = .true.
              IF ( CMa % Conf_clear == Prob_Clear) then
                        CMa % Conf_clear = Prob_Cloudy      
              ELSE IF (CMa % Conf_clear == Conf_Clear) THEN
                        CMa % Conf_clear = Prob_Cloudy
              ENDIF
           endif
           if(idbg % debug) then
              write(*,*) 'maia_CMa_texture: ',pix_id% sd_108,'>', thr_sd108s , pix_id% sd_108,'>', thr_sd108s , &
              pix_id % sd_10837,'>', thr_sd10837sd, pix_id % sd_08,'>', thr_sd08 + 0.03*pix % Obs_08
           endif

        endif   !box % day/glint/night/twilight



     elseif( pix_id % land .or. pix_id % desert ) THEN
        
        IF ( (box % day .or. box % glint) .and. &
              pix_id% sd_108>vmanq .and. pix_id% sd_10837>vmanq .and. pix_id % Ratio_Max_10806 >vmanq) THEN
           seuil_sd108= thr_sd108l  + 1.67* (90.-box%sunzen)
           seuil_sd10837=thr_sd10837l + 1.67* (90.-box%sunzen)
           if(seuil_sd108  >  2.) seuil_sd108=  2.
           if(seuil_sd10837 >  2.) seuil_sd10837= 2.
        
           if(pix_id % Ratio_Max_10806 > 0.) then
              val_dt = pix_id % Ratio_Max_10806
           else
              val_dt=0
           endif
           if(val_dt < thr_valdt(1)) then
              s_dt10806=thr_10806(1)
           elseif(val_dt < thr_valdt(2))        then
              s_dt10806=thr_10806(1)+(val_dt-thr_valdt(1))*thr_slope10806(1)
           elseif(val_dt < thr_valdt(3)) then
              s_dt10806=thr_10806(2)+(val_dt-thr_valdt(2))*thr_slope10806(2)
           else
              s_dt10806=thr_10806(3)
           endif

           if(box %alt <= 500. .and. box % albedo_clim <= cst_desertalbmin ) then       
              if( (pix_id% sd_108>seuil_sd108 .and. pix_id% sd_10837>seuil_sd10837) .or. pix_id % Ratio_Max_10806>s_dt10806 ) then 
                 CMa % texture = .true.
                 IF ( CMa % Conf_clear == Prob_Clear) then
                        CMa % Conf_clear = Prob_Cloudy      
                 ELSE IF (CMa % Conf_clear == Conf_Clear) THEN
                        CMa % Conf_clear = Prob_Cloudy
                 ENDIF
              endif
           endif
           if(idbg % debug) then
              write(*,*) 'maia_CMa_texture: ', pix_id% sd_108,'>', seuil_sd108 , pix_id% sd_10837, '>', &
                                               seuil_sd10837 , pix_id % Ratio_Max_10806,'>', s_dt10806
           endif

        elseif (box % night .and. pix_id% sd_108>vmanq ) THEN
           seuil_sd108= thr_sd108l
           if(box %alt <= 500. .and. box % albedo_clim <= cst_desertalbmin ) then       
              if( pix_id% sd_108 > seuil_sd108 .and. pix_id% sd_10837 > thr_sd10837l) then 
                 CMa % texture = .true.
                 IF ( CMa % Conf_clear == Prob_Clear) then
                        CMa % Conf_clear = Prob_Cloudy      
                 ELSE IF (CMa % Conf_clear == Conf_Clear) THEN
                        CMa % Conf_clear = Prob_Cloudy
                 ENDIF
              endif
           endif
           if(idbg % debug) then
              write(*,*) 'maia_CMa_texture: ', pix_id% sd_108,'>', seuil_sd108 , pix_id % sd_10837,'>', thr_sd10837l
           endif

        elseif (box % twilight .and. pix_id% sd_108>vmanq ) THEN
           seuil_sd108= thr_sd108l  + 1.67* (90.-box%sunzen)
           seuil_sd10837=thr_sd10837l + 1.67* (90.-box%sunzen)
           if(seuil_sd108  >  2.)  seuil_sd108=  2.
           if(seuil_sd10837 >  2.) seuil_sd10837= 2.
           if (pix_id% sd_108 > seuil_sd108 .and. pix_id% sd_10837 > seuil_sd10837)  then 
                 CMa % texture = .true.
                 IF ( CMa % Conf_clear == Prob_Clear) then
                        CMa % Conf_clear = Prob_Cloudy      
                 ELSE IF (CMa % Conf_clear == Conf_Clear) THEN
                        CMa % Conf_clear = Prob_Cloudy
                 ENDIF
           endif
           if(idbg % debug) then
              write(*,*) 'maia_CMa_texture: ', pix_id% sd_108,'>', seuil_sd108 , pix_id % sd_10837,'>', seuil_sd10837
           endif

        endif   !box % day
     endif      !pix_id % land

  ENDIF

END SUBROUTINE maia_CMa_texture
