! Name: $Id: maia_CT_dawn.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_CT_dawn 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_CT_dawn (idbg, box, pix, thres, CT)

! Set Cloud Type in dawn condition

! Authors       : lydie lavanant

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ),      INTENT(in)        :: idbg         ! 
  type( box_id ),     INTENT(in)        :: box          ! info at the center of the box
  type( pix_data),    INTENT(in)        :: pix          ! pix observations (albedo in %, Tb in K)
  type( maia_thres), INTENT(in)         :: thres        ! 
  type( maia_CT),    INTENT(out)        :: CT           ! 

! local
  REAL :: t108120_thick, max108_thin
  REAL :: s_87_108_edge, s_37_108_medcld0, s_37_108_medcld1, s_37_108_medcld2
! ---------------------------------------------------------------------------     
       
! 108_120 threshold
! ------------
  if ( thres% v_CT_108_120_opaq < CT_108_120_opaq_max ) then
         t108120_thick = thres% v_CT_108_120_opaq 
  else
         t108120_thick = CT_108_120_opaq_max
  endif
      
! threshold for separation thin / very low 
! ----------------------------------------    
  if( box % sea ) then                                          ! sea
     max108_thin= thres% v_CT_max108_lo + thres % v_CMa_108_120_sea
  else                                                          ! land or coast
     max108_thin= thres% v_CT_max108_lo + thres % v_CMa_108_120_land
  endif
      
! 87_108
! -------
  s_87_108_edge = thres % v_CT_87_108_opaq
  if( pix % Obs_06 > CT_minalb_arid .and.  box % land ) s_87_108_edge = vmanq

  s_37_108_medcld0= vmanq
  if( pix % Obs_06 < CT_37_108_medclouds_maxalb .or.  box % sea ) then
    s_37_108_medcld0= CT_37_108_medclouds_min_0+ CT_37_108_medclouds_min_slope0*(box % satsec-1)
    s_37_108_medcld1= CT_37_108_medclouds_min_1+ CT_37_108_medclouds_min_slope1*(box % satsec-1)
    s_37_108_medcld2= CT_37_108_medclouds_min_2+ CT_37_108_medclouds_min_slope2*(box % satsec-1)
  endif

  if(idbg%debug) then
    write(*,*) 'ct_dawn'
    write(*,*) t108120_thick, max108_thin
  end if
        
! test serie
! ----------
      if ( pix % Obs_108 <  thres% v_CT_max108_vh ) then
         if ( pix % Obs_108-pix % Obs_120 < t108120_thick ) then
            CT % cld_type = CT_VH_OPAQ
         else 
            CT % cld_type = CT_SEMITR_THICK
         endif
      else if ( pix % Obs_108 < thres% v_CT_max108_hi ) then
         if ( pix % Obs_108-pix % Obs_120 < t108120_thick  ) then
            CT % cld_type = CT_HI_OPAQ
         else 
            CT % cld_type = CT_SEMITR_THICK
         endif
      else if ( pix % Obs_108 < thres% v_CT_max108_me ) then
         if ( pix % Obs_108-pix % Obs_120 < t108120_thick  ) then
            CT % cld_type = CT_MED
         else
            CT % cld_type = CT_SEMITR_MEAN
         endif
      else if ( pix % Obs_108 <  thres% v_CT_max108_lo ) then
         if ( pix % Obs_108-pix % Obs_120 < t108120_thick  ) then
            CT % cld_type = CT_LOW
         else if ( pix % Obs_87-pix % Obs_108 > s_87_108_edge ) then
            CT % cld_type = CT_SEMITR_THIN
         else
            CT % cld_type = CT_FRACTIONAL
         endif
      else if ( pix % Obs_108 < max108_thin ) then
         if ( pix % Obs_108-pix % Obs_120 < t108120_thick ) then
            CT % cld_type = CT_VLOW
         else if ( pix % Obs_87-pix % Obs_108 > s_87_108_edge ) then
            CT % cld_type = CT_SEMITR_THIN
         else
            CT % cld_type = CT_FRACTIONAL
         endif

      else 
         if ( pix % Obs_108-pix % Obs_120 < t108120_thick  ) then 
            CT % cld_type = CT_VLOW      
         else if ( pix % Obs_87-pix % Obs_108 > s_87_108_edge ) then
            CT % cld_type = CT_FRACTIONAL
         else
            CT % cld_type = CT_FRACTIONAL
         endif
      endif

!   if thermal inversion in NWP possibility to reclassify medium in low  
!   use T87-T108 as a security
  if( box % inversion_NWP ) then

      if(pix % Obs_108 >= thres % v_CT_max108_lo) then
            if(CT % cld_type  == CT_MED ) CT % cld_type  = CT_LOW

      else 
            if( s_37_108_medcld0 > vmanq ) then

               if( pix % Obs_87-pix % Obs_108 < s_37_108_medcld0 .and. &
                   pix % Obs_108 >= (thres % v_CT_max108_me - CT_offset_MeLow_InvMax0) ) then
                  if(CT % cld_type  == CT_MED ) CT % cld_type  = CT_LOW

               else if( pix % Obs_87-pix % Obs_108 < s_37_108_medcld1 .and. &
                        pix % Obs_108 >= (thres % v_CT_max108_me - CT_offset_MeLow_InvMax1) ) then
                  if(CT % cld_type  == CT_MED ) CT % cld_type  = CT_LOW

               else if( pix % Obs_87-pix % Obs_108 < s_37_108_medcld2 .and. &
                        pix % Obs_108 >= (thres % v_CT_max108_me - CT_offset_MeLow_InvMax2) )then
                  if(CT % cld_type  == CT_MED ) CT % cld_type  = CT_LOW
               endif  
         
            else
         
               if( pix % Obs_108 >= (thres % v_CT_max108_me - CT_offset_MeLow_InvMax00) ) then
                  if(CT % cld_type  == CT_MED ) CT % cld_type  = CT_LOW

               endif
            endif 
         endif  !test on Obs_108 with max108lo
  endif  !test on inversion


END SUBROUTINE maia_CT_dawn
