! Name: $Id: maia_CT_day.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_CT_day 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_CT_day (idbg, box, pix_id, pix, thres, CT)

! Set Cloud Type in day condition

! Authors       : lydie lavanant

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ),      INTENT(in)        :: idbg         ! 
  type( box_id ),     INTENT(in)        :: box          ! info at the center of the box
  type( pix_data),    INTENT(in)        :: pix          ! pix observations (albedo in %, Tb in K)
  type( pix_info),    INTENT(in)        :: pix_id       ! lat, lon, solar and satellite angles at the pixel
  type( maia_thres), INTENT(in)         :: thres        ! 
  type( maia_CT),    INTENT(out)        :: CT   ! 
  
! local
  REAL          :: sdlog_06, sdlog_108                  ! local log standard deviations (K)
  REAL          :: maxcir06
  REAL          :: s_108_120_verylow, s_108_120_thick, s_108_120_semi_above
  REAL          :: max108_thin, s_108_120
  REAL          :: s_87_108_edge, s_37_108_medcld0, s_37_108_medcld1, s_37_108_medcld2
  integer       :: num_test
! ---------------------------------------------------------------------------       

! local log std
! --------------
  if(pix_id% sd_108>vmanq) then
     sdlog_108= (15.*(log(1+ pix_id% sd_108)))
  else
     sdlog_108= vmanq
  endif
  if(pix_id% sd_06>vmanq) then
     sdlog_06 = 30.*(log(1+pix_id% sd_06))
     sdlog_06 = (CT_sea_logvar_tuning * sdlog_06)
  else
     sdlog_06 = vmanq
  endif

! 06/108 line separation
! ------------------
  maxcir06= thres % v_CT_maxcirrus_06slope * pix % Obs_108 + &
            thres % v_CT_maxcirrus_06intercept

! 108_120 thresholds
! ------------------
  if ( thres % v_CT_108_120_opaq  < CT_108_120_opaq_max ) then
         s_108_120_thick = thres % v_CT_108_120_opaq
  else
         s_108_120_thick = CT_108_120_opaq_max
  endif

 if(box % sea) then                     ! sea
         s_108_120 = thres % v_CMa_108_120_sea
         s_108_120_verylow = s_108_120_thick      
  else                                          ! land or coast
         s_108_120 = thres % v_CMa_108_120_land
         s_108_120_verylow = (s_108_120_thick + s_108_120)/2.
  endif

  s_108_120_semi_above = s_108_120_thick       

! IR threshold
! ------------
  max108_thin = thres % v_CT_max108_lo +2*s_108_120     ! maxt108_thin2 = maxt108lo +thinnest_cloud_fact*thrt108t120

! 87_108
! -------
  s_87_108_edge = thres % v_CT_87_108_opaq
  if( pix % Obs_06 > CT_minalb_arid .and.  box % land ) s_87_108_edge = vmanq

  s_37_108_medcld0= vmanq
  if( pix % Obs_06 < CT_37_108_medclouds_maxalb .or.  box % sea ) then
    s_37_108_medcld0= CT_37_108_medclouds_min_0+ CT_37_108_medclouds_min_slope0*(box % satsec-1)
    s_37_108_medcld1= CT_37_108_medclouds_min_1+ CT_37_108_medclouds_min_slope1*(box % satsec-1)
    s_37_108_medcld2= CT_37_108_medclouds_min_2+ CT_37_108_medclouds_min_slope2*(box % satsec-1)
  endif

  if(idbg%debug) then
     write(*,'(a15,3f10.2)') 'sd ', sdlog_108, sdlog_06, maxcir06
     write(*,'(a15,3f10.2)') 'v_CT_max108_lo', pix % Obs_108 , thres % v_CT_max108_lo
  endif

  num_test =-99
! test serie
! ----------
  if ( pix % Obs_108 < thres % v_CT_max108_vh ) then
     if ( pix % Obs_108 - pix % Obs_120 < s_108_120_thick ) then
            CT % cld_type = CT_VH_OPAQ
            num_test =1
     else 
            CT % cld_type = CT_SEMITR_THICK
            num_test =2
     endif
  
  else if ( pix % Obs_108 < thres % v_CT_max108_hi ) then
       if ( pix % Obs_108-pix % Obs_120 < s_108_120_thick ) then
            CT % cld_type = CT_HI_OPAQ
            num_test =3
       else 
            CT % cld_type = CT_SEMITR_THICK
            num_test =4
       endif
      
  else if ( pix % Obs_108 < thres % v_CT_max108_me ) then
       if ( pix % Obs_06 >  maxcir06 ) then
            if ( pix % Obs_108-pix % Obs_120 > s_108_120_semi_above ) then
               CT % cld_type = CT_SEMITR_ABOVE
               num_test =5
            else
               CT % cld_type = CT_MED
               num_test =6
            endif
       else 
            if ( pix % Obs_108-pix % Obs_120 > s_108_120_semi_above ) then
               CT % cld_type = CT_SEMITR_MEAN
               num_test =7
            else
               CT % cld_type = CT_MED        
               num_test =8
            endif
       endif
  
  else if ( pix % Obs_108 < thres % v_CT_max108_lo ) then
       if ( pix % Obs_06 > maxcir06 ) then
          if ((pix % Obs_108-pix % Obs_120) > s_108_120_semi_above .and. sdlog_108 > sdlog_06 ) then
               CT % cld_type = CT_SEMITR_THIN
               num_test =9
          else
               CT % cld_type = CT_LOW  
               num_test =10
          endif
       else 
            if ( pix % Obs_87-pix % Obs_108 > s_87_108_edge ) then
               CT % cld_type = CT_SEMITR_THIN
               num_test =11
            elseif (.not. pix % Qual_87 .and. (pix % Obs_108-pix % Obs_120) > s_108_120_semi_above) then
               CT % cld_type = CT_SEMITR_THIN
               num_test =11
            else
               CT % cld_type = CT_FRACTIONAL        
               num_test =12
            endif
       endif
  
  else if ( pix % Obs_108 < max108_thin ) then
       if ( pix % Obs_06 > maxcir06 ) then
            if ( pix % Obs_108-pix % Obs_120 < s_108_120_verylow ) then
               CT % cld_type = CT_VLOW
               num_test =13
            else if (pix % Obs_87-pix % Obs_108 > s_87_108_edge) then
               CT % cld_type = CT_SEMITR_THIN
               num_test =14
            else 
               CT % cld_type = CT_FRACTIONAL
               num_test =15
            endif
       else 
            if ( pix % Obs_87-pix % Obs_108 > s_87_108_edge ) then
               CT % cld_type = CT_SEMITR_THIN
               num_test =16
            else
               CT % cld_type = CT_FRACTIONAL
               num_test =17
            endif
       endif
  else 
         if ( pix % Obs_06 > thres % v_CT_minlow_06 ) then
            if ( pix % Obs_108-pix % Obs_120 < s_108_120_verylow ) then
               CT % cld_type = CT_VLOW
               num_test =18
            else if (pix % Obs_87-pix % Obs_108 > s_87_108_edge) then
               CT % cld_type = CT_FRACTIONAL
               num_test =19
            else
               CT % cld_type = CT_FRACTIONAL
               num_test =20
            endif
         else 
            if ( pix % Obs_87-pix % Obs_108 > s_87_108_edge ) then
               CT % cld_type = CT_FRACTIONAL
               num_test =21
            else
               CT % cld_type = CT_FRACTIONAL
               num_test =22
            endif
         endif
  endif

!   if thermal inversion in NWP possibility to reclassify medium in low  
!   use T87-T108 as a security
  if( box % inversion_NWP ) then

      if(pix % Obs_108 >= thres % v_CT_max108_lo) then
            if(CT % cld_type  == CT_MED ) CT % cld_type  = CT_LOW
            num_test =23

      else 
            if( s_37_108_medcld0 > vmanq ) then

               if( pix % Obs_87-pix % Obs_108 < s_37_108_medcld0 .and. &
                   pix % Obs_108 >= (thres % v_CT_max108_me - CT_offset_MeLow_InvMax0) ) then
                  if(CT % cld_type  == CT_MED ) CT % cld_type  = CT_LOW
                  num_test =24

               else if( pix % Obs_87-pix % Obs_108 < s_37_108_medcld1 .and. &
                        pix % Obs_108 >= (thres % v_CT_max108_me - CT_offset_MeLow_InvMax1) ) then
                  if(CT % cld_type  == CT_MED ) CT % cld_type  = CT_LOW
                  num_test =25

               else if( pix % Obs_87-pix % Obs_108 < s_37_108_medcld2 .and. &
                        pix % Obs_108 >= (thres % v_CT_max108_me - CT_offset_MeLow_InvMax2) )then
                  if(CT % cld_type  == CT_MED ) CT % cld_type  = CT_LOW
                  num_test =26
               endif  
         
            else
         
               if( pix % Obs_108 >= (thres % v_CT_max108_me - CT_offset_MeLow_InvMax00) ) then
                  if(CT % cld_type  == CT_MED ) CT % cld_type  = CT_LOW
                  num_test =27

               endif
            endif 
         endif  !test on Obs_108 with max108lo
  endif  !test on inversion

  if(idbg%debug) then
     if(CT % cld_type == CT_FRACTIONAL) then
        write(*,'(a20,i5,3f10.2)') 'FRACTIONAL: s_87_108_edge', num_test, s_87_108_edge, pix % Obs_87-pix % Obs_108
        write(*,'(10f10.2)') pix % Obs_108, thres % v_CT_max108_vh, thres % v_CT_max108_hi, &
                    thres % v_CT_max108_me,thres % v_CT_max108_lo, max108_thin, thres % v_CT_minlow_06
     elseif (CT % cld_type == CT_LOW) then
        write(*,'(a20,i5)') 'LOW: Num_test', num_test
     elseif (CT % cld_type == CT_SEMITR_THICK) then
        write(*,'(a25,i5,2f10.2)') 'SEMITR_THICK: Num_test', num_test, s_108_120_thick, pix % Obs_108-pix % Obs_120
     elseif (CT % cld_type == CT_VLOW) then
        write(*,'(a25,i5,2f10.2)') 'CT_VLOW: Num_test', num_test, s_108_120_verylow, pix % Obs_108-pix % Obs_120
     else
     endif
  endif
      
END SUBROUTINE maia_CT_day
