! Name: $Id: maia_CT_night.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_CT_night 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_CT_night (idbg, box, pix_id, pix, thres, CT)

! Set Cloud Type in night condition

! Authors       : lydie lavanant

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ),      INTENT(in)        :: idbg         ! 
  type( box_id ),     INTENT(in)        :: box          ! info at the center of the box
  type( pix_info),    INTENT(in)        :: pix_id       ! lat, lon, solar and satellite angles at the pixel
  type( pix_data),    INTENT(in)        :: pix          ! pix observations (albedo in %, Tb in K)
  type( maia_thres), INTENT(in)         :: thres        ! 
  type( maia_CT),    INTENT(out)        :: CT   ! 

! local
  REAL          :: t108120_thick
  REAL          :: maxt108_thin, t37108_thin_high, t37108_thin_low
  REAL          :: t37108_vlow, t37108_low
  REAL          :: delta_37_108_thin_high, delta_37_108_thin_low
  LOGICAL       :: use_37
  REAL          :: t108inter, value
  REAL          :: s_87_108_edge, s_87_108_medcld0, s_87_108_medcld1, s_87_108_medcld2
  integer       :: num_test
! ---------------------------------------------------------------------------       
  use_37=(pix % Obs_37 > limit_37)
  if(pix_id% ImgName == 'avhrr' .and. pix_id % sunzen < cst_sunzen_for_use37_avhrr) then 
     use_37=.false.
  endif
      
  t108inter = thres% v_CT_max108_lo + (thres% v_CT_max108_hi - thres% v_CT_max108_lo)/2.
  if(thres% v_CT_max108_me < t108inter) then
     t108inter = thres% v_CT_max108_me + (thres% v_CT_max108_hi - thres% v_CT_max108_me)/2.
  endif
  
! 108_120 thresholds
! -------------
  t108120_thick =  thres% v_CT_108_120_opaq - CT_offset_108_120_opaq
  if ( t108120_thick > CT_108_120_opaq_max ) t108120_thick = CT_108_120_opaq_max
  
! IR threshold where semi transparent are too thin
! ------------------------------------------------
  if(box % sea) then                                            ! sea
     maxt108_thin= thres% v_CT_max108_lo + thres % v_CMa_108_120_sea
  else                                                          ! land or coast
     maxt108_thin= thres% v_CT_max108_lo + thres % v_CMa_108_120_land
  endif

! 37_108 thresholds for semi-transparent clouds depending on M15
! ---------------------------------------------------------
  t37108_low  = thres% v_CT_37_108_opaq - CT_37_108_low_delta
  t37108_vlow = thres% v_CT_37_108_opaq + CT_37_108_low_delta                      

  if(thres% v_CT_max108_lo /= thres% v_CT_max108_hi) then 
     value=(thres% v_CT_max108_lo - pix % Obs_108) / (thres% v_CT_max108_lo - thres% v_CT_max108_hi)
     delta_37_108_thin_high = CT_37_108_thin_high_factor * value  
     delta_37_108_thin_low  = CT_37_108_thin_low_factor * value
  else
     delta_37_108_thin_high = CT_37_108_thin_high_factor/2.
     delta_37_108_thin_low  = CT_37_108_thin_low_factor/2.
  endif                    

  t37108_thin_high= thres% v_CT_37_108_opaq + delta_37_108_thin_high
  t37108_thin_low=  t37108_vlow + delta_37_108_thin_low

  if ( t37108_thin_high < CT_37_108_semi_max ) t37108_thin_high= CT_37_108_semi_max
                                           
! 87_108
! -------
  s_87_108_edge = thres % v_CT_87_108_opaq
  if( pix % Obs_06 > CT_minalb_arid .and.  box % land ) s_87_108_edge = vmanq

  s_87_108_medcld0= vmanq
  if( pix % Obs_06 < CT_37_108_medclouds_maxalb .or.  box % sea ) then
    s_87_108_medcld0= CT_37_108_medclouds_min_0+ CT_37_108_medclouds_min_slope0*(box % satsec-1)
    s_87_108_medcld1= CT_37_108_medclouds_min_1+ CT_37_108_medclouds_min_slope1*(box % satsec-1)
    s_87_108_medcld2= CT_37_108_medclouds_min_2+ CT_37_108_medclouds_min_slope2*(box % satsec-1)
  endif

  ! ----------
  ! test serie
  ! ----------

  ! very high clouds according to M15
  if ( pix % Obs_108 < thres % v_CT_max108_vh ) then
     if ( pix % Obs_108-pix % Obs_120 < t108120_thick) then
        CT % cld_type  = CT_VH_OPAQ
        num_test =1
     else         
        CT % cld_type  = CT_SEMITR_THICK
        num_test =2
     endif
      
  ! high clouds according to M15
  else if ( pix % Obs_108 < thres % v_CT_max108_hi ) then
     if ( pix % Obs_108-pix % Obs_120 < t108120_thick) then
        CT % cld_type  = CT_HI_OPAQ
        num_test =3
     else 
        CT % cld_type  = CT_SEMITR_THICK
        num_test =4
     endif

  ! medium clouds according to M15 but high 37_108 values expected 
  ! for coldest semi-transparent clouds
  else if ( pix % Obs_108 < t108inter ) then

     if ( .not. use_37 ) then
        if ( pix % Obs_108-pix % Obs_120 < t108120_thick ) then
            CT % cld_type  = CT_MED
            num_test =5
        else 
            CT % cld_type  = CT_SEMITR_MEAN
            num_test =6
        endif
     elseif ( pix % Obs_37-pix % Obs_108 < t37108_thin_high .and. &
              pix % Obs_108-pix % Obs_120 < t108120_thick ) then
            CT % cld_type  = CT_MED
            num_test =7
     else
            CT % cld_type  = CT_SEMITR_MEAN
            num_test =8
     endif
  
  ! medium clouds according to M15 , but lower 87_108 values expected 
  ! for semi-transparent clouds
  else if ( pix % Obs_108 < thres % v_CT_max108_me ) then

     if ( .not. use_37 ) then
        if ( pix % Obs_108-pix % Obs_120 < t108120_thick) then
           CT % cld_type  = CT_MED
           num_test =9
        else
           CT % cld_type  = CT_SEMITR_MEAN
           num_test =10
        endif
     elseif ( pix % Obs_37-pix % Obs_108 < t37108_thin_low .and. &
              pix % Obs_108-pix % Obs_120 < t108120_thick) then
        CT % cld_type  = CT_MED
        num_test =11
     else
        CT % cld_type  = CT_SEMITR_MEAN
        num_test =12
     endif
     
  ! low clouds according to M15, but lower 87_108 values expected 
  ! for  semi-transparent clouds
  else if ( pix % Obs_108 <  thres % v_CT_max108_lo ) then
       
     if ( .not. use_37 ) then
       if (pix % Obs_108-pix % Obs_120 < t108120_thick) then
          CT % cld_type  = CT_LOW
          num_test =13
       else if ( pix % Obs_87-pix % Obs_108 > s_87_108_edge ) then
          CT % cld_type  = CT_SEMITR_THIN
          num_test =14
       else
          CT % cld_type = CT_FRACTIONAL
          num_test =15
       endif
       
     else if ( pix % Obs_37-pix % Obs_108 < t37108_low .and. &
               pix % Obs_108-pix % Obs_120 < t108120_thick) then
        CT % cld_type  = CT_LOW
        num_test =16

     else if ( pix % Obs_87-pix % Obs_108 >  s_87_108_edge) then
        CT % cld_type  = CT_SEMITR_THIN
        num_test =17
       
     elseif ( pix % Obs_37-pix % Obs_108 > t37108_thin_low ) then
        CT % cld_type  = CT_SEMITR_THIN                  
        num_test =18
     
     else         
        CT % cld_type = CT_FRACTIONAL
        num_test =19
     endif
  
  ! very low clouds according to M15, but low 87_108 values expected 
  ! for  semi-transparent clouds
  elseif ( pix % Obs_108 < maxt108_thin ) then
     
     if  (pix % Obs_108-pix % Obs_120 > t108120_thick .or. pix % Obs_37-pix % Obs_108 > t37108_vlow) then
        
        if( pix % Obs_87-pix % Obs_108 > s_87_108_edge ) then
               CT % cld_type  = CT_SEMITR_THIN                  
               num_test =20
        elseif ( pix % Obs_37-pix % Obs_108 > t37108_thin_low ) then
               CT % cld_type  = CT_SEMITR_THIN                  
               num_test =21
        else
               CT % cld_type  = CT_FRACTIONAL
               num_test =22
        endif
     else
        CT % cld_type  = CT_VLOW
        num_test =23
     endif      
  
  ! warmer very low clouds according to M15, only very low or fractionnal clouds
  else

     if (pix % Obs_108-pix % Obs_120 > t108120_thick .or. pix % Obs_37-pix % Obs_108 > t37108_vlow) then
         if( pix % Obs_87-pix % Obs_108 > s_87_108_edge ) then
             CT % cld_type  = CT_FRACTIONAL
             num_test =24
         else
             CT % cld_type  = CT_FRACTIONAL
             num_test =25
         endif
     else
        CT % cld_type  = CT_VLOW
        num_test =26
     endif

  endif
 

!  IF thermal inversion in NWP  possibility to reclassify medium in low 
!  use T87-T108 as a security
! ---------------------------------------------------------------------

  if( box % inversion_NWP ) then

      if(pix % Obs_108 >= thres % v_CT_max108_lo) then
         if(CT % cld_type  == CT_MED ) then
            CT % cld_type  = CT_LOW
            num_test =27
         endif

      else 
         if( s_87_108_medcld0 > vmanq ) then

            if( pix % Obs_87-pix % Obs_108 < s_87_108_medcld0 .and. &
                pix % Obs_108 >= (thres % v_CT_max108_me - CT_offset_MeLow_InvMax0) ) then
                if(CT % cld_type  == CT_MED ) then
                   CT % cld_type  = CT_LOW
                   num_test =28
                endif

            else if( pix % Obs_87-pix % Obs_108 < s_87_108_medcld1 .and. &
                     pix % Obs_108 >= (thres % v_CT_max108_me - CT_offset_MeLow_InvMax1) ) then
                if(CT % cld_type  == CT_MED ) then
                   CT % cld_type  = CT_LOW
                   num_test =29
                endif

            else if( pix % Obs_87-pix % Obs_108 < s_87_108_medcld2 .and. &
                     pix % Obs_108 >= (thres % v_CT_max108_me - CT_offset_MeLow_InvMax2) )then
                if(CT % cld_type  == CT_MED ) then
                   CT % cld_type  = CT_LOW
                   num_test =30
                endif
            endif  
         
         else
         
            if( pix % Obs_108 >= (thres % v_CT_max108_me - CT_offset_MeLow_InvMax00) ) then
                if(CT % cld_type  == CT_MED ) then
                   CT % cld_type  = CT_LOW
                   num_test =1
                endif
            endif
         endif 
      endif  !test on Obs_108 with max108lo
  endif  !test on inversion


  if(idbg%debug) then
     if(CT % cld_type == CT_FRACTIONAL) then
        write(*,'(a20,i5,10f10.2)') 'FRACTIONAL: Num_test', num_test, pix % Obs_87-pix % Obs_108, &
                 pix % Obs_108-pix % Obs_120, t108120_thick ,pix % Obs_37-pix % Obs_108 , t37108_low, t37108_vlow
        write(*,'(10f10.2)') pix % Obs_108, thres % v_CT_max108_vh, thres % v_CT_max108_hi, &
                    thres % v_CT_max108_me,thres % v_CT_max108_lo
     elseif (CT % cld_type == CT_LOW) then
        write(*,'(a20,i5,10f10.2)') 'LOW: Num_test', num_test, pix % Obs_37-pix % Obs_108, t37108_low, &
        pix % Obs_108-pix % Obs_120, t108120_thick
     elseif (CT % cld_type == CT_SEMITR_THICK) then
        write(*,'(a25,i5,2f10.2)') 'SEMITR_THICK: Num_test', num_test, pix % Obs_108-pix % Obs_120
     elseif (CT % cld_type == CT_VLOW) then
        write(*,'(a25,i5,2f10.2)') 'CT_VLOW: Num_test', num_test,  pix % Obs_108-pix % Obs_120
     else
     endif
  endif

END SUBROUTINE maia_CT_night
