! Name: $Id: maia_Cal_Fresnel.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_Cal_Fresnel 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE  maia_Cal_Fresnel (nr,ni,coschi, sinchi, R1)

! f90 conversion : lydie lavanant       04/06/02

! to compute the Fresnel's coefficient of reflection (see for
! example M. Born and E. Wolf, Principles of Optics, Pergamon Press, fifth
! edition, 1975, pp 628
! input parameters: 
! nr=index of refraction of the sea water
! ni=extinction coefficient of the sea water
! coschi & sinchi=cosine and sine of the incident radiation  with respect of the wave facet normal.
! output parameter: R1=Fresnel's coefficient for reflection

IMPLICIT NONE

  REAL, INTENT(in) :: nr,ni,coschi,sinchi
  REAL, INTENT(out) :: R1
  REAL ::  a1,a2,u,v,Rr2,Rl2,b1,b2
      
! absolute value for a1 to get v=0 when ni=0
      a1=abs(nr*nr-ni*ni-sinchi*sinchi)
      a2=sqrt((nr*nr-ni*ni-sinchi*sinchi)**2.+4*nr*nr*ni*ni)
      u=sqrt(0.5*(a1+a2))
      v=sqrt(0.5*(-a1+a2))
      Rr2=((coschi-u)**2+v*v)/((coschi+u)**2+v*v)
      b1=(nr*nr-ni*ni)*coschi
      b2=2*nr*ni*coschi
      Rl2=((b1-u)**2+(b2+v)**2)/((b1+u)**2+(b2-v)**2)
      R1=(Rr2+Rl2)/2.
      
END SUBROUTINE maia_Cal_Fresnel

