! Name: $Id: maia_Cal_LeRoux.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: computes the reflectance of the snow  (thesis of Le Roux)
!          Subroutines maia_Cal_LeRoux 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/11/2015   Pascale Roquet Bug fix if diffsat=180
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_Cal_LeRoux(idbg, box, ic,reflec, albmax)

! f90 conversion : lydie lavanant       04/06/02

!  (thesis of Le Roux). https://tel.archives-ouvertes.fr/tel-00781634

 USE MOD_MAIA_CONST
 USE MOD_maia_types

IMPLICIT NONE

  type( debug ),    INTENT(in)  :: idbg         ! 
  type( box_id ),   INTENT(in)  :: box          ! info at the center of the box
  INTEGER, INTENT(in)   :: ic
  REAL, intent(in)      :: reflec(nbreflecsol,nbreflecsat,nbreflecazi,3)
  INTEGER, INTENT(out)  :: albmax       !in %

! local variables
  REAL :: rphaz
  REAL :: difsol,difsat,difaz
  REAL :: calculreflec
  INTEGER :: isol,isat,iazi
  INTEGER :: ksat,ksol,kazi

  REAL :: satzen_loc    ! satellite zenith angle  (deg)
  REAL :: solzen_loc    ! solar zenith angle      (deg)
  
!==========================================================================
  if(idbg%debug) then
    write(*,*) "maia_Cal_LeRoux"  
  endif
  albmax = 0

! Test on angles 
  if(box % sunzen > 89. .or. box % sunzen < 0..or.      &
     box % satzen > 89. .or. box % satzen < 0..or.      &
     abs(box % difazi) > 360.) then
     albmax=-9999
     return
  endif

! computes the difference of azimut between 0 and 180
  rphaz = abs(box % difazi)
  if(rphaz > 180) rphaz = 360.-rphaz
  if(rphaz < reflect_azimin)   rphaz = reflect_azimin
  if(rphaz >= reflect_azimax)   rphaz = reflect_azimax -0.1
  satzen_loc=box % satzen
  solzen_loc=box % sunzen
  if(satzen_loc <  reflect_satmin)  satzen_loc = reflect_satmin
  if(satzen_loc >= reflect_satmax)  satzen_loc = reflect_satmax -0.1
  if(solzen_loc <  reflect_solmin)  solzen_loc = reflect_solmin
  if(solzen_loc >= reflect_solmax)  solzen_loc = reflect_solmax -0.1

! compute indices for reflection
  isol =   (solzen_loc-reflect_solmin)/reflect_solpas + 1
  difsol = (solzen_loc-reflect_solmin-(isol-1)*reflect_solpas)/reflect_solpas 
  isat =   (satzen_loc-reflect_satmin)/reflect_satpas + 1
  difsat = (satzen_loc-reflect_satmin-(isat-1)*reflect_satpas)/reflect_satpas 
  iazi =  (rphaz-reflect_azimin)/reflect_azipas + 1
  difaz = (rphaz-reflect_azimin-(iazi-1)*reflect_azipas)/reflect_azipas

  calculreflec = 0.
  do ksol=0,1,1
  do ksat=0,1,1
  do kazi=0,1,1
     calculreflec = calculreflec +                              &
                    reflec (ksol+isol,ksat+isat,kazi+iazi,ic) * &
                    ((1-difsat)*(1-ksat)+difsat*ksat) *         &
                    ((1-difsol)*(1-ksol)+difsol*ksol) *         &
                    ((1-difaz) *(1-kazi)+difaz*kazi) 
  enddo
  enddo
  enddo

! in %
  albmax = 100*calculreflec

END SUBROUTINE maia_Cal_LeRoux
