! Name: $Id: maia_Cal_Roujean.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_Cal_Roujean 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_Cal_Roujean (idbg, box, brdf)

! Authors       : HLeGleau, lydie lavanant

! computes the Roujean function (brdf des sols).
! the reference albedo (in %) is used to define the coefficients for the model

 USE MOD_MAIA_CONST
 USE MOD_maia_types

IMPLICIT NONE

  type( debug ),    INTENT(in)  :: idbg         ! 
  type( box_id ),   INTENT(in)  :: box          ! infos at the center of the box
  REAL ,            INTENT(out) ::  brdf

  REAL :: albedo
  REAL :: rphaz
  REAL :: cossol, cossat, cosphi,sinphi, ksi,cosksi,sinksi,tansol,tansat
  REAL :: f1,f2
  REAL :: k0,k1,k2

!================================================================================        
  if(idbg%debug) then
    write(*,*) "maiaCal_Roujean"  
  endif    
! Test on angles 
  if(box % sunzen > 89. .or. box % sunzen < 0..or.&
     box % satzen > 89. .or. box % satzen < 0..or. abs(box % difazi) > 360.) then
     brdf=vmanq
     return
  endif

!  k0, k1 et k2 values function of albedo
      if(box % albedo_clim <= 20.) then
        k0 = k0veget
        k1 = k1veget
        k2 = k2veget
      else if(box % albedo_clim > 40.) then
        k0 = k0desert
        k1 = k1desert
        k2 = k2desert
      else
        albedo = box % albedo_clim
        k0 = k0desert*((albedo-20.)/20.) +  k0veget*(1.-(albedo-20.)/20.) 
        k1 = k1desert*((albedo-20.)/20.) +  k1veget*(1.-(albedo-20.)/20.) 
        k2 = k2desert*((albedo-20.)/20.) +  k2veget*(1.-(albedo-20.)/20.)
      endif

! computes angular values   
      cossol = cos(cdr*box % sunzen)
      cossat = cos(cdr*box % satzen)
      tansol = tan(cdr*box % sunzen)
      tansat = tan(cdr*box % satzen)

      rphaz = abs(box % difazi)
      cosphi = cos(cdr*rphaz)
      sinphi = sin(cdr*rphaz)

      cosksi = box % cos_scat_ang
      ksi = acos(cosksi)
      sinksi = sin(ksi)

      f1 = (1./(2.*pi))*((pi-cdr*rphaz)*cosphi+sinphi)*tansol*tansat - &
           (1./pi)*( tansol+tansat + sqrt(tansol**2+tansat**2-2*tansol*tansat*cosphi) ) 

      f2 = 4./(3.*pi*(cossol+cossat))*((pi/2.-ksi)*cosksi+sinksi)-1./3.

      brdf = k0*(1.+k1*f1+k2*f2)

END SUBROUTINE maia_Cal_Roujean
