! Name: $Id: maia_Cal_Texture.F90 548 2017-08-30 09:28:19Z frna $
!
! Purpose: 
!          Subroutines maia_Texture_FromImager maia_Texture_FromNeighbors 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.3    23/12/2016   Lydie Lavanant bug fixes
!  1.4    28/08/2017   Pascale Roquet bug fix div by 0
SUBROUTINE maia_Texture_FromImager (idbg, pix, lig, box, field_I, pix_id)

! computes the local texture (std + max diff) inside the Moderate resolution pixel
! from the coregistered imaging channels

! Authors       : lydie lavanant

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

  type( debug ),  INTENT(in)    :: idbg          
  integer, intent(in)           :: pix, lig
  type( box_id ), intent(in)    :: box          
  Type (field),intent(in)       :: field_I
  type( pix_info), INTENT(inout):: pix_id        

! local
  integer       :: pixI, ligI, pix_ifov, lig_ifov, ifov
  real          :: mean_06, mean_08, mean_108
  real          :: pixImg(4,5)
  logical       :: Qual_I(5)
  REAL(KIND=8)  :: x, mean, mean_carre
  integer       :: ic
!=======================================================
  if( idbg % debug ) then
     write(*,*) '  -----------------------------------------------------------------------'
     write(*,*) '  Texture InPix :'
  endif

  pixI=2*(pix-1) +1
  ligI=2*(lig-1) +1
  Qual_I(:)  = .true.
  do ic=1, 5
  do ifov=1,4
     if(ifov==1 .or. ifov==3) pix_ifov=pixI
     if(ifov==2 .or. ifov==4) pix_ifov=pixI+1
     if(ifov==1 .or. ifov==2) lig_ifov=ligI
     if(ifov==3 .or. ifov==4) lig_ifov=ligI+1
     pixImg (ifov,ic) = field_I % Obs(ic,pix_ifov,lig_ifov)     ! pixel_I in moderate resolution
     if(field_I % Qual_Obs(ic,pix_ifov,lig_ifov)/=0) Qual_I(ic) = .false.
     if(.not. field_I % locqual(pix_ifov,lig_ifov) ) Qual_I(ic)  = .false.
  end do
  end do

  ! initialisations
  mean_06       = vmanq
  mean_08       = vmanq
  mean_108      = vmanq

  pix_id % texture_fromImager   = .true.        ! true: from viirs 2*2 imager in pixel
  pix_id % sd_06                = vmanq
  pix_id % sd_08                = vmanq
  pix_id % sd_108               = vmanq
  pix_id % sd_10837             = vmanq
  pix_id % diff_Max_06          = vmanq
  pix_id % diff_Max_108         = vmanq
  pix_id % Ratio_Max_10806      = vmanq
        
  IF (Qual_I(5)  .and. all(pixImg(:,5)>vmanq)) THEN
     mean   = sum(pixImg(:,5))/4.
     mean_carre = sum(pixImg(:,5)*pixImg(:,5))/4.
     x = mean_carre - mean * mean
     if (x >= 0.) then
         pix_id % sd_108 = sqrt(x)
     else if (x>-0.01) then
         pix_id % sd_108 = 0
     endif
     pix_id % diff_Max_108 =  MAXVAL(pixImg(:,5)) - MINVAL(pixImg(:,5))
     mean_108 = mean
  else
!     pix_id % texture_fromImager = .false.     ! problem in Imager field
  ENDIF
         
  IF (Qual_I(5) .and.  Qual_I(4) .and. all(pixImg(:,5)>vmanq) .and. all(pixImg(:,4)>vmanq) ) THEN
     mean       = sum( pixImg(:,5)-pixImg(:,4))/4.
     mean_carre = sum( (pixImg(:,5)-pixImg(:,4)) * (pixImg(:,5)-pixImg(:,4)) )/4.
     x = mean_carre - mean * mean
     if (x >= 0.) then
         pix_id % sd_10837 = sqrt(x)
     else if (x>-0.01) then
         pix_id % sd_10837 = 0
     endif
  else
!     pix_id % texture_fromImager = .false.     ! problem in Imager field
  ENDIF

  IF (.not. box%night) THEN
!  IF (box % day  .or. box % glint) THEN
     IF (Qual_I(1) .and. all(pixImg(:,1)>vmanq)) THEN
        mean    = sum(pixImg(:,1))/4.
        mean_carre = sum(pixImg(:,1)*pixImg(:,1))/4.
        x = mean_carre - mean * mean
        if (x >= 0.) then
            pix_id % sd_06 = sqrt(x)
        else if (x>-0.01) then
            pix_id % sd_06 = 0
        endif
        pix_id % diff_Max_06 =  MAXVAL(pixImg(:,1)) - MINVAL(pixImg(:,1))
        mean_06 = mean
     ENDIF

     IF (Qual_I(2) .and. all(pixImg(:,2)>vmanq)) THEN
        mean    = sum(pixImg(:,2))/4.
        mean_carre = sum(pixImg(:,2)*pixImg(:,2))/4.
        x = mean_carre - mean * mean
        if (x >= 0.) then
            pix_id % sd_08 = sqrt(x)
        else if (x>-0.01) then
            pix_id % sd_08 = 0
        endif
        mean_08 = mean
     ENDIF
                                             
     IF (Qual_I(1) .and. Qual_I(5) .and. all(pixImg(:,1)>vmanq) .and. all(pixImg(:,5)>vmanq) ) THEN
        pix_id % Ratio_Max_10806 = pix_id % diff_Max_108 / pix_id % diff_Max_06
     ENDIF
   endif

  if(idbg%debug) then
     write(*,*)  ' maia_Texture_FromImager:'
     do ic=1,5
        write(*,'(a10, i5, 4f10.3)') 'I: ', ic, pixImg(1:4,ic)
     end do
     write(*,'(a20,6f10.2)') 'Mean 1 2 4 5 : ', mean_06, mean_08, mean_108
     write(*,'(a20,6f10.2)') 'Sd 1 2 5 54: ',   pix_id%sd_06 , pix_id%sd_08, pix_id%sd_108, pix_id%sd_10837
     write(*,'(a20,8f10.2)') 'Diffs 1 2 4 5: ', pix_id%diff_Max_06, pix_id%diff_Max_108
     write(*,*)  ' texture_fromImager Qual:', pix_id % texture_fromImager
  endif

END SUBROUTINE maia_Texture_FromImager
!=====================================================================================
subroutine maia_Texture_FromNeighbors (idbg, pix, lig, field_1b, pixel_id, field_1b_prev, field_1b_next)
!=====================================================================================

USE MOD_MAIA_CONST
USE MOD_maia_types
implicit none                                                     

  type( debug ), intent(in)             :: idbg          
  integer, intent(in)                   :: pix, lig
  Type (field),intent(in)               :: field_1b
  type( pix_info),intent(inout)         :: pixel_id   
  Type (field),intent(in), optional     :: field_1b_prev
  Type (field),intent(in), optional     :: field_1b_next

  ! local
  REAL(KIND=8)  :: x,xmean, obsC_108, obsC_37
  REAL(KIND=8)  :: xsum108,xsumsq108,xsum10837,xsumsq10837
  REAL(KIND=8)  :: xsum06,xsumsq06,xsum08, xsumsq08
  REAL          :: dtmax108, dt108,dtmax06, dt06, dt10806
  real          :: Obs_06, Obs_08, Obs_37, Obs_108 
  integer       :: Qual_num108, Qual_num37, Qual_num08, Qual_num06
  INTEGER       ::  k, p, l 
  INTEGER       :: num06, num08, num37, num108
  INTEGER       :: nx06, nx08, nx108, nx10837
  logical       :: previous_granule, next_granule

  !channels InMaia:        04,05,06,08,13,16,22,37,40,87,108,120
  !numChan_InField1b_Viirs= 1, 2, 3, 4, 5, 6, 7, 8, 9,10, 11, 12
  !numChan_InField1b_Avhrr= 0, 0, 1, 2, 0, 3, 0, 4, 0, 0,  5,  6


  ! initialisations
  previous_granule = .false.
  if(present(field_1b_prev)) then
     previous_granule = .true.
  endif
  next_granule = .false.
  if(present(field_1b_next)) then
     next_granule = .true.
  endif
  
  Obs_108 = vmanq ; Obs_08 = vmanq ; Obs_06 = vmanq
  xsum108=0.    ; xsumsq108=0.
  xsum10837= 0. ; xsumsq10837= 0.
  xsum06= 0.    ; xsumsq06= 0.
  xsum08= 0.    ; xsumsq08= 0.
  dt108=0.      ; dt06=0.      ; dt10806=0.
  dtmax108=vmanq; dtmax06=vmanq; dt10806=vmanq
  nx06 =0 ; nx08 =0 ; nx108=0 ; nx10837 = 0

  pixel_id % texture_fromNeighbors      = .true.
  pixel_id%sd_M06       = vmanq
  pixel_id%sd_M08       = vmanq
  pixel_id%sd_M108      = vmanq
  pixel_id%sd_M10837    = vmanq
  pixel_id%diff_Max_M108   = vmanq
  pixel_id%diff_Max_M06    = vmanq
  pixel_id%diff_Max_M10806 = vmanq
  pixel_id%Ratio_Max_M10806= vmanq

  if(field_1b % ImgName == 'viirs') then
     num06 = 3
     num08 = 4 
     num37 = 8
     num108= 11
  elseif(field_1b % ImgName == 'avhrr') then
     num06 = 1
     num08 = 2 
     num37 = 4
     num108= 5
  else
     write(*,*) 'PB ImgName'
  endif

  do k=1,8
     p= field_1b % pix_texture(k,pix,lig)
     l= field_1b % lig_texture(k,pix,lig)
     
     if(p>imanq .and. l>imanq) then

     if(l==0) then
        if(previous_granule .and. &
          (field_1b%ij(1) - field_1b_prev%ij(field_1b_prev%Nb_Lines)  ) < 10000) then
           Obs_108= field_1b_prev%Obs(num108,p,field_1b_prev%Nb_Lines) 
           Obs_37 = field_1b_prev%Obs(num37,p,field_1b_prev%Nb_Lines)
           Obs_06 = field_1b_prev%Obs(num06,p,field_1b_prev%Nb_Lines)
           Obs_08 = field_1b_prev%Obs(num08,p,field_1b_prev%Nb_Lines)
           Qual_num108=field_1b_prev % Qual_Obs(num108,p,field_1b_prev%Nb_Lines)
           Qual_num37 =field_1b_prev % Qual_Obs(num37,p,field_1b_prev%Nb_Lines)
           Qual_num08 =field_1b_prev % Qual_Obs(num08,p,field_1b_prev%Nb_Lines)
           Qual_num06 =field_1b_prev % Qual_Obs(num06,p,field_1b_prev%Nb_Lines)
        else
           Obs_108= field_1b%Obs(num108,p,1)
           Obs_37 = field_1b%Obs(num37,p,1)
           Obs_06 = field_1b%Obs(num06,p,1)
           Obs_08 = field_1b%Obs(num08,p,1)
           Qual_num108=field_1b % Qual_Obs(num108,p,1)
           Qual_num37 =field_1b % Qual_Obs(num37,p,1)
           Qual_num08 =field_1b % Qual_Obs(num08,p,1)
           Qual_num06 =field_1b % Qual_Obs(num06,p,1)
        endif
     elseif(l==9999) then
        if(next_granule .and. (field_1b_next%ij(1) - field_1b%ij(field_1b%Nb_Lines) ) < 10000) then
           Obs_108= field_1b_next%Obs(num108,p,1)
           Obs_37 = field_1b_next%Obs(num37,p,1)
           Obs_06 = field_1b_next%Obs(num06,p,1)
           Obs_08 = field_1b_next%Obs(num08,p,1)
           Qual_num108=field_1b_next % Qual_Obs(num108,p,1)
           Qual_num37 =field_1b_next % Qual_Obs(num37,p,1)
           Qual_num08 =field_1b_next % Qual_Obs(num08,p,1)
           Qual_num06 =field_1b_next % Qual_Obs(num06,p,1)
        else
           Obs_108= field_1b%Obs(num108,p,field_1b%Nb_Lines)
           Obs_37 = field_1b%Obs(num37,p,field_1b%Nb_Lines)
           Obs_06 = field_1b%Obs(num06,p,field_1b%Nb_Lines)
           Obs_08 = field_1b%Obs(num08,p,field_1b%Nb_Lines)
           Qual_num108=field_1b % Qual_Obs(num108,p,field_1b%Nb_Lines)
           Qual_num37 =field_1b % Qual_Obs(num37,p,field_1b%Nb_Lines)
           Qual_num08 =field_1b % Qual_Obs(num08,p,field_1b%Nb_Lines)
           Qual_num06 =field_1b % Qual_Obs(num06,p,field_1b%Nb_Lines)
        endif
     else
        Obs_108= field_1b%Obs(num108,p,l)
        Obs_37 = field_1b%Obs(num37,p,l)
        Obs_06 = field_1b%Obs(num06,p,l)
        Obs_08 = field_1b%Obs(num08,p,l)
        Qual_num108=field_1b % Qual_Obs(num108,p,l)
        Qual_num37 =field_1b % Qual_Obs(num37,p,l)
        Qual_num08 =field_1b % Qual_Obs(num08,p,l)
        Qual_num06 =field_1b % Qual_Obs(num06,p,l)
     endif

     IF (Qual_num108==0 .and. Obs_108>vmanq) THEN
        obsC_108 = obs_108 -273.15
        xsum108   = xsum108   + ObsC_108
        xsumsq108 = xsumsq108 + ObsC_108*ObsC_108
        nx108 = nx108 +1
     endif
         
     IF (Qual_num108==0 .and. Qual_num37==0 .and. Obs_108>vmanq .and. Obs_37>vmanq) THEN
         obsC_108 = obs_108 -273.15
         obsC_37  = obs_37  -273.15
        xsum10837   = xsum10837   + (ObsC_108 - ObsC_37)
        xsumsq10837 = xsumsq10837 + (ObsC_108 - ObsC_37)*(ObsC_108 - ObsC_37)
        nx10837 = nx10837 +1
     endif
         
     IF (Qual_num06==0 .and. Obs_06>vmanq) THEN
        xsum06   = xsum06   + Obs_06
        xsumsq06 = xsumsq06 + Obs_06*Obs_06
        nx06 = nx06 +1
     endif

     IF (Qual_num08==0 .and. Obs_08>vmanq) THEN
        xsum08 = xsum08 + Obs_08
        xsumsq08 = xsumsq08 + Obs_08*Obs_08
        nx08 = nx08 +1
     endif
             
     IF (Qual_num108 ==0 .and. field_1b % Qual_Obs(num108,pix,lig)==0 .and. &
         Obs_108>vmanq .and. field_1b%Obs(num108,pix,lig)>vmanq) THEN
        dt108= abs(Obs_108 - field_1b%Obs(num108,pix,lig))
        if(dt108 > dtmax108) dtmax108= dt108     
     endif
    IF (Qual_num06 ==0 .and. field_1b % Qual_Obs(num06,pix,lig)==0 .and.&
         Obs_06>vmanq .and. field_1b%Obs(num06,pix,lig)>vmanq) THEN
        dt06 = abs(Obs_06 - field_1b%Obs(num06,pix,lig))
        if(dt06 > dtmax06) dtmax06= dt06
     endif
     IF (Qual_num108==0 .and. field_1b % Qual_Obs(num108,pix,lig)==0 .and.&
         Qual_num06==0  .and. field_1b % Qual_Obs(num06,pix,lig)==0 .and.&
         Obs_108>vmanq .and. field_1b%Obs(num108,pix,lig)>vmanq .and.&
         Obs_06>vmanq  .and. field_1b%Obs(num06 ,pix,lig)>vmanq) THEN
        if(Obs_06 - field_1b%Obs(num06,pix,lig) /=0) then
           dt10806= (Obs_108 - field_1b%Obs(num108,pix,lig)) /&
                    (Obs_06  - field_1b%Obs(num06 ,pix,lig))
        endif
     endif
     endif      !p>imanq .and. l>imanq
  end do
         
  if(nx108>0) then
     xmean = xsum108/nx108
     x = xsumsq108/nx108 - xmean*xmean
     if (x >= 0.) then
         pixel_id%sd_M108 = sqrt(x)
     else
         pixel_id%sd_M108 = 0
     endif
  endif
    
  if(nx10837>0) then
     xmean = xsum10837/nx10837
     x = xsumsq10837/nx10837 - xmean*xmean
     if (x >= 0.) then
         pixel_id%sd_M10837 = sqrt(x)
     else
         pixel_id%sd_M10837 = 0
     endif
  endif

  if(nx06>0) then
     xmean = xsum06/nx06
     x = xsumsq06/nx06 - xmean*xmean
     if (x >= 0.) then
         pixel_id%sd_M06 = sqrt(x)
     else
         pixel_id%sd_M06 = 0
     endif
  endif

  if(nx08>0) then
     xmean = xsum08/nx08
     x = xsumsq08/nx08 - xmean*xmean
     if (x >= 0.) then
         pixel_id%sd_M08 = sqrt(x)
     else
         pixel_id%sd_M08 = 0.
     endif
  endif

  if(dtmax108>vmanq) pixel_id%diff_Max_M108   = dtmax108
  if(dtmax06 >vmanq) pixel_id%diff_Max_M06    = dtmax06
  if(dt10806>vmanq)  pixel_id%diff_Max_M10806 = dt10806
  if(pixel_id % diff_Max_M108>vmanq .and. pixel_id % diff_Max_M06>vmanq .and.  pixel_id % diff_Max_M06 > 0.000001) then
     pixel_id % Ratio_Max_M10806 = pixel_id % diff_Max_M108 / pixel_id % diff_Max_M06
  endif
  
  if(idbg%debug) then
     write(*,*)  ' maia_Texture_FromNeighbors:'
     write(*,'(a20,10i5)') 'neighbors: ', field_1b % pix_texture(:,pix,lig)
     write(*,'(a20,6f10.2)') 'Sd 1 2 5 54: ',   pixel_id%sd_M06 , pixel_id%sd_M08, pixel_id%sd_M108, pixel_id%sd_M10837
     write(*,'(a20,8f10.2)') 'Diffs 1 2 4 5: ', pixel_id%diff_Max_M06, pixel_id%diff_Max_M108
  endif
end subroutine maia_Texture_FromNeighbors

