! Name: $Id: maia_Cal_Twvc.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_Cal_Twvc 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE  maia_Cal_Twvc (nbniv, pniv, psurf, hum_in, cwv)

! f90 conversion : lydie lavanant       04/06/02

! Computes the total water vapor content 
! from the specific humidity profile and the surface pressure

IMPLICIT NONE

  INTEGER, INTENT(in) :: nbniv
  REAL, INTENT(in)    :: pniv(nbniv)    ! pressure on levels            (hpa)
  REAL, INTENT(in)    :: hum_in(nbniv)  ! specific humidity on levels   (g/g)
  REAL, INTENT(in)    :: psurf          ! surface pressure              (hpa)
  REAL, INTENT(out)   :: cwv            ! total water vapor content     (g/cm2 = cm)

  INTEGER :: lev
  INTEGER :: firstlev
  REAL ::    uh2o, hval
  REAL ::    pres(nbniv)
  REAL ::    dp(nbniv)
  REAL ::    hum(nbniv) ! specific humidity on levels   (g/g)
        
! initialization
        pres(:)=pniv(:)
        hum(:)=hum_in(:)

! defines the first level
        firstlev=0
        do lev=1, nbniv
          if(pniv(lev).le. psurf .and. firstlev.eq.0) then
             firstlev=lev
             call maia_Interp_Plog (pniv,hum,psurf,hval,nbniv,1) 
             hum(lev)=hval
             pres(lev)=psurf
          endif
        end do 
        
       if(firstlev.ge.nbniv .or.firstlev.eq.0) then
          cwv  = -9999.
       else
       
        ! layers centered on pres levels
          dp(nbniv) = (pres(nbniv) + pres(nbniv-1)) /2.
          dp(firstlev) = (pres(firstlev) - pres(firstlev+1)) /2.
          do lev=firstlev+1,nbniv-1
          dp(lev) = abs ((pres(lev+1) + pres(lev)) /2. - &
                         (pres(lev-1) + pres(lev)) /2. )
          end do
          
        ! total water vapor content in kg/m2  
        ! (divide by 10 to obtain g/cm2 or precipitable centimeters units)

          uh2o = 0.
          do  lev=firstlev,nbniv
          uh2o = uh2o + hum(lev) * dp(lev) /9.81
          end do
        
          cwv = uh2o*100.       ! in kg/m2
          !cwv=  cwv/10.        ! in g/cm2 or cm
       end if

END SUBROUTINE  maia_Cal_Twvc
            
            

