! Name: $Id: maia_Cloud_Mask.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_Cloud_Mask 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_Cloud_Mask (idbg, box, pix_id , pix, thres, CMa)

!-----------------------------------------------------------------------
! Cloud mask
!       tm=0 : sea
!       tm=1 : coast
!       tm=2 : land
!-----------------------------------------------------------------------

! Authors       : lydie lavanant

USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ),       INTENT(in) :: idbg       ! 
  type( box_id ),      INTENT(in) :: box        ! info at the center of the box
  type( pix_info ),  INTENT(in) :: pix_id   ! lat, lon, solar and satellite angles at the pixel
  type( pix_data),   INTENT(in) :: pix  ! pix observations (albedo in %, Tb in K)
  type( maia_thres),  INTENT(inout) :: thres    ! 
  type( maia_CMa ), INTENT(out)  :: CMa
                           
!-----------------------------------------------------------------------
  if( idbg % debug ) then
     write(*,*) '  -----------------------------------------------------------------------'
     write(*,*) '  Cloud_Mask Box: day, glint, twilight, night, sea, land, coast, desert snow_always', &
                                   box % day, box % glint, box % twilight , box % night, &
                                   box % sea, box % land, box % coast, box % desert, box % snow_always
     write(*,*) '             Pix: day, glint, twilight, night, sea, land, coast, desert snow_always', &
                                   pix_id % day, pix_id % glint, pix_id % twilight, pix_id% night, &
                                   pix_id% sea, pix_id% land, pix_id% coast, pix_id% desert, pix_id% snow_always
  endif

  !    -------------------------------------------------------------------
  !    SEA
  !    -------------------------------------------------------------------
  if (pix_id % sea) then
      
      CMa % snowice_fl=0        
      !  Cloud detection over SeaIce surface
      if (.not. pix_id % night ) then
             CMa % test_name   = 'CMa_SID'        
             call maia_CMa_IceD (idbg, pix_id, pix, box, thres, CMa)                       
      endif
      !  Cloud detection over Sea
      if(CMa % snowice_fl==0 .or. CMa % thin_cirrus==1) then        
         if (pix_id % day) then                                    ! DAY
             CMa % test_name = 'CMa_SD'        
             call maia_CMa_SD (idbg, pix_id, pix, box, thres, CMa)                       
         else if (pix_id % glint) then                             ! SUNGLINT
             CMa % test_name = 'CMa_SG'        
             call maia_CMa_SG (idbg, pix_id, pix, box, thres, CMa)
         else if (pix_id % night) then                             ! NIGHT    
             CMa % test_name = 'CMa_SN'        
             call maia_CMa_SN (idbg, pix_id, pix, box, thres, CMa)
         else                                                   ! TWILIGHT
             CMa % test_name = 'CMa_ST'        
             call maia_CMa_ST (idbg, pix_id, pix, box, thres, CMa)
         endif
      else
         CMa % conf_clear = 3
      endif
      
  !    -------------------------------------------------------------------
  !    LAND
  !    -------------------------------------------------------------------
  else if (.not. pix_id%coast .and. (pix_id % land .or. pix_id % desert .or. pix_id % snow_always)) then

      CMa % snowice_fl=0        
      !  Cloud detection over Snow surface
      if ( .not. pix_id % night ) then
             CMa % test_name   = 'CMa_IceD'      
             call maia_CMa_IceD (idbg, pix_id, pix, box, thres, CMa)  
      endif

      if(CMa % snowice_fl==0 .or. CMa % thin_cirrus==1) then    
         !  Cloud detection over Land
         if (pix_id % day .or. pix_id % glint) then                   ! DAY or SUNGLINT
             CMa % test_name   = 'CMa_LD'        
             call maia_CMa_LD (idbg, pix_id, pix, box, thres, CMa)
         else if (pix_id % night) then                             ! NIGHT
             CMa % test_name   = 'CMa_LN'        
             call maia_CMa_LN (idbg, pix_id, pix, box, thres, CMa)
         else                                                   ! TWILIGHT
             CMa % test_name   = 'CMa_LT'        
             call maia_CMa_LT (idbg, pix_id, pix, box, thres, CMa)
         endif      
      else
         CMa % conf_clear = 3
      endif

  !    -------------------------------------------------------------------
  !    COAST
  !    -------------------------------------------------------------------
  else
      CMa % snowice_fl=0        
      !  Cloud detection over Snow surface
      if ( .not. pix_id % night ) then
          CMa % test_name   = 'CMa_IceD'        
          call maia_CMa_IceD (idbg, pix_id, pix, box, thres, CMa)                       
      endif

      !  Cloud detection over coast
      if(CMa % snowice_fl==0 .or. CMa % thin_cirrus==1) then        
         if (pix_id % day ) then                                   ! DAY
             CMa % test_name   = 'CMa_CD'       
            call maia_CMa_CD (idbg, pix_id, pix, box, thres, CMa)
         else if (pix_id % glint) then                             ! SUNGLINT
             CMa % test_name   = 'CMa_CG'        
            call maia_CMa_CG (idbg, pix_id, pix, box, thres, CMa)
         else if (pix_id % night) then                             ! NIGHT
             CMa % test_name   = 'CMa_CN'        
            call maia_CMa_LN (idbg, pix_id, pix, box, thres, CMa)
         else                                                   ! TWILIGHT
             CMa % test_name   = 'CMa_CT'        
            call maia_CMa_LT (idbg, pix_id, pix, box, thres, CMa)
         endif
      else
         CMa % conf_clear = 3
      endif
  end if

END SUBROUTINE maia_Cloud_Mask
