subroutine maia_Cloud_Pressure (idbg, box, pix_id, pix, t108_tcld, CT, CH)
!-----------------------------------------------------------------------
! owner         : MF/DP/CMS/R&D
! Authors       : lydie lavanant
!-----------------------------------------------------------------------

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ),     INTENT(in)    :: idbg      ! 
  type( box_id ),    INTENT(in)    :: box       ! info at the center of the box
  type( pix_info),   INTENT(in)    :: pix_id    ! lat, lon, solar and satellite angles at the pixel
  type( pix_data),   INTENT(in)    :: pix       ! pix observations (albedo in %, Tb in K)
  real, INTENT(in)                 :: t108_tcld(t_nb, sec_nb) ! tabulated threshold tables (nb_wv,nb_secant)
  type( maia_CT),    INTENT(in)    :: CT         
  type( maia_CH),    INTENT(out)   :: CH        ! 
                       
  integer :: cld
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.2    23/01/2017   Pascale Roquet remove unused variables
!-----------------------------------------------------------------------
! Cloudy Pixel processing
!-----------------------------------------------------------------------
  if( idbg % debug ) then
     write(*,*) '  -----------------------------------------------------------------------'
     write(*,*) '  Cloud_Pressure '
  endif
                  
  cld = CT % cld_type
  CH % Tcld = vmanq
  CH % Pcld = vmanq

  !! for opaq clouds
  if(pix % Obs_108> vmanq .and. &
    (cld==CT_VLOW .or. cld==CT_LOW .or. cld==CT_MED .or. cld==CT_HI_OPAQ .or. cld==CT_VH_OPAQ)) then

     call maia_CloudTopTemp (pix_id % satsec, pix % Obs_108, t108_tcld, CH % Tcld)      !! cloud top temperature
            
     call maia_CloudTopPres (idbg, cld, box, pix % Obs_108, CH % Tcld, CH % Pcld) 
     if(idbg % debug) write(*,*) 'maia_Cloud_Pressure: ', CH % Tcld, CH % Pcld
  endif    

END SUBROUTINE maia_Cloud_Pressure
                              
subroutine maia_CloudTopTemp (satsec, bt108, t108_tcld, CloudTopTemp)

!-----------------------------------------------------------------------
! owner         : MF/DP/CMS/R&D
! Authors       : lydie lavanant

! computation of the CloudTopTemp corrected with data in table
!-----------------------------------------------------------------------

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

  real, intent(in)  :: satsec, bt108
  real, intent(in)  :: t108_tcld(t_nb, sec_nb)
  real, intent(out) :: CloudTopTemp

  integer, parameter :: sec_maxval= 2.5
  real, parameter ::    sec_step=0.25
  integer, parameter :: sec_deb=1
  real, parameter ::    t_deb= 220.00
  real, parameter ::    t_step= 10.00
  real, parameter ::    t_maxval= 310.00
   
  integer :: it,isec
  real :: difsec,dift, toptemp              
  real :: satsec_loc

  satsec_loc = satsec

  ! Preparation for interpolation in secante
  if (satsec_loc  >  sec_maxval) satsec_loc=sec_maxval
  isec= 1 + (satsec_loc-sec_deb)/sec_step
  if (isec  >=  sec_nb) isec=sec_nb-1
  if (isec  <=  1) isec=1
  difsec = ( satsec_loc - sec_deb - (isec-1)*sec_step ) / sec_step  

  ! Preparation for interpolation in ratiative temperature
  it= 1 + (bt108-t_deb)/t_step
  if (it >= t_nb) it=t_nb-1
  if (it <= 1) it=1
  dift = (bt108 - t_deb - (it-1)*t_step)/t_step

  ! computation of cloud top temperature
  toptemp= (1.-difsec) * (1.-dift) * t108_tcld(it,   isec)      &
         +  difsec     * dift      * t108_tcld(it+1, isec+1)    &
         + (1.-difsec) * dift      * t108_tcld(it+1, isec)      &
         + difsec      * (1.-dift) * t108_tcld(it,   isec+1)

  CloudTopTemp=bt108+toptemp  
 
end subroutine maia_CloudTopTemp

subroutine maia_CloudTopPres (idbg, CT, box, bt108, CloudTopTemp, CloudTopPres)
  
!-----------------------------------------------------------------------
! owner         : MF/DP/CMS/R&D
! Authors       : lydie lavanant

! computation of the CloudTopPres with a CloudTopTemp in input
! verify temperature inversion
!-----------------------------------------------------------------------

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

  type( debug )    :: idbg      ! 
  integer, INTENT(in) :: CT
  type( box_id ), INTENT(in)    :: box          ! info at the center of the box
  real, INTENT(in)      :: CloudTopTemp !
  real, INTENT(out)     :: CloudTopPres !

  integer       :: inver_type           ! 0=No ; 1=yes but not subsident ; 2= subident
  real          :: p_inver, t_inver, pcold_inver, tcold_inver, inver_t700
  real          :: p_mean, bt108
  integer       :: inver_l, inver_cl

  CloudTopPres = vmanq
      
  ! verify temperature inversion
  call Temp_Subsidence (idbg, box ,p_inver, t_inver, pcold_inver, tcold_inver, &
                        inver_l, inver_cl, inver_t700, inver_type)

  if(idbg%debug) write(*,*) 'inver_type : ', inver_type
  idbg%debug=.false.
  if(inver_type .eq. 1) then 
  idbg%debug=.true.
  endif

  if(inver_type.eq.2) inver_type=1
  
  ! cloud pressure associated with subsidence
  if(inver_type.eq.2 .and. (bt108>inver_t700) .and. &
                           (CT ==CT_VLOW .or. CT ==CT_LOW .or. CT ==CT_MED) ) then
     p_mean= (pcold_inver + p_inver)/2.
     CloudTopPres= p_inver- (p_inver- p_mean)* (t_inver-tcold_inver)/6.
     if(CloudTopPres>p_mean) CloudTopPres=p_mean

     if(idbg%debug) write(*,*) 'inver_type : ',pcold_inver, p_inver, p_mean
     if(idbg%debug) write(*,*) 'inver_type : ',p_inver, t_inver, tcold_inver, CloudTopPres
          
  else
     ! retrieve cloud pressure using vertical profile
     if(CloudTopTemp> vmanq) then
        call Temp_2Pres (idbg, box,  inver_type, p_inver, t_inver, pcold_inver, tcold_inver, &
                         inver_l, inver_cl, inver_t700, CloudTopTemp, CloudTopPres)
     endif 
  endif
  if(inver_type .gt. 0) idbg%debug=.false.

end subroutine maia_CloudTopPres


subroutine Temp_2Pres (idbg, box, inver_type, p_inver, t_inver, pcold_inver, tcold_inver, &
                       inver_l, inver_cl, inver_t700, Tcld, Pcld)                

!-----------------------------------------------------------------------
! owner         : MF/DP/CMS/R&D
! Authors       : lydie lavanant

! computation of the CloudTopPres with vertical profile from surface to tropopause 
!-----------------------------------------------------------------------

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

  type( debug )    :: idbg      ! 
  real, intent(in)             :: Tcld
  integer , intent(in)         :: inver_type! 0=No ; 1=yes but not subsident ; 2= subident
  type( box_id ),INTENT(in)    :: box       ! info at the center of the box
  real, intent(in)              :: p_inver, t_inver, pcold_inver, tcold_inver, inver_t700
  integer , intent(in)         :: inver_l, inver_cl
  real, intent(inout)          :: Pcld

  integer :: l_bottom,l_top, l, pos(1)
  real    :: p_mean, p_1, p_2

  ! rough estimation of tropopause from profile
  pos=minloc(box % t_rttov(21:29))+20
  l_top= pos(1)+1

  !nearest level above surface
  Do l = n_rttov - 1, 1, -1
     If ( box % ps_prev > p_rttov( l ) ) then
        Exit
     endif
  End Do
  l_bottom = l + 1

  !vertical profile is examined from surface to tropopause  
  if(Tcld <= box % t_rttov(l_top) ) then
     Pcld=p_rttov(l_top)
  else if(Tcld >= box % t2m_prev  ) then
     Pcld=box % ps_prev
  else
     ! search of the pressure between surface and tropopause
     l=l_bottom
     do while( l>l_top .and. (Tcld- box % t_rttov(l))*(Tcld- box % t_rttov(l-1)) > 0 )
        l=l-1      
     end do

     if(p_rttov(l) <= p_rttov(l_top) .or.  abs(Tcld - box % t_rttov(l)) > 40.) then
       Pcld= vmanq
     else
       Pcld=alog(p_rttov(l-1)) + (Tcld- box % t_rttov(l-1)) *&
            alog(p_rttov(l)/p_rttov(l-1)) / (box % t_rttov(l)-box % t_rttov(l-1))
       Pcld=exp(Pcld)
     endif
  endif
          
  !if inversion is present with input temperature colder than coldest part below inversion-5
  if (inver_type .ge. 1 .and. (Tcld < tcold_inver - 5.)) then
      ! the retrieved pressure is then combined with a possible pressure below inversion
                    
         ! profile is examined from top inversion to tropopause
         l_bottom=inver_l-1     
         if(Tcld <= box % t_rttov(l_top-1)) then
            p_1=p_rttov(l_top-1)
         else if(Tcld >= box % t_rttov(l_bottom-1)) then
            p_1=p_rttov(l_bottom-1)
         else
            l=l_bottom-1
            do while( (l>l_top-1) .and.((Tcld-box % t_rttov(l))*(Tcld-box % t_rttov(l-1)) > 0) )
               l= l-1
            end do
            if(abs((box % t_rttov(l)-box % t_rttov(l-1))) <= 0) then
                  p_1=(p_rttov(l)+p_rttov(l-1))/2.
            else
               p_1=alog((p_rttov(l-1))) + &
                  (Tcld-box%t_rttov(l-1))*alog(p_rttov(l)/p_rttov(l-1))/(box%t_rttov(l)-box%t_rttov(l-1))
               p_1=exp(p_1)
            endif
         endif   

         ! the retrieved pressure is then combined with a possible pressure below inversion
         p_mean = pcold_inver*0.5+p_inver*0.5
         p_2 = p_inver - (p_inver - p_mean)*((t_inver - tcold_inver)/6.)
         if(p_2 > p_mean) p_2 = p_mean
         
         if(Tcld > inver_t700) then
            Pcld = p_2 + (p_1 - p_2)*((tcold_inver -5. - Tcld)/5.)
            if (Pcld < p_1) Pcld=p_1
         else
            Pcld=p_1
         endif
   endif
   
  !inversion is present with input temperature warmer than coldest part below inversion-5
  !retrieved pressure is between colder and warmer part of inversion 
  !  if strong inversion, closer to the colder part
  !  if weak inversion,  closer to the upper part of inversion  
!  if (inver_type .ge. 1 .and. (Tcld >= tcold_inver - 5.)) then
!      p_mean=pcold_inver*0.5+p_inver*0.5
!      Pcld=p_inver-(p_inver-p_mean)*((t_inver - tcold_inver)/6.)
!      if(Pcld>p_mean) Pcld=p_mean          
      !if(idbg%debug) write(*,*) 'Temp_2Pres : ', Pcld
!  endif
     
end subroutine Temp_2Pres

subroutine Temp_Subsidence (idbg, box, inver_p, inver_t, inver_cp, inver_ct, &
                            inver_l, inver_cl, inver_t700, inver_type)

!-----------------------------------------------------------------------
! owner         : MF/DP/CMS/R&D
! Authors       : lydie lavanant

! computation of the CloudTopTemp corrected with data in table
!-----------------------------------------------------------------------

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

  type( debug ),     INTENT(in)    :: idbg      ! 
  type( box_id ),  INTENT(in)   :: box          ! info at the center of the box
  real, intent(out)             :: inver_p, inver_t, inver_cp, inver_ct, inver_t700
  integer, intent(out)          :: inver_type   !0=No ; 1=yes but not subsident ; 2= subident

  integer       :: l, nb_niv
  real          :: pres_tempe_maxi,pres_tempe_mini
  real          :: tempe_maxi,tempe_mini, t_600
  integer       :: ind_tempe_maxi,ind_tempe_mini, inver_l, inver_cl 
  logical       :: starting_level, is_subsident
  integer       :: level_min
  real          :: rh
   
  ! initialisations
  inver_p = vmanq ; inver_t    = vmanq ;  inver_cp = vmanq
  inver_ct= vmanq ; inver_t700 = vmanq
  inver_l = 0 ;  inver_cl = 0
  inver_type = 0

  !look for level where temperature starts to increase with height, starting from the surface
  nb_niv=51
  level_min=51
  starting_level=.true.
  do l=nb_niv,2,-1
     if( box % t_rttov(l-1) > box % t_rttov(l) .and. starting_level .and. p_rttov(l) >= 700. ) then
         level_min = l
         starting_level=.false.
     endif
  end do 

  ! look for maximum temperature of the vertical profile, starting at the level level_min
  tempe_maxi = box % t_rttov(level_min)
  ind_tempe_maxi = level_min
  pres_tempe_maxi=p_rttov(level_min)
  do l=level_min,1,-1
      if( box % t_rttov(l) >= tempe_maxi ) then
         ind_tempe_maxi = l
         pres_tempe_maxi = p_rttov(l)
         tempe_maxi = box % t_rttov(l)
      endif
  end do 
  if(idbg%debug) write(*,*) 'pres_tempe_maxi', level_min, tempe_maxi, ind_tempe_maxi, pres_tempe_maxi
   
  ! if maximum temperature found not too close to surface: thermal inversion
  if(ind_tempe_maxi >1 .and. ind_tempe_maxi < nb_niv .and. (p_rttov(nb_niv)-p_rttov(ind_tempe_maxi)) > 10. ) then

      inver_type = 1            !temperature inversion
      inver_p = pres_tempe_maxi !pressure (hPa) of top inversion
      inver_t = tempe_maxi      !temperature (K) of top inversion
      inver_l = ind_tempe_maxi  !level of top inversion

      ! look for coldest part of profile below inversion level
      tempe_mini = box % t_rttov(nb_niv)
      ind_tempe_mini = nb_niv
      pres_tempe_mini = p_rttov(nb_niv)
      do l=nb_niv, ind_tempe_maxi,-1
         if( box % t_rttov(l) <= tempe_mini ) then
            ind_tempe_mini = l
            tempe_mini = box % t_rttov(l)
            pres_tempe_mini = p_rttov(l)
         endif
      end do
      inver_cp = pres_tempe_mini        !pressure (hPa) of coldest part below inversion
      inver_ct = tempe_mini             !temperature (K) of coldest part below inversion
      inver_cl = ind_tempe_mini+1       !level of coldest part below inversion

      if(idbg%debug) write(*,*) 'inver_p', inver_p, inver_t, inver_l, inver_cp, inver_ct, inver_cl
      
      ! analyse whether inversion is subsident
      is_subsident=.true.
      do l=ind_tempe_maxi,1,-1
         rh = box % h_rttov(l)
         if( rh >= 30. .and.  p_rttov(l) >= 600. .and. p_rttov(l) <= 850. ) then
            is_subsident=.false.
         endif
      enddo  
      if (is_subsident) inver_type = 2  

      ! retain the warmest temperature between:
      ! temp at inversion_subsidente_pres_tmin: 600hPa and coldest temp below inversion-10.
      call maia_interp_Plog (p_rttov, box % t_rttov, 600., t_600, nb_niv, 1)
      if (inver_ct > vmanq) then
         inver_t700= inver_ct -10.              !typical temperature at pres_min (around 700hPa)
         if(t_600>inver_t700) inver_t700=t_600
      endif
!    endif
  endif
      
end subroutine Temp_Subsidence 

! ---------------------------------------------------------------------------------------
        real function ppmv_2q(v)
        ! to compute the volume mixing ratio for Water Vapour en ppmv
        ! for a given specific humidity
        real :: q                       ! specific humidity / air humide en g/g
        real, intent(in) :: v           ! rapport de melange volumique (sans unite)
        real :: vtmp
                
        Real, Parameter :: mh2o = 18.01528
        Real, Parameter :: mair = 28.9644
        Real, Parameter :: mo3  = 47.9982
        Real, Parameter :: eps =  mh2o / mair !(mh2o / mair)

        vtmp = v* 1.e-06                        ! sans unite
        q = (eps * vtmp) /(1 - (1-eps)*vtmp)    
        ppmv_2q = q
        
        return
        end
! ---------------------------------------------------------------------------------------
       real function q_2rh(p, t, q)
        ! to compute the relative humidity for a given specific humidity and temperature
        real, intent(in) :: p, t, q
        real :: rm                      ! rapport de melange / air sec
        real esat, esit
        real e, ew
        real :: rh                      ! relative humidity in %
                
        rm= q / (1-q) 
        e = (p * rm) /(rm + 0.621989)   ! e: pression partielle de la vapeur d'eau
                                        ! ew: pression partielle saturante
        if( t .gt. 273.15 ) then
          ew = esat(t)
        else
          ew = esit(t)  
        endif
     
        rh=100.*e/ew
        q_2rh=rh
        
        return
        end
