! Name: $Id: maia_Cloud_Type.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_Cloud_Type 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_Cloud_Type (idbg, box, pix_id, pix,thres, CT)
!-----------------------------------------------------------------------
! Authors       : lydie lavanant
!-----------------------------------------------------------------------

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ),       INTENT(in) :: idbg       ! 
  type( box_id ),      INTENT(in) :: box        ! info at the center of the box
  type( pix_info),   INTENT(inout) :: pix_id    ! lat, lon, solar and satellite angles at the pixel
  type( pix_data),   INTENT(in) :: pix  ! pix observations (albedo in %, Tb in K)
  type( maia_thres),  INTENT(in) :: thres       ! 
  type( maia_CT),     INTENT(out) :: CT ! 
                       
!-----------------------------------------------------------------------
! Cloudy Pixel processing
!-----------------------------------------------------------------------
  if( idbg % debug ) then
     write(*,*) '  -----------------------------------------------------------------------'
     write(*,*) '  Cloud_Type '
  endif
  
  ! reset flags
  CT % black   =imanq           ! black-body flag (1= black-body)
  CT % cld_type=imanq           ! cloud type    1=CT_VLOW to 10=CT_FRACTIONAL
  CT % class   =imanq           ! classification from 0 to 20 
              
  call maia_CT_Fl_opaq (idbg, pix, CT % black)

  if(pix_id % twilight .and. .not. pix % Qual_06 .and. .not. pix % Qual_08) then
     pix_id % twilight=.false.
     pix_id % night=.true.
  elseif(pix_id % day .and. .not. pix % Qual_06 .and. .not. pix % Qual_08) then
     pix_id % day = .false.
     pix_id % twilight=.true.
  endif

  if (pix_id % day .or. pix_id % glint) then                    ! day or glint
      if(idbg%debug ) write(*,'(a10)') 'CT_day'
      call maia_CT_day (idbg, box, pix_id, pix, thres, CT)     
  else if(pix_id % night) then                          ! night
      if(idbg%debug) write(*,*) 'CT_night'
      call maia_CT_night (idbg, box, pix_id, pix, thres, CT)     
  else                                                  ! twilight 
      if(idbg%debug) write(*,*) 'CT_dawn'
      call maia_CT_dawn (idbg, box, pix, thres, CT)
  endif

END SUBROUTINE maia_Cloud_Type
