! Name: $Id: maia_ConfTest.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_ConfTest 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_ConfTest (idbg, value, s_cld, s_mid, s_cl, confident_clear)

! Authors       : lydie lavanant

! individual clear confidence level from 1. (clear) to 0.(cloudy)

USE MOD_maia_types
USE MOD_MAIA_CONST
IMPLICIT NONE

! input/ output
  type( debug ),INTENT(in) :: idbg      ! 
  REAL, INTENT(in)      :: value
  REAL, INTENT(in)      :: s_cld, s_mid, s_cl
  REAL, INTENT(out)     :: confident_clear
  
! local
  REAL    ::  s_min, s_max, beta, conf_delta
  LOGICAL ::  switched

!=============================================[ instructions ]==========
   confident_clear = vmanq
   if(s_mid==vmanq) then
      confident_clear = vmanq
      return
   endif
   !if(idbg%debug) write(*,*) 'confident_clear', value,  s_cld, s_mid, s_cl

   ! Determine the largest and smallest magnitude threshold values.  
   ! Store the largest value in s_max, the smallest in s_min   
   ! Set a flag, variable switched, to indicate the slope condition.
   IF (s_cl > s_cld) THEN       !s_cld<s_mid<s_cl et btd >= seuil
        s_min = s_cld
        s_max = s_cl
        switched = .false.
   ELSE                         !s_cl<s_mid<s_cld et btd < seuil
        s_min = s_cl
        s_max = s_cld
        switched = .true.
   END IF
   beta = s_mid

   ! Set confidence level if the test value is out of range and return
   IF (.not. switched .AND. value > s_max) THEN
        confident_clear = 1.0
        return 
   ELSE IF (.not. switched .AND. value < s_min) THEN
        confident_clear = 0.0
        return

   ELSE IF (switched .AND. value < s_min) THEN
        confident_clear = 1.0
        return 
   ELSE IF (switched .AND. value > s_max) THEN
        confident_clear = 0.0
        return
   END IF
!   if(idbg%debug) write(*,*) 'confident_clear', confident_clear, switched, value, beta, s_min, s_max

   ! Value lies between the confident clear and confident cloudy threshold.  
   ! Perform linear interpolation to determine the confidence value.
   IF (value <= beta) THEN
        conf_delta = 0.5 * (value - s_min) / (beta - s_min)
        IF (.not. switched) THEN
                confident_clear = conf_delta
        ELSE
                confident_clear = 1.0 - conf_delta
        END IF
   ELSE
        conf_delta = 0.5 * (value - s_max) / (beta - s_max)
        IF (.not. switched) THEN
                confident_clear = 1.0 - conf_delta
        ELSE
                confident_clear = conf_delta
        END IF
   END IF

   ! Correct for machine precision error.  Confidence level should always be between 0 and 1.
   IF (confident_clear > 1.0) THEN
        confident_clear = 1.0
   ELSE IF (confident_clear < 0.0) THEN
        confident_clear = 0.0
   END IF

END SUBROUTINE maia_ConfTest
