! Name: $Id: maia_Fill_Input.F90 530 2017-06-27 11:07:32Z frna $
!
! Purpose: 
!          Subroutines maia_Fill_Input_Virrs maia_Fill_Input_Avhrr 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    01/05/2015   Lydie Lavanant/Pascale Roquet bug fixes
!  1.3    01/11/2015   Pascale Roquet bug fixes in the missing data control
!  1.4    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.5    23/12/2016   Lydie Lavanant texture from I channel is remplaced by neighbours if missing
!  1.6    08/06/2017   Pascale Roquet Fill pixel_id % satimg (used by maia_SST)
!=====================================================================================
subroutine maia_Fill_Input_Virrs (idbg, pix, lig, field_1b, field_I, pixel_id, pixel_, box)
!=====================================================================================

 USE MOD_MAIA_CONST
 USE mod_maia_types
 implicit none                                                     

  !  INTEGER, PARAMETER :: V_maxch_M     = 12   ! maximun number of channels
  !  INTEGER, PARAMETER :: ic_Viirs_M (V_maxch_M)=(/1,4,5,7,9,10,11,12,13,14,15,16/) 

  type( debug ), intent(in)     :: idbg          
  integer, intent(in)           :: pix, lig
  Type (field),intent(in)       :: field_1b
  Type (field),intent(in)       :: field_I
  type( pix_info),intent(out)   :: pixel_id     ! lat, lon, solar and satellite angles at the pixel
  type( pix_data),intent(out)   :: pixel_       ! pixel observations (albedo in %, Tb in K)
  type( box_id ),intent(out)    :: box          ! lat, lon, solar and satellite angles at the center of the box
 
  pixel_id % texture_fromImager    = .false.
  pixel_id % texture_fromNeighbors = .false.

  pixel_id % lon        = field_1b % lon (pix,lig)      ! longitude (degrees)
  pixel_id % lat        = field_1b % lat (pix,lig)      ! latitude (degrees)
  pixel_id % pixel      = pix   ! longitude (degrees)
  pixel_id % line       = lig   ! latitude (degrees)
  
  if(field_1b % satzen (pix,lig)>vmanq) pixel_id % satzen = field_1b % satzen (pix,lig) ! satellite  zenithal angle(deg)
  if(field_1b % satazi (pix,lig)>vmanq) pixel_id % satazi = field_1b % satazi (pix,lig) ! satellite azimutal angle (deg)
  if(field_1b % sunzen (pix,lig)>vmanq) pixel_id % sunzen = field_1b % sunzen (pix,lig) ! solar zenithal angle(deg)
  if(field_1b % sunazi (pix,lig)>vmanq) pixel_id % sunazi = field_1b % sunazi (pix,lig) ! solar azimutal angle(deg)

  pixel_id % ij         = field_1b % ij(lig)    ! julian instant of the pixel
  pixel_id % tm         = imanq
  pixel_id % alt        = imanq

  pixel_id% ImgName     = 'viirs'
  pixel_id% satimg      = field_1b % mission 
  pixel_ %usable        = .True.
  
  if(field_1b % Qual_Obs(1,pix,lig)==0)  pixel_ %Qual_04 = .true.
  if(field_1b % Qual_Obs(2,pix,lig)==0)  pixel_ %Qual_05 = .true.
  if(field_1b % Qual_Obs(3,pix,lig)==0)  pixel_ %Qual_06 = .true.
  if(field_1b % Qual_Obs(4,pix,lig)==0)  pixel_ %Qual_08 = .true.
  if(field_1b % Qual_Obs(5,pix,lig)==0)  pixel_ %Qual_13 = .true.
  if(field_1b % Qual_Obs(6,pix,lig)==0)  pixel_ %Qual_16 = .true.
  if(field_1b % Qual_Obs(7,pix,lig)==0)  pixel_ %Qual_22 = .true.
  if(field_1b % Qual_Obs(8,pix,lig)==0)  pixel_ %Qual_37 = .true.
  if(field_1b % Qual_Obs(9,pix,lig)==0)  pixel_ %Qual_40 = .true.
  if(field_1b % Qual_Obs(10,pix,lig)==0) pixel_ %Qual_87 = .true.
  if(field_1b % Qual_Obs(11,pix,lig)==0) pixel_ %Qual_108 = .true.
  if(field_1b % Qual_Obs(12,pix,lig)==0) pixel_ %Qual_120 = .true.
  
  if(.not. field_1b % locqual(pix,lig)) then
     pixel_ %Qual_04 = .false.
     pixel_ %Qual_05 = .false.
     pixel_ %Qual_06 = .false.
     pixel_ %Qual_08 = .false.
     pixel_ %Qual_13 = .false.
     pixel_ %Qual_16 = .false.
     pixel_ %Qual_22 = .false.
     pixel_ %Qual_37 = .false.
     pixel_ %Qual_40 = .false.
     pixel_ %Qual_87 = .false.
     pixel_ %Qual_108 = .false.
     pixel_ %Qual_120 = .false.
     pixel_ %usable   = .False.
  endif

  if(   pixel_ %usable ) then
     pixel_ %Obs_04       = field_1b % Obs(1,pix,lig)     ! pixel obs
     pixel_ %Obs_05       = field_1b % Obs(2,pix,lig)     ! pixel obs
     pixel_ %Obs_06       = field_1b % Obs(3,pix,lig)     ! pixel obs
     pixel_ %Obs_08       = field_1b % Obs(4,pix,lig)     ! pixel obs
     pixel_ %Obs_13       = field_1b % Obs(5,pix,lig)     ! pixel obs
     pixel_ %Obs_16       = field_1b % Obs(6,pix,lig)     ! pixel obs
     pixel_ %Obs_22       = field_1b % Obs(7,pix,lig)     ! pixel obs
     pixel_ %Obs_37       = field_1b % Obs(8,pix,lig)     ! pixel obs
     pixel_ %Obs_40       = field_1b % Obs(9,pix,lig)     ! pixel obs
     pixel_ %Obs_87       = field_1b % Obs(10,pix,lig)    ! pixel obs
     pixel_ %Obs_108      = field_1b % Obs(11,pix,lig)    ! pixel obs
     pixel_ %Obs_120      = field_1b % Obs(12,pix,lig)    ! pixel obs
  endif
  
  if(pixel_ %Obs_04 == vmanq) pixel_ %Qual_04 = .false.
  if(pixel_ %Obs_05 == vmanq) pixel_ %Qual_05 = .false.
  if(pixel_ %Obs_06 == vmanq) pixel_ %Qual_06 = .false.
  if(pixel_ %Obs_08 == vmanq) pixel_ %Qual_08 = .false.
  if(pixel_ %Obs_13 == vmanq) pixel_ %Qual_13 = .false.
  if(pixel_ %Obs_16 == vmanq) pixel_ %Qual_16 = .false.
  if(pixel_ %Obs_22 == vmanq) pixel_ %Qual_22 = .false.
  if(pixel_ %Obs_37 == vmanq) pixel_ %Qual_37 = .false.
  if(pixel_ %Obs_40 == vmanq) pixel_ %Qual_40 = .false.
  if(pixel_ %Obs_87 == vmanq) pixel_ %Qual_87 = .false.
  if(pixel_ %Obs_108== vmanq) pixel_ %Qual_108 = .false.
  if(pixel_ %Obs_120== vmanq) pixel_ %Qual_120 = .false.

  ! Pixel Quality     3=high / 2=medium / 1=poor / 0=bad
  pixel_ %Qual_tot=3
  if( (box%day .or. box%glint) .and. .not. pixel_ %Qual_06 ) then
     pixel_ %Qual_tot=2
  endif
  if( .not. pixel_ %Qual_108) then
     pixel_ %usable=.False.
     pixel_ %Qual_tot=0
  endif
  if( .not. pixel_ %Qual_16 .and. .not. pixel_ %Qual_37) then
     pixel_ %usable=.False.
     pixel_ %Qual_tot=1
  endif
              
  if(field_I % Nb_Lines>0) then
     pixel_id % texture_fromImager = .true.
     call maia_Texture_FromImager    (idbg, pix, lig, box, field_I, pixel_id)
  endif

  call maia_Texture_FromNeighbors (idbg, pix, lig, field_1b, pixel_id)

  if(pixel_id%sd_06 == vmanq .and. pixel_id%sd_M06 > vmanq) then
     pixel_id%sd_06             = pixel_id%sd_M06 
     pixel_id % texture_fromNeighbors = .true.
  endif
  if(pixel_id%sd_08 == vmanq .and. pixel_id%sd_M08>vmanq) then
     pixel_id%sd_08             = pixel_id%sd_M08 
     pixel_id % texture_fromNeighbors = .true.
  endif
  if(pixel_id%sd_16 == vmanq .and. pixel_id%sd_M16>vmanq) then
     pixel_id%sd_16             = pixel_id%sd_M16               ! local std for channel 0.8mm (%)
     pixel_id % texture_fromNeighbors = .true.
  endif
  if(pixel_id%sd_108 == vmanq .and. pixel_id%sd_M108>vmanq) then
     pixel_id%sd_108            = pixel_id%sd_M108              ! local std for channel 10.8mm (K)
     pixel_id % texture_fromNeighbors = .true.
  endif  
  if(pixel_id%sd_10837 == vmanq .and. pixel_id%sd_M10837>vmanq) then
     pixel_id%sd_10837          = pixel_id%sd_M10837            ! local std for channel 10.8-3.7mm (K)
     pixel_id % texture_fromNeighbors = .true.
  endif  
  if(pixel_id%diff_Max_M06 == vmanq .and. pixel_id%diff_Max_06>vmanq) then
     pixel_id%diff_Max_06       = pixel_id%diff_Max_M06         ! local max values with neighbours (K)
     pixel_id % texture_fromNeighbors = .true.
  endif  
  if(pixel_id%diff_Max_M108 == vmanq .and. pixel_id%diff_Max_108>vmanq) then
     pixel_id%diff_Max_108      = pixel_id%diff_Max_M108        ! local max values with neighbours (K)
     pixel_id % texture_fromNeighbors = .true.
  endif  
  if(pixel_id%diff_Max_M10806 == vmanq .and. pixel_id%diff_Max_10806>vmanq) then
     pixel_id%diff_Max_10806    = pixel_id%diff_Max_M10806      ! local max values with neighbours (K)
     pixel_id % texture_fromNeighbors = .true.
  endif  
  if(pixel_id%Ratio_Max_M10806 == vmanq .and. pixel_id%Ratio_Max_10806>vmanq) then
     pixel_id%Ratio_Max_10806   = pixel_id%Ratio_Max_M10806     !
     pixel_id % texture_fromNeighbors = .true.
  endif  
  
!  if( .not. pixel_id % texture_fromImager .and. pixel_id%sd_M108>vmanq) then    !pb in Imager Field: do texture from field_1b Neighbors
!     pixel_id % texture_fromNeighbors = .true.
!     pixel_id%sd_06             = pixel_id%sd_M06               ! local std for channel 0.6mm (%)
!     pixel_id%sd_08             = pixel_id%sd_M08               ! local std for channel 0.8mm (%)
!     pixel_id%sd_16             = pixel_id%sd_M16               ! local std for channel 0.8mm (%)
!     pixel_id%sd_108            = pixel_id%sd_M108              ! local std for channel 10.8mm (K)
!     pixel_id%sd_10837          = pixel_id%sd_M10837            ! local std for channel 10.8-3.7mm (K)
!     pixel_id%diff_Max_06       = pixel_id%diff_Max_M06         ! local max values with neighbours (K)
!     pixel_id%diff_Max_108      = pixel_id%diff_Max_M108        ! local max values with neighbours (K)
!     pixel_id%diff_Max_10806    = pixel_id%diff_Max_M10806      ! local max values with neighbours (K)
!     pixel_id%Ratio_Max_10806   = pixel_id%Ratio_Max_M10806     !
!  endif

end subroutine maia_Fill_Input_Virrs
!=====================================================================================
subroutine maia_Fill_Input_Avhrr (idbg, pix, lig, field_1b, pixel_id, pixel_, box)
!=====================================================================================

 USE MOD_MAIA_CONST
 USE mod_maia_types
 implicit none                                                     

  type( debug ), intent(in)     :: idbg          
  integer, intent(in)           :: pix, lig
  Type (field),intent(in)       :: field_1b
  type( pix_info),intent(out)   :: pixel_id     ! lat, lon, solar and satellite angles at the pixel
  type( pix_data),intent(out)   :: pixel_       ! pixel observations (albedo in %, Tb in K)
  type( box_id ),intent(in)     :: box          ! lat, lon, solar and satellite angles at the center of the box

  if( idbg % info ) then
      write(*,*) '*******************************************************************************'
      write(*,*)        
      write(*,*)        
      write(*,'(a20,2i10)')  'Fill pixel/line : ',  pix, lig    
  endif

  pixel_id % texture_fromImager    = .false.
  pixel_id % texture_fromNeighbors = .false.

  pixel_id % lon        = field_1b % lon (pix,lig)      ! longitude (degrees)
  pixel_id % lat        = field_1b % lat (pix,lig)      ! latitude (degrees)
  pixel_id % pixel      = pix   ! longitude (degrees)
  pixel_id % line       = lig   ! latitude (degrees)

  pixel_id % satzen = vmanq     ! satellite  zenithal angle(deg)
  pixel_id % satazi = vmanq     ! satellite azimutal angle (deg)
  pixel_id % sunzen = vmanq     ! solar zenithal angle(deg)
  pixel_id % sunazi = vmanq     ! solar azimutal angle(deg)
  pixel_id % difazi = vmanq     ! relative azimuth angle(deg)

  if(field_1b % sunzen (pix,lig)>vmanq) pixel_id % sunzen = field_1b % sunzen (pix,lig) ! solar zenithal angle(deg)
  if(field_1b % satzen (pix,lig)>vmanq) pixel_id % satzen = field_1b % satzen (pix,lig) ! solar azimutal angle(deg)
  if(field_1b % difazi (pix,lig)>vmanq) pixel_id % difazi = field_1b % difazi (pix,lig) ! relative azimuth angle(deg)

  pixel_id % ij         = field_1b % ij(lig)            ! julian instant of the pixel
  pixel_id % tm         = imanq
  pixel_id % alt        = imanq

  pixel_id% ImgName     = 'avhrr'
  pixel_id% satimg      = field_1b % mission 
  pixel_ %usable        = .true.
  
  if(field_1b % Qual_Obs(1,pix,lig)==0) pixel_ %Qual_06  = .true.
  if(field_1b % Qual_Obs(2,pix,lig)==0) pixel_ %Qual_08  = .true.
  if(field_1b % Qual_Obs(3,pix,lig)==0) pixel_ %Qual_16  = .true.
  if(field_1b % Qual_Obs(4,pix,lig)==0) pixel_ %Qual_37  = .true.
  if(field_1b % Qual_Obs(5,pix,lig)==0) pixel_ %Qual_108 = .true.
  if(field_1b % Qual_Obs(6,pix,lig)==0) pixel_ %Qual_120 = .true.

  if( field_1b % scanqual(lig)/=0 ) then
     pixel_ %Qual_06  = .false.
     pixel_ %Qual_08  = .false.
     pixel_ %Qual_16  = .false.
     pixel_ %Qual_37  = .false.
     pixel_ %Qual_108 = .false.
     pixel_ %Qual_120 = .false.
     pixel_ % usable  = .false.
  endif

  if( pixel_ %Qual_06 )  pixel_ %Obs_06 = field_1b % Obs(1,pix,lig)     ! pixel obs
  if( pixel_ %Qual_08 )  pixel_ %Obs_08 = field_1b % Obs(2,pix,lig)     ! pixel obs
  if( pixel_ %Qual_16 )  pixel_ %Obs_16 = field_1b % Obs(3,pix,lig)     ! pixel obs
  if( pixel_ %Qual_37 )  pixel_ %Obs_37 = field_1b % Obs(4,pix,lig)     ! pixel obs
  if( pixel_ %Qual_108 ) pixel_ %Obs_108= field_1b % Obs(5,pix,lig)     ! pixel obs
  if( pixel_ %Qual_120 ) pixel_ %Obs_120= field_1b % Obs(6,pix,lig)     ! pixel obs

  if(pixel_ %Obs_06 == vmanq) pixel_ %Qual_06 = .false.
  if(pixel_ %Obs_08 == vmanq) pixel_ %Qual_08 = .false.
  if(pixel_ %Obs_16 == vmanq) pixel_ %Qual_16 = .false.
  if(pixel_ %Obs_37 == vmanq) pixel_ %Qual_37 = .false.
  if(pixel_ %Obs_108== vmanq) pixel_ %Qual_108 = .false.
  if(pixel_ %Obs_120== vmanq) pixel_ %Qual_120 = .false.
  
  ! Pixel Quality     3=high / 2=medium / 1=poor / 0=bad
  pixel_ %Qual_tot=3
  if( (box%day .or. box%glint) .and. .not. pixel_ %Qual_06 ) then
     pixel_ %Qual_tot=2
  endif
  if( .not. pixel_ %Qual_108 .or. .not. pixel_ %Qual_120 ) then
     pixel_ %usable=.False.
     pixel_ %Qual_tot=0
  endif
  if( .not. pixel_ %Qual_16 .and. .not. pixel_ %Qual_37) then
     pixel_ %usable=.False.
     pixel_ %Qual_tot=1
  endif

  pixel_id % texture_fromImager = .false.       !Imager Field does not exist    
  pixel_id % texture_fromNeighbors = .true.     !do texture from field_1b Neighbors
  pixel_id%sd_06                = pixel_id%sd_M06               ! local std for channel 0.6mm (%)
  pixel_id%sd_08                = pixel_id%sd_M08               ! local std for channel 0.8mm (%)
  pixel_id%sd_16                = pixel_id%sd_M16               ! local std for channel 0.8mm (%)
  pixel_id%sd_108               = pixel_id%sd_M108              ! local std for channel 10.8mm (K)
  pixel_id%sd_10837             = pixel_id%sd_M10837            ! local std for channel 10.8-3.7mm (K)
  pixel_id%diff_Max_06          = pixel_id%diff_Max_M06         ! local max values with neighbours (K)
  pixel_id%diff_Max_108         = pixel_id%diff_Max_M108        ! local max values with neighbours (K)
  pixel_id%diff_Max_10806       = pixel_id%diff_Max_M10806      ! local max values with neighbours (K)
  pixel_id%Ratio_Max_10806      = pixel_id%Ratio_Max_M10806     !
  
end subroutine maia_Fill_Input_Avhrr
