! Name: $Id: maia_Flag_VolcanAsh.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_Flag_VolcanAsh 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/01/2014   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
subroutine maia_Flag_VolcanAsh (idbg, box, pix, pix_id, thres, CMa)

! Authors       : lydie lavanant

! Determines aVolcanic Ash flag
 
 USE MOD_MAIA_CONST
 USE mod_maia_types
IMPLICIT NONE

  type( debug )                         :: idbg         ! 
  type( box_id ),   INTENT(in)          :: box          ! lat, lon, solar and satellite angles at the center of the box
  type( pix_info)                       :: pix_id       ! lat, lon, solar and satellite angles at the pixel
  type( pix_data)                       :: pix          ! viirs observations (albedo in %, Tb in K)
  type( maia_thres), INTENT(in)         :: thres        ! 
  type( maia_CMa),   INTENT(inout)      :: CMa          ! 

  real    :: s_ASHLD_108, s_06bright, s_ASHLD_120108, s_120108_ASHLN, s_108_ASHLN_cold
  real    :: lst, sst, s_ASHSD_120108, s_ASHSN_108cold, s_ASHSN_120108
  
  if( idbg % debug ) write(*,*) "maia_Flag_VolcanAsh"
  CMa % ash_fl=0     
         
  ! LAND and DAY
  ! -----------
  if(pix_id%land .and. pix_id%day) then
     s_ASHLD_108 = thres % v_CMa_108_land + Thr_ASHLD_108_offset
     s_06bright= Thr_ASHLD_06_bright * box%cosol
     
     s_ASHLD_120108 = Thr_ASHLD_120108
     
     if(pix%Obs_06 > s_06bright) then
        s_ASHLD_120108 = Thr_ASHSD_120108+ Thr_ASHLD_120108_offsetbright
     else
        s_ASHLD_120108 = Thr_ASHSD_120108+ Thr_ASHLD_120108_offsetbright*pix%Obs_06/s_06bright
     endif
     s_ASHLD_120108 = s_ASHLD_120108 + Thr_ASHLD_120108_slope * (box%satsec-1.)   
     
     if( (pix%Obs_06-pix%Obs_16   < Thr_ASHLD_0616 * box%cosol .or.     &
         (pix%Obs_06-pix%Obs_16   < Thr_ASHLD_0616_relaxed * box%cosol .and. &
          pix%Obs_37-pix%Obs_108 > Thr_ASHLD_37108_relaxed)) .and.      &
          pix%Obs_120-pix%Obs_108 > s_ASHLD_120108  .and. pix%Obs_37-pix%Obs_108>Thr_ASHLD_37108 .and.  &
          pix%Obs_108<s_ASHLD_108)  CMa % ash_fl=1     
  endif

  ! LAND and NIGHT
  ! -----------
  if(pix_id%land .and. pix_id%night) then
     lst = thres % v_CMa_108_land
     s_120108_ASHLN = -thres % v_CMa_108_120_land + Thr_ASHLN_120108_offset
     s_108_ASHLN_cold = lst + Thr_ASHLN_108_offsetcold

     if(s_120108_ASHLN < Thr_ASHLN_120108) then
     
        if( s_108_ASHLN_cold <= pix%Obs_108 .and. pix%Obs_108 < lst  )then
           s_120108_ASHLN = s_120108_ASHLN + (Thr_ASHLN_120108 -s_120108_ASHLN)* (pix%Obs_108 -lst)/Thr_ASHLN_108_offsetcold
        else if( Thr_ASHLN_108_VeryCold < pix%Obs_108 .and. pix%Obs_108 < s_108_ASHLN_cold )    then
           s_120108_ASHLN = Thr_ASHLN_120108 +  &
                            Thr_ASHLN_120108_offsetVCold* (pix%Obs_108-s_108_ASHLN_cold)/(Thr_ASHLN_108_VeryCold-s_108_ASHLN_cold)
        else if( pix%Obs_108 <= Thr_ASHLN_108_VeryCold ) then
           s_120108_ASHLN = Thr_ASHLN_120108 +Thr_ASHLN_120108_offsetVCold
        endif
     else
        if( Thr_ASHLN_108_VeryCold < pix%Obs_108 .and.pix%Obs_108 < s_108_ASHLN_cold )   then 
           s_120108_ASHLN=s_120108_ASHLN +     &
                          Thr_ASHLN_120108_offsetVCold*(pix%Obs_108-s_108_ASHLN_cold)/ (Thr_ASHLN_108_VeryCold-s_108_ASHLN_cold)
        else if( pix%Obs_108 <= Thr_ASHLN_108_VeryCold ) then
                  s_120108_ASHLN=s_120108_ASHLN+ Thr_ASHLN_120108_offsetVCold
        endif            
     endif
     s_120108_ASHLN = s_120108_ASHLN + Thr_ASHLN_120108_slope * (box%satsec-1.)                       
         
     if( pix%Obs_37-pix%Obs_108 > thres % v_CMa_37_108_land .and. pix%Obs_120-pix%Obs_108>s_120108_ASHLN ) CMa % ash_fl=1 
  endif
  
  ! SEA and DAY
  ! -----------
  if(pix_id%sea .and. pix_id%day) then
      s_06bright= Thr_ASHSD_06_bright * box%cosol

      if(pix%Obs_06 > s_06bright) then
         s_ASHSD_120108 = Thr_ASHSD_120108 + Thr_ASHSD_120108_offsetbright
      else
         s_ASHSD_120108 = Thr_ASHSD_120108 + Thr_ASHSD_120108_offsetbright* pix%Obs_06 /s_06bright
      endif 
      s_ASHSD_120108 = s_ASHSD_120108  +Thr_ASHSD_120108_slope * (box%satsec-1.)            
     
      if( pix%Obs_06-pix%Obs_16 < Thr_ASHSD_0616 .and. pix%Obs_120-pix%Obs_108 > s_ASHSD_120108 )  CMa % ash_fl=1 
  endif
   
  ! SEA and NIGHT
  ! -------------
   if(pix_id%sea .and. pix_id%night) then
      sst = thres % v_CMa_08_sea
      s_ASHSN_108cold= sst + Thr_ASHSN_108_offsetcold
     
      s_ASHSN_120108 = -thres % v_CMa_108_120_sea + Thr_ASHSN_120108_offset
      
      if( s_ASHSN_120108 <  Thr_ASHSN_120108 ) then
      
         if( s_ASHSN_108cold <= pix%Obs_108 .and. pix%Obs_108 < sst ) then
            s_ASHSN_120108 = s_ASHSN_120108 + (Thr_ASHSN_120108-s_ASHSN_120108) * (pix%Obs_108 - sst) /Thr_ASHSN_108_offsetcold   
         else if( Thr_ASHSN_108_VCold < pix%Obs_108 .and. pix%Obs_108 < s_ASHSN_108cold ) then   
            s_ASHSN_120108 = Thr_ASHSN_120108 +     &
                             Thr_ASHSN_120108_offsetVCold * (pix%Obs_108-s_ASHSN_108cold)/(Thr_ASHSN_108_VCold-s_ASHSN_108cold)
         else if( pix%Obs_108 <= Thr_ASHSN_108_VCold ) then
            s_ASHSN_120108=Thr_ASHSN_120108 + Thr_ASHSN_120108_offsetVCold
         else
            if( (Thr_ASHSN_108_VCold < pix%Obs_108) .and. (pix%Obs_108 < s_ASHSN_108cold) )    then
               s_ASHSN_120108 = s_ASHSN_120108+    &
                                (Thr_ASHSN_120108_offsetVCold)*(pix%Obs_108-s_ASHSN_108cold)/(Thr_ASHSN_108_VCold-s_ASHSN_108cold)
            else if( pix%Obs_108 <= Thr_ASHSN_108_VCold ) then
               s_ASHSN_120108=s_ASHSN_120108+ Thr_ASHSN_120108_offsetVCold
            endif           
         endif
      endif
      s_ASHSN_120108 = s_ASHSN_120108 + Thr_ASHSN_120108_slope * (box%satsec-1.)
      
      if( pix%Obs_37-pix%Obs_108> thres % v_CMa_37_108_sea .and. pix%Obs_120-pix%Obs_108 >s_ASHSN_120108 ) CMa % ash_fl=1    
   endif

end subroutine maia_Flag_VolcanAsh
