! Name: $Id: maia_GetClim.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_GetClim 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_GetClim ( idbg, box, clim)

! Authors       : lydie lavanant
!
! Albedo outside the box array will have a value of 20%
! SST outside the box array will have a value of 0c over coast and 0K over sea

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE
  type( debug ), INTENT(inout)  :: idbg ! 
  type( box_id ),INTENT(inout)  :: box  ! box information
  type( clim_field ),intent(in) :: clim ! 

! local variables
! ---------------
  integer, parameter    :: nx=7, ny=7           ! size of input tab
  REAL          :: lat, lon
  INTEGER       :: i, j, l, n, k, ik, p         ! grid node variables
  INTEGER       :: ipg, ilg                     ! nearest or lower grid node
  real          :: tab(nx,ny)
  integer       :: npmin, npmax, nlmin, nlmax
  real          :: value
  integer       :: box_size
!======================================================================   
  lat = box % lat
  lon = box % lon
  if(clim % norm180) then
    if(box % lon  >=  180. - clim % lon_inc/2.) lon = box % lon - 360.
    if(box % lon  <  -180. - clim % lon_inc/2.) lon = box % lon + 360.
  else
    if(box % lon  >=  360. - clim % lon_inc/2.) lon = box % lon - 360.
    if(box % lon  <     0. - clim % lon_inc/2.) lon = box % lon + 360.
  end if

  ipg= 1+ NINT ( (lon - clim % lon1)/ clim % lon_inc)
  ilg= 1+ NINT ( (lat - clim % lat1)/ clim % lat_inc)
  
  box_size=MIN( (ilg-1) , (clim % nbl - ilg))
  box_size=MIN(box_size,3)

  if(idbg % debug ) then
     if(box_size<3) write(*,*) 'maia_GetClim, box_size : ', box_size
  end if
  if( ilg-box_size < 1 .or. ilg+box_size > clim % nbl) then
     write(*,*) 'GetClim: outside atlas for', clim % name
     write(*,*) 'GetClim: outside atlas in Lat:', lat, ilg, clim % lat1, clim % lat_inc, clim%nbl
     stop 2
  endif

  ! take the maximum or minimum or mean value in an array from 3x3 to 7x7
  ! central node: nearest node
  tab(:,:)= vmanq ; p=0
  do n=1,box_size
  l=2*n+1
  p=0
  do k=-n,n
     p=p+1
     ik=ipg+k
     if(ik<1) ik=ik + clim%nbp
     if(ik>clim%nbp) ik=ik - clim%nbp
     tab(p,1:l) = clim % data (ik,ilg-n:ilg+n)
  end do
  if(any(tab> vmanq) ) exit
  end do

  value = vmanq
  if( clim % name == 'albmax') value = MAXVAL(tab)
      
  if( clim % name == 'sstmin' .or. clim % name == 'sstmoy') then
      if( clim % name == 'sstmin') call maia_ValMin (idbg, tab, vmanq, value)
      if( clim % name == 'sstmoy') call maia_ValMoy (idbg, nx,ny, tab, vmanq, value)
  endif

  if(clim % name == 'albmax' .and. value == vmanq) value = 200.
  if(clim % name == 'albmax' .and. value >50. .and. lat>55.) value = 20.        ! snow surface

  if(clim % name == 'sstmin' .or. clim % name == 'sstmoy') then
  endif
  
  if(idbg % debug) then
        write(*,*) 'neighbours of ', clim % name,' ipg ilg,value,n: ',ipg,ilg,value,n
        write(*,*) 'lat, lat deb, inc',lat, clim % lat1, clim % lat_inc
        npmin=ipg-3 ; if(npmin<1)       npmin=1
        npmax=ipg+3 ; if(npmax>clim % nbp) npmax=clim % nbp
        nlmin=ilg-box_size ; if(nlmin<1)       nlmin=1
        nlmax=ilg+box_size ; if(nlmax>clim % nbl) nlmax=clim % nbl
        write(*,'(7f8.2)') ( (clim % data(i,j),i=npmin,npmax),j=nlmin,nlmax)
  end if

  if( clim % name == 'albmax')  box % albedo_clim = value  
  if( clim % name == 'sstmin')  box % sst_clim = value
  if( clim % name == 'sstmoy')  box % sstmoy_clim=value

END SUBROUTINE  maia_GetClim
      
