! Name: $Id: maia_GetGlint.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_GetGlint 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
  SUBROUTINE maia_GetGlint (idbg, box)

! f90 conversion : lydie lavanant               09/02/06

! Sunglint is defined as:
!       sea maximum reflectance(at 0.6 micron) > 10%
!       sun zenith angle less than 75 degres
!       Sea maximum reflectance is computed using Cox&Munck equations
!  dcj=1 sunglint

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

  type( debug ), INTENT(in)     :: idbg ! 
  type( box_id ),INTENT(inout)  :: box  ! box information

! for the computation following the Cox and Munck theory
  REAL :: albmax                        ! maximum reflectance in %

!=============================================[ instructions ]==========
  box % dcj=0
  box % glint=.false.
  albmax=vmanq

! computes the maximum reflectance over sea (Cox et Munck theory)
  call maia_Cal_CoxMunk (idbg, 0.6, box, albmax)

  if(idbg % debug) then
        write(*,*) 'maia_GetGlint: ', albmax
  endif

  if(albmax > 10) then
     box % dcj=1
     box % glint=.true.
  endif

END SUBROUTINE maia_GetGlint

