! Name: $Id: maia_GetPrev.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_GetPrev 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/05/2015   Lydie Lavanant/Pascale Roquet bug fixes
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_GetPrev( idbg, box, pix_id, bg)

! Authors       : lydie lavanant

! Tair is computed using the temperature at 1000hPa and a slope of
!     0.65K per 100m. If missing the value of 0K is given

USE MOD_MAIA_CONST
USE MOD_maia_types

IMPLICIT NONE
  type( debug ), INTENT(inout)  :: idbg ! 
  type( box_id ),INTENT(inout)  :: box  ! box information
  type( pix_info),INTENT(in)    :: pix_id       ! lat, lon, solar and satellite angles at the pixel
  type( nwp_field),INTENT(in)   :: bg(2)! forecast information

! local variables
! ---------------
  CHARACTER (LEN=16) :: chm             ! character string for messages
  REAL,PARAMETER:: p_maia(3)=(/500.,700.,850./)

  INTEGER       :: i, j, no, ip,il
  REAL          :: lat, lon
  INTEGER       :: ipg, ilg                             ! nearest or lower grid node
  REAL          :: rpg, rlg                             ! grid node real value
  REAL          :: t2m_0(2,2), ts_0(2,2)                ! t2m, ts at altitude 0
  REAL          :: ts(2), t2m(2), cwv(2), ps(2), temp(3,2,2), ttt(51,2,2), temp_nwp(3,2), t_rttov(51,2), h_rttov(51,2)
  REAL          :: t2m_tmp(2), ts_tmp(2), cwv_tmp(2), tab_cwv(2,2), tab_alt(2,2), tab_psurf(2,2), tab_seaice(2,2), tab_snow(2,2)
  REAL          :: frac_time(2)
  LOGICAL       :: interp_time
!===================================================================
  lat = box % lat
  lon = box % lon
  do no=1,2
     ts(no) =vmanq
     t2m(no)=vmanq
     cwv(no)=vmanq

     if(bg(no) % exist) then

        if(bg(no) %norm180) then
           if(box % lon  >=  180.- bg(no) % lon_inc/2.) lon = box % lon - 360.
           if(box % lon  <= -180.- bg(no) % lon_inc/2.) lon = box % lon + 360.
        else
           if(box % lon  >=  360.- bg(no) % lon_inc/2.) lon = box % lon - 360.
           if(box % lon  <=    0.- bg(no) % lon_inc/2.) lon = box % lon + 360.
        end if

        rpg= 1+ (lon - bg(no) % lon1) /bg(no) % lon_inc
        rlg= 1+ (lat - bg(no) % lat1) /bg(no) % lat_inc
        ipg = INT(rpg)  ! lower pixel grid
        ilg = INT(rlg)  ! lower line grid
        rpg=rpg-ipg
        rlg=rlg-ilg

        if(idbg %debug) then
              write(*,*) 'GetPrev: norm: ', bg(no) %norm180, 360.- bg(no) % lon_inc/2.,  0.- bg(no) % lon_inc/2.
              write(*,*) 'GetPrev: ipg,ilg,no,nbp,nbl,lon,box%lon: ', ipg, ilg, no, bg(no)%nbp, bg(no)%nbl, lon, box%lon
              write(*,*) 'GetPrev: : ', bg(no) % lon1, bg(no) % lon_inc, 1+ (lon - bg(no) % lon1) /bg(no) % lon_inc, rpg
        end if
      
        ! spatial interpolation 
        if(bg(no)%circumferential .or. (ipg>0 .and. ilg>0 .and. ipg+1<=bg(no)%nbp .and. ilg+1<=bg(no)%nbl) ) then
                   
           ! extrapolation to altitude 0 for t2m and ts
           do i=1,2
           do j=1,2
              ip=ipg+i-1
              if(bg(no)%circumferential .and. ip>bg(no)%nbp) ip=ip-bg(no)%nbp
              if(bg(no)%circumferential .and. ip<1)          ip=ip+bg(no)%nbp
              il=ilg+j-1
              t2m_0(i,j) = bg(no)%t2m(ip,il) + (0.65* bg(no)%alt(ip,il) )/100.
              ts_0 (i,j) = bg(no)%ts (ip,il) + (0.65* bg(no)%alt(ip,il) )/100.
              tab_cwv(i,j)    = bg(no)%cwv  (ip,il)
              tab_alt(i,j)    = bg(no)%alt  (ip,il)
              tab_psurf(i,j)  = bg(no)%psurf  (ip,il)
              tab_seaice(i,j) = bg(no)%seaice  (ip,il)
              tab_snow(i,j)   = bg(no)%snow  (ip,il)
              call maia_interp_Plog (bg(no)%pres, bg(no)%temp(:,ip, il), p_maia, temp(:,i,j),bg(no)%nbniv,3)
              call maia_interp_Plog (bg(no)%pres, bg(no)%temp(:,ip, il), p_rttov,ttt (:,i,j),bg(no)%nbniv,51)
           end do
           end do                
       
           call maia_interp_InGrid (ts_0      , rpg, rlg, ts (no))
           call maia_interp_InGrid (t2m_0     , rpg, rlg, t2m(no))
           call maia_interp_InGrid (tab_cwv   , rpg, rlg, cwv(no))
           call maia_interp_InGrid (tab_alt   , rpg, rlg, box % altnwp)
           call maia_interp_InGrid (tab_psurf , rpg, rlg, ps(no))
           call maia_interp_InGrid (tab_seaice, rpg, rlg,box % seaice_prev)
           call maia_interp_InGrid (tab_snow  , rpg, rlg,box % snow_prev)
            
           do i=1,3
              call maia_interp_InGrid (temp(i,:,:), rpg, rlg, temp_nwp(i,no))
           end do
                       
           do i=1,51
              call maia_interp_InGrid (ttt(i,:,:), rpg, rlg, t_rttov(i,no))
           end do

           ! hu on RTTOV levels: the nearest
           ipg= 1+ NINT( (lon - bg(no) %lon1) /bg(no) %lon_inc )
           ilg= 1+ NINT( (lat - bg(no) %lat1) /bg(no) %lat_inc )
           if(ipg == 1+ bg(no) %nbp) ipg=1
           call maia_interp_Plog (bg(no)%pres, bg(no)%humid(:,ipg,ilg),p_rttov,h_rttov(:,no),bg(no)%nbniv,51)

           if(idbg %debug) then
              write(*,*) 'GetPrev: ipg,ilg, neighbours of: ', ipg,ilg
!              write(*,'(a20, 5f10.3)') 'altnwp : ',  ((bg(no)%alt(i,j),i=ipg,ipg+1),j=ilg,ilg+1)       
!              write(*,'(a20,5i10)') 'tm : ',         ((bg(no)%tm(i,j),i=ipg,ipg+1),j=ilg,ilg+1)
!              write(*,'(a20,5f10.3)') 'T2m_0 : ',   ((t2m_0(i,j),i=1,2),j=1,2)       
!              write(*,'(a20,5f10.3)') 'WV forecast', ((bg(no)%cwv(i,j),i=ipg,ipg+1),j=ilg,ilg+1)
           end if
           
        else    ! takes the nearest
           ipg= 1+ NINT( (lon - bg(no) %lon1) /bg(no) %lon_inc )
           ilg= 1+ NINT( (lat - bg(no) %lat1) /bg(no) %lat_inc )
           if(ipg == 1+ bg(no) %nbp) ipg=1
           if(idbg %debug) then
              write(*,*) 'GetPrev nearest: ipg,ilg : ', ipg, ilg
              write(*,*) 'GetPrev nearest: ', bg(no) % lon1, bg(no) % lon_inc, 1+ NINT( (lon - bg(no) %lon1) /bg(no) %lon_inc )
           end if

           if(ipg <= 0 .or. ipg > bg(no) %nbp .or. ilg <= 0 .or. ilg > bg(no) %nbl) then
              idbg % nbw_bg = idbg % nbw_bg+1
              if(idbg % nbw_bg <= 10) then
                 write(chm,'(2f8.2)') box % lat, box % lon
                 call ml_wt('warning','pixel outside forecast '//chm)
                 if(idbg % nbw_bg == 10) call ml_wt('info','suspending further warnings')
              end if
           else
              ts(no)  = ( bg(no) % ts (ipg,ilg)  + (0.65* bg(no) %alt(ipg,ilg))/100.)
              t2m(no) = ( bg(no) % t2m(ipg,ilg)  + (0.65* bg(no) %alt(ipg,ilg))/100.)
              cwv(no) =   bg(no) % cwv(ipg,ilg)
              box % altnwp  =   bg(no) % alt(ipg,ilg)
              ps(no) =    bg(no) % psurf(ipg,ilg)

              call maia_interp_Plog (bg(no)%pres, bg(no)%temp(:,ipg,ilg),p_maia, temp_nwp(:,no), bg(no)%nbniv,3)

              ! on RTTOV levels
              call maia_interp_Plog (bg(no)%pres, bg(no)%temp(:,ipg, ilg),p_rttov,t_rttov(:,no),bg(no)%nbniv,51)
              call maia_interp_Plog (bg(no)%pres, bg(no)%humid(:,ipg,ilg),p_rttov,h_rttov(:,no),bg(no)%nbniv,51)

           end if
        end if  ! interpolation if possible
     endif      ! previ exist                         
  end do        ! 2 forecast fields (no)
  
  ! takes the smallest value of t2m_prev and the largest value for cwv 
  ! ------------------------------------------------------------------ 
  do no=1,2
     t2m_tmp(no) = 99999.
     ts_tmp(no)  = 99999.
     cwv_tmp(no) = vmanq
     if(t2m(no)  >  2) t2m_tmp(no) = t2m(no)
     if(ts (no)  >  2) ts_tmp (no) = ts(no)   
     if(cwv(no)  > 0)  cwv_tmp(no) = cwv(no)
  end do        ! 2 forecast fields (no)

  box % t2m_prev = MINVAL (t2m_tmp)
  box % ts_prev  = MINVAL (ts_tmp)
  box % cwv_prev = MAXVAL (cwv_tmp)
  box % cwv      = box % cwv_prev

  if(idbg % debug) then
      write(*,*) 'surf_alt', box % altnwp  
      write(*,*) 't2m_tmp', t2m_tmp, box % t2m_prev
      write(*,*) 'ts_tmp ', ts_tmp , box % ts_prev
      write(*,*) 'cwv_tmp, cwv_prev', cwv_tmp, box % cwv_prev
      write(*,*) 'temp_nwp at 500hPa', temp_nwp(1,1:2)
      write(*,*) 'temp_nwp at 700hPa', temp_nwp(2,1:2)
      write(*,*) 'temp_nwp at 850hPa', temp_nwp(3,1:2)
  endif

  ! interpolation of the two forecast fields function of hour
  ! --------------------------------------------------------- 
  box % t500=vmanq; box % t700=vmanq; box % t850=vmanq
  interp_time=.false.
  if(temp_nwp(1,1)> vmanq .and. temp_nwp(1,2)== vmanq) no=1
  if(temp_nwp(1,2)> vmanq .and. temp_nwp(1,1)== vmanq) no=2
  if(temp_nwp(1,1)> vmanq .and. temp_nwp(1,2)> vmanq) interp_time=.true.

  if(.not. interp_time) then
     box % t500= temp_nwp(1,no)
     box % t700= temp_nwp(2,no)
     box % t850= temp_nwp(3,no)
     box % ps_prev = ps(no)
     box % t_rttov(1:51)= t_rttov(1:51,no)
     box % h_rttov(1:51)= h_rttov(1:51,no)
  else
     ! for the interpolation in time between fields 
     if( (bg(2)%ij-bg(1)%ij) /=0) then
      frac_time(1)=(pix_id%ij -bg(1)%ij)  / (bg(2)%ij - bg(1)%ij)
      frac_time(2)=(bg(2)%ij  -pix_id%ij) / (bg(2)%ij - bg(1)%ij)
      ! take into account possible time differences between file name and line time (3mn max)
      if(frac_time(1)>1. .and. frac_time(1)<1.005) frac_time(1)=1.
      if(frac_time(2)>1. .and. frac_time(2)<1.005) frac_time(2)=1.
      if(frac_time(1)<0. .and. frac_time(1)>-.005) frac_time(1)=0.
      if(frac_time(2)<0. .and. frac_time(2)>-.005) frac_time(2)=0.
     else
      frac_time(1)=0
      frac_time(2)=1
     endif
     box % t500= (1.-frac_time(1))*temp_nwp(1,1) + (1.-frac_time(2))*temp_nwp(1,2)
     box % t700= (1.-frac_time(1))*temp_nwp(2,1) + (1.-frac_time(2))*temp_nwp(2,2)
     box % t850= (1.-frac_time(1))*temp_nwp(3,1) + (1.-frac_time(2))*temp_nwp(3,2)
     box % ps_prev = (1.-frac_time(1))*ps(1) + (1.-frac_time(2))*ps(2)

     ! temp on RTTOV levels
     do i=1,51
        box % t_rttov(i)= (1.-frac_time(1))*t_rttov(i,1) + (1.-frac_time(2))*t_rttov(i,2)
     end do
     ! hu on RTTOV levels
     if(frac_time(1) > frac_time(2)) then
        box % h_rttov(:)= h_rttov(:,1)
     else
        box % h_rttov(:)= h_rttov(:,2)
     endif

  endif

  if(frac_time(1)<0 .or. frac_time(2)<0 .or. frac_time(1)>1 .or. frac_time(2)>1) then
     call ml_wt('ERROR','NWP fields times are uncorrect')
     write(*,*)  pix_id%line, pix_id%ij, bg(1)%ij, bg(2)%ij, frac_time(1), frac_time(2)
     write(*,*) "box" , box
     stop 2
  endif
  if(idbg % debug) then
      write(*,*) 'frac_time', pix_id%ij,  bg(1:2)%ij, frac_time
      write(*,*) 'box % t500, ...', box % t500, box % t700, box % t850, box % ps_prev
  endif

  if(box % t2m_prev > 350.0) box % t2m_prev =vmanq
  if(box % ts_prev  > 350.0) box % ts_prev  =vmanq  

   
END SUBROUTINE  maia_GetPrev
