! Name: $Id: maia_GetThres_CMa.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_GetThres_CMa 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_GetThres_CMa (idbg, box, pix_id, reflec, thvis_sea, thvis_land, &
                               tabsea, tabland, thres)

! Authors       : lydie lavanant

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input/output
  REAL, INTENT(in) :: reflec(nbreflecsol,nbreflecsat,nbreflecazi,3)
  type( debug ),               INTENT(in) :: idbg       ! 
  type( box_id ),              INTENT(in) :: box        ! lat, lon, solar and satellite angles at the center of the box
  type( pix_info),             INTENT(in) :: pix_id     ! lat, lon, solar and satellite angles at the pixel
  type( maia_VISThresTables),  INTENT(in) :: thvis_sea   
  type( maia_VISThresTables),  INTENT(in) :: thvis_land   
  type( maia_ThresTables_sea), INTENT(in) :: tabsea
  type( maia_ThresTables_land),INTENT(in) :: tabland
  type( maia_thres),           INTENT(inout):: thres    ! 
          
  REAL          :: cosol_sea, cosol_land, cosol_snowice
  REAL          :: albmax       ! maximum reflectance in %
  REAL          :: corr

!-----------------------------------------------------------------------

  ! ir thresholds
  ! -------------
  if ( box % sea .or. box % coast)  call maia_GetThres_CMaSea  ( idbg, box , tabsea, thres)   
  if ( .not. box % sea)             call maia_GetThres_CMaland ( idbg, box, tabland, thres)

  ! threshold 1.6mm over snow/ice
  ! ----------------------------
  call maia_Get_AlbSnow (idbg, 16, box, reflec, thvis_land, thres % v_CMa_16_ice)

  cosol_snowice=box % cosol
  if(cosol_snowice < cosolmax_snowice) cosol_snowice = cosolmax_snowice    
  thres % v_CMa_16_ice =  thres % v_CMa_16_ice*cosol_snowice

  thres % v_CMa_16_ice =  thres % v_CMa_16_ice + alb_snow_offset2

  ! visible thresholds
  ! ------------------
  cosol_sea=box % cosol
  if(cosol_sea < cosolmax_sea) cosol_sea = cosolmax_sea

  cosol_land=box % cosol
  if(cosol_land < cosolmax_land) cosol_land = cosolmax_land    
    
  if( box % sunzen < cst_sunzen_for_useVIS) then            !not night
     if ( box % sea .or. box % coast) then    
        call maia_Get_AlbSea (idbg, 06, box, thvis_sea, thres % v_CMa_06_sea)
        call maia_Get_AlbSea (idbg, 08, box, thvis_sea, thres % v_CMa_08_sea)
        call maia_Get_AlbSea (idbg, 16, box, thvis_sea, thres % v_CMa_16_sea)

        if (thres % v_CMa_06_sea <=  0) thres % v_CMa_06_sea = 15.      !(%)    
        if (thres % v_CMa_08_sea <=  0) thres % v_CMa_08_sea = 15.      !(%)
        if (thres % v_CMa_16_sea <=  0) thres % v_CMa_16_sea = 15.      !(%)
        thres % v_CMa_06_sea = thres % v_CMa_06_sea * cosol_sea
        thres % v_CMa_08_sea = thres % v_CMa_08_sea * cosol_sea
        thres % v_CMa_16_sea = thres % v_CMa_16_sea * cosol_sea
     else
        thres % v_CMa_06_sea = vmanq
        thres % v_CMa_08_sea = vmanq
        thres % v_CMa_16_sea = vmanq
     endif

    if (  .not. box % sea) then    
        call maia_Get_AlbLand (idbg, 04, box, thvis_land, thres % v_CMa_04_land) 
        call maia_Get_AlbLand (idbg, 06, box, thvis_land, thres % v_CMa_06_land)
        call maia_Get_AlbLand (idbg, 13, box, thvis_land, thres % v_CMa_13_land)
        if (thres % v_CMa_04_land<=  0) thres % v_CMa_04_land=  15.     !(%)
        if (thres % v_CMa_06_land<=  0) thres % v_CMa_06_land=  15.     !(%)
        if (thres % v_CMa_13_land<=  0) thres % v_CMa_13_land=  15.     !(%)
        thres % v_CMa_04_land = thres % v_CMa_04_land * cosol_land          
        thres % v_CMa_06_land = thres % v_CMa_06_land * cosol_land
        thres % v_CMa_13_land = thres % v_CMa_13_land * cosol_land          
     else
        thres % v_CMa_04_land= vmanq
        thres % v_CMa_06_land= vmanq
        thres % v_CMa_13_land= vmanq
     endif
  endif
  if( idbg%debug ) then
        write(*,*) ' seuils albedo sea = ',thres % v_CMa_06_sea, thres % v_CMa_08_sea, thres % v_CMa_16_sea
        write(*,*) ' seuils albedo land= ',thres % v_CMa_04_land, thres % v_CMa_06_land, thres % v_CMa_13_land
        write(*,*) 'THRES land 06, 108-120, 108-37, 37-108, 108 avant: ',                       &
                 thres % v_CMa_06_land, thres % v_CMa_108_120_land, thres % v_CMa_108_37_land, &
                 thres% v_CMa_37_108_land, thres % v_CMa_108_land
  endif         

  ! some additional corrections for particular conditions
  ! -----------------------------------------------------

  ! DAY/GLINT or TWILIGHT
  if (.not. box % night .and. box % albedo_clim>vmanq .and. .not. box%snow_always) then
     thres% v_CMa_37_108_land = thres% v_CMa_37_108_land +                                      &
                cosol_land*(0.4*box % albedo_clim )+2.+36*box % cosol*(box%cos_scat_ang-0.41)**2!+ 0.-6.*box%cos_scat_ang)
      
     call maia_Cal_CoxMunk (idbg, 0.6, box, albmax)

     if(albmax > vmanq) then
        thres% v_CMa_37_108_sea = thres% v_CMa_37_108_sea+ &
                                coxmunck_slope_37108*albmax*cosol_sea + coxmunck_offset_37108
     else
        thres% v_CMa_37_108_sea = thres% v_CMa_37_108_sea+ 2.*coxmunck_offset_37108
     endif
     if(box%Africa) thres % v_CMa_06_land =thres % v_CMa_06_land + box%cosol*(25*(box%cos_scat_ang-0.68)**2)
  endif
      
  ! limits confusion of very moist clear areas with clouds 
  if (box % Africa) thres % v_CMa_108_120_land = thres % v_CMa_108_120_land + A_offset_108120

  ! NIGHT
  if (box % night .and. box % Africa) then
     thres % v_CMa_108_land = thres % v_CMa_108_land + A_offset_108
  endif

  ! NIGHT or TWILIGHT
  if (box % night .or. box % twilight ) then
     ! Low water clouds over Europe
     if (box % Europe) then
        if ( box % satsec < E_seclim_10837(1) ) then
           corr = E_coef_10837(1)
        elseif ( box % satsec < E_seclim_10837(2))  then
           corr = E_coef_10837(1)+  (box % satsec - E_seclim_10837(1))*         &
                 (E_coef_10837(2)-E_coef_10837(1))/(E_seclim_10837(2)-E_seclim_10837(1))
        elseif ( box % satsec < E_seclim_10837(3)) then
           corr = E_coef_10837(2)+ (box % satsec-E_seclim_10837(2))*    &
                 (E_coef_10837(3)-E_coef_10837(2))/(E_seclim_10837(3)-E_seclim_10837(2))
        elseif ( box % satsec< E_seclim_10837(4)) then
           corr = E_coef_10837(3)+ (box % satsec-E_seclim_10837(3))*    &
                 (E_coef_10837(4)-E_coef_10837(3))/(E_seclim_10837(4)-E_seclim_10837(3))
        elseif ( box % satsec < E_seclim_10837(5)) then
           corr = E_coef_10837(4)+ (box % satsec-E_seclim_10837(4))*    &
                 (E_coef_10837(5)-E_coef_10837(4))/(E_seclim_10837(5)-E_seclim_10837(4))
        else
           corr=E_coef_10837(5)
        endif
        thres % v_CMa_108_37_land = thres % v_CMa_108_37_land + corr
     endif                
     if (box % Africa) then
        thres % v_CMa_108_37_land = thres % v_CMa_108_37_land + A_offset_10837
     endif                
  endif

  ! over desert and coast (tbc)
  if(box % albedo_clim > cst_desertalbmin) then
     if( (box % coast .and. pix_id% ImgName == 'viirs') .or. &
                           (pix_id% ImgName == 'avhrr') ) then
        thres % v_CMa_37_108_land = thres % v_CMa_37_108_land + 3
        thres % v_CMa_108_37_land = thres % v_CMa_108_37_land + 3
     endif
  endif

  ! in polar areas to take into account the sea surface change
  if(box%sst_clim <276.15) then
     thres % v_CMa_108_37_sea = thres % v_CMa_108_37_sea + 0.5
     thres % v_CMa_87_108_sea = thres % v_CMa_87_108_sea + 0.5
  endif
  
  if(box%alt>500) then
     thres % v_CMa_108_land = thres % v_CMa_108_land +2.
  endif
  
  ! over snowalways in nighttime and twilight conditions
  if(box % snow_prev > 0.02 .or. box % seaice_prev >0.1 .or. box % snow_always .and. &
     ( box % night .or. box % twilight) .and. pix_id% ImgName == 'viirs' ) then
     if(box % tsurf >= 265.) then
        thres % v_CMa_108_land = thres % v_CMa_108_land +5.
     else
        thres % v_CMa_108_land = thres % v_CMa_108_land +5. + 0.4*(265- box%tsurf)
     endif
  endif
  
  ! prints
  ! ------
  if( idbg%debug ) then
     write(*,*) 'day_time, cos_scat_ang : ', box % day_time, box % cos_scat_ang
     write(*,*) 'alb, sst_clim= ',box % albedo_clim, box % sst_clim
     write(*,*) 'THRES ice= 16:', thres % v_CMa_16_ice          
     write(*,*) 'THRES sea= 06, 08, 16, 108-120, 108-37, 37-108, 108:',                 &
                 thres % v_CMa_06_sea, thres % v_CMa_08_sea, thres % v_CMa_16_sea,      &
                 thres % v_CMa_108_120_sea, thres % v_CMa_108_37_sea, thres% v_CMa_37_108_sea, thres % v_CMa_108_sea
     write(*,*) 'THRES land 06, 108-120, 108-37, 37-108, 108: ',                        &
                 thres % v_CMa_06_land, thres % v_CMa_108_120_land, thres % v_CMa_108_37_land, &
                 thres% v_CMa_37_108_land, thres % v_CMa_108_land
    endif
 
END SUBROUTINE maia_GetThres_CMa
