! Name: $Id: maia_GetThres_CMa_Sea.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_GetThres_CMaSea 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_GetThres_CMaSea(idbg, box, tabsea,thres)

! Authors       : lydie lavanant

! to compute the thresholds used over sea 
!       function of satsec, wv, sst
!       indsec from 1 to 16
!       satsec : secant of local zenith angle
!-----------------------------------------------------------------------
! tabtsurf_cold= tsurft108_ocean_DT+0
! tabtsurf_warm= tsurft108_ocean_DT+0
! tab108_120_cold=  t108120_ocean_DT+3
! tab108_120_warm=  t108120_ocean_DT+0
! tab108_37_cold=  t108038_ocean_DT+3
! tab108_37_warm=  t108038_ocean_DT+3
! tab120_37_cold=  t120038_ocean_DT+3
! tab120_37_warm=  t120038_ocean_DT+3
! tab37_108_cold=  t038108_ocean_DT+0
! tab37_108_warm=  t038108_ocean_DT-3   
! tab87_108_cold=  t087108_ocean_DT-3
! tab87_108_warm=  t087108_ocean_DT-3   
! tab37_40_cold=  t038040_ocean_DT+0    !a verifier  
! tab37_40_warm=  t038040_ocean_DT+0    !a verifier 
!-----------------------------------------------------------------------

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

  type( debug ),                INTENT(in) :: idbg      ! 
  type( box_id ),               INTENT(in) :: box       ! lat, lon, solar and sat angles at the center of the box
  type( maia_ThresTables_sea ),INTENT(in) :: tabsea     ! tabulated threshold tables (nb_wv,nb_secant)
  type( maia_thres ),           INTENT(inout):: thres   ! dynamic thresholds in (deg*100)

! local:
  REAL, PARAMETER   ::  max_noise = 0.3 
  REAL, PARAMETER   ::  sec_maxval=2.5
  REAL, PARAMETER   ::  sec_deb=1.0
  REAL, PARAMETER   ::  sec_step=0.25
  REAL, PARAMETER   ::  wv_deb=2.5              ! in kg/m2
  REAL, PARAMETER   ::  wv_step=5.              ! in kg/m2
      
  INTEGER :: iw,isec
  REAL    :: difsec,difw,difsst 
  REAL    :: seuil,seuilc,seuilw 
  REAL    :: satsec_loc, sst

! functions called
  REAL ::  maia_Interp_InLut

!=============================================[ instructions ]==========
  satsec_loc = box % satsec
  if(box % sst_clim> vmanq) then
     sst = box % sst_clim       !
  else
     sst = box % tsurf  !
  endif  
        
! Preparation for interpolation in secant  
  if (satsec_loc  > sec_maxval) satsec_loc=sec_maxval 
  isec= 1 + (satsec_loc-sec_deb)/sec_step 
  if (isec  >= sec_nb) isec=sec_nb -1 
  if (isec  <= 1) isec=1 
  difsec = ( satsec_loc - sec_deb - (isec-1)*sec_step ) / sec_step   

! Preparation for interpolation in water vapor   
  iw= 1 + (box % cwv - wv_deb)/wv_step 
  if (iw  >= wv_nb) iw=wv_nb -1 
  if (iw  <= 1) iw=1 
  difw = (box % cwv -wv_deb - (iw-1)*wv_step)/ wv_step 

! Preparation for interpolation in cold waters  
  difsst=(sst - cst_sstcold) / (cst_sstwarm - cst_sstcold)        
 
  if(idbg % debug) then
    write(*,*) 'maia_GetThres_CMaSea: satsec, wv, sst ',   satsec_loc, box % cwv , sst      
    write(*,*) 'maia_GetThres_CMaSea: isec,iw,difsec,difw,difsst ',isec, iw, difsec, difw, difsst
  end if

! compute the different thresholds
  if (sst  < cst_sstcold) then
        
      ! s_A4_A5 / s_M15_M16 (t10.8-t12.0) 
        seuil= maia_Interp_InLut ( tabsea%tab108_120_cold, difsec,difw,iw, isec)  
        if(seuil > vmanq ) seuil=seuil+ max_noise      
        thres% v_CMa_108_120_sea=seuil        
      ! s_sst  (t10.8)
        seuil= maia_Interp_InLut (tabsea%tabtsurf_cold, difsec,difw,iw,isec)  
        if(seuil > vmanq ) seuil=seuil+ max_noise  
        thres% v_CMa_108_sea=seuil  
      ! s_A4_A3 / s_M15_M12 (t10.8-t3.7)
        seuil= maia_Interp_InLut (tabsea%tab108_37_cold, difsec,difw,iw,isec)  
        if(seuil > vmanq ) seuil=seuil+ max_noise  
        thres% v_CMa_108_37_sea=seuil  
      ! s_A5_A3  / s_M16_M12  (t12.0-t3.7)
        seuil= maia_Interp_InLut (tabsea%tab120_37_cold, difsec,difw,iw,isec)  
        if(seuil > vmanq ) seuil=seuil+ max_noise  
        thres% v_CMa_120_37_sea=seuil
      ! s_A3_A4 / s_M37_M15 (t3.7-t10.8)
        seuil= maia_Interp_InLut (tabsea%tab37_108_cold, difsec,difw,iw,isec)  
        if(seuil  > vmanq ) seuil=seuil+max_noise       
        thres% v_CMa_37_108_sea=seuil          
      ! s_M37_M13 / (t3.7-t4.0)
        seuil= maia_Interp_InLut (tabsea%tab37_40_cold, difsec,difw,iw,isec)  
        if(seuil  > vmanq ) seuil=seuil+max_noise       
        thres% v_CMa_37_40_sea=seuil           
      ! s_M14_M15 (t8.7-t10.8)
        seuil= maia_Interp_InLut (tabsea%tab87_108_cold, difsec,difw,iw,isec)  
        if(seuil  > vmanq ) seuil=seuil+max_noise       
        thres% v_CMa_87_108_sea=seuil         
   
  else if (sst  > cst_sstwarm) then
   
      ! s_A4_A5 / s_M15_M16 / t108t120 
        seuil= maia_Interp_InLut( tabsea%tab108_120_warm, difsec,difw,iw, isec)  
        if(seuil > vmanq ) seuil=seuil+ max_noise      
        thres% v_CMa_108_120_sea=seuil    
      ! s_sst  (t108)
        seuil= maia_Interp_InLut(tabsea%tabtsurf_warm, difsec,difw,iw,isec)  
        if(seuil > vmanq ) seuil=seuil+ max_noise  
        thres% v_CMa_108_sea=seuil
      ! s_A4_A3 / s_M15_M12 / t108t37
        seuil= maia_Interp_InLut(tabsea%tab108_37_warm, difsec,difw,iw,isec)  
        if(seuil > vmanq ) seuil=seuil+ max_noise  
        thres% v_CMa_108_37_sea=seuil 
      ! s_A5_A3  / s_M16_M12  / t120t37
        seuil= maia_Interp_InLut(tabsea%tab120_37_warm, difsec,difw,iw,isec)  
        if(seuil > vmanq ) seuil=seuil+ max_noise  
        thres% v_CMa_120_37_sea=seuil 
      ! s_A3_A4 / s_M37_M15 / t37t108
        seuil= maia_Interp_InLut(tabsea%tab37_108_warm, difsec,difw,iw,isec)  
        if(seuil  > vmanq ) seuil=seuil+max_noise       
        thres% v_CMa_37_108_sea=seuil           
      ! s_M37_M13 / t37t40
        seuil= maia_Interp_InLut(tabsea%tab37_40_warm, difsec,difw,iw,isec)  
        if(seuil  > vmanq ) seuil=seuil+max_noise       
        thres% v_CMa_37_40_sea=seuil           
      ! s_M14_M15 / t87t108
        seuil= maia_Interp_InLut(tabsea%tab87_108_warm, difsec,difw,iw,isec)  
        if(seuil  > vmanq ) seuil=seuil+max_noise       
        thres% v_CMa_87_108_sea=seuil          
   
  else
      ! s_A4_A5 / s_M15_M16 / t108t120 
        seuilw= maia_Interp_InLut( tabsea%tab108_120_warm, difsec,difw,iw, isec)  
        seuilc= maia_Interp_InLut( tabsea%tab108_120_cold, difsec,difw,iw, isec)  
        seuil=(1-difsst)*seuilc + difsst*seuilw 
        if(seuil > vmanq ) seuil=seuil+ max_noise      
        thres% v_CMa_108_120_sea=seuil       
      ! s_sst  (t108)
        seuilw= maia_Interp_InLut(tabsea%tabtsurf_warm, difsec,difw,iw,isec)  
        seuilc= maia_Interp_InLut(tabsea%tabtsurf_cold, difsec,difw,iw,isec)  
        seuil=(1-difsst)*seuilc + difsst*seuilw 
        if(seuil > vmanq ) seuil=seuil+ max_noise  
        thres% v_CMa_108_sea=seuil  
      ! s_A4_A3 / s_M15_M12 / t108t37
        seuilw= maia_Interp_InLut(tabsea%tab108_37_warm, difsec,difw,iw,isec)  
        seuilc= maia_Interp_InLut(tabsea%tab108_37_cold, difsec,difw,iw,isec)  
        seuil=(1-difsst)*seuilc + difsst*seuilw 
        if(seuil > vmanq ) seuil=seuil+ max_noise  
        thres% v_CMa_108_37_sea=seuil  
      ! s_A5_A3  / s_M16_M12  / t120t37
        seuilw= maia_Interp_InLut(tabsea%tab120_37_warm, difsec,difw,iw,isec)  
        seuilc= maia_Interp_InLut(tabsea%tab120_37_cold, difsec,difw,iw,isec)  
        seuil=(1-difsst)*seuilc + difsst*seuilw 
        if(seuil > vmanq ) seuil=seuil+ max_noise  
        thres% v_CMa_120_37_sea=seuil  
      ! s_A3_A4 / s_M37_M15 / t37t108
        seuilw= maia_Interp_InLut(tabsea%tab37_108_warm, difsec,difw,iw,isec)  
        seuilc= maia_Interp_InLut(tabsea%tab37_108_cold, difsec,difw,iw,isec)  
        seuil=(1-difsst)*seuilc + difsst*seuilw 
        if(seuil  > vmanq ) seuil=seuil+max_noise       
        thres% v_CMa_37_108_sea=seuil           
      ! s_M37_M13 / t37t40
        seuilw= maia_Interp_InLut(tabsea%tab37_40_warm, difsec,difw,iw,isec)  
        seuilc= maia_Interp_InLut(tabsea%tab37_40_cold, difsec,difw,iw,isec)  
        seuil=(1-difsst)*seuilc + difsst*seuilw 
        if(seuil  > vmanq ) seuil=seuil+max_noise       
        thres% v_CMa_37_40_sea=seuil           
      ! s_M14_M15 / t87t108
        seuilw= maia_Interp_InLut(tabsea%tab87_108_warm, difsec,difw,iw,isec)  
        seuilc= maia_Interp_InLut(tabsea%tab87_108_cold, difsec,difw,iw,isec)  
        seuil=(1-difsst)*seuilc + difsst*seuilw 
        if(seuil  > vmanq ) seuil=seuil+max_noise       
        thres% v_CMa_87_108_sea=seuil           
      
  endif                   

END SUBROUTINE maia_GetThres_CMaSea
