! Name: $Id: maia_GetThres_CT.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_GetThres_CT 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_GetThres_CT (idbg, box, tabopaq, thres)

! Authors       : lydie lavanant

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ),    INTENT(in)  :: idbg         ! 
  type( box_id ),   INTENT(in)  :: box          ! lat, lon, solar and satellite angles at the center of the box
  type( maia_ThresTables_opaq),INTENT(in) :: tabopaq   ! tabulated threshold tables (nb_wv,nb_secant)
  type( maia_thres),INTENT(out):: thres ! 
          
!-----------------------------------------------------------------------
  
  ! initialise to missing value 
  ! ---------------------------
  thres % v_CT_max108_lo = vmanq
  thres % v_CT_max108_me = vmanq
  thres % v_CT_max108_hi = vmanq
  thres % v_CT_max108_vh = vmanq
  thres % v_CT_108_120_opaq = vmanq
  thres % v_CT_37_108_opaq = vmanq
  thres % v_CT_mincirrus_06 = vmanq
  thres % v_CT_maxcirrus_06slope  = vmanq
  thres % v_CT_maxcirrus_06intercept  = vmanq
  thres % v_CT_maxcirrus_06  = vmanq
  thres % v_CT_mincirrus_06 = vmanq
  thres % v_CT_minlow_06  = vmanq

  ! get Thresholds 
  ! ---------------------------
  call maia_GetThres_CT_max108 (idbg, box, thres )
  
  call maia_GetThres_CT_Opaq ( idbg, box, tabopaq, thres )
    
  ! day or dawn
  if(box % sunzen < 90. .and. box % sunzen >= 0. .and. &
     box % satzen < 90. .and. box % satzen >= 0. .and. &
     abs(box % difazi) <= 360.) then

     call maia_GetThres_CT_max06 (idbg, box, thres)
  endif

  if( idbg%debug ) then
      write(*,*) 'thresholds CT = ', &
                  thres % v_CT_108_120_opaq, thres % v_CT_37_108_opaq,  &
                  thres % v_CT_max108_lo,    thres % v_CT_max108_me,    &
                  thres % v_CT_max108_hi,    thres % v_CT_max108_vh,    &
                  thres % v_CT_maxcirrus_06slope, thres % v_CT_maxcirrus_06intercept, &
                  thres % v_CT_minlow_06
  endif
 
END SUBROUTINE maia_GetThres_CT
