! Name: $Id: maia_GetThres_CT_max06.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_GetThres_CT_max06 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_GetThres_CT_max06 (idbg, box, thres)

! compute coefs leading to max. R06 (for cirrus) (sea or land)

! Authors       : lydie lavanant

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ),    INTENT(in)  :: idbg         ! 
  type( box_id ),   INTENT(in)  :: box          ! lat, lon, solar and satellite angles at the center of the box
  type( maia_thres),INTENT(out):: thres ! 

! local
  REAL :: alb_low, tempmin_ref                  
  REAL :: coef0,coef1,ts
  REAL :: rnadtormes
! ---------------------------------------------------------------------------     
  ! Surface temperature
  ! -------------------------
  if(box % sst_clim > vmanq ) then      ! sea
      ts=box % sst_clim
  else                                  ! land or coast
      ts=box % tsurf
  endif

  ! thresholds for cloud type
  ! -------------------------
  if (box % tm == 0) then                               ! sea  
      alb_low     = CT_sea_alb_default
      tempmin_ref = CT_sea_min_temp_ref 
  else                                                  ! land  or coast 
    alb_low       = box % albedo_clim + CT_land_alb_safety_offset
    tempmin_ref   = CT_land_min_temp_ref                        
  endif

  ! compute reflectances ratio with bidirectional effects simulation
  call maia_Reflratio_ToObs (idbg, box, rnadtormes)

  coef1 = ((CT_high_alb_ref - alb_low)/ CT_vis_cor_daily) / (tempmin_ref - ts)
  coef0 = (CT_high_alb_ref /CT_vis_cor_daily) - coef1 * tempmin_ref

  thres % v_CT_maxcirrus_06slope     = coef1 * box % cosol / rnadtormes 
  thres % v_CT_maxcirrus_06intercept = coef0 * box % cosol / rnadtormes 
  
  thres % v_CT_maxcirrus_06 = (box % cosol * CT_high_alb_ref /CT_vis_cor_daily) / rnadtormes
  thres % v_CT_mincirrus_06 = (box % cosol * alb_low /CT_vis_cor_daily ) / rnadtormes               
      
  ! over sea       
  if (box%sea) then
        thres % v_CT_minlow_06 =  CT_sea_min_low_vis06  * box % cosol / rnadtormes
  ! over land  or coast    
  else if (box%land .or. box%coast ) then
        thres % v_CT_minlow_06 =  CT_land_min_low_vis06  * box % cosol / rnadtormes
  endif
                
END SUBROUTINE maia_GetThres_CT_max06
