! Name: $Id: maia_GetThres_CT_opaq.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_GetThres_CT_Opaq 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_GetThres_CT_Opaq ( idbg, box, tabopaq,  thres )

! Authors       : lydie lavanant

! to compute the thresholds used for opaque clouds 
!       function of satsec_loc, wv
!       indsec from 1 to 16
!       satsec_loc : secant of local zenith angle
!-----------------------------------------------------------------------
! tab108_120_opaq=  t108t120_nuage_+0
! tab37_108_opaq =  t038t108_nuage_+0
! tab87_108_opaq =      ! 
!-----------------------------------------------------------------------

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

  type( debug ),                INTENT(in) :: idbg      ! 
  type( box_id ),               INTENT(in) :: box       ! lat, lon, solar and sat angles at the center of the box
  type( maia_ThresTables_opaq),INTENT(in) :: tabopaq   ! tabulated threshold tables (nb_wv,nb_secant)
  type( maia_thres ),           INTENT(out):: thres     ! dynamic thresholds in (deg*100)

! local:
  REAL, PARAMETER   :: sec_maxval=2.5
  REAL, PARAMETER   :: sec_deb=1.0
  REAL, PARAMETER   :: sec_step=0.25
  REAL, PARAMETER   :: wv_deb=2.5               ! in kg/m2
  REAL, PARAMETER   :: wv_step=5.               ! in kg/m2
      
  INTEGER :: iw,isec
  REAL    :: satsec_loc, difsec,difw

! functions called
  REAL :: maia_Interp_InLut
!=============================================[ instructions ]==========
  if( idbg % debug ) write(*,*) "maia_GetThres_CT_Opaq"
  satsec_loc=box % satsec
  
! Preparation for interpolation in secant 
      if (box % satsec > sec_maxval) satsec_loc=sec_maxval
      isec= 1 + (satsec_loc-sec_deb)/sec_step
      if (isec >= sec_nb) isec=sec_nb-1
      if (isec <= 1) isec=1
      difsec = ( satsec_loc - sec_deb - (isec-1)*sec_step )/ sec_step  

! Preparation for interpolation in water vapour  
      iw = 1 + (box % cwv-wv_deb)/wv_step
      if (iw >= wv_nb) iw = wv_nb-1
      if (iw <= 1) iw=1
      difw = (box % cwv- wv_deb -(iw-1)*wv_step)/ wv_step       

!M15_M16 
      thres% v_CT_108_120_opaq = maia_Interp_InLut(tabopaq % tab108_120_opaq,difsec,difw,iw,isec)
       
!M14_M15
      thres% v_CT_87_108_opaq = maia_Interp_InLut(tabopaq % tab87_108_opaq,difsec,difw,iw,isec)

!M12_M15
      thres% v_CT_37_108_opaq = maia_Interp_InLut(tabopaq % tab37_108_opaq,difsec,difw,iw,isec)

END SUBROUTINE maia_GetThres_CT_Opaq
