! Name: $Id: maia_Get_AlbLand.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_Get_AlbLand 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE  maia_Get_AlbLand (idbg, num_can, box, thvis_land, albter)

! Authors       : lydie lavanant

 USE MOD_MAIA_CONST
 USE MOD_maia_types

IMPLICIT NONE

  type( debug ),               INTENT(in)       :: idbg         ! 
  INTEGER,                     INTENT(in)       :: num_can
  type( box_id ),              INTENT(in)       :: box          ! info at the center of the box
  type( maia_VISThresTables), intent(in)        :: thvis_land   ! tabulated threshold 

  real,                        INTENT(out)      :: albter       ! reference albedo in %

! for the simulation of BRDF roujean
  REAL          :: brdf
 
! for the simulation of 6s
  REAL          :: a0,a1,a2     ! a0,a1 multiplied by 100 in simulatmos_vis to get brdf6s in %
  integer       :: num_ic, ic

! for the simulation of brdf + atmosphere
  REAL          :: surface, brdf6s

!=======================================================
  num_ic=-99
  do ic=1, nb_canvis
     if(num_can==icanvis(ic)) num_ic= ic
  end do
  if(num_ic<0) then
     return
  endif

  if(box % albedo_clim == 0) then   ! outside atlas
     albter = 0
     return
  endif

! calculation of land surface reflexion from Roujean model
  call maia_Cal_Roujean ( idbg, box, brdf)
  if(brdf.le.0) then
     albter = 0
     return
  endif

! computes parameters for atmospheric correction
  call maia_Cal_AtmCorrVis (idbg, box, thvis_land, a0, a1, a2, num_ic)
  if(a0 == vmanq) then
     albter = 0
     return
  endif

! use of the Roujean function and of the atmospheric correction
  surface = box % albedo_clim * brdf    ! in %

  if(surface > 200.) surface = 200.     ! 200%
  brdf6s = a0+a1*surface/(100.-surface*a2)      ! in %

  albter = brdf6s + alb_land_offset(num_ic)     ! in %

END SUBROUTINE  maia_Get_AlbLand
