! Name: $Id: maia_Get_AlbSea.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_Get_AlbSea 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
subroutine  maia_Get_AlbSea (idbg, num_can, box, thvis_sea, albmer)

! Authors       : lydie lavanant

 USE MOD_MAIA_CONST
 USE MOD_maia_types

IMPLICIT NONE
   
  INTEGER,          INTENT(in)  :: num_can
  type( debug ),    INTENT(in)  :: idbg         ! 
  type( box_id ),   INTENT(in)  :: box          ! lat, lon, solar and satellite angles at the center of the box
  type( maia_VISThresTables), intent(in) :: thvis_sea   ! tabulated threshold 

  real,          INTENT(out)    :: albmer       ! threshold in %

! for the simulation of 6s
  REAL          :: a0,a1,a2             ! a0,a1 are multiplied by 100 in simulatmos_vis to get brdf6s in %
  integer       :: num_ic, ic

! for the computation following the Cox and Munck theory
  REAL          :: albmax               ! maximum reflectance in %
  REAL          :: lambda               ! wave number used in the calculation of Fresnel coefficients

! for the simulation of brdf + atmosphere
  REAL          :: surface, brdf6s

!=============================================[ instructions ]==========
  num_ic=-99
  do ic=1, nb_canvis
     if(num_can==icanvis(ic)) then
        num_ic= ic
        lambda = longueuronde(ic)
     endif
  end do
  if(num_ic<0) then
     return
  endif
 
! computes the maximum reflectance over sea (Cox et Munck theory)
  call maia_Cal_CoxMunk (idbg, lambda, box, albmax)     !%

! computes parameters for atmospheric correction
  call maia_Cal_AtmCorrVis (idbg, box, thvis_sea, a0, a1, a2, num_ic)
  if(a0 == vmanq .or. albmax == vmanq) then
        albmer = 0
        return
  endif

! use of the Cox/Munck function and of the atmospheric correction
  surface = albmax                              ! in %
  
  if(surface > 200.) surface = 200.
  brdf6s = a0+a1*surface/(100.-surface*a2)      ! in %

  albmer = brdf6s + alb_sea_offset(num_ic)      ! in %
  
  if(num_can ==08) albmer=albmer+1

END subroutine maia_Get_AlbSea
