! Name: $Id: maia_Get_AlbSnow.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_Get_AlbSnow 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE  maia_Get_AlbSnow (idbg, num_can, box, reflec, thvis_land, sn16)

! Authors       : lydie lavanant

 USE MOD_MAIA_CONST
 USE MOD_maia_types

IMPLICIT NONE

  INTEGER,          INTENT(in)  :: num_can
  type( debug ),    INTENT(in)  :: idbg         ! 
  type( box_id ),   INTENT(in)  :: box          ! lat, lon, solar and satellite angles at the center of the box
  REAL,             INTENT(in)  :: reflec(nbreflecsol,nbreflecsat,nbreflecazi,3)
  type( maia_VISThresTables), intent(in) :: thvis_land   ! tabulated threshold 
  REAL,             INTENT(out) :: sn16

! for 6s simulation
  REAL          :: a0,a1,a2
  integer       :: num_ic, ic

! for Le Roux model calculation
  INTEGER       :: albsans

! for the simulation of brdf + atmosphere
  REAL          :: surface, brdf6s

!=============================================[ instructions ]==========
  num_ic=-99
  do ic=1, nb_canvis
     if(num_can==icanvis(ic)) num_ic= ic
  end do
  if(num_ic<0) then
     return
  endif
     
! computes the reflectance (thesis of C.Le Roux)
  call maia_Cal_LeRoux (idbg, box, 3, reflec, albsans)
  if(albsans.le.0) then
     sn16 = 0.
     return
  endif

! computes atmospheric parameters
  call maia_Cal_AtmCorrVis (idbg, box, thvis_land, a0, a1, a2, num_ic)
  if(a0== vmanq) then
      sn16 = 0.
      return
  endif

! use of Le Roux model values and of the atmospheric correction
  surface = albsans                             ! in % 
  
  if(surface > 200.) surface = 200.
  brdf6s = a0+a1*surface/(100.-surface*a2)      ! in %

  sn16 = brdf6s+alb_snow_offset                 ! in  %

END SUBROUTINE  maia_Get_AlbSnow
