! Name: $Id: maia_Interp_Plog.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_Interp_Plog 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE  maia_Interp_Plog (pi,ti,pf,tf,ni,nf)

! f90 conversion : lydie lavanant       04/06/02

! logarithm interpolation on pressures

IMPLICIT NONE

  INTEGER ,INTENT(in) :: ni,nf
  REAL, INTENT(in)    :: pi(ni),ti(ni),pf(nf)
  REAL, INTENT(out)   :: tf(nf)

  INTEGER  ::   i,f
  REAL ::       pseuil
  LOGICAL ::    termine

!=============================================[ instructions        ]===
  IF(pi(1) > pi(ni)) then

    DO f=1,nf
       if (pf(f) >= pi(1)) then
          tf(f)=ti(1) + alog(pf(f)/pi(1)) * (ti(2)-ti(1))/alog(pi(2)/pi(1))

       elseif (pf(f) <= pi(ni)) then
          tf(f)=ti(ni-1) + alog(pf(f)/pi(ni-1)) * (ti(ni)-ti(ni-1))/alog(pi(ni)/pi(ni-1))

       else
          i=2
          termine=.false.
          DO WHILE( .not. termine )
            pseuil=0.01
            if(pi(i) <= 1.) pseuil=0.0001
            if(pi(i) > 1. .and. pi(i) <= 10.) pseuil=0.001
            if(abs(pi(i)-pf(f)) <=  pseuil) then
              tf(f)=ti(i)
              termine=.true.
            else
              if(pi(i) < pf(f)) then
                tf(f)=ti(i-1) + alog(pf(f)/pi(i-1)) * (ti(i)-ti(i-1))/alog(pi(i)/pi(i-1))
                termine=.true.
              endif
            endif
            i=i+1
          END DO

       endif
    END DO
  ELSE

    DO  f=1,nf
        if(pf(f) <= pi(1)) then
          tf(f)=ti(1)+alog(pf(f)/pi(1)) * (ti(2)-ti(1))/alog(pi(2)/pi(1))

        elseif(pf(f) >= pi(ni)) then
          tf(f)=ti(ni-1)+alog(pf(f)/pi(ni-1)) * (ti(ni)-ti(ni-1))/alog(pi(ni)/pi(ni-1))

        else
          i=2
          termine=.false.
          DO WHILE( .not. termine )
            pseuil=0.01
            if(pi(i) <= 1.) pseuil=0.0001
            if(pi(i) > 1. .and. pi(i) <= 10.) pseuil=0.001
            if(abs(pi(i)-pf(f)) <=  pseuil) then
              tf(f)=ti(i)
              termine=.true.
            else
              if(pi(i) > pf(f)) then
                tf(f)=ti(i-1)+alog(pf(f)/pi(i-1)) * (ti(i)-ti(i-1))/alog(pi(i)/pi(i-1))
                termine=.true.
              endif
            endif
            i=i+1
          END DO
        endif
    END DO

  ENDIF
  
END SUBROUTINE maia_Interp_Plog

