! Name: $Id: maia_Pixel_reset.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_Pixel_reset 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
!=====================================================================================
subroutine maia_Pixel_reset ( idbg, pix_id, pix )
!=====================================================================================

 USE MOD_MAIA_CONST
 USE mod_maia_types
IMPLICIT NONE

  type( debug )         :: idbg         ! 
  type( pix_info)       :: pix_id       ! lat, lon, solar and satellite angles at the pixel
  type( pix_data)       :: pix  ! pix observations (albedo in %, Tb in K)

 if(idbg % debug)  write(*,*) "maia_Pixel_reset"
 pix_id % lon           =vmanq  ! longitude (degrees)
 pix_id % lat           =vmanq  ! latitude (degrees)
 pix_id % satzen        =vmanq  ! satellite  zenithal angle(deg)
 pix_id % satazi        =vmanq  ! satellite azimutal angle (deg)
 pix_id % sunzen        =vmanq  ! solar zenithal angle(deg)
 pix_id % sunazi        =vmanq  ! solar azimutal angle(deg)
 pix_id % ij            =vmanq  ! julian instant of the pixel
 pix_id % tm            =imanq  ! 0=sea, 1=mixed, 2=land
 pix_id % alt           =imanq  ! surface altitude (m)
 pix_id % day_time      =imanq  ! 0: NIGHT /1: TWILIGHT /2: DAY /3: SUNGLINT
 pix_id % satsec        =vmanq  ! secant of satellite zenith angle
 pix_id % difazi        =vmanq  ! relative azimuth angle  (deg)
 pix_id % Tropic        =.false.
 pix_id % Midlat        =.false.
 pix_id % Polar         =.false.
 pix_id % desert        =.false.
 pix_id % lake          =.false.
 pix_id % Toc_Ndvi      =vmanq  ! top of canopy ndvi
 pix_id % SnowIce_surf  =imanq  ! flag indicating if the surface is covered by snow or ice              

 pix % Obs_04           = vmanq ! pix moderate resolution obs
 pix % Obs_05           = vmanq ! pix moderate resolution obs
 pix % Obs_06           = vmanq ! pix moderate resolution obs
 pix % Obs_08           = vmanq ! pix moderate resolution obs
 pix % Obs_13           = vmanq ! pix moderate resolution obs
 pix % Obs_16           = vmanq ! pix moderate resolution obs
 pix % Obs_22           = vmanq ! pix moderate resolution obs
 pix % Obs_37           = vmanq ! pix moderate resolution obs
 pix % Obs_40           = vmanq ! pix moderate resolution obs
 pix % Obs_87           = vmanq ! pix moderate resolution obs
 pix % Obs_108          = vmanq ! pix moderate resolution obs
 pix % Obs_120          = vmanq ! pix moderate resolution obs

 pix % Qual_04 = .false.
 pix % Qual_05 = .false.
 pix % Qual_06 = .false.
 pix % Qual_08 = .false.
 pix % Qual_13 = .false.
 pix % Qual_16 = .false.
 pix % Qual_22 = .false.
 pix % Qual_37 = .false.
 pix % Qual_40 = .false.
 pix % Qual_87 = .false.
 pix % Qual_108 = .false.
 pix % Qual_120 = .false.

 pix % I(1:4,1:5)       = vmanq ! pix imager resolution obs
 pix % Qual_I(1:5)      = .false.

end subroutine maia_Pixel_reset
 
