! Name: $Id: maia_Pr_InfoPix.F90 514 2017-02-06 11:28:14Z frna $
!
! Purpose: 
!          Subroutines maia_Pr_InfoPix maia_Pr_Thres 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.2    19/01/2016   Pascale Roquet add wrapper and  maia_Pr_field1b
!=====================================================================================
subroutine maia_Pr_InfoPix(pix_id, pix)
!=====================================================================================

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

  type( pix_info), intent(in)   :: pix_id       ! lat, lon, solar and satellite angles at the pixel
  type( pix_data), intent(in)   :: pix          ! pix observations (albedo in %, Tb in K)


  write(*,*) '  Pixel: '
  write(*,'(a20,2i10)')  'line/pixel: ',  pix_id % line, pix_id % pixel ! longitude (degrees)
  write(*,'(a20,2f10.3)')  'lat/lon: ',  pix_id % lat, pix_id % lon             ! longitude (degrees)
  write(*,'(a20,2f10.3)')  'satzen/satazi: ',  pix_id % satzen, pix_id % satazi ! satellite  zenithal angle(deg)
  write(*,'(a20,2f10.3)')  'sunzen/sunazi: ',  pix_id % sunzen,pix_id % sunazi  ! solar zenithal angle(deg)
  write(*,'(a20,2f10.3)')  'difazi: ',  pix_id % difazi                         ! solar zenithal angle(deg)
  write(*,'(a20,f20.9)')   'ij    : ',  pix_id % ij                             ! julian instant of the pixel
  write(*,'(a20, 2i5)')    'alt, tm: ', pix_id % alt, pix_id % tm
  write(*,*) '    day/glint/twilight/night: ', pix_id % day, pix_id % glint, pix_id % twilight, pix_id% night
  write(*,*) '    sea/land/coast/desert/snow_always: ', &
                                   pix_id% sea, pix_id% land, pix_id% coast, pix_id% desert, pix_id% snow_always

  write(*,*) '    Obs:     04,     05,     06,     08,     13,     16,     22,     37,     40,     87,    108,    120'    ! pix moderate resolution obs
  write(*,'(8x,12f8.1)') pix % Obs_04, pix % Obs_05, pix % Obs_06, pix % Obs_08, &
                      pix % Obs_13, pix % Obs_16, pix % Obs_22, pix % Obs_37, & 
                      pix % Obs_40, pix % Obs_87, pix % Obs_108,pix % Obs_120   ! pixel obs
  write(*,*) '    Qual: ', pix % Qual_04, pix % Qual_05, pix % Qual_06, pix % Qual_08, &
                           pix % Qual_13, pix % Qual_16, pix % Qual_22, pix % Qual_37, &
                           pix % Qual_40, pix % Qual_87, pix % Qual_108,pix % Qual_120

  write(*,*) '         texture_fromImager / texture_fromNeighbors : ', &
               pix_id % texture_fromImager, pix_id % texture_fromNeighbors
  write(*,'(a40, 4f10.3)') ' sd_06, sd_08, sd_108, sd_10837 : ',pix_id % sd_06, pix_id % sd_08, pix_id % sd_108, pix_id % sd_10837 
  write(*,'(a40, 4f10.3)') ' diff_Max_06, diff_Max_108 : ',pix_id % diff_Max_06, pix_id % diff_Max_108
  write(*,'(a40, 4f10.3)') ' diff_Max_10806, Ratio_Max_10806 : ',pix_id % diff_Max_10806, pix_id % Ratio_Max_10806

end subroutine maia_Pr_InfoPix

!=====================================================================================
subroutine maia_Pr_InfoPix_lc(pix_id, pix, col, line)
!=====================================================================================

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

  type( pix_info), intent(in)   :: pix_id       ! lat, lon, solar and satellite angles at the pixel
  type( pix_data), intent(in)   :: pix          ! pix observations (albedo in %, Tb in K)
  integer, intent(in)   :: line                 ! line number of the pixel to be printed
  integer, intent(in)   :: col                  ! column number of the pixel to be printed

if ((pix_id % line .eq. line) .AND. (pix_id % pixel .eq. col)) then
  call maia_Pr_InfoPix(pix_id, pix)
endif
end subroutine maia_Pr_InfoPix_lc

!=====================================================================================
subroutine maia_Pr_field1b(field1b, col, line)
!=====================================================================================

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE  
  type( field), intent(in)   :: field1b      ! file1b information
  integer, intent(in)   :: line                 ! line number of the pixel to be printed
  integer, intent(in)   :: col                  ! column number of the pixel to be printed

  write(*,*) "field_1b info for pixel,line",col," ",line
  write(*,*) "mission ", field1b%mission
  write(*,*) "ImgName  ",   field1b%ImgName          ! Imager name (avhrr, viirs)
  write(*,*) "dataname_gmodo ", field1b%dataname_gmodo      ! dataname of the geo file
  write(*,*) "N_Granule_ID ", field1b%N_Granule_ID       ! granule identifier
  write(*,*) "Distributor ",  field1b%Distributor       ! name of l1b Distributor
  write(*,*) "ij_start ", field1b%ij_start           ! Beginning time
  write(*,*) "ij_end", field1b%ij_end              ! Endding time
  write(*,*) "Orbit_num_start ", field1b%Orbit_num_start     ! Beginning orbit number
  write(*,*) "Orbit_num_end ", field1b%Orbit_num_end       ! Endding orbit number
  write(*,*) "Nb_Granules ", field1b%Nb_Granules          ! number of granules in field

  write(*,*) "Nb_Lines ",field1b%Nb_Lines            ! number of lines in field for Moderate resolution channels
  write(*,*) "Nb_pixels ",field1b%Nb_pixels            ! number of pixels in line for Moderate resolution channels
  write(*,*) "Nb_chan ",field1b%Nb_chan              ! number of channels in field
  write(*,*) "NLines_filled ", field1b%NLines_filled       ! effective number of lines in field for Moderate resolution channels
  write(*,*) "lon ",field1b%lon(col,line)           ! longitude (degrees)
  write(*,*) "lat ",field1b%lat(col,line)           ! latitude (degrees)
  write(*,*) "satzen ",field1b%satzen(col,line)          ! satellite  zenithal angle(deg)
  write(*,*) "sunzen ",field1b%sunzen(col,line)          ! solar zenithal angle(deg)
  write(*,*) "difazi ",field1b%difazi(col,line)          ! solar-satellite azimutal angles(deg)
  write(*,*) "Obs ",field1b%Obs(:,col,line)           ! viirs obs (ic, pixel, line)
  write(*,*) "Qual_Obs ",field1b%Qual_Obs(:,col,line)      ! viirs quality(ic, pixel, line) OK=0
  write(*,*) "ij ",field1b%ij(line)               ! time of the line
  write(*,*) "ScanQual ",field1b%ScanQual(line)          ! viirs fov geoloc quality
  write(*,*) "locqual ",field1b%locqual(col,line)         ! viirs fov geoloc quality
  write(*,*) "pix_texture ",field1b%pix_texture(:,col,line)   ! no pixels des voisins 
  write(*,*) "lig_texture ",field1b%lig_texture(:,col,line)   ! no lignes des voisins 
  write(*,*) "ch3b ",field1b% ch3b(line)              ! specific AVHRR on metop

end subroutine maia_Pr_field1b

!=====================================================================================
subroutine maia_Pr_Thres(thres)
!=====================================================================================

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE
  type( maia_thres),INTENT(in)  :: thres        ! 

write(*,*) thres%v_CMa_sst, thres%v_CMa_108_sea, thres%v_CMa_108_120_sea, thres%v_CMa_37_40_sea, thres%v_CMa_108_37_sea, &              
     thres%v_CMa_120_37_sea,  thres%v_CMa_87_108_sea, thres%v_CMa_37_108_sea, thres%v_CMa_37_120_sea, thres%v_CMa_06_sea, &             
     thres%v_CMa_08_sea, thres%v_CMa_06_08_sea, thres%v_CMa_16_sea, thres%v_CMa_108_land, thres%v_CMa_108_120_land, &           
     thres%v_CMa_108_37_land, thres%v_CMa_87_108_land, thres%v_CMa_87_37_land, thres%v_CMa_37_40_land, &
     thres%v_CMa_37_108_land, thres%v_CMa_37_120_land, thres%v_CMa_04_land, thres%v_CMa_06_land, thres%v_CMa_06_08_land, &              
      thres%v_CMa_13_land, thres%v_CMa_108_120_desert, thres%v_CMa_108_37_desert, thres%v_CMa_04_desert , &     
     thres%v_CMa_13_desert, thres%v_CMa_37_40_ice, thres%v_CMa_108_37_ice, thres%v_CMa_13_ice,  thres%v_CMa_16_ice, &           
     thres%v_CMa_37_120_ice,  thres%v_CMa_108_ice               

end subroutine maia_Pr_Thres
