! Name: $Id: maia_Read_Clim.F90 530 2017-06-27 11:07:32Z frna $
!
! Purpose: 
!          Subroutines maia_Read_Clim read_data05_h5 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.3    19/05/2017   Add subroutine for file name determination
subroutine maia_Read_Clim (idbg, field_id, pix_id, clim_id, clim)

! Authors       : lydie lavanant
!
! filename hdf5 atlases are in (%)*10
! SSTMIN and SSTMOY hdf5 atlases are in (°C)*100
! convert to (%) and (K)

#ifdef HAS_LIBHDF5
use HDF5
#endif

USE MOD_MAIA_CONST
USE MOD_maia_types
implicit none

  type( debug ),     INTENT(in)         :: idbg         ! 
  Type (field_info),  INTENT(in)        :: field_id
  type( pix_info ),   intent(in)        :: pix_id       ! infos at the pixel
  character(len=6),   intent(in)        :: clim_id      ! 
  type( clim_field ), intent(out)       :: clim ! 

#ifdef HAS_LIBHDF5
  ! resolution 0.05 degree
  integer, parameter :: xdim = 7200 
  integer, parameter :: nbline = 200 
  CHARACTER(LEN=14)      :: filename0
  CHARACTER(LEN=256)     :: filename
  CHARACTER(LEN=230)     :: dir_maia4_atlas
  INTEGER(HID_T)         :: file_id              ! file identifier


  integer       :: data05(xdim, nbline)
  integer       :: i, k, l, n, p, ios
  integer       :: num_lat, month, day, year, part, ig(7)
  integer       :: lat_banddeb, lat_bandfin, nbbandes
  integer       :: lonFirst, lonLast, kdeb, kfin
  logical       :: file_exist

!----------------------------------------------------------------------------
!----------------------------------------------------------------------------
   call xijg(pix_id % ij,ig)
   year = ig(1)
   month= ig(2) !month
   day  = ig(3) !day
   part=1
   if(day>15) part=2

   write(filename,'(a6,a3)') clim_id,'.h5'
   file_exist = .False.
   inquire(file=filename, exist= file_exist)
   if ( .not. file_exist) then
      call get_environment_variable(name="DIR_MAIA4_ATLAS",value=dir_maia4_atlas,status=ios)
       write(*,*) "DIR_MAIA4_ATLAS=",dir_maia4_atlas
      if (ios == 0 ) then
        call filename_clim (year,month,day,clim_id,filename0)
        filename = trim(dir_maia4_atlas) //'/'//trim(filename0)
        write(*,*) "filename",filename
        inquire(file=filename , exist= file_exist)  
      endif
   endif
   if ( .not. file_exist ) then
     write(*,*) "ERROR: cannot found atlas file", filename
     stop 2
   endif
   
!----------------------------------------------------------------------------
  ! initialise l interface fortran
  CALL h5open_f(ios)

  ! open du fichier
  call H5Fopen_f( filename, H5F_ACC_RDONLY_F, file_id, ios ) 
! ----------------------------------------------------------------------------
  if(MODULO(field_id % LatM_max, 10.)==0)   then
     lat_banddeb= field_id % LatM_max
  else
     lat_banddeb= ( (INT(field_id % LatM_max/10.)+1)*10 )
  endif
  lat_bandfin= (INT(field_id % LatM_min/10.)*10)

  if(field_id % LatM_max+0.5 > lat_banddeb )  lat_banddeb=lat_banddeb+10
  if(field_id % LatM_min-0.5 < lat_bandfin )  lat_bandfin=lat_bandfin-10

  if(lat_banddeb>90)  lat_banddeb=90.
  if(lat_bandfin<-90) lat_bandfin=-90.

  nbbandes= (lat_banddeb - lat_bandfin)/10

  if(idbg%debug)write(*,*) field_id % LatM_max, field_id % LatM_min
  if(idbg%debug)write(*,*) lat_banddeb, lat_bandfin, nbbandes


  lonFirst = -180.
  lonLast  = 360.
  kdeb = 1
  kfin = xdim

  clim % name = clim_id         ! 'albmax', 'sstmin', 'sstmoy'
  clim % nbp = kfin-kdeb+1      ! number of available pixels in array
  clim % nbl = nbline*nbbandes  ! number of available lines of array
  clim % lat1= lat_banddeb      ! latitude of the first pixel of array (deg)
  clim % lon1= lonFirst         ! longitude of the first pixel of array (deg)
  clim % lat_inc = -0.05        ! latitude increment per line array (deg)
  clim % lon_inc =  0.05        ! longitude increment per pixel of array (deg)

  if(idbg%debug) write(*,*) clim_id,' nbl Latdeb nbp Londeb: ', clim % nbl, clim % lat1, clim % nbp, clim % lon1

  clim % norm180 = .true.


  allocate ( clim % data( clim % nbp , clim % nbl) )
  clim % data(:,:)=imanq

  ! lecture du fichier par bandes
  !-------------------------    
  l=0
  do i=1,nbbandes
     num_lat=lat_banddeb - (i-1)*10
     call read_data05_h5 (idbg, file_id, clim_id, nbline, num_lat, data05, ios)
     do n=1,nbline
        l=l+1
        if(l>=1 .and. l<=clim % nbl) then
           p=0
           do k=kdeb, kfin
              p=p+1
              if( clim % name == 'albmax') then
                  if(data05(k,n)>imanq ) then
                     clim % data(p,l) = data05(k,n)/10.                 ! convert to %
                  else
                     clim % data(p,l) = vmanq
                  endif
              endif
              if(clim % name == 'sstmin' .or. clim % name == 'sstmoy') then
                  if(data05(k,n) > -400) then
                     clim % data(p,l) = (data05(k,n)+ 27315)/100.       ! convert to (K)
                  else
                     clim % data(p,l) = vmanq
                  endif
              endif
           end do
        endif
     end do
  end do

  call H5Fclose_f (file_id,ios)

#endif

END SUBROUTINE maia_Read_Clim
!******************************************************************************

#ifdef HAS_LIBHDF5

subroutine read_data05_h5  (idbg, file_id, clim_id, nbline, latdeb, data, status)
!-------------------------------------------------------------------------------
! Purpose: Subroutine to read to HDF-5 file
!-------------------------------------------------------------------------------
 USE HDF5
 USE MOD_maia_types

 IMPLICIT NONE

  type( debug ),    INTENT(in)  :: idbg         ! 
  INTEGER(HID_T), INTENT(IN)    :: file_id              ! file identifier
  character(len=6), intent(in)  :: clim_id      ! 
  INTEGER , INTENT(IN)          :: nbline  , latdeb 
  INTEGER , INTENT(OUT)         :: data(7200*nbline)
  INTEGER , INTENT(OUT)         :: status       
  INTEGER(HID_T)                :: dataset_id           ! dataset identifier
  INTEGER(HSIZE_T)              :: dims_2(2)
  INTEGER                       :: rank

  character(len=17)             ::  dataname  
! ----------------------------------------------------------------------------
  rank = 2
  dims_2(1) = 7200
  dims_2(2) = nbline    
  write(dataname,'(a6, a6,a1, i3)') "/DATA/", clim_id,"_", latdeb

  if(idbg%debug) write(*,*) dataname
  
  call H5Dopen_f (file_id, dataname, dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_INTEGER, data, dims_2, status)
  call H5Dclose_f (dataset_id, status)

  end subroutine read_data05_h5

#endif

subroutine filename_clim (yyyy,mm,dd,clim_id,filename)
!-------------------------------------------------------------------------------
! Purpose: find clim filename
! for climatology filename are
! albmax_CCC.h5 
! sstmin_MM_1|2.h5 
! sstmoy_MM_1|2.h5
!-------------------------------------------------------------------------------
   IMPLICIT NONE
    character(len=6), intent(in)  :: clim_id      !  
    integer , intent(in) :: yyyy
    integer , intent(in) :: dd
    integer, intent(in) :: mm
    character(len=14), intent(out) :: filename
    integer :: NBJY
    integer :: dg(3)
    integer :: dq(2)
    character(len=2) :: months(12) = (/'01','02','03','04','05','06', &
                                      '07','08','09','10','11','12'/)
  if (clim_id == 'sstmin' .or. clim_id == 'sstmoy') then
   if ( dd .le. 15 ) then
    write(filename,'(13a)') clim_id//"_"//months(mm)//"_1.h5"
   else 
    write(filename,'(13a)') clim_id//"_"//months(mm)//"_2.h5"
   endif 
  else
    dq(1) = yyyy
    dg(2) = mm
    dg(3) = dd
    call xdgq(dg,dq)
    NBJY = dq(2)
    if ( NBJY >= 353 )  then
          filename="albmax_353.h5" 
    else if ( NBJY >= 337 )  then 
          filename="albmax_337.h5" 
    else if ( NBJY >= 321 )  then
          filename="albmax_321.h5" 
    else if ( NBJY >= 305 )  then 
          filename="albmax_305.h5" 
    else if ( NBJY >= 289 )  then
          filename="albmax_289.h5" 
    else if ( NBJY >= 273 )  then
        filename="albmax_273.h5" 
    else if ( NBJY >= 257 )  then
         filename="albmax_257.h5" 
    else if ( NBJY >= 241 )  then
          filename="albmax_241.h5" 
    else if ( NBJY >= 225 )  then 
         filename="albmax_225.h5" 
    else if ( NBJY >= 209 )  then 
        filename="albmax_209.h5" 
    else if ( NBJY >= 193 )  then 
         filename="albmax_193.h5" 
    else if ( NBJY >= 177 )  then
         filename="albmax_177.h5" 
    else if ( NBJY >= 161 )  then
         filename="albmax_161.h5" 
    else if ( NBJY >= 145 )  then
         filename="albmax_145.h5" 
    else if ( NBJY >= 129 )  then 
         filename="albmax_129.h5" 
    else if ( NBJY >= 113 )  then
         filename="albmax_113.h5" 
    else if ( NBJY >= 97 )  then 
         filename="albmax_097.h5" 
    else if ( NBJY >= 81 )  then 
         filename="albmax_081.h5" 
    else if ( NBJY >= 65 )  then
          filename="albmax_065.h5" 
    else if ( NBJY >= 49 ) then 
           filename="albmax_049.h5" 
    else if ( NBJY >= 33 )  then 
          filename="albmax_033.h5" 
    else if ( NBJY >= 17 )  then
          filename="albmax_017.h5" 
    else  
      filename="albmax_001.h5" 
    end if
  end if
  return
end subroutine 
