! Name: $Id: maia_Read_IRThres.F90 530 2017-06-27 11:07:32Z frna $
!
! Purpose: 
!          Subroutines maia_Read_IRThres 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_Read_IRThres (tabsea, tabland, tabopaq)

! Authors       : lydie lavanant

! reads the threshold files to initializes the different thresholds

 USE MOD_MAIA_CONST
 USE MOD_maia_types
IMPLICIT NONE
      
  INTEGER, PARAMETER :: numfich =70

  type( maia_ThresTables_sea ), intent(out) :: tabsea           ! tabulated threshold tables (nb_wv,nb_secant)
  type( maia_ThresTables_land), intent(out) :: tabland          ! tabulated threshold tables (nb_wv,nb_secant)
  type( maia_ThresTables_opaq), intent(out) :: tabopaq          ! tabulated threshold tables (nb_wv,nb_secant)

  CHARACTER(LEN=80) :: THfile_Sea='THfile_Sea'          ! name of input file            
  CHARACTER(LEN=80) :: THfile_Land='THfile_Land'        ! name of input file            
  CHARACTER(LEN=80) :: THfile_Opaq='THfile_Opaq'        ! name of input file            
  INTEGER               :: isec, iw, it, ios
  integer               :: Chan1, Chan2, tm, day_time
  CHARACTER(LEN=20)     :: comment, name                
  CHARACTER(LEN=20)     :: tabname              

!=======================================================
  ios = 0
  open(unit=numfich,file=THfile_Sea, status='old', iostat=ios)
  if(ios /= 0) then
         call ml_wt('error','open file threshold')
         write(*,*) 'THfile_Sea= ', THfile_Sea
         stop 2
  endif  

  do while (ios==0) 
  read(numfich, '(2a20)', iostat=ios) comment

  if(comment(1:8)=='TAB_TYPE') then
    read ( numfich, '(a20)') tabname(1:20)
    read ( numfich, *, iostat=ios) Chan1, Chan2, tm, day_time
    do iw=1,wv_nb
       if(tabname(1:15)=='tab108_120_cold')read(70,*) (tabsea % tab108_120_cold (iw,isec),isec=1,sec_nb)
       if(tabname(1:15)=='tab108_120_warm')read(70,*) (tabsea % tab108_120_warm (iw,isec),isec=1,sec_nb)
       if(tabname(1:13)=='tabtsurf_cold')  read(70,*) (tabsea % tabtsurf_cold(iw,isec),isec=1,sec_nb)
       if(tabname(1:13)=='tabtsurf_warm')  read(70,*) (tabsea % tabtsurf_warm(iw,isec),isec=1,sec_nb)
       if(tabname(1:14)=='tab108_37_cold') read(70,*) (tabsea % tab108_37_cold (iw,isec),isec=1,sec_nb)
       if(tabname(1:14)=='tab108_37_warm') read(70,*) (tabsea % tab108_37_warm (iw,isec),isec=1,sec_nb)
       if(tabname(1:14)=='tab120_37_cold') read(70,*) (tabsea % tab120_37_cold (iw,isec),isec=1,sec_nb)
       if(tabname(1:14)=='tab120_37_warm') read(70,*) (tabsea % tab120_37_warm(iw,isec),isec=1,sec_nb)
       if(tabname(1:14)=='tab37_108_cold') read(70,*) (tabsea % tab37_108_cold (iw,isec),isec=1,sec_nb)
       if(tabname(1:14)=='tab37_108_warm') read(70,*) (tabsea % tab37_108_warm (iw,isec),isec=1,sec_nb)
       if(tabname(1:14)=='tab87_108_cold') read(70,*) (tabsea % tab87_108_cold (iw,isec),isec=1,sec_nb)
       if(tabname(1:14)=='tab87_108_warm') read(70,*) (tabsea % tab87_108_warm (iw,isec),isec=1,sec_nb)
       if(tabname(1:13)=='tab37_40_cold')  read(70,*) (tabsea % tab37_40_cold(iw,isec),isec=1,sec_nb)
       if(tabname(1:13)=='tab37_40_warm')  read(70,*) (tabsea % tab37_40_warm(iw,isec),isec=1,sec_nb)
    enddo
    read ( numfich, *, iostat=ios) 
    if(ios/=0) then
       exit
    endif
  endif
  end do
  close(numfich)

  ios = 0
  open(unit=numfich,file=THfile_Land, status='old', iostat=ios)
  if(ios /= 0) then
         call ml_wt('error','open file threshold')
         write(*,*) 'THfile_Land= ', THfile_Land
         stop 2
  endif  
  do while (ios==0) 
  read(numfich, '(2a20)', iostat=ios) comment, name
  if(comment=='TAB_TYPE') then 
    read ( numfich, '(a20)') tabname
    read ( numfich, *, iostat=ios) Chan1, Chan2, tm, day_time 
    do iw=1,wv_nb
       if(tabname(1:13)=='tab108_120_ln')read(70,*)        (tabland % tab108_120_ln (iw,isec),isec=1,sec_nb)
       if(tabname(1:13)=='tab108_120_ld')read(70,*)        (tabland % tab108_120_ld (iw,isec),isec=1,sec_nb)
       if(tabname(1:15)=='tabtsurf_ln_veg')read(70,*)      (tabland % tabtsurf_ln_veg (iw,isec),isec=1,sec_nb)
       if(tabname(1:15)=='tabtsurf_ld_veg')read(70,*)      (tabland % tabtsurf_ld_veg (iw,isec),isec=1,sec_nb)
       if(tabname(1:15)=='tabtsurf_ln_des')read(70,*)      (tabland % tabtsurf_ln_des (iw,isec),isec=1,sec_nb)
       if(tabname(1:15)=='tabtsurf_ld_des')read(70,*)      (tabland % tabtsurf_ld_des (iw,isec),isec=1,sec_nb)
       if(tabname(1:20)=='tab108_37_l_cold_veg')read(70,*) (tabland % tab108_37_l_cold_veg (iw,isec),isec=1,sec_nb)
       if(tabname(1:20)=='tab108_37_l_warm_veg')read(70,*) (tabland % tab108_37_l_warm_veg (iw,isec),isec=1,sec_nb)
       if(tabname(1:20)=='tab108_37_l_cold_des')read(70,*) (tabland % tab108_37_l_cold_des (iw,isec),isec=1,sec_nb)
       if(tabname(1:20)=='tab108_37_l_warm_des')read(70,*) (tabland % tab108_37_l_warm_des (iw,isec),isec=1,sec_nb)
       if(tabname(1:20)=='tab37_108_l_cold_veg')read(70,*) (tabland % tab37_108_l_cold_veg (iw,isec),isec=1,sec_nb)
       if(tabname(1:20)=='tab37_108_l_warm_veg')read(70,*) (tabland % tab37_108_l_warm_veg (iw,isec),isec=1,sec_nb)
       if(tabname(1:20)=='tab37_108_l_cold_des')read(70,*) (tabland % tab37_108_l_cold_des (iw,isec),isec=1,sec_nb)
       if(tabname(1:20)=='tab37_108_l_warm_des')read(70,*) (tabland % tab37_108_l_warm_des (iw,isec),isec=1,sec_nb)
       if(tabname(1:16)=='tab87_108_ln_veg')read(70,*)     (tabland % tab87_108_ln_veg (iw,isec),isec=1,sec_nb)
       if(tabname(1:16)=='tab87_108_ld_veg')read(70,*)     (tabland % tab87_108_ld_veg (iw,isec),isec=1,sec_nb)
       if(tabname(1:16)=='tab87_108_ln_des')read(70,*)     (tabland % tab87_108_ln_des (iw,isec),isec=1,sec_nb)
       if(tabname(1:16)=='tab87_108_ld_des')read(70,*)     (tabland % tab87_108_ld_des (iw,isec),isec=1,sec_nb)
       if(tabname(1:19)=='tab87_37_l_cold_veg')read(70,*)  (tabland % tab87_37_l_cold_veg (iw,isec),isec=1,sec_nb)
       if(tabname(1:19)=='tab87_37_l_warm_veg')read(70,*)  (tabland % tab87_37_l_warm_veg (iw,isec),isec=1,sec_nb)
       if(tabname(1:19)=='tab87_37_l_cold_des')read(70,*)  (tabland % tab87_37_l_cold_des (iw,isec),isec=1,sec_nb)
       if(tabname(1:19)=='tab87_37_l_warm_des')read(70,*)  (tabland % tab87_37_l_warm_des (iw,isec),isec=1,sec_nb)
       if(tabname(1:19)=='tab37_40_l_cold_veg')read(70,*)  (tabland % tab37_40_l_cold_veg (iw,isec),isec=1,sec_nb)
       if(tabname(1:19)=='tab37_40_l_warm_veg')read(70,*)  (tabland % tab37_40_l_warm_veg (iw,isec),isec=1,sec_nb)
       if(tabname(1:19)=='tab37_40_l_cold_des')read(70,*)  (tabland % tab37_40_l_cold_des (iw,isec),isec=1,sec_nb)
       if(tabname(1:19)=='tab37_40_l_warm_des')read(70,*)  (tabland % tab37_40_l_warm_des (iw,isec),isec=1,sec_nb)
    enddo
    read ( numfich, *, iostat=ios) 
    if(ios/=0) then
       exit
    endif
  endif
  end do

  close(numfich)

  ! semble que la table desert ne convienne pas!!!!
  tabland % tab87_108_ld_des (:,:) = tabland % tab87_108_ld_veg(:,:)

  ios = 0
  open(unit=numfich,file=THfile_Opaq, status='old', iostat=ios)
  if(ios /= 0) then
         call ml_wt('error','open file threshold')
         write(*,*) 'THfile_Opaq= ', THfile_Opaq
         stop 2
  endif  
  do while (ios==0) 
  read(numfich, '(2a20)', iostat=ios) comment, name
  if(comment=='TAB_TYPE') then
    read ( numfich, '(a20)') tabname
    read ( numfich, *, iostat=ios) Chan1, Chan2, tm, day_time
    if(tabname(1:11)=='tab108_tcld') then
       do it=1,10
       read(70,*)  (tabopaq % t108_tcld (it,isec),isec=1,sec_nb)
       enddo
    else
    do iw=1,wv_nb
       if(tabname(1:15)=='tab108_120_opaq') read(70,*) (tabopaq %tab108_120_opaq  (iw,isec),isec=1,sec_nb)
       if(tabname(1:14)=='tab87_108_opaq') read(70,*)  (tabopaq % tab87_108_opaq (iw,isec),isec=1,sec_nb)
       if(tabname(1:14)=='tab37_108_opaq') read(70,*)  (tabopaq % tab37_108_opaq (iw,isec),isec=1,sec_nb)
    enddo
    endif
    read ( numfich, *, iostat=ios) 
    if(ios/=0) then
       exit
    endif
  endif
  end do
  close(numfich)

END SUBROUTINE maia_Read_IRThres
