! Name: $Id: maia_Read_Topo.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_Read_Topo Read_LandSea_data Read_Elev_data Read_Landsea_header 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
subroutine maia_Read_Topo (idbg, field_id, landsea, elev)

! ------------------
! lecture de l'atlas landsea à 0.01 degree de resolution en format hdf5
! origine: igbp
!   landsea=0   ! mer
!   landsea=1   ! terre
!   landsea=2   ! desert
!   landsea=3   ! neige perenne
!   landsea=4   ! cote
!   landsea=5   ! inland water
! ------------------
! lecture de l'atlas elevation à 0.02 degree de resolution en format hdf5
! origine: gtopo30
! ------------------
#ifdef HAS_LIBHDF5
 use HDF5
#endif

 USE MOD_MAIA_CONST
 USE MOD_maia_types
 implicit none

  type( debug ),     INTENT(in)         :: idbg         ! 
  Type (field_info), INTENT(in)         :: field_id
  type( topo_field), intent(out)        :: landsea      ! surface topography landsea and elev
  type( topo_field), intent(out)        :: elev         ! surface topography landsea and elev

#ifdef HAS_LIBHDF5

  integer, allocatable  :: ls (:,:)   
  integer, allocatable  :: elev_data(:,:)
  integer       :: k, n, ll, p, l
  integer       :: num_lat, ios, nb_line, nb_pix
  integer       :: nb_band, nbline              ! number of bands read, number of lines in the band
  real          :: lat_firstBand, lat_LastBand  ! Latitude of the first line of the bands read
  real          :: lon_First          ! Latitude of the first line of the bands read
  real          :: lat_step, lon_step           ! Latitude of the first line of the bands read

  INTEGER(HID_T) :: file_id             ! file identifier
  INTEGER(HID_T) :: file_id_elev        ! file identifier

!----------------------------------------------------------------------------
 ! initialise l interface fortran
 CALL h5open_f(ios)

! ----------------------------------------------------------------------------
  ! to read the selected area and fill the landsea and elev tables
  if(MODULO(field_id % LatM_max, 10.)==0)   then
     lat_firstBand= field_id % LatM_max
  else
     lat_firstBand= ( (INT(field_id % LatM_max/10.)+1)*10 )
  endif
  lat_LastBand= (INT(field_id % LatM_min/10.)*10)
  if(field_id % LatM_max+0.5 > lat_firstBand )  lat_firstBand=lat_firstBand+10
  if(field_id % LatM_min-0.5 < lat_LastBand )   lat_LastBand =lat_LastBand-10
  
  ! manage atlas borders in lat
  if(lat_firstBand>90) lat_firstBand=90.
  if(lat_lastBand<-90) lat_lastBand=-90.

  nb_band= (lat_firstBand - lat_LastBand)/10

  if(idbg%debug) write(*,*) field_id % LatM_max, field_id % LatM_min
  if(idbg%debug) write(*,*) lat_firstBand, lat_LastBand, nb_band
  
  ! ----------------------------------------------------------------------------
  ! open du fichier landsea
  call H5Fopen_f( "landsea.h5", H5F_ACC_RDONLY_F, file_id, ios ) 
  call Read_Landsea_header (file_id, nbline, nb_pix, lon_First, lat_step, lon_step)

  nb_line= nbline*nb_band       ! all lines of the bands are kept
  
  if(idbg%debug) write(*,*) lon_First, nb_pix

  landsea % nbl     = nb_line
  landsea % nbp     = nb_pix
  landsea % lat1    = lat_firstBand
  landsea % lon1    = lon_First
  landsea % lat_inc = lat_step
  landsea % lon_inc = lon_step
  landsea % norm180 = .true.    ! norm180 : true if [-180,180[ else [0,360[

  allocate (ls (nb_pix, nbline) )
  allocate (landsea % data (nb_pix, nb_line) )
  if(idbg%debug) write(*,*) '  landsea Latdeb Londeb lat_step : ', landsea % lat1, landsea % lon1, lat_step

  ! read the DATA  par paquets de lignes
  p=0 ; l=0
  do ll=1,nb_band
     ls(:,:) = -99
     num_lat=lat_firstBand - (ll-1)*10
     call Read_LandSea_data  (idbg, file_id, nbline, num_lat, ls, ios)

     do n=1,nbline
        l= l+1
        landsea % data (1:nb_pix,l) = ls(1:nb_pix,n)
     end do
  end do

  call H5Fclose_f (file_id,ios) ! close du fichier
  deallocate (ls)
! ----------------------------------------------------------------------------
  ! open du fichier elevation
  call H5Fopen_f( "elevation.h5", H5F_ACC_RDONLY_F, file_id_elev, ios ) 
  nbline = 500
  nb_line= nbline*nb_band       ! all lines of the bands are kept
  nb_pix = 18000
  elev % nbl     = nb_line
  elev % nbp     = nb_pix
  elev % lat1    = lat_firstBand
  elev % lon1    = -180.
  elev % lat_inc = -0.02
  elev % lon_inc =  0.02 
  elev % norm180 = .true.       ! norm180 : true if [-180,180[ else [0,360[

  allocate (elev_data(18000, 500) )
  allocate (elev % data(nb_pix, nb_line) )
  if(idbg%debug) write(*,*) '  elev Latdeb Londeb lat_step : ', elev % lat1, elev % lon1, elev % lat_inc

  ! read the DATA en hdf5 par paquets de lignes
  p=0 ; l=0
  do ll=1,nb_band
     num_lat=lat_firstBand - (ll-1)*10
     call Read_Elev_data  (idbg,file_id_elev, nbline, num_lat, elev_data, ios)

     do n=1,nbline
        l= l+1
        do k=1,nb_pix
           elev % data(k,l) = elev_data(k,n)
           if(elev % data(k,l)==-99)  elev % data(k,l)= 0
        end do
     end do
  end do
  call H5Fclose_f (file_id_elev,ios)    ! close du fichier
  deallocate (elev_data)

#endif

END subroutine maia_Read_Topo

!******************************************************************************
!******************************************************************************

#ifdef HAS_LIBHDF5

subroutine Read_LandSea_data  (idbg,file_id, nbline, latdeb, landsea, status)
!-------------------------------------------------------------------------------
! Purpose: Subroutine to read to HDF-5 file
!-------------------------------------------------------------------------------
 USE HDF5
 USE MOD_maia_types

 IMPLICIT NONE

  type( debug ),  INTENT(in) :: idbg    ! 
  INTEGER(HID_T), INTENT(IN) :: file_id         ! file identifier
  INTEGER , INTENT(IN)       :: nbline  , latdeb 
  INTEGER , INTENT(OUT)      :: landsea(36000*nbline)
  INTEGER , INTENT(OUT)      :: status          

  INTEGER(HID_T) :: dataset_id          ! dataset identifier
  INTEGER(HSIZE_T)      :: dims_2(2)
  INTEGER               :: rank

  character(len=17) ::  dataname  
! ----------------------------------------------------------------------------
  rank = 2
  dims_2(1) = 36000
  dims_2(2) = nbline    
  write(dataname,'(a14, i3)') "/DATA/landsea_", latdeb
  if(idbg%debug)write(*,*) dataname
  
  call H5Dopen_f (file_id, dataname, dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_INTEGER, landsea, dims_2, status)
  call H5Dclose_f (dataset_id, status)
end subroutine Read_LandSea_data

subroutine Read_Elev_data  (idbg,file_id, nbline, latdeb, elev, status)
!-------------------------------------------------------------------------------
! Purpose: Subroutine to read to HDF-5 file
!-------------------------------------------------------------------------------
 USE HDF5
 USE MOD_maia_types

 IMPLICIT NONE

  type( debug ),  INTENT(in) :: idbg    ! 
  INTEGER(HID_T), INTENT(IN) :: file_id         ! file identifier
  INTEGER , INTENT(IN)       :: nbline  , latdeb 
  INTEGER , INTENT(OUT)      :: elev(18000*nbline)
  INTEGER , INTENT(OUT)      :: status          

  INTEGER(HID_T) :: dataset_id          ! dataset identifier

  INTEGER(HSIZE_T)      :: dims_2(2)
  INTEGER               :: rank

  character(len=14) ::  dataname  
! ----------------------------------------------------------------------------
  rank = 2
  dims_2(1) = 18000
  dims_2(2) = nbline    
  write(dataname,'(a11, i3)') "/DATA/elev_", latdeb
  if(idbg%debug)write(*,*) dataname
  
  call H5Dopen_f (file_id, dataname, dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_INTEGER, elev, dims_2, status)
  call H5Dclose_f (dataset_id, status)

  end subroutine Read_Elev_data
  
subroutine Read_Landsea_header  (file_id, nblat, nblon, lon_start, lat_step, lon_step)
!-------------------------------------------------------------------------------
! Purpose: Subroutine to read header to HDF-5 file
!-------------------------------------------------------------------------------
 use HDF5

 IMPLICIT NONE

  INTEGER(HID_T), INTENT(IN)    :: file_id              ! file identifier
  REAL, intent(out)             :: lat_step, lon_step   !latitude step
  REAL, intent(out)             :: lon_start    !
  INTEGER, intent(out)          :: nblat, nblon         !

  INTEGER(HID_T) :: dataset_id          ! dataset identifier

  INTEGER(HSIZE_T) :: dims(1)
  INTEGER ::  status
  ! ---------------------------------------------------------------------------- 
  dims(1) = 1
  call H5Dopen_f (file_id, "/HEADER/lon_start", dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_REAL, lon_start, dims, status)
  call H5Dclose_f (dataset_id, status)

  call H5Dopen_f (file_id, "/HEADER/lat_step", dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_REAL, lat_step, dims, status)
  call H5Dclose_f (dataset_id, status)
  call H5Dopen_f (file_id, "/HEADER/lon_step", dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_REAL,lon_step , dims, status)
  call H5Dclose_f (dataset_id, status)

  call H5Dopen_f (file_id,  "/HEADER/nblat", dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_INTEGER, nblat, dims, status)
  call H5Dclose_f (dataset_id, status)
  nblat=1000
  call H5Dopen_f (file_id,  "/HEADER/nblon", dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_INTEGER, nblon, dims, status)
  call H5Dclose_f (dataset_id, status)

  end subroutine read_landsea_header

#endif
