! Name: $Id: maia_Read_VISThres.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_Read_VISThres 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE  maia_Read_VISThres (thvis_sea, thvis_land, reflect)

 USE MOD_MAIA_CONST
 USE MOD_maia_types

IMPLICIT NONE

  type( maia_VISThresTables), intent(out) :: thvis_sea          ! tabulated threshold 
  type( maia_VISThresTables), intent(out) :: thvis_land         ! tabulated threshold 
  REAL, intent(out) :: reflect(nbreflecsol,nbreflecsat,nbreflecazi,3)

  CHARACTER(LEN=80) :: THfile_Sea='VIS_sea.txt'         ! name of input file            
  CHARACTER(LEN=80) :: THfile_Land='VIS_land.txt'       ! name of input file            
  CHARACTER(LEN=80) :: THfile_Refl='VIS_reflect.txt'    ! name of input file            

  INTEGER :: ios, ic, no_sat, nbcan
  CHARACTER (LEN=80) :: comment

  !local
  INTEGER       :: list_can(30), ic_vis, numcan
  REAL          :: TrO3(30), Spherediff(30)
  REAL          :: TrH2O(nbh2o), TrOxy(nboxy)
  REAL          :: transdiff(nbtrans), emisdiff(nbemissol,nbemissat,nbemisazi)  
!=======================================================

  open(unit=70,status='old',file=THfile_Land, iostat=ios)

  read(70,*) no_sat
  read(70,*) nbcan
  
  do ic=1,nbcan
    read(70,*) numcan   !comment
    read(70,*) TrH2O(:)
    read(70,*) numcan   !comment
    read(70,*) TrOxy(:)
    do ic_vis=1, nb_canvis
       if(numcan==icanvis(ic_vis)) then
          thvis_land%TrH2O(:,ic_vis) = TrH2O(:)
          thvis_land%TrOxy(:,ic_vis) = TrOxy(:)
       endif
    end do
  end do

  read(70,*) list_can(1:nbcan)
  read(70,*) TrO3(1:nbcan)
  read(70,*) list_can(1:nbcan)
  read(70,*) Spherediff(1:nbcan)
  
  do ic=1, nbcan
  do ic_vis=1, nb_canvis
     if(list_can(ic)==icanvis(ic_vis)) then
        thvis_land%TrO3(ic_vis) = TrO3(ic)
        thvis_land%Spherediff(ic_vis) = Spherediff(ic)
     endif
  end do
  end do
      
  do ic=1,nbcan
    read(70,*) numcan   !comment
    read(70,*) transdiff(:)
    read(70,*) numcan   !comment
    read(70,*) emisdiff(:,:,:)
    do ic_vis=1, nb_canvis
       if(numcan==icanvis(ic_vis)) then
          thvis_land%transdiff(:,ic_vis)    = transdiff(:)
          thvis_land%emisdiff(:,:,:,ic_vis) = emisdiff(:,:,:)
       endif
    end do
  end do 
  
  close(70)

  
  open(unit=70,status='old',file=THfile_Sea, iostat=ios)

  read(70,*) no_sat
  read(70,*) nbcan
  
  do ic=1,nbcan
    read(70,*) numcan   !comment
    read(70,*) TrH2O(:)
    read(70,*) numcan   !comment
    read(70,*) TrOxy(:)
    do ic_vis=1, nb_canvis
       if(numcan==icanvis(ic_vis)) then
          thvis_sea%TrH2O(:,ic_vis) = TrH2O(:)
          thvis_sea%TrOxy(:,ic_vis) = TrOxy(:)
       endif
    end do
  end do

  read(70,*) list_can(1:nbcan)
  read(70,*) TrO3(1:nbcan)
  read(70,*) list_can(1:nbcan)
  read(70,*) Spherediff(1:nbcan)

  do ic=1, nbcan
  do ic_vis=1, nb_canvis
     if(list_can(ic)==icanvis(ic_vis)) then
        thvis_sea%TrO3(ic_vis) = TrO3(ic)
        thvis_sea%Spherediff(ic_vis) = Spherediff(ic)
     endif
  end do
  end do
      
  do ic=1,nbcan
    read(70,*) numcan   !comment
    read(70,*) transdiff(:)
    read(70,*) numcan   !comment
    read(70,*) emisdiff(:,:,:)
    do ic_vis=1, nb_canvis
       if(numcan==icanvis(ic_vis)) then
          thvis_sea%transdiff(:,ic_vis)    = transdiff(:)
          thvis_sea%emisdiff(:,:,:,ic_vis) = emisdiff(:,:,:)
       endif
    end do
  end do
  
  close(70)

  
 ! lecture de la reflectance de la neige canaux 0.6mm, 0.9mm, 1.6mm
  open(unit=70,status='old',file=THfile_Refl, iostat=ios)
  
  do ic=1,3
    read(70,*) comment
    read(70,*) reflect(:,:,:,ic)
  end do

  close(70)
   
END SUBROUTINE maia_Read_VISThres
