! Name: $Id: maia_ReflRatio_ToObs.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_ReflRatio_ToObs 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_ReflRatio_ToObs (idbg, box, rnadtormes)

! compute reflectances ratio after bidirectional effects simulation

! the ratio of the reflectance (simulated for nadir) to
! the measured one is computed knowing the satellite, the channel,
! and the viewing conditions

! Authors       : herve legleau, marcel derrien, lydie lavanant

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ),    INTENT(in)  :: idbg         ! 
  type( box_id ),   INTENT(in)  :: box          ! lat, lon, solar and satellite angles at the center of the box
  REAL, INTENT(out) :: rnadtormes

! local
  REAL :: cozsat , sizsat , cozsol, cozsol2 , sizsol 
  REAL :: cocoz , sisiz , cofi , cogama 
  REAL :: xx , x , rayl , numer , denum
  REAL :: reflect , reflect0
  REAL :: cozsol0 , cozsat0 , cogama0 , cocoz0 , sisiz0 
  REAL :: x0 , rayl0 , numer0 , denum0 
       
  REAL , PARAMETER ::         manalo_a  = 0.018
  REAL , PARAMETER ::         manalo_b  = 1.537
  REAL , PARAMETER ::         manalo_g  = 0.550
  REAL , PARAMETER ::         manalo_k  = 0.625
  REAL , PARAMETER ::         manalo_om = 0.4
  REAL , PARAMETER ::         manalo_c2 = 0.023
  REAL , PARAMETER ::         manalo_c3 = 0.8
!-----------------------------------------------------------------------   
  if(idbg %debug) write(*,*) "maia_ReflRatio_ToObs"
  cozsat = cos ( cdr *box % satzen )
  sizsat = sin ( cdr *box % satzen )
  cozsol = cos ( cdr *box % sunzen )
  cozsol2 = cozsol * cozsol 
  sizsol = sin ( cdr *box % sunzen )
  cocoz = cozsol*cozsat
  sisiz = sizsol * sizsat
  cofi = cos ( cdr *(180. - box % difazi) )
  cogama =  sisiz * cofi - cocoz
  xx = 1. + cozsol
  xx = (xx + 2*cozsol*log(cozsol/xx) - cozsol2/xx) * cozsol
  rayl = manalo_c2 * (1.+cogama*cogama) / cocoz**manalo_c3
  x = cocoz / (cozsol + cozsat) 
  numer = 1 + manalo_k * (manalo_g + cogama) * (manalo_g + cogama)
         
  denum = 1 + manalo_k * &
         (manalo_g*manalo_g - 2*manalo_g*cocoz  + cocoz*cocoz + sisiz*sisiz/2.)
         
  reflect = manalo_om * rayl + &
            ((manalo_a + manalo_b*x*x) /cocoz) * (numer/denum)       
      
  ! compute nadir reflectance    
  cozsol0 = 1. 
  cozsat0 = 1. 
  cogama0 = -1. 
  cocoz0  = 1. 
  sisiz0  = 0. 
  x0     = cocoz0 / ( cozsol0 + cozsat0 ) 
  rayl0  = manalo_c2 * (1.+cogama0*cogama0) / cocoz0**manalo_c3
  numer0 = 1 + manalo_k * (manalo_g + cogama0) * (manalo_g + cogama0)
         
  denum0   = 1 + manalo_k * &
             (manalo_g*manalo_g - 2*manalo_g*cocoz0  + cocoz0*cocoz0 + sisiz0*sisiz0/2.)
  reflect0 = manalo_om * rayl0 + &
            ((manalo_a + manalo_b*x0*x0) /cocoz0) * (numer0/denum0)
      
  ! compute ratio         
  rnadtormes = reflect0 / reflect
             
END SUBROUTINE maia_ReflRatio_ToObs
