! Name: $Id: maia_SnowIce_surf.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_SnowIce_surf 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_SnowIce_surf (idbg, box, pix, snowice_surf) 

! Authors       : lydie lavanant

! flag la surface couverte par de la glace ou de la neige

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ), INTENT(in)     :: idbg         ! 
  type( box_id ), INTENT(in)    :: box          ! 
  type( pix_data), INTENT(in)   :: pix          ! 
  integer , INTENT(out)         :: snowice_surf
  
! local  
  integer       :: init_fl, ice_fl
  real          :: s_snow_37_108, ndsi
!======================================================================
   !    --------------------------
   !    Initialisation
   !    --------------------------
  init_fl =-99
  ice_fl=-99    

  if(box % night) then
     ice_fl=0 ! 
     if( box % snow_prev > 0.02 .or. box % seaice_prev >0.1) ice_fl=1   ! snow/ice
     if( idbg%debug) write(*,*) 'maia_SnowIce_surf: ', ice_fl
  else !day: 

   if(pix % Qual_108 .and. pix % Obs_108> BG_snow_108) then
      ice_fl=0
   else

      if(pix % Qual_08 .and. pix % Obs_05>0 .and. pix % Obs_16>0) then

         ndsi = (pix % Obs_05-pix % Obs_16)/(pix % Obs_05+pix % Obs_16)
         if(ndsi > BG_ndsi_snow .and. pix % Obs_08>BG_snow_08) then

            ice_fl=1    ! snow/ice

            s_snow_37_108 = BG_snow_37_108_HiElev
            if(box % alt <=cst_HiElev) s_snow_37_108 = BG_snow_37_108_LoElev

            if(pix % Obs_13>0 .and. pix % Obs_13> BG_snow_thinCi13)  ice_fl=-99
            if((box % sea .or. box % lake) .and. .not. box % glint) ice_fl=-99
            if(pix % Qual_87 .and. pix % Qual_108 .and. &
               pix % Obs_87-pix % Obs_108> BG_snow_87_108 )         ice_fl=-99
            if(pix % Qual_37 .and. pix % Qual_108 .and. &
               pix % Obs_37-pix % Obs_108> s_snow_37_108 )          ice_fl=-99

         else
            ice_fl=0
         endif  !ndsi_snow
      else
         ice_fl=-99     ! unknown snow/ice
      endif     ! Qual_08
   endif        ! 108
  endif

   if(ice_fl >=0) then
        snowice_surf =ice_fl
   elseif(ice_fl ==-99 .and. init_fl ==-99) then
        snowice_surf=0
   elseif(ice_fl ==-99 .and. init_fl >=0) then
        snowice_surf =init_fl
   endif

END SUBROUTINE maia_SnowIce_surf
