! Name: $Id: maia_ValMin.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_ValMin maia_ValMin2 maia_ValMinMaxLon 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/05/2015   Pascale Roquet bug fixes
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
!=====================================================================================
subroutine maia_ValMin (idbg, tab, valmanq, valmin)
!=====================================================================================

!BUT/PURPOSE 
!       computation of min value on the tab's.
!       missing values are not used.

USE MOD_maia_types
  implicit none     
  integer, parameter    :: nx=7, ny=7           ! size of input tab

  type( debug ), INTENT(inout)  :: idbg  
  real, intent(in)              :: tab (nx,ny)
  real, intent(in)              :: valmanq
  real, intent(out)             :: valmin

  real  :: tab_tmp (nx,ny)

  if(idbg %debug) write(*,*) "maia_ValMin"
  tab_tmp = tab
  WHERE(tab_tmp == valmanq)  tab_tmp = 999999.
  valmin = MINVAL(tab_tmp)

  if(valmin > 999990.) valmin=valmanq
END SUBROUTINE maia_ValMin

!=====================================================================================
subroutine maia_ValMin2 (idbg, tab, Nb_pixels, Nb_Lines, valmanq, valmin)
!=====================================================================================

!BUT/PURPOSE 
!       computation of min value on the tab's.
!       missing values are not used.

USE MOD_maia_types
  implicit none     

  type( debug ), INTENT(in)     :: idbg  
  integer, intent(in)           :: Nb_pixels, Nb_Lines          ! size of input tab
  real, intent(in)              :: tab (Nb_pixels, Nb_Lines)
  real, intent(in)              :: valmanq
  real, intent(out)             :: valmin

  real  :: tab_tmp (Nb_pixels, Nb_Lines)
  if(idbg %debug) write(*,*) "maia_ValMin2"
  tab_tmp = tab
  WHERE(tab_tmp <= valmanq)  tab_tmp = 99999.
  valmin = MINVAL(tab_tmp)

  if(valmin > 99990.) valmin=-9999.
END SUBROUTINE maia_ValMin2
!=====================================================================================
subroutine maia_ValMinMaxLon (idbg, tab, Nb_pixels, Nb_Lines, valmanq, valmin, valmax)
!=====================================================================================

!BUT/PURPOSE 
!       should be used for min longitude when granule is over 180°
!       computation of min value on the tab's.
!       missing values are not used.

USE MOD_maia_types
  implicit none     

  type( debug ), INTENT(in)     :: idbg  
  integer, intent(in)           :: Nb_pixels, Nb_Lines          ! size of input tab
  real, intent(in)              :: tab (Nb_pixels, Nb_Lines)
  real, intent(in)              :: valmanq
  real, intent(out)             :: valmin
  real, intent(out)             :: valmax

  real  :: tab_tmp (Nb_pixels, Nb_Lines)
  if(idbg %debug) write(*,*) "maia_ValMinMaxlon"
  
    tab_tmp = tab
    WHERE(tab_tmp > valmanq .and. tab_tmp < 0. ) tab_tmp=tab+360.
    valmax= MAXVAL(tab_tmp) 
    WHERE(tab_tmp <= valmanq)  tab_tmp = 99999.
    valmin= MINVAL((tab_tmp))
    
    if(valmin > 99990.) valmin=-9999.
 


END SUBROUTINE maia_ValMinMaxLon
