! Name: $Id: maia_VerifMissing_fields.F90 530 2017-06-27 11:07:32Z frna $
!
! Purpose: check if some fields are missing
!          Subroutines maia_VerifMissing_fields 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/01/2014   Lydie Lavanant
!  1.1    01/01/2014   Lydie Lavanant Add control for missing channels; use 0.01 degres landsea atlas; update definition of CloudMask (bits 15-18)
!  1.2    01/11/2014   Pascale Roquet Bug fix :  granules containing 47 scans were not properly processed
!  1.3    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.4    01/05/2015   Lydie Lavanant/Pascale Roquet bug fixes
!  1.5    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.6    23/12/2016   Lydie Lavanant clean code
!  1.7    04/05/2017   Pascale Roquet suppress control on I fields (to allow CVIIRS processing)
!---------------------------------------------------------------------------  
!> maia_VerifMissing_fields subroutine
!
!> @author      lydie lavanant

!> </dl>
!> @brief       Determine conditions of the path (day, night, twilight and 
!> @brief       check if mandatory fields are presents <br>
!---------------------------------------------------------------------------  
subroutine maia_VerifMissing_fields (idbg, field_M, field_I, missing )

 USE MOD_MAIA_CONST
 USE mod_maia_types
IMPLICIT NONE

  type( debug ), intent(in)     :: idbg         ! 
  Type (field),  intent(in)     :: field_M
  Type (field),  intent(in)     :: field_I
  logical, intent(out)          :: missing
  
  logical :: day, night, twilight
  logical :: missing_M(12), missing_I(5)
  integer :: lig, pix, ic
  real :: max_sunzen, min_sunzen
  character(len=3) :: Mcan(12)=(/' 04',' 05',' 06',' 08',' 13',' 16',' 22',' 37',' 40',' 87','108','120'/)
  character(len=3) :: Ican(5)=(/' 06',' 08',' 16',' 37','108'/)

  if(idbg %debug) write(*,*) "maia_VerifMissing_fields"

  day = .false. ;  night = .false. ;  twilight = .false.
  max_sunzen= maxval ( field_M % sunzen)
  min_sunzen=max_sunzen
  
  do lig=1, field_M % Nb_Lines
  do pix=1, field_M % Nb_pixels
    if (field_M % sunzen(pix,lig) > 0) then
      if (min_sunzen > field_M % sunzen(pix,lig)) then
          min_sunzen=  field_M % sunzen(pix,lig)
        endif
      endif
  end do
  end do

  if( max_sunzen <cst_sunzenmax_day ) then            
        day      =.true.
  elseif (min_sunzen> cst_sunzenmin_night ) then                        
        night    =.true.
  else                         
        twilight =.true.
  endif

  missing_M(:) = .true.
  do lig=1, field_M % Nb_Lines
  do pix=1, field_M % Nb_pixels
     do ic =1, 12
        if( field_M % Qual_Obs(ic,pix,lig) ==0 .and. field_M % Obs(ic,pix,lig)>0 ) missing_M(ic) = .false.
     end do
  end do
  end do
  
  missing_I(:) = .true.
  do lig=1, field_I % Nb_Lines
  do pix=1, field_I % Nb_pixels
     do ic =1, 5
        if( field_I % Qual_Obs(ic,pix,lig) ==0 .and. field_I % Obs(ic,pix,lig)>0 ) missing_I(ic) = .false.
     end do
  end do
  end do
  
  if ( twilight ) then
      ! if whe have ir but not vis reconsider night or twilight
      if ( ( missing_M(3) .or. missing_M(4)  )  .and. &
            (min_sunzen> 89 ) ) then
        
        twilight=.false.
        night=.true.
      endif
  endif
  missing=.false.
  if(night) then
    if(missing_M(8) .or. missing_M(11) )                       missing = .true.        ! channels M+I 3.7mm, M10.8mm, 
  elseif (day .or. twilight) then
    if(missing_M(3) .or. missing_M(4) .or. missing_M(11))      missing = .true.        ! channels M+I 0.6mm, 0.8mm, 10.8mm
  else
    write(*,*) 'Pb in maia_VerifMissing_fields'
    call exit(2)
  endif
  
  if(missing) then
     if(night) then
     write(*,*) 'Night necessary M fields missing (False/True) : ', Mcan(8),missing_M(8),Mcan(11),missing_M(11)
     elseif (day) then
     write(*,*) 'Day necessary M fields missing (False/True) : ', Mcan(3),missing_M(3),Mcan(4),missing_M(4),Mcan(11),missing_M(11)
     else
     write(*,*) 'Twilight necessary M missing (False/True) : ', Mcan(3),missing_M(3),Mcan(4),missing_M(4),Mcan(11),missing_M(11)
     endif
  endif
  
end subroutine maia_VerifMissing_fields
