! Name: $Id: maia_reset_CMa.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_reset_CMa 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_reset_CMa (idbg, Cma)

! Authors       : lydie lavanant

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ),    INTENT(in) :: idbg          ! 
  type( maia_CMa), INTENT(out):: CMa    ! 
          
!-----------------------------------------------------------------------
  if(idbg %debug) write(*,*) "maia_reset_CMa"
  CMa % clear                   =imanq  ! clear flag (0= clear, 1= cloudy, 3= snow, 4= ice)
  CMa % conf_clear              =imanq  ! final clear confidence level: 
                                        !       3=confident clear / 2=probably clear /
                                        !       1=probably cloudy / 0=confident cloudy
  CMa % conf_value              =vmanq  ! confidence value 
  CMa % qual                    =imanq  ! test serie quality: 0=high / 1=medium / 2=poor / 3=bad
  CMa % texture                 =.false.        ! 
  CMa % snowice_fl              =imanq  ! no=0 / yes=1
  CMa % shadow_fl               =imanq  ! no=0 / yes=1
  CMa % test_name               ='xxxxxxx'      ! 'CMa_SD','CMa_SN','CMa_SG','CMa_ST',
  CMa % Cld_108                 =.false.        ! Gr 1          true if cloudy
  CMa % Cld_37_40               =.false.        ! Gr 2.1
  CMa % Cld_108_37              =.false.        ! Gr 2.2
  CMa % Cld_87_108              =.false.        ! Gr 2.3
  CMa % Cld_87_37               =.false.        ! Gr 2.3
  CMa % Cld_37_108              =.false.        ! Gr 2.4
  CMa % Cld_37_120              =.false.        ! Gr 2.4
  CMa % Cld_120_37              =.false.        ! Gr 2.5
  CMa % Cld_06                  =.false.        ! Gr 3.1
  CMa % Cld_08                  =.false.        ! Gr 3.1
  CMa % Cld_06_08               =.false.        ! Gr 3.2
  CMa % Cld_LowSunGlint         =.false.        ! Gr 3.3
  CMa % Cld_13                  =.false.        ! Gr 4
  CMa % Cld_16                  =.false.        ! Gr 4
  CMa % Cld_108_120             =.false.        ! Gr 5
  CMa % confcl_108              =vmanq_plus     ! Gr 1   individual clear confidence level from 0. to 1.
  CMa % confcl_37_40            =vmanq_plus     ! Gr 2.1
  CMa % confcl_108_37           =vmanq_plus     ! Gr 2.2
  CMa % confcl_87_108           =vmanq_plus     ! Gr 2.3
  CMa % confcl_87_37            =vmanq_plus     ! Gr 2.3
  CMa % confcl_37_108           =vmanq_plus     ! Gr 2.4
  CMa % confcl_37_120           =vmanq_plus     ! Gr 2.4
  CMa % confcl_120_37           =vmanq_plus     ! Gr 2.5
  CMa % confcl_06               =vmanq_plus     ! Gr 3.1
  CMa % confcl_08               =vmanq_plus     ! Gr 3.1
  CMa % confcl_06_08            =vmanq_plus     ! Gr 3.2
  CMa % confcl_LowSunGlint      =vmanq_plus     ! Gr 3.3
  CMa % confcl_13               =vmanq_plus     ! Gr 4
  CMa % confcl_108_120          =vmanq_plus     ! Gr 5
  CMa % confcl_texture1         =vmanq_plus     ! Gr 6
  CMa % confcl_texture2         =vmanq_plus     ! Gr 6
  CMa % Adjacent_conf_clear     =imanq  ! clear confidence level from Neighbors: 
                                        !       0=confident clear / 1=probably clear /
                                        !       2=probably cloudy / 3=confident cloudy
 
END SUBROUTINE maia_reset_CMa
