! Name: $Id: maia_setup.F90 578 2018-01-22 10:35:59Z frna $
!
! Purpose: 
!          Subroutines maia_setup 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.3    01/06/2017   Pascale Roquet update for old NOAA
!  1.4    13/12/2017   Pascale Roquet update future satellites

SUBROUTINE maia_setup (idbg, field_id, tabsea, tabland, tabopaq, &
                      thvis_sea, thvis_land, reflec)

! Authors       : lydie lavanant

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

  type( debug ),                 INTENT(in) :: idbg             ! 
  Type (field_info), INTENT(in)             :: field_id
  type( maia_ThresTables_sea ), intent(out) :: tabsea           ! tabulated threshold tables (nb_wv,nb_secant)
  type( maia_ThresTables_land), intent(out) :: tabland          ! tabulated threshold tables (nb_wv,nb_secant)
  type( maia_ThresTables_opaq), intent(out) :: tabopaq          ! tabulated threshold tables (nb_wv,nb_secant)
  type( maia_VISThresTables), intent(out) :: thvis_sea          ! tabulated threshold 
  type( maia_VISThresTables), intent(out) :: thvis_land         ! tabulated threshold 
  REAL, intent(out) :: reflec (nbreflecsol,nbreflecsat,nbreflecazi,3)


  INTEGER, PARAMETER :: iu =70
  CHARACTER(LEN=80)  :: SSTfile='SSTfile.txt'   ! name of parameters file
  CHARACTER(LEN=3)  :: SAT, mission             ! name of id            
  CHARACTER(LEN=5)  :: com1, com2       ! name of id            
  real :: a1, a2, a3, a4, a5, a6, a7
  real :: b1, b2, b3, b4, b5, b6
  
  INTEGER :: ios
! ---------------------------------------------------------------------------
! some constants are instruments dependant
  if(field_id % ImgName =='viirs') then
     cst_ice16             = cst_ice16_viirs
     cst_sunzenmax_With16  = cst_sunzenmax_With16_viirs
     cst_sunzenmax_No16    = cst_sunzenmax_No16_viirs
     cst_108ClearMin       = cst_108ClearMin_viirs
     alb_snow_offset2      = alb_snow_offset2_viirs
  elseif(field_id % ImgName =='avhrr') then
     cst_ice16             = cst_ice16_avhrr
     cst_sunzenmax_With16  = cst_sunzenmax_With16_avhrr
     cst_sunzenmax_No16    = cst_sunzenmax_No16_avhrr
     cst_108ClearMin       = cst_108ClearMin_avhrr
     alb_snow_offset2      = alb_snow_offset2_avhrr
  endif

! fill IR thresholds tables
  call maia_Read_IRThres (tabsea, tabland, tabopaq)
  if( idbg % debug) write(*,*) 'maia_Read_IRThres passe'
          
! coeffs for Visible absorption
  call maia_Read_VISThres (thvis_sea, thvis_land, reflec )
  if( idbg % debug) write(*,*) 'maia_Read_VISThres passe'
   
! coeffs for phase determination
!  call maia_Read_PhaseThres (thphase )

! coefs SST 
  ios = 0
  open(unit=iu,file=SSTfile, status='old', iostat=ios)
  if(ios /= 0) then
         call ml_wt('error','open SST file')
         write(*,*) 'SSTfile= ', SSTfile
         stop 2
  endif  

  if(field_id % mission=='noaa16') mission(1:3)='N16'
  if(field_id % mission=='noaa17') mission(1:3)='N17'
  if(field_id % mission=='noaa18') mission(1:3)='N18'
  if(field_id % mission=='noaa19') mission(1:3)='N19'
  if(field_id % mission=='jpss1') mission(1:3)='N20'
  if(field_id % mission=='jpss2') mission(1:3)='N21'
  if(field_id % mission=='meto01') mission(1:3)='M01'
  if(field_id % mission=='meto02') mission(1:3)='M02'
  if(field_id % mission=='meto03') mission(1:3)='M03'
  if(field_id % mission=='npp')    mission(1:3)='NPP'


  do while (ios==0) 
    read ( iu, '(a3)', iostat=ios) SAT
    read ( iu, *, iostat=ios) com1, a1, a2, a3, a4, a5, a6, a7
    read ( iu, *, iostat=ios) com2, b1, b2, b3, b4, b5, b6
    read ( iu, *, iostat=ios) 
    
    if(SAT(1:3)==  mission(1:3)) then
      a_nl= a1
      b_nl= a2
      c_nl= a3 
      d_nl= a4 
      e_nl= a5
      f_nl= a6
      g_nl= a7
      a_37= b1
      b_37= b2
      c_37= b3
      d_37= b4
      e_37= b5
      f_37= b6
    endif
    if(ios/=0) then
       exit
    endif
  end do
  close(iu)

END SUBROUTINE maia_setup
