! Name: $Id: mk_voisinage.F90 541 2017-08-11 09:09:50Z frna $
!
! Purpose: 
!          Subroutines mk_voisinage geogcart sort_distance 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/01/2014   Lydie Lavanant
!  1.1    01/01/2014   Lydie Lavanant Add control for missing channels; use 0.01 degres landsea atlas; update definition of CloudMask (bits 15-18)
!  1.2    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.3    01/05/2015   Lydie Lavanant/Pascale Roquet bug fixes
!  1.4    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.5    08/08/2017   Pascale Roquet optimisation on memory ressources
!=====================================================================================
SUBROUTINE mk_voisinage (idbg, field_M)
!=====================================================================================

 USE MOD_MAIA_CONST
 USE mod_maia_types
IMPLICIT NONE

  type( debug ), intent(in)     :: idbg          
  Type (field),intent(inout)    :: field_M

  real(kind=8) :: lat, lon
  REAL(KIND=8), allocatable :: pos_viirs(:,:,:)     ! position XYZ
  real(KIND=8) :: dist_tmp
  real :: dist(600)
  integer :: i, k, p, l, pix, lig, pp, ll, nk, kp(600), kl(600), indx(8), kp_sel(8), kl_sel(8)

  integer :: no_pix(5), nb_lig(5), ilsize_inf, ilsize_sup, ip, max_kp(V_maxfovs), max_kl(V_maxfovs)
  real    :: dist_max(5), dist_size, max_dist(V_maxfovs), dist_sel
!----------------------------------------------------------------- 

  dist(:)=vmanq
  no_pix=(/640,1008,2192,2560,3200/)
  nb_lig=(/11,   9,   5,  9,  11/)
  dist_max=(/3., 2.,  2.,  2.,  3./)    !kms
  max_kp(:)=-9 ; max_kl(:)=-9 ; max_dist(:)=-9
!----------------------------------------------------------------- 
  if (field_M % ImgName == 'avhrr') then
    allocate (field_M % pix_texture(8,Avhrr_maxfovs,field_M% Nb_Lines))
    allocate (field_M % lig_texture(8,Avhrr_maxfovs,field_M% Nb_Lines))
  else
    allocate (field_M % pix_texture(8,V_maxfovs,field_M% Nb_Lines))
    allocate (field_M % lig_texture(8,V_maxfovs,field_M% Nb_Lines))
  endif
  field_M % pix_texture(:,:,:) = vmanq
  field_M % lig_texture(:,:,:) = vmanq

  if(field_M % ImgName == 'viirs') then

   allocate (pos_viirs(3,V_maxfovs, field_M% Nb_Lines))  
   pos_viirs(:,:,:)=vmanq


  ! converts geographical coordinates into cartesian coordinates
  do lig=1, field_M% Nb_Lines
  do pix=1, V_maxfovs
     if(field_M % lat(pix,lig)>-990. .and. field_M % lon(pix,lig)>-990.) then
        lat = field_M % lat(pix,lig) * cdr
        lon = field_M % lon(pix,lig) * cdr
        call geogcart(lat,lon, pos_viirs(:,pix,lig))
     endif
  end do
  end do

  ! calcul de distance in km
  do lig=2, field_M% Nb_Lines-1
  do pix=1, V_maxfovs
     dist(:)=vmanq
     k=0
     nk=0
     if(pos_viirs(1,pix,lig)>vmanq) then
     do ip=1,5
        if(pix <=no_pix(ip) ) then
           ilsize_inf = 0-nb_lig(ip)    ! la taille de la boite en lignes depend de la position du pixel
           ilsize_sup = nb_lig(ip)
           dist_size=dist_max(ip)
           exit
        endif
     enddo
     if(pix==1 .or. pix==3200.or. lig==1 .or. lig==768) dist_size=5.
     do l=ilsize_inf,ilsize_sup
     do p=-3,3
        pp=pix+p
        ll=lig+l
        if(pp==pix .and. ll==lig) then
           !pixel central
        elseif(pp>0 .and. pp<=V_maxfovs  .and. &
               ll>0 .and. ll<=field_M% Nb_Lines  .and. &
               lig <= field_M% Nb_Lines ) then
               if ( pos_viirs(1,pp,ll)>vmanq .and. pos_viirs(1,pix,lig)>vmanq) then
                  dist_tmp =dsqrt( (pos_viirs(1,pix,lig)-pos_viirs(1,pp,ll))**2 + &
                                   (pos_viirs(2,pix,lig)-pos_viirs(2,pp,ll))**2 + &
                                   (pos_viirs(3,pix,lig)-pos_viirs(3,pp,ll))**2) 
             
                  if(dist_tmp<dist_size) then
                      k=k+1
                      dist(k) = dist_tmp
                      kp(k)=pp
                      kl(k)=ll
               endif
              endif 
        endif
     end do
     end do
     nk=k

     ! on selectionne les 8 plus petites distances
     call sort_distance(dist(1:nk),nk,indx )
     
     if(nk>8) nk=8
     ! test if we have found a point 
     do i=1,nk 
      if ( indx(i) .gt. 1  .and. indx(i) .lt. 600 ) then
        field_M % pix_texture(i,pix,lig) = kp(indx(i))
        field_M % lig_texture(i,pix,lig) = kl(indx(i))
       endif
     end do
     if(idbg%debug ) then
        if(nk>8) nk=8
        kp_sel(:) = 0 ; kl_sel(:) = 0
        kp_sel(1:nk) = kp(indx(1:nk)) -pix
        kl_sel(1:nk) = kl(indx(1:nk)) -lig
        dist_sel = dist(indx(nk))
        if (maxval(ABS(kp_sel(1:nk))) > max_kp(pix)) max_kp(pix)= maxval(ABS(kp_sel(1:nk)))
        if (maxval(ABS(kl_sel(1:nk))) > max_kl(pix)) max_kl(pix)= maxval(ABS(kl_sel(1:nk)))
        if (dist_sel                  > max_dist(pix)) max_dist(pix)= dist_sel
     endif

     endif
  end do
  end do

  if(idbg%debug ) then
     write(*,*) 'max_kp'
     write(*,'(40i3)') max_kp(1), max_kp(320),max_kp(640), max_kp(860),max_kp(1008),max_kp(1300),max_kp(1600)
     write(*,*) 'max_kl'
     write(*,'(40i3)') max_kl(1), max_kl(320),max_kl(640), max_kl(860),max_kl(1008),max_kl(1300),max_kl(1600)
     write(*,*) 'dist_sel'
     write(*,'(40f4.1)') max_dist(1), max_dist(320),max_dist(640),max_dist(860),max_dist(1008),&
                         max_dist(1300),max_dist(1600)
     do pix=1, V_maxfovs
        if(max_kp(pix)==1 .and. max_kl(pix)==1) write(*,*) pix
     enddo
  endif

  deallocate(pos_viirs)

  elseif(field_M % ImgName == 'avhrr') then
     do lig=1, field_M% Nb_Lines
     do pix=1, 2048
        k =0
        do ll = 1,3
        do pp = 1,3
           if(pp==2 .and. ll==2) then
           !pixel central
           else
              k=k+1
              p=pp+pix-2
              if(p < 1) p=1
              if(p > 2048) p=2048
              field_M % pix_texture(k,pix,lig)=p
              l=ll+lig-2
              if(l < 1) l=1
              if(l>field_M% Nb_Lines) l=field_M% Nb_Lines
              field_M % lig_texture(k,pix,lig)=l
           endif
        end do
        end do
     end do
     end do

  else
     write(*,*) 'PB ImgName'
  endif

end subroutine mk_voisinage

!=====================================================================================
subroutine geogcart (lat,lon,pos)
!=====================================================================================

!       converts geographical coordinates into cartesian coordinates
! 
!       input and output coordinates are expressed in Greenwich reference frame:
!         X: in equatorial plane toward Greenwich meridian
!         Y: in equatorial plane toward lon = 90 degrees east
!         Z: terrestrial polar axis

implicit none                                                     

  real(kind=8), intent(in) ::lat                ! geographic latitude           (rad)
  real(kind=8), intent(in) ::lon                ! longitude                     (rad)
  real(kind=8), intent(out) ::pos(3)            ! cartesian position            (km)
!
  real(kind=8), parameter :: req  =  6378.140           ! earth equatorial radius       (km)
  real(kind=8), parameter :: flat  = 1. / 298.25722     ! earth flattening factor
  real(kind=8), parameter :: rpo = ( 1. - flat ) * req  ! earth polar radius            (km)
  real(kind=8), parameter :: re2 = req * req            ! req*req
  real(kind=8), parameter :: rp2 = rpo * rpo            ! rpo*rpo
  real(kind=8), parameter :: re2_rp2 = re2 / rp2        ! re2 / rp2
  real(kind=8), parameter :: rp2_re2= rp2 / re2         ! rp2 / re2
  real(kind=8), parameter :: exct2 = 1. - rp2_re2       ! square of earth eccentricity
  real(kind=8), parameter :: earthrot =7.292115147e-5   ! earth rotation in rad / sec
  real(kind=8), parameter :: mu =398600.64              ! gravitationnal constant km**3/s**2

  real(kind=8) ::n              ! normal to ellipsoid 
  real(kind=8) ::slat, clat     ! latitude sinus and cosinus 
  real(kind=8) ::alt=0.         ! altitude                      (km)
!-
  slat = dsin(lat)
  clat = dcos(lat)
  n = req / dsqrt( 1. - exct2 * slat * slat)
  pos(1) = ( n + alt ) * clat * cos(lon)
  pos(2) = ( n + alt ) * clat * sin(lon)
  pos(3) = ( n * (1 - exct2) + alt ) * slat

end subroutine geogcart

!=====================================================================================
subroutine sort_distance(x, n, indx)
!=====================================================================================

implicit none                                                     

  integer, intent(in) :: n
  real, intent(in) :: x(n)
  integer, intent(out) :: indx(8)

  real :: tmp(n), xmin
  integer :: k, i

  indx=-9
  tmp =x

  do k=1,8
     xmin=999.
     do i=1, n
        if (tmp(i)<xmin) then
           xmin=tmp(i)
           indx(k)=i
        endif
    enddo
    if(k==n) exit
    if ( indx(k) .le. n .and. indx(k) .gt.1 )  tmp( indx(k))= 9999.  
  end do


end subroutine sort_distance
!=====================================================================================
