! Name: $Id: mod_maia_const.F90 514 2017-02-06 11:28:14Z frna $
!
! Purpose: maia attributes definition
!          Subroutines 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    23/01/2017   Pascale Roquet creation
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.3    23/12/2016   Lydie Lavanant new definitions for number of testshg
MODULE mod_maia_const

! Authors       : lydie lavanant

  ! ---------------------------------------------------------------------------------------------------
  INTEGER, PARAMETER ::   Max_Scans_InGran = 48 ! max number of scans in granules

  ! VIIRS-M characteristics per granule
  INTEGER, PARAMETER ::   Max_Mlines_InGran = 768       ! max number of lines in Granule (=V_Scans*V_Det)
  INTEGER, PARAMETER ::   MDet_InScan = 16
  INTEGER, PARAMETER ::   V_maxfovs  = 3200     ! maximun number of Moderate pixels per line
  
  ! VIIRS-I characteristics per granule
  INTEGER, PARAMETER ::   Max_Ilines_InGran = 1536      ! max number of lines in Granule (=V_Scans*V_Det)
  INTEGER, PARAMETER ::   IDet_InScan = 32
  INTEGER, PARAMETER ::   V_maxfovs_I  = 6400   ! maximun number of Imager pixels per line

  ! VIIRS fields retrieved : 04,05,06,08,13,16,22,37,40,87,108,120
  INTEGER, PARAMETER :: V_maxch_M     = 12      ! maximun number of channels
  INTEGER, PARAMETER :: ic_Viirs_M (V_maxch_M)=(/1,4,5,7,9,10,11,12,13,14,15,16/) 

  ! ---------------------------------------------------------------------------------------------------
  ! AVHRR characteristics
  INTEGER, PARAMETER :: Avhrr_maxfovs = 2048    ! number of AVHRR pixels per line
  INTEGER, PARAMETER :: Avhrr_maxch  = 6      ! number of AVHRR channels
  INTEGER, PARAMETER :: Avhrr_ic (Avhrr_maxch)=(/1,2,3,4,5,6/) 

  ! ---------------------------------------------------------------------------------------------------           
  logical         :: norm180 =.true.    ! norm180 : true if [-180,180[ else [0,360[

  REAL, PARAMETER :: pi   = 3.1415926535
  REAL, PARAMETER :: cdr   = pi/180.0
  REAL, PARAMETER :: crd   = 180.0/pi

  INTEGER, PARAMETER :: imanq = -9999
  REAL, PARAMETER ::    vmanq = -9999.
  REAL, PARAMETER ::    vmanq_plus = 9999.
                
  ! Limits of area
  INTEGER, PARAMETER :: mountain  =1500         ! to determine mountain
  REAL, PARAMETER :: aladin_limit_N =  60.0     ! Aladin North limit
  REAL, PARAMETER :: aladin_limit_S =  36.0     ! Aladin South limit
  REAL, PARAMETER :: aladin_limit_E =  20.0     ! Aladin East  limit
  REAL, PARAMETER :: aladin_limit_W = -20.0     ! Aladin West  limit
  REAL, PARAMETER :: europe_limit_N =  90.0     ! Europe North limit
  REAL, PARAMETER :: europe_limit_S =  36.0     ! Europe South limit
  REAL, PARAMETER :: europe_limit_E =  60.0     ! Europe East  limit
  REAL, PARAMETER :: europe_limit_W = -30.0     ! Europe West  limit
  REAL, PARAMETER :: africa_limit_N =  34.0     ! Africa North limit
  REAL, PARAMETER :: africa_limit_S = -35.5     ! Africa South limit
  REAL, PARAMETER :: africa_limit_E =  60.0     ! Africa East  limit
  REAL, PARAMETER :: africa_limit_W = -18.0     ! Africa West  limit

! ----------------------------------------------------------------------------------------
  INTEGER, PARAMETER :: n_rttov = 51
  REAL, PARAMETER :: p_rttov(51) = (/&
    0.005,    0.010,    0.100,    0.200,    0.500,    0.800,    1.200,    1.600,    2.200,    2.700,  &
    3.500,    4.200,    5.000,    6.950,   10.370,   14.810,   20.400,   27.260,   35.510,   45.290,  &
   56.730,   69.970,   85.180,  102.050,  122.040,  143.840,  167.950,  194.360,  222.940,  253.710,  &
  286.600,  321.500,  358.280,  396.810,  436.950,  478.540,  521.460,  565.540,  610.600,  656.430,  &
  702.730,  749.120,  795.090,  839.950,  882.800,  922.460,  957.440,  985.880, 1005.430, 1025.000,  &
 1050.000/)  

! ----------------------------------------------------------------------------------------
! Visible thresholds
! ----------------------------------------------------------------------------------------
  ! thvis_land et thvis_sea: Nb chan used ; List of channels
  INTEGER, PARAMETER :: nb_canvis = 5
  INTEGER, PARAMETER :: icanvis(nb_canvis) = (/04,06,08,13,16/)

  ! for CoxMunk
  REAL, PARAMETER    :: longueuronde(nb_canvis)=(/0.4,0.6,0.9,1.3,1.6/)

  ! snow reflectance (LeRoux)
  INTEGER, PARAMETER :: ican_refl(3) = (/06,07,16/)

  ! chart size for transmittance H2o
  INTEGER, PARAMETER :: nbh2o = 391
  REAL,    PARAMETER :: H2oMin=10.00, H2oMax=400.00, H2oPas=1.00

  ! chart size for transmittance O2Co2...
  INTEGER, PARAMETER :: nboxy = 32
  REAL,    PARAMETER :: OxyMin=2.00 ,OxyMax=9.75 ,OxyPas=0.25

  ! chart size and min/max/step for diffusion
  INTEGER, PARAMETER :: nbtrans = 19
  INTEGER, PARAMETER :: nbemissat = 17
  INTEGER, PARAMETER :: nbemissol = 19
  INTEGER, PARAMETER :: nbemisazi = 13

  REAL,    PARAMETER :: difftransmin=  0.00,difftransmax=  90.00, difftranspas=  5.00
  REAL,    PARAMETER :: diffemissatmin=0.00,diffemissatmax=80.00, diffemissatpas=5.00
  REAL,    PARAMETER :: diffemissolmin=0.00,diffemissolmax=90.00, diffemissolpas=5.00
  REAL,    PARAMETER :: diffemisazimin=0.00,diffemisazimax=180.00,diffemisazipas=15.00

  ! charts size for Leroux vis reflection 
  INTEGER, PARAMETER :: nbreflecsat = 8
  INTEGER, PARAMETER :: nbreflecsol = 6
  INTEGER, PARAMETER :: nbreflecazi = 5

  REAL, PARAMETER :: reflect_satmin=0., reflect_satmax=70., reflect_satpas=10.
  REAL, PARAMETER :: reflect_solmin=40.,reflect_solmax=90., reflect_solpas=10.
  REAL, PARAMETER :: reflect_azimin=0., reflect_azimax=180.,reflect_azipas=45.

  ! Coefficients for Roujean bfrd
  REAL, PARAMETER  :: k0veget=1.4, k1veget=0.15, k2veget=1.0    
  REAL, PARAMETER  :: k0desert=1.3,k1desert=0.05,k2desert=0.5   

! ----------------------------------------------------------------------------------------
! Thresholds for dynamic interpolation in tables
! ----------------------------------------------------------------------------------------
  INTEGER, PARAMETER :: sec_nb = 7
  INTEGER, PARAMETER :: wv_nb = 16
  INTEGER, PARAMETER :: t_nb = 10

  REAL, PARAMETER :: cst_tempcold = 278.
  REAL, PARAMETER :: cst_tempwarm = 288.
  REAL, PARAMETER :: cst_sstcold  = 278.
  REAL, PARAMETER :: cst_sstwarm  = 288.
  REAL, PARAMETER :: cst_solnight = 90.
  REAL, PARAMETER :: cst_solhigh  = 60.
  REAL, PARAMETER :: cst_veg_max_alb = 20.1
  REAL, PARAMETER :: cst_des_min_alb = 30.1
  REAL, PARAMETER :: cst_adiarate = 0.006 

! Limits to solar zenith angle
  REAL :: cosolmax=         0.1736482   ! 80 degree
  REAL :: cosolmax_sea=     0.258819    ! 75 degree
  REAL :: cosolmax_land=    0.1736482   ! 80 degree
  REAL :: cosolmax_snowice= 0.1736482   ! 80 degree
  
! IR thresholds in K
  INTEGER, PARAMETER :: cst_ir   =10    ! for IR threshold ; land /night (K)
  INTEGER, PARAMETER :: cst_irld =7     ! for IR threshold ; land /day (K)
  INTEGER, PARAMETER :: cst_sst  =4     ! for SST threshold ; sea (K)
  INTEGER, PARAMETER :: cst_irs  =3     ! for IR threshold ; sea when no climatologic SST (K)
  
! VIS thresholds offsets 0.4, 0.6, 0.8, 1.3, 1.6mm
  REAL, PARAMETER :: alb_sea_offset(5) =(/0.,5., 5., 0.,5./)    ! sea (%)
  REAL, PARAMETER :: alb_land_offset(5)=(/0.,0., 8., 0.,8./)    ! land (%)
  REAL, PARAMETER :: alb_snow_offset=6.                         ! snow 1.6mm (%)

! threshold 37108 daytime build from nighttime threshold
  REAL, PARAMETER :: coxmunck_slope_37108   = 0.7       ! sea
  REAL, PARAMETER :: coxmunck_offset_37108  = 7.        ! sea
  REAL, PARAMETER :: land_slope_37108       = 0.4       ! land
  REAL, PARAMETER :: land_offset_37108      = 0.        ! land
  REAL, PARAMETER :: land_addoffset_37108   = 2.        ! land
  REAL, PARAMETER :: land_slope_scat_37108  = 36.       ! land
  REAL, PARAMETER :: land_offset_scat_37108 =-0.41      ! land

! threshold 10837 land correction over Europe
  REAL, PARAMETER :: E_seclim_10837(5) =(/1.5, 2.0, 3.0, 3.8, 4.25/) 
  REAL, PARAMETER :: E_coef_10837(5) =(/0.0, -0.6, -1.1, -1.1, 0.5/) 
  REAL, PARAMETER :: A_offset_10837  = 0.8
  REAL, PARAMETER :: A_offset_108120  = 1.2
  REAL, PARAMETER :: A_offset_108   = -3.
 
! Correction to allow high reflectance in forward scat angle
  REAL, PARAMETER :: land_offset_06     = 4.    ! land
  REAL, PARAMETER :: land_slope_scat_06 = 29.   ! land
  REAL, PARAMETER :: land_offset_scat_06        =-0.68  ! land

! snow/ ice detection
  REAL, PARAMETER :: cst_ice108    = 5.                 ! to calculate IR threshold     (K)
  REAL, PARAMETER :: cst_ice37108  = 15.                ! to calculate 37-108
  REAL, PARAMETER :: cst_08shadow  = 20.                ! to determine cloud shadow with 0.8mm
  REAL, PARAMETER :: cst_ice108120   = 2.               ! maximum t4-t5 to determine snow (K)
  REAL, PARAMETER :: ndvi0616_ice_offset      = 0.3     ! values
  REAL, PARAMETER :: ndvi0616_ice_offset_glint= 0.4     ! 
  REAL, PARAMETER :: ndvi0616_ice_slope_ang   = 0.15    ! for minimum ndvi
  REAL, PARAMETER :: ndvi0616_ice_offset_ang  = -1.     ! on ice
  REAL, PARAMETER :: ndvi0616_snow_offset     = 0.3     ! values
  REAL, PARAMETER :: ndvi0616_snow_slope_ang  = 0.15    ! for minimum ndvi
  REAL, PARAMETER :: ndvi0616_snow_offset_ang = -1.     ! on snow
  REAL, PARAMETER :: snow_maxmin_offset       = 20.     ! values
  REAL, PARAMETER :: snow_maxmin_slope_ang    = 45.     ! for minimum ndvi
  REAL, PARAMETER :: snow_maxmin_offset_ang   = -0.55   ! on snow
                        
! vegetation/desert indexes     
  INTEGER, PARAMETER ::cst_desertalbmin =25     ! minimum visible reflectance over desert (%)
  REAL, PARAMETER :: cst_rlivthr =-0.1          ! constant used over desert
  REAL, PARAMETER :: cst_rsivthr =-0.13         ! constant used over land
                
! aqua/modis specific values
  INTEGER, PARAMETER :: cst_glint2_modis =2000  ! for Low cloud test 
  INTEGER, PARAMETER :: cst_mult2_modis  =1.25  ! multiple factor on vis threshold
        
! --------------------------------------------------------------------------------------------
! Texture
! --------------------------------------------------------------------------------------------
!Imagery Band Texture
  REAL, PARAMETER :: cst_I2var          =0.50   !%
  REAL, PARAMETER :: cst_I2max          =1.     !%
  REAL, PARAMETER :: cst_I5var          =0.25   !K
  REAL, PARAMETER :: cst_I5I4var        =0.50   !K
  REAL, PARAMETER :: cst_I4max          =0.5    !K
  REAL, PARAMETER :: cst_I5max          =0.5    !K

! texture from the coregistered Imager channels
  REAL, PARAMETER :: cstI_sd108s        =0.25   ! for local texture, 10.8mm ; sea        (K)
  REAL, PARAMETER :: cstI_sd10837sn     =0.50   ! same for 10.8-3.7 ; sea. night/twilight (K)
  REAL, PARAMETER :: cstI_sd10837sd     =0.50   ! same for 10.8-3.7 ; sea. day  (K)
  REAL, PARAMETER :: cstI_sd08          =0.50   ! same for 0.8mm ; always sea   (%)
  REAL, PARAMETER :: cstI_sd108l        =1.     ! same for 10.8 ; land          (K)
  REAL, PARAMETER :: cstI_sd10837l      =1.     ! same for 10.8-3.7 ; land      (K)
  REAL, PARAMETER :: cstI_valdt(3)      = (/-3., 0., 0.5/)      !(%)
  REAL, PARAMETER :: cstI_10806(3)      = (/ 2., 5., 15./)      !(%)
  REAL, PARAMETER :: cstI_slope10806(2) = (/1., 20./)           !(%)

! texture from the adjacent pixels in field_1b
  REAL, PARAMETER :: cstA_sd108s        =0.25   ! for local texture, 10.8mm ; sea         (K)
  REAL, PARAMETER :: cstA_sd10837sn     =0.50   ! same for 10.8-3.7 ; sea. night/twilight (K)
  REAL, PARAMETER :: cstA_sd10837sd     =0.50   ! same for 10.8-3.7 ; sea. day  (K)
  REAL, PARAMETER :: cstA_sd08          =0.50   ! same for 0.8mm ; always sea   (%)
  REAL, PARAMETER :: cstA_sd108l        =1.     ! same for 10.8 ; land          (K)
  REAL, PARAMETER :: cstA_sd10837l      =1.     ! same for 10.8-3.7 ; land      (K)
  REAL, PARAMETER :: cstA_valdt(3)      = (/-3., 0., 0.5/)      !(%)
  REAL, PARAMETER :: cstA_10806(3)      = (/ 2., 5., 15./)      !(%)
  REAL, PARAMETER :: cstA_slope10806(2)  = (/1., 20./)          !(%)

! --------------------------------------------------------------------------------------------
! features flag
! --------------------------------------------------------------------------------------------
  ! dust/sand detection  
  REAL, PARAMETER ::    Thr_DustLD_solzen               = 70.
  REAL, PARAMETER ::    Thr_DustLD_min108               = 273.10
  REAL, PARAMETER ::    Thr_DustLD_max108               = 315.10  
  REAL, PARAMETER ::    Thr_DustLD_sd06                 = 3.00
  REAL, PARAMETER ::    Thr_DustLD_sd108                = 3.00
  REAL, PARAMETER ::    Thr_DustLD_37108_1              = 13.7
  REAL, PARAMETER ::    Thr_DustLD_120108_1             = 0.62
  REAL, PARAMETER ::    Thr_DustLD_37108_2              =-8.3
  REAL, PARAMETER ::    Thr_DustLD_120108_2             = 2.4
  REAL, PARAMETER ::    Thr_DustLD_120108min            =-1.1
  REAL, PARAMETER ::    Thr_DustLD_0616maxaccurate      = 0.7
  REAL, PARAMETER ::    Thr_DustLD_0616                 = 0.8
  REAL, PARAMETER ::    Thr_DustLD_offsetmax            = 15.0   
  REAL, PARAMETER ::    Thr_DustLD_offsetmin            = -50.0
  REAL, PARAMETER ::    Thr_DustLD_87108_MinAccurate    = -0.5  !-0.6
  REAL, PARAMETER ::    Thr_DustLD_87108_offset06       = 2.9
  REAL, PARAMETER ::    Thr_DustLD_87108_slope06        = -0.18
  REAL, PARAMETER ::    Thr_DustLD_87108                = -0.5
  REAL, PARAMETER ::    Thr_DustLD_ndvi                 = 0.3
  REAL, PARAMETER ::    Thr_DustLD_sdM06                = 0.5
  REAL, PARAMETER ::    Thr_DustLD_108120               = 1.

  REAL, PARAMETER ::    Thr_DustSD_solzen               = 70.
  REAL, PARAMETER ::    Thr_DustSD_sd108max             = 0.4
  REAL, PARAMETER ::    Thr_DustSD_87108_minrough       =-2.6
  REAL, PARAMETER ::    Thr_DustSD_87108_minThick       =-1.1
  REAL, PARAMETER ::    Thr_DustSD_87108_slope          = -0.17
  REAL, PARAMETER ::    Thr_DustSD_sd16offset           = 0.3   
  REAL, PARAMETER ::    Thr_DustSD_sd16slope            = 0.01
  REAL, PARAMETER ::    Thr_DustSD_sd1606               = 1.5
  REAL, PARAMETER ::    Thr_DustSD_sd1606min            = 0.1
  REAL, PARAMETER ::    Thr_DustSD_06Thick              = -99
  REAL, PARAMETER ::    Thr_DustSD_87108MinThick        = -1.1
  REAL, PARAMETER ::    Thr_DustSD_sd06MaxThick         = 1.0
  REAL, PARAMETER ::    Thr_DustSD_06MaxAccurate        = 40.0
  REAL, PARAMETER ::    Thr_DustSD_87108MinAccurate     = -1.6
  REAL, PARAMETER ::    Thr_DustSD_sd06Max              = 0.6
  REAL, PARAMETER ::    Thr_DustSD_sd08Max              = 0.6
  REAL, PARAMETER ::    Thr_DustSD_sd10837Max           = 0.4
  REAL, PARAMETER ::    Thr_DustSD_1606Min              = 0.4
  REAL, PARAMETER ::    Thr_DustSD_sd10837MaxCumulus    = 0.3
  REAL, PARAMETER ::    Thr_DustSD_16MinAccurate        =-99
  REAL, PARAMETER ::    Thr_DustSD_1606MinAccurate      = 0.6
  REAL, PARAMETER ::    Thr_DustSD_16Min                =-99
  REAL, PARAMETER ::    Thr_DustSD_06Min                =-99
  REAL, PARAMETER ::    Thr_DustSD_120108Min            = -1.1
  REAL, PARAMETER ::    Thr_DustSD_108_offset           = -5.00
  REAL, PARAMETER ::    Thr_DustSD_108_slope            = -5.00
  REAL, PARAMETER ::    Thr_DustSD_06_offsetMin         = -5.00
  REAL, PARAMETER ::    Thr_DustSD_06_offsetMax         = 20.00
  REAL, PARAMETER ::    Thr_DustSD_sd108                = 0.4
  REAL, PARAMETER ::    Thr_DustSD_sd06                 = 0.6
  REAL, PARAMETER ::    Thr_DustSD_06_offsetThick       = 10.0
  REAL, PARAMETER ::    Thr_DustSD_87108_offset         = 0.0
  REAL, PARAMETER ::    Thr_DustSD_120108_offset        = 0.0
  REAL, PARAMETER ::    Thr_DustSD_16_offset            = -5.0
  REAL, PARAMETER ::    Thr_DustSD_16_offsetAccurate    = -7.0
  REAL, PARAMETER ::    Thr_DustSD_sdM06                = 0.55
  REAL, PARAMETER ::    Thr_DustSD_108120               = 1.
  REAL, PARAMETER ::    Thr_DustSD_0406                 = 3.
  REAL, PARAMETER ::    Thr_DustSD_04                   = 17.
  REAL, PARAMETER ::    Thr_DustSD_04Slope              = 5.
  REAL, PARAMETER ::    Thr_DustSD_04Offset             = 0.5
 
  REAL, PARAMETER ::    Thr_DustSN_108_offset_accurate = -10.
  REAL, PARAMETER ::    Thr_DustSN_sdi_offset           = 1.28362
  REAL, PARAMETER ::    Thr_DustSN_sdi_slope_3787       = 0.53476
  REAL, PARAMETER ::    Thr_DustSN_sdi_slope_108120     = -0.83871
  REAL, PARAMETER ::    Thr_DustSN_120108Min            = -1.2
  REAL, PARAMETER ::    Thr_DustSN_87108Min             = -2.6
  REAL, PARAMETER ::    Thr_DustSN_87108_slope          = -0.17
  REAL, PARAMETER ::    Thr_DustSN_sd108Max             = 0.4
  REAL, PARAMETER ::    Thr_DustSN_sd10837Max           = 0.4
  REAL, PARAMETER ::    Thr_DustSN_sdiMin               = 0.6
  REAL, PARAMETER ::    Thr_DustSN_108_offset           = -5.0
  REAL, PARAMETER ::    Thr_DustSN_108_slope            = -5.0
  REAL, PARAMETER ::    Thr_DustSN_sdiMinAccurate       = 1.0
  REAL, PARAMETER ::    Thr_DustSN_108MinAccurate       = -10.
  
  !Volcanic ash Detection
  REAL, PARAMETER ::    Thr_ASHLN_120108_offset         = 0.5
  REAL, PARAMETER ::    Thr_ASHLN_120108                = 0.4
  REAL, PARAMETER ::    Thr_ASHLN_108_offsetcold        =-20.0
  REAL, PARAMETER ::    Thr_ASHLN_108_VeryCold          =-50.0
  REAL, PARAMETER ::    Thr_ASHLN_120108_offsetVCold    = 0.5
  REAL, PARAMETER ::    Thr_ASHLN_120108_slope          =0.250

  REAL, PARAMETER ::    Thr_ASHLD_0616                  = 10.0
  REAL, PARAMETER ::    Thr_ASHLD_0616_relaxed          = 20.0
  REAL, PARAMETER ::    Thr_ASHLD_37108_relaxed         = 14.6
  REAL, PARAMETER ::    Thr_ASHLD_120108                = 0.4
  REAL, PARAMETER ::    Thr_ASHLD_37108                 = 6.6
  REAL, PARAMETER ::    Thr_ASHLD_108_offset            = 20.0
  REAL, PARAMETER ::    Thr_ASHLD_06_bright             = 60.0
  REAL, PARAMETER ::    Thr_ASHLD_120108_offsetbright   = 1.0
  REAL, PARAMETER ::    Thr_ASHLD_120108_slope          =0.250
  
  REAL, PARAMETER ::    Thr_ASHSD_0616                  = 10.0
  REAL, PARAMETER ::    Thr_ASHSD_120108                = 0.4
  REAL, PARAMETER ::    Thr_ASHSD_06_bright             = 60.0
  REAL, PARAMETER ::    Thr_ASHSD_120108_offsetbright   = 1.0
  REAL, PARAMETER ::    Thr_ASHSD_120108_slope          =0.250
  
  REAL, PARAMETER ::    Thr_ASHSN_120108_offset         = 0.5
  REAL, PARAMETER ::    Thr_ASHSN_120108                = 0.4
  REAL, PARAMETER ::    Thr_ASHSN_108_offsetcold        =-20.0
  REAL, PARAMETER ::    Thr_ASHSN_108_VCold             =-50.0
  REAL, PARAMETER ::    Thr_ASHSN_120108_offsetVCold    = 0.5
  REAL, PARAMETER ::    Thr_ASHSN_120108_slope          =0.250
 
  !Thin Cirrus Detection
  REAL, PARAMETER ::    Thr_ThinCiLD_13                 =1.25   !(%) land
  REAL, PARAMETER ::    Thr_ThinCiCD_13                 =1.25   !(%) coast
  REAL, PARAMETER ::    Thr_ThinCiDD_13                 =3.00   !(%) desert
  REAL, PARAMETER ::    Thr_ThinCiSID_13                =3.00   !(%) over seaice surface
  REAL, PARAMETER ::    Thr_ThinCiSD_13                 =1.25   !(%) sea
  REAL, PARAMETER ::    Thr_ThinCi_LowCwvCutoff_13      =2.5    !(g/cm2)
  REAL, PARAMETER ::    Thr_ThinCi_HighCwvCutoff_13     =30.0   !(g/cm2)
 
! --------------------------------------------------------------------------------------------
! SnowIce_surface
! --------------------------------------------------------------------------------------------
  REAL, PARAMETER :: BG_snow_108                =280.   !(K)    
  REAL, PARAMETER :: BG_snow_08                 =0.11                           
  REAL, PARAMETER :: BG_ndsi_snow               =0.4                            
  REAL, PARAMETER :: BG_snow_thinCi13           =0.0525                         
  REAL, PARAMETER :: BG_snow_87_108             =0.5    !(K)    
  REAL, PARAMETER :: BG_snow_37_108_LoElev      =9.0    !(K)    
  REAL, PARAMETER :: BG_snow_37_108_HiElev      =10.0   !(K)    

! --------------------------------------------------------------------------------------------
! Coeff SST
! --------------------------------------------------------------------------------------------
  REAL :: a_nl =1.00055, b_nl =0.00852, c_nl =1.29073, d_nl =0.0401,  e_nl =0.77930, f_nl =0.81520, g_nl =1.05141       
  REAL :: a_37 =1.01612, b_37 =0.017093,c_37 =0.85154, d_37 =0.36969, e_37 =0.82285, f_37 =1.13960      
  
! --------------------------------------------------------------------------------------------
! Cloud Mask detection
! --------------------------------------------------------------------------------------------
  ! max_num_tests :
  !
  ! VIIRS
  !    Gr1    Gr5      Gr2     Gr2    Gr2      Gr2  Gr3 Gr3  Gr3   Gr4 Gr4  Gr2    Gr3     Gr6       Gr6  
  !    108  108-120  108-37  87-108  37-108  120-37 06  08  08/06  16  13  87-37  'low' texture1  texture2
  !SN   x      x        x      x       x        x                                          x               = 7
  !ST   x      x        x      x       x        x        x          x                      x               = 9 
  !SD   x      x        x      x                     x   x    x     x   x                  x         x     = 10 (08/06 not counted: done if 08/06<.1)
  !SG   x      x               x                         x                           x     x         x     = 6 ('low'  not counted: done for large zenith angles)
  !LN   x      x        x      x       x                                     x             x               = 6  (87-37 not counted: done over desert)
  !LT   x      x        x      x       x             x                       x             x               = 7  (87-37 not counted: done over desert)
  !LD   x      x        x      x       x             x        x         x                  x         x     = 9  (08/06 not counted: done if 06>10.)
  
  ! AVHRR :lack of 8.7mm and 1.3mm, No 3.7mm during day, No 1.6mm during night for AVHRR
  !    Gr1    Gr5      Gr2     Gr2    Gr2   Gr3 Gr3  Gr3   Gr4  Gr2    Gr6       Gr6  
  !    108  108-120  108-37  37-108  120-37 06  08  08/06  16  'low' texture1  texture2
  !SN   x      x        x      x        x                               x               = 6
  !ST   x      x        x      x        x        x          x           x               = 8 
  !SD   x      x                             x   x    x     x           x         x     = 7  (08/06 not counted: done if 08/06<.1)
  !SG   x      x                                 x               x      x         x     = 5 ('low'  not counted: done for large zenith angles)
  !LN   x      x        x      x                                        x               = 5  
  !LT   x      x        x      x             x                          x               = 6  
  !LD   x      x                             x        x                 x         x     = 6
  !                                                SN ST SD SG LN LT LD
  INTEGER, PARAMETER :: max_num_tests_VIIRS(7) = (/ 7, 9,10, 6, 6, 7, 9/) ! max_num_tests for CMa SN, ST, SD, SG, LN, LT, LD
  INTEGER, PARAMETER :: max_num_tests_AVHRR(7) = (/ 6, 8, 7, 5, 5, 6, 6/) ! max_num_tests for CMa SN, ST, SD, SG, LN, LT, LD
  
  ! ngroup : 
  ! VIIRS                                                 AVHRR
  !    Gr1   Gr2   Gr3   Gr4   Gr5   Gr6           Gr1   Gr2   Gr3   Gr4   Gr5   Gr6
  !SN   1     4     0     0     1     1              1    3     0     0     1    1         
  !ST   1     4     1     1     1     1              1    3     1     1     1    1  
  !SD   1     2     3     2     1     2              1    0     3     1     1    2
  !SG   1     1     2     0     1     2              1    1     1     0     1    2
  !LN   1     4                 1     1              1    2                 1    1
  !LT   1     4     1           1     1              1    2     1           1    1 
  !LD   1     3     2     1     1     2              1    0     2     0     1    2 
  INTEGER, PARAMETER :: ngroups_max =6
  INTEGER , PARAMETER :: max_num_group_VIIRS_SN(ngroups_max)= (/ 1, 4, 0, 0, 1, 1/) ! for CMa SN
  INTEGER , PARAMETER :: max_num_group_VIIRS_ST(ngroups_max)= (/ 1, 4, 1, 1, 1, 1/) ! for CMa ST
  INTEGER , PARAMETER :: max_num_group_VIIRS_SD(ngroups_max)= (/ 1, 2, 3, 2, 1, 2/) ! for CMa SD
  INTEGER , PARAMETER :: max_num_group_VIIRS_SG(ngroups_max)= (/ 1, 1, 2, 0, 1, 2/) ! for CMa SG
  INTEGER , PARAMETER :: max_num_group_VIIRS_LN(ngroups_max)= (/ 1, 4, 0, 0, 1, 1/) ! for CMa LN
  INTEGER , PARAMETER :: max_num_group_VIIRS_LT(ngroups_max)= (/ 1, 4, 1, 0, 1, 1/) ! for CMa LT
  INTEGER , PARAMETER :: max_num_group_VIIRS_LD(ngroups_max)= (/ 1, 3, 2, 1, 1, 2/) ! for CMa LD
  INTEGER , PARAMETER :: max_num_group_AVHRR_SN(ngroups_max)= (/ 1, 3, 0, 0, 1, 1/) ! for CMa SN
  INTEGER , PARAMETER :: max_num_group_AVHRR_ST(ngroups_max)= (/ 1, 3, 1, 1, 1, 1/) ! for CMa ST
  INTEGER , PARAMETER :: max_num_group_AVHRR_SD(ngroups_max)= (/ 1, 0, 3, 1, 1, 2/) ! for CMa SD
  INTEGER , PARAMETER :: max_num_group_AVHRR_SG(ngroups_max)= (/ 1, 1, 1, 0, 1, 1/) ! for CMa SG
  INTEGER , PARAMETER :: max_num_group_AVHRR_LN(ngroups_max)= (/ 1, 2, 0, 0, 1, 1/) ! for CMa LN
  INTEGER , PARAMETER :: max_num_group_AVHRR_LT(ngroups_max)= (/ 1, 2, 1, 0, 1, 1/) ! for CMa LT
  INTEGER , PARAMETER :: max_num_group_AVHRR_LD(ngroups_max)= (/ 1, 0, 2, 0, 1, 2/) ! for CMa LD
  
  REAL, PARAMETER :: highLat            = 60.0  !degrees
  REAL, PARAMETER :: lowLat             =-60.0  !degrees

  REAL, PARAMETER :: CMa_sstfrozen      =271.35 ! ocean could be frozen 
  REAL, PARAMETER :: CMa_37_saturation  =340.0  ! to prevent saturation 
  REAL, PARAMETER :: CMa_37_tempmin     =230.0  ! minimum 37 in night conditions (K)    
  REAL, PARAMETER :: CMa_lstmin         =263.15 ! minimum climatological land surface temperature
  REAL, PARAMETER :: CMa_sstmin         =273.15 ! minimum climatological sea surface temperature
  INTEGER, PARAMETER :: cst_HiElev      = 2000  !minimum high terrain for snow/day and snow/night 37 – 108 (m)  

  REAL, PARAMETER :: CMa_CD_108_37_Ccl  =-10.00 !confident clear   coast/day 108 – 37 (K)       
  REAL, PARAMETER :: CMa_CD_108_37      =-12.00 !clear/cloudy      coast/day 108 – 37 (K)       
  REAL, PARAMETER :: CMa_CD_108_37_Ccld =-14.00 !confident cloudy  coast/day 108 – 37 (K)       
  REAL, PARAMETER :: CMa_CD_108_120     = 3.0   !clear/cloudy      coast/day 108 – 120 thin cirrus (K)  
  REAL, PARAMETER :: CMa_CD_13_Ccl      = 3.0   !confident clear   coast/day M9 thin cirrus (%) 
  REAL, PARAMETER :: CMa_CD_13          = 3.5   !clear/cloudy      coast/day M9 thin cirrus (%) 
  REAL, PARAMETER :: CMa_CD_13_Ccld     = 4.0   !confident cloudy  coast/day M9 thin cirrus (%) 

  REAL, PARAMETER :: CMa_DD_108_37_cwv_switch=2.        !cwv switch for classifying low versus high cwv conditions in the desert/day 108 – 37 (cm)
  REAL, PARAMETER :: CMa_DD_108_37_A1   =5.0    !first degree  coef desert/day 108 – 37  under low cwv according to  A*cwv + B (K/cm)   
  REAL, PARAMETER :: CMa_DD_108_37_B1   =-30.0  !zeroth degree coef desert/day 108 – 37  under low cwv according to  A*cwv + B (K)      
  REAL, PARAMETER :: CMa_DD_108_37_A2   =0.5    !first degree  coef desert/day 108 – 37  under high cwv according to A*cwv + B (K/cm)   
  REAL, PARAMETER :: CMa_DD_108_37_B2   =-21.0  !zeroth degree coef desert/day 108 – 37  under high cwv according to A*cwv + B (K)      
  REAL, PARAMETER :: CMa_DD_108_120     = 3.0   !clear/cloudy     desert/day 108 – 120 emission thin cirrus (K) 
  REAL, PARAMETER :: CMa_DD_04_Ccl      =0.25   !confident clear  desert/day 04 reflectance     
  REAL, PARAMETER :: CMa_DD_04          =0.30   !clear/cloudy     desert/day 04 reflectance     
  REAL, PARAMETER :: CMa_DD_04_Ccld     =0.35   !confident cloudy desert/day 04 reflectance     
  REAL, PARAMETER :: CMa_DD_13_cwv_cutoff=0.25  !minimum cwv      desert/day M9 reflectance ;also used in thin cirrus detection (cm)
  REAL, PARAMETER :: CMa_DD_13_Ccl      =3.0    !confident clear  desert/day M9 reflectance (%)
  REAL, PARAMETER :: CMa_DD_13          =3.5    !clear/cloudy     desert/day M9 reflectance (%) 
  REAL, PARAMETER :: CMa_DD_13_Ccld     =4.0    !confident cloudy desert/day M9 reflectance (%) 

  REAL, PARAMETER :: CMa_LD_37_40_Ccl   =12.0   !9.5    !confident clear  land/day 37 – 40 (K)  
  REAL, PARAMETER :: CMa_LD_37_40       =13.75  !10.00!clear/cloudy     land/day 37 – 40 (K)    
  REAL, PARAMETER :: CMa_LD_37_40_Ccld  =15.5   !10.5!confident cloudy land/day 37 – 40 (K)     
  REAL, PARAMETER :: CMa_LD_108_37_Ccl  =-16.0  !confident clear  land/day 108 – 37 (K) 
  REAL, PARAMETER :: CMa_LD_108_37      =-18.0  !clear/cloudy     land/day 108 – 37 (K) 
  REAL, PARAMETER :: CMa_LD_108_37_Ccld =-20.0  !confident cloudy land/day 108 – 37 (K) 
  REAL, PARAMETER :: CMa_LD_108_120     =3.0    !clear/cloudy     land/day 108 – 120 emission thin cirrus (K)   
  REAL, PARAMETER :: CMa_LD_06_08_Ccl   =1.87   !confident clear  land/day M7/06 reflectance    
  REAL, PARAMETER :: CMa_LD_06_08       =1.82   !clear/cloudy     land/day M7/06 reflectance    
  REAL, PARAMETER :: CMa_LD_06_08_Ccld  =1.78   !confident cloudy land/day M7/06 reflectance    
  REAL, PARAMETER :: CMa_LD_06_GEMI     =10.    !minimum 06 reflectance to perform land/day M7/06 reflectance   
  REAL, PARAMETER :: CMa_LD_13_Ccl      =3.0    !confident clear  land/day M9 thin cirrus reflectance    (%)
  REAL, PARAMETER :: CMa_LD_13          =3.5    !clear/cloudy     land/day M9 thin cirrus reflectance    (%)
  REAL, PARAMETER :: CMa_LD_13_Ccld     =4.0    !confident cloudy land/day M9 thin cirrus reflectance    (%)    

  REAL, PARAMETER :: CMa_LN_37_16_Ccl   =3.50   !confident clear  land/night 37 – 120 (K)       
  REAL, PARAMETER :: CMa_LN_37_16       =4.00   !clear/cloudy     land/night 37 – 120 (K)       
  REAL, PARAMETER :: CMa_LN_37_16_Ccld  =4.50   !confident cloudy land/night 37 – 120 (K)       
  REAL, PARAMETER :: CMa_LN_108_37_Ccl  =2.00   !confident clear  land/night 108 – 37 (K)       
  REAL, PARAMETER :: CMa_LN_108_37      =2.50   !clear/cloudy     land/night 108 – 37 (K)       
  REAL, PARAMETER :: CMa_LN_108_37_Ccld =3.00   !confident cloudy land/night 108 – 37 (K)       
  REAL, PARAMETER :: CMa_LN_108_120     =3.0    !clear/cloudy     land/night 108 – 120 thin cirrus (K)  

  REAL, PARAMETER :: CMa_SID_37_40_Ccl  =5.5    !confident clear  snow/day 37 – 40 (K)  
  REAL, PARAMETER :: CMa_SID_37_40      =6.00   !clear/cloudy     snow/day 37 – 40 (K)  
  REAL, PARAMETER :: CMa_SID_37_40_Ccld =6.5    !confident cloudy snow/day 37 – 40 (K)  
  REAL, PARAMETER :: CMa_SID_37_108_Ccl =4.00   !confident clear  snow/day 108 – 37 when terrain height <= cst_HiElev   (K)     
  REAL, PARAMETER :: CMa_SID_37_108     =7.00   !clear/cloudy     snow/day 108 – 37 when terrain height <= cst_HiElev   (K)     
  REAL, PARAMETER :: CMa_SID_37_108_Ccld        =10.00  !confident cloudy snow/day 108 – 37 when terrain height <= cst_HiElev   (K)
  REAL, PARAMETER :: CMa_SID_37_108_HiElev_Ccl  =6.00   !confident clear  snow/day 108 – 37 when terrain height > cst_HiElev    (K)     
  REAL, PARAMETER :: CMa_SID_37_108_HiElev      =10.00  !clear/cloudy     snow/day 108 – 37 when terrain height > cst_HiElev    (K)     
  REAL, PARAMETER :: CMa_SID_37_108_HiElev_Ccld =14.00  !confident cloudy snow/day 108 – 37 when terrain height > cst_HiElev    (K)     
  REAL, PARAMETER :: CMa_SID_13_Ccl     = 3.0   !confident clear  snow/day M9 thin cirrus reflectance  (%)              
  REAL, PARAMETER :: CMa_SID_13         = 3.5   !clear/cloudy     snow/day M9 thin cirrus reflectance  (%)              
  REAL, PARAMETER :: CMa_SID_13_Ccld    = 4.0   !confident cloudy snow/day M9 thin cirrus reflectance  (%)              
  REAL, PARAMETER :: CMa_SID_108_37_Ccl =-8.00
  REAL, PARAMETER :: CMa_SID_108_37     =-10.00
  REAL, PARAMETER :: CMa_SID_108_37_Ccld        =-12.00
  REAL, PARAMETER :: CMa_SID_108_37_HiElev_Ccl  =-8.00  !confident clear
  REAL, PARAMETER :: CMa_SID_108_37_HiElev      =-10.00 !clear/cloudy
  REAL, PARAMETER :: CMa_SID_108_37_HiElev_Ccld =-12.00 !confident cloudy

  REAL, PARAMETER :: CMa_SIN_37_120_Ccl = 3.500 !confident clear  snow/night 37 – 120 (K)       
  REAL, PARAMETER :: CMa_SIN_37_120     = 4.000 !clear/cloudy     snow/night 37 – 120 (K)       
  REAL, PARAMETER :: CMa_SIN_37_120_Ccld= 4.500 !confident cloudy snow/night 37 – 120 (K)       
  REAL, PARAMETER :: CMa_SIN_108_37_Ccl =-5.00  !confident clear  snow/night 108 – 37 (K)       
  REAL, PARAMETER :: CMa_SIN_108_37     =-6.00  !clear/cloudy     snow/night 108 – 37 (K)       
  REAL, PARAMETER :: CMa_SIN_108_37_Ccld=-7.00  !confident cloudy snow/night 108 – 37 (K)       

  REAL, PARAMETER :: CMa_SD_108_37_Ccl  =-12.00 !confident clear  water/day 108 – 37 (K)        
  REAL, PARAMETER :: CMa_SD_108_37      =-10.00 !clear/cloudy     water/day 108 – 37 (K)        
  REAL, PARAMETER :: CMa_SD_108_37_Ccld =-8.00  !confident cloudy water/day 108 – 37 (K)
  REAL, PARAMETER :: CMa_SD_108_120     = 3.    !clear/cloudy     water/day 108 – 120 emission thin cirrus (K)  
  REAL, PARAMETER :: CMa_SD_06_08_Ccl   = 0.95 !0.94  !confident clear  water/day M7/06 reflectance when no sunglint          
  REAL, PARAMETER :: CMa_SD_06_08       = 1.00 !0.99  !clear/cloudy     water/day M7/06 reflectance when no sunglint          
  REAL, PARAMETER :: CMa_SD_06_08_Ccld  = 1.05  !confident cloudy water/day M7/06 reflectance when no sunglint          
  REAL, PARAMETER :: CMa_SD_08_Ccl      = 3.0   !confident clear  water/day M7 reflectance when no sunglint (%)         
  REAL, PARAMETER :: CMa_SD_08          = 5.5   !clear/cloudy     water/day M7 reflectance when no sunglint (%) 
  REAL, PARAMETER :: CMa_SD_08_Ccld     = 6.5   !confident cloudy water/day M7 reflectance when no sunglint (%) 
  REAL, PARAMETER :: CMa_SD_16_Ccl      = 6.5   !confident clear  snow/day M9 thin cirrus reflectance   (%)
  REAL, PARAMETER :: CMa_SD_16          = 5.5   !clear/cloudy     snow/day M9 thin cirrus reflectance   (%)
  REAL, PARAMETER :: CMa_SD_16_Ccld     = 4.0   !confident cloudy snow/day M9 thin cirrus reflectance   (%)     
  REAL, PARAMETER :: CMa_SD_37_40_Ccl   = 10.0  !5.5!confident clear  snow/day 37 – 40 (K)
  REAL, PARAMETER :: CMa_SD_37_40       = 10.5  !6.00!clear/cloudy     snow/day 37 – 40 (K)     
  REAL, PARAMETER :: CMa_SD_37_40_Ccld  = 11.0  !6.5!confident cloudy snow/day 37 – 40 (K)      

  REAL, PARAMETER :: CMa_SN_108_37_Ccl  = 2.0   !confident clear  water/night 108 – 37 test; adjusted for cwv (K)               
  REAL, PARAMETER :: CMa_SN_108_37      = 2.5   !clear/cloudy     water/night 108 – 37 test; adjusted for cwv (K)               
  REAL, PARAMETER :: CMa_SN_108_37_Ccld = 3.0   !confident cloudy water/night 108 – 37 test; adjusted for cwv( K)               
  REAL, PARAMETER :: CMa_SN_108_120     = 3.0   !clear/cloudy     water/night 108 – 120 emission thin cirrus test (K)           

  REAL, PARAMETER :: CMa_lst_desert     = 20.0  !clear/cloudy     desert land/night  (K)        
  REAL, PARAMETER :: sfc_temp_Min       = 0.0   !minimum surface temp for nighttime 108 – 120 (K)       
  REAL, PARAMETER :: sfc_temp_Max       = 350.0 !maximum surface temp for nighttime 108 – 120 (K)       
  REAL, PARAMETER :: limit_37           = 230.0 !minimum 37 for nighttime 108 – 37 (K)  

! thresholds if sunglint
  REAL, PARAMETER :: CMa_glint_06       = 60.00 ! visible threshold (%)
  REAL, PARAMETER :: CMa_glint_37_108   = 5.00  ! threshold for t3-t4 difference
  REAL, PARAMETER :: CMa_glint06coef    = 0.15  ! coefficient for visible threshold     
  REAL, PARAMETER :: CMa_glint_08_Ccl   = 32    !17!confident clear          water/day 08 reflectance when sunglint (%)         
  REAL, PARAMETER :: CMa_glint_08       = 36    !19!clear/cloudy             water/day 08 reflectance when sunglint (%)         
  REAL, PARAMETER :: CMa_glint_08_Ccld  = 40    !21!confident cloudy         water/day 08 reflectance when sunglint (%)         
  REAL, PARAMETER :: CMa_glintRatio_Ccl = 0.95  !confident clear    water/day M7/06 reflectance when sunglint   
  REAL, PARAMETER :: CMa_glintRatio     = 1.00  !clear/cloudy       water/day M7/06 reflectance when sunglint   
  REAL, PARAMETER :: CMa_glintRatio_Ccld= 1.05  !confident cloudy   water/day M7/06 reflectance when sunglint   

! --------------------------------------------------------------------------------------------
! cloud type
! --------------------------------------------------------------------------------------------
! black body thresholds
  REAL, PARAMETER :: CT_108_120_opaq            = 1.    ! black body threshold (K)
  REAL, PARAMETER :: CT_108_120_opaq_max        = 1.5   ! (K)
  REAL, PARAMETER :: CT_offset_108_120_opaq     = 0.02  ! (K)
  REAL, PARAMETER :: CT_37_108_thin_low_factor  =  2.   ! (K)
  REAL, PARAMETER :: CT_37_108_thin_high_factor =  5.   ! (K)
  REAL, PARAMETER :: CT_37_108_low_delta        =  1.0  ! (K)
  REAL, PARAMETER :: CT_37_108_semi_max         =  0.5  ! (K)

  REAL, PARAMETER :: CT_MINALB_ARID             = 30.0
  REAL, PARAMETER :: CT_offset_MELOW_INVMAX00 = 3.0
  REAL, PARAMETER :: CT_offset_MELOW_INVMAX0  = 5.0
  REAL, PARAMETER :: CT_offset_MELOW_INVMAX1  = 8.0
  REAL, PARAMETER :: CT_offset_MELOW_INVMAX2  = 10.0
  REAL, PARAMETER :: CT_37_108_MEDCLOUDS_MAXALB = 30.0
  REAL, PARAMETER :: CT_37_108_MEDCLOUDS_MIN_0   =   -1.20
  REAL, PARAMETER :: CT_37_108_MEDCLOUDS_MIN_SLOPE0= -1.00
  REAL, PARAMETER :: CT_37_108_MEDCLOUDS_MIN_1   =   -1.70
  REAL, PARAMETER :: CT_37_108_MEDCLOUDS_MIN_SLOPE1 =-1.00
  REAL, PARAMETER :: CT_37_108_MEDCLOUDS_MIN_2   =   -2.20
  REAL, PARAMETER :: CT_37_108_MEDCLOUDS_MIN_SLOPE2= -1.00

  REAL, PARAMETER :: CT_max108_LO_C850          =  1.20
  REAL, PARAMETER :: CT_max108_LO_C700          = -0.20
  REAL, PARAMETER :: CT_max108_LO_OFFSET        = -5.00
  REAL, PARAMETER :: CT_max108_ME_C850          =  0.80
  REAL, PARAMETER :: CT_max108_ME_C700          =  0.20
  REAL, PARAMETER :: CT_max108_ME_OFFSET        = -8.00
  REAL, PARAMETER :: CT_max108_HI_C700          = -0.21
  REAL, PARAMETER :: CT_max108_HI_C500          =  0.50
  REAL, PARAMETER :: CT_max108_HI_OFFSET        =178.10
  REAL, PARAMETER :: CT_max108_VH_C500          =  0.40
  REAL, PARAMETER :: CT_max108_VH_CTROPO        =  0.60
  REAL, PARAMETER :: CT_max108_VH_OFFSET        = -5.00
  REAL, PARAMETER :: CT_OFFTHR_VH_HI            = 25.00

  REAL, PARAMETER :: CT_land_logvar_tuning      =  1.1
  REAL, PARAMETER :: CT_sea_logvar_tuning       =  1.1
  REAL, PARAMETER :: CT_vis_cor_daily           =  1.
  REAL, PARAMETER :: CT_high_alb_ref            = 35.0
  REAL, PARAMETER :: CT_sea_min_low_vis06       = 10.0
  REAL, PARAMETER :: CT_land_min_low_vis06      = 20.0
  REAL, PARAMETER :: CT_min_low_vis06           = 10.0
  REAL, PARAMETER :: CT_sea_alb_default         =  5.0
  REAL, PARAMETER :: CT_land_alb_default        = 15.0
  REAL, PARAMETER :: CT_land_alb_safety_offset  =  7.0
  REAL, PARAMETER :: CT_sea_min_temp_ref        =223.15
  REAL, PARAMETER :: CT_land_min_temp_ref       =223.15

  ! cloud type definition
  INTEGER, PARAMETER :: CT_VLOW =               1
  INTEGER, PARAMETER :: CT_LOW =                2
  INTEGER, PARAMETER :: CT_MED =                3 
  INTEGER, PARAMETER :: CT_HI_OPAQ =            4 
  INTEGER, PARAMETER :: CT_VH_OPAQ =            5
  INTEGER, PARAMETER :: CT_SEMITR_THICK=        6 
  INTEGER, PARAMETER :: CT_SEMITR_MEAN =        7 
  INTEGER, PARAMETER :: CT_SEMITR_THIN =        8 
  INTEGER, PARAMETER :: CT_SEMITR_ABOVE=        9
  INTEGER, PARAMETER :: CT_FRACTIONAL =         10

! --------------------------------------------------------------------------------------------
! Cloud Phase
! --------------------------------------------------------------------------------------------
  REAL, PARAMETER :: CP_BAD                     =9999.0         !Bad data value used in Cloud_Phase
  REAL, PARAMETER :: CP_CERTAIN_ICE_108         =233.16
  REAL, PARAMETER :: CP_MAX_108_WIN_OVER        =270.0
  REAL, PARAMETER :: CP_MAX_108_NIR_OVER        =280.0

  REAL, PARAMETER :: CP_NIR_CI_WATER_37         =0.15
  REAL, PARAMETER :: CP_NIR_CI_LAND_37          =0.15
  REAL, PARAMETER :: CP_NIR_CI_DESERT_37        =0.40

  REAL, PARAMETER :: CP_MIN_CI                  =0.7
  REAL, PARAMETER :: CP_MAX_CI                  =4.0

  REAL, PARAMETER :: CP_MIN_06_OVER             =0.35
  REAL, PARAMETER :: CP_MAX_06_OVER             =1.00

  REAL, PARAMETER :: CP_MIN_04_OVER             =0.50

  REAL, PARAMETER :: CP_MIN_13_WATER_LOW        =0.025
  REAL, PARAMETER :: CP_MIN_13_LAND_LOW         =0.027

  REAL, PARAMETER :: CP_MIN_13_WATER_HIGH       =0.10
  REAL, PARAMETER :: CP_MIN_13_LAND_HIGH        =0.10
  REAL, PARAMETER :: CP_MAX_13_OVER             =0.40
  REAL, PARAMETER :: CP_13_WIN_CHECK_LAND       =0.12
  REAL, PARAMETER :: CP_13_WIN_CHECK_WATER      =0.08

  REAL, PARAMETER :: CP_MAX_108_NIGHT_OVER      =290.0
  REAL, PARAMETER :: CP_108_120_OVERLAP_NIGHT   =0.7
  REAL, PARAMETER :: CP_37_108_OVERLAP_NIGHT    =10.0

  REAL, PARAMETER :: CP_MAX_SZA_OVER            =85.0

  REAL, PARAMETER :: CP_SNOW_10_LOW             =0.10
  REAL, PARAMETER :: CP_SNOW_10_HIGH            =0.30

  REAL, PARAMETER :: CP_108_120_N_OVER_L_TROPWATER=0.58
  REAL, PARAMETER :: CP_108_120_N_OVER_H_TROPWATER=2.50
  REAL, PARAMETER :: CP_108_120_N_OVER_L_MIDWATER=0.58
  REAL, PARAMETER :: CP_108_120_N_OVER_H_MIDWATER=2.00
  REAL, PARAMETER :: CP_108_120_N_OVER_L_LAND   =0.58
  REAL, PARAMETER :: CP_108_120_N_OVER_H_LAND   =2.00

  REAL, PARAMETER :: CP_37_N_OVER_L_TROPWATER   =1.00
  REAL, PARAMETER :: CP_37_N_OVER_H_TROPWATER   =2.60
  REAL, PARAMETER :: CP_37_N_OVER_L_MIDWATER    =1.00
  REAL, PARAMETER :: CP_37_N_OVER_H_MIDWATER    =2.00
  REAL, PARAMETER :: CP_37_N_OVER_L_LAND        =1.00
  REAL, PARAMETER :: CP_37_N_OVER_H_LAND        =2.00

  REAL, PARAMETER :: CP_37_108_N_OVER_L         =3.0
  REAL, PARAMETER :: CP_37_108_N_OVER_H         =15.0
  
! --------------------------------------------------------------------------------------------
! modifs
! --------------------------------------------------------------------------------------------
  REAL, PARAMETER :: cst_ice16_viirs  = 1.
!  REAL, PARAMETER :: cst_sunzenmax_With16_viirs  = 85.
  REAL, PARAMETER :: cst_sunzenmax_With16_viirs  = 90.
  REAL, PARAMETER :: cst_sunzenmax_No16_viirs  = 86.
  REAL, PARAMETER :: cst_108ClearMin_viirs  = 240.
  REAL, PARAMETER :: alb_snow_offset2_viirs     =4.             ! snow 1.6mm (%)
     
  REAL, PARAMETER :: cst_ice16_avhrr  = 0.
!  REAL, PARAMETER :: cst_sunzenmax_With16_avhrr  = 83.
!  REAL, PARAMETER :: cst_sunzenmax_With16_avhrr  = 85.
  REAL, PARAMETER :: cst_sunzenmax_With16_avhrr  = 90.
  REAL, PARAMETER :: cst_sunzenmax_No16_avhrr  = 85.
  REAL, PARAMETER :: cst_108ClearMin_avhrr  = 240.
  REAL, PARAMETER :: alb_snow_offset2_avhrr     =0.             ! snow 1.6mm (%)

  REAL :: cst_ice16
  REAL :: cst_sunzenmax_With16  
  REAL :: cst_sunzenmax_No16  
  REAL :: cst_108ClearMin  
  REAL :: alb_snow_offset2

! --------------------------------------------------------------------------------------------
! thresholds on 3.7 /VIS using sunzen
! --------------------------------------------------------------------------------------------
  REAL :: cst_sunzen_for_use37_avhrr = 92.      ! used in maia_CT_night
  REAL :: cst_sunzen_for_useVIS = 110.          ! used in maia_GetThres_CMa / maia_SST
  REAL :: cst_sunzenmax_day = 83.               ! used in maia_PixEnv_reset / maia_Box_GetTopo / maia_VerifMissing_fields
  REAL :: cst_sunzenmin_night = 90.             ! used in maia_PixEnv_reset / maia_Box_GetTopo / maia_VerifMissing_fields

END MODULE mod_maia_const
