! Name: $Id: mod_maia_types.F90 548 2017-08-30 09:28:19Z frna $
!
! Purpose: 
!          Subroutines 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    01/05/2015   Pascale Roquet bug fixes
!  1.3    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.4    07/05/2017   Pascale Roquet correction of comments
!  1.5    23/08/2017   Pascale Roquet new maia_param type of maia output
module mod_maia_types

 USE MOD_MAIA_CONST
!----------------------------------------------------------------------------
! Purpose:
! defines all types for maia
!
! Authors      : Lydie lavanant
!-----------------------------------------------------------------------
  ! structure for debug
  Type debug
    logical     :: info         ! 
    logical     :: debug        ! 
    integer     :: nbw          ! number of warnings for pixel outside clim
    integer     :: nbw_bg       ! number of warnings for pixel outside clim
  End Type debug

  ! ---------------------------------------------------------------------------------------------------
  ! structure for field info
  Type field_info
    CHARACTER(LEN=6)    :: mission      ! satellite name (noaa18, noaa19, meto02, npp, meto01)
    CHARACTER (LEN=5)  :: ImgName       ! Imager name (avhrr, viirs)
    real        :: LonM_min             ! longitude (degrees)
    real        :: LonM_max             ! longitude (degrees)
    real        :: LatM_min             ! latitude (degrees)
    real        :: LatM_max             ! latitude (degrees)
    integer     :: nbLat_box
    integer     :: nbLon_box
    integer, pointer    :: nb_fov(:,:) 
    integer, pointer    :: pix_box(:,:,:) 
    integer, pointer    :: lig_box(:,:,:)
    logical :: isOver180
    
  End Type field_info

  ! structure for field
  Type field
    CHARACTER(LEN=6)    :: mission              ! satellite name (noaa18, noaa19, meto02, npp, meto01)
    CHARACTER (LEN=5)  :: ImgName               ! Imager name (avhrr, viirs)
    CHARACTER (LEN=80)  :: dataname_gmodo       ! dataname of the geo file
    Character(len=32)   :: N_Granule_ID         ! granule identifier
    Character(len=5)    :: Distributor          ! name of l1b Distributor
    real(kind=8)        :: ij_start             ! Beginning time
    real(kind=8)        :: ij_end               ! Endding time
    integer             :: Orbit_num_start      ! Beginning orbit number
    integer             :: Orbit_num_end        ! Endding orbit number
    integer             :: Nb_Granules          ! number of granules in field
    integer             :: ModScan(48)          ! Scan operational mode (0: processed)
    integer             :: Nb_Lines             ! number of lines in field for Moderate resolution channels
    integer             :: Nb_pixels            ! number of pixels in line for Moderate resolution channels
    integer             :: Nb_chan              ! number of channels in field
    integer             :: NLines_filled        ! effective number of lines in field for Moderate resolution channels
    real, pointer       :: lon(:,:)             ! longitude (degrees)
    real, pointer       :: lat(:,:)             ! latitude (degrees)
    real, pointer       :: satzen(:,:)          ! satellite  zenithal angle(deg)
    real, pointer       :: satazi(:,:)          ! satellite azimutal angle (deg)
    real, pointer       :: sunzen(:,:)          ! solar zenithal angle(deg)
    real, pointer       :: sunazi(:,:)          ! solar azimutal angle(deg)
    real, pointer       :: difazi(:,:)          ! solar-satellite azimutal angles(deg)
    real, pointer       :: Obs(:,:,:)           ! viirs obs (ic, pixel, line)
    integer, pointer    :: Qual_Obs(:,:,:)      ! viirs quality(ic, pixel, line) OK=0
    real(kind=8),pointer :: ij(:)               ! time of the line
    integer,pointer     :: ScanQual(:)          ! viirs fov geoloc quality
    logical,pointer     :: locqual(:,:)         ! viirs fov geoloc quality
    integer, pointer    :: pix_texture(:,:,:)   ! no pixels des voisins 
    integer, pointer    :: lig_texture(:,:,:)   ! no lignes des voisins 
    integer, pointer    :: ch3b(:)              ! specific AVHRR on metop
  End Type field

!-----------------------------------------------------------------------
  ! structure for BOX id
  Type box_id
    real(kind=8) :: ij          ! julian instant of the central pixel
    real        :: lon          ! longitude (degrees)
    real        :: lat          ! latitude (degrees)
    real        :: satzen       ! satellite  zenithal angle(deg)
    real        :: satazi       ! satellite azimutal angle (deg)
    real        :: sunzen       ! solar zenithal angle(deg)
    real        :: sunazi       ! solar azimutal angle(deg)
    integer     :: psize        ! size of the box in Moderate resolution pixels
    integer     :: lsize        ! size of the box in Moderate resolution lines
    integer     :: tm           ! 0=sea, 1=coast, 2=land, 3=desert, 4=long-lasting snow
    logical     :: sea          ! 
    logical     :: land         ! 
    logical     :: coast        ! 
    logical     :: desert       !
    logical     :: snow_always  ! from igbp atlas
    integer     :: alt          ! surface altitude (m)
    integer     :: day_time     ! 0: NIGHT /1: TWILIGHT /2: DAY /3: SUNGLINT
    logical     :: day
    logical     :: night
    logical     :: twilight
    logical     :: glint
    integer     :: dcj          ! specular reflexion 0=no / 1=yes
    real        :: satsec       ! secant of satellite zenith angle
    REAL        :: difazi       ! relative azimuth angle  (deg)
    REAL        :: cos_scat_ang   ! cosinus of the scattering angle 
    real        :: cosol 
    REAL        :: cwv          ! total water vapour content g/cm2
    REAL        :: tsurf        ! surface temperature at surface altitude
    REAL        :: ps_prev        ! pressure   (forecast)  at surface altitude    
    REAL        :: ts_prev        ! Temp surf  (forecast) (K) at 0m altitude
    REAL        :: t2m_prev     ! Temp at 2m (forecast) (K) at 0m altitude
    REAL        :: cwv_prev     ! total water vap content (forecast) (g/cm2)
    REAL        :: seaice_prev  ! sea-ice cover (0-1)
    Real        :: snow_prev    ! snowdepth (m)
    REAL        :: sst_clim     ! SST minimum from atlas (K)
    REAL        :: sstmoy_clim  ! SST moy from atlas  (K)
    REAL        :: albedo_clim  ! Albedo  from atlas (%)
    REAL        :: altnwp       ! altitude of the forecast (m)
    REAL        :: t500         ! forecast temperature at different pressure
    REAL        :: t700         ! forecast temperature at different pressure
    REAL        :: t850         ! forecast temperature at different pressure
    REAL        :: ttropo       ! forecast temperature at tropopause
    REAL        :: t_rttov(n_rttov)
    REAL        :: h_rttov(n_rttov)
    logical     :: inversion_NWP!
    logical     :: Aladin
    logical     :: Europe
    logical     :: Africa
    logical     :: Sahara
    logical     :: Lake
  End Type box_id

  ! ---------------------------------------------------------------------------------------------------
  ! structure for pixel id
  Type pix_info
    CHARACTER (LEN=6):: satimg  ! satellite name
    CHARACTER (LEN=5):: ImgName! Imager name (avhrr, viirs)
    real(kind=8)    ::  ij      ! julian instant of the pixel
    real        :: lon          ! longitude (degrees)
    real        :: lat          ! latitude (degrees)
    integer     :: pixel        ! pixel nummer
    integer     :: line         ! line nummer
    real        :: satzen       ! satellite  zenithal angle(deg)
    real        :: satazi       ! satellite azimutal angle (deg)
    real        :: sunzen       ! solar zenithal angle(deg)
    real        :: sunazi       ! solar azimutal angle(deg)
    REAL        :: difazi       ! relative azimuth angle  (deg)
    real        :: satsec       ! secant of satellite zenith angle
    integer     :: dcj          ! specular reflexion 0=no / 1=yes
    integer     :: alt          ! surface altitude (m)
    integer     :: tm           ! 0=sea, 1=coast, 2=land, 3=desert, 4=long-lasting snow
    logical     :: sea          ! 
    logical     :: land         ! 
    logical     :: coast        ! 
    logical     :: desert       !
    logical     :: snow_always  ! from igbp atlas
    integer     :: day_time     ! 0: NIGHT /1: TWILIGHT /2: DAY /3: SUNGLINT
    logical     :: day
    logical     :: night
    logical     :: twilight
    logical     :: glint

    logical     :: texture_fromImager   ! true: from viirs 2*2 imager in pixel
    logical     :: texture_fromNeighbors! true: from local box of 3*3 around pixel
    real        :: sd_06                ! local std for channel 0.6mm (%)
    real        :: sd_08                ! local std for channel 0.8mm (%)
    real        :: sd_16                ! local std for channel 0.8mm (%)
    real        :: sd_108               ! local std for channel 10.8mm (K)
    real        :: sd_10837             ! local std for channel 10.8-3.7mm (K)
    real        :: diff_Max_06          ! local max values with neighbours (K)
    real        :: diff_Max_108         ! local max values with neighbours (K)
    real        :: diff_Max_10806       ! local max values with neighbours (K)
    real        :: Ratio_Max_10806      !
    real        :: sd_M06               ! local std for channel 0.6mm (%) with Neighbor Medium pixels
    real        :: sd_M08               ! local std for channel 0.8mm (%)
    real        :: sd_M16               ! local std for channel 0.8mm (%)
    real        :: sd_M108              ! local std for channel 10.8mm (K)
    real        :: sd_M10837            ! local std for channel 10.8-3.7mm (K)
    real        :: diff_Max_M06         ! local max values with neighbours (K)
    real        :: diff_Max_M108        ! local max values with neighbours (K)
    real        :: diff_Max_M10806      ! local max values with neighbours (K)
    real        :: Ratio_Max_M10806     !

    logical     :: Tropic
    logical     :: Midlat
    logical     :: Polar
    logical     :: lake
    real        :: Toc_Ndvi
    integer     :: SnowIce_surf          ! 0=no snow/ice from background, 1=surface with snow/ice, -99:not done
  End Type pix_info

  ! structure for data pixel
  Type pix_data
    real        :: Obs_04       ! obs at 0.4mm
    real        :: Obs_05       ! obs at 0.55mm
    real        :: Obs_06       ! obs at 0.6mm
    real        :: Obs_08       ! obs at 0.8mm
    real        :: Obs_13       ! obs at 1.3mm
    real        :: Obs_16       ! obs at 1.6mm
    real        :: Obs_22       ! obs at 2.2mm
    real        :: Obs_37       ! obs at 3.7mm
    real        :: Obs_40       ! obs at 4.0mm
    real        :: Obs_87       ! obs at 8.7mm
    real        :: Obs_108      ! obs at 10.8mm
    real        :: Obs_120      ! obs at 12.0mm
    real        :: Qual_tot 
    logical     :: Qual_04
    logical     :: Qual_05
    logical     :: Qual_06
    logical     :: Qual_08
    logical     :: Qual_13
    logical     :: Qual_16
    logical     :: Qual_22
    logical     :: Qual_37
    logical     :: Qual_40
    logical     :: Qual_87
    logical     :: Qual_108
    logical     :: Qual_120 
    logical     :: usable ! if true we can use this pixel
  
    real        :: I(4,5)       ! imager resolution obs of viirs
    logical     :: Qual_I(5)    ! at imager resolution
  End Type pix_data

  ! ---------------------------------------------------------------------------------------------------
  ! structure for dynamic thresholds
  Type maia_thres
    real        :: v_CMa_sst            
    real        :: v_CMa_108_sea                
    real        :: v_CMa_108_120_sea            
    real        :: v_CMa_37_40_sea
    real        :: v_CMa_108_37_sea             
    real        :: v_CMa_120_37_sea             
    real        :: v_CMa_87_108_sea
    real        :: v_CMa_37_108_sea              
    real        :: v_CMa_37_120_sea              
    real        :: v_CMa_06_sea         
    real        :: v_CMa_08_sea         
    real        :: v_CMa_06_08_sea              
    real        :: v_CMa_16_sea
    real        :: v_CMa_108_land               
    real        :: v_CMa_108_120_land           
    real        :: v_CMa_108_37_land            
    real        :: v_CMa_87_108_land            
    real        :: v_CMa_87_37_land             
    real        :: v_CMa_37_40_land
    real        :: v_CMa_37_108_land            
    real        :: v_CMa_37_120_land            
    real        :: v_CMa_04_land                
    real        :: v_CMa_06_land                
    real        :: v_CMa_06_08_land             
    real        :: v_CMa_13_land                
    real        :: v_CMa_108_120_desert 
    real        :: v_CMa_108_37_desert  
    real        :: v_CMa_04_desert              
    real        :: v_CMa_13_desert              
    real        :: v_CMa_37_40_ice
    real        :: v_CMa_108_37_ice             
    real        :: v_CMa_13_ice         
    real        :: v_CMa_16_ice         
    real        :: v_CMa_37_120_ice             
    real        :: v_CMa_108_ice                
    real        :: v_CT_max108_lo 
    real        :: v_CT_max108_me
    real        :: v_CT_max108_hi 
    real        :: v_CT_max108_vh 
    real      :: v_CT_108_120_opaq
    real      :: v_CT_87_108_opaq 
    real      :: v_CT_37_108_opaq 
    real        :: v_CT_maxcirrus_06slope 
    real        :: v_CT_maxcirrus_06intercept 
    real        :: v_CT_maxcirrus_06 
    real        :: v_CT_mincirrus_06
    real      :: v_CT_minlow_06 
  End Type maia_thres

  ! structure for threshold tables(nb_wv,nb_secant)
  Type maia_ThresTables_sea
    REAL    :: tab108_120_cold (16,7)   ! t108120_ocean_DT+3: 
    REAL    :: tab108_120_warm (16,7)   ! t108120_ocean_DT+0 
    REAL    :: tabtsurf_cold(16,7)      ! tsurft108_ocean_DT+0 
    REAL    :: tabtsurf_warm(16,7)      ! tsurft108_ocean_DT+0 
    REAL    :: tab108_37_cold (16,7)    ! t108038_ocean_DT+3 
    REAL    :: tab108_37_warm (16,7)    ! t108038_ocean_DT+3 
    REAL    :: tab120_37_cold (16,7)    ! t120038_ocean_DT+3 
    REAL    :: tab120_37_warm (16,7)    ! t120038_ocean_DT+3 
    REAL    :: tab37_108_cold (16,7)    ! t038108_ocean_DT+0 
    REAL    :: tab37_108_warm (16,7)    ! t038108_ocean_DT-3 
    REAL    :: tab87_108_cold (16,7)    ! t087108_ocean_DT-3 
    REAL    :: tab87_108_warm (16,7)    ! t087108_ocean_DT-3 
    REAL    :: tab37_40_cold (16,7)     ! t03740_ocean_DT+0 
    REAL    :: tab37_40_warm (16,7)     ! t03740_ocean_DT+0 
  End Type maia_ThresTables_sea

  ! structure for threshold tables(nb_wv,nb_secant)
  Type maia_ThresTables_land
    REAL    :: tab108_120_ln    (16,7)          ! t108t120_veget_+3
    REAL    :: tab108_120_ld     (16,7)         ! t108t120_veget_-10
    REAL    :: tabtsurf_ln_veg  (16,7)          ! tsurft108_veget_+5 
    REAL    :: tabtsurf_ld_veg  (16,7)          ! tsurft108_veget_-5
    REAL    :: tabtsurf_ln_des  (16,7)          ! tsurft108_veget_+5: tabts4_*_des: add offset of 4k in routine to ve
    REAL    :: tabtsurf_ld_des  (16,7)          ! tsurft108_veget_-5: tabts4_*_des: add offset of 4k in routine to veg
    REAL    :: tab108_37_l_cold_veg (16,7)      ! t108t38_veget_+5
    REAL    :: tab108_37_l_warm_veg (16,7)      ! t108t38_veget_+5
    REAL    :: tab108_37_l_cold_des (16,7)      ! t108t38_desert_+5
    REAL    :: tab108_37_l_warm_des (16,7)      ! t108t38_desert_+5
    REAL    :: tab37_108_l_cold_veg (16,7)      ! t38t108_veget_+3
    REAL    :: tab37_108_l_warm_veg (16,7)      ! t38t108_veget_+0 
    REAL    :: tab37_108_l_cold_des (16,7)      ! t38t108_veget_+3 
    REAL    :: tab37_108_l_warm_des (16,7)      ! t38t108_veget_+0
    REAL    :: tab87_108_ln_veg (16,7)          ! t087t108_veget_+5_night
    REAL    :: tab87_108_ld_veg (16,7)          ! t087t108_veget_-5_day
    REAL    :: tab87_108_ln_des (16,7)          ! t087t108_desert_+5_night
    REAL    :: tab87_108_ld_des (16,7)          ! t087t108_desert_-5_day
    REAL    :: tab87_37_l_cold_veg (16,7)       ! t087t038_veget_+5
    REAL    :: tab87_37_l_warm_veg (16,7)       ! t087t038_veget_+5
    REAL    :: tab87_37_l_cold_des (16,7)       ! t087t038_veget_+5
    REAL    :: tab87_37_l_warm_des (16,7)       ! t087t038_veget_+5
    REAL    :: tab37_40_l_cold_veg (16,7)       ! t038t040_veget_+0
    REAL    :: tab37_40_l_warm_veg (16,7)       ! t038t040_veget_+0
    REAL    :: tab37_40_l_cold_des (16,7)       ! t038t040_veget_+0
    REAL    :: tab37_40_l_warm_des (16,7)       ! t038t040_veget_+0
  End Type maia_ThresTables_land

  ! structure for threshold tables(nb_wv,nb_secant)
  Type maia_ThresTables_opaq
    REAL        :: tab108_120_opaq(16,7)        ! 
    REAL        :: tab87_108_opaq(16,7) ! 
    REAL        :: tab37_108_opaq(16,7) ! 
    REAL        :: t108_tcld(10,7)
  End Type maia_ThresTables_opaq

  ! structure for threshold tables(nb_wv,nb_secant)
  Type maia_Thres_Phase
    REAL        :: A_overlap(8,7)       !nb_sunzen, nb_satzen) 
    REAL        :: B_overlap(8,7)       ! 
    REAL        :: C_overlap(8,7)       ! 
    REAL        :: D_overlap(8,7)       ! 
    REAL        :: E_overlap(8,7)       ! 
    REAL        :: A_wis(18)            ! nb_scat
    REAL        :: B_wis(18)            ! 
    REAL        :: C_wis(18)            ! 
    REAL        :: D_wis(18)            ! 
    REAL        :: E_wis(18)            ! 
    REAL        :: A_veg(18)            ! nb_scat
    REAL        :: B_veg(18)            ! 
    REAL        :: C_veg(18)            ! 
    REAL        :: D_veg(18)            ! 
    REAL        :: E_veg(18)            ! 
  End Type maia_Thres_Phase

  ! structure for VIS threshold tables
  Type maia_VISThresTables
    REAL        :: TrO3(nb_canvis)
    REAL        :: TrH2O(nbh2o,nb_canvis)
    REAL        :: TrOxy(nboxy,nb_canvis)
    REAL        :: Spherediff(nb_canvis)
    REAL        :: transdiff(nbtrans,nb_canvis)
    REAL        :: emisdiff(nbemissol,nbemissat,nbemisazi,nb_canvis)
  End Type maia_VISThresTables

  ! ---------------------------------------------------------------------------------------------------
  ! structure for VIIRS output cloud mask
  Type maia_CMa
    INTEGER     :: clear                ! clear flag (0= clear, 1= cloudy, 3= snow, 4= ice)
    integer     :: conf_clear           ! final clear confidence level: 
                                        !       3=confident clear / 2=probably clear /
                                        !       1=probably cloudy / 0=confident cloudy
    real        :: conf_value           ! final clear confidence value from 0 (surelycloudy) to 1 (surelyclear) 
    INTEGER     :: qual                 ! test serie quality: 0=high / 1=medium / 2=poor / 3=bad
    logical     :: texture              ! 
    integer     :: snowice_fl           ! no=0 / yes=1
    integer     :: thin_cirrus          ! no=0 / yes=1
    integer     :: shadow_fl            ! no=0 / yes=1
    integer     :: dust_fl              ! no=0 / yes=1
    integer     :: ash_fl               ! no=0 / yes=1
    character(len=8):: test_name        ! 'CMa_SD','CMa_SN','CMa_SG','CMa_ST','CMa_IceD'
    logical     :: Cld_108              ! Gr 1          true if cloudy
    logical     :: Cld_37_40            ! Gr 2.1
    logical     :: Cld_108_37           ! Gr 2.2
    logical     :: Cld_87_108           ! Gr 2.3
    logical     :: Cld_87_37            ! Gr 2.3
    logical     :: Cld_37_108           ! Gr 2.4
    logical     :: Cld_37_120           ! Gr 2.4
    logical     :: Cld_120_37           ! Gr 2.5
    logical     :: Cld_06               ! Gr 3.1
    logical     :: Cld_08               ! Gr 3.1
    logical     :: Cld_06_08            ! Gr 3.2
    logical     :: Cld_LowSunGlint      ! Gr 3.3
    logical     :: Cld_13               ! Gr 4
    logical     :: Cld_16               ! Gr 4
    logical     :: Cld_108_120          ! Gr 5
    real        :: confcl_108           ! Gr 1   individual clear confidence level from 0. (clear) to 1.(cloudy)
    real        :: confcl_37_40         ! Gr 2.1
    real        :: confcl_108_37        ! Gr 2.2
    real        :: confcl_87_108        ! Gr 2.3
    real        :: confcl_87_37         ! Gr 2.3
    real        :: confcl_37_108        ! Gr 2.4
    real        :: confcl_37_120        ! Gr 2.4
    real        :: confcl_120_37        ! Gr 2.5
    real        :: confcl_06            ! Gr 3.1
    real        :: confcl_08            ! Gr 3.1
    real        :: confcl_06_08         ! Gr 3.2
    real        :: confcl_LowSunGlint   ! Gr 3.3
    real        :: confcl_13            ! Gr 4
    real        :: confcl_16            ! Gr 4
    real        :: confcl_108_120       ! Gr 5
    logical     :: Cld_texture1         ! Gr 6
    logical     :: Cld_texture2         ! Gr 6
    real        :: confcl_texture1      ! Gr 6
    real        :: confcl_texture2      ! Gr 6
    
    integer     :: Adjacent_conf_clear  ! clear confidence level from Neighbors: 
                                        !       3=confident clear / 2=probably clear /
                                        !       1=probably cloudy / 0=confident cloudy
  End Type maia_CMa

  ! structure for VIIRS output cloud type
  Type maia_CT
    INTEGER     :: black        ! black-body flag (1= black-body)
    INTEGER     :: thin_cirrus  ! thin cirrus flag 1= yes
    INTEGER     :: cld_type     ! cloud type    1: CT_VLOW
                                !               2: CT_LOW               
                                !               3: CT_MED                
                                !               4: CT_HI_OPAQ            
                                !               5: CT_VH_OPAQ   
                                !               6: CT_SEMITR_THICK       
                                !               7: CT_SEMITR_MEAN        
                                !               8: CT_SEMITR_THIN        
                                !               9: CT_SEMITR_ABOVE      
                                !               10:CT_FRACTIONAL        
    INTEGER     :: Phase        ! cloud phase   0:  Not Executable
                                !               1:  Clear (from Confidently Clear pixels)
                                !               2:  Partly Cloudy (from Probably Clear and Cloudy pixels)
                                !               3:  Water Cloud
                                !               4:  Supercooled Water or Mixed Phase Cloud
                                !               5:  Opaque Ice Cloud
                                !               6:  Cirrus (Non-Opaque) Cloud
                                !               7:  Cloud Overlap               
    INTEGER     :: class        !classification 0 non-processed         containing no data or corrupted data
                                !               1 cloud free land       no contamination by snow/ice surface, 
                                !               2 cloud free sea        no contamination by snow/ice surface, 
                                !               3 land contaminated by snow
                                !               4 sea contaminated by snow/ice
                                !               5 very low and cumuliform clouds
                                !               6 very low and stratiform clouds
                                !               7 low and cumuliform clouds
                                !               8 low and stratiform clouds
                                !               9 medium and cumuliform clouds
                                !               10 medium and stratiform clouds
                                !               11 high opaque and cumuliform clouds
                                !               12 high opaque and stratiform clouds
                                !               13 very high opaque and cumuliform clouds
                                !               14 very high opaque and stratiform clouds
                                !               15 high semitransparent thin clouds
                                !               16 high semitransparent meanly thick clouds
                                !               17 high semitransparent thick clouds
                                !               18 high semitransparent above low or medium clouds
                                !               19 fractional clouds (sub-pixel water clouds)
                                !               20 undefined (undefined by CMa)
    INTEGER     :: Heavy_Aerosol
    INTEGER     :: Smoke_Candidate
    INTEGER     :: Dust_Candidate
    INTEGER     :: Dust_VolcanicAsh
    INTEGER     :: Activefire_fl
    INTEGER     :: MNeighbors
  End Type maia_CT

  ! structure for VIIRS output cloud height
  Type maia_CH
    real        :: Tcld !
    real        :: Pcld !
  End Type maia_CH

  ! ---------------------------------------------------------------------------------------------------
  ! structure for topography
  Type topo_field
    integer     :: nbl 
    integer     :: nbp 
    REAL        :: lat1         ! 
    REAL        :: lon1 ! 
    REAL        :: lat_inc      ! 
    REAL        :: lon_inc      ! 
    REAL        :: latend       ! 
    REAL        :: lonend       ! 
    logical     :: norm180      ! true if in [-180,180[ else [0,360[
    real, pointer :: data (:,:)
  End Type topo_field

  Type clim_field
    character(len=6):: name     ! 'albmax', 'sstmin', 'sstmoy', 'wvcont'
    integer     :: nbl          ! number of available lines of albedo array
    integer     :: nbp          ! number of available pixels in albedo array
    real        :: lat1         ! latitude of the first pixel of albedo array (deg)
    real        :: lon1         ! longitude of the first pixel of albedo array (deg)
    real        :: lat_inc      ! latitude increment per line albedo array (deg)
    real        :: lon_inc      ! longitude increment per pixel of albedo array (deg)
    logical     :: norm180      ! true if in [-180,180[ else [0,360[
    real, pointer :: data (:,:)
  End Type clim_field

  ! structure for NWP field
  Type nwp_field
    logical     :: exist        ! 
    character(len=4):: mltype   ! 'Pres' or 'ml'
    character(len=6):: origine          ! 'MF', 'EC'
    character(len=6):: alt_origine      ! 'MF', 'EC'
    character(len=6):: tm_origine       ! 'MF', 'EC'
    REAL(KIND=8)    :: ij1      ! Julian instant of NWP time cycle
    integer         :: ech      ! deadline (hours)
    REAL(KIND=8)    :: ij       ! Julian instant corresponding to the field data (ij1+ech)
    integer     :: nbniv        ! number of levels
    integer     :: nbl          ! number of lines in field
    integer     :: nbp          ! number of pixels in field
    Real        :: lat1         ! latitude of the first point of the field (deg)
    Real        :: lon1         ! longitude of the first point of the field (deg)
    Real        :: lat_inc      ! latitude increment per line (deg)
    Real        :: lon_inc      ! longitude increment per pixel (deg)
    Real        :: latend       ! latitude of the last point of the field (deg)
    Real        :: lonend       ! longitude of the last point of the field (deg)
    logical     :: norm180      ! true if in [-180,180[ else [0,360[
    logical     :: circumferential      ! true if cylindrical around the earth
    logical     :: ch_alt       ! true if field found
    logical     :: ch_tm        ! true if field found
    Real, pointer       :: pres(:)      ! pressure levels
    Real, pointer       :: lat(:,:)     ! latitude
    Real, pointer       :: lon(:,:)     ! longitude
    Real, pointer       :: temp(:,:,:)  ! temperature (K)
    Real, pointer       :: humid(:,:,:) ! water vapour (ppmv)
    Real, pointer       :: ts(:,:)      ! surface temperature(K)
    Real, pointer       :: t2m(:,:)     ! temperature 2m (K)
    Real, pointer       :: cwv(:,:)     ! total water vapor content (g/cm2)
    Real, pointer       :: psurf(:,:)   ! psurf (hpa)
    Real, pointer       :: alt(:,:)     ! surface roughness (m)
    Real, pointer       :: seaice(:,:)  ! sea-ice cover (0-1)
    Real, pointer       :: snow(:,:)    ! snowdepth (m)
    integer, pointer    :: tm(:,:)      ! 
    integer :: nmandatory = 3
    real  :: mandatory_press(3) =(/850.,700.,500./) ! mandatory pression levels
    logical :: mandatory_ok (3)
  End Type nwp_field

  ! structure for atmospheric profiles on model rttov pressure levels
  Type profile_nwp
    logical     :: exist        ! 
    character(len=6):: origine  ! 'AC'= Analyse Cep,    'FC'=Forecast Cep 
                                ! 'AA'= Analyse Arpege, 'FA'=Forecast Arpege
    integer*8   :: time_res     ! Julian instant.  of NWP time cycle
    integer     :: ech          ! deadline (hours)
    integer     :: nb_nodes     ! nb nodes for interpolation (4 : bonne qualite)
    Real        :: t(51)        ! temperature (K)
    Real        :: q(51)        ! water vapour (ppmv)
    Real        :: o3(51)       ! ozone (ppmv)
    Real        :: tskin        ! surface temperature(K)
    Real        :: t2m          ! temperature 2m (K)
    Real        :: q2m          ! water vapour 2m (ppmv)
    Real        :: cwv          ! total water vapor content (100*g/cm2)
    Real        :: psurf        ! psurf (hpa)
    Real        :: altsurf      ! surface height (m)
    Real        :: tm           ! rttov: 1=sea, 0=land, 2=ice/snow
    Real        :: nwater       ! Water type for rttov: 0=fresh water, 1=ocean water
    Real        :: zenangle     ! angles
    Real        :: azangle      ! angles
    Real        :: ctp        ! cloud top pressure  (hPa)
    Real        :: cfraction    ! cloud fraction (0 - 1) 1 for 100% cloud cover
  End Type profile_nwp
 
! old maia_par definition :
!       1  - Cloud Mask                 summary of CMa
!                                               0= clear / 1= cloudy / 3= clear over snow / 
!                                               4= clear over ice / 5= aerosol_dust_ash_fire
!       2  - Cloud Mask Quality         from the number of tests involved
!                                               3=high / 2=medium / 1=poor / 0=bad
!       3  - Cloud Mask Confidence      from the proximity of thresholds (before Cloud Adjacency)
!                                               3=confident clear / 2=probably clear /
!                                               1=probably cloudy / 0=confident cloudy                                  
!       4  - Cloud Mask Adjacency       from cloud confidence evaluation of surrounding pixels
!                                               0=confident clear / 1=probably clear /
!                                               2=probably cloudy / 3=confident cloudy                                  
!       5  - Surface temperature        for confident clear
!                                       over sea: SST Split-Window 
!                                       over land: LST from RTE using forecast and emissivity atlas

!       6  - Box size                   number of Moderate pixels/lines
!       7  - Box bg Tskin source        0= from climatology / 1= from forecast 
!       8  - Box Bg Tskin used          (K)
!       9  - Box Bg WV Content source   0= from climatology / 1= from forecast
!       10 - Box Bg WV Content used     (g/cm2)
!       11 - Box surface altitude       (m) 
!       12 - Box atlas surface type     0= sea+inland-water / 1= mixed / 2= land / 3= desert / 4= snow_always
!       13 - Box day_time               0= Night / 1= Twilight / 2= Day / 3= Sunglint
!       14 - Box specular reflexion     0= no / 1= yes

!       15 - pixel surface altitude     (m) 
!       16 - pixel surface type         0= sea+inland-water / 1= mixed / 2= land / 3= desert / 4= snow_always
!       17 - pixel snow/ice             0= no / 1= yes 
!                                       night: from ancillary data or forecast
!                                       day: from test serie

!       18 - cloud type                 0  non-processed (no data or corrupted data)
!                                       1  cloud free land
!                                       2  cloud free sea        
!                                       3  land contaminated by snow
!                                       4  sea contaminated by snow/ice
!                                       5  very low clouds
!                                       7  low clouds
!                                       9  medium clouds
!                                       11 high opaque clouds
!                                       13 very high opaque clouds
!                                       15 thin semitransparent clouds
!                                       16 meanly thick semitransparent  clouds
!                                       17 thick semitransparent clouds
!                                       18 semitransparent above low or medium clouds
!                                       19 fractional clouds (sub-pixel water clouds)
!                                       20 undefined (undefined by CMa)

!       19 - cloud phase                0 Not Executable
!                                       1 Clear (from Confidently Clear pixels)
!                                       2 Partly Cloudy (from Probably Clear and Cloudy pixels)
!                                       3 Water Cloud
!                                       4 Supercooled Water or Mixed Phase Cloud
!                                       5 Opaque Ice Cloud
!                                       6 Cirrus (Non-Opaque) Cloud
!                                       7 Cloud Overlap                 
!       20 - thin cirrus detected       0= no /1= yes
!       21 - cloud shadow detected      0= no /1= yes
!       22 - cloud opacity              0= no /1= yes
!       23 - cloud top temperature      (K) for confident cloudy and opaque
!       24 - cloud top pressure         (hPa) for confident cloudy and opaque using NWP profile

!       25 - Heavy aerosol              0= no / 1= yes
!       26 - Dust                       0= no / 1= yes
!       27 - Volcanic Ash               0= no / 1= yes
!       28 - Smoke                      0= no / 1= yes
!       29 - Fire                       0= no / 1= yes
!       30 - Moderate pixel texture     0= no / 1= yes
!       31 - Temp at 2m (forecast) (K) at 0m altitude  
!
! new definition for output parametres with correspondances with old definition
! only effectively written parametres in hdf files are kept
 Type maia_param 
    real :: alt_surface !11 (m)
    real :: surf_temp !5
    real :: Box_Bg_Tskin_used !8
    real :: Box_Bg_WV_Content_used    !10 (g/cm2)                      
    real :: cloud_top_temp !23
    real :: cloud_top_press ! 24( hPa) for confident cloudy and opaque using NWP profile
    real :: temp_2m !31

    integer (kind=2) ::Cloud_Mask !1
    integer (kind=2)  :: Cloud_Mask_Qual !2
    integer (kind=2) :: Cloud_Mask_Conf !3
    integer (kind=2) :: Cloud_Mask_Ad !4
    integer (kind=2) :: box_bg_tskin_src  !7
    integer (kind=2) :: box_bg_wv_src  !9
    integer (kind=2) :: Box_atlas_surface_type !12
    integer (kind=2) :: box_daytime  !13
    integer (kind=2) :: specular_reflexion !14
    integer (kind=2) :: pixel_surface_type ! 16
    integer (kind=2) :: cloud_type  !18
    integer (kind=2) :: cloud_phase  !19 
    integer (kind=2) :: cloud_opacity !22 0= no /1= yes

 End type maia_param

public:: maia_param_array_init
public::  maia_param_scalar_to_array_item
contains
subroutine maia_param_array_init (maia_param_out,nbpix,nblin, AllocStatus)
 type(maia_param), allocatable, intent(out) :: maia_param_out(:,:)
 integer, intent(in) :: nbpix
 integer, intent(in) :: nblin 
 integer, intent(out), optional :: AllocStatus
 integer :: p,l

allocate ( maia_param_out( nbpix,nblin) , STAT = AllocStatus)   
 do p = 1 ,nbpix
    do l = 1 , nblin
 maia_param_out(p,l) %alt_surface = vmanq
 maia_param_out(p,l) %surf_temp = vmanq
 maia_param_out(p,l) %Box_Bg_Tskin_used = vmanq
 maia_param_out(p,l) %Box_Bg_WV_Content_used = vmanq
 maia_param_out(p,l) %cloud_top_temp = vmanq
 maia_param_out(p,l) %cloud_top_press = vmanq
 maia_param_out(p,l) %temp_2m = vmanq

 maia_param_out(p,l) %Cloud_Mask = imanq
 maia_param_out(p,l) %Cloud_Mask_Qual = imanq
 maia_param_out(p,l) %Cloud_Mask_Conf = imanq 
 maia_param_out(p,l) %Cloud_Mask_Ad = imanq 
 maia_param_out(p,l) %box_bg_tskin_src = imanq
 maia_param_out(p,l) %box_bg_wv_src = imanq
 maia_param_out(p,l) %Box_atlas_surface_type= imanq
 maia_param_out(p,l) %box_daytime =  imanq
 maia_param_out(p,l) %specular_reflexion =  imanq 
 maia_param_out(p,l) %pixel_surface_type = imanq
 maia_param_out(p,l) %cloud_type =  imanq
 maia_param_out(p,l) %cloud_opacity = imanq
 maia_param_out(p,l) %cloud_phase =  imanq

   enddo
  enddo

end subroutine maia_param_array_init

subroutine maia_param_copy_item(maia_param_in, maia_param_out)
type(maia_param), intent(in) :: maia_param_in
type(maia_param), intent(out) :: maia_param_out
 maia_param_out%alt_surface = maia_param_in%alt_surface
 maia_param_out%surf_temp =  maia_param_in%surf_temp
 maia_param_out%Box_Bg_Tskin_used =  maia_param_in%Box_Bg_Tskin_used
 maia_param_out%Box_Bg_WV_Content_used = maia_param_in%Box_Bg_WV_Content_used
 maia_param_out%cloud_top_temp =  maia_param_in%cloud_top_temp
 maia_param_out%cloud_top_press = maia_param_in%cloud_top_press 
 maia_param_out%temp_2m = maia_param_in%temp_2m

 maia_param_out%Cloud_Mask = maia_param_in%Cloud_Mask
 maia_param_out%Cloud_Mask_Qual =  maia_param_in%Cloud_Mask_Qual
 maia_param_out%Cloud_Mask_Conf =  maia_param_in%Cloud_Mask_Conf
 maia_param_out%Cloud_Mask_Ad =  maia_param_in%Cloud_Mask_Ad
 maia_param_out%box_bg_tskin_src =  maia_param_in%box_bg_tskin_src
 maia_param_out%box_bg_wv_src =  maia_param_in%box_bg_wv_src 
 maia_param_out%Box_atlas_surface_type =  maia_param_in%Box_atlas_surface_type 
 maia_param_out%pixel_surface_type =  maia_param_in%pixel_surface_type
 maia_param_out%box_daytime =   maia_param_in%box_daytime
 maia_param_out%specular_reflexion =   maia_param_in%specular_reflexion
 maia_param_out%cloud_type =  maia_param_in%cloud_type 
 maia_param_out%cloud_opacity =  maia_param_in%cloud_opacity
 maia_param_out%cloud_phase =  maia_param_in%cloud_phase

end subroutine maia_param_copy_item


End Module mod_maia_types
