! Name: $Id: avhrr_maia4_header_io.F90 553 2017-09-29 12:34:44Z frna $ 
!
! Purpose:  avhrr maia4 header Fortran I/O module
!
! Method:
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!   1.0   4/4/2017  Initial version (Pascale Roquet) 
!   1.1   28/9/2017 Change comment satid (Pascale Roquet) 
module avhrr_maia4_header_io

#ifdef HAS_LIBHDF5
#include "aapp_throw.h"
!use rttov_print_mod
use aapp_hdf_mod
use avhrr_maia4_header_def
use avhrr_maia4_header_mem
use hdf5

implicit none
private
public :: avhrr_maia4_header_rh
public :: avhrr_maia4_header_wh
public :: avhrr_maia4_header_writea


contains


subroutine avhrr_maia4_header_writea(x,lun,err)
type(avhrr_maia4_header),intent(in)::x
integer,intent(in)::lun
integer,intent(out)::err
integer::lb(7),ub(7)
character(len=nlen)::sname

TRY
sname='CMa_Version'
write(lun,'("SECTION = ",A)',err=999)TRIM(sname)
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("COMMENT = ",A)',err=999) &
 & 'version number of the cloud mask'
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("UNITS = ",A)',iostat=err)''
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
call print_array_integer(lun,'(10000(I4))',a0=x%CMa_Version)
!write(lun,err=999)x%CMa_Version

sname='Nb_lines'
write(lun,'("SECTION = ",A)',err=999)TRIM(sname)
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("COMMENT = ",A)',err=999) &
 & 'count of scan lines in this data set'
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("UNITS = ",A)',iostat=err)''
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
call print_array_integer(lun,'(10000(I4))',a0=x%Nb_lines)
!write(lun,err=999)x%Nb_lines

sname='Nb_pixels'
write(lun,'("SECTION = ",A)',err=999)TRIM(sname)
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("COMMENT = ",A)',err=999) &
 & 'number of pixels in line'
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("UNITS = ",A)',iostat=err)''
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
call print_array_integer(lun,'(10000(I4))',a0=x%Nb_pixels)
!write(lun,err=999)x%Nb_pixels

sname='dataname'
write(lun,'("SECTION = ",A)',err=999)TRIM(sname)
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("COMMENT = ",A)',err=999) &
 & 'dataname of this file'
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("UNITS = ",A)',iostat=err)''
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
call print_array_character(lun,'(10000(A68))',a1=x%dataname)
!write(lun,err=999)x%dataname

sname='datatyp'
write(lun,'("SECTION = ",A)',err=999)TRIM(sname)
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("COMMENT = ",A)',err=999) &
 & 'data type code (1 : Direct Readout, 2 : Global)'
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("UNITS = ",A)',iostat=err)''
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
call print_array_integer(lun,'(10000(I4))',a0=x%datatyp)
!write(lun,err=999)x%datatyp

sname='end_day'
write(lun,'("SECTION = ",A)',err=999)TRIM(sname)
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("COMMENT = ",A)',err=999) &
 & 'end of dataset: day of year'
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("UNITS = ",A)',iostat=err)''
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
call print_array_integer(lun,'(10000(I4))',a0=x%end_day)
!write(lun,err=999)x%end_day

sname='end_orbit'
write(lun,'("SECTION = ",A)',err=999)TRIM(sname)
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("COMMENT = ",A)',err=999) &
 & 'end of orbit number'
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("UNITS = ",A)',iostat=err)''
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
call print_array_integer(lun,'(10000(I8))',a0=x%end_orbit)
!write(lun,err=999)x%end_orbit

sname='end_time'
write(lun,'("SECTION = ",A)',err=999)TRIM(sname)
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("COMMENT = ",A)',err=999) &
 & 'end of dataset: UTC time of day in ms'
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("UNITS = ",A)',iostat=err)''
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
call print_array_integer(lun,'(10000(I10))',a0=x%end_time)
!write(lun,err=999)x%end_time

sname='end_year'
write(lun,'("SECTION = ",A)',err=999)TRIM(sname)
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("COMMENT = ",A)',err=999) &
 & 'end of dataset: year'
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("UNITS = ",A)',iostat=err)''
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
call print_array_integer(lun,'(10000(I4))',a0=x%end_year)
!write(lun,err=999)x%end_year

sname='inst_id'
write(lun,'("SECTION = ",A)',err=999)TRIM(sname)
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("COMMENT = ",A)',err=999) &
 & 'instrument identification (AVHRR=1, VIIRS=2)'
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("UNITS = ",A)',iostat=err)''
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
call print_array_integer(lun,'(10000(I4))',a0=x%inst_id)
!write(lun,err=999)x%inst_id

sname='sat_id'
write(lun,'("SECTION = ",A)',err=999)TRIM(sname)
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("COMMENT = ",A)',err=999) &
 & 'satellite id in series (1=NPP or METOP-1, 2=METOP-2, 19=NOAA19)'
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("UNITS = ",A)',iostat=err)''
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
call print_array_integer(lun,'(10000(I4))',a0=x%sat_id)
!write(lun,err=999)x%sat_id

sname='site1b'
write(lun,'("SECTION = ",A)',err=999)TRIM(sname)
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("COMMENT = ",A)',err=999) &
 & 'creation site of l1b data (CMS, EUM, NOA)'
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("UNITS = ",A)',iostat=err)''
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
call print_array_character(lun,'(10000(A3))',a1=x%site1b)
!write(lun,err=999)x%site1b

sname='site1c'
write(lun,'("SECTION = ",A)',err=999)TRIM(sname)
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("COMMENT = ",A)',err=999) &
 & 'Dataset creation site ID (CMS)'
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("UNITS = ",A)',iostat=err)''
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
call print_array_character(lun,'(10000(A3))',a1=x%site1c)
!write(lun,err=999)x%site1c

sname='start_day'
write(lun,'("SECTION = ",A)',err=999)TRIM(sname)
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("COMMENT = ",A)',err=999) &
 & 'start of dataset: day of year'
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("UNITS = ",A)',iostat=err)''
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
call print_array_integer(lun,'(10000(I4))',a0=x%start_day)
!write(lun,err=999)x%start_day

sname='start_orbit'
write(lun,'("SECTION = ",A)',err=999)TRIM(sname)
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("COMMENT = ",A)',err=999) &
 & 'start orbit number'
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("UNITS = ",A)',iostat=err)''
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
call print_array_integer(lun,'(10000(I8))',a0=x%start_orbit)
!write(lun,err=999)x%start_orbit

sname='start_time'
write(lun,'("SECTION = ",A)',err=999)TRIM(sname)
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("COMMENT = ",A)',err=999) &
 & 'start of dataset: UTC time of day in ms'
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("UNITS = ",A)',iostat=err)''
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
call print_array_integer(lun,'(10000(I10))',a0=x%start_time)
!write(lun,err=999)x%start_time

sname='start_year'
write(lun,'("SECTION = ",A)',err=999)TRIM(sname)
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("COMMENT = ",A)',err=999) &
 & 'start of dataset: year'
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
write(lun,'("UNITS = ",A)',iostat=err)''
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
call print_array_integer(lun,'(10000(I4))',a0=x%start_year)
!write(lun,err=999)x%start_year

write(lun,'("SECTION = EOF")',err=999)
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
err=0
CATCH
end subroutine

subroutine avhrr_maia4_header_rh(x,lun,err)
type(avhrr_maia4_header),intent(out)::x
integer(hid_t),intent(in)::lun
integer,intent(out)::err
character(len=nlen)::sname
logical :: lext
TRY

call avhrr_maia4_header_init(x)

sname='CMa_Version'
call read_array_hdf(lun,sname,err,i0=x%CMa_Version)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='Nb_lines'
call read_array_hdf(lun,sname,err,i0=x%Nb_lines)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='Nb_pixels'
call read_array_hdf(lun,sname,err,i0=x%Nb_pixels)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='dataname'
call read_array_hdf(lun,sname,err,c1=x%dataname)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='datatyp'
call read_array_hdf(lun,sname,err,i0=x%datatyp)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='end_day'
call read_array_hdf(lun,sname,err,i0=x%end_day)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='end_orbit'
call read_array_hdf(lun,sname,err,i0=x%end_orbit)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='end_time'
call read_array_hdf(lun,sname,err,i0=x%end_time)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='end_year'
call read_array_hdf(lun,sname,err,i0=x%end_year)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='inst_id'
call read_array_hdf(lun,sname,err,i0=x%inst_id)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='sat_id'
call read_array_hdf(lun,sname,err,i0=x%sat_id)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='site1b'
call read_array_hdf(lun,sname,err,c1=x%site1b)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='site1c'
call read_array_hdf(lun,sname,err,c1=x%site1c)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='start_day'
call read_array_hdf(lun,sname,err,i0=x%start_day)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='start_orbit'
call read_array_hdf(lun,sname,err,i0=x%start_orbit)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='start_time'
call read_array_hdf(lun,sname,err,i0=x%start_time)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='start_year'
call read_array_hdf(lun,sname,err,i0=x%start_year)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

err=0
CATCH
end subroutine

subroutine avhrr_maia4_header_wh(x,lun,err, compress)
type(avhrr_maia4_header),intent(in)::x
integer(hid_t),intent(in)::lun
integer,intent(out)::err
logical,intent(in),optional::compress
character(len=nlen)::sname
TRY
sname='CMa_Version'
call write_array_hdf(lun,sname,&
  & 'version number of the cloud mask',&
  & err,i0=x%CMa_Version  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='Nb_lines'
call write_array_hdf(lun,sname,&
  & 'count of scan lines in this data set',&
  & err,i0=x%Nb_lines  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='Nb_pixels'
call write_array_hdf(lun,sname,&
  & 'number of pixels in line',&
  & err,i0=x%Nb_pixels  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='dataname'
call write_array_hdf(lun,sname,&
  & 'dataname of this file',&
  & err,c1=x%dataname  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='datatyp'
call write_array_hdf(lun,sname,&
  & 'data type code (1 : Direct Readout, 2 : Global)',&
  & err,i0=x%datatyp  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='end_day'
call write_array_hdf(lun,sname,&
  & 'end of dataset: day of year',&
  & err,i0=x%end_day  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='end_orbit'
call write_array_hdf(lun,sname,&
  & 'end of orbit number',&
  & err,i0=x%end_orbit  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='end_time'
call write_array_hdf(lun,sname,&
  & 'end of dataset: UTC time of day in ms',&
  & err,i0=x%end_time  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='end_year'
call write_array_hdf(lun,sname,&
  & 'end of dataset: year',&
  & err,i0=x%end_year  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='inst_id'
call write_array_hdf(lun,sname,&
  & 'instrument identification (AVHRR=1, VIIRS=2)',&
  & err,i0=x%inst_id  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='sat_id'
call write_array_hdf(lun,sname,&
  & 'satellite id in series (1=NPP or METOP-1, 2=METOP-2, 19=NOAA19)',&
  & err,i0=x%sat_id  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='site1b'
call write_array_hdf(lun,sname,&
  & 'creation site of l1b data (CMS, EUM, NOA)',&
  & err,c1=x%site1b  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='site1c'
call write_array_hdf(lun,sname,&
  & 'Dataset creation site ID (CMS)',&
  & err,c1=x%site1c  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='start_day'
call write_array_hdf(lun,sname,&
  & 'start of dataset: day of year',&
  & err,i0=x%start_day  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='start_orbit'
call write_array_hdf(lun,sname,&
  & 'start orbit number',&
  & err,i0=x%start_orbit  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='start_time'
call write_array_hdf(lun,sname,&
  & 'start of dataset: UTC time of day in ms',&
  & err,i0=x%start_time  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='start_year'
call write_array_hdf(lun,sname,&
  & 'start of dataset: year',&
  & err,i0=x%start_year  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

err=0
CATCH

end subroutine
#endif
end module
