! Name: $Id $
!
! Purpose: 
!          Subroutines maia_Write_AvhCT_hdf5 maia_Write_AvhCT_h_h5 maia_Write_AvhCT_d_h5 
!                      maia_Read_AvhCT_h_h5 maia_Read_AvhCT_d_h5 maia_ReWrite_AvhCT_h_h5 maia_ReWrite_AvhCT_d_h5
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/05/2015   Lydie Lavanant
!  1.1    23/01/2017   Pascale Roquet bug fixes and change some dataset names
!  1.2    25/08/2017   Pascale Roquet  maia_par is an array of maia_param type structure
!  1.3    29/09/2017   Pascale Roquet change one comment
!=====================================================================================
subroutine maia_Write_AvhCT_hdf5 (idbg,field_id,field_1b,maia_par,filename_in,filename_out,compress)
!=====================================================================================

! Purpose: Subroutine to write Maia cloud mask to HDF-5 file
#ifdef HAS_LIBHDF5
use hdf5
use h5lt
use aapp_hdf_mod

#endif

 use mod_maia_att
use mod_maia_const
use mod_maia_types
 use avhrr_maia4_data_def
 use avhrr_maia4_data_io
 use avhrr_maia4_data_mem
 use avhrr_maia4_header_def
 use avhrr_maia4_header_io
 use avhrr_maia4_header_mem


#include "aapp_throw.h"

#include "aapp_throw.h"

  IMPLICIT NONE
  type( debug ),    INTENT(in)  :: idbg
  Type (field_info), intent(in) :: field_id
  Type (field), intent(in)      :: field_1b
  type (maia_param), intent(in)    :: maia_par(field_1b % Nb_pixels,field_1b % Nb_Lines)
  CHARACTER(LEN=13), intent(in) :: filename_in
  CHARACTER(LEN=13), intent(in) :: filename_out 
  LOGICAL, intent (in),optional :: compress  

#ifdef HAS_LIBHDF5

  INTEGER(HID_T) :: file_id    ! file identifier
  INTEGER(HID_T) :: group_id
  INTEGER(HID_T) :: dataset_id
  ! For the header
  ! ...................
  CHARACTER(LEN=80)     :: nameAttribut
  CHARACTER (LEN=3) :: site1b   !creation site for original 1B data
  INTEGER ::  versnb = 4        !Cloud Mask format version number
  INTEGER ::  satid             !satellite id (e.g. 14 for NOAA-14)
  INTEGER ::  instid            !instrument identification AVHRR=1
  INTEGER ::  datatyp           !data type code (1 = LAC; 2 = GAC; 3 = HRPT; 4 = global)
  INTEGER ::  misscnlin         !
  INTEGER ::  startdatayr       !start of dataset year
  INTEGER ::  startdatady       !start of dataset day of year
  INTEGER ::  startdatatime     !start of dataset UTC time of day in ms
  INTEGER ::  enddatayr         !end of dataset year
  INTEGER ::  enddatady         !end of dataset day of year
  INTEGER ::  enddatatime       !end of dataset UTC time of day in ms
  INTEGER ::  startorbit        !orbit
  INTEGER ::  endorbit          !orbit
  INTEGER ::  nbline            !count of scan lines in this data set
  INTEGER ::  callocscnlin      !count of calibrated, earth located
  INTEGER ::  wmosatid          !WMO satellite id (3 for METOP-1)            

  integer       :: l, Nb_Lines, Nb_pixels, Pix_souspoint, jour, jj, ilat, ilon
  integer       :: ic, il, ip, nbcar
  integer :: ival, ival0=0, ival1=1
  INTEGER :: ig_start(7), ig_end(7), iq(3)  ! iq= (year, day of year, msec of day)
  CHARACTER(LEN=1) :: DN
  CHARACTER(LEN=3) :: clat, orbit_asc, SAT
  CHARACTER(LEN=4) :: clon, acq

  type(avhrr_maia4_header) :: mh
  type(avhrr_maia4_data)   :: md

  INTEGER :: Used_Chan_quality=0
  INTEGER ::    geoloc_quality=0
  INTEGER ::    input_info_qual=0
  integer :: lig, First_lig, Last_lig, tmp_lig, lig_mid
  logical :: first_info
  CHARACTER(LEN=68) :: nom_fic
  integer :: format_version(1) = (/ 3 /)
  integer(size_t) :: size_format_version=1
  integer :: err

  character(len=256) :: gname, aname, dname, dataname
  character(len=2560) :: att

! ----------------------------------------------------------------------------
TRY  
  if(idbg %debug) write(*,*) "maia_Write_AvhCT_hdf5"
! initialize Fortran interface
  CALL OPEN_HDF( err )
  THROWM(err.ne.0,"Unable to open HDF library")

! ----------------------------------------------------------------------------
  ! OPEN THE input h5 FILE 
  !-------------------------
  call H5Fopen_f( filename_in, H5F_ACC_RDONLY_F, file_id, err )
  THROWM(ERR.NE.0,"CANNOT OPEN input "//TRIM(Filename_in))

  ! Read the attributes of the input AVHRR HDF5 file 
  ! -------------------------
  nameAttribut="filename"   ; nbcar=256  ; call Read_attribute_CHAR (nameAttribut, file_id, nbcar, dataname)
  nameAttribut="1bsite"     ; nbcar=3   ; call Read_attribute_CHAR (nameAttribut, file_id, nbcar, site1b)  
  nameAttribut="datatyp"          ; call Read_attribute_INT (nameAttribut, file_id,datatyp )    !datatyp_key = 1 = LAC; 2 = GAC; 3 = HRPT
  nameAttribut="callocscnlin"     ; call Read_attribute_INT (nameAttribut, file_id,callocscnlin )
  nameAttribut="misscnlin"        ; call Read_attribute_INT (nameAttribut, file_id,misscnlin )
  nameAttribut="instrument"       ; call Read_attribute_INT (nameAttribut, file_id, instid)
  nameAttribut="satid"            ; call Read_attribute_INT (nameAttribut, file_id, satid)
  nameAttribut="wmosatid"         ; call Read_attribute_INT (nameAttribut, file_id, wmosatid)
  nameAttribut="scnlin"           ; call Read_attribute_INT (nameAttribut, file_id, nbline)
  nameAttribut="startdatayr"      ; call Read_attribute_INT (nameAttribut, file_id,startdatayr )
  nameAttribut="startdatady"      ; call Read_attribute_INT (nameAttribut, file_id,startdatady )
  nameAttribut="startdatatime_ms" ; call Read_attribute_INT (nameAttribut, file_id,startdatatime )
  nameAttribut="startorbit"       ; call Read_attribute_INT (nameAttribut, file_id,startorbit )
  nameAttribut="enddatayr"        ; call Read_attribute_INT (nameAttribut, file_id, enddatayr)
  nameAttribut="enddatady"        ; call Read_attribute_INT (nameAttribut, file_id,enddatady )
  nameAttribut="enddatatime_ms"   ; call Read_attribute_INT (nameAttribut, file_id,enddatatime )
  nameAttribut="endorbit"         ; call Read_attribute_INT (nameAttribut, file_id,endorbit )
 
  ! fermeture du fichier
  call H5Fclose_f (file_id,err)
  THROWM(ERR.NE.0,"CANNOT close input "//TRIM(Filename_in))
! ----------------------------------------------------------------------------
! Create the output HDF5 file 
  call H5Fcreate_f (filename_out, H5F_ACC_TRUNC_F, file_id, err)
  THROWM(ERR.NE.0,"CANNOT CREATE "//TRIM(Filename_out))

! ----------------------------------------------------------------------------
! Determine the name of the output Maia AVHRR HDF5 file

  Nb_Lines      = field_1b % Nb_Lines    !number of scan lines in this dataset
  Nb_pixels     = field_1b % Nb_pixels          !number of pixels in line
  Pix_souspoint = NINT(Nb_pixels/2.)

  Last_lig=imanq
  First_lig=imanq
  do lig=1, field_1b% Nb_Lines
     if( maia_par(pix_souspoint,lig)%box_daytime> vmanq .and. First_lig==imanq) then
         First_lig=lig
         exit
     endif
  end do

  do lig=field_1b% Nb_Lines,1, -1
     if( maia_par(pix_souspoint,lig)%box_daytime> vmanq .and. Last_lig==imanq) then
         Last_lig=lig
         exit
     endif
   end do

  if(Last_lig<First_lig .and. Last_lig>imanq .and. First_lig>imanq) then
     tmp_lig=Last_lig
     Last_lig=First_lig
     First_lig=tmp_lig
  endif

  lig_mid= NINT( (Last_lig-First_lig)/2.)

  if(field_1b % lat(Pix_souspoint,First_lig)<field_1b % lat(Pix_souspoint,Last_lig)) then
     orbit_asc= 'ASC'        !write(*,*) 'Orbite montante'
  else
     orbit_asc= 'DES'        !write(*,*) 'Orbite descendante'
  endif

  first_info = .true.
  jour=-9
  do l=2,Nb_lines-1
     if(maia_par(pix_souspoint,l)%box_daytime> vmanq) then
        if(first_info) then
           jour=maia_par(pix_souspoint,l)%box_daytime  !0= Night / 1= Twilight / 2= Day / 3= Sunglint
           if(jour==3) jour=2
           first_info = .false.
        else
           jj=maia_par(pix_souspoint,l)%box_daytime
           if(jj==3) jj=2
           if(jj /= jour) then
              jour=1
              exit
           endif
        endif
     endif
  end do
  if(jour==0) DN='N'  !write(*,*) 'de nuit'
  if(jour==1) DN='T'  !write(*,*) 'de crepuscule'
  if(jour==2) DN='D'  !write(*,*) 'de jour'
  if(jour==-9) DN='X'  !write(*,*) 'inconnu'

  iq(1) = startdatayr
  iq(2) = startdatady
  iq(3) = startdatatime
  call xiqj(iq, field_1b % ij_start )
  call xijg(field_1b % ij_start, ig_start)

  iq(1) = enddatayr
  iq(2) = enddatady
  iq(3) = enddatatime
  call xiqj(iq, field_1b % ij_end )
  call xijg(field_1b % ij_end, ig_end)

  if(maia_par(Pix_souspoint,lig_mid)%box_daytime> vmanq) then
     ilat= NINT( field_1b % lat(Pix_souspoint,lig_mid) )
     ilon= NINT( field_1b % lon(Pix_souspoint,lig_mid) )
  else
     ilat= NINT( field_id % LatM_min + (field_id % LatM_max -  field_id % LatM_min)/2. )
     ilon= NINT( field_id % LonM_min + (field_id % LonM_max -  field_id % LonM_min)/2. )
  endif

  if(ilat >=10) write(clat,'(a1,i2)') '0',ilat
  if(ilat >=0 .and. ilat <10) write(clat,'(a2,i1)') '00',ilat
  if(ilat >=-10 .and. ilat <0) write(clat,'(a2,i1)') '-0',ABS(ilat)
  if(ilat <=-10) write(clat,'(i3)') ilat

  if(ilon >=100) write(clon,'(a1,i3)') '0',ilon
  if(ilon >=10 .and. ilon<100) write(clon,'(a2,i2)') '00',ilon
  if(ilon >=0 .and. ilon<10) write(clon,'(a3,i1)') '000',ilon
  if(ilon >-10 .and. ilon<0) write(clon,'(a3,i1)') '-00',ABS(ilon)
  if(ilon >-100 .and. ilon<=-10) write(clon,'(a2,i2)') '-0',ABS(ilon)
  if(ilon <=-100) write(clon,'(i4)') ilon

  if(datatyp==3) acq='DB'
  if(datatyp==1 .or. datatyp==4) acq='GL'

  SAT='M02'
  if(field_1b % mission=='noaa15') SAT='N15'
  if(field_1b % mission=='noaa16') SAT='N16'
  if(field_1b % mission=='noaa17') SAT='N17'
  if(field_1b % mission=='noaa18') SAT='N18'
  if(field_1b % mission=='noaa19') SAT='N19'
  if(field_1b % mission=='meto01') SAT='M01'
  if(field_1b % mission=='meto03') SAT='M03'

  write(nom_fic,'(a5,a1,a3,a1,a2,a1,i4.4,2i2.2,a2,3i2.2,a2,3i2.2,a1,a3, 2a1,a3,a3,a3,a4,a1,i5.5,a3)') &
        'avhCT','_', SAT,'_',acq,'_',ig_start(1),ig_start(2),ig_start(3),&
         '_S',ig_start(4),ig_start(5),ig_start(6),'_E',ig_end(4),ig_end(5),ig_end(6),&
        '_',orbit_asc,'_',DN,'_La',clat,'_Lo',clon,'_',startorbit,'.h5'

  write(*,*) '  File Name: '
  write(*,'(2x,a67)') nom_fic
  write(16,'(a67)') nom_fic

! ----------------------------------------------------------------------------
! Write the same header + nom_fic for the output Maia AVHRR HDF5 file 
! ----------------------------------------------------------------------------
  call avhrr_maia4_header_init(mh)


  ! fill the header 
  mh%site1b(1)   = site1b          !creation site id for original 1B data
  mh%CMa_Version   = versnb   !version number of the cloud mask
  mh%Nb_lines      = field_1b % Nb_lines      !number of scan lines in this dataset
  mh%Nb_pixels     = field_1b % Nb_pixels  !number of pixels in line
  mh%dataname(1)      = nom_fic
  mh%end_day       = enddatady     !end of dataset: day  of year
  mh%end_orbit     = endorbit !Ending Orbit Number
  mh%end_time      = enddatatime !end of dataset: UTC time of day in ms
  mh%end_year      = enddatayr !end of dataset: Year
  mh%inst_id       = 1    !instrument id (AVHRR=1 ; VIIRS=2)
  mh%sat_id        = satid     !satellite id in series  (1=NPP or METOP-1, 2=METOP-2, 19=NOAA19)'
  mh%site1c (1)    = AAPP_SITE_ID  !creation site id for cloud mask (CMS)
  mh%start_day     = startdatady     !start of dataset: day of year
  mh%start_orbit   = startorbit   !Beginning Orbit Number
  mh%start_time    = startdatatime !start of dataset: UTC time of day in ms
  mh%start_year    = startdatayr !start of dataset: year
 ! datatyp ="data type code (1 = LAC; 2 = GAC; 3 = HRPT; 4 = global)"
  mh% datatyp = datatyp

  gname = "/HEADER"
  CALL MKPAR( file_id, gname, group_id, err )
  THROWM(err.ne.0,"Cannot create group "//TRIM(gname))

  call avhrr_maia4_header_wh(mh,group_id,err)
  THROWM(ERR.ne.0,"BAD RETURN")

  CALL h5gclose_f( group_id, err)
  THROWM(err.ne.0,"Cannot close group "//TRIM(gname))



! ----------------------------------------------------------------------------
! Write the data of the output Maia AVHRR HDF5 file 
! ----------------------------------------------------------------------------
  ! create the DATA group
  call avhrr_maia4_data_init(md)

  Allocate( md%Alt_surface(Nb_pixels,Nb_lines) ,  &
          & md%CloudMask(Nb_pixels,Nb_lines) ,    &
          & md%CloudTopPres(Nb_pixels,Nb_lines) , &
          & md%CloudTopTemp(Nb_pixels,Nb_lines) , &
          & md%CloudType(Nb_pixels,Nb_lines) ,    &
          & md%Chan_Used(Avhrr_maxch)  ,                    &
          & md%Latitude(Nb_pixels,Nb_lines) ,     &
          & md%Longitude(Nb_pixels,Nb_lines) ,    &
          & md%Mask_ind(Nb_pixels,Nb_lines) ,     &
          & md%Sat_zenith (Nb_pixels,Nb_lines) ,  &
          & md%Tsurf(Nb_pixels,Nb_lines) ,        &
          & md%fov_qual(Nb_pixels,Nb_lines) ,     &
          & md%scnlin_day(Nb_lines) ,             &
          & md%scnlin_time(Nb_lines) ,            &
          & md%scnlin_year(Nb_lines) ,            &
          & md%scnline_qual(Nb_lines) ,           &
          & stat=err)
  THROWM(err.ne.0,"Cannot allocate HDF5 output data arrays")

  md%Nb_Chan_Used= Avhrr_maxch
  md%Chan_Used = Avhrr_ic

  ! SCAN LINE INFORMATION

  do il=1,Nb_lines
     call xijq(field_1b % ij(il), iq)
     md%scnlin_year(il) = iq(1)
     md%scnlin_day(il)  = iq(2)
     md%scnlin_time(il) = iq(3)
     md % scnline_qual(il)= field_1b % ScanQual(il)
  end do

  ! fov_qual
  md%fov_qual = 0
  do il=1,Nb_lines
  do ip=1,Avhrr_maxfovs
     Used_Chan_quality=0
     geoloc_quality=0
     input_info_qual=0

     do ic=1, Avhrr_maxch
        if(field_1b % Qual_Obs(ic,ip,il)==0) then
           call mvbits(ival0, 0,1,md%fov_qual(ip,il),ic+3)
        else
           call mvbits(ival1, 0,1,md%fov_qual(ip,il),ic+3)
        endif
     end do
     if(field_1b % Qual_Obs(1,ip,il)/=0) Used_Chan_quality=1
     if(field_1b % Qual_Obs(2,ip,il)/=0) Used_Chan_quality=1
     if(field_1b % ch3b(il) ==1) then        !3B
         if(field_1b % Qual_Obs(4,ip,il)/=0) Used_Chan_quality=1
     else                    !3A
         if(field_1b % Qual_Obs(3,ip,il)/=0) Used_Chan_quality=1
     endif                
     if(field_1b % Qual_Obs(5,ip,il)/=0) Used_Chan_quality=1
     if(field_1b % Qual_Obs(6,ip,il)/=0) Used_Chan_quality=1
    
     call mvbits(Used_Chan_quality, 0,1,md%fov_qual(ip,il),3)

     if(.not. field_1b % locqual(ip,il) ) geoloc_quality=1
     call mvbits(geoloc_quality, 0,1,md%fov_qual(ip,il),2)

     if(geoloc_quality==0 .and. Used_Chan_quality==0 ) input_info_qual=0
     if(geoloc_quality==1 .or.  Used_Chan_quality==1 ) input_info_qual=1

     if(IBITS(md%fov_qual(ip,il),5,1) ==1) input_info_qual=2       !10.8 mauvais
     call mvbits(input_info_qual, 0,2,md%fov_qual(ip,il),0)
  end do
  end do

  do il=1,Nb_lines
  do ip=1,Avhrr_maxfovs
     md%Latitude(ip,il)    = field_1b % lat(ip,il) * 10000 !10^4 x latitude  in degrees (North latitude positive)
     md%Longitude(ip,il)   = field_1b % lon(ip,il) * 10000 !10^4 x longitude in degrees (East longitude positive)
     md%Sat_zenith(ip,il)      = field_1b % satzen(ip,il)* 100 !10^2 x satellite zenith angle (in degrees)
     md%Alt_surface(ip,il) = maia_par (ip,il) %alt_surface * 100
  end do
  end do


  ! maskind:
  md%Mask_ind(:,:)=0
  ival0=0 ; ival1=1
  do il=1,Nb_lines
  do ip=1,Avhrr_maxfovs
     if(maia_par(ip,il)%Cloud_mask == imanq) then
     md%Mask_ind(ip,il)=0
     else
     call mvbits(ival0, 0,1,md%Mask_ind(ip,il),0)   ! 0  snow/ice from external data  0 Not Done
     call mvbits(ival1, 0,1,md%Mask_ind(ip,il),1)   ! 1  Cloud mask on pixel    0 Not Done
     call mvbits(ival0, 0,1,md%Mask_ind(ip,il),2)   ! 2  Thin Cirrus detection    0 Not Done
     call mvbits(ival1, 0,1,md%Mask_ind(ip,il),3)   ! 3  cloud type        0 Not done
     call mvbits(ival0, 0,1,md%Mask_ind(ip,il),4)   ! 4  cloud phase      0 Not Done
     call mvbits(ival0, 0,1,md%Mask_ind(ip,il),5)   ! 5  Cloud Height      0 Not Done
     call mvbits(ival0, 0,1,md%Mask_ind(ip,il),6)   ! 6  Active Fire detection     0 Not Done !for clear_land pixels
     call mvbits(ival0, 0,1,md%Mask_ind(ip,il),7)   ! 7  Aerosols        0 Not Done
     call mvbits(ival0, 0,1,md%Mask_ind(ip,il),8)   ! 8  Cloud Shadow from Neighbors  0 Not Done
     call mvbits(ival0, 0,1,md%Mask_ind(ip,il),9)   ! 9  CMa update from Neighbors    0 Not Done
     call mvbits(ival0, 0,1,md%Mask_ind(ip,il),10)   ! 10 Fire detection texture    0 Not Done
     call mvbits(ival0, 0,1,md%Mask_ind(ip,il),11)   ! 11 Aerosols texture      0 Not Done
     endif
  end do
  end do

  ! mask:
  md%CloudMask = 0
  do il=1,Nb_lines
  do ip=1,Avhrr_maxfovs
     ival=maia_par(ip,il)%Cloud_mask
     call mvbits(ival, 0,3,md%CloudMask(ip,il),0)   ! 0-2   Cloud Mask before Cloud Adjacency
            !       0=clear/1=cloudy/2=clear_on_snow/3=clear_on_ice/4=aerosol_dust_ash_fire
     ival=maia_par(ip,il)%Cloud_mask_qual
     ival = 3 -ival ! inverse the flag to be conscistent with the documentation
     call mvbits(ival, 0,2,md%CloudMask(ip,il),3)   ! 3-4  Cloud Mask Quality from the number of tests involved
            !       0=high/1=medium/2=low/3=bad
     ival=maia_par(ip,il)%cloud_mask_conf
     call mvbits(ival, 0,2,md%CloudMask(ip,il),5)   ! 5-6  Cloud Mask Confidence from the proximity of thresholds
            !  3=Clear/2=ProbablyClear/1=ProbablyCloudy/0=Cloudy
     ival=maia_par(ip,il)%cloud_mask_ad
     call mvbits(ival, 0,3,md%CloudMask(ip,il),7)  ! 7-9  Adjacent Pixel Cloud Mask from cloud confidence evaluation
            !   of surrounding pixels 0=Not tested
            !  0=Not tested/1=Clear/2=ProbablyClear/3=ProbablyCloudy/4=Cloudy
     ival=maia_par(ip,il)%Cloud_mask
     call mvbits(ival, 0,3,md%CloudMask(ip,il),10)  ! 10-12  Cloud Mask total after Cloud Adjacency process
            !    if Cloud Adjacency process not done same as bits 0-2
            !   0=clear/1=cloudy/2=clear_on_snow/3=clear_on_ice/4=aerosol_dust_ash_fire
     ival=maia_par(ip,il)%box_daytime
     call mvbits(ival, 0,2,md%CloudMask(ip,il),13)  ! 13-14   0=Night/1=Twilight/2=Day/3=Sunglint
     ival=maia_par(ip,il)%pixel_surface_type
     call mvbits(ival, 0,3,md%CloudMask(ip,il),15)      ! 15-17  0=Sea Water/1=Land// 1= mixed / 2= land / 3= desert / 4= snow_always                      
     ival=0
     if(maia_par(ip,il)%box_atlas_surface_type ==1) ival=1     
     call mvbits(ival, 0,1,md%CloudMask(ip,il),18)      ! 18     1= water & land in working box
     call mvbits(ival0, 0,2,md%CloudMask(ip,il),19)  ! 19-20   Snow / Ice Surface from ancillary data or forecast
            !   0=not tested/1=No/2=Yes
     ival=maia_par(ip,il)%cloud_opacity
     if (ival < 0) ival = 0
     call mvbits(ival, 0,1,md%CloudMask(ip,il),21)  ! 21     opaque Cloud  0=No /1=Yes
     call mvbits(ival0, 0,4,md%CloudMask(ip,il),22)  ! 22-25   Shadow/Aerosol/Fire/Dust
            !   0=Not tested/1=No/2=CloudShadow/3=Heavy Aerosol/4=Fire/5=DustAsh
     ival=maia_par(ip,il)%box_bg_tskin_src
     call mvbits(ival, 0,1,md%CloudMask(ip,il),26)  ! 26   Tskin bg source 0=climatology used /1=forecast used
     ival=maia_par(ip,il)%box_bg_wv_src
     call mvbits(ival, 0,1,md%CloudMask(ip,il),27)  ! 27   WV content bg source  0=climatology used /1=forecast used
  end do
  end do

  ! CloudType:
  md%CloudType = 0
  do il=1,Nb_lines
  do ip=1,Avhrr_maxfovs
     call mvbits(ival0, 0,4,md%CloudType(ip,il),0)  ! 0-3   cloud phase 0 Not Done
        ival=maia_par(ip,il)%cloud_type
     call mvbits(ival, 0,5,md%CloudType(ip,il),4)  ! 4-8   cloud type 0  Not done
            !  1=clearLand land/2=clearSea/3=ClearLandOnSnow/4=ClearSeaOnSnow/ice
            !  5=veryLow/7=low /9=medium/11=high opaque/13=very high opaque
            !  15=thinSemiTrans/16=meanlyThickSemiTrans/17=thickSemiTrans
            !  18=SemiTrans above lower/19=fractional(sub-pixel water clouds)
            !  20=undefined by CMa
     call mvbits(ival0, 0,2,md%CloudType(ip,il),9)  ! 9-10  thin cirrus  0= not done  1= yes 2=no
  end do
  end do

  ! CloudTopTemperature:
 md%CloudTopTemp = imanq   ! -9999=missing values
  do il=1,Nb_lines
  do ip=1,Avhrr_maxfovs
     if(maia_par(ip,il)%cloud_top_temp >vmanq) then
        md%CloudTopTemp(ip,il)=(maia_par(ip,il)%cloud_top_temp-273.15)*100  ! cloud top temperature (C*100) for confident cloudy and opaque
     endif
  end do
  end do

  ! CloudTopPressure:
  md%CloudTopPres=imanq       ! -9999=missing values
  do il=1,Nb_lines
  do ip=1,Avhrr_maxfovs
     if(maia_par(ip,il)%cloud_top_press >vmanq) then
        md%CloudTopPres(ip,il)=maia_par(ip,il)%cloud_top_press*10  ! cloud top pressure (hPa*10) for confident cloudy and opaque using NWP profile
     endif
  end do
  end do

  ! SurfaceTemperature:
  md%Tsurf = imanq        ! -9999=missing values
  do il=1,Nb_lines
  do ip=1,Avhrr_maxfovs
     if(maia_par(ip,il)%surf_temp >vmanq) then
        md%Tsurf(ip,il)=(maia_par(ip,il)%surf_temp-273.15)*100  ! if clear retrieved surface temperature in C*100
              ! sea: SST Split-Window/land: LST from RTE using forecast and emissivity atlas
     endif
  end do
  end do
  ! missing value -9999 for altsurf
  where (md%Alt_surface < -30000 ) md%Alt_surface = -9999

! Write HDF5 data

  gname = "/DATA"
  CALL MKPAR( file_id, gname, group_id, err )
  THROWM(err.ne.0,"Cannot create group "//TRIM(gname))

  call avhrr_maia4_data_wh(md,group_id,err, compress)
  THROWM(ERR.ne.0,"BAD RETURN")

  CALL h5gclose_f( group_id, err)
  THROWM(err.ne.0,"Cannot close group "//TRIM(gname))

! Write attributes
  dname="/"

! Write attributes
  dname="/"
  aname="format version"

  call h5ltset_attribute_int_f(file_id,dname,aname,format_version,size_format_version,err)
  THROWM(ERR.ne.0,"Cannot write attribute "//TRIM(aname))

  call fill_attribute_fov_qual_avhrr(att)
  dname = "/DATA/fov_qual"
  CALL MKDSR( file_id, dname, dataset_id, err )
  THROWM(ERR.ne.0,"Cannot create dataset"//TRIM(dname))
  aname="fov status"
  call h5ltset_attribute_string_f(dataset_id,dname,aname,TRIM(att),err)
  THROWM(ERR.ne.0,"Cannot write attribute "//TRIM(aname))
  CALL h5dclose_f( dataset_id, err)
  THROWM(err.ne.0,"Cannot close dataset "//TRIM(dname))
   
  call fill_attribute_Maskind(att)
  dname = "/DATA/Mask_ind"
  CALL MKDSR( file_id, dname, dataset_id, err )
  THROWM(ERR.ne.0,"Cannot create dataset"//TRIM(dname))
  aname="cloud mask status"
  call h5ltset_attribute_string_f(dataset_id,dname,aname,TRIM(att),err)
  THROWM(ERR.ne.0,"Cannot write attribute "//TRIM(aname))
  CALL h5dclose_f( dataset_id, err)
  THROWM(err.ne.0,"Cannot close dataset "//TRIM(dname))


  call fill_attribute_CloudMask(att)
  dname = "/DATA/CloudMask"
  CALL MKDSR( file_id, dname, dataset_id, err )
  THROWM(ERR.ne.0,"Cannot create dataset"//TRIM(dname))
  aname="CloudMask Info"
  call h5ltset_attribute_string_f(dataset_id,dname,aname,TRIM(att),err)
  THROWM(ERR.ne.0,"Cannot write attribute "//TRIM(aname))
  CALL h5dclose_f( dataset_id, err)
  THROWM(err.ne.0,"Cannot close dataset "//TRIM(dname))


  call fill_attribute_CloudType(att)
  dname = "/DATA/CloudType"
  CALL MKDSR( file_id, dname, dataset_id, err )
  THROWM(ERR.ne.0,"Cannot create dataset"//TRIM(dname))
  aname="CloudType Info"
  call h5ltset_attribute_string_f(dataset_id,dname,aname,TRIM(att),err)
  THROWM(ERR.ne.0,"Cannot write attribute "//TRIM(aname))
  CALL h5dclose_f( dataset_id, err)
  THROWM(err.ne.0,"Cannot close dataset "//TRIM(dname))


 call fill_attribute_Tsurf(att)
  dname = "/DATA/Tsurf"
  CALL MKDSR( file_id, dname, dataset_id, err )
  THROWM(ERR.ne.0,"Cannot create dataset"//TRIM(dname))
  aname="Tsurf Info"
  call h5ltset_attribute_string_f(dataset_id,dname,aname,TRIM(att),err)
  THROWM(ERR.ne.0,"Cannot write attribute "//TRIM(aname))
  CALL h5dclose_f( dataset_id, err)
  THROWM(err.ne.0,"Cannot close dataset "//TRIM(dname))


  CALL h5fclose_f(file_id, err)
  THROWM(err.ne.0,"Cannot close file "//TRIM(Filename_out))

  CALL CLOSE_HDF( err )
  THROWM(err.ne.0,"Close HDF library")

CATCH

#endif



end subroutine maia_Write_AvhCT_hdf5

