! Name: $Id$
!
! Purpose: 
!          Subroutines maia_Write_ViiCT_hdf5 
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/05/2015   Lydie Lavanant
!  1.1    23/01/2017   Pascale Roquet Change CMA version to 4.4
!  1.2    05/01/2017   Pascale Roquet : clean code
!  1.3    04/04/2017   Pascale Roquet Change format version to 3 use mod_maia_mod
!  1.5    05/05/2017   Pascale Roquet maia_par size 30->31 (add T2M)  and bug fix write cloud opacity
!  1.6    25/08/2017   Pascale Roquet  maia_par is an array of maia_param type structure
!  1.7    28/09/2017   Pascale Roquet change sat_id to number in serie (1=npp)
!  1.8    11/01/2018   Pascale Roquet adaptation for jpss1 and jpss2
!---------------------------------------------------------------------------  
!> maia_Write_ViiCT_hdf5 subroutine
!
!> @author      MF/DP/CMS/R&D 
!> @author      lydie lavanant/Pascale Roquet
!> </dl>
!> @brief       write output in hdf5 file <br>
!---------------------------------------------------------------------------  
subroutine maia_Write_ViiCT_hdf5 (idbg, field_id, field_M, field_I, maia_par,compress)
!-------------------------------------------------------------------------------
!
! Purpose: Subroutine to write Maia cloud mask to HDF-5 file
!
!-------------------------------------------------------------------------------
#ifdef HAS_LIBHDF5
use hdf5
use h5lt
use aapp_hdf_mod
use mod_maia_att
#endif

use mod_maia_const
use mod_maia_types
use viirs_maia4_data_def
use viirs_maia4_data_io
use viirs_maia4_data_mem
use viirs_maia4_header_def
use viirs_maia4_header_io
use viirs_maia4_header_mem

#include "aapp_throw.h"

  IMPLICIT NONE

  type( debug ),    INTENT(in)  :: idbg
  Type (field_info), intent(in) :: field_id
  Type (field), intent(in)      :: field_M
  Type (field), intent(in)      :: field_I
  type (maia_param), intent(in)    :: maia_par(field_M % Nb_pixels,field_M % Nb_Lines)
  LOGICAL, intent (in),optional :: compress  

#ifdef HAS_LIBHDF5

  CHARACTER(LEN=13), parameter :: filename    ="maiaCT_out.h5"
  CHARACTER(LEN=67)  :: nom_fic
  character(len=256) :: gname, aname, dname

  type(viirs_maia4_header) :: mh
  type(viirs_maia4_data)   :: md

  INTEGER(HID_T) :: file_id    ! file identifier
  INTEGER(HID_T) :: group_id
  INTEGER(HID_T) :: dataset_id

  integer :: l, Nb_lines, Nb_pixels, Pix_souspoint, jour, jj, ilat, ilon, orbite
  integer :: ic, ichan, il, ip, ilI, ipI
  integer :: ival, ival0=0, ival1=1

  INTEGER :: ig_start(7), ig_end(7), iq(3)  ! iq= (year, day of year, msec of day)
  INTEGER :: iq_start(3), iq_end(3),format_version(1)= (/ 3 /)
  integer(size_t) :: size_format_version=1
  CHARACTER(LEN=1) :: DN
  CHARACTER(LEN=3) :: clat, orbit_asc
  CHARACTER(LEN=4) :: clon, acq
  CHARACTER(LEN=5) :: satname

  character(len=2560) :: att

  INTEGER ::    MBAND_Used_Chan_quality=0
  INTEGER ::    IBAND_Used_Chan_quality=0
  INTEGER ::    geoloc_quality=0
  INTEGER ::    input_info_qual=0

  integer :: lig, First_lig, Last_lig, tmp_lig, lig_mid
  logical :: first_info
  integer :: err
  integer :: qual_obs

! ----------------------------------------------------------------------------
TRY  
  if(idbg %debug) write(*,*) "maia_Write_ViiCT_hdf5"
! initialize Fortran interface
  CALL OPEN_HDF( err )
  THROWM(err.ne.0,"Unable to open HDF library")

! ----------------------------------------------------------------------------
! Create the HDF5 file
  call H5Fcreate_f (filename, H5F_ACC_TRUNC_F, file_id, err)
  THROWM(ERR.NE.0,"CANNOT CREATE "//TRIM(Filename))

! ----------------------------------------------------------------------------

  Nb_lines  = field_M % Nb_lines      !number of scan lines in this dataset
  Nb_pixels  = field_M % Nb_pixels    !number of pixels in line
  Pix_souspoint = NINT(Nb_pixels/2.)

  Last_lig=imanq
  First_lig=imanq
  do lig=1, field_M% Nb_lines
     if( maia_par(pix_souspoint,lig)%box_daytime> vmanq .and. First_lig==imanq) then
         First_lig=lig
         exit
     endif
  end do
  do lig=field_M% Nb_lines,1, -1
     if( maia_par(pix_souspoint,lig)%box_daytime> vmanq .and. Last_lig==imanq) then
         Last_lig=lig
         exit
     endif
  end do

  if(Last_lig<First_lig .and. Last_lig>imanq .and. First_lig>imanq) then
     tmp_lig=Last_lig
     Last_lig=First_lig
     First_lig=tmp_lig
  endif

  lig_mid= NINT( (Last_lig-First_lig)/2.)

  if(field_M % lat(Pix_souspoint,First_lig)<field_M % lat(Pix_souspoint,Last_lig)) then
     orbit_asc= 'ASC'        !write(*,*) 'Orbite montante'
  else
     orbit_asc= 'DES'        !write(*,*) 'Orbite descendante'
  endif

  first_info = .true.
  jour=-9
  do l=2,Nb_lines-1
     if(maia_par(pix_souspoint,l)%box_daytime> vmanq) then
        if(first_info) then
           jour=maia_par(pix_souspoint,l)%box_daytime  !0= Night / 1= Twilight / 2= Day / 3= Sunglint
           if(jour==3) jour=2
           first_info = .false.
        else
           jj=maia_par(pix_souspoint,l)%box_daytime
           if(jj==3) jj=2
           if(jj /= jour) then
              jour=1
              exit
           endif
        endif
     endif
  end do
  if(jour==0) DN='N'  
  if(jour==1) DN='T'  
  if(jour==2) DN='D'  
  if(jour==-9) DN='X'  

  call xijg(field_M % ij_start, ig_start)
  call xijg(field_M % ij_end, ig_end)

  if(maia_par(pix_souspoint,lig_mid)%box_daytime> vmanq) then
     ilat= NINT( field_M % lat(Pix_souspoint,lig_mid) )
     ilon= NINT( field_M % lon(Pix_souspoint,lig_mid) )
  else
     ilat= NINT( field_id % LatM_min + (field_id % LatM_max -  field_id % LatM_min)/2. )
     ilon= NINT( field_id % LonM_min + (field_id % LonM_max -  field_id % LonM_min)/2. )
  endif

  if(ilat >=10) write(clat,'(a1,i2)') '0',ilat
  if(ilat >=0 .and. ilat <10) write(clat,'(a2,i1)') '00',ilat
  if(ilat >=-10 .and. ilat <0) write(clat,'(a2,i1)') '-0',ABS(ilat)
  if(ilat <=-10) write(clat,'(i3)') ilat

  if(ilon >=100) write(clon,'(a1,i3)') '0',ilon
  if(ilon >=10 .and. ilon<100) write(clon,'(a2,i2)') '00',ilon
  if(ilon >=0 .and. ilon<10) write(clon,'(a3,i1)') '000',ilon
  if(ilon >-10 .and. ilon<0) write(clon,'(a3,i1)') '-00',ABS(ilon)
  if(ilon >-100 .and. ilon<=-10) write(clon,'(a2,i2)') '-0',ABS(ilon)
  if(ilon <=-100) write(clon,'(i4)') ilon

  ! creation site id for the distinction between BD and Globalliste des mots dans le header
  if( field_M % Distributor(1:4) == 'noaa' .or. field_M % Distributor(1:4) == 'arch') then
     acq='GL'  ! global
  else
     acq='DB'  ! direct broadcast
  endif
  satname = "_npp_"
  if ( field_M % mission .eq. "jpss1" )  then
       satname = "_j01_"       
  elseif  ( field_M % mission .eq. "jpss2" )  then
       satname = "_j02_"  
  endif


  orbite = field_M % Orbit_num_start

  write(nom_fic,'(2a5,a2,a1,i4.4,2i2.2,a2,3i2.2,a2,3i2.2,a1,a3, 2a1,a3,a3,a3,a4,a1,i5.5,a3)') &
        'viiCT',satname,acq,'_',ig_start(1),ig_start(2),ig_start(3),&
         '_S',ig_start(4),ig_start(5),ig_start(6),'_E',ig_end(4),ig_end(5),ig_end(6),&
        '_',orbit_asc,'_',DN,'_La',clat,'_Lo',clon,'_',orbite,'.h5'

  write(*,*) '  File_out: '
  write(*,*) '  ',nom_fic
  write(16,'(a67)') nom_fic

! ----------------------------------------------------------------------------
! write the HEADER
! ----------------------------------------------------------------------------

  call viirs_maia4_header_init(mh)

  call xijq(field_M % ij_start, iq_start)
  call xijq(field_M % ij_end, iq_end)

  ! fill the header 
  if( field_M % Distributor(1:4) == 'noaa' .or. field_M % Distributor(1:4) == 'arch') then
     mh%site1b(1)   = 'NOA'          !creation site id for original 1B data (CMS, NOA)
     mh%datatyp     =  2             !data type code (1 = DB; 2 = global)
  else
     mh%site1b(1)   = AAPP_SITE_ID          !creation site id for original 1B data (CMS, NOA)
     mh%datatyp     =  1
  endif

  Allocate (mh%ModScan(size(field_M % ModScan)),stat=err)
  THROWM(err.ne.0,"Cannot allocate HDF5 output header arrays")
  mh%CMa_Version   = 4   !version number of the cloud mask
  mh%ModScan(:)    = field_M % ModScan(:) !Scan operational mode (0: processed)
  mh%N_Granule_ID(1)  = field_M % N_Granule_ID
  mh%Nb_Granules   = field_M % Nb_Granules !number of granules in this dataset
  mh%Nb_lines      = field_M % Nb_lines      !number of scan lines in this dataset
  mh%Nb_pixels     = field_M % Nb_pixels  !number of pixels in line
  mh%dataname(1)      = nom_fic
  mh%dataname_gmodo (1)= field_M % dataname_gmodo
  mh%end_day       = iq_end(2)     !end of dataset: day  of year
  mh%end_orbit     = field_M % Orbit_num_end !Ending Orbit Number
  mh%end_time      = iq_end(3) !end of dataset: UTC time of day in ms
  mh%end_year      = iq_end(1) !end of dataset: Year
  mh%inst_id       = 2    !instrument id (AVHRR=1 ; VIIRS=2)
  mh%sat_id        = 1     !satellite id in serie (1 = npp , 2 = jpss-1, 3 = jpss-2)
  if ( field_M % mission .eq. "jpss1" )  then
       mh%sat_id        =  2
  elseif  ( field_M % mission .eq. "jpss2" )  then
       mh%sat_id        =  3
  endif
  mh%site1c (1)    = AAPP_SITE_ID  !creation site id for cloud mask (CMS)
  mh%start_day     = iq_start(2)     !start of dataset: day of year
  mh%start_orbit   = field_M % Orbit_num_start   !Beginning Orbit Number
  mh%start_time    = iq_start(3) !start of dataset: UTC time of day in ms
  mh%start_year    = iq_start(1) !start of dataset: year

  gname = "/HEADER"
  CALL MKPAR( file_id, gname, group_id, err )
  THROWM(err.ne.0,"Cannot create group "//TRIM(gname))

  call viirs_maia4_header_wh(mh,group_id,err)
  THROWM(ERR.ne.0,"BAD RETURN")

  CALL h5gclose_f( group_id, err)
  THROWM(err.ne.0,"Cannot close group "//TRIM(gname))



  call viirs_maia4_data_init(md)

  Allocate( md%Alt_surface(Nb_pixels,Nb_lines) ,  &
          & md%CloudMask(Nb_pixels,Nb_lines) ,    &
          & md%CloudTopPres(Nb_pixels,Nb_lines) , &
          & md%CloudTopTemp(Nb_pixels,Nb_lines) , &
          & md%CloudType(Nb_pixels,Nb_lines) ,    &
          & md%MChan_Used(V_maxch_M),             &
          & md%IChan_Used(5) ,                    &
          & md%Latitude(Nb_pixels,Nb_lines) ,     &
          & md%Longitude(Nb_pixels,Nb_lines) ,    &
          & md%Mask_ind(Nb_pixels,Nb_lines) ,     &
          & md%Sat_zenith (Nb_pixels,Nb_lines) ,  &
          & md%Tsurf(Nb_pixels,Nb_lines) ,        &
          & md%fov_qual(Nb_pixels,Nb_lines) ,     &
          & md%scnlin_day(Nb_lines) ,             &
          & md%scnlin_time(Nb_lines) ,            &
          & md%scnlin_year(Nb_lines) ,            &
          & md%scnline_qual(Nb_lines) ,           &
          & stat=err)
  THROWM(err.ne.0,"Cannot allocate HDF5 output data arrays")

  md%Nb_IChan_Used=5
  md%IChan_Used=(/1,2,3,4,5/)
  md%Nb_MChan_Used=V_maxch_M
  md%MChan_Used=ic_Viirs_M (1:V_maxch_M)


  ! SCAN LINE INFORMATION
  do il=1,Nb_lines
     call xijq(field_M % ij(il), iq)
     md%scnlin_year(il) = iq(1)
     md%scnlin_day(il)  = iq(2)
     md%scnlin_time(il) = iq(3)
     md%scnline_qual(il)= field_M % ScanQual(il)
  end do

  ! fov_qual: from QF1_VIIRSIBANDSDR, QF1_SCAN_VIIRSSDRGEO, QF2_VIIRSSDRGEO
  md%fov_qual = 0
  do il=1,Nb_lines
  do ip=1,V_maxfovs
     MBAND_Used_Chan_quality=0
     IBAND_Used_Chan_quality=0
     geoloc_quality=0
     input_info_qual=0

     do ic=1, V_maxch_M
        ichan= ic_Viirs_M (ic)
        if(field_M % Qual_Obs(ic,ip,il)==0) then
           call mvbits(ival0, 0,1,md%fov_qual(ip,il),ichan+4)
        else
           call mvbits(ival1, 0,1,md%fov_qual(ip,il),ichan+4)
        endif
        if(field_M % Qual_Obs(ic,ip,il) /=0) MBAND_Used_Chan_quality=1
     end do
     call mvbits(MBAND_Used_Chan_quality, 0,1,md%fov_qual(ip,il),4)

     if(field_I % Nb_lines >0) then
        do ic=1, 5
           ichan= ic
           ! summarize qual of I resolution to M resolution 
           ipI=2*(ip-1) +1
           ilI=2*(il-1) +1
           qual_obs=field_I % Qual_Obs(ic,ipI,ilI)+ &
                    field_I % Qual_Obs(ic,ipI,ilI+1)+ &
                    field_I % Qual_Obs(ic,ipI+1,ilI)+ &
                    field_I % Qual_Obs(ic,ipI+1,ilI+1)
           if  (qual_obs .GT. 0 ) then
              qual_obs=1
           endif                
           call mvbits(qual_obs , 0,1,md%fov_qual(ip,il),ichan+20)
           if(qual_obs /=0) IBAND_Used_Chan_quality=1
        end do
     else
        do ic=1, 5
           call mvbits(ival1, 0,1,md%fov_qual(ip,il),ic+20)
        end do
        IBAND_Used_Chan_quality=1
     endif
     call mvbits(IBAND_Used_Chan_quality, 0,1,md%fov_qual(ip,il),3)

     if(.not. field_M % locqual(ip,il) ) geoloc_quality=1

     if(field_I % Nb_lines >0) then
        ipI=2*(ip-1) +1
        ilI=2*(il-1) +1
        if(.not. field_I % locqual(ipI,ilI)   .or. .not. field_I % locqual(ipI+1,ilI) .or. &
           .not. field_I % locqual(ipI,ilI+1) .or. .not. field_I % locqual(ipI+1,ilI+1) )  &
           geoloc_quality=1
     else
          geoloc_quality=1
     endif
     call mvbits(geoloc_quality, 0,1,md%fov_qual(ip,il),2)

     if(geoloc_quality==0 .and. MBAND_Used_Chan_quality==0 .and. IBAND_Used_Chan_quality==0) input_info_qual=0
     if(geoloc_quality==1 .or. MBAND_Used_Chan_quality==1 .or. IBAND_Used_Chan_quality==1) input_info_qual=1

     if(IBITS(md%fov_qual(ip,il),19,1) ==1) input_info_qual=2  !M15 mauvais
     call mvbits(input_info_qual, 0,2,md%fov_qual(ip,il),0)
  end do
  end do

  do il=1,Nb_lines
  do ip=1,V_maxfovs
     md%latitude(ip,il)   = field_M % lat(ip,il) * 10000  !10^4 x latitude  in degrees (North latitude positive)
     md%longitude(ip,il)  = field_M % lon(ip,il) * 10000  !10^4 x longitude in degrees (East longitude positive)
     md%Sat_zenith(ip,il) = field_M % satzen(ip,il) * 100  !10^2 x satellite zenith angle (in degrees)
     md%Alt_surface(ip,il) = maia_par (ip,il) %alt_surface * 100
  end do
  end do
   

  ! maskind:
  md%Mask_ind = 0
  ival0=0
  ival1=1
  do il=1,Nb_lines
  do ip=1,V_maxfovs
     if(maia_par(ip,il)%Cloud_mask == imanq) then
     md%Mask_ind(ip,il)=0
     else
     call mvbits(ival0, 0,1,md%Mask_ind(ip,il),0)   ! 0  snow/ice from external data  0 Not Done
     call mvbits(ival1, 0,1,md%Mask_ind(ip,il),1)   ! 1  Cloud mask on pixel    0 Not Done
     call mvbits(ival0, 0,1,md%Mask_ind(ip,il),2)   ! 2  Thin Cirrus detection    0 Not Done
     call mvbits(ival1, 0,1,md%Mask_ind(ip,il),3)   ! 3  cloud type        0 Not done
     call mvbits(ival0, 0,1,md%Mask_ind(ip,il),4)   ! 4  cloud phase      0 Not Done
     call mvbits(ival0, 0,1,md%Mask_ind(ip,il),5)   ! 5  Cloud Height      0 Not Done
     call mvbits(ival0, 0,1,md%Mask_ind(ip,il),6)   ! 6  Active Fire detection     0 Not Done !for clear_land pixels
     call mvbits(ival0, 0,1,md%Mask_ind(ip,il),7)   ! 7  Aerosols        0 Not Done
     call mvbits(ival0, 0,1,md%Mask_ind(ip,il),8)   ! 8  Cloud Shadow from Neighbors  0 Not Done
     call mvbits(ival0, 0,1,md%Mask_ind(ip,il),9)   ! 9  CMa update from Neighbors    0 Not Done
     call mvbits(ival0, 0,1,md%Mask_ind(ip,il),10)   ! 10 Fire detection texture    0 Not Done
     call mvbits(ival0, 0,1,md%Mask_ind(ip,il),11)   ! 11 Aerosols texture      0 Not Done
     endif
  end do
  end do

  ! mask:
  md%CloudMask = 0
  do il=1,Nb_lines
  do ip=1,V_maxfovs
     ival=maia_par(ip,il)%Cloud_mask
     call mvbits(ival, 0,3,md%CloudMask(ip,il),0)   ! 0-2   Cloud Mask before Cloud Adjacency
            !       0=clear/1=cloudy/2=clear_on_snow/3=clear_on_ice/4=aerosol_dust_ash_fire
     ival=maia_par(ip,il)%Cloud_mask_qual
     ival = 3 -ival ! inverse the flag to be conscistent with the documentation
     call mvbits(ival, 0,2,md%CloudMask(ip,il),3)   ! 3-4  Cloud Mask Quality from the number of tests involved
            !       0=high/1=medium/2=low/3=bad
     ival=maia_par(ip,il)%Cloud_mask_conf
     call mvbits(ival, 0,2,md%CloudMask(ip,il),5)   ! 5-6  Cloud Mask Confidence from the proximity of thresholds
            !  3=Clear/2=ProbablyClear/1=ProbablyCloudy/0=Cloudy
     ival=maia_par(ip,il)%Cloud_mask_ad
     call mvbits(ival, 0,3,md%CloudMask(ip,il),7)  ! 7-9  Adjacent Pixel Cloud Mask from cloud confidence evaluation
            !   of surrounding pixels 0=Not tested
            !  0=Not tested/1=Clear/2=ProbablyClear/3=ProbablyCloudy/4=Cloudy
     ival=maia_par(ip,il)%Cloud_mask
     call mvbits(ival, 0,3,md%CloudMask(ip,il),10)  ! 10-12  Cloud Mask total after Cloud Adjacency process
            !    if Cloud Adjacency process not done same as bits 0-2
            !   0=clear/1=cloudy/2=clear_on_snow/3=clear_on_ice/4=aerosol_dust_ash_fire
     ival=maia_par(ip,il)%box_daytime
     call mvbits(ival, 0,2,md%CloudMask(ip,il),13)  ! 13-14   0=Night/1=Twilight/2=Day/3=Sunglint
     ival=maia_par(ip,il)%pixel_surface_type
     call mvbits(ival, 0,3,md%CloudMask(ip,il),15)      ! 15-17  0= sea+inland-water / 1= mixed / 2= land / 3= desert / 4= snow_always
                                                !        at viirs M pixel resolution
     ival=0
     if(maia_par(ip,il)%box_atlas_surface_type ==1) ival=1     
     call mvbits(ival, 0,1,md%CloudMask(ip,il),18)      ! 18     1= water & land in working box
     call mvbits(ival0, 0,2,md%CloudMask(ip,il),19)  ! 19-20   Snow / Ice Surface from ancillary data or forecast
            !   0=not tested/1=No/2=Yes
     ival=maia_par(ip,il)%cloud_opacity
     if (ival < 0) ival = 0
     call mvbits(ival, 0,1,md%CloudMask(ip,il),21)  ! 21     opaque Cloud  0=No /1=Yes
     call mvbits(ival0, 0,4,md%CloudMask(ip,il),22)  ! 22-25   Shadow/Aerosol/Fire/Dust
            !   0=Not tested/1=No/2=CloudShadow/3=Heavy Aerosol/4=Fire/5=DustAsh
     ival=maia_par(ip,il)%box_bg_tskin_src
     call mvbits(ival, 0,1,md%CloudMask(ip,il),26)  ! 26   Tskin bg source 0=climatology used /1=forecast used
     ival=maia_par(ip,il)%box_bg_wv_src
     call mvbits(ival, 0,1,md%CloudMask(ip,il),27)  ! 27   WV content bg source  0=climatology used /1=forecast used
  end do
  end do

  ! CloudType:
  md%CloudType = 0
  do il=1,Nb_lines
  do ip=1,V_maxfovs
     call mvbits(ival0, 0,4,md%CloudType(ip,il),0)  ! 0-3   cloud phase 0 Not Done
     if(maia_par(ip,il)%cloud_type >vmanq) then
        ival=maia_par(ip,il)%cloud_type
     else
        ival=ival0
     endif
     call mvbits(ival, 0,5,md%CloudType(ip,il),4)  ! 4-8   cloud type 0  Not done
            !  1=clearLand land/2=clearSea/3=ClearLandOnSnow/4=ClearSeaOnSnow/ice
            !  5=veryLow/7=low /9=medium/11=high opaque/13=very high opaque
            !  15=thinSemiTrans/16=meanlyThickSemiTrans/17=thickSemiTrans
            !  18=SemiTrans above lower/19=fractional(sub-pixel water clouds)
            !  20=undefined by CMa
     call mvbits(ival0, 0,2,md%CloudType(ip,il),9)  ! 9-10  thin cirrus  0= not done  1= yes 2=no
  end do
  end do

  ! CloudTopTemperature:
  md%CloudTopTemp = imanq
  do il=1,Nb_lines
  do ip=1,V_maxfovs
     if(maia_par(ip,il)%cloud_top_temp >vmanq) then
        md%CloudTopTemp(ip,il)=(maia_par(ip,il)%cloud_top_temp-273.15)*100  ! cloud top temperature (C*100) for confident cloudy and opaque
     endif
  end do
  end do

  ! CloudTopPressure:
  md%CloudTopPres=imanq
  do il=1,Nb_lines
  do ip=1,V_maxfovs
     if(maia_par(ip,il)%cloud_top_press >-9000.) then
        md%CloudTopPres(ip,il)=maia_par(ip,il)%cloud_top_press*10  ! cloud top pressure (hPa*10) for confident cloudy and opaque using NWP profile
     endif
  end do
  end do

  ! SurfaceTemperature:
  md%Tsurf = imanq        ! -9999=missing values
  do il=1,Nb_lines
  do ip=1,V_maxfovs
     if(maia_par(ip,il)%surf_temp >vmanq) then
        md%Tsurf(ip,il)=(maia_par(ip,il)%surf_temp-273.15)*100  ! if clear retrieved surface temperature in C*100
              ! sea: SST Split-Window/land: LST from RTE using forecast and emissivity atlas
     endif
  end do
  end do


! Write HDF5 data

  gname = "/DATA"
  CALL MKPAR( file_id, gname, group_id, err )
  THROWM(err.ne.0,"Cannot create group "//TRIM(gname))

  call viirs_maia4_data_wh(md,group_id,err, compress)
  THROWM(ERR.ne.0,"BAD RETURN")

  CALL h5gclose_f( group_id, err)
  THROWM(err.ne.0,"Cannot close group "//TRIM(gname))

! Write attributes
  dname="/"
  aname="format version"

  call h5ltset_attribute_int_f(file_id,dname,aname,format_version,size_format_version,err)
  THROWM(ERR.ne.0,"Cannot write attribute "//TRIM(aname))

  call fill_attribute_fov_qual_viirs(att)
  dname = "/DATA/fov_qual"
  CALL MKDSR( file_id, dname, dataset_id, err )
  THROWM(ERR.ne.0,"Cannot create dataset"//TRIM(dname))
  aname="fov status"
  call h5ltset_attribute_string_f(dataset_id,dname,aname,TRIM(att),err)
  THROWM(ERR.ne.0,"Cannot write attribute "//TRIM(aname))
  CALL h5dclose_f( dataset_id, err)
  THROWM(err.ne.0,"Cannot close dataset "//TRIM(dname))
   
  call fill_attribute_Maskind(att)
  dname = "/DATA/Mask_ind"
  CALL MKDSR( file_id, dname, dataset_id, err )
  THROWM(ERR.ne.0,"Cannot create dataset"//TRIM(dname))
  aname="cloud mask status"
  call h5ltset_attribute_string_f(dataset_id,dname,aname,TRIM(att),err)
  THROWM(ERR.ne.0,"Cannot write attribute "//TRIM(aname))
  CALL h5dclose_f( dataset_id, err)
  THROWM(err.ne.0,"Cannot close dataset "//TRIM(dname))


  call fill_attribute_CloudMask(att)
  dname = "/DATA/CloudMask"
  CALL MKDSR( file_id, dname, dataset_id, err )
  THROWM(ERR.ne.0,"Cannot create dataset"//TRIM(dname))
  aname="CloudMask Info"
  call h5ltset_attribute_string_f(dataset_id,dname,aname,TRIM(att),err)
  THROWM(ERR.ne.0,"Cannot write attribute "//TRIM(aname))
  CALL h5dclose_f( dataset_id, err)
  THROWM(err.ne.0,"Cannot close dataset "//TRIM(dname))


  call fill_attribute_CloudType(att)
  dname = "/DATA/CloudType"
  CALL MKDSR( file_id, dname, dataset_id, err )
  THROWM(ERR.ne.0,"Cannot create dataset"//TRIM(dname))
  aname="CloudType Info"
  call h5ltset_attribute_string_f(dataset_id,dname,aname,TRIM(att),err)
  THROWM(ERR.ne.0,"Cannot write attribute "//TRIM(aname))
  CALL h5dclose_f( dataset_id, err)
  THROWM(err.ne.0,"Cannot close dataset "//TRIM(dname))


 call fill_attribute_Tsurf(att)
  dname = "/DATA/Tsurf"
  CALL MKDSR( file_id, dname, dataset_id, err )
  THROWM(ERR.ne.0,"Cannot create dataset"//TRIM(dname))
  aname="Tsurf Info"
  call h5ltset_attribute_string_f(dataset_id,dname,aname,TRIM(att),err)
  THROWM(ERR.ne.0,"Cannot write attribute "//TRIM(aname))
  CALL h5dclose_f( dataset_id, err)
  THROWM(err.ne.0,"Cannot close dataset "//TRIM(dname))


  CALL h5fclose_f(file_id, err)
  THROWM(err.ne.0,"Cannot close file "//TRIM(Filename))

  CALL CLOSE_HDF( err )
  THROWM(err.ne.0,"Close HDF library")

CATCH

#endif

end subroutine maia_Write_ViiCT_hdf5
