! Name: $Id $
!
! Purpose: 
!          Subroutines maia_Read_avh1c_d_h5 maia_read_Avh1c_hdf5,locl1b_2full, 
!                      XYZ_TO_LL
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/05/2015   Lydie Lavanant
!  1.1    08/08/2017   Update for old noaa
!  1.2    29/09/2017   Pascale Roquet add #ifdef
#ifdef HAS_LIBHDF5
!=====================================================================================
  subroutine maia_Read_avh1c_d_h5  (file_id, nbline, &
    scnlin,scnlinyr, scnlindy, scnlintime, ch3b, scnlinqual, &
    calqual, scalti_out, solzen_out, satzen_out, relaz_out, lat_out, lon_out, btemps_out, ios)
!=====================================================================================

! Purpose: Subroutine to read AVHRR cloud mask (HDF-5 file)
!----------------------------------------------------------

 USE HDF5

 IMPLICIT NONE

  INTEGER(HID_T), INTENT(IN) :: file_id         ! file identifier
  INTEGER , INTENT(IN)      ::   nbline         !

  REAL, INTENT(OUT)    :: btemps_out(5*2048*nbline)
  REAL, INTENT(OUT)    :: lat_out(51*nbline), lon_out(51*nbline)
  REAL, INTENT(OUT)    :: satzen_out(51*nbline), solzen_out(51*nbline), relaz_out(51*nbline)
  INTEGER, INTENT(OUT) :: scnlintime(nbline)
  INTEGER, INTENT(OUT) :: ch3b(nbline), calqual(nbline*3)
  REAL, INTENT(OUT)    :: scalti_out(nbline)
  INTEGER, INTENT(OUT) :: scnlin(nbline), scnlinyr(nbline), scnlindy(nbline), scnlinqual(nbline)
  INTEGER, INTENT(OUT) ::  ios          !

  INTEGER :: qualind(nbline)
  INTEGER(HID_T) :: dataset_id          ! dataset identifier
  INTEGER(HSIZE_T) :: dims(1), dims_2(2)
  INTEGER(HSIZE_T) :: dims_data(3) 
  INTEGER ::  status
  INTEGER ::  rank, i
  INTEGER    :: scalti(nbline), infos_orbit(nbline)
  INTEGER    :: btemps(5*2048*nbline)
  INTEGER    :: lat(51*nbline), lon(51*nbline)
  INTEGER    :: satzen(51*nbline), solzen(51*nbline), relaz(51*nbline)

  ! ----------------------------------------------------------------------------
  ios=0
  ! scnlin, scnlinyr, scnlindy, scnlintime, ch3b, asc, scalti, qualind, scnlinqual 
  ! ----------------------------------------------------------------------
  dims(1) = nbline

  call H5Dopen_f (file_id, "/Data/scnlin", dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_INTEGER, scnlin, dims, status)
  call H5Dclose_f (dataset_id, status)

  call H5Dopen_f (file_id, "/Data/scnlinyr", dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_INTEGER, scnlinyr, dims, status)
  call H5Dclose_f (dataset_id, status)

  call H5Dopen_f (file_id, "/Data/scnlindy", dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_INTEGER, scnlindy, dims, status)
  call H5Dclose_f (dataset_id, status)

  call H5Dopen_f (file_id, "/Data/scnlintime", dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_INTEGER, scnlintime, dims, status)
  call H5Dclose_f (dataset_id, status)
  
  !Description = 
  ! Bit  0: 0=3A, 1=3B; 
  ! bit 13: 1=corrected for TIP attitude; bit 14: 1=corrected for clock drift; 
  ! bit 15: 0=ascending, 1=descending
  call H5Dopen_f (file_id, "/Data/scnlinbit", dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_INTEGER, infos_orbit, dims, status)
  call H5Dclose_f (dataset_id, status)
  
  do i=1, nbline
  ch3b(i) = IBITS(infos_orbit(i),0,1)
  end do

  call H5Dopen_f (file_id, "/Data/scalti", dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_INTEGER, scalti, dims, status)
  call H5Dclose_f (dataset_id, status)
  scalti_out=scalti/10.

  call H5Dopen_f (file_id, "/Data/qualind", dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_INTEGER, qualind, dims, status)
  call H5Dclose_f (dataset_id, status)

  call H5Dopen_f (file_id, "/Data/scnlinqual", dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_INTEGER, scnlinqual, dims, status)
  call H5Dclose_f (dataset_id, status)

  ! calqual 
  ! -------
  rank = 2
  dims_2(1) = 3 ! 3 IR channels
  dims_2(2) = nbline    

  call H5Dopen_f (file_id, "/Data/calqual", dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_INTEGER, calqual, dims_2, status)
  call H5Dclose_f (dataset_id, status)

  ! lat, lon, satzen, solzen, relaz
  ! -------------------------------
  rank = 2
  dims_2(1) = 51
  dims_2(2) = nbline    
  
  call H5Dopen_f (file_id, "/Geolocation/Latitude", dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_INTEGER, lat, dims_2, status)
  call H5Dclose_f (dataset_id, status)
  lat_out(:)=lat(:)/10000.

  call H5Dopen_f (file_id, "/Geolocation/Longitude", dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_INTEGER, lon, dims_2, status)
  call H5Dclose_f (dataset_id, status)
  lon_out(:)=lon(:)/10000.

  call H5Dopen_f (file_id, "/Geolocation/Satellite_zenith_angle", dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_INTEGER, satzen, dims_2, status)
  call H5Dclose_f (dataset_id, status)
  satzen_out(:)=satzen(:)/100.

  call H5Dopen_f (file_id, "/Geolocation/Solar_zenith_angle", dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_INTEGER, solzen, dims_2, status)
  call H5Dclose_f (dataset_id, status)
  solzen_out(:)=solzen(:)/100.

  call H5Dopen_f (file_id, "/Geolocation/Relative_azimuth_angle", dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_INTEGER, relaz, dims_2, status)
  call H5Dclose_f (dataset_id, status)
  relaz_out(:)=relaz(:)/100.

  ! btemps
  ! --------------
  rank = 3
  dims_data(1) = 5
  dims_data(2) = 2048
  dims_data(3) = nbline
  
  call H5Dopen_f (file_id, "/Data/btemps", dataset_id ,status)
  call H5Dread_f (dataset_id, H5T_NATIVE_INTEGER, btemps, dims_data, status)
  call H5Dclose_f (dataset_id, status)
  ! deal with negatives values (int16 overflow in h5 file)
  write(*,*) "max/min btemps",maxval(btemps(:)), minval(btemps(:))
  where (btemps < -10000)  
       btemps  =  btemps +  65536
  end where
  write(*,*) "max/min btemps",maxval(btemps(:)), minval(btemps(:))
  btemps_out(:) = btemps(:)/100.

  end subroutine maia_Read_avh1c_d_h5

!=====================================================================================
subroutine maia_read_Avh1c_hdf5 ( idbg, filename, field_Avhrr )
!=====================================================================================

! Purpose: read AVHRR 1c in HDF-5 format
!-------------------------------------------------------------------------------

 use HDF5
 USE MOD_MAIA_CONST
 USE mod_maia_types

 IMPLICIT NONE

  type( debug ), intent(in)     :: idbg         ! 
  CHARACTER(LEN=13),intent(in)  :: filename
  Type (field), intent(out)     :: field_Avhrr

  INTEGER                       :: ios, l, p, err(11), iq(3)
  INTEGER(HID_T)                :: file_id              ! file identifier

  ! For the header
  ! ...................
  CHARACTER(LEN=80)     :: nameAttribut
  CHARACTER (LEN=90):: dataname !dataset name 
  INTEGER ::  satid             !satellite id (e.g. 14 for NOAA-14)
  INTEGER ::  startdatayr       !start of dataset year
  INTEGER ::  startdatady       !start of dataset day of year
  INTEGER ::  startdatatime     !start of dataset UTC time of day in ms
  INTEGER ::  enddatayr         !end of dataset year
  INTEGER ::  enddatady         !end of dataset day of year
  INTEGER ::  enddatatime       !end of dataset UTC time of day in ms
  INTEGER ::  nbline            !count of scan lines in this data set
  INTEGER ::  wmosatid          !WMO satellite id (3 for METOP-1)            
  INTEGER ::  datatyp           !data type code (1 = LAC; 2 = GAC; 3 = HRPT; 4 = global)
  INTEGER ::  misscnlin         !
  REAL    ::  ir_wavenumber(3), ir_radtemp_slope(3), ir_radtemp_offset(3)

  REAL, ALLOCATABLE,DIMENSION(:,:,:)    :: Obs_inH5     
  REAL, ALLOCATABLE,DIMENSION(:,:)      :: lat_inH5, lon_inH5, satzen_inH5, sunzen_inH5, relaz_inH5
  REAL, ALLOCATABLE,DIMENSION(:)        :: scalti
  INTEGER, ALLOCATABLE,DIMENSION(:)     :: ch3b , scnlin, scnlinyr, scnlindy, scnlinqual,scnlintime
  INTEGER, ALLOCATABLE,DIMENSION(:,:)   :: calqual
  ! ----------------------------------------------------------------------------  
  if(idbg % debug) then
      write(*,*) 'maia_Read_Avh1c_h5'
  end if
  ! ----------------------------------------------------------------------------  
  ! initialize Fortran interface
  call H5open_f (ios)
  ! ----------------------------------------------------------------------------
  ! OPEN THE MAIA h5 FILE 
  !-------------------------
  ios=0
  call H5Fopen_f( filename, H5F_ACC_RDONLY_F, file_id, ios ) 

  if(ios.ne.0) then
    call ml_wti('ERROR','pb open maia_out h5',ios)
    stop 2
  endif

  ! ----------------------------------------------------------------------------
  ! Read the attributes of the input AVHRR HDF5 file 
  ! -------------------------
  nameAttribut="filename"          ; call Read_attribute_CHAR (nameAttribut, file_id, 84, dataname)
  nameAttribut="datatyp"
  call Read_attribute_INT (nameAttribut, file_id,datatyp )      !datatyp_key = 1 = LAC; 2 = GAC; 3 = HRPT
  nameAttribut="misscnlin"
  call Read_attribute_INT (nameAttribut, file_id,misscnlin )
  nameAttribut="wmosatid"
  call Read_attribute_INT (nameAttribut, file_id, wmosatid)
  nameAttribut="satid"
  call Read_attribute_INT (nameAttribut, file_id, satid)
  nameAttribut="scnlin"
  call Read_attribute_INT (nameAttribut, file_id, nbline)
  nameAttribut="AVHRR_wavenumber"
  call Read_attribute_REAL (nameAttribut, file_id, 3, ir_wavenumber)
  nameAttribut="AVHRR_radtemp_slope"
  call Read_attribute_REAL (nameAttribut, file_id, 3, ir_radtemp_slope)
  nameAttribut="AVHRR_radtemp_offset"
  call Read_attribute_REAL (nameAttribut, file_id, 3, ir_radtemp_offset)
  nameAttribut="startdatayr"
  call Read_attribute_INT (nameAttribut, file_id,startdatayr )
  nameAttribut="startdatady"
  call Read_attribute_INT (nameAttribut, file_id,startdatady )
  nameAttribut="startdatatime_ms"
  call Read_attribute_INT (nameAttribut, file_id,startdatatime )
  nameAttribut="enddatayr"
  call Read_attribute_INT (nameAttribut, file_id, enddatayr)
  nameAttribut="enddatady"
  call Read_attribute_INT (nameAttribut, file_id,enddatady )
  nameAttribut="enddatatime_ms"
  call Read_attribute_INT (nameAttribut, file_id,enddatatime )
  
  field_Avhrr % mission                 = 'xxxxxx'
  if(wmosatid == 205) field_Avhrr % mission     = 'noaa14'
  if(wmosatid == 206) field_Avhrr % mission     = 'noaa15'
  if(wmosatid == 207) field_Avhrr % mission     = 'noaa16'
  if(wmosatid == 208) field_Avhrr % mission     = 'noaa17'
  if(wmosatid == 209) field_Avhrr % mission     = 'noaa18'
  if(wmosatid == 223) field_Avhrr % mission     = 'noaa19'
  if(wmosatid == 3) field_Avhrr % mission       = 'meto01'
  if(wmosatid == 4) field_Avhrr % mission       = 'meto02'
  if(wmosatid == 5) field_Avhrr % mission       = 'meto03'

  field_Avhrr % Nb_Lines        = nbline        ! number of lines in field for Moderate resolution channels
  field_Avhrr % Nb_pixels       = 2048          ! number of pixels in line for Moderate resolution channels
  field_Avhrr % Nb_chan         = 6             ! number of channels in field

  allocate (Obs_inH5(5,2048,nbline) )
  allocate (lat_inH5(51,nbline), lon_inH5(51,nbline) )
  allocate (satzen_inH5(51,nbline), sunzen_inH5(51,nbline), relaz_inH5(51,nbline) )
  allocate (ch3b(nbline), calqual(3,nbline), scnlinqual(nbline))
  allocate (scnlin(nbline), scnlinyr(nbline), scnlindy(nbline), scnlintime(nbline) )
  allocate (scalti(nbline) )

  allocate ( field_Avhrr % ch3b(nbline) )
  allocate ( field_Avhrr % Obs(6,2048,nbline) )
  allocate ( field_Avhrr % Qual_Obs(6,2048,nbline), stat = err(2))
  allocate ( field_Avhrr % lat(2048,nbline),    field_Avhrr % lon(2048,nbline) )
  allocate ( field_Avhrr % satzen(2048,nbline), field_Avhrr % sunzen(2048,nbline) )
  allocate ( field_Avhrr % difazi(2048,nbline) )
  allocate ( field_Avhrr % ij(nbline) , stat = err(9))          ! time of the line
  allocate ( field_Avhrr % locqual(2048,nbline), stat=err(10)) ! fov geoloc quality
  allocate ( field_Avhrr % scanqual(nbline), stat=err(11)) ! scan quality

  ! ----------------------------------------------------------------------------
  ! read the Data en hdf5
  ! -------------------------
  call maia_Read_avh1c_d_h5 (file_id, nbline, &
                        scnlin, scnlinyr, scnlindy, scnlintime, ch3b, scnlinqual, &
                        calqual, scalti, sunzen_inH5, satzen_inH5, relaz_inH5, lat_inH5, lon_inH5, Obs_inH5, ios)
  if(ios.ne.0) then
       call ml_wti('ERROR','pb read data in h5')
       stop 2
  endif

  ! traite les informations
  ! -------------------------
  field_Avhrr % Qual_Obs(:,:,:) =0
  field_Avhrr % ScanQual(:)     =0
  field_Avhrr % Obs(:,:,:)      =vmanq

  do l=1,nbline
     field_Avhrr % ch3b(l) = ch3b(l)

     call locl1b_2full (scalti(l),relaz_inH5(:,l), sunzen_inH5(:,l), satzen_inH5(:,l), &
                        lon_inH5(:,l), lat_inH5(:,l),   &
                        field_Avhrr % difazi(:,l), field_Avhrr % sunzen(:,l), field_Avhrr % satzen(:,l), &
                        field_Avhrr % lon(:,l), field_Avhrr % lat(:,l) )

     iq(1) = scnlinyr(l)
     iq(2) = scnlindy(l)
     iq(3) = scnlintime(l)
     call xiqj(iq, field_Avhrr % ij(l) )

     !scan line quality  and calibration quality flags
     if( scnlinqual(l) /=0)       field_Avhrr % scanqual(l) = 1
     if( any(calqual(:,l) /=0) )  field_Avhrr % scanqual(l) = 1
     if( all(Obs_inH5(3,:,l)<=0)) field_Avhrr % scanqual(l) = 1
     if (any(calqual(:,l) /=0) ) write(*,*) 'CALQUAL IR: ', l, calqual(:,l)

     do p=1,2048
        if( field_Avhrr % sunzen(p,l)<90 .and. Obs_inH5(1,p,l) < 0 ) field_Avhrr % Qual_Obs(1,p,l)=1
        if( field_Avhrr % sunzen(p,l)<90 .and. Obs_inH5(2,p,l) < 0 ) field_Avhrr % Qual_Obs(2,p,l)=1
        if(Obs_inH5(1,p,l) > 100.00 ) field_Avhrr % Qual_Obs(1,p,l)=1
        if(Obs_inH5(2,p,l) > 100.00 ) field_Avhrr % Qual_Obs(2,p,l)=1

        if( field_Avhrr % Qual_Obs(1,p,l)==0 )  field_Avhrr % Obs(1,p,l) = Obs_inH5(1,p,l)      
        if( field_Avhrr % Qual_Obs(2,p,l)==0 )  field_Avhrr % Obs(2,p,l) = Obs_inH5(2,p,l)      

        if(ch3b(l) ==1) then !3B
            field_Avhrr % Qual_Obs(3,p,l)=1
            if(Obs_inH5(3,p,l)<=50.00 .or. Obs_inH5(3,p,l)>350.00) field_Avhrr % Qual_Obs(4,p,l)=1
            if(field_Avhrr % Qual_Obs(4,p,l)==0 )  field_Avhrr % Obs(4,p,l) = Obs_inH5(3,p,l)   
        else                 !3A
            field_Avhrr % Qual_Obs(4,p,l)=1
            ! pb au changement 3B-3A avec metop (valeurs faiblement negatives)
            if(Obs_inH5(3,p,l) < 0. .and. Obs_inH5(3,p,l) > -1.00) field_Avhrr % Qual_Obs(3,p,l)=1
            if(Obs_inH5(3,p,l) < 0. .or.  Obs_inH5(3,p,l) >100.00) field_Avhrr % Qual_Obs(3,p,l)=1
            if(field_Avhrr % Qual_Obs(3,p,l)==0 )  field_Avhrr % Obs(3,p,l) = Obs_inH5(3,p,l)
                        
        endif
        if(Obs_inH5(4,p,l) <=50.00 .or.  Obs_inH5(4,p,l) >350.00) field_Avhrr % Qual_Obs(5,p,l)=1
        if(Obs_inH5(5,p,l) <=50.00 .or.  Obs_inH5(5,p,l) >350.00) field_Avhrr % Qual_Obs(6,p,l)=1
        if( field_Avhrr % Qual_Obs(5,p,l)==0 )  field_Avhrr % Obs(5,p,l) = Obs_inH5(4,p,l)      
        if( field_Avhrr % Qual_Obs(6,p,l)==0 )  field_Avhrr % Obs(6,p,l) = Obs_inH5(5,p,l)             
     end do     ! p=1,2048  
     
     if( ch3b(l) ==1 .and. calqual(1,l) /=0)   field_Avhrr % Qual_Obs(4,:,l)=1 
     if(                   calqual(2,l) /=0)   field_Avhrr % Qual_Obs(5,:,l)=1 
     if(                   calqual(3,l) /=0)   field_Avhrr % Qual_Obs(6,:,l)=1 
  end do        ! l=1,nbline
     

  ! fermeture du fichier
  !-------------------------    
  call H5Fclose_f (file_id,ios)

  deallocate (lat_inH5, lon_inH5, satzen_inH5, sunzen_inH5, relaz_inH5, Obs_inH5 )
  deallocate (ch3b, calqual, scnlinqual, scalti)
  deallocate (scnlin, scnlinyr, scnlindy, scnlintime )

end subroutine maia_read_Avh1c_hdf5

!=====================================================================================
SUBROUTINE locl1b_2full(sat_alt,                                                &
                        difazi_l1b, solzen_l1b, satzen_l1b, lon_l1b, lat_l1b,   &
                        difazi_full, solzen_full, satzen_full, lon_full, lat_full )
!=====================================================================================
      
! owner         : MF/DP/CMS/R&D
! Authors       : Philippe Marguinaud. Lydie Lavanant
! date          : 02/03/06

! PURPOSE 
! interpolation/extrapolation of geometry :
! solar zenith angle, satellite zenith angle, relative azimuth angle 
! latitude, longitude
! from AAPP level1b format resolution:
!           point 25 to point 2025 every 40 points 
! to each 2048 pixels of the line
! no interpolation/extrapolation of difazi in case of 
!           satellite or sun azimuth abrupt shift from -179 to +179 

  IMPLICIT NONE
      
  REAL(KIND=8), PARAMETER :: PI= 3.14159265358979323846
  REAL(KIND=8), PARAMETER :: cdr  = PI/180.0
  REAL(KIND=8), PARAMETER :: RT=6366                                    ! earth radius
  
  REAL, INTENT(in) :: sat_alt                                           ! satellite altitude KMS
  REAL, INTENT(in) :: difazi_l1b(51), solzen_l1b(51), satzen_l1b(51)    ! degrees
  REAL, INTENT(in) :: lon_l1b(51), lat_l1b(51)                          ! degrees
  REAL, INTENT(out):: difazi_full(2048), solzen_full(2048)              ! degrees
  REAL, INTENT(out):: satzen_full(2048)                                 ! degrees
  REAL, INTENT(out):: lon_full(2048), lat_full(2048)                    ! degrees

  REAL(KIND=8) :: H                                                     ! satellite altitude KMS
  REAL(KIND=8) :: AZIDI(51), ZENSSI(51)                                 ! degrees
  REAL(KIND=8) :: LONI(51), LATI(51)                                    ! degrees
  REAL(KIND=8) :: AZIDO(2048), ZENSSO(2048)                             ! degrees
  REAL(KIND=8) :: LONO(2048), LATO(2048)                                ! degrees
     
  REAL(KIND=8) :: ALPHA(2048), GAMMA(2048)                              ! radians
  REAL(KIND=8) :: ZENSI(51), ZENSO(2048)                                ! radians
  REAL(KIND=8) :: X(2048), Y(2048), Z(2048) 
  INTEGER      :: I, J
  REAL(KIND=8) :: A1, A3
  INTEGER      :: J1, J2, J3
  REAL(KIND=8) :: C21, C31, C32
  REAL(KIND=8) :: R
  REAL(KIND=8) :: T
  
  H= sat_alt
  AZIDI(:) =difazi_l1b(:)
  ZENSSI(:)=solzen_l1b(:)
  ZENSI(:) =satzen_l1b(:)*cdr
  LONI(:)  =lon_l1b(:)
  LATI(:)  =lat_l1b(:)
  
  AZIDO(:) =0.
  ZENSSO(:)=0.
  ZENSO(:) =0.
  LONO(:)  =0.
  LATO(:)  =0.
  ALPHA(:) =0.
  GAMMA(:) =0.
  
  T = RT / ( RT + H )  
  
  DO I = 0, 50
     ALPHA( 25+40*I ) = ASIN( T * SIN( ZENSI(1+I) ) )           ! radians
     GAMMA( 25+40*I ) = ZENSI(1+I) - ALPHA(25+40*I)             ! radians
     
     CALL LL_TO_XYZ(LONI(1+I), LATI(1+I), X(25+40*I), Y(25+40*I), Z(25+40*I))
     LONO( 25+40*I )  = LONI(1+I)                               ! degrees
     LATO( 25+40*I )  = LATI(1+I)                               ! degrees
     ZENSSO( 25+40*I )= ZENSSI(1+I)                             ! degrees
     AZIDO( 25+40*I ) = AZIDI(1+I)                              ! degrees
  ENDDO
  ! EXTRAPOLATE ALPHA in radians
  DO I = 1, 24
     R = ( I - 25. ) / 40.
     ALPHA( I ) = ( 1. - R )*ALPHA(25) + R*ALPHA(65)    
  ENDDO
     
  ! INTERPOLATE ALPHA  in radians
  DO I = 0, 49
     DO J = 1, 39
        R = J / 40.
        ALPHA(25+40*I+J) = (1.- R)*ALPHA(25+40*I) + R*ALPHA(25+40*I+40)
     ENDDO
  ENDDO
      
  ! EXTRAPOLATE ALPHA  in radians
  DO I = 2026, 2048
     R = ( I - 1985. ) / 40.
     ALPHA(I) = ( 1. - R )*ALPHA(1985) + R*ALPHA(2025)
  ENDDO

  ! COMPUTE GAMMA & ZENSO in radians
  DO I = 1, 2048
     ZENSO(I) = ASIN( SIN( ALPHA(I) ) / T )
     GAMMA(I) = ZENSO(I) - ALPHA(I)
  ENDDO

  ! EXTRAPOLATE 
  J1 = 25
  J3 = J1 + 40
  C31 = COS( GAMMA(J3) - GAMMA(J1) )
  DO J = 1, 24
     J2 = J
     C21 = COS( GAMMA(J2) - GAMMA(J1) )
     C32 = COS( GAMMA(J3) - GAMMA(J2) )
     A1 = C21 - C31 * C32
     A3 = C32 - C31 * C21
     
     X(J2) = A1 * X(J1) + A3 * X(J3)
     Y(J2) = A1 * Y(J1) + A3 * Y(J3)
     Z(J2) = A1 * Z(J1) + A3 * Z(J3)
   
     CALL XYZ_TO_LL( X(J2), Y(J2), Z(J2), LONO(J2), LATO(J2) )
  ENDDO

  ! INTERPOLATE
  DO I = 0, 49
     J1 = 25+40*I
     J3 = J1 + 40
     C31 = COS( GAMMA(J3) - GAMMA(J1) )
     DO J = 1, 40
        J2 = J1 + J
        C21 = COS( GAMMA(J2) - GAMMA(J1) )
        C32 = COS( GAMMA(J3) - GAMMA(J2) )
        A1 = C21 - C31 * C32
        A3 = C32 - C31 * C21
        
        X(J2) = A1 * X(J1) + A3 * X(J3)
        Y(J2) = A1 * Y(J1) + A3 * Y(J3)
        Z(J2) = A1 * Z(J1) + A3 * Z(J3)
   
        CALL XYZ_TO_LL( X(J2), Y(J2), Z(J2), LONO(J2), LATO(J2) )
     ENDDO
  ENDDO  

  ! EXTRAPOLATE
  J1 = 2025
  J3 = J1 - 40
  C31 = COS( GAMMA(J3) - GAMMA(J1) )
  DO J = 2026, 2048
     J2 = J
     C21 = COS( GAMMA(J2 ) - GAMMA(J1) )
     C32 = COS( GAMMA(J3 ) - GAMMA(J2) )
     A1 = C21 - C31 * C32
     A3 = C32 - C31 * C21
     
     X(J2) = A1 * X(J1) + A3 * X(J3)
     Y(J2) = A1 * Y(J1) + A3 * Y(J3)
     Z(J2) = A1 * Z(J1) + A3 * Z(J3)
   
     CALL XYZ_TO_LL( X(J2), Y(J2), Z(J2), LONO(J2), LATO(J2) )
  ENDDO

  ! EXTRAPOLATE
  DO I = 1, 24
     R =  ( GAMMA(I) - GAMMA(25) ) / ( GAMMA(65) - GAMMA(25) )
     ZENSSO( I ) = ( 1. - R )*ZENSSO(25) + R*ZENSSO(65)
     AZIDO( I ) =  ( 1. - R )*AZIDO(25)  + R*AZIDO(65)
     ! no AZIDO interpolation/extrapolation 
     ! when satazi or sunazi abrupt shift from -179 to +179 
     if(ABS( AZIDO(25)- AZIDO(65)) > 178.) AZIDO( I )=AZIDO(25)
  ENDDO
  
  ! INTERPOLATE
  DO I = 0, 49
     DO J = 1, 40
        R =  ( GAMMA( 25+40*I+J )  - GAMMA( 25+40*I ) ) &
           / ( GAMMA( 25+40*I+40 ) - GAMMA( 25+40*I ) )
        ZENSSO( 25+40*I+J ) = ( 1. - R ) * ZENSSO( 25+40*I ) &
                              + R * ZENSSO( 25+40*I+40 )
        AZIDO( 25+40*I+J ) = ( 1. - R ) * AZIDO( 25+40*I ) &
                              + R * AZIDO( 25+40*I+40 )
     ENDDO
     ! no AZIDO interpolation/extrapolation 
     ! when satazi or sunazi abrupt shift from -179 to +179 
     if(ABS( AZIDO(25+40*I)- AZIDO(25+40*I+40)) > 178.) then
       AZIDO( 25+40*I+1 : 25+40*I+39 ) =AZIDO(25+40*I)
     endif
  ENDDO
      
  ! EXTRAPOLATE 
  DO I = 2026, 2048
     R = ( GAMMA(I) - GAMMA(1985) ) / ( GAMMA(2025) - GAMMA(1985) )
     ZENSSO(I) = ( 1. - R )*ZENSSO(1985)+ R*ZENSSO(2025)
     AZIDO(I)  = ( 1. - R )*AZIDO(1985) + R*AZIDO(2025)
     ! no AZIDO interpolation/extrapolation 
     ! when satazi or sunazi abrupt shift from -179 to +179 
     if(ABS( AZIDO(1985)- AZIDO(2025)) > 178.) AZIDO( I )=AZIDO(2025)
  ENDDO

  difazi_full(:)=AZIDO(:)
  solzen_full(:)=ZENSSO(:)
  satzen_full(:)=ZENSO(:)/cdr
  lon_full(:)   =LONO(:)
  lat_full(:)   =LATO(:)

END SUBROUTINE locl1b_2full

!=====================================================================================
SUBROUTINE XYZ_TO_LL( X, Y, Z, PLON, PLAT )
!=====================================================================================

! owner         : MF/DP/CMS/R&D
! Authors       : Philippe Marguinaud
! date          : 04/07/05

! PURPOSE 
! converts cartesian coordinates to geographical coordinates

  IMPLICIT NONE
      
  REAL(KIND=8), PARAMETER :: PI= 3.14159265358979323846
  REAL(KIND=8), PARAMETER :: EPSILON1= 1E-15
  REAL(KIND=8), PARAMETER :: EPSILON2= 1E-8

  REAL(KIND=8) :: X, Y, Z, PLON, PLAT
  REAL(KIND=8) :: R
  REAL(KIND=8) :: XOCL, YOCL
  REAL(KIND=8) :: LON, LAT
      
  R = DSQRT( X * X + Y * Y + Z * Z )
  X = X / R
  Y = Y / R
  Z = Z / R
      
  IF( Z > ( + 1. - EPSILON1 ) ) THEN
      LON = 0.
      LAT = +90.
  ELSE IF( Z < ( -1. + EPSILON1 ) ) THEN
      LON = 0.;
      LAT = -90.;
  ELSE 
      LAT = ASIN( Z ) ! RADIANS 
      XOCL = X / COS( LAT );
      IF( DABS( XOCL ) > ( +1 - EPSILON2 ) ) THEN
          YOCL = Y / COS( LAT )
          ! RADIANS 
          LON = ASIN( YOCL )
          IF( X > 0 ) THEN
            LON = LON
          ELSE
            LON = - LON + PI
          ENDIF
     ELSE
          ! RADIANS 
          LON = ACOS( XOCL )
          IF( Y > 0 ) THEN
              LON = LON
          ELSE
              LON = - LON
          ENDIF
     ENDIF
     LAT = 180. * LAT / PI ! DEGREES 
     LON = 180. * LON / PI ! DEGREES 
     DO WHILE( LON > 180 ) 
        LON = LON - 360
     ENDDO
  ENDIF

  PLON = LON
  PLAT = LAT
      
END SUBROUTINE

!=====================================================================================
SUBROUTINE LL_TO_XYZ( LON, LAT, PX, PY, PZ )
!=====================================================================================

! owner         : MF/DP/CMS/R&D
! Authors       : Philippe Marguinaud
! date          : 04/07/05

! PURPOSE 
! converts geographical coordinates to cartesian coordinates

  IMPLICIT NONE
          
  REAL(KIND=8), PARAMETER :: PI= 3.14159265358979323846
  REAL(KIND=8) :: LON, LAT, PX, PY, PZ
      
  PX = COS( PI * LON / 180. ) * COS( PI * LAT / 180. )
  PY = SIN( PI * LON / 180. ) * COS( PI * LAT / 180. )
  PZ = SIN( PI * LAT / 180. )
      
END SUBROUTINE

        
subroutine Read_attribute_INT (name, id, buf)

 USE HDF5
 IMPLICIT NONE

  character(len=80), INTENT(IN) :: name  
  INTEGER(HID_T), INTENT(IN)    :: id           ! file identifier
  INTEGER , INTENT(OUT)         :: buf 

  INTEGER(HID_T)        :: attr_id      ! attribute identifier
  INTEGER(HSIZE_T)      :: dims(1)      
  INTEGER               :: status       

  dims(1)=1

  call H5Aopen_f (id, name, attr_id ,status)
  if (status /= 0) then
     call ml_wt('ERROR','Cannot open attribute ',name)
     stop 2
  endif

  call H5Aread_f (attr_id, H5T_NATIVE_INTEGER, buf, dims, status)
  if (status /= 0) then
     call ml_wt('ERROR','Cannot read attribute ',name)
     stop 2
  endif

  call H5Aclose_f (attr_id, status)
end subroutine Read_attribute_INT

subroutine Read_attribute_REAL (name, id, nval, buf)

 USE HDF5
 IMPLICIT NONE

  character(len=80), INTENT(IN) :: name  
  INTEGER(HID_T), INTENT(IN)    :: id           ! file identifier
  INTEGER, INTENT(IN)   :: nval
  REAL , INTENT(OUT)         :: buf(nval) 

  INTEGER(HID_T)        :: attr_id      ! attribute identifier
  INTEGER(HSIZE_T)      :: dims(1)      
  INTEGER               :: status       

  dims(1)=nval

  call H5Aopen_f (id, name, attr_id ,status)
  if (status /= 0) then
     call ml_wt('ERROR','Cannot open attribute ',name)
     stop 2
  endif

  call H5Aread_f (attr_id, H5T_NATIVE_REAL, buf, dims, status)
  if (status /= 0) then
     call ml_wt('ERROR','Cannot read attribute ',name)
     stop 2
  endif

  call H5Aclose_f (attr_id, status)
end subroutine Read_attribute_REAL
subroutine Read_attribute_CHAR (name, id, nbcar, buf)

 USE HDF5
 IMPLICIT NONE

  character(len=80), INTENT(IN)  :: name  
  INTEGER(HID_T), INTENT(IN)     :: id          ! file identifier
  integer, intent(in)            :: nbcar
  character(len=90), INTENT(OUT) :: buf 

  INTEGER(HID_T)        :: strtype              ! type ID string
  INTEGER(HID_T)        :: attr_id      ! attribute identifier
  INTEGER(HSIZE_T)      :: dims(1)      
  INTEGER               :: status       
  INTEGER(SIZE_T) :: dim68

  dims(1)=1
  dim68=nbcar

  ! on cree un format specifique pour un mot de plusieurs characters
  call H5Tcopy_f ( H5T_NATIVE_CHARACTER, strtype, status )
  call H5Tset_size_f (strtype, dim68, status)

  call H5Aopen_f (id, name, attr_id ,status)
  if (status /= 0) then
     call ml_wt('ERROR','Cannot open attribute ',name)
     stop 2
  endif

  call H5Aread_f (attr_id, strtype, buf, dims, status)
  if (status /= 0) then
     call ml_wt('ERROR','Cannot read attribute ',name)
     stop 2
  endif

  call H5Aclose_f (attr_id, status)
end subroutine Read_attribute_CHAR
#endif
        
