! Name: $Id: mod_maia_att.F90 530 2017-06-27 11:07:32Z frna $
!
! Purpose: maia attributes definition
!          Subroutines 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    23/01/2017   Pascale Roquet creation

module mod_maia_att
public :: fill_attribute_Maskind
public :: fill_attribute_CloudMask
public :: fill_attribute_CloudType
public :: fill_attribute_Tsurf
public :: fill_attribute_fov_qual_viirs
public :: fill_attribute_fov_qual_avhrr


contains

 subroutine fill_attribute_Maskind(att)  
  IMPLICIT NONE
  character(len=2560), intent(out) :: att
  att=""
  att=TRIM(att)//char(10)//"bit   0    snow/ice from external data  0 Not Done"
  att=TRIM(att)//char(10)//"bit   1    Cloud mask on pixel          0 Not Done"
  att=TRIM(att)//char(10)//"bit   2    Thin Cirrus detection        0 Not Done"
  att=TRIM(att)//char(10)//"bit   3    cloud type                   0 Not done"
  att=TRIM(att)//char(10)//"bit   4    cloud phase                  0 Not Done"
  att=TRIM(att)//char(10)//"bit   5    Cloud Height                 0 Not Done"
  att=TRIM(att)//char(10)//"bit   6    Active Fire detection        0 Not Done"
  att=TRIM(att)//char(10)//"bit   7    Aerosols                     0 Not Done"
  att=TRIM(att)//char(10)//"bit   8    Cloud Shadow from Neighbors  0 Not Done"
  att=TRIM(att)//char(10)//"bit   9    CMa update from Neighbors    0 Not Done"
  att=TRIM(att)//char(10)//"bit  10    Fire detection texture       0 Not Done"
  att=TRIM(att)//char(10)//"bit  11    Aerosols texture             0 Not Done"
  att=TRIM(att)//char(10)//"bits 12-31 Spares"
 end subroutine fill_attribute_Maskind
  

 subroutine fill_attribute_CloudMask(att)  
  IMPLICIT NONE
  character(len=2560), intent(out) :: att
  att=""
  att=TRIM(att)//char(10)//"bits 0-2    Cloud Mask on pixels       0= clear"
  att=TRIM(att)//char(10)//"            (before Cloud Adjacency)   1= cloudy"
  att=TRIM(att)//char(10)//"                                       2= clear over snow"
  att=TRIM(att)//char(10)//"                                       3= clear over ice"
  att=TRIM(att)//char(10)//"                                       4= aerosol_dust_ash_fire"
  att=TRIM(att)//char(10)//"bits 3-4    Cloud Mask Quality         0=high"
  att=TRIM(att)//char(10)//"            (from nb tests involved)   1=medium"
  att=TRIM(att)//char(10)//"                                       2=low"
  att=TRIM(att)//char(10)//"                                       3=bad"
  att=TRIM(att)//char(10)//"bits 5-6   Cloud Mask Confidence       0=Cloudy"
  att=TRIM(att)//char(10)//"           (from the proximity         1=ProbablyCloudy"
  att=TRIM(att)//char(10)//"           of thresholds)              2=ProbablyClear"
  att=TRIM(att)//char(10)//"                                       3=Clear"
  att=TRIM(att)//char(10)//"bits 7-9   Adjacent Pixel Cloud Mask   0=Not tested"
  att=TRIM(att)//char(10)//"           (from cloud confidence      1=Clear"
  att=TRIM(att)//char(10)//"           evaluation of surrounding   2=ProbablyClear"
  att=TRIM(att)//char(10)//"           pixels)                     3=ProbablyCloudy"
  att=TRIM(att)//char(10)//"                                       4=Cloudy"
  att=TRIM(att)//char(10)//"bits 10-12 Cloud Mask total            0=clear"
  att=TRIM(att)//char(10)//"           (after Cloud Adjacency      1=cloudy"
  att=TRIM(att)//char(10)//"           process. If bits7-9==0      2=clear over snow"
  att=TRIM(att)//char(10)//"           same as bits0-2)            3=clear over ice"
  att=TRIM(att)//char(10)//"                                       4=aerosol_dust_ash_fire"
  att=TRIM(att)//char(10)//"bits 13-14 Day/Night/Twilight/Sunglint 0=Night"
  att=TRIM(att)//char(10)//"                                       1=Twilight"
  att=TRIM(att)//char(10)//"                                       2=Day"
  att=TRIM(att)//char(10)//"                                       3=Sunglint"
  att=TRIM(att)//char(10)//"bits 15-17 Land/Water Background       0=Sea+inland Water"
  att=TRIM(att)//char(10)//"                                       1=mixed"
  att=TRIM(att)//char(10)//"                                       2=land"
  att=TRIM(att)//char(10)//"                                       3=desert"
  att=TRIM(att)//char(10)//"                                       4=snow"
  att=TRIM(att)//char(10)//"bits 18 Land surface type              0=no coast"
  att=TRIM(att)//char(10)//"           (from landsea atlas)        1=coast"
  att=TRIM(att)//char(10)//"bits 19-20 Snow/Ice Surface            0=not tested"
  att=TRIM(att)//char(10)//"           (from ancillary data        1=No"
  att=TRIM(att)//char(10)//"            or forecast)               2=Yes"
  att=TRIM(att)//char(10)//"bits 21    Opaque Cloud                0=No"
  att=TRIM(att)//char(10)//"                                       1=Yes"
  att=TRIM(att)//char(10)//"bits 22-25 Shadow/Aerosol/Fire/Dust    0=Not tested"
  att=TRIM(att)//char(10)//"                                       1=No"
  att=TRIM(att)//char(10)//"                                       2=CloudShadow"
  att=TRIM(att)//char(10)//"                                       3=Heavy Aerosol"
  att=TRIM(att)//char(10)//"                                       4=Fire"
  att=TRIM(att)//char(10)//"                                       5=Dust/Volcanic Ash"
  att=TRIM(att)//char(10)//"bits 26    Tskin bg source             0=climatology used"
  att=TRIM(att)//char(10)//"                                       1=forecast used"
  att=TRIM(att)//char(10)//"bits 27    WV content bg source        0=climatology used"
  att=TRIM(att)//char(10)//"                                       1=forecast used"
  att=TRIM(att)//char(10)//"bits 28-31 Spares"
 end subroutine  fill_attribute_CloudMask

 subroutine fill_attribute_CloudType(att)  
  IMPLICIT NONE
  character(len=2560), intent(out) :: att
  att=""
  att=TRIM(att)//char(10)//"bits 0-3 cloud phase:"
  att=TRIM(att)//char(10)//"          0  Not Done"
  att=TRIM(att)//char(10)//"          1  Clear: from Confidently Clear pixels"
  att=TRIM(att)//char(10)//"          2  Partly Cloudy: from Probably Clear and Cloudy pixels"
  att=TRIM(att)//char(10)//"          3  Water Cloud"
  att=TRIM(att)//char(10)//"          4  Supercooled Water or MixedPhaseCloud"
  att=TRIM(att)//char(10)//"          5  Opaque Ice Cloud"
  att=TRIM(att)//char(10)//"          6  Cirrus Non-Opaque Cloud"
  att=TRIM(att)//char(10)//"          7  Cloud Overlap"
  att=TRIM(att)//char(10)//"bits 4-8 cloud type:"
  att=TRIM(att)//char(10)//"          0  no data or corrupted data"
  att=TRIM(att)//char(10)//"          1  cloud free land"
  att=TRIM(att)//char(10)//"          2  cloud free sea"
  att=TRIM(att)//char(10)//"          3  land contaminated by snow"
  att=TRIM(att)//char(10)//"          4  sea contaminated by snow/ice"
  att=TRIM(att)//char(10)//"          6  very low clouds"
  att=TRIM(att)//char(10)//"          8  low clouds"
  att=TRIM(att)//char(10)//"         10  medium clouds"
  att=TRIM(att)//char(10)//"         12  high opaque clouds"
  att=TRIM(att)//char(10)//"         14  very high opaque clouds"
  att=TRIM(att)//char(10)//"         15  thin semitransparent clouds"
  att=TRIM(att)//char(10)//"         16  meanly thick semitransparent clouds"
  att=TRIM(att)//char(10)//"         17  thick semitransparent clouds"
  att=TRIM(att)//char(10)//"         18  semitransparent above lower clouds"
  att=TRIM(att)//char(10)//"         19  sub-pixel water clouds"
  att=TRIM(att)//char(10)//"         20  undefined by CMa"
  att=TRIM(att)//char(10)//"bits 9-10 thin cirrus:"
  att=TRIM(att)//char(10)//"          0  Not Done"
  att=TRIM(att)//char(10)//"          1  Yes"
  att=TRIM(att)//char(10)//"          2  No"
  att=TRIM(att)//char(10)//"bits 11-15   spares"
 end subroutine fill_attribute_CloudType
 
 subroutine fill_attribute_Tsurf(att)  
  IMPLICIT NONE
  character(len=2560), intent(out) :: att
  att=""
  att=TRIM(att)//char(10)//"for confident clear: retrieved surface temperature in C*100:"
  att=TRIM(att)//char(10)//"       over sea : SST Split-Window"
  att=TRIM(att)//char(10)//"       over land: LST from RTE using forecast and emissivity atlas"
  att=TRIM(att)//char(10)//"          -9999.: missing values"
 end subroutine fill_attribute_Tsurf
 
 subroutine fill_attribute_fov_qual_viirs(att)  
  IMPLICIT NONE
  character(len=2560), intent(out) :: att
  att=""
  att=TRIM(att)//char(10)//"0-1  input info quality 0= ok   mask done with highest information quality"
  att=TRIM(att)//char(10)//"      for cloud mask    1= poor some channels are discarded"
  att=TRIM(att)//char(10)//"                        2= bad  cloud mask not done"
  att=TRIM(att)//char(10)//"2   geoloc quality      0= ok 1= bad summarized geolocation quality"
  att=TRIM(att)//char(10)//"3   IBAND SDR quality   0= ok 1= bad summarized qual flag of used I-Band channels"
  att=TRIM(att)//char(10)//"4   MBAND SDR quality   0= ok 1= bad summarized qual flag of used M-Band channels"
  att=TRIM(att)//char(10)//"5   Qual_M(1)           0= ok 1= bad"
  att=TRIM(att)//char(10)//"6   Qual_M(2)           0= ok 1= bad"
  att=TRIM(att)//char(10)//"7   Qual_M(3)           0= ok 1= bad"
  att=TRIM(att)//char(10)//"8   Qual_M(4)           0= ok 1= bad"
  att=TRIM(att)//char(10)//"9   Qual_M(5)           0= ok 1= bad"
  att=TRIM(att)//char(10)//"10  Qual_M(6)           0= ok 1= bad"
  att=TRIM(att)//char(10)//"11  Qual_M(7)           0= ok 1= bad"
  att=TRIM(att)//char(10)//"12  Qual_M(8)           0= ok 1= bad"
  att=TRIM(att)//char(10)//"13  Qual_M(9)           0= ok 1= bad"
  att=TRIM(att)//char(10)//"14  Qual_M(10)          0= ok 1= bad"
  att=TRIM(att)//char(10)//"15  Qual_M(11)          0= ok 1= bad"
  att=TRIM(att)//char(10)//"16  Qual_M(12)          0= ok 1= bad"
  att=TRIM(att)//char(10)//"17  Qual_M(13)          0= ok 1= bad"
  att=TRIM(att)//char(10)//"18  Qual_M(14)          0= ok 1= bad"
  att=TRIM(att)//char(10)//"19  Qual_M(15)          0= ok 1= bad"
  att=TRIM(att)//char(10)//"20  Qual_M(16)          0= ok 1= bad"
  att=TRIM(att)//char(10)//"21  Qual_I(1)           0= ok 1= bad"
  att=TRIM(att)//char(10)//"22  Qual_I(2)           0= ok 1= bad"
  att=TRIM(att)//char(10)//"23  Qual_I(3)           0= ok 1= bad"
  att=TRIM(att)//char(10)//"24  Qual_I(4)           0= ok 1= bad"
  att=TRIM(att)//char(10)//"25  Qual_I(5)           0= ok 1= bad"
  att=TRIM(att)//char(10)//"26-31   spares"
 end subroutine fill_attribute_fov_qual_viirs

 subroutine fill_attribute_fov_qual_avhrr(att)   
  IMPLICIT NONE
  character(len=2560), intent(out) :: att
  att=""
  att=TRIM(att)//char(10)//"0-1  input info quality 0= ok   mask done with highest information quality"
  att=TRIM(att)//char(10)//"      for cloud mask    1= poor some channels are discarded"
  att=TRIM(att)//char(10)//"                        2= bad  cloud mask not done"
  att=TRIM(att)//char(10)//"2   geoloc quality      0= ok 1= bad summarized geolocation quality"
  att=TRIM(att)//char(10)//"3   Qual AVHRR          0= ok 1= bad summarized qual flag of used AVHRR channels"
  att=TRIM(att)//char(10)//"4   Qual AVHRR chan 1   0= ok 1= bad"
  att=TRIM(att)//char(10)//"5   Qual AVHRR chan 2   0= ok 1= bad"
  att=TRIM(att)//char(10)//"6   Qual AVHRR chan 3A  0= ok 1= bad"
  att=TRIM(att)//char(10)//"7   Qual AVHRR chan 3b  0= ok 1= bad"
  att=TRIM(att)//char(10)//"8   Qual AVHRR chan 4   0= ok 1= bad"
  att=TRIM(att)//char(10)//"9-31   spares"
 end subroutine fill_attribute_fov_qual_avhrr 
end module mod_maia_att
