C LIBS = ephtrack navtool satid f7ml f7tp f7cp f7gp auxaapp
!+
!NOM/NAME $Id: driftephe.F 47 2006-08-21 08:44:41Z frna $
!
        program driftephe
!
!BUT/PURPOSE 
!       DRIFTEPHE drifts the time of start of acquisition for a number of 
!       seconds
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!       Inputs are:
!       standard input
!               ephemeris file
!
!       Output
!       standard output
!               updated ephemeris file
!       
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
! 1.1   01 April 2004   change default integer format (i) on (i3)
!             for better F90 compatibility  A. Marsouin P.Brunel
! 1.2   09 May 2005     correction of bug introduced by previous change
!                         i3 does not work for negative values lower
!                         than -99          P. Brunel
!     
!EXIT CODES    
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!DECLARATIONS
!
! Local parameters:
!
! variables in input

!
! Commons
!
! Functions
!
        integer iargc
! 
! Local variables:
        integer dtsec                   ! time offset (seconds)
        integer gtime(7)                ! gregorian time
        double precision jtime          ! julian time
        double precision dt             ! time offset (julian time)
        character*82 line_ephe          ! ephemeris line message
        integer nbarg                   ! number of arguments
        character*80 chdt               ! offset time string

        character*16 event              ! event message 
        integer lg                      ! lenght of argument one
        character*32 readfmt            ! fortran read format for dt

        integer ios                     ! I/O status variable
!
! Functions
        integer cp_lsc
!
!
!
!
! 
!-
        call ml_wt('info','start')
!
!       Section 1
!       Reads the input parameters
!       --------------------------
!
!
        nbarg = iargc()
        if(nbarg.le.0) then
          call ml_wt('error','missing dt argument')
          call aappexit(1)
        endif
        call getarg(1,chdt)

        ! last significant character of string chdt
        lg = cp_lsc(chdt)
        if( lg .gt. 9 ) then
           dtsec=0.
           call ml_wt('warning','invalid offset for start')
           call ml_wti('warning','new dt offset (sec) ',dtsec)
        else
           write(readfmt,"('(i',i1,')')") lg
           read(chdt,readfmt) dtsec
           call ml_wti('info','dt offset (sec) ',dtsec)
        end if

        if(abs(dtsec).gt.300) then
          if(dtsec.lt.0.) then
            dtsec = -300.
          else
            dtsec = 300.
          endif
          call ml_wt('warning',
     &               'invalid offset for start of acquisition')
          call ml_wti('warning','new dt offset (sec) ',dtsec)
        endif
        dt = dtsec / 86400.d0
!
!       Section 2
!       Loop over the ephemeris file
!       -----------------------------
        read(*,'(a)',iostat=ios) line_ephe
        !event=line_ephe(40:55)
        event=line_ephe(50:65)

        do while (ios.eq.0) 

! start of acquisition
          if(event(1:9) .eq. 'start_acq' .or.
     &       event(1:14) .eq. 'start_conflict' ) then
            read(line_ephe,
     &      '(i4,1x,i2,1x,i2,2x,i2,1x,i2,1x,i2,1x,i3)') gtime
            call xigj(gtime,jtime)
            jtime = jtime + dt
            call xijg(jtime,gtime)
            write(line_ephe(1:24),
     &      "(i4,'/',i2.2,'/',i2.2,'  ',
     &        i2.2,':',i2.2,':',i2.2,'.',i3.3)") gtime
             write(*,'(a)') line_ephe
            
          else
! if event is not start_acq or stop_acq send it to standard output
             write(*,'(a)') line_ephe

          endif

        read(*,'(a)',iostat=ios) line_ephe
        !event=line_ephe(40:55)
        event=line_ephe(50:65)

        end do
!
!
        call ml_wt('info','normal end')
!
        stop

        end
