C LIBS = brolyd ephtrack navtool satid f7ml f7tp f7cp f7gp
!NOM/NAME $Id: extbrolyd.F 25 2006-02-03 15:43:53Z frna $
!
        program extbrolyd
!
!BUT/PURPOSE 
!       extrapolation of orbital elements using the brolyd model
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       beware to the order of the input elements (brolyd order)
!       1: semi major axis
!       2: eccentricity
!       3: inclination
!       4: right ascension
!       5: perigee argument
!       6: mean anomaly
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!     
! EXIT CODES    
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!DECLARATIONS
!
!
!
! Local parameters
        double precision pi
        parameter ( pi = 3.14159265358979311600d0)
        double precision crd
        parameter (crd=180.d0/pi)
!
! Commons
!       Brolyd common
      double precision tto,r,ae,gm,bj2,bj3,bj4,bj5,fltinv,xke,esq
      common /blcnst/ tto,r,ae,gm,bj2,bj3,bj4,bj5,fltinv,xke,esq
!
! Functions
!
! Local variables
        integer ielem                   ! element number
        double precision epoch          ! epoch time of the input elements 
        double precision extdate        ! date for the extrapolation 
        double precision elements(6)    ! input elements for brolyd
        double precision wkelem(6)      ! working elements for brolyd
        double precision oscgtod(6)     ! osculating elements (gama trueofdate)
        double precision auxelem(6)     ! auxiliary elements for brolyd
        double precision positod(3)     ! position in ITOD reference frame
        double precision velitod(3)     ! velocity in ITOD reference frame
        double precision posgrw(3)      ! position in Greenwich reference frame 
        double precision velgrw(3)      ! velocity in Greenwich reference frame
        character*22 epoch_st           ! epoch string
        character*22 extdate_st         ! extdate string
        integer ios
        integer i
! 
!-
!
        read(*,'(a)',iostat=ios) epoch_st
        call xicj(epoch_st,epoch)
        read(*,*,iostat=ios) elements
        do ielem=3,6
          elements(ielem) = elements(ielem) / crd
        end do

        do while (ios.eq.0)

!         intialise brolyd with the new elements
          do ielem=1,6
            wkelem(ielem) = elements(ielem)
          enddo
          tto = 0.d0
          call brolyd(oscgtod,wkelem,0,1,1,auxelem)
          write(*,*)
          write(*,*) 'epoch time of the input elements ',epoch_st
          write(*,1010)
          write(*,1000) elements(1),elements(2),(elements(i)*crd,i=3,6)

!         loop on the extrapolation dates   (until reading: 'end')
          read(*,'(a)',iostat=ios) extdate_st
          do while (ios.eq.0 .and. extdate_st(1:3).ne.'end')

            call xicj(extdate_st,extdate)

            tto = (extdate - epoch) * 86400.d0
            do ielem=1,6
              wkelem(ielem) = elements(ielem)
            enddo
            call brolyd(oscgtod,wkelem,2,2,1,auxelem)
            call celem(oscgtod, gm, positod, velitod)
            call pvitodgrw(extdate,positod,velitod,posgrw,velgrw)

            write(*,*)
            write(*,*) 'extrapolation of the elements for the date ',
     &                  extdate_st
            write(*,1100) oscgtod(1),oscgtod(2),(oscgtod(i)*crd,i=3,6)
            write(*,1200) positod,velitod
            write(*,1300) posgrw,velgrw

            read(*,'(a)',iostat=ios) extdate_st

          end do

          read(*,'(a)',iostat=ios) epoch_st
          call xicj(epoch_st,epoch)
          read(*,*,iostat=ios) elements
          do ielem=3,6
            elements(ielem) = elements(ielem) / crd
          end do

        end do

1000    format('input elements true of date ',f12.4,f12.9,4f12.5)
1010    format(28x,'    s-m axis     eccent. inclination',
     &             '   right asc perigee arg  mean anom')
1100    format('osculating true of date     ',f12.4,f12.9,4f12.5)
1200    format('position vel. true of date  ',3f12.4,3f12.6)
1300    format('position velocity greenwich ',3f12.4,3f12.6)

        end
