#!/bin/ksh
#
# Get the file 'tai-utc.dat' from maia.usno.navy.mil
# The file contains the time difference between
#    TAI (Temps Atomique International) and UTC (sec)
#
# Get the file 'finals2000A.data' from maia.usno.navy.mil
# The file contains 
#   the time difference between UT1 and UTC (sec)
#   the Polar motion (arcsec)  
#
# This script should be run once a month
# The tai-utc.dat is only updated when a new leap second is introduced
#   last ones are Jan 1st 1999 and Jan 1st 2006
# The finals2000A.data is much larger (1Mbytes) and is updated 
#   weekly (Thursday). It contains IERS (I) or Prediction (P) data
#   so that file retrieval once a month could be enough.
#
# global script variables needed which can be set in the parameter 
# file ou ATOVS_ENV file depending on the environment:
# DIR_DATA_TAI_UT1_UTC root directory for stotage of the files 
#   default is DIR_NAVIGATION}/orb_elem
# PAR_NAVIGATION_TAI_UT1_UTC_TIMEOUT time out for connexion 
#   default is 60sec
# PAR_NAVIGATION_TAI_UTC_URL URL file to download for tai-utc.dat
#   default is http://maia.usno.navy.mil/ser7/tai-utc.dat
# PAR_NAVIGATION_UT1_UTC_URL URL file to download for finals2000A.data
#   default is http://maia.usno.navy.mil/ser7/finals2000A.data
#
#
# Please note that wget is able to manage the time out connexion
#
# PROPRIETAIRE/OWNER
#       P. Brunel
#
# HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
# (version   date      comment           <author>)
#  1.0     29/09/2005      none	       P. Brunel
#  1.1     20/02/2006  remove references to archipel P. Marguinaud
#
#    This software was developed within the context of
#    the EUMETSAT Satellite Application Facility on
#    Numerical Weather Prediction (NWP SAF), under the
#    Cooperation Agreement dated 25 November 1998, between
#    EUMETSAT and the Met Office, UK, by one or more partners
#    within the NWP SAF. The partners in the NWP SAF are
#    the Met Office, ECMWF, KNMI and MeteoFrance.
#
#    Copyright 2005, EUMETSAT, All Rights Reserved.
# 
#

. "$AAPP_PREFIX/ATOVS_CONF"

DIR_DATA_TAI_UT1_UTC=${DIR_DATA_TAI_UT1_UTC:=${DIR_NAVIGATION}/orb_elem}
if [ ! -d ${DIR_DATA_TAI_UT1_UTC} ]; then
  mkdir -p ${DIR_DATA_TAI_UT1_UTC} 
fi  
TIMEOUT=${PAR_NAVIGATION_TAI_UT1_UTC_TIMEOUT:=60}


# retrieve UT1-UTC and polar motion file
###############################################
# default value for URL 
DEF_URL='http://maia.usno.navy.mil/ser7/finals2000A.data'
URL=${PAR_NAVIGATION_UT1_UTC_URL:=${DEF_URL}}

# Wget command
WGET='wget'

FILE=${DIR_DATA_TAI_UT1_UTC}/finals2000A.data
FILE_TMP=${FILE}.tmp

${WGET}  -T ${TIMEOUT} ${URL} -O ${FILE_TMP}

if [ -s ${FILE_TMP} ] ; then 
  mv ${FILE_TMP} ${FILE}
  echo "UT1-UTC file saved to ${FILE}"
else
  echo "Unable to retrieve UT1-UTC file from ${URL}"
  exit 1
fi


# retrieve TAI-UTC file
###############################
# default value for URL 
DEF_URL='http://maia.usno.navy.mil/ser7/tai-utc.dat'
URL=${PAR_NAVIGATION_TAI_UTC_URL:=${DEF_URL}}

# Wget command
WGET='wget'

FILE=${DIR_DATA_TAI_UT1_UTC}/tai-utc.dat
FILE_TMP=${FILE}.tmp

${WGET}  -T ${TIMEOUT} ${URL} -O ${FILE_TMP}

if [ -s ${FILE_TMP} ] ; then 
  mv ${FILE_TMP} ${FILE}
  echo "TAI-UTC file saved to ${FILE}"
else
  echo "Unable to retrieve TAI-UTC file from ${URL}"
  exit 1
fi


exit 0
