#! /bin/ksh
#
# Korn Shell to run lgephe.exe: creates the ephemeris file for a long period,
#	one satellite and a list of stations.
#
# The command process only one satellite. 
#
# PROPRIETAIRE/OWNER
#       P. Brunel - A. Marsouin
#
# HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
# (version   date      comment           <author>)
#  1.0     13/02/95   initial            P. Brunel
#  1.1     01/04/04   cleaning           P. Brunel
#  1.2     01/02/06   change config file P. Marguinaud
#
# COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
#     &/'%Z% %M% version %I% on %G%'/       
#

. "$AAPP_PREFIX/ATOVS_CONF"

# 0 initialize the variables
#---------------------------

# variables for the TBUS HOME directory
DIR_DATA_TBUS=${DIR_DATA_TBUS:=${DIR_NAVIGATION}/tbus_db}
if [ ! -d ${DIR_DATA_TBUS} ]; then
  DIR_DATA_TBUS=${DIR_NAVIGATION}/orb_elem
fi  
export DIR_DATA_TBUS

DIR_LGEPHE=${DIR_NAVIGATION}/lgephe
if [ ! -d ${DIR_LGEPHE} ]; then
  mkdir ${DIR_LGEPHE}
fi

# 1 usage
#-------- 
usage(){
	echo
	echo ' LGEPHE  creation of a lgephe file'
	echo
	echo 'usage lgephe  [-o] [-s  satellite_name] [-S station_list]';
	echo '   [-d start_date] [-n number_of_days]';
	echo
	echo '	options:';
	echo '  -s  satellite_name		default: noaa14';
	echo '  -S  station_list		default:lannion';
	echo '  -d  start_date			default:  today 0h';
	echo '  -n  number of days (real)	default:    10.0';
	echo '  -o  output on normalized lgephe name ';
	echo '      ${DIR_NAVIGATION}/lgepeh/lgephe_satellite_date.txt   ';
	echo '      default is standard output';
	echo
	echo ' the date format could be a date or a date/hour string:';
	echo " for example -d 'dd/mm/yy  hh:mm:ss.sss' (2 spaces between yy hh)";
	echo "     or      -d 'dd/mm/yy  hh:mm'";
	echo "     or      -d dd/mm/yy";
	echo
	echo 'environment variables:'
	echo '	DIR_SATID    path for the satellite identification file'
	echo '	SATID_FILE    filename for the satellite identification file'
        echo '  DIR_NAVIGATION  generic path for the tbus dir tbus_db'
	echo '	DIR_STATIONS  path for the stations file'
	}

# 2 get the arguments
#--------------------
          while getopts s:S:b:d:n:i:c:o c
          do
               case $c in
               s)	SAT=$OPTARG;;
               S)	STATION=$OPTARG;;
               d)	DATE=$OPTARG;;
               n)	NBD=$OPTARG;;
               o)	OUT=yes;;
               \?)      usage
                        exit 2;;
               esac
          done

# 2.1 default values
#-------------------

SAT=${SAT='noaa14'}
STATION=${STATION='lannion'}
DATE=${DATE=`date +%d/%m/%y`}
NBD=${NBD='10.00'}
OUT=${OUT='no'}

LGEPH=${DIR_DATA_TBUS}/lgephe_${SAT}.index


if [ "${OUT}" = 'yes' ] ; then
  set `echo ${DATE} | tr '/' ' '`
  if [ $3 -gt 50 ] ; then
    DATEY=19$3$2$1
  else
    DATEY=20$3$2$1
  fi
  OUTPUT="${DIR_LGEPHE}/lgephe_${SAT}_${DATEY}.txt"
else
  OUTPUT=''
fi

EPHEIN=${DIR_LGEPHE}/lgephe$$.in

cat <<EOF > ${EPHEIN}
${SAT}
${DATE}
${NBD}
${LGEPH}
EOF

for STAT in ${STATION}
do
echo ${STAT} >> ${EPHEIN}
done

# 3 creates the lgephe file
#--------------------------
if [ "${OUT}" = 'yes' ] ; then
lgephe.exe < ${EPHEIN} | sort > ${OUTPUT}

else
lgephe.exe < ${EPHEIN} | sort
fi

rm -f ${EPHEIN}

exit
