#! /bin/ksh
#
# Korn Shell to run lgepheing.exe: ingest a tbus bulletin for long term
#		ephemeris calculations.
# The command process several satellites given by the -s option.
# The tbus filename can be given by the -f option.
#
# The default -s option is : noaa09 noaa11 noaa12 noaa14 (LISTESAT)
# The default -f option is : all the tbus files which are newer than the 
#	last update of the historical ephemeris utilities files
#	corresponding to the satellite list
#
# PROPRIETAIRE/OWNER
#       P. Brunel - A. Marsouin
#
# HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
# (version   date      comment           <author>)
#  1.0     13/02/95   initial            P. Brunel
#  1.1     01/04/04   cleaning           P. Brunel
#  1.2     01/02/06   change config file P. Marguinaud
#  1.3     08/06/06   output to stdout not ${WRK}/lgepheing.log   N Atkinson
#
# COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
#     &/'%Z% %M% version %I% on %G%'/       
#

. "$AAPP_PREFIX/ATOVS_CONF"

# developpement
echo `date`' lgephe ingest'
 
# 0 initialize the variables
#---------------------------
PAR_NAVIGATION_DEFAULT_LISTESAT_INGEST_TBUS='noaa15 noaa16 noaa17'

# variables for the TBUS HOME directory
DIR_DATA_TBUS=${DIR_DATA_TBUS:=${DIR_NAVIGATION}/tbus_db}
if [ ! -d ${DIR_DATA_TBUS} ]; then
  DIR_DATA_TBUS=${DIR_NAVIGATION}/orb_elem
fi  
export DIR_DATA_TBUS

# working file
FIC_WRK=${DIR_DATA_TBUS}/tbus_tmp_$$

# 1 usage
#-------- 
usage(){
	echo
	echo ' LGEPHE Ingest'
	echo
	echo 'usage lgepheing [-s satellite_list] [-f tbus_file]';
	echo '	options:';
	echo '  -s  satellite_list	satellites to be considered';
	echo "		default:${PAR_NAVIGATION_DEFAULT_LISTESAT_INGEST_TBUS}";
	echo '  -f tbus_file		for the processing of only ONE file';
	echo
	echo 'environment variables:'
	echo '	DIR_SATID  path for the satellite identification file'
	echo '	SATID_FILE    filename for the satellite identification file'
        echo '  DIR_NAVIGATION  generic path for the tbus dir tbus_db'
	}

# 2 get the arguments
#--------------------
          while getopts s:f: c
          do
               case $c in
               s)	LISTESAT=$OPTARG;;
               f)	FTBUS=$OPTARG;;
               \?|h)    usage
                        exit 2;;
               esac
          done

LISTESAT=${LISTESAT=${PAR_NAVIGATION_DEFAULT_LISTESAT_INGEST_TBUS}}

# 3 loop on the satellites
#-------------------------
#
for SAT in ${LISTESAT}
do

  LGEPH=${DIR_DATA_TBUS}/lgephe_${SAT}.index

# 3.1 tbus file to ingest
# -----------------------
# find for the satellite the tbus files which are newer than
# the last run of lgepheing (date of the historical ephemeris utilities file)
#
  if [ "${FTBUS}" = '' ] ; then
    if [ -s ${LGEPH} ] ; then
      (cd ${DIR_DATA_TBUS} ; find . -follow -name \
     'tbus_*.txt' -newer ${LGEPH} -print | sort > ${FIC_WRK} )
    else
      (cd ${DIR_DATA_TBUS} ; find . -follow -name \
     'tbus_*.txt' -print | sort > ${FIC_WRK} )
    fi
  else
    echo ${FTBUS} > ${FIC_WRK}
  fi

# loop on the tbus files
#-----------------------
  for FILE in `cat ${FIC_WRK}`
  do

# 3.2 ingest one tbus file
#-------------------------
   lgepheing.exe << EOF
${DIR_DATA_TBUS}
${FILE}
${SAT}
${LGEPH}
EOF


# 3.3 Sort the historical ephemeris utilities file
#-------------------------------------------------

  sort -u -o ${LGEPH}.sort +0b -3b ${LGEPH}
  if [ $? = 0 ] ; then
    rm ${LGEPH}
    mv ${LGEPH}.sort ${LGEPH}
  else
    rm ${LGEPH}.sort
  fi

# end of loop on the tbus files
#------------------------------
  done

# end of loop on the satellites
#------------------------------
done

# remove the working file
rm -f ${FIC_WRK}
exit
