C LIBS =  tbus brolyd satid f7ml f7tp f7cp f7gp auxaapp
	program tbusdisp
!+
!NOM/NAME  tbusdisp.F version 1.1 on 3/24/98        
!
!BUT/PURPOSE 
!	Displays the content of a TBUS file for any satellite
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date         comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
! functions
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!EXIT CODE
!	0 = normal execution 
!
!DECLARATIONS
!
! identification variable for "what"
      character*62 sccsid     
!
      data sccsid
     &/'@(#) tbusdisp.F version 1.1 on 3/24/98'/       
!
!
! Parameters
	integer nbsatmax		! maximum number of satellites 
	parameter (nbsatmax=5)
	integer tbunit			! logical unit for the tbus file
	parameter (tbunit=10)
	integer tbindu			! logical unit for the tbus index
	parameter (tbindu=11)
	logical debug
	parameter (debug=.true.)
!
! Functions
	integer genscid			! order number for a satellite name
	character*6 noascnam		! NOAA satellite name
	integer cp_lsc			! last significant character (f7cp)
!
! Inputs
	character*256 dirtbus		! reference directory for the tbus files
	character*256 tbus_file		! filename of the tbus without the
!					!   ref. directory
	character*12 scname(nbsatmax)	! satellite name list
	character*256 tbind_file(nbsatmax)	! filename of the tbus index
!
! Local variables:
	integer ios			! I/O status variable
	integer isat			! current satellite number in the list
	integer nbsat			! number of satellites to process
	integer prtbus			! if no zero tbus is printed on stdout
	integer prtclk			! if no zero clock errors printed 
	integer ordernum		! order number of a satellite
	integer dcerr			! return code of tb_dc
	real dtmax			! maximum range for search in days
	character*6 noaanam		! NOAA name of the current satellite
	character*12 lsatname		! temporary satellite name
	character*256 name		! var. name for the files
	character*256 tbind		! temporary filename of the tbus index
	character*256 full_tbus_file	! full filename of the tbus

	character*256 inpdate		! input date string
	character*256 searmeth		! search method
	double precision epoch		! epoch time
	integer depoch			! epoch date
	integer ierr
! 
!-
!
!	Section 1
!	Reads the input parameters
!	--------------------------
!	Get the home directory of the TBUS file
	read(*,'(a)',iostat=ios) dirtbus
	if(ios.ne.0) then
	  call ml_wti('error','Input error   iostat =',ios)
	  call ml_wt('error','for TBUS home directory ')
	  call ml_wt('error','TBUSDISP aborted ')
	  call aappexit(2)
	endif
!
!	Get the satellite and the index filename
	nbsat=0
	ios = 0
	read(*,'(a)',iostat=ios) lsatname
	read(*,'(a)',iostat=ios) tbind
	if(ios.eq.0) then
	  ordernum= genscid(lsatname)
	  if(ordernum.ne.0) then
	    nbsat = nbsat+1
	    scname(nbsat) = lsatname
	    tbind_file(nbsat)  = tbind
	  else
	    call ml_wt('error','Input satellite '//lsatname)
	    call ml_wt('error',
     &                 'unknown in the satellite identification file')
	    call aappexit(2)
	  endif
	endif

!	Get the date of the tbus
	read(*,'(a)',iostat=ios) inpdate
	if(ios.ne.0) then
	  call ml_wti('error','Input error   iostat =',ios)
	  call ml_wt('error','for date string ')
	  call ml_wt('error','TBUSDISP aborted ')
	  call aappexit(2)
	endif
	if(cp_lsc(inpdate).eq.8) then
	  call xdcj(inpdate,depoch)
	  epoch = depoch
	elseif(cp_lsc(inpdate).eq.22) then
	  call xicj(inpdate,epoch)
	else
	  call ml_wt('error','Error in the date format ')
	  call ml_wt('error','allowed formats: ')
	  call ml_wt('error','dd/mm/yy or dd/mm/yy  hh:mm:ss.sss')
	  call ml_wt('error','TBUSDISP aborted ')
	  call aappexit(2)
	endif


!	Get the search method of the tbus (nearest last_preceeding)
	read(*,'(a)',iostat=ios) searmeth
	if(ios.ne.0) then
	  call ml_wti('error','Input error   iostat =',ios)
	  call ml_wt('error','while reading the search method')
	  call ml_wt('error','TBUSDISP aborted ')
	  call aappexit(2)
	endif

	
!	
!
!	Section 2.
!	Opens the satellite dependent files
!	-----------------------------------
	isat = 1
	ordernum = genscid(scname(isat))
	noaanam  = noascnam(ordernum)
!
	open (unit=tbindu,file=tbind_file(isat),status='old',iostat=ios)
	if( ios.ne.0 ) then
	  call ml_wti('error','open error   iostat =',ios)
	  call ml_wt('error','for the tbus index '//tbind_file(isat))
	  call ml_wt('error','TBUSDISP aborted ')
	  call aappexit(2)
	endif

	if(searmeth.eq.'nearest') then
	  dtmax=99999.			! maximum range for search in days
	  call tb_gnv(tbindu,epoch,dtmax,tbus_file,ierr)
	else
	  call tb_glpv(tbindu,epoch,tbus_file,ierr)
	endif

	if(ierr.eq.0) then

!	create the tbus filename (adding the dirtbus and 
!	suppressing the white spaces)
	name =  dirtbus(1:cp_lsc(dirtbus)) // '/' // tbus_file
 	call cp_ssf(name,full_tbus_file,' ')
!
!	opens the tbus file
	open (unit=tbunit,file=full_tbus_file,status='old',iostat=ios)
	if( ios.ne.0 ) then
	  call ml_wti('error','open error   iostat =',ios)
	  call ml_wt('error','for the tbus file '//full_tbus_file)
	  call ml_wt('error','TBUSDISP aborted ')
	  call aappexit(2)
	else
	  call ml_wt('info','Displays tbus '//full_tbus_file)
	endif

	prtbus = 1
	call tb_dc(noaanam,tbunit,prtbus,dcerr)

	prtclk = 1
	call clkerr_dc(noaanam,tbunit,prtclk,dcerr)
!
	close (tbunit)

	endif

	close (tbindu)

	stop
	end
