#! /bin/ksh
#
# Korn Shell to run tbusing.exe: ingest a tbus bulletin
# The command process several satellites given by the -s option.
# The tbus filename can be given by the -f option.
#
# The default -s option is : noaa09 noaa11 noaa12 noaa14
#	see the variable PAR_NAVIGATION_DEFAULT_LISTESAT_INGEST_TBUS
# The default -f option is : all the tbus files which are newer than the 
#	last update of the index files corresponding to the satellite list
#
# PROPRIETAIRE/OWNER
#       P. Brunel - A. Marsouin
#
# HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
# (version   date      comment           <author>)
#  1.0     13/02/95   initial            P. Brunel
#  1.1     01/04/04   cleaning           P. Brunel
#  1.2     25/04/05   Create log file in ${WRK}. N.Atkinson
#  1.3     01/02/06   change config file P. Marguinaud
#  1.4     08/06/06   output to stdout not ${WRK}/tbusing.log   N Atkinson
#
# COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
#

. "$AAPP_PREFIX/ATOVS_CONF"

echo `date`' tbus ingest '
 
# 0 initialize the variables
#---------------------------

# variables for the TBUS HOME directory
DIR_DATA_TBUS=${DIR_DATA_TBUS:=${DIR_NAVIGATION}/tbus_db}
if [ ! -d ${DIR_DATA_TBUS} ]; then
  DIR_DATA_TBUS=${DIR_NAVIGATION}/orb_elem
fi  
export DIR_DATA_TBUS

# working file
FIC_WRK=${DIR_DATA_TBUS}/tbus_tmp_$$

# 1 usage
#-------- 
usage(){
	echo
	echo ' TBUS Ingest'
	echo
	echo 'usage: tbusing [-s satellite_list] [-f tbus_file]';
	echo
	echo '	options:';
	echo '  -s  satellite_list	satellites to be considered';
	echo '			default: cf ${DIR_NAVIGATION}/parametres';
	echo '  -f tbus_file		to process only ONE file';
	echo
	echo 'environment variables:'
	echo '	DIR_SATID  path for the satellite identification file'
	echo '	SATID_FILE    filename for the satellite identification file'
	echo '	DIR_NAVIGATION	generic path for the tbus dir tbus_db'
	}

# 2 get the arguments
#--------------------
          while getopts s:f:h c
          do
               case $c in
               s)	LISTESAT=$OPTARG;;
               f)	FTBUS=$OPTARG;;
               \?|h)    usage
                        exit 2;;
               esac
          done

LISTESAT=${LISTESAT=${PAR_NAVIGATION_DEFAULT_LISTESAT_INGEST_TBUS}}

#log_info ${LISTESAT}

# 3 loop on the satellites
#-------------------------
#
for SAT in ${LISTESAT}
do

  TBUS_INDEX=${DIR_DATA_TBUS}/tbus_${SAT}.index
  CLOCK_ERROR=${DIR_DATA_TBUS}/clkerr_${SAT}.txt

# 3.1 tbus file list to ingest
# ----------------------------
# find for the satellite the tbus files which are newer than
# the last run of tbusing (date of the tbus index file)
#
  if [ "${FTBUS}" = '' ] ; then
    if [ -s ${TBUS_INDEX} ] ; then
      (cd ${DIR_DATA_TBUS} ; find . -follow -name \
     'tbus_*.txt' -newer ${TBUS_INDEX} -print | sort > ${FIC_WRK} )
    else
      (cd ${DIR_DATA_TBUS} ; find . -follow -name \
     'tbus_*.txt' -print | sort > ${FIC_WRK} )
    fi
  else
    echo ${FTBUS} > ${FIC_WRK}
  fi

# loop on the tbus files
#-----------------------
  for FILE in `cat ${FIC_WRK}`
  do

# 3.2 ingest one tbus file
#-------------------------
   tbusing.exe << EOF
${DIR_DATA_TBUS}
${FILE}
${SAT}
${TBUS_INDEX}
${CLOCK_ERROR}
EOF


# 3.3 Sort the historical files
#------------------------------
  sort -u -o ${CLOCK_ERROR}.sort +0b -2b ${CLOCK_ERROR}
  if [ $? = 0 ] ; then
    rm -f ${CLOCK_ERROR}
    mv ${CLOCK_ERROR}.sort ${CLOCK_ERROR}
  else
    rm -f ${CLOCK_ERROR}.sort
  fi

  sort -u -o ${TBUS_INDEX}.sort +0b -3b ${TBUS_INDEX}
  if [ $? = 0 ] ; then
    rm -f ${TBUS_INDEX}
    mv ${TBUS_INDEX}.sort ${TBUS_INDEX}
  else
    rm -f ${TBUS_INDEX}.sort
  fi

# end of loop on the files
#-------------------------
  done

# end of loop on the satellites
#------------------------------
done

# remove the working file
rm -f ${FIC_WRK}

exit
