#! /bin/ksh
#
# Korn Shell to run tleing.exe: ingest a tle bulletin
# The command process several satellites given by the -s option.
# The tle filename can be given by the -f option.
#
# The default -s option is give by the 
#        variable PAR_NAVIGATION_DEFAULT_LISTESAT_INGEST_TLE
#       which can be set in the parameter file ou ATOVS_ENV file
#       depending on the environment
# The default -f option is : all the tle files which are newer than the 
#	last update of the index files corresponding to the satellite list
#
# PROPRIETAIRE/OWNER
#       P. Brunel - A. Marsouin
#
# HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
# (version   date      comment           <author>)
#  1.0     02/01/2002  initial            P. Brunel
#  1.1     01/02/2006  change config file P. Marguinaud
#  1.2     26/09/2012  change search from 'tle_*_*' to 'tle_*.txt'. NCA
#
# COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
#     &/'%Z% %M% version %I% on %G%'/       
#


. "$AAPP_PREFIX/ATOVS_CONF"
. liblog.sh

#-----------------------------------------------------


 
# 0 initialize the variables
#---------------------------
# SATellite IDentification mandatory
# so take care of default values 
SATID_FILE=${SATID_FILE:=satid.txt}
export DIR_SATID SATID_FILE
 
# variables for the TLE HOME directory
DIR_DATA_TLE=${DIR_DATA_TLE:=${DIR_NAVIGATION}/tle_db}
if [ ! -d ${DIR_DATA_TLE} ]; then
  DIR_DATA_TLE=${DIR_NAVIGATION}/orb_elem
fi  

# working file
FIC_WRK=${DIR_DATA_TLE}/tmp_tle_$$

# 1 usage
#-------- 
usage(){
	echo
	echo ' TLE Ingest'
	echo
	echo 'usage: tleing [-s satellite_list] [-f tle_file] -c ';
	echo
	echo '	options:';
	echo '  -s  satellite_list	satellites to be considered';
	echo '			default: cf ${DIR_NAVIGATION}/parametres';
	echo '  -f tle_file		to process only ONE file';
	echo '  -c       check presence of input 2lines files ';
	echo '           in final index file ';
	echo '       ( not recommanded when several get_tle per day';
	echo
	echo 'environment variables:'
	echo '	DIR_SATID  path for the satellite identification file'
	echo '	DIR_NAVIGATION	generic path for the tle dir tle_db'
	}

CTRL='no'
# 2 get the arguments
#--------------------
          while getopts s:f:hc c
          do
               case $c in
               s)	LISTESAT=$OPTARG;;
               f)	FTLE=$OPTARG;;
               c)	CTRL='yes';;
               \?|h)    usage
                        exit 2;;
               esac
          done

LISTESAT=${LISTESAT:=${PAR_NAVIGATION_DEFAULT_LISTESAT_INGEST_TLE}}

log_info ${LISTESAT}

# 3 loop on the satellites
#-------------------------
#
for SAT in ${LISTESAT}
do

  TLE_INDEX=${DIR_DATA_TLE}/tle_${SAT}.index

# 3.1 tle file list to ingest
# ----------------------------
# find for the satellite the tle files which are newer than
# the last run of tleing (date of the tle index file)
#
  if [ "${FTLE}" = '' ] ; then
    if [ -s ${TLE_INDEX} ] ; then
      (cd ${DIR_DATA_TLE} ; find . -follow -name \
     'tle_*.txt' -newer ${TLE_INDEX} -print | sort > ${FIC_WRK} )
    else
      (cd ${DIR_DATA_TLE} ; find . -follow -name \
     'tle_*.txt' -print | sort > ${FIC_WRK} )
    fi
  else
    echo ${FTLE} > ${FIC_WRK}
  fi

# loop on the tle files
#-----------------------
  for FILE in `cat ${FIC_WRK}`
  do

# 3.2 ingest one tle file
#-------------------------
tleing.exe << EOF
${DIR_DATA_TLE}
${FILE}
${SAT}
${TLE_INDEX}
EOF


# 3.3 Sort the historical files
#------------------------------

  sort -u -o ${TLE_INDEX}.sort +0b -3b ${TLE_INDEX}
  if [ $? = 0 ] ; then
    rm -f ${TLE_INDEX}
    mv ${TLE_INDEX}.sort ${TLE_INDEX}
  else
    rm -f ${TLE_INDEX}.sort
  fi

# 3.4 Control test
#-----------------
# This control test can be enabled (-c argument). It just checks
# that the input 2lines file is present inside the index file
# after the run. But in some normal cases, when the user retrieves
# 2 lines elements several time per day, the abscence of the file
# in the index is not an error.
  if [ "${CTRL}" = 'yes' ] ; then
    OK=`grep "${FILE}" ${TLE_INDEX}`
    if [ -z "${OK}" ] ; then
      log_error "the file  ${FILE} NOT inserted in the index file"
      log_error "index file is: ${TLE_INDEX}"
      log_error "Possible explanaition: the file contains an orbital bulletin which is already present in the index"
      log_error "Please VERIFY"
    fi
  fi

# end of loop on the files
#-------------------------
  done

# end of loop on the satellites
#------------------------------
done

# remove the working file
rm -f ${FIC_WRK}

exit
