C LIBS = ephtrack navtool navnoaa tle sgp satid f7ml f7tp f7cp f7gp auxaapp
        program tleprint
!+
!NOM/NAME $Id: tleprint.F 47 2006-08-21 08:44:41Z frna $
!
!BUT/PURPOSE 
!       Display a TLE bulletin for a specific satellite
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!       Information for each satellite is processed separately.
!
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date         comment           <author>)
! 1.0   01 April 2004      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!       tle_dc                          Decodes TLE orbital elements
!               tle_chk                 Checks TLE orbital elements
!               tle_prn                 Print TLE orbital elements
!
! functions
!       genscid
!       genscnam
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!EXIT CODE
!       0 = normal execution 
!
!DECLARATIONS
!
! identification variable for "what"
      character*62 sccsid     
!
      data sccsid
     &/'%Z% %M% version %I% on %G%'/       
!
!
! Parameters
        integer nbsatmax                ! maximum number of satellites 
        parameter (nbsatmax=5)
        integer tleunit                 ! logical unit for the tle file
        parameter (tleunit=10)
        integer tleindu                 ! logical unit for the tle index
        parameter (tleindu=11)
        logical debug
        parameter (debug=.false.)
!
! Functions
        integer genscid                 ! order number for a satellite name
!
! Local variables:
        integer ios                     ! I/O status variable
        integer prtle                   ! if no zero tle is printed on stdout
        integer ordernum                ! order number of a satellite
        integer dcerr                   ! return code of tle_dc
        character*16 lsatname           ! temporary satellite name
        character*256 full_tle_file     ! full filename of the tle
! 
!-
!
        call sgp_init
!
!       Section 1
!       Reads the input parameters
!       --------------------------
!       Get the full pathname of the TLE file
        write(*,"(/'enter 2-line bulletin filename')") 
        read(*,'(a)',iostat=ios) full_tle_file
        if(ios.ne.0) then
          call ml_wti('error','Input error   iostat =',ios)
          call ml_wt('error','for TLE file name ')
          call ml_wt('error','TLEPRINT aborted ')
          call aappexit(2)
        endif

!       opens the tle file
        open (unit=tleunit,file=full_tle_file,status='old',iostat=ios)
        if( ios.ne.0 ) then
          call ml_wti('error','open error   iostat =',ios)
          call ml_wt('error','for the tle file '//full_tle_file)
          call ml_wt('error','TLEPRINT aborted ')
          call aappexit(2)
        endif
!
!       Get the satellite list and the index filename
        write(*,"(/'enter satellite name or end')") 
        read(*,'(a)',iostat=ios) lsatname
        do while (ios .eq. 0 .and. lsatname.ne.'end')
          if(ios.eq.0) then
            ordernum= genscid(lsatname)
            if(ordernum.ne.0) then

!         Reads the message and checks
!         ----------------------------
          prtle = 1
!         decodes
          call tle_dc(lsatname,tleunit,prtle,dcerr)


            else
            call ml_wt('notice','Input satellite '//lsatname)
            call ml_wt('notice',
     &                 'unknown in the satellite identification file')
            endif
          endif
          write(*,"(/'enter satellite name or end')") 
          read(*,'(a)',iostat=ios) lsatname
        end do

!       

        close (tleunit)

        stop
        end
