      subroutine mrchgrepj2000cm (imodr,jj,sec,dtai,posvit1,posvit2,ier)
C Doc utilisateur:
C   imodel       = imodr
C   ijul1950jour = jj
C   rjul1950sec  = sec
C   rdtai        = dtai
C   rposvit1(6)  = posvit1(6)
C   rposvit2(6)  = posvit2(6)
C
C***********************************************************************
C*
C*PRO MSLIB 0268
C*VER 18.03.99 VERSION 10.0
C*AUT SPEC. CNES - LE DE -
C*AUT PROG. (Mise aux normes MSLIB) CISI DIS/MSS
C*AUT (C) COPYRIGHT CNES - 1999
C*ROL THEME : Changement de systemes de reference
C*ROL Passage du repere equatorial moyen J2000 
C*ROL au repere equatorial moyen a la date T
C
C*PAR (I) imodr   : indicateur des modeles de precession et de nutation
C*PAR                  = 1 : J1900 (NEWCOMB + WOOLARD)
C*PAR                  = 2 : J2000 (LIESKE + WAHR)
C*PAR (I) jj      :} date julienne 1950 T (non normalisee) en jour 
C*PAR (I) sec     :}                                   et secondes
C*PAR (I) dtai    : ecart entre l'echelle de temps TAI et l'echelle de
C*PAR               temps utilisee pour la date T
C*PAR (I) posvit1 : vecteur position-vitesse dans le repere equatorial
C*PAR               moyen J2000
C*PAR (O) posvit2 : vecteur position-vitesse dans le repere equatorial
C*PAR               moyen a la date T
C*PAR (O) ier     : code retour
C
C*NOT VALEURS DU CODE RETOUR :
C*NOT    0: OK
C*NOT   -1 :  Erreur sur imodr: different de 1 ou 2
C*NOT DOCUMENT DE REFERENCE :
C*NOT [DR1] "Les systemes de reference utilises en astronomie"
C*NOT       de M. Chapront-Touze, G. Francou et B. Morando
C*NOT       Bureau Des Longitudes (BDL) novembre 1994
C*NOT       ISSN 1243-4272
C*NOT       ISBN 2-910015-05-X
C*NOT       nomenclature MSLIB M-NT-0-160-CN
C*NOT Nota: Les epoques J1900 et J2000 sont explicitees en
C*NOT       page 23 table 1.1
C*HST VERSION 10.0 - 18.03.99 - CREATION: M-SP-268-1-CIS
C*
C*******************************************************************
C
      implicit none
C
C------------------------------
C     PARAMETRES
C------------------------------
      integer           imodr
      double precision  sec,dtai
      double precision  posvit1(6),posvit2(6)
      integer           jj
      integer           ier
C
C-------------------------------
C     SOUS-PROGRAMMES UTILISES
C-------------------------------
C
      external  mrmatj2000cm, mumatp
C
C
C
C-------------------------------
C     VARIABLES LOCALES
C-------------------------------
C
C     matrices de changement de repere et rotation
      double precision  rmat(3,3), rmatd(3,3)
C
C     code retour des routines appelees
      integer ierapp
C
C     matrices de travail pour les vecteurs position/vitesse
      double precision matpv1(3,3), matpv2(3,3), matpv3(3,3)
C
      integer icol,ilig,icolpv
C
C
C     . . . INITIALISATION DE L'IDENTIFICATEUR SCCS
C      
      CHARACTER*50 SCCSID
      DATA SCCSID/"%Z%  %M%  %Y%  %I%  %E%"/
C
C-------------------------------
C     INITIALISATION
C-------------------------------
C
c
      IF (.FALSE.) THEN
         SCCSID = SCCSID
      ENDIF
c
C
      ier = 0
C
      icolpv = 3
      ilig = 3
      icol = 3
C
C-------------------------------
C Test sur le modele choisi     
C-------------------------------
C
      if ((imodr.ne.1).and.(imodr.ne.2)) then
         ier = -1
         goto 6000
      endif
C
C-------------------------------
C-------------------------------
C     Algorithme
C-------------------------------
C-------------------------------
C
C ----Voir le paragraphe 12.4 de [DR1] pour la theorie------
C
C     calcul de la matrice de passage et de la matrice rotation
C
      call mrmatj2000cm(imodr,jj,sec,dtai,rmat,rmatd)
C
C     changement de repere 
C
      matpv1(1,1) = posvit1(1)
      matpv1(2,1) = posvit1(2)
      matpv1(3,1) = posvit1(3)
      matpv1(1,2) = posvit1(4)
      matpv1(2,2) = posvit1(5)
      matpv1(3,2) = posvit1(6)
      matpv1(1,3) = 0.d0
      matpv1(2,3) = 0.d0
      matpv1(3,3) = 0.d0
C
      call mumatp(rmat,matpv1,ilig,icol,icolpv,ilig,icol,matpv2,ierapp)
C
      call mumatp(rmatd,matpv2,ilig,icol,icolpv,ilig,icol,matpv3,ierapp)
C
C ----calcul effectue sur la base de la
C      formule 12.16 de [DR1] pour le vecteur position ---------
      posvit2(1) = matpv2(1,1)
      posvit2(2) = matpv2(2,1)
      posvit2(3) = matpv2(3,1)
C ----et sur la base de la formule 12.23 pour le vecteur vitesse
      posvit2(4) = matpv2(1,2)+matpv3(1,1)
      posvit2(5) = matpv2(2,2)+matpv3(2,1)
      posvit2(6) = matpv2(3,2)+matpv3(3,1)
C
C
C-------------------------------
C     Gestion des erreurs
C-------------------------------
 6000  continue
C
      end
