      subroutine mrchgrepj2000v (imodr,jj,sec,dtu1,dtai,posvit1,
     &                          posvit4,ier)
C Doc utilisateur:
C   imodel       = imodr
C   ijul1950jour = jj
C   rjul1950sec  = sec
C   rdtu1        = dtu1
C   rdtai        = dtai
C   rposvit1(6)  = posvit1(6)
C   rposvit2(6)  = posvit4(6)
C
C***********************************************************************
C*
C*PRO MSLIB 0270
C*VER 18.03.99 VERSION 10.0
C*AUT SPEC. CNES - LE DE -
C*AUT PROG. (Mise aux normes MSLIB) CISI DIS/MSS
C*AUT (C) COPYRIGHT CNES - 1999
C*ROL THEME : Changement de systemes de reference
C*ROL Passage du repere equatorial moyen J2000 
C*ROL au repere de Veis a la date T
C*
C*PAR (I) imodr   : indicateur des modeles de precession et de nutation
C*PAR                  = 1 : NEWCOMB(B1900) + WOOLARD(J1900)
C*PAR                  = 2 : LIESKE(J2000) + WAHR(J2000)
C*PAR (I) jj      :} date julienne 1950 T (non normalisee) en jour 
C*PAR (I) sec     :}                                   et secondes
C*PAR (I) dtu1    : ecart entre l'echelle de temps TU1 et l'echelle de
C*PAR               temps utilisee pour la date T
C*PAR (I) dtai    : ecart entre l'echelle de temps TAI et l'echelle de
C*PAR               temps utilisee pour la date T
C*PAR (I) posvit1 : vecteur position-vitesse dans le repere equatorial
C*PAR               moyen J2000
C*PAR (O) posvit4 : vecteur position-vitesse dans le repere
C*PAR               de Veis a la date T
C*PAR (O) ier     : code retour
C
C*NOT VALEURS DU CODE RETOUR :
C*NOT    0: OK
C*NOT   -1 :  Erreur sur imodr: different de 1 ou 2
C*NOT DOCUMENT DE REFERENCE :
C*NOT [DR1] "Les systemes de reference utilises en astronomie"
C*NOT       de M. Chapront-Touze, G. Francou et B. Morando
C*NOT       Bureau Des Longitudes (BDL) novembre 1994
C*NOT       ISSN 1243-4272
C*NOT       ISBN 2-910015-05-X
C*NOT       nomenclature MSLIB M-NT-0-160-CN
C*NOT Nota: Les epoques B1900, J1900 et J2000 sont explicitees en
C*NOT       page 23 table 1.1
C*HST VERSION 10.0 - 18.03.99 - CREATION: M-SP-270-1-CIS
C*
C*******************************************************************
C
      implicit none
C
C------------------------------
C     PARAMETRES
C------------------------------
      integer          imodr
      integer          jj
      double precision sec,dtu1,dtai
      double precision posvit1(6),posvit4(6)
      integer ier
C
C-------------------------------
C     SOUS-PROGRAMMES UTILISES
C-------------------------------
C
      external mrchgrepj2000cm, mrchgrepcmcv, mrchgrepcvveis
C
C
C
C-------------------------------
C     VARIABLES LOCALES
C-------------------------------
C     vecteurs position/vitesse intermediaires
      double precision posvit2(6),posvit3(6)
C
C
C     . . . INITIALISATION DE L'IDENTIFICATEUR SCCS
C      
      CHARACTER*50 SCCSID
      DATA SCCSID/"%Z%  %M%  %Y%  %I%  %E%"/
C
C-------------------------------
C     INITIALISATION
C-------------------------------
C
c
      IF (.FALSE.) THEN
         SCCSID = SCCSID
      ENDIF
c
C
      ier = 0
C-------------------------------
C Test sur le modele choisi     
C-------------------------------
C
      if ((imodr.ne.1).and.(imodr.ne.2)) then
         ier = -1
         goto 6000
      endif
C
C
C-------------------------------
C-------------------------------
C     Algorithme
C-------------------------------
C-------------------------------
C
C calcul du passage du repere J2000 au repere celeste moyen a la date t
C
      call mrchgrepj2000cm (imodr,jj,sec,dtai,posvit1,posvit2,ier)
C
C calcul du passage du repere celeste moyen au repere celeste vrai a la
C date t
C
      call mrchgrepcmcv (imodr,jj,sec,dtai,posvit2,posvit3,ier)
C
C calcul du passage du repere celeste vrai au repere de Veis a la date t
C
      call mrchgrepcvveis (imodr,jj,sec,dtu1,dtai,posvit3,posvit4,ier)
C
C-------------------------------
C     Gestion des erreurs
C-------------------------------
 6000  continue
C
      end
