      subroutine mrchgreptvtref (uref,vref,posvit1,posvit2,ier)
C
C Doc utilisateur:
C   ru           = uref
C   rv           = vref
C   rposvit1(6)  = posvit1(6)
C   rposvit2(6)  = posvit2(6)
C
C***********************************************************************
C*
C*PRO MSLIB 0277
C*VER 18.03.99 VERSION 10.0
C*AUT SPEC. CNES - LE DE -
C*AUT PROG. (Mise aux normes MSLIB) CISI DIS/MSS
C*AUT (C) COPYRIGHT CNES - 1999
C*ROL THEME : Changement de systemes de reference
C*ROL  Passage du repere terrestre vrai a la date T 
C*ROL  au repere terrestre de reference
C
C*PAR (I) uref    : coordonnee U du pole vrai a la date T dans le
C*PAR               repere R
C*PAR (I) vref    : coordonnee V du pole vrai a la date T dans le
C*PAR               repere R
C*PAR (I) posvit1 : vecteur position-vitesse dans le repere terrestre
C*PAR               vrai a la date T
C*PAR (O) posvit2 : vecteur position-vitesse dans le repere terrestre
C*PAR               de reference
C*PAR (O) ier     : code retour
C
C*NOT Definition du repere R: 
C*NOT   - il est contenu dans le plan tangent a la Terre au pole Nord
C*NOT     geographique
C*NOT   - axe des coordonnees U: axe selon la direction du meridien 
C*NOT     d'origine (Greenwich), et oriente positivement vers 
C*NOT     Greenwich (Londres)
C*NOT   - axe des coordonnees V: axe selon la direction perpendiculaire
C*NOT     a l'axe U, et oriente positivement vers le Canada
C
C*NOT VALEURS DU CODE RETOUR :
C*NOT    0: OK
C*NOT DOCUMENT DE REFERENCE :
C*NOT [DR1] "Les systemes de reference utilises en astronomie"
C*NOT       de M. Chapront-Touze, G. Francou et B. Morando
C*NOT       Bureau Des Longitudes (BDL) novembre 1994
C*NOT       ISSN 1243-4272
C*NOT       ISBN 2-910015-05-X
C*NOT       nomenclature MSLIB M-NT-0-160-CN
C*NOT Remarque: Dans ce changement de repere,
C*NOT           la composition des vitesses est negligee
C*HST VERSION 10.0 - 18.03.99 - CREATION: M-SP-277-1-CIS
C*
C*******************************************************************
C
      implicit none
C
C------------------------------
C     PARAMETRES
C------------------------------
      double precision uref,vref
      double precision posvit1(6),posvit2(6)
      integer ier
C
C-------------------------------
C     SOUS-PROGRAMMES UTILISES
C-------------------------------
C
      external mrmattvtref, mumatp
C
C-------------------------------
C     VARIABLES LOCALES
C-------------------------------
C     matrice de passage 
      double precision rmat(3,3)
C
C     code retour des routines appelees
      integer ierapp
C
C     matrices de travail pour le vecteur position/vitesse
      double precision matpv1(3,3), matpv2(3,3)
C
      integer icol,ilig,icolpv
C
C
C     . . . INITIALISATION DE L'IDENTIFICATEUR SCCS
C      
      CHARACTER*50 SCCSID
      DATA SCCSID/"%Z%  %M%  %Y%  %I%  %E%"/
C
C-------------------------------
C     INITIALISATION
C-------------------------------
C
      IF (.FALSE.) THEN
         SCCSID = SCCSID
      ENDIF
C
      ier = 0
C
      icolpv = 3
      ilig = 3
      icol = 3
C
C-------------------------------
C-------------------------------
C     Algorithme
C-------------------------------
C-------------------------------
C
C ----Voir le sous-paragraphe 4.4.1 de [DR1] pour la theorie------
C
C     calcul de la matrice de passage
C
      call mrmattvtref(uref,vref,rmat)
C
C     changement de repere 
C
      matpv1(1,1) = posvit1(1)
      matpv1(2,1) = posvit1(2)
      matpv1(3,1) = posvit1(3)
      matpv1(1,2) = posvit1(4)
      matpv1(2,2) = posvit1(5)
      matpv1(3,2) = posvit1(6)
      matpv1(1,3) = 0.d0
      matpv1(2,3) = 0.d0
      matpv1(3,3) = 0.d0
C
      call mumatp(rmat,matpv1,ilig,icol,icolpv,ilig,icol,matpv2,ierapp)
C
      posvit2(1) = matpv2(1,1)
      posvit2(2) = matpv2(2,1)
      posvit2(3) = matpv2(3,1)
      posvit2(4) = matpv2(1,2)
      posvit2(5) = matpv2(2,2)
      posvit2(6) = matpv2(3,2)
C
C-------------------------------
C     Gestion des erreurs
C-------------------------------
 6000  continue
C
      end
