      subroutine mrmatcmcv (imodr,jj,sec,dtai,rmat,rmatd)
C
C***********************************************************************
C*
C*PRO MSLIB 0278
C*VER 18.03.99 VERSION 10.0
C*AUT SPEC. CNES - LE DE -
C*AUT PROG. (Mise aux normes MSLIB) CISI DIS/MSS
C*AUT (C) COPYRIGHT CNES - 1999
C*ROL THEME : Changement de systemes de reference
C*ROL Calcule la matrice de passage et la matrice rotation du repere 
C*ROL equatorial moyen a la date T au repere equatorial vrai a la meme
C*ROL date T
C
C*PAR (I) imodr : indicateur des modeles de precession et de nutation
C*PAR                  = 1 : J1900 (NEWCOMB + WOOLARD)
C*PAR                  = 2 : J2000 (LIESKE + WAHR)
C*PAR (I) jj      :} date julienne 1950 T (non normalisee) en jour 
C*PAR (I) sec     :}                                   et secondes
C*PAR (I) dtai    : ecart entre l'echelle de temps TAI et l'echelle de
C*PAR               temps utilisee pour la date T
C*PAR (O) rmat  : matrice de changement de repere
C*PAR (O) rmatd : matrice de rotation
C
C*NOT Note d'utilisation: 
C*NOT   - Routine interne
C*NOT   - L'appelant doit avoir verifie que: imodr = 1 ou 2
C*NOT     (pas de test effectue ici)
C
C*NOT VALEURS DU CODE RETOUR : sans objet
C*NOT DOCUMENT DE REFERENCE :
C*NOT [DR1] "Les systemes de reference utilises en astronomie"
C*NOT       de M. Chapront-Touze, G. Francou et B. Morando
C*NOT       Bureau Des Longitudes (BDL) novembre 1994
C*NOT       ISSN 1243-4272
C*NOT       ISBN 2-910015-05-X
C*NOT       nomenclature MSLIB M-NT-0-160-CN
C*NOT Nota: Les epoques J1900 et J2000 sont explicitees en
C*NOT       page 23 table 1.1
C*HST VERSION 10.0 - 18.03.99 - CREATION: M-SP-278-1-CIS
C*
C***********************************************************************
C
      implicit none
C
C------------------------------
C     PARAMETRES
C------------------------------
      integer imodr
      integer jj
      double precision sec, dtai
      double precision rmat(3,3), rmatd(3,3)
C
C-------------------------------
C     SOUS-PROGRAMMES UTILISES
C-------------------------------
C
      external mrnuta, mrobli, mrmnut
C
C
C
C-------------------------------
C     VARIABLES LOCALES
C-------------------------------
C
C     date en echelle TAI
      double precision secai
      integer jjai
C     nutation en longitude, nutation en obliquite, obliquite moyenne
      double precision rnutln,rnutob,roblim
C     derivees
      double precision rnderi(2,2),roderi(2)
C     indicateur pour le calcul des derivees (=1 derivees premieres)
      integer          ider
C     vecteur rotation
      double precision om1,om2,om3
C
C     code retour des routines appelees
      integer ierapp
C
C
C     . . . INITIALISATION DE L'IDENTIFICATEUR SCCS
C      
      CHARACTER*50 SCCSID
      DATA SCCSID/"%Z%  %M%  %Y%  %I%  %E%"/
C
C-------------------------------
C     INITIALISATION
C-------------------------------
C
c
      IF (.FALSE.) THEN
         SCCSID = SCCSID
      ENDIF
c
C
      ider = 1
C
C
C-------------------------------
C-------------------------------
C     Algorithme
C-------------------------------
C-------------------------------
C
C     Passage de l'echelle T a TAI:
C     (inutile de normaliser la date ainsi obtenue)
      jjai  = jj
      secai = sec + dtai
C
C
C     nutations en longitude (rnutln) et en obliquite (rnutob)
C
C
      call mrnuta(jjai,secai,ider,imodr,rnutln,rnutob,rnderi,ierapp)
C
C     obliquite moyenne et obliquite vraie
C
      call mrobli(jjai,secai,ider,imodr,roblim,roderi,ierapp)
C
C
C=======================================================================
C     CONSTRUCTION DE LA MATRICE DE CHANGEMENT DE REPERE
C     (MATRICE DE NUTATION)
C=======================================================================
C
C ----Formule issue de 12.37 de [DR1]---------
C
      call mrmnut(rnutln,rnutob,roblim,rmat,ierapp)
C
C
C=======================================================================
C CONSTRUCTION DE LA MATRICE ROTATION
C=======================================================================
C
C calcul du vecteur rotation:
C 
C ----Formule 12.42 de [DR1]---------
      om1 = roderi(1)*cos(rnutln) - roderi(1) - rnderi(2,1)
      om2 = roderi(1)*sin(rnutln)*cos(rnutob+roblim)
     &     + rnderi(1,1)*sin(rnutob+roblim)
      om3 = roderi(1)*sin(rnutln)*sin(rnutob+roblim)
     &     - rnderi(1,1)*cos(rnutob+roblim)
C
C matrice rotation
C
C ----Formule 12.43 de [DR1]---------
      rmatd(1,1)= 0.d0
      rmatd(1,2)= om3
      rmatd(1,3)=-om2
      rmatd(2,1)=-om3
      rmatd(2,2)= 0.d0
      rmatd(2,3)= om1
      rmatd(3,1)= om2
      rmatd(3,2)=-om1
      rmatd(3,3)= 0.d0
C
 6000 continue
      end
