      subroutine mrmatcvveis (imodr,jj,sec,dtu1,dtai,rmat,rmatd)
C
C***********************************************************************
C*
C*PRO MSLIB 0281
C*VER 18.03.99 VERSION 10.0
C*AUT SPEC. CNES - LE DE -
C*AUT PROG. (Mise aux normes MSLIB) CISI DIS/MSS
C*AUT (C) COPYRIGHT CNES - 1999
C*ROL THEME : Changement de systemes de reference
C*ROL Calcule la matrice de passage et la matrice rotation du repere 
C*ROL equatorial vrai a la date T au repere de Veis a la meme date T
C
C*PAR (I) imodr   : indicateur des modeles de precession et de nutation 
C*PAR                  = 1 : NEWCOMB(B1900) + WOOLARD(J1900)
C*PAR                  = 2 : LIESKE(J2000) + WAHR(J2000)
C*PAR (I) jj      :} date julienne 1950 T (non normalisee) en jour 
C*PAR (I) sec     :}                                   et secondes
C*PAR (I) dtu1    : ecart entre l'echelle de temps TU1 et l'echelle de
C*PAR               temps utilisee pour la date T
C*PAR (I) dtai    : ecart entre l'echelle de temps TAI et l'echelle de
C*PAR               temps utilisee pour la date T
C*PAR (O) rmat  : matrice de changement de repere
C*PAR (O) rmatd : matrice de rotation
C
C*NOT Note d'utilisation: 
C*NOT   - Routine interne
C*NOT   - L'appelant doit avoir verifie que: imodr = 1 ou 2
C*NOT     (pas de test effectue ici)
C
C*NOT VALEURS DU CODE RETOUR : sans objet
C*NOT DOCUMENT DE REFERENCE :
C*NOT [DR1] "Les systemes de reference utilises en astronomie"
C*NOT       de M. Chapront-Touze, G. Francou et B. Morando
C*NOT       Bureau Des Longitudes (BDL) novembre 1994
C*NOT       ISSN 1243-4272
C*NOT       ISBN 2-910015-05-X
C*NOT       nomenclature MSLIB M-NT-0-160-CN
C*NOT Nota: Les epoques B1900, J1900 et J2000 sont explicitees en
C*NOT       page 23 table 1.1
C*HST VERSION 10.0 - 18.03.99 - CREATION: M-SP-281-1-CIS
C*
C*******************************************************************
C
      implicit none
C
C     constante pour le calcul de la derivee du temps sideral modifie
C     EPS en radians par jour 
      double precision EPS
      parameter (EPS = 1.7202179573714597d-02)
C
C------------------------------
C     PARAMETRES
C------------------------------
      integer          imodr
      integer          jj
      double precision sec,dtu1,dtai
      double precision rmat(3,3), rmatd(3,3)
C
C-------------------------------
C     SOUS-PROGRAMMES UTILISES
C-------------------------------
C
      double precision mrtsid
      external mrtsidv, mrtsid
C
C
C
C-------------------------------
C     VARIABLES LOCALES
C-------------------------------
C
C     temps sideral vrai, temps sideral modifie (Veis)
      double precision tsvrai, tsveis
C
C     derivee du temps sideral vrai, du temps sideral modifie (Veis)
      double precision tsvraid, tsveisd
C
C     angle de la rotation, derivee de l'angle, cosinus, sinus
      double precision alpha,alphad,cosalpha,sinalpha
C
C     jour julien fractionnaire de calcul intermediaire
      double precision tjj
C
C     vecteur rotation
      double precision om1, om2, om3
C
C     code retour des routines appelees
      integer ierapp
C
C     Inverse du nombre de secondes par jour
      double precision unsur86400
      parameter (unsur86400 = 1.d0/86400.d0)
C
#include "mmath.h"
C
C=======================================================================
C
C     . . . INITIALISATION DE L'IDENTIFICATEUR SCCS
C      
      CHARACTER*50 SCCSID
      DATA SCCSID/"%Z%  %M%  %Y%  %I%  %E%"/
C
C-------------------------------
C     INITIALISATION
C-------------------------------
C
      IF (.FALSE.) THEN
         SCCSID = SCCSID
      ENDIF
C
      ierapp = 0
C
C-------------------------------
C-------------------------------
C     Algorithme
C-------------------------------
C-------------------------------
C
C     Passage de la date T en jour decimal:
C
      tjj = dble(jj) + sec*unsur86400 
C
C
C=======================================================================
C CALCUL DE L'ANGLE DE ROTATION ET DE SA DERIVEE
C=======================================================================
C
C   Voir chapitre 12.10 de [DR1] pour des explications
C
C   calcul du temps sideral vrai de la date T et de sa derivee
C
      call mrtsidv(imodr,jj,sec,dtu1,dtai,tsvrai,tsvraid)
C
C   calcul du temps sideral modifie (Veis) et de sa derivee
C
      tsveis = mrtsid(tjj,dtu1,ierapp)
      tsveisd = (DEUXPI + EPS)*unsur86400
C
C   DEFINITION DE L'ANGLE DE LA ROTATION:
C
      alpha = tsvrai - tsveis
C
C   DEFINITION DE LA DERIVEE DE L'ANGLE DE ROTATION
C
      alphad = tsvraid - tsveisd
C
C=======================================================================
C CONSTRUCTION DE LA MATRICE DE CHANGEMENT DE REPERE 
C=======================================================================
C
C ----Formule 12.61 de [DR1] ---------
      cosalpha = cos(alpha)
      sinalpha = sin(alpha)
C
      rmat(1,1) =  cosalpha
      rmat(1,2) =  sinalpha
      rmat(1,3) =  0.D0
      rmat(2,1) = -sinalpha
      rmat(2,2) =  cosalpha
      rmat(2,3) =  0.D0
      rmat(3,1) =  0.D0
      rmat(3,2) =  0.D0
      rmat(3,3) =  1.D0
C
C
C=======================================================================
C CONSTRUCTION DE LA MATRICE ROTATION
C=======================================================================
C
C calcul du vecteur rotation:
C 
C ----Formule 12.62 de [DR1] ---------
      om1 = 0.d0
      om2 = 0.d0
      om3 = alphad
C
C matrice rotation
C
C ----Formule 12.63 de [DR1]---------
      rmatd(1,1)= 0.d0
      rmatd(1,2)= om3
      rmatd(1,3)=-om2
      rmatd(2,1)=-om3
      rmatd(2,2)= 0.d0
      rmatd(2,3)= om1
      rmatd(3,1)= om2
      rmatd(3,2)=-om1
      rmatd(3,3)= 0.d0
C
C
 6000 continue
      end
