      subroutine mrmatj2000cm (imodr,jj,sec,dtai,rmat,rmatd)
C
C***********************************************************************
C*
C*PRO MSLIB 0282
C*VER 18.03.99 VERSION 10.0
C*AUT SPEC. CNES - LE DE -
C*AUT PROG. (Mise aux normes MSLIB) CISI DIS/MSS
C*AUT (C) COPYRIGHT CNES - 1999
C*ROL THEME : Changement de systemes de reference
C*ROL Calcule la matrice de passage et la matrice rotation du repere 
C*ROL equatorial moyen J2000 au repere equatorial moyen a la date T
C
C*PAR (I) imodr   : indicateur des modeles de precession et de nutation
C*PAR                  = 1 : J1900 (NEWCOMB + WOOLARD)
C*PAR                  = 2 : J2000 (LIESKE + WAHR)
C*PAR (I) jj      :} date julienne 1950 T (non normalisee) en jour 
C*PAR (I) sec     :}                                   et secondes
C*PAR (I) dtai    : ecart entre l'echelle de temps TAI et l'echelle de
C*PAR               temps utilisee pour la date T
C*PAR (O) rmat  : matrice de changement de repere
C*PAR (O) rmatd : matrice de rotation
C
C*NOT Note d'utilisation: 
C*NOT   - Routine interne
C*NOT   - L'appelant doit avoir verifie que: imodr = 1 ou 2
C*NOT     (pas de test effectue ici)
C
C*NOT VALEURS DU CODE RETOUR : sans objet
C*NOT DOCUMENT DE REFERENCE :
C*NOT [DR1] "Les systemes de reference utilises en astronomie"
C*NOT       de M. Chapront-Touze, G. Francou et B. Morando
C*NOT       Bureau Des Longitudes (BDL) novembre 1994
C*NOT       ISSN 1243-4272
C*NOT       ISBN 2-910015-05-X
C*NOT       nomenclature MSLIB M-NT-0-160-CN
C*NOT Nota: Les epoques J1900 et J2000 sont explicitees en
C*NOT       page 23 table 1.1
C*HST VERSION 10.0 - 18.03.99 - CREATION: M-SP-282-1-CIS
C*
C*******************************************************************
C
      implicit none
C
C------------------------------
C     PARAMETRES
C------------------------------
      integer           imodr
      double precision  sec
      double precision  rmat(3,3), rmatd(3,3)
      double precision  dtai
      integer           jj
C
C-------------------------------
C     SOUS-PROGRAMMES UTILISES
C-------------------------------
C
      external mrmatprec2000
C
C-------------------------------
C     VARIABLES LOCALES
C-------------------------------
C
C
C     difference TAI-TE
      double precision diffTaiTe
      parameter (diffTaiTe = 32.184d0)
C     date prenant en compte dtai
      double precision secai
C     date en TE
      integer jjTe
      double precision secTe
C
C
C     . . . INITIALISATION DE L'IDENTIFICATEUR SCCS
C      
      CHARACTER*50 SCCSID
      DATA SCCSID/"%Z%  %M%  %Y%  %I%  %E%"/
C
C-------------------------------
C     INITIALISATION
C-------------------------------
C
c
      IF (.FALSE.) THEN
         SCCSID = SCCSID
      ENDIF
c
C
C-------------------------------
C-------------------------------
C     Algorithme
C-------------------------------
C-------------------------------
C
C     Passage de la date T en date TE:
C     (inutile de normaliser la date ainsi obtenue)
C
C     * passage de T a TAI
C
      secai = sec + dtai
C
C     * puis passage de TAI a TE
C
      jjTe = jj
      secTe = secai + diffTaiTe
C
C=======================================================================
C CONSTRUCTION DE LA MATRICE DE CHANGEMENT DE REPERE
C  (MATRICE DE PRECESSION) ET CONSTRUCTION DE LA MATRICE ROTATION
C=======================================================================
C
      call mrmatprec2000(imodr, jjTe, secTe, rmat, rmatd)
C
C
 6000 continue
      end
