      subroutine mrmatprec2000 (imodr,jj,sec,rmatpr,rmatprd)
C
C***********************************************************************
C*
C*PRO MSLIB 0283
C*VER 18.03.99 VERSION 10.0
C*AUT SPEC. CNES - LE DE -
C*AUT PROG. (Mise aux normes MSLIB) CISI DIS/MSS
C*AUT (C) COPYRIGHT CNES - 1999
C*ROL THEME : Changement de systemes de reference
C*ROL Calcule la matrice de precession et la matrice de rotation du 
C*ROL repere equatorial moyen J2000 au repere equatorial moyen a la
C*ROL date T 
C
C*PAR (I) imodr   : indicateur des modeles de precession et de nutation
C*PAR                  = 1 : J1900 (NEWCOMB + WOOLARD)
C*PAR                  = 2 : J2000 (LIESKE + WAHR)
C*PAR (I) jj      :} date julienne 1950 T (non normalisee) en jour 
C*PAR (I) sec     :}                           et secondes (echelle TE)
C*PAR (O) rmatpr  : matrice de precession
C*PAR (O) rmatprd : matrice de rotation
C
C*NOT Note d'utilisation: 
C*NOT   - Routine interne
C*NOT   - L'appelant doit avoir verifie que: imodr = 1 ou 2
C*NOT     (pas de test effectue ici)
C
C*NOT VALEURS DU CODE RETOUR : sans objet
C*NOT DOCUMENT DE REFERENCE :
C*NOT [DR1] "Les systemes de reference utilises en astronomie"
C*NOT       de M. Chapront-Touze, G. Francou et B. Morando
C*NOT       Bureau Des Longitudes (BDL) novembre 1994
C*NOT       ISSN 1243-4272
C*NOT       ISBN 2-910015-05-X
C*NOT       nomenclature MSLIB M-NT-0-160-CN
C*NOT Nota: Les epoques J1900 et J2000 sont explicitees en
C*NOT       page 23 table 1.1
C*HST VERSION 10.0 - 18.03.99 - CREATION: M-SP-283-1-CIS
C*
C*******************************************************************
C
      implicit none
C
C=======================================================================
C
C modele de precession de Newcomb
C  PAD1,PAD2,PAD3,PAD4,PAD5,PAD6: coefficients pour le calcul de dzeta
C                                   (secondes d'arc) 
C  PAZ1,PAZ2,PAZ3,PAZ4,PAZ5,PAZ6: coefficients pour le calcul de z 
C                                   (secondes d'arc)
C  PAT1,PAT2,PAT3,PAT4,PAT5,PAT6: coefficients pour le calcul de teta
C                                   (secondes d'arc)
C
C=======================================================================
C
C modele de precession de Lieske
C  PBD1,PBD2,PBD3: coefficients pour le calcul de dzeta (secondes d'arc)
C  PBZ1,PBZ2,PBZ3: coefficients pour le calcul de z (secondes d'arc)
C  PBT1,PBT2,PBT3: coefficients pour le calcul de teta (secondes d'arc)
C
C
C=======================================================================
C
C coefficients pour le calcul des parametres du modele de precession de 
C Newcomb
      DOUBLE PRECISION PAD1,PAD2,PAD3,PAD4,PAD5,PAD6
      DOUBLE PRECISION PAZ1,PAZ2,PAZ3,PAZ4,PAZ5,PAZ6 
      DOUBLE PRECISION PAT1,PAT2,PAT3,PAT4,PAT5,PAT6
C
C-----Paragraphe 7.3 de [DR1]--------
      parameter (PAD1 = 23042.53d0)
      parameter (PAD2 = 139.73d0)
      parameter (PAD3 = 0.06d0)
      parameter (PAD4 = 30.23d0)
      parameter (PAD5 = -0.27d0)
      parameter (PAD6 = 18.0d0)
      parameter (PAZ1 = 23042.53d0)
      parameter (PAZ2 = 139.73d0)
      parameter (PAZ3 = 0.06d0)
      parameter (PAZ4 = 109.5d0)
      parameter (PAZ5 = 0.39d0)
      parameter (PAZ6 = 18.32d0)
      parameter (PAT1 = 20046.85d0)
      parameter (PAT2 = -85.33d0)
      parameter (PAT3 = -0.37d0)
      parameter (PAT4 = -42.67d0)
      parameter (PAT5 = -0.37d0)
      parameter (PAT6 = -41.80d0)
C=======================================================================
C
C coefficients pour le calcul des parametres du modele de precession de 
C Lieske
      DOUBLE PRECISION PBD1,PBD2,PBD3,PBZ1,PBZ2,PBZ3,PBT1,PBT2,PBT3
C-----Paragraphe 7.4 de [DR1]-------
      parameter (PBD1 = 23062.181d0)
      parameter (PBD2 = 30.188d0)
      parameter (PBD3 = 17.998d0)
      parameter (PBZ1 = 23062.181d0)
      parameter (PBZ2 = 109.468d0)
      parameter (PBZ3 = 18.203d0)
      parameter (PBT1 = 20043.109d0)
      parameter (PBT2 = -42.665d0)
      parameter (PBT3 = -41.833d0)
C
C
C------------------------------
C     PARAMETRES
C------------------------------
      integer           imodr
      double precision  sec
      double precision  rmatpr(3,3), rmatprd(3,3)
      integer           jj
C
C-------------------------------
C     VARIABLES LOCALES
C-------------------------------
C     t2000 = date julienne CNES au 01-01-2000 a 12h TDB~TE
C             (confondu avec TE)
      double precision t2000
      parameter (t2000 = 18262.5d0)
C
C     t1900 = date julienne CNES au 31-12-1899 a 19h31m28s TE
      double precision t1900
      parameter (t1900 = -18262.18648d0)
C
      double precision T,T2,tu,tu2,tu3,dzeta,z,teta,dzetad,zd,tetad
      double precision om1,om2,om3
      double precision dd,sd,dz,sz,dt,st
C
C
#include "mmath.h"
C
C=======================================================================
C
C Divers parametres de conversion
C
      double precision unsur86400, unsurKanJul, unsurKanTrop, srad
C
C     1 / Nombre de secondes par jour
      parameter (unsur86400 = 1.d0/86400.d0)
C
C     1 / Nombre de jours en mille ans (K = kilo = 1000; Kan = 1000 ans)
C     Juliens :
      parameter (unsurKanJul = 1.d0/365250.d0)
C     Tropiques :
      parameter (unsurKanTrop = 1.d0/365242.198781d0)
c
C     Conversion Secondes d'arc en radians
      parameter (srad = XDERA/3600.d0)
C
C=======================================================================
C
C     . . . INITIALISATION DE L'IDENTIFICATEUR SCCS
C      
      CHARACTER*50 SCCSID
      DATA SCCSID/"%Z%  %M%  %Y%  %I%  %E%"/
C
C-------------------------------
C     INITIALISATION
C-------------------------------
C
      IF (.FALSE.) THEN
         SCCSID = SCCSID
      ENDIF
C
C-------------------------------
C-------------------------------
C     Algorithme
C-------------------------------
C-------------------------------
C
C*****************************************
C  Cas du modele de precession de Newcomb
C*****************************************
C
      if (imodr.eq.1) then
C
C nombre de milliers d'annees tropiques entre le 31-12-1899 a 19h31m28s
C et  le 1-01-2000 a 12h (echelle TE):
C
C-----Formule 7.18 de [DR]------------
         T = (t2000-t1900) * unsurKanTrop
         T2 = T*T
C
C nombre de milliers d'annees tropiques entre le 1-01-2000 a 12h 
C et la date t (echelles TE):
C
C-----Formule 7.18 de [DR1]----------
         tu = ((dble(jj)-t2000) + sec*unsur86400) * unsurKanTrop
         tu2= tu * tu
         tu3= tu2* tu
C
C calcul des parametres de la precession (dzeta,z,teta)
C
C
C-----Paragraphe 7.3 de [DR1]--------
         dzeta=((PAD1 + PAD2*T + PAD3*T2)*tu + (PAD4 + PAD5*T)*tu2
     &          + PAD6*tu3)*srad
         z    =((PAZ1 + PAZ2*T + PAZ3*T2)*tu + (PAZ4 + PAZ5*T)*tu2 
     &          + PAZ6*tu3)*srad
         teta =((PAT1 + PAT2*T + PAT3*T2)*tu + (PAT4 + PAT5*T)*tu2 
     &          + PAT6*tu3)*srad
C
C derivees premieres de ces parametres
C
         dzetad=3.1d-7*unsur86400
         zd    =3.1d-7*unsur86400
         tetad =2.7d-7*unsur86400
C
      else
C
C*****************************************
C  Cas du modele de precession de Lieske
C*****************************************
C
C
C nombre de milliers d'annees juliennes entre le 1-01-2000 a 12h
C et la date t (echelles TE):
C
C-----Formule 7.19 de [DR1]----------
         tu = ((dble(jj)-t2000) + sec*unsur86400) * unsurKanJul
         tu2= tu * tu
         tu3= tu2* tu
C
C calcul des parametres de la precession (dzeta,za,teta)
C
C
C-----Paragraphe 7.4 de [DR1]-------
         dzeta= (PBD1*tu + PBD2*tu2 + PBD3*tu3) * srad
         z    = (PBZ1*tu + PBZ2*tu2 + PBZ3*tu3) * srad
         teta = (PBT1*tu + PBT2*tu2 + PBT3*tu3) * srad
C
C derivees premieres de ces parametres
C
         dzetad=3.1d-7*unsur86400
         zd    =3.1d-7*unsur86400
         tetad =2.7d-7*unsur86400
C
C
      endif
C
C=======================================================================
C CONSTRUCTION DE LA MATRICE DE PRECESSION
C=======================================================================
C
C fonctions trigonometriques des parametres de la precession:
C
      dd = cos(dzeta)
      sd = sin(dzeta)
      dz = cos(z)
      sz = sin(z)
      dt = cos(teta)
      st = sin(teta)
C
C matrice de precession:
C
C-----Formule 7.22 de [DR1]----------
      rmatpr(1,1)= -sz*sd+dz*dd*dt
      rmatpr(1,2)= -sz*dd-dz*sd*dt
      rmatpr(1,3)= -dz*st
      rmatpr(2,1)=  dz*sd+sz*dd*dt
      rmatpr(2,2)=  dz*dd-sz*sd*dt
      rmatpr(2,3)= -sz*st
      rmatpr(3,1)=  dd*st
      rmatpr(3,2)= -sd*st
      rmatpr(3,3)=  dt
C
C=======================================================================
C CONSTRUCTION DE LA MATRICE ROTATION
C=======================================================================
C
C vecteur rotation:
C
C-----Formule 12.22 de [DR1]---------
      om1=  dzetad*dz*st-tetad*sz
      om2=  dzetad*sz*st+tetad*dz
      om3= -dzetad*dt-zd
C
C matrice rotation
C
C ----Formule 12.23 de [DR1]---------
      rmatprd(1,1)= 0.d0
      rmatprd(1,2)= om3
      rmatprd(1,3)=-om2
      rmatprd(2,1)=-om3
      rmatprd(2,2)= 0.d0
      rmatprd(2,3)= om1
      rmatprd(3,1)= om2
      rmatprd(3,2)=-om1
      rmatprd(3,3)= 0.d0
C
C
 6000 continue
      end
