      subroutine mrmattvtref (uref,vref,rmat)
C
C***********************************************************************
C*
C*PRO MSLIB 0285
C*VER 18.03.99 VERSION 10.0
C*AUT SPEC. CNES - LE DE -
C*AUT PROG. (Mise aux normes MSLIB) CISI DIS/MSS
C*AUT (C) COPYRIGHT CNES - 1999
C*ROL THEME : Changement de systemes de reference
C*ROL Calcule la matrice de passage du repere terrestre vrai a la 
C*ROL date T au repere terrestre de reference
C
C*PAR (I) uref  : coordonnee U du pole vrai a la date T dans le
C*PAR             repere R
C*PAR (I) vref  : coordonnee V du pole vrai a la date T dans le
C*PAR             repere R
C*PAR (O) rmat  : matrice de changement de repere
C
C*NOT Definition du repere R: 
C*NOT   - il est contenu dans le plan tangent a la Terre au pole Nord
C*NOT     geographique
C*NOT   - axe des coordonnees U: axe selon la direction du meridien 
C*NOT     d'origine (Greenwich), et oriente positivement vers 
C*NOT     Greenwich (Londres)
C*NOT   - axe des coordonnees V: axe selon la direction perpendiculaire
C*NOT     a l'axe U, et oriente positivement vers le Canada
C
C*NOT VALEURS DU CODE RETOUR : sans objet
C*NOT DOCUMENT DE REFERENCE :
C*NOT [DR1] "Les systemes de reference utilises en astronomie"
C*NOT       de M. Chapront-Touze, G. Francou et B. Morando
C*NOT       Bureau Des Longitudes (BDL) novembre 1994
C*NOT       ISSN 1243-4272
C*NOT       ISBN 2-910015-05-X
C*NOT       nomenclature MSLIB M-NT-0-160-CN
C*NOT Remarque: Dans ce changement de repere,
C*NOT           la composition des vitesses est negligee
C*HST VERSION 10.0 - 18.03.99 - CREATION: M-SP-285-1-CIS
C*
C*******************************************************************
C
       implicit none
C
C------------------------------
C     PARAMETRES
C------------------------------
      double precision uref,vref
      double precision rmat(3,3)
C
C-------------------------------
C     VARIABLES LOCALES
C-------------------------------
C
      double precision cosu,sinu,cosv,sinv
C
C
C     . . . INITIALISATION DE L'IDENTIFICATEUR SCCS
C      
      CHARACTER*50 SCCSID
      DATA SCCSID/"%Z%  %M%  %Y%  %I%  %E%"/
C
C-------------------------------
C     INITIALISATION
C-------------------------------
c
      IF (.FALSE.) THEN
         SCCSID = SCCSID
      ENDIF
c
C
C-------------------------------
C-------------------------------
C     Algorithme
C-------------------------------
C-------------------------------
C
C=======================================================================
C CONSTRUCTION DE LA MATRICE DE CHANGEMENT DE REPERE 
C=======================================================================
C
C ----Voir figure 4.1 de [DR1]
      cosu= cos(uref)
      sinu= sin(uref)
      cosv= cos(vref)
      sinv= sin(vref)
C
C     passage du repere terrestre vrai au repere terrestre de reference
C
C ----Matrice issue de la formule 4.8 de [DR1]---------
C
      rmat(1,1)=  cosu
      rmat(1,2)=  sinu * sinv
      rmat(1,3)=  sinu * cosv
      rmat(2,1)=  0.d0
      rmat(2,2)=  cosv
      rmat(2,3)= -sinv
      rmat(3,1)= -sinu
      rmat(3,2)=  cosu * sinv
      rmat(3,3)=  cosu * cosv
C
 6000 continue
      end
