      SUBROUTINE MRNUTA(IJOURJ,RSECON,IDERIV,IMODEL,
     -                  RNUTLN,RNUTOB,RNDERI,IER)
C
C**********************************************************************
C*
C*PRO MSLIB   0093
C*VER 26.11.87 - VERSION 2.0
C*AUT SPEC. CISI INGENIERIE -- LAGARRIGUE -- 2I/SE/MMS --
C*AUT PROG. CISI INGENIERIE (MATH. ET MECA. SPATIALE)
C*AUT (C) COPYRIGHT CNES  -  1990
C*ROL PROGRAMME :NUTATIONS EN LONGITUDE ET EN OBLIQUITE
C*ROL THEME : CHANGEMENT DE SYSTEME DE REFERENCE
C*ROL CALCUL NUTATIONS EN LONGITUDE ET EN OBLIQUITE A UNE DATE DONNEE
C*ROL SUIVANT LE MODELE DE WOOLARD OU WAHR
C*PAR IJOURJ  (I)   JOUR JULIEN CNES
C*PAR RSECON  (I)   SECONDES TAI
C*PAR IDERIV  (I)   CLE DE CALCUL DES DERIVEES
C*PAR IMODEL  (I)   INDICATEUR DU MODELE DE NUTATION
C*PAR RNUTLN  (O)   NUTATION EN LONGITUDE  ( RAD )
C*PAR RNUTOB  (O)   NUTATION EN OBLIQUITE  ( RAD )
C*PAR RNDERI  (O)   TABLEAU (2*2) DES DERIVEES
C*PAR IER     (O)   CODE RETOUR
C*NOT IDERIV                 1: CALCUL  0: SINON
C*NOT IMODEL                 1: MODELE DE WOOLARD
C*NOT                        2: MODELE DE WAHR
C*PAR RNDERI        COLONNE 1 : DERIVEES PREMIERES  ( RAD / SEC )
C*NOT               COLONNE 2 : DERIVEES SECONDES   ( RAD / SEC**2)
C*NOT               LIGNE   1 : NUTATION EN LONGITUDE
C*NOT               LIGNE   2 : NUTATION EN OBLIQUITE
C*NOT CODE RETOUR           0 : RETOUR NORMAL
C*NOT                      -1 : MAUVAISE VALEUR DE IMODEL
C*NOT                      -2 : MAUVAISE VALEUR DE IDERIV
C*INF REFERENCE DU MODELE DE WOOLARD : CONNAISSANCE DES TEMPS 1967
C*INF REFERENCE DU MODELE DE WAHR    : IAU THEORY OF NUTATION 1980
C*INF MSARGF CALCULE LES ARGUMENTS FONDAMENTAUX
C*HST VERSION 2.0 - 21.09.87 -  CREATION
C*HST VERSION 10.0 - 23.02.99 - EVOLUTION IER : M-DE-093-1-CIS
C*
C***********************************************************************
C
      IMPLICIT NONE
C
C
C     -------------------------
C     DECLARATION DES PARAMETRES
C     --------------------------
      INTEGER IJOURJ, IDERIV, IMODEL, IER
      DOUBLE PRECISION RSECON, RNUTLN, RNUTOB, RNDERI(2,2)
C
C
C     -----------------------------------
C     DECLARATION DES PROCEDURES EXTERNES
C     ___________________________________
      EXTERNAL MSARGF, MUCONVJJSJJF
C
C
C     -------------------------------
C     DECLARATION DES DONNEES LOCALES
C     ________________________________
C     DATE EN JOUR JULIEN FRACTIONNAIRE
      DOUBLE PRECISION RTEMPS
C
C     INDICES DE BOUCLES
      INTEGER I,J,JMAX
C
C     ARGUMENTS ET LEURS DERIVEES
      DOUBLE PRECISION RARFON(5),RADERI(5,2)
C
C     COEFFICIENTS DE L'EXPRESSION DES NUTATIONS
      DOUBLE PRECISION RCOEFF(9)
C
C     VARIABLES DE TRAVAIL
      DOUBLE PRECISION RARGF1,RARGFP,RARGFS,RCARGF,RSARGF,RSECLN,RSECOB
      DOUBLE PRECISION RARGP2,RDCARG,RDSARG,RDERN1,RDERN2,RDERO1,RDERO2
C
C     VARIABLE INTERMEDIAIRE DE TEMPS
      DOUBLE PRECISION RSECTAI
C
C     DECLARATIONS DU TYPE DES CONSTANTES
C
C     ... PASSAGE DE 1.E-04 SECONDES D'ARC EN RADIAN
      DOUBLE PRECISION RSECRD
C
C     ... CONSTANTES UTILISEES DANS LES SYSTEMES DE REFERENCE
      DOUBLE PRECISION RCDERP
C
C     ... DIFFERENCE ENTRE LE TEMPS DES EPHEMERIDES ET LE TAI
      DOUBLE PRECISION RTDTAI
C
C     ... COEFFICIENTS DE L'EXPRESSION DES NUTATIONS
      DOUBLE PRECISION RCOEF1(9,76), RCOEF2(9,106)
C
C     . . . INITIALISATION DE L'IDENTIFICATEUR SCCS 
C      
      CHARACTER*50 SCCSID
      DATA SCCSID/"%Z%  %M%  %Y%  %I%  %E%"/
C
C     ---------------
C     INITIALISATION
C     _______________
C
C     ... MODELE DE WOOLARD
C
      DATA(RCOEF1(I,1),I=1,9)
     -   / 0.D0, 0.D0, 0.D0, 0.D0, 1.D0, -172327.D0, -173.7D0, 92100.D0,
     -     9.1D0/
      DATA(RCOEF1(I,2),I=1,9)
     -   / 0.D0, 0.D0, 0.D0, 0.D0, 2.D0,   2088.D0,   0.2D0, -904.D0,
     -      0.4D0/
      DATA(RCOEF1(I,3),I=1,9)
     -   /-2.D0, 0.D0, 2.D0, 0.D0, 1.D0,     45.D0,   0.0D0,  -24.D0, 
     -     0.0D0/
      DATA(RCOEF1(I,4),I=1,9)
     -   / 2.D0, 0.D0,-2.D0, 0.D0, 0.D0,     10.D0,   0.0D0,    0.D0, 
     -     0.0D0/
      DATA(RCOEF1(I,5),I=1,9)
     -   /-2.D0, 0.D0, 2.D0, 0.D0, 2.D0,     -3.D0,  0.0D0,     2.D0, 
     -     0.0D0/
      DATA(RCOEF1(I,6),I=1,9)
     -   / 1.D0,-1.D0, 0.D0,-1.D0, 0.D0,     -2.D0,  0.0D0,     0.D0,
     -      0.0D0/
      DATA(RCOEF1(I,7),I=1,9)
     -   / 0.D0,-2.D0, 2.D0,-2.D0, 1.D0,     -4.D0,  0.0D0,     2.D0,
     -      0.0D0/
      DATA(RCOEF1(I,8),I=1,9)
     -   / 2.D0, 0.D0,-2.D0, 0.D0, 1.D0,      0.D0,  0.0D0,     0.D0,
     -      0.0D0/
      DATA(RCOEF1(I,9),I=1,9)
     -   / 0.D0, 0.D0, 2.D0,-2.D0, 2.D0, -12729.D0, -1.3D0, 5522.D0, 
     -     -2.9D0/
      DATA(RCOEF1(I,10),I=1,9)
     -   / 0.D0, 1.D0, 0.D0, 0.D0, 0.D0,  1261.D0, -3.1D0,   00.D0, 
     -     0.0D0/
      DATA(RCOEF1(I,11),I=1,9)
     -   / 0.D0, 1.D0, 2.D0,-2.D0, 2.D0,  -497.D0,  1.2D0,  216.D0,
     -     -0.6D0/
      DATA(RCOEF1(I,12),I=1,9)
     -   / 0.D0,-1.D0, 2.D0,-2.D0, 2.D0,   214.D0, -0.5D0,  -93.D0,
     -     0.3D0/
      DATA(RCOEF1(I,13),I=1,9)
     -   / 0.D0, 0.D0, 2.D0,-2.D0, 1.D0,   124.D0,  0.1D0,  -66.D0,
     -     0.0D0/
      DATA(RCOEF1(I,14),I=1,9)
     -   / 2.D0, 0.D0, 0.D0,-2.D0, 0.D0,    45.D0,  0.0D0,    0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,15),I=1,9)
     -   / 0.D0, 0.D0, 2.D0,-2.D0, 0.D0,   -21.D0,  0.0D0,    0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,16),I=1,9)
     -   / 0.D0, 2.D0, 0.D0, 0.D0, 0.D0,    16.D0, -0.1D0,    0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,17),I=1,9)
     -   / 0.D0, 1.D0, 0.D0, 0.D0, 1.D0,   -15.D0,  0.0D0,    8.D0,
     -     0.0D0/
      DATA(RCOEF1(I,18),I=1,9)
     -   / 0.D0, 2.D0, 2.D0,-2.D0, 2.D0,   -15.D0,  0.1D0,    7.D0, 
     -    0.0D0/
      DATA(RCOEF1(I,19),I=1,9)
     -   / 0.D0,-1.D0, 0.D0, 0.D0, 1.D0,   -10.D0,  0.0D0,    5.D0,
     -     0.0D0/
      DATA(RCOEF1(I,20),I=1,9)
     -   /-2.D0, 0.D0, 0.D0, 2.D0, 1.D0,    -5.D0,  0.0D0,    3.D0, 
     -    0.0D0/
      DATA(RCOEF1(I,21),I=1,9)
     -   / 0.D0,-1.D0, 2.D0,-2.D0, 1.D0,    -5.D0, 0.0D0,     3.D0,
     -     0.0D0/
      DATA(RCOEF1(I,22),I=1,9)
     -   / 2.D0, 0.D0, 0.D0,-2.D0, 1.D0,     4.D0, 0.0D0,    -2.D0,
     -     0.0D0/
      DATA(RCOEF1(I,23),I=1,9)
     -   / 0.D0, 1.D0, 2.D0,-2.D0, 1.D0,     3.D0, 0.0D0,    -2.D0,
     -     0.0D0/
      DATA(RCOEF1(I,24),I=1,9)
     -   / 1.D0, 0.D0, 0.D0,-1.D0, 0.D0,    -3.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,25),I=1,9)
     -   / 2.D0, 1.D0, 0.D0,-2.D0, 0.D0,     0.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,26),I=1,9)
     -   / 0.D0, 0.D0,-2.D0, 2.D0, 1.D0,     0.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,27),I=1,9)
     -   / 0.D0, 1.D0,-2.D0, 2.D0, 0.D0,     0.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,28),I=1,9)
     -   / 0.D0, 1.D0, 0.D0, 0.D0, 2.D0,     0.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,29),I=1,9)
     -   /-1.D0, 0.D0, 0.D0, 1.D0, 1.D0,     0.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,30),I=1,9)
     -   / 0.D0, 1.D0, 2.D0,-2.D0, 0.D0,     0.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,31),I=1,9)
     -   / 0.D0, 0.D0, 2.D0, 0.D0, 2.D0, -2037.D0,-0.2D0,   884.D0,
     -    -0.5D0/
      DATA(RCOEF1(I,32),I=1,9)
     -   / 1.D0, 0.D0, 0.D0, 0.D0, 0.D0,   675.D0, 0.1D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,33),I=1,9)
     -   / 0.D0, 0.D0, 2.D0, 0.D0, 1.D0,  -342.D0,-0.4D0,   183.D0,
     -     0.0D0/
      DATA(RCOEF1(I,34),I=1,9)
     -   / 1.D0, 0.D0, 2.D0, 0.D0, 2.D0,  -261.D0, 0.0D0,   113.D0,
     -    -0.1D0/
      DATA(RCOEF1(I,35),I=1,9)
     -   / 1.D0, 0.D0, 0.D0,-2.D0, 0.D0,  -149.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,36),I=1,9)
     -   /-1.D0, 0.D0, 2.D0, 0.D0, 2.D0,   114.D0, 0.0D0,   -50.D0,
     -     0.0D0/
      DATA(RCOEF1(I,37),I=1,9)
     -   / 0.D0, 0.D0, 0.D0, 2.D0, 0.D0,    60.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,38),I=1,9)
     -   / 1.D0, 0.D0, 0.D0, 0.D0, 1.D0,    58.D0, 0.0D0,   -31.D0,
     -     0.0D0/
      DATA(RCOEF1(I,39),I=1,9)
     -   /-1.D0, 0.D0, 0.D0, 0.D0, 1.D0,   -57.D0, 0.0D0,    30.D0,
     -     0.0D0/
      DATA(RCOEF1(I,40),I=1,9)
     -   /-1.D0, 0.D0, 2.D0, 2.D0, 2.D0,   -52.D0, 0.0D0,    22.D0,
     -     0.0D0/
      DATA(RCOEF1(I,41),I=1,9)
     -   / 1.D0, 0.D0, 2.D0, 0.D0, 1.D0,   -44.D0,0.0D0,     23.D0,
     -     0.0D0/
      DATA(RCOEF1(I,42),I=1,9)
     -   / 0.D0, 0.D0, 2.D0, 2.D0, 2.D0,   -32.D0, 0.0D0,    14.D0,
     -     0.0D0/
      DATA(RCOEF1(I,43),I=1,9)
     -   / 2.D0, 0.D0, 0.D0, 0.D0, 0.D0,    28.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,44),I=1,9)
     -   / 1.D0, 0.D0, 2.D0,-2.D0, 2.D0,    26.D0, 0.0D0,   -11.D0,
     -     0.0D0/
      DATA(RCOEF1(I,45),I=1,9)
     -   / 2.D0, 0.D0, 2.D0, 0.D0, 2.D0,   -26.D0, 0.0D0,    11.D0,
     -     0.0D0/
      DATA(RCOEF1(I,46),I=1,9)
     -   / 0.D0, 0.D0, 2.D0, 0.D0, 0.D0,    25.D0, 0.0D0,     0.D0, 
     -    0.0D0/
      DATA(RCOEF1(I,47),I=1,9)
     -   /-1.D0, 0.D0, 2.D0, 0.D0, 1.D0,    19.D0, 0.0D0,   -10.D0,
     -     0.0D0/
      DATA(RCOEF1(I,48),I=1,9)
     -   /-1.D0, 0.D0, 0.D0, 2.D0, 1.D0,    14.D0, 0.0D0,    -7.D0,
     -     0.0D0/
      DATA(RCOEF1(I,49),I=1,9)
     -   / 1.D0, 0.D0, 0.D0,-2.D0, 1.D0,   -13.D0, 0.0D0,     7.D0,
     -     0.0D0/
      DATA(RCOEF1(I,50),I=1,9)
     -   /-1.D0, 0.D0, 2.D0, 2.D0, 1.D0,   - 9.D0, 0.0D0,     5.D0,
     -     0.0D0/
      DATA(RCOEF1(I,51),I=1,9)
     -   / 1.D0, 1.D0, 0.D0,-2.D0, 0.D0,    -7.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,52),I=1,9)
     -   / 0.D0, 1.D0, 2.D0, 0.D0, 2.D0,     7.D0, 0.0D0,    -3.D0,
     -     0.0D0/
      DATA(RCOEF1(I,53),I=1,9)
     -   / 0.D0,-1.D0, 2.D0, 0.D0, 2.D0,    -6.D0, 0.0D0,     3.D0,
     -     0.0D0/
      DATA(RCOEF1(I,54),I=1,9)
     -   / 1.D0, 0.D0, 2.D0, 2.D0, 2.D0,    -6.D0, 0.0D0,     3.D0,
     -     0.0D0/
      DATA(RCOEF1(I,55),I=1,9)
     -   / 1.D0, 0.D0, 0.D0, 2.D0, 0.D0,     6.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,56),I=1,9)
     -   / 2.D0, 0.D0, 2.D0,-2.D0, 2.D0,     6.D0, 0.0D0,    -2.D0,
     -     0.0D0/
      DATA(RCOEF1(I,57),I=1,9)
     -   / 0.D0, 0.D0, 0.D0, 2.D0, 1.D0,    -6.D0, 0.0D0,     3.D0,
     -     0.0D0/
      DATA(RCOEF1(I,58),I=1,9)
     -   / 0.D0, 0.D0, 2.D0, 2.D0, 1.D0,    -5.D0, 0.0D0,     3.D0, 
     -    0.0D0/
      DATA(RCOEF1(I,59),I=1,9)
     -   / 1.D0, 0.D0, 2.D0,-2.D0, 1.D0,     5.D0, 0.0D0,    -3.D0, 
     -    0.0D0/
      DATA(RCOEF1(I,60),I=1,9)
     -   / 0.D0, 0.D0, 0.D0,-2.D0, 1.D0,    -5.D0, 0.0D0,     3.D0, 
     -    0.0D0/
      DATA(RCOEF1(I,61),I=1,9)
     -   / 1.D0,-1.D0, 0.D0, 0.D0, 0.D0,     4.D0, 0.0D0,     0.D0, 
     -    0.0D0/
      DATA(RCOEF1(I,62),I=1,9)
     -   / 2.D0, 0.D0, 2.D0, 0.D0, 1.D0,    -4.D0, 0.0D0,     2.D0, 
     -    0.0D0/
      DATA(RCOEF1(I,63),I=1,9)
     -   / 0.D0, 1.D0, 0.D0,-2.D0, 0.D0,    -4.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,64),I=1,9)
     -   / 1.D0, 0.D0,-2.D0, 0.D0, 0.D0,     4.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,65),I=1,9)
     -   / 0.D0, 0.D0, 0.D0, 1.D0, 0.D0,    -4.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,66),I=1,9)
     -   / 1.D0, 1.D0, 0.D0, 0.D0, 0.D0,    -3.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,67),I=1,9)
     -   / 1.D0, 0.D0, 2.D0, 0.D0, 0.D0,     3.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,68),I=1,9)
     -   / 1.D0,-1.D0, 2.D0, 0.D0, 2.D0,    -3.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,69),I=1,9)
     -   /-1.D0,-1.D0, 2.D0, 2.D0, 2.D0,    -2.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,70),I=1,9)
     -   /-2.D0, 0.D0, 0.D0, 0.D0, 1.D0,    -2.D0, 0.0D0,     0.D0, 
     -    0.0D0/
      DATA(RCOEF1(I,71),I=1,9)
     -   / 3.D0, 0.D0, 2.D0, 0.D0, 2.D0,    -2.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,72),I=1,9)
     -   / 0.D0,-1.D0, 2.D0, 2.D0, 2.D0,    -2.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,73),I=1,9)
     -   / 1.D0, 1.D0, 2.D0, 0.D0, 2.D0,     2.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,74),I=1,9)
     -   /-1.D0, 0.D0, 2.D0,-2.D0, 1.D0,    -2.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,75),I=1,9)
     -   / 2.D0, 0.D0, 0.D0, 0.D0, 1.D0,     2.D0, 0.0D0,     0.D0,
     -     0.0D0/
      DATA(RCOEF1(I,76),I=1,9)
     -   / 1.D0, 0.D0, 0.D0, 0.D0, 2.D0,    -2.D0, 0.0D0,     0.D0,
     -     0.0D0/
C
C     ... MODELE DE WAHR
C
C
      DATA(RCOEF2(I,1),I=1,9)
     -   / 0.D0, 0.D0, 0.D0, 0.D0, 1.D0,-171996.D0,-174.2D0,92025.D0,
     -    8.9D0/
      DATA(RCOEF2(I,2),I=1,9)
     -   /  0.D0, 0.D0, 0.D0, 0.D0, 2.D0,   2062.D0, 0.2D0,-895.D0,
     -    0.5D0/
      DATA(RCOEF2(I,3),I=1,9)
     -   / -2.D0, 0.D0, 2.D0, 0.D0, 1.D0,     46.D0, 0.0D0, -24.D0,
     -    0.0D0/
      DATA(RCOEF2(I,4),I=1,9)
     -   /  2.D0, 0.D0,-2.D0, 0.D0, 0.D0,     11.D0, 0.0D0,   0.D0,
     -    0.0D0/
      DATA(RCOEF2(I,5),I=1,9)
     -   / -2.D0, 0.D0, 2.D0, 0.D0, 2.D0,     -3.D0, 0.0D0,   1.D0,
     -    0.0D0/
      DATA(RCOEF2(I,6),I=1,9)
     -   /  1.D0,-1.D0, 0.D0,-1.D0, 0.D0,     -3.D0, 0.0D0,   0.D0,
     -    0.0D0/
      DATA(RCOEF2(I,7),I=1,9)
     -   /  0.D0,-2.D0, 2.D0,-2.D0, 1.D0,     -2.D0, 0.0D0,   1.D0,
     -    0.0D0/
      DATA(RCOEF2(I,8),I=1,9)
     -   /  2.D0, 0.D0,-2.D0, 0.D0, 1.D0,      1.D0, 0.0D0,   0.D0,
     -    0.0D0/
      DATA(RCOEF2(I,9),I=1,9)
     -   /  0.D0, 0.D0, 2.D0,-2.D0, 2.D0,-13187.D0,-1.6D0,5736.D0,
     -   -3.1D0/
      DATA(RCOEF2(I,10),I=1,9)
     -   / 0.D0, 1.D0, 0.D0, 0.D0, 0.D0,  1426.D0,-3.4D0,  54.D0,-0.1D0/
      DATA(RCOEF2(I,11),I=1,9)
     -   / 0.D0, 1.D0, 2.D0,-2.D0, 2.D0,  -517.D0, 1.2D0, 224.D0,-0.6D0/
      DATA(RCOEF2(I,12),I=1,9)
     -   / 0.D0,-1.D0, 2.D0,-2.D0, 2.D0,   217.D0,-0.5D0, -95.D0, 0.3D0/
      DATA(RCOEF2(I,13),I=1,9)
     -   / 0.D0, 0.D0, 2.D0,-2.D0, 1.D0,   129.D0, 0.1D0, -70.D0, 0.0D0/
      DATA(RCOEF2(I,14),I=1,9)
     -   / 2.D0, 0.D0, 0.D0,-2.D0, 0.D0,    48.D0, 0.0D0,   1.D0, 0.0D0/
      DATA(RCOEF2(I,15),I=1,9)
     -   / 0.D0, 0.D0, 2.D0,-2.D0, 0.D0,   -22.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,16),I=1,9)
     -   / 0.D0, 2.D0, 0.D0, 0.D0, 0.D0,    17.D0,-0.1D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,17),I=1,9)
     -   / 0.D0, 1.D0, 0.D0, 0.D0, 1.D0,   -15.D0, 0.0D0,   9.D0, 0.0D0/
      DATA(RCOEF2(I,18),I=1,9)
     -   / 0.D0, 2.D0, 2.D0,-2.D0, 2.D0,   -16.D0,  .1D0,   7.D0, 0.0D0/
      DATA(RCOEF2(I,19),I=1,9)
     -   / 0.D0,-1.D0, 0.D0, 0.D0, 1.D0,   -12.D0, 0.0D0,   6.D0, 0.0D0/
      DATA(RCOEF2(I,20),I=1,9)
     -   /-2.D0, 0.D0, 0.D0, 2.D0, 1.D0,    -6.D0, 0.0D0,   3.D0, 0.0D0/
      DATA(RCOEF2(I,21),I=1,9)
     -   / 0.D0,-1.D0, 2.D0,-2.D0, 1.D0,    -5.D0, 0.0D0,   3.D0, 0.0D0/
      DATA(RCOEF2(I,22),I=1,9)
     -   / 2.D0, 0.D0, 0.D0,-2.D0, 1.D0,     4.D0, 0.0D0,  -2.D0, 0.0D0/
      DATA(RCOEF2(I,23),I=1,9)
     -   / 0.D0, 1.D0, 2.D0,-2.D0, 1.D0,     4.D0, 0.0D0,  -2.D0, 0.0D0/
      DATA(RCOEF2(I,24),I=1,9)
     -   / 1.D0, 0.D0, 0.D0,-1.D0, 0.D0,    -4.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,25),I=1,9)
     -   / 2.D0, 1.D0, 0.D0,-2.D0, 0.D0,     1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,26),I=1,9)
     -   / 0.D0, 0.D0,-2.D0, 2.D0, 1.D0,     1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,27),I=1,9)
     -   / 0.D0, 1.D0,-2.D0, 2.D0, 0.D0,    -1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,28),I=1,9)
     -   / 0.D0, 1.D0, 0.D0, 0.D0, 2.D0,     1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,29),I=1,9)
     -   /-1.D0, 0.D0, 0.D0, 1.D0, 1.D0,     1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,30),I=1,9)
     -   / 0.D0, 1.D0, 2.D0,-2.D0, 0.D0,    -1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,31),I=1,9)
     -   / 0.D0, 0.D0, 2.D0, 0.D0, 2.D0, -2274.D0,-0.2D0, 977.D0,-0.5D0/
      DATA(RCOEF2(I,32),I=1,9)
     -   / 1.D0, 0.D0, 0.D0, 0.D0, 0.D0,   712.D0, 0.1D0,  -7.D0, 0.0D0/
      DATA(RCOEF2(I,33),I=1,9)
     -   / 0.D0, 0.D0, 2.D0, 0.D0, 1.D0,  -386.D0,-0.4D0, 200.D0, 0.0D0/
      DATA(RCOEF2(I,34),I=1,9)
     -   / 1.D0, 0.D0, 2.D0, 0.D0, 2.D0,  -301.D0, 0.0D0, 129.D0,-0.1D0/
      DATA(RCOEF2(I,35),I=1,9)
     -   / 1.D0, 0.D0, 0.D0,-2.D0, 0.D0,  -158.D0, 0.0D0,  -1.D0, 0.0D0/
      DATA(RCOEF2(I,36),I=1,9)
     -   /-1.D0, 0.D0, 2.D0, 0.D0, 2.D0,   123.D0, 0.0D0, -53.D0, 0.0D0/
      DATA(RCOEF2(I,37),I=1,9)
     -   / 0.D0, 0.D0, 0.D0, 2.D0, 0.D0,    63.D0, 0.0D0,  -2.D0, 0.0D0/
      DATA(RCOEF2(I,38),I=1,9)
     -   / 1.D0, 0.D0, 0.D0, 0.D0, 1.D0,    63.D0, 0.1D0, -33.D0, 0.0D0/
      DATA(RCOEF2(I,39),I=1,9)
     -   /-1.D0, 0.D0, 0.D0, 0.D0, 1.D0,   -58.D0,-0.1D0,  32.D0, 0.0D0/
      DATA(RCOEF2(I,40),I=1,9)
     -   /-1.D0, 0.D0, 2.D0, 2.D0, 2.D0,   -59.D0, 0.0D0,  26.D0, 0.0D0/
      DATA(RCOEF2(I,41),I=1,9)
     -   / 1.D0, 0.D0, 2.D0, 0.D0, 1.D0,   -51.D0,0.0D0,   27.D0, 0.0D0/
      DATA(RCOEF2(I,42),I=1,9)
     -   / 0.D0, 0.D0, 2.D0, 2.D0, 2.D0,   -38.D0, 0.0D0,  16.D0, 0.0D0/
      DATA(RCOEF2(I,43),I=1,9)
     -   / 2.D0, 0.D0, 0.D0, 0.D0, 0.D0,    29.D0, 0.0D0,  -1.D0, 0.0D0/
      DATA(RCOEF2(I,44),I=1,9)
     -   / 1.D0, 0.D0, 2.D0,-2.D0, 2.D0,    29.D0, 0.0D0, -12.D0, 0.0D0/
      DATA(RCOEF2(I,45),I=1,9)
     -   / 2.D0, 0.D0, 2.D0, 0.D0, 2.D0,   -31.D0, 0.0D0,  13.D0, 0.0D0/
      DATA(RCOEF2(I,46),I=1,9)
     -   / 0.D0, 0.D0, 2.D0, 0.D0, 0.D0,    26.D0, 0.0D0,  -1.D0, 0.0D0/
      DATA(RCOEF2(I,47),I=1,9)
     -   /-1.D0, 0.D0, 2.D0, 0.D0, 1.D0,    21.D0, 0.0D0, -10.D0, 0.0D0/
      DATA(RCOEF2(I,48),I=1,9)
     -   /-1.D0, 0.D0, 0.D0, 2.D0, 1.D0,    16.D0, 0.0D0,  -8.D0, 0.0D0/
      DATA(RCOEF2(I,49),I=1,9)
     -   / 1.D0, 0.D0, 0.D0,-2.D0, 1.D0,   -13.D0, 0.0D0,   7.D0, 0.0D0/
      DATA(RCOEF2(I,50),I=1,9)
     -   /-1.D0, 0.D0, 2.D0, 2.D0, 1.D0,   -10.D0, 0.0D0,   5.D0, 0.0D0/
      DATA(RCOEF2(I,51),I=1,9)
     -   / 1.D0, 1.D0, 0.D0,-2.D0, 0.D0,    -7.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,52),I=1,9)
     -   / 0.D0, 1.D0, 2.D0, 0.D0, 2.D0,     7.D0, 0.0D0,  -3.D0, 0.0D0/
      DATA(RCOEF2(I,53),I=1,9)
     -   / 0.D0,-1.D0, 2.D0, 0.D0, 2.D0,    -7.D0, 0.0D0,   3.D0, 0.0D0/
      DATA(RCOEF2(I,54),I=1,9)
     -   / 1.D0, 0.D0, 2.D0, 2.D0, 2.D0,    -8.D0, 0.0D0,   3.D0, 0.0D0/
      DATA(RCOEF2(I,55),I=1,9)
     -   / 1.D0, 0.D0, 0.D0, 2.D0, 0.D0,     6.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,56),I=1,9)
     -   / 2.D0, 0.D0, 2.D0,-2.D0, 2.D0,     6.D0, 0.0D0,  -3.D0, 0.0D0/
      DATA(RCOEF2(I,57),I=1,9)
     -   / 0.D0, 0.D0, 0.D0, 2.D0, 1.D0,    -6.D0, 0.0D0,   3.D0, 0.0D0/
      DATA(RCOEF2(I,58),I=1,9)
     -   / 0.D0, 0.D0, 2.D0, 2.D0, 1.D0,    -7.D0, 0.0D0,   3.D0, 0.0D0/
      DATA(RCOEF2(I,59),I=1,9)
     -   / 1.D0, 0.D0, 2.D0,-2.D0, 1.D0,     6.D0, 0.0D0,  -3.D0, 0.0D0/
      DATA(RCOEF2(I,60),I=1,9)
     -   / 0.D0, 0.D0, 0.D0,-2.D0, 1.D0,    -5.D0, 0.0D0,   3.D0, 0.0D0/
      DATA(RCOEF2(I,61),I=1,9)
     -   / 1.D0,-1.D0, 0.D0, 0.D0, 0.D0,     5.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,62),I=1,9)
     -   / 2.D0, 0.D0, 2.D0, 0.D0, 1.D0,    -5.D0, 0.0D0,   3.D0, 0.0D0/
      DATA(RCOEF2(I,63),I=1,9)
     -   / 0.D0, 1.D0, 0.D0,-2.D0, 0.D0,    -4.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,64),I=1,9)
     -   / 1.D0, 0.D0,-2.D0, 0.D0, 0.D0,     4.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,65),I=1,9)
     -   / 0.D0, 0.D0, 0.D0, 1.D0, 0.D0,    -4.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,66),I=1,9)
     -   / 1.D0, 1.D0, 0.D0, 0.D0, 0.D0,    -3.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,67),I=1,9)
     -   / 1.D0, 0.D0, 2.D0, 0.D0, 0.D0,     3.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,68),I=1,9)
     -   / 1.D0,-1.D0, 2.D0, 0.D0, 2.D0,    -3.D0, 0.0D0,   1.D0, 0.0D0/
      DATA(RCOEF2(I,69),I=1,9)
     -   /-1.D0,-1.D0, 2.D0, 2.D0, 2.D0,    -3.D0, 0.0D0,   1.D0, 0.0D0/
      DATA(RCOEF2(I,70),I=1,9)
     -   /-2.D0, 0.D0, 0.D0, 0.D0, 1.D0,    -2.D0, 0.0D0,   1.D0, 0.0D0/
      DATA(RCOEF2(I,71),I=1,9)
     -   / 3.D0, 0.D0, 2.D0, 0.D0, 2.D0,    -3.D0, 0.0D0,   1.D0, 0.0D0/
      DATA(RCOEF2(I,72),I=1,9)
     -   / 0.D0,-1.D0, 2.D0, 2.D0, 2.D0,    -3.D0, 0.0D0,   1.D0, 0.0D0/
      DATA(RCOEF2(I,73),I=1,9)
     -   / 1.D0, 1.D0, 2.D0, 0.D0, 2.D0,     2.D0, 0.0D0,  -1.D0, 0.0D0/
      DATA(RCOEF2(I,74),I=1,9)
     -   /-1.D0, 0.D0, 2.D0,-2.D0, 1.D0,    -2.D0, 0.0D0,   1.D0, 0.0D0/
      DATA(RCOEF2(I,75),I=1,9)
     -   / 2.D0, 0.D0, 0.D0, 0.D0, 1.D0,     2.D0, 0.0D0,  -1.D0, 0.0D0/
      DATA(RCOEF2(I,76),I=1,9)
     -   / 1.D0, 0.D0, 0.D0, 0.D0, 2.D0,    -2.D0, 0.0D0,   1.D0, 0.0D0/
      DATA(RCOEF2(I,77),I=1,9)
     -   / 3.D0, 0.D0, 0.D0, 0.D0, 0.D0,     2.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,78),I=1,9)
     -   / 0.D0, 0.D0, 2.D0, 1.D0, 2.D0,     2.D0, 0.0D0,  -1.D0, 0.0D0/
      DATA(RCOEF2(I,79),I=1,9)
     -   /-1.D0, 0.D0, 0.D0, 0.D0, 2.D0,     1.D0, 0.0D0,  -1.D0, 0.0D0/
      DATA(RCOEF2(I,80),I=1,9)
     -   / 1.D0, 0.D0, 0.D0,-4.D0, 0.D0,    -1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,81),I=1,9)
     -   /-2.D0, 0.D0, 2.D0, 2.D0, 2.D0,     1.D0, 0.0D0,  -1.D0, 0.0D0/
      DATA(RCOEF2(I,82),I=1,9)
     -   /-1.D0, 0.D0, 2.D0, 4.D0, 2.D0,    -2.D0, 0.0D0,   1.D0, 0.0D0/
      DATA(RCOEF2(I,83),I=1,9)
     -   / 2.D0, 0.D0, 0.D0,-4.D0, 0.D0,    -1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,84),I=1,9)
     -   / 1.D0, 1.D0, 2.D0,-2.D0, 2.D0,     1.D0, 0.0D0,  -1.D0, 0.0D0/
      DATA(RCOEF2(I,85),I=1,9)
     -   / 1.D0, 0.D0, 2.D0, 2.D0, 1.D0,    -1.D0, 0.0D0,   1.D0, 0.0D0/
      DATA(RCOEF2(I,86),I=1,9)
     -   /-2.D0, 0.D0, 2.D0, 4.D0, 2.D0,    -1.D0, 0.0D0,   1.D0, 0.0D0/
      DATA(RCOEF2(I,87),I=1,9)
     -   /-1.D0, 0.D0, 4.D0, 0.D0, 2.D0,     1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,88),I=1,9)
     -   / 1.D0,-1.D0, 0.D0,-2.D0, 0.D0,     1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,89),I=1,9)
     -   / 2.D0, 0.D0, 2.D0,-2.D0, 1.D0,     1.D0, 0.0D0,  -1.D0, 0.0D0/
      DATA(RCOEF2(I,90),I=1,9)
     -   / 2.D0, 0.D0, 2.D0, 2.D0, 2.D0,    -1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,91),I=1,9)
     -   / 1.D0, 0.D0, 0.D0, 2.D0, 1.D0,    -1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,92),I=1,9)
     -   / 0.D0, 0.D0, 4.D0,-2.D0, 2.D0,     1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,93),I=1,9)
     -   / 3.D0, 0.D0, 2.D0,-2.D0, 2.D0,     1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,94),I=1,9)
     -   / 1.D0, 0.D0, 2.D0,-2.D0, 0.D0,    -1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,95),I=1,9)
     -   / 0.D0, 1.D0, 2.D0, 0.D0, 1.D0,     1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,96),I=1,9)
     -   /-1.D0,-1.D0, 0.D0, 2.D0, 1.D0,     1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,97),I=1,9)
     -   / 0.D0, 0.D0,-2.D0, 0.D0, 1.D0,    -1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,98),I=1,9)
     -   / 0.D0, 0.D0, 2.D0,-1.D0, 2.D0,    -1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,99),I=1,9)
     -   / 0.D0, 1.D0, 0.D0, 2.D0, 0.D0,    -1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,100),I=1,9)
     -   / 1.D0, 0.D0,-2.D0,-2.D0, 0.D0,   -1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,101),I=1,9)
     -   / 0.D0,-1.D0, 2.D0, 0.D0, 1.D0,   -1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,102),I=1,9)
     -   / 1.D0, 1.D0, 0.D0,-2.D0, 1.D0,   -1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,103),I=1,9)
     -   / 1.D0, 0.D0,-2.D0, 2.D0, 0.D0,   -1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,104),I=1,9)
     -   / 2.D0, 0.D0, 0.D0, 2.D0, 0.D0,    1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,105),I=1,9)
     -   / 0.D0, 0.D0, 2.D0, 4.D0, 2.D0,   -1.D0, 0.0D0,   0.D0, 0.0D0/
      DATA(RCOEF2(I,106),I=1,9)
     -   / 0.D0, 1.D0, 0.D0, 1.D0, 0.D0,    1.D0, 0.0D0,   0.D0, 0.0D0/
C
C
C     ... PASSAGE DE 1.E-04 SECONDES D'ARC EN RADIANS
      DATA RSECRD / .48481368110953D-09 /
C
C     ... CONSTANTE UTILISEE DANS LES SYSTEMES DE REFERENCE
      DATA RCDERP / 3.168808781403D-10 /
C
C     ... DIFFERENCE ENTRE LE TEMPS DES EPHEMERIDES ET LE TAI
      DATA RTDTAI / 32.184D0 /
C
C     ******************
C     DEBUT DU PROGRAMME
C     ******************
C
      IF (.FALSE.) THEN
         SCCSID = SCCSID
      ENDIF
C
C
      IER = 0
C
C
C     ==================================
C*FON VERIFICATION DES DONNEES EN ENTREE
C     ==================================
C
      IF ( (IMODEL.NE.1).AND.(IMODEL.NE.2) ) THEN
         IER = -1
         GO TO 6000
      ENDIF
C
      IF ( (IDERIV.NE.1).AND.(IDERIV.NE.0) ) THEN
         IER = -2
         GO TO 6000
      ENDIF
C
C
C     ========================
C*FON ARGUMENTS FONDAMENTAUX
C     ========================
C
      CALL MSARGF(IJOURJ,RSECON,IDERIV,IMODEL,RARFON,RADERI,IER)
C
C     IL N'Y A PAS DE CODE RETOUR NEGATIF POSSIBLE
C
C
C     ==========================================================
C*FON DATE EN SIECLES JULIENS
C*FON CONVERSION SECONDES TAI EN TEMPS DYNAMIQUE TERRESTRE (TDT)
C     ==========================================================
C
      RSECTAI = RSECON + RTDTAI
      CALL MUCONVJJSJJF(IJOURJ, RSECTAI, RTEMPS, IER)
C
C     IL N'Y A PAS DE CODE RETOUR NEGATIF OU POSITIF POSSIBLE
C
      RTEMPS = RTEMPS / 36525.D0
C
      IF (IMODEL.EQ.1) THEN
         RTEMPS = RTEMPS + 0.5D0
         JMAX   = 76
      ELSE
         RTEMPS = RTEMPS - 0.5D0
         JMAX   = 106
      ENDIF
C
C
C     ===========================
C*FON NUTATIONS ET LEURS DERIVEES
C     ===========================
C
C     ------------------
C     INITIALISATIONS
C     __________________
C
      RNUTLN = 0.D0
      RNUTOB = 0.D0
C
      DO 100 I = 1,2
      DO 100 J = 1,2
         RNDERI(I,J) = 0.D0
100   CONTINUE
C
C     ---------------------------------
C     BOUCLE SUR LES LIGNES DE LA TABLE
C     _________________________________
C
      DO 200 J = 1,JMAX
C
C     ... TRANSFERT DES COEFFICENTS DE LA TABLE
C         DANS VARIABLES DE TRAVAIL
      DO 210 I = 1,9
C
         IF (IMODEL.EQ.1) THEN
            RCOEFF(I) = RCOEF1(I,J)
         ELSE
            RCOEFF(I) = RCOEF2(I,J)
         ENDIF
C
210   CONTINUE
C
C     ... SOMMATION DES ARGUMENTS ET DE LEURS DERIVEES
C
      RARGF1 = 0.D0
      RARGFP = 0.D0
      RARGFS = 0.D0
C
      DO 220 I = 1,5
C
      RARGF1 = RARGF1 + RARFON(I)*RCOEFF(I)
C
      IF (IDERIV.EQ.1) THEN
         RARGFP = RARGFP + RADERI(I,1)*RCOEFF(I)
         RARGFS = RARGFS + RADERI(I,2)*RCOEFF(I)
      ENDIF
C
220   CONTINUE
C
C     ... NUTATIONS
C
      RCARGF = COS(RARGF1)
      RSARGF = SIN(RARGF1)
C
      RSECLN = RCOEFF(6) + RCOEFF(7)*RTEMPS
      RSECOB = RCOEFF(8) + RCOEFF(9)*RTEMPS
C
      RNUTLN = RNUTLN + RSARGF*RSECLN
      RNUTOB = RNUTOB + RCARGF*RSECOB
C
      IF (IDERIV.EQ.1) THEN
C
C     ... DERIVEES DES NUTATIONS
C
C
         RARGP2=   RARGFP*RARGFP
         RDCARG = -RSARGF*RARGFP
         RDSARG =  RCARGF*RARGFP
C
C     DERIVEE PREMIERE DE LA NUTATION EN LONGITUDE
         RDERN1      = RDSARG*RSECLN + RSARGF*RCOEFF(7)*RCDERP
         RNDERI(1,1) = RNDERI(1,1) + RDERN1
C
C     DERIVEE PREMIERE DE LA NUTATION EN OBLIQUITE
         RDERO1      = RDCARG*RSECOB + RCARGF*RCOEFF(9)*RCDERP
         RNDERI(2,1) = RNDERI(2,1) + RDERO1
C
C     DERIVEE SECONDE DE LA NUTATION EN LONGITUDE
         RDERN2      = ( - RSARGF*RARGP2 + RCARGF*RARGFS ) * RSECLN
         RDERN2      = RDERN2 + 2.D0*RDSARG*RCOEFF(7)*RCDERP
         RNDERI(1,2) = RNDERI(1,2) + RDERN2
C
C     DERIVEE SECONDE DE LA NUTATION EN OBLIQUITE
         RDERO2      = -( RCARGF*RARGP2 + RSARGF*RARGFS ) * RSECOB
         RDERO2      = RDERO2 + 2.D0*RDCARG*RCOEFF(9)*RCDERP
         RNDERI(2,2) = RNDERI(2,2) + RDERO2
C
      ENDIF
C
C
200   CONTINUE
C
C     --------------------------------------
C     FIN BOUCLE SUR LES LIGNES DE LA TABLE
C     ______________________________________
C
C
C     =========================================
C*FON CONVERSION DES SECONDES D'ARC EN RADIANS
C     =========================================
C
      RNUTLN = RNUTLN*RSECRD
      RNUTOB = RNUTOB*RSECRD
C
      IF (IDERIV.EQ.1) THEN
C
         DO 300 I = 1,2
            DO 300 J = 1,2
               RNDERI(I,J) = RNDERI(I,J)*RSECRD
 300        CONTINUE
C
      ENDIF
C
C     ****************
C     FIN DE PROGRAMME
C     ****************
C
 6000 CONTINUE
      RETURN
      END
