      SUBROUTINE MROBLI(IJOURJ,RSECON,IDERIV,IMODEL,ROBLIM,RODERI,IER)
C
C**********************************************************************
C*
C*PRO MSLIB  0095
C*VER 26.11.87 - VERSION 2.0
C*AUT SPEC. CISI INGENIERIE -- LAGARRIGUE -- 2I/SE/MMS --
C*AUT PROG. CISI INGENIERIE (MATH. ET MECA. SPATIALE)
C*AUT (C) COPYRIGHT CNES  -  1990
C*ROL PROGRAMME : CALCUL DE L'OBLIQUITE MOYENNE
C*ROL THEME :  CHANGEMENT DE SYSTEME DE REFERENCE
C*ROL CALCUL DE L'OBLIQUITE MOYENNE A UNE DATE DONNEE, ET DE SES
C*ROL DERIVEES, SELON LE MODELE DE NEWCOMB OU LIESKE
C*PAR IJOURJ  (I)   JOUR JULIEN CNES
C*PAR RSECON  (I)   SECONDES TAI
C*PAR IDERIV  (I)   CLE DE CALCUL DES DERIVEES
C*PAR IMODEL  (I)   INDICATEUR DU MODELE DE PRECESSION
C*PAR ROBLIM  (O)   OBLIQUITE MOYENNE  (RADIAN)
C*PAR RODERI  (O)   VECTEUR DES DERIVEES DE L'OBLIQUITE MOYENNE
C*PAR IER     (O)   CODE RETOUR
C*NOT IDERIV        1: CALCUL  0: SINON
C*NOT IMODEL        1: MODELE DE NEWCOMB
C*NOT               2: MODELE DE LIESKE
C*NOT RODERI        RODERI(1): DERIVEE PREMIERE (RAD / SEC )
C*NOT               RODERI(2): DERIVEE SECONDE  (RAD / SEC**2)
C*NOT IER           0 : RETOUR NORMAL
C*NOT              -1 : MAUVAISE VALEUR DE IMODEL
C*NOT              -2 : MAUVAISE VALEUR DE IDERIV
C*INF REFERENCE DU MODELE DE NEWCOMB : CONNAISSANCE DES TEMPS 1967
C*INF REFERENCE DU MODELE DE LIESKE   : ASTRONOMY AND ASTROPHYSICS 1976
C*HST VERSION 2.0 - 05.10.87 - CREATION
C*HST VERSION 7.0 - 24.07.92 - CORRECTION : M-FA-095-1-II
C*HST VERSION 10.0 - 23.02.99 - EVOLUTION IER : M-DE-095-1-CIS
C*
C***********************************************************************
C
      IMPLICIT NONE
C
C
C     -------------------------
C     DECLARATION DES PARAMETRES
C     --------------------------
C
      INTEGER IJOURJ, IDERIV, IMODEL, IER
      DOUBLE PRECISION RSECON, ROBLIM, RODERI(2)
C
C     -----------------------------------
C     DECLARATION DES PROCEDURES EXTERNES
C     -----------------------------------
C
      EXTERNAL MUCONVJJSJJF
C
C     -------------------------------
C     DECLARATION DES DONNEES LOCALES
C     -------------------------------
C
C     DATE EN JOUR JULIEN FRACTIONNAIRE
      DOUBLE PRECISION RTEMPS
C
C     TABLE DES COEFFICIENTS
      DOUBLE PRECISION RCOEFF(4)
C
C     VARIABLES DE TRAVAIL
      DOUBLE PRECISION ROBLI1,ROBLI2
      INTEGER I
C
C     VARIABLES INTERMEDIAIRE DE TEMPS
      DOUBLE PRECISION RSECTAI
C
C     DECLARATION DU TYPE DES CONSTANTES
C     ... CONSTANTES UTILISEES DANS LES SYSTEMES DE REFERENCES
      DOUBLE PRECISION RCDERP,RCDERS
C
C     ... DIFFERENCE ENTRE LE TEMPS DES EPHEMERIDES ET LE TAI
      DOUBLE PRECISION RTDTAI
C
C     ... TABLES DES COEFFICIENTS DE NEWCOMB ET LIESKE
      DOUBLE PRECISION RCOEF1(4),RCOEF2(4)
C
C     . . . INITIALISATION DE L'IDENTIFICATEUR SCCS 
C      
      CHARACTER*50 SCCSID
      DATA SCCSID/"%Z%  %M%  %Y%  %I%  %E%"/
C
C
C     ---------------
C     INITIALISATION
C     ---------------
C
      DATA RTDTAI / 32.184D0/
      DATA RCDERP / 3.168808781403 D-10/
      DATA RCDERS / 1.00413490931 D-19/
C
C     MODELE DE NEWCOMB
      DATA RCOEF1
     -           /0.40931975520273D0,
     -           -0.22710612077895D-03,
     -           -0.29088820866572D-07,
     -            0.88720903643045D-08/
C
C     MODELE DE LIESKE
      DATA RCOEF2
     -           /0.40909280422233D0,
     -           -0.22696552481143D-03,
     -           -0.28604007185463D-08,
     -            0.87896720385159D-08/
C
C     ******************
C     DEBUT DU PROGRAMME
C     ******************
C
      IF (.FALSE.) THEN
         SCCSID = SCCSID
      ENDIF
C
      IER = 0
C
C     ===================================
C*FON VERIFICATION DES DONNEES EN ENTREE
C     ===================================
C
      IF ( (IMODEL.NE.1).AND.(IMODEL.NE.2) ) THEN
         IER = -1
         GO TO 6000
      ENDIF
C
      IF ( (IDERIV.NE.1).AND.(IDERIV.NE.0) ) THEN
         IER = -2
         GO TO 6000
      ENDIF
C
C
C     ==========================================================
C*FON DATE EN SIECLES JULIENS
C*FON CONVERSION SECONDES TAI EN TEMPS DYNAMIQUE TERRESTRE (TDT)
C     ==========================================================
C
      RSECTAI = RSECON + RTDTAI
      CALL MUCONVJJSJJF(IJOURJ, RSECTAI, RTEMPS, IER)
C
C     IL N'Y A PAS DE CODE RETOUR NEGATIF OU POSITIF POSSIBLE
C
      RTEMPS = RTEMPS / 36525.D0
C
      IF (IMODEL.EQ.1) THEN
         RTEMPS = RTEMPS + 0.5000134D0
      ELSE
         RTEMPS = RTEMPS - 0.5D0
      ENDIF
C
C     ==================
C*FON OBLIQUITE MOYENNE
C     ==================
C
C     ... TRANSFERT DES COEFFICENTS DANS VARIABLES DE TRAVAIL
      DO 100 I = 1,4
         IF (IMODEL.EQ.1) THEN
            RCOEFF(I) = RCOEF1(I)
         ELSE
            RCOEFF(I) = RCOEF2(I)
         ENDIF
100   CONTINUE
C
      ROBLI1 = RCOEFF(4)*RTEMPS + RCOEFF(3)
      ROBLI2 = ROBLI1   *RTEMPS + RCOEFF(2)
      ROBLIM = ROBLI2   *RTEMPS + RCOEFF(1)
C
C     =========
C*FON DERIVEES
C     =========
C
      IF (IDERIV.EQ.1) THEN
C
C     ... DERIVEE PREMIERE
          ROBLI1    = 3.D0*RCOEFF(4)*RTEMPS + 2.D0*RCOEFF(3)
          RODERI(1) = ( ROBLI1*RTEMPS + RCOEFF(2) ) * RCDERP
C
C     ... DERIVEE SECONDE
          RODERI(2) = (6.D0*RCOEFF(4)*RTEMPS + 2.D0*RCOEFF(3))* RCDERS
C
      ELSE
          RODERI(1) = 0.D0
          RODERI(2) = 0.D0
      ENDIF
C
C     ****************
C     FIN DE PROGRAMME
C     ****************
C
 6000 CONTINUE
      RETURN
      END
