      DOUBLE PRECISION FUNCTION MRTSID (RDJD,RDTU1,IER)
C
C**********************************************************************
C*
C*PRO MSLIB 0023
C*VER 24.09.87 V1.0
C*AUT SPEC.  C N E S - LABRUNE - 13 MARS 86
C*AUT PORT.  CISI INGENIERIE - (MATH. ET MECA. SPATIALE)
C*AUT (C) COPYRIGHT CNES  -  1990
C*ROL PROGRAMME : MRTSID   CALCUL DU TEMPS SIDERAL (VEIS)
C*ROL THEME : CHANGEMENT DE SYSTEME DE REFERENCE
C*ROL CALCULE EN RADIANS LE TEMPS SIDERAL AYANT POUR ORIGINE LE POINT
C*ROL GAMMA, QUI SERT D'ORIGINE AU SYSTEME DE REFERENCE ADOPTE AU CNES
C*ROL POUR LES CALCULS D'ORBITE. (REPERE DE VEIS)
C*PAR RDJD   (I) : NOMBRE DE JOURS DEPUIS LE 01/01/1950 A 0 HEURE
C*PAR RDTU1  (I) : ECART (S) ENTRE TU1 ET ECHELLE DE DATATION DE RDJD
C*PAR IER    (O) : INDICATEUR DE RETOUR (0:FORCEMENT)
C*INF MUAMOD : FONCTION MATHEMATIQUE MODULO
C*HST V1.O 24.09.87 CREATION
C*HST V0.0 27.07.87 CORRECTION ANOMALIE M-FA-23-1-CN
C*HST VERSION 7.1 - 18.12.92 - EVOLUTION MCONST : M-DE-001-1-II
C*HST VERSION 7.2 - 22.02.93 - EVOLUTION : M-DE-023-1-II
C*HST VERSION 8.1 - 21.11.94 - EVOLUTION PORTAGE
C*HST VERSION 8.2 - 01.02.96 - RAJOUT DE COMMENTAIRES : M-DE-023-2-II
C*
C**********************************************************************
C
C                      DECLARATION DES PARAMETRES
C
      DOUBLE PRECISION RDJD,RDTU1
      INTEGER IER
C
C                      DECLARATION DES PROCEDURES EXTERNES
C
      EXTERNAL MUAMOD
      DOUBLE PRECISION MUAMOD
C
C                      DECLARATION DES CONSTANTES
C
      DOUBLE PRECISION A1,A3,EPS
C     CONSTANTES DE CALCUL
C
C                      DECLARATION DES VARIABLES LOCALES
C
      DOUBLE PRECISION TSID
C     TSID = TEMPS SIDERAL CALCULE
C
C
C     ------------------------------------------
C     EXPLICATIONS SUR LES CONSTANTES UTILISEES
C     ------------------------------------------
C
C     .................................................................
C     PAR ABUS DE LANGAGE, ET POUR PLUS DE CLARTE DANS LES EXPLICATIONS
C     ON UTILISERA 2.PI AVEC DES VALEURS EN DEGRES DANS LES FORMULES.
C     .................................................................
C     LA REFERENCE GEORGES VEIS - THE SYSTEM REFERENCE, 
C     IN SAO SPECIAL REPORT 200, 1ER VOLUME, 1966
C     DONNE LES CONSTANTES
C             A1=100.075542 DEGRES
C             A2=360.985612288 DEGRES PAR JOUR
C
C     POUR UN CALCUL DU TEMPS SIDERAL:
C                        TSID = ( A1 + A2*RDJD ) MODULO 2.PI
C
C  OR RDJD EST EXPRIME DANS UNE ECHELLE DE TEMPS CNES (TUC) 
C     DIFFERENTE DE L'ECHELLE DE TEMPS TU1 UTILISEE POUR LA FORMULATION
C     DU TEMPS SIDERAL CI-DESSUS. LA FORMULE EST DONC ADAPTEE EN:
C                        TSID = ( A1 + A2*RDJD + A3*RDTU1 ) MODULO 2.PI
C
C        AVEC A3    = A2/86400 DEGRES PAR SECONDE
C             RDTU1 = ECART EN SECONDE ENTRE TU1 ET ECHELLE DE
C                     DATATION DE RDJD
C
C     POUR DES RAISONS DE PERTES DE PRECISION NUMERIQUE L'ALGORITHME
C     EST ADAPTE EN: 
C         TSID = (A1 + 2.PI*FRAC(RDJD) + EPS*RDJD + A3*RDTU1 ) MOD 2.PI
C
C        AVEC FRAC(RDJD) = PARTIE FRACTIONNAIRE DE RDJD
C             EPS        = A2 - 360 (EN DEGRES PAR JOUR)
C
C      REMARQUE1: ON TROUVE SOUVENT DANS LA LITTERATURE LA VALEUR
C                 ERRONNEE A3 = 0.004177 DEGRES PAR SECONDE
C                 QUI CONDUIT A UNE
C                 VALEUR ERRONNE = 0.729024D-04 RADIANS PAR SECONDES
C      REMARQUE2: LES VALEURS EN RADIANS DES PARAMETER ONT ETE VERIFIEES
C                 SOUS MAPLE
C
C     ---------------
C     INITIALISATIONS
C     ---------------
C
C     A1=100.075542 DEGRES, SOIT EN RADIANS:
      PARAMETER (A1=.1746647708617871D+01)
C
C     EPS= A2 - 360 = 0.985612288 DEGRES PAR JOUR,
C        SOIT EN RADIANS PAR JOUR:
      PARAMETER (EPS=1.7202179573714597D-02)
C
C     A3=(360+EPS)/86400 DEGRES PAR SECONDE,SOIT EN RADIANS PAR SECONDE:
      PARAMETER (A3=.7292115146705D-04)
C
#include "mmath.h"
C
C     . . . INITIALISATION DE L'IDENTIFICATEUR SCCS
C      
      CHARACTER*50 SCCSID
      DATA SCCSID/"%Z%  %M%  %Y%  %I%  %E%"/
C
C     ******************
C     DEBUT DE PROGRAMME
C     ******************
C
      IF (.FALSE.) THEN
         SCCSID = SCCSID
      ENDIF
C
C     ---------------
C*FON INITIALISATIONS
C     ---------------
C
C     INITIALISATION:  IER=0  FAITES LORS DE L'APPEL A MUAMOD
C                      MRTSID EST BIEN INITIALISE
C
C     -----------------------
C*FON CALCUL DU TEMPS SIDERAL
C     -----------------------
C
      TSID = A3*RDTU1 + A1 + DEUXPI*(RDJD-AINT(RDJD)) + EPS*RDJD
C
      TSID = MUAMOD(TSID,DEUXPI,IER)
C     PAS DE CODE RETOUR NEGATIF CAR DEUXPI DIFFERENT DE ZERO
C
      MRTSID = TSID
C
C     ****************
C     FIN DE PROGRAMME
C     ****************
C
 6000 CONTINUE
      RETURN
      END
