      subroutine mrtsidv (imodr,jj,sec,dtu1,dtai,tsidv,tsidvd)
C
C***********************************************************************
C*
C*PRO MSLIB 0289
C*VER 27.05.99 VERSION 10.0
C*AUT SPEC. CNES - LE DE -
C*AUT PROG. (Mise aux normes MSLIB) CISI DIS/MSS
C*AUT (C) COPYRIGHT CNES - 1999
C*ROL THEME : Changement de systemes de reference
C*ROL Calcule le temps sideral vrai et sa derivee a la date T
C*ROL         (dans 2 systemes de reference)
C
C*PAR (I) imodr   : indicateur pour les modeles de precession + nutation
C*PAR                  = 1 : NEWCOMB(B1900) + WOOLARD(J1900)
C*PAR                  = 2 : LIESKE(J2000) + WAHR(J2000)
C*PAR (I) jj      :} date julienne 1950 T (non normalisee) en jour 
C*PAR (I) sec     :}                                   et secondes
C*PAR (I) dtu1    : ecart entre l'echelle de temps TU1 et l'echelle de
C*PAR               temps utilisee pour la date T
C*PAR (I) dtai    : ecart entre l'echelle de temps TAI et l'echelle de
C*PAR               temps utilisee pour la date T
C*PAR (O) tsidv   : temps sideral vrai (radians)
C*PAR (O) tsidvd  : derivee du temps sideral vrai (radians/sec)
C
C*NOT Note d'utilisation: 
C*NOT   - Routine interne
C*NOT   - L'appelant doit avoir verifie que: imodr = 1 ou 2
C*NOT     (pas de test effectue ici)
C
C*NOT DOCUMENT DE REFERENCE :
C*NOT [DR1] "Les systemes de reference utilises en astronomie"
C*NOT       de M. Chapront-Touze, G. Francou et B. Morando
C*NOT       Bureau Des Longitudes (BDL) novembre 1994
C*NOT       ISSN 1243-4272
C*NOT       ISBN 2-910015-05-X
C*NOT       nomenclature MSLIB M-NT-0-160-CN
C*NOT Nota: Les epoques B1900, J1900 et J2000 sont explicitees en
C*NOT       page 23 table 1.1
C*NOT [DR2] "Algorithmes des routines du theme 
C*NOT               Les Reperes Fondamentaux de la MSLIB"
C*NOT       nomenclature MSLIB M-NT-0-95-CN
C*HST VERSION 10.0 - 27.05.99 - CREATION: M-SP-289-1-CIS
C*HST VERSION 10.1 - 24.01.00 - CORREC/AMELIORATION ALGO: M-FA-289-1-CIS
C*
C***********************************************************************
C
      implicit none
C
C------------------------------
C     PARAMETRES
C------------------------------
      integer          imodr
      double precision sec,dtu1,dtai
      integer          jj
      double precision tsidv,tsidvd
C
C-------------------------------
C     SOUS-PROGRAMMES UTILISES
C-------------------------------
C
      double precision mrtsmo
      external mrnuta, mrobli, mrtsmo
C
C
C-------------------------------
C     VARIABLES LOCALES
C-------------------------------
C
C     code retour des modules appeles
      integer ierapp
C
C     calcul du temps sideral moyen et de sa derivee
      double precision tsm,tsmd,secut1,secai
C
C     nutation en longitude, nutation en obliquite, 
C     obliquite moyenne
      double precision rnutln,rnutob,roblim
C
C     derivees
      double precision rnderi(2,2), roderi(2)
C
C     calcul des arguments fondamentaux  
      double precision rarfon(5) 
C
C     derivees
      double precision raderi(5,2)
C
C     longitude du noeud ascendant de la Lune, derivee
      double precision omega, omegad
C
C     indicateur pour le calcul des derivees (=1 derivees premieres)
      integer ider
C
C     t2000 = date julienne CNES au 01-01-2000 a 12h TDB~TE
C             (confondu avec TE)
      double precision t2000
      parameter (t2000 = 18262.5d0)
C
C     t1900 = date julienne CNES au 31-12-1899 a 12h TE
      double precision t1900
      parameter (t1900 = -18262.5d0)
C
      double precision tu,tu2
C
C     indicateur pour les modeles de precession+nutation pour mrtsmo
      integer isysrf
C
#include "mmath.h"
C
C=======================================================================
C
C Divers parametres de conversion
C
      double precision unsur86400, unsurCanJul, unsurSecCan, srad
C
C     Inverse du nombre de secondes par jour
      parameter (unsur86400 = 1.d0/86400.d0)
C     Inverse du nombre de jours en cent ans Juliens
C     (C = cent = 100; Can = 100 ans) :
      parameter (unsurCanJul = 1.d0/36525.d0)
C     Inverse du nombre de secondes pour cent ans Juliens
      parameter (unsurSecCan = unsur86400*unsurCanJul)
C
      double precision p11, p12, p21, p22, p23
C     Parametres pour calculer la derivee du temps sideral moyen
C     modele de Newcomb
      parameter (p11 = 8640184.542d0)
      parameter (p12 = 0.0929d0)
C     modele de Lieske
      parameter (p21 = 8640184.812866d0)
      parameter (p22 = 0.093104d0)
      parameter (p23 = 6.2d-6)
C
C     Conversion Secondes d'arc en radians
      parameter (srad = XDERA/3600.d0)
C
C=======================================================================
C
C     . . . INITIALISATION DE L'IDENTIFICATEUR SCCS
C      
      CHARACTER*50 SCCSID
      DATA SCCSID/"%Z%  %M%  %Y%  %I%  %E%"/
C
C-------------------------------
C     INITIALISATION
C-------------------------------
C
      IF (.FALSE.) THEN
         SCCSID = SCCSID
      ENDIF
C
      ierapp = 0
C
      ider = 1
C
      if (imodr.eq.1) then 
      isysrf = 1950 
C     (DANS MRTSMO LE CAS 1900 EST PARAMETRE PAR 1950)
C
      else
      isysrf = 2000
C
      endif
C
C-------------------------------
C-------------------------------
C     Algorithme
C-------------------------------
C-------------------------------
C
C     Passage de la date T en date UT1:
C
      secut1= sec + dtu1
C
C     Passage de la date T en date TAI:
C
      secai = sec + dtai
C
C
C     CALCUL DE OMEGA ET OMEGAD
C
      call msargf(jj,secai,ider,imodr,rarfon,raderi,ierapp)
C
      omega = rarfon(5)
      omegad = raderi(5,1)
C
C
C     TEMPS SIDERAL MOYEN
C
      tsm = mrtsmo(jj,secut1,isysrf,ierapp)
C
C     EQUATION DES EQUINOXES:
C
C     nutations en longitude (rnutln) et en obliquite (rnutob) 
C     de la date
C
      call mrnuta(jj,secai,ider,imodr,rnutln,rnutob,rnderi,
     &            ierapp)
C
C     obliquite moyenne de la date
C
      call mrobli(jj,secai,ider,imodr,roblim,roderi,ierapp)
C
C     TEMPS SIDERAL VRAI
C
C     Voir formules donnees dans [DR2]
C
      tsidv = rnutln*cos(roblim) + tsm + 0.00264d0*srad*sin(omega) 
     &        + 0.000063d0*srad*sin(2*omega)
C
C
C
C     DERIVEE DU TEMPS SIDERAL MOYEN
C
      if (imodr.eq.1) then
C
C     Voir formules 9.7 et 9.8 de [DR1]
C
         tu = ((dble(jj)-t1900) + secut1*unsur86400) * unsurCanJul
C      
C     Derivee de la formule 9.6 de [DR1]
C
        tsmd = DEUXPI * unsur86400 * 
     &         (1.d0 + unsurSecCan*(p11 + 2.d0*p12*tu) )
C
      else
C
C     Voir formules 9.17 et 9.8 de [DR1]
C
        tu = ((dble(jj)-t2000) + secut1*unsur86400) * unsurCanJul
        tu2= tu*tu
C
C     Derivee de la formule 9.16 de [DR1]
C
        tsmd = DEUXPI * unsur86400 * 
     &         (1.d0 + unsurSecCan*(p21 + 2.d0*p22*tu - 3.d0*p23*tu2))
C
      endif
C
C     DERIVEE DU TEMPS SIDERAL VRAI
C
C     Voir formules donnees dans [DR2]
C
      tsidvd =   rnderi(1,1)*cos(roblim) 
     &         - rnutln*roderi(1)*sin(roblim)
     &         + tsmd  
     &         + 0.00264d0*srad*omegad*cos(omega)
     &         + 0.000063d0*srad*2*omegad*cos(2*omega)
C
C
 6000  continue
      end
