      DOUBLE PRECISION FUNCTION MRTSMO (IJOUR,RSEC,ISYSRF,IER)
C
C******************************************************************
C*
C*PRO MSLIB 0024
C*VER 24.09.87 V1.0
C*AUT SPEC.  C N E S - LABRUNE - 13 MARS 86
C*AUT PORT.  CISI INGENIERIE - (MATH. ET MECA. SPATIALE)
C*AUT (C) COPYRIGHT CNES  -  1990
C*ROL PROGRAMME : MRTSMO   TEMPS SIDERAL MOYEN DANS 3 SYSTEMES REFERENCE
C*ROL TEMPS SIDERAL MOYEN DANS 3 SYSTEMES DE REFERENCE
C*ROL - 1950  : J1950 (NEWCOMB)
C*ROL - 2000  : J2000 (AOKI - MERIT STANDARD)
C*ROL - VEIS 1950 (GAMMA 50)  (CNES)
C*PAR IJOUR  (I) : JOUR JULIEN (NOMBRE DE JOURS DEPUIS LE 1/1/1950 0H)
C*PAR RSEC   (I) : SECONDES DANS LE JOUR IJOUR  (ECHELLE TU1)
C*PAR ISYSRF (I) : CHOIX SYST. REF (1950,2000,OU AUTRE=GAMMA 50)
C*PAR IER    (O) : INDICATEUR DE RETOUR (0: OK)
C*INF MRTSID : CALCUL TEMPS SIDERAL (REPERE DE VEIS)
C*INF MUAMOD : FONCTION MATHEMATIQUE [ MODULO ]
C*HST V1.O 24.09.87 CREATION
C*HST V0.0  28.07.87 CORRECTION ANOMALIE M-FA-24-1-C
C*HST VERSION 7.1 - 18.12.92 - EVOLUTION MCONST : M-DE-001-1-II
C*HST VERSION 7.2 - 25.02.93 - EVOLUTION : M-DE-024-1-II
C*HST VERSION 8.1 - 21.11.94 - EVOLUTION PORTAGE
C*HST VERSION 10.0 - 23.02.99 - EVOLUTION MUCONVJJSJJF : M-DE-024-1-CIS
C*
C***********************************************************************
C
      IMPLICIT NONE
C
C
C     --------------------------
C     DECLARATION DES PARAMETRES
C     --------------------------
C
      INTEGER IJOUR, ISYSRF, IER
      DOUBLE PRECISION RSEC
C
C     -----------------------------------
C     DECLARATION DES PROCEDURES EXTERNES
C     -----------------------------------
C
      EXTERNAL MRTSID, MUAMOD, MUCONVJJSJJF
      DOUBLE PRECISION MRTSID, MUAMOD
C
C     --------------------------
C     DECLARATION DES CONSTANTES
C     --------------------------
C
      DOUBLE PRECISION UN
      DOUBLE PRECISION SECJ, UNSSEC, UNSS
C     SECJ : NOMBRE DE SECONDES DANS UNE JOURNEE = 86400
C     UNSSEC : 1/86400 (FACTEUR TRANSFORMATION SECONDE EN JOUR)
C     UNSS : 1/36525 (FACTEUR TRANSFORMATION JOUR JULIEN EN SIECLE)
C
      DOUBLE PRECISION DA0, DA1, DA2
C     DA0,DA1,DA2 : COEFFICIENTS DE CALCUL POUR LA FORMULE DE
C                   REFERENCE POUR 1950 (NEWCOMB)
C                   AVEC LA FORMULE DE REFERENCE:
C                   TETA = S + DA0 + DTU*DA1 + DTU**2*DA2
      DOUBLE PRECISION DB0, DB1, DB2, DB3
C     DB0,DB1,DB2,DB3 : COEFFICIENTS DE CALCUL POUR LA FORMULE DE
C                       REFERENCE POUR 2000 (AOKI)
C                       AVEC LA FORMULE DE REFERENCE:
C                 TETA = S + DB0 + DTU*DB1 + DTU**2*DB2 + DTU**3*DB3
C
C     ---------------------------------
C     DECLARATION DES VARIABLES LOCALES
C     ---------------------------------
C
      DOUBLE PRECISION RDJD, RDTU1, TSMO
C          RDJD : JOUR JULIEN REEL (JOUR+FRACTION DE JOUR)
C          RDTU1: ECART ENTRE ECHELLE DE TEMPS TU1 ET ECHELLE
C                 DE DATATION DE RDJD
C          TSMO : TEMPS SIDERAL MOYEN CALCULE
      DOUBLE PRECISION DTU
C     DTU : NOMBRE DE SIECLES DEPUIS LE 01/01/1950 A 0 HEURE UT1
C           JUSQU'AU JOUR COURANT (IJOUR,RSEC)
      DOUBLE PRECISION DTETA
C     DTETA : TEMPS SIDERAL AVANT OPERATION MODULO [2*PI]
C
C
C     ---------------
C     INITIALISATIONS
C     ---------------
C
      PARAMETER (UN=1.0D0)
      PARAMETER (SECJ=86400.D0)
      PARAMETER (UNSSEC=UN/SECJ, UNSS=UN/36525.0D0)
      PARAMETER (DA0=23925.836D0, DA1=8640184.542D0, DA2=0.0929D0)
      PARAMETER (DB0=24110.54841D0, DB1=8640184.812866D0)
      PARAMETER (DB2=0.093104D0, DB3=-6.2D-6)
C
C
#include "mmath.h"
C
C     . . . INITIALISATION DE L'IDENTIFICATEUR SCCS
C      
      CHARACTER*50 SCCSID
      DATA SCCSID/"%Z%  %M%  %Y%  %I%  %E%"/
C
C
C     ******************
C     DEBUT DE PROGRAMME
C     ******************
C
      IF (.FALSE.) THEN
         SCCSID = SCCSID
      ENDIF
C
C     ---------------
C*FON INITIALISATIONS
C     ---------------
C
C     INITIALISATION:  IER=0  FAITES LORS DE L'APPEL A MUAMOD OU MRTSID
C
C      -----------------------------------------------------------------
C*FON  SI REFERENCE 1950 (NEWCOMB, REFERENCE DE DATATION
C*FON                    LE 01/01/1900 A 12 HEURES UT1): 
C*FON                    DTU = DTU0 + 0.5
C*FON  AVEC DTU0=NBRE DE SIECLE JULIEN ECOULE DEPUIS
C*FON            LE 1/1/1950 A 0H UT1 JUSQU'A LA DATE DE CALCUL
C      -----------------------------------------------------------------
C
      IF (ISYSRF.EQ.1950) THEN
C
         CALL MUCONVJJSJJF(IJOUR, RSEC, DTU, IER)
C
C     IL N'Y A PAS DE CODE RETOUR NEGATIF OU POSITIF POSSIBLE
C
         DTU = DTU * UNSS + 0.5D0
         DTETA = RSEC + (DA2 * DTU + DA1) * DTU + DA0
         TSMO = MUAMOD(DTETA, SECJ, IER)
C
         MRTSMO = TSMO * DEUXPI * UNSSEC
C
C
C      -----------------------------------------------------------------
C*FON  SI REFERENCE 2000 (AOKI, REFERENCE DE DATATION
C*FON                    LE 01/01/2000 A 12 HEURES UT1):
C*FON                    DTU = DTU0 - 0.5
C*FON  AVEC DTU0=NBRE DE SIECLE JULIEN ECOULE DEPUIS 
C*FON            LE 1/1/1950 A 0H UT1 JUSQU'A LA DATE DE CALCUL
C      -----------------------------------------------------------------
C
      ELSEIF (ISYSRF.EQ.2000) THEN
C
         CALL MUCONVJJSJJF(IJOUR, RSEC, DTU, IER)
C
C     IL N'Y A PAS DE CODE RETOUR NEGATIF OU POSITIF POSSIBLE
C
         DTU = DTU * UNSS - 0.5D0
         DTETA = RSEC + ((DB3 * DTU + DB2) * DTU + DB1) * DTU + DB0
         TSMO = MUAMOD(DTETA, SECJ, IER)
C
         MRTSMO = TSMO *DEUXPI * UNSSEC
C
C
C      ---------------------------------------------
C*FON  SI REPERE DE VEIS, APPEL A LA FONCTION MRTSID
C      ---------------------------------------------
C
      ELSE
C
         CALL MUCONVJJSJJF(IJOUR, RSEC, RDJD, IER)
C
C     IL N'Y A PAS DE CODE RETOUR NEGATIF OU POSITIF POSSIBLE
C
         RDTU1 = 0.D0
         MRTSMO = MRTSID(RDJD,RDTU1,IER)
C
      ENDIF
C
C
C     ****************
C     FIN DE PROGRAMME
C     ****************
C
 6000 CONTINUE
      RETURN
      END
