      SUBROUTINE MSARGF(IJOURJ,RSECON,IDERIV,IMODEL,RARFON,RADERI,IER)
C
C**********************************************************************
C*
C*PRO MSLIB 0094
C*VER 26.11.87 - VERSION 2.0
C*AUT SPEC. CISI INGENIERIE -- LAGARRIGUE -- 2I/SE/MMS --
C*AUT PROG. CISI INGENIERIE (MATH. ET MECA. SPATIALE)
C*AUT (C) COPYRIGHT CNES  -  1990
C*ROL PROGRAMME : ARGUMENTS FONDAMENTAUX
C*ROL THEME : EPHEMERIDES DU SYSTEME SOLAIRE
C*ROL CALCUL DES ARGUMENTS FONDAMENTAUX CARACTERISANTS LES POSITIONS
C*ROL DE LA LUNE ET DU SOLEIL, ET DE SES DERIVEES, SUIVANT LE MODELE
C*ROL DE WAHR OU WOOLARD, A UNE DATE DONNEE
C*PAR IJOURJ  (I)   JOUR JULIEN CNES
C*PAR RSECON  (I)   SECONDES TAI
C*PAR IDERIV  (I)   CLE DE CALCUL DES DERIVEES
C*PAR IMODEL  (I)   INDICATEUR DU MODELE
C*PAR RARFON  (O)   TABLEAU DES 5 ARGUMENTS FONDAMENTAUX   (RAD)
C*PAR RADERI  (O)   TABLEAU (5*2) DES DERIVEES
C*PAR IER     (O) : CODE RETOUR
C*NOT IDERIV                 1: CALCUL  0: SINON
C*NOT IMODEL                 1: MODELE DE WOOLARD
C*NOT IMODEL                 2: MODELE DE WAHR
C*NOT RARFON        RARFON(1) : ANOMALIE MOYENNE DE LA LUNE
C*NOT               RARFON(2) : ANOMALIE MOYENNE DU SOLEIL
C*NOT               RARFON(3) : LONGITUDE DE LA LUNE
C*NOT                           A PARTIR DU NOEUD ASCENDANT
C*NOT               RARFON(4) : ANGLE HORAIRE DE LA LUNE AU SOLEIL
C*NOT               RARFON(5) : LONGITUDE DU NOEUD ASCENDANT DE LA LUNE
C*NOT RADERI        COLONNE 1 : DERIVEES PREMIERES   ( RAD/SEC )
C*NOT               COLONNE 2 : DERIVEES SECONDES    ( RAD/SEC**2)
C*NOT               LIGNE     : ARGUMENT FONDAMENTAL
C*NOT CODE RETOUR           0 : RETOUR NORMAL
C*NOT                      -1 : MAUVAISE VALEUR DE IMODEL
C*NOT                      -2 : MAUVAISE VALEUR DE IDERIV
C*INF REFERENCE DU MODELE DE WOOLARD : CONNAISSANCE DES TEMPS 1967
C*INF REFERENCE DU MODELE DE WAHR    : IAU THEORY OF NUTATION 1980
C*HST VERSION 2.0 - 21.09.87 - CREATION
C*HST VERSION 7.1 - 18.12.92 - EVOLUTION MCONST : M-DE-001-1-II
C*HST VERSION 10.0 - 23.02.99 - EVOLUTION IER : M-DE-094-1-CIS
C*
C***********************************************************************
C
      IMPLICIT NONE
C
C
C     -------------------------
C     DECLARATION DES PARAMETRES
C     --------------------------
C
      INTEGER IJOURJ, IDERIV, IMODEL, IER, NIL
      DOUBLE PRECISION RSECON, RARFON(5), RADERI(5,2)
C
C     ----------------------------------
C     DECLARATION DES FONCTIONS EXTERNES
C     ----------------------------------
C
      EXTERNAL MUAMOD, MUCONVJJSJJF
      DOUBLE PRECISION MUAMOD
C
C     -------------------------------
C     DECLARATION DES DONNEES LOCALES
C     -------------------------------
C
C     DATE EN JOUR JULIEN FRACTIONNAIRE
      DOUBLE PRECISION RTEMPS
C
C     COEFFICIENTS POUR UN ARGUMENT
      DOUBLE PRECISION RCOEFF(4)
C
C     INDICES DE BOUCLE
      INTEGER I,J
C
C     VARIABLE INTERMEDIAIRE DE TEMPS
      DOUBLE PRECISION RSECTAI
C
C     DECLARATION DU TYPE DES CONSTANTES
C     ... CONSTANTES UTILISEES DANS LES SYSTEMES DE REFERENCE
      DOUBLE PRECISION RCDERP,RCDERS
C
C ... COEFFICIENTS UTILISES DANS L'EXPRESSION DES ARGUMENTS FONDAMENTAUX
      DOUBLE PRECISION RCOEF1(5,4),RCOEF2(5,4)
C
C     ... DIFFERENCE ENTRE LE TEMPS DES EPHEMERIDES ET LE TAI
      DOUBLE PRECISION RTDTAI
      DOUBLE PRECISION RARGF1,RARGF2,RARGP1,RARGP2,RARGS1
C
C     ... CONSTANTES
       DOUBLE PRECISION RPI,R2PI
C
#include "mmath.h"
C
C     . . . INITIALISATION DE L'IDENTIFICATEUR SCCS
C      
      CHARACTER*50 SCCSID
      DATA SCCSID/"%Z%  %M%  %Y%  %I%  %E%"/
C
C     ---------------
C     INITIALISATION
C     ---------------
C
C
C     ... TABLE DE WOOLARD
C
      DATA RCOEF1/
     -    .51680003457448D+01,         .62565835804970D+01
     -,        .19636505488716D+00,    .61215239428067D+01
     -,        .45236015148518D+01
     -,   .83286911036679D+04,         .62830194572673D+03
     -,        .84334662911718D+04,    .77713771939344D+04
     -,       -.33757146246552D+02
     -,   .16042484707914D-03,        -.26179938779915D-05
     -,       -.56044461536262D-04,   -.25064867313363D-04
     -,        .36264063346993D-04
     -,   .25113348681474D-06,        -.58177641733144D-07
     -,       -.58177641733144D-08,    .32967330315448D-07
     -,        .38785094488762D-07/
C
C     ... TABLE DE WAHR
C
      DATA RCOEF2/
     -    .23555483935439D+1,          .62400359393259D+1,
     -         .16279019339719D+1,     .51984695135798D+1,
     -         .21824386243610D+1,
     -    .83286914228839D+4,          .62830195602418D+3,
     -         .84334661583185D+4,     .77713771461706D+4,
     -        -.33757045933754D+2,
     -    .1517951635554D-3,          -.27004122037801D-5,
     -        -.6427174970469D-4,     -.33408510765258D-4,
     -         .36142859926716D-4,
     -    .3102807559101D-6,          -.58177641733144D-7,
     -        .53329504922048D-7,      .92114599410811D-7,
     -        .38785094488762D-7/
C
       DATA RTDTAI / 32.184D0 /
       DATA RCDERP / 3.168808781403D-10 /
       DATA RCDERS / 1.00413490931D-19 /
C
C
C
C     ******************
C     DEBUT DU PROGRAMME
C     ******************
C
      IF (.FALSE.) THEN
         SCCSID = SCCSID
      ENDIF
C
C
      IER = 0
      RPI  = PI
      R2PI= RPI * 2.D0
C
C     ==================================
C*FON VERIFICATION DES DONNEES EN ENTREE
C     ==================================
C
      IF ( (IMODEL.NE.1).AND.(IMODEL.NE.2) ) THEN
         IER = -1
         GO TO 6000
      ENDIF
C
      IF ( (IDERIV.NE.1).AND.(IDERIV.NE.0) ) THEN
         IER = -2
         GO TO 6000
      ENDIF
C
C     =========================================================
C*FON DATE EN SIECLES JULIENS
C*FON CONVERSION DES SECONDES TAI EN TEMPS DYNAMIQUE TERRESTRE
C     =========================================================
C
      RSECTAI = RSECON + RTDTAI
      CALL MUCONVJJSJJF(IJOURJ, RSECTAI, RTEMPS, IER)
C
C     IL N'Y A PAS DE CODE RETOUR NEGATIF POSSIBLE
C
      RTEMPS = RTEMPS / 36525.D0
C
      IF (IMODEL.EQ.1) THEN
         RTEMPS = RTEMPS + 0.5D0
      ELSE
         RTEMPS = RTEMPS - 0.5D0
      ENDIF
C
C     =======================
C*FON ARGUMENTS FONDAMENTAUX
C     ========================
C
C     --------------------------
C     BOUCLE SUR LES ARGUMENTS
C     --------------------------
C
      DO 10 I = 1,5
         IF (IMODEL.EQ.1) THEN
C
C     ... TRANSFERT DANS VARIABLES DE TRAVAIL DU MODELE DE WOOLARD
            DO 20 J = 1,4
               RCOEFF(J) = RCOEF1(I,J)
 20         CONTINUE
C
      ELSE
C
C     ... TRANSFERT DANS VARIABLES DE TRAVAIL DU MODELE DE WAHR
         DO 30 J = 1,4
            RCOEFF(J) = RCOEF2(I,J)
 30      CONTINUE
C
      ENDIF
C
C
      RARGF1    = RCOEFF(4)*RTEMPS + RCOEFF(3)
      RARGF2    = RARGF1   *RTEMPS + RCOEFF(2)
      RARFON(I) = RARGF2   *RTEMPS + RCOEFF(1)
      RARFON(I) = MUAMOD ( RARFON(I),R2PI,NIL )
C
C     ==============================
C*FON DERIVEES PREMIERES ET SECONDES
C     ===============================
C
      IF (IDERIV.EQ.1) THEN
C
C     ----------------
C     DERIVEE PREMIERE
C     ----------------
         RARGP1      = 3.D0*RCOEFF(4)*RTEMPS + 2.D0*RCOEFF(3)
         RARGP2      = RARGP1      *RTEMPS + RCOEFF(2)
         RADERI(I,1) = RARGP2 *RCDERP
C     
C     ---------------
C     DERIVEE SECONDE
C     ---------------
         RARGS1      = 6.D0*RTEMPS*RCOEFF(4) + 2.D0*RCOEFF(3)
         RADERI(I,2) = RARGS1*RCDERS
C     
      ELSE
C
         RADERI(I,1) = 0.D0
         RADERI(I,2) = 0.D0
C
      ENDIF
C
10    CONTINUE
C
C     ----------------------------
C     FIN BOUCLE SUR LES ARGUMENTS
C     ____________________________
C
C     ****************
C     FIN DE PROGRAMME
C     ****************
C
 6000 CONTINUE
      RETURN
      END
