      subroutine celem (orbel,gmc,pv,vv)
c   original version ...1/22/71....charles k. capps
c   purpose:
c       this routine converts classical osculating orbital elements
c       to cartesian elements.
c   calling sequence:
c       call celem(orbel,gmc,pv,vv)
c   input thru argument list:
c       orbel(1) = semi-major axis,a  (osculating elements)
c       orbel(2) = eccentricity, e
c       orbel(3) = inclination, i
c       orbel(4) = longitude of ascending node, cap omega
c       orbel(5) = argument of perifocus, omega
c       orbel(6) = mean anomaly, m
c       gmc = gravitational constant
c   output thru argument list:
c       pv = cartesian position vector
c       vv = cartesian velocity vector
c   method:
c       uses miles standish iterative scheme for soln to keplers eqn.
c   references:
c       gtds task spec for celem, c.e. velez, 13 january 1971
c       dods system description, subroutine keplr1
c       p. excobal-'methods of orbit determination'
c       x-552-67-421,'comparison fo iterative techniques for the
c       solution of keplers equation', i.cole and r. borchers
c   programmer:
c       charles k. capps,  code 553.2,  gsfc
c
      implicit real*8(a-h,o-z)
      dimension pv(3),vv(3),orbel(6)
      data max /10/
      data tol /+0.5d-16/
      iter = 0
c     find if this is elliptic or hyperbolic orbit
      if (orbel(1).le.0.0d0.and.orbel(2).gt.1.0d0) go to 50
c     elliptic orbit takes this route.
c     first find eccentric anomaly via newtons (miles standish version)
      e1 = orbel(6)
  10  f = e1 - (orbel(2) * dsin(e1)) - orbel(6)
      d = 1.0d0 - (orbel(2) * dcos(e1 - 0.5d0 * f))
      e2 = e1 - (f / d)
      if (dabs (e1-e2)-tol )40,40,20
   20 iter = iter + 1
      e1 = e2
      if(iter - max) 10,10,30
c     set up error code to return from subroutine
   30 nerr = 13
c     eccentric anomaly converged, now get xo, yo, r
   40 cose = dcos(e2)
      sine = dsin (e2)
      temp = 1.0d0 - orbel(2) * orbel(2)
      xo = orbel(1) * (cose - orbel(2))
      yo = orbel(1) * (dsqrt(temp)* sine)
      r = orbel(1) * (1.0d0 - orbel(2) * cose)
      xod = (-dsqrt(gmc* orbel(1))* sine)/r
      yod = (dsqrt(gmc*orbel(1)*(temp))*cose) / r
      go to 100
c     hyperbolic orbits take this route
   50 e1 = orbel(6) / 2.0d0
   60 f = orbel(2) * dsinh(e1) - e1 - orbel(6)
      d = orbel(2) * dcosh(e1 - 0.5d0 * f ) - 1.0d0
      e2 = e1 - (f / d)
      if (dabs (e1-e2)-tol )90,90,70
   70 iter = iter + 1
      e1 = e2
      if (iter - max) 60,60,80
c      set up error code for non-convergence prior to exit.
   80 nerr = 14
c     eccentric anomaly computed, now get xo,yo,r
   90 cose = dcosh (e2)
      sine = dsinh(e2)
      temp = orbel(2) * orbel(2) - 1.0d0
      xo = orbel(1)*(cose- orbel(2))
      yo = - orbel (1)*dsqrt (temp) * sine
      r = orbel(1)*(1.0d0 - orbel(2) * cose)
      xod = (-dsqrt(-gmc*orbel(1))*sine)/r
      yod = (dsqrt(-gmc*orbel(1)*temp)*cose) / r
  100 coso = dcos(orbel(5))
      sino = dsin (orbel(5))
      cosom = dcos (orbel(4))
      sinom = dsin (orbel(4))
      cosi = dcos(orbel(3))
      sini = dsin (orbel(3))
      b11 = coso * cosom - sino * sinom * cosi
      b21 = coso * sinom + sino * cosom * cosi
      b31 = sino * sini
      b12 = -sino * cosom - coso * sinom * cosi
      b22 = -sino * sinom + coso * cosom * cosi
      b32 = coso * sini
c     now multiply 3 x 2 martix by 2 x 1 vectors for position, velocity.
      pv(1) = b11 * xo + b12 * yo
      pv(2) = b21 * xo + b22 * yo
      pv(3) = b31 * xo + b32 * yo
      vv (1) = b11*xod + b12 * yod
      vv(2) = b21 * xod + b22 * yod
      vv(3) = b31 * xod + b32 * yod
  999 return
      end
