      subroutine elem(pv,vv,gmc,apc,dterm,ipdot,classe,orbel)
      implicit real*8(a-h,o-z)
c     version of 7/20/71
c     version of 12/02/71  r.sepanik, gsfc, code 552
c      added calculations for new orbit parameters-
c      denoted by an asterisk(*) below
c    version of 01/13/72
c     version of april 6,1972  call  eval directly  with  tto
c     version of july 31, 1974
c     .  4/1/74 c. l. berry csc
c     .  modified to accomodate n-body capability
c
c     purpose
c         to compute classical orbital elements from position and
c         velocity vectors
c
c     input
c         pv(3)=r=position vector of spacecraft
c         vv(3)=v=velocity vector of spacecraft
c         gmc=mu =gravitational constant of body(km**3/sec**2)
c         apc=ap=mean radius of reference body(km)
c         dterm=j2=second zonal harmonic
c     ipdot  = elements flag indicator 1,2, or 3
c            =0 do not call perdot orbel(3)=0, but call eval + xyzplh
c            =1 call perdot for calculation of period dot orbel(3)
c               and call eval + xyzplh
c            =3 only compute orbel + classe but do not call perdot,
c               eval + xyzplh
c
c     output
c     classe(6)=classical orbital elements
c
c     classe(1)=semi-major axis
c     classe(2)=eccentricity
c     classe(3)=inclination
c     classe(4)=longitude of ascending node
c     classe(5)=argument of perifocus
c     classe(6)=mean anomaly
c
c     orbel(20)=orbit parameters
c
c
c     orbel( 1)=eccentric anomaly
c     orbel( 2)=period
c     orbel( 3)=period dot                  *
c     orbel( 4)=mean motion                 *
c     orbel( 5)=true anomaly
c     orbel( 6)=perifocal height
c     orbel( 7)=apofocal height
c     orbel( 8)=argument of perifocus dot   *
c     orbel( 9)=ascending node dot          *
c     orbel(10)=velocity at apofocus        *
c     orbel(11)=velocity at perifocus       *
c     orbel(12)=geodetic latitude
c     orbel(13)=east longitude
c     orbel(14)=spheroid height
c     orbel(15)=semi-latus rectum
c     orbel(16)=perifocal radius
c     orbel(17)=apofocal radius
c     orbel(18)=energy
c     orbel(19)=spare                       *
c     orbel(20)=spare                       *
c
c     references
c         (1)hope system description-subroutine orbel
c         (2)tdas system description-subroutine class
c         (3)geostar system description-subroutine elem
c
c     analysis
c         raymond v. borchers
c     programmer
c         raymond v. borchers,program systems branch
c             mission and trajectory analysis division
c             goddard space flight center,nasa
c*****start program*****************************************************
c
      dimension  pv(3), vv(3), classe(6), orbel(20),b(3,3),velout(3)
      dimension      ibdy(9),posit(3),vect(6,8),posout(3)
      common/blcnst/tto,r,ae,gm,bj2,bj3,bj4,bj5,fltinv,xke,esq
      common /elmblk/ gha
      data twopi/6.2831853071795864d0/
      dterm = -bj2
      gmc = gm
      apc = ae
c
c     compute r magnitude
c
      rabs=dsqrt(pv(1)*pv(1)+pv(2)*pv(2)+pv(3)*pv(3))
c
c     compute v**2 magnitude
c
      vabs=vv(1)*vv(1)+vv(2)*vv(2)+vv(3)*vv(3)
c
      rdotv=pv(1)*vv(1)+pv(2)*vv(2)+pv(3)*vv(3)
      h1=(pv(2)*vv(3))-(pv(3)*vv(2))
      h2=(pv(3)*vv(1))-(pv(1)*vv(3))
      h3=(pv(1)*vv(2))-(pv(2)*vv(1))
c
c     compute semi-major axis
c
      arg=rabs*vabs
      classe(1)=(gmc*rabs)/((2.0d0*gmc)-(arg))
c
c     compute semi-latus rectum
c
      orbel(15)=((rabs*arg)-(rdotv*rdotv))/gmc
c
c     test semi-latus rectum
c
      if(orbel(15).lt.0.0d0)orbel(15)=0.0d0
c
c     compute eccentricity
c
      classe(2)=dsqrt(1.0d0-(orbel(15)/classe(1)))
c
      a1=dsqrt(gmc*dabs(classe(1)))
      a2=dsqrt(gmc*orbel(15))
      a3=rdotv/a1
      x=a3/(1.0d0-(rabs/classe(1)))
c
c     compute period dot
c     if ipdot is zero or three, do not compute period dot
      orbel(3)=0.0d0
c     if ipdot is one --
c     if(ipdot.eq. 1) call perdot(orbel(3))
c
c     compute inclination
c
      arg=dsqrt(h1*h1+h2*h2)
      classe(3)=datan2(arg,h3)
      if(classe(3).lt.0.0d0)classe(3)=classe(3)+twopi
c
c     test semi-major axis for hyperbolic case
c
   20 if(classe(1).le.0.0d0)go to 40
c
c     elliptic case
c
c     compute mean motion
c
      gmroot=dsqrt(gmc)
      expont=-3.0d0/2.0d0
      orbel(4)=gmroot*(classe(1)**expont)
c
c     compute eccentric anomaly
c
      orbel(1)=datan2(a3,(1.0d0-(rabs/classe(1))))
      if(orbel(1).lt.0.0d0)orbel(1)=orbel(1)+twopi
c
c     compute mean anomaly
c
c
c     test eccentricity. if it is zero,set mean anomaly equal to
c         eccentric anomaly.
c
      if(classe(2))22,25,22
   22 classe(6)=orbel(1)-a3
      if(classe(6).lt.0.0d0)classe(6)=classe(6)+twopi
      go to 28
   25 classe(6)=orbel(1)
c
c     compute common multiplier
   28 cmulti=expont*orbel(4)*dterm*(apc**2)/(orbel(15)**2)
c
c     compute argument of perifocus dot
      orbel(8)=-cmulti*(2.0d0-2.5d0*(dsin(classe(3))**2))
c
c     compute ascending node dot
      orbel(9)=cmulti*dcos(classe(3))
c
c     compute period
c
      orbel(2)=twopi*dsqrt((classe(1)*classe(1)*classe(1))/gmc)
c
c     compute vis viva energy
c
      orbel(18)=-gmc/(2.0d0*classe(1))
   30 go to 50
c
c     hyperbolic case
c
c     compute eccentric anomaly
c
   40 orbel(1)=0.5d0*dlog((1.0d0+x)/(1.0d0-x))
c
c   compute mean motion
c
      gmroot  = dsqrt(gmc)
      expont  = -3.0d0 / 2.0d0
      orbel(4) = gmroot * (dabs(classe(1)) ** expont)
c
c     compute mean anomaly
c
      classe(6)=a3-orbel(1)
c
c     compute argument of perifocus dot
c
      orbel(8)=0.0d0
c
c     compute ascending node dot
c
      orbel(9)=0.0d0
c
c     compute period
c
      orbel(2) = twopi * dsqrt((dabs(classe(1) * classe(1) * classe(1)))
     1           / gmc)
c
c     compute vis viva energy
c
      orbel(18)=gmc/(2.0d0*classe(1))
c
c     calculations common to hyperbolic and elliptic cases
c
c     compute longitude of ascending node
c
c
c     test inclination. if it is zero,set longitude of ascending node
c         equal to zero.
c
   50 if(dmod(classe(3),twopi)) 60,70,60
   60 classe(4)=datan2(h1,-h2)
      if(classe(4).lt.0.0d0)classe(4)=classe(4)+twopi
c
c     compute argument of latitude for the case when the inclination
c         is unequal to zero.
c
      arg=datan2((a2*pv(3)),(pv(2)*h1-pv(1)*h2))
      go to 80
   70 classe(4)=0.0d0
c
c     compute argument of latitude for the case when the incl. is zero.
      arg=datan2(pv(2),pv(1))
c
c     compute true anomaly
c
c
   80 orbel(5)=datan2(rdotv*a2,(gmc*(orbel(15)-rabs)))
      if(orbel(5).lt.0.0d0)orbel(5)=orbel(5)+twopi
c
c     compute argument of perifocus
c
c     test eccentricity. if it is zero,set argument of perifocus equal
c         to zero.
c
      if(classe(2))90,100,90
   90 classe(5)=arg-orbel(5)
      if(classe(5).lt.0.0d0)classe(5)=classe(5)+twopi
      go to 110
  100 classe(5)=0.0d0
c
c     compute radius of perifocus
c
  110 orbel(16)=classe(1)*(1.0d0-classe(2))
c
c     compute radius of apofocus
      amajor=2.d0*classe(1)
      factor=1.0d0+classe(2)
      orbel(17)=classe(1)*factor
c
c     compute velocity at apofocus
      orbel(10)=gmroot*dsqrt(factor*(amajor-orbel(17))/(orbel(17)*
     *                               (amajor-orbel(16))))
c
c     compute velocity at perifocus
c
      orbel(11)=gmroot*dsqrt(factor/orbel(16))
c
c     compute perifocal height
c
      orbel(6)=orbel(16)-apc
c
c     compute apofocal height
c
      orbel(7)=orbel(17)-apc
c     ipdot =3, do not compute eval and xyzplh  just return
      if(ipdot.eq.3)go to 999
c
c     compute latitude, longitude, height
c
c     preliminary calculations before call to subroutine xyzplh
c
c***********************************************************************
c
c   these preliminary calculations for xyzplh use the central
c     body of the integrator.  orbel(12,13,14) will therefor be
c    based on that body
c
c***********************************************************************
c
c
c   prepare to rotate to body fixed
c
  205 posit(1) = pv(1)
      posit(2) = pv(2)
      posit(3) = pv(3)
      ibdy(1) = 1
      do 210 i= 2,9
      ibdy(i)= 0
  210 continue
c
c  call to eval removed 2/2/72 in order to prevent problems of calling
c    routine before tzero set.  should only affect the lat, long, and
c    height comp.  remove comments and do loops when tzero1 is set
c    before first call to eleme.
c
c
c      this is being done for the april 6,1972 update
c       thank you
c
c
c
c   if central body is not moon or earth, set orbel(12,13,14) equal to 0
  220 continue
      call bfixed(3,gha,pv,vv,posout,velout,b)
      call ma3331(b,posit,vect)
  230 vect1 = vect(1,1)
      vect2 = vect(2,1)
      vect3 = vect(3,1)
  250 continue
      call xyzplh(esq,vect1,vect2,vect3,orbel(12),orbel(13),ae,
     1     orbel(14),ierr)
c
c
c     spare locations set equal to zero
      orbel(19)=0.0d0
      orbel(20)=0.0d0
  999 return
      end
