      subroutine osmean(oscelo,secele)
c***********************************************************************
c*                                                                     *
c*    version 3.0                                                      *
c*                                                                     *
c***********************************************************************
c*                                                                     *
c*   purpose                                                           *
c*    to convert osculating orbital elements into mean elements        *
c*                                                                     *
c***********************************************************************
c*                                                                     *
c*   reference                                                         *
c*    'conversion of osculating orbital elements into mean elements'   *
c*                          by h.g. walter                             *
c*                   astronomical journal, vol. 72, no. 8              *
c***********************************************************************
c*                                                                     *
c*   input by calling sequence                                         *
c*    oscele(i) - initial osculating elements                          *
c*                                                                     *
c*   output by calling sequence                                        *
c*    secele(i) - mean elements                                        *
c***********************************************************************
c*   input to osmean by way of common                                  *
c*                                                                     *
c*    common const                                                     *
c*        twopi-2*pi                                                   *
c*                                                                     *
c*    common blcnst
c*        cs(i,j)-harmonic coefficients                                *
c*             cs(2,1)=j2                                              *
c***********************************************************************
c*   for oscelo(i), oscele(i), and secele(i),
c*    if i=1, semi-major axis                                          *
c*    if i=2, eccentricity                                             *
c*    if i=3, inclination                                              *
c*    if i=4, longitude of the ascending node                          *
c*    if i=5, argument of perigee                                      *
c*    if i=6, mean anomaly                                             *
c***********************************************************************
c*   programmer/contributor r.a.gordon                 code 582        *
c***********************************************************************
c*   modifications                                                     *
c*    9/21/73 by robert a. gordon                                      *
c*    9/21/73 by robert w. nelson to use constants in blockdata/const/ *
c*    1/15/74 by r. pajerski to check values of eccentricity and       *
c*               j2 before call to brworb                              *
c*    6/1/74  by john todd, to update utilization of -j2 due to        *
c*                       increase of harmonics field to 21x21          *
c*    9/20/74 by viona brown to change variable names                  *
c*    10/24/75 by a.k. kapoor c.s.c.                                   *
c*                       removed the error message #109. instead a     *
c*                       message has been added in routine setorb.     *
c*    nov. 77  by r.a. gordon, walter's algorithm modified to perform  *
c*                       the iterative osculating to mean element      *
c*                       conversion in the cartesian coordinate space. *
c***************************** start osmean ****************************
c
      implicit real*8(a-h,o-z),integer*4(i-n)
c
      common /blcnst/ tto,r,ae,gm,bj2,bj3,bj4,bj5,fltinv,xke,esq
c
      dimension     cs(21,24)      ,dele(6)       ,
     *              dum(5)         ,oscele(6)     ,
     *              oscelo(6)      ,posi(3)       ,
     *              secele(6)      ,veli(3)
      dimension     xyzo(6)   ,xyzm(6)  ,delosc(6),dum1(6)  ,dum2(20)
c
      twopi=6.283185307179586d0
      cs(2,1)=bj2
c
      if(oscelo(2).le.1.0d0) go to 20
c
      do 10 i=1,6
      oscelo(1)=0.0d0
   10 continue
      stop 'error bad excentricty in osmean'
      go to 999
c
c     save current value of j2 for restoration later
c
   20 csave  =  cs(2,1)
c
      if(dabs(csave).ge.1.0d-05) go to 30
      cs(2,1)= -.10826517d-02
c
   30 do 40 i=1,6
   40 secele(i) = oscelo(i)
      ipass =1
      ictype =2
      call eleme(ictype,1,xyzo(1),xyzo(4),oscelo,dum1,dum2)
c
      do 70 i=1,10
      call brworb (posi,veli,oscele,secele,2,ipass,0,dum)
      do 41 k=1,3
      delosc(k) = xyzo(k) - posi(k)
      delosc(k+3) = xyzo(k+3) - veli(k)
   41 continue
      do 42 k=1,6
      if(i.gt.1) go to 42
      xyzm(k) = xyzo(k)
   42 xyzm(k) = xyzm(k) + delosc(k)
      ictype =1
      call eleme(ictype,1,xyzm(1),xyzm(4),secele,dum1,dum2)
      ipass =2
      sdele =0.0d0
c
      do 50 j=2,6
      dele(j) = oscelo(j) - oscele(j)
      sdele   = sdele + dele(j) * dele(j)
   50 continue
      do 60 k=3,6
      angle = dmod(secele(k),twopi)
      if(angle.lt.0.0d0) angle = angle+twopi
      secele(k) = angle
   60 continue
c
      delta = dsqrt(sdele)
c
c    convergence test
c
      dele(1) = oscelo(1)-oscele(1)
      if(dele(1).lt.0.5d-05.and.delta.lt.0.5d-07) go to 80
   70 continue
c
c    restore j2
c
   80 cs(2,1) = csave
c
  999 return
      end
