      subroutine xyzplh(eqs,xsta,ysta,zsta,rlat,rlon,ae,he,ierr)
c
c     version of 3/20/74
c
c        fortran iv subroutine for the ibm-360
c
c        purpose
c
c           to convert station coordinates the earth-fixed cartesian
c           coordinates to geodetic latitude, east longitude,
c           spheroid height
c
c        calling sequence
c           call xyzplh(eqs,xsta,ysta,zsta,rlat,rlon,he,ierr,ae)
c        input
c           eqs -eccentricity of the body squared
c           ae  -semi-major axis
c           xsta-earth-fixed cartesian coordinate x
c           ysta-earth-fixed cartesian coordinate y
c           zsta-earth-fixed cartesian coordinate z
c        output
c           rlat-geodetic latitude
c           rlon-east longitude
c           he  -spheroid height
c           ierr error flag
c              0=height converged
c              1=height did not converge
c              2=longitude is undefined
c
c        method
c           see documentation
c
c        required subprograms
c           datan0
c
c        programmer
c           r. e. gillian, computing and software
c*********start program*************************************************
c
      implicit real*8(a-h,p-z)
      ierr=0
      t=eqs*zsta
      xysq=xsta**2+ysta**2
      if(dabs(zsta).ge.1.0d-15) go to 5
      he = dsqrt(xysq) - ae
      rlat = 0.0 d0
      go to 21
    5 do 10 j = 1, 25
      zt=zsta+t
      h1=dsqrt(xysq+zt**2)
      sinphi=zt/h1
      esqsp=eqs*sinphi
      h2=ae/dsqrt(1.0d0-esqsp*sinphi)
      t1=h2*esqsp
      if(dabs((t1-t)/t1) .lt..1d-14) go to 20
   10 t=t1
      ierr=1
      go to 30
c
   20 he=h1-h2
      rlat=dasin(sinphi)
c
   21 if(xsta.eq.0.0d0)go to 40
      go to 25
   40 if(ysta.eq.0.0d0) ierr=ierr+2
      if(ierr.gt.0) go to 30
      if(ysta.lt.0.0d0) go to 50
      rlon=3.14159265358793/2.0d0
      go to 30
   50 rlon=3.14159265358793*1.5d0
      go to 30
   25 rlon=datan0(ysta,xsta)
   30 continue
      return
      end
