        subroutine satpoc(semaxis,inclin,lan,nodper,motlan,dtime,
     &  pos,lat,lon)
 
!+
!NOM/NAME $Id: satpoc.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE
!       calculates the satellite position in Greenwich reference frame
!       assuming a circular orbit 
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!       calculates the satellite latitude and longitude
!       converts the longitude in interval [-pi,pi]
!       calculates the satellite cartesian coordinates
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
       double precision semaxis ! semi major axis                       (km)
       double precision inclin  ! inclination                           (rd)
       double precision lan     ! longitude of ascending node           (rd)
       double precision nodper  ! nodal period                          (days)
       double precision motlan  ! motion of longitude of ascd. node     (rd/day)
       double precision dtime   ! time since ascending node             (days)  
!
! Output
        double precision pos(3) ! satellite position            (km)
        double precision lat    ! satellite latitude            (rd)
        double precision lon    ! satellite longitude           (rd)
!               the satellite coordinates are in Greenwich reference frame
!
!
!DECLARATIONS
!
!
!
! Local parameters
        double precision pi,twopi               ! pi,  2 * pi
        parameter ( pi = 3.14159265358979311600D0 ,
     &              twopi = 2.d0 * pi)
!
! Commons
!
! Functions
!
! Local variables
        double precision slat, clat     ! sinus and cosinus of the latitude
        double precision arg            ! satellite argument on circular orbit
        double precision sarg, carg     ! argument sinus and cosinus
        double precision cdlon, sdlon   ! sinus and cosinus of the longitude 
!                                         difference satellite-node
! 
!-
!
! calculates the satellite latitude and longitude
! -----------------------------------------------
        arg = twopi * dtime / nodper
        sarg = dsin(arg)
        carg = dcos(arg)
        slat = sarg * dsin(inclin)
        clat = dsqrt( 1.d0 - slat*slat )
        lat = dasin(slat)
        cdlon = carg / clat
        sdlon = - sarg * dcos(inclin) / clat
        lon = lan + motlan * dtime - datan2(sdlon,cdlon)

! converts the longitude in interval [-pi,pi]
!--------------------------------------------  
        if( lon .gt. pi) then
          lon = lon - twopi
        else if( lon .lt. -pi) then
          lon = lon + twopi
        endif

! calculates the satellite cartesian coordinates
!-----------------------------------------------
        pos(1) = semaxis * clat * dcos(lon)
        pos(2) = semaxis * clat * dsin(lon)
        pos(3) = semaxis * slat

        return
        end
