        subroutine wephmes(luout,time,satname,orbit,event,text)
!+
!NOM/NAME $Id: wephmes.F 38 2006-06-16 13:04:03Z frna $
!       subroutine wephmes(luout,time,satname,orbit,event,text)
!BUT/PURPOSE 
!       writes on luout an ephemeris message
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!       the format of the message is:
!       0        1         2         3         4         5         6         7
!       1234567890123456789012345678901234567890123456789012345678901234567890123456789012
!       yyyy/mm/dd  hh:mm:ss.sss nnnnnnnnnnnnnnnn oooooo eeeeeeeeeeeeeeee tttttttttttttttt
!
!       nn..nn satellite name
!       oooooo orbit number
!       ee..ee event
!       tt..tt text
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
! 1.1   june 2006    change field length for satellite to 16 char  P. Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!       cp_lsc          ! last significant character (f7cp)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        integer luout           ! output logical unit
        double precision time   ! julian instant
        character*(*) satname   ! satellite name
        integer orbit           ! orbit number
        character*(*) event     ! ephemeris event string 
        character*(*) text      ! text
!
! Input/Output
!
! Output
!
!
!DECLARATIONS
!
!
!
! Local parameters
!
! Commons
!
! Functions
        integer cp_lsc                  ! last significant character (f7cp)
!
! Local variables
        integer gtime(7)        ! gregorian time
        integer lens            ! character length of satellite
        integer lene            ! character length of event
        character*16 ev_sat      ! output satellite text
        character*16 ev_txt     ! output event text
        character*16 space      ! string of spaces

        data space/'                '/
! 
!-
!
        call xijg(time,gtime)
        lene = cp_lsc(event)
        if( lene .ge. 16 ) then
          ev_txt = event(1:16)
        else
          ev_txt = event(1:lene)//space(lene+1:16)
        endif

        lens = cp_lsc(satname)
        if( lens .eq. 16 ) then
          ev_sat = satname(1:lens)
        elseif( lens .lt. 16 ) then
          ev_sat = satname(1:lens)//space(lens+1:16)
        else
          ev_sat = satname(1:14)//satname(lens-1:lens)
        endif

        write(luout,9000) gtime,ev_sat,orbit,ev_txt,text

9000    format (
     &     i4,'/',i2.2,'/',i2.2,'  ',i2.2,':',i2.2,':',i2.2,'.',i3.3,
     &     1x,a16,1x,i6,1x,a16,1x,a16)
        return

        end
