        subroutine viewvecttolp(
     &     time,sm_rs,isat,inst,reftime,pixel,line,scnang)
!+
!NOM/NAME $Id: viewvecttolp.F 84 2007-05-16 14:52:37Z frna $
!
!BUT/PURPOSE 
!
!       Convert the viewed vector in the spacecraft fixed reference frame Rs
!       into line and pixel numbers
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       This routine replaces the old clinpix subroutine
!           (version 1 of navigation library). 
!       Several types of scanning geometry can be used.
!
!       viewvecttolp is the inverse of lptoviewvect. 
!       see lptoviewvect.F for more details
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995     clinpix        A. Marsouin P.Brunel
! 2.0   01 Jul 2003    new code     A. Marsouin P.Brunel
!       15 Jul 2004    Fix declaration of r_i - "\" and "/" characters in
!                      comments were upsetting g77 compiler  N Atkinson
!       07 Mar 2007    correct several bugs relative to misalignment angles
!                      change the misalignment signs
!
        implicit none                                                     
!     
!ARGUMENTS
        integer isat                    ! satellite index in satid file
        integer inst                    ! instrument index for satid
        double precision reftime        ! julian instant of line 1
        double precision time           ! julian instant of the viewed point
        double precision sm_rs(3)       ! satellite - viewed point unit vector in Rs
! out
        real pixel                      ! pixel number
        real line                       ! line number 
        double precision scnang         ! scanning angle (rad)
!       
!DECLARATIONS
!
!
!
! Local parameters
        double precision pi
        parameter ( pi = 3.14159265358979311600d0)
!
#include "satid.h"
! Commons
!

! Functions
!
! Local variables
        double precision timlin
        double precision alpha
        double precision beta
        double precision delta
        double precision phi
        double precision sm_ri(3) ! satellite - viewed point unit vector in Ri
        double precision y_i !
        double precision r_i ! Yaw Roll Pitch misalignment angles
        double precision p_i !
        double precision x,y
            
        ! 1. calculate viewing vector in Ri frame 
        ! Ri   Instrument Reference frame
        !----------------------------------------
        if (mis_align_mode(inst,isat) .eq. 0) then
           
           ! Yaw Roll Pitch misalignment definition
           if (mis_align_mode_n(inst,isat) .ge. 3) then
              ! yes 3 parameters
              y_i = mis_align_mode_p(1,inst,isat)
              r_i = mis_align_mode_p(2,inst,isat)
              p_i = mis_align_mode_p(3,inst,isat)
           else
              y_i = 0.
              r_i = 0.
              p_i = 0.
           endif

           sm_ri(1) =      sm_rs(1) + p_i*sm_rs(2) - r_i*sm_rs(3)
           sm_ri(2) = -p_i*sm_rs(1) +     sm_rs(2) + y_i*sm_rs(3)
           sm_ri(3) =  r_i*sm_rs(1) - y_i*sm_rs(2) +     sm_rs(3) 
           
        else
           !No misalignment, or unknown
           sm_ri(1) = sm_rs(1)
           sm_ri(2) = sm_rs(2)
           sm_ri(3) = sm_rs(3)
        endif


        ! 2. calculate line and pixel numbers according to scanning function
        !-------------------------------------------------------------------
        if( inst_scan_mode(inst, isat) .eq. 0 ) then
           ! Scanning in a plane, no tilt
           !
           ! definition of scanning function parameters
           ! 1 = scanning step time  (jdays)
           ! 2 = scanning delta time (jdays)
           ! 3 = scanning step angle (rad)
           ! 4 = pixel number at nadir
           ! 5 = step time between two lines (jdays)
           ! the medium pixel is the real pixel value at nadir(number of steps +1)/2.
!!!           alpha = datan2( sm_rs(3), sm_rs(1) )
           alpha = datan2( sm_ri(3), sm_ri(1) )

           pixel = (alpha / inst_scan_mode_p(3,inst,isat)) +
     &             inst_scan_mode_p(4,inst,isat)

           timlin = time - inst_scan_mode_p(2,inst,isat) -
     &             inst_scan_mode_p(1,inst,isat)*(pixel-1)

           line =  (timlin - reftime) / 
     &             inst_scan_mode_p(5,inst,isat) +1
           scnang   = alpha

        elseif( inst_scan_mode(inst, isat) .eq. 1 ) then
           ! Scanning in a plane with tilt
           ! 
           ! definition of scanning function parameters
           ! 1 = scanning step time  (jdays)
           ! 2 = scanning delta time (jdays)
           ! 3 = scanning step angle (rad)
           ! 4 = pixel number at nadir
           ! 5 = step time between two lines (jdays)
           ! the medium pixel is the real pixel value at nadir(number of steps +1)/2.
           ! 6 = tilt angle (rad)
           beta = inst_scan_mode_p(6,inst,isat)

           ! Take care of beta near PI/2
!!!           alpha = datan2( sm_rs(3), sm_rs(1)/dcos(beta) )
           alpha = datan2( sm_ri(3), sm_ri(1)/dcos(beta) )

           pixel = (alpha / inst_scan_mode_p(3,inst,isat)) +
     &             inst_scan_mode_p(4,inst,isat)

           timlin = time - inst_scan_mode_p(2,inst,isat) -
     &             inst_scan_mode_p(1,inst,isat)*(pixel-1)

           line =  (timlin - reftime) / 
     &             inst_scan_mode_p(5,inst,isat) +1
           scnang   = alpha
           
        elseif( inst_scan_mode(inst, isat) .eq. 2 ) then
           ! Conic scanning w/wo tilt
           !
           ! definition of scanning function parameters
           ! 1 = scanning step time  (jdays)
           ! 2 = scanning delta time (jdays)
           ! 3 = scanning step angle, centre to centre FOV (rad)
           ! 4 = pixel number at nadir
           ! 5 = step time between two lines (jdays)
           ! the medium pixel is the real pixel value at nadir(number of steps +1)/2.
           ! 6 = tilt angle (rad)
           ! 7 = half cone angle (rad)
           ! 8 = forward scanning  >= 0
           !     backward scanning < 0
           beta  = inst_scan_mode_p(6,inst,isat)
           delta = inst_scan_mode_p(7,inst,isat)

           ! take care |delta| > 0
           if( delta .ge. 1e-04 ) then
              x = (sm_ri(2) - dsin(beta)*dcos(delta) ) / 
     &                 (dcos(beta)*dsin(delta))
              y =  sm_ri(3) / dsin(delta)
           else
              x = 1.
              y = 0.
           endif

           phi = datan2( y , x )
           ! datan2 returns phi in  [-pi, +pi[

           if ( inst_scan_mode_p(8,inst,isat)  .ge. 0. ) then
              ! forward scanning 
              scnang = phi -pi
              if( scnang .lt. -pi ) then
                 scnang = scnang + 2.*pi
              end if
           else
              ! backward scanning
              scnang = phi
           endif

           pixel = (scnang / inst_scan_mode_p(3,inst,isat)) +
     &             inst_scan_mode_p(4,inst,isat)

           timlin = time - inst_scan_mode_p(2,inst,isat) -
     &             inst_scan_mode_p(1,inst,isat)*(pixel-1)

           line =  (timlin - reftime) / 
     &             inst_scan_mode_p(5,inst,isat) +1

        else
           pixel = -999.
           line  = -999.
        end if

        return
        end
