        subroutine calatt(yaw,roll,pitch,att)
!+
!NOM/NAME $Id: calatt.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!
!       attitude error matrix calculation for small yaw, roll and pitch angles
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       Definitions of the reference frames
!       -----------------------------------
!
!       Nominal attitude reference frame (Ra)
!         approximate definition
!         X: parallel to the satellite geographic vertical, toward the earth
!         Z: normal to X and to the satellite velocity
!         Y: normal to X and Z (nearly opposite to the velocity)
!         precise definition depends on the attitude mode.
!
!       Spacecraft fixed reference frame (Rs):
!         3 axis linked to the spacecraft structure
!         obtained from Sat_vel_vert by 3 rotations in the following order:
!           yaw rotation around X axis
!           roll rotation around Y axis
!           pitch rotation around Z axis
!       
!       The attitude error matrix permits the Ra <--> Rs transformations
!
!       Transformation of vector U_v in Ra into vector U_s in Rs
!               U_s = att * U_v
!       U_s(1) = att(1,1)*U_v(1) + att(1,2)*U_v(2) + att(1,3)*U_v(3) 
!       U_s(2) = att(2,1)*U_v(1) + att(2,2)*U_v(2) + att(2,3)*U_v(3) 
!       U_s(3) = att(3,1)*U_v(1) + att(3,2)*U_v(2) + att(3,3)*U_v(3) 
!
!       Transformation of vector U_s in Rs into vector U_v in Ra
!               U_v = tranpose of att * U_s
!       U_v(1) = att(1,1)*U_s(1) + att(2,1)*U_s(2) + att(3,1)*U_s(3) 
!       U_v(2) = att(1,2)*U_s(1) + att(2,2)*U_s(2) + att(3,2)*U_s(3) 
!       U_v(3) = att(1,3)*U_s(1) + att(2,3)*U_s(2) + att(3,3)*U_s(3) 
!
!       Approximation for small angles:
!               |     1         pitch   -roll   |
!       att =   |  -pitch         1      yaw    |
!               |   roll        -yaw      1     |       
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no                     A. Marsouin P.Brunel
!       01 April 2004    comments changed       A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        double precision yaw            ! yaw error                     (rad)
        double precision roll           ! roll error                    (rad)
        double precision pitch          ! pitch error                   (rad)
! Output
        double precision att(3,3)       ! attitude error matrix         (rad)
!
!
!DECLARATIONS
!
!
!
! Local parameters
!
! Commons
!
! Functions
!
! Local variables
! 
!-
!
          att(1,1) =   1.d0
          att(1,2) =   pitch
          att(1,3) = - roll
          att(2,1) = - pitch
          att(2,2) =   1.d0
          att(2,3) =   yaw
          att(3,1) =   roll
          att(3,2) = - yaw
          att(3,3) =   1.d0

        return

        end
