        subroutine calatt_exact(yaw,roll,pitch,att)
!+
!NOM/NAME $Id: calatt_exact.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!
!       exact attitude error matrix calculation for yaw, roll and pitch angles
!       the order of the 3 rotations is yaw, then roll, then pitch
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       Definitions of the reference frames
!       -----------------------------------
!
!       Nominal attitude reference frame (Ra)
!         approximate definition
!         X: parallel to the satellite geographic vertical, toward the earth
!         Z: normal to X and to the satellite velocity
!         Y: normal to X and Z (nearly opposite to the velocity)
!         precise definition depends on the attitude mode.
!
!       Spacecraft fixed reference frame (Rs):
!         3 axis linked to the spacecraft structure
!         obtained from Sat_vel_vert by 3 rotations in the following order:
!           yaw rotation around X axis
!           roll rotation around Y axis
!           pitch rotation around Z axis
!       
!       The attitude error matrix permits the Ra <--> Rs transformations
!
!       Transformation of vector U_a in Ra into vector U_s in Rs
!               U_s = att * U_v
!       U_s(1)   | att(1,1)  att(1,2)  att(1,3) |     U_a(1) 
!       U_s(2) = | att(2,1)  att(2,2)  att(2,3) |  x  U_a(2) 
!       U_s(3)   | att(3,1)  att(3,2)  att(3,3) |     U_a(3) 
!
!       Transformation of vector U_s in Rs into vector U_a in Ra
!               U_a = tranpose of att * U_s
!
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 2003      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        double precision yaw            ! yaw error                     (rad)
        double precision roll           ! roll error                    (rad)
        double precision pitch          ! pitch error                   (rad)
! Output
        double precision att(3,3)       ! attitude error matrix         (rad)
!
!
!DECLARATIONS
!
!
!
! 
!-
!
          att(1,1) =  dcos(pitch) * dcos(roll)
          att(1,2) =  dcos(pitch) * dsin(roll) * dsin(yaw) +
     &                dsin(pitch) * dcos(yaw)
          att(1,3) = -dcos(pitch) * dsin(roll) * dcos(yaw) +
     &                dsin(pitch) * dsin(yaw)
          att(2,1) = -dsin(pitch) * dcos(roll)
          att(2,2) = -dsin(pitch) * dsin(roll) * dsin(yaw) +
     &                dcos(pitch) * dcos(yaw)
          att(2,3) =  dsin(pitch) * dsin(roll) * dcos(yaw) +
     &                dcos(pitch) * dsin(yaw)
          att(3,1) =  dsin(roll)
          att(3,2) = -dcos(roll)  * dsin(yaw)
          att(3,3) =  dcos(roll)  * dcos(yaw)

        return

        end
