        subroutine cartlalo(nbp, pos, lat, lon)
!+
!NOM/NAME $Id: cartlalo.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!
!       converts cartesian coordinates into latitude-longitude for a list
!       of points on the earth surface (i.e. altitude = 0)
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions
! 
!       input and output coordinates are expressed in Greenwich reference frame:
!         X: in equatorial plane toward Greenwich meridian
!         Y: in equatorial plane toward lon = 90 degrees east
!         Z: terrestrial polar axis
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 2003      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        integer nbp                     ! number of calculations to be done
        double precision pos(3,nbp)     ! array of cartesian positions  (km)
!
! Output
        double precision lat(nbp)       ! array of geographic latitudes (rad)
        double precision lon(nbp)       ! array of longitudes           (rad)
!
!
!DECLARATIONS
!
!
!
! Local parameters
!
        double precision pi, pi_2       ! pi,  pi/2
        parameter ( pi = 3.14159265358979311600D0 ,
     &              pi_2 = pi / 2.d0)

!
! constants of the earth ellipsoid and rotation
#include "p_earth.h"
!
        double precision epsild         ! threshold in distance (km) to assess 
        parameter (epsild = 1.d-02)     ! north or south pole to the viewed point
!
!
! Local variables
        integer i                       ! loop variable
        double precision projeq         ! projection on equatorial plane of om_g
! 
!-
!
        do i=1,nbp

            projeq = dsqrt(pos(1,i)*pos(1,i) + pos(2,i)*pos(2,i))
            if ( projeq.gt.epsild ) then
              lat(i) = datan ( pos(3,i) * re2_rp2 / projeq )
              lon(i) = datan2( pos(2,i) , pos(1,i) )
            else if (pos(3,i) .gt. 0.d0) then
              lat(i) = pi_2
              lon(i) = 0.d0
            else
              lat(i) = - pi_2
              lon(i) = 0.d0
           endif

        enddo
        return
        end
