        subroutine def_att(scname,att_mode,d_yaw,d_roll,d_pitch)
!+
!NOM/NAME $Id: def_att.F 116 2008-06-27 09:17:04Z frna $
!
!BUT/PURPOSE 
!        returns the attitude mode and the default values of attitude angles 
!       read in the satellite identification file (SATID)
!
!DESCRIPTION 
!        FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       The satellite name is an input and is used to calculate an index
!       in the satid file.
!       If a satpos file has been read (common c_satpos), the bulletin
!       name is used to choose the attitude angle default values.
!       If no satpos file, the first set of attitude angle default values
!       in the satid file is taken
!
!       If no information available in the satid file, 0 values are assumed
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
! 1.1   July 2002     default values for Noaa16    A. Marsouin P.Brunel
! 2.0   01 Jul 2003   entirely new version         A. Marsouin P.Brunel
!                     using satid file instead of hard coded values
! 2.1   15 Jul 2004   Some compilers don't like 'string'//scname. N C Atkinson
! 2.2   26 Jun 2008   Add ind_bul = 4, for Spot bulletins. NCA
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
! genscid cp_cus
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!     
!ARGUMENTS
! inputs
        character*(*) scname                ! satellite name
!  outputs
        integer att_mode                ! attitude mode
                                        ! 0 = NOAA POES operational mode
                                        ! 1 = METOP Yaw steering mode
                                        ! 2 = Geocentric mode
        double precision d_yaw                ! default value of yaw (rad)
        double precision d_roll                ! default value of roll (rad)
        double precision d_pitch        ! default value of pitch (rad)
!        
!DECLARATIONS
!
!
!
! Local parameters
!
! Commons
#include "c_satpos.h"
#include "satid.h"
!

! Functions
        integer genscid                        ! libsatid
        logical cp_cus
!
! Local variables
        integer i
        integer ind_bul
        integer ind
        integer isat
        character*80  str
! 
!-
!
        ! search the index in the satid file corresponding
        ! to the satellite name
        isat=  genscid(scname)
        if(isat .eq. 0) then
           ! error unknown satellite
           ! assume 0. for all attitude angles
           str = scname        !2.1: str is fixed length
           call ml_wt('info','unknown satellite: '//str)
           call ml_wt('info','default attitude is 0')
           d_yaw   = 0.d0
           d_roll  = 0.d0
           d_pitch = 0.d0
           return
        end if

        ! nominal attitude mode
        att_mode = sc_att_mode(isat)

        ! search the index in the satid file corresponding
        ! to the bulletin type
        ! if it fails (satpos file not read) then
        !    take the first bulletin in satid file
        if ( sc_att_bias_n(isat) .gt. 0 ) then
           if(cp_cus(sp_bultyp,'tbus')) then
              ind_bul = 1
           elseif(cp_cus(sp_bultyp,'argos')) then
              ind_bul = 2
           elseif(cp_cus(sp_bultyp,'tle')) then
              ind_bul = 3
           elseif(cp_cus(sp_bultyp,'spm')) then
              ind_bul = 4
           else
              ind_bul = 0
           endif
           if(ind_bul .gt. 0 ) then
              do i = 1,  sc_att_bias_n(isat) 
                 if( sc_att_bias_bid(i,isat) .eq. ind_bul ) then
                    ind = i
                 end if
              end do
           else
              ind = 1
           end if
        else
           ind = 0
        end if

        if(ind .gt. 0 .and. ind .le. sc_att_bias_n(isat)) then
           d_yaw   = sc_att_bias(1,ind,isat)
           d_roll  = sc_att_bias(2,ind,isat)
           d_pitch = sc_att_bias(3,ind,isat)
        else
           d_yaw   = 0.d0
           d_roll  = 0.d0
           d_pitch = 0.d0
        end if

        return
        end
