        subroutine earthpix(
     >             pos, att, Mat_Ra_Rg, smu_s,
     >             om_g, ierr)
!+
!NOM/NAME $Id: earthpix.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!       calculates the cartesian coordinates of the viewed pixel
!       in the Greenwich reference frame
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 2003      no          A. Marsouin P.Brunel
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS
!       Rs is the spacecraft fixed reference frame
!       Ra is the nominal attitude refernce frame
!       Rg is the Greenwich reference frame
!
!input 
        double precision pos(3)         ! Greenwich satellite position  (km)
        double precision att(3,3)       ! attitude error matrix         (rad)
        double precision Mat_Ra_Rg(3,3) ! Ra to Rg conversion matrix
        double precision smu_s(3)       ! satellite - viewed point unit vector in Rs
! output
        double precision om_g(3)        ! cartesian coordinates of viewed point in Rg
        integer ierr                    ! return code 
!       0 = OK  
!       3 = internal error, no solution for the viewed point

!DECLARATIONS
!
!
!
! Local parameters

! constants of the earth ellipsoid
! constants of the earth ellipsoid and rotation
#include "p_earth.h"
!
! Local variables       (distances in km, angles in radians)
! 
!                                         satellite - viewed point unit vector:
        double precision smu_g(3)       !   in Greenwich reference frame
        double precision smu_v(3)       !   in vel_vert reference frame
        double precision smdis          ! distance satellite - viewed point (km)
        double precision a,b,c,delta    ! intermediate variables

!
!-
        ierr = 0

! transform into Rv using attitude error matrix
          smu_v(1) = att(1,1)*smu_s(1) + att(2,1)*smu_s(2) 
     >             + att(3,1)*smu_s(3)
          smu_v(2) = att(1,2)*smu_s(1) + att(2,2)*smu_s(2) 
     >             + att(3,2)*smu_s(3)
          smu_v(3) = att(1,3)*smu_s(1) + att(2,3)*smu_s(2) 
     >             + att(3,3)*smu_s(3)

! transform into Rg
          smu_g(1) = Mat_Ra_Rg(1,1)*smu_v(1) + 
     >               Mat_Ra_Rg(2,1)*smu_v(2) +
     >               Mat_Ra_Rg(3,1)*smu_v(3)
          smu_g(2) = Mat_Ra_Rg(1,2)*smu_v(1) + 
     >               Mat_Ra_Rg(2,2)*smu_v(2) +
     >               Mat_Ra_Rg(3,2)*smu_v(3)
          smu_g(3) = Mat_Ra_Rg(1,3)*smu_v(1) + 
     >               Mat_Ra_Rg(2,3)*smu_v(2) +
     >               Mat_Ra_Rg(3,3)*smu_v(3)


! calculates the distance satellite - viewed point
! the viewed point is on the terrestrial ellipsoid
! the distance is the smaller root of a 2 order equation

          a =   smu_g(1) * smu_g(1) + smu_g(2) * smu_g(2)
     >        + smu_g(3) * smu_g(3) * re2_rp2
          b =   pos(1) * smu_g(1) + pos(2) * smu_g(2) 
     >        + pos(3) * smu_g(3) * re2_rp2
          c = pos(1) * pos(1) + pos(2) * pos(2) 
     >        + pos(3) * pos(3) * re2_rp2 - re2
          delta = b * b - a * c
          if (delta.gt.0.and.a.ne.0) then
            smdis = ( -b - dsqrt(delta) ) / a 

! calculates the viewed point coordinates in Rg
            om_g(1) = pos(1) + smdis * smu_g(1)
            om_g(2) = pos(2) + smdis * smu_g(2)
            om_g(3) = pos(3) + smdis * smu_g(3)
            
          else
            om_g(1) = 0.
            om_g(2) = 0.
            om_g(3) = 0.
            ierr = 3
          endif



        return
        end
