        subroutine geogcart(lat,lon,alt,pos)
!+
!NOM/NAME $Id: geogcart.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!
!       converts geographical coordinates into cartesian coordinates
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions
! 
!       input and output coordinates are expressed in Greenwich reference frame:
!         X: in equatorial plane toward Greenwich meridian
!         Y: in equatorial plane toward lon = 90 degrees east
!         Z: terrestrial polar axis
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        double precision lat            ! geographic latitude           (rad)
        double precision lon            ! longitude                     (rad)
        double precision alt            ! altitude                      (km)
!
! Output
        double precision pos(3)         ! cartesian position            (km)
!
!
!DECLARATIONS
!
!
!
! Local parameters
!
! constants of the earth ellipsoid and rotation
#include "p_earth.h"
!
! Commons
!
! Functions
!
! Local variables
        double precision n              ! normal to ellipsoid 
        double precision slat, clat     ! latitude sinus and cosinus 
! 
!-
!
        slat = dsin(lat)
        clat = dcos(lat)
        n = req / dsqrt( 1.d0 - exct2 * slat * slat)
        pos(1) = ( n + alt ) * clat * dcos(lon)
        pos(2) = ( n + alt ) * clat * dsin(lon)
        pos(3) = ( n * (1 - exct2) + alt ) * slat

        return
        end
