        subroutine gstatc(nbs,station,ierr)
!+
!NOM/NAME $Id: gstatc.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!       gets geographical informations from an ascii file
!               for a list of ground station names 
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!       gstatc opens the file stations.txt on the directory DIR_STATIONS
!       given by an environment variable.
!       If the variable is not present the file is opened on the current
!       directory.
!       It reads the file and searches the latitude, longitude and altitude
!       of the stations. Then it computes the greenwich cartesian coordinates
!       and values useful for the tracking angles calculations.
!       all values are stored in the common /c_stat/
!
!       In the ascii file any comment line should begin by a # character.
!       They must be at the head of the file.
!
!       Remark: unit 99 is used for file I/O
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!       cp_ssf          f7_cp   subsitute the white spaces in a character string
!       cp_cus          f7_cp   string comparison (true if equal)
!       initrack        navtool calculates values for the tracking angles calc.
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!ARGUMENTS 
! Input
        integer nbs                     ! number of stations
        character*(*) station(nbs)      ! ground station name
! Ouput
        integer ierr            ! return code = the number of decoded stations
!
!DECLARATIONS
!
!
!
! Common declarations:
!
#include "c_stations.h"
!
! Local parameters
        integer unit         ! unit for file reading
        parameter (unit=99)
        double precision pi
        parameter ( pi = 3.14159265358979311600d0)
!
! Local variables:
        integer ios          ! iostat I/O check variable
        integer kstat        ! loop variable for the stations
        character*80 chdum   ! dummy character variable
        character*256 path   ! pathname for the data file
        character*256 name   ! fullname of the data file

!       variables for the file reading
        character*64 fnamestat          ! ground station name
        double precision flatstat       ! latitude
        double precision flonstat       ! longitude
        double precision faltstat       ! altitude
        double precision fsitestat      ! minimum site for the antenna
! 
! Functions
        logical cp_cus
!-
!
        name='stations.txt'
!============== NOT STANDARD FOR ATOV ======START============
!       get the environnement variable for the path of the data file
!       and create the filename
        path = ' '
        call getenv('DIR_STATIONS',path)
        if(path.ne.' ') then
          call cp_ssf(path//'/stations.txt',name,' ')
        endif
!============== NOT STANDARD FOR ATOV =======END===========
!
        nbstat = 0
        ierr = 0
!
! open file and check for I/O errors
! data store is aborted on detection of error
!
        open (unit, file=name, status='old', iostat=ios)
        if( ios.eq.0 ) then

!         for all the input stations
!       ----------------------------
          do kstat = 1,nbs

          rewind unit

!         skip the header lines
          read(unit,'(a80)',iostat=ios) chdum
          do while ( ios .eq. 0 .and. chdum(1:1).eq. '#' )
            read(unit,'(a80)',iostat=ios) chdum
          enddo
          if(ios.ne.0) then
            call ml_wti('error',
     &          'in reading the stations file I/O status ',ios)
          endif
          backspace unit
!
!         find the record corresponding to the station 
!         read(unit,*,iostat=ios) flatstat, flonstat, faltstat, 
!     &                            fsitestat, fnamestat
          read(unit,'(f7.3,1x,f8.3,1x,2(f6.3,1x),a)',iostat=ios) 
     &                            flatstat, flonstat, faltstat, 
     &                            fsitestat, fnamestat

          do while ( ios.eq.0  .and. 
     &         .not.cp_cus(station(kstat),fnamestat) )

!           read the next line of information
!           read(unit,*,iostat=ios) flatstat, flonstat, faltstat, 
!     &                              fsitestat, fnamestat
            read(unit,'(f7.3,1x,f8.3,1x,2(f6.3,1x),a)',iostat=ios) 
     &                            flatstat, flonstat, faltstat, 
     &                            fsitestat, fnamestat
          enddo

          if( ios.eq.0 .and. nbstat.lt.maxstat ) then

!           the station is found, converts the angles into radians
            nbstat = nbstat + 1
            latstat(nbstat) = flatstat*pi/180.d0
            lonstat(nbstat) = flonstat*pi/180.d0
            sitestat(nbstat) = fsitestat*pi/180.d0
            altstat(nbstat) = faltstat          
            namestat(nbstat) = fnamestat

!           now computes the coordinates
            call initrack(latstat(nbstat), lonstat(nbstat), 
     &          altstat(nbstat), 
     &          posstat(1,nbstat), matstat(1,1,nbstat))

          endif

          end do
!
        endif

        close (unit)

        ierr = nbstat

        return
        end
