        subroutine lalocart(nbp,lat,lon,pos,ierr)
!+
!NOM/NAME $Id: lalocart.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!
!       converts latitude-longitude into cartesian coordinates for a list
!       of points on the earth surface (i.e. altitude = 0)
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions
! 
!       input and output coordinates are expressed in Greenwich reference frame:
!         X: in equatorial plane toward Greenwich meridian
!         Y: in equatorial plane toward lon = 90 degrees east
!         Z: terrestrial polar axis
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        integer nbp                     ! number of calculations to be done
        double precision lat(nbp)       ! array of geographic latitudes (rad)
        double precision lon(nbp)       ! array of longitudes           (rad)
!
! Output
        double precision pos(3,nbp)     ! array of cartesian positions  (km)
        integer ierr                    ! return code 
!               0 ===> OK   n ===> n erroneous latitude values
!
!
!DECLARATIONS
!
!
!
! Local parameters
!
        double precision pi, pi_2       ! pi,  pi/2
        parameter ( pi = 3.14159265358979311600D0 ,
     &              pi_2 = pi / 2.d0)

!
! constants of the earth ellipsoid and rotation
#include "p_earth.h"
!
! Commons
!
! Functions
!
! Local variables
        integer i                       ! loop variable
        double precision latgeoc        ! geocentric latitude 
        double precision cla            ! cosinus of geocentric latitude
        double precision earthrad       ! earth radius at the current point
 
! 
!-
!
        ierr = 0

        do i=1,nbp

! erroneous latitude value
          if( dabs(lat(i)).ge.(pi_2 + 1.d-6) ) then
            ierr = ierr + 1

! normal case
          else if( dabs(lat(i)).le.(pi_2 - 1.d-6) ) then
            latgeoc = datan( dtan(lat(i)) * rp2_re2 )
            cla = dcos(latgeoc)
            earthrad = rpo / dsqrt( 1.d0 - cla*cla*exct2)
            pos(1,i) = earthrad * cla * dcos(lon(i))
            pos(2,i) = earthrad * cla * dsin(lon(i))
            pos(3,i) = earthrad * dsin(latgeoc)

! north pole
          else if( lat(i).ge.0.d0 ) then
            pos(1,i) = 0.d0  
            pos(2,i) = 0.d0  
            pos(3,i) = rpo

! south pole
          else
            pos(1,i) = 0.d0  
            pos(2,i) = 0.d0  
            pos(3,i) = - rpo
          endif

        enddo
        return
        end
