        double precision function prenut(time)
!+
!NOM/NAME $Id: prenut.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!       calculates the rotation angle in radians
!       from the quasi-inertial 1950 reference frame (Veis reference frame)
!       to the inertial true-of-date reference frame (ITOD)
!
!       ITOD right ascension = 1950 right ascension - prenut
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       rotation angle = precession + nutation in right ascension
!       formulas supplied by CNES
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        double precision time           ! julian instant
!
!
!DECLARATIONS
!
!
!
! Local parameters
        double precision a1, a2, a3     ! constants for precession calculation
        double precision b1, b2, b3, b4 ! constants for nutation calculation
        double precision d1, d3, d4     ! constants for nutation calculation
        double precision e1, e3, e4     ! constants for nutation calculation
        
        data a1,a2,a3 /.6119074d-06,.507847d-14,.361458d-20/
        data b1,b2,b3,b4 /-.7665d-04,.925d-06,-.5672d-05,-.873d-06/
        data d1,d3,d4 /.211408d0,4.888339d0,1.12368d0/
        data e1,e3,e4 /-.92422d-03,.1720279d-01,.229971529d0/
!
! Commons
!
! Functions
!
! Local variables
        double precision pre            ! precession (rad)
        double precision nut            ! nutation (rad)
! 
!-
!
!       right ascension precession
        pre = a1*time + a2*time*time + a3*time*time*time

!       right ascension nutation
        nut = b1*dsin(d1+e1*time) + b2*dsin(2.d0*(d1+e1*time))
     >        + b3*dsin(2.d0*(d3+e3*time)) + b4*dsin(2.d0*(d4+e4*time))

        prenut = - pre - nut

        return
        end
