        subroutine pvi50grw(time,posi50,veli50,posgrw,velgrw)
!+
!NOM/NAME $Id: pvi50grw.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!
!       transforms the satellite position and velocity
!       from the quasi-inertial 1950 reference frame (Veis reference frame)
!       to the Greenwich reference frame
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       transformation from 1950 to Greenwich is a rotation along z axis
!       the output velocity is a relative velocity in a rotating reference frame
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!       sidert50: sidereal time of the 1950 vernal equinox
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        double precision time           ! julian instant
        double precision posi50(3)      ! position in 1950 reference frame
        double precision veli50(3)      ! velocity in 1950 reference frame
! Output
        double precision posgrw(3)      ! position in Greenwich reference frame 
        double precision velgrw(3)      ! velocity in Greenwich reference frame
!
!
!DECLARATIONS
!
!
!
! Local parameters
        double precision earthrot               ! earth rotation in rad / sec
        parameter ( earthrot=7.292115147d-5 )   ! = 6.300387486753296 rad /day
!
! Commons
!
! Functions
        double precision sidert50
!
! Local variables
        double precision angle          ! rotation angle from Greenwich to 1950
! 
!-
!
! calculates the rotation angle from 1950 to Greenwich reference frame
        angle = sidert50(time)

! conversion from 1950 to Greenwich reference frame
        call rotz2v(angle,posi50,veli50,posgrw,velgrw)

! calculation of the relative velocity in Greenwich reference frame
        velgrw(1) =  velgrw(1) + earthrot * posgrw(2)
        velgrw(2) =  velgrw(2) - earthrot * posgrw(1)

        return

        end
