        subroutine pvitodgrw(time,positod,velitod,posgrw,velgrw)
!+
!NOM/NAME $Id: pvitodgrw.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!
!       transforms the satellite position and velocity
!       from inertial true-of-date reference frame (ITOD)
!       to Greenwich reference frame
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       the transformation from ITOD to Greenwich is a rotation along z axis
!       the output velocity is a relative velocity in a rotating reference frame
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!       sidert50: sidereal time of the 1950 vernal equinox
!       prenut: precession + nutation from 1950 to ITOD vernal equinox
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        double precision time           ! julian instant
        double precision positod(3)     ! position in ITOD reference frame
        double precision velitod(3)     ! velocity in ITOD reference frame
! Output
        double precision posgrw(3)      ! position in Greenwich reference frame 
        double precision velgrw(3)      ! velocity in Greenwich reference frame
!
!
!DECLARATIONS
!
!
!
! Local parameters
        double precision earthrot               ! earth rotation in rad / sec
        parameter ( earthrot=7.292115147d-5 )   ! = 6.300387486753296 rad /day
!
! Commons
!
! Functions
        double precision sidert50
        double precision prenut  
!
! Local variables
        double precision angle          ! rotation angle from Greenwich to ITOD
! 
!-
!
! calculates the rotation angle from ITOD to Greenwich reference frame
! sidereal time         = 1950 inertial reference frame to Greenwich 
! precession + nutation = 1950 to ITOD reference frame
        angle = sidert50(time) - prenut(time)

! conversion from ITOD to Greenwich reference frame
        call rotz2v(angle,positod,velitod,posgrw,velgrw)

! calculation of the relative velocity in Greenwich reference frame
        velgrw(1) =  velgrw(1) + earthrot * posgrw(2)
        velgrw(2) =  velgrw(2) - earthrot * posgrw(1)

        return

        end
