      subroutine pvj2000grw ( 
     &   time, ut1_utc, tai_utc, pmx, pmy, 
     &   posj2000, velj2000, posgrw, velgrw )
!+
!NOM/NAME  $Id: pvj2000grw.F 26 2006-03-01 12:28:20Z frna $        
!
!BUT/PURPOSE 
!
!        transforms the satellite position and velocity
!        from J2000  reference frame (EME2000)
!        to Greenwich reference frame
!       J2000 is the reference frame for SPOT-5 outputs 
!
!DESCRIPTION 
!        FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   10 October 2005      no          A. Marsouin P.Brunel
!         
!
!
!COPYRIGHT (c) NWPSAF Meteo-France (EUMETSAT project <2005>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        double precision time                ! julian instant
        Real     pmx          ! polar motion x coord (arcsec)
        Real     pmy          ! polar motion y coord (arcsec)
        Real     ut1_utc      ! UT1 - UTC (sec)
        Real     tai_utc      ! TAI - UTC (sec)
        double precision posj2000(3)        ! position in J2000 reference frame
        double precision velj2000(3)        ! velocity in J2000 reference frame
! Output
        double precision posgrw(3)        ! position in Greenwich reference frame 
        double precision velgrw(3)        ! velocity in Greenwich reference frame
!
!
!DECLARATIONS
!
!
! Local parameters
!
!
! Functions
!
! Local variables
      integer ijul1950jour
      double precision rjul1950sec
      integer imodr                    ! precession/nutation model
      double precision dtu1, dtai      ! UT1 - UTC (sec) TAI - UTC (sec)
      double precision uref, vref      ! polar motion x/y coord (radians)

      double precision posvel_j2000(6) ! position velocity in J2000
      double precision posvel_veis(6)  ! position velocity in Veis
      double precision posvel_tv(6)    ! position velocity in True Earth
      double precision posvel_ef(6)    ! position velocity in Earth Fixed

      integer ier
        double precision pi
        parameter ( pi = 3.14159265358979311600d0)
        double precision crd
        parameter (crd=180.d0/pi)
! 
!-
!
        ! ijul1950jour number of days of the date
        ! rjul1950sec  number of seconds of the date
        ijul1950jour = time
        rjul1950sec  = (time - ijul1950jour) * 86400.d0

        ! indicateur des modeles de precession et de nutation
        !    = 1 : NEWCOMB(B1900) + WOOLARD(J1900)
        !    = 2 : LIESKE(J2000) + WAHR(J2000)
        imodr=2

        ! precision conversion for UT1 TAI times
        dtu1 = ut1_utc
        dtai = tai_utc
        
        ! polar motion x/y coord (radians)
        uref = pmx / crd /  3600.d0
        vref = pmy / crd /  3600.d0

        ! conversion J2000 to VEIS
          posvel_j2000(1) = posj2000(1)
          posvel_j2000(2) = posj2000(2)
          posvel_j2000(3) = posj2000(3)
          posvel_j2000(4) = velj2000(1)
          posvel_j2000(5) = velj2000(2)
          posvel_j2000(6) = velj2000(3)
          call mrchgrepj2000v (imodr, IJUL1950JOUR, RJUL1950SEC,
     &                          dtu1, dtai, posvel_j2000,
     &                          posvel_veis, ier)
          If( ier .ne.0 ) Then
             Write(*,*) 'code retour J2000 veis ',ier
             Stop
          End If

        ! conversion VEIS to True Earth
          call mrchgrepveistv (IJUL1950JOUR, RJUL1950SEC,
     &                           dtu1, posvel_veis, posvel_tv, ier)
          If( ier .ne.0 ) Then
             Write(*,*) 'code retour  VEIS to True Earth',ier
             Stop
          End If

        ! conversion True Earth to Earth Fixed (Greenwich)
          call mrchgreptvtref (uref, vref, posvel_tv, posvel_ef, ier)
          If( ier .ne.0 ) Then
             Write(*,*) 'code retour True Earth to Earth Fixed  ',ier
             Stop
          End If

          posgrw(1) = posvel_ef(1)
          posgrw(2) = posvel_ef(2)
          posgrw(3) = posvel_ef(3)
          velgrw(1) = posvel_ef(4)
          velgrw(2) = posvel_ef(5)
          velgrw(3) = posvel_ef(6)

        return

        end
