        subroutine pvtemegrw(time,posteme,velteme,posgrw,velgrw)
!+
!NOM/NAME $Id: pvtemegrw.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!
!       transforms the satellite position and velocity
!       from True Equator, Mean Equinox of the date reference frame (TEME)
!       to Greenwich reference frame
!       TEME is the reference frame for SGP outputs and 2-Lines elements
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       the transformation from TEME to Greenwich is a rotation along z axis
!       the output velocity is a relative velocity in a rotating reference frame
!
!       the transformation from TEME to Greenwich is approximated by 
!       rotation along Z axis with 2 components:
!       - sidereal time of the 1950 vernal equinox
!       - precession from 1950 to TEME vernal equinox
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   27 Dec 2001      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!       sidert50: sidereal time of the 1950 vernal equinox
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        double precision time           ! julian instant
        double precision posteme(3)     ! position in TEME reference frame
        double precision velteme(3)     ! velocity in TEME reference frame
! Output
        double precision posgrw(3)      ! position in Greenwich reference frame 
        double precision velgrw(3)      ! velocity in Greenwich reference frame
!
!
!DECLARATIONS
!
!
!
! Local parameters
        double precision earthrot               ! earth rotation in rad / sec
        parameter ( earthrot=7.292115147d-5 )   ! = 6.300387486753296 rad /day
! 
        double precision a1, a2, a3     ! constants for precession calculation
        data a1,a2,a3 /.6119074d-06,.507847d-14,.361458d-20/
!
! Commons
!
! Functions
        double precision sidert50
!
! Local variables
        double precision angle          ! rotation angle from Greenwich to TEME
        double precision preces         ! precession angle in radians
! 
!-
!
! calculates the rotation angle from TEME to Greenwich reference frame
! sidereal time         = 1950 inertial reference frame to Greenwich 
! precession            = 1950 to TEME reference frame
        preces = - a1*time - a2*time*time - a3*time*time*time
        angle = sidert50(time) - preces

! conversion from TEME to Greenwich reference frame
        call rotz2v(angle,posteme,velteme,posgrw,velgrw)

! calculation of the relative velocity in Greenwich reference frame
        velgrw(1) =  velgrw(1) + earthrot * posgrw(2)
        velgrw(2) =  velgrw(2) - earthrot * posgrw(1)

        return

        end
