        double precision function refrac(site) 
!+
!NOM/NAME $Id: refrac.F 25 2006-02-03 15:43:53Z frna $
!     double precision function refrac(site)
!
!BUT/PURPOSE 
!       returns the refraction correction (rad) of site angle
!       corrected site = site + refrac(site)
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!       this function is based on results from MODTRAN2 (AFGL) for midlatitude
!       atmospheres.
!       Modtran has been run for site angles from 0 to 30 deg (1 deg increment)
!       and from 30 to 90 deg (5 deg increment)
!
!       The function is separated in two intervals:
!
!       for site less than 30 degrees and positive
!       refrac = exp( a3*site**3 + a2*site**2 + a1*site + a0)
!
!       for site over 30 degrees
!       refrac = b1*(site-30deg) + b0
!
!       typical values:
!       site     refrac  (all in degrees)
!         2     0.64077
!         3     0.47946
!         4     0.36881
!         5     0.29118
!        10     0.12489
!        15     0.08101
!        20     0.06518
!        30     0.03650
!        45     0.02737
!        60     0.01825
!        75     0.00912
!        90     0.00000
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        double precision site   ! site angle (rad)
!
! Input/Output
!
! Output
!
!
!DECLARATIONS
!
!
!
! Local parameters
        double precision pi
        parameter ( pi = 3.14159265358979311600d0)

        double precision v30deg         ! thirty degrees in radians
        parameter ( v30deg = 30.d0*pi/180.d0 )
        double precision v60deg         ! sixty degrees in radians
        parameter ( v60deg = 60.d0*pi/180.d0 )

        double precision a3, a2, a1 ,a0
        parameter ( a3 = -49.64816d0)
        parameter ( a2 =  53.14692d0)
        parameter ( a1 = -20.96701d0)
        parameter ( a0 =  -3.82407d0)

        double precision ref30, b1, b0
        parameter ( ref30 = 6.370188d-04)
        parameter ( b0 = ref30 *1.5d0 )
        parameter ( b1 = -ref30 / v60deg )
!
! Commons
!
! Functions
!
! Local variables
! 
!-
!
        if( site.gt. v30deg ) then
          refrac = b1*site + b0

        elseif( site.ge. 0.d0 ) then
          refrac = dexp( a3*site**3 + a2*site**2 + a1*site + a0)

        else
          refrac = 0.d0

        endif

        return

        end
