        subroutine sungrw(time,posgrw,dist)
!+
!NOM/NAME $Id: sungrw.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!
!       calculates the sun position in Greenwich reference frame  
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       the sun position is:
!       - calculated in an inertial mean-of-date reference frame (IMOD) 
!         with equations issued from the ESOC software PEPSOC
!       - converted into Greenwich reference frame
!
!       the transformation from IMOD to Greenwich is approximated by 
!       rotation along Z axis with 2 components:
!       - sidereal time of the 1950 vernal equinox
!       - precession from 1950 to IMOD vernal equinox
!       this induce a latitude error of about 0.003 deg when compared to
!       the transformation IMOD to Greenwich used in PEPSOC
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!       sidert50: sidereal time of the 1950 vernal equinox
!       sunpos: sun cartesian coordinates in inertial mean-of-date ref. frame
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        double precision time           ! julian instant
!
! Output
        double precision posgrw(3)      ! sun position in Greenwich ref. frame
        double precision dist           ! sun-earth distance
!
!
!DECLARATIONS
!
!
!
! Local parameters
        double precision a1, a2, a3     ! constants for precession calculation
        data a1,a2,a3 /.6119074d-06,.507847d-14,.361458d-20/
!
! Commons
!
! Functions
        double precision sidert50
!
! Local variables
        double precision pos(3)         ! sun position in IMOD reference frame
        double precision preces         ! precession angle in radians
        double precision angle          ! rotation angle from Greenwich to IMOD
        double precision ca, sa         ! cosinus and sinus of angle
! 
!-
!
! calculates the sun position in IMOD reference frame
        call sunpos(time,pos,dist)

! calculates the rotation angle from IMOD to Greenwich reference frame
! sidereal time = 1950 inertial reference frame to Greenwich 
! precession    = 1950 to IMOD reference frame
        preces = - a1*time - a2*time*time - a3*time*time*time
        angle = sidert50(time) - preces

! conversion from IMOD to Greenwich reference frame
        ca = dcos(angle)
        sa = dsin(angle)
        posgrw(1) =   ca * pos(1) + sa * pos(2)
        posgrw(2) = - sa * pos(1) + ca * pos(2)
        posgrw(3) = pos(3)

        return

        end
