        subroutine viewvect(pos, Mat_Ra_Rg, att, om_g, sm_s)
!+
!NOM/NAME $Id: viewvect.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!       calculates the viewing vector, satellite - viewed point, 
!       in the spacecraft fixed reference frame
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!      
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Apr 2004      no          A. Marsouin P.Brunel
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS
!
!       Rs is the spacecraft fixed reference frame
!       Ra is the nominal attitude refernce frame
!       Rg is the Greenwich reference frame
!
!input 
        double precision pos(3)         ! satellite position in Rg      (km)
        double precision Mat_Ra_Rg(3,3) ! Ra to Rg conversion matrix
        double precision att(3,3)       ! attitude error matrix         (rad)
        double precision om_g(3)        ! viewed point in Rg
! output
        double precision sm_s(3)        ! satellite - viewed point in Rs

!DECLARATIONS
!
!
!
! Local parameters
!
! Local variables       (distances in km, angles in radians)
! 
        
        double precision sm_g(3)        !  satellite - viewed point in Rg
        double precision sm_a(3)        !  satellite - viewed point in Ra
!
!-

! calculates the SM vector, satellite - viewed point, in Rg
        sm_g(1) = om_g(1) - pos(1)
        sm_g(2) = om_g(2) - pos(2)
        sm_g(3) = om_g(3) - pos(3)

! transforms into Ra
        sm_a(1) = mat_ra_rg(1,1)*sm_g(1)+mat_ra_rg(1,2)*sm_g(2)+
     >                                   mat_ra_rg(1,3)*sm_g(3)
        sm_a(2) = mat_ra_rg(2,1)*sm_g(1)+mat_ra_rg(2,2)*sm_g(2)+
     >                                   mat_ra_rg(2,3)*sm_g(3)
        sm_a(3) = mat_ra_rg(3,1)*sm_g(1)+mat_ra_rg(3,2)*sm_g(2)+
     >                                   mat_ra_rg(3,3)*sm_g(3)

! transforms into Rs using attitude error matrix
        sm_s(1)=att(1,1)*sm_a(1)+att(1,2)*sm_a(2)+att(1,3)*sm_a(3)
        sm_s(2)=att(2,1)*sm_a(1)+att(2,2)*sm_a(2)+att(2,3)*sm_a(3)
        sm_s(3)=att(3,1)*sm_a(1)+att(3,2)*sm_a(2)+att(3,3)*sm_a(3)

        return
        end
