        subroutine tb_chk(ierr)
!+
!NOM/NAME $Id: tb_chk.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!
!       Checks if the TBUS orbital elements are between allowed values
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!       the variables to be tested are in the common c_tbus4
!       there are two values (min and max) for each variable
!
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!ARGUMENTS 
        integer ierr    ! return code
!                       ! 0 = OK
!                       ! 1 = warning, tbus param. (not orbital) out of range
!                       ! 2 = fatal error one or more orbital elem. out of range
!
!DECLARATIONS
!
!
!
! Common declarations:
!
! Local parameters:
        double precision pi     ! PI constant  
        double precision mpi    ! -PI
        double precision twopi  ! 2xPI
        double precision mtwopi ! -2xPI
        double precision cdr    ! conversion factor degrees to radians 

        parameter ( pi = 3.14159265358979311600d0)
        parameter ( mpi    = -1.d0*pi)
        parameter ( twopi  = 2.d0*pi)
        parameter ( mtwopi = -2.d0*pi)
        parameter ( cdr=pi/180.0d0)
!
! Commons
! common TBUS Part IV
#include "c_tbus4.h"
!
! Functions
!
! Local variables:
! 
!-
!

        ierr = 0

!
!       warning errors (ierr = 1)
!
!       time of ascending node  >0 <367
!       greenwich hour angle    >-2pi <2pi
!       periods                 >90  <110 minutes
!       balistic coefficient    >0 <0.1
!       daily solar flux        >0 <999
!       90-day mean solar flux  >0 <999
!       planetary magnetic index>0 <999
!       drag modulation coef.   >0 <1
!       radiation pressure coef >0 <0.1
!       perigee motion          >-10 <10 deg/day
!       right ascension motion  >-5  <5 deg/day
!       mean anomaly motion     >5040 <5220 deg/day  (14 , 14.5 orbits)
!
        if(tbus_tan .lt. 0.d0 .or. tbus_tan .ge. 367.d0) then
          call ml_wt('notice',
     &            'time of ascending node out of range')
          ierr = 1
        endif
        if(tbus_gha .lt. mtwopi .or. tbus_gha .gt. twopi) then
          call ml_wt('notice',
     &            'greenwich hour angle out of range')
          ierr = 1
        endif
        if(tbus_anoper .lt. 90./1440. .or. 
     &     tbus_anoper .gt. 110./1440.) then
          call ml_wt('notice',
     &            'anomalistic period out of range')
          ierr = 1
        endif
        if(tbus_nodper .lt. 90./1440. .or. 
     &     tbus_nodper .gt. 110./1440.) then
          call ml_wt('notice',
     &            'nodal period out of range')
          ierr = 1
        endif
        if(tbus_balcoef .lt. 0.d0 .or. tbus_balcoef .gt. 0.1d0) then
          call ml_wt('notice',
     &            'balistic coefficient out of range')
          ierr = 1
        endif
        if(tbus_dsolflux .lt. 0 .or. tbus_dsolflux .gt. 999) then
          call ml_wt('notice',
     &            'daily solar flux out of range')
          ierr = 1
        endif
        if(tbus_msolflux .lt. 0 .or. tbus_msolflux .gt. 999) then
          call ml_wt('notice',
     &            '90-day running mean solar flux out of range')
          ierr = 1
        endif
        if(tbus_magind .lt. 0 .or. tbus_magind .gt. 999) then
          call ml_wt('notice',
     &            'planetary magnetic index out of range')
          ierr = 1
        endif
        if(tbus_dragcoef .lt. 0.d0 .or. tbus_dragcoef .gt. 1.d0) then
          call ml_wt('notice',
     &            'drag modulation coefficient out of range')
          ierr = 1
        endif
        if(tbus_radpres .lt. 0.d0 .or. tbus_radpres .gt. 0.1d0) then
          call ml_wt('notice',
     &            'radiation pressure coefficient out of range')
          ierr = 1
        endif
        if(tbus_permot .lt. -10.d0*cdr .or. 
     &     tbus_permot .gt.  10.d0*cdr) then
          call ml_wt('notice',
     &            'perigee motion out of range')
          ierr = 1
        endif
        if(tbus_rascmot .lt. -5.d0*cdr .or. 
     &     tbus_rascmot .gt.  5.d0*cdr) then
          call ml_wt('notice',
     &            'right ascension motion out of range')
          ierr = 1
        endif
        if(tbus_manomot .lt. 5040.d0*cdr .or. 
     &     tbus_manomot .gt. 5220.d0*cdr) then
          call ml_wt('notice',
     &            'mean anomaly motion out of range')
          ierr = 1
        endif


!       fatal errors (ierr = 2)
!
!       epoch time              > 7300 (year 1970) and < 36500 (year 2050)
!       eccentricity            >0  <0.01
!       argument of perigee     >0 < 2pi
!       right ascension         >0 < 2pi
!       mean anomaly            >0 < 2pi
!       inclination             > 96 deg < 102 deg
!       semi-major axis         > 7100  <7300 km
!       x y z                   > +-1.01 * semi-major axis
!       vx vy vz                > +-8  km/s
!
!
        if(tbus_epoch .lt. 7300.d0 .or. 
     &     tbus_epoch .gt. 36500.d0) then
          call ml_wtdp('warning',
     &            'epoch time out of range',tbus_epoch)
          ierr = 2
        endif
        if(tbus_eccent .lt. 0.d0 .or. tbus_eccent .gt. 0.01d0) then
          call ml_wtdp('warning',
     &            'eccentricity out of range',tbus_eccent)
          ierr = 2
        endif
        if(tbus_argper .lt. 0.d0 .or. tbus_argper .gt. twopi) then
          call ml_wtdp('warning',
     &            'argument of perigee out of range',tbus_argper/cdr)
          ierr = 2
        endif
        if(tbus_rigasc .lt. 0.d0 .or. tbus_rigasc .gt. twopi) then
          call ml_wtdp('warning',
     &            'right ascension out of range',tbus_rigasc/cdr)
          ierr = 2
        endif
        if(tbus_inclin .lt. 96.d0*cdr .or.
     &     tbus_inclin .gt. 102.d0*cdr) then
          call ml_wtdp('warning',
     &            'inclination out of range',tbus_inclin/cdr)
          ierr = 2
        endif
        if(tbus_meanano .lt. 0.d0 .or. 
     &     tbus_meanano .gt. twopi) then
          call ml_wtdp('warning',
     &            'mean anomaly out of range',tbus_meanano/cdr)
          ierr = 2
        endif
        if(tbus_semaxis .lt. 7000.d0 .or. 
     &     tbus_semaxis .gt. 7300.d0) then
          call ml_wtdp('warning',
     &            'semi major axis out of range',tbus_semaxis)
          ierr = 2
        endif
        if(tbus_x .lt. -1.05*tbus_semaxis .or.
     &     tbus_x .gt.  1.01*tbus_semaxis) then
          call ml_wtdp('warning',
     &            'x out of range',tbus_x)
          ierr = 2
        endif
        if(tbus_y .lt. -1.01*tbus_semaxis .or.
     &     tbus_y .gt.  1.01*tbus_semaxis) then
          call ml_wtdp('warning',
     &            'y out of range',tbus_y)
          ierr = 2
        endif
        if(tbus_z .lt. -1.01*tbus_semaxis .or.
     &     tbus_z .gt.  1.01*tbus_semaxis) then
          call ml_wtdp('warning',
     &            'z out of range',tbus_z)
          ierr = 2
        endif
        if(tbus_vx .lt. -8.d0 .or. tbus_vx .gt. 8.d0) then
          call ml_wtdp('warning',
     &            'vx out of range',tbus_vx)
          ierr = 2
        endif
        if(tbus_vy .lt. -8.d0 .or. tbus_vy .gt. 8.d0) then
          call ml_wtdp('warning',
     &            'vy out of range',tbus_vy)
          ierr = 2
        endif
        if(tbus_vz .lt. -8.d0 .or. tbus_vz .gt. 8.d0) then
          call ml_wtdp('warning',
     &            'vz out of range',tbus_vz)
          ierr = 2
        endif


        return

        end
