        subroutine tb_forb(tan,orbitan,nodper,epoch,inielem,t,orbit)
!+
!NOM/NAME $Id: tb_forb.F 25 2006-02-03 15:43:53Z frna $
!       subroutine tb_forb(tan,orbitan,nodper,epoch,inielem,t,orbit)
!
!BUT/PURPOSE 
!       computes the orbit number at t strictly consistent with brolyd
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!       An approximate value of the orbit number is calculated using
!       the nodal period and an initial ascending node (time and orbit number)
!
!       The orbital parameters inielem (from tbus) are used as input of brolyd
!       to calculate the satellite position at t.
!       The value of the z component is used to correct the approximate
!       orbit number if t is close to an ascending node.
!
!       It also checks the particular problem of the satellite launch
!       (starting date at 0h could be a negative orbit number).
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        integer orbitan                 ! orbit number at tan
        double precision tan            ! time of ascending node (jdays)
        double precision nodper         ! nodal period          (days)
        double precision epoch          ! epoch time of the elements 
        double precision inielem(6)     ! orbital elements 
        double precision t              ! epoch time of the searched orbit
!
! Output
        integer orbit                   ! orbit number at t
!
!
!DECLARATIONS
!
!
!
! Local parameters
!
!
!
! Commons
! Brolyd common
#include "c_brolyd.h"
!
!
! Local variables
        integer ielem                   ! element number
        double precision realorbit      ! real orbit number at start instant
        double precision wkelem(6)      ! working elements for brolyd
        double precision oscgtod(6)     ! osculating elements (gama trueofdate)
        double precision auxelem(6)     ! auxiliary elements for brolyd
        double precision positod(3)     ! position in ITOD reference frame
        double precision velitod(3)     ! velocity in ITOD reference frame
        double precision zstdate        ! z component of the position at stdate
        double precision vzstdate       ! z component of the velocity at stdate
        double precision resorb         ! residual part of the orbit difference
! 
!-
!
!       realorbit is the estimated real orbit number at t
!       and orbit is the corresponding integer value
        realorbit = orbitan + (t - tan)/ nodper


!       tests for launch of satellite (orbit at 0h could be negative)
        if(realorbit .gt.0.d0) then
          orbit = dint(realorbit)
        else
          orbit = dint(realorbit-1.d0)
        endif

!       compute the position of the satellite by using brolyd at t
        do ielem=1,6
          wkelem(ielem) = inielem(ielem)
        end do

!       initial time difference: t - epoch in seconds
        tto = (t - epoch) * 86400.d0

        call brolyd(oscgtod,wkelem,2,2,1,auxelem)
        call celem(oscgtod, gm, positod, velitod)
        zstdate = positod(3)
        vzstdate = velitod(3)


!       if we are near an ascending node (50 km) we must verify the estimated
!       orbit number. 

        if(dabs(zstdate).lt.50.d0 .and. vzstdate.gt.0.d0) then

!         resorb is the decimal residual value of the estimated orbit
          resorb = orbit*1.d0 - realorbit
!
!         the estimated orbit number (orbit) is correct in the following cases:
!         - z > 0   and resorb close to 0  (just after an ascending node)
!         - z < 0   and resorb close to 1  (just before an ascending node)
!         or it has to be corrected to be consistent with brolyd.

          if (abs(resorb) .lt. 0.5 .and. zstdate.le.0.d0) then
            orbit = orbit - 1
          endif

          if (abs(resorb) .gt. 0.5 .and. zstdate.ge.0.d0) then
            orbit = orbit + 1
          endif

        endif

        return

        end
