        subroutine tb_glpv(tbindu,epoch,tbus_file,ierr)
!+
!NOM/NAME $Id: tb_glpv.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!       get last preceeding valid tbus filename from the index file
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!       the index file is supposed to be chronological
!
!
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!ARGUMENTS 
!input
        integer tbindu          ! logical unit of the index file
        double precision epoch  ! epoch time for the time comparison
!output
        character*(*) tbus_file ! fullname of the tbus file
        integer ierr            ! return code 
!                               ! 0 = OK
!                               ! 1 = failed (end of file)
!                               ! 2 = no valid tbus before the input date
!
!DECLARATIONS
!
!
!
! Common declarations:
!
! Local parameters:
        double precision pi     ! PI constant  
        double precision crd    ! conversion factor radians to degrees 

        parameter ( pi = 3.14159265358979311600d0)
        parameter ( crd=180.0d0/pi)
!
! Commons
! common TBUS Part IV
#include "c_tbus4.h"
!
! Functions
!
! Local variables:
        integer ios                     ! I/O status variable
        character*22 ic                 ! epoch time string
        character*6 dummyst             ! dummy string
        logical endoffile               ! end of file
        logical found                   ! a valid tbus is found
        logical finished                ! end of loop

!       variables for the file reading
        double precision fepoch         ! epoch time    
        integer qflag                   ! quality flag  
        integer forbnum                 ! orbit number
        real exterr(2)                  ! extrapolation errors
        character*256 ftbus             ! tbus filename
! 
!-
!
        ierr = 0

!       rewind the file
        rewind tbindu

!       skip the header lines
        read(tbindu,'(a6)',iostat=ios) dummyst
        do while ( ios.eq.0 .and.
     &            (dummyst(1:1).eq. '#' .or. dummyst(1:1).eq.'%'
     &                                  .or. dummyst.eq.'      ')  )
          read(tbindu,'(a6)',iostat=ios) dummyst
        end do
        if(ios.eq.0) backspace tbindu

!
!       try to find the nearest valid tbus, but with an epoch time BEFORE
!       the input epoch. 
!
        endoffile=.false.
        found=.false.
        finished=.false.

100     continue
          read(tbindu,'(f15.9,1x,i1,1x,i5,2f7.2,1x,a22,1x,a)',
     &         iostat=ios)
     &        fepoch,qflag,forbnum,exterr,ic,ftbus

          if( (fepoch .lt. epoch ) .and. (qflag .eq. 0) 
     &                             .and.   (ios.eq.0)   ) then
            found=.true.
            tbus_file=ftbus
          else
            if(fepoch .ge. epoch) then
              finished=.true.
            endif
          endif

          endoffile= ios.ne.0
          finished= finished .or. endoffile

        if(.not. finished) goto 100  
!
!       final status
!
        if(.not.found) then
          tbus_file = ' '
          if(endoffile) then
            ierr = 1
          else
            ierr = 2
          endif
        else
          ierr = 0
        endif

        return

        end
