        subroutine tle_chk(ierr)
!+
!NOM/NAME $Id: tle_chk.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!       Checks the TLE orbital elements are between allowed values
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!       the variables to be tested are in the common c_tle
!       there are two values (min and max) for each variable
!
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 April 2004      no          A. Marsouin P.Brunel
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!ARGUMENTS 
        integer ierr            ! return code
!                               ! 0 = OK
!                               ! 1 = warning, usable for orbit extrapoaltion
!                               ! 2 = fatal
!
!DECLARATIONS
!
!
!
! Common declarations:
!
! Local parameters:
        double precision pi     ! PI constant  
        double precision mpi    ! -PI
        double precision twopi  ! 2xPI
        double precision mtwopi ! -2xPI
        double precision cdr    ! conversion factor degrees to radians 

        parameter ( pi = 3.14159265358979311600d0)
        parameter ( mpi    = -1.d0*pi)
        parameter ( twopi  = 2.d0*pi)
        parameter ( mtwopi = -2.d0*pi)
        parameter ( cdr=pi/180.0d0)
!
! Commons
! commonn TLE
#include "c_tle.h" 
!
! Functions
!
! Local variables:
! 
!-
!

        ierr = 0

!
!       warning errors (ierr = 1)
!       not implemented
!


!       fatal errors (ierr = 2)
!
!       epoch time              > 7300 (year 1970) and < 36500 (year 2050)
!       eccentricity            >0  <0.01
!       argument of perigee     >0 < 2pi
!       right ascension         >0 < 2pi
!       mean anomaly            >0 < 2pi
!       inclination             > -180 deg < 180 deg
!       mean motion             > 0 < 30 revs/day
!
        if(tle_epoch .lt. 7300.d0 .or. 
     &     tle_epoch .gt. 36500.d0) then
          call ml_wtdp('warning',
     &            'epoch time out of range',tle_epoch)
          ierr = 2
        endif
        if(tle_eccent .lt. 0.d0 .or. tle_eccent .gt. 0.01d0) then
          call ml_wtdp('warning',
     &            'eccentricity out of range',tle_eccent)
          ierr = 2
        endif
        if(tle_argper .lt. 0.d0 .or. tle_argper .gt. twopi) then
          call ml_wtdp('warning',
     &            'argument of perigee out of range',tle_argper/cdr)
          ierr = 2
        endif
        if(tle_rigasc .lt. 0.d0 .or. tle_rigasc .gt. twopi) then
          call ml_wtdp('warning',
     &            'right ascension out of range',tle_rigasc/cdr)
          ierr = 2
        endif
        if(tle_inclin .lt. -180.d0*cdr .or.
     &     tle_inclin .gt. 180.d0*cdr) then
          call ml_wtdp('warning',
     &            'inclination out of range',tle_inclin/cdr)
          ierr = 2
        endif
        if(tle_meanano .lt. 0.d0 .or. 
     &     tle_meanano .gt. twopi) then
          call ml_wtdp('warning',
     &            'mean anomaly out of range',tle_meanano/cdr)
          ierr = 2
        endif
        if(tle_meanmotion .lt. 0.d0 .or. 
     &     tle_meanmotion .gt. 30.d0*twopi/1440.) then
          call ml_wtdp('warning',
     &         'mean motion out of range',tle_meanmotion/twopi*1440.)
          ierr = 2
        endif


        return

        end
