        subroutine tle_ctrl(scnam,path,tleindu,qflag,exterr)
!+
!NOM/NAME $Id: tle_ctrl.F 116 2008-06-27 09:17:04Z frna $
!
!BUT/PURPOSE 
!       controls the quality of the tle orbital parameters
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!       This modules controls the quality of the tle orbital parameters
!       - get the file name of the last preceeding valid tle from the 
!               historical file
!       - opens the last preceeding valid tle
!       - decodes the elements of the last preceeding valid tle (tle_sdc)
!       - closes the last preceeding valid tle
!       - calculates extrapolation errors in km/day, using the SGP/SDP model
!               forward error of the last preceeding valid onto 
!               the current tle
!               backward error of the current onto the last preceeding 
!               valid tle
!       - the tle is declared OK if the errors are less than 
!               15km/day (variables fwtest, bwtest)
!
!       the tests with the last preceeding valid tle are done only if the
!       time difference is less than 7 days (variable dtmax)
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 April 2004      no          A. Marsouin P.Brunel
! 1.2   23 April 2008 make fwerr,bwerr variables not parameters, and allow them
!                     to be over-written by PAR_NAVIGATION_EXTRAP_LIMIT. NCA
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!       cp_lsc          ! last significant character (f7cp)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!ARGUMENTS 
!input
        character*(*) scnam     ! any authorized s/c name
        character*(*) path      ! path name for the index files
        integer tleindu         ! logical unit of the index file
!output
        integer qflag           ! quality flag
!                               ! 0 = OK
!                               ! 1 = erroneous orbital parameters
        real exterr(2)          ! extrapolation errors (km/day)
!
!DECLARATIONS
!
!
!
! Common declarations:
!
! Local parameters:
        double precision lpi    ! PI constant  
        double precision crd    ! conversion factor radians to degrees 
        real ratetest           ! rate for fwd/bwd tests (km/day/day)
        real dtmax              ! max. time difference allowed for tests (days)
        integer lpv_unit        ! unit for the lpv tle file

        parameter ( lpi = 3.14159265358979311600d0)
        parameter ( crd=180.0d0/lpi)
        parameter ( ratetest=0.0 )
        parameter ( dtmax=7. )
        parameter ( lpv_unit = 20 )
!
! Commons
! commonn TLE
#include "c_tle.h"
#include "c_sgp.h"
!
! Functions
        integer cp_lsc                  ! last significant character (f7cp)
!
! Local variables:
        integer ios                     ! I/O status variable
        character*256 name              ! var. name for the last... tle file
        character*256 lpv_file          ! fullname of the last... tle file
        integer lpv_orbnum              ! orbit number of the last... tle file
        integer ierglpv                 ! return code from glpv
        integer iersdc                  ! return code from sdc
        integer ioslpv                  ! I/O status for the tle file openning 
        integer iflag                   ! flag for SGP4 
        double precision epocht         ! epoch time for tle_glpv
        double precision tsince         ! extrapolation duration (mn)
        double precision t0             ! epoch time of the actual tle
        double precision amelem(10)     ! mean elements of the actual tle
        double precision aposvel(6)     ! pos-vel of the actual tle (G50)
        double precision t1             ! epoch time of the preceeding tle
        double precision pmelem(10)     ! mean elements of the preceeding tle
        double precision pposvel(6)     ! pos-vel elements of the prec. tle
        double precision posvel(6)      ! position-vel. elements (gama 50)
        double precision poserr_x       ! position error (x comp.) (km)
        double precision poserr_y       ! position error (y comp.) (km)
        double precision poserr_z       ! position error (z comp.) (km)
        double precision velmod         ! module of velocity   (km/s)
        logical test2                   ! test number 2 (forward)
        logical test3                   ! test number 3 (backward)
        character*22 cht                ! date string

        real fwtest                     ! threshold for forward test (km/day)
        real bwtest                     ! threshold for backward test (km/day)
        character*10 cextrap            ! extrapolation threshold string
! 
!-
!
        fwtest=15.   !Extrapolation limit default
        bwtest=15.
        call GETENV('PAR_NAVIGATION_EXTRAP_LIMIT',cextrap)
        read(cextrap,*,err=100,end=100) fwtest
        read(cextrap,*,err=100,end=100) bwtest
        call ml_wt ('info','Extrapolalation limit is '//cextrap//
     &       ' km/day')
  100   continue

        t0 = tle_epoch

        amelem(1)  = tle_meanano
        amelem(2)  = tle_rigasc
        amelem(3)  = tle_argper
        amelem(4)  = tle_eccent
        amelem(5)  = tle_inclin
        amelem(6)  = tle_meanmotion
        amelem(7)  = tle_ftdmm
        amelem(8)  = tle_stdmm
        amelem(9)  = tle_bstar
        amelem(10) = tle_epoch_tlf

        !E1 common
        xmo    = amelem(1)
        xnodeo = amelem(2)  
        omegao = amelem(3)
        eo     = amelem(4)
        xincl  = amelem(5)  
        xno    = amelem(6)
        xndt2o = amelem(7)
        xndd6o = amelem(8)
        bstar  = amelem(9)
        epoch  = amelem(10)

        !C1 common initialzed in sgp_init       
        !C2 common initialzed in sgp_init

        exterr(1) = 0.
        exterr(2) = 0.

        !Compute position at epoch
        tsince = 0.d0
        iflag = 1
        if(tle_deep) then
           call sdp4(iflag,tsince)
        else
           call sgp4(iflag,tsince)
        endif
        aposvel(1) = x*XKMPER/AE
        aposvel(2) = y*XKMPER/AE
        aposvel(3) = z*XKMPER/AE
        aposvel(4) = xdot*XKMPER/AE*XMNPDA/86400.
        aposvel(5) = ydot*XKMPER/AE*XMNPDA/86400.
        aposvel(6) = zdot*XKMPER/AE*XMNPDA/86400.

!       1. launch bulletin test (position equals 0)
!       -------------------------------------------
!       when we have a launch bulletin it is always declared OK
!       Launch == orbit number less than 250
!

        if(tle_orbnum.lt.250) then
           call ml_wt('warning','Launch bulletin always OK')
          qflag = 0
        else
!       1. forward and backward tests
!       -----------------------------
!       get the last preceeding valid tle filename
!       a one minute difference is put for the test in case of tleing running
!       twice with the same bulletin as input

          epocht = tle_epoch-1.d0/1440d0
          call tle_glpv(tleindu,epocht,lpv_file,lpv_orbnum,ierglpv)

          if( ierglpv .eq. 0 ) then
!           create the tle filename (adding the path and 
!           suppressing the white spaces) and opens the file
            name = 
     &       path(1:cp_lsc(path)) // '/' // lpv_file(1:cp_lsc(lpv_file))
            call cp_ssf(name,lpv_file,' ')
!           opens the tle file
            open (unit=lpv_unit,file=lpv_file,status='old',
     &            iostat=ioslpv)
            if( ioslpv.ne.0 ) then
              call ml_wt('error','AN TLE FILE IS MISSING')
              call ml_wt('error','Filename '//lpv_file)
              call ml_wti('error','I/O status :',ioslpv)
            else
              call tle_sdc(scnam,lpv_unit,t1,pmelem,iersdc)
              if(iersdc.ne.0) then
                call ml_wt('error','AN TLE FILE IS INCORRECT')
                call ml_wt('error','Filename '//lpv_file)
                call ml_wti('error','tle_sdc code :',iersdc)
              endif

            endif
            close (unit=lpv_unit,iostat=ios)

          else
            call ml_wt('warning',
     &                       'No valid TLE before the current one')

          endif

!         if everything is OK we can process the tests
!             opens, reads, decoding and time difference
          if(ierglpv.eq.0 .and. iersdc.eq.0 .and. ioslpv.eq.0 .and.
     &                dabs(t0-t1).lt.dtmax ) then


!           forward test (preceeding to actual)
!           -----------------------------------
            !get position velocity at epoch time of preceeding 
                                !E1 common
             xmo    = pmelem(1)
             xnodeo = pmelem(2)  
             omegao = pmelem(3)
             eo     = pmelem(4)
             xincl  = pmelem(5)  
             xno    = pmelem(6)
             xndt2o = pmelem(7)
             xndd6o = pmelem(8)
             bstar  = pmelem(9)
             epoch  = pmelem(10)
             tsince = 0.d0
             iflag = 1
             if(tle_deep) then
                call sdp4(iflag,tsince)
             else
                call sgp4(iflag,tsince)
             endif
             pposvel(1) = x*XKMPER/AE
             pposvel(2) = y*XKMPER/AE
             pposvel(3) = z*XKMPER/AE
             pposvel(4) = xdot*XKMPER/AE*XMNPDA/86400.
             pposvel(5) = ydot*XKMPER/AE*XMNPDA/86400.
             pposvel(6) = zdot*XKMPER/AE*XMNPDA/86400.

             xmo    = pmelem(1)
             xnodeo = pmelem(2)  
             omegao = pmelem(3)
             eo     = pmelem(4)
             xincl  = pmelem(5)  
             xno    = pmelem(6)
             xndt2o = pmelem(7)
             xndd6o = pmelem(8)
             bstar  = pmelem(9)
             epoch  = pmelem(10)
             tsince = (t0 - t1) *1440.d0
             iflag = 1
             if(tle_deep) then
                call sdp4(iflag,tsince)
             else
                call sgp4(iflag,tsince)
             endif
             posvel(1) = x*XKMPER/AE
             posvel(2) = y*XKMPER/AE
             posvel(3) = z*XKMPER/AE
             posvel(4) = xdot*XKMPER/AE*XMNPDA/86400.
             posvel(5) = ydot*XKMPER/AE*XMNPDA/86400.
             posvel(6) = zdot*XKMPER/AE*XMNPDA/86400.
          
!           position error (extrapolation - tle values) at t0
            poserr_x = posvel(1)-aposvel(1)
            poserr_y = posvel(2)-aposvel(2)
            poserr_z = posvel(3)-aposvel(3)

!           velocity module at t0
            velmod = dsqrt( aposvel(4)*aposvel(4) +
     &                      aposvel(5)*aposvel(5) +
     &                      aposvel(6)*aposvel(6) )

!           along track forward error at t0
            exterr(1) = ( poserr_x*aposvel(4) +
     &                    poserr_y*aposvel(5) +
     &                    poserr_z*aposvel(6) ) / velmod / (t0-t1)

            test2 = abs(exterr(1)) .lt. fwtest+ratetest*(t0-t1)

!           backward test (actual to preceeding)
!           ------------------------------------

            xmo    = amelem(1)
            xnodeo = amelem(2)  
            omegao = amelem(3)
            eo     = amelem(4)
            xincl  = amelem(5)  
            xno    = amelem(6)
            xndt2o = amelem(7)
            xndd6o = amelem(8)
            bstar  = amelem(9)
            epoch  = amelem(10)

            tsince = (t1 - t0) *1440.d0
            iflag = 1
            if(tle_deep) then
               call sdp4(iflag,tsince)
            else
               call sgp4(iflag,tsince)
            endif
            posvel(1) = x*XKMPER/AE
            posvel(2) = y*XKMPER/AE
            posvel(3) = z*XKMPER/AE
            posvel(4) = xdot*XKMPER/AE*XMNPDA/86400.
            posvel(5) = ydot*XKMPER/AE*XMNPDA/86400.
            posvel(6) = zdot*XKMPER/AE*XMNPDA/86400.

!           position error (extrapolation - tle values) at t1
            poserr_x = posvel(1)-pposvel(1)
            poserr_y = posvel(2)-pposvel(2)
            poserr_z = posvel(3)-pposvel(3)

!           velocity module at t1
            velmod = dsqrt( pposvel(4)*pposvel(4) +
     &                      pposvel(5)*pposvel(5) +
     &                      pposvel(6)*pposvel(6) )

!           along track backward error at t1
            exterr(2) = ( poserr_x*pposvel(4) +
     &                    poserr_y*pposvel(5) +
     &                    poserr_z*pposvel(6) ) / velmod / (t0-t1)

            test3 = abs(exterr(2)) .lt. bwtest+ratetest*(t0-t1)

          elseif(ierglpv.eq.0 .and. iersdc.eq.0 .and. ioslpv.eq.0 .and.
     &                dabs(t0-t1).ge.dtmax ) then
            call ml_wtdp('warning','Too many days without valid tle',
     &                  t0-t1)

            test2 = .true.
            test3 = .true.

          else
!           no file is available for test; the tests are set to OK

            if(tle_orbnum.eq.0) then
              call xijc(tle_epoch,cht)
              call ml_wt('notice',
     &           'Le calcul du numero d"orbite est impossible')
              call ml_wt('notice',
     &      'Vous devrez le donner manuellement dans le fichier index')
              call ml_wt('notice','Epoch time '//cht)
            endif
            test2 = .true.
            test3 = .true.
          endif

          if(test2 .and. test3) then
            qflag = 0
          else
            call ml_wt('warning','The TLE bulletin is rejected')
            call ml_wtdp('warning','number of days',t0-t1)
            call ml_wtr('warning',
     &                           'forward error (km/day)',exterr(1))
            call ml_wtr('warning',
     &                           'backward error (km/day)',exterr(2))
            qflag = 1
          endif

        endif ! test launch bulletin

        return

        end
