        subroutine tle_gnv(tleindu,epoch,dtmax,tle_file,
     &                                       tle_orbnum,ierr)
!+
!NOM/NAME $Id: tle_gnv.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!       get nearest valid tle filename from the index file
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!       the searched tle date must be in [epoch-dtmax, epoch+dtmax]
!       the index file is supposed to be chronological
!
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 April 2004      no          A. Marsouin P.Brunel
! 1.1      May   2005  change the on orbit number to allow number 0
!                                         P. Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!ARGUMENTS 
!input
        integer tleindu         ! logical unit of the index file
        double precision epoch  ! epoch time for the time comparison
        real dtmax              ! maximum range for search around the epoch
!                               !   time in number of days
!output
        character*(*) tle_file ! fullname of the tle file (output)
        integer tle_orbnum       ! orbit number of the tle file (output)
        integer ierr            ! return code 
!                               ! 0 = OK
!                               ! 1 = failed (end of file)
!                               ! 2 = not found in the time range
!                               ! 3 = invalid orbit number 
!
!DECLARATIONS
!
!
!
! Common declarations:
!
! Local parameters:
!
! Commons
!
! Functions
!
! Local variables:
        integer ios                     ! I/O status variable
        real dtmin                      ! minmum time diff found (days)
        real dt                         ! time difference (days)
        character*22 ic                 ! epoch time string
        character*6 dummyst             ! dummy string
        logical endoffile               ! end of file
        logical found                   ! a good tle is found
        logical finished                ! end of loop

!       variables for the file reading
        double precision fepoch         ! epoch time    
        integer qflag                   ! quality flag  
        integer forbnum                 ! orbit number
        real exterr(2)                  ! extrapolation errors
        character*256 ftle              ! tle filename
! 
!-
!
        ierr = 0
        dtmin = 99999.

!       rewind the file
        rewind tleindu

!       skip the header lines
        read(tleindu,'(a6)',iostat=ios) dummyst
        do while ( ios.eq.0 .and.
     &            (dummyst(1:1).eq. '#' .or. dummyst(1:1).eq.'%'
     &                                  .or. dummyst.eq.'      ')  )
          read(tleindu,'(a6)',iostat=ios) dummyst
        end do
        if(ios.eq.0) backspace tleindu
!
!
!       try to find the nearest good tle
!       the index file is supposed to be time incrementing
!
        endoffile=.false.
        found=.false.
        finished=.false.

100     continue
          read(tleindu,'(f15.9,1x,i1,1x,i5,2f7.2,1x,a22,1x,a)',
     &         iostat=ios)
     &        fepoch,qflag,forbnum,exterr,ic,ftle

          if( (qflag .eq. 0) .and. (forbnum .ge. 0 ) .and. 
     &                                        (ios.eq.0)   ) then
            dt= dabs(fepoch-epoch)
            if( dt.le.dtmin .and. dt.le.dtmax) then
              dtmin = dt
              found=.true.
              tle_file=ftle
              tle_orbnum=forbnum
            else
              if(fepoch .ge. epoch+dtmax) then
                finished=.true.
              endif
            endif
          endif

          endoffile= ios.ne.0
          finished= finished .or. endoffile

        if(.not. finished) goto 100  
!
!       final status
!
        if(.not.found) then
          tle_file = ' '
          tle_orbnum = 0
          if(endoffile) then
            ierr = 1
          else
            ierr = 2
          endif
        else
          if( tle_orbnum .lt. 0 ) then
            ierr = 3
          else
            ierr = 0
          endif
        endif

        return

        end
