C LIBS = atovpp atov auxaapp f7nl1b
!+
!
      PROGRAM ATMS_BEAMWIDTH
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: atms_beamwidth.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose:
!
! Method:
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  0.1    22/07/2010  Original code. (N.C.Atkinson)
!  1.0    18/01/2011  Add error traps

#include "atms1c.h"

! Parameters
      INTEGER nymax
      PARAMETER (nymax=4096)   !Max number of lines. Allow full orbit ATMS

! Variables
      REAL bt(atms_1cnfov, nymax, atms_1cnchan)
                               !channel as last index, to simplify array passing
      
      INTEGER   AT1cunit   !Unit number
      DATA AT1cunit/17/
      
      INTEGER iscan,ichan,ispot,ios,nchannels,wmosatid,version
      INTEGER channelnumber(atms_1cnchan)
      REAL    sampling_dist
      REAL    beamwidth(atms_1cnchan)
      REAL    newwidth(atms_1cnchan)
      REAL    cutoff(atms_1cnchan)
      INTEGER nxaverage(atms_1cnchan)
      INTEGER nyaverage(atms_1cnchan)
      INTEGER qc_dist(atms_1cnchan)
      DATA nxaverage/atms_1cnchan*0/
      DATA nyaverage/atms_1cnchan*0/
      DATA qc_dist/atms_1cnchan*0/
      CHARACTER*30 Cline
      LOGICAL changes 
!
!- End of header

      WRITE(*,*)'############################'
      WRITE(*,*)'# Start atms_beamwidth.exe #'
      WRITE(*,*)'############################'

! Open the 1c file and read all scan lines
      CALL IOAT1C("OPEN/WRITE",AT1cunit,0)
      CALL IOAT1C("READ HEADER",AT1cunit,1)
      DO iscan=1,atms1c_h_scnlin
        CALL IOAT1C("READ DATA",AT1cunit,1+iscan)
        DO ispot=1,atms_1cnfov
          DO ichan=1,atms_1cnchan
            bt(ispot,iscan,ichan) = atms1c_btemps(ichan,ispot)
          ENDDO
        ENDDO
      ENDDO

! Read the beamwidth requirements
      OPEN(18,file='atms_beamwidth.dat',form='formatted',status='old',
     &   iostat=ios)
      IF (ios .NE. 0) THEN
        WRITE(*,*) 'Unable to open atms_beamwidth.dat'
        CALL AAPPEXIT(1)
      ENDIF
      wmosatid=999
      write(*,*) 'Look for beamwidth spec for satid ',atms1c_h_wmosatid
      DO WHILE (wmosatid .NE. atms1c_h_wmosatid)
        read(18,'(a30)',end=100) Cline
        DO WHILE (Cline(1:1) .eq. '#')
          read(18,'(a30)') Cline
        ENDDO
        READ(Cline,*,iostat=ios) wmosatid
        IF (ios .NE. 0) THEN
          WRITE(*,*)'Error reading wmosatid from atms_beamwidth.dat'
          CALL AAPPEXIT(1)
        ENDIF

        read(18,'(a30)') Cline
        DO WHILE (Cline(1:1) .eq. '#')
          read(18,'(a30)') Cline
        ENDDO
        READ(Cline,*,iostat=ios) version
        IF (ios .NE. 0) THEN
          WRITE(*,*)'Error reading version from atms_beamwidth.dat'
          CALL AAPPEXIT(1)
        ENDIF

        read(18,'(a30)') Cline
        DO WHILE (Cline(1:1) .eq. '#')
          read(18,'(a30)') Cline
        ENDDO
        READ(Cline,*,iostat=ios) sampling_dist
        IF (ios .NE. 0) THEN
          WRITE(*,*)
     &       'Error reading sampling_dist from atms_beamwidth.dat'
          CALL AAPPEXIT(1)
        ENDIF

        read(18,'(a30)') Cline
        DO WHILE (Cline(1:1) .eq. '#')
          read(18,'(a30)') Cline
        ENDDO
        READ(Cline,*,iostat=ios) nchannels
        IF (ios .NE. 0) THEN
          WRITE(*,*)'Error reading nchannels from atms_beamwidth.dat'
          CALL AAPPEXIT(1)
        ENDIF

        read(18,'(a30)') Cline
        DO ichan=1,nchannels
          read(18,'(a30)') Cline
          DO WHILE (Cline(1:1) .eq. '#')
            read(18,'(a30)') Cline
          ENDDO
          READ(Cline,*,iostat=ios)channelnumber(ichan),beamwidth(ichan),
     &       newwidth(ichan),cutoff(ichan),nxaverage(ichan),
     &       nyaverage(ichan),qc_dist(ichan)
          IF (ios .NE. 0) THEN
            WRITE(*,*)'Warning: atms_beamwidth.dat may be incomplete'
          ENDIF
        ENDDO
      ENDDO 
  100 IF (wmosatid .NE. atms1c_h_wmosatid) THEN
        WRITE(*,*) 'WARNING: sat id not matched: use',wmosatid
      ENDIF
        
! Do the image processing (if not already done)
      
      changes = .FALSE.
      DO ichan=1,nchannels
        IF (atms1c_h_beamwidth(channelnumber(ichan)) .EQ. 0 .AND.
     &      atms1c_h_nxaverage(channelnumber(ichan)) .EQ. 0 .AND.
     &      atms1c_h_nyaverage(channelnumber(ichan)) .EQ. 0) THEN
          changes = .true.
          CALL MODIFY_BEAMWIDTH(atms_1cnfov,atms1c_h_scnlin,
     &      bt(1,1,channelnumber(ichan)), sampling_dist,
     &      beamwidth(ichan), newwidth(ichan), cutoff(ichan),
     &      nxaverage(ichan), nyaverage(ichan), qc_dist(ichan))
          atms1c_h_beamwidth(channelnumber(ichan)) = 
     &       NINT(newwidth(ichan)*1.0E3)
          atms1c_h_cutoff(channelnumber(ichan))    =
     &       NINT(cutoff(ichan)*1.0E3)
          atms1c_h_nxaverage(channelnumber(ichan)) = nxaverage(ichan)
          atms1c_h_nyaverage(channelnumber(ichan)) = nyaverage(ichan)
          write(*,*) channelnumber(ichan),newwidth(ichan),cutoff(ichan),
     &       nxaverage(ichan),nyaverage(ichan),qc_dist(ichan)
        ENDIF
      ENDDO
     
! Over-write the input file
    
      IF (changes) THEN
        DO iscan=1,atms1c_h_scnlin
          CALL IOAT1C("READ DATA",AT1cunit,1+iscan)
          DO ispot=1,atms_1cnfov
            DO ichan=1,atms_1cnchan
              atms1c_btemps(ichan,ispot) = NINT(bt(ispot,iscan,ichan))
            ENDDO
          ENDDO
          CALL IOAT1C("WRITE DATA",AT1cunit,1+iscan)
        ENDDO
        atms1c_h_beamversion = version
        CALL IOAT1C("WRITE HEADER",AT1cunit,1)
        WRITE(*,*) 'atms_beamwidth.exe: finished'
      ELSE
        WRITE(*,*) 'atms_beamwidth.exe: no changes made to 1c file'
      ENDIF
      CALL IOAT1C("CLOSE",AT1cunit,0)

      END
