#!/bin/ksh
#$Id: atms_beamwidth.ksh 313 2012-02-28 14:51:43Z frna $
#
# script to run atms_beamwidth.exe: modify the ATMS beam width for a level 1c
# file. Requires a file $ATMS_BEAMWIDTH_FILE (default atms_beamwidth.dat)
# specifying the input and output beam widths.
#
# 22/07/2010  Nigel Atkinson 
#############################################################################

usage(){
  echo
  echo 'usage : atms_beamwidth infile [outfile]'
  echo
}

[ $# -ge 1 ] || { usage; exit 0; }
[ -f $1 ] || { usage; exit 1; }

. "$AAPP_PREFIX/ATOVS_CONF"

delete=Y

ATMS_BEAMWIDTH_FILE=${ATMS_BEAMWIDTH_FILE:-atms_beamwidth.dat}
DIR_ATMS_PREPROC=${DIR_ATMS_PREPROC:-${DIR_PREPROC}}

#Use version in current directory if available, otherwise look in $DIR_ATMS_PREPROC

if [ -f "$ATMS_BEAMWIDTH_FILE" ]; then
  dir=$PWD
  if [ "$ATMS_BEAMWIDTH_FILE" = atms_beamwidth.dat ]; then 
    delete=N
  else
    ln -sf "$ATMS_BEAMWIDTH_FILE" atms_beamwidth.dat
  fi
elif [ -f $DIR_ATMS_PREPROC/$ATMS_BEAMWIDTH_FILE ]; then
  dir=$DIR_ATMS_PREPROC
  ln -sf $DIR_ATMS_PREPROC/$ATMS_BEAMWIDTH_FILE atms_beamwidth.dat
else
  echo "No ATMS beamwidth file supplied"
  exit 1
fi
echo Using $dir/"$ATMS_BEAMWIDTH_FILE"

infile=$1
outfile=${2:-$infile}

[ $outfile != $infile ] && cp $infile $outfile

ln -sf $outfile ${FORT}17

atms_beamwidth.exe

exitc=$?

if [ $exitc != 0 ] && [ outfile != $infile ]; then 
  rm -f $outfile
fi

[ $delete = Y ] && rm -f atms_beamwidth.dat

rm -f ${FORT}17

exit $exitc
