C LIBS = atovpp atov auxaapp f7nl1b
!+
!
      PROGRAM ATMSTHIN1D
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: atmsthin1d.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose: Thin ATMS 1d file to AMSU-A-like sampling
!
! Method: Read record, copy to thinned structure, write output record.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    13/05/2011  Original code. (N.C.Atkinson)

#include "atms1d.h"
#include "atms1dthin.h"

! Variables
      INTEGER   AT1dunit       !Unit number
      DATA AT1dunit/27/
      INTEGER   AT1dthinunit   !Unit number
      DATA AT1dthinunit/37/
      INTEGER   I,J,K,Iscan,Ichan  !Loop variables
!
!- End of header

      WRITE(*,*)'########################'
      WRITE(*,*)'# Start atmsthin1d.exe #'
      WRITE(*,*)'########################'
      
! Open the 1d files

      CALL IOAT1D("OPEN/READ",AT1dunit,0)
      CALL IOATTHIN1D("OPEN/WRITE",AT1dthinunit,0)
      CALL IOAT1D("READ HEADER",AT1dunit,1)

! Create and write the thinned header, most variables are the same as unthinned.

      DO I=1,array_size_tatms1d
        tatms1d_hd(I) = atms1d_hd(I)
      ENDDO
      tatms1d_h_nfov = tatms_1dnfov
      tatms1d_h_recwords = array_size_tatms1d
      write(*,*) 'tatms1d_h_nfov=',tatms1d_h_nfov
      tatms1d_h_scnlin = (atms1d_h_scnlin+2)/3

      CALL IOATTHIN1D("WRITE HEADER",AT1dthinunit,1)
      
! Create and write the data records, reading 1 line in 3

      DO Iscan=1,atms1d_h_scnlin,3
        CALL IOAT1D("READ DATA",AT1dunit,1+Iscan)

        tatms1d_scnlin = atms1d_scnlin
        tatms1d_scnlinyr = atms1d_scnlinyr
        tatms1d_scnlindy = atms1d_scnlindy
        tatms1d_orbit = atms1d_orbit
        tatms1d_granulequal = atms1d_granulequal
        tatms1d_scanqual = atms1d_scanqual
        tatms1d_geolocqual = atms1d_geolocqual
        tatms1d_scalti = atms1d_scalti
        DO I=1,tatms_1dnfov
          J = I*3-1
          tatms1d_scnlintime(I) = atms1d_scnlintime(J)
          DO K=1,2
            tatms1d_latlon(K,I) = atms1d_latlon(K,J)
            tatms1d_surface(K,I) = atms1d_surface(K,J)
            tatms1d_landfrac(K,I) = atms1d_landfrac(K,J)
          ENDDO
          DO K=1,4
            tatms1d_angles(K,I) = atms1d_angles(K,J)
          ENDDO
          DO Ichan=1,tatms_1dnchan
            tatms1d_btemps(Ichan,I) = atms1d_btemps(Ichan,J)
          ENDDO
          tatms1d_atmsqual(I) = atms1d_atmsqual(J)
          DO K=1,8
            tatms1d_prepro(K,I) = atms1d_prepro(K,J)
          ENDDO
        ENDDO
        DO Ichan=1,tatms_1dnchan
          tatms1d_chanqual(Ichan) = atms1d_chanqual(Ichan)
          tatms1d_nedtcold(Ichan) = atms1d_nedtcold(Ichan)
          tatms1d_nedtwarm(Ichan) = atms1d_nedtwarm(Ichan)
        ENDDO

        CALL IOATTHIN1D("WRITE DATA",AT1dthinunit,1+(Iscan+2)/3)
      ENDDO

! Close files

      CALL IOAT1D("CLOSE",AT1dunit,0)
      CALL IOATTHIN1D("CLOSE",AT1dthinunit,0)
      WRITE(*,*)'Number of input scans: ',atms1d_h_scnlin
      WRITE(*,*)'Number of output scans: ',tatms1d_h_scnlin
      WRITE(*,*)'End atmsthin1d.exe'

      END
