C LIBS = atovpp atov auxaapp f7nl1b
!+
!
      PROGRAM CRIS_CHANNELS
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: cris_channels.F 497 2016-08-01 10:41:31Z frna $
!
! Purpose: Reduce the number of channels in a CrIS 1c dataset. Assume we are
! starting from a full channel set.
!
! The most likely purpose for doing this would be pror to running
! aapp_encodebufr_1c to generate a channel-selected 1c BUFR file. It is not
! needed for atovpp as this already incorporates channel selection.
!
! Method:
!   1) Read channel selection from fdf file
!   2) Modify header record
!   3) Modify all the data records
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2011, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    10/08/2011  Original code. (N.C.Atkinson)
!  1.1    15/10/2012  Bug fix
!  1.2    08/07/2016  Allow cris1c_h_recwords in the input file to be less than
!                     the maximum, for compatibility with either FSR or non-FSR.

#include "cris1c.h"
#include "ppparms.h"
#include "ppcrisfdata.h"

! Variables
      INTEGER   C1cunit       !Unit number
      DATA C1cunit/18/
      INTEGER   C1cthinunit   !Unit number
      DATA C1cthinunit/28/
      INTEGER   Cfunit       !Unit number
      DATA Cfunit/55/
      INTEGER   Iscan,Ichan,FOV,FofR  !Loop variables
      INTEGER   rec_orig, rec_new
!
!- End of header

      WRITE(*,*)'###########################'
      WRITE(*,*)'# Start cris_channels.exe #'
      WRITE(*,*)'###########################'
      
! Open the 1c files
! First get the record length of the input file

      cris1c_h_recwords = array_size_cris1c - 
     &      cris_1cnchan*cris_1cnfov*cris_1cnfor !excluding channels
      CALL IOC1C ("OPEN/READ",C1cunit,0)
      CALL IOC1C ("READ HEADER",C1cunit,1)      !Read header
      CALL IOC1C ("CLOSE",C1cunit,1)
      CALL IOC1C ("OPEN/READ",C1cunit,0)        !Re-open with correct length
      CALL IOC1C ("READ HEADER",C1cunit,1)      !Read header

      rec_orig = cris1c_h_recwords

      WRITE(*,*)'Channels in: ',cris1c_h_nchanused,
     &  'record size (words):',cris1c_h_recwords

! Read the fdf file (channels_cris, chanvn_cris, nchan_cris in ppcrisfdata.h)

      CALL PPCRISFDF(Cfunit,0,0,0)

! Modify the header

      cris1c_h_chanvn = chanvn_cris
      cris1c_h_nchanused = nchan_cris
      DO Ichan=1,cris1c_h_nchanused
        cris1c_h_channels(Ichan) = channels_cris(Ichan)
      ENDDO
      DO Ichan=cris1c_h_nchanused+1,cris_1cnchan
        cris1c_h_channels(Ichan) = 0
      ENDDO

    !Reduce size of record
      rec_new = rec_orig - 
     &   (cris1c_h_nchan - cris1c_h_nchanused)*cris_1cnfov*cris_1cnfor
      cris1c_h_recwords = rec_new
      cris1c_h_nchan = cris1c_h_nchanused
      
      CALL IOC1C("OPEN/WRITE",C1cthinunit,0)
      CALL IOC1C("WRITE HEADER",C1cthinunit,1)
      
      WRITE(*,*)'Channels out:',cris1c_h_nchanused,
     &  'record size (words):',cris1c_h_recwords
      WRITE(*,*)'Scan lines:',cris1c_h_scnlin

! Modify the data records

      DO Iscan=1,cris1c_h_scnlin
        cris1c_h_recwords = rec_orig
        CALL IOC1C("READ DATA",C1cunit,1+Iscan)
        cris1c_h_recwords = rec_new

        DO FOV=1,cris_1cnfov
          DO FofR=1,cris_1cnfor
            DO Ichan=1,cris1c_h_nchanused
              cris1c_radiance(FOV,FofR,Ichan) = 
     &           cris1c_radiance(FOV,FofR,cris1c_h_channels(Ichan))
            ENDDO
          ENDDO
        ENDDO

        CALL IOC1C("WRITE DATA",C1cthinunit,1+Iscan)
      ENDDO

! Close files

      CALL IOC1C("CLOSE",C1cunit,0)
      CALL IOC1C("CLOSE",C1cthinunit,0)
      WRITE(*,*)'End cris_channels.exe'

      END
