#!/bin/ksh
#$Id$
#
# script to run cris_channels.exe
#
# Uses the channel selection in CRIS.fdf (in current directory if it exists,
# otherwise in $DIR_PREPROC).
#
# 11/08/2011  Nigel Atkinson 
# 23/11/2017  Examine the end channel number of band 3 to set the default fdf
#             file: CRIS.fdf or CRIS_FSR.fdf. 
#############################################################################

usage(){
  echo
  echo 'usage : cris_channels infile outfile'
  echo
}

[ $# -ge 2 ] || { usage; exit 0; }
[ -f $1 ] || { usage; exit 1; }

. "$AAPP_PREFIX/ATOVS_CONF"

max_chan_number=$(od -An -t u4 -j 164 -N 4 $1)
if [ "$max_chan_number" -gt 2000 ]; then
  CRIS_FDF=${CRIS_FDF:-CRIS_FSR.fdf}
  echo "CrIS file is FSR with $max_chan_number channels"
else
  CRIS_FDF=${CRIS_FDF:-CRIS.fdf}
  echo "CrIS file is NSR with $max_chan_number channels"
fi

DIR_CRIS_PREPROC=${DIR_CRIS_PREPROC:-${DIR_PREPROC}}

rm -f ${FORT}18 ${FORT}28 ${FORT}55 

if [ -f ${CRIS_FDF} ]; then
  ln -sf ${CRIS_FDF} ${FORT}55
else
  ln -sf ${DIR_CRIS_PREPROC}/${CRIS_FDF} ${FORT}55
fi
ln -sf $1 ${FORT}18

cris_channels.exe 
status=$?

[ $status = 0 ] && mv ${FORT}28 $2

rm -f ${FORT}18 ${FORT}28 ${FORT}55

exit $status
