C LIBS = atovpp atov auxaapp f7nl1b
!+
!
      PROGRAM HIRAS_CHANNELS
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: hiras_channels.F 619 2019-02-12 12:49:41Z frna $
!
! Purpose: Reduce the number of channels in a HIRAS 1c dataset. Assume we are
! starting from a full channel set.
!
! The most likely purpose for doing this would be pror to running
! eccodes_encodebufr_1c to generate a channel-selected 1c BUFR file. It is not
! needed for atovpp as this already incorporates channel selection.
!
! Method:
!   1) Read channel selection from fdf file
!   2) Modify header record
!   3) Modify all the data records
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2011, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    12/02/2019  from cris_channels.F

#include "hiras1c.h"
#include "ppparms.h"
#include "pphirasfdata.h"

! Variables
      INTEGER   C1cunit       !Unit number
      DATA C1cunit/18/
      INTEGER   C1cthinunit   !Unit number
      DATA C1cthinunit/28/
      INTEGER   Cfunit       !Unit number
      DATA Cfunit/55/
      INTEGER   Iscan,Ichan,FOV,FofR  !Loop variables
      INTEGER   rec_orig, rec_new
!
!- End of header

      WRITE(*,*)'############################'
      WRITE(*,*)'# Start hiras_channels.exe #'
      WRITE(*,*)'############################'
      
! Open the 1c files
! First get the record length of the input file

      hiras1c_h_recwords = array_size_hiras1c - 
     &      hiras_1cnchan*hiras_1cnfov*hiras_1cnfor !excluding channels
      CALL IOHIRAS1C ("OPEN/READ",C1cunit,0)
      CALL IOHIRAS1C ("READ HEADER",C1cunit,1)      !Read header
      CALL IOHIRAS1C ("CLOSE",C1cunit,1)
      CALL IOHIRAS1C ("OPEN/READ",C1cunit,0)        !Re-open with correct length
      CALL IOHIRAS1C ("READ HEADER",C1cunit,1)      !Read header

      rec_orig = hiras1c_h_recwords

      WRITE(*,*)'Channels in: ',hiras1c_h_nchanused,
     &  'record size (words):',hiras1c_h_recwords

! Read the fdf file (channels_hiras, chanvn_hiras, nchan_hiras in pphirasfdata.h)

      CALL PPHIRASFDF(Cfunit,0,0,0)

! Modify the header

      hiras1c_h_chanvn = chanvn_hiras
      hiras1c_h_nchanused = nchan_hiras
      DO Ichan=1,hiras1c_h_nchanused
        hiras1c_h_channels(Ichan) = channels_hiras(Ichan)
      ENDDO
      DO Ichan=hiras1c_h_nchanused+1,hiras_1cnchan
        hiras1c_h_channels(Ichan) = 0
      ENDDO

    !Reduce size of record
      rec_new = rec_orig - 
     & (hiras1c_h_nchan - hiras1c_h_nchanused)*hiras_1cnfov*hiras_1cnfor
      hiras1c_h_recwords = rec_new
      hiras1c_h_nchan = hiras1c_h_nchanused
      
      CALL IOHIRAS1C("OPEN/WRITE",C1cthinunit,0)
      CALL IOHIRAS1C("WRITE HEADER",C1cthinunit,1)
      
      WRITE(*,*)'Channels out:',hiras1c_h_nchanused,
     &  'record size (words):',hiras1c_h_recwords
      WRITE(*,*)'Scan lines:',hiras1c_h_scnlin

! Modify the data records

      DO Iscan=1,hiras1c_h_scnlin
        hiras1c_h_recwords = rec_orig
        CALL IOHIRAS1C("READ DATA",C1cunit,1+Iscan)
        hiras1c_h_recwords = rec_new

        DO FOV=1,hiras_1cnfov
          DO FofR=1,hiras_1cnfor
            DO Ichan=1,hiras1c_h_nchanused
              hiras1c_radiance(FOV,FofR,Ichan) = 
     &           hiras1c_radiance(FOV,FofR,hiras1c_h_channels(Ichan))
            ENDDO
          ENDDO
        ENDDO

        CALL IOHIRAS1C("WRITE DATA",C1cthinunit,1+Iscan)
      ENDDO

! Close files

      CALL IOHIRAS1C("CLOSE",C1cunit,0)
      CALL IOHIRAS1C("CLOSE",C1cthinunit,0)
      WRITE(*,*)'End hiras_channels.exe'

      END
