C LIBS = atovpp atov cal f7nl1b satid f7ml f7tp f7cp f7gp auxaapp
!+    Main program for converting IASI 1c radiances to PC format
!
      PROGRAM IASI_1C_TO_PC
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: iasi_1c_to_pc.F 382 2014-01-30 14:19:05Z frna $
!
! Purpose:
!
! Method:
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2008, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    12/09/2008  Original code. (Nigel Atkinson)
!  1.1    30/01/2014  Assign npc_IASI before call to PPIFDF
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!
!  general parameters for ATOVPP 
#include "ppparms.h"
!  level 1c format
#include "iasi1c.h"
!  level 1c PC format
#include "iasipc.h"
!  fixed data for processing IASI
#include "ppifdata.h"
!  Record length multiplier 
#include "reclopt.h"

! Local variables:

      integer I1Cunit, IPCunit, I1Crecord, IPCrecord, iscan
      DATA I1Cunit / 15 /
      DATA IPCunit / 16 /
      DATA I1Crecord / 1 /
      DATA IPCrecord / 1 /
      
      integer Ifunit,Ieigunit1,Ieigunit2,Inoiseunit,Imtfunit
      DATA Ifunit     / 45 /     !Channel selection in same format as IASI.fdf
      DATA Ieigunit1  / 46 /     !Eigenvectors for encoding
      DATA Ieigunit2  / 47 /     !Eigenvectors for decoding (not used)
      DATA Inoiseunit / 48 /     !Noise
      DATA Imtfunit   / 49 /     !MTF

!
!= End of header ==========================

! 1. Open the IASI PC input file
!-------------------------------

      write(*,*) 'Open data files'
      CALL IOI1C("OPEN/READ", I1Cunit, I1Crecord )
      CALL IOIPC("OPEN/WRITE", IPCunit, IPCrecord )
      
! 2. Read the eigenvectors etc.
!------------------------------
! Read them into the memory area defined in ppifdata.h though the data file may
! not be the same IASI.fdf as is used in atovpp.

      write(*,*) 'Read eigenvectors'
      npc_IASI = 0         !Use value from IASI.fdf
      CALL PPIFDF(Ifunit,Ieigunit1,Ieigunit2,Inoiseunit,Imtfunit)
      iasipc_h_nrchanused = nchan_IASI
      iasipc_h_npcused = npc_IASI
      iasipc_h_eigvn = eigvn
      iasipc_h_chanvn = chanvn

! 3. Read and process each scan
!------------------------------

      CALL IOI1C("READ HEADER", I1Cunit, I1Crecord )

      DO iscan=1,iasi1c_h_scnlin 
        write(*,*)'Processing scan',iscan
        I1Crecord = iscan + iasi1c_h_hdrcnt
        CALL IOI1C("READ DATA", I1Cunit, I1Crecord )

        CALL IASI_GENERATE_PCS

        IPCrecord = iscan + iasipc_h_hdrcnt
        CALL IOIPC("WRITE DATA", IPCunit, IPCrecord )
      ENDDO
      
! 4. Write the output header
!---------------------------

      CALL IOIPC("WRITE HEADER", IPCunit, 1 )

      END      
