C LIBS = atovpp atov cal f7nl1b satid f7ml f7tp f7cp f7gp auxaapp
!+    Main program for reconstructing IASI radiances from PC scores
!
      PROGRAM IASI_RECONSTRUCT
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: iasi_reconstruct.F 382 2014-01-30 14:19:05Z frna $
!
! Purpose:
!
! Method:
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2008, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    12/09/2008  Original code. (Nigel Atkinson)
!  1.1    09/09/2010  Correct I4 -> I6 when reading ev version number.
!  1.2    30/01/2014  Assign npc_IASI before call to PPIFDF
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!
!  general parameters for ATOVPP 
#include "ppparms.h"
!  level 1c PC format
#include "iasipc.h"
!  fixed data for processing IASI
#include "ppifdata.h"
!  Record length multiplier 
#include "reclopt.h"

! Local variables:

      integer IPCunit, IPCrecord, iscan, chn, iband
      DATA IPCunit / 16 /
      
      integer Ifunit,Ieigunit1,Ieigunit2,Inoiseunit,Imtfunit
      DATA Ifunit     / 45 /     !Channel selection in same format as IASI.fdf
      DATA Ieigunit1  / 46 /     !Eigenvectors for encoding (not used)
      DATA Ieigunit2  / 47 /     !Eigenvectors for decoding
      DATA Inoiseunit / 48 /     !Noise (not used)
      DATA Imtfunit   / 49 /     !MTF (not used)

      integer evfilevn, everr
!
!= End of header ==========================

! 1. Open the IASI PC input file
!-------------------------------

      write(*,*) 'Open data file'
      CALL IOIPC("OPEN/WRITE", IPCunit, IPCrecord )
      
! 2. Read the eigenvectors etc.
!------------------------------
! Read them into the memory area defined in ppifdata.h though the data file may
! not be the same IASI.fdf as is used in atovpp.

      write(*,*) 'Read eigenvectors'
      npc_IASI = Imaxnpc      !Always read the eigenvector files
      CALL PPIFDF(Ifunit,Ieigunit1,Ieigunit2,Inoiseunit,Imtfunit)

! 3. Read the header
!-------------------

      CALL IOIPC("READ HEADER", IPCunit, 1 )
      iasipc_h_nrchanused = nchan_IASI
      iasipc_h_rchanvn = chanvn
      DO chn=1,iasipc_h_nrchanused
        iasipc_h_rchannels(chn) = channels_IASI(chn)
      ENDDO
      
! 4. Check the eigenvector version number (part of file name) is OK
!------------------------------------------------------------------

      everr = 0
      DO iband=1,3
        IF (endpc(iband).GT.startpc(iband)) THEN
          evfilevn = -1
          READ(evfile(iband),'(56X,I6)',err=100) evfilevn
  100     continue
          IF (evfilevn .NE. iasipc_h_evfilevn(iband)) THEN
            everr = 1
            WRITE(*,*)'Error: band ',iband,
     &        ' has incorrect eigenvector version number.'
            WRITE(*,*)' File: ',evfile(iband)
            WRITE(*,*)' Version number of file: ',evfilevn
            WRITE(*,*)' Version number of PC scores: ',
     &          iasipc_h_evfilevn(iband)
          ELSE
            iasipc_h_evfile(iband) = evfile(iband)
          ENDIF
        ENDIF
      ENDDO
      IF (everr .eq. 1) THEN
        call aappexit(1)
      ENDIF
      
! 5. Read and process each scan
!------------------------------

      DO iscan=1,iasipc_h_scnlin 
        write(*,*)'Processing scan',iscan
        IPCrecord = iscan + iasipc_h_hdrcnt
        CALL IOIPC("READ DATA", IPCunit, IPCrecord )
        CALL IASI_GENERATE_RRAD
        CALL IOIPC("WRITE DATA", IPCunit, IPCrecord )
      ENDDO
      
! 6. Update the header
!---------------------

      CALL IOIPC("WRITE HEADER", IPCunit, 1 )

      END      
