#!/bin/ksh
#
# Add reconstructed radiances to a file in the IASI level 1C PC format
#
# Inputs:
#   IASI level 1C PC input file (argument)
#
# Environment variables:
#   $IASI_SPECFILE     File containing eigenvectors spec 
#                       (defaults to iasi_eigenvectors_spec.dat)
#   $IASI_FDF          File containing list of channels to reconstruct,
#                       in FDF format (defaults to IASI.fdf)
#   $DIR_IASI_PREPROC  defaults to $DIR_PREPROC
#
# 12/09/2008 Initial version. Nigel Atkinson
#
###############################################################################

usage(){
  echo
  echo 'usage : iasi_reconstruct filename'
  echo
}

. "$AAPP_PREFIX/ATOVS_CONF"

if [ $# != 1 ]; then
  usage
  exit 1
fi

infile=$1

IASI_SPECFILE=${IASI_SPECFILE:-iasi_eigenvectors_spec.dat}
IASI_FDF=${IASI_FDF:-IASI.fdf}
DIR_IASI_PREPROC=${DIR_IASI_PREPROC:-${DIR_PREPROC}}

if [ ! -s ${DIR_IASI_PREPROC}/$IASI_FDF ]; then
  echo "invalid or missing file ${DIR_IASI_PREPROC}/$IASI_FDF"
  exit 1
fi

#Generate binary eigenvectors/noise files if not already done

iasi_eigenvectors || exit 1

#Link to the binary files (same as in atovpp)

set $(awk '{print $1}' $DIR_IASI_PREPROC/$IASI_SPECFILE | grep -v "!")
evapod=$5
eig_encode=$6
eig_decode=$7
eig_noise=$8
ln -sf ${DIR_IASI_PREPROC}/$IASI_FDF   ${FORT}45
ln -sf ${DIR_IASI_PREPROC}/$eig_encode ${FORT}46
ln -sf ${DIR_IASI_PREPROC}/$eig_decode ${FORT}47
ln -sf ${DIR_IASI_PREPROC}/$eig_noise  ${FORT}48
ln -sf ${DIR_IASI_PREPROC}/$evapod     ${FORT}49

ln -sf $infile ${FORT}16

#Call the Fortran executable

iasi_reconstruct.exe

exitc=$?

#Tidy up

rm -f ${FORT}*
exit $exitc
