C LIBS = atovpp atov auxaapp f7nl1b
!+
!
      PROGRAM MWHS2_BEAMWIDTH
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id$
!
! Purpose:
!
! Method:
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    20/05/2014  Create from mwts2_beamwidth.F

#include "mwhs21c.h"

! Parameters
      INTEGER nymax
      PARAMETER (nymax=4096)   !Max number of lines. Allow full orbit MWHS2

! Variables
      REAL bt(mwhs2_nfovs, nymax, mwhs2_nchan)
                               !channel as last index, to simplify array passing
      
      INTEGER   M1cunit   !Unit number
      DATA M1cunit/17/
      
      INTEGER iscan,ichan,ispot,ios,nchannels,wmosatid,version
      INTEGER channelnumber(mwhs2_nchan)
      REAL    sampling_dist
      REAL    beamwidth(mwhs2_nchan)
      REAL    newwidth(mwhs2_nchan)
      REAL    cutoff(mwhs2_nchan)
      INTEGER nxaverage(mwhs2_nchan)
      INTEGER nyaverage(mwhs2_nchan)
      INTEGER qc_dist(mwhs2_nchan)
      DATA nxaverage/mwhs2_nchan*0/
      DATA nyaverage/mwhs2_nchan*0/
      DATA qc_dist/mwhs2_nchan*0/
      CHARACTER*30 Cline
      LOGICAL changes 
!
!- End of header

      WRITE(*,*)'#############################'
      WRITE(*,*)'# Start mwhs2_beamwidth.exe #'
      WRITE(*,*)'#############################'

! Open the 1c file and read all scan lines
      CALL IOMWHS21C("OPEN/WRITE",M1cunit,0)
      CALL IOMWHS21C("READ HEADER",M1cunit,1)
      DO iscan=1,mwhs21c_h_scnlin
        CALL IOMWHS21C("READ DATA",M1cunit,1+iscan)
        DO ispot=1,mwhs2_nfovs
          DO ichan=1,mwhs2_nchan
            bt(ispot,iscan,ichan) = mwhs21c_btemps(ichan,ispot)
          ENDDO
        ENDDO
      ENDDO

! Read the beamwidth requirements
      OPEN(18,file='mwhs2_beamwidth.dat',form='formatted',status='old',
     &   iostat=ios)
      IF (ios .NE. 0) THEN
        WRITE(*,*) 'Unable to open mwhs2_beamwidth.dat'
        CALL AAPPEXIT(1)
      ENDIF
      wmosatid=999
      write(*,*) 'Look for beamwidth spec for satid ',mwhs21c_h_wmosatid
      DO WHILE (wmosatid .NE. mwhs21c_h_wmosatid)
        read(18,'(a30)',end=100) Cline
        DO WHILE (Cline(1:1) .eq. '#')
          read(18,'(a30)') Cline
        ENDDO
        READ(Cline,*,iostat=ios) wmosatid
        IF (ios .NE. 0) THEN
          WRITE(*,*)'Error reading wmosatid from mwhs2_beamwidth.dat'
          CALL AAPPEXIT(1)
        ENDIF

        read(18,'(a30)') Cline
        DO WHILE (Cline(1:1) .eq. '#')
          read(18,'(a30)') Cline
        ENDDO
        READ(Cline,*,iostat=ios) version
        IF (ios .NE. 0) THEN
          WRITE(*,*)'Error reading version from mwhs2_beamwidth.dat'
          CALL AAPPEXIT(1)
        ENDIF

        read(18,'(a30)') Cline
        DO WHILE (Cline(1:1) .eq. '#')
          read(18,'(a30)') Cline
        ENDDO
        READ(Cline,*,iostat=ios) sampling_dist
        IF (ios .NE. 0) THEN
          WRITE(*,*)
     &       'Error reading sampling_dist from mwhs2_beamwidth.dat'
          CALL AAPPEXIT(1)
        ENDIF

        read(18,'(a30)') Cline
        DO WHILE (Cline(1:1) .eq. '#')
          read(18,'(a30)') Cline
        ENDDO
        READ(Cline,*,iostat=ios) nchannels
        IF (ios .NE. 0) THEN
          WRITE(*,*)'Error reading nchannels from mwhs2_beamwidth.dat'
          CALL AAPPEXIT(1)
        ENDIF

        read(18,'(a30)') Cline
        DO ichan=1,nchannels
          read(18,'(a30)') Cline
          DO WHILE (Cline(1:1) .eq. '#')
            read(18,'(a30)') Cline
          ENDDO
          READ(Cline,*,iostat=ios)channelnumber(ichan),beamwidth(ichan),
     &       newwidth(ichan),cutoff(ichan),nxaverage(ichan),
     &       nyaverage(ichan),qc_dist(ichan)
          IF (ios .NE. 0) THEN
            WRITE(*,*)'Warning: mwhs2_beamwidth.dat may be incomplete'
          ENDIF
        ENDDO
      ENDDO 
  100 IF (wmosatid .NE. mwhs21c_h_wmosatid) THEN
        WRITE(*,*) 'WARNING: sat id not matched: use',wmosatid
      ENDIF
        
! Do the image processing (if not already done)
      
      changes = .FALSE.
      DO ichan=1,nchannels
        IF (mwhs21c_h_beamwidth(channelnumber(ichan)) .EQ. 0 .AND.
     &      mwhs21c_h_nxaverage(channelnumber(ichan)) .EQ. 0 .AND.
     &      mwhs21c_h_nyaverage(channelnumber(ichan)) .EQ. 0) THEN
          changes = .true.
          CALL MODIFY_BEAMWIDTH(mwhs2_nfovs,mwhs21c_h_scnlin,
     &      bt(1,1,channelnumber(ichan)), sampling_dist,
     &      beamwidth(ichan), newwidth(ichan), cutoff(ichan),
     &      nxaverage(ichan), nyaverage(ichan), qc_dist(ichan))
          mwhs21c_h_beamwidth(channelnumber(ichan)) = 
     &       NINT(newwidth(ichan)*1.0E3)
          mwhs21c_h_cutoff(channelnumber(ichan))    =
     &       NINT(cutoff(ichan)*1.0E3)
          mwhs21c_h_nxaverage(channelnumber(ichan)) = nxaverage(ichan)
          mwhs21c_h_nyaverage(channelnumber(ichan)) = nyaverage(ichan)
          write(*,*) channelnumber(ichan),newwidth(ichan),cutoff(ichan),
     &       nxaverage(ichan),nyaverage(ichan),qc_dist(ichan)
        ENDIF
      ENDDO
     
! Over-write the input file
    
      IF (changes) THEN
        DO iscan=1,mwhs21c_h_scnlin
          CALL IOMWHS21C("READ DATA",M1cunit,1+iscan)
          DO ispot=1,mwhs2_nfovs
            DO ichan=1,mwhs2_nchan
              mwhs21c_btemps(ichan,ispot) = NINT(bt(ispot,iscan,ichan))
            ENDDO
          ENDDO
          CALL IOMWHS21C("WRITE DATA",M1cunit,1+iscan)
        ENDDO
        mwhs21c_h_beamversion = version
        CALL IOMWHS21C("WRITE HEADER",M1cunit,1)
        WRITE(*,*) 'mwhs2_beamwidth.exe: finished'
      ELSE
        WRITE(*,*) 'mwhs2_beamwidth.exe: no changes made to 1c file'
      ENDIF
      CALL IOMWHS21C("CLOSE",M1cunit,0)

      END
