#!/bin/ksh
#$Id$
#
# script to run mwhs2_beamwidth.exe: modify the MWTS2 beam width for a level 1c
# file. Requires a file $mwhs2_BEAMWIDTH_FILE (default mwhs2_beamwidth.dat)
# specifying the input and output beam widths.
#
# 20/05/2014  Nigel Atkinson from mwts2_beamwidth.F
#############################################################################

usage(){
  echo
  echo 'usage : mwhs2_beamwidth infile [outfile]'
  echo
}

[ $# -ge 1 ] || { usage; exit 0; }
[ -f $1 ] || { usage; exit 1; }

. "$AAPP_PREFIX/ATOVS_CONF"

delete=Y

mwhs2_BEAMWIDTH_FILE=${mwhs2_BEAMWIDTH_FILE:-mwhs2_beamwidth.dat}
DIR_FY3_PREPROC=${DIR_FY3_PREPROC:-${DIR_PREPROC}}

#Use version in current directory if available, otherwise look in $DIR_FY3_PREPROC

if [ -f "$mwhs2_BEAMWIDTH_FILE" ]; then
  dir=$PWD
  if [ "$mwhs2_BEAMWIDTH_FILE" = mwhs2_beamwidth.dat ]; then 
    delete=N
  else
    ln -sf "$mwhs2_BEAMWIDTH_FILE" mwhs2_beamwidth.dat
  fi
elif [ -f $DIR_FY3_PREPROC/$mwhs2_BEAMWIDTH_FILE ]; then
  dir=$DIR_FY3_PREPROC
  ln -sf $DIR_FY3_PREPROC/$mwhs2_BEAMWIDTH_FILE mwhs2_beamwidth.dat
else
  echo "No MWTS2 beamwidth file supplied"
  exit 1
fi
echo Using $dir/"$mwhs2_BEAMWIDTH_FILE"

infile=$1
outfile=${2:-$infile}

[ $outfile != $infile ] && cp $infile $outfile

ln -sf $outfile ${FORT}17

mwhs2_beamwidth.exe

exitc=$?

if [ $exitc != 0 ] && [ outfile != $infile ]; then 
  rm -f $outfile
fi

[ $delete = Y ] && rm -f mwhs2_beamwidth.dat

rm -f ${FORT}17

exit $exitc
