C LIBS = atovpp atov auxaapp f7nl1b cal
!+
!
      PROGRAM MWHS_TO_DUMMYMWTS
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: mwhs2_to_dummymwts2.F 531 2017-07-07 12:09:43Z frna $
!
! Purpose: Convert MWHS-2 l1c file to a dummy MWTS-2 file, for use when
!          MWTS-2 is broken but we still want to use the MWHS-2 information,
!          on a grid that is approximately the same as MWTS-2.
!
! Method:
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2015, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    23/02/2015  NCA

#include "mwhs21c.h"
#include "mwts21c.h"

! Parameters
      INTEGER   MWTS_SPOTS, MWTS_CHANS
      PARAMETER (MWTS_SPOTS=90)
      PARAMETER (MWTS_CHANS=13)
      INTEGER   MWHS_CHANS
      PARAMETER (MWHS_CHANS=15)

! Variables
      INTEGER   MWTS1cunit   !Unit number
      DATA MWTS1cunit/10/
      
      INTEGER   MWHS1cunit   !Unit number
      DATA MWHS1cunit/11/
      
      INTEGER iscan,ispot,jspot,ichan,i
      
      WRITE(*,*)'#################################'
      WRITE(*,*)'# Start mwhs2_to_dummymwts2.exe #'
      WRITE(*,*)'#################################'

! Open the 1c file and read all scan lines
      CALL IOMWTS21C("OPEN/WRITE",MWTS1cunit,0)
      CALL IOMWHS21C("OPEN/READ",MWHS1cunit,0)
      CALL IOMWHS21C("READ HEADER",MWHS1cunit,1)

      mwts21c_h_1bsite = mwhs21c_h_1bsite
      mwts21c_h_versnb = mwts2_1cvnnb
      mwts21c_h_versyr = mwts2_1cvnyr
      mwts21c_h_versdy = mwts2_1cvndy
      mwts21c_h_hdrcnt = mwhs21c_h_hdrcnt
      mwts21c_h_satid = mwhs21c_h_satid
      mwts21c_h_instrument = 18
      mwts21c_h_satht = mwhs21c_h_satht
      mwts21c_h_period = mwhs21c_h_period
      mwts21c_h_startorbit = mwhs21c_h_startorbit
      mwts21c_h_startdatayr = mwhs21c_h_startdatayr
      mwts21c_h_startdatady = mwhs21c_h_startdatady
      mwts21c_h_startdatatime = mwhs21c_h_startdatatime
      mwts21c_h_endorbit = mwhs21c_h_endorbit
      mwts21c_h_enddatayr = mwhs21c_h_enddatayr
      mwts21c_h_enddatady = mwhs21c_h_enddatady
      mwts21c_h_enddatatime = mwhs21c_h_enddatatime
      mwts21c_h_scnlin = mwhs21c_h_scnlin
      mwts21c_h_misscnlin = mwhs21c_h_misscnlin
      mwts21c_h_vnantennacorr = mwhs21c_h_vnantennacorr
      mwts21c_h_wmosatid = mwhs21c_h_wmosatid
      mwts21c_h_nchan = mwts2_nchan
      mwts21c_h_nmapchan = mwts2_nmapchan
      mwts21c_h_nmapchanused = mwhs2_nchan

      CALL IOMWTS21C("WRITE HEADER",MWTS1cunit,1)

      DO iscan=1,mwhs21c_h_scnlin
        CALL IOMWHS21C("READ DATA",MWHS1cunit,1+iscan)
        mwts21c_scnlin = mwhs21c_scnlin
        mwts21c_scnlinyr = mwhs21c_scnlinyr
        mwts21c_scnlindy = mwhs21c_scnlindy
        mwts21c_scnlintime = mwhs21c_scnlintime
        mwts21c_qualind = mwhs21c_qualind
        mwts21c_scanqual = mwhs21c_scanqual
        mwts21c_instrtemp1 = mwhs21c_instrtemp1
        mwts21c_instrtemp2 = mwhs21c_instrtemp2
        mwts21c_scalti = mwhs21c_scalti
        DO i=1,3
          mwts21c_satposition(i) = mwhs21c_satposition(i)
          mwts21c_satvelocity(i) = mwhs21c_satvelocity(i)
        ENDDO
        mwts21c_orbitangle = mwhs21c_orbitangle
        DO ispot=1,MWTS_SPOTS
          jspot = ispot + 4          !start at MWHS spot 5
          DO i=1,2
            mwts21c_latlon(i,ispot) = mwhs21c_latlon(i,jspot)
            mwts21c_surface(i,ispot) = mwhs21c_surface(i,jspot)
          ENDDO
          DO i=1,4
            mwts21c_angles(i,ispot) = mwhs21c_angles(i,jspot)
          ENDDO
          DO ichan=1,MWHS_CHANS
            mwts21c_btmap(ichan,ispot) = mwhs21c_btemps(ichan,jspot)
          ENDDO
          DO ichan=1,MWTS_CHANS
            mwts21c_btemps(ichan,ispot) = -999999
          ENDDO
          mwts21c_dataqual(ispot) = mwhs21c_dataqual(ispot)
        ENDDO
        CALL IOMWTS21C("WRITE DATA",MWTS1cunit,1+iscan)
      ENDDO 

      CALL IOMWTS21C("CLOSE",MWTS1cunit,0)
      CALL IOMWTS21C("CLOSE",MWHS1cunit,0)
      END
