C LIBS = atovpp atov auxaapp f7nl1b cal
!+
!
      PROGRAM MWHS2_TO_IRAS
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: mwhs2_to_iras.F 396 2014-06-11 12:19:50Z frna $
!
! Purpose:
!
! Method:
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    23/05/2014  From mwts2_to_iras.F (N.C.Atkinson)

#include "mwhs21c.h"
#include "iras1c.h"

! Parameters
      INTEGER   MWHS_SPOTS, MWHS_CHANS, MWHS_SCANS, MWTS_CHANS
      PARAMETER (MWHS_SPOTS=98)
      PARAMETER (MWHS_CHANS=15)
      PARAMETER (MWTS_CHANS=13)
      PARAMETER (MWHS_SCANS=2700)
      INTEGER   IRAS_SPOTS, IRAS_SCANS
      PARAMETER (IRAS_SPOTS=56)
      PARAMETER (IRAS_SCANS=1150)

! Variables
      INTEGER   IRAS1cunit   !Unit number
      DATA IRAS1cunit/10/
      
      INTEGER   MWHS1cunit   !Unit number
      DATA MWHS1cunit/11/
      
      REAL lati(IRAS_SPOTS,IRAS_SCANS)       !deg *1E4
      REAL loni(IRAS_SPOTS,IRAS_SCANS)
      INTEGER timei(IRAS_SCANS)              !milliseconds

      REAL latm(MWHS_SPOTS,MWHS_SCANS)       !deg
      REAL lonm(MWHS_SPOTS,MWHS_SCANS)
      INTEGER timem(MWHS_SCANS)              !milliseconds
      REAL  satheight(IRAS_SCANS)            !km
      INTEGER qualindm(MWHS_SCANS)
      REAL btm(MWHS_CHANS,MWHS_SPOTS,MWHS_SCANS)  !K

      REAL btmap(MWHS_CHANS,IRAS_SPOTS,IRAS_SCANS)  !K
      
      INTEGER iscan,ispot,ichan
      
      LOGICAL use_median
      CHARACTER*1 c_use_median
      
      WRITE(*,*)'###########################'
      WRITE(*,*)'# Start mwhs2_to_iras.exe #'
      WRITE(*,*)'###########################'

      c_use_median = 'N'
      CALL GETENV('MWHS2_USE_MEDIAN',c_use_median)
      IF (c_use_median .EQ.'Y' .OR.c_use_median .EQ.'y') THEN
        use_median = .true.
      ELSE
        use_median = .false.
      ENDIF
      write(*,*)'use_median = ',use_median

! Open the 1c file and read all scan lines
      CALL IOMWHS21C("OPEN/READ",MWHS1cunit,0)
      CALL IOMWHS21C("READ HEADER",MWHS1cunit,1)
      CALL IOIRAS1C("OPEN/WRITE",IRAS1cunit,0)
      CALL IOIRAS1C("READ HEADER",IRAS1cunit,1)

      DO iscan=1,iras1c_h_scnlin
        CALL IOIRAS1C("READ DATA",IRAS1cunit,1+iscan)
        timei(iscan) = iras1c_scnlintime
!        satheight(iscan) = iras1c_scalti*0.1
        satheight(iscan) = iras1c_h_satht*0.1
        DO ispot=1,IRAS_SPOTS
          lati(ispot,iscan) = iras1c_latlon(1,ispot)*1E-4
          loni(ispot,iscan) = iras1c_latlon(2,ispot)*1E-4
        ENDDO
      ENDDO 

      DO iscan=1,mwhs21c_h_scnlin
        CALL IOMWHS21C("READ DATA",MWHS1cunit,1+iscan)
        timem(iscan) = mwhs21c_scnlintime
        qualindm(iscan) = mwhs21c_qualind
        DO ispot=1,MWHS_SPOTS
          latm(ispot,iscan) = mwhs21c_latlon(1,ispot)*1E-4
          lonm(ispot,iscan) = mwhs21c_latlon(2,ispot)*1E-4
          DO ichan=1,MWHS_CHANS
            btm(ichan,ispot,iscan) = mwhs21c_btemps(ichan,ispot)*1E-2
          ENDDO
        ENDDO
      ENDDO 

! Re-map
      CALL map_mwhs2_to_iras(use_median,lati,loni,timei,
     &              satheight,latm,lonm,timem,qualindm,btm,btmap)

! Save mapped BT values and re-write the IRAS file

      iras1c_h_nmapchanused_mwhs = mwhs21c_h_nchan
      CALL IOIRAS1C("WRITE HEADER",IRAS1cunit,1)

      DO iscan=1,iras1c_h_scnlin
        CALL IOIRAS1C("READ DATA",IRAS1cunit,1+iscan)
        DO ispot=1,IRAS_SPOTS
          DO ichan=1,MWHS_CHANS
            iras1c_btmap(ichan+MWTS_CHANS,ispot)=
     &         NINT(btmap(ichan,ispot,iscan)*1E2)
          ENDDO
        ENDDO
        CALL IOIRAS1C("WRITE DATA",IRAS1cunit,1+iscan)
      ENDDO
      
      CALL IOIRAS1C("CLOSE",MWHS1cunit,0)
      CALL IOIRAS1C("CLOSE",IRAS1cunit,0)
      END
