#!/bin/ksh
#$Id$
#
# script to run mwhs_beamwidth.exe: modify the MWTS2 beam width for a level 1c
# file. Requires a file $MWHS_BEAMWIDTH_FILE (default mwhs_beamwidth.dat)
# specifying the input and output beam widths.
#
# 23/04/2015  Nigel Atkinson from mwhs2_beamwidth.F
#############################################################################

usage(){
  echo
  echo 'usage : mwhs_beamwidth infile [outfile]'
  echo
}

[ $# -ge 1 ] || { usage; exit 0; }
[ -f $1 ] || { usage; exit 1; }

. "$AAPP_PREFIX/ATOVS_CONF"

delete=Y

mwhs_beamwidth_FILE=${mwhs_beamwidth_FILE:-mwhs_beamwidth.dat}
DIR_FY3_PREPROC=${DIR_FY3_PREPROC:-${DIR_PREPROC}}

#Use version in current directory if available, otherwise look in $DIR_FY3_PREPROC

if [ -f "$mwhs_beamwidth_FILE" ]; then
  dir=$PWD
  if [ "$mwhs_beamwidth_FILE" = mwhs_beamwidth.dat ]; then 
    delete=N
  else
    ln -sf "$mwhs_beamwidth_FILE" mwhs_beamwidth.dat
  fi
elif [ -f $DIR_FY3_PREPROC/$mwhs_beamwidth_FILE ]; then
  dir=$DIR_FY3_PREPROC
  ln -sf $DIR_FY3_PREPROC/$mwhs_beamwidth_FILE mwhs_beamwidth.dat
else
  echo "No MWHS beamwidth file supplied"
  exit 1
fi
echo Using $dir/"$MWHS_BEAMWIDTH_FILE"

infile=$1
outfile=${2:-$infile}

[ $outfile != $infile ] && cp $infile $outfile

ln -sf $outfile ${FORT}17

mwhs_beamwidth.exe

exitc=$?

if [ $exitc != 0 ] && [ outfile != $infile ]; then 
  rm -f $outfile
fi

[ $delete = Y ] && rm -f mwhs_beamwidth.dat

rm -f ${FORT}17

exit $exitc
