C LIBS = atovpp atov auxaapp f7nl1b cal
!+
!
      PROGRAM MWHS_TO_MWTS
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id$
!
! Purpose:
!
! Method:
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  0.1    10/12/2010  Original code. (N.C.Atkinson)

#include "mwhs1c.h"
#include "mwts1c.h"

! Parameters
      INTEGER   MWTS_SPOTS, MWTS_CHANS, MWTS_SCANS
      PARAMETER (MWTS_SPOTS=15)
      PARAMETER (MWTS_CHANS=4)
      PARAMETER (MWTS_SCANS=500)
      INTEGER   MWHS_SPOTS, MWHS_CHANS, MWHS_SCANS
      PARAMETER (MWHS_SPOTS=98)
      PARAMETER (MWHS_CHANS=5)
      PARAMETER (MWHS_SCANS=2700)

! Variables
      INTEGER   MWTS1cunit   !Unit number
      DATA MWTS1cunit/10/
      
      INTEGER   MWHS1cunit   !Unit number
      DATA MWHS1cunit/11/
      
      REAL latt(MWTS_SPOTS,MWTS_SCANS)       !deg
      REAL lont(MWTS_SPOTS,MWTS_SCANS)
      INTEGER timet(MWTS_SCANS)              !milliseconds
      REAL  satheight(MWTS_SCANS)            !km

      REAL lath(MWHS_SPOTS,MWHS_SCANS)       !deg *1E4
      REAL lonh(MWHS_SPOTS,MWHS_SCANS)
      INTEGER timeh(MWHS_SCANS)              !milliseconds
      INTEGER qualindh(MWHS_SCANS)
      REAL bth(MWHS_CHANS,MWHS_SPOTS,MWHS_SCANS)  !K

      REAL btmap(MWHS_CHANS,MWTS_SPOTS,MWTS_SCANS)  !K
      
      INTEGER iscan,ispot,ichan
      
      WRITE(*,*)'##########################'
      WRITE(*,*)'# Start mwhs_to_mwts.exe #'
      WRITE(*,*)'##########################'

! Open the 1c file and read all scan lines
      CALL IOMWTS1C("OPEN/WRITE",MWTS1cunit,0)
      CALL IOMWTS1C("READ HEADER",MWTS1cunit,1)
      CALL IOMWHS1C("OPEN/WRITE",MWHS1cunit,0)
      CALL IOMWHS1C("READ HEADER",MWHS1cunit,1)

      DO iscan=1,mwts1c_h_scnlin
        CALL IOMWTS1C("READ DATA",MWTS1cunit,1+iscan)
        timet(iscan) = mwts1c_scnlintime
        satheight(iscan) = mwts1c_scalti*0.1
        DO ispot=1,MWTS_SPOTS
          latt(ispot,iscan) = mwts1c_latlon(1,ispot)*1E-4
          lont(ispot,iscan) = mwts1c_latlon(2,ispot)*1E-4
        ENDDO
      ENDDO 

      DO iscan=1,mwhs1c_h_scnlin
        CALL IOMWHS1C("READ DATA",MWHS1cunit,1+iscan)
        timeh(iscan) = mwhs1c_scnlintime
        qualindh(iscan) = mwhs1c_qualind
        DO ispot=1,MWHS_SPOTS
          lath(ispot,iscan) = mwhs1c_latlon(1,ispot)*1E-4
          lonh(ispot,iscan) = mwhs1c_latlon(2,ispot)*1E-4
          DO ichan=1,MWHS_CHANS
            bth(ichan,ispot,iscan) = mwhs1c_btemps(ichan,ispot)*1E-2
          ENDDO
        ENDDO
      ENDDO 

! Re-map
      CALL map_mwhs_to_mwts(latt,lont,timet,satheight,
     &                      lath,lonh,timeh,qualindh,bth,btmap)

! Save mapped BT values and re-write the MWTS file

      DO iscan=1,mwts1c_h_scnlin
        CALL IOMWTS1C("READ DATA",MWTS1cunit,1+iscan)
        DO ispot=1,MWTS_SPOTS
          DO ichan=1,MWHS_CHANS
            mwts1c_btmap(ichan,ispot)=NINT(btmap(ichan,ispot,iscan)*1E2)
          ENDDO
        ENDDO
        CALL IOMWTS1C("WRITE DATA",MWTS1cunit,1+iscan)
      ENDDO
      
      CALL IOMWTS1C("CLOSE",MWTS1cunit,0)
      CALL IOMWTS1C("CLOSE",MWHS1cunit,0)
      END
