C LIBS = atovpp atov auxaapp f7nl1b
!+
!
      PROGRAM MWTS2_BEAMWIDTH
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id$
!
! Purpose:
!
! Method:
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    19/05/2014  Create from mwts2_beamwidth.F

#include "mwts21c.h"

! Parameters
      INTEGER nymax
      PARAMETER (nymax=4096)   !Max number of lines. Allow full orbit MWTS2

! Variables
      REAL bt(mwts2_nfovs, nymax, mwts2_nchan)
                               !channel as last index, to simplify array passing
      
      INTEGER   M1cunit   !Unit number
      DATA M1cunit/17/
      
      INTEGER iscan,ichan,ispot,ios,nchannels,wmosatid,version
      INTEGER channelnumber(mwts2_nchan)
      REAL    sampling_dist
      REAL    beamwidth(mwts2_nchan)
      REAL    newwidth(mwts2_nchan)
      REAL    cutoff(mwts2_nchan)
      INTEGER nxaverage(mwts2_nchan)
      INTEGER nyaverage(mwts2_nchan)
      INTEGER qc_dist(mwts2_nchan)
      DATA nxaverage/mwts2_nchan*0/
      DATA nyaverage/mwts2_nchan*0/
      DATA qc_dist/mwts2_nchan*0/
      CHARACTER*30 Cline
      LOGICAL changes 
!
!- End of header

      WRITE(*,*)'#############################'
      WRITE(*,*)'# Start mwts2_beamwidth.exe #'
      WRITE(*,*)'#############################'

! Open the 1c file and read all scan lines
      CALL IOMWTS21C("OPEN/WRITE",M1cunit,0)
      CALL IOMWTS21C("READ HEADER",M1cunit,1)
      DO iscan=1,mwts21c_h_scnlin
        CALL IOMWTS21C("READ DATA",M1cunit,1+iscan)
        DO ispot=1,mwts2_nfovs
          DO ichan=1,mwts2_nchan
            bt(ispot,iscan,ichan) = mwts21c_btemps(ichan,ispot)
          ENDDO
        ENDDO
      ENDDO

! Read the beamwidth requirements
      OPEN(18,file='mwts2_beamwidth.dat',form='formatted',status='old',
     &   iostat=ios)
      IF (ios .NE. 0) THEN
        WRITE(*,*) 'Unable to open mwts2_beamwidth.dat'
        CALL AAPPEXIT(1)
      ENDIF
      wmosatid=999
      write(*,*) 'Look for beamwidth spec for satid ',mwts21c_h_wmosatid
      DO WHILE (wmosatid .NE. mwts21c_h_wmosatid)
        read(18,'(a30)',end=100) Cline
        DO WHILE (Cline(1:1) .eq. '#')
          read(18,'(a30)') Cline
        ENDDO
        READ(Cline,*,iostat=ios) wmosatid
        IF (ios .NE. 0) THEN
          WRITE(*,*)'Error reading wmosatid from mwts2_beamwidth.dat'
          CALL AAPPEXIT(1)
        ENDIF

        read(18,'(a30)') Cline
        DO WHILE (Cline(1:1) .eq. '#')
          read(18,'(a30)') Cline
        ENDDO
        READ(Cline,*,iostat=ios) version
        IF (ios .NE. 0) THEN
          WRITE(*,*)'Error reading version from mwts2_beamwidth.dat'
          CALL AAPPEXIT(1)
        ENDIF

        read(18,'(a30)') Cline
        DO WHILE (Cline(1:1) .eq. '#')
          read(18,'(a30)') Cline
        ENDDO
        READ(Cline,*,iostat=ios) sampling_dist
        IF (ios .NE. 0) THEN
          WRITE(*,*)
     &       'Error reading sampling_dist from mwts2_beamwidth.dat'
          CALL AAPPEXIT(1)
        ENDIF

        read(18,'(a30)') Cline
        DO WHILE (Cline(1:1) .eq. '#')
          read(18,'(a30)') Cline
        ENDDO
        READ(Cline,*,iostat=ios) nchannels
        IF (ios .NE. 0) THEN
          WRITE(*,*)'Error reading nchannels from mwts2_beamwidth.dat'
          CALL AAPPEXIT(1)
        ENDIF

        read(18,'(a30)') Cline
        DO ichan=1,nchannels
          read(18,'(a30)') Cline
          DO WHILE (Cline(1:1) .eq. '#')
            read(18,'(a30)') Cline
          ENDDO
          READ(Cline,*,iostat=ios)channelnumber(ichan),beamwidth(ichan),
     &       newwidth(ichan),cutoff(ichan),nxaverage(ichan),
     &       nyaverage(ichan),qc_dist(ichan)
          IF (ios .NE. 0) THEN
            WRITE(*,*)'Warning: mwts2_beamwidth.dat may be incomplete'
          ENDIF
        ENDDO
      ENDDO 
  100 IF (wmosatid .NE. mwts21c_h_wmosatid) THEN
        WRITE(*,*) 'WARNING: sat id not matched: use',wmosatid
      ENDIF
        
! Do the image processing (if not already done)
      
      changes = .FALSE.
      DO ichan=1,nchannels
        IF (mwts21c_h_beamwidth(channelnumber(ichan)) .EQ. 0 .AND.
     &      mwts21c_h_nxaverage(channelnumber(ichan)) .EQ. 0 .AND.
     &      mwts21c_h_nyaverage(channelnumber(ichan)) .EQ. 0) THEN
          changes = .true.
          CALL MODIFY_BEAMWIDTH(mwts2_nfovs,mwts21c_h_scnlin,
     &      bt(1,1,channelnumber(ichan)), sampling_dist,
     &      beamwidth(ichan), newwidth(ichan), cutoff(ichan),
     &      nxaverage(ichan), nyaverage(ichan), qc_dist(ichan))
          mwts21c_h_beamwidth(channelnumber(ichan)) = 
     &       NINT(newwidth(ichan)*1.0E3)
          mwts21c_h_cutoff(channelnumber(ichan))    =
     &       NINT(cutoff(ichan)*1.0E3)
          mwts21c_h_nxaverage(channelnumber(ichan)) = nxaverage(ichan)
          mwts21c_h_nyaverage(channelnumber(ichan)) = nyaverage(ichan)
          write(*,*) channelnumber(ichan),newwidth(ichan),cutoff(ichan),
     &       nxaverage(ichan),nyaverage(ichan),qc_dist(ichan)
        ENDIF
      ENDDO
     
! Over-write the input file
    
      IF (changes) THEN
        DO iscan=1,mwts21c_h_scnlin
          CALL IOMWTS21C("READ DATA",M1cunit,1+iscan)
          DO ispot=1,mwts2_nfovs
            DO ichan=1,mwts2_nchan
              mwts21c_btemps(ichan,ispot) = NINT(bt(ispot,iscan,ichan))
            ENDDO
          ENDDO
          CALL IOMWTS21C("WRITE DATA",M1cunit,1+iscan)
        ENDDO
        mwts21c_h_beamversion = version
        CALL IOMWTS21C("WRITE HEADER",M1cunit,1)
        WRITE(*,*) 'mwts2_beamwidth.exe: finished'
      ELSE
        WRITE(*,*) 'mwts2_beamwidth.exe: no changes made to 1c file'
      ENDIF
      CALL IOMWTS21C("CLOSE",M1cunit,0)

      END
