C LIBS = atovpp atov auxaapp f7nl1b cal
!+
!
      PROGRAM MWTS2_TO_IRAS
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: mwts2_to_iras.F 396 2014-06-11 12:19:50Z frna $
!
! Purpose:
!
! Method:
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    23/05/2014  From mwts2_to_mwhs2.F (N.C.Atkinson)

#include "mwts21c.h"
#include "iras1c.h"

! Parameters
      INTEGER   MWTS_SPOTS, MWTS_CHANS, MWTS_SCANS
      PARAMETER (MWTS_SPOTS=90)
      PARAMETER (MWTS_CHANS=13)
      PARAMETER (MWTS_SCANS=2700)
      INTEGER   IRAS_SPOTS, IRAS_SCANS
      PARAMETER (IRAS_SPOTS=56)
      PARAMETER (IRAS_SCANS=1150)

! Variables
      INTEGER   IRAS1cunit   !Unit number
      DATA IRAS1cunit/10/
      
      INTEGER   MWTS1cunit   !Unit number
      DATA MWTS1cunit/11/
      
      REAL lath(IRAS_SPOTS,IRAS_SCANS)       !deg *1E4
      REAL lonh(IRAS_SPOTS,IRAS_SCANS)
      INTEGER timeh(IRAS_SCANS)              !milliseconds

      REAL latt(MWTS_SPOTS,MWTS_SCANS)       !deg
      REAL lont(MWTS_SPOTS,MWTS_SCANS)
      INTEGER timet(MWTS_SCANS)              !milliseconds
      REAL  satheight(IRAS_SCANS)            !km
      INTEGER qualindt(MWTS_SCANS)
      REAL btt(MWTS_CHANS,MWTS_SPOTS,MWTS_SCANS)  !K

      REAL btmap(MWTS_CHANS,IRAS_SPOTS,IRAS_SCANS)  !K
      
      INTEGER iscan,ispot,ichan
      
      LOGICAL use_median
      CHARACTER*1 c_use_median
      
      WRITE(*,*)'###########################'
      WRITE(*,*)'# Start mwts2_to_iras.exe #'
      WRITE(*,*)'###########################'

      c_use_median = 'N'
      CALL GETENV('MWTS2_USE_MEDIAN',c_use_median)
      IF (c_use_median .EQ.'Y' .OR.c_use_median .EQ.'y') THEN
        use_median = .true.
      ELSE
        use_median = .false.
      ENDIF
      write(*,*)'use_median = ',use_median

! Open the 1c file and read all scan lines
      CALL IOMWTS21C("OPEN/READ",MWTS1cunit,0)
      CALL IOMWTS21C("READ HEADER",MWTS1cunit,1)
      CALL IOIRAS1C("OPEN/WRITE",IRAS1cunit,0)
      CALL IOIRAS1C("READ HEADER",IRAS1cunit,1)

      DO iscan=1,iras1c_h_scnlin
        CALL IOIRAS1C("READ DATA",IRAS1cunit,1+iscan)
        timeh(iscan) = iras1c_scnlintime
!        satheight(iscan) = iras1c_scalti*0.1
        satheight(iscan) = iras1c_h_satht*0.1
        DO ispot=1,IRAS_SPOTS
          lath(ispot,iscan) = iras1c_latlon(1,ispot)*1E-4
          lonh(ispot,iscan) = iras1c_latlon(2,ispot)*1E-4
        ENDDO
      ENDDO 

      DO iscan=1,mwts21c_h_scnlin
        CALL IOMWTS21C("READ DATA",MWTS1cunit,1+iscan)
        timet(iscan) = mwts21c_scnlintime
        qualindt(iscan) = mwts21c_qualind
        DO ispot=1,MWTS_SPOTS
          latt(ispot,iscan) = mwts21c_latlon(1,ispot)*1E-4
          lont(ispot,iscan) = mwts21c_latlon(2,ispot)*1E-4
          DO ichan=1,MWTS_CHANS
            btt(ichan,ispot,iscan) = mwts21c_btemps(ichan,ispot)*1E-2
          ENDDO
        ENDDO
      ENDDO 

! Re-map
      CALL map_mwts2_to_iras(use_median,lath,lonh,timeh,
     &              satheight,latt,lont,timet,qualindt,btt,btmap)

! Save mapped BT values and re-write the IRAS file

      iras1c_h_nmapchanused_mwts = mwts21c_h_nchan
      CALL IOIRAS1C("WRITE HEADER",IRAS1cunit,1)

      DO iscan=1,iras1c_h_scnlin
        CALL IOIRAS1C("READ DATA",IRAS1cunit,1+iscan)
        DO ispot=1,IRAS_SPOTS
          DO ichan=1,MWTS_CHANS
            iras1c_btmap(ichan,ispot)=
     &         NINT(btmap(ichan,ispot,iscan)*1E2)
          ENDDO
        ENDDO
        CALL IOIRAS1C("WRITE DATA",IRAS1cunit,1+iscan)
      ENDDO
      
      CALL IOIRAS1C("CLOSE",MWTS1cunit,0)
      CALL IOIRAS1C("CLOSE",IRAS1cunit,0)
      END
