C LIBS = atovpp atov auxaapp f7nl1b cal
!+
!
      PROGRAM MWTS2_TO_MWHS2
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: mwts2_to_mwhs2.F 396 2014-06-11 12:19:50Z frna $
!
! Purpose:
!
! Method:
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    23/05/2014  From mwts2_to_mwhs2.F (N.C.Atkinson)

#include "mwts21c.h"
#include "mwhs21c.h"

! Parameters
      INTEGER   MWTS_SPOTS, MWTS_CHANS, MWTS_SCANS
      PARAMETER (MWTS_SPOTS=90)
      PARAMETER (MWTS_CHANS=13)
      PARAMETER (MWTS_SCANS=2700)
      INTEGER   MWHS_SPOTS, MWHS_CHANS, MWHS_SCANS
      PARAMETER (MWHS_SPOTS=98)
      PARAMETER (MWHS_CHANS=15)
      PARAMETER (MWHS_SCANS=2700)

! Variables
      INTEGER   MWHS1cunit   !Unit number
      DATA MWHS1cunit/10/
      
      INTEGER   MWTS1cunit   !Unit number
      DATA MWTS1cunit/11/
      
      REAL lath(MWHS_SPOTS,MWHS_SCANS)       !deg *1E4
      REAL lonh(MWHS_SPOTS,MWHS_SCANS)
      INTEGER timeh(MWHS_SCANS)              !milliseconds

      REAL latt(MWTS_SPOTS,MWTS_SCANS)       !deg
      REAL lont(MWTS_SPOTS,MWTS_SCANS)
      INTEGER timet(MWTS_SCANS)              !milliseconds
      REAL  satheight(MWHS_SCANS)            !km
      INTEGER qualindt(MWTS_SCANS)
      REAL btt(MWTS_CHANS,MWTS_SPOTS,MWTS_SCANS)  !K

      REAL btmap(MWTS_CHANS,MWHS_SPOTS,MWHS_SCANS)  !K
      
      INTEGER iscan,ispot,ichan
      
      LOGICAL use_median
      CHARACTER*1 c_use_median
      
      WRITE(*,*)'############################'
      WRITE(*,*)'# Start mwts2_to_mwhs2.exe #'
      WRITE(*,*)'############################'

      c_use_median = 'N'
      CALL GETENV('MWTS2_USE_MEDIAN',c_use_median)
      IF (c_use_median .EQ.'Y' .OR.c_use_median .EQ.'y') THEN
        use_median = .true.
      ELSE
        use_median = .false.
      ENDIF
      write(*,*)'use_median = ',use_median

! Open the 1c file and read all scan lines
      CALL IOMWTS21C("OPEN/READ",MWTS1cunit,0)
      CALL IOMWTS21C("READ HEADER",MWTS1cunit,1)
      CALL IOMWHS21C("OPEN/WRITE",MWHS1cunit,0)
      CALL IOMWHS21C("READ HEADER",MWHS1cunit,1)

      DO iscan=1,mwhs21c_h_scnlin
        CALL IOMWHS21C("READ DATA",MWHS1cunit,1+iscan)
        timeh(iscan) = mwhs21c_scnlintime
        satheight(iscan) = mwhs21c_scalti*0.1
        DO ispot=1,MWHS_SPOTS
          lath(ispot,iscan) = mwhs21c_latlon(1,ispot)*1E-4
          lonh(ispot,iscan) = mwhs21c_latlon(2,ispot)*1E-4
        ENDDO
      ENDDO 

      DO iscan=1,mwts21c_h_scnlin
        CALL IOMWTS21C("READ DATA",MWTS1cunit,1+iscan)
        timet(iscan) = mwts21c_scnlintime
        qualindt(iscan) = mwts21c_qualind
        DO ispot=1,MWTS_SPOTS
          latt(ispot,iscan) = mwts21c_latlon(1,ispot)*1E-4
          lont(ispot,iscan) = mwts21c_latlon(2,ispot)*1E-4
          DO ichan=1,MWTS_CHANS
            btt(ichan,ispot,iscan) = mwts21c_btemps(ichan,ispot)*1E-2
          ENDDO
        ENDDO
      ENDDO 

! Re-map
      CALL map_mwts2_to_mwhs2(use_median,lath,lonh,timeh,
     &              satheight,latt,lont,timet,qualindt,btt,btmap)

! Save mapped BT values and re-write the MWHS file

      mwhs21c_h_nmapchanused = mwhs21c_h_nmapchan
      CALL IOMWHS21C("WRITE HEADER",MWHS1cunit,1)

      DO iscan=1,mwhs21c_h_scnlin
        CALL IOMWHS21C("READ DATA",MWHS1cunit,1+iscan)
        DO ispot=1,MWHS_SPOTS
          DO ichan=1,MWTS_CHANS
            mwhs21c_btmap(ichan,ispot)=
     &         NINT(btmap(ichan,ispot,iscan)*1E2)
          ENDDO
        ENDDO
        CALL IOMWHS21C("WRITE DATA",MWHS1cunit,1+iscan)
      ENDDO
      
      CALL IOMWHS21C("CLOSE",MWTS1cunit,0)
      CALL IOMWHS21C("CLOSE",MWHS1cunit,0)
      END
