!+  Convert day of year to month/day
!
      SUBROUTINE ConvDay (Year, YrDay, Month, Day )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: convday.F 25 2006-02-03 15:43:53Z frna $
!
! Description:
!   Convert day of year to month/day
!
! Method:
!   Count up the days of the months until we get past YrDay.
!   If year is divisible by 4, it is a leap year,
!   unless it is divisible by 100, excepting when it is
!   divisible by 400 !
!   If YrDay is greater than one year, then Month=0 is returned.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version    Date     Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Subroutine arguments
!
!   Scalar arguments with intent(In):
      INTEGER Year                ! year, e.g. 1995
      INTEGER YrDay               ! day of the year (1-366)
!
!   Scalar arguments with intent(Out):
      INTEGER Month               ! month (1-12 or 0 on error)
      INTEGER Day                 ! day of the month
!
!   Local scalars
      INTEGER ndays               ! count up the days through each month
      INTEGER yrdays              ! number of days in the year
!
!   Local arrays
      INTEGER Mdays(12)           ! number of days in each month
      DATA Mdays / 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 /
!
!- End of header


! 1.  Is this a leap year ?
!     -- ---- - ---- ----

      yrdays = 365   ! number of days in year
      Mdays(2) = 28  ! not a leap year, unless..
      IF ( MOD(Year,4).EQ.0 ) THEN

         IF (MOD(Year,100).NE.0 .OR. MOD(Year,400).EQ.0) THEN
            yrdays = 366
            Mdays(2) = 29   ! leap year
         ENDIF

      ENDIF


! 2.  Count the months
!     ----- --- ------

      Month = 0
      Day = 0
      IF ( YrDay.LE.yrdays ) THEN    ! valid year day

         ndays = 0
         DO WHILE ( ndays.LT.YrDay )
            Month = Month + 1
            ndays = ndays + Mdays(Month)
         ENDDO  ! WHILE

         Day = YrDay - (ndays-Mdays(Month))

      ENDIF


      RETURN
      END
