!+ Routine to handle error reports (ATOVIN & ATOVPP)
!
      SUBROUTINE ErrorReport (ErrStatus, ErrMessage, NameOfRoutine)
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: errorrep.F 55 2006-09-21 08:35:34Z frna $
!
! Purpose:
!   Write out fatal and warning error messages to unit 6.
!   Execution is stopped in the event of a fatal error.
!
! Method:
!   If ErrStatus = 0 no action is taken.
!   If ErrStatus < 0 a Warning message is output.
!   If ErrStatus > 0 a Fatal Error message is output, and execution stopped.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      2/2/96  Original code. (R.J.Renshaw)
!       - based on routine of same name in Met Office VAR project.
!  1.1     27/11/96 STOP 999 changed to CALL EXIT to alert
!                   unix that program has failed.  (R.J.Renshaw)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations
!
! Global variables:
!
! Subroutine arguments
!   Scalar arguments with intent(in):
      INTEGER       ErrStatus     ! +ve => fatal error, -ve => warning
      CHARACTER*(*) ErrMessage    ! ..to output
      CHARACTER*(*) NameOfRoutine ! ..calling this one

!- End of header

! *** Warning

      IF (ErrStatus .LT. 0) THEN
         WRITE(*,"(1X,A,I6,A,A)") "!! ErrorReport: WARNING ",
     &                  ErrStatus," in module ",NameOfRoutine
         WRITE(*,"(1X,A)") ErrMessage


! *** Fatal Error

      ELSEIF (ErrStatus .GT. 0) THEN
         WRITE(*,"(1X,A,I6,A,A)") "!! ErrorReport: FATAL ERROR ",
     &                  ErrStatus," in module ",NameOfRoutine
         WRITE(*,"(1X,A)") ErrMessage
         CALL aappexit(ErrStatus)

      ENDIF

! *** ELSE No problems

      RETURN
      END
