!+  Increment a Date/Time by a specified number of milliseconds    (ATOVPP)
!
      SUBROUTINE TimeAdd ( Year1,Day1,Time1,TimeInc, Year2,Day2,Time2 )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: timeadd.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Return the date/time that is TimeInc milliseconds after the input date/time.
!
! Method:
!   Partition TimeInc into days and milliseconds. TimeInc may be negative.
!   Adjust the new date and time so that day is between 1 and 365(6),
!   and time is between 0 and 86400000-1.
!   Code takes proper account of leap years, using function NumDays.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      22/02/95  Original code. (R.J.Renshaw)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!
! Subroutine arguments
!   Scalar arguments with intent(in):
      INTEGER Year1               ! year
      INTEGER Day1                ! day of year
      INTEGER Time1               ! time of day (milliseconds)
      INTEGER TimeInc             ! time increment (milliseconds)

!   Scalar arguments with intent(out):
      INTEGER Year2               ! year
      INTEGER Day2                ! day of year
      INTEGER Time2               ! time of day (milliseconds)

!   Local scalars:
      INTEGER  NumDays              ! ATOVPP fn returning no of days in year

! Function & Subroutine calls:
      External NumDays              ! ATOVPP fn returning no of days in year

!- End of header

! 1.  Partition TimeInc into days and milliseconds
!     --------- ------- ---- ---- --- ------------

      Time2 = Time1 + MOD(TimeInc,86400000)
      Day2  = Day1 + TimeInc / 86400000  ! divide by 1 day (in milliseconds)
      Year2 = Year1

! 2.  Adjust new date & time to be "legal"
!     ------ --- ---- - ---- -- -- -------

      IF (TimeInc.GE.0) THEN

!        increment day number if necessary
         IF ( Time2 .GE. 86400000 ) THEN
              Time2 = Time2 - 86400000
              Day2  = Day2 + 1
         ENDIF

!        increment year number as necessary
         DO WHILE ( Day2.GT.NumDays(Year2) )
            Day2  = Day2  - NumDays(Year2)
            Year2 = Year2 + 1
         ENDDO

      ELSE      ! time increment .LT. 0

!        decrement day number if necessary
         IF ( Time2 .LT. 0 ) THEN
              Day2  = Day2 - 1
              Time2 = Time2 + 86400000
         ENDIF

!        decrement year number as necessary
         DO WHILE ( Day2.LE.0 )
            Year2 = Year2 - 1
            Day2  = Day2  +  NumDays(Year2)
         ENDDO

      ENDIF

      RETURN
      END
