!+  Calculate difference in milliseconds between two date/times    (ATOVPP)
!
      Subroutine TimeSub ( Year1, Day1, Time1,
     &                     Year2, Day2, Time2,
     &                     dTime, ErrStatus )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: timesub.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Calculate difference in milliseconds between two date/times.
!   ( Date1 - Date2 )
!
! Method:
!   Calculate difference in whole days between the two dates.
!   Convert to milliseconds, and add difference in times.
!   Code takes proper account of leap years, using function INDays.
!
!   Have to be careful, because numbers can get too big for integer*4.
!   Subroutine returns ErrStatus=1 if dates are > 23 days apart.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      22/02/95  Original code. (R.J.Renshaw)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!
! Subroutine arguments
!   Scalar arguments with intent(in):
                                  ! First Date & Time
      INTEGER Year1               !    year
      INTEGER Day1                !    day of year
      INTEGER Time1               !    time of day (milliseconds)
                                  ! Second Date & Time
      INTEGER Year2               !    year
      INTEGER Day2                !    day of year
      INTEGER Time2               !    time of day (milliseconds)

!   Scalar arguments with intent(out):
      INTEGER dTime               ! time difference (milliseconds)
      INTEGER ErrStatus           ! 0 = OK, 1 = numbers out of range

!   Local scalars:
      INTEGER NumDays             ! ATOVPP fn returning no of days in year
      INTEGER dDay                ! difference in days between dates
      INTEGER Year                ! year

! Function & Subroutine calls:
      External NumDays            ! ATOVPP fn returning no of days in year

!- End of header

      ErrStatus = 0               ! no errors yet

!     can't handle very big numbers with integer*4 !
      IF (ABS(Year1-Year2).GT.1) THEN
         ErrStatus = 1
         GOTO 999
      ENDIF

!     difference in days
      dDay  = Day1 - Day2

!     + add on for difference in years
      Year = Year2
      DO WHILE ( Year1 .NE. Year )
        IF ( Year1.GT.Year ) THEN
             dDay = dDay + NumDays(Year)
             Year = Year + 1
        ELSE
             Year = Year - 1
             dDay = dDay - NumDays(Year)
        ENDIF
      ENDDO

!     can't handle very big numbers with integer*4 !
      IF (ABS(dDay).GT.23) THEN
         ErrStatus = 1
         GOTO 999
      ENDIF

!     difference in milliseconds
      dTime = Time1 - Time2   + 86400000*dDay

 999  CONTINUE   ! jump here on error

      RETURN
      END

