!+
!ROUTINE    AMB_EARTHCORR    for use within ATOVIN
!
!Name: $Id: amb_earthcorr.F 14 2006-01-17 10:32:42Z frna $
!
!PURPOSE    Correct AMSU-B earth-view counts for bias errors. Use 
!           transmitter powers from amb_ahk for the scan line being read.
!           Apply anomalous bias if bit 5 in 'amb_qualind' is set.
!
!VERSION    1.1      20/01/99     N C Atkinson, UKMO
!           1.2      02/03/99     Correct bug
!           1.3      23/04/99     Add STX-1 March 99 bias
!           1.4      05/03/04     Changes for Linux (A Korpela)
!                                 and correct initialisation (N Atkinson)
!           1.5      16/01/05     remove comment in include statement
!
!ARGUMENTS  None
!   
!##############################################################################

      SUBROUTINE AMB_EARTHCORR

      implicit none
c-----------------------------------------
c
c
#include "inparms.h"
#include "amb1b.h"
#include "inbstore.h" 
c                       Defines: INTEGER Bcounts(Bnchan,Bnfovs)
c                                INTEGER*2 CountsCorr(5,90,4)
c                                INTEGER*2 NewCorr(5,90,3)
c                                INTEGER*2 STXPowTmp(5) 

      INTEGER*2 STX1Corr(5,90),STX1Table(5,21),STX1Threshold(2)
      COMMON/STX1Com/ STX1Corr,STX1Table,STX1Threshold

      INTEGER Chan,I,Tx,scnlinmod,ECorr(5,90),Tmp
      REAL F
      INTEGER ahkold, scan1offset
      DATA ahkold/0/, scan1offset/0/
      SAVE ahkold, scan1offset     !Retain values after end of subroutine
!
! Look for updating of an 8-second analogs to confirm where in the 8-second
! cycle we are. This is necessary for AAPP, which can start at any scan.
! ANAGET is only called once per 8 seconds.
!
      Tmp = amb_ahk(9)+amb_ahk(14)
      IF (Tmp.NE.ahkold) THEN
        ahkold = Tmp
        scan1offset = MOD(amb_scnlin-1,3)
!        write(6,*) amb_scnlin,tmp
      ENDIF
!
! Calculate current corrections for 'normal' bias
!
      IF (amb_scnlin.LT.6) THEN         !For scans 1-5, use powers from line 6
        DO Tx=1,5
          amb_ahk(22+Tx) = STXPowTmp(Tx)
        ENDDO
      ENDIF

      DO Tx=1,4
        IF (Tx.LT.4) THEN       !STX 1-3
          F = amb_ahk(22+Tx)/(0.1*amb_h_txpow(Tx))
        ELSE                    !Use sum of SARR-A and SARR-B powers
          F = (amb_ahk(22+Tx)+amb_ahk(23+Tx))/(0.1*amb_h_txpow(Tx))
        ENDIF
        IF (Tx.EQ.1.OR.F.GT.0) THEN   !Always initialise ECorr using STX-1
          DO Chan=1,5
            IF (Tx.eq.1) THEN 
              DO I=1,90
                ECorr(Chan,I) = CountsCorr(Chan,I,Tx)*F
              ENDDO
            ELSE
              DO I=1,90
                ECorr(Chan,I)=ECorr(Chan,I)+CountsCorr(Chan,I,Tx)*F
              ENDDO
            ENDIF
          ENDDO
        ENDIF
      ENDDO
!
! Add anomalous bias if flag is set
!
!     IF ((amb_qualind.AND.32).GT.0) THEN
      IF (IAND(amb_qualind,32).GT.0) THEN
!
! Use scan line number to deduce which scan in the 8 second cycle we are in.
!
        scnlinmod = mod(amb_scnlin-scan1offset-1, 3) + 1
!        write(6,*) amb_scnlin, scnlinmod
        DO Chan=1,5
          Do I=1,90
            ECorr(Chan,I) = ECorr(Chan,I) + NewCorr(Chan,I,scnlinmod)
          ENDDO
        ENDDO
      ENDIF
!
! Add STX-1 anomaly if transmitter power is within defined range
!
      IF (amb_ahk(23).GE.STX1Threshold(1).AND.
     &    amb_ahk(23).LE.STX1Threshold(2)) THEN
        DO Chan=1,5
          Do I=1,90
            ECorr(Chan,I) = ECorr(Chan,I) + STX1Corr(Chan,I)
          ENDDO
        ENDDO
      ENDIF  
!
! Apply current corrections
!
      DO Chan=1,5
        Do I=1,90
          BCounts(Chan,I) = BCounts(Chan,I) + ECorr(Chan,I)
        ENDDO
      ENDDO
      RETURN
      END
!##############################################################################
