!+
!ROUTINE    AMB_GETCORR
!
!Name: $Id: amb_getcorr.F 25 2006-02-03 15:43:53Z frna $
!
!PURPOSE    Extract normal and anomalous bias coefficients from AMSU-B 1B
!           header, interpolate to 90 pixels and store in common area.
!
!VERSION    1.0      15/12/98    N C Atkinson, UKMO
!
!ARGUMENTS  None
!##############################################################################

      SUBROUTINE AMB_GETCORR

      implicit none
c-----------------------------------------
c
c
#include "inparms.h"
#include "amb1b.h"
!  Defines CountsCorr and NewCorr 
#include "inbstore.h"

      INTEGER Tx,Chan,Pix,Pix1,Pix2,P1,P2
      REAL F,FF
      REAL Grad(5,31,4)    !First derivative of correction curve
!
! 1) Normal Bias at 5 pixel intervals, 19 points
!
! Calculate Gradients
!
      DO Chan=1,5
        DO Tx=1,4
          DO Pix=2,18
            Grad(Chan,Pix,Tx) = 0.1*(amb_h_CountsCorr(Chan,Pix+1,Tx) -
     &                               amb_h_CountsCorr(Chan,Pix-1,Tx))
          ENDDO
          Grad(Chan,1,Tx) = 2*Grad(Chan,2,Tx) - Grad(Chan,3,Tx)
          Grad(Chan,19,Tx) = 2*Grad(Chan,18,Tx) - Grad(Chan,17,Tx)
        ENDDO
      ENDDO
!
! Interpolate
!
      DO Pix=1,90
        P1 = Pix/5 + 1      ! 1 to 19
        P2 = Pix/5 + 2      ! 2 to 20
        Pix1 = (P1-1)*5     !0 to 90
        Pix2 = (P2-1)*5     !5 to 95
        IF (Pix1 .EQ. 0) Pix1=1              !First point is pixel 1
        IF (P2 .GT. 19) P2 = 19
        F = (Pix2-Pix)/(1.0*(Pix2-Pix1))
        FF = 0.5*F*(Pix-Pix1)
        DO Chan=1,5
          DO Tx=1,4
            CountsCorr(Chan,Pix,Tx)=NINT(amb_h_CountsCorr(Chan,P1,Tx)*F
     &           + amb_h_CountsCorr(Chan,P2,Tx)*(1.0-F)
     &           + (Grad(Chan,P1,Tx)-Grad(Chan,P2,Tx))*FF)
          ENDDO
        ENDDO
      ENDDO  
!
! 2) Anomalous Bias at 3 pixel intervals, 31 points
!
! Calculate Gradients
!
      DO Chan=1,5
        DO Tx=1,3      !Separate correction for each of 3 scans
          DO Pix=2,30
            Grad(Chan,Pix,Tx) = (amb_h_NewCorr(Chan,Pix+1,Tx)-
     &                           amb_h_NewCorr(Chan,Pix-1,Tx))/6.0
          ENDDO
          Grad(Chan,1,Tx) = 2*Grad(Chan,2,Tx) - Grad(Chan,3,Tx)
          Grad(Chan,31,Tx) = 2*Grad(Chan,30,Tx) - Grad(Chan,29,Tx)
        ENDDO
      ENDDO
!
! Interpolate
!
      DO Pix=1,90
        P1 = Pix/3 + 1      ! 1 to 31
        P2 = Pix/3 + 2      ! 2 to 32
        Pix1 = (P1-1)*3     !0 to 90
        Pix2 = (P2-1)*3     !3 to 93
        IF (Pix1 .EQ. 0) Pix1=1              !First point is pixel 1
        IF (P2 .GT. 31) P2 = 31
        F = (Pix2-Pix)/(1.0*(Pix2-Pix1))
        FF = 0.5*F*(Pix-Pix1)
        DO Chan=1,5
          DO Tx=1,3
            NewCorr(Chan,Pix,Tx)=NINT(amb_h_NewCorr(Chan,P1,Tx)*F
     &           + amb_h_NewCorr(Chan,P2,Tx)*(1.0-F)
     &           + (Grad(Chan,P1,Tx)-Grad(Chan,P2,Tx))*FF)
          ENDDO
        ENDDO
      ENDDO  

      RETURN
      END
!##############################################################################
