!+
!ROUTINE    AMB_GETSTX1
!
!Name: $Id: amb_getstx1.F 25 2006-02-03 15:43:53Z frna $
!
!PURPOSE    Read March 99 STX-1 corrections from file.
!           Interpolate to 90 pixels and store in common area.
!
!VERSION    1.0      23/04/99    N C Atkinson, UKMO
!           1.1      08/02/01    unit no for STX-1 corrections changed to 99
!           1.2      09/03/04    Modify for Linux (N Atkinson / A Korpela)
!
!ARGUMENTS  None
!##############################################################################

      SUBROUTINE AMB_GETSTX1

#include "inparms.h"
#include "amb1b.h"
!  Defines CountsCorr and NewCorr 
#include "inbstore.h"

      INTEGER*2 STX1Corr(5,90),STX1Table(5,21),STX1Threshold(2)
      COMMON/STX1Com/ STX1Corr,STX1Table,STX1Threshold

      INTEGER Tx,Chan,Pix,Pix1,Pix2,P1,P2,I,Dummy(6),errStatus
      REAL F,FF
      REAL Grad(5,19)    !First derivative of correction curve
      CHARACTER*80 charact

      OPEN(99,form='formatted',access='sequential',status='old',
     &  IOSTAT=errStatus)

!     If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
      IF ( errStatus.NE.0 )
     &  OPEN(99,form='formatted',access='sequential',status='unknown',
     &  IOSTAT=errStatus)
!
! Read the correction values for each channel and pixel 
!
      read(99,'(A80)',err=901) charact !Transmitter string (ignore)
      DO Pix=1,21
        read(99,*,err=901) (Dummy(I),I=1,6)
        DO I=1,5
          STX1Table(I,Pix) = Dummy(I+1)
        ENDDO
      ENDDO
      read(99,*,err=901) STX1Threshold(1),STX1Threshold(2)
      write(6,*) STX1Threshold(1),STX1Threshold(2)
      CLOSE(99)
!
! Interpolate STX-1 extra bias at 5 pixel intervals, 19 points
!
! Calculate Gradients
!
      DO Chan=1,5
        DO Pix=2,18
          Grad(Chan,Pix) = 0.1*(STX1Table(Chan,Pix+1) -
     &                          STX1Table(Chan,Pix-1))
        ENDDO
        Grad(Chan,1) = 2*Grad(Chan,2) - Grad(Chan,3)
        Grad(Chan,19) = 2*Grad(Chan,18) - Grad(Chan,17)
      ENDDO
!
! Interpolate
!
      DO Pix=1,90
        P1 = Pix/5 + 1      ! 1 to 19
        P2 = Pix/5 + 2      ! 2 to 20
        Pix1 = (P1-1)*5     !0 to 90
        Pix2 = (P2-1)*5     !5 to 95
        IF (Pix1 .EQ. 0) Pix1=1              !First point is pixel 1
        IF (P2 .GT. 19) P2 = 19
        F = (Pix2-Pix)/(1.0*(Pix2-Pix1))
        FF = 0.5*F*(Pix-Pix1)
        Tx = 1                  !STX-1
        DO Chan=1,5
          STX1Corr(Chan,Pix) = NINT(STX1Table(Chan,P1)*F 
     &           + STX1Table(Chan,P2)*(1.0-F)
     &           + (Grad(Chan,P1)-Grad(Chan,P2))*FF)
        ENDDO
      ENDDO  
      RETURN

  901 write(6,*) 'Error reading STX-1 correction file'
      STX1Threshold(1)=0
      STX1Threshold(2)=0
      return
      end

!##############################################################################
