!+ Set up 1C AMSU-A NEDT information from 1B scanline within ATOVIN
!
      SUBROUTINE INAGET_NEDT ( scan )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inaget_nedt.F 459 2015-09-11 14:55:32Z frna $
!
! Purpose:
!   Get calibration counts and coefs from level 1B and store them.
!
! Current Code Owner: N.C.Atkinson
!
! History:
! Version  Date      Comment
!
!  1.0  10/09/2015  Initial. Nigel Atkinson
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2015, EUMETSAT, All Rights Reserved.
!
! Declarations:
!
! Global variables:
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  AMSU-A Data Variables for processing within ATOVIN 
#include "inastore.h"
!  AMSU-A level 1B header/data common blocks 
#include "ama1b.h"
!
! Arguments:
      INTEGER       Scan
!
! Local scalars:
      INTEGER       chn           ! loop variable over all channels
      INTEGER       chn3          ! = chn*3
!
!- End of header

      DO chn=1, 2
        Acoldcals(1,chn,Scan) = ama2_coldcal(chn+2)
        Awarmcals(1,chn,Scan) = ama2_warmcal(chn+2)
        Acoldcals(2,chn,Scan) = ama2_coldcal(chn+4)
        Awarmcals(2,chn,Scan) = ama2_warmcal(chn+4)
      ENDDO
      DO chn=3, Anchan
        Acoldcals(1,chn,Scan) = ama1_coldcal(chn+2)
        Awarmcals(1,chn,Scan) = ama1_warmcal(chn+2)
        Acoldcals(2,chn,Scan) = ama1_coldcal(chn+15)
        Awarmcals(2,chn,Scan) = ama1_warmcal(chn+15)
      ENDDO
      DO chn=1, Anchan
        chn3 = chn*3
        Acalibs(1,chn,Scan) = ama_pmcal(chn3-2) * 1.E-19
        Acalibs(2,chn,Scan) = ama_pmcal(chn3-1) * 1.E-13
        Acalibs(3,chn,Scan) = ama_pmcal(chn3)   * 1.E-09
      ENDDO

      RETURN
      END
