!+ Initialise moon calculation for AMSU-A
!
      SUBROUTINE INAMOONINIT
      IMPLICIT NONE
!
! Name: $Id: inamooninit.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!  Calculate position of moon at start and end of an AMSU-A orbit. 
!  Also calculate space view elevation using space view position information 
!  in 1B header. Results are stored in common area defined in 'inastore.h'.
!
!  Current code owner: N.C.Atkinson
!
!  History:
!
!  Version  Date       Comment
!  1.0    06/06/01  Original code.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
#include "inparms.h"
#include "inastore.h"
#include "ama1b.h"
!
! Local variables:
      INTEGER Instr, I          !Array index
      INTEGER spview(2)         !Space view for AMSU-A1 and AMSU-A2
      REAL*4  dec               !Declination
      REAL*4  amsual            !Altitude of space view
!
! Function and subroutine calls:
      EXTERNAL MOON_POSITION
!
!- End of header

!Calculate start and end times, in days since 12h 1 Jan 2000 (t=0)
!Note: ama_h_startdatajd is day number starting 00h 1 Jan 1950 (day=1)
!50 years = 18262 days.

      tdays(1) = ama_h_startdatajd - 18262.5
     &    + ama_h_startdatatime/(24*3600000.0)
      tdays(2) = ama_h_enddatajd - 18262.5 
     &    + ama_h_enddatatime/(24*3600000.0)

!Calculate moon position at start and end of orbit.
!Calculates geocentric right ascension and declination

      CALL MOON_POSITION(tdays(1), ra(1), dec)
      sindec(1) = sin(dec)                        !Stored via inastore.h
      cosdec(1) = cos(dec)
      CALL MOON_POSITION(tdays(2), ra(2), dec)
      sindec(2) = sin(dec)                        !Stored via inastore.h
      cosdec(2) = cos(dec)
      IF ((ra(1)-ra(2)).gt.3.14159) THEN     !Allow for wrap-around in RA
        ra(2)=ra(2)+6.283185
      ENDIF

!Space view altitude. Use space view information in 1B header, for A1 and A2.
!Apply a parallax correction to allow for the fact that the moon position
!calculated in the previous step is geocentric rather than topocentric.

      spview(1) = MOD(ama1_h_inststat1/8192,4)   !Space view (0..3)
      spview(2) = MOD(ama2_h_inststat1/8192,4)
      DO Instr=1,2
        IF (spview(Instr).eq.0) THEN
          amsual = (-6.667+1.04)*3.1415927/180   !Add 1.04 degrees to allow for 
        ELSE IF (spview(Instr).eq.1) THEN        ! moon parallax
          amsual = (-8.333+1.04)*3.1415927/180  
        ELSE IF (spview(Instr).eq.2) THEN
          amsual = (-10.000+1.04)*3.1415927/180  
        ELSE IF (spview(Instr).eq.3) THEN
          amsual = (-13.333+1.04)*3.1415927/180  
        ENDIF
        sinamsual(Instr) = sin(amsual)    !Stored via inastore.h
        cosamsual(Instr) = cos(amsual)
      ENDDO

      DO I=1,mxlin_moon
        Moonflag(I)=-1
      ENDDO
      RETURN
      END  
