!+ Calculate angle between moon and AMSU-A space view for each scan line
!
      SUBROUTINE INAMOONTEST
      IMPLICIT NONE
!
! Name: $Id: inamoontest.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!  Convert moon coordinates to local azimuth and altitude and compare with
!  space view coordinates. Set flag if angular difference is less than a
!  threshold value of 3.5 degrees.
!
!  Current code owner: N.C.Atkinson
!
!  History:
!
!  Version  Date       Comment
!  1.0    06/06/01  Original code.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
#include "inparms.h"
#include "inastore.h"
#include "ama1b.h"
!
! Local scalars:
      REAL    lat(2)            !Latitude of samples 15 and 16
      REAL    lon(2)            !Longitude of samples 15 and 16
      REAL    azcorr            !Azimuth correction (radians)
      REAL    hour              !Time of day in hours
      REAL    f1,f2             !Used for interpolation
      REAL*8  scandays          !Time of scan in days from 12Z, 1 Jan 2000
!
! Function and subroutine calls:
      EXTERNAL MOON_AMSU        !This subroutine does the geometry
!
!- End of header

!Select latitude and longitude in centre of swath, samples 15 and 16
      lat(1) = ama_pos(15*2-1)*1E-4
      lat(2) = ama_pos(16*2-1)*1E-4
      lon(1) = ama_pos(15*2)*1E-4
      lon(2) = ama_pos(16*2)*1E-4
      IF ((lat(1).EQ.lat(2)).AND.(lon(1).EQ.lon(2))) THEN
        angle_diff(1)=180.0   !Cannot use navigation
        angle_diff(2)=180.0
      ELSE

!Correct for finite time delay between samples. For AMSU-A the delay is
!0.202516 sec, and the cross-track sampling distance is approximately 
!equivalent to 8 seconds in along-track time. So the azimuth correction is
!atan(0.202516/8)

        azcorr = 0.02531    !radians

!Time of day in hours and days

        hour = ama_scnlintime/3600000.0
        scandays = hour/24.0
        scandays = scandays + NINT(tdays(1)) - 0.5
        IF (scandays.LT.tdays(1)) THEN
          scandays = tdays(1)         !This should not happen
        ELSE IF (scandays.GT.tdays(2)) THEN
          scandays = tdays(2)
        ENDIF
        f1 = (tdays(2)-scandays)/(tdays(2)-tdays(1))  !Start/end weighting
        f2 = 1.0-f1
      
!Calculate angular separation between space view and moon, for A1 and A2.
!Linearly interpolate the moon parameters.

        CALL moon_amsu(lat,lon, azcorr, scandays, hour, 
     &   f1*ra(1)+f2*ra(2), f1*sindec(1)+f2*sindec(2), 
     &   f1*cosdec(1)+f2*cosdec(2), 2, sinamsual, cosamsual, angle_diff)
      ENDIF

!See if angular separation exceeds threshold, and set flag

      MFlag = 0
      IF (angle_diff(1) .LE. 3.5) THEN    !Threshold 3.5 degrees for AMSU-A1
        MFlag = MFlag+1
      ENDIF
      IF (angle_diff(2) .LE. 3.5) THEN    !Threshold 3.5 degrees for AMSU-A2
        MFlag = MFlag+2
      ENDIF

      RETURN
      END  
