!+ Set up 1C HIRS NEDT information from 1B scanline within ATOVIN
!
      SUBROUTINE INHGET_NEDT ( scan )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inhget_nedt.F 467 2015-11-17 10:27:43Z frna $
!
! Purpose:
!   Get calibration information from level 1B and store the resulting NEDTs.
!
! Current Code Owner: N.C.Atkinson
!
! History:
! Version  Date      Comment
!
!  1.0  15/09/2015  Initial. Nigel Atkinson
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2015, EUMETSAT, All Rights Reserved.
!
! Declarations:
!
! Global variables:
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  HIRS Data Variables for processing within ATOVIN 
#include "inhstore.h"
!  HIRS level 1B header/data common blocks 
#include "hrs1b.h"
!
! Arguments:
      INTEGER       Scan
!
! Local scalars:
      INTEGER       chn           ! loop variable over all channels
      INTEGER       i
      INTEGER       chan_index(Hnchan)  !Position of each channel in 1B
      DATA chan_index
     &   /1,3,4,6,16,15,10,11,20,13,8,18,5,14,17,19,2,7,9,12/
      INTEGER coldcal_number
      DATA coldcal_number/0/
      INTEGER warmcal_number
      DATA warmcal_number/0/
      REAL*8 radsum, radsumsq, radsdev
      INTEGER num_ok
      REAL tref
      DATA tref/280.0/    !Reference temp - also in hrs1c_to_hdf5_write.c
      REAL e,rad,x
      REAL a0(Hnchan-1),a1(Hnchan-1),a2(Hnchan-1)
      DATA a0 /19*0.0/
      DATA a1 /19*0.0/
      DATA a2 /19*0.0/
      INTEGER scntyp_warm
      DATA scntyp_warm/2/   !hrs_scntyp value for warm cal position
!
!- End of header

      IF (scan .EQ. 1) THEN
        DO chn=1, Hnchan-1
          e = exp(HPlanck(2,chn)/tref)
          Hdbdt(chn) = 
     &      HPlanck(1,chn)/(e - 1)**2 * e * HPlanck(2,chn)/tref**2
          HcoldNEDT_rms(chn) = 0.0
          HwarmNEDT_rms(chn) = 0.0
        ENDDO
        DO i=1,mxlin_nedt_hrs
          Hcoldline(i) = 0
          Hwarmline(i) = 0
        ENDDO
      ENDIF

      IF (hrs_scntyp .EQ. 1) THEN   !space view
        coldcal_number = coldcal_number + 1
        Hcoldline(coldcal_number) = scan
        DO chn=1, Hnchan-1
          a0(chn) = hrs_calcof(3,chan_index(chn))*1.0E-6
          a1(chn) = hrs_calcof(2,chan_index(chn))*1.0E-9
          a2(chn) = hrs_calcof(1,chan_index(chn))*1.0E-12
          radsum = 0.0
          radsumsq = 0.0
          num_ok = 0
          DO i=9,56     !omit the first 8
            x = hrs_elem(chan_index(chn)+2,i-1)
            IF (x .NE. 0) THEN
              IF ( hrs_h_siteid .EQ. "NSS" ) x = x - 4096  !NESDIS convention
              num_ok = num_ok + 1
              rad = a0(chn) + x*(a1(chn) + x*a2(chn))
              radsum = radsum + rad
              radsumsq = radsumsq + rad*rad
            ENDIF
          ENDDO
          radsdev = sqrt((radsumsq-radsum*radsum/num_ok)/(num_ok-1))
          HcoldNEDT(chn,coldcal_number) = radsdev/Hdbdt(chn)
          HcoldNEDT_rms(chn) = HcoldNEDT_rms(chn) + 
     &         HcoldNEDT(chn,coldcal_number)**2
        ENDDO
      ENDIF

!If we find a scan type of 3 then we must have a HIRS/2 instrument

      IF (hrs_scntyp .EQ. 3 .AND. scntyp_warm .EQ. 2) THEN
        scntyp_warm = 3
        warmcal_number = 0   !Reset warm cal counter
      ENDIF
      
      IF (hrs_scntyp .EQ. scntyp_warm .AND. scan .NE. 1) THEN   !warm view
        warmcal_number = warmcal_number + 1
        Hwarmline(warmcal_number) = scan
        DO chn=1, Hnchan-1     !use the cold cal a0, a1, a2
          radsum = 0.0
          radsumsq = 0.0
          num_ok = 0
          DO i=1,56
            x = hrs_elem(chan_index(chn)+2,i-1)
            IF (x .NE. 0) THEN
              IF ( hrs_h_siteid .EQ. "NSS" ) x = x - 4096  !NESDIS convention
              num_ok = num_ok + 1
              rad = a0(chn) + x*(a1(chn) + x*a2(chn))
              radsum = radsum + rad
              radsumsq = radsumsq + rad*rad
            ENDIF
          ENDDO
          radsdev = sqrt((radsumsq-radsum*radsum/num_ok)/(num_ok-1))
          HwarmNEDT(chn,warmcal_number) = radsdev/Hdbdt(chn)
          HwarmNEDT_rms(chn) = HwarmNEDT_rms(chn) + 
     &         HwarmNEDT(chn,coldcal_number)**2
        ENDDO
      ENDIF

      IF (scan .EQ. hrs_h_scnlin) THEN
        DO chn=1, Hnchan-1
          IF (coldcal_number .GT. 0)
     &      HcoldNEDT_rms(chn) = sqrt(HcoldNEDT_rms(chn)/coldcal_number)
          IF (warmcal_number .GT. 0)
     &      HwarmNEDT_rms(chn) = sqrt(HwarmNEDT_rms(chn)/warmcal_number)
        ENDDO
      ENDIF

      RETURN
      END
