!+ Set up 1C MSU scanline from 1B scanline within ATOVIN
!
      SUBROUTINE INMGET
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inmget.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Place information from common blocks /msu1bdt/ & /inmstore/
!   into common block /msu1cdt/.
!
! Method:
! 0.  Check format version numbers  (first call only)
!     Check that format of 1B & 1C include files (msu1b.h & msu1c.h) are
!     compatible with the code in this subroutine.
!     Call ErrorReport to print a warning if there is a problem.
! 1.  Extract required information from 1B into 1C
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0     19/3/96  Original code. (R.J.Renshaw)
!  1.1    12/12/96  Calibration flagword deleted
!  1.2    20/05/98  checks on 1B version number commented out
!  1.3    18/06/98  bug fixed in converting azimuth angle to [0,360]
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  MSU Data Variables for processing within ATOVIN 
#include "inmstore.h"
!  MSU level 1B header/data common blocks 
#include "msu1b.h"
!  MSU level 1C header/data common blocks 
#include "msu1c.h"
!
! Local parameters:
      INTEGER       M1Bvnnb       ! version number of 1B format compatible
         PARAMETER (M1Bvnnb=1)    ! with the code in this subroutine
      INTEGER       M1Cvnnb       ! version number of 1C format compatible
         PARAMETER (M1Cvnnb=180)  ! with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       fov           ! loop variable over all fovs
      INTEGER       i             ! general purpose loop variable
      INTEGER       angle         ! satellite azimuth angle (deg*100)
      INTEGER       year          ! scan line year
      INTEGER       day           ! scan line day of year
      INTEGER       time          ! scan line UTC time of day in milliseconds
      INTEGER       clockdrift    ! satellite clock drift delta in milliseconds
      LOGICAL first               ! .TRUE. on first call, then .FALSE.
         DATA first / .TRUE. /
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
      External TimeAdd              ! add an increment to a date/time
!
!- End of header

      SAVE first                    ! .TRUE. on first call

! 0.  Check format version numbers  (first call only)
!     ----- ------ ------- -------

      IF ( first ) THEN

!!        Is the 1B include file compatible with this source code ?
!         IF ( M1Bvnnb .NE. msu_1bvnnb ) THEN
!            errStatus = -11  ! warning
!            errMessage = " "
!            WRITE ( errMessage,
!     &      "("" Code expects 1B format"",I6,"" but msu1b.h is "",I6)")
!     &           M1Bvnnb, msu_1bvnnb
!            CALL ErrorReport ( errStatus, errMessage, "INMGET")
!         ENDIF

!        Is the 1C include file compatible with this source code ?
         IF ( M1Cvnnb .NE. msu_1cvnnb ) THEN
            errStatus = -12  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &      "("" Code expects 1C format"",I6,"" but msu1c.h is "",I6)")
     &           M1Cvnnb, msu_1cvnnb
            CALL ErrorReport ( errStatus, errMessage, "INMGET")
         ENDIF

         first = .FALSE.

      ENDIF  ! ( first )


! 1.  Extract required information from 1B into 1C
!     ------- -------- ----------- ---- -- ---- --

! 1.1 scanline number
      msu1c_scnlin = msu_scnlin

! 1.2 scanline date/time

      msu1c_scnlinyr = msu_scnlinyr        ! scan line year
      msu1c_scnlindy = msu_scnlindy        ! scan line day of year
      msu1c_scnlintime = msu_scnlintime    ! UTC time of day in milliseconds

      year = msu1c_scnlinyr                ! scan line year
      day  = msu1c_scnlindy                ! scan line day of year
      time = msu1c_scnlintime              ! UTC time of day in milliseconds

!      - adding clock drift delta if not already done so
      IF ( .NOT.BTEST(INT(msu_scnlinbit),bits(14)) ) THEN !add clock drift delta
         clockdrift = msu_clockdrift       ! (msecs)    (msu_... is I2)
         CALL TimeAdd ( year, day, time, clockdrift,
     &        msu1c_scnlinyr, msu1c_scnlindy, msu1c_scnlintime)
      ENDIF

! 1.3 other scanline information
      msu1c_mjfrcnt = msu_mjfrcnt          !major frame count

!     Split scanline flags into two, for ATOVS compatibility
      msu1c_qualind = 0                    !quality indicator bit field
      DO i = 24, 31
         IF (BTEST(msu_qualind,bits(i)))
     &             msu1c_qualind = IBSET(msu1c_qualind,bits(i))
      ENDDO  ! i
      msu1c_scanqual = 0                   !scan line quality flags
      DO i = 0, 23
         IF (BTEST(msu_qualind,bits(i)))
     &             msu1c_scanqual = IBSET(msu1c_scanqual,bits(i))
      ENDDO  ! i

!     Quality control word for each channel
      DO chn = 1, Mnchan
         msu1c_chanqual(chn) = msu_scnlinqual(chn)
      ENDDO  ! chn

      DO i = 1, 3
         msu1c_spare1(i) = 0               !spare
      ENDDO  ! i

! 1.4 Navigation

      msu1c_scalti = msu_scalti  ! sat altitude above ref. ellipsoid (km*10)

      DO fov = 1, Mnfovs
         msu1c_latlon(1,fov) = msu_pos(1,fov)        ! latitude
         msu1c_latlon(2,fov) = msu_pos(2,fov)        ! longitude
      ENDDO  ! fov

!     angles
      DO fov = 1, Mnfovs
         msu1c_angles(1,fov) = msu_ang(2,fov)        ! zenith
         angle = msu_ang(3,fov)        ! azimuth
!        make sure azimuth between 0-360, rather than +/-180
         IF ( angle .LT. 0 ) angle = 36000 + angle
         msu1c_angles(2,fov) = angle
      ENDDO  ! fov

! 1.5 Initialise variables new in 1C
      msu1c_spare2 = 0                     !spare
      DO chn = 1, Mnchan
      DO fov = 1, Mnfovs
         msu1c_btemps(chn,fov) = MDI       !brightness temperatures
      ENDDO  ! fov
      ENDDO  ! chn
      DO fov = 1, Mnfovs
         msu1c_dataqual(fov) = 0           !qc word for each field of view
      ENDDO  ! fov


! 2.  Extract other information required here
!     ------- ----- ----------- -------- ----

! 2.1 Calibration coefficients (primary & secondary)
      DO chn = 1, Mnchan
         Mcalib(1,1,chn) = REAL(msu_pmcal(1,chn)) * 1.E-12  ! primary
         Mcalib(1,2,chn) = REAL(msu_pmcal(2,chn)) * 1.E-12
         Mcalib(1,3,chn) = REAL(msu_pmcal(3,chn)) * 1.E-08
         Mcalib(2,1,chn) = REAL(msu_sdcal(1,chn)) * 1.E-12  ! secondary
         Mcalib(2,2,chn) = REAL(msu_sdcal(2,chn)) * 1.E-12
         Mcalib(2,3,chn) = REAL(msu_sdcal(3,chn)) * 1.E-08
      ENDDO  ! chn

! 2.2 Counts from the instrument
      DO fov = 1, Mnfovs
      DO chn = 1, Mnchan
         Mcounts(chn,fov) = msu_data(chn+8*fov-5)
      ENDDO  ! chn
      ENDDO  ! fov

      DO i = 1, 34
         msu1c_filler(i) = 0      !filler
      ENDDO


      RETURN
      END
