!+ Set up 1C MHS NEDT information from 1B scanline within ATOVIN
!
      SUBROUTINE INMHSGET_NEDT ( scan )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inmhsget_nedt.F 459 2015-09-11 14:55:32Z frna $
!
! Purpose:
!   Get calibration counts and coefs from level 1B and store them.
!
! Current Code Owner: N.C.Atkinson
!
! History:
! Version  Date      Comment
!
!  1.0  10/09/2015  Initial. Nigel Atkinson
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2015, EUMETSAT, All Rights Reserved.
!
! Declarations:
!
! Global variables:
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  AMSU-A Data Variables for processing within ATOVIN 
#include "inbstore.h"
!  MHS level 1B header/data common blocks 
#include "mhs1b.h"
!
! Arguments:
      INTEGER       Scan
!
! Local scalars:
      INTEGER       chn           ! loop variable over all channels
      INTEGER       chn3          ! = chn*3
      INTEGER       i,j
!
!- End of header

      DO chn=1, Bnchan
        DO i=1,4
          j = chn+i*6-5
          IF (mhs_coldcal(j).GE.0) THEN
            Bcoldcals(i,chn,Scan) = mhs_coldcal(j)
          ELSE
            Bcoldcals(i,chn,Scan) = mhs_coldcal(j)+65536
          ENDIF
          IF (mhs_warmcal(j).GE.0) THEN
            Bwarmcals(i,chn,Scan) = mhs_warmcal(j)
          ELSE
            Bwarmcals(i,chn,Scan) = mhs_warmcal(j)+65536
          ENDIF
        ENDDO

        chn3 = chn*3
        Bcalibs(1,chn,Scan) = mhs_pmcal(chn3-2) * 1.E-16
        Bcalibs(2,chn,Scan) = mhs_pmcal(chn3-1) * 1.E-10
        Bcalibs(3,chn,Scan) = mhs_pmcal(chn3)   * 1.E-06
      ENDDO

      RETURN
      END
