!+ set up variables for ATOVIN
!
      SUBROUTINE INSETU ( LHIRS, LAMSA, LAMSB, LMSU,
     &                    H1Bunit, H1Cunit,
     &                    A1Bunit, A1Cunit,
     &                    B1Bunit, B1Cunit,
     &                    M1Bunit, M1Cunit,
     &                    FDFunit )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: insetu.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Perform all set-up required for program ATOVIN.
!
! Method:
! 1.  Read fixed data files
! 2.  Define bit numbers
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      2/2/96  Original code. (R.J.Renshaw)
!  1.1     19/3/96  Process ATOVS and TOVS.  RJR
!  1.2      5/3/03  Call INFDF with satid zero.   Keith Whyte
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
! Subroutine arguments
!   Scalar arguments with intent(in):
      LOGICAL       LHIRS         ! .TRUE. to process HIRS
      LOGICAL       LAMSA         ! .TRUE. to process AMSU-A
      LOGICAL       LAMSB         ! .TRUE. to process AMSU-B
      LOGICAL       LMSU          ! .TRUE. to process MSU
      INTEGER       H1Bunit       ! unit number for HIRS level 1B file
      INTEGER       H1Cunit       ! unit number for HIRS level 1C file
      INTEGER       A1Bunit       ! unit number for AMSU-A level 1B file
      INTEGER       A1Cunit       ! unit number for AMSU-A level 1C file
      INTEGER       B1Bunit       ! unit number for AMSU-B level 1B file
      INTEGER       B1Cunit       ! unit number for AMSU-B level 1C file
      INTEGER       M1Bunit       ! unit number for MSU level 1B file
      INTEGER       M1Cunit       ! unit number for MSU level 1C file
      INTEGER       FDFunit       ! unit number for fixed data file
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status (0 is OK, +ve is fatal)
      INTEGER       i, j          ! used in setting bits
!
! Function & Subroutine calls:
      External INFDF              ! Read fixed data file
      External ErrorReport        ! ATOVIN/ATOVPP error routine
!
!- End of header


! 1.  Read fixed data file
!     ---- ----- ---- ----

      CALL INFDF (FDFunit, 0)     ! Don't read antenna efficiencies until later


! 2.  Define bit numbers   (in "bits.h")
!     ------ --- -------

! Convention used in 1B & 1C files is that an INTEGER*4 word has bits
! numbered 0-31, with bit 0 being the least significant bit.
! Some platforms take bit 31 as the LSB. Here we explicitly define
! the order of bits that we use, to keep the code portable.

      i = 1

      IF ( BTEST(i,0) ) THEN      !  bit 0 is LSB

         DO j = 0, 31
            bits(j)  =  j
         ENDDO  ! j

      ELSEIF( BTEST(i,31) ) THEN      !  bit 31 is LSB

         DO j = 0, 31
            bits(j)  =  31 - j
         ENDDO  ! j

      ELSE                            !  something wrong

         errStatus = 31
         errMessage =
     & " Bit problem: BTEST(1,0) and BTEST(1,31) are both FALSE"
         CALL ErrorReport ( errStatus, errMessage, "INSETU")

      ENDIF


      RETURN
      END
