!+ I/O routine for level 1B MHS data
!
      SUBROUTINE IOMHS1B ( COMMAND, B1Bunit, B1Brecord )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: iomhs1b.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   interface to files containing level 1B MHS data
!
! Method:
!   Performs one of several functions, according to the value of COMMAND :
!
!   "OPEN/READ"          : open file for readonly
!   "OPEN/WRITE"         : open file for write
!   "CLOSE"              : close file
!   "READ HEADER"        : read header into common
!   "READ DATA"          : read data record into common
!   "WRITE HEADER"       : write header from common
!   "WRITE DATA"         : write data from common
!
!   Note: COMMAND must be typed exactly, and in upper case.
!   Routine ends by calling "ErrorReport" to check on error status.
!   If "COMMAND" is not recognised, "ErrorReport"
!   will be called with fatal error status.
!
! Current Code Owner: N C Atkinson
!
! History:
! Version  Date       Comment
!
!  1.0     July 2003  Created from iob1b.F v1.1           N C Atkinson
!  1.1      02/04/04  Modify for Linux
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  MHS level 1B header/data common blocks 
#include "mhs1b.h"
!  Record length dividor 
#include "reclopt.h"
!
!   Scalar arguments with intent(In):
      CHARACTER*(*) COMMAND       ! instruction on what function to perform
      INTEGER       B1Bunit       ! unit number
      INTEGER       B1Brecord     ! record number
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status: 0 OK, +ve for fatal error
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
!- End of header


! Act on value of COMMAND
! -----------------------

! 1.  OPEN/READ
!     ---------
      IF ( COMMAND .EQ. "OPEN/READ" ) THEN

         OPEN ( UNIT=B1Bunit, STATUS="OLD", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=reclmhs/rldiv, IOSTAT=errStatus )

!        If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=B1Bunit, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=reclmhs/rldiv, IOSTAT=errStatus )

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" OPEN/READ 1B MHS file: UNIT="",I3,"" RECL="",I6)")
     &                                B1Bunit, reclmhs/rldiv


! 2.  OPEN/WRITE
!     ----------
      ELSEIF ( COMMAND .EQ. "OPEN/WRITE" ) THEN

!        First try with STATUS="NEW", i.e. file doesn"t yet exist
         OPEN ( UNIT=B1Bunit, STATUS="NEW", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=reclmhs/rldiv, IOSTAT=errStatus )

!        If that didn't work, use STATUS="OLD"
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=B1Bunit, STATUS="OLD", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=reclmhs/rldiv, IOSTAT=errStatus )
! N.B. This seems to be needed on HPs, where "unknown" is treated as "scratch"

!        If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=B1Bunit, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=reclmhs/rldiv, IOSTAT=errStatus )

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" OPEN/WRITE 1B MHS file: UNIT="",I3,"" RECL="",I6)")
     &                                 B1Bunit, reclmhs/rldiv


! 3.  READ HEADER
!     ---- ------
      ELSEIF ( COMMAND .EQ. "READ HEADER" ) THEN

!        read first header record
         READ ( UNIT=B1Bunit,REC=B1Brecord,IOSTAT=errStatus ) mhs_hd
         CALL byteswap1b("MHS1BHD")

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &"("" READing 1B MHS header: UNIT="",I3,"" REC ="",I6)")
     &                             B1Bunit, B1Brecord


! 4.  WRITE HEADER
!     ----- ------
      ELSEIF ( COMMAND .EQ. "WRITE HEADER" ) THEN

         WRITE ( UNIT=B1Bunit,REC=B1Brecord,IOSTAT=errStatus ) mhs_hd

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &"("" WRITEing 1B MHS header: UNIT="",I3,"" REC="",I6)")
     &                              B1Bunit, B1Brecord


! 5.  READ DATA
!     ---- ----
      ELSEIF ( COMMAND .EQ. "READ DATA" ) THEN

         READ ( UNIT=B1Bunit,REC=B1Brecord,IOSTAT=errStatus ) mhs_dt
         CALL byteswap1b("MHS1BDT")

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" READing MHS 1B data, UNIT="",I3,"" REC="",I6)")
     &                             B1Bunit, B1Brecord


! 6.  WRITE DATA
!     ----- ----
      ELSEIF ( COMMAND .EQ. "WRITE DATA" ) THEN

         WRITE ( UNIT=B1Bunit,REC=B1Brecord,IOSTAT=errStatus ) mhs_dt

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" WRITEing MHS 1B data, UNIT="",I3,"" REC="",I6)")
     &                              B1Bunit, B1Brecord


! 7.  CLOSE
!     -----
      ELSEIF ( COMMAND .EQ. "CLOSE" ) THEN

         CLOSE ( UNIT=B1Bunit, IOSTAT=errStatus )

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" CLOSEing MHS 1B file, UNIT="",I3)") B1Bunit


! 8.  COMMAND not recognized
!     ------- --- ----------
      ELSE

         errStatus = 999
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Command not recognised: "",A50)") COMMAND


      ENDIF   ! (COMMAND)


! 8.  Check error status
!     ----- ----- ------

      CALL ErrorReport ( errStatus, errMessage, "IOMHS1B")


      RETURN
      END
