      SUBROUTINE average_T89_T166_atms(atmslastline, width, 
     &     avdiff_89_166)
      
! Name: $Id: average_T89_T166_atms.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose: Compute average 89GHz & 166GHz diff over sea over a defined area,
! for use with scattering index calculation
!
! Code Owner : NWP SAF
!
! Version 1.0  08/09/2010  From average_T89_T150.F
!
! Declarations:

#include "ppparms.h"
#include "ppatmsstore.h"

! Arguments
      INTEGER atmslastline, width
      REAL avdiff_89_166(atmsnfovs, atmsnlines)

! Local variables
      INTEGER ifov,iline,jfov,jline,fov1,fov2,line1,line2,n,w2
      REAL T89,T166
!
!- End of header
      
      w2 = width/2

      DO iline = 1,atmslastline
        DO ifov = 1,atmsnfovs
          avdiff_89_166(ifov,iline) = 0.0
          IF (atmsLandFracb(ifov,iline).NE.100) THEN
            n = 0
            fov1 = max(ifov-w2,1)
            fov2 = min(ifov+w2,atmsnfovs)
            line1 = max(iline-w2,1)
            line2 = min(iline+w2,atmslastline)
            DO jline = line1,line2
              DO jfov = fov1,fov2
                T89 = atmsbtemps(16,jfov,jline) 
                T166 = atmsbtemps(17,jfov,jline)
                IF (T89.GE.atmsminBT.AND.T166.GE.atmsminBT.AND.
     &              T89.LE.atmsmaxBT.AND.T166.LE.atmsmaxBT.AND.
     &              atmsLandFracb(jfov,jline).EQ.0) THEN
                  avdiff_89_166(ifov,iline) = avdiff_89_166(ifov,iline)
     &                     +  T89 - T166 - 0.11*atmsangles(1,jfov,jline)
                               !Allow for zenith angle (see retrieve_one_si.F)
                  n = n + 1
                ENDIF
              ENDDO
            ENDDO
            IF (n .GE. width) THEN   !Must have >=sqrt(no of points considered)
              avdiff_89_166(ifov,iline) = avdiff_89_166(ifov,iline)/n
            ELSE
              avdiff_89_166(ifov,iline) = -99999.9     !Insufficient sea points
            ENDIF
          ENDIF     !atmsLandFracb(ifov,iline).NE.100
        ENDDO     !ifov
      ENDDO     !iline
      END
