!+ Apply scan-dependent bias corrections within ATOVPP
!
      SUBROUTINE BIASCORR ( Biases, CHAN, Nchan, Nfovs, Nlines, Btemps )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: biascorr.F 5 2006-01-04 09:11:15Z frna $
!
! Purpose:
!   Add scan-dependent bias corrections to level 1c brightness temps
!
! Method:
!   Values are added to the brightness temperatures,
!   but only where the BTs are not set to missing (i.e. within 4-400 Kelvin)
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!   Scalar arguments with intent(In):
      INTEGER       CHAN          ! number of channels in the arrays
      INTEGER       Nchan         ! number to process (starting at 1)
      INTEGER       Nfovs         ! number of fovs in the arrays
      INTEGER       Nlines        ! number of scanlines to process
!
!   Array arguments with intent(In):
      REAL          Biases(Nchan,Nfovs) !scan-dependent bias corrections to add
!
!   Array arguments with intent(InOut):
      REAL          Btemps(CHAN,Nfovs,Nlines) !brightness temperatures (K)
!
! Local scalars:
      INTEGER       line          ! loop index: scanline
      INTEGER       fov           ! loop index: fov
      INTEGER       chn           ! loop index: channel
      REAL          bt            ! brightness temperature
!
!- End of header


! 1.  Add scan-dependent bias corrections   ( - only where data is valid )
!     --- -------------- ---- -----------

!     loop through scanlines, spots and channels
      DO line = 1, Nlines
      DO fov  = 1, Nfovs
      DO chn  = 1, Nchan
         bt = Btemps(chn,fov,line)
         IF ( bt.GT.4. .AND. bt.LT.400. ) THEN
            Btemps(chn,fov,line) = bt + Biases(chn,fov)
         ENDIF
      ENDDO  ! chn
      ENDDO  ! fov
      ENDDO  ! line


      RETURN
      END
