      SUBROUTINE IASI_GENERATE_PCS
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: iasi_generate_pcs.F 636 2019-04-16 11:12:39Z frna $
!
! Purpose: Generate PC scores from 1C IASI
!      
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2009, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0   04/03/2009  Original code. (N.C.Atkinson)
!  1.1   04/09/2009  Add iasipc_h_evfile and iasipc_h_evfilevn
!  1.2   09/09/2010  Correct I4 -> I6 in iasipc_h_evfilevn
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!
!  general parameters for ATOVPP 
#include "ppparms.h"
!  level 1c format
#include "iasi1c.h"
!  level 1c PC format
#include "iasipc.h"
!  fixed data for processing IASI
#include "ppifdata.h"

! Local variables:

      INTEGER idet,ipc,ifov,iband,chn,I,J,K,L,n,counts
      REAL Recon     
      REAL scaling(Inchan)
      DATA scaling / Inchan*0.0 /
      REAL Radiance(Inchan)      !Radiance (mW/m^2/str/cm^-1)
      REAL NormRadiance(Inchan)  !Noise normalised radiance
      REAL RRadiance(Inchan)     !Reconstructed noise normalised radiance
      REAL Score                 !PC score (temporary)
      REAL Scores(iasi_pcnpc)    !PC scores
      REAL q                     !Quantization
      LOGICAL badcounts,allzero
!
!= End of header ==========================
      
! 1. Compute counts to radiance scaling factors first time round
!---------------------------------------------------------------
! Radiance is in mW/(m^2 str cm^-1). See ppispectra.F

      IF (scaling(1) .EQ. 0.0) THEN
        I = 1
        DO chn=1,Inchan
          DO WHILE (iasi1c_h_endchan(I).LT.chn)
            I = I + 1
            IF (i.GT.10) THEN
              WRITE(*,*) "Error computing scaling factors in " //
     &           "iasi_generate_pcs.F"
              STOP
            ENDIF
          ENDDO
          scaling(chn) = 10.0**(-iasi1c_h_scalefactor(I)*1.0)*100000.0
        ENDDO

! 2. Fill the 1C PC header first time round
!------------------------------------------

        iasipc_h_site = iasi1c_h_site
        iasipc_h_1bsite = iasi1c_h_1bsite
        iasipc_h_versnb = iasi_pcvnnb
        iasipc_h_versyr = iasi_pcvnyr
        iasipc_h_versdy = iasi_pcvndy
        iasipc_h_hdrcnt = 1
        iasipc_h_satid = iasi1c_h_satid
        iasipc_h_instrument = 14               !14 = IASI PC format
        iasipc_h_satht = iasi1c_h_satht
        iasipc_h_period = iasi1c_h_period
        iasipc_h_startorbit = iasi1c_h_startorbit
        iasipc_h_startdatayr = iasi1c_h_startdatayr
        iasipc_h_startdatady = iasi1c_h_startdatady
        iasipc_h_startdatatime = iasi1c_h_startdatatime
        iasipc_h_endorbit = iasi1c_h_endorbit
        iasipc_h_enddatayr = iasi1c_h_enddatayr
        iasipc_h_enddatady = iasi1c_h_enddatady
        iasipc_h_enddatatime = iasi1c_h_enddatatime
        iasipc_h_scnlin = iasi1c_h_scnlin
        iasipc_h_misscnlin = iasi1c_h_misscnlin
        iasipc_h_wmosatid  = iasi1c_h_wmosatid
        DO I=1,10
          iasipc_h_startchan(I) = iasi1c_h_startchan(I)
          iasipc_h_endchan(I) = iasi1c_h_endchan(I)
          iasipc_h_scalefactor(I) = iasi1c_h_scalefactor(I)
        ENDDO
        iasipc_h_recl = iasi_pcrecl
        iasipc_h_nchan = iasi_pcnchan
        iasipc_h_nrchan = iasi_pcnrecon
        iasipc_h_npc = iasi_pcnpc
        iasipc_h_nchanused = MIN(nchan_IASI,iasi_pcnchan)
        iasipc_h_nrchanused = 0            !No reconstructed radiances yet
        iasipc_h_npcused = MAX(endpc(1),MAX(endpc(2),endpc(3)))
        iasipc_h_npcused = MIN(iasipc_h_npcused,iasi_pcnpc)        
        iasipc_h_npcbands = npcbands
        DO I=1,3
          iasipc_h_startpcchan(I) = startpcchan(I)
          iasipc_h_endpcchan(I) = endpcchan(I)
          iasipc_h_startpc(I) = startpc(I)
          iasipc_h_endpc(I) = endpc(I)
          iasipc_h_quantization(I) = 50      !Set quantization step to 0.5
          iasipc_h_evfile(I) = evfile(I)
          iasipc_h_evfilevn(I) = MDI
            !The "production date" field in file name includes a version number
          read(evfile(I),'(56X,I6)',err=100) iasipc_h_evfilevn(I)
  100     continue
        ENDDO
        iasipc_h_chanvn = chanvn
        iasipc_h_rchanvn = 0
        iasipc_h_eigvn = eigvn
        DO I=1,iasipc_h_nchanused
          iasipc_h_channels(I) = channels_IASI(I)
        ENDDO
        IF (iasi_pcnchan.GT.nchan_IASI) THEN
          DO I=iasi_pcnchan+1,nchan_IASI
            if (I .le. iasi_pcnchan) then
              iasipc_h_channels(I) = 0       !Spare channel slots set to zero
            endif
          ENDDO
        ENDIF  
        DO I=1,iasi_pcnrecon               !No reconstructed channels yet
          iasipc_h_rchannels(I) = 0
        ENDDO
        
      ENDIF   !First time round
        
! 3. Fill the parts of the data record that are same as 1C
!---------------------------------------------------------

      iasipc_scnlin = iasi1c_scnlin
      iasipc_granule = iasi1c_granule
      iasipc_scnlinyr = iasi1c_scnlinyr
      iasipc_scnlindy = iasi1c_scnlindy
      DO I=1,iasi_pcnfov
        iasipc_scnlintime(I) = iasi1c_scnlintime(I)
        DO J=1,iasi_pcndet
          iasipc_GQisFlagQual(J,I) = iasi1c_GQisFlagQual(J,I)
        ENDDO
      ENDDO
      iasipc_GQisQualIndex = iasi1c_GQisQualIndex
      iasipc_GQisQualIndexLoc = iasi1c_GQisQualIndexLoc
      iasipc_GQisQualIndexRad = iasi1c_GQisQualIndexRad
      iasipc_GQisQualIndexSpect = iasi1c_GQisQualIndexSpect
      iasipc_GQisSysTecSondQual = iasi1c_GQisSysTecSondQual

      DO I=1,iasi_pcnfov
        DO J=1,iasi_pcndet
          DO K=1,2
            iasipc_latlon(K,J,I) = iasi1c_latlon(K,J,I)
          ENDDO
          DO K=1,4
            iasipc_angles(K,J,I) = iasi1c_angles(K,J,I)
          ENDDO
        ENDDO
      ENDDO
      iasipc_scalti = iasi1c_scalti
      iasipc_avhcombination = iasi1c_avhcombination
        
      DO I=1,iasi_pcnfov
        DO J=1,iasi_pcndet
          DO K=1,7         !Clusters
            DO L=1,6       !Channels
              iasipc_avhrrmean(L,K,J,I) = iasi1c_avhrrmean(L,K,J,I)
              iasipc_avhmeanscalefactor(L,K,J,I) = 
     &            iasi1c_avhmeanscalefactor(L,K,J,I)
              iasipc_avhrrsdev(L,K,J,I)  = iasi1c_avhrrsdev(L,K,J,I)
              iasipc_avhsdevscalefactor(L,K,J,I) = 
     &            iasi1c_avhsdevscalefactor(L,K,J,I)
              iasipc_avhrrfrac(K,J,I) = iasi1c_avhrrfrac(K,J,I)
              iasipc_avhrrCGy(K,J,I) = iasi1c_avhrrCGy(K,J,I)
              iasipc_avhrrCGz(K,J,I) = iasi1c_avhrrCGz(K,J,I)
            ENDDO
          ENDDO
          iasipc_GEUMAvhrr1BCldFrac(J,I) =
     &          iasi1c_GEUMAvhrr1BCldFrac(J,I)
          iasipc_GEUMAvhrr1BLandFrac(J,I) =
     &          iasi1c_GEUMAvhrr1BLandFrac(J,I)
          iasipc_GEUMAvhrr1BQual(J,I) = iasi1c_GEUMAvhrr1BQual(J,I)
        ENDDO
        iasipc_GIacVarImagIIS(I) = iasi1c_GIacVarImagIIS(I)
        iasipc_GIacAvgImagIIS(I) = iasi1c_GIacAvgImagIIS(I)
      ENDDO

! 4. Do the channel selection
!----------------------------

      DO I=1,iasi_pcnfov
        DO J=1,iasi_pcndet
          DO K=1,iasipc_h_nchanused
            iasipc_scalrad(K,J,I) = 
     &          iasi1c_scalrad(iasipc_h_channels(K),J,I)
          ENDDO
        ENDDO
      ENDDO

! 5. Compute PC scores for all spots in the scan
!-----------------------------------------------

      DO idet=1,iasi_pcndet
        IF (OKdetector(idet).EQ.1) THEN
          DO ifov=1,iasi_pcnfov

            badcounts = .false.
            allzero = .true.
            DO chn=1,Inchan
              counts = iasi1c_scalrad(chn,idet,ifov)
              IF (counts .EQ.-32768) THEN
                badcounts = .true.
              ELSE
                Radiance(chn) = counts*scaling(chn)
              ENDIF
              IF (counts .ne. 0) allzero=.false.
            ENDDO
            IF (.not.badcounts.and..not.allzero) THEN  

!De-apodise if this has been specified
              IF (evlevel.EQ.'1B') THEN
                CALL deapodise_iasi(Radiance, mtf_IASI, 1)
              ENDIF

!Noise normalise
              DO chn=1,Inchan
                NormRadiance(chn) = Radiance(chn)/nedr_IASI(chn)
                RRadiance(chn) = 0.0
              ENDDO

!Subtract mean (if eigformat=1)
              IF (eigformat.EQ.1) THEN
                DO chn=1,Inchan
                  NormRadiance(chn) = 
     &               NormRadiance(chn)-mean_encode_IASI(chn)
                ENDDO
              ENDIF

!Compute PC scores and residuals
              DO iband=1,npcbands
                q = iasipc_h_quantization(iband)*1.0E-2
                DO Ipc=startpc(iband),endpc(iband)
                  Score = 0.0
                  DO chn=startpcchan(iband),endpcchan(iband)
                    Score = Score + 
     &                 eig_encode_IASI(chn,Ipc)*NormRadiance(chn)
                  ENDDO
                  iasipc_scores(Ipc,idet,ifov) = NINT(Score/q)  !Quantise

                  DO chn=startpcchan(iband),endpcchan(iband)
                    RRadiance(chn) = RRadiance(chn) + 
     &                 eig_encode_IASI(chn,Ipc)*Score
                  ENDDO
                ENDDO
              ENDDO

!Compute reconstruction score for each band (using the ppifdata.h bands)
              DO iband=1,3
                Score = 0.0
                DO chn=Firstchan(iband),Lastchan(iband)
                  Score = Score + 
     &               (RRadiance(chn) - NormRadiance(chn))**2
                ENDDO
                n = Lastchan(iband) - Firstchan(iband) + 1
                Score = sqrt(Score/n)
                iasipc_logfit(iband,idet,ifov) = NINT(LOG10(Score)*1E4)
              ENDDO

            ELSE
              DO Ipc=1,iasipc_h_npcused 
                iasipc_scores(Ipc,idet,ifov) = -32768    !Set missing
              ENDDO
              DO iband=1,3
                iasipc_logfit(iband,idet,ifov) = MDI
              ENDDO
            ENDIF   !badcounts
          ENDDO   !ifov
        ENDIF   !OKdetector
      ENDDO   !idet
      
      RETURN
      END
            
