      SUBROUTINE IASI_GENERATE_RRAD
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: iasi_generate_rrad.F 182 2010-01-12 15:50:02Z frna $
!
! Purpose: Apply PC scores to generate reconstructed radiances
!      
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2009, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0   04/03/2009  Original code. (N.C.Atkinson)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!
!  general parameters for ATOVPP 
#include "ppparms.h"
!  level 1c PC format
#include "iasipc.h"
!  level 1d format
#include "iasi1d.h"
!  fixed data for processing IASI
#include "ppifdata.h"

! Local variables:

      INTEGER idet,ipc,ifov,ichn,iband,chn,I
      REAL Recon     
      REAL scaling(iasi_pcnrecon)
      DATA scaling / iasi_pcnrecon*0.0 /
      REAL q
      INTEGER default_startchan(10), default_endchan(10), 
     &        default_scalefactor(10)
      DATA default_startchan/1,3341,6429,6961,8141,0,0,0,0,0/
      DATA default_endchan/3340,6428,6960,8140,8461,0,0,0,0,0/
      DATA default_scalefactor/7,8,9,8,9,0,0,0,0,0/
!
!= End of header ==========================
      
! 1. Compute counts to radiance scaling factors first time round
!---------------------------------------------------------------
! Radiance is in mW/(m^2 str cm^-1). See ppispectra.F.
! Only need scaling for required channels.
! The scaling factors are not held in the BUFR, so put in defaults if needed.

      IF (scaling(1) .EQ. 0.0) THEN
        IF (iasipc_h_startchan(1) .le. 0) THEN
          DO i=1,10
            iasipc_h_startchan(i) = default_startchan(i)
            iasipc_h_endchan(i) = default_endchan(i)
            iasipc_h_scalefactor(i) = default_scalefactor(i)
          ENDDO
        ENDIF
        I = 1
        DO chn=1,iasipc_h_nrchanused
          ichn = channels_IASI(chn)
          DO WHILE (iasipc_h_endchan(I).LT.ichn)
            I = I + 1
            IF (i.GT.10) THEN
              WRITE(*,*) "Error computing scaling factors in " //
     &           "iasi_generate_rrad.F"
              STOP
            ENDIF
          ENDDO
          scaling(chn) = 10.0**(-iasipc_h_scalefactor(I)*1.0)*100000.0
        ENDDO
      ENDIF
        

! 2. Compute reconstructed radiances for all spots in the scan
!-------------------------------------------------------------
! These are apodised (1C) and not noise normalised.
! The scores are quantised.

      DO idet=1,4
        IF (OKdetector(idet).EQ.1) THEN
          DO ifov=1,iasi_pcnfov
            IF (iasipc_scores(1,idet,ifov).NE.-32768) THEN
              DO chn=1,iasipc_h_nrchanused
                ichn = channels_IASI(chn)

!Add mean (if eigformat=1)
                IF (eigformat.EQ.1) THEN
                  Recon = mean_decode_IASI(ichn)
                ELSE
                  Recon = 0.0
                ENDIF

!Multiply eigenvectors by scores
                DO iband=1,npcbands
                  q = iasipc_h_quantization(iband)*1.0E-2
                  IF (channels_IASI(chn).GE.startpcchan(iband)
     &              .AND. channels_IASI(chn).LE.endpcchan(iband)) THEN
                    DO ipc=startpc(iband),endpc(iband)
                      Recon = Recon + eig_decode_IASI(ichn,ipc)
     &                 *q*iasipc_scores(ipc,idet,ifov)
                    ENDDO
                    iasipc_reconrad(chn,idet,ifov)=
     &                          NINT(Recon/scaling(chn))
                  ENDIF
                ENDDO
              ENDDO
            ELSE
              DO chn=1,iasipc_h_nchanused
                iasipc_reconrad(chn,idet,ifov) = -32768
              ENDDO
            ENDIF
          ENDDO
        ENDIF
      ENDDO
      
      RETURN
      END
            
