      SUBROUTINE map_mwhs2_to_iras(use_median,lati,loni,timeh,
     &                    satheight,latm,lonm,timem,qualindm,btm,btmap)
      IMPLICIT NONE

c-----------------------------------------
! Name: $Id: map_mwhs2_to_iras.F 396 2014-06-11 12:19:50Z frna $
!
#include "ppparms.h"
#include "constants_preproc.h"
#include "mwhs21c.h"
#include "iras1c.h"

! Parameters
      INTEGER   MWHS_SPOTS, MWHS_CHANS, MWHS_SCANS
      PARAMETER (MWHS_SPOTS=98)
      PARAMETER (MWHS_CHANS=15)
      PARAMETER (MWHS_SCANS=2700)
      INTEGER   IRAS_SPOTS, IRAS_SCANS
      PARAMETER (IRAS_SPOTS=56)
      PARAMETER (IRAS_SCANS=1150)
      REAL      ANGLE_TOLERANCE
      PARAMETER (ANGLE_TOLERANCE = 0.8) !degrees

! Arguments
      LOGICAL use_median   !true if median filter to be used, otherwise nearest
      REAL latm(MWHS_SPOTS,MWHS_SCANS)       !deg
      REAL lonm(MWHS_SPOTS,MWHS_SCANS)
      INTEGER timem(MWHS_SCANS)              !milliseconds
      REAL  satheight(MWHS_SCANS)            !km

      REAL lati(IRAS_SPOTS,IRAS_SCANS)       !deg *1E4
      REAL loni(IRAS_SPOTS,IRAS_SCANS)
      INTEGER timeh(IRAS_SCANS)              !milliseconds
      INTEGER qualindm(IRAS_SCANS)
      REAL btm(MWHS_CHANS,MWHS_SPOTS,MWHS_SCANS)  !K

      REAL btmap(MWHS_CHANS,IRAS_SPOTS,IRAS_SCANS)  !K

! Variables
      REAL mx(MWHS_SPOTS,MWHS_SCANS)
      REAL my(MWHS_SPOTS,MWHS_SCANS)
      REAL mz(MWHS_SPOTS,MWHS_SCANS)
      REAL ix(IRAS_SPOTS,IRAS_SCANS)
      REAL iy(IRAS_SPOTS,IRAS_SCANS)
      REAL iz(IRAS_SPOTS,IRAS_SCANS)
      INTEGER mscan, mspot, iscan, ispot, mchan, tdiff, tdiff_old, i
      INTEGER num_t(MWHS_CHANS)   !Number of MWHS spots contributing
      INTEGER refscan(IRAS_SCANS) !reference MWHS scan for each IRAS scan
      INTEGER refspot(IRAS_SPOTS) !reference MWHS spot for each IRAS spot
      INTEGER mspotmin, mscanmin
      REAL    hfactor
      REAL    xsat,ysat,zsat
      REAL    ivec(3),tvec(3),itprod(3)
      REAL    tolerance_radsq, ivecsq, tvecsq, sinsq, itprodsq
      REAL    btm_use(100,MWHS_CHANS), btmedian, btmmp
      REAL    sinsq_min
        
!- End of header

      WRITE(*,*)'         Mapping MWHS2 to IRAS'

      tolerance_radsq = (ANGLE_TOLERANCE*deg2rad)**2

!Note MWHS2 and IRAS scan right to left.
      DO ispot=1,IRAS_SPOTS
        refspot(ispot) = 3 + ispot*90/55.0
      ENDDO

!For each IRAS scan find the nearest MWHS scan, according to the time

      DO iscan=1,iras1c_h_scnlin
        tdiff_old = 86400000
        refscan(iscan) = -9999
        DO mscan=1,mwhs21c_h_scnlin
          tdiff = abs(timeh(iscan)-timem(mscan))
          IF (tdiff .GT. 43200000) tdiff = tdiff - 86400000
          IF (abs(tdiff) .lt. tdiff_old) THEN
            refscan(iscan) = mscan
            tdiff_old = abs(tdiff)
          ENDIF
        ENDDO
!        write(*,*) iscan,refscan(iscan),tdiff_old
      ENDDO

!Convert all the lat/lons to x/y/z

      CALL latlon_to_xyz (IRAS_SPOTS,iras1c_h_scnlin,lati,loni,
     &     ix,iy,iz)
      CALL latlon_to_xyz (MWHS_SPOTS,mwhs21c_h_scnlin,latm,lonm,
     &     mx,my,mz)

!Loop over IRAS scans

      DO iscan=1,iras1c_h_scnlin

  !Find the satellite position (using nadir lat/lon)

        hfactor = (rEarth + satheight(iscan))/rEarth
        xsat = ix(28,iscan) * hfactor
        ysat = iy(28,iscan) * hfactor
        zsat = iz(28,iscan) * hfactor
!        write(*,*) 'sat: ',xsat,ysat,zsat

        DO ispot=1,56
          ivec(1) = ix(ispot,iscan) - xsat
          ivec(2) = iy(ispot,iscan) - ysat
          ivec(3) = iz(ispot,iscan) - zsat
          ivecsq = ivec(1)**2 + ivec(2)**2 + ivec(3)**2
          DO mchan=1,MWHS_CHANS
            num_t(mchan) = 0
          ENDDO
          
  !Loop over a range of MWHS scans and spots, and calculate the angle between
  !MWHS and IRAS
  
          sinsq_min = 100.0
          mspotmin = 0
          DO mscan=refscan(iscan)-3, refscan(iscan)+1
            IF (mscan.GT.0 .AND. mscan.LE.mwhs21c_h_scnlin) THEN
              IF (qualindm(mscan) .GE. 0) THEN
                DO mspot=refspot(ispot)-2, refspot(ispot)+2
                 IF (mspot.GT.0 .AND. mspot.LE.98) THEN
                  tvec(1) = mx(mspot,mscan) - xsat
                  tvec(2) = my(mspot,mscan) - ysat
                  tvec(3) = mz(mspot,mscan) - zsat
                  tvecsq = tvec(1)**2 + tvec(2)**2 + tvec(3)**2
                  CALL crossprod(tvec,ivec,itprod)
                  itprodsq = itprod(1)**2 + itprod(2)**2 + itprod(3)**2
                  sinsq =  itprodsq/(tvecsq * ivecsq)
                  IF (sinsq .LT. tolerance_radsq) THEN    !include this spot
!                    write(*,*)'match: ',iscan,ispot,
!     &                 mscan-refscan(iscan),mspot-refspot(ispot),
!     &                 sinsq/tolerance_radsq,
!     &                 lati(ispot,iscan),loni(ispot,iscan),
!     &                 latm(mspot,mscan),lonm(mspot,mscan)
                    IF (sinsq .LT. sinsq_min) THEN
                      sinsq_min = sinsq   !New nearest neighbour
                      mspotmin = mspot
                      mscanmin = mscan
                    ENDIF
                    IF (use_median) THEN
                      DO mchan=1,MWHS_CHANS
                        btmmp = btm(mchan,mspot,mscan)
                        IF (btmmp.GE.AminBT.AND.btmmp.LE.AmaxBT) THEN
                          num_t(mchan) = num_t(mchan) + 1
                          btm_use(num_t(mchan),mchan) = 
     &                       btm(mchan,mspot,mscan)
                        ENDIF
                      ENDDO
                    ENDIF
!                  ELSE
!                    write(*,*)'no match: ',iscan,ispot,
!     &                 mscan-refscan(iscan),mspot-refspot(ispot),
!     &                 sinsq/tolerance_radsq,
!     &                 lati(ispot,iscan),loni(ispot,iscan),
!     &                 latm(mspot,mscan),lonm(mspot,mscan)
                  ENDIF
                 ENDIF
                ENDDO
              ENDIF
            ENDIF
          ENDDO
          
  !Find the median MWHS BT (use of median avoids problems with outliers)
  !  or use the nearest neighbour BT
  
          DO mchan=1,MWHS_CHANS
            IF (use_median) THEN
              IF (num_t(mchan).GT.0) THEN
                CALL amedian(btm_use(1,mchan),num_t(mchan),btmedian)
                btmap(mchan,ispot,iscan) = btmedian
              ELSE
                btmap(mchan,ispot,iscan) = RMDI
              ENDIF
            ELSE
              IF (mspotmin.GT.0) THEN
                btmap(mchan,ispot,iscan) = btm(mchan,mspotmin,mscanmin)
              ELSE
                btmap(mchan,ispot,iscan) = RMDI
              ENDIF
            ENDIF
          ENDDO
          
!          write(*,*) iscan,ispot,num_t(1)

        ENDDO   !ispot
      ENDDO   !iscan

      END         
