      SUBROUTINE map_mwhs2_to_mwts2(use_median,latt,lont,timet,
     &                    satheight,lath,lonh,timeh,qualindh,bth,btmap)
      IMPLICIT NONE

c-----------------------------------------
! Name: $Id$
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    11/06/2014  
!  1.1    09/08/2018  Add FY-3D
!
c-----------------------------------------
#include "ppparms.h"
#include "constants_preproc.h"
#include "mwts21c.h"
#include "mwhs21c.h"

! Parameters
      INTEGER   MWTS_SPOTS, MWTS_CHANS, MWTS_SCANS
      PARAMETER (MWTS_SPOTS=90)
      PARAMETER (MWTS_CHANS=13)
      PARAMETER (MWTS_SCANS=2700)
      INTEGER   MWHS_SPOTS, MWHS_CHANS, MWHS_SCANS
      PARAMETER (MWHS_SPOTS=98)
      PARAMETER (MWHS_CHANS=15)
      PARAMETER (MWHS_SCANS=2700)
      REAL      ANGLE_TOLERANCE
!      PARAMETER (ANGLE_TOLERANCE = 0.8) !degrees
      PARAMETER (ANGLE_TOLERANCE = 1.25) !degrees

! Arguments
      LOGICAL use_median   !true if median filter to be used, otherwise nearest
      REAL latt(MWTS_SPOTS,MWTS_SCANS)       !deg
      REAL lont(MWTS_SPOTS,MWTS_SCANS)
      INTEGER timet(MWTS_SCANS)              !milliseconds
      REAL  satheight(MWTS_SCANS)            !km

      REAL lath(MWHS_SPOTS,MWHS_SCANS)       !deg *1E4
      REAL lonh(MWHS_SPOTS,MWHS_SCANS)
      INTEGER timeh(MWHS_SCANS)              !milliseconds
      INTEGER qualindh(MWHS_SCANS)
      REAL bth(MWHS_CHANS,MWHS_SPOTS,MWHS_SCANS)  !K

      REAL btmap(MWHS_CHANS,MWTS_SPOTS,MWTS_SCANS)  !K

! Variables
      REAL tx(MWTS_SPOTS,MWTS_SCANS)
      REAL ty(MWTS_SPOTS,MWTS_SCANS)
      REAL tz(MWTS_SPOTS,MWTS_SCANS)
      REAL hx(MWHS_SPOTS,MWHS_SCANS)
      REAL hy(MWHS_SPOTS,MWHS_SCANS)
      REAL hz(MWHS_SPOTS,MWHS_SCANS)
      INTEGER tscan, tspot, hscan, hspot, hchan, tdiff, tdiff_old, i
      INTEGER num_h(MWHS_CHANS)   !Number of MWHS spots contributing
      INTEGER refscan(MWTS_SCANS) !reference MWHS scan for each MWTS scan
      INTEGER refspot(MWTS_SPOTS) !reference MWHS spot for each MWTS spot
      INTEGER hspotmin, hscanmin
      REAL    hfactor
      REAL    xsat,ysat,zsat
      REAL    hvec(3),tvec(3),htprod(3)
      REAL    tolerance_radsq, hvecsq, tvecsq, sinsq, htprodsq
      REAL    bth_use(100,MWHS_CHANS), btmedian, bttmp
      REAL    sinsq_min
        
!- End of header

      WRITE(*,*)'         Mapping MWHS to MWTS'

      tolerance_radsq = (ANGLE_TOLERANCE*deg2rad)**2

!Note MWHS scans right to left, MWTS scans left to right for FY-3C and right to
!left for FY-3D
      if (mwts21c_h_wmosatid .eq. 522) then   !FY-3C
        DO tspot=1,MWTS_SPOTS
          refspot(tspot) = 95 - tspot
        ENDDO
      else if (mwts21c_h_wmosatid .eq. 523) then   !FY-3D
        DO tspot=1,MWTS_SPOTS
          refspot(tspot) = tspot + 3
        ENDDO
      else
        write(*,*)'Unknown satid: ',mwts21c_h_wmosatid
        stop
      endif

!For each MWTS scan find the nearest MWHS scan, according to the time

      DO tscan=1,mwts21c_h_scnlin
        tdiff_old = 86400000
        refscan(tscan) = -9999
        DO hscan=1,mwhs21c_h_scnlin
          tdiff = abs(timet(tscan)-timeh(hscan))
          IF (tdiff .GT. 43200000) tdiff = tdiff - 86400000
          IF (tdiff .LT. -43200000) tdiff = tdiff + 86400000
          IF (abs(tdiff) .lt. tdiff_old) THEN
            refscan(tscan) = hscan
            tdiff_old = abs(tdiff)
          ENDIF
        ENDDO
      ENDDO

!Convert all the lat/lons to x/y/z

      CALL latlon_to_xyz (MWTS_SPOTS,mwts21c_h_scnlin,latt,lont,
     &     tx,ty,tz)
      CALL latlon_to_xyz (MWHS_SPOTS,mwhs21c_h_scnlin,lath,lonh,
     &     hx,hy,hz)

!Loop over MWTS scans

      DO tscan=1,mwts21c_h_scnlin

  !Find the satellite position (using nadir lat/lon)

        hfactor = (rEarth + satheight(tscan))/rEarth
        xsat = tx(45,tscan) * hfactor
        ysat = ty(45,tscan) * hfactor
        zsat = tz(45,tscan) * hfactor
!        write(*,*) 'sat: ',xsat,ysat,zsat

        DO tspot=1,90
          tvec(1) = tx(tspot,tscan) - xsat
          tvec(2) = ty(tspot,tscan) - ysat
          tvec(3) = tz(tspot,tscan) - zsat
          tvecsq = tvec(1)**2 + tvec(2)**2 + tvec(3)**2
          DO hchan=1,MWHS_CHANS
            num_h(hchan) = 0
          ENDDO
          
  !Loop over a range of MWHS scans and spots, and calculate the angle between
  !MWTS and MWHS
  
          sinsq_min = 100.0
          hspotmin = -1
          DO hscan=refscan(tscan)-3, refscan(tscan)+3
            IF (hscan.GT.0 .AND. hscan.LE.mwhs21c_h_scnlin) THEN
              IF (qualindh(hscan) .GE. 0) THEN
                DO hspot=refspot(tspot)-5, refspot(tspot)+5
                  hvec(1) = hx(hspot,hscan) - xsat
                  hvec(2) = hy(hspot,hscan) - ysat
                  hvec(3) = hz(hspot,hscan) - zsat
                  hvecsq = hvec(1)**2 + hvec(2)**2 + hvec(3)**2
                  CALL crossprod(hvec,tvec,htprod)
                  htprodsq = htprod(1)**2 + htprod(2)**2 + htprod(3)**2
                  sinsq =  htprodsq/(hvecsq * tvecsq)
                  IF (sinsq .LT. tolerance_radsq) THEN    !include this spot
!                    write(*,*)'match: ',tscan,tspot,
!     &                 hscan-refscan(tscan),hspot-refspot(tspot)  !!!!!!!
                    IF (sinsq .LT. sinsq_min) THEN
                      sinsq_min = sinsq   !New nearest neighbour
                      hspotmin = hspot
                      hscanmin = hscan
                    ENDIF
                    IF (use_median) THEN
                      DO hchan=1,MWHS_CHANS
                        bttmp = bth(hchan,hspot,hscan)
                        IF (bttmp.GE.BminBT.AND.bttmp.LE.BmaxBT) THEN
                          num_h(hchan) = num_h(hchan) + 1
                          bth_use(num_h(hchan),hchan) = 
     &                       bth(hchan,hspot,hscan)
                        ENDIF
                      ENDDO
                    ENDIF
                  ENDIF
                ENDDO
              ENDIF
            ENDIF
          ENDDO
          
  !Find the median MWHS BT (use of median avoids problems with outliers)
  !  or use the nearest neighbour BT
  
          DO hchan=1,MWHS_CHANS
            IF (use_median) THEN
              IF (num_h(hchan).GT.0) THEN
                CALL amedian(bth_use(1,hchan),num_h(hchan),btmedian)
                btmap(hchan,tspot,tscan) = btmedian
              ELSE
                btmap(hchan,tspot,tscan) = RMDI
              ENDIF
            ELSE
              if (hspotmin .gt. 0) then
                btmap(hchan,tspot,tscan) = bth(hchan,hspotmin,hscanmin)
              else
                btmap(hchan,tspot,tscan) = RMDI
              ENDIF
            ENDIF
          ENDDO
          
!          write(*,*) tscan,tspot,num_h(1)

        ENDDO   !tspot
      ENDDO   !tscan

      END         
