      SUBROUTINE map_mwhs_to_mwts(latt,lont,timet,satheight,
     &                            lath,lonh,timeh,qualindh,bth,btmap)
      IMPLICIT NONE

c-----------------------------------------
! Name: $Id: map_mwhs_to_mwts.F 313 2012-02-28 14:51:43Z frna $
!
#include "ppparms.h"
#include "constants_preproc.h"
#include "mwts1c.h"
#include "mwhs1c.h"

! Parameters
      INTEGER   MWTS_SPOTS, MWTS_CHANS, MWTS_SCANS
      PARAMETER (MWTS_SPOTS=15)
      PARAMETER (MWTS_CHANS=4)
      PARAMETER (MWTS_SCANS=500)
      INTEGER   MWHS_SPOTS, MWHS_CHANS, MWHS_SCANS
      PARAMETER (MWHS_SPOTS=98)
      PARAMETER (MWHS_CHANS=5)
      PARAMETER (MWHS_SCANS=2700)
      REAL      ANGLE_TOLERANCE
      PARAMETER (ANGLE_TOLERANCE = 2.0) !degrees

! Arguments
      REAL latt(MWTS_SPOTS,MWTS_SCANS)       !deg
      REAL lont(MWTS_SPOTS,MWTS_SCANS)
      INTEGER timet(MWTS_SCANS)              !milliseconds
      REAL  satheight(MWTS_SCANS)            !km

      REAL lath(MWHS_SPOTS,MWHS_SCANS)       !deg *1E4
      REAL lonh(MWHS_SPOTS,MWHS_SCANS)
      INTEGER timeh(MWHS_SCANS)              !milliseconds
      INTEGER qualindh(MWHS_SCANS)
      REAL bth(MWHS_CHANS,MWHS_SPOTS,MWHS_SCANS)  !K

      REAL btmap(MWHS_CHANS,MWTS_SPOTS,MWTS_SCANS)  !K

! Variables
      REAL tx(MWTS_SPOTS,MWTS_SCANS)
      REAL ty(MWTS_SPOTS,MWTS_SCANS)
      REAL tz(MWTS_SPOTS,MWTS_SCANS)
      REAL hx(MWHS_SPOTS,MWHS_SCANS)
      REAL hy(MWHS_SPOTS,MWHS_SCANS)
      REAL hz(MWHS_SPOTS,MWHS_SCANS)
      INTEGER tscan, tspot, hscan, hspot, hchan, tdiff, tdiff_old, i
      INTEGER num_h(MWHS_CHANS)   !Number of MWHS spots contributing
      INTEGER refscan(MWTS_SCANS) !reference MWHS scan for each MWTS scan
      INTEGER refspot(MWTS_SPOTS) !reference MWHS spot for each MWTS spot
      REAL    hfactor
      REAL    xsat,ysat,zsat
      REAL    hvec(3),tvec(3),htprod(3)
      REAL    tolerance_radsq, hvecsq, tvecsq, sinsq, htprodsq
      REAL    bth_use(100,MWHS_CHANS), btmedian, bttmp
        
!- End of header

      WRITE(*,*)'         Mapping MWHS to MWTS'

      tolerance_radsq = (ANGLE_TOLERANCE*deg2rad)**2

!Note MWTS scans left to right, MWHS scans right to left
      DO tspot=1,MWTS_SPOTS
        refspot(tspot) = 99 - NINT((tspot-8)*6.25 + 49.5)
      ENDDO

!For each MWTS scan find the nearest MWHS scan, according to the time (but note
!that the MWTS time stamps can be out by up to 16 seconds)

      DO tscan=1,mwts1c_h_scnlin
        tdiff_old = 86400000
        refscan(tscan) = -9999
        DO hscan=1,mwhs1c_h_scnlin
          tdiff = abs(timet(tscan)-timeh(hscan))
          IF (tdiff .GT. 43200000) tdiff = tdiff - 86400000
          IF (tdiff .LT. -43200000) tdiff = tdiff + 86400000
          IF (abs(tdiff) .lt. tdiff_old) THEN
            refscan(tscan) = hscan
            tdiff_old = abs(tdiff)
          ENDIF
        ENDDO
      ENDDO

!Convert all the lat/lons to x/y/z

      CALL latlon_to_xyz (MWTS_SPOTS,mwts1c_h_scnlin,latt,lont,
     &     tx,ty,tz)
      CALL latlon_to_xyz (MWHS_SPOTS,mwhs1c_h_scnlin,lath,lonh,
     &     hx,hy,hz)

!Loop over MWTS scans

      DO tscan=1,mwts1c_h_scnlin

  !Find the satellite position (using nadir lat/lon)

        hfactor = (rEarth + satheight(tscan))/rEarth
        xsat = tx(8,tscan) * hfactor
        ysat = ty(8,tscan) * hfactor
        zsat = tz(8,tscan) * hfactor
!        write(*,*) 'sat: ',xsat,ysat,zsat

        DO tspot=1,15
          tvec(1) = tx(tspot,tscan) - xsat
          tvec(2) = ty(tspot,tscan) - ysat
          tvec(3) = tz(tspot,tscan) - zsat
          tvecsq = tvec(1)**2 + tvec(2)**2 + tvec(3)**2
          DO hchan=1,MWHS_CHANS
            num_h(hchan) = 0
          ENDDO
          
  !Loop over a range of MWHS scans and spots, and calculate the angle between
  !MWTS and MWHS
  
          DO hscan=refscan(tscan)-10, refscan(tscan)+10
            IF (hscan.GT.0 .AND. hscan.LE.mwhs1c_h_scnlin) THEN
              IF (qualindh(hscan) .GE. 0) THEN
                DO hspot=refspot(tspot)-5, refspot(tspot)+5
                  hvec(1) = hx(hspot,hscan) - xsat
                  hvec(2) = hy(hspot,hscan) - ysat
                  hvec(3) = hz(hspot,hscan) - zsat
                  hvecsq = hvec(1)**2 + hvec(2)**2 + hvec(3)**2
                  CALL crossprod(hvec,tvec,htprod)
                  htprodsq = htprod(1)**2 + htprod(2)**2 + htprod(3)**2
                  sinsq =  htprodsq/(hvecsq * tvecsq)
                  IF (sinsq .LT. tolerance_radsq) THEN    !include this spot
C                    write(*,*)'match: ',tscan,tspot,
C     &                 hscan-refscan(tscan),hspot-refspot(tspot)
                    DO hchan=1,MWHS_CHANS
                      bttmp = bth(hchan,hspot,hscan)
                      IF (bttmp .GE. BminBT .AND. bttmp .LE. BmaxBT)THEN
                        num_h(hchan) = num_h(hchan) + 1
                        bth_use(num_h(hchan),hchan) = 
     &                     bth(hchan,hspot,hscan)
                      ENDIF
                    ENDDO
                  ENDIF
                ENDDO
              ENDIF
            ENDIF
          ENDDO
          
  !Find the median MWHS BT (use of median avoids problems with outliers)
  
          DO hchan=1,MWHS_CHANS
            IF (num_h(hchan).GT.0) THEN
              CALL amedian(bth_use(1,hchan),num_h(hchan),btmedian)
              btmap(hchan,tspot,tscan) = btmedian
            ELSE
              btmap(hchan,tspot,tscan) = RMDI
            ENDIF
          ENDDO
          
!          write(*,*) tscan,tspot,num_h(1)

        ENDDO   !tspot
      ENDDO   !tscan

      END         
