      SUBROUTINE map_mwts2_to_iras(use_median,lati,loni,timeh,
     &                    satheight,latt,lont,timet,qualindt,btt,btmap)
      IMPLICIT NONE

c-----------------------------------------
! Name: $Id: map_mwts2_to_iras.F 411 2014-09-12 13:38:20Z frna $
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.1    09/09/2014  Increase ANGLE_TOLERANCE, since MWTS2 scan rate is lower
!                     than it was originally

#include "ppparms.h"
#include "constants_preproc.h"
#include "mwts21c.h"
#include "iras1c.h"

! Parameters
      INTEGER   MWTS_SPOTS, MWTS_CHANS, MWTS_SCANS
      PARAMETER (MWTS_SPOTS=90)
      PARAMETER (MWTS_CHANS=13)
      PARAMETER (MWTS_SCANS=2700)
      INTEGER   IRAS_SPOTS, IRAS_SCANS
      PARAMETER (IRAS_SPOTS=56)
      PARAMETER (IRAS_SCANS=1150)
      REAL      ANGLE_TOLERANCE
      PARAMETER (ANGLE_TOLERANCE = 1.3) !degrees

! Arguments
      LOGICAL use_median   !true if median filter to be used, otherwise nearest
      REAL latt(MWTS_SPOTS,MWTS_SCANS)       !deg
      REAL lont(MWTS_SPOTS,MWTS_SCANS)
      INTEGER timet(MWTS_SCANS)              !milliseconds
      REAL  satheight(MWTS_SCANS)            !km

      REAL lati(IRAS_SPOTS,IRAS_SCANS)       !deg *1E4
      REAL loni(IRAS_SPOTS,IRAS_SCANS)
      INTEGER timeh(IRAS_SCANS)              !milliseconds
      INTEGER qualindt(IRAS_SCANS)
      REAL btt(MWTS_CHANS,MWTS_SPOTS,MWTS_SCANS)  !K

      REAL btmap(MWTS_CHANS,IRAS_SPOTS,IRAS_SCANS)  !K

! Variables
      REAL tx(MWTS_SPOTS,MWTS_SCANS)
      REAL ty(MWTS_SPOTS,MWTS_SCANS)
      REAL tz(MWTS_SPOTS,MWTS_SCANS)
      REAL ix(IRAS_SPOTS,IRAS_SCANS)
      REAL iy(IRAS_SPOTS,IRAS_SCANS)
      REAL iz(IRAS_SPOTS,IRAS_SCANS)
      INTEGER tscan, tspot, iscan, ispot, tchan, tdiff, tdiff_old, i
      INTEGER num_t(MWTS_CHANS)   !Number of MWTS spots contributing
      INTEGER refscan(IRAS_SCANS) !reference MWTS scan for each IRAS scan
      INTEGER refspot(IRAS_SPOTS) !reference MWTS spot for each IRAS spot
      INTEGER tspotmin, tscanmin
      REAL    hfactor
      REAL    xsat,ysat,zsat
      REAL    ivec(3),tvec(3),itprod(3)
      REAL    tolerance_radsq, ivecsq, tvecsq, sinsq, itprodsq
      REAL    btt_use(100,MWTS_CHANS), btmedian, bttmp
      REAL    sinsq_min
        
!- End of header

      WRITE(*,*)'         Mapping MWTS2 to IRAS'

      tolerance_radsq = (ANGLE_TOLERANCE*deg2rad)**2

!Note MWTS2 scans left to right, IRAS right to left. Not quite centred.
      DO ispot=1,IRAS_SPOTS
        refspot(ispot) = 90 - ispot*87/53.0
      ENDDO

!For each IRAS scan find the nearest MWTS scan, according to the time

      DO iscan=1,iras1c_h_scnlin
        tdiff_old = 86400000
        refscan(iscan) = -9999
        DO tscan=1,mwts21c_h_scnlin
          tdiff = abs(timeh(iscan)-timet(tscan))
          IF (tdiff .GT. 43200000) tdiff = tdiff - 86400000
          IF (abs(tdiff) .lt. tdiff_old) THEN
            refscan(iscan) = tscan
            tdiff_old = abs(tdiff)
          ENDIF
        ENDDO
!        write(*,*) iscan,refscan(iscan),tdiff_old
      ENDDO

!Convert all the lat/lons to x/y/z

      CALL latlon_to_xyz (IRAS_SPOTS,iras1c_h_scnlin,lati,loni,
     &     ix,iy,iz)
      CALL latlon_to_xyz (MWTS_SPOTS,mwts21c_h_scnlin,latt,lont,
     &     tx,ty,tz)

!Loop over IRAS scans

      DO iscan=1,iras1c_h_scnlin

  !Find the satellite position (using nadir lat/lon)

        hfactor = (rEarth + satheight(iscan))/rEarth
        xsat = ix(28,iscan) * hfactor
        ysat = iy(28,iscan) * hfactor
        zsat = iz(28,iscan) * hfactor
!        write(*,*) 'sat: ',xsat,ysat,zsat

        DO ispot=1,56
          ivec(1) = ix(ispot,iscan) - xsat
          ivec(2) = iy(ispot,iscan) - ysat
          ivec(3) = iz(ispot,iscan) - zsat
          ivecsq = ivec(1)**2 + ivec(2)**2 + ivec(3)**2
          DO tchan=1,MWTS_CHANS
            num_t(tchan) = 0
          ENDDO
          
  !Loop over a range of MWTS scans and spots, and calculate the angle between
  !MWTS and IRAS
  
          sinsq_min = 100.0
          tspotmin = 0
          DO tscan=refscan(iscan)-2, refscan(iscan)+1   !slight asymmetry
            IF (tscan.GT.0 .AND. tscan.LE.mwts21c_h_scnlin) THEN
              IF (qualindt(tscan) .GE. 0) THEN
                DO tspot=refspot(ispot)-2, refspot(ispot)+2
                 IF (tspot.GT.0 .AND. tspot.LE.90) THEN
                  tvec(1) = tx(tspot,tscan) - xsat
                  tvec(2) = ty(tspot,tscan) - ysat
                  tvec(3) = tz(tspot,tscan) - zsat
                  tvecsq = tvec(1)**2 + tvec(2)**2 + tvec(3)**2
                  CALL crossprod(tvec,ivec,itprod)
                  itprodsq = itprod(1)**2 + itprod(2)**2 + itprod(3)**2
                  sinsq =  itprodsq/(tvecsq * ivecsq)
                  IF (sinsq .LT. tolerance_radsq) THEN    !include this spot
!                    write(*,*)'match: ',iscan,ispot,
!     &                 tscan-refscan(iscan),tspot-refspot(ispot),
!     &                 sinsq/tolerance_radsq,
!     &                 lati(ispot,iscan),loni(ispot,iscan),
!     &                 latt(tspot,tscan),lont(tspot,tscan)
                    IF (sinsq .LT. sinsq_min) THEN
                      sinsq_min = sinsq   !New nearest neighbour
                      tspotmin = tspot
                      tscanmin = tscan
                    ENDIF
                    IF (use_median) THEN
                      DO tchan=1,MWTS_CHANS
                        bttmp = btt(tchan,tspot,tscan)
                        IF (bttmp.GE.AminBT.AND.bttmp.LE.AmaxBT) THEN
                          num_t(tchan) = num_t(tchan) + 1
                          btt_use(num_t(tchan),tchan) = 
     &                       btt(tchan,tspot,tscan)
                        ENDIF
                      ENDDO
                    ENDIF
                  ELSE
!                    write(*,*)'no match: ',iscan,ispot,
!     &                 tscan-refscan(iscan),tspot-refspot(ispot),
!     &                 sinsq/tolerance_radsq,
!     &                 lati(ispot,iscan),loni(ispot,iscan),
!     &                 latt(tspot,tscan),lont(tspot,tscan)
                  ENDIF
                 ENDIF
                ENDDO
              ENDIF
            ENDIF
          ENDDO
          
  !Find the median MWTS BT (use of median avoids problems with outliers)
  !  or use the nearest neighbour BT
  
          DO tchan=1,MWTS_CHANS
            IF (use_median) THEN
              IF (num_t(tchan).GT.0) THEN
                CALL amedian(btt_use(1,tchan),num_t(tchan),btmedian)
                btmap(tchan,ispot,iscan) = btmedian
              ELSE
                btmap(tchan,ispot,iscan) = RMDI
              ENDIF
            ELSE
              IF (tspotmin.GT.0) THEN
                btmap(tchan,ispot,iscan) = btt(tchan,tspotmin,tscanmin)
              ELSE
                btmap(tchan,ispot,iscan) = RMDI
              ENDIF
            ENDIF
          ENDDO
          
!          write(*,*) iscan,ispot,num_t(1)

        ENDDO   !ispot
      ENDDO   !iscan

      END         
