      SUBROUTINE map_mwts2_to_mwhs2(use_median,lath,lonh,timeh,
     &                    satheight,latt,lont,timet,qualindt,btt,btmap)
      IMPLICIT NONE

c-----------------------------------------
! Name: $Id: map_mwts2_to_mwhs2.F 592 2018-08-09 14:36:40Z frna $
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    11/06/2014  
!  1.1    09/08/2018  Add FY-3D
!
c-----------------------------------------
#include "ppparms.h"
#include "constants_preproc.h"
#include "mwts21c.h"
#include "mwhs21c.h"

! Parameters
      INTEGER   MWTS_SPOTS, MWTS_CHANS, MWTS_SCANS
      PARAMETER (MWTS_SPOTS=90)
      PARAMETER (MWTS_CHANS=13)
      PARAMETER (MWTS_SCANS=2700)
      INTEGER   MWHS_SPOTS, MWHS_CHANS, MWHS_SCANS
      PARAMETER (MWHS_SPOTS=98)
      PARAMETER (MWHS_CHANS=15)
      PARAMETER (MWHS_SCANS=2700)
      REAL      ANGLE_TOLERANCE
!      PARAMETER (ANGLE_TOLERANCE = 0.8) !degrees
      PARAMETER (ANGLE_TOLERANCE = 1.25)

! Arguments
      LOGICAL use_median   !true if median filter to be used, otherwise nearest
      REAL latt(MWTS_SPOTS,MWTS_SCANS)       !deg
      REAL lont(MWTS_SPOTS,MWTS_SCANS)
      INTEGER timet(MWTS_SCANS)              !milliseconds
      REAL  satheight(MWTS_SCANS)            !km

      REAL lath(MWHS_SPOTS,MWHS_SCANS)       !deg *1E4
      REAL lonh(MWHS_SPOTS,MWHS_SCANS)
      INTEGER timeh(MWHS_SCANS)              !milliseconds
      INTEGER qualindt(MWHS_SCANS)
      REAL btt(MWTS_CHANS,MWTS_SPOTS,MWTS_SCANS)  !K

      REAL btmap(MWTS_CHANS,MWHS_SPOTS,MWHS_SCANS)  !K

! Variables
      REAL tx(MWTS_SPOTS,MWTS_SCANS)
      REAL ty(MWTS_SPOTS,MWTS_SCANS)
      REAL tz(MWTS_SPOTS,MWTS_SCANS)
      REAL hx(MWHS_SPOTS,MWHS_SCANS)
      REAL hy(MWHS_SPOTS,MWHS_SCANS)
      REAL hz(MWHS_SPOTS,MWHS_SCANS)
      INTEGER tscan, tspot, hscan, hspot, tchan, tdiff, tdiff_old, i
      INTEGER num_t(MWTS_CHANS)   !Number of MWTS spots contributing
      INTEGER refscan(MWHS_SCANS) !reference MWTS scan for each MWHS scan
      INTEGER refspot(MWHS_SPOTS) !reference MWTS spot for each MWHS spot
      INTEGER tspotmin, tscanmin
      REAL    hfactor
      REAL    xsat,ysat,zsat
      REAL    hvec(3),tvec(3),htprod(3)
      REAL    tolerance_radsq, hvecsq, tvecsq, sinsq, htprodsq
      REAL    btt_use(100,MWTS_CHANS), btmedian, bttmp
      REAL    sinsq_min
        
!- End of header

      WRITE(*,*)'         Mapping MWTS to MWHS'

      tolerance_radsq = (ANGLE_TOLERANCE*deg2rad)**2

!Note MWHS scans right to left, MWTS scans left to right for FY-3C and right to
!left for FY-3D
      if (mwhs21c_h_wmosatid .eq. 522) then   !FY-3C
        DO hspot=1,MWHS_SPOTS
          refspot(hspot) = 95 - hspot
        ENDDO
      else if (mwhs21c_h_wmosatid .eq. 523) then   !FY-3D
        DO hspot=1,MWHS_SPOTS
          refspot(hspot) = hspot - 3
        ENDDO
      else
        write(*,*)'Unknown satid: ',mwhs21c_h_wmosatid
        stop
      endif

!For each MWHS scan find the nearest MWTS scan, according to the time

      DO hscan=1,mwhs21c_h_scnlin
        tdiff_old = 86400000
        refscan(hscan) = -9999
        DO tscan=1,mwts21c_h_scnlin
          tdiff = abs(timeh(hscan)-timet(tscan))
          IF (tdiff .GT. 43200000) tdiff = tdiff - 86400000
          IF (abs(tdiff) .lt. tdiff_old) THEN
            refscan(hscan) = tscan
            tdiff_old = abs(tdiff)
          ENDIF
        ENDDO
      ENDDO

!Convert all the lat/lons to x/y/z

      CALL latlon_to_xyz (MWHS_SPOTS,mwhs21c_h_scnlin,lath,lonh,
     &     hx,hy,hz)
      CALL latlon_to_xyz (MWTS_SPOTS,mwts21c_h_scnlin,latt,lont,
     &     tx,ty,tz)

!Loop over MWHS scans

      DO hscan=1,mwhs21c_h_scnlin

  !Find the satellite position (using nadir lat/lon)

        hfactor = (rEarth + satheight(hscan))/rEarth
        xsat = hx(48,hscan) * hfactor
        ysat = hy(48,hscan) * hfactor
        zsat = hz(48,hscan) * hfactor
!        write(*,*) 'sat: ',xsat,ysat,zsat

        DO hspot=1,98
          hvec(1) = hx(hspot,hscan) - xsat
          hvec(2) = hy(hspot,hscan) - ysat
          hvec(3) = hz(hspot,hscan) - zsat
          hvecsq = hvec(1)**2 + hvec(2)**2 + hvec(3)**2
          DO tchan=1,MWTS_CHANS
            num_t(tchan) = 0
          ENDDO
          
  !Loop over a range of MWHS scans and spots, and calculate the angle between
  !MWTS and MWHS
  
          sinsq_min = 100.0
          tspotmin = 0
          DO tscan=refscan(hscan)-1, refscan(hscan)+1
            IF (tscan.GT.0 .AND. tscan.LE.mwts21c_h_scnlin) THEN
              IF (qualindt(tscan) .GE. 0) THEN
                DO tspot=refspot(hspot)-5, refspot(hspot)+5
                 IF (tspot.GT.0 .AND. tspot.LE.90) THEN
                  tvec(1) = tx(tspot,tscan) - xsat
                  tvec(2) = ty(tspot,tscan) - ysat
                  tvec(3) = tz(tspot,tscan) - zsat
                  tvecsq = tvec(1)**2 + tvec(2)**2 + tvec(3)**2
                  CALL crossprod(tvec,hvec,htprod)
                  htprodsq = htprod(1)**2 + htprod(2)**2 + htprod(3)**2
                  sinsq =  htprodsq/(tvecsq * hvecsq)
                  IF (sinsq .LT. tolerance_radsq) THEN    !include this spot
!                    write(*,*)'match: ',hscan,hspot,
!     &                 tscan-refscan(hscan),tspot-refspot(hspot),
!     &                 sinsq/tolerance_radsq,
!     &                 lath(hspot,hscan),lonh(hspot,hscan),
!     &                 latt(tspot,tscan),lont(tspot,tscan),
!     &                 htprodsq,tvecsq,hvecsq
                    IF (sinsq .LT. sinsq_min) THEN
                      sinsq_min = sinsq   !New nearest neighbour
                      tspotmin = tspot
                      tscanmin = tscan
                    ENDIF
                    IF (use_median) THEN
                      DO tchan=1,MWTS_CHANS
                        bttmp = btt(tchan,tspot,tscan)
                        IF (bttmp.GE.BminBT.AND.bttmp.LE.BmaxBT) THEN
                          num_t(tchan) = num_t(tchan) + 1
                          btt_use(num_t(tchan),tchan) = 
     &                       btt(tchan,tspot,tscan)
                        ENDIF
                      ENDDO
                    ENDIF
                  ENDIF
                 ENDIF
                ENDDO
              ENDIF
            ENDIF
          ENDDO
          
  !Find the median MWHS BT (use of median avoids problems with outliers)
  !  or use the nearest neighbour BT
  
          DO tchan=1,MWTS_CHANS
            IF (use_median) THEN
              IF (num_t(tchan).GT.0) THEN
                CALL amedian(btt_use(1,tchan),num_t(tchan),btmedian)
                btmap(tchan,hspot,hscan) = btmedian
              ELSE
                btmap(tchan,hspot,hscan) = RMDI
              ENDIF
            ELSE
              IF (tspotmin.GT.0) THEN
                btmap(tchan,hspot,hscan) = btt(tchan,tspotmin,tscanmin)
              ELSE
                btmap(tchan,hspot,hscan) = RMDI
              ENDIF
            ENDIF
          ENDDO
          
!          write(*,*) hscan,hspot,num_t(1)

        ENDDO   !hspot
      ENDDO   !hscan

      END         
