!+ Map AMSU-A to an individual AMSU-B fov
!
      SUBROUTINE PPA2B ( TFOV, TLine,
     &                   Ncolocs, List_fov, List_line, List_wt )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppa2b.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Map values from AMSU-A grid to AMSU-B fov.
!   Note that we are using nearest AMSU-A fov only so this is a simple task.
!
! Method:
! 0.  Initialisation
! 1.  Brightness temperatures
! 2.  Pre-processing variables
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      29/06/98  Original code. (R.J.Renshaw)
!  1.1      01/08/98  Fix bug comparing A/B 89GHz. (R.J.Renshaw)
!  1.2      22/03/05  Add BMapLandFrac (NCA)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"
! fixed data for processing AMSU-A  
#include "ppafdata.h" 
! block of AMSU-A data
#include "ppastore.h" 
! block of AMSU-B data     
#include "ppbstore.h" 
! bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"    
!
!   Arguments with intent(In):
      INTEGER       TFOV          ! target fov number
      INTEGER       TLine         ! number of target line in btemps array
      INTEGER       Ncolocs       ! no of colocated mapping FOVs
      INTEGER       List_line(*)  ! list of coloc line numbers (1st=nearest)
      INTEGER       List_fov(*)   !               spot numbers
      REAL          List_wt(*)    !               LUT weights
!
! Local scalars:
      INTEGER       chn           ! loop variable over all channels
      INTEGER       fov           ! mapping fov number
      INTEGER       line          ! number of mapping line in btemps array
      INTEGER       qca, qcb      ! AMSU-A,B qc words
      INTEGER       qcmask        ! bitword picks out selected bits from Aqc

!- End of header

! 0.  Initialisation
!     --------------

!     we use only the nearest neighbour
      fov  = List_fov(1)
      line = List_line(1)
      Bmapfov(TFOV,TLine) = fov

!     set up a bit mask to transfer qc bits from AMSU-A fov
      qcmask = 0
      qcmask = IBSET ( qcmask, bits(22) )  ! surface type mismatch
      qcmask = IBSET ( qcmask, bits(23) )  ! AMSU-A/B 89GHz mismatch
      qcmask = IBSET ( qcmask, bits(24) )  ! Grody light rainfall test
      qcmask = IBSET ( qcmask, bits(25) )  ! logistic precip probability test
      qcmask = IBSET ( qcmask, bits(26) )  ! scattering test
      qcmask = IBSET ( qcmask, bits(27) )  ! max prob scheme cloud flag


! 1.  Brightness temperatures
!     ---------- ------------

      DO chn = 1, Anchan
         BmapBT(chn,TFOV,TLine) = Abtemps(chn,fov,line)
      ENDDO  ! chn


! 2.  Pre-processing variables
!     -------------- ---------

! 2.1 pre-processing quality control word
      qca = Aqc(fov,line)
      qcb = Bqc(TFOV,TLine)

!     AMSU-A secondary calibration
      IF ( BTEST(qca,bits(30)) ) qcb = IBSET(qcb,bits(29))

!     AMSU-A pre-processing tests
!     qcb = qcb .OR. (qca.AND.qcmask)
      qcb = IOR(qcb, IAND(qca,qcmask))

!     mismatch between AMSU-A/B 89GHz values
      IF ( ABS( BmapBT(15,TFOV,TLine) - Bbtemps(1,TFOV,TLine) )
     &       .GT.  ABmax89 ) qcb = IBSET(qcb,bits(18))

! 2.2 results from nearest AMSU-A surface identification (PPASURF)
      Bmapsurf(TFOV,TLine) = Appsurf(fov,line)
      Bsurfcost(TFOV,TLine) = Asurfcost(fov,line)
      BMapLandFrac(TFOV,TLine) = ALandFrac(fov,line)

      RETURN
      END
