!+ calculate cirrus scattering index
!
      FUNCTION PPACIRR ( TB1, TB16, TB17, TB20, FOV )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppacirr.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Estimate AMSU-B channel 20 brightness temperature from AMSU-A
!   channel 1, and AMSU-B channels 16 and 17 and calculate difference
!   from measured channel 20 value.
!
! Method:
!   Estimate AMSU-B channel 20 brightness temperature from channels 1,16,17
!   The equation we are are using is T20 = A1 + T1*A2 + T16*A3 + T17*A4
!   A1, A2, A3 and A4 are functions of the secant of the view angle - 1
!   Then calculate scattering index as estimated TB20 - observed TB20
!
! Theory: This is an extension of the 89 GHz scattering index which is
!         evaluated in the routine PPASCAT.  Using channels 1, 16 and 17
!         to predict channel 20, the routine is complementary to the 89 GHz
!         index in that if the 89 GHz scattering index is high, the values
!         generated by this 183 GHz index will be both irrelevant and
!         misleading.  This is because the coefficients are derived assuming
!         liquid water cloud or clear skies only.
!
! Current Code Owner: D.C.Jones
!
! History:
! Version  Date      Comment
!
!  1.0     14/08/00  Adapted from PPASCAT (D.C.Jones)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"
! fixed data for processing AMSU-A
#include "ppafdata.h"       
!
!   Scalar arguments with intent(In):
      REAL          TB1           ! brightness temp in AMSU-A channel  1 (K)
      REAL          TB16          ! brightness temp in AMSU-B channel 16 (K)
      REAL          TB17          ! brightness temp in AMSU-B channel 17 (K)
      REAL          TB20          ! brightness temp in AMSU-B channel 20 (K)
      REAL          ETB20         ! estimated BT in AMSU-B channel 20 (K)
      INTEGER       FOV           ! fov number in scan
!
!   Return value of function:
      REAL          PPACIRR       ! scattering index = ETB20 - TB20
!
!- End of header


! 1.  Estimate channel 20
!     -------- ------- --

      ETB20 = ABcirr(1,FOV)
     &      + ABcirr(2,FOV)*TB1
     &      + ABcirr(3,FOV)*TB16
     &      + ABcirr(4,FOV)*TB17

! 2.  Calculate scattering index
!     --------- ---------- -----

      PPACIRR = ETB20 - TB20

      RETURN
      END
