!+ calculate cirrus scattering index
!
      FUNCTION PPATMSCIRR ( TB1, TB16, TB17, TB18, FOV )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppatmscirr.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose:
!   Estimate ATMS channel 18 brightness temperature from ATMS
!   channels 1, 16 and 17 and calculate difference
!   from measured channel 18 value. Note ATMS channel 18 is equivalent to
!   AMSU-B channel 20.
!
! Method:
!   Estimate channel 18 brightness temperature from channels 1,16,17
!   The equation we are are using is T18 = A1 + T1*A2 + T16*A3 + T17*A4
!   A1, A2, A3 and A4 are functions of the secant of the view angle - 1
!   Then calculate scattering index as estimated TB18 - observed TB18
!
! Theory: This is an extension of the 89 GHz scattering index which is
!         evaluated in the routine PPASCAT.  Using channels 1, 16 and 17
!         to predict channel 18, the routine is complementary to the 89 GHz
!         index in that if the 89 GHz scattering index is high, the values
!         generated by this 183 GHz index will be both irrelevant and
!         misleading.  This is because the coefficients are derived assuming
!         liquid water cloud or clear skies only.
!
! Current Code Owner: NWP SAF
!
! History:
! Version  Date      Comment
!
!  1.0   08/09/2010  From PPACIRR
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"
! fixed data for processing ATMS
#include "ppatmsfdata.h"       
!
!   Scalar arguments with intent(In):
      REAL          TB1           ! brightness temp in ATMS channel  1 (K)
      REAL          TB16          ! brightness temp in ATMS channel 16 (K)
      REAL          TB17          ! brightness temp in ATMS channel 17 (K)
      REAL          TB18          ! brightness temp in ATMS channel 18 (K)
      REAL          ETB18         ! estimated BT in ATMS channel 18 (K)
      INTEGER       FOV           ! fov number in scan
!
!   Return value of function:
      REAL          PPATMSCIRR    ! scattering index = ETB18 - TB18
!
!- End of header


! 1.  Estimate channel 18
!     -------- ------- --

      ETB18 = atmscirr(1,FOV)
     &      + atmscirr(2,FOV)*TB1
     &      + atmscirr(3,FOV)*TB16
     &      + atmscirr(4,FOV)*TB17

! 2.  Calculate scattering index
!     --------- ---------- -----

      PPATMSCIRR = ETB18 - TB18

      RETURN
      END
