!+ Crosby precipitation test
!
      FUNCTION PPATMSCROSBY ( T1, T16 )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppatmscrosby.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose:
!   calculate Crosby precipitation probability from ATMS channels 1 & 16
!
! Method:
!   Theory is in
!   "Crosby, Ferraro & Wu, "Estimating the probability of rain in an SSM/I FOV
!    using logistic regression", Journal of Applied Met., 34 no.11 1995"
!
! Current Code Owner: NWP SAF
!
! History:
! Version  Date      Comment
!
!  1.0    06/09/2010  from ppcrosby.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"         
! fixed data for processing ATMS
#include "ppatmsfdata.h"        
!
!   Scalar arguments with intent(In):
      REAL          T1            ! brightness temp in ATMS channel 1 (K)
      REAL          T16           ! brightness temp in ATMS channel 16 (K)
!
!   Return value of function:
      REAL          PPATMSCROSBY      ! probability of precipitation
!
!   Local Scalars:
      REAL          f
!
!- End of header

      f = atmscrosby1 + atmscrosby2*T1 + atmscrosby3*T16
      PPATMSCROSBY = 1. / (1.+exp(-f))  ! probability of precipitation

      RETURN
      END
