!+ calculate scattering index
!
      FUNCTION PPATMSSCAT ( TB1, TB2, TB3, TB16, FOV )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppatmsscat.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose:
!   estimate ATMS channel 16 brightness temperature from channels 1,2,3
!
! Method:
!   Estimate ATMS channel 16 brightness temperature from channels 1,2,3
!   The equation we are are using is T16 = A1 + T1*A2 + T2*A3 + T3*A4
!   A1, A2, A3 and A4 are functions of the secant of the view angle - 1
!   Then calculate scattering index as estimated TB16 - observed TB16
!
! Theory: For long wavelengths (>3mm) very few objects cause significant
!         scattering. For wavelengths between 0.5 and 3mm an increasing
!         number of surfaces significantly scatter radiation e.g. sand
!         grains in a desert, brine pockets in sea ice, air pockets in sea
!         ice, snow grains, ripples on the ocean surface. At very short
!         wavelengths less than 0.5mm scattering by ice hydrometeors and
!         precipitation sized water droplets becomes significant. In practise
!         the relatively wide spacing of AMSU window channels 20, 30, 50, 90,
!         160 GHz means all these surfaces/particles appear to scatter most
!         significantly at high frequency and less at lower frequency. A simple
!         scattering index can be devised that predicts the 85.5/89/91 GHz
!         channel (89 in the case of ATMS; most radiometers have a channel
!         near 90 GHz) from the lower frequency channels. In the absence of
!         scattering brightness temperature generally increases with frequency
!         and therefore the prediction will always predict TB89>TB31 etc. But
!         in the presence of scattering the 89 GHz brightness temperature is
!         lowered. Therefore the low frequency channels overestimate the high
!         frequency channel. Therefore the difference between the predicted
!         and measured 89 GHz TB gives us a simple scattering index.
!
! Current Code Owner: NWP SAF
!
! History:
! Version  Date      Comment
!
!  1.0    06/09/2010  from ppascat.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"
! fixed data for processing ATMS    
#include "ppatmsfdata.h"       
!
!   Scalar arguments with intent(In):
      REAL          TB1           ! brightness temp in ATMS channel 1 (K)
      REAL          TB2           ! brightness temp in ATMS channel 2 (K)
      REAL          TB3           ! brightness temp in ATMS channel 3 (K)
      REAL          TB16          ! brightness temp in ATMS channel 16 (K)
      REAL          ETB16         ! estimated BT in ATMS channel 16 (K)
      INTEGER       FOV           ! fov number in scan
!
!   Return value of function:
      REAL          PPATMSSCAT    ! scattering index = ETB16 - TB16
!
!- End of header


! 1.  Estimate channel 16
!     -------- ------- --

      ETB16 = atmsscat(1,FOV)
     &      + atmsscat(2,FOV)*TB1
     &      + atmsscat(3,FOV)*TB2
     &      + atmsscat(4,FOV)*TB3

! 2.  Calculate scattering index
!     --------- ---------- -----

      PPATMSSCAT = ETB16 - TB16

      RETURN
      END
