!+ Read fixed data file for AMSU-B within ATOVPP
!
      SUBROUTINE PPBFDF ( Bfunit )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppbfdf.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Read fixed data file required for AMSU-B within ATOVPP.
!
! Method:
! 0.  initialisation
! 1.  open file
! 2.  look for sections in the file
! 3.  read numbers for the pre-processing
! 4.  read bias corrections
! 5.  close file
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1       9/12/96  include Bboxlv89
!  1.2      09/03/04  Modify for Linux (N Atkinson / A Korpela)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"       
! fixed data for processing AMSU-B
#include "ppbfdata.h"       
!
! Subroutine arguments:
!   Scalar arguments with intent(in)
      INTEGER       Bfunit        ! unit number for AMSU-B-specific data
!
! Local Scalars:
      CHARACTER*80  input_line    ! line read from instream data
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status value
      INTEGER       chn           ! loop index: channel number
      INTEGER       fov           ! loop index: fov number

! Function & Subroutine calls:
      External ErrorReport        ! ATOVPP error routine

!- End of header


! 0.  initialisation
!     --------------

! 0.1 bias corrections
      DO fov = 1, Bnfovs
      DO chn = 1, Bnchan
         Bbiases(chn,fov) = 0.
      ENDDO  ! chn
      ENDDO  ! fov


! 1.  open file
!     ---- ----

      OPEN ( UNIT=Bfunit, STATUS="OLD", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus )

!     If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
      IF ( errStatus.NE.0 )
     &  OPEN ( UNIT=Bfunit, STATUS="UNKNOWN", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus )

      WRITE ( errMessage,
     & "("" Error opening AMSU-B fixed data file, unit "",I8)" ) Bfunit
      errStatus = ABS (errStatus)   ! end-of-file returns -1
      CALL ErrorReport(errStatus,errMessage,"PPBFDF")


! 2.  look for sections in the file
!     ---- --- -------- -- --- ----

      DO WHILE ( errStatus.EQ.0 .AND. input_line(1:3).NE."END" )
         input_line = " "
         READ(Bfunit,"(A80)",IOSTAT=errStatus) input_line


         IF ( input_line(1:6) .EQ. "PREPRO" ) THEN

! 3.        read numbers for the pre-processing
!           ---- ------- --- --- --------------

! 3.1       box size and threshold for local variance of AMSUB 89GHz channel
!           (routine PPAMSUB1)

            input_line = "!"
            errStatus = 0
            errMessage = "Error reading local variance numbers"
            DO WHILE ( input_line(1:1) .EQ. "!" .AND. errStatus.EQ.0 )
               READ(Bfunit,"(A80)",IOSTAT=errStatus) input_line
            ENDDO
            CALL ErrorReport(errStatus,errMessage,"PPBFDF")
            READ(input_line,*,IOSTAT=errStatus)
     &           Bboxlv89, Bmaxlv89       !  box size (fovs), test threshold
            errStatus = ABS (errStatus)   ! end-of-file returns -1
            CALL ErrorReport(errStatus,errMessage,"PPBFDF")


         ELSEIF ( input_line(1:4) .EQ. "BIAS" ) THEN

! 4.        read bias-corrections
!           ---- ----------------

!           read scan-dependent bias corrections
            errMessage = "Reading BIAS section"
            errStatus = 0
            DO fov = 1, Bnfovs
               IF ( errStatus.EQ.0 ) THEN
                  READ(Bfunit,"(10F8.3)",IOSTAT=errStatus)
     &                       (Bbiases(chn,fov),chn=1,Bnchan)
               ENDIF
            ENDDO  ! fov
            errStatus = ABS (errStatus)   ! end-of-file returns -1
            CALL ErrorReport(errStatus,errMessage,"PPBFDF")


         ENDIF  ! input_line

      ENDDO  ! WHILE ( errStatus.EQ.0 )


! 5.  close file
!     ----- ----

      CLOSE(Bfunit)


      RETURN
      END
