      SUBROUTINE ppbgb2a(TFov,TLine,
     +                      NColocs,List_fov,List_line,List_wt)
      IMPLICIT NONE
!
! Name: $Id: ppbgb2a.F 128 2008-10-07 09:33:03Z frna $
!
! This pre-existing routine has been included in AAPP as part 
! of EUMETSAT NWP-SAF
! IT BELONGS TO BACKUS-GILBERT-CONVOLUTION AMSU-B to AMSU-A
! see Bennartz (2000), J. Atmos. Ocean. Technology, Vol. 17, No. 9, 1215-1225
!
!
!  It performs a convolution of 7x7 fields of AMSU-B data to AMSU-A for
!  the AMSU-A-pixel (TFOV,TLINE)
!  
!   1. Find the first AMSU-B scan after the start of the current AMSU-A scan 
!      (This is the synchronisation line)
!   2. Look for quality of AMSU-B 7x7 field 
!      (If bad pixels/lines are detected, the convolution will not be performed)
!   3. If AMSU-B data is ok, do cobnvolution otherwise set AMSU-B missing flag in
!      in 1d FLags and  set AMSU-B convolved TbBs to RMDI 
!
!
! Ralf Bennartz, 06/12/2000
!
! Fixed bug that affects N-16 and higher (expanded Lookuptables and
! added one dimension to amb_lin and amb_pix) 
! Ralf Bennartz, 04/12/2004
!
! Move setting of isat to top of subroutine. Prevent array bounds error
! at start/end (using variable ii). Use NOAA-17 weights for NOAA-18.
! Nigel Atkinson, 22/09/2008
!
      
      INTEGER       TFOV          ! target fov number
      INTEGER       TLine         ! number of target line in btemps array
      INTEGER       Ncolocs       ! no of colocated mapping FOVs
      INTEGER       List_line(*)  ! list of coloc line numbers (1st=nearest)
      INTEGER       List_fov(*)   !               spot numbers (NOT USED)
      REAL          List_wt(*)    !               LUT weights  (NOT USED)

! general parameters for ATOVPP
#include "ppparms.h"        
! general parameters for ATOVPP
#include "ama1c.h"          
! block of AMSU-A data
#include "ppastore.h"       
! fixed data for processing AMSU-A
#include "ppafdata.h"        
! block of AMSU-B data
#include "ppbstore.h"       
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"      
#include "ppbgb2a.h"


      !
      ! Variables used for AMSU-A to AMSU-B synchronisation
      !
      INTEGER line0     ! AMSU-B line closest to AMSU-A (TFOV,TLINE)
                        ! Note, this is NOT always the first line after 
                        ! synchronisation pulse
      INTEGER syncline  ! First AMSU-B line after synchronisation pulse
                        ! inferred from AMSU-A and AMSU-B times
      INTEGER tmin      ! used to find the AMSU-B synchronisation line
      INTEGER tdiff     !  dito
      INTEGER imin      !  dito
      INTEGER imax      !  dito
      INTEGER ifound    ! set to 1 if sync. line has been identified 
      
      !
      ! Flag variables
      !
      INTEGER failflag(conv_size,conv_size) ! Flag indicating that no BG-Conv
                                            ! is to be performed for
                                            ! current (TFOV,TLINE)
      INTEGER Bsec       ! set if secondary calibration is used
      INTEGER Bbad       ! set if AMSU-B field of view is bad
      INTEGER pp_qc      ! post processing quality flag
      INTEGER Bq_flag    ! temporarily holds the quality flag of AMSU-B pixel
      INTEGER nbad       ! number of bad AMSU-B FOVS in 7x7 field
      
      !
      ! Loop variables
      !
      INTEGER i,j,ii     ! Loop variables over 7x7 field of AMSU-B for conv.
      INTEGER chn        ! Loop variable over AMSU-B channels

      !
      ! Variables for actual convolution
      !
      INTEGER isat        ! Field for weights for different satellites 
                          ! Must be    1 for NOAA-15
                          !            2 for NOAA-16
                          !            3 for NOAA-17
                          !            4 otherwise 
      REAL    sum(BNCHAN) ! holds for each channel the sum of BTs
      REAL    sum_weights ! used for normalization of sum
      

! get satellite ID
! Note that NOAA-18 and MetOp-A seem to work best using NOAA-17 weights so
! don't use the 4th set of weights (which are same as NOAA-16).
       isat = ama1c_h_satid - 14
       IF ((isat.LT.1).OR.(isat.GT.3)) THEN
          isat = 3
       ENDIF
!       WRITE(*,*) ama1c_h_satid,isat
        
!
! 1.  FIND synchronisation line
!     if found then ifound=1 and syncline holds the line number
!     if not found  ifound=0
!
      line0 = List_line(1) ! This is the closest AMSU-B line,
                           ! Syn-line should be close by, so 
                           ! search for it in the surrounding by
                           ! looking at the time differences between 
                           ! AMSU-A and AMSU-B line
      tmin  = 9999999
      imin=MAX(line0-50,1)
      imax=MIN(line0+50,bnlines)
      ifound=0
      syncline = line0
      DO i=imin,imax
        tdiff = ABS(btime(i)-atime(TLine))
        IF (tdiff.LT.tmin) THEN
           syncline= i
           ifound  = 1
           tmin    = tdiff
        ENDIF     
      ENDDO
      IF (tmin.GT.18) THEN 
        ifound=0 ! more than one FOV away
      ENDIF
       
!
! 2. Look for quality of AMSU-B
!    nbad holds the number of bad pixels
!    failflag indicates which ones are bad

!
! Note that currently bad data flag and scanqual are not used for quality 
! control. This is because otherwise NOAA-15 data seem not to work
! For NOAA-16 the lines commented out as !!! can be used to increase 
! accuracy of quality check
!         
      Bbad  = 0
      Bbad  = IBSET ( Bbad, bits(0) )  ! bad/missing data
      Bbad  = IBSET ( Bbad, bits(30) )  ! bad/missing data
      Bsec  = 0
      Bsec  = IBSET ( Bsec, bits(30) )  !secondary cal.

      PP_qc = 0
      nbad = 0
!     Look what AMSU-B data is acceptable (primary or secondary cal. ok)
      DO i = 1,conv_size      !  line 
         DO j = 1,conv_size   ! pixel
            ii = syncline + amb_lin(j,i,TFOV,isat)

            IF (ii.LT.1.OR.ii.GT.bnlines) THEN
              nbad = nbad + 1
              failflag(i,j) = 1
            ELSE
              Bq_flag       = Bqc( amb_pix(j,i,TFOV,isat) , ii )

              IF (
     +            !!! ( (Bq_flag.AND.Bbad) .ne. 0).OR.
     +                 (Bqualind (ii).NE.0)  
!!!     +             .OR.(Bscanqual(ii).NE.0) 
     +           ) THEN  
                 failflag(i,j) = 1
                 nbad = nbad + 1
              ELSE
                 failflag(i,j) = 0  
              ENDIF       

              IF (Bq_flag.EQ.Bsec) THEN 
                 pp_qc = IBSET(pp_qc,bits(29))  ! secondary calibration flag
              ENDIF

            ENDIF
         ENDDO    ! pixel
      ENDDO       ! line

!
! 3. If there are no bad pixels and the syncline is found 
!    then do the convolution
!

      IF ((nbad.EQ.0).AND.(ifound.EQ.1)) THEN 

! Initialize temporary fields
        DO chn=1,BNCHAN  
           sum(chn) = 0.0
        ENDDO   
        sum_weights=0.0

!  3.1 Now do the weighted sum over all AMSU-B FOVS

        DO i = 1,conv_size      !  line 
           DO j = 1,conv_size   ! pixel

             IF (failflag(i,j) .eq. 0) THEN
               DO chn=1,BNCHAN
                  sum(chn) = sum(chn) + 
     +                     amb_weight(j,i,TFOV,isat) *
     +                     Bbtemps(chn,       
     +                                amb_pix(j,i,TFOV,isat),
     +                     syncline + amb_lin(j,i,TFOV,isat)) 
               ENDDO             

               sum_weights = sum_weights + amb_weight(j,i,TFOV,isat) 

             ENDIF
           ENDDO    ! pixel
        ENDDO       ! line
        
!
!  3.2 Normalize and store results
!
        DO chn = 1, Bnchan
             Abtemps(chn+Anchan,TFOV,TLine) = sum(chn) / sum_weights
        ENDDO  ! chn

      ELSE 
!
! 3.3 If either syncline was not found or bad AMSU-B FOVS were found
!     set pp_qc FLag to missing andTBS to RMDI
!
         DO chn = 1, Bnchan
            Abtemps(chn+Anchan,TFOV,TLine) = RMDI
         ENDDO  ! chn
         pp_qc = IBCLR(pp_qc,bits(29))  ! secondary calibration flag
         pp_qc = IBSET(pp_qc,bits(28))  ! AMSU-B missing  
      ENDIF

      END
