!+ pre-process AMSU on CRIS grid
!
      SUBROUTINE PPCATMS ( crisLastLine )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppcatms.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose:
!   pre-process a block of AMSU data mapped to CRIS grid
!
! Method:
! 1.  Test for precipitation (PPCAPCP)
! 2.  Estimate surface type from the data (PPCASURF)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0   02/09/2010  Based on ppiamsu.F (N.C.Atkinson)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  parameters and look-up tables for the mapping 
#include "ppluts.h"
!  CRIS level 1d format 
#include "cris1d.h"
!  fixed data for processing CRIS 
#include "ppcrisfdata.h"
!  block of CRIS level 1d data 
#include "ppcrisstore.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
!   Scalar arguments with intent(In):
      INTEGER       crisLastLine     ! number of last CRIS line in store
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
!
! Function & Subroutine calls:
      External ErrorReport        ! ATOVPP error routine
      External PPCAPCP            ! test for precipitation, etc.
      External PPCASURF           ! estimate surface type from the data
      External PPCRISCLOUD        ! cloud test
!
!- End of header

! 1.  test for precipitation
!     ---- --- -------------

      CALL PPCAPCP ( crisLastLine )

! 2.  Estimate surface type from the data
!     -------- ------- ---- ---- --- ----

!    (uses channels from ATMS)
      CALL PPCASURF ( crisLastLine )

! 3.  Run cloud test(s) that make use of ATMS data
!     --------------------------------------------

      CALL PPCRISCLOUD(2, crisLastLine)

      RETURN
      END
