!+ pre-process CRIS before mapping
!
      SUBROUTINE PPCRIS1 ( crisLastLine, LSunit, HTunit )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppcris1.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose:
!   pre-process a block of level 1c CRIS data
!
! Method:
! 1.  Get land/sea information & elevation (SURFELEV)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0   06/09/2010  N.C.Atkinson
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  CRIS 1d format 
#include "cris1d.h"
!  fixed data for processing CRIS 
#include "ppcrisfdata.h"
!  block of CRIS data 
#include "ppcrisstore.h"
!
!   Scalar arguments with intent(In):
      INTEGER       crisLastLine  ! number of last CRIS line in store
!
      INTEGER       HTunit        ! unit no. for heights dataset
      INTEGER       LSunit        ! unit no. for land/sea bitmap
!
! Local scalars:
      INTEGER       line          ! loop index: pointer to scanline in buffer
      INTEGER       fov           ! loop index: fov number
!
! Local arrays:
      REAL          lats(crisnfovs)  ! array of latitudes for a scanline
      REAL          lons(crisnfovs)  ! array of longitudes for a scanline
      INTEGER       surftype(crisnfovs) !surface type for a scanline (SURFELEV)
      INTEGER       elevation(crisnfovs)!elevation for a scanline (SURFELEV)
      INTEGER       landfrac(crisnfovs) !land fraction for a scanline (SURFELEV)
!
! Function & Subroutine calls:
      External SURFELEV           ! get land/sea information & elevation
!
!- End of header

      WRITE(*,"(10X,""Pre-processing CRIS before the mapping"")")

! 1.  Get land/sea information
!     --- -------- -----------

!     loop through scanlines not already processed
      DO line = 1, crisLastLine

! 1.1    pick out lat's and lon's from scanline
         DO fov = 1, crisnfovs
            lats(fov) = crislats(fov,line)        ! degrees N
            lons(fov) = crislons(fov,line)        ! degrees E
         ENDDO  ! fov

! 1.2    find surface type for each fov
         CALL SURFELEV ( LSunit, HTunit,
     &                   lats, lons, crisnfovs, crisboxSize,
     &                   surftype, elevation, landfrac )

! 1.3    store surface type & elevation
         DO fov = 1, crisnfovs
            crissurftype(fov,line) = surftype(fov)
            criselev(fov,line) = elevation(fov)
            crislandfrac(fov,line) = landfrac(fov)
         ENDDO  ! fov

      ENDDO  ! line

      RETURN
      END
