!+ Copy level 1C CRIS data to level 1D common area
!
      SUBROUTINE PPCRIS1D ( crisrec, line )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppcris1d.F 317 2012-07-03 10:38:38Z frna $
!
! Purpose:
!   Copy CRIS data from cris1c_dt to cris1d_dt, applying spatial
!   thinning and channel selection as specified by user. Call the routine to
!   compute PC scores, if required.
!
! Method:
!   Channel selection is provided in common ppifdata. If spatial thinning is
!   specified, the best detector for each FOV is obtained from ppistore.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0   17/08/2010  From ppi1d.F
!  1.1   20/06/2012  Shorten name of "Use_reconstructed_radiances_cris" to less
!                    than 32 characters. NCA
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  CRIS level 1C header/data common blocks 
#include "cris1c.h"
!  CRIS level 1D header/data common blocks 
#include "cris1d.h"
!  block of CRIS data 
#include "ppcrisstore.h"
!  fixed data 
#include "ppcrisfdata.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
!   Scalar arguments with intent(In):

      INTEGER crisrec    !CRIS record number 1 to 9 (for full spatial res mode)
      INTEGER line       !Scan line in ppcrisstore

!   Local scalars:

      INTEGER Ispot, Iclass, Ichan, I, Ifor, Ipc      !Loop indices
      INTEGER BestDet    !Chosen detector for each FOV
      REAL    Recon
      LOGICAL First
      DATA First/.TRUE./
      INTEGER Cloudchan      !CrIS channel number for cloud test
      INTEGER Mindiff, chandiff
      INTEGER channels_index (crisnchan)
      DATA    channels_index /crisnchan*0/
      
! Find the channel closest to nominal wavenumber for cloud tests
      IF (First) THEN
        crisCloudW = 2390.0       !wavenumber. This is in band 3.
        Mindiff = 99999
        
        Cloudchan = (crisCloudW - 0.01*cris1c_h_startw(3))/
     &      (0.01*cris1c_h_endw(3) - 0.01*cris1c_h_startw(3)) *
     &      (cris1c_h_endchan(3) - cris1c_h_startchan(3)) + 
     &       cris1c_h_startchan(3)
        DO Ichan=1,cris1c_h_nchanused
          chandiff = ABS(cris1c_h_channels(Ichan) - Cloudchan)
          IF (chandiff .LT. Mindiff) THEN
            Mindiff = chandiff 
            crisCloudindex = Ichan
          ENDIF
        ENDDO

! Create index to the level 1C channels
        DO Ichan=1,cris1c_h_nchanused
          channels_index(cris1c_h_channels(Ichan)) = Ichan
        ENDDO

        First = .false.
      ENDIF
!
!- End of header
      
      cris1d_scnlin = cris1c_scnlin
      cris1d_scnlinyr = cris1c_scnlinyr
      cris1d_scnlindy = cris1c_scnlindy
      cris1d_satxyz(1) = cris1c_satxyz(1)
      cris1d_satxyz(2) = cris1c_satxyz(2)
      cris1d_satxyz(3) = cris1c_satxyz(3)
      cris1d_ascdesc = cris1c_ascdesc
      cris1d_orbit = cris1c_orbit
      cris1d_scalti = cris1c_scalti
      cris1d_scanqualflgs = cris1c_scanqualflgs
      cris1d_geolocqual = cris1c_geolocqual

      DO Ifor=1,cris_1dnfor
      
        IF (thinmode_cris .GE. 2) THEN
          BestDet = crisbestFOV(Ifor,line)   !intelligent thinning
        ELSE
          BestDet = detector_use_cris(crisrec) !fixed thinning
        ENDIF
         
        cris1d_FOV(Ifor) = (Ifor-1)*9 + BestDet          !1 to 270
        cris1d_scnlintime(Ifor) = cris1c_scnlintime(Ifor)

        DO I=1,2
          cris1d_latlon(I,Ifor) = cris1c_latlon(I,BestDet,Ifor)
        enddo
        DO I=1,4
          cris1d_angles(I,Ifor) = cris1c_angles(I,BestDet,Ifor)
        ENDDO
        
        cris1d_surfht(Ifor) = cris1c_surfht(BestDet,Ifor)
        cris1d_landfrac(Ifor) = cris1c_landfrac(BestDet,Ifor)
        cris1d_landsea(Ifor) = cris1c_landsea(BestDet,Ifor)
        cris1d_cloudcover(Ifor) = cris1c_cloudcover(BestDet,Ifor)
        cris1d_cloudtopht(Ifor) = cris1c_cloudtopht(BestDet,Ifor)
        cris1d_radianceflgs(Ifor) = cris1c_radianceflgs(BestDet,Ifor)

        DO I=1,cris_1cnband
          cris1d_calqualflgs(I,Ifor)= cris1c_calqualflgs(I,BestDet)
          cris1d_fovqualflgs(I,Ifor)= cris1c_fovqualflgs(I,BestDet,Ifor)
        ENDDO
        cris1d_quality(Ifor) = cris1c_quality(BestDet,Ifor)

        DO Ichan=1,nchan_cris             !CRIS radiances (selected channels)
          I = channels_index(channels_cris(Ichan))
          IF (I .NE. 0) THEN
            cris1d_radiance(Ifor,Ichan)= cris1c_radiance(BestDet,Ifor,I)
          ELSE
            cris1d_radiance(Ifor,Ichan)= MDI
          ENDIF
        ENDDO

!Store 2390 cm^-1 scaled radiance for use in cloud test 2 (see ppicloud)
        crisCloudflag(2,Ifor,line) = 
     &            cris1c_radiance(BestDet,Ifor,crisCloudindex)
        
        DO I=1,10
          cris1d_cloudflag(I,Ifor)=0     !Initialise cloud test flags
        ENDDO
        
        IF (npc_CRIS .GT. 1) THEN
          CALL PPCRISSPECTRA (BestDet,Ifor)       !Generate PC scores (later)

!Produce reconstructed radiances if user has requested it, over-writing the raw
!radiance

          IF (Use_reconstructed_rad_cris) THEN
            DO Ichan=1,nchan_cris
              Recon = mean_decode_CRIS(channels_cris(Ichan))
              DO Ipc=1,npc_CRIS
                Recon = Recon+eig_decode_CRIS(channels_cris(Ichan),Ipc)
     &                       * cris1d_scores(Ipc,Ifor)/2.0
              ENDDO
              IF (Recon .LT. 0.0) Recon = 0.0  !Expect positive recon radiances
              cris1d_radiance(Ichan,Ifor) = 
     &            NINT(Recon*1E4)
            ENDDO
          ENDIF    !Use_reconstructed_rad_cris

        ENDIF    !npc_CRIS .GT. 1

      ENDDO   !Ifor
      
      CALL PPCRISCLOUD(1, line)    !Run first cloud test (sdev of 9 CRIS spots)
      
      END
