!+ Crosby precipitation test
!
      FUNCTION PPCROSBY ( T1, T15 )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppcrosby.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   calculate Crosby precipitation probability from AMSU-A channels 1 & 15
!
! Method:
!   Theory is in
!   "Crosby, Ferraro & Wu, "Estimating the probability of rain in an SSM/I FOV
!    using logistic regression", Journal of Applied Met., 34 no.11 1995"
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Derived from Steve English's PV-Wave code (27/10/95)
!  1.1      10/12/96  Changed to a logical function, with coeffs in ppafdata.h
!  1.2      20/05/98  Changed back to return just the probability
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"         
! fixed data for processing AMSU-A
#include "ppafdata.h"        
!
!   Scalar arguments with intent(In):
      REAL          T1            ! brightness temp in AMSU-A channel 1 (K)
      REAL          T15           ! brightness temp in AMSU-A channel 15 (K)
!
!   Return value of function:
      REAL          PPCROSBY      ! probability of precipitation
!
!   Local Scalars:
      REAL          f
!
!- End of header

      f = Acrosby1 + Acrosby2*T1 + Acrosby3*T15
      PPCROSBY = 1. / (1.+exp(-f))  ! probability of precipitation

      RETURN
      END
