!+ Controlling routine: write out level 1D headers within ATOVPP
!
      SUBROUTINE PPFINISH ( TOVS, LAMSA, LAMSB, LMSU, LPCIASI,
     &                      LATMS, LCRIS, LHIRAS,
     &                      GHIRS, GAMSA, GAMSB, GIASI,
     &                      GATMS, GCRIS, GHIRAS,
     &                      H1Dunit, A1Dunit, B1Dunit, I1Dunit,
     &                      atms1Dunit, cris1Dunit, hiras1Dunit )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppfinish.F 618 2019-02-12 10:54:03Z frna $
!
! Purpose:
!   Controlling routine: write out level 1D headers within ATOVPP
!
! Method:
! 1.  Write HIRS header (TOVS or ATOVS)
! 2.  Write AMSU-A header
! 3.  Write AMSU-B header
! 4.  Write IASI header
! 5.  Write ATMS header
! 6.  Write CRIS header
! 7.  Write HIRAS header
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      29/06/98  Extra argument to PPBOUTH (for mapped AMSU-A)
!  1.2      12/08/05  Add IASI
!  1.3      23/09/08  Add IASI PC
!  1.4      16/08/10  Add ATMS and CRIS
!  1.5      11/02/19  Add HIRAS
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"        
!
!   Scalar arguments with intent(In):
      LOGICAL       TOVS          ! TRUE if instrument is TOVS, else FALSE
      LOGICAL       LAMSA         ! .TRUE. to process AMSU-A
      LOGICAL       LAMSB         ! .TRUE. to process AMSU-B
      LOGICAL       LMSU          ! .TRUE. to process MSU
      LOGICAL       LPCIASI       ! .TRUE. to process IASI PC
      LOGICAL       LATMS         ! .TRUE. to process ATMS
      LOGICAL       LCRIS         ! .TRUE. to process CRIS
      LOGICAL       LHIRAS        ! .TRUE. to process HIRAS
!
      LOGICAL       GHIRS         ! .TRUE. to output on HIRS grid
      LOGICAL       GAMSA         ! .TRUE. to output on AMSU-A grid
      LOGICAL       GAMSB         ! .TRUE. to output on AMSU-B grid
      LOGICAL       GIASI         ! .TRUE. to output on IASI grid
      LOGICAL       GATMS         ! .TRUE. to output on ATMS grid
      LOGICAL       GCRIS         ! .TRUE. to output on CRIS grid
      LOGICAL       GHIRAS        ! .TRUE. to output on HIRAS grid
!
      INTEGER       H1Dunit       ! logical unit no. for HIRS 1D output
      INTEGER       A1Dunit       ! logical unit no. for AMSU-A 1D output
      INTEGER       B1Dunit       ! logical unit no. for AMSU-B 1D output
      INTEGER       I1Dunit       ! logical unit no. for IASI 1D output
      INTEGER       atms1Dunit    ! logical unit no. for ATMS 1D output
      INTEGER       cris1Dunit    ! logical unit no. for CRIS 1D output
      INTEGER       hiras1Dunit   ! logical unit no. for HIRAS 1D output
!
! Local scalars:
      INTEGER       H1Drecord     !record number for HIRS level 1D file
      INTEGER       A1Drecord     !record number for AMSU-A level 1D file
      INTEGER       B1Drecord     !record number for AMSU-B level 1D file
      INTEGER       I1Drecord     !record number for IASI level 1D file
      INTEGER       atms1Drecord  !record number for ATMS level 1D file
      INTEGER       cris1Drecord  !record number for CRIS level 1D file
      INTEGER       hiras1Drecord !record number for HIRAS level 1D file
!
! Function & Subroutine calls:
      External PPHOUTH            ! Set up HIRS/ATOVS level 1D header
      External IOH1D              ! write HIRS/ATOVS 1D scanline
      External PPHOUTHM           ! Set up HIRS/TOVS level 1D header
      External IOH1DM             ! write HIRS/TOVS 1D scanline
      External PPAOUTH            ! Set up AMSU-A level 1D header
      External IOA1D              ! write AMSU-A 1D scanline
      External PPBOUTH            ! Set up AMSU-B level 1D header
      External IOB1D              ! write AMSU-B 1D scanline
      External PPIOUTH            ! Set up IASI level 1D header
      External IOI1D              ! write IASI 1D scanline
      External PPATOUTH           ! Set up ATMS level 1D header
      External IOAT1D             ! write ATMS 1D scanline
      External PPCOUTH            ! Set up CRIS level 1D header
      External IOC1D              ! write CRIS 1D scanline
      External PPHIRASOUT         ! Set up HIRAS level 1D header
      External IOHIRAS1D          ! write HIRAS 1D scanline
!
!- End of header


! 1.  Write HIRS header
!     ----- ---- ------

      IF (GHIRS) THEN      ! output on HIRS grid required

!        Separate routines for TOVS/ATOVS
         IF ( TOVS ) THEN

! 1.1       Create 1D header record from store
            CALL PPHOUTHM ( LMSU )

! 1.2       Write out 1D header record
            H1Drecord = 1
            CALL IOH1DM ("WRITE HEADER", H1Dunit, H1Drecord)

         ELSE              ! ATOVS

! 1.3       Create 1D header record from store
            CALL PPHOUTH ( LAMSA, LAMSB )

! 1.4       Write out 1D header record
            H1Drecord = 1
            CALL IOH1D ("WRITE HEADER", H1Dunit, H1Drecord)

         ENDIF  ! TOVS

      ENDIF   ! (GHIRS)


! 2.  Write AMSU-A header
!     ----- ------ ------

      IF (GAMSA) THEN     ! output on AMSU-A grid required

! 2.1    Create 1D header record from store
         CALL PPAOUTH ( LAMSB )

! 2.2    Write out 1D header record
         A1Drecord = 1
         CALL IOA1D ("WRITE HEADER", A1Dunit, A1Drecord)

      ENDIF   ! (GAMSA)


! 3.  Write AMSU-B header
!     ----- ------ ------

      IF (GAMSB) THEN     ! output on AMSU-B grid required

! 3.1    Create 1D header record from store
         CALL PPBOUTH ( LAMSA )

! 3.2    Write out 1D header record
         B1Drecord = 1
         CALL IOB1D ("WRITE HEADER", B1Dunit, B1Drecord)

      ENDIF   ! (GAMSB)


! 4.  Write IASI header
!     ----- ---- ------

      IF (GIASI) THEN     ! output on IASI grid required

! 4.1    Create 1D header record from store
         IF (LPCIASI) THEN
           CALL PPIPCOUTH ( LAMSA, LAMSB )
         ELSE
           CALL PPIOUTH ( LAMSA, LAMSB )
         ENDIF

! 4.2    Write out 1D header record
         I1Drecord = 1
         CALL IOI1D ("WRITE HEADER", I1Dunit, I1Drecord)

      ENDIF   ! (GIASI)


! 5.  Write ATMS header
!     ----- ---- ------

      IF (GATMS) THEN     ! output on ATMS grid required

! 5.1    Create 1D header record from store
         CALL PPATOUTH

! 5.2    Write out 1D header record
         atms1Drecord = 1
         CALL IOAT1D ("WRITE HEADER", atms1Dunit, atms1Drecord)

      ENDIF   ! (GIASI)


! 6.  Write CRIS header
!     ----- ---- ------

      IF (GCRIS) THEN     ! output on CRIS grid required

! 6.1    Create 1D header record from store
         CALL PPCOUTH ( LATMS )

! 6.2    Write out 1D header record
         cris1Drecord = 1
         CALL IOC1D ("WRITE HEADER", cris1Dunit, cris1Drecord)

      ENDIF   ! (GCRIS)

! 6.  Write HIRAS header
!     ----- ---- ------

      IF (GHIRAS) THEN     ! output on HIRAS grid required

! 6.1    Create 1D header record from store
         CALL PPHIRASOUTH ( .false. )     !No LMWTS at present

! 6.2    Write out 1D header record
         hiras1Drecord = 1
         CALL IOHIRAS1D ("WRITE HEADER", hiras1Dunit, hiras1Drecord)

      ENDIF   ! (GHIRAS)


      RETURN
      END
