!+ pre-process AMSU on HIRS grid
!
      SUBROUTINE PPHAMSU ( HLastLine )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: pphamsu.F 257 2011-03-31 08:55:52Z frna $
!
! Purpose:
!   pre-process a block of AMSU data mapped to HIRS grid
!
! Method:
! 1.  Test for precipitation (PPHAPCP)
! 2.  Estimate surface type from the data (PPHASURF)
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      07/09/98  Original code. (R.J.Renshaw)
!  1.1      31/03/11  Since AMSU-B is not used in the surface test, comment
!                     out the requirement for it. NCA
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"         
! parameters and look-up tables for the mapping
#include "ppluts.h"          
! fixed data for processing HIRS
#include "pphfdata.h"        
! block of HIRS level 1d data
#include "pphstore.h"        
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"     
!
!   Scalar arguments with intent(In):
      INTEGER       HLastLine     ! number of last HIRS line in store
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
!
! Function & Subroutine calls:
      External ErrorReport        ! ATOVPP error routine
      External PPHAPCP            ! test for precipitation, etc.
      External PPHASURF           ! estimate surface type from the data
!
!- End of header

! 1.  test for precipitation
!     ---- --- -------------

      CALL PPHAPCP ( HLastLine )

! 2.  Estimate surface type from the data
!     -------- ------- ---- ---- --- ----

!    (uses channels from AMSU-A and AMSU-B)
!      IF ( BTEST(Hinstruments,bits(4)) ) THEN

         CALL PPHASURF ( HLastLine )

!      ELSE
!         errStatus = -21
!         errMessage =
!     &   "Cannot guess AMSU-A surface type without mapped AMSU-B"
!         CALL ErrorReport(errStatus,errMessage,"PPHAMSU")
!      ENDIF

      RETURN
      END
