!+ Read fixed data file for HIRS within ATOVPP
!
      SUBROUTINE PPHFDF ( Hfunit, TOVS )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: pphfdf.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Read fixed data file required for HIRS within ATOVPP.
!
! Method:
! 0.  initialisation
! 1.  open file
! 2.  look for sections in the file
! 3.  read numbers for the pre-processing (PREPRO) (nothing as yet)
! 4.  read bias corrections (BIAS)
! 5.  close file
!
!   File format is specified in AAPP documentation APP204F, vn of 27/11/96
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      09/03/04  Modify for Linux (N Atkinson / A Korpela)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  fixed data for processing HIRS 
#include "pphfdata.h"
!
! Subroutine arguments:
!   Scalar arguments with intent(in)
      INTEGER       Hfunit        ! unit number for HIRS-specific data
      LOGICAL       TOVS          ! TRUE if instrument is TOVS, else FALSE
!
! Local Scalars:
      CHARACTER*80  input_line    ! line read from instream data
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status value
      INTEGER       chn           ! loop index: channel number
      INTEGER       fov           ! loop index: fov number

! Function & Subroutine calls:
      External ErrorReport        ! ATOVPP error routine

!- End of header


! 0.  initialisation
!     --------------

! 0.1 bias corrections
      DO fov = 1, Hnfovs
      DO chn = 1, Hnchan
         Hbiases(chn,fov) = 0.
      ENDDO  ! chn
      ENDDO  ! fov


! 1.  open file
!     ---- ----

      OPEN ( UNIT=Hfunit, STATUS="OLD", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus )

!     If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
      IF ( errStatus.NE.0 )
     &  OPEN ( UNIT=Hfunit, STATUS="UNKNOWN", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus )

      WRITE ( errMessage,
     & "("" Error opening HIRS fixed data file, unit "",I8)" ) Hfunit
      errStatus = ABS (errStatus)   ! end-of-file returns -1
      CALL ErrorReport(errStatus,errMessage,"PPHFDF")


! 2.  look for sections in the file
!     ---- --- -------- -- --- ----

      DO WHILE ( errStatus.EQ.0 .AND. input_line(1:3).NE."END" )
         input_line = " "
         READ(Hfunit,"(A80)",IOSTAT=errStatus) input_line


         IF ( input_line(1:6) .EQ. "PREPRO" ) THEN

! 3.        read numbers for the pre-processing
!           ---- ------- --- --- --------------

            errMessage = "Reading PREPRO section"
            errStatus = 0
!           (as yet there are no numbers to read)
            errStatus = ABS (errStatus)   ! end-of-file returns -1
            CALL ErrorReport(errStatus,errMessage,"PPHFDF")


         ELSEIF ( input_line(1:4) .EQ. "BIAS" ) THEN

! 4.        read bias-corrections
!           ---- ----------------

!           read scan-dependent bias corrections
            errMessage = "Reading BIAS section"
            errStatus = 0
            DO fov = 1, Hnfovs
               IF ( errStatus.EQ.0 ) THEN
                  READ(Hfunit,"(10F8.3)",IOSTAT=errStatus)
     &                       (Hbiases(chn,fov),chn=1,Hnchan)
               ENDIF
            ENDDO  ! fov
            errStatus = ABS (errStatus)   ! end-of-file returns -1
            CALL ErrorReport(errStatus,errMessage,"PPHFDF")


         ENDIF  ! input_line

      ENDDO  ! WHILE ( errStatus.EQ.0 )


! 5.  close file
!     ----- ----

      CLOSE(Hfunit)


      RETURN
      END
