!+ pre-process HIRAS before mapping
!
      SUBROUTINE PPHIRAS1 ( hirasLastLine, LSunit, HTunit )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: pphiras1.F 618 2019-02-12 10:54:03Z frna $
!
! Purpose:
!   pre-process a block of level 1c HIRAS data
!
! Method:
! 1.  Get land/sea information & elevation (SURFELEV)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0   12/02/2019  From ppcris1.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  HIRAS 1d format 
#include "hiras1d.h"
!  fixed data for processing HIRAS 
#include "pphirasfdata.h"
!  block of HIRAS data 
#include "pphirasstore.h"
!
!   Scalar arguments with intent(In):
      INTEGER       hirasLastLine  ! number of last HIRAS line in store
!
      INTEGER       HTunit        ! unit no. for heights dataset
      INTEGER       LSunit        ! unit no. for land/sea bitmap
!
! Local scalars:
      INTEGER       line          ! loop index: pointer to scanline in buffer
      INTEGER       fov           ! loop index: fov number
!
! Local arrays:
      REAL          lats(hirasnfovs)  ! array of latitudes for a scanline
      REAL          lons(hirasnfovs)  ! array of longitudes for a scanline
      INTEGER       surftype(hirasnfovs) !surface type for a scanline (SURFELEV)
      INTEGER       elevation(hirasnfovs)!elevation for a scanline (SURFELEV)
      INTEGER       landfrac(hirasnfovs) !land fraction for a scanline (SURFELEV)
!
! Function & Subroutine calls:
      External SURFELEV           ! get land/sea information & elevation
!
!- End of header

      WRITE(*,"(10X,""Pre-processing HIRAS before the mapping"")")

! 1.  Get land/sea information
!     --- -------- -----------

!     loop through scanlines not already processed
      DO line = 1, hirasLastLine

! 1.1    pick out lat's and lon's from scanline
         DO fov = 1, hirasnfovs
            lats(fov) = hiraslats(fov,line)        ! degrees N
            lons(fov) = hiraslons(fov,line)        ! degrees E
         ENDDO  ! fov

! 1.2    find surface type for each fov
         CALL SURFELEV ( LSunit, HTunit,
     &                   lats, lons, hirasnfovs, hirasboxSize,
     &                   surftype, elevation, landfrac )

! 1.3    store surface type & elevation
         DO fov = 1, hirasnfovs
            hirassurftype(fov,line) = surftype(fov)
            hiraselev(fov,line) = elevation(fov)
            hiraslandfrac(fov,line) = landfrac(fov)
         ENDDO  ! fov

      ENDDO  ! line

      RETURN
      END
