!+ Copy level 1C HIRAS data to level 1D common area
!
      SUBROUTINE PPHIRAS1D ( hirasrec, line )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: pphiras1d.F 618 2019-02-12 10:54:03Z frna $
!
! Purpose:
!   Copy HIRAS data from hiras1c_dt to hiras1d_dt, applying spatial
!   thinning and channel selection as specified by user. Call the routine to
!   compute PC scores, if required.
!
! Method:
!   Channel selection is provided in common ppifdata. If spatial thinning is
!   specified, the best detector for each FOV is obtained from ppistore.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0   08/02/2019  From ppi1d.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  HIRAS level 1C header/data common blocks 
#include "hiras1c.h"
!  HIRAS level 1D header/data common blocks 
#include "hiras1d.h"
!  block of HIRAS data 
#include "pphirasstore.h"
!  fixed data 
#include "pphirasfdata.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
!   Scalar arguments with intent(In):

      INTEGER hirasrec   !HIRAS record number 1 to 4 (for full spatial res mode)
      INTEGER line       !Scan line in pphirasstore

!   Local scalars:

      INTEGER Ispot, Iclass, Ichan, I, Ifor, Ipc      !Loop indices
      INTEGER BestDet    !Chosen detector for each FOV
      REAL    Recon
      LOGICAL First
      DATA First/.TRUE./
      INTEGER Cloudchan      !HIRAS channel number for cloud test
      INTEGER Mindiff, chandiff
      INTEGER channels_index (hirasnchan)
      DATA    channels_index /hirasnchan*0/
      
! Find the channel closest to nominal wavenumber for cloud tests
      IF (First) THEN
        hirasCloudW = 2390.0       !wavenumber. This is in band 3.
        Mindiff = 99999
        
        Cloudchan = (hirasCloudW - 0.01*hiras1c_h_startw(3))/
     &      (0.01*hiras1c_h_endw(3) - 0.01*hiras1c_h_startw(3)) *
     &      (hiras1c_h_endchan(3) - hiras1c_h_startchan(3)) + 
     &       hiras1c_h_startchan(3)
        DO Ichan=1,hiras1c_h_nchanused
          chandiff = ABS(hiras1c_h_channels(Ichan) - Cloudchan)
          IF (chandiff .LT. Mindiff) THEN
            Mindiff = chandiff 
            hirasCloudindex = Ichan
          ENDIF
        ENDDO

! Create index to the level 1C channels
        DO Ichan=1,hiras1c_h_nchanused
          channels_index(hiras1c_h_channels(Ichan)) = Ichan
        ENDDO

        First = .false.
      ENDIF
!
!- End of header
      
      hiras1d_scnlin = hiras1c_scnlin
      hiras1d_scnlinyr = hiras1c_scnlinyr
      hiras1d_scnlindy = hiras1c_scnlindy
      hiras1d_satxyz(1) = hiras1c_satxyz(1)
      hiras1d_satxyz(2) = hiras1c_satxyz(2)
      hiras1d_satxyz(3) = hiras1c_satxyz(3)
      hiras1d_ascdesc = hiras1c_ascdesc
      hiras1d_orbit = hiras1c_orbit
      hiras1d_scalti = hiras1c_scalti
      hiras1d_scanqualflgs = hiras1c_scanqualflgs
      hiras1d_geolocqual = hiras1c_geolocqual

      DO Ifor=1,hiras_1dnfor
      
        IF (thinmode_hiras .GE. 2) THEN
          BestDet = hirasbestFOV(Ifor,line)   !intelligent thinning
        ELSE
          BestDet = detector_use_hiras(hirasrec) !fixed thinning
        ENDIF
         
        hiras1d_FOV(Ifor) = (Ifor-1)*4 + BestDet
        hiras1d_scnlintime(Ifor) = hiras1c_scnlintime(Ifor)

        DO I=1,2
          hiras1d_latlon(I,Ifor)=hiras1c_latlon(I,BestDet,Ifor)
        enddo
        DO I=1,4
          hiras1d_angles(I,Ifor)=hiras1c_angles(I,BestDet,Ifor)
        ENDDO
        
        hiras1d_surfht(Ifor) = hiras1c_surfht(BestDet,Ifor)
        hiras1d_landfrac(Ifor) = hiras1c_landfrac(BestDet,Ifor)
        hiras1d_landsea(Ifor) = hiras1c_landsea(BestDet,Ifor)
        hiras1d_cloudcover(Ifor) =
     &       hiras1c_cloudcover(BestDet,Ifor)
        hiras1d_cloudtopht(Ifor) =
     &       hiras1c_cloudtopht(BestDet,Ifor)
        hiras1d_radianceflgs(Ifor) =
     &       hiras1c_radianceflgs(BestDet,Ifor)

        DO I=1,hiras_1cnband
          hiras1d_calqualflgs(I,Ifor) =
     &       hiras1c_calqualflgs(I,BestDet)
          hiras1d_fovqualflgs(I,Ifor) =
     &       hiras1c_fovqualflgs(I,BestDet,Ifor)
        ENDDO
        hiras1d_quality(Ifor) = hiras1c_quality(BestDet,Ifor)

        DO Ichan=1,nchan_hiras             !HIRAS radiances (selected channels)
          I = channels_index(channels_hiras(Ichan))
          IF (I .NE. 0) THEN
            hiras1d_radiance(Ifor,Ichan)=
     &         hiras1c_radiance(BestDet,Ifor,I)
          ELSE
            hiras1d_radiance(Ifor,Ichan)= MDI
          ENDIF
        ENDDO

!Store 2390 cm^-1 scaled radiance for use in cloud test 2 (see ppicloud)
        hirasCloudflag(2,Ifor,line) = 
     &            hiras1c_radiance(BestDet,Ifor,hirasCloudindex)
        
        DO I=1,10
          hiras1d_cloudflag(I,Ifor)=0     !Initialise cloud test flags
        ENDDO
        
!PC code not used (see ppcris1d.F)

      ENDDO   !Ifor
      
      CALL PPHIRASCLOUD(1, line)    !Run first cloud test (sdev of 4 HIRAS spots)
      
      END
