!+ pre-process HIRS before mapping
!
      SUBROUTINE PPHIRS1 ( HLastLine, LSunit, HTunit )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: pphirs1.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   pre-process a block of level 1c HIRS data
!
! Method:
! 1.  Add bias corrections (BIASCORR)
! 2.  Get land/sea information & elevation (SURFELEV)
! 3.  Correct for limb effect and surface emissivity (PPHCORR)
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      16/01/97  Bug fixed: lat & lon not now multiplied by 0.01
!  1.2      15/03/05  Add LandFrac in call to surfelev
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  fixed data for processing HIRS 
#include "pphfdata.h"
!  block of HIRS data 
#include "pphstore.h"
!
!   Scalar arguments with intent(In):
      INTEGER       HLastLine     ! number of last HIRS line in store
!
      INTEGER       HTunit        ! unit no. for heights dataset
      INTEGER       LSunit        ! unit no. for land/sea bitmap
!
! Local scalars:
      INTEGER       line          ! loop index: pointer to scanline in buffer
      INTEGER       fov           ! loop index: fov number
!
! Local arrays:
      REAL          lats(Hnfovs)  ! array of latitudes for a scanline
      REAL          lons(Hnfovs)  ! array of longitudes for a scanline
      INTEGER       surftype(Hnfovs) !surface type for a scanline (SURFELEV)
      INTEGER       elevation(Hnfovs)!elevation for a scanline (SURFELEV)
      INTEGER       LandFrac(Hnfovs) !Land fraction for a scanline (SURFELEV)
!
! Function & Subroutine calls:
      External BIASCORR           ! scan-dependent bias corrections
      External SURFELEV           ! get land/sea information & elevation
      External PPHCORR            ! Correct for limb effect and surface emiss
!
!- End of header

      WRITE(*,"(10X,""Pre-processing HIRS before the mapping"")")

! 1.  Add bias corrections
!     --- ---- -----------

      CALL BIASCORR ( Hbiases, Hnchan, Hnchan, Hnfovs,
     &                HLastLine, Hbtemps )


! 2.  Get land/sea information
!     --- -------- -----------

!     loop through scanlines not already processed
      DO line = 1, HLastLine

! 2.1    pick out lat's and lon's from scanline
         DO fov = 1, Hnfovs
            lats(fov) = Hlats(fov,line)        ! degrees N
            lons(fov) = Hlons(fov,line)        ! degrees E
         ENDDO  ! fov

! 2.2    find surface type for each fov
         CALL SURFELEV ( LSunit, HTunit,
     &                   lats, lons, Hnfovs, HboxSize,
     &                   surftype, elevation, LandFrac )

! 2.3    store surface type & elevation
         DO fov = 1, Hnfovs
            Hsurftype(fov,line) = surftype(fov)
            Helev(fov,line) = elevation(fov)
         ENDDO  ! fov

      ENDDO  ! line


! 3.  Correct for limb effect and surface emissivity
!     ------- --- ---- ------ --- ------- ----------

      CALL PPHCORR ( HLastLine )


      RETURN
      END
