!+ pre-process HIRS after mapping
!
      SUBROUTINE PPHIRS2 ( HLastLine, LAMSA, LAMSB, LMSU )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: pphirs2.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   pre-process a block of level 1d HIRS data, after mapping
!
! Method:
! 1.  Add back MSU limb-darkening curve
! 2.  Test for cloud (PPHCLOUD)
! 3.  Repeat pre-processing tests for AMSU but on HIRS grid
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  fixed data for processing HIRS 
#include "pphfdata.h"
!  fixed data for processing MSU 
#include "ppmfdata.h"
!  block of HIRS data 
#include "pphstore.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
!   Scalar arguments with intent(In):
      INTEGER       HLastLine     ! number of last HIRS line in store
!
      LOGICAL       LAMSA         ! .TRUE. to process AMSU-A
      LOGICAL       LAMSB         ! .TRUE. to process AMSU-B
      LOGICAL       LMSU          ! .TRUE. to process MSU
!
! Local scalars:
      INTEGER       chn           ! loop variable over all channels
      INTEGER       line          ! loop index: pointer to scanline in buffer
      INTEGER       fov           ! loop index: fov number
      INTEGER       fov2          ! fov number going from 1-28 and then 28-1
      INTEGER       isurf         ! surface type: 1=sea, 2=not sea
!
! Local arrays:
!
! Function & Subroutine calls:
      External PPHCLOUD           ! test for cloud (after the mapping)
      External PPHAMSU            ! AMSU pre-pro tests on HIRS grid
!
!- End of header

      WRITE(*,"(10X,""Pre-processing HIRS after the mapping"")")

! 1.  Add back MSU limb-darkening curve
!     --- ---- --- -------------- -----

      IF ( BTEST(Hinstruments,bits(1)) ) THEN  !  MSU present on HIRS grid
!        addition is on HIRS grid
         DO line = 1, HLastLine
         DO fov = 1, Hnfovs
            IF ( .NOT.BTEST(Hmapqc(fov,line),bits(0)) ) THEN ! fov has values
               IF ( Hsurftype(fov,line).EQ.0 ) THEN
                 isurf = 1  ! sea
               ELSE
                 isurf = 2  ! non-sea
               ENDIF
               IF ( fov.LE.28 ) THEN  ! we only define limb curve for fovs 1-28
                  fov2 = fov
               ELSE
                  fov2 = 57 - fov
               ENDIF
               DO chn = 1, Mnchan
                  Hmapbt(chn,fov,line) = Hmapbt(chn,fov,line)
     &                                 + Mlimb(chn,fov2,isurf)
               ENDDO  ! chn
            ENDIF
         ENDDO  ! fov
         ENDDO  ! line
      ENDIF  ! MSU present


! 2.  Test for cloud
!     ---- --- -----

      CALL PPHCLOUD ( HLastLine, LAMSA, LAMSB, LMSU )


! 3.  Repeat pre-processing tests for AMSU but on HIRS grid
!     ------ -------------- ----- --- ---- --- -- ---- ----

      IF ( LAMSA ) CALL PPHAMSU ( HLastLine )

      RETURN
      END
