!+ Copy level 1C IASI data to level 1D common area
!
      SUBROUTINE PPI1D ( Irec, line )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppi1d.F 215 2010-06-07 10:18:29Z frna $
!
! Purpose:
!   Copy IASI and AVHRR data from iasi1c_dt to iasi1d_dt, applying spatial
!   thinning and channel selection as specified by user. Call the routine to
!   compute PC scores, if required.
!
! Method:
!   Channel selection is provided in common ppifdata. If spatial thinning is
!   specified, the best detector for each FOV is obtained from ppistore.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0     28/01/05  N.C.Atkinson
!  1.1     28/02/07  Correct dimensioning of iasi1d_cloudflag, and add call to
!                    PPICLOUD. NCA
!  1.2     11/11/08  Pass the correct channel to ppicloud (was 2400cm-1in error)
!  1.3     18/05/09  Option to produce reconstructed radiances
!  1.4     07/10/10  Store the EUMETSAT cloud and land fractions in ICloudflag
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  IASI level 1C header/data common blocks 
#include "iasi1c.h"
!  IASI level 1D header/data common blocks 
#include "iasi1d.h"
!  block of IASI data 
#include "ppistore.h"
!  fixed data 
#include "ppifdata.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
!   Scalar arguments with intent(In):

      INTEGER Irec       !IASI record number 1 to 4 (for full spatial res mode)
      INTEGER line       !Scan line in ppistore

!   Local scalars:

      INTEGER Ispot, Iclass, Ichan, I, Ifov, Ipc      !Loop indices
      INTEGER BestDet    !Chosen detector for each FOV
      REAL    Recon
      LOGICAL First
      DATA First/.TRUE./
      INTEGER Cloudchan
      DATA Cloudchan / 6981 /
!
!- End of header
      
      iasi1d_scnlin = iasi1c_scnlin
      iasi1d_scnlinyr = iasi1c_scnlinyr
      iasi1d_scnlindy = iasi1c_scnlindy
      iasi1d_GQisQualIndex = iasi1c_GQisQualIndex
      iasi1d_GQisQualIndexLoc = iasi1c_GQisQualIndexLoc 
      iasi1d_GQisQualIndexRad = iasi1c_GQisQualIndexRad
      iasi1d_GQisQualIndexSpect = iasi1c_GQisQualIndexSpect
      iasi1d_GQisSysTecSondQual = iasi1c_GQisSysTecSondQual
      iasi1d_avhcombination = iasi1c_avhcombination
      iasi1d_scalti = iasi1c_scalti
      iasi1d_avhcombination = iasi1c_avhcombination

      BestDet = Irec

      DO Ifov=1,iasi_1dnfov
      
        IF (nrec_IASI .EQ. 1) BestDet = IbestFOV(Ifov,line)   !1d thinned

         
        iasi1d_FOV(Ifov) = (Ifov-1)*4 + BestDet          !1 to 120
        iasi1d_scnlintime(Ifov) = iasi1c_scnlintime(Ifov)
        iasi1d_GQisFlagQual(Ifov) = iasi1c_GQisFlagQual(BestDet,Ifov)

        DO I=1,2
          iasi1d_latlon(I,Ifov) = iasi1c_latlon(I,BestDet,Ifov)
        enddo
        DO I=1,4
          iasi1d_angles(I,Ifov) = iasi1c_angles(I,BestDet,Ifov)
        ENDDO

        DO Ichan=1,nchan_IASI             !IASI radiances (selected channels)
          iasi1d_scalrad(Ichan,Ifov) = 
     &                 iasi1c_scalrad(channels_IASI(Ichan),BestDet,Ifov)
        ENDDO

!Store 2390 cm^-1 scaled radiance for use in cloud test 2 (see ppicloud)
        ICloudflag(2,Ifov,line) = 
     &            iasi1c_scalrad(Cloudchan,BestDet,Ifov)
        
!Store the EUMETSAT cloud and surface type values and set the header flag if
!value is not "missing"
        ICloudflag(3,Ifov,line) = 
     &            iasi1c_GEUMAvhrr1BCldFrac(BestDet,Ifov)
        IF (ICloudflag(3,Ifov,line).NE.255) 
     &            iasi1d_h_cloudtests = IBSET(iasi1d_h_cloudtests,2)
        ICloudflag(4,Ifov,line) = 
     &            iasi1c_GEUMAvhrr1BLandFrac(BestDet,Ifov)
        IF (ICloudflag(4,Ifov,line).NE.255) 
     &            iasi1d_h_cloudtests = IBSET(iasi1d_h_cloudtests,3)


        DO Iclass=1,7                     !AVHRR data
          iasi1d_avhrrfrac(Iclass,Ifov) = 
     &      iasi1c_avhrrfrac(Iclass,BestDet,Ifov)
          iasi1d_avhrrCGy(Iclass,Ifov) = 
     &      iasi1c_avhrrCGy(Iclass,BestDet,Ifov)
          iasi1d_avhrrCGz(Iclass,Ifov) = 
     &      iasi1c_avhrrCGz(Iclass,BestDet,Ifov)

          DO Ichan=1,6
            iasi1d_avhrrmean(Ichan,Iclass,Ifov) = 
     &              iasi1c_avhrrmean(Ichan,Iclass,BestDet,Ifov)
            iasi1d_avhrrsdev(Ichan,Iclass,Ifov) = 
     &              iasi1c_avhrrsdev(Ichan,Iclass,BestDet,Ifov)
            iasi1d_avhmeanscalefactor(Ichan,Iclass,Ifov) = 
     &              iasi1c_avhmeanscalefactor(Ichan,Iclass,BestDet,Ifov)
            iasi1d_avhsdevscalefactor(Ichan,Iclass,Ifov) = 
     &              iasi1c_avhsdevscalefactor(Ichan,Iclass,BestDet,Ifov)
          ENDDO   !Ichan
        ENDDO   !Iclass

        DO I=1,10
          iasi1d_cloudflag(I,Ifov)=0     !Initialise cloud test flags
        ENDDO
        
        IF (npc_IASI .GT. 1) THEN
          CALL PPISPECTRA (BestDet,Ifov)       !Generate PC scores

!Produce reconstructed radiances if user has requested it, over-writing the raw
!radiance

          IF (Use_reconstructed_radiances) THEN
            DO Ichan=1,nchan_IASI
              Recon = mean_decode_IASI(channels_IASI(Ichan))
              DO Ipc=1,npc_IASI
                Recon = Recon+eig_decode_IASI(channels_IASI(Ichan),Ipc)
     &                       * iasi1d_scores(Ipc,Ifov)/2.0
              ENDDO
              IF (Recon .LT. 0.0) Recon = 0.0  !Expect positive recon radiances
              iasi1d_scalrad(Ichan,Ifov) = 
     &            NINT(Recon/Scalefactors(channels_IASI(Ichan)))
            ENDDO
          ENDIF    !Use_reconstructed_radiances

        ENDIF    !npc_IASI .GT. 1

      ENDDO   !Ifov
      
      CALL PPICLOUD(1, line)    !Run first cloud test (sdev of 4 IASI spots)
      
      END
