!+ pre-process AMSU on IASI grid
!
      SUBROUTINE PPIAMSU ( ILastLine )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppiamsu.F 257 2011-03-31 08:55:52Z frna $
!
! Purpose:
!   pre-process a block of AMSU data mapped to IASI grid
!
! Method:
! 1.  Test for precipitation (PPIAPCP)
! 2.  Estimate surface type from the data (PPIASURF)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0     11/08/05  Based on pphamsu.F (N.C.Atkinson)
!  1.1     20/03/07  Add call to PPICLOUD
!  1.2     31/03/11  Since AMSU-B is not used in the surface test, comment
!                    out the requirement for it. NCA
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  parameters and look-up tables for the mapping 
#include "ppluts.h"
!  IASI level 1d format 
#include "iasi1d.h"
!  fixed data for processing IASI 
#include "ppifdata.h"
!  block of IASI level 1d data 
#include "ppistore.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
!   Scalar arguments with intent(In):
      INTEGER       ILastLine     ! number of last IASI line in store
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
!
! Function & Subroutine calls:
      External ErrorReport        ! ATOVPP error routine
      External PPIAPCP            ! test for precipitation, etc.
      External PPIASURF           ! estimate surface type from the data
!
!- End of header

! 1.  test for precipitation
!     ---- --- -------------

      CALL PPIAPCP ( ILastLine )

! 2.  Estimate surface type from the data
!     -------- ------- ---- ---- --- ----

!    (uses channels from AMSU-A and AMSU-B)
!      IF ( BTEST(Iinstruments,bits(4)) ) THEN

         CALL PPIASURF ( ILastLine )

!      ELSE
!         errStatus = -21
!         errMessage =
!     &   "Cannot guess AMSU-A surface type without mapped AMSU-B"
!         CALL ErrorReport(errStatus,errMessage,"PPIAMSU")
!      ENDIF

! 3.  Run cloud test(s) that make use of AMSU data
!     --------------------------------------------

      CALL PPICLOUD(2, ILastLine)

      RETURN
      END
