!+ Look for precipitation signal in AMSU-A on IASI grid
!
      SUBROUTINE PPIAPCP ( ILastLine )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppiapcp.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Look for precipitation signal in AMSU-A mapped to IASI grid
!   and set flags accordingly.
!
! Method:
! 0.  Loop through scanlines and fields-of-view
! 1.  Scattering test
! 2.  Crosby, Ferraro & Wu test
! 3.  Grody light rainfall test
! 4.  Set flags
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0     11/08/05 Derived from subroutine pphapcp. (N.C.Atkinson)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  fixed data for processing AMSU-A 
#include "ppafdata.h"
!  IASI 1d format 
#include "iasi1d.h"
!  block of IASI level 1d data 
#include "ppistore.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
!   Scalar arguments with intent(In):
      INTEGER       ILastLine     ! number of last HIRS line in store
!
! Local scalars:
      INTEGER       line          ! loop index: pointer to scanline in buffer
      INTEGER       fov           ! loop index: fov number
      INTEGER       Afov          ! nearest AMSU-A fov to HIRS fov
      REAL          T1            ! brightness temp in AMSU-A channel 1 (K)
      REAL          T2            ! brightness temp in AMSU-A channel 2 (K)
      REAL          T3            ! brightness temp in AMSU-A channel 3 (K)
      REAL          T15           ! brightness temp in AMSU-A channel 15 (K)
      REAL          SI            ! scattering index
      REAL          probprecip    ! Crosby probability of precipitation
      LOGICAL       scat          ! .TRUE. if scattering detected
      LOGICAL       crosby        ! .TRUE. if Crosby test flags fov
      LOGICAL       grody         ! .TRUE. if Grody test flags fov
!
! Local arrays:
      INTEGER NearestAfov(Infovs) ! nearest AMSU-A fov to each IASI fov
      DATA NearestAfov / 1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
     &                  16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,
     &                   1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
     &                  16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,
     &                   1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
     &                  16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,
     &                   1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
     &                  16,17,18,19,20,21,22,23,24,25,26,27,28,29,30/
!
! Function & Subroutine calls:
      External PPASCAT            ! scattering index
          REAL PPASCAT
      External PPCROSBY           ! logistic regression
          REAL PPCROSBY
      External PPGRODY            ! light rainfall test
       LOGICAL PPGRODY
!
!- End of header

! 0.  Loop through scanlines and fields-of-view
!     ---- ------- --------- --- --------------

      DO line = 1, ILastLine
      DO fov = 1, Infovs

         Afov = NearestAfov(fov)
         T1  = Imapbt(1,fov,line)
         T2  = Imapbt(2,fov,line)
         T3  = Imapbt(3,fov,line)
         T15 = Imapbt(15,fov,line)

! 1.     Scattering test  (only over the sea)
!        ---------- ----

!        only do if channels present
         IF (  T1 .GT. RMDI  .AND.  T2 .GT. RMDI
     &   .AND. T3 .GT. RMDI  .AND.  T15 .GT. RMDI ) THEN
            SI = PPASCAT ( T1, T2, T3, T15, Afov )
            Iscatindx(fov,line) = SI
            IF ( Isurftype(fov,line).EQ.0 ) THEN
               scat = SI .GT. Amaxscat
            ELSE
               scat = .FALSE.
            ENDIF
         ELSE
            Iscatindx(fov,line) = RMDI
            scat = .TRUE.  ! default is to err on side of caution
         ENDIF


! 2.     Crosby, Ferraro & Wu test
!        ------  ------- - -- ----

         IF (  T1 .GT. RMDI  .AND.  T15 .GT. RMDI ) THEN
            probprecip = PPCROSBY(T1,T15 )
            Icrosby(fov,line) = probprecip
            crosby = probprecip .GT. Amaxcrosby
         ELSE
            Icrosby(fov,line) = RMDI
            crosby = .TRUE.  ! default is to err on side of caution
         ENDIF


! 3.        Grody light rainfall test
!           ----- ----- -------- ----

         IF (  T1 .GT. RMDI  .AND.  T2 .GT. RMDI ) THEN
            grody = PPGRODY( T1, T2 )
         ELSE
            grody = .TRUE.  ! default is to err on side of caution
         ENDIF


! 4.     Set flags
!        --- -----

         IF ( scat )
     &      Imapqc(fov,line) = IBSET ( Imapqc(fov,line), bits(1) )
         IF ( crosby )
     &      Imapqc(fov,line) = IBSET ( Imapqc(fov,line), bits(25) )
         IF ( grody )
     &      Imapqc(fov,line) = IBSET ( Imapqc(fov,line), bits(24) )

      ENDDO  ! fov
      ENDDO  ! line


      RETURN
      END
