!+ pre-process IASI before mapping
!
      SUBROUTINE PPIASI1 ( ILastLine, LSunit, HTunit )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppiasi1.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   pre-process a block of level 1c IASI data
!
! Method:
! 1.  Get land/sea information & elevation (SURFELEV)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0     18/01/05  N.C.Atkinson
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  IASI 1d format 
#include "iasi1d.h"
!  fixed data for processing IASI 
#include "ppifdata.h"
!  block of IASI data 
#include "ppistore.h"
!
!   Scalar arguments with intent(In):
      INTEGER       ILastLine     ! number of last IASI line in store
!
      INTEGER       HTunit        ! unit no. for heights dataset
      INTEGER       LSunit        ! unit no. for land/sea bitmap
!
! Local scalars:
      INTEGER       line          ! loop index: pointer to scanline in buffer
      INTEGER       fov           ! loop index: fov number
!
! Local arrays:
      REAL          lats(Infovs)  ! array of latitudes for a scanline
      REAL          lons(Infovs)  ! array of longitudes for a scanline
      INTEGER       surftype(Infovs) !surface type for a scanline (SURFELEV)
      INTEGER       elevation(Infovs)!elevation for a scanline (SURFELEV)
      INTEGER       landfrac(Infovs) !land fraction for a scanline (SURFELEV)
!
! Function & Subroutine calls:
      External SURFELEV           ! get land/sea information & elevation
!
!- End of header

      WRITE(*,"(10X,""Pre-processing IASI before the mapping"")")

! 1.  Get land/sea information
!     --- -------- -----------

!     loop through scanlines not already processed
      DO line = 1, ILastLine

! 1.1    pick out lat's and lon's from scanline
         DO fov = 1, Infovs
            lats(fov) = Ilats(fov,line)        ! degrees N
            lons(fov) = Ilons(fov,line)        ! degrees E
         ENDDO  ! fov

! 1.2    find surface type for each fov
         CALL SURFELEV ( LSunit, HTunit,
     &                   lats, lons, Infovs, IboxSize,
     &                   surftype, elevation, landfrac )

! 1.3    store surface type & elevation
         DO fov = 1, Infovs
            Isurftype(fov,line) = surftype(fov)
            Ielev(fov,line) = elevation(fov)
         ENDDO  ! fov

      ENDDO  ! line

      RETURN
      END
