!+ Copy level PC IASI data to level 1D common area
!
      SUBROUTINE PPIPC1D ( Irec, line )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppipc1d.F 215 2010-06-07 10:18:29Z frna $
!
! Purpose:
!   Copy IASI and AVHRR data from iasiPc_dt to iasi1d_dt, applying spatial
!   thinning and channel selection as specified by user.
!
! Method:
!   Channel selection is provided in common ppifdata. If spatial thinning is
!   specified, the best detector for each FOV is obtained from ppistore.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0     27/10/08  N.C.Atkinson, created from ppi1d.F
!  1.1     07/10/10  Store the EUMETSAT cloud and land fractions in ICloudflag
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  IASI level PC header/data common blocks 
#include "iasipc.h"
!  IASI level 1C header/data common blocks 
#include "iasi1c.h"
!  IASI level 1D header/data common blocks 
#include "iasi1d.h"
!  block of IASI data 
#include "ppistore.h"
!  fixed data 
#include "ppifdata.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
!   Scalar arguments with intent(In):

      INTEGER Irec       !IASI record number 1 to 4 (for full spatial res mode)
      INTEGER line       !Scan line in ppistore

!   Local scalars:

      INTEGER Ispot, Iclass, Ichan, I, Ifov, Idet, Ipc      !Loop indices
      INTEGER BestDet    !Chosen detector for each FOV
      LOGICAL First
      DATA First/.TRUE./
      INTEGER Cloudchan
      DATA Cloudchan / 6981 /
      REAL Recon
!
!- End of header
      
!Need 2390 cm-1 for the cloud tests. See if this is already in the list of
!channels. If it isn't we will need to reconstruct it.

      IF (First) THEN
        First = .FALSE.
        ICloudchan = -1      !Index to this channel
        IF (Use_reconstructed_radiances) THEN 
          DO I=1,iasipc_h_nrchanused
            IF (iasipc_h_rchannels(I) .EQ. Cloudchan) ICloudchan = I
          ENDDO
        ELSE
          DO I=1,iasipc_h_nchanused
            IF (iasipc_h_channels(I) .EQ. Cloudchan) ICloudchan = I
          ENDDO
        ENDIF
        IF (ICloudchan.EQ.-1) THEN
          write(*,'(10X,A)') 'Reconstruct radiance for cloud tests'
        ELSE
          write(*,'(10X,A,I4)') 'Cloud channel index is ',ICloudchan
        ENDIF
      ENDIF

      iasi1d_scnlin = iasipc_scnlin
      iasi1d_scnlinyr = iasipc_scnlinyr
      iasi1d_scnlindy = iasipc_scnlindy
      iasi1d_GQisQualIndex = iasipc_GQisQualIndex
      iasi1d_GQisQualIndexLoc = iasipc_GQisQualIndexLoc 
      iasi1d_GQisQualIndexRad = iasipc_GQisQualIndexRad
      iasi1d_GQisQualIndexSpect = iasipc_GQisQualIndexSpect
      iasi1d_GQisSysTecSondQual = iasipc_GQisSysTecSondQual
      iasi1d_avhcombination = iasipc_avhcombination
      iasi1d_scalti = iasipc_scalti
      iasi1d_avhcombination = iasipc_avhcombination

      BestDet = Irec
      
!Main loop over FOVs

      DO Ifov=1,iasi_1dnfov
      
        IF (nrec_IASI .EQ. 1) BestDet = IbestFOV(Ifov,line)   !1d thinned

         
        iasi1d_FOV(Ifov) = (Ifov-1)*4 + BestDet          !1 to 120
        iasi1d_scnlintime(Ifov) = iasipc_scnlintime(Ifov)
        iasi1d_GQisFlagQual(Ifov) = iasipc_GQisFlagQual(BestDet,Ifov)

        DO I=1,2
          iasi1d_latlon(I,Ifov) = iasipc_latlon(I,BestDet,Ifov)
        enddo
        DO I=1,4
          iasi1d_angles(I,Ifov) = iasipc_angles(I,BestDet,Ifov)
        ENDDO

!Copy the raw or reconstructed radiances first to 1c array, then copy required
!channels to 1d. This allows for possible different channel selections.

        IF (Use_reconstructed_radiances) THEN         
          DO I=1,iasipc_h_nrchanused
            iasi1c_scalrad(iasipc_h_rchannels(I),BestDet,Ifov) = 
     &            iasipc_reconrad(I,BestDet,Ifov)
          ENDDO
        ELSE
          DO I=1,iasipc_h_nchanused
            iasi1c_scalrad(iasipc_h_channels(I),BestDet,Ifov) = 
     &            iasipc_scalrad(I,BestDet,Ifov)
          ENDDO
        ENDIF

        DO Ichan=1,nchan_IASI
          iasi1d_scalrad(Ichan,Ifov) = 
     &          iasi1c_scalrad(channels_IASI(Ichan),BestDet,Ifov)
        ENDDO

!Store scaled radiance for cloud tests in the 1c area. Use the channel selection
!if available, else do a reconstruction

        DO Idet=1,4
          IF (ICloudchan .GT. 0) THEN
            IF (Use_reconstructed_radiances) THEN
              iasi1c_scalrad(Cloudchan,Idet,Ifov) = 
     &            iasipc_reconrad(ICloudchan,IDet,Ifov)
            ELSE
              iasi1c_scalrad(Cloudchan,Idet,Ifov) = 
     &            iasipc_scalrad(ICloudchan,IDet,Ifov)
            ENDIF
          ELSE
            Recon = mean_decode_IASI(Cloudchan)
            DO Ipc=1,npc_IASI
              Recon = Recon + eig_decode_IASI(Cloudchan,Ipc)
     &                       * iasipc_scores(Ipc,Idet,Ifov)/2.0
            ENDDO
            IF (Recon .LT. 0.0) Recon = 0.0
            iasi1c_scalrad(Cloudchan,Idet,Ifov) = 
     &                NINT(Recon/Scalefactors(Cloudchan))
          ENDIF
        ENDDO

!Store scaled radiance for use in cloud test 2 (see ppicloud), which is done
!after all lines have been read in

        ICloudflag(2,Ifov,line) = iasi1c_scalrad(Cloudchan,BestDet,Ifov)
        
!Store the EUMETSAT cloud and surface type values and set the header flag if
!value is not "missing"
        ICloudflag(3,Ifov,line) = 
     &            iasipc_GEUMAvhrr1BCldFrac(BestDet,Ifov)
        IF (ICloudflag(3,Ifov,line).NE.255) 
     &            iasi1d_h_cloudtests = IBSET(iasi1d_h_cloudtests,2)
        ICloudflag(4,Ifov,line) = 
     &            iasipc_GEUMAvhrr1BLandFrac(BestDet,Ifov)
        IF (ICloudflag(4,Ifov,line).NE.255) 
     &            iasi1d_h_cloudtests = IBSET(iasi1d_h_cloudtests,3)

!AVHRR data
        DO Iclass=1,7                     !AVHRR data
          iasi1d_avhrrfrac(Iclass,Ifov) = 
     &      iasipc_avhrrfrac(Iclass,BestDet,Ifov)
          iasi1d_avhrrCGy(Iclass,Ifov) = 
     &      iasipc_avhrrCGy(Iclass,BestDet,Ifov)
          iasi1d_avhrrCGz(Iclass,Ifov) = 
     &      iasipc_avhrrCGz(Iclass,BestDet,Ifov)

          DO Ichan=1,6
            iasi1d_avhrrmean(Ichan,Iclass,Ifov) = 
     &              iasipc_avhrrmean(Ichan,Iclass,BestDet,Ifov)
            iasi1d_avhrrsdev(Ichan,Iclass,Ifov) = 
     &              iasipc_avhrrsdev(Ichan,Iclass,BestDet,Ifov)
            iasi1d_avhmeanscalefactor(Ichan,Iclass,Ifov) = 
     &              iasipc_avhmeanscalefactor(Ichan,Iclass,BestDet,Ifov)
            iasi1d_avhsdevscalefactor(Ichan,Iclass,Ifov) = 
     &              iasipc_avhsdevscalefactor(Ichan,Iclass,BestDet,Ifov)
          ENDDO   !Ichan
        ENDDO   !Iclass

        DO I=1,10
          iasi1d_cloudflag(I,Ifov)=0     !Initialise cloud test flags
        ENDDO
        
!PC scores
        IF (npc_IASI .GT. 1) THEN
          DO I=1,npc_IASI
            iasi1d_scores(I,Ifov) = iasipc_scores(I,BestDet,Ifov)
            IF (iasi1d_scores(I,Ifov).EQ.-32768) 
     &          iasi1d_scores(I,Ifov) = -999999
          ENDDO
        ENDIF

      ENDDO   !Ifov
      
      CALL PPICLOUD(1, line)    !Run first cloud test (sdev of 4 IASI spots)
      
      END
