!+ Map data between instrument grids
!
      SUBROUTINE PPMAP ( CMap, CTarg,
     &    HLastLine, ALastLine, BLastLine, MLastLine, ILastLine, IFOV )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppmap.F 45 2006-08-11 09:29:19Z frna $
!
! Purpose:
!   Map data from instrument grid to another
!
! Method:
! 1.  Map data to HIRS grid
! 2.  Map data to AMSU-A grid
! 3.  Map data to AMSU-B grid
! 4.  Map data to IASI grid
!
! Note that many of the mappings are commented out.
! Their inclusion would require other modifications to ATOVPP,
! such as the coding of appropriate pre-processing tests and of
! the necessary FOV-selection routines PPA2B,..etc.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      11/06/98  *cycle REAL not INTEGER
!  1.2      29/06/98  New option of mapping AMSU-A to AMSU-B
!  1.3      26/01/05  New option of mapping AMSU-A to IASI. N.C.Atkinson
!  1.4      10/08/06  Change INT to NINT to avoid truncation error. NCA
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  parameters and look-up tables for the mapping 
#include "ppluts.h"
!  block of HIRS data 
#include "pphstore.h"
!  block of AMSU-A data 
#include "ppastore.h"
!  block of AMSU-B data 
#include "ppbstore.h"
!  block of MSU data 
#include "ppmstore.h"
!  block of IASI data 
#include "ppistore.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
!   Scalar arguments with intent(In):
      CHARACTER*6   CMap          ! mapping instrument
      CHARACTER*6   CTarg         ! target instrument
!                                      (i.e. HIRS, AMSU-A, AMSU-B, MSU)
!
      INTEGER       HLastLine     ! number of last HIRS line in store
      INTEGER       ALastLine     ! number of last AMSU-A line in store
      INTEGER       BLastLine     ! number of last AMSU-B line in store
      INTEGER       MLastLine     ! number of last MSU line in store
      INTEGER       ILastLine     ! number of last IASI line in store
      INTEGER       IFOV          !IASI detector FOV number (1 to 4)
!
! Function & Subroutine calls:
      External PPBTMAP            ! map brightness temperatures to grid
!
!- End of header

      WRITE(*,"(10X,""Mapping "",A6,"" to "",A6,"" grid"")")
     &     CMap, CTarg

! 1.  Map data to HIRS grid
!     --- ---- -- ---- ----

      IF ( CTarg.EQ."HIRS" ) THEN

! 1.1    map AMSU-A
         IF    ( CMap.EQ."AMSU-A" ) THEN

            CALL PPBTMAP ( "AMSU-A", "HIRS",
     &      ALastLine, Alineno(1)-1,
     &      Hnfovs, HLastLine, Hlineno(1)-1,
     &      LUTelements, A2Hncolocs, A2Hnlines, A2Hstart,
     &      NINT((A2Hnlines*Hcycle)/Acycle), A2HLUT, A2Hmode, IFOV )
            Hinstruments = IBSET ( Hinstruments, bits(3) )
            IF (BTEST( Ainstruments, bits(4))) !See if AMSUB/MHS has been mapped
     &        Hinstruments = IBSET ( Hinstruments, bits(4) )

C! 1.2    map AMSU-B
C         ELSEIF( CMap.EQ."AMSU-B" ) THEN
C
C!           note that we don't allow for AMSU-A to B to HIRS
C            CALL PPBTMAP ( "AMSU-B", "HIRS",
C     &      BLastLine, Blineno(1)-1,
C     &      Hnfovs, HLastLine, Hlineno(1)-1,
C     &      LUTelements, B2Hncolocs, B2Hnlines, B2Hstart,
C     &      NINT((B2Hnlines*Hcycle)/Bcycle), B2HLUT, B2Hmode, IFOV )

! 1.3    map MSU
         ELSEIF( CMap.EQ."MSU" ) THEN

            CALL PPBTMAP ( "MSU", "HIRS",
     &      MLastLine, Mlineno(1)-1,
     &      Hnfovs, HLastLine, Hlineno(1)-1,
     &      LUTelements, M2Hncolocs, M2Hnlines, M2Hstart,
     &      NINT((M2Hnlines*Hcycle)/Mcycle), M2HLUT, M2Hmode, IFOV )
            Hinstruments = IBSET ( Hinstruments, bits(1) )

         ENDIF  ! CMap


! 2.  Map data to AMSU-A grid
!     --- ---- -- ------ ----

      ELSEIF ( CTarg.EQ."AMSU-A" ) THEN

C! 2.1    map HIRS
C         IF    ( CMap.EQ."HIRS" ) THEN
C
C!           note that we don't allow for AMSU-B to HIRS to AMSU-A
C            CALL PPBTMAP ( "HIRS", "AMSU-A",
C     &      HLastLine, Hlineno(1)-1,
C     &      Anfovs, ALastLine, Alineno(1)-1,
C     &      LUTelements, H2Ancolocs, H2Anlines, H2Astart,
C     &      NINT((H2Anlines*Acycle)/Hcycle), H2ALUT, H2Amode, IFOV )
C            Ainstruments = IBSET ( Ainstruments, bits(0) )
C
! 2.2    map AMSU-B
C         ELSEIF( CMap.EQ."AMSU-B" ) THEN
         IF( CMap.EQ."AMSU-B" ) THEN

!           note that we don't allow for HIRS to AMSU-B to AMSU-A
            CALL PPBTMAP ( "AMSU-B", "AMSU-A",
     &      BLastLine, Blineno(1)-1,
     &      Anfovs, ALastLine, Alineno(1)-1,
     &      LUTelements, B2Ancolocs, B2Anlines, B2Astart,
     &      NINT((B2Anlines*Acycle)/Bcycle), B2ALUT, B2Amode, IFOV )
            Ainstruments = IBSET ( Ainstruments, bits(4) )

         ENDIF  ! CMap


! 3.  Map data to AMSU-B grid
!     --- ---- -- ------ ----

      ELSEIF ( CTarg.EQ."AMSU-B" ) THEN

C! 3.1    map HIRS
C         IF    ( CMap.EQ."HIRS" ) THEN
C
C!           note that we don't allow for AMSU-A to HIRS to AMSU-B
C            CALL PPBTMAP ( "HIRS", "AMSU-B",
C     &      HLastLine, Hlineno(1)-1,
C     &      Bnfovs, BLastLine, Blineno(1)-1,
C     &      LUTelements, H2Bncolocs, H2Bnlines, H2Bstart,
C     &      NINT((H2Bnlines*Bcycle)/Hcycle), H2BLUT, H2Bmode, IFOV )
C            Binstruments = IBSET ( Binstruments, bits(0) )
C
! 3.2    map AMSU-A
         IF( CMap.EQ."AMSU-A" ) THEN

!           note that we don't allow for HIRS to AMSU-A to AMSU-B
            CALL PPBTMAP ( "AMSU-A", "AMSU-B",
     &      ALastLine, Alineno(1)-1,
     &      Bnfovs, BLastLine, Blineno(1)-1,
     &      LUTelements, A2Bncolocs, A2Bnlines, A2Bstart,
     &      NINT((A2Bnlines*Bcycle)/Acycle), A2BLUT, A2Bmode, IFOV )
            Binstruments = IBSET ( Binstruments, bits(3) )

         ENDIF  ! CMap

      ELSEIF ( CTarg.EQ."IASI" ) THEN

! 4.1    map AMSU-A
         IF    ( CMap.EQ."AMSU-A" ) THEN

            CALL PPBTMAP ( "AMSU-A", "IASI",
     &      ALastLine, Alineno(1)-1,
     &      Inscanpos, ILastLine, Ilineno(1)-1,
     &      LUTelements, A2Incolocs, A2Inlines, A2Istart,
     &      NINT((A2Inlines*Icycle)/Acycle), A2ILUT, A2Imode, IFOV )
            Iinstruments = IBSET ( Iinstruments, bits(3) )     !AMSU-A
            IF (BTEST( Ainstruments, bits(4)))      !See if MHS has been mapped
     &           Iinstruments = IBSET ( Iinstruments, bits(4) )

         ENDIF  ! CMap

      ENDIF  ! CTarg

      RETURN
      END
