!+ pre-process MSU before mapping
!
      SUBROUTINE PPMSU1 ( MLastLine, LSunit, HTunit )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppmsu1.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   pre-process a block of level 1c MSU data before the mapping
!
! Method:
! 1.  Add bias corrections (BIASCORR)
! 2.  Get land/sea information & elevation (SURFELEV)
! 3.  Subtract limb-darkening curve
! 4.  Correct for limb effect and surface emissivity
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      16/01/97  Bug fixed: lat & lon not now multiplied by 0.01
!  1.2      15/03/05  Add LandFrac in call to surfelev
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"        
! fixed data for processing MSU
#include "ppmfdata.h"        
! block of MSU data
#include "ppmstore.h"       
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"      
!
!   Scalar arguments with intent(In):
      INTEGER       MLastLine     ! number of last MSU line in store
!
      INTEGER       HTunit        ! unit no. for heights dataset
      INTEGER       LSunit        ! unit no. for land/sea bitmap
!
! Local scalars:
      INTEGER       chn           ! loop variable over all channels
      INTEGER       line          ! loop index: pointer to scanline in buffer
      INTEGER       fov           ! loop index: fov number
      INTEGER       isurf         ! surface type: 1=sea, 2=not sea
      INTEGER       nearest       ! nearest HIRS fov to MSU fov (from nearMSU)
!
! Local arrays:
      REAL          lats(Mnfovs)  ! array of latitudes for a scanline
      REAL          lons(Mnfovs)  ! array of longitudes for a scanline
      INTEGER       surftype(Mnfovs) !surface type for a scanline (SURFELEV)
      INTEGER       elevation(Mnfovs)!elevation for a scanline (SURFELEV)
      INTEGER       LandFrac(Mnfovs) !Land fraction for a scanline (SURFELEV)
      INTEGER       nearMSU(Mnfovs) ! approx location of MSU fovs on HIRS grid
!                                    (we assume limb effect is symmetric)
      DATA nearMSU / 2, 7, 13, 18, 23, 28, 23, 18, 13, 7, 2 /
!
! Function & Subroutine calls:
      External BIASCORR           ! scan-dependent bias corrections
      External SURFELEV           ! get land/sea information, (etc.)
      External PPMCORR            ! Correct for limb effect and surface emiss
!
!- End of header

      WRITE(*,"(10X,""Pre-processing MSU before the mapping"")")

! 1.  Add bias corrections
!     --- ---- -----------

      CALL BIASCORR ( Mbiases, Mnchan, Mnchan, Mnfovs,
     &                MLastLine, Mbtemps )


! 2.  Get land/sea information
!     --- -------- -----------

!     loop through scanlines
      DO line = 1, MLastLine

! 2.1    pick out lat's and lon's from scanline
         DO fov = 1, Mnfovs
            lats(fov) = Mlats(fov,line)        ! degrees N
            lons(fov) = Mlons(fov,line)        ! degrees E
         ENDDO  ! fov

! 2.2    find surface type for each fov - using options read in PPUSER
         CALL SURFELEV ( LSunit, HTunit,
     &                   lats, lons, Mnfovs, MboxSize,
     &                   surftype, elevation, LandFrac )

! 2.3    store surface type & elevation
         DO fov = 1, Mnfovs
            Msurftype(fov,line) = surftype(fov)
            Melev(fov,line) = elevation(fov)
         ENDDO  ! fov

      ENDDO  ! line


! 3.  Subtract limb-darkening curve
!     -------- -------------- -----

!     Curve is defined for HIRS scanline, but array 'nearest' tells
!     us which values to take for the MSU scanline.
      DO fov = 1, Mnfovs
         nearest = nearMSU(fov)
         DO line = 1, MLastLine
            IF ( .NOT.BTEST(Mqc(fov,line),bits(0)) ) THEN  ! fov is ok
!              curve depends on surface type
               IF ( Msurftype(fov,line).EQ.0 ) THEN
                 isurf = 1  ! sea
               ELSE
                 isurf = 2  ! non-sea
               ENDIF
               DO chn = 1, Mnchan
                  Mbtemps(chn,fov,line) = Mbtemps(chn,fov,line)
     &                                 - Mlimb(chn,nearest,isurf)
               ENDDO  ! chn
            ENDIF
         ENDDO  ! line
      ENDDO  ! fov


! 4.  Correct for limb effect and surface emissivity
!     ------- --- ---- ------ --- ------- ----------

      CALL PPMCORR ( MLastLine )


      RETURN
      END
