!+ Pre-process level 1c ATOVS data to 1d within ATOVPP before the mapping
!
      SUBROUTINE PPPROC1 ( LHIRS, LAMSA, LAMSB, LMSU, LIASI, LPCIASI,
     &                     LATMS, LCRIS, LHIRAS, LSunit, HTunit,
     &                     HLastLine, ALastLine, BLastLine, MLastLine,
     &                     ILastLine, atmsLastLine, crisLastLine,
     &                     hirasLastLine )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppproc1.F 618 2019-02-12 10:54:03Z frna $
!
! Purpose:
!   pre-process a block of data for each instrument
!
! Method:
! 1.  Pre-process MSU
! 2.  Pre-process AMSU-B
! 3.  Pre-process AMSU-A
! 4.  Pre-process HIRS
! 5.  Pre-process IASI
! 6.  Pre-process ATMS
! 7.  Pre-process CrIS
! 7.  Pre-process HIRAS
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      27/01/05  Add IASI (N.C.Atkinson)
!  1.2      23/09/08  Add IASIPC
!  1.3      02/09/10  Add ATMS and CrIS
!  1.4      12/02/19  Add HIRAS
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!   Scalar arguments with intent(In):
      LOGICAL       LHIRS         ! .TRUE. to process HIRS
      LOGICAL       LAMSA         ! .TRUE. to process AMSU-A
      LOGICAL       LAMSB         ! .TRUE. to process AMSU-B
      LOGICAL       LMSU          ! .TRUE. to process MSU
      LOGICAL       LIASI         ! .TRUE. to process IASI
      LOGICAL       LPCIASI       ! .TRUE. to process IASI PC
      LOGICAL       LATMS         ! .TRUE. to process ATMS
      LOGICAL       LCRIS         ! .TRUE. to process CRIS
      LOGICAL       LHIRAS        ! .TRUE. to process HIRAS
!
      INTEGER       HLastLine     ! number of last HIRS line in store
      INTEGER       ALastLine     ! number of last AMSU-A line in store
      INTEGER       BLastLine     ! number of last AMSU-B line in store
      INTEGER       MLastLine     ! number of last MSU line in store
      INTEGER       ILastLine     ! number of last IASI line in store
      INTEGER       atmsLastLine  ! number of last ATMS line in store
      INTEGER       crisLastLine  ! number of last CRIS line in store
      INTEGER       hirasLastLine ! number of last HIRAS line in store
!
      INTEGER       HTunit        ! unit no. for heights dataset
      INTEGER       LSunit        ! unit no. for land/sea bitmap
!
! Function & Subroutine calls:
      External PPMSU1             ! pre-process MSU before mapping
      External PPAMSUB1           ! pre-process AMSU-B before mapping
      External PPAMSUA1           ! pre-process AMSU-A before mapping
      External PPHIRS1            ! pre-process HIRS before mapping
      External PPIASI1            ! pre-process IASI before mapping
      External PPATMS1            ! pre-process ATMS before mapping
!
!- End of header


! 1.  Pre-process MSU
!     ----------- ---

      IF (LMSU) THEN

         CALL PPMSU1 ( MLastLine, LSunit, HTunit )

      ENDIF


! 2.  Pre-process AMSU-B
!     ----------- ------

      IF (LAMSB) THEN

         CALL PPAMSUB1 ( BLastLine, LSunit, HTunit )

      ENDIF


! 3.  Pre-process AMSU-A
!     ----------- ------

      IF (LAMSA) THEN

         CALL PPAMSUA1 ( ALastLine, BLastLine, LSunit, HTunit )

      ENDIF


! 4.  Pre-process HIRS
!     ----------- ----

      IF (LHIRS) THEN

         CALL PPHIRS1 ( HLastLine, LSunit, HTunit )

      ENDIF


! 5.  Pre-process IASI
!     ----------- ----

      IF (LIASI .OR. LPCIASI) THEN

         CALL PPIASI1 ( ILastLine, LSunit, HTunit )

      ENDIF


! 6.  Pre-process ATMS
!     ----------- ----

      IF (LATMS) THEN

         CALL PPATMS1 ( atmsLastLine, LSunit, HTunit )

      ENDIF


! 7.  Pre-process CRIS
!     ----------- ----

      IF (LCRIS) THEN

         CALL PPCRIS1 ( crisLastLine, LSunit, HTunit )

      ENDIF

! 7.  Pre-process HIRAS
!     ----------- ----

      IF (LHIRAS) THEN

         CALL PPHIRAS1 ( hirasLastLine, LSunit, HTunit )

      ENDIF

      RETURN
      END
