!+ Pre-process level 1c ATOVS data to 1d within ATOVPP after the mapping
!
      SUBROUTINE PPPROC2 ( LHIRS, LAMSA, LAMSB, LMSU, LIASI,LATMS,LCRIS,
     &                     GHIRS, GAMSA, GAMSB, GIASI, GATMS, GCRIS,
     &                     HLastLine, ALastLine, BLastLine, MLastLine,
     &                     ILastLine, atmsLastLine, crisLastLine )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppproc2.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose:
!   pre-process a block of data for each instrument
!
! Method:
! 1.  Pre-process AMSU-B
! 2.  Pre-process AMSU-A
! 3.  Pre-process HIRS
! 4.  Pre-process IASI
! 5.  Pre-process ATMS
! 6.  Pre-process CrIS
!
! Note that instrument is only processed here if output has been requested
! on that instrument grid.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      29/06/98  LHIRS argument deleted from PPAMSUB2
!  1.2      11/08/05  Add IASI (N.C.Atkinson)
!  1.3      02/09/10  Add ATMS and CrIS (N.C.Atkinson)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!   Scalar arguments with intent(In):
      LOGICAL       LHIRS         ! .TRUE. to process HIRS
      LOGICAL       LAMSA         ! .TRUE. to process AMSU-A
      LOGICAL       LAMSB         ! .TRUE. to process AMSU-B
      LOGICAL       LMSU          ! .TRUE. to process MSU
      LOGICAL       LIASI         ! .TRUE. to process IASI
      LOGICAL       LATMS         ! .TRUE. to process ATMS
      LOGICAL       LCRIS         ! .TRUE. to process CrIS
!
      LOGICAL       GHIRS         ! .TRUE. to output on HIRS grid
      LOGICAL       GAMSA         ! .TRUE. to output on AMSU-A grid
      LOGICAL       GAMSB         ! .TRUE. to output on AMSU-B grid
      LOGICAL       GIASI         ! .TRUE. to output on IASI grid
      LOGICAL       GATMS         ! .TRUE. to output on ATMS grid
      LOGICAL       GCRIS         ! .TRUE. to output on CRIS grid
!
      INTEGER       HLastLine     ! number of last HIRS line in store
      INTEGER       ALastLine     ! number of last AMSU-A line in store
      INTEGER       BLastLine     ! number of last AMSU-B line in store
      INTEGER       MLastLine     ! number of last MSU line in store
      INTEGER       ILastLine     ! number of last IASI line in store
      INTEGER       atmsLastLine  ! number of last ATMS line in store
      INTEGER       crisLastLine  ! number of last CrIS line in store
!
! Function & Subroutine calls:
      External PPAMSUB2           ! pre-process AMSU-B
      External PPAMSUA2           ! pre-process AMSU-A
      External PPHIRS2            ! pre-process HIRS
      External PPIASI2            ! pre-process IASI
      External PPCRIS2            ! pre-process CRIS
!
!- End of header


! 1.  Pre-process AMSU-B
!     ----------- ------

      IF (GAMSB) THEN

         CALL PPAMSUB2 ( BLastLine, LAMSA )

      ENDIF


! 2.  Pre-process AMSU-A
!     ----------- ------

      IF (GAMSA) THEN

         CALL PPAMSUA2 ( ALastLine, LHIRS, LAMSB )

      ENDIF


! 3.  Pre-process HIRS
!     ----------- ----

      IF (GHIRS) THEN

         CALL PPHIRS2 ( HLastLine, LAMSA, LAMSB, LMSU )

      ENDIF


! 4.  Pre-process IASI
!     ----------- ----

      IF (GIASI) THEN

         CALL PPIASI2 ( ILastLine, LAMSA, LAMSB )

      ENDIF

! 5.  Pre-process ATMS (not required)
!     ----------- ----

! 6.  Pre-process CRIS
!     ----------- ----

      IF (GCRIS) THEN

         CALL PPCRIS2 ( crisLastLine, LATMS )

      ENDIF


      RETURN
      END
