      subroutine ppscan_params(satnum, wmosatnum,
     & cone_angles, step_angles, step_times, line_times,
     & spots_per_lines, left_to_right_scans, halfcone_iasi, ierr)
!
! Name: $Id: ppscan_params.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!  Standardise AAPP so that instrument scan parameters are only defined once.
!  This subroutine provides the interface for atovpp. 
!
! Method:
!  Read the satid.txt file via call to rdnoaaid. Scan parameters are passed
!  back as arguments for lutmap.F 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
!  Version  Date      Comment
!  1.0      20/07/04  Original code. N C Atkinson
!  1.1      15/09/04  Use constants_preproc.h for fundamental constants - NCA
!  1.2      23/12/04  Add wmosatnum and halfcone_iasi as arguments.
!                     Add IASI as instrument number 6 - NCA
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:

      IMPLICIT NONE

! Arguments:
! Inputs
      INTEGER satnum            !satellite number in series
      INTEGER wmosatnum         !WMO satellite identifier
! Outputs
      REAL cone_angles(6)
      REAL step_angles(6)
      REAL step_times(6)
      REAL line_times(6)
      REAL halfcone_iasi
      INTEGER spots_per_lines(6)
      INTEGER left_to_right_scans(6)
      INTEGER ierr
      
! Local variables:
      INTEGER isatid, instr, i
      LOGICAL found
      CHARACTER*8 sat_series  !satellite series
      
! External
      EXTERNAL rdnoaaid

! Include:
#include "satid.h"
!  Fundamental and physical constants 
#include "constants_preproc.h"

!----End of declarations

!Use the WMO satellite number to get the satellite series
      
      IF (wmosatnum .GE. 3 .AND. wmosatnum .LE. 5) THEN
        sat_series = "metop"
      ELSE IF (wmosatnum .GE. 783 .AND. wmosatnum .LE. 784) THEN
        sat_series = "eos"
      ELSE
        sat_series = "noaa"
      ENDIF

      CALL rdnoaaid(ierr)   !Reads the satid.txt file into satid.h variables

      found = .false.
      IF (ierr.EQ.0) THEN
       DO isatid = 1, maxsatid
        IF (sat_serie(isatid).EQ.sat_series.AND.
     &      noa_sc_num(isatid).EQ.satnum) THEN
          found = .true.

! Set array index for lutmap.F

          DO instr = 1, sc_num_inst(isatid)   !Loop over defined instruments

            i=0

            IF (inst_gen_name(instr, isatid).EQ.'avhrr') THEN
              i=1
            ELSE IF (inst_gen_name(instr, isatid).EQ.'hirs') THEN
              i=2
            ELSE IF (inst_gen_name(instr, isatid).EQ.'amsua') THEN
              i=3
            ELSE IF (inst_gen_name(instr, isatid).EQ.'amsub') THEN
              i=4
            ELSE IF (inst_gen_name(instr, isatid).EQ.'mhs') THEN
              i=4
            ELSE IF (inst_gen_name(instr, isatid).EQ.'msu') THEN
              i=5
            ELSE IF (inst_gen_name(instr, isatid).EQ.'iasi') THEN
              i=6
            ENDIF

! Set return arguments

            IF (i .GT. 0) THEN
              cone_angles(i) = ifov_mode_p(1,instr,isatid)*rad2deg        !deg
              step_angles(i) = 
     &               ABS(inst_scan_mode_p(3,instr,isatid))*rad2deg        !deg
              step_times(i) = inst_scan_mode_p(1,instr,isatid)*86400.0    !sec
              line_times(i) = inst_scan_mode_p(5,instr,isatid)*86400.0    !sec
              IF (i .EQ. 6) THEN      !IASI
                spots_per_lines(i) = inst_scan_mode_p(7,instr,isatid)
                halfcone_iasi = inst_scan_mode_p(6,instr,isatid)*rad2deg  !deg
              ELSE
                spots_per_lines(i) = inst_scan_mode_p(6,instr,isatid)
              ENDIF
              IF (inst_scan_mode_p(3,instr,isatid).LT.0) THEN 
                left_to_right_scans(i) = 1
              ELSE
                left_to_right_scans(i) = 0
              ENDIF
            ENDIF
          ENDDO
        ENDIF
       ENDDO
       IF (.not.found) THEN
        ierr=1
       ENDIF
          
      ENDIF        !ierr=0 from rdnoaaid
      
      RETURN
      END
