!+ read which instruments to process, etc within ATOVPP
!
      SUBROUTINE PPUSER ( LHIRS, LAMSA, LAMSB, LMSU, LIASI, LPCIASI,
     &                    LATMS, LCRIS, LHIRAS,
     &                    GHIRS, GAMSA, GAMSB, GIASI, GATMS, GCRIS,
     &                    GHIRAS,
     &                    H1Cunit, A1Cunit, B1Cunit, M1Cunit, I1Cunit,
     &                    IPCunit, atms1Cunit, cris1Cunit, hiras1Cunit,
     &                    H1Dunit, A1Dunit, B1Dunit, I1Dunit,
     &                    atms1Dunit, cris1Dunit, hiras1Dunit,
     &                    Hfunit, Afunit, Bfunit, Mfunit,
     &                    Lfunit, LSunit, HTunit, Scatfunit,
     &                    Ifunit, Ieigunit1, Ieigunit2, Inoiseunit,
     &                    Imtfunit, atmsfunit, crisfunit, hirasfunit,
     &                    atmsignoreunit,
     &                    criseigunit1, criseigunit2,
     &                    hiraseigunit1, hiraseigunit2
     &                     )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppuser.F 618 2019-02-12 10:54:03Z frna $
!
! Purpose:
!   Define unit numbers for input/output 1C/1D files, and for
!   land/sea topography datasets.
!   Read from standard input whether or not to process each instrument
!   and whether or not to output 1D data on that instrument grid.
!   Read list of intra-instrument grid mappings to perform.
!
! Method:
! 0.  Initialisation
! 1.  Which 1c files to read ?
! 2.  Which 1d files to write ?
! 3.  Mapping requirements
!   Format of input is specified in AAPP documentation APP204U, vn of 27/11/96
!   Note that input data with "!" in the first column are ignored.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      06/12/00  Added Unit number 70 for BG-Convolution 
!                      and the call for ppbginit (Ralf Bennartz)
!  1.2      16/03/05  Added Scatfunit (N.C.Atkinson) 
!  1.3      15/08/05  Added IASI (NCA)
!  1.4      23/09/09  Added IASIPC
!  1.5      16/08/10  Added ATMS and CRIS
!  1.5      07/02/19  Added HIRAS
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  parameters and look-up tables for the mapping 
#include "ppluts.h"
!
      INTEGER       Ninstruments         ! number of instruments recognised:
      PARAMETER   ( Ninstruments = 9 )   !   HIRS, AMSU-A, AMSU-B, MSU, IASI,
                                         !     IASIPC, ATMS, CRIS, HIRAS
!
! Subroutine arguments
!   Scalar arguments with intent(out):
      LOGICAL       LHIRS         ! .TRUE. to process HIRS
      LOGICAL       LAMSA         ! .TRUE. to process AMSU-A
      LOGICAL       LAMSB         ! .TRUE. to process AMSU-B
      LOGICAL       LMSU          ! .TRUE. to process MSU
      LOGICAL       LIASI         ! .TRUE. to process IASI
      LOGICAL       LPCIASI       ! .TRUE. to process IASI PC
      LOGICAL       LATMS         ! .TRUE. to process ATMS
      LOGICAL       LCRIS         ! .TRUE. to process CRIS
      LOGICAL       LHIRAS        ! .TRUE. to process HIRAS
!
      LOGICAL       GHIRS         ! .TRUE. to output on HIRS grid
      LOGICAL       GAMSA         ! .TRUE. to output on AMSU-A grid
      LOGICAL       GAMSB         ! .TRUE. to output on AMSU-B grid
      LOGICAL       GIASI         ! .TRUE. to output on IASI grid
      LOGICAL       GATMS         ! .TRUE. to output on ATMS grid
      LOGICAL       GCRIS         ! .TRUE. to output on CRIS grid
      LOGICAL       GHIRAS        ! .TRUE. to output on HIRAS grid
!
      INTEGER       H1Cunit       ! unit number for HIRS level 1C file
      INTEGER       A1Cunit       ! unit number for AMSU-A level 1C file
      INTEGER       B1Cunit       ! unit number for AMSU-B level 1C file
      INTEGER       M1Cunit       ! unit number for MSU level 1C file
      INTEGER       I1Cunit       ! unit number for IASI level 1C file
      INTEGER       IPCunit       ! unit number for IASI PC file
      INTEGER       atms1Cunit    ! unit number for ATMS level 1C file
      INTEGER       cris1Cunit    ! unit number for CRIS level 1C file
      INTEGER       hiras1Cunit   ! unit number for HIRAS level 1C file
!
      INTEGER       H1Dunit       ! unit number for HIRS level 1D file
      INTEGER       A1Dunit       ! unit number for AMSU-A level 1D file
      INTEGER       B1Dunit       ! unit number for AMSU-B level 1D file
      INTEGER       I1Dunit       ! unit number for IASI level 1D file
      INTEGER       atms1Dunit    ! unit number for ATMS level 1D file
      INTEGER       cris1Dunit    ! unit number for CRIS level 1D file
      INTEGER       hiras1Dunit   ! unit number for HIRAS level 1D file
!
      INTEGER       Hfunit        ! unit number for HIRS-specific data
      INTEGER       Afunit        ! unit number for AMSU-A-specific data
      INTEGER       Bfunit        ! unit number for AMSU-B-specific data
      INTEGER       Mfunit        ! unit number for MSU-specific data
      INTEGER       Ifunit        ! unit number for IASI-specific data
      INTEGER       Ieigunit1     ! unit number for IASI eigenvectors (encoding)
      INTEGER       Ieigunit2     ! unit number for IASI eigenvectors (decoding)
      INTEGER       Inoiseunit    ! unit number for IASI noise
      INTEGER       Imtfunit      ! unit number for IASI MTF
      INTEGER       atmsfunit     ! unit number for ATMS
      INTEGER       crisfunit     ! unit number for CRIS
      INTEGER       hirasfunit    ! unit number for HIRAS
      INTEGER       atmsignoreunit ! unit number for ATMS ignore file
      INTEGER       criseigunit1  ! unit number for CRIS eigenvectors (encoding)
      INTEGER       criseigunit2  ! unit number for CRIS eigenvectors (decoding)
      INTEGER       hiraseigunit1 ! unit number for HIRAS eigenvectors (encoding)
      INTEGER       hiraseigunit2 ! unit number for HIRAS eigenvectors (decoding)
!
      INTEGER       Lfunit        ! unit number for LUT corrections
      INTEGER       LSunit        ! unit no. for land/sea bitmap
      INTEGER       HTunit        ! unit no. for heights dataset
      INTEGER       Scatfunit     ! unit no. for NWC-SAF scattering datasets

      INTEGER       B2AUNIT       ! unit number for convolution 
!
! Local Scalars:
      CHARACTER*80  input_line    ! line read from instream data
      INTEGER       errStatus     ! error status
                                  ! +ve => fatal error, -ve => warning
      CHARACTER*80  errMessage    ! error message
      INTEGER       i             ! loop variable over instruments
      INTEGER       indx          ! index to instrument in input_line
      INTEGER       maxindx       ! upper limit on indx
      INTEGER       ninst         ! number of instruments found
      CHARACTER*6   cmap          ! mapping instrument (e.g. "MSU")
      CHARACTER*6   ctarg         ! target instrument (e.g. "HIRS")
!
! Local Arrays:
      INTEGER       inst(Ninstruments)       ! list of instruments found
      INTEGER       leninstr(Ninstruments)    ! lengths of instrument names
      CHARACTER*6   cinstrument(Ninstruments) ! instrument names
      DATA cinstrument / "HIRS", "AMSU-A", "AMSU-B", "MSU", "IASI",
     &                   "PCIASI", "ATMS", "CRIS", "HIRAS" /
!
! Function & Subroutine calls:
      External ErrorReport        ! ATOVPP error routine
      External C2Upper            ! convert character string to upper case
      External ppbginit           ! Initialize bg-convolution
!
!- End of header

! 0.  Initialisation
!     --------------

! 0.1 whether to read and process these instruments:
      LHIRS = .FALSE.
      LAMSA = .FALSE.
      LAMSB = .FALSE.
      LMSU  = .FALSE.
      LIASI = .FALSE.
      LPCIASI = .FALSE.
      LATMS = .FALSE.
      LCRIS = .FALSE.
      LHIRAS = .FALSE.

! 0.2 whether to output level 1d on these instrument grids:
      GHIRS = .FALSE.
      GAMSA = .FALSE.
      GAMSB = .FALSE.
      GIASI = .FALSE.
      GATMS = .FALSE.
      GCRIS = .FALSE.
      GHIRAS = .FALSE.

! 0.3 Define unit numbers
! (these could be read in if there is a real requirement for
!  unit numbers to be different for different runs of ATOVPP)

!     level 1c datasets
      H1Cunit = 11      ! unit number for HIRS level 1C file
      A1Cunit = 12      ! unit number for AMSU-A level 1C file
      B1Cunit = 13      ! unit number for AMSU-B level 1C file
      M1Cunit = 14      ! unit number for MSU level 1C file
      I1Cunit = 15      ! unit number for IASI level 1C file
      IPCunit = 16      ! unit number for IASI PC file
      atms1Cunit = 17   ! unit number for ATMS level 1C file
      cris1Cunit = 18   ! unit number for CRIS level 1C file
      hiras1Cunit = 19  ! unit number for HIRAS level 1C file

!     level 1d datasets
      H1Dunit = 21      ! unit number for HIRS level 1D file
      A1Dunit = 22      ! unit number for AMSU-A level 1D file
      B1Dunit = 23      ! unit number for AMSU-B level 1D file
      I1Dunit = 25      ! unit number for IASI level 1D file
      atms1Dunit = 27   ! unit number for ATMS level 1D file
      cris1Dunit = 28   ! unit number for CRIS level 1D file
      hiras1Dunit = 29   ! unit number for CRIS level 1D file

!     instrument-specific fixed data files
      Hfunit  = 41      ! unit number for HIRS-specific data
      Afunit  = 42      ! unit number for AMSU-A-specific data
      Bfunit  = 43      ! unit number for AMSU-B-specific data
      Mfunit  = 44      ! unit number for MSU-specific data
      Ifunit  = 45      ! unit number for IASI-specific data
      Ieigunit1 = 46    ! unit number for IASI eigenvectors (encode)
      Ieigunit2 = 47    ! unit number for IASI eigenvectors (decode)
      Inoiseunit = 48   ! unit number for IASI noise
      Imtfunit = 49     ! unit number for IASI MTF
      atmsfunit = 54    ! unit number for ATMS
      crisfunit = 55    ! unit number for CRIS
      hirasfunit = 57   ! unit number for HIRAS
      atmsignoreunit = 56 ! unit number for ATMS ignore file

!     LUT fixed data file
      Lfunit  = 50      ! unit no. for mapping LUT corrections

!     land/sea datasets (subroutine SURFELEV)
      LSunit  = 51      ! unit no. for land/sea bitmap
      HTunit  = 52      ! unit no. for heights dataset

!     NWC-SAF scattering data
      Scatfunit = 53
      
      B2AUNIT = 70      ! Unit number for convolution

! 0.4 count the length of the instrument names
      maxindx = LEN(cinstrument(1)) ! upper limit for indx
      DO i = 1, Ninstruments
         cmap = cinstrument(i)
         indx = maxindx   ! point to last character in string
         DO WHILE (cmap(indx:indx).EQ." ")
            indx = indx - 1
         ENDDO  ! WHILE
         leninstr(i) = indx
      ENDDO  ! i
      maxindx = LEN(input_line)+1 - LEN(cinstrument(1)) ! upper limit for indx


! 1.  Which 1c files to read ?
!     ----- -- ----- -- ----

!     read the whole line into memory  (ignoring lines starting "!")
      input_line = "!"
      errStatus = 0
      errMessage = "Error reading user input: input instruments"
      DO WHILE ( input_line(1:1) .EQ. "!" .AND. errStatus.EQ.0 )
         READ(*,"(A80)",IOSTAT=errStatus) input_line
      ENDDO

!     check for errors
      errStatus = ABS (errStatus)   ! end-of-file returns -1, which isn't fatal
      CALL ErrorReport(errStatus,errMessage,"PPUSER")

!     convert to upper case
      CALL C2Upper ( input_line )

!     look through the whole string for instrument names, stopping at "!"
      indx = 1   ! pointer to character in input_line
      ninst = 0  ! number of instrument names found
      DO WHILE ( indx.LT.maxindx
     &           .AND. input_line(indx:indx+1).NE."!" )
         DO i = 1, Ninstruments
            IF  ( input_line(indx:indx+leninstr(i)-1)
     &           .EQ. cinstrument(i) ) THEN      ! found it here
                 ninst = ninst + 1
                 inst(ninst) = i
                 indx = indx + leninstr(i) - 1   ! move past this instrument
            ENDIF
         ENDDO  ! i
         indx = indx + 1
      ENDDO  ! WHILE

      IF ( ninst.EQ.0 ) THEN
         errMessage = "No instruments requested for processing"
         errStatus = 10
         CALL ErrorReport(errStatus,errMessage,"PPUSER")
      ENDIF

!     see which instruments have been requested
      DO i = 1, ninst
         IF     ( inst(i).EQ.1 ) THEN
            LHIRS = .TRUE.
         ELSEIF ( inst(i).EQ.2 ) THEN
            LAMSA = .TRUE.
         ELSEIF ( inst(i).EQ.3 ) THEN
            LAMSB = .TRUE.
         ELSEIF ( inst(i).EQ.4 ) THEN
            LMSU  = .TRUE.
         ELSEIF ( inst(i).EQ.5 ) THEN
            LIASI  = .TRUE.
         ELSEIF ( inst(i).EQ.6 ) THEN
            LPCIASI  = .TRUE.
         ELSEIF ( inst(i).EQ.7 ) THEN
            LATMS  = .TRUE.
         ELSEIF ( inst(i).EQ.8 ) THEN
            LCRIS  = .TRUE.
         ELSEIF ( inst(i).EQ.9 ) THEN
            LHIRAS = .TRUE.
         ENDIF
      ENDDO  !

!     check we're not mixing TOVS and ATOVS
      IF ( LMSU .AND. (LAMSA.OR.LAMSB) ) THEN
         errMessage = "Cannot mix TOVS and ATOVS"
         errStatus = 11
         CALL ErrorReport(errStatus,errMessage,"PPUSER")
      ENDIF


! 2.  Which 1d files to write ?
!     ----- -- ----- -- -----

!     read the whole line into memory  (ignoring lines starting "!")
      input_line = "!"
      errStatus = 0
      errMessage = "Error reading user input: output instrument grids"
      DO WHILE ( input_line(1:1) .EQ. "!" .AND. errStatus.EQ.0 )
         READ(*,"(A80)",IOSTAT=errStatus) input_line
      ENDDO

!     check for errors
      errStatus = ABS (errStatus)   ! end-of-file returns -1, which isn't fatal
      CALL ErrorReport(errStatus,errMessage,"PPUSER")

!     convert to upper case
      CALL C2Upper ( input_line )

!     look through the whole string for instrument names, stopping at "!"
      indx = 1   ! pointer to character in input_line
      ninst = 0  ! number of instrument names found
      DO WHILE ( indx.LT.maxindx
     &           .AND. input_line(indx:indx+1).NE."!" )
         DO i = 1, Ninstruments
           IF ( i .NE. 4) THEN       !MSU is not an option
             IF  ( input_line(indx:indx+leninstr(i)-1)
     &           .EQ. cinstrument(i) ) THEN      ! found it here
                 ninst = ninst + 1
                 inst(ninst) = i
                 indx = indx + leninstr(i) - 1   ! move past this instrument
             ENDIF
           ENDIF
         ENDDO  ! i
         indx = indx + 1
      ENDDO  ! WHILE

      IF ( ninst.EQ.0 ) THEN
         errMessage = "No instrument grids requested for output"
         errStatus = 20
         CALL ErrorReport(errStatus,errMessage,"PPUSER")
      ENDIF

!     see which instruments have been requested
!     - can't output on instrument grid if we're not processing that instrument
      errMessage = "Instrument output requested but not input"
      DO i = 1, ninst
         IF     ( inst(i).EQ.1 ) THEN
            IF ( LHIRS ) THEN
               GHIRS = .TRUE.
            ELSE
               errStatus = 21
            ENDIF
         ELSEIF ( inst(i).EQ.2 ) THEN
            IF ( LAMSA ) THEN
               GAMSA = .TRUE.
            ELSE
               errStatus = 22
            ENDIF
         ELSEIF ( inst(i).EQ.3 ) THEN
            IF ( LAMSB ) THEN
               GAMSB = .TRUE.
            ELSE
               errStatus = 23
            ENDIF
         ELSEIF ( inst(i).EQ.5 ) THEN
            IF ( LIASI.OR.LPCIASI ) THEN
               GIASI = .TRUE.
            ELSE
               errStatus = 24
            ENDIF
         ELSEIF ( inst(i).EQ.7 ) THEN
            IF ( LATMS ) THEN
               GATMS = .TRUE.
            ELSE
               errStatus = 25
            ENDIF
         ELSEIF ( inst(i).EQ.8 ) THEN
            IF ( LCRIS ) THEN
               GCRIS = .TRUE.
            ELSE
               errStatus = 26
            ENDIF
         ELSEIF ( inst(i).EQ.9 ) THEN
            IF ( LHIRAS ) THEN
               GHIRAS = .TRUE.
            ELSE
               errStatus = 27
            ENDIF
         ENDIF
      ENDDO  !

!     check for errors
      CALL ErrorReport(errStatus,errMessage,"PPUSER")


! 3.  Mapping requirements
!     ------- ------------

! 3.1 Read the requirements (..read until end-of-file)
      Nmappings  = 0  ! number of mappings requested by user
      READ(*,"(A80)",IOSTAT=errStatus) input_line
      DO WHILE ( errStatus.EQ.0 )

         IF ( input_line(1:1) .NE. "!" ) THEN
!            not a comment line  -interpret it as a mapping e.g. "MSU to HIRS"
             CALL C2Upper ( input_line )  ! convert to upper case

!            look for the first 2 instrument names, stopping at "!"
             indx = 1   ! pointer to character in input_line
             ninst = 0  ! number of instrument names found
             DO WHILE ( indx.LT.maxindx .AND. ninst.LT.2
     &           .AND. input_line(indx:indx+1).NE."!" )
                DO i = 1, Ninstruments
                   IF  ( input_line(indx:indx+leninstr(i)-1)
     &                  .EQ. cinstrument(i) ) THEN ! found it here
                        ninst = ninst + 1
                        inst(ninst) = i
                        indx = indx + leninstr(i) - 1 ! move past this name
                   ENDIF
                ENDDO  ! i
                indx = indx + 1
             ENDDO  ! WHILE

             IF ( ninst.LT.2 ) THEN
                errMessage = "Mapping line not understood"//input_line
                errStatus = 31
                CALL ErrorReport(errStatus,errMessage,"PPUSER")
             ENDIF

!            First name is the mapping instrument, second the target
             Nmappings = Nmappings + 1
             MapInstr(Nmappings)  = cinstrument(inst(1))
             TargInstr(Nmappings) = cinstrument(inst(2))

         ENDIF
         READ(*,"(A80)",IOSTAT=errStatus) input_line
      ENDDO
      errStatus = 0   ! don't complain because of end-of-file

! 3.2 Check the requirements
      IF ( Nmappings.GT.0 ) THEN

         DO i = 1, Nmappings
            cmap  = MapInstr(i)
            ctarg = TargInstr(i)

!           Have we been asked to process these instruments ?
            IF ( (cmap.EQ."HIRS" .OR. ctarg.EQ."HIRS")
     &         .AND. .NOT.LHIRS ) THEN
               errStatus = 321
               errMessage = "HIRS level 1c data not requested"
            ENDIF
            IF ( (cmap.EQ."AMSU-A" .OR. ctarg.EQ."AMSU-A")
     &         .AND. .NOT.LAMSA ) THEN
               errStatus = 322
               errMessage = "AMSU-A level 1c data not requested"
            ENDIF
            IF ( (cmap.EQ."AMSU-B" .OR. ctarg.EQ."AMSU-B")
     &         .AND. .NOT.LAMSB ) THEN
               errStatus = 323
               errMessage = "AMSU-B level 1c data not requested"
            ENDIF
            IF ( cmap.EQ."MSU" .AND. .NOT.LMSU )  THEN
               errStatus = 324
               errMessage = "MSU level 1c data not requested"
            ENDIF
            IF ( cmap.EQ."IASI" .AND. .NOT.LIASI )  THEN
               errStatus = 327
               errMessage = "IASI level 1c data not requested"
            ENDIF
            IF ( cmap.EQ."IASIPC" .AND. .NOT.LPCIASI )  THEN
               errStatus = 328
               errMessage = "IASI PC data not requested"
            ENDIF
            IF ( (cmap.EQ."ATMS" .OR. ctarg.EQ."ATMS")
     &         .AND. .NOT.LATMS ) THEN
               errStatus = 329
               errMessage = "ATMS level 1c data not requested"
            ENDIF
            IF ( (cmap.EQ."CRIS" .OR. ctarg.EQ."CRIS")
     &         .AND. .NOT.LCRIS ) THEN
               errStatus = 330
               errMessage = "CRIS level 1c data not requested"
            ENDIF
            IF ( (cmap.EQ."HIRAS" .OR. ctarg.EQ."HIRAS")
     &         .AND. .NOT.LHIRAS ) THEN
               errStatus = 330
               errMessage = "HIRAS level 1c data not requested"
            ENDIF

!           Are we mapping an instrument to itself ?
            IF ( cmap.EQ.ctarg ) THEN
               errStatus = 325
               errMessage = "Cannot map instrument to itself"
            ENDIF

!           Block out here possibility of mapping to MSU
            IF ( ctarg.EQ."MSU" ) THEN
               errStatus   = 326
               errMessage = "Cannot map BTs to MSU grid"
            ENDIF

         ENDDO  ! i

!        check for errors
         CALL ErrorReport(errStatus,errMessage,"PPUSER")

      ENDIF

      IF (B2Amode .EQ. 5) CALL ppbginit(B2AUNIT)

      RETURN
      END
