!+ Read integers from a character string
!
      SUBROUTINE RDINTS ( ChString, Int_List, Num_Ints )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: rdints.F 5 2006-01-04 09:11:15Z frna $
!
! Purpose:
!   Extract a list of integers from a line of text.
!
! Method:
!   ChString is a list of integers, but in unknown format, and possibly
!   interspersed with text. Look for distinct sequences of digits, and
!   store these in array Int_List. This version is limited to storing
!   only positive integers. Stops reading when a "!" is encountered.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/95  Original code. (R.J.Renshaw)
!  1.1      30/06/98  Format "(I99)" changed to *
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!
! Subroutine arguments
!   Scalar arguments with intent(in):
      CHARACTER*(*) ChString      ! line of text containing integers
!
!   Scalar arguments with intent(out):
      INTEGER       Num_Ints      ! number of integers found in ChString
!
!   Array arguments with intent(out):
      INTEGER       Int_List(*)   ! array of integers in ChString
!
! Local Scalars:
      CHARACTER*1   ch_point      ! char in ChString pointed to by in_point
      INTEGER       in_point      ! pointer that loops through ChString
      INTEGER       len_seq       ! length of latest digit-sequence found
      INTEGER       len_total     ! length of ChString
!
!- End of header

!     initialise counters
      Num_Ints = 0   ! number of integers found so far
      in_point = 1   ! position of "pointer" in string ChString
      len_seq = 0    ! length of latest sequence of digits
      len_total = LEN( ChString ) ! length of ChString

!     loop through ChString, looking for continuous sequences of digits

      DO WHILE (in_point.LT.len_total)

!        Get the character pointed to by in_point.
         ch_point = ChString(in_point:in_point)

!        Is it a digit ?  If so, increment len_seq
         IF (ch_point.GE."0" .and. ch_point.LE."9") THEN
            len_seq = len_seq + 1

!        else, have we come to the end of a sequence ?
         ELSEIF (len_seq.GT.0) THEN

!        .. then read what the integer is
            Num_Ints = Num_Ints + 1        ! .. and add it to the list.
            read ( ChString (in_point-len_seq : in_point-1), *)
     &      Int_List(Num_Ints)

!        .. Set sequence length back to zero for next find.
            len_seq = 0

         ENDIF

!        force a stop at "!"s - these might be comments in the data
         IF (ch_point.EQ."!") in_point = len_total

!        now go back and look at the next character in ChString
         in_point = in_point + 1

      ENDDO ! - DO WHILE loop through ChString

      RETURN
      END
