      SUBROUTINE retrieve_one_si(t23,t89,t150,zen,flagb,lsma,lsmb,
     +                          ave_back_sea,si,
     +                          m0,m1,m2,m3,si_flag)  
     
! Name: $Id: retrieve_one_si.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose: Retrieves scattering index for one pixel over sea,land, or coast
!
! Reference: "Precipitation analysis using the Advanced Microwave Sounder Unit
! in support of nowcasting applications", Ralf Bennartz and Anke Thoss,
! Meteorol. Appl. 9, 177-189 (2002).
! 
! Input: 
!   REAL         t23,t89,t150,zen     TB23,TB89,TB150,local zenith angle 
!   INTEGER      flagb                Indicates SI out of range
!   INTEGER      lsmb,lsma            Fraction of land in AMSU-B (A) pixel [%]
!   REAL         ave_back_sea         Average background (T89-T150) over sea
! 
! Output:
!   REAL         si            Scattering index
!   REAL         m0,m1,m2,m3   Likelihood of different intesity classes of rain
!   INTEGER      si_flag       Flag that holds proessing info
!
! Code Owner : Ralf Bennartz
!
!   Written    : 10 OCober 2003
!   History    : (derived from NWC-SAF C-programs)
!
! Declarations:

#include "safnwc_scat.h"

! INPUT variables 
      REAL    t23,t89,t150,ave_back_sea,zen
      INTEGER lsma,lsmb,flagb

! OUTPUT variables 
      REAL    si,m0,m1,m2,m3
      INTEGER si_flag
!
!- End of header

      si_flag = 0

!  COAST only. This requires lsmb to give the land fraction in per cent
!      0 sea ...... 100 land
!  IF AAPP does not have this information this has to be bypassed and 
!      si = -999.9
!
      IF ((lsmb .GT. 0) .AND. (lsmb .LT. 100)) THEN
           si = (t89- t150)   
     +        - (1. - lsmb/100.0) * ( ave_back_sea  + 0.11   * zen) 
     +        - (     lsmb/100.0) * (  0.158 + 0.0163 * zen)
           si_flag = IOR(si_flag,F_COAST)     
      ELSE
         si=-999.9
      ENDIF

! SEA
      IF (lsmb .EQ. 0) THEN
        si = (t89- t150) - ( ave_back_sea  + 0.11   * zen) 
        si_flag = IOR(si_flag,F_OCEAN)     
      ENDIF

! LAND
      IF (lsmb .EQ. 100) THEN
        si = (t89- t150) - (  0.158 + 0.0163 * zen)
        si_flag = IOR(si_flag,F_LAND)     
      ENDIF

! LAND, if T23 is also over land
      IF ((t23 .GT. 0.0) .AND. (lsma .EQ. 100)) THEN
        si = (t23 - t150)  - (-1.7428 + 0.0776 * zen)  
        si_flag = IOR(si_flag,F_SI_INCLUDES_A)     
        si_flag = IOR(si_flag,F_LAND)     
      ENDIF

! Check whether SI is out of range and set flag if so
      IF ((si .LT. MIN_SI) .OR. (si .GT. MAX_SI)) THEN
        si_flag = IOR(si_flag,F_SI_FAIL)
        IF (si .gt. 200 .or. si .lt. -100) si    = -999.9
        m0    = -999.9
        m1    = -999.9 
        m2    = -999.9 
        m3    = -999.9 
      ENDIF

! Now get likelihood for precipitation classes

      IF (IAND(si_flag,F_SI_FAIL) .EQ. 0)       THEN

! Coast and land without AMSU-A
        m0 = nwc_si_coast  (INT(si+21),1)
        m1 = nwc_si_coast  (INT(si+21),2)
        m2 = nwc_si_coast  (INT(si+21),3)
        m3 = nwc_si_coast  (INT(si+21),4) 

! SEA
        IF (IAND(si_flag,F_OCEAN) .EQ. 1) THEN 
          m0 = nwc_si_sea  (INT(si+21),1)
          m1 = nwc_si_sea  (INT(si+21),2)
          m2 = nwc_si_sea  (INT(si+21),3)
          m3 = nwc_si_sea  (INT(si+21),4) 
        ENDIF

! LAND using AMSU-A
        IF ((IAND(si_flag,F_LAND) .EQ. 1) .AND.
     +      (IAND(si_flag,F_SI_INCLUDES_A) .EQ. 1)) THEN 
          m0 = nwc_si_land  (INT(si+21),1)
          m1 = nwc_si_land  (INT(si+21),2)
          m2 = nwc_si_land  (INT(si+21),3)
          m3 = nwc_si_land  (INT(si+21),4) 
        ENDIF

      ENDIF

      END
