!+  Rank an array of reals in increasing order
!
      SUBROUTINE RRANK (NSIZE, ARRAY, LIST)
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: rrank.F 5 2006-01-04 09:11:15Z frna $
!
! Purpose:
!   Rank an array ARRAY of NSIZE reals in order of increasing size.
!
! Method:
!   Not known!
!   This routine was modified by Brian Barwell from an
!   old Oxford University ALGOL routine, and then modified
!   further by Richard Renshaw to bring the FORTRAN up to date.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0    27/11/96  New to ATOVPP.  (R.J.Renshaw)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Subroutine arguments
!
!   Scalar arguments with intent(in):
      INTEGER NSIZE                      ! SIZE OF ARRAY
      REAL ARRAY(NSIZE)                  ! ARRAY OF REALS TO BE RANKED
!
!   Array arguments with intent(Out):
      INTEGER LIST(NSIZE)                ! ranking of elements of ARRAY
!                       e.g. ARRAY=(5.,7.,3.) => LIST=(3,1,2)
!
! Local Scalars:
      INTEGER i,j,k,l,m                  ! loop variables & indices
      INTEGER itmp                       ! temporary store of rank value
!
!- End of header

!     Initialise LIST
      DO i = 1, NSIZE
         LIST(i) = i
      ENDDO  ! i

!     Find the smallest power of 2 greater than NSIZE
      i = 1
      DO WHILE (i.LE.NSIZE)
         i = i + i
      ENDDO

!     Define m to be this -1
      m = i - 1

!     Don't ask!
      DO WHILE (m.GT.1)
         m = m/2
         k = NSIZE - m
         DO j = 1, k
            i = j + m
            DO WHILE (i.GT.m)
               l = i
               i = i - m
               IF (ARRAY(LIST(l)).LT.ARRAY(LIST(i))) THEN ! swap "i" and "l"
                  itmp    = LIST(i)
                  LIST(i) = LIST(l)
                  LIST(l) = itmp
               ELSE                           ! end the DO WHILE (i.GT.m) loop
                  i = 0
               ENDIF
            ENDDO  ! (i.GT.m)
         ENDDO  ! j
      ENDDO  ! (m.GT.1)

      RETURN
      END
