!+ derive AVHRR parameters on ATOVS grid
!
      SUBROUTINE AV_MAP_MAIA ( 
     &           has_maia, 
     &           Map_Nfovs,Map_Nlines,Map_mxch,Map_line0,
     &           Map_chan0,  Map_bitQC,  Map_surf,
     &           Map_Rad, Map_BTs, 
     &           Map_QC, 
     &           Map_bitcloud1,
     &           Targ_Nfovs, Targ_Nlines, Targ_mxch, Targ_line0,
     &           Targ_chan0, Targ_bitQC,
     &           Targ_surf,
     &           Targ_BTs,Targ_QC,
     &           Elms_per_coloc, Max_colocs, LUT_targ_lines,
     &           First_LUT_targ_line, LUT_map_lines, Map_LUT,
     &           Surf_option, Mode, RMDI,
     &           ErrStatus, ErrMessage,
     &           clear_box)

      use mod_maia_types
      use mod_maia_wrapper
      IMPLICIT NONE
!
! Name: $Id$
!
! Purpose:
!   derive AVHRR parameters on ATOVS grid
!
! Method:
!   SR LUTMAP generates a Look-Up-Table (LUT) which identifies those mapping
!   fovs which colocate with a target fov.  AV_MAP uses the colocated AVHRR
!   data to derive several products at ATOVS fields-of-view.
!
!   Whether or not a mapping fov should take part in the
!   mapping can be decided prior to the AV_MAP call and indicated by the
!   setting (or not) of a bit in a QC word.  A further option is available of
!   only using mapping fovs having a like surface to the target fov.
!   Note a block of data is mapped by one call to AV_MAP.
!
!   Map_chan0 is the 'zeroth' mapping channel (i.e. slot in Map_BTs), and
!   Targ_chan0 the 'zeroth' target slot (i.e. slot in Targ_BTs).
!   The parameters output to 'Targ_BTs' are as follows (Mode=1;GAC output):
!      Targ_BTs(*,*,Targ_chan0 + 1 ) = fraction of clear pixels in fov
!                               2-6  = mean albedo/BT chan's 1-5
!                               7-11 = mean albedo/BT 1-5, clear only
!                                12  = standard deviation channel 4
!                                13  = standard deviation channel 4, clear only
!   For this output it is assumed that the AVHRR data will have been through
!   'cloud-mask' processing and that counts will have been converted to albedo
!   or brightness temperature (the processing merely provides statistics on
!   the contents of AVHRR locations).
!
!   The AVHRR will be processed insides boxes centered on each HIRS pixel, a
!   cloud mask will be applied using the MAIA library.
!   if mode = 1
!    Targ_BTS different output: 
!               1 - pourcentage clear AVHRR in HIRS FOV (*100)
!               2 - mean AVHRR channel 1 over HIRS FOV (albedo*10^4)
!               3 - mean AVHRR channel 2 over HIRS FOV (albedo*10^4)
!               4 - mean AVHRR channel 3 over HIRS FOV (degK*100 or albedo*10^4)
!               5 - mean AVHRR channel 4 over HIRS FOV (degK*100)
!               6 - mean AVHRR channel 5 over HIRS FOV (degK*100)
!               7 - mean clear AVHRR channel 1 over HIRS FOV (albedo*10^4)
!               8 - mean clear AVHRR channel 2 over HIRS FOV (albedo*10^4)
!               9 - mean clear AVHRR channel 3 over HIRS FOV (degK*100 or albedo*10^4)
!              10 - mean clear AVHRR channel 4 over HIRS FOV (degK*100)
!              11 - mean clear AVHRR channel 5 over HIRS FOV (degK*100)
!              12 - std  AVHRR channel 4 over HIRS FOV (degK*100)
!              13 - std  clear AVHRR channel 4 over HIRS FOV (degK*100)
!
!   if mode = 2
!   Targ_BTS different output:
!               1 - pourcentage clear AVHRR in HIRS FOV (*100)
!               2 - surface temperature (K*100)
!               3 - climatological temperature or t2m (K*100)
!               4 - mean AVHRR channel 3 over HIRS FOV (degK*100)
!               5 - mean AVHRR channel 4 over HIRS FOV (degK*100)
!               6 - mean AVHRR channel 5 over HIRS FOV (degK*100)
!               7 - percentage of black body in HIRS FOV (*100)
!               8 - top cloud temperature over HIRS FOV (degK*100)
!               9 - standart deviation corresponding to the top cloud T (degK*100)
!              10 - mean clear AVHRR channel 4 over HIRS FOV (degK*100)
!              11 - mean clear AVHRR channel 5 over HIRS FOV (degK*100)
!              12 - std  AVHRR channel 4 over HIRS FOV (degK*100)
!              13 - std  clear AVHRR channel 4 over HIRS FOV (degK*100)                      
!
! History:
! Version  Date      Comment
!
!  1.0      05/07/95  Original code. (P.C.Dibben)
!  1.1      22/02/96  Code adapted along lines of BT_MAP. (R.J.Renshaw)
!  1.2      04/03/97  Major changes to get code working (R.J.Renshaw)
!  1.3      06/10/97  Changes to interface with MAIA library, add 
!                       Map_Rad argument. Surf_option not used in mode 2
!  1.4      dec 2000  Upgrade to MAIA 2.1 P. Brunel
!  1.5      11 June 01 The integer Targ_Nfovs was declared after the array
!                      amsu_wv ==> move the declaration of amsu_wv after the
!                      declaration of Targ_Nfovs (T.Labrot)
!  1.6      9 Janu 02 For mode=1, Targ_BTs is returned with unit*100 to be in
!                     agreement with mode=2   (T.Labrot)
!  1.7      13 April 17 renammed to av_map_maia and use pre computed maia_tab (maia4 output)
!  1.8      24 August 2017 use the new maia_param type
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION 
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH 
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED 
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!

      character *7 satimg         ! satellite name (6char) noaaxx ex: noaa16 
      integer idbg_maia           ! debug level (0= nodebug, 2 is maximum)
      logical has_maia            ! if false maia has not been run and par_maia is not usable
   !   type(maia_param) map_maia(:,:)
!   Mapping data  - all with intent(In):
      INTEGER    Map_Nfovs        ! number of fovs per map line
      INTEGER    Map_Nlines       ! number of lines in Map_BTs
      INTEGER    Map_mxch         ! 1st dimension of Map_BTs
      INTEGER    Map_line0        ! line number of 'zeroth' mapping line
      INTEGER    Map_chan0        ! slot in Map_BTs of 'zeroth' mapping channel
      INTEGER    Map_bitQC        ! bit number of Map_QC set for bad data
      INTEGER    Map_surf(Map_Nfovs,Map_Nlines) ! surface type of mapping fov's
      REAL       Map_Rad(Map_mxch,Map_Nfovs,Map_Nlines) !original radiances
      REAL       Map_BTs(Map_mxch,Map_Nfovs,Map_Nlines) !original b.temps
      INTEGER    Map_QC(Map_Nfovs,Map_Nlines)   ! QC word for each mapping fov
                                  !    (bit 'Map_bitQC' is 0=good, 1=bad fov)
      INTEGER    Map_bitcloud1    ! the 1st of 2 bits in Map_QC word to
                                  ! indicate outcome of cloud-mask processing
      INTEGER    Map_bitcloud2    ! the 2nd of 2 cloud bits in Map_QC word:
                                  !             1st bit    2nd bit
                                  ! clear         1          0
                                  ! cloudy        0          1
                                  ! undefined     0          0
!
!   Target data  - all with intent(In):
      INTEGER    Targ_Nfovs       ! number of fovs per line
      INTEGER    Targ_Nlines      ! number of lines in Targ_BTs
      INTEGER    Targ_mxch        ! 1st dimension of Targ_BTs
      INTEGER    Targ_line0       ! line number of 'zeroth' target line
      INTEGER    Targ_chan0       ! slot in Targ_BTs of 'zeroth' target channel
      INTEGER    Targ_bitQC       ! flag bit in Targ_QC (0=good, 1=bad fov)
      real       Targ_angles(6,Targ_Nfovs,Targ_Nlines) ! geometry for each fov
      real       Targ_alt(Targ_Nfovs,Targ_Nlines) ! surface altitude for each fov
      INTEGER    Targ_surf(Targ_Nfovs,Targ_Nlines) ! surface type for each fov
      INTEGER    amsu_wv(Targ_Nfovs) ! amsu total water vapor (g/cm2*100)
!      
!   Target data  - intent(Out):
      REAL       Targ_BTs(Targ_mxch,Targ_Nfovs,Targ_Nlines) ! mapped values:
!                                 For details, see under 'Method' above.
      INTEGER    Targ_QC(Targ_Nfovs,Targ_Nlines) ! QC word for each fov
                                  !    (bit 'Targ_bitQC' is 0=good, 1=bad fov)
!
!   LUTMAP data  - all with intent(In):
      INTEGER    Elms_per_coloc   ! number of elements of information per coloc
      INTEGER    Max_colocs       ! max number of colocations
      INTEGER    LUT_targ_lines   ! number of target lines represented in LUT
      INTEGER    First_LUT_targ_line ! first target line represented in LUT
      INTEGER    LUT_map_lines    ! number of mapping lines represented in LUT
      REAL       Map_LUT(Elms_per_coloc,Max_colocs,Targ_Nfovs,
     &                   LUT_targ_lines)  ! look-up-table from LUTMAP
!
!
!   Mapping options  - intent(In):
      INTEGER    Surf_option      ! to indicate processing of surface types.
                                  !     0=map irrespective of surface types
                                  !     1=map only if surface type of mapping
                                  !       fov is same as that of target fov
      INTEGER    Mode             ! mapping mode, used to indicate alternative
                                  ! processing. Mode=1 for GAC-style output.
      REAL       RMDI             ! real missing data indicator
!
!   Subroutine return status  - intent(Out):
      INTEGER       ErrStatus     ! error status (0 = OK)
      CHARACTER*80  ErrMessage    ! error message
!
! Local parameters:
      INTEGER    Nchans           ! number of AVHRR channels to be mapped
      PARAMETER (Nchans = 5)
      INTEGER    mx_chan          ! number of mapping parameters calculated
      PARAMETER (mx_chan = 13)    !  - for this version of the code
                                  ! (see array 'sum' below)
!
! Local scalars:
      INTEGER    chan             ! channel/slot number
      INTEGER    count_sum        ! number of values summed in 'sum'
      INTEGER    count_clear      ! number of clear pixels summed
      LOGICAL    fov_ok           ! TRUE if mapping fov is OK to use
      INTEGER    map_line_offset  ! offset to be added to a LUT mapping
                                  ! line number to give a `1c' line number 
      INTEGER    offset           ! used in calculating relative line numbers
      INTEGER    modulo_line      ! used in calculating relative line numbers
      INTEGER    no_entries       ! number of colocated fov's in LUT
      INTEGER    LUT_line         ! appropriate LUT entry line
      INTEGER    map_line         ! line number for mapped instrument
      INTEGER    map_spot         ! fov number for mapped instrument (in loop)
      INTEGER    map_start_spot   ! .. in colocation line
      INTEGER    map_end_spot     ! .. in colocation line
      INTEGER    ii               ! loop variable, target instrument line number
      INTEGER    jj               ! loop variable, target instrument fov
      INTEGER    kk               ! loop variable, colocated fov's
      INTEGER    ll               ! loop variable, channels
      INTEGER    tt               ! loop variable
      REAL       xsum             ! inverse of count, used for mean & s.dev


! Interface with MAIA 
!       Box is a rectangular AVHRR array centered on the HIRS pixel.
!       The size is map_box_psize pixels by map_box_lsize lines
!       The values of 33x38 is enough for HIRS
!       box_map gives the HIRS field of view in the AVHRR scene, 
!               0 is for pixels inside the Hirs fov
!               1 is for pixels outside
!               2 is for missing lines or bad pixels
!          box_map can be filled with zeros for the processing of the all box
!       box_tb contains the brightness temperature 
!       box_rad contains the radiances
!       local is used for the definition of the Targ_BTS output


      integer map_box_pixel
      integer map_box_line
      real box_rad(map_box_psize, map_box_lsize,5)
      real box_tb(map_box_psize, map_box_lsize,5)
      !type(maia_param) maia_box( map_box_psize, map_box_lsize)

      integer box_map(map_box_psize, map_box_lsize)
      logical local
      logical bad_line     
      integer pp
      integer c

      integer clear_box(map_box_psize,map_box_lsize) !33,38
      real maia_info(13)
! Local arrays:
      REAL*8     sum(mx_chan-1)   ! to sum parameters for Mode=2,3:
                                  ! 1-5  = sum albedo/BT chan's 1-5
                                  ! 6-10 = sum albedo/BT 1-5, clear only
                                  !  11  = sum chan 4 squared  (to get std dev)
                                  !  12  = sum chan 4 squared, clear only

! Functions
!- End of header
      

! 1.  Initialisation & check input parameters
!     -------------- _ _____ _____ __________

! 1.1 Check input parameters are within range and consistent

      ErrStatus = 0
      
      IF ( Mode.NE.1 .and. Mode.NE.2 ) THEN
         ErrStatus = 1
         ErrMessage = ' '
         WRITE ( ErrMessage,
     &   '('' Mode in AV_MAP not available: Mode = '',I8)' ) Mode
         GOTO 999
      ENDIF

      IF ( Nchans.LT.1 .OR. Nchans.GT.Map_mxch ) THEN
         ErrStatus = 2
         ErrMessage = ' '
         WRITE ( ErrMessage,
     &   '('' Nchans in AV_MAP outside range: Nchans,Map_mxch= '',2I8)'
     &   ) Nchans, Map_mxch
         GOTO 999
      ENDIF

!     look through all channels to check they are all valid channel numbers
      DO ll = 1, Nchans

         chan = ll + Map_chan0
         IF ( chan.LT.1 .OR. chan.GT.Map_mxch ) THEN
            ErrStatus = 3
            ErrMessage = ' '
            WRITE ( ErrMessage,
     &      '('' Bad mapping channel in AV_MAP: chan,ll,Map_mxch = '',
     &      3I8)' )  chan, ll, Map_mxch
            GOTO 999
         ENDIF

         chan = ll + Targ_chan0
         IF ( chan.LT.1 .OR. chan.GT.Targ_mxch ) THEN
            ErrStatus = 4
            ErrMessage = ' '
            WRITE ( ErrMessage,
     &      '('' Bad target channel in AV_MAP: chan,ll,Targ_mxch = '',
     &      3I8)' )  chan, ll, Targ_mxch
            GOTO 999
         ENDIF
         
      ENDDO  ! ll

!-------------------------------------------------------------------------
! MODE=1: GAC-style output
!
      IF ( Mode.EQ.1 ) THEN

! 2.  Loop through all target lines
!     ---- ------- --- ------ -----

      DO ii = 1, Targ_Nlines

! 2.1   Calculate line number relative to look-up-table from LUTMAP
        offset = Targ_line0 + ii - First_LUT_targ_line
        modulo_line = MOD(offset,LUT_targ_lines)
        IF (modulo_line.LT.0) modulo_line = modulo_line+LUT_targ_lines
        LUT_line = modulo_line + 1         ! appropriate LUT entry line

!       now calculate a mapping line offset ie the offset between the mapping
!       LUT line and the level 1c mapping line. This offset will be some integer
!       multiple of the number of mapping lines in the duration of the LUT.
        IF (offset.GE.0) THEN
          map_line_offset = (offset/LUT_targ_lines) * LUT_map_lines
        ELSE
          IF (modulo_line.EQ.0) THEN
            map_line_offset = (offset/LUT_targ_lines) * LUT_map_lines
          ELSE
            map_line_offset = ((offset-LUT_targ_lines)/LUT_targ_lines)*
     &                        LUT_map_lines
          ENDIF
        ENDIF

! 2.2   Loop through all target fovs in line
!       ---- ------- --- ------ ---- -- ----
        DO jj = 1, Targ_Nfovs

! 2.2.1    Initialise variables for this fov
           count_sum   = 0   ! number of values summed in 'sum'
           count_clear = 0   ! number of clear pixels summed
!          number of colocation entries in LUT
           no_entries = NINT(Map_LUT(Elms_per_coloc,1,jj,LUT_line))
!          set all summation variables to zero
           DO ll = 1, mx_chan-1
              sum(ll) = 0.D0
           ENDDO  ! ll

!          loop through colocations for target fov
           DO kk = 2, no_entries+1

! 2.2.2       get line & spot number for colocated mapping fov's
              map_start_spot = NINT(Map_LUT(2,kk,jj,LUT_line))   ! spot range
              map_end_spot = NINT(Map_LUT(3,kk,jj,LUT_line))   ! in coloc line
              map_line = NINT(Map_LUT(1,kk,jj,LUT_line)) + 1
     &                   + map_line_offset - Map_line0   ! LUT coloc line
!             Check that line is in Map_BTs
              IF (map_line.GE.1 .AND. map_line.LE.Map_Nlines) THEN

!                loop through colocations in this line
                 DO map_spot = map_start_spot, map_end_spot

! 2.2.3             check QC flag

          
                    fov_ok =
     &                .NOT.BTEST(Map_QC(map_spot,map_line),Map_bitQC)

! 2.2.4             if required, check surface type
                    IF (Surf_option.NE.0) fov_ok = fov_ok .AND.
     &                 Map_surf(map_spot,map_line) .EQ. Targ_surf(jj,ii)

! 2.2.5             Loop through channels, summing values
                    IF (fov_ok) THEN

                       count_sum = count_sum + 1   ! number of values summed

!                      1-5 = sum albedo/BT chan's 1-5
                       DO ll = 1, Nchans
                          sum(ll) = sum(ll) +
     &                      Map_BTs(Map_chan0+ll,map_spot,map_line)
                       ENDDO  ! ll

!                      11  = sum chan 4 squared  (to get std dev)
                       sum(11) = sum(11) +
     &                    Map_BTs(Map_chan0+4,map_spot,map_line)**2

!                      Is this a clear pixel ?

                       IF (
     &                 BTEST(Map_QC(map_spot,map_line),Map_bitcloud1)
     &                 ) THEN
     
                          count_clear = count_clear + 1 !number of clear pixels

!                         6-10 = sum albedo/BT 1-5, clear only
                          DO ll = 1, Nchans
                             sum(5+ll) = sum(5+ll) +
     &                         Map_BTs(Map_chan0+ll,map_spot,map_line)
                          ENDDO  ! ll

!                         12  = sum chan 4 squared, clear only
                          sum(12) = sum(12) +
     &                       Map_BTs(Map_chan0+4,map_spot,map_line)**2

                       ENDIF  ! clear pixel
                    ENDIF  ! fov_ok
                 ENDDO  ! map_spot

              ENDIF                      ! Map line OK

            ENDDO  ! kk                  ! LUT colocations

! 2.2.6     Loop through channels again, to calculate means & std dev's

!           Initially set everything to missing, and flag
            DO ll = 1, mx_chan
               Targ_BTs(Targ_chan0+ll,jj,ii) = RMDI
            ENDDO  ! ll
            Targ_QC(jj,ii) = IBSET(Targ_QC(jj,ii),Targ_bitQC)

!           When we have some values (clear or cloudy)...
            IF (count_sum.GT.0) THEN

               xsum = 1. / REAL(count_sum)

!              1 = fraction of clear pixels in fov
               Targ_BTs(Targ_chan0+1,jj,ii) = REAL(count_clear) * xsum

!              2-6 = mean albedo/BT chan's 1-5
               DO ll = 1, Nchans
                  Targ_BTs(Targ_chan0+ll+1,jj,ii) = sum(ll) * xsum
               ENDDO  ! ll

!              12  = standard deviation channel 4
               Targ_BTs(Targ_chan0+12,jj,ii) =
     &         SQRT( sum(11)/REAL(count_sum-1) - (sum(4)*xsum)**2 )

!              Clear flag to show that there is data in target fov
               Targ_QC(jj,ii) = IBCLR(Targ_QC(jj,ii),Targ_bitQC)

            ENDIF  ! count_sum.GT.0

!           When we have some clear values...
            IF (count_clear.GT.0) THEN

               xsum = 1. / REAL(count_clear)

!              7-11 = mean albedo/BT 1-5, clear only
               DO ll = 1, Nchans
                  Targ_BTs(Targ_chan0+ll+6,jj,ii) = sum(ll+5) * xsum
               ENDDO  ! ll

!              13  = standard deviation channel 4, clear only
               Targ_BTs(Targ_chan0+13,jj,ii) =
     &         SQRT( sum(12)/REAL(count_clear-1) - (sum(9)*xsum)**2 )

            ENDIF  ! count_clear.GT.0
            
! Put Targ_BTs in unit*100 to be written in the result file
! Agreement with mode=2
            DO ll = 1, mx_chan
              Targ_BTs(Targ_chan0+ll,jj,ii) = 
     &                      Targ_BTs(Targ_chan0+ll,jj,ii)*100
            ENDDO
            
         ENDDO  ! jj                       ! target fovs in line

        ENDDO  ! ii = 1, Targ_Nlines       ! target lines
             
!
! End of MODE=1: GAC-style output
!-------------------------------------------------------------------------
      ENDIF

! MODE=2: Compute the AVHRR cloud mask
!-------------------------------------------------------------------------
!
      IF ( Mode.EQ.2) THEN

! 3.  Loop through all target lines
!     ---- ------- --- ------ -----
      DO ii = 1, Targ_Nlines

! 3.1   Calculate line number relative to look-up-table from LUTMAP
        offset = Targ_line0 + ii - First_LUT_targ_line
        modulo_line = MOD(offset,LUT_targ_lines)
        IF (modulo_line.LT.0) modulo_line = modulo_line+LUT_targ_lines
        LUT_line = modulo_line + 1         ! appropriate LUT entry line

!       now calculate a mapping line offset ie the offset between the mapping
!       LUT line and the level 1c mapping line. This offset will be some integer
!       multiple of the number of mapping lines in the duration of the LUT.
        IF (offset.GE.0) THEN
          map_line_offset = (offset/LUT_targ_lines) * LUT_map_lines
        ELSE
          IF (modulo_line.EQ.0) THEN
            map_line_offset = (offset/LUT_targ_lines) * LUT_map_lines
          ELSE
            map_line_offset = ((offset-LUT_targ_lines)/LUT_targ_lines)*
     &                        LUT_map_lines
          ENDIF
        ENDIF

! 3.2   Loop through all target fovs in line
!       ---- ------- --- ------ ---- -- ----

        DO jj = 1, Targ_Nfovs

! 3.2.1    Initialise variables for this fov
!          number of colocation entries in LUT
           
           no_entries = NINT(Map_LUT(Elms_per_coloc,1,jj,LUT_line))

           ! First pixel and line of the AVHRR box
           map_box_pixel =  NINT(Map_LUT(2,1,jj,LUT_line)) -
     &                                   map_box_psize/2
           
           map_box_line  =  NINT(Map_LUT(1,1,jj,LUT_line)) + 1
     &                   + map_line_offset - Map_line0     - 
     &                     map_box_lsize/2
     

           if(map_box_pixel.le.0) then
              map_box_pixel=1
           end if
           if(map_box_line.le.0) then
              map_box_line=1
           end if
           if(map_box_pixel+map_box_psize-1.gt.map_nfovs) then
              map_box_pixel = map_nfovs - map_box_psize +1
           end if
           if(map_box_line+map_box_lsize-1.gt.map_nlines) then
              map_box_line = map_nlines - map_box_lsize +1
           end if

           ! create the Bts, Rads and maia boxes
           do ll =1 , map_box_lsize
              do pp = 1, map_box_psize
                 do c=1,5
                 box_tb(pp,ll,c) = 
     &            map_bts(c,pp+map_box_pixel-1, ll+map_box_line-1)
                 end do
              end do
           end do
           do ll =1 , map_box_lsize
              do pp = 1, map_box_psize
                 do c=1,5
                 box_rad(pp,ll,c) = 
     &            map_rad(c,pp+map_box_pixel-1, ll+map_box_line-1)
                 end do
              end do
           end do 
           do ll =1 , map_box_lsize
              do pp = 1, map_box_psize
                 call maia_param_copy_item 
     & (map_maia(pp+map_box_pixel-1, ll+map_box_line-1),maia_box(pp,ll))               
              end do
           end do

           ! "mapping" 1 outside the ellispe 0 inside
           ! and 2 for avhrr missing lines
           do ll =1 , map_box_lsize
              do pp = 1, map_box_psize
                 box_map(pp,ll) = 1
              end do
           end do
!          loop through colocations for target fov
           DO kk = 2, no_entries+1

! 3.2.2       get line & spot number for colocated mapping fov's
              map_start_spot = NINT(Map_LUT(2,kk,jj,LUT_line))   ! spot range
              map_end_spot = NINT(Map_LUT(3,kk,jj,LUT_line))   ! in coloc line
              map_line = NINT(Map_LUT(1,kk,jj,LUT_line)) + 1
     &                   + map_line_offset - Map_line0   ! LUT coloc line
!             Check that line is in Map_BTs
              IF (map_line.GE.1 .AND. map_line.LE.Map_Nlines) THEN
                 ll=map_line - map_box_line +1
!                loop through colocations in this line
                 bad_line=.false.
                 DO map_spot = map_start_spot, map_end_spot
                    fov_ok =
     &                .NOT.BTEST(Map_QC(map_spot,map_line),Map_bitQC)
                    if(fov_ok) then
                       pp = map_spot - map_box_pixel +1 
                       box_map(pp,ll) = 0

                    else
                       bad_line=.true.
                    end if
                 ENDDO  ! map_spot

                 if(bad_line) then
                    do pp=1,map_box_psize
                       box_map(pp,ll) = 2 ! missing pixel
                    end do
                 end if

              ENDIF! Map line OK

           ENDDO! kk                  ! LUT colocations 

!          Initially set everything to missing, and flag
           DO ll = 1, mx_chan
              Targ_BTs(Targ_chan0+ll,jj,ii) = RMDI
           ENDDO  ! ll
           Targ_QC(jj,ii) = IBSET(Targ_QC(jj,ii),Targ_bitQC)


           local =.true.
           

           call fill_maia(has_maia,
     &               box_tb,
     &               box_rad,
     &               box_map,
     &               local,
     &               maia_info, 
     &               clear_box)

           
            do tt=1,13
               Targ_BTs(tt,jj,ii)= maia_info(tt)
            end do
!           Clear flag to show that there is data in target fov
            Targ_QC(jj,ii) = IBCLR(Targ_QC(jj,ii),Targ_bitQC) 


        ENDDO! jj                       ! target fovs in line
      ENDDO! ii = 1, Targ_Nlines       ! target lines
!
! End of MODE=2: 
!-------------------------------------------------------------------------

      ENDIF

 999  CONTINUE   ! jump here on error

      RETURN
      END

