!
!NOM/NAME $Id: ppellip.F 25 2006-02-03 15:43:53Z frna $
!
      subroutine ppellip(npb, nlb, tavh_b, ravh_b, map, lside,
     &                  tavh_el, ravh_el, sd33_el, el_lut, np)

!-----------------------------------------------------------------------
! ... INTRODUCTION 
! ... Pre-Processing for AVHRR data inside a HIRS ellipse
!
! ... create the ellipse arrays and compute the local standard deviation
! ... in channel 4
!-----------------------------------------------------------------------
! ... arguments:
! inputs
!       npb                     -- input pixel size of the box
!                               integer
!       nlb                     -- input line size of the box
!                               integer
!       tavh_b(npb,nlb,5)       -- input brightness temperature on box
!                               channels 1, 2 albedo en %
!                               channels 3, 4, 5 Tb (degK) 
!                               real 
!       ravh_b(npb,nlb,5)       -- input radiance on box
!                               channels 3, 4, 5  (mw/...) 
!                               real 
!       map(npb,nlb)            -- mapping (0: in the ellipse, 1 outside
!                                2 for missing lines )
!                               integer 
!       lside                   input length of side of square in pixels
!                               integer
! outputs
!       tavh_el                 brightness temperature on ellipse and albedos
!                               K*100  Alb (%*100)
!                               real
!       ravh_el                 radiance on ellipse mW/cm2/st/cm-1
!                               real 
!       sd33_el                 standard deviations for pixels inside ellipse
!                               K*100
!                               real
!       el_lut                  look up table: tavh_b(el_lut(n)) == tavh_el(n)
!                               integer
!       np                      no of values inside the ellipse
!                               integer
!-----------------------------------------------------------------------
# include "avh2hirs.h"
!-----------------------------------------------------------------------
! ... DECLARATIONS 

! ... scalar arguments ..
      integer lside
      integer np
! ...
! ... array arguments ..
      integer npb
      integer nlb
      real tavh_b(npb,nlb,5)
      real ravh_b(npb,nlb,5)
      integer map(npb,nlb)
      real tavh_el(nb_el_max,5)
      real ravh_el(nb_el_max,5)
      integer el_lut(2,nb_el_max)       ! 1 --> pixel  2 --> line
      real sd33_el(nb_el_max)
! ...
! ... local scalars ..
      double precision x,xsum,xsumsq,xmean
      integer i,ial,ias,j
      integer nx
      integer h_size
! ...
! ... intrinsic functions ..
      intrinsic sqrt
!-----------------------------------------------------------------------

! ... half-size of the box      lside=2 creates a 3x3 local box
! ... and                       lside=4 creates a 5x5 local box
      h_size = lside/2

! ... initialisation
      do n=1,nb_el_max
           do ic=1,5
              tavh_el(n,ic)= 0
           end do
           do ic=3,5
              ravh_el(n,ic)= 0
           end do
           el_lut(1,n) = 0
           el_lut(2,n) = 0
           sd33_el(n) = 0
      end do
            
      np = 0
      do ial = 1,nlb
         do ias = 1,npb
         
            if (map(ias,ial).eq.0) then         ! pixel inside the ellipse
               np = np + 1
               if(np .gt. nb_el_max) then
                  call ml_wti('warning',
     &            'maximum number of pixels in the ellipse is reached',
     &            nb_el_max)
                  np = nb_el_max
                  return
               end if
               ! ellipse look up table
               el_lut(1,np)=ias
               el_lut(2,np)=ial
               ! create the ellipse array and change units
               do ic=1,5
                  tavh_el(np,ic)= tavh_b(ias,ial,ic)*100.
               end do
               do ic=3,5
                  ravh_el(np,ic)= ravh_b(ias,ial,ic)
               end do
      
               ! local channel 4 standard deviation 
               xmean = 0.d0
               xsum = 0.d0
               xsumsq = 0.d0
               nx=0
               do j = ial-h_size, ial+h_size
                  do i = ias-h_size, ias+h_size
                     if(i.gt.0 .and. i.le.npb .and.
     &                  j.gt.0 .and. j.le.nlb) then
                        if(map(i,j).le.1) then
                           nx = nx+1
                           x = tavh_b(i,j,4)
                           xsum = xsum + x
                           xsumsq = xsumsq + x*x
                        end if
                     end if
                  end do
               end do
               if(nx.gt.0) then
                  xmean = xsum/nx
                  x = xsumsq/nx - xmean*xmean
               else
                  xmean = 0.d0
                  x = 0.d0
               end if
               if (x.lt.0.) then
                  sd33_el(np)=0.
               else
                  sd33_el(np) = sqrt(x) *100.
               end if

            end if
            
         end do
      end do
 
      return
      end
