/* Name: $Id$*/
/* this code has been extracted from OPS-LRS V7 and adapted for stand alone execution*/
#include <stdio.h>
#include <Nuees.h>
#include <math.h>
#include <stdlib.h>
#include <lapack-lite.h>

typedef int bool;
enum { False, True };
/*====================================================================*/
// Local types Declaration
/*====================================================================*/
typedef struct __type_paire_i_val
{
  double val;
  int    i;
} type_paire_i_val;


typedef struct __type_sort_eigen
{
/* bma complexes->reels */
  double val;
  double vect[SD_BAS_NBK];
/* fin bma */
} type_sort_eigen;

const double MAX_DOUBLE=   1.0E100;

void print_array_1d_double (double * tab,int size,char * comment, char * name)
{
int i;
int max;
max=size;
if (size <= 30) max = 30;
printf(">> %s \n",comment);
for (i=0;i<max;i++) {
    printf("%s[%d]=%e\n",name,i,tab[i]);
}
}



//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_NueesDynamiques
//@
//@ objet :  Traitement de l'image AVHRR
//@
//@
//@ Parametres :
//@         (ENTREE) ProduitCourant ...... : Pour pointer sur les variables temporaires
//@         (ENTREE) Contexte_courant .... : contexte de l'etape (numero de ligne SN traite)
//@         (ENTREE) RadAv ............... : Vecteur de radiance de l'image AVHRR
//@         (ENTREE) NbPixAv ............. : Nombre de pixels du vecteur
//@         (ENTREE) Nch ................. : Nombre de canaux presents
//@         (ENTREE) SortingChannel ...... : Canal de rangement
//@         (ENTREE) ChannelNoise ........ : Bruit des canaux
//@         (ENTREE) ChannelWgt .......... : Poids des canaux
//@         (ENTREE) NbClusMax ........... : Nombre maximal de clusters
//@         (ENTREE) NbIterMax ........... : Nombre maximal d'iterations
//@         (ENTREE) IterConvCutoff ...... : Seuil de convergence
//@         (ENTREE) SplitCutoff ......... : Seuil d'eclatement
//@         (ENTREE) GlueCutoff .......... : Seuil de coalescence
//@         (ENTREE) EigValCutoff ........ : Seuil de valeur propre
//@         (ENTREE) DistAgloInit ........ : Valeur initiale de la distance d'aglomeration
//@         (ENTREE) MinIterNCCutOff ..... : Nombre d'iteration minimales pour le calcul du seuil des non classifies
//@         (ENTREE) NbNonClassifMax ..... : Nombre maximal de non classifies
//@         (ENTREE) DistMax ............. : Distance maximale interclasses
//@
//@         (SORTIE) IdPix ............... : Flags de pixels valides
//@         (SORTIE) NoExt ............... : Flags de classes non extremes
//@         (SORTIE) NbClass ............. : Nombre de classes
//@         (SORTIE) FlagPostProcessing .. : Indique si le traitement a utilise un post processing ou non
//@         (SORTIE) NonClassifRate ...... : Taux de pixels non classes
//@         (SORTIE) VarianceRate ........ : Rapport : variance image classifiee / image non classifiee
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_NueesDynamiques(
  // Entrees
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  t_SD_BAS_DataThread  *Contexte_courant,    // t_SD_BAS_DataThread*
  float                *RadAv,               // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
                                             // Considere comme NBK vecteurs de NPI_AVHRR * NLI_MAX_UTIL_AVHRR valeurs
  int                   NbPixAv,             // int(1)
  int                   Nch,                 // int(1)
  int                   SortingChannel,      // int(1)
  double               *ChannelNoise,        // double( SD_BAS_NBK)
  int                  *ChannelWgt,          // int( SD_BAS_NBK)
  int                   NbClusMax,           // int(1)
  int                   NbIterMax,           // int(1)
  double                IterConvCutoff,      // double(1)
  double                SplitCutoff,         // double(1)
  double                GlueCutoff,          // double(1)
  double                EigValCutoff,        // double(1)
  double                DistAgloInit,        // double(1)
  int                   MinIterNCCutOff,     // int(1)
  int                   NbNonClassifMax,     // int(1)
  double                DistMax,             // double(1)
  // Sorties
  int                  *IdPix,               // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  unsigned int         *NoExt,               // unsigned int( SD_BAS_NCL_MAX)
  int                  *NbClass,             // int(1)
  unsigned int         *FlagPostProcessing,  // unsigned int(1)
  double               *NonClassifRate,      // double(1)
  double               *VarianceRate         // double(1)
)
{
  static const char tcProcName_F[] = "SD_ALG_NueesDynamiques";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  // Definitions
  // (#) (1 a Nch) nombre de canaux/pseudo-canaux Avhrr utilises
  // (*) (1 a NbPixAv) nombre de pixels Avhrr
  // ($) (1 a NbPro) nombre de classes prototype
  double   RadMean[SD_BAS_NBK];                      // double( SD_BAS_NBK)
  double   RadStd[SD_BAS_NBK];                       // double( SD_BAS_NBK)
  float  *RadNW;                                    // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
  double  ProtoZero[SD_BAS_NBK];                    // double( SD_BAS_NBK)
/* bma complexes->reels */
  double  EigVal[SD_BAS_NBK];                       // double( SD_BAS_NBK)
  double  EigVect[SD_BAS_NBK*SD_BAS_NBK];           // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
  int     NbPro;                                    // int(1)
  double  Proto[SD_BAS_NBK*SD_BAS_NCL_MAX];             // double( SD_BAS_NCL_MAX * SD_BAS_NBK)
  double  Covar[SD_BAS_NBK*SD_BAS_NBK*SD_BAS_NCL_MAX];  // double( SD_BAS_NBK * SD_BAS_NBK * SD_BAS_NCL_MAX)
  int     NbPixClas[SD_BAS_NCL_MAX];                    // int( SD_BAS_NCL_MAX)
  double  SumDistClas[SD_BAS_NCL_MAX];                  // double( SD_BAS_NCL_MAX)
  double *Dist;                                     // double( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER * SD_BAS_NCL_MAX)
  int     Iter, IterLast;                           // int(1)
//CR 236 modif pixels non classes CCS
  int 	Ipix; // boucle sur les pixels
  int  Ich,Ipro; // boucle d'initialisation
  int myI;
  FILE * myFileIn;
  FILE * myFileOut;

       
#ifdef CMS_DEBUG
    printf (" NueesDynamiques NbPixAv ............. : Number of pixels %d\n",NbPixAv);
    printf (" NueesDynamiques Nch ................. : Number of channels %d\n",Nch);
    printf (" NueesDynamiques NbClusMax ........... : Maximum number of clusters %d\n",NbClusMax);
    printf (" NueesDynamiques NbIterMax ........... : Maximum number of iterations %d\n",NbIterMax);
    printf (" NueesDynamiques IterConvCutoff..... . : Convergence threshold %f\n",IterConvCutoff);
    printf (" NueesDynamiques SplitCutoff ......... : Split threshold %f\n",SplitCutoff );
    printf (" NueesDynamiques GlueCutoff .......... : Coalescence threshold %f\n",GlueCutoff );
    printf (" NueesDynamiques EigValCutoff ........ : Seuil de valeur propre %f\n",EigValCutoff);
    printf (" NueesDynamiques DistAgloInit ........ : Initial value of the aglomeration distance %f\n",DistAgloInit);
    printf (" NueesDynamiques MinIterNCCutOff ..... : Minimum number of iteration for the calculation of the unclassified threshold. %d\n",MinIterNCCutOff);
    printf (" NueesDynamiques NbNonClassifMax ..... : Maximum number of unclassified %d\n",NbNonClassifMax);
    printf (" NueesDynamiques DistMax ............. : Maximum interclass distance %f\n",DistMax);
    for (myI=1;myI<= SD_BAS_NBK; myI++) {
    printf (" NueesDynamiques channel %d bruit %f poids %d\n", myI, ChannelNoise[myI-1],ChannelWgt [myI-1]);
    }
   printf("Input RadVect:\n");
   for (int k=1; k<10; k++){
    for (int c=1; c<= Nch; c++){
     printf("%e ", RadAv[IND2(k,c,SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)]);
     }
     printf("\n");
   }
#endif
 
// initialisation du tableau
  for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
  {
    // (#) = (1 a NbChannel) nombre de canaux
    for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
    {
      // ProtoWork(#,1 a NbPro) = Proto(#,1 a NbPro)
      Proto[IND2(Ich,Ipro,SD_BAS_NBK)]= 0.;
    }
  }
  RadNW= ProduitCourant->RadNW;
  Dist= ProduitCourant->Dist;

  // 1) Preparation des radiances et du prototype zero
  cr_op= SD_ALG_InitRad(
           // Entrees
           ProduitCourant,    // t_SD_BAS_DataThreadProduit*
           Contexte_courant,  // t_SD_BAS_DataThread*
           RadAv,             // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
           NbPixAv,           // int(1)
           Nch,               // int(1)
           ChannelNoise,      // double( SD_BAS_NBK)
           ChannelWgt,        // int( SD_BAS_NBK)
           // Sorties
           RadMean,           // double( SD_BAS_NBK)
           RadStd,            // double( SD_BAS_NBK)
           RadNW,             // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
           ProtoZero,         // double( SD_BAS_NBK)
/* bma complexes->reels */
           EigVal,            // double( SD_BAS_NBK)
           EigVect);          // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
 
  // 2) Recherche du premier echantillon de prototypes
  cr_op= SD_ALG_InitProto(
           // Entrees
            Contexte_courant,      // t_SD_BAS_DataThread*
            ProduitCourant,  // t_SD_BAS_DataThreadProduit*
            RadNW,           // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
            Nch,             // int(1)
            NbPixAv,         // int(1)
            ProtoZero,       // double( SD_BAS_NBK)
            NbClusMax,       // int(1)
            DistMax,         // double(1)
            SortingChannel,  // int(1)
           // Sorties
           &NbPro,           // int(1)
            Proto,           // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
            Covar,           // double( SD_BAS_NBK * SD_BAS_NBK * SD_BAS_NCL_MAX)
            Dist,            // double( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NCL_MAX)
            IdPix,           // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
            NbPixClas,       // int( SD_BAS_NCL_MAX)
            SumDistClas);    // double( SD_BAS_NCL_MAX)
     if(cr_op != CMN__STATUT_OK)
     {
         goto fin;
     }

  // 3) Processus iteratif de caracterisation des classes
  cr_op= SD_ALG_IterNuDyn(
           // Entree
              Contexte_courant,      // t_SD_BAS_DataThread*
            ProduitCourant,  // t_SD_BAS_DataThreadProduit*
            RadNW,           // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
            NbPixAv,         // int(1)
            Nch,             // int(1)
            NbPro,           // int(1)
            NbClusMax,       // int(1)
            NbIterMax,       // int(1)
/* bma complexes->reels */
            EigVal,          // double( SD_BAS_NBK)
            EigVect,         // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
            DistAgloInit,    // double(1)
            SplitCutoff,     // double(1)
            GlueCutoff,      // double(1)
            EigValCutoff,    // double(1)
            MinIterNCCutOff, // int(1)
            NbNonClassifMax, // int(1)
            SortingChannel,  // int(1)
            DistMax,         // double(1)
            IterConvCutoff,  // double(1)
           // Entrees / Sorties
            Proto,           // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
            IdPix,           // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
            NbPixClas,       // int( SD_BAS_NCL_MAX)
            SumDistClas,     // double( SD_BAS_NCL_MAX)
           // Sortie
            NbClass,         // int(1)
            NoExt,           // unsigned int( SD_BAS_NCL_MAX)
           &Iter);           // int(1)
     if(cr_op != CMN__STATUT_OK)
     {
         goto fin;
     }

  // 4) Post-processing
  *FlagPostProcessing = CMN__NON;
  
  // Si NbClass > NbClusMax alors
  if( *NbClass > NbClusMax)
  {
    IterLast= Iter;
    #ifdef CMS_DEBUG
     printf("IterNuDynPost IterLast=%d\n",IterLast);
    #endif
    cr_op= SD_ALG_IterNuDynPost(
             // Entrees
              ProduitCourant,  // t_SD_BAS_DataThreadProduit*
              IterLast,        // int(1)
              NbPixAv,         // int(1)
              Nch,             // int(1)
              NbClusMax,       // int(1)
              EigVal,          // double( SD_BAS_NBK)
              EigVect,         // double( SD_BAS_NBK * SD_BAS_NBK)
              DistAgloInit,    // double(1)
              EigValCutoff,    // double(1)
              SortingChannel,  // int(1)
             // Entrees / Sorties
              Proto,           // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
              IdPix,           // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
              NbPixClas,       // int( SD_BAS_NCL_MAX)
              SumDistClas,     // double( SD_BAS_NCL_MAX)
              NoExt,           // unsigned int( SD_BAS_NCL_MAX)
             // Sortie
              NbClass,         // int(1)
             &Iter);           // int(1)

    *FlagPostProcessing= CMN__OUI;
  }

  // 5) Tests qualite :
  // Si Post-processing alors ... Passe au dessus
  cr_op= SD_ALG_TestQual(
           // Entrees
            NbPixAv,        // int(1)
            Nch,            // int(1)
           *NbClass,        // int(1)
/* bma complexes->reels */
            EigVal,         // double( SD_BAS_NBK)
/* fin bma */
            EigValCutoff,   // double(1)
            NbPixClas,      // int( SD_BAS_NCL_MAX)
            SumDistClas,    // double( SD_BAS_NCL_MAX)
           // Sortie
            NonClassifRate, // double(1)
            VarianceRate);  // double(1)

//CR 236 modif pixels non classes CCS
// on affecte ici tous les pixels non classes a la classe 7
// initialement ils sont a la classe NbClass+1 on les transforme en 7
// ceci est valable pour toutes les sorties de l'algo NueesDynamiques
// c.a.d IdPix 

 // Boucle sur les pixels (Ipix = 1 a NbPixAv)
for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
  {
  if ( IdPix[IND1(Ipix)] == *NbClass + 1 )
   {
    IdPix[IND1(Ipix)] = 7;
   }
  } // Fin de boucle sur les pixels
*NbClass = 7;
//CR 236 modif pixels non classes CCS

fin:

  // Fin de la fonction NueesDynamiques
  #ifdef OPS_DEBUG
    printf( "End NueesDynamiques\n----------------------------------\n");
  #endif

  
//@         (SORTIE) IdPix ............... : Flags de pixels valides
//@         (SORTIE) NoExt ............... : Flags de classes non extremes
//@         (SORTIE) NbClass ............. : Nombre de classes
//@         (SORTIE) FlagPostProcessing .. : Indique si le traitement a utilise un post processing ou non
//@         (SORTIE) NonClassifRate ...... : Taux de pixels non classes
//@         (SORTIE) VarianceRate ........ : Rapport : variance image classifiee / image non classifiee
      // Sorties
 // int                  *IdPix,               // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
 // unsigned int         *NoExt,               // unsigned int( SD_BAS_NCL_MAX)
 // int                  *NbClass,             // int(1)
 // unsigned int         *FlagPostProcessing,  // unsigned int(1)
//  double               *NonClassifRate,      // double(1)
//  double               *VarianceRate         // double(1)
#if CMS_DEBUG
  for ( myI =1; myI <=  SD_BAS_NCL_MAX ; myI++) {
    printf("NueesDynamiques Flags de classes non extremes %d -> %d\n", myI,NoExt[myI-1]);
  };
  printf("NueesDynamiques OUTPUT NbClass %d\n",*NbClass);
  printf("NueesDynamiques OUTPUT lagPostProcessing %d\n",*FlagPostProcessing);
  printf("NueesDynamiques OUTPUT NonClassifRate %f\n", *NonClassifRate);
  printf("NueesDynamiques OUTPUT VarianceRate %f\n",*VarianceRate);

 
#endif

  // sortie de la fonction
  return( cr_op);
}
// FIN NueesDynamiques ===========================================================

//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_InitRad
//@
//@ objet :   Preparation des radiances et du prototype zero
//@
//@
//@ Parametres :
//@         (ENTREE) ProduitCourant ...... : Pour pointer sur les variables temporaires
//@         (ENTREE) Contexte_courant .... : contexte de l'etape (numero de ligne SN traite)
//@         (ENTREE) RadAv ............... : Vecteur de radiance de l'image AVHRR
//@         (ENTREE) NbPixAv ............. : Nombre de pixels du vecteur
//@         (ENTREE) Nch ................. : Nombre de canaux presents
//@         (ENTREE) ChannelNoise ........ : Bruit des canaux
//@         (ENTREE) ChannelWgt .......... : Poids des canaux
//@
//@         (SORTIE) RadMean ............. : Radiance moyenne
//@         (SORTIE) RadStd .............. : Ecart type de la radiance
//@         (SORTIE) RadNW ............... : Vecteur de radiance de l'image AVHRR
//@         (SORTIE) ProtoZero ........... : Prototype initial
//@         (SORTIE) EigVal .............. : Valeurs propres
//@         (SORTIE) EigVect ............. : Vecteurs propres
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_InitRad(
  // Entrees
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  t_SD_BAS_DataThread  *Contexte_courant,   // t_SD_BAS_DataThread*
  float                *RadAv,              // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
  int                   NbPixAv,            // int(1)
  int                   Nch,                // int(1)
  double               *ChannelNoise,       // int( SD_BAS_NBK)
  int                  *ChannelWgt,         // int( SD_BAS_NBK)
  // Sorties
  double                *RadMean,            // double( SD_BAS_NBK)
  double                *RadStd,             // double( SD_BAS_NBK)
  float                *RadNW,              // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
  double               *ProtoZero,          // double( SD_BAS_NBK)
/* bma complexes->reels */
  double               *EigVal,             // double( SD_BAS_NBK)
  double               *EigVect             // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
)
{
  static const char tcProcName_F[] = "SD_ALG_InitRad";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  int    Ich;                               // Index sur les canaux
  int    PixNum;                            // Numero de pixel
  float *RadN;                              // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
  int   *IdPix;                             // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  int    Ipro;                              // int(1) Numero de classe
  double CovarZero[SD_BAS_NBK*SD_BAS_NBK];  // double( SD_BAS_NBK * SD_BAS_NBK)
  int    TailleImage;
  double  diff;
  double auxv[2*SD_BAS_NBK];



  #ifdef OPS_DEBUG
    printf( "Debut de la fonction InitRad\n----------------------------------\n");
  #endif


  TailleImage= SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER;

  RadN= ProduitCourant->RadN;
  IdPix= ProduitCourant->IdPix;

  // Preparation des radiances et du prototype zero

  // Normalisation
  // Boucle sur les canaux ( Ich = 1 a Nch)
  for( Ich= 1; Ich <= Nch; Ich++)
  {
    // Calcul de la moyenne
    RadMean[IND1(Ich)]= (double)0.0;
    for( PixNum= 1; PixNum <= NbPixAv; PixNum++)
    {
      RadMean[IND1(Ich)]=   RadMean[IND1(Ich)]
                          + RadAv[IND2(PixNum,Ich,TailleImage)];
    }
    RadMean[IND1(Ich)]= RadMean[IND1(Ich)] / NbPixAv;
    #ifdef CMS_DEBUG
     printf("RadMean channel %d = %e\n",Ich, RadMean[IND1(Ich)]);
    #endif
    // Calcul de l'ecart type
    RadStd[IND1(Ich)]= (double)0.0;
    for( PixNum= 1; PixNum <= NbPixAv; PixNum++)
    {
      diff= RadAv[IND2(PixNum,Ich,TailleImage)] - RadMean[IND1(Ich)];
      RadStd[IND1(Ich)]= RadStd[IND1(Ich)] + (diff * diff);
    }
    RadStd[IND1(Ich)]= (float)sqrt( RadStd[IND1(Ich)] / NbPixAv);

    // Si RadStd(Ich) < 3*ChannelNoise(Ich) alors RadStd(Ich) = 3*ChannelNoise(Ich)
    /*RIC chgt temporaire pour que le TU fonctionne*/
    // VERRUE a enlever  if( RadStd[IND1(Ich)] == ((float)3.0*ChannelNoise[IND1(Ich)]))
    if( RadStd[IND1(Ich)] < ((double)3.0*ChannelNoise[IND1(Ich)]))
    {
      RadStd[IND1(Ich)]= (double)(3.0 * ChannelNoise[IND1(Ich)]);
    }

    // Calcul des Radiances normalisees
    for( PixNum= 1; PixNum <= NbPixAv; PixNum++)
    {
      RadN[IND2(PixNum,Ich,TailleImage)]=
        ((double)RadAv[IND2(PixNum,Ich,TailleImage)] - RadMean[IND1(Ich)]) / RadStd[IND1(Ich)];
    }
  } //Fin de boucle sur les canaux (Ich)

  // Ponderation des canaux
  for( Ich= 1; Ich <= Nch; Ich++)
  {
    for( PixNum= 1; PixNum <= NbPixAv; PixNum++)
    {
      RadNW[IND2(PixNum,Ich,TailleImage)]=   RadN[IND2(PixNum,Ich,TailleImage)]
                                           * (float)ChannelWgt[IND1(Ich)];
    }
  }

  // Prototype zero (moyenne des radiances normalisees = 0.) et matrice de covariance
  Ipro= 0;
  for( PixNum= 1; PixNum <= NbPixAv; PixNum++)    IdPix[IND1(PixNum)]= Ipro;
  #ifdef CMS_DEBUG
     printf("Dans InitRad appel StatClas\n");
    #endif
  cr_op= SD_ALG_StatClas(
                          // Entrees
                           ProduitCourant,  // t_SD_BAS_DataThreadProduit*
                           RadN,            // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
                           IdPix,           // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
                           Nch,             // int(1)
                           NbPixAv,         // int(1)
                           Ipro,            // int(1)
                          // Sorties
                           ProtoZero,       // double( SD_BAS_NBK)
                           CovarZero);      // double( SD_BAS_NBK * SD_BAS_NBK)
   # ifdef CMS_DEBUG
    printf("Dans InitRad apres appel a StatClas protozero=\n");
   for ( Ich= 0; Ich <= Nch; Ich++)
   {
    printf("protozero[%d]=%e\n",Ich, ProtoZero[Ich]);
   }
   #endif
  // Calcul des valeurs propres et vecteurs propres de la matrice de covariance
  // Note : il faut que EigVal(1) > EigVal(2) > ...
  // [ EigVal(#), EigVect(#,#) ] = EigenAnalysis{ CovarZero(#,#) } 
  cr_op= SD_ALG_EigenAnalysis(
        // Entrees
         Contexte_courant,  // t_SD_BAS_DataThread*
         SD_BAS_NBK,        // int(1) Nombre de points
         CovarZero,         // double( SD_BAS_NBK * SD_BAS_NBK),
         CMN__OUI,          // unsigned int(1) Transposition de CovarZero avant calcul et EigVect apres calcul
        // Sorties
/* bma complexes->reels */
         EigVal,            // double( SD_BAS_NBK)
         EigVect);          // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */

fin:
  // Fin de la fonction InitRad
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction InitRad\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN InitRad ===================================================================
// ===============================================================================
//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_IterNuDyn
//@
//@ objet :   Une iteration de classification
//@
//@
//@ Parametres :
//@         (ENTREE) ProduitCourant ... : Pour pointer sur les variables temporaires
//@         (ENTREE) RadN ............. : Vecteur de radiance de l'image AVHRR
//@         (ENTREE) NbPixAv .......... : Nombre de pixels du vecteur
//@         (ENTREE) Nch .............. : Nombre de canaux presents
//@         (ENTREE) NbPro ............ : Nombre de prototypes
//@         (ENTREE) NbClusMax ........ : Nombre maximal de clusters
//@         (ENTREE) NbIterMax ........ : Nombre maximal d'iterations
//@         (ENTREE) EigVal ........... : Valeurs propres
//@         (ENTREE) EigVect .......... : Vecteurs propres
//@         (ENTREE) DistAgloInit ..... : Valeur initiale de la distance d'aglomeration
//@         (ENTREE) SplitCutoff ...... : Seuil d'eclatement
//@         (ENTREE) GlueCutoff ....... : Seuil de coalescence
//@         (ENTREE) EigValCutoff ..... : Seuil de valeur propre
//@         (ENTREE) MinIterNCCutOff .. : Nombre d'iteration minimales pour le calcul du seuil des non classifies
//@         (ENTREE) NbNonClassifMax .. : Nombre maximal de non classifies
//@         (ENTREE) SortingChannel ... : Canal de rangement
//@         (ENTREE) DistMax .......... : Distance maximale interclasses
//@         (ENTREE) IterConvCutoff ... : Seuil de convergence
//@
//@         (ENTSOR) Proto ............ : Prototypes de classes
//@         (ENTSOR) IdPix ............ : Flags de pixels valides
//@         (ENTSOR) NbPixClas ........ : Nombre pixels par classe
//@         (ENTSOR) SumDistClas ...... : Somme des distances intra-classes
//@
//@         (SORTIE) NbClass .......... : Nombre de classes
//@         (SORTIE) NoExt ............ : Flags de classes non extremes
//@         (SORTIE) Iter ............. : Numero d'iteration
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_IterNuDyn(
     // Entree
   t_SD_BAS_DataThread*           Contexte_courant,      // t_SD_BAS_DataThread*
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
     float                *RadN,                  // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
     int                   NbPixAv,               // int(1)
     int                   Nch,                   // int(1)
     int                   NbPro,                 // int(1)
     int                   NbClusMax,             // int(1)
     int                   NbIterMax,             // int(1)
/* bma complexes->reels */
     double               *EigVal,                // double( SD_BAS_NBK)
     double               *EigVect,               // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
     double                DistAgloInit,          // double(1)
     double                SplitCutoff,           // double(1)
     double                GlueCutoff,            // double(1)
     double                EigValCutoff,          // double(1)
     int                   MinIterNCCutoff,       // int(1)
     int                   NbNonClassifMax,       // int(1)
     int                   SortingChannel,        // int(1)
     double                DistMax,               // double(1)
     double                IterConvCutoff,        // double(1)
     // Entrees/Sortie
     double               *Proto,                 // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
     int                  *IdPix,                 // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
     int                  *NbPixClas,             // int( SD_BAS_NCL_MAX)
     double               *SumDistClas,           // double( SD_BAS_NCL_MAX)
     // Sortie
     int                  *NbClass,               // int(1)
     unsigned int         *NoExt,                 // unsigned int( SD_BAS_NCL_MAX)
     int                  *Iter                   // int(1)
)
{
  static const char tcProcName_F[] = "SD_ALG_IterNuDyn";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  unsigned int FinIter;
  unsigned int Aglo;
  unsigned int Separ;
  int          TailleAvhrr;                    // Taille d'une image AVHRR
  int          Ipro;                           // Index sur les canaux
  int          Ich;                            // Index sur les canaux
  int          Ipix;                           // Index sur les pixels
  double      *Dist;                           // double( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NCL_MAX)
  int          NbNCMax;                        // Nombre max non classifies
  double       SumDistTot0;
  double       SumDistTot[SD_ALG_MAX_ITER];    // double( SD_ALG_MAX_ITER)
  int          IdProLast;
  double       Covar[SD_BAS_NBK*SD_BAS_NBK];   // double( SD_BAS_NBK * SD_BAS_NBK)
  FILE * myFile;

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction IterNuDyn\n----------------------------------\n");
  #endif
  

  Dist= ProduitCourant->Dist;

  TailleAvhrr= SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER;

  // StopIter = False
  FinIter= CMN__NON;


  // SumDistTot0 = Somme (SumDistClass(i)) pour i = 1 NbPro
  SumDistTot0= 0.0;
  for( Ipro= 1; Ipro <= NbPro; Ipro++)
  {
    SumDistTot0= SumDistTot0 + SumDistClas[IND1(Ipro)];
  }

  // Boucle iterative si StopIter = False (Iter = 2 a NbIterMax)
  *Iter= 2;
  while( (FinIter == CMN__NON) && (*Iter <= NbIterMax))
  {
    // Gestion des mecanismes d'agglomeration et de separation
    // Aglo = True si NbPro > NbClusMax
    #if CMS_DEBUG
     printf("inside IterNuDyn Iter=%d\n",*Iter);
    #endif
    if( NbPro > NbClusMax)   Aglo= CMN__OUI;
    else Aglo=CMN__NON;

    // Separ = True si NbPro < NbClusMax + (NbIterMax - Iter - 1)
    //jlp30
    if( NbPro < NbClusMax + (NbIterMax - *Iter - 1))
       Separ= CMN__OUI; 
    else 
       Separ= CMN__NON;
   #ifdef CMS_DEBUG
   printf(">>>> inside iterNuDyn before GestionProto NbPro %d NbClusMax %d NbIterMax %d Iter %d\n",NbPro,NbClusMax,NbIterMax, *Iter);
   printf (">>>> Separ= %d\n",Separ);
   printf(">>>> Algo=%d\n",Aglo);  

    for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
    {
      printf( "      Proto(Ich=%12d,$) : ", Ich);
      for( Ipro= 1; Ipro <= NbPro; Ipro++)
      {
        printf( " Proto[%d,%d]=%12.5f", Ich,Ipro,Proto[IND2(Ich,Ipro,SD_BAS_NBK)]);
      }
      printf( "\n");
    }

   #endif
    cr_op= SD_ALG_GestionProto(
             // Entrees
              Contexte_courant,      // t_SD_BAS_DataThread*
              ProduitCourant,  // t_SD_BAS_DataThreadProduit*
              RadN,            // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
              NbPixAv,         // int(1)
              Nch,             // int(1)
              Aglo,            // unsigned int(1)
              Separ,           // unsigned int(1)
/* bma complexes->reels */
              EigVal,          // double( SD_BAS_NBK)
              EigVect,         // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
              SplitCutoff,     // double(1)
              DistAgloInit,    // double(1)
              GlueCutoff,      // double(1)
              EigValCutoff,    // double(1)
             // Entrees / Sorties
              Proto,           // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
             &NbPro,           // int(1)
              IdPix,           // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
              NbPixClas,       // int( SD_BAS_NCL_MAX)
              SumDistClas,     // double( SD_BAS_NCL_MAX)
             // Sorties
              NoExt);          // unsigned int( SD_BAS_NCL_MAX)
     if(cr_op != CMN__STATUT_OK)
     {
         goto fin;
     }
#ifdef CMS_DEBUG
    printf("inside iterNuDyn after GestionProto NbPro %d\n",NbPro);

    for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
    {
      printf( "      Proto(Ich=%12d,$) : ", Ich);
      for( Ipro= 1; Ipro <= NbPro; Ipro++)
      {
        printf( " Proto[%d,%d]=%12.5f", Ich,Ipro,Proto[IND2(Ich,Ipro,SD_BAS_NBK)]);
      }
      printf( "\n");
    }

#endif
    // Calcul des distances aux prototypes
    // Dist(#,$) = 0.  => modifie en Dist($,*)
    for( Ipro= 1; Ipro <= NbPro; Ipro++)
    {
      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
      {
        Dist[IND2(Ipix,Ipro,TailleAvhrr)]= 0.0;
      }
    }

    // Boucle sur les prototypes (Ipro = $)
    // Dist(Ipro,*) = DistPixProto{ RadN(#,*), Nch, NbPixAv, Proto(#,Ipro) }
    for( Ipro= 1; Ipro <= NbPro; Ipro++)
    {
      cr_op= SD_ALG_DistPixProto(
                 // Entrees
                  RadN,         // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
                  Nch,                              // int(1)
                  NbPixAv,                           // int(1)
                 &Proto[IND2(1,Ipro,SD_BAS_NBK)],   // double( SD_BAS_NBK)
                 // Sorties
                 &Dist[IND2(1,Ipro,TailleAvhrr)]);  // double(SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
    } // Fin de boucle sur les prototypes

    // Attribution des pixels aux classes
    cr_op= SD_ALG_LinkPixProto(
              // Entrees
               Dist,                // double( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NCL_MAX)
               DistMax,             // double(1)
               NbPixAv,             // int(1)
               NbPro,               // int(1)
              // Sorties
               IdPix,                     // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
               NbPixClas,                 // int( SD_BAS_NCL_MAX)
               SumDistClas,               // double( SD_BAS_NCL_MAX)
              &SumDistTot[IND1(*Iter)]);   // double(1)

    // Gestion des non classes
    // Nombre max non classes en fonction de l'iteration
    // NbNCMax = NbNonClassifMax / Min{ MinIterNCCutoff, Max{ (NbIterMax - Iter), 1 }}
    NbNCMax= NbNonClassifMax / MIN( MinIterNCCutoff, MAX( (NbIterMax - *Iter), 1));
  
  
    // Si NbPixClas(NbPro+1) >= NbNCMax et Iter =/= NbIterMax alors
    if( (NbPixClas[IND1(NbPro+1)] >= NbNCMax) && (*Iter != NbIterMax))
    {
      // Addition d'une nouvelle classe a partir des non classes
       #ifdef CMS_DEBUG
        printf(">>>> Addition d'une nouvelle classe a partir des non classes\n");
       #endif
      // IdProLast = NbPro + 1
      IdProLast= NbPro + 1;

      // Dist(IdProLast,*) = 0.
      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
      {
        Dist[IND2(Ipix,IdProLast,TailleAvhrr)]= 0.0;
      }

      // Caracteristiques de la classe
      // [ Proto(#,IdProLast), Covar(#,#,IdProLast) ] = 
      //      StatClas{ RadN(#,*), IdPix(*), Nch, NbPixAv, IdProLast }
      cr_op= SD_ALG_StatClas(
                // Entrees
                 ProduitCourant,  // t_SD_BAS_DataThreadProduit*
                 RadN,            // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
                 IdPix,           // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
                 Nch,             // int(1)
                 NbPixAv,         // int(1)
                 IdProLast,       // int(1)
                // Sorties
                &Proto[IND2(1,IdProLast,SD_BAS_NBK)],  // double( SD_BAS_NBK)
                 Covar);                               // double( SD_BAS_NBK * SD_BAS_NBK)

      // Calcul de la taille de la classe
      // Dist(IdProLast,*) = DistPixProtoClas{ RadN(#,*), Nch, NbPixAv, IdPix(*), IdProLast, Proto(#,IdProLast) }
      cr_op= SD_ALG_DistPixProtoClas(
                // Entrees
                 RadN,         // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
                 Nch,                              // int(1)
                 NbPixAv,                          // int(1)
                 IdPix,                            // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
                 IdProLast,                        // int(1)
                &Proto[IND2(1,IdProLast,SD_BAS_NBK)],   // double( SD_BAS_NBK)
                // Sorties
                &Dist[IND2(1,IdProLast,TailleAvhrr)]);  // double( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)

      // SumDistClas(IdProLast) = somme(...)
      SumDistClas[IND1(IdProLast)]= 0.0;
      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
      {
        SumDistClas[IND1(IdProLast)] += Dist[IND2(Ipix,IdProLast,TailleAvhrr)];
      }

      // SumDistTot(Iter) = SumDistTot(Iter) + SumDistClas(IdProLast)
      SumDistTot[IND1(*Iter)]= SumDistTot[IND1(*Iter)] + SumDistClas[IND1(IdProLast)];

      // NbPro = IdProLast
      NbPro= IdProLast;

    } // Fin de si (NbPixClas(NbPro+1) NbNCMax et Iter NbIterMax)

    // Test d'iteration
    // FinIter = { Iter = NbIterMax} ou 
    // { {NbPro NbClusMax} et {SumDistTot(Iter}/SumDistTot0 < IterConvCutoff} }
    FinIter=    (*Iter == NbIterMax)
             || (    (NbPro <= NbClusMax)
                  && ( (SumDistTot[IND1(*Iter)]/SumDistTot0) < IterConvCutoff));

    // Mise a jour des proprietes des classes
    // Boucle sur les prototypes (Ipro = $)
    for( Ipro= 1; Ipro <= NbPro; Ipro++)
    {
       #ifdef CMS_DEBUG
        printf(" Mise a jour des proprietes des classes pour le prototype %d\n",Ipro);
       #endif
      // StatClas{ RadN(#,*), IdPix(*), Nch, NbPixAv, Ipro }
      //   [ Proto(#,Ipro), Covar(#,#,Ipro) ] = 
      cr_op= SD_ALG_StatClas(
                // Entrees
                 ProduitCourant,  // t_SD_BAS_DataThreadProduit*
                 RadN,            // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
                 IdPix,           // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
                 Nch,             // int(1)
                 NbPixAv,         // int(1)
                 Ipro,            // int(1)
                // Sorties
                &Proto[IND2(1,Ipro,SD_BAS_NBK)],  // double( SD_BAS_NBK)
                 Covar);                          // double( SD_BAS_NBK * SD_BAS_NBK)

    } // Fin de boucle sur les prototypes

    // Reclassement sur les valeurs du canal de classement par ordre decroissant des radiances
    // [ Proto(#,$), IdPix(*), NbPixClas($), SumDistClas($) ] = SortClas{ Proto(#,$), IdPix(*), NbPixClas($), SumDistClas($), SortingChannel, Nch, NbPixAv, NbPro }
    #ifdef CMS_DEBUG
    printf("Nb Iter=%d   avant reclassement\n", *Iter);

    for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
    {
      printf( "      Proto(Ich=%12d,$) : ", Ich);
      for( Ipro= 1; Ipro <= NbPro; Ipro++)
      {
        printf( " Proto[%d,%d]=%12.5f", Ich,Ipro,Proto[IND2(Ich,Ipro,SD_BAS_NBK)]);
      }
      printf( "\n");
    }
    #endif
    cr_op= SD_ALG_SortClas(
              // Entrees
               SortingChannel,  // int(1)
               Nch,             // int(1)
               NbPixAv,         // int(1)
               NbPro,           // int(1)
              // Entrees / Sorties
               Proto,           // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
               IdPix,           // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
               NbPixClas,       // int( SD_BAS_NCL_MAX)
               SumDistClas);    // double( SD_BAS_NCL_MAX)
    // VERRUE MOR 21/08/2003
  #ifdef CMS_DEBUG
    printf("Nb Iter=%d   apres reclassement\n", *Iter);

    for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
    {
      printf( "      Proto(Ich=%12d,$) : ", Ich);
      for( Ipro= 1; Ipro <= NbPro; Ipro++)
      {
        printf( " Proto[%d,%d]=%12.5f", Ich,Ipro,Proto[IND2(Ich,Ipro,SD_BAS_NBK)]);
      }
      printf( "\n");
    }
    printf("NbPixClass: ");
    for( Ipro= 1; Ipro <= NbPro; Ipro++)
      printf("%d  ", NbPixClas[IND1(Ipro)]);
    printf("\n");

  #endif
    //fin MOR 21/08/2003

    // Si FinIter =True alors
    if( FinIter == CMN__OUI)
    {
      // Caracterisation des non classes 
      // Si NbPixClas(NbPro+1) =/= 0 alors
      
      if( NbPixClas[IND1(NbPro+1)] != 0)
      {
        // IdProLast = NbPro +1
        IdProLast= NbPro + 1;

        // [ Proto(#,IdProLast), Covar(#,#,IdProLast) ] = 
        // StatClas{ RadN(#,*), IdPix(*), Nch, NbPixAv, IdProLast }
        cr_op= SD_ALG_StatClas(
                  // Entrees
                   ProduitCourant,  // t_SD_BAS_DataThreadProduit*
                   RadN,            // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
                   IdPix,           // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
                   Nch,             // int(1)
                   NbPixAv,         // int(1)
                   IdProLast,       // int(1)
                  // Sorties
                  &Proto[IND2(1,IdProLast,SD_BAS_NBK)],  // double( SD_BAS_NBK)
                   Covar);                               // double( SD_BAS_NBK * SD_BAS_NBK)

      } // Fin de si (NbPixClas(NbPro+1) =/= 0)
    } // Fin de si (FinIter =True)

    *Iter= *Iter + 1;

  } // Fin de boucle iterative si StopIter = False (Iter = 2 a NbIterMax)

  // Nombre de classes identifiees
  // NbClass = NbPro
  *NbClass= NbPro;

fin:
  // Fin de la fonction IterNuDyn
 #ifdef CMS_DEBUG

printf(">>>>>fin NbClass=\n", NbPro);
    for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
    {
      printf( "      Proto(Ich=%12d,$) : ", Ich);
      for( Ipro= 1; Ipro <= NbPro; Ipro++)
      {
        printf( " %12.5f, ", Proto[IND2(Ich,Ipro,SD_BAS_NBK)]);
      }
      printf( "\n");
    }
    printf("NbPixClass: ");
    for( Ipro= 1; Ipro <= NbPro; Ipro++)
      printf("%d  ", NbPixClas[IND1(Ipro)]);
    printf("\n");

  #endif
  #ifdef OPS_DEBUG
    fclose(myFile);
    printf( "Fin de la fonction IterNuDyn\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN IterNuDyn =================================================================


//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_IterNuDynPost
//@
//@ objet :   Une iteration de classification
//@
//@
//@ Parametres :
//@         (ENTREE) IterLast ......... : Derniere iteration
//@         (ENTREE) NbPixAv .......... : Nombre de pixels du vecteur
//@         (ENTREE) Nch .............. : Nombre de canaux presents
//@         (ENTREE) NbPro ............ : Nombre de prototypes
//@         (ENTREE) NbClusMax ........ : Nombre maximal de clusters
//@         (ENTREE) EigVal ........... : Valeurs propres
//@         (ENTREE) EigVect .......... : Vecteurs propres
//@         (ENTREE) DistAgloInit ..... : Valeur initiale de la distance d'aglomeration
//@         (ENTREE) EigValCutoff ..... : Seuil de valeur propre
//@         (ENTREE) SortingChannel ... : Canal de rangement
//@
//@         (ENTSOR) Proto ............ : Prototypes de classes
//@         (ENTSOR) IdPix ............ : Flags de pixels valides
//@         (ENTSOR) NbPixClas ........ : Nombre pixels par classe
//@         (ENTSOR) SumDistClas ...... : Somme des distances intra-classes
//@         (ENTSOR) NoExt ............ : Flags de classes non extremes
//@
//@         (SORTIE) NbClass .......... : Nombre de classes
//@         (SORTIE) Iter ............. : Numero d'iteration
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_IterNuDynPost(
  // Entree
  t_SD_BAS_DataThreadProduit  *ProduitCourant,  // t_SD_BAS_DataThreadProduit*
  int                          IterLast,        // int(1)
  int                          NbPixAv,         // int(1)
  int                          Nch,             // int(1)
  int                          NbClusMax,       // int(1)
/* bma complexes->reels */
  double                      *EigVal,          // double( SD_BAS_NBK)
  double                      *EigVect,         // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
  double                       DistAgloInit,    // double(1)
  double                       EigValCutoff,    // double(1)
  int                          SortingChannel,  // int(1)
  // Entrees / Sortie
  double                      *Proto,           // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
  int                         *IdPix,           // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  int                         *NbPixClas,       // int( SD_BAS_NCL_MAX)
  double                      *SumDistClas,     // double( SD_BAS_NCL_MAX)
  unsigned int                *NoExt,           // unsigned int( SD_BAS_NCL_MAX)
  // Sortie
  int                         *NbClass,         // int(1)
  int                         *Iter             // int(1)
)
{
  static const char tcProcName_F[] = "SD_ALG_IterNuDynPost";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  unsigned int Aglo;
  unsigned int Separ;
  int          IterMin, IterMax;
  int NbPro;
  NbPro=*NbClass;
  #ifdef OPS_DEBUG
    printf( "Debut de la fonction IterNuDynPost\n----------------------------------\n");
  #endif

  // IterMin = IterLast +1
  IterMin= IterLast + 1;

  // IterMax = IterMin +(NbPro -NbClusMax) -1
  IterMax= IterMin + (NbPro - NbClusMax) - 1;

  // Boucle iterative Iter = IterMin,IterMax
  for( *Iter= IterMin; *Iter <= IterMax; (*Iter)++)
  {
    // Agglomeration forcee
    cr_op= SD_ALG_GestionProtoPost(
             // Entrees
              ProduitCourant,  // t_SD_BAS_DataThreadProduit*
              Nch,             // int(1)
/* bma ajout du nombre de pixels AVHRR dans la signature de la fonction GestionProtoPost*/
              NbPixAv,            // int(1)
/* fin bma */
/* bma complexes->reels */
              EigVal,          // double( SD_BAS_NBK)
              EigVect,         // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
              DistAgloInit,    // double(1)
              EigValCutoff,    // double(1)
             // Entrees / Sortie
              Proto,           // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
             &NbPro,           // int(1)
              IdPix,           // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
              NbPixClas,       // int( SD_BAS_NCL_MAX)
              SumDistClas,     // double( SD_BAS_NCL_MAX)
              NoExt);          // unsigned int( SD_BAS_NCL_MAX)
    // Reclassement sur les valeurs du canal de classement par ordre decroissant des radiances
    cr_op= SD_ALG_SortClas(
            // Entrees
            SortingChannel,  // int(1)
            Nch,             // int(1)
            NbPixAv,         // int(1)
            NbPro,           // int(1)
            // Rnetrees / Sorties
            Proto,           // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
            IdPix,           // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
            NbPixClas,       // int( SD_BAS_NCL_MAX)
            SumDistClas);    // double( SD_BAS_NCL_MAX)

  } // Fin de boucle iterative si StopIter = False (Iter = 2 a NbIterMax)

  // Nombre de classes identifiees
  // NbClass = NbPro
  *NbClass= NbPro;

fin:
  // Fin de la fonction IterNuDynPost
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction IterNuDynPost\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN IterNuDynPost =============================================================
// ===============================================================================



//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_InitProto
//@
//@ objet :   Initialisation des prototypes
//@
//@
//@ Parametres :
//@         (ENTREE) ProduitCourant .. : Pour pointer sur les variables temporaires
//@         (ENTREE) RadN ............ : Vecteur de radiance de l'image AVHRR
//@         (ENTREE) Nch ............. : Nombre de canaux presents
//@         (ENTREE) NbPixAv ......... : Nombre de pixels du vecteur
//@         (ENTREE) ProtoZero ....... : Prototype initial
//@         (ENTREE) NbClusMax ....... : Nombre maximal de clusters
//@         (ENTREE) DistMax ......... : Distance maximale interclasses
//@         (ENTREE) SortingChannel .. : Canal de rangement
//@
//@         (SORTIE) NbPro ........... : Nombre de prototypes
//@         (SORTIE) Proto ........... : Prototypes de classes
//@         (SORTIE) Covar ........... : Matrice de covariance
//@         (SORTIE) Dist ............ : Matrice des distances inter-prototypes
//@         (SORTIE) IdPix ........... : Flags de pixels valides
//@         (SORTIE) NbPixClas ....... : Nombre pixels par classe
//@         (SORTIE) SumDistClas ..... : Somme des distances intra-classes
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_InitProto(
  // Entrees
  t_SD_BAS_DataThread* ctx,      // t_SD_BAS_DataThread*
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  float                *RadN,               // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
  int                   Nch,                // int(1)
  int                   NbPixAv,            // int(1)
  double               *ProtoZero,          // double( SD_BAS_NBK)
  int                   NbClusMax,          // int(1)
  double                DistMax,            // double(1)
  int                   SortingChannel,     // int(1)
  // Sorties
  int                  *NbPro,              // int(1)
  double               *Proto,              // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
  double               *Covar,              // double( SD_BAS_NBK * SD_BAS_NBK * SD_BAS_NCL_MAX)
  double               *Dist,               // double( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NCL_MAX)
  int                  *IdPix,              // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  int                  *NbPixClas,          // int( SD_BAS_NCL_MAX)
  double               *SumDistClas         // double( SD_BAS_NCL_MAX)
)
{
  static const char tcProcName_F[] = "SD_ALG_InitProto";
  t_CMN__Statut cr_op=CMN__STATUT_OK;

  // Gestion de erreurs
/* bma intialisation */
/*
  char *msg_err;          // Message d'erreur
  int num_lig;            // Ligne d'erreur
*/
  char *msg_err = (char*) NULL;          // Message d'erreur
  int num_lig = 0;            // Ligne d'erreur
/* fin bma */

  // Definitions
  // (#) (1 a Nch) nombre de canaux/pseudo-canaux Avhrr utilises
  // (*) (1 a NbPixAv) nombre de pixels Avhrr
  // ($) (1 a NbPro) nombre de classes prototype
  unsigned int *UsedPixTest;    // unsigned int(SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  double       *DistZero;
  double       *Dist1;
  int           Ipix;           // Index de pixels
  int           Ich;            // Index sur les canaux
  int           Ipro;           // Index sur les prototypes
  double        MaxDist0;       // Distance maximale dans DistZero
  int           Imax;           // Index de distance maximale
  int           TailleAvhrr;    // Taille d'une image AVHRR
  double        SumDistTot;     // double(1) - Utilise en retour de fonction, mais pas utile
  double        *discu1, *discu;  //CF code Noveltis
//jlp30 pour la rapidite
  const int irk=3;
  int k;
  discu1=NULL;
  discu=NULL;
  discu1 = (double *) calloc( NbPixAv,sizeof(double) );
  discu = (double *) calloc( NbPixAv,sizeof(double) );
 if( (discu1 == NULL) || (discu == NULL) )
  {
    cr_op   = SD_ALG_AVHRR_ERR_ALLOCATION;
    msg_err= (char *)SD_BAS_MSG_ALLOC;
    num_lig= __LINE__;

    goto fin;
  } 

  #ifdef CMS_DEBUG
    printf ( "Debut de la fonction InitProto\n----------------------------------\n");
    printf ( "Nch %d\n",Nch);
    printf ( "NbPixAv%d\n",NbPixAv ); 
    printf ("ProtoZero :\n");

    for (Ipix = 0; Ipix < SD_BAS_NBK;Ipix ++)
    {
      printf ( "    %d -> %e \n",Ipix, ProtoZero[Ipix]);
    }    
   printf ("NbClusMax : %d\n", NbClusMax);
   printf ("DistMax: %e\n",DistMax);
   printf ("SortingChannel %d\n",SortingChannel);

//    for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
  for( Ipix= 1; Ipix <= 10; Ipix++)
    {
      printf ( "RadN[%4d] : %10.5f %10.5f %10.5f %10.5f %10.5f %10.5f\n",
              Ipix,
              RadN[IND2(Ipix,1,SD_BAS_NPI_IMAGER*SD_BAS_NLI_MAX_UTIL_IMAGER)],
              RadN[IND2(Ipix,2,SD_BAS_NPI_IMAGER*SD_BAS_NLI_MAX_UTIL_IMAGER)],
              RadN[IND2(Ipix,3,SD_BAS_NPI_IMAGER*SD_BAS_NLI_MAX_UTIL_IMAGER)],
              RadN[IND2(Ipix,4,SD_BAS_NPI_IMAGER*SD_BAS_NLI_MAX_UTIL_IMAGER)],
              RadN[IND2(Ipix,5,SD_BAS_NPI_IMAGER*SD_BAS_NLI_MAX_UTIL_IMAGER)],
              RadN[IND2(Ipix,6,SD_BAS_NPI_IMAGER*SD_BAS_NLI_MAX_UTIL_IMAGER)]);
    }
    printf ( "=========> Appel SD_ALG_DistPixProto\n");
  #endif

  TailleAvhrr= SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER;

  UsedPixTest= ProduitCourant->UsedPixTest;
  DistZero= ProduitCourant->DistZero;
  Dist1= ProduitCourant->Dist1;

  // Nombre initial de prototype 
  // NbPro = Min{ NbClusMax - 2, 3 }
  *NbPro= MIN( NbClusMax - 2, 3);

  // Calcul de la distance quadratique au prototype zero
  // DistZero(*) = DistPixProto{ RadN(#,*), Nch, NbPixAv, ProtoZero(#) }
  cr_op= SD_ALG_DistPixProto(
                 // Entrees
                 RadN,         // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
                 Nch,          // int(1)
                 NbPixAv,      // int(1)
                 ProtoZero,    // double( SD_BAS_NBK)
                 // Sorties
                 DistZero);    // double(SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)

  #ifdef CMS_DEBUG
    //for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
      //printf( "DistZero[%d]= %f\n", Ipix, DistZero[IND1(Ipix)]); 
  #endif

  // Recherche du pixel le plus eloigne du prototype zero
  // UsedPixTest(*) = False
  for( Ipix= 1; Ipix <= NbPixAv; Ipix++)   UsedPixTest[IND1(Ipix)]= CMN__NON;

  for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
    discu1[IND1(Ipix)] = DistZero[IND1(Ipix)];

// version 5.0 
// Imax = Index de Max3{ Dist1(*) }
//jlp30 pour la rapidite
// cr_op = SD_ALG_Index_De_Max3(ctx,NbPixAv,discu1,&Imax );
  for( k=1 ; k<= irk ; k++)
  {
    MaxDist0= -100000.0;
    Imax= 0;
    for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
    {
      if( discu1[IND1(Ipix)] > MaxDist0)
      {
        MaxDist0= discu1[IND1(Ipix)];
        Imax= Ipix;
      }
    }
    discu1[IND1(Imax)] = -1000.0;
  }    

  #ifdef OPS_DEBUG
MaxDist0=discu1[IND1(Imax)];
    printf( "MaxDist0= %f      Imax= %d\n", MaxDist0, Imax);
  #endif

  // Probleme si l'index de DistMax est nul
  if( Imax == 0)
  {
    cr_op=   SD_ALG_AVHRR_ERR_NUL_DISTANCE;
    msg_err= (char *)SD_ALG_MSG_ERR_NUL_DISTANCE;
    goto fin;
  }

  // UsedPixTest(Imax) = True
  UsedPixTest[IND1(Imax)]= CMN__OUI;

  // Premier prototype
  // Proto(#,1) = RadN(#,Imax)
  for( Ich= 1; Ich <= Nch; Ich++)
    Proto[IND2(Ich,1,SD_BAS_NBK)]= (double)RadN[IND2(Imax,Ich,TailleAvhrr)];


  #ifdef CMS_DEBUG
    for( Ich= 1; Ich <= Nch; Ich++)
      printf( "Proto[%d,1]= %f\n", Ich , Proto[IND2(Ich,1,SD_BAS_NBK)]);
    printf( "NbPro Init: %d\n", *NbPro);
  #endif

  for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
    Dist1[IND1(Ipix)]= 0.0; 

  // Boucle sur les prototypes (Ipro = 2 a NbPro)
  for( Ipro= 2; Ipro <= *NbPro; Ipro++)
  {
    #ifdef CMS_DEBUG
      printf( "=========> Appel SD_ALG_DistPixProto\n");
    #endif

    // Calcul de la distance quadratique au prototype precedent
    // Dist(Ipro-1,*) = DistPixProto{ RadN(#,*), Nch, NbPixAv, Proto(#,Ipro-1) }
    cr_op= SD_ALG_DistPixProto(
                 // Entrees
                  RadN,         // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
                  Nch,                                // int(1)
                  NbPixAv,                            // int(1)
                 &Proto[IND2(1,Ipro-1,SD_BAS_NBK)],   // double( SD_BAS_NBK)
                 // Sorties
                 &Dist[IND2(1,Ipro-1,TailleAvhrr)]);  // double( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)

    // Recherche du pixel non utilise le plus eloigne du prototype precedent
    // Dist1(*) = Dist(Ipro-1,*)
    for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
    {
      Dist1[IND1(Ipix)]= Dist1[IND1(Ipix)]+Dist[IND2(Ipix,Ipro-1,TailleAvhrr)];
      discu1[IND1(Ipix)]=Dist1[IND1(Ipix)];
    }

    #ifdef CMS_DEBUG
     // for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
      //  printf( "Dist1[%d]= %f\n", Ipix, Dist1[IND1(Ipix)]); 
    #endif

    if(Ipro == 2) //Initialisation
      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
	discu[IND1(Ipix)] = 0.0;

      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
	{
	  discu[IND1(Ipix)] = discu[IND1(Ipix)] + Dist[IND2(Ipix,Ipro-1,TailleAvhrr)];
	  discu1[IND1(Ipix)] = discu[IND1(Ipix)];
	}
// version 5.0 
// Imax = Index de Max3{ Dist1(*) }
//jlp30 pour la rapidite
//    cr_op = SD_ALG_Index_De_Max3(ctx,NbPixAv,discu1 ,&Imax);
    for(k=1 ; k<=irk ; k++)
    {
    MaxDist0= -100000.0;
    Imax= 0;
      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
      {
        if(discu1[IND1(Ipix)] > MaxDist0)
        {
          MaxDist0= discu1[IND1(Ipix)];
          Imax= Ipix;
        }
      }
      discu1[IND1(Imax)] = -1000.0;
    }

    #ifdef CMS_DEBUG
MaxDist0=discu1[IND1(Imax)];
      printf( "MaxDist0= %f      Imax= %d\n", MaxDist0, Imax);
    #endif

    // Tant que UsedPixTest(Imax) = True
    while( UsedPixTest[IND1(Imax)] == CMN__OUI)
    {
      // Dist1(Imax) = 0.
      Dist1[IND1(Imax)]= 0.0;

// version 5.0 
//     Imax= Index_De_Max3{ Dist1(*) }
//jlp30 pour la rapidite
//     cr_op= SD_ALG_Index_De_Max3(ctx,NbPixAv,Dist1,&Imax );
     for( k=1 ; k<= irk ; k++)
     {
      MaxDist0= -10000000.0;
      Imax= 0;
      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
      {
        if( Dist1[IND1(Ipix)] > MaxDist0)
        {
          MaxDist0= Dist1[IND1(Ipix)];
          Imax= Ipix;
        }
      }
      Dist1[IND1(Imax)] = -1000.0;      
     }
      
    }  // Fin de Tant que

    // UsedPixTest(Imax) = True
    UsedPixTest[IND1(Imax)]= CMN__OUI;

    // Nouveau prototype
    // Proto(#,Ipro) = RadN(#,Imax)
    for( Ich= 1; Ich <= Nch; Ich++)
      Proto[IND2(Ich,Ipro,SD_BAS_NBK)]= (double)RadN[IND2(Imax,Ich,TailleAvhrr)];

    #ifdef CMS_DEBUG
      for( Ich= 1; Ich <= Nch; Ich++)
        printf( "Proto[%d,%d]= %f\n", Ich, Ipro, Proto[IND2(Ich,Ipro,SD_BAS_NBK)]);
    #endif
  } // Fin de boucle sur les prototypes

  #ifdef CMS_DEBUG
    printf( "=========> Appel SD_ALG_DistPixProto\n");
  #endif
  // Calcul de la distance au dernier prototype
  // Dist(NbPro,*) = DistPixProto{ RadN(#,*), Nch, NbPixAv, Proto(#,NbPro) }
  cr_op= SD_ALG_DistPixProto(
            // Entrees
             RadN,         // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
             Nch,                                // int(1)
             NbPixAv,                            // int(1)
            &Proto[IND2(1,*NbPro,SD_BAS_NBK)],   // double( SD_BAS_NBK)
            // Sorties
            &Dist[IND2(1,*NbPro,TailleAvhrr)]);  // double( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)

  // Addition d'un autre prototype egal a la moyenne
  // NbPro = NbPro + 1
  *NbPro= *NbPro + 1;

  // Proto(#,NbPro) = 0.
  for( Ich= 1; Ich <= Nch; Ich++)
    Proto[IND2(Ich,*NbPro,SD_BAS_NBK)]= 0.0;

  // Distance a la moyenne
  // Dist(NbPro,*) = DistZero(*)
  for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
    Dist[IND2(Ipix,*NbPro,TailleAvhrr)]= DistZero[IND1(Ipix)];

  #ifdef CMS_DEBUG
    //for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
     // printf( "Dist[%d,%d]= %f\n", *NbPro, Ipix, Dist[IND2(Ipix,*NbPro,TailleAvhrr)]); 
    printf( "=========> Appel SD_ALG_LinkPixProto\n");
  #endif

  // Attribution des pixels aux prototypes
  cr_op= SD_ALG_LinkPixProto(
           // Entrees
            Dist,            // double( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NCL_MAX)
// version 5.0 
            2.0*DistMax,         // double(1)
            NbPixAv,         // int(1)
           *NbPro,           // int(1)
           // Sorties
            IdPix,           // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
            NbPixClas,       // int( SD_BAS_NCL_MAX)
            SumDistClas,     // double( SD_BAS_NCL_MAX)
           &SumDistTot);     // double(1)

  #ifdef CMS_DEBUG
    for( Ipix= 1; Ipix <= 20 /*NbPixAv*/; Ipix++)
      printf( "IdPix[%d]= %d\n", Ipix, IdPix[IND1(Ipix)]); 

    for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
      printf( "NbPixClas[%d]= %d\n", Ipro, NbPixClas[IND1(Ipro)]); 

    for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
      printf( "SumDistClas[%d]= %f\n", Ipro, SumDistClas[IND1(Ipro)]); 

    printf( "SumDistTot= %f\n", SumDistTot); 
  #endif

  // Caracteristiques des classes
  // Boucle sur les prototypes (Ipro = $)
  for( Ipro= 1; Ipro <= *NbPro; Ipro++)
  {
    #ifdef CMS_DEBUG
      printf( "=========> Appel SD_ALG_StatClas\n");
    #endif
    cr_op= SD_ALG_StatClas(
              // Entrees
               ProduitCourant,  // t_SD_BAS_DataThreadProduit*
               RadN,            // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
               IdPix,           // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
               Nch,             // int(1)
               NbPixAv,         // int(1)
               Ipro,            // int(1)
              // Sorties
              &Proto[IND2(1,Ipro,SD_BAS_NBK)],               // double( SD_BAS_NBK)
              &Covar[IND3(1,1,Ipro,SD_BAS_NBK,SD_BAS_NBK)]); // double( SD_BAS_NBK * SD_BAS_NBK)
  } // Fin de boucle sur les prototypes

  #ifdef CMS_DEBUG
    printf( "        SortingChannel : %d\n",  SortingChannel);
    printf( "                   Nch : %d\n",  Nch);
    printf( "               NbPixAv : %d\n",  NbPixAv);
    printf( "                 NbPro : %d\n", *NbPro);
    for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
    {
      printf( "      Proto(Ich=%12d,$) : ", Ich);
      for( Ipro= 1; Ipro <= *NbPro; Ipro++)
      {
        printf( "%12.5f", Proto[IND2(Ich,Ipro,SD_BAS_NBK)]);
      }
      printf( "\n");
    }

    for( Ipix= 1; Ipix <= 20 /*NbPixAv*/; Ipix++)
      printf( "IdPix[%d]= %d\n", Ipix, IdPix[IND1(Ipix)]); 

    for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
      printf( "NbPixClas[%d]= %d\n", Ipro, NbPixClas[IND1(Ipro)]); 

    for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
      printf( "SumDistClas[%d]= %f\n", Ipro, SumDistClas[IND1(Ipro)]); 

    printf( "=========> Appel SD_ALG_SortClas\n");
  #endif
  // Reclassement sur les valeurs du canal de classement par ordre decroissant des radiances
  cr_op= SD_ALG_SortClas(
            // Entrees
             SortingChannel,  // int(1)
             Nch,             // int(1)
             NbPixAv,         // int(1)
            *NbPro,           // int(1)
            // Entrees / Sorties
             Proto,           // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
             IdPix,           // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
             NbPixClas,       // int( SD_BAS_NCL_MAX)
             SumDistClas);    // double( SD_BAS_NCL_MAX)

fin:
SD_BAS_MacFREE(discu1);
SD_BAS_MacFREE(discu);
    // Traitement des erreurs
    //================================================================
    if ( cr_op != CMN__STATUT_OK )
    {
        // mise a jour de la variable ctx->cr.errinterne
        printf ("error after SD_ALG_SortClas");
        //TODO
    }

  // Fin de la fonction InitProto
  #ifdef CMS_DEBUG
    for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
    {
      printf( "      Proto(Ich=%d,$) : ", Ich);
      for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
      {
        printf( "%12.5f", Proto[IND2(Ich,Ipro,SD_BAS_NBK)]);
      }
      printf( "\n");
    }

    for( Ipix= 1; Ipix <= 20 /*NbPixAv*/; Ipix++)
      printf( "IdPix[%d]= %d\n", Ipix, IdPix[IND1(Ipix)]); 

    for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
      printf( "NbPixClas[%d]= %d\n", Ipro, NbPixClas[IND1(Ipro)]); 

    for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
      printf( "SumDistClas[%d]= %f\n", Ipro, SumDistClas[IND1(Ipro)]); 

    printf( "Fin de la fonction InitProto\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN InitProto =================================================================

//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_GestionProto
//@
//@ objet :  Construction des prototypes par eclatement et aglomeration
//@
//@
//@ Parametres :
//@         (ENTREE) ProduitCourant  : Pour pointer sur les variables temporaires
//@         (ENTREE) RadN .......... : Vecteur de radiance de l'image AVHRR
//@         (ENTREE) NbPixAv ....... : Nombre de pixels du vecteur
//@         (ENTREE) Nch ........... : Nombre de canaux presents
//@         (ENTREE) Aglo .......... : Flag d'agglomeration de classes
//@         (ENTREE) Separ ......... : Flag d'eclatement (split) de classes
//@         (ENTREE) EigVal ........ : Valeurs propres
//@         (ENTREE) EigVect ....... : Vecteurs propres
//@         (ENTREE) SplitCutoff ... : Seuil d'eclatement
//@         (ENTREE) GlueCutoff .... : Seuil de coalescence
//@         (ENTREE) EigValCutoff .. : Seuil de valeur propre
//@
//@         (ENTSOR) Proto ......... : Prototypes de classes
//@         (ENTSOR) NbPro ......... : Nombre de prototypes
//@         (ENTSOR) NbPixClas ..... : Nombre pixels par classe
//@         (ENTSOR) SumDistClas ... : Somme des distances intra-classes
//@
//@         (SORTIE) NoExt ......... : Flags de classes non extremes
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_GestionProto(
  // Entrees
  t_SD_BAS_DataThread* ctx,      // t_SD_BAS_DataThread*
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  float                *RadN,               // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
  int                   NbPixAv,            // int(1)
  int                   Nch,                // int(1)
  unsigned int          Aglo,               // unsigned int(1)
  unsigned int          Separ,              // unsigned int(1)
/* bma complexes->reels */
  double               *EigVal,             // double( SD_BAS_NBK)
  double               *EigVect,            // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
  double                SplitCutoff,        // double(1)
  double                DistAgloInit,       // double(1)
  double                GlueCutoff,         // double(1)
  double                EigValCutoff,       // double(1)
  // Entrees / Sortie
  double               *Proto,              // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
  int                  *NbPro,              // int(1)
  int                  *IdPix,              // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  int                  *NbPixClas,          // int( SD_BAS_NCL_MAX)
  double               *SumDistClas,        // double( SD_BAS_NCL_MAX)
  // Sorties
  unsigned int         *NoExt               // unsigned int( SD_BAS_NCL_MAX)
)
{
  static const char tcProcName_F[] = "SD_ALG_GestionProto";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  int     Ipro, Jpro;
  int     Ich, Jch;
  int     Ipix;
  int     Ivect;
  int     Id;
/* bma complexes->reels */
  double  ProtoEigVect[SD_BAS_NBK * SD_BAS_NCL_MAX];   // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
/* fin bma */
  double  DistAglo[SD_BAS_NCL_MAX * SD_BAS_NCL_MAX];       // double( SD_BAS_NCL_MAX * SD_BAS_NCL_MAX)
  double  ProtoAglo[SD_BAS_NBK];                   // double( SD_BAS_NBK)
  int    *IdPixAglo;                               // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  double *Dist;                                    // double( SD_BAS_NCL_MAX * SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  double  SumDistClasN[SD_BAS_NCL_MAX];                // double( SD_BAS_NCL_MAX)
  int     IdProSort[SD_BAS_NCL_MAX];                   // int( SD_BAS_NCL_MAX)
/* bma complexes->reels */
  int     NbNoExt;
  double  Minval, Maxval;
/* fin bma */
  int     NbVect;
  double  EigValCutoff2;
  double  DistMin;
  int     IdAglo1;
  int     IdAglo2;
  int     NbPixClasAglo;
  int     TailleAvhrr;
/* bma complexes->reels */
  double  DistIJ;
//  double  ModEigVal;
/* fin bma */
  double  DistAglo1;
  double  DistAglo2;
  double  Diff;
  double  SumDistClasAglo;
  int     IdProLast;
  int     NbPixLast;
  double  MaxDist0;
  int     IdMax;
  int k;
//jlp30 pour la rapidite
  const int irk=3;
  double *discu1;
  char *msg_err = (char*) NULL;          // Message d'erreur
  int num_lig = 0;            // Ligne d'erreur
  
  discu1=NULL;
  discu1 = (double *) calloc( NbPixAv,sizeof(double) );
 if( (discu1 == NULL)  )
  {
    cr_op   = SD_ALG_AVHRR_ERR_ALLOCATION;
    msg_err= (char *)SD_BAS_MSG_ALLOC;
    num_lig= __LINE__;

    goto fin;
  } 

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction GestionProto\n----------------------------------\n");
  #endif

  TailleAvhrr= SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER;

  IdPixAglo= ProduitCourant->IdPixAglo;
  Dist= ProduitCourant->Dist;

  // Definitions
  // (#) (1 a Nch) nombre de canaux/pseudo-canaux Avhrr utilises
  // (*) (1 a NbPixAv) nombre de pixels Avhrr
  // ($) (1 a NbPro) nombre de classes prototype

  // Si Aglo = True alors
  if( Aglo == CMN__OUI)
  {
    // Mecanisme d'agglomeration
    // Projection sur les vecteurs propres
    // ProtoEigVect(#,$) = EigVect(#,#) Proto(#,$)
    for( Ipro= 1; Ipro <= *NbPro; Ipro++)
    {
      for( Ich= 1; Ich <= Nch; Ich++)
      {
/* bma complexes->reels */
        ProtoEigVect[IND2(Ich,Ipro,SD_BAS_NBK)]= 0.0;
/* fin bma */
/* bma la boucle doit aller jusqu'a Nch et pas Nbpro*/
/*
        for( Jch= 1; Jch < *NbPro; Jch++)
*/
//jlp 2905 modif <= pour correction validation
        for( Jch= 1; Jch <= Nch; Jch++)
/* fin bma */
        {
/* bma complexes->reels */
	  //MOR 22/08/2003 pas de somme
// EigVect est stockee en ligne 
          ProtoEigVect[IND2(Ich,Ipro,SD_BAS_NBK)]+=   EigVect[IND2(Ich,Jch,SD_BAS_NBK)]
                                                   * Proto[IND2(Jch,Ipro,SD_BAS_NBK)];
//          ProtoEigVect[IND2(Ich,Ipro,SD_BAS_NBK)]+=   EigVect[IND2(Jch,Ich,SD_BAS_NBK)]
//                                                   * Proto[IND2(Ich,Ipro,SD_BAS_NBK)];
/* fin bma */
        }
      }
    }
  #ifdef OPS_DEBUG
//VERRUE
    printf("VERRUE EigVect\n");
    for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
    {
 
      for( Jch= 1; Jch <= SD_BAS_NBK; Jch++)
	{
	  printf( "%9.6f   ",  EigVect[IND2(Ich,Jch,SD_BAS_NBK)]);
	}
      printf( "|\n");
    }
  #endif

    // Recherche des classes non extremes sur les deux premiers vecteurs propres
    // NoExt($) = True
    for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)      NoExt[IND1(Ipro)]= CMN__OUI;

    // Boucle sur les vecteurs propres (Ivect = 1 a 2)
    for( Ivect= 1; Ivect <= 2; Ivect++)
    {
      // Id = index de Max{ ProtoEigVect(Ivect,$) }
      Id= 1;
      Maxval=  ProtoEigVect[IND2(Ivect,Id,SD_BAS_NBK)];
      for( Ipro= 1; Ipro <= *NbPro; Ipro++)
      {
        // Id= index de Max{ ProtoEigVect(Ivect,$) }
/* bma complexes->reels */
        if( ProtoEigVect[IND2(Ivect,Ipro,SD_BAS_NBK)] > Maxval)
        {
          Maxval= ProtoEigVect[IND2(Ivect,Ipro,SD_BAS_NBK)];
          Id= Ipro;
        }
/* fin bma */
      }

      // NoExt(Id) = False
      NoExt[IND1(Id)]= CMN__NON;

     // Id = index de Min{ ProtoEigVect(Ivect,$) }
      Id= 0;
      Minval= MAX_DOUBLE;
      for( Ipro= 1; Ipro <= *NbPro; Ipro++)
      {
        // Id= index de Max{ ProtoEigVect(Ivect,$) }
/* bma complexes->reels */
        if( ProtoEigVect[IND2(Ivect,Ipro,SD_BAS_NBK)] < Minval)
        {
          Minval= ProtoEigVect[IND2(Ivect,Ipro,SD_BAS_NBK)];
          Id= Ipro;
        }
/* fin bma */
      }

      // NoExt(Id) = False
      NoExt[IND1(Id)]= CMN__NON;

    } //Fin de boucle sur les vecteurs propres

    // NbNoExt = 0
    NbNoExt= 0;

    // Boucle sur les prototypes (Ipro = 1 a NbPro)
    for( Ipro= 1; Ipro <= *NbPro; Ipro++)
    {
      // Si NoExt(Ipro) = True alors NbNoExt = NbNoExt + 1 
      if( NoExt[IND1(Ipro)] == CMN__OUI)   NbNoExt= NbNoExt + 1;
    }

    // Si NbNoExt   2 alors
    if( NbNoExt >= 2)
    {
      // Limitation vecteurs propres hors bruit
      // NbVect= Max( index( EigVal(#)/EigVal(1) >= EigValCutoff ) }
      //MOR 01 sept 2003
      //NbVect= 0;
      //EigValCutoff2= EigValCutoff * EigValCutoff;
/* bma complexes->reels */

      for( Ich= 1; Ich <= Nch; Ich++)
      {
	//MOR 22/08/2003 pas de carre
        if( (EigVal[IND1(Ich)] / EigVal[IND1(1)]) >= EigValCutoff)     NbVect= Ich;
      }
/* fin bma */

      // Calcul des distances entre les agglomerables (non extremes)
      // DistAglo($,$) = DistAgloInit
      for( Ipro= 1; Ipro <= *NbPro; Ipro++)
      {
        for( Jpro= 1; Jpro <= *NbPro; Jpro++)
        {
          DistAglo[IND2(Ipro,Jpro,SD_BAS_NCL_MAX)]= DistAgloInit;
        }
      }
        
      // Boucle sur les Prototypes (Ipro = $) si NoExt(Ipro) = True
      for( Ipro= 2; Ipro <= *NbPro; Ipro++)
      {
        if( NoExt[IND1(Ipro)] == CMN__OUI)
        {
          // Boucle sur les prototypes (Jpro = Ipro - 1) si NoExt(Jpro) = True
          for( Jpro= 1; Jpro <= Ipro-1; Jpro++)
          {
            if( NoExt[IND1(Jpro)] == CMN__OUI)
            {
//jlp30 erreur de dimension
              DistAglo[IND2(Ipro,Jpro,SD_BAS_NCL_MAX)]= 0.0;
              for( Ich= 1; Ich <= NbVect; Ich++)
              {
/* bma complexes->reels */
                Diff=   ProtoEigVect[IND2(Ich,Ipro,SD_BAS_NBK)]
                      - ProtoEigVect[IND2(Ich,Jpro,SD_BAS_NBK)];
                DistIJ= Diff * Diff;
                DistAglo[IND2(Ipro,Jpro,SD_BAS_NCL_MAX)]=   DistAglo[IND2(Ipro,Jpro,SD_BAS_NCL_MAX)]
                                                      + DistIJ / EigVal[IND1(Ich)];
/* fin bma */
              }
            }
          }
        }
      }
        
      // Agglomeration des 2 classes les plus proches
      // [DistMin, IdAglo1, IdAglo2] = MinDistAgloLoc{ DistAglo($,$), NbPro }
      cr_op= SD_ALG_MinDistAgloLoc(
                                    // Entrees
                                     DistAglo,    // double( SD_BAS_NCL_MAX * SD_BAS_NCL_MAX)
                                    *NbPro,       // int(1)
                                    // Sorties
                                    &DistMin,     // double(1)
                                    &IdAglo1,     // int(1)
                                    &IdAglo2);    // int(1)

      // Si DistMin < GlueCutoff alors
      if( DistMin < GlueCutoff)
      {
        // Caracteristique de la classe resultante
        // NbPixClasAglo = NbPixClas(IdAglo1) + NbPixClas(IdAglo2)
        NbPixClasAglo= NbPixClas[IND1(IdAglo1)] + NbPixClas[IND1(IdAglo2)];

        DistAglo1= 0.0;
        DistAglo2= 0.0;
        
        // Canaux
        for( Ich= 1; Ich <= Nch; Ich++)
        {
          // ProtoAglo(#) =   [ Proto(#,IdAglo1)* NbPixClas(IdAglo1) 
          //                + Proto(#,IdAglo2)* NbPixClas(IdAglo2) ] / NbPixClasAglo
          ProtoAglo[IND1(Ich)]= (   Proto[IND2(Ich,IdAglo1,SD_BAS_NBK)]
                                  * NbPixClas[IND1(IdAglo1)]
                                  + Proto[IND2(Ich,IdAglo2,SD_BAS_NBK)]
                                  * NbPixClas[IND1(IdAglo2)] )
                                / (double)NbPixClasAglo;

          // DistAglo1= somme( ...)
          // DistAglo2= somme( ...)
          Diff= Proto[IND2(Ich,IdAglo1,SD_BAS_NBK)] - ProtoAglo[IND1(Ich)];
          DistAglo1= DistAglo1 + (Diff * Diff);
          Diff= Proto[IND2(Ich,IdAglo2,SD_BAS_NBK)] - ProtoAglo[IND1(Ich)];
          DistAglo2= DistAglo2 + (Diff * Diff);

        }

          // SumDistClasAglo = SumDistClas(IdAglo1) 
          //   + NbPixClas(IdAglo1)*DistAglo1 + NbPixClas(IdAglo2)*DistAglo2
          SumDistClasAglo=   SumDistClas[IND1(IdAglo1)]
                           + NbPixClas[IND1(IdAglo1)] * DistAglo1
                           + NbPixClas[IND1(IdAglo2)] * DistAglo2;

          // Reclassement des prototypes + non classes

          // IdPixAglo(*) = IdPix(*)
          for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
          {
            IdPixAglo[IND1(Ipix)]= IdPix[IND1(Ipix)];
          }

          // Boucle sur les prototypes (Ipro = 1 a NbPro +1)
          for( Ipro= 1; Ipro <= (*NbPro + 1); Ipro++)
          {
            // Si Ipro > IdAglo2 alors
            if( Ipro > IdAglo2)
            {
              // Proto(#,Ipro-1) = Proto(#,Ipro) et Proto(#,Ipro) = 0.
              for( Ich= 1; Ich <= Nch; Ich++)
              {
                Proto[IND2(Ich,Ipro-1,SD_BAS_NBK)]= Proto[IND2(Ich,Ipro,SD_BAS_NBK)];
                Proto[IND2(Ich,Ipro,SD_BAS_NBK)]= 0.0;
              } //FIN for Ich

              // SumDistClas(Ipro-1) = SumDistClas(Ipro) et SumDistClas(Ipro) =  0.
              SumDistClas[IND1(Ipro-1)]= SumDistClas[IND1(Ipro)];
              SumDistClas[IND1(Ipro)]= 0.0;

              // NbPixClas(Ipro-1) = NbPixClas(Ipro)
              NbPixClas[IND1(Ipro-1)]= NbPixClas[IND1(Ipro)];

              // NoExt(Ipro-1) = NoExt(Ipro)
              NoExt[IND1(Ipro-1)]= NoExt[IND1(Ipro)];
            } // Fin de si
//jlp30 
         } // Fin de boucle sur les prototypes
         // Boucle sur les pixels (Ipix = *)
         for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
         {
              // Si IdPix(Ipix) = IdAglo2 alors IdPixAglo(Ipix) = IdAglo1
              if( IdPix[IND1(Ipix)] == IdAglo2)
                IdPixAglo[IND1(Ipix)]= IdAglo1;

              // Si IdPix(Ipix) > IdAglo2 alors IdPixAglo(Ipix) = IdPix(Ipix) - 1
              if( IdPix[IND1(Ipix)] > IdAglo2)
                IdPixAglo[IND1(Ipix)]= IdPix[IND1(Ipix)] - 1;

         } // Fin de boucle sur les pixels

 
          // Proto(#,IdAglo1) = ProtoAglo(#)
            for( Ich= 1; Ich <= Nch; Ich++)
                Proto[IND2(Ich,IdAglo1,SD_BAS_NBK)]= ProtoAglo[IND1(Ich)];
          // SumDistClas(IdAglo1) = SumDistClasAglo
          SumDistClas[IND1(IdAglo1)]= SumDistClasAglo;

          // NbPixClas(IdAglo1) = NbPixClasAglo
          NbPixClas[IND1(IdAglo1)]= NbPixClasAglo;

          // NoExt(IdAglo1) = True
          NoExt[IND1(IdAglo1)]= CMN__OUI;

          // IdPix(*) = IdPixAglo(*)
          for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
          {
            IdPix[IND1(Ipix)]= IdPixAglo[IND1(Ipix)];
          }

          // NbPro = NbPro - 1
          *NbPro= *NbPro - 1;
      } // Fin de si (DistMin < GlueCutoff)
    } // Fin de si (NbNoExt   2)
  } // Fin de si (Aglo = True)

  // Si Separ = True
  if( Separ == CMN__OUI)
  {
    // Mecanisme de Separation
    // Classement en fonction des dispersions
    // Distance intra-classe normalisee
    // SumDistClasN($) = SumDistClas($) / Max{ NbPixClas($), SplitCutoff }
    for( Ipro= 1; Ipro <= *NbPro; Ipro++)
    {
      SumDistClasN[IND1(Ipro)]=   SumDistClas[IND1(Ipro)]
                                / MAX( NbPixClas[IND1(Ipro)], SplitCutoff);
    }
    
    // Tri ascendant
    // IdProSort($) = Tri ascendant{ SumDistClasN }
    cr_op= SD_ALG_Tri_ascendant( // Entrees
  				  ctx,      // t_SD_BAS_DataThread*
                                  SumDistClasN,   // double( SD_BAS_NCL_MAX)
                                 *NbPro,          // int(1)
                                 // Sorties
                                  IdProSort);     // int( SD_BAS_NCL_MAX)
    if ( cr_op != CMN__STATUT_OK )
    {
    goto fin;
    }

    // IdProLast = IdProSort(NbPro)
    IdProLast= IdProSort[IND1(*NbPro)];

    // NbPixLast = NbPixClas(IdProLast)
    NbPixLast= NbPixClas[IND1(IdProLast)];

    // Si NbPixLast > SplitCutoff alors
    if( NbPixLast > SplitCutoff)
    {
      // Recherche des extrema dans la classe la plus forte et separation
      // Dist(IdProLast,*) = DistPixProtoClas{ RadN(#,*), Nch, NbPixAv, IdPix(*), IdProLast, Proto(#,IdProLast) }
      cr_op= SD_ALG_DistPixProtoClas(
                // Entrees
                 RadN,                                  // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
                 Nch,                                   // int(1)
                 NbPixAv,                               // int(1)
                 IdPix,                                 // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
                 IdProLast,                             // int(1)
                &Proto[IND2(1,IdProLast,SD_BAS_NBK)],   // double( SD_BAS_NBK)
                // Sorties
                &Dist[IND2(1,IdProLast,TailleAvhrr)]);  // double( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)

      // IdMax= Index de Max{Dist(IdProLast,*)}
      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
        discu1[IND1(Ipix)] = Dist[IND2(Ipix,IdProLast,TailleAvhrr)];
// fonction index de max 3
// version 5.0 
//jlp30 pour la rapidite
// Imax = Index de Max3{ Dist1(Idprolast,*) }
//    cr_op= SD_ALG_Index_De_Max3(ctx,NbPixAv,discu1, &IdMax);
      for(k=1 ; k<=irk ; k++)
      {
          MaxDist0= -1000000.0;
        IdMax= 0;
        for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
        {
          if( discu1[IND1(Ipix)] > MaxDist0)
          {
            MaxDist0= discu1[IND1(Ipix)];
            IdMax= Ipix;
           }
        }
        discu1[IND1(IdMax)] = -1000.0;
      }

      // Proto(#,IdProLast) = RadN(#,IdMax)
      for( Ich= 1; Ich <= Nch; Ich++)
        Proto[IND2(Ich,IdProLast,SD_BAS_NBK)]= (double)RadN[IND2(IdMax,Ich,TailleAvhrr)];

      // Dist(IdProLast,*) = DistPixProtoClas{ RadN(#,*), Nch, NbPixAv, IdPix(*), IdProLast, Proto(#,IdProLast) }
      cr_op= SD_ALG_DistPixProtoClas(
                // Entrees
                 RadN,                                  // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
                 Nch,                                   // int(1)
                 NbPixAv,                               // int(1)
                 IdPix,                                 // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
                 IdProLast,                             // int(1)
                &Proto[IND2(1,IdProLast,SD_BAS_NBK)],   // double( SD_BAS_NBK)
                // Sorties
                &Dist[IND2(1,IdProLast,TailleAvhrr)]);  // double( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)

      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
        discu1[IND1(Ipix)] = Dist[IND2(Ipix,IdProLast,TailleAvhrr)];
// fonction index de max 3
// version 5.0 
//jlp30 pour la rapidite
// Imax = Index de Max3{ Dist1(Idprolast,*) }
//    cr_op= SD_ALG_Index_De_Max3(ctx,NbPixAv,discu1,&IdMax );
      for(k=1 ; k<=irk ; k++)
      {
       MaxDist0= -1000000.0;
         IdMax= 0;
        for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
        {
          if( discu1[IND1(Ipix)] > MaxDist0)
          {
            MaxDist0= discu1[IND1(Ipix)];
            IdMax= Ipix;
          }
        }
        discu1[IND1(IdMax)] = -1000.0;
      }

      // Proto(#,NbPro+1) = RadN(#,IdMax)
      for( Ich= 1; Ich <= Nch; Ich++)
        Proto[IND2(Ich,*NbPro+1,SD_BAS_NBK)]= (double)RadN[IND2(IdMax,Ich,TailleAvhrr)];
      
  #ifdef OPS_DEBUG
    for( Ich= 1; Ich <= Nch; Ich++)
      printf( "Proto[%d,%d]= %f\n", Ich, *NbPro+1, Proto[IND2(Ich,*NbPro+1,SD_BAS_NBK)]); 
  #endif

      // NbPro = NbPro + 1
      *NbPro= *NbPro + 1;

    } // Fin de si (NbPixLast > SplitCutoff)
  } // Fin de si (Separ = True)


fin:
  SD_BAS_MacFREE(discu1);
    // Traitement des erreurs
    //================================================================
    if ( cr_op != CMN__STATUT_OK )
    {
        // TODO
        printf ("Error\n");
    }


  // Fin de la fonction GestionProto
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction GestionProto\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN GestionProto ==============================================================
// ===============================================================================




//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_GestionProtoPost
//@
//@ objet :  Agglomeration forcee des 2 classes non extremes les plus proches
//@
//@
//@ Parametres :
//@         (ENTREE) ProduitCourant  : Pour pointer sur les variables temporaires
//@         (ENTREE) Nch ........... : Nombre de canaux presents
//@         (ENTREE) NbPixAv........ : Nombre de Pixels Avhrr
//@         (ENTREE) EigVal ........ : Valeurs propres
//@         (ENTREE) EigVect ....... : Vecteurs propres
//@         (ENTREE) DistAgloInit .. : Valeur initiale de la distance d'aglomeration
//@         (ENTREE) EigValCutoff .. : Seuil de valeur propre
//@
//@         (ENTSOR) Proto ......... : Prototypes de classes
//@         (ENTSOR) NbPro ......... : Nombre de prototypes
//@         (ENTSOR) IdPix ......... : Flags de pixels valides
//@         (ENTSOR) NbPixClas ..... : Nombre pixels par classe
//@         (ENTSOR) SumDistClas ... : Somme des distances intra-classes
//@         (ENTSOR) NoExt ......... : Flags de classes non extremes
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_GestionProtoPost(
  // Entrees
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  int                   Nch,                // int(1)
/* bma ajout du nombre de pixels AVHRR dans la signature de la fonction GestionProtoPost*/
  int                          NbPixAv,            // int(1)
/* fin bma */
/* bma complexes->reels */
  double               *EigVal,             // double( SD_BAS_NBK)
  double               *EigVect,            // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
  double                DistAgloInit,       // double(1)
  double                EigValCutoff,       // double(1)
  // Entrees / Sortie
  double               *Proto,              // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
  int                  *NbPro,              // int(1)
  int                  *IdPix,              // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  int                  *NbPixClas,          // int( SD_BAS_NCL_MAX)
  double               *SumDistClas,        // double( SD_BAS_NCL_MAX)
  unsigned int         *NoExt               // unsigned int( SD_BAS_NCL_MAX)
)
{
  static const char tcProcName_F[] = "SD_ALG_GestionProtoPost";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  int     Ipro, Jpro;
  int     Ich, Jch;
  int     Ipix;
  int     Ivect;
  int     Id;
/* bma complexes->reels */
  double  ProtoEigVect[SD_BAS_NBK * SD_BAS_NCL_MAX];   // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
/* fin bma */
  double  DistAglo[SD_BAS_NCL_MAX * SD_BAS_NCL_MAX];       // double( SD_BAS_NCL_MAX * SD_BAS_NCL_MAX)
  double  ProtoAglo[SD_BAS_NBK];                   // double( SD_BAS_NBK)
  int    *IdPixAglo;                               // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  double  SumDistClasN[SD_BAS_NCL_MAX];                // double( SD_BAS_NCL_MAX)
  double  IdProSort[SD_BAS_NCL_MAX];                   // double( SD_BAS_NCL_MAX)
/* bma complexes->reels */
  int     NbNoExt;
  double  Minval, Maxval;
/* fin bma */
  int     NbVect;
  double  EigValCutoff2;
  double  DistMin;
  int     IdAglo1;
  int     IdAglo2;
  int     NbPixClasAglo;
  int     TailleAvhrr;
/* bma complexes->reels */
  double  Diff;
  double  DistIJ;
//  double  ModEigVal;
/* fin bma */
  double  DistAglo1;
  double  DistAglo2;
  double  SumDistClasAglo;

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction GestionProtoPost\n----------------------------------\n");
  #endif

  TailleAvhrr= SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER;

  IdPixAglo= ProduitCourant->IdPixAglo;

  // Definitions
  // (#) (1 a Nch) nombre de canaux/pseudo-canaux Avhrr utilises
  // (*) (1 a NbPixAv) nombre de pixels Avhrr
  // ($) (1 a NbPro) nombre de classes prototype

  // Projection sur les vecteurs propres
  // ProtoEigVect(#,$) = EigVect(#,#) Proto(#,$)   (Produit scalaire)
  for( Ipro= 1; Ipro <= *NbPro; Ipro++)
  {
    for( Ich= 1; Ich <= Nch; Ich++)
    {
      ProtoEigVect[IND2(Ich,Ipro,SD_BAS_NBK)]= 0.0;
//jlp 2905 modif <= Nch pour correction validation
      for( Jch= 1; Jch <= Nch; Jch++)
      {
        // Sommation
// EigVect est stockee en ligne 
          ProtoEigVect[IND2(Ich,Ipro,SD_BAS_NBK)]+=   EigVect[IND2(Ich,Jch,SD_BAS_NBK)]
                                                   * Proto[IND2(Jch,Ipro,SD_BAS_NBK)];
      }
    }
  }

  // Limitation vecteurs propres hors bruit
  // NbVect= Max( index( EigVal(#)/EigVal(1) >= EigValCutoff ) }
  NbVect= 0;
//jlp  25/05 correction seuil
//  EigValCutoff2= EigValCutoff * EigValCutoff;

/* bma complexes->reels */
  for( Ich= 1; Ich <= Nch; Ich++)
  {
//jlp? 25/05 correction seuil
    if( (EigVal[IND1(Ich)] / EigVal[IND1(1)]) >= EigValCutoff)     NbVect= Ich;
  }
/* fin bma */

  // Calcul des distances entre les agglomerables (non extremes)
  for( Ipro= 1; Ipro <= *NbPro; Ipro++)
  {
    for( Jpro= 1; Jpro <= *NbPro; Jpro++)
    {
      DistAglo[IND2(Ipro,Jpro,SD_BAS_NCL_MAX)]= DistAgloInit;
    }
  }
        
  // Double boucle sur les Prototypes (Ipro, Jpro = $) si NoExt(Ipro) & NoExt(Jpro) = True
  for( Ipro= 2; Ipro <= *NbPro; Ipro++)
  {
    if( NoExt[IND1(Ipro)] == CMN__OUI)
    {
      for( Jpro= 1; Jpro <= Ipro-1; Jpro++)
     {
        if( NoExt[IND1(Ipro)] == CMN__OUI)
        {
          DistAglo[IND2(Ipro,Jpro,SD_BAS_NCL_MAX)]= 0.0;
/* bma :  a boucle ne doit pas aller a Nch mais NbVect*/
/*
          for( Ich= 1; Ich <= Nch; Ich++)
*/
          for( Ich= 1; Ich <= NbVect; Ich++)
/* fin bma */
          {
/* bma complexes->reels */
            Diff=   ProtoEigVect[IND2(Ich,Ipro,SD_BAS_NBK)]
                  - ProtoEigVect[IND2(Ich,Jpro,SD_BAS_NBK)];
            DistIJ= Diff * Diff;
            DistAglo[IND2(Ipro,Jpro,SD_BAS_NCL_MAX)]=   DistAglo[IND2(Ipro,Jpro,SD_BAS_NCL_MAX)]
                                                  + DistIJ / EigVal[IND1(Ich)];
/* fin bma */
          }
        }
      }
    }
  } // Fin de double boucle sur les prototypes

  // Agglomeration des 2 classes les plus proches
  // [DistMin, IdAglo1, IdAglo2] = MinDistAgloLoc{ DistAglo($,$), NbPro }
  cr_op= SD_ALG_MinDistAgloLoc( // Entrees
                                 DistAglo,    // double( SD_BAS_NCL_MAX * SD_BAS_NCL_MAX)
                                *NbPro,       // int(1)
                                // Sorties
                                &DistMin,     // double(1)
                                &IdAglo1,     // int(1)
                                &IdAglo2);    // int(1)

  // Caracteristique de la classe resultante
  // NbPixClasAglo = NbPixClas(IdAglo1) + NbPixClas(IdAglo2)
  NbPixClasAglo= NbPixClas[IND1(IdAglo1)] + NbPixClas[IND1(IdAglo2)];

  DistAglo1= 0.0;
  DistAglo2= 0.0;

  // Canaux
  for( Ich= 1; Ich <= Nch; Ich++)
  {
    // ProtoAglo(#) =   [ Proto(#,IdAglo1)* NbPixClas(IdAglo1) 
    //                + Proto(#,IdAglo2)* NbPixClas(IdAglo2) ] / NbPixClasAglo
    ProtoAglo[IND1(Ich)]= (   Proto[IND2(Ich,IdAglo1,SD_BAS_NBK)]
                            * NbPixClas[IND1(IdAglo1)]
                            + Proto[IND2(Ich,IdAglo2,SD_BAS_NBK)]
                            * NbPixClas[IND1(IdAglo2)] )
                          / NbPixClasAglo;

    Diff= Proto[IND2(Ich,IdAglo1,SD_BAS_NBK)] - ProtoAglo[IND1(Ich)];
    DistAglo1= DistAglo1 + (Diff * Diff);

    Diff= Proto[IND2(Ich,IdAglo2,SD_BAS_NBK)] - ProtoAglo[IND1(Ich)];
    DistAglo2= DistAglo2 + (Diff * Diff);

  } //FIN for Ich (mal place)

    // SumDistClasAglo = SumDistClas(IdAglo1) 
    //   + NbPixClas(IdAglo1)*DistAglo1 + NbPixClas(IdAglo2)*DistAglo2
    SumDistClasAglo=   SumDistClas[IND1(IdAglo1)]
                     + NbPixClas[IND1(IdAglo1)] * DistAglo1
                     + NbPixClas[IND1(IdAglo2)] * DistAglo2;

    // Reclassement des prototypes + non classes

    // IdPixAglo(*) = IdPix(*)
    for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
    {
      IdPixAglo[IND1(Ipix)]= IdPix[IND1(Ipix)];
    }

    // Boucle sur les prototypes (Ipro = 1 a NbPro +1)
    for( Ipro= 1; Ipro <= (*NbPro + 1); Ipro++)
    {
      // Si Ipro > IdAglo2 alors
      if( Ipro > IdAglo2)
      {
        for( Ich= 1; Ich <= Nch; Ich++)
        {
          // Proto(#,Ipro-1) = Proto(#,Ipro) et Proto(#,Ipro) = 0.
          Proto[IND2(Ich,Ipro-1,SD_BAS_NBK)]= Proto[IND2(Ich,Ipro,SD_BAS_NBK)];
          Proto[IND2(Ich,Ipro,SD_BAS_NBK)]= 0.0;
        } //FIN for Ich

        // SumDistClas(Ipro-1) = SumDistClas(Ipro) et SumDistClas(Ipro) =  0.
        SumDistClas[IND1(Ipro-1)]= SumDistClas[IND1(Ipro)];
        SumDistClas[IND1(Ipro)]= 0.0;

        // NbPixClas(Ipro-1) = NbPixClas(Ipro)
        NbPixClas[IND1(Ipro-1)]= NbPixClas[IND1(Ipro)];

        // NoExt(Ipro-1) = NoExt(Ipro)
        NoExt[IND1(Ipro-1)]= NoExt[IND1(Ipro)];
      } // Fin de si
    } // Fin de boucle sur les prototypes

      // Boucle sur les pixels (Ipix = *)
      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
      {
        // Si IdPix(Ipix) = IdAglo2 alors IdPixAglo(Ipix) = IdAglo1
        if( IdPix[IND1(Ipix)] == IdAglo2)
          IdPixAglo[IND1(Ipix)]= IdAglo1;

        // Si IdPix(Ipix) > IdAglo2 alors IdPixAglo(Ipix) = IdPix(Ipix) - 1
        if( IdPix[IND1(Ipix)] > IdAglo2)
          IdPixAglo[IND1(Ipix)]= IdPix[IND1(Ipix)] - 1;

      } // Fin de boucle sur les pixels


    // Proto(#,IdAglo1) = ProtoAglo(#)
    for( Ich= 1; Ich <= Nch; Ich++)
      Proto[IND2(Ich,IdAglo1,SD_BAS_NBK)]= ProtoAglo[IND1(Ich)];

    // SumDistClas(IdAglo1) = SumDistClasAglo
    SumDistClas[IND1(IdAglo1)]= SumDistClasAglo;

    // NbPixClas(IdAglo1) = NbPixClasAglo
    NbPixClas[IND1(IdAglo1)]= NbPixClasAglo;

    // NoExt(IdAglo1) = True
    NoExt[IND1(IdAglo1)]= CMN__OUI;

    // IdPix(*) = IdPixAglo(*)
    for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
    {
      IdPix[IND1(Ipix)]= IdPixAglo[IND1(Ipix)];
    }

    // NbPro = NbPro - 1
    *NbPro= *NbPro - 1;

fin:
  // Fin de la fonction GestionProtoPost
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction GestionProtoPost\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN GestionProtoPost ==========================================================
// ===============================================================================



//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_DistPixProto
//@
//@ objet :   Distances entre les valeurs des pixels de l'imagette AVHRR
//@           et les valeurs d'un prototype donne
//@
//@
//@ Parametres :
//@         (ENTREE) ImgAv ...... : Vecteur de radiance de l'image AVHRR
//@         (ENTREE) NbChannel .. : Nombre de canaux AVHRR a traiter
//@         (ENTREE) NbPixAv .... : Nombre de pixels du vecteur
//@         (ENTREE) Proto ...... : Prototypes de classes
//@
//@         (SORTIE) Dist ....... : Distances des pixels aux prototypes
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_DistPixProto(
  // Entrees
  float  *ImgAv,      // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
  int     NbChannel,  // int(1)
  int     NbPixAv,    // int(1)
  double *Proto,      // double( SD_BAS_NBK)
  // Sorties
  double *Dist        // double( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
)
{
  // Distance quadratiques entre les valeurs des pixels de
  // l'imagette AVHRR et la valeur d'un prototype donne
  static const char tcProcName_F[] = "SD_ALG_DistPixProto";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  int    Ipix;        // Index sur les pixels
  int    Ich;         // Index sur les canaux
  int    TailleAvhrr; // Taille image AVHRR
  double Diff;

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction DistPixProto\n----------------------------------\n");
  #endif

  TailleAvhrr= SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER;

  // Boucle sur les pixels (Ipix = 1 a NbPixAv)
  for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
  {
    Dist[IND1(Ipix)]= 0.0;
    for( Ich= 1; Ich <= NbChannel; Ich++)
    {
      Diff= ImgAv[IND2(Ipix,Ich,TailleAvhrr)] - Proto[IND1(Ich)];
      Dist[IND1(Ipix)]= Dist[IND1(Ipix)] + Diff*Diff;
    }
  }  // Fin de boucle sur les pixels AVHRR

fin:
  // Fin de la fonction DistPixProto
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction DistPixProto\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN DistPixProto ==============================================================
// ===============================================================================



//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_DistPixProtoClas
//@
//@ objet :   Distances entre les valeurs des pixels d'une classe et
//@           les valeurs du prototype de cette classe
//@
//@
//@ Parametres :
//@         (ENTREE) ImgAv ...... : Vecteur de radiance de l'image AVHRR
//@         (ENTREE) NbChannel .. : Nombre de canaux AVHRR a traiter
//@         (ENTREE) NbPixAv .... : Nombre de pixels du vecteur
//@         (ENTREE) IdPix ...... : Flags de pixels valides
//@         (ENTREE) IdPro ...... : Numero de prototype
//@         (ENTREE) Proto ...... : Prototypes de classes
//@
//@         (SORTIE) DistClass .. : Distances des pixels aux prototypes
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_DistPixProtoClas(
  // Entrees
  float  *ImgAv,      // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
  int     NbChannel,  // int(1)
  int     NbPixAv,    // int(1)
  int    *IdPix,      // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  int     IdPro,      // int(1)
  double *Proto,      // double( SD_BAS_NBK)
  // Sorties
  double *DistClas    // double( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NCL_MAX)
)
{
  // Distance quadratiques entre les valeurs des pixels de
  // l'imagette AVHRR et la valeur d'un prototype donne
  static const char tcProcName_F[] = "SD_ALG_DistPixProtoClas";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  int    Ipix;        // Index sur les pixels
  int    Ich;         // Index sur les canaux
  int    IndClas;     // Index sur les classes
  int    TailleAvhrr; // Taille image AVHRR
  double Diff;

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction DistPixProtoClas\n----------------------------------\n");
  #endif

  TailleAvhrr= SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER;

  //MOR 22/08/2003  DistClas a une dimension ici 

  // Boucle sur les pixels (Ipix = 1 a NbPixAv)
  for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
  {
    DistClas[IND1(Ipix)]= 0.0;
    if( IdPix[IND1(Ipix)] == IdPro)
    {
      for( Ich= 1; Ich <= NbChannel; Ich++)
      {
        Diff= ImgAv[IND2(Ipix,Ich,TailleAvhrr)] - Proto[IND1(Ich)];
        DistClas[IND1(Ipix)]=   DistClas[IND1(Ipix)] + Diff*Diff;
      }
    }
  }  // Fin de boucle sur les pixels AVHRR

fin:
  // Fin de la fonction DistPixProtoClas
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction DistPixProtoClas\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN DistPixProtoClas ==========================================================
// ===============================================================================



//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_MinDistAgloLoc
//@
//@ objet :  Recherche la valeur minimale de la matrice DistAglo
//@          ainsi que les 2 indices correspondants 
//@
//@
//@ Parametres :
//@         (ENTREE) DistAglo .. : Matrice des distances inter-prototypes
//@         (ENTREE) NbPro ..... : Nombre de prototypes
  // Sorties
//@         (SORTIE) DistMin ... : Distance minimale
//@         (SORTIE) Ind1 ...... : Indice colonne
//@         (SORTIE) Ind2 ...... : Indice ligne
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_MinDistAgloLoc(
  // Entrees
  double *DistAglo,    // double( SD_BAS_NCL_MAX * SD_BAS_NCL_MAX)
  int     NbPro,       // int(1)
  // Sorties
  double *DistMin,     // double(1)
  int    *Ind1,        // int(1)
  int    *Ind2         // int(1)
)
{
  // Recherche la valeur minimale de la matrice DistAglo
  // ainsi que les 2 indices correspondants 
  static const char tcProcName_F[] = "SD_ALG_MinDistAgloLoc";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  int Ipro, Jpro;
  int temp;  
  #ifdef OPS_DEBUG
    printf( "Debut de la fonction MinDistAgloLoc\n----------------------------------\n");
  #endif

  // ($) = (1 a NbPro) nombre de prototypes
  // DistMin = Min{ DistAglo($,$)}
  // [ Ind1, Ind2 ] = indices de Min{ DistAglo($,$)}

  *DistMin= MAX_DOUBLE;
  *Ind1= 0;
  *Ind2= 0;
  for( Jpro= 1; Jpro <= NbPro; Jpro++)
  {
    for( Ipro= 1; Ipro <= NbPro; Ipro++)
    {
      if( DistAglo[IND2(Ipro,Jpro,SD_BAS_NCL_MAX)] < *DistMin)
      {
        *DistMin= DistAglo[IND2(Ipro,Jpro,SD_BAS_NCL_MAX)];
        *Ind1= Ipro;
        *Ind2= Jpro;
      }
    }
  }
// 5.0 PM 3
// si ind 1 > ind2 on echange ind 1 et ind2
if (*Ind1 > *Ind2)
  {
	temp=*Ind1;
        *Ind1= *Ind2;
        *Ind2= temp;
  }

fin:
  // Fin de la fonction MinDistAgloLoc
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction MinDistAgloLoc\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN MinDistAgloLoc ============================================================
// ===============================================================================


//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_LinkPixProto
//@
//@ objet :  Attribution des pixels aux differentes classes definies.
//@          Calcul du nombre de pixels par classe, de la taille de
//@          chaque classe et de la taille globale 
//@
//@ Parametres :
//@         (ENTREE) Dist ...... : Matrice des distances inter-prototypes
//@         (ENTREE) DistMax ... : Seuil de validite en distance
//@         (ENTREE) NbPixAv ... : Nombre de pixels du vecteur AVHRR
//@         (ENTREE) NbPro ..... : Nombre de prototypes
//@
//@         (SORTIE) IdPix ..... : Flags de pixels valides
//@         (SORTIE) NbPixClas . : Nombre pixels par classe
//@         (SORTIE) SumDistClas : Somme des distances intra-classes (Taille des classes)
//@         (SORTIE) SumDistTot  : Somme des distances (Taille de l'ensemble des classes)
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_LinkPixProto(
  // Entrees
  double *Dist,         // double( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NCL_MAX)
  double  DistMax,      // double(1)
  int     NbPixAv,      // int(1)
  int     NbPro,        // int(1)
  // Sorties
  int    *IdPix,        // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  int    *NbPixClas,    // int( SD_BAS_NCL_MAX)
  double *SumDistClas,  // double( SD_BAS_NCL_MAX)
  double *SumDistTot    // double(1)
)
{
  // Attribution des pixels aux differentes classes definies.
  // Calcul de nombre de pixels par classe, de la taille de chaque
  // classe et de la taille globale 
  static const char tcProcName_F[] = "SD_ALG_LinkPixProto";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  int    Ipro;           // Index sur les prototypes
  int    Ipix;           // Index sur les pixels
  double MinDist;        // Distance minimale
  int    TailleAvhrr;    // Taille d'une image AVHRR
  int    Id;             // Index sur les pixels

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction LinkPixProto\n----------------------------------\n");
    printf( "DistMax= %f\n", DistMax);
    printf( "NbPixAv= %d\n", NbPixAv);
    printf( "NbPro= %d\n",   NbPro);
  #endif

  TailleAvhrr= SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER;

  // Initialisation
  // NbPixClas(1 a NbPro+1) = 0
  // SumDistClas(1 a NbPro+1) =  0
  for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
  {
    NbPixClas[IND1(Ipro)]= 0;
    SumDistClas[IND1(Ipro)]= 0.0;
  }


  // Boucle sur les pixels (Ipix = 1 a NbPixAv)
  for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
  {
    // Recherche du numero de prototype associe au pixel
    // Id = indice de Min{Dist(1 a NbPro, Ipix)}
    MinDist= MAX_DOUBLE;
    Id= 0;
    for( Ipro= 1; Ipro <= NbPro; Ipro++)
    {
      if( Dist[IND2(Ipix,Ipro,TailleAvhrr)] < MinDist)
      {
        MinDist= Dist[IND2(Ipix,Ipro,TailleAvhrr)];
        Id= Ipro;
      }
    }

    // Si Dist(Id,Ipix) <= DistMax alors (attribution OK)
    if( Dist[IND2(Ipix,Id,TailleAvhrr)] <= DistMax)
    {
      // IdPix(Ipix) = Id
      IdPix[IND1(Ipix)]= Id;

      // SumDistClas(Id) = SumDistClas(Id) + Dist(Id,Ipix)
      SumDistClas[IND1(Id)]= SumDistClas[IND1(Id)] + Dist[IND2(Ipix,Id,TailleAvhrr)];
    }
    else
    {
      // Sinon (attribution " non classe ")
      //IdPix(Ipix) = NbPro +1
      Id= NbPro + 1;

      IdPix[IND1(Ipix)]= NbPro + 1;
    } // Fin de si (Dist(Id,Ipix) DistMax)

    // NbPixClas(Id) = NbPixClas(Id) +1
    NbPixClas[IND1(Id)]= NbPixClas[IND1(Id)] + 1;

  } // Fin de boucle sur les pixels

  // Calcul de la somme des distances intra-classes
  // SumDistTot = somme(...)
  *SumDistTot= 0.0;
  for( Ipro= 1; Ipro <= NbPro; Ipro++)
    *SumDistTot= *SumDistTot + SumDistClas[IND1(Ipro)];


fin:
  // Fin de la fonction LinkPixProto
  #ifdef OPS_DEBUG
    printf( "      NbPixClas($) : ");
    for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
    {
      printf( "%7d", NbPixClas[IND1(Ipro)]);
    }
    printf( "\n");
    printf( "NbPixClas(NbPro+1) : %7d\n", NbPixClas[IND1(NbPro+1)]);
    printf( "    SumDistClas($) : ");
    for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
    {
      printf( "%12.5f", SumDistClas[IND1(Ipro)]);
    }
    printf( "\n");
    printf( "SumDistTot= %f\n", *SumDistTot);
    printf( "Fin de la fonction LinkPixProto\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN LinkPixProto ==============================================================
// ===============================================================================


//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_SortClas
//@
//@ objet :  Rangement des classes par ordre decroissant de la valeur
//@          du prototype sur le canal de classement
//@
//@ Parametres :
//@         (ENTREE) SortingChannel : Canal de rangement
//@         (ENTREE) NbChannel .... : Nombre de canaux a traiter
//@         (ENTREE) NbPixAv ...... : Nombre de pixels du vecteur
//@         (ENTREE) Proto ........ : Prototypes de classes
//@
//@         (SORTIE) IdPix ........ : Flags de pixels valides
//@         (SORTIE) NbPixClas .... : Nombre pixels par classe
//@         (SORTIE) SumDistClas .. : Somme des distances intra-classes (Taille des classes)
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_SortClas(
    // Entrees
    int     SortingChannel,  // int(1)
    int     NbChannel,       // int(1)
    int     NbPixAv,         // int(1)
    int     NbPro,           // int(1)
    // Entrees / Sorties
    double *Proto,           // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
    int    *IdPix,           // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
    int    *NbPixClas,       // int( SD_BAS_NCL_MAX)
    double *SumDistClas      // double( SD_BAS_NCL_MAX)
)
{
  static const char tcProcName_F[] = "SD_ALG_SortClas";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  double ProtoWork[SD_BAS_NBK*SD_BAS_NCL_MAX];      // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
  int    Ich;                                   // Index sur les canaux
  int    Ipix;                                  // Index sur les pixels
  int    Ipro;                                  // Index sur les prototypes
  int    Jpro;                                  // Index sur les prototypes
  int    IndexSort[SD_BAS_NCL_MAX];                 // int( SD_BAS_NCL_MAX)
  int    IndexSortWork[SD_BAS_NCL_MAX];             // int( SD_BAS_NCL_MAX)
  int    NbPixClasWork[SD_BAS_NCL_MAX];             // int( SD_BAS_NCL_MAX)
  double SumDistClasWork[SD_BAS_NCL_MAX];           // double( SD_BAS_NCL_MAX)
  int    Id[SD_BAS_NCL_MAX];                        // int( SD_BAS_NCL_MAX)
  int    Ind;                                   // Index general


  #ifdef OPS_DEBUG
    int  IproTr;           // Index sur les prototypes
    int  JproTr;           // Index sur les prototypes
    printf( "Debut de la fonction SortClas\n----------------------------------\n");
    printf( "SortingChannel : %d\n", SortingChannel);
    printf( "     NbChannel : %d\n", NbChannel);
    printf( "       NbPixAv : %d\n", NbPixAv);
    printf( "         NbPro : %d\n", NbPro);
  #endif

  // Initialisation
  for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
  {
    // (#) = (1 a NbChannel) nombre de canaux
    for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
    {
      // ProtoWork(#,1 a NbPro) = Proto(#,1 a NbPro)
      ProtoWork[IND2(Ich,Ipro,SD_BAS_NBK)]= Proto[IND2(Ich,Ipro,SD_BAS_NBK)];
    }

    // IndexSort(1 a NbPro) = 0
    IndexSort[IND1(Ipro)]= 0;
    IndexSortWork[IND1(Ipro)]= 1;

    // NbPixClasWork(1 a NbPro) = NbPixClas(1 a NbPro)
    NbPixClasWork[IND1(Ipro)]= NbPixClas[IND1(Ipro)];

    // SumDistClasWork(1 a NbPro) = SumDistClas(1 a NbPro)
    SumDistClasWork[IND1(Ipro)]= SumDistClas[IND1(Ipro)];
  }

  #ifdef OPS_DEBUG
    printf( "Variables de travail : ----------------------------------\n");
    for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
    {
      printf( "  ProtoWork(Ich=%d,$) : ", Ich);
      for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
      {
        printf( "%12.5f", ProtoWork[IND2(Ich,Ipro,SD_BAS_NBK)]);
      }
      printf( "\n");
    }

    printf( "      IndexSort : ");
    for( Ipro= 1; Ipro <= NbPro; Ipro++)
      printf( "%12d ", IndexSort[IND1(Ipro)]);
    printf( "\n");

    printf( "  NbPixClasWork : ");
    for( Ipro= 1; Ipro <= NbPro; Ipro++)
      printf( "%12d ", NbPixClasWork[IND1(Ipro)]);
    printf( "\n");

    printf( "SumDistClasWork : ");
    for( Ipro= 1; Ipro <= NbPro; Ipro++)
      printf( "%12.5f ", SumDistClasWork[IND1(Ipro)]);
    printf( "\n");
    printf( "---------------------------------------------------------\n");
  #endif

// OK JUSQU'ICI

  // Boucle sur les prototypes (Ipro = 1 a NbPro)
  for( Ipro= 1; Ipro <= NbPro; Ipro++)
  {
    #ifdef OPS_DEBUG
      printf( "Debut boucle Ipro ==================================\n");
      printf( "Ipro=%d => Id : ", Ipro);
      for( IproTr= 1; IproTr <= SD_BAS_NCL_MAX; IproTr++)
      {
        printf( "%4d ", Id[IND1(IproTr)]);
      }
      printf( "\n");
      printf( "   IndexSort : ");
      for( IproTr= 1; IproTr <= SD_BAS_NCL_MAX; IproTr++)
        printf( "%4d ", IndexSort[IND1(IproTr)]);
      printf( "\n");
    #endif

    // Id(Ipro) = 0
    Id[IND1(Ipro)]= 0;

    // Boucle sur les prototypes (Jpro = 1 a NbPro)
    for( Jpro= 1; Jpro <= NbPro; Jpro++)
    {
      // Si ProtoWork(SortingChannel,Ipro) >= ProtoWork(SortingChannel,Jpro) alors 
      if(    ProtoWork[IND2(SortingChannel,Ipro,SD_BAS_NBK)]
          >= ProtoWork[IND2(SortingChannel,Jpro,SD_BAS_NBK)])
      {
        // Id(Ipro) = Id(Ipro) +1
        Id[IND1(Ipro)]= Id[IND1(Ipro)] + 1;
      } // Fin de si
    }   // Fin de boucle sur les prototypes 

    #ifdef OPS_DEBUG
      printf( "Apres boucle Jpro ==================================\n");
      printf( "Ipro=%d => Id : ", Ipro);
      for( IproTr= 1; IproTr <= SD_BAS_NCL_MAX; IproTr++)
      {
        printf( "%4d ", Id[IND1(IproTr)]);
      }
      printf( "\n");
    #endif

    // Jpro = Id(Ipro)
    Jpro= Id[IND1(Ipro)];

    // Traitement des ex-aequo
    // Tant que IndexSort(Jpro) =/= 0 et Jpro < NbPro 
    while( (IndexSort[IND1(Jpro)] != 0) && (Jpro < NbPro))
    {
      // IndexSort(Jpro) = Ipro
      Jpro = Jpro+1;
    } // Fin de Tant que
    IndexSort[IND1(Jpro)]= Ipro;

    #ifdef OPS_DEBUG
      printf("Jpro:%d\n",Jpro);
      printf( "   IndexSort : ");
      for( IproTr= 1; IproTr <= SD_BAS_NCL_MAX; IproTr++)
        printf( "%4d ", IndexSort[IND1(IproTr)]);
      printf( "\n");
      printf( "Fin   boucle Ipro ==================================\n");
    #endif
  }   // Fin de boucle sur les prototypes

  // Boucle sur les prototypes (Ipro = 1 a NbPro)
  for( Ipro= 1; Ipro <= NbPro; Ipro++)
  {
    // Ind = IndexSort(NbPro -Ipro -1)
    Ind= IndexSort[IND1(NbPro-Ipro+1)];

    // IndexSortWork(Ind) = Ipro
    IndexSortWork[IND1(Ind)]= Ipro;

    // NbPixClas(Ipro) = NbPixClasWork(Ind)
    NbPixClas[IND1(Ipro)]= NbPixClasWork[IND1(Ind)];

    // SumDistClas(Ipro) = SumDistClasWork(Ind)
    SumDistClas[IND1(Ipro)]= SumDistClasWork[IND1(Ind)];

    // Proto(#,Ipro) = ProtoWork(#,Ind)
    //jlp 2905 correction : on travaille sur le nombre de canaux reels
    for( Ich= 1; Ich <= NbChannel; Ich++)
      Proto[IND2(Ich,Ipro,SD_BAS_NBK)]= ProtoWork[IND2(Ich,Ind,SD_BAS_NBK)];
  } // Fin de boucle sur les prototypes

  #ifdef OPS_DEBUG
    printf( "Apres arrangement ==================================\n");
    for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
    {
      printf( "      Proto(Ich=%d,$) : ", Ich);
      for( IproTr= 1; IproTr <= SD_BAS_NCL_MAX; IproTr++)
      {
        printf( "%12.5f", Proto[IND2(Ich,IproTr,SD_BAS_NBK)]);
      }
      printf( "\n");
    }
  #endif

  // IndexSortWork(NbPro+1) = NbPro+1
  IndexSortWork[IND1(NbPro+1)]= NbPro + 1;

  // Boucle sur les pixels (Ipix = 1 a NbPixAv)
  for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
  {
    IdPix[IND1(Ipix)]= IndexSortWork[IND1(IdPix[IND1(Ipix)])];
  }
// VERRUE
  #ifdef OPS_DEBUG
    printf( "Apres boucle sur les pixels ==================================\n");
    for( Ipix= 1; Ipix <= 20 /*NbPixAv*/; Ipix++)
    {
      printf( "IdPix[%d]= %d\n", Ipix, IdPix[IND1(Ipix)]);
    }
  #endif

fin:
  // Fin de la fonction SortClas
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction SortClas\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN SortClas ==================================================================
// ===============================================================================


//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_StatClas
//@
//@ objet :  Calcul du prototype (valeur moyenne intra-classe) et
//@          de la matrice de covariance d'une classe. 
//@
//@ Parametres :
//@         (ENTREE) ProduitCourant : Pour pointer sur les variables temporaires
//@         (ENTREE) ImgAv ........ : Vecteur de radiance de l'image AVHRR
//@         (ENTREE) IdPix ........ : Flags de pixels valides
//@         (ENTREE) NbChannel .... : Nombre de canaux AVHRR a traiter
//@         (ENTREE) NbPixAv ...... : Nombre de pixels du vecteur
//@         (ENTREE) IdPro ........ : Numero du prototype a traiter
//@
//@         (SORTIE) Proto ........ : Prototypes de classes
//@         (SORTIE) Covar ........ : Matrice de covariance
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_StatClas(
  // Entrees
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  float                *ImgAv,              // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
  int                  *IdPix,              // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  int                   NbChannel,          // int(1)
  int                   NbPixAv,            // int(1)
  int                   IdPro,              // int(1)
  // Sorties
  double               *Proto,              // double( SD_BAS_NBK)
  double               *Covar               // double( SD_BAS_NBK * SD_BAS_NBK)
)
{
  static const char tcProcName_F[] = "SD_ALG_StatClas";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  int     Ich, Jch;            // Index sur les canaux
  int     Ipix;                // Index sur les pixels
  double  MoyI, MoyJ, MoyIJ;   // Moyennes
  int     TailleAvhrr;         // Taille d'une image AVHRR
  float  *ImgIJ;               // Vecteur de produit de pixels

  // Gestion de erreurs
  char *msg_err = (char*) NULL;                     // Message d'erreur
  int   num_lig = 0;                     // Ligne d'erreur
  double MoyIJtemp[SD_BAS_NBK*SD_BAS_NBK];
  // Calcul du prototype (valeur moyenne intra-classe) et de la matrice de covariance d'une classe.

  #ifdef CMS_DEBUG
    printf( "Debut de la fonction StatClas\n----------------------------------\n");
  #endif

  TailleAvhrr= SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER;

  ImgIJ= ProduitCourant->ImgIJ;

  // Mise a zero de la matrice de covariance
  for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
  {
    for( Jch= 1; Jch <= SD_BAS_NBK; Jch++)
    {
      Covar[IND2(Jch,Ich,SD_BAS_NBK)]= 0.0;
    }
  }

  // Boucle sur les canaux (Ich = 1 a NbChannel)
  for( Ich= 1; Ich <= NbChannel; Ich++)
  {
    // Calcul de la valeur moyenne et des covariances intra-classe,
    // c'est a dire pour tous les pixels appartenant a la classe donnee 

    // Ind= indices pour lesquels IdPix(Ipix) = IdPro
    MoyI= SD_ALG_Moyenne( &ImgAv[IND2(1,Ich,TailleAvhrr)], NbPixAv, IdPix, IdPro);
    Proto[IND1(Ich)]= MoyI;
  }
  for( Ich= 1; Ich <= NbChannel; Ich++)
  {
     MoyI=Proto[IND1(Ich)];
    // Boucle sur les canaux (Jch = 1 a NbChannel)
    for( Jch= 1; Jch <= NbChannel; Jch++)
    {
//      MoyJ= SD_ALG_Moyenne( &ImgAv[IND2(1,Jch,TailleAvhrr)], NbPixAv, IdPix, IdPro);
//optimisation
      MoyJ=Proto[IND1(Jch)];

      // Produit des 2 vecteurs canaux Ich et Jch
      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
      {
        ImgIJ[IND1(Ipix)]=   ImgAv[IND2(Ipix,Ich,TailleAvhrr)]
                           * ImgAv[IND2(Ipix,Jch,TailleAvhrr)];
      }
     
//optimisation
      if (Jch>=Ich) 
      
      MoyIJtemp[IND2(Ich,Jch,SD_BAS_NBK)]= SD_ALG_Moyenne( ImgIJ, NbPixAv, IdPix, IdPro);
      else
      MoyIJtemp[IND2(Ich,Jch,SD_BAS_NBK)]= MoyIJtemp[IND2(Jch,Ich,SD_BAS_NBK)];
      
      MoyIJ=MoyIJtemp[IND2(Ich,Jch,SD_BAS_NBK)];
     
      // Moyenne du produit / Produit des moyennes
      Covar[IND2(Jch,Ich,SD_BAS_NBK)]= MoyIJ - (MoyI * MoyJ);

    }   // Fin de boucle sur les canaux Jch
  }     // Fin de boucle sur les canaux Ich

fin:
  // Fin de la fonction StatClas
  #ifdef CMS_DEBUG
    printf( "Fin de la fonction StatClas\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN StatClas ==================================================================
// ===============================================================================


//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_Tri_ascendant
//@
//@ objet :   Tri d'un vecteur (Tableau) de reels
//@
//@
//@ Parametres :
//@         (ENTREE) Vecteur ...... : Vecteur de reels
//@         (ENTREE) NbElem ....... : Nombre d'elements du vecteur
//@
//@         (SORTIE) IndicesClasses : Vecteur d'indices classes (selon reels)
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_Tri_ascendant(
     // Entrees
     t_SD_BAS_DataThread* ctx,      // t_SD_BAS_DataThread*
     double *Vecteur,        // double( NbElem)
     int     NbElem,         // int(1)
     // Sorties
     int    *IndicesClasses  // int( NbElem)
)
{
  static const char tcProcName_F[]= "SD_ALG_Tri_ascendant";
  t_CMN__Statut cr_op= CMN__STATUT_OK;

  // Gestion de erreurs
/* bma intialisation */
/*
  char *msg_err;          // Message d'erreur
  int num_lig;            // Ligne d'erreur
*/
  char *msg_err = (char*) NULL;          // Message d'erreur
  int num_lig = 0;            // Ligne d'erreur
/* fin bma */

  int ind;
  type_paire_i_val *paires;

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction Tri_ascendant\n----------------------------------\n");
  #endif

  paires=NULL;
  if( SD_BAS_MacMALLOC( paires, type_paire_i_val, NbElem) == FALSE)
  {
    cr_op=   SD_ALG_AVHRR_ERR_ALLOCATION;
    msg_err= (char *)SD_BAS_MSG_ALLOC;
    num_lig= __LINE__;

    goto fin;
  }

  for( ind= 0; ind < NbElem; ind++)
  {
    paires[ind].i=   ind;
    paires[ind].val= Vecteur[ind];
  }

  #ifdef OPS_DEBUG
    printf( "----------------------------------\n");
    printf( "Indices :\n");
    for( ind= 0; ind < NbElem; ind++)
    {
      printf( "%5d  ", paires[ind].i);
    }
    printf( "\n----------------------------------\n");
    printf( "Valeurs a classer :\n");
    for( ind= 0; ind < NbElem; ind++)
    {
      printf( "%5.1f  ", paires[ind].val);
    }
    printf( "\n----------------------------------\n");
  #endif

  qsort( (void *)paires, NbElem, sizeof( type_paire_i_val), SD_ALG_sort_paire_i_val);

  #ifdef OPS_DEBUG
    printf( "----------------------------------\n");
    printf( "Indices :\n");
    for( ind= 0; ind < NbElem; ind++)
    {
      printf( "%5d  ", paires[ind].i);
    }
    printf( "\n----------------------------------\n");
    printf( "Valeurs classees :\n");
    for( ind= 0; ind < NbElem; ind++)
    {
      printf( "%5.1f  ", paires[ind].val);
    }
    printf( "\n----------------------------------\n");
  #endif

  for( ind= 0; ind < NbElem; ind++)
  {
    IndicesClasses[ind]= paires[ind].i + 1;
  }


fin:
  SD_BAS_MacFREE( paires);
  if ( cr_op != CMN__STATUT_OK )
    {
        // TODO
        printf ("errors \n");
    }
  // Fin de la fonction Tri_ascendant
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction Tri_ascendant\n----------------------------------\n");
  #endif

  return( cr_op);
}
// FIN Tri_ascendant =============================================================


//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_TestQual
//@
//@ objet :   Evaluation de la qualite de la classification
//@
//@
//@ Parametres :
//@         (ENTREE) NbPixAv ............. : Nombre de pixels du vecteur
//@         (ENTREE) Nch ................. : Nombre de canaux presents
//@         (ENTREE) NbClass ............. : Nombre de classes a traiter
//@         (ENTREE) EigVal .............. : Valeurs propres
//@         (ENTREE) EigValCutoff ........ : Seuil de valeur propre
//@         (ENTREE) NbPixClas ........... : Nombre pixels par classe
//@         (ENTREE) SumDistClas ......... : Somme des distances intra-classes
//@
//@         (SORTIE) NonClassifRate ...... : Taux de pixels non classes
//@         (SORTIE) NormVarRate ......... : Rapport : variance image classifiee / image non classifiee normalise
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_TestQual(
  // Entrees
  int     NbPixAv,          // int(1)
  int     Nch,              // int(1)
  int     NbClass,          // int(1)
/* bma complexes->reels */
  double *EigVal,           // double( SD_BAS_NBK)
/* fin bma */
  double  EigValCutoff,     // double(1)
  int    *NbPixClas,        // int( SD_BAS_NCL_MAX)
  double *SumDistClas,      // double( SD_BAS_NCL_MAX)
  // Sorties
  double *NonClassifRate,   // double(1)
  double *NormVarRate       // double(1)
)
{
  // Calcul des parametres qualite de l'analyse des radiances
  static const char tcProcName_F[] = "SD_ALG_TestQual";
  t_CMN__Statut cr_op=CMN__STATUT_OK;

  // Gestion de erreurs
/* bma intialisation */
/*
  char *msg_err;                     // Message d'erreur
  int   num_lig;                     // Ligne d'erreur
*/
  char *msg_err = (char*) NULL;                     // Message d'erreur
  int   num_lig = 0;                     // Ligne d'erreur
/* fin bma */

/* bma complexes->reels */
  double NbEigVal;     // Nombre de valeurs propre > EigValCutoff
  double SumDist;      // Somme des distances de toutes les classes
  int    NbPix;        // Nombre de pixels de toutes les classes
  double VarRate;      // Variance
  int    Ich;          // Index canaux
  int    Iclas;        // Index sur les classes

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction TestQual\n----------------------------------\n");
  #endif

  // Nombre de valeurs hors bruit
  // NbEigVal = Nombre-d'elements{ Find{EigVal(1aNch)/EigVal(1) > EigValCutoff } }

  if( EigVal[IND1(1)] < 1.0E-6)
  {
    cr_op=   SD_ALG_AVHRR_ERR_NUL_EIG_VAL;
    msg_err= (char *)SD_BAS_MSG_ERR_NUL_EIG_VAL;
    num_lig= __LINE__;

    goto fin;
  }

  NbEigVal= 0.0;
  for( Ich= 1; Ich <= Nch; Ich++)
  {
    if( (EigVal[IND1(Ich)] / EigVal[IND1(1)]) > EigValCutoff)
      NbEigVal= NbEigVal + 1.0;
  }
/* fin bma */

  // Proportion du nombre de pixels non classes
  // NonClassifRate = NbPixClas(NbClass+1) / NbPixAv
  *NonClassifRate= ((double)NbPixClas[IND1(NbClass+1)]) / ((double)NbPixAv);

  // Rapport entre la variance de l'image classee et celle de l'image non classee
  // VarRate = somme(...)
  SumDist= 0.0;
  NbPix= 0;
  for( Iclas= 1; Iclas <= NbClass; Iclas++)
  {
    SumDist= SumDist + SumDistClas[IND1(Iclas)];
    NbPix=   NbPix   + NbPixClas[IND1(Iclas)];
  }
  VarRate= SumDist / (double)(Nch * NbPix);

  // Rapport des variances normalise
  // NormVarRate = VarRate.NbClass^(2/NbEigVal)
  *NormVarRate= VarRate * pow( (double)NbClass, 2.0 / NbEigVal);

fin:
  // Fin de la fonction TestQual
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction TestQual\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN TestQual ==================================================================
// ===============================================================================

//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_EigenAnalysis
//@
//@ objet :   Calcul et classement des valeurs et vecteurs propres
//@
//@
//@ Parametres :
//@         (ENTREE) Contexte_courant .... : Contexte de l'etape (numero de ligne SN traite)
//@         (ENTREE) Taille .............. : Taille de la matrice carree
//@         (ENTREE) Covar ............... : Matrice de covariance
//@         (ENTREE) Transpose ........... : Flag de transposition de la covariance puis des vecteurs
//@
//@         (SORTIE) EigVal .............. : Valeurs propres ordonnees
//@         (SORTIE) EigVect ............. : Vecteurs propres ordonnes
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_EigenAnalysis(
  // Entrees
  t_SD_BAS_DataThread *Contexte_courant,  // t_SD_BAS_DataThread*
  int                  Taille,            // int(1) Nombre de points (Max = SD_BAS_NBK)
  double              *Covar,             // double( Taille * Taille),
  unsigned int         Transpose,         // unsigned int(1)                  
  // Sorties
/* bma complexes->reels */
  double              *EigVal,            // double( Taille)
  double              *EigVect            // double( Taille * Taille)
/* fin bma */
)
{
  static const char tcProcName_F[] = "SD_ALG_EigenAnalysis";
  t_CMN__Statut cr_op=CMN__STATUT_OK;

  // Gestion de erreurs
/* bma intialisation */
/*
  char            *msg_err;                     // Message d'erreur
  int              num_lig;                     // Ligne d'erreur
*/
  char            *msg_err = (char*) NULL;                     // Message d'erreur
  int              num_lig = 0;                     // Ligne d'erreur
/* fin bma */

  double           auxv[2*SD_BAS_NBK];          // 2*Taille (Cf. Doc ESSL)
  double           tmp;
/* bma complexes->reels */
  dcmplx           CEigVal[SD_BAS_NBK];         // dcmplx( Taille)
  dcmplx           CEigVect[SD_BAS_NBK*SD_BAS_NBK];                    // dcmplx( Taille * Taille)
/* fin bma */
  type_sort_eigen  ValVect[SD_BAS_NBK];         // Structures de tri
  int              i, j;                        // Index generaux

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction EigenAnalysis\n----------------------------------\n");

    // ================================================================================
    // Trace des entrees ==============================================================
    // ================================================================================
    printf( "Taille= %d\n", Taille);
    printf( "Matrice de covariance :\n");
    for( i= 0; i < Taille; i++)
    {
      printf( "|   ");
      for( j= 0; j < Taille; j++)
      {
        printf( "%5.1f   ", Covar[IND2(j+1,i+1,Taille)]);
      }
      printf( "|\n");
    }
  #endif

  if( Transpose == CMN__OUI)
  {
    // La matrice est donnee lig1,col1 ; lig1,col2 ; ... ; lig1,colN
    //                       lig2,col1 ; lig2,col2 ; ... ; lig2,colN
    //                          ...         ...      ...      ...
    //                       ligM,col1 ; ligM,col2 ; ... ; ligM,colN
    // => Transposition de la matrice pour etre en accord avec ESSL
    for( i= 1; i < Taille; i++)  // Lignes
    {
      for( j= 0; j < i; j++)     // Colonnes
      {
        tmp= Covar[i*Taille+j];
        Covar[i*Taille+j]= Covar[j*Taille+i];
        Covar[j*Taille+i]= tmp;
      }
    }
  }

  // Calcul des valeurs et vecteurs propres
  cr_op= SD_BAS_ValVectPropre(
        // Entrees
        Contexte_courant,   // t_SD_BAS_DataThread*
        Taille,             // int(1) Nombre de points
        Covar,              // double( SD_BAS_NBK * SD_BAS_NBK),
        // Sorties
        auxv,               // double( 2*SD_BAS_NBK)
/* bma complexes->reels */
        CEigVal,            // dcmplx( SD_BAS_NBK)
        CEigVect);          // dcmplx( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */

  if( cr_op != CMN__STATUT_OK)
  {
    cr_op=   SD_ALG_AVHRR_ERR_EIG_VAL_PROCESSING;
    msg_err= (char *)SD_BAS_MSG_ERR_EIG_VAL_PROCESSING;
    num_lig= __LINE__;
    goto fin;
  }

/* bma complexes->reels */
  for( i= 0; i < Taille; i++)
  {
    EigVal[i]= RE( CEigVal[i]);
    for( j= 0; j < Taille; j++)
    {
      /*MOR suppression fabs*/
      EigVect[i*Taille+j]=  RE( CEigVect[i*Taille+j]);
    }
  }

//jlp30 inutile ici de transposer on stocke la matrice en colonne
//  if( Transpose == CMN__OUI)
//  {
//    // Transposition des vecteurs propres
//    for( i= 1; i < Taille; i++)  // Lignes
//    {
//      for( j= 0; j < i; j++)     // Colonnes
//      {
//        tmp= EigVect[i*Taille+j];
//        EigVect[i*Taille+j]= EigVect[j*Taille+i];
//        EigVect[j*Taille+i]= tmp;
//      }
//   }
//  }
/* fin bma */

  #ifdef OPS_DEBUG
    // ================================================================================
    // Trace des sorties des valeurs propres ==========================================
    // ================================================================================
    printf( "Valeurs propres :\n");
    printf( "|   ");
    for( i= 0; i < Taille; i++)
    {
/* bma complexes->reels */
      printf( "%9.6f   ", EigVal[i]);
/* fin bma */
    }
    printf( "|\n");

    printf( "Vecteurs propres :\n");
    for( j= 0; j < Taille; j++)
    {
      printf( "|   ");
      for( i= 0; i < Taille; i++)
      {
/* bma complexes->reels */
        printf( "%9.6f   ", EigVect[IND2(j+1,i+1,Taille)]);
/* fin bma */
      }
      printf( "|\n");
    }
  #endif

  // Classement des valeurs et vecteurs propres

/* bma complexes->reels */
  // Initialisation de la structure (toutes les valeurs propres au max)
  for( i= 0; i < SD_BAS_NBK; i++)   ValVect[i].val= MAX_DOUBLE;

  // Chargement de la structure
//la matrice en sortie est stockee vecteur propre par vecteur propre
  for( j= 0; j < Taille; j++)
  {
    ValVect[j].val= EigVal[j];
    for( i= 0; i < Taille; i++)
    {
      ValVect[j].vect[i]= EigVect[j*Taille+i];
    }
  }
/* fin bma */

  // Classement
  qsort( (void *)ValVect, Taille, sizeof( type_sort_eigen), SD_ALG_sort_eigen_val);

  // Recuperation des valeurs classees
/* bma complexes->reels */
//la matrice en sortie doit  stocker les vecteurs propre en colonne
  for( j= 0; j < Taille; j++)
  {
    EigVal[j]= ValVect[j].val;
    for( i= 0; i < Taille; i++)
    {
      // rangement ( en colonne)
      /* FA135 : correction transposition de la matrice des valeurs propres */
      EigVect[i*Taille+j]= ValVect[j].vect[i];
    }
  }
/* fin bma */
//RMO VERRUE A ENLEVER ( elle sert a remettre les vecteurs propre avec le bon signe
//  for(j= 0; j < Taille; j++)
//    if(j != 2)
//      for( i= 0; i < Taille; i++)
//	EigVect[j*Taille+i]=EigVect[j*Taille+i]*(-1.0);

  #ifdef OPS_DEBUG
    // ================================================================================
    // Trace des sorties des valeurs propres ==========================================
    // ================================================================================
//RMO  verrue
   printf( "Valeurs propres clasees :\n");
    printf( "|   ");
    for( i= 0; i < Taille; i++)
    {
/* bma complexes->reels */
      printf( "%9.6f   ", EigVal[i]);
/* fin bma */
    }
    printf( "|\n");

    printf( "Vecteurs propres classes :\n");
    for( j= 0; j < Taille; j++)
    {
      printf( "|   ");
      for( i= 0; i < Taille; i++)
      {
/* bma complexes->reels */
        printf( "%9.6f   ", EigVect[IND2(j+1,i+1,Taille)]);
/* fin bma */
      }
      printf( "|\n");
    }
  #endif

fin:
  // Fin de la fonction EigenAnalysis
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction EigenAnalysis\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN EigenAnalysis =============================================================
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_sort_eigen_val
//@
//@ Description : Tri croissant de valeur/vecteurs propres
//@               en fonction des valeurs propres
//@ ------------------------------------------------------------------*/
int SD_ALG_sort_eigen_val( const void *a, const void *b)
{ int res;

/* bma complexes->reels */
  res= ((const type_sort_eigen *)a)->val < ((const type_sort_eigen *)b)->val;
/* fin bma */
  if( res == 0)  res= -1;
  return( res);
}

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_sort_paire_i_val
//@
//@ Description : Tri croissant de paires valeur/indice sur la valeur
//@ But : obtenir la liste des indices correspondant aux valeurs croissantes
//@ ------------------------------------------------------------------*/
int SD_ALG_sort_paire_i_val( const void *a, const void *b)
{ int res;

  res= ((const type_paire_i_val *)a)->val > ((const type_paire_i_val *)b)->val;
  if( res == 0)  res= -1;
  return( res);
}

//@ ------------------------------------------------------------------*/
//@ Function : SD_ALG_Moyenne
//@
//@ objet :  Moyenne des pixels de ImgAv de classe IdPro
//@
//@
//@ Parametres :
//@         (ENTREE) ImgAv ...... : Vecteur de radiance de l'image AVHRR
//@         (ENTREE) NbPixAv .... : Nombre de pixels du vecteur
//@         (ENTREE) IdPix ...... : Flags de pixels valides
//@         (ENTREE) IdPro ...... : Numero du prototype a traiter
//@
//@ ------------------------------------------------------------------*/
double SD_ALG_Moyenne(
       // Entrees
       float *ImgAv,     // float( NbPixAv)
       int    NbPixAv,   // int(1)
       int   *IdPix,     // int( NbPixAv)
       int    IdPro      // int(1)
)
{
  static const char tcProcName_F[]= "SD_ALG_Moyenne";
  double Moy= 0.0;
  int    NbPix= 0;
  int    i;

  #ifdef OPS_DEBUG2
    printf( "Debut de la fonction Moyenne\n----------------------------------\n");
  #endif

  for( i= 1; i <= NbPixAv; i++)
  {
    if( IdPix[IND1(i)] == IdPro)
    {
      Moy= Moy + (double)ImgAv[IND1(i)];
      NbPix++;
    }
  }

  if( NbPix > 0)  Moy= Moy / (double)NbPix;
  #ifdef XCMS_DEBUG
   printf( "Moyenne IdPro=%d\n",IdPro);
   printf( "Moy=%e NbPix=%d \n",Moy, NbPix);
  #endif
  // Fin de la fonction Moyenne
  #ifdef OPS_DEBUG2
    printf( "Fin de la fonction Moyenne\n----------------------------------\n");
  #endif

  return( Moy);
}
// FIN Moyenne ===================================================================


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_ValVectPropre
//@
//@ Description : calcul des valeurs propres et des vecteurs propres
//@ d'une matrice carree
//@
//@
//@ Parameters :
//@         (ENTREE)  nbpts : nombre de pts
//@         (ENTREE)  a : matrice ( overwritten ESSL & LAPACK ) // FT-2765
//@         (SORTIE)  auxv : tableau temporaire
//@         (SORTIE) valpropre : valeurs propres
//@         (SORTIE) vecpropre : vecteurs propres
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_ValVectPropre(
        t_SD_BAS_DataThread * ctx,
        int   nbpts,
        double * a,
        double * auxv,
        dcmplx * valpropre,
        dcmplx * vecpropre
        )
{
static const char tcProcName_F[] = "SD_BAS_ValVectPropre";
t_CMN__Statut cr_op=CMN__STATUT_OK;

#ifdef OPS_USE_ESSL
//@ 1- appel fonction ESSL DGEEV
//@ iopt = 1
//@ select = dummy
//@ a = matrice
//@ lda = nbpts
//@ ldz = nbpts
//@ n = nbpts
//@ aux = auxv
//@ naux = 2*nbpts
//@ w = valpropre
//@ ldz = nbpts
//@ z = vecpropre

    int irc;            // Status de retour

    int iopt;
    int lda;            // Taille de la matrice a;
    int ldz;            // Nombre mas de valeurs propres
    int select;         // Pas utilise si iopt = 1
    int naux;           // Taille de l'espace tampon

    errset(&ierno,&inoal,&inomes,&itrace,&iusadr,&irange);

    // Initialisations relatives a DGEEV
    iopt = 1;           // Pour calculer les valeurs propres et vecteur propres
    lda = nbpts;
    ldz = nbpts;
    naux = 2*nbpts;     // Cf recommandations ESSL

    // Appel de la fonction ESSL DGEEV
    irc = dgeev(iopt,a,lda,valpropre,vecpropre,ldz,&select,nbpts,auxv,&naux);

    // En cas d'erreur de calcul dans la fonction DGEEV
    if(irc != 0)
    {
        cr_op = CMN__STATUT_NOK;

        //@ on recupere le premier message d'erreur pour l'emission d'une trace
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_ESSL,
                            SD_BAS_LibESSL_ERR_CONVERG,tcProcName_F,__LINE__,ERREUR_CONVERGENCE);
        //@ on imprime les message d'erreurs sur la sortie standard
        #ifdef DEBUG
            SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__, ERREUR_CONVERGENCE);
        #endif

    }


// sortie de la fonction
return(cr_op);
#else
 int irc;            // Status de retour
 irc = 0;
 int n;
 int lda;
 double *wr = NULL, *wi = NULL; // real & imag parts of eigenvalues
 double *vr = NULL;             // eigenvectors ( to be recombined )
 int lwork;
 double *work = NULL;
 double dlwork;
 int ldvr;
 int info;
 int i,j;

 n = nbpts;
 lda = nbpts;
 ldvr = n;

#ifdef ESSL_TEST
     ESSL_TEST_BEGIN( SD_BAS_ValVectPropre );
     fwrite( &nbpts, sizeof( nbpts ), 1, fp );
     fwrite( a, sizeof( double ), nbpts * nbpts, fp );
     ESSL_TEST_END;
#endif
  lwork = -1; info = 0; // query LAPACK
  lapack_dgeev( "N", "V", n, a, lda, wr, wi, NULL, 1, vr, ldvr, &dlwork, lwork, &info );
  lwork = dlwork;

 if(  SD_BAS_MacMALLOC( work,  double, lwork ) == FALSE
   || SD_BAS_MacMALLOC( vr,    double, n * n ) == FALSE
   || SD_BAS_MacMALLOC( wr,    double, n     ) == FALSE
   || SD_BAS_MacMALLOC( wi,    double, n     ) == FALSE
   ) {
     cr_op   = SD_BAS_LibESSL_ERR_ALLOCATION;
     
     return cr_op;
 }

 irc = 0;
 lapack_dgeev( "N", "V", n, a, lda, wr, wi, NULL, 1, vr, ldvr, work, lwork, &info );

  
 if( info != 0 ) {
   irc = 1;
 }

 for( i = 0; i < n; i++ ) {
   valpropre[i]._data._re = wr[i];
   valpropre[i]._data._im = wi[i];
 }

 for( i = 0; i < n; i++ ) {
   if( wi[i] != (double)0 ) {
     for( j = 0; j < n; j++ ) {
       vecpropre[ n * i         + j ]._data._re = + vr[ n * i         + j ];
       vecpropre[ n * i         + j ]._data._im = + vr[ n * ( i + 1 ) + j ];
       vecpropre[ n * ( i + 1 ) + j ]._data._re = + vr[ n * i         + j ];
       vecpropre[ n * ( i + 1 ) + j ]._data._im = - vr[ n * ( i + 1 ) + j ];
     }
     i++;
   } else {
     for( j = 0; j < n; j++ ) {
       vecpropre[ n * i + j ]._data._re = + vr[ n * i + j ];
       vecpropre[ n * i + j ]._data._im = 0;
     }
   }
 }

 SD_BAS_MacFREE( work );
 SD_BAS_MacFREE( vr );
 SD_BAS_MacFREE( wr );
 SD_BAS_MacFREE( wi );



fin:
 if( irc != 0 ) {
     cr_op = CMN__STATUT_NOK;

     //TODO
     printf ("errors\n");
 }

 return cr_op;
#endif
}
