/*
! Name: %Id%
! Purpose: Call clusters
!
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    30/08/2018  New. P Roquet


*/

#define NB_PIX_AVHRR 2048
#define NB_CHAN_AVHRR 5
#define CHAN_3B 2
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <Nuees.h>
#include <NueesParam.h>
#include <strings.h>
#include <clusters.h>
typedef struct {
  t_SD_BAS_DataThreadProduit  *ProduitCourant;     // t_SD_BAS_DataThreadProduit*
  t_SD_BAS_DataThread  *ContexteCourant;   // t_SD_BAS_DataThread*
  float        *RadAv;               // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
                                     // Considere comme NBK vecteurs de NPI_AVHRR * NLI_MAX_UTIL_AVHRR valeurs
  int           NbPixAv;             // int(1)
  int           Nch;                 // int(1)
  int           SortingChannel;      // int(1)
  double       *ChannelNoise;        // double( SD_BAS_NBK)
  int          *ChannelWgt;          // int( SD_BAS_NBK)
  int           NbClusMax;           // int(1)
  int           NbIterMax;           // int(1)
  double        IterConvCutoff;      // double(1)
  double        SplitCutoff;         // double(1)
  double        GlueCutoff;         // double(1)
  double        EigValCutoff;        // double(1)
  double        DistAgloInit;        // double(1)
  int           MinIterNCCutOff;     // int(1)
  int           NbNonClassifMax;     // int(1)
  double        DistMax;             // double(1)
  // Sorties
  int          *IdPix;               // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  unsigned int *NoExt;               // unsigned int( SD_BAS_NCL_MAX)
  int          *NbClass;            // int(1)
  unsigned int *FlagPostProcessing;  // unsigned int(1)
  double       *NonClassifRate;      // double(1)
  double       *VarianceRate;         // double(1)
  int          cr; //exit code

}  NueesArgsStruct;

void * fct_nuees_thread (void * arg) 
{
  NueesArgsStruct *params = arg;
  int cr;
  int k;
  int c;
  printf("call SD_ALG_NueesDynamiques\n");
  printf("Input RadVect:\n");
  printf("Nch=%d\n",params->Nch);
   for (k=1; k<10; k++){
    for (c=1; c<= params->Nch; c++){
     printf("%e ", params->RadAv[IND2(k,c,SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)]);
     }
     printf("\n");
   }
  cr = SD_ALG_NueesDynamiques(params->ProduitCourant,
                              params->ContexteCourant,
                              params->RadAv,
                              params->NbPixAv,             // int(1)
                              params->Nch,                 // int(1)
                              params->SortingChannel,      // int(1)
                              params->ChannelNoise,        // double( SD_BAS_NBK)
                              params->ChannelWgt,          // int( SD_BAS_NBK)
                              params->NbClusMax,           // int(1)
                              params->NbIterMax,           // int(1)
                              params->IterConvCutoff,      // double(1)
                              params->SplitCutoff,         // double(1)
                              params->GlueCutoff,          // double(1)
                              params->EigValCutoff,        // double(1)
                              params->DistAgloInit,        // double(1)
                              params->MinIterNCCutOff,     // int(1)
                              params->NbNonClassifMax,     // int(1)
                              params->DistMax,             // double(1)
  // Sorties
                params->IdPix,               // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
                params->NoExt,               // unsigned int( SD_BAS_NCL_MAX)
                params->NbClass,             // int(1)
                params->FlagPostProcessing,  // unsigned int(1)
                params->NonClassifRate,      // double(1)
                params->VarianceRate         // double(1)
  );
  params->cr=cr;
  printf("SD_ALG_NueesDynamiques return %d\n",cr);
  return NULL;
}
int initDataThreadProduit(t_SD_BAS_DataThreadProduit  * ProduitCourant)
{	
/*	
  int           *IdPix;                 // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  float         *RadNW;                 // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
  double        *Dist;                  // double( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER * SD_BAS_NCL_MAX)
  float         *RadN;                  // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
  unsigned int  *UsedPixTest;           // unsigned int(SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  double        *DistZero;              // double( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  double        *Dist1;                 // double( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  int           *IdPixAglo;             // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  float         *ImgIJ;                 // float( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
*/
      int cr = 0;
      int dataSize;
 
	dataSize = SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER;
	if ( (ProduitCourant-> IdPix = calloc (  dataSize     ,sizeof(int))) == NULL ) return 1;
	dataSize =(SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK;
	if ( (ProduitCourant->RadNW = calloc (  dataSize     ,sizeof(float))) == NULL ) return 1;
	dataSize =  SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER * SD_BAS_NCL_MAX;
	if ( (ProduitCourant->Dist = calloc (  dataSize     ,sizeof(double))) == NULL ) return 1;
	dataSize = ( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK;
	if ( (ProduitCourant->RadN = calloc (  dataSize     ,sizeof(float))) == NULL ) return 1;
	dataSize = SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER;
	if ( (ProduitCourant->UsedPixTest= calloc (  dataSize     ,sizeof(unsigned int))) == NULL ) return 1;
	if ( (ProduitCourant->DistZero = calloc (  dataSize     ,sizeof(double))) == NULL ) return 1;
	if ( (ProduitCourant->Dist1 = calloc (  dataSize     ,sizeof(double))) == NULL ) return 1;
	if ( (ProduitCourant->IdPixAglo = calloc (  dataSize     ,sizeof(int))) == NULL ) return 1;
	if ( (ProduitCourant->ImgIJ = calloc (  dataSize   ,sizeof(float)))  == NULL ) return 1;
      
	return cr;
}
void freeDataThreadProduit(t_SD_BAS_DataThreadProduit  * ProduitCourant)
{	
/*	
  int           *IdPix;                 // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  float         *RadNW;                 // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
  double        *Dist;                  // double( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER * SD_BAS_NCL_MAX)
  float         *RadN;                  // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
  unsigned int  *UsedPixTest;           // unsigned int(SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  double        *DistZero;              // double( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  double        *Dist1;                 // double( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  int           *IdPixAglo;             // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
  float         *ImgIJ;                 // float( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
*/
      
	free (ProduitCourant-> IdPix) ;
	free (ProduitCourant->RadNW) ;
	free (ProduitCourant->Dist) ;
	free (ProduitCourant->RadN);
	free(ProduitCourant->UsedPixTest);
	free(ProduitCourant->DistZero);
	free(ProduitCourant->Dist1);
	free(ProduitCourant->IdPixAglo);
	free(ProduitCourant->ImgIJ);
}
int initDataThread(t_SD_BAS_DataThread *  dataThread, int nb_threads) {
/*
typedef struct
    {
    t_SD_BAS_ORIGINE ctxtrt; // contexte du traitement ( SN,PN ...);
    t_SD_BAS_CompteRendu cr;
    t_SD_BAS_MetopMSG  * ptmsg;
    } t_SD_BAS_DataThread;
*/
    if ( (dataThread= calloc(nb_threads, sizeof(t_SD_BAS_DataThread))) == NULL) return 1; 
    return 0;

}



/// input float * RadVec  contains vectors of radiances (SI units) size: (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK 
///                Units must be : W/(m2.sr) or W/m2.st.m-1 for visible channels, W/m2.st.m-1 for IR channels
///              * nbPixelImg   number of pixel for each vectors of radiances    (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK 
///                nb_cases  ; number of cases
///                nb_threads : number of threads  
/// output int * PixId : contains clusters (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) 
///               nbClass : for each case contains the number of Class 
void clustersnothread( float *RadVect, int * NbPixImg, int * Nch, int * SortCh, double * Noise,int * Weight, int *IdPix ,int * NbClass, int * ErrCode)
// for fortran
{

  t_CMN__Statut cr_op= CMN__STATUT_OK;
  t_SD_BAS_DataThreadProduit ProduitCourant; 
  t_SD_BAS_DataThread ContexteCourant;   
  NueesArgsStruct NueesArgs[1];

  // Input Nuee


  double        ChNoise[SD_BAS_NBK];      // double( SD_BAS_NBK)
  int           ChWgt[SD_BAS_NBK];        // int( SD_BAS_NBK)
  int t = 0;
  
  // Output Nuee

  unsigned int * RadAnalType; // unsigned int( SD_BAS_NCL_MAX) Flags de classes non extremes

  unsigned int  FlagPostProcessing;
  double  NonClassifRate;
  double  VarianceRate;
  int dataSize;
  int k;
  
  dataSize =   (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK  ;

   // initialisation ProduitCourant et ContextCourant	
     if (  initDataThreadProduit(&ProduitCourant) != 0){ 
	 fprintf(stderr,SD_BAS_MSG_ALLOC);
      }

   // Calloc output Nuees :

     if ((RadAnalType = calloc(SD_BAS_NCL_MAX,sizeof(unsigned int))) == NULL )      { 
		 fprintf(stderr,SD_BAS_MSG_ALLOC);
      }
  
   printf ("Nch :  %d\n",*Nch);
   printf ("SortCh :  %d\n",*SortCh);
   printf ("CCSNbClusMax  : %d\n",CCSNbClusMax);
   printf ("CCSNbIterMax  : %d\n",CCSNbIterMax);
   printf ("CCSIterConvCutoff : %f\n",CCSIterConvCutoff);
   printf ("CCSSplitCutoff : %f\n",CCSSplitCutoff );
   printf ("CCSGlueCutoff  : %f\n",CCSGlueCutoff );
   printf ("EigValCutoff : %f\n",EigValCutoff);
   printf ("CCSDistAgloInit : %f\n",CCSDistAgloInit);
   printf ("CCSMinIterNCCutoff: %d\n",CCSMinIterNCCutoff);
   printf ("CCSNbNonClassifMax : %d\n",CCSNbNonClassifMax);
   printf ("CCSDistMax : %f\n",CCSDistMax);
   
   for (k =0; k< *Nch; k++){
    ChNoise[k] = Noise[k];
    printf("ChNoise[%d] = %e\n",k, ChNoise[k]);
    }

   for (k=0; k< *Nch; k++){
    ChWgt[k] = Weight[k];
    printf("ChWgt[%d] = %d\n",k, ChWgt[k]);
    }

   printf("nb pixels image: %d\n", *NbPixImg);

      // create thread nuees 
      NueesArgs[t].ProduitCourant = &ProduitCourant;
      NueesArgs[t].ContexteCourant = &ContexteCourant;
      NueesArgs[t].RadAv = RadVect;
      NueesArgs[t].NbPixAv = *NbPixImg;
      NueesArgs[t].Nch = *Nch;
      NueesArgs[t].SortingChannel= *SortCh;      // int(1)
      NueesArgs[t].ChannelNoise=   ChNoise;      // double( SD_BAS_NBK)
      NueesArgs[t].ChannelWgt = ChWgt   ;       // int( SD_BAS_NBK)
      NueesArgs[t].NbClusMax= CCSNbClusMax ;         // int(1)
      NueesArgs[t].NbIterMax=    CCSNbIterMax   ;    // int(1)
      NueesArgs[t].IterConvCutoff= CCSIterConvCutoff ;    // double(1)
      NueesArgs[t].SplitCutoff=   CCSSplitCutoff ;     // double(1)
      NueesArgs[t].GlueCutoff=   CCSGlueCutoff   ;   // double(1)
      NueesArgs[t].EigValCutoff=   EigValCutoff  ;   // double(1)
      NueesArgs[t].DistAgloInit=   CCSDistAgloInit   ;  // double(1)
      NueesArgs[t].MinIterNCCutOff=  CCSMinIterNCCutoff  ; // int(1)
      NueesArgs[t].NbNonClassifMax=   CCSNbNonClassifMax ; // int(1)
      NueesArgs[t].DistMax=      CCSDistMax    ;   // double(1)
  // Sorties
      NueesArgs[t].IdPix = IdPix;               // int( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER)
      NueesArgs[t].NoExt= RadAnalType  ;            // unsigned int( SD_BAS_NCL_MAX)
      NueesArgs[t].NbClass=   NbClass ;        // int(1)
      NueesArgs[t].FlagPostProcessing= &FlagPostProcessing ;// unsigned int(1)
      NueesArgs[t].NonClassifRate=  &NonClassifRate ;   // double(1)
      NueesArgs[t].VarianceRate= &VarianceRate ;     // double(1)

      fct_nuees_thread( (void *) &NueesArgs[t])  ;
        
/*
      cr_op= SD_ALG_NueesDynamiques(
           // Input
            &ProduitCourant[t],     // t_SD_BAS_DataThreadProduit*
            &ContexteCourant[t],   // t_SD_BAS_DataThread*
            RadVect[c],            // float( (SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK)
            NbPixImg[c],           // int(1)
            Nch,                // int(1)
            SortCh,             // int(1)
            ChNoise,            // double( SD_BAS_NBK)
            ChWgt,              // int( SD_BAS_NBK)
            CCSNbClusMax,       // int(1)
            CCSNbIterMax,       // int(1)
            CCSIterConvCutoff,  // double(1)
            CCSSplitCutoff,     // double(1)
            CCSGlueCutoff,      // double(1)
            EigValCutoff,       // double(1)
            CCSDistAgloInit,    // double(1)
            CCSMinIterNCCutoff, // int(1)
            CCSNbNonClassifMax, // int(1)
            CCSDistMax,         // double(1)
           // Output
            IdPix[c],
            RadAnalType,  // unsigned int( SD_BAS_NCL_MAX)
            &NbClass[c],                        // int(1)
            &FlagPostProcessing,                    // unsigned int(1)
            &NonClassifRate,                        // double(1)
            &VarianceRate);  
*/

     printf("fct_nuees_thread return code :%d\n", NueesArgs[t].cr);
     *ErrCode= NueesArgs[t].cr;     
     freeDataThreadProduit( &ProduitCourant);

}
