        Subroutine avhrr_resolution(
     >         satid_sat, satid_inst,
     >         ipixel, iline,
     >         delta_pix, delta_lin,
     >         reftime,  
     >         att_mode, att, 
     >         km_per_pix, km_per_lin,
     >         ierr)
!+
!NOM/NAME $Id: avhrr_resolution.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!
!DESCRIPTION 
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 2003       no          A. Marsouin P.Brunel
! 1.1   15 Jul 2004   Remove multiple declarations of ierr  N C Atkinson
!          
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        Implicit None                                                     
!
!ARGUMENTS

!input 
        Integer satid_sat               ! satellite index in satid file
        Integer satid_inst              ! instrument index for satid
        Integer iline                   ! line number 
        Integer ipixel                  ! pixel number
        Integer delta_pix               ! 
        Integer delta_lin  
        Integer att_mode                ! attitude mode
        Double Precision reftime        ! julian instant of the first line (reference)
        Double Precision att(3,3)       ! attitude error

! output
        real km_per_lin                 ! taille en km entre 2 lignes AVHRR
        real km_per_pix                 ! taille en km entre 2 pixels AVHRR
        Integer ierr                    ! return code 

!DECLARATIONS
!
!
!
! Local variables       (distances in km, angles in radians)
        Integer orbnum          ! orbit number
        Real pixel
        Real line
        Double Precision Mat_Ra_Rg(3,3) ! Ra to Rg conversion matrix
        Double Precision sm_rs(3)       ! viewing vector for optical axis
        Double Precision om_g1(3)       ! viewed point in Greenwich ref. frame
        Double Precision om_g2(3)       ! viewed point in Greenwich ref. frame
        Double Precision om_g3(3)       ! viewed point in Greenwich ref. frame


        Double Precision time           ! julian instant
        Double Precision scnang         ! viewed points scanning angles (rad)
        Double Precision pos(3)         ! Greenwich satellite position  (km)
        Double Precision vel(3)         ! Greenwich satellite velocity  (km/s)
        double precision x, y, z
        Character*22 cht                ! date-time string
!-
!
        pixel = ipixel
        line  = iline

        ! 1. calculates viewing vectors
        !------------------------------
        Call lptoviewvect(
     &        pixel, line, satid_sat, satid_inst, reftime,
     &        time, sm_rs, scnang)

        ! 2. calculates position velocity
        !--------------------------------------------
        Call intposvel(1,time,pos,vel,orbnum,ierr)
        If( ierr .Ne. 0 ) Then
           Call xijc(time,cht)
           Call ml_wti('error','intposvel return code',ierr)
           Call ml_wt('error','intposvel input time '//cht)
           Return
        End If

        ! 3. calculates the Rv <--> Rg transformation
        !--------------------------------------------
        Call snagre(att_mode, pos, vel, Mat_Ra_Rg, ierr)
        If( ierr .Ne. 0 ) Then
           Call ml_wti('error','snagre return code',ierr)
           Call ml_wti('error','snagre input att mode ',att_mode)
           Return
        Endif
        
        ! 4. calculates the optical axis coordinates in Rg
        !-------------------------------------------------
        Call earthpix( pos, att, Mat_Ra_Rg, sm_rs, om_g1, ierr)
        If( ierr .Ne. 0 ) Then
           Call ml_wti('error','earthpix return code',ierr)
           Return
        End If

          

        ! + delta_Pix
        pixel = ipixel + delta_pix
        line  = iline
        ! 1. calculates viewing vectors
        !------------------------------
        Call lptoviewvect(
     &        pixel, line, satid_sat, satid_inst, reftime,
     &        time, sm_rs, scnang)

        ! 2. calculates position velocity
        !--------------------------------------------
        Call intposvel(1,time,pos,vel,orbnum,ierr)
        If( ierr .Ne. 0 ) Then
           Call xijc(time,cht)
           Call ml_wti('error','intposvel return code',ierr)
           Call ml_wt('error','intposvel input time '//cht)
           Return
        End If

        ! 3. calculates the Rv <--> Rg transformation
        !--------------------------------------------
        Call snagre(att_mode, pos, vel, Mat_Ra_Rg, ierr)
        If( ierr .Ne. 0 ) Then
           Call ml_wti('error','snagre return code',ierr)
           Call ml_wti('error','snagre input att mode ',att_mode)
           Return
        Endif
        
        ! 4. calculates the optical axis coordinates in Rg
        !-------------------------------------------------
        Call earthpix( pos, att, Mat_Ra_Rg, sm_rs, om_g2, ierr)
        If( ierr .Ne. 0 ) Then
           Call ml_wti('error','earthpix return code',ierr)
           Return
        End If



        ! + delta_lin
        pixel = ipixel
        line  = iline + delta_lin
        ! 1. calculates viewing vectors
        !------------------------------
        Call lptoviewvect(
     &        pixel, line, satid_sat, satid_inst, reftime,
     &        time, sm_rs, scnang)

        ! 2. calculates position velocity
        !--------------------------------------------
        Call intposvel(1,time,pos,vel,orbnum,ierr)
        If( ierr .Ne. 0 ) Then
           Call xijc(time,cht)
           Call ml_wti('error','intposvel return code',ierr)
           Call ml_wt('error','intposvel input time '//cht)
           Return
        End If

        ! 3. calculates the Rv <--> Rg transformation
        !--------------------------------------------
        Call snagre(att_mode, pos, vel, Mat_Ra_Rg, ierr)
        If( ierr .Ne. 0 ) Then
           Call ml_wti('error','snagre return code',ierr)
           Call ml_wti('error','snagre input att mode ',att_mode)
           Return
        Endif
        
        ! 4. calculates the optical axis coordinates in Rg
        !-------------------------------------------------
        Call earthpix( pos, att, Mat_Ra_Rg, sm_rs, om_g3, ierr)
        If( ierr .Ne. 0 ) Then
           Call ml_wti('error','earthpix return code',ierr)
           Return
        End If


        ! calcul distance en km selon les pixels
        x = om_g2(1) - om_g1(1)
        y = om_g2(2) - om_g1(2)
        z = om_g2(3) - om_g1(3)
        km_per_pix = dsqrt( x*x + y*y + z*z) / delta_pix

        ! calcul distance en km selon les lignes
        x = om_g3(1) - om_g1(1)
        y = om_g3(2) - om_g1(2)
        z = om_g3(3) - om_g1(3)
        km_per_lin = dsqrt( x*x + y*y + z*z) / delta_lin


        Return
        End
