      subroutine contour_iasi_avhrr(isat, inst_s, inst_a,
     >         reftime_a, reftime_s, 
     >         ipixel, iline,
     >         att_mode, att,
     >         ncontour ,
     >         avhrr_oa_c_pix, avhrr_oa_c_lin,
     >         avhrr_c_pix, avhrr_c_lin,
     >         avhrr_oa_pix, avhrr_oa_lin ,
     >         avhrr_pix, avhrr_lin  )
!+
!NOM/NAME $Id: contour_iasi_avhrr.F 84 2007-05-16 14:52:37Z frna $
!
!BUT/PURPOSE 
!       Exact calculation in AVHRR coordinates of IASI footprints:
!         Optical axis
!         4 IASI pixels
!
!DESCRIPTION
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 2003      no          A. Marsouin P.Brunel
!       07 Mar 2007    correct several bugs relative to misalignment angles
!                      update call to invloc
!
        implicit none                                                     
!
!ARGUMENTS
!input
        integer isat                    ! satellite number
        integer inst_a                  ! avhrr instrument number
        integer inst_s                  ! sounding instrument number
        double precision reftime_a      ! instant julien de reference AVHRR
        double precision reftime_s      ! instant julien de reference sundeur
        integer iline
        integer ipixel
        integer att_mode
        double precision att(3,3)       ! attitude error
        integer ncontour                ! nb de points du contour demande
!output
        real avhrr_oa_c_pix     ! pixels avhrr du centre optique IASI
        real avhrr_oa_c_lin     ! lignes avhrr du centre optique IASI
        real avhrr_c_pix(4)     ! pixels avhrr des 4 centres IFOV
        real avhrr_c_lin(4)     ! lignes avhrr des 4 centres IFOV
        real avhrr_oa_pix(ncontour)     ! pixels lignes avhrr des points 
        real avhrr_oa_lin(ncontour)     !    du contour de l'axe optique
        real avhrr_pix(ncontour,4)      ! pixels lignes avhrr des points
        real avhrr_lin(ncontour,4)      !    du contour des 4 pixels


!DECLARATIONS
! Local parameters:
        integer max_nbp_edge
        parameter (max_nbp_edge = 360)
!
! Local variables
        integer i, j
        integer ierr                    ! return code of sp_read

        real line_s
        real pixel_s

        real pixel_a
        real line_a

        integer orbnum                  ! orbit number
        double precision Mat_Ra_Rg(3,3) ! Ra to Rg conversion matrix
        double precision pos(3) ! satellite position vector             (km)
        double precision vel(3) ! satellite inertial velocity vector    (km/s)
        double precision time_s ! julian instant of sounder pixel 
        double precision time   ! julian instant of avhrr pixel
        double precision tfirst ! julian instant of the first line (julian day)
        double precision tlast  ! julian instant of the last line  (julian day)
        double precision scnang ! scanning angles of viewed points (rad)
        double precision om_g(3)        ! earth viewed point vector (km)

        ! viewing vector in in spacecraft fixed ref. frame
        double precision so_rs(3)   ! for optical axis
        double precision sm_rs(3,4) ! for 4 IFOVs center

        ! Viewing vectors for one point of the footprint edge
        double precision soa_rs(3)   ! for optical axis
        double precision sma_rs(3)   ! for 4 IFOVS

        ! Viewing vectors for all points of the footprint edge
        double precision sop_rs(3, max_nbp_edge)    ! for optical axis
        double precision smp_rs(3, 4, max_nbp_edge) ! for 4 IFOVS
!-
!       
        pixel_s = ipixel
        line_s  = iline
        
        call footprint_iasi(
     &        pixel_s, line_s, isat, inst_s, ncontour, reftime_s, 
     &        time_s, so_rs, sm_rs, sop_rs, smp_rs, ierr)

        call intposvel(1,time_s,pos,vel,orbnum,ierr)

        call snagre(att_mode, pos, vel, Mat_Ra_Rg, ierr)

        tfirst = time_s - 360. / 86400d0
        tlast  = time_s + 360. / 86400d0

        !Optical axis
        call earthpix(pos, att, Mat_Ra_Rg, so_rs, om_g, ierr)
        call invloc(isat, inst_a,
     &       att_mode, att, om_g, tfirst, tlast,
     &       time, soa_rs, ierr)
        call viewvecttolp(time,soa_rs,isat,inst_a,
     &       reftime_a, pixel_a, line_a, scnang)
        avhrr_oa_c_pix = pixel_a
        avhrr_oa_c_lin = line_a
        
        do i=1, ncontour

           call earthpix(pos, att, Mat_Ra_Rg, sop_rs(1,i), om_g, ierr)

           call invloc(isat, inst_a,
     &          att_mode, att, om_g, tfirst, tlast,
     &          time, soa_rs, ierr)
 
           call viewvecttolp(time,soa_rs,isat,inst_a,
     &          reftime_a, pixel_a, line_a, scnang)

           avhrr_oa_pix(i) = pixel_a
           avhrr_oa_lin(i) = line_a
        enddo

        ! 4 IASI pixels
        do j = 1, 4
           call earthpix(pos, att, Mat_Ra_Rg, sm_rs(1,j), om_g, ierr)
           call invloc(isat, inst_a,
     &          att_mode, att, om_g, tfirst, tlast,
     &          time, sma_rs, ierr)
           call viewvecttolp(time,sma_rs,isat,inst_a,
     &          reftime_a, pixel_a, line_a, scnang)
           avhrr_c_pix(j) = pixel_a
           avhrr_c_lin(j) = line_a
        
           do i=1, ncontour
              
              call earthpix(pos, att, Mat_Ra_Rg, smp_rs(1,j,i),
     &             om_g, ierr)
                            
              call invloc(isat, inst_a,
     &             att_mode, att, om_g, tfirst, tlast,
     &             time, sma_rs, ierr)
 
              call viewvecttolp(time,sma_rs,isat,inst_a,
     &             reftime_a, pixel_a, line_a, scnang)

              avhrr_pix(i,j) = pixel_a
              avhrr_lin(i,j) = line_a
           end do
        end do
        

        return
        end
      
