      subroutine contour_sondeur_avhrr(isat, inst_s, inst_a,
     >         reftime_a, reftime_s, 
     >         ipixel, iline,
     >         att_mode, att,
     >         ncontour ,
     >         avhrr_c_pix, avhrr_c_lin,
     >         avhrr_pix, avhrr_lin,
     >         lat, lon )

!NOM/NAME $Id: contour_sondeur_avhrr.F 84 2007-05-16 14:52:37Z frna $
!
!BUT/PURPOSE 
!       Exact calculation of a sounder footprint in AVHRR 
!       coordinates and latitude and longitue 
!
!DESCRIPTION
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 2003      no          A. Marsouin P.Brunel
! 1.1   15 Jul 2004   Remove multiple declaration of ncontour  N C Atkinson
!       07 Mar 2007   correct several bugs relative to misalignment angles
!                     update call to invloc  P. Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!DECLARATIONS
!
! Local parameters:
        integer max_nbp_edge
        parameter (max_nbp_edge = 360)

!ARGUMENTS
!input
        integer isat                    ! satellite number
        integer inst_a                  ! avhrr instrument number
        integer inst_s                  ! sounding instrument number
        double precision reftime_a      ! instant julien de reference AVHRR
        double precision reftime_s      ! instant julien de reference sondeur
        integer iline
        integer ipixel
        integer att_mode
        double precision att(3,3)       ! attitude error
        integer ncontour                ! nb de points du contour demande

!output
        real avhrr_c_pix        ! pixels avhrr du centre
        real avhrr_c_lin        ! lignes avhrr du centre
        real avhrr_pix(ncontour)        ! pixels avhrr des points du contour
        real avhrr_lin(ncontour)        ! lignes avhrr des points du contour
        double precision lat(ncontour)  ! latitude des points du contour
        double precision lon(ncontour)  ! longitude des points du contour

! Local variables
        integer ierr                    ! return code of sp_read
        integer i

        real line_s
        real pixel_s

        real pixel_a
        real line_a

        integer orbnum                  ! orbit number
        double precision Mat_Ra_Rg(3,3) ! Ra to Rg conversion matrix
        double precision pos(3) ! satellite position vector             (km)
        double precision vel(3) ! satellite inertial velocity vector    (km/s)
        double precision time_s ! julian instant of sounder pixel 
        double precision time   ! julian instant of avhrr pixel
        double precision tfirst ! julian instant of the first line (julian day)
        double precision tlast  ! julian instant of the last line  (julian day)
        double precision psi0   ! initial angle for boundary (rad)
        double precision scnang ! scanning angles of viewed points (rad)
        double precision om_g(3)        ! earth viewed point vector (km)

        ! viewing vector in in spacecraft fixed ref. frame
        double precision sm_rs(3) ! for sounder optical axis
        double precision sa_rs(3) ! for avhrr optical axis
        double precision sp_rs(3, max_nbp_edge) ! for contour
!-
!       

        pixel_s = ipixel
        line_s  = iline
        psi0 = 0.

        call footprint(
     &       pixel_s, line_s, isat, inst_s, ncontour, psi0, reftime_s, 
     &       time_s, sm_rs, sp_rs, ierr)
        If( ierr .Ne. 0 ) Then
           Call ml_wti('error','footprint return code',ierr)
           Return
        Endif

        call intposvel(1,time_s,pos,vel,orbnum,ierr)

        call snagre(att_mode, pos, vel, Mat_Ra_Rg, ierr)
        If( ierr .Ne. 0 ) Then
           Call ml_wti('error','snagre return code',ierr)
           Call ml_wti('error','snagre input att mode ',att_mode)
           Return
        Endif

        tfirst = time_s - 360. / 86400d0
        tlast  = time_s + 360. / 86400d0

        ! center position in avhrr coordinates
        call earthpix(pos, att, Mat_Ra_Rg, sm_rs, om_g, ierr)
        If( ierr .Ne. 0 ) Then
           Call ml_wti('error','earthpix return code',ierr)
           Return
        End If

        call invloc(isat, inst_a,
     &       att_mode, att, om_g, tfirst, tlast,
     &       time, sa_rs, ierr)
        If( ierr .Ne. 0 ) Then
           Call ml_wti('error','invloc return code',ierr)
           Return
        End If

        call viewvecttolp(time,sa_rs,isat,inst_a,
     &       reftime_a, pixel_a, line_a, scnang)
        avhrr_c_pix = pixel_a
        avhrr_c_lin = line_a
        

        do i=1, ncontour

           call earthpix(pos, att, Mat_Ra_Rg, sp_rs(1,i), om_g, ierr)
           If( ierr .Ne. 0 ) Then
              Call ml_wti('error','earthpix return code',ierr)
              Return
           End If

           call cartlalo(1, om_g, lat(i), lon(i))

           call invloc(isat, inst_a,
     &                    att_mode, att, om_g, tfirst, tlast,
     &                    time, sa_rs, ierr)
           If( ierr .Ne. 0 ) Then
              Call ml_wti('error','invloc return code',ierr)
              Return
           End If
 
           call viewvecttolp(time,sa_rs,isat,inst_a,
     &                reftime_a, pixel_a, line_a, scnang)

           avhrr_pix(i) = pixel_a
           avhrr_lin(i) = line_a
        enddo
        

        return
        end
      
