      subroutine ellipse_sondeur_avhrr(isat, inst_s, inst_a,
     >         reftime_a, reftime_s, 
     >         ipixel, iline,
     >         att_mode, att,
     >         avhrr_c_pix, avhrr_c_lin,
     >         axe_pix, axe_lin, 
     >         km_per_pix, km_per_lin  )

!NOM/NAME $Id: ellipse_sondeur_avhrr.F 100 2007-10-18 11:57:46Z frna $
!
!BUT/PURPOSE 
!       Calculation of an approximate footprint in AVHRR coordinates. 
!       This footprint is an ellipse, the routine calculates
!       its centre and its 2 axis. 
!
!DESCRIPTION
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 2003      no          A. Marsouin P.Brunel
!       07 Mar 2007   correct several bugs relative to misalignment angles
!                     update call to invloc  P. Brunel
!
!
        implicit none                                                     
!
!DECLARATIONS
!
! Local parameters:
        double precision pi
        parameter ( pi = 3.14159265358979311600d0)

!ARGUMENTS
!input
        integer isat                    ! satellite number
        integer inst_a                  ! avhrr instrument number
        integer inst_s                  ! sounding instrument number
        double precision reftime_a      ! instant julien de reference AVHRR
        double precision reftime_s      ! instant julien de reference sondeur
        integer iline
        integer ipixel
        integer att_mode
        double precision att(3,3)       ! attitude error
!
!output
        real avhrr_c_pix                ! pixels avhrr du centre
        real avhrr_c_lin                ! lignes avhrr du centre
        real axe_pix                    ! axe ellipse en pixel (pixel avhrr)
        real axe_lin                    ! axe ellipse en ligne (ligne avhrr)
        real km_per_lin                 ! taille en km entre 2 lignes AVHRR
        real km_per_pix                 ! taille en km entre 2 pixels AVHRR

! Local variables
        integer ncontour                ! nb de points du contour demande
        integer ierr                    ! return code of sp_read

        real line_s
        real pixel_s
        real pixel_a
        real line_a

        integer orbnum                  ! orbit number
        double precision Mat_Ra_Rg(3,3) ! Ra to Rg conversion matrix
        double precision pos(3) ! satellite position vector             (km)
        double precision vel(3) ! satellite inertial velocity vector    (km/s)
        double precision time_s ! julian instant of sounder pixel 
        double precision time   ! julian instant of avhrr pixel
        double precision tfirst ! julian instant of the first line (julian day)
        double precision tlast  ! julian instant of the last line  (julian day)
        double precision psi0   ! initial angle for boundary (rad)
        double precision scnang ! scanning angles of viewed points (rad)
        double precision om_g1(3)       ! earth viewed point vector (km)
        double precision om_g2(3)       ! earth viewed point vector (km)

        ! viewing vector in in spacecraft fixed ref. frame
        double precision sm_rs(3) ! for sounder optical axis
        double precision sa_rs(3) ! for avhrr optical axis
        double precision sp_rs(3) ! for contour
        double precision x, y, z
!-
!       
        pixel_s = ipixel
        line_s  = iline

        ! calculates for only center and back border
        !-------------------------------------------
        psi0     = 0.
        ncontour = 1
        call footprint(
     &       pixel_s, line_s, isat, inst_s, ncontour, psi0, reftime_s, 
     &       time_s, sm_rs, sp_rs, ierr)

        call intposvel(1,time_s,pos,vel,orbnum,ierr)

        call snagre(att_mode, pos, vel, Mat_Ra_Rg, ierr)

        tfirst = time_s - 360. / 86400d0
        tlast  = time_s + 360. / 86400d0

        ! center position in avhrr coordinates
        call earthpix(pos, att, Mat_Ra_Rg, sm_rs, om_g1, ierr)
        call invloc(isat, inst_a,
     &          att_mode, att, om_g1, tfirst, tlast,
     &          time, sa_rs, ierr)
        call viewvecttolp(time,sa_rs,isat,inst_a,
     &       reftime_a, pixel_a, line_a, scnang)
        avhrr_c_pix = pixel_a
        avhrr_c_lin = line_a
        

        ! Across scan axis
        !------------------
        call earthpix(pos, att, Mat_Ra_Rg, sp_rs, om_g2, ierr)
        
        call invloc(isat, inst_a,
     &       att_mode, att, om_g2, tfirst, tlast,
     &       time, sa_rs, ierr)
 
        call viewvecttolp(time,sa_rs,isat,inst_a,
     &       reftime_a, pixel_a, line_a, scnang)

        axe_lin  = abs( line_a - avhrr_c_lin ) *2.
        
        ! calcul distance en km centre-bord arriere de l'ellipse
        x = om_g2(1) - om_g1(1)
        y = om_g2(2) - om_g1(2)
        z = om_g2(3) - om_g1(3)
        km_per_lin = 2.d0 * dsqrt( x*x + y*y + z*z) / axe_lin


        ! Along scan axis
        !-----------------
        psi0     = pi / 2.
        ncontour = 1
        call footprint(
     &       pixel_s, line_s, isat, inst_s, ncontour, psi0, reftime_s, 
     &       time_s, sm_rs, sp_rs, ierr)
       
        call earthpix(pos, att, Mat_Ra_Rg, sp_rs, om_g2, ierr)
        
        call invloc(isat, inst_a,
     &       att_mode, att, om_g2, tfirst, tlast,
     &       time, sa_rs, ierr)
 
        call viewvecttolp(time,sa_rs,isat,inst_a,
     &       reftime_a, pixel_a, line_a, scnang)

!        axe_pix  = ifov_s / scstang_a
        axe_pix  = abs( pixel_a - avhrr_c_pix ) *2.
        
        ! calcul distance en km centre-bord arriere de l'ellipse
        x = om_g2(1) - om_g1(1)
        y = om_g2(2) - om_g1(2)
        z = om_g2(3) - om_g1(3)
        km_per_pix = 2.d0 * dsqrt( x*x + y*y + z*z) / axe_pix


        return
        end
      
