#!/bin/ksh
#$Id: AAPP_RUN_simplified,v 1.1 2005/04/18 12:30:24 frna Exp $
#
#Simplified version of the AAPP_RUN script. For ATOVS only (not TOVS)
#
#Assume that the following environment variables have been set:
# FILE		- HRPT file
# YEAR		- Year
# YYYYMMDD	- Year month day
# HHMN		- Hour & minute
# NNNNN		- orbit
# SATIMG	- e.g. noaa16
# AVHRR         - set to 1 if AVHRR is required (default 1)
# ATOVS         - set to 1 if ATOVS is required (default 1)
#
# HISTORY        
# version date     comment                                   author
# 1.0     18/04/05 initial                                   N C Atkinson
#
##############################################################################

. ${HOME}/ATOVS_ENV
. liblog.sh

if [ ${#AVHRR} = 0 ]; then  #check number of characters in $AVHRR
  AVHRR=1                   #default
fi
if [ ${#ATOVS} = 0 ]; then  #check number of characters in $ATOVS
  ATOVS=1                   #default
fi

# Which type of bulletin for the requested satellite
# this determines which satpos program to run
#---------------------------------------------------
  i=0
  set -A ABUL ${PAR_NAVIGATION_DEFAULT_LISTEBUL}
  for SAT in ${PAR_NAVIGATION_DEFAULT_LISTESAT}
  do
    if [ "${SAT}" = "${SATIMG}" ] ; then
       BUL=${ABUL[i]}
    fi
    i=`expr ${i} + 1`
  done
  BUL=${BUL=tbus}
  case ${BUL} in
    tbus)  BULING_COM='tbusing';;
    tle)   BULING_COM='tleing';;
    *) BULING_COM='tbusing';;
  esac

#Run TBUS or TWO LINES ingest for the satellite
#----------------------------------------------
log_info "running ${BULING_COM}"
${BULING_COM} -s ${SATIMG}

#Run decommutation
#-----------------
log_info "decom ATOVS"

if [ $AVHRR = 1 ] && [ $ATOVS = 1 ]; then
  options=" 1, 1, 1, 1, 1,0,0,0,0,0,0, 1"  #11=HIRS, 15=AMSUA, 16=AMSUB, 14=AVHRR
  streams="10,11,15,15,16,0,0,0,0,0,0,14"
elif [ $AVHRR = 0 ] && [ $ATOVS = 1 ]; then
  options=" 1, 1, 1, 1, 1,0,0,0,0,0,0,0"
  streams="10,11,15,15,16,0,0,0,0,0,0,0"
elif [ $AVHRR = 1 ] && [ $ATOVS = 0 ]; then
  options=" 1,0,0,0,0,0,0,0,0,0,0, 1"
  streams="10,0,0,0,0,0,0,0,0,0,0,14"
else
  echo Neither ATOVS nor AVHRR requested!
  exit
fi

cat << eof_inp >  decommutation.par
${options}
${streams}
${YEAR}                                    ! year of the data
0                                          ! operational mode
${NNNNN},${NNNNN}                          ! start and end orbit numbers
eof_inp


decommutation ATOVS decommutation.par ${FILE}  \
 || { log_error "decommutation failed"; exit 1; }

#Run calibration
#----------------
if [ $ATOVS = 1 ]; then

  log_info "AMSUACL"
  amsuacl -c -l -s $SATIMG -d $YYYYMMDD -h $HHMN  -n $NNNNN aman.l1b \
    || { log_error "amsuacl failed"; exit 1; }

  SATNB=`echo ${SATIMG} | cut -c5-6`
  if [ ${SATNB} -le 17 ];then
    log_info "AMSUBCL"
    amsubcl -c -l -s $SATIMG -d $YYYYMMDD -h $HHMN  -n $NNNNN ambn.l1b \
      || { log_error "amsubcl failed"; exit 1; }
  else
    log_info "MHSCL"
    mhsl -c -l -s $SATIMG -d $YYYYMMDD -h $HHMN  -n $NNNNN ambn.l1b \
      || { log_error "amsubcl failed"; exit 1; }
  fi

  if [ x${HIRSCL_VERSION} = x1 ]; then
    log_info "HIRSCL_ALGOV4"
    FILE_HISTORIC=${PAR_CALIBRATION_MONITOR}/${SATIMG}/hirs_historic.txt
    hirs_historic_file_manage -m $HIST_SIZE_HIGH -r $HIST_SIZE_LOW \
     -n $HIST_NMAX $FILE_HISTORIC
    hcalcb1_algoV4 -s $SATIMG -y $YYYY -m $MM -d $DD -h $HH  -n $MN \
      || { log_error "hcalcb1_algoV4 failed"; exit 1; }
    hirscl_algoV4 -c -l -s $SATIMG -d $YYYYMMDD -h $HHMN  -n $NNNNN hrsn.l1b \
      || { log_error "hirscl failed"; exit 1; }
  else
    log_info "HIRSCL"
    hirscl -c -l -s $SATIMG -d $YYYYMMDD -h $HHMN  -n $NNNNN hrsn.l1b \
      || { log_error "hirscl failed"; exit 1; }

  fi
fi

if [ $AVHRR = 1 ]; then
  log_info "AVHRCL"
  avhrcl -c -l -s $SATIMG -d $YYYYMMDD -h $HHMN  -n $NNNNN hrpt.l1b \
    || { log_error "avhrcl failed"; exit 1; }
fi

#Run preprocessing
#-----------------
if [ $ATOVS = 1 ]; then
  log_info "ATOVIN"
  atovin HIRS AMSU-A AMSU-B || { log_error "atovin failed"; exit 1; }

  log_info "ATOVPP"
  atovpp ATOVS || { log_error "atovpp failed"; exit 1; }

  if [ $AVHRR = 1 ]; then
    log_info "AVH2HIRS"
    avh2hirs ATOVS || { log_error "avh2hirs failed"; exit 1; }
  fi
fi
exit
