/* 
! Name: $Id: amb1d_to_hdf5.c 635 2019-04-15 15:08:00Z frna $
!
! Purpose: Read the contents of a AMSUB/MHS l1d file into memory and write out
!          in hdf5 format.
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    25/09/2014  New. N C Atkinson
!
! Code Description: C
!
* LIBS = aapphdf5 atovpp atov auxaapp f7nl1b
!-------------------------------------------------------------------------------
*/

#include <stdio.h>

#ifdef HAS_LIBHDF5
#include "hdf5.h"
#include "aapp_hdf5.h"
#include "amb1d_c.h"
#endif

int main (int argc, char **argv){

#ifdef HAS_LIBHDF5

  int clevel=0;
  if (argc < 3) {
    printf("Usage: amb1d_to_hdf5.exe infile outfile\n");
    return 0;
  }
  if (argc > 3) {
    sscanf(argv[3],"%d",&clevel);
    printf("compression level is %d\n",clevel);
  }
  amb1d_read(argv[1]);
  amb1d_to_hdf5_write(argv[2],clevel);
  return 0;

#else

  printf("amb1d_to_hdf5.exe has not been built with HDF-5\n");
  printf("Please run \"configure\" with --external-libs=\"-Lxxx/lib -lhdf5\"\n");
  printf("  and --external-includes=\"-Lxxx/include\"\n");
  printf("  where xxx is the location of the HDF-5 library\n");
  return 1;
  
#endif

}
