C LIBS = atov auxaapp f7nl1b
!+
!
      PROGRAM ATMS1C_PRINT_NEDT
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: atms1c_print_nedt.F 320 2012-07-17 08:46:19Z frna $
!
! Purpose: Print mean and sd of the NEDT values for an ATMS l1c file
!
! Method:
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2012, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    13/07/2012  New (Nigel Atkinson)

#include "atms1c.h"

! Variables
      REAL warmsum(atms_1cnchan)
      REAL warmsumsq(atms_1cnchan)
      REAL coldsum(atms_1cnchan)
      REAL coldsumsq(atms_1cnchan)

      DATA warmsum/atms_1cnchan*0.0/
      DATA warmsumsq/atms_1cnchan*0.0/
      DATA coldsum/atms_1cnchan*0.0/
      DATA coldsumsq/atms_1cnchan*0.0/

      INTEGER nwarm(atms_1cnchan)      
      INTEGER ncold(atms_1cnchan)      

      DATA nwarm/atms_1cnchan*0/
      DATA ncold/atms_1cnchan*0/

      INTEGER   AT1cunit   !Unit number
      DATA AT1cunit/17/
      
      REAL x, warmmean, coldmean, warmsd, coldsd

      INTEGER iscan,ichan,n
!
!- End of header

! Open the 1c file and read all scan lines
      CALL IOAT1C("OPEN/WRITE",AT1cunit,0)
      CALL IOAT1C("READ HEADER",AT1cunit,1)
      DO iscan=1,atms1c_h_scnlin
        CALL IOAT1C("READ DATA",AT1cunit,1+iscan)
        DO ichan=1,atms_1cnchan
          x = 1.0E-4*atms1c_nedtwarm(ichan)
          IF (x .GT. 0.0) THEN
            nwarm(ichan) = nwarm(ichan) + 1
            warmsum(ichan) = warmsum(ichan) + x
            warmsumsq(ichan) = warmsumsq(ichan) + x*x
          ENDIF
          x = 1.0E-4*atms1c_nedtcold(ichan)
          IF (x .GT. 0.0) THEN
            ncold(ichan) = ncold(ichan) + 1
            coldsum(ichan) = coldsum(ichan) + x
            coldsumsq(ichan) = coldsumsq(ichan) + x*x
          ENDIF
        ENDDO
      ENDDO

!Compute mean and sd
      WRITE(*,*) 'Chan warmmean  coldmean   warmsd    coldsd'
      DO ichan=1,atms_1cnchan
        n = nwarm(ichan)
        IF (n .GT. 2) THEN
          warmmean = warmsum(ichan)/n
          warmsd = sqrt((warmsumsq(ichan) - warmsum(ichan)**2/n)/(n-1))
        ENDIF
        n = ncold(ichan)
        IF (n .GT. 2) THEN
          coldmean = coldsum(ichan)/n
          coldsd = sqrt((coldsumsq(ichan) - coldsum(ichan)**2/n)/(n-1))
        ENDIF
        WRITE(*,'(I3,4F10.3)') ichan,warmmean,coldmean,warmsd,coldsd
      ENDDO

      CALL IOAT1C("CLOSE",AT1cunit,0)

      END
