/*
! name: $Id: atms_sdr.c 635 2019-04-15 15:08:00Z frna $
!
! Purpose: Read an ATMS SDR file and geolocation file and convert to AAPP 1c format
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    13/10/2010  Original code. (N.C.Atkinson)
!  1.1    17/10/2011  Allow user to specify the geo file (as in cris_sdr.c)
!
! Code Description: C
!
! C subroutines and functions called:
!     atms_sdr_read
!     atms_geo_read
!     atms_tdr_read
!     atms_sdr_intf (interface to Fortran) 
!
* LIBS = aapphdf5 atov atovin auxaapp f7nl1b atov
!-------------------------------------------------------------------------------
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#ifdef HAS_LIBHDF5
#include "hdf5.h"
#include "aapp_hdf5.h"
#include "atms_sdr.h"
#endif

int main (int argc, char **argv){

#ifdef HAS_LIBHDF5

  char tmp_geo_file[200];
  int len;

  strcpy(tmp_geo_file, argv[2]);
  len = strlen(tmp_geo_file);
  
  atms_sdr_read(argv[1], tmp_geo_file[len-1]);
  
  if (tmp_geo_file[len-1] == '/')
    atms_geo_read(strcat(tmp_geo_file, N_GEO_Ref));
  else
    atms_geo_read(tmp_geo_file);
  
  if (argc > 3) atms_tdr_read(argv[3]);

  atms_sdr_intf();
  return 0;

#else

  printf("atms_sdr.exe has not been built with HDF-5\n");
  printf("Please run \"configure\" with --external-libs=\"-Lxxx/lib -lhdf5\"\n");
  printf("  and --external-includes=\"-Lxxx/include\"\n");
  printf("  where xxx is the location of the HDF-5 library\n");
  return 1;
  
#endif

}
