#!/bin/ksh
#
#$Id$
#
#Convert ATMS SDR files in HDF-5 format to AAPP 1c format
#
#History:
# 1.0   13/10/2010  initial version. N C Atkinson
# 1.1   24/06/2011  exe file could be elsewhere in $PATH
# 1.2   17/10/2011  add -g option (same as cris_sdr)
#==============================================================================

. "$AAPP_PREFIX/ATOVS_CONF"

usage(){
  echo
  echo 'usage : atms_sdr [-o Outputfile] [-g Geofile] SDRfile [TDRfile]'
  echo
}

#Look for the exe file, first in script directory then in $PATH

exe=$(dirname $0)/atms_sdr.exe

[ -x $exe ] || exe=$(IFS=":"; for p in $PATH; do q=$p/atms_sdr.exe; \
  [ -x $q ] && echo $q && return; done)

[ -x $exe ] || { echo "Cannot find atms_sdr.exe"; exit 1; }

missing=$(ldd $exe | grep "not found")

if [ "$missing" != "" ]; then
  echo $missing
  echo "Library not found, please locate the shared library listed above"
  echo " and add it to your LD_LIBRARY_PATH environment variable"
  exit 1
fi

while getopts o:g: c
do
  case $c in
     o)	outfile=$OPTARG;;
     g)	geofile=$OPTARG;;
     \?)      usage
              exit 2;;
  esac
done
shift `expr $OPTIND - 1`

[ $# -lt 1 ] && { usage; exit 1; }
[ -f $1 ] || { echo "SDR file $1 not found"; exit 1; }

outfile=${outfile:-${1%.*}.l1c}
geofile=${geofile:-$(dirname $1)/}   #give directory if not specified

rm -f ${FORT}10
ln -sf $DIR_PREPROC/fdf.dat ${FORT}20
ln -sf $DIR_PREPROC/atms_channels.dat ${FORT}21

$exe $1 $geofile $2

status=$?
if [ $status = 0 ]; then
  mv ${FORT}10 $outfile
  echo "created $outfile"
else
  echo "atms_sdr.exe failed"
fi

rm -f ${FORT}20 ${FORT}21
exit $status
