! Name: $Id: atovsCompare.F 134 2008-11-11 14:32:07Z frna $

        PROGRAM atovsCompare

        IMPLICIT none
!
!--
!
!  D E S C R I P T I O N
!
!        Compare two files of ATOVS data  
!
!  I N P U T S :
!
!       HIRS1               Calibrated HIRS+MSU+AVHRR data
!       HIRS2               Calibrated HIRS+MSU+AVHRR data
!
!  O U T P U T S :
!        
!       ATOVSCOMPARE.LOG     Differences
!
!--
!
!  A U T H O R S:   A.Korpela
!  CREATION DATE:   15-Dec-1997
!
!--
!
!  C H A N G E   L O G
!
!      Date     | Name  | Description
!---------------+-------+-----------------------------------------------------
!  15-Dec-1997  |  AK   | Original code
!---------------+-------+-----------------------------------------------------
!  29-Aug-2003  |  AK   | Improved output
!---------------+-------+-----------------------------------------------------
!  11-Mar-2004  |  NA   | Change integer*1 to character*1 for portability
!---------------+-------+-----------------------------------------------------
!  12-Apr-2004  |  AK   | HIRS Level 1d comparison for pre NOAA-15
!---------------+-------+-----------------------------------------------------
!  14-Dec-2004  |  NA   | Change 'TYPE' to 'STATUS' in OPEN statements
!===============+=======+=====================================================
!  24-Jan-2006  |  PM   | Change INCLUDE to #include
!===============+=======+=====================================================
!  11-Nov-2008  |  NA   | Accommodate MetOp (satID=1,2,3)
!===============+=======+=====================================================
!
        INTEGER*4 HNFOVS,ANFOVS,BNFOVS,HNCHAN,ANCHAN,BNCHAN
        PARAMETER (HNFOVS = 56)
        PARAMETER (ANFOVS = 30)
        PARAMETER (BNFOVS = 90)
        PARAMETER (HNCHAN = 20)
        PARAMETER (ANCHAN = 15)
        PARAMETER (BNCHAN = 5)

#include "hrs1b.h"
#include "ama1b.h"
#include "amb1b.h"
#include "hrs1c.h"
#include "ama1c.h"
#include "amb1c.h"
#include "hrs1d.h"
#include "ama1d.h"
#include "amb1d.h"
#include "reclopt.h"

        INTEGER    Hirs_brec_words, Amsua_brec_words, Amsub_brec_words
        INTEGER    Hirs_crec_words, Amsua_crec_words, Amsub_crec_words
        INTEGER    Hirs_drec_words, Amsua_drec_words, Amsub_drec_words
        INTEGER    DataMissing
        INTEGER    MaxWords
        PARAMETER  (Hirs_brec_words  = reclhrs/4)        ! =
        PARAMETER  (Amsua_brec_words = reclama/4)        ! =
        PARAMETER  (Amsub_brec_words = reclamb/4)        ! =
        PARAMETER  (Hirs_crec_words  = hrs_1crecl/4)     ! =
        PARAMETER  (Amsua_crec_words = ama_1crecl/4)     ! =
        PARAMETER  (Amsub_crec_words = amb_1crecl/4)     ! =
        PARAMETER  (Hirs_drec_words  = hrs_1drecl/4)     ! = 3968
        PARAMETER  (Amsua_drec_words = ama_1drecl/4)     ! = 1152
        PARAMETER  (Amsub_drec_words = amb_1drecl/4)     ! = 3072
        PARAMETER  (MaxWords = 4000)
        PARAMETER  (DataMissing = -999999)

        INTEGER    MaxInst, MaxLevel, MaxDesc
        PARAMETER  ( MaxInst = 3 )
        PARAMETER  ( MaxLevel = 3 )
        PARAMETER  ( MaxDesc = 50 )
        INTEGER    Hirs,AmsuB,AmsuA
        PARAMETER  ( AmsuA = 1 )
        PARAMETER  ( AmsuB = 2 )
        PARAMETER  ( Hirs = 3 )
        INTEGER    Lev1b,Lev1c,Lev1d
        PARAMETER  ( Lev1b = 1 )
        PARAMETER  ( Lev1c = 2 )
        PARAMETER  ( Lev1d = 3 )

        LOGICAL         presentA,presentB,compare,bigendian1,bigendian2
        LOGICAL         report,reportAny,reportAll,makeSpace,remindMe
        LOGICAL         haveInput
        INTEGER         limit,prvIdx
        INTEGER         uin,uout,Ulog,UdatA,UdatB
        INTEGER         satID
        INTEGER         iD,irec,kk,recwords,reccnt,cnt,diff,iInst,iLev
        INTEGER         flen,idx,dbgIdx,item,int1,int2
        INTEGER         valA,valB
        INTEGER         recWordCnt(MaxInst,MaxLevel)
        INTEGER         recA(MaxWords)
        INTEGER         recB(MaxWords)
        INTEGER         descIdx(MaxWords)
        INTEGER         wordMinA(MaxWords)
        INTEGER         wordMaxA(MaxWords)
        INTEGER         wordMinB(MaxWords)
        INTEGER         wordMaxB(MaxWords)
        INTEGER         wordDifMax(MaxWords)
        INTEGER         dataDifMax(MaxDesc)
        INTEGER         dataDifA(MaxDesc)
        INTEGER         dataDifB(MaxDesc)
        INTEGER         dataDifTot(MaxDesc)
        INTEGER         dataDifCnt(MaxDesc)
        INTEGER         dataInAB(MaxDesc)
        INTEGER         dataOnlyA(MaxDesc)
        INTEGER         dataOnlyB(MaxDesc)
        INTEGER         dataMissA(MaxDesc)
        INTEGER         dataMissB(MaxDesc)
        INTEGER         dataMissAB(MaxDesc)
        INTEGER         dataMinA(MaxDesc)
        INTEGER         dataMaxA(MaxDesc)
        INTEGER         dataMinB(MaxDesc)
        INTEGER         dataMaxB(MaxDesc)
        CHARACTER*1     levelID
        CHARACTER*80    fname
        CHARACTER*80    dirA,fileA
        CHARACTER*80    dirB,fileB
        CHARACTER*40    descHtxt(MaxDesc,MaxInst,MaxLevel)     ! Header descriptions
        INTEGER         descHwrd(MaxDesc,3,MaxInst,MaxLevel)   ! Header description range
        CHARACTER*40    descDtxt(MaxDesc,MaxInst,MaxLevel)     ! Data   descriptions
        INTEGER         descDwrd(MaxDesc,3,MaxInst,MaxLevel)   ! Data   description range
        CHARACTER*40    desc

        DATA uin,uout,Ulog,UdatA,UdatB/5,6,7,11,12/
        DATA dbgIdx/99999/
!
!  Starting up
!----------------------------------------------------------
!
        recWordCnt(AmsuA,Lev1c) = Amsua_crec_words
        recWordCnt(AmsuB,Lev1c) = Amsub_crec_words
        recWordCnt(Hirs, Lev1c) = Hirs_crec_words
        recWordCnt(AmsuA,Lev1d) = Amsua_drec_words
        recWordCnt(AmsuB,Lev1d) = Amsub_drec_words
        recWordCnt(Hirs, Lev1d) = Hirs_drec_words
        haveInput = .FALSE.

        WRITE(6,*) 'Subdirectory A:'
        dirA = 'REF'
        READ(5,'(A)') dirA
        WRITE(6,*) 'Subdirectory B:'
        dirB = 'RESULT'
        READ(5,'(A)') dirB

        DO iLev = Lev1c,Lev1d

        IF (iLev.EQ.Lev1b) levelID='b'
        IF (iLev.EQ.Lev1c) levelID='c'
        IF (iLev.EQ.Lev1d) levelID='d'

        DO iInst = 1,MaxInst

        IF (iInst.EQ.AmsuA) THEN
           fname='aman.l1'//levelID
           flen = 8
        ENDIF
        IF (iInst.EQ.AmsuB) THEN
           fname='ambn.l1'//levelID
           flen = 8
        ENDIF
        IF (iInst.EQ.Hirs) THEN
           fname='hrsn.l1'//levelID
           flen = 8
        ENDIF

        DO kk = 1,MaxWords
           wordMinA(kk)   = 999999
           wordMaxA(kk)   = -999999
           wordMinB(kk)   = 999999
           wordMaxB(kk)   = -999999
           wordDifMax(kk) = -999999
        ENDDO
        DO kk = 1,MaxDesc
           dataMinA(kk)   =  999999999
           dataMaxA(kk)   = -999999999
           dataMinB(kk)   =  999999999
           dataMaxB(kk)   = -999999999
           dataDifMax(kk) = -999999
           dataDifA(kk)   = -999999
           dataDifB(kk)   = -999999
           dataDifTot(kk) = 0
           dataDifCnt(kk) = 0
           dataInAB(kk) = 0
           dataOnlyA(kk)  = 0
           dataOnlyB(kk)  = 0
           dataMissA(kk)  = 0
           dataMissB(kk)  = 0
           dataMissAB(kk) = 0
        ENDDO

        compare = .FALSE.
        makeSpace = .FALSE.
        reportAny = .FALSE.
        reportAll = .FALSE.
        remindMe = .FALSE.
        fileA=dirA(1:index(dirA,' ')-1)//'/'//fname
        fileB=dirB(1:index(dirB,' ')-1)//'/'//fname
        INQUIRE(FILE=fileA,EXIST=presentA)
        INQUIRE(FILE=fileB,EXIST=presentB)
        IF (presentA .AND. presentB) THEN
           compare = .TRUE.
           haveInput= .TRUE.
           WRITE(6,*)
           WRITE(6,'(A,A)') ' Input A from: ',fileA
           WRITE(6,'(A,A)') ' Input B from: ',fileB
           OPEN(UNIT=UdatA, FILE=fileA, STATUS='OLD', ACCESS='DIRECT',
     &       RECL=4*7/rldiv)
           READ(UdatA,REC=1) (recA(kk),kk=1,7)
           bigendian1=(recA(4).LT.0)
           IF (bigendian1) CALL reWrite(recA,7)
           satID = recA(7)
           CLOSE(UdatA)
           IF ((satID.LT.15 .AND. satID.GT.3) .AND. 
     &         (iInst.NE.Hirs .OR. iLev.NE.Lev1d)) 
     &     THEN
              WRITE(6,'(A)') ' atovsCompare: not implemented'
              compare = .FALSE.
           ENDIF
        ENDIF
!
!       Read through the files
!
!------------------------------------------------------------

        IF (compare) THEN

           CALL setDescriptions ( satID, descHtxt, 
     &            descHwrd, descDtxt, descDwrd )
           IF (satID.LT.15 .AND. satID.GT.3)
     &         recWordCnt(Hirs, Lev1d) = 2818
           recwords = recWordCnt(iInst,iLev)

           OPEN(UNIT=UdatA, FILE=fileA, STATUS='OLD', ACCESS='DIRECT',
     &        RECL=4*recwords/rldiv)
           OPEN(UNIT=UdatB, FILE=fileB, STATUS='OLD', ACCESS='DIRECT',
     &        RECL=4*recwords/rldiv)
           OPEN(UNIT=Ulog, FILE='compare_'//fname(1:flen)//'.log',
     &        FORM='FORMATTED', STATUS='UNKNOWN')
           WRITE(Ulog,*)
           WRITE(Ulog,'(A,A)') ' Input A from: ',fileA
           WRITE(Ulog,'(A,A)') ' Input B from: ',fileB
!
!  Compare header records
!-----------------------------
!
           WRITE(Ulog,*)
           WRITE(Ulog,*) 'HEADER RECORD'
           WRITE(Ulog,*) '============='
           WRITE(Ulog,*)
           READ(UdatA,REC=1) (recA(kk),kk=1,recwords)
           READ(UdatB,REC=1) (recB(kk),kk=1,recwords)
           bigendian1=(recA(4).LT.0)
           bigendian2=(recB(4).LT.0)
           IF (bigendian1) CALL reWrite(recA,recwords)
           IF (bigendian2) CALL reWrite(recB,recwords)
           DO kk = 1,recwords
              IF (recA(kk).NE.recB(kk)) THEN
                 CALL FindDescription( descHtxt(1,iInst,iLev),
     &                     descHwrd(1,1,iInst,iLev),kk,idx)
                 desc = descHtxt(idx,iInst,iLev)
                 WRITE(Ulog,'(I6,2X,2I12,7X,A)') kk,recA(kk),recB(kk),
     &                 desc
              ENDIF
           ENDDO
           reccnt=recA(19)
!
!  Compare data records
!---------------------------
!
           WRITE(Ulog,*)
           WRITE(Ulog,*)
           WRITE(Ulog,*) 'DATA RECORDS'
           WRITE(Ulog,*) '============'
           DO kk = 1,recwords
              CALL FindDescription( descDtxt(1,iInst,iLev),
     &               descDwrd(1,1,iInst,iLev),kk,idx )
              descIdx(kk) = idx
           ENDDO

           DO irec = 1,reccnt

              READ(UdatA,REC=1+irec,ERR=997) (recA(kk),kk=1,recwords)
              READ(UdatB,REC=1+irec,ERR=998) (recB(kk),kk=1,recwords)
              IF (bigendian1) CALL reWrite(recA,recwords)
              IF (bigendian2) CALL reWrite(recB,recwords)
              IF (iInst.EQ.Hirs .AND. iLev.EQ.Lev1d) THEN
                 CALL reArrange(recA,satID)
                 CALL reArrange(recB,satID)
              ENDIF
              CALL checkMissing(recA,recwords,satID,iInst,iLev,remindMe)
              CALL checkMissing(recB,recwords,satID,iInst,iLev,remindMe)

              DO kk = 1,recwords

                 valA = recA(kk)
                 valB = recB(kk)
                 diff = ABS(valB-valA)
                 IF (diff.NE.0) THEN
c                   WRITE(Ulog,*) 'RECORD ',irec,kk,recA(kk),recB(kk)
                 ENDIF

                 IF (valA.NE.DataMissing) THEN
                    wordMinA(kk) = min ( wordMinA(kk), valA )
                    wordMaxA(kk) = max ( wordMaxA(kk), valA )
                 ENDIF
                 IF (valB.NE.DataMissing) THEN
                    wordMinB(kk) = min ( wordMinB(kk), valB )
                    wordMaxB(kk) = max ( wordMaxB(kk), valB )
                 ENDIF
                 IF (valA.NE.DataMissing .AND. valB.NE.DataMissing) THEN
                    wordDifMax(kk) = max ( wordDifMax(kk), diff )
                 ENDIF

                 idx = descIdx(kk)
                 desc = descDtxt(idx,iInst,iLev)
                 limit = descDwrd(idx,3,iInst,iLev)
                 item = 1 + kk - descDwrd(idx,1,iInst,iLev)
                 IF (valA.NE.DataMissing) THEN
                    dataMinA(idx) = min ( dataMinA(idx), valA )
                    dataMaxA(idx) = max ( dataMaxA(idx), valA )
                 ENDIF
                 IF (valB.NE.DataMissing) THEN
                    dataMinB(idx) = min ( dataMinB(idx), valB )
                    dataMaxB(idx) = max ( dataMaxB(idx), valB )
                 ENDIF
                 IF (valA.NE.DataMissing .AND.valB.NE.DataMissing) THEN
                    IF (diff.GT.0) 
     &              dataDifTot(idx) = dataDifTot(idx) + 1
                    IF (diff.GE.max(1,limit)) 
     &              dataDifCnt(idx) = dataDifCnt(idx) + 1
                    IF (diff.GT.dataDifMax(idx)) THEN
                       dataDifMax(idx) = diff
                       dataDifA(idx) = valA
                       dataDifB(idx) = valB
                       IF (idx.EQ.dbgIdx.AND.diff.NE.0) 
     &                 WRITE(6,*) desc,irec,item,' ==>',valA,valB,diff
                    ENDIF
                    dataInAB(idx) = dataInAB(idx) + 1
                 ELSEIF (valA.EQ.DataMissing .AND. 
     &                   valB.NE.DataMissing) THEN
                    dataOnlyB(idx) = dataOnlyB(idx) + 1
                    dataMissA(idx) = dataMissA(idx) + 1
                 ELSEIF (valB.EQ.DataMissing .AND. 
     &                   valA.NE.DataMissing) THEN
                    dataOnlyA(idx) = dataOnlyA(idx) + 1
                    dataMissB(idx) = dataMissB(idx) + 1
                 ELSE
                    dataMissAB(idx) = dataMissAB(idx) + 1
                 ENDIF

              ENDDO   ! recwords

              GOTO 999

 997          WRITE(6,*)
              WRITE(6,'(A,I4,A)')   ' atovsCompare WARNING: Record ',
     &              1+irec,' not present.'
              WRITE(6,'(A,I4,A,A)') 
     & '                       Expected recordcount: ',reccnt,' in ',
     &        fileA
              GOTO 1000
 998          WRITE(6,*)
              WRITE(6,'(A,I4,A)')   
     &         ' atovsCompare WARNING: Record ',1+irec,' not present.'
              WRITE(6,'(A,I4,A,A)') 
     & '                       Expected recordcount: ',reccnt,' in ',
     &        fileB
              GOTO 1000
 999          CONTINUE

           ENDDO      ! reccnt
!
!  Report differences in record words
!
 1000      prvIdx = 0
           WRITE(Ulog,*)
           DO kk = 1,recwords
              idx = descIdx(kk)
              desc = descDtxt(idx,iInst,iLev)
              int1  = descDwrd(idx,1,iInst,iLev)
              int2  = descDwrd(idx,2,iInst,iLev)
              limit = descDwrd(idx,3,iInst,iLev)
              IF (limit.GT.0) reportAll = .TRUE.
              report = (reportAny .AND. idx.NE.prvIdx .AND. reportAll)
              IF (report) THEN
                 WRITE(Ulog,*)
                 WRITE(Ulog,'(2I6,A18,A30,I6)') int1,int2,
     &                 ' -- I/O threshold ',desc(1:30),limit
                 makeSpace = .TRUE.
              ENDIF
              IF (reportAny .AND. wordDifMax(kk).GE.limit) THEN
                 IF (makeSpace) WRITE(Ulog,*)
                 WRITE(Ulog,'(I6,2X,2I12,2X,2I12,2X,I12,7X,A)')
     &                 kk,wordMinA(kk),wordMaxA(kk),wordMinB(kk),
     &                 wordMaxB(kk),wordDifMax(kk),desc
                 makeSpace = .FALSE.
              ENDIF
              prvIdx = idx
           ENDDO
!
!  Report data ranges
!
           WRITE(Ulog,*)
           WRITE(Ulog,*) 'Data ranges:'
           WRITE(Ulog,*) '============'
           WRITE(Ulog,*)
           WRITE(Ulog,'(A)')
     &     '        minA        maxA |      minB        maxB |  Descript
     &ion'
           WRITE(Ulog,'(A)')
     &     ' ------------------------|-----------------------|----------
     &----------------------'
           DO iD = 1,MaxDesc
              desc = descDtxt(iD,iInst,iLev)
              IF (desc.NE.'none') THEN
                 WRITE(Ulog,'(4I12,3X,A)') dataMinA(iD),dataMaxA(iD),
     &                 dataMinB(iD),dataMaxB(iD),desc
              ENDIF
           ENDDO
           cnt = reccnt
           IF (iInst.EQ.Amsua) cnt = cnt*30
           IF (iInst.EQ.Amsub) cnt = cnt*90
           IF (iInst.EQ.Hirs)  cnt = cnt*56
           WRITE(Ulog,*)
           WRITE(Ulog,*) 'Total IFOV count:',cnt
!
!  Report data differences
!
           WRITE(Ulog,*)
           WRITE(Ulog,*)
           WRITE(Ulog,*) 'Data differences (long format):'
           WRITE(Ulog,*) '==============================='
           WRITE(Ulog,*)
           WRITE(Ulog,'(A)')
     &     '  Data in both | Present      | # of different     | Max dif
     &ference                 | Description'
           WRITE(Ulog,'(A)')
     &     '  Miss Present | onlyA  onlyB | dif>0   diff>thres.|   valA 
     &       valB        diff |            '
           WRITE(Ulog,'(A,A)')
     &     ' --------------|--------------|--------------------|--------
     &------------------------|',
     &     '-------------------------------'
           DO iD = 1,MaxDesc
              diff = dataDifMax(iD)
              desc = descDtxt(iD,iInst,iLev)
              IF (desc.NE.'none') THEN
                 limit = max(1,descDwrd(iD,3,iInst,iLev))
                 IF (diff.GT.0) THEN
                    WRITE(Ulog,'(4I7,1X,2I7,I4,3I12,3X,A)')
     &              dataMissAB(iD),dataInAB(iD),dataOnlyA(iD),
     &              dataOnlyB(iD),dataDifTot(iD),dataDifCnt(iD),
     &              limit,dataDifA(iD),DataDifB(iD),diff,desc
                 ELSE
                    WRITE(Ulog,'(4I7,1X,2I7,I4,24X,I12,3X,A)')
     &              dataMissAB(iD),dataInAB(iD),dataOnlyA(iD),
     &              dataOnlyB(iD),dataDifTot(iD),dataDifCnt(iD),
     &              limit,diff,desc
                 ENDIF
              ENDIF
           ENDDO

           WRITE(Ulog,*)
           WRITE(Ulog,*)
           WRITE(Ulog,*) 'Data differences (short format):'
           WRITE(Ulog,*) '================================'
           WRITE(Ulog,*)
           WRITE(Ulog,'(A)')
     &     '  # of data in  | # diff. | Max difference                 |
     & Description'
           WRITE(Ulog,'(A)')
     &     '  onlyA  onlyB  | dif>0   |   valA        valB        diff |
     &            '
           WRITE(Ulog,'(A)')
     &     ' ---------------|---------|--------------------------------|
     &-------------------------------'
           DO iD = 1,MaxDesc
              diff = dataDifMax(iD)
              desc = descDtxt(iD,iInst,iLev)
              IF (desc.NE.'none') THEN
                 limit = max(1,descDwrd(iD,3,iInst,iLev))
                 IF (diff.GT.0) THEN
                    WRITE(Ulog,'(2I7,1X,I7,3I12,3X,A)')
     &              dataOnlyA(iD),dataOnlyB(iD),dataDifTot(iD),
     &              dataDifA(iD),DataDifB(iD),diff,desc
                 ELSE
                    WRITE(Ulog,'(2I7,1X,I7,24X,I12,3X,A)')
     &              dataOnlyA(iD),dataOnlyB(iD),dataDifTot(iD),
     &              diff,desc
                 ENDIF
              ENDIF
           ENDDO

           CLOSE(UdatA)
           CLOSE(UdatB)
           CLOSE(Ulog)

        ENDIF           ! IF (compare)

        ENDDO           ! DO iInst
        ENDDO           ! DO iLev

        WRITE(6,*)
        IF (.NOT.haveInput) WRITE(6,*) 'No comparable files present...'
        IF (remindMe) THEN
           write(6,*) 'Found 0 as missing value ...'
        ENDIF

        END

C==========================================================

        SUBROUTINE setDescriptions ( satID, txtH, intH, txtD, intD )

        INTEGER    MaxInst, MaxLevel, MaxDesc
        PARAMETER  ( MaxInst = 3 )
        PARAMETER  ( MaxLevel = 3 )
        PARAMETER  ( MaxDesc = 50 )
        INTEGER    Hirs,AmsuB,AmsuA
        PARAMETER  ( AmsuA = 1 )
        PARAMETER  ( AmsuB = 2 )
        PARAMETER  ( Hirs = 3 )
        INTEGER    Lev1b,Lev1c,Lev1d
        PARAMETER  ( Lev1b = 1 )
        PARAMETER  ( Lev1c = 2 )
        PARAMETER  ( Lev1d = 3 )
        INTEGER    TempThreshold
        PARAMETER  ( TempThreshold = 5 )

        INTEGER*4       satID
        CHARACTER*40    txtH(MaxDesc,MaxInst,MaxLevel)     ! Header descriptions
        INTEGER*4       intH(MaxDesc,3,MaxInst,MaxLevel)   ! Header description range
        CHARACTER*40    txtD(MaxDesc,MaxInst,MaxLevel)     ! Data   descriptions
        INTEGER*4       intD(MaxDesc,3,MaxInst,MaxLevel)   ! Data   description range

        INTEGER   iD
!
! Missing descriptions
!
        DO iL = Lev1c,Lev1d
        DO iI = AmsuA,Hirs
           iD = 1 
           txtH(iD,  iI,iL) = 'Miscellaneous...'
           intH(iD,1,iI,iL) = 0
           intH(iD,2,iI,iL) = 0
           intH(iD,3,iI,iL) = 0
           txtD(iD,  iI,iL) = 'Miscellaneous...'
           intD(iD,1,iI,iL) = 0
           intD(iD,2,iI,iL) = 0
           intD(iD,3,iI,iL) = 0
           DO iD = 2,MaxDesc
              txtH(iD,iI,iL) = 'none'
              txtD(iD,iI,iL) = 'none'
           ENDDO
        ENDDO
        ENDDO
!
!=========
! Level 1c
!=========
!
        iL = Lev1c
!
! Lev1c header records
!
        DO iI = AmsuA,Hirs
           iD = 2 
           txtH(iD,  iI,iL) = 'Site ID (ASCII)'
           intH(iD,1,iI,iL) = 1
           intH(iD,2,iI,iL) = 2
           intH(iD,3,iI,iL) = 0
           iD = iD + 1
           txtH(iD,  iI,iL) = 'Start of data set UTC time of day (ms)'
           intH(iD,1,iI,iL) = 14
           intH(iD,2,iI,iL) = 14
           intH(iD,3,iI,iL) = 0
           iD = iD + 1
           txtH(iD,  iI,iL) = 'End   of data set UTC time of day (ms)'
           intH(iD,1,iI,iL) = 18
           intH(iD,2,iI,iL) = 18
           intH(iD,3,iI,iL) = 0
        ENDDO
!
! Lev1c AmsuA data record
!
        iI = AmsuA
        iD = 2 
        txtD(iD,  AmsuA,iL) = 'Scan line UTC time of day (ms)'
        intD(iD,1,AmsuA,iL) = 4
        intD(iD,2,AmsuA,iL) = 4
        intD(iD,3,AmsuA,iL) = 0
        iD = iD + 1
        txtD(iD,  AmsuA,iL) = 'Quality indicator'
        intD(iD,1,AmsuA,iL) = 5
        intD(iD,2,AmsuA,iL) = 5
        intD(iD,3,AmsuA,iL) = 0
        iD = iD + 1
        txtD(iD,  AmsuA,iL) = 'Scan line quality flags'
        intD(iD,1,AmsuA,iL) = 6
        intD(iD,2,AmsuA,iL) = 6
        intD(iD,3,AmsuA,iL) = 0
        iD = iD + 1
        txtD(iD,  AmsuA,iL) = 'Channel quality flags'
        intD(iD,1,AmsuA,iL) = 7
        intD(iD,2,AmsuA,iL) = 21
        intD(iD,3,AmsuA,iL) = 0
        iD = iD + 1
        txtD(iD,  AmsuA,iL) = 'RF shelf temp'
        intD(iD,1,AmsuA,iL) = 22
        intD(iD,2,AmsuA,iL) = 23
        intD(iD,3,AmsuA,iL) = 0
        iD = iD + 1
        txtD(iD,  AmsuA,iL) = 'Lat/Lon (degree*10000)'
        intD(iD,1,AmsuA,iL) = 26
        intD(iD,2,AmsuA,iL) = 85
        intD(iD,3,AmsuA,iL) = 1
        iD = iD + 1
        txtD(iD,  AmsuA,iL) = 'Scan angles (degree*100)'
        intD(iD,1,AmsuA,iL) = 86
        intD(iD,2,AmsuA,iL) = 205
        intD(iD,3,AmsuA,iL) = 1
        iD = iD + 1
        txtD(iD,  AmsuA,iL) = 'Sat altitude (km*10)'
        intD(iD,1,AmsuA,iL) = 206
        intD(iD,2,AmsuA,iL) = 206
        intD(iD,3,AmsuA,iL) = 0
        iD = iD + 1
        txtD(iD,  AmsuA,iL) = 'Brightness temperature (K*100)'
        intD(iD,1,AmsuA,iL) = 209
        intD(iD,2,AmsuA,iL) = 658
        intD(iD,3,AmsuA,iL) = TempThreshold
        iD = iD + 1
        txtD(iD,  AmsuA,iL) = 'IFOV quality control'
        intD(iD,1,AmsuA,iL) = 659
        intD(iD,2,AmsuA,iL) = 688
        intD(iD,3,AmsuA,iL) = 1
        iD = iD + 1
        txtD(iD,  AmsuA,iL) = 'Filler'
        intD(iD,1,AmsuA,iL) = 689
        intD(iD,2,AmsuA,iL) = 768
        intD(iD,3,AmsuA,iL) = 1
!
! Lev1c AmsuB data record
!
        iD = 2
        txtD(iD,  AmsuB,iL) = 'Scan line UTC time of day (ms)'
        intD(iD,1,AmsuB,iL) = 4
        intD(iD,2,AmsuB,iL) = 4
        intD(iD,3,AmsuB,iL) = 0
        iD = iD + 1
        txtD(iD,  AmsuB,iL) = 'Quality indicator'
        intD(iD,1,AmsuB,iL) = 5
        intD(iD,2,AmsuB,iL) = 5
        intD(iD,3,AmsuB,iL) = 0
        iD = iD + 1
        txtD(iD,  AmsuB,iL) = 'Scan line quality flags'
        intD(iD,1,AmsuB,iL) = 6
        intD(iD,2,AmsuB,iL) = 6
        intD(iD,3,AmsuB,iL) = 0
        iD = iD + 1
        txtD(iD,  AmsuB,iL) = 'Channel quality flags'
        intD(iD,1,AmsuB,iL) = 7
        intD(iD,2,AmsuB,iL) = 11
        intD(iD,3,AmsuB,iL) = 0
        iD = iD + 1
        txtD(iD,  AmsuB,iL) = 'AMSU-B Mixer chan 18-20 temp'
        intD(iD,1,AmsuB,iL) = 12
        intD(iD,2,AmsuB,iL) = 12
        intD(iD,3,AmsuB,iL) = 0
        iD = iD + 1
        txtD(iD,  AmsuB,iL) = 'Lat/Lon (degree*10000)'
        intD(iD,1,AmsuB,iL) = 15
        intD(iD,2,AmsuB,iL) = 194
        intD(iD,3,AmsuB,iL) = 1
        iD = iD + 1
        txtD(iD,  AmsuB,iL) = 'Scan angles (degree*100)'
        intD(iD,1,AmsuB,iL) = 195
        intD(iD,2,AmsuB,iL) = 554
        intD(iD,3,AmsuB,iL) = 1
        iD = iD + 1
        txtD(iD,  AmsuB,iL) = 'Sat altitude (km*10)'
        intD(iD,1,AmsuB,iL) = 555
        intD(iD,2,AmsuB,iL) = 555
        intD(iD,3,AmsuB,iL) = 0
        iD = iD + 1
        txtD(iD,  AmsuB,iL) = 'Brightness temperature (K*100)'
        intD(iD,1,AmsuB,iL) = 558
        intD(iD,2,AmsuB,iL) = 1007
        intD(iD,3,AmsuB,iL) = TempThreshold
        iD = iD + 1
        txtD(iD,  AmsuB,iL) = 'IFOV quality control'
        intD(iD,1,AmsuB,iL) = 1008
        intD(iD,2,AmsuB,iL) = 1097
        intD(iD,3,AmsuB,iL) = 1
        iD = iD + 1
        txtD(iD,  AmsuB,iL) = 'Filler'
        intD(iD,1,AmsuB,iL) = 1098
        intD(iD,2,AmsuB,iL) = 1152
        intD(iD,3,AmsuB,iL) = 1
!
! Lev1c Hirs data record
!
        iD = 2 
        txtD(iD,  Hirs ,iL) = 'Scan line UTC time of day (ms)'
        intD(iD,1,Hirs ,Lev1c) = 4
        intD(iD,2,Hirs ,Lev1c) = 4
        intD(iD,3,Hirs ,Lev1c) = 0
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1c) = 'Major frame count'
        intD(iD,1,Hirs ,Lev1c) = 5
        intD(iD,2,Hirs ,Lev1c) = 5
        intD(iD,3,Hirs ,Lev1c) = 0
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1c) = 'Quality indicator'
        intD(iD,1,Hirs ,Lev1c) = 6
        intD(iD,2,Hirs ,Lev1c) = 6
        intD(iD,3,Hirs ,Lev1c) = 0
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1c) = 'Scan line quality flags'
        intD(iD,1,Hirs ,Lev1c) = 7
        intD(iD,2,Hirs ,Lev1c) = 7
        intD(iD,3,Hirs ,Lev1c) = 0
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1c) = 'Channel quality flags'
        intD(iD,1,Hirs ,Lev1c) = 8
        intD(iD,2,Hirs ,Lev1c) = 27
        intD(iD,3,Hirs ,Lev1c) = 0
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1c) = 'HIRS baseplate temperature (K*100)'
        intD(iD,1,Hirs ,Lev1c) = 28
        intD(iD,2,Hirs ,Lev1c) = 28
        intD(iD,3,Hirs ,Lev1c) = 0
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1c) = 'Lat/Lon (degree*10000)'
        intD(iD,1,Hirs ,Lev1c) = 31
        intD(iD,2,Hirs ,Lev1c) = 142
        intD(iD,3,Hirs ,Lev1c) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1c) = 'Scan angles (degree*100)'
        intD(iD,1,Hirs ,Lev1c) = 143
        intD(iD,2,Hirs ,Lev1c) = 366
        intD(iD,3,Hirs ,Lev1c) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1c) = 'Sat altitude (km*10)'
        intD(iD,1,Hirs ,Lev1c) = 367
        intD(iD,2,Hirs ,Lev1c) = 367
        intD(iD,3,Hirs ,Lev1c) = 0
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1c) = 'Brightness temperature (K*100)'
        intD(iD,1,Hirs ,Lev1c) = 370
        intD(iD,2,Hirs ,Lev1c) = 1489
        intD(iD,3,Hirs ,Lev1c) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1c) = 'IFOV quality control'
        intD(iD,1,Hirs ,Lev1c) = 1490
        intD(iD,2,Hirs ,Lev1c) = 1545
        intD(iD,3,Hirs ,Lev1c) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1c) = 'Filler'
        intD(iD,1,Hirs ,Lev1c) = 1546
        intD(iD,2,Hirs ,Lev1c) = 1664
        intD(iD,3,Hirs ,Lev1c) = 1
!
!=========
! Level 1d
!=========
!
        iL = Lev1d
!
! Lev1d header records
!
        DO iI = Hirs,Hirs
           iD = 2 
           txtH(iD,  iI,iL) = 'Site ID (ASCII)'
           intH(iD,1,iI,iL) = 1
           intH(iD,2,iI,iL) = 2
           intH(iD,3,iI,iL) = 0
           iD = iD + 1
           txtH(iD,  iI,iL) = 'Start of data set UTC time of day (ms)'
           intH(iD,1,iI,iL) = 14
           intH(iD,2,iI,iL) = 14
           intH(iD,3,iI,iL) = 0
           iD = iD + 1
           txtH(iD,  iI,iL) = 'End   of data set UTC time of day (ms)'
           intH(iD,1,iI,iL) = 18
           intH(iD,2,iI,iL) = 18
           intH(iD,3,iI,iL) = 0
           iD = iD + 1 
           txtH(iD,  iI,iL) = 'Instruments present'
           intH(iD,1,iI,iL) = 22
           intH(iD,2,iI,iL) = 22
           intH(iD,3,iI,iL) = 0
        ENDDO
!
! Lev1d Hirs data record (NOAA-15 and later)
!
        IF (satID.GE.15 .OR. satID.LE.3) THEN
        iL = Lev1d
        iI = Hirs
        iD = 2 
        txtD(iD,  Hirs ,iL) = 'Scan line UTC time of day (ms)'
        intD(iD,1,Hirs ,iL) = 4
        intD(iD,2,Hirs ,iL) = 4
        intD(iD,3,Hirs ,iL) = 0
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Quality indicator'
        intD(iD,1,Hirs ,Lev1d) = 5
        intD(iD,2,Hirs ,Lev1d) = 5
        intD(iD,3,Hirs ,Lev1d) = 0
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Scan line quality flags'
        intD(iD,1,Hirs ,Lev1d) = 6
        intD(iD,2,Hirs ,Lev1d) = 6
        intD(iD,3,Hirs ,Lev1d) = 0
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Lat/Lon (degree*10000)'
        intD(iD,1,Hirs ,Lev1d) = 13
        intD(iD,2,Hirs ,Lev1d) = 124
        intD(iD,3,Hirs ,Lev1d) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Surface height (m)'
        intD(iD,1,Hirs ,Lev1d) = 125
        intD(iD,2,Hirs ,Lev1d) = 180
        intD(iD,3,Hirs ,Lev1d) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Surface type (0/1/2)'
        intD(iD,1,Hirs ,Lev1d) = 181
        intD(iD,2,Hirs ,Lev1d) = 236
        intD(iD,3,Hirs ,Lev1d) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Scan angles (degree*100)'
        intD(iD,1,Hirs ,Lev1d) = 237
        intD(iD,2,Hirs ,Lev1d) = 460
        intD(iD,3,Hirs ,Lev1d) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Sat altitude (km*10)'
        intD(iD,1,Hirs ,Lev1d) = 461
        intD(iD,2,Hirs ,Lev1d) = 461
        intD(iD,3,Hirs ,Lev1d) = 0
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Hirs BT (K*100)'
        intD(iD,1,Hirs ,Lev1d) = 464
        intD(iD,2,Hirs ,Lev1d) = 1527
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Hirs ch 20 radiance (*100)'
        intD(iD,1,Hirs ,Lev1d) = 1528
        intD(iD,2,Hirs ,Lev1d) = 1583
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Amsua BT (K*100)'
        intD(iD,1,Hirs ,Lev1d) = 1584
        intD(iD,2,Hirs ,Lev1d) = 2423
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Amsub BT (K*100)'
        intD(iD,1,Hirs ,Lev1d) = 2424
        intD(iD,2,Hirs ,Lev1d) = 2703
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'AVHRR clear fraction (%)'
        intD(iD,1,Hirs ,Lev1d) = 2704
        intD(iD,2,Hirs ,Lev1d) = 2759
        intD(iD,3,Hirs ,Lev1d) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Surface temperature (K*100)'
        intD(iD,1,Hirs ,Lev1d) = 2760
        intD(iD,2,Hirs ,Lev1d) = 2815
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Clim. temperature or t2m (K*100)'
        intD(iD,1,Hirs ,Lev1d) = 2816
        intD(iD,2,Hirs ,Lev1d) = 2871
        intD(iD,3,Hirs ,Lev1d) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'AVHRR ch 3 (% / K*100)'
        intD(iD,1,Hirs ,Lev1d) = 2872
        intD(iD,2,Hirs ,Lev1d) = 2927
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'AVHRR ch 4/5 (K*100)'
        intD(iD,1,Hirs ,Lev1d) = 2928
        intD(iD,2,Hirs ,Lev1d) = 3039
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Percentage of black body (%)'
        intD(iD,1,Hirs ,Lev1d) = 3040
        intD(iD,2,Hirs ,Lev1d) = 3095
        intD(iD,3,Hirs ,Lev1d) = 2
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'CTT (K*100)'
        intD(iD,1,Hirs ,Lev1d) = 3096
        intD(iD,2,Hirs ,Lev1d) = 3151
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'CTT std (K*100)'
        intD(iD,1,Hirs ,Lev1d) = 3152
        intD(iD,2,Hirs ,Lev1d) = 3207
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'AVHRR clear ch 4/5 (K*100)'
        intD(iD,1,Hirs ,Lev1d) = 3208
        intD(iD,2,Hirs ,Lev1d) = 3319
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'AVHRR std ch 4 (K*100)'
        intD(iD,1,Hirs ,Lev1d) = 3320
        intD(iD,2,Hirs ,Lev1d) = 3375
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'AVHRR std clear ch 4 (K*100)'
        intD(iD,1,Hirs ,Lev1d) = 3376
        intD(iD,2,Hirs ,Lev1d) = 3431
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'IFOV HIRS quality control'
        intD(iD,1,Hirs ,Lev1d) = 3432
        intD(iD,2,Hirs ,Lev1d) = 3487
        intD(iD,3,Hirs ,Lev1d) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'PP quality control'
        intD(iD,1,Hirs ,Lev1d) = 3488
        intD(iD,2,Hirs ,Lev1d) = 3543
        intD(iD,3,Hirs ,Lev1d) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'PP surface type (1-9)'
        intD(iD,1,Hirs ,Lev1d) = 3544
        intD(iD,2,Hirs ,Lev1d) = 3599
        intD(iD,3,Hirs ,Lev1d) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'PP cost function'
        intD(iD,1,Hirs ,Lev1d) = 3600
        intD(iD,2,Hirs ,Lev1d) = 3655
        intD(iD,3,Hirs ,Lev1d) = 50
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'PP scattering index'
        intD(iD,1,Hirs ,Lev1d) = 3656
        intD(iD,2,Hirs ,Lev1d) = 3711
        intD(iD,3,Hirs ,Lev1d) = 3
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'PP precipitation probability (%)'
        intD(iD,1,Hirs ,Lev1d) = 3712
        intD(iD,2,Hirs ,Lev1d) = 3767
        intD(iD,3,Hirs ,Lev1d) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'PP spare'
        intD(iD,1,Hirs ,Lev1d) = 3768
        intD(iD,2,Hirs ,Lev1d) = 3879
        intD(iD,3,Hirs ,Lev1d) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Filler'
        intD(iD,1,Hirs ,Lev1d) = 3880
        intD(iD,2,Hirs ,Lev1d) = 3968
        intD(iD,3,Hirs ,Lev1d) = 1
!
! Lev1d Hirs data record (NOAA-14 and earlier)
!
        ELSE
        iL = Lev1d
        iI = Hirs
        iD = 2
        txtD(iD,  Hirs ,iL) = 'Scan line UTC time of day (ms)'
        intD(iD,1,Hirs ,iL) = 4
        intD(iD,2,Hirs ,iL) = 4
        intD(iD,3,Hirs ,iL) = 0
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Quality indicator'
        intD(iD,1,Hirs ,Lev1d) = 5
        intD(iD,2,Hirs ,Lev1d) = 5
        intD(iD,3,Hirs ,Lev1d) = 0
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Scan line quality flags'
        intD(iD,1,Hirs ,Lev1d) = 6
        intD(iD,2,Hirs ,Lev1d) = 6
        intD(iD,3,Hirs ,Lev1d) = 0
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Lat/Lon (degree*10000)'
        intD(iD,1,Hirs ,Lev1d) = 9
        intD(iD,2,Hirs ,Lev1d) = 120
        intD(iD,3,Hirs ,Lev1d) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Surface height (m)'
        intD(iD,1,Hirs ,Lev1d) = 121
        intD(iD,2,Hirs ,Lev1d) = 176
        intD(iD,3,Hirs ,Lev1d) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Surface type (0/1/2)'
        intD(iD,1,Hirs ,Lev1d) = 177
        intD(iD,2,Hirs ,Lev1d) = 232
        intD(iD,3,Hirs ,Lev1d) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Scan angles (degree*100)'
        intD(iD,1,Hirs ,Lev1d) = 233
        intD(iD,2,Hirs ,Lev1d) = 456
        intD(iD,3,Hirs ,Lev1d) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Sat altitude (km*10)'
        intD(iD,1,Hirs ,Lev1d) = 457
        intD(iD,2,Hirs ,Lev1d) = 457
        intD(iD,3,Hirs ,Lev1d) = 0
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Hirs BT (K*100)'
        intD(iD,1,Hirs ,Lev1d) = 459
        intD(iD,2,Hirs ,Lev1d) = 1522
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Hirs ch 20 radiance (*100)'
        intD(iD,1,Hirs ,Lev1d) = 1523
        intD(iD,2,Hirs ,Lev1d) = 1578
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'MSU BT (K*100)'
        intD(iD,1,Hirs ,Lev1d) = 1579
        intD(iD,2,Hirs ,Lev1d) = 1802
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'AVHRR clear fraction (%)'
        intD(iD,1,Hirs ,Lev1d) = 1803
        intD(iD,2,Hirs ,Lev1d) = 1858
        intD(iD,3,Hirs ,Lev1d) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Surface temperature (K*100)'
        intD(iD,1,Hirs ,Lev1d) = 1859
        intD(iD,2,Hirs ,Lev1d) = 1914
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Clim. temperature or t2m (K*100)'
        intD(iD,1,Hirs ,Lev1d) = 1915
        intD(iD,2,Hirs ,Lev1d) = 1970
        intD(iD,3,Hirs ,Lev1d) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'AVHRR ch 3 (% / K*100)'
        intD(iD,1,Hirs ,Lev1d) = 1971
        intD(iD,2,Hirs ,Lev1d) = 2026
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'AVHRR ch 4/5 (K*100)'
        intD(iD,1,Hirs ,Lev1d) = 2027
        intD(iD,2,Hirs ,Lev1d) = 2138
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Percentage of black body (%)'
        intD(iD,1,Hirs ,Lev1d) = 2139
        intD(iD,2,Hirs ,Lev1d) = 2194
        intD(iD,3,Hirs ,Lev1d) = 2
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'CTT (K*100)'
        intD(iD,1,Hirs ,Lev1d) = 2195
        intD(iD,2,Hirs ,Lev1d) = 2250
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'CTT std (K*100)'
        intD(iD,1,Hirs ,Lev1d) = 2251
        intD(iD,2,Hirs ,Lev1d) = 2306
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'AVHRR clear ch 4/5 (K*100)'
        intD(iD,1,Hirs ,Lev1d) = 2307
        intD(iD,2,Hirs ,Lev1d) = 2418
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'AVHRR std ch 4 (K*100)'
        intD(iD,1,Hirs ,Lev1d) = 2419
        intD(iD,2,Hirs ,Lev1d) = 2474
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'AVHRR std clear ch 4 (K*100)'
        intD(iD,1,Hirs ,Lev1d) = 2475
        intD(iD,2,Hirs ,Lev1d) = 2530
        intD(iD,3,Hirs ,Lev1d) = TempThreshold
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'IFOV HIRS quality control'
        intD(iD,1,Hirs ,Lev1d) = 2531
        intD(iD,2,Hirs ,Lev1d) = 2586
        intD(iD,3,Hirs ,Lev1d) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'PP quality control'
        intD(iD,1,Hirs ,Lev1d) = 2587
        intD(iD,2,Hirs ,Lev1d) = 2642
        intD(iD,3,Hirs ,Lev1d) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'PP spare'
        intD(iD,1,Hirs ,Lev1d) = 2643
        intD(iD,2,Hirs ,Lev1d) = 2810
        intD(iD,3,Hirs ,Lev1d) = 1
        iD = iD + 1
        txtD(iD,  Hirs ,Lev1d) = 'Filler'
        intD(iD,1,Hirs ,Lev1d) = 2811
        intD(iD,2,Hirs ,Lev1d) = 2818
        intD(iD,3,Hirs ,Lev1d) = 1
        ENDIF

        RETURN
        END

C==========================================================

        SUBROUTINE reWrite(rec,n)

        INTEGER*4        n
        INTEGER*4        rec(n)
        INTEGER*4        word
        CHARACTER*1      bytes(4),ctmp
        INTEGER*4        kk
        
        EQUIVALENCE        (word,bytes)

        DO kk = 1,n
           word = rec(kk) 
           ctmp = bytes(1)
           bytes(1) = bytes(4)
           bytes(4) = ctmp
           ctmp = bytes(2)
           bytes(2) = bytes(3)
           bytes(3) = ctmp
           rec(kk) = word
        ENDDO
 
        RETURN
        END

C==========================================================

        SUBROUTINE reArrange(rec,satID)

        INTEGER   satID
        INTEGER   rec(*)

        IF (satID.GE.15 .OR. satID.LE.3) THEN
           CALL reArrangeSection(rec,125,2)    ! Re-arrange hrs1d_surface(2,Hnfovs)
           CALL reArrangeSection(rec,464,40)   ! Re-arrange hrs1d_btemps(40,Hnfovs)
           CALL reArrangeSection(rec,2704,13)  ! Re-arrange hrs1d_avhrr(13,Hnfovs)
           CALL reArrangeSection(rec,3488,7)   ! Re-arrange hrs1d_prepro(7,Hnfovs)
        ELSE
           CALL reArrangeSection(rec,121,2)    ! Re-arrange hrs1d_surface(2,Hnfovs)
           CALL reArrangeSection(rec,459,24)   ! Re-arrange hrs1d_btemps(24,Hnfovs)
           CALL reArrangeSection(rec,1803,13)  ! Re-arrange hrs1d_avhrr(13,Hnfovs)
           CALL reArrangeSection(rec,2587,4)   ! Re-arrange hrs1d_prepro(4,Hnfovs)
        ENDIF

        RETURN
        END

C==========================================================

        SUBROUTINE reArrangeSection(rec,startIdx,dataCnt)

        INTEGER   rec(*)
        INTEGER   startIdx
        INTEGER   dataCnt

        INTEGER   iW,iFov,iDat
        INTEGER   data(56,40)

        DO iFov = 1,56
        DO iDat = 1,dataCnt
           iW = startIdx + (iFov-1)*dataCnt + (iDat-1)
           data(iFov,iDat) = rec(iW)
        ENDDO
        ENDDO

        DO iFov = 1,56
        DO iDat = 1,dataCnt
           iW = startIdx + (iDat-1)*56 + (iFov-1)
           rec(iW) = data(iFov,iDat)
        ENDDO
        ENDDO

        RETURN
        END

C==========================================================

        SUBROUTINE checkMissing(rec,n,satID,iInst,iLev,remindMe)

        INTEGER   n,satID,iInst,iLev
        INTEGER   rec(n)
        LOGICAL   remindMe

        INTEGER    Hirs,AmsuB,AmsuA
        PARAMETER  ( AmsuA = 1 )
        PARAMETER  ( AmsuB = 2 )
        PARAMETER  ( Hirs = 3 )
        INTEGER    Lev1b,Lev1c,Lev1d
        PARAMETER  ( Lev1b = 1 )
        PARAMETER  ( Lev1c = 2 )
        PARAMETER  ( Lev1d = 3 )
        INTEGER    DataMissing
        PARAMETER  (DataMissing = -999999)

        INTEGER   iW
!
! Change -9999
!
        DO iW = 1,n
           IF (rec(iw).EQ.-9999) rec(iW) = DataMissing
        ENDDO
!
! Change 0 from  CTT / CTT std / AVHRR clear ch 4/5 / AVHRR std ch 4 / AVHRR std clear ch 4
!
        IF (iInst.EQ.Hirs .AND. iLev.EQ.Lev1d) THEN
           IF (satID.GE.15 .OR. satID.LE.3) THEN
              DO iW = 3096,3431
                 IF (rec(iw).EQ.0) remindMe = .TRUE.
                 IF (rec(iw).EQ.0) rec(iW) = DataMissing
              ENDDO
           ELSE
              DO iW = 2195,2530
                 IF (rec(iw).EQ.0) remindMe = .TRUE.
                 IF (rec(iw).EQ.0) rec(iW) = DataMissing
              ENDDO
           ENDIF
        ENDIF

        RETURN
        END

C==========================================================

        SUBROUTINE FindDescription ( txt, range, word, idx )

        INTEGER    MaxDesc
        PARAMETER  ( MaxDesc = 50 )

        CHARACTER*40    txt(MaxDesc)       ! Header descriptions
        INTEGER*4       range(MaxDesc,3)   ! Header description range
        INTEGER*4       word
        INTEGER*4       idx

        idx = 1
        DO iD = 1,MaxDesc
           IF (range(iD,1).LE.word .AND. word.LE.range(iD,2)) idx = iD
        ENDDO

        RETURN
        END

C==========================================================
