/* 
! Name: $Id$
!
! Purpose: Top-level C program to convert AVHRR level 1b AAPP format to HDF5
!
! Method: Read the whole AVHRR 1b file into memory. Convert raw counts to scaled
!   radiance and reflectivities. Write out as HDF5.
!   Interfaces with HDF5 Release 1.8, see http://www.hdfgroup.org/HDF5/
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2008, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0      17/11/08  Original code. (N.C.Atkinson)
!
! Code Description: C
!
! C subroutines and functions called:
!     avh1b_to_hdf5_intf
!
* LIBS = aapphdf5
!-------------------------------------------------------------------------------
*/

#include <stdio.h>
#include <stdlib.h>

int main( int argc, char* argv[] ) {

#ifdef HAS_LIBHDF5
#include "aapp_hdf5.h"

  if (argc != 3){
    printf("Usage: avh1b_to_hdf.exe infile outfile\n");
    exit(1);
  }

  avh1b_to_hdf5_intf (argc, argv);
  
  return 0;

#else

  printf("avh1b_to_hdf5.exe has not been built with HDF-5\n");
  printf("Please run \"configure\" with --external-libs=\"-Lxxx/lib -lhdf5\"\n");
  printf("  and --external-includes=\"-Lxxx/include\"\n");
  printf("  where xxx is the location of the HDF-5 library\n");
  return 1;
  
#endif

}
