C LIBS = f7nl1b f7ml f7tp f7cp f7gp auxaapp
! 
      PROGRAM APPLY_VIS_COEFS
      IMPLICIT NONE
!----------------------------------------------------------------------------
! Name: $Id: avh_get_vis_coefs.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose: Insert the operational AVHRR visible coefficients from NOAA
!   into an AAPP-format level 1b file
!
! Inputs (read from std input):
!   1b file name
!   Number of channels
!   For each channel: channel name, 4 coefficients, calibration year and day 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2009, EUMETSAT, All Rights Reserved.
!
! Version:
! 1.0  01/04/2009  N C Atkinson
! 1.1  10/10/2011  Avoid integer overflow in the low-gain slope by storing it
!                  as an unsigned integer.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:

#include "avh1b.h"
#include "reclopt.h"    

      character*200 avhrrfile
      integer nchannels,i,j,iscan,irec
      character*2 channel
      real viscoefs_tmp(4)
      real solarcalyr
      real solarcaldy
      real viscoefs(4,avh_mxvischn)
      integer loc_rl
      integer ios
      integer ichan
!
!- End of header

      do ichan=1,avh_mxvischn
        do j=1,4
          viscoefs(j,ichan) = 0.0
        enddo
      enddo
      
!Open the 1b file

      read(*,'(A)') avhrrfile
      write(*,*) avhrrfile

      loc_rl=reclavh/rldiv
      open(1,file=avhrrfile,form='unformatted',access='direct',
     &     recl=loc_rl,status='old',iostat=ios)
      if(ios.ne.0) then
        write(*,*)'AVHRR input file could not be opened'
        stop
      endif
      
!Read the header

      call avhhdr(1,ios)
      if(ios.ne.0) then
        write(*,*)'AVHRR header could not be read'
        stop
      endif

!Read the new coefficients

      read(*,*) nchannels
      do i=1,nchannels
        read(*,*) channel,viscoefs_tmp,solarcalyr,solarcaldy
        READ(channel,'(I1)') ichan
        write(*,*) ichan,viscoefs_tmp,solarcalyr,solarcaldy
        do j=1,4
          viscoefs(j,ichan) = viscoefs_tmp(j)
        enddo
      enddo
      
!Go through the data records, updating operational (first) calibration set

      do iscan=1,avh_h_scnlin
        call avhdtr(1,iscan,ios)
        if(ios.ne.0) then
          write(*,*)'Error reading AVHRR scan',iscan
          stop
        endif
        do ichan=1,avh_mxvischn
          if (viscoefs(1,ichan) .ne. 0.0) then
            avh_calvis(1,1,ichan) = NINT(viscoefs(1,ichan)*1E10) !Slope
            avh_calvis(2,1,ichan) = NINT(viscoefs(2,ichan)*1E7)  !Intercept
!Make sure the low-gain slope will not exceed size of 4-byte integer
            IF (viscoefs(3,ichan) .LT. 0.2147483648) THEN
              avh_calvis(3,1,ichan) = NINT(viscoefs(3,ichan)*1E10) !Slope
            ELSE
              avh_calvis(3,1,ichan) = 
     &            NINT((viscoefs(3,ichan)-0.4294967296)*1E10)
            ENDIF
            avh_calvis(4,1,ichan) = NINT(viscoefs(4,ichan)*1E7)  !Intercept
            avh_calvis(5,1,ichan) = 
     &         NINT((viscoefs(2,ichan)-viscoefs(4,ichan)) /
     &              (viscoefs(3,ichan)-viscoefs(1,ichan)))       !Intersection
          endif
        enddo
        call avhdtw(1,iscan,ios)
        if(ios.ne.0) then
          write(*,*)'Error writing AVHRR scan',iscan
          stop
        endif
      enddo
      
!Update the header

      avh_h_solarcalyr = solarcalyr
      avh_h_solarcaldy = solarcaldy
      call avhhdw(1,ios)
      if(ios.ne.0) then
        write(*,*)'Error updating AVHRR header'
        stop
      endif

      END
