C LIBS = decom satid f7nl1b f7ml f7tp f7cp f7gp

        program avhrr_1b_subset
        implicit none
!----------------------------------------------------------------------------
! Name: $Id: avhrr_1b_subset.F 487 2016-04-14 13:21:14Z frna $
!
! Purpose: a tool to output part of an AVHRR l1b file, based on lat/lon range
!   
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2016, EUMETSAT, All Rights Reserved.
!
! Version:
! 1.0  14/04/2016  N C Atkinson
!
!------------------------------------------------------------------------------

! includes/commons
#include "avh1b.h"
#include "reclopt.h"

! variables
      integer luavh_in          ! logical unit for input l1b file
      integer luavh_out         ! logical unit for output l1b file
      character*256 filename_in  ! input file name
      character*256 filename_out ! output file name
      real lat1, lat2, latmin, latmax, lat
      integer direction, direction_scan
      integer loc_rl
      integer iscan, scan_out
      integer ios

!--END OF DECLARATIONS----------------------------------------------------------

      luavh_in=30
      luavh_out=31
      loc_rl = reclavh /rldiv

! get requirements from standard input
      read (*,'(a)') filename_in
      read (*,'(a)') filename_out
      read (*,*) lat1
      read (*,*) lat2
      read (*,*) direction
      latmin = min(lat1, lat2)
      latmax = max(lat1, lat2)

      write(*,*) filename_in

! open the input file
      open(unit=luavh_in,
     &       file   = filename_in,
     &       access = 'DIRECT',
     &       recl   = loc_rl,
     &       form   = 'UNFORMATTED',
     &       status = 'OLD',
     &       iostat = ios)

      if (ios.ne.0) then
        write(*,*) 'Problem opening input file'
        stop
      endif

! open the output file
      open(unit=luavh_out,
     &       file   = filename_out,
     &       access = 'DIRECT',
     &       recl   = loc_rl,
     &       form   = 'UNFORMATTED',
     &       status = 'NEW',
     &       iostat = ios)

      if (ios.ne.0) then
        write(*,*) 'Problem opening output file'
        stop
      endif

! header reading
      call avhhdr(luavh_in,ios)
        
      if (ios.ne.0) then
        write(*,*) 'Problem reading AVHRR 1b header'
        stop
      endif
        
! data reading
      scan_out = 0
      do iscan=1,avh_h_scnlin
        call avhdtr(luavh_in,iscan,ios)
        lat = avh_pos(1,26)*1E-4
        if (avh_scnlinbit .lt. 0) then
          direction_scan = -1
        else
          direction_scan = 1
        endif
        if (lat .ge. latmin .and. lat .le. latmax .and.
     &      (direction .eq. 0 .or. direction .eq. direction_scan)) then
          scan_out = scan_out + 1
          call avhdtw(luavh_out,scan_out,ios)
          if (avh_scnlindy.ne.avh_h_enddatady) then
            avh_h_enddatajd = avh_h_enddatajd - 1
          endif
          avh_h_enddatayr = avh_scnlinyr
          avh_h_enddatady = avh_scnlindy
          avh_h_enddatatime = avh_scnlintime
          avh_h_scnlin = scan_out
          if (scan_out .eq. 1) then
            if (avh_scnlindy.ne.avh_h_startdatady) then
              avh_h_startdatajd = avh_h_startdatajd + 1
            endif
            avh_h_startdatayr = avh_scnlinyr
            avh_h_startdatady = avh_scnlindy
            avh_h_startdatatime = avh_scnlintime
            call avhhdw(luavh_out,ios)
          endif
        endif
      enddo
      if (scan_out .gt. 0) then
        call avhhdw(luavh_out,ios)
        write(*,*) 'Number of scans written: ',scan_out
      else
        write(*,*) 'No scans found in requested range'
      endif
          
      close(unit=luavh_in,iostat=ios)
      close(unit=luavh_out,iostat=ios)

      END
