#!/bin/ksh
#
#$Id: combine_1c.ksh 313 2012-02-28 14:51:43Z frna $
#
#Combine 1c files for adjacent granules of level 1c data
#
#Usage: combine_1c files
#  where the input files must be in time order.
#  The output file is in native endian-ness but inputs do not have to be.
#
#History:
# 1.1     21/09/06  initial version. N C Atkinson
# 1.2     08/12/11  Add ATMS and CrIS

usage(){
  echo
  echo 'usage : combine_1c file1 [file2 file3 ....]'
  echo
  echo '  where the files must be in time order and must all be for the'
  echo '  same instrument. Output is to hrsn.l1c, aman.l1c, ambn.l1c,'
  echo '  mhsn.l1c, iasi.l1c, atms.l1c, cris.l1c'
  echo
}

. "$AAPP_PREFIX/ATOVS_CONF"

if [ $# = 0 ]; then
  usage
  exit 1
fi

#See which instrument we have in the first file

files="$*"
file=$1
if [ ! -f $file ]; then
  echo "no such file $file"
  exit 1
fi

set $(od -t u4 $file | head -2 | tail -1)  # get word 8
result=$(expr $5)                          # (index + 4 words per line)
case $result in
   5 |  83886080) instr=HIRS  ; outfile=hrsn.l1c ;;
  10 | 167772160) instr=AMSU-A; outfile=aman.l1c ;;
  11 | 184549376) instr=AMSU-B; outfile=ambn.l1c ;;
  12 | 201326592) instr=MHS   ; outfile=mhsn.l1c ;;
  13 | 218103808) instr=IASI  ; outfile=iasi.l1c ;;
 620 | 1812070400) instr=CRIS ; outfile=cris.l1c ;;
 621 | 1828847616) instr=ATMS ; outfile=atms.l1c ;;
    *) echo "invalid instrument code $5"
    exit 1;;
esac

rm -f $outfile         #output file starts empty
touch $outfile
ln -sf $outfile ${FORT}11 

for file in $files; do
  if [ ! -f $file ]; then
    echo "no such file $file"
    exit 1
  fi
  ln -sf $file ${FORT}10
  combine_1c.exe <<EOF
$instr
EOF
  if [ $? != 0 ]; then
    echo " file $file not added"
    rm -f ${FORT}10 ${FORT}11
    exit 1
  else
    echo "added $file to $outfile"
  fi
done

rm -f ${FORT}10 ${FORT}11
