#!/bin/ksh
#
# $Id: convert_from_hdf5.ksh 610 2019-01-17 10:47:43Z frna $
#
# Inverse of convert_to_hdf5, for selected instruments at level 1c
#
# History:
# 1.1   04/01/2019  NCA
#
#==============================================================================

if [ "$AAPP_PREFIX" = "" ]; then
  echo 'Error: AAPP_PREFIX not defined'
  exit 2
fi
if [ ! -d "$AAPP_PREFIX" ]; then
  echo 'Error: AAPP_PREFIX is not a directory:' $AAPP_PREFIX
  exit 2
fi

. "$AAPP_PREFIX/ATOVS_CONF"

status=0

[ $# -lt 2 ] && { echo "Usage: convert_from_hdf5 file INSTRUMENT"; \
  echo "where file is an AAPP-generated l1c.h5 file";
  echo "and INSTRUMENT is one of: AMSU-A, MHS, HIRS, IASI, PCIASI, MWHS2, MWTS2, IRAS";
  exit 0; }

infile=$1
instr=$2

#if the input file name has a .l1c.h5 suffix then just change to l1c.
#Otherwise append .l1c 

if [ $instr = PCIASI ]; then
  outfile=${infile%.lpc.h5}.lpc
else
  outfile=${infile%.l1c.h5}.l1c
fi
  
if [ -s $infile ]; then

  case $instr in 
     AMSU-A) ins=ama1c   ;;
     AMSU-B) ins=amb1c   ;;
     MHS)    ins=amb1c   ;;
     HIRS)   ins=hrs1c   ;;
     IASI)   ins=iasi1c  ;;
     PCIASI) ins=iasipc  ;;
     MWHS2)  ins=mwhs21c ;;
     MWTS2)  ins=mwts21c ;;
     IRAS)   ins=iras1c ;;
  esac
  
  prog=${ins}_from_hdf5.exe

  $prog $infile $outfile
  status=$?
  [ -s $outfile ] && echo "Created $outfile"

else
  
  echo "Cannot find $infile"
  status=1

fi


[ $status = 0 ] || echo "There were errors!" 
exit $status
