#!/bin/ksh
#
#$Id: cris_sdr.ksh 441 2015-04-01 14:58:16Z frna $
#
#Convert CRIS SDR files in HDF-5 format to AAPP 1c format
#
#History:
# 1.0   19/10/2010  initial version. N C Atkinson
# 1.1   24/06/2011  exe file could be elsewhere in $PATH
#==============================================================================

. "$AAPP_PREFIX/ATOVS_CONF"

usage(){
  echo
  echo 'usage : cris_sdr [-o Outputfile] [-g Geofile] [-H] [-B] [-N] SDRfile'
  echo
  echo ' Default apodisation is Hamming (-H); alternatives are'
  echo ' Blackman-Harris (-B) or none (-N)'
  echo
}

#Look for the exe file, first in script directory then in $PATH

exe=$(dirname $0)/cris_sdr.exe

[ -x $exe ] || exe=$(IFS=":"; for p in $PATH; do q=$p/cris_sdr.exe; \
  [ -x $q ] && echo $q && return; done)

[ -x $exe ] || { echo "Cannot find cris_sdr.exe"; exit 1; }

missing=$(ldd $exe | grep "not found")

if [ "$missing" != "" ]; then
  echo $missing
  echo "Library not found, please locate the shared library listed above"
  echo " and add it to your LD_LIBRARY_PATH environment variable"
  exit 1
fi

apod=Hamming

while getopts o:g:HBN c
do
  case $c in
     o)	outfile=$OPTARG;;
     g)	geofile=$OPTARG;;
     H)	apod=Hamming;;
     B)	apod=Blackman;;
     N)	apod=None;;
     \?)      usage
              exit 2;;
  esac
done
shift `expr $OPTIND - 1`

[ $# -lt 1 ] && { usage; exit 1; }
[ -f $1 ] || { echo "SDR file $1 not found"; exit 1; }

outfile=${outfile:-${1%.*}.l1c}
geofile=${geofile:-$(dirname $1)/}   #give directory if not specified

rm -f ${FORT}10 ${FORT}20 ${FORT}51 ${FORT}52
ln -s $DIR_PREPROC/fdf.dat ${FORT}20
ln -s ${DIR_PREPROC}/mapbitls.dat ${FORT}51
ln -s ${DIR_PREPROC}/maptopog.dat ${FORT}52

$exe $1 $geofile $apod

status=$?
if [ $status = 0 ]; then
  mv ${FORT}10 $outfile
  echo "created $outfile"
else
  echo "cris_sdr.exe failed"
fi

rm -f ${FORT}20 ${FORT}51 ${FORT}52
exit $status
