/*
! name: $Id: hiras_sdr.c 635 2019-04-15 15:08:00Z frna $
!
! Purpose: Read an HIRAS SDR file and convert to AAPP 1c format
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    05/02/2019  Original code. (N.C.Atkinson)
!
! Code Description: C
!
! C subroutines and functions called:
!     hiras_sdr_read
!     hiras_sdr_intf (interface to Fortran) 
!
* LIBS = aapphdf5 atov atovin atovpp auxaapp f7nl1b atov
!-------------------------------------------------------------------------------
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#ifdef HAS_LIBHDF5
#include "hdf5.h"
#include "aapp_hdf5.h"
#include "hiras_sdr.h"
#endif

int main (int argc, char **argv){

#ifdef HAS_LIBHDF5

  hiras_sdr_read(argv[1]);
  
  strcpy(CApod, argv[2]);

  printf("scans: %d\n",Count_scnlines);
  hiras_sdr_intf();

  return 0;

#else

  printf("hiras_sdr.exe has not been built with HDF-5\n");
  printf("Please run \"configure\" with --external-libs=\"-Lxxx/lib -lhdf5\"\n");
  printf("  and --external-includes=\"-Lxxx/include\"\n");
  printf("  where xxx is the location of the HDF-5 library\n");
  return 1;
  
#endif

}
