/* 
! Name: $Id: hrp1b_to_hdf5.c 635 2019-04-15 15:08:00Z frna $
!
! Purpose: Read the contents of an AVHRR l1b file into memory and write out
!          in hdf5 format.
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    09/10/2014  New. N C Atkinson
!  1.1    12/02/2015  Remove dependency on libmaia3_tools
!  1.2    28/01/2016  Add -G option for full resolution geolocation
!
! Code Description: C
!
* LIBS = aapphdf5 auxaapp f7nl1b f7ml f7cp f7gp f7tp
!-------------------------------------------------------------------------------
*/

#include <stdio.h>
#include <string.h>

#ifdef HAS_LIBHDF5
#include "hdf5.h"
#include "aapp_hdf5.h"
#include "avh1b_c.h"
#endif

int main (int argc, char **argv){

#ifdef HAS_LIBHDF5

  int clevel=0;
  int fullresgeo=0;

  if (argc < 3) {
    printf("Usage: hrp1b_to_hdf5.exe [-G] infile outfile [clevel]\n");
    return 0;
  }

  if (strncmp(argv[1],"-G",2) == 0) fullresgeo = 1;
    
  if (argc > 3+fullresgeo) {
    sscanf(argv[3+fullresgeo],"%d",&clevel);
    printf("compression level is %d\n",clevel);
  }
  hrp1b_read(argv[1+fullresgeo]);
  hrp1b_to_hdf5_write(argv[2+fullresgeo],clevel,fullresgeo);
  return 0;

#else

  printf("hrp1b_to_hdf5.exe has not been built with HDF-5\n");
  printf("Please run \"configure\" with --external-libs=\"-Lxxx/lib -lhdf5\"\n");
  printf("  and --external-includes=\"-Lxxx/include\"\n");
  printf("  where xxx is the location of the HDF-5 library\n");
  return 1;
  
#endif

}
