C LIBS = navnoaa navtool satid f7ml f7tp f7cp f7gp auxaapp
!+
!NOM/NAME $Id: hrpidf.F 313 2012-02-28 14:51:43Z frna $
!
        program hrpidf 
!
!BUT/PURPOSE 
!
! Lire sur unit 10  un fichier HRPT ( rec de 11090 mots de 16 bits)
! et ecrire sur stdout ( unit 6 ) la chaine "scname yyyymmdd hhmn"
! ou scname est le nom du satellite ( noaaxx ), yyyymmdd est la date du
! fichier et hhmn l'heure du fichier.
! L'annee yyyy n'etant pas disponible dans le fichier est recuperee via 
! la variable d'environnmenent YYYY.
!
! L'executable hrpidf.exe est destine a etre utilise seulement par le script 
! hrpidf.
!   
!
!DESCRIPTION 
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!
!  
!PROPRIETAIRE/OWNER  
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.1      12/07/95                     M. Derrien / G. Legendre
! 1.2      05/02/97  test simultane sur heure et satid avhrr et tip
!                    pour eviter desagrement des donnees Halifax
!                                       M. Derrien
!          24/11/98  Version EXPORT Pascal Brunel
! 1.3      March 03                     T.Labrot
!      f90 test: I.and.J not correct and modified by iand(I,J)
!                I and Y must have the same type. Defined integer*2 local variables
! 1.4      June 04   Byte swap input if needed
!                    and make sure leading bits are zero. N C Atkinson
! 1.5      17/01/2006 : Philippe Marguinaud : change hex constants from
!                       '1234'X to Z'1234'
! 1.6      31/08/2011 : Increase nrec_max from 300 to 700. NCA
!
!EXIT CODES
!
! 0 : ok
! 
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!DECLARATIONS
!
! parameters
        double precision dij_min, dij_max, dij_moy
        integer nrec_max
        parameter (dij_min= 165. / 86400.e3 )
        parameter (dij_moy= 166.666 / 86400.e3 )
        parameter (dij_max= 167. / 86400.e3 )
        parameter ( nrec_max= 700 )

!    
     
!commons
#include "reclopt.h"

!local variables


        character*4 str_yyyy    ! chaine annee 
        integer iq(3)
        double precision ij, ijc, ijp
        integer ig(7)
        integer dj
        double precision dij
        character*6 scname
        integer ord_sc_num_av
        integer ord_sc_num_ti
        integer ord_sc_num
        integer avid
        integer tiid

        integer nrec
        integer*2 minor(11090)
        character*256 filename


        integer k 
        integer ios
        integer status

        integer hrptmf
        logical found
        logical error
        logical first
        integer*2 i2word
        integer*2 i2word120
        parameter (i2word120=120)

        character*1 minorc(22180),ctmp
        equivalence (minorc(1),minor(1))
        integer*2 sync1swap
        data sync1swap/Z'8402'/     !normal sync is Z'0284'
        integer*2 tenbits
        data tenbits/Z'03FF'/       !mask for required bits
        integer*2 tenbitswap
        data tenbitswap/Z'FF03'/    !byte swapped bit mask

!functions
        integer cp_cti
        integer hrpdat
        integer hrptim
        integer ordavid
        integer ordtiid
        character*6 noascnam
        
!
! lecture de l'annee et du nom de fichier
        read(*,'(a)') str_yyyy
        iq(1)=cp_cti( str_yyyy, k)
        if ( k.ne.4 ) then
          status=1
          call aappexit( status )
        endif   
        read(*,'(a)') filename

        open(unit=10,iostat=ios,status='unknown',form='unformatted',
     &    access ='direct',recl=22180/rldiv,file=filename)
        if ( ios.ne.0 ) then
           status=2
           call aappexit(status)
        endif

! recherche datation correcte
        first=.true.
        found=.false.
        error=.false.
        nrec=1
        do while (.not.found.and.(nrec.le.nrec_max).and..not.error) 
          read(10,iostat=ios,rec=nrec) minor 
          if ( ios.eq.0 ) then

!Byte swap if needed
            if (IAND(minor(1),tenbitswap).EQ.sync1swap) then
              do k=1,22180,2
                ctmp=minorc(k)
                minorc(k)=minorc(k+1)                
                minorc(k+1)=ctmp                
              enddo
            endif

!Set leading bits to zero if not already
            if (IAND(minor(1),tenbits).ne.minor(1)) THEN
              do k=1,11090
                minor(k) = IAND(minor(k),tenbits)
              enddo
            endif

!           lecture datation
            iq(2)=hrpdat( minor(9) )
            iq(3)=hrptim( minor(10) )
            call xiqj( iq, ijc )
            if ( nrec.gt.2 ) then
              dij=ijc-ijp
              if ( dij.ge.dij_min.and.dij.le.dij_max ) then
                 ij=ijp - ( nrec - 2 ) * dij_moy
!
!                pour les minor frames 1 seulement
!                (a cause de noaa-k)
!
                 if(first) then
                   dj = int ( ij )
                   call xijg( ij, ig )
                   first=.false.
                 endif
!                hrptmf=(minor(7)/128).and.3
                 hrptmf=IAND((minor(7)/128),3)

                 if (hrptmf.eq.1) then
                   i2word = IAND(minor(7),i2word120)
                   avid   = i2word/8
                   
!                  tiid=(minor(106)/4).and.15
                   tiid=IAND((minor(106)/4),15)
                   ord_sc_num_av=ordavid( avid, dj )
                   ord_sc_num_ti=ordtiid( tiid, dj )
                   if ( ord_sc_num_av.eq.ord_sc_num_ti ) then
                     ord_sc_num = ord_sc_num_av
                     scname=noascnam( ord_sc_num )
                     found=.true.
                   endif
                 endif
               endif              
             endif
             ijp=ijc
           else
             error=.true.
           endif
           nrec=nrec+1
        enddo
        if ( found ) then

            write(*,'(a,1x,i5,i2.2,i2.2,1x,i2.2,i2.2)') 
     &      scname,ig(1),ig(2),ig(3),ig(4),ig(5)

            status=0
        else if ( error ) then
            status=2
        else
             status=11
        endif  
!exit
        close(10)
        if(status.ne.0) then
        call aappexit(status)
        endif

        stop
        end

        integer function hrpdat(hrpadd)       
!+
!NAME  hrpdat.F version 1.1 on 3/24/98 
!
!PURPOSE 
!  extraction of binary day count from the integer*2 hrpadd 
!   where hrpadd is the 9th 16 bit word of hrpt minor frame  
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!  
!OWNER  Marcel Derrien
!
!HISTORY 
!(version   date      comment           <author>)
!
!  1.0     08/02/94   Original code  (M. Derrien)
!         
!ARGUMENTS 
!   input: integer*2 hrpadd
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!      returns the binary day count in an integer
! 
!CHANGES 
!     none
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!DECLARATIONS
!
      integer*2 hrpadd
      integer*2 i2word1022
      parameter (i2word1022=1022)
!    
!-
!      hrpdat=(i2word1022.and.hrpadd)/2
       hrpdat=IAND(i2word1022,hrpadd)/2

       return
       end
       
        integer function hrptim (hrpadd)       
!+
!NAME  hrptim.F version 1.1 on 3/24/98 
!
!PURPOSE 
!  extraction part of binary millisecond of day count from the integer*2 hrpadd 
!   where hrpadd is the 10th to 12th 16 bit words of hrpt minor frame  
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!  
!OWNER  Marcel Derrien
!
!HISTORY 
!(version   date      comment           <author>)
!
!  1.0     08/02/94   Original code  (M. Derrien)
!         
!ARGUMENTS 
!   input: integer*2 hrpadd(3)
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!      returns the 
! 
!CHANGES 
!     none
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!DECLARATIONS
!
      integer*2 hrpadd(3)
      integer part1,part2,part3 
!    
!-
!       version without iand,ishft (using only and,+,*)
!       take 1rst word mask with 127, shift left 20 bit
!       take second word shift left 10 bit
!       take last word
!       add the three values
!
        integer*2 i2word
        integer*2 mask1
        integer*4 mask2
        integer*4 mask3

        data mask1/Z'7F'/
        data mask2/Z'7F00000'/
        data mask3/Z'FFC00'/

!       part1 = ((hrpadd(1).and.127)*1048576).and.133169152
!       part2 = (hrpadd(2)*1024).and.1047552
!       part3 = hrpadd(3)

        I2WORD=IAND(hrpadd(1),mask1)
        part1 = I2WORD * 1048576
        part1 = IAND( part1, mask2)

        part2 = hrpadd(2)*1024
        part2 = IAND(part2, mask3)

        part3 = hrpadd(3)

        hrptim = part1 + part2 + part3

       return
       end
