#! /bin/sh
#
#==============================================================================
#
# @(#)hrpidf.ksh version 1.1 11/25/98 COPYRIGHT (c) Meteo-France 1996
#
#
# NOM DU MODULE/MODULE NAME
#
# hrpidf
#
# BUT/PURPOSE
#
# Utilitaire minimal du format hrp
#
# DESCRIPTION    
#
# PROPRIETAIRE/OWNER
#
# HISTORIQUE/HISTORY 
# 
#  1.1     11/07/95  version initiale		 M.Derrien / G. Legendre
#          04/09/96  ajout log erreurs et exit code interne Marcel Derrien
#          19/11/96  correction hrpsetorb Marcel Derrien
#  2.0     24/11/98  Export version Pascal Brunel
#			Year == current year OR year is given in argument
#			Orbit is unknown
#  2.1     01/02/06  change config file P. Marguinaud
#  2.2     07/06/06  remove initialisation of DIR_SATID P. Marguinaud
#==============================================================================

#==============================================================================
# functions 
#==============================================================================
EXIT_OK=0
EXIT_USAGE=1
EXIT_SYSTEM_OTHER=1
EXIT_INTERNAL=1


#------------------------------------------------------------------------------
hrpidf_exit()
{
  exit $1
}

#------------------------------------------------------------------------------
hrpidf_usage()
{
  log_error syntaxe : ${SYNTAXE} 
  exit ${EXIT_USAGE}
}


#==============================================================================
# main
#==============================================================================

#------------------------------------------------------------------------------
# includes
#------------------------------------------------------------------------------
. "$AAPP_PREFIX/ATOVS_CONF"
. liblog.sh

#------------------------------------------------------------------------------
# recuperation des options 
#------------------------------------------------------------------------------
#set -x +o xtrace
SYNTAXE="hrpid [-Y yyyy] [-s | -d | -h | -n | -i] file"
MODE=UNKNOWN
YYYY=`date +%Y`
NNNNN=0

while getopts Y:sdhni c
do
    case $c in
      Y)    YYYY=$OPTARG;;
      s)    MODE=S ;;
      d)    MODE=D ;;
      h)    MODE=H ;;
      n)    MODE=N ;;
      i)    MODE=I ;;
      \?)   hrpidf_usage ;; 
    esac
done
shift `expr $OPTIND - 1`

#------------------------------------------------------------------------------
# recuperation des arguments
#------------------------------------------------------------------------------

FILE=$1

#------------------------------------------------------------------------------
# initialisation log  
#------------------------------------------------------------------------------
log_init `basename $0`
#------------------------------------------------------------------------------
# valeurs par defaut
#------------------------------------------------------------------------------

SATID_YYYYMMDD_HHMN=`hrpidf.exe<<EOF
${YYYY}
${FILE}
EOF`
stat=$?
if [ ${stat} -ne 0 ] ; then
   log_line
   log_info  infos recuperees ${SATID_YYYYMMDD_HHMN}
   log_error hrpidf fichier ${FILE} non valable
fi
if [ ${stat} -eq 0 ] ; then
  SATID=`echo ${SATID_YYYYMMDD_HHMN} | awk '{ print $1}'`
  YYYYMMDD=`echo ${SATID_YYYYMMDD_HHMN} | awk '{ print $2}'`
  HHMN=`echo ${SATID_YYYYMMDD_HHMN} | awk '{ print $3}'`
  if [ ${NNNNN} -eq 0 ]; then
    NNNNN=`sdh2orbnum -s ${SATID} -d ${YYYYMMDD} -h ${HHMN}`
    stat=$?
    if [ ${stat} -ne 0 ] ; then
      log_line
      log_info  appel sdh2orbnum -s ${SATID} -d ${YYYYMMDD} -h ${HHMN}
      log_error impossible recuperer numero orbite     
    fi
    #year end orbit nubmber are written at the end of the hrpt file  
    YYYY_NNNNN=`echo ${YYYY} ${NNNNN}`
    EEEE_RRRRR=`tail -10c ${FILE}`     
     if [ "${EEEE_RRRRR}" != "${YYYY_NNNNN}" ] ; then
     echo ${YYYY} ${NNNNN}'\c' >> ${FILE}
     fi
  fi
fi
case ${MODE} in
  S)	echo ${SATID} ;;
  D)	echo ${YYYYMMDD} ;;
  H)	echo ${HHMN} ;;
  N)	echo ${NNNNN} ;;
  I)    echo ${SATID} ${YYYYMMDD} ${HHMN} ${NNNNN} ;;
esac

#--------
# sortie 
#--------
exit_hrp=${EXIT_OK}
if [ ${stat} -ne 0 ] ; then
 exit_hrp=${EXIT_INTERNAL}
fi
hrpidf_exit ${exit_hrp} 

