      PROGRAM hrpt_noaa
      IMPLICIT none
     
! NAME: $Id: hrpt1b_noaa.F 325 2012-09-14 13:25:17Z frna $
!
! DESCRIPTION
!
!  Convert from NOAA HRPT/LAC level 1b (16 bit format) to AAPP AVHRR level 1a.
!  You then need to run avhrcl, in order to re-create the calibration
!  information in the format and with the scaling factors expected by AAPP.
!
! INPUT
!
!  Name of NOAA 1b file
!
! OUTPUT
!  
!  hrpt.l1b
!
! Change History:
!
! Version Date     Comment
!  1.0    15/4/05  Initial version. N C Atkinson
!  1.1    12/09/12 Include reclopt.h and rldiv.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET OFFICE.
!#########################################################################

#include "reclopt.h"

      character*80 file_noaa, file_aapp
      integer irec
      character*1 hrptn(22528)
      character*1 hrpta(22016)
      equivalence (hrptn(1),hrpta(1))
      integer ios
      
      write(*,*) 'NOAA l1b file name? '
      read(*,*) file_noaa      

      file_aapp = "hrpt.l1b"
      
      open(10,file=file_noaa,recl=22528/rldiv,access='Direct',
     &  status='old',iostat=ios)
     
      if (ios .ne. 0) then
        write(*,*)'Unable to open',file_noaa
        stop
      endif    

      open(11,file=file_aapp,recl=22016/rldiv,access='Direct',
     &  status='new',iostat=ios)     
      
      if (ios .ne. 0) then
        write(*,'(A,A)')'Unable to create new file ',file_aapp
        stop
      endif    

      irec = 1
      
      do while (ios .eq. 0)
        read(10,rec=irec,iostat=ios) hrptn
        if (ios.eq.0) write(11,rec=irec) hrpta 
        irec = irec + 1
      end do
      
      close(10)
      close(11)
      write(*,'(I5,A,A)') irec-1,' records written to ',file_aapp
      end
