#!/bin/ksh
#
#$Id: iras_sdr.ksh 345 2013-02-01 13:54:00Z frna $
#
#Convert IRAS SDR files in HDF-5 format to AAPP 1c format
#
#History:
# 1.0   31/01/2013  initial version. N C Atkinson
#==============================================================================

. "$AAPP_PREFIX/ATOVS_CONF"

usage(){
  echo
  echo 'usage : iras_sdr [-o Outputfile] SDRfile'
  echo
}

#Look for the exe file, first in script directory then in $PATH

exe=$(dirname $0)/iras_sdr.exe

[ -x $exe ] || exe=$(IFS=":"; for p in $PATH; do q=$p/iras_sdr.exe; \
  [ -x $q ] && echo $q && return; done)

[ -x $exe ] || { echo "Cannot find iras_sdr.exe"; exit 1; }

missing=$(ldd $exe | grep "not found")

if [ "$missing" != "" ]; then
  echo $missing
  echo "Library not found, please locate the shared library listed above"
  echo " and add it to your LD_LIBRARY_PATH environment variable"
  exit 1
fi

while getopts o: c
do
  case $c in
     o)	outfile=$OPTARG;;
     \?)      usage
              exit 2;;
  esac
done
shift `expr $OPTIND - 1`

[ $# -lt 1 ] && { usage; exit 1; }
[ -f $1 ] || { echo "SDR file $1 not found"; exit 1; }

outfile=${outfile:-${1%.*}.l1c}
rm -f ${FORT}10

$exe $1

status=$?
if [ $status = 0 ]; then
  mv ${FORT}10 $outfile
  echo "created $outfile"
else
  rm -f ${FORT}10
  echo "iras_sdr.exe failed"
fi

exit $status
