C LIBS =  navnoaa navtool satid f7ml f7tp f7cp f7gp auxaapp
!+
!NOM/NAME $Id: l1bidf.F 47 2006-08-21 08:44:41Z frna $
!
        program l1bidf
!
!BUT/PURPOSE 
! reads from unit lul1b  a NOAA 1B file 
! and write on stdout (unit 6) the string
! "scname yyyymmdd hhmn orbnm type" 
! where scname is the satellite name (noaaxx)
! yyyymmdd is the start of data set date
! hhmn is the hour
! orbnm is the orbit number (from the internal data set name)
! dtype is the data set type
! 
!
!DESCRIPTION 
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
!  1.0     01/09/95  initial       Marcel Derrien
!  1.1     June 98    conform to new 1B format Marcel Derrien
!  1.2     Jan 2005  Linux compatibilty and add MHS  P. Brunel
!  1.3     Jun 2006  Returns metop04 for metop P. Marguinaud
!
!EXIT CODES
!   0 is ok
!   2 problem with open fort.10
!   11 problem with read fort.10
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!DECLARATIONS
!
! identification variable for "what"
      character*62 sccsid 
!    
      data sccsid
     &/'%Z% %M% version %I% on %G%'/ 
!local variables
        integer lul1b
        integer ier0,ier1,ier2,ier3
        integer status
        integer iq(3)
        integer ig(7)
        integer ord_sc_num
        integer satid,jdate
        character*8 scname
        character*5 orbnum
        character*256 filename
!
!
       integer*2    h_l1bversnb     !level 1b format version number
       integer*2    h_l1bversyr     !level 1b format version year
       integer*2    h_l1bversdy     !level 1b format version day of year
       integer*2    h_reclg         !reserved for logical record length
       integer*2    h_blksz         !reserved for block size
       integer*2    h_hdrcnt        !count of header records in data set
       character*42 h_dataname      !data set name
       character*8  h_prblkid       !processing block identification
       integer*2    h_satid         !NOAA spacecraft identification code
       integer*2    h_instid        !instrument identification
       integer*2    h_datatyp       !data type code
!                (1 = LAC; 2 = GAC; 3 = HRPT; 4 = TIP; 5 = HIRS; 6 = MSU;
!                 7 = SSU; 8 = DCS; 9 = SEM; 10 = AMSU-A; 11 = AMSU-B; 12 = MHS)
       integer*2    h_tipsrc        !TIP source code
       integer*2    h_bound1
       integer      h_startdatajd   !start of data set day count 
!                                        from 00h,1 Jan 1950
       integer*2    h_startdatayr   !start of data set year
       integer*2    h_startdatady   !start of data set day of year
       integer      h_startdatatime !start of data set UTC time of day in
!                                        milliseconds  
       character*3 h_siteid
       character*1 h_blank 
       integer*2 h_filler0(3)    
       data lul1b/10/
       character*8 datatyp(13)
       data datatyp/'  ','lac','gac','hrpt',
     &              'tip','hirs','msu','ssu',
     &              'dcs','sem','amsua','amsub','mhs'/

        integer iw4
        integer*2 iw2
        character*1 ch4(4), ch2(2), ch
        equivalence (iw4, ch4)
        equivalence (iw2, ch2)
        logical swap_1b
!functions
        character*6 noascnam    
        integer ord1bid
!-
      status = 0
      ier1=0
      ier2=1
      ier3=1
!get l1b file
      read (*,'(a)',iostat=ier0) filename
      if(ier0.eq.0) then
        open(lul1b,form='unformatted',access='direct',
     &     recl=512,status='old',iostat=ier1,file=filename)
        if(ier1.eq.0) then
            read(lul1b,rec=1,iostat=ier2)
     &          h_siteid,h_blank,
     &          h_l1bversnb,h_l1bversyr,h_l1bversdy,
     &          h_reclg,h_blksz,
     &          h_hdrcnt,h_filler0,
     &          h_dataname,
     &          h_prblkid,
     &          h_satid,
     &          h_instid,
     &          h_datatyp,
     &          h_tipsrc,
     &          h_startdatajd,
     &          h_startdatayr,
     &          h_startdatady,
     &          h_startdatatime 

          if(ier2.ne.0) then
          h_siteid='xxx'
          read(lul1b,rec=1,iostat=ier3)
     &          h_l1bversnb,h_l1bversyr,h_l1bversdy,
     &          h_reclg,h_blksz,
     &          h_hdrcnt,
     &          h_dataname,
     &          h_prblkid,
     &          h_satid,
     &          h_instid,
     &          h_datatyp,
     &          h_tipsrc,
     &          h_bound1,
     &          h_startdatajd,
     &          h_startdatayr,
     &          h_startdatady,
     &          h_startdatatime
          endif                   
          if(ier2.eq.0.or.ier3.eq.0) then
             swap_1b = h_datatyp .gt. 255

             if(swap_1b) then
                !h_startdatayr   i*2
                iw2    = h_startdatayr
                ch     = ch2(2)
                ch2(2) = ch2(1)
                ch2(1) = ch
                h_startdatayr = iw2

                !h_startdatady   i*2
                iw2    = h_startdatady
                ch     = ch2(2)
                ch2(2) = ch2(1)
                ch2(1) = ch
                h_startdatady = iw2

                !h_startdatatime i*4
                iw4    = h_startdatatime
                ch     = ch4(4)
                ch4(4) = ch4(1)
                ch4(1) = ch
                ch     = ch4(3)
                ch4(3) = ch4(2)
                ch4(2) = ch
                h_startdatatime = iw4

                !h_satid         i*2
                iw2    = h_satid
                ch     = ch2(2)
                ch2(2) = ch2(1)
                ch2(1) = ch
                h_satid = iw2

                !h_startdatajd   i*4
                iw4    = h_startdatajd
                ch     = ch4(4)
                ch4(4) = ch4(1)
                ch4(1) = ch
                ch     = ch4(3)
                ch4(3) = ch4(2)
                ch4(2) = ch
                h_startdatajd = iw4

                !h_datatyp       i*2
                iw2    = h_datatyp
                ch     = ch2(2)
                ch2(2) = ch2(1)
                ch2(1) = ch
                h_datatyp = iw2

             end if
! get date yyyyddmm
! get hour hhmn
! from the start time of the data set
            iq(1)=h_startdatayr
            iq(2)=h_startdatady
            iq(3)=h_startdatatime
            call xiqg(iq,ig)
! get spacecraft name
! from the satid  and julian day found in the header
            satid=h_satid
            jdate=h_startdatajd
            ord_sc_num = 
     &        ord1bid(satid,jdate)
            scname = 
     &        noascnam(ord_sc_num)
            if( scname(1:4) .eq. 'meto' ) then
              scname(6:7) = scname(5:6)
              scname(1:5) = 'metop'
            endif
       
! get orbit number
! using the data set name to find the orbit number from the 1B file

            orbnum = h_dataname(33:37)
        
!write on output
            write(*,'(a,1x,i4,i2.2,i2.2,1x,i2.2,i2.2,1x,a,1x,a,1x,a)') 
     &         scname,ig(1),ig(2),ig(3),ig(4),ig(5),orbnum,
     &         datatyp(h_datatyp+1)(1:6) , h_siteid
          else
            status=2
          endif
        else
          status=11
        endif
      else
        status=12
      endif
!exit
      call aappexit(status)
      
      end
