C LIBS =  navnoaa navtool satid f7ml f7tp f7cp f7gp auxaapp
!+
!NOM/NAME $Id: l1cidf.F 325 2012-09-14 13:25:17Z frna $
!
        program l1cidf
!
!BUT/PURPOSE 
! reads from unit lul1c  a AAPP level 1c file 
! and write on stdout (unit 6) the string
! "scname yyyymmdd hhmn orbnm type" 
! where scname is the satellite name (noaaxx)
! yyyymmdd is the start of data set date
! hhmn is the hour
! orbnm is the orbit number (from the internal data set name)
! dtype is the data set type (hirs msu amsua amsub)
! 
!
!DESCRIPTION 
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
!  1.0     01/09/95  initial(1b)   Marcel Derrien
!  1.0     30/04/97  initial       Pascal Brunel
!  1.1     28/02/03  ajout site_id P. Brunel
!  1.2     15/12/03  correction site_id=??? dans certains cas P. Brunel
!  1.3     Jan 2005  Linux compatibilty and add MHS  P. Brunel
!  1.4     Feb 2006  Metop capability P. Marguinaud
!  1.5     Jun 2006  Returns metop04 for metop P. Marguinaud
!  1.6     Sep 2012  Include reclopt.h; change condition for swap_1b 256->65536
!
!EXIT CODES
!   0 is ok
!   2 problem with open fort.10
!   11 problem with read fort.10
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!DECLARATIONS
!
!    
     
!commons
#include "reclopt.h"
        integer*4 header(22)

!local variables
        integer satid
        character*7 scname
        integer lul1c
        integer ier0,ier1,ier2
        integer status

        integer iq(3)
        integer ig(7)

        character*4 siteid
        character*4 siteid1b
        equivalence (siteid,header)
        equivalence (siteid1b,header(2))

        integer i
        integer iw4
        character*1 ch4(4), ch
        equivalence (iw4, ch4)
        logical swap_1b

        character*256 filename
        data lul1c/10/
        character*8 datatyp(14)
        data datatyp/'  ','lac','gac','hrpt',
     &              'tip','hirs','msu','ssu',
     &              'dcs','sem','amsua','amsub','mhs','iasi'/

!functions
        character*6 noascnam    
        integer ord1bid
!-
      status = 0

!get l1c file
      read (*,'(a)',iostat=ier0) filename
      if(ier0.eq.0) then

        open(lul1c,form='unformatted',access='direct',
     &     recl=88/rldiv,status='old',iostat=ier1,file=filename)
        if(ier1.eq.0) then
          read(lul1c,rec=1,iostat=ier2) header
          if(ier2.eq.0) then

             ! Tests datatype value
             ! normal case is 0-13
             swap_1b = header(8) .gt. 65536
             if(swap_1b) then
                ! swap words 7 8 11 12 13 14
                ! for satellite, datatype, orbit, date
                iw4       = header(7)
                ch        = ch4(4)
                ch4(4)    = ch4(1)
                ch4(1)    = ch
                ch        = ch4(3)
                ch4(3)    = ch4(2)
                ch4(2)    = ch
                header(7) = iw4

                iw4       = header(8)
                ch        = ch4(4)
                ch4(4)    = ch4(1)
                ch4(1)    = ch
                ch        = ch4(3)
                ch4(3)    = ch4(2)
                ch4(2)    = ch
                header(8) = iw4

                iw4       = header(11)
                ch        = ch4(4)
                ch4(4)    = ch4(1)
                ch4(1)    = ch
                ch        = ch4(3)
                ch4(3)    = ch4(2)
                ch4(2)    = ch
                header(11)= iw4

                iw4       = header(12)
                ch        = ch4(4)
                ch4(4)    = ch4(1)
                ch4(1)    = ch
                ch        = ch4(3)
                ch4(3)    = ch4(2)
                ch4(2)    = ch
                header(12)= iw4

                iw4       = header(13)
                ch        = ch4(4)
                ch4(4)    = ch4(1)
                ch4(1)    = ch
                ch        = ch4(3)
                ch4(3)    = ch4(2)
                ch4(2)    = ch
                header(13) = iw4

                iw4       = header(14)
                ch        = ch4(4)
                ch4(4)    = ch4(1)
                ch4(1)    = ch
                ch        = ch4(3)
                ch4(3)    = ch4(2)
                ch4(2)    = ch
                header(14) = iw4

             end if

! get date yyyyddmm
! get hour hhmn
! from the start time of the data set
            iq(1)=header(12)
            iq(2)=header(13)
            iq(3)=header(14)
            call xiqg(iq,ig)

! spacecraft number (14 = noaa14) is the word 7       
            satid=header(7)
            if (satid.GT.99) satid=99

! get spacecraft name
! note that to uniquely identify a satellite we also need the WMO satid. But
! the position of this field is different for different 1c/1d formats. For now
! it does not matter, since noaa and metop can be distinguished simply from the
! series number.

            if (satid.le.4) then
              write(scname,'(A5,I2.2)') "metop",satid
            else
              write(scname,'(A4,I2.2)') "noaa",satid
            endif 

! orbit number is the word 11
    
            ! For some specific cases siteid can
            ! equal ???. So take 1b value when available
            ! in any case do not return ??? because
            ! it can generate so side effects in shell environment
            if( siteid(1:1) .eq. '?' ) then
               if( siteid1b(1:1) .eq. '?' ) then
                 siteid   = 'xxxx'
                 siteid1b = 'xxxx'
               else
                 siteid   = siteid1b
               endif     
            endif
!write on output
            write(*,
     &       "(a,1x,i4,i2.2,i2.2,1x,i2.2,i2.2,
     &         1x,i5.5,1x,a,1x,a,1x,a)") 
     &         scname,
     &         ig(1),ig(2),ig(3),ig(4),ig(5),
     &         header(11),
     &         datatyp(header(8)+1)(1:6),
     &         siteid(1:3) ,
     &         siteid1b(1:3) 
          else
            status=2
          endif
        else
          status=11
       endif
      else
        status=12
      endif
      
!exit
      if(status.ne.0) then
         call aappexit(status)
      endif

      stop
      end
