#!/bin/ksh
#
# For a given AVHRR l1b file, display the pixel and line numbers that
# correspond to a series of specified lat/lons.
#
# Calls latlon_to_swath.exe
#
# 13/09/2016 N Atkinson
# 29/08/2017 Support BSD version of the date command (for macOS)
#
#------------------------------------------------------------------------------------

[ $# -ge 2 ] || { echo "usage: latlon_to_swath.ksh avhrrl1bfile latlonfile"; exit 0; }

l1bfile=$1
[ -f $l1bfile ] || { echo "1b file does not exist"; exit 1; }

latlonfile=$2
[ -f $l1bfile ] || { echo "A file of lat/lons is needed"; exit 1; }

[ -d "$AAPP_PREFIX" ] || { echo "Please define AAPP_PREFIX"; exit 1; }
. $AAPP_PREFIX/ATOVS_CONF

#Read the l1b file header and get the satellite, start and end times.
#Assume native endian-ness.

set $(od -An -t u2 -N 104 $l1bfile)
satid=${37}

set $(od -An -t u4 -N 104 $l1bfile)
startjd=${21}
endjd=${24}
startsec=${23}
endsec=${26}
 
tfirst=$(( ${startjd} + $((${startsec}/86400000.0)) ))
tlast=$(( ${endjd} + $((${endsec}/86400000.0)) ))

case $satid in
  4) satname=noaa15;;
  7) satname=noaa18;;
  8) satname=noaa19;;
 11) satname=M01;;
 12) satname=M02;;
  *) echo "unknown satid: $satid"
     exit 1;;
esac

#Get the satpos file, or generate it if it does not exist, assuming use of TLEs.

if [[ "$(date --version 2>/dev/null)" == *GNU* ]]; then
  yyyymmdd=$(date -u +%Y%m%d -d "19500101 + $startjd days")
else
  sec0=$(date -jnu -f "%Y%m%d%H%M" 195001010000 +%s)     #BSD version (macOS)
  yyyymmdd=$(date -jnu -r $sec0 -v +${startjd}d +%Y%m%d)
fi  
satpos_file=$DIR_NAVIGATION/satpos/satpos_${satname}_${yyyymmdd}.txt

if [ ! -f $satpos_file ]; then
  tleing -s $satname
  dd=${yyyymmdd:6:2}
  mm=${yyyymmdd:4:2}
  yy=${yyyymmdd:2:2}
  satpostle -o -s $satname -d ${dd}/${mm}/${yy} -n 1.2
fi

[ -f $satpos_file ] || { echo "Problem generating satpos file $satpos_file"; exit 1; }

#Call the Fortran routine

latlon_to_swath.exe <<EOF
$satpos_file
$tfirst $tlast
$latlonfile
EOF

