/*
! name: $Id: mwhs_sdr.c 635 2019-04-15 15:08:00Z frna $
!
! Purpose: Read an MWHS SDR file and convert to AAPP 1c format
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    30/11/2010  Original code. (N.C.Atkinson)
!
! Code Description: C
!
! C subroutines and functions called:
!     mwhs_sdr_read
!     mwhs_sdr_intf (interface to Fortran) 
!
* LIBS = aapphdf5 atov atovin cal auxaapp f7nl1b atov
!-------------------------------------------------------------------------------
*/

#include <stdio.h>

#ifdef HAS_LIBHDF5
#include "hdf5.h"
#include "aapp_hdf5.h"
#include "mwhs_sdr.h"
#endif

int main (int argc, char **argv){

#ifdef HAS_LIBHDF5

  mwhs_sdr_read(argv[1]);
  mwhs_sdr_intf();
  return 0;

#else

  printf("mwhs_sdr.exe has not been built with HDF-5\n");
  printf("Please run \"configure\" with --external-libs=\"-Lxxx/lib -lhdf5\"\n");
  printf("  and --external-includes=\"-Lxxx/include\"\n");
  printf("  where xxx is the location of the HDF-5 library\n");
  return 1;
  
#endif

}
