#!/bin/ksh
#
#$Id$
#
#Convert MWHS SDR files in HDF-5 format to AAPP 1c format
#
#History:
# 1.0   02/12/2010  initial version. N C Atkinson
# 1.1   24/06/2011  exe file could be elsewhere in $PATH
# 1.2   23/08/2012  Link data file mwhs_channels.dat
#==============================================================================

. "$AAPP_PREFIX/ATOVS_CONF"

usage(){
  echo
  echo 'usage : mwhs_sdr [-o Outputfile] SDRfile'
  echo
}

#Look for the exe file, first in script directory then in $PATH

exe=$(dirname $0)/mwhs_sdr.exe

[ -x $exe ] || exe=$(IFS=":"; for p in $PATH; do q=$p/mwhs_sdr.exe; \
  [ -x $q ] && echo $q && return; done)

[ -x $exe ] || { echo "Cannot find mwhs_sdr.exe"; exit 1; }

missing=$(ldd $exe | grep "not found")

if [ "$missing" != "" ]; then
  echo $missing
  echo "Library not found, please locate the shared library listed above"
  echo " and add it to your LD_LIBRARY_PATH environment variable"
  exit 1
fi

while getopts o: c
do
  case $c in
     o)	outfile=$OPTARG;;
     \?)      usage
              exit 2;;
  esac
done
shift `expr $OPTIND - 1`

[ $# -lt 1 ] && { usage; exit 1; }
[ -f $1 ] || { echo "SDR file $1 not found"; exit 1; }

outfile=${outfile:-${1%.*}.l1c}
rm -f ${FORT}11
ln -sf $DIR_PREPROC/fdf.dat ${FORT}20
ln -sf $DIR_PREPROC/mwhs_channels.dat ${FORT}21

$exe $1

status=$?
if [ $status = 0 ]; then
  mv ${FORT}11 $outfile
  echo "created $outfile"
else
  echo "mwhs_sdr.exe failed"
fi

rm -f ${FORT}20

exit $status
