#!/bin/ksh
#
#$Id: mwts2_sdr.ksh 391 2014-05-09 11:11:30Z frna $
#
#Convert MWTS2 SDR files in HDF-5 format to AAPP 1c format
#
#History:
# 1.0   08/04/2014 based on mwts_sdr.ksh
#==============================================================================

. "$AAPP_PREFIX/ATOVS_CONF"

usage(){
  echo
  echo 'usage : mwts2_sdr [-o Outputfile] SDRfile'
  echo
}

#Look for the exe file, first in script directory then in $PATH

exe=$(dirname $0)/mwts2_sdr.exe

[ -x $exe ] || exe=$(IFS=":"; for p in $PATH; do q=$p/mwts2_sdr.exe; \
  [ -x $q ] && echo $q && return; done)

[ -x $exe ] || { echo "Cannot find mwts2_sdr.exe"; exit 1; }

missing=$(ldd $exe | grep "not found")

if [ "$missing" != "" ]; then
  echo $missing
  echo "Library not found, please locate the shared library listed above"
  echo " and add it to your LD_LIBRARY_PATH environment variable"
  exit 1
fi

while getopts o: c
do
  case $c in
     o)	outfile=$OPTARG;;
     \?)      usage
              exit 2;;
  esac
done
shift `expr $OPTIND - 1`

[ $# -lt 1 ] && { usage; exit 1; }
[ -f $1 ] || { echo "SDR file $1 not found"; exit 1; }

outfile=${outfile:-${1%.*}.l1c}
rm -f ${FORT}10
ln -sf $DIR_PREPROC/fdf.dat ${FORT}20
ln -sf $DIR_PREPROC/mwts_channels.dat ${FORT}21

$exe $1

status=$?
if [ $status = 0 ]; then
  mv ${FORT}10 $outfile
  echo "created $outfile"
else
  rm -f ${FORT}10
  echo "mwts2_sdr.exe failed"
fi
rm -f ${FORT}20 ${FORT}21

exit $status
