#!/bin/ksh
#
#$Id: reorder_1c.ksh 620 2019-02-15 12:41:33Z frna $
#
#Re-order scan lines within a level 1c file
#
#Usage: reorder_1c infile outfile
#  The output file is in native endian-ness but inputs do not have to be.
#
#History:
# 1.1     11/06/2010  initial version. N C Atkinson
# 1.2     12/01/2012  Add ATMS and CrIS and allow for leading zeros in 'od'
# 1.3     20/12/2018  Add MWHS2
# 1.4     14/02/2019  Add IRAS

usage(){
  echo
  echo 'usage : reorder_1c infile outfile'
  echo
}

. "$AAPP_PREFIX/ATOVS_CONF"

if [ $# = 0 ]; then
  usage
  exit 1
fi

#See which instrument we have in the input file

infile=$1
outfile=$2
if [ ! -f $infile ]; then
  echo "no such file $infile"
  exit 1
fi

set $(od -t u4 $infile | head -2 | tail -1)  # get word 8
result=$(expr $5)                          # (index + 4 words per line)
case $result in
   5 |  83886080) instr=HIRS ;;
   6 | 100663296) instr=MSU  ;;
  10 | 167772160) instr=AMSU-A ;;
  11 | 184549376) instr=AMSU-B ;;
  12 | 201326592) instr=MHS  ;;
  13 | 218103808) instr=IASI ;;
  19 | 318767104) instr=MWHS2 ;;
  17 | 285212672) instr=IRAS ;;
 620 | 1812070400) instr=CRIS ;;
 621 | 1828847616) instr=ATMS ;;
    *) echo "invalid instrument code $5"
    exit 1;;
esac

rm -f $outfile         #output file starts empty
touch $outfile
ln -sf $infile  ${FORT}10 
ln -sf $outfile ${FORT}11 

reorder_1c.exe <<EOF
$instr
EOF

status=$?
if [ $status != 0 ]; then
  echo "reorder_1c failed for $infile"
else
  echo "created $outfile"
fi
rm -f ${FORT}10 ${FORT}11
exit $status
