#! /bin/ksh
#
#==============================================================================
#
# @(#)sdh2orbnum.ksh version 1.1 11/25/98 COPYRIGHT (c) Meteo-France 1996
#
#
# NOM DU MODULE/MODULE NAME
#
# sdh2orbnum 
#
# BUT/PURPOSE
#
# Fournir le numero d'orbite d'un satellite noaa donne pour un instant
# donne.
#
#
# DESCRIPTION
#
# sdh2orbnum -s satid -d yyyymmdd -h hhmn    
#
# PROPRIETAIRE/OWNER
#
# HISTORIQUE/HISTORY 
# 
#  1.1     05/07/95  version initiale		   		G. Legendre
#  1.2     06/11/95  modification notice si satpos avec TBUS    M. Derrien
#          28/08/96  suppression PATH_SATID
#          25/11/98  version EXPORT Pascal Brunel
#  1.3     01/02/06  change config file P. Marguinaud
#  1.4     07/06/06  check exit codes of satpos Pascal Brunel, Philippe Marguinaud
#
#==============================================================================

#==============================================================================
# functions 
#==============================================================================

#------------------------------------------------------------------------------

EXIT_OK=0
EXIT_USAGE=1
EXIT_SYSTEM_OTHER=1
EXIT_INTERNAL=1

sdh2orbnum_exit()
{
  exit $1
}

#------------------------------------------------------------------------------
sdh2orbnum_usage()
{
echo error syntaxe : ${SYNTAXE} 
  sdh2orbnum_exit ${EXIT_USAGE} 
}

#------------------------------------------------------------------------------

#==============================================================================
# main
#==============================================================================

#------------------------------------------------------------------------------
# includes
#------------------------------------------------------------------------------
. "$AAPP_PREFIX/ATOVS_CONF"
. liblog.sh

#------------------------------------------------------------------------------
# recuperation des options 
#------------------------------------------------------------------------------
SATID=UNKNOWN
YYYY=`date +%Y`
MM=`date +%m`
DD=`date +%d`
HH=`date +%H`
MN=`date +%M`
YYYYMMDD=${YYYY}${MM}${DD}
HHMN=${HH}${MN}

SYNTAXE="sdh2orbnum -s satid [-d yyyymmdd] [-h hhmn]"
while getopts s:d:h: c
do
    case $c in
       s)   SATID=$OPTARG;;
       d)   YYYYMMDD=$OPTARG;;
       h)   HHMN=$OPTARG;;
      \?)   sdh2orbnum_usage ;; 
    esac
done
#------------------------------------------------------------------------------
# recuperation des arguments
#------------------------------------------------------------------------------
DIR_SATID=${DIR_SATID=/soft/chaine_noaa/data}; export DIR_SATID
YY=`echo ${YYYYMMDD} | cut -b 3-4`
MM=`echo ${YYYYMMDD} | cut -b 5-6`
DD=`echo ${YYYYMMDD} | cut -b 7-8`
HH=`echo ${HHMN} | cut -b 1-2`
MN=`echo ${HHMN} | cut -b 3-4`

DIR_SATPOS=${DIR_NAVIGATION}/satpos
FILE_SATPOS=${DIR_SATPOS}/satpos_${SATID}_${YYYYMMDD}.txt

if [ ! -s ${FILE_SATPOS} ]; then
  sdh2orbnum_exit ${EXIT_INTERNAL} 
fi

SDH2ORBNUM_IJC="${DD}/${MM}/${YY}  ${HH}:${MN}:00.000"

orbnum=`sdh2orbnum.exe <<EOF
${SATID}
${SDH2ORBNUM_IJC}
${FILE_SATPOS}
EOF`
exit=$?

if [ ! ${exit} -eq ${EXIT_OK} ]; then
  sdh2orbnum_exit ${EXIT_INTERNAL} 
fi

if [ ${orbnum} -gt 0 -a ${orbnum} -lt 100000 ]; then
  echo ${orbnum}
fi

#------------------------------------------------------------------------------
# sortie normale
#------------------------------------------------------------------------------
sdh2orbnum_exit ${EXIT_OK} 


