C LIBS = atovpp atov auxaapp f7nl1b
!
      PROGRAM SELECT_HIRS1D
      IMPLICIT NONE
!----------------------------------------------------------------------------
! Name: $Id: select_hirs1d.F 118 2008-06-27 09:19:25Z frna $
!
! Purpose: Extract data for a defined geographical area from a HIRS 1d file
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2006, EUMETSAT, All Rights Reserved.
!
! Version:
! 1.0  30/3/06  N C Atkinson
! 1.1  25/4/06  Write start and end times
! 1.2  04/5/06  Write satid, day and orbit also
! 1.3  08/5/06  Include in AAPP v6 framework
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:

#include "ppparms.h"
#include "hrs1d.h"
#include "stdio_unit.h"

      INTEGER iscan,ispot,nscan
      INTEGER starthh,startmm,startss,endhh,endmm,endss
      INTEGER H1dunitin, H1dunitout
      DATA H1dunitin/10/, H1dunitout/11/
      
      REAL south,north,west,east
      REAL lat,lon
      
      LOGICAL first,required
      DATA first/.true./
      
      CHARACTER*1 satchar
!
!- End of header

!1) Get required goegraphical region

      read(*,*) south,north,west,east
      
!2) Open the 1d files and read the header

      CALL IOH1D ("OPEN/READ",H1dunitin,0)
      CALL IOH1D ("READ HEADER",H1dunitin,1)
      CALL IOH1D ("OPEN/WRITE",H1dunitout,0)
      CALL IOH1D ("WRITE HEADER",H1dunitout,1)
      
!3) Loop round scan lines

      nscan = 0
      hrs1d_h_misscnlin = 0    !Not applicable

      DO iscan=1,hrs1d_h_scnlin
        CALL IOH1D ("READ DATA",H1dunitin,iscan+hrs1d_h_hdrcnt)
        required=.false.
        DO ispot=1,56
          lat = hrs1d_latlon(1,ispot)*1.0E-4
          lon = hrs1d_latlon(2,ispot)*1.0E-4
          IF (east .gt. west) THEN           !Normal case
            IF (lat.ge.south.and.lat.le.north.and.
     &          lon.ge.west.and.lon.le.east) THEN
              required=.true.
              goto 100
            ENDIF
          ELSE                               !Region straddles +-180
            IF (lat.ge.south.and.lat.le.north.and.
     &         (lon.ge.west.or.lon.le.east)) THEN
              required=.true.
              goto 100
            ENDIF
          ENDIF
        ENDDO
  100   CONTINUE
        IF (required) THEN
          nscan = nscan + 1
          CALL IOH1D ("WRITE DATA",H1dunitout,nscan+hrs1d_h_hdrcnt)
          IF (first) THEN
            hrs1d_h_startdatayr = hrs1d_scnlinyr
            hrs1d_h_startdatady = hrs1d_scnlindy
            hrs1d_h_startdatatime = hrs1d_scnlintime
            first=.false.
          ENDIF
          hrs1d_h_enddatayr = hrs1d_scnlinyr
          hrs1d_h_enddatady = hrs1d_scnlindy
          hrs1d_h_enddatatime = hrs1d_scnlintime
          hrs1d_h_scnlin = nscan
        ENDIF
      ENDDO     !Loop round scan lines
      
!3) Update header and close files

      IF (nscan.GT.0) THEN
        CALL IOH1D ("WRITE HEADER",H1dunitout,1)
        WRITE(stderru,*)'Number of scan lines selected = ',nscan
        starthh = hrs1d_h_startdatatime/3600000
        startmm = mod(hrs1d_h_startdatatime,3600000)/60000
        startss = mod(hrs1d_h_startdatatime,60000)/1000
        endhh = hrs1d_h_enddatatime/3600000
        endmm = mod(hrs1d_h_enddatatime,3600000)/60000
        endss = mod(hrs1d_h_enddatatime,60000)/1000

!Generate new file name and write in 3 lines to standard output
        IF (hrs1d_h_satid.GT.3) THEN
          satchar='N'     !NOAA
        else
          satchar='M'     !MetOp
        ENDIF
        WRITE(6,'(A5,A1,I2.2,A2,I2.2,I3.3,A2,I5.5,I2.2)') 
     &                 'hirs_',satchar,hrs1d_h_satid,'_D',
     &                 mod(hrs1d_h_startdatayr,100),
     &                 hrs1d_h_startdatady,'_B',
     &                 hrs1d_h_startorbit,
     &                 mod(hrs1d_h_endorbit,100)
        WRITE(6,'(A2,3I2.2,A2,3I2.2)')
     &       '_S',starthh,startmm,startss,'_E',endhh,endmm,endss
        WRITE(6,'(A4)') '.l1d'

      ELSE
        WRITE(stderru,*)'No scan lines within range'
      ENDIF
      CALL IOH1D ("CLOSE",H1dunitout,0)
      CALL IOH1D ("CLOSE",H1dunitin,0)
      
      END
          
