#!/bin/ksh
#
#$Id$
#
#Extract data for a defined geographical area from a HIRS 1d file
#
#Usage : select_hirs1d [-k] hirs1dfile south north west east
#
#Output: Creates new file containing selected lines from the HIRS 1d input
#   file, with an updated header. The output filename can either be generated
#   from scratch using the satid, orbit, time etc. of the data (default), or can
#   be based on the input file name (-k for "keep" specified).
#
#History:
# 1.1     08/05/06  N C Atkinson
#==============================================================================

usage(){
  echo
  echo 'usage : select_hirs1d [-k] hirs1dfile south north west east'
  echo
  echo '    (south,north are -90 to +90; west,east are -180 to +180)'
  echo
  echo '   If "-k" is specified then it keeps the original file name,'
  echo '     appended with start and end time of selected portion. Othewise'
  echo '     it generates a new file name based on satid, time, orbit, etc.'
  echo
}

keepname=N
while getopts k c      #get filename option
do
  case $c in
    k) keepname=Y;;
    *) ;;
  esac
done
shift `expr $OPTIND - 1`

if [ $# != 5 ]; then
  usage
  exit 1
fi

hirs1dfile=$1
minlat=$2
maxlat=$3
minlon=$4
maxlon=$5

. $HOME/ATOVS_ENV

if [ ! -f $hirs1dfile ]; then
  echo "Error - input file doesn't exist"
  exit 1
fi
ln -sf $hirs1dfile ${FORT}10
ln -sf tmp.l1d ${FORT}11

id=$(select_hirs1d.exe <<EOF
$minlat $maxlat $minlon $maxlon
EOF)

if [ $? != 0 ]; then
  echo "select_hirs1d.exe failed"
else
  if [ ${#id} -gt 0 ]; then
    set $id
    if [ $keepname = Y ]; then
      prefix=${hirs1dfile%.*}
      suffix=${hirs1dfile#*.}
      outfile=${prefix}${2}.${suffix}
    else
      set $id
      outfile=${1}${2}${3}
    fi
    mv tmp.l1d $outfile
    echo "created $outfile"
  fi
fi

rm -f ${FORT}10 ${FORT}11 tmp.l1d
