!+ Read level 1c AAPP header/data records and put values in BUFR array
!
      SUBROUTINE AAPP_PUT_1C_AMSUA(A1cunit, Current_scan, FOV,
     &   FOVOffset, ICENTREID, ISUBCENTRE, TimeDelay, NOBS, KEL, VALUES,
     &   ENCODE_NEDT, AAPP_VERSION)
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_put_1c_amsua.F 568 2017-11-24 13:27:53Z frna $
!
! Purpose:
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0    23/09/2010  Create from aapp_put_1c.F
! 1.1    14/09/2015  Append NEDT information to the VALUES array
! 1.2    09/12/2016  Extra arguments ENCODE_NEDT and AAPP_VERSION
! 1.3    20/01/2017  Switch to using class 33 quality descriptors for NEDT, as
!                    recommended by IPET-DRMM meeting in July 2015.
! 1.4    24/11/2017  Add ISUBCENTRE argument.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "inparms.h"
#include "ama1c.h"

! Arguments:
      INTEGER A1cunit      
      INTEGER Current_scan
      INTEGER ICENTREID   !Centre ID, e.g. 255=Missing
      INTEGER ISUBCENTRE  !Subcentre ID, e.g. 255=Missing
      INTEGER FOV         !Numbered from 1
      INTEGER FOVOffset   !Normally 0
      INTEGER TimeDelay   !Milliseconds per FOV
      INTEGER KEL         !Array elements per ob
      INTEGER NOBS        !Number of obs in BUFR message
      REAL*8 VALUES(*)    !Data values
      LOGICAL ENCODE_NEDT
      INTEGER AAPP_VERSION

! Parameters:

      REAL*8    Rmiss
      INTEGER   Imiss
      PARAMETER (Rmiss=1.7E+38)     !Missing value for BUFR
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
      INTEGER    NumBitsIn          !- size of raw data bit fields
      PARAMETER  (NumBitsIn  = 32)
      INTEGER    NumBitsOut         !- set BUFR bit field width
      PARAMETER  (NumBitsOut = 24)

! Local variables:
      INTEGER I,J
      INTEGER Irec
      INTEGER BIT                   !For use with unbitswap
      INTEGER Mask033030(NumBitsIn) !- bit mask for 033030
      INTEGER Mask033031(NumBitsIn) !- bit mask for 033031
      INTEGER Mask033032(NumBitsIn) !- bit mask for 033032
      INTEGER Mask033033(NumBitsIn) !- bit mask for 033033
      DATA Mask033030 /1,2,3,4,5,6,7,25*0/
      DATA Mask033031 /8*0,1,2,3,4,4*0,5,6,7,8,9,10,11,12,13,14,15,16,
     &                 17,3*0/
      DATA Mask033032 /26*0,1,2,3,4,5,6/
      DATA Mask033033 /0,1,14*0,16,15,14,13,12,11,10,9,8,7,6,5,4,3,2,22/
      INTEGER scnlindy,scnlinyr,FovTime  !date and time
      INTEGER Month,Day,Hour,Min
      REAL*8  Sec

! Functions
      REAL*8 FNRSCALE

!End of header
!------------------------------------------------------------------------------

            IF (FOV.EQ.1) THEN
              irec = Current_scan + ama1c_h_hdrcnt
              CALL IOA1C ("READ DATA", A1cunit, irec)
              write(*,*)'Read scan',Current_scan
            ENDIF

!Point to data in VALUES array
            NOBS = NOBS + 1
            I = (NOBS-1)*KEL

!1c dataset creation site
            VALUES(i+1)  = 3                                     !- 008070
            VALUES(i+2)  = ICENTREID                             !- 001033
            VALUES(i+3)  = ISUBCENTRE                            !- 001034       

!1b dataset creation site
            VALUES(i+4)  = 2                                     !- 008070
            VALUES(i+5)  = ICENTREID                             !- 001033
            VALUES(i+6)  = ISUBCENTRE                            !- 001034       

            VALUES(i+7)  = ama1c_h_wmosatid                      !- 001007       
            VALUES(i+8) = 3                   !Sensor ID         !- 002048
            VALUES(i+9) = FNRSCALE(ama1c_h_startorbit,1.0D0)     !- 005040
            VALUES(i+10) = FNRSCALE(ama1c_h_vnantennacorr,1.0D0) !- 025075 
            VALUES(i+11) = FNRSCALE(ama1c_scnlin,1.0D0)          !- 005041
            VALUES(I+12) = FOV + FOVOffset                       !- 005043
            VALUES(i+13) = Rmiss                                 !- 025070

            CALL BITSWAP(ama1c_qualind, Mask033030, NumBitsIn, 
     &                   Bit, NumBitsOut)
            VALUES(i+14) = 1.0D0*Bit                             !- 033030
            
            CALL BITSWAP(ama1c_scanqual, Mask033031, NumBitsIn, 
     &                   Bit, NumBitsOut)
            VALUES(i+15) = 1.0D0*Bit                             !- 033031
          
!Date and time of the FOV
            scnlindy = ama1c_scnlindy
            scnlinyr = ama1c_scnlinyr
            FovTime = ama1c_scnlintime+((FOV-1)*TimeDelay)
            CALL DAY_TO_DATE(scnlinyr,scnlindy,FovTime,
     &           Month,Day,Hour,Min,Sec)

            VALUES(i+16) = scnlinyr                              !- 004001
            VALUES(i+17) = Month                                 !- 004002
            VALUES(i+18) = Day                                   !- 004003
            VALUES(i+19) = Hour                                  !- 004004
            VALUES(i+20) = Min                                   !- 004005
            VALUES(i+21) = Sec                                   !- 004006

            VALUES(i+22) = FNRSCALE(ama1c_latlon(1,FOV),1.0D-4)  !- 005001
            VALUES(i+23) = FNRSCALE(ama1c_latlon(2,FOV),1.0D-4)  !- 006001
            VALUES(i+24) = FNRSCALE(ama1c_scalti,1.0D2)          !- 007001
            VALUES(i+25) = FNRSCALE(ama1c_angles(1,FOV),1.0D-2)  !- 007024
            IF (ama1c_angles(2,FOV).NE.Imiss .AND.
     &          ama1c_angles(2,FOV).LT.0) THEN
              ama1c_angles(2,FOV) = ama1c_angles(2,FOV) + 36000  !make +ve
            ENDIF
            VALUES(i+26) = FNRSCALE(ama1c_angles(2,FOV),1.0D-2)  !- 005021
            VALUES(i+27) = FNRSCALE(ama1c_angles(3,FOV),1.0D-2)  !- 007025
            IF (ama1c_angles(4,FOV).NE.Imiss .AND.
     &          ama1c_angles(4,FOV).LT.0) THEN
              ama1c_angles(4,FOV) = ama1c_angles(4,FOV) + 36000  !make +ve
            ENDIF
            VALUES(i+28) = FNRSCALE(ama1c_angles(4,FOV),1.0D-2)  !- 005022
            CALL BITSWAP(ama1c_dataqual(FOV), Mask033033, 
     &                    NumBitsIn, Bit, NumBitsOut)
            VALUES(i+29) = 1.0D0 * Bit                           !- 033033 

!Radiometer identifier and instr temp for HIRS, A1-1, A1-2 and B
            VALUES(i+30) = 0                                     !- 002151
            VALUES(i+31) = Rmiss                                 !- 012064
            VALUES(i+32) = 3                                     !- 002151
            VALUES(i+33) = FNRSCALE(ama1c_instrtemp1,1.0D-2)     !- 012064
            VALUES(i+34) = 4                                     !- 002151
            VALUES(i+35) = FNRSCALE(ama1c_instrtemp2,1.0D-2)     !- 012064
            VALUES(i+36) = 6                                     !- 002151
            VALUES(i+37) = Rmiss                                 !- 012064

!Loop over channels

            DO j = 1,Anchan

!TOVS/ATOVS/AVHRR instrumentation channel number (28 -> 42)
              VALUES(i+38+(j-1)*6) = j + 27                      !- 002150

!log_10 of (Temperature-radiance central wavenumber) for ATOVS
              IF (ama1c_h_tempradcnv(1,j) .NE. Imiss) THEN
                VALUES(i+39+(j-1)*6) = 
     &          log10(0.0001 * ama1c_h_tempradcnv(1,j))          !- 025076 
              ELSE
                VALUES(i+39+(j-1)*6) = Rmiss
              ENDIF

!Bandwidth correction coefficients 1 & 2
              VALUES(i+40+(j-1)*6) = 
     &              FNRSCALE(ama1c_h_tempradcnv(2,j),1.0D-6)     !- 025077
              VALUES(i+41+(j-1)*6) = 
     &              FNRSCALE(ama1c_h_tempradcnv(3,j),1.0D-6)     !- 025078


!Channel quality flags
              CALL BITSWAP(ama1c_chanqual(j), Mask033032, NumBitsIn, 
     &                     Bit, NumBitsOut)
              VALUES(i+42+(j-1)*6) = 1.0*Bit                     !- 033032

!Brightness temperatures
              VALUES(i+43+(j-1)*6) = 
     &              FNRSCALE(ama1c_btemps(j,FOV),1.0D-2)         !- 012063

            ENDDO  !- j

!NEDT information goes after the 127 existing elements.
!2 dummy entries; 127 bit map values; centre and application; 15 NEDTs; version

            IF (ENCODE_NEDT) THEN
              VALUES(i+128) = 0             !- 222000
              VALUES(i+129) = 0             !- 236000
              DO j=1,127
                VALUES(i+j+129) = 1           !- 031031 Set all to "no"
              ENDDO
              DO j=1,15
                VALUES(i+(j-1)*6+43+129) = 0  !- 031031 Reset BTs to "yes"
              ENDDO
              VALUES(i+257) = ICENTREID       !- 001031 Centre
              VALUES(i+258) = 0               !- 001032 Generating application
              DO j=1,15
                VALUES(i+j+258) = FNRSCALE(ama1c_warmnedt(j),1.0D-3) !- 033089 NEDT
              ENDDO
              VALUES(i+274) = AAPP_VERSION   !- 025060
            ELSE
              VALUES(i+128) = AAPP_VERSION   !- 025060
            ENDIF

      END
