!+ Read level 1c AAPP header/data records and put values in BUFR array
!
      SUBROUTINE AAPP_PUT_1C_ATMS(AT1cunit, Current_scan, FOV,
     &   FOVOffset, ICENTREID, ISUBCENTRE, NOBS, KEL, VALUES,
     &   AAPP_VERSION, CVALS)
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_put_1c_atms.F 509 2016-12-15 09:51:10Z frna $
!
! Purpose:
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2011, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0    04/03/2011  Initial
! 1.1    17/11/2011  Correct nedt scaling
! 1.2    12/12/2016  Extra arguments AAPP_VERSION and CVALS
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "constants_preproc.h"
#include "inparms.h"
#include "atms1c.h"

! Arguments:
      INTEGER AT1cunit      
      INTEGER Current_scan
      INTEGER ICENTREID   !Centre ID, e.g. 255=Missing
      INTEGER ISUBCENTRE  !Subcentre ID, e.g. 255=Missing
      INTEGER FOV         !Numbered from 1
      INTEGER FOVOffset   !Normally 0
      INTEGER KEL         !Array elements per ob
      INTEGER NOBS        !Number of obs in BUFR message
      REAL*8 VALUES(*)    !Data values
      INTEGER AAPP_VERSION
      CHARACTER*80 CVALS(20000)  !For character descriptors (used for CSPP version number)

! Parameters:

      REAL*8    Rmiss
      INTEGER   Imiss
      PARAMETER (Rmiss=1.7E+38)     !Missing value for BUFR
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
! Local variables:
      INTEGER I,J,chan
      INTEGER Irec
      INTEGER scnlindy,scnlinyr,FovTime  !date and time
      INTEGER Month,Day,Hour,Min
      INTEGER ios
      REAL*8  Sec
      REAL    freq
      INTEGER Polarisation(22)
      DATA Polarisation/7,7,13*6,7,6*6/       !7=QV, 6=QH

! Functions
      REAL*8 FNRSCALE

!End of header
!------------------------------------------------------------------------------

      IF (FOV.EQ.1) THEN
        irec = Current_scan + atms1c_h_hdrcnt
        CALL IOAT1C ("READ DATA", AT1cunit, irec)
        write(*,*)'Read scan',Current_scan
      ENDIF

!Point to data in VALUES array
      NOBS = NOBS + 1
      I = (NOBS-1)*KEL

      VALUES(i+1)  = atms1c_h_wmosatid                     !- 001007
      VALUES(i+2) = ICENTREID                              !- 001033
      VALUES(i+3) = ISUBCENTRE                             !- 001034       
      VALUES(i+4) = atms1c_h_instrument                    !- 002019
      VALUES(i+5) = 3                                      !- 002020

!Date and time of the FOV
      scnlindy = atms1c_scnlindy
      scnlinyr = atms1c_scnlinyr
      FovTime = atms1c_scnlintime(FOV)
      IF (FovTime .GE. 0) THEN
        CALL DAY_TO_DATE(scnlinyr,scnlindy,FovTime,
     &           Month,Day,Hour,Min,Sec)

        VALUES(i+6) = scnlinyr                                     !- 004001
        VALUES(i+7) = Month                                        !- 004002
        VALUES(i+8) = Day                                          !- 004003
        VALUES(i+9) = Hour                                         !- 004004
        VALUES(i+10) = Min                                         !- 004005
        VALUES(i+11) = Sec                                         !- 004006
      ELSE
        VALUES(i+6) = Rmiss
        VALUES(i+7) = Rmiss
        VALUES(i+8) = Rmiss
        VALUES(i+9) = Rmiss
        VALUES(i+10) = Rmiss
        VALUES(i+11) = Rmiss
      ENDIF

      VALUES(i+12) = FNRSCALE(atms1c_orbit,1.0D0)                !- 005040
      IF (atms1c_scnlin .NE. Imiss) THEN         !Limit to 8 bits
        VALUES(i+13) = MOD(atms1c_scnlin,255)                    !- 005041
      ELSE
        VALUES(i+13) = Rmiss
      ENDIF
      VALUES(I+14) = FOV + FOVOffset                             !- 005043
      VALUES(I+15) = FNRSCALE(atms1c_granulequal,1.0D0)          !- 033079
      VALUES(I+16) = FNRSCALE(atms1c_scanqual,1.0D0)             !- 033080
      VALUES(I+17) = IAND(atms1c_geolocqual,3)                   !- 033078

!Geolocation
!Note: check the nadir zenith angle is less than 2 degrees; if not, then a
!manoeuvre may be taking place so set geolocation to missing.
      IF (atms1c_latlon(1,FOV).GE.-900000 .AND.
     &    atms1c_latlon(1,FOV).LE. 900000 .AND.
     &    atms1c_latlon(2,FOV).GE.-1800000 .AND.
     &    atms1c_latlon(2,FOV).LE. 1800000 .AND.
     &    atms1c_angles(1,48) .LT. 200) THEN
        VALUES(i+18) = FNRSCALE(atms1c_latlon(1,FOV),1.0D-4)       !- 005001
        VALUES(i+19) = FNRSCALE(atms1c_latlon(2,FOV),1.0D-4)       !- 006001
        VALUES(i+20) = FNRSCALE(atms1c_scalti,1.0D2)               !- 007001
        VALUES(i+21) = FNRSCALE(atms1c_angles(1,FOV),1.0D-2)       !- 007024
        IF (atms1c_angles(2,FOV).NE.Imiss .AND.
     &      atms1c_angles(2,FOV).LT.0) THEN
          atms1c_angles(2,FOV) = atms1c_angles(2,FOV) + 36000  !make +ve
        ENDIF
        VALUES(i+22) = FNRSCALE(atms1c_angles(2,FOV),1.0D-2)       !- 005021
        VALUES(i+23) = FNRSCALE(atms1c_angles(3,FOV),1.0D-2)       !- 007025
        IF (atms1c_angles(4,FOV).NE.Imiss .AND.
     &      atms1c_angles(4,FOV).LT.0) THEN
          atms1c_angles(4,FOV) = atms1c_angles(4,FOV) + 36000  !make +ve
        ENDIF
        VALUES(i+24) = FNRSCALE(atms1c_angles(4,FOV),1.0D-2)       !- 005022
      ELSE
        IF (FOV.EQ.1) WRITE(*,*)'Bad geolocation for scan',Current_scan
        VALUES(i+18) = Rmiss
        VALUES(i+19) = Rmiss
        VALUES(i+20) = Rmiss
        VALUES(i+21) = Rmiss
        VALUES(i+22) = Rmiss
        VALUES(i+23) = Rmiss
        VALUES(i+24) = Rmiss
      ENDIF

!Channel data
      VALUES(i+25) = FNRSCALE(atms1c_h_vnantennacorr,1.0D0)      !- 025075 
      VALUES(i+26) = atms_1cnchan                                !- 031002

      DO chan = 1,atms_1cnchan
        j = i+27+(chan-1)*9

!channel number (1-22)
        VALUES(j) = chan                                         !- 002150

!centre frequency and bandwidth (Hz)
        IF (atms1c_h_tempradcnv(1,chan).NE.Imiss) THEN
          freq = C_light*atms1c_h_tempradcnv(1,chan)*1.0E-6*100
          VALUES(j+1) = freq                                     !- 002153
        ELSE
          VALUES(j+1) = Rmiss
        ENDIF
        VALUES(j+2) = 
     &       FNRSCALE(atms1c_h_bandwidth(chan),1.0D3)            !- 002154

!brightness temps
        VALUES(j+3) = Polarisation(chan)                         !- 002104
        VALUES(j+4) = FNRSCALE(atms1c_anttemps(chan,FOV),1.0D-2) !- 012066
        VALUES(j+5) = FNRSCALE(atms1c_btemps(chan,FOV),1.0D-2)   !- 012163
        IF (atms1c_nedtcold(chan) .GE. 0) THEN
          VALUES(j+6) = atms1c_nedtcold(chan)*1.0D-4             !- 012158
        ELSE
          VALUES(j+6) = Rmiss
        ENDIF
        IF (atms1c_nedtwarm(chan) .GE. 0) THEN
          VALUES(j+7) = atms1c_nedtwarm(chan)*1.0D-4             !- 012159
        ELSE
          VALUES(j+7) = Rmiss
        ENDIF
        VALUES(j+8) = FNRSCALE(atms1c_chanqual(chan),1.0D0)      !- 033081
      ENDDO
      VALUES(j+9) = AAPP_VERSION   !- 025060
      VALUES(j+10) = 1012          !- 1000*position in CVALS array + length 
      IF (NOBS.EQ.1) CVALS(1) = atms1c_h_software_version  !- 025061
      END
