!+ Read level 1c AAPP header/data records and put values in BUFR array
!
      SUBROUTINE AAPP_PUT_1C_CRIS(C1cunit, Current_scan, FofR,
     &   DETOffset, ICENTREID, NOBS, KEL, VALUES,
     &   AAPP_VERSION, CVALS)
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_put_1c_cris.F 577 2018-01-15 13:58:49Z frna $
!
! Purpose:
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2011, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0    04/03/2011  Initial
! 1.1    15/10/2012  Correct scaling in scalti
! 1.2    09/12/2016  Extra arguments AAPP_VERSION and CVALS
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "constants_preproc.h"
#include "inparms.h"
#include "cris1c.h"

! Arguments:
      INTEGER C1cunit      
      INTEGER Current_scan
      INTEGER ICENTREID   !Centre ID, e.g. 255=Missing
      INTEGER FofR         !Numbered from 1
      INTEGER DETOffset   !Normally 0
      INTEGER KEL         !Array elements per ob
      INTEGER NOBS        !Number of obs in BUFR message
      REAL*8 VALUES(*)    !Data values
      INTEGER AAPP_VERSION
      CHARACTER*80 CVALS(20000)  !For character descriptors (used for CSPP version number)

! Parameters:

      REAL*8    Rmiss
      INTEGER   Imiss
      PARAMETER (Rmiss=1.7E+38)     !Missing value for BUFR
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
! Local variables:
      INTEGER I,J,chan,DET
      INTEGER Irec
      INTEGER scnlindy,scnlinyr,FovTime  !date and time
      INTEGER Month,Day,Hour,Minute
      INTEGER ios
      INTEGER zenmin
      REAL*8  Sec

! Functions
      REAL*8 FNRSCALE

!End of header
!------------------------------------------------------------------------------

      IF (FofR.EQ.1) THEN
        irec = Current_scan + cris1c_h_hdrcnt
        CALL IOC1C ("READ DATA", C1cunit, irec)
        write(*,*)'Read scan',Current_scan
      ENDIF

      DO DET=1,9               !Loop round the 9 detectors

        NOBS = NOBS + 1
        I = (NOBS-1)*KEL

        VALUES(i+1)  = cris1c_h_wmosatid                     !- 001007
        VALUES(i+2) = ICENTREID                              !- 001033
        VALUES(i+3) = cris1c_h_instrument                    !- 002019
        VALUES(i+4) = 3                                      !- 002020

!Date and time of the FofR
        scnlindy = cris1c_scnlindy
        scnlinyr = cris1c_scnlinyr
        FovTime = cris1c_scnlintime(FofR)
        CALL DAY_TO_DATE(scnlinyr,scnlindy,FovTime,
     &           Month,Day,Hour,Minute,Sec)

        VALUES(i+5) = scnlinyr                                     !- 004001
        VALUES(i+6) = Month                                        !- 004002
        VALUES(i+7) = Day                                          !- 004003
        VALUES(i+8) = Hour                                         !- 004004
        VALUES(i+9) = Minute                                          !- 004005
        VALUES(i+10) = Sec                                         !- 004006

        VALUES(i+11) = FNRSCALE(cris1c_satxyz(1),1.0D0)            !- 304030
        VALUES(i+12) = FNRSCALE(cris1c_satxyz(2),1.0D0)            !- 304030
        VALUES(i+13) = FNRSCALE(cris1c_satxyz(3),1.0D0)            !- 304030

!Geolocation
!Note: check the nadir zenith angle is less than 3 degrees; if not, then a
!manoeuvre may be taking place so set geolocation to missing.
        zenmin = MIN(cris1c_angles(1,DET,15),cris1c_angles(1,DET,16))
        IF (cris1c_latlon(1,DET,FofR).GE.-900000 .AND.
     &      cris1c_latlon(1,DET,FofR).LE. 900000 .AND.
     &      cris1c_latlon(2,DET,FofR).GE.-1800000 .AND.
     &      cris1c_latlon(2,DET,FofR).LE. 1800000 .AND.
     &      zenmin .LT. 300) THEN
          VALUES(i+14) = FNRSCALE(cris1c_latlon(1,DET,FofR),1.0D-4)  !- 005001
          VALUES(i+15) = FNRSCALE(cris1c_latlon(2,DET,FofR),1.0D-4)  !- 006001
          VALUES(i+16) = FNRSCALE(cris1c_angles(1,DET,FofR),1.0D-2)  !- 007024
          IF (cris1c_angles(2,DET,FofR).NE.Imiss .AND.
     &        cris1c_angles(2,DET,FofR).LT.0) THEN
            cris1c_angles(2,DET,FofR) = cris1c_angles(2,DET,FofR)+36000 !make +ve
          ENDIF
          VALUES(i+17) = FNRSCALE(cris1c_angles(2,DET,FofR),1.0D-2)  !- 005021
          VALUES(i+18) = FNRSCALE(cris1c_angles(3,DET,FofR),1.0D-2)  !- 007025
          IF (cris1c_angles(4,DET,FofR).NE.Imiss .AND.
     &        cris1c_angles(4,DET,FofR).LT.0) THEN
            cris1c_angles(4,DET,FofR) = cris1c_angles(4,DET,FofR)+36000 !make +ve
          ENDIF
          VALUES(i+19) = FNRSCALE(cris1c_angles(4,DET,FofR),1.0D-2)  !- 005022
          VALUES(i+20) = FNRSCALE(cris1c_ascdesc,1.0D0)              !- 008075
        ELSE
          IF (FofR.EQ.1 .AND. DET.EQ.1)
     &         WRITE(*,*)'Bad geolocation for scan',Current_scan
          VALUES(i+14) = Rmiss
          VALUES(i+15) = Rmiss
          VALUES(i+16) = Rmiss
          VALUES(i+17) = Rmiss
          VALUES(i+18) = Rmiss
          VALUES(i+19) = Rmiss
          VALUES(i+20) = Rmiss
        ENDIF

        VALUES(i+21) = FNRSCALE(cris1c_scnlin,1.0D0)               !- 005041
        VALUES(i+22) = FofR                                        !- 005045
        VALUES(i+23) = DET                                         !- 005043  
        VALUES(i+24) = FNRSCALE(cris1c_orbit,1.0D0)                !- 005040

        VALUES(i+25) = FNRSCALE(cris1c_surfht(DET,FofR),1.0D0)     !- 010001
        VALUES(i+26) = FNRSCALE(cris1c_scalti,1.0D2)               !- 007002
        VALUES(i+27) = FNRSCALE(cris1c_landfrac(DET,FofR),1.0D-2)  !- 021166
        VALUES(i+28) = FNRSCALE(cris1c_landsea(DET,FofR),1.0D-2)   !- 008012
        VALUES(i+29) = FNRSCALE(cris1c_cloudcover(DET,FofR),1.0D-2)!- 020010
        VALUES(i+30) = FNRSCALE(cris1c_cloudtopht(DET,FofR),1.0D-2)!- 020014
        VALUES(i+31) = FNRSCALE(cris1c_radianceflgs(DET,FofR),1.0D0)!- 002165
        VALUES(i+32) = FNRSCALE(cris1c_scanqualflgs,1.0D0)         !- 033075

!Band definitions and flags
        VALUES(i+33) = 2                                           !- 008076
        VALUES(i+34) = FNRSCALE(cris1c_h_startw(1),1.0D0)          !- 006029
        VALUES(i+35) = FNRSCALE(cris1c_h_endw(1),1.0D0)            !- 006029
        VALUES(i+36) = FNRSCALE(cris1c_h_startchan(1),1.0D0)       !- 025140
        VALUES(i+37) = FNRSCALE(cris1c_h_endchan(1),1.0D0)         !- 025141
        VALUES(i+38) = FNRSCALE(cris1c_calqualflgs(1,DET),1.0D00)  !- 033076
        VALUES(i+39) = FNRSCALE(cris1c_fovqualflgs(1,DET,FofR),1.0D00)!- 033077
        VALUES(i+40) = 3                                           !- 008076
        VALUES(i+41) = FNRSCALE(cris1c_h_startw(2),1.0D0)          !- 006029
        VALUES(i+42) = FNRSCALE(cris1c_h_endw(2),1.0D0)            !- 006029
        VALUES(i+43) = FNRSCALE(cris1c_h_startchan(2),1.0D0)       !- 025140
        VALUES(i+44) = FNRSCALE(cris1c_h_endchan(2),1.0D0)         !- 025141
        VALUES(i+45) = FNRSCALE(cris1c_calqualflgs(2,DET),1.0D00)  !- 033076
        VALUES(i+46) = FNRSCALE(cris1c_fovqualflgs(2,DET,FofR),1.0D00)!- 033077
        VALUES(i+47) = 4                                           !- 008076
        VALUES(i+48) = FNRSCALE(cris1c_h_startw(3),1.0D0)          !- 006029
        VALUES(i+49) = FNRSCALE(cris1c_h_endw(3),1.0D0)            !- 006029
        VALUES(i+50) = FNRSCALE(cris1c_h_startchan(3),1.0D0)       !- 025140
        VALUES(i+51) = FNRSCALE(cris1c_h_endchan(3),1.0D0)         !- 025141
        VALUES(i+52) = FNRSCALE(cris1c_calqualflgs(3,DET),1.0D00)  !- 033076
        VALUES(i+53) = FNRSCALE(cris1c_fovqualflgs(3,DET,FofR),1.0D00)!- 033077
        VALUES(i+54) = Rmiss                                       !- 008076

!Flags
        VALUES(i+55) = FNRSCALE(cris1c_geolocqual,1.0D0)         !- 033078
        VALUES(i+56) = FNRSCALE(cris1c_quality(DET,FofR),1.0D0)  !- 033003

!Loop over channels
        VALUES(i+57) = cris1c_h_nchanused                        !- 031002 
        j = i+58
        DO chan = 1,cris1c_h_nchanused
	  IF (cris1c_h_channels(chan).le. 2046) THEN  !Only 11 bits
            VALUES(j) = cris1c_h_channels(chan)                  !- 005042
          ELSE
            VALUES(j) = cris1c_h_channels(chan)-2047
          ENDIF
          VALUES(j+1) = FNRSCALE(cris1c_radiance(DET,FofR,chan),1.0D-7)!- 014044
          j = j + 2
        ENDDO
        VALUES(j) = AAPP_VERSION   !- 025060
        VALUES(j+1) = 1012         !- 1000*position in CVALS array + length 
        IF (NOBS.EQ.1) CVALS(1) = cris1c_h_software_version  !- 025061

      ENDDO
      END
