!+ Read level 1c AAPP header/data records and put values in BUFR array
!
      SUBROUTINE AAPP_PUT_1C_IASI(I1cunit, Current_scan, FOV,
     &   FOVOffset, ICENTREID, ENHANCED_IASI, NOBS, KEL, VALUES,
     &   AAPP_VERSION)
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_put_1c_iasi.F 509 2016-12-15 09:51:10Z frna $
!
! Purpose:
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0    23/09/2010  Create from aapp_put_1c.F
! 1.1    09/12/2016  Extra argument AAPP_VERSION
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "inparms.h"
#include "iasi1c.h"

! Arguments:
      INTEGER I1cunit      
      INTEGER Current_scan
      INTEGER ICENTREID   !Centre ID, e.g. 255=Missing
      INTEGER FOV         !Numbered from 1
      INTEGER FOVOffset   !Normally 0
      LOGICAL ENHANCED_IASI !True if sequence is to be 3-40-007
      INTEGER KEL         !Array elements per ob
      INTEGER NOBS        !Number of obs in BUFR message
      REAL*8 VALUES(*)    !Data values
      INTEGER AAPP_VERSION

! Parameters:

      REAL*8    Rmiss
      INTEGER   Imiss
      PARAMETER (Rmiss=1.7E+38)     !Missing value for BUFR
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
      INTEGER    NumBitsIn          !- size of raw data bit fields
      PARAMETER  (NumBitsIn  = 32)
      INTEGER    NumBitsOut         !- set BUFR bit field width
      PARAMETER  (NumBitsOut = 24)

! Local variables:
      INTEGER I,J,K
      INTEGER Irec
      INTEGER BIT                   !For use with unbitswap
      INTEGER scnlindy,scnlinyr,FovTime  !date and time
      INTEGER Month,Day,Hour,Min
      REAL*8  Sec
      INTEGER Ichan     !Channels loop index
      INTEGER Ipc       !PC loop index
      INTEGER IBAND     !IASI band index
      INTEGER CLASS     !AVHRR cloud class
      INTEGER DET       !IASI detector (1 to 4)
      INTEGER scalrad   !IASI scaled radiance (unsigned)
      INTEGER Offset    !For use with Enhanced IASI
      INTEGER Qual      !Quality flag

!Band definitions for GQisFlagQual are derived from On Board Parameters version
!11, ref email from Tim Hultberg 2009/09/05. These are lower boundaries of the
!band overlap regions.
      INTEGER   GQisFlagQual_start_chan(3)
      DATA      GQisFlagQual_start_chan/1,1998,5117/
      INTEGER   GQisFlagQual_end_chan(3)
      DATA      GQisFlagQual_end_chan/1997,5116,8461/
      INTEGER   GQisFlagMask(3)
      DATA      GQisFlagMask/2,4,8/

! Functions
      REAL*8 FNRSCALE

!End of header
!------------------------------------------------------------------------------

            IF (FOV.EQ.1) THEN
              irec = Current_scan + iasi1c_h_hdrcnt
              CALL IOI1C ("READ DATA", I1cunit, irec)     !Read next scan
              write(*,*)'Read scan',Current_scan
            ENDIF

            DO DET=1,4               !Loop round the 4 detectors

              NOBS = NOBS + 1
              I = (NOBS-1)*KEL       !Point to data in VALUES array
          
              VALUES(i+1) = iasi1c_h_wmosatid                      !- 001007       
              VALUES(i+2) = ICENTREID                              !- 001031
              VALUES(i+3) = 221                 !221=IASI          !- 002019
              VALUES(i+4) = 61                  !61=EPS            !- 002020

!Date and time of the FOV
              scnlindy = iasi1c_scnlindy
              scnlinyr = iasi1c_scnlinyr
              FovTime = iasi1c_scnlintime(FOV)
              CALL DAY_TO_DATE(scnlinyr,scnlindy,FovTime,
     &           Month,Day,Hour,Min,Sec)

              VALUES(i+5) = scnlinyr                              !- 004001
              VALUES(i+6) = Month                                 !- 004002
              VALUES(i+7) = Day                                   !- 004003
              VALUES(i+8) = Hour                                  !- 004004
              VALUES(i+9) = Min                                   !- 004005
              VALUES(i+10) = Sec                                  !- 004006

              VALUES(i+11) = FNRSCALE(iasi1c_latlon(1,DET,FOV),1.0D-4) !- 005001
              VALUES(i+12) = FNRSCALE(iasi1c_latlon(2,DET,FOV),1.0D-4) !- 006001
              VALUES(i+13) = FNRSCALE(iasi1c_angles(1,DET,FOV),1.0D-2) !- 007024
              IF (iasi1c_angles(2,DET,FOV).NE.Imiss .AND.
     &            iasi1c_angles(2,DET,FOV).LT.0) THEN
                iasi1c_angles(2,DET,FOV)=iasi1c_angles(2,DET,FOV)+36000  !+ve
              ENDIF
              VALUES(i+14) = FNRSCALE(iasi1c_angles(2,DET,FOV),1.0D-2) !- 005021
              VALUES(i+15) = FNRSCALE(iasi1c_angles(3,DET,FOV),1.0D-2) !- 007025
              IF (iasi1c_angles(4,DET,FOV).NE.Imiss .AND.
     &            iasi1c_angles(4,DET,FOV).LT.0) THEN
                iasi1c_angles(4,DET,FOV)=iasi1c_angles(4,DET,FOV)+36000  !+ve
              ENDIF
              VALUES(i+16) = FNRSCALE(iasi1c_angles(4,DET,FOV),1.0D-2) !- 005022

              VALUES(i+17) = (FOV-1)*4 + DET + FOVOFFSET            !- 005043
              VALUES(i+18) = FNRSCALE(iasi1c_h_startorbit,1.0D0)    !- 005040
              VALUES(i+19) = FNRSCALE(iasi1c_scnlin,1.0D0)          !- 005041
              IF (iasi1c_granule.NE.Imiss) THEN
                iasi1c_granule = MOD(iasi1c_granule,256)  !fit in 8 bits
              ENDIF
              VALUES(i+20) = FNRSCALE(iasi1c_granule,1.0D0)         !- 025070
              VALUES(i+21) = FNRSCALE(iasi1c_scalti,1.0D2)          !- 007001
              IF (ENHANCED_IASI) THEN
                Offset = 8
                J = 22
                DO IBAND=1,3
                  VALUES(i+J) = GQisFlagQual_start_chan(IBAND)        !- 025140
                  VALUES(i+J+1) = GQisFlagQual_end_chan(IBAND)        !- 025141
                  IF (iasi1c_GQisFlagQual(DET,FOV).NE.Imiss) THEN
                    IF (iasi1c_GQisFlagQual(DET,FOV).GT.1) THEN  !Enhanced flag
                      VALUES(i+J+2) = IAND(iasi1c_GQisFlagQual(DET,FOV),
     &                    GQisFlagMask(IBAND))/GQisFlagMask(IBAND)   !- 033060
                    ELSE       !Either zero or the original single flag
                      VALUES(i+j+2) = 
     &                     IAND(iasi1c_GQisFlagQual(DET,FOV),1)
                    ENDIF
                  ELSE
                    VALUES(i+J+2) = Rmiss
                  ENDIF
                  J = J + 3
                ENDDO
              ELSE
                Offset = 0
                VALUES(i+22) = IAND(iasi1c_GQisFlagQual(DET,FOV),1)   !- 033060
              ENDIF
              VALUES(i+23+Offset) = iasi1c_GQisQualIndex            !- 033061
              VALUES(i+24+Offset) = iasi1c_GQisQualIndexLoc         !- 033062
              VALUES(i+25+Offset) = iasi1c_GQisQualIndexRad         !- 033063
              VALUES(i+26+Offset) = iasi1c_GQisQualIndexSpect       !- 033064
              VALUES(i+27+Offset) = iasi1c_GQisSysTecSondQual       !- 033065

                  !Detailed quality flag
              IF (ENHANCED_IASI) THEN
                VALUES(i+36) = iasi1c_GQisFlagQual(DET,FOV)/16      !- 040020
                Offset = 9
              ENDIF

!Scaling factors (zero or missing encoded as missing)
              J = 28 + Offset
              DO IBAND=1,10
                IF (iasi1c_h_startchan(IBAND).GT.0) THEN
                  VALUES(I+J) = iasi1c_h_startchan(IBAND)            !- 025140
                  VALUES(I+J+1) = iasi1c_h_endchan(IBAND)            !- 025141
                  VALUES(I+J+2) = iasi1c_h_scalefactor(IBAND)        !- 025142
                ELSE
                  VALUES(I+J) = Rmiss
                  VALUES(I+J+1) = Rmiss
                  VALUES(I+J+2) = Rmiss
                ENDIF
                J = J+3
              ENDDO

!Radiances
              DO J=1,8700        !Chan 1=645cm^-1, chan 8461=2760cm^-1
                VALUES(I+2*J+56+Offset) = J                         !- 005042
                scalrad = iasi1c_scalrad(J,DET,FOV)
                if (scalrad.LT.-5000) scalrad = scalrad + 65536
                VALUES(I+2*J+57+Offset) = scalrad                   !- 014046
              ENDDO

!AVHRR
              J = 17458 + Offset
              VALUES(i+J) = 591       !591=AVHRR/3              !- 002019
              VALUES(I+J+1) = iasi1c_avhcombination             !- 025051
              J = J + 2

              DO CLASS=1,7
                VALUES(I+J)   = FNRSCALE(iasi1c_avhrrCGy
     &                             (CLASS,DET,FOV), 1.0D-6)         !- 005060
                VALUES(I+J+1) = FNRSCALE(iasi1c_avhrrCGz
     &                             (CLASS,DET,FOV), 1.0D-6)         !- 005061
                VALUES(I+J+2) = FNRSCALE(iasi1c_avhrrfrac
     &                             (CLASS,DET,FOV), 1.0D-4)         !- 025085
                              !Convert 10^6 x fraction to percentage

                DO ICHAN=1,6
                  K = J + 3 + (ICHAN-1)*5
                  VALUES(I+K) = ICHAN                                !- 005042
                  VALUES(I+K+1) = FNRSCALE(iasi1c_avhmeanscalefactor
     &                                  (ICHAN,CLASS,DET,FOV),1.0D0) !- 025142
                  VALUES(I+K+2) = FNRSCALE(iasi1c_avhrrmean
     &                                  (ICHAN,CLASS,DET,FOV),1.0D0) !- 014047
                  VALUES(I+K+3) = FNRSCALE(iasi1c_avhsdevscalefactor
     &                                  (ICHAN,CLASS,DET,FOV),1.0D0) !- 025142
                  VALUES(I+K+4) = FNRSCALE(iasi1c_avhrrsdev
     &                                  (ICHAN,CLASS,DET,FOV),1.0D0) !- 014048
                ENDDO
                
                J = J + 33
              ENDDO  !- CLASS

!Cloud analysis
              IF (ENHANCED_IASI) THEN
                VALUES(I+J) =
     &            FNRSCALE(iasi1c_GEUMAvhrr1BCldFrac(DET,FOV),1.0D0)  !- 020081
                IF (VALUES(I+J).GT.100.AND.VALUES(I+J).NE.Rmiss)
     &              VALUES(I+J) = Rmiss
                VALUES(I+J+1) = 3                             !- 008029 (3=land)
                VALUES(I+J+2) =
     &            FNRSCALE(iasi1c_GEUMAvhrr1BLandFrac(DET,FOV),1.0D0) !- 020083
                IF (VALUES(I+J+2).GT.100.AND.VALUES(I+J+2).NE.Rmiss)
     &              VALUES(I+J+2) = Rmiss
                VALUES(I+J+3) = Rmiss                         !- 008029 (reset)
                VALUES(I+J+4) = 
     &            FNRSCALE(iasi1c_GIacAvgImagIIS(FOV),1.0D-6)         !- 040018
                VALUES(I+J+5) = 
     &            FNRSCALE(iasi1c_GIacVarImagIIS(FOV),1.0D-6)         !- 040019
                Qual = iasi1c_GEUMAvhrr1BQual(DET,FOV)
                IF (Qual.EQ.Imiss.OR.Qual.LT.0) THEN
                  VALUES(I+J+6) = Rmiss                               !- 040021
                  VALUES(I+J+7) = Rmiss                               !- 040022
                ELSE IF (Qual.LT.128) THEN
                  VALUES(I+J+6) = Qual                                !- 040021
                  VALUES(I+J+7) = Rmiss                               !- 040022
                ELSE
                  VALUES(I+J+6) = Rmiss                               !- 040021
                  VALUES(I+J+7) = MOD(Qual,128)                       !- 040022
                ENDIF
                VALUES(I+J+8) = AAPP_VERSION   !- 025060
              ELSE
                VALUES(I+J) = AAPP_VERSION     !- 025060
              ENDIF
            ENDDO  !- DET
      END
