!+ Read level 1c AAPP header/data records and put values in BUFR array
!
      SUBROUTINE AAPP_PUT_1C_MWTS2(MWTS1cunit, Current_scan, FOV,
     &   FOVOffset, ICENTREID, ISUBCENTRE, TimeDelay, NOBS, KEL, VALUES,
     &   ITHIN)
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_put_1c_mwts2.F 586 2018-04-30 10:32:45Z frna $
!
! Purpose:
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0    03/06/2014  Create from aapp_put_1c_mwts
! 1.1    26/06/2014  Round FovTime to nearest second before encoding
! 1.3    27/04/2018  Encode inland water as coast in 008012
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "inparms.h"
#include "mwts21c.h"
#include "constants_preproc.h"

! Arguments:
      INTEGER MWTS1cunit
      INTEGER Current_scan
      INTEGER ICENTREID   !Centre ID, e.g. 255=Missing
      INTEGER ISUBCENTRE  !Subcentre ID, e.g. 255=Missing
      INTEGER FOV         !Numbered from 1
      INTEGER FOVOffset   !Normally 0
      INTEGER TimeDelay   !Milliseconds per FOV
      INTEGER KEL         !Array elements per ob
      INTEGER NOBS        !Number of obs in BUFR message
      REAL*8 VALUES(*)    !Data values
      INTEGER ITHIN       !Number of samples/scans to increment

! Parameters:

      REAL*8    Rmiss
      INTEGER   Imiss
      PARAMETER (Rmiss=1.7E+38)     !Missing value for BUFR
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
      INTEGER    NumBitsIn          !- size of raw data bit fields
      PARAMETER  (NumBitsIn  = 32)
      INTEGER    NumBitsOut         !- set BUFR bit field width
      PARAMETER  (NumBitsOut = 24)

! Local variables:
      INTEGER I,J
      INTEGER Irec
      INTEGER scnlindy,scnlinyr,FovTime  !date and time
      INTEGER Month,Day,Hour,Min
      REAL*8  Sec

! Functions
      REAL*8 FNRSCALE

!End of header
!------------------------------------------------------------------------------

            IF (FOV.EQ.1) THEN
              irec = Current_scan + mwts21c_h_hdrcnt
              CALL IOMWTS21C ("READ DATA", MWTS1cunit, irec)
              write(*,*)'Read scan',Current_scan
            ENDIF

            IF (MOD(FOV-ITHIN/2-1,ITHIN) .NE. 0) THEN
              RETURN                            !This FOV is thinned out
            ENDIF

!Point to data in VALUES array
            NOBS = NOBS + 1
            I = (NOBS-1)*KEL

            VALUES(i+1)  = ICENTREID                             !- 001033
            VALUES(i+2)  = ISUBCENTRE                            !- 001034

            VALUES(i+3)  = mwts21c_h_wmosatid                    !- 001007
            VALUES(i+4)  = 954                    !MWTS-2        !- 002019
            VALUES(i+5)  = FNRSCALE(mwts21c_h_startorbit,1.0D0)  !- 005040
            VALUES(i+6)  = FNRSCALE(mwts21c_scnlin,1.0D0)        !- 005041
            VALUES(I+7)  = FOV + FOVOffset                       !- 005043
          
!Date and time of the FOV
            scnlindy = mwts21c_scnlindy
            scnlinyr = mwts21c_scnlinyr
            FovTime = mwts21c_scnlintime+((FOV-1)*TimeDelay)
            FovTime = ((FovTime + 500)/1000)*1000    !Round to nearest second
            CALL DAY_TO_DATE(scnlinyr,scnlindy,FovTime,
     &           Month,Day,Hour,Min,Sec)

            VALUES(i+8)  = scnlinyr                              !- 004001
            VALUES(i+9)  = Month                                 !- 004002
            VALUES(i+10) = Day                                   !- 004003
            VALUES(i+11) = Hour                                  !- 004004
            VALUES(i+12) = Min                                   !- 004005
            VALUES(i+13) = Sec                                   !- 004006

            VALUES(i+14) = FNRSCALE(mwts21c_latlon(1,FOV),1.0D-4)  !- 005001
            VALUES(i+15) = FNRSCALE(mwts21c_latlon(2,FOV),1.0D-4)  !- 006001

!Surface height
            VALUES(i+16) = FNRSCALE(mwts21c_surface(1,FOV),1.0D0)  !- 007002 

!Surface type AAPP 0=sea, 1=mixed, 2=land, 3=inland water; 008012 0=land, 1=sea, 2=coast
            IF (mwts21c_surface(2,FOV).EQ.0) THEN
              VALUES(i+17) = 1
            ELSE IF (mwts21c_surface(2,FOV).EQ.1) THEN
              VALUES(i+17) = 2
            ELSE IF (mwts21c_surface(2,FOV).EQ.2) THEN
              VALUES(i+17) = 0
            ELSE IF (mwts21c_surface(2,FOV).EQ.3) THEN
              VALUES(i+17) = 2   !encode inland water as coast
            ELSE
              VALUES(i+17) = Rmiss
            ENDIF

            VALUES(i+18) = FNRSCALE(mwts21c_angles(1,FOV),1.0D-2)  !- 007024
            IF (mwts21c_angles(2,FOV).NE.Imiss .AND.
     &          mwts21c_angles(2,FOV).LT.0) THEN
              mwts21c_angles(2,FOV) = mwts21c_angles(2,FOV) + 36000  !make +ve
            ENDIF
            VALUES(i+19) = FNRSCALE(mwts21c_angles(2,FOV),1.0D-2)  !- 005021
            VALUES(i+20) = FNRSCALE(mwts21c_angles(3,FOV),1.0D-2)  !- 007025
            IF (mwts21c_angles(4,FOV).NE.Imiss .AND.
     &          mwts21c_angles(4,FOV).LT.0) THEN
              mwts21c_angles(4,FOV) = mwts21c_angles(4,FOV) + 36000  !make +ve
            ENDIF
            VALUES(i+21) = FNRSCALE(mwts21c_angles(4,FOV),1.0D-2)  !- 005022

!Loop over channels

            VALUES(i+22) = mwts2_nchan                             !- 031001

            DO j = 1,mwts2_nchan

!Channel number
              VALUES(i+23+(j-1)*4) = j                            !- 005042

!Centre freq in Hz
              IF (mwts21c_h_tempradcnv(1,j) .NE. Imiss) THEN
                VALUES(i+24+(j-1)*4) = 
     &            C_light*1E-4*mwts21c_h_tempradcnv(1,j)           !- 002153 
              ELSE
                VALUES(i+24+(j-1)*4) = Rmiss
              ENDIF

!Bandwidth in Hz
              VALUES(i+25+(j-1)*4) = 
     &             FNRSCALE(mwts21c_h_bandwidth(j),1.0D3)       !- 002154

!Brightness temperatures
              VALUES(i+26+(j-1)*4) = 
     &              FNRSCALE(mwts21c_btemps(j,FOV),1.0D-2)         !- 012063

            ENDDO  !- j
      END
