!+ Read level 1c AAPP header/data records and put values in BUFR array
!
      SUBROUTINE AAPP_PUT_1C_PCIASI(Ipcunit, Current_scan, FOV,
     &   FOVOffset, ICENTREID, ENHANCED_IASI, NOBS, KEL, VALUES,
     &   AAPP_VERSION)
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_put_1c_pciasi.F 509 2016-12-15 09:51:10Z frna $
!
! Purpose:
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0    23/09/2010  Create from aapp_put_1c.F
! 1.1    09/12/2016  Extra argument AAPP_VERSION
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "inparms.h"
#include "iasipc.h"

! Arguments:
      INTEGER Ipcunit      
      INTEGER Current_scan
      INTEGER ICENTREID   !Centre ID, e.g. 255=Missing
      INTEGER FOV         !Numbered from 1
      INTEGER FOVOffset   !Normally 0
      LOGICAL ENHANCED_IASI !True if "day 2" IASI
      INTEGER KEL         !Array elements per ob
      INTEGER NOBS        !Number of obs in BUFR message
      REAL*8 VALUES(*)    !Data values
      INTEGER AAPP_VERSION

! Parameters:

      REAL*8    Rmiss
      INTEGER   Imiss
      PARAMETER (Rmiss=1.7E+38)     !Missing value for BUFR
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
      INTEGER    NumBitsIn          !- size of raw data bit fields
      PARAMETER  (NumBitsIn  = 32)
      INTEGER    NumBitsOut         !- set BUFR bit field width
      PARAMETER  (NumBitsOut = 24)

! Local variables:
      INTEGER I,J,K
      INTEGER Irec
      INTEGER BIT                   !For use with unbitswap
      INTEGER scnlindy,scnlinyr,FovTime  !date and time
      INTEGER Month,Day,Hour,Min
      REAL*8  Sec
      INTEGER Ichan     !Channels loop index
      INTEGER Ipc       !PC loop index
      INTEGER Npc       !Number of PCs
      INTEGER IBAND     !IASI band index
      INTEGER CLASS     !AVHRR cloud class
      INTEGER DET       !IASI detector (1 to 4)
      INTEGER scalrad   !IASI scaled radiance (unsigned)
      INTEGER Offset    !For use with Enhanced IASI
      INTEGER Qual      !Quality flag
      INTEGER Score     !PC score

!Band definitions for GQisFlagQual are derived from On Board Parameters version
!11, ref email from Tim Hultberg 2009/09/05. These are lower boundaries of the
!band overlap regions.
      INTEGER   GQisFlagQual_start_chan(3)
      DATA      GQisFlagQual_start_chan/1,1998,5117/
      INTEGER   GQisFlagQual_end_chan(3)
      DATA      GQisFlagQual_end_chan/1997,5116,8461/
      INTEGER   GQisFlagMask(3)
      DATA      GQisFlagMask/2,4,8/

! Functions
      REAL*8 FNRSCALE

!End of header
!------------------------------------------------------------------------------

            IF (FOV.EQ.1) THEN
              irec = Current_scan + iasipc_h_hdrcnt
              CALL IOIPC ("READ DATA", Ipcunit, irec)     !Read next scan
              write(*,*)'Read scan',Current_scan
            ENDIF

            DO DET=1,4               !Loop round the 4 detectors

              NOBS = NOBS + 1
              I = (NOBS-1)*KEL       !Point to data in VALUES array
          
              VALUES(i+1) = iasipc_h_wmosatid                      !- 001007       
              VALUES(i+2) = ICENTREID                              !- 001031
              VALUES(i+3) = 221                 !221=IASI          !- 002019
              VALUES(i+4) = 61                  !61=EPS            !- 002020

!Date and time of the FOV
              scnlindy = iasipc_scnlindy
              scnlinyr = iasipc_scnlinyr
              FovTime = iasipc_scnlintime(FOV)
              CALL DAY_TO_DATE(scnlinyr,scnlindy,FovTime,
     &           Month,Day,Hour,Min,Sec)

              VALUES(i+5) = scnlinyr                              !- 004001
              VALUES(i+6) = Month                                 !- 004002
              VALUES(i+7) = Day                                   !- 004003
              VALUES(i+8) = Hour                                  !- 004004
              VALUES(i+9) = Min                                   !- 004005
              VALUES(i+10) = Sec                                  !- 004006

              VALUES(i+11) = FNRSCALE(iasipc_latlon(1,DET,FOV),1.0D-4) !- 005001
              VALUES(i+12) = FNRSCALE(iasipc_latlon(2,DET,FOV),1.0D-4) !- 006001
              VALUES(i+13) = FNRSCALE(iasipc_angles(1,DET,FOV),1.0D-2) !- 007024
              IF (iasipc_angles(2,DET,FOV).NE.Imiss .AND.
     &            iasipc_angles(2,DET,FOV).LT.0) THEN
                iasipc_angles(2,DET,FOV)=iasipc_angles(2,DET,FOV)+36000  !+ve
              ENDIF
              VALUES(i+14) = FNRSCALE(iasipc_angles(2,DET,FOV),1.0D-2) !- 005021
              VALUES(i+15) = FNRSCALE(iasipc_angles(3,DET,FOV),1.0D-2) !- 007025
              IF (iasipc_angles(4,DET,FOV).NE.Imiss .AND.
     &            iasipc_angles(4,DET,FOV).LT.0) THEN
                iasipc_angles(4,DET,FOV)=iasipc_angles(4,DET,FOV)+36000  !+ve
              ENDIF
              VALUES(i+16) = FNRSCALE(iasipc_angles(4,DET,FOV),1.0D-2) !- 005022

              VALUES(i+17) = (FOV-1)*4 + DET + FOVOFFSET            !- 005043
              VALUES(i+18) = FNRSCALE(iasipc_h_startorbit,1.0D0)    !- 005040
              VALUES(i+19) = FNRSCALE(iasipc_scnlin,1.0D0)          !- 005041
              IF (iasipc_granule.NE.Imiss) THEN
                iasipc_granule = MOD(iasipc_granule,256)  !fit in 8 bits
              ENDIF
              VALUES(i+20) = FNRSCALE(iasipc_granule,1.0D0)         !- 025070
              VALUES(i+21) = FNRSCALE(iasipc_scalti,1.0D2)          !- 007001
              J = 22
              DO IBAND=1,3
                VALUES(i+J) = GQisFlagQual_start_chan(IBAND)        !- 025140
                VALUES(i+J+1) = GQisFlagQual_end_chan(IBAND)        !- 025141
                IF (iasipc_GQisFlagQual(DET,FOV).NE.Imiss) THEN
                  IF (iasipc_GQisFlagQual(DET,FOV).GT.1) THEN  !Enhanced flag
                    VALUES(i+J+2) = IAND(iasipc_GQisFlagQual(DET,FOV),
     &                    GQisFlagMask(IBAND))/GQisFlagMask(IBAND)   !- 033060
                  ELSE       !Either zero or the original single flag
                    VALUES(i+j+2) = IAND(iasipc_GQisFlagQual(DET,FOV),1)
                  ENDIF
                ELSE
                  VALUES(i+J+2) = Rmiss
                ENDIF
                J = J + 3
              ENDDO
              VALUES(i+31) = iasipc_GQisQualIndex                   !- 033061
              VALUES(i+32) = iasipc_GQisQualIndexLoc                !- 033062
              VALUES(i+33) = iasipc_GQisQualIndexRad                !- 033063
              VALUES(i+34) = iasipc_GQisQualIndexSpect              !- 033064
              VALUES(i+35) = iasipc_GQisSysTecSondQual              !- 033065
                  !Detailed quality flag
              VALUES(i+36) = iasipc_GQisFlagQual(DET,FOV)/16        !- 040020

!Channel selection (zero or missing encoded as missing)
              J = 37
              DO IBAND=1,10
                IF (iasipc_h_startchan(IBAND).GT.0) THEN
                  VALUES(I+J) = iasipc_h_startchan(IBAND)            !- 025140
                  VALUES(I+J+1) = iasipc_h_endchan(IBAND)            !- 025141
                  VALUES(I+J+2) = iasipc_h_scalefactor(IBAND)        !- 025142
                ELSE
                  VALUES(I+J) = Rmiss
                  VALUES(I+J+1) = Rmiss
                  VALUES(I+J+2) = Rmiss
                ENDIF
                J = J+3
              ENDDO
              VALUES(I+67) = iasipc_h_nchanused                      !-031002
              J = 68
              DO ICHAN=1,iasipc_h_nchanused
                VALUES(I+J) = iasipc_h_channels(ICHAN)               !-005042
                VALUES(I+J+1) = iasipc_scalrad(ICHAN,DET,FOV)        !-014046
                J = J+2
              ENDDO

!Instrument band definition
              DO IBAND=1,3
                IF (iasipc_h_startpcchan(IBAND).GT.0) THEN
                  VALUES(I+J) = iasipc_h_startpcchan(IBAND)           !- 025140
                  VALUES(I+J+1) = iasipc_h_endpcchan(IBAND)           !- 025141
                  VALUES(I+J+2) = iasipc_h_quantization(IBAND)*1.0E-2 !- 040015
                  VALUES(I+J+4) = iasipc_h_evfilevn(IBAND)             !- 025062
                ELSE
                  VALUES(I+J) = Rmiss
                  VALUES(I+J+1) = Rmiss
                  VALUES(I+J+2) = Rmiss
                  VALUES(I+J+4) = Rmiss
                ENDIF
                IF (iasipc_logfit(IBAND,DET,FOV).NE.Imiss) THEN
                  VALUES(I+J+3) = 
     &               10.0**(iasipc_logfit(IBAND,DET,FOV)*1E-4)       !- 040016
                ELSE
                  VALUES(I+J+3) = Rmiss
                ENDIF

!Principal component scores for band
                Npc = iasipc_h_endpc(IBAND)-iasipc_h_startpc(IBAND)+1
                VALUES(I+J+5) = Npc                                  !- 031002
                DO Ipc=1,Npc
                  K = iasipc_h_startpc(IBAND)+Ipc-1
                  Score = iasipc_scores(K,DET,FOV)   !Convert to 4-byte Integer
                  VALUES(I+J+5+Ipc) = FNRSCALE(Score,1.0D0)          !- 040017
                ENDDO
                J = J + 6 + Npc
              ENDDO  !- IBAND

!AVHRR
              VALUES(i+J) = 591       !591=AVHRR/3                !- 002019
              VALUES(i+J+1) = iasipc_avhcombination               !- 025051
              J = J + 2

              DO CLASS=1,7
                VALUES(I+J)   = FNRSCALE(iasipc_avhrrCGy
     &                             (CLASS,DET,FOV), 1.0D-6)         !- 005060
                VALUES(I+J+1) = FNRSCALE(iasipc_avhrrCGz
     &                             (CLASS,DET,FOV), 1.0D-6)         !- 005061
                VALUES(I+J+2) = FNRSCALE(iasipc_avhrrfrac
     &                             (CLASS,DET,FOV), 1.0D-4)         !- 025085
                              !Convert 10^6 x fraction to percentage

                DO ICHAN=1,6
                  K = J + 3 + (ICHAN-1)*5
                  VALUES(I+K) = ICHAN                                !- 005042
                  VALUES(I+K+1) = FNRSCALE(iasipc_avhmeanscalefactor
     &                                  (ICHAN,CLASS,DET,FOV),1.0D0) !- 025142
                  VALUES(I+K+2) = FNRSCALE(iasipc_avhrrmean
     &                                  (ICHAN,CLASS,DET,FOV),1.0D0) !- 014047
                  VALUES(I+K+3) = FNRSCALE(iasipc_avhsdevscalefactor
     &                                  (ICHAN,CLASS,DET,FOV),1.0D0) !- 025142
                  VALUES(I+K+4) = FNRSCALE(iasipc_avhrrsdev
     &                                  (ICHAN,CLASS,DET,FOV),1.0D0) !- 014048
                ENDDO
                
                J = J + 33
              ENDDO  !- CLASS

!Cloud analysis
              VALUES(I+J) =
     &            FNRSCALE(iasipc_GEUMAvhrr1BCldFrac(DET,FOV),1.0D0)  !- 020081
              IF (VALUES(I+J).GT.100.AND.VALUES(I+J).NE.Rmiss)
     &            VALUES(I+J) = Rmiss
              VALUES(I+J+1) = 3                               !- 008029 (3=land)
              VALUES(I+J+2) =
     &            FNRSCALE(iasipc_GEUMAvhrr1BLandFrac(DET,FOV),1.0D0) !- 020083
              IF (VALUES(I+J+2).GT.100.AND.VALUES(I+J+2).NE.Rmiss)
     &            VALUES(I+J+2) = Rmiss
              VALUES(I+J+3) = Rmiss                           !- 008029 (reset)
              VALUES(I+J+4) = 
     &            FNRSCALE(iasipc_GIacAvgImagIIS(FOV),1.0D-6)         !- 040018
              VALUES(I+J+5) = 
     &            FNRSCALE(iasipc_GIacVarImagIIS(FOV),1.0D-6)         !- 040019
              Qual = iasipc_GEUMAvhrr1BQual(DET,FOV)
              IF (Qual.EQ.Imiss.OR.Qual.LT.0) THEN
                VALUES(I+J+6) = Rmiss                                 !- 040021
                VALUES(I+J+7) = Rmiss                                 !- 040022
              ELSE IF (Qual.LT.128) THEN
                VALUES(I+J+6) = Qual                                  !- 040021
                VALUES(I+J+7) = Rmiss                                 !- 040022
              ELSE
                VALUES(I+J+6) = Rmiss                                 !- 040021
                VALUES(I+J+7) = MOD(Qual,128)                         !- 040022
              ENDIF
              VALUES(I+J+8) = AAPP_VERSION   !- 025060
            ENDDO  !- DET
      END
