!+ Read level 1c AAPP header/data records and put values in BUFR array
!
      SUBROUTINE AAPP_PUT_1D_AMSUB(B1Dunit, Current_scan, FOV,
     &   FOVOffset, ICENTREID, TimeDelay, NOBS, KEL, VALUES)
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_put_1d_amsub.F 400 2014-06-26 08:49:16Z frna $
!
! Purpose:
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2011, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0    17/06/2011  NCA. Format is that of AMSUB at Met Office.
! 1.1    26/06/2014  Round FovTime to nearest second before encoding
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "inparms.h"
#include "amb1d.h"

! Arguments:
      INTEGER B1Dunit      
      INTEGER Current_scan
      INTEGER ICENTREID   !Centre ID, e.g. 255=Missing
      INTEGER FOV         !Numbered from 1
      INTEGER FOVOffset   !Normally 0
      INTEGER TimeDelay   !Milliseconds per FOV
      INTEGER KEL         !Array elements per ob
      INTEGER NOBS        !Number of obs in BUFR message
      REAL*8 VALUES(*)    !Data values

! Parameters:

      REAL*8    Rmiss
      INTEGER   Imiss
      PARAMETER (Rmiss=1.7E+38)     !Missing value for BUFR
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
      INTEGER    NumBitsIn          !- size of raw data bit fields
      PARAMETER  (NumBitsIn  = 32)
      INTEGER    NumBitsOut         !- set BUFR bit field width
      PARAMETER  (NumBitsOut = 24)

! Local variables:
      INTEGER I,J
      INTEGER Irec
      INTEGER BIT                   !For use with unbitswap
      INTEGER Mask033030(NumBitsIn) !- bit mask for 033030
      INTEGER Mask033031(NumBitsIn) !- bit mask for 033031
      INTEGER Mask025214(NumBitsIn) !- bit mask for 025214
      INTEGER Mask025215(NumBitsIn) !- bit mask for 025215
      DATA Mask033030 /1,2,3,4,5,6,7,25*0/
      DATA Mask033031 /8*0,1,2,3,4,4*0,5,6,7,8,9,10,11,12,13,14,15,16,
     &                 17,3*0/
      DATA Mask025214 /1,2,3,4,5,6,7,8,9,10,18*0,11,12,13,14/
      DATA Mask025215 /0,1,8*0,2,22,21,20,19,18,17,16,15,14,13,12,11,10,
     &                 9,8,7,6,5,4,3,23/

      INTEGER scnlindy,scnlinyr,FovTime  !date and time
      INTEGER Month,Day,Hour,Min
      REAL*8  Sec

! Functions
      REAL*8 FNRSCALE

!End of header
!------------------------------------------------------------------------------

      IF (FOV.EQ.1) THEN
        irec = Current_scan + amb1d_h_hdrcnt
        CALL IOB1D ("READ DATA", B1Dunit, irec)
        write(*,*)'Read scan',Current_scan
      ENDIF

!Point to data in VALUES array
      NOBS = NOBS + 1
      I = (NOBS-1)*KEL

      VALUES(i+1) = amb1d_h_wmosatid              !- 001007       

!Date and time of the FOV
      scnlindy = amb1d_scnlindy
      scnlinyr = amb1d_scnlinyr
      FovTime = amb1d_scnlintime+((FOV-1)*TimeDelay)
      FovTime = ((FovTime + 500)/1000)*1000    !Round to nearest second
      CALL DAY_TO_DATE(scnlinyr,scnlindy,FovTime,
     &    Month,Day,Hour,Min,Sec)

      VALUES(i+2) = scnlinyr                              !- 004001
      VALUES(i+3) = Month                                 !- 004002
      VALUES(i+4) = Day                                   !- 004003
      VALUES(i+5) = Hour                                  !- 004004
      VALUES(i+6) = Min                                   !- 004005
      VALUES(i+7) = Sec                                   !- 004006

      VALUES(i+8) = FNRSCALE(amb1d_latlon(1,FOV),1.0D-4)  !- 005001
      VALUES(i+9) = FNRSCALE(amb1d_latlon(2,FOV),1.0D-4)  !- 006001

! surface type: raw 1d data  : 0 = Sea, 1 = Mixed, 2 = Land
!             : BUFR 1d data : 0 = Land, 1 = Sea, 2 = Coast    !- 008012 

      IF (amb1d_surface(2,FOV) .EQ. 0) THEN
        VALUES(i+10) = 1
      ELSEIF (amb1d_surface(2,FOV) .EQ. 1) THEN
        VALUES(i+10) = 2
      ELSEIF (amb1d_surface(2,FOV) .EQ. 2) THEN
        VALUES(i+10) = 0
      ELSE
        VALUES(i+10) = Rmiss
      ENDIF

!1d dataset creation site
      VALUES(i+11)  = 4                                     !- 008070
      VALUES(i+12)  = ICENTREID                             !- 001031

!1b dataset creation site
      VALUES(i+13)  = 2                                     !- 008070
      VALUES(i+14)  = ICENTREID                             !- 001031

!orbit
      VALUES(i+15) = FNRSCALE(amb1d_h_startorbit,1.0D0)     !- 005040

!atovpp version number
      VALUES(i+16) = FNRSCALE(amb1d_h_atovppvn,1.0D0)       !- 055023

!scan line and FOV
      VALUES(i+17) = FNRSCALE(amb1d_scnlin,1.0D0)           !- 005041
      VALUES(I+18) = FOV + FOVOffset                        !- 005043

!quality flags
      CALL BITSWAP(amb1d_qualind, Mask033030, NumBitsIn, 
     &                   Bit, NumBitsOut)
      VALUES(i+19) = 1.0D0*Bit                              !- 033030
      CALL BITSWAP(amb1d_scanqual, Mask033031, NumBitsIn, 
     &                   Bit, NumBitsOut)
      VALUES(i+20) = 1.0D0*Bit                              !- 033031

!preprocessing flags
      CALL BITSWAP(amb1d_prepro(1,FOV), Mask025214, NumBitsIn, 
     &                   Bit, NumBitsOut)
      VALUES(i+21) = 1.0D0*Bit                              !- 025214
      IF (amb1d_prepro(2,FOV) .GT. 0) THEN
        VALUES(i+22) = amb1d_prepro(2,FOV) - 1              !- 008205
      ELSE
        VALUES(i+22) = Rmiss
      ENDIF
      VALUES(i+23) = FNRSCALE(amb1d_prepro(3,FOV),1.0D-2)   !- 002224
      IF (VALUES(i+23).GT.163.82) VALUES(i+23) = Rmiss
      VALUES(i+24) = FNRSCALE(amb1d_prepro(4,FOV),1.0D-2)   !- 025206
      IF (VALUES(i+24).GT.250.0) VALUES(i+24) = Rmiss

!navigation
      VALUES(i+25) = FNRSCALE(amb1d_scalti,1.0D2)           !- 007001
      VALUES(i+26) = FNRSCALE(amb1d_surface(1,FOV),1.0D0)   !- 010001
      VALUES(i+27) = FNRSCALE(amb1d_angles(1,FOV),1.0D-2)   !- 007024
      IF (amb1d_angles(2,FOV).NE.Imiss .AND.
     &    amb1d_angles(2,FOV).LT.0) THEN
        amb1d_angles(2,FOV)=amb1d_angles(2,FOV)+36000  !+ve
      ENDIF
      VALUES(i+28) = FNRSCALE(amb1d_angles(2,FOV),1.0D-2)     !- 005021
      VALUES(i+29) = FNRSCALE(amb1d_angles(3,FOV),1.0D-2)     !- 007025
      IF (amb1d_angles(4,FOV).NE.Imiss .AND.
     &    amb1d_angles(4,FOV).LT.0) THEN
        amb1d_angles(4,FOV)=amb1d_angles(4,FOV)+36000  !+ve
      ENDIF
      VALUES(i+30) = FNRSCALE(amb1d_angles(4,FOV),1.0D-2)     !- 005022

!Instrument temperatures
      VALUES(i+31) = 6                                        !- 002151
      VALUES(i+32) = FNRSCALE(amb1d_instrtemp,1.0D-2)         !- 012064

!Version numbers
      VALUES(i+33) = FNRSCALE(amb1d_h_Bcorrvn,1.0D0)          !- 055024
      VALUES(i+34) = FNRSCALE(amb1d_h_Bpcpvn,1.0D0)           !- 055026

!Microwave cirrus index 
      IF (amb1d_prepro(5,FOV) .GT. Imiss) THEN
        VALUES(i+35) = amb1d_prepro(5,FOV)*1.0D-3             !- 025208
      ELSE
        VALUES(i+35) = Rmiss
      ENDIF

!-------------------------------
!Loop over AMSU-B channels 1-5
!-------------------------------
      DO j = 1,Bnchan

!TOVS/ATOVS/AVHRR instrumentation channel number (43 -> 47)
        VALUES(i+36+(j-1)*5) = j + 42                          !- 002150

!log_10 of (Temperature-radiance central wavenumber (per metre)) for ATOVS
        IF (amb1d_h_amsPlanck(1,j+15) .NE. Imiss) THEN
           VALUES(i+37+(j-1)*5) = 
     &        log10(0.0001 * amb1d_h_amsPlanck(1,j+15))            !-025076
        ELSE
          VALUES(i+37+(j-1)*5) = Rmiss
        ENDIF

!Bandwidth correction coefficients 1 & 2
        VALUES(i+38+(j-1)*5) = 
     &     FNRSCALE(amb1d_h_amsPlanck(2,j+15),1.0D-6)              !- 025077
        VALUES(i+39+(j-1)*5) = 
     &     FNRSCALE(amb1d_h_amsPlanck(3,j+15),1.0D-6)              !- 025078

!Brightness temperatures
        VALUES(i+40+(j-1)*5) = FNRSCALE(amb1d_btemps(j+15,FOV),1.0D-2) !-012063

      ENDDO  !- j

      END
