!+ Read level 1c AAPP header/data records and put values in BUFR array
!
      SUBROUTINE AAPP_PUT_1D_ATMS(A1dunit, Current_scan, FOV,
     &   FOVOffset, ICENTREID, ISUBCENTRE, NOBS, KEL, VALUES, ITHIN)
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_put_1d_atms.F 317 2012-07-03 10:38:38Z frna $
!
! Purpose:
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0    03/11/2010  Create from aapp_put_1d_cris.F
! 1.1    03/07/2012  Correct NEdT scaling (make it same as the hdf5 SDR) 
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "inparms.h"
#include "atms1d.h"

! Arguments:
      INTEGER A1dunit      
      INTEGER Current_scan
      INTEGER ICENTREID   !Centre ID, e.g. 255=Missing
      INTEGER ISUBCENTRE  !Subcentre ID, e.g. 255=Missing
      INTEGER FOV         !Numbered from 1
      INTEGER FOVOffset   !Normally 0
      INTEGER KEL         !Array elements per ob
      INTEGER NOBS        !Number of obs in BUFR message
      REAL*8 VALUES(*)    !Data values
      INTEGER ITHIN       !Number of samples/scans to increment

! Parameters:

      REAL*8    Rmiss
      INTEGER   Imiss
      PARAMETER (Rmiss=1.7E+38)     !Missing value for BUFR
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
      INTEGER    NumBitsIn          !- size of raw data bit fields
      PARAMETER  (NumBitsIn  = 32)
      INTEGER    NumBitsOut         !- set BUFR bit field width
      PARAMETER  (NumBitsOut = 24)
      
! Local variables:
      INTEGER I,J,K
      INTEGER Irec
      INTEGER BIT                   !For use with unbitswap
      INTEGER Mask025214(NumBitsIn) !- bit mask for 025214
      INTEGER Mask025220(NumBitsIn) !- bit mask for 025220
      DATA Mask025214 /1,2,3,4,5,6,7,8,9,10,18*0,11,12,13,14/
      DATA Mask025220 /6*0,25,24,23,22,21,20,19,18,17,16,15,14,13,   !- FOV qual flgs
     &                 12,11,10,9,8,7,6,5,4,3,2,1,26/
      INTEGER scnlindy,scnlinyr,FovTime  !date and time
      INTEGER Month,Day,Hour,Min
      REAL*8  Sec
      INTEGER Ichan     !Channels loop index
      INTEGER Ipc       !PC loop index
      INTEGER IBAND     !IASI band index
      INTEGER CLASS     !AVHRR cloud class
      INTEGER DET       !IASI detector (1 to 4)
      INTEGER scalrad   !IASI scaled radiance (unsigned)
      INTEGER Qual      !Quality flag
      INTEGER IOS
      INTEGER midlat_old
      DATA midlat_old/MDI/

! Functions
      REAL*8 FNRSCALE

!End of header
!------------------------------------------------------------------------------

      IF (FOV.EQ.1) THEN
        IF (Current_scan.NE.1)
     &     midlat_old = atms1d_latlon(2,atms_1dnfov/2)
        irec = Current_scan + atms1d_h_hdrcnt
        CALL IOAT1D ("READ DATA", A1dunit, irec)     !Read next scan
        write(*,*)'Read scan',Current_scan
      ENDIF

      IF (MOD(FOV-ITHIN/2-1,ITHIN) .NE. 0) THEN
        RETURN                            !This FOV is thinned out
      ENDIF
       
      NOBS = NOBS + 1
      I = (NOBS-1)*KEL       !Point to data in VALUES array
          
      VALUES(i+1) = atms1d_h_wmosatid              !- 001007       
      VALUES(i+2) = 4                              !- 008070
      VALUES(i+3) = ICENTREID                      !- 001033
      VALUES(i+4) = ISUBCENTRE                     !- 001034
      VALUES(i+5) = 2                              !- 008070
      VALUES(i+6) = atms1d_h_centre                !- 001033
      VALUES(i+7) = atms1d_h_subcentre             !- 001034

!621=ATMS
      VALUES(i+8) = 621                            !- 002019

!Date and time of the FOV
      scnlindy = atms1d_scnlindy
      scnlinyr = atms1d_scnlinyr
      FovTime = atms1d_scnlintime(FOV)
      CALL DAY_TO_DATE(scnlinyr,scnlindy,FovTime,
     &           Month,Day,Hour,Min,Sec)

      VALUES(i+9) = scnlinyr                       !- 004001
      VALUES(i+10) = Month                          !- 004002
      VALUES(i+11) = Day                            !- 004003
      VALUES(i+12) = Hour                          !- 004004
      VALUES(i+13) = Min                           !- 004005
      VALUES(i+14) = Sec                           !- 004006

      VALUES(i+15) = Rmiss       !Sat position     !- 027031
      VALUES(i+16) = Rmiss                         !- 028031
      VALUES(i+17) = Rmiss                         !- 010031

!Geolocation
!Note: check the nadir zenith angle is less than 2 degrees; if not, then a
!manoeuvre may be taking place so set geolocation to missing.
      IF (atms1d_latlon(1,FOV).GE.-900000 .AND.
     &    atms1d_latlon(1,FOV).LE. 900000 .AND.
     &    atms1d_latlon(2,FOV).GE.-1800000 .AND.
     &    atms1d_latlon(2,FOV).LE. 1800000 .AND.
     &    atms1d_angles(1,48) .LT. 200) THEN
        VALUES(i+18) = FNRSCALE(atms1d_latlon(1,FOV),1.0D-4) !- 005001
        VALUES(i+19) = FNRSCALE(atms1d_latlon(2,FOV),1.0D-4) !- 006001
        VALUES(i+20) = FNRSCALE(atms1d_angles(1,FOV),1.0D-2) !- 007024
        IF (atms1d_angles(2,FOV).NE.Imiss .AND.
     &      atms1d_angles(2,FOV).LT.0) THEN
          atms1d_angles(2,FOV)=atms1d_angles(2,FOV)+36000  !+ve
        ENDIF
        VALUES(i+21) = FNRSCALE(atms1d_angles(2,FOV),1.0D-2)     !- 005021
        VALUES(i+22) = FNRSCALE(atms1d_angles(3,FOV),1.0D-2)     !- 007025
        IF (atms1d_angles(4,FOV).NE.Imiss .AND.
     &    atms1d_angles(4,FOV).LT.0) THEN
          atms1d_angles(4,FOV)=atms1d_angles(4,FOV)+36000  !+ve
        ENDIF
        VALUES(i+23) = FNRSCALE(atms1d_angles(4,FOV),1.0D-2)     !- 005022

!Ascending/descending flag is not in the ATMS stream, so work it out

        IF (midlat_old .NE. MDI) THEN
          IF (atms1d_latlon(2,atms_1dnfov/2) .GT. midlat_old) THEN
            VALUES(i+24) = 0                                     !- 008075
          ELSE
            VALUES(i+24) = 1
          ENDIF
        ELSE
          VALUES(i+24) = Rmiss
        ENDIF
      ELSE
        IF (FOV.EQ.1) WRITE(*,*)'Bad geolocation for scan',Current_scan
        VALUES(i+18) = Rmiss
        VALUES(i+19) = Rmiss
        VALUES(i+20) = Rmiss
        VALUES(i+21) = Rmiss
        VALUES(i+22) = Rmiss
        VALUES(i+23) = Rmiss
        VALUES(i+24) = Rmiss
      ENDIF

      VALUES(i+25) = FNRSCALE(atms1d_scnlin,1.0D0)             !- 005041
      VALUES(i+26) = Rmiss               !Field of regard      !- 005045
      VALUES(i+27) = FOV                                       !- 005043

      VALUES(i+28) = FNRSCALE(atms1d_orbit,1.0D0)              !- 005040

! surface height (metres)
      VALUES(i+29) = FNRSCALE(atms1d_surface(1,FOV),1.0D0)     !- 010001

! satellite altitude
      VALUES(i+30) = FNRSCALE(atms1d_scalti,1.0D2)             !- 007002
      VALUES(i+31) = Rmiss                  !Land fraction     !- 021166

! surface type: raw 1d data  : 0 = Sea, 1 = Mixed, 2 = Land
!             : BUFR 1d data : 0 = Land, 1 = Sea, 2 = Coast    !- 008012 

      IF (atms1d_surface(2,FOV) .EQ. 0) THEN
        VALUES(i+32) = 1
      ELSEIF (atms1d_surface(2,FOV) .EQ. 1) THEN
        VALUES(i+32) = 2
      ELSEIF (atms1d_surface(2,FOV) .EQ. 2) THEN
        VALUES(i+32) = 0
      ELSE
        VALUES(i+32) = Rmiss
      ENDIF

      VALUES(i+33) = Rmiss                  !Cloud cover       !- 020010
      VALUES(i+34) = Rmiss                  !Cloud top height  !- 020014
      VALUES(i+35) = IAND(atms1d_geolocqual,3)                 !- 033078

!ATOVPP derived quantities

      VALUES(i+36) = FNRSCALE(atms1d_h_atovppvn,1.0D0)         !- 055023
      CALL BITSWAP(atms1d_prepro(1,FOV), Mask025214, NumBitsIn, 
     &                   Bit, NumBitsOut)
      VALUES(i+37) = 1.0D0*Bit                                 !- 025214
      IF (atms1d_prepro(2,FOV) .GT. 0) THEN
        VALUES(i+38) = atms1d_prepro(2,FOV) - 1                !- 008205
      ELSE
        VALUES(i+38) = Rmiss
      ENDIF
      VALUES(i+39) = FNRSCALE(atms1d_prepro(3,FOV),1.0D-2)     !- 002224
      IF (VALUES(i+39).GT.163.82) VALUES(i+39) = Rmiss
      VALUES(i+40) = FNRSCALE(atms1d_prepro(4,FOV),1.0D-2)     !- 025206
      IF (VALUES(i+40).GT.250.0) VALUES(i+40) = Rmiss
      VALUES(i+41) = FNRSCALE(atms1d_prepro(5,FOV),1.0D-2)     !- 025206
      IF (VALUES(i+41).GT.250.0) VALUES(i+41) = Rmiss
      VALUES(i+42) = FNRSCALE(atms1d_prepro(6,FOV),1.0D-2)     !- 025207
      IF (VALUES(i+42).GT.126.0) VALUES(i+42) = Rmiss
      VALUES(i+43) = FNRSCALE(atms1d_prepro(7,FOV),1.0D-2)     !- 025206
      IF (VALUES(i+43).GT.250.0) VALUES(i+43) = Rmiss
      IF (atms1d_atmsqual(FOV) .NE. Imiss) THEN
        CALL BITSWAP(atms1d_atmsqual(FOV), Mask025220, NumBitsIn, 
     &                   Bit, 26)
        VALUES(i+44) = 1.0*Bit                                 !- 025220
      ELSE
        VALUES(i+44) = Rmiss
      ENDIF

!Re-mapped ATMS

      VALUES(i+45) = 621                                       !- 002019
      VALUES(i+46) = FNRSCALE(atms1d_granulequal,1.0D0)        !- 033079
      VALUES(i+47) = FNRSCALE(atms1d_scanqual,1.0D0)           !- 033080
      DO J=1,22
        VALUES(i+40+J*8) = J                                   !- 005042
        IF (atms1d_h_Planck(1,J).NE.Imiss) THEN
          VALUES(i+41+J*8) = ALOG10(atms1d_h_Planck(1,J)*1E-4) !- 025076
        ELSE
          VALUES(i+41+J*8) = Rmiss
        ENDIF
        VALUES(i+42+J*8) = FNRSCALE(atms1d_h_Planck(2,J),1D-6) !- 025077
        VALUES(i+43+J*8) = FNRSCALE(atms1d_h_Planck(3,J),1D-6) !- 025077
        VALUES(i+44+J*8) = FNRSCALE(atms1d_btemps(J,FOV),1D-2) !- 012163
        VALUES(i+45+J*8) = FNRSCALE(atms1d_nedtcold(J),1D-4)   !- 012158
        VALUES(i+46+J*8) = FNRSCALE(atms1d_nedtwarm(J),1D-4)   !- 012159
        VALUES(i+47+J*8) = FNRSCALE(atms1d_chanqual(J), 1.0D0) !- 033081
      ENDDO
      VALUES(i+224) = FNRSCALE(atms1d_h_beamversion,1D0)       !- 055007

!CrIS radiance data

      VALUES(i+225) = 620                                      !- 002019
      VALUES(i+226) = Rmiss                                    !- 002165 
      VALUES(i+227) = Rmiss                                    !- 033075
      DO J=1,3
        VALUES(i+221+J*7) = Rmiss                              !- 008076 
        VALUES(i+222+J*7) = Rmiss                              !- 006029
        VALUES(i+223+J*7) = Rmiss                              !- 006029
        VALUES(i+224+J*7) = Rmiss                              !- 025140
        VALUES(i+225+J*7) = Rmiss                              !- 025141
        VALUES(i+226+J*7) = Rmiss                              !- 033076
        VALUES(i+227+J*7) = Rmiss                              !- 033076
      ENDDO
      VALUES(i+249) = Rmiss                                    !- 033003
      VALUES(i+250) = Rmiss                                    !- 055007
      VALUES(i+251) = 0            !Number of CrIS channels    !- 031002
      K = i+250
!      DO J=1,0
!        K = i+250+2*J
!        VALUES(K) = Rmiss                                      !- 005042
!        VALUES(K+1) = Rmiss                                    !- 014044
!      ENDDO
      K = K+1
      VALUES(K+1) = Rmiss                                      !- 055007
      VALUES(K+2) = Rmiss                                      !- 040026
      VALUES(K+3) = 0              !Number of CrIS PCs         !- 031002
!      DO J=1,0
!        VALUES(K+3+J) = Rmiss                                  !- 025050
!      ENDDO
      K = K+3+0
      DO J=1,3
        VALUES(K+J) = Rmiss                                    !- 040016
      ENDDO
      K = K+3

!ATMS/CrIS cloud tests

      VALUES(K+1) = Rmiss                                      !- 014204
      VALUES(K+2) = Rmiss                                      !- 014203

!VIIRS

      K = K+2
      VALUES(K+1) = 616            !VIIRS                      !- 002109
      VALUES(K+2) = 0              !Number of VIIRS channels   !- 031002
      K = K+2
!      DO J=1,0
!        VALUES(K-8+J*9) = Rmiss    !Channel                    !- 005042
!        VALUES(K-7+J*9) = Rmiss    !Band                       !- 008076
!        VALUES(K-6+J*9) = 4        !Significance - mean        !- 008023
!        VALUES(K-5+J*9) = Rmiss    !Radiance                   !- 014043
!        VALUES(K-4+J*9) = Rmiss    !Reflectance                !- 015042
!        VALUES(K-3+J*9) = 10       !Significance - SD          !- 008023
!        VALUES(K-2+J*9) = Rmiss    !Radiance                   !- 014043
!        VALUES(K-1+J*9) = Rmiss    !Reflectance                !- 015042
!        VALUES(K  +J*9) = Rmiss    !Significance - reset       !- 008023
!      ENDDO
      
      END
