!+ Read level 1c AAPP header/data records and put values in BUFR array
!
      SUBROUTINE AAPP_PUT_1D_CRIS(C1dunit, Current_scan, FOV,
     &   FOVOffset, ICENTREID, ISUBCENTRE, NOBS, KEL, VALUES)
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_put_1d_cris.F 577 2018-01-15 13:58:49Z frna $
!
! Purpose:
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0    23/09/2010  Create from aapp_put_1c_iasi.F
! 1.1    13/03/2012  Correct lat/lon checking
! 1.2    03/07/2012  Correct NEdT scaling (make it same as the hdf5 SDR)
! 1.3    20/03/2013  Correct scaling of surface height (was 1E-2 -> 1E0)
! 1.4    19/12/2017  Max channel number is 2046 in this sequence
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "inparms.h"
#include "cris1d.h"

! Arguments:
      INTEGER C1dunit      
      INTEGER Current_scan
      INTEGER ICENTREID   !Centre ID, e.g. 255=Missing
      INTEGER ISUBCENTRE  !Subcentre ID, e.g. 255=Missing
      INTEGER FOV         !Numbered from 1
      INTEGER FOVOffset   !Normally 0
      INTEGER KEL         !Array elements per ob
      INTEGER NOBS        !Number of obs in BUFR message
      REAL*8 VALUES(*)    !Data values

! Parameters:

      REAL*8    Rmiss
      INTEGER   Imiss
      PARAMETER (Rmiss=1.7E+38)     !Missing value for BUFR
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
      INTEGER    NumBitsIn          !- size of raw data bit fields
      PARAMETER  (NumBitsIn  = 32)
      INTEGER    NumBitsOut         !- set BUFR bit field width
      PARAMETER  (NumBitsOut = 24)
      
! Local variables:
      INTEGER I,J,K
      INTEGER Irec
      INTEGER BIT                   !For use with unbitswap
      INTEGER Mask025214(NumBitsIn) !- bit mask for 025214
      INTEGER Mask025220(NumBitsIn) !- bit mask for 025220
      DATA Mask025214 /1,2,3,4,5,6,7,8,9,10,18*0,11,12,13,14/
      DATA Mask025220 /6*0,25,24,23,22,21,20,19,18,17,16,15,14,13,   !- FOV qual flgs
     &                 12,11,10,9,8,7,6,5,4,3,2,1,26/
      INTEGER scnlindy,scnlinyr,FovTime  !date and time
      INTEGER Month,Day,Hour,Minute
      REAL*8  Sec
      INTEGER Ichan     !Channels loop index
      INTEGER Ipc       !PC loop index
      INTEGER IBAND     !IASI band index
      INTEGER CLASS     !AVHRR cloud class
      INTEGER Qual      !Quality flag
      INTEGER IOS
      INTEGER zenmin

! Functions
      REAL*8 FNRSCALE

!End of header
!------------------------------------------------------------------------------

      IF (FOV.EQ.1) THEN
        irec = Current_scan + cris1d_h_hdrcnt
        CALL IOC1D ("READ DATA", C1dunit, irec)     !Read next scan
        write(*,*)'Read scan',Current_scan
      ENDIF

      NOBS = NOBS + 1
      I = (NOBS-1)*KEL       !Point to data in VALUES array
          
      VALUES(i+1) = cris1d_h_wmosatid              !- 001007       
      VALUES(i+2) = 4                              !- 008070
      VALUES(i+3) = ICENTREID                      !- 001033
      VALUES(i+4) = ISUBCENTRE                     !- 001034
      VALUES(i+5) = 2                              !- 008070
      VALUES(i+6) = cris1d_h_centre                !- 001033
      VALUES(i+7) = cris1d_h_subcentre             !- 001034

!620=CrIS
      VALUES(i+8) = 620                            !- 002019

!Date and time of the FOV
      scnlindy = cris1d_scnlindy
      scnlinyr = cris1d_scnlinyr
      FovTime = cris1d_scnlintime(FOV)
      CALL DAY_TO_DATE(scnlinyr,scnlindy,FovTime,
     &           Month,Day,Hour,Minute,Sec)

      VALUES(i+9) = scnlinyr                       !- 004001
      VALUES(i+10) = Month                         !- 004002
      VALUES(i+11) = Day                           !- 004003
      VALUES(i+12) = Hour                          !- 004004
      VALUES(i+13) = Minute                        !- 004005
      VALUES(i+14) = Sec                           !- 004006

      VALUES(i+15) = FNRSCALE(cris1d_satxyz(1),1.0D0)     !- 027031
      VALUES(i+16) = FNRSCALE(cris1d_satxyz(2),1.0D0)     !- 028031
      VALUES(i+17) = FNRSCALE(cris1d_satxyz(3),1.0D0)     !- 010031

!Geolocation
!Note: check the nadir zenith angle is less than 4 degrees; if not, then a
!manoeuvre may be taking place so set geolocation to missing. The 1d file may be
!thinned, so use a larger tolerance than 1c.
      zenmin = MIN(cris1d_angles(1,15),cris1d_angles(1,16))
      IF (cris1d_latlon(1,FOV).GE.-900000 .AND.
     &    cris1d_latlon(1,FOV).LE. 900000 .AND.
     &    cris1d_latlon(2,FOV).GE.-1800000 .AND.
     &    cris1d_latlon(2,FOV).LE. 1800000 .AND.
     &    zenmin .LT. 400) THEN
        VALUES(i+18) = FNRSCALE(cris1d_latlon(1,FOV),1.0D-4) !- 005001
        VALUES(i+19) = FNRSCALE(cris1d_latlon(2,FOV),1.0D-4) !- 006001
        VALUES(i+20) = FNRSCALE(cris1d_angles(1,FOV),1.0D-2) !- 007024
        IF (cris1d_angles(2,FOV).NE.Imiss .AND.
     &      cris1d_angles(2,FOV).LT.0) THEN
          cris1d_angles(2,FOV)=cris1d_angles(2,FOV)+36000  !+ve
        ENDIF
        VALUES(i+21) = FNRSCALE(cris1d_angles(2,FOV),1.0D-2)     !- 005021
        VALUES(i+22) = FNRSCALE(cris1d_angles(3,FOV),1.0D-2)     !- 007025
        IF (cris1d_angles(4,FOV).NE.Imiss .AND.
     &      cris1d_angles(4,FOV).LT.0) THEN
          cris1d_angles(4,FOV)=cris1d_angles(4,FOV)+36000  !+ve
        ENDIF
        VALUES(i+23) = FNRSCALE(cris1d_angles(4,FOV),1.0D-2)     !- 005022
        VALUES(i+24) = FNRSCALE(cris1d_ascdesc,1.0D0)            !- 008075
      ELSE
        IF (FOV.EQ.1) WRITE(*,*)'Bad geolocation for scan',Current_scan
        VALUES(i+18) = Rmiss
        VALUES(i+19) = Rmiss
        VALUES(i+20) = Rmiss
        VALUES(i+21) = Rmiss
        VALUES(i+22) = Rmiss
        VALUES(i+23) = Rmiss
        VALUES(i+24) = Rmiss
      ENDIF

      VALUES(i+25) = FNRSCALE(cris1d_scnlin,1.0D0)             !- 005041
      VALUES(i+26) = FOV                 !Field of regard      !- 005045
      VALUES(i+27) = MOD(cris1d_FOV(FOV)-1,9) + 1  !Detector   !- 005043

      VALUES(i+28) = FNRSCALE(cris1d_orbit,1.0D0)              !- 005040

! surface height (metres)
      VALUES(i+29) = FNRSCALE(cris1d_surface(1,FOV),1.0D0)     !- 010001

! satellite altitude
      VALUES(i+30) = FNRSCALE(cris1d_scalti,1.0D2)             !- 007002

! land fraction
      VALUES(i+31) = FNRSCALE(cris1d_landfrac(FOV),1.0D-2)     !- 021166

! surface type: raw 1d data  : 0 = Sea, 1 = Mixed, 2 = Land
!             : BUFR 1d data : 0 = Land, 1 = Sea, 2 = Coast    !- 008012 

      IF (cris1d_surface(2,FOV) .EQ. 0) THEN
        VALUES(i+32) = 1
      ELSEIF (cris1d_surface(2,FOV) .EQ. 1) THEN
        VALUES(i+32) = 2
      ELSEIF (cris1d_surface(2,FOV) .EQ. 2) THEN
        VALUES(i+32) = 0
      ELSE
        VALUES(i+32) = Rmiss
      ENDIF

      VALUES(i+33) = FNRSCALE(cris1d_cloudcover(FOV),1.0D-2)   !- 020010
      VALUES(i+34) = FNRSCALE(cris1d_cloudtopht(FOV),1.0D-2)   !- 020014
      VALUES(i+35) = FNRSCALE(cris1d_geolocqual,1.0D0)         !- 033078

!ATOVPP derived quantities

      VALUES(i+36) = FNRSCALE(cris1d_h_atovppvn,1.0D0)         !- 055023
      CALL BITSWAP(cris1d_prepro(1,FOV), Mask025214, NumBitsIn, 
     &                   Bit, NumBitsOut)
      VALUES(i+37) = 1.0D0*Bit                                 !- 025214
      IF (cris1d_prepro(2,FOV) .GT. 0) THEN
        VALUES(i+38) = cris1d_prepro(2,FOV) - 1                !- 008205
      ELSE
        VALUES(i+38) = Rmiss
      ENDIF
      VALUES(i+39) = FNRSCALE(cris1d_prepro(3,FOV),1.0D-2)     !- 002224
      IF (VALUES(i+39).GT.163.82) VALUES(i+39) = Rmiss
      VALUES(i+40) = FNRSCALE(cris1d_prepro(4,FOV),1.0D-2)     !- 025206
      IF (VALUES(i+40).GT.250.0) VALUES(i+40) = Rmiss
      VALUES(i+41) = FNRSCALE(cris1d_prepro(5,FOV),1.0D-2)     !- 025206
      IF (VALUES(i+41).GT.250.0) VALUES(i+41) = Rmiss
      VALUES(i+42) = FNRSCALE(cris1d_prepro(6,FOV),1.0D-2)     !- 025207
      IF (VALUES(i+42).GT.126.0) VALUES(i+42) = Rmiss
      VALUES(i+43) = FNRSCALE(cris1d_prepro(7,FOV),1.0D-2)     !- 025206
      IF (VALUES(i+43).GT.250.0) VALUES(i+43) = Rmiss
      IF (cris1d_crisqual(FOV) .NE. Imiss) THEN
        CALL BITSWAP(cris1d_crisqual(FOV), Mask025220, NumBitsIn, 
     &                   Bit, 26)
        VALUES(i+44) = 1.0*Bit                                 !- 025220
      ELSE
        VALUES(i+44) = Rmiss
      ENDIF

!Re-mapped ATMS

      VALUES(i+45) = 621                                       !- 002019
      VALUES(i+46) = Rmiss              !Granule qual          !- 033079
      VALUES(i+47) = Rmiss              !Scan qual             !- 033080
      DO J=1,22
        VALUES(i+40+J*8) = J                                   !- 005042
        IF (cris1d_h_atmsPlanck(1,J).NE.Imiss) THEN
          VALUES(i+41+J*8) = ALOG10(cris1d_h_atmsPlanck(1,J)*1E-4) !- 025076
        ELSE
          VALUES(i+41+J*8) = Rmiss
        ENDIF
        VALUES(i+42+J*8) = FNRSCALE(cris1d_h_atmsPlanck(2,J),1D-6) !- 025077
        VALUES(i+43+J*8) = FNRSCALE(cris1d_h_atmsPlanck(3,J),1D-6) !- 025077
        VALUES(i+44+J*8) = FNRSCALE(cris1d_btemps(J,FOV),1D-2)     !- 012163
        VALUES(i+45+J*8) = FNRSCALE(cris1d_nedtcoldatms(J),1D-4)   !- 012158
        VALUES(i+46+J*8) = FNRSCALE(cris1d_nedtwarmatms(J),1D-4)   !- 012159
        VALUES(i+47+J*8) = Rmiss        !Chan qual                 !- 033081
      ENDDO
      VALUES(i+224) = FNRSCALE(cris1d_h_atmsbeamversion,1D0)   !- 055007

!CrIS radiance data

      VALUES(i+225) = 620                                      !- 002019
      VALUES(i+226) = FNRSCALE(cris1d_radianceflgs(FOV),1.0D0) !- 002165 
      VALUES(i+227) = FNRSCALE(cris1d_scanqualflgs,1.0D0)      !- 033075
      DO J=1,3
        VALUES(i+221+J*7) = J+1                                       !- 008076 
        VALUES(i+222+J*7) = FNRSCALE(cris1d_h_startw(J),1.0D0)        !- 006029
        VALUES(i+223+J*7) = FNRSCALE(cris1d_h_endw(J),1.0D0)          !- 006029
        VALUES(i+224+J*7) = FNRSCALE(cris1d_h_startchan(J),1.0D0)     !- 025140
        VALUES(i+225+J*7) = FNRSCALE(cris1d_h_endchan(J),1.0D0)       !- 025141
        VALUES(i+226+J*7) = FNRSCALE(cris1d_calqualflgs(J,FOV),1.0D0) !- 033076
        VALUES(i+227+J*7) = FNRSCALE(cris1d_fovqualflgs(J,FOV),1.0D0) !- 033076
      ENDDO
      VALUES(i+249) = FNRSCALE(cris1d_quality(FOV),1.0D0)      !- 033003
      VALUES(i+250) = FNRSCALE(cris1d_h_chanvn,1.0D0)          !- 055007
      VALUES(i+251) = cris1d_h_nchanused                       !- 031002
      DO J=1,cris1d_h_nchanused
        K = i+250+2*J
         !Only 11 bits for channel number: 201133,005042
	IF (cris1d_h_channels(J).le. 2046) THEN
          VALUES(K) = cris1d_h_channels(J)                       !- 005042
        ELSE
          VALUES(K) = cris1d_h_channels(J)-2047
        ENDIF
        VALUES(K+1) = FNRSCALE(cris1d_radiance(FOV,J),1.0D-7)  !- 014044
      ENDDO
      K = K+1
      VALUES(K+1) = FNRSCALE(cris1d_h_eigvn,1.0D0)             !- 055007
      VALUES(K+2) = 0.5                                        !- 040026
      VALUES(K+3) = cris1d_h_npcused                           !- 031002
      DO J=1,cris1d_h_npcused
        VALUES(K+3+J) = FNRSCALE(cris1d_scores(J,FOV),1.0D0)   !- 025050
      ENDDO
      K = K+3+cris1d_h_npcused
      DO J=1,3
        IF (cris1d_logfit(J,FOV) .NE. Imiss) THEN
          VALUES(K+J) = 10.0**(cris1d_logfit(J,FOV)*1E-4)      !- 040016
        ELSE
          VALUES(K+J) = Rmiss
        ENDIF
      ENDDO
      K = K+3

!ATMS/CrIS cloud tests

      VALUES(K+1) = FNRSCALE(cris1d_h_cloudtests,1.0D0)        !- 014204
      VALUES(K+2) = FNRSCALE(cris1d_cloudflag(10,FOV),1.0D0)   !- 014203

!VIIRS

      K = K+2
      VALUES(K+1) = 616            !VIIRS                      !- 002109
      VALUES(K+2) = cris1d_h_nviirschanused                    !- 031002
      K = K+2
      DO J=1,cris1d_h_nviirschanused
        VALUES(K-8+J*9) = cris1d_h_viirschans(J)               !- 005042
        VALUES(K-7+J*9) = cris1d_h_viirsbands(J)               !- 008076
        VALUES(K-6+J*9) = 4        !Significance - mean        !- 008023
        VALUES(K-5+J*9) = Rmiss    !Radiance                   !- 014043
        VALUES(K-4+J*9) = Rmiss    !Reflectance                !- 015042
        VALUES(K-3+J*9) = 10       !Significance - SD          !- 008023
        VALUES(K-2+J*9) = Rmiss    !Radiance                   !- 014043
        VALUES(K-1+J*9) = Rmiss    !Reflectance                !- 015042
        VALUES(K  +J*9) = Rmiss    !Significance - reset       !- 008023
      ENDDO
      
      END
