!+ Read level 1d AAPP header/data records and put values in BUFR array
!
      SUBROUTINE AAPP_PUT_1D_IASI(I1Dunit, Current_scan, FOV,
     &   FOVOffset, ICENTREID, NOBS, KEL, VALUES)
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_put_1d_iasi.F 400 2014-06-26 08:49:16Z frna $
!
! Purpose:
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2011, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0    13/06/2011  NCA. Format is that of IASIG/IASIL at Met Office.
! 1.1    30/05/2012  Change second value of 002151 from 4 (A1-2) to 5 (A2) 
! 1.2    26/06/2014  Round FovTime to nearest second before encoding
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "inparms.h"
#include "iasi1d.h"

! Arguments:
      INTEGER I1Dunit      
      INTEGER Current_scan
      INTEGER ICENTREID   !Centre ID, e.g. 255=Missing
      INTEGER FOV         !Numbered from 1
      INTEGER FOVOffset   !Normally 0
      INTEGER KEL         !Array elements per ob
      INTEGER NOBS        !Number of obs in BUFR message
      REAL*8 VALUES(*)    !Data values

! Parameters:

      REAL*8    Rmiss
      INTEGER   Imiss
      PARAMETER (Rmiss=1.7E+38)     !Missing value for BUFR
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
      INTEGER    NumBitsIn          !- size of raw data bit fields
      PARAMETER  (NumBitsIn  = 32)
      INTEGER    NumBitsOut         !- set BUFR bit field width
      PARAMETER  (NumBitsOut = 24)

! Local variables:
      INTEGER I,J,K,N2,INDEX
      INTEGER Irec
      INTEGER BIT                   !For use with unbitswap
      INTEGER Mask002223(NumBitsIn) !- bit mask for 002223
      INTEGER Mask025214(NumBitsIn) !- bit mask for 025214
      INTEGER Mask025220(NumBitsIn) !- bit mask for 025220
      DATA Mask002223 /25*0,12,11,10,9,0,2,1/      !- instruments - 12=IASI
      DATA Mask025214 /1,2,3,4,5,6,7,8,9,10,18*0,11,12,13,14/
      DATA Mask025220 /6*0,25,24,23,22,21,20,19,18,17,16,15,14,13,   !- FOV qual flgs
     &                 12,11,10,9,8,7,6,5,4,3,2,1,26/

      INTEGER scnlindy,scnlinyr,FovTime  !date and time
      INTEGER Month,Day,Hour,Min
      REAL*8  Sec

! Functions
      REAL*8 FNRSCALE

!End of header
!------------------------------------------------------------------------------

      IF (FOV.EQ.1) THEN
        irec = Current_scan + iasi1d_h_hdrcnt
        CALL IOI1D ("READ DATA", I1Dunit, irec)
        write(*,*)'Read scan',Current_scan
      ENDIF

!Point to data in VALUES array
      NOBS = NOBS + 1
      I = (NOBS-1)*KEL

      VALUES(i+1) = iasi1d_h_wmosatid              !- 001007       

!Date and time of the FOV
      scnlindy = iasi1d_scnlindy
      scnlinyr = iasi1d_scnlinyr
      FovTime = iasi1d_scnlintime(FOV)
      FovTime = ((FovTime + 500)/1000)*1000    !Round to nearest second
      CALL DAY_TO_DATE(scnlinyr,scnlindy,FovTime,
     &    Month,Day,Hour,Min,Sec)

      VALUES(i+2) = scnlinyr                              !- 004001
      VALUES(i+3) = Month                                 !- 004002
      VALUES(i+4) = Day                                   !- 004003
      VALUES(i+5) = Hour                                  !- 004004
      VALUES(i+6) = Min                                   !- 004005
      VALUES(i+7) = Sec                                   !- 004006

      VALUES(i+8) = FNRSCALE(iasi1d_latlon(1,FOV),1.0D-4)  !- 005001
      VALUES(i+9) = FNRSCALE(iasi1d_latlon(2,FOV),1.0D-4)  !- 006001

! surface type: raw 1d data  : 0 = Sea, 1 = Mixed, 2 = Land
!             : BUFR 1d data : 0 = Land, 1 = Sea, 2 = Coast    !- 008012 

      IF (iasi1d_surface(2,FOV) .EQ. 0) THEN
        VALUES(i+10) = 1
      ELSEIF (iasi1d_surface(2,FOV) .EQ. 1) THEN
        VALUES(i+10) = 2
      ELSEIF (iasi1d_surface(2,FOV) .EQ. 2) THEN
        VALUES(i+10) = 0
      ELSE
        VALUES(i+10) = Rmiss
      ENDIF

!1d dataset creation site
      VALUES(i+11)  = 4                                     !- 008070
      VALUES(i+12)  = ICENTREID                             !- 001031

!1b dataset creation site
      VALUES(i+13)  = 2                                     !- 008070
      VALUES(i+14)  = ICENTREID                             !- 001031

!orbit
      VALUES(i+15) = FNRSCALE(iasi1d_h_startorbit,1.0D0)     !- 005040

!atovpp version number
      VALUES(i+16) = FNRSCALE(iasi1d_h_atovppvn,1.0D0)       !- 055023

!instruments
      IF (iasi1d_h_instruments .NE. Imiss) THEN
        CALL BITSWAP(iasi1d_h_instruments, Mask002223, NumBitsIn, 
     &       Bit, 12)
        VALUES(i+17) = 1.0D0*Bit                            !- 002223
      ELSE
        VALUES(i+17) = Rmiss
      ENDIF

!scan line and FOV
      VALUES(i+18) = FNRSCALE(iasi1d_scnlin,1.0D0)           !- 005041
      VALUES(I+19) = FNRSCALE(iasi1d_FOV(FOV),1.0D0)         !- 005043

!preprocessing flags
      CALL BITSWAP(iasi1d_prepro(1,FOV), Mask025214, NumBitsIn, 
     &                   Bit, NumBitsOut)
      VALUES(i+20) = 1.0D0*Bit                               !- 025214
      IF (iasi1d_prepro(2,FOV) .GT. 0) THEN
        VALUES(i+21) = iasi1d_prepro(2,FOV) - 1              !- 008205
      ELSE
        VALUES(i+21) = Rmiss
      ENDIF
      VALUES(i+22) = FNRSCALE(iasi1d_prepro(3,FOV),1.0D-2)   !- 002224
      IF (VALUES(i+22).GT.163.82) VALUES(i+22) = Rmiss
      VALUES(i+23) = FNRSCALE(iasi1d_prepro(4,FOV),1.0D-2)   !- 025206
      IF (VALUES(i+23).GT.250.0) VALUES(i+23) = Rmiss
      VALUES(i+24) = FNRSCALE(iasi1d_prepro(5,FOV),1.0D-2)   !- 025207
      IF (VALUES(i+24).GT.126.0) VALUES(i+24) = Rmiss

!sensors and version numbers
      VALUES(i+25) = 3               !AMSU-A                 !- 002048      
      VALUES(i+26) = FNRSCALE(iasi1d_h_Acorrvn,1.0D0)        !- 055024
      VALUES(i+27) = FNRSCALE(iasi1d_h_Apcpvn,1.0D0)         !- 055025
      VALUES(i+28) = 4               !AMSU-B/MHS             !- 002048      
      VALUES(i+29) = FNRSCALE(iasi1d_h_Bcorrvn,1.0D0)        !- 055024
      VALUES(i+30) = FNRSCALE(iasi1d_h_Bpcpvn,1.0D0)         !- 055026

!navigation
      VALUES(i+31) = FNRSCALE(iasi1d_scalti,1.0D2)           !- 007001
      VALUES(i+32) = FNRSCALE(iasi1d_surface(1,FOV),1.0D0)   !- 010001
      VALUES(i+33) = FNRSCALE(iasi1d_angles(1,FOV),1.0D-2)   !- 007024
      IF (iasi1d_angles(2,FOV).NE.Imiss .AND.
     &    iasi1d_angles(2,FOV).LT.0) THEN
        iasi1d_angles(2,FOV)=iasi1d_angles(2,FOV)+36000  !+ve
      ENDIF
      VALUES(i+34) = FNRSCALE(iasi1d_angles(2,FOV),1.0D-2)     !- 005021
      VALUES(i+35) = FNRSCALE(iasi1d_angles(3,FOV),1.0D-2)     !- 007025
      IF (iasi1d_angles(4,FOV).NE.Imiss .AND.
     &    iasi1d_angles(4,FOV).LT.0) THEN
        iasi1d_angles(4,FOV)=iasi1d_angles(4,FOV)+36000  !+ve
      ENDIF
      VALUES(i+36) = FNRSCALE(iasi1d_angles(4,FOV),1.0D-2)     !- 005022

!FOV quality
      IF (iasi1d_iasiqual(FOV) .NE. Imiss) THEN
        CALL BITSWAP(iasi1d_iasiqual(FOV), Mask025220, NumBitsIn, 
     &                   Bit, 26)
        VALUES(i+37) = 1.0*Bit                                 !- 025220
      ELSE
        VALUES(i+37) = Rmiss
      ENDIF

!Instrument temperatures
      VALUES(i+38) = 3                                         !- 002151
      VALUES(i+39) = FNRSCALE(iasi1d_instrtempa1,1.0D-2)       !- 012064
      VALUES(i+40) = 5                                         !- 002151
      VALUES(i+41) = FNRSCALE(iasi1d_instrtempa2,1.0D-2)       !- 012064
      VALUES(i+42) = 6                                         !- 002151
      VALUES(i+43) = FNRSCALE(iasi1d_instrtempb,1.0D-2)        !- 012064

!-------------------------------
!Loop over AMSU channels 1 -> 20
!-------------------------------
      DO j = 1,Anchan+Bnchan

!TOVS/ATOVS/AVHRR instrumentation channel number (28 -> 47)
        VALUES(i+44+(j-1)*5) = j + 27                          !- 002150

!log_10 of (Temperature-radiance central wavenumber (per metre)) for ATOVS
        IF (iasi1d_h_amsPlanck(1,j) .NE. Imiss) THEN
           VALUES(i+45+(j-1)*5) = 
     &        log10(0.0001 * iasi1d_h_amsPlanck(1,j))          !- 025076
        ELSE
          VALUES(i+45+(j-1)*5) = Rmiss
        ENDIF

!Bandwidth correction coefficients 1 & 2
        VALUES(i+46+(j-1)*5) = 
     &     FNRSCALE(iasi1d_h_amsPlanck(2,j),1.0D-6)            !- 025077
        VALUES(i+47+(j-1)*5) = 
     &     FNRSCALE(iasi1d_h_amsPlanck(3,j),1.0D-6)            !- 025078

!Brightness temperatures
        VALUES(i+48+(j-1)*5) = FNRSCALE(iasi1d_btemps(j,FOV),1.0D-2) !-012163

      ENDDO  !- j

! emissivity (only used at Met Office - set to missing)
      DO j=1,13
        VALUES(i+143+j) = Rmiss
      ENDDO

!-------------------------------
!IASI data
!-------------------------------

      VALUES(i+157) = 221                                        !- 002019
      VALUES(i+158) = FNRSCALE(iasi1d_GQisFlagQual(FOV),1.0D0)   !- 033060
      VALUES(i+159) = FNRSCALE(iasi1d_GQisQualIndex,1.0D0)       !- 033061
      VALUES(i+160) = FNRSCALE(iasi1d_GQisQualIndexLoc,1.0D0)    !- 033062
      VALUES(i+161) = FNRSCALE(iasi1d_GQisQualIndexRad,1.0D0)    !- 033063
      VALUES(i+162) = FNRSCALE(iasi1d_GQisQualIndexSpect,1.0D0)  !- 033064
      VALUES(i+163) = FNRSCALE(iasi1d_GQisSysTecSondQual,1.0D0)  !- 033065

      DO J=1,10
        IF (iasi1d_h_startchan(J) .NE. Imiss) THEN
          VALUES(i+164+3*(J-1)) = iasi1d_h_startchan(J)          !- 025140
        ELSE
          VALUES(i+164+3*(J-1)) = Rmiss
        ENDIF
        IF (iasi1d_h_endchan(J) .NE. Imiss) THEN
          VALUES(i+165+3*(J-1)) = iasi1d_h_endchan(J)            !- 025141
        ELSE
          VALUES(i+165+3*(J-1)) = Rmiss
        ENDIF
        IF (iasi1d_h_scalefactor(J) .NE. Imiss) THEN
          VALUES(i+166+3*(J-1)) = iasi1d_h_scalefactor(J)        !- 025142
        ELSE
          VALUES(i+166+3*(J-1)) = Rmiss
        ENDIF
      ENDDO

      VALUES(i+194) = iasi1d_h_chanvn                            !- 055007
      VALUES(i+195) = iasi1d_h_nchanused                         !- 031002

      DO J=1,iasi1d_h_nchanused
	VALUES(i+194+2*J) = iasi1d_h_channels(J)                 !- 005042
        IF (iasi1d_scalrad(J,FOV).EQ.-32768) THEN
          VALUES(i+195+2*J) = Rmiss
        ELSE IF (iasi1d_scalrad(J,FOV).GE.-5000) THEN
          VALUES(i+195+2*J) = iasi1d_scalrad(J,FOV)              !- 014046
        ELSE
          VALUES(i+195+2*J) = iasi1d_scalrad(J,FOV) + 65536.0
        ENDIF
      ENDDO

      N2 = 2*iasi1d_h_nchanused
      VALUES(i+196+N2) = iasi1d_h_eigvn                          !- 055007
      VALUES(i+197+N2) = iasi1d_h_npcused                        !- 031002

      DO J=1,iasi1d_h_npcused
        VALUES(i+197+N2+J) = FNRSCALE(iasi1d_scores(J,FOV),1.0D0) !- 025050
      ENDDO

! Assign a new reference value for the log10(PC score), to allow scores less
! than 1.0, i.e. negative log. Scaling is 10^4 in AAPP and in BUFR.
!
! Note that with the ECMWF library you have to subtract the new offset from the
! data value; with Met Office library you don't.

      INDEX = 198+N2+iasi1d_h_npcused
      VALUES(i+INDEX) = -16383             !New reference value for 025052
      DO J=1,3
        IF (iasi1d_logfit(J,FOV).GT.Imiss) THEN
          VALUES(i+INDEX+J) = 0.0001*(iasi1d_logfit(J,FOV)+16383) !- 025052
        ELSE
          VALUES(i+INDEX+J) = Rmiss
        ENDIF
      ENDDO

!-----------------------------------------------------------------------
! AVHRR data
!-----------------------------------------------------------------------

      INDEX = INDEX+4

      VALUES(i+INDEX) = 591                      !591=AVHRR/3    !- 002019
      VALUES(i+INDEX+1) = iasi1d_avhcombination                  !- 025051

      INDEX = INDEX+2

      DO J=1,7   !Loop over classes
        VALUES(i+INDEX) = FNRSCALE(iasi1d_avhrrCGy(J,FOV),1.0D-6)   !- 005060
        VALUES(i+INDEX+1) = FNRSCALE(iasi1d_avhrrCGz(J,FOV),1.0D-6) !- 005061
            
!Convert from 10^6 * fraction to percentage
        VALUES(i+INDEX+2) = FNRSCALE(iasi1d_avhrrfrac(J,FOV),1.0D-4) !- 025085
        INDEX = INDEX + 2
        DO K=1,6  !Loop over channels
          VALUES(i+INDEX+1) = K                                      !- 005042
          IF (iasi1d_avhmeanscalefactor(K,J,FOV).GT.0) THEN
            VALUES(i+INDEX+2) = iasi1d_avhmeanscalefactor(K,J,FOV)   !- 025142
          ELSE
            VALUES(i+INDEX+2) = Rmiss
          ENDIF
          VALUES(i+INDEX+3) = FNRSCALE(iasi1d_avhrrmean(K,J,FOV),1.0D0)!- 014047
          IF (iasi1d_avhsdevscalefactor(K,J,FOV).GT.0) THEN
            VALUES(i+INDEX+4) = iasi1d_avhsdevscalefactor(K,J,FOV)     !- 025142
          ELSE
            VALUES(i+INDEX+4) = Rmiss
          ENDIF
          VALUES(i+INDEX+5) = FNRSCALE(iasi1d_avhrrsdev(K,J,FOV),1.0D0) !-014048
          INDEX = INDEX + 5
        ENDDO
        INDEX = INDEX+1
      ENDDO

!-----------------------------------------------------------------------
! Cloud tests
! cloudflag word 10 is a flag table containing results of different tests
! Words 3 and 4 are the EUMETSAT cloud tests, 8 bits each, so includes these.
!-----------------------------------------------------------------------
!factor 64 because of MetDB problems in extracting 30-bit flag tables

      IF (iasi1d_cloudflag(3,FOV).EQ.Imiss)iasi1d_cloudflag(3,FOV)=255
      IF (iasi1d_cloudflag(4,FOV).EQ.Imiss)iasi1d_cloudflag(4,FOV)=255
          
      iasi1d_cloudflag(10,FOV) = iasi1d_cloudflag(10,FOV) + 
     &      IAND(iasi1d_cloudflag(3,FOV),255)*256 +
     &      IAND(iasi1d_cloudflag(4,FOV),255)*65536
   
      VALUES(i+INDEX) = iasi1d_h_cloudtests*64                  !- 014204
      VALUES(i+INDEX+1) = iasi1d_cloudflag(10,FOV)*64           !- 014203

      END
