!+ Read level 1d AAPP header/data records and put values in BUFR array
!
      SUBROUTINE AAPP_PUT_1D_MWRI(M1cunit, Current_scan, FOV,
     &   FOVOffset, ICENTREID, ISUBCENTRE, TimeDelay, NOBS, KEL, VALUES,
     &   ITHIN)
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_put_1d_mwri.F 586 2018-04-30 10:32:45Z frna $
!
! Purpose:
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2016, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0    07/04/2016  Create from aapp_put_1d_mwhs2.F
! 1.1    27/04/2018  Add inland water category
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "inparms.h"
#include "mwri1c.h"
#include "constants_preproc.h"

!Note that the mwhs1c.h format is used for both 1c and 1d

! Arguments:
      INTEGER M1cunit      
      INTEGER Current_scan
      INTEGER ICENTREID   !Centre ID, e.g. 255=Missing
      INTEGER ISUBCENTRE  !Subcentre ID, e.g. 255=Missing
      INTEGER FOV         !Numbered from 1
      INTEGER FOVOffset   !Normally 0
      INTEGER TimeDelay   !Milliseconds per FOV
      INTEGER KEL         !Array elements per ob
      INTEGER NOBS        !Number of obs in BUFR message
      REAL*8 VALUES(*)    !Data values
      INTEGER ITHIN       !Number of samples/scans to increment

! Parameters:

      REAL*8    Rmiss
      INTEGER   Imiss
      PARAMETER (Rmiss=1.7E+38)     !Missing value for BUFR
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
      INTEGER    NumBitsIn          !- size of raw data bit fields
      PARAMETER  (NumBitsIn  = 32)
      INTEGER    NumBitsOut         !- set BUFR bit field width
      PARAMETER  (NumBitsOut = 24)
      
! Local variables:
      INTEGER I,J,K
      INTEGER Irec
      INTEGER BIT                   !For use with unbitswap
      INTEGER Mask025214(NumBitsIn) !- bit mask for 025214
      INTEGER Mask025220(NumBitsIn) !- bit mask for 025220
      DATA Mask025214 /1,2,3,4,5,6,7,8,9,10,18*0,11,12,13,14/
      DATA Mask025220 /6*0,25,24,23,22,21,20,19,18,17,16,15,14,13,   !- FOV qual flgs
     &                 12,11,10,9,8,7,6,5,4,3,2,1,26/
      INTEGER scnlindy,scnlinyr,FovTime  !date and time
      INTEGER Month,Day,Hour,Min
      REAL*8  Sec
      INTEGER IOS
      INTEGER midlat_old
      DATA midlat_old/MDI/

! Functions
      REAL*8 FNRSCALE

!End of header
!------------------------------------------------------------------------------

      IF (FOV.EQ.1) THEN
        irec = Current_scan + mwri1c_h_hdrcnt
        CALL IOMWRI1C ("READ DATA", M1cunit, irec)
          write(*,*)'Read scan',Current_scan
        ENDIF

        IF (MOD(FOV-ITHIN/2-1,ITHIN) .NE. 0) THEN
          RETURN                            !This FOV is thinned out
        ENDIF

!Point to data in VALUES array
        NOBS = NOBS + 1
        I = (NOBS-1)*KEL

        VALUES(i+1)  = ICENTREID                             !- 001033
        VALUES(i+2)  = ISUBCENTRE                            !- 001034

        VALUES(i+3)  = mwri1c_h_wmosatid                     !- 001007
        VALUES(i+4)  = 938                     !MWRI         !- 002019
        VALUES(i+5)  = FNRSCALE(mwri1c_h_startorbit,1.0D0)   !- 005040
        VALUES(i+6)  = FNRSCALE(mwri1c_scnlin,1.0D0)         !- 005041
        VALUES(I+7)  = FOV + FOVOffset                       !- 005043
          
!Date and time of the FOV
        scnlindy = mwri1c_scnlindy
        scnlinyr = mwri1c_scnlinyr
        FovTime = mwri1c_scnlintime+((FOV-1)*TimeDelay)
        FovTime = ((FovTime + 500)/1000)*1000    !Round to nearest second
        CALL DAY_TO_DATE(scnlinyr,scnlindy,FovTime,
     &       Month,Day,Hour,Min,Sec)

        VALUES(i+8)  = scnlinyr                              !- 004001
        VALUES(i+9)  = Month                                 !- 004002
        VALUES(i+10) = Day                                   !- 004003
        VALUES(i+11) = Hour                                  !- 004004
        VALUES(i+12) = Min                                   !- 004005
        VALUES(i+13) = Sec                                   !- 004006

        VALUES(i+14) = FNRSCALE(mwri1c_latlon(1,FOV),1.0D-4)  !- 005001
        VALUES(i+15) = FNRSCALE(mwri1c_latlon(2,FOV),1.0D-4)  !- 006001

!Surface height
        VALUES(i+16) = FNRSCALE(mwri1c_surface(1,FOV),1.0D0)  !- 007002 

!Surface type AAPP 0=sea, 1=mixed, 2=land, 3=inland water; 013040 0=land, 5=ocean, 6=coast, 7=inland water
        IF (mwri1c_surface(2,FOV).EQ.0) THEN
          VALUES(i+17) = 5
        ELSE IF (mwri1c_surface(2,FOV).EQ.1) THEN
          VALUES(i+17) = 6
        ELSE IF (mwri1c_surface(2,FOV).EQ.2) THEN
          VALUES(i+17) = 0
        ELSE IF (mwri1c_surface(2,FOV).EQ.3) THEN
          VALUES(i+17) = 7
        ELSE
          VALUES(i+17) = Rmiss
        ENDIF

        VALUES(i+18) = FNRSCALE(mwri1c_angles(1,FOV),1.0D-2)  !- 007024
        IF (mwri1c_angles(2,FOV).NE.Imiss .AND.
     &      mwri1c_angles(2,FOV).LT.0) THEN
          mwri1c_angles(2,FOV) = mwri1c_angles(2,FOV) + 36000  !make +ve
        ENDIF
        VALUES(i+19) = FNRSCALE(mwri1c_angles(2,FOV),1.0D-2)  !- 005021
        VALUES(i+20) = FNRSCALE(mwri1c_angles(3,FOV),1.0D-2)  !- 007025
        IF (mwri1c_angles(4,FOV).NE.Imiss .AND.
     &      mwri1c_angles(4,FOV).LT.0) THEN
          mwri1c_angles(4,FOV) = mwri1c_angles(4,FOV) + 36000  !make +ve
        ENDIF
        VALUES(i+21) = FNRSCALE(mwri1c_angles(4,FOV),1.0D-2)  !- 005022

!Satellite position and velocity, and angle in orbit

        VALUES(i+22) = FNRSCALE(mwri1c_satposition(1),1.0D0)  !- 027031
        VALUES(i+23) = FNRSCALE(mwri1c_satposition(2),1.0D0)  !- 028031
        VALUES(i+24) = FNRSCALE(mwri1c_satposition(3),1.0D0)  !- 010031
        VALUES(i+25) = FNRSCALE(mwri1c_satvelocity(1),1.0D0)  !- 001041
        VALUES(i+26) = FNRSCALE(mwri1c_satvelocity(2),1.0D0)  !- 001042
        VALUES(i+27) = FNRSCALE(mwri1c_satvelocity(3),1.0D0)  !- 001043
        VALUES(i+28) = FNRSCALE(mwri1c_orbitangle,1.0D-2)     !- 025084
        IF (VALUES(i+28).LT.0) VALUES(i+28) = VALUES(i+28)+360.0D0

!Geolocation quality flag

        VALUES(i+29) = Rmiss  !- 033082
        k = i+30

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Instrument MWRI

        VALUES(k) = 938                                       !- 002019 
        k=k+1

!Instrument temp

        VALUES(k) = FNRSCALE(mwri1c_instrtemp1,1.0D-2)        !- 012064
        k=k+1

!Radiance quality flag

        VALUES(k) = Rmiss                                     !- 033083
        k=k+1

!Loop over channels

        VALUES(k) = mwri_nchan                                !- 031001
        k=k+1

        DO j = 1,mwri_nchan

!Channel number

          VALUES(k) = j                                       !- 005042
          k=k+1

!Brightness temperatures

          IF (mwri1c_btemps(j,FOV) .ge. 32768) THEN
            VALUES(k) = Rmiss
          ELSE
            VALUES(k) = FNRSCALE(mwri1c_btemps(j,FOV),1.0D-2) !- 012063
          ENDIF
          k=k+1

        ENDDO  !- j

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Instrument (not used)

        VALUES(k) = Rmiss                                     !- 002019
        k=k+1

!Instrument temp

        VALUES(k) = Rmiss                                     !- 012064
        k=k+1

!Radiance quality flag

        VALUES(k) = Rmiss                                     !- 033083
        k=k+1

!Loop over channels (none)

        VALUES(k) = 0                                         !- 031001
        k=k+1

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Instrument (not used)

        VALUES(k) = Rmiss                                     !- 002019
        k=k+1

!Instrument temp

        VALUES(k) = Rmiss                                     !- 012064
        k=k+1

!Radiance quality flag

        VALUES(k) = Rmiss                                     !- 033083
        k=k+1

!Loop over channels (none)

        VALUES(k) = 0                                         !- 031001
        k=k+1

      END
