#include <stdlib.h>

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include <stdio.h>
#include <string.h>

#include "add_hdf5_attributes.h"

/******************************************************************************/
   
herr_t add_string_attribute(hid_t dataset_id, char *name, char *description){
  hid_t       dataspace_id;
  hid_t       attribute_id;
  hid_t       strtype;       /* type ID string */
  herr_t      status;
  hsize_t     dim;
 
  strtype = H5Tcopy (H5T_C_S1);
  dim = 1;
  dataspace_id = H5Screate_simple(1, &dim, NULL);
  if (strlen(description) == 0) description = "None";
  status = H5Tset_size (strtype, strlen(description));
  attribute_id = H5Acreate(dataset_id, name, strtype, dataspace_id, H5P_DEFAULT, H5P_DEFAULT);
  status = H5Awrite(attribute_id, strtype, description);
  status = H5Aclose(attribute_id);
  H5Sclose(dataspace_id);
}  

/******************************************************************************/
   
herr_t add_int_attribute(hid_t dataset_id, char *name, int *values, hsize_t *dims){
  hid_t       dataspace_id;
  hid_t       attribute_id;
  herr_t      status;
 
  dataspace_id = H5Screate_simple(1, dims, NULL);
  attribute_id = H5Acreate(dataset_id, name, H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, H5P_DEFAULT);
  status = H5Awrite(attribute_id, H5T_NATIVE_INT, values);
  H5Aclose(attribute_id);
  H5Sclose(dataspace_id);
}  

/******************************************************************************/
   
herr_t add_short_attribute(hid_t dataset_id, char *name, short *values, hsize_t *dims){
  hid_t       dataspace_id;
  hid_t       attribute_id;
  herr_t      status;
 
  dataspace_id = H5Screate_simple(1, dims, NULL);
  attribute_id = H5Acreate(dataset_id, name, H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, H5P_DEFAULT);
  status = H5Awrite(attribute_id, H5T_NATIVE_SHORT, values);
  H5Aclose(attribute_id);
  H5Sclose(dataspace_id);
}  

/******************************************************************************/
   
herr_t add_ushort_attribute(hid_t dataset_id, char *name, unsigned short *values, hsize_t *dims){
  hid_t       dataspace_id;
  hid_t       attribute_id;
  herr_t      status;
 
  dataspace_id = H5Screate_simple(1, dims, NULL);
  attribute_id = H5Acreate(dataset_id, name, H5T_STD_U16LE, dataspace_id, H5P_DEFAULT, H5P_DEFAULT);
  status = H5Awrite(attribute_id, H5T_NATIVE_USHORT, values);
  H5Aclose(attribute_id);
  H5Sclose(dataspace_id);
}  

/******************************************************************************/
   
herr_t add_float_attribute(hid_t dataset_id, char *name, float *values, hsize_t *dims){
  hid_t       dataspace_id;
  hid_t       attribute_id;
  herr_t      status;
 
  dataspace_id = H5Screate_simple(1, dims, NULL);
  attribute_id = H5Acreate(dataset_id, name, H5T_IEEE_F32LE, dataspace_id, H5P_DEFAULT, H5P_DEFAULT);
  status = H5Awrite(attribute_id, H5T_NATIVE_FLOAT, values);
  H5Aclose(attribute_id);
  H5Sclose(dataspace_id);
}  

#else

int dummy_routine(){
}

#endif
