! Name: $Id: ama1d_read_f.F 418 2014-09-29 15:22:46Z frna $
!
! Purpose: Read AMSUB/MHS l1d file and pass information back to C calling routine.
!          Subroutines ama1d_read_HEADER and ama1d_read_DATA
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    26/09/2014  N C Atkinson
!-----------------------------------------------------------------------

      SUBROUTINE AMA1D_READ_HEADER (
     & filename, filename_length,
     & h_site, h_1bsite, h_versnb,
     & h_satid, h_grid, h_satht, h_period,
     & h_startorbit, h_startdatayr, h_startdatady, h_startdatatime,
     & h_endorbit, h_enddatayr, h_enddatady, h_enddatatime,
     & h_scnlin, h_misscnlin, h_atovppvn, h_instruments,
     & h_Aantenna, h_wmosatid, h_amsPlanck, h_dT, h_mapBTmode,
     & h_Acorrvn, h_Apcpvn, h_Bcorrvn, h_Bpcpvn, h_Bantenna,
     & h_Ascat, h_ACrosby, h_AB89)

      IMPLICIT NONE
      
! Parameters
      INTEGER   UNIT_1D
      PARAMETER (UNIT_1D=10)
      INTEGER*4 AMSU_CHANS
      PARAMETER (AMSU_CHANS=20)

! Arguments: 
      CHARACTER*(*) filename
      INTEGER       filename_length
      CHARACTER*3   h_site
      CHARACTER*3   h_1bsite
      INTEGER*4     h_versnb
      INTEGER*4     h_satid
      INTEGER*4     h_grid
      INTEGER*4     h_satht
      INTEGER*4     h_period
      INTEGER*4     h_startorbit
      INTEGER*4     h_startdatayr
      INTEGER*4     h_startdatady
      INTEGER*4     h_startdatatime
      INTEGER*4     h_endorbit
      INTEGER*4     h_enddatayr
      INTEGER*4     h_enddatady
      INTEGER*4     h_enddatatime
      INTEGER*4     h_scnlin
      INTEGER*4     h_misscnlin
      INTEGER*4     h_atovppvn
      INTEGER*4     h_instruments
      INTEGER*4     h_Aantenna
      INTEGER*4     h_wmosatid
      INTEGER*4     h_amsPlanck(3,AMSU_CHANS)
      INTEGER*4     h_dT
      INTEGER*4     h_mapBTmode
      INTEGER*4     h_Acorrvn
      INTEGER*4     h_Apcpvn 
      INTEGER*4     h_Bcorrvn
      INTEGER*4     h_Bpcpvn
      INTEGER*4     h_Bantenna
      INTEGER*4     h_Ascat
      INTEGER*4     h_ACrosby
      INTEGER*4     h_AB89

! Local:
      INTEGER errStatus, i, ichan

#include "inparms.h"
#include "ama1d.h"
#include "reclopt.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Open AMSU-A 1d file ',filename(1:filename_length)
      OPEN ( UNIT=UNIT_1D, FILE=filename(1:filename_length), 
     &   STATUS="OLD", FORM="UNFORMATTED", ACCESS="DIRECT",
     &   RECL=ama_1drecl/rldiv, IOSTAT=errStatus )
      IF (errStatus.NE.0) THEN
        write(*,*)'Cannot open file ',filename(1:filename_length)
        call aappexit(1)
      ENDIF

      write(*,*)'Read AMSU-A header'
      CALL IOA1D("READ HEADER",UNIT_1D,1)

      h_scnlin = ama1d_h_scnlin
      h_site = ama1d_h_site
      h_1bsite = ama1d_h_1bsite
      h_versnb = ama1d_h_versnb
      h_satid = ama1d_h_satid
      h_grid = ama1d_h_grid
      h_satht = ama1d_h_satht
      h_period = ama1d_h_period
      h_startorbit = ama1d_h_startorbit
      h_startdatayr = ama1d_h_startdatayr
      h_startdatady = ama1d_h_startdatady
      h_startdatatime = ama1d_h_startdatatime
      h_endorbit = ama1d_h_endorbit
      h_enddatayr = ama1d_h_enddatayr
      h_enddatady = ama1d_h_enddatady
      h_enddatatime = ama1d_h_enddatatime
      h_scnlin = ama1d_h_scnlin
      h_misscnlin = ama1d_h_misscnlin
      h_atovppvn = ama1d_h_atovppvn
      h_instruments = ama1d_h_instruments
      h_wmosatid = ama1d_h_wmosatid
      do ichan=1,AMSU_CHANS
        do i=1,3
          h_amsPlanck(i,ichan) = ama1d_h_amsPlanck(i,ichan)
        enddo
      enddo
      h_Bcorrvn = ama1d_h_Bcorrvn
      h_Bpcpvn = ama1d_h_Bpcpvn
      h_Bantenna = ama1d_h_Bantenna

      RETURN
      END
      
!-------------------------------------------------------------------------------

      SUBROUTINE AMA1D_READ_DATA (
     &  h_scnlin,
     &  scnlin, scnlinyr, scnlindy, scnlintime,
     &  qualind, scanqual, instrtemp1, instrtemp2, instrtempb, 
     &  latitude, longitude, surfheight, surftype, satzen, solzen,
     &  satazi, solazi, scalti, btemps, prepro)
      IMPLICIT NONE

! Parameters
      INTEGER*4 AMA_SPOTS
      PARAMETER (AMA_SPOTS=30)
      INTEGER*4 UNIT_1D
      PARAMETER (UNIT_1D=10)
      INTEGER*4 AMSU_CHANS
      PARAMETER (AMSU_CHANS=20)

! Arguments: 
      INTEGER*4 h_scnlin
      INTEGER*4 scnlin(h_scnlin)
      INTEGER*4 scnlinyr(h_scnlin)
      INTEGER*4 scnlindy(h_scnlin)
      INTEGER*4 scnlintime(h_scnlin)
      INTEGER*4 qualind(h_scnlin)
      INTEGER*4 scanqual(h_scnlin)
      INTEGER*4 instrtemp1(h_scnlin)
      INTEGER*4 instrtemp2(h_scnlin)
      INTEGER*4 instrtempb(h_scnlin)
      INTEGER*4 latitude(AMA_SPOTS,h_scnlin)
      INTEGER*4 longitude(AMA_SPOTS,h_scnlin)
      INTEGER*4 surfheight(AMA_SPOTS,h_scnlin)
      INTEGER*4 surftype(AMA_SPOTS,h_scnlin)
      INTEGER*4 satzen(AMA_SPOTS,h_scnlin)
      INTEGER*4 satazi(AMA_SPOTS,h_scnlin)
      INTEGER*4 solzen(AMA_SPOTS,h_scnlin)
      INTEGER*4 solazi(AMA_SPOTS,h_scnlin)
      INTEGER*4 scalti(h_scnlin)
      INTEGER*4 btemps(AMSU_CHANS,AMA_SPOTS,h_scnlin)
      INTEGER*4 prepro(6,AMA_SPOTS,h_scnlin)

! Local
      INTEGER iscan,ichan,ispot

#include "inparms.h"
#include "ama1d.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Read AMSU-A data records'
      DO iscan=1,h_scnlin
        CALL IOA1D("READ DATA",UNIT_1D,iscan+1)
        scnlin(iscan) = ama1d_scnlin
        scnlinyr(iscan) = ama1d_scnlinyr
        scnlindy(iscan) = ama1d_scnlindy
        scnlintime(iscan) = ama1d_scnlintime
        qualind(iscan) = ama1d_qualind
        scanqual(iscan) = ama1d_scanqual
        instrtemp1(iscan) = ama1d_instrtemp1
        instrtemp2(iscan) = ama1d_instrtemp2
        instrtempb(iscan) = ama1d_instrtempb
        scalti(iscan) = ama1d_scalti
        DO ispot=1,AMA_SPOTS
          DO ichan=1,AMSU_CHANS
            btemps(ichan,ispot,iscan) = ama1d_btemps(ichan,ispot)
          ENDDO
          latitude(ispot,iscan) = ama1d_latlon(1,ispot)
          longitude(ispot,iscan) = ama1d_latlon(2,ispot)
          surfheight(ispot,iscan) = ama1d_surface(1,ispot)
          surftype(ispot,iscan) = ama1d_surface(2,ispot)
          satzen(ispot,iscan) = ama1d_angles(1,ispot)
          satazi(ispot,iscan) = ama1d_angles(2,ispot)
          solzen(ispot,iscan) = ama1d_angles(3,ispot)
          solazi(ispot,iscan) = ama1d_angles(4,ispot)
          DO ichan=1,6
            prepro(ichan,ispot,iscan) = ama1d_prepro(ichan,ispot)
          ENDDO
        ENDDO
      ENDDO

      CALL IOA1D("CLOSE",UNIT_1D,0)

      RETURN
      END
      
