! Name: $Id: amb1d_read_f.F 418 2014-09-29 15:22:46Z frna $
!
! Purpose: Read AMSUB/MHS l1d file and pass information back to C calling routine.
!          Subroutines amb1d_READ_HEADER and amb1d_READ_DATA
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    25/09/2014  N C Atkinson
!-----------------------------------------------------------------------

      SUBROUTINE AMB1D_READ_HEADER (
     & filename, filename_length,
     & h_site, h_1bsite, h_versnb,
     & h_satid, h_grid, h_satht, h_period,
     & h_startorbit, h_startdatayr, h_startdatady, h_startdatatime,
     & h_endorbit, h_enddatayr, h_enddatady, h_enddatatime,
     & h_scnlin, h_misscnlin, h_atovppvn, h_instruments, h_wmosatid,
     & h_amsPlanck, h_Bcorrvn, h_Bpcpvn, h_Bantenna)

      IMPLICIT NONE
      
! Parameters
      INTEGER   UNIT_1D
      PARAMETER (UNIT_1D=10)
      INTEGER*4 AMSU_CHANS
      PARAMETER (AMSU_CHANS=20)

! Arguments: 
      CHARACTER*(*) filename
      INTEGER       filename_length
      CHARACTER*3   h_site
      CHARACTER*3   h_1bsite
      INTEGER*4     h_versnb
      INTEGER*4     h_satid
      INTEGER*4     h_grid
      INTEGER*4     h_satht
      INTEGER*4     h_period
      INTEGER*4     h_startorbit
      INTEGER*4     h_startdatayr
      INTEGER*4     h_startdatady
      INTEGER*4     h_startdatatime
      INTEGER*4     h_endorbit
      INTEGER*4     h_enddatayr
      INTEGER*4     h_enddatady
      INTEGER*4     h_enddatatime
      INTEGER*4     h_scnlin
      INTEGER*4     h_misscnlin
      INTEGER*4     h_atovppvn
      INTEGER*4     h_instruments
      INTEGER*4     h_wmosatid
      INTEGER*4     h_amsPlanck(3,AMSU_CHANS)
      INTEGER*4     h_Bcorrvn
      INTEGER*4     h_Bpcpvn
      INTEGER*4     h_Bantenna

! Local:
      INTEGER errStatus, i, ichan

#include "inparms.h"
#include "amb1d.h"
#include "reclopt.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Open AMSU-B/MHS 1d file ',filename(1:filename_length)
      OPEN ( UNIT=UNIT_1D, FILE=filename(1:filename_length), 
     &   STATUS="OLD", FORM="UNFORMATTED", ACCESS="DIRECT",
     &   RECL=amb_1drecl/rldiv, IOSTAT=errStatus )
      IF (errStatus.NE.0) THEN
        write(*,*)'Cannot open file ',filename(1:filename_length)
        call aappexit(1)
      ENDIF

      write(*,*)'Read AMSU-B/MHS header'
      CALL IOB1D("READ HEADER",UNIT_1D,1)

      h_scnlin = amb1d_h_scnlin
      h_site = amb1d_h_site
      h_1bsite = amb1d_h_1bsite
      h_versnb = amb1d_h_versnb
      h_satid = amb1d_h_satid
      h_grid = amb1d_h_grid
      h_satht = amb1d_h_satht
      h_period = amb1d_h_period
      h_startorbit = amb1d_h_startorbit
      h_startdatayr = amb1d_h_startdatayr
      h_startdatady = amb1d_h_startdatady
      h_startdatatime = amb1d_h_startdatatime
      h_endorbit = amb1d_h_endorbit
      h_enddatayr = amb1d_h_enddatayr
      h_enddatady = amb1d_h_enddatady
      h_enddatatime = amb1d_h_enddatatime
      h_scnlin = amb1d_h_scnlin
      h_misscnlin = amb1d_h_misscnlin
      h_atovppvn = amb1d_h_atovppvn
      h_instruments = amb1d_h_instruments
      h_wmosatid = amb1d_h_wmosatid
      do ichan=1,AMSU_CHANS
        do i=1,3
          h_amsPlanck(i,ichan) = amb1d_h_amsPlanck(i,ichan)
        enddo
      enddo
      h_Bcorrvn = amb1d_h_Bcorrvn
      h_Bpcpvn = amb1d_h_Bpcpvn
      h_Bantenna = amb1d_h_Bantenna

      RETURN
      END
      
!-------------------------------------------------------------------------------

      SUBROUTINE AMB1D_READ_DATA (
     &  h_scnlin,
     &  scnlin, scnlinyr, scnlindy, scnlintime,
     &  qualind, scanqual, instrtemp, 
     &  latitude, longitude, surfheight, surftype, satzen, solzen,
     &  satazi, solazi, scalti, btemps, prepro)
      IMPLICIT NONE

! Parameters
      INTEGER*4 AMB_CHANS
      PARAMETER (AMB_CHANS=5)
      INTEGER*4 AMB_SPOTS
      PARAMETER (AMB_SPOTS=90)
      INTEGER*4 UNIT_1D
      PARAMETER (UNIT_1D=10)
      INTEGER*4 AMSU_CHANS
      PARAMETER (AMSU_CHANS=20)

! Arguments: 
      INTEGER*4 h_scnlin
      INTEGER*4 scnlin(h_scnlin)
      INTEGER*4 scnlinyr(h_scnlin)
      INTEGER*4 scnlindy(h_scnlin)
      INTEGER*4 scnlintime(h_scnlin)
      INTEGER*4 qualind(h_scnlin)
      INTEGER*4 scanqual(h_scnlin)
      INTEGER*4 instrtemp(h_scnlin)
      INTEGER*4 instrtempa1(h_scnlin)
      INTEGER*4 instrtempa2(h_scnlin)
      INTEGER*4 instrtempb(h_scnlin)
      INTEGER*4 latitude(AMB_SPOTS,h_scnlin)
      INTEGER*4 longitude(AMB_SPOTS,h_scnlin)
      INTEGER*4 surfheight(AMB_SPOTS,h_scnlin)
      INTEGER*4 surftype(AMB_SPOTS,h_scnlin)
      INTEGER*4 satzen(AMB_SPOTS,h_scnlin)
      INTEGER*4 satazi(AMB_SPOTS,h_scnlin)
      INTEGER*4 solzen(AMB_SPOTS,h_scnlin)
      INTEGER*4 solazi(AMB_SPOTS,h_scnlin)
      INTEGER*4 scalti(h_scnlin)
      INTEGER*4 btemps(AMSU_CHANS,AMB_SPOTS,h_scnlin)
      INTEGER*4 prepro(6,AMB_SPOTS,h_scnlin)

! Local
      INTEGER iscan,ichan,ispot

#include "inparms.h"
#include "amb1d.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Read AMSUB/MHS data records'
      DO iscan=1,h_scnlin
        CALL IOB1D("READ DATA",UNIT_1D,iscan+1)
        scnlin(iscan) = amb1d_scnlin
        scnlinyr(iscan) = amb1d_scnlinyr
        scnlindy(iscan) = amb1d_scnlindy
        scnlintime(iscan) = amb1d_scnlintime
        qualind(iscan) = amb1d_qualind
        scanqual(iscan) = amb1d_scanqual
        instrtemp(iscan) = amb1d_instrtemp
        scalti(iscan) = amb1d_scalti
        DO ispot=1,AMB_SPOTS
          DO ichan=1,AMSU_CHANS
            btemps(ichan,ispot,iscan) = amb1d_btemps(ichan,ispot)
          ENDDO
          latitude(ispot,iscan) = amb1d_latlon(1,ispot)
          longitude(ispot,iscan) = amb1d_latlon(2,ispot)
          surfheight(ispot,iscan) = amb1d_surface(1,ispot)
          surftype(ispot,iscan) = amb1d_surface(2,ispot)
          satzen(ispot,iscan) = amb1d_angles(1,ispot)
          satazi(ispot,iscan) = amb1d_angles(2,ispot)
          solzen(ispot,iscan) = amb1d_angles(3,ispot)
          solazi(ispot,iscan) = amb1d_angles(4,ispot)
          DO ichan=1,6
            prepro(ichan,ispot,iscan) = amb1d_prepro(ichan,ispot)
          ENDDO
        ENDDO
      ENDDO

      CALL IOB1D("CLOSE",UNIT_1D,0)

      RETURN
      END
      
