/*
! name: $Id: atms_geo_read.c 635 2019-04-15 15:08:00Z frna $
!
! Purpose: Read the contents of an ATMS Geolocation file in HDF5 format
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    12/10/2010  Original code. (N.C.Atkinson)
!
! Code Description: C
!
!-------------------------------------------------------------------------------
*/

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include "atms_sdr.h"
#include <stdio.h>
#include <stdlib.h>

#include "aapp_hdf5.h"
#include "common_read.h"

int atms_geo_read(char *filename){
  
  int     iscan;              /* Loop variables */
  hid_t   file_id, group_id;  /* handles */
  
/****** Open the file ******/

  file_id = H5Fopen(filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if (file_id < 0){
    fprintf(stderr, "Cannot open file %s\n",filename);
    exit(10);
  }

/****** Open the group ATMS-SDR-GEO_All ******/

  group_id  = H5Gopen(file_id, "All_Data/ATMS-SDR-GEO_All", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open group All_Data/ATMS-SDR-GEO_All\n");
    exit(10);
  }

/****** Read the data ******/

  readdataset("Latitude", group_id, H5T_NATIVE_FLOAT, Latitude);
  readdataset("Longitude", group_id, H5T_NATIVE_FLOAT, Longitude);
  readdataset("SatelliteAzimuthAngle", group_id, H5T_NATIVE_FLOAT, SatelliteAzimuthAngle);
  readdataset("SatelliteZenithAngle", group_id, H5T_NATIVE_FLOAT, SatelliteZenithAngle);
  readdataset("SolarAzimuthAngle", group_id, H5T_NATIVE_FLOAT, SolarAzimuthAngle);
  readdataset("SolarZenithAngle", group_id, H5T_NATIVE_FLOAT, SolarZenithAngle);
  readdataset("SCPosition", group_id, H5T_NATIVE_FLOAT, SCPosition);
  readdataset("QF1_ATMSSDRGEO", group_id, H5T_NATIVE_UCHAR, QF1_ATMSSDRGEO);

  H5Gclose(group_id);

/****** Process the quality flags ******/

  for (iscan=0; iscan<ATMS_SCANS; iscan++){
    geolocqual[iscan] = QF1_ATMSSDRGEO[iscan];
  }

/****** Display ******/

  printf("Finished reading %s\n",filename);

/****** Clean up ******/

  H5Fclose(file_id);
  return 0;

}
#endif
