/*
! name: $Id: atms_tdr_read.c 635 2019-04-15 15:08:00Z frna $
!
! Purpose: Read the contents of an ATMS TDR in HDF5 format
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    15/10/2010  Original code. (N.C.Atkinson)
!
! Code Description: C
!
!-------------------------------------------------------------------------------
*/

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include "atms_sdr.h"
#include <stdio.h>
#include <stdlib.h>

#include "aapp_hdf5.h"
#include "common_read.h"

int atms_tdr_read(char *filename){
  
  int     igran,iscan,jscan,ispot,ichan;    /* Loop variables */
  hid_t   file_id, group_id;                /* handles */
  herr_t  status;
  
/****** Open the file ******/

  file_id = H5Fopen(filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if (file_id < 0){
    fprintf(stderr, "cannot open file %s\n",filename);
    exit(10);
  }

/****** Open the group All_Data/ATMS-TDR_All ******/

  group_id  = H5Gopen(file_id, "All_Data/ATMS-TDR_All", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Error opening group All_Data/ATMS-SDR_All\n");
    exit(10);
  }

/****** Read the data ******/

  readdataset("AntennaTemperature", group_id, H5T_NATIVE_USHORT,
    AntennaTemperature);
  readdataset("AntennaTemperatureFactors", group_id, H5T_NATIVE_FLOAT,
    AntennaTemperatureFactors);

  H5Gclose(group_id);

/****** Convert BTs ******/

  for (igran=0;igran<AggregateNumberGranules;igran++) {
    for (iscan=0;iscan<ATMS_SCANS_PER_GRANULE;iscan++) {
      jscan = igran*ATMS_SCANS_PER_GRANULE + iscan;
      for (ispot=0;ispot<ATMS_SPOTS;ispot++) {
        for (ichan=0;ichan<ATMS_CHANS;ichan++) {
          if (AntennaTemperature[jscan][ispot][ichan] < 65528) {
            AT_real[jscan][ispot][ichan] = AntennaTemperature[jscan][ispot][ichan]
               *AntennaTemperatureFactors[igran*2] + AntennaTemperatureFactors[igran*2 + 1];
          } else {
            AT_real[jscan][ispot][ichan] = RMDI;
          }
        }
      }
    }
  }
        
/****** Display ******/

  printf("Finished reading %s\n",filename);

/****** Clean up ******/

  H5Fclose(file_id);
  return 0;

}
#endif
