! Name: $Id: avh1c_read_f.F 478 2016-02-10 13:38:43Z frna $
!
! Purpose: Read AVHRR l1c file and pass information back to C calling routine.
!          Subroutines AMB1C_READ_HEADER and AMB1C_READ_DATA
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    29/09/2014  N C Atkinson
!  1.1    10/02/2016  Add nedt
!-----------------------------------------------------------------------

      SUBROUTINE AVH1C_READ_HEADER (
     & filename, filename_length,
     & h_site, h_1bsite, h_versnb,
     & h_satid, h_instrument, h_satht, h_period,
     & h_startorbit, h_startdatayr, h_startdatady, h_startdatatime,
     & h_endorbit, h_enddatayr, h_enddatady, h_enddatatime,
     & h_scnlin, h_misscnlin, h_callocscnlin, h_datagaps, h_datatyp,
     & h_infilename, h_wmosatid, h_cpidsyr, h_cpidsdy, h_rollerr,
     & h_pitcherr, h_yawerr, h_albcnv, h_radtempcnv)

      IMPLICIT NONE
      
! Parameters
      INTEGER   UNIT_1C
      PARAMETER (UNIT_1C=10)
      INTEGER*4 AVH_VISCHN
      PARAMETER (AVH_VISCHN=3)
      INTEGER*4 AVH_IRCHN
      PARAMETER (AVH_IRCHN=3)

! Arguments: 
      CHARACTER*(*) filename
      INTEGER       filename_length
      CHARACTER*3   h_site
      CHARACTER*3   h_1bsite
      INTEGER*4     h_versnb
      INTEGER*4     h_satid
      INTEGER*4     h_instrument
      INTEGER*4     h_satht
      INTEGER*4     h_period
      INTEGER*4     h_startorbit
      INTEGER*4     h_startdatayr
      INTEGER*4     h_startdatady
      INTEGER*4     h_startdatatime
      INTEGER*4     h_endorbit
      INTEGER*4     h_enddatayr
      INTEGER*4     h_enddatady
      INTEGER*4     h_enddatatime
      INTEGER*4     h_scnlin
      INTEGER*4     h_misscnlin
      INTEGER*4     h_callocscnlin
      INTEGER*4     h_datagaps
      INTEGER*4     h_datatyp
      CHARACTER*68  h_infilename
      INTEGER*4     h_wmosatid
      INTEGER*4     h_cpidsyr
      INTEGER*4     h_cpidsdy
      INTEGER*4     h_rollerr
      INTEGER*4     h_pitcherr
      INTEGER*4     h_yawerr
      INTEGER*4     h_albcnv(2,AVH_VISCHN)
      INTEGER*4     h_radtempcnv(3,AVH_IRCHN)

! Local:
      INTEGER errStatus, i, ichan

#include "inparms.h"
#include "avh1c.h"
#include "reclopt.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Open AVHRR 1c file ',filename(1:filename_length)
      OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &   STATUS="OLD", FORM="UNFORMATTED", ACCESS="DIRECT",
     &   RECL=avh1c_record_size_byte/rldiv, IOSTAT=errStatus )
      IF (errStatus.NE.0) THEN
        write(*,*)'Cannot open file ',filename(1:filename_length)
        call aappexit(1)
      ENDIF

      write(*,*)'Read AVHRR header'
      CALL IOAVH1C1D("READ HEADER",UNIT_1C,1)

      h_site = avh1c_h_site
      h_1bsite = avh1c_h_1bsite
      h_versnb = avh1c_h_versnb
      h_satid = avh1c_h_satid
      h_instrument = avh1c_h_instrument
      h_satht = avh1c_h_satht
      h_period = avh1c_h_period
      h_startorbit = avh1c_h_startorbit
      h_startdatayr = avh1c_h_startdatayr
      h_startdatady = avh1c_h_startdatady
      h_startdatatime = avh1c_h_startdatatime
      h_endorbit = avh1c_h_endorbit
      h_enddatayr = avh1c_h_enddatayr
      h_enddatady = avh1c_h_enddatady
      h_enddatatime = avh1c_h_enddatatime
      h_scnlin = avh1c_h_scnlin
      h_misscnlin = avh1c_h_misscnlin
      h_callocscnlin = avh1c_h_callocscnlin
      h_datagaps = avh1c_h_datagaps
      h_datatyp = avh1c_h_datatyp
      h_infilename = avh1c_h_infilename
      h_wmosatid = avh1c_h_wmosatid
      do ichan=1,AVH_VISCHN
        do i=1,2
          h_albcnv(i,ichan) = avh1c_h_albcnv(i,ichan)
        enddo
      enddo
      do ichan=1,AVH_IRCHN
        do i=1,3
          h_radtempcnv(i,ichan) = avh1c_h_radtempcnv(i,ichan)
        enddo
      enddo

      RETURN
      END
      
!-------------------------------------------------------------------------------

      SUBROUTINE AVH1C_READ_DATA (
     &  h_scnlin,
     &  scnlin, scnlinyr, scnlindy, scnlintime,
     &  clockdrift, scnlinbit,
     &  qualind, scnlinqual, calqual, scalti,
     &  latitude, longitude, satzen, solzen, relazi,
     &  btemps, maskind, mask, tsurf, nedt)
      IMPLICIT NONE

! Parameters
      INTEGER*4 AVH_CHANS
      PARAMETER (AVH_CHANS=5)
      INTEGER*4 AVH_VISCHN
      PARAMETER (AVH_VISCHN=3)
      INTEGER*4 AVH_IRCHN
      PARAMETER (AVH_IRCHN=3)
      INTEGER*4 AVH_SPOTS
      PARAMETER (AVH_SPOTS=2048)
      INTEGER*4 AVH_TIE
      PARAMETER (AVH_TIE=51)
      INTEGER*4 UNIT_1C
      PARAMETER (UNIT_1C=10)

! Arguments: 
      INTEGER*4 h_scnlin
      INTEGER*4 scnlin(h_scnlin)
      INTEGER*4 scnlinyr(h_scnlin)
      INTEGER*4 scnlindy(h_scnlin)
      INTEGER*4 scnlintime(h_scnlin)
      INTEGER*4 clockdrift(h_scnlin)
      INTEGER*4 scnlinbit(h_scnlin)
      INTEGER*4 qualind(h_scnlin)
      INTEGER*4 scnlinqual(h_scnlin)
      INTEGER*4 calqual(AVH_IRCHN,h_scnlin)	
      INTEGER*4 scalti(h_scnlin)
      INTEGER*4 latitude(AVH_TIE,h_scnlin)
      INTEGER*4 longitude(AVH_TIE,h_scnlin)
      INTEGER*4 satzen(AVH_TIE,h_scnlin)
      INTEGER*4 solzen(AVH_TIE,h_scnlin)
      INTEGER*4 relazi(AVH_TIE,h_scnlin)
      INTEGER*2 btemps(AVH_CHANS,AVH_SPOTS,h_scnlin)
      INTEGER*4 maskind(h_scnlin)
      INTEGER*2 mask(AVH_SPOTS,h_scnlin)
      INTEGER*2 tsurf(AVH_SPOTS,h_scnlin)
      INTEGER*4 nedt(AVH_IRCHN,h_scnlin)

! Local
      INTEGER iscan,ichan,ispot,errStatus,lastscan

#include "inparms.h"
#include "avh1c.h"

! End of declarations
!-----------------------------------------------------------------------

!Find the actual number of records present
      write(*,*)'Read AVHRR data records'
      lastscan = 0
      DO iscan = h_scnlin,1,-1
        IF (lastscan.eq.0) THEN
          READ ( UNIT=UNIT_1C,REC=iscan+1,IOSTAT=errStatus ) avh1c_dt
          IF (errStatus.EQ.0) lastscan = iscan
        ENDIF
      ENDDO
      IF (lastscan.NE.h_scnlin) THEN
        write(*,*)'WARNING: number of data records:',lastscan,
     &            ' is less than avh1c_h_scnlin:',h_scnlin
      ENDIF

      DO iscan=1,lastscan
        CALL IOAVH1C1D("READ DATA",UNIT_1C,iscan+1)
        scnlin(iscan) = avh1c_scnlin
        scnlinyr(iscan) = avh1c_scnlinyr
        scnlindy(iscan) = avh1c_scnlindy
        scnlintime(iscan) = avh1c_scnlintime
        clockdrift(iscan) = avh1c_clockdrift
        scnlinbit(iscan) = avh1c_scnlinbit
        qualind(iscan) = avh1c_qualind
        scnlinqual(iscan) = avh1c_scnlinqual
        scalti(iscan) = avh1c_scalti
        maskind(iscan) = avh1c_maskind
        DO ichan=1,AVH_IRCHN
          calqual(ichan,iscan) = avh1c_calqual(ichan)
          nedt(ichan,iscan) = avh1c_nedt(ichan)
        ENDDO
        DO ispot=1,AVH_SPOTS
          DO ichan=1,AVH_CHANS
            btemps(ichan,ispot,iscan) = avh1c_btemps(ichan,ispot)
          ENDDO
          mask(ispot,iscan) = avh1c_mask(ispot)
          tsurf(ispot,iscan) = avh1c_tsurf(ispot)
        ENDDO
        DO ispot=1,AVH_TIE
          latitude(ispot,iscan) = avh1c_pos(1,ispot)
          longitude(ispot,iscan) = avh1c_pos(2,ispot)
          solzen(ispot,iscan) = avh1c_ang(1,ispot)
          satzen(ispot,iscan) = avh1c_ang(2,ispot)
          relazi(ispot,iscan) = avh1c_ang(3,ispot)
        ENDDO
      ENDDO

      CALL IOAVH1C1D("CLOSE",UNIT_1C,0)

      DO iscan=lastscan+1,h_scnlin    !if missing records at end
        scnlin(iscan) = MDI
        scnlinyr(iscan) = MDI
        scnlindy(iscan) = MDI
        scnlintime(iscan) = MDI
        DO ispot=1,AVH_TIE
          latitude(ispot,iscan) = MDI
          longitude(ispot,iscan) = MDI
          satzen(ispot,iscan) = MDI
          solzen(ispot,iscan) = MDI
          relazi(ispot,iscan) = MDI
        ENDDO
      ENDDO      
      RETURN
      END
      
