/*
! name: $Id: common_read.c 635 2019-04-15 15:08:00Z frna $
!
! Purpose: Common subroutines for reading HDF5 data
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    19/10/2010  Original code. (N.C.Atkinson)
!  1.1    17/01/2012  Continue if attribute not found
!
! Code Description: C
!
!-------------------------------------------------------------------------------
*/

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include <stdio.h>
#include <stdlib.h>

#include "common_read.h"

void readdataset(char *name, hid_t id, hid_t mem_type_id, void *buf){
  hid_t dataset_id;
  dataset_id = H5Dopen(id, name, H5P_DEFAULT);
  if (dataset_id < 0) {
    fprintf(stderr, "cannot open dataset %s\n",name);
    exit(10);
  }
  if (H5Dread(dataset_id, mem_type_id, H5S_ALL, H5S_ALL, H5P_DEFAULT, buf) < 0){
    fprintf(stderr, "cannot read dataset %s\n",name);
    exit(11);
  }
  H5Dclose(dataset_id);
}

void readattribute(char *name, hid_t id, hid_t mem_type_id, void *buf){
  hid_t attr_id;
  attr_id = H5Aopen(id, name, H5P_DEFAULT);
  if (attr_id < 0) {
    fprintf(stderr, "cannot open attribute %s\n",name);
  } else {
    if (H5Aread(attr_id, mem_type_id, buf) < 0){
      fprintf(stderr, "cannot read attribute %s\n",name);
      exit(11);
    }
    H5Aclose(attr_id);
  }
}

#endif
