! Name: $Id: cris1c_read_f.F 454 2015-07-14 13:15:09Z frna $
!
! Purpose: Read CRIS l1c file and pass information back to C calling routine.
!          Subroutines CRIS1C_READ_HEADER and CRIS1C_READ_DATA
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    09/07/2015  N C Atkinson
!-----------------------------------------------------------------------

      SUBROUTINE CRIS1C_READ_HEADER (
     & filename, filename_length,
     & h_site, h_1bsite, h_versnb,
     & h_satid, h_instrument, h_satht, h_period,
     & h_startorbit, h_startdatayr, h_startdatady, h_startdatatime,
     & h_endorbit, h_enddatayr, h_enddatady, h_enddatatime,
     & h_scnlin, h_misscnlin, h_wmosatid,
     & h_centre, h_subcentre, h_nfov, h_nfor, h_nband,
     & h_startw, h_endw, h_startchan, h_endchan,
     & h_eigvn, h_npcused, h_chanvn, h_nchanused,
     & h_channels )

      IMPLICIT NONE
      
#include "inparms.h"
#include "cris1c.h"
#include "reclopt.h"

! Parameters
      INTEGER   UNIT_1C
      PARAMETER (UNIT_1C=10)

! Arguments: 
      CHARACTER*(*) filename
      INTEGER       filename_length
      CHARACTER*3   h_site
      CHARACTER*3   h_1bsite
      INTEGER*4     h_versnb
      INTEGER*4     h_satid
      INTEGER*4     h_instrument
      INTEGER*4     h_satht
      INTEGER*4     h_period
      INTEGER*4     h_startorbit
      INTEGER*4     h_startdatayr
      INTEGER*4     h_startdatady
      INTEGER*4     h_startdatatime
      INTEGER*4     h_endorbit
      INTEGER*4     h_enddatayr
      INTEGER*4     h_enddatady
      INTEGER*4     h_enddatatime
      INTEGER*4     h_scnlin
      INTEGER*4     h_misscnlin
      INTEGER*4     h_wmosatid
      INTEGER*4     h_centre
      INTEGER*4     h_subcentre
      INTEGER*4     h_nfov
      INTEGER*4     h_nfor
      INTEGER*4     h_nband
      INTEGER*4     h_startw(3)
      INTEGER*4     h_endw(3)
      INTEGER*4     h_startchan(3)
      INTEGER*4     h_endchan(3)
      INTEGER*4     h_eigvn
      INTEGER*4     h_npcused
      INTEGER*4     h_chanvn
      INTEGER*4     h_nchanused
      INTEGER*4     h_channels(cris_1cnchan)

! Local:
      INTEGER errStatus, i, ichan


! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Open CRIS 1c file ',filename(1:filename_length)
      cris1c_h_recwords = 48    !first part of header only
      OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &   STATUS="OLD", FORM="UNFORMATTED", ACCESS="DIRECT",
     &   RECL=cris1c_h_recwords*4/rldiv, IOSTAT=errStatus )
      IF (errStatus.NE.0) THEN
        write(*,*)'Cannot open file ',filename(1:filename_length)
        call aappexit(1)
      ENDIF

      write(*,*)'Read CRIS header'
      CALL IOC1C("READ HEADER",UNIT_1C,1)   !Get cris1c_h_recwords
      CALL IOC1C("CLOSE",UNIT_1C,1)

      OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &   STATUS="OLD", FORM="UNFORMATTED", ACCESS="DIRECT",
     &   RECL=cris1c_h_recwords*4/rldiv, IOSTAT=errStatus )     !Re-open with correct record length
      IF (errStatus.NE.0) THEN
        write(*,*)'Cannot open file ',filename(1:filename_length)
        call aappexit(1)
      ENDIF
      CALL IOC1C("READ HEADER",UNIT_1C,1)

      h_scnlin = cris1c_h_scnlin
      h_site = cris1c_h_site
      h_1bsite = cris1c_h_1bsite
      h_versnb = cris1c_h_versnb
      h_satid = cris1c_h_satid
      h_instrument = cris1c_h_instrument
      h_satht = cris1c_h_satht
      h_period = cris1c_h_period
      h_startorbit = cris1c_h_startorbit
      h_startdatayr = cris1c_h_startdatayr
      h_startdatady = cris1c_h_startdatady
      h_startdatatime = cris1c_h_startdatatime
      h_endorbit = cris1c_h_endorbit
      h_enddatayr = cris1c_h_enddatayr
      h_enddatady = cris1c_h_enddatady
      h_enddatatime = cris1c_h_enddatatime
      h_scnlin = cris1c_h_scnlin
      h_misscnlin = cris1c_h_misscnlin
      h_wmosatid = cris1c_h_wmosatid
      h_centre = cris1c_h_centre
      h_subcentre = cris1c_h_subcentre
      h_nfov = cris1c_h_nfov
      h_nfor = cris1c_h_nfor
      h_nband = cris1c_h_nband
      DO i=1,3
        h_startw(i) = cris1c_h_startw(i)
        h_endw(i) = cris1c_h_endw(i)
        h_startchan(i) = cris1c_h_startchan(i)
        h_endchan(i) = cris1c_h_endchan(i)
      ENDDO
      h_eigvn = cris1c_h_eigvn
      h_npcused = cris1c_h_npcused
      h_chanvn = cris1c_h_chanvn
      h_nchanused = cris1c_h_nchanused
      DO i=1,cris1c_h_nchanused
        h_channels(i) = cris1c_h_channels(i)
      ENDDO

      RETURN
      END
      
!-----------------------------------------------------------------------

      SUBROUTINE CRIS1C_READ_DATA (
     &  h_scnlin, h_npc, h_nchan,
     &  scnlin, scnlinyr, scnlindy, scnlintime,
     &  latitude, longitude, satzen, solzen, satazi, solazi,
     &  satxyz, ascdesc, orbit,
     &  scalti, surfht, landfrac, landsea, cloudcover, cloudtopht, 
     &  radianceflgs, scanqualflgs, calqualflgs, fovqualflgs, 
     &  geolocqual, quality, scores, logfit, radiance )

      IMPLICIT NONE

#include "inparms.h"
#include "cris1c.h"

! Parameters
      INTEGER*4 CRIS_FOR
      PARAMETER (CRIS_FOR=cris_1cnfor)
      INTEGER*4 CRIS_FOV
      PARAMETER (CRIS_FOV=cris_1cnfov)
      INTEGER   UNIT_1C
      PARAMETER (UNIT_1C=10)

! Arguments: 
      INTEGER*4 h_scnlin
      INTEGER*4 h_npc
      INTEGER*4 h_nchan
      INTEGER*4 scnlin(h_scnlin)
      INTEGER*4 scnlinyr(h_scnlin)
      INTEGER*4 scnlindy(h_scnlin)
      INTEGER*4 scnlintime(CRIS_FOR,h_scnlin)
      INTEGER*4 latitude(CRIS_FOV,CRIS_FOR,h_scnlin)
      INTEGER*4 longitude(CRIS_FOV,CRIS_FOR,h_scnlin)
      INTEGER*4 satzen(CRIS_FOV,CRIS_FOR,h_scnlin)
      INTEGER*4 satazi(CRIS_FOV,CRIS_FOR,h_scnlin)
      INTEGER*4 solzen(CRIS_FOV,CRIS_FOR,h_scnlin)
      INTEGER*4 solazi(CRIS_FOV,CRIS_FOR,h_scnlin)
      INTEGER*4 satxyz(3,h_scnlin)
      INTEGER*4 ascdesc(h_scnlin)
      INTEGER*4 orbit(h_scnlin)
      INTEGER*4 scalti(h_scnlin)
      INTEGER*4 surfht(CRIS_FOV,CRIS_FOR,h_scnlin)
      INTEGER*4 landfrac(CRIS_FOV,CRIS_FOR,h_scnlin)
      INTEGER*4 landsea(CRIS_FOV,CRIS_FOR,h_scnlin)
      INTEGER*4 cloudcover(CRIS_FOV,CRIS_FOR,h_scnlin)
      INTEGER*4 cloudtopht(CRIS_FOV,CRIS_FOR,h_scnlin)
      INTEGER*4 radianceflgs(CRIS_FOV,CRIS_FOR,h_scnlin)
      INTEGER*4 scanqualflgs(h_scnlin)
      INTEGER*4 calqualflgs(3,CRIS_FOV,h_scnlin)
      INTEGER*4 fovqualflgs(3,CRIS_FOV,CRIS_FOR,h_scnlin)
      INTEGER*4 geolocqual(h_scnlin)
      INTEGER*4 quality(CRIS_FOV,CRIS_FOR,h_scnlin)
      INTEGER*4 scores(h_npc,CRIS_FOV,CRIS_FOR,h_scnlin)
      INTEGER*4 logfit(3,CRIS_FOV,CRIS_FOR,h_scnlin)
      INTEGER*4 radiance(h_nchan,CRIS_FOV,CRIS_FOR,h_scnlin)

! Local
      INTEGER iscan,ichan,ispot,ifov,ifor,iband,ipc,errStatus,i

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Read CRIS data records, h_scnlin=',h_scnlin

      DO iscan=1,h_scnlin
        CALL IOC1C("READ DATA",UNIT_1C,iscan+1)
        scnlin(iscan) = cris1c_scnlin
        scnlinyr(iscan) = cris1c_scnlinyr
        scnlindy(iscan) = cris1c_scnlindy
        scalti(iscan) = cris1c_scalti
        ascdesc(iscan) = cris1c_ascdesc
        orbit(iscan) = cris1c_orbit
        scanqualflgs(iscan) = cris1c_scanqualflgs
        geolocqual(iscan) = cris1c_geolocqual
        satxyz(1,iscan) = cris1c_satxyz(1)
        satxyz(2,iscan) = cris1c_satxyz(2)
        satxyz(3,iscan) = cris1c_satxyz(3)
        DO ifor=1,CRIS_FOR
          scnlintime(ifor,iscan) = cris1c_scnlintime(ifor)
          DO ifov=1,CRIS_FOV
            DO ichan=1,h_nchan
              radiance(ichan,ifov,ifor,iscan) =
     &            cris1c_radiance(ifov,ifor,ichan)
            ENDDO
            latitude(ifov,ifor,iscan) = cris1c_latlon(1,ifov,ifor)
            longitude(ifov,ifor,iscan) = cris1c_latlon(2,ifov,ifor)
            satzen(ifov,ifor,iscan) = cris1c_angles(1,ifov,ifor)
            satazi(ifov,ifor,iscan) = cris1c_angles(2,ifov,ifor)
            solzen(ifov,ifor,iscan) = cris1c_angles(3,ifov,ifor)
            solazi(ifov,ifor,iscan) = cris1c_angles(4,ifov,ifor)
            surfht(ifov,ifor,iscan) = cris1c_surfht(ifov,ifor)
            landsea(ifov,ifor,iscan) = cris1c_landsea(ifov,ifor)/100
            cloudcover(ifov,ifor,iscan) = cris1c_cloudcover(ifov,ifor)
            cloudtopht(ifov,ifor,iscan) = cris1c_cloudtopht(ifov,ifor)
            radianceflgs(ifov,ifor,iscan) = 
     &           cris1c_radianceflgs(ifov,ifor)
            DO iband=1,3
              fovqualflgs(iband,ifov,ifor,iscan) = 
     &           cris1c_fovqualflgs(iband,ifov,ifor)
              logfit(iband,ifov,ifor,iscan) = 
     &           cris1c_logfit(iband,ifov,ifor)
            ENDDO
            quality(ifov,ifor,iscan) = cris1c_quality(ifov,ifor)
            DO ipc=1,h_npc
              scores(ipc,ifov,ifor,iscan) = cris1c_scores(ipc,ifov,ifor)
            ENDDO
          ENDDO  !ifov
        ENDDO  !ifor
        DO ifov=1,CRIS_FOV
          DO iband=1,3
            calqualflgs(iband,ifov,iscan)=cris1c_calqualflgs(iband,ifov)
          ENDDO
        ENDDO
      ENDDO  !iscan

      CALL IOC1C("CLOSE",UNIT_1C,0)

      RETURN
      END
      
