! Name: $Id: hrp1b_read_f.F 479 2016-02-10 15:24:33Z frna $
!
! Purpose: Read AVHRR l1b file and pass information back to C calling routine.
!          Subroutines HRP1B_READ_HEADER and HRP1B_READ_DATA
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    09/10/2014  N C Atkinson
!  1.1    17/02/2015  Check whether h_scnlin is negative
!  1.2    13/10/2015  Change order of calvis indices, to allow slope to be 
!                     written as unsigned
!  1.3    10/02/2016  Add nedt and irscale
!-----------------------------------------------------------------------

      SUBROUTINE HRP1B_READ_HEADER (
     & filename, filename_length,
     & h_siteid,h_l1bversnb,h_l1bversyr,h_l1bversdy,h_dataname,
     & h_prblkid,
     & h_satid,h_instid,h_datatyp,h_tipsrc,h_startdatajd,h_startdatayr,
     & h_startdatady,h_startdatatime,h_enddatajd,h_enddatayr,
     & h_enddatady,
     & h_enddatatime,h_cpidsyr,h_cpidsdy,h_inststat1,h_statchrecnb,
     & h_inststat2,h_scnlin,h_callocscnlin,h_misscnlin,h_datagaps,
     & h_okdatafr,h_pacsparityerr,h_auxsyncerrsum,h_timeseqerr,
     & h_timeseqerrcode,h_socclockupind,h_locerrind,h_locerrcode,
     & h_pacsstatfield,h_pacsdatasrc,h_racalind,h_solarcalyr,
     & h_solarcaldy,
     & h_pcalalgind,h_pcalalgopt,h_scalalgind,h_scalalgopt,
     & h_irttcoef,h_albcnv,h_radtempcnv,h_modelid,h_nadloctol,h_locbit,
     & h_rollerr,h_pitcherr,h_yawerr,h_epoyr,h_epody,h_epotime,
     & h_smaxis,h_eccen,h_incli,h_argper,h_rascnod,h_manom,
     & h_xpos,h_ypos,h_zpos,h_xvel,h_yvel,h_zvel,h_earthsun)


      IMPLICIT NONE
      
! Parameters
      INTEGER   UNIT_1B
      PARAMETER (UNIT_1B=10)
      INTEGER*4 AVH_VISCHN
      PARAMETER (AVH_VISCHN=3)
      INTEGER*4 AVH_IRCHN
      PARAMETER (AVH_IRCHN=3)

! Arguments: 
      CHARACTER*(*) filename
      INTEGER       filename_length

      CHARACTER*3  h_siteid
      INTEGER*2    h_l1bversnb
      INTEGER*2    h_l1bversyr
      INTEGER*2    h_l1bversdy
      CHARACTER*42 h_dataname
      CHARACTER*8  h_prblkid
      INTEGER*2    h_satid
      INTEGER*2    h_instid
      INTEGER*2    h_datatyp
      INTEGER*2    h_tipsrc
      INTEGER*4    h_startdatajd
      INTEGER*2    h_startdatayr
      INTEGER*2    h_startdatady
      INTEGER*4    h_startdatatime
      INTEGER*4    h_enddatajd
      INTEGER*2    h_enddatayr
      INTEGER*2    h_enddatady
      INTEGER*4    h_enddatatime
      INTEGER*2    h_cpidsyr
      INTEGER*2    h_cpidsdy
      INTEGER*4    h_inststat1
      INTEGER*2    h_statchrecnb
      INTEGER*4    h_inststat2
      INTEGER*2    h_scnlin
      INTEGER*2    h_callocscnlin
      INTEGER*2    h_misscnlin
      INTEGER*2    h_datagaps
      INTEGER*2    h_okdatafr
      INTEGER*2    h_pacsparityerr
      INTEGER*2    h_auxsyncerrsum
      INTEGER*2    h_timeseqerr
      INTEGER*2    h_timeseqerrcode
      INTEGER*2    h_socclockupind
      INTEGER*2    h_locerrind
      INTEGER*2    h_locerrcode
      INTEGER*2    h_pacsstatfield
      INTEGER*2    h_pacsdatasrc
      INTEGER*2    h_racalind
      INTEGER*2    h_solarcalyr
      INTEGER*2    h_solarcaldy
      INTEGER*2    h_pcalalgind
      INTEGER*2    h_pcalalgopt
      INTEGER*2    h_scalalgind
      INTEGER*2    h_scalalgopt
      INTEGER*2    h_irttcoef(6,4)
      INTEGER*4    h_albcnv(2,AVH_VISCHN)
      INTEGER*4    h_radtempcnv(3,AVH_IRCHN)
      CHARACTER*8  h_modelid
      INTEGER*2    h_nadloctol
      INTEGER*2    h_locbit
      INTEGER*2    h_rollerr
      INTEGER*2    h_pitcherr
      INTEGER*2    h_yawerr
      INTEGER*2    h_epoyr
      INTEGER*2    h_epody
      INTEGER*4    h_epotime
      INTEGER*4    h_smaxis
      INTEGER*4    h_eccen
      INTEGER*4    h_incli
      INTEGER*4    h_argper
      INTEGER*4    h_rascnod
      INTEGER*4    h_manom
      INTEGER*4    h_xpos
      INTEGER*4    h_ypos
      INTEGER*4    h_zpos
      INTEGER*4    h_xvel
      INTEGER*4    h_yvel
      INTEGER*4    h_zvel
      INTEGER*4    h_earthsun

! Local:
      INTEGER errStatus, i, ichan, icoef

#include "inparms.h"
#include "avh1b.h"
#include "reclopt.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Open AVHRR 1b file ',filename(1:filename_length)
      OPEN ( UNIT=UNIT_1B, FILE=filename(1:filename_length), 
     &   STATUS="OLD", FORM="UNFORMATTED", ACCESS="DIRECT",
     &   RECL=reclavh/rldiv, IOSTAT=errStatus )
      IF (errStatus.NE.0) THEN
        write(*,*)'Cannot open file ',filename(1:filename_length)
        call aappexit(1)
      ENDIF

      write(*,*)'Read AVHRR header'
      CALL AVHHDR (UNIT_1B, errStatus)
      IF (errStatus .NE. 0) THEN
        write(*,*)'Error reading AVHRR header'
        call aappexit(1)
      ENDIF

      h_siteid = avh_h_siteid
      h_l1bversnb = avh_h_l1bversnb
      h_l1bversyr = avh_h_l1bversyr
      h_l1bversdy = avh_h_l1bversdy
      h_dataname = avh_h_dataname
      h_prblkid = avh_h_prblkid
      h_satid = avh_h_satid
      h_instid = avh_h_instid
      h_datatyp = avh_h_datatyp
      h_tipsrc = avh_h_tipsrc
      h_startdatajd = avh_h_startdatajd
      h_startdatayr = avh_h_startdatayr
      h_startdatady = avh_h_startdatady
      h_startdatatime = avh_h_startdatatime
      h_enddatajd = avh_h_enddatajd
      h_enddatayr = avh_h_enddatayr
      h_enddatady = avh_h_enddatady
      h_enddatatime = avh_h_enddatatime
      h_cpidsyr = avh_h_cpidsyr
      h_cpidsdy = avh_h_cpidsdy
      h_inststat1 = avh_h_inststat1
      h_statchrecnb = avh_h_statchrecnb
      h_inststat2 = avh_h_inststat2
      h_scnlin = avh_h_scnlin
      h_callocscnlin = avh_h_callocscnlin
      h_misscnlin = avh_h_misscnlin
      h_datagaps = avh_h_datagaps
      h_okdatafr = avh_h_okdatafr
      h_pacsparityerr = avh_h_pacsparityerr
      h_auxsyncerrsum = avh_h_auxsyncerrsum
      h_timeseqerr = avh_h_timeseqerr
      h_timeseqerrcode = avh_h_timeseqerrcode
      h_socclockupind = avh_h_socclockupind
      h_locerrind = avh_h_locerrind
      h_locerrcode = avh_h_locerrcode
      h_pacsstatfield = avh_h_pacsstatfield
      h_pacsdatasrc = avh_h_pacsdatasrc
      h_racalind = avh_h_racalind
      h_solarcalyr = avh_h_solarcalyr
      h_solarcaldy = avh_h_solarcaldy
      h_pcalalgind = avh_h_pcalalgind
      h_pcalalgopt = avh_h_pcalalgopt
      h_scalalgind = avh_h_scalalgind
      h_scalalgopt = avh_h_scalalgopt
      do icoef=1,6
        do i=1,4
          h_irttcoef(icoef,i) = avh_h_irttcoef(icoef,i)
        enddo
      enddo
      do ichan=1,AVH_VISCHN
        do i=1,2
          h_albcnv(i,ichan) = avh_h_albcnv(i,ichan)
        enddo
      enddo
      do ichan=1,AVH_IRCHN
        do i=1,3
          h_radtempcnv(i,ichan) = avh_h_radtempcnv(i,ichan)
        enddo
      enddo
      h_modelid = avh_h_modelid
      h_nadloctol = avh_h_nadloctol
      h_locbit = avh_h_locbit
      h_rollerr = avh_h_rollerr
      h_pitcherr = avh_h_pitcherr
      h_yawerr = avh_h_yawerr
      h_epoyr = avh_h_epoyr
      h_epody = avh_h_epody
      h_epotime = avh_h_epotime
      h_smaxis = avh_h_smaxis
      h_eccen = avh_h_eccen
      h_incli = avh_h_incli
      h_argper = avh_h_argper
      h_rascnod = avh_h_rascnod 
      h_manom = avh_h_manom
      h_xpos = avh_h_xpos
      h_ypos = avh_h_ypos 
      h_zpos = avh_h_zpos 
      h_xvel = avh_h_xvel
      h_yvel = avh_h_yvel
      h_zvel = avh_h_zvel
      h_earthsun = avh_h_earthsun

      RETURN
      END
      
!-------------------------------------------------------------------------------

      SUBROUTINE HRP1B_READ_DATA (
     &  h_scnlin, nspots,
     &  scnlin, scnlinyr, scnlindy, scnlintime,
     &  clockdrift, scnlinbit, 
     &  qualind, scnlinqual, calqual, cbiterr, calvis, calir,
     &  navstat, attangtime, rollang, pitchang, yawang, scalti,
     &  latitude, longitude, satzen, solzen, relazi, telem, hrpt, nedt,
     &  irscale)   
      IMPLICIT NONE

! Parameters
      INTEGER*4 AVH_CHANS
      PARAMETER (AVH_CHANS=5)
      INTEGER*4 AVH_VISCHN
      PARAMETER (AVH_VISCHN=3)
      INTEGER*4 AVH_IRCHN
      PARAMETER (AVH_IRCHN=3)
      INTEGER*4 AVH_TIE
      PARAMETER (AVH_TIE=51)
      INTEGER*4 UNIT_1B
      PARAMETER (UNIT_1B=10)

! Arguments: 

      INTEGER*2 h_scnlin
      INTEGER*4 nspots
      INTEGER*2 scnlin(h_scnlin)
      INTEGER*2 scnlinyr(h_scnlin)
      INTEGER*2 scnlindy(h_scnlin)
      INTEGER*2 clockdrift(h_scnlin)
      INTEGER*4 scnlintime(h_scnlin)
      INTEGER*2 scnlinbit(h_scnlin)
      INTEGER*4 qualind(h_scnlin)
      INTEGER*4 scnlinqual(h_scnlin)
      INTEGER*2 calqual(AVH_IRCHN, h_scnlin)
      INTEGER*2 cbiterr(h_scnlin)
!      INTEGER*4 calvis(5,3,AVH_VISCHN,h_scnlin)
      INTEGER*4 calvis(3,AVH_VISCHN,h_scnlin,5)
      INTEGER*4 calir(3,2,AVH_IRCHN,h_scnlin)
      INTEGER*4 navstat(h_scnlin)
      INTEGER*4 attangtime(h_scnlin)
      INTEGER*2 rollang(h_scnlin)
      INTEGER*2 pitchang(h_scnlin)
      INTEGER*2 yawang(h_scnlin)
      INTEGER*2 scalti(h_scnlin)
      INTEGER*2 ang(3, AVH_TIE, h_scnlin)
      INTEGER*4 latitude(AVH_TIE, h_scnlin)
      INTEGER*4 longitude(AVH_TIE, h_scnlin)
      INTEGER*2 satzen(AVH_TIE, h_scnlin)
      INTEGER*2 solzen(AVH_TIE, h_scnlin)
      INTEGER*2 relazi(AVH_TIE, h_scnlin)
      INTEGER*2 telem(103, h_scnlin)
      INTEGER*2 hrpt(AVH_CHANS, nspots, h_scnlin)
      INTEGER*2 nedt(AVH_IRCHN, h_scnlin)
      INTEGER*2 irscale(3)

! Local
      INTEGER iscan,ichan,ispot,jspot,errStatus,iset,icoef,i
      INTEGER nscan

#include "inparms.h"
#include "avh1b.h"

! End of declarations
!-----------------------------------------------------------------------

      nscan = h_scnlin
      if (nscan .LT. 0) nscan = nscan + 65536
      DO iscan=1,nscan
        CALL AVHDTR (UNIT_1B, iscan, errStatus)
        IF (errStatus.NE.0) THEN
          write(*,*)'Error reading AVHRR scan', iscan,' of',nscan
          call aappexit(1)
        ENDIF

        scnlin(iscan) = avh_scnlin
        scnlinyr(iscan) = avh_scnlinyr
        scnlindy(iscan) = avh_scnlindy
        clockdrift(iscan) = avh_clockdrift
        scnlintime(iscan) = avh_scnlintime
        scnlinbit(iscan) = avh_scnlinbit
        qualind(iscan) = avh_qualind
        scnlinqual(iscan) = avh_scnlinqual
        DO ichan=1,AVH_IRCHN
          calqual(ichan,iscan) = avh_calqual(ichan)
          nedt(ichan,iscan) = avh_filler3(ichan)/256
        ENDDO
        cbiterr(iscan) = avh_cbiterr
        DO ichan=1,AVH_VISCHN
          DO iset=1,3
            DO icoef=1,5
              calvis(iset,ichan,iscan,icoef) = 
     &           avh_calvis(icoef,iset,ichan)
            ENDDO
          ENDDO
        ENDDO
        DO ichan=1,AVH_IRCHN
          DO iset=1,2
            DO icoef=1,3
              calir(icoef,iset,ichan,iscan) = 
     &           avh_calir(icoef,iset,ichan)
            ENDDO
          ENDDO
        ENDDO
        DO icoef=1,3
          irscale(icoef) = max(irscale(icoef),avh_filler2(icoef))
        ENDDO
        navstat(iscan) = avh_navstat
        attangtime(iscan) = avh_attangtime
        rollang(iscan) = avh_rollang
        pitchang(iscan) = avh_pitchang
        yawang(iscan) = avh_yawang
        scalti(iscan) = avh_scalti
        DO ispot=1,AVH_TIE
          latitude(ispot,iscan) = avh_pos(1,ispot)
          longitude(ispot,iscan) = avh_pos(2,ispot)
          solzen(ispot,iscan) = avh_ang(1,ispot)
          satzen(ispot,iscan) = avh_ang(2,ispot)
          relazi(ispot,iscan) = avh_ang(3,ispot)
        ENDDO
        DO i=1,103
          telem(i,iscan) = avh_telem(i)
        ENDDO
        DO ispot=1,nspots
          IF (nspots .EQ. 409) THEN
            jspot = ispot*5        !For GAC
          ELSE
            jspot = ispot
          ENDIF
          DO ichan=1,AVH_CHANS
            hrpt(ichan, ispot, iscan) = avh_hrpt(ichan, jspot)
          ENDDO
        ENDDO
      ENDDO

      CLOSE (UNIT_1B)

      RETURN
      END
      
