! Name: $Id: hrs1d_read_f.F 418 2014-09-29 15:22:46Z frna $
!
! Purpose: Read HIRS l1d file and pass information back to C calling routine.
!          Subroutines hrs1d_READ_HEADER and hrs1d_READ_DATA
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    25/09/2014  N C Atkinson
!-----------------------------------------------------------------------

      SUBROUTINE HRS1D_READ_HEADER (
     & filename, filename_length,
     & h_site, h_1bsite, h_versnb,
     & h_satid, h_grid, h_satht, h_period,
     & h_startorbit, h_startdatayr, h_startdatady, h_startdatatime,
     & h_endorbit, h_enddatayr, h_enddatady, h_enddatatime,
     & h_scnlin, h_misscnlin, h_atovppvn, h_instruments, h_wmosatid,
     & h_hrsPlanck, h_20solfiltirrad, h_20equifiltwidth, h_amsPlanck,
     & h_dT, h_mapBTmode, h_Hcorrvn, h_Acorrvn, h_Apcpvn,
     & h_Aantenna, h_Bcorrvn, h_Bpcpvn, h_Bantenna, h_Ascat,
     & h_ACrosby, h_AB89)

      IMPLICIT NONE
      
! Parameters
      INTEGER   UNIT_1D
      PARAMETER (UNIT_1D=10)
      INTEGER*4 HIRS_CHANS
      PARAMETER (HIRS_CHANS=20)
      INTEGER*4 AMSU_CHANS
      PARAMETER (AMSU_CHANS=20)

! Arguments: 
      CHARACTER*(*) filename
      INTEGER       filename_length
      CHARACTER*3   h_site
      CHARACTER*3   h_1bsite
      INTEGER*4     h_versnb
      INTEGER*4     h_satid
      INTEGER*4     h_grid
      INTEGER*4     h_satht
      INTEGER*4     h_period
      INTEGER*4     h_startorbit
      INTEGER*4     h_startdatayr
      INTEGER*4     h_startdatady
      INTEGER*4     h_startdatatime
      INTEGER*4     h_endorbit
      INTEGER*4     h_enddatayr
      INTEGER*4     h_enddatady
      INTEGER*4     h_enddatatime
      INTEGER*4     h_scnlin
      INTEGER*4     h_misscnlin
      INTEGER*4     h_atovppvn
      INTEGER*4     h_instruments
      INTEGER*4     h_wmosatid
      INTEGER*4     h_hrsPlanck(3,HIRS_CHANS-1)
      INTEGER*4     h_20solfiltirrad
      INTEGER*4     h_20equifiltwidth
      INTEGER*4     h_amsPlanck(3,AMSU_CHANS)
      INTEGER*4     h_dT
      INTEGER*4     h_mapBTmode
      INTEGER*4     h_Hcorrvn
      INTEGER*4     h_Acorrvn
      INTEGER*4     h_Apcpvn
      INTEGER*4     h_Aantenna
      INTEGER*4     h_Bcorrvn
      INTEGER*4     h_Bpcpvn
      INTEGER*4     h_Bantenna
      INTEGER*4     h_Ascat
      INTEGER*4     h_ACrosby
      INTEGER*4     h_AB89

! Local:
      INTEGER errStatus, i, ichan

#include "inparms.h"
#include "hrs1d.h"
#include "reclopt.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Open HIRS 1d file ',filename(1:filename_length)
      OPEN ( UNIT=UNIT_1D, FILE=filename(1:filename_length), 
     &   STATUS="OLD", FORM="UNFORMATTED", ACCESS="DIRECT",
     &   RECL=hrs_1drecl/rldiv, IOSTAT=errStatus )
      IF (errStatus.NE.0) THEN
        write(*,*)'Cannot open file ',filename(1:filename_length)
        call aappexit(1)
      ENDIF

      write(*,*)'Read HIRS header'
      CALL IOH1D("READ HEADER",UNIT_1D,1)

      h_scnlin = hrs1d_h_scnlin
      h_site = hrs1d_h_site
      h_1bsite = hrs1d_h_1bsite
      h_versnb = hrs1d_h_versnb
      h_satid = hrs1d_h_satid
      h_grid = hrs1d_h_grid
      h_satht = hrs1d_h_satht
      h_period = hrs1d_h_period
      h_startorbit = hrs1d_h_startorbit
      h_startdatayr = hrs1d_h_startdatayr
      h_startdatady = hrs1d_h_startdatady
      h_startdatatime = hrs1d_h_startdatatime
      h_endorbit = hrs1d_h_endorbit
      h_enddatayr = hrs1d_h_enddatayr
      h_enddatady = hrs1d_h_enddatady
      h_enddatatime = hrs1d_h_enddatatime
      h_scnlin = hrs1d_h_scnlin
      h_misscnlin = hrs1d_h_misscnlin
      h_atovppvn = hrs1d_h_atovppvn
      h_instruments = hrs1d_h_instruments
      h_wmosatid = hrs1d_h_wmosatid
      do ichan=1,HIRS_CHANS-1
        do i=1,3
          h_hrsPlanck(i,ichan) = hrs1d_h_hrsPlanck(i,ichan)
        enddo
      enddo
      h_20solfiltirrad = hrs1d_h_20solfiltirrad
      h_20equifiltwidth = hrs1d_h_20equifiltwidth
      do ichan=1,AMSU_CHANS
        do i=1,3
          h_amsPlanck(i,ichan) = hrs1d_h_amsPlanck(i,ichan)
        enddo
      enddo
      h_dT = hrs1d_h_dT
      h_mapBTmode = hrs1d_h_mapBTmode
      h_Hcorrvn = hrs1d_h_Hcorrvn
      h_Acorrvn = hrs1d_h_Acorrvn
      h_Apcpvn = hrs1d_h_Apcpvn
      h_Aantenna = hrs1d_h_Aantenna
      h_Bcorrvn = hrs1d_h_Bcorrvn
      h_Bpcpvn = hrs1d_h_Bpcpvn
      h_Bantenna = hrs1d_h_Bantenna
      h_Ascat = hrs1d_h_Ascat
      h_ACrosby = hrs1d_h_ACrosby
      h_AB89 = hrs1d_h_AB89

      RETURN
      END
      
!-------------------------------------------------------------------------------

      SUBROUTINE HRS1D_READ_DATA (
     &  h_scnlin,
     &  scnlin, scnlinyr, scnlindy, scnlintime,
     &  qualind, scanqual, instrtemp, instrtempa1,
     &  instrtempa2, instrtempb,
     &  latitude, longitude, surfheight, surftype, satzen, solzen,
     &  satazi, solazi, scalti, btemps, avhrr, hirsqual, prepro)
      IMPLICIT NONE

! Parameters
      INTEGER*4 HIRS_CHANS
      PARAMETER (HIRS_CHANS=20)
      INTEGER*4 HIRS_SPOTS
      PARAMETER (HIRS_SPOTS=56)
      INTEGER*4 UNIT_1D
      PARAMETER (UNIT_1D=10)
      INTEGER*4 TOTAL_CHANS
      PARAMETER (TOTAL_CHANS=40)

! Arguments: 
      INTEGER*4 h_scnlin
      INTEGER*4 scnlin(h_scnlin)
      INTEGER*4 scnlinyr(h_scnlin)
      INTEGER*4 scnlindy(h_scnlin)
      INTEGER*4 scnlintime(h_scnlin)
      INTEGER*4 qualind(h_scnlin)
      INTEGER*4 scanqual(h_scnlin)
      INTEGER*4 instrtemp(h_scnlin)
      INTEGER*4 instrtempa1(h_scnlin)
      INTEGER*4 instrtempa2(h_scnlin)
      INTEGER*4 instrtempb(h_scnlin)
      INTEGER*4 latitude(HIRS_SPOTS,h_scnlin)
      INTEGER*4 longitude(HIRS_SPOTS,h_scnlin)
      INTEGER*4 surfheight(HIRS_SPOTS,h_scnlin)
      INTEGER*4 surftype(HIRS_SPOTS,h_scnlin)
      INTEGER*4 satzen(HIRS_SPOTS,h_scnlin)
      INTEGER*4 satazi(HIRS_SPOTS,h_scnlin)
      INTEGER*4 solzen(HIRS_SPOTS,h_scnlin)
      INTEGER*4 solazi(HIRS_SPOTS,h_scnlin)
      INTEGER*4 scalti(h_scnlin)
      INTEGER*4 btemps(TOTAL_CHANS,HIRS_SPOTS,h_scnlin)
      INTEGER*4 avhrr(13,HIRS_SPOTS,h_scnlin)
      INTEGER*4 hirsqual(HIRS_SPOTS,h_scnlin)
      INTEGER*4 prepro(7,HIRS_SPOTS,h_scnlin)

! Local
      INTEGER iscan,ichan,ispot

#include "inparms.h"
#include "hrs1d.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Read HIRS data records'
      DO iscan=1,h_scnlin
        CALL IOH1D("READ DATA",UNIT_1D,iscan+1)
        scnlin(iscan) = hrs1d_scnlin
        scnlinyr(iscan) = hrs1d_scnlinyr
        scnlindy(iscan) = hrs1d_scnlindy
        scnlintime(iscan) = hrs1d_scnlintime
        qualind(iscan) = hrs1d_qualind
        scanqual(iscan) = hrs1d_scanqual
        instrtemp(iscan) = hrs1d_instrtemp
        instrtempa1(iscan) = hrs1d_instrtempa1
        instrtempa2(iscan) = hrs1d_instrtempa2
        instrtempb(iscan) = hrs1d_instrtempb
        scalti(iscan) = hrs1d_scalti
        DO ispot=1,HIRS_SPOTS
          DO ichan=1,TOTAL_CHANS
            btemps(ichan,ispot,iscan) = hrs1d_btemps(ichan,ispot)
          ENDDO
          latitude(ispot,iscan) = hrs1d_latlon(1,ispot)
          longitude(ispot,iscan) = hrs1d_latlon(2,ispot)
          surfheight(ispot,iscan) = hrs1d_surface(1,ispot)
          surftype(ispot,iscan) = hrs1d_surface(2,ispot)
          satzen(ispot,iscan) = hrs1d_angles(1,ispot)
          satazi(ispot,iscan) = hrs1d_angles(2,ispot)
          solzen(ispot,iscan) = hrs1d_angles(3,ispot)
          solazi(ispot,iscan) = hrs1d_angles(4,ispot)
          hirsqual(ispot,iscan) = hrs1d_hirsqual(ispot)
          DO ichan=1,13
            avhrr(ichan,ispot,iscan) = hrs1d_avhrr(ichan,ispot)
          ENDDO
          DO ichan=1,7
            prepro(ichan,ispot,iscan) = hrs1d_prepro(ichan,ispot)
          ENDDO
        ENDDO
      ENDDO

      CALL IOH1D("CLOSE",UNIT_1D,0)

      RETURN
      END
      
