! Name: $Id: iasi1d_read_f.F 421 2014-10-07 12:29:25Z frna $
!
! Purpose: Read IASI l1d file and pass information back to C calling routine.
!          Subroutines iasi1d_READ_HEADER and iasi1d_READ_DATA
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    06/10/2014  N C Atkinson
!-----------------------------------------------------------------------

      SUBROUTINE IASI1D_READ_HEADER (
     & filename, filename_length,
     & h_site, h_1bsite, h_versnb,
     & h_satid, h_grid, h_satht, h_period,
     & h_startorbit, h_startdatayr, h_startdatady, h_startdatatime,
     & h_endorbit, h_enddatayr, h_enddatady, h_enddatatime,
     & h_scnlin, h_misscnlin, h_atovppvn, h_instruments, h_wmosatid,
     & h_startchan, h_endchan, h_scalefactor,
     & h_eigvn, h_npc, h_npcused,
     & h_chanvn, h_nchan, h_nchanused, h_channels,
     & h_amsplanck, h_dT, h_mapBTmode, h_Acorrvn,
     & h_Apcpvn, h_Aantenna, h_Bcorrvn, h_Bpcpvn, h_Bantenna, h_Ascat,
     & h_ACrosby, h_AB89, h_cloudtests, h_recperscan, h_fovmode );

      IMPLICIT NONE
      
! Parameters
      INTEGER   UNIT_1D
      PARAMETER (UNIT_1D=10)
      INTEGER*4 IASI_CHANS
      PARAMETER (IASI_CHANS=8461)
      INTEGER*4 MAP_CHANS
      PARAMETER (MAP_CHANS=20)

! Arguments: 
      CHARACTER*(*) filename
      INTEGER       filename_length
      CHARACTER*3   h_site
      CHARACTER*3   h_1bsite
      INTEGER*4     h_versnb
      INTEGER*4     h_satid
      INTEGER*4     h_grid
      INTEGER*4     h_satht
      INTEGER*4     h_period
      INTEGER*4     h_startorbit
      INTEGER*4     h_startdatayr
      INTEGER*4     h_startdatady
      INTEGER*4     h_startdatatime
      INTEGER*4     h_endorbit
      INTEGER*4     h_enddatayr
      INTEGER*4     h_enddatady
      INTEGER*4     h_enddatatime
      INTEGER*4     h_scnlin
      INTEGER*4     h_misscnlin
      INTEGER*4     h_atovppvn
      INTEGER*4     h_instruments
      INTEGER*4     h_wmosatid
      INTEGER*4     h_startchan(10)
      INTEGER*4     h_endchan(10)
      INTEGER*4     h_scalefactor(10)
      INTEGER*4     h_eigvn
      INTEGER*4     h_npc
      INTEGER*4     h_npcused
      INTEGER*4     h_chanvn
      INTEGER*4     h_nchan
      INTEGER*4     h_nchanused
      INTEGER*4     h_channels(IASI_CHANS)
      INTEGER*4     h_amsplanck(3, MAP_CHANS)
      INTEGER*4     h_dT
      INTEGER*4     h_mapBTmode
      INTEGER*4     h_Acorrvn
      INTEGER*4     h_Apcpvn
      INTEGER*4     h_Aantenna
      INTEGER*4     h_Bcorrvn
      INTEGER*4     h_Bpcpvn
      INTEGER*4     h_Bantenna
      INTEGER*4     h_Ascat
      INTEGER*4     h_ACrosby
      INTEGER*4     h_AB89
      INTEGER*4     h_cloudtests
      INTEGER*4     h_recperscan
      INTEGER*4     h_fovmode

! Local:
      INTEGER errStatus, i, ichan

#include "inparms.h"
#include "iasi1d.h"
#include "reclopt.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Open IASI 1d file ',filename(1:filename_length)
      OPEN ( UNIT=UNIT_1D, FILE=filename(1:filename_length), 
     &   STATUS="OLD", FORM="UNFORMATTED", ACCESS="DIRECT",
     &   RECL=iasi_1drecl/rldiv, IOSTAT=errStatus )
      IF (errStatus.NE.0) THEN
        write(*,*)'Cannot open file ',filename(1:filename_length)
        call aappexit(1)
      ENDIF

      write(*,*)'Read IASI header'
      CALL IOI1D("READ HEADER",UNIT_1D,1)

      h_site = iasi1d_h_site
      h_1bsite = iasi1d_h_1bsite
      h_versnb = iasi1d_h_versnb
      h_satid = iasi1d_h_satid
      h_grid = iasi1d_h_grid
      h_satht = iasi1d_h_satht
      h_period = iasi1d_h_period
      h_startorbit = iasi1d_h_startorbit
      h_startdatayr = iasi1d_h_startdatayr
      h_startdatady = iasi1d_h_startdatady
      h_startdatatime = iasi1d_h_startdatatime
      h_endorbit = iasi1d_h_endorbit
      h_enddatayr = iasi1d_h_enddatayr
      h_enddatady = iasi1d_h_enddatady
      h_enddatatime = iasi1d_h_enddatatime
      h_scnlin = iasi1d_h_scnlin
      h_misscnlin = iasi1d_h_misscnlin
      h_atovppvn = iasi1d_h_atovppvn
      h_instruments = iasi1d_h_instruments
      h_wmosatid = iasi1d_h_wmosatid
      DO i=1,10
        h_startchan(i) = iasi1d_h_startchan(i)
        h_endchan(i) = iasi1d_h_endchan(i)
        h_scalefactor(i) = iasi1d_h_scalefactor(i)
      ENDDO
      h_eigvn = iasi1d_h_eigvn
      h_npc = iasi1d_h_npc
      h_npcused = iasi1d_h_npcused
      h_chanvn = iasi1d_h_chanvn
      h_nchan = iasi1d_h_nchan 
      h_nchanused = iasi1d_h_nchanused
      DO i=1, h_nchanused
        h_channels(i) = iasi1d_h_channels(i)
      ENDDO
      DO i=1, MAP_CHANS
        h_amsPlanck(1,I) = iasi1d_h_amsPlanck(1,I)
        h_amsPlanck(2,I) = iasi1d_h_amsPlanck(2,I)
        h_amsPlanck(3,I) = iasi1d_h_amsPlanck(3,I)
      ENDDO
      h_dT = iasi1d_h_dT
      h_mapBTmode = iasi1d_h_mapBTmode
      h_Acorrvn = iasi1d_h_Acorrvn
      h_Apcpvn = iasi1d_h_Apcpvn
      h_Aantenna = iasi1d_h_Aantenna
      h_Bcorrvn = iasi1d_h_Bcorrvn
      h_Bpcpvn = iasi1d_h_Bpcpvn
      h_Bantenna = iasi1d_h_Bantenna
      h_Ascat = iasi1d_h_Ascat
      h_ACrosby = iasi1d_h_ACrosby
      h_AB89 = iasi1d_h_AB89
      h_cloudtests = iasi1d_h_cloudtests
      h_recperscan = iasi1d_h_recperscan
      h_fovmode = iasi1d_h_fovmode

      IF (h_nchan .NE. iasi_1dnchan) THEN
        write(*,*) 'Error: number of channels in dataset:',h_nchan
        write(*,*) '  differs from iasi_1dnchan in iasi1d.h: ',
     &     iasi_1dnchan
        write(*,*) 'To read this data, you need to modify iasi1d.h'//
     &    ' then re-build libatovpp, libaapphdf5 and iasi1d_to_hdf5.exe'
        call aappexit(1)
      ENDIF

      IF (h_npc .NE. iasi_1dnpc) THEN
        write(*,*) 'Error: number of PCs in dataset:',h_npc
        write(*,*) '  differs from iasi_1dnpc in iasi1d.h:',iasi_1dnpc
        write(*,*) 'To read this data, you need to modify iasi1d.h'//
     &    ' then re-build libatovpp, libaapphdf5 and iasi1d_to_hdf5.exe'
        call aappexit(1)
      ENDIF

      RETURN
      END
      
!-----------------------------------------------------------------------

      SUBROUTINE IASI1D_READ_DATA (
     &  nrecords, nchanstore, npcstore,
     &  scnlin, scnlinyr, scnlindy, scnlintime,
     &  instrtempa1,instrtempa2,instrtempb,
     &  latitude, longitude, surfheight, surftype, satzen, solzen,
     &  satazi, solazi, scalti, btemps, iasiqual, prepro, emiss,
     &  GQisFlagQual, GQisQualIndex, GQisQualIndexLoc, GQisQualIndexRad,
     &  GQisQualIndexSpect, GQisSysTecSondQual, scalrad, scores,
     &  logfit, FOV, avhcombination, avhrrmean, avhrrsdev,
     &  avhmeanscalefactor, avhsdevscalefactor, avhrrfrac,
     &  avhrrCGy, avhrrCGz, cloudflag)

      IMPLICIT NONE

! Parameters
      INTEGER*4 IASI_CHANS
      PARAMETER (IASI_CHANS=8461)
      INTEGER*4 IASI_FOR
      PARAMETER (IASI_FOR=30)
      INTEGER*4 AVHRR_CLASS
      PARAMETER (AVHRR_CLASS=7)
      INTEGER*4 AVHRR_CHANS
      PARAMETER (AVHRR_CHANS=6)
      INTEGER*4 MAP_CHANS
      PARAMETER (MAP_CHANS=20)
      INTEGER   UNIT_1D
      PARAMETER (UNIT_1D=10)

! Arguments: 
      INTEGER*4 nrecords
      INTEGER*4 nchanstore
      INTEGER*4 npcstore
      INTEGER*4 scnlin(nrecords)
      INTEGER*4 scnlinyr(nrecords)
      INTEGER*4 scnlindy(nrecords)
      INTEGER*4 scnlintime(IASI_FOR,nrecords)
      INTEGER*4 instrtempa1(nrecords)
      INTEGER*4 instrtempa2(nrecords)
      INTEGER*4 instrtempb(nrecords)
      INTEGER*4 latitude(IASI_FOR,nrecords)
      INTEGER*4 longitude(IASI_FOR,nrecords)
      INTEGER*4 surfheight(IASI_FOR,nrecords)
      INTEGER*4 surftype(IASI_FOR,nrecords)
      INTEGER*4 satzen(IASI_FOR,nrecords)
      INTEGER*4 solzen(IASI_FOR,nrecords)
      INTEGER*4 satazi(IASI_FOR,nrecords)
      INTEGER*4 solazi(IASI_FOR,nrecords)
      INTEGER*4 scalti(nrecords)
      INTEGER*4 btemps(MAP_CHANS,IASI_FOR,nrecords)
      INTEGER*4 iasiqual(IASI_FOR,nrecords)
      INTEGER*4 prepro(7,IASI_FOR,nrecords)
      INTEGER*4 emiss(13,IASI_FOR,nrecords)
      INTEGER*4 GQisFlagQual(IASI_FOR,nrecords)
      INTEGER*4 GQisQualIndex(nrecords)
      INTEGER*4 GQisQualIndexLoc(nrecords)
      INTEGER*4 GQisQualIndexRad(nrecords)
      INTEGER*4 GQisQualIndexSpect(nrecords)
      INTEGER*4 GQisSysTecSondQual(nrecords)
      INTEGER*2 scalrad(nchanstore,IASI_FOR,nrecords)
      INTEGER*4 scores(npcstore,IASI_FOR,nrecords)
      INTEGER*4 logfit(3,IASI_FOR,nrecords)
      INTEGER*4 FOV(IASI_FOR,nrecords)
      INTEGER*4 avhcombination(nrecords)
      INTEGER*4 avhrrmean(AVHRR_CHANS,AVHRR_CLASS,IASI_FOR,nrecords)
      INTEGER*4 avhrrsdev(AVHRR_CHANS,AVHRR_CLASS,IASI_FOR,nrecords)
      INTEGER*2 avhmeanscalefactor(AVHRR_CHANS,AVHRR_CLASS,IASI_FOR,
     &  nrecords)
      INTEGER*2 avhsdevscalefactor(AVHRR_CHANS,AVHRR_CLASS,IASI_FOR,
     &  nrecords)
      INTEGER*4 avhrrfrac(AVHRR_CLASS,IASI_FOR,nrecords)
      INTEGER*4 avhrrCGy(AVHRR_CLASS,IASI_FOR,nrecords)
      INTEGER*4 avhrrCGz(AVHRR_CLASS,IASI_FOR,nrecords)
      INTEGER*4 cloudflag(10,IASI_FOR,nrecords)

! Local
      INTEGER iscan,ichan,ipc,ispot,ifor,iclass,errStatus,i

#include "inparms.h"
#include "iasi1d.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Read IASI data records, nrecords=',nrecords

      DO iscan=1,nrecords
        CALL IOI1D("READ DATA",UNIT_1D,iscan+1)
        scnlin(iscan) = iasi1d_scnlin
        scnlinyr(iscan) = iasi1d_scnlinyr
        scnlindy(iscan) = iasi1d_scnlindy
        instrtempa1(iscan) = iasi1d_instrtempa1
        instrtempa2(iscan) = iasi1d_instrtempa2
        instrtempb(iscan) = iasi1d_instrtempb
        scalti(iscan) = iasi1d_scalti
        GQisQualIndex(iscan) = iasi1d_GQisQualIndex
        GQisQualIndexLoc(iscan) = iasi1d_GQisQualIndexLoc
        GQisQualIndexRad(iscan) = iasi1d_GQisQualIndexRad
        GQisQualIndexSpect(iscan) = iasi1d_GQisQualIndexSpect
        GQisSysTecSondQual(iscan) = iasi1d_GQisSysTecSondQual
        avhcombination(iscan) = iasi1d_avhcombination
        DO ifor=1,IASI_FOR
          scnlintime(ifor,iscan) = iasi1d_scnlintime(ifor)
          GQisFlagQual(ifor,iscan) = iasi1d_GQisFlagQual(ifor)
          DO ichan=1,nchanstore
            scalrad(ichan,ifor,iscan) = iasi1d_scalrad(ichan,ifor)
          ENDDO
          DO ipc=1,npcstore
            scores(ipc,ifor,iscan) = iasi1d_scores(ipc,ifor)
          ENDDO
          latitude(ifor,iscan) = iasi1d_latlon(1,ifor)
          longitude(ifor,iscan) = iasi1d_latlon(2,ifor)
          surfheight(ifor,iscan) = iasi1d_surface(1,ifor)
          surftype(ifor,iscan) = iasi1d_surface(2,ifor)
          satzen(ifor,iscan) = iasi1d_angles(1,ifor)
          satazi(ifor,iscan) = iasi1d_angles(2,ifor)
          solzen(ifor,iscan) = iasi1d_angles(3,ifor)
          solazi(ifor,iscan) = iasi1d_angles(4,ifor)
          iasiqual(ifor,iscan) = iasi1d_iasiqual(ifor)
          DO i=1,7
            prepro(i,ifor,iscan) = iasi1d_prepro(i,ifor)
          ENDDO
          DO i=1,13
            emiss(i,ifor,iscan) = iasi1d_emiss(i,ifor)
          ENDDO
          logfit(1,ifor,iscan) = iasi1d_logfit(1,ifor)
          logfit(2,ifor,iscan) = iasi1d_logfit(2,ifor)
          logfit(3,ifor,iscan) = iasi1d_logfit(3,ifor)
          FOV(ifor,iscan) = iasi1d_FOV(ifor)
          DO ichan=1,MAP_CHANS
            btemps(ichan,ifor,iscan) = iasi1d_btemps(ichan,ifor)
          ENDDO
          DO iclass=1,AVHRR_CLASS
            DO ichan=1,AVHRR_CHANS
              avhrrmean(ichan,iclass,ifor,iscan) = 
     &            iasi1d_avhrrmean(ichan,iclass,ifor)
              avhmeanscalefactor(ichan,iclass,ifor,iscan) = 
     &            iasi1d_avhmeanscalefactor(ichan,iclass,ifor)
              avhrrsdev(ichan,iclass,ifor,iscan) = 
     &            iasi1d_avhrrsdev(ichan,iclass,ifor)
              avhsdevscalefactor(ichan,iclass,ifor,iscan) = 
     &            iasi1d_avhsdevscalefactor(ichan,iclass,ifor)
            ENDDO
            avhrrfrac(iclass,ifor,iscan) =
     &         iasi1d_avhrrfrac(iclass,ifor)
            avhrrCGy(iclass,ifor,iscan) = 
     &         iasi1d_avhrrCGy(iclass,ifor)
            avhrrCGz(iclass,ifor,iscan) =
     &         iasi1d_avhrrCGz(iclass,ifor)
          ENDDO  !iclass
          DO i=1,10
            cloudflag(i,ifor,iscan) = iasi1d_cloudflag(i,ifor)
          ENDDO
        ENDDO  !ifor
      ENDDO  !iscan

      CALL IOI1D("CLOSE",UNIT_1D,0)

      RETURN
      END
      
