! Name: $Id: iasipc_write_f.F 608 2019-01-11 14:13:00Z frna $
!
! Purpose: Write IASI l1c file using information from C calling routine.
!          Subroutines IASIPC_WRITE_HEADER and IASIPC_WRITE_DATA
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2019, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    10/01/2019  N C Atkinson
!-----------------------------------------------------------------------

      SUBROUTINE IASIPC_WRITE_HEADER (
     & filename, filename_length,
     & h_site, h_1bsite, h_versnb,
     & h_satid, h_instrument, h_satht, h_period,
     & h_startorbit, h_startdatayr, h_startdatady, h_startdatatime,
     & h_endorbit, h_enddatayr, h_enddatady, h_enddatatime,
     & h_scnlin, h_misscnlin, h_wmosatid,
     & h_startchan, h_endchan, h_scalefactor,
     & h_nchanused, h_nrchanused, h_npcused, h_npcbands,
     & h_startpcchan, h_endpcchan,
     & h_startpc, h_endpc, h_quantization, h_chanvn, h_rchanvn,
     & h_eigvn, h_evfile1, h_evfile2, h_evfile3, h_evfilevn,
     & h_channels, h_rchannels)

      IMPLICIT NONE
      
! Parameters
      INTEGER   UNIT_1C
      PARAMETER (UNIT_1C=10)
      INTEGER   IASI_PCCHANS
      PARAMETER (IASI_PCCHANS=500)

! Arguments: 
      CHARACTER*(*) filename
      INTEGER       filename_length
      CHARACTER*3   h_site
      CHARACTER*3   h_1bsite
      INTEGER*4     h_versnb
      INTEGER*4     h_satid
      INTEGER*4     h_instrument
      INTEGER*4     h_satht
      INTEGER*4     h_period
      INTEGER*4     h_startorbit
      INTEGER*4     h_startdatayr
      INTEGER*4     h_startdatady
      INTEGER*4     h_startdatatime
      INTEGER*4     h_endorbit
      INTEGER*4     h_enddatayr
      INTEGER*4     h_enddatady
      INTEGER*4     h_enddatatime
      INTEGER*4     h_scnlin
      INTEGER*4     h_misscnlin
      INTEGER*4     h_wmosatid
      INTEGER*4     h_startchan(10)
      INTEGER*4     h_endchan(10)
      INTEGER*4     h_scalefactor(10)
      INTEGER*4     h_nchanused
      INTEGER*4     h_nrchanused
      INTEGER*4     h_npcused
      INTEGER*4     h_npcbands
      INTEGER*4     h_startpcchan(3)
      INTEGER*4     h_endpcchan(3)
      INTEGER*4     h_startpc(3)
      INTEGER*4     h_endpc(3)
      INTEGER*4     h_quantization(3)
      INTEGER*4     h_chanvn
      INTEGER*4     h_rchanvn
      INTEGER*4     h_eigvn
      CHARACTER*80  h_evfile1
      CHARACTER*80  h_evfile2
      CHARACTER*80  h_evfile3
      INTEGER*4     h_evfilevn(3)
      INTEGER*4     h_channels(IASI_PCCHANS)
      INTEGER*4     h_rchannels(IASI_PCCHANS)

! Local:
      INTEGER errStatus, i, ichan

#include "inparms.h"
#include "iasipc.h"
#include "reclopt.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Open IASI 1c file ',filename(1:filename_length)
      OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &   STATUS="NEW", FORM="UNFORMATTED", ACCESS="DIRECT",
     &   RECL=iasi_pcrecl/rldiv, IOSTAT=errStatus )
      IF (errStatus.NE.0) THEN
        OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &     STATUS="OLD", FORM="UNFORMATTED", ACCESS="DIRECT",
     &     RECL=iasi_pcrecl/rldiv, IOSTAT=errStatus )
        IF (errStatus.NE.0) THEN
          OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &     STATUS="UNKNOWN", FORM="UNFORMATTED", ACCESS="DIRECT",
     &     RECL=iasi_pcrecl/rldiv, IOSTAT=errStatus )
          IF (errStatus.NE.0) THEN
            write(*,*)'Cannot open file ',filename(1:filename_length)
            call aappexit(1)
          ENDIF
        ENDIF
      ENDIF

      write(*,*)'Write IASI header'
      iasipc_h_scnlin = h_scnlin
      iasipc_h_site = h_site
      iasipc_h_cfill1 = ' '
      iasipc_h_1bsite = h_1bsite
      iasipc_h_cfill2 = ' '
      iasipc_h_versnb = iasi_pcvnnb
      iasipc_h_versyr = iasi_pcvnyr
      iasipc_h_versdy = iasi_pcvndy
      iasipc_h_hdrcnt = 1
      iasipc_h_satid = h_satid
      iasipc_h_instrument = h_instrument
      iasipc_h_satht = h_satht
      iasipc_h_period = h_period
      iasipc_h_startorbit = h_startorbit 
      iasipc_h_startdatayr = h_startdatayr
      iasipc_h_startdatady = h_startdatady
      iasipc_h_startdatatime = h_startdatatime
      iasipc_h_endorbit = h_endorbit
      iasipc_h_enddatayr = h_enddatayr
      iasipc_h_enddatady = h_enddatady
      iasipc_h_enddatatime = h_enddatatime
      iasipc_h_scnlin = h_scnlin
      iasipc_h_misscnlin = h_misscnlin
      iasipc_h_wmosatid = h_wmosatid
      DO i=1,10
        iasipc_h_startchan(i) = h_startchan(i)
        iasipc_h_endchan(i) = h_endchan(i)
        iasipc_h_scalefactor(i) = h_scalefactor(i)
      ENDDO
      iasipc_h_recl = iasi_pcrecl
      iasipc_h_nchan = iasi_pcnchan
      iasipc_h_nrchan = iasi_pcnrecon
      iasipc_h_npc = iasi_pcnpc
      iasipc_h_nchanused = h_nchanused
      iasipc_h_nrchanused = h_nrchanused
      iasipc_h_npcused = h_npcused
      DO i=1,3
        iasipc_h_startpcchan(i) = h_startpcchan(i)
        iasipc_h_endpcchan(i) = h_endpcchan(i)
        iasipc_h_startpc(i) = h_startpc(i)
        iasipc_h_endpc(i) = h_endpc(i)
        iasipc_h_quantization(i) = h_quantization(i)
        iasipc_h_evfilevn(i) = h_evfilevn(i)
      ENDDO
      iasipc_h_chanvn = h_chanvn
      iasipc_h_rchanvn = h_rchanvn
      iasipc_h_eigvn = h_eigvn
      iasipc_h_evfile(1) = h_evfile1
      iasipc_h_evfile(2) = h_evfile2
      iasipc_h_evfile(3) = h_evfile3
      DO i=1, h_nchanused
        iasipc_h_channels(i) = h_channels(i)
      ENDDO
      DO i=1, h_nrchanused
        iasipc_h_rchannels(i) = h_rchannels(i)
      ENDDO

      CALL IOIPC("WRITE HEADER",UNIT_1C,1)

      RETURN
      END
      
!-------------------------------------------------------------------------------

      SUBROUTINE IASIPC_WRITE_DATA (
     &  h_scnlin,
     &  scnlin, granule, scnlinyr, scnlindy, scnlintime,
     &  GQisFlagQual, GQisQualIndex, GQisQualIndexLoc, GQisQualIndexRad,
     &  GQisQualIndexSpect, GQisSysTecSondQual,
     &  latitude, longitude, satzen, solzen, satazi, solazi,
     &  scalti, scalrad, reconrad, scores, logfit, 
     &  avhcombination, avhrrmean, avhmeanscalefactor,
     &  avhrrsdev, avhsdevscalefactor, avhrrfrac, avhrrCGy, avhrrCGz,
     &  GIacVarImagIIS, GIacAvgImagIIS, GEUMAvhrr1BCldFrac,
     &  GEUMAvhrr1BLandFrac, GEUMAvhrr1BQual)
      IMPLICIT NONE

! Parameters
      INTEGER*4 IASI_PCCHANS
      PARAMETER (IASI_PCCHANS=500)
      INTEGER*4 IASI_PCS
      PARAMETER (IASI_PCS=300)
      INTEGER*4 IASI_FOR
      PARAMETER (IASI_FOR=30)
      INTEGER*4 IASI_FOV
      PARAMETER (IASI_FOV=4)
      INTEGER*4 AVHRR_CLASS
      PARAMETER (AVHRR_CLASS=7)
      INTEGER*4 AVHRR_CHANS
      PARAMETER (AVHRR_CHANS=6)
      INTEGER   UNIT_1C
      PARAMETER (UNIT_1C=10)

! Arguments: 
      INTEGER*4 h_scnlin
      INTEGER*4 scnlin(h_scnlin)
      INTEGER*4 granule(h_scnlin)
      INTEGER*4 scnlinyr(h_scnlin)
      INTEGER*4 scnlindy(h_scnlin)
      INTEGER*4 scnlintime(IASI_FOR,h_scnlin)
      INTEGER*4 GQisFlagQual(IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 GQisQualIndex(h_scnlin)
      INTEGER*4 GQisQualIndexLoc(h_scnlin)
      INTEGER*4 GQisQualIndexRad(h_scnlin)
      INTEGER*4 GQisQualIndexSpect(h_scnlin)
      INTEGER*4 GQisSysTecSondQual(h_scnlin)
      INTEGER*4 latitude(IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 longitude(IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 satzen(IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 satazi(IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 solzen(IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 solazi(IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 scalti(h_scnlin)
      INTEGER*2 scalrad(IASI_PCCHANS,IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*2 reconrad(IASI_PCCHANS,IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 scores(IASI_PCS,IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 logfit(3,IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 avhcombination(h_scnlin)
      INTEGER*4 avhrrmean(AVHRR_CHANS,AVHRR_CLASS,IASI_FOV,IASI_FOR,
     &  h_scnlin)
      INTEGER*2 avhmeanscalefactor(AVHRR_CHANS,AVHRR_CLASS,IASI_FOV,
     &  IASI_FOR, h_scnlin)
      INTEGER*4 avhrrsdev(AVHRR_CHANS,AVHRR_CLASS,IASI_FOV,IASI_FOR,
     &  h_scnlin)
      INTEGER*2 avhsdevscalefactor(AVHRR_CHANS,AVHRR_CLASS,IASI_FOV,
     &  IASI_FOR, h_scnlin)
      INTEGER*4 avhrrfrac(AVHRR_CLASS,IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 avhrrCGy(AVHRR_CLASS,IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 avhrrCGz(AVHRR_CLASS,IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 GIacVarImagIIS(IASI_FOR,h_scnlin)
      INTEGER*4 GIacAvgImagIIS(IASI_FOR,h_scnlin)
      INTEGER*4 GEUMAvhrr1BCldFrac(IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 GEUMAvhrr1BLandFrac(IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 GEUMAvhrr1BQual(IASI_FOV,IASI_FOR,h_scnlin)

! Local
      INTEGER iscan,ichan,ispot,ifov,ifor,iclass,errStatus,i

#include "inparms.h"
#include "iasipc.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Write IASI data records'
      DO iscan=1,h_scnlin
        iasipc_scnlin = scnlin(iscan)
        iasipc_granule = granule(iscan)
        iasipc_scnlinyr = scnlinyr(iscan)
        iasipc_scnlindy = scnlindy(iscan)
        iasipc_scalti = scalti(iscan)
        iasipc_avhcombination = avhcombination(iscan)
        iasipc_GQisQualIndex = GQisQualIndex(iscan)
        iasipc_GQisQualIndexLoc = GQisQualIndexLoc(iscan)
        iasipc_GQisQualIndexRad = GQisQualIndexRad(iscan)
        iasipc_GQisQualIndexSpect = GQisQualIndexSpect(iscan)
        iasipc_GQisSysTecSondQual = GQisSysTecSondQual(iscan)
        DO ifor=1,IASI_FOR
          iasipc_scnlintime(ifor) = scnlintime(ifor,iscan)
          DO ifov=1,IASI_FOV
            iasipc_GQisFlagQual(ifov,ifor) =
     &            GQisFlagQual(ifov,ifor,iscan)
            DO ichan=1,IASI_PCCHANS
              iasipc_scalrad(ichan,ifov,ifor) = 
     &            scalrad(ichan,ifov,ifor,iscan)
            ENDDO
            DO ichan=1,IASI_PCCHANS
              iasipc_reconrad(ichan,ifov,ifor) =
     &            reconrad(ichan,ifov,ifor,iscan)
            ENDDO
            DO ichan=1,IASI_PCS
              iasipc_scores(ichan,ifov,ifor) =
     &            scores(ichan,ifov,ifor,iscan)
            ENDDO
            DO i=1,3
              iasipc_logfit(i,ifov,ifor) = logfit(i,ifov,ifor,iscan)
            ENDDO
            iasipc_latlon(1,ifov,ifor) = latitude(ifov,ifor,iscan)
            iasipc_latlon(2,ifov,ifor) = longitude(ifov,ifor,iscan)
            iasipc_angles(1,ifov,ifor) = satzen(ifov,ifor,iscan)
            iasipc_angles(2,ifov,ifor) = satazi(ifov,ifor,iscan)
            iasipc_angles(3,ifov,ifor) = solzen(ifov,ifor,iscan)
            iasipc_angles(4,ifov,ifor) = solazi(ifov,ifor,iscan)
            DO iclass=1,AVHRR_CLASS
              DO ichan=1,AVHRR_CHANS
                iasipc_avhrrmean(ichan,iclass,ifov,ifor) =
     &              avhrrmean(ichan,iclass,ifov,ifor,iscan)
                iasipc_avhmeanscalefactor(ichan,iclass,ifov,ifor) = 
     &              avhmeanscalefactor(ichan,iclass,ifov,ifor,iscan)
                iasipc_avhrrsdev(ichan,iclass,ifov,ifor) = 
     &              avhrrsdev(ichan,iclass,ifov,ifor,iscan)
                iasipc_avhsdevscalefactor(ichan,iclass,ifov,ifor) = 
     &              avhsdevscalefactor(ichan,iclass,ifov,ifor,iscan)
              ENDDO
              iasipc_avhrrfrac(iclass,ifov,ifor) =
     &           avhrrfrac(iclass,ifov,ifor,iscan)
              iasipc_avhrrCGy(iclass,ifov,ifor) =
     &           avhrrCGy(iclass,ifov,ifor,iscan)
              iasipc_avhrrCGz(iclass,ifov,ifor) =
     &           avhrrCGz(iclass,ifov,ifor,iscan)
            ENDDO
            iasipc_GEUMAvhrr1BCldFrac(ifov,ifor) =
     &         GEUMAvhrr1BCldFrac(ifov,ifor,iscan)
            iasipc_GEUMAvhrr1BLandFrac(ifov,ifor) =
     &         GEUMAvhrr1BLandFrac(ifov,ifor,iscan)
            iasipc_GEUMAvhrr1BQual(ifov,ifor) =
     &         GEUMAvhrr1BQual(ifov,ifor,iscan)
          ENDDO
          iasipc_GIacVarImagIIS(ifor) = GIacVarImagIIS(ifor,iscan)
          iasipc_GIacAvgImagIIS(ifor) = GIacAvgImagIIS(ifor,iscan)
        ENDDO  !ifor
        CALL IOIPC("WRITE DATA",UNIT_1C,iscan+1)
      ENDDO  !iscan

      CALL IOIPC("CLOSE",UNIT_1C,0)

      RETURN
      END
      
