! Name: $Id: iras_sdr_out.F 586 2018-04-30 10:32:45Z frna $
!
! Purpose: Fill IRAS 1c structure and write out file
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2013, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    31/01/2013  From mwts_sdr_out.F. (N.C.Atkinson)
!  1.1    05/06/2014  Set iras1c_h_nchan, iras1c_h_nmapchan, iras1c_h_nmapchanused
!  1.2    29/09/2015  Fix startdatady and enddatady
!  1.3    27/04/2018  Add inland water category in surface type
!
!-------------------------------------------------------------------------------

      SUBROUTINE IRAS_SDR_OUT (
     &  Count_scnlines, Orbit_Number, Orbit_Period,
     &  Earth_Obs_BT, Latitude, Longitude, Scan_Time, Scan_Day,
     &  Ave_Instr_Temp, EVS_orb_pos, Central_Wn, geo_qc, scnlin_qc,
     &  cal_qc, pixel_qc,
     &  SensorZenith, SensorAzimuth, SolarZenith, SolarAzimuth, Height,
     &  LandSeaMask, Satellite_Name,
     &  Beginning_Date, Beginning_Time, Ending_Date, Ending_Time)
     
      IMPLICIT NONE

      INTEGER   IRAS_SPOTS, IRAS_CHANS, IRAS_SCANS
      PARAMETER (IRAS_SPOTS=56)
      PARAMETER (IRAS_CHANS=26)
      PARAMETER (IRAS_SCANS=1300)

! Arguments: 
      INTEGER*4    Count_scnlines
      INTEGER*4    Orbit_Number
      INTEGER*4    Orbit_Period
      REAL         Earth_Obs_BT(IRAS_CHANS*IRAS_SCANS*IRAS_SPOTS)
      REAL         Latitude(IRAS_SPOTS,IRAS_SCANS)
      REAL         Longitude(IRAS_SPOTS,IRAS_SCANS)
      REAL         Ave_Instr_Temp(9)
      REAL         EVS_orb_pos(3,IRAS_SCANS)
      REAL         Central_Wn(IRAS_CHANS)
      INTEGER*4    Scan_Time(IRAS_SCANS)
      INTEGER*4    Scan_Day(IRAS_SCANS)
      INTEGER*2    SensorZenith(IRAS_SPOTS,IRAS_SCANS)
      INTEGER*2    SensorAzimuth(IRAS_SPOTS,IRAS_SCANS)
      INTEGER*2    SolarZenith(IRAS_SPOTS,IRAS_SCANS)
      INTEGER*2    SolarAzimuth(IRAS_SPOTS,IRAS_SCANS)
      INTEGER*2    Height(IRAS_SPOTS,IRAS_SCANS)
      INTEGER*2    LandSeaMask(IRAS_SPOTS,IRAS_SCANS)
      INTEGER*2    geo_qc(IRAS_SCANS)
      INTEGER*4    scnlin_qc(IRAS_SCANS)
      INTEGER*4    cal_qc(IRAS_SCANS)
      INTEGER*4    pixel_qc(IRAS_SPOTS,IRAS_SCANS)
      CHARACTER*6  Satellite_Name
      CHARACTER*11 Beginning_Date, Ending_Date
      CHARACTER*13 Beginning_Time, Ending_Time

! Local variables

      INTEGER i,iscan,ispot,ichan,unit,jscan,jchan,iout,id
      INTEGER num_obs
      INTEGER year, month, day, hour, minute, dayofyear
      INTEGER scanline, scnlinold
      INTEGER scantime, scanday, scandate, firsttime, tdiff, scandiff
      REAL    Black_Body_View_Ang_real(IRAS_SCANS), bbref
      REAL    latdiff, londiff, latlondiff, LatEdgediff
      REAL    second
      REAL    BT
      LOGICAL LatOK, LatMissing
      LOGICAL BBOK
      REAL    xysq,z,lat,semiMinor,r_sat,r_ellipse
      CHARACTER*80 line

!Function
      INTEGER    IDAYOFYEAR

! Include
#include "iras1c.h"
#include "inparms.h"
#include "constants_preproc.h"

! End of declarations
!-----------------------------------------------------------------------

      unit = 10
      CALL IOIRAS1C("OPEN/WRITE",unit,0)
      
      semiMinor = semiMajor - semiDiff

      IF (Satellite_Name(1:5) .EQ. "FY-3A") THEN
        iras1c_h_satid = 1
        iras1c_h_wmosatid = 520
      ELSE IF (Satellite_Name(1:5) .EQ. "FY-3B") THEN
        iras1c_h_satid = 2
        iras1c_h_wmosatid = 521
      ELSE IF (Satellite_Name(1:5) .EQ. "FY-3C") THEN
        iras1c_h_satid = 3
        iras1c_h_wmosatid = 522
      ELSE
        iras1c_h_satid = MDI
        iras1c_h_wmosatid = MDI
      ENDIF

      DO ichan=1,IRAS_CHANS
        iras1c_h_tempradcnv(1,ichan) = Central_Wn(ichan)*1E4
           !wavenumber*1E4 (note different scaling from microwave)
        iras1c_h_tempradcnv(2,ichan) = 0  !offset
        iras1c_h_tempradcnv(3,ichan) = 1.0E6  !slope
      ENDDO

      iras1c_h_instrument = 17
      iras1c_h_startorbit = Orbit_Number
      iras1c_h_endorbit = Orbit_Number
      iras1c_h_hdrcnt = 1
      iras1c_h_versnb = IRAS_1cvnnb
      iras1c_h_versyr = IRAS_1cvnyr
      iras1c_h_versdy = IRAS_1cvndy
      iras1c_h_satht = 8350      !nominal
      iras1c_h_period = Orbit_Period*60
      iras1c_h_nchan = iras_nchan
      iras1c_h_nmapchan = iras_nmapchan
      iras1c_h_nmapchanused_mwts = 0
      iras1c_h_nmapchanused_mwhs = 0
      
      CALL IOIRAS1C("WRITE HEADER",unit,1)

      num_obs = Count_scnlines*IRAS_SPOTS
      iout = 0
      iras1c_scnlin = 0

!Main loop

      DO iscan=1,Count_scnlines
        scanday = Scan_Day(iscan)

        IF (scanday .GT. 0) THEN

!Time origin in input is 01/01/2010 12:00
          scantime = Scan_Time(iscan) + 43200000   !reference to midnight (ms)
          IF (scantime .GE. 86400000) THEN
            scantime = scantime - 86400000
            scanday = scanday + 1
          ENDIF
          iras1c_scnlindy = MOD(scanday*4,1461)/4 + 1
          iras1c_scnlinyr = (scanday*4)/1461 + 2000
          iras1c_scnlintime = scantime

          IF (iras1c_h_startdatayr .EQ. 0) THEN
            iras1c_h_startdatayr = iras1c_scnlinyr
            iras1c_h_startdatady = iras1c_scnlindy
            iras1c_h_startdatatime = iras1c_scnlintime
          ENDIF
          iras1c_h_enddatayr = iras1c_scnlinyr
          iras1c_h_enddatady = iras1c_scnlindy
          iras1c_h_enddatatime = iras1c_scnlintime

          IF (iras1c_scnlin .EQ. 0) THEN
            firsttime = scantime
          ENDIF
          
          tdiff = scantime - firsttime
          IF (tdiff .LT. -43200000) THEN 
            tdiff = tdiff + 86400000
          ENDIF
          iras1c_scnlin = NINT(tdiff/6400.0) + 1

          !Check that the latitudes are sensible. Expect max diff of 1 deg 
          !from one scan line to the next
          LatOK = .TRUE.
          LatMissing = .FALSE.
          DO ispot=1,IRAS_SPOTS
            IF (ABS(Latitude(ispot,iscan)).GT.90.0) THEN
              LatOK = .FALSE.
              LatMissing = .TRUE.
            ENDIF
          ENDDO
          !Check latitude difference across the scan
          LatEdgediff = Latitude(1,iscan)-Latitude(56,iscan)
          LatEdgediff = LatEdgediff*cos(Latitude(28,iscan)*deg2rad)
          IF (LatEdgediff .LT. 1.0 .OR. LatEdgediff .GT. 5.0) THEN
            LatOK = .FALSE.
          ENDIF
          scandiff = iras1c_scnlin-scnlinold
          IF (LatOK .AND. iout.GT.0) THEN
            DO ispot=1,IRAS_SPOTS
              latdiff=Latitude(ispot,iscan)-1E-4*iras1c_latlon(1,ispot)
              londiff=Longitude(ispot,iscan)-1E-4*iras1c_latlon(2,ispot)
              IF (londiff .GT. 180.0) THEN
                londiff = londiff - 360.0
              ELSE IF (londiff .LT. -180) THEN
                londiff = londiff + 360.0
              ENDIF
              latlondiff = sqrt(latdiff**2 + 
     &              (londiff*cos(deg2rad*Latitude(ispot,iscan)))**2)
              IF (latlondiff .GT. 1.5*scandiff) THEN
                LatOK = .FALSE.
              ENDIF
            ENDDO
          ENDIF

          IF (LatOK) THEN

         !Continue with filling the 1c
            iras1c_geo_qc = geo_qc(iscan)
            iras1c_scnlin_qc = scnlin_qc(iscan)
            iras1c_cal_qc = cal_qc(iscan)
            DO ispot=1,IRAS_SPOTS
              iras1c_pixel_qc(ispot) = pixel_qc(ispot,iscan)
            ENDDO

            DO I=1,9
              iras1c_instrtemp(I) = Ave_Instr_Temp(I)*1E2
            ENDDO
            DO ispot=1,IRAS_SPOTS
              iras1c_dataqual(ispot) = 0
              iras1c_latlon(1,ispot) = 1E4*Latitude(ispot,iscan)
              iras1c_latlon(2,ispot) = 1E4*Longitude(ispot,iscan)
              iras1c_angles(1,ispot) = SensorZenith(ispot,iscan)
              iras1c_angles(2,ispot) = SensorAzimuth(ispot,iscan)
              iras1c_angles(3,ispot) = SolarZenith(ispot,iscan)
              iras1c_angles(4,ispot) = SolarAzimuth(ispot,iscan)
              iras1c_surface(1,ispot) = Height(ispot,iscan)
              IF (LandSeaMask(ispot,iscan).EQ.3) THEN
                iras1c_surface(2,ispot) = 0                 !Sea
              ELSE IF (LandSeaMask(ispot,iscan).EQ.1) THEN
                iras1c_surface(2,ispot) = 2                 !Land
              ELSE IF (LandSeaMask(ispot,iscan).EQ.5) THEN
                iras1c_surface(2,ispot) = 1                 !Mixed
              ELSE IF (LandSeaMask(ispot,iscan).EQ.2) THEN
                iras1c_surface(2,ispot) = 3                 !Inland water
              ELSE
                iras1c_surface(2,ispot) = MDI
              ENDIF              
            ENDDO
            jscan = (iscan-1)*IRAS_SPOTS

           !Dataqual bit 0 - set if all channels bad
           !Dataqual bit 1 - set if some channels bad

            DO ichan=1,IRAS_CHANS
              DO ispot=1,IRAS_SPOTS
                iras1c_dataqual(ispot) = 1  !Initialise to "all bad"
              ENDDO
            ENDDO
            DO ichan=1,IRAS_CHANS
              jchan = (ichan-1)*num_obs
              DO ispot=1,IRAS_SPOTS
                BT = Earth_Obs_BT(jchan+jscan+ispot)
                iras1c_btemps(ichan,ispot) = 100.0*BT + 0.5

                IF (ichan .LE. 20) THEN   !check thermal IR only
                 IF (BT.LT.HminBT .OR. BT.GT.HmaxBT) THEN
                  iras1c_dataqual(ispot) = IOR(iras1c_dataqual(ispot),2)
                 ELSE
                  iras1c_dataqual(ispot) =IAND(iras1c_dataqual(ispot),2)
                 ENDIF
                ENDIF
              ENDDO
            ENDDO
          
            xysq = EVS_orb_pos(1,iscan)**2 +     
     &             EVS_orb_pos(2,iscan)**2
            z = EVS_orb_pos(3,iscan)
            r_sat = sqrt(xysq + z**2)       !Centre of Earth to satellite (m)
            IF (r_sat .GT. semiMajor) THEN
              lat = atan2(z,sqrt(xysq))
              r_ellipse = sqrt((semiMajor*cos(lat))**2 + 
     &                         (semiMinor*sin(lat))**2)
              iras1c_scalti = NINT((r_sat - r_ellipse)*0.01)    !km *10
            ELSE
              iras1c_scalti = MDI
            ENDIF

            scnlinold = iras1c_scnlin
            iout = iout + 1
            CALL IOIRAS1C("WRITE DATA",unit,iout+1)
          ELSE
            IF (LatMissing) THEN
              write(*,*)'Missing latitude at iscan=',iscan
            ELSE
              write(*,*)'Bad latitude at iscan=',iscan
            ENDIF
          ENDIF
        ELSE
          write(*,*)'Bad time at iscan=',iscan,scantime
        ENDIF
      ENDDO
      
      WRITE(*,*)'Number of scan lines in input file: ', Count_scnlines
      WRITE(*,*)'Number of scan lines written to output file: ', iout

      iras1c_h_scnlin = iout
      iras1c_h_misscnlin = iras1c_scnlin - iras1c_h_scnlin
      CALL IOIRAS1C("WRITE HEADER",unit,1)

      CALL IOIRAS1C("CLOSE",unit,0)

      IF (iout .EQ. 0) CALL AAPPEXIT(1)

      RETURN
      END

