! Name: $Id: mwhs21c_read_f.F 463 2015-09-28 15:29:53Z frna $
!
! Purpose: Read MWHS2 l1c file and pass information back to C calling routine.
!          Subroutines MWHS21C_READ_HEADER and MWHS21C_READ_DATA
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2015, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    25/09/2015  N C Atkinson
!-----------------------------------------------------------------------

      SUBROUTINE MWHS21C_READ_HEADER (
     & filename, filename_length,
     & h_site, h_1bsite, h_versnb,
     & h_satid, h_instrument, h_satht, h_period,
     & h_startorbit, h_startdatayr, h_startdatady, h_startdatatime,
     & h_endorbit, h_enddatayr, h_enddatady, h_enddatatime,
     & h_scnlin, h_misscnlin, h_vnantennacorr, h_tempradcnv, h_wmosatid,
     & h_bandwidth, h_nchan, h_nmapchan, h_nmapchanused, h_beamversion,
     & h_beamwidth, h_cutoff, h_nxaverage, h_nyaverage )

      IMPLICIT NONE
      
! Parameters
      INTEGER   UNIT_1C
      PARAMETER (UNIT_1C=10)
      INTEGER*4 MWHS2_CHANS
      PARAMETER (MWHS2_CHANS=15)

! Arguments: 
      CHARACTER*(*) filename
      INTEGER       filename_length
      CHARACTER*3   h_site
      CHARACTER*3   h_1bsite
      INTEGER*4     h_versnb
      INTEGER*4     h_satid
      INTEGER*4     h_instrument
      INTEGER*4     h_satht
      INTEGER*4     h_period
      INTEGER*4     h_startorbit
      INTEGER*4     h_startdatayr
      INTEGER*4     h_startdatady
      INTEGER*4     h_startdatatime
      INTEGER*4     h_endorbit
      INTEGER*4     h_enddatayr
      INTEGER*4     h_enddatady
      INTEGER*4     h_enddatatime
      INTEGER*4     h_scnlin
      INTEGER*4     h_misscnlin
      INTEGER*4     h_vnantennacorr
      INTEGER*4     h_tempradcnv(3,MWHS2_CHANS)
      INTEGER*4     h_wmosatid
      INTEGER*4     h_bandwidth(MWHS2_CHANS)
      INTEGER*4     h_nchan
      INTEGER*4     h_nmapchan
      INTEGER*4     h_nmapchanused
      INTEGER*4     h_beamversion
      INTEGER*4     h_beamwidth(MWHS2_CHANS)
      INTEGER*4     h_cutoff(MWHS2_CHANS)
      INTEGER*4     h_nxaverage(MWHS2_CHANS)
      INTEGER*4     h_nyaverage(MWHS2_CHANS)

! Local:
      INTEGER errStatus, i, ichan

#include "inparms.h"
#include "mwhs21c.h"
#include "reclopt.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Open MWHS2 1c file ',filename(1:filename_length)
      OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &   STATUS="OLD", FORM="UNFORMATTED", ACCESS="DIRECT",
     &   RECL=mwhs2_1crecl/rldiv, IOSTAT=errStatus )
      IF (errStatus.NE.0) THEN
        write(*,*)'Cannot open file ',filename(1:filename_length)
        call aappexit(1)
      ENDIF

      write(*,*)'Read MWHS2 header'
      CALL IOMWHS21C("READ HEADER",UNIT_1C,1)

      h_scnlin = mwhs21c_h_scnlin
      h_site = mwhs21c_h_site
      h_1bsite = mwhs21c_h_1bsite
      h_versnb = mwhs21c_h_versnb
      h_satid = mwhs21c_h_satid
      h_instrument = mwhs21c_h_instrument
      h_satht = mwhs21c_h_satht
      h_period = mwhs21c_h_period
      h_startorbit = mwhs21c_h_startorbit
      h_startdatayr = mwhs21c_h_startdatayr
      h_startdatady = mwhs21c_h_startdatady
      h_startdatatime = mwhs21c_h_startdatatime
      h_endorbit = mwhs21c_h_endorbit
      h_enddatayr = mwhs21c_h_enddatayr
      h_enddatady = mwhs21c_h_enddatady
      h_enddatatime = mwhs21c_h_enddatatime
      h_scnlin = mwhs21c_h_scnlin
      h_misscnlin = mwhs21c_h_misscnlin
      h_vnantennacorr = mwhs21c_h_vnantennacorr
      do ichan=1,MWHS2_CHANS
        do i=1,3
          h_tempradcnv(i,ichan) = mwhs21c_h_tempradcnv(i,ichan)
        enddo
        h_bandwidth(ichan) = mwhs21c_h_bandwidth(ichan)
        h_beamwidth(ichan) = mwhs21c_h_beamwidth(ichan)
        h_cutoff(ichan) = mwhs21c_h_cutoff(ichan)
        h_nxaverage(ichan) = mwhs21c_h_nxaverage(ichan)
        h_nyaverage(ichan) = mwhs21c_h_nyaverage(ichan)
      enddo
      h_wmosatid = mwhs21c_h_wmosatid
      h_nchan = mwhs21c_h_nchan
      h_nmapchan = mwhs21c_h_nmapchan
      h_nmapchanused = mwhs21c_h_nmapchanused
      h_beamversion = mwhs21c_h_beamversion

      RETURN
      END
      
!-------------------------------------------------------------------------------

      SUBROUTINE MWHS21C_READ_DATA (
     &  h_scnlin,
     &  scnlin, scnlinyr, scnlindy, scnlintime,
     &  qualind, scanqual, chanqual, instrtemp1, instrtemp2,
     &  latitude, longitude, satzen, solzen, satazi, solazi, scalti,
     &  satposition, satvelocity, orbitangle,
     &  btemps, btmap, dataqual)
      IMPLICIT NONE

! Parameters
      INTEGER*4 MWHS2_CHANS
      PARAMETER (MWHS2_CHANS=15)
      INTEGER*4 MWHS2_MAPCHANS
      PARAMETER (MWHS2_MAPCHANS=13)
      INTEGER*4 MWHS2_SPOTS
      PARAMETER (MWHS2_SPOTS=98)
      INTEGER*4 UNIT_1C
      PARAMETER (UNIT_1C=10)

! Arguments: 
      INTEGER*4 h_scnlin
      INTEGER*4 scnlin(h_scnlin)
      INTEGER*4 scnlinyr(h_scnlin)
      INTEGER*4 scnlindy(h_scnlin)
      INTEGER*4 scnlintime(h_scnlin)
      INTEGER*4 qualind(h_scnlin)
      INTEGER*4 scanqual(h_scnlin)
      INTEGER*4 chanqual(MWHS2_CHANS,h_scnlin)
      INTEGER*4 instrtemp1(h_scnlin)
      INTEGER*4 instrtemp2(h_scnlin)
      INTEGER*4 latitude(MWHS2_SPOTS,h_scnlin)
      INTEGER*4 longitude(MWHS2_SPOTS,h_scnlin)
      INTEGER*4 satzen(MWHS2_SPOTS,h_scnlin)
      INTEGER*4 satazi(MWHS2_SPOTS,h_scnlin)
      INTEGER*4 solzen(MWHS2_SPOTS,h_scnlin)
      INTEGER*4 solazi(MWHS2_SPOTS,h_scnlin)
      INTEGER*4 scalti(h_scnlin)
      INTEGER*4 satposition(3,h_scnlin)
      INTEGER*4 satvelocity(3,h_scnlin)
      INTEGER*4 orbitangle(h_scnlin)
      INTEGER*4 btemps(MWHS2_CHANS,MWHS2_SPOTS,h_scnlin)
      INTEGER*4 btmap(MWHS2_MAPCHANS,MWHS2_SPOTS,h_scnlin)
      INTEGER*4 dataqual(MWHS2_SPOTS,h_scnlin)

! Local
      INTEGER iscan,ichan,ispot,i

#include "inparms.h"
#include "mwhs21c.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Read MWHS2 data records'
      DO iscan=1,h_scnlin
        CALL IOMWHS21C("READ DATA",UNIT_1C,iscan+1)
        scnlin(iscan) = mwhs21c_scnlin
        scnlinyr(iscan) = mwhs21c_scnlinyr
        scnlindy(iscan) = mwhs21c_scnlindy
        scnlintime(iscan) = mwhs21c_scnlintime
        qualind(iscan) = mwhs21c_qualind
        scanqual(iscan) = mwhs21c_scanqual
        instrtemp1(iscan) = mwhs21c_instrtemp1
        instrtemp2(iscan) = mwhs21c_instrtemp2
        scalti(iscan) = mwhs21c_scalti
        DO ichan=1,MWHS2_CHANS
          chanqual(ichan,iscan) = mwhs21c_chanqual(ichan)
        ENDDO
        DO ispot=1,MWHS2_SPOTS
          DO ichan=1,MWHS2_CHANS
            btemps(ichan,ispot,iscan) = mwhs21c_btemps(ichan,ispot)
          ENDDO
          DO ichan=1,MWHS2_MAPCHANS
            btmap(ichan,ispot,iscan) = mwhs21c_btmap(ichan,ispot)
          ENDDO
          latitude(ispot,iscan) = mwhs21c_latlon(1,ispot)
          longitude(ispot,iscan) = mwhs21c_latlon(2,ispot)
          satzen(ispot,iscan) = mwhs21c_angles(1,ispot)
          satazi(ispot,iscan) = mwhs21c_angles(2,ispot)
          solzen(ispot,iscan) = mwhs21c_angles(3,ispot)
          solazi(ispot,iscan) = mwhs21c_angles(4,ispot)
          dataqual(ispot,iscan) = mwhs21c_dataqual(ispot)
        ENDDO
        DO i=1,3
          satposition(i,iscan) = mwhs21c_satposition(i)
          satvelocity(i,iscan) = mwhs21c_satvelocity(i)
        ENDDO
        orbitangle(iscan) = mwhs21c_orbitangle
      ENDDO

      CALL IOMWHS21C("CLOSE",UNIT_1C,0)

      RETURN
      END
      
