! Name: $Id: mwhs2_sdr_out.F 592 2018-08-09 14:36:40Z frna $
!
! Purpose: Fill MWHS 1c structure and write out file
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    20/05/2014  Create from mwts_sdr_out.F
!  1.1    25/09/2015  Correction: mwhs21c_h_instrument should be 19
!  1.2    27/04/2018  Add inland water category in surface type
!  1.3    30/07/2018  Add FY-3D
!
!-------------------------------------------------------------------------------

      SUBROUTINE MWHS2_SDR_OUT (
     &  Count_scnlines, Orbit_Number, Orbit_Period,
     &  Earth_Obs_BT, Pixel_View_Angle, Latitude, Longitude, 
     &  Scnlin_daycnt, Scnlin_mscnt,
     &  SensorZenith, SensorAzimuth, SolarZenith, SolarAzimuth, DEM,
     &  LandSeaMask, QA_Ch_Flag, QA_Scan_Flag, scnlin_qc, 
     &  Beginning_Date, Beginning_Time, Ending_Date, Ending_Time,
     &  Satellite_Name)
     
      IMPLICIT NONE

      INTEGER   MWHS2_SPOTS, MWHS2_CHANS, MWHS2_SCANS
      PARAMETER (MWHS2_SPOTS=98)
      PARAMETER (MWHS2_CHANS=15)
      PARAMETER (MWHS2_SCANS=2700)

! Arguments: 
      INTEGER*4    Count_scnlines
      INTEGER*4    Orbit_Number
      INTEGER*4    Orbit_Period
      REAL         Latitude(MWHS2_SPOTS,MWHS2_SCANS)
      REAL         Longitude(MWHS2_SPOTS,MWHS2_SCANS)
      INTEGER*2    Pixel_View_Angle(2,MWHS2_SCANS)
      INTEGER*2    Scnlin_daycnt(MWHS2_SCANS)
      INTEGER*4    Scnlin_mscnt(MWHS2_SCANS)
      REAL         Earth_Obs_BT(MWHS2_SPOTS*MWHS2_SCANS*MWHS2_CHANS)
      INTEGER*2    SensorZenith(MWHS2_SPOTS,MWHS2_SCANS)
      INTEGER*2    SensorAzimuth(MWHS2_SPOTS,MWHS2_SCANS)
      INTEGER*2    SolarZenith(MWHS2_SPOTS,MWHS2_SCANS)
      INTEGER*2    SolarAzimuth(MWHS2_SPOTS,MWHS2_SCANS)
      INTEGER*2    DEM(MWHS2_SPOTS,MWHS2_SCANS)
      INTEGER*2    LandSeaMask(MWHS2_SPOTS,MWHS2_SCANS)
      INTEGER*2    QA_Ch_Flag(MWHS2_SCANS)
      INTEGER*2    QA_Scan_Flag(MWHS2_SCANS)
      INTEGER*4    scnlin_qc(MWHS2_SCANS)
      CHARACTER*11 Beginning_Date, Ending_Date
      CHARACTER*13 Beginning_Time, Ending_Time
      CHARACTER*6  Satellite_Name

! Local variables

      INTEGER iscan,ispot,ichan,unit,jscan,jchan,iout,id,scan1,scan2,i
      INTEGER num_obs
      INTEGER year, month, day, hour, minute
      INTEGER scanline, scnlinold
      INTEGER firsttime, tdiff, scandiff, scnlintime_old
      INTEGER nbad, nbad_threshold
      DATA    nbad_threshold/30/   !cal assumed bad if nbad>threshold
      INTEGER BTmin(MWHS2_CHANS), BTmax(MWHS2_CHANS)
      REAL    latdiff, londiff, latlondiff, LatEdgediff
      REAL    lat(2), lon(2), x(2), y(2), z(2)
      REAL    xpos(MWHS2_SCANS),ypos(MWHS2_SCANS),zpos(MWHS2_SCANS)
      REAL    second
      REAL    BT
      REAL    scanangle, centreangle, x1, x2, y1, y2, z1, z2, height_km
      REAL    rad, dt, earth_rotation
      REAL    pos_vector(3),vel_vector(3),angle,xpos2mod,ypos2mod
      REAL    RMDI
      DATA    RMDI/-9999.99/
      LOGICAL LatOK, LatMissing
      INTEGER qualind(MWHS2_SCANS)
      DATA qualind / MWHS2_SCANS*0 /
      INTEGER qualval(MWHS2_CHANS+1)
      DATA qualval/2,4,8,16,32,64,128,256,512,1024,2048,4096,8192,
     &  16384,32768,65534/  !Last is the sum
      CHARACTER*80 line
      REAL    channel_info(7,MWHS2_CHANS)
      DATA    channel_info /105*0/
      LOGICAL badcal(MWHS2_SCANS), badspot

!Function
      INTEGER    IDAYOFYEAR

! Include
#include "mwhs21c.h"
#include "inparms.h"
#include "constants_preproc.h"

! End of declarations
!-----------------------------------------------------------------------

      unit = 10
      CALL IOMWHS21C("OPEN/WRITE",unit,0)
      
      IF (Satellite_Name(1:5) .EQ. "FY-3A") THEN
        mwhs21c_h_satid = 1
        mwhs21c_h_wmosatid = 520
      ELSE IF (Satellite_Name(1:5) .EQ. "FY-3B") THEN
        mwhs21c_h_satid = 2
        mwhs21c_h_wmosatid = 521
      ELSE IF (Satellite_Name(1:5) .EQ. "FY-3C") THEN
        mwhs21c_h_satid = 3
        mwhs21c_h_wmosatid = 522
      ELSE IF (Satellite_Name(1:5) .EQ. "FY-3D") THEN
        mwhs21c_h_satid = 4
        mwhs21c_h_wmosatid = 523
      ELSE
        mwhs21c_h_satid = MDI
        mwhs21c_h_wmosatid = MDI
      ENDIF

!Read the MWHS channel info on unit 21
      OPEN (21,form='formatted',status='old')
      id = 0
      DO while (id .NE. mwhs21c_h_wmosatid)
        line="     "
        DO while (line(1:5) .ne. "# WMO")
          read(21,'(A)',END=100) line
        ENDDO
        read(21,*) id
        IF (id .eq. mwhs21c_h_wmosatid) THEN
          read(21,*) line
          read(21,*) channel_info
        ENDIF
      ENDDO
  100 CLOSE (21)
      DO ichan=1,MWHS2_CHANS
        mwhs21c_h_tempradcnv(1,ichan) = 
     &    channel_info(2,ichan)*1.0E13/C_light   !wavenumber*1E6
        mwhs21c_h_tempradcnv(2,ichan) = channel_info(4,ichan)*1.0E6  !offset
        mwhs21c_h_tempradcnv(3,ichan) = channel_info(5,ichan)*1.0E6  !slope
        mwhs21c_h_bandwidth(ichan) = channel_info(3,ichan)*1.0E6     !GHz*1E6
        BTmin(ichan) = channel_info(6,ichan)
        BTmax(ichan) = channel_info(7,ichan)
      ENDDO

      READ(Ending_Date,'(I4,1X,I2,1X,I2)') year, month, day
      READ(Ending_Time,'(I2,1X,I2,1X,F6.3)') hour, minute, second
      mwhs21c_h_enddatayr = year
      mwhs21c_h_enddatady = IDAYOFYEAR(year, month, day)
      mwhs21c_h_enddatatime = 1000*(hour*3600 + minute*60 + second)
      
      READ(Beginning_Date,'(I4,1X,I2,1X,I2)') year, month, day
      READ(Beginning_Time,'(I2,1X,I2,1X,F6.3)') hour, minute, second
      mwhs21c_h_startdatayr = year
      mwhs21c_h_startdatady = IDAYOFYEAR(year, month, day)
      mwhs21c_h_startdatatime = 1000*(hour*3600 + minute*60 + second)
      
      mwhs21c_h_instrument = 19
      mwhs21c_h_startorbit = Orbit_Number
      mwhs21c_h_endorbit = Orbit_Number
      mwhs21c_h_hdrcnt = 1
      mwhs21c_h_versnb = MWHS2_1cvnnb
      mwhs21c_h_versyr = MWHS2_1cvnyr
      mwhs21c_h_versdy = MWHS2_1cvndy
      mwhs21c_h_satht = 8360      !nominal
      mwhs21c_h_period = Orbit_Period*60
      mwhs21c_h_nchan = mwhs2_nchan
      mwhs21c_h_nmapchan = mwhs2_nmapchan
      mwhs21c_h_nmapchanused = 0          !No mapping in this program
      mwhs21c_h_beamversion = 0
      DO ichan=1,MWHS2_CHANS
        mwhs21c_h_beamwidth(ichan) = 0
        mwhs21c_h_cutoff(ichan) = 0
        mwhs21c_h_nxaverage(ichan) = 0    !No averaging in this program
        mwhs21c_h_nyaverage(ichan) = 0
      ENDDO
      
      CALL IOMWHS21C("WRITE HEADER",unit,1)

      num_obs = Count_scnlines*MWHS2_SPOTS
      iout = 0
      mwhs21c_scnlin = 0

!Estimate satellite position in Greenwich co-ords - not present in the hdf5
!Could get a more accurate value from the satpos file, but this will do for
!now. Ignore elipticity of earth. Use nominal satellite height.

      rad = rEarth*1000 + mwhs21c_h_satht*100   !distance from centre of earth
      DO iscan=1,Count_scnlines
        lat(1) = Latitude(45,iscan)
        lat(2) = Latitude(46,iscan)
        lon(1) = Longitude(45,iscan)
        lon(2) = Longitude(46,iscan)
        IF (lat(1) .LT. 32767) THEN
          call latlon_to_xyz(2,1,lat,lon,x,y,z)
          xpos(iscan) = rad*0.5*(x(1) + x(2))
          ypos(iscan) = rad*0.5*(y(1) + y(2))
          zpos(iscan) = rad*0.5*(z(1) + z(2))
        ELSE
          xpos(iscan) = RMDI
          ypos(iscan) = RMDI
          zpos(iscan) = RMDI
        ENDIF
      ENDDO

!Quality control on the brightness temperatures
!If any channel has BT outside the expected range then set them all to missing.
!If number of bad spots exceeds a threshold then it's probably a calibration
!error, so set missing for !+-3 scan lines.

      DO iscan=1,Count_scnlines
        badcal(iscan) = .false.
        nbad = 0
        jscan = (iscan-1)*MWHS2_SPOTS
        DO ispot=1,MWHS2_SPOTS
          badspot = .false.
          DO ichan=1,MWHS2_CHANS
            jchan = (ichan-1)*num_obs
            IF (Earth_Obs_BT(jchan+jscan+ispot).LT.BTmin(ichan)
     &       .OR.Earth_Obs_BT(jchan+jscan+ispot).GT.BTmax(ichan)) THEN
              badspot = .true.
            ENDIF
          ENDDO
          IF (badspot) THEN
            nbad = nbad + 1
            if (nbad .GT. nbad_threshold) badcal(iscan) = .true.
            DO ichan=1,MWHS2_CHANS
              jchan = (ichan-1)*num_obs
              Earth_Obs_BT(jchan+jscan+ispot) = -327.68
            ENDDO
          ENDIF
        ENDDO
      ENDDO
      DO iscan=1,Count_scnlines
        jscan = (iscan-1)*MWHS2_SPOTS
        DO i=-3,3
          IF (iscan+i.GT.0.AND.iscan+i.LT.Count_scnlines) THEN
            IF (badcal(iscan+i)) THEN
              DO ispot=1,MWHS2_SPOTS
                DO ichan=1,MWHS2_CHANS
                  jchan = (ichan-1)*num_obs
                  Earth_Obs_BT(jchan+jscan+ispot) = -327.68
                ENDDO
              ENDDO
            ENDIF
          ENDIF
        ENDDO
      ENDDO
          
!Main loop

      DO iscan=1,Count_scnlines
        mwhs21c_qualind = QA_Ch_Flag(iscan)
        mwhs21c_scanqual = QA_Scan_Flag(iscan)
        scnlintime_old = mwhs21c_scnlintime
        mwhs21c_scnlintime = Scnlin_mscnt(iscan) + 43200000   !12 hour offset
        IF (mwhs21c_scnlintime .GE. 86400000)
     &    mwhs21c_scnlintime = mwhs21c_scnlintime - 86400000

        IF (mwhs21c_scnlintime - mwhs21c_h_startdatatime .LT. 
     &             -43200000) THEN
            mwhs21c_scnlindy = mwhs21c_h_enddatady
            mwhs21c_scnlinyr = mwhs21c_h_enddatayr
          ELSE
            mwhs21c_scnlindy = mwhs21c_h_startdatady
            mwhs21c_scnlinyr = mwhs21c_h_startdatayr
          ENDIF
        
        IF (mwhs21c_scnlintime .GE. 0) THEN

          IF (mwhs21c_scnlin .EQ. 0) THEN
            firsttime = mwhs21c_scnlintime
          ENDIF
          
          tdiff = mwhs21c_scnlintime - firsttime
          IF (tdiff .LT. -43200000) THEN 
            tdiff = tdiff + 86400000
          ENDIF
          mwhs21c_scnlin = NINT(tdiff/2666.7) + 1

          !Check that the latitudes are sensible: latitude difference across the
          !scan and also the difference from one scan to the next. 
          LatOK = .TRUE.
          LatMissing = .FALSE.
          DO ispot=1,MWHS2_SPOTS
            IF (ABS(Latitude(ispot,iscan)).GT.90.0) THEN
              LatOK = .FALSE.
              LatMissing = .TRUE.
            ENDIF
          ENDDO
          !Check latitude difference across the scan
          LatEdgediff = Latitude(1,iscan)-Latitude(90,iscan)
          LatEdgediff = LatEdgediff*cos(Latitude(45,iscan)*deg2rad)
          IF (LatEdgediff .GT. 5.0 .OR. LatEdgediff .LT. 1.0) THEN
            LatOK = .FALSE.
          ENDIF
          scandiff = mwhs21c_scnlin-scnlinold
          IF (LatOK .AND. iout.GT.0) THEN
            DO ispot=1,MWHS2_SPOTS
              latdiff =
     &           Latitude(ispot,iscan) - 1E-4*mwhs21c_latlon(1,ispot)
              londiff =
     &           Longitude(ispot,iscan) - 1E-4*mwhs21c_latlon(2,ispot)
              IF (londiff .GT. 180.0) THEN
                londiff = londiff - 360.0
              ELSE IF (londiff .LT. -180) THEN
                londiff = londiff + 360.0
              ENDIF
              latlondiff = sqrt(latdiff**2 + 
     &              (londiff*cos(deg2rad*Latitude(ispot,iscan)))**2)
              IF (latlondiff .GT. 0.5*scandiff) THEN
                LatOK = .FALSE.
              ENDIF
            ENDDO
          ENDIF
          IF (LatOK) THEN

            mwhs21c_instrtemp1 = MDI
            mwhs21c_instrtemp2 = MDI
            DO ispot=1,MWHS2_SPOTS
              mwhs21c_dataqual(ispot) = 0
              mwhs21c_latlon(1,ispot) = 1E4*Latitude(ispot,iscan)
              mwhs21c_latlon(2,ispot) = 1E4*Longitude(ispot,iscan)
              mwhs21c_angles(1,ispot) = SensorZenith(ispot,iscan)
              mwhs21c_angles(2,ispot) = SensorAzimuth(ispot,iscan)
              mwhs21c_angles(3,ispot) = SolarZenith(ispot,iscan)
              mwhs21c_angles(4,ispot) = SolarAzimuth(ispot,iscan)
              mwhs21c_surface(1,ispot) = DEM(ispot,iscan)
              IF (LandSeaMask(ispot,iscan).EQ.3) THEN
                mwhs21c_surface(2,ispot) = 0                 !Sea
              ELSE IF (LandSeaMask(ispot,iscan).EQ.1) THEN
                mwhs21c_surface(2,ispot) = 2                 !Land
              ELSE IF (LandSeaMask(ispot,iscan).EQ.5) THEN
                mwhs21c_surface(2,ispot) = 1                 !Mixed
              ELSE IF (LandSeaMask(ispot,iscan).EQ.2) THEN
                mwhs21c_surface(2,ispot) = 3                 !Inland water
              ELSE
                mwhs21c_surface(2,ispot) = MDI
              ENDIF              
            ENDDO
            jscan = (iscan-1)*MWHS2_SPOTS
            DO ichan=1,MWHS2_CHANS
              jchan = (ichan-1)*num_obs
              DO ispot=1,MWHS2_SPOTS
                BT = Earth_Obs_BT(jchan+jscan+ispot)
                IF (BT .GT. 0) THEN
                  mwhs21c_btemps(ichan,ispot) = BT*100
                ELSE
                  mwhs21c_btemps(ichan,ispot) = MDI
                ENDIF
                IF (BT.LT.AminBT .OR. BT.GT.AmaxBT .OR.
     &               mwhs21c_qualind .LT. 0) THEN
                  mwhs21c_dataqual(ispot) = mwhs21c_dataqual(ispot) + 
     &               qualval(ichan)
                  IF (ichan.EQ.MWHS2_CHANS .AND. 
     &               mwhs21c_dataqual(ispot).EQ.qualval(ichan+1)) THEN
                    mwhs21c_dataqual(ispot) = mwhs21c_dataqual(ispot)+1  !All channels bad
                  ENDIF
                ENDIF
              ENDDO
            ENDDO
          
!Estimate the satellite height.
!Original assumption was that Pixel_View_Angle is from start of pixel 1 integration
!to end of pixel 98 (as seems to be the case for MWTS-2), but this gives too large a height.
!Should be 836.4km mean height. Fine-tune the factor.

!            scanangle = 0.5*deg2rad*0.01*(Pixel_View_Angle(2,iscan) - 
!     &                            Pixel_View_Angle(1,iscan))*97.0/98.0
            scanangle = 0.5*deg2rad*0.01*(Pixel_View_Angle(2,iscan) - 
     &                            Pixel_View_Angle(1,iscan))*97.85/98.0
            IF (scanangle .gt. 0) THEN
              x1 = cos(Latitude(1,iscan)*deg2rad) * 
     &             cos(Longitude(1,iscan)*deg2rad)
              y1 = cos(Latitude(1,iscan)*deg2rad) * 
     &             sin(Longitude(1,iscan)*deg2rad)
              z1 = sin(Latitude(1,iscan)*deg2rad)

              x2 = cos(Latitude(98,iscan)*deg2rad) * 
     &             cos(Longitude(98,iscan)*deg2rad)
              y2 = cos(Latitude(98,iscan)*deg2rad) * 
     &             sin(Longitude(98,iscan)*deg2rad)
              z2 = sin(Latitude(98,iscan)*deg2rad)

              centreangle = 0.5*(acos(x1*x2 + y1*y2 + z1*z2))
            
              height_km = 
     &           rEarth*(sin(scanangle+centreangle)/sin(scanangle)-1.0)

              mwhs21c_scalti = 10.0*height_km + 0.5
              
            ELSE
              mwhs21c_scalti = MDI
            ENDIF

!Satellite position and velocity 
            
            IF (xpos(iscan) .NE. RMDI) THEN
              mwhs21c_satposition(1) = xpos(iscan)
              mwhs21c_satposition(2) = ypos(iscan)
              mwhs21c_satposition(3) = zpos(iscan)
            ELSE
              mwhs21c_satposition(1) = MDI
              mwhs21c_satposition(2) = MDI
              mwhs21c_satposition(3) = MDI
            ENDIF

            IF (iscan .EQ. 1) THEN
              scan1 = iscan
              scan2 = iscan + 1
            ELSE IF (iscan .EQ. Count_scnlines) THEN
              scan1 = iscan - 1
              scan2 = iscan
            ELSE
              scan1 = iscan - 1
              scan2 = iscan + 1
              IF (xpos(iscan-1).EQ.RMDI) scan1 = iscan
              IF (xpos(iscan+1).EQ.RMDI) scan2 = iscan
            ENDIF

            IF (scan1 .NE. scan2 .AND. xpos(scan1).NE.RMDI .AND. 
     &          xpos(scan2).NE.RMDI) THEN
              dt = Scnlin_mscnt(scan2)*0.001 - Scnlin_mscnt(scan1)*0.001
              IF (dt .LT. 0) dt = dt + 86400.0

              earth_rotation = dt/86400.0 * 2 * PI   !angle is small
              xpos2mod = xpos(scan2) - ypos(scan2)*earth_rotation
              ypos2mod = ypos(scan2) + xpos(scan2)*earth_rotation

              mwhs21c_satvelocity(1) = 
     &          (xpos2mod - xpos(scan1))/dt
              mwhs21c_satvelocity(2) = 
     &          (ypos2mod - ypos(scan1))/dt
              mwhs21c_satvelocity(3) = 
     &          (zpos(scan2) - zpos(scan1))/dt

              pos_vector(1) = mwhs21c_satposition(1)
              pos_vector(2) = mwhs21c_satposition(2)
              pos_vector(3) = mwhs21c_satposition(3)
              vel_vector(1) = mwhs21c_satvelocity(1)
              vel_vector(2) = mwhs21c_satvelocity(2)
              vel_vector(3) = mwhs21c_satvelocity(3)

              CALL orbit_angle (1,mwhs21c_scnlinyr,mwhs21c_scnlindy,0,
     &          mwhs21c_scnlintime,pos_vector,vel_vector,angle)
              mwhs21c_orbitangle = angle*100
            ELSE
              mwhs21c_orbitangle = MDI
            ENDIF

            scnlinold = mwhs21c_scnlin
            iout = iout + 1
            CALL IOMWHS21C("WRITE DATA",unit,iout+1)
          ELSE
            IF (LatMissing) THEN
              write(*,*)'Missing latitude at iscan=',iscan
            ELSE
              write(*,*)'Bad latitude at iscan=',iscan
            ENDIF
          ENDIF
        ELSE
          write(*,*)'Bad time at iscan=',iscan,mwhs21c_scnlintime
        ENDIF
      ENDDO
      
      WRITE(*,*)'Number of scan lines in input file: ', Count_scnlines
      WRITE(*,*)'Number of scan lines written to output file: ', iout
      mwhs21c_h_scnlin = iout
      mwhs21c_h_misscnlin = mwhs21c_scnlin - mwhs21c_h_scnlin
      CALL IOMWHS21C("WRITE HEADER",unit,1)

      CALL IOMWHS21C("CLOSE",unit,0)

      IF (iout .EQ. 0) CALL AAPPEXIT(1)

      RETURN
      END

