! Name: $Id: mwhs_sdr_out.F 586 2018-04-30 10:32:45Z frna $
!
! Purpose: Fill MWHS 1c structure and write out file
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    30/11/2010  Original code. (N.C.Atkinson)
!  1.1    03/10/2012  Add Height and LandSeaMask (as for mwts)
!  1.2    15/10/2012  Fix channel info read. 
!  1.3    27/04/2018  Add inland water category in surface type
!
!-------------------------------------------------------------------------------

      SUBROUTINE MWHS_SDR_OUT (
     &  Count_scnlines, Orbit_Number, Orbit_Period,
     &  Earth_Obs_BT, Latitude, Longitude, Scan_Time,
     &  Mixer_Temp, GPS_Position_Data, GPS_Position_Time,
     &  Package_Counter,
     &  SensorZenith, SensorAzimuth, SolarZenith, SolarAzimuth, Height,
     &  LandSeaMask,Black_Body_View_Ang, Space_View_Ang, Satellite_Name,
     &  Beginning_Date, Beginning_Time, Ending_Date, Ending_Time)
     
      IMPLICIT NONE

      INTEGER   MWHS_SPOTS, MWHS_CHANS, MWHS_SCANS
      PARAMETER (MWHS_SPOTS=98)
      PARAMETER (MWHS_CHANS=5)
      PARAMETER (MWHS_SCANS=2700)

! Arguments: 
      INTEGER*4    Count_scnlines
      INTEGER*4    Orbit_Number
      INTEGER*4    Orbit_Period
      REAL         Earth_Obs_BT(MWHS_CHANS*MWHS_SCANS*MWHS_SPOTS)
      REAL         Latitude(MWHS_SPOTS,MWHS_SCANS)
      REAL         Longitude(MWHS_SPOTS,MWHS_SCANS)
      REAL         Mixer_Temp(MWHS_SCANS)
      INTEGER      GPS_Position_Data(3,MWHS_SCANS)
      INTEGER      GPS_Position_Time(MWHS_SCANS)
      INTEGER      Scan_Time(MWHS_SCANS)
      INTEGER*2    Package_Counter(MWHS_SCANS)
      INTEGER*2    SensorZenith(MWHS_SPOTS,MWHS_SCANS)
      INTEGER*2    SensorAzimuth(MWHS_SPOTS,MWHS_SCANS)
      INTEGER*2    SolarZenith(MWHS_SPOTS,MWHS_SCANS)
      INTEGER*2    SolarAzimuth(MWHS_SPOTS,MWHS_SCANS)
      INTEGER*2    Height(MWHS_SPOTS,MWHS_SCANS)
      INTEGER*2    LandSeaMask(MWHS_SPOTS,MWHS_SCANS)
      INTEGER*2    Black_Body_View_Ang(3,MWHS_SCANS)
      INTEGER*2    Space_View_Ang(3,MWHS_SCANS)
      CHARACTER*6  Satellite_Name
      CHARACTER*11 Beginning_Date, Ending_Date
      CHARACTER*13 Beginning_Time, Ending_Time

! Local variables

      INTEGER iscan,ispot,ichan,unit,jscan,jchan,iout,id
      INTEGER num_obs
      INTEGER year, month, day, hour, minute
      INTEGER scanline, scnlinold
      INTEGER timeoffset
      INTEGER scantime, firsttime, tdiff, scandiff
      REAL    latdiff, londiff, latlondiff
      REAL    second
      REAL    BT
      LOGICAL LatOK, LatMissing
      LOGICAL BBOK, SPOK
      INTEGER qualind(MWHS_SCANS)
      DATA qualind / MWHS_SCANS*0 /
      INTEGER qualval(MWHS_CHANS+1)
      DATA qualval/2,4,8,16,32,62/  !Last is the sum
      REAL    xysq,z,lat,semiMinor,r_sat,r_ellipse
      INTEGER*2 Package_Counter_first
      CHARACTER*80 line
      REAL    channel_info(5,MWHS_CHANS)
      DATA    channel_info /25*0/
      
      !Variables for moon calculation
      REAL    midlat(2)         !Latitude of samples 48 and 51
      REAL    midlon(2)         !Longitude of samples 48 and 51
      REAL    azcorr            !Azimuth correction (radians)
      REAL    thour             !Time of day in hours
      REAL*8  tdays             !Time of scan in days from 12Z, 1 Jan 2000
      REAL    ra                !right ascension of moon (radians)
      REAL    dec               !moon declination
      REAL    sindec            !sine of moon declination
      REAL    cosdec            !cosine of moon declination
      REAL    amsual            !MWTS space view altitude
      REAL    sinamsual         !sine of MWTS space view altitude
      REAL    cosamsual         !cosine of MWTS space view altitude
      REAL    angle_diff        !Angle between moon and sp view (deg)
      REAL    angle_diff_min, angle_diff_max
      DATA    angle_diff_min/9999.9/
      DATA    angle_diff_max/0.0/
      INTEGER num_moon_reject
      DATA    num_moon_reject/0/

!Function
      INTEGER    IDAYOFYEAR

! Include
#include "mwhs1c.h"
#include "inparms.h"
#include "constants_preproc.h"

! End of declarations
!-----------------------------------------------------------------------

      unit = 11
      CALL IOMWHS1C("OPEN/WRITE",unit,0)
      
      semiMinor = semiMajor - semiDiff

      IF (Satellite_Name(1:5) .EQ. "FY-3A") THEN
        mwhs1c_h_satid = 1
        mwhs1c_h_wmosatid = 520
      ELSE IF (Satellite_Name(1:5) .EQ. "FY-3B") THEN
        mwhs1c_h_satid = 2
        mwhs1c_h_wmosatid = 521
      ELSE
        mwhs1c_h_satid = MDI
        mwhs1c_h_wmosatid = MDI
      ENDIF

!Read the MWHS channel info on unit 21
      OPEN (21,form='formatted',status='old')
      id = 0
      DO while (id .NE. mwhs1c_h_wmosatid)
        line="     "
        DO while (line(1:5) .ne. "# WMO")
          read(21,'(A)',END=100) line
        ENDDO
        read(21,*) id
        IF (id .eq. mwhs1c_h_wmosatid) THEN
          read(21,*) line
          read(21,*) channel_info
        ENDIF
      ENDDO
  100 CLOSE (21)
      DO ichan=1,MWHS_CHANS
        mwhs1c_h_tempradcnv(1,ichan) = 
     &    channel_info(2,ichan)*1.0E13/C_light   !wavenumber*1E6
        mwhs1c_h_tempradcnv(2,ichan) = channel_info(4,ichan)*1.0E6  !offset
        mwhs1c_h_tempradcnv(3,ichan) = channel_info(5,ichan)*1.0E6  !slope
        mwhs1c_h_bandwidth(ichan) = channel_info(3,ichan)*1.0E6     !GHz*1E6
      ENDDO

      READ(Ending_Date,'(I4,1X,I2,1X,I2)') year, month, day
      READ(Ending_Time,'(I2,1X,I2,1X,F6.3)') hour, minute, second
      mwhs1c_h_enddatayr = year
      mwhs1c_h_enddatady = IDAYOFYEAR(year, month, day)
      mwhs1c_h_enddatatime = 1000*(hour*3600 + minute*60 + second)
      
      READ(Beginning_Date,'(I4,1X,I2,1X,I2)') year, month, day
      READ(Beginning_Time,'(I2,1X,I2,1X,F6.3)') hour, minute, second
      mwhs1c_h_startdatayr = year
      mwhs1c_h_startdatady = IDAYOFYEAR(year, month, day)
      mwhs1c_h_startdatatime = 1000*(hour*3600 + minute*60 + second)
      
      mwhs1c_h_instrument = 16
      mwhs1c_h_startorbit = Orbit_Number
      mwhs1c_h_endorbit = Orbit_Number
      mwhs1c_h_hdrcnt = 1
      mwhs1c_h_versnb = mwhs_1cvnnb
      mwhs1c_h_versyr = mwhs_1cvnyr
      mwhs1c_h_versdy = mwhs_1cvndy
      mwhs1c_h_satht = 8350      !nominal
      mwhs1c_h_period = Orbit_Period*60
      
      CALL IOMWHS1C("WRITE HEADER",unit,1)

      num_obs = Count_scnlines*MWHS_SPOTS
      iout = 0
      mwhs1c_scnlin = 0

!Check the black body angles. Expect +-100 approx. If outside this range,
!set the "do not use" flag for this and adjacent scans
          
      DO iscan=1,Count_scnlines
        BBOK = .true.
        SPOK = .true.
        DO ispot=1,3
          IF (ABS(Black_Body_View_Ang(ispot,iscan)).GT.100) THEN
            BBOK = .false.
          ENDIF
          IF (ABS(Space_View_Ang(ispot,iscan)-19500).GT.100) THEN
            SPOK = .false.
          ENDIF
        ENDDO
        IF (.not. BBOK) WRITE(*,*)'Bad BB view at iscan=',iscan
        IF (.not. SPOK) WRITE(*,*)'Bad Sp view at iscan=',iscan
        IF (.not. BBOK .OR. .not. SPOK) THEN
          DO jscan=iscan-2,iscan+3
            qualind(jscan) = -2147483648     !set bit 31
          ENDDO
        ENDIF
      ENDDO

!Main loop

      DO iscan=1,Count_scnlines
        mwhs1c_qualind = qualind(iscan)
        scantime = Scan_Time(iscan)
        IF (scantime .GT. 0) THEN

         IF (mwhs1c_scnlin .EQ. 0) THEN      !First time round
C           mwhs1c_scnlintime = mwhs1c_h_startdatatime
C           timeoffset = scantime - mwhs1c_scnlintime
           firsttime = scantime
           timeoffset = GPS_Position_Time(iscan) - scantime
           WRITE(*,*)'GPS minus raw time offset (ms) = ',timeoffset
           Package_Counter_first = Package_Counter(iscan)
         ENDIF

         IF (abs(timeoffset) .LT. 10000) THEN      !Expect within 10 sec
          
          tdiff = scantime - firsttime
          IF (tdiff .LT. -43200000) THEN 
            tdiff = tdiff + 86400000
          ENDIF
          mwhs1c_scnlin = NINT(tdiff/2666.6667) + 1
C          mwhs1c_scnlin = Package_Counter(iscan) - 
C     &            Package_Counter_first + 1

          mwhs1c_scnlintime = scantime + timeoffset + 43200000   !12 hour offset
          IF (mwhs1c_scnlintime .GE. 86400000)
     &       mwhs1c_scnlintime = mwhs1c_scnlintime - 86400000
          IF (mwhs1c_scnlintime - mwhs1c_h_startdatatime .LT. 
     &             -43200000) THEN
            mwhs1c_scnlindy = mwhs1c_h_enddatady
            mwhs1c_scnlinyr = mwhs1c_h_enddatayr
          ELSE
            mwhs1c_scnlindy = mwhs1c_h_startdatady
            mwhs1c_scnlinyr = mwhs1c_h_startdatayr
          ENDIF
C          mwhs1c_scnlintime = mwhs1c_h_startdatatime + tdiff
C          IF (mwhs1c_scnlintime .GT. 86400000) THEN
C            mwhs1c_scnlintime = mwhs1c_scnlintime - 86400000
C            mwhs1c_scnlindy = mwhs1c_h_enddatady
C            mwhs1c_scnlinyr = mwhs1c_h_enddatayr
C          ENDIF
          
          
          !Check that the latitudes are sensible. Expect max diff of 0.16 deg 
          !from one scan line to the next
          LatOK = .TRUE.
          LatMissing = .FALSE.
          DO ispot=1,MWHS_SPOTS
            IF (ABS(Latitude(ispot,iscan)).GT.90.0) THEN
              LatOK = .FALSE.
              LatMissing = .TRUE.
            ENDIF
          ENDDO
          scandiff = mwhs1c_scnlin-scnlinold
C          IF (LatOK .AND. iout.GT.0 .AND. scandiff .LT. 5) THEN
          IF (LatOK .AND. iout.GT.0) THEN
            DO ispot=1,MWHS_SPOTS
              latdiff=Latitude(ispot,iscan)-1E-4*mwhs1c_latlon(1,ispot)
              londiff=Longitude(ispot,iscan)-1E-4*mwhs1c_latlon(2,ispot)
              IF (londiff .GT. 180.0) THEN
                londiff = londiff - 360.0
              ELSE IF (londiff .LT. -180) THEN
                londiff = londiff + 360.0
              ENDIF
              latlondiff = sqrt(latdiff**2 + 
     &              (londiff*cos(deg2rad*Latitude(ispot,iscan)))**2)
              IF (latlondiff .GT. 0.3*scandiff) THEN
                LatOK = .FALSE.
              ENDIF
            ENDDO
          ENDIF
          IF (LatOK) THEN

         !Calculate time in days since 12h 1 Jan 2000 (t=0)
            tdays = INT((mwhs1c_scnlinyr-2001)*365.25D0)
     &        + mwhs1c_scnlindy + 364.5D0
     &        + mwhs1c_scnlintime/86400000.0D0
            thour = mwhs1c_scnlintime/3600000.0
         !Calculate moon position
            CALL MOON_POSITION(tdays, ra, dec)
            sindec = sin(dec)
            cosdec = cos(dec)
         !Space view altitude (with moon parallax of 1.04 deg)
         !The middle space view (of 3) is 17.1 deg below horizontal
            amsual = (-17.1 + 1.04)*deg2rad
            sinamsual = sin(amsual)
            cosamsual = cos(amsual)
         !Central lat/lon
            midlat(1) = Latitude(51,iscan)   !Note right to left scan
            midlat(2) = Latitude(48,iscan)
            midlon(1) = Longitude(51,iscan)
            midlon(2) = Longitude(48,iscan)
            azcorr = -0.00675   !radians: 18ms cross-track, 8/3 sec along-track 
         !Calculate the angular separation between moon and space view
            CALL moon_amsu(midlat,midlon, azcorr, tdays, thour, 
     &         ra, sindec, cosdec, 1, sinamsual, cosamsual, angle_diff)
!            write(*,*) iscan, angle_diff
            angle_diff_min = MIN(angle_diff,angle_diff_min)
            angle_diff_max = MAX(angle_diff,angle_diff_max)
            IF (angle_diff .LT. 1.5) THEN
              num_moon_reject = num_moon_reject + 1
              mwhs1c_scanqual = IOR(mwhs1c_scanqual, 65536)     !set bit 16
              mwhs1c_qualind = IOR(mwhs1c_qualind, -2147483648) !set bit 31
            ENDIF

         !Continue with filling the 1c
            mwhs1c_instrtemp = (Mixer_Temp(iscan) + 273.15)*1E2
            DO ispot=1,MWHS_SPOTS
              mwhs1c_dataqual(ispot) = 0
              mwhs1c_latlon(1,ispot) = 1E4*Latitude(ispot,iscan)
              mwhs1c_latlon(2,ispot) = 1E4*Longitude(ispot,iscan)
              mwhs1c_angles(1,ispot) = SensorZenith(ispot,iscan)
              mwhs1c_angles(2,ispot) = SensorAzimuth(ispot,iscan)
              mwhs1c_angles(3,ispot) = SolarZenith(ispot,iscan)
              mwhs1c_angles(4,ispot) = SolarAzimuth(ispot,iscan)
              mwhs1c_surface(1,ispot) = Height(ispot,iscan)
              IF (LandSeaMask(ispot,iscan).EQ.3) THEN
                mwhs1c_surface(2,ispot) = 0                 !Sea
              ELSE IF (LandSeaMask(ispot,iscan).EQ.1) THEN
                mwhs1c_surface(2,ispot) = 2                 !Land
              ELSE IF (LandSeaMask(ispot,iscan).EQ.5) THEN
                mwhs1c_surface(2,ispot) = 1                 !Mixed
              ELSE IF (LandSeaMask(ispot,iscan).EQ.2) THEN
                mwhs1c_surface(2,ispot) = 3                 !Inland water
              ELSE
                mwhs1c_surface(2,ispot) = MDI
              ENDIF              
            ENDDO
            jscan = (iscan-1)*MWHS_SPOTS
            DO ichan=1,MWHS_CHANS
              jchan = (ichan-1)*num_obs
              DO ispot=1,MWHS_SPOTS
                BT = Earth_Obs_BT(jchan+jscan+ispot)
                mwhs1c_btemps(ichan,ispot) = 100.0*BT + 0.5
                IF (BT.LT.BminBT .OR. BT.GT.BmaxBT) THEN
                  mwhs1c_dataqual(ispot) = mwhs1c_dataqual(ispot) + 
     &               qualval(ichan)
                  IF (ichan.EQ.MWHS_CHANS .AND. 
     &               mwhs1c_dataqual(ispot).EQ.qualval(ichan+1)) THEN
                    mwhs1c_dataqual(ispot) = mwhs1c_dataqual(ispot) + 1  !All channels bad
                  ENDIF
                ENDIF
              ENDDO
            ENDDO
          
            xysq = (GPS_Position_Data(1,iscan)*0.1)**2 +     
     &             (GPS_Position_Data(2,iscan)*0.1)**2
            z = GPS_Position_Data(3,iscan)*0.1
            r_sat = sqrt(xysq + z**2)       !Centre of Earth to satellite (m)
            lat = atan2(z,sqrt(xysq))
            r_ellipse = sqrt((semiMajor*cos(lat))**2 + 
     &                       (semiMinor*sin(lat))**2)
            mwhs1c_scalti = NINT((r_sat - r_ellipse)*0.01)    !km *10

            scnlinold = mwhs1c_scnlin
            iout = iout + 1
            CALL IOMWHS1C("WRITE DATA",unit,iout+1)
          ELSE
            IF (LatMissing) THEN
              write(*,*)'Missing latitude at iscan=',iscan
            ELSE
              write(*,*)'Bad latitude at iscan=',iscan,
     &            ', scandiff=',scandiff
            ENDIF
          ENDIF    !latitude check
         ENDIF    !timeoffset check
        ELSE
          write(*,*)'Bad time at iscan=',iscan
        ENDIF
      ENDDO
      
      WRITE(*,*)'Number of scan lines in input file: ', Count_scnlines
      WRITE(*,*)'Number of scan lines written to output file: ', iout
      WRITE(*,*)'Min/max moon angle: ', angle_diff_min, angle_diff_max
      WRITE(*,*)'Number of moon rejected scans: ', num_moon_reject

      mwhs1c_h_scnlin = iout
      mwhs1c_h_misscnlin = mwhs1c_scnlin - mwhs1c_h_scnlin
      CALL IOMWHS1C("WRITE HEADER",unit,1)

      CALL IOMWHS1C("CLOSE",unit,0)

      IF (iout .EQ. 0) CALL AAPPEXIT(1)

      RETURN
      END

