! Name: $Id: mwri_sdr_out.F 592 2018-08-09 14:36:40Z frna $
!
! Purpose: Fill MWHS 1c structure and write out file
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    20/05/2014  Create from mwts_sdr_out.F
!  1.1    06/01/2017  Correct some bugs: calculation of satellite velocity was
!                     wrong (hence orbit angle wrong) and satellite position
!                     did not take account of conical scan.
!  1.2    08/08/2017  MWRI1c_h_instrument 18 -> 20
!  1.3    11/08/2017  Allow for missing values
!  1.4    02/01/2018  Bug fix in velocity calculation
!
!-------------------------------------------------------------------------------

      SUBROUTINE MWRI_SDR_OUT (
     &  Count_scnlines, Orbit_Number, Orbit_Period,
     &  Earth_Obs_BT1, Pixel_View_Angle, Latitude, Longitude, 
     &  Scnlin_daycnt, Scnlin_mscnt1,
     &  SensorZenith, SensorAzimuth, SolarZenith, SolarAzimuth, DEM,
     &  LandSeaMask, QA_Ch_Flag, QA_Scan_Flag, scnlin_qc, 
     &  Beginning_Date, Beginning_Time, Ending_Date, Ending_Time,
     &  Satellite_Name)
     
      IMPLICIT NONE

      INTEGER   MWRI_SPOTS, MWRI_CHANS, MWRI_SCANS
      PARAMETER (MWRI_SPOTS=254)
      PARAMETER (MWRI_CHANS=10)
      PARAMETER (MWRI_SCANS=2000)

! Arguments: 
      INTEGER*4    Count_scnlines
      INTEGER*4    Orbit_Number
      INTEGER*4    Orbit_Period
      REAL         Latitude(MWRI_SPOTS,MWRI_SCANS)
      REAL         Longitude(MWRI_SPOTS,MWRI_SCANS)
      INTEGER*2    Pixel_View_Angle(2,MWRI_SCANS)
      INTEGER*2    Scnlin_daycnt(MWRI_SCANS)
!      INTEGER*4    Scnlin_mscnt(MWRI_SCANS)
      REAL         Scnlin_mscnt(MWRI_SCANS)
      REAL         Scnlin_mscnt1(2,MWRI_SCANS)
!      REAL         Earth_Obs_BT(MWRI_SPOTS*MWRI_SCANS*MWRI_CHANS)
      REAL         Earth_Obs_BT(MWRI_SPOTS*MWRI_SCANS*MWRI_CHANS)
      INTEGER*2    Earth_Obs_BT1(MWRI_SPOTS*MWRI_SCANS*MWRI_CHANS)
      INTEGER*2    SensorZenith(MWRI_SPOTS,MWRI_SCANS)
      INTEGER*2    SensorAzimuth(MWRI_SPOTS,MWRI_SCANS)
      INTEGER*2    SolarZenith(MWRI_SPOTS,MWRI_SCANS)
      INTEGER*2    SolarAzimuth(MWRI_SPOTS,MWRI_SCANS)
      INTEGER*2    DEM(MWRI_SPOTS,MWRI_SCANS)
      INTEGER*2    LandSeaMask(MWRI_SPOTS,MWRI_SCANS)
      INTEGER*2    QA_Ch_Flag(MWRI_SCANS)
      INTEGER*2    QA_Scan_Flag(MWRI_SCANS)
      INTEGER*4    scnlin_qc(MWRI_SCANS)
      CHARACTER*11 Beginning_Date, Ending_Date
      CHARACTER*13 Beginning_Time, Ending_Time
      CHARACTER*6  Satellite_Name

! Local variables

      INTEGER iscan,ispot,ichan,unit,jscan,jchan,iout,id,scan1,scan2,i
      INTEGER num_obs
      INTEGER year, month, day, hour, minute
      INTEGER scanline, scnlinold
      INTEGER firsttime, tdiff, scandiff, scnlintime_old
      INTEGER nbad, nbad_threshold
      DATA    nbad_threshold/30/   !cal assumed bad if nbad>threshold
      INTEGER BTmin(MWRI_CHANS), BTmax(MWRI_CHANS)
      REAL    latdiff, londiff, latlondiff, LatEdgediff
      REAL    lat(3), lon(3), x(3), y(3), z(3)
      REAL    xpos(MWRI_SCANS),ypos(MWRI_SCANS),zpos(MWRI_SCANS)
      REAL    second
      REAL    BT
      REAL    scanangle, centreangle, x1, x2, y1, y2, z1, z2, height_km
      REAL    rad, dt, earth_rotation, corr
      REAL    pos_vector(3),vel_vector(3),angle,xpos2mod,ypos2mod
      REAL    RMDI
      DATA    RMDI/-9999.99/
      LOGICAL LatOK, LatMissing
      INTEGER qualind(MWRI_SCANS)
      DATA qualind / MWRI_SCANS*0 /
      INTEGER qualval(MWRI_CHANS+1)
      DATA qualval/2,4,8,16,32,64,128,256,512,1024,2048/  !Last is the sum
      CHARACTER*80 line
      REAL    channel_info(7,MWRI_CHANS)
      DATA    channel_info /70*0/
      LOGICAL badcal(MWRI_SCANS), badspot

!Function
      INTEGER    IDAYOFYEAR

! Include
#include "mwri1c.h"
#include "inparms.h"
#include "constants_preproc.h"

! End of declarations
!-----------------------------------------------------------------------

      unit = 10
      CALL IOMWRI1C("OPEN/WRITE",unit,0)
      
      IF (Satellite_Name(1:5) .EQ. "FY-3A") THEN
        MWRI1c_h_satid = 1
        MWRI1c_h_wmosatid = 520
      ELSE IF (Satellite_Name(1:5) .EQ. "FY-3B") THEN
        MWRI1c_h_satid = 2
        MWRI1c_h_wmosatid = 521
      ELSE IF (Satellite_Name(1:5) .EQ. "FY-3C") THEN
        MWRI1c_h_satid = 3
        MWRI1c_h_wmosatid = 522
      ELSE IF (Satellite_Name(1:5) .EQ. "FY-3D") THEN
        MWRI1c_h_satid = 4
        MWRI1c_h_wmosatid = 523
      ELSE
        MWRI1c_h_satid = MDI
        MWRI1c_h_wmosatid = MDI
      ENDIF

!Read the MWHS channel info on unit 21
      OPEN (21,form='formatted',status='old')
      id = 0
      DO while (id .NE. MWRI1c_h_wmosatid)
        line="     "
        DO while (line(1:5) .ne. "# WMO")
          read(21,'(A)',END=100) line
        ENDDO
        read(21,*) id
        IF (id .eq. MWRI1c_h_wmosatid) THEN
          read(21,*) line
          read(21,*) channel_info
        ENDIF
      ENDDO
  100 CLOSE (21)
      DO ichan=1,MWRI_CHANS
        MWRI1c_h_tempradcnv(1,ichan) = 
     &    channel_info(2,ichan)*1.0E13/C_light   !wavenumber*1E6
        MWRI1c_h_tempradcnv(2,ichan) = channel_info(4,ichan)*1.0E6  !offset
        MWRI1c_h_tempradcnv(3,ichan) = channel_info(5,ichan)*1.0E6  !slope
        MWRI1c_h_bandwidth(ichan) = channel_info(3,ichan)*1.0E6     !GHz*1E6
        BTmin(ichan) = channel_info(6,ichan)
        BTmax(ichan) = channel_info(7,ichan)
      ENDDO

!The time attributes are unreliable for MWRI, so comment out the following

!      READ(Ending_Date,'(I4,1X,I2,1X,I2)') year, month, day
!      READ(Ending_Time,'(I2,1X,I2,1X,F6.3)')hour, minute, second
!      MWRI1c_h_enddatayr = year
!      MWRI1c_h_enddatady = IDAYOFYEAR(year, month, day)
!      MWRI1c_h_enddatatime = 1000*(hour*3600 + minute*60 + second)
      
!      READ(Beginning_Date,'(I4,1X,I2,1X,I2)') year, month, day
!      READ(Beginning_Time,'(I2,1X,I2,1X,F6.3)') hour, minute, second
!      MWRI1c_h_startdatayr = year
!      MWRI1c_h_startdatady = IDAYOFYEAR(year, month, day)
!      MWRI1c_h_startdatatime = 1000*(hour*3600 + minute*60 + second)
      
      MWRI1c_h_instrument = 20
      MWRI1c_h_startorbit = Orbit_Number
      MWRI1c_h_endorbit = Orbit_Number
      MWRI1c_h_hdrcnt = 1
      MWRI1c_h_versnb = MWRI_1cvnnb
      MWRI1c_h_versyr = MWRI_1cvnyr
      MWRI1c_h_versdy = MWRI_1cvndy
      MWRI1c_h_satht = 8360      !nominal
      MWRI1c_h_period = Orbit_Period*60
      MWRI1c_h_nchan = MWRI_nchan
      MWRI1c_h_nmapchan = MWRI_nmapchan
      MWRI1c_h_nmapchanused = 0          !No mapping in this program
      MWRI1c_h_beamversion = 0
      DO ichan=1,MWRI_CHANS
        MWRI1c_h_beamwidth(ichan) = 0
        MWRI1c_h_cutoff(ichan) = 0
        MWRI1c_h_nxaverage(ichan) = 0    !No averaging in this program
        MWRI1c_h_nyaverage(ichan) = 0
      ENDDO
      
      CALL IOMWRI1C("WRITE HEADER",unit,1)

      num_obs = Count_scnlines*MWRI_SPOTS
      iout = 0
      MWRI1c_scnlin = 0
!Estimate satellite position in Greenwich co-ords - not present in the hdf5
!Could get a more accurate value from the satpos file, but this will do for
!now. Ignore elipticity of earth. Use nominal satellite height.
!
!For the MWRI conical scan geometry, use the scan extremes as well as the
!centre to estimate the satellite position. This is an approximation: the
!weighting coefficients were estimated from a plot of lat/lon.

      rad = rEarth*1000 + MWRI1c_h_satht*100   !distance from centre of earth
      DO iscan=1,Count_scnlines
        lat(1) = Latitude(1,iscan)
        lat(2) = Latitude(MWRI_SPOTS,iscan)
        lat(3) = Latitude(MWRI_SPOTS/2,iscan)
        lon(1) = Longitude(1,iscan)
        lon(2) = Longitude(MWRI_SPOTS,iscan)
        lon(3) = Longitude(MWRI_SPOTS/2,iscan)
        IF (lat(1) .LT. 32767) THEN
          call latlon_to_xyz(3,1,lat,lon,x,y,z)
          xpos(iscan) = rad*(1.25*x(1) + 1.25*x(2) - 1.5*x(3))
          ypos(iscan) = rad*(1.25*y(1) + 1.25*y(2) - 1.5*y(3))
          zpos(iscan) = rad*(1.25*z(1) + 1.25*z(2) - 1.5*z(3))
          corr = rad/sqrt(xpos(iscan)**2+ypos(iscan)**2+zpos(iscan)**2)
          xpos(iscan) = xpos(iscan)*corr
          ypos(iscan) = ypos(iscan)*corr
          zpos(iscan) = zpos(iscan)*corr
        ELSE
          xpos(iscan) = RMDI
          ypos(iscan) = RMDI
          zpos(iscan) = RMDI
        ENDIF
      ENDDO
!Quality control on the brightness temperatures
!If any channel has BT outside the expected range then set them all to missing.
!If number of bad spots exceeds a threshold then it's probably a calibration
!error, so set missing for !+-3 scan lines.

      DO iscan=1,Count_scnlines
        badcal(iscan) = .false.
        nbad = 0
        jscan = (iscan-1)*MWRI_SPOTS
        DO ispot=1,MWRI_SPOTS
          badspot = .false.
          DO ichan=1,MWRI_CHANS
            jchan = (ichan-1)*num_obs
            i = Earth_Obs_BT1(jchan+jscan+ispot)
!Special values:   0 = missing at edge of swath
!               -999 = value that we seem to get when BT>340K
!              29999 = "FillValue" according to the HDF file
            IF (i .eq. 0 .or. i .eq. -999 .or. i .eq. 29999) THEN
              Earth_Obs_BT(jchan+jscan+ispot) = -327.68
            ELSE
              Earth_Obs_BT(jchan+jscan+ispot)=327.68+0.01*i
              IF (Earth_Obs_BT(jchan+jscan+ispot).LT.BTmin(ichan)
     &         .OR.Earth_Obs_BT(jchan+jscan+ispot).GT.BTmax(ichan)) THEN
                badspot = .true.
              ENDIF
            ENDIF
          ENDDO
          IF (badspot) THEN
            nbad = nbad + 1
            if (nbad .GT. nbad_threshold) badcal(iscan) = .true.
            DO ichan=1,MWRI_CHANS
              jchan = (ichan-1)*num_obs
              !luqf
              !Earth_Obs_BT(jchan+jscan+ispot) = -327.68
            ENDDO
          ENDIF
        ENDDO
      ENDDO
      DO iscan=1,Count_scnlines
        jscan = (iscan-1)*MWRI_SPOTS
        DO i=-3,3
          IF (iscan+i.GT.0.AND.iscan+i.LT.Count_scnlines) THEN
            IF (badcal(iscan+i)) THEN
              DO ispot=1,MWRI_SPOTS
                DO ichan=1,MWRI_CHANS
                  jchan = (ichan-1)*num_obs
                  !luqf
                  !Earth_Obs_BT(jchan+jscan+ispot) = -327.68
                ENDDO
              ENDDO
            ENDIF
          ENDIF
        ENDDO
      ENDDO
          
!Main loop
      DO iscan=1,Count_scnlines
        MWRI1c_qualind = QA_Ch_Flag(iscan)
        MWRI1c_scanqual = QA_Scan_Flag(iscan)
        scnlintime_old = MWRI1c_scnlintime
        !luqf
        Scnlin_mscnt(iscan)=Scnlin_mscnt1(1,iscan)
     &                      +Scnlin_mscnt1(2,iscan)

        CALL convert_timecnt_fy3(Scnlin_daycnt(iscan),
     &    Scnlin_mscnt(iscan),mwri1c_scnlinyr,mwri1c_scnlindy,
     &    mwri1c_scnlintime)

        IF (MWRI1c_scnlintime .GE. 0) THEN

          IF (MWRI1c_scnlin .EQ. 0) THEN
            firsttime = MWRI1c_scnlintime
            mwri1c_h_startdatayr = mwri1c_scnlinyr
            mwri1c_h_startdatady = mwri1c_scnlindy
            mwri1c_h_startdatatime = mwri1c_scnlintime
          ENDIF
          
          tdiff = MWRI1c_scnlintime - firsttime
          IF (tdiff .LT. -43200000) THEN 
            tdiff = tdiff + 86400000
          ENDIF
          MWRI1c_scnlin = NINT(tdiff/1800.0) + 1   !1.8 seconds per scan

          !Check that the latitudes are sensible: latitude difference across the
          !scan and also the difference from one scan to the next. 
          LatOK = .TRUE.
          LatMissing = .FALSE.
          DO ispot=1,MWRI_SPOTS
            IF (.not. ABS(Latitude(ispot,iscan)).LE.90.0) THEN
              LatOK = .FALSE.
              LatMissing = .TRUE.
            ENDIF
          ENDDO
          !Check latitude difference across the scan
          LatEdgediff = -(Latitude(1,iscan)-Latitude(254,iscan))
          LatEdgediff = LatEdgediff*cos(Latitude(127,iscan)*deg2rad)
          IF (LatEdgediff .GT. 3.0 .OR. LatEdgediff .LT. 1.0) THEN
            LatOK = .FALSE.
          ENDIF

          scandiff = MWRI1c_scnlin-scnlinold
          IF (LatOK .AND. iout.GT.0) THEN
            DO ispot=1,MWRI_SPOTS
              latdiff =
     &           Latitude(ispot,iscan) - 1E-4*MWRI1c_latlon(1,ispot)
              londiff =
     &           Longitude(ispot,iscan) - 1E-4*MWRI1c_latlon(2,ispot)
              IF (londiff .GT. 180.0) THEN
                londiff = londiff - 360.0
              ELSE IF (londiff .LT. -180) THEN
                londiff = londiff + 360.0
              ENDIF
              latlondiff = sqrt(latdiff**2 + 
     &              (londiff*cos(deg2rad*Latitude(ispot,iscan)))**2)
              IF (latlondiff .GT. 0.5*scandiff) THEN
                LatOK = .FALSE.
              ENDIF
            ENDDO
          ENDIF
          IF (LatOK) THEN

            mwri1c_h_enddatayr = mwri1c_scnlinyr
            mwri1c_h_enddatady = mwri1c_scnlindy
            mwri1c_h_enddatatime = mwri1c_scnlintime

            MWRI1c_instrtemp1 = MDI
            MWRI1c_instrtemp2 = MDI
            DO ispot=1,MWRI_SPOTS
              MWRI1c_dataqual(ispot) = 0
              MWRI1c_latlon(1,ispot) = 1E4*Latitude(ispot,iscan)
              MWRI1c_latlon(2,ispot) = 1E4*Longitude(ispot,iscan)
              MWRI1c_angles(1,ispot) = SensorZenith(ispot,iscan)
              MWRI1c_angles(2,ispot) = SensorAzimuth(ispot,iscan)
              MWRI1c_angles(3,ispot) = SolarZenith(ispot,iscan)
              MWRI1c_angles(4,ispot) = SolarAzimuth(ispot,iscan)
              MWRI1c_surface(1,ispot) = DEM(ispot,iscan)
              IF (LandSeaMask(ispot,iscan).EQ.3) THEN
                MWRI1c_surface(2,ispot) = 0                 !Sea
              ELSE IF (LandSeaMask(ispot,iscan).EQ.1) THEN
                MWRI1c_surface(2,ispot) = 2                 !Land
              ELSE IF (LandSeaMask(ispot,iscan).EQ.5) THEN
                MWRI1c_surface(2,ispot) = 1                 !Mixed
              ELSE IF (LandSeaMask(ispot,iscan).EQ.2) THEN
                MWRI1c_surface(2,ispot) = 3                 !Inland water
              ELSE
                MWRI1c_surface(2,ispot) = MDI
              ENDIF              
            ENDDO
            jscan = (iscan-1)*MWRI_SPOTS
            DO ichan=1,MWRI_CHANS
              jchan = (ichan-1)*num_obs
              DO ispot=1,MWRI_SPOTS
                BT = Earth_Obs_BT(jchan+jscan+ispot)
                IF (BT .GT. 0) THEN
                  MWRI1c_btemps(ichan,ispot) = BT*100
                ELSE
                  MWRI1c_btemps(ichan,ispot) = MDI
                ENDIF
                IF (BT.LT.AminBT .OR. BT.GT.AmaxBT .OR.
     &               MWRI1c_qualind .LT. 0) THEN
                  MWRI1c_dataqual(ispot) = MWRI1c_dataqual(ispot) + 
     &               qualval(ichan)
                  IF (ichan.EQ.MWRI_CHANS .AND. 
     &               MWRI1c_dataqual(ispot).EQ.qualval(ichan+1)) THEN
                    MWRI1c_dataqual(ispot) = MWRI1c_dataqual(ispot)+1  !All channels bad
                  ENDIF
                ENDIF
              ENDDO
            ENDDO
          
!For MWRI conical scan geometry we can't easily estimate the height.
!So set it to nominal value.

            MWRI1c_scalti = MWRI1c_h_satht

!Satellite position and velocity 
            
            IF (xpos(iscan) .NE. RMDI) THEN
              MWRI1c_satposition(1) = xpos(iscan)
              MWRI1c_satposition(2) = ypos(iscan)
              MWRI1c_satposition(3) = zpos(iscan)
            ELSE
              MWRI1c_satposition(1) = MDI
              MWRI1c_satposition(2) = MDI
              MWRI1c_satposition(3) = MDI
            ENDIF

            IF (iscan .EQ. 1) THEN
              scan1 = iscan
              scan2 = iscan + 1
            ELSE IF (iscan .EQ. Count_scnlines) THEN
              scan1 = iscan - 1
              scan2 = iscan
            ELSE
              scan1 = iscan - 1
              scan2 = iscan + 1
              IF (xpos(iscan-1).EQ.RMDI) scan1 = iscan
              IF (xpos(iscan+1).EQ.RMDI) scan2 = iscan
            ENDIF

            IF (scan1 .NE. scan2 .AND. xpos(scan1).NE.RMDI .AND. 
     &          xpos(scan2).NE.RMDI) THEN
              dt = (Scnlin_mscnt1(1,scan2)+Scnlin_mscnt1(2,scan2))*0.001
     &           - (Scnlin_mscnt1(1,scan1)+Scnlin_mscnt1(2,scan1))*0.001

              IF (dt .LT. 0) dt = dt + 86400.0

              earth_rotation = dt/86400.0 * 2 * PI   !angle is small
              xpos2mod = xpos(scan2) - ypos(scan2)*earth_rotation
              ypos2mod = ypos(scan2) + xpos(scan2)*earth_rotation

              MWRI1c_satvelocity(1) = 
     &          (xpos2mod - xpos(scan1))/dt
              MWRI1c_satvelocity(2) = 
     &          (ypos2mod - ypos(scan1))/dt
              MWRI1c_satvelocity(3) = 
     &          (zpos(scan2) - zpos(scan1))/dt

              pos_vector(1) = MWRI1c_satposition(1)
              pos_vector(2) = MWRI1c_satposition(2)
              pos_vector(3) = MWRI1c_satposition(3)
              vel_vector(1) = MWRI1c_satvelocity(1)
              vel_vector(2) = MWRI1c_satvelocity(2)
              vel_vector(3) = MWRI1c_satvelocity(3)

              CALL orbit_angle (1,MWRI1c_scnlinyr,MWRI1c_scnlindy,0,
     &          MWRI1c_scnlintime,pos_vector,vel_vector,angle)
              MWRI1c_orbitangle = angle*100
            ELSE
              MWRI1c_orbitangle = MDI
            ENDIF

            scnlinold = MWRI1c_scnlin
            iout = iout + 1
            CALL IOMWRI1C("WRITE DATA",unit,iout+1)
          ELSE
            IF (LatMissing) THEN
              write(*,*)'Missing latitude at iscan=',iscan
            ELSE
              write(*,*)'Bad latitude at iscan=',iscan
            ENDIF
          ENDIF
        ELSE
          write(*,*)'Bad time at iscan=',iscan,MWRI1c_scnlintime
        ENDIF
      ENDDO
      
      WRITE(*,*)'Number of scan lines in input file: ', Count_scnlines
      WRITE(*,*)'Number of scan lines written to output file: ', iout
      MWRI1c_h_scnlin = iout
      MWRI1c_h_misscnlin = MWRI1c_scnlin - MWRI1c_h_scnlin
      CALL IOMWRI1C("WRITE HEADER",unit,1)

      CALL IOMWRI1C("CLOSE",unit,0)

      IF (iout .EQ. 0) CALL AAPPEXIT(1)

      RETURN
      END

!------------

      SUBROUTINE convert_timecnt_fy3(daycnt,mscnt,year,jd,ms)
      IMPLICIT NONE
      integer*2 daycnt
      real      mscnt       !day 0 second 0 is 2000-01-01 12Z
      integer   year,jd,ms  !outputs following AAPP convention
      integer   daycnt2,i
      integer   days_in_year(0:3)
      data days_in_year/366,365,365,365/   !2000, 2001, 2002, etc.
      
      if (mscnt .ge. 43200000) then
        daycnt2 = daycnt + 2
        ms = mscnt - 43200000
      else
        daycnt2 = daycnt + 1
        ms = mscnt + 43200000
      endif
      year = 1999
      i = 0
      do while (i .lt. daycnt2)
        year = year + 1
        i = i + days_in_year(mod(year,4))
      enddo
      jd = daycnt2 - i + days_in_year(mod(year,4))
      return
      end
