! Name: $Id: mwts21c_read_f.F 463 2015-09-28 15:29:53Z frna $
!
! Purpose: Read MWTS2 l1c file and pass information back to C calling routine.
!          Subroutines MWTS21C_READ_HEADER and MWTS21C_READ_DATA
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2015, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    25/09/2015  N C Atkinson
!-----------------------------------------------------------------------

      SUBROUTINE MWTS21C_READ_HEADER (
     & filename, filename_length,
     & h_site, h_1bsite, h_versnb,
     & h_satid, h_instrument, h_satht, h_period,
     & h_startorbit, h_startdatayr, h_startdatady, h_startdatatime,
     & h_endorbit, h_enddatayr, h_enddatady, h_enddatatime,
     & h_scnlin, h_misscnlin, h_vnantennacorr, h_tempradcnv, h_wmosatid,
     & h_bandwidth, h_nchan, h_nmapchan, h_nmapchanused, h_beamversion,
     & h_beamwidth, h_cutoff, h_nxaverage, h_nyaverage )

      IMPLICIT NONE
      
! Parameters
      INTEGER   UNIT_1C
      PARAMETER (UNIT_1C=10)
      INTEGER*4 MWTS2_CHANS
      PARAMETER (MWTS2_CHANS=13)

! Arguments: 
      CHARACTER*(*) filename
      INTEGER       filename_length
      CHARACTER*3   h_site
      CHARACTER*3   h_1bsite
      INTEGER*4     h_versnb
      INTEGER*4     h_satid
      INTEGER*4     h_instrument
      INTEGER*4     h_satht
      INTEGER*4     h_period
      INTEGER*4     h_startorbit
      INTEGER*4     h_startdatayr
      INTEGER*4     h_startdatady
      INTEGER*4     h_startdatatime
      INTEGER*4     h_endorbit
      INTEGER*4     h_enddatayr
      INTEGER*4     h_enddatady
      INTEGER*4     h_enddatatime
      INTEGER*4     h_scnlin
      INTEGER*4     h_misscnlin
      INTEGER*4     h_vnantennacorr
      INTEGER*4     h_tempradcnv(3,MWTS2_CHANS)
      INTEGER*4     h_wmosatid
      INTEGER*4     h_bandwidth(MWTS2_CHANS)
      INTEGER*4     h_nchan
      INTEGER*4     h_nmapchan
      INTEGER*4     h_nmapchanused
      INTEGER*4     h_beamversion
      INTEGER*4     h_beamwidth(MWTS2_CHANS)
      INTEGER*4     h_cutoff(MWTS2_CHANS)
      INTEGER*4     h_nxaverage(MWTS2_CHANS)
      INTEGER*4     h_nyaverage(MWTS2_CHANS)

! Local:
      INTEGER errStatus, i, ichan

#include "inparms.h"
#include "mwts21c.h"
#include "reclopt.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Open MWTS2 1c file ',filename(1:filename_length)
      OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &   STATUS="OLD", FORM="UNFORMATTED", ACCESS="DIRECT",
     &   RECL=mwts2_1crecl/rldiv, IOSTAT=errStatus )
      IF (errStatus.NE.0) THEN
        write(*,*)'Cannot open file ',filename(1:filename_length)
        call aappexit(1)
      ENDIF

      write(*,*)'Read MWTS2 header'
      CALL IOMWTS21C("READ HEADER",UNIT_1C,1)

      h_scnlin = mwts21c_h_scnlin
      h_site = mwts21c_h_site
      h_1bsite = mwts21c_h_1bsite
      h_versnb = mwts21c_h_versnb
      h_satid = mwts21c_h_satid
      h_instrument = mwts21c_h_instrument
      h_satht = mwts21c_h_satht
      h_period = mwts21c_h_period
      h_startorbit = mwts21c_h_startorbit
      h_startdatayr = mwts21c_h_startdatayr
      h_startdatady = mwts21c_h_startdatady
      h_startdatatime = mwts21c_h_startdatatime
      h_endorbit = mwts21c_h_endorbit
      h_enddatayr = mwts21c_h_enddatayr
      h_enddatady = mwts21c_h_enddatady
      h_enddatatime = mwts21c_h_enddatatime
      h_scnlin = mwts21c_h_scnlin
      h_misscnlin = mwts21c_h_misscnlin
      h_vnantennacorr = mwts21c_h_vnantennacorr
      do ichan=1,MWTS2_CHANS
        do i=1,3
          h_tempradcnv(i,ichan) = mwts21c_h_tempradcnv(i,ichan)
        enddo
        h_bandwidth(ichan) = mwts21c_h_bandwidth(ichan)
        h_beamwidth(ichan) = mwts21c_h_beamwidth(ichan)
        h_cutoff(ichan) = mwts21c_h_cutoff(ichan)
        h_nxaverage(ichan) = mwts21c_h_nxaverage(ichan)
        h_nyaverage(ichan) = mwts21c_h_nyaverage(ichan)
      enddo
      h_wmosatid = mwts21c_h_wmosatid
      h_nchan = mwts21c_h_nchan
      h_nmapchan = mwts21c_h_nmapchan
      h_nmapchanused = mwts21c_h_nmapchanused
      h_beamversion = mwts21c_h_beamversion

      RETURN
      END
      
!-------------------------------------------------------------------------------

      SUBROUTINE MWTS21C_READ_DATA (
     &  h_scnlin,
     &  scnlin, scnlinyr, scnlindy, scnlintime,
     &  qualind, scanqual, chanqual, instrtemp1, instrtemp2,
     &  latitude, longitude, satzen, solzen, satazi, solazi, scalti,
     &  satposition, satvelocity, orbitangle,
     &  btemps, btmap, dataqual)
      IMPLICIT NONE

! Parameters
      INTEGER*4 MWTS2_CHANS
      PARAMETER (MWTS2_CHANS=13)
      INTEGER*4 MWTS2_MAPCHANS
      PARAMETER (MWTS2_MAPCHANS=15)
      INTEGER*4 MWTS2_SPOTS
      PARAMETER (MWTS2_SPOTS=90)
      INTEGER*4 UNIT_1C
      PARAMETER (UNIT_1C=10)

! Arguments: 
      INTEGER*4 h_scnlin
      INTEGER*4 scnlin(h_scnlin)
      INTEGER*4 scnlinyr(h_scnlin)
      INTEGER*4 scnlindy(h_scnlin)
      INTEGER*4 scnlintime(h_scnlin)
      INTEGER*4 qualind(h_scnlin)
      INTEGER*4 scanqual(h_scnlin)
      INTEGER*4 chanqual(MWTS2_CHANS,h_scnlin)
      INTEGER*4 instrtemp1(h_scnlin)
      INTEGER*4 instrtemp2(h_scnlin)
      INTEGER*4 latitude(MWTS2_SPOTS,h_scnlin)
      INTEGER*4 longitude(MWTS2_SPOTS,h_scnlin)
      INTEGER*4 satzen(MWTS2_SPOTS,h_scnlin)
      INTEGER*4 satazi(MWTS2_SPOTS,h_scnlin)
      INTEGER*4 solzen(MWTS2_SPOTS,h_scnlin)
      INTEGER*4 solazi(MWTS2_SPOTS,h_scnlin)
      INTEGER*4 scalti(h_scnlin)
      INTEGER*4 satposition(3,h_scnlin)
      INTEGER*4 satvelocity(3,h_scnlin)
      INTEGER*4 orbitangle(h_scnlin)
      INTEGER*4 btemps(MWTS2_CHANS,MWTS2_SPOTS,h_scnlin)
      INTEGER*4 btmap(MWTS2_MAPCHANS,MWTS2_SPOTS,h_scnlin)
      INTEGER*4 dataqual(MWTS2_SPOTS,h_scnlin)

! Local
      INTEGER iscan,ichan,ispot,i

#include "inparms.h"
#include "mwts21c.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Read MWTS2 data records'
      DO iscan=1,h_scnlin
        CALL IOMWTS21C("READ DATA",UNIT_1C,iscan+1)
        scnlin(iscan) = mwts21c_scnlin
        scnlinyr(iscan) = mwts21c_scnlinyr
        scnlindy(iscan) = mwts21c_scnlindy
        scnlintime(iscan) = mwts21c_scnlintime
        qualind(iscan) = mwts21c_qualind
        scanqual(iscan) = mwts21c_scanqual
        instrtemp1(iscan) = mwts21c_instrtemp1
        instrtemp2(iscan) = mwts21c_instrtemp2
        scalti(iscan) = mwts21c_scalti
        DO ichan=1,MWTS2_CHANS
          chanqual(ichan,iscan) = mwts21c_chanqual(ichan)
        ENDDO
        DO ispot=1,MWTS2_SPOTS
          DO ichan=1,MWTS2_CHANS
            btemps(ichan,ispot,iscan) = mwts21c_btemps(ichan,ispot)
          ENDDO
          DO ichan=1,MWTS2_MAPCHANS
            btmap(ichan,ispot,iscan) = mwts21c_btmap(ichan,ispot)
          ENDDO
          latitude(ispot,iscan) = mwts21c_latlon(1,ispot)
          longitude(ispot,iscan) = mwts21c_latlon(2,ispot)
          satzen(ispot,iscan) = mwts21c_angles(1,ispot)
          satazi(ispot,iscan) = mwts21c_angles(2,ispot)
          solzen(ispot,iscan) = mwts21c_angles(3,ispot)
          solazi(ispot,iscan) = mwts21c_angles(4,ispot)
          dataqual(ispot,iscan) = mwts21c_dataqual(ispot)
        ENDDO
        DO i=1,3
          satposition(i,iscan) = mwts21c_satposition(i)
          satvelocity(i,iscan) = mwts21c_satvelocity(i)
        ENDDO
        orbitangle(iscan) = mwts21c_orbitangle
      ENDDO

      CALL IOMWTS21C("CLOSE",UNIT_1C,0)

      RETURN
      END
      
