! Name: $Id: mwts21c_write_f.F 606 2019-01-09 10:12:51Z frna $
!
! Purpose: Write MWTS2 l1c file using information from C calling routine.
!          Subroutines MWTS21C_WRITE_HEADER and MWTS21C_WRITE_DATA
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2019, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    08/01/2019  N C Atkinson
!-----------------------------------------------------------------------

      SUBROUTINE MWTS21C_WRITE_HEADER (
     & filename, filename_length,
     & h_site, h_1bsite, h_versnb,
     & h_satid, h_instrument, h_satht, h_period,
     & h_startorbit, h_startdatayr, h_startdatady, h_startdatatime,
     & h_endorbit, h_enddatayr, h_enddatady, h_enddatatime,
     & h_scnlin, h_misscnlin, h_vnantennacorr, h_tempradcnv, h_wmosatid,
     & h_bandwidth, h_nmapchanused, h_beamversion, h_cutoff, 
     & h_nxaverage, h_nyaverage)

      IMPLICIT NONE
      
! Parameters
      INTEGER   UNIT_1C
      PARAMETER (UNIT_1C=10)
      INTEGER*4 MWTS2_CHANS
      PARAMETER (MWTS2_CHANS=13)

! Arguments: 
      CHARACTER*(*) filename
      INTEGER       filename_length
      CHARACTER*3   h_site
      CHARACTER*3   h_1bsite
      INTEGER*4     h_versnb
      INTEGER*4     h_satid
      INTEGER*4     h_instrument
      INTEGER*4     h_satht
      INTEGER*4     h_period
      INTEGER*4     h_startorbit
      INTEGER*4     h_startdatayr
      INTEGER*4     h_startdatady
      INTEGER*4     h_startdatatime
      INTEGER*4     h_endorbit
      INTEGER*4     h_enddatayr
      INTEGER*4     h_enddatady
      INTEGER*4     h_enddatatime
      INTEGER*4     h_scnlin
      INTEGER*4     h_misscnlin
      INTEGER*4     h_vnantennacorr
      INTEGER*4     h_tempradcnv(3,MWTS2_CHANS)
      INTEGER*4     h_wmosatid
      INTEGER*4     h_bandwidth(MWTS2_CHANS)
      INTEGER*4     h_nmapchanused
      INTEGER*4     h_beamversion
      INTEGER*4     h_cutoff(MWTS2_CHANS)
      INTEGER*4     h_nxaverage(MWTS2_CHANS)
      INTEGER*4     h_nyaverage(MWTS2_CHANS)

! Local:
      INTEGER errStatus, i, ichan

#include "inparms.h"
#include "mwts21c.h"
#include "reclopt.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Open MWTS2 1c file ',filename(1:filename_length)
      OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &   STATUS="NEW", FORM="UNFORMATTED", ACCESS="DIRECT",
     &   RECL=mwts2_1crecl/rldiv, IOSTAT=errStatus )
      IF (errStatus.NE.0) THEN
        OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &     STATUS="OLD", FORM="UNFORMATTED", ACCESS="DIRECT",
     &     RECL=mwts2_1crecl/rldiv, IOSTAT=errStatus )
        IF (errStatus.NE.0) THEN
          OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &     STATUS="UNKNOWN", FORM="UNFORMATTED", ACCESS="DIRECT",
     &     RECL=mwts2_1crecl/rldiv, IOSTAT=errStatus )
          IF (errStatus.NE.0) THEN
            write(*,*)'Cannot open file ',filename(1:filename_length)
            call aappexit(1)
          ENDIF
        ENDIF
      ENDIF

      write(*,*)'Write MWTS2 header'
      mwts21c_h_scnlin = h_scnlin
      mwts21c_h_site = h_site
      mwts21c_h_cfill1 = ' '
      mwts21c_h_1bsite = h_1bsite
      mwts21c_h_cfill2 = ' '
      mwts21c_h_versnb = mwts2_1cvnnb
      mwts21c_h_versyr = mwts2_1cvnyr
      mwts21c_h_versdy = mwts2_1cvndy
      mwts21c_h_hdrcnt = 1
      mwts21c_h_satid = h_satid
      mwts21c_h_instrument = h_instrument
      mwts21c_h_satht = h_satht
      mwts21c_h_period = h_period
      mwts21c_h_startorbit = h_startorbit 
      mwts21c_h_startdatayr = h_startdatayr
      mwts21c_h_startdatady = h_startdatady
      mwts21c_h_startdatatime = h_startdatatime
      mwts21c_h_endorbit = h_endorbit
      mwts21c_h_enddatayr = h_enddatayr
      mwts21c_h_enddatady = h_enddatady
      mwts21c_h_enddatatime = h_enddatatime
      mwts21c_h_scnlin = h_scnlin
      mwts21c_h_misscnlin = h_misscnlin
      mwts21c_h_vnantennacorr = h_vnantennacorr
      do ichan=1,MWTS2_CHANS
        do i=1,3
         mwts21c_h_tempradcnv(i,ichan) = h_tempradcnv(i,ichan) 
        enddo
        mwts21c_h_bandwidth(ichan) = h_bandwidth(ichan)
        mwts21c_h_cutoff(ichan) = h_cutoff(ichan)
        mwts21c_h_nxaverage(ichan) = h_nxaverage(ichan)
        mwts21c_h_nyaverage(ichan) = h_nyaverage(ichan)
      enddo
      mwts21c_h_wmosatid = h_wmosatid
      mwts21c_h_nchan = mwts2_nchan
      mwts21c_h_nmapchan = mwts2_nmapchan
      mwts21c_h_nmapchanused = h_nmapchanused
      mwts21c_h_beamversion = h_beamversion

      CALL IOMWTS21C("WRITE HEADER",UNIT_1C,1)

      RETURN
      END
      
!-------------------------------------------------------------------------------

      SUBROUTINE MWTS21C_WRITE_DATA (
     &  h_scnlin,
     &  scnlin, scnlinyr, scnlindy, scnlintime,
     &  qualind, scanqual, chanqual, instrtemp1, instrtemp2,
     &  latitude, longitude, satzen, solzen, satazi, solazi,
     &  scalti, satposition, satvelocity, orbitangle, btemps,
     &  btmap, dataqual)
      IMPLICIT NONE

! Parameters
      INTEGER*4 MWTS2_CHANS
      PARAMETER (MWTS2_CHANS=13)
      INTEGER*4 MWTS2_MAPCHANS
      PARAMETER (MWTS2_MAPCHANS=15)
      INTEGER*4 MWTS2_SPOTS
      PARAMETER (MWTS2_SPOTS=90)
      INTEGER*4 UNIT_1C
      PARAMETER (UNIT_1C=10)

! Arguments: 
      INTEGER*4 h_scnlin
      INTEGER*4 scnlin(h_scnlin)
      INTEGER*4 scnlinyr(h_scnlin)
      INTEGER*4 scnlindy(h_scnlin)
      INTEGER*4 scnlintime(h_scnlin)
      INTEGER*4 qualind(h_scnlin)
      INTEGER*4 scanqual(h_scnlin)
      INTEGER*4 chanqual(MWTS2_CHANS,h_scnlin)
      INTEGER*4 instrtemp1(h_scnlin)
      INTEGER*4 instrtemp2(h_scnlin)
      INTEGER*4 latitude(MWTS2_SPOTS,h_scnlin)
      INTEGER*4 longitude(MWTS2_SPOTS,h_scnlin)
      INTEGER*4 satzen(MWTS2_SPOTS,h_scnlin)
      INTEGER*4 satazi(MWTS2_SPOTS,h_scnlin)
      INTEGER*4 solzen(MWTS2_SPOTS,h_scnlin)
      INTEGER*4 solazi(MWTS2_SPOTS,h_scnlin)
      INTEGER*4 scalti(h_scnlin)
      INTEGER*4 satposition(3,h_scnlin)
      INTEGER*4 satvelocity(3,h_scnlin)
      INTEGER*4 orbitangle(h_scnlin)
      INTEGER*4 btemps(MWTS2_CHANS,MWTS2_SPOTS,h_scnlin)
      INTEGER*4 btmap(MWTS2_MAPCHANS,MWTS2_SPOTS,h_scnlin)
      INTEGER*4 dataqual(MWTS2_SPOTS,h_scnlin)

! Local
      INTEGER iscan,ichan,ispot,i

#include "inparms.h"
#include "mwts21c.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Write MWTS2 data records'
      DO iscan=1,h_scnlin
        mwts21c_scnlin = scnlin(iscan)
        mwts21c_scnlinyr = scnlinyr(iscan)
        mwts21c_scnlindy = scnlindy(iscan)
        mwts21c_scnlintime = scnlintime(iscan)
        mwts21c_qualind = qualind(iscan)
        mwts21c_scanqual = scanqual(iscan)
        mwts21c_instrtemp1 = instrtemp1(iscan)
        mwts21c_instrtemp2 = instrtemp2(iscan)
        mwts21c_scalti = scalti(iscan)
        DO ichan=1,MWTS2_CHANS
          mwts21c_chanqual(ichan) = chanqual(ichan,iscan)
        ENDDO
        DO i=1,3
          mwts21c_satposition(i) = satposition(i,iscan)
          mwts21c_satvelocity(i) = satvelocity(i,iscan)
        ENDDO
        mwts21c_orbitangle = orbitangle(iscan)
        DO ispot=1,MWTS2_SPOTS
          DO ichan=1,MWTS2_CHANS
            mwts21c_btemps(ichan,ispot) = btemps(ichan,ispot,iscan)
          ENDDO
          DO ichan=1,MWTS2_MAPCHANS
            mwts21c_btmap(ichan,ispot) = btmap(ichan,ispot,iscan)
          ENDDO
          mwts21c_latlon(1,ispot) = latitude(ispot,iscan)
          mwts21c_latlon(2,ispot) = longitude(ispot,iscan)
          mwts21c_angles(1,ispot) = satzen(ispot,iscan)
          mwts21c_angles(2,ispot) = satazi(ispot,iscan)
          mwts21c_angles(3,ispot) = solzen(ispot,iscan)
          mwts21c_angles(4,ispot) = solazi(ispot,iscan)
          mwts21c_dataqual(ispot) = dataqual(ispot,iscan)
        ENDDO
        CALL IOMWTS21C("WRITE DATA",UNIT_1C,iscan+1)
      ENDDO

      CALL IOMWTS21C("CLOSE",UNIT_1C,0)

      RETURN
      END
      
