! Name: $Id: mwts2_sdr_out.F 592 2018-08-09 14:36:40Z frna $
!
! Purpose: Fill MWTS 1c structure and write out file
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    08/05/2014  Create from mwts_sdr_out.F
!  1.1    27/04/2018  Add inland water category in surface type
!  1.2    30/07/2018  Add FY-3D
!
!-------------------------------------------------------------------------------

      SUBROUTINE MWTS2_SDR_OUT (
     &  Count_scnlines, Orbit_Number, Orbit_Period,
     &  Earth_Obs_BT, Earth_Obs_Angle, Earth_Obs_Angle_all, 
     &  Latitude, Longitude, Time,
     &  SensorZenith, SensorAzimuth, SolarZenith, SolarAzimuth, DEM,
     &  LandSeaMask, Quality_Flag_Channels, Quality_Flag_Scnlin,
     &  Beginning_Date, Beginning_Time, Ending_Date, Ending_Time,
     &  Satellite_Name)
     
      IMPLICIT NONE

      INTEGER   MWTS2_SPOTS, MWTS2_CHANS, MWTS2_SCANS
      PARAMETER (MWTS2_SPOTS=90)
      PARAMETER (MWTS2_CHANS=13)
      PARAMETER (MWTS2_SCANS=2700)

! Arguments: 
      INTEGER*4    Count_scnlines
      INTEGER*4    Orbit_Number
      INTEGER*4    Orbit_Period
      REAL         Latitude(MWTS2_SPOTS,MWTS2_SCANS)
      REAL         Longitude(MWTS2_SPOTS,MWTS2_SCANS)
      REAL         Earth_Obs_Angle(2,MWTS2_SCANS)
      REAL         Earth_Obs_Angle_all(MWTS2_SPOTS,MWTS2_SCANS)
      INTEGER*4    Time(8,MWTS2_SCANS)
      INTEGER*2    Earth_Obs_BT(MWTS2_SPOTS*MWTS2_SCANS*MWTS2_CHANS)
      INTEGER*2    SensorZenith(MWTS2_SPOTS,MWTS2_SCANS)
      INTEGER*2    SensorAzimuth(MWTS2_SPOTS,MWTS2_SCANS)
      INTEGER*2    SolarZenith(MWTS2_SPOTS,MWTS2_SCANS)
      INTEGER*2    SolarAzimuth(MWTS2_SPOTS,MWTS2_SCANS)
      INTEGER*2    DEM(MWTS2_SPOTS,MWTS2_SCANS)
      INTEGER*2    LandSeaMask(MWTS2_SPOTS,MWTS2_SCANS)
      INTEGER*2    Quality_Flag_Channels(MWTS2_SCANS)
      INTEGER*2    Quality_Flag_Scnlin(MWTS2_SCANS)
      CHARACTER*11 Beginning_Date, Ending_Date
      CHARACTER*13 Beginning_Time, Ending_Time
      CHARACTER*6  Satellite_Name

! Local variables

      INTEGER iscan,ispot,ichan,unit,jscan,jchan,iout,id,scan1,scan2,i
      INTEGER num_obs
      INTEGER year, month, day, hour, minute
      INTEGER scanline, scnlinold
      INTEGER firsttime, tdiff, scandiff, scnlintime_old
      INTEGER nbad, nbad_threshold
      DATA    nbad_threshold/30/   !cal assumed bad if nbad>threshold
      INTEGER BTmin(MWTS2_CHANS), BTmax(MWTS2_CHANS)
      REAL    latdiff, londiff, latlondiff, LatEdgediff
      REAL    lat(2), lon(2), x(2), y(2), z(2)
      REAL    xpos(MWTS2_SCANS),ypos(MWTS2_SCANS),zpos(MWTS2_SCANS)
      REAL    second
      REAL    BT
      REAL    scanangle, centreangle, x1, x2, y1, y2, z1, z2, height_km
      REAL    rad, dt, earth_rotation
      REAL    pos_vector(3),vel_vector(3),angle,xpos2mod,ypos2mod
      REAL    RMDI
      DATA    RMDI/-9999.99/
      LOGICAL LatOK, LatMissing
      INTEGER qualind(MWTS2_SCANS)
      DATA qualind / MWTS2_SCANS*0 /
      INTEGER qualval(MWTS2_CHANS+1)
      DATA qualval/2,4,8,16,32,64,128,256,512,1024,2048,4096,8192,16382/  !Last is the sum
      CHARACTER*80 line
      REAL    channel_info(7,MWTS2_CHANS)
      DATA    channel_info /91*0/
      LOGICAL badcal(MWTS2_SCANS), badspot

!Function
      INTEGER    IDAYOFYEAR

! Include
#include "mwts21c.h"
#include "inparms.h"
#include "constants_preproc.h"

! End of declarations
!-----------------------------------------------------------------------

      unit = 10
      CALL IOMWTS21C("OPEN/WRITE",unit,0)
      
      IF (Satellite_Name(1:5) .EQ. "FY-3A") THEN
        mwts21c_h_satid = 1
        mwts21c_h_wmosatid = 520
      ELSE IF (Satellite_Name(1:5) .EQ. "FY-3B") THEN
        mwts21c_h_satid = 2
        mwts21c_h_wmosatid = 521
      ELSE IF (Satellite_Name(1:5) .EQ. "FY-3C") THEN
        mwts21c_h_satid = 3
        mwts21c_h_wmosatid = 522
      ELSE IF (Satellite_Name(1:5) .EQ. "FY-3D") THEN
        mwts21c_h_satid = 4
        mwts21c_h_wmosatid = 523
      ELSE
        mwts21c_h_satid = MDI
        mwts21c_h_wmosatid = MDI
      ENDIF

!Read the MWTS channel info on unit 21
      OPEN (21,form='formatted',status='old')
      id = 0
      DO while (id .NE. mwts21c_h_wmosatid)
        line="     "
        DO while (line(1:5) .ne. "# WMO")
          read(21,'(A)',END=100) line
        ENDDO
        read(21,*) id
        IF (id .eq. mwts21c_h_wmosatid) THEN
          read(21,*) line
          read(21,*) channel_info
        ENDIF
      ENDDO
  100 CLOSE (21)
      DO ichan=1,MWTS2_CHANS
        mwts21c_h_tempradcnv(1,ichan) = 
     &    channel_info(2,ichan)*1.0E13/C_light   !wavenumber*1E6
        mwts21c_h_tempradcnv(2,ichan) = channel_info(4,ichan)*1.0E6  !offset
        mwts21c_h_tempradcnv(3,ichan) = channel_info(5,ichan)*1.0E6  !slope
        mwts21c_h_bandwidth(ichan) = channel_info(3,ichan)*1.0E6     !GHz*1E6
        BTmin(ichan) = channel_info(6,ichan)*100
        BTmax(ichan) = channel_info(7,ichan)*100
      ENDDO

      READ(Ending_Date,'(I4,1X,I2,1X,I2)') year, month, day
      READ(Ending_Time,'(I2,1X,I2,1X,F6.3)') hour, minute, second
      mwts21c_h_enddatayr = year
      mwts21c_h_enddatady = IDAYOFYEAR(year, month, day)
      mwts21c_h_enddatatime = 1000*(hour*3600 + minute*60 + second)
      
      READ(Beginning_Date,'(I4,1X,I2,1X,I2)') year, month, day
      READ(Beginning_Time,'(I2,1X,I2,1X,F6.3)') hour, minute, second
      mwts21c_h_startdatayr = year
      mwts21c_h_startdatady = IDAYOFYEAR(year, month, day)
      mwts21c_h_startdatatime = 1000*(hour*3600 + minute*60 + second)
      
      mwts21c_h_instrument = 18
      mwts21c_h_startorbit = Orbit_Number
      mwts21c_h_endorbit = Orbit_Number
      mwts21c_h_hdrcnt = 1
      mwts21c_h_versnb = MWTS2_1cvnnb
      mwts21c_h_versyr = MWTS2_1cvnyr
      mwts21c_h_versdy = MWTS2_1cvndy
      mwts21c_h_satht = 8360      !nominal
      mwts21c_h_period = Orbit_Period*60
      mwts21c_h_nchan = mwts2_nchan
      mwts21c_h_nmapchan = mwts2_nmapchan
      mwts21c_h_nmapchanused = 0          !No mapping in this program
      mwts21c_h_beamversion = 0
      DO ichan=1,MWTS2_CHANS
        mwts21c_h_beamwidth(ichan) = 0
        mwts21c_h_cutoff(ichan) = 0
        mwts21c_h_nxaverage(ichan) = 0    !No averaging in this program
        mwts21c_h_nyaverage(ichan) = 0
      ENDDO
      
      CALL IOMWTS21C("WRITE HEADER",unit,1)

      num_obs = Count_scnlines*MWTS2_SPOTS
      iout = 0
      mwts21c_scnlin = 0

!Estimate satellite position in Greenwich co-ords - not present in the hdf5
!Could get a more accurate value from the satpos file, but this will do for
!now. Ignore elipticity of earth. Use nominal satellite height.

      rad = rEarth*1000 + mwts21c_h_satht*100   !distance from centre of earth
      DO iscan=1,Count_scnlines
        lat(1) = Latitude(45,iscan)
        lat(2) = Latitude(46,iscan)
        lon(1) = Longitude(45,iscan)
        lon(2) = Longitude(46,iscan)
        IF (lat(1) .LT. 32767) THEN
          call latlon_to_xyz(2,1,lat,lon,x,y,z)
          xpos(iscan) = rad*0.5*(x(1) + x(2))
          ypos(iscan) = rad*0.5*(y(1) + y(2))
          zpos(iscan) = rad*0.5*(z(1) + z(2))
        ELSE
          xpos(iscan) = RMDI
          ypos(iscan) = RMDI
          zpos(iscan) = RMDI
        ENDIF
      ENDDO

!Quality control on the brightness temperatures
!If any channel has BT outside the expected range then set them all to missing.
!If number of bad spots exceeds a threshold then it's probably a calibration
!error, so set missing for !+-3 scan lines.

      DO iscan=1,Count_scnlines
        badcal(iscan) = .false.
        nbad = 0
        jscan = (iscan-1)*MWTS2_SPOTS
        DO ispot=1,MWTS2_SPOTS
          badspot = .false.
          DO ichan=1,MWTS2_CHANS
            jchan = (ichan-1)*num_obs
            IF (Earth_Obs_BT(jchan+jscan+ispot).LT.BTmin(ichan)
     &       .OR.Earth_Obs_BT(jchan+jscan+ispot).GT.BTmax(ichan)) THEN
              badspot = .true.
            ENDIF
          ENDDO
          IF (badspot) THEN
            nbad = nbad + 1
            if (nbad .GT. nbad_threshold) badcal(iscan) = .true.
            DO ichan=1,MWTS2_CHANS
              jchan = (ichan-1)*num_obs
              Earth_Obs_BT(jchan+jscan+ispot) = -32768
            ENDDO
          ENDIF
        ENDDO
      ENDDO
      DO iscan=1,Count_scnlines
        jscan = (iscan-1)*MWTS2_SPOTS
        DO i=-3,3
          IF (iscan+i.GT.0.AND.iscan+i.LT.Count_scnlines) THEN
            IF (badcal(iscan+i)) THEN
              DO ispot=1,MWTS2_SPOTS
                DO ichan=1,MWTS2_CHANS
                  jchan = (ichan-1)*num_obs
                  Earth_Obs_BT(jchan+jscan+ispot) = -32768
                ENDDO
              ENDDO
            ENDIF
          ENDIF
        ENDDO
      ENDDO
          
!Main loop

      DO iscan=1,Count_scnlines
        mwts21c_qualind = Quality_Flag_Channels(iscan)
        mwts21c_scanqual = Quality_Flag_Scnlin(iscan)
        scnlintime_old = mwts21c_scnlintime
        mwts21c_scnlintime = Time(7,iscan) + Time(6,iscan)*1000 + 
     &          Time(5,iscan)*60000 + Time(4,iscan)*3600000
        
        mwts21c_scnlinyr = Time(1,iscan)
        mwts21c_scnlindy = IDAYOFYEAR(Time(1,iscan), Time(2,iscan), 
     &             Time(3,iscan))
        IF (mwts21c_scnlintime .GE. 0) THEN

          IF (mwts21c_scnlin .EQ. 0) THEN
            firsttime = mwts21c_scnlintime
          ENDIF
          
          tdiff = mwts21c_scnlintime - firsttime
          IF (tdiff .LT. -43200000) THEN 
            tdiff = tdiff + 86400000
          ENDIF
          mwts21c_scnlin = NINT(tdiff/2666.7) + 1

          !Check that the latitudes are sensible: latitude difference across the
          !scan and also the difference from one scan to the next. 
          LatOK = .TRUE.
          LatMissing = .FALSE.
          DO ispot=1,MWTS2_SPOTS
            IF (.not. ABS(Latitude(ispot,iscan)).LE.90.0) THEN
              LatOK = .FALSE.
              LatMissing = .TRUE.
            ENDIF
          ENDDO
          IF (LatOK) THEN
            !Check latitude difference across the scan
            LatEdgediff = Latitude(1,iscan)-Latitude(90,iscan)
            LatEdgediff = LatEdgediff*cos(Latitude(45,iscan)*deg2rad)
          
            IF (Satellite_Name(1:5) .EQ. "FY-3C") THEN
              IF (LatEdgediff .GT. -1.0 .OR. LatEdgediff .LT. -5.0) THEN
                write(*,*) 'LatEdgediff=',LatEdgediff
                LatOK = .FALSE.
              ENDIF
            ELSE          !FY-3D has right to left scan
              IF (LatEdgediff .GT. 5.0 .OR. LatEdgediff .LT. 1.0) THEN
                write(*,*) 'LatEdgediff=',LatEdgediff
                LatOK = .FALSE.
              ENDIF
            ENDIF
          ENDIF
          scandiff = mwts21c_scnlin-scnlinold
          IF (LatOK .AND. iout.GT.0) THEN
            DO ispot=1,MWTS2_SPOTS
              latdiff =
     &           Latitude(ispot,iscan) - 1E-4*mwts21c_latlon(1,ispot)
              londiff =
     &           Longitude(ispot,iscan) - 1E-4*mwts21c_latlon(2,ispot)
              IF (londiff .GT. 180.0) THEN
                londiff = londiff - 360.0
              ELSE IF (londiff .LT. -180) THEN
                londiff = londiff + 360.0
              ENDIF
              latlondiff = sqrt(latdiff**2 + 
     &              (londiff*cos(deg2rad*Latitude(ispot,iscan)))**2)
              IF (latlondiff .GT. 0.5*scandiff) THEN
                LatOK = .FALSE.
              ENDIF
            ENDDO
          ENDIF
          IF (LatOK) THEN

            mwts21c_instrtemp1 = MDI
            mwts21c_instrtemp2 = MDI
            DO ispot=1,MWTS2_SPOTS
              mwts21c_dataqual(ispot) = 0
              mwts21c_latlon(1,ispot) = 1E4*Latitude(ispot,iscan)
              mwts21c_latlon(2,ispot) = 1E4*Longitude(ispot,iscan)
              mwts21c_angles(1,ispot) = SensorZenith(ispot,iscan)
              mwts21c_angles(2,ispot) = SensorAzimuth(ispot,iscan)
              mwts21c_angles(3,ispot) = SolarZenith(ispot,iscan)
              mwts21c_angles(4,ispot) = SolarAzimuth(ispot,iscan)
              mwts21c_surface(1,ispot) = DEM(ispot,iscan)
              IF (LandSeaMask(ispot,iscan).EQ.3) THEN
                mwts21c_surface(2,ispot) = 0                 !Sea
              ELSE IF (LandSeaMask(ispot,iscan).EQ.1) THEN
                mwts21c_surface(2,ispot) = 2                 !Land
              ELSE IF (LandSeaMask(ispot,iscan).EQ.5) THEN
                mwts21c_surface(2,ispot) = 1                 !Mixed
              ELSE IF (LandSeaMask(ispot,iscan).EQ.2) THEN
                mwts21c_surface(2,ispot) = 3                 !Inland water
              ELSE
                mwts21c_surface(2,ispot) = MDI
              ENDIF              
            ENDDO
            jscan = (iscan-1)*MWTS2_SPOTS
            DO ichan=1,MWTS2_CHANS
              jchan = (ichan-1)*num_obs
              DO ispot=1,MWTS2_SPOTS
                BT = Earth_Obs_BT(jchan+jscan+ispot)*0.01
                IF (BT .GT. 0) THEN
                  mwts21c_btemps(ichan,ispot) = 
     &               Earth_Obs_BT(jchan+jscan+ispot)
                ELSE
                  mwts21c_btemps(ichan,ispot) = MDI
                ENDIF
                IF (BT.LT.AminBT .OR. BT.GT.AmaxBT .OR.
     &               mwts21c_qualind .LT. 0) THEN
                  mwts21c_dataqual(ispot) = mwts21c_dataqual(ispot) + 
     &               qualval(ichan)
                  IF (ichan.EQ.MWTS2_CHANS .AND. 
     &               mwts21c_dataqual(ispot).EQ.qualval(ichan+1)) THEN
                    mwts21c_dataqual(ispot) = mwts21c_dataqual(ispot)+1  !All channels bad
                  ENDIF
                ENDIF
              ENDDO
            ENDDO
          
!Estimate the satellite height using the scan angle.
!For FY-3C, assume that Earth_Obs_Angle is from start of pixel 1 to end of pixel 90.
!For FY-3D, all angles are given.

            IF (Satellite_Name(1:5) .EQ. "FY-3C") THEN
              scanangle = 0.5*deg2rad*(Earth_Obs_Angle(2,iscan) - 
     &                               Earth_Obs_Angle(1,iscan))*89.0/90.0
            ELSE          !FY-3D 
              scanangle = 0.5*deg2rad*(Earth_Obs_Angle_all(90,iscan) - 
     &                               Earth_Obs_Angle_all(1,iscan))
            ENDIF
            IF (scanangle .gt. 0) THEN
              x1 = cos(Latitude(1,iscan)*deg2rad) * 
     &             cos(Longitude(1,iscan)*deg2rad)
              y1 = cos(Latitude(1,iscan)*deg2rad) * 
     &             sin(Longitude(1,iscan)*deg2rad)
              z1 = sin(Latitude(1,iscan)*deg2rad)

              x2 = cos(Latitude(90,iscan)*deg2rad) * 
     &             cos(Longitude(90,iscan)*deg2rad)
              y2 = cos(Latitude(90,iscan)*deg2rad) * 
     &             sin(Longitude(90,iscan)*deg2rad)
              z2 = sin(Latitude(90,iscan)*deg2rad)

              centreangle = 0.5*(acos(x1*x2 + y1*y2 + z1*z2))
            
              height_km = 
     &           rEarth*(sin(scanangle+centreangle)/sin(scanangle)-1.0)

              mwts21c_scalti = 10.0*height_km + 0.5
              
            ELSE
              mwts21c_scalti = MDI
            ENDIF

!Satellite position and velocity 
            
            IF (xpos(iscan) .NE. RMDI) THEN
              mwts21c_satposition(1) = xpos(iscan)
              mwts21c_satposition(2) = ypos(iscan)
              mwts21c_satposition(3) = zpos(iscan)
            ELSE
              mwts21c_satposition(1) = MDI
              mwts21c_satposition(2) = MDI
              mwts21c_satposition(3) = MDI
            ENDIF

            IF (iscan .EQ. 1) THEN
              scan1 = iscan
              scan2 = iscan + 1
            ELSE IF (iscan .EQ. Count_scnlines) THEN
              scan1 = iscan - 1
              scan2 = iscan
            ELSE
              scan1 = iscan - 1
              scan2 = iscan + 1
              IF (xpos(iscan-1).EQ.RMDI) scan1 = iscan
              IF (xpos(iscan+1).EQ.RMDI) scan2 = iscan
            ENDIF

            IF (scan1 .NE. scan2 .AND. xpos(scan1).NE.RMDI .AND. 
     &          xpos(scan2).NE.RMDI) THEN
              dt = Time(7,scan2)*0.001 + Time(6,scan2) 
     &           - Time(7,scan1)*0.001 - Time(6,scan1)   !ms and seconds
              IF (dt .LT. 0) dt = dt + 60.0

              earth_rotation = dt/86400.0 * 2 * PI   !angle is small
              xpos2mod = xpos(scan2) - ypos(scan2)*earth_rotation
              ypos2mod = ypos(scan2) + xpos(scan2)*earth_rotation

              mwts21c_satvelocity(1) = 
     &          (xpos2mod - xpos(scan1))/dt
              mwts21c_satvelocity(2) = 
     &          (ypos2mod - ypos(scan1))/dt
              mwts21c_satvelocity(3) = 
     &          (zpos(scan2) - zpos(scan1))/dt

              pos_vector(1) = mwts21c_satposition(1)
              pos_vector(2) = mwts21c_satposition(2)
              pos_vector(3) = mwts21c_satposition(3)
              vel_vector(1) = mwts21c_satvelocity(1)
              vel_vector(2) = mwts21c_satvelocity(2)
              vel_vector(3) = mwts21c_satvelocity(3)

              CALL orbit_angle (1,mwts21c_scnlinyr,mwts21c_scnlindy,0,
     &            mwts21c_scnlintime,pos_vector,vel_vector,angle)
              mwts21c_orbitangle = angle*100
            ELSE
              mwts21c_orbitangle = MDI
            ENDIF

            scnlinold = mwts21c_scnlin
            iout = iout + 1
            CALL IOMWTS21C("WRITE DATA",unit,iout+1)
          ELSE
            IF (LatMissing) THEN
              write(*,*)'Missing latitude at iscan=',iscan
            ELSE
              write(*,*)'Bad latitude at iscan=',iscan
            ENDIF
          ENDIF
        ELSE
          write(*,*)'Bad time at iscan=',iscan,mwts21c_scnlintime
        ENDIF
      ENDDO
      
      WRITE(*,*)'Number of scan lines in input file: ', Count_scnlines
      WRITE(*,*)'Number of scan lines written to output file: ', iout
      mwts21c_h_scnlin = iout
      mwts21c_h_misscnlin = mwts21c_scnlin - mwts21c_h_scnlin
      CALL IOMWTS21C("WRITE HEADER",unit,1)

      CALL IOMWTS21C("CLOSE",unit,0)

      IF (iout .EQ. 0) CALL AAPPEXIT(1)

      RETURN
      END

