! Name: $Id: mwts_sdr_out.F 336 2012-10-15 14:00:57Z frna $
!
! Purpose: Fill MWTS 1c structure and write out file
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    30/11/2010  Original code. (N.C.Atkinson)
!  1.1    15/10/2012  Fix channel info read. 
!
!-------------------------------------------------------------------------------

      SUBROUTINE MWTS_SDR_OUT (
     &  Count_scnlines, Orbit_Number, Orbit_Period,
     &  Earth_Obs_BT, Latitude, Longitude, Scan_Time,
     &  Ave_Instr_Temp, EVS_orb_pos, Cal_Coefficient, cal_qc,
     &  SensorZenith, SensorAzimuth, SolarZenith, SolarAzimuth, Height,
     &  LandSeaMask, Black_Body_View_Ang, Satellite_Name,
     &  Beginning_Date, Beginning_Time, Ending_Date, Ending_Time)
     
      IMPLICIT NONE

      INTEGER   MWTS_SPOTS, MWTS_CHANS, MWTS_SCANS
      PARAMETER (MWTS_SPOTS=15)
      PARAMETER (MWTS_CHANS=4)
      PARAMETER (MWTS_SCANS=500)

! Arguments: 
      INTEGER*4    Count_scnlines
      INTEGER*4    Orbit_Number
      INTEGER*4    Orbit_Period
      REAL         Earth_Obs_BT(MWTS_CHANS*MWTS_SCANS*MWTS_SPOTS)
      REAL         Latitude(MWTS_SPOTS,MWTS_SCANS)
      REAL         Longitude(MWTS_SPOTS,MWTS_SCANS)
      REAL         Ave_Instr_Temp(4)
      REAL         EVS_orb_pos(3,MWTS_SCANS)
      INTEGER*4    Scan_Time(MWTS_SCANS)
      INTEGER*4    Cal_Coefficient(3,MWTS_CHANS,MWTS_SCANS)
      INTEGER*2    SensorZenith(MWTS_SPOTS,MWTS_SCANS)
      INTEGER*2    SensorAzimuth(MWTS_SPOTS,MWTS_SCANS)
      INTEGER*2    SolarZenith(MWTS_SPOTS,MWTS_SCANS)
      INTEGER*2    SolarAzimuth(MWTS_SPOTS,MWTS_SCANS)
      INTEGER*2    Height(MWTS_SPOTS,MWTS_SCANS)
      INTEGER*2    LandSeaMask(MWTS_SPOTS,MWTS_SCANS)
      INTEGER*2    Black_Body_View_Ang(MWTS_SCANS)
      INTEGER*2    cal_qc(MWTS_SCANS)
      CHARACTER*6  Satellite_Name
      CHARACTER*11 Beginning_Date, Ending_Date
      CHARACTER*13 Beginning_Time, Ending_Time

! Local variables

      INTEGER iscan,ispot,ichan,unit,jscan,jchan,iout,id
      INTEGER num_obs
      INTEGER year, month, day, hour, minute
      INTEGER scanline, scnlinold
      INTEGER scantime, firsttime, tdiff, scandiff
      REAL    Black_Body_View_Ang_real(MWTS_SCANS), bbref
      REAL    latdiff, londiff, latlondiff, LatEdgediff
      REAL    second
      REAL    BT
      REAL    Cal_slope(MWTS_SCANS), Cal_slope_diff(MWTS_SCANS) 
      REAL    Cal_slope_ref, Cal_slope_limit
      REAL    Cal_slope_range(MWTS_CHANS)
      LOGICAL LatOK, LatMissing
      LOGICAL BBOK
      INTEGER qualind(MWTS_SCANS)
      DATA qualind / MWTS_SCANS*0 /
      INTEGER qualval(MWTS_CHANS+1)
      DATA qualval/2,4,8,16,30/  !Last is the sum
      REAL    xysq,z,lat,semiMinor,r_sat,r_ellipse
      CHARACTER*80 line
      REAL    channel_info(5,MWTS_CHANS)
      DATA    channel_info /20*0/

      !Variables for moon calculation
      REAL    midlat(2)         !Latitude of samples 7 and 9
      REAL    midlon(2)         !Longitude of samples 7 and 9
      REAL    azcorr            !Azimuth correction (radians)
      REAL    thour             !Time of day in hours
      REAL*8  tdays             !Time of scan in days from 12Z, 1 Jan 2000
      REAL    ra                !right ascension of moon (radians)
      REAL    dec               !moon declination
      REAL    sindec            !sine of moon declination
      REAL    cosdec            !cosine of moon declination
      REAL    amsual            !MWTS space view altitude
      REAL    sinamsual         !sine of MWTS space view altitude
      REAL    cosamsual         !cosine of MWTS space view altitude
      REAL    angle_diff        !Angle between moon and sp view (deg)
      REAL    angle_diff_min, angle_diff_max
      DATA    angle_diff_min/9999.9/
      DATA    angle_diff_max/0.0/
      INTEGER num_moon_reject
      DATA    num_moon_reject/0/
      INTEGER num_cal_reject(MWTS_CHANS)
      DATA    num_cal_reject/MWTS_CHANS*0/
      
!Function
      INTEGER    IDAYOFYEAR

! Include
#include "mwts1c.h"
#include "inparms.h"
#include "constants_preproc.h"

! End of declarations
!-----------------------------------------------------------------------

      unit = 10
      CALL IOMWTS1C("OPEN/WRITE",unit,0)
      
      semiMinor = semiMajor - semiDiff

      IF (Satellite_Name(1:5) .EQ. "FY-3A") THEN
        mwts1c_h_satid = 1
        mwts1c_h_wmosatid = 520
      ELSE IF (Satellite_Name(1:5) .EQ. "FY-3B") THEN
        mwts1c_h_satid = 2
        mwts1c_h_wmosatid = 521
      ELSE
        mwts1c_h_satid = MDI
        mwts1c_h_wmosatid = MDI
      ENDIF

!Read the MWTS channel info on unit 21
      OPEN (21,form='formatted',status='old')
      id = 0
      DO while (id .NE. mwts1c_h_wmosatid)
        line="     "
        DO while (line(1:5) .ne. "# WMO")
          read(21,'(A)',END=100) line
        ENDDO
        read(21,*) id
        IF (id .eq. mwts1c_h_wmosatid) THEN
          read(21,*) line
          read(21,*) channel_info
        ENDIF
      ENDDO
  100 CLOSE (21)
      DO ichan=1,MWTS_CHANS
        mwts1c_h_tempradcnv(1,ichan) = 
     &    channel_info(2,ichan)*1.0E13/C_light   !wavenumber*1E6
        mwts1c_h_tempradcnv(2,ichan) = channel_info(4,ichan)*1.0E6  !offset
        mwts1c_h_tempradcnv(3,ichan) = channel_info(5,ichan)*1.0E6  !slope
        mwts1c_h_bandwidth(ichan) = channel_info(3,ichan)*1.0E6     !GHz*1E6
      ENDDO

      READ(Ending_Date,'(I4,1X,I2,1X,I2)') year, month, day
      READ(Ending_Time,'(I2,1X,I2,1X,F6.3)') hour, minute, second
      mwts1c_h_enddatayr = year
      mwts1c_h_enddatady = IDAYOFYEAR(year, month, day)
      mwts1c_h_enddatatime = 1000*(hour*3600 + minute*60 + second)
      
      READ(Beginning_Date,'(I4,1X,I2,1X,I2)') year, month, day
      READ(Beginning_Time,'(I2,1X,I2,1X,F6.3)') hour, minute, second
      mwts1c_h_startdatayr = year
      mwts1c_h_startdatady = IDAYOFYEAR(year, month, day)
      mwts1c_h_startdatatime = 1000*(hour*3600 + minute*60 + second)
      
      mwts1c_h_instrument = 15
      mwts1c_h_startorbit = Orbit_Number
      mwts1c_h_endorbit = Orbit_Number
      mwts1c_h_hdrcnt = 1
      mwts1c_h_versnb = mwts_1cvnnb
      mwts1c_h_versyr = mwts_1cvnyr
      mwts1c_h_versdy = mwts_1cvndy
      mwts1c_h_satht = 8350      !nominal
      mwts1c_h_period = Orbit_Period*60
      
      CALL IOMWTS1C("WRITE HEADER",unit,1)

      num_obs = Count_scnlines*MWTS_SPOTS
      iout = 0
      mwts1c_scnlin = 0

!Check the black body angles. Expect +-100 from the median, approx. If outside
!this range, set the "do not use" flag for this and adjacent scans
          
      DO iscan=1,Count_scnlines
        Black_Body_View_Ang_real(iscan) = Black_Body_View_Ang(iscan)
      ENDDO
      CALL amedian(Black_Body_View_Ang_real,Count_scnlines,bbref)
C      write(*,*)'Median BB view angle=',bbref
      
      DO iscan=1,Count_scnlines
        BBOK = .true.
C        IF (ABS(Black_Body_View_Ang(iscan)-bbref).GT.100) THEN
C          BBOK = .false.
C        ENDIF
        IF (.not. BBOK) THEN
          WRITE(*,*)'Bad BB view angle at iscan=',iscan
C          DO jscan=iscan-2,iscan+3
C            qualind(jscan) = -2147483648     !set bit 31
C          ENDDO
        ENDIF
        IF (cal_qc(iscan) .NE. 0) THEN
          WRITE(*,*)'cal_qc flag set at iscan=',iscan
          qualind(iscan) = -2147483648     !set bit 31
        ENDIF
      ENDDO

!Check the calibration slopes, for those scans not already flagged.
!In some error modes, the slope is anomalously low.

      DO ichan=1,MWTS_CHANS
        jscan = 0
        DO iscan=1,Count_scnlines
          IF (cal_qc(iscan) .EQ. 0) THEN
            jscan = jscan + 1
            Cal_slope(jscan) = Cal_Coefficient(2,ichan,iscan)
          ENDIF
        ENDDO
        IF (jscan .GT. 0) THEN
          CALL amedian(Cal_slope,jscan,Cal_slope_ref)
          Cal_slope_limit = 0.99*Cal_slope_ref
        ENDIF        
        jscan = 0
        DO iscan=1,Count_scnlines
          IF (cal_qc(iscan) .EQ. 0) THEN
            jscan = jscan + 1
            Cal_slope_diff(jscan) = ABS(Cal_slope(jscan)-Cal_slope_ref)
            IF (Cal_slope(jscan).LT.Cal_slope_limit) THEN
              WRITE(*,*)'Bad cal slope at iscan=',iscan,
     &            ', channel',ichan
              num_cal_reject(ichan) = num_cal_reject(ichan) + 1
              qualind(iscan) = -2147483648     !set bit 31
            ENDIF
          ENDIF
        ENDDO
        IF (jscan .GT. 0) THEN
          CALL amedian(Cal_slope_diff,jscan,Cal_slope_range(ichan))
          Cal_slope_range(ichan) = 
     &      Cal_slope_range(ichan)/Cal_slope_ref   !normalise
        ENDIF
      ENDDO

!Main loop

      DO iscan=1,Count_scnlines
        mwts1c_qualind = qualind(iscan)
        mwts1c_scanqual = cal_qc(iscan)
        scantime = Scan_Time(iscan)
        IF (scantime .GT. 0) THEN

          IF (mwts1c_scnlin .EQ. 0) THEN
            firsttime = scantime
          ENDIF
          
          tdiff = scantime - firsttime
          IF (tdiff .LT. -43200000) THEN 
            tdiff = tdiff + 86400000
          ENDIF
          mwts1c_scnlin = NINT(tdiff/16000.0) + 1

          mwts1c_scnlintime = scantime + 43200000   !12 hour offset
          IF (mwts1c_scnlintime .GE. 86400000)
     &       mwts1c_scnlintime = mwts1c_scnlintime - 86400000
          IF (mwts1c_scnlintime - mwts1c_h_startdatatime .LT. 
     &             -43200000) THEN
            mwts1c_scnlindy = mwts1c_h_enddatady
            mwts1c_scnlinyr = mwts1c_h_enddatayr
          ELSE
            mwts1c_scnlindy = mwts1c_h_startdatady
            mwts1c_scnlinyr = mwts1c_h_startdatayr            
          ENDIF

          !Check that the latitudes are sensible. Expect max diff of 1 deg 
          !from one scan line to the next
          LatOK = .TRUE.
          LatMissing = .FALSE.
          DO ispot=1,MWTS_SPOTS
            IF (ABS(Latitude(ispot,iscan)).GT.90.0) THEN
              LatOK = .FALSE.
              LatMissing = .TRUE.
            ENDIF
          ENDDO
          !Check latitude difference across the scan
          LatEdgediff = Latitude(1,iscan)-Latitude(15,iscan)
          LatEdgediff = LatEdgediff*cos(Latitude(8,iscan)*deg2rad)
          IF (LatEdgediff .GT. -1.0 .OR. LatEdgediff .LT. -5.0) THEN
            LatOK = .FALSE.
          ENDIF
          scandiff = mwts1c_scnlin-scnlinold
C          IF (LatOK .AND. iout.GT.0 .AND. scandiff .LT. 5) THEN
          IF (LatOK .AND. iout.GT.0) THEN
            DO ispot=1,MWTS_SPOTS
              latdiff=Latitude(ispot,iscan)-1E-4*mwts1c_latlon(1,ispot)
              londiff=Longitude(ispot,iscan)-1E-4*mwts1c_latlon(2,ispot)
              IF (londiff .GT. 180.0) THEN
                londiff = londiff - 360.0
              ELSE IF (londiff .LT. -180) THEN
                londiff = londiff + 360.0
              ENDIF
              latlondiff = sqrt(latdiff**2 + 
     &              (londiff*cos(deg2rad*Latitude(ispot,iscan)))**2)
              IF (latlondiff .GT. 1.5*scandiff) THEN
                LatOK = .FALSE.
              ENDIF
            ENDDO
          ENDIF
          IF (LatOK) THEN

         !Calculate time in days since 12h 1 Jan 2000 (t=0)
            tdays = INT((mwts1c_scnlinyr-2001)*365.25D0)
     &        + mwts1c_scnlindy + 364.5D0
     &        + mwts1c_scnlintime/86400000.0D0
            thour = mwts1c_scnlintime/3600000.0
         !Calculate moon position
            CALL MOON_POSITION(tdays, ra, dec)
            sindec = sin(dec)
            cosdec = cos(dec)
         !Space view altitude (with moon parallax of 1.04 deg)
            amsual = (-17 + 1.04)*deg2rad      !Guess space view angle
            sinamsual = sin(amsual)
            cosamsual = cos(amsual)
         !Central lat/lon
            midlat(1) = Latitude(7,iscan)
            midlat(2) = Latitude(9,iscan)
            midlon(1) = Longitude(7,iscan)
            midlon(2) = Longitude(9,iscan)
            azcorr = 0.0625    !radians: 1 in 16 approx 
         !Calculate the angular separation between moon and space view
            CALL moon_amsu(midlat,midlon, azcorr, tdays, thour, 
     &         ra, sindec, cosdec, 1, sinamsual, cosamsual, angle_diff)
C            write(*,*) iscan, angle_diff
            angle_diff_min = MIN(angle_diff,angle_diff_min)
            angle_diff_max = MAX(angle_diff,angle_diff_max)
            IF (angle_diff .LT. 3.0) THEN
              num_moon_reject = num_moon_reject + 1
              mwts1c_scanqual = IOR(mwts1c_scanqual, 65536)     !set bit 16
              mwts1c_qualind = IOR(mwts1c_qualind, -2147483648) !set bit 31
            ENDIF

         !Continue with filling the 1c
            mwts1c_instrtemp1 = Ave_Instr_Temp(1)*1E2
            mwts1c_instrtemp2 = MDI
            DO ispot=1,MWTS_SPOTS
              mwts1c_dataqual(ispot) = 0
              mwts1c_latlon(1,ispot) = 1E4*Latitude(ispot,iscan)
              mwts1c_latlon(2,ispot) = 1E4*Longitude(ispot,iscan)
              mwts1c_angles(1,ispot) = SensorZenith(ispot,iscan)
              mwts1c_angles(2,ispot) = SensorAzimuth(ispot,iscan)
              mwts1c_angles(3,ispot) = SolarZenith(ispot,iscan)
              mwts1c_angles(4,ispot) = SolarAzimuth(ispot,iscan)
              mwts1c_surface(1,ispot) = Height(ispot,iscan)
              IF (LandSeaMask(ispot,iscan).EQ.3) THEN
                mwts1c_surface(2,ispot) = 0                 !Sea
              ELSE IF (LandSeaMask(ispot,iscan).EQ.1) THEN
                mwts1c_surface(2,ispot) = 2                 !Land
              ELSE IF (LandSeaMask(ispot,iscan).EQ.5) THEN
                mwts1c_surface(2,ispot) = 1                 !Mixed
              ELSE
                mwts1c_surface(2,ispot) = MDI
              ENDIF              
            ENDDO
            jscan = (iscan-1)*MWTS_SPOTS
            DO ichan=1,MWTS_CHANS
              jchan = (ichan-1)*num_obs
              DO ispot=1,MWTS_SPOTS
C                IF (mwts1c_qualind .LT. 0) THEN
C                  BT = MDI
C                ELSE
                  BT = Earth_Obs_BT(jchan+jscan+ispot)
C                ENDIF
                mwts1c_btemps(ichan,ispot) = 100.0*BT + 0.5
                IF (BT.LT.AminBT .OR. BT.GT.AmaxBT .OR.
     &               mwts1c_qualind .LT. 0) THEN
                  mwts1c_dataqual(ispot) = mwts1c_dataqual(ispot) + 
     &               qualval(ichan)
                  IF (ichan.EQ.MWTS_CHANS .AND. 
     &               mwts1c_dataqual(ispot).EQ.qualval(ichan+1)) THEN
                    mwts1c_dataqual(ispot) = mwts1c_dataqual(ispot) + 1  !All channels bad
                  ENDIF
                ENDIF
              ENDDO
            ENDDO
          
            xysq = EVS_orb_pos(1,iscan)**2 +     
     &             EVS_orb_pos(2,iscan)**2
            z = EVS_orb_pos(3,iscan)
            r_sat = sqrt(xysq + z**2)       !Centre of Earth to satellite (m)
            lat = atan2(z,sqrt(xysq))
            r_ellipse = sqrt((semiMajor*cos(lat))**2 + 
     &                       (semiMinor*sin(lat))**2)
            mwts1c_scalti = NINT((r_sat - r_ellipse)*0.01)    !km *10

            scnlinold = mwts1c_scnlin
            iout = iout + 1
            CALL IOMWTS1C("WRITE DATA",unit,iout+1)
          ELSE
            IF (LatMissing) THEN
              write(*,*)'Missing latitude at iscan=',iscan
            ELSE
              write(*,*)'Bad latitude at iscan=',iscan
            ENDIF
          ENDIF
        ELSE
          write(*,*)'Bad time at iscan=',iscan,scantime
        ENDIF
      ENDDO
      
      WRITE(*,*)'Number of scan lines in input file: ', Count_scnlines
      WRITE(*,*)'Number of scan lines written to output file: ', iout
      WRITE(*,*)'Min/max moon angle: ', angle_diff_min, angle_diff_max
      WRITE(*,*)'Number of moon rejected scans: ', num_moon_reject
      WRITE(*,*)'Number of bad cal slope: ',
     &  (num_cal_reject(ichan),ichan=1,MWTS_CHANS)
      WRITE(*,'(1X,A,4F10.4)') 'Normalised cal slope range:',
     &  (Cal_slope_range(ichan),ichan=1,MWTS_CHANS)

      mwts1c_h_scnlin = iout
      mwts1c_h_misscnlin = mwts1c_scnlin - mwts1c_h_scnlin
      CALL IOMWTS1C("WRITE HEADER",unit,1)

      CALL IOMWTS1C("CLOSE",unit,0)

      IF (iout .EQ. 0) CALL AAPPEXIT(1)

      RETURN
      END

