!+
!NOM/NAME $Id: gp_x16.F 313 2012-02-28 14:51:43Z frna $
 
      subroutine gp_x16(stab,dim,nb,ofg,word16)
!----------------------------------------------------< role >------------
! 
! BUT/PURPOSE:
! extraire un pixel et le cadrer a droite dans 16 bits
! extracts a sample and put it into an integer*2
! DESCRIPTION: FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!
! PROPRIETAIRE/OWNER: PAUL FLAMENT
!
! HISTORIQUE/HISTORY  (MODIFICATIONS/CHANGES)
! version       date            comment                 author
! 1.0        13/03/95                           PAUL FLAMENT
! 2.0        March 03                           T.Labrot
!        f90 test : MOD(ofg,16) doesn't work because 
!                    ofg and 16 have not the same type. 
!                   Create   integer*2 ofg16
!                              parameter (ofg16=16)  
! 2.1        March 04  Introduce w2shift for linux portability. A Korpela
!                      Use ofg16 and of16 in calls to w2shift. N Atkinson
!
! ARGUMENTS:
!     input: stab(dim) ! input array
!            dim       ! dimension of input array
!            nb        ! 1 <= nb <= 16 , number of bits for sample
!            ofg       ! global offset in bits for the first bit 
!                          of the sample
!
!    output: word16    ! to put the sample
!            ofg       ! global offset in bits for the bit following the
!                        last sample extacted from stab array
!                        ( may directly be used as input offset when
!                         the next sample to extract is contiguous to the
!                       previous
!
! FUNCTIONS AND SUBROUTINES CALLS (RETURN CODES FOR FUNCTIONS):
! 
!       MOD, ISHFT, IAND ! Fortran 77
! 
!
! COPYRIGHT(c) Meteo-France ( EVOLUNIX - ATOVS - EUMETSAT project 1994 )
!
!-----------------------------------------------------------------------
 
      implicit none
c-----------------------------------------
c
c

!---- DECLARATIONS -----------------------------------------------------

!---- ARGUMENTS --------------------------------------------------------
      integer*2 dim      ! its dimension 
      integer*2 stab(dim)!input array
      integer*2 nb       ! 1 <= nb <= 16 , number of bits by sample           
      integer*2 ofg      ! global offset in bits for the first bit
      integer*2 ofg16
      parameter (ofg16=16) 
      integer*2 word16    ! word receiving the result
 
!---- COMMONS ----------------------------------------------------------

!---- LOCAL VARIABLES --------------------------------------------------
                     
      integer is    ! index for the sample in array
      integer of    ! offset for the beginning of the sample
!                      in the word of the array
      integer bd    ! bit for beginning of the sample ( left to right)  
      integer*2 bf  ! bit for end of the sample      
      integer*2 c1,c2,c3
      integer*2 msk16(17)
      integer*2 sd,of16  

!---- FUNCTIONS --------------------------------------------------------

      integer*2 w2shift

!---- CODE ------------------------------------------------------------- 

!---- masques
      msk16(1) =Z'0001'
      msk16(2) =Z'0003'
      msk16(3) =Z'0007'
      msk16(4) =Z'000F'
      msk16(5) =Z'001F'
      msk16(6) =Z'003F'
      msk16(7) =Z'007F'
      msk16(8) =Z'00FF'
      msk16(9) =Z'01FF'
      msk16(10)=Z'03FF'
      msk16(11)=Z'07FF'
      msk16(12)=Z'0FFF'
      msk16(13)=Z'1FFF'
      msk16(14)=Z'3FFF'
      msk16(15)=Z'7FFF'
      msk16(16)=Z'FFFF'

!---- index of the word of the beginning of the pixel according to the offset
      is=(ofg/16)+1

!---- offset of the beginning of the sample in the first word
      of=MOD(ofg,ofg16)
      of16=of

!---- decommutation
      bd=of+1                     ! rang bit debut pixel 
      bf=bd+nb-1                  ! rang bit fin  pixel 
      c1=stab(is)                 ! transfert mot debut pixel dans c1
      if(bf.le.16)then            ! si bit fin pixel dans mot courant 
!       write(*,*)'bit fin dans mot courant' 
        if(bf.lt.16)then          ! si bit fin pixel non cadre a droite
          c3=c1 
          c1=W2SHIFT(c3,-(ofg16-bf))  ! cadrage a droite de c1                
        else                      ! sinon, bit fin pixel cadre a droite 
          of=0                    ! pixel suivant en debut de mot suivant       
          is=is+1                 ! incrementation de is    
        endif                     ! fsi
        c3=c1 
        c1=IAND(c3,msk16(nb))     ! masque sur nombre de bits du pixel         
        word16=c1                  ! rangement pixel
      else                        ! cas du chevauchement
!       write(*,*)'chevauchement'
        c3=c1 
        c1=W2SHIFT(c3,(of16+nb)-ofg16) ! shift gauche de c1
        is=is+1                   ! incrementation de is                       
        c2=stab(is)               ! mot suivant du tableau dans c2
        c3=c2  
        sd=-(16-(bf-16))          ! variable intermediaire
        c2=W2SHIFT(c3,sd)         ! shift droit de c2       
        c3=c1
        c1=IOR(c3,c2)             ! OU inclusif de c1 et c2 
        c3=c1 
        c1=IAND(c3,msk16(nb))     ! masque sur nombre de bits                  
        word16=c1                  ! rangement pixel dans word16                           
      endif                       ! fsi

!---- offset de fin de pixel
!---- end offset for the last sample
      ofg=ofg+nb

      return
      end
!------------------------------------------------------------------------

      INTEGER*2 FUNCTION w2shift(w2,cnt)

      IMPLICIT none
!
!  Function to replace calls ISHIFT(w2,cnt) for portability.
!  A. Korpela  Nov-2003
!  Testing with Intel IFC compiler showed a different return value
!  from ISHIFT(w2,cnt) compared to other compilers, when the word w2 is negative.
!
!  Modified 12/3/04 to check bit numbering convention for IBCLR (see ppsetup.F)
!  20/07/2011: Use i2bit to ensure IBCLR arguments are same type. NCA
! 
      INTEGER*2 w2
      INTEGER*2 cnt

      INTEGER*2 i2bit
      INTEGER   iB

      w2shift = ISHFT(w2,cnt)
      IF (w2.LT.0 .AND.cnt.LT.0) THEN
         IF (BTEST(1,0)) THEN       !Bit 0 is least significant
           DO iB = 1,-cnt
             i2bit = 16-iB
             w2shift = IBCLR(w2shift,i2bit)
           ENDDO
         ELSE                      !Bit 0 is most significant
           DO iB = 1,-cnt
             i2bit = iB-1
             w2shift = IBCLR(w2shift,i2bit)
           ENDDO
         ENDIF
      ENDIF

      END
!------------------------------------------------------------------------
