!+
!NOM/NAME $Id: hrsddp.F 25 2006-02-03 15:43:53Z frna $
!
        subroutine hrsddp
!-----------------------------------------------------------------------
!
!BUT/PURPOSE: HiRS 1B Data Display Procedure
!
!DESCRIPTION: FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER: PAUL FLAMENT  
!   
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment                     <author>)
! 1.0       06/05/95  none                        P.FLAMENT
! 2.0       Feb 05    Update for new l1b format   T.LABROT
!         
!ARGUMENTS: 
!               input common: hrs1b.h 
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!    subroutines: 
!                gp_wb16: affichage binaire d'un mot de 16 bits
!                gp_wb32: affichage binaire d'un mot de 32 bits
!                gp_wb8:  affichage binaire d'un mot de 8  bits
! 
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!-----------------------------------------------------------------------

      implicit none                                                     

!---- DECLARATIONS -----------------------------------------------------

!---- ARGUMENTS --------------------------------------------------------

!---- COMMONS ----------------------------------------------------------

#include "hrs1b.h"

!---- LOCAL VARIABLES --------------------------------------------------

       integer*4 ordchn(20)
       data ordchn/1,17,2,3,13,4,18,11,19,7,8,20,10,14,6,5,15,12,16,9/
       integer*4 i,j
       character*12 scnlintime
!---- FUNCTIONS --------------------------------------------------------

!---- CODE ------------------------------------------------------------- 

       write(*,*) ' '
       write(*,*) '             ========================='
       write(*,*) '             HIRS/HRPT 1B  DATA RECORD'
       write(*,*) '             ========================='

       write(*,*) ' '

       write(*,*) 'scan line number ---------------: ',hrs_scnlin
       write(*,*) 'scan line year -----------------: ',hrs_scnlinyr
       write(*,*) 'scan line day of year ----------: ',hrs_scnlindy
       write(*,*) 'satellite clock drift delta ----: ',hrs_clockdrift
       
       call xhqc(hrs_scnlintime,scnlintime) 
       write(*,*) 'scan line UTC time of day ------: ',hrs_scnlintime,
     &                             '  (',scnlintime,')'

       write(*,'('' scan line bit field ------------: '',$)')
       call gp_wb16( hrs_scnlinf )
       
       write(*,*) 'major frame count --------------: ',hrs_mjfrcnt
       write(*,*) 'scan position number in cycle---: ',hrs_scnpos
       write(*,*) 'scan type code -----------------: ',hrs_scntyp
              
       write(*,'('' quality indicator bit field ----: '',$)') 
       call gp_wb32( hrs_qualind )
       write(*,'('' scan line quality flags --------: '',$)') 
       call gp_wb32( hrs_linqualflgs )
       
       do i=1,hrs_mxchn
        write(*,'('' quality flag for each chanel['',i2.2,'']: '',$)')
     &              ordchn(i)
        call gp_wb16( hrs_chqualflg(i) )
       end do
       
       write(*,
     &  '('' minor frame quality indicator bit fields -: '',$)') 
       call gp_wb8( hrs_mnfrqual )
       

       write(*,*) ' '
       write(*,*) 'primary calibration coefficients'
       write(*,*) '================================'
       write(*,*) ' '
       do i=1,hrs_mxchn
        write(*,'(''the second one ['',i2.2,'']: '',$)')ordchn(i)
        write(*,*)hrs_calcof(1,i)*1.e-12
        write(*,'(''the first one  ['',i2.2,'']: '',$)')ordchn(i)
        write(*,*)hrs_calcof(2,i)*1.e-9
        write(*,'(''the zeroth one ['',i2.2,'']: '',$)')ordchn(i)
        write(*,*)hrs_calcof(3,i)*1.e-6
       end do

       write(*,*) ' '
       write(*,*) 'spare calibration coefficients'
       write(*,*) '=============================='
       write(*,*) ' '
       do i=1,hrs_mxchn
        write(*,'(''the second one ['',i2.2,'']: '',$)')ordchn(i)
        write(*,*)hrs_scalcof(1,i)*1.e-12
        write(*,'(''the first one  ['',i2.2,'']: '',$)')ordchn(i)
        write(*,*)hrs_scalcof(2,i)*1.e-9
        write(*,'(''the zeroth one ['',i2.2,'']: '',$)')ordchn(i)
        write(*,*)hrs_scalcof(3,i)*1.e-6
       end do
       
       write(*,*) ' '
       write(*,*) 'navigation'
       write(*,*) '=========='
       write(*,*) ' '
       write(*,'('' navigation status bit field      : '',$)')
       call gp_wb32( hrs_navstat )
       
       write(*,*) 'time associated angles (seconds) : ',
     & hrs_attangtime
       write(*,*) 'Roll angle in degrees            : ',
     & hrs_rollang*1.e-3
       write(*,*) 'pitch angle in degrees           : ',
     & hrs_pitchang*1.e-3
       write(*,*) 'yaw  angle in degrees            : ',
     & hrs_yawang*1.e-3       
       write(*,*) 'spacecraft altitude (MSL) in km  : ',
     & hrs_scalti*1.e-1


       write(*,*) ' '
       write(*,*) 'scan angles in degrees'
       write(*,*) '======================'
       write(*,*) ' '
       do i=1,3*hrs_mxpix,3
        write(*,*) 'solar zen: ',hrs_ang(i)*1.e-2,
     & '     sat zen: ',hrs_ang(i+1)*1.e-2,
     & '     local azimuth: ',hrs_ang(i+2)*1.e-2
       end do
 

       write(*,*) ' '
       write(*,*) 'lat/lon in degrees'
       write(*,*) '=================='
       write(*,*) ' '
       do i=1,2*hrs_mxpix,2
        write(*,*) 'lat: ',hrs_pos(i)*1.e-4,
     &  '       lon: ',hrs_pos(i+1)*1.e-4
       end do
       
       write(*,*) ' '
       write(*,*) 'lunar angle'
       write(*,*) '==========='
       write(*,*) ' '
       write(*,*) 'lunar angle :',hrs_moonang*1.e-2
 
       write(*,*) ' '
       write(*,*) 'element data'
       write(*,*) '============'
       write(*,*) ' '

       do i=0,63
        write(*,*) ' '
        write(*,*) 'numero element: ',i
        write(*,*) '--------------'
        write(*,*) ' '
        write(*,'('' word 1 : '',$)')
        call gp_wb16( hrs_elem(1,i) )
        write(*,'('' word 2 : '',$)')
        call gp_wb16( hrs_elem(2,i) )
        write(*,'('' word 23: '',$)')
        call gp_wb16( hrs_elem(23,i) )
        write(*,'('' word 24: '',$)')
        call gp_wb16( hrs_elem(24,i) )
        write(*,*) ' '

        if(i.lt.56)then
         do j=3,22
          write(*,'('' radiometric channel ['',i2.2,'']:'',i5)') 
     &    ordchn(j-2),hrs_elem(j,i)
         end do 
        end if

        if(i.eq.56)then
         do j=3,22
          write(*,'('' positive cal. channel ['',i2.2,'']:'',i5)') 
     &    ordchn(j-2),hrs_elem(j,i)
         end do
        end if

        if(i.eq.57)then
         do j=3,22
          write(*,'('' negative cal. channel ['',i2.2,'']:'',i5)') 
     &    ordchn(j-2),hrs_elem(j,i)
         end do
        end if

        if(i.eq.58)then
         do j=3,7
          write(*,*)'internal warm target #1: ',hrs_elem(j,i)
         end do
         do j=8,12
          write(*,*)'internal warm target #2: ',hrs_elem(j,i)
         end do
         do j=13,17
          write(*,*)'internal warm target #3: ',hrs_elem(j,i)
         end do
         do j=18,22
          write(*,*)'internal warm target #4: ',hrs_elem(j,i)
         end do
        end if

        if(i.eq.59)then
         do j=3,7
          write(*,*)'internal cold target #1: ',hrs_elem(j,i)
         end do
         do j=8,12
          write(*,*)'internal cold target #2 (NOAA KLM)   :'
          write(*,*)'or analog ground 3 (NOAA NN`)        :',
     &    hrs_elem(j,i)
         end do
         do j=13,17
          write(*,*)'internal cold target #3 (NOAA KLM)   :'
          write(*,*)'or internal warm target #5 (NOAA NN`):', 
     &    hrs_elem(j,i)
         end do
         do j=18,22
          write(*,*)'internal cold target #4 (NOAA KLM)   :'
          write(*,*)'or tertiary telescope (NOAA NN`)     :',
     &    hrs_elem(j,i)
         end do
        end if

        if(i.eq.60)then
         do j=3,7
          write(*,*)'filter wheel housing #1       : ',hrs_elem(j,i)
         end do
         do j=8,12
          write(*,*)'filter wheel housing #2       : ',hrs_elem(j,i)
         end do
         do j=13,17
          write(*,*)'filter wheel housing #3       : ',hrs_elem(j,i)
         end do
         do j=18,22
          write(*,*)'filter wheel housing #4       : ',hrs_elem(j,i)
         end do
        end if

        if(i.eq.61)then
         do j=3,7
          write(*,*)'patch temp -------------------------: ',
     &    hrs_elem(j,i)
         end do
         do j=8,12
          write(*,*)'1st stage radiator -----------------: ',
     &    hrs_elem(j,i)
         end do
         do j=13,17
          write(*,*)'filter wheel housing heater current : ',
     &    hrs_elem(j,i)
         end do
         do j=18,22
          write(*,*)'elec.cal.digital to analog converter: ',
     &    hrs_elem(j,i)
         end do
        end if

        if(i.eq.62)then
          write(*,*)'scan mirror temperature ---------: ',hrs_elem(3,i)
          write(*,*)'primary telescope temperature ---: ',hrs_elem(4,i)
          write(*,*)'secondary telescope temperature -: ',hrs_elem(5,i)
          write(*,*)'baseplate temperature -----------: ',hrs_elem(6,i)
          write(*,*)'electronics temperature ---------: ',hrs_elem(7,i)
          write(*,*)'patch temperature full range ----: ',hrs_elem(8,i)
          write(*,*)'scan motor temperature ----------: ',hrs_elem(9,i)
          write(*,*)'filter whell motor temperature --: ',hrs_elem(10,i)
          write(*,*)'cooler housing temperature ------: ',hrs_elem(11,i)
          write(*,*)'patch control power -------------: ',hrs_elem(12,i)
          write(*,*)'scan motor current --------------: ',hrs_elem(13,i)
          write(*,*)'filter motor current ------------: ',hrs_elem(14,i)
          write(*,*)'+15 VDC -------------------------: ',hrs_elem(15,i)
          write(*,*)'-15 VDC -------------------------: ',hrs_elem(16,i)
          write(*,*)'+7.5 VDC ------------------------: ',hrs_elem(17,i)
          write(*,*)'-7.5 VDC ------------------------: ',hrs_elem(18,i)
          write(*,*)'+10 VDC -------------------------: ',hrs_elem(19,i)
          write(*,*)'+5 VDC --------------------------: ',hrs_elem(20,i)
          write(*,*)'analog ground 1 -----------------: ',hrs_elem(21,i)
          write(*,*)'analog ground 2 -----------------: ',hrs_elem(22,i)
        end if

        if(i.eq.63)then
          write(*,*)'line counter: ',hrs_elem(3,i)
          write(*,'('' first status word : '',$)')
          call gp_wb16( hrs_elem(4,i) )
          write(*,'('' second status word: '',$)')
          call gp_wb16( hrs_elem(5,i) )
          do j=6,22
          write(*,*)'data verification binary code: ',hrs_elem(j,i)
          end do
        endif

      end do
      
      write(*,*) ' '
      write(*,*) 'digital B housekeeping telemetry'
      write(*,*) '================================'
      write(*,*) ' '
      
      write(*,'('' digital B telemetry update flags----: '',$)')
      call gp_wb16( hrs_digitbupdatefg)

      write(*,'('' digital B data ---------------------: '',$)')
      call gp_wb16( hrs_digitbwrd)
      
      write(*,*) ' '
      write(*,*) 'analog housekeeping telemetry'
      write(*,*) '============================='
      write(*,*) ' '

      write(*,'('' analog telemetry update flags-------: '',$)')
      call gp_wb32( hrs_analogupdatefg)
      
      do i=1,16
      write(*,*)'hrs_anwrd --------------------------: ',
     &i,'  ',hrs_anwrd(i)
      enddo  
      write(*,*) ' ' 
      write(*,*) '================================================='
      
      return
       end
