#ifdef HAS_DATE_AS_MDY
      subroutine GHG (HG)
C
C=============================================[ documentation ]=========
C
C---------------------------------------------[ identification ]--------
C
C     auteur            : Gerard LEGENDRE
C     date de creation  : 26/08/87
C     support           : CMS:F7BBT.FTN
C     account           : 920
C
C Version 1.1 17/8/04, N C Atkinson, check time() is non-zero (needed for Linux)
C Version 1.2 07/07/05 ghg_vers2.F: This version for compilers that have time()
C       as a character function (e.g. Intel v7.x)
C
C---------------------------------------------[ role ]------------------
C
C     Obtention de l'heure gregorienne machine.
C
C---------------------------------------------[ domaine de validite ]---
C
C     de 00h 00mn 00s 000ms a 23h 59 mn 59s 999ms
C
C---------------------------------------------[ modifications ]---------
C
C     Pascal brunel   1/12/94
c       utilisation de time et ltime (sp fortrans)
c       ceci a pour consequence de supprimer l'acces aux millisecondes.
c       les ms en sortie sont a 0
c
c       Le probleme etait que le s-p cghg.c fait appel a ftime qui est
c       une procedure Berkeley, non entierement compatible
c       avec Solaris 2.x en particulier pour les multi-treads...
c
c       le man de ftime(3) indique :
cNOTES
c     Use of these interfaces should be restricted to only  appli-
c     cations  written  on BSD platforms.  Use of these interfaces
c     with any of the system libraries or in multi-thread applica-
c     tions is unsupported.
C
C---------------------------------------------[ remarques ]-------------
C
C     ************************************************
C     * Ce S/P doit etre adapte a la machine support *
C     ************************************************
C
C---------------------------------------------[ syntaxe d'appel ]-------
C
C     call GHG(HG)
C
C     HG : integer(4) = heure gregorienne.
C
C---------------------------------------------[ variables globales ]----
C
C     neant
C
C---------------------------------------------[ common ]----------------
C
C     neant
C
C---------------------------------------------[ entrees/sorties ]-------
C
C     neant
C
C---------------------------------------------[ modules appeles ]-------
C
C     neant
C
C=============================================[ declarations ]==========
C
      implicit none
c-----------------------------------------
c
c
C
C---------------------------------------------[ parametres ]------------
C
      integer HG(4)
C
C---------------------------------------------[ variables locales ]-----
C
      integer tarray(9)
      character*8 stime      !integer in standard ghg.F
C
C---------------------------------------------[ fonctions appelees ]----
C
      character*8 time         !integer in standard ghg.F
      logical VHQ
c      external cghg !$pragma C (cghg)
C
C=============================================[ instructions ]==========
C
      call SFBBT(.true.)
        stime = time()
C        call ltime(stime, tarray)
C        hg(1) = tarray(3)
C        hg(2) = tarray(2)
C        hg(3) = tarray(1)
        hg(4) = 0
        read(stime,'(I2,1X,I2,1X,I2)') hg(1), hg(2), hg(3)
      if (VHQ(HG)) then
      else
        call SFBBT(.false.)
        call ZHG(HG)
      endif
      return
C
C=============================================[ formats ]===============
C
      end
#else

#ifdef HAS_DATE_AND_TIME

      subroutine GHG (HG)
!
!     AUTHOR : Philippe Marguinaud
!
!     DATE : 17/01/2006
!     HISTORY : 22/11/2006 : Add a call to init function SFBBT
!
!
      integer HG(4)
      integer vl( 8 )
      logical VHQ
      
      call SFBBT(.true.)
      call date_and_time( values = vl )
      
      HG( 1 ) = vl( 5 )
      HG( 2 ) = vl( 6 )
      HG( 3 ) = vl( 7 )
      HG( 4 ) = vl( 8 )

      if (VHQ(HG)) then
      else
        call SFBBT(.false.)
        call ZHG(HG)
      endif
      
      end
      

#else

      subroutine GHG (HG)
C
C=============================================[ documentation ]=========
C
C---------------------------------------------[ identification ]--------
C
C     auteur            : Gerard LEGENDRE
C     date de creation  : 26/08/87
C     support           : CMS:F7BBT.FTN
C     account           : 920
C
C---------------------------------------------[ role ]------------------
C
C     Obtention de l'heure gregorienne machine.
C
C---------------------------------------------[ domaine de validite ]---
C
C     de 00h 00mn 00s 000ms a 23h 59 mn 59s 999ms
C
C---------------------------------------------[ modifications ]---------
C
C     Pascal brunel   1/12/94
c       utilisation de time et ltime (sp fortrans)
c       ceci a pour consequence de supprimer l'acces aux millisecondes.
c       les ms en sortie sont a 0
c
c       Le probleme etait que le s-p cghg.c fait appel a ftime qui est
c       une procedure Berkeley, non entierement compatible
c       avec Solaris 2.x en particulier pour les multi-treads...
c
c       le man de ftime(3) indique :
cNOTES
c     Use of these interfaces should be restricted to only  appli-
c     cations  written  on BSD platforms.  Use of these interfaces
c     with any of the system libraries or in multi-thread applica-
c     tions is unsupported.
C
C---------------------------------------------[ remarques ]-------------
C
C     ************************************************
C     * Ce S/P doit etre adapte a la machine support *
C     ************************************************
C
C---------------------------------------------[ syntaxe d'appel ]-------
C
C     call GHG(HG)
C
C     HG : integer(4) = heure gregorienne.
C
C---------------------------------------------[ variables globales ]----
C
C     neant
C
C---------------------------------------------[ common ]----------------
C
C     neant
C
C---------------------------------------------[ entrees/sorties ]-------
C
C     neant
C
C---------------------------------------------[ modules appeles ]-------
C
C     neant
C
C=============================================[ declarations ]==========
C
      implicit none
c-----------------------------------------
c
c
C
C---------------------------------------------[ parametres ]------------
C
      integer HG(4)
C
C---------------------------------------------[ variables locales ]-----
C
        integer stime, tarray(9)
C---------------------------------------------[ fonctions appelees ]----
C
      integer time
      logical VHQ
c      external cghg !$pragma C (cghg)
C
C=============================================[ instructions ]==========
C
      call SFBBT(.true.)
        stime = time()
        call ltime(stime, tarray)
        hg(1) = tarray(3)
        hg(2) = tarray(2)
        hg(3) = tarray(1)
        hg(4) = 0
c        call cghg(HG)
      if (VHQ(HG)) then
      else
        call SFBBT(.false.)
        call ZHG(HG)
      endif
      return
C
C=============================================[ formats ]===============
C
      end
      
#endif
     
#endif
