        logical function chksatid ( hrptmf )
!/NIWA  logical function chksatid ( av_num , ti_num )
!+
!NOM/NAME $Id: chksatid.F 517 2017-05-03 14:31:39Z frna $
!
!BUT/PURPOSE 
!       checks that the av_num and ti_num from hrpt id and tip id probably
!       identifies the same spacecraft.
!       To be exact a date is necessary, but we want only to control
!       the content of an hrpt minor frame (without its date).
! 
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!       The function searches in the array avh_sc_num the element which
!       has the same value as the input argument, then in the array tip_sc_num.
!       It returns true if the found satid are the same and  not eq to zero
!       false in other cases.
!       
!
!PROPRIETAIRE/OWNER
!       M.Derrien
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0     20/09/96   initial            M. Derrien
! 2.0     03/07/02   modified the test to get "isat" because for some satellites
!                    the avhrr number or the tip number can be the same for
!                    2 different satellites.     (T.Labrot)
! 2.1     24/05/04   More complete TIP minor frame number and satid checking
!                    using the HRPT minor frame passed as the new argument
!                    to this subroutine   (A.Korpela)
! 2.2     23/06/04   Correct i2 problem by introducing i2word4
! 2.3     10-Aug-2004  Check all 5 TIP minor frames (A.Korpela)
! 2.4     02-Sep-2004  Split lines that exceed 72 characters (N.Atkinson)
! 3.0     07/04/04   SATID version 2      P. Brunel         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!ARGUMENTS 
!MOD/NIWA begin
!/NIWA  integer av_num ! avhrr telemetry satellite id
!/NIWA  integer ti_num ! tip   telemetry satellite id
        integer*2 hrptmf(11090)
!MOD/NIWA end
!
!DECLARATIONS
!
!
!
! Common declarations:
#include "satid.h"
!
! Local variables:
        integer ierr         ! rdnoaaid return code =0 OK  else I/O error
        integer isat         ! loop satellite order number
        integer i
        logical test
!MOD/NIWA begin
        integer av_num ! avhrr telemetry satellite id
        integer ti_num ! tip   telemetry satellite id
        integer tsatid(5),tipnum(5)
        logical ltsat,ltipnum,found
        integer*2 i2word120,i2word15,i2word4
        parameter (i2word120=120)
        parameter (i2word15=15)
        parameter (i2word4=4)
!MOD/NIWA end
! 
!-
!
!MOD/NIWA begin
        av_num = IAND(hrptmf(7),i2word120)/8
        ti_num = IAND((hrptmf(106)/i2word4),i2word15)
!MOD/NIWA end
!       checks if it is the first calling sequence and then
!       call the file reading routine
        ierr = 0
        chksatid=.false.
        if(sat_name(1)(1:2).eq.'xx') then
          call rdnoaaid(ierr)
        endif
!       checks the return code
        if(ierr.eq.0) then

!         search the argument value in the common array for the
!         right avhrr telem number and the right tip telem number
!         without taking account of dates

          do i=1,maxsatid
            test=(avh_sc_num(i).eq.av_num).and.
     &           ( tip_sc_num(1,i).eq.ti_num.or.
     &             tip_sc_num(2,i).eq.ti_num )
            if (test) then
              isat=i
              chksatid=.true.
            endif         
          enddo
        endif  

!MOD/NIWA begin
!--------------------------------------------------------------------------
! Check that satid and tip minor frame counter are consistent in all 5
! TIP minor frames
! Copied/modified from genqc.F
! ( genqc will do the test later )
!--------------------------------------------------------------------------
          ltsat = .true.
          ltipnum = .true.
          found = .false.
          do i = 1,5
            tsatid(i) = iand(15,(hrptmf(106+(i-1)*104)/4))
            tipnum(i) = iand(1,hrptmf(108+(i-1)*104)/4)*256 
     &        + hrptmf(109+(i-1)*104)/4
            if (i.GT.1 .AND. i.LE.5) THEN
              if (tipnum(i).EQ.tipnum(i-1)-319)
     &        tipnum(i) = 320+tipnum(i)    ! tipnum range 0-319
            endif
            ltsat = ltsat .AND. ( tsatid(i).eq.tsatid(1) )
            ltipnum = ltipnum .AND. ( tipnum(i).EQ.(tipnum(1)+i-1) )
            found = ltipnum.AND.ltsat
          enddo
        IF (.NOT.found) chksatid=.false.
!MOD/NIWA end
          
        return
        end
