        integer function convl1binst(scid, l1binst)
!+
!NOM/NAME $Id: convl1binst.F 84 2007-05-16 14:52:37Z frna $
!
!BUT/PURPOSE 
!       delivers the order instrument number for a given satellite id
!       and an NOAA level 1b instrument name
!       the name can be of any type
!
!DEINSTRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!       The function searches in the name arrays the element which
!       has the same name as the input argument
!       The function returns the order number 
!       the function returns 0 if the instrument is not found or 
!       if an error occur
!       
!
!PROPRIETAIRE/OWNER
!       P. Brunel
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0     07/04/04  SATID version 2     P. Brunel
! 1.1     14/04/05  Add MHS             N. Atkinson
! 1.2     05/03/07  Change do while loop to avoid array overflow. PB
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!ARGUMENTS 
      integer scid      ! satellite id
      integer l1binst   ! level1b instrument number
            ! 1  avhrr lac
            ! 2  gac
            ! 3  avhrr hrpt
            ! 4  tip
            ! 5  hirs
            ! 6  msu
            ! 7  ssu
            ! 8  dcs
            ! 9  sem
            ! 10 amsua
            ! 11 amsub
            ! 12 mhs
!
!DECLARATIONS
!
!
!
! Common declarations:
#include "satid.h"
!
! functions
        logical cp_cus
! Local variables:
        integer ierr          ! rdnoaaid return code =0 OK  else I/O error
        integer inst          ! loop instrument order number
        character*8 inst_name ! instrument reference name
        logical cont_loop     ! do while loop logical expression
        logical found         ! true if corresponding satellite is found
! 
!-
!
!       checks if it is the first calling sequence and then
!       call the file reading routine
        ierr = 0
        if(sat_name(1)(1:2).eq.'xx') then
          call rdnoaaid(ierr)
        endif

!       checks the return code
        if(ierr.eq.0) then

           if( l1binst .eq. 1 ) then
              inst_name = 'avhrr'
           elseif( l1binst .eq. 2 ) then
              inst_name = 'avhrr'
           elseif( l1binst .eq. 3 ) then
              inst_name = 'avhrr'
           elseif( l1binst .eq. 4 ) then
              inst_name = 'tip'
           elseif( l1binst .eq. 5 ) then
              inst_name = 'hirs'
           elseif( l1binst .eq. 6 ) then
              inst_name = 'msu'
           elseif( l1binst .eq. 7 ) then
              inst_name = 'ssu'
           elseif( l1binst .eq. 8 ) then
              inst_name = 'dcs'
           elseif( l1binst .eq. 9 ) then
              inst_name = 'sem'
           elseif( l1binst .eq. 10 ) then
              inst_name = 'amsua'
           elseif( l1binst .eq. 11 ) then
              inst_name = 'amsub'
           elseif( l1binst .eq. 12 ) then
              inst_name = 'mhs'
           else
              inst_name = 'toto'
           endif

!         search the argument value in the common array for the
!         right avhrr telem number in the good range of dates.
          inst=0
          convl1binst = 0
          cont_loop = .true.
          do while ( cont_loop )
            inst=inst+1
            if ( inst.le.sc_num_inst(scid) ) then
              found= cp_cus(inst_gen_name(inst,scid),inst_name) .or. 
     &               cp_cus(inst_ref_name(inst,scid),inst_name)
            else
              found     = .false.
              cont_loop = .false.
            end if
            if ( found ) then
!             the satellite id is found
              convl1binst = inst
              cont_loop = .false.
            end if
          end do

        else
!         an error occured during the file reading
          convl1binst = 0
        endif

        return
        end
        
