        character*16 function genscnam(sc_name)
!+
!NOM/NAME $Id: genscnam.F 84 2007-05-16 14:52:37Z frna $
!
!BUT/PURPOSE 
!       delivers the generic satellite name for a given satellite name
!       the name can be of any type
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!       The function searches in the name arrays the element which
!       has the same name as the input argument
!       The function returns the generic name  made of the platform
!       name followed by the sc number (2digits) 
!       ex: noaa12  metop01 adeos02
!       the function returns 'xxxxxx' if the satellite is not found or 
!       if an error occur
!       
!
!PROPRIETAIRE/OWNER
!       P. Brunel
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0     13/02/95   initial            P. Brunel
! 2.0     07/04/04  SATID version 2     P. Brunel
! 2.1     05/03/07  Change do while loop to avoid array overflow. PB
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!ARGUMENTS 
        character*(*) sc_name ! satellite name
!
!DECLARATIONS
!
!
!
! Common declarations:
#include "satid.h"
!
! functions
        logical cp_cus
! Local variables:
        integer ierr         ! rdnoaaid return code =0 OK  else I/O error
        integer isat         ! loop satellite order number
        logical cont_loop    ! do while loop logical expression
        logical found        ! true if corresponding satellite
! 
!-
!
!       checks if it is the first calling sequence and then
!       call the file reading routine
        ierr = 0
        if(sat_name(1)(1:2).eq.'xx') then
          call rdnoaaid(ierr)
        endif

!       checks the return code
        if(ierr.eq.0) then

!         search the argument value in the common array for an
!         identical satellite name
          isat=0
          genscnam = 'xxxxxxxxxxxx'
          cont_loop = .true.
          do while ( cont_loop )
            isat=isat+1
            if ( isat.le. maxsatid ) then
              found=  cp_cus(launch_name(isat),sc_name) .or. 
     &                cp_cus(sc_int_des(isat),sc_name)  .or. 
     &                cp_cus(sat_name(isat),sc_name)    .or. 
     &                cp_cus(alias1(isat),sc_name)      .or. 
     &                cp_cus(alias2(isat),sc_name)      .or. 
     &                cp_cus(alias3(isat),sc_name)
            else
              found     = .false.
              cont_loop = .false.
            end if
            if ( found ) then
!             the satellite id is found
              genscnam = sat_name(isat)
              cont_loop = .false.
            end if
          end do

        else
!         an error occured during the file reading
          genscnam = 'xxxxxxxxxxxx'
        endif

        return
        end
        
