        character*6 function noascnam(ord_sc_num)
!+
!NOM/NAME $Id: noascnam.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!       delivers the NOAA satellite name for a given 
!       order number.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!       The function searches in the name arrays the element which
!       has the same name as the input argument
!       The function returns the NOAA name  (noaaxx)
!       The function returns the NOAA name  made of the platform
!       name followed by the sc number (2digits) The name is 
!       truncated to 6 characters (noaaxx). If the lenght of
!       the platform exceeds 4 char noascnam is platform(1:4)//number
!       ex: for Meteosat08 : mete08
!       
!       the function returns 'xxxxxx' if the satellite is not found or 
!       if an error occur
!       
!
!PROPRIETAIRE/OWNER
!       P. Brunel
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0     13/02/95   initial            P. Brunel
! 1.1     23/06/03   change test on maxsatid P. Brunel
! 2.0     07/04/04  SATID version 2     P. Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!       cp_lsc          ! last significant character (f7cp)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!ARGUMENTS 
        integer ord_sc_num ! order number
!
!DECLARATIONS
!
!
!
! Common declarations:
#include "satid.h"
!
! Functions
        integer cp_lsc                  ! last significant character (f7cp)
!
! Local variables:
        integer ierr         ! rdnoaaid return code =0 OK  else I/O error
        integer lens         ! character length of satellite
! 
!-
!
!       checks if it is the first calling sequence and then
!       call the file reading routine
        ierr = 0
        if(sat_name(1)(1:2).eq.'xx') then
          call rdnoaaid(ierr)
        endif

!       checks the return code
        if(ierr.eq.0) then

!         search the argument value in the common array for the
!         right noaa s/c number.
          if( ord_sc_num.gt.0 .and. ord_sc_num.le.maxsatid) then
             lens = cp_lsc(sat_name(ord_sc_num))
             if( lens .le. 6 ) then
                noascnam = sat_name(ord_sc_num)(1:6)
             else
                noascnam = sat_name(ord_sc_num)(1:4)//
     &                     sat_name(ord_sc_num)(lens-1:lens)
             endif
          else
!           the order number is out of range
            noascnam = 'xxxxxx'
          endif

        else
!         an error occured during the file reading
          noascnam = 'xxxxxx'
        endif

        return
        end
        
