        integer function ord1bid(l1bscnum,jdate)
!+
!NOM/NAME $Id: ord1bid.F 84 2007-05-16 14:52:37Z frna $
!
!BUT/PURPOSE 
!       delivers the order satellite number for a given 
!       level 1b id number (PACS number) and a julian date.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!       The function searches in the array avh_sc_num the element which
!       has the same value as the input argument. A date is given to
!       avoid the conflicts between satellites 
!
!       the function returns 0 if the satellite is not found or 
!       if an error occur
!       
!
!PROPRIETAIRE/OWNER
!       M. Derrien
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0     13/02/95   initial            M. Derrien
! 2.0     07/04/04  SATID version 2     P. Brunel
! 2.1     05/01/05  check "isat.le.maxsatid" first, to avoid array overflow. NCA
! 2.2     05/03/07  Change do while loop to avoid array overflow. PB
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!ARGUMENTS 
        integer l1bscnum !  satellite id read from a 1B file
        integer jdate     ! julian date of the data set
!
!DECLARATIONS
!
!
!
! Common declarations:
#include "satid.h"
!
! Local variables:
        integer ierr         ! rdnoaaid return code =0 OK  else I/O error
        integer isat         ! loop satellite order number
        logical cont_loop    ! do while loop logical expression
        logical found        ! true if corresponding satellite
! 
!-
!
!       checks if it is the first calling sequence and then
!       call the file reading routine
        ierr = 0
        if(sat_name(1)(1:2).eq.'xx') then
          call rdnoaaid(ierr)
        endif

!       checks the return code
        if(ierr.eq.0) then

!         search the argument value in the common array for the
!         right avhrr telem number in the good range of dates.
          isat=0
          ord1bid   = 0
          cont_loop = .true.
          do while ( cont_loop )
            isat=isat+1
            if ( isat.le. maxsatid ) then
              found= (l1b_sc_num(isat).eq.l1bscnum  .and. 
     &                jdate.gt.sc_date_range(1,isat) .and.
     &                jdate.lt.sc_date_range(2,isat) )
            else
              found     = .false.
              cont_loop = .false.
            end if
            if ( found ) then
!             the satellite id is found
              ord1bid   = isat
              cont_loop = .false.
            end if
          end do

        else
!         an error occured during the file reading
          ord1bid = 0
        endif

        return
        end
        
