        subroutine rdnoaaid(ierr)
!+
!NOM/NAME $Id: rdnoaaid.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!       reads the satellite description file which contains the
!       correspondance between the possible numbers and names for the
!       same satellites      
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!       rdnoaaid opens the file SATID_FILE (default satid.txt) 
!       on the directory DIR_SATID  (default . ) given
!       by environment variables.
!       It reads the file and creates the generic names for each
!       satellite and converts units.
!       The variables are stored in commons (satid.h)
!
!       man -s 5 satid   for description of file
!
!PROPRIETAIRE/OWNER
!       P. Brunel
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0     13/02/95   initial            P. Brunel
! 2.0     07/04/04  SATID version 2     P. Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!       xdcj    conversion from character date to integer julian date
!       cp_ssf  subsitute the white spaces in a character variable
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!ARGUMENTS 
!       return code
        integer ierr ! =0 OK    else I/O error
!
!DECLARATIONS
!
!
!
! Common declarations:
#include "satid.h"
!
! Functions
! Local parameters
        integer unit         ! unit for file reading
        parameter (unit=99)
        double precision pi
        parameter ( pi = 3.14159265358979311600d0)
        double precision cdr ! degrees to radian conversion
        parameter ( cdr = pi / 180.d0 )
! Local variables:
        integer i, j
        integer inst
        integer isat         ! loop satellite order number
        integer ios          ! iostat I/O check variable
        character*80 chdum   ! dummy character variable
        character*8 chdate1  ! launch date
        character*8 chdate2  ! declared date for end of operations
        character*256 path   ! pathname for the data file
        character*256 satid_file   ! name of the data file
        character*256 filename   ! full path name of the data file

        ! Units definition for conversion
        ! 1 is for mseconds to julian days
        ! 2 is for seconds  to julian days
        ! 10 is for degrees to radians
        ! 11 is for milli radians to radians
        integer unit_inst_scan_mode(maxparascan)
        integer unit_mis_align_mode(maxparamisalign)
        integer unit_ifov_mode(maxparaifov)
        integer unit_att_bias
! 
!-
!
!============== NOT STANDARD FOR ATOV ======START============
!       get the environnement variable for the path of the data file
!       and create the filename

        path=' '
        satid_file = ' '
        call getenv('DIR_SATID',path)

        call getenv('SATID_FILE',satid_file)

        if( path .ne. ' ' ) then
           ! copy path to file name variable
           call cp_ssf(path,filename,' ')
        else
           ! default pathname
           filename='.'
        endif

        if(satid_file.ne.' ') then
           ! add satid_file to file name variable
           call cp_ssf(filename//'/'//satid_file,filename,' ')
        else
           ! default filename
           call cp_ssf(filename//'/satid.txt',filename,' ')
        endif
           
!============== NOT STANDARD FOR ATOV =======END===========
!
        ierr = 0
        isat = 0
! open file and check for I/O errors
! data store is aborted on detection of error
        open (unit, file=filename, status='old', iostat=ios)

        if(ios .eq. 0) then

           ! while end of file
           read(unit,'(a80)',iostat=ios) chdum
           do while ( ios .eq. 0 .and. chdum(1:3).ne. 'END' )

              if( chdum(1:7).eq. 'VERSION' ) then
                 read(unit,*,iostat=ios) satid_version   
              end if ! Version

              if( chdum(1:9).eq. 'SATELLITE' ) then
              if(ios.eq.0 .and. isat .lt. maxsatid ) then
                 isat = isat+1
                 read(unit,'(a)',iostat=ios) sat_serie(isat)
                 read(unit,*,iostat=ios) noa_sc_num(isat)
                 read(unit,'(a)',iostat=ios) sc_int_des(isat)
                 read(unit,'(a)',iostat=ios) launch_name(isat)
                 read(unit,'(a)',iostat=ios) alias1(isat)
                 read(unit,'(a)',iostat=ios) alias2(isat)
                 read(unit,'(a)',iostat=ios) alias3(isat)
                 read(unit,'(a)',iostat=ios) chdate1
                 read(unit,'(a)',iostat=ios) chdate2
                 read(unit,*,iostat=ios) sc_num_inst(isat)
                 

                 !creates the satellite reference name (noaaxx meteosatxx)
                 ! made of the satellite serie followed by 
                 ! 2 digits for the satellite number
                 write(chdum,"(i2.2)") noa_sc_num(isat)
                 call cp_ssf(sat_serie(isat)//chdum(1:2),
     &                               sat_name(isat),' ')

                 ! transform the character dates in julian dates
                 call xdcj(chdate1,sc_date_range(1,isat))
                 call xdcj(chdate2,sc_date_range(2,isat))

              end if
              end if  ! SATELLITE

              if( chdum(1:12).eq. 'NOAA_LEVEL1B' ) then
              if(ios.eq.0 .and. isat .lt. maxsatid .and. 
     &                                        isat.gt.0) then
                 read(unit,*,iostat=ios) l1b_sc_num(isat)
                 read(unit,*,iostat=ios) avh_sc_num(isat)
                 read(unit,*,iostat=ios) tip_sc_num(1,isat), 
     &                                   tip_sc_num(2,isat)
              end if
              end if  ! NOAA_LEVEL1B

              if( chdum(1:9).eq. 'ATTITUDE' ) then
              if(ios.eq.0 .and. isat .lt. maxsatid .and. 
     &                                        isat.gt.0) then
                 read(unit,*,iostat=ios) sc_att_mode(isat)
                 read(unit,*,iostat=ios) sc_att_bias_n(isat)
                 if(sc_att_bias_n(isat) .gt. 0) then
                    do i = 1, sc_att_bias_n(isat)
                       read(unit,*,iostat=ios) 
     &                        sc_att_bias_bid(i,isat), 
     &                        (sc_att_bias(j,i,isat),j=1,3)
                       read(unit,*,iostat=ios) unit_att_bias
                       if( unit_att_bias .eq. 10) then
                       ! convert degrees to radians
                       !------------------------
                          do j = 1, 3
                             sc_att_bias(j,i,isat) = 
     &                          sc_att_bias(j,i,isat) * cdr
                          end do
                       elseif( unit_att_bias .eq. 11) then
                       ! convert milli radians to radians
                       !------------------------
                          do j = 1, 3
                             sc_att_bias(j,i,isat) = 
     &                          sc_att_bias(j,i,isat) / 1.d3
                          end do
                       end if
                    end do
                 end if
              end if
              end if  ! ATTITUDE

              if( chdum(1:10).eq. 'INSTRUMENT' ) then
              if(ios.eq.0 .and. isat .lt. maxsatid .and. 
     &                                          isat.gt.0) then
                 read(unit,*,iostat=ios) inst
                 if(inst.le.maxinst) then
                 read(unit,'(a)',iostat=ios) inst_gen_name(inst,isat)
                 read(unit,'(a)',iostat=ios) inst_ref_name(inst,isat)
                 read(unit,*,iostat=ios) inst_scan_mode(inst,isat)
                 read(unit,*,iostat=ios) inst_scan_mode_n(inst,isat)
                 read(unit,*,iostat=ios) 
     &                   (inst_scan_mode_p(i,inst,isat),
     &                     i=1,inst_scan_mode_n(inst,isat))
                 read(unit,*,iostat=ios) 
     &                   (unit_inst_scan_mode(i),
     &                     i=1,inst_scan_mode_n(inst,isat))

                 read(unit,*,iostat=ios) mis_align_mode(inst,isat)
                 read(unit,*,iostat=ios) mis_align_mode_n(inst,isat)
                 read(unit,*,iostat=ios) 
     &                   (mis_align_mode_p(i,inst,isat),
     &                       i=1,mis_align_mode_n(inst,isat))
                 read(unit,*,iostat=ios) 
     &                   (unit_mis_align_mode(i),
     &                       i=1,mis_align_mode_n(inst,isat))

                 read(unit,*,iostat=ios) ifov_mode(inst,isat)
                 read(unit,*,iostat=ios) ifov_mode_n(inst,isat)
                 read(unit,*,iostat=ios) 
     &                   (ifov_mode_p(i,inst,isat),
     &                       i=1,ifov_mode_n(inst,isat))
                 read(unit,*,iostat=ios) 
     &                   (unit_ifov_mode(i),
     &                       i=1,ifov_mode_n(inst,isat))


                 do i = 1, inst_scan_mode_n(inst,isat)
                    if( unit_inst_scan_mode(i) .eq. 1) then
                       ! convert ms to julian days
                       !---------------------------
                       inst_scan_mode_p(i,inst,isat) = 
     &                  inst_scan_mode_p(i,inst,isat) / 86400.d3
                    elseif( unit_inst_scan_mode(i) .eq. 2) then
                       ! convert seconds to julian days
                       !------------------------
                       inst_scan_mode_p(i,inst,isat) = 
     &                  inst_scan_mode_p(i,inst,isat) / 86400.d0
                    elseif( unit_inst_scan_mode(i) .eq. 10) then
                       ! convert degrees to radians
                       !------------------------
                       inst_scan_mode_p(i,inst,isat) = 
     &                  inst_scan_mode_p(i,inst,isat) * cdr
                    elseif( unit_inst_scan_mode(i) .eq. 11) then
                       ! convert milli radians to radians
                       !------------------------
                       inst_scan_mode_p(i,inst,isat) = 
     &                  inst_scan_mode_p(i,inst,isat) / 1.d3
                    end if
                 end do

                 do i = 1, mis_align_mode_n(inst,isat)
                    if( unit_mis_align_mode(i) .eq. 1) then
                       ! convert ms to julian days
                       !---------------------------
                       mis_align_mode_p(i,inst,isat) = 
     &                  mis_align_mode_p(i,inst,isat) / 86400.d3
                    elseif( unit_mis_align_mode(i) .eq. 2) then
                       ! convert seconds to julian days
                       !------------------------
                       mis_align_mode_p(i,inst,isat) = 
     &                  mis_align_mode_p(i,inst,isat) / 86400.d0
                    elseif( unit_mis_align_mode(i) .eq. 10) then
                       ! convert degrees to radians
                       !------------------------
                       mis_align_mode_p(i,inst,isat) = 
     &                  mis_align_mode_p(i,inst,isat) * cdr
                    elseif( unit_mis_align_mode(i) .eq. 11) then
                       ! convert milli radians to radians
                       !------------------------
                       mis_align_mode_p(i,inst,isat) = 
     &                  mis_align_mode_p(i,inst,isat) / 1.d3
                    end if
                 end do
               
                 do i = 1, ifov_mode_n(inst,isat)
                    if( unit_ifov_mode(i) .eq. 1) then
                       ! convert ms to julian days
                       !---------------------------
                       ifov_mode_p(i,inst,isat) = 
     &                  ifov_mode_p(i,inst,isat) / 86400.d3
                    elseif( unit_ifov_mode(i) .eq. 2) then
                       ! convert degrees to radians
                       !------------------------
                       ifov_mode_p(i,inst,isat) = 
     &                  ifov_mode_p(i,inst,isat) / 86400.d0
                    elseif( unit_ifov_mode(i) .eq. 10) then
                       ! convert degrees to radians
                       !------------------------
                       ifov_mode_p(i,inst,isat) = 
     &                  ifov_mode_p(i,inst,isat) * cdr
                    elseif( unit_ifov_mode(i) .eq. 11) then
                       ! convert degrees to radians
                       !------------------------
                       ifov_mode_p(i,inst,isat) = 
     &                  ifov_mode_p(i,inst,isat) / 1.d3
                    end if
                 end do
                       
              else
                 print *,'maximum instrument number reached'
                 ierr = 1
                 return
              endif

              end if
              end if  ! INSTRUMENT


              read(unit,'(a80)',iostat=ios) chdum
           end do               ! while end of file

           if(ios.ne.0) then
              print *,'Error in reading the noaa satellite desc. file'
              print *,'filename     :',filename
              print *,'IOstat value :',ios
              ierr = ios
              return
           endif
                                !
        else
           print *,'Error in opening the noaa ',
     &                     'satellite description file'
           print *,'filename     :',filename
           print *,'IOstat value :',ios
           print *,'Stage aborted'
           ierr = ios
        endif

        close (unit)

        return
        end
        block data noaaid
!+
!NOM/NAME $Id: rdnoaaid.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!       initialize the common  /c_satid/
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!
!PROPRIETAIRE/OWNER
!       P. Brunel
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0     13/02/95   initial            P. Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!ARGUMENTS 
!
!DECLARATIONS
!
!
! Common declarations:
#include "satid.h"
!
! Data
        integer size1
        parameter(size1=maxinst*maxsatid)
        integer size2
        parameter(size2=maxparascan*maxinst*maxsatid)
        integer size3
        parameter(size3=maxparamisalign*maxinst*maxsatid)
        integer size4
        parameter(size4=maxparaifov*maxinst*maxsatid)
        integer size5
        parameter(size5=maxattbid*maxsatid)
        integer size6
        parameter(size6=3*maxattbid*maxsatid)
!
        data satid_version /0/

        data sat_serie/maxsatid*'xxxxxxxx'/
        data inst_gen_name/size1*'xxxxxxxxxxxxxxxx'/
        data inst_ref_name/size1*'xxxxxxxxxxxxxxxx'/
        data launch_name/maxsatid*'xxxxxxxxxxxxxxxx'/
        data sc_int_des/maxsatid*'xxxxxxxxx'/
        data sat_name/maxsatid*'xxxxxxxxxxxxxxxx'/
        data alias1/maxsatid*'xxxxxxxxxxxxxxxx'/
        data alias2/maxsatid*'xxxxxxxxxxxxxxxx'/
        data alias3/maxsatid*'xxxxxxxxxxxxxxxx'/


        data noa_sc_num/maxsatid*0/
        data l1b_sc_num/maxsatid*-1/
        data avh_sc_num/maxsatid*-1/
        data tip_sc_num/maxsatid*-1,maxsatid*-1/
        data sc_date_range/maxsatid*0,maxsatid*0/

        data sc_num_inst/maxsatid*0/
        data inst_num/size1*0/
        data inst_scan_mode/size1*0/
        data inst_scan_mode_n/size1*0/
        data mis_align_mode/size1*0/
        data mis_align_mode_n/size1*0/
        data ifov_mode/size1*0/
        data ifov_mode_n/size1*0/

        data inst_scan_mode_p/size2*0.d0/
        data mis_align_mode_p/size3*0.d0/
        data ifov_mode_p/size4*0.d0/

        data sc_att_mode/maxsatid*0/
        data sc_att_bias_n/maxsatid*0/
        data sc_att_bias_bid/size5*0.d0/
        data sc_att_bias/size6*0.d0/

        end

        
