#!/bin/ksh
#
#$Id: eccodes_decodebufr_1c.ksh 536 2017-08-09 10:40:30Z frna $
#
#Decode level 1c ATOVS or IASI BUFR files into AAPP binary format, using ecCodes
#
#Usage: aapp_decodebufr_1c [-i files] [-v]
#     -i  specifies input file names.
#         Defaults to "hrsn.bufr aman.bufr ambn.bufr mhsn.bufr iasi.bufr"
#
#Outputs: AAPP files with .l1c suffix
#
#History:
# 1.1     30/03/17  Initial version, based on aapp_decodebufr_1c. N C Atkinson
#==============================================================================

usage(){
  echo
  echo 'usage : aapp_decodebufr_1c [-i files] [instruments]'
  echo
  echo '  where files is a list of files to decode.'
  echo '  Quotes " " are necessary if there is more than 1 file'
  echo
}

. "$AAPP_PREFIX/ATOVS_CONF"

files="hrsn.bufr aman.bufr ambn.bufr mhsn.bufr iasi.bufr"   #default files

while getopts i: c
do
  case $c in
     i)	files=$OPTARG;;
     \?)      usage
              exit 2;;
  esac

done

shift `expr $OPTIND - 1` #Leaves an optional list of instruments (one per file)

rm -f ${FORT}10 ${FORT}20

# fixed data files
ln -sf ${DIR_PREPROC}/fdf.dat ${FORT}20
ln -sf ${DIR_PREPROC}/atms_channels.dat ${FORT}30

for file in $files; do
  if [ -s $file ]; then
    if [ $# != 0 ]; then
      export INSTRUMENT=$1
      shift
    fi
    
    export BUFRDEC_FILE=$file
    echo processing $file
    eccodes_decodebufr_1c.exe

    if [ -s ${FORT}10 ]; then
      set $(head -c 32 ${FORT}10 | od -An -v -i)   #read first 8 words
      instr=${8}
      if [ $instr = 14 ]; then
        outfile=${file%.*}.lpc   #PCIASI
      else
        outfile=${file%.*}.l1c
      fi
      mv ${FORT}10 $outfile
      echo created $outfile
    else
      echo Processing failed for $file
    fi

  fi
done
rm -f ${FORT}20 ${FORT}30
exit
