      subroutine eccodes_get_1c_amsua(ibufr,out1cunit,imessage,
     & nedt_present)
      
#ifdef HAS_LIBECCODES
      use eccodes
#endif
      implicit none

!-----------------------------------------------------------------------
!
! Name: $Id: eccodes_get_1c_amsua.F 601 2018-11-21 15:57:14Z frna $
!
! Purpose: BUFR decoder subroutine for AMSU-A l1c
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 7 December 2016, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, DWD and MeteoFrance.
!
!    Copyright 2017, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0     24/01/2017 Initial
! 1.1     21/11/2018 orbitNumber can change within a message
!
! Code Description:
!   Fortran90, but using FORTRAN77 fixed-formatting, in order to interface with
!   existing AAPP "include" files.
!
! ecCodes subroutines: codes_get
! AAPP subroutines: ioa1c, UNBITSWAP
!
! Declarations:

#include "inparms.h"
#include "ama1c.h"

!Functions
      integer FNSCALEEC, DAYOFYEAR

!Arguments
      integer ibufr
      integer out1cunit
      integer imessage
      logical nedt_present
      
#ifdef HAS_LIBECCODES

!Variables
      integer   Imiss
      parameter (Imiss=-999999)     !AAPP missing value integer

  !Miscellaneous variables 
      integer i, j, irec, iscan, isub, ispot, ichan, jchan
      integer sensor
      integer nsubsets
      integer scans_to_store
      integer scans_to_write
      integer skipscan
      real*8  rtmp

  !Variables to be saved between calls
      integer, save      :: spots_to_write
      character*10, save :: formatstr(2*Anchan)
      real*8  warmnedt_sumsq(Anchan)
      integer warmnedt_num(Anchan)
      DATA warmnedt_sumsq/Anchan*0.0D0/
      DATA warmnedt_num/Anchan*0/

  !AAPP data store
      integer, dimension(:,:), allocatable :: ama1c_dt_store
      integer, dimension(:), allocatable   :: scans
      integer, dimension(:), allocatable   :: spots

  !Arrays that are passed to eccodes
      real*8,    dimension(:), allocatable :: rvalues
      integer*4, dimension(:), allocatable :: ivalues
      character*80 :: descriptor

  !Date/time arrays and scalars
      integer, dimension(:), allocatable :: years
      integer, dimension(:), allocatable :: months
      integer, dimension(:), allocatable :: days
      integer, dimension(:), allocatable :: hours
      integer, dimension(:), allocatable :: minutes
      real*8, dimension(:), allocatable  :: seconds

      real*8  Year,Month,Day      !for passing to DAYOFYEAR
      integer millisecs, jday
      
  !Index variables for AAPP l1c array
      integer, save :: index_scnlin, index_scnlinyr, index_scnlindy,
     & index_scnlintime, index_qualind, index_scanqual, 
     & index_chanqual, index_instrtemp1, index_instrtemp2,
     & index_spare1, index_latlon, index_angles,
     & index_scalti, index_spare2, index_btemps, index_dataqual,
     & index_coldnedt, index_warmnedt, index_filler

  !Arrays for flag table manipulation 
      integer    NumBitsIn          !- size of raw data bit fields
      parameter  (NumBitsIn  = 32)
      integer    NumBitsOut         !- set BUFR bit field width
      parameter  (NumBitsOut = 24)
      integer BIT                   !For use with unbitswap
      integer Mask033030(NumBitsIn) !- bit mask for 033030
      integer Mask033031(NumBitsIn) !- bit mask for 033031
      integer Mask033032(NumBitsIn) !- bit mask for 033032
      integer Mask033033(NumBitsIn) !- bit mask for 033033
      DATA Mask033030 /1,2,3,4,5,6,7,25*0/
      DATA Mask033031 /8*0,1,2,3,4,4*0,5,6,7,8,9,10,11,12,13,14,15,16,
     &                 17,3*0/
      DATA Mask033032 /26*0,1,2,3,4,5,6/
      DATA Mask033033 /0,1,14*0,16,15,14,13,12,11,10,9,8,7,6,5,4,3,2,22/

!-----------------------------------------------------------------------

      call codes_get(ibufr,'satelliteSensorIndicator',sensor)
      if (sensor .ne. 3) then
        write(*,*) 'Skip message. Sensor=',sensor
        return
      endif

      imessage = imessage + 1

      if (imessage .eq. 1) then

!Format string to use with multi-channel descriptors
        do ichan=1,9
          write(formatstr(ichan),'(A)') '(A,I1,A)'
        enddo
        do ichan=10,2*Anchan
          write(formatstr(ichan),'(A)') '(A,I2,A)'
        enddo

!Find the position in the AAPP data record of each variable
        do i=1,array_size_aa1c
          ama1c_dt(i) = i
        enddo
        index_scnlin = ama1c_scnlin
        index_scnlinyr = ama1c_scnlinyr
        index_scnlindy = ama1c_scnlindy
        index_scnlintime = ama1c_scnlintime
        index_qualind = ama1c_qualind
        index_scanqual = ama1c_scanqual
        index_chanqual = ama1c_chanqual(1)
        index_instrtemp1 = ama1c_instrtemp1
        index_instrtemp2 = ama1c_instrtemp2
        index_spare1 = ama1c_spare1(1)
        index_latlon = ama1c_latlon(1,1)
        index_angles = ama1c_angles(1,1)
        index_scalti = ama1c_scalti
        index_spare2 = ama1c_spare2(1)
        index_btemps = ama1c_btemps(1,1)
        index_dataqual = ama1c_dataqual(1)
        index_coldnedt = ama1c_coldnedt(1)
        index_warmnedt = ama1c_warmnedt(1)
        index_filler = ama1c_filler(1)
      
!Initialise some header variables
        call codes_get(ibufr,'bufrHeaderCentre',i)
        if (i .ge. 0 .and. i .lt. 999) then
          write(ama1c_h_site,'(I3.3)') i
        else
          ama1c_h_site = "xxx"
        endif
        call codes_get(ibufr,'bufrHeaderSubCentre',i)
        if (i .ge. 0 .and. i .le. 999) then
          write(ama1c_h_1bsite,'(I3.3)') i
        else
          ama1c_h_1bsite = "xxx"
        endif
        ama1c_h_cfill1 = ' '
        ama1c_h_cfill2 = ' '
        ama1c_h_scnlin = 0
        ama1c_h_hdrcnt = 1
        ama1c_h_startdatatime = -1
        ama1c_h_versnb = ama_1cvnnb
        ama1c_h_versyr = ama_1cvnyr
        ama1c_h_versdy = ama_1cvndy
        ama1c_h_instrument = 10       !AMSU-A
        spots_to_write = 0

        call codes_get(ibufr,'satelliteIdentifier',ama1c_h_wmosatid)
        IF (ama1c_h_wmosatid .ge. 206) THEN
          ama1c_h_satid = ama1c_h_wmosatid-191   !NOAA satellites
        ELSE
          ama1c_h_satid = ama1c_h_wmosatid-2     !METOP
        ENDIF
        IF (ama1c_h_wmosatid .eq. Imiss) ama1c_h_satid = Imiss
        !!call codes_get(ibufr,'orbitNumber',ama1c_h_startorbit)
        call codes_get(ibufr,
     &    'satelliteAntennaCorrectionsVersionNumber',
     &    ama1c_h_vnantennacorr)

  !Nominal height and period are not in the BUFR, so get them from fdf file
        DO J=1,MaxSats
          IF (WMOid(J).EQ.ama1c_h_wmosatid) THEN
            ama1c_h_satht = SatHt(J)*10   !nominal ht, km*10
            ama1c_h_period = OrbitPd(J)   !period, seconds
            ama1c_h_satid = NOAAid(J)     !Satellite ID
          ENDIF
        ENDDO

  !Open the output file and write header

        call ioa1c("OPEN/WRITE", out1cunit, 0)
        call ioa1c("WRITE HEADER", out1cunit, 1)

      endif     !(imessage .eq. 1)

      !!call codes_get(ibufr,'orbitNumber',ama1c_h_endorbit)

  !Read the total number of subsets. Normally an integer multiple of the number
  !of spots, but make the code general in case scans are split across messages. 

      call codes_get(ibufr,'numberOfSubsets',nsubsets)
      scans_to_store = (nsubsets+spots_to_write)/Anfovs + 1    !add 1 scan to be safe

      allocate(ivalues(nsubsets))
      allocate(rvalues(nsubsets))
      allocate(scans(nsubsets))
      allocate(spots(nsubsets))
      allocate(years(nsubsets))
      allocate(months(nsubsets))
      allocate(days(nsubsets))
      allocate(hours(nsubsets))
      allocate(minutes(nsubsets))
      allocate(seconds(nsubsets))
      allocate(ama1c_dt_store(array_size_aa1c,scans_to_store))
      ama1c_dt_store = 0

  !if there is a partial scan left over from previous message, load the data
  !into the first scan
      if (spots_to_write .ne. 0) then
        do i=1,array_size_aa1c
          ama1c_dt_store(i,1) = ama1c_dt(i)
        enddo
      endif

  !Orbit number
      call codes_get(ibufr,'orbitNumber',ivalues)
      if (imessage .eq. 1) ama1c_h_startorbit = ivalues(1)
      ama1c_h_endorbit = ivalues(nsubsets)

  !FOV number
      call codes_get(ibufr,'fieldOfViewNumber',spots)
      do isub = 1,nsubsets
        if (spots(isub).le.0.or.spots(isub).gt.Anfovs) then
          write(*,*)'Error: invalid fov number',spots(isub)
          call aappexit(1)
        endif
      enddo
      if (imessage .eq. 1) then
        if (spots(1).ne.1) then
          skipscan = 1      !ignore initial partial scan if there is one
          spots_to_write = nsubsets + 1 - spots(1)
        else
          skipscan = 0
          spots_to_write = nsubsets
        endif
      else
        skipscan = 0
        spots_to_write = spots_to_write + nsubsets
      endif

  !scan number
      call codes_get(ibufr,'scanLineNumber',ivalues)
      scans(1) = 1
      do isub = 2,nsubsets
        scans(isub) = scans(isub-1)
        if (spots(isub).eq.1) scans(isub) = scans(isub) + 1
      enddo
      do isub = 1,nsubsets
        ama1c_dt_store(index_scnlin,scans(isub)) = ivalues(isub)
      enddo

  !qualind
      call codes_get(ibufr,'scanLineStatusFlagsForAtovs',ivalues)
      do isub = 1,nsubsets,Anfovs
        Bit = ivalues(isub)
        CALL UNBITSWAP(i, Mask033030, NumBitsIn, Bit, NumBitsOut)
        ama1c_dt_store(index_qualind,scans(isub)) = i
      enddo

  !scanqual
      call codes_get(ibufr,'scanLineQualityFlagsForAtovs',ivalues)
      do isub = 1,nsubsets,Anfovs
        Bit = ivalues(isub)
        CALL UNBITSWAP(i, Mask033031, NumBitsIn, Bit, NumBitsOut)
        ama1c_dt_store(index_scanqual,scans(isub)) = i
      enddo

  !date/time
      call codes_get(ibufr,'year',years)
      call codes_get(ibufr,'month',months)
      call codes_get(ibufr,'day',days)
      call codes_get(ibufr,'hour',hours)
      call codes_get(ibufr,'minute',minutes)
      call codes_get(ibufr,'second',seconds)
        
  !transform the date/time variables
      do isub = 1,nsubsets
        if (spots(isub).eq.1) then
          Year = years(isub)
          Month = months(isub)
          Day = days(isub)
          ama1c_dt_store(index_scnlinyr, scans(isub)) = Year
          jday = DAYOFYEAR(Year,Month,Day)
          ama1c_dt_store(index_scnlindy, scans(isub)) = jday
          millisecs = hours(isub)*3600000 + minutes(isub)*60000 
     &                  + seconds(isub)*1000   
          ama1c_dt_store(index_scnlintime, scans(isub)) = millisecs
        endif
      enddo

  !latitude
      call codes_get(ibufr,'latitude',rvalues)
      do isub = 1,nsubsets
        ama1c_dt_store(index_latlon+2*spots(isub)-2, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D4)
      enddo

  !longitude
      call codes_get(ibufr,'longitude',rvalues)
      do isub = 1,nsubsets
        ama1c_dt_store(index_latlon+2*spots(isub)-1, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D4)
      enddo

  !scalti
      call codes_get(ibufr,'heightOfStation',rvalues)
      do isub = 1,nsubsets,Anfovs
        ama1c_dt_store(index_scalti,scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D-2)  !store as km*10
      enddo

  !sat zenith
      call codes_get(ibufr,'satelliteZenithAngle',rvalues)
      do isub = 1,nsubsets
        ama1c_dt_store(index_angles+4*spots(isub)-4, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !sat azimuth (0 to 360 deg)
      call codes_get(ibufr,'bearingOrAzimuth',rvalues)
      do isub = 1,nsubsets
        ama1c_dt_store(index_angles+4*spots(isub)-3, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !solar zenith
      call codes_get(ibufr,'solarZenithAngle',rvalues)
      do isub = 1,nsubsets
        ama1c_dt_store(index_angles+4*spots(isub)-2, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !solar azimuth (0 to 360 deg)
      call codes_get(ibufr,'solarAzimuth',rvalues)
      do isub = 1,nsubsets
        ama1c_dt_store(index_angles+4*spots(isub)-1, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !dataqual
      call codes_get(ibufr,'fieldOfViewQualityFlagsForAtovs',ivalues)
      do isub = 1,nsubsets
        Bit = ivalues(isub)
        CALL UNBITSWAP(i, Mask033033, NumBitsIn, Bit, NumBitsOut)
        ama1c_dt_store(index_dataqual+spots(isub)-1,scans(isub)) = i
      enddo

  !instrument temps
      call codes_get(ibufr,'#2#instrumentTemperature',rvalues)
      do isub = 1,nsubsets,Anfovs
        ama1c_dt_store(index_instrtemp1, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo
      call codes_get(ibufr,'#3#instrumentTemperature',rvalues)
      do isub = 1,nsubsets,Anfovs
        ama1c_dt_store(index_instrtemp2, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !wavenumber
      do ichan=1,Anchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#log10OfTemperatureRadianceCentralWaveNumberForAtovs'
        call codes_get(ibufr, descriptor, rvalues)
        ama1c_h_tempradcnv(1,ichan) = nint(1.0D4*(10.0d0**rvalues(1)))
      enddo

  !band corr 1
      do ichan=1,Anchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#bandwidthCorrectionCoefficient1ForAtovs'
        call codes_get(ibufr, descriptor, rvalues)
        ama1c_h_tempradcnv(2,ichan) = FNSCALEEC(rvalues(1),1.0D6)
      enddo

  !band corr 2
      do ichan=1,Anchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#bandwidthCorrectionCoefficient2ForAtovs'
        call codes_get(ibufr, descriptor, rvalues)
        ama1c_h_tempradcnv(3,ichan) = FNSCALEEC(rvalues(1),1.0D6)
      enddo

  !chanqual (chan,scan)
      do ichan=1,Anchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#channelQualityFlagsForAtovs'
        call codes_get(ibufr, descriptor, ivalues)
        do isub = 1,nsubsets,Anfovs
          Bit = ivalues(isub)
          call unbitswap(i, Mask033032, NumBitsIn, Bit, NumBitsOut)
          ama1c_dt_store(index_chanqual+ichan-1, scans(isub)) = i
        enddo   
      enddo

  !btemps (chan,spot,scan)
      do ichan=1,Anchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#brightnessTemperature'
        call codes_get(ibufr, descriptor, rvalues)
        do isub = 1,nsubsets
          rtmp = rvalues(isub)
          ama1c_dt_store(index_btemps+ichan-1+Anchan*(spots(isub)-1), 
     &        scans(isub)) = FNSCALEEC(rtmp, 1.0D2)
        enddo
      enddo

  !nedt
      if (nedt_present) then
        do ichan=1,Anchan
          jchan = ichan + Anchan
          write(descriptor,formatstr(jchan)) '#',jchan,
     &       '#brightnessTemperature'
          call codes_get(ibufr, descriptor, rvalues)
          do isub = 1,nsubsets,Anfovs
            rtmp = rvalues(isub)
            i = FNSCALEEC(rtmp, 1.0D3)
            ama1c_dt_store(index_warmnedt+ichan-1,scans(isub)) = i
            ama1c_dt_store(index_coldnedt+ichan-1,scans(isub)) = 0
            if (i .ne. Imiss .and. i .ne. 0) then
              warmnedt_sumsq(ichan) = warmnedt_sumsq(ichan)+rtmp*rtmp
              warmnedt_num(ichan) = warmnedt_num(ichan) + 1
            endif
          enddo
        enddo
      else
        do ichan=1,Anchan
          do isub = 1,nsubsets,Anfovs
            ama1c_dt_store(index_warmnedt+ichan-1,scans(isub)) = 0
            ama1c_dt_store(index_coldnedt+ichan-1,scans(isub)) = 0
          enddo
        enddo
      endif

  !save the data
      scans_to_write = spots_to_write/Anfovs
      DO iscan=1,scans_to_write
        spots_to_write = spots_to_write - Anfovs
        do i=1,array_size_aa1c
          ama1c_dt(i) = ama1c_dt_store(i,iscan+skipscan)
        enddo
        ama1c_h_scnlin = ama1c_h_scnlin + 1
        if (ama1c_h_startdatatime .lt. 0) then
          ama1c_h_startdatatime = ama1c_scnlintime
          ama1c_h_startdatady = ama1c_scnlindy
          ama1c_h_startdatayr = ama1c_scnlinyr
        endif
        ama1c_h_enddatatime = ama1c_scnlintime
        ama1c_h_enddatady = ama1c_scnlindy
        ama1c_h_enddatayr = ama1c_scnlinyr
        call ioa1c("WRITE DATA",out1cunit,ama1c_h_scnlin+ama1c_h_hdrcnt)
      ENDDO

  !Update the header
      do ichan=1,Anchan
        if (warmnedt_num(ichan) .gt. 0) then
          ama1c_h_warmnedt(ichan) = 
     &       1000*sqrt(warmnedt_sumsq(ichan)/warmnedt_num(ichan))+0.5
        else
          ama1c_h_warmnedt(ichan) = ama1c_warmnedt(ichan)
        endif
      enddo
      call ioa1c("WRITE HEADER", out1cunit, 1)
      write(*,*)'Message:',imessage, ' subsets:',nsubsets,
     &  ' scan count:',ama1c_h_scnlin


  !If there is a partial scan left over, save the data
      if (spots_to_write .ne. 0) then
        do i=1,array_size_aa1c
          ama1c_dt(i) = ama1c_dt_store(i,scans_to_write+1)
        enddo
      endif

  !Deallocate arrays before exiting the subroutine
      if(allocated(ivalues)) deallocate(ivalues)
      if(allocated(rvalues)) deallocate(rvalues)
      if(allocated(scans)) deallocate(scans)
      if(allocated(spots)) deallocate(spots)
      if(allocated(years)) deallocate(years)
      if(allocated(months)) deallocate(months)
      if(allocated(days)) deallocate(days)
      if(allocated(hours)) deallocate(hours)
      if(allocated(minutes)) deallocate(minutes)
      if(allocated(seconds)) deallocate(seconds)
      if(allocated(ama1c_dt_store)) deallocate(ama1c_dt_store)
#endif

      end subroutine eccodes_get_1c_amsua
