      subroutine eccodes_get_1c_iasi(ibufr,out1cunit,imessage)
      
#ifdef HAS_LIBECCODES
      use eccodes
#endif
      implicit none

!-----------------------------------------------------------------------
!
! Name: $Id: eccodes_get_1c_iasi.F 601 2018-11-21 15:57:14Z frna $
!
! Purpose: BUFR decoder subroutine for IASI l1c
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 7 December 2016, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, DWD and MeteoFrance.
!
!    Copyright 2017, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0     13/03/2017 Initial
! 1.1     21/11/2018 orbitNumber can change within a message
!
! Code Description:
!   Fortran90, but using FORTRAN77 fixed-formatting, in order to interface with
!   existing AAPP "include" files.
!
! ecCodes subroutines: codes_get
! AAPP subroutines: ioi1c, UNBITSWAP
!
! Declarations:

#include "inparms.h"
#include "iasi1c.h"

!Functions
      integer FNSCALEEC, FNISCALEEC, DAYOFYEAR

!Arguments
      integer ibufr
      integer out1cunit
      integer imessage
      
#ifdef HAS_LIBECCODES

!Variables
      integer   Imiss
      parameter (Imiss=-999999)     !AAPP missing value integer
      integer   Infovs
      parameter (Infovs=iasi_1cnfov*iasi_1cndet)

  !Miscellaneous variables 
      integer i, j, irec, iscan, isub, jsub, ispot, ichan, jchan, kchan
      integer pos, pos0, iband, iclass, sensor
      integer nsubsets, scans_to_store, scans_to_write, skipscan

  !Variables to be saved between calls
      integer, save      :: spots_to_write
      character*10, save :: formatstr(iasi_1cnchan)

  !AAPP data store
      integer, dimension(:,:), allocatable :: iasi1c_dt_store
      integer, dimension(:), allocatable   :: scans
      integer, dimension(:), allocatable   :: spots
      integer   scalrad_i4(1)
      integer*2 scalrad_i2(2)
      equivalence(scalrad_i4(1),scalrad_i2(1))

  !Arrays that are passed to eccodes
      real*8,    dimension(:), allocatable :: rvalues
      integer*4, dimension(:), allocatable :: ivalues
      integer*4, dimension(:), allocatable :: ivalues2
      integer*4, dimension(:), allocatable :: ivalues3
      character*80 :: descriptor

  !Date/time arrays and scalars
      integer, dimension(:), allocatable :: years
      integer, dimension(:), allocatable :: months
      integer, dimension(:), allocatable :: days
      integer, dimension(:), allocatable :: hours
      integer, dimension(:), allocatable :: minutes
      real*8, dimension(:), allocatable  :: seconds
      integer scnlindy,scnlinyr,FovTime  !date and time
      real*8  Year,Month,Day      !for passing to DAYOFYEAR
      integer millisecs, jday
      
  !Index variables for AAPP l1c array
      integer, save :: index_scnlin,index_granule,index_scnlinyr,
     &  index_scnlindy,index_scnlintime,index_GQisFlagQual,
     &  index_GQisQualIndex,index_GQisQualIndexLoc,
     &  index_GQisQualIndexRad,index_GQisQualIndexSpect,
     &  index_GQisSysTecSondQual,index_latlon,index_angles,
     &  index_scalti,index_scalrad,index_avhcombination,index_avhrrmean,
     &  index_avhmeanscalefactor,index_avhrrsdev,
     &  index_avhsdevscalefactor,index_avhrrfrac,index_avhrrCGy,
     &  index_avhrrCGz,index_GIacVarImagIIS,index_GIacAvgImagIIS,
     &  index_GEUMAvhrr1BCldFrac,index_GEUMAvhrr1BLandFrac,
     &  index_GEUMAvhrr1BQual

!-----------------------------------------------------------------------

      call codes_get(ibufr,'#1#satelliteInstruments',sensor)
      if (sensor .ne. 221) then
        write(*,*) 'Skip message. Sensor=',sensor
        return
      endif

      imessage = imessage + 1

      if (imessage .eq. 1) then

!Format string to use with multi-channel descriptors
        do ichan=1,9
          write(formatstr(ichan),'(A)') '(A,I1,A)'
        enddo
        do ichan=10,99
          write(formatstr(ichan),'(A)') '(A,I2,A)'
        enddo
        do ichan=100,999
          write(formatstr(ichan),'(A)') '(A,I3,A)'
        enddo
        do ichan=1000,iasi_1cnchan
          write(formatstr(ichan),'(A)') '(A,I4,A)'
        enddo

!Find the position in the AAPP data record of each variable
        do i=1,array_size_iasi1c
          iasi1c_dt(i) = i
        enddo
        index_scnlin = iasi1c_scnlin
        index_granule = iasi1c_granule
        index_scnlinyr = iasi1c_scnlinyr
        index_scnlindy = iasi1c_scnlindy
        index_scnlintime = iasi1c_scnlintime(1)
        index_GQisFlagQual = iasi1c_GQisFlagQual(1,1)
        index_GQisQualIndex = iasi1c_GQisQualIndex
        index_GQisQualIndexLoc = iasi1c_GQisQualIndexLoc
        index_GQisQualIndexRad = iasi1c_GQisQualIndexRad
        index_GQisQualIndexSpect = iasi1c_GQisQualIndexSpect
        index_GQisSysTecSondQual = iasi1c_GQisSysTecSondQual
        index_latlon = iasi1c_latlon(1,1,1)
        index_angles = iasi1c_angles(1,1,1)
        index_scalti = iasi1c_scalti
        index_scalrad = iasi1c_scalti + 1       !index in integer*4 words
        index_avhcombination = iasi1c_avhcombination
        index_avhrrmean = iasi1c_avhrrmean(1,1,1,1)
        index_avhmeanscalefactor =iasi1c_avhmeanscalefactor(1,1,1,1)
        index_avhrrsdev = iasi1c_avhrrsdev(1,1,1,1)
        index_avhsdevscalefactor =iasi1c_avhsdevscalefactor(1,1,1,1)
        index_avhrrfrac = iasi1c_avhrrfrac(1,1,1)
        index_avhrrCGy = iasi1c_avhrrCGy(1,1,1)
        index_avhrrCGz = iasi1c_avhrrCGz(1,1,1)
        index_GIacAvgImagIIS = iasi1c_GIacAvgImagIIS(1)
        index_GIacVarImagIIS = iasi1c_GIacVarImagIIS(1)
        index_GEUMAvhrr1BCldFrac = iasi1c_GEUMAvhrr1BCldFrac(1,1)
        index_GEUMAvhrr1BLandFrac = iasi1c_GEUMAvhrr1BLandFrac(1,1)
        index_GEUMAvhrr1BQual = iasi1c_GEUMAvhrr1BQual(1,1)
      
!Initialise some header variables
        call codes_get(ibufr,'bufrHeaderCentre',i)
        if (i .ge. 0 .and. i .lt. 999) then
          write(iasi1c_h_site,'(I3.3)') i
        else
          iasi1c_h_site = "xxx"
        endif
        call codes_get(ibufr,'bufrHeaderSubCentre',i)
        if (i .ge. 0 .and. i .le. 999) then
          write(iasi1c_h_1bsite,'(I3.3)') i
        else
          iasi1c_h_1bsite = "xxx"
        endif
        iasi1c_h_cfill1 = ' '
        iasi1c_h_cfill2 = ' '
        iasi1c_h_scnlin = 0
        iasi1c_h_hdrcnt = 1
        iasi1c_h_startdatatime = -1
        iasi1c_h_versnb = iasi_1cvnnb
        iasi1c_h_versyr = iasi_1cvnyr
        iasi1c_h_versdy = iasi_1cvndy
        iasi1c_h_instrument = 13       !IASI
        spots_to_write = 0

        call codes_get(ibufr,'satelliteIdentifier',iasi1c_h_wmosatid)

        iasi1c_h_satid = iasi1c_h_wmosatid-2     !METOP
        IF (iasi1c_h_wmosatid .eq. Imiss) iasi1c_h_satid = Imiss
        !!call codes_get(ibufr,'orbitNumber',iasi1c_h_startorbit)

  !Nominal height and period are not in the BUFR, so get them from fdf file
        DO J=1,MaxSats
          IF (WMOid(J).EQ.iasi1c_h_wmosatid) THEN
            iasi1c_h_satht = SatHt(J)*10   !nominal ht, km*10
            iasi1c_h_period = OrbitPd(J)   !period, seconds
            iasi1c_h_satid = NOAAid(J)     !Satellite ID
          ENDIF
        ENDDO

  !Open the output file and write header

        call ioi1c("OPEN/WRITE", out1cunit, 0)
        call ioi1c("WRITE HEADER", out1cunit, 1)

      endif     !(imessage .eq. 1)

      !!call codes_get(ibufr,'orbitNumber',iasi1c_h_endorbit)

  !Read the total number of subsets. Normally an integer multiple of the number
  !of spots, but make the code general in case scans are split across messages. 

      call codes_get(ibufr,'numberOfSubsets',nsubsets)
      scans_to_store = (nsubsets+spots_to_write)/Infovs + 1    !add 1 scan to be safe

      allocate(ivalues(nsubsets))
      allocate(rvalues(nsubsets))
      allocate(scans(nsubsets))
      allocate(spots(nsubsets))
      allocate(years(nsubsets))
      allocate(months(nsubsets))
      allocate(days(nsubsets))
      allocate(hours(nsubsets))
      allocate(minutes(nsubsets))
      allocate(seconds(nsubsets))
      allocate(iasi1c_dt_store(array_size_iasi1c,scans_to_store))
      iasi1c_dt_store = 0

  !if there is a partial scan left over from previous message, load the data
  !into the first scan
      if (spots_to_write .ne. 0) then
        do i=1,array_size_iasi1c
          iasi1c_dt_store(i,1) = iasi1c_dt(i)
        enddo
      endif

  !Orbit number
      call codes_get(ibufr,'orbitNumber',ivalues)
      if (imessage .eq. 1) iasi1c_h_startorbit = ivalues(1)
      iasi1c_h_endorbit = ivalues(nsubsets)

  !FOV number
      call codes_get(ibufr,'fieldOfViewNumber',spots)
      if (spots(1) .eq. 0) spots(:) = spots(:)+1    !EUMETSAT use 0-119
      do isub = 1,nsubsets
        if (spots(isub).le.0.or.spots(isub).gt.Infovs) then
          write(*,*)'Error: invalid fov number',spots(isub)
          call aappexit(1)
        endif
      enddo
      if (imessage .eq. 1) then
        if (spots(1).ne.1) then
          skipscan = 1      !ignore initial partial scan if there is one
          spots_to_write = nsubsets + 1 - spots(1)
        else
          skipscan = 0
          spots_to_write = nsubsets
        endif
      else
        skipscan = 0
        spots_to_write = spots_to_write + nsubsets
      endif

  !scan number
      call codes_get(ibufr,'scanLineNumber',ivalues)
      scans(1) = 1
      do isub = 2,nsubsets
        scans(isub) = scans(isub-1)
        if (spots(isub).eq.1) scans(isub) = scans(isub) + 1
      enddo
      do isub = 1,nsubsets
        iasi1c_dt_store(index_scnlin,scans(isub)) = ivalues(isub)
      enddo

  !date/time
      call codes_get(ibufr,'year',years)
      call codes_get(ibufr,'month',months)
      call codes_get(ibufr,'day',days)
      call codes_get(ibufr,'hour',hours)
      call codes_get(ibufr,'minute',minutes)
      call codes_get(ibufr,'second',seconds)
        
  !transform the date/time variables
      do isub = 1,nsubsets
        if (spots(isub).eq.1) then
          Year = years(isub)
          Month = months(isub)
          Day = days(isub)
          iasi1c_dt_store(index_scnlinyr, scans(isub)) = Year
          jday = DAYOFYEAR(Year,Month,Day)
          iasi1c_dt_store(index_scnlindy, scans(isub)) = jday
        endif
        if (MOD(spots(isub),4) .eq. 1) then
          millisecs = hours(isub)*3600000 + minutes(isub)*60000 
     &                  + seconds(isub)*1000   
          iasi1c_dt_store(index_scnlintime+spots(isub)/4, scans(isub)) = 
     &                  millisecs
        endif
      enddo

  !gqisFlagQual (store in bits 0-3 of iasi1c_gqisFlagQual)
      allocate(ivalues2(nsubsets))
      allocate(ivalues3(nsubsets))
      call codes_get(ibufr,'#1#gqisFlagQual',ivalues)
      call codes_get(ibufr,'#2#gqisFlagQual',ivalues2)
      call codes_get(ibufr,'#3#gqisFlagQual',ivalues3)
      do isub=1,nsubsets
        i = index_GQisFlagQual+spots(isub)-1
        if (ivalues(isub) .eq. CODES_MISSING_LONG) then
          iasi1c_dt_store(i, scans(isub)) = Imiss
        else
          j = ivalues(isub)*2 + ivalues2(isub)*4 + 
     &           ivalues3(isub)*8
          if (j .ne. 0) j = j+1
          iasi1c_dt_store(i, scans(isub)) = j
        endif
      enddo
      deallocate(ivalues2)
      deallocate(ivalues3)

  !gqisFlagQualDetailed (store in bits 4+ of iasi1c_gqisFlagQual)
      call codes_get(ibufr,'gqisFlagQualDetailed',ivalues)
      do isub = 1,nsubsets
        i = index_GQisFlagQual+spots(isub)-1
        if (ivalues(isub) .ne. CODES_MISSING_LONG) then
          iasi1c_dt_store(i, scans(isub)) = 
     &       iasi1c_dt_store(i, scans(isub)) + ivalues(isub)*16
        endif
      enddo

  !gqisQualIndex
      call codes_get(ibufr,'gqisQualIndex',ivalues)
      do isub = 1,nsubsets
        iasi1c_dt_store(index_gqisQualIndex,scans(isub)) = ivalues(isub) 
      enddo

  !gqisQualIndexLoc
      call codes_get(ibufr,'gqisQualIndexLoc',ivalues)
      do isub = 1,nsubsets
        iasi1c_dt_store(index_gqisQualIndexLoc,scans(isub)) =
     &      ivalues(isub) 
      enddo

  !gqisQualIndexRad
      call codes_get(ibufr,'gqisQualIndexRad',ivalues)
      do isub = 1,nsubsets
        iasi1c_dt_store(index_gqisQualIndexRad,scans(isub)) =
     &      ivalues(isub) 
      enddo

  !gqisQualIndexSpect
      call codes_get(ibufr,'gqisQualIndexSpect',ivalues)
      do isub = 1,nsubsets
        iasi1c_dt_store(index_gqisQualIndexSpect,scans(isub)) =
     &      ivalues(isub) 
      enddo

  !gqisSysTecQual
      call codes_get(ibufr,'gqisSysTecQual',ivalues)
      do isub = 1,nsubsets
        iasi1c_dt_store(index_GQisSysTecSondQual,scans(isub)) =
     &      ivalues(isub) 
      enddo

  !Scale factors (note we have already used three 'startChannel' descriptors)
      do iband=1,10
        write(descriptor,formatstr(iband+3))'#',iband+3,'#startChannel'
        call codes_get(ibufr,descriptor,i)
        iasi1c_h_startchan(iband) = FNISCALEEC(i,1)
        write(descriptor,formatstr(iband+3))'#',iband+3,'#endChannel'
        call codes_get(ibufr,descriptor,i)
        iasi1c_h_endchan(iband) = FNISCALEEC(i,1)
        write(descriptor,formatstr(iband)) 
     &        '#',iband,'#channelScaleFactor'
        call codes_get(ibufr,descriptor,i)
        iasi1c_h_scalefactor(iband) = FNISCALEEC(i,1)
      enddo

  !latitude
      call codes_get(ibufr,'latitude',rvalues)
      do isub = 1,nsubsets
        iasi1c_dt_store(index_latlon+2*spots(isub)-2, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D4)
      enddo

  !longitude
      call codes_get(ibufr,'longitude',rvalues)
      do isub = 1,nsubsets
        iasi1c_dt_store(index_latlon+2*spots(isub)-1, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D4)
      enddo

  !scalti
      call codes_get(ibufr,'heightOfStation',rvalues)
      do isub = 1,nsubsets,Infovs
        iasi1c_dt_store(index_scalti,scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D-2)  !store as km*10
      enddo

  !sat zenith
      call codes_get(ibufr,'satelliteZenithAngle',rvalues)
      do isub = 1,nsubsets
        iasi1c_dt_store(index_angles+4*spots(isub)-4, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !sat azimuth (0 to 360 deg)
      call codes_get(ibufr,'bearingOrAzimuth',rvalues)
      do isub = 1,nsubsets
        iasi1c_dt_store(index_angles+4*spots(isub)-3, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !solar zenith
      call codes_get(ibufr,'solarZenithAngle',rvalues)
      do isub = 1,nsubsets
        iasi1c_dt_store(index_angles+4*spots(isub)-2, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !solar azimuth (0 to 360 deg)
      call codes_get(ibufr,'solarAzimuth',rvalues)
      do isub = 1,nsubsets
        iasi1c_dt_store(index_angles+4*spots(isub)-1, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !Scaled radiance (stored as integer*2 in AAPP format)
      if(allocated(ivalues)) deallocate(ivalues)
      allocate(ivalues(nsubsets*iasi_1cnchan))
      call codes_get(ibufr,'scaledIasiRadiance',ivalues)
      do ichan=1,iasi_1cnchan
        jchan = (ichan-1)/2 + 1        !1, 1, 2, 2, 3, 3, etc.
        kchan = MOD((ichan-1), 2)+1    !1, 2, 1, 2, 1, 2, etc.
        pos0 = (ichan-1)*nsubsets      !position of channel in ivalues array
        do isub = 1,nsubsets
          pos = pos0+isub
          if (ivalues(pos) .gt. 32767) ivalues(pos)=ivalues(pos)-65536
          scalrad_i2(kchan) = ivalues(pos)
          scalrad_i2(3-kchan) = 0
          i = index_scalrad+jchan-1+(spots(isub)-1)*iasi_1cnchan/2
          iasi1c_dt_store(i,scans(isub)) = 
     &        IOR(iasi1c_dt_store(i,scans(isub)),scalrad_i4(1))
        enddo
      enddo
      deallocate(ivalues)
      allocate(ivalues(nsubsets))

  !AVHRR channel combination
      call codes_get(ibufr,'avhrrChannelCombination',ivalues)
      do isub = 1,nsubsets
        iasi1c_dt_store(index_avhcombination,scans(isub))=ivalues(isub)
      enddo

  !avhrrCGy, avhrrCGz, avhrrfrac
      do iclass=1,7
        write(descriptor,formatstr(iclass))
     &    '#',iclass,'#yAngularPositionFromCentreOfGravity'  
        call codes_get(ibufr,descriptor,rvalues)
        do isub = 1,nsubsets
          i = index_avhrrCGy + (spots(isub)-1)*7 + iclass-1
          iasi1c_dt_store(i,scans(isub))=FNSCALEEC(rvalues(isub),1.0D6)
        enddo

        write(descriptor,formatstr(iclass))
     &    '#',iclass,'#zAngularPositionFromCentreOfGravity'  
        call codes_get(ibufr,descriptor,rvalues)
        do isub = 1,nsubsets
          i = index_avhrrCGz + (spots(isub)-1)*7 + iclass-1
          iasi1c_dt_store(i,scans(isub))=FNSCALEEC(rvalues(isub),1.0D6)
        enddo

        write(descriptor,formatstr(iclass))
     &    '#',iclass,'#fractionOfClearPixelsInHirsFov'  
        call codes_get(ibufr,descriptor,rvalues)
        do isub = 1,nsubsets
          i = index_avhrrfrac + (spots(isub)-1)*7 + iclass-1
          iasi1c_dt_store(i,scans(isub))=FNSCALEEC(rvalues(isub),1.0D4)
        enddo
      enddo

  !avhrrmean, avhrrsdev, avhmeanscalefactor, avhsdevscalefactor
      do ichan=1,6
        do iclass=1,7
          iband = ichan + (iclass-1)*6
          write(descriptor,formatstr(iband))
     &         '#',iband,'#scaledMeanAvhrrRadiance'
          call codes_get(ibufr,descriptor,ivalues)
          do isub = 1,nsubsets
            i = index_avhrrmean + (spots(isub)-1)*42 
     &                    + (iclass-1)*6 + ichan-1
            iasi1c_dt_store(i,scans(isub)) = ivalues(isub)
          enddo
          write(descriptor,formatstr(iband))
     &         '#',iband,'#scaledStandardDeviationAvhrrRadiance'
          call codes_get(ibufr,descriptor,ivalues)
          do isub = 1,nsubsets
            i = index_avhrrsdev + (spots(isub)-1)*42 
     &                    + (iclass-1)*6 + ichan-1
            iasi1c_dt_store(i,scans(isub)) = ivalues(isub)
          enddo

  !note: ignore the first 10 entries of channelScaleFactor as they have already
  !been used in scaled radiances.
          do j=1,2
            iband = 10 + j + (ichan-1)*2 + (iclass-1)*12
            write(descriptor,formatstr(iband))
     &         '#',iband,'#channelScaleFactor'
            call codes_get(ibufr,descriptor,ivalues)
            do isub = 1,nsubsets
              if (j .eq. 1) then
                i = index_avhmeanscalefactor + (spots(isub)-1)*42
     &                    + (iclass-1)*6 + ichan-1
              else
                i = index_avhsdevscalefactor + (spots(isub)-1)*42
     &                    + (iclass-1)*6 + ichan-1
              endif
              iasi1c_dt_store(i,scans(isub)) = ivalues(isub)
            enddo
          enddo
        enddo
      enddo

  !GIacAvgImagIIS (30 values across scan)
      call codes_get(ibufr,'averageOfImagerMeasurements',rvalues)
      do isub = 1,nsubsets,4
        iasi1c_dt_store(index_GIacAvgImagIIS+(spots(isub)-1)/4,
     &       scans(isub)) = FNSCALEEC(rvalues(isub),1.0D6)
      enddo

  !GIacVarImagIIS (30 values across scan)
      call codes_get(ibufr,'giacvarimagiisVarianceOfImagerMeasurements',
     &       rvalues)
      do isub = 1,nsubsets,4
        iasi1c_dt_store(index_GIacVarImagIIS+(spots(isub)-1)/4,
     &       scans(isub)) = FNSCALEEC(rvalues(isub),1.0D6)
      enddo

  !GEUMAvhrr1BCldFrac
      call codes_get(ibufr,'cloudAmountInSegment',ivalues)
      do isub = 1,nsubsets
        i = index_GEUMAvhrr1BCldFrac+spots(isub)-1
        if (ivalues(isub) .ne. CODES_MISSING_LONG) then
          iasi1c_dt_store(i, scans(isub)) = ivalues(isub)
        else
          iasi1c_dt_store(i, scans(isub)) = 255
        endif
      enddo

  !GEUMAvhrr1BLandFrac
      call codes_get(ibufr,'amountOfSegmentCoveredByScene',ivalues)
      do isub = 1,nsubsets
        i = index_GEUMAvhrr1BLandFrac+spots(isub)-1
        if (ivalues(isub) .ne. CODES_MISSING_LONG) then
          iasi1c_dt_store(i,scans(isub)) = ivalues(isub)
        else
          iasi1c_dt_store(i,scans(isub)) = 255
        endif
      enddo

  !GEUMAvhrr1BQual
  !This is the snow/ice cover if available. Otherwise it is the nuber of bad
  !pixels (0-126) and bit 7 is set to 1. Set to -1 is missing.

      call codes_get(ibufr,
     &  'fractionOfWeightedAvhrrPixelInIasiFovCoveredWithSnowOrIce',
     &  ivalues)
      do isub = 1,nsubsets
        i = index_GEUMAvhrr1BQual+spots(isub)-1
        if (ivalues(isub) .ne. CODES_MISSING_LONG) then
          iasi1c_dt_store(i,scans(isub)) = ivalues(isub)
        else
          iasi1c_dt_store(i,scans(isub)) = -1
        endif
      enddo

      call codes_get(ibufr,'numberOfMissingBadOrFailedAvhrrPixels',
     &  ivalues)
      do isub = 1,nsubsets
        i = index_GEUMAvhrr1BQual+spots(isub)-1
        if (iasi1c_dt_store(i,scans(isub)) .eq. -1) then
          if (ivalues(isub) .ne. CODES_MISSING_LONG) then
            iasi1c_dt_store(i,scans(isub)) = ivalues(isub) + 128   !Set bit 7
          endif
        endif
      enddo

  !save the data
      scans_to_write = spots_to_write/Infovs
      DO iscan=1,scans_to_write
        spots_to_write = spots_to_write - Infovs
        do i=1,array_size_iasi1c
          iasi1c_dt(i) = iasi1c_dt_store(i,iscan+skipscan)
        enddo
        iasi1c_h_scnlin = iasi1c_h_scnlin + 1
        if (iasi1c_h_startdatatime .lt. 0) then
          iasi1c_h_startdatatime = iasi1c_scnlintime(1)
          iasi1c_h_startdatady = iasi1c_scnlindy
          iasi1c_h_startdatayr = iasi1c_scnlinyr
        endif
        iasi1c_h_enddatatime = iasi1c_scnlintime(iasi_1cnfov)
        iasi1c_h_enddatady = iasi1c_scnlindy
        iasi1c_h_enddatayr = iasi1c_scnlinyr
        if ((iasi1c_scnlintime(1)-iasi1c_scnlintime(iasi_1cnfov))
     &      .gt.86392000) then
          iasi1c_h_enddatady = iasi1c_h_enddatady + 1
          if ((iasi1c_h_enddatady .gt. 366) .or. 
     &         (MOD(iasi1c_h_enddatayr,4) .ne. 0 .and. 
     &         iasi1c_h_enddatady .gt. 365)) then
            iasi1c_h_enddatady = 1
            iasi1c_h_enddatayr = iasi1c_h_enddatayr + 1
          endif
        endif
        call ioi1c("WRITE DATA", out1cunit,
     &      iasi1c_h_scnlin+iasi1c_h_hdrcnt)
      ENDDO

  !Update the header
      call ioi1c("WRITE HEADER", out1cunit, 1)
      write(*,*)'Message:',imessage, ' subsets:',nsubsets,
     &  ' scan count:',iasi1c_h_scnlin

  !If there is a partial scan left over, save the data
      if (spots_to_write .ne. 0) then
        do i=1,array_size_iasi1c
          iasi1c_dt(i) = iasi1c_dt_store(i,scans_to_write+1)
        enddo
      endif

  !Deallocate arrays before exiting the subroutine
      if(allocated(ivalues)) deallocate(ivalues)
      if(allocated(rvalues)) deallocate(rvalues)
      if(allocated(scans)) deallocate(scans)
      if(allocated(spots)) deallocate(spots)
      if(allocated(years)) deallocate(years)
      if(allocated(months)) deallocate(months)
      if(allocated(days)) deallocate(days)
      if(allocated(hours)) deallocate(hours)
      if(allocated(minutes)) deallocate(minutes)
      if(allocated(seconds)) deallocate(seconds)
      if(allocated(iasi1c_dt_store)) deallocate(iasi1c_dt_store)

#endif
      end subroutine eccodes_get_1c_iasi
