      subroutine eccodes_get_1c_mwri(ibufr,out1cunit,imessage)
      
#ifdef HAS_LIBECCODES
      use eccodes
#endif
      implicit none

!-----------------------------------------------------------------------
!
! Name: $Id: eccodes_get_1c_mwri.F 601 2018-11-21 15:57:14Z frna $
!
! Purpose: BUFR decoder subroutine for MWRI l1c
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 7 December 2016, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, DWD and MeteoFrance.
!
!    Copyright 2017, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0     12/05/2017 Initial
! 1.1     21/11/2018 orbitNumber can change within a message
!
! Code Description:
!   Fortran90, but using FORTRAN77 fixed-formatting, in order to interface with
!   existing AAPP "include" files.
!
! ecCodes subroutines: codes_get
! AAPP subroutines: iomwri1c
!
! Declarations:

#include "mwri1c.h"
#include "constants_preproc.h"
#include "mwrichandata.h"

!Functions
      integer FNSCALEEC, DAYOFYEAR

!Arguments
      integer ibufr
      integer out1cunit
      integer imessage
      logical nedt_present
      
#ifdef HAS_LIBECCODES

!Variables
      integer   Imiss
      parameter (Imiss=-999999)     !AAPP missing value integer

  !Miscellaneous variables 
      integer i, j, irec, iscan, isub, ispot, ichan, jchan
      integer sensor
      integer nsubsets
      integer scans_to_store
      integer scans_to_write
      integer skipscan
      real*8  rtmp

  !Variables to be saved between calls
      integer, save      :: spots_to_write
      character*10, save :: formatstr(mwri_nchan)

  !AAPP data store
      integer, dimension(:,:), allocatable :: mwri1c_dt_store
      integer, dimension(:), allocatable   :: scans
      integer, dimension(:), allocatable   :: spots

  !Arrays that are passed to eccodes
      real*8,    dimension(:), allocatable :: rvalues
      integer*4, dimension(:), allocatable :: ivalues
      character*80 :: descriptor

  !Date/time arrays and scalars
      integer, dimension(:), allocatable :: years
      integer, dimension(:), allocatable :: months
      integer, dimension(:), allocatable :: days
      integer, dimension(:), allocatable :: hours
      integer, dimension(:), allocatable :: minutes
      real*8, dimension(:), allocatable  :: seconds

      real*8  Year,Month,Day      !for passing to DAYOFYEAR
      integer millisecs, jday
      
  !Index variables for AAPP l1c array
      integer, save :: index_scnlin, index_scnlinyr, index_scnlindy,
     & index_scnlintime, index_qualind, index_scanqual, 
     & index_chanqual, index_instrtemp1, index_instrtemp2,
     & index_spare1, index_latlon, index_surface, index_angles,
     & index_scalti, index_satposition, index_satvelocity, 
     & index_orbitangle, index_spare2, index_btemps, index_btmap,
     & index_dataqual, index_filler

!-----------------------------------------------------------------------

      call codes_get(ibufr,'satelliteInstruments',sensor)
      if (sensor .ne. 938) then
        write(*,*) 'Skip message. Sensor=',sensor
        return
      endif

      imessage = imessage + 1

      if (imessage .eq. 1) then

!Format string to use with multi-channel descriptors
        do ichan=1,9
          write(formatstr(ichan),'(A)') '(A,I1,A)'
        enddo
        do ichan=10,mwri_nchan
          write(formatstr(ichan),'(A)') '(A,I2,A)'
        enddo

!Find the position in the AAPP data record of each variable
        do i=1,array_size_mwri1c
          mwri1c_dt(i) = i
        enddo
        index_scnlin = mwri1c_scnlin
        index_scnlinyr = mwri1c_scnlinyr
        index_scnlindy = mwri1c_scnlindy
        index_scnlintime = mwri1c_scnlintime
        index_qualind = mwri1c_qualind
        index_scanqual = mwri1c_scanqual
        index_chanqual = mwri1c_chanqual(1)
        index_instrtemp1 = mwri1c_instrtemp1
        index_instrtemp2 = mwri1c_instrtemp2
        index_spare1 = mwri1c_spare1(1)
        index_latlon = mwri1c_latlon(1,1)
        index_surface = mwri1c_surface(1,1)
        index_angles = mwri1c_angles(1,1)
        index_scalti = mwri1c_scalti
        index_satposition = mwri1c_satposition(1)
        index_satvelocity = mwri1c_satvelocity(1)
        index_orbitangle = mwri1c_orbitangle
        index_spare2 = mwri1c_spare2(1)
        index_btemps = mwri1c_btemps(1,1)
        index_btmap = mwri1c_btmap(1,1)
        index_dataqual = mwri1c_dataqual(1)
        index_filler = mwri1c_filler(1)
      
!Initialise some header variables
        call codes_get(ibufr,'bufrHeaderCentre',i)
        if (i .ge. 0 .and. i .lt. 999) then
          write(mwri1c_h_site,'(I3.3)') i
        else
          mwri1c_h_site = "xxx"
        endif
        call codes_get(ibufr,'bufrHeaderSubCentre',i)
        if (i .ge. 0 .and. i .le. 999) then
          write(mwri1c_h_1bsite,'(I3.3)') i
        else
          mwri1c_h_1bsite = "xxx"
        endif
        mwri1c_h_cfill1 = ' '
        mwri1c_h_cfill2 = ' '
        mwri1c_h_scnlin = 0
        mwri1c_h_hdrcnt = 1
        mwri1c_h_startdatatime = -1
        mwri1c_h_versnb = mwri_1cvnnb
        mwri1c_h_versyr = mwri_1cvnyr
        mwri1c_h_versdy = mwri_1cvndy
        mwri1c_h_instrument = 20       !MWRI
        spots_to_write = 0

        call codes_get(ibufr,'satelliteIdentifier',mwri1c_h_wmosatid)
        IF (mwri1c_h_wmosatid .eq. CODES_MISSING_LONG) THEN
          mwri1c_h_wmosatid = Imiss
          mwri1c_h_satid = Imiss
        ELSE
          mwri1c_h_satid = mwri1c_h_wmosatid-519     !1 for FY-3A
        ENDIF
        !!call codes_get(ibufr,'orbitNumber',mwri1c_h_startorbit)
        mwri1c_h_vnantennacorr = Imiss
        mwri1c_h_satht = 8360   !nominal ht, km*10
        mwri1c_h_period = 6120   !period, seconds
        mwri1c_h_nchan = mwri_nchan
        mwri1c_h_nmapchan = mwri_nmapchan

  !Open the output file and write header

        call iomwri1c("OPEN/WRITE", out1cunit, 0)
        call iomwri1c("WRITE HEADER", out1cunit, 1)

      endif     !(imessage .eq. 1)

      !!call codes_get(ibufr,'orbitNumber',mwri1c_h_endorbit)

  !Read the total number of subsets. Normally an integer multiple of the number
  !of spots, but make the code general in case scans are split across messages. 

      call codes_get(ibufr,'numberOfSubsets',nsubsets)
      scans_to_store = (nsubsets+spots_to_write)/mwri_nfovs + 1    !add 1 scan to be safe

      allocate(ivalues(nsubsets))
      allocate(rvalues(nsubsets))
      allocate(scans(nsubsets))
      allocate(spots(nsubsets))
      allocate(years(nsubsets))
      allocate(months(nsubsets))
      allocate(days(nsubsets))
      allocate(hours(nsubsets))
      allocate(minutes(nsubsets))
      allocate(seconds(nsubsets))
      allocate(mwri1c_dt_store(array_size_mwri1c,scans_to_store))
      mwri1c_dt_store = 0

  !if there is a partial scan left over from previous message, load the data
  !into the first scan
      if (spots_to_write .ne. 0) then
        do i=1,array_size_mwri1c
          mwri1c_dt_store(i,1) = mwri1c_dt(i)
        enddo
      endif

  !Orbit number
      call codes_get(ibufr,'orbitNumber',ivalues)
      if (imessage .eq. 1) mwri1c_h_startorbit = ivalues(1)
      mwri1c_h_endorbit = ivalues(nsubsets)

  !FOV number
      call codes_get(ibufr,'fieldOfViewNumber',spots)
      do isub = 1,nsubsets
        if (spots(isub).le.0.or.spots(isub).gt.mwri_nfovs) then
          write(*,*)'Error: invalid fov number',spots(isub)
          call aappexit(1)
        endif
      enddo
      if (imessage .eq. 1) then
        if (spots(1).ne.1) then
          skipscan = 1      !ignore initial partial scan if there is one
          spots_to_write = nsubsets + 1 - spots(1)
        else
          skipscan = 0
          spots_to_write = nsubsets
        endif
      else
        skipscan = 0
        spots_to_write = spots_to_write + nsubsets
      endif

  !scan number
      call codes_get(ibufr,'scanLineNumber',ivalues)
      scans(1) = 1
      do isub = 2,nsubsets
        scans(isub) = scans(isub-1)
        if (spots(isub).eq.1) scans(isub) = scans(isub) + 1
      enddo
      do isub = 1,nsubsets
        mwri1c_dt_store(index_scnlin,scans(isub)) = ivalues(isub)
      enddo

  !date/time
      call codes_get(ibufr,'year',years)
      call codes_get(ibufr,'month',months)
      call codes_get(ibufr,'day',days)
      call codes_get(ibufr,'hour',hours)
      call codes_get(ibufr,'minute',minutes)
      call codes_get(ibufr,'second',seconds)
        
  !transform the date/time variables
      do isub = 1,nsubsets
        if (spots(isub).eq.1) then
          Year = years(isub)
          Month = months(isub)
          Day = days(isub)
          mwri1c_dt_store(index_scnlinyr, scans(isub)) = Year
          jday = DAYOFYEAR(Year,Month,Day)
          mwri1c_dt_store(index_scnlindy, scans(isub)) = jday
          millisecs = hours(isub)*3600000 + minutes(isub)*60000 
     &                  + seconds(isub)*1000   
          mwri1c_dt_store(index_scnlintime, scans(isub)) = millisecs
        endif
      enddo

  !latitude
      call codes_get(ibufr,'latitude',rvalues)
      do isub = 1,nsubsets
        mwri1c_dt_store(index_latlon+2*spots(isub)-2, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D4)
      enddo

  !longitude
      call codes_get(ibufr,'longitude',rvalues)
      do isub = 1,nsubsets
        mwri1c_dt_store(index_latlon+2*spots(isub)-1, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D4)
      enddo

  !scalti (not available)
      do isub = 1,nsubsets,mwri_nfovs
        mwri1c_dt_store(index_scalti,scans(isub)) = Imiss
      enddo

  !height
      call codes_get(ibufr,'height',rvalues)
      do isub = 1,nsubsets
        mwri1c_dt_store(index_surface+2*spots(isub)-2, scans(isub)) = 
     &      FNSCALEEC(rvalues(isub),1.0D0)
      enddo

  !landsea
      call codes_get(ibufr,'landOrSeaQualifier',ivalues)
      do isub = 1,nsubsets
        IF (ivalues(isub) .EQ. 1) THEN
          mwri1c_dt_store(index_surface+2*spots(isub)-1,scans(isub))=0  !Sea
        ELSE IF (ivalues(isub) .EQ. 2) THEN
          mwri1c_dt_store(index_surface+2*spots(isub)-1,scans(isub))=1  !Mixed
        ELSE IF (ivalues(isub) .EQ. 0) THEN
          mwri1c_dt_store(index_surface+2*spots(isub)-1,scans(isub))=2  !Land
        ELSE
          mwri1c_dt_store(index_surface+2*spots(isub)-1,scans(isub))
     &          = Imiss
        ENDIF
      enddo

  !sat zenith
      call codes_get(ibufr,'satelliteZenithAngle',rvalues)
      do isub = 1,nsubsets
        mwri1c_dt_store(index_angles+4*spots(isub)-4, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !sat azimuth (0 to 360 deg)
      call codes_get(ibufr,'bearingOrAzimuth',rvalues)
      do isub = 1,nsubsets
        mwri1c_dt_store(index_angles+4*spots(isub)-3, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !solar zenith
      call codes_get(ibufr,'solarZenithAngle',rvalues)
      do isub = 1,nsubsets
        mwri1c_dt_store(index_angles+4*spots(isub)-2, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !solar azimuth (0 to 360 deg)
      call codes_get(ibufr,'solarAzimuth',rvalues)
      do isub = 1,nsubsets
        mwri1c_dt_store(index_angles+4*spots(isub)-1, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !Use the nominal frequencies rather than the ones in the input BUFR file
      do ichan=1,mwri_nchan
        mwri1c_h_tempradcnv(1,ichan) = 
     &        NINT(mwricentrefreq(ichan)/C_light*1.0D13) !10^6*wavenumber cm^-1
        mwri1c_h_tempradcnv(2,ichan) = 
     &        NINT(mwribandcorr_offset(ichan)*1.0D6)     !10^6*offset in K
        mwri1c_h_tempradcnv(3,ichan) = 
     &        NINT(mwribandcorr_slope(ichan)*1.0D6)      !10^6*slope
        mwri1c_h_bandwidth(ichan) = 
     &        NINT(mwribandwidth(ichan)*1E6)             !10^6*bandwidth in GHz
      enddo

  !btemps (chan,spot,scan)
      do ichan=1,mwri_nchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#brightnessTemperature'
        call codes_get(ibufr, descriptor, rvalues)
        do isub = 1,nsubsets
          rtmp = rvalues(isub)
          mwri1c_dt_store(
     &        index_btemps+ichan-1+mwri_nchan*(spots(isub)-1), 
     &        scans(isub)) = FNSCALEEC(rtmp, 1.0D2)
        enddo
      enddo

  !save the data
      scans_to_write = spots_to_write/mwri_nfovs
      DO iscan=1,scans_to_write
        spots_to_write = spots_to_write - mwri_nfovs
        do i=1,array_size_mwri1c
          mwri1c_dt(i) = mwri1c_dt_store(i,iscan+skipscan)
        enddo
        mwri1c_h_scnlin = mwri1c_h_scnlin + 1
        if (mwri1c_h_startdatatime .lt. 0) then
          mwri1c_h_startdatatime = mwri1c_scnlintime
          mwri1c_h_startdatady = mwri1c_scnlindy
          mwri1c_h_startdatayr = mwri1c_scnlinyr
        endif
        mwri1c_h_enddatatime = mwri1c_scnlintime
        mwri1c_h_enddatady = mwri1c_scnlindy
        mwri1c_h_enddatayr = mwri1c_scnlinyr
        call iomwri1c("WRITE DATA",out1cunit,
     &    mwri1c_h_scnlin+mwri1c_h_hdrcnt)
      ENDDO

  !Update the header
      call iomwri1c("WRITE HEADER", out1cunit, 1)
      write(*,*)'Message:',imessage, ' subsets:',nsubsets,
     &  ' scan count:',mwri1c_h_scnlin


  !If there is a partial scan left over, save the data
      if (spots_to_write .ne. 0) then
        do i=1,array_size_mwri1c
          mwri1c_dt(i) = mwri1c_dt_store(i,scans_to_write+1)
        enddo
      endif

  !Deallocate arrays before exiting the subroutine
      if(allocated(ivalues)) deallocate(ivalues)
      if(allocated(rvalues)) deallocate(rvalues)
      if(allocated(scans)) deallocate(scans)
      if(allocated(spots)) deallocate(spots)
      if(allocated(years)) deallocate(years)
      if(allocated(months)) deallocate(months)
      if(allocated(days)) deallocate(days)
      if(allocated(hours)) deallocate(hours)
      if(allocated(minutes)) deallocate(minutes)
      if(allocated(seconds)) deallocate(seconds)
      if(allocated(mwri1c_dt_store)) deallocate(mwri1c_dt_store)
#endif

      end subroutine eccodes_get_1c_mwri
