      subroutine eccodes_get_1c_pciasi(ibufr,out1cunit,imessage)
      
#ifdef HAS_LIBECCODES
      use eccodes
#endif
      implicit none

!-----------------------------------------------------------------------
!
! Name: $Id: eccodes_get_1c_pciasi.F 636 2019-04-16 11:12:39Z frna $
!
! Purpose: BUFR decoder subroutine for IASI lpc
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 7 December 2016, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, DWD and MeteoFrance.
!
!    Copyright 2017, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0     10/05/2017 Initial
! 1.1     21/11/2018 orbitNumber can change within a message
!
! Code Description:
!   Fortran90, but using FORTRAN77 fixed-formatting, in order to interface with
!   existing AAPP "include" files.
!
! ecCodes subroutines: codes_get
! AAPP subroutines: ioipc, UNBITSWAP
!
! Declarations:

#include "inparms.h"
#include "iasipc.h"

!Functions
      integer FNSCALEEC, FNISCALEEC, DAYOFYEAR

!Arguments
      integer ibufr
      integer out1cunit
      integer imessage
      
#ifdef HAS_LIBECCODES

!Variables
      integer   Imiss
      parameter (Imiss=-999999)     !AAPP missing value integer
      integer   Infovs
      parameter (Infovs=iasi_pcnfov*iasi_pcndet)

  !Miscellaneous variables 
      integer i, j, irec, iscan, isub, ispot, ichan, jchan, kchan
      integer Npc, Ipc, K
      integer iband, iclass, sensor
      integer nsubsets, scans_to_store, scans_to_write, skipscan
      real*8  rtmp, logfit
      real    ResidualRMS

  !Variables to be saved between calls
      integer, save      :: spots_to_write
      character*10, save :: formatstr(iasi_pcnchan+42+iasi_pcnpc)

  !AAPP data store
      integer, dimension(:,:), allocatable :: iasipc_dt_store
      integer, dimension(:), allocatable   :: scans
      integer, dimension(:), allocatable   :: spots
      integer   scalrad_i4(1)
      integer*2 scalrad_i2(2)
      equivalence(scalrad_i4(1),scalrad_i2(1))

  !Arrays that are passed to eccodes
      real*8,    dimension(:), allocatable :: rvalues
      integer*4, dimension(:), allocatable :: ivalues
      integer*4, dimension(:), allocatable :: ivalues2
      integer*4, dimension(:), allocatable :: ivalues3
      character*80 :: descriptor

  !Date/time arrays and scalars
      integer, dimension(:), allocatable :: years
      integer, dimension(:), allocatable :: months
      integer, dimension(:), allocatable :: days
      integer, dimension(:), allocatable :: hours
      integer, dimension(:), allocatable :: minutes
      real*8, dimension(:), allocatable  :: seconds
      integer scnlindy,scnlinyr,FovTime  !date and time
      real*8  Year,Month,Day      !for passing to DAYOFYEAR
      integer millisecs, jday
      
  !Index variables for AAPP lpc array
      integer, save :: index_scnlin,index_granule,index_scnlinyr,
     &  index_scnlindy,index_scnlintime,index_GQisFlagQual,
     &  index_GQisQualIndex,index_GQisQualIndexLoc,
     &  index_GQisQualIndexRad,index_GQisQualIndexSpect,
     &  index_GQisSysTecSondQual,index_latlon,index_angles,
     &  index_scalti,index_scalrad,index_reconrad,index_scores,
     &  index_logfit,index_avhcombination,index_avhrrmean,
     &  index_avhmeanscalefactor,index_avhrrsdev,
     &  index_avhsdevscalefactor,index_avhrrfrac,index_avhrrCGy,
     &  index_avhrrCGz,index_GIacVarImagIIS,index_GIacAvgImagIIS,
     &  index_GEUMAvhrr1BCldFrac,index_GEUMAvhrr1BLandFrac,
     &  index_GEUMAvhrr1BQual


!-----------------------------------------------------------------------

      call codes_get(ibufr,'#1#satelliteInstruments',sensor)
      if (sensor .ne. 221) then
        write(*,*) 'Skip message. Sensor=',sensor
        return
      endif
      call codes_get(ibufr,
     &  '#1#extendedDelayedDescriptorReplicationFactor',
     &  iasipc_h_nchanused)

      imessage = imessage + 1

      if (imessage .eq. 1) then

!Format string to use with multi-channel descriptors
        do ichan=1,9
          write(formatstr(ichan),'(A)') '(A,I1,A)'
        enddo
        do ichan=10,min(iasi_pcnchan+42+iasi_pcnpc,99)
          write(formatstr(ichan),'(A)') '(A,I2,A)'
        enddo
        do ichan=100,min(iasi_pcnchan+42+iasi_pcnpc,999)
          write(formatstr(ichan),'(A)') '(A,I3,A)'
        enddo
        do ichan=1000,iasi_pcnchan+42+iasi_pcnpc
          if (ichan .le. iasi_pcnchan+42+iasi_pcnpc) then
            write(formatstr(ichan),'(A)') '(A,I4,A)'
          endif
        enddo

!Find the position in the AAPP data record of each variable
        do i=1,array_size_iasipc
          iasipc_dt(i) = i
        enddo
        index_scnlin = iasipc_scnlin
        index_granule = iasipc_granule
        index_scnlinyr = iasipc_scnlinyr
        index_scnlindy = iasipc_scnlindy
        index_scnlintime = iasipc_scnlintime(1)
        index_GQisFlagQual = iasipc_GQisFlagQual(1,1)
        index_GQisQualIndex = iasipc_GQisQualIndex
        index_GQisQualIndexLoc = iasipc_GQisQualIndexLoc
        index_GQisQualIndexRad = iasipc_GQisQualIndexRad
        index_GQisQualIndexSpect = iasipc_GQisQualIndexSpect
        index_GQisSysTecSondQual = iasipc_GQisSysTecSondQual
        index_latlon = iasipc_latlon(1,1,1)
        index_angles = iasipc_angles(1,1,1)
        index_scalti = iasipc_scalti
        index_scalrad = iasipc_scalti + 1       !index in integer*4 words
        index_reconrad = index_scalrad +
     &        iasi_pcnchan*iasi_pcndet*iasi_pcnfov/2
        index_scores = iasipc_scores(1,1,1)
        index_logfit = iasipc_logfit(1,1,1)
        index_avhcombination = iasipc_avhcombination
        index_avhrrmean = iasipc_avhrrmean(1,1,1,1)
        index_avhmeanscalefactor =iasipc_avhmeanscalefactor(1,1,1,1)
        index_avhrrsdev = iasipc_avhrrsdev(1,1,1,1)
        index_avhsdevscalefactor =iasipc_avhsdevscalefactor(1,1,1,1)
        index_avhrrfrac = iasipc_avhrrfrac(1,1,1)
        index_avhrrCGy = iasipc_avhrrCGy(1,1,1)
        index_avhrrCGz = iasipc_avhrrCGz(1,1,1)
        index_GIacAvgImagIIS = iasipc_GIacAvgImagIIS(1)
        index_GIacVarImagIIS = iasipc_GIacVarImagIIS(1)
        index_GEUMAvhrr1BCldFrac = iasipc_GEUMAvhrr1BCldFrac(1,1)
        index_GEUMAvhrr1BLandFrac = iasipc_GEUMAvhrr1BLandFrac(1,1)
        index_GEUMAvhrr1BQual = iasipc_GEUMAvhrr1BQual(1,1)
      
!Initialise some header variables
        call codes_get(ibufr,'bufrHeaderCentre',i)
        if (i .ge. 0 .and. i .lt. 999) then
          write(iasipc_h_site,'(I3.3)') i
        else
          iasipc_h_site = "xxx"
        endif
        call codes_get(ibufr,'bufrHeaderSubCentre',i)
        if (i .ge. 0 .and. i .le. 999) then
          write(iasipc_h_1bsite,'(I3.3)') i
        else
          iasipc_h_1bsite = "xxx"
        endif
        iasipc_h_cfill1 = ' '
        iasipc_h_cfill2 = ' '
        iasipc_h_scnlin = 0
        iasipc_h_hdrcnt = 1
        iasipc_h_startdatatime = -1
        iasipc_h_versnb = iasi_pcvnnb
        iasipc_h_versyr = iasi_pcvnyr
        iasipc_h_versdy = iasi_pcvndy
        iasipc_h_instrument = 14       !PCIASI
        iasipc_h_recl = iasi_pcrecl
        iasipc_h_nchan = iasi_pcnchan
        iasipc_h_nrchan = iasi_pcnrecon
        iasipc_h_npc = iasi_pcnpc
        iasipc_h_npcbands = 0
        spots_to_write = 0

        call codes_get(ibufr,'satelliteIdentifier',iasipc_h_wmosatid)

        iasipc_h_satid = iasipc_h_wmosatid-2     !METOP
        IF (iasipc_h_wmosatid .eq. Imiss) iasipc_h_satid = Imiss
        !!call codes_get(ibufr,'orbitNumber',iasipc_h_startorbit)

  !Nominal height and period are not in the BUFR, so get them from fdf file
        DO J=1,MaxSats
          IF (WMOid(J).EQ.iasipc_h_wmosatid) THEN
            iasipc_h_satht = SatHt(J)*10   !nominal ht, km*10
            iasipc_h_period = OrbitPd(J)   !period, seconds
            iasipc_h_satid = NOAAid(J)     !Satellite ID
          ENDIF
        ENDDO

  !Open the output file and write header

        call ioipc("OPEN/WRITE", out1cunit, 0)
        call ioipc("WRITE HEADER", out1cunit, 1)

      endif     !(imessage .eq. 1)

      !!call codes_get(ibufr,'orbitNumber',iasipc_h_endorbit)

  !Read the total number of subsets. Normally an integer multiple of the number
  !of spots, but make the code general in case scans are split across messages. 

      call codes_get(ibufr,'numberOfSubsets',nsubsets)
      scans_to_store = (nsubsets+spots_to_write)/Infovs + 1    !add 1 scan to be safe

      allocate(ivalues(nsubsets))
      allocate(rvalues(nsubsets))
      allocate(scans(nsubsets))
      allocate(spots(nsubsets))
      allocate(years(nsubsets))
      allocate(months(nsubsets))
      allocate(days(nsubsets))
      allocate(hours(nsubsets))
      allocate(minutes(nsubsets))
      allocate(seconds(nsubsets))
      allocate(iasipc_dt_store(array_size_iasipc,scans_to_store))
      iasipc_dt_store = 0

  !if there is a partial scan left over from previous message, load the data
  !into the first scan
      if (spots_to_write .ne. 0) then
        do i=1,array_size_iasipc
          iasipc_dt_store(i,1) = iasipc_dt(i)
        enddo
      endif

  !Orbit number
      call codes_get(ibufr,'orbitNumber',ivalues)
      if (imessage .eq. 1) iasipc_h_startorbit = ivalues(1)
      iasipc_h_endorbit = ivalues(nsubsets)

  !FOV number
      call codes_get(ibufr,'fieldOfViewNumber',spots)
      if (spots(1) .eq. 0) spots(:) = spots(:)+1    !EUMETSAT use 0-119
      do isub = 1,nsubsets
        if (spots(isub).le.0.or.spots(isub).gt.Infovs) then
          write(*,*)'Error: invalid fov number',spots(isub)
          call aappexit(1)
        endif
      enddo
      if (imessage .eq. 1) then
        if (spots(1).ne.1) then
          skipscan = 1      !ignore initial partial scan if there is one
          spots_to_write = nsubsets + 1 - spots(1)
        else
          skipscan = 0
          spots_to_write = nsubsets
        endif
      else
        skipscan = 0
        spots_to_write = spots_to_write + nsubsets
      endif

  !scan number
      call codes_get(ibufr,'scanLineNumber',ivalues)
      scans(1) = 1
      do isub = 2,nsubsets
        scans(isub) = scans(isub-1)
        if (spots(isub).eq.1) scans(isub) = scans(isub) + 1
      enddo
      do isub = 1,nsubsets
        iasipc_dt_store(index_scnlin,scans(isub)) = ivalues(isub)
      enddo

  !date/time
      call codes_get(ibufr,'year',years)
      call codes_get(ibufr,'month',months)
      call codes_get(ibufr,'day',days)
      call codes_get(ibufr,'hour',hours)
      call codes_get(ibufr,'minute',minutes)
      call codes_get(ibufr,'second',seconds)
        
  !transform the date/time variables
      do isub = 1,nsubsets
        if (spots(isub).eq.1) then
          Year = years(isub)
          Month = months(isub)
          Day = days(isub)
          iasipc_dt_store(index_scnlinyr, scans(isub)) = Year
          jday = DAYOFYEAR(Year,Month,Day)
          iasipc_dt_store(index_scnlindy, scans(isub)) = jday
        endif
        if (MOD(spots(isub),4) .eq. 1) then
          millisecs = hours(isub)*3600000 + minutes(isub)*60000 
     &                  + seconds(isub)*1000   
          iasipc_dt_store(index_scnlintime+spots(isub)/4, scans(isub)) = 
     &                  millisecs
        endif
      enddo

  !gqisFlagQual (store in bits 0-3 of iasipc_gqisFlagQual)
      allocate(ivalues2(nsubsets))
      allocate(ivalues3(nsubsets))
      call codes_get(ibufr,'#1#gqisFlagQual',ivalues)
      call codes_get(ibufr,'#2#gqisFlagQual',ivalues2)
      call codes_get(ibufr,'#3#gqisFlagQual',ivalues3)
      do isub=1,nsubsets
        i = index_GQisFlagQual+spots(isub)-1
        if (ivalues(isub) .eq. CODES_MISSING_LONG) then
          iasipc_dt_store(i, scans(isub)) = Imiss
        else
          j = ivalues(isub)*2 + ivalues2(isub)*4 + 
     &           ivalues3(isub)*8
          if (j .ne. 0) j = j+1
          iasipc_dt_store(i, scans(isub)) = j
        endif
      enddo
      deallocate(ivalues2)
      deallocate(ivalues3)

  !gqisFlagQualDetailed (store in bits 4+ of iasipc_gqisFlagQual)
      call codes_get(ibufr,'gqisFlagQualDetailed',ivalues)
      do isub = 1,nsubsets
        i = index_GQisFlagQual+spots(isub)-1
        if (ivalues(isub) .ne. CODES_MISSING_LONG) then
          iasipc_dt_store(i, scans(isub)) = 
     &       iasipc_dt_store(i, scans(isub)) + ivalues(isub)*16
        endif
      enddo

  !gqisQualIndex
      call codes_get(ibufr,'gqisQualIndex',ivalues)
      do isub = 1,nsubsets
        iasipc_dt_store(index_gqisQualIndex,scans(isub)) = ivalues(isub) 
      enddo

  !gqisQualIndexLoc
      call codes_get(ibufr,'gqisQualIndexLoc',ivalues)
      do isub = 1,nsubsets
        iasipc_dt_store(index_gqisQualIndexLoc,scans(isub)) =
     &      ivalues(isub) 
      enddo

  !gqisQualIndexRad
      call codes_get(ibufr,'gqisQualIndexRad',ivalues)
      do isub = 1,nsubsets
        iasipc_dt_store(index_gqisQualIndexRad,scans(isub)) =
     &      ivalues(isub) 
      enddo

  !gqisQualIndexSpect
      call codes_get(ibufr,'gqisQualIndexSpect',ivalues)
      do isub = 1,nsubsets
        iasipc_dt_store(index_gqisQualIndexSpect,scans(isub)) =
     &      ivalues(isub) 
      enddo

  !gqisSysTecQual
      call codes_get(ibufr,'gqisSysTecQual',ivalues)
      do isub = 1,nsubsets
        iasipc_dt_store(index_GQisSysTecSondQual,scans(isub)) =
     &      ivalues(isub) 
      enddo

  !Scale factors (note we have already used three 'startChannel' descriptors)
      do iband=1,10
        write(descriptor,formatstr(iband+3))'#',iband+3,'#startChannel'
        call codes_get(ibufr,descriptor,i)
        iasipc_h_startchan(iband) = FNISCALEEC(i,1)
        write(descriptor,formatstr(iband+3))'#',iband+3,'#endChannel'
        call codes_get(ibufr,descriptor,i)
        iasipc_h_endchan(iband) = FNISCALEEC(i,1)
        write(descriptor,formatstr(iband)) 
     &        '#',iband,'#channelScaleFactor'
        call codes_get(ibufr,descriptor,i)
        iasipc_h_scalefactor(iband) = FNISCALEEC(i,1)
      enddo

  !latitude
      call codes_get(ibufr,'latitude',rvalues)
      do isub = 1,nsubsets
        iasipc_dt_store(index_latlon+2*spots(isub)-2, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D4)
      enddo

  !longitude
      call codes_get(ibufr,'longitude',rvalues)
      do isub = 1,nsubsets
        iasipc_dt_store(index_latlon+2*spots(isub)-1, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D4)
      enddo

  !scalti
      call codes_get(ibufr,'heightOfStation',rvalues)
      do isub = 1,nsubsets,Infovs
        iasipc_dt_store(index_scalti,scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D-2)  !store as km*10
      enddo

  !sat zenith
      call codes_get(ibufr,'satelliteZenithAngle',rvalues)
      do isub = 1,nsubsets
        iasipc_dt_store(index_angles+4*spots(isub)-4, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !sat azimuth (0 to 360 deg)
      call codes_get(ibufr,'bearingOrAzimuth',rvalues)
      do isub = 1,nsubsets
        iasipc_dt_store(index_angles+4*spots(isub)-3, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !solar zenith
      call codes_get(ibufr,'solarZenithAngle',rvalues)
      do isub = 1,nsubsets
        iasipc_dt_store(index_angles+4*spots(isub)-2, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !solar azimuth (0 to 360 deg)
      call codes_get(ibufr,'solarAzimuth',rvalues)
      do isub = 1,nsubsets
        iasipc_dt_store(index_angles+4*spots(isub)-1, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !Channels
      if (imessage .eq. 1) then
        do ichan=1,iasipc_h_nchanused
          write(descriptor,formatstr(ichan)) '#',ichan,'#channelNumber'
          call codes_get(ibufr, descriptor, iasipc_h_channels(ichan))
        enddo
      endif

  !Scaled radiance (stored as integer*2 in AAPP format)
      do ichan=1,iasipc_h_nchanused
        write(descriptor,formatstr(ichan))
     &     '#',ichan,'#scaledIasiRadiance'
        call codes_get(ibufr,descriptor,ivalues)
        jchan = (ichan-1)/2 + 1        !1, 1, 2, 2, 3, 3, etc.
        kchan = MOD((ichan-1), 2)+1    !1, 2, 1, 2, 1, 2, etc.
        do isub = 1,nsubsets
          if (ivalues(isub) .gt. 32767) ivalues(isub)=ivalues(isub)-65536
          scalrad_i2(kchan) = ivalues(isub)
          scalrad_i2(3-kchan) = 0
          i = index_scalrad+jchan-1+(spots(isub)-1)*iasi_pcnchan/2
          iasipc_dt_store(i,scans(isub)) = 
     &        IOR(iasipc_dt_store(i,scans(isub)),scalrad_i4(1))
        enddo
      enddo

  !PC scores
      do iband=1,3
        write(descriptor,formatstr(iband+13))
     &     '#',iband+13,'#startChannel'
        call codes_get(ibufr,descriptor,i)
        iasipc_h_startpcchan(iband) = FNISCALEEC(i,1)

        write(descriptor,formatstr(iband+13))
     &     '#',iband+13,'#endChannel'
        call codes_get(ibufr,descriptor,i)
        iasipc_h_endpcchan(iband) = FNISCALEEC(i,1)

        write(descriptor,formatstr(iband))
     &     '#',iband,'#scoreQuantizationFactor'
        call codes_get(ibufr,descriptor,rtmp)
        iasipc_h_quantization(iband) = FNSCALEEC(rtmp,1.0D2)

        write(descriptor,formatstr(iband))
     &     '#',iband,'#databaseIdentification'
        call codes_get(ibufr,descriptor,i)
        iasipc_h_evfilevn(iband) = FNISCALEEC(i,1)

        write(descriptor,formatstr(iband))
     &     '#',iband,'#residualRmsInBand'
        call codes_get(ibufr,descriptor,rvalues)
        do isub = 1,nsubsets
          if (rvalues(isub) .ne. CODES_MISSING_DOUBLE) then
            ResidualRMS = rvalues(isub)    !ResidualRMS is default precision
            logfit = NINT(alog10(ResidualRMS)*1.0D4)
          else
            logfit = Imiss
          endif
          iasipc_dt_store(index_logfit+iband-1+3*(spots(isub)-1),
     &      scans(isub)) = logfit
        enddo

        write(descriptor,formatstr(iband+1))
     &     '#',iband+1,'#extendedDelayedDescriptorReplicationFactor'
        call codes_get(ibufr,descriptor,Npc)
        if (iband .eq. 1) then
          iasipc_h_npcused = Npc
          iasipc_h_startpc(iband) = 1
          iasipc_h_endpc(iband) = Npc
        else
          iasipc_h_npcused = iasipc_h_npcused + Npc
          iasipc_h_startpc(iband) = iasipc_h_endpc(iband-1) + 1
          iasipc_h_endpc(iband) = iasipc_h_endpc(iband-1) + Npc
        endif
        if (Npc .gt. 1 .and. imessage .eq. 1) 
     &      iasipc_h_npcbands = iasipc_h_npcbands+1

        do Ipc=1,Npc
          K = iasipc_h_startpc(iband)+Ipc-1
          write(descriptor,formatstr(K))
     &       '#',K,'#nonNormalizedPrincipalComponentScore'
          call codes_get(ibufr,descriptor,ivalues)
          do isub = 1,nsubsets
            iasipc_dt_store(index_scores+K-1+
     &        (spots(isub)-1)*iasi_pcnpc,scans(isub)) = 
     &        FNISCALEEC(ivalues(isub),1)
          enddo
        enddo
      enddo

  !AVHRR channel combination
      call codes_get(ibufr,'avhrrChannelCombination',ivalues)
      do isub = 1,nsubsets
        iasipc_dt_store(index_avhcombination, scans(isub))=ivalues(isub)
      enddo

  !avhrrCGy, avhrrCGz, avhrrfrac
      do iclass=1,7
        write(descriptor,formatstr(iclass))
     &    '#',iclass,'#yAngularPositionFromCentreOfGravity'  
        call codes_get(ibufr,descriptor,rvalues)
        do isub = 1,nsubsets
          i = index_avhrrCGy + (spots(isub)-1)*7 + iclass-1
          iasipc_dt_store(i,scans(isub))=FNSCALEEC(rvalues(isub),1.0D6)
        enddo

        write(descriptor,formatstr(iclass))
     &    '#',iclass,'#zAngularPositionFromCentreOfGravity'  
        call codes_get(ibufr,descriptor,rvalues)
        do isub = 1,nsubsets
          i = index_avhrrCGz + (spots(isub)-1)*7 + iclass-1
          iasipc_dt_store(i,scans(isub))=FNSCALEEC(rvalues(isub),1.0D6)
        enddo

        write(descriptor,formatstr(iclass))
     &    '#',iclass,'#fractionOfClearPixelsInHirsFov'  
        call codes_get(ibufr,descriptor,rvalues)
        do isub = 1,nsubsets
          i = index_avhrrfrac + (spots(isub)-1)*7 + iclass-1
          iasipc_dt_store(i,scans(isub))=FNSCALEEC(rvalues(isub),1.0D4)
        enddo
      enddo

  !avhrrmean, avhrrsdev, avhmeanscalefactor, avhsdevscalefactor
      do ichan=1,6
        do iclass=1,7
          iband = ichan + (iclass-1)*6
          write(descriptor,formatstr(iband))
     &         '#',iband,'#scaledMeanAvhrrRadiance'
          call codes_get(ibufr,descriptor,ivalues)
          do isub = 1,nsubsets
            i = index_avhrrmean + (spots(isub)-1)*42 
     &                    + (iclass-1)*6 + ichan-1
            iasipc_dt_store(i,scans(isub)) = ivalues(isub)
          enddo
          write(descriptor,formatstr(iband))
     &         '#',iband,'#scaledStandardDeviationAvhrrRadiance'
          call codes_get(ibufr,descriptor,ivalues)
          do isub = 1,nsubsets
            i = index_avhrrsdev + (spots(isub)-1)*42 
     &                    + (iclass-1)*6 + ichan-1
            iasipc_dt_store(i,scans(isub)) = ivalues(isub)
          enddo

  !note: ignore the first 10 entries of channelScaleFactor as they have already
  !been used in scaled radiances.
          do j=1,2
            iband = 10 + j + (ichan-1)*2 + (iclass-1)*12
            write(descriptor,formatstr(iband))
     &         '#',iband,'#channelScaleFactor'
            call codes_get(ibufr,descriptor,ivalues)
            do isub = 1,nsubsets
              if (j .eq. 1) then
                i = index_avhmeanscalefactor + (spots(isub)-1)*42
     &                    + (iclass-1)*6 + ichan-1
              else
                i = index_avhsdevscalefactor + (spots(isub)-1)*42
     &                    + (iclass-1)*6 + ichan-1
              endif
              iasipc_dt_store(i,scans(isub)) = ivalues(isub)
            enddo
          enddo
        enddo
      enddo

  !GIacAvgImagIIS (30 values across scan)
      call codes_get(ibufr,'averageOfImagerMeasurements',rvalues)
      do isub = 1,nsubsets,4
        iasipc_dt_store(index_GIacAvgImagIIS+(spots(isub)-1)/4,
     &       scans(isub)) = FNSCALEEC(rvalues(isub),1.0D6)
      enddo

  !GIacVarImagIIS (30 values across scan)
      call codes_get(ibufr,'giacvarimagiisVarianceOfImagerMeasurements',
     &       rvalues)
      do isub = 1,nsubsets,4
        iasipc_dt_store(index_GIacVarImagIIS+(spots(isub)-1)/4,
     &       scans(isub)) = FNSCALEEC(rvalues(isub),1.0D6)
      enddo

  !GEUMAvhrr1BCldFrac
      call codes_get(ibufr,'cloudAmountInSegment',ivalues)
      do isub = 1,nsubsets
        i = index_GEUMAvhrr1BCldFrac+spots(isub)-1
        if (ivalues(isub) .ne. CODES_MISSING_LONG) then
          iasipc_dt_store(i, scans(isub)) = ivalues(isub)
        else
          iasipc_dt_store(i, scans(isub)) = 255
        endif
      enddo

  !GEUMAvhrr1BLandFrac
      call codes_get(ibufr,'amountOfSegmentCoveredByScene',ivalues)
      do isub = 1,nsubsets
        i = index_GEUMAvhrr1BLandFrac+spots(isub)-1
        if (ivalues(isub) .ne. CODES_MISSING_LONG) then
          iasipc_dt_store(i,scans(isub)) = ivalues(isub)
        else
          iasipc_dt_store(i,scans(isub)) = 255
        endif
      enddo

  !GEUMAvhrr1BQual
  !This is the snow/ice cover if available. Otherwise it is the nuber of bad
  !pixels (0-126) and bit 7 is set to 1. Set to -1 is missing.

      call codes_get(ibufr,
     &  'fractionOfWeightedAvhrrPixelInIasiFovCoveredWithSnowOrIce',
     &  ivalues)
      do isub = 1,nsubsets
        i = index_GEUMAvhrr1BQual+spots(isub)-1
        if (ivalues(isub) .ne. CODES_MISSING_LONG) then
          iasipc_dt_store(i,scans(isub)) = ivalues(isub)
        else
          iasipc_dt_store(i,scans(isub)) = -1
        endif
      enddo

      call codes_get(ibufr,'numberOfMissingBadOrFailedAvhrrPixels',
     &  ivalues)
      do isub = 1,nsubsets
        i = index_GEUMAvhrr1BQual+spots(isub)-1
        if (iasipc_dt_store(i,scans(isub)) .eq. -1) then
          if (ivalues(isub) .ne. CODES_MISSING_LONG) then
            iasipc_dt_store(i,scans(isub)) = ivalues(isub) + 128   !Set bit 7
          endif
        endif
      enddo

  !save the data
      scans_to_write = spots_to_write/Infovs
      DO iscan=1,scans_to_write
        spots_to_write = spots_to_write - Infovs
        do i=1,array_size_iasipc
          iasipc_dt(i) = iasipc_dt_store(i,iscan+skipscan)
        enddo
        iasipc_h_scnlin = iasipc_h_scnlin + 1
        if (iasipc_h_startdatatime .lt. 0) then
          iasipc_h_startdatatime = iasipc_scnlintime(1)
          iasipc_h_startdatady = iasipc_scnlindy
          iasipc_h_startdatayr = iasipc_scnlinyr
        endif
        iasipc_h_enddatatime = iasipc_scnlintime(iasi_pcnfov)
        iasipc_h_enddatady = iasipc_scnlindy
        iasipc_h_enddatayr = iasipc_scnlinyr
        if ((iasipc_scnlintime(1)-iasipc_scnlintime(iasi_pcnfov))
     &      .gt.86392000) then
          iasipc_h_enddatady = iasipc_h_enddatady + 1
          if ((iasipc_h_enddatady .gt. 366) .or. 
     &         (MOD(iasipc_h_enddatayr,4) .ne. 0 .and. 
     &         iasipc_h_enddatady .gt. 365)) then
            iasipc_h_enddatady = 1
            iasipc_h_enddatayr = iasipc_h_enddatayr + 1
          endif
        endif
        call ioipc("WRITE DATA", out1cunit,
     &      iasipc_h_scnlin+iasipc_h_hdrcnt)
      ENDDO

  !Update the header
      call ioipc("WRITE HEADER", out1cunit, 1)
      write(*,*)'Message:',imessage, ' subsets:',nsubsets,
     &  ' scan count:',iasipc_h_scnlin

  !If there is a partial scan left over, save the data
      if (spots_to_write .ne. 0) then
        do i=1,array_size_iasipc
          iasipc_dt(i) = iasipc_dt_store(i,scans_to_write+1)
        enddo
      endif

  !Deallocate arrays before exiting the subroutine
      if(allocated(ivalues)) deallocate(ivalues)
      if(allocated(rvalues)) deallocate(rvalues)
      if(allocated(scans)) deallocate(scans)
      if(allocated(spots)) deallocate(spots)
      if(allocated(years)) deallocate(years)
      if(allocated(months)) deallocate(months)
      if(allocated(days)) deallocate(days)
      if(allocated(hours)) deallocate(hours)
      if(allocated(minutes)) deallocate(minutes)
      if(allocated(seconds)) deallocate(seconds)
      if(allocated(iasipc_dt_store)) deallocate(iasipc_dt_store)

#endif
      end subroutine eccodes_get_1c_pciasi
