      subroutine eccodes_put_1c_amsua(ibufr,in1cunit,imessage,iscan,
     &  finished,ICENTREID,ISUBCENTREID,ICENTREID_1B,
     &  ISUBCENTREID_1B,AAPP_VERSION,ENCODE_NEDT)

#ifdef HAS_LIBECCODES
      use eccodes
#endif
      implicit none

!-----------------------------------------------------------------------
!
! Name: $Id: eccodes_put_1c_amsua.F 519 2017-05-05 10:04:04Z frna $
!
! Purpose: BUFR encoder subroutine for AMSU-A l1c
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 7 December 2016, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, DWD and MeteoFrance.
!
!    Copyright 2017, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0     24/01/2017 Initial
!
! Code Description:
!   Fortran90, but using FORTRAN77 fixed-formatting, in order to interface with
!   existing AAPP "include" files.
!
! ecCodes subroutines: codes_set
! AAPP subroutines: ioa1c, BITSWAP
!
! Declarations:

#include "inparms.h"
#include "ama1c.h"

!Functions
      REAL*8 FNRSCALEEC

!Arguments
      integer ibufr
      integer in1cunit
      integer imessage
      integer iscan
      logical finished
      INTEGER ICENTREID       !Centre code (1c in Section 4)
      INTEGER ISUBCENTREID    !Centre code (1c in Section 4)
      INTEGER ICENTREID_1B    !Centre code (1b in Section 4)
      INTEGER ISUBCENTREID_1B !Centre code (1b in Section 4)
      INTEGER AAPP_VERSION    !e.g. 715 for v7.15
      LOGICAL ENCODE_NEDT
      
#ifdef HAS_LIBECCODES

  !Parameters and fixed items
      integer TimeDelay
      DATA TimeDelay/203/   !milliseconds per spot for AMSU-A
      integer scans_per_message
      PARAMETER (scans_per_message = 5)  !Use several scans for longer messages

  !Miscellaneous variables 
      integer i, irec, isub, ispot, ichan, jchan, jscan
      integer number_of_messages
      integer nsubsets
      real*8  rtmp

  !Input data store
      integer ama1c_dt_store(array_size_aa1c,scans_per_message)
      integer scans(scans_per_message*Anfovs)
      integer spots(scans_per_message*Anfovs)

  !Date/time arrays
      integer years(Anfovs,scans_per_message)
      integer months(Anfovs,scans_per_message)
      integer days(Anfovs,scans_per_message)
      integer hours(Anfovs,scans_per_message)
      integer minutes(Anfovs,scans_per_message)
      real*8  seconds(Anfovs,scans_per_message)
      integer scnlindy,scnlinyr,FovTime  !date and time
      integer Month,Day,Hour,Min
      real*8  Sec
      
  !Index variables for AAPP l1c array
      integer, save :: index_scnlin, index_scnlinyr, index_scnlindy,
     & index_scnlintime, index_qualind, index_scanqual, 
     & index_chanqual, index_instrtemp1, index_instrtemp2,
     & index_spare1, index_latlon, index_angles,
     & index_scalti, index_spare2, index_btemps, index_dataqual,
     & index_coldnedt, index_warmnedt, index_filler

  !Arrays that are passed to eccodes
      real*8,    dimension(:), allocatable :: rvalues
      integer*4, dimension(:), allocatable :: ivalues
      character*80 descriptor
      
  !Variables to be saved between calls
      character*10, save :: formatstr(2*Anchan)

  !Arrays for flag table manipulation 
      INTEGER    NumBitsIn          !- size of raw data bit fields
      PARAMETER  (NumBitsIn  = 32)
      INTEGER    NumBitsOut         !- set BUFR bit field width
      PARAMETER  (NumBitsOut = 24)
      INTEGER BIT                   !For use with unbitswap
      INTEGER Mask033030(NumBitsIn) !- bit mask for 033030
      INTEGER Mask033031(NumBitsIn) !- bit mask for 033031
      INTEGER Mask033032(NumBitsIn) !- bit mask for 033032
      INTEGER Mask033033(NumBitsIn) !- bit mask for 033033
      DATA Mask033030 /1,2,3,4,5,6,7,25*0/
      DATA Mask033031 /8*0,1,2,3,4,4*0,5,6,7,8,9,10,11,12,13,14,15,16,
     &                 17,3*0/
      DATA Mask033032 /26*0,1,2,3,4,5,6/
      DATA Mask033033 /0,1,14*0,16,15,14,13,12,11,10,9,8,7,6,5,4,3,2,22/
      
!--------------------------------------------------------------

      imessage = imessage + 1

      if (imessage .eq. 1) then

!Format string to use with multi-channel descriptors
        do ichan=1,9
          write(formatstr(ichan),'(A)') '(A,I1,A)'
        enddo
        do ichan=10,2*Anchan
          write(formatstr(ichan),'(A)') '(A,I2,A)'
        enddo

!Find the position in the AAPP data record of each variable
        do i=1,array_size_aa1c
          ama1c_dt(i) = i
        enddo
        index_scnlin = ama1c_scnlin
        index_scnlinyr = ama1c_scnlinyr
        index_scnlindy = ama1c_scnlindy
        index_scnlintime = ama1c_scnlintime
        index_qualind = ama1c_qualind
        index_scanqual = ama1c_scanqual
        index_chanqual = ama1c_chanqual(1)
        index_instrtemp1 = ama1c_instrtemp1
        index_instrtemp2 = ama1c_instrtemp2
        index_spare1 = ama1c_spare1(1)
        index_latlon = ama1c_latlon(1,1)
        index_angles = ama1c_angles(1,1)
        index_scalti = ama1c_scalti
        index_spare2 = ama1c_spare2(1)
        index_btemps = ama1c_btemps(1,1)
        index_dataqual = ama1c_dataqual(1)
        index_coldnedt = ama1c_coldnedt(1)
        index_warmnedt = ama1c_warmnedt(1)
        index_filler = ama1c_filler(1)
      
!Open the l1c file and read the header

        call ioa1c("OPEN/READ", in1cunit, 0)
        call ioa1c("READ HEADER", in1cunit, 1)

      endif
        
      i = scans_per_message-1
      number_of_messages = (ama1c_h_scnlin+i)/scans_per_message
      write(*,*)'Message:',imessage,' of',number_of_messages
      
      nsubsets = 0
      do jscan=1,scans_per_message
        if (iscan .lt. ama1c_h_scnlin) then
          iscan = iscan + 1
          irec = iscan + ama1c_h_hdrcnt
          call ioa1c("READ DATA", in1cunit, irec)

          !Convert year, day_of_year, milliseconds to YYMMDD HHMMSS

          do ispot=1,Anfovs
            scnlindy = ama1c_scnlindy
            scnlinyr = ama1c_scnlinyr
            FovTime = ama1c_scnlintime + (ispot-1)*TimeDelay
            CALL DAY_TO_DATE(scnlinyr,scnlindy,FovTime,
     &           Month,Day,Hour,Min,Sec)            
            years(ispot,jscan) = scnlinyr
            months(ispot,jscan) = Month
            days(ispot,jscan) = Day
            hours(ispot,jscan) = Hour
            minutes(ispot,jscan) = Min
            seconds(ispot,jscan) = Sec
          enddo

          nsubsets = nsubsets + Anfovs
          do i=1,array_size_aa1c
            ama1c_dt_store(i,jscan) = ama1c_dt(i)
          enddo
          irec = irec + 1
        endif
      enddo

!Section 1 information

      call codes_set(ibufr,'dataCategory',3)                 !Table C-13
      call codes_set(ibufr,'internationalDataSubCategory',3) !Table C-13 - AMSUA
      call codes_set(ibufr,'typicalYear',years(1,1))
      call codes_set(ibufr,'typicalMonth',months(1,1))
      call codes_set(ibufr,'typicalDay',days(1,1))
      call codes_set(ibufr,'typicalHour',hours(1,1))
      call codes_set(ibufr,'typicalMinute',minutes(1,1))
      call codes_set(ibufr,'typicalSecond',seconds(1,1))
      call codes_set(ibufr,'numberOfSubsets',nsubsets)
      if (ENCODE_NEDT) then
        allocate(ivalues(127))
        ivalues=(/
     &      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     &      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     &      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     &      1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     &      1, 1, 0, 1, 1, 1, 1, 1, 0, 1,
     &      1, 1, 1, 1, 0, 1, 1, 1, 1, 1,
     &      0, 1, 1, 1, 1, 1, 0, 1, 1, 1,
     &      1, 1, 0, 1, 1, 1, 1, 1, 0, 1,
     &      1, 1, 1, 1, 0, 1, 1, 1, 1, 1,
     &      0, 1, 1, 1, 1, 1, 0, 1, 1, 1,
     &      1, 1, 0, 1, 1, 1, 1, 1, 0, 1,
     &      1, 1, 1, 1, 0, 1, 1, 1, 1, 1,
     &      0, 1, 1, 1, 1, 1, 0 /)
        call codes_set(ibufr,'inputDataPresentIndicator',ivalues)
        deallocate(ivalues)
        if (AAPP_VERSION .ne. 0) then
          allocate(ivalues(10))
          ivalues=(/ 310009, 222000, 236000, 101127, 31031,
     &      1031, 1032, 101015, 33089, 25060 /)
        else  
          allocate(ivalues(9))
          ivalues=(/ 310009, 222000, 236000, 101127, 31031,
     &      1031, 1032, 101015, 33089 /)
        endif
      else
        if (AAPP_VERSION .ne. 0) then
          allocate(ivalues(2))
          ivalues=(/ 310009, 25060 /)
        else  
          allocate(ivalues(1))
          ivalues=(/ 310009 /)
        endif
      endif
      call codes_set(ibufr,'unexpandedDescriptors',ivalues)
      deallocate(ivalues)
      call codes_set(ibufr,'#1#tovsOrAtovsProductQualifier',3)  !1C (0-08-070)
      call codes_set(ibufr,'#1#centre',ICENTREID)
      call codes_set(ibufr,'#1#subCentre',ISUBCENTREID)
      call codes_set(ibufr,'#2#tovsOrAtovsProductQualifier',2)  !1B (0-08-070)
      call codes_set(ibufr,'#2#centre',ICENTREID_1B)
      call codes_set(ibufr,'#2#subCentre',ISUBCENTREID_1B)
      call codes_set(ibufr,'satelliteIdentifier',ama1c_h_wmosatid)
      call codes_set(ibufr,'satelliteSensorIndicator',3)    !AMSU-A (0-02-048)
      call codes_set(ibufr,'orbitNumber',ama1c_h_startorbit)
      call codes_set(ibufr,'satelliteAntennaCorrectionsVersionNumber',
     &    ama1c_h_vnantennacorr)

      allocate(ivalues(nsubsets))
      allocate(rvalues(nsubsets))

      do isub = 1,nsubsets
        scans(isub) = (isub+Anfovs-1)/Anfovs
        spots(isub) = MOD(isub-1,Anfovs)+1
      enddo

!Encode the data variables

  !Scan number
      do isub = 1,nsubsets
        ivalues(isub) = ama1c_dt_store(index_scnlin,scans(isub))
      enddo
      call codes_set(ibufr,'scanLineNumber',ivalues)
        
  !FOV number
      do isub = 1,nsubsets
        ivalues(isub) = spots(isub)
      enddo
      call codes_set(ibufr,'fieldOfViewNumber',ivalues)
        
  !qualind
      do isub = 1,nsubsets
        i = ama1c_dt_store(index_qualind,scans(isub))
        call bitswap(i, Mask033030, NumBitsIn, Bit, NumBitsOut)
        ivalues(isub) = Bit
      enddo
      call codes_set(ibufr,'scanLineStatusFlagsForAtovs',ivalues)
        
  !scanqual
      do isub = 1,nsubsets
        i = ama1c_dt_store(index_scanqual,scans(isub))
        call bitswap(i, Mask033031, NumBitsIn, Bit, NumBitsOut)
        ivalues(isub) = Bit
      enddo
      call codes_set(ibufr,'scanLineQualityFlagsForAtovs',ivalues)
        
  !year
      do isub = 1,nsubsets
        ivalues(isub) = years(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'year',ivalues)
        
  !month
      do isub = 1,nsubsets
        ivalues(isub) = months(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'month',ivalues)
        
  !day
      do isub = 1,nsubsets
        ivalues(isub) = days(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'day',ivalues)
        
  !hour
      do isub = 1,nsubsets
        ivalues(isub) = hours(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'hour',ivalues)
        
  !minute
      do isub = 1,nsubsets
        ivalues(isub) = minutes(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'minute',ivalues)
        
  !second
      do isub = 1,nsubsets
        rvalues(isub) = seconds(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'second',rvalues)
        
  !latitude
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(ama1c_dt_store(index_latlon+2*spots(isub)-2,
     &        scans(isub)), 1.0D-4)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'latitude',rvalues)
        
  !longitude
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(ama1c_dt_store(index_latlon+2*spots(isub)-1,
     &        scans(isub)), 1.0D-4)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'longitude',rvalues)
        
  !scalti
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(ama1c_dt_store(index_scalti,
     &        scans(isub)), 1.0D2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'heightOfStation',rvalues)
        
  !sat zenith
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(ama1c_dt_store(index_angles+4*spots(isub)-4,
     &        scans(isub)), 1.0D-2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'satelliteZenithAngle',rvalues)
        
  !sat azimuth
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(ama1c_dt_store(index_angles+4*spots(isub)-3,
     &        scans(isub)), 1.0D-2)
        if (rtmp.LT.0 .and. rtmp.GT.-360) rtmp = rtmp + 360.0D0
        rvalues(isub) = rtmp 
      enddo
      call codes_set(ibufr,'bearingOrAzimuth',rvalues)
        
  !solar zenith
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(ama1c_dt_store(index_angles+4*spots(isub)-2,
     &        scans(isub)), 1.0D-2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'solarZenithAngle',rvalues)
        
  !solar azimuth
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(ama1c_dt_store(index_angles+4*spots(isub)-1,
     &        scans(isub)), 1.0D-2)
        if (rtmp.LT.0 .and. rtmp.GT.-360) rtmp = rtmp + 360.0D0
        rvalues(isub) = rtmp 
      enddo
      call codes_set(ibufr,'solarAzimuth',rvalues)

  !dataqual
      do isub = 1,nsubsets
        i = ama1c_dt_store(index_dataqual+spots(isub)-1,scans(isub))
        call bitswap(i, Mask033033, NumBitsIn, Bit, NumBitsOut)
        ivalues(isub) = Bit
      enddo
      call codes_set(ibufr,'fieldOfViewQualityFlagsForAtovs',ivalues)

  !instrument temps
      call codes_set(ibufr,'#1#radiometerIdentifier',0)   !HIRS
      call codes_set(ibufr,'#2#radiometerIdentifier',3)   !AMSU-A1-1
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(ama1c_dt_store(index_instrtemp1,
     &        scans(isub)), 1.0D-2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'#2#instrumentTemperature',rvalues)
      call codes_set(ibufr,'#3#radiometerIdentifier',4)   !AMSU-A1-2
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(ama1c_dt_store(index_instrtemp2,
     &        scans(isub)), 1.0D-2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'#3#instrumentTemperature',rvalues)
      call codes_set(ibufr,'#4#radiometerIdentifier',6)   !AMSU-B

  !channel number
      do ichan=1,Anchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#tovsOrAtovsOrAvhrrInstrumentationChannelNumber'
        call codes_set(ibufr, descriptor, ichan+27)
      enddo

  !wavenumber
      do ichan=1,Anchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#log10OfTemperatureRadianceCentralWaveNumberForAtovs'
        call codes_set(ibufr, descriptor, 
     &       log10(1.0D-4 * ama1c_h_tempradcnv(1,ichan)))
      enddo

  !band corr 1
      do ichan=1,Anchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#bandwidthCorrectionCoefficient1ForAtovs'
        call codes_set(ibufr, descriptor, 
     &       1.0D-6*ama1c_h_tempradcnv(2,ichan))
      enddo

  !band corr 2
      do ichan=1,Anchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#bandwidthCorrectionCoefficient2ForAtovs'
        call codes_set(ibufr, descriptor, 
     &       1.0D-6*ama1c_h_tempradcnv(3,ichan))
      enddo

  !chanqual (chan,scan)
      do ichan=1,Anchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#channelQualityFlagsForAtovs'
        do isub = 1,nsubsets
          i = ama1c_dt_store(index_chanqual+ichan-1, scans(isub))
          call bitswap(i, Mask033032, NumBitsIn, Bit, NumBitsOut)
          ivalues(isub) = Bit
        enddo   
        call codes_set(ibufr,descriptor,ivalues)
      enddo

  !btemps (chan,spot,scan)
      do ichan=1,Anchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#brightnessTemperature'
        do isub = 1,nsubsets
          rtmp = FNRSCALEEC(ama1c_dt_store(index_btemps + 
     &        ichan-1 + Anchan*(spots(isub)-1), scans(isub)), 1.0D-2)
          rvalues(isub) = rtmp
        enddo
        call codes_set(ibufr,descriptor,rvalues)
      enddo

  !NEDT
      if (ENCODE_NEDT) then
        do ichan=1,Anchan
          jchan = ichan + Anchan
          write(descriptor,formatstr(jchan)) '#',jchan,
     &       '#brightnessTemperature'
          do isub = 1,nsubsets
            rtmp = FNRSCALEEC(ama1c_dt_store(index_warmnedt + 
     &          ichan-1, scans(isub)), 1.0D-3)
            rvalues(isub) = rtmp
          enddo
          call codes_set(ibufr,descriptor,rvalues)
        enddo
        call codes_set(ibufr,'#3#centre',ICENTREID)
        call codes_set(ibufr,'generatingApplication',0)   !Can be defined by centre
      endif

  !AAPP version
      if (AAPP_VERSION .NE. 0)
     &    call codes_set(ibufr,'softwareIdentification',AAPP_VERSION)

  !Deallocate arrays before exiting the subroutine
      deallocate(ivalues)
      deallocate(rvalues)

  !Have we finished?
      if (iscan .eq. ama1c_h_scnlin) then
        finished = .true.
        call ioa1c("CLOSE", in1cunit, 0)
      endif

#endif
      RETURN
      END subroutine eccodes_put_1c_amsua
