      subroutine eccodes_put_1c_cris(ibufr,in1cunit,imessage,iscan,
     &  finished,ICENTREID,AAPP_VERSION)

#ifdef HAS_LIBECCODES
      use eccodes
#endif
      implicit none

!-----------------------------------------------------------------------
!
! Name: $Id: eccodes_put_1c_cris.F 579 2018-02-12 11:09:36Z frna $
!
! Purpose: BUFR encoder subroutine for CrIS l1c
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 7 December 2016, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, DWD and MeteoFrance.
!
!    Copyright 2017, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0     27/04/2017 Initial
!
! Code Description:
!   Fortran90, but using FORTRAN77 fixed-formatting, in order to interface with
!   existing AAPP "include" files.
!
! ecCodes subroutines: codes_set
! AAPP subroutines: ioc1c
!
! Declarations:

#include "constants_preproc.h"
#include "inparms.h"
#include "cris1c.h"

!Functions
      REAL*8 FNRSCALEEC
      INTEGER FNJSCALEEC

!Arguments
      integer ibufr
      integer in1cunit
      integer imessage
      integer iscan
      logical finished
      INTEGER ICENTREID       !Centre code (1c in Section 4)
      INTEGER AAPP_VERSION    !e.g. 715 for v7.15
      
#ifdef HAS_LIBECCODES

  !Parameters and fixed items
      integer scans_per_message
      PARAMETER (scans_per_message = 1)  !Use several scans for longer messages
      INTEGER   Imiss
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      INTEGER   Cnfovs
      PARAMETER (Cnfovs=cris_1cnfov*cris_1cnfor)

  !Miscellaneous variables 
      integer, save :: messages_per_scan
      integer i, irec, isub, jsub, ispot, ichan, jchan, jscan
      integer number_of_messages
      integer nsubsets
      real*8  rtmp
      real*8  freq
      CHARACTER*3 CMESSAGES_PER_SCAN

  !Input data store
      integer cris1c_dt_store(array_size_cris1c,scans_per_message)
      integer scans(scans_per_message*Cnfovs)
      integer spots(scans_per_message*Cnfovs)
      integer fovs(scans_per_message*Cnfovs)
      integer fors(scans_per_message*Cnfovs)

  !Date/time arrays
      integer years(Cnfovs,scans_per_message)
      integer months(Cnfovs,scans_per_message)
      integer days(Cnfovs,scans_per_message)
      integer hours(Cnfovs,scans_per_message)
      integer minutes(Cnfovs,scans_per_message)
      real*8  seconds(Cnfovs,scans_per_message)
      integer scnlindy,scnlinyr,FovTime  !date and time
      integer Month,Day,Hour,Min
      real*8  Sec
      
  !Index variables for AAPP l1c array
      integer, save :: index_scnlin,index_scnlinyr,index_scnlindy,
     & index_scnlintime,index_latlon,index_angles,
     & index_satxyz,index_ascdesc,index_orbit,index_scalti,
     & index_surfht,index_landfrac,index_landsea,index_cloudcover,
     & index_cloudtopht,index_radianceflgs,index_scanqualflgs,
     & index_calqualflgs,index_fovqualflgs,index_geolocqual,
     & index_quality,index_scores,index_logfit,index_radiance

  !Arrays that are passed to eccodes
      real*8,    dimension(:), allocatable :: rvalues
      integer*4, dimension(:), allocatable :: ivalues
      character*80 descriptor
      
  !Variables to be saved between calls
      character*10, save :: formatstr(cris_1cnchan)

!--------------------------------------------------------------

      imessage = imessage + 1

      if (imessage .eq. 1) then

        messages_per_scan = 1    !default
        CALL GETENV('MESSAGES_PER_SCAN_CRIS',CMESSAGES_PER_SCAN)
        READ(CMESSAGES_PER_SCAN,*,ERR=101,END=101) messages_per_scan
  101   CONTINUE
        if (scans_per_message .gt. 1) messages_per_scan = 1

!Format string to use with multi-channel descriptors
        do ichan=1,9
          write(formatstr(ichan),'(A)') '(A,I1,A)'
        enddo
        do ichan=10,99
          write(formatstr(ichan),'(A)') '(A,I2,A)'
        enddo
        do ichan=100,999
          write(formatstr(ichan),'(A)') '(A,I3,A)'
        enddo
        do ichan=1000,cris_1cnchan
          write(formatstr(ichan),'(A)') '(A,I4,A)'
        enddo

!Find the position in the AAPP data record of each variable
        do i=1,array_size_cris1c
          cris1c_dt(i) = i
        enddo
        index_scnlin = cris1c_scnlin
        index_scnlinyr = cris1c_scnlinyr
        index_scnlindy = cris1c_scnlindy
        index_scnlintime = cris1c_scnlintime(1)
        index_latlon = cris1c_latlon(1,1,1)
        index_angles = cris1c_angles(1,1,1)
        index_satxyz = cris1c_satxyz(1)
        index_ascdesc = cris1c_ascdesc
        index_orbit = cris1c_orbit
        index_scalti = cris1c_scalti
        index_surfht = cris1c_surfht(1,1)
        index_landfrac = cris1c_landfrac(1,1)
        index_landsea = cris1c_landsea(1,1)
        index_cloudcover = cris1c_cloudcover(1,1)
        index_cloudtopht = cris1c_cloudtopht(1,1)
        index_radianceflgs = cris1c_radianceflgs(1,1)
        index_scanqualflgs = cris1c_scanqualflgs
        index_calqualflgs = cris1c_calqualflgs(1,1)
        index_fovqualflgs = cris1c_fovqualflgs(1,1,1)
        index_geolocqual = cris1c_geolocqual
        index_quality = cris1c_quality(1,1)
        index_scores = cris1c_scores(1,1,1)
        index_logfit = cris1c_logfit(1,1,1)
        index_radiance = cris1c_radiance(1,1,1)

!Open the l1c file and read the header

        call ioc1c("OPEN/READ", in1cunit, 0)
        call ioc1c("READ HEADER", in1cunit, 1)
        IF (cris1c_h_recwords .NE. array_size_cris1c) THEN
          CALL IOC1C("CLOSE",in1cunit,0)
          CALL IOC1C("OPEN/READ",in1cunit,0)  !Re-open with correct rec len
          CALL IOC1C("READ HEADER",in1cunit,1)
        ENDIF

      endif
        
      i = scans_per_message-1
      number_of_messages = (cris1c_h_scnlin+i)/scans_per_message
      number_of_messages = number_of_messages*messages_per_scan
      write(*,*)'Message:',imessage,' of',number_of_messages
      
      if (messages_per_scan .gt. 1) then
        nsubsets = (Cnfovs*scans_per_message)/messages_per_scan
      else
        nsubsets = 0    !actual value is computed below
      endif

!If there are >1 messages per scan, jsub indicates where we are in the scan

      jsub = mod(imessage-1,messages_per_scan)*nsubsets

      do jscan=1,scans_per_message
        if (iscan .lt. cris1c_h_scnlin .and. jsub .eq. 0) then
          iscan = iscan + 1
          irec = iscan + cris1c_h_hdrcnt
          call ioc1c("READ DATA", in1cunit, irec)

          !Convert year, day_of_year, milliseconds to YYMMDD HHMMSS

          do ispot=1,Cnfovs
            i = (ispot-1)/cris_1cnfov + 1
            scnlindy = cris1c_scnlindy
            scnlinyr = cris1c_scnlinyr
            FovTime = cris1c_scnlintime(i)
            CALL DAY_TO_DATE(scnlinyr,scnlindy,FovTime,
     &           Month,Day,Hour,Min,Sec)            
            years(ispot,jscan) = scnlinyr
            months(ispot,jscan) = Month
            days(ispot,jscan) = Day
            hours(ispot,jscan) = Hour
            minutes(ispot,jscan) = Min
            seconds(ispot,jscan) = Sec
          enddo

          if (messages_per_scan .eq. 1) nsubsets = nsubsets + Cnfovs
          do i=1,array_size_cris1c
            cris1c_dt_store(i,jscan) = cris1c_dt(i)
          enddo
          irec = irec + 1
        endif
      enddo

!Section 1 information

      call codes_set(ibufr,'dataCategory',21)               !21 = radiances
      call codes_set(ibufr,'internationalDataSubCategory',5)!5 = CrIS in CCT-C13
      call codes_set(ibufr,'typicalYear',years(1,1))
      call codes_set(ibufr,'typicalMonth',months(1,1))
      call codes_set(ibufr,'typicalDay',days(1,1))
      call codes_set(ibufr,'typicalHour',hours(1,1))
      call codes_set(ibufr,'typicalMinute',minutes(1,1))
      call codes_set(ibufr,'typicalSecond',seconds(1,1))
      call codes_set(ibufr,'numberOfSubsets',nsubsets)
      call codes_set(ibufr,
     & 'inputExtendedDelayedDescriptorReplicationFactor',
     & cris1c_h_nchanused)
      if (AAPP_VERSION .ne. 0) then
        allocate(ivalues(3))
        ivalues=(/ 310060, 25060, 25061 /)    !AAPP and CSPP version numbers
      else  
        allocate(ivalues(1))
        ivalues=(/ 310060 /)
      endif
      call codes_set(ibufr,'unexpandedDescriptors',ivalues)
      deallocate(ivalues)
      call codes_set(ibufr,'centre',ICENTREID)
      call codes_set(ibufr,'satelliteIdentifier',cris1c_h_wmosatid)
      call codes_set(ibufr,'satelliteInstruments',620)    !CrIS
      call codes_set(ibufr,'satelliteClassification',3)

      allocate(ivalues(nsubsets))
      allocate(rvalues(nsubsets))

      do isub = 1,nsubsets
        scans(isub) = (isub+jsub+Cnfovs-1)/Cnfovs
        spots(isub) = MOD(isub+jsub-1,Cnfovs)+1     ! 1 to 270
        fors(isub) = (spots(isub)-1)/9 + 1     ! 1 to 30
        fovs(isub) = MOD(isub-1,9) + 1         ! 1 to 9
      enddo

!Encode the data variables

  !year
      do isub = 1,nsubsets
        ivalues(isub) = years(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'year',ivalues)
        
  !month
      do isub = 1,nsubsets
        ivalues(isub) = months(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'month',ivalues)
        
  !day
      do isub = 1,nsubsets
        ivalues(isub) = days(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'day',ivalues)
        
  !hour
      do isub = 1,nsubsets
        ivalues(isub) = hours(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'hour',ivalues)
        
  !minute
      do isub = 1,nsubsets
        ivalues(isub) = minutes(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'minute',ivalues)
        
  !second
      do isub = 1,nsubsets
        rvalues(isub) = seconds(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'second',rvalues)

  !satxyz
      do isub = 1,nsubsets
        rvalues(isub) = cris1c_dt_store(index_satxyz,scans(isub))
      enddo
      call codes_set(ibufr,
     &  'DistanceFromEarthCentreInDirectionOf0DegreesLongitude',rvalues)

      do isub = 1,nsubsets
        rvalues(isub) = cris1c_dt_store(index_satxyz+1,scans(isub))
      enddo
      call codes_set(ibufr,
     &  'DistanceFromEarthCentreInDirection90DegreesEast',rvalues)

      do isub = 1,nsubsets
        rvalues(isub) = cris1c_dt_store(index_satxyz+2,scans(isub))
      enddo
      call codes_set(ibufr,
     &  'DistanceFromEarthCentreInDirectionOfNorthPole',rvalues)

  !latitude
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(cris1c_dt_store(index_latlon+2*spots(isub)-2,
     &        scans(isub)), 1.0D-4)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'latitude',rvalues)
        
  !longitude
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(cris1c_dt_store(index_latlon+2*spots(isub)-1,
     &        scans(isub)), 1.0D-4)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'longitude',rvalues)
        
  !sat zenith
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(cris1c_dt_store(index_angles+4*spots(isub)-4,
     &        scans(isub)), 1.0D-2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'satelliteZenithAngle',rvalues)
        
  !sat azimuth
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(cris1c_dt_store(index_angles+4*spots(isub)-3,
     &        scans(isub)), 1.0D-2)
        if (rtmp.LT.0 .and. rtmp.GT.-360) rtmp = rtmp + 360.0D0
        rvalues(isub) = rtmp 
      enddo
      call codes_set(ibufr,'bearingOrAzimuth',rvalues)
        
  !solar zenith
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(cris1c_dt_store(index_angles+4*spots(isub)-2,
     &        scans(isub)), 1.0D-2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'solarZenithAngle',rvalues)
        
  !solar azimuth
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(cris1c_dt_store(index_angles+4*spots(isub)-1,
     &        scans(isub)), 1.0D-2)
        if (rtmp.LT.0 .and. rtmp.GT.-360) rtmp = rtmp + 360.0D0
        rvalues(isub) = rtmp 
      enddo
      call codes_set(ibufr,'solarAzimuth',rvalues)

  !ascending/descending
      do isub = 1,nsubsets
        ivalues(isub) = 
     &     FNJSCALEEC(cris1c_dt_store(index_ascdesc,scans(isub)),1)
      enddo
      call codes_set(ibufr,'orbitQualifier',ivalues)

  !Scan number
      do isub = 1,nsubsets
        ivalues(isub) = cris1c_dt_store(index_scnlin,scans(isub))
      enddo
      call codes_set(ibufr,'scanLineNumber',ivalues)
        
  !FOV number
      do isub = 1,nsubsets
        ivalues(isub) = fovs(isub)
      enddo
      call codes_set(ibufr,'fieldOfViewNumber',ivalues)
        
  !FOR number
      do isub = 1,nsubsets
        ivalues(isub) = fors(isub)
      enddo
      call codes_set(ibufr,'fieldOfRegardNumber',ivalues)
        
  !Orbit
      do isub = 1,nsubsets
        rvalues(isub) = FNRSCALEEC(cris1c_dt_store(index_orbit,
     &        scans(isub)), 1.0D0)
      enddo
      call codes_set(ibufr,'orbitNumber',rvalues)

  !surfht
      do isub = 1,nsubsets
        rvalues(isub) = 
     &     FNRSCALEEC(cris1c_dt_store(index_surfht+spots(isub)-1,
     &     scans(isub)), 1.0D0)
      enddo
      call codes_set(ibufr,'heightOfLandSurface',rvalues)
        
  !scalti
      do isub = 1,nsubsets
        rvalues(isub) = FNRSCALEEC(cris1c_dt_store(index_scalti,
     &     scans(isub)), 1.0D2)
      enddo
      call codes_set(ibufr,'height',rvalues)
        
  !landfrac
      do isub = 1,nsubsets
        rvalues(isub) = 
     &     FNRSCALEEC(cris1c_dt_store(index_landfrac+spots(isub)-1,
     &     scans(isub)), 1.0D-2)
      enddo
      call codes_set(ibufr,'landFraction',rvalues)
        
  !landsea
      do isub = 1,nsubsets
        rvalues(isub) = 
     &     FNRSCALEEC(cris1c_dt_store(index_landsea+spots(isub)-1,
     &     scans(isub)), 1.0D-2)
      enddo
      call codes_set(ibufr,'landOrSeaQualifier',rvalues)
        
  !cloudcover
      do isub = 1,nsubsets
        rvalues(isub) = 
     &     FNRSCALEEC(cris1c_dt_store(index_cloudcover+spots(isub)-1,
     &     scans(isub)), 1.0D-2)
      enddo
      call codes_set(ibufr,'cloudCoverTotal',rvalues)
        
  !cloudtopht
      do isub = 1,nsubsets
        rvalues(isub) = 
     &     FNRSCALEEC(cris1c_dt_store(index_cloudtopht+spots(isub)-1,
     &     scans(isub)), 1.0D-2)
      enddo
      call codes_set(ibufr,'heightOfTopOfCloud',rvalues)
        
  !radianceflgs
      do isub = 1,nsubsets
        ivalues(isub) = 
     &     FNJSCALEEC(cris1c_dt_store(index_radianceflgs+spots(isub)-1,
     &     scans(isub)), 1)
      enddo
      call codes_set(ibufr,'radianceTypeFlags',ivalues)
        
  !scanqualflgs
      do isub = 1,nsubsets
        ivalues(isub) = 
     &     FNJSCALEEC(cris1c_dt_store(index_scanqualflgs,
     &     scans(isub)), 1)
      enddo
      call codes_set(ibufr,'scanLevelQualityFlags',ivalues)

  !Band definitions and flags, band 1
      call codes_set(ibufr,'#1#band',2)
      i = FNJSCALEEC(cris1c_h_startw(1),1)
      call codes_set(ibufr,'#1#waveNumber',i)
      i = FNJSCALEEC(cris1c_h_endw(1),1)
      call codes_set(ibufr,'#2#waveNumber',i)
      i = FNJSCALEEC(cris1c_h_startchan(1),1)
      call codes_set(ibufr,'#1#startChannel',i)
      i = FNJSCALEEC(cris1c_h_endchan(1),1)
      call codes_set(ibufr,'#1#endChannel',i)
      do isub = 1,nsubsets
        ivalues(isub) = 
     &     FNJSCALEEC(cris1c_dt_store(index_calqualflgs+3*fovs(isub)-3,
     &     scans(isub)), 1)
      enddo
      call codes_set(ibufr,'#1#calibrationQualityFlags',ivalues)
      do isub = 1,nsubsets
        ivalues(isub) = 
     &     FNJSCALEEC(cris1c_dt_store(index_fovqualflgs+3*spots(isub)-3,
     &     scans(isub)), 1)
      enddo
      call codes_set(ibufr,'#1#fieldOfViewQualityFlags',ivalues)

  !Band definitions and flags, band 2
      call codes_set(ibufr,'#2#band',3)
      i = FNJSCALEEC(cris1c_h_startw(2),1)
      call codes_set(ibufr,'#3#waveNumber',i)
      i = FNJSCALEEC(cris1c_h_endw(2),1)
      call codes_set(ibufr,'#4#waveNumber',i)
      i = FNJSCALEEC(cris1c_h_startchan(2),1)
      call codes_set(ibufr,'#2#startChannel',i)
      i = FNJSCALEEC(cris1c_h_endchan(2),1)
      call codes_set(ibufr,'#2#endChannel',i)
      do isub = 1,nsubsets
        ivalues(isub) = 
     &     FNJSCALEEC(cris1c_dt_store(index_calqualflgs+3*fovs(isub)-2,
     &     scans(isub)), 1)
      enddo
      call codes_set(ibufr,'#2#calibrationQualityFlags',ivalues)
      do isub = 1,nsubsets
        ivalues(isub) = 
     &     FNJSCALEEC(cris1c_dt_store(index_fovqualflgs+3*spots(isub)-2,
     &     scans(isub)), 1)
      enddo
      call codes_set(ibufr,'#2#fieldOfViewQualityFlags',ivalues)

  !Band definitions and flags, band 3
      call codes_set(ibufr,'#3#band',4)
      i = FNJSCALEEC(cris1c_h_startw(3),1)
      call codes_set(ibufr,'#5#waveNumber',i)
      i = FNJSCALEEC(cris1c_h_endw(3),1)
      call codes_set(ibufr,'#6#waveNumber',i)
      i = FNJSCALEEC(cris1c_h_startchan(3),1)
      call codes_set(ibufr,'#3#startChannel',i)
      i = FNJSCALEEC(cris1c_h_endchan(3),1)
      call codes_set(ibufr,'#3#endChannel',i)
      do isub = 1,nsubsets
        ivalues(isub) = 
     &     FNJSCALEEC(cris1c_dt_store(index_calqualflgs+3*fovs(isub)-1,
     &     scans(isub)), 1)
      enddo
      call codes_set(ibufr,'#3#calibrationQualityFlags',ivalues)
      do isub = 1,nsubsets
        ivalues(isub) = 
     &     FNJSCALEEC(cris1c_dt_store(index_fovqualflgs+3*spots(isub)-1,
     &     scans(isub)), 1)
      enddo
      call codes_set(ibufr,'#3#fieldOfViewQualityFlags',ivalues)

  !geolocqual
      do isub = 1,nsubsets
        ivalues(isub) = 
     &     FNJSCALEEC(cris1c_dt_store(index_geolocqual,
     &     scans(isub)), 1)
      enddo
      call codes_set(ibufr,'geolocationQuality',ivalues)

  !quality
      do isub = 1,nsubsets
        ivalues(isub) = 
     &     FNJSCALEEC(cris1c_dt_store(index_quality+spots(isub)-1,
     &     scans(isub)), 1)
      enddo
      call codes_set(ibufr,'qualityInformation',ivalues)

  !channel number
      do ichan=1,cris1c_h_nchanused
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#channelNumber'
        if (cris1c_h_channels(ichan).le.2046) then   !only 11 bits 201133,005042
          ivalues(1) = cris1c_h_channels(ichan)
        else
          ivalues(1) = cris1c_h_channels(ichan)-2047
        endif
        call codes_set(ibufr, descriptor, ivalues(1))
      enddo

  !radiance (fov,for,chan)
      do ichan=1,cris1c_h_nchanused
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#channelRadiance'
        do isub = 1,nsubsets
          rvalues(isub) = FNRSCALEEC(cris1c_dt_store(index_radiance + 
     &     spots(isub)-1 + (ichan-1)*Cnfovs, 
     &     scans(isub)), 1.0D-7)
        enddo
        call codes_set(ibufr,descriptor,rvalues)
      enddo

  !AAPP and CSPP versions
      if (AAPP_VERSION .NE. 0) then
        call codes_set(ibufr,'softwareIdentification',AAPP_VERSION)
        call codes_set(ibufr,'softwareVersionNumber',
     &     cris1c_h_software_version)
      endif

  !Deallocate arrays before exiting the subroutine
      deallocate(ivalues)
      deallocate(rvalues)

  !Have we finished?
      if (iscan .eq. cris1c_h_scnlin) then
        finished = .true.
        call ioc1c("CLOSE", in1cunit, 0)
      endif

#endif
      RETURN
      END subroutine eccodes_put_1c_cris
