      subroutine eccodes_put_1c_hiras(ibufr,in1cunit,imessage,iscan,
     &  finished,ICENTREID,AAPP_VERSION)

#ifdef HAS_LIBECCODES
      use eccodes
#endif
      implicit none

!-----------------------------------------------------------------------
!
! Name: $Id: eccodes_put_1c_hiras.F 622 2019-02-26 13:40:09Z frna $
!
! Purpose: BUFR encoder subroutine for HIRAS l1c, based on CrIS FSR
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 7 December 2016, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, DWD and MeteoFrance.
!
!    Copyright 2017, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0     12/02/2019 Initial, based on eccodes_put_1c_crisfsr.F
!
! Code Description:
!   Fortran90, but using FORTRAN77 fixed-formatting, in order to interface with
!   existing AAPP "include" files.
!
! ecCodes subroutines: codes_set
! AAPP subroutines: iohiras1c
!
! Declarations:

#include "constants_preproc.h"
#include "inparms.h"
#include "hiras1c.h"

!Functions
      REAL*8 FNRSCALEEC
      INTEGER FNJSCALEEC

!Arguments
      integer ibufr
      integer in1cunit
      integer imessage
      integer iscan
      logical finished
      INTEGER ICENTREID       !Centre code (1c in Section 4)
      INTEGER AAPP_VERSION    !e.g. 715 for v7.15
      
#ifdef HAS_LIBECCODES

  !Parameters and fixed items
      integer scans_per_message
      PARAMETER (scans_per_message = 1)  !Use several scans for longer messages
      INTEGER   Imiss
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      INTEGER   Cnfovs
      PARAMETER (Cnfovs=hiras_1cnfov*hiras_1cnfor)

  !Miscellaneous variables 
      integer, save :: messages_per_scan
      integer i, irec, isub, jsub, ispot, ichan, jchan, jscan, iclass
      integer number_of_messages
      integer nsubsets
      real*8  rtmp
      real*8  freq
      CHARACTER*3 CMESSAGES_PER_SCAN

  !Input data store
      integer hiras1c_dt_store(array_size_hiras1c,scans_per_message)
      integer scans(scans_per_message*Cnfovs)
      integer spots(scans_per_message*Cnfovs)
      integer fovs(scans_per_message*Cnfovs)
      integer fors(scans_per_message*Cnfovs)

  !Date/time arrays
      integer years(Cnfovs,scans_per_message)
      integer months(Cnfovs,scans_per_message)
      integer days(Cnfovs,scans_per_message)
      integer hours(Cnfovs,scans_per_message)
      integer minutes(Cnfovs,scans_per_message)
      real*8  seconds(Cnfovs,scans_per_message)
      integer scnlindy,scnlinyr,FovTime  !date and time
      integer Month,Day,Hour,Min
      real*8  Sec
      
  !Index variables for AAPP l1c array
      integer, save :: index_scnlin,index_scnlinyr,index_scnlindy,
     & index_scnlintime,index_latlon,index_angles,
     & index_satxyz,index_ascdesc,index_orbit,index_scalti,
     & index_surfht,index_landfrac,index_landsea,index_cloudcover,
     & index_cloudtopht,index_radianceflgs,index_scanqualflgs,
     & index_calqualflgs,index_fovqualflgs,index_geolocqual,
     & index_quality,index_scores,index_logfit,index_extraradiance,
     & index_radiance

  !Arrays that are passed to eccodes
      real*8,    dimension(:), allocatable :: rvalues
      integer*4, dimension(:), allocatable :: ivalues
      character*80 descriptor
      
  !Variables to be saved between calls
      character*10, save :: formatstr(hiras_1cnchan+12+7*16)

!--------------------------------------------------------------

      imessage = imessage + 1

      if (imessage .eq. 1) then

        messages_per_scan = 1    !default
        CALL GETENV('MESSAGES_PER_SCAN_HIRAS',CMESSAGES_PER_SCAN)
        READ(CMESSAGES_PER_SCAN,*,ERR=101,END=101) messages_per_scan
  101   CONTINUE
        if (scans_per_message .gt. 1) messages_per_scan = 1

!Format string to use with multi-channel descriptors
        do ichan=1,9
          write(formatstr(ichan),'(A)') '(A,I1,A)'
        enddo
        do ichan=10,99
          write(formatstr(ichan),'(A)') '(A,I2,A)'
        enddo
        do ichan=100,999
          write(formatstr(ichan),'(A)') '(A,I3,A)'
        enddo
        do ichan=1000,hiras_1cnchan+12+7*16
          write(formatstr(ichan),'(A)') '(A,I4,A)'
        enddo

!Find the position in the AAPP data record of each variable
        do i=1,array_size_hiras1c
          hiras1c_dt(i) = i
        enddo
        index_scnlin = hiras1c_scnlin
        index_scnlinyr = hiras1c_scnlinyr
        index_scnlindy = hiras1c_scnlindy
        index_scnlintime = hiras1c_scnlintime(1)
        index_latlon = hiras1c_latlon(1,1,1)
        index_angles = hiras1c_angles(1,1,1)
        index_satxyz = hiras1c_satxyz(1)
        index_ascdesc = hiras1c_ascdesc
        index_orbit = hiras1c_orbit
        index_scalti = hiras1c_scalti
        index_surfht = hiras1c_surfht(1,1)
        index_landfrac = hiras1c_landfrac(1,1)
        index_landsea = hiras1c_landsea(1,1)
        index_cloudcover = hiras1c_cloudcover(1,1)
        index_cloudtopht = hiras1c_cloudtopht(1,1)
        index_radianceflgs = hiras1c_radianceflgs(1,1)
        index_scanqualflgs = hiras1c_scanqualflgs
        index_calqualflgs = hiras1c_calqualflgs(1,1)
        index_fovqualflgs = hiras1c_fovqualflgs(1,1,1)
        index_geolocqual = hiras1c_geolocqual
        index_quality = hiras1c_quality(1,1)
        index_scores = hiras1c_scores(1,1,1)
        index_logfit = hiras1c_logfit(1,1,1)
        index_extraradiance = hiras1c_extraradiance(1,1,1)
        index_radiance = hiras1c_radiance(1,1,1)

!Open the l1c file and read the header

        call iohiras1c("OPEN/READ", in1cunit, 0)
        call iohiras1c("READ HEADER", in1cunit, 1)
        IF (hiras1c_h_recwords .NE. array_size_hiras1c) THEN
          CALL iohiras1c("CLOSE",in1cunit,0)
          CALL iohiras1c("OPEN/READ",in1cunit,0)  !Re-open with correct rec len
          CALL iohiras1c("READ HEADER",in1cunit,1)
        ENDIF

      endif
        
      i = scans_per_message-1
      number_of_messages = (hiras1c_h_scnlin+i)/scans_per_message
      number_of_messages = number_of_messages*messages_per_scan
      write(*,*)'Message:',imessage,' of',number_of_messages
      
      if (messages_per_scan .gt. 1) then
        nsubsets = (Cnfovs*scans_per_message)/messages_per_scan
      else
        nsubsets = 0    !actual value is computed below
      endif

!If there are >1 messages per scan, jsub indicates where we are in the scan

      jsub = mod(imessage-1,messages_per_scan)*nsubsets

      do jscan=1,scans_per_message
        if (iscan .lt. hiras1c_h_scnlin .and. jsub .eq. 0) then
          iscan = iscan + 1
          irec = iscan + hiras1c_h_hdrcnt
          call iohiras1c("READ DATA", in1cunit, irec)

          !Convert year, day_of_year, milliseconds to YYMMDD HHMMSS

          do ispot=1,Cnfovs
            i = (ispot-1)/hiras_1cnfov + 1
            scnlindy = hiras1c_scnlindy
            scnlinyr = hiras1c_scnlinyr
            FovTime = hiras1c_scnlintime(i)
            CALL DAY_TO_DATE(scnlinyr,scnlindy,FovTime,
     &           Month,Day,Hour,Min,Sec)            
            years(ispot,jscan) = scnlinyr
            months(ispot,jscan) = Month
            days(ispot,jscan) = Day
            hours(ispot,jscan) = Hour
            minutes(ispot,jscan) = Min
            seconds(ispot,jscan) = Sec
          enddo

          if (messages_per_scan .eq. 1) nsubsets = nsubsets + Cnfovs
          do i=1,array_size_hiras1c
            hiras1c_dt_store(i,jscan) = hiras1c_dt(i)
          enddo
          irec = irec + 1
        endif
      enddo

!Section 1 information

      call codes_set(ibufr,'dataCategory',3)                  !3 = vertical soundings
      call codes_set(ibufr,'internationalDataSubCategory',30) !30 = hyperspectral in CCT-C13
      call codes_set(ibufr,'typicalYear',years(1,1))
      call codes_set(ibufr,'typicalMonth',months(1,1))
      call codes_set(ibufr,'typicalDay',days(1,1))
      call codes_set(ibufr,'typicalHour',hours(1,1))
      call codes_set(ibufr,'typicalMinute',minutes(1,1))
      call codes_set(ibufr,'typicalSecond',seconds(1,1))
      call codes_set(ibufr,'numberOfSubsets',nsubsets)
      call codes_set(ibufr,
     & 'inputExtendedDelayedDescriptorReplicationFactor',
     & hiras1c_h_nchanused)
      call codes_set(ibufr,
     & 'inputDelayedDescriptorReplicationFactor',12)
      if (AAPP_VERSION .ne. 0) then
        allocate(ivalues(75))
      else  
        allocate(ivalues(73))
      endif
      ivalues(1:73) = (/
     &  001007,001033,002019,002020,301011,301012,207003,004006,207000,
     &  304030,301021,007024,005021,007025,005022,008075,201133,005041,
     &  201000,005045,005043,005040,010001,201129,007002,201000,202127,
     &  201125,021166,201000,202000,008012,020010,020014,002165,033075,
     &  107003,008076,006029,006029,025140,025141,033076,033077,008076,
     &  033078,033003,104000,031002,201134,005042,201000,014044,002141,
     &  008076,104000,031001,201134,005042,201000,014044,008076,002019,
     &  109007,005060,005061,025085,105016,005042,008023,014043,008023,
     &  014043 /)

      if (AAPP_VERSION .ne. 0) then
        ivalues(74) = 025060    !AAPP version number
        ivalues(75) = 025061    !CSPP version number
      endif
      call codes_set(ibufr,'unexpandedDescriptors',ivalues)
      deallocate(ivalues)
      call codes_set(ibufr,'centre',ICENTREID)
      call codes_set(ibufr,'satelliteIdentifier',hiras1c_h_wmosatid)
      call codes_set(ibufr,'#1#satelliteInstruments',955)    !HIRAS
      call codes_set(ibufr,'satelliteClassification',3)

      allocate(ivalues(nsubsets))
      allocate(rvalues(nsubsets))

      do isub = 1,nsubsets
        scans(isub) = (isub+jsub+Cnfovs-1)/Cnfovs
        spots(isub) = MOD(isub+jsub-1,Cnfovs)+1     ! 1 to 116
        fors(isub) = (spots(isub)-1)/4 + 1     ! 1 to 29
        fovs(isub) = MOD(isub-1,4) + 1         ! 1 to 4
      enddo

!Encode the data variables

  !year
      do isub = 1,nsubsets
        ivalues(isub) = years(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'year',ivalues)
        
  !month
      do isub = 1,nsubsets
        ivalues(isub) = months(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'month',ivalues)
        
  !day
      do isub = 1,nsubsets
        ivalues(isub) = days(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'day',ivalues)
        
  !hour
      do isub = 1,nsubsets
        ivalues(isub) = hours(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'hour',ivalues)
        
  !minute
      do isub = 1,nsubsets
        ivalues(isub) = minutes(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'minute',ivalues)
        
  !second
      do isub = 1,nsubsets
        rvalues(isub) = seconds(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'second',rvalues)

  !satxyz
      do isub = 1,nsubsets
        rvalues(isub) = hiras1c_dt_store(index_satxyz,scans(isub))
      enddo
      call codes_set(ibufr,
     &  'DistanceFromEarthCentreInDirectionOf0DegreesLongitude',rvalues)

      do isub = 1,nsubsets
        rvalues(isub) = hiras1c_dt_store(index_satxyz+1,scans(isub))
      enddo
      call codes_set(ibufr,
     &  'DistanceFromEarthCentreInDirection90DegreesEast',rvalues)

      do isub = 1,nsubsets
        rvalues(isub) = hiras1c_dt_store(index_satxyz+2,scans(isub))
      enddo
      call codes_set(ibufr,
     &  'DistanceFromEarthCentreInDirectionOfNorthPole',rvalues)

  !latitude
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(hiras1c_dt_store(index_latlon+2*spots(isub)-2,
     &        scans(isub)), 1.0D-4)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'latitude',rvalues)
        
  !longitude
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(hiras1c_dt_store(index_latlon+2*spots(isub)-1,
     &        scans(isub)), 1.0D-4)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'longitude',rvalues)
        
  !sat zenith
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(hiras1c_dt_store(index_angles+4*spots(isub)-4,
     &        scans(isub)), 1.0D-2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'satelliteZenithAngle',rvalues)
        
  !sat azimuth
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(hiras1c_dt_store(index_angles+4*spots(isub)-3,
     &        scans(isub)), 1.0D-2)
        if (rtmp.LT.0 .and. rtmp.GT.-360) rtmp = rtmp + 360.0D0
        rvalues(isub) = rtmp 
      enddo
      call codes_set(ibufr,'bearingOrAzimuth',rvalues)
        
  !solar zenith
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(hiras1c_dt_store(index_angles+4*spots(isub)-2,
     &        scans(isub)), 1.0D-2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'solarZenithAngle',rvalues)
        
  !solar azimuth
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(hiras1c_dt_store(index_angles+4*spots(isub)-1,
     &        scans(isub)), 1.0D-2)
        if (rtmp.LT.0 .and. rtmp.GT.-360) rtmp = rtmp + 360.0D0
        rvalues(isub) = rtmp 
      enddo
      call codes_set(ibufr,'solarAzimuth',rvalues)

  !ascending/descending
      do isub = 1,nsubsets
        ivalues(isub) = 
     &     FNJSCALEEC(hiras1c_dt_store(index_ascdesc,scans(isub)),1)
      enddo
      call codes_set(ibufr,'orbitQualifier',ivalues)

  !Scan number
      do isub = 1,nsubsets
        ivalues(isub) = hiras1c_dt_store(index_scnlin,scans(isub))
      enddo
      call codes_set(ibufr,'scanLineNumber',ivalues)
        
  !FOV number
      do isub = 1,nsubsets
        ivalues(isub) = fovs(isub)
      enddo
      call codes_set(ibufr,'fieldOfViewNumber',ivalues)
        
  !FOR number
      do isub = 1,nsubsets
        ivalues(isub) = fors(isub)
      enddo
      call codes_set(ibufr,'fieldOfRegardNumber',ivalues)
        
  !Orbit
      do isub = 1,nsubsets
        rvalues(isub) = FNRSCALEEC(hiras1c_dt_store(index_orbit,
     &        scans(isub)), 1.0D0)
      enddo
      call codes_set(ibufr,'orbitNumber',rvalues)

  !surfht
      do isub = 1,nsubsets
        rvalues(isub) = 
     &     FNRSCALEEC(hiras1c_dt_store(index_surfht+spots(isub)-1,
     &     scans(isub)), 1.0D0)
      enddo
      call codes_set(ibufr,'heightOfLandSurface',rvalues)
        
  !scalti
      do isub = 1,nsubsets
        rvalues(isub) = FNRSCALEEC(hiras1c_dt_store(index_scalti,
     &     scans(isub)), 1.0D2)
      enddo
      call codes_set(ibufr,'height',rvalues)
        
  !landfrac
      do isub = 1,nsubsets
        rvalues(isub) = 
     &     FNRSCALEEC(hiras1c_dt_store(index_landfrac+spots(isub)-1,
     &     scans(isub)), 1.0D-2)
        if (rvalues(isub)>1.27 .and. rvalues(isub)<2.56)
     &     rvalues(isub) = CODES_MISSING_DOUBLE
      enddo
      call codes_set(ibufr,'landFraction',rvalues)
        
  !landsea
      do isub = 1,nsubsets
        rvalues(isub) = 
     &     FNRSCALEEC(hiras1c_dt_store(index_landsea+spots(isub)-1,
     &     scans(isub)), 1.0D-2)
      enddo
      call codes_set(ibufr,'landOrSeaQualifier',rvalues)
        
  !cloudcover
      do isub = 1,nsubsets
        rvalues(isub) = 
     &     FNRSCALEEC(hiras1c_dt_store(index_cloudcover+spots(isub)-1,
     &     scans(isub)), 1.0D-2)
      enddo
      call codes_set(ibufr,'cloudCoverTotal',rvalues)
        
  !cloudtopht
      do isub = 1,nsubsets
        rvalues(isub) = 
     &     FNRSCALEEC(hiras1c_dt_store(index_cloudtopht+spots(isub)-1,
     &     scans(isub)), 1.0D-2)
      enddo
      call codes_set(ibufr,'heightOfTopOfCloud',rvalues)
        
  !radianceflgs
      do isub = 1,nsubsets
        ivalues(isub) = 
     &     FNJSCALEEC(hiras1c_dt_store(index_radianceflgs+spots(isub)-1,
     &     scans(isub)), 1)
      enddo
      call codes_set(ibufr,'radianceTypeFlags',ivalues)
        
  !scanqualflgs
      do isub = 1,nsubsets
        ivalues(isub) = 
     &     FNJSCALEEC(hiras1c_dt_store(index_scanqualflgs,
     &     scans(isub)), 1)
      enddo
      call codes_set(ibufr,'scanLevelQualityFlags',ivalues)

  !Band definitions and flags, band 1
      call codes_set(ibufr,'#1#band',2)
      i = FNJSCALEEC(hiras1c_h_startw(1),1)
      call codes_set(ibufr,'#1#waveNumber',i)
      i = FNJSCALEEC(hiras1c_h_endw(1),1)
      call codes_set(ibufr,'#2#waveNumber',i)
      i = FNJSCALEEC(hiras1c_h_startchan(1),1)
      call codes_set(ibufr,'#1#startChannel',i)
      i = FNJSCALEEC(hiras1c_h_endchan(1),1)
      call codes_set(ibufr,'#1#endChannel',i)
      do isub = 1,nsubsets
        ivalues(isub) = 
     &     FNJSCALEEC(hiras1c_dt_store(index_calqualflgs+3*fovs(isub)-3,
     &     scans(isub)), 1)
      enddo
      call codes_set(ibufr,'#1#calibrationQualityFlags',ivalues)
      do isub = 1,nsubsets
        ivalues(isub) = 
     &    FNJSCALEEC(hiras1c_dt_store(index_fovqualflgs+3*spots(isub)-3,
     &    scans(isub)), 1)
      enddo
      call codes_set(ibufr,'#1#fieldOfViewQualityFlags',ivalues)

  !Band definitions and flags, band 2
      call codes_set(ibufr,'#2#band',3)
      i = FNJSCALEEC(hiras1c_h_startw(2),1)
      call codes_set(ibufr,'#3#waveNumber',i)
      i = FNJSCALEEC(hiras1c_h_endw(2),1)
      call codes_set(ibufr,'#4#waveNumber',i)
      i = FNJSCALEEC(hiras1c_h_startchan(2),1)
      call codes_set(ibufr,'#2#startChannel',i)
      i = FNJSCALEEC(hiras1c_h_endchan(2),1)
      call codes_set(ibufr,'#2#endChannel',i)
      do isub = 1,nsubsets
        ivalues(isub) = 
     &    FNJSCALEEC(hiras1c_dt_store(index_calqualflgs+3*fovs(isub)-2,
     &    scans(isub)), 1)
      enddo
      call codes_set(ibufr,'#2#calibrationQualityFlags',ivalues)
      do isub = 1,nsubsets
        ivalues(isub) = 
     &    FNJSCALEEC(hiras1c_dt_store(index_fovqualflgs+3*spots(isub)-2,
     &    scans(isub)), 1)
      enddo
      call codes_set(ibufr,'#2#fieldOfViewQualityFlags',ivalues)

  !Band definitions and flags, band 3
      call codes_set(ibufr,'#3#band',4)
      i = FNJSCALEEC(hiras1c_h_startw(3),1)
      call codes_set(ibufr,'#5#waveNumber',i)
      i = FNJSCALEEC(hiras1c_h_endw(3),1)
      call codes_set(ibufr,'#6#waveNumber',i)
      i = FNJSCALEEC(hiras1c_h_startchan(3),1)
      call codes_set(ibufr,'#3#startChannel',i)
      i = FNJSCALEEC(hiras1c_h_endchan(3),1)
      call codes_set(ibufr,'#3#endChannel',i)
      do isub = 1,nsubsets
        ivalues(isub) = 
     &    FNJSCALEEC(hiras1c_dt_store(index_calqualflgs+3*fovs(isub)-1,
     &    scans(isub)), 1)
      enddo
      call codes_set(ibufr,'#3#calibrationQualityFlags',ivalues)
      do isub = 1,nsubsets
        ivalues(isub) = 
     &    FNJSCALEEC(hiras1c_dt_store(index_fovqualflgs+3*spots(isub)-1,
     &    scans(isub)), 1)
      enddo
      call codes_set(ibufr,'#3#fieldOfViewQualityFlags',ivalues)

  !#4#band is set to missing, to cancel previous setting

  !geolocqual
      do isub = 1,nsubsets
        ivalues(isub) = 
     &     FNJSCALEEC(hiras1c_dt_store(index_geolocqual,
     &     scans(isub)), 1)
      enddo
      call codes_set(ibufr,'geolocationQuality',ivalues)

  !quality
      do isub = 1,nsubsets
        ivalues(isub) = 
     &     FNJSCALEEC(hiras1c_dt_store(index_quality+spots(isub)-1,
     &     scans(isub)), 1)
      enddo
      call codes_set(ibufr,'qualityInformation',ivalues)

  !channel number
  !The commented out lines are for when the extra channels are inserted before 
  !the normal ones.
      do ichan=1,hiras1c_h_nchanused
        jchan = ichan
!        jchan = ichan + 12
        write(descriptor,formatstr(jchan)) '#',jchan,'#channelNumber'
        call codes_set(ibufr, descriptor, hiras1c_h_channels(ichan))
      enddo

  !radiance (fov,for,chan)
  !The commented out lines are for when the extra channels are inserted before 
  !the normal ones.
      do ichan=1,hiras1c_h_nchanused
        jchan = ichan
!        jchan = ichan + 12
        write(descriptor,formatstr(jchan)) '#',jchan,'#channelRadiance'
        do isub = 1,nsubsets
          rvalues(isub) = FNRSCALEEC(hiras1c_dt_store(index_radiance + 
     &     spots(isub)-1 + (ichan-1)*Cnfovs, 
     &     scans(isub)), 1.0D-7)
        enddo
        call codes_set(ibufr,descriptor,rvalues)
      enddo

  !New descriptors for FSR
      call codes_set(ibufr,'measurementType','FSR')
      call codes_set(ibufr,'#5#band',8)
      do ichan=1,12
        jchan = hiras1c_h_nchanused+ichan
!        jchan = ichan
        write(descriptor,formatstr(jchan)) '#',jchan,'#channelNumber'
        call codes_set(ibufr, descriptor, ichan)
      enddo

  !radiance (fov,for,chan)
      do ichan=1,12
        jchan = hiras1c_h_nchanused+ichan
!        jchan = ichan
        write(descriptor,formatstr(jchan)) '#',jchan,'#channelRadiance'
        do isub = 1,nsubsets
          rvalues(isub) = 
     &     FNRSCALEEC(hiras1c_dt_store(index_extraradiance + 
     &     spots(isub)-1 + (ichan-1)*Cnfovs, 
     &     scans(isub)), 1.0D-7)
        enddo
        call codes_set(ibufr,descriptor,rvalues)
      enddo

  !MERSI (not in CCT C-8, so use 950=VIRR)
      call codes_set(ibufr,'#2#satelliteInstruments',950)
      do iclass=1,7
        do ichan=1,16
          i = (iclass-1)*16 + ichan
          write(descriptor,formatstr(hiras1c_h_nchanused + 12 + i)) 
     &       '#',hiras1c_h_nchanused + 12 + i,'#channelNumber'
          call codes_set(ibufr, descriptor, ichan)
          write(descriptor,formatstr(2*i-1)) 
     &       '#',2*i-1,'#firstOrderStatistics'
          call codes_set(ibufr, descriptor, 4)
          write(descriptor,formatstr(2*i)) 
     &       '#',2*i,'#firstOrderStatistics'
          call codes_set(ibufr, descriptor, 10)
        enddo
      enddo

  !AAPP and CSPP versions
      if (AAPP_VERSION .NE. 0) then
        call codes_set(ibufr,'softwareIdentification',AAPP_VERSION)
        call codes_set(ibufr,'softwareVersionNumber',
     &     hiras1c_h_software_version)
      endif

  !Deallocate arrays before exiting the subroutine
      deallocate(ivalues)
      deallocate(rvalues)

  !Have we finished?
      if (iscan .eq. hiras1c_h_scnlin) then
        finished = .true.
        call iohiras1c("CLOSE", in1cunit, 0)
      endif

#endif
      RETURN
      END subroutine eccodes_put_1c_hiras
