      subroutine eccodes_put_1c_pciasi(ibufr,in1cunit,imessage,iscan,
     &  finished,ICENTREID,AAPP_VERSION)

#ifdef HAS_LIBECCODES
      use eccodes
#endif
      implicit none

!-----------------------------------------------------------------------
!
! Name: $Id: eccodes_put_1c_pciasi.F 636 2019-04-16 11:12:39Z frna $
!
! Purpose: BUFR encoder subroutine for IASI lpc
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 7 December 2016, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, DWD and MeteoFrance.
!
!    Copyright 2017, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0     24/01/2017 Initial
! 1.1     06/07/2018 Add out-of-range checking for quality flags
!
! Code Description:
!   Fortran90, but using FORTRAN77 fixed-formatting, in order to interface with
!   existing AAPP "include" files.
!
! ecCodes subroutines: codes_set
! AAPP subroutines: ioipc
!
! Declarations:

#include "ppparms.h"
#include "iasipc.h"

!Functions
      REAL*8 FNRSCALEEC
      INTEGER FNJSCALEEC

!Arguments
      integer ibufr
      integer in1cunit
      integer imessage
      integer iscan
      logical finished
      INTEGER ICENTREID       !Centre code
      INTEGER AAPP_VERSION    !e.g. 715 for v7.15
      
#ifdef HAS_LIBECCODES

  !Parameters and fixed items
      integer TimeDelay
      DATA TimeDelay/216/   !milliseconds per spot for IASI
      integer scans_per_message
      PARAMETER (scans_per_message = 1)  !Usually 1 scan for IASI
      INTEGER   Imiss
      PARAMETER (Imiss=-999999)     !AAPP missing value integer

  !Miscellaneous variables 
      integer, save :: messages_per_scan
      integer i, K, irec, isub, jsub, ispot, iband, ichan, iclass
      integer jchan, jscan, jscale, jmean
      integer Npc, Ipc
      integer number_of_messages
      integer nsubsets
      integer scalrad
      integer Qual
      real*8  rtmp, logfit
      CHARACTER*3 CMESSAGES_PER_SCAN

  !Input data store
      integer iasipc_dt_store(array_size_iasipc,scans_per_message)
      integer*2
     &   iasipc_dt_store_i2(2*array_size_iasipc,scans_per_message)
      equivalence (iasipc_dt_store(1,1),iasipc_dt_store_i2(1,1))
      integer scans(scans_per_message*Infovs)
      integer spots(scans_per_message*Infovs)
      integer dets(scans_per_message*Infovs)
      integer fovs(scans_per_message*Infovs)
      save iasipc_dt_store,scans,spots,dets,fovs

  !Date/time arrays
      integer years(Infovs,scans_per_message)
      integer months(Infovs,scans_per_message)
      integer days(Infovs,scans_per_message)
      integer hours(Infovs,scans_per_message)
      integer minutes(Infovs,scans_per_message)
      real*8  seconds(Infovs,scans_per_message)
      integer scnlindy,scnlinyr,FovTime  !date and time
      integer Month,Day,Hour,Minute
      real*8  Sec
      
  !Index variables for AAPP lpc array
      integer, save :: index_scnlin,index_granule,index_scnlinyr,
     &  index_scnlindy,index_scnlintime,index_GQisFlagQual,
     &  index_GQisQualIndex,index_GQisQualIndexLoc,
     &  index_GQisQualIndexRad,index_GQisQualIndexSpect,
     &  index_GQisSysTecSondQual,index_latlon,index_angles,
     &  index_scalti,index_scalrad,index_reconrad,index_scores,
     &  index_logfit,index_avhcombination,index_avhrrmean,
     &  index_avhmeanscalefactor,index_avhrrsdev,
     &  index_avhsdevscalefactor,index_avhrrfrac,index_avhrrCGy,
     &  index_avhrrCGz,index_GIacVarImagIIS,index_GIacAvgImagIIS,
     &  index_GEUMAvhrr1BCldFrac,index_GEUMAvhrr1BLandFrac,
     &  index_GEUMAvhrr1BQual

  !Arrays that are passed to eccodes
      real*8,    dimension(:), allocatable :: rvalues
      integer*4, dimension(:), allocatable :: ivalues
      character*80 descriptor
      
  !Variables to be saved between calls
      character*10, save :: formatstr(iasi_pcnchan+42+iasi_pcnpc)

!Band definitions for GQisFlagQual are derived from On Board Parameters version
!11, ref email from Tim Hultberg 2009/09/05. These are lower boundaries of the
!band overlap regions.
      INTEGER   GQisFlagQual_start_chan(3)
      DATA      GQisFlagQual_start_chan/1,1998,5117/
      INTEGER   GQisFlagQual_end_chan(3)
      DATA      GQisFlagQual_end_chan/1997,5116,8461/
      INTEGER   GQisFlagMask(3)
      DATA      GQisFlagMask/2,4,8/

      
!--------------------------------------------------------------

      imessage = imessage + 1

      if (imessage .eq. 1) then

        messages_per_scan = 1    !default (AAPP v7 used 6, but 1 is faster and compresses better)
        CALL GETENV('MESSAGES_PER_SCAN_PCIASI',CMESSAGES_PER_SCAN)
        READ(CMESSAGES_PER_SCAN,*,ERR=101,END=101) messages_per_scan
  101   CONTINUE
        if (scans_per_message .gt. 1) messages_per_scan = 1

!Format string to use with multi-channel descriptors
        do ichan=1,9
          write(formatstr(ichan),'(A)') '(A,I1,A)'
        enddo
        do ichan=10,min(iasi_pcnchan+42+iasi_pcnpc,99)
          write(formatstr(ichan),'(A)') '(A,I2,A)'
        enddo
        do ichan=100,min(iasi_pcnchan+42+iasi_pcnpc,999)
          write(formatstr(ichan),'(A)') '(A,I3,A)'
        enddo
        do ichan=1000,iasi_pcnchan+42+iasi_pcnpc
          if (ichan .le. iasi_pcnchan+42+iasi_pcnpc) then
            write(formatstr(ichan),'(A)') '(A,I4,A)'
          endif
        enddo

!Find the position in the AAPP data record of each variable
        do i=1,array_size_iasipc
          iasipc_dt(i) = i
        enddo
        index_scnlin = iasipc_scnlin
        index_granule = iasipc_granule
        index_scnlinyr = iasipc_scnlinyr
        index_scnlindy = iasipc_scnlindy
        index_scnlintime = iasipc_scnlintime(1)
        index_GQisFlagQual = iasipc_GQisFlagQual(1,1)
        index_GQisQualIndex = iasipc_GQisQualIndex
        index_GQisQualIndexLoc = iasipc_GQisQualIndexLoc
        index_GQisQualIndexRad = iasipc_GQisQualIndexRad
        index_GQisQualIndexSpect = iasipc_GQisQualIndexSpect
        index_GQisSysTecSondQual = iasipc_GQisSysTecSondQual
        index_latlon = iasipc_latlon(1,1,1)
        index_angles = iasipc_angles(1,1,1)
        index_scalti = iasipc_scalti
        index_scalrad = 2*iasipc_scalti + 1       !index in integer*2 words
        index_reconrad = index_scalrad +
     &        iasi_pcnchan*iasi_pcndet*iasi_pcnfov
        index_scores = iasipc_scores(1,1,1)
        index_logfit = iasipc_logfit(1,1,1)
        index_avhcombination = iasipc_avhcombination
        index_avhrrmean = iasipc_avhrrmean(1,1,1,1)
        index_avhmeanscalefactor =iasipc_avhmeanscalefactor(1,1,1,1)
        index_avhrrsdev = iasipc_avhrrsdev(1,1,1,1)
        index_avhsdevscalefactor =iasipc_avhsdevscalefactor(1,1,1,1)
        index_avhrrfrac = iasipc_avhrrfrac(1,1,1)
        index_avhrrCGy = iasipc_avhrrCGy(1,1,1)
        index_avhrrCGz = iasipc_avhrrCGz(1,1,1)
        index_GIacAvgImagIIS = iasipc_GIacAvgImagIIS(1)
        index_GIacVarImagIIS = iasipc_GIacVarImagIIS(1)
        index_GEUMAvhrr1BCldFrac = iasipc_GEUMAvhrr1BCldFrac(1,1)
        index_GEUMAvhrr1BLandFrac = iasipc_GEUMAvhrr1BLandFrac(1,1)
        index_GEUMAvhrr1BQual = iasipc_GEUMAvhrr1BQual(1,1)
    
!Open the l1c file and read the header

        call ioipc("OPEN/READ", in1cunit, 0)
        call ioipc("READ HEADER", in1cunit, 1)

      endif
        
      i = scans_per_message-1
      number_of_messages = (iasipc_h_scnlin+i)/scans_per_message
      number_of_messages = number_of_messages*messages_per_scan
      write(*,*)'Message:',imessage,' of',number_of_messages

      if (messages_per_scan .gt. 1) then
        nsubsets = (Infovs*scans_per_message)/messages_per_scan
      else
        nsubsets = 0    !actual value is computed below
      endif
      
!If there are >1 messages per scan, jsub indicates where we are in the scan

      jsub = mod(imessage-1,messages_per_scan)*nsubsets

      do jscan=1,scans_per_message
        if (iscan .lt. iasipc_h_scnlin .and. jsub .eq. 0) then
          iscan = iscan + 1
          irec = iscan + iasipc_h_hdrcnt
          call ioipc("READ DATA", in1cunit, irec)

          !Convert year, day_of_year, milliseconds to YYMMDD HHMMSS

          do ispot=1,Infovs
            scnlindy = iasipc_scnlindy
            scnlinyr = iasipc_scnlinyr
            FovTime = iasipc_scnlintime((ispot-1)/4+1)
            CALL DAY_TO_DATE(scnlinyr,scnlindy,FovTime,
     &           Month,Day,Hour,Minute,Sec)            
            years(ispot,jscan) = scnlinyr
            months(ispot,jscan) = Month
            days(ispot,jscan) = Day
            hours(ispot,jscan) = Hour
            minutes(ispot,jscan) = Minute
            seconds(ispot,jscan) = Sec
          enddo

          if (messages_per_scan .eq. 1) nsubsets = nsubsets + Infovs
          do i=1,array_size_iasipc
            iasipc_dt_store(i,jscan) = iasipc_dt(i)
          enddo
          irec = irec + 1
        endif
      enddo

!Section 1 information

    !delayed replication: set at least 1 for each of the 4 slots, for consistency with EUMETSAT's data
      allocate(ivalues(4))
      ivalues(1) = max(iasipc_h_nchanused,1)
      do iband=1,3
        ivalues(iband+1) = 
     &    max(iasipc_h_endpc(iband)-iasipc_h_startpc(iband)+1, 1)
      enddo
      call codes_set(ibufr,
     &    'inputExtendedDelayedDescriptorReplicationFactor',ivalues)
      deallocate(ivalues)

      call codes_set(ibufr,'dataCategory',3)                   !Table C-13
      call codes_set(ibufr,'internationalDataSubCategory',7)   !Table C-13
      call codes_set(ibufr,'typicalYear',years(1,1))
      call codes_set(ibufr,'typicalMonth',months(1,1))
      call codes_set(ibufr,'typicalDay',days(1,1))
      call codes_set(ibufr,'typicalHour',hours(1,1))
      call codes_set(ibufr,'typicalMinute',minutes(1,1))
      call codes_set(ibufr,'typicalSecond',seconds(1,1))
      call codes_set(ibufr,'numberOfSubsets',nsubsets)
      if (AAPP_VERSION .ne. 0) then
        allocate(ivalues(2))
        ivalues=(/ 340008, 25060 /)
      else  
        allocate(ivalues(1))
        ivalues=(/ 340008 /)
      endif
      call codes_set(ibufr,'unexpandedDescriptors',ivalues)
      deallocate(ivalues)
      call codes_set(ibufr,'centre',ICENTREID)
      call codes_set(ibufr,'satelliteIdentifier',iasipc_h_wmosatid)
      call codes_set(ibufr,'#1#satelliteInstruments',221)      !IASI (0-02-019)
      call codes_set(ibufr,'satelliteClassification',61)       !EPS  (0-02-020)
      call codes_set(ibufr,'orbitNumber',iasipc_h_startorbit)

      allocate(ivalues(nsubsets))
      allocate(rvalues(nsubsets))

      do isub = 1,nsubsets
        scans(isub) = (isub+jsub+Infovs-1)/Infovs
        spots(isub) = MOD(isub+jsub-1,Infovs)+1     ! 1 to 120
        fovs(isub) = (spots(isub)-1)/4 + 1     ! 1 to 30
        dets(isub) = MOD(isub-1,4) + 1         ! 1 to 4
      enddo

!Encode the data variables

  !year
      do isub = 1,nsubsets
        ivalues(isub) = years(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'year',ivalues)
        
  !month
      do isub = 1,nsubsets
        ivalues(isub) = months(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'month',ivalues)
        
  !day
      do isub = 1,nsubsets
        ivalues(isub) = days(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'day',ivalues)
        
  !hour
      do isub = 1,nsubsets
        ivalues(isub) = hours(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'hour',ivalues)
        
  !minute
      do isub = 1,nsubsets
        ivalues(isub) = minutes(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'minute',ivalues)
        
  !second
      do isub = 1,nsubsets
        rvalues(isub) = seconds(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'second',rvalues)
        
  !latitude
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(iasipc_dt_store(index_latlon+2*spots(isub)-2,
     &        scans(isub)), 1.0D-4)
        if (rtmp.gt.90.0.or.rtmp.lt.-90.0) rtmp=CODES_MISSING_DOUBLE
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'latitude',rvalues)
        
  !longitude
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(iasipc_dt_store(index_latlon+2*spots(isub)-1,
     &        scans(isub)), 1.0D-4)
        if (rtmp.gt.360.0.or.rtmp.lt.-180.0) rtmp=CODES_MISSING_DOUBLE
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'longitude',rvalues)
        
  !sat zenith
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(iasipc_dt_store(index_angles+4*spots(isub)-4,
     &        scans(isub)), 1.0D-2)
        if (rtmp.gt.180.0.or.rtmp.lt.-90.0) rtmp=CODES_MISSING_DOUBLE
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'satelliteZenithAngle',rvalues)
        
  !sat azimuth
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(iasipc_dt_store(index_angles+4*spots(isub)-3,
     &        scans(isub)), 1.0D-2)
        if (rtmp.LT.0 .and. rtmp.GT.-360) rtmp = rtmp + 360.0D0
        if (rtmp.gt.360.0.or.rtmp.lt.0.0) rtmp=CODES_MISSING_DOUBLE
        rvalues(isub) = rtmp 
      enddo
      call codes_set(ibufr,'bearingOrAzimuth',rvalues)
        
  !solar zenith
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(iasipc_dt_store(index_angles+4*spots(isub)-2,
     &        scans(isub)), 1.0D-2)
        if (rtmp.gt.180.0.or.rtmp.lt.-90) rtmp=CODES_MISSING_DOUBLE
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'solarZenithAngle',rvalues)
        
  !solar azimuth
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(iasipc_dt_store(index_angles+4*spots(isub)-1,
     &        scans(isub)), 1.0D-2)
        if (rtmp.LT.0 .and. rtmp.GT.-360) rtmp = rtmp + 360.0D0
        if (rtmp.gt.360.0.or.rtmp.lt.0.0) rtmp=CODES_MISSING_DOUBLE
        rvalues(isub) = rtmp 
      enddo
      call codes_set(ibufr,'solarAzimuth',rvalues)

  !FOV number
      do isub = 1,nsubsets
        ivalues(isub) = spots(isub)   !1 to 120
      enddo
      call codes_set(ibufr,'fieldOfViewNumber',ivalues)
        
  !Scan number
      do isub = 1,nsubsets
        ivalues(isub) = iasipc_dt_store(index_scnlin,scans(isub))
      enddo
      call codes_set(ibufr,'scanLineNumber',ivalues)
        
  !Granule (major frame count)
      do isub = 1,nsubsets
        ivalues(isub) = iasipc_dt_store(index_granule,scans(isub))
        IF (ivalues(isub).NE.Imiss) THEN
          ivalues(isub) = MOD(ivalues(isub),256)  !fit in 8 bits
        ENDIF
      enddo
      call codes_set(ibufr,'majorFrameCount',ivalues)
        
  !scalti
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(iasipc_dt_store(index_scalti,
     &        scans(isub)), 1.0D2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'heightOfStation',rvalues)
        
  !gqisFlagQual
      do iband=1,3
        write(descriptor,formatstr(iband)) '#',iband,'#startChannel'
          call codes_set(ibufr,descriptor,
     &        GQisFlagQual_start_chan(iband))
        write(descriptor,formatstr(iband)) '#',iband,'#endChannel'
          call codes_set(ibufr,descriptor,
     &        GQisFlagQual_end_chan(iband))

        do isub = 1,nsubsets
          i = iasipc_GQisFlagQual(dets(isub),fovs(isub))
          if (i .ne. Imiss) then
            if (i .gt. 1) then  !Enhanced flag
              ivalues(isub) = 
     &            IAND(i,GQisFlagMask(IBAND))/GQisFlagMask(IBAND)
            else 
              ivalues(isub) = IAND(i,1)
            endif
          else
            ivalues(isub) = CODES_MISSING_LONG
          endif
        enddo
        write(descriptor,formatstr(iband)) '#',iband,'#gqisFlagQual'
        call codes_set(ibufr,descriptor,ivalues)
      enddo

  !gqisQualIndex
      do isub = 1,nsubsets
        ivalues(isub) = iasipc_dt_store(index_gqisQualIndex,scans(isub))
        if (ivalues(isub) .gt. 126) ivalues(isub)=CODES_MISSING_LONG
      enddo
      call codes_set(ibufr,'gqisQualIndex',ivalues)

  !gqisQualIndexLoc
      do isub = 1,nsubsets
        ivalues(isub) = 
     &      iasipc_dt_store(index_gqisQualIndexLoc,scans(isub))
        if (ivalues(isub) .gt. 126) ivalues(isub)=CODES_MISSING_LONG
      enddo
      call codes_set(ibufr,'gqisQualIndexLoc',ivalues)

  !gqisQualIndexRad
      do isub = 1,nsubsets
        ivalues(isub) =
     &      iasipc_dt_store(index_gqisQualIndexRad,scans(isub))
        if (ivalues(isub) .gt. 126) ivalues(isub)=CODES_MISSING_LONG
      enddo
      call codes_set(ibufr,'gqisQualIndexRad',ivalues)

  !gqisQualIndexSpect
      do isub = 1,nsubsets
        ivalues(isub) =
     &      iasipc_dt_store(index_GQisQualIndexSpect,scans(isub))
        if (ivalues(isub) .gt. 126) ivalues(isub)=CODES_MISSING_LONG
      enddo
      call codes_set(ibufr,'gqisQualIndexSpect',ivalues)

  !gqisSysTecQual
      do isub = 1,nsubsets
        ivalues(isub) =
     &      iasipc_dt_store(index_GQisSysTecSondQual,scans(isub))
        if (ivalues(isub).gt.16777214) ivalues(isub)=CODES_MISSING_LONG
      enddo
      call codes_set(ibufr,'gqisSysTecQual',ivalues)

  !Detailed quality flag
      do isub = 1,nsubsets
        ivalues(isub) = 
     &      iasipc_dt_store(index_GQisFlagQual+spots(isub)-1,
     &      scans(isub))/16
      enddo
      call codes_set(ibufr,'gqisFlagQualDetailed',ivalues)

  !Scale factors (note we have already used three 'startChannel' descriptors)
      do iband=1,10
        if (iasipc_h_startchan(iband) .gt. 0) then
          write(descriptor,formatstr(iband+3))
     &        '#',iband+3,'#startChannel'
          call codes_set(ibufr,descriptor,
     &        FNJSCALEEC(iasipc_h_startchan(iband),1))
          write(descriptor,formatstr(iband+3))
     &        '#',iband+3,'#endChannel'
          call codes_set(ibufr,descriptor,
     &        FNJSCALEEC(iasipc_h_endchan(iband),1))
          write(descriptor,formatstr(iband))
     &        '#',iband,'#channelScaleFactor'
          call codes_set(ibufr,descriptor,
     &        FNJSCALEEC(iasipc_h_scalefactor(iband),1))
        endif
      enddo

  !Channel number
      do ichan=1,iasipc_h_nchanused
        write(descriptor,formatstr(ichan)) '#',ichan,'#channelNumber'
        call codes_set(ibufr, descriptor, iasipc_h_channels(ichan))
      enddo

  !Scaled radiance
      do ichan=1,iasipc_h_nchanused
        write(descriptor,formatstr(ichan))
     &      '#',ichan,'#scaledIasiRadiance'
        do isub = 1,nsubsets
          scalrad = iasipc_dt_store_i2(index_scalrad+ichan-1+
     &      (spots(isub)-1)*iasi_pcnchan,scans(isub))
          if (scalrad.LT.-5000) scalrad = scalrad + 65536
          ivalues(isub) = scalrad
        enddo
        call codes_set(ibufr,descriptor,ivalues)
      enddo

  !PC scores
      do iband=1,3
        if (iasipc_h_startpcchan(iband).GT.0) THEN
          write(descriptor,formatstr(iband+13))
     &       '#',iband+13,'#startChannel'
          call codes_set(ibufr,descriptor,iasipc_h_startpcchan(iband))

          write(descriptor,formatstr(iband+13))
     &       '#',iband+13,'#endChannel'
          call codes_set(ibufr,descriptor,iasipc_h_endpcchan(iband))

          write(descriptor,formatstr(iband))
     &       '#',iband,'#scoreQuantizationFactor'
          call codes_set(ibufr,descriptor,
     &       iasipc_h_quantization(iband)*1.0D-2)

          write(descriptor,formatstr(iband))
     &       '#',iband,'#databaseIdentification'
          call codes_set(ibufr,descriptor,iasipc_h_evfilevn(iband))

          do isub = 1,nsubsets
            logfit = iasipc_dt_store(index_logfit+iband-1+
     &        (spots(isub)-1)*3,scans(isub))
            if (logfit .ne. Imiss) then
              rvalues(isub) = 10.0D0**(logfit*1.0D-4)
            else
              rvalues(isub) = CODES_MISSING_DOUBLE
            endif
          enddo
          write(descriptor,formatstr(iband))
     &       '#',iband,'#residualRmsInBand'
          call codes_set(ibufr,descriptor,rvalues)

          Npc = iasipc_h_endpc(iband)-iasipc_h_startpc(iband)+1
          do Ipc=1,Npc
            K = iasipc_h_startpc(iband)+Ipc-1
            do isub = 1,nsubsets
              ivalues(isub) = iasipc_dt_store(index_scores+K-1+
     &          (spots(isub)-1)*iasi_pcnpc,scans(isub))
            enddo
            write(descriptor,formatstr(K))
     &         '#',K,'#nonNormalizedPrincipalComponentScore'
            call codes_set(ibufr,descriptor,ivalues)
          enddo
        endif
      enddo

  !AVHRR
      call codes_set(ibufr,'#2#satelliteInstruments',591)    !AVHRR/3 (0-02-019)

  !avhcombination
      do isub = 1,nsubsets
        ivalues(isub)=iasipc_dt_store(index_avhcombination,scans(isub))
      enddo
      call codes_set(ibufr,'avhrrChannelCombination',ivalues)

  !avhrrCGy (7,iasi_pcndet,iasi_pcnfov)
      do iclass=1,7
        write(descriptor,formatstr(iclass))
     &       '#',iclass,'#yAngularPositionFromCentreOfGravity'
        do isub = 1,nsubsets
          rvalues(isub) = FNRSCALEEC(
     &      iasipc_dt_store(index_avhrrCGy+iclass-1+(spots(isub)-1)*7,
     &      scans(isub)), 1.0D-6)
        enddo
        call codes_set(ibufr,descriptor,rvalues)
      enddo
      
  !avhrrCGz (7,iasi_pcndet,iasi_pcnfov)
      do iclass=1,7
        write(descriptor,formatstr(iclass))
     &       '#',iclass,'#zAngularPositionFromCentreOfGravity'
        do isub = 1,nsubsets
          rvalues(isub) = FNRSCALEEC(
     &      iasipc_dt_store(index_avhrrCGz+iclass-1+(spots(isub)-1)*7,
     &      scans(isub)), 1.0D-6)
        enddo
        call codes_set(ibufr,descriptor,rvalues)
      enddo
      
  !avhrrfrac (7,iasi_pcndet,iasi_pcnfov)
      do iclass=1,7
        write(descriptor,formatstr(iclass))
     &       '#',iclass,'#fractionOfClearPixelsInHirsFov'
        do isub = 1,nsubsets
          rvalues(isub) = FNRSCALEEC(
     &      iasipc_dt_store(index_avhrrfrac+iclass-1+(spots(isub)-1)*7,
     &      scans(isub)), 1.0D-4)  !Convert 10^6 x fraction to percentage
        enddo
        call codes_set(ibufr,descriptor,rvalues)
      enddo

  !AVHRR radiance information
      jchan = max(iasipc_h_nchanused,1)   !for channelNumber
      jscale = 10                         !for channelScaleFactor
      jmean = 0                           !for scaledMeanAvhrrRadiance and
                                          !scaledStandardDeviationAvhrrRadiance
      do iclass=1,7
        do ichan=1,6
          jchan = jchan + 1
          jscale = jscale + 1
          jmean = jmean + 1

    !channel number
          write(descriptor,formatstr(jchan))
     &       '#',jchan,'#channelNumber'
          call codes_set(ibufr,descriptor,ichan)

    !avhmeanscalefactor
          write(descriptor,formatstr(jscale))
     &       '#',jscale,'#channelScaleFactor'
          do isub = 1,nsubsets
            ivalues(isub) = FNJSCALEEC(
     &        iasipc_dt_store(index_avhmeanscalefactor+ichan-1
     &          +(iclass-1)*6+(spots(isub)-1)*42,scans(isub)), 1)      
          enddo
          call codes_set(ibufr,descriptor,ivalues)

    !avhrrmean
          write(descriptor,formatstr(jmean))
     &       '#',jmean,'#scaledMeanAvhrrRadiance'
          do isub = 1,nsubsets
            rvalues(isub) = FNRSCALEEC(
     &        iasipc_dt_store(index_avhrrmean+ichan-1
     &          +(iclass-1)*6+(spots(isub)-1)*42,scans(isub)), 1.0D0)      
            if (rvalues(isub).lt.0.or.rvalues(isub).gt.2147483646.0D0)
     &        rvalues(isub) = CODES_MISSING_DOUBLE     !014047 = 31 bits    
          enddo
          call codes_set(ibufr,descriptor,rvalues)

    !avhsdevscalefactor
          jscale = jscale + 1
          write(descriptor,formatstr(jscale))
     &       '#',jscale,'#channelScaleFactor'
          do isub = 1,nsubsets
            ivalues(isub) = FNJSCALEEC(
     &        iasipc_dt_store(index_avhsdevscalefactor+ichan-1
     &          +(iclass-1)*6+(spots(isub)-1)*42,scans(isub)), 1)      
          enddo
          call codes_set(ibufr,descriptor,ivalues)

    !avhrrsdev
          write(descriptor,formatstr(jmean))
     &       '#',jmean,'#scaledStandardDeviationAvhrrRadiance'
          do isub = 1,nsubsets
            rvalues(isub) = FNRSCALEEC(
     &        iasipc_dt_store(index_avhrrsdev+ichan-1
     &          +(iclass-1)*6+(spots(isub)-1)*42,scans(isub)), 1.0D0)      
            if (rvalues(isub).gt.2147483646.0D0)
     &        rvalues(isub) = CODES_MISSING_DOUBLE     !014048 = 31 bits    
          enddo
          call codes_set(ibufr,descriptor,rvalues)
        enddo   !ichan
      enddo   !iclass

  !Cloud amount
      do isub = 1,nsubsets
        rvalues(isub) = FNRSCALEEC(
     &    iasipc_dt_store(index_GEUMAvhrr1BCldFrac+spots(isub)-1,
     &    scans(isub)), 1.0D0)
        if (rvalues(isub) .gt. 100) rvalues(isub) = CODES_MISSING_DOUBLE
      enddo
      call codes_set(ibufr,'cloudAmountInSegment',rvalues)

  !Land fraction
      call codes_set(ibufr,'#1#surfaceType',3)   !3=land
      do isub = 1,nsubsets
        rvalues(isub) = FNRSCALEEC(
     &    iasipc_dt_store(index_GEUMAvhrr1BLandFrac+spots(isub)-1,
     &    scans(isub)), 1.0D0)
        if (rvalues(isub) .gt. 100) rvalues(isub) = CODES_MISSING_DOUBLE
      enddo
      call codes_set(ibufr,'amountOfSegmentCoveredByScene',rvalues)

  !Imager
      do isub = 1,nsubsets
        rvalues(isub) = FNRSCALEEC(
     &    iasipc_dt_store(index_GIacAvgImagIIS+fovs(isub)-1,
     &    scans(isub)), 1.0D-6)
      enddo
      call codes_set(ibufr,'averageOfImagerMeasurements',rvalues)
      do isub = 1,nsubsets
        rvalues(isub) = FNRSCALEEC(
     &    iasipc_dt_store(index_GIacVarImagIIS+fovs(isub)-1,
     &    scans(isub)), 1.0D-6)
      enddo
      call codes_set(ibufr,'giacvarimagiisVarianceOfImagerMeasurements',
     &  rvalues)

  !Snow/ice
      do isub = 1,nsubsets
        Qual = iasipc_dt_store(index_GEUMAvhrr1BQual+spots(isub)-1,
     &    scans(isub))
        if (Qual .ge. 0 .and. Qual .lt. 128) then
          ivalues(isub) = Qual
        else
          ivalues(isub) = CODES_MISSING_LONG
        endif
      enddo
      call codes_set(ibufr,
     &  'fractionOfWeightedAvhrrPixelInIasiFovCoveredWithSnowOrIce',
     &  ivalues)

  !Missing, bad or failed AVHRR
      do isub = 1,nsubsets
        Qual = iasipc_dt_store(index_GEUMAvhrr1BQual+spots(isub)-1,
     &    scans(isub))
        if (Qual .ge. 128) then
          ivalues(isub) = MOD(Qual,128)
        else
          ivalues(isub) = CODES_MISSING_LONG
        endif
      enddo
      call codes_set(ibufr,'numberOfMissingBadOrFailedAvhrrPixels',
     &  ivalues)

  !AAPP version
      if (AAPP_VERSION .NE. 0)
     &    call codes_set(ibufr,'softwareIdentification',AAPP_VERSION)

  !Deallocate arrays before exiting the subroutine
      deallocate(ivalues)
      deallocate(rvalues)

  !Have we finished?
      if (imessage .eq. number_of_messages) then
        finished = .true.
        call ioipc("CLOSE", in1cunit, 0)
      endif

#endif
      RETURN
      END subroutine eccodes_put_1c_pciasi
