      subroutine eccodes_put_1d_hirs(ibufr,in1dunit,imessage,iscan,
     &  finished,ICENTREID,ISUBCENTREID)

#ifdef HAS_LIBECCODES
      use eccodes
#endif
      implicit none

!-----------------------------------------------------------------------
!
! Name: $Id: eccodes_put_1d_hirs.F 635 2019-04-15 15:08:00Z frna $
!
! Purpose: BUFR encoder subroutine for HIRS l1d
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 7 December 2016, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, DWD and MeteoFrance.
!
!    Copyright 2017, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0     24/07/2017 Initial
!
! Code Description:
!   Fortran90, but using FORTRAN77 fixed-formatting, in order to interface with
!   existing AAPP "include" files.
!
! ecCodes subroutines: codes_set
! AAPP subroutines: ioh1d, BITSWAP
!
! Declarations:

#include "inparms.h"
#include "hrs1d.h"

!Functions
      REAL*8 FNRSCALEEC

!Arguments
      integer ibufr
      integer in1dunit
      integer imessage
      integer iscan
      logical finished
      INTEGER ICENTREID       !Centre code (1c in Section 4)
      INTEGER ISUBCENTREID    !Centre code (1c in Section 4)
      
#ifdef HAS_LIBECCODES

  !Parameters and fixed items
      integer TimeDelay
      DATA TimeDelay/100/   !milliseconds per spot for HIRS
      integer scans_per_message
      PARAMETER (scans_per_message = 1)  !Use several scans for longer messages
      integer array_size_h1d
      PARAMETER (array_size_h1d = hrs_1drecl/4)
      INTEGER   Imiss
      PARAMETER (Imiss=-999999)     !AAPP missing value integer

  !Miscellaneous variables 
      integer i, irec, isub, ispot, ichan, jchan, jscan
      integer number_of_messages
      integer nsubsets
      real*8  rtmp

  !Input data store
      integer hrs1d_dt_store(array_size_h1d,scans_per_message)
      integer scans(scans_per_message*Hnfovs)
      integer spots(scans_per_message*Hnfovs)

  !Date/time arrays
      integer years(Hnfovs,scans_per_message)
      integer months(Hnfovs,scans_per_message)
      integer days(Hnfovs,scans_per_message)
      integer hours(Hnfovs,scans_per_message)
      integer minutes(Hnfovs,scans_per_message)
      real*8  seconds(Hnfovs,scans_per_message)
      integer scnlindy,scnlinyr,FovTime  !date and time
      integer Month,Day,Hour,Min
      real*8  Sec
      
  !Index variables for AAPP l1c array
      integer, save :: index_scnlin, index_scnlinyr, index_scnlindy,
     & index_scnlintime, index_qualind, index_scanqual, 
     & index_instrtemp, index_instrtempa1, index_instrtempa2,
     & index_instrtempb, index_spare1, index_latlon, index_surface,
     & index_angles, index_scalti, index_spare2, index_btemps,
     & index_avhrr, index_hirsqual, index_prepro, index_filler

  !Arrays that are passed to eccodes
      real*8,    dimension(:), allocatable :: rvalues
      integer*4, dimension(:), allocatable :: ivalues
      character*80 descriptor
      
  !Variables to be saved between calls
      character*10, save :: formatstr(Hnchan+Anchan+Bnchan+6)

  !Arrays for flag table manipulation 
      INTEGER    NumBitsIn          !- size of raw data bit fields
      PARAMETER  (NumBitsIn  = 32)
      INTEGER    NumBitsOut         !- set BUFR bit field width
      PARAMETER  (NumBitsOut = 24)
      INTEGER BIT                   !For use with unbitswap
      INTEGER Mask033030(NumBitsIn) !- bit mask for 033030
      INTEGER Mask033031(NumBitsIn) !- bit mask for 033031
      INTEGER Mask002223(NumBitsIn) !- bit mask for 002223
      INTEGER Mask025214(NumBitsIn) !- bit mask for 025214
      INTEGER Mask025215(NumBitsIn) !- bit mask for 025215
      DATA Mask033030 /1,2,3,4,5,6,7,25*0/
      DATA Mask033031 /8*0,1,2,3,4,4*0,5,6,7,8,9,10,11,12,13,14,15,16,
     &                 17,3*0/
      DATA Mask002223 /26*0,11,10,9,0,2,1/
      DATA Mask025214 /1,2,3,4,5,6,7,8,9,10,18*0,11,12,13,14/
      DATA Mask025215 /0,1,8*0,2,22,21,20,19,18,17,16,15,14,13,12,11,10,
     &                 9,8,7,6,5,4,3,23/
      
!--------------------------------------------------------------

      imessage = imessage + 1

      if (imessage .eq. 1) then

!Format string to use with multi-channel descriptors
        do ichan=1,9
          write(formatstr(ichan),'(A)') '(A,I1,A)'
        enddo
        do ichan=10,Hnchan+Anchan+Bnchan+6
          write(formatstr(ichan),'(A)') '(A,I2,A)'
        enddo

!Find the position in the AAPP data record of each variable
        do i=1,array_size_h1d
          hrs1d_dt(i) = i
        enddo
        index_scnlin = hrs1d_scnlin
        index_scnlinyr = hrs1d_scnlinyr
        index_scnlindy = hrs1d_scnlindy
        index_scnlintime = hrs1d_scnlintime
        index_qualind = hrs1d_qualind
        index_scanqual = hrs1d_scanqual
        index_instrtemp = hrs1d_instrtemp
        index_instrtempa1 = hrs1d_instrtempa1
        index_instrtempa2 = hrs1d_instrtempa2
        index_instrtempb = hrs1d_instrtempb
        index_spare1 = hrs1d_spare1(1)
        index_latlon = hrs1d_latlon(1,1)
        index_surface = hrs1d_surface(1,1)
        index_angles = hrs1d_angles(1,1)
        index_scalti = hrs1d_scalti
        index_spare2 = hrs1d_spare2(1)
        index_btemps = hrs1d_btemps(1,1)
        index_avhrr = hrs1d_avhrr(1,1)
        index_hirsqual = hrs1d_hirsqual(1)
        index_prepro = hrs1d_prepro(1,1)
        index_filler = hrs1d_filler(1)
      
!Open the l1c file and read the header

        call ioh1d("OPEN/READ", in1dunit, 0)
        call ioh1d("READ HEADER", in1dunit, 1)

      endif
        
      i = scans_per_message-1
      number_of_messages = (hrs1d_h_scnlin+i)/scans_per_message
      write(*,*)'Message:',imessage,' of',number_of_messages
      
      nsubsets = 0
      do jscan=1,scans_per_message
        if (iscan .lt. hrs1d_h_scnlin) then
          iscan = iscan + 1
          irec = iscan + hrs1d_h_hdrcnt
          call ioh1d("READ DATA", in1dunit, irec)

          !Convert year, day_of_year, milliseconds to YYMMDD HHMMSS

          do ispot=1,Hnfovs
            scnlindy = hrs1d_scnlindy
            scnlinyr = hrs1d_scnlinyr
            FovTime = hrs1d_scnlintime + (ispot-1)*TimeDelay
            FovTime = ((FovTime + 500)/1000)*1000  !Round to nearest sec
            CALL DAY_TO_DATE(scnlinyr,scnlindy,FovTime,
     &           Month,Day,Hour,Min,Sec)            
            years(ispot,jscan) = scnlinyr
            months(ispot,jscan) = Month
            days(ispot,jscan) = Day
            hours(ispot,jscan) = Hour
            minutes(ispot,jscan) = Min
            seconds(ispot,jscan) = Sec
          enddo

          nsubsets = nsubsets + Hnfovs
          do i=1,array_size_h1d
            hrs1d_dt_store(i,jscan) = hrs1d_dt(i)
          enddo
          irec = irec + 1
        endif
      enddo

!Section 1 information

      call codes_set(ibufr,'dataCategory',3)                 !Table C-13
      call codes_set(ibufr,'internationalDataSubCategory',5) !Table C-13 - HIRS
      call codes_set(ibufr,'typicalYear',years(1,1))
      call codes_set(ibufr,'typicalMonth',months(1,1))
      call codes_set(ibufr,'typicalDay',days(1,1))
      call codes_set(ibufr,'typicalHour',hours(1,1))
      call codes_set(ibufr,'typicalMinute',minutes(1,1))
      call codes_set(ibufr,'typicalSecond',seconds(1,1))
      call codes_set(ibufr,'numberOfSubsets',nsubsets)
      allocate(ivalues(133))
      ivalues = (/
     &  001007,004001,004002,004003,004004,004005,004006,005001,006001,
     &  008012,008070,001031,008070,001031,005040,055023,002223,201133,
     &  005041,201000,005043,033030,033031,025214,008205,002224,025206,
     &  025207,002048,055024,055025,002048,055024,055026,002048,055024,
     &  055026,202126,007001,202000,010001,007024,005021,007025,005022,
     &  025215,002151,012064,002151,012064,002151,012064,002151,012064,
     &  109019,002150,025076,025077,025078,201132,202129,012063,202000,
     &  201000,002150,025079,025080,014045,109020,002150,025076,025077,
     &  025078,201132,202129,012063,202000,201000,025085,201131,202129,
     &  002150,008023,008072,014027,008072,014027,002150,008023,008072,
     &  014027,008072,014027,002150,008023,008072,014027,008072,014027,
     &  202000,201000,201132,202129,002150,008023,008072,012063,008072,
     &  012063,002150,008023,008072,012063,008072,012063,008023,008072,
     &  012063,008072,012063,002150,008023,008072,012063,008072,012063,
     &  202000,201000,101010,014201,025194,008194,025195 /)
      call codes_set(ibufr,'unexpandedDescriptors',ivalues)
      deallocate(ivalues)
      call codes_set(ibufr,'#1#tovsOrAtovsProductQualifier',4)  !1D (0-08-070)
      call codes_set(ibufr,'#1#centre',ICENTREID)
!The second identifier is sub-centre (as in earlier aapp_put_1d_hirs.F)
      call codes_set(ibufr,'#2#tovsOrAtovsProductQualifier',2)  !1B (0-08-070)
      call codes_set(ibufr,'#2#centre',ISUBCENTREID)
      call codes_set(ibufr,'satelliteIdentifier',hrs1d_h_wmosatid)
      call codes_set(ibufr,'atovppVersionNumber',1)
      call codes_set(ibufr,'orbitNumber',hrs1d_h_startorbit)

!instruments
      IF (hrs1d_h_instruments .NE. Imiss) THEN
        CALL BITSWAP(hrs1d_h_instruments, Mask002223, NumBitsIn, 
     &       Bit, 12)
        call codes_set(ibufr,'satelliteInstrumentUsed',Bit)                           !- 002223
      ENDIF

      allocate(ivalues(nsubsets))
      allocate(rvalues(nsubsets))

      do isub = 1,nsubsets
        scans(isub) = (isub+Hnfovs-1)/Hnfovs
        spots(isub) = MOD(isub-1,Hnfovs)+1
      enddo

!Encode the data variables

  !year
      do isub = 1,nsubsets
        ivalues(isub) = years(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'year',ivalues)
        
  !month
      do isub = 1,nsubsets
        ivalues(isub) = months(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'month',ivalues)
        
  !day
      do isub = 1,nsubsets
        ivalues(isub) = days(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'day',ivalues)
        
  !hour
      do isub = 1,nsubsets
        ivalues(isub) = hours(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'hour',ivalues)
        
  !minute
      do isub = 1,nsubsets
        ivalues(isub) = minutes(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'minute',ivalues)
        
  !second
      do isub = 1,nsubsets
        rvalues(isub) = seconds(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'second',rvalues)
        
  !latitude
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(hrs1d_dt_store(index_latlon+2*spots(isub)-2,
     &        scans(isub)), 1.0D-4)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'latitude',rvalues)
        
  !longitude
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(hrs1d_dt_store(index_latlon+2*spots(isub)-1,
     &        scans(isub)), 1.0D-4)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'longitude',rvalues)
        
  !surface type: raw 1d data  : 0 = Sea, 1 = Mixed, 2 = Land
  !            : BUFR 1d data : 0 = Land, 1 = Sea, 2 = Coast    !- 008012 
      do isub = 1,nsubsets
        i = hrs1d_dt_store(index_surface+2*spots(isub)-1,scans(isub))
        IF (i .EQ. 0) THEN
          ivalues(isub) = 1
        ELSEIF (i .EQ. 1) THEN
          ivalues(isub) = 2
        ELSEIF (i .EQ. 2) THEN
          ivalues(isub) = 0
        ELSE
          ivalues(isub) = CODES_MISSING_LONG
        ENDIF
      enddo
      call codes_set(ibufr,'landOrSeaQualifier',ivalues)

  !Scan number
      do isub = 1,nsubsets
        ivalues(isub) = hrs1d_dt_store(index_scnlin,scans(isub))
      enddo
      call codes_set(ibufr,'scanLineNumber',ivalues)
        
  !FOV number
      do isub = 1,nsubsets
        ivalues(isub) = spots(isub)
      enddo
      call codes_set(ibufr,'fieldOfViewNumber',ivalues)
        
  !qualind
      do isub = 1,nsubsets
        i = hrs1d_dt_store(index_qualind,scans(isub))
        call bitswap(i, Mask033030, NumBitsIn, Bit, NumBitsOut)
        ivalues(isub) = Bit
      enddo
      call codes_set(ibufr,'scanLineStatusFlagsForAtovs',ivalues)
        
  !scanqual
      do isub = 1,nsubsets
        i = hrs1d_dt_store(index_scanqual,scans(isub))
        call bitswap(i, Mask033031, NumBitsIn, Bit, NumBitsOut)
        ivalues(isub) = Bit
      enddo
      call codes_set(ibufr,'scanLineQualityFlagsForAtovs',ivalues)
        
  !prepro
      do isub = 1,nsubsets
        i = hrs1d_dt_store(index_prepro+7*spots(isub)-7,scans(isub))
        call bitswap(i, Mask025214, NumBitsIn, Bit, NumBitsOut)
        ivalues(isub) = Bit
      enddo
      call codes_set(ibufr,'preprocessingQualityFlagsForAtovs',ivalues)

      do isub = 1,nsubsets
        i = hrs1d_dt_store(index_prepro+7*spots(isub)-6,scans(isub))
        if (i .gt. 0) then
          ivalues(isub) = i-1
        else
          ivalues(isub) = CODES_MISSING_LONG
        endif
      enddo
      call codes_set(ibufr,'estimatedNearestAmsuSurfaceType',ivalues)

      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(hrs1d_dt_store(index_prepro+7*spots(isub)-5,
     &     scans(isub)),1.0D-2)
        if (rtmp .le. 163.82) then
          rvalues(isub) = rtmp
        else    
          rvalues(isub) = CODES_MISSING_LONG
        endif
      enddo
      call codes_set(ibufr,'surfaceCostFunction',rvalues)

      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(hrs1d_dt_store(index_prepro+7*spots(isub)-4,
     &     scans(isub)),1.0D-2)
        if (rtmp .le. 250.0 .and. rtmp .ge. -150) then
          rvalues(isub) = rtmp
        else    
          rvalues(isub) = CODES_MISSING_LONG
        endif
      enddo
      call codes_set(ibufr,'scatteringIndex',rvalues)

      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(hrs1d_dt_store(index_prepro+7*spots(isub)-3,
     &     scans(isub)),1.0D-2)
        if (rtmp .le. 1.26 .and. rtmp .ge. 0) then
          rvalues(isub) = rtmp
        else    
          rvalues(isub) = CODES_MISSING_LONG
        endif
      enddo
      call codes_set(ibufr,'logisticPrecipitationProbability',rvalues)

  !sensors and version numbers
      call codes_set(ibufr,'#1#satelliteSensorIndicator',0)  !HIRS
      call codes_set(ibufr,
     &  '#1#atovsLimbSurfaceEmissivityCorrectionVersionNumber',
     &  FNRSCALEEC(hrs1d_h_Hcorrvn,1.0D0))
      call codes_set(ibufr,
     &  'atovsCloudDetectionVersionNumber',
     &  FNRSCALEEC(hrs1d_h_Hcloudvn,1.0D0))
      call codes_set(ibufr,'#2#satelliteSensorIndicator',3)  !AMSU-A
      call codes_set(ibufr,
     &  '#2#atovsLimbSurfaceEmissivityCorrectionVersionNumber',
     &  FNRSCALEEC(hrs1d_h_Acorrvn,1.0D0))
      call codes_set(ibufr,
     &  '#1#atovsPrecipitationScatterDetectionVersionNumber',
     &  FNRSCALEEC(hrs1d_h_Apcpvn,1.0D0))
      call codes_set(ibufr,'#3#satelliteSensorIndicator',4)   !AMSU-B/MHS
      call codes_set(ibufr,
     &  '#3#atovsLimbSurfaceEmissivityCorrectionVersionNumber',
     &  FNRSCALEEC(hrs1d_h_Bcorrvn,1.0D0))
      call codes_set(ibufr,
     &  '#2#atovsPrecipitationScatterDetectionVersionNumber',
     &  FNRSCALEEC(hrs1d_h_Bpcpvn,1.0D0))

  !scalti
      do isub = 1,nsubsets
        rvalues(isub) = FNRSCALEEC(hrs1d_dt_store(index_scalti,
     &        scans(isub)), 1.0D2)
      enddo
      call codes_set(ibufr,'heightOfStation',rvalues)
        
  !surface height
      do isub = 1,nsubsets
        rvalues(isub) = FNRSCALEEC(hrs1d_dt_store(
     &    index_surface+2*spots(isub)-2,scans(isub)), 1.0D0)
      enddo
      call codes_set(ibufr,'heightOfLandSurface',rvalues)
        
  !sat zenith
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(hrs1d_dt_store(index_angles+4*spots(isub)-4,
     &        scans(isub)), 1.0D-2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'satelliteZenithAngle',rvalues)
        
  !sat azimuth
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(hrs1d_dt_store(index_angles+4*spots(isub)-3,
     &        scans(isub)), 1.0D-2)
        if (rtmp.LT.0 .and. rtmp.GT.-360) rtmp = rtmp + 360.0D0
        rvalues(isub) = rtmp 
      enddo
      call codes_set(ibufr,'bearingOrAzimuth',rvalues)
        
  !solar zenith
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(hrs1d_dt_store(index_angles+4*spots(isub)-2,
     &        scans(isub)), 1.0D-2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'solarZenithAngle',rvalues)
        
  !solar azimuth
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(hrs1d_dt_store(index_angles+4*spots(isub)-1,
     &        scans(isub)), 1.0D-2)
        if (rtmp.LT.0 .and. rtmp.GT.-360) rtmp = rtmp + 360.0D0
        rvalues(isub) = rtmp 
      enddo
      call codes_set(ibufr,'solarAzimuth',rvalues)

  !hirsqual
      do isub = 1,nsubsets
        i = hrs1d_dt_store(index_hirsqual+spots(isub)-1,scans(isub))
        call bitswap(i, Mask025215, NumBitsIn, Bit, NumBitsOut)
        ivalues(isub) = Bit
      enddo
      call codes_set(ibufr,'fieldOfViewQualityFlagsForAtovs1d',ivalues)

  !instrument temps
      call codes_set(ibufr,'#1#radiometerIdentifier',0)   !HIRS
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(hrs1d_dt_store(index_instrtemp,
     &        scans(isub)), 1.0D-2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'#1#instrumentTemperature',rvalues)
      call codes_set(ibufr,'#2#radiometerIdentifier',3)   !AMSU-A1-1
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(hrs1d_dt_store(index_instrtempa1,
     &        scans(isub)), 1.0D-2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'#2#instrumentTemperature',rvalues)
      call codes_set(ibufr,'#3#radiometerIdentifier',4)   !AMSU-A1-2
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(hrs1d_dt_store(index_instrtempa2,
     &        scans(isub)), 1.0D-2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'#3#instrumentTemperature',rvalues)
      call codes_set(ibufr,'#4#radiometerIdentifier',6)   !AMSU-B
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(hrs1d_dt_store(index_instrtempb,
     &        scans(isub)), 1.0D-2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'#4#instrumentTemperature',rvalues)

!-----HIRS

  !channel number
      do ichan=1,Hnchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#tovsOrAtovsOrAvhrrInstrumentationChannelNumber'
        call codes_set(ibufr, descriptor, ichan)
      enddo

  !wavenumber for IR
      do ichan=1,Hnchan-1
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#log10OfTemperatureRadianceCentralWaveNumberForAtovs'
        if (ichan .lt. 13) then
          call codes_set(ibufr, descriptor, 
     &       log10(1.0D-4 * hrs1d_h_hrsPlanck(1,ichan)))
        else
          call codes_set(ibufr, descriptor, 
     &       log10(1.0D-3 * hrs1d_h_hrsPlanck(1,ichan)))
        endif
      enddo

  !band corr 1
      do ichan=1,Hnchan-1
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#bandwidthCorrectionCoefficient1ForAtovs'
        call codes_set(ibufr, descriptor, 
     &       1.0D-6*hrs1d_h_hrsPlanck(2,ichan))
      enddo

  !band corr 2
      do ichan=1,Hnchan-1
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#bandwidthCorrectionCoefficient2ForAtovs'
        call codes_set(ibufr, descriptor, 
     &       1.0D-6*hrs1d_h_hrsPlanck(3,ichan))
      enddo

  !Channel 20
      rtmp = FNRSCALEEC(hrs1d_h_20solfiltirrad,1.0D-6)
      call codes_set(ibufr, 
     &  "albedoRadianceSolarFilteredIrradianceForAtovs",rtmp)
      rtmp = FNRSCALEEC(hrs1d_h_20equifiltwidth,1.0D-12)
      call codes_set(ibufr, 
     &  "albedoRadianceEquivalentFilterWidthForAtovs",rtmp)

  !btemps (chan,spot,scan)
      do ichan=1,Hnchan-1
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#brightnessTemperature'
        do isub = 1,nsubsets
          rtmp = FNRSCALEEC(hrs1d_dt_store(index_btemps + 
     &        ichan-1 + 40*(spots(isub)-1), scans(isub)), 1.0D-2)
          rvalues(isub) = rtmp
        enddo
        call codes_set(ibufr,descriptor,rvalues)
      enddo

  !Channel 20 radiance (chan,spot,scan)
      do ichan=20,20
        do isub = 1,nsubsets
          rtmp = FNRSCALEEC(hrs1d_dt_store(index_btemps +
     &        ichan-1+40*(spots(isub)-1), scans(isub)), 1.0D-2) 
          rvalues(isub) = rtmp
        enddo
        call codes_set(ibufr, 'channelRadiance', rvalues)
      enddo

!-----AMSU-A + MHS

  !channel number
      do ichan=1,Anchan+Bnchan
        write(descriptor,formatstr(ichan+20)) '#',ichan+20,
     &       '#tovsOrAtovsOrAvhrrInstrumentationChannelNumber'
        call codes_set(ibufr, descriptor, ichan+27)
      enddo

  !wavenumber for IR
      do ichan=1,Anchan+Bnchan
        write(descriptor,formatstr(ichan+19)) '#',ichan+19,
     &       '#log10OfTemperatureRadianceCentralWaveNumberForAtovs'
        call codes_set(ibufr, descriptor, 
     &       log10(1.0D-4 * hrs1d_h_amsPlanck(1,ichan)))
      enddo

  !band corr 1
      do ichan=1,Anchan+Bnchan
        write(descriptor,formatstr(ichan+19)) '#',ichan+19,
     &       '#bandwidthCorrectionCoefficient1ForAtovs'
        call codes_set(ibufr, descriptor, 
     &       1.0D-6*hrs1d_h_amsPlanck(2,ichan))
      enddo

  !band corr 2
      do ichan=1,Anchan+Bnchan
        write(descriptor,formatstr(ichan+19)) '#',ichan+19,
     &       '#bandwidthCorrectionCoefficient2ForAtovs'
        call codes_set(ibufr, descriptor, 
     &       1.0D-6*hrs1d_h_amsPlanck(3,ichan))
      enddo

  !btemps (chan,spot,scan)
      do ichan=1,Anchan+Bnchan
        write(descriptor,formatstr(ichan+19)) '#',ichan+19,
     &       '#brightnessTemperature'
        do isub = 1,nsubsets
          rtmp = FNRSCALEEC(hrs1d_dt_store(index_btemps + 
     &        ichan+19 + 40*(spots(isub)-1), scans(isub)), 1.0D-2)
          rvalues(isub) = rtmp
        enddo
        call codes_set(ibufr,descriptor,rvalues)
      enddo

!----- AVHRR data

  !fraction of clear pixels in HIRS FOV
  ! Note that 025085 has scale 0, so it is really a percentage
      do isub = 1,nsubsets
        i = hrs1d_dt_store(index_avhrr + 13*spots(isub)-13, scans(isub))
        IF (i .GE. 0 .and. i .le. 126) THEN
          ivalues(isub) = i
        ELSE
          ivalues(isub) = CODES_MISSING_LONG
        ENDIF
      enddo
      call codes_set(ibufr,'fractionOfClearPixelsInHirsFov',ivalues)  

  !channel 1
      call codes_set(ibufr,
     &  '#41#tovsOrAtovsOrAvhrrInstrumentationChannelNumber',48)
      call codes_set(ibufr,'#1#firstOrderStatistics',4)  !Mean
      call codes_set(ibufr,'#1#pixelType',0) !All
      rvalues(:) = CODES_MISSING_DOUBLE
      do isub = 1,nsubsets
        i = hrs1d_dt_store(index_avhrr + 13*spots(isub)-12, scans(isub))
        IF (i .ge. 0 .AND. i .lt. 1270) rvalues(isub) = i*0.1
      enddo
      call codes_set(ibufr,'#1#albedo',rvalues)

      call codes_set(ibufr,'#2#pixelType',1) !Clear
      rvalues(:) = CODES_MISSING_DOUBLE
      do isub = 1,nsubsets
        i = hrs1d_dt_store(index_avhrr + 13*spots(isub)-7, scans(isub))
        IF (i .ge. 0 .AND. i .lt. 1270) rvalues(isub) = i*0.1
      enddo
      call codes_set(ibufr,'#2#albedo',rvalues) !Clear

  !channel 2
      call codes_set(ibufr,
     &  '#42#tovsOrAtovsOrAvhrrInstrumentationChannelNumber',49)
      call codes_set(ibufr,'#2#firstOrderStatistics',4)  !Mean
      call codes_set(ibufr,'#3#pixelType',0) !All
      rvalues(:) = CODES_MISSING_DOUBLE
      do isub = 1,nsubsets
        i = hrs1d_dt_store(index_avhrr + 13*spots(isub)-11, scans(isub)) 
        IF (i .ge. 0 .AND. i .lt. 1270) rvalues(isub) = i*0.1
      enddo
      call codes_set(ibufr,'#3#albedo',rvalues)

      call codes_set(ibufr,'#4#pixelType',1) !Clear
      rvalues(:) = CODES_MISSING_DOUBLE
      do isub = 1,nsubsets
        i = hrs1d_dt_store(index_avhrr + 13*spots(isub)-6, scans(isub)) 
        IF (i .ge. 0 .AND. i .lt. 1270) rvalues(isub) = i*0.1
      enddo
      call codes_set(ibufr,'#4#albedo',rvalues)

  !channel 3A
      call codes_set(ibufr,
     &  '#43#tovsOrAtovsOrAvhrrInstrumentationChannelNumber',50)
      call codes_set(ibufr,'#3#firstOrderStatistics',4)  !Mean
      call codes_set(ibufr,'#5#pixelType',0) !All
      rvalues(:) = CODES_MISSING_DOUBLE
      do isub = 1,nsubsets
        i = hrs1d_dt_store(index_prepro + 7*spots(isub)-7, scans(isub))
        IF (IAND(i,8) .ne. 0) THEN
          i = hrs1d_dt_store(index_avhrr+13*spots(isub)-10, scans(isub))
          IF (i .ge. 0 .AND. i .lt. 1270) rvalues(isub) = i*0.1
        ENDIF
      enddo

      call codes_set(ibufr,'#5#albedo',rvalues)
      call codes_set(ibufr,'#6#pixelType',1) !Clear
      rvalues(:) = CODES_MISSING_DOUBLE
      do isub = 1,nsubsets
        i = hrs1d_dt_store(index_prepro + 7*spots(isub)-7, scans(isub))
        IF (IAND(i,8) .ne. 0) THEN
          i = hrs1d_dt_store(index_avhrr+13*spots(isub)-5, scans(isub))
          IF (i .ge. 0 .AND. i .lt. 1270) rvalues(isub) = i*0.1
        ENDIF
      enddo
      call codes_set(ibufr,'#6#albedo',rvalues)

  !channel 3B
      call codes_set(ibufr,
     &  '#44#tovsOrAtovsOrAvhrrInstrumentationChannelNumber',51)
      call codes_set(ibufr,'#4#firstOrderStatistics',4)  !Mean
      call codes_set(ibufr,'#7#pixelType',0) !All
      rvalues(:) = CODES_MISSING_DOUBLE
      do isub = 1,nsubsets
        i = hrs1d_dt_store(index_prepro + 7*spots(isub)-7, scans(isub))
        IF (IAND(i,8) .eq. 0) THEN
          i = hrs1d_dt_store(index_avhrr+13*spots(isub)-10, scans(isub))
          IF (i .ge. 0) rvalues(isub) = i*0.01
        ENDIF
      enddo
      call codes_set(ibufr,'#40#brightnessTemperature',rvalues)

      call codes_set(ibufr,'#8#pixelType',1) !Clear
      rvalues(:) = CODES_MISSING_DOUBLE
      do isub = 1,nsubsets
        i = hrs1d_dt_store(index_prepro + 7*spots(isub)-7, scans(isub))
        IF (IAND(i,8) .eq. 0) THEN
          i = hrs1d_dt_store(index_avhrr+13*spots(isub)-5, scans(isub))
          IF (i .ge. 0) rvalues(isub) = i*0.01
        ENDIF
      enddo
      call codes_set(ibufr,'#41#brightnessTemperature',rvalues)

  !channel 4
      call codes_set(ibufr,
     &  '#45#tovsOrAtovsOrAvhrrInstrumentationChannelNumber',52)
      call codes_set(ibufr,'#5#firstOrderStatistics',4)  !Mean
      call codes_set(ibufr,'#9#pixelType',0) !All
      rvalues(:) = CODES_MISSING_DOUBLE
      do isub = 1,nsubsets
        i = hrs1d_dt_store(index_avhrr + 13*spots(isub)-9, scans(isub)) 
        IF (i .ge. 0) rvalues(isub) = i*0.01
      enddo
      call codes_set(ibufr,'#42#brightnessTemperature',rvalues) !All

      call codes_set(ibufr,'#10#pixelType',1) !Clear
      rvalues(:) = CODES_MISSING_DOUBLE
      do isub = 1,nsubsets
        i = hrs1d_dt_store(index_avhrr + 13*spots(isub)-4, scans(isub)) 
        IF (i .ge. 0) rvalues(isub) = i*0.01
      enddo
      call codes_set(ibufr,'#43#brightnessTemperature',rvalues) !Clear

      call codes_set(ibufr,'#6#firstOrderStatistics',10)  !std dev
      call codes_set(ibufr,'#11#pixelType',0) !All
      rvalues(:) = CODES_MISSING_DOUBLE
      do isub = 1,nsubsets
        i = hrs1d_dt_store(index_avhrr + 13*spots(isub)-2, scans(isub)) 
        IF (i .ge. 0) rvalues(isub) = i*0.01
      enddo

      call codes_set(ibufr,'#44#brightnessTemperature',rvalues) !All, std dev
      call codes_set(ibufr,'#12#pixelType',1) !Clear
      rvalues(:) = CODES_MISSING_DOUBLE
      do isub = 1,nsubsets
        i = hrs1d_dt_store(index_avhrr + 13*spots(isub)-1, scans(isub)) 
        IF (i .ge. 0) rvalues(isub) = i*0.01
      enddo
      call codes_set(ibufr,'#45#brightnessTemperature',rvalues) !Clear, std dev

  !channel 5
      call codes_set(ibufr,
     &  '#46#tovsOrAtovsOrAvhrrInstrumentationChannelNumber',53)
      call codes_set(ibufr,'#7#firstOrderStatistics',4)  !Mean
      call codes_set(ibufr,'#13#pixelType',0) !All
      rvalues(:) = CODES_MISSING_DOUBLE
      do isub = 1,nsubsets
        i = hrs1d_dt_store(index_avhrr + 13*spots(isub)-8, scans(isub)) 
        IF (i .ge. 0) rvalues(isub) = i*0.01
      enddo

      call codes_set(ibufr,'#46#brightnessTemperature',rvalues) !All
      call codes_set(ibufr,'#14#pixelType',1) !Clear
      rvalues(:) = CODES_MISSING_DOUBLE
      do isub = 1,nsubsets
        i = hrs1d_dt_store(index_avhrr + 13*spots(isub)-3, scans(isub)) 
        IF (i .ge. 0) rvalues(isub) = i*0.01
      enddo
      call codes_set(ibufr,'#47#brightnessTemperature',rvalues) !Clear

  !Deallocate arrays before exiting the subroutine
      deallocate(ivalues)
      deallocate(rvalues)

  !Have we finished?
      if (iscan .eq. hrs1d_h_scnlin) then
        finished = .true.
        call ioh1d("CLOSE", in1dunit, 0)
      endif

#endif
      RETURN
      END subroutine eccodes_put_1d_hirs
