      subroutine eccodes_put_1d_mwhs2(ibufr,in1cunit,imessage,iscan,
     &  finished,ICENTREID,ISUBCENTREID)

#ifdef HAS_LIBECCODES
      use eccodes
#endif
      implicit none

!-----------------------------------------------------------------------
!
! Name: $Id: eccodes_put_1d_mwhs2.F 536 2017-08-09 10:40:30Z frna $
!
! Purpose: BUFR encoder subroutine for MWHS-2 l1d
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 7 December 2016, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, DWD and MeteoFrance.
!
!    Copyright 2017, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0     22/06/2017 Initial
!
! Code Description:
!   Fortran90, but using FORTRAN77 fixed-formatting, in order to interface with
!   existing AAPP "include" files.
!
! ecCodes subroutines: codes_set
! AAPP subroutines: iomwhs21c
!
! Declarations:

#include "inparms.h"
#include "mwhs21c.h"
#include "constants_preproc.h"

!Note that the mwhs21c.h format is used for both 1c and 1d

!Functions
      REAL*8 FNRSCALEEC

!Arguments
      integer ibufr
      integer in1cunit
      integer imessage
      integer iscan
      logical finished
      INTEGER ICENTREID       !Centre code (1c in Section 4)
      INTEGER ISUBCENTREID    !Centre code (1c in Section 4)
      
#ifdef HAS_LIBECCODES

  !Parameters and fixed items
      integer TimeDelay
      DATA TimeDelay/19/      !milliseconds per spot
      integer scans_per_message
      PARAMETER (scans_per_message = 5)  !Use several scans for longer messages

  !Miscellaneous variables 
      integer i, irec, isub, jsub, ispot, ichan, jchan, jscan, scans_out
      integer number_of_messages
      integer nspots_in, nsubsets
      real*8  rtmp
      INTEGER,save :: ITHIN_X           !Number of samples to increment
      INTEGER,save :: ITHIN_Y           !Number of scans to increment
      CHARACTER*3 CTHIN

  !Input data store
      integer mwhs21c_dt_store(array_size_mwhs21c,scans_per_message)
      integer scans(scans_per_message*mwhs2_nfovs)
      integer spots(scans_per_message*mwhs2_nfovs)

  !Date/time arrays
      integer years(mwhs2_nfovs,scans_per_message)
      integer months(mwhs2_nfovs,scans_per_message)
      integer days(mwhs2_nfovs,scans_per_message)
      integer hours(mwhs2_nfovs,scans_per_message)
      integer minutes(mwhs2_nfovs,scans_per_message)
      real*8  seconds(mwhs2_nfovs,scans_per_message)
      integer scnlindy,scnlinyr,FovTime  !date and time
      integer Month,Day,Hour,Min
      real*8  Sec
      
  !Index variables for AAPP l1c array
      integer, save :: index_scnlin, index_scnlinyr, index_scnlindy,
     & index_scnlintime, index_qualind, index_scanqual, 
     & index_chanqual, index_instrtemp1, index_instrtemp2,
     & index_spare1, index_latlon, index_surface, index_angles,
     & index_scalti, index_satposition, index_satvelocity, 
     & index_orbitangle, index_spare2, index_btemps, index_btmap,
     & index_dataqual, index_filler

  !Arrays that are passed to eccodes
      real*8,    dimension(:), allocatable :: rvalues
      integer*4, dimension(:), allocatable :: ivalues
      character*80 descriptor
      
  !Variables to be saved between calls
      character*10, save :: formatstr(mwhs2_nchan+mwhs2_nmapchan)

!--------------------------------------------------------------

      imessage = imessage + 1

      if (imessage .eq. 1) then

        ITHIN_X = 1
        ITHIN_Y = 1
        CALL GETENV('MWHS2THIN_X',CTHIN)
        READ(CTHIN,*,ERR=101,END=101) ITHIN_X
  101   CALL GETENV('MWHS2THIN_Y',CTHIN)
        READ(CTHIN,*,ERR=102,END=102) ITHIN_Y
  102   CONTINUE

!Format string to use with multi-channel descriptors
        do ichan=1,9
          write(formatstr(ichan),'(A)') '(A,I1,A)'
        enddo
        do ichan=10,mwhs2_nchan+mwhs2_nmapchan
          write(formatstr(ichan),'(A)') '(A,I2,A)'
        enddo

!Find the position in the AAPP data record of each variable
        do i=1,array_size_mwhs21c
          mwhs21c_dt(i) = i
        enddo
        index_scnlin = mwhs21c_scnlin
        index_scnlinyr = mwhs21c_scnlinyr
        index_scnlindy = mwhs21c_scnlindy
        index_scnlintime = mwhs21c_scnlintime
        index_qualind = mwhs21c_qualind
        index_scanqual = mwhs21c_scanqual
        index_chanqual = mwhs21c_chanqual(1)
        index_instrtemp1 = mwhs21c_instrtemp1
        index_instrtemp2 = mwhs21c_instrtemp2
        index_spare1 = mwhs21c_spare1(1)
        index_latlon = mwhs21c_latlon(1,1)
        index_surface = mwhs21c_surface(1,1)
        index_angles = mwhs21c_angles(1,1)
        index_scalti = mwhs21c_scalti
        index_satposition = mwhs21c_satposition(1)
        index_satvelocity = mwhs21c_satvelocity(1)
        index_orbitangle = mwhs21c_orbitangle
        index_spare2 = mwhs21c_spare2(1)
        index_btemps = mwhs21c_btemps(1,1)
        index_btmap = mwhs21c_btmap(1,1)
        index_dataqual = mwhs21c_dataqual(1)
        index_filler = mwhs21c_filler(1)
      
!Open the l1c file and read the header

        call iomwhs21c("OPEN/READ", in1cunit, 0)
        call iomwhs21c("READ HEADER", in1cunit, 1)

      endif
        
      scans_out = (mwhs21c_h_scnlin+ITHIN_Y-1)/ITHIN_Y
      i = scans_per_message-1
      number_of_messages = (scans_out+i)/scans_per_message
      
      nspots_in = 0
      do jscan=1,scans_per_message
        if (iscan .le. mwhs21c_h_scnlin-ITHIN_Y) then
          if (iscan .eq. 0 .and. ITHIN_Y .gt. 1) then
            iscan = 1   !first scan (reproduce behaviour of aapp_encodebufr_1c)
          else
            iscan = iscan + ITHIN_Y
          endif
          iscan = iscan + ITHIN_Y
          irec = iscan + mwhs21c_h_hdrcnt
          call iomwhs21c("READ DATA", in1cunit, irec)

          !Convert year, day_of_year, milliseconds to YYMMDD HHMMSS

          do ispot=1,mwhs2_nfovs
            scnlindy = mwhs21c_scnlindy
            scnlinyr = mwhs21c_scnlinyr
            FovTime = mwhs21c_scnlintime + (ispot-1)*TimeDelay
            FovTime = ((FovTime + 500)/1000)*1000    !Round to nearest second
            CALL DAY_TO_DATE(scnlinyr,scnlindy,FovTime,
     &           Month,Day,Hour,Min,Sec)            
            years(ispot,jscan) = scnlinyr
            months(ispot,jscan) = Month
            days(ispot,jscan) = Day
            hours(ispot,jscan) = Hour
            minutes(ispot,jscan) = Min
            seconds(ispot,jscan) = Sec
          enddo

          nspots_in = nspots_in + mwhs2_nfovs

          do i=1,array_size_mwhs21c
            mwhs21c_dt_store(i,jscan) = mwhs21c_dt(i)
          enddo
          irec = irec + 1
        endif
      enddo
      nsubsets = 0
      do isub = (ITHIN_X+1)/2,nspots_in,ITHIN_X
        nsubsets = nsubsets + 1
      enddo
      write(*,*)'Message:',imessage,' of',number_of_messages,
     & ' subsets',nsubsets

!Section 1 information

      allocate(ivalues(3))
      ivalues(1) = mwhs2_nchan
      ivalues(2) = mwhs21c_h_nmapchanused
      ivalues(3) = 0
      call codes_set(ibufr,'inputDelayedDescriptorReplicationFactor',
     &  ivalues)
      deallocate(ivalues)

      call codes_set(ibufr,'dataCategory',3)                 !Table C-13
      call codes_set(ibufr,'typicalYear',years(1,1))
      call codes_set(ibufr,'typicalMonth',months(1,1))
      call codes_set(ibufr,'typicalDay',days(1,1))
      call codes_set(ibufr,'typicalHour',hours(1,1))
      call codes_set(ibufr,'typicalMinute',minutes(1,1))
      call codes_set(ibufr,'typicalSecond',seconds(1,1))
      call codes_set(ibufr,'numberOfSubsets',nsubsets)
      allocate(ivalues(55))
      ivalues = (/
     & 001033,001034,001007,002019,005040,201136,005041,201000,005043,
     & 301011,301013,301021,007002,013040,007024,005021,007025,005022,
     & 304030,304031,025084,033082,002019,012064,033083,106000,031001,
     & 005042,201132,202129,012063,202000,201000,002019,012064,033083,
     & 106000,031001,005042,201132,202129,012063,202000,201000,002019,
     & 012064,033083,106000,031001,005042,201132,202129,012063,202000,
     & 201000 /)
      call codes_set(ibufr,'unexpandedDescriptors',ivalues)
      deallocate(ivalues)
      call codes_set(ibufr,'centre',ICENTREID)
      call codes_set(ibufr,'subCentre',ISUBCENTREID)
      call codes_set(ibufr,'satelliteIdentifier',mwhs21c_h_wmosatid)
      call codes_set(ibufr,'#1#satelliteInstruments',953)   !MWHS-2, table C-8
      if (mwhs21c_h_nmapchanused .gt. 0) then
        call codes_set(ibufr,'#2#satelliteInstruments',954) !MWTS-2, table C-8
      endif
      call codes_set(ibufr,'orbitNumber',mwhs21c_h_startorbit)

      allocate(ivalues(nsubsets))
      allocate(rvalues(nsubsets))

      do isub = (ITHIN_X+1)/2,nspots_in,ITHIN_X
        jsub = (isub+ITHIN_X-1)/ITHIN_X
        scans(jsub) = (isub+mwhs2_nfovs-1)/mwhs2_nfovs
        spots(jsub) = MOD(isub-1,mwhs2_nfovs)+1
      enddo

!Encode the data variables

  !Scan number
      do isub = 1,nsubsets
        ivalues(isub) = mwhs21c_dt_store(index_scnlin,scans(isub))
      enddo
      call codes_set(ibufr,'scanLineNumber',ivalues)
        
  !FOV number
      do isub = 1,nsubsets
        ivalues(isub) = spots(isub)
      enddo
      call codes_set(ibufr,'fieldOfViewNumber',ivalues)
        
  !year
      do isub = 1,nsubsets
        ivalues(isub) = years(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'year',ivalues)
        
  !month
      do isub = 1,nsubsets
        ivalues(isub) = months(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'month',ivalues)
        
  !day
      do isub = 1,nsubsets
        ivalues(isub) = days(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'day',ivalues)
        
  !hour
      do isub = 1,nsubsets
        ivalues(isub) = hours(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'hour',ivalues)
        
  !minute
      do isub = 1,nsubsets
        ivalues(isub) = minutes(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'minute',ivalues)
        
  !second
      do isub = 1,nsubsets
        ivalues(isub) = seconds(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'second',ivalues)

  !latitude
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(mwhs21c_dt_store(index_latlon+2*spots(isub)-2,
     &        scans(isub)), 1.0D-4)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'latitude',rvalues)

  !longitude
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(mwhs21c_dt_store(index_latlon+2*spots(isub)-1,
     &        scans(isub)), 1.0D-4)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'longitude',rvalues)
        
  !Surface height
      do isub = 1,nsubsets
        i = mwhs21c_dt_store(index_surface+2*spots(isub)-2, scans(isub))
        rvalues(isub) = FNRSCALEEC(i, 1.0D0)
      enddo
      call codes_set(ibufr,'height',rvalues)

  !Surface type: AAPP 0=sea, 1=mixed, 2=land; 013040 0=land, 5=ocean, 6=coast
  !We should really be using a coordinate class descriptor here (i.e. a
  !significance qualifier) in order for the subsequent non-coordinate
  !descriptors to inherit the attribute. But in this 1d sequence we use a
  !surface flag, 013040. 

      do isub = 1,nsubsets
        i = mwhs21c_dt_store(index_surface+2*spots(isub)-1, scans(isub))
        IF (i .EQ. 0) THEN
          ivalues(isub) = 5
        ELSE IF (i .EQ. 1) THEN
          ivalues(isub) = 6
        ELSE IF (i .EQ. 2) THEN
          ivalues(isub) = 0
        ELSE
          ivalues(isub) = CODES_MISSING_LONG
        ENDIF
      enddo
      call codes_set(ibufr,'surfaceFlag',ivalues)

  !sat zenith
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(mwhs21c_dt_store(index_angles+4*spots(isub)-4,
     &        scans(isub)), 1.0D-2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'satelliteZenithAngle',rvalues)
        
  !sat azimuth
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(mwhs21c_dt_store(index_angles+4*spots(isub)-3,
     &        scans(isub)), 1.0D-2)
        if (rtmp.LT.0 .and. rtmp.GT.-360) rtmp = rtmp + 360.0D0
        rvalues(isub) = rtmp 
      enddo
      call codes_set(ibufr,'bearingOrAzimuth',rvalues)
        
  !solar zenith
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(mwhs21c_dt_store(index_angles+4*spots(isub)-2,
     &        scans(isub)), 1.0D-2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'solarZenithAngle',rvalues)
        
  !solar azimuth
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(mwhs21c_dt_store(index_angles+4*spots(isub)-1,
     &        scans(isub)), 1.0D-2)
        if (rtmp.LT.0 .and. rtmp.GT.-360) rtmp = rtmp + 360.0D0
        rvalues(isub) = rtmp 
      enddo
      call codes_set(ibufr,'solarAzimuth',rvalues)

  !Satellite position

      do isub = 1,nsubsets
        rvalues(isub)=FNRSCALEEC(mwhs21c_dt_store(index_satposition,
     &        scans(isub)), 1.0D0)
      enddo
      call codes_set(ibufr,
     & 'DistanceFromEarthCentreInDirectionOf0DegreesLongitude',rvalues)

      do isub = 1,nsubsets
        rvalues(isub) = FNRSCALEEC(mwhs21c_dt_store(index_satposition+1,
     &        scans(isub)), 1.0D0)
      enddo
      call codes_set(ibufr,
     & 'DistanceFromEarthCentreInDirection90DegreesEast',rvalues)

      do isub = 1,nsubsets
        rvalues(isub) = FNRSCALEEC(mwhs21c_dt_store(index_satposition+2,
     &        scans(isub)), 1.0D0)
      enddo
      call codes_set(ibufr,
     & 'DistanceFromEarthCentreInDirectionOfNorthPole',rvalues)

  !Satellite velocity

      do isub = 1,nsubsets
        rvalues(isub) = FNRSCALEEC(mwhs21c_dt_store(index_satvelocity,
     &        scans(isub)), 1.0D0)
      enddo
      call codes_set(ibufr,
     & 'absolutePlatformVelocityFirstComponent',rvalues)

      do isub = 1,nsubsets
        rvalues(isub) = FNRSCALEEC(mwhs21c_dt_store(index_satvelocity+1,
     &        scans(isub)), 1.0D0)
      enddo
      call codes_set(ibufr,
     & 'absolutePlatformVelocitySecondComponent',rvalues)

      do isub = 1,nsubsets
        rvalues(isub) = FNRSCALEEC(mwhs21c_dt_store(index_satvelocity+2,
     &        scans(isub)), 1.0D0)
      enddo
      call codes_set(ibufr,
     & 'absolutePlatformVelocityThirdComponent',rvalues)

  !orbit angle
      do isub = 1,nsubsets
        rvalues(isub) = FNRSCALEEC(mwhs21c_dt_store(index_orbitangle,
     &        scans(isub)), 1.0D-2)
        if (rvalues(isub) .lt. 0) rvalues(isub) = rvalues(isub)+360.0D0
      enddo
      call codes_set(ibufr,'geometricRotationalAngle',rvalues)

  !instrument temperature
      do isub = 1,nsubsets
        rvalues(isub) = FNRSCALEEC(mwhs21c_dt_store(index_instrtemp1,
     &        scans(isub)), 1.0D-2)
      enddo
      call codes_set(ibufr,'#1#instrumentTemperature',rvalues)

  !channel number
      do ichan=1,mwhs2_nchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#channelNumber'
        call codes_set(ibufr, descriptor, ichan)
      enddo

  !mapped channel number
      do ichan=1,mwhs21c_h_nmapchanused
        jchan = ichan+mwhs2_nchan
        write(descriptor,formatstr(jchan)) '#',jchan,
     &       '#channelNumber'
        call codes_set(ibufr, descriptor, ichan)
      enddo

  !btemps (chan,spot,scan)
      do ichan=1,mwhs2_nchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#brightnessTemperature'
        do isub = 1,nsubsets
          i = mwhs21c_dt_store(index_btemps + 
     &        ichan-1+mwhs2_nchan*(spots(isub)-1),scans(isub))
          if (i .ge. 0 .and. i .lt. 65535) then
            rvalues(isub) = i*1.0D-2
          else
            rvalues(isub) = CODES_MISSING_DOUBLE
          endif
        enddo
        call codes_set(ibufr,descriptor,rvalues)
      enddo

  !mapped btemps (chan,spot,scan)
      do ichan=1,mwhs21c_h_nmapchanused
        jchan = ichan+mwhs2_nchan
        write(descriptor,formatstr(jchan)) '#',jchan,
     &       '#brightnessTemperature'
        do isub = 1,nsubsets
          i = mwhs21c_dt_store(index_btmap + 
     &       ichan-1+mwhs21c_h_nmapchanused*(spots(isub)-1),scans(isub))
          if (i .ge. 0 .and. i .lt. 65535) then
            rvalues(isub) = i*1.0D-2
          else
            rvalues(isub) = CODES_MISSING_DOUBLE
          endif
        enddo
        call codes_set(ibufr,descriptor,rvalues)
      enddo

  !Deallocate arrays before exiting the subroutine
      deallocate(ivalues)
      deallocate(rvalues)

  !Have we finished?
      if (iscan .gt. mwhs21c_h_scnlin-ITHIN_Y) then
        finished = .true.
        call iomwhs21c("CLOSE", in1cunit, 0)
      endif

#endif
      RETURN
      END subroutine eccodes_put_1d_mwhs2
