package F90;

use strict;
use FileHandle;

sub what {
  my $code = shift;

  my $what;

  ( $code->[0] =~ m/SUBROUTINE/io ) && ( $what = 'Subroutine' );
  ( $code->[0] =~ m/FUNCTION/io )   && ( $what = 'Function' );
  ( $code->[0] =~ m/MODULE/io )     && ( $what = 'Module' );
  ( $code->[0] =~ m/PROGRAM/io )    && ( $what = 'Program' );

  return $what;

}

sub code {
  my $F90 = shift;

  ( my $fhcode = 'FileHandle'->new( "<$F90" ) )
    or die( "Cannot open <$F90\n" );

 
  my @code = <$fhcode>;

  @code = grep { ! ( m/^\s*!\s*/o or m/^\s*$/o ) } @code;
  
  return \@code;
}


sub use {
  my $code = shift;
  my @code = @$code;
  return [ grep { $_ !~ m/yomlun/io } grep { m/^\s*use\s*(\w+)\b/io; $_ = lc( $1 ) } @code ];
}

sub inc {
  my $code = shift;
  my @code = @$code;
  return [ grep { m/^\s*#include\s*\"(.*\.interface)\"/io; $_ = $1 } @code ];
}

1;

package main;
use strict;
use File::Find;
use File::Basename;
use FileHandle;
use Cwd;
use File::Basename;

my $cwd = &cwd();

my ( @LIB, @BIN );

sub wanted {
  
  return unless( -d );
  
  ( my $D = $_ ) =~ s,^\./,,o;

  my @D = split( m,/,o, $D );
  my $T = join( '/', map { '..' } @D );
  
  my $d = $D[-1];

  my @F90_bin;
  my @F90_sub;
  my @F90_mod;
  my %code;  

  if( $d =~ m/^lib.+/o ) {
    push @LIB, $D;
  
    my @C = grep { !m/intf\.c$/o } <$D/*.c>;
    s,^.*/,,o for( @C );
  
    my @CF = <$D/*.c-F>;
    s,^.*/,,o for( @CF );
    
  
    my @F = <$D/*.F>;
    s,^.*/,,o for( @F );
    
    my @F90 = <$D/*.F90>;
    s,^.*/,,o for( @F90 );

    my @O = ( @F, @C, @CF );
    s/\.(?:c|F|c-F)$/\.o/o for( @O );
    
    
    my @D = ( 
      <$D/*.dat>, <$D/*.txt>, <$D/*.TXT>, <$D/*.fdf>, <$D/*.gz>, <$D/IASI_EV*[0-9,x,Z]>, 
      grep { ( -d $_ ) && ( -f "$_.loc" ) } <$D/*>
    );
    s,^.*/,,o for( @D );

    my @DT;
    for my $d ( @D ) {
      ( my $b = $d ) =~ s/\.\w+$//o;
      my $l;
      if( -f "$D/$b.loc" ) {
        $l = do { my $fl = 'FileHandle'->new( "<$D/$b.loc" ); <$fl> };
	$l =~ s/(?:^\s*|\s*$)//go;
      } else {
        my @l = grep { s/(?:^.*LOC\s*=\s*|\s*$)//go } 
          do { my $fl = 'FileHandle'->new( "<$D/$d" ); ( <$fl> ) };
	$l = $l[0];
      }
      push @DT, "\$(TOPDIR)/$l";
    }
    
    my @M = grep { m/\.\d[a-z]?$/o } <$D/*>;
    s,^.*/,,o for( @M );
    
    my @MT = map { m/\.(\d)[a-z]?$/o; "\$(TOPDIR)/man/man$1/$_" } @M;
    
  
    for my $F90 ( @F90 ) {

      my $code = &F90::code( "$D/$F90" );
      $code{$F90} = $code;
      my $what = &F90::what( $code );
  
      if( $what eq 'Program' ) {
        push @F90_bin, $F90;
      } elsif( $what eq 'Module' ) {
        push @F90_mod, $F90;
      } else {
        push @F90_sub, $F90;
      }
    }

    my @F90_bin = map { '$(BINDIR)/' . basename( $_, qw( .F90 ) ) . '$(EXT_EXE)' } @F90_bin;
    my @F90_obj_mod = map { basename( $_, qw( .F90 ) ) . '.o' } @F90_mod;
    my @F90_obj_sub = map { basename( $_, qw( .F90 ) ) . '.o' } @F90_sub;


    my $fh = 'FileHandle'->new( ">$D/Makefile" );
    
    $fh->print( << "EOF" );
TOPDIR=$T

EOF
    if (length($F90_obj_mod[0])) {
      $fh->print( << "EOF" );
MOD=@F90_obj_mod

EOF
    }
    $fh->print( << "EOF" );
LIB=$d.a

OBJ=@O @F90_obj_sub

MAN=@MT

DAT=@DT

include \$(TOPDIR)/../Makefile.inc
   
EOF

    for my $m ( @MT ) {
      ( my $mh = $m ) =~ s,^.*/,,o;
      $fh->print( "$m: $mh\n\t\\cp $mh $m\n\n" );
    }
    
    for my $d ( @D ) {
      ( my $b = $d ) =~ s/\.\w+$//o;
      my $l;
      if( -f "$D/$b.loc" ) {
        $l = do { my $fl = 'FileHandle'->new( "<$D/$b.loc" ); <$fl> };
	$l =~ s/(?:^\s*|\s*$)//go;
      } else {
        my @l = grep { s/(?:^.*LOC\s*=\s*|\s*$)//go } 
          do { my $fl = 'FileHandle'->new( "<$D/$d" ); ( <$fl> ) };
	$l = $l[0];
      }
      my $p = dirname( $l );
      $fh->print( "\$(TOPDIR)/$l: $d\n\t\\mkdir -p \$(TOPDIR)/$p ; \\rm -rf \$(TOPDIR)/$l ; \\cp -r $d \$(TOPDIR)/$l\n\n" );
    }
    
    for my $F90 ( @F90_sub ) {

      my $obj = basename( $F90, qw( .F90 ) ) . '.o';
  
      my $inc = &F90::inc( $code{$F90} );
      my $use = &F90::use( $code{$F90} );
  
      my @inc = map { "\$(INCDIR)/$_" } @$inc;
      my @obj_use = map { "$_.o" } grep { -f "$D/$_.F90" } @$use;
  
      $fh->print( "$obj: $F90 @inc @obj_use \n" );
    }

    for my $F90 ( @F90_mod ) {

      my $mod = basename( $F90, qw( .F90 ) ) . '.mod';
      my $obj = basename( $F90, qw( .F90 ) ) . '.o';
  
      my $use = &F90::use( $code{$F90} );
      my @obj_use = map { "$_.o" } grep { -f "$D/$_.F90" } @$use;

      $fh->print( "
$obj: $F90 @obj_use
\t\$(FC) \$(FFLAGS) -I\$(TOPDIR)/include \$(MODFLAG)\$(TOPDIR)/module -c -o $obj $F90
\tmv $mod \$(TOPDIR)/module/$mod
" );
    }



    
  }
  
  if( $d eq 'bin' ) {
    push @BIN, $D;
  
    my @F = <$D/*.F>;
    s,^.*/,,o for( @F );
    
    my @F90 = <$D/*.F90>;
    s,^.*/,,o for( @F90 );

    my @C = grep { !m/intf\.c$/o } <$D/*.c>;
    s,^.*/,,o for( @C );

    my @B = ( @F, @F90, @C );
    my %B;
    
    for my $x ( @B ) {
      my $y = $x;
      unless( $x =~ m/\.pl$/o ) {
        $x =~ s/\.(?:F|F90|c)$/\.exe/o;
      }
      $B{$x} = $y;
    }
      
    my @K = <$D/*.ksh>;
    s,^.*/,,o for( @K );
    s,\.ksh$,,o for( @K );

    my @S = <$D/*.sh>;
    s,^.*/,,o for( @S );
    
    my @P = <$D/*.p?>;       #.pl or .py
    s,^.*/,,o for( @P );


    my @BT = map { "\$(TOPDIR)/bin/$_" } @B;
    
    my @KT = map { "\$(TOPDIR)/\$(SCRIPTDIR)/$_" } @K;

    my @ST = map { "\$(TOPDIR)/\$(SCRIPTDIR)/$_" } @S;

    my @PT = map { "\$(TOPDIR)/\$(SCRIPTDIR)/$_" } @P;

    my @D = ( 
      <$D/*.dat>, <$D/*.txt>, <$D/*.fdf>, <$D/*.gz>, <$D/IASI_EV*[0-9,x]>, 
      grep { -d && -f "$_.loc" } <$D/*>
    );
    s,^.*/,,o for( @D );

    my @DT;
    for my $d ( @D ) {
      ( my $b = $d ) =~ s/\.\w+$//o;
      my $l;
      if( -f "$D/$b.loc" ) {
        $l = do { my $fl = 'FileHandle'->new( "<$D/$b.loc" ); <$fl> };
	$l =~ s/(?:^\s*|\s*$)//go;
      } else {
        my @l = grep { s/(?:^.*LOC\s*=\s*|\s*$)//go } do { my $fl = 'FileHandle'->new( "<$D/$d" ); ( <$fl> ) };
	$l = $l[0];
      }
      push @DT, "\$(TOPDIR)/$l";
    }
    
    my @M = grep { m/\.\d[a-z]?$/o } <$D/*>;
    s,^.*/,,o for( @M );
    
    my @MT = map { m/\.(\d)[a-z]?$/o; "\$(TOPDIR)/man/man$1/$_" } @M;

  
    my $fh = 'FileHandle'->new( ">$D/Makefile" );
    
    $fh->print( << "EOF" );
TOPDIR=$T

BIN=@BT @ST @PT

KSH=@KT 

MAN=@MT

DAT=@DT

include \$(TOPDIR)/../Makefile.inc
   
EOF
 
    
    for my $b ( @B ) {
      my $f = $B{$b};
      my @text = grep { s/^.*LIBS\s*=\s*//o } do { my $ff = 'FileHandle'->new( "<$D/$f" ); ( <$ff> ) };
      my @libs = map { split( m/\s+/o, $_ ) } @text;
      my @alibs = map { "\$(TOPDIR)/lib/lib$_.a" } @libs;
      my @llibs = map { "-l$_" } @libs;
      if( $f =~ m/\.F$/io ) {
        $fh->print( "\$(TOPDIR)/bin/$b: $f @alibs\n\t\$(FC) \$(FFLAGS) -I\$(TOPDIR)/include \$(MODFLAG)\$(TOPDIR)/module "
                  . "-o \$(TOPDIR)/bin/$b $f -L\$(TOPDIR)/lib @llibs \$(FLIBS) \$(ARCH_LIBS)\n\n" );
      }
      if( $f =~ m/\.F90$/io ) {
        $fh->print( "\$(TOPDIR)/bin/$b: $f @alibs\n\t\$(FC) \$(FFLAGS) -I\$(TOPDIR)/include \$(MODFLAG)\$(TOPDIR)/module "
                  . "-o \$(TOPDIR)/bin/$b $f -L\$(TOPDIR)/lib @llibs \$(FLIBS) \$(ARCH_LIBS)\n\n" );
      }
      if( $f =~ m/\.c$/io ) {
        ( my $o = $b ) =~ s/\.exe$/\.o/o;
        $fh->print( "\$(TOPDIR)/bin/$b: $o $f @alibs\n\t\$(FC) \$(FFLAGS)"
                  . " -o \$(TOPDIR)/bin/$b $o -L\$(TOPDIR)/lib @llibs \$(FLIBS) \$(CLIBS) \$(ARCH_LIBS)\n\n" );
      }
    }
    
    for my $k ( @K ) {
      $fh->print( "\$(TOPDIR)/\$(SCRIPTDIR)/$k: $k.ksh\n\t\\cp $k.ksh \$(TOPDIR)/\$(SCRIPTDIR)/$k\n\n" );
    }
    
    for my $s ( @S ) {
      $fh->print( "\$(TOPDIR)/\$(SCRIPTDIR)/$s: $s\n\t\\cp $s \$(TOPDIR)/\$(SCRIPTDIR)/$s\n\n" );
    }
    
    for my $p ( @P ) {
      $fh->print( "\$(TOPDIR)/\$(SCRIPTDIR)/$p: $p\n\t\\cp $p \$(TOPDIR)/\$(SCRIPTDIR)/$p\n\n" );
    }
    
    for my $m ( @MT ) {
      ( my $mh = $m ) =~ s,^.*/,,o;
      $fh->print( "$m: $mh\n\t\\cp $mh $m\n\n" );
    }
    
    
    for my $d ( @D ) {
      ( my $b = $d ) =~ s/\.\w+$//o;
      my $l;
      if( -f "$D/$b.loc" ) {
        $l = do { my $fl = 'FileHandle'->new( "<$D/$b.loc" ); <$fl> };
	$l =~ s/(?:^\s*|\s*$)//go;
      } else {
        my @l = grep { s/(?:^.*LOC\s*=\s*|\s*$)//go } do { my $fl = 'FileHandle'->new( "<$D/$d" ); ( <$fl> ) };
	$l = $l[0];
      }
      my $p = dirname( $l );
      $fh->print( "\$(TOPDIR)/$l: $d\n\t\\mkdir -p \$(TOPDIR)/$p ; \\rm -rf \$(TOPDIR)/$l ; \\cp -r $d \$(TOPDIR)/$l\n\n" );
    }
    
  }
  

}

my @p = qw( AAPP metop-tools iasi-tools );

for my $p ( @p ) {

  chdir( $p );
  
  @LIB = ();
  @BIN = ();

  find( { wanted => \&wanted, no_chdir => 1 }, 'src' );
  @LIB = sort(@LIB);


  my $fh = 'FileHandle'->new( ">Makefile" );
  $fh->print( "include ../Makefile.ARCH\n\n" );
  $fh->print( "all:\n" );
  $fh->print( "\t\mkdir -p \$(SCRIPTDIR)\n" );
  for( @LIB ) {
    $fh->print( "\tcd $_ && \$(MAKE) lib && \$(MAKE) man && \$(MAKE) dat\n" );
  }
  for( @BIN ) {
    $fh->print( "\tcd $_ && \$(MAKE) bin\n" );
  }

  $fh->print( "clean_obj:\n" );
  for( @LIB, @BIN ) {
    $fh->print( "\tcd $_ && \$(MAKE) clean\n" );
  }

  $fh->print( "clean: clean_obj\n\t\\rm -rf bin/* module/* lib/* man/*/* data\n\tif [ \$(SCRIPTDIR) ]; then \\rm -f \$(SCRIPTDIR)/* ; fi\n\n" );

  $fh->print( "install: all\n\t\\rm -rf \$(PREFIX)/$p\n\t\\mkdir -p \$(PREFIX)/$p\n\t\\cp -r lib bin include man \$(PREFIX)/$p\n\tif [ \$(SCRIPTDIR) != bin ]; then \\cp -r \$(SCRIPTDIR) \$(PREFIX)/$p ; fi\n\tif [ -d data ] ; then \\cp -r data \$(PREFIX)/$p ; fi\n\tif [ -d data_maia3 ] ; then \\cp -r data_maia3 \$(PREFIX)/$p ; fi\n\tif [ -d data_maia4 ] ; then \\cp -r data_maia4 \$(PREFIX)/$p ; fi\n\n" );

  chdir( $cwd );
}

my $fh = 'FileHandle'->new( ">Makefile" );

$fh->print( "include Makefile.ARCH\n\n" );

$fh->print( "\nall:\n" );

$fh->print( "\t./fortran_compatibility \$(FC)\n" );

for my $p ( @p ) {
  $fh->print( "\tcd $p && \$(MAKE) all\n" );
}

$fh->print( "\nclean:\n" );

for my $p ( @p ) {
  $fh->print( "\tcd $p && \$(MAKE) clean\n" );
}


$fh->print( "\ninstall:\n" );

for my $p ( @p ) {
  $fh->print( "\tcd $p && \$(MAKE) install\n" );
}
$fh->print( "\tcp ATOVS_ENV8 ATOVS_CONF AAPP_version.txt \$(PREFIX)/\n" );


$fh->close();


