/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2006/3
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * VERSION   : 1.0
 *
 *
 */
#ifndef _CNES_IASI_CTX_1_2_H
#define _CNES_IASI_CTX_1_2_H
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "eps_types.h"
#include "eps_buffer.h"
#include "eps_grh.h"


extern int cnes_iasi_ctx_1_2_check_subclass;
extern int cnes_iasi_ctx_1_2_check_subclass_version;

#define SPCT_COD 550
#define IMLI 64
#define IMCO 64
#define CCD 2
#define NB_VAL 2

typedef struct {
  eps_ccsdstime GiaxDateElt;
  eps_real8 GiaxAxeYElt;
  eps_real8 GiaxAxeZElt;
  eps_real8 GiaxAxeQualElt;
  eps_uinteger2 GiaxFlagAxeNonQual;
} t_IASI_CTX_GIaxHistAxeInterf;

int read_IASI_CTX_GIaxHistAxeInterf( t_IASI_CTX_GIaxHistAxeInterf *X, buffer* B, int n );

int write_IASI_CTX_GIaxHistAxeInterf( t_IASI_CTX_GIaxHistAxeInterf *X, buffer* B, int n );

int free_IASI_CTX_GIaxHistAxeInterf( t_IASI_CTX_GIaxHistAxeInterf *X, int n );

int init_IASI_CTX_GIaxHistAxeInterf( t_IASI_CTX_GIaxHistAxeInterf *X, int n );

int size_IASI_CTX_GIaxHistAxeInterf( t_IASI_CTX_GIaxHistAxeInterf *X );

typedef struct {
  eps_ccsdstime GiaxDateElt;
  eps_real8 BIMSBBT;
  eps_uinteger2 GBBTMissing;
} t_IASI_CTX_GBBTHistTempCN;

int read_IASI_CTX_GBBTHistTempCN( t_IASI_CTX_GBBTHistTempCN *X, buffer* B, int n );

int write_IASI_CTX_GBBTHistTempCN( t_IASI_CTX_GBBTHistTempCN *X, buffer* B, int n );

int free_IASI_CTX_GBBTHistTempCN( t_IASI_CTX_GBBTHistTempCN *X, int n );

int init_IASI_CTX_GBBTHistTempCN( t_IASI_CTX_GBBTHistTempCN *X, int n );

int size_IASI_CTX_GBBTHistTempCN( t_IASI_CTX_GBBTHistTempCN *X );

typedef struct {
  eps_uinteger2 integ; /* Integrity indicator */
  eps_uinteger2 GCTXVersOPS_major; /* OPS version major number */
  eps_uinteger2 GCTXVersOPS_minor; /* OPS version minor number */
  eps_uinteger2 GCTXVersCTX_major; /* Context version major number */
  eps_uinteger2 GCTXVersCTX_minor; /* Context version minor number */
  eps_uinteger2 GCTXNatCTX; /* Context nature */
  eps_ccsdstime GEPSDatIASI[NB_VAL];
  eps_real8 BIMSBBT[NB_VAL];
  eps_uinteger4 GCTXNumDerLig[NB_VAL];
  eps_uinteger2 PTSI[NB_VAL];
  eps_integer2 BIISEqualCountOld[NB_VAL];
  eps_integer4 GIaxNbElt; /* Interferometric axis history size */
  t_IASI_CTX_GIaxHistAxeInterf* GIaxHistAxeInterf;
  eps_real8 MExsSmin[SPCT_COD];
  eps_real8 MExsSmax[SPCT_COD];
  eps_uinteger4 GBBTNbElt;
  t_IASI_CTX_GBBTHistTempCN* GBBTHistTempCN;
  eps_real8 GIccRadCalSlopeImag[NB_VAL][IMLI][IMCO];
  eps_real8 GIccRadCalOffsetImag[NB_VAL][IMLI][IMCO];
  eps_real8 GIacOffsetIISAVHRRLin[NB_VAL];
  eps_real8 GIacOffsetIISAVHRRCol[NB_VAL];
  eps_integer4 MDptIisLnHomePix[NB_VAL][IMCO][IMLI];
  eps_integer4 MDptIisBadHealthPix[NB_VAL][IMCO][IMLI];
  eps_real8 MDptVarImag[NB_VAL][IMCO][IMLI];
  eps_ccsdstime GOPSDatIsrfemOff[NB_VAL];
  eps_ccsdstime GOPSDatBandMiss[NB_VAL];
  eps_ccsdstime GOPSDatBBTMiss[NB_VAL];
  eps_ccsdstime GOPSDatEWMiss[NB_VAL];
  eps_ccsdstime GOPSDatImgBBMiss[NB_VAL];
  eps_ccsdstime GOPSDatImgCSMiss[NB_VAL];
  eps_ccsdstime GOPSDatRadAvhrrMiss[NB_VAL];
  eps_uinteger4 GCTXNumGranule[NB_VAL];
} t_IASI_CTX;

int read_IASI_CTX( t_IASI_CTX *X, buffer* B, int n );

int write_IASI_CTX( t_IASI_CTX *X, buffer* B, int n );

int free_IASI_CTX( t_IASI_CTX *X, int n );

int init_IASI_CTX( t_IASI_CTX *X, int n );

int size_IASI_CTX( t_IASI_CTX *X );

#endif
