/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2013/9
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *             NCA - based on eps_iasil1b_6.6.h
 *
 * VERSION   : 1.0
 *
 */
#ifndef _EPS_IASIL1B_9_0_H
#define _EPS_IASIL1B_9_0_H
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "eps_types.h"
#include "eps_buffer.h"
#include "eps_grh.h"


extern int eps_iasil1b_9_0_check_subclass;
extern int eps_iasil1b_9_0_check_subclass_version;

#define N 221000
#define AMCO 100
#define AMLI 100
#define SGI 25
#define IMLI 64
#define NBK 6
#define IMCO 64
#define SS 8700
#define VP 1
#define NCL 7
#define NIM 28
#define PN 4
#define MAXBA 3600
#define SNOT_4 34
#define SNOT 30
#define CCD 2
#define SB 3

typedef struct {
  /* Product_Details */
  eps_byte PRODUCT_NAME[67]; /* Complete name of the product */
  eps_byte PARENT_PRODUCT_NAME_1[67]; /* Name of the parent product fro
                                         m which this product has been 
                                         produced. For Level 0 products
                                         , this field is filled with lo
                                         wer case x's. */
  eps_byte PARENT_PRODUCT_NAME_2[67]; /* Name of the parent product fro
                                         m which this product has been 
                                         produced. For Level 0 products 
                                         or products for which this is 
                                         not appropriate, this field is 
                                         filled with lower case x's. */
  eps_byte PARENT_PRODUCT_NAME_3[67]; /* Name of the parent product fro
                                         m which this product has been 
                                         produced. For Level 0 products 
                                         or products for which this is 
                                         not appropriate, this field is 
                                         filled with lower case x's. */
  eps_byte PARENT_PRODUCT_NAME_4[67]; /* Name of the parent product fro
                                         m which this product has been 
                                         produced. For Level 0 products 
                                         or products for which this is 
                                         not appropriate, this field is 
                                         filled with lower case x's. */
  eps_byte INSTRUMENT_ID[4]; /* Instrument identification */
  eps_byte INSTRUMENT_MODEL[3]; /* Instrument Model identification */
  eps_byte PRODUCT_TYPE[3]; /* Product Type */
  eps_byte PROCESSING_LEVEL[2]; /* Processing Level Identification */
  eps_byte SPACECRAFT_ID[3]; /* Spacecraft identification */
  eps_gentime SENSING_START; /* UTC Time of start of sensing data in th
                                is object (PDU, ROI or Full Product) */
  eps_gentime SENSING_END; /* UTC Time of end of sensing data in this o
                              bject (PDU, ROI or Full Product) */
  eps_gentime SENSING_START_THEORETICAL; /* Theoretical UTC Time of sta
                                            rt of sensing data in the d
                                            ump from which this object 
                                            is derived. This data is th
                                            e predicted start time at t
                                            he MPF level. */
  eps_gentime SENSING_END_THEORETICAL; /* Theoretical UTC Time of end o
                                          f sensing data in the dump fr
                                          om which this object is deriv
                                          ed. This data is the predicte
                                          d end time at the MPF level. 
                                          */
  eps_byte PROCESSING_CENTRE[4]; /* Processing Centre Identification */
  eps_uinteger PROCESSOR_MAJOR_VERSION; /* Processing chain major versi
                                           on number */
  eps_uinteger PROCESSOR_MINOR_VERSION; /* Processing chain minor versi
                                           on number */
  eps_uinteger FORMAT_MAJOR_VERSION; /* Dataset Format Major Version nu
                                        mber */
  eps_uinteger FORMAT_MINOR_VERSION; /* Dataset Format Minor Version nu
                                        mber */
  eps_gentime PROCESSING_TIME_START; /* UTC time of the processing at s
                                        tart of processing for the prod
                                        uct */
  eps_gentime PROCESSING_TIME_END; /* UTC time of the processing at end 
                                      of processing for the product */
  eps_byte PROCESSING_MODE[1]; /* Identification of the mode of process
                                  ing */
  eps_byte DISPOSITION_MODE[1]; /* Identification of the diposition mod
                                   e */
  eps_byte RECEIVING_GROUND_STATION[3]; /* Acquisition Station Identifi
                                           cation */
  eps_gentime RECEIVE_TIME_START; /* UTC time of the reception at CDA f
                                     or first Data Item */
  eps_gentime RECEIVE_TIME_END; /* UTC time of the reception at CDA for 
                                   last Data Item */
  eps_uinteger ORBIT_START; /* Start Orbit Number, counted incrementall
                               y since launch */
  eps_uinteger ORBIT_END; /* Stop Orbit Number */
  eps_uinteger ACTUAL_PRODUCT_SIZE; /* Size of the complete product */
  /* ASCENDING_NODE_ORBIT_PARAMETERS */
  eps_longgentime STATE_VECTOR_TIME; /* Epoch time (in UTC) of the orbi
                                        tal elements and the orbit stat
                                        e vector. this corresponds to t
                                        he time of crossing the ascendi
                                        ng node for ORBIT_START */
  eps_integer SEMI_MAJOR_AXIS; /* Semi major axis of orbit at time of t
                                  he ascending node crossing. */
  eps_integer ECCENTRICITY; /* Orbit eccentricity at time of the ascend
                               ing node crossing */
  eps_integer INCLINATION; /* Orbit inclination at time of the ascendin
                              g node crossing */
  eps_integer PERIGEE_ARGUMENT; /* Argument of perigee at time of the a
                                   scending node crossing */
  eps_integer RIGHT_ASCENSION; /* Right ascension at time of the ascend
                                  ing node crossing */
  eps_integer MEAN_ANOMALY; /* Mean anomaly at time of the ascending no
                               de crossing */
  eps_integer X_POSITION; /* X position of the orbit state vector in th
                             e orbit frame at ascending node */
  eps_integer Y_POSITION; /* Y position of the orbit state vector in th
                             e orbit frame at ascending node */
  eps_integer Z_POSITION; /* Z position of the orbit state vector in th
                             e orbit frame at ascending node */
  eps_integer X_VELOCITY; /* X velocity of the orbit state vector in th
                             e orbit frame at ascending node */
  eps_integer Y_VELOCITY; /* Y velocity of the orbit state vector in th
                             e orbit frame at ascending node */
  eps_integer Z_VELOCITY; /* Z velocity of the orbit state vector in th
                             e orbit frame at ascending node */
  eps_integer EARTH_SUN_DISTANCE_RATIO; /* Earth-Sun distance ratio - r
                                           atio of current Eart-Sun dis
                                           tance to Mean Earth-Sun dist
                                           ance */
  eps_integer LOCATION_TOLERANCE_RADIAL; /* Nadir Earth location tolera
                                            nce radial */
  eps_integer LOCATION_TOLERANCE_CROSSTRACK; /* Nadir Earth location to
                                                lerance cross-track */
  eps_integer LOCATION_TOLERANCE_ALONGTRACK; /* Nadir Earth location to
                                                lerance along-track */
  eps_integer YAW_ERROR; /* Constant Yaw attitude error */
  eps_integer ROLL_ERROR; /* Constant Roll attitude error */
  eps_integer PITCH_ERROR; /* Constant Pitch attitude error */
  /* LOCATION_SUMMARY */
  eps_integer SUBSAT_LATITUDE_START; /* Latitude of sub-satellite point 
                                        at start of the data set */
  eps_integer SUBSAT_LONGITUDE_START; /* Longitude of sub-satellite poi
                                         nt at start of the data set */
  eps_integer SUBSAT_LATITUDE_END; /* Latitude of sub-satellite point a
                                      t end of the data set */
  eps_integer SUBSAT_LONGITUDE_END; /* Longitude of sub-satellite point 
                                       at end of the data set */
  /* Leap_Second_Information */
  eps_integer LEAP_SECOND; /* Occurence of Leap second within the produ
                              ct. Field is set to -1, 0 or +1 dependent 
                              upon occurrence of leap second and direct
                              ion. */
  eps_gentime LEAP_SECOND_UTC; /* UTC time of occurrence of the Leap Se
                                  cond (If no leap second in the produc
                                  t, value is null) */
  /* Record_counts */
  eps_uinteger TOTAL_RECORDS; /* Total count of all records in the prod
                                 uct */
  eps_uinteger TOTAL_MPHR; /* Total count of all MPHRs in product (shou
                              ld always be 1!) */
  eps_uinteger TOTAL_SPHR; /* Total count of all SPHRs in product (shou
                              ld be 0 or 1 only) */
  eps_uinteger TOTAL_IPR; /* Total count of all IPRs in the product */
  eps_uinteger TOTAL_GEADR; /* Total count of all GEADRs in the product 
                               */
  eps_uinteger TOTAL_GIADR; /* Total count of all GIADRs in the product 
                               */
  eps_uinteger TOTAL_VEADR; /* Total count of all VEADRs in the product 
                               */
  eps_uinteger TOTAL_VIADR; /* Total count of all VIADRs in the product 
                               */
  eps_uinteger TOTAL_MDR; /* Total count of all MDRs in the product */
  /* Record_Based_Generic_Quality_Flags */
  eps_uinteger COUNT_DEGRADED_INST_MDR; /* Count of MDRs with degradati
                                           on due to instrument problem
                                           s */
  eps_uinteger COUNT_DEGRADED_PROC_MDR; /* Count of MDRs with degradati
                                           on due to processing problem
                                           s */
  eps_uinteger COUNT_DEGRADED_INST_MDR_BLOCKS; /* Count of the number o
                                                  f blocks of MDRs degr
                                                  aded due to degraded 
                                                  instrument */
  eps_uinteger COUNT_DEGRADED_PROC_MDR_BLOCKS; /* Count of the number o
                                                  f blocks of MDRs degr
                                                  aded due to degraded 
                                                  processing */
  /* Time_Based_Generic_Quality_Flags */
  eps_uinteger DURATION_OF_PRODUCT; /* The duration of the product in m
                                       illiseconds */
  eps_uinteger MILLISECONDS_OF_DATA_PRESENT; /* The total amount of dat
                                                a present in the produc
                                                t */
  eps_uinteger MILLISECONDS_OF_DATA_MISSING; /* The total amount of dat
                                                a missing from the prod
                                                cut */
  /* Regional_Product_Information */
  eps_boolean SUBSETTED_PRODUCT; /* Set when product has been subsetted 
                                    (e.g. geographically subsetted usin
                                    g a region of interest filter). Imp
                                    lies the presence of one or more UM
                                    ARF GIADRs in GAD section for produ
                                    ct retrieved from UMARF. */
} t_mphr;

typedef struct {
  /* Product_Details */
  eps_byte PRODUCT_NAME[67]; /* Complete name of the product */
  eps_byte PARENT_PRODUCT_NAME_1[67]; /* Name of the parent product fro
                                         m which this product has been 
                                         produced. For Level 0 products
                                         , this field is filled with lo
                                         wer case x's. */
  eps_byte PARENT_PRODUCT_NAME_2[67]; /* Name of the parent product fro
                                         m which this product has been 
                                         produced. For Level 0 products 
                                         or products for which this is 
                                         not appropriate, this field is 
                                         filled with lower case x's. */
  eps_byte PARENT_PRODUCT_NAME_3[67]; /* Name of the parent product fro
                                         m which this product has been 
                                         produced. For Level 0 products 
                                         or products for which this is 
                                         not appropriate, this field is 
                                         filled with lower case x's. */
  eps_byte PARENT_PRODUCT_NAME_4[67]; /* Name of the parent product fro
                                         m which this product has been 
                                         produced. For Level 0 products 
                                         or products for which this is 
                                         not appropriate, this field is 
                                         filled with lower case x's. */
  eps_byte INSTRUMENT_ID[4]; /* Instrument identification */
  eps_byte INSTRUMENT_MODEL[3]; /* Instrument Model identification */
  eps_byte PRODUCT_TYPE[3]; /* Product Type */
  eps_byte PROCESSING_LEVEL[2]; /* Processing Level Identification */
  eps_byte SPACECRAFT_ID[3]; /* Spacecraft identification */
  eps_gentime SENSING_START; /* UTC Time of start of sensing data in th
                                is object (PDU, ROI or Full Product) */
  eps_gentime SENSING_END; /* UTC Time of end of sensing data in this o
                              bject (PDU, ROI or Full Product) */
  eps_gentime SENSING_START_THEORETICAL; /* Theoretical UTC Time of sta
                                            rt of sensing data in the d
                                            ump from which this object 
                                            is derived. This data is th
                                            e predicted start time at t
                                            he MPF level. */
  eps_gentime SENSING_END_THEORETICAL; /* Theoretical UTC Time of end o
                                          f sensing data in the dump fr
                                          om which this object is deriv
                                          ed. This data is the predicte
                                          d end time at the MPF level. 
                                          */
  eps_byte PROCESSING_CENTRE[4]; /* Processing Centre Identification */
  eps_uinteger PROCESSOR_MAJOR_VERSION; /* Processing chain major versi
                                           on number */
  eps_uinteger PROCESSOR_MINOR_VERSION; /* Processing chain minor versi
                                           on number */
  eps_uinteger FORMAT_MAJOR_VERSION; /* Dataset Format Major Version nu
                                        mber */
  eps_uinteger FORMAT_MINOR_VERSION; /* Dataset Format Minor Version nu
                                        mber */
  eps_gentime PROCESSING_TIME_START; /* UTC time of the processing at s
                                        tart of processing for the prod
                                        uct */
  eps_gentime PROCESSING_TIME_END; /* UTC time of the processing at end 
                                      of processing for the product */
  eps_byte PROCESSING_MODE[1]; /* Identification of the mode of process
                                  ing */
  eps_byte DISPOSITION_MODE[1]; /* Identification of the diposition mod
                                   e */
  eps_byte RECEIVING_GROUND_STATION[3]; /* Acquisition Station Identifi
                                           cation */
  eps_gentime RECEIVE_TIME_START; /* UTC time of the reception at CDA f
                                     or first Data Item */
  eps_gentime RECEIVE_TIME_END; /* UTC time of the reception at CDA for 
                                   last Data Item */
  eps_uinteger ORBIT_START; /* Start Orbit Number, counted incrementall
                               y since launch */
  eps_uinteger ORBIT_END; /* Stop Orbit Number */
  eps_uinteger ACTUAL_PRODUCT_SIZE; /* Size of the complete product */
  /* ASCENDING_NODE_ORBIT_PARAMETERS */
  eps_longgentime STATE_VECTOR_TIME; /* Epoch time (in UTC) of the orbi
                                        tal elements and the orbit stat
                                        e vector. this corresponds to t
                                        he time of crossing the ascendi
                                        ng node for ORBIT_START */
  eps_integer SEMI_MAJOR_AXIS; /* Semi major axis of orbit at time of t
                                  he ascending node crossing. */
  eps_real ECCENTRICITY; /* Orbit eccentricity at time of the ascending 
                            node crossing */
  eps_real INCLINATION; /* Orbit inclination at time of the ascending n
                           ode crossing */
  eps_real PERIGEE_ARGUMENT; /* Argument of perigee at time of the asce
                                nding node crossing */
  eps_real RIGHT_ASCENSION; /* Right ascension at time of the ascending 
                               node crossing */
  eps_real MEAN_ANOMALY; /* Mean anomaly at time of the ascending node 
                            crossing */
  eps_real X_POSITION; /* X position of the orbit state vector in the o
                          rbit frame at ascending node */
  eps_real Y_POSITION; /* Y position of the orbit state vector in the o
                          rbit frame at ascending node */
  eps_real Z_POSITION; /* Z position of the orbit state vector in the o
                          rbit frame at ascending node */
  eps_real X_VELOCITY; /* X velocity of the orbit state vector in the o
                          rbit frame at ascending node */
  eps_real Y_VELOCITY; /* Y velocity of the orbit state vector in the o
                          rbit frame at ascending node */
  eps_real Z_VELOCITY; /* Z velocity of the orbit state vector in the o
                          rbit frame at ascending node */
  eps_integer EARTH_SUN_DISTANCE_RATIO; /* Earth-Sun distance ratio - r
                                           atio of current Eart-Sun dis
                                           tance to Mean Earth-Sun dist
                                           ance */
  eps_integer LOCATION_TOLERANCE_RADIAL; /* Nadir Earth location tolera
                                            nce radial */
  eps_integer LOCATION_TOLERANCE_CROSSTRACK; /* Nadir Earth location to
                                                lerance cross-track */
  eps_integer LOCATION_TOLERANCE_ALONGTRACK; /* Nadir Earth location to
                                                lerance along-track */
  eps_real YAW_ERROR; /* Constant Yaw attitude error */
  eps_real ROLL_ERROR; /* Constant Roll attitude error */
  eps_real PITCH_ERROR; /* Constant Pitch attitude error */
  /* LOCATION_SUMMARY */
  eps_real SUBSAT_LATITUDE_START; /* Latitude of sub-satellite point at 
                                     start of the data set */
  eps_real SUBSAT_LONGITUDE_START; /* Longitude of sub-satellite point 
                                      at start of the data set */
  eps_real SUBSAT_LATITUDE_END; /* Latitude of sub-satellite point at e
                                   nd of the data set */
  eps_real SUBSAT_LONGITUDE_END; /* Longitude of sub-satellite point at 
                                    end of the data set */
  /* Leap_Second_Information */
  eps_integer LEAP_SECOND; /* Occurence of Leap second within the produ
                              ct. Field is set to -1, 0 or +1 dependent 
                              upon occurrence of leap second and direct
                              ion. */
  eps_gentime LEAP_SECOND_UTC; /* UTC time of occurrence of the Leap Se
                                  cond (If no leap second in the produc
                                  t, value is null) */
  /* Record_counts */
  eps_uinteger TOTAL_RECORDS; /* Total count of all records in the prod
                                 uct */
  eps_uinteger TOTAL_MPHR; /* Total count of all MPHRs in product (shou
                              ld always be 1!) */
  eps_uinteger TOTAL_SPHR; /* Total count of all SPHRs in product (shou
                              ld be 0 or 1 only) */
  eps_uinteger TOTAL_IPR; /* Total count of all IPRs in the product */
  eps_uinteger TOTAL_GEADR; /* Total count of all GEADRs in the product 
                               */
  eps_uinteger TOTAL_GIADR; /* Total count of all GIADRs in the product 
                               */
  eps_uinteger TOTAL_VEADR; /* Total count of all VEADRs in the product 
                               */
  eps_uinteger TOTAL_VIADR; /* Total count of all VIADRs in the product 
                               */
  eps_uinteger TOTAL_MDR; /* Total count of all MDRs in the product */
  /* Record_Based_Generic_Quality_Flags */
  eps_uinteger COUNT_DEGRADED_INST_MDR; /* Count of MDRs with degradati
                                           on due to instrument problem
                                           s */
  eps_uinteger COUNT_DEGRADED_PROC_MDR; /* Count of MDRs with degradati
                                           on due to processing problem
                                           s */
  eps_uinteger COUNT_DEGRADED_INST_MDR_BLOCKS; /* Count of the number o
                                                  f blocks of MDRs degr
                                                  aded due to degraded 
                                                  instrument */
  eps_uinteger COUNT_DEGRADED_PROC_MDR_BLOCKS; /* Count of the number o
                                                  f blocks of MDRs degr
                                                  aded due to degraded 
                                                  processing */
  /* Time_Based_Generic_Quality_Flags */
  eps_uinteger DURATION_OF_PRODUCT; /* The duration of the product in m
                                       illiseconds */
  eps_uinteger MILLISECONDS_OF_DATA_PRESENT; /* The total amount of dat
                                                a present in the produc
                                                t */
  eps_uinteger MILLISECONDS_OF_DATA_MISSING; /* The total amount of dat
                                                a missing from the prod
                                                cut */
  /* Regional_Product_Information */
  eps_boolean SUBSETTED_PRODUCT; /* Set when product has been subsetted 
                                    (e.g. geographically subsetted usin
                                    g a region of interest filter). Imp
                                    lies the presence of one or more UM
                                    ARF GIADRs in GAD section for produ
                                    ct retrieved from UMARF. */
} t_s_mphr;

int read_mphr( t_mphr *X, buffer* B, int n );

int write_mphr( t_mphr *X, buffer* B, int n );

int free_mphr( t_mphr *X, int n );

int init_mphr( t_mphr *X, int n );

int size_mphr( t_mphr *X );

int scale_mphr( const t_mphr *X, t_s_mphr *Y, int n );

int print_mphr( const char* label, t_mphr *X, FILE *fp, int ndim, const int *dims );

typedef struct {
  eps_uinteger1 TARGET_RECORD_CLASS; /* Class of target record as deriv
                                        ed from the GRH of target recor
                                        d */
  eps_uinteger1 TARGET_INSTRUMENT_GROUP; /* Instrument group of target 
                                            record as derived from the 
                                            GRH of target record */
  eps_uinteger1 TARGET_RECORD_SUBCLASS; /* Subclass of target record as 
                                           derived from the GRH of targ
                                           et record */
  eps_uinteger4 TARGET_RECORD_OFFSET; /* Offset of target record from s
                                         tart of product */
} t_ipr;

typedef struct {
  eps_uinteger1 TARGET_RECORD_CLASS; /* Class of target record as deriv
                                        ed from the GRH of target recor
                                        d */
  eps_uinteger1 TARGET_INSTRUMENT_GROUP; /* Instrument group of target 
                                            record as derived from the 
                                            GRH of target record */
  eps_uinteger1 TARGET_RECORD_SUBCLASS; /* Subclass of target record as 
                                           derived from the GRH of targ
                                           et record */
  eps_uinteger4 TARGET_RECORD_OFFSET; /* Offset of target record from s
                                         tart of product */
} t_s_ipr;

int read_ipr( t_ipr *X, buffer* B, int n );

int write_ipr( t_ipr *X, buffer* B, int n );

int free_ipr( t_ipr *X, int n );

int init_ipr( t_ipr *X, int n );

int size_ipr( t_ipr *X );

int scale_ipr( const t_ipr *X, t_s_ipr *Y, int n );

int print_ipr( const char* label, t_ipr *X, FILE *fp, int ndim, const int *dims );

typedef struct {
  /* QUALITY_PARAMETERS */
  eps_integer4 IDefPsfSondNbLin[PN];
  eps_integer4 IDefPsfSondNbCol[PN];
  eps_vinteger4 IDefPsfSondOverSampFactor; /* Oversampling factor for s
                                              ounder IPSF */
  eps_integer4 IDefPsfSondY[PN][100];
  eps_integer4 IDefPsfSondZ[PN][100];
  eps_vinteger4 IDefPsfSondWgt[PN][100][100];
  eps_integer4 IDefllSSrfNsfirst; /* Number of first sample in spectral 
                                     imager filter */
  eps_integer4 IDefllSSrfNslast; /* Number of last sample in spectral i
                                    mager filter */
  eps_vinteger4 IDefllSSrf[100];
  eps_vinteger4 IDefllSSrfDWn; /* Sample width of spectral imager filte
                                  r */
  eps_vinteger4 IDefIISNeDT[IMLI][IMCO];
  eps_boolean IDefDptIISDeadPix[IMLI][IMCO];
} t_giadr_quality;

typedef struct {
  /* QUALITY_PARAMETERS */
  eps_integer4 IDefPsfSondNbLin[PN];
  eps_integer4 IDefPsfSondNbCol[PN];
  eps_real4 IDefPsfSondOverSampFactor; /* Oversampling factor for sound
                                          er IPSF */
  eps_real4 IDefPsfSondY[PN][100];
  eps_real4 IDefPsfSondZ[PN][100];
  eps_real4 IDefPsfSondWgt[PN][100][100];
  eps_integer4 IDefllSSrfNsfirst; /* Number of first sample in spectral 
                                     imager filter */
  eps_integer4 IDefllSSrfNslast; /* Number of last sample in spectral i
                                    mager filter */
  eps_real4 IDefllSSrf[100];
  eps_real4 IDefllSSrfDWn; /* Sample width of spectral imager filter */
  eps_real4 IDefIISNeDT[IMLI][IMCO];
  eps_boolean IDefDptIISDeadPix[IMLI][IMCO];
} t_s_giadr_quality;

int read_giadr_quality( t_giadr_quality *X, buffer* B, int n );

int write_giadr_quality( t_giadr_quality *X, buffer* B, int n );

int free_giadr_quality( t_giadr_quality *X, int n );

int init_giadr_quality( t_giadr_quality *X, int n );

int size_giadr_quality( t_giadr_quality *X );

int scale_giadr_quality( const t_giadr_quality *X, t_s_giadr_quality *Y, int n );

int print_giadr_quality( const char* label, t_giadr_quality *X, FILE *fp, int ndim, const int *dims );

typedef struct {
  /* Scale_Factors */
  eps_integer2 IDefScaleSondNbScale; /* Number of bands used for applyi
                                        ng scale factors to spectra (up 
                                        to a maximum of 10) */
  eps_integer2 IDefScaleSondNsfirst[10];
  eps_integer2 IDefScaleSondNslast[10];
  eps_integer2 IDefScaleSondScaleFactor[10];
  eps_integer2 IDefScaleIISScaleFactor; /* Scale factor (power of 10) t
                                           o be applied to the IIS imag
                                           er. */
} t_giadr_scalefactors;

typedef struct {
  /* Scale_Factors */
  eps_integer2 IDefScaleSondNbScale; /* Number of bands used for applyi
                                        ng scale factors to spectra (up 
                                        to a maximum of 10) */
  eps_integer2 IDefScaleSondNsfirst[10];
  eps_integer2 IDefScaleSondNslast[10];
  eps_integer2 IDefScaleSondScaleFactor[10];
  eps_integer2 IDefScaleIISScaleFactor; /* Scale factor (power of 10) t
                                           o be applied to the IIS imag
                                           er. */
} t_s_giadr_scalefactors;

int read_giadr_scalefactors( t_giadr_scalefactors *X, buffer* B, int n );

int write_giadr_scalefactors( t_giadr_scalefactors *X, buffer* B, int n );

int free_giadr_scalefactors( t_giadr_scalefactors *X, int n );

int init_giadr_scalefactors( t_giadr_scalefactors *X, int n );

int size_giadr_scalefactors( t_giadr_scalefactors *X );

int scale_giadr_scalefactors( const t_giadr_scalefactors *X, t_s_giadr_scalefactors *Y, int n );

int print_giadr_scalefactors( const char* label, t_giadr_scalefactors *X, FILE *fp, int ndim, const int *dims );

typedef struct {
  /* GENERIC_QUALITY_INDICATORS */
  eps_boolean DEGRADED_INST_MDR; /* Quality of MDR has been degraded fr
                                    om nominal due to an instrument deg
                                    radation */
  eps_boolean DEGRADED_PROC_MDR; /* Quality of MDR has been degraded fr
                                    om nominal due to a processing degr
                                    adation */
  /* Level_1_Data */
  eps_byte GEPSIasiMode[4]; /* Instrument mode */
  eps_byte GEPSOPSProcessingMode[4]; /* Processing mode */
  eps_byte GEPSIdConf[32]; /* System configuration at line level: PTSI, 
                              TEC conf file ID,..... */
  eps_vinteger4 GEPSLocIasiAvhrr_IASI[SNOT][PN][2];
  eps_vinteger4 GEPSLocIasiAvhrr_IIS[SNOT][SGI][2];
  eps_byte OBT[SNOT][6];
  eps_ccsdstime OnboardUTC[SNOT];
  eps_ccsdstime GEPSDatIasi[SNOT];
  eps_integer4 GIsfLinOrigin[CCD];
  eps_integer4 GIsfColOrigin[CCD];
  eps_integer4 GIsfPds1[CCD];
  eps_integer4 GIsfPds2[CCD];
  eps_integer4 GIsfPds3[CCD];
  eps_integer4 GIsfPds4[CCD];
  eps_boolean GEPS_CCD[SNOT];
  eps_integer4 GEPS_SP[SNOT];
  eps_uinteger2 GIrcImage[SNOT][IMLI][IMCO];
  eps_boolean GQisFlagQual[SNOT][PN][SB];
  eps_byte GQisFlagQualDetailed[SNOT][PN][2];
  eps_vinteger4 GQisQualIndex; /* System-IASI general quality index */
  eps_vinteger4 GQisQualIndexIIS; /* IIS imager quality index inside 1b 
                                     [product */
  eps_vinteger4 GQisQualIndexLoc; /* Geometric quality index for sounde
                                     r product */
  eps_vinteger4 GQisQualIndexRad; /* Radiometric quality index for soun
                                     der product */
  eps_vinteger4 GQisQualIndexSpect; /* Spectral quality index for sound
                                       er product */
  eps_uinteger4 GQisSysTecIISQual; /* System -TEC quality index for IIS 
                                      */
  eps_uinteger4 GQisSysTecSondQual; /* System -TEC quality index for so
                                       under */
  eps_integer4 GGeoSondLoc[SNOT][PN][2];
  eps_integer4 GGeoSondAnglesMETOP[SNOT][PN][2];
  eps_integer4 GGeoIISAnglesMETOP[SNOT][SGI][2];
  eps_integer4 GGeoSondAnglesSUN[SNOT][PN][2];
  eps_integer4 GGeoIISAnglesSUN[SNOT][SGI][2];
  eps_integer4 GGeoIISLoc[SNOT][SGI][2];
  eps_uinteger4 EARTH_SATELLITE_DISTANCE; /* Distance of satellite from 
                                             Earth centre */
  /* Level_1b_Specific_Data */
  eps_vinteger4 IDefSpectDWn1b; /* Sample width of IASI 1b spectra (sam
                                   e as 1B) */
  eps_integer4 IDefNsfirst1b; /* Number of the first sample of IASI 1b 
                                 spectra (same as 1B) */
  eps_integer4 IDefNslast1b; /* Number of the last sample of IASI 1b sp
                                ectra (same as 1B) */
  eps_integer2 GS1bSpect[SNOT][PN][SS];
  eps_vinteger4 IDefCovarMatEigenVal1b[100][CCD];
} t_mdr_1b;

typedef struct {
  /* GENERIC_QUALITY_INDICATORS */
  eps_boolean DEGRADED_INST_MDR; /* Quality of MDR has been degraded fr
                                    om nominal due to an instrument deg
                                    radation */
  eps_boolean DEGRADED_PROC_MDR; /* Quality of MDR has been degraded fr
                                    om nominal due to a processing degr
                                    adation */
  /* Level_1_Data */
  eps_byte GEPSIasiMode[4]; /* Instrument mode */
  eps_byte GEPSOPSProcessingMode[4]; /* Processing mode */
  eps_byte GEPSIdConf[32]; /* System configuration at line level: PTSI, 
                              TEC conf file ID,..... */
  eps_real4 GEPSLocIasiAvhrr_IASI[SNOT][PN][2];
  eps_real4 GEPSLocIasiAvhrr_IIS[SNOT][SGI][2];
  eps_byte OBT[SNOT][6];
  eps_ccsdstime OnboardUTC[SNOT];
  eps_ccsdstime GEPSDatIasi[SNOT];
  eps_integer4 GIsfLinOrigin[CCD];
  eps_integer4 GIsfColOrigin[CCD];
  eps_real4 GIsfPds1[CCD];
  eps_real4 GIsfPds2[CCD];
  eps_real4 GIsfPds3[CCD];
  eps_real4 GIsfPds4[CCD];
  eps_boolean GEPS_CCD[SNOT];
  eps_integer4 GEPS_SP[SNOT];
  eps_uinteger2 GIrcImage[SNOT][IMLI][IMCO];
  eps_boolean GQisFlagQual[SNOT][PN][SB];
  eps_byte GQisFlagQualDetailed[SNOT][PN][2];
  eps_real4 GQisQualIndex; /* System-IASI general quality index */
  eps_real4 GQisQualIndexIIS; /* IIS imager quality index inside 1b [pr
                                 oduct */
  eps_real4 GQisQualIndexLoc; /* Geometric quality index for sounder pr
                                 oduct */
  eps_real4 GQisQualIndexRad; /* Radiometric quality index for sounder 
                                 product */
  eps_real4 GQisQualIndexSpect; /* Spectral quality index for sounder p
                                   roduct */
  eps_uinteger4 GQisSysTecIISQual; /* System -TEC quality index for IIS 
                                      */
  eps_uinteger4 GQisSysTecSondQual; /* System -TEC quality index for so
                                       under */
  eps_real4 GGeoSondLoc[SNOT][PN][2];
  eps_real4 GGeoSondAnglesMETOP[SNOT][PN][2];
  eps_real4 GGeoIISAnglesMETOP[SNOT][SGI][2];
  eps_real4 GGeoSondAnglesSUN[SNOT][PN][2];
  eps_real4 GGeoIISAnglesSUN[SNOT][SGI][2];
  eps_real4 GGeoIISLoc[SNOT][SGI][2];
  eps_uinteger4 EARTH_SATELLITE_DISTANCE; /* Distance of satellite from 
                                             Earth centre */
  /* Level_1b_Specific_Data */
  eps_real4 IDefSpectDWn1b; /* Sample width of IASI 1b spectra (same as 
                               1B) */
  eps_integer4 IDefNsfirst1b; /* Number of the first sample of IASI 1b 
                                 spectra (same as 1B) */
  eps_integer4 IDefNslast1b; /* Number of the last sample of IASI 1b sp
                                ectra (same as 1B) */
  eps_integer2 GS1bSpect[SNOT][PN][SS];
  eps_real4 IDefCovarMatEigenVal1b[100][CCD];
  eps_integer4 IDefCcsChannelId[NBK];
  eps_integer4 GCcsRadAnalNbClass[SNOT][PN];
  eps_real4 GCcsRadAnalWgt[SNOT][PN][NCL];
  eps_real4 GCcsRadAnalY[SNOT][PN][NCL];
  eps_real4 GCcsRadAnalZ[SNOT][PN][NCL];
  eps_real4 GCcsRadAnalMean[SNOT][PN][NCL][NBK];
  eps_real4 GCcsRadAnalStd[SNOT][PN][NCL][NBK];
  eps_uinteger1 GCcsImageClassified[SNOT][AMLI][AMCO];
  eps_byte IDefCcsMode[4]; /* Radiance Analysis: Image used is from AVH
                              RR or IIS imager (degraded cases) */
  eps_integer2 GCcsImageClassifiedNbLin[SNOT];
  eps_integer2 GCcsImageClassifiedNbCol[SNOT];
  eps_real4 GCcsImageClassifiedFirstLin[SNOT];
  eps_real4 GCcsImageClassifiedFirstCol[SNOT];
  eps_boolean GCcsRadAnalType[SNOT][NCL];
} t_s_mdr_1b;

int read_mdr_1b( t_mdr_1b *X, buffer* B, int n );

int write_mdr_1b( t_mdr_1b *X, buffer* B, int n );

int free_mdr_1b( t_mdr_1b *X, int n );

int init_mdr_1b( t_mdr_1b *X, int n );

int size_mdr_1b( t_mdr_1b *X );

int scale_mdr_1b( const t_mdr_1b *X, t_s_mdr_1b *Y, int n );

int print_mdr_1b( const char* label, t_mdr_1b *X, FILE *fp, int ndim, const int *dims );


typedef struct {
  REC_HEAD grh;
  union {
     t_mphr* MPHR;
     t_ipr* IPR;
     t_giadr_quality* GIADR_QUALITY;
     t_giadr_scalefactors* GIADR_SCALEFACTORS;
     t_mdr_1b* MDR_1B;
  } content;
} t_eps_iasil1b_9_0;

int get_eps_iasil1b_9_0( t_eps_iasil1b_9_0 *R, FILE *fp );
int put_eps_iasil1b_9_0( t_eps_iasil1b_9_0 *R, FILE *fp );
int free_eps_iasil1b_9_0( t_eps_iasil1b_9_0 *R );

int convert_iasi1b_intf( int argc, char* argv[] );

#endif
