/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/5
 *
 * SYNOPSIS  : Swap bytes of IASI_BRD file; endianness is auto-detected
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * VERSION   : 1.2
 *
 * HISTORY   : 16/01/2006 : Add LIBS information
 *             05/09/2006 : Auto-detect endianness
 *             07/06/2009 : Support format 1.7
 *
 * LIBS = cnes_iasi_brd_1.6 cnes_iasi_brd_1.7
 *
 */
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>

#include "cnes_iasi_brd_1.7.h"
#include "cnes_iasi_brd_1.6.h"
#include "eps_grh.h"
#include "eps_buffer.h"


int main( int argc, char* argv[] ) {
  FILE *fpr, *fpw;
  t_IASI_BRD X;
  t_IASI_BRD_1_7 X_1_7;
  buffer BR, BW;
  struct stat st;
  char buf[32];
  int argi = 1;
  int local = 0;
  eps_integer4 IDefIssueIcd;
  eps_integer4 IDefRevisionIcd;
  
  if( argc < 3 ) {
    fprintf( stderr, "Usage: [ -local ] IASI_BRD_AAPP_BIG_ENDIAN  IASI_BRD_AAPP_LITTLE_ENDIAN\n" );
    exit( 0 );
  }
  
  if( ! strcmp( argv[argi], "-local" ) ) {
    local++;
    argi++;
  }

  
  stat( argv[argi], &st );
  fpr = fopen( argv[argi], "r" );
  argi++;
  

  fread( buf, 32, 1, fpr );
  rewind( fpr );

  if( ( buf[0] == 0 ) && ( buf[1] == 0 ) ) {
    decode_endian = AAPP_BIG_ENDIAN;   
    encode_endian = AAPP_LITTLE_ENDIAN; 
  } else {
    decode_endian = AAPP_LITTLE_ENDIAN;
    encode_endian = AAPP_BIG_ENDIAN;
  }

  if( local ) 
    encode_endian = AAPP_BYTE_ORDER;

  if( ( decode_endian == AAPP_BIG_ENDIAN ) && ( encode_endian == AAPP_LITTLE_ENDIAN ) )
    printf( "Converting from BIG to LITTLE\n" );
  else if( ( decode_endian == AAPP_LITTLE_ENDIAN ) && ( encode_endian == AAPP_BIG_ENDIAN ) )
    printf( "Converting from LITTLE to BIG\n" );


  init_buffer( &BR, st.st_size );
  fread( BR.p, st.st_size, 1, fpr );

  read_int_buffer( &IDefIssueIcd, 4, 1, &BR );
  read_int_buffer( &IDefRevisionIcd, 4, 1, &BR );

  reset_buffer( &BR );
  
  switch (IDefRevisionIcd)
    {
    case 6:
      read_IASI_BRD( &X, &BR, 1 );
      break;
    case 7:
      read_IASI_BRD_1_7( &X_1_7, &BR, 1 );
      break;
    default:
      printf( "Unknown version: %d.%d\n", IDefIssueIcd, IDefRevisionIcd );
      return 1;
    }

  free_buffer( &BR );

  fclose( fpr );
  
  fpw = fopen( argv[argi], "w" );
  argi++;

  init_buffer( &BW, st.st_size );

  switch (IDefRevisionIcd)
    {
    case 6:
      write_IASI_BRD( &X, &BW, 1 );
      break;
    case 7:
      write_IASI_BRD_1_7( &X_1_7, &BW, 1 );
      break;
    default:
      printf( "Unknown version: %d.%d\n", IDefIssueIcd, IDefRevisionIcd );
      return 1;
    }

  fwrite( BW.p, st.st_size, 1, fpw );
  free_buffer( &BW );
  
  fclose( fpw );

  return 0;
}
