/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/5
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * SYNOPSIS  : Prints a text representation of a IASI_GRD
 *             file in big-endian format.
 *
 * VERSION   : 1.2
 *
 * HISTORY   : 16/01/2006 : Add LIBS information
 *             06/06/2009 : Support format 1.7
 *
 * LIBS = cnes_iasi_grd_1.6 cnes_iasi_grd_1.7
 *
 */
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>

#include "cnes_iasi_grd_1.6.h"
#include "cnes_iasi_grd_1.7.h"
#include "eps_grh.h"
#include "eps_buffer.h"


int main( int argc, char* argv[] ) {
  FILE *fpr;
  t_IASI_GRD X;
  t_IASI_GRD_1_7 X_1_7;
  buffer BR;
  struct stat st;
  eps_integer4 IDefIssueIcd;
  eps_integer4 IDefRevisionIcd;
  
  if( argc != 2 ) {
    fprintf( stderr, "Usage: IASI_GRD_AAPP_BIG_ENDIAN\n" );
    exit( 0 );
  }
  
  fpr = fopen( argv[1], "r" );
  
  stat( argv[1], &st );
  
  init_buffer( &BR, st.st_size );
  fread( BR.p, st.st_size, 1, fpr );

  read_int_buffer( &IDefIssueIcd, 4, 1, &BR );
  read_int_buffer( &IDefRevisionIcd, 4, 1, &BR );

  reset_buffer( &BR );
  
  switch (IDefRevisionIcd)
    {
    case 6:
      read_IASI_GRD( &X, &BR, 1 );
      print_IASI_GRD( "IASI_GRD", &X, stdout, 0, NULL ); 
      break;
    case 7:
      read_IASI_GRD_1_7( &X_1_7, &BR, 1 );
      print_IASI_GRD_1_7( "IASI_GRD", &X_1_7, stdout, 0, NULL ); 
      break;
    default:
      printf( "Unknown version: %d.%d\n", IDefIssueIcd, IDefRevisionIcd );
      return 1;
    }
  
  fclose( fpr );

  return 0;
}
