/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/5
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * SYNOPSIS  : Swap bytes of a IASI_ODB file; endianness is auto-detected, 
 *             -local option converts local endianess
 *
 * VERSION   : 1.1
 *
 * HISTORY   : 16/01/2006 : Add LIBS information
 *             05/09/2006 : Auto-detect endianness
 *
 * LIBS = cnes_iasi_odb_1.4
 *
 */
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>

#include "cnes_iasi_odb_1.4.h"
#include "eps_grh.h"
#include "eps_buffer.h"


int main( int argc, char* argv[] ) {
  FILE *fpr, *fpw;
  t_IASI_ODB* PX;
  buffer BR, BW;
  struct stat st;
  char buf[32];
  int argi = 1;
  int local = 0;
  
  if( argc < 3 ) {
    fprintf( stderr, "Usage: [ -local ] IASI_ODB_AAPP_BIG_ENDIAN  IASI_ODB_AAPP_LITTLE_ENDIAN\n" );
    exit( 0 );
  }
  
  if( ! strcmp( argv[argi], "-local" ) ) {
    local++;
    argi++;
  }

  
  stat( argv[argi], &st );
  fpr = fopen( argv[argi], "r" );
  argi++;


  fread( buf, 32, 1, fpr );
  rewind( fpr );

  if( ( buf[0] == 0 ) && ( buf[1] == 0 ) ) {
    decode_endian = AAPP_BIG_ENDIAN;   
    encode_endian = AAPP_LITTLE_ENDIAN; 
  } else {
    decode_endian = AAPP_LITTLE_ENDIAN;
    encode_endian = AAPP_BIG_ENDIAN;
  }

  if( local ) 
    encode_endian = AAPP_BYTE_ORDER;

  if( ( decode_endian == AAPP_BIG_ENDIAN ) && ( encode_endian == AAPP_LITTLE_ENDIAN ) )
    printf( "Converting from BIG to LITTLE\n" );
  else if( ( decode_endian == AAPP_LITTLE_ENDIAN ) && ( encode_endian == AAPP_BIG_ENDIAN ) )
    printf( "Converting from LITTLE to BIG\n" );


  PX = (t_IASI_ODB*)malloc( sizeof( t_IASI_ODB ) );

  init_buffer( &BR, st.st_size );
  fread( BR.p, st.st_size, 1, fpr );
  read_IASI_ODB( PX, &BR, 1 );
  free_buffer( &BR );

  fclose( fpr );
  


  fpw = fopen( argv[argi], "w" );
  argi++;

  init_buffer( &BW, st.st_size );
  write_IASI_ODB( PX, &BW, 1 );
  fwrite( BW.p, st.st_size, 1, fpw );
  free_buffer( &BW );
  
  fclose( fpw );

  free( PX );

  return 0;
}
