#!/bin/ksh
#
# Convert IASI level 1c PFS format to AAPP format (fixed record length)
#
# Arguments: (1) Input file (2) Output file (default ${1}.l1c)
#
# v1.1 13/3/2006 N C Atkinson
# v1.2 19/3/2007 fix return code
# v1.3 10/8/2009 Allow for format version 8.0
# v1.4 28/9/2009 rename 8.0 -> 9.0
# v1.5 22/07/2011 Remove ${FORT}10 before running exe
###############################################################################

. "$AAPP_PREFIX/ATOVS_CONF"

if [ ! -f "$1" ]; then
  echo "Usage: convert_iasi1c infile [outfile]"
  exit 1
fi

FORMAT_VERSION=$(strings $1 | head -20 | awk '$1=="FORMAT_MAJOR_VERSION" {print $3}')

rm -f ${FORT}10       #remove any existing link
if [ "$FORMAT_VERSION" -ge 11 ]; then
  echo "running convert_iasi1c_9.0.exe"
  convert_iasi1c_9.0.exe $1
else
  echo "running convert_iasi1c.exe"
  convert_iasi1c.exe $1
fi

code=$?
if [ $code -ne 0 ]
then 
  exit $code
fi

outfile=${2:-${1}.l1c}
mv ${FORT}10 $outfile
