/* 
! Name: $Id$
!
! Purpose: Top-level C program to convert IASI level 1c PFS format to HDF5 
!
* LIBS = eps_iasil1c_9.0
!-------------------------------------------------------------------------------
*/

#include "eps_iasil1c_9.0.h"
#include <stdio.h>

#ifdef HAS_LIBHDF5
#include <hdf5.h>
#endif

int main( int argc, char* argv[] ) {

#ifdef HAS_LIBHDF5
  t_eps_iasil1c_9_0 ER;
  FILE* fpr;
  hid_t       file_id, dataset_id, dataspace_id;  /* identifiers */
  hid_t       grp_mdr, grp_mphr, grp_giadr;
  hsize_t     dims[2];
  herr_t      status;
  int mdr   = 0;
  int mphr  = 0;
  int giadr = 0;
  
  fpr = fopen( argv[1], "r" );
  file_id = H5Fcreate("iasi1c.h5", H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT); /* Open the HDF5 1c file */

  while( get_eps_iasil1c_9_0( &ER, fpr ) ) {
  
    if( ER.grh.record_class == MDR ) {
      if( ER.grh.instrument_group != DUMMY ){
          mdr +=1; 
          if ( mdr == 1 ) { 
              grp_mdr = H5Gcreate(file_id, "/MDR", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
          }
          put_eps_iasil1c_hdf5( &ER, grp_mdr, mdr );

      }
    }

    if( ER.grh.record_class == MPHR ) {
        mphr +=1;
        if ( mphr == 1 ) { 
            grp_mphr = H5Gcreate(file_id, "/MPHR", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
        }
        put_eps_iasil1c_hdf5( &ER, grp_mphr, mphr );
    }

    if( ER.grh.record_class == GIADR && ER.grh.instrument_group != ARCHIVE ) {

      if (ER.content.GIADR_SCALEFACTORS->IDefScaleSondNbScale <= 10){
          giadr +=1;
          if ( giadr == 1 ) { 
            grp_giadr = H5Gcreate(file_id, "/GIADR", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
          }
          put_eps_iasil1c_hdf5( &ER, grp_giadr, giadr );
      }
    }
  
  free_eps_iasil1c_9_0( &ER );
  }


  fprintf( stderr, "mdr   %d \n", mdr);
  fprintf( stderr, "mphr  %d \n", mphr);
  fprintf( stderr, "giadr %d \n", giadr);

  if ( mdr >= 1 ) { 
    status = H5Gclose(grp_mdr);
  }
  if ( mphr >= 1 ) { 
    status = H5Gclose(grp_mphr);
  }
  if ( giadr >= 1 ) { 
    status = H5Gclose(grp_giadr);
  }

  /* Close the HDF5 1c file */   
  status = H5Fclose(file_id);

  return 0;

#else

  printf("ama1c_to_hdf5.exe has not been built with HDF-5\n");
  printf("Please run \"configure\" with --external-libs=\"-Lxxx/lib -lhdf5\"\n");
  printf("  and --external-includes=\"-Lxxx/include\"\n");
  printf("  where xxx is the location of the HDF-5 library\n");
  return 1;
  
#endif
}


