
/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2007/05
 *
 * SYNOPSIS  : Prints out the processing mode
 *             of the IASI instrument:
 *             either EXTERNAL_CALIBRATION or NORMAL_OPERATION
 *
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * VERSION   : 1.0
 *
 * LIBS = eps_iasil1c_6.6
 */



#include "eps_iasil1c_6.6.h"
#include <stdio.h>

int main( int argc, char * argv[] ) {
  FILE *fpr = fopen( argv[1], "r" );
  t_eps_iasil1c_6_6 ER;
  int mode;
  unsigned char* p;
  int c = 0;

  if( ! fpr ) {
    fprintf( stderr, "Usage: %s IASI_xx_1C_...\n", argv[0] );
    exit( 1 );
  }

  while( get_eps_iasil1c_6_6( &ER, fpr ) ) {
    if( ( ER.grh.record_class == MDR ) && ( ER.grh.instrument_group != DUMMY ) ) {
      p = (unsigned char*)ER.content.MDR_1C->GEPSIasiMode;
      mode = p[3] + 256 * p[2];
      c = c || ( mode == 0xa2 );
    }
    free_eps_iasil1c_6_6( &ER );
  }

  printf( c ? "EXTERNAL_CALIBRATION" : "NORMAL_OPERATION" );

  return 0;
}
