#!/usr/bin/perl -w

#
# COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
#
# DATE      : 2005/5
#
# AUTHOR    : Philippe.Marguinaud@meteo.fr
#
# SYNOPSIS  : Reformats ascii messages to an OSV file
#
# VERSION   : 1.0
#


use strict;
use POSIX qw( mktime );
use FileHandle;

@ARGV or die "Usage: $0 ascii-message [ output file ]\n";

my $fho;

if( @ARGV == 2 ) {
  $fho =  'FileHandle'->new( ">$ARGV[1]" );
  pop( @ARGV );
} else {
  $fho = \*STDOUT;
}

my $OSV = '';

my $ON = -1;

my $time2000 = mktime( 0, 0, 0, 0, 0, 2000 - 1900 );



while( <> ) {

  if( s/^(\d{2})-([A-Z]{3})-(\d{4}) (\d\d):(\d\d):(\d\d)\.(\d\d\d)\s*//o ) {
    my ( $DD, $MO, $YYYY, $hh, $mm, $ss, $uu ) =
       ( $1, $2, $3, $4, $5, $6, $7 );
    
    my ( $on, $fa, $dt, $id, $ev, $st ) = split;
    
    my @MM = qw( JAN FEB MAR APR MAY JUN JUL AUG SEP OCT NOV DEC );
    
    my $i = 0;
    while( @MM ) {
      my $mm = shift( @MM );
      last if( $MO =~ m/^$mm/i );
      $i++;
    }
    
    my $MM = $i;
    
    
    
    $OSV or do {
      $OSV = "<ELEMENT ID=\"$st\">";
    };
    
  
    if( ( $on != $ON ) && ( $ev =~ m/^(?:ANX|OOP_END|IP_END|IP_START|OOP_START)$/o ) ) {

      my $UUD = $uu + 1000 * ( $ss + 60 * ( $mm + 60 * $hh ) ) + $fa;
      
      my $time = mktime( 0, 0, 0, $DD, $MM, $YYYY - 1900 );
      
      my $DD2000 = ( $time - $time2000 ) / ( 24 * 3600 );
      
      while( $UUD > ( 24 * 3600 * 1000 ) ) {
        $UUD -= 24 * 3600 * 1000;
	$DD2000++;
      }
      
      
      $OSV .= "       
      <ORBIT_START ID=\"$on\">                                                    
              <FLAG>$ev</FLAG>      
              <TIME>$DD2000:$UUD:00000</TIME>
       </ORBIT_START>";

      $ON = $on;

    }
    
  
  }

}

$OSV && ( $OSV .= "</ELEMENT>\n" );

$OSV = << "EOF";
<?xml version='1.0'?>
<!DOCTYPE FD_Metop_NOAA_Orbit_Prediction SYSTEM "FD_Metop_NOAA_Orbit_Prediction.dtd">                                                                                                                                                                           
<FD_Metop_NOAA_Orbit_Prediction>
$OSV
</FD_Metop_NOAA_Orbit_Prediction>
EOF

print( $fho $OSV );


__DATA__
FILE NAME : xxxx_SVM_xx_M01_20030525150502Z_20030526085919Z_20030526104150Z_FDFx_EVENTSMESS
CREATION DATE : 25-MAY-2003 15:05:02.133

        UTC at ANX        Orbit No. fromANX  Durat. Unique Id. Event Description                        Sat   Target   Orbit Model Specific Information
======================== ========== ======= ======= ========== ======================================== === ========== =========== ========================================


25-MAY-2003 08:10:48.248         63       0       0    4536000 ANX                                      M01                 2
25-MAY-2003 09:52:12.726         64       0       0    4608000 ANX                                      M01                 2
25-MAY-2003 11:33:37.179         65       0       0    4680000 ANX                                      M01                 2
25-MAY-2003 13:15:01.655         66       0       0    4752000 ANX                                      M01                 2
25-MAY-2003 14:56:26.204         67       0       0    4824000 ANX                                      M01                 2
25-MAY-2003 16:37:50.816         68       0       0    4896000 ANX                                      M01                 2
25-MAY-2003 18:19:15.414         69       0       0    4968000 ANX                                      M01                 2
25-MAY-2003 20:00:39.950         70       0       0    5040000 ANX                                      M01                 2
25-MAY-2003 21:42:04.409         71       0       0    5112000 ANX                                      M01                 2
25-MAY-2003 23:23:28.866         72       0       0    5184000 ANX                                      M01                 2
26-MAY-2003 01:04:53.327         73       0       0    5256000 ANX                                      M01                 2
26-MAY-2003 02:46:17.830         74       0       0    5328000 ANX                                      M01                 2
26-MAY-2003 04:27:42.437         75       0       0    5400000 ANX                                      M01                 2
26-MAY-2003 06:09:07.026         76       0       0    5472000 ANX                                      M01                 2
26-MAY-2003 07:50:31.553         77       0       0    5544000 ANX                                      M01                 2
26-MAY-2003 09:31:56.039         78       0       0    5616000 ANX                                      M01                 2
26-MAY-2003 11:13:20.492         79       0       0    5688000 ANX                                      M01                 2
26-MAY-2003 12:54:44.960         80       0       0    5760000 ANX                                      M01                 2
26-MAY-2003 14:36:09.490         81       0       0    5832000 ANX                                      M01                 2
26-MAY-2003 16:17:34.095         82       0       0    5904000 ANX                                      M01                 2
26-MAY-2003 17:58:58.696         83       0       0    5976000 ANX                                      M01                 2
26-MAY-2003 19:40:23.251         84       0       0    6048000 ANX                                      M01                 2
26-MAY-2003 21:21:47.726         85       0       0    6120000 ANX                                      M01                 2
26-MAY-2003 23:03:12.183         86       0       0    6192000 ANX                                      M01                 2
26-MAY-2003 01:04:53.327         73 1157575  611171    5267576 STAT_AOS                                 M01 CDA_1           2      mask
26-MAY-2003 02:46:17.830         74 1259190  591857    5340592 STAT_AOS                                 M01 CDA_1           2      mask
26-MAY-2003 04:27:42.437         75 1309649  651438    5413097 STAT_AOS                                 M01 CDA_1           2      mask
