#!/usr/bin/perl -w

#
# COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
#
# DATE      : 2005/5
#
# AUTHOR    : Philippe.Marguinaud@meteo.fr
#
# SYNOPSIS  : Create an SVM file from a satpos file.
#
# VERSION   : 1.2
# 
# HISTORY   : 24/05/2007 : Change $hh - 1 to $hh in localtime
#           : 12/07/2013 : Add estimation of pure night, starting/ending 
#                          with an offset of 10 minutes compared to umbra
#


use strict;
use FileHandle;
use POSIX qw( mktime );

# offset between umbra and night in seconds 
my $night_offset = 600; 

my $fhi = 'FileHandle'->new( "<$ARGV[0]" );
my $fho = $ARGV[1] ?  'FileHandle'->new( ">$ARGV[1]" ) : \*STDOUT;

$fhi or die "Usage: $0 satpos_Mxx_YYYYMMDD.txt [ xxxx_SVM_xx_Mxx_... ]\n";
$fho or die "Cannot write to output file\n";

my ( $DD, $MM, $YY, $hh, $mm, $ss, $uuu );
my $dt;

while( <$fhi> ) {
  if( m{start\s+date\s+(\d{2})/(\d{2})/(\d{2})\s+(\d{2}):(\d{2}):(\d{2})\.(\d{3})}o ) {
    ( $DD, $MM, $YY, $hh, $mm, $ss, $uuu ) = ( $1, $2, $3, $4, $5, $6, $7 );
  }
  if( m{time\s+step\s+\(s\)\s+(\d+(?:\.\d+))}o ) {
    $dt = $1;
  }
  last if( m/^\s*step\s+x\s+y\s+z\s+vx\s+vy\s+vz\s+orb\s+sun\s+sta/o );
}

for( $DD, $MM, $YY, $hh, $mm, $ss, $uuu ) {
  s/^0*//o;
  $_ ||= 0;
}



my $time = mktime( $ss, $mm, $hh, $DD, $MM - 1, 100 + $YY );


my ( $f, $id, $SUN ) = ( 1, 1, 0 );

while( <$fhi> ) {
  my ( $step, $x, $y, $z, $vx, $vy, $vz, $orb, $sun, $sta ) = split;
  if( $f ) {
    printf( 
      $fho
      "%2.2d-%s-%4.4d %2.2d:%2.2d:%2.2d.%3.3d     %6d %7d %7d %10d %-40s\n", 
      1, "JAN", 1950, 0, 0, 0, 0, 0, 0, 0, $id++, 'UMBRA_END'
    );
    $SUN = 1;
    $f = 0;
  }
  
  if( $SUN != $sun ) {
  
    # 08-AUG-2002 14:32:05.461        109 5555631 1794262    7903556 UMBRA_STR
    #                             M01 EARTH           1      5.7383



    my ( $ss, $mm, $hh, $DD, $MM, $YYYY ) = localtime( $time + ( $step - 1 ) * $dt );

    my ( $ssa, $mma, $hha, $DDa, $MMa, $YYYYa ) = localtime( $time + ( $step - 1 ) * $dt -$night_offset);
    my ( $ssp, $mmp, $hhp, $DDp, $MMp, $YYYYp ) = localtime( $time + ( $step - 1 ) * $dt +$night_offset);


    $YYYY  += 1900;
    $YYYYa += 1900;
    $YYYYp += 1900;
    my @MM = qw( JAN FEB MAR APR MAY JUN JUL AUG SEP OCT NOV DEC );
    $MM  = $MM[$MM];
    $MMa = $MM[$MMa];
    $MMp = $MM[$MMp];

    my @U = qw( UMBRA_END UMBRA_STR );
    my @N = qw( NIGHT_END NIGHT_STR );

    if( $N[$sun] eq "NIGHT_STR" ) {
      printf( 
        $fho
        "%2.2d-%s-%4.4d %2.2d:%2.2d:%2.2d.%3.3d     %6d %7d %7d %10d %-40s\n", 
        $DDa, $MMa, $YYYYa, $hha, $mma, $ssa, $uuu, $orb, 0, 0, $id++, $N[$sun]
      );
    }
    
    printf( 
      $fho
      "%2.2d-%s-%4.4d %2.2d:%2.2d:%2.2d.%3.3d     %6d %7d %7d %10d %-40s\n", 
      $DD, $MM, $YYYY, $hh, $mm, $ss, $uuu, $orb, 0, 0, $id++, $U[$sun]
    );
    
    if( $N[$sun] eq "NIGHT_END" ) {
      printf( 
        $fho
        "%2.2d-%s-%4.4d %2.2d:%2.2d:%2.2d.%3.3d     %6d %7d %7d %10d %-40s\n", 
        $DDp, $MMp, $YYYYp, $hhp, $mmp, $ssp, $uuu, $orb, 0, 0, $id++, $N[$sun]
      );
    }
 
    $SUN = $sun;
  }
}
