/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/5
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * VERSION   : 1.0
 *
 */
#include "cnes_iasi_grd_1.6.h"

int cnes_iasi_grd_1_6_check_subclass = 1;
int cnes_iasi_grd_1_6_check_subclass_version = 1;

static const int dims_IASI_GRD_IDefDptIISDeadPix[] = { IMCO, IMLI };
static const int ndims_IASI_GRD_IDefDptIISDeadPix = 2;
static const int dims_IASI_GRD_IDefIISNeDT[] = { IMCO, IMLI };
static const int ndims_IASI_GRD_IDefIISNeDT = 2;
static const int dims_IASI_GRD_IDefIISSrf[] = { IIS_MAX };
static const int ndims_IASI_GRD_IDefIISSrf = 1;
static const int dims_IASI_GRD_IDefPsfSondWgt[] = { NB_COL_SOND, NB_LIN_SOND, PN };
static const int ndims_IASI_GRD_IDefPsfSondWgt = 3;
static const int dims_IASI_GRD_IDefPsfSondY[] = { NB_COL_SOND, PN };
static const int ndims_IASI_GRD_IDefPsfSondY = 2;
static const int dims_IASI_GRD_IDefPsfSondZ[] = { NB_COL_SOND, PN };
static const int ndims_IASI_GRD_IDefPsfSondZ = 2;
static const int dims_IASI_GRD_ISmaRbb[] = { SS1 };
static const int ndims_IASI_GRD_ISmaRbb = 1;
static const int dims_IASI_GRD_ISmaRcs[] = { SS1 };
static const int ndims_IASI_GRD_ISmaRcs = 1;
static const int dims_IASI_GRD_ISmaRew[] = { SS1, SN };
static const int ndims_IASI_GRD_ISmaRew = 2;
static const int dims_IASI_GRD_IWnpWnSW[] = { NS_SW, TAILLE_NB_SI_B };
static const int ndims_IASI_GRD_IWnpWnSW = 2;
static const int dims_IASI_GRD_IDefDeltaAvhrrDTbefore[] = { SNOT };
static const int ndims_IASI_GRD_IDefDeltaAvhrrDTbefore = 1;
static const int dims_IASI_GRD_IDefDeltaAvhrrDTafter[] = { SNOT };
static const int ndims_IASI_GRD_IDefDeltaAvhrrDTafter = 1;
static const int dims_IASI_GRD_IDefDeltaAvhrrColLeft[] = { SNOT };
static const int ndims_IASI_GRD_IDefDeltaAvhrrColLeft = 1;
static const int dims_IASI_GRD_IDefDeltaAvhrrColRight[] = { SNOT };
static const int ndims_IASI_GRD_IDefDeltaAvhrrColRight = 1;
static const int dims_IASI_GRD_IDefScaleSondNsfirst[] = { TAILLE_NB_SI_B };
static const int ndims_IASI_GRD_IDefScaleSondNsfirst = 1;
static const int dims_IASI_GRD_IDefScaleSondNslast[] = { TAILLE_NB_SI_B };
static const int ndims_IASI_GRD_IDefScaleSondNslast = 1;
static const int dims_IASI_GRD_IDefScaleSondScaleFactor[] = { TAILLE_NB_SI_B };
static const int ndims_IASI_GRD_IDefScaleSondScaleFactor = 1;
static const int dims_IASI_GRD_IDefScaleSondNsfirst1a[] = { TAILLE_NB_SI_B };
static const int ndims_IASI_GRD_IDefScaleSondNsfirst1a = 1;
static const int dims_IASI_GRD_IDefScaleSondNslast1a[] = { TAILLE_NB_SI_B };
static const int ndims_IASI_GRD_IDefScaleSondNslast1a = 1;
static const int dims_IASI_GRD_IDefScaleSondScaleFactor1a[] = { TAILLE_NB_SI_B };
static const int ndims_IASI_GRD_IDefScaleSondScaleFactor1a = 1;

int read_IASI_GRD( t_IASI_GRD *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    read_eps_integer4( &X->IDefIssueIcd, B, 1 );
    read_eps_integer4( &X->IDefRevisionIcd, B, 1 );
    read_eps_integer4( &X->IDefStableParamID, B, 1 );
    read_eps_integer4( &X->IDefStableParamDate_day, B, 1 );
    read_eps_integer4( &X->IDefStableParamDate_millis, B, 1 );
    read_eps_integer4( &X->IDefMETOPNumber, B, 1 );
    read_eps_integer4( &X->IDefAvhrrMarginBefore, B, 1 );
    read_eps_integer4( &X->IDefAvhrrMarginAfter, B, 1 );
    read_eps_boolean( (eps_boolean*)X->IDefDptIISDeadPix, B, ( IMCO )*( IMLI ) );
    read_eps_real8( (eps_real8*)X->IDefIISNeDT, B, ( IMCO )*( IMLI ) );
    read_eps_real8( (eps_real8*)X->IDefIISSrf, B, ( IIS_MAX ) );
    read_eps_real8( &X->IDefIISSrfDWn, B, 1 );
    read_eps_integer4( &X->IDefIISSrfNsfirst, B, 1 );
    read_eps_integer4( &X->IDefIISSrfNslast, B, 1 );
    read_eps_integer4( &X->IDefPsfSondNbCol, B, 1 );
    read_eps_integer4( &X->IDefPsfSondNbLin, B, 1 );
    read_eps_integer4( &X->IDefPsfSondOverSampFactor, B, 1 );
    read_eps_real8( (eps_real8*)X->IDefPsfSondWgt, B, ( NB_COL_SOND )*( NB_LIN_SOND )*( PN ) );
    read_eps_real8( (eps_real8*)X->IDefPsfSondY, B, ( NB_COL_SOND )*( PN ) );
    read_eps_real8( (eps_real8*)X->IDefPsfSondZ, B, ( NB_COL_SOND )*( PN ) );
    read_eps_real8( (eps_real8*)X->ISmaRbb, B, ( SS1 ) );
    read_eps_real8( (eps_real8*)X->ISmaRcs, B, ( SS1 ) );
    read_eps_real8( (eps_real8*)X->ISmaRew, B, ( SS1 )*( SN ) );
    read_eps_integer4( &X->ISmaRNs, B, 1 );
    read_eps_real8( (eps_real8*)X->IWnpWnSW, B, ( NS_SW )*( TAILLE_NB_SI_B ) );
    read_eps_real8( (eps_real8*)X->IDefDeltaAvhrrDTbefore, B, ( SNOT ) );
    read_eps_real8( (eps_real8*)X->IDefDeltaAvhrrDTafter, B, ( SNOT ) );
    read_eps_integer4( (eps_integer4*)X->IDefDeltaAvhrrColLeft, B, ( SNOT ) );
    read_eps_integer4( (eps_integer4*)X->IDefDeltaAvhrrColRight, B, ( SNOT ) );
    read_eps_integer2( &X->IDefScaleSondNbScale, B, 1 );
    read_eps_integer2( (eps_integer2*)X->IDefScaleSondNsfirst, B, ( TAILLE_NB_SI_B ) );
    read_eps_integer2( (eps_integer2*)X->IDefScaleSondNslast, B, ( TAILLE_NB_SI_B ) );
    read_eps_integer2( (eps_integer2*)X->IDefScaleSondScaleFactor, B, ( TAILLE_NB_SI_B ) );
    read_eps_integer2( &X->IDefScaleIISScaleFactor, B, 1 );
    read_eps_integer2( &X->IDefScaleSondNbScale1a, B, 1 );
    read_eps_integer2( (eps_integer2*)X->IDefScaleSondNsfirst1a, B, ( TAILLE_NB_SI_B ) );
    read_eps_integer2( (eps_integer2*)X->IDefScaleSondNslast1a, B, ( TAILLE_NB_SI_B ) );
    read_eps_integer2( (eps_integer2*)X->IDefScaleSondScaleFactor1a, B, ( TAILLE_NB_SI_B ) );
    read_eps_integer2( &X->IDefAvhrrSubGridDl, B, 1 );
    read_eps_integer2( &X->IDefAvhrrSubGridDc, B, 1 );
    read_eps_integer2( &X->IDefIacAvhrrChannel, B, 1 );
  }
  return status_buffer( B );
}


int write_IASI_GRD( t_IASI_GRD *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    write_eps_integer4( &X->IDefIssueIcd, B, 1 );
    write_eps_integer4( &X->IDefRevisionIcd, B, 1 );
    write_eps_integer4( &X->IDefStableParamID, B, 1 );
    write_eps_integer4( &X->IDefStableParamDate_day, B, 1 );
    write_eps_integer4( &X->IDefStableParamDate_millis, B, 1 );
    write_eps_integer4( &X->IDefMETOPNumber, B, 1 );
    write_eps_integer4( &X->IDefAvhrrMarginBefore, B, 1 );
    write_eps_integer4( &X->IDefAvhrrMarginAfter, B, 1 );
    write_eps_boolean( (eps_boolean*)X->IDefDptIISDeadPix, B, ( IMCO )*( IMLI ) );
    write_eps_real8( (eps_real8*)X->IDefIISNeDT, B, ( IMCO )*( IMLI ) );
    write_eps_real8( (eps_real8*)X->IDefIISSrf, B, ( IIS_MAX ) );
    write_eps_real8( &X->IDefIISSrfDWn, B, 1 );
    write_eps_integer4( &X->IDefIISSrfNsfirst, B, 1 );
    write_eps_integer4( &X->IDefIISSrfNslast, B, 1 );
    write_eps_integer4( &X->IDefPsfSondNbCol, B, 1 );
    write_eps_integer4( &X->IDefPsfSondNbLin, B, 1 );
    write_eps_integer4( &X->IDefPsfSondOverSampFactor, B, 1 );
    write_eps_real8( (eps_real8*)X->IDefPsfSondWgt, B, ( NB_COL_SOND )*( NB_LIN_SOND )*( PN ) );
    write_eps_real8( (eps_real8*)X->IDefPsfSondY, B, ( NB_COL_SOND )*( PN ) );
    write_eps_real8( (eps_real8*)X->IDefPsfSondZ, B, ( NB_COL_SOND )*( PN ) );
    write_eps_real8( (eps_real8*)X->ISmaRbb, B, ( SS1 ) );
    write_eps_real8( (eps_real8*)X->ISmaRcs, B, ( SS1 ) );
    write_eps_real8( (eps_real8*)X->ISmaRew, B, ( SS1 )*( SN ) );
    write_eps_integer4( &X->ISmaRNs, B, 1 );
    write_eps_real8( (eps_real8*)X->IWnpWnSW, B, ( NS_SW )*( TAILLE_NB_SI_B ) );
    write_eps_real8( (eps_real8*)X->IDefDeltaAvhrrDTbefore, B, ( SNOT ) );
    write_eps_real8( (eps_real8*)X->IDefDeltaAvhrrDTafter, B, ( SNOT ) );
    write_eps_integer4( (eps_integer4*)X->IDefDeltaAvhrrColLeft, B, ( SNOT ) );
    write_eps_integer4( (eps_integer4*)X->IDefDeltaAvhrrColRight, B, ( SNOT ) );
    write_eps_integer2( &X->IDefScaleSondNbScale, B, 1 );
    write_eps_integer2( (eps_integer2*)X->IDefScaleSondNsfirst, B, ( TAILLE_NB_SI_B ) );
    write_eps_integer2( (eps_integer2*)X->IDefScaleSondNslast, B, ( TAILLE_NB_SI_B ) );
    write_eps_integer2( (eps_integer2*)X->IDefScaleSondScaleFactor, B, ( TAILLE_NB_SI_B ) );
    write_eps_integer2( &X->IDefScaleIISScaleFactor, B, 1 );
    write_eps_integer2( &X->IDefScaleSondNbScale1a, B, 1 );
    write_eps_integer2( (eps_integer2*)X->IDefScaleSondNsfirst1a, B, ( TAILLE_NB_SI_B ) );
    write_eps_integer2( (eps_integer2*)X->IDefScaleSondNslast1a, B, ( TAILLE_NB_SI_B ) );
    write_eps_integer2( (eps_integer2*)X->IDefScaleSondScaleFactor1a, B, ( TAILLE_NB_SI_B ) );
    write_eps_integer2( &X->IDefAvhrrSubGridDl, B, 1 );
    write_eps_integer2( &X->IDefAvhrrSubGridDc, B, 1 );
    write_eps_integer2( &X->IDefIacAvhrrChannel, B, 1 );
  }
  return status_buffer( B );
}


int free_IASI_GRD( t_IASI_GRD *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
  }
}


int init_IASI_GRD( t_IASI_GRD *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
  }
}


int size_IASI_GRD( t_IASI_GRD *X ) {
  int i;
  return 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + ( IMCO )*( IMLI ) * 1 + ( IMCO )*( IMLI ) * 8 + ( IIS_MAX ) * 8 + 8 + 4 + 4 + 4 + 4 + 4 + ( NB_COL_SOND )*( NB_LIN_SOND )*( PN ) * 8 + ( NB_COL_SOND )*( PN ) * 8 + ( NB_COL_SOND )*( PN ) * 8 + ( SS1 ) * 8 + ( SS1 ) * 8 + ( SS1 )*( SN ) * 8 + 4 + ( NS_SW )*( TAILLE_NB_SI_B ) * 8 + ( SNOT ) * 8 + ( SNOT ) * 8 + ( SNOT ) * 4 + ( SNOT ) * 4 + 2 + ( TAILLE_NB_SI_B ) * 2 + ( TAILLE_NB_SI_B ) * 2 + ( TAILLE_NB_SI_B ) * 2 + 2 + 2 + ( TAILLE_NB_SI_B ) * 2 + ( TAILLE_NB_SI_B ) * 2 + ( TAILLE_NB_SI_B ) * 2 + 2 + 2 + 2;
}


int print_IASI_GRD( const char* label, t_IASI_GRD *X, FILE *fp, int ndim, const int *dims ) {
  int i;
  if( ndim == 0 ) {
    fprintf( fp, label );
    fprintf( fp, "\n" );
    print_eps_integer4( "IDefIssueIcd", &X->IDefIssueIcd, fp, 0, NULL );
    print_eps_integer4( "IDefRevisionIcd", &X->IDefRevisionIcd, fp, 0, NULL );
    print_eps_integer4( "IDefStableParamID", &X->IDefStableParamID, fp, 0, NULL );
    print_eps_integer4( "IDefStableParamDate_day", &X->IDefStableParamDate_day, fp, 0, NULL );
    print_eps_integer4( "IDefStableParamDate_millis", &X->IDefStableParamDate_millis, fp, 0, NULL );
    print_eps_integer4( "IDefMETOPNumber", &X->IDefMETOPNumber, fp, 0, NULL );
    print_eps_integer4( "IDefAvhrrMarginBefore", &X->IDefAvhrrMarginBefore, fp, 0, NULL );
    print_eps_integer4( "IDefAvhrrMarginAfter", &X->IDefAvhrrMarginAfter, fp, 0, NULL );
    print_eps_boolean( "IDefDptIISDeadPix", (eps_boolean*)X->IDefDptIISDeadPix, fp, ndims_IASI_GRD_IDefDptIISDeadPix, dims_IASI_GRD_IDefDptIISDeadPix );
    print_eps_real8( "IDefIISNeDT", (eps_real8*)X->IDefIISNeDT, fp, ndims_IASI_GRD_IDefIISNeDT, dims_IASI_GRD_IDefIISNeDT );
    print_eps_real8( "IDefIISSrf", (eps_real8*)X->IDefIISSrf, fp, ndims_IASI_GRD_IDefIISSrf, dims_IASI_GRD_IDefIISSrf );
    print_eps_real8( "IDefIISSrfDWn", &X->IDefIISSrfDWn, fp, 0, NULL );
    print_eps_integer4( "IDefIISSrfNsfirst", &X->IDefIISSrfNsfirst, fp, 0, NULL );
    print_eps_integer4( "IDefIISSrfNslast", &X->IDefIISSrfNslast, fp, 0, NULL );
    print_eps_integer4( "IDefPsfSondNbCol", &X->IDefPsfSondNbCol, fp, 0, NULL );
    print_eps_integer4( "IDefPsfSondNbLin", &X->IDefPsfSondNbLin, fp, 0, NULL );
    print_eps_integer4( "IDefPsfSondOverSampFactor", &X->IDefPsfSondOverSampFactor, fp, 0, NULL );
    print_eps_real8( "IDefPsfSondWgt", (eps_real8*)X->IDefPsfSondWgt, fp, ndims_IASI_GRD_IDefPsfSondWgt, dims_IASI_GRD_IDefPsfSondWgt );
    print_eps_real8( "IDefPsfSondY", (eps_real8*)X->IDefPsfSondY, fp, ndims_IASI_GRD_IDefPsfSondY, dims_IASI_GRD_IDefPsfSondY );
    print_eps_real8( "IDefPsfSondZ", (eps_real8*)X->IDefPsfSondZ, fp, ndims_IASI_GRD_IDefPsfSondZ, dims_IASI_GRD_IDefPsfSondZ );
    print_eps_real8( "ISmaRbb", (eps_real8*)X->ISmaRbb, fp, ndims_IASI_GRD_ISmaRbb, dims_IASI_GRD_ISmaRbb );
    print_eps_real8( "ISmaRcs", (eps_real8*)X->ISmaRcs, fp, ndims_IASI_GRD_ISmaRcs, dims_IASI_GRD_ISmaRcs );
    print_eps_real8( "ISmaRew", (eps_real8*)X->ISmaRew, fp, ndims_IASI_GRD_ISmaRew, dims_IASI_GRD_ISmaRew );
    print_eps_integer4( "ISmaRNs", &X->ISmaRNs, fp, 0, NULL );
    print_eps_real8( "IWnpWnSW", (eps_real8*)X->IWnpWnSW, fp, ndims_IASI_GRD_IWnpWnSW, dims_IASI_GRD_IWnpWnSW );
    print_eps_real8( "IDefDeltaAvhrrDTbefore", (eps_real8*)X->IDefDeltaAvhrrDTbefore, fp, ndims_IASI_GRD_IDefDeltaAvhrrDTbefore, dims_IASI_GRD_IDefDeltaAvhrrDTbefore );
    print_eps_real8( "IDefDeltaAvhrrDTafter", (eps_real8*)X->IDefDeltaAvhrrDTafter, fp, ndims_IASI_GRD_IDefDeltaAvhrrDTafter, dims_IASI_GRD_IDefDeltaAvhrrDTafter );
    print_eps_integer4( "IDefDeltaAvhrrColLeft", (eps_integer4*)X->IDefDeltaAvhrrColLeft, fp, ndims_IASI_GRD_IDefDeltaAvhrrColLeft, dims_IASI_GRD_IDefDeltaAvhrrColLeft );
    print_eps_integer4( "IDefDeltaAvhrrColRight", (eps_integer4*)X->IDefDeltaAvhrrColRight, fp, ndims_IASI_GRD_IDefDeltaAvhrrColRight, dims_IASI_GRD_IDefDeltaAvhrrColRight );
    print_eps_integer2( "IDefScaleSondNbScale", &X->IDefScaleSondNbScale, fp, 0, NULL );
    print_eps_integer2( "IDefScaleSondNsfirst", (eps_integer2*)X->IDefScaleSondNsfirst, fp, ndims_IASI_GRD_IDefScaleSondNsfirst, dims_IASI_GRD_IDefScaleSondNsfirst );
    print_eps_integer2( "IDefScaleSondNslast", (eps_integer2*)X->IDefScaleSondNslast, fp, ndims_IASI_GRD_IDefScaleSondNslast, dims_IASI_GRD_IDefScaleSondNslast );
    print_eps_integer2( "IDefScaleSondScaleFactor", (eps_integer2*)X->IDefScaleSondScaleFactor, fp, ndims_IASI_GRD_IDefScaleSondScaleFactor, dims_IASI_GRD_IDefScaleSondScaleFactor );
    print_eps_integer2( "IDefScaleIISScaleFactor", &X->IDefScaleIISScaleFactor, fp, 0, NULL );
    print_eps_integer2( "IDefScaleSondNbScale1a", &X->IDefScaleSondNbScale1a, fp, 0, NULL );
    print_eps_integer2( "IDefScaleSondNsfirst1a", (eps_integer2*)X->IDefScaleSondNsfirst1a, fp, ndims_IASI_GRD_IDefScaleSondNsfirst1a, dims_IASI_GRD_IDefScaleSondNsfirst1a );
    print_eps_integer2( "IDefScaleSondNslast1a", (eps_integer2*)X->IDefScaleSondNslast1a, fp, ndims_IASI_GRD_IDefScaleSondNslast1a, dims_IASI_GRD_IDefScaleSondNslast1a );
    print_eps_integer2( "IDefScaleSondScaleFactor1a", (eps_integer2*)X->IDefScaleSondScaleFactor1a, fp, ndims_IASI_GRD_IDefScaleSondScaleFactor1a, dims_IASI_GRD_IDefScaleSondScaleFactor1a );
    print_eps_integer2( "IDefAvhrrSubGridDl", &X->IDefAvhrrSubGridDl, fp, 0, NULL );
    print_eps_integer2( "IDefAvhrrSubGridDc", &X->IDefAvhrrSubGridDc, fp, 0, NULL );
    print_eps_integer2( "IDefIacAvhrrChannel", &X->IDefIacAvhrrChannel, fp, 0, NULL );
  } else {
    int k = 1;
    char label[16];
    for( i = 1; i < ndim; i++ ) {
      k *= dims[i];
    }
    for( i = 0; i < dims[0]; i++ ) {
      sprintf( label, "[%d]", i ); 
      print_IASI_GRD( label, X + i * k, fp, ndim-1, dims + 1 );
    }
  }
}


