/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/5
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * VERSION   : 1.0
 *
 */
#include "cnes_iasi_odb_1.4.h"

int cnes_iasi_odb_1_4_check_subclass = 1;
int cnes_iasi_odb_1_4_check_subclass_version = 1;


int read_IASI_ODB_ISdbGridAf( t_IASI_ODB_ISdbGridAf *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    read_eps_real4( &X->real, B, 1 );
    read_eps_real4( &X->imag, B, 1 );
  }
  return status_buffer( B );
}


int write_IASI_ODB_ISdbGridAf( t_IASI_ODB_ISdbGridAf *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    write_eps_real4( &X->real, B, 1 );
    write_eps_real4( &X->imag, B, 1 );
  }
  return status_buffer( B );
}


int free_IASI_ODB_ISdbGridAf( t_IASI_ODB_ISdbGridAf *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
  }
}


int init_IASI_ODB_ISdbGridAf( t_IASI_ODB_ISdbGridAf *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
  }
}


int size_IASI_ODB_ISdbGridAf( t_IASI_ODB_ISdbGridAf *X ) {
  int i;
  return 4 + 4;
}


int print_IASI_ODB_ISdbGridAf( const char* label, t_IASI_ODB_ISdbGridAf *X, FILE *fp, int ndim, const int *dims ) {
  int i;
  if( ndim == 0 ) {
    fprintf( fp, label );
    fprintf( fp, "\n" );
    print_eps_real4( "real", &X->real, fp, 0, NULL );
    print_eps_real4( "imag", &X->imag, fp, 0, NULL );
  } else {
    int k = 1;
    char label[16];
    for( i = 1; i < ndim; i++ ) {
      k *= dims[i];
    }
    for( i = 0; i < dims[0]; i++ ) {
      sprintf( label, "[%d]", i ); 
      print_IASI_ODB_ISdbGridAf( label, X + i * k, fp, ndim-1, dims + 1 );
    }
  }
}


static const int dims_IASI_ODB_IDefSdbGridY[] = { CDX, SDB_NB_COL };
static const int ndims_IASI_ODB_IDefSdbGridY = 2;
static const int dims_IASI_ODB_IDefSdbGridZ[] = { CDX, SDB_NB_LIG };
static const int ndims_IASI_ODB_IDefSdbGridZ = 2;
static const int dims_IASI_ODB_ISdbGridAf[] = { SDB_NB_LIG, SDB_NB_COL, CDX, PN, SDB_NFSMAX, SAF_NS_OPD };
static const int ndims_IASI_ODB_ISdbGridAf = 6;
static const int dims_IASI_ODB_ISdbGridFcs[] = { SDB_NB_LIG, SDB_NB_COL, CDX, PN, SDB_NFSMAX };
static const int ndims_IASI_ODB_ISdbGridFcs = 5;
static const int dims_IASI_ODB_ISdbGridWnShift[] = { SDB_NB_LIG, SDB_NB_COL, WNP_NB_SW, CDX, PN };
static const int ndims_IASI_ODB_ISdbGridWnShift = 5;

int read_IASI_ODB( t_IASI_ODB *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    read_eps_integer4( &X->IDefIssueIcd, B, 1 );
    read_eps_integer4( &X->IDefRevisionIcd, B, 1 );
    read_eps_integer4( &X->IDefSdbID, B, 1 );
    read_eps_integer4( &X->IDefSdbDate_day, B, 1 );
    read_eps_integer4( &X->IDefSdbDate_millis, B, 1 );
    read_eps_integer4( &X->IDefMETOPNumber, B, 1 );
    read_eps_real8( &X->IDefSafDWn, B, 1 );
    read_eps_integer4( &X->IDefSafNsfirst, B, 1 );
    read_eps_integer4( &X->IDefSafNslast, B, 1 );
    read_eps_integer4( &X->IDefSafNsOpd, B, 1 );
    read_eps_real8( &X->IDefSafOpdMax, B, 1 );
    read_eps_integer4( &X->IDefSdbGridNbCol, B, 1 );
    read_eps_integer4( &X->IDefSdbGridNbLin, B, 1 );
    read_eps_real8( (eps_real8*)X->IDefSdbGridY, B, ( CDX )*( SDB_NB_COL ) );
    read_eps_real8( (eps_real8*)X->IDefSdbGridZ, B, ( CDX )*( SDB_NB_LIG ) );
    read_IASI_ODB_ISdbGridAf( (t_IASI_ODB_ISdbGridAf*)X->ISdbGridAf, B, ( SDB_NB_LIG )*( SDB_NB_COL )*( CDX )*( PN )*( SDB_NFSMAX )*( SAF_NS_OPD ) );
    read_eps_real8( (eps_real8*)X->ISdbGridFcs, B, ( SDB_NB_LIG )*( SDB_NB_COL )*( CDX )*( PN )*( SDB_NFSMAX ) );
    read_eps_real8( (eps_real8*)X->ISdbGridWnShift, B, ( SDB_NB_LIG )*( SDB_NB_COL )*( WNP_NB_SW )*( CDX )*( PN ) );
  }
  return status_buffer( B );
}


int write_IASI_ODB( t_IASI_ODB *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    write_eps_integer4( &X->IDefIssueIcd, B, 1 );
    write_eps_integer4( &X->IDefRevisionIcd, B, 1 );
    write_eps_integer4( &X->IDefSdbID, B, 1 );
    write_eps_integer4( &X->IDefSdbDate_day, B, 1 );
    write_eps_integer4( &X->IDefSdbDate_millis, B, 1 );
    write_eps_integer4( &X->IDefMETOPNumber, B, 1 );
    write_eps_real8( &X->IDefSafDWn, B, 1 );
    write_eps_integer4( &X->IDefSafNsfirst, B, 1 );
    write_eps_integer4( &X->IDefSafNslast, B, 1 );
    write_eps_integer4( &X->IDefSafNsOpd, B, 1 );
    write_eps_real8( &X->IDefSafOpdMax, B, 1 );
    write_eps_integer4( &X->IDefSdbGridNbCol, B, 1 );
    write_eps_integer4( &X->IDefSdbGridNbLin, B, 1 );
    write_eps_real8( (eps_real8*)X->IDefSdbGridY, B, ( CDX )*( SDB_NB_COL ) );
    write_eps_real8( (eps_real8*)X->IDefSdbGridZ, B, ( CDX )*( SDB_NB_LIG ) );
    write_IASI_ODB_ISdbGridAf( (t_IASI_ODB_ISdbGridAf*)X->ISdbGridAf, B, ( SDB_NB_LIG )*( SDB_NB_COL )*( CDX )*( PN )*( SDB_NFSMAX )*( SAF_NS_OPD ) );
    write_eps_real8( (eps_real8*)X->ISdbGridFcs, B, ( SDB_NB_LIG )*( SDB_NB_COL )*( CDX )*( PN )*( SDB_NFSMAX ) );
    write_eps_real8( (eps_real8*)X->ISdbGridWnShift, B, ( SDB_NB_LIG )*( SDB_NB_COL )*( WNP_NB_SW )*( CDX )*( PN ) );
  }
  return status_buffer( B );
}


int free_IASI_ODB( t_IASI_ODB *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    free_IASI_ODB_ISdbGridAf( (t_IASI_ODB_ISdbGridAf*)X->ISdbGridAf, ( SDB_NB_LIG) * (SDB_NB_COL) * (CDX) * (PN) * (SDB_NFSMAX) * (SAF_NS_OPD ) );
  }
}


int init_IASI_ODB( t_IASI_ODB *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
  }
}


int size_IASI_ODB( t_IASI_ODB *X ) {
  int i;
  return 4 + 4 + 4 + 4 + 4 + 4 + 8 + 4 + 4 + 4 + 8 + 4 + 4 + ( CDX )*( SDB_NB_COL ) * 8 + ( CDX )*( SDB_NB_LIG ) * 8 + ( SDB_NB_LIG )*( SDB_NB_COL )*( CDX )*( PN )*( SDB_NFSMAX )*( SAF_NS_OPD ) * size_IASI_ODB_ISdbGridAf( (t_IASI_ODB_ISdbGridAf*)X->ISdbGridAf ) + ( SDB_NB_LIG )*( SDB_NB_COL )*( CDX )*( PN )*( SDB_NFSMAX ) * 8 + ( SDB_NB_LIG )*( SDB_NB_COL )*( WNP_NB_SW )*( CDX )*( PN ) * 8;
}


int print_IASI_ODB( const char* label, t_IASI_ODB *X, FILE *fp, int ndim, const int *dims ) {
  int i;
  if( ndim == 0 ) {
    fprintf( fp, label );
    fprintf( fp, "\n" );
    print_eps_integer4( "IDefIssueIcd", &X->IDefIssueIcd, fp, 0, NULL );
    print_eps_integer4( "IDefRevisionIcd", &X->IDefRevisionIcd, fp, 0, NULL );
    print_eps_integer4( "IDefSdbID", &X->IDefSdbID, fp, 0, NULL );
    print_eps_integer4( "IDefSdbDate_day", &X->IDefSdbDate_day, fp, 0, NULL );
    print_eps_integer4( "IDefSdbDate_millis", &X->IDefSdbDate_millis, fp, 0, NULL );
    print_eps_integer4( "IDefMETOPNumber", &X->IDefMETOPNumber, fp, 0, NULL );
    print_eps_real8( "IDefSafDWn", &X->IDefSafDWn, fp, 0, NULL );
    print_eps_integer4( "IDefSafNsfirst", &X->IDefSafNsfirst, fp, 0, NULL );
    print_eps_integer4( "IDefSafNslast", &X->IDefSafNslast, fp, 0, NULL );
    print_eps_integer4( "IDefSafNsOpd", &X->IDefSafNsOpd, fp, 0, NULL );
    print_eps_real8( "IDefSafOpdMax", &X->IDefSafOpdMax, fp, 0, NULL );
    print_eps_integer4( "IDefSdbGridNbCol", &X->IDefSdbGridNbCol, fp, 0, NULL );
    print_eps_integer4( "IDefSdbGridNbLin", &X->IDefSdbGridNbLin, fp, 0, NULL );
    print_eps_real8( "IDefSdbGridY", (eps_real8*)X->IDefSdbGridY, fp, ndims_IASI_ODB_IDefSdbGridY, dims_IASI_ODB_IDefSdbGridY );
    print_eps_real8( "IDefSdbGridZ", (eps_real8*)X->IDefSdbGridZ, fp, ndims_IASI_ODB_IDefSdbGridZ, dims_IASI_ODB_IDefSdbGridZ );
    print_IASI_ODB_ISdbGridAf( "ISdbGridAf", (t_IASI_ODB_ISdbGridAf*)X->ISdbGridAf, fp, ndims_IASI_ODB_ISdbGridAf, dims_IASI_ODB_ISdbGridAf );
    print_eps_real8( "ISdbGridFcs", (eps_real8*)X->ISdbGridFcs, fp, ndims_IASI_ODB_ISdbGridFcs, dims_IASI_ODB_ISdbGridFcs );
    print_eps_real8( "ISdbGridWnShift", (eps_real8*)X->ISdbGridWnShift, fp, ndims_IASI_ODB_ISdbGridWnShift, dims_IASI_ODB_ISdbGridWnShift );
  } else {
    int k = 1;
    char label[16];
    for( i = 1; i < ndim; i++ ) {
      k *= dims[i];
    }
    for( i = 0; i < dims[0]; i++ ) {
      sprintf( label, "[%d]", i ); 
      print_IASI_ODB( label, X + i * k, fp, ndim-1, dims + 1 );
    }
  }
}


