! Name: $Id$
!
! Purpose: Convert IASI level 1c PFS format to AAPP format (fixed record length)
!
! Method: This file contains a collection of Fortran subroutines that are 
!         called by the main "c" routine
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0      02/11/05  Original code. (N.C.Atkinson)
!  1.1      25/01/06  Change ??? character constants to XXX ( AIX
!                     compiler complains )
!  1.2      17/01/07  Change GEPSDatIasi argument to mdr1c
!                     to two distinct arrays GEPSDatIasi_days
!                     and GEPSDatIasi_milliseconds (PM)
!  1.3      22/02/07  Change all vinteger4 to distinct scale/value
!                     arrays
!  1.4      15/03/07  Default period if velocity bad. Fix bug in velocity
!                     formula. Exit with error if no scan lines (NCA)
!  1.5      25/05/07  Fix bug in start/end time header (PM)
!  1.6      23/11/07  Created from convert_iasi1c.F (NCA)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Functions and subroutines called:
!   ioi1c
!-------------------------------------------------------------------------------

      subroutine open1b
#include "inparms.h"
#include "iasi1c.h"
      call ioi1c ("OPEN/WRITE", 10, 1)
      iasi1c_h_site = 'XXX'
      iasi1c_h_cfill1 = ' '
      iasi1c_h_1bsite = 'XXX'
      iasi1c_h_cfill2 = ' '
      iasi1c_h_versnb = iasi_1cvnnb
      iasi1c_h_versyr = iasi_1cvnyr
      iasi1c_h_versdy = iasi_1cvndy
      iasi1c_h_hdrcnt = 1
      iasi1c_h_instrument = 13       !13=IASI
C      iasi1c_h_period = 6060         !6060 seconds = 101 minutes
      iasi1c_h_startdatayr = MDI     !update later
      iasi1c_h_scnlin = 0
      iasi1c_h_misscnlin = 0
      return
      end
      
!------------------------------------------------------------------------------
      subroutine finish1b
#include "iasi1c.h"
      call ioi1c ("WRITE HEADER", 10, 1)
      call ioi1c ("CLOSE", 10, 1)
      write(*,*) iasi1c_h_scnlin,
     &     ' scans written to output file on unit 10'
      write(*,*) 
     & " iasi1c_h_site = ", iasi1c_h_site,
     & " iasi1c_h_1bsite = ", iasi1c_h_1bsite,
     & " iasi1c_h_versnb = ", iasi1c_h_versnb,
     & " iasi1c_h_versyr = ", iasi1c_h_versyr,
     & " iasi1c_h_versdy = ", iasi1c_h_versdy,
     & " iasi1c_h_hdrcnt = ", iasi1c_h_hdrcnt,
     & " iasi1c_h_satid = ", iasi1c_h_satid,
     & " iasi1c_h_instrument = ", iasi1c_h_instrument,
     & " iasi1c_h_satht = ", iasi1c_h_satht,
     & " iasi1c_h_period = ", iasi1c_h_period,
     & " iasi1c_h_startorbit = ", iasi1c_h_startorbit,
     & " iasi1c_h_startdatayr = ", iasi1c_h_startdatayr,
     & " iasi1c_h_startdatady = ", iasi1c_h_startdatady,
     & " iasi1c_h_startdatatime = ", iasi1c_h_startdatatime,
     & " iasi1c_h_endorbit = ", iasi1c_h_endorbit,
     & " iasi1c_h_enddatayr = ", iasi1c_h_enddatayr,
     & " iasi1c_h_enddatady = ", iasi1c_h_enddatady,
     & " iasi1c_h_enddatatime = ", iasi1c_h_enddatatime,
     & " iasi1c_h_scnlin = ", iasi1c_h_scnlin,
     & " iasi1c_h_misscnlin = ", iasi1c_h_misscnlin,
     & " iasi1c_h_wmosatid = ", iasi1c_h_wmosatid,
     & " iasi1c_h_startchan = ", iasi1c_h_startchan,
     & " iasi1c_h_endchan = ", iasi1c_h_endchan,
     & " iasi1c_h_scalefactor = ", iasi1c_h_scalefactor

      if (iasi1c_h_scnlin .le. 0) then
        call aappexit(1)         !no scan lines: exit with error code
      endif
      return
      end

!------------------------------------------------------------------------------
      subroutine mdr1b(
     &          GEPSDatIasi_days,          
     &          GEPSDatIasi_milliseconds, ! Date and time 
     &          GQisFlagQual,        ! Quality flags 
     &          GQisQualIndex_scale,
     &          GQisQualIndex_value,
     &          GQisQualIndexLoc_scale,
     &          GQisQualIndexLoc_value,
     &          GQisQualIndexRad_scale,
     &          GQisQualIndexRad_value,
     &          GQisQualIndexSpect_scale,
     &          GQisQualIndexSpect_value,
     &          GQisSysTecSondQual,
     &          GGeoSondLoc,         ! Lat and lon 
     &          GGeoSondAnglesMETOP, ! Local zen and az 
     &          GGeoSondAnglesSUN,   ! Solar zen and az 
     &          EARTH_SATELLITE_DISTANCE, ! Height 
     &          GS1cSpect)           ! IASI spectrum 

      
      implicit none
      integer*4   GEPSDatIasi_days(30)
      integer*4   GEPSDatIasi_milliseconds(30) !days from 1/1/2000, milliseconds
      character*1 GQisFlagQual(4,30)
      integer*4   GQisQualIndex_scale      
      integer*4   GQisQualIndex_value        
      integer*4   GQisQualIndexLoc_scale     
      integer*4   GQisQualIndexLoc_value    
      integer*4   GQisQualIndexRad_scale     
      integer*4   GQisQualIndexRad_value     
      integer*4   GQisQualIndexSpect_scale   
      integer*4   GQisQualIndexSpect_value   
      integer*4   GQisSysTecSondQual
      integer*4   GGeoSondLoc(2,4,30)      !longitude, latitude *1E6
      integer*4   GGeoSondAnglesMETOP(2,4,30)
      integer*4   GGeoSondAnglesSUN(2,4,30)
      integer*4   EARTH_SATELLITE_DISTANCE
      integer*2   GS1cSpect(8700,4,30)
      
#include "inparms.h"
#include "iasi1c.h"
#include "constants_preproc.h"
      
      integer     I,ichan,ispot,idet,iclass
      integer     avhchannel(6)
      real        avlat

      do ichan=1,6
        avhchannel(ichan) = 0   !initialise AVHRR channel indicator
      enddo

!Use time of first spot in the scan line. Approximation valid 2001 to 2099.
      iasi1c_scnlinyr = INT((GEPSDatIasi_days(1)-1)/365.25)
      iasi1c_scnlindy = GEPSDatIasi_days(1) - 
     &                  INT(iasi1c_scnlinyr*365.25)
      iasi1c_scnlinyr = iasi1c_scnlinyr + 2000
      do I=1,30
        iasi1c_scnlintime(I) = GEPSDatIasi_milliseconds(I)
      enddo
      iasi1c_granule = iasi1c_scnlintime(1)/180000     !3 minute granules
      
!Satellite height (km*10). Calculate from latitudes of 8 spots in mid scan.
      avlat = 0.0
      do idet=1,4
        do ispot=15,16
          avlat = avlat + GGeoSondLoc(2,idet,ispot)*0.125E-6
        enddo
      enddo
      iasi1c_scalti = 0.01*(EARTH_SATELLITE_DISTANCE - semiMajor 
     &   + semiDiff*sin(avlat*deg2rad)**2)

!Quality flags
      iasi1c_GQisQualIndex = 
     &     GQisQualIndex_value*10**(-GQisQualIndex_scale)*100  !percent
      iasi1c_GQisQualIndexLoc = 
     &     GQisQualIndexLoc_value*10**(-GQisQualIndexLoc_scale)*100
      iasi1c_GQisQualIndexRad = 
     &     GQisQualIndexRad_value*10**(-GQisQualIndexRad_scale)*100
      iasi1c_GQisQualIndexSpect = 
     &     GQisQualIndexSpect_scale*10**(-GQisQualIndexSpect_value)*100
      iasi1c_GQisSysTecSondQual = GQisSysTecSondQual
      
      do ispot=1,30
        do idet=1,4
          iasi1c_GQisFlagQual(idet,ispot) = 
     &                      ICHAR(GQisFlagQual(idet,ispot))

!Navigation
          do i=1,2
            iasi1c_latlon(i,idet,ispot) = 
     &                 GGeoSondLoc(3-i,idet,ispot)/100            !deg*10^4
            iasi1c_angles(i,idet,ispot) = 
     &                 GGeoSondAnglesMETOP(i,idet,ispot)/10000    !deg*10^2
            iasi1c_angles(i+2,idet,ispot) = 
     &                 GGeoSondAnglesSUN(i,idet,ispot)/10000      !deg*10^2
          enddo
          
!IASI spectrum
          do ichan=1,8700
            iasi1c_scalrad(ichan,idet,ispot) = 
     &                            GS1cSpect(ichan,idet,ispot)
          enddo
          
        enddo
      enddo
      iasi1c_avhcombination = 0
      
!Increment scan line and write data record

      iasi1c_h_scnlin = iasi1c_h_scnlin + 1
      iasi1c_scnlin = iasi1c_h_scnlin
      call ioi1c("WRITE DATA",10,iasi1c_h_scnlin+iasi1c_h_hdrcnt)

!Update header record

      if (iasi1c_h_startdatayr.eq.MDI) then
        iasi1c_h_startdatayr = iasi1c_scnlinyr
        iasi1c_h_startdatady = iasi1c_scnlindy
        do i = 1, 30
          if( iasi1c_scnlintime(i) .ne. 0 ) then
            iasi1c_h_startdatatime = iasi1c_scnlintime(i)
            goto 901
          endif
        enddo
 901    continue
C        iasi1c_h_satht = iasi1c_scalti
      endif
      iasi1c_h_enddatayr = iasi1c_scnlinyr
      iasi1c_h_enddatady = iasi1c_scnlindy
      do i = 30, 1, -1
        if( iasi1c_scnlintime(i) .ne. 0 ) then
          iasi1c_h_enddatatime = iasi1c_scnlintime(i)
          goto 902
        endif
      enddo
 902  continue
      return
      end

!------------------------------------------------------------------------------
      subroutine mphr(SPACECRAFT_ID, ORBIT_START, ORBIT_END, X_VELOCITY, 
     & Y_VELOCITY, Z_VELOCITY, SEMI_MAJOR_AXIS, ECCENTRICITY)
      implicit none
      character*3 SPACECRAFT_ID
      integer     ORBIT_START, ORBIT_END, X_VELOCITY, Y_VELOCITY, 
     &            Z_VELOCITY, SEMI_MAJOR_AXIS, ECCENTRICITY
      real velocity, period, height, eccen, mean_orbit

#include "iasi1c.h"
#include "constants_preproc.h"

      read(SPACECRAFT_ID,'(2X,I1)') iasi1c_h_satid
      iasi1c_h_wmosatid = iasi1c_h_satid + 2       !3=METOP-1, etc.
      iasi1c_h_startorbit = ORBIT_START
      iasi1c_h_endorbit = ORBIT_END
      velocity = sqrt((X_VELOCITY*1.0E-3)**2 + (Y_VELOCITY*1.0E-3)**2
     &              + (Z_VELOCITY*1.0E-3)**2)       !m/s
      eccen = ECCENTRICITY/1.0E6
      mean_orbit = SEMI_MAJOR_AXIS*(1-0.5*eccen)    !m
      IF (velocity.GT.5000.0) THEN                  !expect ~7500m/s
        period = 2*PI*mean_orbit/velocity           !period in seconds
      ELSE
        period = 101.0 * 60.0       !Default 101 minutes if velocity is missing
      ENDIF
      iasi1c_h_period = NINT(period)
      height = mean_orbit/1000.0 - rEarth           !km
      iasi1c_h_satht = NINT(height*10)              !0.1km
      return
      end

!------------------------------------------------------------------------------
      subroutine giadr(IDefScaleSondNbScale,
     &            IDefScaleSondNsfirst,
     &            IDefScaleSondNslast,
     &            IDefScaleSondScaleFactor)
     
      implicit none
      integer*2   IDefScaleSondNbScale
      integer*2   IDefScaleSondNsfirst(10)
      integer*2   IDefScaleSondNslast(10)
      integer*2   IDefScaleSondScaleFactor(10)
      
#include "iasi1c.h"

      integer     I
      
!The 1c spectra start at 645cm^-1, so skip 2580 points in the "raw" spectrum
!which starts at DC
      DO I=1,IDefScaleSondNbScale
        iasi1c_h_startchan(I) = IDefScaleSondNsfirst(I)-2580
        iasi1c_h_endchan(I) = IDefScaleSondNslast(I)-2580
        iasi1c_h_scalefactor(I) = IDefScaleSondScaleFactor(I)
      ENDDO
      DO I=IDefScaleSondNbScale+1,10
        iasi1c_h_startchan(I) = 0
        iasi1c_h_endchan(I) = 0
        iasi1c_h_scalefactor(I) = 0
      ENDDO
      
      return
      end
