/* 
! Name: $Id$
! Purpose: functions to read/write IASI PFS file
!
! Method: 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2008, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0                Original code
!  1.1      31/06/18  Add HDF5 I/O functions (P. Brunel)
!
! Code Description: C
!
! 
*/
#include "eps_iasil1c_9.0.h"
#ifdef HAS_LIBHDF5
#include <hdf5.h>
#endif
#include "eps_buffer.h"
#include "eps_grh.h"

int eps_iasil1c_9_0_check_subclass = 1;
int eps_iasil1c_9_0_check_subclass_version = 1;


int read_mphr( t_mphr *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    while( 1 ) {
      const char* label;
      int len;
      if( ! read_label( &label, &len, B ) ) break;
      if( ( len == 12 ) && ( ! strncmp( label, "PRODUCT_NAME", 12 ) ) ) {
        read_ascii_string( X->PRODUCT_NAME, B, 67 );
      } else if( ( len == 21 ) && ( ! strncmp( label, "PARENT_PRODUCT_NAME_1", 21 ) ) ) {
        read_ascii_string( X->PARENT_PRODUCT_NAME_1, B, 67 );
      } else if( ( len == 21 ) && ( ! strncmp( label, "PARENT_PRODUCT_NAME_2", 21 ) ) ) {
        read_ascii_string( X->PARENT_PRODUCT_NAME_2, B, 67 );
      } else if( ( len == 21 ) && ( ! strncmp( label, "PARENT_PRODUCT_NAME_3", 21 ) ) ) {
        read_ascii_string( X->PARENT_PRODUCT_NAME_3, B, 67 );
      } else if( ( len == 21 ) && ( ! strncmp( label, "PARENT_PRODUCT_NAME_4", 21 ) ) ) {
        read_ascii_string( X->PARENT_PRODUCT_NAME_4, B, 67 );
      } else if( ( len == 13 ) && ( ! strncmp( label, "INSTRUMENT_ID", 13 ) ) ) {
        read_ascii_string( X->INSTRUMENT_ID, B, 4 );
      } else if( ( len == 16 ) && ( ! strncmp( label, "INSTRUMENT_MODEL", 16 ) ) ) {
        read_ascii_string( X->INSTRUMENT_MODEL, B, 3 );
      } else if( ( len == 12 ) && ( ! strncmp( label, "PRODUCT_TYPE", 12 ) ) ) {
        read_ascii_string( X->PRODUCT_TYPE, B, 3 );
      } else if( ( len == 16 ) && ( ! strncmp( label, "PROCESSING_LEVEL", 16 ) ) ) {
        read_ascii_string( X->PROCESSING_LEVEL, B, 2 );
      } else if( ( len == 13 ) && ( ! strncmp( label, "SPACECRAFT_ID", 13 ) ) ) {
        read_ascii_string( X->SPACECRAFT_ID, B, 3 );
      } else if( ( len == 13 ) && ( ! strncmp( label, "SENSING_START", 13 ) ) ) {
        read_ascii_eps_gentime( &X->SENSING_START, B );
      } else if( ( len == 11 ) && ( ! strncmp( label, "SENSING_END", 11 ) ) ) {
        read_ascii_eps_gentime( &X->SENSING_END, B );
      } else if( ( len == 25 ) && ( ! strncmp( label, "SENSING_START_THEORETICAL", 25 ) ) ) {
        read_ascii_eps_gentime( &X->SENSING_START_THEORETICAL, B );
      } else if( ( len == 23 ) && ( ! strncmp( label, "SENSING_END_THEORETICAL", 23 ) ) ) {
        read_ascii_eps_gentime( &X->SENSING_END_THEORETICAL, B );
      } else if( ( len == 17 ) && ( ! strncmp( label, "PROCESSING_CENTRE", 17 ) ) ) {
        read_ascii_string( X->PROCESSING_CENTRE, B, 4 );
      } else if( ( len == 23 ) && ( ! strncmp( label, "PROCESSOR_MAJOR_VERSION", 23 ) ) ) {
        read_ascii_eps_uinteger( &X->PROCESSOR_MAJOR_VERSION, B, 5 );
      } else if( ( len == 23 ) && ( ! strncmp( label, "PROCESSOR_MINOR_VERSION", 23 ) ) ) {
        read_ascii_eps_uinteger( &X->PROCESSOR_MINOR_VERSION, B, 5 );
      } else if( ( len == 20 ) && ( ! strncmp( label, "FORMAT_MAJOR_VERSION", 20 ) ) ) {
        read_ascii_eps_uinteger( &X->FORMAT_MAJOR_VERSION, B, 5 );
      } else if( ( len == 20 ) && ( ! strncmp( label, "FORMAT_MINOR_VERSION", 20 ) ) ) {
        read_ascii_eps_uinteger( &X->FORMAT_MINOR_VERSION, B, 5 );
      } else if( ( len == 21 ) && ( ! strncmp( label, "PROCESSING_TIME_START", 21 ) ) ) {
        read_ascii_eps_gentime( &X->PROCESSING_TIME_START, B );
      } else if( ( len == 19 ) && ( ! strncmp( label, "PROCESSING_TIME_END", 19 ) ) ) {
        read_ascii_eps_gentime( &X->PROCESSING_TIME_END, B );
      } else if( ( len == 15 ) && ( ! strncmp( label, "PROCESSING_MODE", 15 ) ) ) {
        read_ascii_string( X->PROCESSING_MODE, B, 1 );
      } else if( ( len == 16 ) && ( ! strncmp( label, "DISPOSITION_MODE", 16 ) ) ) {
        read_ascii_string( X->DISPOSITION_MODE, B, 1 );
      } else if( ( len == 24 ) && ( ! strncmp( label, "RECEIVING_GROUND_STATION", 24 ) ) ) {
        read_ascii_string( X->RECEIVING_GROUND_STATION, B, 3 );
      } else if( ( len == 18 ) && ( ! strncmp( label, "RECEIVE_TIME_START", 18 ) ) ) {
        read_ascii_eps_gentime( &X->RECEIVE_TIME_START, B );
      } else if( ( len == 16 ) && ( ! strncmp( label, "RECEIVE_TIME_END", 16 ) ) ) {
        read_ascii_eps_gentime( &X->RECEIVE_TIME_END, B );
      } else if( ( len == 11 ) && ( ! strncmp( label, "ORBIT_START", 11 ) ) ) {
        read_ascii_eps_uinteger( &X->ORBIT_START, B, 5 );
      } else if( ( len == 9 ) && ( ! strncmp( label, "ORBIT_END", 9 ) ) ) {
        read_ascii_eps_uinteger( &X->ORBIT_END, B, 5 );
      } else if( ( len == 19 ) && ( ! strncmp( label, "ACTUAL_PRODUCT_SIZE", 19 ) ) ) {
        read_ascii_eps_uinteger( &X->ACTUAL_PRODUCT_SIZE, B, 11 );
      } else if( ( len == 17 ) && ( ! strncmp( label, "STATE_VECTOR_TIME", 17 ) ) ) {
        read_ascii_eps_longgentime( &X->STATE_VECTOR_TIME, B );
      } else if( ( len == 15 ) && ( ! strncmp( label, "SEMI_MAJOR_AXIS", 15 ) ) ) {
        read_ascii_eps_integer( &X->SEMI_MAJOR_AXIS, B, 11 );
      } else if( ( len == 12 ) && ( ! strncmp( label, "ECCENTRICITY", 12 ) ) ) {
        read_ascii_eps_integer( &X->ECCENTRICITY, B, 11 );
      } else if( ( len == 11 ) && ( ! strncmp( label, "INCLINATION", 11 ) ) ) {
        read_ascii_eps_integer( &X->INCLINATION, B, 11 );
      } else if( ( len == 16 ) && ( ! strncmp( label, "PERIGEE_ARGUMENT", 16 ) ) ) {
        read_ascii_eps_integer( &X->PERIGEE_ARGUMENT, B, 11 );
      } else if( ( len == 15 ) && ( ! strncmp( label, "RIGHT_ASCENSION", 15 ) ) ) {
        read_ascii_eps_integer( &X->RIGHT_ASCENSION, B, 11 );
      } else if( ( len == 12 ) && ( ! strncmp( label, "MEAN_ANOMALY", 12 ) ) ) {
        read_ascii_eps_integer( &X->MEAN_ANOMALY, B, 11 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "X_POSITION", 10 ) ) ) {
        read_ascii_eps_integer( &X->X_POSITION, B, 11 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "Y_POSITION", 10 ) ) ) {
        read_ascii_eps_integer( &X->Y_POSITION, B, 11 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "Z_POSITION", 10 ) ) ) {
        read_ascii_eps_integer( &X->Z_POSITION, B, 11 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "X_VELOCITY", 10 ) ) ) {
        read_ascii_eps_integer( &X->X_VELOCITY, B, 11 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "Y_VELOCITY", 10 ) ) ) {
        read_ascii_eps_integer( &X->Y_VELOCITY, B, 11 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "Z_VELOCITY", 10 ) ) ) {
        read_ascii_eps_integer( &X->Z_VELOCITY, B, 11 );
      } else if( ( len == 24 ) && ( ! strncmp( label, "EARTH_SUN_DISTANCE_RATIO", 24 ) ) ) {
        read_ascii_eps_integer( &X->EARTH_SUN_DISTANCE_RATIO, B, 11 );
      } else if( ( len == 25 ) && ( ! strncmp( label, "LOCATION_TOLERANCE_RADIAL", 25 ) ) ) {
        read_ascii_eps_integer( &X->LOCATION_TOLERANCE_RADIAL, B, 11 );
      } else if( ( len == 29 ) && ( ! strncmp( label, "LOCATION_TOLERANCE_CROSSTRACK", 29 ) ) ) {
        read_ascii_eps_integer( &X->LOCATION_TOLERANCE_CROSSTRACK, B, 11 );
      } else if( ( len == 29 ) && ( ! strncmp( label, "LOCATION_TOLERANCE_ALONGTRACK", 29 ) ) ) {
        read_ascii_eps_integer( &X->LOCATION_TOLERANCE_ALONGTRACK, B, 11 );
      } else if( ( len == 9 ) && ( ! strncmp( label, "YAW_ERROR", 9 ) ) ) {
        read_ascii_eps_integer( &X->YAW_ERROR, B, 11 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "ROLL_ERROR", 10 ) ) ) {
        read_ascii_eps_integer( &X->ROLL_ERROR, B, 11 );
      } else if( ( len == 11 ) && ( ! strncmp( label, "PITCH_ERROR", 11 ) ) ) {
        read_ascii_eps_integer( &X->PITCH_ERROR, B, 11 );
      } else if( ( len == 21 ) && ( ! strncmp( label, "SUBSAT_LATITUDE_START", 21 ) ) ) {
        read_ascii_eps_integer( &X->SUBSAT_LATITUDE_START, B, 11 );
      } else if( ( len == 22 ) && ( ! strncmp( label, "SUBSAT_LONGITUDE_START", 22 ) ) ) {
        read_ascii_eps_integer( &X->SUBSAT_LONGITUDE_START, B, 11 );
      } else if( ( len == 19 ) && ( ! strncmp( label, "SUBSAT_LATITUDE_END", 19 ) ) ) {
        read_ascii_eps_integer( &X->SUBSAT_LATITUDE_END, B, 11 );
      } else if( ( len == 20 ) && ( ! strncmp( label, "SUBSAT_LONGITUDE_END", 20 ) ) ) {
        read_ascii_eps_integer( &X->SUBSAT_LONGITUDE_END, B, 11 );
      } else if( ( len == 11 ) && ( ! strncmp( label, "LEAP_SECOND", 11 ) ) ) {
        read_ascii_eps_integer( &X->LEAP_SECOND, B, 2 );
      } else if( ( len == 15 ) && ( ! strncmp( label, "LEAP_SECOND_UTC", 15 ) ) ) {
        read_ascii_eps_gentime( &X->LEAP_SECOND_UTC, B );
      } else if( ( len == 13 ) && ( ! strncmp( label, "TOTAL_RECORDS", 13 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_RECORDS, B, 6 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "TOTAL_MPHR", 10 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_MPHR, B, 6 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "TOTAL_SPHR", 10 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_SPHR, B, 6 );
      } else if( ( len == 9 ) && ( ! strncmp( label, "TOTAL_IPR", 9 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_IPR, B, 6 );
      } else if( ( len == 11 ) && ( ! strncmp( label, "TOTAL_GEADR", 11 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_GEADR, B, 6 );
      } else if( ( len == 11 ) && ( ! strncmp( label, "TOTAL_GIADR", 11 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_GIADR, B, 6 );
      } else if( ( len == 11 ) && ( ! strncmp( label, "TOTAL_VEADR", 11 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_VEADR, B, 6 );
      } else if( ( len == 11 ) && ( ! strncmp( label, "TOTAL_VIADR", 11 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_VIADR, B, 6 );
      } else if( ( len == 9 ) && ( ! strncmp( label, "TOTAL_MDR", 9 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_MDR, B, 6 );
      } else if( ( len == 23 ) && ( ! strncmp( label, "COUNT_DEGRADED_INST_MDR", 23 ) ) ) {
        read_ascii_eps_uinteger( &X->COUNT_DEGRADED_INST_MDR, B, 6 );
      } else if( ( len == 23 ) && ( ! strncmp( label, "COUNT_DEGRADED_PROC_MDR", 23 ) ) ) {
        read_ascii_eps_uinteger( &X->COUNT_DEGRADED_PROC_MDR, B, 6 );
      } else if( ( len == 30 ) && ( ! strncmp( label, "COUNT_DEGRADED_INST_MDR_BLOCKS", 30 ) ) ) {
        read_ascii_eps_uinteger( &X->COUNT_DEGRADED_INST_MDR_BLOCKS, B, 6 );
      } else if( ( len == 30 ) && ( ! strncmp( label, "COUNT_DEGRADED_PROC_MDR_BLOCKS", 30 ) ) ) {
        read_ascii_eps_uinteger( &X->COUNT_DEGRADED_PROC_MDR_BLOCKS, B, 6 );
      } else if( ( len == 19 ) && ( ! strncmp( label, "DURATION_OF_PRODUCT", 19 ) ) ) {
        read_ascii_eps_uinteger( &X->DURATION_OF_PRODUCT, B, 8 );
      } else if( ( len == 28 ) && ( ! strncmp( label, "MILLISECONDS_OF_DATA_PRESENT", 28 ) ) ) {
        read_ascii_eps_uinteger( &X->MILLISECONDS_OF_DATA_PRESENT, B, 8 );
      } else if( ( len == 28 ) && ( ! strncmp( label, "MILLISECONDS_OF_DATA_MISSING", 28 ) ) ) {
        read_ascii_eps_uinteger( &X->MILLISECONDS_OF_DATA_MISSING, B, 8 );
      } else if( ( len == 17 ) && ( ! strncmp( label, "SUBSETTED_PRODUCT", 17 ) ) ) {
        read_ascii_eps_boolean( &X->SUBSETTED_PRODUCT, B, 1 );
      } else  {
        /* Should issue warning */
      }
      skip_line( B );
    }
  }
  return status_buffer( B );
}


int write_mphr( t_mphr *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    write_ascii_string( "PRODUCT_NAME", X->PRODUCT_NAME, B, 67 );
    write_ascii_string( "PARENT_PRODUCT_NAME_1", X->PARENT_PRODUCT_NAME_1, B, 67 );
    write_ascii_string( "PARENT_PRODUCT_NAME_2", X->PARENT_PRODUCT_NAME_2, B, 67 );
    write_ascii_string( "PARENT_PRODUCT_NAME_3", X->PARENT_PRODUCT_NAME_3, B, 67 );
    write_ascii_string( "PARENT_PRODUCT_NAME_4", X->PARENT_PRODUCT_NAME_4, B, 67 );
    write_ascii_string( "INSTRUMENT_ID", X->INSTRUMENT_ID, B, 4 );
    write_ascii_string( "INSTRUMENT_MODEL", X->INSTRUMENT_MODEL, B, 3 );
    write_ascii_string( "PRODUCT_TYPE", X->PRODUCT_TYPE, B, 3 );
    write_ascii_string( "PROCESSING_LEVEL", X->PROCESSING_LEVEL, B, 2 );
    write_ascii_string( "SPACECRAFT_ID", X->SPACECRAFT_ID, B, 3 );
    write_ascii_eps_gentime( "SENSING_START", &X->SENSING_START, B );
    write_ascii_eps_gentime( "SENSING_END", &X->SENSING_END, B );
    write_ascii_eps_gentime( "SENSING_START_THEORETICAL", &X->SENSING_START_THEORETICAL, B );
    write_ascii_eps_gentime( "SENSING_END_THEORETICAL", &X->SENSING_END_THEORETICAL, B );
    write_ascii_string( "PROCESSING_CENTRE", X->PROCESSING_CENTRE, B, 4 );
    write_ascii_eps_uinteger( "PROCESSOR_MAJOR_VERSION", &X->PROCESSOR_MAJOR_VERSION, B, 5 );
    write_ascii_eps_uinteger( "PROCESSOR_MINOR_VERSION", &X->PROCESSOR_MINOR_VERSION, B, 5 );
    write_ascii_eps_uinteger( "FORMAT_MAJOR_VERSION", &X->FORMAT_MAJOR_VERSION, B, 5 );
    write_ascii_eps_uinteger( "FORMAT_MINOR_VERSION", &X->FORMAT_MINOR_VERSION, B, 5 );
    write_ascii_eps_gentime( "PROCESSING_TIME_START", &X->PROCESSING_TIME_START, B );
    write_ascii_eps_gentime( "PROCESSING_TIME_END", &X->PROCESSING_TIME_END, B );
    write_ascii_string( "PROCESSING_MODE", X->PROCESSING_MODE, B, 1 );
    write_ascii_string( "DISPOSITION_MODE", X->DISPOSITION_MODE, B, 1 );
    write_ascii_string( "RECEIVING_GROUND_STATION", X->RECEIVING_GROUND_STATION, B, 3 );
    write_ascii_eps_gentime( "RECEIVE_TIME_START", &X->RECEIVE_TIME_START, B );
    write_ascii_eps_gentime( "RECEIVE_TIME_END", &X->RECEIVE_TIME_END, B );
    write_ascii_eps_uinteger( "ORBIT_START", &X->ORBIT_START, B, 5 );
    write_ascii_eps_uinteger( "ORBIT_END", &X->ORBIT_END, B, 5 );
    write_ascii_eps_uinteger( "ACTUAL_PRODUCT_SIZE", &X->ACTUAL_PRODUCT_SIZE, B, 11 );
    write_ascii_eps_longgentime( "STATE_VECTOR_TIME", &X->STATE_VECTOR_TIME, B );
    write_ascii_eps_integer( "SEMI_MAJOR_AXIS", &X->SEMI_MAJOR_AXIS, B, 11 );
    write_ascii_eps_integer( "ECCENTRICITY", &X->ECCENTRICITY, B, 11 );
    write_ascii_eps_integer( "INCLINATION", &X->INCLINATION, B, 11 );
    write_ascii_eps_integer( "PERIGEE_ARGUMENT", &X->PERIGEE_ARGUMENT, B, 11 );
    write_ascii_eps_integer( "RIGHT_ASCENSION", &X->RIGHT_ASCENSION, B, 11 );
    write_ascii_eps_integer( "MEAN_ANOMALY", &X->MEAN_ANOMALY, B, 11 );
    write_ascii_eps_integer( "X_POSITION", &X->X_POSITION, B, 11 );
    write_ascii_eps_integer( "Y_POSITION", &X->Y_POSITION, B, 11 );
    write_ascii_eps_integer( "Z_POSITION", &X->Z_POSITION, B, 11 );
    write_ascii_eps_integer( "X_VELOCITY", &X->X_VELOCITY, B, 11 );
    write_ascii_eps_integer( "Y_VELOCITY", &X->Y_VELOCITY, B, 11 );
    write_ascii_eps_integer( "Z_VELOCITY", &X->Z_VELOCITY, B, 11 );
    write_ascii_eps_integer( "EARTH_SUN_DISTANCE_RATIO", &X->EARTH_SUN_DISTANCE_RATIO, B, 11 );
    write_ascii_eps_integer( "LOCATION_TOLERANCE_RADIAL", &X->LOCATION_TOLERANCE_RADIAL, B, 11 );
    write_ascii_eps_integer( "LOCATION_TOLERANCE_CROSSTRACK", &X->LOCATION_TOLERANCE_CROSSTRACK, B, 11 );
    write_ascii_eps_integer( "LOCATION_TOLERANCE_ALONGTRACK", &X->LOCATION_TOLERANCE_ALONGTRACK, B, 11 );
    write_ascii_eps_integer( "YAW_ERROR", &X->YAW_ERROR, B, 11 );
    write_ascii_eps_integer( "ROLL_ERROR", &X->ROLL_ERROR, B, 11 );
    write_ascii_eps_integer( "PITCH_ERROR", &X->PITCH_ERROR, B, 11 );
    write_ascii_eps_integer( "SUBSAT_LATITUDE_START", &X->SUBSAT_LATITUDE_START, B, 11 );
    write_ascii_eps_integer( "SUBSAT_LONGITUDE_START", &X->SUBSAT_LONGITUDE_START, B, 11 );
    write_ascii_eps_integer( "SUBSAT_LATITUDE_END", &X->SUBSAT_LATITUDE_END, B, 11 );
    write_ascii_eps_integer( "SUBSAT_LONGITUDE_END", &X->SUBSAT_LONGITUDE_END, B, 11 );
    write_ascii_eps_integer( "LEAP_SECOND", &X->LEAP_SECOND, B, 2 );
    write_ascii_eps_gentime( "LEAP_SECOND_UTC", &X->LEAP_SECOND_UTC, B );
    write_ascii_eps_uinteger( "TOTAL_RECORDS", &X->TOTAL_RECORDS, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_MPHR", &X->TOTAL_MPHR, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_SPHR", &X->TOTAL_SPHR, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_IPR", &X->TOTAL_IPR, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_GEADR", &X->TOTAL_GEADR, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_GIADR", &X->TOTAL_GIADR, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_VEADR", &X->TOTAL_VEADR, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_VIADR", &X->TOTAL_VIADR, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_MDR", &X->TOTAL_MDR, B, 6 );
    write_ascii_eps_uinteger( "COUNT_DEGRADED_INST_MDR", &X->COUNT_DEGRADED_INST_MDR, B, 6 );
    write_ascii_eps_uinteger( "COUNT_DEGRADED_PROC_MDR", &X->COUNT_DEGRADED_PROC_MDR, B, 6 );
    write_ascii_eps_uinteger( "COUNT_DEGRADED_INST_MDR_BLOCKS", &X->COUNT_DEGRADED_INST_MDR_BLOCKS, B, 6 );
    write_ascii_eps_uinteger( "COUNT_DEGRADED_PROC_MDR_BLOCKS", &X->COUNT_DEGRADED_PROC_MDR_BLOCKS, B, 6 );
    write_ascii_eps_uinteger( "DURATION_OF_PRODUCT", &X->DURATION_OF_PRODUCT, B, 8 );
    write_ascii_eps_uinteger( "MILLISECONDS_OF_DATA_PRESENT", &X->MILLISECONDS_OF_DATA_PRESENT, B, 8 );
    write_ascii_eps_uinteger( "MILLISECONDS_OF_DATA_MISSING", &X->MILLISECONDS_OF_DATA_MISSING, B, 8 );
    write_ascii_eps_boolean( "SUBSETTED_PRODUCT", &X->SUBSETTED_PRODUCT, B, 1 );
  }
  return status_buffer( B );
}
#ifdef HAS_LIBHDF5
int write_mphr_hdf5( t_mphr *X, hid_t gid, int n ) {
  char        grp_name [5];
  hid_t       grp;
  herr_t      status;

  hsize_t dims[1];
  int i;
  for( i = 0; i < n; i++, X++ ) {
    dims[0]=1;
    write_epsh5_string(  X->PRODUCT_NAME, gid, 67, "PRODUCT_NAME");
    write_epsh5_string( X->PARENT_PRODUCT_NAME_1,gid, 67,  "PARENT_PRODUCT_NAME_1");
    write_epsh5_string( X->PARENT_PRODUCT_NAME_2,gid, 67,  "PARENT_PRODUCT_NAME_2");
    write_epsh5_string( X->PARENT_PRODUCT_NAME_3,gid, 67,  "PARENT_PRODUCT_NAME_3");
    write_epsh5_string( X->PARENT_PRODUCT_NAME_4,gid, 67,  "PARENT_PRODUCT_NAME_4");
    write_epsh5_string( X->INSTRUMENT_ID,gid, 4, "INSTRUMENT_ID");
    write_epsh5_string( X->INSTRUMENT_MODEL,gid, 3, "INSTRUMENT_MODEL");
    write_epsh5_string( X->PRODUCT_TYPE,gid, 3, "PRODUCT_TYPE");
    write_epsh5_string( X->PROCESSING_LEVEL,gid, 2, "PROCESSING_LEVEL");
    write_epsh5_string( X->SPACECRAFT_ID,gid, 3, "SPACECRAFT_ID");
    write_epsh5_gentime( &X->SENSING_START, gid,  1, dims, "SENSING_START" );
    write_epsh5_gentime( &X->SENSING_END, gid,  1, dims, "SENSING_END" );
    write_epsh5_gentime( &X->SENSING_START_THEORETICAL, gid,  1, dims, "SENSING_START_THEORETICAL" );
    write_epsh5_gentime( &X->SENSING_END_THEORETICAL, gid,  1, dims, "SENSING_END_THEORETICAL" );
    write_epsh5_string( X->PROCESSING_CENTRE,gid, 4, "PROCESSING_CENTRE");
    write_epsh5_uinteger4(  &X->PROCESSOR_MAJOR_VERSION, gid,  1, dims ,"PROCESSOR_MAJOR_VERSION"  );
    write_epsh5_uinteger4(  &X->PROCESSOR_MINOR_VERSION, gid,  1, dims ,"PROCESSOR_MINOR_VERSION"  );
    write_epsh5_uinteger4(  &X->FORMAT_MAJOR_VERSION, gid,  1, dims ,"FORMAT_MAJOR_VERSION"  );
    write_epsh5_uinteger4(  &X->FORMAT_MINOR_VERSION, gid,  1, dims ,"FORMAT_MINOR_VERSION"  );
    write_epsh5_gentime(&X->PROCESSING_TIME_START, gid,  1, dims, "PROCESSING_TIME_START" );
    write_epsh5_gentime( &X->PROCESSING_TIME_END, gid,  1, dims, "PROCESSING_TIME_END" );
    write_epsh5_string( X->PROCESSING_MODE, gid, 1, "PROCESSING_MODE");
    write_epsh5_string( X->DISPOSITION_MODE, gid, 1, "DISPOSITION_MODE" );
    write_epsh5_string( X->RECEIVING_GROUND_STATION, gid, 3 , "RECEIVING_GROUND_STATION");
    write_epsh5_gentime( &X->RECEIVE_TIME_START, gid,  1, dims, "RECEIVE_TIME_START" );
    write_epsh5_gentime( &X->RECEIVE_TIME_END, gid,  1, dims, "RECEIVE_TIME_END" );
    write_epsh5_uinteger4(  &X->ORBIT_START, gid,  1, dims ,"ORBIT_START");
    write_epsh5_uinteger4(  &X->ORBIT_END, gid,  1, dims ,"ORBIT_END");
    write_epsh5_uinteger4(  &X->ACTUAL_PRODUCT_SIZE, gid,  1, dims ,"ACTUAL_PRODUCT_SIZE");
    write_epsh5_longgentime( &X->STATE_VECTOR_TIME, gid,  1, dims, "STATE_VECTOR_TIME" );
    write_epsh5_integer4(  &X->SEMI_MAJOR_AXIS, gid,  1, dims ,"SEMI_MAJOR_AXIS");
    write_epsh5_integer4(  &X->ECCENTRICITY, gid,  1, dims ,"ECCENTRICITY");
    write_epsh5_integer4(  &X->INCLINATION, gid,  1, dims ,"INCLINATION");
    write_epsh5_integer4(  &X->PERIGEE_ARGUMENT, gid,  1, dims ,"PERIGEE_ARGUMENT");
    write_epsh5_integer4(  &X->RIGHT_ASCENSION, gid,  1, dims ,"RIGHT_ASCENSION");
    write_epsh5_integer4(  &X->MEAN_ANOMALY, gid,  1, dims ,"MEAN_ANOMALY");
    write_epsh5_integer4(  &X->X_POSITION, gid,  1, dims ,"X_POSITION");
    write_epsh5_integer4(  &X->Y_POSITION, gid,  1, dims ,"Y_POSITION");
    write_epsh5_integer4(  &X->Z_POSITION, gid,  1, dims ,"Z_POSITION");
    write_epsh5_integer4(  &X->X_VELOCITY, gid,  1, dims ,"X_VELOCITY");
    write_epsh5_integer4(  &X->Y_VELOCITY, gid,  1, dims ,"Y_VELOCITY");
    write_epsh5_integer4(  &X->Z_VELOCITY, gid,  1, dims ,"Z_VELOCITY");
    write_epsh5_integer4(  &X->EARTH_SUN_DISTANCE_RATIO, gid,  1, dims ,"EARTH_SUN_DISTANCE_RATIO");
    write_epsh5_integer4(  &X->LOCATION_TOLERANCE_RADIAL, gid,  1, dims ,"LOCATION_TOLERANCE_RADIAL");
    write_epsh5_integer4(  &X->LOCATION_TOLERANCE_CROSSTRACK, gid,  1, dims ,"LOCATION_TOLERANCE_CROSSTRACK");
    write_epsh5_integer4(  &X->LOCATION_TOLERANCE_ALONGTRACK, gid,  1, dims ,"LOCATION_TOLERANCE_ALONGTRACK");
    write_epsh5_integer4(  &X->YAW_ERROR, gid,  1, dims ,"YAW_ERROR");
    write_epsh5_integer4(  &X->ROLL_ERROR, gid,  1, dims ,"ROLL_ERROR");
    write_epsh5_integer4(  &X->PITCH_ERROR, gid,  1, dims ,"PITCH_ERROR");
    write_epsh5_integer4(  &X->SUBSAT_LATITUDE_START, gid,  1, dims ,"SUBSAT_LATITUDE_START");
    write_epsh5_integer4(  &X->SUBSAT_LONGITUDE_START, gid,  1, dims ,"SUBSAT_LONGITUDE_START");
    write_epsh5_integer4(  &X->SUBSAT_LATITUDE_END, gid,  1, dims ,"SUBSAT_LATITUDE_END");
    write_epsh5_integer4(  &X->SUBSAT_LONGITUDE_END, gid,  1, dims ,"SUBSAT_LONGITUDE_END");
    write_epsh5_integer4(  &X->LEAP_SECOND, gid,  1, dims ,"LEAP_SECOND");
    write_epsh5_gentime( &X->LEAP_SECOND_UTC, gid,  1, dims, "LEAP_SECOND_UTC" );
    write_epsh5_uinteger4(  &X->TOTAL_RECORDS, gid,  1, dims ,"TOTAL_RECORDS");
    write_epsh5_uinteger4(  &X->TOTAL_MPHR, gid,  1, dims ,"TOTAL_MPHR");
    write_epsh5_uinteger4(  &X->TOTAL_SPHR, gid,  1, dims ,"TOTAL_SPHR");
    write_epsh5_uinteger4(  &X->TOTAL_IPR, gid,  1, dims ,"TOTAL_IPR");
    write_epsh5_uinteger4(  &X->TOTAL_GEADR, gid,  1, dims ,"TOTAL_GEADR");
    write_epsh5_uinteger4(  &X->TOTAL_GIADR, gid,  1, dims ,"TOTAL_GIADR");
    write_epsh5_uinteger4(  &X->TOTAL_VEADR, gid,  1, dims ,"TOTAL_VEADR");
    write_epsh5_uinteger4(  &X->TOTAL_VIADR, gid,  1, dims ,"TOTAL_VIADR");
    write_epsh5_uinteger4(  &X->TOTAL_MDR, gid,  1, dims ,"TOTAL_MDR");
    write_epsh5_uinteger4(  &X->COUNT_DEGRADED_INST_MDR, gid,  1, dims ,"COUNT_DEGRADED_INST_MDR");
    write_epsh5_uinteger4(  &X->COUNT_DEGRADED_PROC_MDR, gid,  1, dims ,"COUNT_DEGRADED_PROC_MDR");
    write_epsh5_uinteger4(  &X->COUNT_DEGRADED_INST_MDR_BLOCKS, gid,  1, dims ,"COUNT_DEGRADED_INST_MDR_BLOCKS");
    write_epsh5_uinteger4(  &X->COUNT_DEGRADED_PROC_MDR_BLOCKS, gid,  1, dims ,"COUNT_DEGRADED_PROC_MDR_BLOCKS");
    write_epsh5_uinteger4(  &X->DURATION_OF_PRODUCT, gid,  1, dims ,"DURATION_OF_PRODUCT");
    write_epsh5_uinteger4(  &X->MILLISECONDS_OF_DATA_PRESENT, gid,  1, dims ,"MILLISECONDS_OF_DATA_PRESENT");
    write_epsh5_uinteger4(  &X->MILLISECONDS_OF_DATA_MISSING, gid,  1, dims ,"MILLISECONDS_OF_DATA_MISSING");
    write_epsh5_boolean(  &X->SUBSETTED_PRODUCT, gid,  1, dims ,"SUBSETTED_PRODUCT");
  }
}
#endif

int free_mphr( t_mphr *X, int n ) {
}


int init_mphr( t_mphr *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    init_string( X->PRODUCT_NAME, 67 );
    init_string( X->PARENT_PRODUCT_NAME_1, 67 );
    init_string( X->PARENT_PRODUCT_NAME_2, 67 );
    init_string( X->PARENT_PRODUCT_NAME_3, 67 );
    init_string( X->PARENT_PRODUCT_NAME_4, 67 );
    init_string( X->INSTRUMENT_ID, 4 );
    init_string( X->INSTRUMENT_MODEL, 3 );
    init_string( X->PRODUCT_TYPE, 3 );
    init_string( X->PROCESSING_LEVEL, 2 );
    init_string( X->SPACECRAFT_ID, 3 );
    init_eps_gentime( &X->SENSING_START );
    init_eps_gentime( &X->SENSING_END );
    init_eps_gentime( &X->SENSING_START_THEORETICAL );
    init_eps_gentime( &X->SENSING_END_THEORETICAL );
    init_string( X->PROCESSING_CENTRE, 4 );
    init_eps_uinteger( &X->PROCESSOR_MAJOR_VERSION );
    init_eps_uinteger( &X->PROCESSOR_MINOR_VERSION );
    init_eps_uinteger( &X->FORMAT_MAJOR_VERSION );
    init_eps_uinteger( &X->FORMAT_MINOR_VERSION );
    init_eps_gentime( &X->PROCESSING_TIME_START );
    init_eps_gentime( &X->PROCESSING_TIME_END );
    init_string( X->PROCESSING_MODE, 1 );
    init_string( X->DISPOSITION_MODE, 1 );
    init_string( X->RECEIVING_GROUND_STATION, 3 );
    init_eps_gentime( &X->RECEIVE_TIME_START );
    init_eps_gentime( &X->RECEIVE_TIME_END );
    init_eps_uinteger( &X->ORBIT_START );
    init_eps_uinteger( &X->ORBIT_END );
    init_eps_uinteger( &X->ACTUAL_PRODUCT_SIZE );
    init_eps_longgentime( &X->STATE_VECTOR_TIME );
    init_eps_integer( &X->SEMI_MAJOR_AXIS );
    init_eps_integer( &X->ECCENTRICITY );
    init_eps_integer( &X->INCLINATION );
    init_eps_integer( &X->PERIGEE_ARGUMENT );
    init_eps_integer( &X->RIGHT_ASCENSION );
    init_eps_integer( &X->MEAN_ANOMALY );
    init_eps_integer( &X->X_POSITION );
    init_eps_integer( &X->Y_POSITION );
    init_eps_integer( &X->Z_POSITION );
    init_eps_integer( &X->X_VELOCITY );
    init_eps_integer( &X->Y_VELOCITY );
    init_eps_integer( &X->Z_VELOCITY );
    init_eps_integer( &X->EARTH_SUN_DISTANCE_RATIO );
    init_eps_integer( &X->LOCATION_TOLERANCE_RADIAL );
    init_eps_integer( &X->LOCATION_TOLERANCE_CROSSTRACK );
    init_eps_integer( &X->LOCATION_TOLERANCE_ALONGTRACK );
    init_eps_integer( &X->YAW_ERROR );
    init_eps_integer( &X->ROLL_ERROR );
    init_eps_integer( &X->PITCH_ERROR );
    init_eps_integer( &X->SUBSAT_LATITUDE_START );
    init_eps_integer( &X->SUBSAT_LONGITUDE_START );
    init_eps_integer( &X->SUBSAT_LATITUDE_END );
    init_eps_integer( &X->SUBSAT_LONGITUDE_END );
    init_eps_integer( &X->LEAP_SECOND );
    init_eps_gentime( &X->LEAP_SECOND_UTC );
    init_eps_uinteger( &X->TOTAL_RECORDS );
    init_eps_uinteger( &X->TOTAL_MPHR );
    init_eps_uinteger( &X->TOTAL_SPHR );
    init_eps_uinteger( &X->TOTAL_IPR );
    init_eps_uinteger( &X->TOTAL_GEADR );
    init_eps_uinteger( &X->TOTAL_GIADR );
    init_eps_uinteger( &X->TOTAL_VEADR );
    init_eps_uinteger( &X->TOTAL_VIADR );
    init_eps_uinteger( &X->TOTAL_MDR );
    init_eps_uinteger( &X->COUNT_DEGRADED_INST_MDR );
    init_eps_uinteger( &X->COUNT_DEGRADED_PROC_MDR );
    init_eps_uinteger( &X->COUNT_DEGRADED_INST_MDR_BLOCKS );
    init_eps_uinteger( &X->COUNT_DEGRADED_PROC_MDR_BLOCKS );
    init_eps_uinteger( &X->DURATION_OF_PRODUCT );
    init_eps_uinteger( &X->MILLISECONDS_OF_DATA_PRESENT );
    init_eps_uinteger( &X->MILLISECONDS_OF_DATA_MISSING );
    init_eps_boolean( &X->SUBSETTED_PRODUCT );
  }
}


int size_mphr( t_mphr *X ) {
  return 3287;
}


int scale_mphr( const t_mphr *X, t_s_mphr *Y, int n ) {
  int i;
  for( i = 0; i < n; i++, X++, Y++ ) {
    memcpy( Y->PRODUCT_NAME, X->PRODUCT_NAME, sizeof( X->PRODUCT_NAME ) );
    memcpy( Y->PARENT_PRODUCT_NAME_1, X->PARENT_PRODUCT_NAME_1, sizeof( X->PARENT_PRODUCT_NAME_1 ) );
    memcpy( Y->PARENT_PRODUCT_NAME_2, X->PARENT_PRODUCT_NAME_2, sizeof( X->PARENT_PRODUCT_NAME_2 ) );
    memcpy( Y->PARENT_PRODUCT_NAME_3, X->PARENT_PRODUCT_NAME_3, sizeof( X->PARENT_PRODUCT_NAME_3 ) );
    memcpy( Y->PARENT_PRODUCT_NAME_4, X->PARENT_PRODUCT_NAME_4, sizeof( X->PARENT_PRODUCT_NAME_4 ) );
    memcpy( Y->INSTRUMENT_ID, X->INSTRUMENT_ID, sizeof( X->INSTRUMENT_ID ) );
    memcpy( Y->INSTRUMENT_MODEL, X->INSTRUMENT_MODEL, sizeof( X->INSTRUMENT_MODEL ) );
    memcpy( Y->PRODUCT_TYPE, X->PRODUCT_TYPE, sizeof( X->PRODUCT_TYPE ) );
    memcpy( Y->PROCESSING_LEVEL, X->PROCESSING_LEVEL, sizeof( X->PROCESSING_LEVEL ) );
    memcpy( Y->SPACECRAFT_ID, X->SPACECRAFT_ID, sizeof( X->SPACECRAFT_ID ) );
    memcpy( &Y->SENSING_START, &X->SENSING_START, sizeof( X->SENSING_START ) );
    memcpy( &Y->SENSING_END, &X->SENSING_END, sizeof( X->SENSING_END ) );
    memcpy( &Y->SENSING_START_THEORETICAL, &X->SENSING_START_THEORETICAL, sizeof( X->SENSING_START_THEORETICAL ) );
    memcpy( &Y->SENSING_END_THEORETICAL, &X->SENSING_END_THEORETICAL, sizeof( X->SENSING_END_THEORETICAL ) );
    memcpy( Y->PROCESSING_CENTRE, X->PROCESSING_CENTRE, sizeof( X->PROCESSING_CENTRE ) );
    memcpy( &Y->PROCESSOR_MAJOR_VERSION, &X->PROCESSOR_MAJOR_VERSION, sizeof( X->PROCESSOR_MAJOR_VERSION ) );
    memcpy( &Y->PROCESSOR_MINOR_VERSION, &X->PROCESSOR_MINOR_VERSION, sizeof( X->PROCESSOR_MINOR_VERSION ) );
    memcpy( &Y->FORMAT_MAJOR_VERSION, &X->FORMAT_MAJOR_VERSION, sizeof( X->FORMAT_MAJOR_VERSION ) );
    memcpy( &Y->FORMAT_MINOR_VERSION, &X->FORMAT_MINOR_VERSION, sizeof( X->FORMAT_MINOR_VERSION ) );
    memcpy( &Y->PROCESSING_TIME_START, &X->PROCESSING_TIME_START, sizeof( X->PROCESSING_TIME_START ) );
    memcpy( &Y->PROCESSING_TIME_END, &X->PROCESSING_TIME_END, sizeof( X->PROCESSING_TIME_END ) );
    memcpy( Y->PROCESSING_MODE, X->PROCESSING_MODE, sizeof( X->PROCESSING_MODE ) );
    memcpy( Y->DISPOSITION_MODE, X->DISPOSITION_MODE, sizeof( X->DISPOSITION_MODE ) );
    memcpy( Y->RECEIVING_GROUND_STATION, X->RECEIVING_GROUND_STATION, sizeof( X->RECEIVING_GROUND_STATION ) );
    memcpy( &Y->RECEIVE_TIME_START, &X->RECEIVE_TIME_START, sizeof( X->RECEIVE_TIME_START ) );
    memcpy( &Y->RECEIVE_TIME_END, &X->RECEIVE_TIME_END, sizeof( X->RECEIVE_TIME_END ) );
    memcpy( &Y->ORBIT_START, &X->ORBIT_START, sizeof( X->ORBIT_START ) );
    memcpy( &Y->ORBIT_END, &X->ORBIT_END, sizeof( X->ORBIT_END ) );
    memcpy( &Y->ACTUAL_PRODUCT_SIZE, &X->ACTUAL_PRODUCT_SIZE, sizeof( X->ACTUAL_PRODUCT_SIZE ) );
    memcpy( &Y->STATE_VECTOR_TIME, &X->STATE_VECTOR_TIME, sizeof( X->STATE_VECTOR_TIME ) );
    memcpy( &Y->SEMI_MAJOR_AXIS, &X->SEMI_MAJOR_AXIS, sizeof( X->SEMI_MAJOR_AXIS ) );
    scale_eps_integer( &X->ECCENTRICITY, &Y->ECCENTRICITY, 1, 1.000000e+06 );
    scale_eps_integer( &X->INCLINATION, &Y->INCLINATION, 1, 1.000000e+03 );
    scale_eps_integer( &X->PERIGEE_ARGUMENT, &Y->PERIGEE_ARGUMENT, 1, 1.000000e+03 );
    scale_eps_integer( &X->RIGHT_ASCENSION, &Y->RIGHT_ASCENSION, 1, 1.000000e+03 );
    scale_eps_integer( &X->MEAN_ANOMALY, &Y->MEAN_ANOMALY, 1, 1.000000e+03 );
    scale_eps_integer( &X->X_POSITION, &Y->X_POSITION, 1, 1.000000e+03 );
    scale_eps_integer( &X->Y_POSITION, &Y->Y_POSITION, 1, 1.000000e+03 );
    scale_eps_integer( &X->Z_POSITION, &Y->Z_POSITION, 1, 1.000000e+03 );
    scale_eps_integer( &X->X_VELOCITY, &Y->X_VELOCITY, 1, 1.000000e+03 );
    scale_eps_integer( &X->Y_VELOCITY, &Y->Y_VELOCITY, 1, 1.000000e+03 );
    scale_eps_integer( &X->Z_VELOCITY, &Y->Z_VELOCITY, 1, 1.000000e+03 );
    memcpy( &Y->EARTH_SUN_DISTANCE_RATIO, &X->EARTH_SUN_DISTANCE_RATIO, sizeof( X->EARTH_SUN_DISTANCE_RATIO ) );
    memcpy( &Y->LOCATION_TOLERANCE_RADIAL, &X->LOCATION_TOLERANCE_RADIAL, sizeof( X->LOCATION_TOLERANCE_RADIAL ) );
    memcpy( &Y->LOCATION_TOLERANCE_CROSSTRACK, &X->LOCATION_TOLERANCE_CROSSTRACK, sizeof( X->LOCATION_TOLERANCE_CROSSTRACK ) );
    memcpy( &Y->LOCATION_TOLERANCE_ALONGTRACK, &X->LOCATION_TOLERANCE_ALONGTRACK, sizeof( X->LOCATION_TOLERANCE_ALONGTRACK ) );
    scale_eps_integer( &X->YAW_ERROR, &Y->YAW_ERROR, 1, 1.000000e+03 );
    scale_eps_integer( &X->ROLL_ERROR, &Y->ROLL_ERROR, 1, 1.000000e+03 );
    scale_eps_integer( &X->PITCH_ERROR, &Y->PITCH_ERROR, 1, 1.000000e+03 );
    scale_eps_integer( &X->SUBSAT_LATITUDE_START, &Y->SUBSAT_LATITUDE_START, 1, 1.000000e+03 );
    scale_eps_integer( &X->SUBSAT_LONGITUDE_START, &Y->SUBSAT_LONGITUDE_START, 1, 1.000000e+03 );
    scale_eps_integer( &X->SUBSAT_LATITUDE_END, &Y->SUBSAT_LATITUDE_END, 1, 1.000000e+03 );
    scale_eps_integer( &X->SUBSAT_LONGITUDE_END, &Y->SUBSAT_LONGITUDE_END, 1, 1.000000e+03 );
    memcpy( &Y->LEAP_SECOND, &X->LEAP_SECOND, sizeof( X->LEAP_SECOND ) );
    memcpy( &Y->LEAP_SECOND_UTC, &X->LEAP_SECOND_UTC, sizeof( X->LEAP_SECOND_UTC ) );
    memcpy( &Y->TOTAL_RECORDS, &X->TOTAL_RECORDS, sizeof( X->TOTAL_RECORDS ) );
    memcpy( &Y->TOTAL_MPHR, &X->TOTAL_MPHR, sizeof( X->TOTAL_MPHR ) );
    memcpy( &Y->TOTAL_SPHR, &X->TOTAL_SPHR, sizeof( X->TOTAL_SPHR ) );
    memcpy( &Y->TOTAL_IPR, &X->TOTAL_IPR, sizeof( X->TOTAL_IPR ) );
    memcpy( &Y->TOTAL_GEADR, &X->TOTAL_GEADR, sizeof( X->TOTAL_GEADR ) );
    memcpy( &Y->TOTAL_GIADR, &X->TOTAL_GIADR, sizeof( X->TOTAL_GIADR ) );
    memcpy( &Y->TOTAL_VEADR, &X->TOTAL_VEADR, sizeof( X->TOTAL_VEADR ) );
    memcpy( &Y->TOTAL_VIADR, &X->TOTAL_VIADR, sizeof( X->TOTAL_VIADR ) );
    memcpy( &Y->TOTAL_MDR, &X->TOTAL_MDR, sizeof( X->TOTAL_MDR ) );
    memcpy( &Y->COUNT_DEGRADED_INST_MDR, &X->COUNT_DEGRADED_INST_MDR, sizeof( X->COUNT_DEGRADED_INST_MDR ) );
    memcpy( &Y->COUNT_DEGRADED_PROC_MDR, &X->COUNT_DEGRADED_PROC_MDR, sizeof( X->COUNT_DEGRADED_PROC_MDR ) );
    memcpy( &Y->COUNT_DEGRADED_INST_MDR_BLOCKS, &X->COUNT_DEGRADED_INST_MDR_BLOCKS, sizeof( X->COUNT_DEGRADED_INST_MDR_BLOCKS ) );
    memcpy( &Y->COUNT_DEGRADED_PROC_MDR_BLOCKS, &X->COUNT_DEGRADED_PROC_MDR_BLOCKS, sizeof( X->COUNT_DEGRADED_PROC_MDR_BLOCKS ) );
    memcpy( &Y->DURATION_OF_PRODUCT, &X->DURATION_OF_PRODUCT, sizeof( X->DURATION_OF_PRODUCT ) );
    memcpy( &Y->MILLISECONDS_OF_DATA_PRESENT, &X->MILLISECONDS_OF_DATA_PRESENT, sizeof( X->MILLISECONDS_OF_DATA_PRESENT ) );
    memcpy( &Y->MILLISECONDS_OF_DATA_MISSING, &X->MILLISECONDS_OF_DATA_MISSING, sizeof( X->MILLISECONDS_OF_DATA_MISSING ) );
    memcpy( &Y->SUBSETTED_PRODUCT, &X->SUBSETTED_PRODUCT, sizeof( X->SUBSETTED_PRODUCT ) );
  }
}


int print_mphr( const char* label, t_mphr *X, FILE *fp, int ndim, const int *dims ) {
  int i;
  if( ndim == 0 ) {
    fprintf( fp, label );
    fprintf( fp, "\n" );
    /* Product_Details */
    print_string( "PRODUCT_NAME", X->PRODUCT_NAME, fp, 67, 0, NULL );
    print_string( "PARENT_PRODUCT_NAME_1", X->PARENT_PRODUCT_NAME_1, fp, 67, 0, NULL );
    print_string( "PARENT_PRODUCT_NAME_2", X->PARENT_PRODUCT_NAME_2, fp, 67, 0, NULL );
    print_string( "PARENT_PRODUCT_NAME_3", X->PARENT_PRODUCT_NAME_3, fp, 67, 0, NULL );
    print_string( "PARENT_PRODUCT_NAME_4", X->PARENT_PRODUCT_NAME_4, fp, 67, 0, NULL );
    print_string( "INSTRUMENT_ID", X->INSTRUMENT_ID, fp, 4, 0, NULL );
    print_string( "INSTRUMENT_MODEL", X->INSTRUMENT_MODEL, fp, 3, 0, NULL );
    print_string( "PRODUCT_TYPE", X->PRODUCT_TYPE, fp, 3, 0, NULL );
    print_string( "PROCESSING_LEVEL", X->PROCESSING_LEVEL, fp, 2, 0, NULL );
    print_string( "SPACECRAFT_ID", X->SPACECRAFT_ID, fp, 3, 0, NULL );
    print_eps_gentime( "SENSING_START", &X->SENSING_START, fp, 0, NULL );
    print_eps_gentime( "SENSING_END", &X->SENSING_END, fp, 0, NULL );
    print_eps_gentime( "SENSING_START_THEORETICAL", &X->SENSING_START_THEORETICAL, fp, 0, NULL );
    print_eps_gentime( "SENSING_END_THEORETICAL", &X->SENSING_END_THEORETICAL, fp, 0, NULL );
    print_string( "PROCESSING_CENTRE", X->PROCESSING_CENTRE, fp, 4, 0, NULL );
    print_eps_uinteger( "PROCESSOR_MAJOR_VERSION", &X->PROCESSOR_MAJOR_VERSION, fp, 0, NULL );
    print_eps_uinteger( "PROCESSOR_MINOR_VERSION", &X->PROCESSOR_MINOR_VERSION, fp, 0, NULL );
    print_eps_uinteger( "FORMAT_MAJOR_VERSION", &X->FORMAT_MAJOR_VERSION, fp, 0, NULL );
    print_eps_uinteger( "FORMAT_MINOR_VERSION", &X->FORMAT_MINOR_VERSION, fp, 0, NULL );
    print_eps_gentime( "PROCESSING_TIME_START", &X->PROCESSING_TIME_START, fp, 0, NULL );
    print_eps_gentime( "PROCESSING_TIME_END", &X->PROCESSING_TIME_END, fp, 0, NULL );
    print_string( "PROCESSING_MODE", X->PROCESSING_MODE, fp, 1, 0, NULL );
    print_string( "DISPOSITION_MODE", X->DISPOSITION_MODE, fp, 1, 0, NULL );
    print_string( "RECEIVING_GROUND_STATION", X->RECEIVING_GROUND_STATION, fp, 3, 0, NULL );
    print_eps_gentime( "RECEIVE_TIME_START", &X->RECEIVE_TIME_START, fp, 0, NULL );
    print_eps_gentime( "RECEIVE_TIME_END", &X->RECEIVE_TIME_END, fp, 0, NULL );
    print_eps_uinteger( "ORBIT_START", &X->ORBIT_START, fp, 0, NULL );
    print_eps_uinteger( "ORBIT_END", &X->ORBIT_END, fp, 0, NULL );
    print_eps_uinteger( "ACTUAL_PRODUCT_SIZE", &X->ACTUAL_PRODUCT_SIZE, fp, 0, NULL );
    /* ASCENDING_NODE_ORBIT_PARAMETERS */
    print_eps_longgentime( "STATE_VECTOR_TIME", &X->STATE_VECTOR_TIME, fp, 0, NULL );
    print_eps_integer( "SEMI_MAJOR_AXIS", &X->SEMI_MAJOR_AXIS, fp, 0, NULL );
    print_eps_integer( "ECCENTRICITY", &X->ECCENTRICITY, fp, 0, NULL );
    print_eps_integer( "INCLINATION", &X->INCLINATION, fp, 0, NULL );
    print_eps_integer( "PERIGEE_ARGUMENT", &X->PERIGEE_ARGUMENT, fp, 0, NULL );
    print_eps_integer( "RIGHT_ASCENSION", &X->RIGHT_ASCENSION, fp, 0, NULL );
    print_eps_integer( "MEAN_ANOMALY", &X->MEAN_ANOMALY, fp, 0, NULL );
    print_eps_integer( "X_POSITION", &X->X_POSITION, fp, 0, NULL );
    print_eps_integer( "Y_POSITION", &X->Y_POSITION, fp, 0, NULL );
    print_eps_integer( "Z_POSITION", &X->Z_POSITION, fp, 0, NULL );
    print_eps_integer( "X_VELOCITY", &X->X_VELOCITY, fp, 0, NULL );
    print_eps_integer( "Y_VELOCITY", &X->Y_VELOCITY, fp, 0, NULL );
    print_eps_integer( "Z_VELOCITY", &X->Z_VELOCITY, fp, 0, NULL );
    print_eps_integer( "EARTH_SUN_DISTANCE_RATIO", &X->EARTH_SUN_DISTANCE_RATIO, fp, 0, NULL );
    print_eps_integer( "LOCATION_TOLERANCE_RADIAL", &X->LOCATION_TOLERANCE_RADIAL, fp, 0, NULL );
    print_eps_integer( "LOCATION_TOLERANCE_CROSSTRACK", &X->LOCATION_TOLERANCE_CROSSTRACK, fp, 0, NULL );
    print_eps_integer( "LOCATION_TOLERANCE_ALONGTRACK", &X->LOCATION_TOLERANCE_ALONGTRACK, fp, 0, NULL );
    print_eps_integer( "YAW_ERROR", &X->YAW_ERROR, fp, 0, NULL );
    print_eps_integer( "ROLL_ERROR", &X->ROLL_ERROR, fp, 0, NULL );
    print_eps_integer( "PITCH_ERROR", &X->PITCH_ERROR, fp, 0, NULL );
    /* LOCATION_SUMMARY */
    print_eps_integer( "SUBSAT_LATITUDE_START", &X->SUBSAT_LATITUDE_START, fp, 0, NULL );
    print_eps_integer( "SUBSAT_LONGITUDE_START", &X->SUBSAT_LONGITUDE_START, fp, 0, NULL );
    print_eps_integer( "SUBSAT_LATITUDE_END", &X->SUBSAT_LATITUDE_END, fp, 0, NULL );
    print_eps_integer( "SUBSAT_LONGITUDE_END", &X->SUBSAT_LONGITUDE_END, fp, 0, NULL );
    /* Leap_Second_Information */
    print_eps_integer( "LEAP_SECOND", &X->LEAP_SECOND, fp, 0, NULL );
    print_eps_gentime( "LEAP_SECOND_UTC", &X->LEAP_SECOND_UTC, fp, 0, NULL );
    /* Record_counts */
    print_eps_uinteger( "TOTAL_RECORDS", &X->TOTAL_RECORDS, fp, 0, NULL );
    print_eps_uinteger( "TOTAL_MPHR", &X->TOTAL_MPHR, fp, 0, NULL );
    print_eps_uinteger( "TOTAL_SPHR", &X->TOTAL_SPHR, fp, 0, NULL );
    print_eps_uinteger( "TOTAL_IPR", &X->TOTAL_IPR, fp, 0, NULL );
    print_eps_uinteger( "TOTAL_GEADR", &X->TOTAL_GEADR, fp, 0, NULL );
    print_eps_uinteger( "TOTAL_GIADR", &X->TOTAL_GIADR, fp, 0, NULL );
    print_eps_uinteger( "TOTAL_VEADR", &X->TOTAL_VEADR, fp, 0, NULL );
    print_eps_uinteger( "TOTAL_VIADR", &X->TOTAL_VIADR, fp, 0, NULL );
    print_eps_uinteger( "TOTAL_MDR", &X->TOTAL_MDR, fp, 0, NULL );
    /* Record_Based_Generic_Quality_Flags */
    print_eps_uinteger( "COUNT_DEGRADED_INST_MDR", &X->COUNT_DEGRADED_INST_MDR, fp, 0, NULL );
    print_eps_uinteger( "COUNT_DEGRADED_PROC_MDR", &X->COUNT_DEGRADED_PROC_MDR, fp, 0, NULL );
    print_eps_uinteger( "COUNT_DEGRADED_INST_MDR_BLOCKS", &X->COUNT_DEGRADED_INST_MDR_BLOCKS, fp, 0, NULL );
    print_eps_uinteger( "COUNT_DEGRADED_PROC_MDR_BLOCKS", &X->COUNT_DEGRADED_PROC_MDR_BLOCKS, fp, 0, NULL );
    /* Time_Based_Generic_Quality_Flags */
    print_eps_uinteger( "DURATION_OF_PRODUCT", &X->DURATION_OF_PRODUCT, fp, 0, NULL );
    print_eps_uinteger( "MILLISECONDS_OF_DATA_PRESENT", &X->MILLISECONDS_OF_DATA_PRESENT, fp, 0, NULL );
    print_eps_uinteger( "MILLISECONDS_OF_DATA_MISSING", &X->MILLISECONDS_OF_DATA_MISSING, fp, 0, NULL );
    /* Regional_Product_Information */
    print_eps_boolean( "SUBSETTED_PRODUCT", &X->SUBSETTED_PRODUCT, fp, 0, NULL );
  } else {
    int k = 1;
    char label[16];
    for( i = 1; i < ndim; i++ ) {
      k *= dims[i];
    }
    for( i = 0; i < dims[0]; i++ ) {
      sprintf( label, "[%d]", i ); 
      print_mphr( label, X + i * k, fp, ndim-1, dims + 1 );
    }
  }
}



int read_ipr( t_ipr *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    read_eps_uinteger1( &X->TARGET_RECORD_CLASS, B, 1 );
    read_eps_uinteger1( &X->TARGET_INSTRUMENT_GROUP, B, 1 );
    read_eps_uinteger1( &X->TARGET_RECORD_SUBCLASS, B, 1 );
    read_eps_uinteger4( &X->TARGET_RECORD_OFFSET, B, 1 );
  }
  return status_buffer( B );
}


int write_ipr( t_ipr *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    write_eps_uinteger1( &X->TARGET_RECORD_CLASS, B, 1 );
    write_eps_uinteger1( &X->TARGET_INSTRUMENT_GROUP, B, 1 );
    write_eps_uinteger1( &X->TARGET_RECORD_SUBCLASS, B, 1 );
    write_eps_uinteger4( &X->TARGET_RECORD_OFFSET, B, 1 );
  }
  return status_buffer( B );
}


int free_ipr( t_ipr *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
  }
}


int init_ipr( t_ipr *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
  }
}


int size_ipr( t_ipr *X ) {
  int i;
  return 1 + 1 + 1 + 4;
}


int scale_ipr( const t_ipr *X, t_s_ipr *Y, int n ) {
  int i;
  for( i = 0; i < n; i++, X++, Y++ ) {
    memcpy( &Y->TARGET_RECORD_CLASS, &X->TARGET_RECORD_CLASS, sizeof( X->TARGET_RECORD_CLASS ) );
    memcpy( &Y->TARGET_INSTRUMENT_GROUP, &X->TARGET_INSTRUMENT_GROUP, sizeof( X->TARGET_INSTRUMENT_GROUP ) );
    memcpy( &Y->TARGET_RECORD_SUBCLASS, &X->TARGET_RECORD_SUBCLASS, sizeof( X->TARGET_RECORD_SUBCLASS ) );
    memcpy( &Y->TARGET_RECORD_OFFSET, &X->TARGET_RECORD_OFFSET, sizeof( X->TARGET_RECORD_OFFSET ) );
  }
}


int print_ipr( const char* label, t_ipr *X, FILE *fp, int ndim, const int *dims ) {
  int i;
  fprintf( fp, label );
  fprintf( fp, "\n" );
  if( ndim == 0 ) {
    print_eps_uinteger1( "TARGET_RECORD_CLASS", &X->TARGET_RECORD_CLASS, fp, 0, NULL );
    print_eps_uinteger1( "TARGET_INSTRUMENT_GROUP", &X->TARGET_INSTRUMENT_GROUP, fp, 0, NULL );
    print_eps_uinteger1( "TARGET_RECORD_SUBCLASS", &X->TARGET_RECORD_SUBCLASS, fp, 0, NULL );
    print_eps_uinteger4( "TARGET_RECORD_OFFSET", &X->TARGET_RECORD_OFFSET, fp, 0, NULL );
  } else {
    int k = 1;
    char label[16];
    for( i = 1; i < ndim; i++ ) {
      k *= dims[i];
    }
    for( i = 0; i < dims[0]; i++ ) {
      sprintf( label, "[%d]", i ); 
      print_ipr( label, X + i * k, fp, ndim-1, dims + 1 );
    }
  }
}


static const int dims_giadr_quality_IDefPsfSondNbLin[] = { PN };
static const int ndims_giadr_quality_IDefPsfSondNbLin = 1;
static const int dims_giadr_quality_IDefPsfSondNbCol[] = { PN };
static const int ndims_giadr_quality_IDefPsfSondNbCol = 1;
static const int dims_giadr_quality_IDefPsfSondY[] = { PN, 100 };
static const int ndims_giadr_quality_IDefPsfSondY = 2;
static const int dims_giadr_quality_IDefPsfSondZ[] = { PN, 100 };
static const int ndims_giadr_quality_IDefPsfSondZ = 2;
static const int dims_giadr_quality_IDefPsfSondWgt[] = { PN, 100, 100 };
static const int ndims_giadr_quality_IDefPsfSondWgt = 3;
static const int dims_giadr_quality_IDefllSSrf[] = { 100 };
static const int ndims_giadr_quality_IDefllSSrf = 1;
static const int dims_giadr_quality_IDefIISNeDT[] = { IMLI, IMCO };
static const int ndims_giadr_quality_IDefIISNeDT = 2;
static const int dims_giadr_quality_IDefDptIISDeadPix[] = { IMLI, IMCO };
static const int ndims_giadr_quality_IDefDptIISDeadPix = 2;

int read_giadr_quality( t_giadr_quality *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* QUALITY_PARAMETERS */
    read_eps_integer4( (eps_integer4*)X->IDefPsfSondNbLin, B, ( PN ) );
    read_eps_integer4( (eps_integer4*)X->IDefPsfSondNbCol, B, ( PN ) );
    read_eps_vinteger4( &X->IDefPsfSondOverSampFactor, B, 1 );
    read_eps_integer4( (eps_integer4*)X->IDefPsfSondY, B, ( PN )*( 100 ) );
    read_eps_integer4( (eps_integer4*)X->IDefPsfSondZ, B, ( PN )*( 100 ) );
    read_eps_vinteger4( (eps_vinteger4*)X->IDefPsfSondWgt, B, ( PN )*( 100 )*( 100 ) );
    read_eps_integer4( &X->IDefllSSrfNsfirst, B, 1 );
    read_eps_integer4( &X->IDefllSSrfNslast, B, 1 );
    read_eps_vinteger4( (eps_vinteger4*)X->IDefllSSrf, B, ( 100 ) );
    read_eps_vinteger4( &X->IDefllSSrfDWn, B, 1 );
    read_eps_vinteger4( (eps_vinteger4*)X->IDefIISNeDT, B, ( IMLI )*( IMCO ) );
    read_eps_boolean( (eps_boolean*)X->IDefDptIISDeadPix, B, ( IMLI )*( IMCO ) );
  }
  return status_buffer( B );
}


int write_giadr_quality( t_giadr_quality *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* QUALITY_PARAMETERS */
    write_eps_integer4( (eps_integer4*)X->IDefPsfSondNbLin, B, ( PN ) );
    write_eps_integer4( (eps_integer4*)X->IDefPsfSondNbCol, B, ( PN ) );
    write_eps_vinteger4( &X->IDefPsfSondOverSampFactor, B, 1 );
    write_eps_integer4( (eps_integer4*)X->IDefPsfSondY, B, ( PN )*( 100 ) );
    write_eps_integer4( (eps_integer4*)X->IDefPsfSondZ, B, ( PN )*( 100 ) );
    write_eps_vinteger4( (eps_vinteger4*)X->IDefPsfSondWgt, B, ( PN )*( 100 )*( 100 ) );
    write_eps_integer4( &X->IDefllSSrfNsfirst, B, 1 );
    write_eps_integer4( &X->IDefllSSrfNslast, B, 1 );
    write_eps_vinteger4( (eps_vinteger4*)X->IDefllSSrf, B, ( 100 ) );
    write_eps_vinteger4( &X->IDefllSSrfDWn, B, 1 );
    write_eps_vinteger4( (eps_vinteger4*)X->IDefIISNeDT, B, ( IMLI )*( IMCO ) );
    write_eps_boolean( (eps_boolean*)X->IDefDptIISDeadPix, B, ( IMLI )*( IMCO ) );
  }
  return status_buffer( B );
}


int free_giadr_quality( t_giadr_quality *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* QUALITY_PARAMETERS */
  }
}


int init_giadr_quality( t_giadr_quality *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* QUALITY_PARAMETERS */
  }
}


int size_giadr_quality( t_giadr_quality *X ) {
  int i;
  return ( PN ) * 4 + ( PN ) * 4 + 5 + ( PN )*( 100 ) * 4 + ( PN )*( 100 ) * 4 + ( PN )*( 100 )*( 100 ) * 5 + 4 + 4 + ( 100 ) * 5 + 5 + ( IMLI )*( IMCO ) * 5 + ( IMLI )*( IMCO ) * 1;
}


int scale_giadr_quality( const t_giadr_quality *X, t_s_giadr_quality *Y, int n ) {
  int i;
  for( i = 0; i < n; i++, X++, Y++ ) {
    memcpy( &Y->IDefPsfSondNbLin, &X->IDefPsfSondNbLin, sizeof( X->IDefPsfSondNbLin ) );
    memcpy( &Y->IDefPsfSondNbCol, &X->IDefPsfSondNbCol, sizeof( X->IDefPsfSondNbCol ) );
    scale_eps_vinteger4( &X->IDefPsfSondOverSampFactor, &Y->IDefPsfSondOverSampFactor, 1, 1 );
    scale_eps_integer4( (eps_integer4*)X->IDefPsfSondY, (eps_real4*)Y->IDefPsfSondY, ( PN )*( 100 ), 1.000000e+06 );
    scale_eps_integer4( (eps_integer4*)X->IDefPsfSondZ, (eps_real4*)Y->IDefPsfSondZ, ( PN )*( 100 ), 1.000000e+06 );
    scale_eps_vinteger4( (eps_vinteger4*)X->IDefPsfSondWgt, (eps_real4*)Y->IDefPsfSondWgt, ( PN )*( 100 )*( 100 ), 1 );
    memcpy( &Y->IDefllSSrfNsfirst, &X->IDefllSSrfNsfirst, sizeof( X->IDefllSSrfNsfirst ) );
    memcpy( &Y->IDefllSSrfNslast, &X->IDefllSSrfNslast, sizeof( X->IDefllSSrfNslast ) );
    scale_eps_vinteger4( (eps_vinteger4*)X->IDefllSSrf, (eps_real4*)Y->IDefllSSrf, ( 100 ), 1 );
    scale_eps_vinteger4( &X->IDefllSSrfDWn, &Y->IDefllSSrfDWn, 1, 1 );
    scale_eps_vinteger4( (eps_vinteger4*)X->IDefIISNeDT, (eps_real4*)Y->IDefIISNeDT, ( IMLI )*( IMCO ), 1 );
    memcpy( &Y->IDefDptIISDeadPix, &X->IDefDptIISDeadPix, sizeof( X->IDefDptIISDeadPix ) );
  }
}


int print_giadr_quality( const char* label, t_giadr_quality *X, FILE *fp, int ndim, const int *dims ) {
  int i;
  fprintf( fp, label );
  fprintf( fp, "\n" );
  if( ndim == 0 ) {
    /* QUALITY_PARAMETERS */
    print_eps_integer4( "IDefPsfSondNbLin", (eps_integer4*)X->IDefPsfSondNbLin, fp, ndims_giadr_quality_IDefPsfSondNbLin, dims_giadr_quality_IDefPsfSondNbLin );
    print_eps_integer4( "IDefPsfSondNbCol", (eps_integer4*)X->IDefPsfSondNbCol, fp, ndims_giadr_quality_IDefPsfSondNbCol, dims_giadr_quality_IDefPsfSondNbCol );
    print_eps_vinteger4( "IDefPsfSondOverSampFactor", &X->IDefPsfSondOverSampFactor, fp, 0, NULL );
    print_eps_integer4( "IDefPsfSondY", (eps_integer4*)X->IDefPsfSondY, fp, ndims_giadr_quality_IDefPsfSondY, dims_giadr_quality_IDefPsfSondY );
    print_eps_integer4( "IDefPsfSondZ", (eps_integer4*)X->IDefPsfSondZ, fp, ndims_giadr_quality_IDefPsfSondZ, dims_giadr_quality_IDefPsfSondZ );
    print_eps_vinteger4( "IDefPsfSondWgt", (eps_vinteger4*)X->IDefPsfSondWgt, fp, ndims_giadr_quality_IDefPsfSondWgt, dims_giadr_quality_IDefPsfSondWgt );
    print_eps_integer4( "IDefllSSrfNsfirst", &X->IDefllSSrfNsfirst, fp, 0, NULL );
    print_eps_integer4( "IDefllSSrfNslast", &X->IDefllSSrfNslast, fp, 0, NULL );
    print_eps_vinteger4( "IDefllSSrf", (eps_vinteger4*)X->IDefllSSrf, fp, ndims_giadr_quality_IDefllSSrf, dims_giadr_quality_IDefllSSrf );
    print_eps_vinteger4( "IDefllSSrfDWn", &X->IDefllSSrfDWn, fp, 0, NULL );
    print_eps_vinteger4( "IDefIISNeDT", (eps_vinteger4*)X->IDefIISNeDT, fp, ndims_giadr_quality_IDefIISNeDT, dims_giadr_quality_IDefIISNeDT );
    print_eps_boolean( "IDefDptIISDeadPix", (eps_boolean*)X->IDefDptIISDeadPix, fp, ndims_giadr_quality_IDefDptIISDeadPix, dims_giadr_quality_IDefDptIISDeadPix );
  } else {
    int k = 1;
    char label[16];
    for( i = 1; i < ndim; i++ ) {
      k *= dims[i];
    }
    for( i = 0; i < dims[0]; i++ ) {
      sprintf( label, "[%d]", i ); 
      print_giadr_quality( label, X + i * k, fp, ndim-1, dims + 1 );
    }
  }
}


static const int dims_giadr_scalefactors_IDefScaleSondNsfirst[] = { 10 };
static const int ndims_giadr_scalefactors_IDefScaleSondNsfirst = 1;
static const int dims_giadr_scalefactors_IDefScaleSondNslast[] = { 10 };
static const int ndims_giadr_scalefactors_IDefScaleSondNslast = 1;
static const int dims_giadr_scalefactors_IDefScaleSondScaleFactor[] = { 10 };
static const int ndims_giadr_scalefactors_IDefScaleSondScaleFactor = 1;

int read_giadr_scalefactors( t_giadr_scalefactors *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* Scale_Factors */
    read_eps_integer2( &X->IDefScaleSondNbScale, B, 1 );
    read_eps_integer2( (eps_integer2*)X->IDefScaleSondNsfirst, B, ( 10 ) );
    read_eps_integer2( (eps_integer2*)X->IDefScaleSondNslast, B, ( 10 ) );
    read_eps_integer2( (eps_integer2*)X->IDefScaleSondScaleFactor, B, ( 10 ) );
    read_eps_integer2( &X->IDefScaleIISScaleFactor, B, 1 );
  }
  return status_buffer( B );
}


int write_giadr_scalefactors( t_giadr_scalefactors *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* Scale_Factors */
    write_eps_integer2( &X->IDefScaleSondNbScale, B, 1 );
    write_eps_integer2( (eps_integer2*)X->IDefScaleSondNsfirst, B, ( 10 ) );
    write_eps_integer2( (eps_integer2*)X->IDefScaleSondNslast, B, ( 10 ) );
    write_eps_integer2( (eps_integer2*)X->IDefScaleSondScaleFactor, B, ( 10 ) );
    write_eps_integer2( &X->IDefScaleIISScaleFactor, B, 1 );
  }
  return status_buffer( B );
}


int free_giadr_scalefactors( t_giadr_scalefactors *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* Scale_Factors */
  }
}


int init_giadr_scalefactors( t_giadr_scalefactors *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* Scale_Factors */
  }
}


int size_giadr_scalefactors( t_giadr_scalefactors *X ) {
  int i;
  return 2 + ( 10 ) * 2 + ( 10 ) * 2 + ( 10 ) * 2 + 2;
}


int scale_giadr_scalefactors( const t_giadr_scalefactors *X, t_s_giadr_scalefactors *Y, int n ) {
  int i;
  for( i = 0; i < n; i++, X++, Y++ ) {
    memcpy( &Y->IDefScaleSondNbScale, &X->IDefScaleSondNbScale, sizeof( X->IDefScaleSondNbScale ) );
    memcpy( &Y->IDefScaleSondNsfirst, &X->IDefScaleSondNsfirst, sizeof( X->IDefScaleSondNsfirst ) );
    memcpy( &Y->IDefScaleSondNslast, &X->IDefScaleSondNslast, sizeof( X->IDefScaleSondNslast ) );
    memcpy( &Y->IDefScaleSondScaleFactor, &X->IDefScaleSondScaleFactor, sizeof( X->IDefScaleSondScaleFactor ) );
    memcpy( &Y->IDefScaleIISScaleFactor, &X->IDefScaleIISScaleFactor, sizeof( X->IDefScaleIISScaleFactor ) );
  }
}


int print_giadr_scalefactors( const char* label, t_giadr_scalefactors *X, FILE *fp, int ndim, const int *dims ) {
  int i;
  fprintf( fp, label );
  fprintf( fp, "\n" );
  if( ndim == 0 ) {
    /* Scale_Factors */
    print_eps_integer2( "IDefScaleSondNbScale", &X->IDefScaleSondNbScale, fp, 0, NULL );
    print_eps_integer2( "IDefScaleSondNsfirst", (eps_integer2*)X->IDefScaleSondNsfirst, fp, ndims_giadr_scalefactors_IDefScaleSondNsfirst, dims_giadr_scalefactors_IDefScaleSondNsfirst );
    print_eps_integer2( "IDefScaleSondNslast", (eps_integer2*)X->IDefScaleSondNslast, fp, ndims_giadr_scalefactors_IDefScaleSondNslast, dims_giadr_scalefactors_IDefScaleSondNslast );
    print_eps_integer2( "IDefScaleSondScaleFactor", (eps_integer2*)X->IDefScaleSondScaleFactor, fp, ndims_giadr_scalefactors_IDefScaleSondScaleFactor, dims_giadr_scalefactors_IDefScaleSondScaleFactor );
    print_eps_integer2( "IDefScaleIISScaleFactor", &X->IDefScaleIISScaleFactor, fp, 0, NULL );
  } else {
    int k = 1;
    char label[16];
    for( i = 1; i < ndim; i++ ) {
      k *= dims[i];
    }
    for( i = 0; i < dims[0]; i++ ) {
      sprintf( label, "[%d]", i ); 
      print_giadr_scalefactors( label, X + i * k, fp, ndim-1, dims + 1 );
    }
  }
}


static const int dims_mdr_1c_GEPSLocIasiAvhrr_IASI[] = { SNOT, PN, 2 };
static const int ndims_mdr_1c_GEPSLocIasiAvhrr_IASI = 3;
static const int dims_mdr_1c_GEPSLocIasiAvhrr_IIS[] = { SNOT, SGI, 2 };
static const int ndims_mdr_1c_GEPSLocIasiAvhrr_IIS = 3;
static const int dims_mdr_1c_OBT[] = { SNOT };
static const int ndims_mdr_1c_OBT = 1;
static const int dims_mdr_1c_OnboardUTC[] = { SNOT };
static const int ndims_mdr_1c_OnboardUTC = 1;
static const int dims_mdr_1c_GEPSDatIasi[] = { SNOT };
static const int ndims_mdr_1c_GEPSDatIasi = 1;
static const int dims_mdr_1c_GIsfLinOrigin[] = { CCD };
static const int ndims_mdr_1c_GIsfLinOrigin = 1;
static const int dims_mdr_1c_GIsfColOrigin[] = { CCD };
static const int ndims_mdr_1c_GIsfColOrigin = 1;
static const int dims_mdr_1c_GIsfPds1[] = { CCD };
static const int ndims_mdr_1c_GIsfPds1 = 1;
static const int dims_mdr_1c_GIsfPds2[] = { CCD };
static const int ndims_mdr_1c_GIsfPds2 = 1;
static const int dims_mdr_1c_GIsfPds3[] = { CCD };
static const int ndims_mdr_1c_GIsfPds3 = 1;
static const int dims_mdr_1c_GIsfPds4[] = { CCD };
static const int ndims_mdr_1c_GIsfPds4 = 1;
static const int dims_mdr_1c_GEPS_CCD[] = { SNOT };
static const int ndims_mdr_1c_GEPS_CCD = 1;
static const int dims_mdr_1c_GEPS_SP[] = { SNOT };
static const int ndims_mdr_1c_GEPS_SP = 1;
static const int dims_mdr_1c_GIrcImage[] = { SNOT, IMLI, IMCO };
static const int ndims_mdr_1c_GIrcImage = 3;
static const int dims_mdr_1c_GQisFlagQual[] = { SNOT, PN, SB };
static const int ndims_mdr_1c_GQisFlagQual = 3;
static const int dims_mdr_1c_GQisFlagQualDetailed[] = { SNOT, PN };
static const int ndims_mdr_1c_GQisFlagQualDetailed = 2;
static const int dims_mdr_1c_GGeoSondLoc[] = { SNOT, PN, 2 };
static const int ndims_mdr_1c_GGeoSondLoc = 3;
static const int dims_mdr_1c_GGeoSondAnglesMETOP[] = { SNOT, PN, 2 };
static const int ndims_mdr_1c_GGeoSondAnglesMETOP = 3;
static const int dims_mdr_1c_GGeoIISAnglesMETOP[] = { SNOT, SGI, 2 };
static const int ndims_mdr_1c_GGeoIISAnglesMETOP = 3;
static const int dims_mdr_1c_GGeoSondAnglesSUN[] = { SNOT, PN, 2 };
static const int ndims_mdr_1c_GGeoSondAnglesSUN = 3;
static const int dims_mdr_1c_GGeoIISAnglesSUN[] = { SNOT, SGI, 2 };
static const int ndims_mdr_1c_GGeoIISAnglesSUN = 3;
static const int dims_mdr_1c_GGeoIISLoc[] = { SNOT, SGI, 2 };
static const int ndims_mdr_1c_GGeoIISLoc = 3;
static const int dims_mdr_1c_GS1cSpect[] = { SNOT, PN, SS };
static const int ndims_mdr_1c_GS1cSpect = 3;
static const int dims_mdr_1c_IDefCovarMatEigenVal1c[] = { 100, CCD };
static const int ndims_mdr_1c_IDefCovarMatEigenVal1c = 2;
static const int dims_mdr_1c_IDefCcsChannelId[] = { NBK };
static const int ndims_mdr_1c_IDefCcsChannelId = 1;
static const int dims_mdr_1c_GCcsRadAnalNbClass[] = { SNOT, PN };
static const int ndims_mdr_1c_GCcsRadAnalNbClass = 2;
static const int dims_mdr_1c_GCcsRadAnalWgt[] = { SNOT, PN, NCL };
static const int ndims_mdr_1c_GCcsRadAnalWgt = 3;
static const int dims_mdr_1c_GCcsRadAnalY[] = { SNOT, PN, NCL };
static const int ndims_mdr_1c_GCcsRadAnalY = 3;
static const int dims_mdr_1c_GCcsRadAnalZ[] = { SNOT, PN, NCL };
static const int ndims_mdr_1c_GCcsRadAnalZ = 3;
static const int dims_mdr_1c_GCcsRadAnalMean[] = { SNOT, PN, NCL, NBK };
static const int ndims_mdr_1c_GCcsRadAnalMean = 4;
static const int dims_mdr_1c_GCcsRadAnalStd[] = { SNOT, PN, NCL, NBK };
static const int ndims_mdr_1c_GCcsRadAnalStd = 4;
static const int dims_mdr_1c_GCcsImageClassified[] = { SNOT, AMLI, AMCO };
static const int ndims_mdr_1c_GCcsImageClassified = 3;
static const int dims_mdr_1c_GCcsImageClassifiedNbLin[] = { SNOT };
static const int ndims_mdr_1c_GCcsImageClassifiedNbLin = 1;
static const int dims_mdr_1c_GCcsImageClassifiedNbCol[] = { SNOT };
static const int ndims_mdr_1c_GCcsImageClassifiedNbCol = 1;
static const int dims_mdr_1c_GCcsImageClassifiedFirstLin[] = { SNOT };
static const int ndims_mdr_1c_GCcsImageClassifiedFirstLin = 1;
static const int dims_mdr_1c_GCcsImageClassifiedFirstCol[] = { SNOT };
static const int ndims_mdr_1c_GCcsImageClassifiedFirstCol = 1;
static const int dims_mdr_1c_GCcsRadAnalType[] = { SNOT, NCL };
static const int ndims_mdr_1c_GCcsRadAnalType = 2;
static const int dims_mdr_1c_GIacVarImagIIS[] = { SNOT };
static const int ndims_mdr_1c_GIacVarImagIIS = 1;
static const int dims_mdr_1c_GIacAvgImagIIS[] = { SNOT };
static const int ndims_mdr_1c_GIacAvgImagIIS = 1;
static const int dims_mdr_1c_GEUMAvhrr1BCldFrac[] = { SNOT, PN };
static const int ndims_mdr_1c_GEUMAvhrr1BCldFrac = 2;
static const int dims_mdr_1c_GEUMAvhrr1BLandFrac[] = { SNOT, PN };
static const int ndims_mdr_1c_GEUMAvhrr1BLandFrac = 2;
static const int dims_mdr_1c_GEUMAvhrr1BQual[] = { SNOT, PN };
static const int ndims_mdr_1c_GEUMAvhrr1BQual = 2;

int read_mdr_1c( t_mdr_1c *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* GENERIC_QUALITY_INDICATORS */
    read_eps_boolean( &X->DEGRADED_INST_MDR, B, 1 );
    read_eps_boolean( &X->DEGRADED_PROC_MDR, B, 1 );
    /* Level_1_Data */
    read_eps_byte( X->GEPSIasiMode, B, 4 );
    read_eps_byte( X->GEPSOPSProcessingMode, B, 4 );
    read_eps_byte( X->GEPSIdConf, B, 32 );
    read_eps_vinteger4( (eps_vinteger4*)X->GEPSLocIasiAvhrr_IASI, B, ( SNOT )*( PN )*( 2 ) );
    read_eps_vinteger4( (eps_vinteger4*)X->GEPSLocIasiAvhrr_IIS, B, ( SNOT )*( SGI )*( 2 ) );
    read_eps_byte( X->OBT, B, 6 * ( SNOT ) );
    read_eps_ccsdstime( (eps_ccsdstime*)X->OnboardUTC, B, ( SNOT ) );
    read_eps_ccsdstime( (eps_ccsdstime*)X->GEPSDatIasi, B, ( SNOT ) );
    read_eps_integer4( (eps_integer4*)X->GIsfLinOrigin, B, ( CCD ) );
    read_eps_integer4( (eps_integer4*)X->GIsfColOrigin, B, ( CCD ) );
    read_eps_integer4( (eps_integer4*)X->GIsfPds1, B, ( CCD ) );
    read_eps_integer4( (eps_integer4*)X->GIsfPds2, B, ( CCD ) );
    read_eps_integer4( (eps_integer4*)X->GIsfPds3, B, ( CCD ) );
    read_eps_integer4( (eps_integer4*)X->GIsfPds4, B, ( CCD ) );
    read_eps_boolean( (eps_boolean*)X->GEPS_CCD, B, ( SNOT ) );
    read_eps_integer4( (eps_integer4*)X->GEPS_SP, B, ( SNOT ) );
    read_eps_uinteger2( (eps_uinteger2*)X->GIrcImage, B, ( SNOT )*( IMLI )*( IMCO ) );
    read_eps_boolean( (eps_boolean*)X->GQisFlagQual, B, ( SNOT )*( PN )*( SB ) );
    read_eps_byte( X->GQisFlagQualDetailed, B, 2 * ( SNOT )*( PN ) );
    read_eps_vinteger4( &X->GQisQualIndex, B, 1 );
    read_eps_vinteger4( &X->GQisQualIndexIIS, B, 1 );
    read_eps_vinteger4( &X->GQisQualIndexLoc, B, 1 );
    read_eps_vinteger4( &X->GQisQualIndexRad, B, 1 );
    read_eps_vinteger4( &X->GQisQualIndexSpect, B, 1 );
    read_eps_uinteger4( &X->GQisSysTecIISQual, B, 1 );
    read_eps_uinteger4( &X->GQisSysTecSondQual, B, 1 );
    read_eps_integer4( (eps_integer4*)X->GGeoSondLoc, B, ( SNOT )*( PN )*( 2 ) );
    read_eps_integer4( (eps_integer4*)X->GGeoSondAnglesMETOP, B, ( SNOT )*( PN )*( 2 ) );
    read_eps_integer4( (eps_integer4*)X->GGeoIISAnglesMETOP, B, ( SNOT )*( SGI )*( 2 ) );
    read_eps_integer4( (eps_integer4*)X->GGeoSondAnglesSUN, B, ( SNOT )*( PN )*( 2 ) );
    read_eps_integer4( (eps_integer4*)X->GGeoIISAnglesSUN, B, ( SNOT )*( SGI )*( 2 ) );
    read_eps_integer4( (eps_integer4*)X->GGeoIISLoc, B, ( SNOT )*( SGI )*( 2 ) );
    read_eps_uinteger4( &X->EARTH_SATELLITE_DISTANCE, B, 1 );
    /* Level_1c_Specific_Data */
    read_eps_vinteger4( &X->IDefSpectDWn1b, B, 1 );
    read_eps_integer4( &X->IDefNsfirst1b, B, 1 );
    read_eps_integer4( &X->IDefNslast1b, B, 1 );
    read_eps_integer2( (eps_integer2*)X->GS1cSpect, B, ( SNOT )*( PN )*( SS ) );
    read_eps_vinteger4( (eps_vinteger4*)X->IDefCovarMatEigenVal1c, B, ( 100 )*( CCD ) );
    read_eps_integer4( (eps_integer4*)X->IDefCcsChannelId, B, ( NBK ) );
    read_eps_integer4( (eps_integer4*)X->GCcsRadAnalNbClass, B, ( SNOT )*( PN ) );
    read_eps_vinteger4( (eps_vinteger4*)X->GCcsRadAnalWgt, B, ( SNOT )*( PN )*( NCL ) );
    read_eps_integer4( (eps_integer4*)X->GCcsRadAnalY, B, ( SNOT )*( PN )*( NCL ) );
    read_eps_integer4( (eps_integer4*)X->GCcsRadAnalZ, B, ( SNOT )*( PN )*( NCL ) );
    read_eps_vinteger4( (eps_vinteger4*)X->GCcsRadAnalMean, B, ( SNOT )*( PN )*( NCL )*( NBK ) );
    read_eps_vinteger4( (eps_vinteger4*)X->GCcsRadAnalStd, B, ( SNOT )*( PN )*( NCL )*( NBK ) );
    read_eps_uinteger1( (eps_uinteger1*)X->GCcsImageClassified, B, ( SNOT )*( AMLI )*( AMCO ) );
    read_eps_byte( X->IDefCcsMode, B, 4 );
    read_eps_integer2( (eps_integer2*)X->GCcsImageClassifiedNbLin, B, ( SNOT ) );
    read_eps_integer2( (eps_integer2*)X->GCcsImageClassifiedNbCol, B, ( SNOT ) );
    read_eps_vinteger4( (eps_vinteger4*)X->GCcsImageClassifiedFirstLin, B, ( SNOT ) );
    read_eps_vinteger4( (eps_vinteger4*)X->GCcsImageClassifiedFirstCol, B, ( SNOT ) );
    read_eps_boolean( (eps_boolean*)X->GCcsRadAnalType, B, ( SNOT )*( NCL ) );
    read_eps_vinteger4( (eps_vinteger4*)X->GIacVarImagIIS, B, ( SNOT ) );
    read_eps_vinteger4( (eps_vinteger4*)X->GIacAvgImagIIS, B, ( SNOT ) );
    read_eps_uinteger1( (eps_uinteger1*)X->GEUMAvhrr1BCldFrac, B, ( SNOT )*( PN ) );
    read_eps_uinteger1( (eps_uinteger1*)X->GEUMAvhrr1BLandFrac, B, ( SNOT )*( PN ) );
    read_eps_byte( X->GEUMAvhrr1BQual, B, 1 * ( SNOT )*( PN ) );
  }
  return status_buffer( B );
}


int write_mdr_1c( t_mdr_1c *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* GENERIC_QUALITY_INDICATORS */
    write_eps_boolean( &X->DEGRADED_INST_MDR, B, 1 );
    write_eps_boolean( &X->DEGRADED_PROC_MDR, B, 1 );
    /* Level_1_Data */
    write_eps_byte( X->GEPSIasiMode, B, 4 );
    write_eps_byte( X->GEPSOPSProcessingMode, B, 4 );
    write_eps_byte( X->GEPSIdConf, B, 32 );
    write_eps_vinteger4( (eps_vinteger4*)X->GEPSLocIasiAvhrr_IASI, B, ( SNOT )*( PN )*( 2 ) );
    write_eps_vinteger4( (eps_vinteger4*)X->GEPSLocIasiAvhrr_IIS, B, ( SNOT )*( SGI )*( 2 ) );
    write_eps_byte( X->OBT, B, 6 * ( SNOT ) );
    write_eps_ccsdstime( (eps_ccsdstime*)X->OnboardUTC, B, ( SNOT ) );
    write_eps_ccsdstime( (eps_ccsdstime*)X->GEPSDatIasi, B, ( SNOT ) );
    write_eps_integer4( (eps_integer4*)X->GIsfLinOrigin, B, ( CCD ) );
    write_eps_integer4( (eps_integer4*)X->GIsfColOrigin, B, ( CCD ) );
    write_eps_integer4( (eps_integer4*)X->GIsfPds1, B, ( CCD ) );
    write_eps_integer4( (eps_integer4*)X->GIsfPds2, B, ( CCD ) );
    write_eps_integer4( (eps_integer4*)X->GIsfPds3, B, ( CCD ) );
    write_eps_integer4( (eps_integer4*)X->GIsfPds4, B, ( CCD ) );
    write_eps_boolean( (eps_boolean*)X->GEPS_CCD, B, ( SNOT ) );
    write_eps_integer4( (eps_integer4*)X->GEPS_SP, B, ( SNOT ) );
    write_eps_uinteger2( (eps_uinteger2*)X->GIrcImage, B, ( SNOT )*( IMLI )*( IMCO ) );
    write_eps_boolean( (eps_boolean*)X->GQisFlagQual, B, ( SNOT )*( PN )*( SB ) );
    write_eps_byte( X->GQisFlagQualDetailed, B, 2 * ( SNOT )*( PN ) );
    write_eps_vinteger4( &X->GQisQualIndex, B, 1 );
    write_eps_vinteger4( &X->GQisQualIndexIIS, B, 1 );
    write_eps_vinteger4( &X->GQisQualIndexLoc, B, 1 );
    write_eps_vinteger4( &X->GQisQualIndexRad, B, 1 );
    write_eps_vinteger4( &X->GQisQualIndexSpect, B, 1 );
    write_eps_uinteger4( &X->GQisSysTecIISQual, B, 1 );
    write_eps_uinteger4( &X->GQisSysTecSondQual, B, 1 );
    write_eps_integer4( (eps_integer4*)X->GGeoSondLoc, B, ( SNOT )*( PN )*( 2 ) );
    write_eps_integer4( (eps_integer4*)X->GGeoSondAnglesMETOP, B, ( SNOT )*( PN )*( 2 ) );
    write_eps_integer4( (eps_integer4*)X->GGeoIISAnglesMETOP, B, ( SNOT )*( SGI )*( 2 ) );
    write_eps_integer4( (eps_integer4*)X->GGeoSondAnglesSUN, B, ( SNOT )*( PN )*( 2 ) );
    write_eps_integer4( (eps_integer4*)X->GGeoIISAnglesSUN, B, ( SNOT )*( SGI )*( 2 ) );
    write_eps_integer4( (eps_integer4*)X->GGeoIISLoc, B, ( SNOT )*( SGI )*( 2 ) );
    write_eps_uinteger4( &X->EARTH_SATELLITE_DISTANCE, B, 1 );
    /* Level_1c_Specific_Data */
    write_eps_vinteger4( &X->IDefSpectDWn1b, B, 1 );
    write_eps_integer4( &X->IDefNsfirst1b, B, 1 );
    write_eps_integer4( &X->IDefNslast1b, B, 1 );
    write_eps_integer2( (eps_integer2*)X->GS1cSpect, B, ( SNOT )*( PN )*( SS ) );
    write_eps_vinteger4( (eps_vinteger4*)X->IDefCovarMatEigenVal1c, B, ( 100 )*( CCD ) );
    write_eps_integer4( (eps_integer4*)X->IDefCcsChannelId, B, ( NBK ) );
    write_eps_integer4( (eps_integer4*)X->GCcsRadAnalNbClass, B, ( SNOT )*( PN ) );
    write_eps_vinteger4( (eps_vinteger4*)X->GCcsRadAnalWgt, B, ( SNOT )*( PN )*( NCL ) );
    write_eps_integer4( (eps_integer4*)X->GCcsRadAnalY, B, ( SNOT )*( PN )*( NCL ) );
    write_eps_integer4( (eps_integer4*)X->GCcsRadAnalZ, B, ( SNOT )*( PN )*( NCL ) );
    write_eps_vinteger4( (eps_vinteger4*)X->GCcsRadAnalMean, B, ( SNOT )*( PN )*( NCL )*( NBK ) );
    write_eps_vinteger4( (eps_vinteger4*)X->GCcsRadAnalStd, B, ( SNOT )*( PN )*( NCL )*( NBK ) );
    write_eps_uinteger1( (eps_uinteger1*)X->GCcsImageClassified, B, ( SNOT )*( AMLI )*( AMCO ) );
    write_eps_byte( X->IDefCcsMode, B, 4 );
    write_eps_integer2( (eps_integer2*)X->GCcsImageClassifiedNbLin, B, ( SNOT ) );
    write_eps_integer2( (eps_integer2*)X->GCcsImageClassifiedNbCol, B, ( SNOT ) );
    write_eps_vinteger4( (eps_vinteger4*)X->GCcsImageClassifiedFirstLin, B, ( SNOT ) );
    write_eps_vinteger4( (eps_vinteger4*)X->GCcsImageClassifiedFirstCol, B, ( SNOT ) );
    write_eps_boolean( (eps_boolean*)X->GCcsRadAnalType, B, ( SNOT )*( NCL ) );
    write_eps_vinteger4( (eps_vinteger4*)X->GIacVarImagIIS, B, ( SNOT ) );
    write_eps_vinteger4( (eps_vinteger4*)X->GIacAvgImagIIS, B, ( SNOT ) );
    write_eps_uinteger1( (eps_uinteger1*)X->GEUMAvhrr1BCldFrac, B, ( SNOT )*( PN ) );
    write_eps_uinteger1( (eps_uinteger1*)X->GEUMAvhrr1BLandFrac, B, ( SNOT )*( PN ) );
    write_eps_byte( X->GEUMAvhrr1BQual, B, 1 * ( SNOT )*( PN ) );
  }
  return status_buffer( B );
}
#ifdef HAS_LIBHDF5
int write_mdr_1c_hdf5( t_mdr_1c *X, hid_t gid, int n ) {
  char        grp_name [5];
  hid_t       grp;
  herr_t      status;

  /* on ecrit dans le groupe courant passe par file_id 
 * on ajoute le nom du dataset
 * 
 * */
  int i;
  hsize_t dims[4];
  for( i = 0; i < n; i++, X++ ) {

    /* GENERIC_QUALITY_INDICATORS */
    dims[0] = 1;
    write_epsh5_boolean( &X->DEGRADED_INST_MDR, gid, 1, dims , "DEGRADED_INST_MDR" );
    write_epsh5_boolean( &X->DEGRADED_PROC_MDR, gid, 1, dims , "DEGRADED_PROC_MDR" );

    /* Level_1_Data */
    dims[0] = 4;
    write_epsh5_byte( X->GEPSIasiMode, gid, 1 , dims ,"GEPSIasiMode");
    write_epsh5_byte( X->GEPSOPSProcessingMode, gid, 1, dims ,"GEPSOPSProcessingMode");
    dims[0] = 32;
    write_epsh5_byte( X->GEPSIdConf, gid, 1, dims ,"GEPSIdConf");
    dims[0] = 6;
    dims[1] = SNOT;
    write_epsh5_byte( X->OBT, gid, 2, dims ,"OBT");
    dims[0] = CCD;
    write_epsh5_integer4( (eps_integer4*)X->GIsfLinOrigin, gid,  1, dims ,"GIsfLinOrigin");
    write_epsh5_integer4( (eps_integer4*)X->GIsfColOrigin, gid,  1, dims ,"GIsfColOrigin" );
    write_epsh5_integer4( (eps_integer4*)X->GIsfPds1, gid,  1, dims ,"GIsfPds1" );
    write_epsh5_integer4( (eps_integer4*)X->GIsfPds2, gid,  1, dims ,"GIsfPds2" );
    write_epsh5_integer4( (eps_integer4*)X->GIsfPds3, gid,  1, dims ,"GIsfPds3" );
    write_epsh5_integer4( (eps_integer4*)X->GIsfPds4, gid,  1, dims ,"GIsfPds4" );
    dims[0] = SNOT;
    write_epsh5_boolean( (eps_boolean*)X->GEPS_CCD, gid,  1, dims ,"GEPS_CCD" );
    write_epsh5_integer4( (eps_integer4*)X->GEPS_SP, gid, 1, dims ,"GEPS_SP" );
    dims[0] = SNOT;
    dims[1] = IMLI;
    dims[2] = IMCO;
    write_epsh5_uinteger2( (eps_uinteger2*)X->GIrcImage, gid, 3, dims ,"GIrcImage");
    dims[0] = SNOT;
    dims[1] = PN;
    dims[2] = SB;
    write_epsh5_boolean( (eps_boolean*)X->GQisFlagQual, gid,  3, dims ,"GQisFlagQual");
    dims[0] = 2;
    dims[1] = SNOT; 
    dims[2] = PN; 
    write_epsh5_byte( X->GQisFlagQualDetailed, gid, 3, dims ,"GQisFlagQualDetailed" );
    dims[0] = SNOT; 
    dims[1] = PN; 
    dims[2] = 2; 
    write_epsh5_integer4( (eps_integer4*)X->GGeoSondLoc, gid,  3, dims ,"GGeoSondLoc" );
    write_epsh5_integer4( (eps_integer4*)X->GGeoSondAnglesMETOP, gid,  3, dims ,"GGeoSondAnglesMETOP"  );
    write_epsh5_integer4( (eps_integer4*)X->GGeoSondAnglesSUN, gid,  3, dims ,"GGeoSondAnglesSUN"  );
    dims[0] = SNOT; 
    dims[1] = SGI; 
    dims[2] = 2; 
    write_epsh5_integer4( (eps_integer4*)X->GGeoIISAnglesMETOP, gid,3, dims ,"GGeoIISAnglesMETOP" );
    write_epsh5_integer4( (eps_integer4*)X->GGeoIISAnglesSUN, gid, 3, dims ,"GGeoIISAnglesSUN");
    write_epsh5_integer4( (eps_integer4*)X->GGeoIISLoc, gid, 3, dims ,"GGeoIISLoc");
    dims[0] = 1; 
    write_epsh5_uinteger4( &X->EARTH_SATELLITE_DISTANCE, gid, 1, dims ,"EARTH_SATELLITE_DISTANCE" );
    write_epsh5_uinteger4( &X->GQisSysTecIISQual, gid, 1, dims ,"GQisSysTecIISQual" );
    write_epsh5_uinteger4( &X->GQisSysTecSondQual, gid, 1, dims ,"GQisSysTecSondQual" );

//typedef struct {
  //eps_byte scale;
  //eps_integer4 value;
//} eps_vinteger4;

  
    dims[0] = SNOT; 
    dims[1] = PN; 
    dims[2] = 2; 
    write_epsh5_vinteger4( (eps_vinteger4*)X->GEPSLocIasiAvhrr_IASI, gid, 3, dims, "GEPSLocIasiAvhrr_IASI" );
    dims[0] = SNOT; 
    dims[1] = SGI; 
    dims[2] = 2; 
    write_epsh5_vinteger4( (eps_vinteger4*)X->GEPSLocIasiAvhrr_IIS, gid,  3, dims, "GEPSLocIasiAvhrr_IIS" );
    dims[0] = SNOT; 
    write_epsh5_ccsdstime( (eps_ccsdstime*)X->OnboardUTC, gid, 1, dims, "OnboardUTC" );
    write_epsh5_ccsdstime( (eps_ccsdstime*)X->GEPSDatIasi, gid, 1, dims, "GEPSDatIasi" );
    dims[0] = 1; 
    write_epsh5_vinteger4( &X->GQisQualIndex, gid, 1, dims, "GQisQualIndex"  );
    write_epsh5_vinteger4( &X->GQisQualIndexIIS, gid, 1, dims, "GQisQualIndexIIS"  );
    write_epsh5_vinteger4( &X->GQisQualIndexLoc, gid, 1, dims, "GQisQualIndexLoc"  );
    write_epsh5_vinteger4( &X->GQisQualIndexRad, gid, 1, dims, "GQisQualIndexRad"  );
    write_epsh5_vinteger4( &X->GQisQualIndexSpect, gid, 1, dims, "GQisQualIndexSpect"  );


    /* Level_1c_Specific_Data */
    dims[0] = 1; 
    write_epsh5_integer4( &X->IDefNsfirst1b, gid, 1, dims ,"IDefNsfirst1b" );
    write_epsh5_integer4( &X->IDefNslast1b, gid, 1, dims ,"IDefNslast1b" );
    dims[0] = 4; 
    write_epsh5_byte( X->IDefCcsMode, gid, 1, dims ,"IDefCcsMode" );
    dims[0] =NBK;
    write_epsh5_integer4( (eps_integer4*)X->IDefCcsChannelId, gid, 1, dims ,"IDefCcsChannelId"  );
    dims[0] =SNOT;
    write_epsh5_integer2( (eps_integer2*)X->GCcsImageClassifiedNbLin, gid,1, dims ,"GCcsImageClassifiedNbLin" );
    write_epsh5_integer2( (eps_integer2*)X->GCcsImageClassifiedNbCol, gid,1, dims ,"GCcsImageClassifiedNbCol" );
    dims[0] =SNOT;
    dims[1] =NCL;
    write_epsh5_boolean( (eps_boolean*)X->GCcsRadAnalType, gid, 2, dims ,"GCcsRadAnalType" );
    dims[0] =SNOT;
    dims[1] =PN;
    write_epsh5_integer4( (eps_integer4*)X->GCcsRadAnalNbClass, gid, 2, dims ,"GCcsRadAnalNbClass" );
    write_epsh5_uinteger1( (eps_uinteger1*)X->GEUMAvhrr1BCldFrac, gid, 2, dims ,"GEUMAvhrr1BCldFrac" );
    write_epsh5_uinteger1( (eps_uinteger1*)X->GEUMAvhrr1BLandFrac, gid, 2, dims ,"GEUMAvhrr1BLandFrac" );
    dims[0] =SNOT;
    dims[1] =PN;
    dims[2] =SS;
    write_epsh5_integer2( (eps_integer2*)X->GS1cSpect, gid, 3, dims ,"GS1cSpect" );
    dims[0] =SNOT;
    dims[1] =PN;
    dims[2] =NCL;
    write_epsh5_integer4( (eps_integer4*)X->GCcsRadAnalY, gid, 3, dims ,"GCcsRadAnalY" );
    write_epsh5_integer4( (eps_integer4*)X->GCcsRadAnalZ, gid, 3, dims ,"GCcsRadAnalZ" );
    dims[0] =SNOT;
    dims[1] =AMLI;
    dims[2] =AMCO;
    write_epsh5_uinteger1( (eps_uinteger1*)X->GCcsImageClassified, gid, 3, dims ,"GCcsImageClassified" );
    dims[0] =1;
    dims[1] =SNOT;
    dims[2] =PN;
    write_epsh5_byte( X->GEUMAvhrr1BQual, gid, 3, dims ,"GEUMAvhrr1BQual" );
    dims[0] =1;
    write_epsh5_vinteger4( &X->IDefSpectDWn1b, gid, 1, dims ,"IDefSpectDWn1b" );
    dims[0] =100;
    dims[1] =CCD;
    write_epsh5_vinteger4( (eps_vinteger4*)X->IDefCovarMatEigenVal1c, gid, 2, dims ,"IDefCovarMatEigenVal1c"  );
    dims[0] =SNOT;
    dims[1] =PN;
    dims[2] =NCL;
    write_epsh5_vinteger4( (eps_vinteger4*)X->GCcsRadAnalWgt, gid, 2, dims ,"GCcsRadAnalWgt" );
    dims[0] =SNOT;
    dims[1] =PN;
    dims[2] =NCL;
    dims[3] =NBK;
    write_epsh5_vinteger4( (eps_vinteger4*)X->GCcsRadAnalMean, gid, 4, dims ,"GCcsRadAnalMean" );
    write_epsh5_vinteger4( (eps_vinteger4*)X->GCcsRadAnalStd, gid, 4, dims ,"GCcsRadAnalStd" );
    dims[0] =SNOT;
    write_epsh5_vinteger4( (eps_vinteger4*)X->GCcsImageClassifiedFirstLin, gid, 1, dims ,"GCcsImageClassifiedFirstLin"  );
    write_epsh5_vinteger4( (eps_vinteger4*)X->GCcsImageClassifiedFirstCol, gid, 1, dims ,"GCcsImageClassifiedFirstCol" );
    write_epsh5_vinteger4( (eps_vinteger4*)X->GIacVarImagIIS, gid, 1, dims ,"GIacVarImagIIS");
    write_epsh5_vinteger4( (eps_vinteger4*)X->GIacAvgImagIIS, gid, 1, dims ,"GIacAvgImagIIS" );
  }
}
#endif
int free_mdr_1c( t_mdr_1c *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* GENERIC_QUALITY_INDICATORS */
    /* Level_1_Data */
    /* Level_1c_Specific_Data */
  }
}


int init_mdr_1c( t_mdr_1c *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* GENERIC_QUALITY_INDICATORS */
    /* Level_1_Data */
    /* Level_1c_Specific_Data */
  }
}


int size_mdr_1c( t_mdr_1c *X ) {
  int i;
  return 1 + 1 + 4 + 4 + 32 + ( SNOT )*( PN )*( 2 ) * 5 + ( SNOT )*( SGI )*( 2 ) * 5 + ( SNOT ) * 6 + ( SNOT ) * 6 + ( SNOT ) * 6 + ( CCD ) * 4 + ( CCD ) * 4 + ( CCD ) * 4 + ( CCD ) * 4 + ( CCD ) * 4 + ( CCD ) * 4 + ( SNOT ) * 1 + ( SNOT ) * 4 + ( SNOT )*( IMLI )*( IMCO ) * 2 + ( SNOT )*( PN )*( SB ) * 1 + ( SNOT )*( PN ) * 2 + 5 + 5 + 5 + 5 + 5 + 4 + 4 + ( SNOT )*( PN )*( 2 ) * 4 + ( SNOT )*( PN )*( 2 ) * 4 + ( SNOT )*( SGI )*( 2 ) * 4 + ( SNOT )*( PN )*( 2 ) * 4 + ( SNOT )*( SGI )*( 2 ) * 4 + ( SNOT )*( SGI )*( 2 ) * 4 + 4 + 5 + 4 + 4 + ( SNOT )*( PN )*( SS ) * 2 + ( 100 )*( CCD ) * 5 + ( NBK ) * 4 + ( SNOT )*( PN ) * 4 + ( SNOT )*( PN )*( NCL ) * 5 + ( SNOT )*( PN )*( NCL ) * 4 + ( SNOT )*( PN )*( NCL ) * 4 + ( SNOT )*( PN )*( NCL )*( NBK ) * 5 + ( SNOT )*( PN )*( NCL )*( NBK ) * 5 + ( SNOT )*( AMLI )*( AMCO ) * 1 + 4 + ( SNOT ) * 2 + ( SNOT ) * 2 + ( SNOT ) * 5 + ( SNOT ) * 5 + ( SNOT )*( NCL ) * 1 + ( SNOT ) * 5 + ( SNOT ) * 5 + ( SNOT )*( PN ) * 1 + ( SNOT )*( PN ) * 1 + ( SNOT )*( PN ) * 1;
}


int scale_mdr_1c( const t_mdr_1c *X, t_s_mdr_1c *Y, int n ) {
  int i;
  for( i = 0; i < n; i++, X++, Y++ ) {
    memcpy( &Y->DEGRADED_INST_MDR, &X->DEGRADED_INST_MDR, sizeof( X->DEGRADED_INST_MDR ) );
    memcpy( &Y->DEGRADED_PROC_MDR, &X->DEGRADED_PROC_MDR, sizeof( X->DEGRADED_PROC_MDR ) );
    memcpy( &Y->GEPSIasiMode, &X->GEPSIasiMode, sizeof( X->GEPSIasiMode ) );
    memcpy( &Y->GEPSOPSProcessingMode, &X->GEPSOPSProcessingMode, sizeof( X->GEPSOPSProcessingMode ) );
    memcpy( &Y->GEPSIdConf, &X->GEPSIdConf, sizeof( X->GEPSIdConf ) );
    scale_eps_vinteger4( (eps_vinteger4*)X->GEPSLocIasiAvhrr_IASI, (eps_real4*)Y->GEPSLocIasiAvhrr_IASI, ( SNOT )*( PN )*( 2 ), 1 );
    scale_eps_vinteger4( (eps_vinteger4*)X->GEPSLocIasiAvhrr_IIS, (eps_real4*)Y->GEPSLocIasiAvhrr_IIS, ( SNOT )*( SGI )*( 2 ), 1 );
    memcpy( &Y->OBT, &X->OBT, sizeof( X->OBT ) );
    memcpy( &Y->OnboardUTC, &X->OnboardUTC, sizeof( X->OnboardUTC ) );
    memcpy( &Y->GEPSDatIasi, &X->GEPSDatIasi, sizeof( X->GEPSDatIasi ) );
    memcpy( &Y->GIsfLinOrigin, &X->GIsfLinOrigin, sizeof( X->GIsfLinOrigin ) );
    memcpy( &Y->GIsfColOrigin, &X->GIsfColOrigin, sizeof( X->GIsfColOrigin ) );
    scale_eps_integer4( (eps_integer4*)X->GIsfPds1, (eps_real4*)Y->GIsfPds1, ( CCD ), 1.000000e+06 );
    scale_eps_integer4( (eps_integer4*)X->GIsfPds2, (eps_real4*)Y->GIsfPds2, ( CCD ), 1.000000e+06 );
    scale_eps_integer4( (eps_integer4*)X->GIsfPds3, (eps_real4*)Y->GIsfPds3, ( CCD ), 1.000000e+06 );
    scale_eps_integer4( (eps_integer4*)X->GIsfPds4, (eps_real4*)Y->GIsfPds4, ( CCD ), 1.000000e+06 );
    memcpy( &Y->GEPS_CCD, &X->GEPS_CCD, sizeof( X->GEPS_CCD ) );
    memcpy( &Y->GEPS_SP, &X->GEPS_SP, sizeof( X->GEPS_SP ) );
    memcpy( &Y->GIrcImage, &X->GIrcImage, sizeof( X->GIrcImage ) );
    memcpy( &Y->GQisFlagQual, &X->GQisFlagQual, sizeof( X->GQisFlagQual ) );
    memcpy( &Y->GQisFlagQualDetailed, &X->GQisFlagQualDetailed, sizeof( X->GQisFlagQualDetailed ) );
    scale_eps_vinteger4( &X->GQisQualIndex, &Y->GQisQualIndex, 1, 1 );
    scale_eps_vinteger4( &X->GQisQualIndexIIS, &Y->GQisQualIndexIIS, 1, 1 );
    scale_eps_vinteger4( &X->GQisQualIndexLoc, &Y->GQisQualIndexLoc, 1, 1 );
    scale_eps_vinteger4( &X->GQisQualIndexRad, &Y->GQisQualIndexRad, 1, 1 );
    scale_eps_vinteger4( &X->GQisQualIndexSpect, &Y->GQisQualIndexSpect, 1, 1 );
    memcpy( &Y->GQisSysTecIISQual, &X->GQisSysTecIISQual, sizeof( X->GQisSysTecIISQual ) );
    memcpy( &Y->GQisSysTecSondQual, &X->GQisSysTecSondQual, sizeof( X->GQisSysTecSondQual ) );
    scale_eps_integer4( (eps_integer4*)X->GGeoSondLoc, (eps_real4*)Y->GGeoSondLoc, ( SNOT )*( PN )*( 2 ), 1.000000e+06 );
    scale_eps_integer4( (eps_integer4*)X->GGeoSondAnglesMETOP, (eps_real4*)Y->GGeoSondAnglesMETOP, ( SNOT )*( PN )*( 2 ), 1.000000e+06 );
    scale_eps_integer4( (eps_integer4*)X->GGeoIISAnglesMETOP, (eps_real4*)Y->GGeoIISAnglesMETOP, ( SNOT )*( SGI )*( 2 ), 1.000000e+06 );
    scale_eps_integer4( (eps_integer4*)X->GGeoSondAnglesSUN, (eps_real4*)Y->GGeoSondAnglesSUN, ( SNOT )*( PN )*( 2 ), 1.000000e+06 );
    scale_eps_integer4( (eps_integer4*)X->GGeoIISAnglesSUN, (eps_real4*)Y->GGeoIISAnglesSUN, ( SNOT )*( SGI )*( 2 ), 1.000000e+06 );
    scale_eps_integer4( (eps_integer4*)X->GGeoIISLoc, (eps_real4*)Y->GGeoIISLoc, ( SNOT )*( SGI )*( 2 ), 1.000000e+06 );
    memcpy( &Y->EARTH_SATELLITE_DISTANCE, &X->EARTH_SATELLITE_DISTANCE, sizeof( X->EARTH_SATELLITE_DISTANCE ) );
    scale_eps_vinteger4( &X->IDefSpectDWn1b, &Y->IDefSpectDWn1b, 1, 1 );
    memcpy( &Y->IDefNsfirst1b, &X->IDefNsfirst1b, sizeof( X->IDefNsfirst1b ) );
    memcpy( &Y->IDefNslast1b, &X->IDefNslast1b, sizeof( X->IDefNslast1b ) );
    memcpy( &Y->GS1cSpect, &X->GS1cSpect, sizeof( X->GS1cSpect ) );
    scale_eps_vinteger4( (eps_vinteger4*)X->IDefCovarMatEigenVal1c, (eps_real4*)Y->IDefCovarMatEigenVal1c, ( 100 )*( CCD ), 1 );
    memcpy( &Y->IDefCcsChannelId, &X->IDefCcsChannelId, sizeof( X->IDefCcsChannelId ) );
    memcpy( &Y->GCcsRadAnalNbClass, &X->GCcsRadAnalNbClass, sizeof( X->GCcsRadAnalNbClass ) );
    scale_eps_vinteger4( (eps_vinteger4*)X->GCcsRadAnalWgt, (eps_real4*)Y->GCcsRadAnalWgt, ( SNOT )*( PN )*( NCL ), 1 );
    scale_eps_integer4( (eps_integer4*)X->GCcsRadAnalY, (eps_real4*)Y->GCcsRadAnalY, ( SNOT )*( PN )*( NCL ), 1.000000e+06 );
    scale_eps_integer4( (eps_integer4*)X->GCcsRadAnalZ, (eps_real4*)Y->GCcsRadAnalZ, ( SNOT )*( PN )*( NCL ), 1.000000e+06 );
    scale_eps_vinteger4( (eps_vinteger4*)X->GCcsRadAnalMean, (eps_real4*)Y->GCcsRadAnalMean, ( SNOT )*( PN )*( NCL )*( NBK ), 1 );
    scale_eps_vinteger4( (eps_vinteger4*)X->GCcsRadAnalStd, (eps_real4*)Y->GCcsRadAnalStd, ( SNOT )*( PN )*( NCL )*( NBK ), 1 );
    memcpy( &Y->GCcsImageClassified, &X->GCcsImageClassified, sizeof( X->GCcsImageClassified ) );
    memcpy( &Y->IDefCcsMode, &X->IDefCcsMode, sizeof( X->IDefCcsMode ) );
    memcpy( &Y->GCcsImageClassifiedNbLin, &X->GCcsImageClassifiedNbLin, sizeof( X->GCcsImageClassifiedNbLin ) );
    memcpy( &Y->GCcsImageClassifiedNbCol, &X->GCcsImageClassifiedNbCol, sizeof( X->GCcsImageClassifiedNbCol ) );
    scale_eps_vinteger4( (eps_vinteger4*)X->GCcsImageClassifiedFirstLin, (eps_real4*)Y->GCcsImageClassifiedFirstLin, ( SNOT ), 1 );
    scale_eps_vinteger4( (eps_vinteger4*)X->GCcsImageClassifiedFirstCol, (eps_real4*)Y->GCcsImageClassifiedFirstCol, ( SNOT ), 1 );
    memcpy( &Y->GCcsRadAnalType, &X->GCcsRadAnalType, sizeof( X->GCcsRadAnalType ) );
    scale_eps_vinteger4( (eps_vinteger4*)X->GIacVarImagIIS, (eps_real4*)Y->GIacVarImagIIS, ( SNOT ), 1 );
    scale_eps_vinteger4( (eps_vinteger4*)X->GIacAvgImagIIS, (eps_real4*)Y->GIacAvgImagIIS, ( SNOT ), 1 );
    memcpy( &Y->GEUMAvhrr1BCldFrac, &X->GEUMAvhrr1BCldFrac, sizeof( X->GEUMAvhrr1BCldFrac ) );
    memcpy( &Y->GEUMAvhrr1BLandFrac, &X->GEUMAvhrr1BLandFrac, sizeof( X->GEUMAvhrr1BLandFrac ) );
    memcpy( &Y->GEUMAvhrr1BQual, &X->GEUMAvhrr1BQual, sizeof( X->GEUMAvhrr1BQual ) );
  }
}


int print_mdr_1c( const char* label, t_mdr_1c *X, FILE *fp, int ndim, const int *dims ) {
  int i;
  fprintf( fp, label );
  fprintf( fp, "\n" );
  if( ndim == 0 ) {
    /* GENERIC_QUALITY_INDICATORS */
    print_eps_boolean( "DEGRADED_INST_MDR", &X->DEGRADED_INST_MDR, fp, 0, NULL );
    print_eps_boolean( "DEGRADED_PROC_MDR", &X->DEGRADED_PROC_MDR, fp, 0, NULL );
    /* Level_1_Data */
    print_eps_bitfield( "GEPSIasiMode", X->GEPSIasiMode, fp, 32, 0, NULL );
    print_eps_bitfield( "GEPSOPSProcessingMode", X->GEPSOPSProcessingMode, fp, 32, 0, NULL );
    print_eps_bitfield( "GEPSIdConf", X->GEPSIdConf, fp, 256, 0, NULL );
    print_eps_vinteger4( "GEPSLocIasiAvhrr_IASI", (eps_vinteger4*)X->GEPSLocIasiAvhrr_IASI, fp, ndims_mdr_1c_GEPSLocIasiAvhrr_IASI, dims_mdr_1c_GEPSLocIasiAvhrr_IASI );
    print_eps_vinteger4( "GEPSLocIasiAvhrr_IIS", (eps_vinteger4*)X->GEPSLocIasiAvhrr_IIS, fp, ndims_mdr_1c_GEPSLocIasiAvhrr_IIS, dims_mdr_1c_GEPSLocIasiAvhrr_IIS );
    print_eps_bitfield( "OBT", (eps_byte*)X->OBT, fp, 48, ndims_mdr_1c_OBT, dims_mdr_1c_OBT );
    print_eps_ccsdstime( "OnboardUTC", (eps_ccsdstime*)X->OnboardUTC, fp, ndims_mdr_1c_OnboardUTC, dims_mdr_1c_OnboardUTC );
    print_eps_ccsdstime( "GEPSDatIasi", (eps_ccsdstime*)X->GEPSDatIasi, fp, ndims_mdr_1c_GEPSDatIasi, dims_mdr_1c_GEPSDatIasi );
    print_eps_integer4( "GIsfLinOrigin", (eps_integer4*)X->GIsfLinOrigin, fp, ndims_mdr_1c_GIsfLinOrigin, dims_mdr_1c_GIsfLinOrigin );
    print_eps_integer4( "GIsfColOrigin", (eps_integer4*)X->GIsfColOrigin, fp, ndims_mdr_1c_GIsfColOrigin, dims_mdr_1c_GIsfColOrigin );
    print_eps_integer4( "GIsfPds1", (eps_integer4*)X->GIsfPds1, fp, ndims_mdr_1c_GIsfPds1, dims_mdr_1c_GIsfPds1 );
    print_eps_integer4( "GIsfPds2", (eps_integer4*)X->GIsfPds2, fp, ndims_mdr_1c_GIsfPds2, dims_mdr_1c_GIsfPds2 );
    print_eps_integer4( "GIsfPds3", (eps_integer4*)X->GIsfPds3, fp, ndims_mdr_1c_GIsfPds3, dims_mdr_1c_GIsfPds3 );
    print_eps_integer4( "GIsfPds4", (eps_integer4*)X->GIsfPds4, fp, ndims_mdr_1c_GIsfPds4, dims_mdr_1c_GIsfPds4 );
    print_eps_boolean( "GEPS_CCD", (eps_boolean*)X->GEPS_CCD, fp, ndims_mdr_1c_GEPS_CCD, dims_mdr_1c_GEPS_CCD );
    print_eps_integer4( "GEPS_SP", (eps_integer4*)X->GEPS_SP, fp, ndims_mdr_1c_GEPS_SP, dims_mdr_1c_GEPS_SP );
    print_eps_uinteger2( "GIrcImage", (eps_uinteger2*)X->GIrcImage, fp, ndims_mdr_1c_GIrcImage, dims_mdr_1c_GIrcImage );
    print_eps_boolean( "GQisFlagQual", (eps_boolean*)X->GQisFlagQual, fp, ndims_mdr_1c_GQisFlagQual, dims_mdr_1c_GQisFlagQual );
    print_eps_bitfield( "GQisFlagQualDetailed", (eps_byte*)X->GQisFlagQualDetailed, fp, 16, ndims_mdr_1c_GQisFlagQualDetailed, dims_mdr_1c_GQisFlagQualDetailed );
    print_eps_vinteger4( "GQisQualIndex", &X->GQisQualIndex, fp, 0, NULL );
    print_eps_vinteger4( "GQisQualIndexIIS", &X->GQisQualIndexIIS, fp, 0, NULL );
    print_eps_vinteger4( "GQisQualIndexLoc", &X->GQisQualIndexLoc, fp, 0, NULL );
    print_eps_vinteger4( "GQisQualIndexRad", &X->GQisQualIndexRad, fp, 0, NULL );
    print_eps_vinteger4( "GQisQualIndexSpect", &X->GQisQualIndexSpect, fp, 0, NULL );
    print_eps_uinteger4( "GQisSysTecIISQual", &X->GQisSysTecIISQual, fp, 0, NULL );
    print_eps_uinteger4( "GQisSysTecSondQual", &X->GQisSysTecSondQual, fp, 0, NULL );
    print_eps_integer4( "GGeoSondLoc", (eps_integer4*)X->GGeoSondLoc, fp, ndims_mdr_1c_GGeoSondLoc, dims_mdr_1c_GGeoSondLoc );
    print_eps_integer4( "GGeoSondAnglesMETOP", (eps_integer4*)X->GGeoSondAnglesMETOP, fp, ndims_mdr_1c_GGeoSondAnglesMETOP, dims_mdr_1c_GGeoSondAnglesMETOP );
    print_eps_integer4( "GGeoIISAnglesMETOP", (eps_integer4*)X->GGeoIISAnglesMETOP, fp, ndims_mdr_1c_GGeoIISAnglesMETOP, dims_mdr_1c_GGeoIISAnglesMETOP );
    print_eps_integer4( "GGeoSondAnglesSUN", (eps_integer4*)X->GGeoSondAnglesSUN, fp, ndims_mdr_1c_GGeoSondAnglesSUN, dims_mdr_1c_GGeoSondAnglesSUN );
    print_eps_integer4( "GGeoIISAnglesSUN", (eps_integer4*)X->GGeoIISAnglesSUN, fp, ndims_mdr_1c_GGeoIISAnglesSUN, dims_mdr_1c_GGeoIISAnglesSUN );
    print_eps_integer4( "GGeoIISLoc", (eps_integer4*)X->GGeoIISLoc, fp, ndims_mdr_1c_GGeoIISLoc, dims_mdr_1c_GGeoIISLoc );
    print_eps_uinteger4( "EARTH_SATELLITE_DISTANCE", &X->EARTH_SATELLITE_DISTANCE, fp, 0, NULL );
    /* Level_1c_Specific_Data */
    print_eps_vinteger4( "IDefSpectDWn1b", &X->IDefSpectDWn1b, fp, 0, NULL );
    print_eps_integer4( "IDefNsfirst1b", &X->IDefNsfirst1b, fp, 0, NULL );
    print_eps_integer4( "IDefNslast1b", &X->IDefNslast1b, fp, 0, NULL );
    print_eps_integer2( "GS1cSpect", (eps_integer2*)X->GS1cSpect, fp, ndims_mdr_1c_GS1cSpect, dims_mdr_1c_GS1cSpect );
    print_eps_vinteger4( "IDefCovarMatEigenVal1c", (eps_vinteger4*)X->IDefCovarMatEigenVal1c, fp, ndims_mdr_1c_IDefCovarMatEigenVal1c, dims_mdr_1c_IDefCovarMatEigenVal1c );
    print_eps_integer4( "IDefCcsChannelId", (eps_integer4*)X->IDefCcsChannelId, fp, ndims_mdr_1c_IDefCcsChannelId, dims_mdr_1c_IDefCcsChannelId );
    print_eps_integer4( "GCcsRadAnalNbClass", (eps_integer4*)X->GCcsRadAnalNbClass, fp, ndims_mdr_1c_GCcsRadAnalNbClass, dims_mdr_1c_GCcsRadAnalNbClass );
    print_eps_vinteger4( "GCcsRadAnalWgt", (eps_vinteger4*)X->GCcsRadAnalWgt, fp, ndims_mdr_1c_GCcsRadAnalWgt, dims_mdr_1c_GCcsRadAnalWgt );
    print_eps_integer4( "GCcsRadAnalY", (eps_integer4*)X->GCcsRadAnalY, fp, ndims_mdr_1c_GCcsRadAnalY, dims_mdr_1c_GCcsRadAnalY );
    print_eps_integer4( "GCcsRadAnalZ", (eps_integer4*)X->GCcsRadAnalZ, fp, ndims_mdr_1c_GCcsRadAnalZ, dims_mdr_1c_GCcsRadAnalZ );
    print_eps_vinteger4( "GCcsRadAnalMean", (eps_vinteger4*)X->GCcsRadAnalMean, fp, ndims_mdr_1c_GCcsRadAnalMean, dims_mdr_1c_GCcsRadAnalMean );
    print_eps_vinteger4( "GCcsRadAnalStd", (eps_vinteger4*)X->GCcsRadAnalStd, fp, ndims_mdr_1c_GCcsRadAnalStd, dims_mdr_1c_GCcsRadAnalStd );
    print_eps_uinteger1( "GCcsImageClassified", (eps_uinteger1*)X->GCcsImageClassified, fp, ndims_mdr_1c_GCcsImageClassified, dims_mdr_1c_GCcsImageClassified );
    print_eps_bitfield( "IDefCcsMode", X->IDefCcsMode, fp, 32, 0, NULL );
    print_eps_integer2( "GCcsImageClassifiedNbLin", (eps_integer2*)X->GCcsImageClassifiedNbLin, fp, ndims_mdr_1c_GCcsImageClassifiedNbLin, dims_mdr_1c_GCcsImageClassifiedNbLin );
    print_eps_integer2( "GCcsImageClassifiedNbCol", (eps_integer2*)X->GCcsImageClassifiedNbCol, fp, ndims_mdr_1c_GCcsImageClassifiedNbCol, dims_mdr_1c_GCcsImageClassifiedNbCol );
    print_eps_vinteger4( "GCcsImageClassifiedFirstLin", (eps_vinteger4*)X->GCcsImageClassifiedFirstLin, fp, ndims_mdr_1c_GCcsImageClassifiedFirstLin, dims_mdr_1c_GCcsImageClassifiedFirstLin );
    print_eps_vinteger4( "GCcsImageClassifiedFirstCol", (eps_vinteger4*)X->GCcsImageClassifiedFirstCol, fp, ndims_mdr_1c_GCcsImageClassifiedFirstCol, dims_mdr_1c_GCcsImageClassifiedFirstCol );
    print_eps_boolean( "GCcsRadAnalType", (eps_boolean*)X->GCcsRadAnalType, fp, ndims_mdr_1c_GCcsRadAnalType, dims_mdr_1c_GCcsRadAnalType );
    print_eps_vinteger4( "GIacVarImagIIS", (eps_vinteger4*)X->GIacVarImagIIS, fp, ndims_mdr_1c_GIacVarImagIIS, dims_mdr_1c_GIacVarImagIIS );
    print_eps_vinteger4( "GIacAvgImagIIS", (eps_vinteger4*)X->GIacAvgImagIIS, fp, ndims_mdr_1c_GIacAvgImagIIS, dims_mdr_1c_GIacAvgImagIIS );
    print_eps_uinteger1( "GEUMAvhrr1BCldFrac", (eps_uinteger1*)X->GEUMAvhrr1BCldFrac, fp, ndims_mdr_1c_GEUMAvhrr1BCldFrac, dims_mdr_1c_GEUMAvhrr1BCldFrac );
    print_eps_uinteger1( "GEUMAvhrr1BLandFrac", (eps_uinteger1*)X->GEUMAvhrr1BLandFrac, fp, ndims_mdr_1c_GEUMAvhrr1BLandFrac, dims_mdr_1c_GEUMAvhrr1BLandFrac );
    print_eps_bitfield( "GEUMAvhrr1BQual", (eps_byte*)X->GEUMAvhrr1BQual, fp, 8, ndims_mdr_1c_GEUMAvhrr1BQual, dims_mdr_1c_GEUMAvhrr1BQual );
  } else {
    int k = 1;
    char label[16];
    for( i = 1; i < ndim; i++ ) {
      k *= dims[i];
    }
    for( i = 0; i < dims[0]; i++ ) {
      sprintf( label, "[%d]", i ); 
      print_mdr_1c( label, X + i * k, fp, ndim-1, dims + 1 );
    }
  }
}



int get_eps_iasil1c_9_0( t_eps_iasil1c_9_0 *R, FILE *fp ) {
  buffer B;
  
   read_grh( &( R->grh ), fp );
 if( R->grh.instrument_group == DUMMY || R->grh.instrument_group == ARCHIVE ) {
   fseek( fp, R->grh.record_size - SIZE_OF_REC_HEAD, SEEK_CUR );
   return 1;        
 }


  
  if( 
      ( R->grh.record_class == MPHR )
   && ( ( ! eps_iasil1c_9_0_check_subclass ) || R->grh.record_subclass == 0 )
   && ( ( ! eps_iasil1c_9_0_check_subclass_version ) || R->grh.record_subclass_version == 2 )
  ) {
  
    init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
    fread( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );
    R->content.MPHR = (t_mphr*)getmem( sizeof( t_mphr ) );
    read_mphr( R->content.MPHR, &B, 1 );
    free_buffer( &B );
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == IPR )
  ) {
  
    init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
    fread( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );
    R->content.IPR = (t_ipr*)getmem( sizeof( t_ipr ) );
    read_ipr( R->content.IPR, &B, 1 );
    free_buffer( &B );
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == GIADR )
   && ( ( ! eps_iasil1c_9_0_check_subclass ) || R->grh.record_subclass == 0 )
   && ( ( ! eps_iasil1c_9_0_check_subclass_version ) || R->grh.record_subclass_version == 2 )
  ) {
  
    init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
    fread( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );
    R->content.GIADR_QUALITY = (t_giadr_quality*)getmem( sizeof( t_giadr_quality ) );
    read_giadr_quality( R->content.GIADR_QUALITY, &B, 1 );
    free_buffer( &B );
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == GIADR )
   && ( ( ! eps_iasil1c_9_0_check_subclass ) || R->grh.record_subclass == 1 )
   && ( ( ! eps_iasil1c_9_0_check_subclass_version ) || R->grh.record_subclass_version == 2 )
  ) {
  
    init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
    fread( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );
    R->content.GIADR_SCALEFACTORS = (t_giadr_scalefactors*)getmem( sizeof( t_giadr_scalefactors ) );
    read_giadr_scalefactors( R->content.GIADR_SCALEFACTORS, &B, 1 );
    free_buffer( &B );
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == MDR )
   && ( ( ! eps_iasil1c_9_0_check_subclass ) || R->grh.record_subclass == 2 )
   && ( ( ! eps_iasil1c_9_0_check_subclass_version ) || R->grh.record_subclass_version == 5 )
  ) {
  
    init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
    fread( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );
    R->content.MDR_1C = (t_mdr_1c*)getmem( sizeof( t_mdr_1c ) );
    read_mdr_1c( R->content.MDR_1C, &B, 1 );
    free_buffer( &B );
    return 1;

  
  }
  

  fseek( fp, R->grh.record_size - SIZE_OF_REC_HEAD, SEEK_CUR );

  return 0;

}


int put_eps_iasil1c_9_0( t_eps_iasil1c_9_0 *R, FILE *fp ) {
  buffer B;
  
  

  
  if( 
      ( R->grh.record_class == MPHR )
   && ( ( ! eps_iasil1c_9_0_check_subclass ) || R->grh.record_subclass == 0 )
   && ( ( ! eps_iasil1c_9_0_check_subclass_version ) || R->grh.record_subclass_version == 2 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      R->grh.record_size = size_mphr( R->content.MPHR ) + SIZE_OF_REC_HEAD;
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_mphr( R->content.MPHR, &B, 1 );

      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    } else {
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    }
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == IPR )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      R->grh.record_size = size_ipr( R->content.IPR ) + SIZE_OF_REC_HEAD;
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_ipr( R->content.IPR, &B, 1 );

      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    } else {
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    }
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == GIADR )
   && ( ( ! eps_iasil1c_9_0_check_subclass ) || R->grh.record_subclass == 0 )
   && ( ( ! eps_iasil1c_9_0_check_subclass_version ) || R->grh.record_subclass_version == 2 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      R->grh.record_size = size_giadr_quality( R->content.GIADR_QUALITY ) + SIZE_OF_REC_HEAD;
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_giadr_quality( R->content.GIADR_QUALITY, &B, 1 );

      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    } else {
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    }
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == GIADR )
   && ( ( ! eps_iasil1c_9_0_check_subclass ) || R->grh.record_subclass == 1 )
   && ( ( ! eps_iasil1c_9_0_check_subclass_version ) || R->grh.record_subclass_version == 2 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      R->grh.record_size = size_giadr_scalefactors( R->content.GIADR_SCALEFACTORS ) + SIZE_OF_REC_HEAD;
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_giadr_scalefactors( R->content.GIADR_SCALEFACTORS, &B, 1 );

      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    } else {
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    }
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == MDR )
   && ( ( ! eps_iasil1c_9_0_check_subclass ) || R->grh.record_subclass == 2 )
   && ( ( ! eps_iasil1c_9_0_check_subclass_version ) || R->grh.record_subclass_version == 5 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      R->grh.record_size = size_mdr_1c( R->content.MDR_1C ) + SIZE_OF_REC_HEAD;
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_mdr_1c( R->content.MDR_1C, &B, 1 );

      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    } else {
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    }
    return 1;

  
  }
  

  

  return 0;

}
#ifdef HAS_LIBHDF5
int put_eps_iasil1c_hdf5( t_eps_iasil1c_9_0 *R, hid_t file_id, int irec ) {
  char        grp_name [5];
  hid_t       grp;
  herr_t      status;

  sprintf(grp_name,"%05u",irec);
  grp = H5Gcreate(file_id, grp_name, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);

  if( 
      ( R->grh.record_class == MPHR )
   && ( ( ! eps_iasil1c_9_0_check_subclass ) || R->grh.record_subclass == 0 )
   && ( ( ! eps_iasil1c_9_0_check_subclass_version ) || R->grh.record_subclass_version == 2 )
  ) {

    if( R->grh.instrument_group != DUMMY ) {
      /*write_mphr( R->content.MPHR, &B, 1 );*/
      /*
       * creation du groupe  /MPHR/<irec>
       */         
      write_mphr_hdf5( R->content.MPHR, grp, 1 );

      fprintf( stderr, "write mphr   %d  %s\n", irec, grp_name);

    }
    return 1;
  }


  if( 
      ( R->grh.record_class == IPR )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      /*write_ipr( R->content.IPR, &B, 1 );*/

    }
    return 1;
  }


  if( 
      ( R->grh.record_class == GIADR )
   && ( ( ! eps_iasil1c_9_0_check_subclass ) || R->grh.record_subclass == 0 )
   && ( ( ! eps_iasil1c_9_0_check_subclass_version ) || R->grh.record_subclass_version == 2 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      /*write_giadr_quality( R->content.GIADR_QUALITY, &B, 1 );*/
      fprintf( stderr, "write giadr qual   %d \n", irec);

    }
    return 1;
  }


  if( 
      ( R->grh.record_class == GIADR )
   && ( ( ! eps_iasil1c_9_0_check_subclass ) || R->grh.record_subclass == 1 )
   && ( ( ! eps_iasil1c_9_0_check_subclass_version ) || R->grh.record_subclass_version == 2 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      /*write_giadr_scalefactors( R->content.GIADR_SCALEFACTORS, &B, 1 );*/
      fprintf( stderr, "write giadr scale   %d \n", irec);

    }
    return 1;
  }


  if( 
      ( R->grh.record_class == MDR )
   && ( ( ! eps_iasil1c_9_0_check_subclass ) || R->grh.record_subclass == 2 )
   && ( ( ! eps_iasil1c_9_0_check_subclass_version ) || R->grh.record_subclass_version == 5 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      /*write_mdr_1c( R->content.MDR_1C, &B, 1 );*/
      write_mdr_1c_hdf5( R->content.MDR_1C, grp, 1 );
      write_grh_hdf5( &R->grh, grp );
      fprintf( stderr, "write mdr   %d \n", irec);

    }
    return 1;
  }


  status = H5Gclose(grp);

  return 0;
}
#endif

int free_eps_iasil1c_9_0( t_eps_iasil1c_9_0 *R ) {
  buffer B;
  
  

  
  if( 
      ( R->grh.record_class == MPHR )
   && ( ( ! eps_iasil1c_9_0_check_subclass ) || R->grh.record_subclass == 0 )
   && ( ( ! eps_iasil1c_9_0_check_subclass_version ) || R->grh.record_subclass_version == 2 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      free_mphr( R->content.MPHR, 1 );
      free( R->content.MPHR );
    } 
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == IPR )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      free_ipr( R->content.IPR, 1 );
      free( R->content.IPR );
    } 
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == GIADR )
   && ( ( ! eps_iasil1c_9_0_check_subclass ) || R->grh.record_subclass == 0 )
   && ( ( ! eps_iasil1c_9_0_check_subclass_version ) || R->grh.record_subclass_version == 2 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      free_giadr_quality( R->content.GIADR_QUALITY, 1 );
      free( R->content.GIADR_QUALITY );
    } 
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == GIADR )
   && ( ( ! eps_iasil1c_9_0_check_subclass ) || R->grh.record_subclass == 1 )
   && ( ( ! eps_iasil1c_9_0_check_subclass_version ) || R->grh.record_subclass_version == 2 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      free_giadr_scalefactors( R->content.GIADR_SCALEFACTORS, 1 );
      free( R->content.GIADR_SCALEFACTORS );
    } 
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == MDR )
   && ( ( ! eps_iasil1c_9_0_check_subclass ) || R->grh.record_subclass == 2 )
   && ( ( ! eps_iasil1c_9_0_check_subclass_version ) || R->grh.record_subclass_version == 5 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      free_mdr_1c( R->content.MDR_1C, 1 );
      free( R->content.MDR_1C );
    } 
    return 1;

  
  }
  

  

  return 0;

}

