/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/5
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * VERSION   : 1.0
 *
 * HISTORY   : 02/11/2005 fixup filler7 in t_record 191 -> 67
 *
 */
#ifndef _AAPP_AVHRRL1B_H
#define _AAPP_AVHRRL1B_H
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "eps_types.h"
#include "eps_buffer.h"
#include "eps_grh.h"


extern int aapp_avhrrl1b_check_subclass;
extern int aapp_avhrrl1b_check_subclass_version;


typedef struct {
  eps_byte avh_h_siteid[3]; /* dataset creation site id */
  eps_byte avh_h_blank[1]; /* blank */
  eps_integer2 avh_h_l1bversnb; /* Level 1b format version number */
  eps_integer2 avh_h_l1bversyr; /* Level 1b format version year */
  eps_integer2 avh_h_l1bversdy; /* Level 1d format version day of year 
                                   */
  eps_integer2 avh_h_reclg; /* Reserved for logical record length */
  eps_integer2 avh_h_blksz; /* Reserved for block size */
  eps_integer2 avh_h_hdrcnt; /* Count of header records in this data se
                                t */
  eps_integer2 avh_h_filler0[3];
  eps_byte avh_h_dataname[42]; /* Data set name */
  eps_byte avh_h_prblkid[8]; /* Processing block identification */
  eps_integer2 avh_h_satid; /* Satellite id */
  eps_integer2 avh_h_instid; /* Instrument id */
  eps_uinteger2 avh_h_datatyp; /* Data type code */
  eps_uinteger2 avh_h_tipsrc; /* TIP source code */
  eps_integer4 avh_h_startdatajd; /* Start of data set Julian Day(00h,1
                                     jan50) */
  eps_integer2 avh_h_startdatayr; /* Start of data set year */
  eps_integer2 avh_h_startdatady; /* Start of data set day of the year 
                                     */
  eps_integer4 avh_h_startdatatime; /* Start of data set UTC time of da
                                       y in milliseconds */
  eps_integer4 avh_h_enddatajd; /* End of data set Julian Day(00h,1jan5
                                   0) */
  eps_integer2 avh_h_enddatayr; /* End of data set year */
  eps_integer2 avh_h_enddatady; /* End of data set day of the year */
  eps_integer4 avh_h_enddatatime; /* End of data set UTC time of day in 
                                     milliseconds */
  eps_integer2 avh_h_cpidsyr; /* Year of last CPIDS Update */
  eps_integer2 avh_h_cpidsdy; /* Day of year of last CPIDS Update */
  eps_integer2 avh_h_filler1[4];
  eps_integer4 avh_h_inststat1; /* Instrument status */
  eps_integer2 avh_h_filler2;
  eps_integer2 avh_h_statchrecnb; /* Record number of status change */
  eps_integer4 avh_h_inststat2; /* Second instrument status */
  eps_integer2 avh_h_scnlin; /* Count of scan lines in this data set */
  eps_integer2 avh_h_callocsclin; /* Count of calibrated, earth located 
                                     scan lines in this data set */
  eps_integer2 avh_h_misscnlin; /* Count of missing scan lines */
  eps_integer2 avh_h_datagaps; /* Count of data gaps in this data set */
  eps_integer2 avh_h_okdatafr; /* Count of data frames without frame sy
                                  nc word errors */
  eps_integer2 avh_h_pacsparityerr; /* Count of PACS detected TIP parit
                                       y errors */
  eps_integer2 avh_h_auxsyncerrsum; /* Sum of all auxiliary sync errors 
                                       detected in the input data */
  eps_integer2 avh_h_timeseqerr; /* Time sequence error */
  eps_integer2 avh_h_timeseqerrcode; /* Time sequence error code */
  eps_integer2 avh_h_socclockupind; /* Socc clock update indicator */
  eps_integer2 avh_h_locerrind; /* Earth location error indicator */
  eps_integer2 avh_h_locerrcode; /* Earth location error code */
  eps_integer2 avh_h_pacsstatfield;
  eps_integer2 avh_h_pacsdatasrc;
  eps_integer4 avh_h_filler3;
  eps_byte avh_h_spare1[8]; /* Reserved for the ingester */
  eps_byte avh_h_spare2[8]; /* Reserved of the decommutation */
  eps_integer2 avh_h_filler4[5];
  eps_integer2 avh_h_racalind; /* Ramp/auto calibration indicators */
  eps_integer2 avh_h_solarcalyr; /* Year of most recent solar ch. calib
                                    . */
  eps_integer2 avh_h_solarcaldy; /* Day of year of most recent solar ch
                                    annel calibration */
  eps_integer2 avh_h_pcalalgind; /* Primary calibration algorithm ident
                                    ification */
  eps_integer2 avh_h_pcalalgopt; /* Primary calibration algorithm optio
                                    n selected */
  eps_integer2 avh_h_scalalgind; /* Secundary calibration algorithm ide
                                    ntification */
  eps_integer2 avh_h_scalalgopt; /* Secundary calibration algorithm opt
                                    ion selected */
  eps_integer2 avh_h_irttcoef[4][6];
  eps_integer4 avh_h_filler5[2];
  eps_integer4 avh_h_albcnv[3][2];
  eps_integer4 avh_h_radtempcv[3][3];
  eps_integer4 avh_h_filler6[3];
  eps_byte avh_h_modelid[8];
  eps_integer2 avh_h_nadloctol; /* Nadir earth location tolerance */
  eps_integer2 avh_h_locbit; /* Earth location bit field */
  eps_integer2 avh_h_filler7;
  eps_integer2 avh_h_rollerr; /* Constant roll attitude error */
  eps_integer2 avh_h_pitcherr; /* Constant pitch attitude error */
  eps_integer2 avh_h_yawerr; /* Constant yaw attitude error */
  eps_integer2 avh_h_epoyr; /* Epoch year for orbit vector */
  eps_integer2 avh_h_epody; /* Day of epoch year for orbit vector */
  eps_integer4 avh_h_epotime; /* Epoch UTC time of day in milliseconds 
                                 */
  eps_integer4 avh_h_smaxis; /* Semi-major axis in kilometers */
  eps_integer4 avh_h_eccen; /* Eccentricity */
  eps_integer4 avh_h_incli; /* Inclination in degrees */
  eps_integer4 avh_h_argper; /* Argument of perigee in degrees */
  eps_integer4 avh_h_rascnod; /* Right ascension of the ascending node 
                                 in degrees */
  eps_integer4 avh_h_manom; /* Mean anomaly in degrees */
  eps_integer4 avh_h_xpos; /* Position vector X component in km */
  eps_integer4 avh_h_ypos; /* Position vector Y component in km */
  eps_integer4 avh_h_zpos; /* Position vector Z component in km */
  eps_integer4 avh_h_xvel; /* Velocity vector X-dot component in kilome
                              ters/seconds */
  eps_integer4 avh_h_yvel; /* Velocity vector Y-dot component in kilome
                              ters/seconds */
  eps_integer4 avh_h_zvel; /* Velocity vector Z-dot component in kilome
                              ters/seconds */
  eps_integer4 avh_h_earthsun; /* Earth/sun distance ratio */
  eps_integer4 avh_h_filler8[4];
  eps_integer2 avh_h_pchtemp[5];
  eps_integer2 avh_h_reserved1;
  eps_integer2 avh_h_pchtempext[5];
  eps_integer2 avh_h_reserved2;
  eps_integer2 avh_h_pchpow[5];
  eps_integer2 avh_h_reserved3;
  eps_integer2 avh_h_rdtemp[5];
  eps_integer2 avh_h_reserved4;
  eps_integer2 avh_h_bbtemp1[5];
  eps_integer2 avh_h_reserved5;
  eps_integer2 avh_h_bbtemp2[5];
  eps_integer2 avh_h_reserved6;
  eps_integer2 avh_h_bbtemp3[5];
  eps_integer2 avh_h_reserved7;
  eps_integer2 avh_h_bbtemp4[5];
  eps_integer2 avh_h_reserved8;
  eps_integer2 avh_h_eleccur[5];
  eps_integer2 avh_h_reserved9;
  eps_integer2 avh_h_motorcur[5];
  eps_integer2 avh_h_reserved10;
  eps_integer2 avh_h_earthpos[5];
  eps_integer2 avh_h_reserved11;
  eps_integer2 avh_h_electemp[5];
  eps_integer2 avh_h_reserved12;
  eps_integer2 avh_h_chtemp[5];
  eps_integer2 avh_h_reserved13;
  eps_integer2 avh_h_bptemp[5];
  eps_integer2 avh_h_reserved14;
  eps_integer2 avh_h_mhtemp[5];
  eps_integer2 avh_h_reserved15;
  eps_integer2 avh_h_adcontemp[5];
  eps_integer2 avh_h_reserved16;
  eps_integer2 avh_h_d4bvolt[5];
  eps_integer2 avh_h_reserved17;
  eps_integer2 avh_h_d5bvolt[5];
  eps_integer2 avh_h_reserved18;
  eps_integer2 avh_h_bbtempchn3B[5];
  eps_integer2 avh_h_reserved19;
  eps_integer2 avh_h_bbtempchn4[5];
  eps_integer2 avh_h_reserved20;
  eps_integer2 avh_h_bbtempchn5[5];
  eps_integer2 avh_h_reserved21;
  eps_integer2 avh_h_refvolt[5];
  eps_integer2 avh_h_reserved22;
  eps_integer2 avh_h_filler9[10664];
} t_header;

int read_header( t_header *X, buffer* B, int n );

int write_header( t_header *X, buffer* B, int n );

int free_header( t_header *X, int n );

int init_header( t_header *X, int n );

int size_header( t_header *X );

typedef struct {
  eps_integer2 avh_scnlin; /* Scan line number */
  eps_integer2 avh_scnlinyr; /* Scan line year */
  eps_integer2 avh_scnlindy; /* Scan line day of year */
  eps_integer2 avh_clockdrift; /* Satellite clock drift delta in millis
                                  econds */
  eps_integer4 avh_scnlintime; /* Scan line UTC time of day in millisec
                                  onds */
  eps_integer2 avh_scnlinbit; /* Scan line bit field */
  eps_integer2 avh_filler0[5];
  eps_integer4 avh_qualind; /* Quality indicator bit field */
  eps_integer4 avh_scnlinqual; /* Scan line quality flags */
  eps_integer2 avh_calqual[3];
  eps_integer2 avh_cbiterr; /* Count of bit errors in frame sync */
  eps_integer4 avh_filler1[2];
  eps_integer4 avh_calvis[3][3][5];
  eps_integer4 avh_calir[3][2][3];
  eps_integer4 avh_filler2[3];
  eps_integer4 avh_navstat; /* Navigation status bit field */
  eps_integer4 avh_attangtime; /* Time associated angles */
  eps_integer2 avh_rollang; /* Roll angle in degrees */
  eps_integer2 avh_pitchang; /* Pitch angle in degrees */
  eps_integer2 avh_yawang; /* Yaw angle in degrees */
  eps_integer2 avh_scalti; /* Spacecraft altitude (MSL) in km */
  eps_integer2 avh_ang[51][3];
  eps_integer2 avh_filler3[3];
  eps_integer4 avh_pos[51][2];
  eps_integer4 avh_filler4[2];
  eps_integer2 avh_telem[103];
  eps_integer2 avh_filler5;
  eps_integer2 avh_hrpt[2048][5];
  eps_integer4 avh_filler6[2];
  eps_integer2 avh_tipmfhd[5][7];
  eps_integer2 avh_icputel[5][6];
  eps_integer2 avh_filler7[67];
} t_record;

int read_record( t_record *X, buffer* B, int n );

int write_record( t_record *X, buffer* B, int n );

int free_record( t_record *X, int n );

int init_record( t_record *X, int n );

int size_record( t_record *X );

#endif
