#ifndef _AAPP_AVHRRL1C_H
#define _AAPP_AVHRRL1C_H
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "eps_types.h"
#include "eps_buffer.h"
#include "eps_grh.h"


extern int aapp_avhrrl1c_check_subclass;
extern int aapp_avhrrl1c_check_subclass_version;

#define avh1c_mxpix 2048
#define avh1c_mxhrpchn 5
#define avh1c_mxirchn 3
#define avh1c_versyr 2009
#define avh1c_mxvischn 3
#define avh1c_versdy 91
#define avh1c_vers 1

typedef struct {
  eps_byte avh1c_h_site[3]; /* Data set creation site ID */
  eps_byte avh1c_h_cfill1[1];
  eps_byte avh1c_h_l1bsite[3]; /* creation site for original 1B data */
  eps_byte avh1c_h_cfill2[1];
  eps_integer4 avh1c_h_versnb; /* level 1c format version number */
  eps_integer4 avh1c_h_versyr; /* level 1c format version year */
  eps_integer4 avh1c_h_versdy; /* level 1c format version day of year */
  eps_integer4 avh1c_h_hdrcnt; /* count of header records in data set */
  eps_integer4 avh1c_h_satid; /* satellite id (e.g 14 for NOAA-14) */
  eps_integer4 avh1c_h_instrument; /* Instrument id (1=AVHRR) */
  eps_integer4 avh1c_h_satht; /* nominal satellite altitude, km*10 */
  eps_integer4 avh1c_h_period; /* nominal orbit period (seconds) */
  eps_integer4 avh1c_h_startorbit; /* orbit number (at start of file) */
  eps_integer4 avh1c_h_startdatayr; /* start of data set year */
  eps_integer4 avh1c_h_startdatady; /* start of data set day of the yea
                                       r */
  eps_integer4 avh1c_h_startdatatime; /* start of data set UTC time of 
                                         day (ms)start of data set UTC 
                                         time of day (ms) */
  eps_integer4 avh1c_h_endorbit; /* orbit number (at end of file) */
  eps_integer4 avh1c_h_enddatayr; /* end of data set year */
  eps_integer4 avh1c_h_enddatady; /* end of data set day of the year */
  eps_integer4 avh1c_h_enddatatime; /* end of data set UTC time of day 
                                       (ms) */
  eps_integer4 avh1c_h_scnlin; /* count of scan lines in this data set 
                                  */
  eps_integer4 avh1c_h_misscnlin; /* count of missing scan lines */
  eps_integer4 avh1c_h_callocscnlin; /* count of calibrated, earth loca
                                        ted scan lines in this data set 
                                        */
  eps_integer4 avh1c_h_datagaps; /* count of data gaps in this data set 
                                    */
  eps_integer4 avh1c_h_datatyp; /* data type code (1 = LAC; 2 = GAC; 3 
                                   = HRPT) */
  eps_byte avh1c_h_infilename[68]; /* input file name (allow pfs name) 
                                      */
  eps_integer4 avh1c_h_wmosatid; /* WMO satellite id (3 for METOP-1) */
  eps_integer4 avh1c_h_cpidsyr; /* year of Last CPIDS Update */
  eps_integer4 avh1c_h_cpidsdy; /* Day of Year of Last CPIDS Update */
  eps_integer4 avh1c_h_rollerr; /* constant roll attitude error in degr
                                   ees (x10^3) */
  eps_integer4 avh1c_h_pitcherr; /* constant pitch attitude error in de
                                    grees (x10^3) */
  eps_integer4 avh1c_h_yawerr; /* constant yaw attitude error in degree
                                  s (x10^3) */
  eps_integer4 avh1c_h_albcnv[2][avh1c_mxvischn];
  eps_integer4 avh1c_h_radtempcnv[3][avh1c_mxirchn];
  eps_integer4 avh1c_h_filler[7388];
} t_header;

int read_header( t_header *X, buffer* B, int n );

int write_header( t_header *X, buffer* B, int n );

int free_header( t_header *X, int n );

int init_header( t_header *X, int n );

int size_header( t_header *X );

int print_header( const char* label, t_header *X, FILE *fp, int ndim, const int *dims );

typedef struct {
  eps_integer4 avh1c_scnlin; /* scan line number */
  eps_integer4 avh1c_scnlinyr; /* scan line year */
  eps_integer4 avh1c_scnlindy; /* scan line day of year */
  eps_integer4 avh1c_scnlintime; /* scan line UTC time of day in millis
                                    econds */
  eps_integer4 avh1c_clockdrift; /* satellite clock drift delta in mill
                                    iseconds */
  eps_integer4 avh1c_scnlinbit; /* scan line bit field */
  eps_integer4 avh1c_spare0[2];
  eps_integer4 avh1c_qualind; /* quality indicator bit field */
  eps_integer4 avh1c_scnlinqual; /* scan line quality flags */
  eps_integer4 avh1c_calqual[avh1c_mxirchn];
  eps_integer4 avh1c_spare1[2];
  eps_integer4 avh1c_scalti; /* 10x Spacecraft Altitude (MSL) in km */
  eps_integer4 avh1c_ang[51][3];
  eps_integer4 avh1c_pos[51][2];
  eps_integer4 avh1c_spare2[2];
  eps_uinteger2 avh1c_btemps[avh1c_mxpix][avh1c_mxhrpchn];
  eps_integer4 avh1c_maskind; /* cloud mask done =1 */
  eps_uinteger2 avh1c_mask[avh1c_mxpix];
  eps_uinteger2 avh1c_tsurf[avh1c_mxpix];
  eps_integer4 avh1c_spare3[10];
} t_record;

int read_record( t_record *X, buffer* B, int n );

int write_record( t_record *X, buffer* B, int n );

int free_record( t_record *X, int n );

int init_record( t_record *X, int n );

int size_record( t_record *X );

int print_record( const char* label, t_record *X, FILE *fp, int ndim, const int *dims );

#endif
