/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/5
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * VERSION   : 1.1
 *
 * HISTORY   : 20/10/2005 : Change BYTE_ORDER to AAPP_BYTE_ORDER
 *                          Change LITTLE_ENDIAN to AAPP_LITTLE_ENDIAN
 *                          Change BIG_ENDIAN to AAPP_BIG_ENDIAN
 *             26/01/2006 : Add definition of AAPP_BIG_ENDIAN &
 *                          AAPP_LITTLE_ENDIAN
 */
#ifndef _CCSDS_H
#define _CCSDS_H

#include <stdlib.h>
#include <stdio.h>

#ifndef AAPP_BIG_ENDIAN
#define AAPP_BIG_ENDIAN 4321
#define AAPP_LITTLE_ENDIAN 1234
#endif

typedef unsigned long sword;
typedef unsigned long long dword;
typedef unsigned short hword;
typedef unsigned char qword;

typedef struct {
  int version;
  int type;
  int secondary_header_flag;
  int APID;
  int sequence_flag;
  int packet_seq_count;
  int packet_length;
} packet_primary_header;

#define SIZE_OF_PACKET_PRIMARY_HEADER 6

typedef struct {
  int days;
  unsigned int msec;
  int usec;
} packet_secondary_header;

#define SIZE_OF_PACKET_SECONDARY_HEADER 8

typedef struct {
  unsigned int coarse_time;
  unsigned int fine_time;
} satellite_binary_time;

#define SIZE_OF_SATELLITE_BINARY_TIME 6

typedef struct {
  packet_primary_header pph;
  packet_secondary_header psh;
  satellite_binary_time sbt;
  char *packet;
} ccsds_packet;


#if defined( AAPP_BYTE_ORDER ) && AAPP_BYTE_ORDER == AAPP_LITTLE_ENDIAN

#define SWAP_INT_BUFFER( p, n, t ) \
  do { ccsds_swap_int_buffer( (char*)p, n, t ); } while( 0 )

#else

#define SWAP_INT_BUFFER( p, n, t ) \
      do { } while( 0 )

#endif

int ccsds_swap_int_buffer( char *p, int t, int n );

int ccsds_read_packet_primary_header(
  packet_primary_header *pph, char* buffer
);

int ccsds_read_packet_secondary_header(
  packet_secondary_header *psh, char* buffer
);

int ccsds_read_satellite_binary_time(
  satellite_binary_time *sbt, char* buffer
);

int ccsds_read_pph_psh_sbt( 
  packet_primary_header *pph, packet_secondary_header *psh, 
  satellite_binary_time *sbt, char* buffer
);

int ccsds_fread_packet( ccsds_packet* packet, FILE* fpr );

int ccsds_bread_packet( ccsds_packet* packet, char* buf );

void ccsds_free_packet( ccsds_packet* packet );



#endif
