/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/5
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * VERSION   : 1.0
 *
 */
#ifndef _EPS_AVHRRL1B_6_5_H
#define _EPS_AVHRRL1B_6_5_H
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "eps_types.h"
#include "eps_buffer.h"
#include "eps_grh.h"


extern int eps_avhrrl1b_6_5_check_subclass;
extern int eps_avhrrl1b_6_5_check_subclass_version;

#define NE 2048
#define NP 103

typedef struct {
  /* Product_Details */
  eps_byte PRODUCT_NAME[67]; /* Complete name of the product */
  eps_byte PARENT_PRODUCT_NAME_1[67]; /* Name of the parent product fro
                                         m which this product has been 
                                         produced. For Level 0 products
                                         , this field is filled with lo
                                         wer case x's. */
  eps_byte PARENT_PRODUCT_NAME_2[67]; /* Name of the parent product fro
                                         m which this product has been 
                                         produced. For Level 0 products 
                                         or products for which this is 
                                         not appropriate, this field is 
                                         filled with lower case x's. */
  eps_byte PARENT_PRODUCT_NAME_3[67]; /* Name of the parent product fro
                                         m which this product has been 
                                         produced. For Level 0 products 
                                         or products for which this is 
                                         not appropriate, this field is 
                                         filled with lower case x's. */
  eps_byte PARENT_PRODUCT_NAME_4[67]; /* Name of the parent product fro
                                         m which this product has been 
                                         produced. For Level 0 products 
                                         or products for which this is 
                                         not appropriate, this field is 
                                         filled with lower case x's. */
  eps_byte INSTRUMENT_ID[4]; /* Instrument identification */
  eps_byte INSTRUMENT_MODEL[3]; /* Instrument Model identification */
  eps_byte PRODUCT_TYPE[3]; /* Product Type */
  eps_byte PROCESSING_LEVEL[2]; /* Processing Level Identification */
  eps_byte SPACECRAFT_ID[3]; /* Spacecraft identification */
  eps_gentime SENSING_START; /* UTC Time of start of sensing data in th
                                is object (PDU, ROI or Full Product) */
  eps_gentime SENSING_END; /* UTC Time of end of sensing data in this o
                              bject (PDU, ROI or Full Product) */
  eps_gentime SENSING_START_THEORETICAL; /* Theoretical UTC Time of sta
                                            rt of sensing data in the d
                                            ump from which this object 
                                            is derived. This data is th
                                            e predicted start time at t
                                            he MPF level. */
  eps_gentime SENSING_END_THEORETICAL; /* Theoretical UTC Time of end o
                                          f sensing data in the dump fr
                                          om which this object is deriv
                                          ed. This data is the predicte
                                          d end time at the MPF level. 
                                          */
  eps_byte PROCESSING_CENTRE[4]; /* Processing Centre Identification */
  eps_uinteger PROCESSOR_MAJOR_VERSION; /* Processing chain major versi
                                           on number */
  eps_uinteger PROCESSOR_MINOR_VERSION; /* Processing chain minor versi
                                           on number */
  eps_uinteger FORMAT_MAJOR_VERSION; /* Dataset Format Major Version nu
                                        mber */
  eps_uinteger FORMAT_MINOR_VERSION; /* Dataset Format Minor Version nu
                                        mber */
  eps_gentime PROCESSING_TIME_START; /* UTC time of the processing at s
                                        tart of processing for the prod
                                        uct */
  eps_gentime PROCESSING_TIME_END; /* UTC time of the processing at end 
                                      of processing for the product */
  eps_byte PROCESSING_MODE[1]; /* Identification of the mode of process
                                  ing */
  eps_byte DISPOSITION_MODE[1]; /* Identification of the diposition mod
                                   e */
  eps_byte RECEIVING_GROUND_STATION[3]; /* Acquisition Station Identifi
                                           cation */
  eps_gentime RECEIVE_TIME_START; /* UTC time of the reception at CDA f
                                     or first Data Item */
  eps_gentime RECEIVE_TIME_END; /* UTC time of the reception at CDA for 
                                   last Data Item */
  eps_uinteger ORBIT_START; /* Start Orbit Number, counted incrementall
                               y since launch */
  eps_uinteger ORBIT_END; /* Stop Orbit Number */
  eps_uinteger ACTUAL_PRODUCT_SIZE; /* Size of the complete product */
  /* ASCENDING_NODE_ORBIT_PARAMETERS */
  eps_longgentime STATE_VECTOR_TIME; /* Epoch time (in UTC) of the orbi
                                        tal elements and the orbit stat
                                        e vector. this corresponds to t
                                        he time of crossing the ascendi
                                        ng node for ORBIT_START */
  eps_integer SEMI_MAJOR_AXIS; /* Semi major axis of orbit at time of t
                                  he ascending node crossing. */
  eps_integer ECCENTRICITY; /* Orbit eccentricity at time of the ascend
                               ing node crossing */
  eps_integer INCLINATION; /* Orbit inclination at time of the ascendin
                              g node crossing */
  eps_integer PERIGEE_ARGUMENT; /* Argument of perigee at time of the a
                                   scending node crossing */
  eps_integer RIGHT_ASCENSION; /* Right ascension at time of the ascend
                                  ing node crossing */
  eps_integer MEAN_ANOMALY; /* Mean anomaly at time of the ascending no
                               de crossing */
  eps_integer X_POSITION; /* X position of the orbit state vector in th
                             e orbit frame at ascending node */
  eps_integer Y_POSITION; /* Y position of the orbit state vector in th
                             e orbit frame at ascending node */
  eps_integer Z_POSITION; /* Z position of the orbit state vector in th
                             e orbit frame at ascending node */
  eps_integer X_VELOCITY; /* X velocity of the orbit state vector in th
                             e orbit frame at ascending node */
  eps_integer Y_VELOCITY; /* Y velocity of the orbit state vector in th
                             e orbit frame at ascending node */
  eps_integer Z_VELOCITY; /* Z velocity of the orbit state vector in th
                             e orbit frame at ascending node */
  eps_integer EARTH_SUN_DISTANCE_RATIO; /* Earth-Sun distance ratio - r
                                           atio of current Eart-Sun dis
                                           tance to Mean Earth-Sun dist
                                           ance */
  eps_integer LOCATION_TOLERANCE_RADIAL; /* Nadir Earth location tolera
                                            nce radial */
  eps_integer LOCATION_TOLERANCE_CROSSTRACK; /* Nadir Earth location to
                                                lerance cross-track */
  eps_integer LOCATION_TOLERANCE_ALONGTRACK; /* Nadir Earth location to
                                                lerance along-track */
  eps_integer YAW_ERROR; /* Constant Yaw attitude error */
  eps_integer ROLL_ERROR; /* Constant Roll attitude error */
  eps_integer PITCH_ERROR; /* Constant Pitch attitude error */
  /* LOCATION_SUMMARY */
  eps_integer SUBSAT_LATITUDE_START; /* Latitude of sub-satellite point 
                                        at start of the data set */
  eps_integer SUBSAT_LONGITUDE_START; /* Longitude of sub-satellite poi
                                         nt at start of the data set */
  eps_integer SUBSAT_LATITUDE_END; /* Latitude of sub-satellite point a
                                      t end of the data set */
  eps_integer SUBSAT_LONGITUDE_END; /* Longitude of sub-satellite point 
                                       at end of the data set */
  /* Leap_Second_Information */
  eps_integer LEAP_SECOND; /* Occurence of Leap second within the produ
                              ct. Field is set to -1, 0 or +1 dependent 
                              upon occurrence of leap second and direct
                              ion. */
  eps_gentime LEAP_SECOND_UTC; /* UTC time of occurrence of the Leap Se
                                  cond (If no leap second in the produc
                                  t, value is null) */
  /* Record_counts */
  eps_uinteger TOTAL_RECORDS; /* Total count of all records in the prod
                                 uct */
  eps_uinteger TOTAL_MPHR; /* Total count of all MPHRs in product (shou
                              ld always be 1!) */
  eps_uinteger TOTAL_SPHR; /* Total count of all SPHRs in product (shou
                              ld be 0 or 1 only) */
  eps_uinteger TOTAL_IPR; /* Total count of all IPRs in the product */
  eps_uinteger TOTAL_GEADR; /* Total count of all GEADRs in the product 
                               */
  eps_uinteger TOTAL_GIADR; /* Total count of all GIADRs in the product 
                               */
  eps_uinteger TOTAL_VEADR; /* Total count of all VEADRs in the product 
                               */
  eps_uinteger TOTAL_VIADR; /* Total count of all VIADRs in the product 
                               */
  eps_uinteger TOTAL_MDR; /* Total count of all MDRs in the product */
  /* Record_Based_Generic_Quality_Flags */
  eps_uinteger COUNT_DEGRADED_INST_MDR; /* Count of MDRs with degradati
                                           on due to instrument problem
                                           s */
  eps_uinteger COUNT_DEGRADED_PROC_MDR; /* Count of MDRs with degradati
                                           on due to processing problem
                                           s */
  eps_uinteger COUNT_DEGRADED_INST_MDR_BLOCKS; /* Count of the number o
                                                  f blocks of MDRs degr
                                                  aded due to degraded 
                                                  instrument */
  eps_uinteger COUNT_DEGRADED_PROC_MDR_BLOCKS; /* Count of the number o
                                                  f blocks of MDRs degr
                                                  aded due to degraded 
                                                  processing */
  /* Time_Based_Generic_Quality_Flags */
  eps_uinteger DURATION_OF_PRODUCT; /* The duration of the product in m
                                       illiseconds */
  eps_uinteger MILLISECONDS_OF_DATA_PRESENT; /* The total amount of dat
                                                a present in the produc
                                                t */
  eps_uinteger MILLISECONDS_OF_DATA_MISSING; /* The total amount of dat
                                                a missing from the prod
                                                cut */
  /* Regional_Product_Information */
  eps_boolean SUBSETTED_PRODUCT; /* Set when product has been subsetted 
                                    (e.g. geographically subsetted usin
                                    g a region of interest filter). Imp
                                    lies the presence of one or more UM
                                    ARF GIADRs in GAD section for produ
                                    ct retrieved from UMARF. */
} t_mphr;

int read_mphr( t_mphr *X, buffer* B, int n );

int write_mphr( t_mphr *X, buffer* B, int n );

int free_mphr( t_mphr *X, int n );

int init_mphr( t_mphr *X, int n );

int size_mphr( t_mphr *X );

typedef struct {
  /* QUALITY */
  eps_byte SRC_DATA_QUAL[2]; /* Flag to specify which combination of au
                                xiliary data is used for the scenes ana
                                lysis. */
  /* VARIABLE_PARAMETERS */
  eps_integer EARTH_VIEWS_PER_SCANLINE; /* Number of Earth views per sc
                                           anline (2048 for Full, 409 f
                                           or GAC data) */
  eps_integer NAV_SAMPLE_RATE; /* Rate of sampling the navigation data 
                                  (angular relation and earth location)
                                  . For Full data every 40th or 20th Ea
                                  rth view. For GAC data every 8th or 4
                                  th Earth view */
} t_sphr;

int read_sphr( t_sphr *X, buffer* B, int n );

int write_sphr( t_sphr *X, buffer* B, int n );

int free_sphr( t_sphr *X, int n );

int init_sphr( t_sphr *X, int n );

int size_sphr( t_sphr *X );

typedef struct {
  eps_uinteger1 TARGET_RECORD_CLASS; /* Class of target record as deriv
                                        ed from the GRH of target recor
                                        d */
  eps_uinteger1 TARGET_INSTRUMENT_GROUP; /* Instrument group of target 
                                            record as derived from the 
                                            GRH of target record */
  eps_uinteger1 TARGET_RECORD_SUBCLASS; /* Subclass of target record as 
                                           derived from the GRH of targ
                                           et record */
  eps_uinteger4 TARGET_RECORD_OFFSET; /* Offset of target record from s
                                         tart of product */
} t_ipr;

int read_ipr( t_ipr *X, buffer* B, int n );

int write_ipr( t_ipr *X, buffer* B, int n );

int free_ipr( t_ipr *X, int n );

int init_ipr( t_ipr *X, int n );

int size_ipr( t_ipr *X );

typedef struct {
  /* RADIANCE_CONVERSION */
  eps_byte RAMP_CALIBRATION_COEFFICIENT[2]; /* Ramp/Auto Calibration In
                                               dicators Bit Field */
  eps_uinteger2 YEAR_RECENT_CALIBRATION; /* Year of Most Recent Solar C
                                            hannel Calibration (e.g. 19
                                            99) */
  eps_uinteger2 DAY_RECENT_CALIBRATION; /* Day of Year of Most Recent S
                                           olar Channel Calibration (e.
                                           g. 365) */
  eps_uinteger2 PRIMARY_CALIBRATION_ALGORITHM_ID; /* Primary Calibratio
                                                     n Algorithm ID */
  eps_byte PRIMARY_CALIBRATION_ALGORITHM_OPTION[2]; /* Primary Calibrat
                                                       ion Algorithm Se
                                                       lected Options */
  eps_uinteger2 SECONDARY_CALIBRATION_ALGORITHM_ID; /* Secondary Calibr
                                                       ation Algorithm 
                                                       ID */
  eps_byte SECONDARY_CALIBRATION_ALGORITHM_OPTION[2]; /* Secondary Cali
                                                         bration Algori
                                                         thm Selected O
                                                         ptions */
  eps_integer2 IR_TEMPERATURE1_COEFFICIENT1; /* IR Target Temp 1 Conver
                                                sion Coefficient 1 */
  eps_integer2 IR_TEMPERATURE1_COEFFICIENT2; /* IR Target Temp 1 Conver
                                                sion Coefficient 2 */
  eps_integer2 IR_TEMPERATURE1_COEFFICIENT3; /* IR Target Temp 1 Conver
                                                sion Coefficient 3 */
  eps_integer2 IR_TEMPERATURE1_COEFFICIENT4; /* IR Target Temp 1 Conver
                                                sion Coefficient 4 */
  eps_integer2 IR_TEMPERATURE1_COEFFICIENT5; /* IR Target Temp 1 Conver
                                                sion Coefficient 5 */
  eps_integer2 IR_TEMPERATURE1_COEFFICIENT6; /* IR Target Temp 1 Conver
                                                sion Coefficient 6 */
  eps_integer2 IR_TEMPERATURE2_COEFFICIENT1; /* IR Target Temp 2 Conver
                                                sion Coefficient 1 */
  eps_integer2 IR_TEMPERATURE2_COEFFICIENT2; /* IR Target Temp 2 Conver
                                                sion Coefficient 2 */
  eps_integer2 IR_TEMPERATURE2_COEFFICIENT3; /* IR Target Temp 2 Conver
                                                sion Coefficient 3 */
  eps_integer2 IR_TEMPERATURE2_COEFFICIENT4; /* IR Target Temp 2 Conver
                                                sion Coefficient 4 */
  eps_integer2 IR_TEMPERATURE2_COEFFICIENT5; /* IR Target Temp 2 Conver
                                                sion Coefficient 5 */
  eps_integer2 IR_TEMPERATURE2_COEFFICIENT6; /* IR Target Temp 2 Conver
                                                sion Coefficient 6 */
  eps_integer2 IR_TEMPERATURE3_COEFFICIENT1; /* IR Target Temp 3 Conver
                                                sion Coefficient 1 */
  eps_integer2 IR_TEMPERATURE3_COEFFICIENT2; /* IR Target Temp 3 Conver
                                                sion Coefficient 2 */
  eps_integer2 IR_TEMPERATURE3_COEFFICIENT3; /* IR Target Temp 3 Conver
                                                sion Coefficient 3 */
  eps_integer2 IR_TEMPERATURE3_COEFFICIENT4; /* IR Target Temp 3 Conver
                                                sion Coefficient 4 */
  eps_integer2 IR_TEMPERATURE3_COEFFICIENT5; /* IR Target Temp 3 Conver
                                                sion Coefficient 5 */
  eps_integer2 IR_TEMPERATURE3_COEFFICIENT6; /* IR Target Temp 3 Conver
                                                sion Coefficient 6 */
  eps_integer2 IR_TEMPERATURE4_COEFFICIENT1; /* IR Target Temp 4 Conver
                                                sion Coefficient 1 */
  eps_integer2 IR_TEMPERATURE4_COEFFICIENT2; /* IR Target Temp 4 Conver
                                                sion Coefficient 2 */
  eps_integer2 IR_TEMPERATURE4_COEFFICIENT3; /* IR Target Temp 4 Conver
                                                sion Coefficient 3 */
  eps_integer2 IR_TEMPERATURE4_COEFFICIENT4; /* IR Target Temp 4 Conver
                                                sion Coefficient 4 */
  eps_integer2 IR_TEMPERATURE4_COEFFICIENT5; /* IR Target Temp 4 Conver
                                                sion Coefficient 5 */
  eps_integer2 IR_TEMPERATURE4_COEFFICIENT6; /* IR Target Temp 4 Conver
                                                sion Coefficient 6 */
  eps_integer2 CH1_SOLAR_FILTERED_IRRADIANCE; /* Ch 1 Solar Filtered Ir
                                                 radiance in Wavelength 
                                                 */
  eps_integer2 CH1_EQUIVALENT_FILTER_WIDTH; /* Ch 1 Equivalent Filter W
                                               idth in Wavelength */
  eps_integer2 CH2_SOLAR_FILTERED_IRRADIANCE; /* Ch 2 Solar Filtered Ir
                                                 radiance in Wavelength 
                                                 */
  eps_integer2 CH2_EQUIVALENT_FILTER_WIDTH; /* Ch 2 Equivalent Filter W
                                               idth in Wavelength */
  eps_integer2 CH3A_SOLAR_FILTERED_IRRADIANCE; /* Ch 3a Solar Filtered 
                                                  Irradiance in Wavelen
                                                  gth */
  eps_integer2 CH3A_EQUIVALENT_FILTER_WIDTH; /* Ch 3a Equivalent Filter 
                                                Width in Wavelength */
  eps_integer4 CH3B_CENTRAL_WAVENUMBER; /* Ch 3b Central Wavenumber */
  eps_integer4 CH3B_CONSTANT1; /* Ch 3b Constant 1 for band correction 
                                  */
  eps_integer4 CH3B_CONSTANT2_SLOPE; /* Ch 3b Constant 2 slope for band 
                                        correction */
  eps_integer4 CH4_CENTRAL_WAVENUMBER; /* Ch 4 Central Wavenumber */
  eps_integer4 CH4_CONSTANT1; /* Ch 4 Constant 1 for band correction */
  eps_integer4 CH4_CONSTANT2_SLOPE; /* Ch 4 Constant 2 slope for band c
                                       orrection */
  eps_integer4 CH5_CENTRAL_WAVENUMBER; /* Ch 5 Central Wavenumber */
  eps_integer4 CH5_CONSTANT1; /* Ch 5 Constant 1 for band correction */
  eps_integer4 CH5_CONSTANT2_SLOPE; /* Ch 5 Constant 2 slope for band c
                                       orrection */
} t_giadr_radiance;

int read_giadr_radiance( t_giadr_radiance *X, buffer* B, int n );

int write_giadr_radiance( t_giadr_radiance *X, buffer* B, int n );

int free_giadr_radiance( t_giadr_radiance *X, int n );

int init_giadr_radiance( t_giadr_radiance *X, int n );

int size_giadr_radiance( t_giadr_radiance *X );

typedef struct {
  /* A_D_CONVERSION */
  eps_integer2 PATCH_TEMPERATURE_COEFFICIENT1; /* Patch Temperature Con
                                                  version Coefficient */
  eps_integer2 PATCH_TEMPERATURE_COEFFICIENT2; /* Patch Temperature Con
                                                  version Coefficient */
  eps_integer2 PATCH_TEMPERATURE_COEFFICIENT3; /* Patch Temperature Con
                                                  version Coefficient */
  eps_integer2 PATCH_TEMPERATURE_COEFFICIENT4; /* Patch Temperature Con
                                                  version Coefficient */
  eps_integer2 PATCH_TEMPERATURE_COEFFICIENT5; /* Patch Temperature Con
                                                  version Coefficient */
  eps_integer2 PATCH_TEMPERATURE_EXTENDED_COEFFICIENT1; /* Patch Temper
                                                           ature Extend
                                                           ed Conversio
                                                           n Coefficien
                                                           t */
  eps_integer2 PATCH_TEMPERATURE_EXTENDED_COEFFICIENT2; /* Patch Temper
                                                           ature Extend
                                                           ed Conversio
                                                           n Coefficien
                                                           t */
  eps_integer2 PATCH_TEMPERATURE_EXTENDED_COEFFICIENT3; /* Patch Temper
                                                           ature Extend
                                                           ed Conversio
                                                           n Coefficien
                                                           t */
  eps_integer2 PATCH_TEMPERATURE_EXTENDED_COEFFICIENT4; /* Patch Temper
                                                           ature Extend
                                                           ed Conversio
                                                           n Coefficien
                                                           t */
  eps_integer2 PATCH_TEMPERATURE_EXTENDED_COEFFICIENT5; /* Patch Temper
                                                           ature Extend
                                                           ed Conversio
                                                           n Coefficien
                                                           t */
  eps_integer2 PATCH_POWER_COEFFICIENT1; /* Patch Power Conversion Coef
                                            ficient */
  eps_integer2 PATCH_POWER_COEFFICIENT2; /* Patch Power Conversion Coef
                                            ficient */
  eps_integer2 PATCH_POWER_COEFFICIENT3; /* Patch Power Conversion Coef
                                            ficient */
  eps_integer2 PATCH_POWER_COEFFICIENT4; /* Patch Power Conversion Coef
                                            ficient */
  eps_integer2 PATCH_POWER_COEFFICIENT5; /* Patch Power Conversion Coef
                                            ficient */
  eps_integer2 RADIATOR_TEMPERATURE_COEFFICIENT1; /* Radiator Temperatu
                                                     re Conversion Coef
                                                     ficient */
  eps_integer2 RADIATOR_TEMPERATURE_COEFFICIENT2; /* Radiator Temperatu
                                                     re Conversion Coef
                                                     ficient */
  eps_integer2 RADIATOR_TEMPERATURE_COEFFICIENT3; /* Radiator Temperatu
                                                     re Conversion Coef
                                                     ficient */
  eps_integer2 RADIATOR_TEMPERATURE_COEFFICIENT4; /* Radiator Temperatu
                                                     re Conversion Coef
                                                     ficient */
  eps_integer2 RADIATOR_TEMPERATURE_COEFFICIENT5; /* Radiator Temperatu
                                                     re Conversion Coef
                                                     ficient */
  eps_integer2 BLACKBODY_TEMPERATURE1_COEFFICIENT1; /* Blackbody Temper
                                                       ature 1 Conversi
                                                       on Coefficient */
  eps_integer2 BLACKBODY_TEMPERATURE1_COEFFICIENT2; /* Blackbody Temper
                                                       ature 1 Conversi
                                                       on Coefficient */
  eps_integer2 BLACKBODY_TEMPERATURE1_COEFFICIENT3; /* Blackbody Temper
                                                       ature 1 Conversi
                                                       on Coefficient */
  eps_integer2 BLACKBODY_TEMPERATURE1_COEFFICIENT4; /* Blackbody Temper
                                                       ature 1 Conversi
                                                       on Coefficient */
  eps_integer2 BLACKBODY_TEMPERATURE1_COEFFICIENT5; /* Blackbody Temper
                                                       ature 1 Conversi
                                                       on Coefficient */
  eps_integer2 BLACKBODY_TEMPERATURE2_COEFFICIENT1; /* Blackbody Temper
                                                       ature 2 Conversi
                                                       on Coefficient */
  eps_integer2 BLACKBODY_TEMPERATURE2_COEFFICIENT2; /* Blackbody Temper
                                                       ature 2 Conversi
                                                       on Coefficient */
  eps_integer2 BLACKBODY_TEMPERATURE2_COEFFICIENT3; /* Blackbody Temper
                                                       ature 2 Conversi
                                                       on Coefficient */
  eps_integer2 BLACKBODY_TEMPERATURE2_COEFFICIENT4; /* Blackbody Temper
                                                       ature 2 Conversi
                                                       on Coefficient */
  eps_integer2 BLACKBODY_TEMPERATURE2_COEFFICIENT5; /* Blackbody Temper
                                                       ature 2 Conversi
                                                       on Coefficient */
  eps_integer2 BLACKBODY_TEMPERATURE3_COEFFICIENT1; /* Blackbody Temper
                                                       ature 3 Conversi
                                                       on Coefficient */
  eps_integer2 BLACKBODY_TEMPERATURE3_COEFFICIENT2; /* Blackbody Temper
                                                       ature 3 Conversi
                                                       on Coefficient */
  eps_integer2 BLACKBODY_TEMPERATURE3_COEFFICIENT3; /* Blackbody Temper
                                                       ature 3 Conversi
                                                       on Coefficient */
  eps_integer2 BLACKBODY_TEMPERATURE3_COEFFICIENT4; /* Blackbody Temper
                                                       ature 3 Conversi
                                                       on Coefficient */
  eps_integer2 BLACKBODY_TEMPERATURE3_COEFFICIENT5; /* Blackbody Temper
                                                       ature 3 Conversi
                                                       on Coefficient */
  eps_integer2 BLACKBODY_TEMPERATURE4_COEFFICIENT1; /* Blackbody Temper
                                                       ature 4 Conversi
                                                       on Coefficient */
  eps_integer2 BLACKBODY_TEMPERATURE4_COEFFICIENT2; /* Blackbody Temper
                                                       ature 4 Conversi
                                                       on Coefficient */
  eps_integer2 BLACKBODY_TEMPERATURE4_COEFFICIENT3; /* Blackbody Temper
                                                       ature 4 Conversi
                                                       on Coefficient */
  eps_integer2 BLACKBODY_TEMPERATURE4_COEFFICIENT4; /* Blackbody Temper
                                                       ature 4 Conversi
                                                       on Coefficient */
  eps_integer2 BLACKBODY_TEMPERATURE4_COEFFICIENT5; /* Blackbody Temper
                                                       ature 4 Conversi
                                                       on Coefficient */
  eps_integer2 ELECTRONIC_CURRENT_COEFFICIENT1; /* Electronics Current 
                                                   Conversion Coefficie
                                                   nt */
  eps_integer2 ELECTRONIC_CURRENT_COEFFICIENT2; /* Electronics Current 
                                                   Conversion Coefficie
                                                   nt */
  eps_integer2 ELECTRONIC_CURRENT_COEFFICIENT3; /* Electronics Current 
                                                   Conversion Coefficie
                                                   nt */
  eps_integer2 ELECTRONIC_CURRENT_COEFFICIENT4; /* Electronics Current 
                                                   Conversion Coefficie
                                                   nt */
  eps_integer2 ELECTRONIC_CURRENT_COEFFICIENT5; /* Electronics Current 
                                                   Conversion Coefficie
                                                   nt */
  eps_integer2 MOTOR_CURRENT_COEFFICIENT1; /* Motor Current Conversion 
                                              Coefficient */
  eps_integer2 MOTOR_CURRENT_COEFFICIENT2; /* Motor Current Conversion 
                                              Coefficient */
  eps_integer2 MOTOR_CURRENT_COEFFICIENT3; /* Motor Current Conversion 
                                              Coefficient */
  eps_integer2 MOTOR_CURRENT_COEFFICIENT4; /* Motor Current Conversion 
                                              Coefficient */
  eps_integer2 MOTOR_CURRENT_COEFFICIENT5; /* Motor Current Conversion 
                                              Coefficient */
  eps_integer2 EARTH_SHIELD_POSITION_COEFFICIENT1; /* Earth Shield Posi
                                                      tion Conversion C
                                                      oefficient */
  eps_integer2 EARTH_SHIELD_POSITION_COEFFICIENT2; /* Earth Shield Posi
                                                      tion Conversion C
                                                      oefficient */
  eps_integer2 EARTH_SHIELD_POSITION_COEFFICIENT3; /* Earth Shield Posi
                                                      tion Conversion C
                                                      oefficient */
  eps_integer2 EARTH_SHIELD_POSITION_COEFFICIENT4; /* Earth Shield Posi
                                                      tion Conversion C
                                                      oefficient */
  eps_integer2 EARTH_SHIELD_POSITION_COEFFICIENT5; /* Earth Shield Posi
                                                      tion Conversion C
                                                      oefficient */
  eps_integer2 ELECTRONIC_TEMPERATURE_COEFFICIENT1; /* Electronics Temp
                                                       erature Conversi
                                                       on Coefficient */
  eps_integer2 ELECTRONIC_TEMPERATURE_COEFFICIENT2; /* Electronics Temp
                                                       erature Conversi
                                                       on Coefficient */
  eps_integer2 ELECTRONIC_TEMPERATURE_COEFFICIENT3; /* Electronics Temp
                                                       erature Conversi
                                                       on Coefficient */
  eps_integer2 ELECTRONIC_TEMPERATURE_COEFFICIENT4; /* Electronics Temp
                                                       erature Conversi
                                                       on Coefficient */
  eps_integer2 ELECTRONIC_TEMPERATURE_COEFFICIENT5; /* Electronics Temp
                                                       erature Conversi
                                                       on Coefficient */
  eps_integer2 COOLER_HOUSING_TEMPERATURE_COEFFICIENT1; /* Cooler Housi
                                                           ng Temperatu
                                                           re Conversio
                                                           n Coefficien
                                                           t */
  eps_integer2 COOLER_HOUSING_TEMPERATURE_COEFFICIENT2; /* Cooler Housi
                                                           ng Temperatu
                                                           re Conversio
                                                           n Coefficien
                                                           t */
  eps_integer2 COOLER_HOUSING_TEMPERATURE_COEFFICIENT3; /* Cooler Housi
                                                           ng Temperatu
                                                           re Conversio
                                                           n Coefficien
                                                           t */
  eps_integer2 COOLER_HOUSING_TEMPERATURE_COEFFICIENT4; /* Cooler Housi
                                                           ng Temperatu
                                                           re Conversio
                                                           n Coefficien
                                                           t */
  eps_integer2 COOLER_HOUSING_TEMPERATURE_COEFFICIENT5; /* Cooler Housi
                                                           ng Temperatu
                                                           re Conversio
                                                           n Coefficien
                                                           t */
  eps_integer2 BASEPLATE_TEMPERATURE_COEFFICIENT1; /* Baseplate Tempera
                                                      ture Conversion C
                                                      oefficient */
  eps_integer2 BASEPLATE_TEMPERATURE_COEFFICIENT2; /* Baseplate Tempera
                                                      ture Conversion C
                                                      oefficient */
  eps_integer2 BASEPLATE_TEMPERATURE_COEFFICIENT3; /* Baseplate Tempera
                                                      ture Conversion C
                                                      oefficient */
  eps_integer2 BASEPLATE_TEMPERATURE_COEFFICIENT4; /* Baseplate Tempera
                                                      ture Conversion C
                                                      oefficient */
  eps_integer2 BASEPLATE_TEMPERATURE_COEFFICIENT5; /* Baseplate Tempera
                                                      ture Conversion C
                                                      oefficient */
  eps_integer2 MOTOR_HOUSING_TEMPERATURE_COEFFICIENT1; /* Motor Housing 
                                                          Temperature C
                                                          onversion Coe
                                                          fficient */
  eps_integer2 MOTOR_HOUSING_TEMPERATURE_COEFFICIENT2; /* Motor Housing 
                                                          Temperature C
                                                          onversion Coe
                                                          fficient */
  eps_integer2 MOTOR_HOUSING_TEMPERATURE_COEFFICIENT3; /* Motor Housing 
                                                          Temperature C
                                                          onversion Coe
                                                          fficient */
  eps_integer2 MOTOR_HOUSING_TEMPERATURE_COEFFICIENT4; /* Motor Housing 
                                                          Temperature C
                                                          onversion Coe
                                                          fficient */
  eps_integer2 MOTOR_HOUSING_TEMPERATURE_COEFFICIENT5; /* Motor Housing 
                                                          Temperature C
                                                          onversion Coe
                                                          fficient */
  eps_integer2 AD_CONVERTER_TEMPERATURE_COEFFICIENT1; /* A/D Converter 
                                                         Temperature Co
                                                         nversion Coeff
                                                         icient */
  eps_integer2 AD_CONVERTER_TEMPERATURE_COEFFICIENT2; /* A/D Converter 
                                                         Temperature Co
                                                         nversion Coeff
                                                         icient */
  eps_integer2 AD_CONVERTER_TEMPERATURE_COEFFICIENT3; /* A/D Converter 
                                                         Temperature Co
                                                         nversion Coeff
                                                         icient */
  eps_integer2 AD_CONVERTER_TEMPERATURE_COEFFICIENT4; /* A/D Converter 
                                                         Temperature Co
                                                         nversion Coeff
                                                         icient */
  eps_integer2 AD_CONVERTER_TEMPERATURE_COEFFICIENT5; /* A/D Converter 
                                                         Temperature Co
                                                         nversion Coeff
                                                         icient */
  eps_integer2 DETECTOR4_BIAS_VOLTAGE_COEFFICIENT1; /* Detector #4 Bias 
                                                       Voltage Conversi
                                                       on Coefficient */
  eps_integer2 DETECTOR4_BIAS_VOLTAGE_COEFFICIENT2; /* Detector #4 Bias 
                                                       Voltage Conversi
                                                       on Coefficient */
  eps_integer2 DETECTOR4_BIAS_VOLTAGE_COEFFICIENT3; /* Detector #4 Bias 
                                                       Voltage Conversi
                                                       on Coefficient */
  eps_integer2 DETECTOR4_BIAS_VOLTAGE_COEFFICIENT4; /* Detector #4 Bias 
                                                       Voltage Conversi
                                                       on Coefficient */
  eps_integer2 DETECTOR4_BIAS_VOLTAGE_COEFFICIENT5; /* Detector #4 Bias 
                                                       Voltage Conversi
                                                       on Coefficient */
  eps_integer2 DETECTOR5_BIAS_VOLTAGE_COEFFICIENT1; /* Detector #5 Bias 
                                                       Voltage Conversi
                                                       on Coefficient */
  eps_integer2 DETECTOR5_BIAS_VOLTAGE_COEFFICIENT2; /* Detector #5 Bias 
                                                       Voltage Conversi
                                                       on Coefficient */
  eps_integer2 DETECTOR5_BIAS_VOLTAGE_COEFFICIENT3; /* Detector #5 Bias 
                                                       Voltage Conversi
                                                       on Coefficient */
  eps_integer2 DETECTOR5_BIAS_VOLTAGE_COEFFICIENT4; /* Detector #5 Bias 
                                                       Voltage Conversi
                                                       on Coefficient */
  eps_integer2 DETECTOR5_BIAS_VOLTAGE_COEFFICIENT5; /* Detector #5 Bias 
                                                       Voltage Conversi
                                                       on Coefficient */
  eps_integer2 CH3B_BLACKBODY_VIEW_COEFFICIENT1; /* Channel 3b Blackbod
                                                    y View Conversion C
                                                    oefficient */
  eps_integer2 CH3B_BLACKBODY_VIEW_COEFFICIENT2; /* Channel 3b Blackbod
                                                    y View Conversion C
                                                    oefficient */
  eps_integer2 CH3B_BLACKBODY_VIEW_COEFFICIENT3; /* Channel 3b Blackbod
                                                    y View Conversion C
                                                    oefficient */
  eps_integer2 CH3B_BLACKBODY_VIEW_COEFFICIENT4; /* Channel 3b Blackbod
                                                    y View Conversion C
                                                    oefficient */
  eps_integer2 CH3B_BLACKBODY_VIEW_COEFFICIENT5; /* Channel 3b Blackbod
                                                    y View Conversion C
                                                    oefficient */
  eps_integer2 CH4_BLACKBODY_VIEW_COEFFICIENT1; /* Channel 4 Blackbody 
                                                   View Conversion Coef
                                                   ficient */
  eps_integer2 CH4_BLACKBODY_VIEW_COEFFICIENT2; /* Channel 4 Blackbody 
                                                   View Conversion Coef
                                                   ficient */
  eps_integer2 CH4_BLACKBODY_VIEW_COEFFICIENT3; /* Channel 4 Blackbody 
                                                   View Conversion Coef
                                                   ficient */
  eps_integer2 CH4_BLACKBODY_VIEW_COEFFICIENT4; /* Channel 4 Blackbody 
                                                   View Conversion Coef
                                                   ficient */
  eps_integer2 CH4_BLACKBODY_VIEW_COEFFICIENT5; /* Channel 4 Blackbody 
                                                   View Conversion Coef
                                                   ficient */
  eps_integer2 CH5_BLACKBODY_VIEW_COEFFICIENT1; /* Channel 5 Blackbody 
                                                   View Conversion Coef
                                                   ficient */
  eps_integer2 CH5_BLACKBODY_VIEW_COEFFICIENT2; /* Channel 5 Blackbody 
                                                   View Conversion Coef
                                                   ficient */
  eps_integer2 CH5_BLACKBODY_VIEW_COEFFICIENT3; /* Channel 5 Blackbody 
                                                   View Conversion Coef
                                                   ficient */
  eps_integer2 CH5_BLACKBODY_VIEW_COEFFICIENT4; /* Channel 5 Blackbody 
                                                   View Conversion Coef
                                                   ficient */
  eps_integer2 CH5_BLACKBODY_VIEW_COEFFICIENT5; /* Channel 5 Blackbody 
                                                   View Conversion Coef
                                                   ficient */
  eps_integer2 REFERENCE_VOLTAGE_COEFFICIENT1; /* Reference Voltage Con
                                                  version Coefficient */
  eps_integer2 REFERENCE_VOLTAGE_COEFFICIENT2; /* Reference Voltage Con
                                                  version Coefficient */
  eps_integer2 REFERENCE_VOLTAGE_COEFFICIENT3; /* Reference Voltage Con
                                                  version Coefficient */
  eps_integer2 REFERENCE_VOLTAGE_COEFFICIENT4; /* Reference Voltage Con
                                                  version Coefficient */
  eps_integer2 REFERENCE_VOLTAGE_COEFFICIENT5; /* Reference Voltage Con
                                                  version Coefficient */
} t_giadr_analog;

int read_giadr_analog( t_giadr_analog *X, buffer* B, int n );

int write_giadr_analog( t_giadr_analog *X, buffer* B, int n );

int free_giadr_analog( t_giadr_analog *X, int n );

int init_giadr_analog( t_giadr_analog *X, int n );

int size_giadr_analog( t_giadr_analog *X );

typedef struct {
  /* GENERIC_QUALITY_INDICATORS */
  eps_boolean DEGRADED_INST_MDR; /* Quality of MDR has been degraded fr
                                    om nominal due to an instrument deg
                                    radation */
  eps_boolean DEGRADED_PROC_MDR; /* Quality of MDR has been degraded fr
                                    om nominal due to a processing degr
                                    adation */
  /* MEASUREMENT_DATA */
  eps_integer2 EARTH_VIEWS_PER_SCANLINE; /* Earth views per scanline, N
                                            E */
  eps_integer2 SCENE_RADIANCES[5][NE];
  /* NAVIGATION_DATA_AT_SCAN_LINE */
  eps_uinteger4 TIME_ATTITUDE; /* Time Associated with Attitude Angles 
                                  */
  eps_integer2 EULER_ANGLE[3];
  eps_byte NAVIGATION_STATUS[4]; /* Navigation Status Bit Field */
  eps_uinteger4 SPACECRAFT_ALTITUDE; /* Spacecraft Altitude Above Refer
                                        ence Geoid (MSL) */
  eps_integer2 ANGULAR_RELATIONS_FIRST[4];
  eps_integer2 ANGULAR_RELATIONS_LAST[4];
  eps_integer4 EARTH_LOCATION_FIRST[2];
  eps_integer4 EARTH_LOCATION_LAST[2];
  eps_integer2 NUM_NAVIGATION_POINTS; /* Number of navigation points in 
                                         angular_relations and earth_lo
                                         cations arrays, NP */
  eps_integer2 ANGULAR_RELATIONS[NP][4];
  eps_integer4 EARTH_LOCATIONS[NP][2];
  eps_byte QUALITY_INDICATOR[4]; /* Quality Indicator Bit Field */
  eps_byte SCAN_LINE_QUALITY[4]; /* Scan Line Quality Flags */
  eps_byte CALIBRATION_QUALITY[3][2];
  eps_uinteger2 COUNT_ERROR_FRAME; /* Count of Bit Errors in Frame Sync
                                      . This field is applicable only t
                                      o NOAA data. For Metop data it sh
                                      ould be defaulted to zero. */
  /* Calibration_Coefficients */
  eps_integer4 CH123A_CURVE_SLOPE1[3];
  eps_integer4 CH123A_CURVE_INTERCEPT1[3];
  eps_integer4 CH123A_CURVE_SLOPE2[3];
  eps_integer4 CH123A_CURVE_INTERCEPT2[3];
  eps_integer4 CH123A_CURVE_INTERCEPTION[3];
  eps_integer4 CH123A_TEST_CURVE_SLOPE1[3];
  eps_integer4 CH123A_TEST_CURVE_INTERCEPT1[3];
  eps_integer4 CH123A_TEST_CURVE_SLOPE2[3];
  eps_integer4 CH123A_TEST_CURVE_INTERCEPT2[3];
  eps_integer4 CH123A_TEST_CURVE_INTERCEPTION[3];
  eps_integer4 CH123A_PRELAUNCH_CURVE_SLOPE1[3];
  eps_integer4 CH123A_PRELAUNCH_CURVE_INTERCEPT1[3];
  eps_integer4 CH123A_PRELAUNCH_CURVE_SLOPE2[3];
  eps_integer4 CH123A_PRELAUNCH_CURVE_INTERCEPT2[3];
  eps_integer4 CH123A_PRELAUNCH_CURVE_INTERCEPTION[3];
  eps_integer4 CH3B45_SECOND_TERM[3];
  eps_integer4 CH3B45_FIRST_TERM[3];
  eps_integer4 CH3B45_ZEROTH_TERM[3];
  eps_integer4 CH3B45_TEST_SECOND_TERM[3];
  eps_integer4 CH3B45_TEST_FIRST_TERM[3];
  eps_integer4 CH3B45_TEST_ZEROTH_TERM[3];
  eps_byte CLOUD_INFORMATION[NE][2];
  /* DIGITAL_A_Telemetry */
  eps_uinteger2 FRAME_SYNCHRONISATION[6];
  eps_byte FRAME_INDICATOR[4]; /* Frame ID. This field is applicable on
                                  ly to NOAA data. For Metop data the a
                                  ll bits will be unset. */
  eps_byte TIME_CODE[8]; /* Time Code. This field is applicable only to 
                            NOAA data. For Metop data the all bits will 
                            be unset. */
  eps_uinteger2 RAMP_CALIB[5];
  eps_uinteger2 INTERNAL_TARGET_TEMPERATURE_COUNT[3];
  /* Digital_B_telemetry */
  eps_byte INSTRUMENT_INVALID_WORD_FLAG[2]; /* Invalid Word Bit Flags */
  eps_byte DIGITAL_B_DATA[2]; /* AVHRR Digital B Data */
  /* Analog_housekeeping_data */
  eps_byte INSTRUMENT_INVALID_ANALOG_WORD_FLAG[4]; /* Invalid Word Bit 
                                                      Flags */
  eps_uinteger2 PATCH_TEMPERATURE; /* Word 1: Patch Temperature */
  eps_uinteger2 PATCH_EXTENDED_TEMPERATURE; /* Word 2: Patch Temperatur
                                               e Extended */
  eps_uinteger2 PATCH_POWER; /* Word 3: Patch Power */
  eps_uinteger2 RADIATOR_TEMPERATURE; /* Word 4: Radiator Temperature */
  eps_uinteger2 BLACKBODY_TEMPERATURE1; /* Word 5: Black Body Temperatu
                                           re 1 */
  eps_uinteger2 BLACKBODY_TEMPERATURE2; /* Word 6: Black Body Temperatu
                                           re 2 */
  eps_uinteger2 BLACKBODY_TEMPERATURE3; /* Word 7: Black Body Temperatu
                                           re 3 */
  eps_uinteger2 BLACKBODY_TEMPERATURE4; /* Word 8: Black Body Temperatu
                                           re 4 */
  eps_uinteger2 ELECTRONIC_CURRENT; /* Word 9: Electronics Current */
  eps_uinteger2 MOTOR_CURRENT; /* Word 10: Motor Current */
  eps_uinteger2 EARTH_SHIELD_POSITION; /* Word 11: Earth Shield Positio
                                          n */
  eps_uinteger2 ELECTRONIC_TEMPERATURE; /* Word 12: Electronics Tempera
                                           ture */
  eps_uinteger2 COOLER_HOUSING_TEMPERATURE; /* Word 13: Cooler Housing 
                                               Temperature */
  eps_uinteger2 BASEPLATE_TEMPERATURE; /* Word 14: Baseplate Temperatur
                                          e */
  eps_uinteger2 MOTOR_HOUSING_TEMPERATURE; /* Word 15: Motor Housing Te
                                              mperature */
  eps_uinteger2 AD_CONVERTER_TEMPERATURE; /* Word 16: A/D Converter Tem
                                             perature */
  eps_uinteger2 DETECTOR4_VOLTAGE; /* Word 17: Detector #4 Bias Voltage 
                                      */
  eps_uinteger2 DETECTOR5_VOLTAGE; /* Word 18: Detector #5 Bias Voltage 
                                      */
  eps_uinteger2 CH3_BLACKBODY_VIEW; /* Word 19: Channel 3b Blackbody Vi
                                       ew */
  eps_uinteger2 CH4_BLACKBODY_VIEW; /* Word 20: Channel 4 Blackbody Vie
                                       w */
  eps_uinteger2 CH5_BLACKBODY_VIEW; /* Word 21: Channel 5 Blackbody Vie
                                       w */
  eps_uinteger2 REFERENCE_VOLTAGE; /* Word 22: Reference Voltage */
} t_mdr_1b;

int read_mdr_1b( t_mdr_1b *X, buffer* B, int n );

int write_mdr_1b( t_mdr_1b *X, buffer* B, int n );

int free_mdr_1b( t_mdr_1b *X, int n );

int init_mdr_1b( t_mdr_1b *X, int n );

int size_mdr_1b( t_mdr_1b *X );


typedef struct {
  REC_HEAD grh;
  union {
     t_mphr* MPHR;
     t_sphr* SPHR;
     t_ipr* IPR;
     t_giadr_radiance* GIADR_RADIANCE;
     t_giadr_analog* GIADR_ANALOG;
     t_mdr_1b* MDR_1B;
  } content;
} t_eps_avhrrl1b_6_5;

int get_eps_avhrrl1b_6_5( t_eps_avhrrl1b_6_5 *R, FILE *fp );
int put_eps_avhrrl1b_6_5( t_eps_avhrrl1b_6_5 *R, FILE *fp );
int free_eps_avhrrl1b_6_5( t_eps_avhrrl1b_6_5 *R );
int convert_avh1b_intf( int argc, char* argv[] );

#endif
