/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/5
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * VERSION   : 1.0
 *
 */
#ifndef EPS_GRH_H
#define EPS_GRH_H
#include "eps_types.h"
#include <stdio.h>
#ifdef HAS_LIBHDF5
#include <hdf5.h>
#endif

#define SIZE_OF_REC_HEAD 20

/*
  Generic record header
*/


typedef enum {
  RESERVED, MPHR, SPHR, IPR, GEADR, 
  GIADR, VEADR, VIADR, MDR
} RECORD_CLASS;

typedef enum {
  GENERIC, AMSU_A, ASCAT, ATOVS, AVHRR, 
  GOME, GRAS, HIRS, IASI, MHS, SEM, 
  ADCS, SBUV, DUMMY, ARCHIVE, IASIL2
} INSTRUMENT_GROUP;

typedef struct {
    eps_uinteger2 days;
    eps_uinteger4 milliseconds;
} ShortCDSTime;

typedef struct {
    eps_uinteger1 record_class;
    eps_uinteger1 instrument_group;
    eps_uinteger1 record_subclass;
    eps_uinteger1 record_subclass_version;
    eps_uinteger4 record_size;
    ShortCDSTime record_start;
    ShortCDSTime record_stop;
} REC_HEAD;

int read_grh( REC_HEAD *R, FILE* fpr );
int write_grh( REC_HEAD *R, FILE* fpw );

#ifdef HAS_LIBHDF5
int write_grh_hdf5( REC_HEAD *R, hid_t gid );
#endif

#endif
