#ifndef _MAIA_AVH1C_H
#define _MAIA_AVH1C_H
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "eps_types.h"
#include "eps_buffer.h"
#include "eps_grh.h"


extern int maia_avh1c_check_subclass;
extern int maia_avh1c_check_subclass_version;


typedef struct {
  eps_byte avh1c_h_siteid[3]; 		/* Data set creation site ID */
  eps_byte avh1c_h_cfill1[1];
  eps_byte avh1c_h_l1bsiteid[3]; 	/* creation site for original 1B data 
                                    	*/
  eps_byte avh1c_h_cfill2[1];
  eps_integer2 avh1c_h_versnb; 		/* level 1c format version number */
  eps_integer2 avh1c_h_versyr; 		/* level 1c format version year */
  eps_integer2 avh1c_h_versdy; 		/* level 1c format version day of year */
  eps_integer2 avh1c_h_hdrcnt; 		/* count of header records in data set */
  eps_byte avh1c_h_dataname[42];	/* data set name */
  eps_byte avh1c_h_cfill3[2];
  eps_integer2 avh1c_h_satid; 		/* satellite id (13=NOAA-16 ;14=NOAA-17) 
                                 	*/
  eps_integer2 avh1c_h_instid; 		/* Instrument id */
  eps_integer2 avh1c_h_datatyp; 	/* data type code */
  eps_integer2 avh1c_h_filler0;
  eps_integer4 avh1c_h_startdatajd; 	/* start of data set day from 00h,1 
                                       	Jan 1950 */
  eps_integer2 avh1c_h_startdatayr; 	/* start of data set year */
  eps_integer2 avh1c_h_startdatady; 	/* start of data set day of year */
  eps_integer4 avh1c_h_startdatatime; 	/* start of data set UTC time of 
                                         day in ms */
  eps_integer4 avh1c_h_enddatajd; 	/* end of data set day from 00h,1 Jan 
                                     	1950 */
  eps_integer2 avh1c_h_enddatayr; 	/* end of data set year */
  eps_integer2 avh1c_h_enddatady; 	/* end of data set day of year */
  eps_integer4 avh1c_h_enddatatime; 	/* end of data set UTC time of day 
                                       	in ms */
  eps_integer2 avh1c_h_startorbit;	/* orbit number */
  eps_integer2 avh1c_h_filler1[3];

  eps_integer4 avh1c_h_inststat1; 	/* first instrument status */
  eps_integer2 avh1c_h_filler2;
  eps_integer2 avh1c_h_statchrecnb; 	/* record number of status change  
                                       	*/
  eps_integer4 avh1c_h_inststat2; 	/* second instrument status  */
  eps_integer2 avh1c_h_scnlin; 		/* count of scan lines in this data set 
                                  	*/
  eps_integer2 avh1c_h_callocscnlin; 	/* count of calibrated, earth loca
                                        ted */
  eps_integer2 avh1c_h_filler5[4];
  eps_integer4 avh1c_h_rascnod; 	/* 10^5 x right ascension of the ascend
                                   	ing node */
  eps_integer2 avh1c_h_rollerr; 	/* 10^3 x constant roll attitude error 
                                   	*/
  eps_integer2 avh1c_h_pitcherr; 	/* 10^3 x onstant pitch attitude error 
                                    	*/
  eps_integer2 avh1c_h_yawerr; 		/* 10^3 x constant yaw attitude error */
  eps_integer2 avh1c_h_filler6;
  eps_byte avh1c_h_pfsdataname[68];	/* pfs data name */
  eps_integer4 avh1c_h_filler[7306];
} t_header;

int read_header( t_header *X, buffer* B, int n );

int write_header( t_header *X, buffer* B, int n );

int free_header( t_header *X, int n );

int init_header( t_header *X, int n );

int size_header( t_header *X );

typedef struct {
  eps_integer2 avh1c_scnlin; /* scan line number */
  eps_integer2 avh1c_scnlinyr; /* scan line year */
  eps_integer2 avh1c_scnlindy; /* scan line day of year */
  eps_integer2 avh1c_filler; /* alignment */
  eps_integer4 avh1c_scnlintime; /* scan line UTC time of day in millis
                                    econds */
  eps_integer2 avh1c_scnlinbit; /* scan line bit field */
  eps_integer2 avh1c_filler0[1];
  eps_integer4 avh1c_qualind; /* quality indicator bit field */
  eps_integer4 avh1c_scnlinqual; /* scan line quality flags */
  eps_integer2 avh1c_calqual[3];
  eps_integer2 avh1c_spare1;
  eps_integer2 avh1c_scalti; /*  10 x Spacecraft Altitude (MSL) in km */
  eps_integer2 avh1c_ang[51][3];
  eps_integer4 avh1c_pos[51][2];
  eps_integer4 avh1c_spare4;
  eps_integer4 avh1c_maskind;
  eps_integer2 avh1c_btemps[2048][5];
  eps_integer2 avh1c_mask[2048];
  eps_integer2 avh1c_tsurf[2048];
} t_record;

int read_record( t_record *X, buffer* B, int n );

int write_record( t_record *X, buffer* B, int n );

int free_record( t_record *X, int n );

int init_record( t_record *X, int n );

int size_record( t_record *X );

#endif
