/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/5
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * VERSION   : 1.0
 *
 */
#ifndef _EPS_METOPL0_H
#define _EPS_METOPL0_H
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "eps_types.h"
#include "eps_buffer.h"
#include "eps_grh.h"


extern int eps_metopl0_check_subclass;
extern int eps_metopl0_check_subclass_version;


typedef struct {
  /* Product_Details */
  eps_byte PRODUCT_NAME[67]; /* Complete name of the product */
  eps_byte PARENT_PRODUCT_NAME_1[67]; /* Name of the parent product fro
                                         m which this product has been 
                                         produced. For Level 0 products
                                         , this field is filled with lo
                                         wer case x's. */
  eps_byte PARENT_PRODUCT_NAME_2[67]; /* Name of the parent product fro
                                         m which this product has been 
                                         produced. For Level 0 products 
                                         or products for which this is 
                                         not appropriate, this field is 
                                         filled with lower case x's. */
  eps_byte PARENT_PRODUCT_NAME_3[67]; /* Name of the parent product fro
                                         m which this product has been 
                                         produced. For Level 0 products 
                                         or products for which this is 
                                         not appropriate, this field is 
                                         filled with lower case x's. */
  eps_byte PARENT_PRODUCT_NAME_4[67]; /* Name of the parent product fro
                                         m which this product has been 
                                         produced. For Level 0 products 
                                         or products for which this is 
                                         not appropriate, this field is 
                                         filled with lower case x's. */
  eps_byte INSTRUMENT_ID[4]; /* Instrument identification */
  eps_byte INSTRUMENT_MODEL[3]; /* Instrument Model identification */
  eps_byte PRODUCT_TYPE[3]; /* Product Type */
  eps_byte PROCESSING_LEVEL[2]; /* Processing Level Identification */
  eps_byte SPACECRAFT_ID[3]; /* Spacecraft identification */
  eps_gentime SENSING_START; /* UTC Time of start of sensing data in th
                                is object (PDU, ROI or Full Product) */
  eps_gentime SENSING_END; /* UTC Time of end of sensing data in this o
                              bject (PDU, ROI or Full Product) */
  eps_gentime SENSING_START_THEORETICAL; /* Theoretical UTC Time of sta
                                            rt of sensing data in the d
                                            ump from which this object 
                                            is derived. This data is th
                                            e predicted start time at t
                                            he MPF level. */
  eps_gentime SENSING_END_THEORETICAL; /* Theoretical UTC Time of end o
                                          f sensing data in the dump fr
                                          om which this object is deriv
                                          ed. This data is the predicte
                                          d end time at the MPF level. 
                                          */
  eps_byte PROCESSING_CENTRE[4]; /* Processing Centre Identification */
  eps_uinteger PROCESSOR_MAJOR_VERSION; /* Processing chain major versi
                                           on number */
  eps_uinteger PROCESSOR_MINOR_VERSION; /* Processing chain minor versi
                                           on number */
  eps_uinteger FORMAT_MAJOR_VERSION; /* Dataset Format Major Version nu
                                        mber */
  eps_uinteger FORMAT_MINOR_VERSION; /* Dataset Format Minor Version nu
                                        mber */
  eps_gentime PROCESSING_TIME_START; /* UTC time of the processing at s
                                        tart of processing for the prod
                                        uct */
  eps_gentime PROCESSING_TIME_END; /* UTC time of the processing at end 
                                      of processing for the product */
  eps_byte PROCESSING_MODE[1]; /* Identification of the mode of process
                                  ing */
  eps_byte DISPOSITION_MODE[1]; /* Identification of the diposition mod
                                   e */
  eps_byte RECEIVING_GROUND_STATION[3]; /* Acquisition Station Identifi
                                           cation */
  eps_gentime RECEIVE_TIME_START; /* UTC time of the reception at CDA f
                                     or first Data Item */
  eps_gentime RECEIVE_TIME_END; /* UTC time of the reception at CDA for 
                                   last Data Item */
  eps_uinteger ORBIT_START; /* Start Orbit Number, counted incrementall
                               y since launch */
  eps_uinteger ORBIT_END; /* Stop Orbit Number */
  eps_uinteger ACTUAL_PRODUCT_SIZE; /* Size of the complete product */
  /* ASCENDING_NODE_ORBIT_PARAMETERS */
  eps_longgentime STATE_VECTOR_TIME; /* Epoch time (in UTC) of the orbi
                                        tal elements and the orbit stat
                                        e vector. this corresponds to t
                                        he time of crossing the ascendi
                                        ng node for ORBIT_START */
  eps_integer SEMI_MAJOR_AXIS; /* Semi major axis of orbit at time of t
                                  he ascending node crossing. */
  eps_integer ECCENTRICITY; /* Orbit eccentricity at time of the ascend
                               ing node crossing */
  eps_integer INCLINATION; /* Orbit inclination at time of the ascendin
                              g node crossing */
  eps_integer PERIGEE_ARGUMENT; /* Argument of perigee at time of the a
                                   scending node crossing */
  eps_integer RIGHT_ASCENSION; /* Right ascension at time of the ascend
                                  ing node crossing */
  eps_integer MEAN_ANOMALY; /* Mean anomaly at time of the ascending no
                               de crossing */
  eps_integer X_POSITION; /* X position of the orbit state vector in th
                             e orbit frame at ascending node */
  eps_integer Y_POSITION; /* Y position of the orbit state vector in th
                             e orbit frame at ascending node */
  eps_integer Z_POSITION; /* Z position of the orbit state vector in th
                             e orbit frame at ascending node */
  eps_integer X_VELOCITY; /* X velocity of the orbit state vector in th
                             e orbit frame at ascending node */
  eps_integer Y_VELOCITY; /* Y velocity of the orbit state vector in th
                             e orbit frame at ascending node */
  eps_integer Z_VELOCITY; /* Z velocity of the orbit state vector in th
                             e orbit frame at ascending node */
  eps_integer EARTH_SUN_DISTANCE_RATIO; /* Earth-Sun distance ratio - r
                                           atio of current Eart-Sun dis
                                           tance to Mean Earth-Sun dist
                                           ance */
  eps_integer LOCATION_TOLERANCE_RADIAL; /* Nadir Earth location tolera
                                            nce radial */
  eps_integer LOCATION_TOLERANCE_CROSSTRACK; /* Nadir Earth location to
                                                lerance cross-track */
  eps_integer LOCATION_TOLERANCE_ALONGTRACK; /* Nadir Earth location to
                                                lerance along-track */
  eps_integer YAW_ERROR; /* Constant Yaw attitude error */
  eps_integer ROLL_ERROR; /* Constant Roll attitude error */
  eps_integer PITCH_ERROR; /* Constant Pitch attitude error */
  /* LOCATION_SUMMARY */
  eps_integer SUBSAT_LATITUDE_START; /* Latitude of sub-satellite point 
                                        at start of the data set */
  eps_integer SUBSAT_LONGITUDE_START; /* Longitude of sub-satellite poi
                                         nt at start of the data set */
  eps_integer SUBSAT_LATITUDE_END; /* Latitude of sub-satellite point a
                                      t end of the data set */
  eps_integer SUBSAT_LONGITUDE_END; /* Longitude of sub-satellite point 
                                       at end of the data set */
  /* Leap_Second_Information */
  eps_integer LEAP_SECOND; /* Occurence of Leap second within the produ
                              ct. Field is set to -1, 0 or +1 dependent 
                              upon occurrence of leap second and direct
                              ion. */
  eps_gentime LEAP_SECOND_UTC; /* UTC time of occurrence of the Leap Se
                                  cond (If no leap second in the produc
                                  t, value is null) */
  /* Record_counts */
  eps_uinteger TOTAL_RECORDS; /* Total count of all records in the prod
                                 uct */
  eps_uinteger TOTAL_MPHR; /* Total count of all MPHRs in product (shou
                              ld always be 1!) */
  eps_uinteger TOTAL_SPHR; /* Total count of all SPHRs in product (shou
                              ld be 0 or 1 only) */
  eps_uinteger TOTAL_IPR; /* Total count of all IPRs in the product */
  eps_uinteger TOTAL_GEADR; /* Total count of all GEADRs in the product 
                               */
  eps_uinteger TOTAL_GIADR; /* Total count of all GIADRs in the product 
                               */
  eps_uinteger TOTAL_VEADR; /* Total count of all VEADRs in the product 
                               */
  eps_uinteger TOTAL_VIADR; /* Total count of all VIADRs in the product 
                               */
  eps_uinteger TOTAL_MDR; /* Total count of all MDRs in the product */
  /* Record_Based_Generic_Quality_Flags */
  eps_uinteger COUNT_DEGRADED_INST_MDR; /* Count of MDRs with degradati
                                           on due to instrument problem
                                           s */
  eps_uinteger COUNT_DEGRADED_PROC_MDR; /* Count of MDRs with degradati
                                           on due to processing problem
                                           s */
  eps_uinteger COUNT_DEGRADED_INST_MDR_BLOCKS; /* Count of the number o
                                                  f blocks of MDRs degr
                                                  aded due to degraded 
                                                  instrument */
  eps_uinteger COUNT_DEGRADED_PROC_MDR_BLOCKS; /* Count of the number o
                                                  f blocks of MDRs degr
                                                  aded due to degraded 
                                                  processing */
  /* Time_Based_Generic_Quality_Flags */
  eps_uinteger DURATION_OF_PRODUCT; /* The duration of the product in m
                                       illiseconds */
  eps_uinteger MILLISECONDS_OF_DATA_PRESENT; /* The total amount of dat
                                                a present in the produc
                                                t */
  eps_uinteger MILLISECONDS_OF_DATA_MISSING; /* The total amount of dat
                                                a missing from the prod
                                                cut */
  /* Regional_Product_Information */
  eps_boolean SUBSETTED_PRODUCT; /* Set when product has been subsetted 
                                    (e.g. geographically subsetted usin
                                    g a region of interest filter). Imp
                                    lies the presence of one or more UM
                                    ARF GIADRs in GAD section for produ
                                    ct retrieved from UMARF. */
} t_mphr;

int read_mphr( t_mphr *X, buffer* B, int n );

int write_mphr( t_mphr *X, buffer* B, int n );

int free_mphr( t_mphr *X, int n );

int init_mphr( t_mphr *X, int n );

int size_mphr( t_mphr *X );

typedef struct {
  /* Product_details */
  eps_byte AUX_DATA_POINTER[100]; /* Unique pointer to auxiliary datase
                                     t */
} t_geadr_0;

int read_geadr_0( t_geadr_0 *X, buffer* B, int n );

int write_geadr_0( t_geadr_0 *X, buffer* B, int n );

int free_geadr_0( t_geadr_0 *X, int n );

int init_geadr_0( t_geadr_0 *X, int n );

int size_geadr_0( t_geadr_0 *X );

typedef struct {
  eps_uinteger1 TARGET_RECORD_CLASS; /* Class of target record as deriv
                                        ed from the GRH of target recor
                                        d */
  eps_uinteger1 TARGET_INSTRUMENT_GROUP; /* Instrument group of target 
                                            record as derived from the 
                                            GRH of target record */
  eps_uinteger1 TARGET_RECORD_SUBCLASS; /* Subclass of target record as 
                                           derived from the GRH of targ
                                           et record */
  eps_uinteger4 TARGET_RECORD_OFFSET; /* Offset of target record from s
                                         tart of product */
} t_ipr_0;

int read_ipr_0( t_ipr_0 *X, buffer* B, int n );

int write_ipr_0( t_ipr_0 *X, buffer* B, int n );

int free_ipr_0( t_ipr_0 *X, int n );

int init_ipr_0( t_ipr_0 *X, int n );

int size_ipr_0( t_ipr_0 *X );

typedef struct {
  eps_uinteger2 days;
  eps_uinteger4 msec;
  eps_uinteger2 usec;
} t_viadr_0_UTC_OBT_TIME;

int read_viadr_0_UTC_OBT_TIME( t_viadr_0_UTC_OBT_TIME *X, buffer* B, int n );

int write_viadr_0_UTC_OBT_TIME( t_viadr_0_UTC_OBT_TIME *X, buffer* B, int n );

int free_viadr_0_UTC_OBT_TIME( t_viadr_0_UTC_OBT_TIME *X, int n );

int init_viadr_0_UTC_OBT_TIME( t_viadr_0_UTC_OBT_TIME *X, int n );

int size_viadr_0_UTC_OBT_TIME( t_viadr_0_UTC_OBT_TIME *X );

typedef struct {
  eps_uinteger2 not_used;
  eps_uinteger4 rtime;
} t_viadr_0_OBT_UTC_TIME;

int read_viadr_0_OBT_UTC_TIME( t_viadr_0_OBT_UTC_TIME *X, buffer* B, int n );

int write_viadr_0_OBT_UTC_TIME( t_viadr_0_OBT_UTC_TIME *X, buffer* B, int n );

int free_viadr_0_OBT_UTC_TIME( t_viadr_0_OBT_UTC_TIME *X, int n );

int init_viadr_0_OBT_UTC_TIME( t_viadr_0_OBT_UTC_TIME *X, int n );

int size_viadr_0_OBT_UTC_TIME( t_viadr_0_OBT_UTC_TIME *X );

typedef struct {
  /* METOP_OBT_to_UTC_correlation_parameters */
  t_viadr_0_UTC_OBT_TIME UTC_OBT_TIME; /* UTC time corresponding to OBT 
                                          */
  t_viadr_0_OBT_UTC_TIME OBT_UTC_TIME; /* OBT time corresponding to UTC 
                                          */
  eps_uinteger2 CLOCK_STEP[2];
} t_viadr_0;

int read_viadr_0( t_viadr_0 *X, buffer* B, int n );

int write_viadr_0( t_viadr_0 *X, buffer* B, int n );

int free_viadr_0( t_viadr_0 *X, int n );

int init_viadr_0( t_viadr_0 *X, int n );

int size_viadr_0( t_viadr_0 *X );

typedef struct {
  /* GENERIC_QUALITY_INDICATORS */
  eps_boolean DEGRADED_INST_MDR; /* Quality of MDR has been degraded fr
                                    om nominal due to an instrument deg
                                    radation */
  eps_boolean DEGRADED_PROC_MDR; /* Quality of MDR has been degraded fr
                                    om nominal due to a processing degr
                                    adation */
  /* MEASUREMENT_DATA */
  eps_uinteger4 SIZE_INST_DATA;
  eps_integer1* INST_DATA;
} t_mdr_0;

int read_mdr_0( t_mdr_0 *X, buffer* B, int n );

int write_mdr_0( t_mdr_0 *X, buffer* B, int n );

int free_mdr_0( t_mdr_0 *X, int n );

int init_mdr_0( t_mdr_0 *X, int n );

int size_mdr_0( t_mdr_0 *X );


typedef struct {
  REC_HEAD grh;
  union {
     t_mphr* MPHR;
     t_geadr_0* GEADR_0;
     t_ipr_0* IPR_0;
     t_viadr_0* VIADR_0;
     t_mdr_0* MDR_0;
  } content;
} t_eps_metopl0;

int get_eps_metopl0( t_eps_metopl0 *R, FILE *fp );
int put_eps_metopl0( t_eps_metopl0 *R, FILE *fp );
int free_eps_metopl0( t_eps_metopl0 *R );
#endif
