/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/5
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * VERSION   : 1.1
 *
 * HISTORY   :
 * 
 *             03/08/2006 : Change eps_integer from 4b to 8b
 *             30/07/2009 : Add eps_nreal4, eps_nreal8 
 *
 */
#ifndef EPS_TYPES_H
#define EPS_TYPES_H

/* 
  EUMETSAT data types as in EPS-GGS-SPE-96167-6.4.pdf
*/

typedef char eps_character;
typedef char eps_byte;

typedef char eps_integer1;
typedef unsigned char eps_uinteger1;

typedef short eps_integer2;
typedef unsigned short eps_uinteger2;

typedef int eps_integer4;
typedef unsigned int eps_uinteger4;

typedef long long eps_integer8;
typedef unsigned long long eps_uinteger8;

typedef float eps_real4;
typedef double eps_real8;
typedef float eps_nreal4;
typedef double eps_nreal8;

typedef struct {
  eps_byte scale;
  eps_integer4 value;
} eps_vinteger4;

typedef struct {
  eps_byte scale;
  eps_uinteger4 value;
} eps_vuinteger4;

typedef struct {
  eps_byte scale;
  eps_integer2 value;
} eps_vinteger2;

typedef struct {
  eps_byte scale;
  eps_uinteger2 value;
} eps_vuinteger2;

typedef long long eps_integer;
typedef unsigned long long eps_uinteger;

typedef unsigned char bit;

typedef struct {
  eps_integer YYYY;
  eps_integer MM;
  eps_integer DD;
  eps_integer HH;
  eps_integer mm;
  eps_integer ss;
} eps_gentime;

typedef struct {
  eps_integer YYYY;
  eps_integer MM;
  eps_integer DD;
  eps_integer HH;
  eps_integer mm;
  eps_integer ss;
  eps_integer uuu;
} eps_longgentime;

typedef struct {
  eps_uinteger2 days;
  eps_uinteger4 milliseconds;
} eps_ccsdstime;

typedef char eps_boolean;

typedef float eps_real;


typedef struct {
  int version;
  int type;
  int secondary_header_flag;
  int APID;
  int sequence_flag;
  int packet_seq_count;
  int packet_length;
} t_pph;

#define SIZE_OF_PACKET_PRIMARY_HEADER 6

typedef struct {
  int days;
  unsigned int msec;
  int usec; 
} t_psh;
  
#define SIZE_OF_PACKET_SECONDARY_HEADER 8

typedef struct {
  unsigned int coarse_time;
  unsigned int fine_time;
} t_obt;
  
#define SIZE_OF_SATELLITE_BINARY_TIME 6


#endif
