#ifndef _METOP_ADMIN_H
#define _METOP_ADMIN_H
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "eps_types.h"
#include "eps_buffer.h"
#include "eps_grh.h"


extern int metop_admin_check_subclass;
extern int metop_admin_check_subclass_version;


typedef struct {
  eps_integer4 M;
  eps_uinteger4 J;
  eps_uinteger4 D;
  eps_byte tbus[503];
} t_METOP_TBUS;

int read_METOP_TBUS( t_METOP_TBUS *X, buffer* B, int n );

int write_METOP_TBUS( t_METOP_TBUS *X, buffer* B, int n );

int free_METOP_TBUS( t_METOP_TBUS *X, int n );

int init_METOP_TBUS( t_METOP_TBUS *X, int n );

int size_METOP_TBUS( t_METOP_TBUS *X );

int print_METOP_TBUS( const char* label, t_METOP_TBUS *X, FILE *fp, int ndim, const int *dims );

typedef struct {
  eps_integer4 M;
  eps_uinteger4 J;
  eps_uinteger4 D;
  eps_real8 spot[13];
} t_METOP_SPOT;

int read_METOP_SPOT( t_METOP_SPOT *X, buffer* B, int n );

int write_METOP_SPOT( t_METOP_SPOT *X, buffer* B, int n );

int free_METOP_SPOT( t_METOP_SPOT *X, int n );

int init_METOP_SPOT( t_METOP_SPOT *X, int n );

int size_METOP_SPOT( t_METOP_SPOT *X );

int print_METOP_SPOT( const char* label, t_METOP_SPOT *X, FILE *fp, int ndim, const int *dims );

typedef struct {
  eps_integer4 M;
  eps_uinteger4 J;
  eps_uinteger4 D;
} t_ADMIN_PACKET_validity_time;

int read_ADMIN_PACKET_validity_time( t_ADMIN_PACKET_validity_time *X, buffer* B, int n );

int write_ADMIN_PACKET_validity_time( t_ADMIN_PACKET_validity_time *X, buffer* B, int n );

int free_ADMIN_PACKET_validity_time( t_ADMIN_PACKET_validity_time *X, int n );

int init_ADMIN_PACKET_validity_time( t_ADMIN_PACKET_validity_time *X, int n );

int size_ADMIN_PACKET_validity_time( t_ADMIN_PACKET_validity_time *X );

int print_ADMIN_PACKET_validity_time( const char* label, t_ADMIN_PACKET_validity_time *X, FILE *fp, int ndim, const int *dims );

typedef struct {
  eps_real8 a;
  eps_real8 b;
} t_ADMIN_PACKET_time_correlation;

int read_ADMIN_PACKET_time_correlation( t_ADMIN_PACKET_time_correlation *X, buffer* B, int n );

int write_ADMIN_PACKET_time_correlation( t_ADMIN_PACKET_time_correlation *X, buffer* B, int n );

int free_ADMIN_PACKET_time_correlation( t_ADMIN_PACKET_time_correlation *X, int n );

int init_ADMIN_PACKET_time_correlation( t_ADMIN_PACKET_time_correlation *X, int n );

int size_ADMIN_PACKET_time_correlation( t_ADMIN_PACKET_time_correlation *X );

int print_ADMIN_PACKET_time_correlation( const char* label, t_ADMIN_PACKET_time_correlation *X, FILE *fp, int ndim, const int *dims );

typedef struct {
  eps_uinteger1 message_number; /* Message Number ( incremented number 
                                   used as rolling counter ) */
  t_ADMIN_PACKET_validity_time validity_time; /* Starting time of valid
                                                 ity for this admin mes
                                                 sage ( MJD 2000 ) */
  eps_byte satellite_id[10]; /* METOP Satellite ID ( International Desi
                                gnator ) */
  t_ADMIN_PACKET_time_correlation time_correlation; /* METOP time corre
                                                       lation */
  t_METOP_TBUS tbus_parameters[3];
  t_METOP_SPOT spot_parameters[3];
  eps_real8 calibration_parameters[50];
  eps_uinteger2 compression_parameters[20];
  eps_byte events_and_messages[5200]; /* METOP events and messages ( ro
                                         lling buffer, each event shall 
                                         be preceded by an event serial 
                                         number ) */
  eps_byte noaa_satellite_id[10]; /* NOAA satellite ID ( International 
                                     designator ) */
  t_METOP_SPOT noaa_sc_spot[3];
  eps_uinteger1 spare1[70];
  eps_uinteger1 spare2[12];
} t_ADMIN_PACKET;

int read_ADMIN_PACKET( t_ADMIN_PACKET *X, buffer* B, int n );

int write_ADMIN_PACKET( t_ADMIN_PACKET *X, buffer* B, int n );

int free_ADMIN_PACKET( t_ADMIN_PACKET *X, int n );

int init_ADMIN_PACKET( t_ADMIN_PACKET *X, int n );

int size_ADMIN_PACKET( t_ADMIN_PACKET *X );

int print_ADMIN_PACKET( const char* label, t_ADMIN_PACKET *X, FILE *fp, int ndim, const int *dims );

#endif
