#!/bin/ksh
#
# Convert AVHRR level 1b PFS format to AAPP format (fixed record length).
# This program retains scaled radiances rather than going back to raw counts.
# Not all 1b fields are filled.
#
# Arguments: (1) Input file (2) Output file (default ${1}.l1c)
#
# Usage: convert_avh1b [-m mode] infile [outfile]
#   where mode=0 (default) to keep PFS counts, mode=1 to limit to 10 bits
#
# v1.1 07/12/2006 N C Atkinson
#  1.2 09/05/2007 Check input file present
#  1.3 22/07/2011 Remove ${FORT}10 before running exe
###############################################################################

. "$AAPP_PREFIX/ATOVS_CONF"

while getopts m: c
do
  case $c in
     m)	CONVERT_AVH1B_MODE=$OPTARG;;
  esac
done
shift `expr $OPTIND - 1`
CONVERT_AVH1B_MODE=${CONVERT_AVH1B_MODE:-0}  #default
export CONVERT_AVH1B_MODE

if [[ $# -lt 1 ]]; then
  echo
  echo "Usage: convert_avh1b [-m mode] infile [outfile] "
  echo "where mode=0 (default) to keep PFS counts, mode=1 to limit to 10 bits"
  echo
  exit 1
fi

if [ ! -s $1 ]; then
  echo "Input file missing"
  exit 1
fi

rm -f ${FORT}10       #remove any existing link
convert_avh1b.exe $1

if [ $? -ne 0 ]
then 
  exit $?
fi

outfile=${2:-${1}.l1b}
mv ${FORT}10 $outfile
