/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2012/1
 * 
 * AUTHOR    : Pascale.Roquet@meteo.fr
 *
 * VERSION   : 1.1
 *
 * SYNOPSIS  : Decoding of mmam packets and extract mmam xml bz2 message
 *             Takes mmam packets in binary format, as well
 *             as L0 files, and ccsds packets.
 *
 * HISTORY   : 1.2 19/07/2013 Ignore DUMMY instrument group. NCA
 *
 * 
 * LIBS = obtutc ccsds metop_mmam eps_metopl0 eps_common metop_common
 */



#include "metop_mmam.h"
#include "eps_buffer.h"
#include "eps_metopl0.h"
#include <stdio.h>
#include "ccsds.h"
#include "common-aapp.h"


int main( int argc, char* argv[] ) {

  t_MMAM_PACKET MP;
  buffer B;
  char buf[6];
  ccsds_packet packet;
  FILE *fpr;
  FILE *fpw;
  int i;
  int formatted = 0;
  int ccsds = 0;
  int pfsl0 = 0;
  int obtutc = 0;
 
  t_eps_metopl0 ER;
  
  if( argc < 2 ) {
    fprintf( stdout, "Usage: %s [ -formatted -ccsds -pfsl0  ] mmam-packet | HKTM_...  mmamBZ2file\n", argv[0] );
    return 0;
  }
  
  for( i = 1; i < argc; i++ ) {
    if( ! strcmp( argv[i], "-formatted" ) )
     formatted++;
    if( ! strcmp( argv[i], "-ccsds" ) )
     ccsds++;
    if( ! strcmp( argv[i], "-pfsl0" ) )
     pfsl0++;
    if( argv[i][0] != '-' )
      break;
  }
  if ( ccsds == 0 && pfsl0 == 0 ) pfsl0++;

  if( ( fpr = fopen( argv[i], "r" ) ) == NULL ) {
    printf( "Cannot open %s for reading\n", argv[i] );
    return 1;
  }
  i++;
  
  if( ( fpw = fopen( argv[i], "w" ) ) == NULL ) {
    printf( "Cannot open %s for writing\n", argv[i] );
    return 1;
  }
  if( pfsl0 ) {
  
    eps_metopl0_check_subclass = 0;
    eps_metopl0_check_subclass_version = 0;
    
    while( get_eps_metopl0( &ER, fpr ) ) {
      if( ER.grh.record_class == MDR && ER.grh.instrument_group != DUMMY ){
        ccsds_bread_packet( &packet, ER.content.MDR_0->INST_DATA );
       
	if( packet.pph.APID == 6 ) {
        
	  B.p = B.c = packet.packet + 20; 
	  B.lp = B.lc = packet.pph.packet_length;
          read_MMAM_PACKET(&MP, &B, 1 );
       
          free_eps_metopl0( &ER );
	  break;
	}
      }
      free_eps_metopl0( &ER );
    }
  
  } else {
  
    if( ccsds ) {                                                                             
    
      while(ccsds_fread_packet( &packet, fpr )) {
        B.p = B.c = packet.packet; 
        B.lp = B.lc = packet.pph.packet_length;
        
        if( packet.pph.APID == 6 ) {
        B.p = B.c = packet.packet + 20; 
        break;
        }
       }
    } else {
      init_buffer( &B, size_MMAM_PACKET( &MP ) );
      fread( B.p, B.lp, 1, fpr );
   }
   read_MMAM_PACKET(&MP, &B, 1 );
  }
if (formatted)
 print_formatted_mmam( &MP );
 write_file_MMAM_PACKET( &MP, fpw, 1 );
 fclose(fpw);
 fclose(fpr);
 return 0;
}
