/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2012/06
 * 
 * AUTHOR    : Pascale.Roquet@meteo.fr
 *
 * SYNOPSIS  : Change VIADR OBTC/UTC correlation parameters
 *
 * VERSION   : 1.0
 *
 * HISTORY   : 
 *          Oct 2012 1.1 Pascale Roquet 
 *            if VIADR doesn't exist create the record but keep only MDR records 
 *          Nov 2012 1.2 Pascale Roquet
 *            keep also IPR Record used by OPS for IASI
 *          Apr 2013 1.3 Nigel Atkinson
 *            an[4] -> an[5]. Change comments for portability.
 *          June 2013 1.4 Nigel Atkinson
 *            make sure daylight saving time is not used
 *
 * LIBS = eps_metopl0 eps_common 
 */

#include <stdio.h>
#include <stdlib.h>
#include "eps_metopl0.h"
#include <time.h>
#include <string.h>
#include <ctype.h>
void pfsl0copy(const char *src, const char *dst) {
/* copy a pfs file */
FILE *in = fopen(src,"r" );
FILE *out = fopen(dst, "w" );
t_eps_metopl0 ER;

while( 1 ) {
    if( ! get_eps_metopl0( &ER, in ) )
      break;
    else 
       (void) (put_eps_metopl0(&ER,out) ) ;
  }    
  fclose(in);
  fflush(out);
  fclose(out);
  
  
}
double string2utc (char * strdate) {
struct tm tm_date;
struct tm tm_2000;
char an[5];
char mon[3];
char day[3];
char hour[3];
char min[3];
char sec[3];
int m_sec;
char msec[4];
tm_2000.tm_sec=0;
tm_2000.tm_min=0;
tm_2000.tm_hour=0;
tm_2000.tm_mday=1;
tm_2000.tm_mon=0;
tm_2000.tm_year=2000-1900;
tm_2000.tm_isdst = 0;  /* not daylight saving */

/*example of date : 2012-01-19T07:02:17.255*/
strncpy((char*)&an,strdate,(size_t)4);
an[4]='\0';
tm_date.tm_year=atoi(an)-1900;

mon[0]=strdate[5];
mon[1]=strdate[6];
mon[2]='\0';
tm_date.tm_mon=atoi(mon)-1;

day[0]=strdate[8];
day[1]=strdate[9];
day[2]='\0';

tm_date.tm_mday=atoi(day);

hour[0]=strdate[11];
hour[1]=strdate[12];
hour[2]='\0';
tm_date.tm_hour=atoi(hour);

min[0]=strdate[14];
min[1]=strdate[15];
min[2]='\0';
tm_date.tm_min=atoi(min);

sec[0]=strdate[17];
sec[1]=strdate[18];
sec[2]='\0';
tm_date.tm_sec=atoi(sec);
tm_date.tm_isdst = 0;  /* not daylight saving */

msec[0]=strdate[20];
msec[1]=strdate[21];
msec[2]=strdate[22];
msec[3]='\0';

m_sec=atoi(msec);

return  difftime(mktime(&tm_date),mktime(&tm_2000)) + m_sec*1E-3;


}

int main( int argc, char* argv[] ) {
  const int MPHRsize=3307 ;
  const int SPHRsize=0 ;
  const int IPRsize= 27; 
  const int VIADRsize=38;
  int VIADRcount=0;
  int MPHRcount=0;
  int SPHRcount=0;
  int IPRcount=0;
  

  t_eps_metopl0 ER;
  FILE *fp;
  FILE *fpnew; 
  char * utc0;
  int npatch = 0;
  long offset;
  double utc,a,b;
  double  days, msec, usec;
  long long ccuobt0;
  eps_integer8 cs0, cs1 ;
  eps_uinteger4 rtime;
  eps_uinteger2 start_day,stop_day;
  eps_uinteger4 start_ms,stop_ms;
  int i=0; 
  t_viadr_0 viadr_content;
  t_eps_metopl0 Record;
  t_ipr_0 ipr_content;
  char * filename;
  char * oldfilename;


  if( argc < 5 ) {
    printf( "Usage: %s utc0 ccu-obt-0 clock-step ...._xxx_00_...\n", argv[0] );
    return 0;
  }

  if ( (filename=(char *)malloc (strlen(argv[4])+1 )) == NULL ) {
   fprintf(stderr,"error in malloc\n");
   exit(1);
  } 
  
   filename=strcpy(filename,argv[4]);
if( ( fp = fopen( argv[4], "r+" ) ) == NULL ) {
    printf( "Cannot open %s for r/w\n", argv[4] );
    return 1;
}
  
  eps_metopl0_check_subclass = 0;
  eps_metopl0_check_subclass_version = 0;
 
 
  utc0=argv[1];
  if (strlen(utc0) != 23 ) {
   fprintf(stderr,"utc0 wrong format must be : YYYY-MM-DDTHH:MN:SS.SSS\n");
   printf( "Usage: %s utc0 ccu-obt-0 clock-step ...._xxx_00_...\n", argv[0] );
   exit(EXIT_FAILURE);
  }
  i=0;
  while (i < strlen(argv[2])) 
   if (! isdigit(argv[2][i++])) {
     fprintf(stderr,"ccu-obt-0 wrong format must be numeric\n");
     printf( "Usage: %s utc0 ccu-obt-0 clock-step ...._xxx_00_...\n", argv[0] );
     exit(EXIT_FAILURE);
   }
  
  i=0;
  while (i < strlen(argv[3])) 
   if (! isdigit(argv[3][i++])) {
    fprintf(stderr,"clock-step wrong format must be numeric\n");
    printf( "Usage: %s utc0 ccu-obt-0 clock-step ...._xxx_00_...\n", argv[0] );
    exit(EXIT_FAILURE);
   }
  
  ccuobt0=strtoll(argv[2],(char **)NULL,10);
  rtime=( eps_uinteger4)  ccuobt0;
 
  a=atof(argv[3]);
  cs0 = (eps_integer8) a ;
  cs1 = (eps_integer8) a ;
  cs0 /= 0x10000;
  cs1 %= 0x10000;
  
/* compute number of seconds since 01/01/2000 00:00:00.000 */
  utc=string2utc(utc0);  /* utc : number of second since 01/01/2000 00:00:00.000 */
  days= (int) (utc/(3600*24)); /* days : number of days */
  b = utc - days * 3600 * 24; /* b= number of seconds */
  msec = (int) (b * 1000); /* number of millisecond */
  b = b - (double)msec / 1000.;
  usec =  (b * 1e6);
  
  while( 1 ) {
    offset = ftell( fp );
    if( ! get_eps_metopl0( &ER, fp ) )
      break;
   
    
    if( ER.grh.record_class ==  (eps_uinteger1) VIADR ) {
      
    
      if ( fseek( fp, offset, SEEK_SET ) != 0 ) {
        perror("fseek");
        exit(EXIT_FAILURE);
      }
      ER.content.VIADR_0->UTC_OBT_TIME.days = (eps_uinteger2) days;
      ER.content.VIADR_0->UTC_OBT_TIME.msec = (eps_uinteger4) msec;
      ER.content.VIADR_0->UTC_OBT_TIME.usec = (eps_uinteger2) usec;
      ER.content.VIADR_0->OBT_UTC_TIME.not_used = 0;
      ER.content.VIADR_0->OBT_UTC_TIME.rtime    = rtime;
      ER.content.VIADR_0->CLOCK_STEP[0] = (eps_uinteger2) cs0;
      ER.content.VIADR_0->CLOCK_STEP[1] = (eps_uinteger2) cs1;
    
      /* For some systems, read and write operations
                                      have to be interleaved with fseek or fflush */
      if (fseek( fp, 0, SEEK_CUR )!= 0 ) {
        perror("fseek");
        exit(EXIT_FAILURE);
      };   
 
      (void) put_eps_metopl0( &ER, fp );  
      
      if (fseek( fp, 0, SEEK_CUR )!= 0 ) {
        perror("fseek");
        exit(EXIT_FAILURE);
      };
      npatch++;
    }
    if( ER.grh.record_class == (eps_uinteger1) MDR ) {
     
      continue;   
    }
   if( ER.grh.record_class == (eps_uinteger1) MPHR ) {
      fprintf(stderr,"MPHR\n");
      fprintf(stderr,"start:%d.%d \n",ER.grh.record_start.days,ER.grh.record_start.milliseconds);
      fprintf(stderr,"stop:%d.%d \n",ER.grh.record_stop.days,ER.grh.record_stop.milliseconds);
      start_day=ER.grh.record_start.days;
      stop_day=ER.grh.record_stop.days;
      start_ms=	ER.grh.record_start.milliseconds;
      stop_ms=ER.grh.record_stop.milliseconds;
	
      continue;
     }
    if( ER.grh.record_class == (eps_uinteger1) SPHR ) {
    /* keeping this record for the second patch (non VIADR) */
      fprintf(stderr,"SPHR\n");
      /* TODO */
	
      continue;
     }
  }
  
  fclose( fp );
  if( npatch == 0 ) {
    printf( "missing VIADR0 in %s : create the record\n", argv[4] );
   
   /* copy the file to file.old */ 
   if ( (oldfilename=(char *)malloc (strlen(filename) +4)) == NULL ) {
   fprintf(stderr,"error in malloc\n");
   exit(1);
   }
   oldfilename=strcpy(oldfilename,filename);
   oldfilename=strcat(oldfilename,".old"); 
   (void)pfsl0copy(filename,oldfilename);

   /* rewrite the file and read the old one */
   if( ( fp = fopen(oldfilename, "r" ) ) == NULL ) {
    printf( "Cannot open %s for r\n", argv[4] );
    return 1;
    }
   if ( ( fpnew = fopen( filename, "w" ) ) == NULL ) {
    printf( "Cannot open %s for r\n", argv[4] );
    return 1;
    }
    while( 1 ) {
    offset = ftell( fp );
    if( ! get_eps_metopl0( &ER, fp ) )
      break;
   
    
    if( ER.grh.record_class ==  (eps_uinteger1) MPHR ) {

      fprintf(stderr,"MPHR\n");
      fprintf(stderr,"start:%d.%d \n",ER.grh.record_start.days,ER.grh.record_start.milliseconds);
      fprintf(stderr,"stop:%d.%d \n",ER.grh.record_stop.days,ER.grh.record_stop.milliseconds);
      start_day=ER.grh.record_start.days;
      stop_day=ER.grh.record_stop.days;
      start_ms=	ER.grh.record_start.milliseconds;
      stop_ms=ER.grh.record_stop.milliseconds;
      IPRcount=ER.content.MPHR->TOTAL_IPR;
      MPHRcount=ER.content.MPHR->TOTAL_MPHR;
      SPHRcount=ER.content.MPHR->TOTAL_SPHR;
      ER.content.MPHR->TOTAL_VIADR=1;
      ER.content.MPHR->TOTAL_GEADR=0;
      ER.content.MPHR->TOTAL_VEADR=0;
      ER.content.MPHR->TOTAL_SPHR=0;
      ER.content.MPHR->TOTAL_IPR=IPRcount+1;
      (void) (put_eps_metopl0(&ER,fpnew) ) ;
    }
    if( ER.grh.record_class == (eps_uinteger1) MDR ) {
     if (VIADRcount==0 ) {
       fprintf(stderr,"ERROR record order is not correct (missing an IPR ?) can't apply the patch\n");
       exit(1);
      }
     (void) (put_eps_metopl0(&ER,fpnew) ) ; 
    }
    if ( ER.grh.record_class == (eps_uinteger1) VEADR ||
         ER.grh.record_class == (eps_uinteger1) GEADR  ||
         ER.grh.record_class == (eps_uinteger1) GIADR  ||
         ER.grh.record_class == (eps_uinteger1) SPHR ) {
     fprintf(stderr,"WARNING record class %d not managed by this patch \n", ER.grh.record_class);
     fprintf(stderr,"WARNING new files may be inconsistent \n");
     continue;   
    }
  
    if( ER.grh.record_class == (eps_uinteger1) IPR ) {
   
     
  if ( VIADRcount==0 ) {    
    /*  write an new IPR for a VIADR record */
    /*  write the current IPR with an offset */
   /*  and finaly write a VIADR record */
     Record.grh.record_class=(eps_uinteger1) IPR;
     Record.grh.instrument_group=(eps_uinteger1) GENERIC;
     Record.grh.record_subclass=7;
     Record.grh.record_subclass_version=1;
     Record.grh.record_size=IPRsize;
     Record.grh.record_start.days=start_day;
     Record.grh.record_start.milliseconds=start_ms;
     Record.grh.record_stop.days=stop_day;
     Record.grh.record_stop.milliseconds=stop_ms;
     
     ipr_content.TARGET_RECORD_CLASS=7;
     ipr_content.TARGET_INSTRUMENT_GROUP=GENERIC;
     ipr_content.TARGET_RECORD_SUBCLASS=0;
     ipr_content.TARGET_RECORD_OFFSET=MPHRcount*MPHRsize+SPHR*SPHRsize+IPRsize*2;
     Record.content.IPR_0=&ipr_content;
     (void) (put_eps_metopl0(&Record,fpnew) ) ;

     /* write the curent IPR for MDR with an offset */
     ER.content.IPR_0->TARGET_RECORD_OFFSET=(ER.content.IPR_0->TARGET_RECORD_OFFSET)+38+27;
     (void) (put_eps_metopl0(&ER,fpnew) ) ;
     
     /* init a viadr_0 and write it to the file */
     Record.grh.record_class=(eps_uinteger1) VIADR;
     Record.grh.instrument_group=(eps_uinteger1) GENERIC;
     Record.grh.record_subclass=0;
     Record.grh.record_subclass_version=2;
     Record.grh.record_size=20+size_viadr_0(Record.content.VIADR_0);
     Record.grh.record_start.days=start_day;
     Record.grh.record_start.milliseconds=start_ms;
     Record.grh.record_stop.days=stop_day;
     Record.grh.record_stop.milliseconds=stop_ms;
    
     viadr_content.UTC_OBT_TIME.days = (eps_uinteger2) days;
     viadr_content.UTC_OBT_TIME.msec = (eps_uinteger4) msec;
     viadr_content.UTC_OBT_TIME.usec = (eps_uinteger2) usec;
     viadr_content.OBT_UTC_TIME.not_used = 0;
     viadr_content.OBT_UTC_TIME.rtime    = rtime;
     viadr_content.CLOCK_STEP[0] = (eps_uinteger2) cs0;
     viadr_content.CLOCK_STEP[1] = (eps_uinteger2) cs1;
     Record.content.VIADR_0=&viadr_content;  
     /* fprintf(stderr,"size viadr:%d\n",size_viadr_0(Record.content.VIADR_0)); */
     (void) (put_eps_metopl0(&Record,fpnew) ) ;
     VIADRcount=1;
      } 
	else {
      /* write the curent IPR  with a offset */
     ER.content.IPR_0->TARGET_RECORD_OFFSET=(ER.content.IPR_0->TARGET_RECORD_OFFSET)+38+27;
     (void) (put_eps_metopl0(&ER,fpnew) ) ;
      }
     
     continue;
  }
}
    
   fflush(fpnew);
   fclose (fpnew);
   fclose( fp ); 
   
   fprintf(stderr," Original File %s \n",oldfilename);
   fprintf(stderr," patched File %s \n", filename);
  
  
}
  
  exit(0);
}
