/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2007/01
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * SYNOPSIS  : Change VIADR OBTC/UTC correlation parameters
 *
 * VERSION   : 1.0
 *
 * HISTORY   : 27/01/2007 : Support multiple VIADRs
 *             29/01/2007 : Add dummy fseeks before and after
 *                          put_eps_metopl0 for systems which
 *                          need it.
 *             21/10/2008 : Re-initialise "b" between multiple VIADRs
 *
 *
 * LIBS = eps_metopl0 eps_common
 */

#include <stdio.h>
#include <stdlib.h>
#include "eps_metopl0.h"


int main( int argc, char* argv[] ) {
  t_eps_metopl0 ER;
  FILE *fp;
  double a, b;
  long offset;
  int npatch = 0;

  if( argc < 4 ) {
    printf( "Usage: %s a b ...._xxx_00_...\n", argv[0] );
    return 0;
  }
  
  if( ( fp = fopen( argv[3], "r+" ) ) == NULL ) {
    printf( "Cannot open %s for r/w\n", argv[3] );
    return 1;
  }
  
  a = atof( argv[1] );
  b = atof( argv[2] );
  
  
  
  eps_metopl0_check_subclass = 0;
  eps_metopl0_check_subclass_version = 0;
  while( 1 ) {
    offset = ftell( fp );
    if( ! get_eps_metopl0( &ER, fp ) )
      break;
    
    
    if( ER.grh.record_class == VIADR ) {
      int days, msec, usec;
      eps_integer8 cs0, cs1;
    
      fseek( fp, offset, SEEK_SET );
      
      b = atof( argv[2] );
      days = b / ( 3600 * 24  );
      b = b - days * 3600 * 24;
      msec = b * 1000;
      b = b - (double)msec / 1000.;
      usec = b * 1e6;
      
      
      ER.content.VIADR_0->UTC_OBT_TIME.days = days;
      ER.content.VIADR_0->UTC_OBT_TIME.msec = msec;
      ER.content.VIADR_0->UTC_OBT_TIME.usec = usec;
      ER.content.VIADR_0->OBT_UTC_TIME.not_used = 0;
      ER.content.VIADR_0->OBT_UTC_TIME.rtime    = 0;
      
      cs0 = a * 1e12;
      cs1 = a * 1e12;
      
      cs0 /= 0x10000;
      cs1 %= 0x10000;
      
      ER.content.VIADR_0->CLOCK_STEP[0] = cs0;
      ER.content.VIADR_0->CLOCK_STEP[1] = cs1;
      
      fseek( fp, 0, SEEK_CUR );    /* For some systems, read and write operations
                                      have to be interleaved with fseek or fflush */
      put_eps_metopl0( &ER, fp );  
      fseek( fp, 0, SEEK_CUR );
      
      npatch++;
    }
    if( ER.grh.record_class == MDR ) 
      break;
  }
  
  
  if( npatch == 0 )
    printf( "Could not apply patch to %s, missing VIADR0 ???\n", argv[3] );
  

  fclose( fp );

  return 1;
}
