/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/5
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * VERSION   : 1.0
 *
 * HISTORY   : 01/05/2005 : Initial
 *             24/05/2010 : Allow for extra GIADR in UMARF files. NCA
 *             07/02/2014 : Set to zero the NEdT information in CALIBRATION_QUALITY.
 *             03/02/2016 : Reverse the above and change convert_avh1b.F instead.
*/
#include "eps_avhrrl1b_6.5.h"

int eps_avhrrl1b_6_5_check_subclass = 1;
int eps_avhrrl1b_6_5_check_subclass_version = 1;


int read_mphr( t_mphr *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    while( 1 ) {
      const char* label;
      int len;
      if( ! read_label( &label, &len, B ) ) break;
      if( ( len == 12 ) && ( ! strncmp( label, "PRODUCT_NAME", 12 ) ) ) {
        read_ascii_string( X->PRODUCT_NAME, B, 67 );
      } else if( ( len == 21 ) && ( ! strncmp( label, "PARENT_PRODUCT_NAME_1", 21 ) ) ) {
        read_ascii_string( X->PARENT_PRODUCT_NAME_1, B, 67 );
      } else if( ( len == 21 ) && ( ! strncmp( label, "PARENT_PRODUCT_NAME_2", 21 ) ) ) {
        read_ascii_string( X->PARENT_PRODUCT_NAME_2, B, 67 );
      } else if( ( len == 21 ) && ( ! strncmp( label, "PARENT_PRODUCT_NAME_3", 21 ) ) ) {
        read_ascii_string( X->PARENT_PRODUCT_NAME_3, B, 67 );
      } else if( ( len == 21 ) && ( ! strncmp( label, "PARENT_PRODUCT_NAME_4", 21 ) ) ) {
        read_ascii_string( X->PARENT_PRODUCT_NAME_4, B, 67 );
      } else if( ( len == 13 ) && ( ! strncmp( label, "INSTRUMENT_ID", 13 ) ) ) {
        read_ascii_string( X->INSTRUMENT_ID, B, 4 );
      } else if( ( len == 16 ) && ( ! strncmp( label, "INSTRUMENT_MODEL", 16 ) ) ) {
        read_ascii_string( X->INSTRUMENT_MODEL, B, 3 );
      } else if( ( len == 12 ) && ( ! strncmp( label, "PRODUCT_TYPE", 12 ) ) ) {
        read_ascii_string( X->PRODUCT_TYPE, B, 3 );
      } else if( ( len == 16 ) && ( ! strncmp( label, "PROCESSING_LEVEL", 16 ) ) ) {
        read_ascii_string( X->PROCESSING_LEVEL, B, 2 );
      } else if( ( len == 13 ) && ( ! strncmp( label, "SPACECRAFT_ID", 13 ) ) ) {
        read_ascii_string( X->SPACECRAFT_ID, B, 3 );
      } else if( ( len == 13 ) && ( ! strncmp( label, "SENSING_START", 13 ) ) ) {
        read_ascii_eps_gentime( &X->SENSING_START, B );
      } else if( ( len == 11 ) && ( ! strncmp( label, "SENSING_END", 11 ) ) ) {
        read_ascii_eps_gentime( &X->SENSING_END, B );
      } else if( ( len == 25 ) && ( ! strncmp( label, "SENSING_START_THEORETICAL", 25 ) ) ) {
        read_ascii_eps_gentime( &X->SENSING_START_THEORETICAL, B );
      } else if( ( len == 23 ) && ( ! strncmp( label, "SENSING_END_THEORETICAL", 23 ) ) ) {
        read_ascii_eps_gentime( &X->SENSING_END_THEORETICAL, B );
      } else if( ( len == 17 ) && ( ! strncmp( label, "PROCESSING_CENTRE", 17 ) ) ) {
        read_ascii_string( X->PROCESSING_CENTRE, B, 4 );
      } else if( ( len == 23 ) && ( ! strncmp( label, "PROCESSOR_MAJOR_VERSION", 23 ) ) ) {
        read_ascii_eps_uinteger( &X->PROCESSOR_MAJOR_VERSION, B, 5 );
      } else if( ( len == 23 ) && ( ! strncmp( label, "PROCESSOR_MINOR_VERSION", 23 ) ) ) {
        read_ascii_eps_uinteger( &X->PROCESSOR_MINOR_VERSION, B, 5 );
      } else if( ( len == 20 ) && ( ! strncmp( label, "FORMAT_MAJOR_VERSION", 20 ) ) ) {
        read_ascii_eps_uinteger( &X->FORMAT_MAJOR_VERSION, B, 5 );
      } else if( ( len == 20 ) && ( ! strncmp( label, "FORMAT_MINOR_VERSION", 20 ) ) ) {
        read_ascii_eps_uinteger( &X->FORMAT_MINOR_VERSION, B, 5 );
      } else if( ( len == 21 ) && ( ! strncmp( label, "PROCESSING_TIME_START", 21 ) ) ) {
        read_ascii_eps_gentime( &X->PROCESSING_TIME_START, B );
      } else if( ( len == 19 ) && ( ! strncmp( label, "PROCESSING_TIME_END", 19 ) ) ) {
        read_ascii_eps_gentime( &X->PROCESSING_TIME_END, B );
      } else if( ( len == 15 ) && ( ! strncmp( label, "PROCESSING_MODE", 15 ) ) ) {
        read_ascii_string( X->PROCESSING_MODE, B, 1 );
      } else if( ( len == 16 ) && ( ! strncmp( label, "DISPOSITION_MODE", 16 ) ) ) {
        read_ascii_string( X->DISPOSITION_MODE, B, 1 );
      } else if( ( len == 24 ) && ( ! strncmp( label, "RECEIVING_GROUND_STATION", 24 ) ) ) {
        read_ascii_string( X->RECEIVING_GROUND_STATION, B, 3 );
      } else if( ( len == 18 ) && ( ! strncmp( label, "RECEIVE_TIME_START", 18 ) ) ) {
        read_ascii_eps_gentime( &X->RECEIVE_TIME_START, B );
      } else if( ( len == 16 ) && ( ! strncmp( label, "RECEIVE_TIME_END", 16 ) ) ) {
        read_ascii_eps_gentime( &X->RECEIVE_TIME_END, B );
      } else if( ( len == 11 ) && ( ! strncmp( label, "ORBIT_START", 11 ) ) ) {
        read_ascii_eps_uinteger( &X->ORBIT_START, B, 5 );
      } else if( ( len == 9 ) && ( ! strncmp( label, "ORBIT_END", 9 ) ) ) {
        read_ascii_eps_uinteger( &X->ORBIT_END, B, 5 );
      } else if( ( len == 19 ) && ( ! strncmp( label, "ACTUAL_PRODUCT_SIZE", 19 ) ) ) {
        read_ascii_eps_uinteger( &X->ACTUAL_PRODUCT_SIZE, B, 11 );
      } else if( ( len == 17 ) && ( ! strncmp( label, "STATE_VECTOR_TIME", 17 ) ) ) {
        read_ascii_eps_longgentime( &X->STATE_VECTOR_TIME, B );
      } else if( ( len == 15 ) && ( ! strncmp( label, "SEMI_MAJOR_AXIS", 15 ) ) ) {
        read_ascii_eps_integer( &X->SEMI_MAJOR_AXIS, B, 11 );
      } else if( ( len == 12 ) && ( ! strncmp( label, "ECCENTRICITY", 12 ) ) ) {
        read_ascii_eps_integer( &X->ECCENTRICITY, B, 11 );
      } else if( ( len == 11 ) && ( ! strncmp( label, "INCLINATION", 11 ) ) ) {
        read_ascii_eps_integer( &X->INCLINATION, B, 11 );
      } else if( ( len == 16 ) && ( ! strncmp( label, "PERIGEE_ARGUMENT", 16 ) ) ) {
        read_ascii_eps_integer( &X->PERIGEE_ARGUMENT, B, 11 );
      } else if( ( len == 15 ) && ( ! strncmp( label, "RIGHT_ASCENSION", 15 ) ) ) {
        read_ascii_eps_integer( &X->RIGHT_ASCENSION, B, 11 );
      } else if( ( len == 12 ) && ( ! strncmp( label, "MEAN_ANOMALY", 12 ) ) ) {
        read_ascii_eps_integer( &X->MEAN_ANOMALY, B, 11 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "X_POSITION", 10 ) ) ) {
        read_ascii_eps_integer( &X->X_POSITION, B, 11 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "Y_POSITION", 10 ) ) ) {
        read_ascii_eps_integer( &X->Y_POSITION, B, 11 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "Z_POSITION", 10 ) ) ) {
        read_ascii_eps_integer( &X->Z_POSITION, B, 11 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "X_VELOCITY", 10 ) ) ) {
        read_ascii_eps_integer( &X->X_VELOCITY, B, 11 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "Y_VELOCITY", 10 ) ) ) {
        read_ascii_eps_integer( &X->Y_VELOCITY, B, 11 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "Z_VELOCITY", 10 ) ) ) {
        read_ascii_eps_integer( &X->Z_VELOCITY, B, 11 );
      } else if( ( len == 24 ) && ( ! strncmp( label, "EARTH_SUN_DISTANCE_RATIO", 24 ) ) ) {
        read_ascii_eps_integer( &X->EARTH_SUN_DISTANCE_RATIO, B, 11 );
      } else if( ( len == 25 ) && ( ! strncmp( label, "LOCATION_TOLERANCE_RADIAL", 25 ) ) ) {
        read_ascii_eps_integer( &X->LOCATION_TOLERANCE_RADIAL, B, 11 );
      } else if( ( len == 29 ) && ( ! strncmp( label, "LOCATION_TOLERANCE_CROSSTRACK", 29 ) ) ) {
        read_ascii_eps_integer( &X->LOCATION_TOLERANCE_CROSSTRACK, B, 11 );
      } else if( ( len == 29 ) && ( ! strncmp( label, "LOCATION_TOLERANCE_ALONGTRACK", 29 ) ) ) {
        read_ascii_eps_integer( &X->LOCATION_TOLERANCE_ALONGTRACK, B, 11 );
      } else if( ( len == 9 ) && ( ! strncmp( label, "YAW_ERROR", 9 ) ) ) {
        read_ascii_eps_integer( &X->YAW_ERROR, B, 11 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "ROLL_ERROR", 10 ) ) ) {
        read_ascii_eps_integer( &X->ROLL_ERROR, B, 11 );
      } else if( ( len == 11 ) && ( ! strncmp( label, "PITCH_ERROR", 11 ) ) ) {
        read_ascii_eps_integer( &X->PITCH_ERROR, B, 11 );
      } else if( ( len == 21 ) && ( ! strncmp( label, "SUBSAT_LATITUDE_START", 21 ) ) ) {
        read_ascii_eps_integer( &X->SUBSAT_LATITUDE_START, B, 11 );
      } else if( ( len == 22 ) && ( ! strncmp( label, "SUBSAT_LONGITUDE_START", 22 ) ) ) {
        read_ascii_eps_integer( &X->SUBSAT_LONGITUDE_START, B, 11 );
      } else if( ( len == 19 ) && ( ! strncmp( label, "SUBSAT_LATITUDE_END", 19 ) ) ) {
        read_ascii_eps_integer( &X->SUBSAT_LATITUDE_END, B, 11 );
      } else if( ( len == 20 ) && ( ! strncmp( label, "SUBSAT_LONGITUDE_END", 20 ) ) ) {
        read_ascii_eps_integer( &X->SUBSAT_LONGITUDE_END, B, 11 );
      } else if( ( len == 11 ) && ( ! strncmp( label, "LEAP_SECOND", 11 ) ) ) {
        read_ascii_eps_integer( &X->LEAP_SECOND, B, 2 );
      } else if( ( len == 15 ) && ( ! strncmp( label, "LEAP_SECOND_UTC", 15 ) ) ) {
        read_ascii_eps_gentime( &X->LEAP_SECOND_UTC, B );
      } else if( ( len == 13 ) && ( ! strncmp( label, "TOTAL_RECORDS", 13 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_RECORDS, B, 6 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "TOTAL_MPHR", 10 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_MPHR, B, 6 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "TOTAL_SPHR", 10 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_SPHR, B, 6 );
      } else if( ( len == 9 ) && ( ! strncmp( label, "TOTAL_IPR", 9 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_IPR, B, 6 );
      } else if( ( len == 11 ) && ( ! strncmp( label, "TOTAL_GEADR", 11 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_GEADR, B, 6 );
      } else if( ( len == 11 ) && ( ! strncmp( label, "TOTAL_GIADR", 11 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_GIADR, B, 6 );
      } else if( ( len == 11 ) && ( ! strncmp( label, "TOTAL_VEADR", 11 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_VEADR, B, 6 );
      } else if( ( len == 11 ) && ( ! strncmp( label, "TOTAL_VIADR", 11 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_VIADR, B, 6 );
      } else if( ( len == 9 ) && ( ! strncmp( label, "TOTAL_MDR", 9 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_MDR, B, 6 );
      } else if( ( len == 23 ) && ( ! strncmp( label, "COUNT_DEGRADED_INST_MDR", 23 ) ) ) {
        read_ascii_eps_uinteger( &X->COUNT_DEGRADED_INST_MDR, B, 6 );
      } else if( ( len == 23 ) && ( ! strncmp( label, "COUNT_DEGRADED_PROC_MDR", 23 ) ) ) {
        read_ascii_eps_uinteger( &X->COUNT_DEGRADED_PROC_MDR, B, 6 );
      } else if( ( len == 30 ) && ( ! strncmp( label, "COUNT_DEGRADED_INST_MDR_BLOCKS", 30 ) ) ) {
        read_ascii_eps_uinteger( &X->COUNT_DEGRADED_INST_MDR_BLOCKS, B, 6 );
      } else if( ( len == 30 ) && ( ! strncmp( label, "COUNT_DEGRADED_PROC_MDR_BLOCKS", 30 ) ) ) {
        read_ascii_eps_uinteger( &X->COUNT_DEGRADED_PROC_MDR_BLOCKS, B, 6 );
      } else if( ( len == 19 ) && ( ! strncmp( label, "DURATION_OF_PRODUCT", 19 ) ) ) {
        read_ascii_eps_uinteger( &X->DURATION_OF_PRODUCT, B, 8 );
      } else if( ( len == 28 ) && ( ! strncmp( label, "MILLISECONDS_OF_DATA_PRESENT", 28 ) ) ) {
        read_ascii_eps_uinteger( &X->MILLISECONDS_OF_DATA_PRESENT, B, 8 );
      } else if( ( len == 28 ) && ( ! strncmp( label, "MILLISECONDS_OF_DATA_MISSING", 28 ) ) ) {
        read_ascii_eps_uinteger( &X->MILLISECONDS_OF_DATA_MISSING, B, 8 );
      } else if( ( len == 17 ) && ( ! strncmp( label, "SUBSETTED_PRODUCT", 17 ) ) ) {
        read_ascii_eps_boolean( &X->SUBSETTED_PRODUCT, B, 1 );
      } else  {
        /* Should issue warning */
      }
      skip_line( B );
    }
  }
  return status_buffer( B );
}


int write_mphr( t_mphr *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    write_ascii_string( "PRODUCT_NAME", X->PRODUCT_NAME, B, 67 );
    write_ascii_string( "PARENT_PRODUCT_NAME_1", X->PARENT_PRODUCT_NAME_1, B, 67 );
    write_ascii_string( "PARENT_PRODUCT_NAME_2", X->PARENT_PRODUCT_NAME_2, B, 67 );
    write_ascii_string( "PARENT_PRODUCT_NAME_3", X->PARENT_PRODUCT_NAME_3, B, 67 );
    write_ascii_string( "PARENT_PRODUCT_NAME_4", X->PARENT_PRODUCT_NAME_4, B, 67 );
    write_ascii_string( "INSTRUMENT_ID", X->INSTRUMENT_ID, B, 4 );
    write_ascii_string( "INSTRUMENT_MODEL", X->INSTRUMENT_MODEL, B, 3 );
    write_ascii_string( "PRODUCT_TYPE", X->PRODUCT_TYPE, B, 3 );
    write_ascii_string( "PROCESSING_LEVEL", X->PROCESSING_LEVEL, B, 2 );
    write_ascii_string( "SPACECRAFT_ID", X->SPACECRAFT_ID, B, 3 );
    write_ascii_eps_gentime( "SENSING_START", &X->SENSING_START, B );
    write_ascii_eps_gentime( "SENSING_END", &X->SENSING_END, B );
    write_ascii_eps_gentime( "SENSING_START_THEORETICAL", &X->SENSING_START_THEORETICAL, B );
    write_ascii_eps_gentime( "SENSING_END_THEORETICAL", &X->SENSING_END_THEORETICAL, B );
    write_ascii_string( "PROCESSING_CENTRE", X->PROCESSING_CENTRE, B, 4 );
    write_ascii_eps_uinteger( "PROCESSOR_MAJOR_VERSION", &X->PROCESSOR_MAJOR_VERSION, B, 5 );
    write_ascii_eps_uinteger( "PROCESSOR_MINOR_VERSION", &X->PROCESSOR_MINOR_VERSION, B, 5 );
    write_ascii_eps_uinteger( "FORMAT_MAJOR_VERSION", &X->FORMAT_MAJOR_VERSION, B, 5 );
    write_ascii_eps_uinteger( "FORMAT_MINOR_VERSION", &X->FORMAT_MINOR_VERSION, B, 5 );
    write_ascii_eps_gentime( "PROCESSING_TIME_START", &X->PROCESSING_TIME_START, B );
    write_ascii_eps_gentime( "PROCESSING_TIME_END", &X->PROCESSING_TIME_END, B );
    write_ascii_string( "PROCESSING_MODE", X->PROCESSING_MODE, B, 1 );
    write_ascii_string( "DISPOSITION_MODE", X->DISPOSITION_MODE, B, 1 );
    write_ascii_string( "RECEIVING_GROUND_STATION", X->RECEIVING_GROUND_STATION, B, 3 );
    write_ascii_eps_gentime( "RECEIVE_TIME_START", &X->RECEIVE_TIME_START, B );
    write_ascii_eps_gentime( "RECEIVE_TIME_END", &X->RECEIVE_TIME_END, B );
    write_ascii_eps_uinteger( "ORBIT_START", &X->ORBIT_START, B, 5 );
    write_ascii_eps_uinteger( "ORBIT_END", &X->ORBIT_END, B, 5 );
    write_ascii_eps_uinteger( "ACTUAL_PRODUCT_SIZE", &X->ACTUAL_PRODUCT_SIZE, B, 11 );
    write_ascii_eps_longgentime( "STATE_VECTOR_TIME", &X->STATE_VECTOR_TIME, B );
    write_ascii_eps_integer( "SEMI_MAJOR_AXIS", &X->SEMI_MAJOR_AXIS, B, 11 );
    write_ascii_eps_integer( "ECCENTRICITY", &X->ECCENTRICITY, B, 11 );
    write_ascii_eps_integer( "INCLINATION", &X->INCLINATION, B, 11 );
    write_ascii_eps_integer( "PERIGEE_ARGUMENT", &X->PERIGEE_ARGUMENT, B, 11 );
    write_ascii_eps_integer( "RIGHT_ASCENSION", &X->RIGHT_ASCENSION, B, 11 );
    write_ascii_eps_integer( "MEAN_ANOMALY", &X->MEAN_ANOMALY, B, 11 );
    write_ascii_eps_integer( "X_POSITION", &X->X_POSITION, B, 11 );
    write_ascii_eps_integer( "Y_POSITION", &X->Y_POSITION, B, 11 );
    write_ascii_eps_integer( "Z_POSITION", &X->Z_POSITION, B, 11 );
    write_ascii_eps_integer( "X_VELOCITY", &X->X_VELOCITY, B, 11 );
    write_ascii_eps_integer( "Y_VELOCITY", &X->Y_VELOCITY, B, 11 );
    write_ascii_eps_integer( "Z_VELOCITY", &X->Z_VELOCITY, B, 11 );
    write_ascii_eps_integer( "EARTH_SUN_DISTANCE_RATIO", &X->EARTH_SUN_DISTANCE_RATIO, B, 11 );
    write_ascii_eps_integer( "LOCATION_TOLERANCE_RADIAL", &X->LOCATION_TOLERANCE_RADIAL, B, 11 );
    write_ascii_eps_integer( "LOCATION_TOLERANCE_CROSSTRACK", &X->LOCATION_TOLERANCE_CROSSTRACK, B, 11 );
    write_ascii_eps_integer( "LOCATION_TOLERANCE_ALONGTRACK", &X->LOCATION_TOLERANCE_ALONGTRACK, B, 11 );
    write_ascii_eps_integer( "YAW_ERROR", &X->YAW_ERROR, B, 11 );
    write_ascii_eps_integer( "ROLL_ERROR", &X->ROLL_ERROR, B, 11 );
    write_ascii_eps_integer( "PITCH_ERROR", &X->PITCH_ERROR, B, 11 );
    write_ascii_eps_integer( "SUBSAT_LATITUDE_START", &X->SUBSAT_LATITUDE_START, B, 11 );
    write_ascii_eps_integer( "SUBSAT_LONGITUDE_START", &X->SUBSAT_LONGITUDE_START, B, 11 );
    write_ascii_eps_integer( "SUBSAT_LATITUDE_END", &X->SUBSAT_LATITUDE_END, B, 11 );
    write_ascii_eps_integer( "SUBSAT_LONGITUDE_END", &X->SUBSAT_LONGITUDE_END, B, 11 );
    write_ascii_eps_integer( "LEAP_SECOND", &X->LEAP_SECOND, B, 2 );
    write_ascii_eps_gentime( "LEAP_SECOND_UTC", &X->LEAP_SECOND_UTC, B );
    write_ascii_eps_uinteger( "TOTAL_RECORDS", &X->TOTAL_RECORDS, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_MPHR", &X->TOTAL_MPHR, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_SPHR", &X->TOTAL_SPHR, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_IPR", &X->TOTAL_IPR, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_GEADR", &X->TOTAL_GEADR, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_GIADR", &X->TOTAL_GIADR, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_VEADR", &X->TOTAL_VEADR, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_VIADR", &X->TOTAL_VIADR, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_MDR", &X->TOTAL_MDR, B, 6 );
    write_ascii_eps_uinteger( "COUNT_DEGRADED_INST_MDR", &X->COUNT_DEGRADED_INST_MDR, B, 6 );
    write_ascii_eps_uinteger( "COUNT_DEGRADED_PROC_MDR", &X->COUNT_DEGRADED_PROC_MDR, B, 6 );
    write_ascii_eps_uinteger( "COUNT_DEGRADED_INST_MDR_BLOCKS", &X->COUNT_DEGRADED_INST_MDR_BLOCKS, B, 6 );
    write_ascii_eps_uinteger( "COUNT_DEGRADED_PROC_MDR_BLOCKS", &X->COUNT_DEGRADED_PROC_MDR_BLOCKS, B, 6 );
    write_ascii_eps_uinteger( "DURATION_OF_PRODUCT", &X->DURATION_OF_PRODUCT, B, 8 );
    write_ascii_eps_uinteger( "MILLISECONDS_OF_DATA_PRESENT", &X->MILLISECONDS_OF_DATA_PRESENT, B, 8 );
    write_ascii_eps_uinteger( "MILLISECONDS_OF_DATA_MISSING", &X->MILLISECONDS_OF_DATA_MISSING, B, 8 );
    write_ascii_eps_boolean( "SUBSETTED_PRODUCT", &X->SUBSETTED_PRODUCT, B, 1 );
  }
  return status_buffer( B );
}


int free_mphr( t_mphr *X, int n ) {
}


int init_mphr( t_mphr *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    init_string( X->PRODUCT_NAME, 67 );
    init_string( X->PARENT_PRODUCT_NAME_1, 67 );
    init_string( X->PARENT_PRODUCT_NAME_2, 67 );
    init_string( X->PARENT_PRODUCT_NAME_3, 67 );
    init_string( X->PARENT_PRODUCT_NAME_4, 67 );
    init_string( X->INSTRUMENT_ID, 4 );
    init_string( X->INSTRUMENT_MODEL, 3 );
    init_string( X->PRODUCT_TYPE, 3 );
    init_string( X->PROCESSING_LEVEL, 2 );
    init_string( X->SPACECRAFT_ID, 3 );
    init_eps_gentime( &X->SENSING_START );
    init_eps_gentime( &X->SENSING_END );
    init_eps_gentime( &X->SENSING_START_THEORETICAL );
    init_eps_gentime( &X->SENSING_END_THEORETICAL );
    init_string( X->PROCESSING_CENTRE, 4 );
    init_eps_uinteger( &X->PROCESSOR_MAJOR_VERSION );
    init_eps_uinteger( &X->PROCESSOR_MINOR_VERSION );
    init_eps_uinteger( &X->FORMAT_MAJOR_VERSION );
    init_eps_uinteger( &X->FORMAT_MINOR_VERSION );
    init_eps_gentime( &X->PROCESSING_TIME_START );
    init_eps_gentime( &X->PROCESSING_TIME_END );
    init_string( X->PROCESSING_MODE, 1 );
    init_string( X->DISPOSITION_MODE, 1 );
    init_string( X->RECEIVING_GROUND_STATION, 3 );
    init_eps_gentime( &X->RECEIVE_TIME_START );
    init_eps_gentime( &X->RECEIVE_TIME_END );
    init_eps_uinteger( &X->ORBIT_START );
    init_eps_uinteger( &X->ORBIT_END );
    init_eps_uinteger( &X->ACTUAL_PRODUCT_SIZE );
    init_eps_longgentime( &X->STATE_VECTOR_TIME );
    init_eps_integer( &X->SEMI_MAJOR_AXIS );
    init_eps_integer( &X->ECCENTRICITY );
    init_eps_integer( &X->INCLINATION );
    init_eps_integer( &X->PERIGEE_ARGUMENT );
    init_eps_integer( &X->RIGHT_ASCENSION );
    init_eps_integer( &X->MEAN_ANOMALY );
    init_eps_integer( &X->X_POSITION );
    init_eps_integer( &X->Y_POSITION );
    init_eps_integer( &X->Z_POSITION );
    init_eps_integer( &X->X_VELOCITY );
    init_eps_integer( &X->Y_VELOCITY );
    init_eps_integer( &X->Z_VELOCITY );
    init_eps_integer( &X->EARTH_SUN_DISTANCE_RATIO );
    init_eps_integer( &X->LOCATION_TOLERANCE_RADIAL );
    init_eps_integer( &X->LOCATION_TOLERANCE_CROSSTRACK );
    init_eps_integer( &X->LOCATION_TOLERANCE_ALONGTRACK );
    init_eps_integer( &X->YAW_ERROR );
    init_eps_integer( &X->ROLL_ERROR );
    init_eps_integer( &X->PITCH_ERROR );
    init_eps_integer( &X->SUBSAT_LATITUDE_START );
    init_eps_integer( &X->SUBSAT_LONGITUDE_START );
    init_eps_integer( &X->SUBSAT_LATITUDE_END );
    init_eps_integer( &X->SUBSAT_LONGITUDE_END );
    init_eps_integer( &X->LEAP_SECOND );
    init_eps_gentime( &X->LEAP_SECOND_UTC );
    init_eps_uinteger( &X->TOTAL_RECORDS );
    init_eps_uinteger( &X->TOTAL_MPHR );
    init_eps_uinteger( &X->TOTAL_SPHR );
    init_eps_uinteger( &X->TOTAL_IPR );
    init_eps_uinteger( &X->TOTAL_GEADR );
    init_eps_uinteger( &X->TOTAL_GIADR );
    init_eps_uinteger( &X->TOTAL_VEADR );
    init_eps_uinteger( &X->TOTAL_VIADR );
    init_eps_uinteger( &X->TOTAL_MDR );
    init_eps_uinteger( &X->COUNT_DEGRADED_INST_MDR );
    init_eps_uinteger( &X->COUNT_DEGRADED_PROC_MDR );
    init_eps_uinteger( &X->COUNT_DEGRADED_INST_MDR_BLOCKS );
    init_eps_uinteger( &X->COUNT_DEGRADED_PROC_MDR_BLOCKS );
    init_eps_uinteger( &X->DURATION_OF_PRODUCT );
    init_eps_uinteger( &X->MILLISECONDS_OF_DATA_PRESENT );
    init_eps_uinteger( &X->MILLISECONDS_OF_DATA_MISSING );
    init_eps_boolean( &X->SUBSETTED_PRODUCT );
  }
}


int size_mphr( t_mphr *X ) {
  return 3287;
}



int read_sphr( t_sphr *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    while( 1 ) {
      const char* label;
      int len;
      if( ! read_label( &label, &len, B ) ) break;
      if( ( len == 13 ) && ( ! strncmp( label, "SRC_DATA_QUAL", 13 ) ) ) {
        read_ascii_eps_bitfield( &X->SRC_DATA_QUAL, B, 16 );
      } else if( ( len == 24 ) && ( ! strncmp( label, "EARTH_VIEWS_PER_SCANLINE", 24 ) ) ) {
        read_ascii_eps_integer( &X->EARTH_VIEWS_PER_SCANLINE, B, 5 );
      } else if( ( len == 15 ) && ( ! strncmp( label, "NAV_SAMPLE_RATE", 15 ) ) ) {
        read_ascii_eps_integer( &X->NAV_SAMPLE_RATE, B, 3 );
      } else  {
        /* Should issue warning */
      }
      skip_line( B );
    }
  }
  return status_buffer( B );
}


int write_sphr( t_sphr *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    write_ascii_eps_bitfield( "SRC_DATA_QUAL", &X->SRC_DATA_QUAL, B, 16 );
    write_ascii_eps_integer( "EARTH_VIEWS_PER_SCANLINE", &X->EARTH_VIEWS_PER_SCANLINE, B, 5 );
    write_ascii_eps_integer( "NAV_SAMPLE_RATE", &X->NAV_SAMPLE_RATE, B, 3 );
  }
  return status_buffer( B );
}


int free_sphr( t_sphr *X, int n ) {
}


int init_sphr( t_sphr *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    init_eps_bitfield( &X->SRC_DATA_QUAL );
    init_eps_integer( &X->EARTH_VIEWS_PER_SCANLINE );
    init_eps_integer( &X->NAV_SAMPLE_RATE );
  }
}


int size_sphr( t_sphr *X ) {
  return 123;
}



int read_ipr( t_ipr *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    read_eps_uinteger1( &X->TARGET_RECORD_CLASS, B, 1 );
    read_eps_uinteger1( &X->TARGET_INSTRUMENT_GROUP, B, 1 );
    read_eps_uinteger1( &X->TARGET_RECORD_SUBCLASS, B, 1 );
    read_eps_uinteger4( &X->TARGET_RECORD_OFFSET, B, 1 );
  }
  return status_buffer( B );
}


int write_ipr( t_ipr *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    write_eps_uinteger1( &X->TARGET_RECORD_CLASS, B, 1 );
    write_eps_uinteger1( &X->TARGET_INSTRUMENT_GROUP, B, 1 );
    write_eps_uinteger1( &X->TARGET_RECORD_SUBCLASS, B, 1 );
    write_eps_uinteger4( &X->TARGET_RECORD_OFFSET, B, 1 );
  }
  return status_buffer( B );
}


int free_ipr( t_ipr *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
  }
}


int init_ipr( t_ipr *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
  }
}


int size_ipr( t_ipr *X ) {
  int i;
  return 1 + 1 + 1 + 4;
}



int read_giadr_radiance( t_giadr_radiance *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* RADIANCE_CONVERSION */
    read_eps_byte( X->RAMP_CALIBRATION_COEFFICIENT, B, 2 );
    read_eps_uinteger2( &X->YEAR_RECENT_CALIBRATION, B, 1 );
    read_eps_uinteger2( &X->DAY_RECENT_CALIBRATION, B, 1 );
    read_eps_uinteger2( &X->PRIMARY_CALIBRATION_ALGORITHM_ID, B, 1 );
    read_eps_byte( X->PRIMARY_CALIBRATION_ALGORITHM_OPTION, B, 2 );
    read_eps_uinteger2( &X->SECONDARY_CALIBRATION_ALGORITHM_ID, B, 1 );
    read_eps_byte( X->SECONDARY_CALIBRATION_ALGORITHM_OPTION, B, 2 );
    read_eps_integer2( &X->IR_TEMPERATURE1_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE1_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE1_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE1_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE1_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE1_COEFFICIENT6, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE2_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE2_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE2_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE2_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE2_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE2_COEFFICIENT6, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE3_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE3_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE3_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE3_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE3_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE3_COEFFICIENT6, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE4_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE4_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE4_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE4_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE4_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->IR_TEMPERATURE4_COEFFICIENT6, B, 1 );
    read_eps_integer2( &X->CH1_SOLAR_FILTERED_IRRADIANCE, B, 1 );
    read_eps_integer2( &X->CH1_EQUIVALENT_FILTER_WIDTH, B, 1 );
    read_eps_integer2( &X->CH2_SOLAR_FILTERED_IRRADIANCE, B, 1 );
    read_eps_integer2( &X->CH2_EQUIVALENT_FILTER_WIDTH, B, 1 );
    read_eps_integer2( &X->CH3A_SOLAR_FILTERED_IRRADIANCE, B, 1 );
    read_eps_integer2( &X->CH3A_EQUIVALENT_FILTER_WIDTH, B, 1 );
    read_eps_integer4( &X->CH3B_CENTRAL_WAVENUMBER, B, 1 );
    read_eps_integer4( &X->CH3B_CONSTANT1, B, 1 );
    read_eps_integer4( &X->CH3B_CONSTANT2_SLOPE, B, 1 );
    read_eps_integer4( &X->CH4_CENTRAL_WAVENUMBER, B, 1 );
    read_eps_integer4( &X->CH4_CONSTANT1, B, 1 );
    read_eps_integer4( &X->CH4_CONSTANT2_SLOPE, B, 1 );
    read_eps_integer4( &X->CH5_CENTRAL_WAVENUMBER, B, 1 );
    read_eps_integer4( &X->CH5_CONSTANT1, B, 1 );
    read_eps_integer4( &X->CH5_CONSTANT2_SLOPE, B, 1 );
  }
  return status_buffer( B );
}


int write_giadr_radiance( t_giadr_radiance *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* RADIANCE_CONVERSION */
    write_eps_byte( X->RAMP_CALIBRATION_COEFFICIENT, B, 2 );
    write_eps_uinteger2( &X->YEAR_RECENT_CALIBRATION, B, 1 );
    write_eps_uinteger2( &X->DAY_RECENT_CALIBRATION, B, 1 );
    write_eps_uinteger2( &X->PRIMARY_CALIBRATION_ALGORITHM_ID, B, 1 );
    write_eps_byte( X->PRIMARY_CALIBRATION_ALGORITHM_OPTION, B, 2 );
    write_eps_uinteger2( &X->SECONDARY_CALIBRATION_ALGORITHM_ID, B, 1 );
    write_eps_byte( X->SECONDARY_CALIBRATION_ALGORITHM_OPTION, B, 2 );
    write_eps_integer2( &X->IR_TEMPERATURE1_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE1_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE1_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE1_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE1_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE1_COEFFICIENT6, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE2_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE2_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE2_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE2_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE2_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE2_COEFFICIENT6, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE3_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE3_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE3_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE3_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE3_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE3_COEFFICIENT6, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE4_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE4_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE4_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE4_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE4_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->IR_TEMPERATURE4_COEFFICIENT6, B, 1 );
    write_eps_integer2( &X->CH1_SOLAR_FILTERED_IRRADIANCE, B, 1 );
    write_eps_integer2( &X->CH1_EQUIVALENT_FILTER_WIDTH, B, 1 );
    write_eps_integer2( &X->CH2_SOLAR_FILTERED_IRRADIANCE, B, 1 );
    write_eps_integer2( &X->CH2_EQUIVALENT_FILTER_WIDTH, B, 1 );
    write_eps_integer2( &X->CH3A_SOLAR_FILTERED_IRRADIANCE, B, 1 );
    write_eps_integer2( &X->CH3A_EQUIVALENT_FILTER_WIDTH, B, 1 );
    write_eps_integer4( &X->CH3B_CENTRAL_WAVENUMBER, B, 1 );
    write_eps_integer4( &X->CH3B_CONSTANT1, B, 1 );
    write_eps_integer4( &X->CH3B_CONSTANT2_SLOPE, B, 1 );
    write_eps_integer4( &X->CH4_CENTRAL_WAVENUMBER, B, 1 );
    write_eps_integer4( &X->CH4_CONSTANT1, B, 1 );
    write_eps_integer4( &X->CH4_CONSTANT2_SLOPE, B, 1 );
    write_eps_integer4( &X->CH5_CENTRAL_WAVENUMBER, B, 1 );
    write_eps_integer4( &X->CH5_CONSTANT1, B, 1 );
    write_eps_integer4( &X->CH5_CONSTANT2_SLOPE, B, 1 );
  }
  return status_buffer( B );
}


int free_giadr_radiance( t_giadr_radiance *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* RADIANCE_CONVERSION */
  }
}


int init_giadr_radiance( t_giadr_radiance *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* RADIANCE_CONVERSION */
  }
}


int size_giadr_radiance( t_giadr_radiance *X ) {
  int i;
  return 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4;
}



int read_giadr_analog( t_giadr_analog *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* A_D_CONVERSION */
    read_eps_integer2( &X->PATCH_TEMPERATURE_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->PATCH_TEMPERATURE_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->PATCH_TEMPERATURE_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->PATCH_TEMPERATURE_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->PATCH_TEMPERATURE_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->PATCH_TEMPERATURE_EXTENDED_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->PATCH_TEMPERATURE_EXTENDED_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->PATCH_TEMPERATURE_EXTENDED_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->PATCH_TEMPERATURE_EXTENDED_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->PATCH_TEMPERATURE_EXTENDED_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->PATCH_POWER_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->PATCH_POWER_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->PATCH_POWER_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->PATCH_POWER_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->PATCH_POWER_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->RADIATOR_TEMPERATURE_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->RADIATOR_TEMPERATURE_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->RADIATOR_TEMPERATURE_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->RADIATOR_TEMPERATURE_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->RADIATOR_TEMPERATURE_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->BLACKBODY_TEMPERATURE1_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->BLACKBODY_TEMPERATURE1_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->BLACKBODY_TEMPERATURE1_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->BLACKBODY_TEMPERATURE1_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->BLACKBODY_TEMPERATURE1_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->BLACKBODY_TEMPERATURE2_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->BLACKBODY_TEMPERATURE2_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->BLACKBODY_TEMPERATURE2_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->BLACKBODY_TEMPERATURE2_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->BLACKBODY_TEMPERATURE2_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->BLACKBODY_TEMPERATURE3_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->BLACKBODY_TEMPERATURE3_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->BLACKBODY_TEMPERATURE3_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->BLACKBODY_TEMPERATURE3_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->BLACKBODY_TEMPERATURE3_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->BLACKBODY_TEMPERATURE4_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->BLACKBODY_TEMPERATURE4_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->BLACKBODY_TEMPERATURE4_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->BLACKBODY_TEMPERATURE4_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->BLACKBODY_TEMPERATURE4_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->ELECTRONIC_CURRENT_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->ELECTRONIC_CURRENT_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->ELECTRONIC_CURRENT_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->ELECTRONIC_CURRENT_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->ELECTRONIC_CURRENT_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->MOTOR_CURRENT_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->MOTOR_CURRENT_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->MOTOR_CURRENT_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->MOTOR_CURRENT_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->MOTOR_CURRENT_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->EARTH_SHIELD_POSITION_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->EARTH_SHIELD_POSITION_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->EARTH_SHIELD_POSITION_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->EARTH_SHIELD_POSITION_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->EARTH_SHIELD_POSITION_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->ELECTRONIC_TEMPERATURE_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->ELECTRONIC_TEMPERATURE_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->ELECTRONIC_TEMPERATURE_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->ELECTRONIC_TEMPERATURE_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->ELECTRONIC_TEMPERATURE_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->COOLER_HOUSING_TEMPERATURE_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->COOLER_HOUSING_TEMPERATURE_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->COOLER_HOUSING_TEMPERATURE_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->COOLER_HOUSING_TEMPERATURE_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->COOLER_HOUSING_TEMPERATURE_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->BASEPLATE_TEMPERATURE_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->BASEPLATE_TEMPERATURE_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->BASEPLATE_TEMPERATURE_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->BASEPLATE_TEMPERATURE_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->BASEPLATE_TEMPERATURE_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->MOTOR_HOUSING_TEMPERATURE_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->MOTOR_HOUSING_TEMPERATURE_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->MOTOR_HOUSING_TEMPERATURE_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->MOTOR_HOUSING_TEMPERATURE_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->MOTOR_HOUSING_TEMPERATURE_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->AD_CONVERTER_TEMPERATURE_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->AD_CONVERTER_TEMPERATURE_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->AD_CONVERTER_TEMPERATURE_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->AD_CONVERTER_TEMPERATURE_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->AD_CONVERTER_TEMPERATURE_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->DETECTOR4_BIAS_VOLTAGE_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->DETECTOR4_BIAS_VOLTAGE_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->DETECTOR4_BIAS_VOLTAGE_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->DETECTOR4_BIAS_VOLTAGE_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->DETECTOR4_BIAS_VOLTAGE_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->DETECTOR5_BIAS_VOLTAGE_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->DETECTOR5_BIAS_VOLTAGE_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->DETECTOR5_BIAS_VOLTAGE_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->DETECTOR5_BIAS_VOLTAGE_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->DETECTOR5_BIAS_VOLTAGE_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->CH3B_BLACKBODY_VIEW_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->CH3B_BLACKBODY_VIEW_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->CH3B_BLACKBODY_VIEW_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->CH3B_BLACKBODY_VIEW_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->CH3B_BLACKBODY_VIEW_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->CH4_BLACKBODY_VIEW_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->CH4_BLACKBODY_VIEW_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->CH4_BLACKBODY_VIEW_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->CH4_BLACKBODY_VIEW_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->CH4_BLACKBODY_VIEW_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->CH5_BLACKBODY_VIEW_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->CH5_BLACKBODY_VIEW_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->CH5_BLACKBODY_VIEW_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->CH5_BLACKBODY_VIEW_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->CH5_BLACKBODY_VIEW_COEFFICIENT5, B, 1 );
    read_eps_integer2( &X->REFERENCE_VOLTAGE_COEFFICIENT1, B, 1 );
    read_eps_integer2( &X->REFERENCE_VOLTAGE_COEFFICIENT2, B, 1 );
    read_eps_integer2( &X->REFERENCE_VOLTAGE_COEFFICIENT3, B, 1 );
    read_eps_integer2( &X->REFERENCE_VOLTAGE_COEFFICIENT4, B, 1 );
    read_eps_integer2( &X->REFERENCE_VOLTAGE_COEFFICIENT5, B, 1 );
  }
  return status_buffer( B );
}


int write_giadr_analog( t_giadr_analog *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* A_D_CONVERSION */
    write_eps_integer2( &X->PATCH_TEMPERATURE_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->PATCH_TEMPERATURE_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->PATCH_TEMPERATURE_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->PATCH_TEMPERATURE_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->PATCH_TEMPERATURE_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->PATCH_TEMPERATURE_EXTENDED_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->PATCH_TEMPERATURE_EXTENDED_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->PATCH_TEMPERATURE_EXTENDED_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->PATCH_TEMPERATURE_EXTENDED_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->PATCH_TEMPERATURE_EXTENDED_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->PATCH_POWER_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->PATCH_POWER_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->PATCH_POWER_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->PATCH_POWER_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->PATCH_POWER_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->RADIATOR_TEMPERATURE_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->RADIATOR_TEMPERATURE_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->RADIATOR_TEMPERATURE_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->RADIATOR_TEMPERATURE_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->RADIATOR_TEMPERATURE_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->BLACKBODY_TEMPERATURE1_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->BLACKBODY_TEMPERATURE1_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->BLACKBODY_TEMPERATURE1_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->BLACKBODY_TEMPERATURE1_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->BLACKBODY_TEMPERATURE1_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->BLACKBODY_TEMPERATURE2_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->BLACKBODY_TEMPERATURE2_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->BLACKBODY_TEMPERATURE2_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->BLACKBODY_TEMPERATURE2_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->BLACKBODY_TEMPERATURE2_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->BLACKBODY_TEMPERATURE3_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->BLACKBODY_TEMPERATURE3_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->BLACKBODY_TEMPERATURE3_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->BLACKBODY_TEMPERATURE3_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->BLACKBODY_TEMPERATURE3_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->BLACKBODY_TEMPERATURE4_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->BLACKBODY_TEMPERATURE4_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->BLACKBODY_TEMPERATURE4_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->BLACKBODY_TEMPERATURE4_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->BLACKBODY_TEMPERATURE4_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->ELECTRONIC_CURRENT_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->ELECTRONIC_CURRENT_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->ELECTRONIC_CURRENT_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->ELECTRONIC_CURRENT_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->ELECTRONIC_CURRENT_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->MOTOR_CURRENT_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->MOTOR_CURRENT_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->MOTOR_CURRENT_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->MOTOR_CURRENT_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->MOTOR_CURRENT_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->EARTH_SHIELD_POSITION_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->EARTH_SHIELD_POSITION_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->EARTH_SHIELD_POSITION_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->EARTH_SHIELD_POSITION_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->EARTH_SHIELD_POSITION_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->ELECTRONIC_TEMPERATURE_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->ELECTRONIC_TEMPERATURE_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->ELECTRONIC_TEMPERATURE_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->ELECTRONIC_TEMPERATURE_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->ELECTRONIC_TEMPERATURE_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->COOLER_HOUSING_TEMPERATURE_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->COOLER_HOUSING_TEMPERATURE_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->COOLER_HOUSING_TEMPERATURE_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->COOLER_HOUSING_TEMPERATURE_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->COOLER_HOUSING_TEMPERATURE_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->BASEPLATE_TEMPERATURE_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->BASEPLATE_TEMPERATURE_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->BASEPLATE_TEMPERATURE_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->BASEPLATE_TEMPERATURE_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->BASEPLATE_TEMPERATURE_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->MOTOR_HOUSING_TEMPERATURE_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->MOTOR_HOUSING_TEMPERATURE_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->MOTOR_HOUSING_TEMPERATURE_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->MOTOR_HOUSING_TEMPERATURE_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->MOTOR_HOUSING_TEMPERATURE_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->AD_CONVERTER_TEMPERATURE_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->AD_CONVERTER_TEMPERATURE_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->AD_CONVERTER_TEMPERATURE_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->AD_CONVERTER_TEMPERATURE_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->AD_CONVERTER_TEMPERATURE_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->DETECTOR4_BIAS_VOLTAGE_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->DETECTOR4_BIAS_VOLTAGE_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->DETECTOR4_BIAS_VOLTAGE_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->DETECTOR4_BIAS_VOLTAGE_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->DETECTOR4_BIAS_VOLTAGE_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->DETECTOR5_BIAS_VOLTAGE_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->DETECTOR5_BIAS_VOLTAGE_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->DETECTOR5_BIAS_VOLTAGE_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->DETECTOR5_BIAS_VOLTAGE_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->DETECTOR5_BIAS_VOLTAGE_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->CH3B_BLACKBODY_VIEW_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->CH3B_BLACKBODY_VIEW_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->CH3B_BLACKBODY_VIEW_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->CH3B_BLACKBODY_VIEW_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->CH3B_BLACKBODY_VIEW_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->CH4_BLACKBODY_VIEW_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->CH4_BLACKBODY_VIEW_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->CH4_BLACKBODY_VIEW_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->CH4_BLACKBODY_VIEW_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->CH4_BLACKBODY_VIEW_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->CH5_BLACKBODY_VIEW_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->CH5_BLACKBODY_VIEW_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->CH5_BLACKBODY_VIEW_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->CH5_BLACKBODY_VIEW_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->CH5_BLACKBODY_VIEW_COEFFICIENT5, B, 1 );
    write_eps_integer2( &X->REFERENCE_VOLTAGE_COEFFICIENT1, B, 1 );
    write_eps_integer2( &X->REFERENCE_VOLTAGE_COEFFICIENT2, B, 1 );
    write_eps_integer2( &X->REFERENCE_VOLTAGE_COEFFICIENT3, B, 1 );
    write_eps_integer2( &X->REFERENCE_VOLTAGE_COEFFICIENT4, B, 1 );
    write_eps_integer2( &X->REFERENCE_VOLTAGE_COEFFICIENT5, B, 1 );
  }
  return status_buffer( B );
}


int free_giadr_analog( t_giadr_analog *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* A_D_CONVERSION */
  }
}


int init_giadr_analog( t_giadr_analog *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* A_D_CONVERSION */
  }
}


int size_giadr_analog( t_giadr_analog *X ) {
  int i;
  return 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2;
}



int read_mdr_1b( t_mdr_1b *X, buffer* B, int n ) {
  int i, i_NE, i_NP;
  for( i = 0; i < n; i++, X++ ) {
    /* GENERIC_QUALITY_INDICATORS */
    read_eps_boolean( &X->DEGRADED_INST_MDR, B, 1 );
    read_eps_boolean( &X->DEGRADED_PROC_MDR, B, 1 );
    /* MEASUREMENT_DATA */
    read_eps_integer2( &X->EARTH_VIEWS_PER_SCANLINE, B, 1 );
    i_NE = X->EARTH_VIEWS_PER_SCANLINE;
    read_eps_integer2( (eps_integer2*)X->SCENE_RADIANCES, B, ( 5 )*( i_NE ) );
    /* NAVIGATION_DATA_AT_SCAN_LINE */
    read_eps_uinteger4( &X->TIME_ATTITUDE, B, 1 );
    read_eps_integer2( (eps_integer2*)X->EULER_ANGLE, B, ( 3 ) );
    read_eps_byte( X->NAVIGATION_STATUS, B, 4 );
    read_eps_uinteger4( &X->SPACECRAFT_ALTITUDE, B, 1 );
    read_eps_integer2( (eps_integer2*)X->ANGULAR_RELATIONS_FIRST, B, ( 4 ) );
    read_eps_integer2( (eps_integer2*)X->ANGULAR_RELATIONS_LAST, B, ( 4 ) );
    read_eps_integer4( (eps_integer4*)X->EARTH_LOCATION_FIRST, B, ( 2 ) );
    read_eps_integer4( (eps_integer4*)X->EARTH_LOCATION_LAST, B, ( 2 ) );
    read_eps_integer2( &X->NUM_NAVIGATION_POINTS, B, 1 );
    i_NP = X->NUM_NAVIGATION_POINTS;
    read_eps_integer2( (eps_integer2*)X->ANGULAR_RELATIONS, B, ( i_NP )*( 4 ) );
    read_eps_integer4( (eps_integer4*)X->EARTH_LOCATIONS, B, ( i_NP )*( 2 ) );
    read_eps_byte( X->QUALITY_INDICATOR, B, 4 );
    read_eps_byte( X->SCAN_LINE_QUALITY, B, 4 );
    read_eps_byte( X->CALIBRATION_QUALITY, B, 2 * ( 3 ) );
/* In EUMETSAT's Feb 2014 version of the AVHRR processor, bits 8-15 are used to store NEDT values, and 
   bit 3 indicates NEDT exceeds specification. These are not required in AAPP, so set to zero.
   03/02/16: to avoid losing the NEDT information, comment out this bit and instead modify convert_avh1b.F */
/*    X->CALIBRATION_QUALITY[0][0] = 0;
    X->CALIBRATION_QUALITY[1][0] = 0;
    X->CALIBRATION_QUALITY[2][0] = 0;
    X->CALIBRATION_QUALITY[0][1] &= 247;  
    X->CALIBRATION_QUALITY[1][1] &= 247;  
    X->CALIBRATION_QUALITY[2][1] &= 247; */
    read_eps_uinteger2( &X->COUNT_ERROR_FRAME, B, 1 );
    /* Calibration_Coefficients */
    read_eps_integer4( (eps_integer4*)X->CH123A_CURVE_SLOPE1, B, ( 3 ) );
    read_eps_integer4( (eps_integer4*)X->CH123A_CURVE_INTERCEPT1, B, ( 3 ) );
    read_eps_integer4( (eps_integer4*)X->CH123A_CURVE_SLOPE2, B, ( 3 ) );
    read_eps_integer4( (eps_integer4*)X->CH123A_CURVE_INTERCEPT2, B, ( 3 ) );
    read_eps_integer4( (eps_integer4*)X->CH123A_CURVE_INTERCEPTION, B, ( 3 ) );
    read_eps_integer4( (eps_integer4*)X->CH123A_TEST_CURVE_SLOPE1, B, ( 3 ) );
    read_eps_integer4( (eps_integer4*)X->CH123A_TEST_CURVE_INTERCEPT1, B, ( 3 ) );
    read_eps_integer4( (eps_integer4*)X->CH123A_TEST_CURVE_SLOPE2, B, ( 3 ) );
    read_eps_integer4( (eps_integer4*)X->CH123A_TEST_CURVE_INTERCEPT2, B, ( 3 ) );
    read_eps_integer4( (eps_integer4*)X->CH123A_TEST_CURVE_INTERCEPTION, B, ( 3 ) );
    read_eps_integer4( (eps_integer4*)X->CH123A_PRELAUNCH_CURVE_SLOPE1, B, ( 3 ) );
    read_eps_integer4( (eps_integer4*)X->CH123A_PRELAUNCH_CURVE_INTERCEPT1, B, ( 3 ) );
    read_eps_integer4( (eps_integer4*)X->CH123A_PRELAUNCH_CURVE_SLOPE2, B, ( 3 ) );
    read_eps_integer4( (eps_integer4*)X->CH123A_PRELAUNCH_CURVE_INTERCEPT2, B, ( 3 ) );
    read_eps_integer4( (eps_integer4*)X->CH123A_PRELAUNCH_CURVE_INTERCEPTION, B, ( 3 ) );
    read_eps_integer4( (eps_integer4*)X->CH3B45_SECOND_TERM, B, ( 3 ) );
    read_eps_integer4( (eps_integer4*)X->CH3B45_FIRST_TERM, B, ( 3 ) );
    read_eps_integer4( (eps_integer4*)X->CH3B45_ZEROTH_TERM, B, ( 3 ) );
    read_eps_integer4( (eps_integer4*)X->CH3B45_TEST_SECOND_TERM, B, ( 3 ) );
    read_eps_integer4( (eps_integer4*)X->CH3B45_TEST_FIRST_TERM, B, ( 3 ) );
    read_eps_integer4( (eps_integer4*)X->CH3B45_TEST_ZEROTH_TERM, B, ( 3 ) );
    read_eps_byte( X->CLOUD_INFORMATION, B, 2 * ( i_NE ) );
    /* DIGITAL_A_Telemetry */
    read_eps_uinteger2( (eps_uinteger2*)X->FRAME_SYNCHRONISATION, B, ( 6 ) );
    read_eps_byte( X->FRAME_INDICATOR, B, 4 );
    read_eps_byte( X->TIME_CODE, B, 8 );
    read_eps_uinteger2( (eps_uinteger2*)X->RAMP_CALIB, B, ( 5 ) );
    read_eps_uinteger2( (eps_uinteger2*)X->INTERNAL_TARGET_TEMPERATURE_COUNT, B, ( 3 ) );
    /* Digital_B_telemetry */
    read_eps_byte( X->INSTRUMENT_INVALID_WORD_FLAG, B, 2 );
    read_eps_byte( X->DIGITAL_B_DATA, B, 2 );
    /* Analog_housekeeping_data */
    read_eps_byte( X->INSTRUMENT_INVALID_ANALOG_WORD_FLAG, B, 4 );
    read_eps_uinteger2( &X->PATCH_TEMPERATURE, B, 1 );
    read_eps_uinteger2( &X->PATCH_EXTENDED_TEMPERATURE, B, 1 );
    read_eps_uinteger2( &X->PATCH_POWER, B, 1 );
    read_eps_uinteger2( &X->RADIATOR_TEMPERATURE, B, 1 );
    read_eps_uinteger2( &X->BLACKBODY_TEMPERATURE1, B, 1 );
    read_eps_uinteger2( &X->BLACKBODY_TEMPERATURE2, B, 1 );
    read_eps_uinteger2( &X->BLACKBODY_TEMPERATURE3, B, 1 );
    read_eps_uinteger2( &X->BLACKBODY_TEMPERATURE4, B, 1 );
    read_eps_uinteger2( &X->ELECTRONIC_CURRENT, B, 1 );
    read_eps_uinteger2( &X->MOTOR_CURRENT, B, 1 );
    read_eps_uinteger2( &X->EARTH_SHIELD_POSITION, B, 1 );
    read_eps_uinteger2( &X->ELECTRONIC_TEMPERATURE, B, 1 );
    read_eps_uinteger2( &X->COOLER_HOUSING_TEMPERATURE, B, 1 );
    read_eps_uinteger2( &X->BASEPLATE_TEMPERATURE, B, 1 );
    read_eps_uinteger2( &X->MOTOR_HOUSING_TEMPERATURE, B, 1 );
    read_eps_uinteger2( &X->AD_CONVERTER_TEMPERATURE, B, 1 );
    read_eps_uinteger2( &X->DETECTOR4_VOLTAGE, B, 1 );
    read_eps_uinteger2( &X->DETECTOR5_VOLTAGE, B, 1 );
    read_eps_uinteger2( &X->CH3_BLACKBODY_VIEW, B, 1 );
    read_eps_uinteger2( &X->CH4_BLACKBODY_VIEW, B, 1 );
    read_eps_uinteger2( &X->CH5_BLACKBODY_VIEW, B, 1 );
    read_eps_uinteger2( &X->REFERENCE_VOLTAGE, B, 1 );
  }
  return status_buffer( B );
}


int write_mdr_1b( t_mdr_1b *X, buffer* B, int n ) {
  int i, i_NE, i_NP;
  for( i = 0; i < n; i++, X++ ) {
    /* GENERIC_QUALITY_INDICATORS */
    write_eps_boolean( &X->DEGRADED_INST_MDR, B, 1 );
    write_eps_boolean( &X->DEGRADED_PROC_MDR, B, 1 );
    /* MEASUREMENT_DATA */
    write_eps_integer2( &X->EARTH_VIEWS_PER_SCANLINE, B, 1 );
    i_NE = X->EARTH_VIEWS_PER_SCANLINE;
    write_eps_integer2( (eps_integer2*)X->SCENE_RADIANCES, B, ( 5 )*( i_NE ) );
    /* NAVIGATION_DATA_AT_SCAN_LINE */
    write_eps_uinteger4( &X->TIME_ATTITUDE, B, 1 );
    write_eps_integer2( (eps_integer2*)X->EULER_ANGLE, B, ( 3 ) );
    write_eps_byte( X->NAVIGATION_STATUS, B, 4 );
    write_eps_uinteger4( &X->SPACECRAFT_ALTITUDE, B, 1 );
    write_eps_integer2( (eps_integer2*)X->ANGULAR_RELATIONS_FIRST, B, ( 4 ) );
    write_eps_integer2( (eps_integer2*)X->ANGULAR_RELATIONS_LAST, B, ( 4 ) );
    write_eps_integer4( (eps_integer4*)X->EARTH_LOCATION_FIRST, B, ( 2 ) );
    write_eps_integer4( (eps_integer4*)X->EARTH_LOCATION_LAST, B, ( 2 ) );
    write_eps_integer2( &X->NUM_NAVIGATION_POINTS, B, 1 );
    i_NP = X->NUM_NAVIGATION_POINTS;
    write_eps_integer2( (eps_integer2*)X->ANGULAR_RELATIONS, B, ( i_NP )*( 4 ) );
    write_eps_integer4( (eps_integer4*)X->EARTH_LOCATIONS, B, ( i_NP )*( 2 ) );
    write_eps_byte( X->QUALITY_INDICATOR, B, 4 );
    write_eps_byte( X->SCAN_LINE_QUALITY, B, 4 );
    write_eps_byte( X->CALIBRATION_QUALITY, B, 2 * ( 3 ) );
    write_eps_uinteger2( &X->COUNT_ERROR_FRAME, B, 1 );
    /* Calibration_Coefficients */
    write_eps_integer4( (eps_integer4*)X->CH123A_CURVE_SLOPE1, B, ( 3 ) );
    write_eps_integer4( (eps_integer4*)X->CH123A_CURVE_INTERCEPT1, B, ( 3 ) );
    write_eps_integer4( (eps_integer4*)X->CH123A_CURVE_SLOPE2, B, ( 3 ) );
    write_eps_integer4( (eps_integer4*)X->CH123A_CURVE_INTERCEPT2, B, ( 3 ) );
    write_eps_integer4( (eps_integer4*)X->CH123A_CURVE_INTERCEPTION, B, ( 3 ) );
    write_eps_integer4( (eps_integer4*)X->CH123A_TEST_CURVE_SLOPE1, B, ( 3 ) );
    write_eps_integer4( (eps_integer4*)X->CH123A_TEST_CURVE_INTERCEPT1, B, ( 3 ) );
    write_eps_integer4( (eps_integer4*)X->CH123A_TEST_CURVE_SLOPE2, B, ( 3 ) );
    write_eps_integer4( (eps_integer4*)X->CH123A_TEST_CURVE_INTERCEPT2, B, ( 3 ) );
    write_eps_integer4( (eps_integer4*)X->CH123A_TEST_CURVE_INTERCEPTION, B, ( 3 ) );
    write_eps_integer4( (eps_integer4*)X->CH123A_PRELAUNCH_CURVE_SLOPE1, B, ( 3 ) );
    write_eps_integer4( (eps_integer4*)X->CH123A_PRELAUNCH_CURVE_INTERCEPT1, B, ( 3 ) );
    write_eps_integer4( (eps_integer4*)X->CH123A_PRELAUNCH_CURVE_SLOPE2, B, ( 3 ) );
    write_eps_integer4( (eps_integer4*)X->CH123A_PRELAUNCH_CURVE_INTERCEPT2, B, ( 3 ) );
    write_eps_integer4( (eps_integer4*)X->CH123A_PRELAUNCH_CURVE_INTERCEPTION, B, ( 3 ) );
    write_eps_integer4( (eps_integer4*)X->CH3B45_SECOND_TERM, B, ( 3 ) );
    write_eps_integer4( (eps_integer4*)X->CH3B45_FIRST_TERM, B, ( 3 ) );
    write_eps_integer4( (eps_integer4*)X->CH3B45_ZEROTH_TERM, B, ( 3 ) );
    write_eps_integer4( (eps_integer4*)X->CH3B45_TEST_SECOND_TERM, B, ( 3 ) );
    write_eps_integer4( (eps_integer4*)X->CH3B45_TEST_FIRST_TERM, B, ( 3 ) );
    write_eps_integer4( (eps_integer4*)X->CH3B45_TEST_ZEROTH_TERM, B, ( 3 ) );
    write_eps_byte( X->CLOUD_INFORMATION, B, 2 * ( i_NE ) );
    /* DIGITAL_A_Telemetry */
    write_eps_uinteger2( (eps_uinteger2*)X->FRAME_SYNCHRONISATION, B, ( 6 ) );
    write_eps_byte( X->FRAME_INDICATOR, B, 4 );
    write_eps_byte( X->TIME_CODE, B, 8 );
    write_eps_uinteger2( (eps_uinteger2*)X->RAMP_CALIB, B, ( 5 ) );
    write_eps_uinteger2( (eps_uinteger2*)X->INTERNAL_TARGET_TEMPERATURE_COUNT, B, ( 3 ) );
    /* Digital_B_telemetry */
    write_eps_byte( X->INSTRUMENT_INVALID_WORD_FLAG, B, 2 );
    write_eps_byte( X->DIGITAL_B_DATA, B, 2 );
    /* Analog_housekeeping_data */
    write_eps_byte( X->INSTRUMENT_INVALID_ANALOG_WORD_FLAG, B, 4 );
    write_eps_uinteger2( &X->PATCH_TEMPERATURE, B, 1 );
    write_eps_uinteger2( &X->PATCH_EXTENDED_TEMPERATURE, B, 1 );
    write_eps_uinteger2( &X->PATCH_POWER, B, 1 );
    write_eps_uinteger2( &X->RADIATOR_TEMPERATURE, B, 1 );
    write_eps_uinteger2( &X->BLACKBODY_TEMPERATURE1, B, 1 );
    write_eps_uinteger2( &X->BLACKBODY_TEMPERATURE2, B, 1 );
    write_eps_uinteger2( &X->BLACKBODY_TEMPERATURE3, B, 1 );
    write_eps_uinteger2( &X->BLACKBODY_TEMPERATURE4, B, 1 );
    write_eps_uinteger2( &X->ELECTRONIC_CURRENT, B, 1 );
    write_eps_uinteger2( &X->MOTOR_CURRENT, B, 1 );
    write_eps_uinteger2( &X->EARTH_SHIELD_POSITION, B, 1 );
    write_eps_uinteger2( &X->ELECTRONIC_TEMPERATURE, B, 1 );
    write_eps_uinteger2( &X->COOLER_HOUSING_TEMPERATURE, B, 1 );
    write_eps_uinteger2( &X->BASEPLATE_TEMPERATURE, B, 1 );
    write_eps_uinteger2( &X->MOTOR_HOUSING_TEMPERATURE, B, 1 );
    write_eps_uinteger2( &X->AD_CONVERTER_TEMPERATURE, B, 1 );
    write_eps_uinteger2( &X->DETECTOR4_VOLTAGE, B, 1 );
    write_eps_uinteger2( &X->DETECTOR5_VOLTAGE, B, 1 );
    write_eps_uinteger2( &X->CH3_BLACKBODY_VIEW, B, 1 );
    write_eps_uinteger2( &X->CH4_BLACKBODY_VIEW, B, 1 );
    write_eps_uinteger2( &X->CH5_BLACKBODY_VIEW, B, 1 );
    write_eps_uinteger2( &X->REFERENCE_VOLTAGE, B, 1 );
  }
  return status_buffer( B );
}


int free_mdr_1b( t_mdr_1b *X, int n ) {
  int i, i_NE, i_NP;
  for( i = 0; i < n; i++, X++ ) {
    /* GENERIC_QUALITY_INDICATORS */
    /* MEASUREMENT_DATA */
    /* NAVIGATION_DATA_AT_SCAN_LINE */
    /* Calibration_Coefficients */
    /* DIGITAL_A_Telemetry */
    /* Digital_B_telemetry */
    /* Analog_housekeeping_data */
  }
}


int init_mdr_1b( t_mdr_1b *X, int n ) {
  int i, i_NE, i_NP;
  for( i = 0; i < n; i++, X++ ) {
    /* GENERIC_QUALITY_INDICATORS */
    /* MEASUREMENT_DATA */
    i_NE = X->EARTH_VIEWS_PER_SCANLINE;
    /* NAVIGATION_DATA_AT_SCAN_LINE */
    i_NP = X->NUM_NAVIGATION_POINTS;
    /* Calibration_Coefficients */
    /* DIGITAL_A_Telemetry */
    /* Digital_B_telemetry */
    /* Analog_housekeeping_data */
  }
}


int size_mdr_1b( t_mdr_1b *X ) {
  int i, i_NE, i_NP;
  i_NE = X->EARTH_VIEWS_PER_SCANLINE;
  i_NP = X->NUM_NAVIGATION_POINTS;
  return 1 + 1 + 2 + ( 5 )*( i_NE ) * 2 + 4 + ( 3 ) * 2 + 4 + 4 + ( 4 ) * 2 + ( 4 ) * 2 + ( 2 ) * 4 + ( 2 ) * 4 + 2 + ( i_NP )*( 4 ) * 2 + ( i_NP )*( 2 ) * 4 + 4 + 4 + 3 * 2 + 2 + ( 3 ) * 4 + ( 3 ) * 4 + ( 3 ) * 4 + ( 3 ) * 4 + ( 3 ) * 4 + ( 3 ) * 4 + ( 3 ) * 4 + ( 3 ) * 4 + ( 3 ) * 4 + ( 3 ) * 4 + ( 3 ) * 4 + ( 3 ) * 4 + ( 3 ) * 4 + ( 3 ) * 4 + ( 3 ) * 4 + ( 3 ) * 4 + ( 3 ) * 4 + ( 3 ) * 4 + ( 3 ) * 4 + ( 3 ) * 4 + ( 3 ) * 4 + i_NE * 2 + ( 6 ) * 2 + 4 + 8 + ( 5 ) * 2 + ( 3 ) * 2 + 2 + 2 + 4 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2 + 2;
}



int get_eps_avhrrl1b_6_5( t_eps_avhrrl1b_6_5 *R, FILE *fp ) {
  buffer B;
  
   read_grh( &( R->grh ), fp );
 if( R->grh.instrument_group == DUMMY ) {
   fseek( fp, R->grh.record_size - SIZE_OF_REC_HEAD, SEEK_CUR );
   return 1;        
 }

 if( R->grh.record_class == GEADR ) {
   fseek( fp, R->grh.record_size - SIZE_OF_REC_HEAD, SEEK_CUR );
   return 1;        
 }

 if( R->grh.record_class == VEADR ) {
   fseek( fp, R->grh.record_size - SIZE_OF_REC_HEAD, SEEK_CUR );
   return 1;        
 }


  
  if( 
      ( R->grh.record_class == MPHR )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass ) || R->grh.record_subclass == 0 )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass_version ) || R->grh.record_subclass_version == 2 )
  ) {
  
    init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
    fread( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );
    R->content.MPHR = (t_mphr*)getmem( sizeof( t_mphr ) );
    read_mphr( R->content.MPHR, &B, 1 );
    free_buffer( &B );
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == SPHR )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass ) || R->grh.record_subclass == 0 )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass_version ) || R->grh.record_subclass_version == 3 )
  ) {
  
    init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
    fread( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );
    R->content.SPHR = (t_sphr*)getmem( sizeof( t_sphr ) );
    read_sphr( R->content.SPHR, &B, 1 );
    free_buffer( &B );
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == IPR )
  ) {
  
    init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
    fread( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );
    R->content.IPR = (t_ipr*)getmem( sizeof( t_ipr ) );
    read_ipr( R->content.IPR, &B, 1 );
    free_buffer( &B );
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == GIADR )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass ) || R->grh.record_subclass == 1 )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass_version ) || R->grh.record_subclass_version == 3 )
  ) {
  
    init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
    fread( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );
    R->content.GIADR_RADIANCE = (t_giadr_radiance*)getmem( sizeof( t_giadr_radiance ) );
    read_giadr_radiance( R->content.GIADR_RADIANCE, &B, 1 );
    free_buffer( &B );
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == GIADR )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass ) || R->grh.record_subclass == 2 )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass_version ) || R->grh.record_subclass_version == 2 )
  ) {
  
    init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
    fread( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );
    R->content.GIADR_ANALOG = (t_giadr_analog*)getmem( sizeof( t_giadr_analog ) );
    read_giadr_analog( R->content.GIADR_ANALOG, &B, 1 );
    free_buffer( &B );
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == MDR )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass ) || R->grh.record_subclass == 2 )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass_version ) || R->grh.record_subclass_version == 4 )
  ) {
  
    init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
    fread( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );
    R->content.MDR_1B = (t_mdr_1b*)getmem( sizeof( t_mdr_1b ) );
    read_mdr_1b( R->content.MDR_1B, &B, 1 );
    free_buffer( &B );
    return 1;

  
  }
  

  fseek( fp, R->grh.record_size - SIZE_OF_REC_HEAD, SEEK_CUR );

  if( R->grh.record_class == GIADR ) return 1;  /* Allow for extra GIADR in UMARF */
  
  return 0;

}


int put_eps_avhrrl1b_6_5( t_eps_avhrrl1b_6_5 *R, FILE *fp ) {
  buffer B;
  
  

  
  if( 
      ( R->grh.record_class == MPHR )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass ) || R->grh.record_subclass == 0 )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass_version ) || R->grh.record_subclass_version == 2 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      R->grh.record_size = size_mphr( R->content.MPHR ) + SIZE_OF_REC_HEAD;
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_mphr( R->content.MPHR, &B, 1 );

      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    } else {
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    }
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == SPHR )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass ) || R->grh.record_subclass == 0 )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass_version ) || R->grh.record_subclass_version == 3 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      R->grh.record_size = size_sphr( R->content.SPHR ) + SIZE_OF_REC_HEAD;
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_sphr( R->content.SPHR, &B, 1 );

      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    } else {
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    }
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == IPR )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      R->grh.record_size = size_ipr( R->content.IPR ) + SIZE_OF_REC_HEAD;
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_ipr( R->content.IPR, &B, 1 );

      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    } else {
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    }
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == GIADR )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass ) || R->grh.record_subclass == 1 )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass_version ) || R->grh.record_subclass_version == 3 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      R->grh.record_size = size_giadr_radiance( R->content.GIADR_RADIANCE ) + SIZE_OF_REC_HEAD;
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_giadr_radiance( R->content.GIADR_RADIANCE, &B, 1 );

      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    } else {
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    }
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == GIADR )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass ) || R->grh.record_subclass == 2 )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass_version ) || R->grh.record_subclass_version == 2 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      R->grh.record_size = size_giadr_analog( R->content.GIADR_ANALOG ) + SIZE_OF_REC_HEAD;
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_giadr_analog( R->content.GIADR_ANALOG, &B, 1 );

      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    } else {
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    }
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == MDR )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass ) || R->grh.record_subclass == 2 )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass_version ) || R->grh.record_subclass_version == 4 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      R->grh.record_size = size_mdr_1b( R->content.MDR_1B ) + SIZE_OF_REC_HEAD;
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_mdr_1b( R->content.MDR_1B, &B, 1 );

      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    } else {
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    }
    return 1;

  
  }
  

  

  return 0;

}


int free_eps_avhrrl1b_6_5( t_eps_avhrrl1b_6_5 *R ) {
  buffer B;
  
  

  
  if( 
      ( R->grh.record_class == MPHR )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass ) || R->grh.record_subclass == 0 )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass_version ) || R->grh.record_subclass_version == 2 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      free_mphr( R->content.MPHR, 1 );
      free( R->content.MPHR );
    } 
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == SPHR )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass ) || R->grh.record_subclass == 0 )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass_version ) || R->grh.record_subclass_version == 3 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      free_sphr( R->content.SPHR, 1 );
      free( R->content.SPHR );
    } 
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == IPR )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      free_ipr( R->content.IPR, 1 );
      free( R->content.IPR );
    } 
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == GIADR )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass ) || R->grh.record_subclass == 1 )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass_version ) || R->grh.record_subclass_version == 3 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      free_giadr_radiance( R->content.GIADR_RADIANCE, 1 );
      free( R->content.GIADR_RADIANCE );
    } 
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == GIADR )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass ) || R->grh.record_subclass == 2 )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass_version ) || R->grh.record_subclass_version == 2 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      free_giadr_analog( R->content.GIADR_ANALOG, 1 );
      free( R->content.GIADR_ANALOG );
    } 
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == MDR )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass ) || R->grh.record_subclass == 2 )
   && ( ( ! eps_avhrrl1b_6_5_check_subclass_version ) || R->grh.record_subclass_version == 4 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      free_mdr_1b( R->content.MDR_1B, 1 );
      free( R->content.MDR_1B );
    } 
    return 1;

  
  }
  

  

  return 0;

}

