/* Name: $Id$
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/5
 *
 * SYNOPSIS  : Generic record header decoding/encoding 
 *             routines
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * VERSION   : 1.0
 * HISTORY   : 01/09/2018 : (P. Brunel) Add hdf5 function  
 *
 */
#include "eps_grh.h"
#include "eps_buffer.h"
#include <stdio.h>
#ifdef HAS_LIBHDF5
#include <hdf5.h>
#endif

#define readint( a, t, n, b ) read_int_buffer( a, t, n, b )

#define writeint( a, t, n, b ) write_int_buffer( a, t, n, b )


int read_grh( REC_HEAD *R, FILE* fpr ) {
  buffer B;
  
  init_buffer( &B, SIZE_OF_REC_HEAD );

  fread( B.p, SIZE_OF_REC_HEAD, 1, fpr );
  
  readint( &(*R).record_class, 1, 1, &B );
  readint( &(*R).instrument_group, 1, 1, &B );
  readint( &(*R).record_subclass, 1, 1, &B );
  readint( &(*R).record_subclass_version, 1, 1, &B );

  readint( &(*R).record_size, 4, 1, &B );

  readint( &(*R).record_start.days, 2, 1, &B );
  readint( &(*R).record_start.milliseconds, 4, 1, &B );

  readint( &(*R).record_stop.days, 2, 1, &B );
  readint( &(*R).record_stop.milliseconds, 4, 1, &B );
  
  free_buffer( &B );

}

int write_grh( REC_HEAD *R, FILE* fpw ) {
  buffer B;
  
  init_buffer( &B, SIZE_OF_REC_HEAD );
  
  writeint( &(*R).record_class, 1, 1, &B );
  writeint( &(*R).instrument_group, 1, 1, &B );
  writeint( &(*R).record_subclass, 1, 1, &B );
  writeint( &(*R).record_subclass_version, 1, 1, &B );

  writeint( &(*R).record_size, 4, 1, &B );

  writeint( &(*R).record_start.days, 2, 1, &B );
  writeint( &(*R).record_start.milliseconds, 4, 1, &B );

  writeint( &(*R).record_stop.days, 2, 1, &B );
  writeint( &(*R).record_stop.milliseconds, 4, 1, &B );
  
  fwrite( B.p, SIZE_OF_REC_HEAD, 1, fpw );
  
  free_buffer( &B );

}
#ifdef HAS_LIBHDF5
int write_grh_hdf5( REC_HEAD *R, hid_t gid ){

   hsize_t dims[1];
   herr_t      status; 
//typedef struct {
    //eps_uinteger2 days;
    //eps_uinteger4 milliseconds;
//} ShortCDSTime;

//typedef struct {
    //eps_uinteger1 record_class;
    //eps_uinteger1 instrument_group;
    //eps_uinteger1 record_subclass;
    //eps_uinteger1 record_subclass_version;
    //eps_uinteger4 record_size;
    //ShortCDSTime record_start;
    //ShortCDSTime record_stop;
//} REC_HEAD;
   dims[0] = 1;
   status = write_epsh5_uinteger1( &(*R).record_class, gid, 1, dims, "record_class" );
   status = write_epsh5_uinteger1( &(*R).instrument_group, gid, 1, dims, "instrument_group" );
   status = write_epsh5_uinteger1( &(*R).record_subclass, gid, 1, dims, "record_subclass" );
   status = write_epsh5_uinteger1( &(*R).record_subclass_version, gid, 1, dims, "record_subclass_version" );
   status = write_epsh5_uinteger4( &(*R).record_size, gid, 1, dims, "record_size" );
   status = write_epsh5_ShortCDSTime( &(*R).record_start, gid, 1, dims, "record_start" );
   status = write_epsh5_ShortCDSTime( &(*R).record_stop, gid, 1, dims, "record_stop" );

   return 0;

}
#endif
