/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/5
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * VERSION   : 1.1
 *
 * HISTORY   : 01/05/2005 : Initial
 *             24/05/2010 : Allow for extra GIADR in UMARF files. NCA
 * 
 */
#include "eps_metopl0.h"

int eps_metopl0_check_subclass = 1;
int eps_metopl0_check_subclass_version = 1;


int read_mphr( t_mphr *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    while( 1 ) {
      const char* label;
      int len;
      if( ! read_label( &label, &len, B ) ) break;
      if( ( len == 12 ) && ( ! strncmp( label, "PRODUCT_NAME", 12 ) ) ) {
        read_ascii_string( X->PRODUCT_NAME, B, 67 );
      } else if( ( len == 21 ) && ( ! strncmp( label, "PARENT_PRODUCT_NAME_1", 21 ) ) ) {
        read_ascii_string( X->PARENT_PRODUCT_NAME_1, B, 67 );
      } else if( ( len == 21 ) && ( ! strncmp( label, "PARENT_PRODUCT_NAME_2", 21 ) ) ) {
        read_ascii_string( X->PARENT_PRODUCT_NAME_2, B, 67 );
      } else if( ( len == 21 ) && ( ! strncmp( label, "PARENT_PRODUCT_NAME_3", 21 ) ) ) {
        read_ascii_string( X->PARENT_PRODUCT_NAME_3, B, 67 );
      } else if( ( len == 21 ) && ( ! strncmp( label, "PARENT_PRODUCT_NAME_4", 21 ) ) ) {
        read_ascii_string( X->PARENT_PRODUCT_NAME_4, B, 67 );
      } else if( ( len == 13 ) && ( ! strncmp( label, "INSTRUMENT_ID", 13 ) ) ) {
        read_ascii_string( X->INSTRUMENT_ID, B, 4 );
      } else if( ( len == 16 ) && ( ! strncmp( label, "INSTRUMENT_MODEL", 16 ) ) ) {
        read_ascii_string( X->INSTRUMENT_MODEL, B, 3 );
      } else if( ( len == 12 ) && ( ! strncmp( label, "PRODUCT_TYPE", 12 ) ) ) {
        read_ascii_string( X->PRODUCT_TYPE, B, 3 );
      } else if( ( len == 16 ) && ( ! strncmp( label, "PROCESSING_LEVEL", 16 ) ) ) {
        read_ascii_string( X->PROCESSING_LEVEL, B, 2 );
      } else if( ( len == 13 ) && ( ! strncmp( label, "SPACECRAFT_ID", 13 ) ) ) {
        read_ascii_string( X->SPACECRAFT_ID, B, 3 );
      } else if( ( len == 13 ) && ( ! strncmp( label, "SENSING_START", 13 ) ) ) {
        read_ascii_eps_gentime( &X->SENSING_START, B );
      } else if( ( len == 11 ) && ( ! strncmp( label, "SENSING_END", 11 ) ) ) {
        read_ascii_eps_gentime( &X->SENSING_END, B );
      } else if( ( len == 25 ) && ( ! strncmp( label, "SENSING_START_THEORETICAL", 25 ) ) ) {
        read_ascii_eps_gentime( &X->SENSING_START_THEORETICAL, B );
      } else if( ( len == 23 ) && ( ! strncmp( label, "SENSING_END_THEORETICAL", 23 ) ) ) {
        read_ascii_eps_gentime( &X->SENSING_END_THEORETICAL, B );
      } else if( ( len == 17 ) && ( ! strncmp( label, "PROCESSING_CENTRE", 17 ) ) ) {
        read_ascii_string( X->PROCESSING_CENTRE, B, 4 );
      } else if( ( len == 23 ) && ( ! strncmp( label, "PROCESSOR_MAJOR_VERSION", 23 ) ) ) {
        read_ascii_eps_uinteger( &X->PROCESSOR_MAJOR_VERSION, B, 5 );
      } else if( ( len == 23 ) && ( ! strncmp( label, "PROCESSOR_MINOR_VERSION", 23 ) ) ) {
        read_ascii_eps_uinteger( &X->PROCESSOR_MINOR_VERSION, B, 5 );
      } else if( ( len == 20 ) && ( ! strncmp( label, "FORMAT_MAJOR_VERSION", 20 ) ) ) {
        read_ascii_eps_uinteger( &X->FORMAT_MAJOR_VERSION, B, 5 );
      } else if( ( len == 20 ) && ( ! strncmp( label, "FORMAT_MINOR_VERSION", 20 ) ) ) {
        read_ascii_eps_uinteger( &X->FORMAT_MINOR_VERSION, B, 5 );
      } else if( ( len == 21 ) && ( ! strncmp( label, "PROCESSING_TIME_START", 21 ) ) ) {
        read_ascii_eps_gentime( &X->PROCESSING_TIME_START, B );
      } else if( ( len == 19 ) && ( ! strncmp( label, "PROCESSING_TIME_END", 19 ) ) ) {
        read_ascii_eps_gentime( &X->PROCESSING_TIME_END, B );
      } else if( ( len == 15 ) && ( ! strncmp( label, "PROCESSING_MODE", 15 ) ) ) {
        read_ascii_string( X->PROCESSING_MODE, B, 1 );
      } else if( ( len == 16 ) && ( ! strncmp( label, "DISPOSITION_MODE", 16 ) ) ) {
        read_ascii_string( X->DISPOSITION_MODE, B, 1 );
      } else if( ( len == 24 ) && ( ! strncmp( label, "RECEIVING_GROUND_STATION", 24 ) ) ) {
        read_ascii_string( X->RECEIVING_GROUND_STATION, B, 3 );
      } else if( ( len == 18 ) && ( ! strncmp( label, "RECEIVE_TIME_START", 18 ) ) ) {
        read_ascii_eps_gentime( &X->RECEIVE_TIME_START, B );
      } else if( ( len == 16 ) && ( ! strncmp( label, "RECEIVE_TIME_END", 16 ) ) ) {
        read_ascii_eps_gentime( &X->RECEIVE_TIME_END, B );
      } else if( ( len == 11 ) && ( ! strncmp( label, "ORBIT_START", 11 ) ) ) {
        read_ascii_eps_uinteger( &X->ORBIT_START, B, 5 );
      } else if( ( len == 9 ) && ( ! strncmp( label, "ORBIT_END", 9 ) ) ) {
        read_ascii_eps_uinteger( &X->ORBIT_END, B, 5 );
      } else if( ( len == 19 ) && ( ! strncmp( label, "ACTUAL_PRODUCT_SIZE", 19 ) ) ) {
        read_ascii_eps_uinteger( &X->ACTUAL_PRODUCT_SIZE, B, 11 );
      } else if( ( len == 17 ) && ( ! strncmp( label, "STATE_VECTOR_TIME", 17 ) ) ) {
        read_ascii_eps_longgentime( &X->STATE_VECTOR_TIME, B );
      } else if( ( len == 15 ) && ( ! strncmp( label, "SEMI_MAJOR_AXIS", 15 ) ) ) {
        read_ascii_eps_integer( &X->SEMI_MAJOR_AXIS, B, 11 );
      } else if( ( len == 12 ) && ( ! strncmp( label, "ECCENTRICITY", 12 ) ) ) {
        read_ascii_eps_integer( &X->ECCENTRICITY, B, 11 );
      } else if( ( len == 11 ) && ( ! strncmp( label, "INCLINATION", 11 ) ) ) {
        read_ascii_eps_integer( &X->INCLINATION, B, 11 );
      } else if( ( len == 16 ) && ( ! strncmp( label, "PERIGEE_ARGUMENT", 16 ) ) ) {
        read_ascii_eps_integer( &X->PERIGEE_ARGUMENT, B, 11 );
      } else if( ( len == 15 ) && ( ! strncmp( label, "RIGHT_ASCENSION", 15 ) ) ) {
        read_ascii_eps_integer( &X->RIGHT_ASCENSION, B, 11 );
      } else if( ( len == 12 ) && ( ! strncmp( label, "MEAN_ANOMALY", 12 ) ) ) {
        read_ascii_eps_integer( &X->MEAN_ANOMALY, B, 11 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "X_POSITION", 10 ) ) ) {
        read_ascii_eps_integer( &X->X_POSITION, B, 11 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "Y_POSITION", 10 ) ) ) {
        read_ascii_eps_integer( &X->Y_POSITION, B, 11 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "Z_POSITION", 10 ) ) ) {
        read_ascii_eps_integer( &X->Z_POSITION, B, 11 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "X_VELOCITY", 10 ) ) ) {
        read_ascii_eps_integer( &X->X_VELOCITY, B, 11 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "Y_VELOCITY", 10 ) ) ) {
        read_ascii_eps_integer( &X->Y_VELOCITY, B, 11 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "Z_VELOCITY", 10 ) ) ) {
        read_ascii_eps_integer( &X->Z_VELOCITY, B, 11 );
      } else if( ( len == 24 ) && ( ! strncmp( label, "EARTH_SUN_DISTANCE_RATIO", 24 ) ) ) {
        read_ascii_eps_integer( &X->EARTH_SUN_DISTANCE_RATIO, B, 11 );
      } else if( ( len == 25 ) && ( ! strncmp( label, "LOCATION_TOLERANCE_RADIAL", 25 ) ) ) {
        read_ascii_eps_integer( &X->LOCATION_TOLERANCE_RADIAL, B, 11 );
      } else if( ( len == 29 ) && ( ! strncmp( label, "LOCATION_TOLERANCE_CROSSTRACK", 29 ) ) ) {
        read_ascii_eps_integer( &X->LOCATION_TOLERANCE_CROSSTRACK, B, 11 );
      } else if( ( len == 29 ) && ( ! strncmp( label, "LOCATION_TOLERANCE_ALONGTRACK", 29 ) ) ) {
        read_ascii_eps_integer( &X->LOCATION_TOLERANCE_ALONGTRACK, B, 11 );
      } else if( ( len == 9 ) && ( ! strncmp( label, "YAW_ERROR", 9 ) ) ) {
        read_ascii_eps_integer( &X->YAW_ERROR, B, 11 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "ROLL_ERROR", 10 ) ) ) {
        read_ascii_eps_integer( &X->ROLL_ERROR, B, 11 );
      } else if( ( len == 11 ) && ( ! strncmp( label, "PITCH_ERROR", 11 ) ) ) {
        read_ascii_eps_integer( &X->PITCH_ERROR, B, 11 );
      } else if( ( len == 21 ) && ( ! strncmp( label, "SUBSAT_LATITUDE_START", 21 ) ) ) {
        read_ascii_eps_integer( &X->SUBSAT_LATITUDE_START, B, 11 );
      } else if( ( len == 22 ) && ( ! strncmp( label, "SUBSAT_LONGITUDE_START", 22 ) ) ) {
        read_ascii_eps_integer( &X->SUBSAT_LONGITUDE_START, B, 11 );
      } else if( ( len == 19 ) && ( ! strncmp( label, "SUBSAT_LATITUDE_END", 19 ) ) ) {
        read_ascii_eps_integer( &X->SUBSAT_LATITUDE_END, B, 11 );
      } else if( ( len == 20 ) && ( ! strncmp( label, "SUBSAT_LONGITUDE_END", 20 ) ) ) {
        read_ascii_eps_integer( &X->SUBSAT_LONGITUDE_END, B, 11 );
      } else if( ( len == 11 ) && ( ! strncmp( label, "LEAP_SECOND", 11 ) ) ) {
        read_ascii_eps_integer( &X->LEAP_SECOND, B, 2 );
      } else if( ( len == 15 ) && ( ! strncmp( label, "LEAP_SECOND_UTC", 15 ) ) ) {
        read_ascii_eps_gentime( &X->LEAP_SECOND_UTC, B );
      } else if( ( len == 13 ) && ( ! strncmp( label, "TOTAL_RECORDS", 13 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_RECORDS, B, 6 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "TOTAL_MPHR", 10 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_MPHR, B, 6 );
      } else if( ( len == 10 ) && ( ! strncmp( label, "TOTAL_SPHR", 10 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_SPHR, B, 6 );
      } else if( ( len == 9 ) && ( ! strncmp( label, "TOTAL_IPR", 9 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_IPR, B, 6 );
      } else if( ( len == 11 ) && ( ! strncmp( label, "TOTAL_GEADR", 11 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_GEADR, B, 6 );
      } else if( ( len == 11 ) && ( ! strncmp( label, "TOTAL_GIADR", 11 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_GIADR, B, 6 );
      } else if( ( len == 11 ) && ( ! strncmp( label, "TOTAL_VEADR", 11 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_VEADR, B, 6 );
      } else if( ( len == 11 ) && ( ! strncmp( label, "TOTAL_VIADR", 11 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_VIADR, B, 6 );
      } else if( ( len == 9 ) && ( ! strncmp( label, "TOTAL_MDR", 9 ) ) ) {
        read_ascii_eps_uinteger( &X->TOTAL_MDR, B, 6 );
      } else if( ( len == 23 ) && ( ! strncmp( label, "COUNT_DEGRADED_INST_MDR", 23 ) ) ) {
        read_ascii_eps_uinteger( &X->COUNT_DEGRADED_INST_MDR, B, 6 );
      } else if( ( len == 23 ) && ( ! strncmp( label, "COUNT_DEGRADED_PROC_MDR", 23 ) ) ) {
        read_ascii_eps_uinteger( &X->COUNT_DEGRADED_PROC_MDR, B, 6 );
      } else if( ( len == 30 ) && ( ! strncmp( label, "COUNT_DEGRADED_INST_MDR_BLOCKS", 30 ) ) ) {
        read_ascii_eps_uinteger( &X->COUNT_DEGRADED_INST_MDR_BLOCKS, B, 6 );
      } else if( ( len == 30 ) && ( ! strncmp( label, "COUNT_DEGRADED_PROC_MDR_BLOCKS", 30 ) ) ) {
        read_ascii_eps_uinteger( &X->COUNT_DEGRADED_PROC_MDR_BLOCKS, B, 6 );
      } else if( ( len == 19 ) && ( ! strncmp( label, "DURATION_OF_PRODUCT", 19 ) ) ) {
        read_ascii_eps_uinteger( &X->DURATION_OF_PRODUCT, B, 8 );
      } else if( ( len == 28 ) && ( ! strncmp( label, "MILLISECONDS_OF_DATA_PRESENT", 28 ) ) ) {
        read_ascii_eps_uinteger( &X->MILLISECONDS_OF_DATA_PRESENT, B, 8 );
      } else if( ( len == 28 ) && ( ! strncmp( label, "MILLISECONDS_OF_DATA_MISSING", 28 ) ) ) {
        read_ascii_eps_uinteger( &X->MILLISECONDS_OF_DATA_MISSING, B, 8 );
      } else if( ( len == 17 ) && ( ! strncmp( label, "SUBSETTED_PRODUCT", 17 ) ) ) {
        read_ascii_eps_boolean( &X->SUBSETTED_PRODUCT, B, 1 );
      } else  {
        /* Should issue warning */
      }
      skip_line( B );
    }
  }
  return status_buffer( B );
}


int write_mphr( t_mphr *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    write_ascii_string( "PRODUCT_NAME", X->PRODUCT_NAME, B, 67 );
    write_ascii_string( "PARENT_PRODUCT_NAME_1", X->PARENT_PRODUCT_NAME_1, B, 67 );
    write_ascii_string( "PARENT_PRODUCT_NAME_2", X->PARENT_PRODUCT_NAME_2, B, 67 );
    write_ascii_string( "PARENT_PRODUCT_NAME_3", X->PARENT_PRODUCT_NAME_3, B, 67 );
    write_ascii_string( "PARENT_PRODUCT_NAME_4", X->PARENT_PRODUCT_NAME_4, B, 67 );
    write_ascii_string( "INSTRUMENT_ID", X->INSTRUMENT_ID, B, 4 );
    write_ascii_string( "INSTRUMENT_MODEL", X->INSTRUMENT_MODEL, B, 3 );
    write_ascii_string( "PRODUCT_TYPE", X->PRODUCT_TYPE, B, 3 );
    write_ascii_string( "PROCESSING_LEVEL", X->PROCESSING_LEVEL, B, 2 );
    write_ascii_string( "SPACECRAFT_ID", X->SPACECRAFT_ID, B, 3 );
    write_ascii_eps_gentime( "SENSING_START", &X->SENSING_START, B );
    write_ascii_eps_gentime( "SENSING_END", &X->SENSING_END, B );
    write_ascii_eps_gentime( "SENSING_START_THEORETICAL", &X->SENSING_START_THEORETICAL, B );
    write_ascii_eps_gentime( "SENSING_END_THEORETICAL", &X->SENSING_END_THEORETICAL, B );
    write_ascii_string( "PROCESSING_CENTRE", X->PROCESSING_CENTRE, B, 4 );
    write_ascii_eps_uinteger( "PROCESSOR_MAJOR_VERSION", &X->PROCESSOR_MAJOR_VERSION, B, 5 );
    write_ascii_eps_uinteger( "PROCESSOR_MINOR_VERSION", &X->PROCESSOR_MINOR_VERSION, B, 5 );
    write_ascii_eps_uinteger( "FORMAT_MAJOR_VERSION", &X->FORMAT_MAJOR_VERSION, B, 5 );
    write_ascii_eps_uinteger( "FORMAT_MINOR_VERSION", &X->FORMAT_MINOR_VERSION, B, 5 );
    write_ascii_eps_gentime( "PROCESSING_TIME_START", &X->PROCESSING_TIME_START, B );
    write_ascii_eps_gentime( "PROCESSING_TIME_END", &X->PROCESSING_TIME_END, B );
    write_ascii_string( "PROCESSING_MODE", X->PROCESSING_MODE, B, 1 );
    write_ascii_string( "DISPOSITION_MODE", X->DISPOSITION_MODE, B, 1 );
    write_ascii_string( "RECEIVING_GROUND_STATION", X->RECEIVING_GROUND_STATION, B, 3 );
    write_ascii_eps_gentime( "RECEIVE_TIME_START", &X->RECEIVE_TIME_START, B );
    write_ascii_eps_gentime( "RECEIVE_TIME_END", &X->RECEIVE_TIME_END, B );
    write_ascii_eps_uinteger( "ORBIT_START", &X->ORBIT_START, B, 5 );
    write_ascii_eps_uinteger( "ORBIT_END", &X->ORBIT_END, B, 5 );
    write_ascii_eps_uinteger( "ACTUAL_PRODUCT_SIZE", &X->ACTUAL_PRODUCT_SIZE, B, 11 );
    write_ascii_eps_longgentime( "STATE_VECTOR_TIME", &X->STATE_VECTOR_TIME, B );
    write_ascii_eps_integer( "SEMI_MAJOR_AXIS", &X->SEMI_MAJOR_AXIS, B, 11 );
    write_ascii_eps_integer( "ECCENTRICITY", &X->ECCENTRICITY, B, 11 );
    write_ascii_eps_integer( "INCLINATION", &X->INCLINATION, B, 11 );
    write_ascii_eps_integer( "PERIGEE_ARGUMENT", &X->PERIGEE_ARGUMENT, B, 11 );
    write_ascii_eps_integer( "RIGHT_ASCENSION", &X->RIGHT_ASCENSION, B, 11 );
    write_ascii_eps_integer( "MEAN_ANOMALY", &X->MEAN_ANOMALY, B, 11 );
    write_ascii_eps_integer( "X_POSITION", &X->X_POSITION, B, 11 );
    write_ascii_eps_integer( "Y_POSITION", &X->Y_POSITION, B, 11 );
    write_ascii_eps_integer( "Z_POSITION", &X->Z_POSITION, B, 11 );
    write_ascii_eps_integer( "X_VELOCITY", &X->X_VELOCITY, B, 11 );
    write_ascii_eps_integer( "Y_VELOCITY", &X->Y_VELOCITY, B, 11 );
    write_ascii_eps_integer( "Z_VELOCITY", &X->Z_VELOCITY, B, 11 );
    write_ascii_eps_integer( "EARTH_SUN_DISTANCE_RATIO", &X->EARTH_SUN_DISTANCE_RATIO, B, 11 );
    write_ascii_eps_integer( "LOCATION_TOLERANCE_RADIAL", &X->LOCATION_TOLERANCE_RADIAL, B, 11 );
    write_ascii_eps_integer( "LOCATION_TOLERANCE_CROSSTRACK", &X->LOCATION_TOLERANCE_CROSSTRACK, B, 11 );
    write_ascii_eps_integer( "LOCATION_TOLERANCE_ALONGTRACK", &X->LOCATION_TOLERANCE_ALONGTRACK, B, 11 );
    write_ascii_eps_integer( "YAW_ERROR", &X->YAW_ERROR, B, 11 );
    write_ascii_eps_integer( "ROLL_ERROR", &X->ROLL_ERROR, B, 11 );
    write_ascii_eps_integer( "PITCH_ERROR", &X->PITCH_ERROR, B, 11 );
    write_ascii_eps_integer( "SUBSAT_LATITUDE_START", &X->SUBSAT_LATITUDE_START, B, 11 );
    write_ascii_eps_integer( "SUBSAT_LONGITUDE_START", &X->SUBSAT_LONGITUDE_START, B, 11 );
    write_ascii_eps_integer( "SUBSAT_LATITUDE_END", &X->SUBSAT_LATITUDE_END, B, 11 );
    write_ascii_eps_integer( "SUBSAT_LONGITUDE_END", &X->SUBSAT_LONGITUDE_END, B, 11 );
    write_ascii_eps_integer( "LEAP_SECOND", &X->LEAP_SECOND, B, 2 );
    write_ascii_eps_gentime( "LEAP_SECOND_UTC", &X->LEAP_SECOND_UTC, B );
    write_ascii_eps_uinteger( "TOTAL_RECORDS", &X->TOTAL_RECORDS, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_MPHR", &X->TOTAL_MPHR, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_SPHR", &X->TOTAL_SPHR, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_IPR", &X->TOTAL_IPR, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_GEADR", &X->TOTAL_GEADR, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_GIADR", &X->TOTAL_GIADR, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_VEADR", &X->TOTAL_VEADR, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_VIADR", &X->TOTAL_VIADR, B, 6 );
    write_ascii_eps_uinteger( "TOTAL_MDR", &X->TOTAL_MDR, B, 6 );
    write_ascii_eps_uinteger( "COUNT_DEGRADED_INST_MDR", &X->COUNT_DEGRADED_INST_MDR, B, 6 );
    write_ascii_eps_uinteger( "COUNT_DEGRADED_PROC_MDR", &X->COUNT_DEGRADED_PROC_MDR, B, 6 );
    write_ascii_eps_uinteger( "COUNT_DEGRADED_INST_MDR_BLOCKS", &X->COUNT_DEGRADED_INST_MDR_BLOCKS, B, 6 );
    write_ascii_eps_uinteger( "COUNT_DEGRADED_PROC_MDR_BLOCKS", &X->COUNT_DEGRADED_PROC_MDR_BLOCKS, B, 6 );
    write_ascii_eps_uinteger( "DURATION_OF_PRODUCT", &X->DURATION_OF_PRODUCT, B, 8 );
    write_ascii_eps_uinteger( "MILLISECONDS_OF_DATA_PRESENT", &X->MILLISECONDS_OF_DATA_PRESENT, B, 8 );
    write_ascii_eps_uinteger( "MILLISECONDS_OF_DATA_MISSING", &X->MILLISECONDS_OF_DATA_MISSING, B, 8 );
    write_ascii_eps_boolean( "SUBSETTED_PRODUCT", &X->SUBSETTED_PRODUCT, B, 1 );
  }
  return status_buffer( B );
}


int free_mphr( t_mphr *X, int n ) {
}


int init_mphr( t_mphr *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    init_string( X->PRODUCT_NAME, 67 );
    init_string( X->PARENT_PRODUCT_NAME_1, 67 );
    init_string( X->PARENT_PRODUCT_NAME_2, 67 );
    init_string( X->PARENT_PRODUCT_NAME_3, 67 );
    init_string( X->PARENT_PRODUCT_NAME_4, 67 );
    init_string( X->INSTRUMENT_ID, 4 );
    init_string( X->INSTRUMENT_MODEL, 3 );
    init_string( X->PRODUCT_TYPE, 3 );
    init_string( X->PROCESSING_LEVEL, 2 );
    init_string( X->SPACECRAFT_ID, 3 );
    init_eps_gentime( &X->SENSING_START );
    init_eps_gentime( &X->SENSING_END );
    init_eps_gentime( &X->SENSING_START_THEORETICAL );
    init_eps_gentime( &X->SENSING_END_THEORETICAL );
    init_string( X->PROCESSING_CENTRE, 4 );
    init_eps_uinteger( &X->PROCESSOR_MAJOR_VERSION );
    init_eps_uinteger( &X->PROCESSOR_MINOR_VERSION );
    init_eps_uinteger( &X->FORMAT_MAJOR_VERSION );
    init_eps_uinteger( &X->FORMAT_MINOR_VERSION );
    init_eps_gentime( &X->PROCESSING_TIME_START );
    init_eps_gentime( &X->PROCESSING_TIME_END );
    init_string( X->PROCESSING_MODE, 1 );
    init_string( X->DISPOSITION_MODE, 1 );
    init_string( X->RECEIVING_GROUND_STATION, 3 );
    init_eps_gentime( &X->RECEIVE_TIME_START );
    init_eps_gentime( &X->RECEIVE_TIME_END );
    init_eps_uinteger( &X->ORBIT_START );
    init_eps_uinteger( &X->ORBIT_END );
    init_eps_uinteger( &X->ACTUAL_PRODUCT_SIZE );
    init_eps_longgentime( &X->STATE_VECTOR_TIME );
    init_eps_integer( &X->SEMI_MAJOR_AXIS );
    init_eps_integer( &X->ECCENTRICITY );
    init_eps_integer( &X->INCLINATION );
    init_eps_integer( &X->PERIGEE_ARGUMENT );
    init_eps_integer( &X->RIGHT_ASCENSION );
    init_eps_integer( &X->MEAN_ANOMALY );
    init_eps_integer( &X->X_POSITION );
    init_eps_integer( &X->Y_POSITION );
    init_eps_integer( &X->Z_POSITION );
    init_eps_integer( &X->X_VELOCITY );
    init_eps_integer( &X->Y_VELOCITY );
    init_eps_integer( &X->Z_VELOCITY );
    init_eps_integer( &X->EARTH_SUN_DISTANCE_RATIO );
    init_eps_integer( &X->LOCATION_TOLERANCE_RADIAL );
    init_eps_integer( &X->LOCATION_TOLERANCE_CROSSTRACK );
    init_eps_integer( &X->LOCATION_TOLERANCE_ALONGTRACK );
    init_eps_integer( &X->YAW_ERROR );
    init_eps_integer( &X->ROLL_ERROR );
    init_eps_integer( &X->PITCH_ERROR );
    init_eps_integer( &X->SUBSAT_LATITUDE_START );
    init_eps_integer( &X->SUBSAT_LONGITUDE_START );
    init_eps_integer( &X->SUBSAT_LATITUDE_END );
    init_eps_integer( &X->SUBSAT_LONGITUDE_END );
    init_eps_integer( &X->LEAP_SECOND );
    init_eps_gentime( &X->LEAP_SECOND_UTC );
    init_eps_uinteger( &X->TOTAL_RECORDS );
    init_eps_uinteger( &X->TOTAL_MPHR );
    init_eps_uinteger( &X->TOTAL_SPHR );
    init_eps_uinteger( &X->TOTAL_IPR );
    init_eps_uinteger( &X->TOTAL_GEADR );
    init_eps_uinteger( &X->TOTAL_GIADR );
    init_eps_uinteger( &X->TOTAL_VEADR );
    init_eps_uinteger( &X->TOTAL_VIADR );
    init_eps_uinteger( &X->TOTAL_MDR );
    init_eps_uinteger( &X->COUNT_DEGRADED_INST_MDR );
    init_eps_uinteger( &X->COUNT_DEGRADED_PROC_MDR );
    init_eps_uinteger( &X->COUNT_DEGRADED_INST_MDR_BLOCKS );
    init_eps_uinteger( &X->COUNT_DEGRADED_PROC_MDR_BLOCKS );
    init_eps_uinteger( &X->DURATION_OF_PRODUCT );
    init_eps_uinteger( &X->MILLISECONDS_OF_DATA_PRESENT );
    init_eps_uinteger( &X->MILLISECONDS_OF_DATA_MISSING );
    init_eps_boolean( &X->SUBSETTED_PRODUCT );
  }
}


int size_mphr( t_mphr *X ) {
  return 3287;
}



int read_geadr_0( t_geadr_0 *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* Product_details */
    read_string( X->AUX_DATA_POINTER, B, 100, 1 );
  }
  return status_buffer( B );
}


int write_geadr_0( t_geadr_0 *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* Product_details */
    write_string( X->AUX_DATA_POINTER, B, 100, 1 );
  }
  return status_buffer( B );
}


int free_geadr_0( t_geadr_0 *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* Product_details */
  }
}


int init_geadr_0( t_geadr_0 *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* Product_details */
  }
}


int size_geadr_0( t_geadr_0 *X ) {
  int i;
  return 100;
}



int read_ipr_0( t_ipr_0 *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    read_eps_uinteger1( &X->TARGET_RECORD_CLASS, B, 1 );
    read_eps_uinteger1( &X->TARGET_INSTRUMENT_GROUP, B, 1 );
    read_eps_uinteger1( &X->TARGET_RECORD_SUBCLASS, B, 1 );
    read_eps_uinteger4( &X->TARGET_RECORD_OFFSET, B, 1 );
  }
  return status_buffer( B );
}


int write_ipr_0( t_ipr_0 *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    write_eps_uinteger1( &X->TARGET_RECORD_CLASS, B, 1 );
    write_eps_uinteger1( &X->TARGET_INSTRUMENT_GROUP, B, 1 );
    write_eps_uinteger1( &X->TARGET_RECORD_SUBCLASS, B, 1 );
    write_eps_uinteger4( &X->TARGET_RECORD_OFFSET, B, 1 );
  }
  return status_buffer( B );
}


int free_ipr_0( t_ipr_0 *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
  }
}


int init_ipr_0( t_ipr_0 *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
  }
}


int size_ipr_0( t_ipr_0 *X ) {
  int i;
  return 1 + 1 + 1 + 4;
}



int read_viadr_0_UTC_OBT_TIME( t_viadr_0_UTC_OBT_TIME *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    read_eps_uinteger2( &X->days, B, 1 );
    read_eps_uinteger4( &X->msec, B, 1 );
    read_eps_uinteger2( &X->usec, B, 1 );
  }
  return status_buffer( B );
}


int write_viadr_0_UTC_OBT_TIME( t_viadr_0_UTC_OBT_TIME *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    write_eps_uinteger2( &X->days, B, 1 );
    write_eps_uinteger4( &X->msec, B, 1 );
    write_eps_uinteger2( &X->usec, B, 1 );
  }
  return status_buffer( B );
}


int free_viadr_0_UTC_OBT_TIME( t_viadr_0_UTC_OBT_TIME *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
  }
}


int init_viadr_0_UTC_OBT_TIME( t_viadr_0_UTC_OBT_TIME *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
  }
}


int size_viadr_0_UTC_OBT_TIME( t_viadr_0_UTC_OBT_TIME *X ) {
  int i;
  return 2 + 4 + 2;
}



int read_viadr_0_OBT_UTC_TIME( t_viadr_0_OBT_UTC_TIME *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    read_eps_uinteger2( &X->not_used, B, 1 );
    read_eps_uinteger4( &X->rtime, B, 1 );
  }
  return status_buffer( B );
}


int write_viadr_0_OBT_UTC_TIME( t_viadr_0_OBT_UTC_TIME *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    write_eps_uinteger2( &X->not_used, B, 1 );
    write_eps_uinteger4( &X->rtime, B, 1 );
  }
  return status_buffer( B );
}


int free_viadr_0_OBT_UTC_TIME( t_viadr_0_OBT_UTC_TIME *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
  }
}


int init_viadr_0_OBT_UTC_TIME( t_viadr_0_OBT_UTC_TIME *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
  }
}


int size_viadr_0_OBT_UTC_TIME( t_viadr_0_OBT_UTC_TIME *X ) {
  int i;
  return 2 + 4;
}



int read_viadr_0( t_viadr_0 *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* METOP_OBT_to_UTC_correlation_parameters */
    read_viadr_0_UTC_OBT_TIME( &X->UTC_OBT_TIME, B, 1 );
    read_viadr_0_OBT_UTC_TIME( &X->OBT_UTC_TIME, B, 1 );
    read_eps_uinteger2( (eps_uinteger2*)X->CLOCK_STEP, B, ( 2 ) );
  }
  return status_buffer( B );
}


int write_viadr_0( t_viadr_0 *X, buffer* B, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* METOP_OBT_to_UTC_correlation_parameters */
    write_viadr_0_UTC_OBT_TIME( &X->UTC_OBT_TIME, B, 1 );
    write_viadr_0_OBT_UTC_TIME( &X->OBT_UTC_TIME, B, 1 );
    write_eps_uinteger2( (eps_uinteger2*)X->CLOCK_STEP, B, ( 2 ) );
  }
  return status_buffer( B );
}


int free_viadr_0( t_viadr_0 *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* METOP_OBT_to_UTC_correlation_parameters */
  }
}


int init_viadr_0( t_viadr_0 *X, int n ) {
  int i;
  for( i = 0; i < n; i++, X++ ) {
    /* METOP_OBT_to_UTC_correlation_parameters */
  }
}


int size_viadr_0( t_viadr_0 *X ) {
  int i;
  return size_viadr_0_UTC_OBT_TIME( &X->UTC_OBT_TIME ) + size_viadr_0_OBT_UTC_TIME( &X->OBT_UTC_TIME ) + ( 2 ) * 2;
}



int read_mdr_0( t_mdr_0 *X, buffer* B, int n ) {
  int i, i_SD;
  for( i = 0; i < n; i++, X++ ) {
    /* GENERIC_QUALITY_INDICATORS */
    read_eps_boolean( &X->DEGRADED_INST_MDR, B, 1 );
    read_eps_boolean( &X->DEGRADED_PROC_MDR, B, 1 );
    /* MEASUREMENT_DATA */
    read_eps_uinteger4( &X->SIZE_INST_DATA, B, 1 );
    i_SD = X->SIZE_INST_DATA;
    X->INST_DATA = (eps_integer1*)getmem( sizeof( eps_integer1 ) * ( i_SD ) );
    read_eps_integer1( (eps_integer1*)X->INST_DATA, B, ( i_SD ) );
  }
  return status_buffer( B );
}


int write_mdr_0( t_mdr_0 *X, buffer* B, int n ) {
  int i, i_SD;
  for( i = 0; i < n; i++, X++ ) {
    /* GENERIC_QUALITY_INDICATORS */
    write_eps_boolean( &X->DEGRADED_INST_MDR, B, 1 );
    write_eps_boolean( &X->DEGRADED_PROC_MDR, B, 1 );
    /* MEASUREMENT_DATA */
    write_eps_uinteger4( &X->SIZE_INST_DATA, B, 1 );
    i_SD = X->SIZE_INST_DATA;
    write_eps_integer1( (eps_integer1*)X->INST_DATA, B, ( i_SD ) );
  }
  return status_buffer( B );
}


int free_mdr_0( t_mdr_0 *X, int n ) {
  int i, i_SD;
  for( i = 0; i < n; i++, X++ ) {
    /* GENERIC_QUALITY_INDICATORS */
    /* MEASUREMENT_DATA */
    freemem( X->INST_DATA );
  }
}


int init_mdr_0( t_mdr_0 *X, int n ) {
  int i, i_SD;
  for( i = 0; i < n; i++, X++ ) {
    /* GENERIC_QUALITY_INDICATORS */
    /* MEASUREMENT_DATA */
    i_SD = X->SIZE_INST_DATA;
    X->INST_DATA = (eps_integer1*)getmem( sizeof( eps_integer1 ) * ( i_SD ) );
  }
}


int size_mdr_0( t_mdr_0 *X ) {
  int i, i_SD;
  i_SD = X->SIZE_INST_DATA;
  return 1 + 1 + 4 + ( i_SD ) * 1;
}



int get_eps_metopl0( t_eps_metopl0 *R, FILE *fp ) {
  buffer B;
  
  read_grh( &( R->grh ), fp );

  
  if( 
      ( R->grh.record_class == MPHR )
   && ( ( ! eps_metopl0_check_subclass ) || R->grh.record_subclass == 0 )
   && ( ( ! eps_metopl0_check_subclass_version ) || R->grh.record_subclass_version == 2 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      fread( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );
      R->content.MPHR = (t_mphr*)getmem( sizeof( t_mphr ) );
      read_mphr( R->content.MPHR, &B, 1 );
      free_buffer( &B );
    } else {
      fseek( fp, R->grh.record_size - SIZE_OF_REC_HEAD, SEEK_CUR );
    }
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == GEADR )
   && ( ( ! eps_metopl0_check_subclass ) || R->grh.record_subclass == 1 )
   && ( ( ! eps_metopl0_check_subclass_version ) || R->grh.record_subclass_version == 4 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      fread( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );
      R->content.GEADR_0 = (t_geadr_0*)getmem( sizeof( t_geadr_0 ) );
      read_geadr_0( R->content.GEADR_0, &B, 1 );
      free_buffer( &B );
    } else {
      fseek( fp, R->grh.record_size - SIZE_OF_REC_HEAD, SEEK_CUR );
    }
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == IPR )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      fread( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );
      R->content.IPR_0 = (t_ipr_0*)getmem( sizeof( t_ipr_0 ) );
      read_ipr_0( R->content.IPR_0, &B, 1 );
      free_buffer( &B );
    } else {
      fseek( fp, R->grh.record_size - SIZE_OF_REC_HEAD, SEEK_CUR );
    }
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == VIADR )
   && ( ( ! eps_metopl0_check_subclass ) || R->grh.record_subclass == 1 )
   && ( ( ! eps_metopl0_check_subclass_version ) || R->grh.record_subclass_version == 4 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      fread( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );
      R->content.VIADR_0 = (t_viadr_0*)getmem( sizeof( t_viadr_0 ) );
      read_viadr_0( R->content.VIADR_0, &B, 1 );
      free_buffer( &B );
    } else {
      fseek( fp, R->grh.record_size - SIZE_OF_REC_HEAD, SEEK_CUR );
    }
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == MDR )
   && ( ( ! eps_metopl0_check_subclass ) || R->grh.record_subclass == 1 )
   && ( ( ! eps_metopl0_check_subclass_version ) || R->grh.record_subclass_version == 4 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      fread( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );
      R->content.MDR_0 = (t_mdr_0*)getmem( sizeof( t_mdr_0 ) );
      read_mdr_0( R->content.MDR_0, &B, 1 );
      free_buffer( &B );
    } else {
      fseek( fp, R->grh.record_size - SIZE_OF_REC_HEAD, SEEK_CUR );
    }
    return 1;

  
  }
  

  fseek( fp, R->grh.record_size - SIZE_OF_REC_HEAD, SEEK_CUR );

  if( R->grh.record_class == GIADR ) return 1;  /* Allow for extra GIADR in UMARF */

  return 0;

}


int put_eps_metopl0( t_eps_metopl0 *R, FILE *fp ) {
  buffer B;
  
  

  
  if( 
      ( R->grh.record_class == MPHR )
   && ( ( ! eps_metopl0_check_subclass ) || R->grh.record_subclass == 0 )
   && ( ( ! eps_metopl0_check_subclass_version ) || R->grh.record_subclass_version == 2 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      R->grh.record_size = size_mphr( R->content.MPHR ) + SIZE_OF_REC_HEAD;
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_mphr( R->content.MPHR, &B, 1 );

      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    } else {
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    }
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == GEADR )
   && ( ( ! eps_metopl0_check_subclass ) || R->grh.record_subclass == 1 )
   && ( ( ! eps_metopl0_check_subclass_version ) || R->grh.record_subclass_version == 4 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      R->grh.record_size = size_geadr_0( R->content.GEADR_0 ) + SIZE_OF_REC_HEAD;
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_geadr_0( R->content.GEADR_0, &B, 1 );

      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    } else {
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    }
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == IPR )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      R->grh.record_size = size_ipr_0( R->content.IPR_0 ) + SIZE_OF_REC_HEAD;
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_ipr_0( R->content.IPR_0, &B, 1 );

      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    } else {
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    }
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == VIADR )
   && ( ( ! eps_metopl0_check_subclass ) || R->grh.record_subclass == 1 )
   && ( ( ! eps_metopl0_check_subclass_version ) || R->grh.record_subclass_version == 4 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      R->grh.record_size = size_viadr_0( R->content.VIADR_0 ) + SIZE_OF_REC_HEAD;
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_viadr_0( R->content.VIADR_0, &B, 1 );

      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    } else {
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    }
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == MDR )
   && ( ( ! eps_metopl0_check_subclass ) || R->grh.record_subclass == 1 )
   && ( ( ! eps_metopl0_check_subclass_version ) || R->grh.record_subclass_version == 4 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      R->grh.record_size = size_mdr_0( R->content.MDR_0 ) + SIZE_OF_REC_HEAD;
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_mdr_0( R->content.MDR_0, &B, 1 );

      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    } else {
      init_buffer( &B, R->grh.record_size - SIZE_OF_REC_HEAD );
      write_grh( &R->grh, fp );
      fwrite( B.p, R->grh.record_size - SIZE_OF_REC_HEAD, 1, fp );

      free_buffer( &B );
    }
    return 1;

  
  }
  

  

  return 0;

}


int free_eps_metopl0( t_eps_metopl0 *R ) {
  buffer B;
  
  

  
  if( 
      ( R->grh.record_class == MPHR )
   && ( ( ! eps_metopl0_check_subclass ) || R->grh.record_subclass == 0 )
   && ( ( ! eps_metopl0_check_subclass_version ) || R->grh.record_subclass_version == 2 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      free_mphr( R->content.MPHR, 1 );
      free( R->content.MPHR );
    } 
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == GEADR )
   && ( ( ! eps_metopl0_check_subclass ) || R->grh.record_subclass == 1 )
   && ( ( ! eps_metopl0_check_subclass_version ) || R->grh.record_subclass_version == 4 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      free_geadr_0( R->content.GEADR_0, 1 );
      free( R->content.GEADR_0 );
    } 
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == IPR )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      free_ipr_0( R->content.IPR_0, 1 );
      free( R->content.IPR_0 );
    } 
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == VIADR )
   && ( ( ! eps_metopl0_check_subclass ) || R->grh.record_subclass == 1 )
   && ( ( ! eps_metopl0_check_subclass_version ) || R->grh.record_subclass_version == 4 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      free_viadr_0( R->content.VIADR_0, 1 );
      free( R->content.VIADR_0 );
    } 
    return 1;

  
  }
  
  
  if( 
      ( R->grh.record_class == MDR )
   && ( ( ! eps_metopl0_check_subclass ) || R->grh.record_subclass == 1 )
   && ( ( ! eps_metopl0_check_subclass_version ) || R->grh.record_subclass_version == 4 )
  ) {
  
    if( R->grh.instrument_group != DUMMY ) {
      free_mdr_0( R->content.MDR_0, 1 );
      free( R->content.MDR_0 );
    } 
    return 1;

  
  }
  

  

  return 0;

}

