
=head1 NAME

Fortran.PL

=head1 SYNOPSIS

 perl Fortran.PL file.c-F

=head1 DESCRIPTION

Fortran.PL preprocesses a F<.c-F> file and creates a F<.c> file. Identifiers
enclosed in C<FORTRAN_SUBROUTINE( ... )> are modified in order to match the 
corresponding Fortran symbol.
Most Fortran compilers use lower case symbol names with a trailing 
underscore. However, g77 prepends a second underscore to symbol names which 
already contain an underscore. HPUX Fortran compiler does not add any
trailing underscores.
F<Fortran.PL> relies on the C<FORTRAN_UDC> environment variable to guess
what modifications should be applied to Fortran symbols. C<U> stands
for `underscore', C<D> for `double underscore', and C<C> for case. All
values are assumed to be either 0 or 1. For instance, setting 
C<FORTRAN_UDC> to 110 implies double underscore, trailing underscore
case set to lowercase.

=head1 AUTHOR

philippe.marguinaud@meteo.fr

=cut

exists $ENV{FORTRAN_UDC} or die 
 "$0 requires the FORTRAN_UDC environment variable to be set\n";

use strict;
use FileHandle;
use File::Basename;

my ( $UNDERSCORE, $DOUBLE_UNDERSCORE, $CASE ) = split( //, $ENV{FORTRAN_UDC} );

my ( $C ) = @ARGV;

my $c = basename( $C, '.c-F' ) . '.c';

my $fhi = 'FileHandle'->new( "<$C" );
my $fho = 'FileHandle'->new( ">$c" );

while( <$fhi> ) {

  no strict 'refs';
  
  s{FORTRAN_(?:SUBROUTINE|COMMON)\(\s*(\w+)\s*\)}
  {

    my $s = $1;
    my %x;
    for( split( '', $s ) ) { $x{$_}++ }

    my $S = $CASE == 0 ? lc( $s ) : uc( $s );

    if( $UNDERSCORE ) {
      $S = $S . '_';
    }

    if( $DOUBLE_UNDERSCORE && exists( $x{'_'} ) ) {
      $S = $S . '_';
    }

    $S
  }exgo;
  

  $fho->print( $_ );
  
}

$fho->close();

