

use strict;
use File::Find;
use FileHandle;
use Cwd qw( cwd );

sub zip(\@\@) { 
  ( $_[0] && @{ $_[0] } ) or ( $_[0] && @{ $_[1] } ) or return ();
  return ( shift( @{ $_[0] } ), shift( @{ $_[1] } ) ); 
} 

my @DIR = qw( CMN ENV MSG MSGS JDBS TES MP SD WOM );


my @BIN = qw(
  JDBS__Serveur   SD_FRW__ServeurDonnees     
  MP__MainProcess WOM__ServeurWorkOrder
  MSGS__Serveur   TES__ServeurTemps
);


my @C = qw( MSG/src/MSG.a ENV/src/ENV.a CMN/src/CMN.a CMN/PRS/src/CMN_PRS.a 
CMN/DAT/src/CMN_DAT.a CMN/OBS/src/CMN_OBS.a MSG/src/MSG.a ENV/src/ENV.a 
CMN/src/CMN.a CMN/PRS/src/CMN_PRS.a CMN/DAT/src/CMN_DAT.a CMN/OBS/src/CMN_OBS.a );


my %LIBS_BIN = (
  JDBS__Serveur           =>  [ @C ],
  MP__MainProcess         =>  [ @C ],
  MSGS__Serveur           =>  [ @C ],
  TES__ServeurTemps       =>  [ @C ],
  WOM__ServeurWorkOrder   =>  [ qw( SD/DON/src/SD_DON.a SD/BAS/src/SD_BAS.a ), @C ],
  SD_FRW__ServeurDonnees  =>  [ qw( SD/ALG/src/SD_ALG.a SD/GES/src/SD_GES.a 
    SD/ACT/src/SD_ACT.a SD/EVT/src/SD_EVT.a SD/BAS/src/SD_BAS.a SD/SRV/src/SD_SRV.a
    SD/TAC/src/SD_TAC.a SD/DON/src/SD_DON.a ), @C ],
);

my @INC;
my @SRC;
my @LIB;


chdir( 'src' );

find( sub { m/^src/o && push( @SRC, $File::Find::name ) }, @DIR );

push @INC, @SRC;

( my $pwd = &cwd() ) =~ s{/$}{}o;

$pwd =~ m{/([^/]+)/src$}o;
my $RELEASE = $1;

for my $src ( @SRC ) {
  
  my @e = split( m!/!o, $src );
  shift( @e ) if( $e[0] eq '.' );
  pop( @e ); # 'src'

  chdir( "$pwd/$src" );
  
  my $topdir = join( '/', ( '..' ) x ( scalar( @e ) + 1 ) );
  $topdir ||= '.';
  
  my $fh = 'FileHandle'->new( '>Makefile' ); 
  
  my @c = ( <*.c>, <*.cc> );
  my @o = map { 
    my $o = $_; 
    $o =~ s/\.cc?$/.o/o; 
    #"\$(LIB)($o)" 
    $o
  } @c;
  
  my @t = my @m = <*.msg>;
  s/\.msg$/\.cat/o for( @t );
  
  
  my $lib = join( '_', @e );
  my @bin = grep { m/^$lib\_/ } @BIN;
  
  unshift( @LIB, "\$(TOPDIR)/$src/$lib.a" ) unless( @bin );

  my @libs_bin = map { "\$\(TOPDIR)/$_" } map { @{ $LIBS_BIN{$_} } } @bin;

  $fh->print( << "EOF" );

TOPDIR=$topdir

LIB=$lib.a

BIN=@bin

LIBS_BIN=@libs_bin

SRC=@c

OBJ=@o

CAT=@t

MSG=@m

include \$(TOPDIR)/Makefile.inc

EOF


  my @y = sort <*.y>;
  my @l = sort <*.l>;
  
  while( my ( $y, $l ) = zip( @y, @l ) ) {
    ( my $b = $y ) =~ s/_yacc.y$//o;
    my ( $c ) = grep { m/^$b/ } @c;
    ( my $o = $c ) =~ s/\.cc?$/.o/o;
    $fh->print( << "EOF" );

$o: $c $l $y
\t\$(LEX) $l ; \$(YACC) $y
\t\$(CC) \$(INCLUDES) \$(CFLAGS) -o $o -c y.tab.c
\t\\rm -f lex.yy.c y.tab.c
 
EOF
  }

  $fh->print( "# DO NOT DELETE THIS LINE -- make depend depends on it.\n\n" );
  
}




for( @INC ) {
  $_ = "-I\$(TOPDIR)/$_";
}

unshift( @INC, '-I.' );

chdir( $pwd );

my $fh = 'FileHandle'->new( ">Makefile.inc" );

$fh->print( << "EOF" );

include \$(TOPDIR)/../Makefile.local
include \$(TOPDIR)/../Makefile.ARCH

EXT_INCLUDES=-I\$(TOPDIR)/EXT/navigation/include   \\
             -I\$(TOPDIR)/EXT/MLA/include          \\
             -I\$(TOPDIR)/EXT/env/include          \\
             -I\$(TOPDIR)/EXT/misc/include         \\
             -I\$(TOPDIR)/EXT/lapack-lite/include  \\
             -I\$(TOPDIR)/EXT/ngmath-lite/include  \\
             -I\$(TOPDIR)/EXT/spline-lite/include  \\
             -I\$(TOPDIR)/EXT/numrec-lite/include  \\
             -I\$(TOPDIR)/EXT/fortran-lock/include  

EXT_LIBS=-L\$(TOPDIR)/EXT/navigation/lib -lnavigation     \\
         -L\$(TOPDIR)/EXT/MLA/lib -lMLA                   \\
         -L\$(TOPDIR)/EXT/env/lib -lenv                   \\
         -L\$(TOPDIR)/EXT/ngmath-lite/lib -lngmath-lite   \\
         -L\$(TOPDIR)/EXT/lapack-lite/lib -llapack-lite   \\
         -L\$(TOPDIR)/EXT/spline-lite/lib -lspline-lite   \\
         -L\$(TOPDIR)/EXT/numrec-lite/lib -lnumrec-lite   \\
         -L\$(TOPDIR)/EXT/fortran-lock/lib -lfortran-lock

LOCAL_INCLUDES=\$(AAPP_INCLUDES)            \\
               \$(XRCS_INCLUDES)            \\
               \$(FFT_INCLUDES)             \\
               \$(ESSL_INCLUDES)            \\
               \$(METOPLIB_INCLUDES)
              
LOCAL_LIBS=\$(AAPP_LIBS)                    \\
           \$(XRCS_LIBS)                    \\
           \$(FFT_LIBS)                     \\
           \$(ESSL_LIBS)                    \\
           \$(METOPLIB_LIBS)



INCLUDES=@INC \$(LOCAL_INCLUDES) \$(EXT_INCLUDES)
CFLAGS=\$(ARCH_CFLAGS) -DOPS_RELEASE="\\"$RELEASE\\""              \\
       \$(AAPP_FLAGS) \$(XRCS_FLAGS) \$(FFT_FLAGS) \$(ESSL_FLAGS) \\
       \$(METOPLIB_FLAGS) 
CPPFLAGS=\$(ARCH_CPPFLAGS) -DOPS_RELEASE="\\"$RELEASE\\""            \\
         \$(AAPP_FLAGS) \$(XRCS_FLAGS) \$(FFT_FLAGS) \$(ESSL_FLAGS) \\
	 \$(METOPLIB_FLAGS) 
FFLAGS=\$(ARCH_FFLAGS) -DOPS_RELEASE="\\"$RELEASE\\""              \\
       \$(AAPP_FLAGS) \$(XRCS_FLAGS) \$(FFT_FLAGS) \$(ESSL_FLAGS) \\
       \$(METOPLIB_FLAGS) 
LDFLAGS=\$(ARCH_LDFLAGS)
LIBS=@LIB @LIB 

depend: 
\tperl \$(TOPDIR)/../Makedepend.PL -- -Y \$(CFLAGS) \$(INCLUDES) -- \$(SRC)

all: lib bin cat 

lib: \$(LIB)

bin: \$(BIN)

cat: \$(CAT)

\$(BIN): \$(LIBS_BIN) \$(LIB)
\t\$(LD) \$(LDFLAGS) -o \$(BIN) \$(LIB) \$(LIBS_BIN) \$(EXT_LIBS) \$(LOCAL_LIBS) \$(ARCH_LIBS)

\$(LIB): \$(OBJ)
\t\$(AR) \$(LIB) \$(OBJ)

\$(CAT): \$(MSG)
\t\$(GENCAT) \$(CAT) \$(MSG)

clean: arch_clean
\t\\rm -f *.o *.a \$(BIN) Makefile.bak *.cat


install: installbin installcat

installbin: \$(TOPDIR)/../run/OPS/bin/\$(BIN)

installcat: \$(TOPDIR)/../run/OPS/conf/\$(CAT)

\$(TOPDIR)/../run/OPS/bin/\$(BIN): \$(BIN)
\t\\mkdir -p \$(TOPDIR)/../run/OPS/bin
\t\\cp \$(BIN) \$(TOPDIR)/../run/OPS/bin/\$(BIN)

\$(TOPDIR)/../run/OPS/conf/\$(CAT): \$(CAT)
\t\\mkdir -p \$(TOPDIR)/../run/OPS/conf
\t\\cp \$(CAT) \$(TOPDIR)/../run/OPS/conf/\$(CAT)

.SUFFIXES: .o .cc .c .h

.cc.o:
\t\$(CPP) \$(INCLUDES) \$(CPPFLAGS) -c \$*.cc

.c.o:
\t\$(CC) \$(INCLUDES) \$(CFLAGS) -c \$*.c

.C.c:
\t\$(TOPDIR)/../Fortran.PL \$*.C

EOF

$fh->close();


$fh->open( '>Makefile' );

my @all = qw( lib bin cat );

$fh->print( "\nall: ext @all\n" );

$fh->print( "\ninclude ../Makefile.ARCH\n" );

my %deps = ( install => [ qw( all ) ], bin => [ qw( lib ) ] );


for my $i ( 'depend', @all, 'install' ) {

  $fh->printf( "\n$i: %s\n", join( ' ', @{ $deps{$i} || [ ] } ) );
  for my $src ( @SRC ) {
    $fh->print( "\tcd $src ; \$(MAKE) $i\n" );
  }

}

my @ext = map { ( split( '/' ) )[1] } <EXT/*/lib>;

$fh->print( "ext:\n" );

for my $ext ( @ext ) {
  $fh->print( "\t\\cd EXT/$ext/lib ; \$(MAKE) \n" );
}

$fh->print( "clean:\n" );
for my $d ( map( { "EXT/$_/lib" } @ext ), @SRC ) {
  $fh->print( "\t\\cd $d ; \$(MAKE) clean\n" );
}


$fh->print( "\n\n" );

$fh->close();


chdir( '..' );

$fh = 'FileHandle'->new( '>Makefile' );


for( qw( all lib bin cat clean depend ) ) {
  $fh->print( "$_:\n\tcd src && make $_\n" );
}

$fh->print( "include Makefile.local\ninclude Makefile.ARCH\n\n" );


$fh->print(
  "install:\n\tcd src && make install\n"
. "\t\@if [ \"\$(PREFIX)\" != \"\" ]; then "
. "\\mkdir -p \"\$(PREFIX)/OPS-LRS-run\" ; \\rsync --archive --delete run/ \"\$(PREFIX)/OPS-LRS-run\"; fi \n\n"
);


















