package IASI::OPS::REPORT;

use strict;
use FileHandle;
use XML::Mini::Document;
use Data::Dumper;




=head1 NAME

IASI::OPS::REPORT

=head1 SYNOPSIS

  use IASI::OPS::REPORT;

  my $rpt = 'IASI::OPS::REPORT'->new( file => 'IASI_wo_001.rpt' );
  $rpt->read();
  print $rpt->LogMessage();
  

=head1 DESCRIPTION

This module parses XML report files, and extracts provides some methods
to retrieve the list of generated IASI 1C files, and extract the log message
issued by the OPS.

=head1 METHODS

=over 4

=cut



sub new {

=item new

Constructor.

=cut


  my $class = shift;
  return bless { @_ }, $class;
}

sub read : method {

=item read

Read and parse the report file, using C<XML::Mini::Document>.

=cut

  my $self = shift;
  my $xml = do { 
    ( my $fh = 'FileHandle'->new( "<$self->{file}" ) )
      or die( "Cannot open $self->{file} for reading : $!\n" );
    $fh->getline() for( 1 .. 2 ); # skip <?xml..> and <!DOCTYPE..>
    local $/ = undef; <$fh> 
  };
  return unless( $self->{xml} = $xml );
  my $p = 'XML::Mini::Document'->new();
  eval {
    $p->parse( $xml );
  };
  $@ && return;
  %$self = ( %$self, %{ ( $p->toHash() )->{PPF_report} } );
  return 1;
}

sub GeneratedFile_1C {

=item GeneratedFile_1C

Returns the list of generated 1C files, as read from report file.

=cut

  my $self = shift;
  my $A = $self->{StageInfo};
  
  my @F;
  
  if( ref( $A ) eq 'ARRAY' ) {
  
    for my $a ( @$A ) {
      push @F, map { $_->{FileName} } @{ $a->{GeneratedFile} };
    }
  
  } elsif( ref( $A ) eq 'HASH' ) {
  
    push @F, map { $_->{FileName} } @{ $A->{GeneratedFile} };
  
  }
  
  @F = grep { m/^IASI_xxx_1C_\w{3}_\d{14}Z_\d{14}Z_\w_\w_\d{14}Z$/o } @F;
  
  
  return @F;
}

sub LogMessage {

=item LogMessage

Returns the message found in the report file.

=cut

  my $self = shift;
  my $A = $self->{StageInfo};
  
  my @F;
  
  if( ref( $A ) eq 'ARRAY' ) {
  
    for my $a ( @$A ) {
      push @F, $a->{LogMessage};
    }
  
  } elsif( ref( $A ) eq 'HASH' ) {
  
    push @F, $A->{LogMessage};
  
  }
    
  
  return @F;
}


1;

=back

=head1 SEE ALSO

L<IASI::OPS>, L<IASI::OPS::WORKORDER>, L<XML::Mini::Document>

=head1 AUTHORS

Philippe.Marguinaud@meteo.fr

=head1 COPYRIGHT

Meteo-France/EUMETSAT/SAFNWP

=head1 HISTORY

=over 4

=item 01/03/2007

First version for HRPT processing.

=back

=cut

