package METOP::PFS::GRH;

use strict;
use Fcntl qw( SEEK_CUR );

=head1 NAME

METOP::PFS::GRH

=head1 SYNOPSIS

  use METOP::PFS::GRH;
  my $fh = 'FileHandle'->new( 
    "<IASI_P71_00_M02_20070305224145Z_20070305224455Z_N_O_20070305224515Z" );
  my $grh = &METOP::PFS::GRH::grh_readahead( $fh );
  print $grh->{record_class};

=head1 DESCRIPTION

Routine to parse METOP PFS generic record headers.

=head1 ROUTINES

=over 4

=cut



sub grh_readahead {
  my $fh = shift;

  my $grh = { };

  return unless( $fh->read( my $buf20, 20 ) == 20 );

  my ( $record_class, $instrument_group, $record_subclass, $record_subclass_version,
  $record_size, $record_start_days, $record_start_milliseconds, $record_stop_days,
  $record_stop_milliseconds ) = unpack( 'CCCCNnNnN', $buf20 );

  $grh->{record_class}            = $record_class;
  $grh->{instrument_group}        = $instrument_group;
  $grh->{record_subclass}         = $record_subclass;
  $grh->{record_subclass_version} = $record_subclass_version;
  $grh->{record_size}             = $record_size;

  $grh->{record_start}            = { 
                                       days => $record_start_days, 
                                       milliseconds => $record_start_milliseconds 
                                    };
  $grh->{record_stop}             = { 
                                       days => $record_stop_days, 
                                       milliseconds => $record_stop_milliseconds 
                                    };

  $fh->seek( -20, SEEK_CUR );


  return $grh;
}



1;

=back

=head1 SEE ALSO

L<METOP::PFS>, L<EPS.GGS.SPE.96167>

=head1 AUTHORS

Philippe.Marguinaud@meteo.fr

=head1 COPYRIGHT

Meteo-France/EUMETSAT/SAFNWP

=head1 HISTORY

=over 4

=item 01/03/2007

First version for HRPT processing.

=back

=cut

