package METOP::PFS::RECORD;

use strict;
use METOP::PFS::GRH;

=head1 NAME

METOP::PFS::RECORD

=head1 SYNOPSIS

  use METOP::PFS::RECORD;
  my $fh = 'FileHandle'->new( 
  'IASI_P71_00_M02_20070305125529Z_20070305125839Z_N_O_20070305125838Z' );
  my $rec = &METOP::PFS::RECORD::record_read( $fh );

=head1 DESCRIPTION

Routine to read grh and record data.

=head1 ROUTINES

=over 4

=cut



sub record_read {

=item record_read

Read a record and advance.

=cut

  my $fh = shift;
  
  return unless( my $grh = METOP::PFS::GRH::grh_readahead( $fh ) );


  my $rec = { grh => $grh };

  $fh->read( $rec->{buf}, $rec->{grh}{record_size} );
  
  return $rec;
}

1;

=back

=head1 SEE ALSO

L<METOP::PFS>, <METOP::PFS::GRH>, L<EPS.GGS.SPE.96167>

=head1 AUTHORS

Philippe.Marguinaud@meteo.fr

=head1 COPYRIGHT

Meteo-France/EUMETSAT/SAFNWP

=head1 HISTORY

=over 4

=item 01/03/2007

First version for HRPT processing.

=back

=cut

