#
# @(#)liblog.pm version 1.1 03/12/02
#
package liblog;

use vars qw(@ISA @EXPORT $VERSION);
use Exporter;
use File::Basename;

$VERSION=1.1;
@ISA=qw(Exporter);
@EXPORT=qw(log_emergency log_critical log_error log_warning log_notice log_info log_debug log_text log_line);

sub log_init {
  my $header;
  my $host=`uname -n`; chomp $host;
  my $pid=$$;
  my $name=basename("$0");
  $header=sprintf("%-8s %-5s %-12s",$host,$pid,$name);
  return $header;
}

BEGIN {
my $header='';
sub log_msg {
  $header or $header=log_init();
  my @instant=(localtime)[0..5];
  $instant[4]+=1;	# month
  $instant[5]+=1900;	# year
  printf STDERR ("> %.4d/%.2d/%.2d %.2d:%.2d:%.2d $header @_\n",reverse @instant);
}
}

sub log_level {
  my $msg=sprintf("%-10s : %s",@_);
  log_msg($msg);
}

sub log_emergency { log_level('EMERGENCY',@_); }
sub log_critical { log_level('CRITICAL',@_); }
sub log_error { log_level('ERROR',@_); }
sub log_warning { log_level('WARNING',@_); }
sub log_notice { log_level('NOTICE',@_); }
sub log_info { log_level('INFO',@_); }
sub log_debug {
if (defined $ENV{'DEBUG'}) {
if ($ENV{'DEBUG'} eq "ON") {
log_level('DEBUG',@_);
}
}
}
sub log_text { print STDERR "@_\n"; } 
sub log_line { log_msg('') };

1;

__END__
