static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/DAT/src/CMN_DAT_Evenement.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ ---------------------------------------------------------------------------
//@ Composant : CMN
//@ Sous-composant : CMN_DAT
//@ CLASSE : CMN_DAT_Evenement
//@ 
//@ Commentaire :
//@    Ce module gere la classe CMN_DAT_Evenement, qui permet de 
//@    stocker l'ensemble des evenements
//@                                     
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Adaptation a l'OPS : ajout des parametres JDB
//@ ---------------------------------------------------------------------------
//@ VERSION : 26/07/1999 : GRT
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

//
// Fichiers include
//

// STL

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN_DAT_Evenement.h"


//
// Definition des constantes
//
const t_Chaine CMN_DAT_Evenement::CHAMP_DATE           = "DATE";
const t_Chaine CMN_DAT_Evenement::CHAMP_TYPE_EVENEMENT = "TYPE EVENEMENT";
const t_Chaine CMN_DAT_Evenement::CHAMP_FONCTION       = "FONCTION";
const t_Chaine CMN_DAT_Evenement::CHAMP_LIBELLE        = "LIBELLE";


//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CMN_DAT_Evenement
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_Evenement::CMN_DAT_Evenement()
{
  m_DateEvenement.DateCourante();
  m_TypeEvenement = SD__Message::MSG_SD_GENERIC;
  m_Fonction      = "";
  m_Libelle       = "";
  m_Prm1          = "";
  m_Prm2          = "";
  m_Prm3          = "";
}


//@ ------------------------------------------------------------------*/
//@ Methode : CMN_DAT_Evenement
//@
//@ Description : Constructeur copie
//@
//@ Parametres :
//@               (ENTREE) unEvenement
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_Evenement::CMN_DAT_Evenement(CMN_DAT_Evenement& unEvenement)
{
  m_DateEvenement = unEvenement.DateEvenement();
  m_TypeEvenement = unEvenement.TypeEvenement();
  m_Fonction      = unEvenement.Fonction();
  m_Libelle       = unEvenement.Libelle();
  m_Prm1          = unEvenement.Prm1();
  m_Prm2          = unEvenement.Prm2();
  m_Prm3          = unEvenement.Prm3();
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~CMN_DAT_Evenement
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_Evenement::~CMN_DAT_Evenement()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator =
//@
//@ Description : Operateur =
//@
//@ Parametres :
//@               (ENTREE) unEvenement
//@
//@ Valeur de retour : 
//@               CMN_DAT_Evenement : Evenement mis a jour
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_Evenement& 
CMN_DAT_Evenement::operator =(CMN_DAT_Evenement& unEvenement)
{
  if (&unEvenement == this)
    return *this;

  CMN_DAT_Objet::operator =( unEvenement);
  m_DateEvenement = unEvenement.DateEvenement();
  m_TypeEvenement = unEvenement.TypeEvenement();
  m_Fonction      = unEvenement.Fonction();
  m_Libelle       = unEvenement.Libelle();
  m_Prm1          = unEvenement.Prm1();
  m_Prm2          = unEvenement.Prm2();
  m_Prm3          = unEvenement.Prm3();

  return *this;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Synthetiser
//@
//@ Description : Genere une section pour l'evenement dans le fichier
//@               de configuration
//@
//@ Parametres :
//@               (ENTREE) Config : Fichier de configuration
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_DAT_Evenement::Synthetiser(CMN__Configuration& Config)
{
  //@ Generation de la section 
  CMN_DAT_Objet::Synthetiser(Config);

  //@ On sauvegarde :
  //@  - la date de l'evenement
  Config.EcrireValeur(m_Identifiant, CHAMP_DATE, 
                      m_DateEvenement.Chaine());

  //@  - le type d'evenement
  Config.EcrireValeur(m_Identifiant, CHAMP_TYPE_EVENEMENT, 
                      m_TypeEvenement);
  
  //@  - la fonction ayant produit l'evenement
  Config.EcrireValeur(m_Identifiant, CHAMP_FONCTION, m_Fonction);

  //@  - le libelle de l'evenement
  Config.EcrireValeur(m_Identifiant, CHAMP_LIBELLE, m_Libelle);
}


//@ ------------------------------------------------------------------*/
//@ Methode : Analyser
//@
//@ Description : Analyse une section du fichier de configuration pour 
//@               remplir l'evenement
//@
//@ Parametres :
//@               (ENTREE) Config : Fichier de configuration
//@               (ENTREE) sNomEvenement: Section du fichier de
//@                                       configuration decrivant
//@                                       l'evenement
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_DAT_Evenement::Analyser(CMN__Configuration& Config,
                                 t_Chaine sNomEvenement)
{
  t_Chaine s;

  //@ Analyse de la section 
  CMN_DAT_Objet::Analyser( Config, sNomEvenement);

  //@ On recupere :
  //@  - la date de l'evenement
  Config.LireValeur(m_Identifiant, CHAMP_DATE, s);
  m_DateEvenement.Chaine(s);

  //@  - le type d'evenement
  Config.LireValeur(m_Identifiant, CHAMP_TYPE_EVENEMENT, s);
  m_TypeEvenement = (SD__Message::e_TypeMessageJDB) int(s);

  //@  - la fonction ayant produit l'evenement
  Config.LireValeur(m_Identifiant, CHAMP_FONCTION, m_Fonction);

  //@  - le libelle de l'evenement
  Config.LireValeur(m_Identifiant, CHAMP_LIBELLE, m_Libelle);
}


//======================================================================
//                                      METHODES PRIVEES
//======================================================================
