static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/DAT/src/CMN_DAT_GeneratedFile.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ ---------------------------------------------------------------------------
//@ Composant : CMN
//@ Sous-composant : CMN_DAT
//@ CLASSE : CMN_DAT_GeneratedFile
//@ 
//@ Commentaire :
//@    Ce module gere la classe CMN_DAT_GeneratedFile, qui stocke les 
//@    informations liees a un fichier genere par l'application
//@
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 06/01/2003 : P. CABANE
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-2100-CR-290-CN : VERSION : 31/03/2006 : B. GUENDE
//@ Degraded cases counters added as a private member
//@ See IA-2100-CR-290-THA.
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

//
// Fichiers include
//

// STL
#include <iostream>
#include <cstdio>

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN_DAT_GeneratedFile.h"

//
// Definition des constantes
//

//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CMN_DAT_GeneratedFile
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_GeneratedFile::CMN_DAT_GeneratedFile()
  : m_FileName(""), m_MdrDefined(CMN__FAUX), 
    m_MdrOk(0), m_MdrDegraded(0)
{
/* CR290 */ 
  int i=0;
  for (i=0; i<CMN__NB_CGXX; i++)
    m_ListeCasDeg[i]=0;
/* fin CR290 */
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~CMN_DAT_GeneratedFile
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_GeneratedFile::~CMN_DAT_GeneratedFile()
{
  //@ RAZ de la liste des data gap
  m_ListeDataGap.clear();
}


//@ ------------------------------------------------------------------*/
//@ Methode : MdrOk
//@
//@ Description : Met a jour l'attribut m_MdrOk
//@
//@ Parametres :
//@               (ENTREE) valeur : Valeur pour l'attribut
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_DAT_GeneratedFile::MdrOk(unsigned long valeur)
{
  //@ Les informations MDR sont definies
  m_MdrDefined = CMN__VRAI;

  //@ Mettre a jour la valeur de m_MdrOk
  m_MdrOk      = valeur;
}


//@ ------------------------------------------------------------------*/
//@ Methode : MdrDegraded
//@
//@ Description : Met a jour l'attribut m_MdrDegraded
//@
//@ Parametres :
//@               (ENTREE) valeur : Valeur pour l'attribut
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_DAT_GeneratedFile::MdrDegraded(unsigned long valeur)
{
  //@ Les informations MDR sont definies
  m_MdrDefined  = CMN__VRAI;

  //@ Mettre a jour la valeur de m_MdrDegraded
  m_MdrDegraded = valeur;
#ifdef OPS_DEBUG
  cout << "MAJ de m_MdrDegraded: " << m_MdrDegraded << endl;
#endif /* OPS_DEBUG */
}


//@ ------------------------------------------------------------------*/
//@ Methode : AjouterDataGap
//@
//@ Description : Ajoute un element dans la liste des trous de donnees
//@
//@ Parametres :
//@               (ENTREE) dataGap : Valeur pour le nouvel element
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_DAT_GeneratedFile::AjouterDataGap(t_DataGap& dataGap)
{
  // Iterateur pour la liste des trous de donnees
  t_ListeDataGap::iterator pInfo;

  // Temoin de recherche
  t_CMN__Booleen           trouve = CMN__FAUX;


  //@ SI un element ayant la meme raison existe deja
  //@ Ajouter la valeur a cet element
  //@ SINON ajouter l'element a la fin de la liste m_ListeDataGap

  pInfo = m_ListeDataGap.begin();
  while ( (pInfo != m_ListeDataGap.end()) && (trouve == CMN__FAUX) )
    {
      if (pInfo->Reason() == dataGap.Reason())
        {
          trouve = CMN__VRAI;
        }
      else
        {
          pInfo++;
        }
    }

  if (trouve == CMN__VRAI)
    {
      pInfo->Count(pInfo->Count() + dataGap.Count());
    }
  else
    {
      m_ListeDataGap.push_back(dataGap);
    }
}
/* CR290 */
//@ ------------------------------------------------------------------*/
//@ Methode: AjouterCasDeg
//@
//@ Description: met a jour le compteur des cas degrades
//@
//@ Parametres: tableau listant les cas degrades d'une ligne
//@
//@ Valeur de retour:
//@            aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_DAT_GeneratedFile::AjouterCasDeg( bool * listCG)
{
  int i;
  for (i=0; i<CMN__NB_CGXX; i++)
  {
    if ( listCG[i] )
    {
      m_ListeCasDeg[i]++;
#ifdef OPS_DEBUG
      cout << "MAJ de ListeCasDeg: Cas "<< i+1 << " = " << m_ListeCasDeg[i] << " lignes." << endl; 
#endif /* OPS_DEBUG */
    }
  }
}

//@ ------------------------------------------------------------------*/
//@ Methode: GetNbCas
//@
//@ Description: renvoie le nombre de lignes ayant leve le cas degrade noCas
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN_DAT_GeneratedFile::GetNbCas(int noCas)
{
  if ( (noCas > 15) || (noCas<1) )
    return "0";
#ifdef OPS_DEBUG
  cout << "Nombre de lignes ayant le CG"<<noCas<<": "<<m_ListeCasDeg[noCas-1]<<endl;
#endif
  return m_ListeCasDeg[noCas-1];
}
//@ ------------------------------------------------------------------*/
//@ Methode : SetNbCas 
//@
//@ Description : Met a jour le tableau compteur des cas degrades 
//@
//@ Parametres : numero du cas degrade, valeur
//@               
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_DAT_GeneratedFile::SetNbCas( int noCas, unsigned long valeur )
{
  if (( noCas > 0 ) && ( noCas < 16 ))
    m_ListeCasDeg[noCas-1]=valeur;
}  
/* Fin CR290 */
//@ ------------------------------------------------------------------*/
//@ Methode : Afficher
//@
//@ Description : Affiche le contenu de l'objet
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_DAT_GeneratedFile::Afficher()
{
  // Iterateur de data gap
  t_ListeDataGap::iterator pInfoDataGap;
  
  // Compteur de data gap
  int                      i = 1;


  cout << "nom fichier : " << m_FileName << endl;
  if (m_MdrDefined != CMN__VRAI)
    {
      cout << "\t\tpas de MDR de defini" << endl;
    }
  else
    {
      cout << "\t\tmdr OK      : " << m_MdrOk << endl;
      cout << "\t\tmdr degraded: " << m_MdrDegraded << endl;
/* CR290 */
      for (int i=0; i<CMN__NB_CGXX; i++)
      {
        if (m_ListeCasDeg[i]>0)
          cout << "\t\tCG" << i+1 << ": " << m_ListeCasDeg[i] << " lines." << endl;
      }
/* Fin CR290 */
    }
  for (pInfoDataGap = m_ListeDataGap.begin(); 
       pInfoDataGap != m_ListeDataGap.end();
       pInfoDataGap++) 
    {
      cout << "\t\tdata gap no" << i++ << ": "; 
      pInfoDataGap->Afficher();
    }
}


//======================================================================
//                                      METHODES PRIVEES
//======================================================================
