static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/DAT/src/CMN_DAT_ListeObjets.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ ---------------------------------------------------------------------------
//@ Composant : CMN
//@ Sous-composant : CMN_DAT
//@ CLASSE : CMN_DAT_ListeObjets
//@ 
//@ Commentaire :
//@    Ce module gere la classe CMN_DAT_ListeObjets
//@                                     
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ FA-ID : SMM-M4-FA-2366-CN : 04/09/2001 : GRT
//@ On appelle la methode Initialiser dans LireElement
//@ ---------------------------------------------------------------------------
//@ FA-ID : SMM-M4-FA-2413-CN : 30/10/2001 : GRT
//@ Rajout de la methode SauverListeTriee
//@ ---------------------------------------------------------------------------
//@ VERSION : 23/07/1999 : GRT
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

//
// Fichiers include
//

// STL

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN__Config.h"
#include "CMN_DAT_Objet.h"
#include "CMN_DAT_ListeObjets.h"


//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CMN_DAT_ListeObjets
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_ListeObjets::CMN_DAT_ListeObjets()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~CMN_DAT_ListeObjets
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_ListeObjets::~CMN_DAT_ListeObjets()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Contenu
//@
//@ Description : Retourne le contenu de l'objet sous forme de chaine
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Contenu sous forme de chaine de caractere
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN_DAT_ListeObjets::Contenu()
{
  t_Chaine contenu("");

  if (m_Config.SauverChaine(contenu) == CMN__STATUT_OK)
    {
      return contenu;
    }

  return "";
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator =
//@
//@ Description : operateur =
//@
//@ Parametres :
//@               (ENTREE) ListeObjets : Liste a affecter
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_DAT_ListeObjets::operator =(CMN_DAT_ListeObjets& ListeObjets)
{
  if (&ListeObjets == this)
    return ;

  t_Chaine contenu = ListeObjets.Contenu();
  m_Config.ChargerChaine(contenu);
}


//@ ------------------------------------------------------------------*/
//@ Methode : CompterElements   
//@
//@ Description : Compte le nombre d'elements dans la liste
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Nombre d'elements dans la liste (-1 si erreur)
//@
//@ ------------------------------------------------------------------*/
int CMN_DAT_ListeObjets::CompterElements()
{
  int nbElements = 0;


  m_Config.NombreDeSections(&nbElements);


  return nbElements;
}


//@ ------------------------------------------------------------------*/
//@ Methode : LireIdentifiantElement
//@
//@ Description : Recupere l'identifiant d'un objet a partir de sa 
//@               position
//@
//@ Parametres :
//@               (ENTREE) nPosition: Position de l'objet dans la liste
//@               (SORTIE) identifiant: Identifiant de l'objet se 
//@                                     trouvant a cette position dans
//@                                     la liste
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK si la fonction a reussi
//@               CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN_DAT_ListeObjets::LireIdentifiantElement(int nPosition, 
                                                          t_Chaine&
                                                          identifiant)
{
  t_CMN__Statut statut = CMN__STATUT_OK;


  if (m_Config.LireSection(nPosition, identifiant) != CMN__STATUT_OK) 
    {
      statut = CMN__STATUT_NOK;
    }
  

  return statut;
}


//@ ------------------------------------------------------------------*/
//@ Methode : LireElement
//@
//@ Description : Lit un element de la liste a partir de sa position
//@
//@ Parametres :
//@               (ENTREE) nPosition: Position de l'objet a lire 
//@                                   dans la liste 
//@               (SORTIE) objet: Objet se trouvant a cette position
//@                               dans la liste
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK si la fonction a reussi
//@               CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN_DAT_ListeObjets::LireElement(int nPosition,
                                               CMN_DAT_Objet& objet)
{
  t_CMN__Statut statut = CMN__STATUT_OK;
  t_Chaine      sSection("");


  if (LireIdentifiantElement(nPosition, sSection) == CMN__STATUT_OK) 
    {
      //@ On initialise l'objet pour que les informations susceptibles 
      //@ d'etre contenues dans l'Objet en entree ne perturbent pas le
      //@ resultat
      objet.Initialiser();
      objet.Analyser(m_Config, sSection);
    } 
  else
    {
      statut = CMN__STATUT_NOK;
    }

  return statut;
}


//@ ------------------------------------------------------------------*/
//@ Methode : RechercherElement
//@
//@ Description : Recherche un element de la liste a partir de 
//@               l'identifiant
//@
//@ Parametres :
//@               (ENTREE) sIdentifiant : Identifiant de l'objet a
//@                                       rechercher
//@
//@ Valeur de retour : 
//@               Position de l'element dans la liste (-1 si erreur)
//@
//@ ------------------------------------------------------------------*/
int CMN_DAT_ListeObjets::RechercherElement(t_Chaine sIdentifiant)
{
  int      i = 0;
  int      nbElements = CompterElements();
  t_Chaine identifiant("");


  for (i=0; i<nbElements; i++)
    {
      LireIdentifiantElement(i, identifiant);
      if (sIdentifiant == identifiant)
        {
          return i;
        }
    }


  return -1;
}


//@ ------------------------------------------------------------------*/
//@ Methode : AjouterElement
//@
//@ Description : Ajoute un element dans la liste
//@
//@ Parametres :
//@               (ENTREE) Objet : Objet a ajouter dans la liste
//@
//@ Valeur de retour : 
//@               Position de l'objet ajoute (-1 si erreur)
//@
//@ ------------------------------------------------------------------*/
void CMN_DAT_ListeObjets::AjouterElement(CMN_DAT_Objet& objet)
{
  objet.Synthetiser(m_Config);
}


//@ ------------------------------------------------------------------*/
//@ Methode : SupprimerElement
//@
//@ Description : Supprime un element de la liste
//@
//@ Parametres :
//@               (ENTREE) nPosition : Position de l'objet a
//@                                    supprimer dans la liste
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK si la fonction a reussi
//@               CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN_DAT_ListeObjets::SupprimerElement(int nPosition)
{
  t_CMN__Statut statut = CMN__STATUT_OK;
  t_Chaine      identifiant("");


  LireIdentifiantElement(nPosition, identifiant);
  if (m_Config.DetruireSection(identifiant) != CMN__STATUT_OK)
    {
      statut = CMN__STATUT_NOK;
    }


  return statut;        
}


//@ ------------------------------------------------------------------*/
//@ Methode : ChargerListe
//@
//@ Description : Charge la liste a partir d'un fichier
//@
//@ Parametres :
//@               (ENTREE) nomFichier : Nom du fichier contenant la
//@                                     description des objets
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK si la fonction a reussi
//@               CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN_DAT_ListeObjets::ChargerListe(t_Chaine nomFichier)
{
  int statutCfg = m_Config.ChargerFichier(nomFichier);
 
  if ((statutCfg != CMN__STATUT_OK) &&
      (statutCfg != CMN__CONFIGURATION_ERR_AUCUN_CHARGEMENT))
    {
      statutCfg = CMN__STATUT_NOK;
    }
  else
    {
      statutCfg = CMN__STATUT_OK;
    }


  return statutCfg;     
}


//@ ------------------------------------------------------------------*/
//@ Methode : SauverListe
//@
//@ Description : Sauve la liste dans un fichier
//@
//@ Parametres :
//@               (ENTREE) nomFichier : Nom du fichier qui contiendra
//@                                     la description des objets
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK si la fonction a reussi
//@               CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN_DAT_ListeObjets::SauverListe(t_Chaine nomFichier)
{
  t_CMN__Statut statut = CMN__STATUT_OK;


  if (m_Config.SauverFichier(nomFichier) != CMN__STATUT_OK)
    {
      statut = CMN__STATUT_NOK;
    }


  return statut;        
}


//======================================================================
//                                      METHODES PRIVEES
//======================================================================
