static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/DAT/src/CMN_DAT_ModeleAction.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ ---------------------------------------------------------------------------
//@ Composant : CMN
//@ Sous-composant : CMN_DAT
//@ CLASSE : CMN_DAT_ModeleAction
//@ 
//@ Commentaire :
//@    Ce module gere la classe CMN_DAT_ModeleAction, qui permet de 
//@    stocker l'ensemble des actions
//@                                     
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

//
// Fichiers include
//

// STL
#include <iostream>

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN_DAT_ModeleAction.h"


//
// Definition des constantes
//
const t_Chaine CMN_DAT_ModeleAction::CHAMP_SIMULATION = "SIMULATION";


//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CMN_DAT_ModeleAction()
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_ModeleAction::CMN_DAT_ModeleAction()
{
  m_Simulation = -1;
}


//@ ------------------------------------------------------------------*/
//@ Methode : CMN_DAT_ModeleAction()
//@
//@ Description : Constructeur de recopie
//@
//@ Parametres :
//@               (ENTREE) unModeleAction
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_ModeleAction::CMN_DAT_ModeleAction(CMN_DAT_ModeleAction& 
                                           unModeleAction)
{
  if (&unModeleAction == this)
    {
      return;
    }
  
  CMN_DAT_Objet::operator =( unModeleAction);

  //@ Affectation du parametre de simulation du modele d'action
  m_Simulation = unModeleAction.Simulation();
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~CMN_DAT_ModeleAction()
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_ModeleAction::~CMN_DAT_ModeleAction()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator =
//@
//@ Description : Operateur =
//@
//@ Parametres :
//@               (ENTREE) unModeleAction : Objet source
//@
//@ Valeur de retour : 
//@               CMN_DAT_ModeleAction : Objet destination
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_ModeleAction& 
CMN_DAT_ModeleAction::operator =(CMN_DAT_ModeleAction& unModeleAction)
{
  if (&unModeleAction == this)
    {
      return *this;
    }

  CMN_DAT_Objet::operator =( unModeleAction);

  //@ Affectation du parametre de simulation du modele d'action
  m_Simulation = unModeleAction.Simulation();


  return *this;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Analyser
//@
//@ Description : Analyse une section du fichier de configuration pour 
//@               remplir le ModeleAction
//@
//@ Parametres :
//@               (ENTREE) Config : Fichier de configuration
//@               (ENTREE) sIdentifiantElement: Section du fichier de
//@                                             configuration decrivant
//@                                             le modele des actions
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_DAT_ModeleAction::Analyser(CMN__Configuration& Config,
                                    t_Chaine sIdentifiantElement)
{
  t_Chaine s("");


  CMN_DAT_Objet::Analyser(Config, sIdentifiantElement);

  //@ On lit le temps de simulation
  Config.LireValeur(m_Identifiant, CHAMP_SIMULATION, s);
  if (s.empty())
    {
      m_Simulation = -1;
    }
  else
    {
      m_Simulation = ChaineVersEntier(s);
    }
}
