static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/DAT/src/CMN_DAT_ModeleTache.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ ---------------------------------------------------------------------------
//@ Composant : CMN
//@ Sous-composant : CMN_DAT
//@ CLASSE : CMN_DAT_ModeleTache
//@ 
//@ Commentaire :
//@    Ce module gere la classe CMN_DAT_ModeleTache, qui permet de 
//@    stocker l'ensemble des taches 
//@                                     
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Adaptation a l'OPS : ajout des parametres de simulation
//@ ---------------------------------------------------------------------------
//@ VERSION : 22/07/1999 : GRT
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

//
// Fichiers include
//

// STL
#include <iostream>

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN_DAT_ModeleTache.h"

//
// Definition des constantes
//
const t_Chaine CMN_DAT_ModeleTache::CHAMP_SIMULATION         = "SIMULATION";
const t_Chaine CMN_DAT_ModeleTache::CHAMP_ACTION             = "ACTION";
const t_Chaine CMN_DAT_ModeleTache::CHAMP_NOMBRE_ACTIONS     = "NOMBRE ACTIONS";


//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CMN_DAT_ModeleTache()
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@               
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_ModeleTache::CMN_DAT_ModeleTache()
{
  m_Simulation    = -1;
  m_NombreActions = 0;
}


//@ ------------------------------------------------------------------*/
//@ Methode : CMN_DAT_ModeleTache()
//@
//@ Description : Constructeur de recopie
//@
//@ Parametres :
//@               (ENTREE) unModeleTache
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_ModeleTache::CMN_DAT_ModeleTache (CMN_DAT_ModeleTache& 
                                          unModeleTache)
{
  // Compteur d'iteration
  int i = 0;
  

  if (&unModeleTache == this)
    {
      return;
    }

  CMN_DAT_Objet::operator =( unModeleTache);

  //@ Affectation du parametre de simulation du modele de tache
  m_Simulation    = unModeleTache.Simulation();

  //@ Affectation du nombre d'actions du modele de tache
  m_NombreActions = unModeleTache.NombreActions();

  //@ POUR le nombre d'actions
  for (i=0;i<m_NombreActions;i++)
    {
      //@ Ajouter dans la liste des actions l'action du modele
      m_ListeActions.push_back(unModeleTache.Action(i));
    }
  //@ FIN POUR le nombre d'actions
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~CMN_DAT_ModeleTache()
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_ModeleTache::~CMN_DAT_ModeleTache()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator =
//@
//@ Description : Operateur =
//@
//@ Parametres :
//@               (ENTREE) unModeleTache : Objet source
//@
//@ Valeur de retour : 
//@               CMN_DAT_ModeleTache : Objet destination
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_ModeleTache&
CMN_DAT_ModeleTache::operator =(CMN_DAT_ModeleTache& unModeleTache)
{
  // Compteur d'iteration
  int i = 0;


  if (&unModeleTache == this)
    {
      return *this;
    }

  CMN_DAT_Objet::operator =( unModeleTache);

  //@ Affectation du parametre de simulation du modele de tache
  m_Simulation    = unModeleTache.Simulation();
  
  //@ Affectation du nombre d'actions du modele de tache
  m_NombreActions = unModeleTache.NombreActions();
  
  //@ POUR le nombre d'actions
  for (i=0;i<m_NombreActions;i++)
    {
      //@ Ajouter dans la liste des actions l'action du modele
     m_ListeActions.push_back(unModeleTache.Action(i));
    }
  //@ FIN POUR le nombre d'actions


  return *this;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Analyser
//@
//@ Description : Analyse une section du fichier de configuration pour 
//@               remplir le ModeleTache
//@
//@ Parametres :
//@               (ENTREE) Config : Fichier de configuration
//@               (ENTREE) sIdentifiantElement: Section du fichier de
//@                                             configuration decrivant
//@                                             le modele de taches
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_DAT_ModeleTache::Analyser(CMN__Configuration& Config,
                                   t_Chaine sIdentifiantElement)
{
  t_Chaine s("");
  t_Chaine nomChamp("");
  int      i = 0;


  CMN_DAT_Objet::Analyser(Config, sIdentifiantElement);

  //@ On lit le temps de simulation
  Config.LireValeur(m_Identifiant, CHAMP_SIMULATION, s);
  if (s.empty())
    {
      m_Simulation = -1;
    }
  else
    {
      m_Simulation = ChaineVersEntier(s);
    }

  //@ On lit le nombre d'actions
  Config.LireValeur(m_Identifiant, CHAMP_NOMBRE_ACTIONS, s);
  if (s.empty()) 
    {
      m_NombreActions = 0;
    }
  else
    {
      m_NombreActions = ChaineVersEntier(s);
    }

  //@ Sauvegarde des actions
  for (i=1; i<=m_NombreActions; i++)
    {
      nomChamp = CHAMP_ACTION;
      nomChamp += EntierVersChaine(i);
      Config.LireValeur(m_Identifiant, nomChamp, s);
      m_ListeActions.push_back(s);
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : Action
//@
//@ Description : Retourne une action
//@
//@ Parametres :
//@               (ENTREE) NoAction (0 a m_NombreActions-1): Numero de
//@                                                          l'action a
//@                                                          retourner
//@
//@ Valeur de retour : 
//@               Nom de l'action
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN_DAT_ModeleTache::Action(int NoAction)
{
  t_Chaine                s("");
  t_ListeChaine::iterator i;
  int                     j = 0;


  //@ SI le numero de l'action depasse le nombre d'actions du modele
  if (NoAction >= m_NombreActions)
    {
      //@ Affecter une chaine vide au nom de l'action
      s = "";
    }
  //@ SINON
  else
    {
      //@ Parcours de la liste des actions jusqu'a l'action
      //@ d'indice recherche
      for (i = m_ListeActions.begin(),j=0; j<NoAction; i++,j++);

      //@ Affecter la valeur de la position courante de la liste 
      //@ des actions au nom de l'action
      s = *i;
    }
  //@ FIN SI


  //@ Retourner le nom de l'action
  return s;
}
