static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/DAT/src/CMN_DAT_ModeleTraitement.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ ---------------------------------------------------------------------------
//@ Composant : CMN
//@ Sous-composant : CMN_DAT
//@ CLASSE : CMN_DAT_ModeleTraitement
//@ 
//@ Commentaire :
//@    Ce module gere la classe CMN_DAT_ModeleTraitement, qui permet 
//@    de stocker l'ensemble des traitements
//@                                     
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

//
// Fichiers include
//

// STL
#include <iostream>

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN_DAT_ModeleTraitement.h"

// FT-2765
const t_Chaine CMN_DAT_ModeleTraitement::TYPE_TACHE_AUCUN = "AUCUN";
const t_Chaine CMN_DAT_ModeleTraitement::TYPE_TACHE_RDV   = "RDV";
const t_Chaine CMN_DAT_ModeleTraitement::TYPE_TACHE_LIGNE = "LIGNE";
// ...FT-2765

//
// Definition des constantes
//
const t_Chaine CMN_DAT_ModeleTraitement::CHAMP_TACHE         = "TACHE";
const t_Chaine CMN_DAT_ModeleTraitement::CHAMP_TYPE_TACHE    = "TYPE TACHE";
const t_Chaine CMN_DAT_ModeleTraitement::CHAMP_NOMBRE_TACHES = "NOMBRE TACHES";


//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CMN_DAT_ModeleTraitement()
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_ModeleTraitement::CMN_DAT_ModeleTraitement()
{
  m_NombreTaches = 0;
}


//@ ------------------------------------------------------------------*/
//@ Methode : CMN_DAT_ModeleTraitement
//@
//@ Description : Constructeur de recopie
//@
//@ Parametres :     
//@               (ENTREE) unModeleTraitement
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_ModeleTraitement::
CMN_DAT_ModeleTraitement(CMN_DAT_ModeleTraitement& unModeleTraitement)
{
  // Compteur d'iteration
  int i = 0;


  if (&unModeleTraitement == this)
    {
      return;
    }
  
  CMN_DAT_Objet::operator =( unModeleTraitement);

  //@ Affectation du nombre de taches du modele de traitement
  m_NombreTaches = unModeleTraitement.NombreTaches();

  //@ POUR le nombre de taches
  for (i=0;i<m_NombreTaches;i++)
    {
      //@ Ajouter dans la liste des taches la tache du modele
      m_ListeTaches.push_back(unModeleTraitement.Tache(i));

      //@ Ajouter dans la liste des types de taches le type de 
      //@ tache du modele
      m_TypeTaches.push_back(unModeleTraitement.TypeTache(i));
    }
  //@ FIN POUR le nombre de taches
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~CMN_DAT_ModeleTraitement()
//@
//@ Description : Destructeur 
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_ModeleTraitement::~CMN_DAT_ModeleTraitement()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator =
//@
//@ Description : Operateur =
//@
//@ Parametres :
//@               (ENTREE) unModeleTraitement : Objet source
//@
//@ Valeur de retour : 
//@               CMN_DAT_ModeleTraitement : Objet destination
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_ModeleTraitement& 
CMN_DAT_ModeleTraitement::operator =(CMN_DAT_ModeleTraitement& 
                                     unModeleTraitement)
{
  // Compteur d'iteration
  int i = 0;


  if (&unModeleTraitement == this)
    {
      return *this;
    }

  CMN_DAT_Objet::operator =( unModeleTraitement);

  //@ Affectation du nombre de taches du modele de traitement
  m_NombreTaches = unModeleTraitement.NombreTaches();

  //@ POUR le nombre de taches
  for (i=0; i<m_NombreTaches; i++)
    {
      //@ Ajouter dans la liste des taches la tache du modele
      m_ListeTaches.push_back(unModeleTraitement.Tache(i));

      //@ Ajouter dans la liste des types de taches le type de 
      //@ tache du modele
      m_TypeTaches.push_back(unModeleTraitement.TypeTache(i));
    }
  //@ FIN POUR le nombre de taches
  

  return *this;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Analyser
//@
//@ Description : Analyse une section du fichier de configuration pour 
//@               remplir le modele de traitement
//@
//@ Parametres :
//@               (ENTREE) Config : Fichier de configuration
//@               (ENTREE) sIdentifiantElement: Section du fichier de
//@                                             configuration decrivant
//@                                             le modeleTraitement
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_DAT_ModeleTraitement::Analyser(CMN__Configuration& Config,
                                        t_Chaine sIdentifiantElement)
{
  int      i = 0;
  t_Chaine s("");
  t_Chaine nomChamp("");


  CMN_DAT_Objet::Analyser(Config, sIdentifiantElement);

  //@ On lit le nombre de traitements
  Config.LireValeur(m_Identifiant,CHAMP_NOMBRE_TACHES,s);
  if (s.empty()) 
    {
      m_NombreTaches = 0;
    } 
  else
    {
      m_NombreTaches = ChaineVersEntier(s);
    }

  //@ Sauvegarde des taches
  for (i=1; i<=m_NombreTaches; i++)
    {
      nomChamp = CHAMP_TACHE;
      nomChamp += EntierVersChaine(i);
      Config.LireValeur(m_Identifiant, nomChamp, s);
      m_ListeTaches.push_back(s);

      nomChamp = CHAMP_TYPE_TACHE;
      nomChamp += EntierVersChaine(i);
      Config.LireValeur(m_Identifiant, nomChamp, s);
      m_TypeTaches.push_back(s);
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : Tache
//@
//@ Description : Retourne une tache
//@
//@ Parametres :
//@               (ENTREE) NoTache (0 a m_NombreTaches-1): Numero de
//@                                                        la tache a
//@                                                        retourner
//@
//@ Valeur de retour : 
//@               Nom de la tache
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN_DAT_ModeleTraitement::Tache(int NoTache)
{
  t_ListeChaine::iterator i;
  t_Chaine                s("");
  int                     j = 0;


  //@ SI le numero de la tache depasse le nombre de taches du modele
  if (NoTache >= m_NombreTaches)
    {
      //@ Affecter une chaine vide au nom de la tache
      s = "";
    }
  //@ SINON
  else
    {
      //@ Parcours de la liste des taches jusqu'a la tache
      //@ d'indice recherche
      for (i = m_ListeTaches.begin(),j=0; j<NoTache; i++,j++);

      //@ Affecter la valeur de la position courante de la liste 
      //@ des taches au nom de la tache
      s = *i;
    }
  //@ FIN SI


  //@ Retourner le nom de la tache
  return s;
}


//@ ------------------------------------------------------------------*/
//@ Methode : TypeTache
//@
//@ Description : Retourne le type d'une tache
//@
//@ Parametres :
//@               (ENTREE) NoTache (0 a m_NombreTaches-1): Numero de
//@                                                        la tache a
//@                                                        retourner
//@
//@ Valeur de retour : 
//@               Type de la tache
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN_DAT_ModeleTraitement::TypeTache(int NoTache)
{
  t_ListeChaine::iterator i;
  t_Chaine                s("");
  int                     j = 0;


  //@ SI le numero de la tache depasse le nombre de taches du modele
  if (NoTache >= m_NombreTaches)
    {
      //@ Affecter une chaine vide au type de la tache
      s = "";
    }
  //@ SINON
  else
    {
      //@ Parcours de la liste des types de taches jusqu'au type
      //@ de tache d'indice recherche
      for (i = m_TypeTaches.begin(),j=0; j<NoTache; i++,j++);

      //@ Affecter la valeur de la position courante de la liste 
      //@ des types de taches au type de la tache
      s = *i;
    }
  //@ FIN SI


  //@ Retourner le type de la tache
  return s;
}
