static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/DAT/src/CMN_DAT_Objet.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ ---------------------------------------------------------------------------
//@ Composant : CMN
//@ Sous-composant : CMN_DAT
//@ CLASSE : CMN_DAT_Objet
//@ 
//@ Commentaire :
//@    Ce module gere la classe CMN_DAT_Objet
//@                                     
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 22/07/1999 : GRT
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

//
// Fichiers include
//

// STL
#include <iostream>
#include <cstdio>

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN__Version.h"
#include "CMN_DAT_Objet.h"


//
// Definition des constantes
//
#define IDENTIFIANT   "Objet"
#define CHAMP_VERSION "Version"


//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CMN_DAT_Objet()
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_Objet::CMN_DAT_Objet()
{
  m_Version = CMN__Version_Valeur();
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~CMN_DAT_Objet()
//@
//@ Description : Destructeur 
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_Objet::~CMN_DAT_Objet()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator =
//@
//@ Description : Operateur =
//@
//@ Parametres :
//@               (ENTREE) unObjet : Objet origine
//@
//@ Valeur de retour : 
//@               CMN_DAT_Objet : Objet destination
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_Objet& CMN_DAT_Objet::operator =(const CMN_DAT_Objet& unObjet)
{
  if (&unObjet == this)
    return *this;

  m_Identifiant = unObjet.Identifiant();

  return *this;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Synthetiser
//@
//@ Description : Genere une section pour l'objet dans le fichier de 
//@               configuration
//@
//@ Parametres :
//@               (ENTREE) Config : Fichier de configuration
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_DAT_Objet::Synthetiser(CMN__Configuration& Config)
{
  //@ On n'ecrit rien car l'identifiant est le nom de la section
  //@ qui est cree lorsque l'on enregistre un de ses champs
  if (m_Identifiant == "")
    {
      cerr << "ERREUR : Identifiant non defini" << endl;
    }

  Config.EcrireValeur(m_Identifiant,CHAMP_VERSION, m_Version);
}


//@ ------------------------------------------------------------------*/
//@ Methode : Analyser
//@
//@ Description : Analyse une section du fichier de configuration pour 
//@               remplir l'objet
//@
//@ Parametres :
//@               (ENTREE) Config : Fichier de configuration
//@               (ENTREE) sIdentifiantElement: Section du fichier 
//@                                             de configuration
//@                                             decrivant l'objet
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_DAT_Objet::Analyser(CMN__Configuration& Config,
                             t_Chaine sIdentifiantElement)
{
  //@ Pour l'objet de base, l'identifiant est le nom de la section
  m_Identifiant = sIdentifiantElement;

  //@ Lire la valeur
  Config.LireValeur(m_Identifiant,CHAMP_VERSION, m_Version);
}


//@ ------------------------------------------------------------------*/
//@ Methode : ChargerFichier 
//@
//@ Description : Charge la liste a partir d'un fichier
//@
//@ Parametres :
//@               (ENTREE) NomFichier : Nom du fichier contenant la 
//@                                     description des objets
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK si le chargement a ete realise
//@               CMN__STATUT_NOK sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN_DAT_Objet::ChargerFichier(t_Chaine NomFichier)
{
  CMN__Configuration Config;
  t_CMN__Statut StatutCfg;


  StatutCfg = Config.ChargerFichier(NomFichier);
  if ((StatutCfg != CMN__STATUT_OK) &&
      (StatutCfg != CMN__CONFIGURATION_ERR_AUCUN_CHARGEMENT))
    {
      StatutCfg = CMN__STATUT_NOK;
    }
  else
    {
      Analyser(Config,IDENTIFIANT);
      StatutCfg = CMN__STATUT_OK;
    }

  return StatutCfg;     
}


//@ ------------------------------------------------------------------*/
//@ Methode : SauverFichier
//@
//@ Description : Sauve l'objet dans un fichier
//@
//@ Parametres :
//@               (ENTREE) NomFichier du fichier qui contiendra l'objet
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK si la sauvegarde a ete realisee
//@               CMN__STATUT_NOK sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN_DAT_Objet::SauverFichier(t_Chaine NomFichier)
{
  CMN__Configuration Config;
  t_CMN__Statut      Statut = CMN__STATUT_OK;

  m_Identifiant = IDENTIFIANT;
  Synthetiser(Config);
  if (Config.SauverFichier(NomFichier) != CMN__STATUT_OK)
    {
      Statut = CMN__STATUT_NOK;
    }

  return Statut;        
}


//======================================================================
//                                      METHODES PROTEGEES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : BouleenVersChaine
//@
//@ Description : Convertit une valeur booleenne en chaine de caractere
//@               (utilise pour la synthese d'un objet)
//@
//@ Parametres :
//@               (ENTREE) Valeur booleenne : 0 : OUI, 1: NON
//@
//@ Valeur de retour : 
//@               Chaine contenant la valeur
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN_DAT_Objet::BouleenVersChaine(t_CMN__Booleen valeur)
{
  t_Chaine reponse("NON");

  if (valeur == 1)
    {
      reponse = "OUI";
    }

  return reponse;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ChaineVersBouleen
//@
//@ Description : Convertit une chaine de caractere en valeur booleenne 
//@               (utilise pour l'analyse d'un objet)
//@
//@ Parametres :
//@               (ENTREE) Chaine : OUI ou NON
//@
//@ Valeur de retour : 
//@               Booleen contenant la valeur
//@
//@ ------------------------------------------------------------------*/
t_CMN__Booleen CMN_DAT_Objet::ChaineVersBouleen(t_Chaine valeur)
{
  t_CMN__Booleen reponse = CMN__FAUX;

  if (valeur == "OUI")
    {
      reponse = CMN__VRAI;
    }

  return reponse;
}


//@ ------------------------------------------------------------------*/
//@ Methode : EntierVersChaine
//@
//@ Description : Convertit une valeur entiere en chaine de caractere
//@               (utilise pour la synthese d'un objet)
//@
//@ Parametres :
//@               (ENTREE) Valeur : Valeur a convertir
//@
//@ Valeur de retour : 
//@               Chaine contenant la valeur
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN_DAT_Objet::EntierVersChaine(long valeur)
{
  t_Chaine s("");

  if (valeur != INT_NAN)
   {
     char buffer[100];
     
     snprintf(buffer, 100, "%ld", valeur);
     s = buffer;
   }

  return s;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ChaineVersEntier
//@
//@ Description : Convertit une chaine de caractere en valeur entiere 
//@               (utilise pour l'analyse d'un objet)
//@
//@ Parametres :
//@               (ENTREE) s: Valeur a convertir
//@
//@ Valeur de retour : 
//@               Nombre contenant la valeur
//@
//@ ------------------------------------------------------------------*/
long CMN_DAT_Objet::ChaineVersEntier(t_Chaine s)
{
  long         i = 0;
  t_ChaineFixe Buffer;


  Buffer[0] = 0;
  if (sscanf(s.c_str(), "%ld%s", &i, Buffer) == 1)
    {
      return i;
    }
  else
    {
      return INT_NAN;
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : ReelVersChaine
//@
//@ Description : Convertit une valeur reelle en chaine de caractere
//@               (utilise pour la synthese d'un objet)
//@
//@ Parametres :
//@               (ENTREE) Valeur : Valeur a convertir
//@
//@ Valeur de retour : 
//@               Chaine contenant la valeur
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN_DAT_Objet::ReelVersChaine(double valeur)
{
  t_Chaine s("");

  if (valeur != DBL_NAN)
    {
      char buffer[100];

      snprintf(buffer, 100, "%e", valeur);
      s = buffer;
    }

  return s;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ChaineVersReel
//@
//@ Description : Convertit une chaine de caractere en valeur reelle
//@               (utilise pour l'analyse d'un objet)
//@
//@ Parametres :
//@               (ENTREE) s: Valeur a convertir
//@
//@ Valeur de retour : 
//@               Nombre contenant la valeur
//@
//@ ------------------------------------------------------------------*/
double CMN_DAT_Objet::ChaineVersReel(t_Chaine s)
{
  double       f = 0.;
  t_ChaineFixe Buffer;

  Buffer[0] = 0;
  if (sscanf(s.c_str(), "%le%s", &f, Buffer)== 1)
    {
      return f;
    }
  else
    {
      return DBL_NAN;
    }
}


//======================================================================
//                                      METHODES PRIVEES
//======================================================================
