// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/DAT/src/CMN_DAT_Objet.h,v 1.3 2011/11/25 10:17:39 dpauly Exp $
//@ ---------------------------------------------------------------------------
//@ Composant : CMN
//@ Sous-composant : CMN_DAT
//@ CLASSE : CMN_DAT_Objet
//@
//@ Commentaire :
//@    Fichier d'entete de la classe CMN_DAT_Objet
//@                                                                     
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ SMM-M4-FA-2366-CN   4/10/01 GRT
//@ Chaque objet derive de CMN_DAT_Objet devrait implementer la methode
//@ Initialiser qui devrait etre appelee dans le constructeur (cette
//@ modification a ete faite dans le cadre de la FA2366 mais est de niveau
//@ plus general)
//@ ---------------------------------------------------------------------------
//@ VERSION : 22/07/1999 : GRT
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

#ifndef CMN_DAT_Objet_h
#define CMN_DAT_Objet_h


//
// Fichiers include
//

// STL

// Standard

// Projet
#include "CMN__Config.h"


#define FLOAT_MIN       -1e32


class CMN_DAT_Objet {

    /*================================================================*/

 public:

    ////////////////////////////////
    // Constructeur
    ////////////////////////////////
    CMN_DAT_Objet();

    ////////////////////////////////
    // Destructeur
    ////////////////////////////////
    virtual ~CMN_DAT_Objet();

    ////////////////////////////////
    // Accesseurs
    ////////////////////////////////

    // Positionne l'identifiant de l'objet
    virtual void Identifiant(t_Chaine unIdentifiant)
        {m_Identifiant = unIdentifiant;}
    virtual void Identifiant(long unIdentifiant)     
        {m_Identifiant = EntierVersChaine(unIdentifiant);}
        
    // Recupere l'identifiant de l'objet
    virtual const t_Chaine& Identifiant() const {return m_Identifiant;}


    ////////////////////////////////
    // Methodes publiques
    ////////////////////////////////

    // Initialiseur dans le cadre de la FA2366
    virtual void   Initialiser() {CMN_DAT_Objet();}; 

    // Genere une section pour l'objet dans le fichier de configuration
    virtual void   Synthetiser(CMN__Configuration& Config);

    // Analyse une section du fichier de configuration pour remplir
    // l'objet
    virtual void   Analyser(CMN__Configuration& Config,
                            t_Chaine sIdentifiantElement);

    // Methodes de conversion
    t_Chaine       BouleenVersChaine(t_CMN__Booleen valeur);
    t_Chaine       EntierVersChaine(long Valeur);
    t_Chaine       ReelVersChaine(double Valeur);
    t_CMN__Booleen ChaineVersBouleen(t_Chaine Valeur);
    long           ChaineVersEntier(t_Chaine s);
    double         ChaineVersReel(t_Chaine s);

    // Chargement de l'objet a partir d'un fichier
    t_CMN__Statut  ChargerFichier(t_Chaine NomFichier);

    // Sauvegarde de l'objet dans un fichier
    t_CMN__Statut  SauverFichier(t_Chaine NomFichier);


    /*================================================================*/

 protected:

    ////////////////////////////////
    // Methodes protegees
    ////////////////////////////////
    virtual CMN_DAT_Objet& operator =(const CMN_DAT_Objet& unObjet);

    ////////////////////////////////
    // Attributs proteges
    ////////////////////////////////
    t_Chaine m_Identifiant;


    /*================================================================*/

 private:

    ////////////////////////////////
    // Methodes privees
    ////////////////////////////////

    ////////////////////////////////
    // Attributs prives
    ////////////////////////////////
    t_Chaine m_Version;
};

#endif
