static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/DAT/src/CMN_DAT_StageInfo.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ ---------------------------------------------------------------------------
//@ Composant : CMN
//@ Sous-composant : CMN_DAT
//@ CLASSE : CMN_DAT_StageInfo
//@ 
//@ Commentaire :
//@    Ce module gere la classe CMN_DAT_StageInfo, qui stocke les 
//@    informations liees au compte-rendu d'une etape d'un Work Order
//@
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

//
// Fichiers include
//

// STL
#include <iostream>
#include <cstdio>

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN_DAT_StageInfo.h"


//
// Definition des constantes
//
const unsigned int CMN_DAT_StageInfo::LENGTH_STAGE_NAME  = 40;
const unsigned int CMN_DAT_StageInfo::LENGTH_LOG_MESSAGE = 255;
const unsigned int CMN_DAT_StageInfo::LENGTH_TIME        = 20;
const t_Chaine     CMN_DAT_StageInfo::FORMAT_TIME        = "%5.3f";


//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CMN_DAT_StageInfo
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_StageInfo::CMN_DAT_StageInfo() 
  : m_StageName(""), m_MHSTelemetry(""), m_ProcessingStart(""), 
    m_ProcessingStop(""), m_ElapsedTime(""), m_UserTime(""), 
    m_LogMessage("")
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~CMN_DAT_StageInfo
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_StageInfo::~CMN_DAT_StageInfo()
{
  //@ RAZ des listes m_ListeUsedFileName et m_ListeGeneratedFile
  m_ListeUsedFileName.clear();
  m_ListeGeneratedFile.clear();
}


//@ ------------------------------------------------------------------*/
//@ Methode : StageName
//@
//@ Description : Met a jour l'attribut StageName
//@
//@ Parametres :
//@               (ENTREE) stageName : Valeur pour l'attribut
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_DAT_StageInfo::StageName(t_Chaine stageName)
{
  if (stageName.length() > LENGTH_STAGE_NAME)
    {
      m_StageName = stageName.substr(0, LENGTH_STAGE_NAME);
    }
  else
    {
      m_StageName = stageName;
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : AjouterGeneratedFile
//@
//@ Description : Ajoute un element dans la liste des fichiers generes
//@
//@ Parametres :
//@               (ENTREE) generatedFile : Valeur pour le nouvel element
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_DAT_StageInfo::AjouterGeneratedFile(t_GeneratedFile& generatedFile)
{
  // Iterateur de data gap
  t_ListeDataGap::iterator pInfo;

  for (pInfo = generatedFile.ListeDataGap().begin(); 
       pInfo != generatedFile.ListeDataGap().end(); pInfo++)
    {
      if ((pInfo->Reason()).length() > CMN_DAT_DataGap::LENGTH_GAP_REASON)
        {
          pInfo->Reason(pInfo->Reason().substr(0, CMN_DAT_DataGap::
                                               LENGTH_GAP_REASON));
        }
    }

  m_ListeGeneratedFile.push_back(generatedFile);
}


//@ ------------------------------------------------------------------*/
//@ Methode : ElapsedTime
//@
//@ Description : Met a jour l'attribut ElapsedTime
//@
//@ Parametres :
//@               (ENTREE) valeur : Valeur pour l'attribut
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_DAT_StageInfo::ElapsedTime(double valeur)
{
  char buffer[LENGTH_TIME];

  sprintf(buffer, FORMAT_TIME.c_str(), valeur);
  m_ElapsedTime = buffer;
}


//@ ------------------------------------------------------------------*/
//@ Methode : UserTime
//@
//@ Description : Met a jour l'attribut UserTime
//@
//@ Parametres :
//@               (ENTREE) valeur : Valeur pour l'attribut
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_DAT_StageInfo::UserTime(double valeur)
{
  char buffer[LENGTH_TIME];

  sprintf(buffer, FORMAT_TIME.c_str(), valeur);
  m_UserTime = buffer;
}


//@ ------------------------------------------------------------------*/
//@ Methode : LogMessage
//@
//@ Description : Met a jour l'attribut LogMessage
//@
//@ Parametres :
//@               (ENTREE) logMessage : Valeur pour l'attribut
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_DAT_StageInfo::LogMessage(t_Chaine logMessage)
{
  if (logMessage.length() > LENGTH_LOG_MESSAGE)
    {
      m_LogMessage = logMessage.substr(0, LENGTH_LOG_MESSAGE);
    }
  else
    {
      m_LogMessage = logMessage;
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : Afficher
//@
//@ Description : Affiche le contenu de l'objet
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_DAT_StageInfo::Afficher()
{
  // Iterateurs
  t_ListeChaine::iterator        pInfo;
  t_ListeGeneratedFile::iterator pInfoGeneratedFile;
  
  // Compteur d'iterations
  int                            i = 1;


  cout << "StageInfo:" << endl;
  cout << "\tStageName      : " << m_StageName << endl;
  cout << "\tMHSTelemetry   : " << m_MHSTelemetry << endl;
  cout << "\tProcessingStart: " << m_ProcessingStart << endl;
  cout << "\tProcessingStop : " << m_ProcessingStop  << endl;
  cout << "\tElapsedTime    : " << m_ElapsedTime << endl;
  cout << "\tUserTime       : " << m_UserTime << endl;

  cout << "\tNombre UsedFileName : " << m_ListeUsedFileName.size() 
       << endl;
  for (pInfo = m_ListeUsedFileName.begin();
       pInfo != m_ListeUsedFileName.end(); 
       pInfo++) 
    {
      cout << "\t\tno" << i++ << ":" << *pInfo << endl;
    }

  i = 1;
  cout << "\tNombre GeneratedFile: " << m_ListeGeneratedFile.size() 
       << endl;
  for (pInfoGeneratedFile = m_ListeGeneratedFile.begin(); 
       pInfoGeneratedFile != m_ListeGeneratedFile.end();
       pInfoGeneratedFile++) 
    {
      cout << "\t\tno" << i++ << ": ";
      pInfoGeneratedFile->Afficher();
      cout << endl;
    }

  cout << "Fin StageInfo" << endl;
}


//======================================================================
//                                      METHODES PRIVEES
//======================================================================
