static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/DAT/src/CMN_DAT_WorkOrder.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ ---------------------------------------------------------------------------
//@ Composant : CMN
//@ Sous-composant : CMN_DAT
//@ CLASSE : CMN_DAT_WorkOrder
//@ 
//@ Commentaire :
//@    Ce module gere la classe CMN_DAT_WorkOrder, qui stocke les 
//@    informations liees a un Work Order
//@
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

//
// Fichiers include
//

// STL
#include <iostream>

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN__CGSCst.h"
#include "CMN_DAT_WorkOrder.h"

// FT-2765
const t_Chaine CMN_DAT_WorkOrder::WORK_ORDER_TYPE_FIRST      = "FIRST";
const t_Chaine CMN_DAT_WorkOrder::WORK_ORDER_TYPE_MIDDLE     = "MIDDLE";
const t_Chaine CMN_DAT_WorkOrder::WORK_ORDER_TYPE_LAST       = "LAST";
const t_Chaine CMN_DAT_WorkOrder::WORK_ORDER_TYPE_FULL       = "FULL";
// ...FT-2765


//
// Definition des constantes
//
const t_Chaine CMN_DAT_WorkOrder::TIME_FLAG_NOT_DEFINED = "NOT DEFINED";


//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CMN_DAT_WorkOrder
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_WorkOrder::CMN_DAT_WorkOrder()
{
  m_ProcessingType      = CMN__StageTM::L1a;
  m_SensingStart.DateInit();
  m_SensingStop.DateInit();
  m_TimeIntervalFlag    = TIME_FLAG_NOT_DEFINED;
  m_TimeIntervalCounter = 0;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~CMN_DAT_WorkOrder
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_DAT_WorkOrder::~CMN_DAT_WorkOrder()
{
  m_ListeUnprocData.clear();
  m_ListeAuxData.clear();
  m_ListeProdModel.clear();
}


//@ ------------------------------------------------------------------*/
//@ Methode : ProcessingType
//@
//@ Description : Permet de mettre a jour l'attribut ProcessingType
//@
//@ Parametres :
//@               (ENTREE) type : Chaine contenant le type
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Type valide
//@               CMN__STATUT_NOK : Type invalide
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN_DAT_WorkOrder::ProcessingType(t_Chaine type)
{
  t_Chaine       typeCourant("");
  int            iStage  = 0;
  t_CMN__Booleen idFound = CMN__FAUX;

  // Valeur de retour de la methode
  t_CMN__Statut  status  = CMN__STATUT_NOK;


  //@ Convertir la chaine en majuscules
  type.Upper();

  //@ TANT QUE stage id non trouve
  while ((iStage < CMN__StageTM::NbrStageId) && (idFound == CMN__FAUX))
    {
      typeCourant = CMN_PGF_STAGE_ID[iStage];
      typeCourant.Upper();

      //@ Comparer le type et l'identifiant passe en parametre
      if (type == typeCourant)
        {
          m_ProcessingType = (CMN__StageTM::e_CMN__StageId)iStage;
          idFound = CMN__VRAI;
          status = CMN__STATUT_OK;
        }
      iStage++; 
    }
  //@ FIN TANT QUE


  //@ Renvoyer la valeur de retour de la methode
  return status;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Afficher
//@
//@ Description : Affiche le contenu de l'objet
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_DAT_WorkOrder::Afficher()
{
  t_ListeChaine::iterator pInfo;
  int i = 1;


  cout << "WorkOrder:" << endl;
  cout << "\tProcessingType     : " << m_ProcessingType << endl;
  cout << "\tSensingStart       : " << m_SensingStart.Chaine() << endl;
  cout << "\tSensingStop        : " << m_SensingStop.Chaine() << endl;
  cout << "\tTimeIntervalFlag   : " << m_TimeIntervalFlag << endl;
  cout << "\tTimeIntervalCounter: " << m_TimeIntervalCounter << endl;

  cout << "\tNombre UnprocData  : " << m_ListeUnprocData.size() << endl;
  for (pInfo = m_ListeUnprocData.begin(); 
       pInfo != m_ListeUnprocData.end(); 
       pInfo++) 
    {
      cout << "\t\tno" << i++ << ":" << *pInfo << endl;
    }

  i = 1;
  cout << "\tNombre AuxData     : " << m_ListeAuxData.size() << endl;
  for (pInfo = m_ListeAuxData.begin(); 
       pInfo != m_ListeAuxData.end(); 
       pInfo++) 
    {
      cout << "\t\tno" << i++ << ":" << *pInfo << endl;
    }

  i = 1;
  cout << "\tNombre ProdModel   : " << m_ListeProdModel.size() << endl;
  for (pInfo = m_ListeProdModel.begin(); 
       pInfo != m_ListeProdModel.end(); 
       pInfo++) 
    {
      cout << "\t\tno" << i++ << ":" << *pInfo << endl;
    }

  cout << "Fin WorkOrder" << endl;
}


//======================================================================
//                                      METHODES PRIVEES
//======================================================================
