static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/PRS/src/CMN_PRS_Parser.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ ---------------------------------------------------------------------------
//@ Composant : CMN
//@ CLASSE : CMN_PRS_Parser
//@
//@ Commentaire :
//@    Ce fichier gere la classe CMN_PRS_Parser
//@
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-2100-CR-290-CN : 04/04/2006 : B. GUENDE
//@ Added degraded cases encountered in the field: LogMessage of .rpt file
//@ See IA-2100-CR-290-THA
//@ ---------------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-144-CN : 21/09/2004 : B. MASSART
//@ Problemes lors du portage d'OPS vers Solaris par Meteo France
//@ Voir IA-2100-FC-144-THA.
//@ ---------------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-136-CN : 01/09/2004 : P.CABANE
//@ Reading of elements <UTC_LEAP_SECOND> and <B_OFFSET> has been added
//@ fiche de correction correspondante : IA-2100-FC-136-THA
//@ ---------------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-172-CN : 21/09/2006 : R.MORENO
//@ Gestion des manoeuvres: prise en compte des tags ANX signalant des
//@ passages au noeud ascendant.
//@ ---------------------------------------------------------------------------
//@ FA-ID : IA-FT-2512 (FA-2100-OPS) : 17/11/2008 : S. BRUNEL
//@ Fuites memoires signalees par EUMETSAT (EUM.EPS.NCR.3627.1)
//@ ---------------------------------------------------------------------------
//@ FA-ID : IA-FT-2364(FA-2100-OPS) : 20/09/2010 : S. BRUNEL
//@ Product without  manoeuvre flag raised 
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

/* FA144 : suppression define __MINMAX_DEFINED, residu BORLAND */

//
// Fichiers include
//

// STL
#include <string>
#include <iostream>
#include <fstream>

// Standard

// Xerces
#include <xercesc/dom/DOM.hpp>
#include <xercesc/dom/DomMemDebug.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLException.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/util/XMLUniDefs.hpp>
#include <xercesc/util/TranscodingException.hpp>
#include <xercesc/framework/XMLFormatter.hpp>
#include <xercesc/dom/DOM_DOMException.hpp>
#include <xercesc/sax/SAXParseException.hpp>
#include <xercesc/parsers/DOMParser.hpp>

// Projet
#include "CMN__Commun.h"
#include "CMN__Date.h"
#include "ERR__Exception.h"
#include "CMN_PRS_DumpFormatTarget.h"
#include "CMN_PRS_Parser.h"
#include "SD_DON_OBTUTC.h"
#include "SD_DON_OSV.h"


//
// Definition des constantes 
//

// Elements XML WorkOrder
const t_Chaine CMN_PRS_Parser::ELMT_PPF_WORK_ORDER    = "PPF_Work_Order";
const t_Chaine CMN_PRS_Parser::ELMT_PROCESSING_TYPE   = "ProcessingType";
const t_Chaine CMN_PRS_Parser::ELMT_SENSING_START     = "SensingStart";
const t_Chaine CMN_PRS_Parser::ELMT_SENSING_STOP      = "SensingStop";
const t_Chaine CMN_PRS_Parser::ELMT_TIME_FLAG         = "TimeIntervalFlag";
const t_Chaine CMN_PRS_Parser::ELMT_TIME_COUNTER      = "TimeIntervalCounter";
const t_Chaine CMN_PRS_Parser::ELMT_UNPROC_DATA       = "UnProcData";
const t_Chaine CMN_PRS_Parser::ELMT_AUX_DATA          = "AuxData";
const t_Chaine CMN_PRS_Parser::ELMT_PROD_MODEL        = "ProdModel";

// Elements XML OBTUTC
const t_Chaine CMN_PRS_Parser::ELMT_TYPE_ARBRE_OBTUTC = "FD_OBT_UTC_Time_Correlation";
const t_Chaine CMN_PRS_Parser::ELMT_CCU_OBT           = "CCU_OBT";
const t_Chaine CMN_PRS_Parser::ELMT_UTC               = "UTC";
const t_Chaine CMN_PRS_Parser::ELMT_A_STEP            = "A_STEP";

/* FA136 */
const t_Chaine CMN_PRS_Parser::ELMT_UTC_LEAP_SECOND   = "UTC_LEAP_SECOND";
const t_Chaine CMN_PRS_Parser::ELMT_B_OFFSET          = "B_OFFSET";

// Elements XML OSV
const t_Chaine CMN_PRS_Parser::ELMT_TYPE_ARBRE_OSV    = "FD_Metop_NOAA_Orbit_Prediction";
const t_Chaine CMN_PRS_Parser::ELMT_OSV_ORBIT_START   = "ORBIT_START";
const t_Chaine CMN_PRS_Parser::ELMT_OSV_FLAG          = "FLAG";
const t_Chaine CMN_PRS_Parser::ELMT_OSV_TIME          = "TIME";
/* FA 172 */
const t_Chaine CMN_PRS_Parser::FLAG_ANX               = "ANX";
const t_Chaine CMN_PRS_Parser::FLAG_OOP_END           = "OOP_END";
const t_Chaine CMN_PRS_Parser::FLAG_IP_END            = "IP_END";
const t_Chaine CMN_PRS_Parser::FLAG_IP_START          = "IP_START";
const t_Chaine CMN_PRS_Parser::FLAG_OOP_START         = "OOP_START";
/* ..FA 172 */

// Elements XML Report
const t_Chaine CMN_PRS_Parser::ELMT_PPF_REPORT        = "PPF_report";
const t_Chaine CMN_PRS_Parser::VERSION_REPORT         = "1.0";
const t_Chaine CMN_PRS_Parser::STANDALONE_REPORT      = "no";
const t_Chaine CMN_PRS_Parser::ELMT_WORK_ORDER        = "WorkOrder";
const t_Chaine CMN_PRS_Parser::ELMT_STAGE_INFO        = "StageInfo";
const t_Chaine CMN_PRS_Parser::ELMT_STAGE_NAME        = "StageName";
const t_Chaine CMN_PRS_Parser::ELMT_USED_FILE_NAME    = "UsedFileName";
const t_Chaine CMN_PRS_Parser::ELMT_GENERATED_FILE    = "GeneratedFile";
const t_Chaine CMN_PRS_Parser::ELMT_FILE_NAME         = "FileName";
const t_Chaine CMN_PRS_Parser::ELMT_MDR_OK            = "MdrTotalCount";
const t_Chaine CMN_PRS_Parser::ELMT_MDR_DEGRADED      = "MdrDegraded";
const t_Chaine CMN_PRS_Parser::ELMT_DATA_GAP          = "DataGap";
const t_Chaine CMN_PRS_Parser::ELMT_GAP_REASON        = "GapReason";
const t_Chaine CMN_PRS_Parser::ELMT_GAP_COUNT         = "GapCount";
const t_Chaine CMN_PRS_Parser::ELMT_MHS_TELEMETRY     = "InstrumentTelemetry";
const t_Chaine CMN_PRS_Parser::ELMT_PROCESSING_INFO   = "ProcessingInfo";
const t_Chaine CMN_PRS_Parser::ELMT_PROCESSING_START  = "ProcessingStart";
const t_Chaine CMN_PRS_Parser::ELMT_PROCESSING_STOP   = "ProcessingStop";
const t_Chaine CMN_PRS_Parser::ELMT_ELAPSED_TIME      = "ElapsedTime";
const t_Chaine CMN_PRS_Parser::ELMT_USER_TIME         = "UserTime";
const t_Chaine CMN_PRS_Parser::ELMT_LOG_MESSAGE       = "LogMessage";

// Configuration du parser
const t_Chaine CMN_PRS_Parser::UNREPFLAG_FAIL         = "fail";
const t_Chaine CMN_PRS_Parser::UNREPFLAG_REP          = "rep";
const t_Chaine CMN_PRS_Parser::VALSCHEME_NEVER        = "never";
const t_Chaine CMN_PRS_Parser::VALSCHEME_ALWAYS       = "always";
const t_Chaine CMN_PRS_Parser::PRS_DEFAULT_ENCODING   = "UTF-8";

// Messages d'erreur
const t_Chaine CMN_PRS_Parser::STR_XML_EXCEPTION_MSG  = "XML Exception message:";
const t_Chaine CMN_PRS_Parser::STR_DOM_EXCEPTION_MSG  = "DOM Exception message:";
const t_Chaine CMN_PRS_Parser::STR_ERROR_INIT         = "Error during Xerces-c Initialization";
const t_Chaine CMN_PRS_Parser::STR_ERROR_PARSING      = "An error occured during parsing ";
const t_Chaine CMN_PRS_Parser::STR_ERROR_PARSING_WO   = "An error occured during parsing work order ";
const t_Chaine CMN_PRS_Parser::STR_ERROR_PARSING_OBT  = "An error occured during parsing OBTUTC ";
const t_Chaine CMN_PRS_Parser::STR_ERROR_PARSING_OSV  = "An error occured during parsing OSV ";
const t_Chaine CMN_PRS_Parser::STR_ERROR_REPORTING    = "An error occured during writing report ";
const t_Chaine CMN_PRS_Parser::STR_ERROR_INVALID_TREE = "No valid XML tree";
const t_Chaine CMN_PRS_Parser::STR_ERROR_DUMPING_TREE = "Error during creation of output ";


//=====================================================================
//                        DONNEES STATIQUES
//=====================================================================


static const t_Chaine DOM_EXCEPTION_TYPE_STR [] ={"",
                                                  "INDEX_SIZE_ERR",
                                                  "DOMSTRING_SIZE_ERR",
                                                  "HIERARCHY_REQUEST_ERR",
                                                  "WRONG_DOCUMENT_ERR", 
                                                  "INVALID_CHARACTER_ERR",
                                                  "NO_DATA_ALLOWED_ERR",
                                                  "NO_MODIFICATION_ALLOWED_ERR",
                                                  "NOT_FOUND_ERR", 
                                                  "NOT_SUPPORTED_ERR",
                                                  "INUSE_ATTRIBUTE_ERR",
                                                  "INVALID_STATE_ERR",
                                                  "SYNTAX_ERR", 
                                                  "INVALID_MODIFICATION_ERR",
                                                  "NAMESPACE_ERR",
                                                  "INVALID_ACCESS_ERR"};

static  XMLFormatter* S_Formatter;
static  XMLCh*        S_EncodingName;
static  bool          S_LineFeed      = false;
static  bool          S_LineFeedBegin = false;


static const XMLCh S_EndElement[] = { chOpenAngle, chForwardSlash, chNull };
static const XMLCh S_EndPI[] = { chQuestion, chCloseAngle, chNull};
static const XMLCh S_StartPI[] = { chOpenAngle, chQuestion, chNull };
static const XMLCh S_XMLDecl1[] =
  {
    chOpenAngle, chQuestion, chLatin_x, chLatin_m, chLatin_l,
    chSpace, chLatin_v, chLatin_e, chLatin_r, chLatin_s, chLatin_i,
    chLatin_o, chLatin_n, chEqual, chDoubleQuote, chNull
  };
static const XMLCh S_XMLDecl2[] =
  {
    chDoubleQuote, chSpace, chLatin_e, chLatin_n, chLatin_c,
    chLatin_o, chLatin_d, chLatin_i, chLatin_n, chLatin_g, chEqual,
    chDoubleQuote, chNull
  };
static const XMLCh S_XMLDecl3[] =
  {
    chDoubleQuote, chSpace, chLatin_s, chLatin_t, chLatin_a,
    chLatin_n, chLatin_d, chLatin_a, chLatin_l, chLatin_o,
    chLatin_n, chLatin_e, chEqual, chDoubleQuote, chNull
  };
static const XMLCh S_XMLDecl4[] =
  {
    chDoubleQuote, chQuestion, chCloseAngle, chNull
  };

static const XMLCh S_StartCDATA[] =
  {
    chOpenAngle, chBang, chOpenSquare, chLatin_C, chLatin_D,
    chLatin_A, chLatin_T, chLatin_A, chOpenSquare, chNull
  };

static const XMLCh S_EndCDATA[] =
  {
    chCloseSquare, chCloseSquare, chCloseAngle, chNull
  };
static const XMLCh S_StartComment[] =
  {
    chOpenAngle, chBang, chDash, chDash, chNull
  };

static const XMLCh S_EndComment[] =
  {
    chDash, chDash, chCloseAngle, chNull
  };

static const XMLCh S_StartDoctype[] =
  {
    chOpenAngle, chBang, chLatin_D, chLatin_O, chLatin_C, chLatin_T,
    chLatin_Y, chLatin_P, chLatin_E, chSpace, chNull
  };
static const XMLCh S_Public[] =
  {
    chLatin_P, chLatin_U, chLatin_B, chLatin_L, chLatin_I,
    chLatin_C, chSpace, chDoubleQuote, chNull
  };
static const XMLCh S_System[] =
  {
    chLatin_S, chLatin_Y, chLatin_S, chLatin_T, chLatin_E,
    chLatin_M, chSpace, chDoubleQuote, chNull
  };
static const XMLCh S_StartEntity[] =
  {
    chOpenAngle, chBang, chLatin_E, chLatin_N, chLatin_T, chLatin_I,
    chLatin_T, chLatin_Y, chSpace, chNull
  };
static const XMLCh S_Notation[] =
  {
    chLatin_N, chLatin_D, chLatin_A, chLatin_T, chLatin_A,
    chSpace, chDoubleQuote, chNull
  };


//=====================================================================
//                        FONCTIONS STATIQUES
//=====================================================================

void PrintDocumentTypeNode(DOMString nodeName, DOM_Node& toWrite);
void PrintEntityNode(DOMString nodeName, DOM_Node& toWrite);


//=====================================================================
//                        METHODES AMIES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : ostream << DOMString
//@
//@ Description : Stream out a DOM string
//@
//@ Parametres :
//@               (ENTREE) target
//@               (ENTREE) s
//@
//@ Valeur de retour :
//@               ostream
//@
//@ ------------------------------------------------------------------*/
ostream& operator<<(ostream& target, const DOMString& s)
{
  char *p = s.transcode();
  target << p;
  delete [] p;

  return target;
}


//@ ------------------------------------------------------------------*/
//@ Methode : XMLFormatter << DOMString
//@
//@ Description : Format a DOM string
//@
//@ Parametres :
//@               (ENTREE) strm
//@               (ENTREE) s
//@
//@ Valeur de retour :
//@               XMLFormatter
//@
//@ ------------------------------------------------------------------*/
XMLFormatter& operator<<(XMLFormatter& strm, const DOMString& s)
{
  unsigned int lent = s.length();

  if (lent <= 0)
    {
      return strm;
    }

  XMLCh* buf = new XMLCh[lent + 1];
  XMLString::copyNString(buf, s.rawBuffer(), lent);
  buf[lent] = 0;
  strm << buf;
  delete [] buf;
  
  return strm;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ostream << DOM_Node
//@
//@ Description : Stream out a DOM node, and, recursively, all of its
//@               children
//@
//@ Parametres :
//@               (ENTREE) target
//@               (ENTREE) toWrite
//@
//@ Valeur de retour :
//@               ostream
//@
//@ ------------------------------------------------------------------*/
ostream& operator<<(ostream& target, DOM_Node& toWrite)
{
  // Get the name and value out for convenience
  DOMString nodeName  = toWrite.getNodeName();
  DOMString nodeValue = toWrite.getNodeValue();


  //@ SELON le type du noeud
  switch (toWrite.getNodeType())
    {
    case DOM_Node::TEXT_NODE :
      {
        S_Formatter->formatBuf(nodeValue.rawBuffer(), 
                               nodeValue.length(), 
                               XMLFormatter::CharEscapes);
        break;
      }

    case DOM_Node::PROCESSING_INSTRUCTION_NODE :
      {
        *S_Formatter << XMLFormatter::NoEscapes << S_StartPI 
                     << nodeName;
        if (nodeValue.length() > 0)
          {
            *S_Formatter << chSpace << nodeValue;
          }
        *S_Formatter << XMLFormatter::NoEscapes << S_EndPI;
        break;
      }

    case DOM_Node::DOCUMENT_NODE :
      { 
        DOM_Node child = toWrite.getFirstChild();
        while( child != 0)
          {
            target << child;
          
            // Add linefeed in requested output encoding
            *S_Formatter << chLF;
            target << flush;
            child = child.getNextSibling();
          }
        break;
      }

    case DOM_Node::ELEMENT_NODE :
      {
        if (S_LineFeed && S_LineFeedBegin)
          {
            *S_Formatter << chLF;
          }
        S_LineFeedBegin = true;

        // The name has to be representable without any escapes
        // Output the element start tag.
        *S_Formatter << XMLFormatter::NoEscapes
                     << chOpenAngle << nodeName;

        // Output any attributes on this element
        DOM_NamedNodeMap attributes = toWrite.getAttributes();
        int attrCount = attributes.getLength();
        for (int i = 0; i < attrCount; i++)
          {
            DOM_Node  attribute = attributes.item(i);

            // Again the name has to be completely representable. But
            // the attribute can have refs and requires the attribute
            // style escaping.
            *S_Formatter << XMLFormatter::NoEscapes
                         << chSpace << attribute.getNodeName()
                         << chEqual << chDoubleQuote
                         << XMLFormatter::AttrEscapes
                         << attribute.getNodeValue()
                         << XMLFormatter::NoEscapes
                         << chDoubleQuote;
          }

        // Test for the presence of children, which includes both
        // text content and nested elements.
        DOM_Node child = toWrite.getFirstChild();
        if (child != 0)
          {
            // There are children. Close start-tag, and output children.
            // No escapes are legal here
            *S_Formatter << XMLFormatter::NoEscapes << chCloseAngle;
            while( child != 0)
              {
                target << child;
                child = child.getNextSibling();
              }

            // Done with children.
            // Output the end tag.
            *S_Formatter << XMLFormatter::NoEscapes << S_EndElement
                         << nodeName << chCloseAngle;
          }
        else
          {
            // There were no children. Output the short form close of
            // the element start tag, making it an empty-element tag.
            *S_Formatter << XMLFormatter::NoEscapes 
                         << chForwardSlash << chCloseAngle;
          }
        break;
      }

    case DOM_Node::ENTITY_REFERENCE_NODE :
      {
        DOM_Node child;
        
        // Instead of printing the reference tree
        // we'd output the actual text as it appeared in the xml file.
        // This would be the case when -e option was chosen
        *S_Formatter << XMLFormatter::NoEscapes << chAmpersand
                     << nodeName << chSemiColon;
        break;
      }

    case DOM_Node::CDATA_SECTION_NODE :
      {
        *S_Formatter << XMLFormatter::NoEscapes << S_StartCDATA
                     << nodeValue << S_EndCDATA;
        break;
      }

    case DOM_Node::COMMENT_NODE :
      {
        *S_Formatter << XMLFormatter::NoEscapes << S_StartComment
                     << nodeValue << S_EndComment;
        break;
      }

    case DOM_Node::DOCUMENT_TYPE_NODE :
      {
        PrintDocumentTypeNode(nodeName, toWrite);
        break;
      }

    case DOM_Node::ENTITY_NODE :
      {
        PrintEntityNode(nodeName, toWrite);
        break;
      }

    case DOM_Node::XML_DECL_NODE :
      {
        DOMString  str;

        *S_Formatter << S_XMLDecl1 
                     << ((DOM_XMLDecl &)toWrite).getVersion()
                     << S_XMLDecl2 << S_EncodingName;

        str = ((DOM_XMLDecl &)toWrite).getStandalone();
        if (str != 0)
          {
            *S_Formatter << S_XMLDecl3 << str;
          }

        *S_Formatter << S_XMLDecl4;
        break;
      }

    default :
      break;
    }
  //@ FIN SELON

  return target;
}


//@ ------------------------------------------------------------------*/
//@ Methode : PrintDocumentTypeNode
//@
//@ Description : Stream out a DocumentType node
//@
//@ Parametres :
//@               (ENTREE) nodeName
//@               (ENTREE) toWrite
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void PrintDocumentTypeNode(DOMString nodeName, DOM_Node& toWrite)
{
  DOM_DocumentType doctype = (DOM_DocumentType &)toWrite;
  
  *S_Formatter << XMLFormatter::NoEscapes << S_StartDoctype
               << nodeName;
  
  DOMString id = doctype.getPublicId();
  if (id != 0)
    {
      *S_Formatter << XMLFormatter::NoEscapes << chSpace 
                   << S_Public << id << chDoubleQuote;
      id = doctype.getSystemId();
      if (id != 0)
        {
          *S_Formatter << XMLFormatter::NoEscapes << chSpace
                       << chDoubleQuote << id << chDoubleQuote;
        }
    }
  else
    {
      id = doctype.getSystemId();
      if (id != 0)
        {
          *S_Formatter << XMLFormatter::NoEscapes 
                       << chSpace << S_System
                       << id << chDoubleQuote;
        }
    }
  
  id = doctype.getInternalSubset();
  if (id !=0)
    {
      *S_Formatter << XMLFormatter::NoEscapes << chOpenSquare
                   << id << chCloseSquare;
    }
  
  *S_Formatter << XMLFormatter::NoEscapes << chCloseAngle;
}


//@ ------------------------------------------------------------------*/
//@ Methode : PrintEntityNode
//@
//@ Description : Stream out a Entity node
//@
//@ Parametres :
//@               (ENTREE) nodeName
//@               (ENTREE) toWrite
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void PrintEntityNode(DOMString nodeName, DOM_Node& toWrite)
{
  *S_Formatter << XMLFormatter::NoEscapes << S_StartEntity
               << nodeName;

  DOMString id = ((DOM_Entity &)toWrite).getPublicId();
  if (id != 0)
    {
      *S_Formatter << XMLFormatter::NoEscapes << S_Public
                   << id << chDoubleQuote;
    }
  
  id = ((DOM_Entity &)toWrite).getSystemId();
  if (id != 0)
    {
      *S_Formatter << XMLFormatter::NoEscapes << S_System
                   << id << chDoubleQuote;
    }
  
  id = ((DOM_Entity &)toWrite).getNotationName();
  if (id != 0)
    {
      *S_Formatter << XMLFormatter::NoEscapes << S_Notation
                   << id << chDoubleQuote;
    }
  
  *S_Formatter << XMLFormatter::NoEscapes << chCloseAngle << chLF;
}


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CMN_PRS_Parser
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_PRS_Parser::CMN_PRS_Parser() throw (ERR__Exception)
  : m_ErrReporter(NULL), m_RedirectHandler(NULL), m_Parser(NULL),
    m_DoNamespaces(false), m_DoSchema(false), 
    m_SchemaFullChecking(false), m_DoCreate(false), 
    m_UnRepFlags(XMLFormatter::UnRep_CharRef), 
    m_ValScheme(DOMParser::Val_Auto) 
{
/* CR290 */
  CGList="";
/* Fin CR290 */

  // FT-2512
  char *str;
  // ..FT-2512

  //@ En captant les exceptions
  try
    {
      //@ Initialise le modele XML de type XML4C2
      XMLPlatformUtils::Initialize();
    }
  
  //@ Traiter les exceptions
  catch(const XMLException& toCatch)
    {
      ERR__Exception exception;
      
#ifdef DEBUG
      cerr << STR_ERROR_INIT << endl << STR_XML_EXCEPTION_MSG
           << DOMString(toCatch.getMessage()) << endl;
#endif

      //@ Generation d'une exception INIT_FAILED
      exception.SetErrorCode(ERR__Exception::INIT_FAILED);
      exception.SetParametre(STR_ERROR_INIT);
      // FT-2512
      str =DOMString(toCatch.getMessage()).transcode(); 
      exception.SetParametre(str);
      delete [] str;
      // ..FT-2512
	  THROW(exception)
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~CMN_PRS_Parser
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_PRS_Parser::~CMN_PRS_Parser()
{
  //@ Nettoyage
  if (m_RedirectHandler)
    {
      delete m_RedirectHandler;
    }
  if (m_ErrReporter)
    {
      delete m_ErrReporter;
    }
  if (m_Parser)
    {
      delete m_Parser;
    }


  //@ Appel de la methode Terminate
  XMLPlatformUtils::Terminate();
}


//@ ------------------------------------------------------------------*/
//@ Methode : UnRepFlags
//@
//@ Description : Met a jour la prise en compte des caracteres non 
//@               visibles
//@
//@ Parametres :
//@               (ENTREE) flag : Mode de gestion a prendre en compte
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_PRS_Parser::UnRepFlags(const char* flag)
{
  if (!strcmp(flag, UNREPFLAG_FAIL.c_str()))
    {
      m_UnRepFlags = XMLFormatter::UnRep_Fail;
    }
  else if (!strcmp(flag, UNREPFLAG_REP.c_str()))
    {
      m_UnRepFlags = XMLFormatter::UnRep_Replace;
    }
  else
    {
      m_UnRepFlags = XMLFormatter::UnRep_CharRef;
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : ValScheme
//@
//@ Description : Met a jour le schema de validation
//@
//@ Parametres :
//@               (ENTREE) scheme : Mode de validation a prendre en
//@                                 compte
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_PRS_Parser::ValScheme(const char* scheme)
{
  if (!strcmp(scheme, VALSCHEME_NEVER.c_str()))
    {
      m_ValScheme = DOMParser::Val_Never;
    }
  else if (!strcmp(scheme, VALSCHEME_ALWAYS.c_str()))
    {
      m_ValScheme = DOMParser::Val_Always;
    }
  else
    {
      m_ValScheme = DOMParser::Val_Auto;
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : ParseFile
//@
//@ Description : Initialise un parser, valide et parse un fichier XML
//@
//@ Parametres :
//@               (ENTREE) xmlFile : Nom du fichier XML lu
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_PRS_Parser::ParseFile(CMN__Chaine xmlFile) throw (ERR__Exception)
{
  // FT-2512
  char *str;
  // ..FT-2512
  
  //@ Creer le parser
  m_Parser = new DOMParser;
  m_Parser->setValidationScheme(m_ValScheme);
  m_Parser->setDoNamespaces(m_DoNamespaces);
  m_Parser->setDoSchema(m_DoSchema);
  m_Parser->setValidationSchemaFullChecking(m_SchemaFullChecking);
  m_Parser->setCreateEntityReferenceNodes(m_DoCreate);
  m_Parser->setToCreateXMLDeclTypeNode(true);

  //@ Lui associer un gestionnaire d'erreur
  m_ErrReporter = new CMN_PRS_ErrorReporter();
  m_Parser->setErrorHandler(m_ErrReporter);

  //@ Lui associer un gestionnaire pour la redirection de la DTD
  m_RedirectHandler = new CMN_PRS_RedirectHandler();
  m_Parser->setEntityResolver(m_RedirectHandler);
 
 
  //@ En captant les exceptions
  try
    {
     //@ Parse le fichier XML, en captant toutes les exceptions
      m_Parser->parse(xmlFile.c_str());
    }

  
  //@ Traiter les exceptions de type XMLException
  catch (const XMLException& toCatch)
    {
      ERR__Exception exception;

#ifdef DEBUG
      cerr << STR_ERROR_PARSING << xmlFile << endl 
           << STR_XML_EXCEPTION_MSG
           << DOMString(toCatch.getMessage()) << endl;
#endif
      
      //@ Generation d'une exception PARSING_FAILED
      exception.SetErrorCode(ERR__Exception::PARSING_FAILED);
      exception.SetParametre(STR_ERROR_PARSING+xmlFile);

      // ..FT-2512
      str =DOMString(toCatch.getMessage()).transcode(); 
      exception.SetParametre(str);
      if (str) delete [] str;
      // ..FT-2512
      THROW(exception)
    }

  //@ Traiter les exceptions de type SAXParseException
  catch (const SAXParseException& toCatch)
    {
      ERR__Exception exception;
      
#ifdef DEBUG
      cerr << STR_ERROR_PARSING << xmlFile << endl 
           << STR_XML_EXCEPTION_MSG
           << DOMString(toCatch.getMessage()) << endl;
#endif
      
      //@ Generation d'une exception PARSING_FAILED
      exception.SetErrorCode(ERR__Exception::PARSING_FAILED);
      exception.SetParametre(STR_ERROR_PARSING+xmlFile);
      // FT-2512
      str =DOMString(toCatch.getMessage()).transcode(); 
      exception.SetParametre(str);
      if (str) delete [] str;
      // ..FT-2512
      THROW(exception)
    }

  //@ Traiter les exceptions de type DOMException
  catch (const DOM_DOMException& toCatch)
    {
      ERR__Exception exception;

#ifdef DEBUG
      cerr << STR_ERROR_PARSING << xmlFile << endl 
           << STR_DOM_EXCEPTION_MSG
           << DOM_EXCEPTION_TYPE_STR[toCatch.code] << endl;
#endif
      
      //@ Generation d'une exception PARSING_FAILED
      exception.SetErrorCode(ERR__Exception::PARSING_FAILED);
      exception.SetParametre(STR_ERROR_PARSING+xmlFile);
      exception.SetParametre(DOM_EXCEPTION_TYPE_STR[toCatch.code]); 
      THROW(exception)
    }

  //@ Traiter les exceptions de type exception
  catch (exception& e)
    {
      ERR__Exception exception;
      
#ifdef DEBUG
      cerr << STR_ERROR_PARSING<< xmlFile << endl
           << e.what() << endl;
#endif
      
      //@ Generation d'une exception PARSING_FAILED
      exception.SetErrorCode(ERR__Exception::PARSING_FAILED);
      exception.SetParametre(STR_ERROR_PARSING+xmlFile);
      exception.SetParametre(e.what());
      THROW(exception)
    }

  //@ Traiter les autres exceptions
  catch (...)
    {
      ERR__Exception exception;

#ifdef DEBUG
      cerr << STR_ERROR_PARSING << xmlFile << endl;
#endif

      //@ Generation d'une exception PARSING_FAILED
      exception.SetErrorCode(ERR__Exception::PARSING_FAILED);
      exception.SetParametre(STR_ERROR_PARSING+xmlFile);
      THROW(exception)
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : ParseWorkOrder
//@
//@ Description : Parse un arbre de type WorkOrder et construit un
//@               objet Work Order
//@
//@ Parametres :
//@               (SORTIE) workOrder
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK si l'objet a ete construit 
//@               CMN__STATUT_NOK sinon
//@
//@ ------------------------------------------------------------------*/
void CMN_PRS_Parser::ParseWorkOrder(CMN_DAT_WorkOrder& workOrder)
{
  // Exception
  ERR__Exception except;
  // FT-2512
  char *str;
  // ..FT-2512

  //@ SI un arbre n'est pas charge en memoire
  if ((m_ErrReporter == NULL) || (m_Parser == NULL))
    {
      //@ Generation d'une exception INVALID_TREE
      except.SetErrorCode(ERR__Exception::INVALID_TREE);
      except.SetParametre(STR_ERROR_INVALID_TREE);
      THROW(except)
    }
  //@ FIN SI un arbre n'est pas charge en memoire

  //@ SI il y a eu des erreurs precedemment
  if (m_ErrReporter->getSawErrors())
    {
      //@ Generation d'une exception INVALID_TREE
      except.SetErrorCode(ERR__Exception::INVALID_TREE);
      except.SetParametre(STR_ERROR_INVALID_TREE);
      THROW(except)
    }
   //@ FIN SI il y a eu des erreurs precedemment
  
  //@ En captant les exceptions
  try
    {
      //@ Recuperer le document XML
      DOM_Document docWorkOrder  = m_Parser->getDocument();
      DOM_Node     rootWorkOrder = docWorkOrder.getDocumentElement();
      rootWorkOrder.normalize();
      DOMString    typeArbre     = rootWorkOrder.getNodeName();
      
      //@ Verifier que le document parse est bien du type
      //@ WorkOrder
      //@ SI l'arbre parse n'est pas de type WorkOrder
      if (! typeArbre.equals(ELMT_PPF_WORK_ORDER.c_str()))
        {
          //@ Generation d'une exception INVALID_TREE
          except.SetErrorCode(ERR__Exception::INVALID_TREE);
          except.SetParametre(STR_ERROR_INVALID_TREE);
          THROW(except)
        }
      //@ FIN SI l'arbre parse n'est pas de type WorkOrder

      DOM_Node noeudRacine = m_Parser->getDocument();
      
      //@ Parser recursivement l'arbre a partir de la racine
      ParseNode(noeudRacine, 
                ELMT_PPF_WORK_ORDER.c_str(),
                workOrder);
    }
  
  //@ Traiter les exceptions de type XMLException
  catch (const XMLException& toCatch)
    {
      
#ifdef DEBUG
      cerr << STR_ERROR_PARSING_WO << endl 
           << STR_XML_EXCEPTION_MSG
           << DOMString(toCatch.getMessage()) << endl;
#endif
      
      //@ Generation d'une exception PARSING_WO_FAILED
      except.SetErrorCode(ERR__Exception::PARSING_WO_FAILED);
      except.SetParametre(STR_ERROR_PARSING_WO);
      // FT-2512
      str =DOMString(toCatch.getMessage()).transcode(); 
      except.SetParametre(str);
      if (str)	delete [] str;
      // ..FT-2512
      THROW(except)
    }
  
  //@ Traiter les exceptions de type SAXParseException
  catch (const SAXParseException& toCatch)
    {
      
#ifdef DEBUG
      cerr << STR_ERROR_PARSING_WO << endl 
           << STR_XML_EXCEPTION_MSG
           << DOMString(toCatch.getMessage()) << endl;
#endif
      
      //@ Generation d'une exception PARSING_WO_FAILED
      except.SetErrorCode(ERR__Exception::PARSING_WO_FAILED);
      except.SetParametre(STR_ERROR_PARSING_WO);
      // FT-2512
      str =DOMString(toCatch.getMessage()).transcode(); 
      except.SetParametre(str);
      if (str) delete [] str;
      // ..FT-2512
      THROW(except)
    }

  //@ Traiter les exceptions de type DOMException
  catch (const DOM_DOMException& toCatch)
    {
      
#ifdef DEBUG
      cerr << STR_ERROR_PARSING_WO << endl 
           << STR_DOM_EXCEPTION_MSG
           << DOM_EXCEPTION_TYPE_STR[toCatch.code] << endl;
#endif
              
      //@ Generation d'une exception PARSING_WO_FAILED
      except.SetErrorCode(ERR__Exception::PARSING_WO_FAILED);
      except.SetParametre(STR_ERROR_PARSING_WO);
      except.SetParametre(DOM_EXCEPTION_TYPE_STR[toCatch.code]);
      THROW(except)
    }
  
  //@ Traiter les exceptions de type exception
  catch (exception& e)
    {
      
#ifdef DEBUG
      cerr << STR_ERROR_PARSING_WO << e.what() << endl;
#endif
      
      //@ Generation d'une exception PARSING_FAILED
      except.SetErrorCode(ERR__Exception::PARSING_WO_FAILED);
      except.SetParametre(STR_ERROR_PARSING_WO);
      except.SetParametre(e.what());
      THROW(except)
    }
  
  //@ Traiter les autres exceptions
  catch (...)
    {
              
#ifdef DEBUG
      cerr << STR_ERROR_PARSING_WO << endl;
#endif
              
      //@ Generation d'une exception PARSING_FAILED
      except.SetErrorCode(ERR__Exception::PARSING_WO_FAILED);
      except.SetParametre(STR_ERROR_PARSING_WO);
      THROW(except)
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : ParseOBTUTC
//@
//@ Description : Parse un arbre de type OBTUTC et construit un
//@               objet OBTUTC
//@
//@ Parametres :
//@               (SORTIE) OBTUTC
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK si l'objet a ete construit 
//@               CMN__STATUT_NOK sinon
//@
//@ ------------------------------------------------------------------*/
void CMN_PRS_Parser::ParseOBTUTC(SD_DON_OBTUTC& OBTUTC)
  throw (ERR__Exception)
{
  // Exception
  ERR__Exception exception;
  // FT-2512
  char *str;
  // ..FT-2512

  //@ SI un arbre n'est pas charge en memoire
  if ((m_ErrReporter == NULL) || (m_Parser == NULL))
    {
      //@ Generation d'une exception INVALID_TREE
      exception.SetErrorCode(ERR__Exception::INVALID_TREE);
      exception.SetParametre(STR_ERROR_INVALID_TREE);
      THROW(exception)
    }
  //@ SI un arbre n'est pas charge en memoire

  //@ SI il y a eu des erreurs precedemment
  if (m_ErrReporter->getSawErrors())
    {
      //@ Generation d'une exception INVALID_TREE
      exception.SetErrorCode(ERR__Exception::INVALID_TREE);
      exception.SetParametre(STR_ERROR_INVALID_TREE);
      THROW(exception)
    }
  //@ FIN SI il y a eu des erreurs precedemment

  //@ En captant les exceptions
  try
    {
      //@ Recuperer le document XML
      DOM_Document docOBTUTC  = m_Parser->getDocument();
      DOM_Node     rootOBTUTC = docOBTUTC.getDocumentElement();
      rootOBTUTC.normalize();
      DOMString    typeArbre  = rootOBTUTC.getNodeName();
      
      //@ Verifier que le document parse est bien du type OBTUTC
      //@ SI l'arbre parse n'est pas de type OBTUTC
      if (! typeArbre.equals(ELMT_TYPE_ARBRE_OBTUTC.c_str()))
        {
          //@ Generation d'une exception INVALID_TREE
          exception.SetErrorCode(ERR__Exception::INVALID_TREE);
          exception.SetParametre(STR_ERROR_INVALID_TREE);
          THROW(exception)
        }
      //@ FIN SI l'arbre parse n'est pas de type OBTUTC

      DOM_Node noeudRacine = m_Parser->getDocument();
          
      //@ Parser recursivement l'arbre a partir de la racine
      ParseNode(noeudRacine, ELMT_TYPE_ARBRE_OBTUTC.c_str(), OBTUTC);
    }
  

  //@ Traiter les exceptions
  catch (const XMLException& toCatch)
    {
      
#ifdef DEBUG
      cerr << STR_ERROR_PARSING_OBT << endl << STR_XML_EXCEPTION_MSG
           << DOMString(toCatch.getMessage()) << endl;
#endif
      
      //@ Generation d'une exception PARSING_OBT_FAILED
      exception.SetErrorCode(ERR__Exception::PARSING_OBT_FAILED);
      exception.SetParametre(STR_ERROR_PARSING_OBT);
      // ..FT-2512
      str =DOMString(toCatch.getMessage()).transcode(); 
      exception.SetParametre(str);
      if (str) delete [] str;
      // ..FT-2512
      THROW(exception)
    }
  
  catch (const DOM_DOMException& toCatch)
    {
      
#ifdef DEBUG
      cerr << STR_ERROR_PARSING_OBT << endl << STR_XML_EXCEPTION_MSG
           << DOM_EXCEPTION_TYPE_STR[toCatch.code] << endl;
#endif
              
      //@ Generation d'une exception PARSING_OBT_FAILED
      exception.SetErrorCode(ERR__Exception::PARSING_OBT_FAILED);
      exception.SetParametre(STR_ERROR_PARSING_OBT);
      exception.SetParametre(DOM_EXCEPTION_TYPE_STR[toCatch.code]);
      THROW(exception)
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : ParseOSV
//@
//@ Description : Parse un arbre de type OSV et renvoie le dernier FLAG
//@               de l'orbite juste anterieure a la date UTC
//@
//@ Parametres :
//@               (ENTREE) dateDebutUTC : date UTC (debut de ligne)
//@               (ENTREE) dateFinUTC : date UTC (fin de ligne)
//@               (SORTIE) flag    : flag lu dans <FLAG>
//@               (SORTIE) flagManoeuvre : vrai si une manoeuvre est 
//@                                  effectue sur la ligne sinon false
//@
//@ Valeur de retour : aucune
//@
//@ ------------------------------------------------------------------*/
// FT-2364
void CMN_PRS_Parser::ParseOSV(t_Date& dateDebutUTC, t_Date& dateFinUTC, t_Chaine& flag, bool &flagManoeuvre) 
		throw (ERR__Exception)
// ...FT-2364
{
/* FA 172 */
  t_Chaine flagCourant;
  t_Chaine oldFlag;
  bool initFaite=false;
/* ..FA 172 */

    // FT-2364
    bool presenceDateDebut;
    bool presenceDateFin;
    t_Chaine flagDateDebut;
    t_Chaine flagDateFin;

    // mode normal par defaut
    flag = FLAG_ANX;
    oldFlag = FLAG_ANX;
    presenceDateDebut = false;
    presenceDateFin = false;
    // pas de manoeuvre par defaut
    flagManoeuvre = false;
    // ...FT-2364
 
    // Exception
    ERR__Exception exception;
    // FT-2512
    char *str;
    // ..FT-2512

  //@ SI un arbre n'est pas charge en memoire
  if ((m_ErrReporter == NULL) || (m_Parser == NULL))
    {
      //@ Generation d'une exception INVALID_TREE
      exception.SetErrorCode(ERR__Exception::INVALID_TREE);
      exception.SetParametre(STR_ERROR_INVALID_TREE);
      THROW(exception)
    }
  //@ FIN SI un arbre n'est pas charge en memoire

  //@ SI il y a eu des erreurs precedemment
  if (m_ErrReporter->getSawErrors())
    {
      //@ Generation d'une exception INVALID_TREE
      exception.SetErrorCode(ERR__Exception::INVALID_TREE);
      exception.SetParametre(STR_ERROR_INVALID_TREE);
      THROW(exception)
    }
  //@ FIN SI il y a eu des erreurs precedemment


  //@ En captant les exceptions
  try
    {
      DOM_NodeList listeNoeuds;
      DOM_NodeList listeNoeudsFlag;
      DOM_NodeList listeNoeudsTime;
      t_Chaine     chaine; // chaine utilisee pour stocker la date
      t_Date       date;   // date courante de l'element ORBIT_START

      //@ Recuperer le document XML
      DOM_Document docOSV    = m_Parser->getDocument();
      DOM_Node     rootOSV   = docOSV.getDocumentElement();
      rootOSV.normalize();
      DOMString    typeArbre = rootOSV.getNodeName();
      
      //@ Verifier que le document parse est bien du type OSV
      //@ SI l'arbre parse n'est pas de type OSV
      if (! typeArbre.equals(ELMT_TYPE_ARBRE_OSV.c_str()))
        {
          //@ Generation d'une exception INVALID_TREE
          exception.SetErrorCode(ERR__Exception::INVALID_TREE);
          exception.SetParametre(STR_ERROR_INVALID_TREE);
          THROW(exception)
        }

      //@ Recuperation de la liste des noeuds <ORBIT_START>
      //@ du fichier OSV
      listeNoeuds = docOSV.getElementsByTagName(CMN_PRS_Parser::ELMT_OSV_ORBIT_START.c_str());

      // FT-2364
      //@ POUR chaque noeud <ORBIT_START> de la liste
      int i;
      for (i=0; (i < listeNoeuds.getLength()) && (presenceDateFin == false); i++) 
      {
          //@ Recuperation de la valeur situee dans <ELMT_OSV_TIME>
          listeNoeudsTime = docOSV.getElementsByTagName(CMN_PRS_Parser::ELMT_OSV_TIME.c_str());
			
          //@ Recuperer la valeur du noeud (a convertir en date)
          str = listeNoeudsTime.item(i).getFirstChild().getNodeValue().transcode();
          chaine = str;
          if (str) 
          {
              delete [] str;	
          }
          date.ChaineCCSDS(chaine);

          //@ Recuperation de la valeur situee dans <FLAG>
          listeNoeudsFlag = docOSV.getElementsByTagName(CMN_PRS_Parser::ELMT_OSV_FLAG.c_str());

          //@ Recuperer la valeur du noeud (chaine de caracteres)
          str = listeNoeudsFlag.item(i).getFirstChild().getNodeValue().transcode();
          flagCourant = str;
          if (str) 
          {
              delete [] str;	
          }
	      flagCourant.Upper();
#ifndef DEBUG
          cout << "OSV \tdate                    : " << date.Chaine() << endl;
          cout << "OSV \tflag                    : " << flagCourant << endl;
          cout << "OSV Element: " << i+1 << endl;
          cout << "OSV \tchaine recuperee: " << chaine << endl;
          cout << "OSV \tdate issue      : " << date.Chaine() << endl;
#endif
          if (presenceDateDebut == false)
          {
              if ((date > dateDebutUTC) && (date > dateFinUTC))
              {
                  // aucun flag OSV correspondant a la ligne 
                  presenceDateFin = true;
                  flag = oldFlag; 
                  if ((flag == FLAG_IP_START) || (flag == FLAG_OOP_START))
                  {
                     flagManoeuvre = true;
                  }
              }
              else if (date > dateDebutUTC)
              {
                  presenceDateDebut = true;
                  flagDateDebut = oldFlag;
              }            
              else if (date  == dateDebutUTC)
              { 
                  presenceDateDebut = true;
                  flagDateDebut = flagCourant;
              }
              if ((flagDateDebut == FLAG_IP_START) || (flagDateDebut == FLAG_OOP_START))
              {
                 flagManoeuvre = true;
              }
          }
          if ((presenceDateDebut == true) && (presenceDateFin == false))
          {
              if (date > dateFinUTC)
              {
                  // apres la date de fin de ligne
                  presenceDateFin = true;
                  flagDateFin = oldFlag;
              }            
              else 
              {
                  flagDateFin = flagCourant;
              } 
              if ((flagDateFin == FLAG_IP_START) || (flagDateFin == FLAG_OOP_START) || \
                  (flagDateFin == FLAG_IP_END)   || (flagDateFin == FLAG_OOP_END)) {
                      flagManoeuvre = true;   
              }
          }
          oldFlag = flagCourant;
      } 
      if (presenceDateDebut == true)
	  { 
           if (presenceDateFin == false)
           {  
               if ((i == listeNoeuds.getLength()) && (date <= dateFinUTC))
               {
                   flag = flagDateFin;
               }
               else
               {
                   flag = flagDateDebut;
               }
           }
           else
           {
               flag = flagDateFin;
           }       
      }
      if ((flag == FLAG_IP_END) || (flag == FLAG_OOP_END))
      {
         flag = FLAG_ANX;
      } 
#ifdef DEBUG
        cout << "OSV \tdate debut ligne        : " << dateDebutUTC.Chaine() << endl;
        cout << "OSV \tdate fin   ligne        : " << dateFinUTC.Chaine() << endl;
        cout << "OSV \tflag                    : " << flag << endl;
        cout << "OSV \tflag manoeuvre          : " << flagManoeuvre << endl;
#endif
      // ...FT-2364
      //@ FIN POUR chaque noeud <ORBIT_START> de la liste
    }
  
  //@ Traiter les exceptions
  catch (const XMLException& toCatch)
    {
      
#ifdef DEBUG
      cerr << STR_ERROR_PARSING_OSV << endl << STR_XML_EXCEPTION_MSG
           << DOMString(toCatch.getMessage()) << endl;
#endif
    
      //@ Generation d'une exception PARSING_OSV_FAILED
      exception.SetErrorCode(ERR__Exception::PARSING_OSV_FAILED);
      exception.SetParametre(STR_ERROR_PARSING_OSV);
      // FT-2512
      str = DOMString(toCatch.getMessage()).transcode();
      exception.SetParametre(str);
      if (str) delete [] str;
      // ..FT-2512
      THROW(exception)
    }

  catch (const DOM_DOMException& toCatch)
    {
      
#ifdef DEBUG
      cerr << STR_ERROR_PARSING_OSV << endl << STR_XML_EXCEPTION_MSG
           << DOM_EXCEPTION_TYPE_STR[toCatch.code] << endl;
#endif
    
      //@ Generation d'une exception PARSING_OSV_FAILED
      exception.SetErrorCode(ERR__Exception::PARSING_OSV_FAILED);
      exception.SetParametre(STR_ERROR_PARSING_OSV);
      exception.SetParametre(DOM_EXCEPTION_TYPE_STR[toCatch.code]);
      THROW(exception)
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : EcrireReport
//@
//@ Description : Ecrit un fichier report XML a partir d'un arbre XML
//@               de type Work order et d'une structure de donnees
//@               contenant les informations liees aux traitements
//@
//@ Parametres :
//@               (ENTREE) xmlFile : Nom du fichier report XML a creer 
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_PRS_Parser::EcrireReport(t_Chaine xmlFile, 
                                  t_ListeStageInfo listeStageInfo)
{
  // Exception
  ERR__Exception exception;
  // FT-2512
  char *str;
  // ..FT-2512

  //@ SI un arbre n'est pas charge en memoire
  if ((m_ErrReporter == NULL) || (m_Parser == NULL))
    {
      //@ Generation d'une exception INVALID_TREE
      exception.SetErrorCode(ERR__Exception::INVALID_TREE);
      exception.SetParametre(STR_ERROR_INVALID_TREE);
      THROW(exception)
    }
  //@ FIN SI un arbre n'est pas charge en memoire

  //@ SI il y a eu des erreurs precedemment
  if (m_ErrReporter->getSawErrors())
    {
      //@ Generation d'une exception INVALID_TREE
      exception.SetErrorCode(ERR__Exception::INVALID_TREE);
      exception.SetParametre(STR_ERROR_INVALID_TREE);
      THROW(exception)
    }
  //@ FIN SI il y a eu des erreurs precedemment


  //@ En captant les exceptions
  try
    {
      //@ Verifier que le document parse est bien du type
      //@ WorkOrder
      DOM_Document docWorkOrder  = m_Parser->getDocument();
      DOMString    typeArbre     = docWorkOrder.
        getDocumentElement().getNodeName();
              

      //@ SI l'arbre parse n'est pas de type WorkOrder
      if (! typeArbre.equals(ELMT_PPF_WORK_ORDER.c_str()))
        {
          //@ Generation d'une exception INVALID_TREE
          exception.SetErrorCode(ERR__Exception::INVALID_TREE);
          exception.SetParametre(STR_ERROR_INVALID_TREE);
          THROW(exception)
        }
      //@ FIN SI l'arbre parse n'est pas de type WorkOrder

      DOM_Document docReport;
          
      //@
      //@ 1. Construction de l'arbre XML
      //@
      //@ Creation du document
      CreerArbreReport(docWorkOrder, 
                       listeStageInfo, 
                       docReport);
      
      //@
      //@ 2. Ecriture de l'arbre XML dans le fichier
      //@
      docReport.normalize();
      S_LineFeed = true;
      DumpTree(xmlFile, docReport);
      S_LineFeed = false;
    }
  
  //@ Traiter les exceptions de type XMLException
  catch (const XMLException& toCatch)
    {
      
#ifdef DEBUG
      cerr << STR_ERROR_REPORTING << endl 
           << STR_XML_EXCEPTION_MSG
           << DOMString(toCatch.getMessage()) << endl;
#endif
      
      //@ Generation d'une exception REPORTING_FAILED
      exception.SetErrorCode(ERR__Exception::REPORTING_FAILED);
      exception.SetParametre(STR_ERROR_REPORTING+xmlFile);
      // FT-2512
      str = DOMString(toCatch.getMessage()).transcode();
      exception.SetParametre(str);
      if (str) delete [] str;
      // ..FT-2512
      S_LineFeed = false;
      THROW(exception)
    }

  //@ Traiter les exceptions de type SAXParseException
  catch (const SAXParseException& toCatch)
    {
              
#ifdef DEBUG
      cerr << STR_ERROR_REPORTING << endl 
           << STR_XML_EXCEPTION_MSG
           << DOMString(toCatch.getMessage()) << endl;
#endif
      
      //@ Generation d'une exception REPORTING_FAILED
      exception.SetErrorCode(ERR__Exception::REPORTING_FAILED);
      exception.SetParametre(STR_ERROR_REPORTING+xmlFile);
      // FT-2512
      str = DOMString(toCatch.getMessage()).transcode();
      exception.SetParametre(str);
      if (str) delete [] str;
      // ..FT-2512
      S_LineFeed = false;
      THROW(exception)
    }
  
  //@ Traiter les exceptions de type DOMException
  catch (const DOM_DOMException& toCatch)
    {
      
#ifdef DEBUG
      cerr << STR_ERROR_REPORTING << endl 
           << STR_DOM_EXCEPTION_MSG
           << DOM_EXCEPTION_TYPE_STR[toCatch.code] << endl;
#endif
      
      //@ Generation d'une exception REPORTING_FAILED
      exception.SetErrorCode(ERR__Exception::REPORTING_FAILED);
      exception.SetParametre(STR_ERROR_REPORTING+xmlFile);
      exception.SetParametre(DOM_EXCEPTION_TYPE_STR[toCatch.code]);
      S_LineFeed = false;
      THROW(exception)
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : DumpTree
//@
//@ Description : Ecrit un fichier XML a partir de l'arbre XML charge
//@               en memoire
//@
//@ Parametres :
//@               (ENTREE) xmlFile : Nom du fichier XML a creer
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_PRS_Parser::DumpTree(t_Chaine xmlFile)
{
  // Exception
  ERR__Exception exception;

  //@ SI un arbre n'est pas charge en memoire
  if ((m_ErrReporter == NULL) || (m_Parser == NULL))
    {

      //@ Generation d'une exception INVALID_TREE
      exception.SetErrorCode(ERR__Exception::INVALID_TREE);
      exception.SetParametre(STR_ERROR_INVALID_TREE);
      THROW(exception)
    }
  //@ FIN SI un arbre n'est pas charge en memoire

  //@ SI il y a eu des erreurs precedemment
  if (m_ErrReporter->getSawErrors())
    {
      ERR__Exception exception;
      
      //@ Generation d'une exception INVALID_TREE
      exception.SetErrorCode(ERR__Exception::INVALID_TREE);
      exception.SetParametre(STR_ERROR_INVALID_TREE);
      THROW(exception)
    }
  //@ FIN SI il y a eu des erreurs precedemment

  //@ Appel de DumpTree(fichier, document) sur l'arbre charge
  //@ en memoire
  DOM_Document doc = m_Parser->getDocument();
  DumpTree(xmlFile, doc);
}


//======================================================================
//                                      METHODES PRIVEES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : ParseNode
//@
//@ Description : Parse un element d'un arbre de type WorkOrder et met
//@               a jour une structure de donnees Work Order
//@
//@ Parametres :
//@               (ENTREE) node : Noeud de l'arbre XML WorkOrder
//@               (ENTREE) element : Element de l'arbre XML WorkOrder
//@               (SORTIE) workOrder : Structure de donnees Work Order
//@                                    mise a jour
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_PRS_Parser::ParseNode(DOM_Node& node, 
                               DOMString element, 
                               CMN_DAT_WorkOrder& workOrder)
{
  // FT-2512
  char *str;
  // ..FT-2512

  //@ SELON le type du noeud
  switch (node.getNodeType())
    {
    //@ SI le type est TEXT_NODE:
    case DOM_Node::TEXT_NODE :
      {
        //@ Recuperer le nom et la valeur du noeud
        DOMString     nodeValue = node.getNodeValue();
        unsigned long lent      = nodeValue.length();
        
        //@ Supprimer les blancs avant et apres
        //@ S'il ne reste rien, eliminer le noeud
        XMLCh* buf = new XMLCh[lent + 1];
        XMLString::copyNString(buf, nodeValue.rawBuffer(), lent);
        buf[lent] = 0;
        XMLString::trim(buf);

        //@ SI le noeud n'est pas vide
        if (XMLString::stringLen(buf) > 0)
         { 
           //@ Recuperer la valeur du noeud
           // FT-2512
           str = nodeValue.transcode();
           t_Chaine valeur = str;
           if (str) delete [] str;	
           // ..FT-2512

#ifdef DEBUG
           cout << "TEXT_NODE:" << endl 
                << "\telement : " << element << endl
                << "\tvaleur  : \"" << nodeValue << "\"" << endl
                << "\tlongueur: " << lent << endl;
#endif

           //@ Affecter la valeur du noeud a un attribut du
           //@ WorkOrder en fonction de l'element auquel il
           //@ appartient
           if (element.equals(ELMT_PROCESSING_TYPE.c_str()))
             {
               workOrder.ProcessingType(valeur); 
             }
           else if (element.equals(ELMT_SENSING_START.c_str()))
             {
               workOrder.SensingStart(valeur);
             }
           else if (element.equals(ELMT_SENSING_STOP.c_str()))
             {
               workOrder.SensingStop(valeur);
             }
           else if (element.equals(ELMT_TIME_FLAG.c_str()))
             {
               workOrder.TimeIntervalFlag(valeur);
             }
           else if (element.equals(ELMT_TIME_COUNTER.c_str()))
             {
               workOrder.TimeIntervalCounter(long(valeur));
             }
           else if (element.equals(ELMT_UNPROC_DATA.c_str()))
             {
               workOrder.AjouterUnprocData(valeur);
             }
           else if (element.equals(ELMT_AUX_DATA.c_str()))
             {
               workOrder.AjouterAuxData(valeur);
             }
           else if (element.equals(ELMT_PROD_MODEL.c_str()))
             {
               workOrder.AjouterProdModel(valeur);
             }
         }
        //@ FIN SI le noeud n'est pas vide

        delete [] buf;
        break;
      }

    //@ SI le type est DOCUMENT_NODE:
    case DOM_Node::DOCUMENT_NODE :
      {
        //@ Rechercher les noeuds fils
        DOM_Node child = node.getFirstChild();

        //@ TANT QUE il existe un noeud fils
        while (child != 0)
          {
            //@ Parser le noeud fils
            ParseNode(child, element, workOrder);

            //@ Passer au noeud suivant
            child = child.getNextSibling();
          }
        //@ FIN TANT QUE il existe un noeud fils
        break;
      }

    //@ SI le type est ELEMENT_NODE:
    case DOM_Node::ELEMENT_NODE :
      {
        element = node.getNodeName();

#ifdef DEBUG
        cout << "ELEMENT_NODE: " << endl 
             << "\tnom   : " << node.getNodeName() << endl;
#endif

        //@ Rechercher le premier noeud fils
        DOM_Node child = node.getFirstChild();

        //@ SI il existe un noeud fils
        if (child != 0)
          {
            //@ TANT QUE il existe un noeud fils
            while( child != 0)
              {
                //@ Parser le noeud fils
                ParseNode(child, element, workOrder);

                //@ Passer au noeud suivant
                child = child.getNextSibling();
              }
            //@ FIN TANT QUE il existe un noeud fils
          }
        //@ FIN SI
        break;
      }

    //@ SI le type est PROCESSING_INSTRUCTION_NODE:
    case DOM_Node::PROCESSING_INSTRUCTION_NODE :

    //@ SI le type est ENTITY_REFERENCE_NODE:
    case DOM_Node::ENTITY_REFERENCE_NODE :

    //@ SI le type est CDATA_SECTION_NODE:
    case DOM_Node::CDATA_SECTION_NODE :

    //@ SI le type est COMMENT_NODE:
    case DOM_Node::COMMENT_NODE :

    //@ SI le type est DOCUMENT_TYPE_NODE:
    case DOM_Node::DOCUMENT_TYPE_NODE :

    //@ SI le type est ENTITY_NODE:
    case DOM_Node::ENTITY_NODE :

    //@ SI le type est XML_DECL_NODE:
    case DOM_Node::XML_DECL_NODE :
      {
        //@ Ignore
        break;
      }

    //@ Defaut:
    default :
      //@ Par defaut, on ne fait rien
      break;
    }
}

//@ ------------------------------------------------------------------*/
//@ Methode : ParseNode
//@
//@ Description : Parse un element d'un arbre de type OBTUTC et met
//@               a jour une structure de donnees OBTUTC
//@
//@ Parametres :
//@               (ENTREE) node    : Noeud de l'arbre XML OBTUTC
//@               (ENTREE) element : Element de l'arbre XML OBTUTC
//@               (SORTIE) OBTUTC  : Structure de donnees OBTUTC
//@                                  mise a jour
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_PRS_Parser::ParseNode(DOM_Node& node, 
                               DOMString element, 
                               SD_DON_OBTUTC& OBTUTC)
{
  t_Date date;
  // FT-2512
  char *str;
  // ..FT-2512

  switch (node.getNodeType())
    {

    // TEXT_NODE -------------------------------------------------
    case DOM_Node::TEXT_NODE :
      {
        // Recuperer le nom et la valeur du noeud
        DOMString     nodeValue = node.getNodeValue();
        unsigned long lent      = nodeValue.length();
        
        // Supprimer les blancs avant et apres
        // S'il ne reste rien, eliminer le noeud
        XMLCh* buf = new XMLCh[lent + 1];
        XMLString::copyNString(buf, nodeValue.rawBuffer(), lent);
        buf[lent] = 0;
        XMLString::trim(buf);

        if (XMLString::stringLen(buf) > 0)
  { 
        // FT-2512
    	str = nodeValue.transcode();
    	t_Chaine valeur = str;
    	if (str) delete [] str;
    	// ..FT-2512

#ifdef DEBUG
    cout << "TEXT_NODE:" << endl 
    << "\telement : " << element << endl
    << "\tvaleur  : \"" << nodeValue << "\"" << endl
    << "\tlongueur: " << lent << endl;
#endif

    /* FA136 */
    // date UTC_LEAP_SECOND, extraite du tag XML <UTC_LEAP_SECOND>
    if (element.equals(ELMT_UTC_LEAP_SECOND.c_str()))
    {
      // cree une date a partir d'une chaine au format long CCSDS
      date.ChaineCCSDS(valeur);
      OBTUTC.SetDateUTCLEAPSECOND(date);
    }

    // date OBT de reference, extraite du tag XML <CCU_OBT>, notee k0 dans l'algo
    else if (element.equals(ELMT_CCU_OBT.c_str()))
    {
      long long obt = strtoll(valeur.c_str(), NULL, 0);
 
      OBTUTC.SetDateOBTREF(obt);

#ifdef DEBUG
    cout << "ParseNode(OBTUTC) :" << endl 
    << "valeur du tag : " << valeur << endl
    << "date OBT      : " << OBTUTC.GetDateOBTREF() << endl;
#endif

    }

        // date UTC de reference, extraite du tag XML <UTC>, notee t0 dans l'algo
    else if (element.equals(ELMT_UTC.c_str()))
    {
      // cree une date a partir d'une chaine au format long CCSDS
      date.ChaineCCSDS(valeur);
      OBTUTC.SetDateUTCREF(date);
#ifdef DEBUG
    cout << "ParseNode(OBTUTC) :" << endl 
    << "valeur du tag : " << valeur << endl
    << "date UTCREF   : " << OBTUTC.GetDateUTCREF().Chaine() << endl;
#endif
    }

        // step a, extrait du tag XML <A_STEP>, note a dans l'algo
    else if (element.equals(ELMT_A_STEP.c_str()))
    {
      OBTUTC.SetAStep((double)valeur);
#ifdef DEBUG
    cout << "ParseNode(OBTUTC) :" << endl 
    << "valeur du tag : " << valeur << endl
    << "a_Step        : " << OBTUTC.GetAStep() << endl;
#endif
    }
    
    /* FA136 */
    // offset b, extrait du tag XML <B_OFFSET>
    else if (element.equals(ELMT_B_OFFSET.c_str()))
    {
      OBTUTC.SetBOffset((double)valeur);
#ifdef DEBUG
      cout << "ParseNode(OBTUTC) :" << endl 
           << "valeur du tag : " << valeur << endl
           << "b_offset      : " << OBTUTC.GetBOffset() << endl;
#endif
    }
  }

        delete [] buf;
        break;
      }

    // DOCUMENT_NODE -------------------------------------------------
    case DOM_Node::DOCUMENT_NODE :
      {
        DOM_Node child = node.getFirstChild();
        while (child != 0)
          {
            ParseNode(child, element, OBTUTC);
            child = child.getNextSibling();
          }
        break;
      }

    // ELEMENT_NODE -------------------------------------------------
    case DOM_Node::ELEMENT_NODE :
      {
        element = node.getNodeName();

#ifdef DEBUG
        cout << "ELEMENT_NODE: " << endl 
             << "\tnom   : " << node.getNodeName() << endl;
#endif

        // Recherche les noeuds fils
        DOM_Node child = node.getFirstChild();
        if (child != 0)
          {
            // Parser les noeuds fils
            while( child != 0)
              {
                ParseNode(child, element, OBTUTC);
                child = child.getNextSibling();
              }
          }
        break;
      }

    // autres cas -------------------------------------------------
    case DOM_Node::PROCESSING_INSTRUCTION_NODE :
    case DOM_Node::ENTITY_REFERENCE_NODE :
    case DOM_Node::CDATA_SECTION_NODE :
    case DOM_Node::COMMENT_NODE :
    case DOM_Node::DOCUMENT_TYPE_NODE :
    case DOM_Node::ENTITY_NODE :
    case DOM_Node::XML_DECL_NODE :
      {
        // Ignore
        break;
      }

    default :
      break;
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : CreerArbreReport
//@
//@ Description : Cree l'arbre XML d'un report
//@
//@ Parametres :
//@               (ENTREE) docWorkOrder
//@               (ENTREE) listeStageInfo
//@               (SORTIE) docReport
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_PRS_Parser::CreerArbreReport(DOM_Document& docWorkOrder,
                                      t_ListeStageInfo& listeStageInfo,
                                      DOM_Document& docReport)
{
  // Elements XML
  DOM_XMLDecl      decl;
  DOM_DocumentType type;
  DOM_Node         rootElem;
  DOM_Node         ajoutReport;
  DOM_Element      workOrderElem;
  DOM_Element      elem;
  DOM_NodeList     listeNoeudWorkOrder;

  // Nombre d'elements de la liste des noeuds du Work Order
  int              nbElemListeNoeudWorkOrder = 0;

  // Compteur d'iteration
  int              i = 0;


  //@ Creation de la racine
  docReport = DOM_Document::createDocument();

  //@ Ajout de l'element DocumentDeclaration
  decl = docReport.createXMLDecl(VERSION_REPORT.c_str(),
                                 PRS_DEFAULT_ENCODING.c_str(),
                                 STANDALONE_REPORT.c_str());
  docReport.appendChild(decl);

  //@ Ajout de l'element DocumentType
  type = docReport.createDocumentType(ELMT_PPF_REPORT.c_str());
  docReport.appendChild(type);
                  
  //@ Creation du noeud racine
  elem = docReport.createElement(ELMT_PPF_REPORT.c_str());
  docReport.appendChild(elem);

  //@ Recuperation de l'element racine; il faut sauter
  //@ l'element DocumentDeclaration et l'element
  //@ DocumentType
  rootElem = docReport.getFirstChild().getNextSibling().
    getNextSibling();
                  
  //@ Ajout d'un element WorkOrder
  workOrderElem = docReport.createElement(ELMT_WORK_ORDER.c_str());
  rootElem.appendChild(workOrderElem);

  //@ Ajout de l'arbre Work Order

  //@ Recuperation de la liste des noeuds du Work Order
  listeNoeudWorkOrder       = docWorkOrder.getElementsByTagName("*");

  //@ calcul de la taille de la liste des noeuds du Work Order
  nbElemListeNoeudWorkOrder = listeNoeudWorkOrder.getLength();
  
  //@ POUR tous les noeuds de la liste
  for (i=0; i < nbElemListeNoeudWorkOrder; i++) 
    {
      //@ SI ce n'est pas le premier element de la liste
      if (i!=0)
        {
          //@ Recuperation du noeud
          DOM_Node child = listeNoeudWorkOrder.item(i);

          //@ Import du noeud courant
          ajoutReport = docReport.importNode(child, true);

          //@ Ajout dans l'element Work Order du report
          workOrderElem.appendChild(ajoutReport);
        }
      //@ FIN SI
    }
  //@ FIN POUR tous les noeuds de la liste
          
  //@ Ajout des elements StageInfo
  AjouterListeStageInfo(docReport, rootElem, listeStageInfo);
}


//@ ------------------------------------------------------------------*/
//@ Methode : AjouterListeStageInfo
//@
//@ Description : Ajoute liste d'entites StageInfo dans l'arbre
//@               XML d'un report
//@
//@ Parametres :
//@               (ENTREE/SORTIE) docReport
//@               (ENTREE) rootElem
//@               (ENTREE) listeStageInfo
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_PRS_Parser::AjouterListeStageInfo(DOM_Document& docReport,
                                           DOM_Node& rootElem, 
                                           t_ListeStageInfo& 
                                           listeStageInfo)
{
  // Iterateur sur la liste passee en parametre
  t_ListeStageInfo::iterator pInfo;

  // Elements XML
  DOM_Element                stageElem;
  DOM_Element                processingElem;
  DOM_Element                elem;
  DOM_Text                   text;


  //@ POUR toutes les informations d'etapes
  for (pInfo = listeStageInfo.begin(); 
       pInfo != listeStageInfo.end();
       pInfo++)
    {
      //@ Ajout d'un element StageInfo
      stageElem = docReport.createElement(ELMT_STAGE_INFO.c_str());
      rootElem.appendChild(stageElem);
                      
      //@ Ajout d'un element StageName
      elem = docReport.createElement(ELMT_STAGE_NAME.c_str());
      stageElem.appendChild(elem);
      text = docReport.createTextNode(pInfo->StageName().c_str());
      elem.appendChild(text);
                      
      //@ Ajouter l'information UsedFileName
      AjouterListeUsedFileName(docReport, 
                               stageElem, 
                               pInfo->ListeUsedFileName());
      
      //@ Ajouter l'information GeneratedFile
      AjouterListeGeneratedFile(docReport, 
                                stageElem, 
                                pInfo->ListeGeneratedFile());
                      
      //@ Ajout d'un element MHSTelemetry
      elem = docReport.createElement(ELMT_MHS_TELEMETRY.c_str());
      stageElem.appendChild(elem);
      text = docReport.createTextNode(pInfo->MHSTelemetry().c_str());
      elem.appendChild(text);
                  
      //@ Ajout d'un element ProcessingInfo
      processingElem = docReport.createElement(ELMT_PROCESSING_INFO.
                                               c_str());
      stageElem.appendChild(processingElem);
                  
      //@ Ajout d'un element ProcessingStart
      elem = docReport.createElement(ELMT_PROCESSING_START.c_str());
      processingElem.appendChild(elem);
      text = docReport.createTextNode(pInfo->ProcessingStart().c_str());
      elem.appendChild(text);
                  
      //@ Ajout d'un element ProcessingStop
      elem = docReport.createElement(ELMT_PROCESSING_STOP.c_str());
      processingElem.appendChild(elem);
      text = docReport.createTextNode(pInfo->ProcessingStop().c_str());
      elem.appendChild(text);
                  
      //@ Ajout d'un element ElapsedTime
      elem = docReport.createElement(ELMT_ELAPSED_TIME.c_str());
      processingElem.appendChild(elem);
      text = docReport.createTextNode(pInfo->ElapsedTime().c_str());
      elem.appendChild(text);
                  
      //@ Ajout d'un element UserTime
      elem = docReport.createElement(ELMT_USER_TIME.c_str());
      processingElem.appendChild(elem);
      text = docReport.createTextNode(pInfo->UserTime().c_str());
      elem.appendChild(text);
/* CR290 */
      //@ Ajout d'un element LogMessage
      elem = docReport.createElement(ELMT_LOG_MESSAGE.c_str());
      stageElem.appendChild(elem);
      //@ On concatene la liste des cas degrades trouves au LogMessage genere
      t_Chaine LogText=pInfo->LogMessage() + CGList;
      text = docReport.createTextNode(LogText.c_str());
      elem.appendChild(text);
/* Fin CR290 */
    }
  //@ FIN POUR toutes les informations d'etapes
}


//@ ------------------------------------------------------------------*/
//@ Methode : AjouterListeUsedFileName
//@
//@ Description : Ajoute liste d'entites UsedFileName dans l'arbre
//@               XML d'un report
//@
//@ Parametres :
//@               (ENTREE/SORTIE) docReport
//@               (ENTREE) stageElem
//@               (ENTREE) listeUsedFileName
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_PRS_Parser::AjouterListeUsedFileName(DOM_Document& docReport,
                                              DOM_Element& stageElem, 
                                              t_ListeChaine& 
                                              listeUsedFileName)
{
  // Iterateur sur la liste passee en parametre
  t_ListeChaine::iterator pUsedFileName;
  
  // Elements XML
  DOM_Element             elem;
  DOM_Text                text;

  
  //@ POUR tous les fichiers utilises
  for (pUsedFileName = listeUsedFileName.begin();
       pUsedFileName != listeUsedFileName.end();
       pUsedFileName++)
    {
      //@ Ajouter l'information UsedFileName
 
      //@ Ajout d'un element UsedFileName
      elem = docReport.createElement(ELMT_USED_FILE_NAME.c_str());
      stageElem.appendChild(elem);
      text = docReport.createTextNode(pUsedFileName->c_str());
      elem.appendChild(text);
    }
  //@ FIN POUR tous les fichiers utilises
}


//@ ------------------------------------------------------------------*/
//@ Methode : AjouterListeGeneratedFile
//@
//@ Description : Ajoute liste d'entites GeneratedFile dans l'arbre
//@               XML d'un report
//@
//@ Parametres :
//@               (ENTREE/SORTIE) docReport
//@               (ENTREE) stageElem
//@               (ENTREE) listeGeneratedFile
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_PRS_Parser::AjouterListeGeneratedFile(DOM_Document& docReport,
                                               DOM_Element& stageElem, 
                                               t_ListeGeneratedFile& 
                                               listeGeneratedFile)
{
  // Iterateur sur la liste passee en parametre
  t_ListeGeneratedFile::iterator pGeneratedFile;
  
  // Iterateur sur la liste passee en parametre
  t_ListeDataGap::iterator       pDataGap;

  // Elements XML
  DOM_Element                    elem;
  DOM_Text                       text;
  DOM_Element                    generatedElem;
  DOM_Element                    dataGapElem;


  //@ POUR tous les fichiers generes
  for (pGeneratedFile = listeGeneratedFile.begin();
       pGeneratedFile != listeGeneratedFile.end();
       pGeneratedFile++)
    {
      //@ Ajouter l'information GeneratedFile
      
      //@ Ajout d'un element GeneratedFile
      generatedElem = docReport.createElement(ELMT_GENERATED_FILE.
                                              c_str());
      stageElem.appendChild(generatedElem);
      
      //@ Ajout d'un element FileName
      elem = docReport.createElement(ELMT_FILE_NAME.c_str());
      generatedElem.appendChild(elem);
      text = docReport.createTextNode(pGeneratedFile->FileName().
                                      c_str());
      elem.appendChild(text);
      
      //@ SI le MDR est defini
      if (pGeneratedFile->MdrDefined() == CMN__VRAI)
        {
          t_Chaine mdrOk(pGeneratedFile->MdrOk());
          t_Chaine mdrDegraded(pGeneratedFile->MdrDegraded());
/* CR290 */
          t_Chaine nbCasXX;
/* Fin CR290 */
          
          //@ Ajout d'un element MdrOk
          elem = docReport.createElement(ELMT_MDR_OK.c_str());
          generatedElem.appendChild(elem);
          text = docReport.createTextNode(mdrOk.c_str());
          elem.appendChild(text);
          
          //@ Ajout d'un element MdrDegraded
          elem = docReport.createElement(ELMT_MDR_DEGRADED.c_str());
          generatedElem.appendChild(elem);
          text = docReport.createTextNode(mdrDegraded.c_str());
          elem.appendChild(text);
 /* CR290 */
          CGList="";
          for (int i=1; i<=15; i++)
          {  
            nbCasXX=pGeneratedFile->GetNbCas(i);
            if (nbCasXX != "0")
            {
              CGList=CGList+"; CG"+(t_Chaine)i+":"+nbCasXX;
            }
          }
 /* Fin CR290 */
        }
      //@ FIN SI le MDR est defini
      
      //@ POUR tous les ecarts
      for (pDataGap = pGeneratedFile->ListeDataGap().begin();
           pDataGap != pGeneratedFile->ListeDataGap().end();
           pDataGap++)
        {
          t_Chaine gapCount(pDataGap->Count());
      
          //@ Ajout d'un element DataGap
          dataGapElem = docReport.createElement(ELMT_DATA_GAP.c_str());
          generatedElem.appendChild(dataGapElem);
          
          //@ Ajout d'un element GapReason
          elem = docReport.createElement(ELMT_GAP_REASON.c_str());
          dataGapElem.appendChild(elem);
          text = docReport.createTextNode(pDataGap->Reason().c_str());
          elem.appendChild(text);
          
          //@ Ajout d'un element GapCount
          elem = docReport.createElement(ELMT_GAP_COUNT.c_str());
          dataGapElem.appendChild(elem);
          text = docReport.createTextNode(gapCount.c_str());
          elem.appendChild(text);
        }
      //@ FIN POUR tous les ecarts
    }
  //@ FIN POUR tous les fichiers generes
}


//@ ------------------------------------------------------------------*/
//@ Methode : DumpTree
//@
//@ Description : Ecrit un fichier XML a partir d'un document XML
//@               passe en parametre
//@
//@ Parametres :
//@               (ENTREE) xmlFile : Nom du fichier XML cree
//@               (ENTREE) xmlDoc : Document a ecrire dans le fichier
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_PRS_Parser::DumpTree(t_Chaine xmlFile, DOM_Node& xmlDoc)
{
  CMN_PRS_DumpFormatTarget* formatTarget = new
    CMN_PRS_DumpFormatTarget(xmlFile);
  DOMString                 encNameStr(PRS_DEFAULT_ENCODING.c_str());
  DOM_Node                  aNode;
  unsigned int              lent = 0;
  // FT-2512
  char *str;
  // ..FT-2512

  //@ Recherche de l'encodage
  aNode = xmlDoc.getFirstChild();
  if (aNode.getNodeType() == DOM_Node::XML_DECL_NODE)
    {
      DOMString aStr = ((DOM_XMLDecl &)aNode).getEncoding();
      if (aStr != "")
        {
          encNameStr = aStr;
        }
    }
  
  lent = encNameStr.length();
  S_EncodingName = new XMLCh[lent + 1];
  XMLString::copyNString(S_EncodingName, encNameStr.rawBuffer(), lent);
  S_EncodingName[lent] = 0;
  
  //@ En captant les exceptions
  try
    {
      S_Formatter = new XMLFormatter(S_EncodingName,
                                     formatTarget,
                                     XMLFormatter::NoEscapes, 
                                     m_UnRepFlags);
      //@ Sortie du document
      cout << xmlDoc;
      
      //@ Ajouter linefeed dans la sortie
      *S_Formatter << chLF;
      
      cout << flush;
      
      delete S_Formatter;
    }
  
  //@ Traiter les exceptions
  catch (const XMLException& toCatch)
    {
      ERR__Exception exception;
      
#ifdef DEBUG
      cerr << STR_ERROR_DUMPING_TREE << endl << STR_XML_EXCEPTION_MSG
           << DOMString(toCatch.getMessage()) << endl;
#endif
      
      delete [] S_EncodingName;
      delete formatTarget;
      
      //@ Generation d'une exception DUMPING_FAILED
      exception.SetErrorCode(ERR__Exception::DUMPING_FAILED);
      exception.SetParametre(STR_ERROR_DUMPING_TREE);
      // FT-2512
      str = DOMString(toCatch.getMessage()).transcode();
      exception.SetParametre(str);
      if (str) delete [] str;
      // ..FT-2512
      THROW(exception)
    }
  
  delete [] S_EncodingName;
  delete formatTarget;
}
