// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/PRS/src/CMN_PRS_Parser.h,v 1.3 2011/11/25 10:17:39 dpauly Exp $
//@ ---------------------------------------------------------------------------
//@ Composant : CMN
//@ Sous-composant : CMN_PRS
//@ CLASSE : CMN_PRS_Parser
//@
//@ Commentaire :
//@    Ce fichier gere la classe CMN_PRS_Parser
//@
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-136-CN : 01/09/2004 : P.CABANE
//@ Reading of elements <UTC_LEAP_SECOND> and <B_OFFSET> has been added
//@ fiche de correction correspondante : IA-2100-FC-136-THA
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-2100-CR-290-CN : 31/03/2006 : B. GUENDE
//@ Added new private member: CGList, to store a t_Chaine with a list of CG encountered 
//@ See IA-2100-CR-290-THA
//@ ---------------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-172-CN : 21/09/2006 : R.MORENO
//@ Gestion des manoeuvres: prise en compte des tags ANX signalant des
//@ passages au noeud ascendant.
//@ ---------------------------------------------------------------------------
//@ FA-ID : IA-FT-2364(FA-2100-OPS) : 20/09/2010 : S. BRUNEL
//@ Product without  manoeuvre flag raised 
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

#ifndef CMN_PRS_Parser_h
#define CMN_PRS_Parser_h

//
// Fichiers include
//

// STL
#include <iostream>

// Standard

// Xerces
#include <xercesc/framework/XMLFormatter.hpp>
#include <xercesc/parsers/DOMParser.hpp>

// Projet
#include "CMN__Commun.h"
#include "ERR__Exception.h"
#include "CMN_DAT_WorkOrder.h"
#include "CMN_DAT_StageInfo.h"
#include "CMN_DAT_GeneratedFile.h"
#include "CMN_DAT_DataGap.h"
#include "CMN_PRS_ErrorReporter.h"
#include "CMN_PRS_RedirectHandler.h"

class SD_DON_OBTUTC;


class CMN_PRS_Parser {
    
    /*================================================================*/

    friend ostream&       operator<<(ostream& target, 
                                     const DOMString& s);
    friend XMLFormatter&  operator<<(XMLFormatter& strm, 
                                     const DOMString& s);
    friend ostream&       operator<<(ostream& target, 
                                     DOM_Node& toWrite);



 public:

    ////////////////////////////////
    // Constructeur
    ////////////////////////////////
    CMN_PRS_Parser() throw (ERR__Exception);

    ////////////////////////////////
    // Destructeur
    ////////////////////////////////
    virtual ~CMN_PRS_Parser();

    ////////////////////////////////
    // Accesseurs
    ////////////////////////////////
    void DoNamespaces(bool toBeDone) {m_DoNamespaces = toBeDone;};
    void DoSchema(bool toBeDone) {m_DoSchema = toBeDone;};
    void SchemaFullChecking(bool toBeDone) 
        {m_SchemaFullChecking = toBeDone;};
    void DoCreate(bool toBeDone) {m_DoCreate = toBeDone;};

    void UnRepFlags(const char* flag);
    void ValScheme(const char* scheme);


    ////////////////////////////////
    // Methodes publiques
    ////////////////////////////////
    void ParseFile     (CMN__Chaine xmlFile) throw (ERR__Exception);
    void ParseWorkOrder(CMN_DAT_WorkOrder& workOrder);
    void ParseOBTUTC   (SD_DON_OBTUTC& OBTUTC) throw (ERR__Exception);
    // FT-2364
    void ParseOSV      (t_Date& dateDebutUTC, t_Date& dateFinUTC,
                        t_Chaine& flag, bool &flagManoeuvre) throw (ERR__Exception);
    // ...FT-2364
    void EcrireReport  (t_Chaine xmlFile, t_ListeStageInfo listeStageInfo);
    void DumpTree      (t_Chaine xmlFile);

    
    /*================================================================*/

 protected:

    ////////////////////////////////
    // Methodes protegees
    ////////////////////////////////

    ////////////////////////////////
    // Attributs proteges
    ////////////////////////////////
    

    /*================================================================*/

 private:
 
    //
    // Constantes
    //

    // Elements XML WorkOrder
    static const t_Chaine ELMT_PPF_WORK_ORDER;
    static const t_Chaine ELMT_PROCESSING_TYPE;
    static const t_Chaine ELMT_SENSING_START;
    static const t_Chaine ELMT_SENSING_STOP;
    static const t_Chaine ELMT_TIME_FLAG;
    static const t_Chaine ELMT_TIME_COUNTER;
    static const t_Chaine ELMT_UNPROC_DATA;
    static const t_Chaine ELMT_AUX_DATA;
    static const t_Chaine ELMT_PROD_MODEL;

    // Elements XML OBTUTC
    static const t_Chaine ELMT_TYPE_ARBRE_OBTUTC;
    static const t_Chaine ELMT_CCU_OBT;
    static const t_Chaine ELMT_UTC;
    static const t_Chaine ELMT_A_STEP;
    
    /* FA136 */
    static const t_Chaine ELMT_UTC_LEAP_SECOND;
    static const t_Chaine ELMT_B_OFFSET;
    
    // Elements XML OSV
    static const t_Chaine ELMT_TYPE_ARBRE_OSV;
    static const t_Chaine ELMT_OSV_ORBIT_START;
    static const t_Chaine ELMT_OSV_FLAG;
    static const t_Chaine ELMT_OSV_TIME;
/* FA 172 */
    static const t_Chaine FLAG_ANX;
    static const t_Chaine FLAG_OOP_END;
    static const t_Chaine FLAG_IP_END;
    static const t_Chaine FLAG_IP_START;
    static const t_Chaine FLAG_OOP_START;
/* ..FA 172 */

    // Elements XML Report
    static const t_Chaine ELMT_PPF_REPORT;
    static const t_Chaine VERSION_REPORT;
    static const t_Chaine STANDALONE_REPORT;
    static const t_Chaine ELMT_WORK_ORDER;
    static const t_Chaine ELMT_STAGE_INFO;
    static const t_Chaine ELMT_STAGE_NAME;
    static const t_Chaine ELMT_USED_FILE_NAME;
    static const t_Chaine ELMT_GENERATED_FILE;
    static const t_Chaine ELMT_FILE_NAME;
    static const t_Chaine ELMT_MDR_OK;
    static const t_Chaine ELMT_MDR_DEGRADED;
    static const t_Chaine ELMT_DATA_GAP;
    static const t_Chaine ELMT_GAP_REASON;
    static const t_Chaine ELMT_GAP_COUNT;
    static const t_Chaine ELMT_MHS_TELEMETRY;
    static const t_Chaine ELMT_PROCESSING_INFO;
    static const t_Chaine ELMT_PROCESSING_START;
    static const t_Chaine ELMT_PROCESSING_STOP;
    static const t_Chaine ELMT_ELAPSED_TIME;
    static const t_Chaine ELMT_USER_TIME;
    static const t_Chaine ELMT_LOG_MESSAGE;

    // Configuration du parser
    static const t_Chaine UNREPFLAG_FAIL;
    static const t_Chaine UNREPFLAG_REP;
    static const t_Chaine VALSCHEME_NEVER;
    static const t_Chaine VALSCHEME_ALWAYS;
    static const t_Chaine PRS_DEFAULT_ENCODING;

    // Messages d'erreur
    static const t_Chaine STR_XML_EXCEPTION_MSG;
    static const t_Chaine STR_DOM_EXCEPTION_MSG;
    static const t_Chaine STR_ERROR_INIT;
    static const t_Chaine STR_ERROR_PARSING;
    static const t_Chaine STR_ERROR_PARSING_WO;
    static const t_Chaine STR_ERROR_PARSING_OBT;
    static const t_Chaine STR_ERROR_PARSING_OSV;
    static const t_Chaine STR_ERROR_REPORTING;
    static const t_Chaine STR_ERROR_INVALID_TREE;
    static const t_Chaine STR_ERROR_DUMPING_TREE;
   
    //
    // Types
    //
    typedef enum {
        PPF_WORK_ORDER        = 0,
        PROCESSING_TYPE       = 1,
        SENSING_START         = 2,
        SENSING_STOP          = 3,
        TIME_INTERVAL_FLAG    = 4,
        TIME_INTERVAL_COUNTER = 5,
        UNPROC_DATA           = 6,
        AUX_DATA              = 7,
        PROD_MODEL            = 8
    }e_CMN_PRS_ElmtWorkOrder;


    ////////////////////////////////
    // Methodes privees
    ////////////////////////////////
    void ParseNode(DOM_Node& node, 
                   DOMString element, 
                   CMN_DAT_WorkOrder& workOrder);
    void ParseNode(DOM_Node& node, 
                   DOMString element, 
                   SD_DON_OBTUTC& OBTUTC);
    void CreerArbreReport(DOM_Document& docWorkOrder,
                          t_ListeStageInfo& listeStageInfo,
                          DOM_Document& docReport);
    void AjouterListeStageInfo(DOM_Document& docReport,
                               DOM_Node& rootElem, 
                               t_ListeStageInfo& listeStageInfo);
    void AjouterListeUsedFileName(DOM_Document& docReport,
                                  DOM_Element& stageElem, 
                                  t_ListeChaine& listeUsedFileName);
    void AjouterListeGeneratedFile(DOM_Document& docReport,
                                   DOM_Element& stageElem, 
                                   t_ListeGeneratedFile& 
                                   listeGeneratedFile);
    void DumpTree(t_Chaine xmlFile, DOM_Node& doc);

    ////////////////////////////////
    // Attributs prives
    ////////////////////////////////

    CMN_PRS_ErrorReporter*   m_ErrReporter;
    CMN_PRS_RedirectHandler* m_RedirectHandler;
    DOMParser*               m_Parser;

    // Parser configuration

    // -Indicates whether namespace processing should be done
    bool                     m_DoNamespaces;

    // -Indicates whether schema processing should be done
    bool                     m_DoSchema;

    // -Indicates whether full schema constraint checking should be done
    bool                     m_SchemaFullChecking;

    // -Indicates whether entity reference nodes needs to be created
    bool                     m_DoCreate;

    // -Indicates the way to handle unrepresentable chars
    XMLFormatter::UnRepFlags m_UnRepFlags;

    // -Indicates what validation scheme to use
    DOMParser::ValSchemes    m_ValScheme;
/* CR290 */
    // - Indicates in <LogMessage> field a list of CG encountered
    t_Chaine CGList;
/* Fin CR290 */
};

#endif
