static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/PRS/src/CMN_PRS_RedirectHandler.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ ---------------------------------------------------------------------------
//@ Composant : CMN
//@ CLASSE : CMN_PRS_RedirectHandler
//@
//@ Commentaire :
//@    Ce fichier contient la classe CMN_PRS_RedirectHandler, qui 
//@    gere le changement de fichier DTD lors de la lecture des
//@    fichiers XML
//@
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 22/01/2002 : P. CABANE
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-144-CN : 21/09/2004 : B. MASSART
//@ Problemes lors du portage d'OPS vers Solaris par Meteo France
//@ Voir IA-2100-FC-144-THA.
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

/* FA144 : suppression define __MINMAX_DEFINED, residu BORLAND */

//
// Fichiers include
//

// STL
#include <iostream>
#include <string>
#include <fstream>

using namespace std;

// Standard

// Xerces
#include <xercesc/dom/DOMString.hpp>
#include <xercesc/util/XMLUniDefs.hpp>
#include <xercesc/util/XMLUni.hpp>
#include <xercesc/sax/AttributeList.hpp>
#include <xercesc/sax/SAXParseException.hpp>
#include <xercesc/sax/SAXException.hpp>
#include <xercesc/framework/LocalFileInputSource.hpp>

// Projet
#include "CMN__Commun.h"
#include "CMN__Fichier.h"
#include "CFG__Configuration.h"
#include "CMN_PRS_RedirectHandler.h"


//
// Definition des constantes
//
const t_Chaine CMN_PRS_RedirectHandler::EXTENSION_DTD = "dtd";
const t_Chaine CMN_PRS_RedirectHandler::SEP_EXTENSION = ".";


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CMN_PRS_RedirectHandler
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_PRS_RedirectHandler::CMN_PRS_RedirectHandler() 
  : m_AttrCount(0), m_CharacterCount(0), m_ElementCount(0), 
    m_SpaceCount(0), m_SawErrors(false), m_ConfDir("")
{
  //@ Recuperer la variable d'environnement REP_CONF_DIR
  m_ConfDir = CFG__Configuration::GetRepertoire(CONF_DIR);

  //@ SI la variable d'environnement REP_CONF_DIR est definie
  if (! m_ConfDir.empty())
    {
      //@ Lui ajouter le separateur de repertoire
      m_ConfDir += CMN__SEP_REPERTOIRE;
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~CMN_PRS_RedirectHandler
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN_PRS_RedirectHandler::~CMN_PRS_RedirectHandler()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : startElement
//@
//@ Description : Gere le debut d'un element
//@
//@ Parametres :
//@               (ENTREE) name
//@               (ENTREE) attributes
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_PRS_RedirectHandler::startElement(const XMLCh* const name,
                                           AttributeList& attributes)
{
  m_ElementCount++;
  m_AttrCount += attributes.getLength();
}


//@ ------------------------------------------------------------------*/
//@ Methode : characters
//@
//@ Description : Ajoute un nombre de caracteres a m_CharacterCount 
//@
//@ Parametres :
//@               (ENTREE) chars
//@               (ENTREE) length
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_PRS_RedirectHandler::characters(const XMLCh* const chars,
                                         const unsigned int length)
{
  m_CharacterCount += length;
}
    

//@ ------------------------------------------------------------------*/
//@ Methode : ignorableWhitespace
//@
//@ Description : Ajoute un nombre d'espaces a m_SpaceCount
//@
//@ Parametres :
//@               (ENTREE) chars
//@               (ENTREE) length
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_PRS_RedirectHandler::ignorableWhitespace(const XMLCh* const chars,
                                                  const unsigned int 
                                                  length)
{
  m_SpaceCount += length;
}
    

//@ ------------------------------------------------------------------*/
//@ Methode : resetDocument
//@
//@ Description : RAZ des attributs
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_PRS_RedirectHandler::resetDocument()
{
  //@ RAZ des attributs
  m_AttrCount      = 0;
  m_CharacterCount = 0;
  m_ElementCount   = 0;
  m_SpaceCount     = 0;
}
 

//@ ------------------------------------------------------------------*/
//@ Methode : warning
//@
//@ Description : Gestion des warnings
//@
//@ Parametres :
//@               (ENTREE) toCatch
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_PRS_RedirectHandler::warning(const SAXParseException& toCatch)
{
  //@ Ignorer tous les warnings
}


//@ ------------------------------------------------------------------*/
//@ Methode : error
//@
//@ Description : Affiche une erreur
//@
//@ Parametres :
//@               (ENTREE) toCatch
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_PRS_RedirectHandler::error(const SAXParseException& toCatch)
{
  m_SawErrors = true;

  //@ Re-emission de l'exception
  throw(toCatch);
}


//@ ------------------------------------------------------------------*/
//@ Methode : fatalError
//@
//@ Description : Affiche une erreur fatale
//@
//@ Parametres :
//@               (ENTREE) toCatch
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_PRS_RedirectHandler::fatalError(const SAXParseException& toCatch)
{
  m_SawErrors = true;

  //@ Re-emission de l'exception
  throw(toCatch);
}


//@ ------------------------------------------------------------------*/
//@ Methode : resetErrors
//@
//@ Description : Reinitialise les erreurs
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN_PRS_RedirectHandler::resetErrors()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : resolveEntity
//@
//@ Description : Redirige le fichier DTD
//@
//@ Parametres :
//@               (ENTREE) publicId
//@               (ENTREE) systemId
//@
//@ Valeur de retour :
//@               NULL pour aucun changement
//@               Sinon nouvelle valeur pour le fichier DTD
//@
//@ ------------------------------------------------------------------*/
InputSource* CMN_PRS_RedirectHandler::resolveEntity(const XMLCh* const 
                                                    publicId,
                                                    const XMLCh* const
                                                    systemId)
{
  // Valeur de retour de la methode
  InputSource* nouvelleSource = NULL;

#ifdef DEBUG
  cout << "CMN_PRS_RedirectHandler::resolveEntity: " << endl;
  cout << "\tpublicId: " << XMLString::transcode(publicId) << endl;
  cout << "\tsystemId: " << XMLString::transcode(systemId) << endl;
#endif    

  //@ SI le repertoire m_ConfDir a ete defini
  if (! m_ConfDir.empty())
    {
      // La valeur de la DTD est contenue dans systemId (balise SYSTEM)
      t_Chaine ancienneDTD = XMLString::transcode(systemId);

      //@ SI l'entite est un fichier DTD
      if (EstDeTypeDTD(ancienneDTD))
        {
          // Nouveau fichier DTD
          t_Chaine nouvelleDTD(m_ConfDir+ancienneDTD);

#ifdef DEBUG
          cout << "Changement de DTD: " << endl;
          cout << "\tancienne DTD: " << ancienneDTD << endl;
          cout << "\tnouvelle DTD: " << nouvelleDTD << endl;
#endif

          XMLCh *final =  XMLString::transcode(nouvelleDTD.c_str());

          //@ Rediriger vers le nouveau fichier DTD
          nouvelleSource = new LocalFileInputSource(final);
        }
      //@ FIN SI l'entite est un fichier DTD
    }
  //@ FIN SI le repertoire m_ConfDir a ete defini


  //@ Retourner la valeur de retour de la methode
  return nouvelleSource;
}


//======================================================================
//                                      METHODES PRIVEES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : EstDeTypeDTD
//@
//@ Description : Teste si un fichier est du type DTD
//@
//@ Parametres :
//@               (ENTREE) nomFichier : Nom du fichier
//@
//@ Valeur de retour : 
//@               CMN__VRAI si le fichier est de type DTD
//@               CMN__FAUX sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Booleen CMN_PRS_RedirectHandler::EstDeTypeDTD(const t_Chaine&
                                                     nomFichier)
{
  // Extension du nom du fichier
  t_Chaine       extension("");

  // Valeur de retour de la fonction
  t_CMN__Booleen valRetour = CMN__FAUX;


  //@ Recuperation de l'extension du fichier
  extension = nomFichier.substr(nomFichier.rfind(SEP_EXTENSION)+1, 
                                EXTENSION_DTD.length());

  //@ Comparaison de la chaine recuperee avec l'extension DTD
  //@ SI les deux chaines sont identiques
  if (extension == EXTENSION_DTD)
    {
      //@ Positionner la valeur de retour a VRAI
      valRetour = CMN__VRAI;
    }
  //@ FIN SI les deux chaines sont identiques


  //@ Retourner la valeur de retour de la fonction
  return valRetour;
}
