static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/CFG__Configuration.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : CMN
//@ CLASSE : CFG__Configuration
//@ 
//@ Commentaire :
//@    Ce module gere la classe CFG__Configuration
//@ 					
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 18/04/2000 : C.LACAZE
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//
#include <stdio.h>
#include <stdlib.h>
#include <values.h>

#include "CMN__Commun.h"
#include "CFG__Configuration.h"


//
// Definition des constantes
//
const char *CFG__Configuration::CFG__PREFIXE_FICHIER = "FIC_";
const char *CFG__Configuration::CFG__PREFIXE_REPERTOIRE = "REP_";
const char *CFG__Configuration::CFG__PREFIXE_NUMERIQUE = "NUM_";
const char *CFG__Configuration::CFG__PREFIXE_ADRESSE = "ADR_";
const char *CFG__Configuration::CFG__PREFIXE_LISTE_ADRESSE = "LISTE_ADR_";
const char *CFG__Configuration::CFG__PREFIXE_LISTE_CHAINE = "LISTE_CHN_";
const char *CFG__Configuration::CFG__PREFIXE_CHAINE = "CHN_";
const char CFG__Configuration::CFG__SEPARATOR = ',';


//======================================================================
//					METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CFG__Configuration
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CFG__Configuration::CFG__Configuration()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~CFG__Configuration
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CFG__Configuration::~CFG__Configuration()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetFichier
//@
//@ Description : Permet de recuperer le nom du fichier dont le nom 
//@               logique est passe en parametre
//@
//@ Parametres :
//@               (ENTREE) fichier : nom logique du fichier
//@
//@ Valeur de retour :
//@               Nom physique du fichier
//@
//@ ------------------------------------------------------------------*/
t_Chaine CFG__Configuration::GetFichier ( const t_Chaine& fichier)
{
  //@ Retour de la valeur de la variable d'environnement
  //@ dont le nom passe en parametre est prefixe par "FIC_"
  return GetValeurEnvironnement (CFG__PREFIXE_FICHIER + fichier);
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetRepertoire
//@
//@ Description : Permet de recuperer le nom du repertoire dont le nom
//@               logique est passe en parametre
//@
//@ Parametres :
//@               (ENTREE) repertoire : nom logique du repertoire
//@
//@ Valeur de retour :
//@               Nom physique du repertoire
//@
//@ ------------------------------------------------------------------*/
t_Chaine CFG__Configuration::GetRepertoire ( const t_Chaine& repertoire)
{
  //@ Retour de la valeur de la variable d'environnement
  //@ dont le nom passe en parametre est prefixe par "REP_"
  return GetValeurEnvironnement (CFG__PREFIXE_REPERTOIRE + repertoire);
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetNumerique
//@
//@ Description : Permet de recuperer la valeur numerique dont le nom 
//@               logique est passe en parametre
//@
//@ Parametres :
//@               (ENTREE) numerique : nom logique de la valeur
//@
//@ Valeur de retour :
//@               Valeur associee, MAXDOUBLE si non connu
//@
//@ ------------------------------------------------------------------*/
double CFG__Configuration::GetNumerique(const t_Chaine& numerique)
{
  double   valeur = MAXDOUBLE;
  t_Chaine strValeur("");

  //@ Retour de la valeur de la variable d'environnement
  //@ dont le nom passe en parametre est prefixe par "NUM_"
  //@ et transformation de la valeur en double
  strValeur = GetValeurEnvironnement(CFG__PREFIXE_NUMERIQUE + 
                                     numerique);

  if (strValeur != "")
    {
      valeur = double(strValeur);
    }

  return valeur;
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetAdresse
//@
//@ Description : Permet de recuperer l'adresse dont le nom logique
//@               est passe en parametre
//@
//@ Parametres :
//@               (ENTREE) adresse : nom logique de l'adresse
//@
//@ Valeur de retour :
//@               Adresse reelle associee
//@
//@ ------------------------------------------------------------------*/
t_Chaine CFG__Configuration::GetAdresse ( const t_Chaine& adresse )
{
  //@ Retour de la valeur de la variable d'environnement
  //@ dont le nom passe en parametre est prefixe par "ADR_"
  return GetValeurEnvironnement (CFG__PREFIXE_ADRESSE + adresse);
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetListeAdresse
//@
//@ Description : Permet de recuperer la liste d'adresses dont le nom
//@               logique est passe en parametre
//@
//@ Parametres :
//@               (ENTREE) adresses : nom logique de la liste d'adresses
//@
//@ Valeur de retour :
//@               Liste d'adresses reelle associee triees
//@
//@ ------------------------------------------------------------------*/
t_ListeChaineTriee CFG__Configuration::GetListeAdresse ( const t_Chaine& adresses )
{
  t_ListeChaineTriee listeValeur;
  t_Chaine      valeurs;
  
  //@ Recuperation de la valeur de la variable d'environnement
  //@ dont le nom passe en parametre est prefixe par "LISTE_ADR_"
  valeurs = GetValeurEnvironnement (CFG__PREFIXE_LISTE_ADRESSE + adresses);

  //@ Construction de la liste d'adresses
  if( !valeurs.empty() ){
    size_t next = 0;
    size_t start = 0;
    
    while( start != valeurs.npos ){
      next = valeurs.find(CFG__SEPARATOR, start);
      if( next != valeurs.npos ){
	listeValeur.insert (valeurs.substr(start,next-start));
	start = next + 1;
      }else{
	listeValeur.insert (valeurs.substr(start,valeurs.length()-start));
        start = next;
      }
    }
  }
  
  //@ Retour de cette derniere
  return listeValeur;
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetListeChaine
//@
//@ Description : Permet de recuperer la liste de chaines dont le nom
//@               logique est passe en parametre
//@
//@ Parametres :
//@               (ENTREE) chaine : nom logique de la liste de chaines
//@
//@ Valeur de retour :
//@               Liste d'adresses reelle associee
//@
//@ ------------------------------------------------------------------*/
t_ListeChaine CFG__Configuration::GetListeChaine ( const t_Chaine& chaine )
{
  t_ListeChaine listeValeur;
  t_Chaine      valeurs;

  //@ Recuperation de la valeur de la variable d'environnement
  //@ dont le nom passe en parametre est prefixe par "LISTE_CHN_"
  valeurs = GetValeurEnvironnement (CFG__PREFIXE_LISTE_CHAINE + chaine);

  //@ Construction de la liste d'adresses
  if( !valeurs.empty() ){
    size_t next = 0;
    size_t start = 0;
    
    while( start != valeurs.npos ){
      next = valeurs.find(CFG__SEPARATOR, start);
      if( next != valeurs.npos ){
	listeValeur.insert (listeValeur.end(),valeurs.substr(start,next-start));
	start = next + 1;
      }else{
	listeValeur.insert (listeValeur.end(),valeurs.substr(start,valeurs.length()-start));
        start = next;
      }
    }
  }

  //@ Retour de cette derniere
  return listeValeur;
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetChaine
//@
//@ Description : Permet de recuperer la liste de chaines dont le nom 
//@               logique est passe en parametre
//@
//@ Parametres :
//@               (ENTREE) chaine : nom logique de la chaine
//@
//@ Valeur de retour :
//@               Chaine de caracteres associee
//@
//@ ------------------------------------------------------------------*/
t_Chaine CFG__Configuration::GetChaine ( const t_Chaine& chaine )
{
  //@ Retour de la valeur de la variable d'environnement
  //@ dont le nom passe en parametre est prefixe par "CHN_"
  return GetValeurEnvironnement (CFG__PREFIXE_CHAINE + chaine);
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetValeurEnvironnement
//@
//@ Description : Permet de recuperer la valeur de la variable
//@	           d'environnement dont le nom est passe en parametre
//@
//@ Parametres :
//@               (ENTREE) variable : nom de la variable d'environnement
//@
//@ Valeur de retour :
//@               Valeur de la variable d'environnement trouvee
//@
//@ ------------------------------------------------------------------*/
t_Chaine CFG__Configuration::GetValeurEnvironnement (const t_Chaine& variable)
{
  t_Chaine retour("");

  //@ Recuperation de la valeur de variable d'environnement
  char *   valeur = getenv (variable.c_str());

  //@ Transformation de cette derniere en chaine
  if( valeur != NULL )
    {
      retour = valeur;
    }
	
  //@ Retour de la valeur
  return retour;
}

