static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/CMN__Chaine.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : CMN
//@ CLASSE : CMN__Chaine
//@
//@ Commentaire :
//@    Ce module gere la classe CMN__Chaine, qui permet de manipuler
//@    des chaines de caracteres
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 07/10/1999 : BBS
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 14/12/2000 : C.LACAZE
//@  Portage sous systeme AIX 4.3
//@ --------------------------------------------------------------------
//@ VERSION : 29/09/2000 : C.LACAZE
//@  Verification de mot-cles errones
//@ FA-ID : FPA-F11-FA-634-CN : 29/09/2000 :
//@  Ajout de la methode VerifierChamp.
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-3316(DM-2100-OPS) : 18/05/2017 : MPE
//@ PORTAGE OPS SOUS RHEL 6.5 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

#include <cstdio>
#include <cstdlib>

#include <iostream>
#include <fstream>
#include <iomanip> //pour setprecision etc...
#include <cctype>
#include <sstream>



#include "ERR__Exception.h"
#include "CMN__Commun.h"
#include "CMN__Chaine.h"

// Migration RHEL 6.5
#ifdef LINUX
#include <cstring>
#endif

//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CMN__Chaine
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Chaine::CMN__Chaine() : string()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~CMN__Chaine
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Chaine::~CMN__Chaine()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : CMN__Chaine
//@
//@ Description : Constructeur avec affectation d'un objet string
//@
//@ Parametres :
//@               (ENTREE) s : string d'initialisation
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Chaine::CMN__Chaine(const string s) : string(s)
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : CMN__Chaine
//@
//@ Description : Constructeur avec affectation d'une chaine de
//@               caracteres
//@
//@ Parametres :
//@               (ENTREE) s : chaine de caraceteres d'initialisation
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Chaine::CMN__Chaine(const char *s) : string(s)
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : CMN__Chaine
//@
//@ Description : Constructeur avec affectation d'un objet CMN__chaine
//@
//@ Parametres :
//@               (ENTREE) chaine : objet CMN__Chaine d'initialisation
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Chaine::CMN__Chaine(const CMN__Chaine & chaine)
{
  *this = chaine.c_str();
}


//@ ------------------------------------------------------------------*/
//@ Methode : CMN__Chaine
//@
//@ Description : Constructeur avec affectation d'une valeur entiere
//@               longue
//@
//@ Parametres :
//@               (ENTREE) valeur : valeur entiere longue
//@                                 d'initialisation
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Chaine::CMN__Chaine(const long valeur)
{
  char buffer[100];
  
  sprintf(buffer,"%ld",valeur);
  *this = buffer;
}


//@ ------------------------------------------------------------------*/
//@ Methode : CMN__Chaine
//@
//@ Description : Constructeur avec affectation d'une valeur entiere
//@               longue non signee
//@
//@ Parametres :
//@               (ENTREE) valeur : valeur d'initialisation
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Chaine::CMN__Chaine(const unsigned long valeur)
{
  char buffer[100];

  sprintf(buffer, "%lu", valeur);
  *this = buffer;
}


//@ ------------------------------------------------------------------*/
//@ Methode : CMN__Chaine
//@
//@ Description : Constructeur avec affectation d'une valeur entiere
//@
//@ Parametres :
//@               (ENTREE) valeur : valeur entiere d'initialisation
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Chaine::CMN__Chaine(const int valeur)
{
  char buffer[100];

  sprintf(buffer,"%d",valeur);
  *this = buffer;
}


//@ ------------------------------------------------------------------*/
//@ Methode : CMN__Chaine
//@
//@ Description : Constructeur avec affectation d'une valeur reelle
//@
//@ Parametres :
//@               (ENTREE) valeur : valeur reelle d'initialisation
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Chaine::CMN__Chaine(const double valeur)
{
  char buffer[100];
  
  sprintf(buffer,"%.15e",valeur);
  *this = buffer;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Upper
//@
//@ Description : Convertit toute les lettres de la chaine en majuscule 
//@
//@ Parametres :
//@               Aucun         
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__Chaine::Upper()
{
  //@ Dupliquer la chaine
  char * copie= strdup(c_str());
  char *p=copie ;
        
  //@ Convertir tous les caracteres de la chaine dupliquee en majuscule
  while (*p) 
    {
      *p=toupper(*p);
      p++;
    }
        
  //@ Affecter la nouvelle chaine
  *this = copie;
        
  //@ Liberer la chaine dupliquee
  free(copie);
}


//@ ------------------------------------------------------------------*/
//@ Methode : Lower
//@
//@ Description : Convertit toute les lettres de la chaine en minuscule 
//@
//@ Parametres :
//@               Aucun         
//@
//@ Valeur de retour : 
//@               Aucune  
//@
//@ ------------------------------------------------------------------*/
void CMN__Chaine::Lower()
{
  //@ Dupliquer la chaine
  char * copie= strdup(c_str());
  char *p=copie ;
        
  //@ Convertir tous les caracteres de la chaine duppliquee en majuscule
  while (*p) 
    {
      *p=tolower(*p);
      p++;
    }
        
  //@ Affecter la nouvelle chaine
  *this = copie;
        
  //@ Liberer la chaine dupliquee
  free(copie);
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator long()
//@
//@ Description : Convertit la chaine de caractere en valeur entiere 
//@
//@ Parametres :
//@               (ENTREE)  valeur a convertir
//@
//@ Valeur de retour : 
//@               long
//@
//@ ------------------------------------------------------------------*/
CMN__Chaine::operator long()
{
  long i=0;
  t_ChaineFixe Buffer;
  
  Buffer[0] = 0;
  if (sscanf(c_str(),"%ld%s",&i,Buffer)== 1)
    return i;
  else
    return INT_NAN;
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator int()
//@
//@ Description : Convertit la chaine de caractere en valeur entiere 
//@
//@ Parametres :
//@               (ENTREE)  valeur a convertir
//@
//@ Valeur de retour : 
//@               int
//@
//@ ------------------------------------------------------------------*/
CMN__Chaine::operator int()
{
  int i=0;
  t_ChaineFixe Buffer;
  
  Buffer[0] = 0;
  if (sscanf(c_str(),"%d%s",&i,Buffer)== 1)
    return i;
  else
    return INT_NAN;
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator double()
//@
//@ Description : Convertit la chaine de caractere en valeur reelle
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               double
//@
//@ ------------------------------------------------------------------*/
CMN__Chaine::operator double()
{
  double f=0.;
  t_ChaineFixe Buffer;
  
  Buffer[0] = 0;
  if (sscanf(c_str(),"%le%s",&f,Buffer)== 1)
    return f;
  else
    return DBL_MAX;
}


//@ ------------------------------------------------------------------*/
//@ Methode : FormaterValeurF
//@
//@ Description : Met une valeur au format voulu sous forme de string
//@
//@ Parametres :
//@               (ENTREE) Valeur : Valeur a mettre au format 
//@                                 FTaille.Precision
//@               (ENTREE) Taille : Taille du string, signe et point
//@                                 compris 
//@               (ENTREE) Precision : Nombre de chiffres apres la 
//@                                    virgule
//@
//@ Valeur de retour : 
//@               Valeur au format sous forme de string
//@
//@ ------------------------------------------------------------------*/
void CMN__Chaine::FormaterValeurF(double Valeur, 
                                  int Taille, 
                                  int Precision = 0)
{
  stringstream message;
  CMN__Chaine  s("");

  message.fill('0'); //Complete la chaine par des 0
  message << setprecision(Precision) //Nombre de chiffres max apres la virgule
          << setiosflags(ios::fixed) //Pour faire apparraitre les 0 deriere la virgule
          << setiosflags(ios::showpos) //Pour afficher le signe +
          << setiosflags(ios::internal) //Pour placer les 0 entre le signe et la valeur
          << setw(Taille) //Specifie le nombre de caracteres a ecrire
          << Valeur << '\0';
  s = message.str();
  s = s.substr(0, Taille);
  *this = s.c_str();
}

void CMN__Chaine::FormaterValeurFSansSigne(double Valeur, 
                                           int Taille, 
                                           int Precision = 0)
{
  CMN__Chaine s("");

  s.FormaterValeurF(Valeur, Taille + 1, Precision);
  *this = s.substr(1, Taille);
}

void CMN__Chaine::FormaterValeurF(const CMN__Chaine& Valeur, 
                                  int Taille, 
                                  int Precision = 0)
{
  double ValeurDouble;
  char * stopstring;
  
  ValeurDouble = strtod(Valeur.c_str(), & stopstring);
  
  FormaterValeurF(ValeurDouble, Taille, Precision);
}

void CMN__Chaine::FormaterValeurFSansSigne(const CMN__Chaine& Valeur, 
                                           int Taille, 
                                           int Precision = 0)
{
  CMN__Chaine s("");

  s.FormaterValeurF(Valeur, Taille + 1, Precision);
  *this = s.substr(1, Taille);
}


//@ ------------------------------------------------------------------*/
//@ Methode : FormaterValeurE
//@
//@ Description : Met une valeur au format voulu sous forme de string
//@
//@ Parametres :
//@               (ENTREE) Valeur : Valeur a mettre au format 
//@                                 ETaille.Precision
//@               (ENTREE) Taille : Taille du string, signe et point
//@                                 compris 
//@               (ENTREE) Precision : Nombre de chiffres apres la 
//@                                    virgule
//@
//@ Valeur de retour : 
//@               Valeur au format sous forme de string
//@
//@ ------------------------------------------------------------------*/
void CMN__Chaine::FormaterValeurE(double Valeur, 
                                  int Taille, 
                                  int Precision = 0)
{
  stringstream message;
  CMN__Chaine  s("");

  message.fill('0'); //Complete la chaine par des 0
  message << setprecision(Precision) //Nombre de chiffres max apres la virgule
          << setiosflags(ios::scientific) //Pour ecriture exponentielle
          << setiosflags(ios::showpoint) //Pour afficher le point
          << setiosflags(ios::internal) //Pour placer les 0 entre le signe et la valeur
          << setw(Taille) //Specifie le nombre de caracteres a ecrire
          << Valeur << '\0';
  s = message.str();
  s = s.substr(0, Taille);
  *this = s.c_str();
}


void CMN__Chaine::FormaterValeurE(const CMN__Chaine& Valeur, 
                                  int Taille, 
                                  int Precision = 0)
{
  double ValeurDouble;
  char * stopstring;
  
  ValeurDouble = strtod(Valeur.c_str(), & stopstring);
  
  FormaterValeurE(ValeurDouble, Taille, Precision);
}


//@ ------------------------------------------------------------------*/
//@ Methode : CopySansLigneComment
//@
//@ Description : Recopie les lignes d'un string sans les lignes de 
//@               commentaires
//@
//@ Parametres :
//@               (ENTREE) Original: string original 
//@               (ENTREE) Symbole : caractere(s) au debut des lignes
//@                                  de commentaire
//@
//@ Valeur de retour : 
//@               Le string sans les lignes de commentaires
//@
//@ ------------------------------------------------------------------*/
void CMN__Chaine::CopierSansLigneCommentaires(const CMN__Chaine& Original, 
                                              const CMN__Chaine& Symbole)
{
  CMN__Chaine Orig(Original);
  CMN__Chaine Copy;
  
  CMN__Chaine DebutDeLigne, NL = "\n";
  int TailleSymbole = Symbole.size();
  unsigned long PosDebutLigne = 0, PosFinLigne;
  
  PosFinLigne = Orig.ExtrairePosApresChamp(NL);
  
  while (PosFinLigne !=-1)
    {
      DebutDeLigne = Orig.substr(PosDebutLigne, TailleSymbole);
      if (DebutDeLigne != Symbole)
        Copy += Orig.substr(PosDebutLigne, PosFinLigne - PosDebutLigne);
      PosDebutLigne = PosFinLigne;
      PosFinLigne = Orig.ExtrairePosApresChamp(NL, PosDebutLigne);
    }
  
  DebutDeLigne = Orig.substr(PosDebutLigne, TailleSymbole);
  if (DebutDeLigne != Symbole)
    Copy += Orig.substr(PosDebutLigne, PosFinLigne - PosDebutLigne);
  
  *this=Copy;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ExtraireValeurChamp
//@
//@ Description : Renvoie la valeur d'un champ
//@
//@ Parametres :
//@               (ENTREE) ValeurDest : String ou est stockee la valeur
//@               (ENTREE) NomChamp : Nom du champ a rechercher
//@               (ENTREE) Delimiter : Delimiter de la fin du champ
//@               (ENTREE) APartirDe : Position a partir de laquelle ca
//@                                    cherche (si rien n'est specifie, 
//@                                    ca commence au debut du fichier)
//@
//@ Valeur de retour : 
//@               npos si le champ n'a pas ete trouve
//@
//@ ------------------------------------------------------------------*/
unsigned long CMN__Chaine::ExtraireValeurChamp(CMN__Chaine& ValeurDest, 
                                               const CMN__Chaine& 
                                               NomChamp, 
                                               const CMN__Chaine& 
                                               Delimiter, 
                                               unsigned long APartirDe)
{
  CMN__Chaine ValeurChamp;
  unsigned long PosDebutValeur, PosFinValeur;

  PosDebutValeur = ExtrairePosApresChamp(NomChamp, APartirDe);
  if (PosDebutValeur != -1)
    {
      PosFinValeur = find(Delimiter, PosDebutValeur);
      ValeurDest = substr(PosDebutValeur, PosFinValeur - PosDebutValeur);
    }
  
  return (PosDebutValeur);
}

unsigned long CMN__Chaine::ExtraireValeurChamp(CMN__Chaine& ValeurDest, 
                                               const CMN__Chaine& 
                                               NomChamp, 
                                               const CMN__Chaine& 
                                               Delimiter)
{
  return (ExtraireValeurChamp(ValeurDest, NomChamp, Delimiter, 0));
}


//@ ------------------------------------------------------------------*/
//@ Methode : RemplacerValeurPos
//@
//@ Description : Va ecrire dans le string source la nouvelle valeur
//@
//@ Parametres :
//@               (ENTREE) Valeur : Valeur a ecrire 
//@               (ENTREE) Pos : Positon ou erire
//@
//@ Valeur de retour : 
//@               npos si le champ n'a pas ete trouve
//@
//@ ------------------------------------------------------------------*/
unsigned long CMN__Chaine::RemplacerValeurPos(const CMN__Chaine& Valeur,
                                              unsigned long Pos)
{
  replace(Pos, Valeur.size(), Valeur);

  return (Pos);
}


//@ ------------------------------------------------------------------*/
//@ Methode : RemplacerValeurAncienne
//@
//@ Description : Va ecrire dans le string source la nouvelle valeur 
//@               du champ
//@
//@ Parametres :
//@               (ENTREE) ValeurAncienne : Valeur a remplacer 
//@               (ENTREE) ValeurNouvelle : Valeur a ecrire
//@
//@ Valeur de retour : 
//@               npos si le champ n'a pas ete trouve
//@
//@ ------------------------------------------------------------------*/
unsigned long CMN__Chaine::RemplacerValeurAncienne(const CMN__Chaine& 
                                                   ValeurAncienne,
                                                   const CMN__Chaine& 
                                                   ValeurNouvelle)
{
  unsigned long Pos;
  
  Pos = ExtrairePosAvantChamp( ValeurAncienne);
  
  if (Pos == -1)
    cout << "fichier d'entree incorrect : la valeur ancienne n'existe pas \n";
  else
    replace(Pos, ValeurNouvelle.size(), ValeurNouvelle);
  
  return (Pos);
}


//@ ------------------------------------------------------------------*/
//@ Methode : RemplacerValeurChamp
//@
//@ Description : Va ecrire dans le string source la nouvelle valeur
//@               du champ
//@
//@ Parametres :
//@               (ENTREE) Valeur : Valeur a ecrire 
//@               (ENTREE) NomChamp : Nom du champ a rechercher
//@
//@ Valeur de retour : 
//@               npos si le champ n'a pas ete trouve
//@
//@ ------------------------------------------------------------------*/
unsigned long CMN__Chaine::RemplacerValeurChamp(const CMN__Chaine& Valeur, 
                                                const CMN__Chaine& 
                                                NomChamp)
{
  unsigned long Pos;
  
  Pos = ExtrairePosApresChamp( NomChamp);
  
  if (Pos == -1)
    cout << "fichier d'entree incorrect : le champ n'existe pas \n";
  else
    replace(Pos, Valeur.size(), Valeur);
  
  return (Pos);
}


//@ ------------------------------------------------------------------*/
//@ Methode : RemplacerValeurChamp
//@
//@ Description : Va ecrire dans le string source la nouvelle valeur
//@               du champ
//@
//@ Parametres :
//@               (ENTREE) Valeur : Valeur a ecrire 
//@               (ENTREE) NomChamp : Nom du champ a rechercher
//@               (ENTREE) TailleChamp : Nbre de caractere a ecrire (met
//@                                      des espace en plus si 
//@                                      necessaire)
//@
//@ Valeur de retour : 
//@               npos si le champ n'a pas ete trouve
//@
//@ ------------------------------------------------------------------*/
unsigned long CMN__Chaine::RemplacerValeurChamp(const CMN__Chaine& Valeur, 
                                                const CMN__Chaine&
                                                NomChamp,
                                                int TailleChamp)
{
  CMN__Chaine NewValeur = Valeur;
  
  NewValeur.resize(TailleChamp,' ');

  return (RemplacerValeurChamp(NewValeur, NomChamp));
}


//@ ------------------------------------------------------------------*/
//@ Methode : RemplacerValeurChamp
//@
//@ Description : Va ecrire dans le string source la nouvelle valeur 
//@               du champ
//@
//@ Parametres :
//@               (ENTREE) Valeur : Valeur a ecrire 
//@               (ENTREE) NomChamp : Nom du champ a rechercher
//@               (ENTREE) APartirDe : Position a partir de laquelle
//@                                    ca cherche
//@
//@ Valeur de retour : 
//@               npos si le champ n'a pas ete trouve
//@
//@ ------------------------------------------------------------------*/
unsigned long CMN__Chaine::RemplacerValeurChamp(const CMN__Chaine& Valeur, 
                                                const CMN__Chaine& 
                                                NomChamp, 
                                                unsigned long APartirDe)

{
  unsigned long Pos;
        
  Pos = ExtrairePosApresChamp( NomChamp, APartirDe);
  
  if (Pos == -1)
    cout << "fichier d'entree incorrect : le champ n'existe pas \n";
  else
    replace(Pos, Valeur.size(), Valeur);
  
  return (Pos);
}


//@ ------------------------------------------------------------------*/
//@ Methode : ExtrairePosAvantChamp
//@
//@ Description : Renvoie l'adresse pointant sur le champ 
//@
//@ Parametres :
//@               (ENTREE) NomChamp : Nom du champ a rechercher
//@
//@ Valeur de retour : 
//@               Adresse sur le champ, npos si le champ n'est pas 
//@               trouve
//@
//@ ------------------------------------------------------------------*/
unsigned long CMN__Chaine::ExtrairePosAvantChamp(const CMN__Chaine& 
                                                 NomChamp)
{
  unsigned long Pos;
  
  Pos = find(NomChamp);
  
  return (Pos);
}


//@ ------------------------------------------------------------------*/
//@ Methode : ExtrairePosAprsChamp
//@
//@ Description : Renvoie l'adresse pointant juste apres le champ
//@
//@ Parametres :
//@               (ENTREE) NomChamp : Nom du champ a rechercher
//@
//@ Valeur de retour : 
//@               Adresse sur la Valeur du champ, npos si le champ n'a
//@               pas ete trouve
//@
//@ ------------------------------------------------------------------*/
unsigned long CMN__Chaine::ExtrairePosApresChamp(const CMN__Chaine& 
                                                 NomChamp)
{
  unsigned long Pos;
  
  Pos = ExtrairePosAvantChamp( NomChamp);
  if (Pos != -1) Pos += NomChamp.size();
  
  return Pos;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ExtrairePosAvantChamp
//@
//@ Description : Renvoie l'adresse pointant sur le champ 
//@
//@ Parametres :
//@               (ENTREE) NomChamp : Nom du champ a rechercher
//@               (ENTREE) PosDepart : Endroit a partir duquel ca 
//@                                    cherche
//@
//@ Valeur de retour : 
//@               Adresse sur le champ, npos si le champ n'est pas 
//@               trouve
//@
//@ ------------------------------------------------------------------*/
unsigned long CMN__Chaine::ExtrairePosAvantChamp(const CMN__Chaine& NomChamp, 
                                                 unsigned long PosDepart)
{
  unsigned long Pos;
  
  Pos = find(NomChamp, PosDepart);
  
  return Pos;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ExtrairePosAprsChamp
//@
//@ Description : Renvoie l'adresse pointant juste apres le champ
//@
//@ Parametres :
//@               (ENTREE) NomChamp : Nom du champ a rechercher
//@               (ENTREE) PosDepart : Endroit a partir duquel ca 
//@                                    cherche
//@
//@ Valeur de retour : 
//@               Adresse sur la Valeur du champ, npos si le champ 
//@               n'est pas trouve
//@
//@ ------------------------------------------------------------------*/
unsigned long CMN__Chaine::ExtrairePosApresChamp(const CMN__Chaine& 
                                                 NomChamp, 
                                                 unsigned long PosDepart)
{
  unsigned long Pos;
  
  Pos = ExtrairePosAvantChamp( NomChamp, PosDepart);
  if (Pos != -1) Pos += NomChamp.size();
  
  return Pos;
}


//@ ------------------------------------------------------------------*/
//@ Methode : VerifierChamp
//@
//@ Description : Verifie la presence d'un mot-cle
//@
//@ Parametres :
//@               (ENTREE) nomChamp : Nom du champ a rechercher
//@
//@ Valeur de retour : 
//@               Aucun. Une exception est emise si le champ
//@               n'a pas ete trouve.
//@
//@ ------------------------------------------------------------------*/
void CMN__Chaine::VerifierChamp(const CMN__Chaine& nomChamp)
{
  ERR__Exception exception;

  //@ Recherche de la position du champ
  unsigned long Pos = ExtrairePosAvantChamp(nomChamp);
  
  //@ SI le champ n'a pas ete trouve
  if( Pos == -1 )
    {
      //@ emission d'une exception
      exception.SetParametre (nomChamp);
      exception.SetErrorCode(ERR__Exception::FIELD_NOT_FOUND);
      THROW(exception);
    }
  //@ SINON
  else if( Pos != 0)
    {
      //@ verification que le caractere avant le champ est
      //@ un espace
      CMN__Chaine car = substr(Pos-1,1);
      
      //@ SI ce n'est pas le cas
      if( !isspace(car.c_str()[0]) )
        {
          //@ emission d'une exception
          exception.SetParametre (nomChamp);
          exception.SetErrorCode(ERR__Exception::FIELD_NOT_FOUND);
          THROW(exception);
        }
      //@ FIN SI
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : RemplacerChaine
//@
//@ Description : Remplace dans une chaine toutes les occurences d'une 
//@               sous-chaine par une autre de meme longueur
//@
//@ Parametres :
//@               (ENTREE) AncienneValeur
//@               (ENTREE) NouvelleValeur
//@
//@ Valeur de retour : 
//@               Chaine avec les remplacements effectues
//@
//@ ------------------------------------------------------------------*/
CMN__Chaine& t_Chaine::RemplacerChaine(string AncienneValeur, 
                                       string NouvelleValeur)
{
  unsigned long PosReplace;
  unsigned long PosDepart;


  PosDepart=0;
  for (PosReplace = find(AncienneValeur , PosDepart);
       PosReplace !=  -1 ; PosReplace = find(AncienneValeur, PosDepart))
    {
      replace(PosReplace, AncienneValeur.size(), NouvelleValeur);
      PosDepart = PosReplace + NouvelleValeur.size() ;
    }

  return *this;
}

//@ ------------------------------------------------------------------*/
//@ Methode : NombreDeMot
//@
//@ Description : Remplace dans une chaine toutes les occurences d'une 
//@               sous-chaine par une autre de meme longueur
//@
//@ Parametres :
//@               (ENTREE) AncienneValeur
//@               (ENTREE) NouvelleValeur
//@
//@ Valeur de retour : 
//@               Chaine avec les remplacements effectues
//@
//@ ------------------------------------------------------------------*/
int  t_Chaine::NombreDeMot (const CMN__Chaine& Delimiter)
{
  string ::size_type pos=0;
  string ::size_type lastPos;
  int nombreMot=0;

  while (pos != string::npos){

    lastPos = pos;
    pos = this->find_first_of (Delimiter,lastPos);

    // Si la fin de la string est atteinte
    if (pos != string::npos) {
      nombreMot++;
      pos ++;
    }

  }

  // Si la derniere position traite n'est pas le dernier caractere 
  // de la chaine alors il y a un mot supplementaire
  if ( ((this->length())-lastPos)>0) {
      nombreMot++;
  }

  return (nombreMot);
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetFirstMot
//@
//@ Description : Extrait le 1ier mot de la chaine jusqu'au delimiteur
//@
//@ Parametres :
//@               (ENTREE) Delimiter
//@               (SORTIE) mot       : Mot extrait
//@
//@ Valeur de retour : 
//@
//@ ------------------------------------------------------------------*/
void t_Chaine::GetFirstMot(const CMN__Chaine& Delimiter,
                                   CMN__Chaine& mot)

{
  //@ Se positionner sur le 1ier char de la chaine 
   m_pos = 0;

  //@ extraire le 1ier mot par rapport au delimiteur
  this->GetMot(m_pos, Delimiter, mot);
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetNextMot
//@
//@ Description : Extrait le mot suivant par rapport a la position
//@               courante dans la chaine jusqu'au delimiteur
//@
//@               (ENTREE) Delimiter
//@               (SORTIE) mot       : Mot suivant extrait
//@ Valeur de retour : 
//@               
//@
//@ ------------------------------------------------------------------*/
void t_Chaine::GetNextMot(const CMN__Chaine& Delimiter, CMN__Chaine& mot)

{
  if (m_pos!=string::npos) {m_pos++;}

  this->GetMot(m_pos, Delimiter, mot);
}


//======================================================================
//                                      METHODES PRIVEES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : GetMot
//@
//@ Description : Extrait un mot a partir de la position passe en
//@               parametre jusqu'au delimiteur
//@
//@               (ENTREE) Delimiter : delimiteur du mot
//@               (ENTREE) aPartirDe : position de depart de la recherche
//@               (SORTIE) mot       : Mot extrait
//@   
//@ Valeur de retour : 
//@               
//@ ------------------------------------------------------------------*/
void t_Chaine::GetMot(string ::size_type aPartirDe, 
                      const CMN__Chaine& Delimiter,
                      CMN__Chaine& mot)
{
  int taille=0;

  if (aPartirDe != string::npos) {
    m_pos = this->find_first_of (Delimiter,aPartirDe);

    if (m_pos == string::npos) {
       taille = (this->length())-aPartirDe;}
    else {
       taille = m_pos-aPartirDe;}

    mot =this->substr (aPartirDe, taille);
  }

  else {
   mot= "\0";
  }
}
