// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/CMN__Chaine.h,v 1.3 2011/11/25 10:17:39 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : CMN
//@ CLASSE : CMN__Chaine
//@
//@ Commentaire :
//@    Ce module gere la classe CMN__Chaine, qui permet de manipuler
//@    des chaines de caracteres
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 07/10/99 : BBS
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 14/12/2000 : C.LACAZE
//@  Portage sous systeme AIX 4.3
//@ --------------------------------------------------------------------
//@ VERSION : 29/09/2000 : C.LACAZE
//@  Verification de mot-cles errones
//@ FA-ID : FPA-F11-FA-634-CN : 29/09/2000 :
//@  Ajout de la methode VerifierChamp.
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef CMN__Chaine_h
#define CMN__Chaine_h


//
// Fichiers include
//

// STL
#include <string>
#include <climits>
#include <cfloat>
#include <list>
#include <vector>
#include <set>

// Standard

// Projet

#define INT_NAN INT_MAX
#define DBL_NAN DBL_MAX

using namespace std;


class CMN__Chaine : public string  
{

    /*=================================================================*/

 public:
    
    //
    // Constructeurs
    //
    CMN__Chaine();
    CMN__Chaine(const string s);
    CMN__Chaine(const char *s);
    CMN__Chaine(const CMN__Chaine & s);
    CMN__Chaine(const long valeur);
    CMN__Chaine(const unsigned long valeur);
    CMN__Chaine(const int valeur);
    CMN__Chaine(const double valeur);
    
    //
    // Destructeur
    //
    virtual ~CMN__Chaine();
    
    //
    // Methodes publiques
    //
    
    CMN__Chaine& operator=(const string& rhs) 
	{string::operator=(rhs); return *this;}
    CMN__Chaine& operator=(const char *s) {string::operator=(s);return *this;}
    CMN__Chaine& operator=(char c) {string::operator=(c);return *this;}
    //operator string() {return string(c_str());};
    void				Upper();	
    void				Lower();

    operator int() ;
    operator long() ;
    operator double();

    void FormaterValeurF(const CMN__Chaine&, //Valeur Source a mettre au format
			 int,	//Nombre de caracteres (signe et virgule inclus)
			 int);	//Nombre de caracteres apres la virgule

    void FormaterValeurF(double, int, int);
    
    void FormaterValeurFSansSigne(const CMN__Chaine&, //Valeur Source a mettre au format
				  int,	//Nombre de caracteres (signe et virgule inclus)
				  int);	//Nombre de caracteres apres la virgule
    void FormaterValeurFSansSigne(double, int, int);
    
    void FormaterValeurE(const CMN__Chaine&, //Valeur Source a mettre au format
			 int,	//Nombre de caracteres (signe et virgule inclus)
			 int);	//Nombre de caracteres apres la virgule
    
    void FormaterValeurE(double, int, int);
    
    void CopierSansLigneCommentaires(const CMN__Chaine&,  //Fichier source a copier
				     const CMN__Chaine&); //Delimiter qui marque le debut 
    //d'une ligne de commentaires
    
    unsigned long ExtraireValeurChamp(CMN__Chaine&,	//String resultat de la requete
				      const CMN__Chaine&,	//Nom du champ
				      const CMN__Chaine&);	//Delimiter qui marque la fin de la valeur
    
    unsigned long ExtraireValeurChamp(CMN__Chaine&, const CMN__Chaine&,
				      const CMN__Chaine&, 
				      unsigned long);	//Specifie l'endroit dans le fichier a partir 
    //duquel on cherche
    
    unsigned long RemplacerValeurAncienne(const CMN__Chaine&, //Valeur Ancienne qui sera ecrasee
					  const CMN__Chaine&); //Nouvelle Valeur
    
    unsigned long RemplacerValeurPos(const CMN__Chaine&,	//Valeur a ecrire
				     unsigned long);	//Position ou l'ecrire
    
    unsigned long RemplacerValeurChamp(const CMN__Chaine&,	//Valeur a ecrire
				       const CMN__Chaine&,	//Nom du champ ou ecrire
				       unsigned long);	// Position a partir de laquelle
    
    //chercher le champ
    //Si ce parametre ne figure pas, ca
    //cherche a partir du debut du fichier
    
    unsigned long RemplacerValeurChamp(const CMN__Chaine&,	 // Valeur a ecrire
				       const CMN__Chaine&, // Nom du champ ou
				       // ecrire
				       int); // Nombre max de 
    // caracteres a ecrire 
    
    unsigned long RemplacerValeurChamp(const CMN__Chaine&, const CMN__Chaine&);
    
    unsigned long ExtrairePosAvantChamp(const CMN__Chaine&, // Champ a chercher
					unsigned long); // Position a partir
    // de laquelle chercher le champ. Si ce parametre ne figure pas, ca
    //cherche a partir du debut du fichier
    
    unsigned long ExtrairePosApresChamp(const CMN__Chaine&, unsigned long);
    unsigned long ExtrairePosAvantChamp(const CMN__Chaine&);
    unsigned long ExtrairePosApresChamp(const CMN__Chaine&);
    
    void VerifierChamp(const CMN__Chaine&);
    CMN__Chaine& RemplacerChaine(string AncienneValeur, string NouvelleValeur);

   int NombreDeMot (const CMN__Chaine& Delimiter);
   void GetFirstMot(const CMN__Chaine& Delimiter,CMN__Chaine& mot);
   void GetNextMot(const CMN__Chaine& Delimiter, CMN__Chaine& mot);

protected:

private:
   void GetMot (string ::size_type aPartirDe, 
			const CMN__Chaine& Delimiter,CMN__Chaine& mot);

    //@ Position courante dans la string (utiliser pour extraire les mots)
    string ::size_type m_pos;
};


//
// Type
//
typedef CMN__Chaine	                    t_Chaine;

// Liste de chaine
typedef list<CMN__Chaine>	            t_ListeChaine;
typedef vector<CMN__Chaine>	            t_TableauChaine;
typedef set<CMN__Chaine,less<CMN__Chaine> > t_ListeChaineTriee;

#endif
