static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/CMN__Config.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : CMN
//@ CLASSE : CMN__Config
//@ 
//@ Commentaire :
//@    Ce module gere la classe CMN__Configuration
//@    en s'appuyant sur le module C CMN__Configuration
//@ 					
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 18/04/2000 : C.LACAZE
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------


//
// Fichiers include
//
#include <stdio.h>

#include "CMN__Commun.h"
#include "CMN__Config.h"


//======================================================================
//					METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CMN__Configuration
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Configuration::CMN__Configuration()
{
  t_CMN__Statut cr;
  
  cr = CMN__Configuration_Ouvrir(&m_IdConfiguration);
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~CMN__Configuration
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Configuration::~CMN__Configuration()
{
  CMN__Configuration_Fermer(m_IdConfiguration);
}


//@ ------------------------------------------------------------------*/
//@ Methode : ChargerFichier
//@
//@ Description : Permet de charger en gestion le contenu d'un
//@               fichier au format configuration
//@
//@ Parametres :
//@               (ENTREE) Fichier : nom du fichier
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__Configuration_ERR_MAUVAIS_ID : Mauvais 
//@                                       identifiant de configuration
//@               CMN__ERR_PLUS_DE_MEMOIRE : Plus de memoire
//@               CMN__ERR_FICHIER_INTROUVABLE : Fichier introuvable
//@               CMN__Configuration_ERR_AUCUN_CHARGEMENT : Aucun
//@                                                   element charge
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration::ChargerFichier(t_Chaine Fichier)
{
  t_CMN__Statut cr;
  cr = CMN__Configuration_ChargerFichier( m_IdConfiguration,
					  (char *)Fichier.c_str());
  if (cr!=CMN__STATUT_OK)
    {
#ifdef DEBUG
      printf("Erreur a l'ouverture du fichier %s \n",
	     (char *)Fichier.c_str());
      printf("Verifier l'existence du fichier et ses droits d'acces\n"); 
#endif
    }
  else
    {
#ifdef DEBUG
      printf("ouverture reussie en lecture du fichier %s \n",
	     (char *)Fichier.c_str());
#endif
    }
	
  return cr;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ChargerChaine
//@
//@ Description : Permet de charger en gestion le contenu d'une zone
//@               memoire au format configuration
//@
//@ Parametres :
//@               (ENTREE) chaine
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__Configuration_ERR_MAUVAIS_ID : Mauvais
//@                                       identifiant de configuration
//@               CMN__ERR_PLUS_DE_MEMOIRE : Plus de memoire
//@               CMN__Configuration_ERR_AUCUN_CHARGEMENT : Aucun
//@                                                    element charge
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration::ChargerChaine(t_Chaine chaine)
{
  return CMN__Configuration_ChargerChaine(m_IdConfiguration,
					  (char *)chaine.c_str(),
					  chaine.length());
}


//@ ------------------------------------------------------------------*/
//@ Methode : CMN__Configuration_NombreDeSections
//@
//@ Description : Permet d'avoir le nombre de sections actuellement
//@               dans la gestion de configuration au format
//@               configuration
//@
//@ Parametres :
//@               (SORTIE) NombreSections : Nombre de sections
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__Configuration_ERR_MAUVAIS_ID : Mauvais
//@                                       identifiant de configuration
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut  CMN__Configuration::NombreDeSections(int *NombreSections)
{
  return CMN__Configuration_NombreDeSections(m_IdConfiguration,NombreSections);
}


//@ ------------------------------------------------------------------*/
//@ Methode : CMN__Configuration_LireSection
//@
//@ Description : Permet d'avoir le nom d'une section d'indice i
//@               actuellement dans la gestion de configuration
//@
//@ Parametres :
//@               (ENTREE) Indice
//@               (SORTIE) Section : Nom de la section
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__ERR_PLUS_DE_MEMOIRE : Plus de memoire
//@               CMN__Configuration_ERR_MAUVAIS_ID : Mauvais
//@                                       identifiant de configuration
//@               CMN__Configuration_ERR_MAUVAIS_INDICE : Mauvais
//@                                       indice de section
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration::LireSection(int Indice,
							    t_Chaine& Section)
{
  t_CMN__Configuration_Statut Statut;
  char *Valeur=0;

  Statut = CMN__Configuration_LireSection(m_IdConfiguration,Indice,&Valeur);
  if (Statut == CMN__STATUT_OK )
    {
      if (Valeur != 0)
	{
	  Section = Valeur;
	  FREECPP(Valeur);
	}
      else
	Statut  = CMN__STATUT_NOK;
    }
  
  return Statut;
}


//@ ------------------------------------------------------------------*/
//@ Methode : EcrireValeur
//@
//@ Description : Permet d'avoir le nom d'une section d'indice i
//@               actuellement dans la gestion de configuration
//@
//@ Parametres :
//@               (ENTREE) Section
//@               (ENTREE) Champ
//@               (ENTREE) Valeur
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__ERR_PLUS_DE_MEMOIRE : Plus de memoire
//@               CMN__Configuration_ERR_MAUVAIS_ID : Mauvais
//@                                       identifiant de configuration
//@               CMN__Configuration_ERR_MAUVAIS_INDICE : Mauvais
//@                                       indice de section
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration::EcrireValeur(t_Chaine Section,
							     t_Chaine Champ,
							     t_Chaine Valeur)
{
  return CMN__Configuration_EcrireValeur(m_IdConfiguration,
					 (char *)Section.c_str(),
					 (char *)Champ.c_str(),
					 (char *)Valeur.c_str());
}


//@ ------------------------------------------------------------------*/
//@ Methode : LireValeur
//@
//@ Description : Permet de lire la valeur d'un champ 
//@               dans une configuration en gestion
//@
//@ Parametres :
//@               (ENTREE) Section
//@               (ENTREE) Champ
//@               (SORTIE) Valeur
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__Configuration_ERR_MAUVAIS_ID : Mauvais
//@                                       identifiant de configuration
//@               CMN__ERR_PLUS_DE_MEMOIRE : Plus de memoire
//@               CMN__Configuration_ERR_SECTION_INEXISTANTE : Section
//@                                                        inexistante
//@               CMN__Configuration_ERR_CHAMP_INEXISTANT : Champ
//@                                                        inexistant
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration::LireValeur(t_Chaine Section,
							   t_Chaine Champ,
							   t_Chaine& Valeur)
{
  t_CMN__Configuration_Statut Statut;
  char *sValeur=0;
  
  Statut = CMN__Configuration_LireValeur(m_IdConfiguration,
					 (char *)Section.c_str(),
					 (char *)Champ.c_str(),&sValeur);
  if (Statut == CMN__STATUT_OK)
    {
      if (sValeur != 0)
	{
	  Valeur = sValeur;
	  FREECPP(sValeur);
	}
      else
	{
	  Statut  = CMN__STATUT_NOK;
	  Valeur = "";
	}
    } else 
      {
	Valeur = "";
      }
  
  return Statut;
}


//@ ------------------------------------------------------------------*/
//@ Methode : DetruireSection
//@
//@ Description : Permet de detruire une section et tous les champs
//@               associes dans une configuration en gestion
//@
//@ Parametres :
//@               (ENTREE) section : Nom de la section
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__Configuration_ERR_MAUVAIS_ID : Mauvais
//@                                       identifiant de configuration
//@               CMN__Configuration_ERR_SECTION_INEXISTANTE : Section
//@                                                        inexistante
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration::DetruireSection(t_Chaine section)
{
  return CMN__Configuration_DetruireSection(m_IdConfiguration,
					    (char *)section.c_str());
}


//@ ------------------------------------------------------------------*/
//@ Methode : DetruireChamp
//@
//@ Description : Permet de detruire un champ et la section si le
//@               champ etait le dernier dans une configuration en 
//@               gestion
//@
//@ Parametres :
//@               (ENTREE) Section : Nom de la section
//@               (ENTREE) Champ : Nom du champ
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__Configuration_ERR_MAUVAIS_ID : Mauvais
//@                                       identifiant de configuration
//@               CMN__Configuration_ERR_SECTION_INEXISTANTE : Section
//@                                                        inexistante
//@               CMN__Configuration_ERR_CHAMP_INEXISTANT : Champ
//@                                                        inexistant
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration::DetruireChamp(t_Chaine Section,
							      t_Chaine Champ)
{
  return CMN__Configuration_DetruireChamp(m_IdConfiguration,
					  (char *)Section.c_str(),
					  (char *)Champ.c_str());
}


//@ ------------------------------------------------------------------*/
//@ Methode : SauverChaine
//@
//@ Description : Permet de sauvegarder une configuration en gestion
//@               dans une zone memoire au format configuration
//@
//@ Parametres :
//@               (ENTREE) chaine
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__ERR_PLUS_DE_MEMOIRE : Plus de memoire
//@               CMN__Configuration_ERR_MAUVAIS_ID : Mauvais
//@                                       identifiant de configuration
//@               CMN__ERR_CREATION_FICHIER : Erreur de creation de 
//@                                           fichier
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration::SauverChaine(t_Chaine& chaine)
{
  t_CMN__Configuration_Statut Statut;
  char *Valeur=0;
  int Longueur=0;
  
  Statut = CMN__Configuration_SauverChaine(m_IdConfiguration, &Valeur, &Longueur);
  if (Statut == CMN__STATUT_OK)
    {
      if (Valeur != 0)
	{
	  chaine = Valeur;
	  FREECPP(Valeur);
	}
      else
	{
	  Statut  = CMN__STATUT_NOK;
	}
    }
  
  return Statut;
}


//@ ------------------------------------------------------------------*/
//@ Methode : SauverFichier
//@
//@ Description : Permet de sauvegarder une configuration en gestion
//@               dans un fichier au format configuration
//@
//@ Parametres :
//@               (ENTREE) Fichier : Fichier pour sauvegarde
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__Configuration_ERR_MAUVAIS_ID : Mauvais
//@                                       identifiant de configuration
//@               CMN__ERR_CREATION_FICHIER : Erreur de creation de
//@                                           fichier
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration::SauverFichier(t_Chaine Fichier)
{
  t_CMN__Statut cr;
  
  cr =  CMN__Configuration_SauverFichier(m_IdConfiguration,
					 (char *)Fichier.c_str());
  if (cr!=CMN__STATUT_OK)
    {
      printf("Erreur a l'ouverture du fichier %s \n",
	     (char *)Fichier.c_str());
      printf("Verifier l'existence du fichier et ses droits d'acces\n"); 
    }
  else
    {
      printf("ouverture reussie en ecriture du fichier %s \n",
	     (char *)Fichier.c_str());
    }
  
  return cr;
}


//@ ------------------------------------------------------------------*/
//@ Methode : AjouterFichier
//@
//@ Description : Permet de sauvegarder une configuration en gestion
//@               dans un fichier au format configuration dans un 
//@               fichier existant et a la fin
//@
//@ Parametres :
//@               (ENTREE) Fichier : Fichier pour sauvegarde
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__Configuration_ERR_MAUVAIS_ID : Mauvais
//@                                       identifiant de configuration
//@               CMN__ERR_CREATION_FICHIER : Erreur de creation de 
//@                                           fichier
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration::AjouterFichier(t_Chaine Fichier)
{
  return CMN__Configuration_AjouterFichier(m_IdConfiguration,
					   (char *)Fichier.c_str());
}
