static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/CMN__Configuration.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : CMN
//@ MODULE : CMN__Configuration
//@
//@ Commentaire :
//@    Ce module gere les fonctionnalites de gestion de fichier de 
//@    configuration pour la gestion des fichiers de configuration
//@       Ouvrir permet d'initialiser la gestion
//@       ChargerFichier permet de charger en memoire le contenu
//@                      d'un fichier de configuration
//@       ChargerChaine permet de charger en memoire le contenu
//@                      d'une chaine contenant la configuration
//@       NombreDeSections permet d'avoir le nombre de sections de
//@                        la conf
//@       LireSection permet de lire la valeur d'une section
//@       EcrireValeur permet d'ecrire une valeur d'un champ dans
//@                    une section
//@       LireValeur permet de lire la valeur d'un champ dans une
//@                  section
//@       DetruireSection permet de detruire une section avec tous
//@                       ses champs
//@       DetruireValeur permet de detruire un champ et la section
//@                      associee si s'etait le dernier champ
//@       SauverFichier permet de sauver le contenu de la memoire
//@                     de configuration dans un fichier
//@       AjouterFichier permet de sauver le contenu de la memoire
//@                      de configuration dans un fichier a la fin 
//@                      d'un fichier
//@       SauverChaine permet de sauver le contenu de la memoire
//@                    de configuration dans une chaine
//@       Fermer permet de terminer la gestion de configuration et
//@              liberer
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 10/03/2000 : Pierre-Louis Agnetti
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#ifdef DEBUG
#ifdef _PC
#include <assert.h>
#endif
#endif


#include "CMN__Commun.h"
#include "CMN__Trace.h"
#include "CMN__Configuration.h"


//====================================================================*/
// Declaration des define internes
//====================================================================*/

// Taille des realloc
#define CMN__CONFIGURATION_TAILLE_ALLOC 100

// Type de gestion des champs
typedef struct _t_CMN__Configuration_Champ {
   char *nom;
   char *valeur;
   struct _t_CMN__Configuration_Champ *suivant;
} t_CMN__Configuration_Champ;

// Type de gestion des sections
typedef struct _t_CMN__Configuration_Section {
   char *nom;
   t_CMN__Configuration_Champ *champs;
} t_CMN__Configuration_Section;


// Type chapeau de la gestion des configurations
typedef struct  _t_CMN__Configuration_Conf {
   struct _t_CMN__Configuration_Conf *self;
   t_CMN__Configuration_Section **tab_sections;
   int   dernier_acces;
   int   nombre_sections;
   int   nombre_total;
} t_CMN__Configuration_Conf;

#ifndef _PC
#define CMN__CONFIGURATION_MODE_OPEN_READ "r"
#else
#define CMN__CONFIGURATION_MODE_OPEN_READ "rb"
#endif

//======================================================================
// Declaration des fonctions internes
//======================================================================

// Controle la validite de l'identifiant de configuration
static t_CMN__Configuration_Conf*   CMN__Configuration_ControlerId(t_CMN__Configuration_Id IdConfiguration);

// Interprete un buffer memoire de configuration et cree les sections et champs
static t_CMN__Configuration_Statut  CMN__Configuration_Interpreter(t_CMN__Configuration_Conf *IdConf,char *memoire,int taille_memoire);

// Allocation d'un element de gestion section
static t_CMN__Configuration_Section*  CMN__Configuration_AllouerSection(t_CMN__Configuration_Conf *IdConf,char *section);

// Rechercher un element de gestion section
static t_CMN__Configuration_Section* CMN__Configuration_RechercherSection(t_CMN__Configuration_Conf  *IdConf,char *section);

// Rechercher un element de gestion section
static int CMN__Configuration_RechercherIndiceSection(t_CMN__Configuration_Conf  *IdConf,char *section);

// Rechercher un element de gestion champ
static t_CMN__Configuration_Champ* CMN__Configuration_RechercherChamp(t_CMN__Configuration_Section* section,char *champ);

// Allocation d'un element de gestion champ
static t_CMN__Configuration_Champ* CMN__Configuration_AllouerChamp(t_CMN__Configuration_Section* section,char *champ);

// Affecter une valeur a un champ alloue
static t_CMN__Configuration_Statut CMN__Configuration_AffecterValeur(t_CMN__Configuration_Champ* champ,char *valeur);

// Cherche un caractere significatif dans la memoire pointee
static char CMN__Configuration_ChercherCaractere(char **pt_courant,char *pt_fin);

// Calcul la longueur de la configuration (en ascii) de la 
// configuration en gestion
static int CMN__Configuration_CalculerLongueurConfiguration(t_CMN__Configuration_Conf *IdConf);

// Calcul la longueur d'un nom de section dans la memoire pointee
static int CMN__Configuration_CalculerLongueurSection(char *pt_courant,char *pt_fin);

// Pointe sur le caractere apres le CR dans la memoire pointee
static void CMN__Configuration_AllerApresCr(char **pt_courant,char *pt_fin);

// Calcul la longueur d'un nom d'un champ dans la memoire pointee
static int CMN__Configuration_CalculerLongueurChamp(char **pt_courant,char *pt_fin);

// Pointe sur le caractere apres le = dans la memoire pointee
static void CMN__Configuration_AllerApresEgal(char **pt_courant,char *pt_fin);

// Calcul la longueur d'une valeur d'un champ dans la memoire pointee
static int CMN__Configuration_CalculerLongueurValeur(char *pt_courant,char *pt_fin);

// Extrait le nom d'une section de la zone memoire pointee
static void CMN__Configuration_CopierChaineSection(char *pt_courant,char *pt_fin,char *chaine,int lg);

// Extrait le nom d'un champ de la zone memoire pointee
static void CMN__Configuration_CopierChaineChamp(char *pt_courant,char *pt_fin,char *chaine,int lg);

// Extrait la valeur d'un champ de la zone memoire pointee
static void CMN__Configuration_CopierChaineValeur(char *pt_courant,char *pt_fin,char *chaine,int lg);

// Recalcul la longueur pour enlever les blancs a la fin
static void CMN__Configuration_SoustraireBlancEnFin(char *pt_debut,int *lg);

// Trace les elements de gestion de la configuration en cours
static void CMN__Configuration_DumpTrace(t_CMN__Configuration_Conf* conf);

// Ecriture dans un fichier de la configuration memoire
static t_CMN__Configuration_Statut CMN__Configuration_SauverFichierMode(t_CMN__Configuration_Id IdConfiguration,char *fichier,char *mode);

// Enleve les blancs devant et derriere une chaine
static void CMN__Configuration_SansBlancDevantDerriere(char *in,char *out);

// Test si la chaine est vide
static t_CMN__Configuration_Statut CMN__Configuration_TestVide(char *valeur);

// Fusion des sections apres chargement
static void CMN__Configuration_FusionnerSections(t_CMN__Configuration_Conf *IdConf);

// Comparaison de sections
static int CMN__Configuration_CompareSections(t_CMN__Configuration_Section** sec1,t_CMN__Configuration_Section** sec2);

// Transfert de valeurs de champs
static void CMN__Configuration_TransfertSection(t_CMN__Configuration_Conf *IdConf,
                t_CMN__Configuration_Section* pt_section1,
                t_CMN__Configuration_Section* pt_section2);


//======================================================================
// Declaration des variables globales
//======================================================================


//======================================================================
//                                      FONCTIONS EXTERNES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_Ouvrir
//@
//@ Description : Initialise une configuration en rendant un
//@               identifiant.
//@               Cet identifiant sera utilise sur toutes les
//@               procedures
//@
//@ Parametres :
//@               (SORTIE) IdConfiguration : Identifiant de 
//@                                          configuration
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__ERR_PLUS_DE_MEMOIRE : Plus de memoire
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration_Ouvrir(t_CMN__Configuration_Id *IdConfiguration)
{
  t_CMN__Configuration_Statut cr=CMN__STATUT_OK;
  t_CMN__Configuration_Conf *IdConf=NULL;


  TRDEBUT("CMN__Configuration_Ouvrir");
  
  //@ Allouer une structure de gestion
  IdConf=(t_CMN__Configuration_Conf *)MALLOC(sizeof(t_CMN__Configuration_Conf));
  
  TR1(99,"Id conf:0x%x",IdConf);

  
  //@ SI l'allocation est en echec
  if (IdConf==NULL)
    {
      //@ sortir en erreur
      TRERR(CMN__ERR_PLUS_DE_MEMOIRE);
      TRFIN("CMN__Configuration_Ouvrir");
      return CMN__ERR_PLUS_DE_MEMOIRE;
   }
  //@ FIN SI

  //@ Initialiser la structure
  IdConf->tab_sections=(t_CMN__Configuration_Section**)MALLOC( CMN__CONFIGURATION_TAILLE_ALLOC * sizeof(t_CMN__Configuration_Section**));
  IdConf->dernier_acces=0;
  IdConf->nombre_sections=0;
  IdConf->nombre_total=CMN__CONFIGURATION_TAILLE_ALLOC;
  IdConf->self=IdConf;
  
  //@ Positionner l'identifiant en retour
  *IdConfiguration=(t_CMN__Configuration_Id)IdConf;
  
  
  TRFIN("CMN__Configuration_Ouvrir");
  
  return cr;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_ChargerFichier
//@
//@ Description : Permet de charger en gestion le contenu d'un
//@               fichier au format configuration
//@
//@ Parametres :
//@               (ENTREE) IdConfiguration : Identifiant de 
//@                                          configuration
//@               (ENTREE) fichier : Nom du fichier
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__CONFIGURATION_ERR_MAUVAIS_ID : Mauvais
//@                                      identifiant de configuration
//@               CMN__ERR_PLUS_DE_MEMOIRE : Plus de memoire
//@               CMN__ERR_FICHIER_INTROUVABLE : Fichier introuvable
//@               CMN__CONFIGURATION_ERR_AUCUN_CHARGEMENT : Aucun 
//@                                                     element charge
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration_ChargerFichier(t_CMN__Configuration_Id IdConfiguration,char *fichier)
{
  // Pointeur de gestion de configuration
  t_CMN__Configuration_Conf*   IdConf=NULL; 

  // Compte-rendu de sortie
  t_CMN__Configuration_Statut  cr=CMN__STATUT_OK; 
  
  // Compte-rendu de sortie de l'interpreteur
  t_CMN__Configuration_Statut  cr_inter=CMN__STATUT_OK;

  // Espace memoire de travail
  char  *fichier_en_memoire=NULL;  
  
  // Fichier de configuration ouvert
  FILE*   fichier_conf=NULL;

  // Struture d'information sur le fichier ouvert
  struct stat st; 

  // Compte_rendu information fichier
  int crst=0;

  // Taille du fichier ouvert
  int taille_fichier=0;


  TRDEBUT("CMN__Configuration_ChargerFichier");

  //@ Controler l'identifiant
  IdConf=CMN__Configuration_ControlerId(IdConfiguration);

  //@ Si mauvais identifiant sortir en erreur
  if (IdConf==NULL)
    {
      TRFIN("CMN__Configuration_ChargerFichier");
      return CMN__CONFIGURATION_ERR_MAUVAIS_ID;
    }
  
  //@ Ouvrir le fichier
  fichier_conf=fopen(fichier,CMN__CONFIGURATION_MODE_OPEN_READ);
  
  //@ S'il y a un probleme a l'ouverture
  if (fichier_conf==NULL)
    {
      //@ sortir en erreur
      TRERR(CMN__ERR_FICHIER_INTROUVABLE);
      TRFIN("CMN__Configuration_ChargerFichier");
      return CMN__ERR_FICHIER_INTROUVABLE;
    }
  
  //@ Lire la taille du fichier
  crst=fstat(fileno(fichier_conf),&st);
  
  //@ Si erreur sur la lecture d'information fichier sortir en erreur
  if (crst!=0)
    {
      fclose(fichier_conf);
      TRERR(CMN__ERR_FICHIER_INTROUVABLE);
      TRFIN("CMN__Configuration_ChargerFichier");
      return CMN__ERR_FICHIER_INTROUVABLE;
    }
  
  //@ Recuperer la taille du fichier
  taille_fichier=st.st_size;
  
  TR1(99,"Taille fichier:%d",taille_fichier);
  
  //@ Si le fichier est vide sortir en erreur
  if (taille_fichier<=1)
    {
      fclose(fichier_conf);
      TRFIN("CMN__Configuration_ChargerFichier");
      return CMN__CONFIGURATION_ERR_AUCUN_CHARGEMENT;
   }
  
  //@ Allouer une zone memoire de la taille du fichier
  fichier_en_memoire=(char*)MALLOC(taille_fichier);
  
  //@ Si plus de memoire sortir en erreur
  if (fichier_en_memoire==NULL)
    {
      fclose(fichier_conf);
      TRERR(CMN__ERR_PLUS_DE_MEMOIRE);
      TRFIN("CMN__Configuration_ChargerFichier");
      return CMN__ERR_PLUS_DE_MEMOIRE;
      
    }
  
  //@ Charger le fichier dans la memoire
  fread(fichier_en_memoire,taille_fichier,1,fichier_conf);
  
  //@ Appeler l'interpreteur sur la zone memoire
  cr_inter=CMN__Configuration_Interpreter(IdConf,
                                          fichier_en_memoire,
                                          taille_fichier);
  
  //@ Fusionner les sections rajoutees si necessaire si il
  //@ y a doublon
  if (cr_inter == CMN__STATUT_OK)
    {
      CMN__Configuration_FusionnerSections(IdConf);
    }
  
  
  //@ Fermer le fichier
  fclose(fichier_conf);
  
  
  //@ Liberer la memoire fichier
  FREE(fichier_en_memoire);
  
  
  //@ Trace en debug
  CMN__Configuration_DumpTrace(IdConf);
  
  TRFIN("CMN__Configuration_ChargerFichier");
  
  return cr_inter;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_ChargerChaine
//@
//@ Description : Permet de charger en gestion le contenu d'une zone
//@               memoire au format configuration
//@
//@ Parametres :
//@               (ENTREE) IdConfiguration : Identifiant de
//@                                          configuration
//@               (ENTREE) chaine : Pointeur de memoire
//@               (ENTREE) longueur : Longueur de la memoire
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__CONFIGURATION_ERR_MAUVAIS_ID : Mauvais
//@                                       identifiant de configuration
//@               CMN__ERR_PLUS_DE_MEMOIRE : Plus de memoire
//@               CMN__CONFIGURATION_ERR_AUCUN_CHARGEMENT : Aucun
//@                                                     element charge
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration_ChargerChaine(t_CMN__Configuration_Id IdConfiguration,
                                                             char *chaine,
                                                             int longueur)
{
  // Pointeur de gestion de configuration
  t_CMN__Configuration_Conf*   IdConf=NULL;

  // Compte-rendu de sortie de l'interpreteur
  t_CMN__Configuration_Statut  cr_inter=CMN__STATUT_OK;
  
  
  TRDEBUT("CMN__Configuration_ChargerChaine");
  
  //@ Controler l'identifiant
  IdConf=CMN__Configuration_ControlerId(IdConfiguration);
  
  //@ Si mauvais identifiant sortir en erreur
  if (IdConf==NULL)
    {
      TRFIN("CMN__Configuration_ChargerChaine");
      return CMN__CONFIGURATION_ERR_MAUVAIS_ID;
    }
  
  //@ Appeler l'interpreteur sur la zone memoire
  cr_inter=CMN__Configuration_Interpreter(IdConf,chaine,longueur);
  
  //@ Fusionner les sections rajoutees si necessaire si il y a doublon
  if (cr_inter == CMN__STATUT_OK)
    {
      CMN__Configuration_FusionnerSections(IdConf);
    }
  
  //@ Trace en debug
  CMN__Configuration_DumpTrace(IdConf);

  TRFIN("CMN__Configuration_ChargerChaine");
  
  return cr_inter;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_NombreDeSections
//@
//@ Description : Permet d'avoir le nombre de sections actuellement 
//@               dans la gestion de configuration au format 
//@               configuration
//@
//@ Parametres :
//@               (ENTREE) IdConfiguration : Identifiant de 
//@                                          configuration
//@               (SORTIE) nombre : Nombre de sections
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__CONFIGURATION_ERR_MAUVAIS_ID : Mauvais 
//@                                       identifiant de configuration
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration_NombreDeSections(t_CMN__Configuration_Id IdConfiguration,
                                                                 int *nombre)
{
  // Pointeur de gestion de configuration
  t_CMN__Configuration_Conf*   IdConf=NULL; 
 
  // Compte-rendu de sortie
  t_CMN__Configuration_Statut  cr=CMN__STATUT_OK;

  // Pointeur sur les sections
  t_CMN__Configuration_Section* pt_section=NULL;

  // Nombre de sections
  int nb_sections=0;
  
  
  TRDEBUT("CMN__Configuration_NombreDeSections");
  
  *nombre=0;
  
  //@ Controler l'identifiant
  IdConf=CMN__Configuration_ControlerId(IdConfiguration);
  
  //@ Si mauvais identifiant sortir en erreur
  if (IdConf==NULL)
    {
       TRFIN("CMN__Configuration_NombreDeSections");
       
       return CMN__CONFIGURATION_ERR_MAUVAIS_ID;
    }
  
  //@ Sortir avec le nombre
  *nombre=IdConf->nombre_sections;

  TR1(99,"Nombre de sections:%d",nb_sections);
  TRFIN("CMN__Configuration_NombreDeSections");

  return cr;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_LireSection
//@
//@ Description : Permet d'avoir le nom d'une section d'indice i
//@               actuellement dans la gestion de configuration
//@
//@ Parametres :
//@               (ENTREE) IdConfiguration : Identifiant de 
//@                                          configuration
//@               (ENTREE) indice : Indice (0 -> n)
//@               (SORTIE) valeur : Nom de la section
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__ERR_PLUS_DE_MEMOIRE : Plus de memoire
//@               CMN__CONFIGURATION_ERR_MAUVAIS_ID : Mauvais 
//@                                       identifiant de configuration
//@               CMN__CONFIGURATION_ERR_MAUVAIS_INDICE : Mauvais
//@                                       indice de section
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration_LireSection(t_CMN__Configuration_Id IdConfiguration,
                                                           int indice,
                                                           char **valeur)
{
  // Pointeur de gestion de configuration
  t_CMN__Configuration_Conf*   IdConf=NULL; 

  // Compte-rendu de sortie
  t_CMN__Configuration_Statut  cr=CMN__STATUT_OK; 

  // Pointeur sur les sections
  t_CMN__Configuration_Section* pt_section=NULL;  

  // Compteur de section
  int cpt_section=0; 

  int lg=0;
  
  
  TRDEBUT("CMN__Configuration_LireSection");
  
  *valeur=NULL;
  
  //@ Controler l'identifiant
  IdConf=CMN__Configuration_ControlerId(IdConfiguration);
  
  //@ Si mauvais identifiant sortir en erreur
  if (IdConf==NULL)
    {
      TRFIN("CMN__Configuration_LireSection");
  
      return CMN__CONFIGURATION_ERR_MAUVAIS_ID;
    }
  
  //@ Si indice hors bornes sortir en erreur
  if ( (indice < 0) || (indice >= IdConf->nombre_sections) )
    {
      TRERR(CMN__CONFIGURATION_ERR_MAUVAIS_INDICE);
      TRFIN("CMN__Configuration_LireSection");
      return CMN__CONFIGURATION_ERR_MAUVAIS_INDICE;
    }

  //@ Se positionner sur la section
  pt_section=IdConf->tab_sections[indice];
  IdConf->dernier_acces=indice;

  //@ Allouer une taille pour le retour
  lg=strlen(pt_section->nom)+1;
  *valeur=(char*)MALLOC(lg);

  //@ SI plus de memoire positionner le code erreur
  if (*valeur==NULL)
    {
      cr=CMN__ERR_PLUS_DE_MEMOIRE;
    }
  //@ SINON
  else
    {
      //@ Affecter le nom de la section
      strncpy(*valeur,pt_section->nom,lg);

      //@ Positionner le code retour OK
      cr=CMN__STATUT_OK;
    }
  //@ FIN SI
  
  //@ Sortir avec le compte-rendu adapte
  TRFIN("CMN__Configuration_LireSection");
  
  return cr;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_EcrireValeur
//@
//@ Description : Permet d'ecrire la valeur d'un champ dans une
//@               section dans une configuration en gestion.
//@               Si le champ n'existe pas il est cree, sinon il est
//@               mis a jour.
//@               Si la section n'existe pas, elle est creee
//@
//@ Parametres :
//@               (ENTREE) IdConfiguration : Identifiant de
//@                                          configuration
//@               (ENTREE) section : Nom de la section
//@               (ENTREE) champ : Nom du champ
//@               (ENTREE) valeur : valeur du champ
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__ERR_PLUS_DE_MEMOIRE : Plus de memoire
//@               CMN__CONFIGURATION_ERR_MAUVAIS_ID : Mauvais 
//@                                       identifiant de configuration
//@               CMN__ERR_PARAMETRE_NULL : Section ou champ null ou
//@                                         vide
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration_EcrireValeur(t_CMN__Configuration_Id IdConfiguration,
                                                            char *section,
                                                            char *champ,
                                                            char *valeur)
{
  // Pointeur de gestion de configuration
  t_CMN__Configuration_Conf*   IdConf=NULL;

  // Pointeur sur les sections
  t_CMN__Configuration_Section* pt_section=NULL;

  // Compte-rendu de sortie
  t_CMN__Configuration_Statut  cr=CMN__STATUT_OK; 

  // Pointeur sur les champs
  t_CMN__Configuration_Champ* pt_champ=NULL;  

  // Memoire pour stocker la valeur
  char  *valeur_allouee=NULL; 

  int lg=0;
  
  
  TRDEBUT("CMN__Configuration_EcrireValeur");

  //@ Controler l'identifiant
  IdConf=CMN__Configuration_ControlerId(IdConfiguration);
  
  //@ Si mauvais identifiant sortir en erreur
  if (IdConf==NULL)
    {
      TRFIN("CMN__Configuration_EcrireValeur");
      
      return CMN__CONFIGURATION_ERR_MAUVAIS_ID;
    }

  if ( (!section) || (!champ) ) 
    {
      TRFIN("CMN__Configuration_EcrireValeur");
#ifdef DEBUG
#ifdef _PC
      assert(0);
#endif
#endif
      return CMN__ERR_PARAMETRE_NULL;
    }

  if ( (CMN__Configuration_TestVide(section) != CMN__STATUT_OK) ||
       (CMN__Configuration_TestVide(champ) != CMN__STATUT_OK) )
    {
      TRFIN("CMN__Configuration_EcrireValeur");
#ifdef DEBUG
#ifdef _PC
      assert(0);
#endif
#endif
      return CMN__ERR_PARAMETRE_NULL;
    }

  //@ Rechercher la section avec le nom donne en parametre
  pt_section=CMN__Configuration_RechercherSection(IdConf,section);

  //@ SI il n'y a pas de section avec ce nom
  if (pt_section==NULL)
    {
      //@ Creer la section
      pt_section=CMN__Configuration_AllouerSection(IdConf,section);

      //@ Si probleme de memoire sortir en erreur
      if (pt_section==NULL)
        {
          TRFIN("CMN__Configuration_EcrireValeur");
          return CMN__ERR_PLUS_DE_MEMOIRE;
        }
    }
  //@ FIN SI

  //@ Allouer une memoire pour la valeur
  lg=strlen(valeur)+1;
  valeur_allouee=(char*)MALLOC(lg);

  //@ Si probleme de memoire sortir en erreur
  if (valeur_allouee==NULL)
    {
      TRERR(CMN__ERR_PLUS_DE_MEMOIRE);
      TRFIN("CMN__Configuration_EcrireValeur");
      return CMN__ERR_PLUS_DE_MEMOIRE;
    }

  //@ Affecter la valeur
  strncpy(valeur_allouee,valeur,lg);

  //@ Rechercher le champ dans la section avec le nom donne
  //@ en parametre
  pt_champ=CMN__Configuration_RechercherChamp(pt_section,champ);

  //@ SI il n'y a pas de champ avec ce nom
  if (pt_champ==NULL)
    {
      //@ Creer le champ
      pt_champ=CMN__Configuration_AllouerChamp(pt_section,champ);

      //@ Si probleme de memoire sortir en erreur
      if (pt_champ==NULL)
        {
          TRFIN("CMN__Configuration_EcrireValeur");
          return CMN__ERR_PLUS_DE_MEMOIRE;
        }
    }
  //@ SINON le champ existe deja
  else
    {
      //@ Liberer l'ancienne valeur
      FREE(pt_champ->valeur);
    }
  //@ FIN SI

  //@ Affecter le (nouveau) pointeur
  pt_champ->valeur=valeur_allouee;

  //@ Trace en debug
  CMN__Configuration_DumpTrace(IdConf);

  TRFIN("CMN__Configuration_EcrireValeur");

  return cr;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_LireValeur
//@
//@ Description : Permet de lire la valeur d'un champ dans une 
//@               configuration en gestion
//@
//@ Parametres :
//@               (ENTREE) IdConfiguration : Identifiant de 
//@                                          configuration
//@               (ENTREE) section : Nom de la section
//@               (ENTREE) champ : Nom du champ
//@               (SORTIE) valeur : valeur du champ
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__CONFIGURATION_ERR_MAUVAIS_ID : Mauvais 
//@                                       identifiant de configuration
//@               CMN__ERR_PLUS_DE_MEMOIRE : Plus de memoire
//@               CMN__CONFIGURATION_ERR_SECTION_INEXISTANTE : Section
//@                                                        inexistante
//@               CMN__CONFIGURATION_ERR_CHAMP_INEXISTANT : Champ
//@                                                        inexistant
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration_LireValeur(t_CMN__Configuration_Id IdConfiguration,
                                                          char *section,
                                                          char *champ,
                                                          char **valeur)
{
  // Pointeur de gestion de configuration
  t_CMN__Configuration_Conf*   IdConf=NULL;
  
  // Pointeur sur les sections
  t_CMN__Configuration_Section* pt_section=NULL;

  // Compte-rendu de sortie
  t_CMN__Configuration_Statut  cr=CMN__STATUT_OK;

  // Pointeur sur les champs
  t_CMN__Configuration_Champ* pt_champ=NULL; 

  int lg=0;


  TRDEBUT("CMN__Configuration_LireValeur");

  *valeur=NULL;

  //@ Controler l'identifiant
  IdConf=CMN__Configuration_ControlerId(IdConfiguration);

  //@ Si mauvais identifiant sortir en erreur
  if (IdConf==NULL)
    {
      TRFIN("CMN__Configuration_LireValeur");
      return CMN__CONFIGURATION_ERR_MAUVAIS_ID;
    }

  //@ Rechercher la section avec le nom donne en parametre
  pt_section=CMN__Configuration_RechercherSection(IdConf,section);

  //@ S'il n'y a pas de section avec ce nom
  if (pt_section==NULL)
    {
      //@ rendre section inexistante
      TRERR(CMN__CONFIGURATION_ERR_SECTION_INEXISTANTE);
      TRFIN("CMN__Configuration_LireValeur");

      return CMN__CONFIGURATION_ERR_SECTION_INEXISTANTE;
    }

  //@ Rechercher le champ dans la section avec le nom donne
  //@ en parametre
  pt_champ=CMN__Configuration_RechercherChamp(pt_section,champ);

  //@ S'il n'y a pas de champ avec ce nom
  if (pt_champ==NULL)
    {
      //@ rendre champ inexistant
      TRERR(CMN__CONFIGURATION_ERR_CHAMP_INEXISTANT);
      TRFIN("CMN__Configuration_LireValeur");

      return CMN__CONFIGURATION_ERR_CHAMP_INEXISTANT;
    }

  //@ Allouer une taille pour le retour
  lg=strlen(pt_champ->valeur)+1;
  *valeur=(char*)MALLOC(lg);

  //@ SI plus de memoire positionner le code erreur
  if (*valeur==NULL)
    {
      cr=CMN__ERR_PLUS_DE_MEMOIRE;
    }
  //@ SINON copier la valeur
  else
    {
      strncpy(*valeur,pt_champ->valeur,lg);
    }
  //@ FIN SI

  TRFIN("CMN__Configuration_LireValeur");

  return cr;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_DetruireSection
//@
//@ Description : Permet de detruire une section et tous les champs 
//@               associes dans une configuration en gestion
//@
//@ Parametres :
//@               (ENTREE) IdConfiguration : Identifiant de
//@                                          configuration
//@               (ENTREE) section : Nom de la section
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__CONFIGURATION_ERR_MAUVAIS_ID : Mauvais 
//@                                       identifiant de configuration
//@               CMN__CONFIGURATION_ERR_SECTION_INEXISTANTE : Section
//@                                                        inexistante
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration_DetruireSection(t_CMN__Configuration_Id IdConfiguration,
                                                               char *section)
{
  // Pointeur de gestion de configuration
  t_CMN__Configuration_Conf*   IdConf=NULL;

  // Pointeur sur les sections
  t_CMN__Configuration_Section* pt_section=NULL;

  int indice=0;

  // Compte-rendu de sortie
  t_CMN__Configuration_Statut  cr=CMN__STATUT_OK; 

  // Pointeur sur les champs
  t_CMN__Configuration_Champ* pt_champ=NULL;

  // Pointeur sur les champs avant un free
  t_CMN__Configuration_Champ* pt_champ_free=NULL; 

  int nb=0;
  int dernier=0;

  TRDEBUT("CMN__Configuration_DetruireSection");


  TR1(99,"Detruire section:%s",section);

  //@ Controler l'identifiant
  IdConf=CMN__Configuration_ControlerId(IdConfiguration);

  //@ Si mauvais identifiant sortir en erreur
  if (IdConf==NULL)
    {
      TRFIN("CMN__Configuration_DetruireSection");
     
      return CMN__CONFIGURATION_ERR_MAUVAIS_ID;
    }

  //@ S'il n'y a pas de section avec ce nom
  if (section == NULL)
    {
      //@ rendre section inexistante
      TR0(99,"Fin detruire section");
      TRERR(CMN__CONFIGURATION_ERR_SECTION_INEXISTANTE);
      TRFIN("CMN__Configuration_DetruireSection");
      
      return CMN__CONFIGURATION_ERR_SECTION_INEXISTANTE;
    }

  //@ Rechercher la section avec le nom donne en parametre
  indice=CMN__Configuration_RechercherIndiceSection(IdConf,section);

  //@ S'il n'y a pas de section
  if (indice<0)
    {
      //@ rendre section inexistante
      TR0(99,"Fin detruire section");
      TRERR(CMN__CONFIGURATION_ERR_SECTION_INEXISTANTE);
      TRFIN("CMN__Configuration_DetruireSection");
      
      return CMN__CONFIGURATION_ERR_SECTION_INEXISTANTE;
    }

  pt_section=IdConf->tab_sections[indice];

  nb=IdConf->nombre_sections;

  //@ Retailler la table
  if ( indice != (IdConf->nombre_sections - 1) )
    {
      memmove(&(IdConf->tab_sections[indice]),
              &(IdConf->tab_sections[indice+1]),
              (IdConf->nombre_sections - indice - 1) * sizeof(t_CMN__Configuration_Section**));
    }

  
  //@ Liberer la chaine champs
  pt_champ=pt_section->champs;
  while (pt_champ)
    {
      TR1(99,"Liberer champ:%s",pt_champ->nom);
      FREE(pt_champ->nom);
      FREE(pt_champ->valeur);
      pt_champ_free=pt_champ;
      pt_champ=pt_champ->suivant;
      FREE(pt_champ_free);
    }

  //@ Liberer la section
  FREE(pt_section->nom);

  dernier=IdConf->dernier_acces;

  if (IdConf->tab_sections[IdConf->dernier_acces] == pt_section)
    {
      if (indice == (IdConf->nombre_sections - 1) )
        {
          IdConf->dernier_acces = 0;
        }
      else
        {
          IdConf->dernier_acces = indice;
        }
    }
  else
    {
      if (IdConf->dernier_acces > indice)
        {
          IdConf->dernier_acces--;
        }
    }

  FREE(pt_section);

  //@ Actualiser le nombre de sections
  IdConf->nombre_sections--;

  //@ Trace en debug
  CMN__Configuration_DumpTrace(IdConf);

  TR0(99,"Fin detruire section");

  TRFIN("CMN__Configuration_DetruireSection");

  return cr;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_DetruireChamp
//@
//@ Description : Permet de detruire un champ et la section si le
//@               champ etait le dernier dans une configuration en
//@               gestion
//@
//@ Parametres :
//@               (ENTREE) IdConfiguration : Identifiant de
//@                                          configuration
//@               (ENTREE) section : Nom de la section
//@               (ENTREE) champ : Nom du champ
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__CONFIGURATION_ERR_MAUVAIS_ID : Mauvais 
//@                                       identifiant de configuration
//@               CMN__CONFIGURATION_ERR_SECTION_INEXISTANTE : Section
//@                                                        inexistante
//@               CMN__CONFIGURATION_ERR_CHAMP_INEXISTANT : Champ
//@                                                        inexistant
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration_DetruireChamp(t_CMN__Configuration_Id IdConfiguration,
                                                             char *section,
                                                             char *champ)
{
  // Pointeur de gestion de configuration
  t_CMN__Configuration_Conf*   IdConf=NULL;

  // Pointeur sur les sections
  t_CMN__Configuration_Section* pt_section=NULL;

  // Compte-rendu de sortie
  t_CMN__Configuration_Statut  cr=CMN__STATUT_OK;

  // Pointeur sur les champs
  t_CMN__Configuration_Champ* pt_champ=NULL;  

  // Pointeur sur le champ avant
  t_CMN__Configuration_Champ* pt_champ_next=NULL; 

  
  TRDEBUT("CMN__Configuration_DetruireChamp");
  TR2(99,"Detruire champ:%s de section:%s",champ,section);

  //@ Controler l'identifiant
  IdConf=CMN__Configuration_ControlerId(IdConfiguration);

  //@ Si mauvais identifiant sortir en erreur
  if (IdConf==NULL)
    {
      TRFIN("CMN__Configuration_DetruireChamp");
      return CMN__CONFIGURATION_ERR_MAUVAIS_ID;
    }

  //@ Rechercher la section avec le nom donne en parametre
  pt_section=CMN__Configuration_RechercherSection(IdConf,section);

  //@ S'il n'y a pas de section avec ce nom
  if (pt_section==NULL)
    {
      //@ rendre section inexistante
      TRERR(CMN__CONFIGURATION_ERR_SECTION_INEXISTANTE);
      TRFIN("CMN__Configuration_DetruireChamp");
      return CMN__CONFIGURATION_ERR_SECTION_INEXISTANTE;
    }

  //@ Positionner le code retour a section inexistante par defaut
  cr=CMN__CONFIGURATION_ERR_CHAMP_INEXISTANT;

  pt_champ=pt_section->champs;

  //@ TANT QUE il y a une section faire
  while (pt_champ)
    {
      //@ SI l'element courant a le meme nom que le champ en parametre
      if (pt_champ->nom && (strcmp(pt_champ->nom,champ)==0))
        {
          //@ refaire les chainages des champ
          if (pt_champ==pt_section->champs)
            {
              pt_section->champs=pt_champ->suivant;
            }
          else
            {
              pt_champ_next->suivant=pt_champ->suivant;
            }
          TR1(99,"Detruire champ:%s",pt_champ->nom);

          //@ liberer le champ
          FREE(pt_champ->nom);
          FREE(pt_champ->valeur);
          FREE(pt_champ);

          //@ si c'etait le dernier, liberer la section
          if (pt_section->champs==NULL)
            {
              CMN__Configuration_DetruireSection(IdConfiguration,section);
            }

          //@ positionner le code retour
          cr=CMN__STATUT_OK;
          pt_champ=NULL;
        }
      else
        {
          pt_champ_next=pt_champ;
          pt_champ=pt_champ->suivant;
        }
      //@ FIN SI
    }
  //@ FIN TANT QUE

  //@ Trace en debug
  CMN__Configuration_DumpTrace(IdConf);

  TR0(99,"Fin detruire champ");
  TRFIN("CMN__Configuration_DetruireChamp");

  return cr;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_SauverChaine
//@
//@ Description : Permet de sauvegarder une configuration en gestion
//@               dans une zone memoire au format configuration
//@
//@ Parametres :
//@               (ENTREE) IdConfiguration : Identifiant de
//@                                          configuration
//@               (ENTREE/SORTIE) chaine : Pointeur zone memoire pour
//@                                        allocation
//@               (ENTREE/SORTIE) longueur : Longueur de la zone 
//@                                          memoire
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__ERR_PLUS_DE_MEMOIRE : Plus de memoire
//@               CMN__CONFIGURATION_ERR_MAUVAIS_ID : Mauvais 
//@                                       identifiant de configuration
//@               CMN__ERR_CREATION_FICHIER : Erreur de creation de
//@                                           fichier
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration_SauverChaine(t_CMN__Configuration_Id IdConfiguration,
                                                            char **chaine,
                                                            int *longueur)
{
  // Pointeur de gestion de configuration
  t_CMN__Configuration_Conf*   IdConf=NULL; 

  // Compte-rendu de sortie
  t_CMN__Configuration_Statut  cr=CMN__STATUT_OK;

  // Pointeur sur les sections
  t_CMN__Configuration_Section* pt_section=NULL;

  // Pointeur sur les champs
  t_CMN__Configuration_Champ* pt_champ=NULL; 

  // Pointeur pour sauvegarde
  char *pt_chaine=NULL;     

  // Indice
  int ind=0;

  int lgspr=0;
  int decal=0;

  TRDEBUT("CMN__Configuration_SauverChaine");
  *chaine=0;
  *longueur=0;

  //@ Controler l'identifiant
  IdConf=CMN__Configuration_ControlerId(IdConfiguration);

  //@ Si mauvais identifiant sortir en erreur
  if (IdConf==NULL)
    {
      TRFIN("CMN__Configuration_SauverChaine");
      return CMN__CONFIGURATION_ERR_MAUVAIS_ID;
    }


  *longueur=CMN__Configuration_CalculerLongueurConfiguration(IdConf);

  //@ SI il y a des choses a sauver
  if (*longueur)
    {
      *longueur=*longueur+1;

      //@ Allouer une zone memoire pour le retour
      *chaine=(char*)MALLOC(*longueur);
      lgspr=*longueur;

      //@ SI plus de memoire sortir en erreur
      if (*chaine == NULL )
        {
          *longueur=0;
          cr=CMN__ERR_PLUS_DE_MEMOIRE;
        }
      //@ SINON
      else
        {
          //@ POUR toutes les sections
          pt_chaine=*chaine;

         
          for (ind=0;ind<IdConf->nombre_sections;ind++)
            {
              pt_section=IdConf->tab_sections[ind];
              TR1(99,"Section:\"%s\"",pt_section->nom);
              pt_champ=pt_section->champs;

              //@ sauver le nom de section
              sprintf(pt_chaine,"[%s]\n",pt_section->nom);
              decal=strlen(pt_section->nom)+3; // 3 pour [ ] et \n
              pt_chaine=pt_chaine + decal ;
              lgspr=lgspr - decal;
              if (pt_champ==NULL)
                {
                  TR0(99,"Pas de champ");
                }
              else
                {
                  //@ POUR tous les champs
                  while(pt_champ)
                    {
                      TR2(99,"Champ:\"%s\" valeur:\"%s\"",
                          pt_champ->nom,pt_champ->valeur);

                      //@ sauver le nom de champ et la valeur
                      sprintf(pt_chaine,"%s=%s\n",pt_champ->nom,pt_champ->valeur);

                      decal=strlen(pt_champ->nom)+
                        strlen(pt_champ->valeur)+2; // 2 pour = et \n

                      pt_chaine=pt_chaine + decal;
                      lgspr=lgspr - decal;
                      pt_champ=pt_champ->suivant;
                    }
                  //@ FIN POUR
                }
            }
          //@ FIN POUR
          
          *pt_chaine=0;
        }
      //@ FIN SI
    }
  else
    {
      *longueur=1;

      //@ Allouer une zone memoire pour le retour
      *chaine=(char*)MALLOC(*longueur);
      **chaine=0;

    }
  //@ FIN SI

  
  TRFIN("CMN__Configuration_SauverChaine");
  
  return cr;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_SauverFichier
//@
//@ Description : Permet de sauvegarder une configuration en gestion
//@               dans un fichier au format configuration
//@
//@ Parametres :
//@               (ENTREE) IdConfiguration : Identifiant de
//@                                          configuration
//@               (ENTREE) fichier : Fichier pour sauvegarde
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__CONFIGURATION_ERR_MAUVAIS_ID : Mauvais 
//@                                       identifiant de configuration
//@               CMN__ERR_CREATION_FICHIER : Erreur de creation de
//@                                           fichier
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration_SauverFichier(t_CMN__Configuration_Id IdConfiguration,
                                                             char *fichier)
{
  // FT-2765
  return CMN__Configuration_SauverFichierMode(IdConfiguration,fichier,static_cast<char *>("w"));
  // ...FT-2765
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_AjouterFichier
//@
//@ Description : Permet de sauvegarder une configuration en gestion
//@               dans un fichier au format configuration en ajout 
//@               sur un fichier existant
//@
//@ Parametres :
//@               (ENTREE) IdConfiguration : Identifiant de
//@                                          configuration
//@               (ENTREE) fichier : Fichier pour sauvegarde
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__CONFIGURATION_ERR_MAUVAIS_ID : Mauvais 
//@                                       identifiant de configuration
//@               CMN__ERR_CREATION_FICHIER : Erreur de creation de
//@                                           fichier
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration_AjouterFichier(t_CMN__Configuration_Id IdConfiguration,
                                                              char *fichier)
{
  // FT-2765
  return CMN__Configuration_SauverFichierMode(IdConfiguration,fichier,static_cast<char *>("a"));
  // ...FT-2765
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_Fermer
//@
//@ Description : Permet de liberer une configuration en gestion.
//@ Parametres :
//@               (ENTREE) IdConfiguration : Identifiant de
//@                                          configuration
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__CONFIGURATION_ERR_MAUVAIS_ID : Mauvais 
//@                                       identifiant de configuration
//@
//@ ------------------------------------------------------------------*/
t_CMN__Configuration_Statut CMN__Configuration_Fermer(t_CMN__Configuration_Id IdConfiguration)
{
  // Pointeur de gestion de configuration
  t_CMN__Configuration_Conf*   IdConf=NULL; 

  // Compte-rendu de sortie
  t_CMN__Configuration_Statut  cr=CMN__STATUT_OK;

  // Pointeur sur les sections
  t_CMN__Configuration_Section* pt_section=NULL;

  // Pointeur sur les champs
  t_CMN__Configuration_Champ* pt_champ=NULL; 

  // Pointeur sur les champs avant un free
  t_CMN__Configuration_Champ* pt_champ_free=NULL;

  // Nombre de sections
  int nombre=0; 

  // Indice de section
  int section=0; 

  // Nom d'une section
  char *nom_section=NULL; 

  TRDEBUT("CMN__Configuration_Fermer");

  //@ Controler l'identifiant
  IdConf=CMN__Configuration_ControlerId(IdConfiguration);

  //@ Si mauvais identifiant sortir en erreur
  if (IdConf==NULL)
    {
      TRFIN("CMN__Configuration_Fermer");

      return CMN__CONFIGURATION_ERR_MAUVAIS_ID;
    }


  //@ Recuperer le nombre de section
  nombre=IdConf->nombre_sections;

  //@ POUR toutes les sections en commencant par la fin faire
  for (section=0;section<nombre;section++)
    {
      pt_section=IdConf->tab_sections[section];

      //@ liberer la chaine champs
      pt_champ=pt_section->champs;
      while (pt_champ)
        {
          TR1(99,"Liberer champ:%s",pt_champ->nom);
          FREE(pt_champ->nom);
          FREE(pt_champ->valeur);
          pt_champ_free=pt_champ;
          pt_champ=pt_champ->suivant;
          FREE(pt_champ_free);
        }

      //@ liberer la section
      FREE(pt_section->nom);
      FREE(pt_section);
    }
  //@ FIN POUR

  FREE(IdConf->tab_sections);
  FREE(IdConfiguration);

  TRFIN("CMN__Configuration_Fermer");

  return cr;
}


//======================================================================
//                                      FONCTIONS INTERNES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_ControlerId
//@
//@ Description : Controle la validite de l'identifiant passe et rend
//@               un pointeur sur la structure de gestion associee
//@
//@ Parametres :
//@               (ENTREE) IdConfiguration : Identifiant de
//@                                          configuration
//@
//@ Valeur de retour :
//@               Pointeur sur la struture de gestion associee a
//@               l'identifiant
//@               NULL : Mauvais identifiant
//@
//@ ------------------------------------------------------------------*/
static t_CMN__Configuration_Conf*   CMN__Configuration_ControlerId(t_CMN__Configuration_Id IdConfiguration)
{
  t_CMN__Configuration_Conf* IdConf=(t_CMN__Configuration_Conf*)IdConfiguration;

  TRDEBUT("CMN__Configuration_ControlerId");

  //@ SI le champs self pointe par l'identifiant n'est pas egal a
  //@ l'identifiant
  if (IdConf->self != IdConf)
    {
      //@ sortir en erreur
      IdConf=NULL;
    }
  //@ SINON rendre la structure de gestion de configuration
  //@ FIN SI

  TRFIN("CMN__Configuration_ControlerId");

  return IdConf;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_RechercherSection
//@
//@ Description : Permet de rechercher une section dans la 
//@               configurationen cours et rendre le pointeur sur elle
//@               si elle existe
//@
//@ Parametres :
//@               (ENTREE) IdConf : Pointeur sur la configuration
//@               (ENTREE) section : Nom de la section
//@
//@ Valeur de retour :
//@               Pointeur sur la section
//@               NULL : Section introuvable
//@
//@ ------------------------------------------------------------------*/
static t_CMN__Configuration_Section* CMN__Configuration_RechercherSection(t_CMN__Configuration_Conf *IdConf,
                                                                          char *section)
{
  // Pointeur sur les sections
  t_CMN__Configuration_Section* pt_section=NULL;  

  // Indice
  int indice=0; 


  TRDEBUT("CMN__Configuration_RechercherSection");
   
  indice=CMN__Configuration_RechercherIndiceSection(IdConf,section);

  //@ S'il n'y a pas de section
  if (indice < 0)
    {
      //@ rendre NULL et sortir
      TRFIN("CMN__Configuration_RechercherSection");
      return (t_CMN__Configuration_Section*)NULL;
    }

  //@ Sortir avec le pointeur
  TRFIN("CMN__Configuration_RechercherSection");

  return IdConf->tab_sections[indice];
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_RechercherIndiceSection
//@
//@ Description : Permet de rechercher un indice de section dans la
//@               configuration en cours
//@
//@ Parametres :
//@               (ENTREE) IdConf : Pointeur sur la configuration
//@               (ENTREE) section : Nom de la section
//@
//@ Valeur de retour :
//@               >=0 : indice de section
//@               -1  : non trouve
//@
//@ ------------------------------------------------------------------*/
static int CMN__Configuration_RechercherIndiceSection(t_CMN__Configuration_Conf *IdConf,char *section)
{
  // Pointeur sur les sections
  t_CMN__Configuration_Section* pt_section=NULL; 

  int indice=0;
  t_CMN__Booleen trouve=CMN__FAUX;

  
  TRDEBUT("CMN__Configuration_RechercherIndiceSection");

  //@ S'il n'y a pas de section en parametre
  if (section == NULL)
    {
      //@ rendre NULL et sortir
      TRFIN("CMN__Configuration_RechercherIndiceSection");
      return -1;
    }
   
  //@ Si le nombre de section est egal a 0
  if ( IdConf->nombre_sections == 0 )
    {
      //@ rendre NULL et sortir
      TRFIN("CMN__Configuration_RechercherIndiceSection");
      return -1;
    }
  
  //@ Regarder si la derniere section accedee n'est pas celle demandee
  pt_section=IdConf->tab_sections[IdConf->dernier_acces];

  //@ SI oui rendre l'indice
  if ( strcmp(pt_section->nom,section) == 0 )
    {
      TRFIN("CMN__Configuration_RechercherIndiceSection");
      return IdConf->dernier_acces;
    }

  if (IdConf->nombre_sections == 1)
    {
      //@ rendre NULL et sortir
      TRFIN("CMN__Configuration_RechercherIndiceSection");
      return -1;
    }

  //@ SINON parcourir a partir du dernier acces jusqu'a la fin
  indice=IdConf->dernier_acces;
  if ( indice != (IdConf->nombre_sections - 1) )
    {
      indice++;
      while ( indice < IdConf->nombre_sections )
        {
          pt_section=IdConf->tab_sections[indice];
          if (strcmp(pt_section->nom,section) == 0)
            {
              trouve=CMN__VRAI;
              break;
            }
          indice++;
        }
    }
  //@ FIN SI

  //@ SI pas trouve jusqu'a la fin
  if ( trouve == CMN__FAUX ) 
    {
      //@ sinon parcourir a partir du premier jusqu'au dernier acces
      indice=0;
      while ( indice < IdConf->dernier_acces )
        {
          pt_section=IdConf->tab_sections[indice];
          if (strcmp(pt_section->nom,section) == 0)
            {
              trouve=CMN__VRAI;
              break;
            }
          indice++;
        }
    }
  //@ FIN SI

  //@ S'il n'y a pas de section
  if (trouve == CMN__FAUX)
    {
      //@ rendre NULL et sortir
      TRFIN("CMN__Configuration_RechercherIndiceSection");
      return -1;
    }

  //@ Sortir avec l'indice
  TRFIN("CMN__Configuration_RechercherIndiceSection");

  return indice;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_RechercherChamp
//@
//@ Description : Permet de rechercher un champ dans la configuration
//@               en cours et rendre le pointeur sur elle si elle
//@               existe
//@
//@ Parametres :
//@               (ENTREE) section : Pointeur sur la section
//@               (ENTREE) champ : Nom du champ
//@
//@ Valeur de retour :
//@               Pointeur sur le champ
//@               NULL : Champ introuvable
//@
//@ ------------------------------------------------------------------*/
static t_CMN__Configuration_Champ* CMN__Configuration_RechercherChamp(t_CMN__Configuration_Section *section,
                                                                      char *champ)
{
  // Pointeur sur les champs
  t_CMN__Configuration_Champ* pt_champ=NULL; 


  TRDEBUT("CMN__Configuration_RechercherChamp");
  TR1(99,"Debut RechercherChamp sec:0x%x",section);

  if (section)
    {
      pt_champ=section->champs;
    }
  TR2(99,"RechercherChamp sec:%s ch:%s",section->nom,champ);

  //@ S'il n'y a pas de champ
  if (pt_champ==NULL)
    {
      //@ rendre NULL et sortir
      TR0(99,"Fin RechercherChamp");
      TRFIN("CMN__Configuration_RechercherChamp");
      
      return (t_CMN__Configuration_Champ*)NULL;
    }

  //@ TANT QUE qu'il y a un champ faire
  while (pt_champ)
    {
      //@ si l'element courant a le meme nom que le champ en parametre
      if (pt_champ->nom && (strcmp(pt_champ->nom,champ)==0))
        {
          //@ rendre le pointeur dessus
          TRFIN("CMN__Configuration_RechercherChamp");
          return pt_champ;
        }
      pt_champ=pt_champ->suivant;
    }
  //@ FIN TANT QUE

  //@ Sortir avec NULL
  TR0(99,"Fin RechercherChamp");
  TRFIN("CMN__Configuration_RechercherChamp");

  return (t_CMN__Configuration_Champ*)NULL;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_AllouerSection
//@
//@ Description : Permet de creer une section dans la configuration
//@               en cours et rendre le pointeur sur elle
//@
//@ Parametres :
//@               (ENTREE) IdConf : Pointeur sur la configuration
//@               (ENTREE) section : Nom de la section
//@
//@ Valeur de retour :
//@               Pointeur sur la section
//@               NULL : Plus de memoire
//@
//@ ------------------------------------------------------------------*/
static t_CMN__Configuration_Section* CMN__Configuration_AllouerSection(t_CMN__Configuration_Conf *IdConf,
                                                                       char *section)
{
  // Pointeur sur les sections
  t_CMN__Configuration_Section* pt_section=NULL; 

  // Pointeur sur la nouvelle structure
  t_CMN__Configuration_Section* section_nouvelle=NULL;

   // Pointeur sur le nom alloue
  char *nom_section=NULL;
  
  int lg=0;

  
  TRDEBUT("CMN__Configuration_AllouerSection");

  //@ Allouer une memoire pour la structure de gestion
  section_nouvelle=(t_CMN__Configuration_Section*)
    MALLOC(sizeof(t_CMN__Configuration_Section));

  //@ Si probleme d'allocation
  if (section_nouvelle==NULL)
    {
      //@ sortir avec NULL
      TRERR(CMN__ERR_PLUS_DE_MEMOIRE);
      TRFIN("CMN__Configuration_AllouerSection");
      return (t_CMN__Configuration_Section*) NULL;
    }

  //@ Allouer une memoire pour le nom de la section
  lg=strlen(section)+1;
  nom_section=(char*)MALLOC(lg);

  //@ Si probleme d'allocation
  if (nom_section==NULL)
    {
      //@ sortir avec NULL
      FREE(section_nouvelle);
      TRERR(CMN__ERR_PLUS_DE_MEMOIRE);
      TRFIN("CMN__Configuration_AllouerSection");
      return (t_CMN__Configuration_Section*) NULL;
    }

  //@ Copier le parametre dans la memoire allouee
  strncpy(nom_section,section,lg);

  //@ Affecter le pointeur dans la structure
  section_nouvelle->nom=nom_section;
  section_nouvelle->champs=NULL;

  if (IdConf->nombre_sections == IdConf->nombre_total)
    {
      IdConf->tab_sections=(t_CMN__Configuration_Section**)
        REALLOC(IdConf->tab_sections,
                (IdConf->nombre_sections + 
                 CMN__CONFIGURATION_TAILLE_ALLOC) * 
                sizeof(t_CMN__Configuration_Section**));
      
      IdConf->nombre_total=IdConf->nombre_sections +
        CMN__CONFIGURATION_TAILLE_ALLOC;
    }

  IdConf->tab_sections[IdConf->nombre_sections]=section_nouvelle;
  IdConf->nombre_sections++;
  IdConf->dernier_acces=IdConf->nombre_sections - 1;

  //@ Rendre le pointeur
  TRFIN("CMN__Configuration_AllouerSection");

  return section_nouvelle;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_AllouerChamp
//@
//@ Description : Permet de creer un champ dans la configuration
//@               en cours et rendre le pointeur sur elle
//@
//@ Parametres :
//@               (ENTREE) section : Pointeur sur la section
//@               (ENTREE) champ : Nom du champ
//@
//@ Valeur de retour :
//@               Pointeur sur le champ
//@               NULL : Plus de memoire
//@
//@ ------------------------------------------------------------------*/
static t_CMN__Configuration_Champ* CMN__Configuration_AllouerChamp(t_CMN__Configuration_Section *section,
                                                                   char *champ)
{
  // Pointeur sur les champs
  t_CMN__Configuration_Champ* pt_champ=section->champs;  

  // Pointeur sur la nouvelle structure
  t_CMN__Configuration_Champ* champ_nouveau=NULL; 

  // Pointeur sur le nom alloue
  char *nom_champ=NULL; 

  int lg=0;


  TRDEBUT("CMN__Configuration_AllouerChamp");

  //@ Allouer une memoire pour la structure de gestion
  champ_nouveau=(t_CMN__Configuration_Champ*)
    MALLOC(sizeof(t_CMN__Configuration_Champ));

  //@ Si probleme d'allocation
  if (champ_nouveau==NULL)
    {
      //@ sortir avec NULL
      TR0(99,"Fin AllouerChamp");
      TRERR(CMN__ERR_PLUS_DE_MEMOIRE);
      TRFIN("CMN__Configuration_AllouerChamp");
      return (t_CMN__Configuration_Champ*) NULL;
    }

  //@ Allouer une memoire pour le nom du champ
  lg=strlen(champ)+1;
  nom_champ=(char*)MALLOC(lg);

  //@ Si probleme d'allocation
  if (nom_champ==NULL)
    {
      //@ sortir avec NULL
      FREE(champ_nouveau);
      TR0(99,"Fin AllouerChamp");
      TRERR(CMN__ERR_PLUS_DE_MEMOIRE);
      TRFIN("CMN__Configuration_AllouerChamp");

      return (t_CMN__Configuration_Champ*) NULL;
    }

  //@ Copier le parametre dans la memoire allouee
  strncpy(nom_champ,champ,lg);

  //@ Affecter le pointeur dans la structure
  champ_nouveau->nom=nom_champ;
  champ_nouveau->suivant=NULL;
  champ_nouveau->valeur=NULL;

  //@ SI c'est le premier champ
  if (pt_champ==NULL)
    {
      //@ le mettre dans le pointeur de tete de section
      section->champs=champ_nouveau;
    }
  //@ SINON
  else
    {
      //@ aller sur le dernier champ
      while (pt_champ->suivant)
        {
          pt_champ=pt_champ->suivant;
        }
      //@ raccrocher a la queue du dernier
      pt_champ->suivant=champ_nouveau;
    }
  //@ FIN SI
  
  //@ Rendre le pointeur
  TR0(99,"Fin AllouerChamp");
  TRFIN("CMN__Configuration_AllouerChamp");

  return champ_nouveau;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_AffecterValeur
//@
//@ Description : Permet de mettre a jour la valeur d'un champ
//@               si il y avait deja un valeur elle est liberee
//@
//@ Parametres :
//@               (ENTREE) champ : Pointeur sur le champ
//@               (ENTREE) valeur : Valeur
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__ERR_PLUS_DE_MEMOIRE : Plus de memoire
//@
//@ ------------------------------------------------------------------*/
static t_CMN__Configuration_Statut CMN__Configuration_AffecterValeur(t_CMN__Configuration_Champ* champ,
                                                                     char *valeur)
{
  // Pointeur sur le nom alloue
  char *nouvelle_valeur=NULL; 

  int lg=0;

  TRDEBUT("CMN__Configuration_AffecterValeur");
  TR1(99,"Affecter valeur:%s",valeur);

  //@ Allouer une memoire pour la valeur
  lg=strlen(valeur)+1;
  nouvelle_valeur=(char*)MALLOC(lg);

  //@ Si probleme d'allocation
  if (nouvelle_valeur==NULL)
    {
      //@ sortir avec NULL
      TR0(99,"Fin affecter valeur");
      TRERR(CMN__ERR_PLUS_DE_MEMOIRE);
      TRFIN("CMN__Configuration_AffecterValeur");
      return CMN__ERR_PLUS_DE_MEMOIRE;
    }

  //@ Copier le parametre dans la memoire allouee
  strncpy(nouvelle_valeur,valeur,lg);

  //@ Si il y a une valeur dans le champ
  if (champ->valeur)
    {
      //@ on la libere
      TR0(99,"Liberation de l'ancienne valeur");
      FREE(champ->valeur);
      champ->valeur=NULL;
    }

  //@ Positionner la nouvelle valeur pour le champ
  champ->valeur=nouvelle_valeur;

  TR0(99,"Fin affecter valeur");
  TRFIN("CMN__Configuration_AffecterValeur");
 
  return CMN__STATUT_OK;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_Interpreter
//@
//@ Description : Permet de creer les sections et les champs
//@               pour la gestion de configuration en analysant
//@               un buffer memoire au format configuration
//@
//@ Parametres :
//@               (ENTREE) IdConf
//@               (ENTREE) memoire : Buffer
//@               (ENTREE) taille_memoire : Taille du buffer
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__ERR_PARAMETRE_NULL : Un parametre nul
//@               CMN__ERR_PLUS_DE_MEMOIRE : Plus de memoire
//@               CMN__CONFIGURATION_ERR_AUCUN_CHARGEMENT : Aucun
//@                                                     element charge
//@
//@ ------------------------------------------------------------------*/
static t_CMN__Configuration_Statut CMN__Configuration_Interpreter(t_CMN__Configuration_Conf *IdConf,
                                                                  char *memoire,
                                                                  int taille_memoire)
{
  // Compte_rendu
  t_CMN__Configuration_Statut cr=CMN__STATUT_OK;

  // Longueur d'un champ
  int lg=0;
 
  // Caractere lu dans le buffer
  char carac=' '; 

  // Pointeur courant dans le buffer
  char *pt_courant=memoire; 

  // Pointeur test de fin de memoire
  char *pt_fin=memoire+taille_memoire;
 
  t_CMN__Configuration_Section* section_courante=NULL;

  t_CMN__Configuration_Champ* champ_courant=NULL;

  // Pointeur sur une valeur de nom de section
  char *une_section=NULL;    

  // Pointeur sur une valeur de nom de champ
  char *un_champ=NULL;   

  // Pointeur sur une valeur de champ
  char *une_valeur=NULL;   

  // Nombre de sections creees
  int   nb_sections_crees=0; 

  // Nombre de champs crees
  int   nb_champs_crees=0; 


  TRDEBUT("CMN__Configuration_Interpreter");
  TR2(99,"Debut interprete adr:0x%x lg:%d",memoire,taille_memoire);

  //@ Controler les parametres
  if ( (memoire==NULL) || (taille_memoire==0) )
    {
      //@ sortie en erreur si un parametre est NULL
      TRERR(CMN__ERR_PARAMETRE_NULL);
      TRFIN("CMN__Configuration_Interpreter");
      return CMN__ERR_PARAMETRE_NULL;
    }

  //@ Initialiser une boucle sur la memoire
  carac=*memoire;

  //@ TANT QUE on est pas a la fin de la memoire faire
  while(pt_courant!=pt_fin)
    {
      TR0(99,"Boucle interprete");

      carac=CMN__Configuration_ChercherCaractere(&pt_courant,pt_fin);

      TR1(99,"Caractere lu:\"%c\"",carac);

      //@ SELON le caractere
      switch(carac)
        {
        //@ CAS '[': debut de section
        case '[':
          //@ Calculer la longueur du nom de section
          lg=CMN__Configuration_CalculerLongueurSection(pt_courant,pt_fin);
          TR1(99,"Longueur section:%d",lg);
          //@ SI le calcul est correct
          if (lg>0)
            {
              //@ Allouer une zone memoire pour stocker le nom de
              //@ la section
              une_section=(char*)MALLOC(lg+1);

              //@ SI l'allocation est correcte
              if (une_section!=NULL)
                {
                  //@ Copier le nom dans la memoire
                  CMN__Configuration_CopierChaineSection(pt_courant,pt_fin,une_section,lg);
                  TR1(99,"Nom section:%s",une_section);

                  //@ Creer un element section dans la 
                  //@ configuration
                  section_courante=CMN__Configuration_AllouerSection(IdConf,une_section);
                  TR1(99,"section allouee:%x",section_courante);
                  
                  //@ Si la creation se passe mal
                  if (section_courante==NULL)
                    {
                      //@ Positionner le flag de fin de boucle et
                      //@ le compte-rendu PLUS_DE_MEMOIRE
                      pt_courant=pt_fin;
                      cr=CMN__ERR_PLUS_DE_MEMOIRE;
                    }
                  //@ FIN SI
                      
                  nb_sections_crees++;

                  //@ Liberer la zone memoire
                  FREE(une_section); une_section=NULL;
                  
                  //@ Aller a la fin de la ligne
                  if (cr==CMN__STATUT_OK)
                    {
                      CMN__Configuration_AllerApresCr(&pt_courant,pt_fin);
                    }
                }
              //@ SINON positionner le flag de fin de boucle et
              //@ le compte-rendu PLUS_DE_MEMOIRE
              else
                {
                  pt_courant=pt_fin;
                  cr=CMN__ERR_PLUS_DE_MEMOIRE;
                }
              //@ FIN SI
            }
          //@ FIN SI
          break;

        //@ CAS '\r': fin de ligne
        case '\r':
          pt_courant++;
          break;

        //@ CAS '\n': fin de ligne
        case '\n':
          pt_courant++;
          break;

        //@ CAS EOF: fin buffer
        case EOF:
          break;

        //@ CAS '#': commentaire
        case '#':
          CMN__Configuration_AllerApresCr(&pt_courant,pt_fin);
          break;

        //@ Default: il s'agit d'un champ
        default:
          //@ Calculer la longueur du nom de champ
          lg=CMN__Configuration_CalculerLongueurChamp(&pt_courant,pt_fin);
          TR1(99,"Longueur champ:%d",lg);

          //@ SI le calcul est correct
          if ((lg>0) && (section_courante))
            {
              //@ Allouer une zone memoire pour stocker le nom
              //@ du champ
              un_champ=(char*)MALLOC(lg+1);

              //@ SI l'allocation est correcte
              if (un_champ!=NULL)
                {
                  //@ Copier le nom dans la memoire
                  CMN__Configuration_CopierChaineChamp(pt_courant,pt_fin,un_champ,lg);
                  TR1(99,"Nom champ:%s",un_champ);

                  //@ Aller sur le =
                  CMN__Configuration_AllerApresEgal(&pt_courant,pt_fin);

                  lg=CMN__Configuration_CalculerLongueurValeur(pt_courant,pt_fin);
                  TR1(99,"Longueur valeur:%d",lg);

                  //@ Si le calcul est correct
                  if (lg>=0)
                    {
                      //@ Allouer une zone memoire pour stocker la
                      //@ valeur du champ
                      une_valeur=(char*)MALLOC(lg+1);

                      //@ Si l'allocation est correcte
                      if (une_valeur!=NULL)
                        {
                          //@ Copier la valeur dans la memoire
                          CMN__Configuration_CopierChaineValeur(pt_courant,pt_fin,une_valeur,lg);
                          TR1(99,"valeur:%s",une_valeur);

                          //@ Creer un champ avec une valeur
                          //@ Si la section n'existe pas
                          if ((champ_courant=CMN__Configuration_RechercherChamp(section_courante,un_champ)) ==NULL)
                            {
                              //@ Creer un element champ dans la
                              //@ configuration
                              champ_courant=CMN__Configuration_AllouerChamp(section_courante,un_champ);
                              //@ SI la creation se passe mal
                              if (champ_courant==NULL)
                                {
                                  //@ Positionner le flag de fin de
                                  //@ boucle et le compte-rendu
                                  //@ PLUS_DE_MEMOIRE
                                  pt_courant=pt_fin;
                                  cr=CMN__ERR_PLUS_DE_MEMOIRE;
                                }
                              //@ SINON
                              else
                                {
                                  //@ Mettre la valeur dans le champ
                                  if (CMN__Configuration_AffecterValeur(champ_courant,une_valeur) != CMN__STATUT_OK)
                                    {
                                      //@ Si erreur d'allocation
                                      //@ memoire positionner le
                                      //@ flag de fin de boucle 
                                      //@ et le compte-rendu 
                                      //@ PLUS_DE_MEMOIRE
                                      pt_courant=pt_fin;
                                      cr=CMN__ERR_PLUS_DE_MEMOIRE;
                                    }
                                }
                              //@ FIN SI

                              nb_champs_crees++;
                            }
                          //@ SINON
                          else
                            {
                              //@ Mettre a jour la valeur dans
                              //@ le champ
                              if (CMN__Configuration_AffecterValeur(champ_courant,une_valeur) != CMN__STATUT_OK)
                                {
                                  //@ Si erreur d'allocation memoire 
                                  //@ positionner le flag de fin de
                                  //@ boucle et le compte-rendu 
                                  //@ PLUS_DE_MEMOIRE
                                  pt_courant=pt_fin;
                                  cr=CMN__ERR_PLUS_DE_MEMOIRE;
                                }
                              nb_champs_crees++;
                            }
                          //@ FIN SI

                          //@ Liberer la zone memoire
                          FREE(une_valeur); une_valeur=NULL;
                        }
                      //@ SINON positionner le flag de fin de boucle
                      //@ et le compte-rendu PLUS_DE_MEMOIRE
                      else
                        {
                          pt_courant=pt_fin;
                          cr=CMN__ERR_PLUS_DE_MEMOIRE;
                        }
                      //@ FIN SI
                    }

                  //@ Liberer la zone memoire
                  FREE(un_champ); un_champ=NULL;

                  //@ Aller a la fin de la ligne
                  if (cr==CMN__STATUT_OK)
                    {
                      CMN__Configuration_AllerApresCr(&pt_courant,pt_fin);
                    }
                }
              //@ SINON positionner le flag de fin de boucle et
              //@ le compte-rendu PLUS_DE_MEMOIRE
              else
                {
                  pt_courant=pt_fin;
                  cr=CMN__ERR_PLUS_DE_MEMOIRE;
                }
            }
          else
            {
              CMN__Configuration_AllerApresCr(&pt_courant,pt_fin);
            }
          //@ FIN SI
          break;
        }

    }
  //@ FIN TANT QUE

  //@ SI le statut est OK
  if (cr==CMN__STATUT_OK)
    {
      //@ SI aucune creation de sections ou de champs n'a ete faite
      if ( (nb_sections_crees==0) && (nb_champs_crees==0) )
        {
          //@ Positionner une erreur de "pas de creation"
          cr=CMN__CONFIGURATION_ERR_AUCUN_CHARGEMENT;
        }
      //@ FIN SI

    }
  //@ FIN SI
  
  if (cr==CMN__ERR_PLUS_DE_MEMOIRE)
    {
      TRERR(CMN__ERR_PLUS_DE_MEMOIRE);
    }

  TRFIN("CMN__Configuration_Interpreter");

  return cr;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_ChercherCaractere
//@
//@ Description : Permet de se positionner sur un caractere 
//@               significatif du buffer, cad autre que ESPACE et TAB
//@
//@ Parametres :
//@               (ENTREE/SORTIE) pt_courant : Pointeur sur le debut
//@                                            de traitement
//@               (ENTREE) pt_fin : Pointeur sur la fin a ne pas
//@                                 depasser
//@
//@ Valeur de retour :
//@               Le caractere significatif ou EOF en fin de buffer
//@
//@ ------------------------------------------------------------------*/
static char CMN__Configuration_ChercherCaractere(char **pt_courant,char *pt_fin)
{
  // Pointeur courant de travail
  char *pt=*pt_courant;

  // Caractere pour le retour
  char cret=EOF;   


  TRDEBUT("CMN__Configuration_ChercherCaractere");

  //@ Au debut tant que le caractere est ESPACE ou TAB et qu'on
  //@ n'est pas a la fin, se deplacer dans le buffer
  while ((pt!=pt_fin)&&((*pt==' ')||(*pt=='\t')))
    {
      pt++;
    }

  //@ SI le pointeur de deplacement n'est pas a la fin
  if (pt!=pt_fin)
    {
      //@ positionner le caractere de retour et mettre
      //@ a jour la valeur du pointeur courant
      cret=*pt;
      *pt_courant=pt;
    }
  //@ SINON le retour est EOF
  //@ FIN SI


  TRFIN("CMN__Configuration_ChercherCaractere");
  
  return cret;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_CalculerLongueurSection
//@
//@ Description : Permet de calculer le nombre de caracteres du nom
//@               d'une section dans le buffer, le pointeur courant
//@               doit etre positionner sur le [ ou juste apres
//@
//@ Parametres :
//@               (ENTREE) pt_courant : Pointeur sur le debut
//@                                     de traitement
//@               (ENTREE) pt_fin : Pointeur sur la fin a ne pas
//@                                 depasser
//@
//@ Valeur de retour :
//@               Le nombre de caractere ou 0
//@
//@ ------------------------------------------------------------------*/
static int CMN__Configuration_CalculerLongueurSection(char *pt_courant,char *pt_fin)
{
  // Pointeur courant de travail
  char *pt=pt_courant; 

  // Debut significatif
  char *pt_debut=NULL;

  // Longueur pour le retour
  int lg=0;    


  TRDEBUT("CMN__Configuration_CalculerLongueurSection");

  //@ Si le pointeur courant est sur le [ et pas en fin, 
  //@ sauter le caractere
  if (pt_courant!=pt_fin)
    {
      if (*pt=='[')
        {
          pt++;
        }
    }

  //@ Au debut tant que le caractere est ESPACE ou TAB et qu'on 
  //@ n'est pas a la fin, se deplacer dans le buffer
  while ((pt!=pt_fin)&&((*pt==' ')||(*pt=='\t')))
    {
      pt++;
    }


  //@ Si le pointeur de deplacement n'est pas a la fin
  if (pt!=pt_fin)
    {
      pt_debut=pt;
      //@ compter les caracteres jusqu'a la fin, ou un ESPACE 
      //@ ou un TAB ou un CR ou un ]
      while ((pt!=pt_fin)&&(*pt!='\n')&&(*pt!='\r')&&(*pt!=']'))
        {
          lg++;
          pt++;
        }
      CMN__Configuration_SoustraireBlancEnFin(pt_debut,&lg);
    }

  TRFIN("CMN__Configuration_CalculerLongueurSection");

  return lg;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_CopierChaineSection
//@
//@ Description : Permet de copier le nom d'une section situe
//@               dans le buffer dans une chaine passee en parametre,
//@               le pointeur courant doit etre positionner sur le [
//@               ou juste apres.
//@               La chaine passee en parametre doit avoir la taille
//@               rendue par 
//@               CMN__Configuration_CalculerLongueurSection + 1 
//@               pour le NULL
//@
//@ Parametres :
//@               (ENTREE) pt_courant : Pointeur sur le debut
//@                                     de traitement
//@               (ENTREE) pt_fin : Pointeur sur la fin a ne pas
//@                                 depasser
//@               (ENTREE/SORTIE) chaine : Pointeur sur la chaine 
//@                                        pour copie
//@               (ENTREE) lg : Longueur a copier
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void CMN__Configuration_CopierChaineSection(char *pt_courant,
                                                   char *pt_fin,
                                                   char *chaine,
                                                   int lg)
{
  // Pointeur courant de travail
  char *pt=pt_courant; 

  // Pointeur sur chaine de copie
  char *pt_out=chaine;


  TRDEBUT("CMN__Configuration_CopierChaineSection");

  //@ Si le pointeur courant est sur le [ et pas en fin, 
  //@ sauter le caractere
  if (pt_courant!=pt_fin)
    {
      if (*pt=='[')
        {
          pt++;
        }
    }

  //@ Tant que le caractere est ESPACE ou TAB et qu'on n'est 
  //@ pas a la fin, se deplacer dans le buffer
  while ((pt!=pt_fin)&&((*pt==' ')||(*pt=='\t')))
    {
      pt++;
    }


  //@ Si le pointeur de deplacement n'est pas a la fin
  if (pt!=pt_fin)
    {
      //@ copier lg caracteres
      memcpy(chaine,pt,lg);
      chaine[lg]=0;
    }

  TRFIN("CMN__Configuration_CopierChaineSection");
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_AllerApresCr
//@
//@ Description : Permet de se positionner sur un caractere apres le
//@               prochain CR
//@
//@ Parametres :
//@               (ENTREE/SORTIE) pt_courant : Pointeur sur le debut
//@                                            de traitement
//@               (ENTREE) pt_fin : Pointeur sur la fin a ne pas
//@                                 depasser
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void CMN__Configuration_AllerApresCr(char **pt_courant,char *pt_fin)
{
  // Pointeur courant de travail
  char *pt=*pt_courant; 

  TRDEBUT("CMN__Configuration_AllerApresCr");
  TR2(99,"Aller apres CR adr courant:0x%x fin:0x%x",*pt_courant,pt_fin);

  //@ Tant que le caractere n'est pas CR et qu'on n'est pas a la fin,
  //@ se deplacer dans le buffer
  while ((pt!=pt_fin)&&(*pt!='\n')&&(*pt!='\r'))
    {
      pt++;
    }


  //@ Si le pointeur de deplacement n'est pas a la fin
  if (pt!=pt_fin)
    {
      //@ passer apres le cr
      pt++;
    }

  //@ Mettre a jour le pointeur en parametre
  *pt_courant=pt;
  TR1(99,"Aller apres CR courant:0x%x",*pt_courant);
  TRFIN("CMN__Configuration_AllerApresCr");
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_CalculerLongueurChamp
//@
//@ Description : Permet de calculer le nombre de caracteres du nom 
//@               d'un champ dans le buffer
//@
//@ Parametres :
//@               (ENTREE) pt_courant : Pointeur sur le debut
//@                                     de traitement
//@               (ENTREE) pt_fin : Pointeur sur la fin a ne pas
//@                                 depasser
//@
//@ Valeur de retour :
//@               Le nombre de caractere ou 0
//@
//@ ------------------------------------------------------------------*/
static int CMN__Configuration_CalculerLongueurChamp(char **pt_courant,
                                                    char *pt_fin)
{
  // Pointeur courant de travail
  char *pt=*pt_courant;

  // Debut significatif
  char *pt_debut=NULL; 

  // Longueur pour le retour
  int lg=0;    


  TRDEBUT("CMN__Configuration_CalculerLongueurChamp");

  //@ Purge les blancs au debut
  //@ Tant que le caractere est ESPACE ou TAB et qu'on n'est pas
  //@ a la fin, se deplacer dans le buffer
  while ((pt!=pt_fin)&&((*pt==' ')||(*pt=='\t')))
    {
      pt++;
    }


  //@ SI le pointeur de deplacement n'est pas a la fin
  if (pt!=pt_fin)
    {
      pt_debut=pt;
      //@ compter les caracteres jusqu'a la fin, ou un ESPACE ou
      //@ un TAB ou un CR ou un =
      while ((pt!=pt_fin)&&(*pt!='\n')&&(*pt!='=')&&(*pt!='\r'))
        {
          lg++;
          pt++;
        }

      //@ si on n'a pas trouver de =
      if ((pt==pt_fin)||(*pt!='='))
        {
          //@ fin anormale on se positionne a la fin
          *pt_courant=pt;
          lg=0;
        }
      else
        {
          CMN__Configuration_SoustraireBlancEnFin(pt_debut,&lg);
        }
    }
  //@ FIN SI


  TRFIN("CMN__Configuration_CalculerLongueurChamp");

  return lg;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_CopierChaineChamp
//@
//@ Description : Permet de copier le nom d'une section situee dans
//@               le buffer dans une chaine passee en parametre.
//@               La chaine passee en parametre doit avoir la taille
//@               rendue par CMN__Configuration_CalculerLongueurChamp
//@               + 1 pour le NULL
//@
//@ Parametres :
//@               (ENTREE) pt_courant : Pointeur sur le debut
//@                                     de traitement
//@               (ENTREE) pt_fin : Pointeur sur la fin a ne pas
//@                                 depasser
//@               (ENTREE/SORTIE) chaine : Pointeur sur la chaine 
//@                                        pour copie
//@               (ENTREE) lg : Longueur a copier
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void CMN__Configuration_CopierChaineChamp(char *pt_courant,
                                                 char *pt_fin,
                                                 char *chaine,
                                                 int lg)
{
  // Pointeur courant de travail
  char *pt=pt_courant; 

  // Pointeur sur chaine de copie
  char *pt_out=chaine;


  TRDEBUT("CMN__Configuration_CopierChaineChamp");

  //@ Purge des blancs au debut
  //@ Tant que le caractere est ESPACE ou TAB et qu'on n'est 
  //@ pas a la fin, se deplacer dans le buffer
  while ((pt!=pt_fin)&&((*pt==' ')||(*pt=='\t')))
    {
      pt++;
    }


  //@ SI le pointeur de deplacement n'est pas a la fin
  if (pt!=pt_fin)
    {
      //@ copier lg caracteres
      memcpy(chaine,pt,lg);
      chaine[lg]=0;
    }
  //@ FIN SI

  TRFIN("CMN__Configuration_CopierChaineChamp");
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_AllerApresEgal
//@
//@ Description : Permet de se positionner sur un caractere apres
//@               le prochain =
//@
//@ Parametres :
//@               (ENTREE/SORTIE) pt_courant : Pointeur sur le debut
//@                                            de traitement
//@               (ENTREE) pt_fin : Pointeur sur la fin a ne pas
//@                                 depasser
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void CMN__Configuration_AllerApresEgal(char **pt_courant,
                                              char *pt_fin)
{
  // Pointeur courant de travail
  char *pt=*pt_courant;


  TRDEBUT("CMN__Configuration_AllerApresEgal");

  //@ Tant que le caractere n'est pas CR et pas = et qu'on n'est
  //@ pas a la fin, se deplacer dans le buffer
  while ((pt!=pt_fin)&&(*pt!='\n')&&(*pt!='\r')&&(*pt!='='))
    {
      pt++;
    }

  //@ Si le pointeur de deplacement n'est pas a la fin et n'est pas CR
  if ((pt!=pt_fin)&&(*pt!='\n')&&(*pt!='\r'))
    {
      //@ passer apres le =
      pt++;
    }

  //@ Mettre a jour le pointeur en parametre
  *pt_courant=pt;

  TRFIN("CMN__Configuration_AllerApresEgal");
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_CalculerLongueurValeur
//@
//@ Description : Permet de calculer le nombre de caracteres de la
//@               valeur d'un champ dans le buffer, le pointeur
//@               courant doit etre positionner sur le = ou juste
//@               apres
//@
//@ Parametres :
//@               (ENTREE) pt_courant : Pointeur sur le debut de
//@                                     traitement
//@               (ENTREE) pt_fin : Pointeur sur la fin a ne pas 
//@                                 depasser
//@
//@ Valeur de retour :
//@               Le nombre de caractere ou 0
//@
//@ ------------------------------------------------------------------*/
static int CMN__Configuration_CalculerLongueurValeur(char *pt_courant,char *pt_fin)
{ 
  // Pointeur courant de travail
  char *pt=pt_courant;

  // Debut significatif
  char *pt_debut=NULL;

  // Longueur pour le retour
  int lg=0;    

  
  TRDEBUT("CMN__Configuration_CalculerLongueurValeur");
  
  //@ Si le pointeur courant est sur le = et pas en fin, 
  //@ sauter le caractere
  if (pt_courant!=pt_fin)
    {
      if (*pt=='=')
        {
          pt++;
        }
    }

  //@ Au debut tant que le caractere est ESPACE ou TAB et qu'on
  //@ n'est pas a la fin, se deplacer dans le buffer
  while ((pt!=pt_fin)&&((*pt==' ')||(*pt=='\t')))
    {
      pt++;
    }


  //@ Si le pointeur de deplacement n'est pas a la fin
  if (pt!=pt_fin)
    {
      pt_debut=pt;
      
      //@ compter les caracteres jusqu'a la fin ou un CR
      while ((pt!=pt_fin)&&(*pt!='\n')&&(*pt!='\r'))
        {
          lg++;
          pt++;
        }
      CMN__Configuration_SoustraireBlancEnFin(pt_debut,&lg);
    }

  TRFIN("CMN__Configuration_CalculerLongueurValeur");

  return lg;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_CopierChaineValeur
//@
//@ Description : Permet de copier la valeur d'un champ situe
//@               dans le buffer dans une chaine passee en parametre,
//@               le pointeur courant doit etre positionner sur le =
//@               ou juste apres. La chaine passee en parametre doit
//@               avoir la taille rendue par
//@               CMN__Configuration_CalculerLongueurValeur + 1 pour
//@               le NULL
//@
//@ Parametres :
//@               (ENTREE) pt_courant : Pointeur sur le debut de
//@                                     traitement
//@               (ENTREE) pt_fin : Pointeur sur la fin a ne pas
//@                                 depasser
//@               (ENTREE/SORTIE) chaine : Pointeur sur la chaine 
//@                                        pour copie
//@               (ENTREE) lg : Longueur a copier
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void CMN__Configuration_CopierChaineValeur(char *pt_courant,
                                                  char *pt_fin,
                                                  char *chaine,
                                                  int lg)
{
  // Pointeur courant de travail
  char *pt=pt_courant; 

  // Pointeur sur chaine de copie
  char *pt_out=chaine; 


  TRDEBUT("CMN__Configuration_CopierChaineValeur");

  //@ Si le pointeur courant est sur le [ et pas en fin, 
  //@ sauter le caractere
  if (pt_courant!=pt_fin)
    {
      if (*pt=='=')
        {
          pt++;
        }
    }


  //@ Au debut tant que le caractere est ESPACE ou TAB et qu'on
  //@ n'est pas a la fin, se deplacer dans le buffer
  while ((pt!=pt_fin)&&((*pt==' ')||(*pt=='\t')))
    {
      pt++;
    }


  //@ Si le pointeur de deplacement n'est pas a la fin
  if (pt!=pt_fin)
    {
      //@ copier lg caracteres
      memcpy(chaine,pt,lg);
      chaine[lg]=0;
    }

  TRFIN("CMN__Configuration_CopierChaineValeur");
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_CalculerLongueurConfiguration
//@
//@ Description : Permet de d'avoir la longueur (chaine ascii) de la
//@               configuration en gestion
//@
//@ Parametres :
//@               (ENTREE) IdConf
//@
//@ Valeur de retour :
//@               Longueur de la configuration
//@
//@ ------------------------------------------------------------------*/
static int CMN__Configuration_CalculerLongueurConfiguration(t_CMN__Configuration_Conf *IdConf)
{ 
  // Pointeur sur les sections
  t_CMN__Configuration_Section* pt_section=NULL; 
  
  // Pointeur sur les champs
  t_CMN__Configuration_Champ* pt_champ=NULL; 
  
  // Longueur de la configuration en ascii
  int lg=0;   
  
  int ind=0;
  
  
  TRDEBUT("CMN__Configuration_CalculerLongueurConfiguration");
  
  if (IdConf->nombre_sections == 0)
    {
      TR0(99,"Pas de section");
      lg=0;
    }
   else
     {
       for (ind=0;ind<IdConf->nombre_sections;ind++)
         {
           pt_section=IdConf->tab_sections[ind];
           
           TR1(99,"Section:\"%s\"",pt_section->nom);
           lg=lg+strlen(pt_section->nom)+3; // 3 pour [  ] et \n
           pt_champ=pt_section->champs;
           if (pt_champ==NULL)
             {
               TR0(99,"Pas de champ");
             }
           else
             {
               while(pt_champ)
                 {
                   TR2(99,"Champ:\"%s\" valeur:\"%s\"",
                       pt_champ->nom,pt_champ->valeur);
                   lg=lg+strlen(pt_champ->nom)+
                     strlen(pt_champ->valeur)+2; // 2 pour = et \n
                   pt_champ=pt_champ->suivant;
                 }
             }
         }
     }
  
  TR1(99,"Longueur de la configuration en ascii:%d\n",lg);
  TRFIN("CMN__Configuration_CalculerLongueurConfiguration");

  return lg;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_SoustraireBlancEnFin
//@
//@ Description : Permet de d'avoir la longueur significatif d'un
//@               champ en enlevant les blanc en fin
//@
//@ Parametres :
//@               (ENTREE) pt_debut : Pointeur sur le debut
//@                                   significatif
//@               (ENTREE/SORTIE) lg : Longueur avec les blancs 
//@                                    eventuels
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void CMN__Configuration_SoustraireBlancEnFin(char *pt_debut,int *lg)
{
  // Pointeur sur fin
  char *pt=NULL; 
  
  
  TRDEBUT("CMN__Configuration_SoustraireBlancEnFin");
  
  //@ Si la longueur n'est pas positive sortir
  if (*lg<=0) return;
  
  pt=pt_debut+(*lg)-1;
  
  //@ TANT QUE il y a des blancs a la fin diminuer la longueur
  while ((pt!=pt_debut)&&((*pt==' ')||(*pt=='\t')))
    {
      *lg=*lg - 1;
      pt--;
    }
  //@ FIN TANT QUE
  
  TRFIN("CMN__Configuration_SoustraireBlancEnFin");
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_DumpTrace
//@
//@ Description : Permet de tracer l'arborescence des sections et
//@               champs
//@
//@ Parametres :
//@               (ENTREE) IdConf : Pointeur sur la configuration
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void CMN__Configuration_DumpTrace(t_CMN__Configuration_Conf *IdConf)
{
  // Pointeur sur les sections
  t_CMN__Configuration_Section* pt_section=NULL; 

  // Pointeur sur les champs
  t_CMN__Configuration_Champ* pt_champ=NULL; 

  // Indice
  int ind=0;
  
  
  TRDEBUT("CMN__Configuration_DumpTrace");
  TR1(99,"DEBUT TRACE CONFIGURATION ID:0x%x",IdConf);
  
  if (IdConf->nombre_sections == 0)
    {
      TR0(99,"Pas de section");
    }
  else
    {
      for (ind=0;ind<IdConf->nombre_sections;ind++)
        {
          pt_section=IdConf->tab_sections[ind];
          TR1(99,"Section:\"%s\"",pt_section->nom);
          pt_champ=pt_section->champs;
          if (pt_champ==NULL)
            {
              TR0(99,"Pas de champ");
            }
          else
            {
              while(pt_champ)
                {
                  TR2(99,"Champ:\"%s\" valeur:\"%s\"",
                      pt_champ->nom,
                      pt_champ->valeur);
                 
                  pt_champ=pt_champ->suivant;
                }
            }
        }
    }
  
  TR1(99,"FIN TRACE CONFIGURATION ID:0x%x",IdConf);
  TRFIN("CMN__Configuration_DumpTrace");
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_SauverFichierMode
//@
//@ Description : Permet de sauvegarder une configuration en gestion
//@               dans un fichier au format configuration en
//@               exploitant le mode d'ecriture du fichier passe
//@               en parametre
//@
//@ Parametres :
//@               (ENTREE) IdConfiguration : Identifiant de
//@                                          configuration
//@               (ENTREE) fichier : Fichier pour sauvegarde
//@               (ENTREE) mode : Mode de sauvegarde
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Fonctionnement correct
//@               CMN__CONFIGURATION_ERR_MAUVAIS_ID : Mauvais 
//@                                       identifiant de configuration
//@               CMN__ERR_CREATION_FICHIER : Erreur de creation de
//@                                           fichier
//@
//@ ------------------------------------------------------------------*/
static t_CMN__Configuration_Statut CMN__Configuration_SauverFichierMode(t_CMN__Configuration_Id IdConfiguration,
                                                                        char *fichier,
                                                                        char *mode)
{
  // Pointeur de gestion de configuration
  t_CMN__Configuration_Conf*   IdConf=NULL; 
  
  // Compte-rendu de sortie
  t_CMN__Configuration_Statut  cr=CMN__STATUT_OK; 

  // Pointeur sur les sections
  t_CMN__Configuration_Section* pt_section=NULL;  

  // Pointeur sur les champs
  t_CMN__Configuration_Champ* pt_champ=NULL; 

  // Fichier de configuration ouvert
  FILE*   fichier_conf=NULL;

  // Indice
  int ind=0;
  
  TRDEBUT("CMN__Configuration_SauverFichier");
  
   //@ Controler l'identifiant
  IdConf=CMN__Configuration_ControlerId(IdConfiguration);
  
  //@ Si mauvais identifiant sortir en erreur
  if (IdConf==NULL)
    {
      TRFIN("CMN__Configuration_SauverFichier");
      
      return CMN__CONFIGURATION_ERR_MAUVAIS_ID;
    }
  
  
  //@ Ouvrir le fichier
  fichier_conf=fopen(fichier,mode);
  
  //@ S'il y a un probleme a l'ouverture
  if (fichier_conf==NULL)
    {
      //@ sortir en erreur
      TRERR(CMN__ERR_CREATION_FICHIER);
      TRFIN("CMN__Configuration_SauverFichier");
      return CMN__ERR_CREATION_FICHIER;
    }

  //@ SI pas de section ne rien faire
  if (IdConf->nombre_sections == 0)
    {
      TR0(99,"Pas de section");
    }
  //@ SINON
  else
    {
      //@ POUR toutes les sections
      for (ind=0;ind<IdConf->nombre_sections;ind++)
        {
          pt_section=IdConf->tab_sections[ind];
          TR1(99,"Section:\"%s\"",pt_section->nom);
          pt_champ=pt_section->champs;
          
          //@ sauver le nom de section
          fprintf(fichier_conf,"\n[%s]\n",pt_section->nom);
          if (pt_champ==NULL)
            {
              TR0(99,"Pas de champ");
            }
          else
         {
           //@ POUR tous les champs
           while(pt_champ)
             {
               TR2(99,"Champ:\"%s\" valeur:\"%s\"",pt_champ->nom,pt_champ->valeur);
               //@ sauver le nom de champ et la valeur
               fprintf(fichier_conf,"%s=%s\n",pt_champ->nom,pt_champ->valeur);
               pt_champ=pt_champ->suivant;
             }
           //@ FIN POUR
         }
        }
      //@ FIN POUR
    }
  //@ FIN SI

  //@ Fermer le fichier
  fclose(fichier_conf);
  
  TRFIN("CMN__Configuration_SauverFichier");
  
  return cr;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_TestVide
//@
//@ Description : Permet de tester si un champ est vide
//@
//@ Parametres :
//@               (ENTREE) valeur : Valeur a tester
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Valeur non vide
//@               CMN__STATUT_NOK : Valeur vide
//@
//@ ------------------------------------------------------------------*/
static t_CMN__Configuration_Statut CMN__Configuration_TestVide(char *valeur)
{
  t_CMN__Configuration_Statut cr=CMN__STATUT_NOK;
  char *pt=valeur;
  
  if (pt!=NULL)
    {
      while(*pt!=0)
        {
          if ((*pt != ' ')||(*pt != '\t')) 
            {
              cr=CMN__STATUT_OK;
              break;
            }
          pt++;
        }
    }
  
  return cr;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_SansBlancDevantDerriere
//@
//@ Description : Permet de supprimer les blancs devant et derriere
//@               une valeur
//@
//@ Parametres :
//@               (ENTREE) in : Valeur a transformer
//@               (SORTIE) out : Valeur transformee
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void CMN__Configuration_SansBlancDevantDerriere(char *in,char *out)
{
  char *ptin=in;
  char *ptout=out;
  char *ptin_deb=in;
  char *ptin_fin=NULL;
  
  *ptout=0;
  
  if (ptin!=NULL)
    {
      ptin_fin=in+strlen(in)-1;
      while( ((*ptin_fin==' ')||(*ptin_fin=='\t')) && (ptin_fin!=ptin_deb))
        ptin_fin--;
      while( ((*ptin_deb==' ')||(*ptin_fin=='\t')) && (ptin_fin!=ptin_deb)) 
        ptin_deb++;
      
      if ((ptin_fin==ptin_deb) && ((*ptin_deb==' ')||(*ptin_deb=='\t')))  {
      }
      else
        {
          *ptout=*ptin_deb;
          ptout++;
          while(ptin_deb!=ptin_fin) { ptin_deb++; *ptout=*ptin_deb; ptout++; }
          *ptout=0;
        }      
    }
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_FusionnerSections
//@
//@ Description : Permet de sauvegarder une configuration en gestion
//@               dans un fichier au format configuration en
//@               exploitant le mode d'ecriture du fichier passe en
//@               parametre
//@
//@ Parametres :
//@               (ENTREE) IdConf : Identifiant de configuration
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void CMN__Configuration_FusionnerSections(t_CMN__Configuration_Conf *IdConf)
{
  // Pointeur sur les sections
  t_CMN__Configuration_Section* pt_section1=NULL;  

  // Pointeur sur les sections
  t_CMN__Configuration_Section* pt_section2=NULL;  

  // Pointeur sur les sections
  t_CMN__Configuration_Section** pt_pt_section=NULL; 

  // Pointeur sur les champs
  t_CMN__Configuration_Champ* pt_champ=NULL; 

  // Indice
  int ind=0; 
  int ind_premier=0;
  int nb_sections  = 0;


  TRDEBUT("CMN__Configuration_FusionnerSections");
  
  if ( IdConf->nombre_sections <= 1 )
    {
      return;
    }
  
  //@ Dupliquer la table des sections
  pt_pt_section=(t_CMN__Configuration_Section**)
    MALLOC(IdConf->nombre_sections * sizeof(t_CMN__Configuration_Section**));
  if (pt_pt_section == NULL)
   {
     return;
   }
  memcpy(pt_pt_section,
         IdConf->tab_sections,
         IdConf->nombre_sections * sizeof(t_CMN__Configuration_Section**));

  //@ Trier le tableau
  qsort((void*)pt_pt_section,
        (size_t)(IdConf->nombre_sections),
        (size_t)sizeof(t_CMN__Configuration_Section**),
        (int (*)(const void *, const void *))CMN__Configuration_CompareSections);

  ind_premier=0;

  //@ Eliminer les doublons
  nb_sections  = IdConf->nombre_sections;
  for (ind=1;ind<nb_sections;ind++)
    {
      pt_section1=pt_pt_section[ind_premier];
      pt_section2=pt_pt_section[ind];
      if ( strcmp(pt_section1->nom,pt_section2->nom) == 0 )
        {
          CMN__Configuration_TransfertSection(IdConf,pt_section1,pt_section2);
        }
      else
        {
          ind_premier=ind;
        }
    }
  
  FREE(pt_pt_section);
  
  TRFIN("CMN__Configuration_FusionnerSections");
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_CompareSections
//@
//@ Description : Permet de comparer deux pointeurs de section
//@ 
//@ Parametres :
//@               (ENTREE) sec1 : Pointeur de pointeur section 1
//@               (ENTREE) sec2 : Pointeur de pointeur section 2
//@
//@ Valeur de retour :
//@               >0 : sec 1 > sec2
//@               <0 : sec 1 < sec2
//@               0  : sec 1 = sec 2
//@
//@ ------------------------------------------------------------------*/
static int CMN__Configuration_CompareSections(t_CMN__Configuration_Section** sec1,
                                              t_CMN__Configuration_Section** sec2)
{
  if (sec1==NULL) return -1;
  if (sec2==NULL) return 1;
  return ( strcmp( (*sec1)->nom , (*sec2)->nom) );
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Configuration_TransfertSection
//@
//@ Description : Permet de mettre a jour une section avec une autre
//@
//@ Parametres :
//@               (ENTREE) IdConf : Identifiant configuration
//@               (ENTREE) pt_section1 : Pointeur  section 1
//@               (ENTREE) pt_section2 : Pointeur  section 2
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void CMN__Configuration_TransfertSection(t_CMN__Configuration_Conf *IdConf,
                                                t_CMN__Configuration_Section* pt_section1,
                                                t_CMN__Configuration_Section* pt_section2)
{
  // Indice
  int ind=0; 
  
  // Travail sur champ
  t_CMN__Configuration_Champ* champs=NULL;
  

  //@ Si il n'y a pas de champs dans section2 sortir
  if ( pt_section2->champs == NULL ) return;

  //@ Positionner le dernier acces sur la section 1
  for (ind=0;ind<IdConf->nombre_sections;ind++)
    {
      if (IdConf->tab_sections[ind] == pt_section1)
        {
          IdConf->dernier_acces=ind;
          break;
        }
    }
  
  //@ Transferer les champs de section 2 dans section 1
  champs=pt_section2->champs;
  while( champs )
    {
      CMN__Configuration_EcrireValeur((t_CMN__Configuration_Id)IdConf,
                                      pt_section1->nom,
                                      champs->nom,
                                      champs->valeur);
      champs=champs->suivant;
    }

  //@ Positionner le dernier acces sur la section 2
  for (ind=0;ind<IdConf->nombre_sections;ind++)
    {
      if (IdConf->tab_sections[ind] == pt_section2)
        {
          IdConf->dernier_acces=ind;
          break;
        }
    }
  
  //@ Detruire la section
  CMN__Configuration_DetruireSection((t_CMN__Configuration_Id)IdConf,
                                     pt_section2->nom);
}
