// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/CMN__Configuration.h,v 1.3 2011/11/25 10:17:39 dpauly Exp $
//@ ------------------------------------------------------------------*/
//@ Composant : CMN
//@ MODULE : CMN__Configuration
//@
//@ Commentaire :
//@    Fichier d'entete du module CMN__Configuration
//@     definie les variables et les procedures exportees pour
//@     pour la gestion des fichiers de configuration
//@       Ouvrir permet d'initialiser la gestion
//@       ChargerFichier permet de charger en memoire le contenu
//@                      d'un fichier de configuration
//@       ChargerChaine permet de charger en memoire le contenu
//@                      d'une chaine contenant la configuration
//@       NombreDeSections permet d'avoir le nombre de sections de la conf
//@       LireSection permet de lire la valeur d'une section 
//@       EcrireValeur permet d'ecrire une valeur d'un champ dans une section
//@       LireValeur permet de lire la valeur d'un champ dans une section
//@       DetruireSection permet de detruire une section avec tous ses champs
//@       DetruireChamp permet de detruire un champ et la section associee
//@                      si s'etait le dernier champ
//@       SauverFichier permet de sauver le contenu de la memoire 
//@                     de configuration dans un fichier
//@       AjouterFichier permet de sauver le contenu de la memoire 
//@                     de configuration dans un fichier a la fin
//@       SauverChaine permet de sauver le contenu de la memoire 
//@                     de configuration dans une chaine
//@       Fermer permet de terminer la gestion de configuration et liberer
//@       La syntaxe BNF des fichiers de configuration est la suivante:
//@            <Configuration>         :: <Section>*
//@            <Sections>              :: [<Nom section>] <Champ>*
//@            <Champ>                 :: <Nom champ> = <Valeur champ>
//@            <Nom Section>           :: <Chaine de caracteres>
//@            <Nom champ>             :: <Chaine de caracteres>
//@            <Valeur champ>          :: [<Chaine de caracteres>]
//@            <Chaine de caracteres>  :: {A-Za-z0-9_:\/ }+
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 10/03/2000 : Pierre-Louis Agnetti
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif
#ifndef CMN__Configuration_h
#define CMN__Configuration_h

#include "ERR__Erreur.h"

//====================================================================*/
// Declaration des define externes
//====================================================================*/

// type pour la gestion de configuration rendu par Ouvrir
// et mentionne a chaque procedures et sur Fermer en dernier
typedef char* t_CMN__Configuration_Id;

// type pour la gestion des compte_rendu des procedures
// de gestion des configurations
typedef int t_CMN__Configuration_Statut;


// Les erreurs
#define CMN__CONFIGURATION_ERR_MAUVAIS_ID          (CFG_BASE_ERROR+1)
#define CMN__CONFIGURATION_ERR_CHAMP_INEXISTANT    (CFG_BASE_ERROR+2)
#define CMN__CONFIGURATION_ERR_SECTION_INEXISTANTE (CFG_BASE_ERROR+3)
#define CMN__CONFIGURATION_ERR_RIEN_A_SAUVER       (CFG_BASE_ERROR+4)
#define CMN__CONFIGURATION_ERR_AUCUN_CHARGEMENT    (CFG_BASE_ERROR+5)
#define CMN__CONFIGURATION_ERR_MAUVAIS_INDICE      (CFG_BASE_ERROR+6)
#define CMN__ERR_PARAMETRE_NULL                    (CFG_BASE_ERROR+7)
#define CMN__ERR_PLUS_DE_MEMOIRE                   (CFG_BASE_ERROR+8)
#define CMN__ERR_FICHIER_INTROUVABLE               (CFG_BASE_ERROR+9)
#define CMN__ERR_CREATION_FICHIER                  (CFG_BASE_ERROR+10)

//====================================================================*/
// Declaration des fonctions externes
//====================================================================*/

//         Initialise un configuration en rendant un identifiant
//         Cet identifiant sera utilise sur toutes les procedures
t_CMN__Configuration_Statut CMN__Configuration_Ouvrir(t_CMN__Configuration_Id *IdConfiguration);


//         Permet de charger en gestion le contenu d'un fichier au
//         format configuration
t_CMN__Configuration_Statut CMN__Configuration_ChargerFichier(t_CMN__Configuration_Id IdConfiguration,char *fichier);

//         Permet de charger en gestion le contenu d'une zone memoire
//         au format configuration
t_CMN__Configuration_Statut CMN__Configuration_ChargerChaine(t_CMN__Configuration_Id IdConfiguration,char *chaine,int longueur);

//         Permet d'avoir le nombre de sections actuellement dans la
//         gestion de configuration
//         au format configuration
t_CMN__Configuration_Statut CMN__Configuration_NombreDeSections(t_CMN__Configuration_Id IdConfiguration,int *nombre);

//         Permet d'avoir le nom d'une section d'indice i actuellement dans la
//         gestion de configuration
t_CMN__Configuration_Statut CMN__Configuration_LireSection(t_CMN__Configuration_Id IdConfiguration,int indice,char **section);


//         Permet d'ecrire la valeur d'un champ dans une section
//         dans une configuration en gestion.
//         Si le champ n'existe pas il est cree, sinon il est mis a jour
//         Si la section n'existe pas il est cree.
t_CMN__Configuration_Statut CMN__Configuration_EcrireValeur(t_CMN__Configuration_Id IdConfiguration,char *section,char *champ,char *valeur);

//         Permet de lire la valeur d'un champ
//         dans une configuration en gestion
t_CMN__Configuration_Statut CMN__Configuration_LireValeur(t_CMN__Configuration_Id IdConfiguration,char *section,char *champ,char **valeur);

//         Permet de detruire une section et tous les champs associes
//         dans une configuration en gestion
t_CMN__Configuration_Statut CMN__Configuration_DetruireSection(t_CMN__Configuration_Id IdConfiguration,char *section);

//         Permet de detruire un champ et la section si le champ etait
//         le dernier dans une configuration en gestion
t_CMN__Configuration_Statut CMN__Configuration_DetruireChamp(t_CMN__Configuration_Id IdConfiguration,char *section,char *champ);


//         permet de sauvegarder une configuration en gestion
//         dans une zone memoire au format configuration.
t_CMN__Configuration_Statut CMN__Configuration_SauverChaine(t_CMN__Configuration_Id IdConfiguration,char **chaine,int *longueur);

//         permet de sauvegarder une configuration en gestion
//         dans un fichier au format configuration.
t_CMN__Configuration_Statut CMN__Configuration_SauverFichier(t_CMN__Configuration_Id IdConfiguration,char *fichier);

//         permet de sauvegarder une configuration en gestion
//         dans un fichier au format configuration a la fin d'un fichier.
t_CMN__Configuration_Statut CMN__Configuration_AjouterFichier(t_CMN__Configuration_Id IdConfiguration,char *fichier);

//         permet de liberer une configuration en gestion.
t_CMN__Configuration_Statut CMN__Configuration_Fermer(t_CMN__Configuration_Id IdConfiguration);


//====================================================================*/
// Declaration des variables externes
//====================================================================*/



#endif
#ifdef __cplusplus
}
#endif
