static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/CMN__Date.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : CMN
//@ CLASSE : CMN__Date
//@ 
//@ Commentaire : 
//@  Ce module gere la classe CMN__Date. Il permet de
//@  faire des calculs et des comparaisons sur les dates.
//@  Il manipule des date comprises entre le 1er Janvier 
//@  1970 et le 5 Fevrier 2036.
//@    
//@  Reference jour Julien standard = 1 Janvier 4712 Avant JC 12:00:00
//@  Reference jour Julien CNES = 1 janvier 1950 Apres JC 00:00:00
//@  Reference jour Julien 2000 = 1 janvier 2000 Apres JC 00:00:00
//@ 					
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 18/04/2000 : G.ROBERT
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 14/12/2000 : C.LACAZE
//@ Portage sous systeme AIX 4.3
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Adaptation a l'OPS : ajout des fonctions DateInit, TransformeDate
//@                      et DateUTC
//@ --------------------------------------------------------------------
//@ VERSION : 12/12/2002 : O. DUFFEZ
//@ Ajout des methodes de gestion des jours julien 2000
//@ et de conversion a partir de dates ASCII long CCSDS
//@ --------------------------------------------------------------------
//@ VERSION : 04/04/2006 : B. GUENDE
//@ FA-ID : IA-2100-FA-175-CN
//@ Correction coquille
//@ See IA-2100-FA-175-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-218-CN : 12/06/2006 : S.BRUNEL
//@ Correction du format de l'heure pour etre conforme a l'UTC (hh:mm:ss.mls)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-3316(DM-2100-OPS) : 18/05/2017 : MPE
//@ PORTAGE OPS SOUS RHEL 6.5 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//
#include <sys/time.h>
#include <iostream>
#include <cstdio>
#include <climits>
#include <ctime>
#include <cmath>

#include "CMN__Commun.h"
#include "CMN__Date.h"
#include "locale.h"

// Migration RHEL 6.5
#ifdef LINUX
#include <cstring>
#include <stdlib.h>
#endif

//
// Definition des constantes
//
const char* FORMAT_IN_JOUR              = "%ld/%ld/%ld%s";
const char* FORMAT_OUT_JOUR             = "%02d/%02d/%04d"; // jj/mm/aaaa

const char* FORMAT_IN_HEURE             = "%ld:%ld:%lf%s";
// FA 218
const char* FORMAT_OUT_LONG_HEURE       = "%02d:%02d:%2.3f"; // hh:mm:ss.mls
const char* FORMAT_OUT_LONG0_HEURE       = "%02d:%02d:0%2.3f"; // hh:mm:ss.mls
// ...FA 218

const char* FORMAT_IN_DATE              = "%s %s %s";
const char* FORMAT_OUT_DATE             = "%s %s";

const char* FORMAT_FRAC_SECONDE         = "%.*f";

const char* VALEUR_INIT_HEURE           = "00:00:00";
const char* CAR_FRAC_SEC                = ".";
const char  VALEUR_CAR_FRAC_SEC         = '.';

// Dates UTC
const char  CAR_ZULU                    = 'Z';
const char* FORMAT_UTC_TIME             = "%04d%02d%02d%02d%02d%02dZ";
const char* FORMAT_NA_UTC_TIME          = "xxxxxxxxxxxxxxZ";
const char* FORMAT_LONG_UTC_TIME        = "%04d%02d%02d%02d%02d%02d%03dZ";
const char* FORMAT_NA_LONG_UTC_TIME     = "xxxxxxxxxxxxxxxxxZ";
const char* FORMAT_LONG_SEP_UTC_TIME    = "%04d%02d%02d%02d%02d%02d.%03dZ";
const char* FORMAT_NA_LONG_SEP_UTC_TIME = "xxxxxxxxxxxxxx.xxxZ";
const int   NB_DIGIT_FRAC_SEC_UTC_TIME  = 3;
const int   NB_TOKEN_UTC_TIME           = 6;
const int   NB_TOKEN_LONG_UTC_TIME      = 7;
const int   NB_TOKEN_LONG_SEP_UTC_TIME  = 7;
const int   TAILLE_UTC_TIME             = 15;
const int   TAILLE_LONG_UTC_TIME        = 18;
const int   TAILLE_LONG_SEP_UTC_TIME    = 19;

const char* FORMAT_CHAINE_OUT           = "%04d %02d %02d %02d %02d %02d.%06d";

// Dates UTC au format ASCII "JJ-MMM-AAAA HH:MN:SS.MLS"
const int   COLONNE_JOUR                = 0; // no de colonne du jour
const int   COLONNE_MOIS                = 3; // no de colonne du mois
const int   COLONNE_ANNEE               = 7; // no de colonne de l'annee
const int   COLONNE_HEURE               = 12;// no de colonne de l'heure
const int   LONGUEUR_JOUR               = 2; // nb de caract. du format jour
const int   LONGUEUR_MOIS               = 3; // nb de caract. du format mois
const int   LONGUEUR_ANNEE              = 4; // nb de caract. du format annee
const int   LONGUEUR_HEURE              = 12;// nb de caract. du format heure
const int   NB_MOIS                     = 12;// nb de mois

/* les mois de l'annee dans un format anglais sur 3 lettres */
const t_Chaine TAB_MOIS_ANGLAIS[NB_MOIS] =
  { "JAN", "FEB", "MAR", "APR", "MAY", "JUN",
    "JUL", "AUG", "SEP", "OCT", "NOV", "DEC" };

const char* C_FMT_FRAC_SEC = "%s";
const char* MOIS_INCONNU   = "mois inconnu";
const char* tabMois[]      =
  {
    "Janvier",
    "Fevrier",
    "Mars",
    "Avril",
    "Mai",
    "Juin",
    "Juillet",
    "Aout",
    "Septembre",
    "Octobre",
    "Novembre",
    "Decembre",
  } ;

// Ecart de jour entre la reference des jours julien CNES et 
// la reference des jours julien standard.
//  Reference jour julien standard = 1er Janvier 4712 Avant JC 12:00:00
//  Reference jour Julien CNES = 1er janvier 1950 Apres JC 00:00:00
const double CEcartJJCnes_JJ = 2433282.5;

// Ecart de jour entre la reference des jours julien 2000 et 
// la reference des jours julien standard.
//  Reference jour julien standard = 1er Janvier 4712 Avant JC 12:00:00
//  Reference jour Julien 2000 = 1er janvier 2000 Apres JC 00:00:00
const double CEcartJJ2000_JJ = 2451544.5;


//======================================================================
//					METHODES PUBLIQUES
//======================================================================

//======================================================================
//					Gestion des heures
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CMN__Heure
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Heure::CMN__Heure()
{
  m_Heure   = 0;
  m_Minute  = 0;
  m_Seconde = 0.0;
  m_Duree   = 0;
  m_Valide  = false;
}


//@ ------------------------------------------------------------------*/
//@ Methode : CMN__Heure
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               (ENTREE) sHeure : Heure sous forme de chaine
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Heure::CMN__Heure (t_Chaine sHeure)
{
  Chaine(sHeure);
}


//@ ------------------------------------------------------------------*/
//@ Methode : CMN__Heure
//@
//@ Description : Constructeur
//@
//@
//@ Parametres :
//@               (ENTREE) sHeure : Chaine a partir de laquelle l'heure
//@                                 doit etre construite 
//@               (ENTREE) sFormat : Format permettant d'analyser la
//@                                  chaine 
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Heure::CMN__Heure(t_Chaine sHeure,t_Chaine sFormat)
{
  Chaine(sHeure,sFormat);
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~CMN__Heure
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Heure::~CMN__Heure()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Chaine
//@
//@ Description : Convertit une heure en chaine de caractere
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               t_Chaine : Heure resultat sous forme de chaine 
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN__Heure::Chaine()
{
  t_ChaineFixe s;
	
  // FA 218
  if (m_Seconde < 10) {
  snprintf(s, CMN__LG_CHAINE_FIXE, FORMAT_OUT_LONG0_HEURE,
       m_Heure,
       m_Minute,
       m_Seconde);
  } else {
         snprintf(s, CMN__LG_CHAINE_FIXE, FORMAT_OUT_LONG_HEURE,
       m_Heure,
       m_Minute,
       m_Seconde);
  
  }
  // ...FA 218

  return(s);  
}


//@ ------------------------------------------------------------------*/
//@ Methode : Chaine
//@
//@ Description : Positionne la valeur d'une heure a partir d'une 
//@               chaine de caractere
//@
//@ Parametres :
//@               (ENTREE) uneHeure : Heure sous forme de chaine
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK : format de la chaine valide
//@               CMN__STATUT_NOK : format de la chaine invalide
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__Heure::Chaine(t_Chaine uneHeure)
{
  t_ChaineFixe Reste;

  Reste[0] = 0;
  int n = sscanf(uneHeure.c_str(), FORMAT_IN_HEURE, 
		 &m_Heure, &m_Minute, &m_Seconde, Reste);

  switch (n)
    {
    case 3:
      break;
    case 2:
      m_Seconde = 0.0;
      break;
    case 1:
      m_Seconde = 0.0;
      m_Minute = 0;
      break;
    default:
      m_Heure = 0;
      m_Minute = 0;
      m_Seconde = 0.0;
      m_Valide = false;
      return CMN__STATUT_NOK;
      break;
    }
  if (uneHeure != Chaine())
    return CMN__STATUT_NOK;

  if ((m_Heure>=0 && m_Heure<=23) && (m_Minute>=0 && m_Minute<=59) && 
      (m_Seconde>=0.0 && m_Seconde<60.0))
    {
      m_Valide = true;
      return CMN__STATUT_OK;
    }
  else
    {
      m_Valide = false;
      m_Heure = 0;
      m_Minute = 0;
      m_Seconde = 0.0;
      return CMN__STATUT_NOK;
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : Chaine
//@
//@ Description : Affecte la valeur de l'heure a partir d'une chaine
//@               et d'un format
//@
//@ Parametres :
//@               (ENTREE) sHeure : Heure sous forme de chaine
//@               (ENTREE) sFormat : Format de la date
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK : format de la chaine valide
//@               CMN__STATUT_NOK : format de la chaine invalide
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__Heure::Chaine(t_Chaine sHeure, t_Chaine sFormat)
{
  t_CMN__Statut Statut=CMN__STATUT_OK;

  struct tm Tm;
  char * pReste;
  t_Chaine FormatReel;
	
  //@ SI le format ne specifie pas les fractions de secondes
  t_Chaine::size_type pos=sFormat.find(C_FMT_FRAC_SEC);
  if ( pos == t_Chaine::npos )
    {
      //@ parser la chaine en entree selon le format
      if ( (pReste=strptime(sHeure.c_str(),sFormat.c_str(),&Tm)) != NULL)
	{
      //@ mettre a jour la forme interne
	  Seconde (Tm.tm_sec); 
	  Minute (Tm.tm_min); 
	  Heure (Tm.tm_hour);
	}
    }
  //@ SINON le format specifie les fractions de secondes
  else
    {
      struct tm TmPrfx;
      //@ Separer le format en 3 parties: 
      //@  . Avant %s
      //@  . %s format pour decoder la partie fractionnaire des secondes
      //@  . Apres %s
      t_Chaine FormatPrfx = sFormat.substr(0,pos);
      t_Chaine FormatSfx = sFormat.substr(pos+strlen(C_FMT_FRAC_SEC),
					  sFormat.length());

      //@ Parser la chaine en entree selon le format Prefixe
      if ( (pReste=strptime(sHeure.c_str(),FormatPrfx.c_str(),&TmPrfx)) != NULL)
	{
	  //@ Recuperer les fractions de secondes
	  int IntFracSeconde;
	  int nScan= sscanf(pReste, "%d", &IntFracSeconde);
	  if (nScan >0)
	    {
	      double NbDigits = floor(log10((double)IntFracSeconde)) + 1;
	      double FracSeconde = IntFracSeconde * pow(10,-NbDigits);
	      char sMilieu[20];

	      snprintf(sMilieu, 20, "%d", IntFracSeconde);

	  //@ Construire Format Reel
	      t_Chaine sFormatReel = FormatPrfx + t_Chaine(sMilieu) + FormatSfx;

	  //@ Parser la chaine en entree selon le format Reel
	      if ( (pReste=strptime(sHeure.c_str(),
				    sFormatReel.c_str(),&Tm)) != NULL)
		{
	  //@ Mettre a jour la forme interne
		  Seconde (Tm.tm_sec + FracSeconde); 
		  Minute (Tm.tm_min); 
		  Heure (Tm.tm_hour);
		}
	    }
	}
    }
  //@ FIN SI

  //@ Mettre a jour le status en fonction de la validite
  Statut = m_Valide ? CMN__STATUT_OK : CMN__STATUT_NOK;

  return Statut;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Formater
//@
//@ Description : Convertit une heure en chaine de caractere suivant
//@               le format.
//@               Le format est celui utilise par la fonction strftime
//@
//@ Parametres :
//@               (ENTREE) Format : format de l'heure
//@               (ENTREE) NbDigitFrac : Nombre de digit des fractions
//@                                      de secondes
//@
//@ Valeur de retour : 
//@               La date formatee ou "" si erreur
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN__Heure::Formater( const char*	Format , 
			       const int		NbDigFracSec)

{
  t_Chaine    sFormat(Format);
  struct tm 	date;
  struct tm	*pDate;
  char		DateTexte[255];
  time_t		time;
  t_Chaine	s="";
  

  //@ SI le format ne specifie pas les fractions de secondes
  t_Chaine::size_type pos=sFormat.find(C_FMT_FRAC_SEC);
  if ( pos == t_Chaine::npos )
    {
      time = long(Duree());
      pDate = localtime_r(&time, &date);
      strftime( DateTexte, 255-1, Format, pDate );
      s= DateTexte;
    }
  //@ SINON le format specifie les fractions de secondes
  else
    {
      char sTemp[20];
      
      //@ Formater les fractions de secondes
      double FracSeconde = Seconde() - floor(Seconde());
      snprintf(sTemp, sizeof(sTemp), FORMAT_FRAC_SECONDE, 
	       NbDigFracSec, FracSeconde);
      char * posPoint = strchr(sTemp,'.');
      if ( posPoint != NULL )
	{
	  t_Chaine sFracSeconde = posPoint + 1;
	  t_Chaine sNewFormat(sFormat);
      //@ Remplacer le %s dans le format par les fractions de seconde
	  sNewFormat.replace(pos,strlen(C_FMT_FRAC_SEC),sFracSeconde);
	 
      //@ Formater avec le nouveau format
	  s=Formater(sNewFormat.c_str(),NbDigFracSec);
	}
    }
  //@ FIN SI
  
  return  s;
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator =
//@
//@ Description : Affectation d'une heure
//@
//@ Parametres :
//@               (ENTREE) uneHeure : Heure a affecter
//@
//@ Valeur de retour : 
//@               CMN__Heure : Heure resultat
//@
//@ ------------------------------------------------------------------*/
CMN__Heure& CMN__Heure::operator =(CMN__Heure& uneHeure)
{
  if (&uneHeure == this)
    {
      return *this;
    }

  m_Heure   = uneHeure.Heure();
  m_Minute  = uneHeure.Minute();
  m_Seconde = uneHeure.Seconde();
  m_Valide  = uneHeure.m_Valide;

  return *this;
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator +
//@
//@ Description : Ajoute une heure
//@
//@ Parametres :
//@               (ENTREE) uneHeure : Heure a ajouter
//@
//@ Valeur de retour : 
//@               CMN__Heure : Heure resultat
//@
//@ ------------------------------------------------------------------*/
CMN__Heure CMN__Heure::operator +(CMN__Heure& uneHeure)
{
  CMN__Heure Heure;

  Heure.Duree (Duree()+uneHeure.Duree());

  return Heure;
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator -
//@
//@ Description : Soustrait une heure
//@
//@ Parametres :
//@               (ENTREE) uneHeure : Heure a soustraire
//@
//@ Valeur de retour : 
//@               CMN__Heure : Heure resultat
//@
//@ ------------------------------------------------------------------*/
CMN__Heure CMN__Heure::operator -(CMN__Heure& uneHeure)
{
  CMN__Heure Heure;

  Heure.Duree (Duree()-uneHeure.Duree());

  return Heure;
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator ==
//@
//@ Description : Teste l'egalite de deux heures
//@
//@ Parametres :
//@               (ENTREE) unJour : Jour de comparaison
//@
//@ Valeur de retour : 
//@               bool : Resultat du test (true si egalite)
//@
//@ ------------------------------------------------------------------*/
bool CMN__Heure::operator ==(CMN__Heure& uneHeure)
{
  return (m_Heure == uneHeure.Heure()) && (m_Minute == uneHeure.Minute()) && 
    (m_Seconde == uneHeure.Seconde());
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator >=
//@
//@ Description : Teste si l'heure est superieure ou egale a l'heure
//@               passee en parametre
//@
//@ Parametres :
//@               (ENTREE) uneHeure : Heure de comparaison
//@
//@ Valeur de retour : 
//@               bool : Resultat du test
//@
//@ ------------------------------------------------------------------*/
bool CMN__Heure::operator >=(CMN__Heure& uneHeure)
{
  return	(m_Heure> uneHeure.Heure()) ||
    (m_Heure == uneHeure.Heure() && m_Minute > uneHeure.Minute()) ||
    (m_Heure == uneHeure.Heure() && m_Minute == uneHeure.Minute() 
     && m_Seconde >= uneHeure.Seconde());		
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator <=
//@
//@ Description : Teste si l'heure est inferieure ou egale a l'heure
//@               passee en parametre
//@
//@ Parametres :
//@               (ENTREE) uneHeure : Heure de comparaison
//@
//@ Valeur de retour : 
//@               bool : Resultat du test
//@
//@ ------------------------------------------------------------------*/
bool CMN__Heure::operator <=(CMN__Heure& uneHeure)
{
  return	(m_Heure< uneHeure.Heure()) ||
    (m_Heure == uneHeure.Heure() && m_Minute < uneHeure.Minute()) ||
    (m_Heure == uneHeure.Heure() && m_Minute == uneHeure.Minute()
     && m_Seconde <= uneHeure.Seconde());
			
}


//@ ------------------------------------------------------------------*/
//@ Methode : MilliSeconde
//@
//@ Description : Retourne le nombre de milisecondes
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Nombre de milisecondes
//@
//@ ------------------------------------------------------------------*/
int CMN__Heure::MilliSeconde()
{
  int          milli = 0;
  double       fracSeconde = 0.0;
  t_ChaineFixe fracTemp;
  t_Chaine     sFracSeconde = "0";
  char*        posPoint = NULL;


  fracSeconde = m_Seconde - floor(m_Seconde);
  
  snprintf(fracTemp, sizeof(fracTemp), FORMAT_FRAC_SECONDE, 
	   NB_DIGIT_FRAC_SEC_UTC_TIME, fracSeconde);

  posPoint = strchr(fracTemp, VALEUR_CAR_FRAC_SEC);
  if (posPoint != NULL)
    {
      sFracSeconde = posPoint + 1;
    }
  
  milli = int(sFracSeconde);
  

  return (milli);
}


//======================================================================
//					METHODES PRIVEES de CMN__Heure
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : TesterValidite
//@
//@ Description : Teste la validite d'une heure
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__Heure::TesterValidite(  )
{
  m_Valide = ((m_Minute>=0 && m_Minute<=59) && 
	      (m_Seconde>=0.0 && m_Seconde<60.0));
}


//@ ------------------------------------------------------------------*/
//@ Methode : ConvertirHeureEnDuree
//@
//@ Description : Convertit une heure en Duree
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__Heure::ConvertirHeureEnDuree(  )
{
  m_Duree = m_Heure * NB_SECONDES_PAR_HEURE +
    m_Minute * NB_SECONDES_PAR_MINUTE + 
    m_Seconde ;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ConvertirDureeEnHeure
//@
//@ Description : Convertit la Duree en heure 
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__Heure::ConvertirDureeEnHeure(  )
{
  long Sens = 1;
  

  if ( m_Duree < 0.0 )
    {
    Sens = -1;
    m_Duree = -m_Duree;
  }
 
  // FT-2765
  m_Heure = static_cast<int>(m_Duree / NB_SECONDES_PAR_HEURE);
  // ...FT-2765
  m_Duree -= m_Heure * NB_SECONDES_PAR_HEURE ;
  // FT-2765
  m_Minute = static_cast<int>(m_Duree / NB_SECONDES_PAR_MINUTE);
  // ...FT-2765
  m_Duree -= m_Minute * NB_SECONDES_PAR_MINUTE;
  m_Seconde = m_Duree;
  m_Heure = m_Heure * Sens;
}


//======================================================================
//					Gestion des Jours
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CMN__Jour
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Jour::CMN__Jour()
{
  m_Jour        = 0;
  m_JourSemaine = 0;
  m_Mois        = 0;
  m_Annee       = 0;
  m_Valide      = false;
}


//@ ------------------------------------------------------------------*/
//@ Methode : CMN__Jour
//@
//@ Description : Constructeur a partir d'une chaine
//@
//@ Parametres :
//@               (ENTREE) sJour : jour sous forme de chaine
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Jour::CMN__Jour(t_Chaine sJour)
{
  Chaine(sJour);
}


//@ ------------------------------------------------------------------*/
//@ Methode : CMN__Jour
//@
//@ Description : Constructeur a partir d'une chaine
//@
//@ Parametres :
//@               (ENTREE) sJour : jour sous forme de chaine
//@               (ENTREE) sFormat : format de la chaine
//@               (ENTREE) sLangue : langue a utiliser
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Jour::CMN__Jour(t_Chaine sJour, t_Chaine sFormat, t_Chaine sLangue)
{
  Chaine(sJour, sFormat, sLangue);
}


//@ ------------------------------------------------------------------*/
//@ Methode : Chaine
//@
//@ Description : Affecte la valeur du jour a partir d'une chaine et
//@               d'un format
//@
//@ Parametres :
//@               (ENTREE) sJour : Jour sous forme de chaine
//@               (ENTREE) sFormat : Format du jour
//@               (ENTREE) sLangue : Langue pour le decodage de la date
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK : Format de la chaine valide
//@               CMN__STATUT_NOK : Format de la chaine invalide
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__Jour::Chaine(t_Chaine sJour, t_Chaine sFormat, t_Chaine sLangue)
{
  t_CMN__Statut Statut = CMN__STATUT_OK;

  struct tm Tm;
  char * pReste;
  t_Chaine FormatReel;

  //@ Memorisation du LcTime courant
  t_Chaine OldLcTime=setlocale(LC_TIME,NULL);

  //@ Positionnement du Nouveau LcTime 
  setlocale(LC_TIME,sLangue.c_str());
	
  //@ Parser la chaine en entree selon le format
  if ( (pReste=strptime(sJour.c_str(),sFormat.c_str(),&Tm)) != NULL)
    {
  //@ Mettre a jour la forme interne
      Jour(Tm.tm_mday);
      Mois(Tm.tm_mon+1);
      Annee(Tm.tm_year+1900);
    }
  
  //@ Mettre a jour le statut de retour
  Statut = m_Valide ? CMN__STATUT_OK : CMN__STATUT_NOK;

  //@ Restauration de l'ancien LcTime
  setlocale(LC_TIME,OldLcTime.c_str());

  return Statut;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~CMN__Jour
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Jour::~CMN__Jour()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : NomMois
//@
//@ Description : Extrait le mois de la date
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Le mois de la date
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN__Jour::NomMois()
{
  if (m_Mois<1 || m_Mois> 12)
    {
      return(MOIS_INCONNU);
    }
  else
    {
      return(tabMois[m_Mois-1]);
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : Chaine
//@
//@ Description : Retourne la date sous forme de chaine
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Date sous forme de chaine (format jj/mm/aaaa)
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN__Jour::Chaine()
{
  t_ChaineFixe s;

  snprintf(s, CMN__LG_CHAINE_FIXE, FORMAT_OUT_JOUR, m_Jour, m_Mois, m_Annee);
  
  return s;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Chaine
//@
//@ Description : Affecte la valeur de la date a partir d'une chaine
//@
//@ Parametres :
//@               (ENTREE) sDate : Date sous forme de chaine (format
//@                                jj/mm/aaaa)
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Format de la chaine valide
//@               CMN__STATUT_NOK : Format de la chaine invalide
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__Jour::Chaine(t_Chaine sDate)
{
  t_ChaineFixe Reste;

  Reste[0] = 0;
  if (sscanf(sDate.c_str(), FORMAT_IN_JOUR, 
	     &m_Jour, &m_Mois, &m_Annee, Reste) == 3 )
    {
      int Jour = m_Jour;
      int Mois = m_Mois;
      int Annee = m_Annee;

      if (NormaliserDate() == CMN__STATUT_OK)
	{
	  /* SMM-M4-FA-68-CN on verifie que la date est valide */
	  if ((Jour == m_Jour) && (Mois == m_Mois) && (Annee == m_Annee))
	    {
	      return CMN__STATUT_OK;
	    }
	}
    }

  m_Jour = 0;
  m_JourSemaine = 0;
  m_Mois = 0;
  m_Annee = 0;
  m_Valide = false;
  
  return CMN__STATUT_NOK;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Formater
//@
//@ Description : Convertit un jour en chaine de caractere suivant le
//@               format.
//@               Le format est celui utilise par la fonction strftime
//@
//@ Parametres :
//@               (ENTREE) Format : format du jour
//@               (ENTREE) sLangue : Langue
//@
//@ Valeur de retour : 
//@               La date formatee ou "" si erreur
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN__Jour::Formater( const char*	Format , 
			      const t_Chaine sLangue)

{
  t_Chaine    sFormat(Format);
  struct tm       date;
  struct tm       *pDate;
  char            DateTexte[255];
  time_t          time;
  t_Chaine        s="";
  double Temps;

  //@ Memorisation du LcTime courant
  t_Chaine OldLcTime = setlocale(LC_TIME,NULL);

  //@ Positionnement du Nouveau LcTime
  setlocale(LC_TIME, sLangue.c_str());

  //@ Conversion
  if ( m_Annee < 1900 )
    {
      s = "";
    }
  else
    {
      Temps = ConvertirJourEnTemps();
      time = long(Temps);
      pDate = localtime_r(&time, &date);
      strftime( DateTexte, 255-1, Format, pDate );
      s= DateTexte;
    }

  //@ Restauration de l'ancien LcTime
  setlocale(LC_TIME,OldLcTime.c_str());

  return  s;
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator =
//@
//@ Description : Operation d'affection entre deux jours
//@
//@ Parametres :
//@               (ENTREE) unJour : Jour origine
//@
//@ Valeur de retour : 
//@               CMN__Jour& : Jour destination
//@
//@ ------------------------------------------------------------------*/
CMN__Jour& CMN__Jour::operator =(CMN__Jour& unJour)
{
  if (&unJour == this)
    {
      return *this;
    }

  m_Annee  = unJour.Annee();
  m_Mois   = unJour.Mois();
  m_Jour   = unJour.Jour();
  m_Valide = unJour.m_Valide;

  return *this;
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator ==
//@
//@ Description : Teste l'egalite de deux jours
//@
//@ Parametres :
//@               (ENTREE) unJour : Jour de comparaison
//@
//@ Valeur de retour : 
//@               bool : Resultat du test
//@
//@ ------------------------------------------------------------------*/
bool CMN__Jour::operator ==(CMN__Jour& unJour)
{
  return (m_Jour==unJour.Jour() && m_Mois == unJour.Mois() 
	  && m_Annee == unJour.Annee());
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator >=
//@
//@ Description : Teste si le Jour est superieur ou egal au Jour passe
//@               en parametre
//@
//@ Parametres :
//@               (ENTREE) unJour : Jour de comparaison
//@
//@ Valeur de retour : 
//@               bool : Resultat du test
//@
//@ ------------------------------------------------------------------*/
bool CMN__Jour::operator >=(CMN__Jour& unJour)
{
  return	(m_Annee > unJour.Annee()) || 
    (m_Annee == unJour.Annee() && m_Mois > unJour.Mois())  ||
    (m_Annee == unJour.Annee() && m_Mois == unJour.Mois() 
     && m_Jour >= unJour.Jour()) ;		
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator <=
//@
//@ Description : Teste si le Jour est inferieur ou egal au Jour passe
//@               en parametre
//@
//@ Parametres :
//@               (ENTREE) unJour : Jour de comparaison
//@
//@ Valeur de retour : 
//@               bool : Resultat du test
//@
//@ ------------------------------------------------------------------*/
bool CMN__Jour::operator <=(CMN__Jour& unJour)
{
  return	(m_Annee < unJour.Annee()) || 
    (m_Annee == unJour.Annee() && m_Mois < unJour.Mois())  ||
    (m_Annee == unJour.Annee() && m_Mois == unJour.Mois()
     && m_Jour <= unJour.Jour()) ;
}


//======================================================================
//					METHODES PRIVEES de CMN__Jour
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : TesterValidite
//@
//@ Description : Verifie q'un jour est coherent 
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__Jour::TesterValidite(  )
{
  CMN__Jour JourTmp(*this);

  //@ Normalisation
  JourTmp.NormaliserDate();

  //@ Si la date n'a pas bouge apres normalisation, c'est qu'elle
  //@ etait coherente
  m_Valide = (JourTmp == *this);
}


//@ ------------------------------------------------------------------*/
//@ Methode : NormaliserDate
//@
//@ Description : Normalise une date
//@               Le jour doit etre superieur au 01/01/1970
//@	
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK : le jour est valide
//@               CMN__STATUT_NOK : le jour est invalide
//@			
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__Jour::NormaliserDate()
{
  struct tm Date;
  struct tm *pDate;
  time_t Temps;

  Date.tm_sec 	= 0;
  Date.tm_min 	= 0;
  Date.tm_hour 	= 0;

  Date.tm_mday 	= m_Jour;
  Date.tm_mon 	= m_Mois-1;
  Date.tm_year 	= m_Annee-1900;

  Date.tm_isdst = 0;

  //@ Convertit la date en une date normalisee
  if ( (Temps = mktime(&Date)) == (time_t)-1 )
    {
      m_Valide = false;

      return CMN__STATUT_NOK;
    }
  else 
    {
  //@ Prend en compte la zone de temps locale
      pDate = localtime_r(&Temps, &Date) ;
      m_Annee =pDate->tm_year+1900;
      m_Mois =pDate->tm_mon+1;
      m_Jour =pDate->tm_mday;
      m_JourSemaine =(pDate->tm_wday+6) % 7 ;
      m_Valide = true;
      
      return CMN__STATUT_OK;
    }

  return CMN__STATUT_OK;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ConvertirJourEnTemps
//@
//@ Description : Convertit un jour en temps
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
double CMN__Jour::ConvertirJourEnTemps()
{
  struct tm Date;
  double FracSeconde=0.0;
  double Temps;


  //@ Verifier si la date est posterieure au 1er Janvier 70
  if (m_Annee >= 1970)
    {
      Date.tm_sec 	= 0;
      Date.tm_min 	= 0;
      Date.tm_hour 	= 0;

      Date.tm_mday 	= m_Jour;
      Date.tm_mon 	= m_Mois-1;
      Date.tm_year 	= m_Annee-1900;

      Date.tm_isdst = -1;
  
      Temps = mktime(&Date);
    }
  else
    {
      Temps = 0.0;
    }

  return Temps;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ConvertirTempsEnJour
//@
//@ Description : Convertit le temps en un jour
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__Jour::ConvertirTempsEnJour( double Temps )
{
  struct tm 	Date;
  struct tm *	pDate;
  time_t		time;

  // FT-2765
  time = static_cast<time_t>(Temps);
  // ...FT-2765
  pDate = localtime_r(&time, &Date);

  Jour(Date.tm_mday);
  Mois(Date.tm_mon+1);
  Annee(Date.tm_year+1900);
}


//======================================================================
//					Gestion des dates
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CMN__Date
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Date::CMN__Date() : m_Heure(), m_Jour()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : CMN__Date
//@
//@ Description : Constructeur de recopie
//@
//@ Parametres :
//@               (ENTREE) Date : Date a recopier
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Date::CMN__Date(const CMN__Date & Date) : m_Heure(Date.m_Heure), 
					       m_Jour(Date.m_Jour)
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : CMN__Date
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               (ENTREE) sDate : Chaine a partir de laquelle la date
//@                                doit etre construite 
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Date::CMN__Date(const t_Chaine sDate)
{
  Chaine(sDate);
}


//@ ------------------------------------------------------------------*/
//@ Methode : CMN__Date
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               (ENTREE) sDate : Chaine a partir de laquelle la date
//@                                doit etre construite 
//@               (ENTREE) sFormat : Format permettant d'analyser la
//@                                  chaine 
//@               (ENTREE) sLangue : Langue selon laquelle la chaine
//@                                  est analysee
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Date::CMN__Date(const t_Chaine sDate, t_Chaine sFormat, t_Chaine sLangue)
{
  Chaine(sDate, sFormat, sLangue);
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~CMN__Date
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Date::~CMN__Date()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : DateCourante
//@
//@ Description : Positionne la date a la date du Jour
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune 
//@
//@ ------------------------------------------------------------------*/
void CMN__Date::DateCourante()
{
  struct tm date;
  struct tm *pDate;
  time_t now = (long)time(0) ;

  pDate = localtime_r(&now, &date) ;

  m_Heure.Seconde(pDate->tm_sec); 
  m_Heure.Minute(pDate->tm_min); 
  m_Heure.Heure(pDate->tm_hour);
  
  m_Jour.Jour(pDate->tm_mday);
  m_Jour.Mois (pDate->tm_mon+1);
  m_Jour.Annee (pDate->tm_year+1900);
}


//@ ------------------------------------------------------------------*/
//@ Methode : DateCourantePrecise
//@
//@ Description : Recupere la date courante en microsecondes
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune 
//@
//@ ------------------------------------------------------------------*/
void CMN__Date::DateCourantePrecise()
{
  struct timeval time_val;
  struct timezone time_zone;
  double Temps;


  //@ Recuperer la date courante en millisecondes
  gettimeofday(&time_val,&time_zone);

  Temps = time_val.tv_sec + (time_val.tv_usec/1000000);

  ConvertirTempsEnDate(Temps);
}


//@ ------------------------------------------------------------------*/
//@ Methode : LaDateCourantePrecise
//@
//@ Description : Fournit la date courante en microsecondes
//@
//@ Parametres :
//@               (ENTREE) precision : Precision du champ tv_usec (0
//@                                    a 6)
//@               (SORTIE) laDate : La date courante au format timeval
//@
//@ Valeur de retour : 
//@               Aucune 
//@
//@ ------------------------------------------------------------------*/
void CMN__Date::LaDateCourantePrecise(int precision, timeval& laDate)
{
  double Temps;

  Temps = ConvertirDateEnTemps();

  laDate.tv_sec = (time_t)Temps;

  // FT-2765
  laDate.tv_usec = static_cast<suseconds_t>((Temps - laDate.tv_sec)*(10*precision));
  // ...FT-2765
}


//@ ------------------------------------------------------------------*/
//@ Methode : DateInit
//@
//@ Description : Initialise la date au 1er janvier 2000
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune 
//@
//@ ------------------------------------------------------------------*/
void CMN__Date::DateInit()
{
  m_Heure.Seconde(0); 
  m_Heure.Minute(0); 
  m_Heure.Heure(0);
  
  m_Jour.Jour(1);
  m_Jour.Mois(1);
  m_Jour.Annee(2000);
}


//@ ------------------------------------------------------------------*/
//@ Methode : JourJulien
//@
//@ Description : Retourne le Jour Julien
//@               Nombre de jours depuis le 1er Janvier 4712 Avant JC
//@               a 12h 0m 0s		
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Nombre de jours julien 
//@
//@ ------------------------------------------------------------------*/
double CMN__Date::JourJulien()
{
  double x_MM;
  double x_DD;
  double x_YY;
  double x_HR;
  double x_MN;
  double x_SS;
  double x_GGG;
  double x_JD;
  double x_A;
  double x_J1;
  double x_S;

  // Recuperation de Mois, Jour, Annee, Heure, Minute, Seconde
  x_MM = Jour().Mois();
  x_DD = Jour().Jour();
  x_YY = Jour().Annee();
  x_HR = Heure().Heure();
  x_MN = Heure().Minute();
  x_SS = Heure().Seconde();

  x_HR = x_HR + (x_MN / 60) + (x_SS / 3600);
  x_GGG = 1;
  if( x_YY < 1582 ) x_GGG = 0;
  if( x_YY <= 1582 && x_MM < 10 ) x_GGG = 0;
  if( x_YY <= 1582 && x_MM == 10 && x_DD < 5 ) x_GGG = 0;
  x_JD = -1 * floor(7 * (floor((x_MM + 9) / 12) + x_YY) / 4);
  x_S = 1;
  if ((x_MM - 9)<0) x_S=-1;
  x_A = abs(x_MM - 9);
  x_J1 = floor(x_YY + x_S * floor(x_A / 7));
  x_J1 = -1 * floor((floor(x_J1 / 100) + 1) * 3 / 4);
  x_JD = x_JD + floor(275 * x_MM / 9) + x_DD + (x_GGG * x_J1);
  x_JD = x_JD + 1721027 + 2 * x_GGG + 367 * x_YY - 0.5;
  x_JD = x_JD + (x_HR / 24);

  return (x_JD);
}


//@ ------------------------------------------------------------------*/
//@ Methode : JourJulienCnes
//@
//@ Description : Retourne le Jour Julien CNES 
//@               Nombre de jours depuis le 1er Janvier 1950 a 0h 0m 0s
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Nombre de jours julien CNES 
//@
//@ ------------------------------------------------------------------*/
double CMN__Date::JourJulienCnes()
{
  // Jour julien Cnes = Jour julien standard - ecart entre 1er Janvier
  // 1950 00:00:00 et le 1er Janvier 4712 Avant JC 12:00:00
  return ( JourJulien() - CEcartJJCnes_JJ);
}


//@ ------------------------------------------------------------------*/
//@ Methode : JourJulien2000
//@
//@ Description : Retourne le Jour Julien 2000
//@               Nombre de jours depuis le 1er Janvier 2000 a 0h 0m 0s
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Nombre de jours julien 2000
//@
//@ ------------------------------------------------------------------*/
double CMN__Date::JourJulien2000()
{
  // Jour julien 2000 = Jour julien standard - ecart entre 1er Janvier
  // 2000 00:00:00 et le 1er Janvier 4712 Avant JC 12:00:00
  return ( JourJulien() - CEcartJJ2000_JJ);
}

//@ ------------------------------------------------------------------*/
//@ Methode : SCDSTime
//@
//@ Description : Retourne la date au format
//@               t_SCDSTime, structure constituee de :
//@                unsigned short int day; // day since 1 january 2000
//@                unsigned int millis; // number of ms in the day
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               date SCDSTime
//@
//@ ------------------------------------------------------------------*/
t_SCDSTime CMN__Date::SCDSTime()
{
  // Jour julien 2000 = Jour julien standard - ecart entre 1er Janvier
  double jj2000;
  double fract_millis; // partie fractionnaire (exprimee en millisecondes)
  t_SCDSTime dateSCDSTime;

  // on recupere le nb de jours julien 2000 a partir de l'objet courant
  jj2000 = JourJulien2000();

  // mise a jour du nb de jours
  dateSCDSTime.day = (int)floor(jj2000);

  // partie fractionnaire (exprimee en millisecondes)
  fract_millis = (jj2000 - dateSCDSTime.day) * NB_MILLIS_JOUR;

  // mise a jour du nb de millisecondes
  dateSCDSTime.millis = (int)floor(fract_millis + 0.5);

  return(dateSCDSTime);
}


//@ ------------------------------------------------------------------*/
//@ Methode : LCDSTime
//@
//@ Description : Retourne la date au format
//@               t_LCDSTime, structure constituee de :
//@                unsigned short int day; // day since 1 january 2000
//@                unsigned int millis; // number of ms in the day
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               date LCDSTime
//@
//@ ------------------------------------------------------------------*/
t_LCDSTime CMN__Date::LCDSTime()
{
  // Jour julien 2000 = Jour julien standard - ecart entre 1er Janvier
  double jj2000;
  double fract_millis; // partie fractionnaire (exprimee en millisecondes)
  double fract_micros; // partie fractionnaire (exprimee en microsecondes)
  t_LCDSTime dateLCDSTime;

  // on recupere le nb de jours julien 2000 a partir de l'objet courant
  jj2000 = JourJulien2000();

  // mise a jour du nb de jours
  dateLCDSTime.day = (int)floor(jj2000);

  // partie fractionnaire (exprimee en millisecondes)
  fract_millis = (jj2000 - dateLCDSTime.day) * NB_MILLIS_JOUR;

  // mise a jour du nb de millisecondes
  dateLCDSTime.millis = (int)floor(fract_millis);

  // partie fractionnaire (exprimee en millisecondes)
  fract_micros = (fract_millis - dateLCDSTime.millis) * NB_MICROS_JOUR / NB_MILLIS_JOUR;

  // mise a jour du nb de microsecondes
  dateLCDSTime.micros = (int)floor(fract_micros);

  return(dateLCDSTime);
}


//@ ------------------------------------------------------------------*/
//@ Methode : JourJulien
//@
//@ Description : Construire la date a partir du nombre de jours Julien 
//@               		
//@ Parametres :
//@               (ENTREE) JourJulien : Nombre de jours  
//@
//@ Valeur de retour : 
//@               Aucune 
//@
//@ ------------------------------------------------------------------*/
void CMN__Date::JourJulien(const double JourJulien)
{
  double JD = JourJulien;
  double Z;
  double F;
  double A;
  double B;
  double C;
  double D;
  double I;
  double T;
  double RJ;
  double JJ;
  double RH;
  double MM;
  double AA;
  double Hr;
  double Mn;
  double Sec;
	
  //@ Convertir le jour julien en temps UTC
  Z = floor(JD+0.5);
  F = JD+0.5 - Z;
  if (Z < 2299161) 
    {
      A = Z;
    } 
  else
    {
      I = floor((Z - 1867216.25)/36524.25);
      A = Z + 1 + I - floor(I/4);
    }
  B = A + 1524;
  C = floor((B - 122.1)/365.25);
  D = floor(365.25 * C);
/* FA175 */
  T = floor((B - D)/ 30.6001);
/* Fin FA175 */
  RJ = B - D - floor(30.6001 * T) + F;
  JJ = floor(RJ);
  RH = (RJ - floor(RJ)) * 24;

  Hr = floor(RH);
  Mn = floor((RH - Hr)*60);
  Sec = ((RH - Hr )*60 - Mn )*60;

  if (T < 14) 
    {
      MM = T - 1;
    } 
  else
    { 
      if ((T == 14) || (T == 15))  
	MM = T - 13;
    } 
  if (MM > 2) 
    {
      AA = C - 4716 ;
    } 
  else
    { 
      if ((MM == 1) || (MM == 2)) 
	AA = C - 4715;
    }

  //@ Metttre a jour la representation interne
  // FT-2765
  Heure().Heure(static_cast<int>(Hr));
  Heure().Minute(static_cast<int>(Mn));
  Heure().Seconde(static_cast<double>(Sec));
  Jour().Jour(static_cast<int>(JJ));
  Jour().Mois(static_cast<int>(MM));
  Jour().Annee(static_cast<int>(AA));
  // ...FT-2765
}


//@ ------------------------------------------------------------------*/
//@ Methode : JourJulienCnes
//@
//@ Description : Construire la date a partir du nombre de jours
//@               Julien Cnes
//@               		
//@ Parametres :
//@               (ENTREE) JourJulienCnes : Nombre de jours julien CNES
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__Date::JourJulienCnes(const double JJCnes)
{
  // Convertir le jour julien CNES en jour Julien standard
  JourJulien(JJCnes + CEcartJJCnes_JJ);
}


//@ ------------------------------------------------------------------*/
//@ Methode : JourJulien2000
//@
//@ Description : Construit la date a partir du nombre de jours
//@               Julien 2000
//@               		
//@ Parametres :
//@               (ENTREE) JourJulien2000 : Nombre de jours julien 2000
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__Date::JourJulien2000(const double JJ2000)
{
  // Convertir le jour julien CNES en jour Julien standard
  JourJulien(JJ2000 + CEcartJJ2000_JJ);
}

//@ ------------------------------------------------------------------*/
//@ Methode : SCDSTime
//@
//@ Description : Construit la date a partir d'une date au format
//@               t_SCDSTime, structure constituee de :
//@                unsigned short int day; // day since 1 january 2000
//@                unsigned int millis; // number of ms in the day
//@               		
//@ Parametres :
//@               (ENTREE) dateSCDSTime : date SCDSTime
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__Date::SCDSTime(const t_SCDSTime dateSCDSTime)
{
  double jj2000;

  // creation du nombre de jours julien 2000
  jj2000 = (double)(dateSCDSTime.day +
                    dateSCDSTime.millis / NB_MILLIS_JOUR);

  // creation de la date a partir du nb de jours Julien 2000
  JourJulien2000(jj2000);
}


//@ ------------------------------------------------------------------*/
//@ Methode : LCDSTime
//@
//@ Description : Construit la date a partir d'une date au format
//@               t_LCDSTime, structure constituee de :
//@                unsigned short int day; // day since 1 january 2000
//@                unsigned int millis; // number of ms in the day
//@                unsigned short int micros; // number of micros in the day
//@               		
//@ Parametres :
//@               (ENTREE) dateLCDSTime : date LCDSTime
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__Date::LCDSTime(const t_LCDSTime dateLCDSTime)
{
  double jj2000;

  // creation du nombre de jours julien 2000
  jj2000 = (double)(dateLCDSTime.day +
                    dateLCDSTime.millis / NB_MILLIS_JOUR +
                    dateLCDSTime.micros / NB_MICROS_JOUR);

  // creation de la date a partir du nb de jours Julien 2000
  JourJulien2000(jj2000);
}


//@ ------------------------------------------------------------------*/
//@ Methode : Formater
//@
//@ Description : Convertit une date en chaine de caractere suivant 
//@               le format.
//@               Le format est celui utilise par la fonction strftime
//@
//@ Parametres :
//@               (ENTREE) Format : format de la date
//@               (ENTREE) NbDigitFrac : format de la date
//@               (ENTREE) sLangue : format de la date
//@
//@ Valeur de retour : 
//@               La date formatee ou "" si erreur
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN__Date::Formater( const char*	Format , 
			      const int NbDigFracSec,
			      const t_Chaine sLangue)

{
  t_Chaine    sFormat(Format);
  struct tm 	date;
  struct tm	*pDate;
  char		DateTexte[255];
  time_t		time;
  t_Chaine	s="";
  double Temps;
		
  //@ Memorisation du LcTime courant
  t_Chaine OldLcTime=setlocale(LC_TIME,NULL);

  //@ Positionnement du Nouveau LcTime 
  setlocale(LC_TIME,sLangue.c_str());

  //@ SI le format ne specifie pas les fractions de secondes
  t_Chaine::size_type pos=sFormat.find(C_FMT_FRAC_SEC);
  if ( pos == t_Chaine::npos )
    {
      if ( m_Jour.Annee() < 1900 )
	s = "";
      else
	{
	  Temps = ConvertirDateEnTemps();
	  time = long(Temps);
	  pDate = localtime_r(&time, &date);
	  strftime( DateTexte, 255-1, Format, pDate );
	  s= DateTexte;
	}
    }
  //@ SINON le format specifie les fractions de secondes
  else 
    {
      char sTemp[20];
      //@ Formater les fractions de secondes
      double FracSeconde = m_Heure.Seconde() - floor(m_Heure.Seconde());

      snprintf(sTemp, sizeof(sTemp), "%.*f", NbDigFracSec, FracSeconde);

      char * posPoint = strchr(sTemp,'.');
      if ( posPoint != NULL )
	{
	  t_Chaine sFracSeconde = posPoint + 1;
	  t_Chaine sNewFormat(sFormat);
	 
	  //@ Remplacer le %s dans le format par les fractions de seconde
	  sNewFormat.replace(pos,strlen(C_FMT_FRAC_SEC),sFracSeconde);
	  
	  //@ Formater avec le nouveau format
	  s=Formater(sNewFormat.c_str(),NbDigFracSec,sLangue);
	}

    }
  //@ FIN SI

  //@ Restauration de l'ancien LcTime
  setlocale(LC_TIME,OldLcTime.c_str());

  return  s;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Chaine
//@
//@ Description : Retourne la date sous forme de chaine
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Date sous forme de chaine
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN__Date::Chaine()
{
  t_ChaineFixe s;

  snprintf(s, CMN__LG_CHAINE_FIXE, FORMAT_OUT_DATE,
	   m_Jour.Chaine().c_str(), m_Heure.Chaine().c_str());

  return s;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Chaine
//@
//@ Description : Affecte la valeur de la date a partir d'une chaine
//@
//@ Parametres :
//@               (ENTREE) sDate : Date sous forme de chaine
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK : format de la chaine valide
//@               CMN__STATUT_NOK : format de la chaine invalide
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__Date::Chaine(t_Chaine sDate)
{
  int n;
  t_ChaineFixe Reste;
  t_ChaineFixe Jour;
  t_ChaineFixe Heure;
  t_Chaine HeureZero;

  Reste[0] = 0;
  n = sscanf(sDate.c_str(), FORMAT_IN_DATE, Jour, Heure, Reste);
  switch (n)
    {
    case 2:
      m_Jour.Chaine(Jour);
      m_Heure.Chaine(Heure);
      if (!m_Jour.EstValide())
	{
	  return CMN__STATUT_NOK;
	}
      if (!m_Heure.EstValide())
	{
	  return CMN__STATUT_NOK;
	}
      break;

    case 1:
      m_Jour.Chaine(Jour);
      if (!m_Jour.EstValide())
	{
	  return CMN__STATUT_NOK;
	}
	
      // On positionne l'heure a zero quand seul le jour est connu
      HeureZero = VALEUR_INIT_HEURE;
      m_Heure.Chaine(HeureZero);
      break;

    default :
      return CMN__STATUT_NOK;
      break;
    }

  return CMN__STATUT_OK;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Chaine
//@
//@ Description : Affecte la valeur de la date a partir d'une chaine
//@               et d'un format
//@
//@ Parametres :
//@               (ENTREE) sDate : Date sous forme de chaine
//@               (ENTREE) sFormat : Format de la date
//@               (ENTREE) sLangue : Langue pour le decodage de la date
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK : format de la chaine valide
//@               CMN__STATUT_NOK : format de la chaine invalide
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__Date::Chaine(t_Chaine sDate, t_Chaine sFormat, t_Chaine sLangue)
{
  t_CMN__Statut statut = CMN__STATUT_OK;

  struct tm Tm;
  char * pReste;
  t_Chaine FormatReel;


  //@ Memorisation du LcTime courant
  t_Chaine OldLcTime = setlocale(LC_TIME,NULL);

  //@ Positionnement du Nouveau LcTime 
  setlocale(LC_TIME, sLangue.c_str());
	
  //@ SI le format ne specifie pas les fractions de secondes
  t_Chaine::size_type pos = sFormat.find(C_FMT_FRAC_SEC);
  if ( pos == t_Chaine::npos )
    {
      //@ Parser la chaine en entree selon le format
      //@ SI le parsing a echoue
      if ( (pReste=strptime(sDate.c_str(),sFormat.c_str(),&Tm)) == NULL)
	{
	  //@ Mettre a jour le statut en erreur
	  statut = CMN__STATUT_NOK;
	}
      //@ SINON
      else
	{
	  //@ Mettre a jour la forme interne
	  Heure().Seconde (Tm.tm_sec); 
	  Heure().Minute (Tm.tm_min); 
	  Heure().Heure (Tm.tm_hour);

	  Jour().Jour(Tm.tm_mday);
	  Jour().Mois(Tm.tm_mon+1);
	  Jour().Annee(Tm.tm_year+1900);
	}
      //@ FIN SI le parsing a echoue
    }
  //@ SINON le format specifie les fractions de secondes
  else
    {
      struct tm TmPrfx;

      //@ Separer le format en 3 parties: 
      //@  . Avant %s
      //@  . %s format pour decoder la partie fractionnaire des
      //@    secondes
      //@  . Apres %s
      t_Chaine FormatPrfx = sFormat.substr(0,pos);
      t_Chaine FormatSfx = sFormat.substr(pos+strlen(C_FMT_FRAC_SEC),
					  sFormat.length());

      //@ Parser la chaine en entree selon le format Prefixe
      //@ SI le parsing a echoue
      if ( (pReste=strptime(sDate.c_str(),FormatPrfx.c_str(),&TmPrfx)) == NULL)
	{
	  //@ Mettre a jour le statut en erreur
	  statut = CMN__STATUT_NOK;
	}
      //@ SINON
      else
	{
	  //@ On recupere tous les digits des fractions de secondes
	  //@ (meme les 0 au debut)
	  char szFracSeconde[30];
	  int nScan = sscanf(pReste, "%[0-9]", szFracSeconde);

	  //@ SI la recuperation a echoue
	  if (nScan <= 0)
	    {
	      //@ Mettre a jour le statut en erreur
	      statut = CMN__STATUT_NOK;
	    }
	  //@ SINON
	  else
	    {
	      double NbDigits = strlen(szFracSeconde);
	      int IntFracSeconde = atoi(szFracSeconde);
	      double FracSeconde = IntFracSeconde * pow(10,-NbDigits);

	      //@ Construire Format Reel
	      t_Chaine sFormatReel = FormatPrfx + t_Chaine(szFracSeconde) +
		FormatSfx;

	      //@ Parser la chaine en entree selon le format Reel
	      //@ SI le parsing a echoue
	      if ( (pReste=strptime(sDate.c_str(),sFormatReel.c_str(),&Tm)) == NULL)
		{
		  //@ Mettre a jour le statut en erreur
		  statut = CMN__STATUT_NOK;
		}
	      //@ SINON
	      else
		{
		  //@ Mettre a jour la forme interne
		  Heure().Seconde (Tm.tm_sec + FracSeconde); 
		  Heure().Minute (Tm.tm_min); 
		  Heure().Heure (Tm.tm_hour);
		  
		  Jour().Jour(Tm.tm_mday);
		  Jour().Mois(Tm.tm_mon+1);
		  Jour().Annee(Tm.tm_year+1900);
		}
	      //@ FIN SI le parsing a echoue
	    }
	  //@ FIN SI la recuperation a echoue
	}
      //@ FIN SI le parsing a echoue
    }
  //@ FIN SI

  //@ SI aucune erreur n'est apparue
  if (statut == CMN__STATUT_OK)
    {
      //@ Mettre a jour la validite de la date et le statut de retour
      statut = EstValide() ? CMN__STATUT_OK : CMN__STATUT_NOK;
    }
  //@ FIN SI

  //@ Restauration de l'ancien LcTime
  setlocale(LC_TIME, OldLcTime.c_str());


  return statut;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ChaineCCSDS
//@
//@ Description : Construit une date a partir d'une chaine de
//@               caracteres au format CCSDS ASCII : "day:ms:us" avec :
//@               'day' sur 5  chiffres = nb de jours julien 2000
//@               'ms'  sur 10 chiffres = nb de millisecondes dans le jour
//@               'us'  sur 5  chiffres = nb de microsecondes dans la millis
//@               L'origine des jours Julien 2000 est le 01/01/2000 a 0h00
//@
//@ Parametres :
//@               (ENTREE) chaine : chaine a convertir en date
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK : format de la chaine valide
//@               CMN__STATUT_NOK : format de la chaine invalide
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__Date::ChaineCCSDS(const t_Chaine& chaine)
{
  t_CMN__Statut statut = CMN__STATUT_OK;

  int days;   // nombre de jours Julien 2000
  int millis; // nombre de millisecondes dans le jour
  int micros; // nombre de microsecondes dans la milliseconde

  // extraction des 3 nombres dans la chaine
  days   = atoi(chaine.substr(OFFSET_DAYS,   LONG_DAYS).c_str());
  millis = atoi(chaine.substr(OFFSET_MILLIS, LONG_MILLIS).c_str());
  micros = atoi(chaine.substr(OFFSET_MICROS, LONG_MICROS).c_str());

  // creation de la date a partir d'un nombre de jours Julien 2000
  JourJulien2000((double)(days + millis / NB_MILLIS_JOUR + micros / NB_MICROS_JOUR));

  return statut;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ChaineUTC
//@
//@ Description : Construit une date a partir d'une chaine au format
//@               ASCII suivant en date UTC :
//@               JJ-MMM-AAAA HH:MN:SS.MLS
//@               avec JJ et AAAA des nombres, et MMM l'abreviation du
//@               mois sur 3 lettres en anglais,
//@               suivi de heure, minutes, secondes, millisecondes
//@
//@ Parametres :
//@               (ENTREE) chaine : chaine a convertir en date
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK : format de la chaine valide
//@               CMN__STATUT_NOK : format de la chaine invalide
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__Date::ChaineUTC(const t_Chaine& chaine)
{
  t_CMN__Statut statut = CMN__STATUT_OK;
  t_Chaine      chaine_mois;
  int           int_mois;
  int           iMois;

  // recherche du numero du mois a partir de l'abreviation sur 3 lettres
  chaine_mois = chaine.substr(COLONNE_MOIS, LONGUEUR_MOIS );
  int_mois    = 0;
  iMois       = 0;

  // on va comparer aux chaines definies dans le tableau TAB_MOIS_ANGLAIS
  while ( (iMois < NB_MOIS) && (int_mois == 0) )
  {
    // si les chaines correspondent, on a trouve le mois
    if ( chaine_mois == TAB_MOIS_ANGLAIS[iMois] )
    {
      int_mois = iMois + 1;
    }
    iMois++;
  }

  // cas ou le mois n'a pas ete reconnu
  if (int_mois == 0)
  {
    statut = CMN__STATUT_NOK;
  }
  else
  {
    // extraction de l'heure
    statut = m_Heure.Chaine(chaine.substr(COLONNE_HEURE, LONGUEUR_HEURE));

    if (statut == CMN__STATUT_OK)
    {
      // creation de l'objet jour (stockant jour, mois et annee)
      m_Jour.Jour (atoi(chaine.substr(COLONNE_JOUR,  LONGUEUR_JOUR ).c_str()));
      m_Jour.Annee(atoi(chaine.substr(COLONNE_ANNEE, LONGUEUR_ANNEE).c_str()));
      m_Jour.Mois (int_mois);
    }
  }

  return(statut);
}

//@ ------------------------------------------------------------------*/
//@ Methode : operator =
//@
//@ Description : Operation d'affectation d'une date
//@
//@
//@ Parametres :
//@               (ENTREE) uneDate : Date origine
//@
//@ Valeur de retour : 
//@               CMN__Date& : Date destination
//@
//@ ------------------------------------------------------------------*/
CMN__Date& CMN__Date::operator =(CMN__Date& uneDate)
{
  if (&uneDate == this)
    {
      return *this;
    }

  m_Jour  = uneDate.Jour();
  m_Heure = uneDate.Heure();

  return *this;
}

CMN__Date& CMN__Date::operator =(const CMN__Date& uneDate)
{
  if (&uneDate == this)
    {
      return *this;
    }
  
  t_Date dateTmp = uneDate;
  m_Jour  = dateTmp.Jour();
  m_Heure = dateTmp.Heure();

  return *this;
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator +
//@
//@ Description : Ajoute une duree
//@
//@ Parametres :
//@               (ENTREE) Duree : Duree a ajouter
//@
//@ Valeur de retour : 
//@               CMN__Date : Resultat de l'operation
//@
//@ ------------------------------------------------------------------*/
CMN__Date CMN__Date::operator +(double Duree)
{
  CMN__Date Date;

  Date = *this;
  Date.AjouterDuree(Duree);

  return Date;
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator -
//@
//@ Description : Soustrait une duree
//@
//@ Parametres :
//@               (ENTREE) Duree : Duree a soustraire
//@
//@ Valeur de retour : 
//@               CMN__Date : Resultat de l'operation
//@
//@ ------------------------------------------------------------------*/
CMN__Date CMN__Date::operator -(double Duree)
{
  CMN__Date Date;

  Date = *this;

  Date.AjouterDuree(-Duree);

  return Date;
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator +
//@
//@ Description : Ajoute une duree exprimee en heure
//@
//@ Parametres :
//@               (ENTREE) uneHeure : Heure a ajouter
//@
//@ Valeur de retour : 
//@               CMN__Date : Resultat de l'operation
//@
//@ ------------------------------------------------------------------*/
CMN__Date CMN__Date::operator +(CMN__Heure& uneHeure)
{
  return operator +(uneHeure.Duree());
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator -
//@
//@ Description : Soustrait une duree exprimee en heure
//@
//@ Parametres :
//@               (ENTREE) uneHeure : Heure a soustraire
//@
//@ Valeur de retour : 
//@               CMN__Date : Resultat de l'operation
//@
//@ ------------------------------------------------------------------*/
CMN__Date CMN__Date::operator -(CMN__Heure& uneHeure)
{
  return operator -(uneHeure.Duree());
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator -
//@
//@ Description : Soustrait une date a une autre date.
//@               Le resultat est une duree en seconde
//@
//@ Parametres :
//@               (ENTREE) uneHeure : Heure a soustraire
//@
//@ Valeur de retour : 
//@               double : Resultat de l'operation
//@
//@ ------------------------------------------------------------------*/
double CMN__Date::operator -(CMN__Date& uneDate)
{
  return (ConvertirDateEnTemps() - uneDate.ConvertirDateEnTemps());
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator ==
//@
//@ Description : Teste l'egalite de deux dates
//@
//@ Parametres :
//@               (ENTREE) uneDate : Date de comparaison
//@
//@ Valeur de retour : 
//@                bool : Resultat du test
//@
//@ ------------------------------------------------------------------*/
bool CMN__Date::operator ==(CMN__Date& uneDate)
{
  return (m_Jour==uneDate.Jour() && m_Heure==uneDate.Heure());
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator >=
//@
//@ Description : Teste si la date est superieure ou egale a la date
//@               passee en parametre
//@
//@ Parametres :
//@               (ENTREE) uneDate : Date de comparaison
//@
//@ Valeur de retour : 
//@               bool : Resultat du test
//@
//@ ------------------------------------------------------------------*/
bool CMN__Date::operator >=(CMN__Date& uneDate)
{
  return (m_Jour > uneDate.Jour()) ||
    (m_Jour == uneDate.Jour() && m_Heure >= uneDate.Heure());
}

bool CMN__Date::operator >=(const CMN__Date& uneDate)
{
  t_Date dateTmp = uneDate;
  return (m_Jour > dateTmp.Jour()) || 
    (m_Jour == dateTmp.Jour() && m_Heure >= dateTmp.Heure());
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator <=
//@
//@ Description : Teste si la date est inferieure ou egale a la date
//@               passee en parametre
//@
//@ Parametres :
//@               (ENTREE) uneDate : Date de comparaison
//@
//@ Valeur de retour : 
//@               bool : Resultat du test
//@
//@ ------------------------------------------------------------------*/
bool CMN__Date::operator <=(CMN__Date& uneDate)
{
  return (m_Jour < uneDate.Jour()) || 
    (m_Jour == uneDate.Jour() && m_Heure <= uneDate.Heure());
}

bool CMN__Date::operator <=(const CMN__Date& uneDate)
{
  t_Date dateTmp = uneDate;
  return (m_Jour < dateTmp.Jour()) || 
    (m_Jour == dateTmp.Jour() && m_Heure <= dateTmp.Heure());
}


//@ ------------------------------------------------------------------*/
//@ Methode : ConvertirDateEnTemps
//@
//@ Description : Convertit une date en temps
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
double CMN__Date::ConvertirDateEnTemps(  )
{
  return ( m_Jour.ConvertirJourEnTemps() + m_Heure.Duree());
}


//@ ------------------------------------------------------------------*/
//@ Methode : ConvertirTempsEnDate
//@
//@ Description : Convertit le temps en une date
//@
//@ Parametres :
//@               (ENTREE) Temps : Temps en secondes
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__Date::ConvertirTempsEnDate( double Temps )
{
  struct tm 	Date;
  struct tm *	pDate;
  time_t		time;
  double		FracSeconde = 0.0;

  // FT-2765
  time = static_cast<time_t>(Temps);
  // ...FT-2765
  FracSeconde = Temps - time; 
  pDate = localtime_r(&time, &Date);

  m_Heure.Seconde (Date.tm_sec + FracSeconde); 
  m_Heure.Minute (Date.tm_min); 
  m_Heure.Heure (Date.tm_hour);

  m_Jour.Jour(Date.tm_mday);
  m_Jour.Mois(Date.tm_mon+1);
  m_Jour.Annee(Date.tm_year+1900);
}


//@ ------------------------------------------------------------------*/
//@ Methode : TransformeDate
//@
//@ Description : Permet de transformer une date sous forme d'une
//@               chaine de caracteres en objet Date
//@
//@ Parametres :
//@               (ENTREE) dateIn : Chaine contenant la date
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK : Format de la date valide
//@               CMN__STATUT_NOK : Format de la date invalide
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__Date::TransformeDate(t_Chaine dateIn)
{
  int                 annee    = 0;
  int                 mois     = 0;
  int                 jour     = 0;
  int                 heure    = 0;
  int                 minute   = 0;
  int                 seconde  = 0;
  int                 milli    = 0;
  int                 fraction = 0;
  int                 nbToken  = 0;
  t_ChaineFixe        dateFormattee;

  // Valeur de retour de la methode
  t_CMN__Booleen      status = CMN__STATUT_NOK;


  //@ SI la date est renseignee
  if (!dateIn.empty())
    {
      //@ Tester si la date est une date UTC: recherche du
      //@ caractere Zulu
      int taille = dateIn.length();

      //@ SI la date est une date UTC
      if (dateIn.at(taille-1) == CAR_ZULU)
	{
	  //@ Tester si la date est une date UTC avec millisecondes

	  //@ SELON la taille de la date
	  switch (taille)
	    {
	      //@ CAS taille UTC simple:
	      case TAILLE_UTC_TIME:
		{
		  nbToken = sscanf(dateIn.c_str(), FORMAT_UTC_TIME, 
				   &annee, &mois, &jour, 
				   &heure, &minute, &seconde);

		  if (nbToken == NB_TOKEN_UTC_TIME)
		    {
		      sprintf(dateFormattee, FORMAT_CHAINE_OUT, 
			      annee, mois, jour, 
			      heure, minute, seconde, fraction);
		      
		      status = this->Chaine(dateFormattee, FORMAT_DATE);
		    }	  
		  break;
		}
	      
	      //@ CAS taille UTC long:
	      case TAILLE_LONG_UTC_TIME:
		{
		  nbToken = sscanf(dateIn.c_str(), 
				   FORMAT_LONG_UTC_TIME, 
				   &annee, &mois, &jour, 
				   &heure, &minute, &seconde, &milli);
		  
		  if (nbToken == NB_TOKEN_LONG_UTC_TIME)
		    {
		      fraction = milli * 1000;
		      sprintf(dateFormattee, FORMAT_CHAINE_OUT, 
			      annee, mois, jour, 
			      heure, minute, seconde, fraction);
		      
		      status = this->Chaine(dateFormattee, FORMAT_DATE);
		    }	  
		  break;
		}
	      
	      //@ CAS taille UTC long avec separateur: 
	      case TAILLE_LONG_SEP_UTC_TIME:
		{
		  nbToken = sscanf(dateIn.c_str(), 
				   FORMAT_LONG_SEP_UTC_TIME, 
				   &annee, &mois, &jour, 
				   &heure, &minute, &seconde, &milli);
		  
		  if (nbToken == NB_TOKEN_LONG_SEP_UTC_TIME)
		    {
		      fraction = milli * 1000;
		      sprintf(dateFormattee, FORMAT_CHAINE_OUT, 
			      annee, mois, jour, 
			      heure, minute, seconde, fraction);
		      
		      status = this->Chaine(dateFormattee, FORMAT_DATE);
		    }	  
		  break;
		}
	      
	    //@ Defaut:
	    default :
	      {
		//@ Par defaut, aucun traitement
		break;
	      }
	    }
	  //@ FIN SELON
	}
      //@ FIN SI la date est une date UTC
    }
  //@ FIN SI la date est renseignee
	  
  return status;
}



//@ ------------------------------------------------------------------*/
//@ Methode : DateUTC
//@
//@ Description : Renvoie la date sous forme d'une date UTC ASCII
//@
//@ Parametres :
//@               (ENTREE) typeLong : Si vrai, renvoie une date UTC
//@                                   longue 
//@               (ENTREE) sep : Si vrai, renvoie une date UTC longue
//@                              avec un separateur pour les secondes 
//@
//@ Valeur de retour :
//@               Date UTC ASCII
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN__Date::DateUTC(t_CMN__Booleen typeLong, t_CMN__Booleen sep)
{
  t_ChaineFixe s;
  int          seconde = 0;
  int          milli = 0;


  seconde = int(m_Heure.Seconde());

  //@ SI une date UTC de type long est demandee
  if (typeLong == CMN__VRAI)
    {
      milli = m_Heure.MilliSeconde();

      //@ SI une date UTC de type long avec un separateur est demandee
      if (sep == CMN__VRAI)
	{
	  //@ Retourner une date UTC de type long avec un separateur
	  snprintf(s, CMN__LG_CHAINE_FIXE, FORMAT_LONG_SEP_UTC_TIME,
		   m_Jour.Annee(), m_Jour.Mois(), m_Jour.Jour(),
		   m_Heure.Heure(), m_Heure.Minute(), seconde, milli);
	}
      //@ SINON
      else
	{
	  //@ Retourner une date UTC de type long sans separateur
	  snprintf(s, CMN__LG_CHAINE_FIXE, FORMAT_LONG_UTC_TIME,
		   m_Jour.Annee(), m_Jour.Mois(), m_Jour.Jour(),
		   m_Heure.Heure(), m_Heure.Minute(), seconde, milli);
	}
      //@ FIN SI une date UTC de type long avec un
      //@ separateur est demandee
    }
  //@ SINON
  else
    {
      //@ Retourner une date UTC courte
      snprintf(s, CMN__LG_CHAINE_FIXE, FORMAT_UTC_TIME,
	       m_Jour.Annee(), m_Jour.Mois(), m_Jour.Jour(),
	       m_Heure.Heure(), m_Heure.Minute(), seconde);
    }
  //@ FIN SI une date UTC de type long est demandee

  return s;
}


//======================================================================
//					METHODES PRIVEES de CMN__Date
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : AjouterDuree
//@
//@ Description : Ajoute une duree
//@
//@ Parametres :
//@               (ENTREE) Duree : Duree a ajouter
//@
//@ Valeur de retour : 
//@               Resultat de l'operation
//@
//@ ------------------------------------------------------------------*/
void CMN__Date::AjouterDuree(double Duree)
{
  ConvertirTempsEnDate(ConvertirDateEnTemps() + Duree);
}
