// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/CMN__Date.h,v 1.3 2011/11/25 10:17:39 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : CMN
//@ CLASSE : CMN__Date
//@
//@ Commentaire :
//@    Ce fichier gere la classe CMN__Date
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 07/10/1999 : BBS
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Adaptation a l'OPS : ajout des fonctions DateInit, TransformeDate
//@                      et DateUTC
//@ --------------------------------------------------------------------
//@ VERSION : 12/12/2002 : O. DUFFEZ
//@ Ajout des methodes de gestion des jours julien 2000
//@ et de conversion a partir de dates ASCII long CCSDS
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-180-CN : 30/08/2006 : C.MERIGUET
//@ Mise a jour de l'entete MPHR en mode DUMP: definition
//@ macro pour difference de date
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef CMN__Date_h
#define CMN__Date_h

// Divers
#define NB_SECONDES_PAR_JOUR   (60*60*24)
#define NB_SECONDES_PAR_HEURE  (60*60)
#define NB_SECONDES_PAR_MINUTE (60)
#define NB_JOURS_PAR_SEMAINE   (7)

// creation d'une date a partir d'une chaine de caracteres au format long CCSDS
// FT-2765
static const int    OFFSET_DAYS        = 0;
static const int    OFFSET_MILLIS      = 6;
static const int    OFFSET_MICROS      = 15;
static const int    LONG_DAYS          = 5;
static const int    LONG_MILLIS        = 10;
static const int    LONG_MICROS        = 5;
static const double NB_MILLIS_SEC      = 1.e3;    /* nb de millis dans une seconde */
static const double NB_MILLIS_JOUR     = 8.64e7;  /* nb de millis dans un jour */
static const double NB_MICROS_JOUR     = 8.64e10; /* nb de micros dans un jour */
// ...FT-2765

/*-------------------------------------------------*/
/* Definition des macros qui operent sur des dates */
/*-------------------------------------------------*/
#define CMN__DATE_SEC_PAR_JOUR (86400)

#define CMN__DATE_CONV_SEC_2_MSEC(Date) ((Date) * 1000)

#define CMN__DATE_CONV_MSEC_2_SEC(Date) ((Date) / 1000.0)

#define CMN__DATE_CONV_SCDS_2_SEC(Date) \
        ((Date).day * CMN__DATE_SEC_PAR_JOUR + \
        CMN__DATE_CONV_MSEC_2_SEC((Date).millis))

#define CMN__DATE_DIFF_SCDS_2_SEC(Date1, Date2) \
        (fabs(CMN__DATE_CONV_SCDS_2_SEC(Date2) - CMN__DATE_CONV_SCDS_2_SEC(Date2)))

/* FA 180*/
#define CMN__DATE_DIFF_SCDS_2_SEC_CME(Date1, Date2) \
        (fabs(CMN__DATE_CONV_SCDS_2_SEC(Date2) - CMN__DATE_CONV_SCDS_2_SEC(Date1)))
/* ..FA 180*/

#define CMN__DATE_ANTERIEURE(Date1, Date2) \
        ((CMN__DATE_CONV_SCDS_2_SEC(Date1) < CMN__DATE_CONV_SCDS_2_SEC(Date2)) ? \
        CMN__VRAI : CMN__FAUX)

//--------------------------------------------------------------------*/
/* Dates au format IASI */
//--------------------------------------------------------------------*/
typedef struct
    {
    unsigned short int day; // day since 1 january 2000
    unsigned int millis; // number of ms in the day
    } t_SCDSTime;

typedef struct
    {
    unsigned short int day; // day since 1 january 2000
    unsigned int millis; // number of ms in the day
    unsigned short int micros; // number of micros in the day
    } t_LCDSTime;


#ifdef __cplusplus

#include <vector>

// FT-2765
static const char* FORMAT_DATE                 = "%Y %m %d %H %M %S.%s";
// ...FT-2765

//====================================================================*/
// classe des Jour
//====================================================================*/
class CMN__Jour {

    /*================================================================*/

 public:
    
    friend class CMN__Date;
    
    //
    // Constructeur
    //
    CMN__Jour();
    CMN__Jour(t_Chaine);
    CMN__Jour(t_Chaine sJour, 
              t_Chaine sFormat,
              t_Chaine sLangue=(char *)"C");

    //
    // Destructeur
    //
    ~CMN__Jour();
    
        
    //
    // Accesseurs
    //
    void Jour(int Jour) {m_Jour = Jour; TesterValidite(); }
    void Mois(int Mois) {m_Mois = Mois;  TesterValidite();}
    void Annee(int Annee) {m_Annee = Annee; TesterValidite();}
    
    int Jour() {return m_Jour;}
    int JourSemaine() {return m_JourSemaine;}
    int Mois() {return m_Mois;}
    int Annee() {return m_Annee;}
    bool EstValide() {return m_Valide;}


    //
    // Methodes publiques
    //

    // Retourne le nom du mois de la date
    t_Chaine NomMois();

    // Retourne la date sous forme de chaine
    t_Chaine Chaine();

    // Affecte la valeur de la date a partir d'une chaine
    t_CMN__Statut Chaine(t_Chaine sDate);
    t_CMN__Statut Chaine(t_Chaine sJour, 
                         t_Chaine sFormat,
                         t_Chaine sLangue = (char *)"C");
    
    t_Chaine Formater(const char* Format, 
                      const t_Chaine sLangue = (char *)"C");
    
    CMN__Jour& operator =(CMN__Jour& unJour);
    
    bool operator ==(CMN__Jour& unJour);
    bool operator !=(CMN__Jour& unJour) 
        {return ! (operator ==(unJour));};
    bool operator >=(CMN__Jour& unJour);
    bool operator <=(CMN__Jour& unJour);
    bool operator >(CMN__Jour& unJour) {return ! (operator <=(unJour));}
    bool operator <(CMN__Jour& unJour) {return ! (operator >=(unJour));}


    /*================================================================*/

 protected:

    //
    // Methodes protegees
    //

    //
    // Attributs proteges
    //
    

    /*================================================================*/

 private:
    
    //
    // Methodes privees
    //
    void TesterValidite();
    void ConvertirTempsEnJour(double Temps);
    double ConvertirJourEnTemps();
    t_CMN__Statut NormaliserDate();
        
    //
    // Attributs prives
    //
    int         m_Jour;
    int         m_JourSemaine;
    int         m_Mois;
    int         m_Annee;
    bool        m_Valide;
};


//====================================================================*/
// classe des heures
//====================================================================*/
class CMN__Heure {

    /*================================================================*/
    
 public:

    //
    // Constructeur
    //
    CMN__Heure();
    CMN__Heure(t_Chaine sHeure);
    CMN__Heure(t_Chaine sHeure, t_Chaine sFormat);

    //
    // Destructeur
    //
    virtual ~CMN__Heure();
    
    //
    // Accesseurs
    //
    void Heure(int Heure) {m_Heure = Heure; TesterValidite();}
    void Minute(int Minute) {m_Minute = Minute; TesterValidite();}
    void Seconde(double Seconde) 
        {m_Seconde = Seconde; TesterValidite();}
    void Duree (double Duree) 
        {m_Duree = Duree; ConvertirDureeEnHeure();}
    
    int Heure() {return m_Heure;}
    int Minute() {return m_Minute;}
    double Seconde() {return m_Seconde;}
    int MilliSeconde();
    double Duree() {ConvertirHeureEnDuree( );return m_Duree;}
    bool EstValide() {return m_Valide;}
    
    // Valeur de l'heure sous forme de chaine
    t_Chaine Chaine();
    
    t_CMN__Statut Chaine(t_Chaine Chaine);
    t_CMN__Statut Chaine(t_Chaine Chaine, t_Chaine Format);
    t_Chaine Formater(const char* Format, const int NbDigFracSec);

    CMN__Heure& operator =(CMN__Heure& uneHeure);
    CMN__Heure operator +(CMN__Heure& uneHeure);
    CMN__Heure operator -(CMN__Heure& uneHeure);
    bool operator ==(CMN__Heure& uneHeure);
    bool operator !=(CMN__Heure& uneHeure) 
        {return ! (operator ==(uneHeure));};
    bool operator >=(CMN__Heure& uneHeure);
    bool operator <=(CMN__Heure& uneHeure);
    bool operator >(CMN__Heure& uneHeure) 
        {return ! (operator <=(uneHeure));}
    bool operator <(CMN__Heure& uneHeure) 
        {return ! (operator >=(uneHeure));}


    /*================================================================*/

 protected:

    //
    // Methodes protegees
    //

    //
    // Attributs proteges
    //


    /*================================================================*/

 private:

    //
    // Methodes privees
    //
    void TesterValidite();
    void ConvertirHeureEnDuree();
    void ConvertirDureeEnHeure();

    //
    // Attributs prives
    //
    int    m_Heure;
    int    m_Minute;
    double m_Seconde;
    double m_Duree;
    bool   m_Valide;
};


//====================================================================*/
// classe des dates
//====================================================================*/
class CMN__Date {

    /*================================================================*/

 public:
    
    // On autorise la classe a acceder aux donnees privees des objets
    // de la classe
    //friend class CMN__Date;

    //
    // Constructeur
    //
    CMN__Date();
    CMN__Date(const t_Chaine Chaine);
    CMN__Date(const t_Chaine sDate,
              t_Chaine sFormat,
              t_Chaine sLangue=(char *)"C");
    CMN__Date(const CMN__Date &);       // Constructeur de recopie
    
    //
    // Destructeur
    //
    virtual ~CMN__Date();
    
    //
    // Accesseurs
    //
    void Heure(CMN__Heure Heure) {m_Heure = Heure;}
    void Jour(CMN__Jour Jour) {m_Jour = Jour;}
    
    CMN__Heure& Heure() {return m_Heure;}
    CMN__Jour& Jour() {return m_Jour;}
    bool EstValide() 
        {return (Heure().EstValide() && Jour().EstValide());}
    

    //
    // Methodes publiques
    //

    // Recupere la date courante
    void DateCourante();          // Precision a la sec
    void DateCourantePrecise();   // Precision a la microsec
    void DateInit();         // Initialise la date au 1er janvier 2000 00h00
    double JourJulien() ;    // Retourne la date en nb de Jour Julien
    double JourJulienCnes(); // Retourne la date en nb de Jour Julien CNES
    double JourJulien2000(); // Retourne la date en nb de Jour Julien 2000
    t_SCDSTime SCDSTime(); // Retourne la date en t_SCDSTime
    t_LCDSTime LCDSTime(); // Retourne la date en t_LCDSTime

    
    // Cree une date en fonction d'un nombre de Jours Julien standard
    void JourJulien(const double JourJulien);
    
    // Cree une date en fonction d'un nombre de Jours Julien CNES
    void JourJulienCnes(const double JJCNES);
    
    // Cree une date en fonction d'un nombre de Jours Julien 2000
    void JourJulien2000(const double JJ2000);

    // Cree une date en fonction d'une date au format t_SCDSTime
    void SCDSTime(const t_SCDSTime dateSCDSTime);

    // Cree une date en fonction d'une date au format t_LCDSTime
    void LCDSTime(const t_LCDSTime dateLCDSTime);

    // Retourne la date sous forme de chaine
    t_Chaine Formater(const char* Format,
                      const int NbDigitFracSeconde = 6, 
                      const t_Chaine sLangue = (char *)"C");
    t_Chaine Chaine();

    // Affecte la valeur de la date a partir d'une chaine
    t_CMN__Statut Chaine(t_Chaine Chaine);
    t_CMN__Statut Chaine(t_Chaine Chaine, 
                         t_Chaine Format, 
                         t_Chaine Langue=(char *)"C");

    // Affecte la valeur de la date a partir d'une chaine au format long CCSDS
    t_CMN__Statut ChaineCCSDS(const t_Chaine& Chaine);
    
    // Affecte la valeur de la date a partir d'une chaine au format UTC ASCII :
    // "JJ-MMM-AAAA HH:MN:SS.MLS" ou MMM est l'abreviation du mois sur 3 lettres
    t_CMN__Statut ChaineUTC(const t_Chaine& chaine);

    // Retourne la date courante sous le format timeval
    // FT-2765
    void LaDateCourantePrecise(int precision, struct timeval& laDate);
    // ...FT-2765
    
    // Surcharge de l'operateur d'affectation
    CMN__Date& operator =(CMN__Date& uneDate);
    CMN__Date& operator =(const CMN__Date& uneDate);

    // Ajouter ou retrancher une duree a la date en cours
    double ConvertirDateEnTemps(  );
    void ConvertirTempsEnDate(double Temps);
    
    CMN__Date operator +(double Duree);
    CMN__Date operator -(double Duree);
    CMN__Date operator +(CMN__Heure& uneHeure);
    CMN__Date operator -(CMN__Heure& uneHeure);
    double operator -(CMN__Date& uneDate);
    
    bool operator ==(CMN__Date& uneDate);
    bool operator !=(CMN__Date& uneDate) 
        {return ! (operator ==(uneDate));};
    bool operator >=(CMN__Date& uneDate);
    bool operator >=(const CMN__Date& uneDate);
    bool operator <=(CMN__Date& uneDate);
    bool operator <=(const CMN__Date& uneDate);
    bool operator >(CMN__Date& uneDate) 
        {return ! (operator <=(uneDate));}
    bool operator >(const CMN__Date& uneDate) 
        {return ! (operator <=(uneDate));}
    bool operator <(CMN__Date& uneDate) 
        {return ! (operator >=(uneDate));}
    bool operator <(const CMN__Date& uneDate) 
        {return ! (operator >=(uneDate));}

    t_CMN__Statut TransformeDate(t_Chaine dateIn);
    t_Chaine      DateUTC(t_CMN__Booleen typeLong=CMN__FAUX, 
                          t_CMN__Booleen sep=CMN__FAUX);


    /*=================================================================*/

 protected:

    //
    // Methodes protegees
    //

    //
    // Attributs proteges
    //
    

    /*=================================================================*/

 private:

    //
    // Methodes privees
    //
    void AjouterDuree(double Duree);
        
    //
    // Attributs prives
    //
    CMN__Jour   m_Jour;
    CMN__Heure  m_Heure;
};


//
// Type
//
typedef CMN__Date         t_Date;
typedef vector<CMN__Date> t_TableauDate;
typedef double            t_DateNum;

#endif /* bloc uniquement C++ */

typedef struct 
{
    short dayAfterEPOCH;
    long  milliInDay;
} t_CDS_ShortTime;

typedef struct 
{
    short dayAfterEPOCH;
    long  milliInDay;
    short microInMilli;
} t_CDS_LongTime;

#endif
