static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/CMN__Fichier.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : CMN
//@ MODULE : CMN__Fichier
//@ 
//@ Commentaire :
//@    Ce module propose un ensemble de services concernant les
//@    fichiers :
//@     - TrouverFichier: retourne la liste des fichiers correspondant
//@                       a une expression reguliere
//@     - TrouverFichierEx: retourne la liste des infos sur les
//@                         fichiers correspondant a une expression 
//@                         reguliere. Elle permet de specifier une 
//@                         fonction de selection ainsi qu'une 
//@                         fonction de tri
//@     - CopierFichier: copie un fichier dans un autre fichier	
//@ 		
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 10/03/2000 : Serge Bobin
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 29/09/2000 : C.LACAZE
//@   Modification de la methode CMN__TrouverFichiersEx pour qu'elle ne
//@   retourne que les fichiers reguliers
//@ FA-ID : FPA-F11-FA-628-CN : 29/09/2000
//@   Modification de la methode CMN__TrouverFichiersEx pour qu'elle ne
//@   retourne que les fichiers reguliers
//@ --------------------------------------------------------------------
//@ VERSION : 14/12/2000 : C.LACAZE
//@  Portage sous systeme AIX 4.3
//@ --------------------------------------------------------------------
//@ VERSION : 15/01/2001 : S.BOBIN
//@  Correction anomalie 2387
//@ --------------------------------------------------------------------
//@ FA-ID : FPA-F11-FA-2387-CN : 12/10/2001
//@   Modification de la methode CMN__PlusRecentQue pour qu'elle
//@   retourne les fichiers > date et non pas les fichiers >= date
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Adaptation a l'OPS : ajout de la fonction CMN__RechercherFichier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-144-CN : 21/09/2004 : B. MASSART
//@ Problemes lors du portage d'OPS vers Solaris par Meteo France
//@ Voir IA-2100-FC-144-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-184-CN : 18/10/2006 : C.MERIGUET
//@ Codage des fonctions reentrantes
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-3316(DM-2100-OPS) : 18/05/2017 : MPE
//@ PORTAGE OPS SOUS RHEL 6.5 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//
extern "C" {
#include <sys/types.h>
// FT-2765
#ifdef AIX
#include <sys/statfs.h>
#include <sys/vnode.h>
#else // LINUX
#include <sys/statvfs.h>
#endif
// ...FT-2765
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fnmatch.h>
#include <regex.h>
}
#include <algorithm>
#include <functional>
#include <vector>
#include <list>

#include <cstdio>

// Migration RHEL 6.5
#ifdef LINUX
#include <cstring>
#endif

#include "CMN__Commun.h"
#include "CMN__Fichier.h"
#include "CMN__Date.h"

#include <list>


//======================================================================
// Declaration des fonctions internes
//======================================================================

//======================================================================
// Declaration des variables globales
//======================================================================


//======================================================================
//					FONCTIONS EXTERNES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__RenommerFichier
//@
//@ Description : Deplace un fichier d'un repertoire a un autre
//@
//@ Parametres :
//@               (ENTREE) source : Fichier a deplacer
//@               (ENTREE) dest : Fichier destination
//@
//@ Valeur de retour : 
//@               0 si le deplacement a ete effectue
//@               -1 si erreur
//@
//@ ------------------------------------------------------------------*/
int CMN__RenommerFichier(const t_Chaine& source, const t_Chaine& dest)
{
  int res = 0;
  char *ficTmp = NULL;
  char *ficTemp = NULL;

  //@ Determination d'un fichier temporaire
  // Migration RHEL 6.5 : copier du const pour le cast
  char *dest_aux = new char[dest.length()];
  strncpy(dest_aux, dest.c_str(), dest.length());

  // FT-2765
  char *slash = strrchr((dest_aux), '/');
  // ...FT-2765
  if (slash == NULL ) ficTemp = strdup ("./.TMPFXXXXXX");
  else {
    char car;
   
    car  = *(slash + 1);
    *(slash + 1) = '\0';
    t_Chaine templ = t_Chaine(dest_aux) + t_Chaine(".TMPFXXXXXX");
    ficTemp = strdup (templ.c_str());
    *(slash + 1) = car;
  }
  if( ficTemp != NULL ) ficTmp = mktemp (ficTemp);
  else res = -1;

  //@ Copie du fichier dans le fichier temporaire
  if( ficTmp != NULL ){
    if( CMN__CopierFichier (source,t_Chaine(ficTmp),false) == CMN__STATUT_OK ){
  
      //@ Renommage du fichier temporaire avec le nom du fichier cible
      rename (ficTmp,dest_aux);
    }else{
      unlink (ficTmp);
      res = -1;
    }
  }else res = -1;
  if( ficTemp != NULL ) free(ficTemp);

  //@ Suppression du fichier source
  if( res == 0 ){
    unlink(source.c_str());
  }

  return res;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__TrouverFichiers
//@
//@ Description : Rend la liste des fichiers repondant a un pattern
//@               expression reguliere en local avec la date des
//@               fichiers
//@
//@ Parametres :
//@               (ENTREE) path: Path des fichier en local
//@               (ENTREE) pattern : Expression reguliere decrivant
//@                                  le fichier
//@               (SORTIE) liste_fichiers : Liste des fichiers
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK si la fonction a reussi
//@               CMN__STATUT_NOK si la fonction a echoue
//@               CMN__ERR_PERMISSION_REFUSEE : Probleme d'acces au
//@                                             repertoire
//@               CMN__ERR_PLUS_DE_MEMOIRE : Plus de memoire
//@               CMN__ERR_MAUVAIS_FORMAT si le pattern est incorrect
//@               CMN__ERR_FICHIER_INTROUVABLE : Pas de commande
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__TrouverFichiers(const t_Chaine & path,
				   const t_Chaine & pattern,
				   list<t_Chaine> & ListeFichiers,
				   int		NbMaxTentatives,
				   int		Interval)
{
  t_CMN__Statut cr=CMN__STATUT_OK;
  CMN__ListeInfoFichier ListeInfoFichier;

  //@ Vider la liste de retour
  ListeFichiers.clear();

  //@ Rechercher les fichiers qui match le pattern 
  cr=CMN__TrouverFichiersEx(path,pattern,ListeInfoFichier,NULL,
			    CMN__OrdreAlphaFichier,NULL,NbMaxTentatives,Interval);

  //@ Recopier les noms des fichiers dans la liste de retour
  CMN__ListeInfoFichier::iterator pInfo;
  for ( pInfo = ListeInfoFichier.begin(); pInfo !=  ListeInfoFichier.end();pInfo++)
    {
      ListeFichiers.insert(ListeFichiers.end(),pInfo->Name());
    }

  //@ Retourner le statut
  return cr;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__TrouverFichiersEx
//@
//@ Description : Rend la liste des fichiers qui match un pattern. 
//@               Si une fonction de selection est passee en argument 
//@               alors une 2eme selection est pratiquee selon ce qui
//@               est defini par cette fonction.
//@               Si une fonction de tri est passee en argument alors
//@               la liste est triee selon cette fonction.
//@               La fonction de selection est appelee (Si elle existe)
//@               pour chaque fichier qui match le pattern avec en 
//@               argument :
//@               	. un parametre libre qui est celui passee en 
//@               	  argument de CMN__TrouverFichiersEx,
//@               	. le nom du fichier
//@               	. la structure stat associe a ce fichier
//@               Le fichier pour lequel cette fonction est appelee,
//@               sera inclu dans la liste des fichiers si la fonction
//@               de selection retourne true.
//@               La fonction de tri est appelee pour comparer 2 
//@               fichiers de la liste.
//@               Elle doit retourner :
//@               	< 0 si fic1 < fic2
//@               	= 0 si fic1 == fic2
//@               	> 0 si fic1 > fic2	
//@               		
//@ Parametres :
//@               (ENTREE) path: Path des fichier en local
//@               (ENTREE) pattern : Expression reguliere decrivant
//@                                  le fichier
//@               (SORTIE) liste_fichiers : Liste des fichiers
//@               (ENTREE) fnSelect : Fonction de selection
//@               (ENTREE) fnTri : Fonction de comparaison utilisee
//@                                pour le tri
//@               (ENTREE) param : Parametre libre pour l'utilisateur
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK si la fonction a reussi
//@               CMN__STATUT_NOK si la fonction a echoue
//@               CMN__ERR_PERMISSION_REFUSEE: Probleme d'acces au
//@                                            repertoire
//@               CMN__ERR_PLUS_DE_MEMOIRE : Plus de memoire
//@               CMN__ERR_MAUVAIS_FORMAT si le pattern est incorrect
//@               CMN__ERR_FICHIER_INTROUVABLE : Pas de commande
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__TrouverFichiersEx(const t_Chaine &	path,
				     const t_Chaine &	pattern,
				     CMN__ListeInfoFichier & ListeInfoFichier,
				     t_fnSelect		fnSelect, 
				     t_fnTri		fnTri,
				     void *		param, 
				     int		NbMaxTentatives,
				     int		Interval)
{
  t_CMN__Statut	cr=CMN__STATUT_OK;
  CMN__InfoFichier InfoFichier;


  //@ Vider la liste des infos
  ListeInfoFichier.clear();

  int			 ind=0;
  DIR			 *repertoire=NULL; // repertoire
  struct dirent *pDirEnt=NULL; // fichier
  regex_t		 regexbuf; // expression reguliere
  regmatch_t	 regmatchbuf; // expression reguliere
  CMN__Stat  strStat;
/* FA 184 */
  struct dirent pResult; // fichier
  int    status;
/* ..FA 184 */
  //@ Compilation de l'expression reguliere
  if ( regcomp(&regexbuf,pattern.c_str(),REG_EXTENDED) != 0)
    {
      return CMN__ERR_MAUVAIS_FORMAT;
    }

  //@ TANT QUE on a rien trouve et que le nombre max de tentatives
  //@ n'est pas depasse
  for ( int NbTentatives=0 ; 
	(NbTentatives < NbMaxTentatives) && (ListeInfoFichier.size() == 0) ;
	NbTentatives++)
    {
      //@ pour tous les fichiers du repertoire
      if ( (repertoire=opendir(path.c_str())) != NULL ) 
   	{
/* FA 184 */
// FT-2765
#ifdef AIX
 	  while(  (status=readdir_r(repertoire,&pResult,&pDirEnt)) ==0 )
#else // LINUX
      while( (pDirEnt = readdir(repertoire)) != NULL)
#endif
// ...FT-2765
	    {
/* ..FA 184 */
	      t_Chaine FileName = pDirEnt->d_name;
	      t_Chaine PathName = path + CMN__SEP_REPERTOIRE + FileName;
	      //@ comparer le nom au pattern (expression reguliere)
	      if ( regexec(&regexbuf,FileName.c_str(), 1, &regmatchbuf, 0) == 0 )
		{
		  stat(PathName.c_str(),&strStat);
    
    /* FA144 : norme POSIX */
		  if( ! S_ISDIR( strStat.st_mode ) ){

		    //@ SI le fichier correspond au critere de selection
		    if ( (fnSelect == NULL) ||  
			 fnSelect(param,FileName,strStat) )
		      {
			InfoFichier.Stat(strStat);
			InfoFichier.Name(FileName);

			//@ Inserer le fichier dans la liste des
			//@ fichiers selectionnes
			ListeInfoFichier.insert(ListeInfoFichier.end(),
						InfoFichier);
		      }
		    //@ FIN SI
		  }
		}
	    }

	  //@ Fermer le repertoire
	  closedir(repertoire);
   	}
      else
   	{
	  cr=CMN__ERR_PERMISSION_REFUSEE;
   	}

      //@ SI une prochaine tentative est autorisee et si on 
      //@ a rien trouve
      if ( (NbTentatives+1 < NbMaxTentatives) && (ListeInfoFichier.size() == 0) )
	{
	  //@ Attendre
	  sleep(Interval);
	}
      //@ FIN SI
    } 
  //@ FIN TANT QUE boucle sur le nombre de tentatives

  //@ Liberer l'expression reguliere compilee
  regfree(&regexbuf);

  //@ SI il faut trier
  if (fnTri != NULL )
    {
      //@ Trier
      sort(ListeInfoFichier.begin(),ListeInfoFichier.end(),fnTri);
    }
  //@ FIN SI
  
  return cr;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Copier
//@
//@ Description : Copie fichierSrc dans fichierDest
//@
//@ Parametres :
//@               (ENTREE) path : Path des fichier en local
//@               (ENTREE) pattern : Expression reguliere decrivant
//@                                  le fichier
//@               (SORTIE) liste_fichiers : Liste des fichiers
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK si la fonction a reussi
//@               CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__CopierFichier(const t_Chaine & FichierSrc,
				 const t_Chaine & FichierDest,
				 bool bFailIfExist)
{
  t_CMN__Statut cr = CMN__STATUT_OK;
  FILE * pfSource;		// Strean Source
  FILE * pfDest;		// Stream destination
  char Buffer[2048];	// Buffer de lecture
  int  n;				// Nombre de caracteres lus

  //@ SI le fichier Destination existe et si une erreur doit etre 
  //@ retournee en cas d'existence
  if ( (access ( FichierDest.c_str(), 0 ) != -1) && bFailIfExist)
    {
      //@ Retourner une erreur
      return CMN__STATUT_NOK;
    }
  //@ FIN SI

  //@ Ouverture en lecture du fichier source et en ecriture du
  //@ fichier destination
  pfSource = fopen ( FichierSrc.c_str(), "rb");
  pfDest = fopen ( FichierDest.c_str(), "wb");

  //@ SI ouvertures reussies
  if ( (pfSource != NULL) && (pfDest != NULL) )
    {
      //@ Copier le fichier source dans le fichier Dest par bloc
      //@ de 2048 caracteres
      while ( ! feof( pfSource) )
	{
	  //@ Lecture d'un buffer dans le fichier source
	  n=fread( Buffer, sizeof(char), sizeof(Buffer), pfSource) ;
	  //@ Ecriture du buffer dans le fichier dest
	  fwrite( Buffer, sizeof(char), n, pfDest );
	}

      //@ Fermeture des 2 fichiers
      fclose(pfSource);
      fclose(pfDest);
    }
  //@ SINON erreur d'ouverture d'au moins un des fichiers
  else 
    {
      cr = CMN__STATUT_NOK;
    }
  //@ FIN SI
  
  return cr;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__OrdreAlphaFichier
//@
//@ Description : Compare 2 objets CMN__InfoFichier selon un ordre 
//@               alphabetique sur les noms de fichier
//@
//@ Parametres :
//@               (ENTREE) CMN__InfoFichier info1
//@               (ENTREE) CMN__InfoFichier info2
//@
//@ Valeur de retour : 
//@               Retourne 0 si info1 == info2
//@               Retourne <0 si info1 < info2
//@               Retourne >0 si info1 > info2
//@
//@ ------------------------------------------------------------------*/
int CMN__OrdreAlphaFichier(const CMN__InfoFichier & info1, 
			   const CMN__InfoFichier & info2)
{
  return ( info1.Name().compare(info2.Name()) );
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__PlusVieuxQue
//@
//@ Description : Compare la date de creation d'un fichier a une date
//@               donnee
//@
//@ Parametres :
//@               (ENTREE) void * ParamDate
//@               (ENTREE) t_chaine NomDuFichier
//@               (ENTREE) CMN__Stat Stat
//@
//@ Valeur de retour : 
//@               Retourne true si le fichier a ete cree avant la
//@               date passee en parametre
//@
//@ ------------------------------------------------------------------*/
bool CMN__PlusVieuxQue(void * ParamDate, 
		       const t_Chaine & Fichier,
		       const CMN__Stat & Stat)
{
  CMN__Date * pDateRef = (CMN__Date *)ParamDate;
  double TempsRef = pDateRef->ConvertirDateEnTemps();

  return ( Stat.st_mtime < TempsRef );
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__PlusRecentQue
//@
//@ Description : Compare la date de creation d'un fichier a une date
//@               donnee
//@
//@ Parametres :
//@               (ENTREE) void * ParamDate
//@               (ENTREE) t_chaine NomDuFichier
//@               (ENTREE) CMN__Stat Stat
//@
//@ Valeur de retour : 
//@               Retourne true si le fichier a ete cree apres la
//@               date passee en parametre
//@
//@ ------------------------------------------------------------------*/
bool CMN__PlusRecentQue(void * ParamDate, 
			const t_Chaine & Fichier,
			const CMN__Stat & Stat)
{
  //	return ( !CMN__PlusVieuxQue(ParamDate, Fichier, Stat));
  CMN__Date * pDateRef = (CMN__Date *)ParamDate;
  double TempsRef = pDateRef->ConvertirDateEnTemps();

  return ( Stat.st_mtime > TempsRef );
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__TriDateCroissante
//@
//@ Description : Compare 2 objets CMN__InfoFichier selon les dates
//@               de creation des fichiers
//@
//@ Parametres :
//@               (ENTREE) CMN__InfoFichier info1
//@               (ENTREE) CMN__InfoFichier info2
//@
//@ Valeur de retour : 
//@               Retourne 0 si info1 == info2
//@               Retourne <0 si info1 < info2
//@               Retourne >0 si info1 > info2
//@
//@ ------------------------------------------------------------------*/
int CMN__TriDateCroissante( const CMN__InfoFichier & info1,
			    const CMN__InfoFichier & info2)
{
  return (info1.Stat().st_mtime < info2.Stat().st_mtime);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__CreerArborescence
//@
//@ Description : Cree si necessaire les repertoires utiles a 
//@               l'existence du chemin d'acces passe en parametre
//@
//@ Parametres :
//@               (ENTREE) Chemin
//@               (ENTREE) droits
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__CreerArborescence(const t_Chaine & Chemin,mode_t droits) 
{
  t_Chaine RepertoireCourant;
  t_Chaine DernierCaractere;
  t_Chaine Delimiter;
  t_Chaine CheminAvecSlashFinal;
  unsigned long Pos;
		
  DernierCaractere = Chemin.substr(Chemin.size() - 1, 1);
  Delimiter = CMN__SEP_REPERTOIRE;
	
  CheminAvecSlashFinal = Chemin;
	
  //@ Cree un chemin tel que il y ait un slash a la fin
  if(DernierCaractere != Delimiter)
    CheminAvecSlashFinal += Delimiter.c_str();
	
  //@ Positionne Pos apres le premier Slash
  Pos = CheminAvecSlashFinal.ExtrairePosAvantChamp(Delimiter) + 1;

  //@ Positionne Pos avant le second Slash
  for(Pos = CheminAvecSlashFinal.ExtrairePosAvantChamp(Delimiter, Pos);
      (long) Pos >=0;
      )
    {
      //@ Cree le chemin partiel si il n'existe pas
      RepertoireCourant = CheminAvecSlashFinal.substr(0, Pos);
#ifdef _PC
      mkdir(RepertoireCourant.c_str());
#else
      mode_t oldmask=0;
      oldmask=umask(000);
      mkdir(RepertoireCourant.c_str(),droits);
      umask(oldmask);
#endif
      //@ Positionne Pos apres le slash
      Pos++;
      //@ Positionne Pos avant le slash suivant
      Pos = CheminAvecSlashFinal.ExtrairePosAvantChamp(Delimiter, Pos);
    }
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__RecuperEtatFileSystem
//@
//@ Description : Retourne le nombre d'octets utilises et libres pour
//@               le filesystem dont le chemin est passe en parametre
//@
//@ Parametres :
//@               (ENTREE) t_Chaine Chemin
//@               (SORTIE) double Taille
//@               (SORTIE) double Libre
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__RecupererEtatFileSystem(const t_Chaine& Chemin, 
				  double& Taille, 
				  double& Libre )
{
// FT-2765
#ifdef AIX
  struct statfs bstat;
  
  if( statfs( (char *) Chemin.c_str(), &bstat ) != -1 ){
#else
  struct statvfs bstat;

  if( statvfs( (const char *) Chemin.c_str(), &bstat ) != -1 ){
#endif
// ...FT-2765
    Libre = (double) bstat.f_bsize * (double) bstat.f_bfree;
    Taille = (double) bstat.f_bsize * (double) bstat.f_blocks;
  }else{
    Taille = Libre = -1.;
  }
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Acces
//@
//@ Description : Teste si les droits d'un fichier de donnee sont 
//@               corrects (rw-r--r--). Le chemin d'acces est passe
//@               en parametre
//@
//@ Parametres :
//@               (ENTREE) Chemin
//@               (ENTREE) Droits
//@
//@ Valeur de retour : 
//@               True si les droits rw-r--r-- sont positionnes
//@               False sinon
//@
//@ ------------------------------------------------------------------*/
int CMN__Acces(const t_Chaine & Chemin, 
	       int droits) 
{
  CMN__Stat  strStat;
  int retour;
  
  if (stat(Chemin.c_str(),&strStat) == 0)  {
    if( (strStat.st_mode & droits) == droits ) {
      retour =  0;
    } else {
      retour = -1;
    }
  } else {
    retour = -1;
  }
  
  return retour;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__RechercherFichier
//@
//@ Description : Teste si un fichier existe. Le nom du fichier peut
//@               contenir des patterns (* et ?)
//@
//@ Parametres :
//@               (ENTREE) nomComplet : Nom du fichier
//@               (ENTREE) pattern : Presence ou non de pattern dans
//@                                  le nom du fichier
//@               (SORTIE) listeFichier : Liste de fichiers
//@                                       correspondant a la recherche
//@
//@ Valeur de retour : 
//@               CMN__VRAI si le(un) fichier existe (match le pattern)
//@               CMN__FAUX sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Booleen CMN__RechercherFichier(const t_Chaine& nomComplet, 
				      t_CMN__Booleen pattern,
				      t_ListeChaine& listeFichier)
{
  CMN__Stat      strStat;

  // Code retour des fonctions utilisees
  t_CMN__Statut  status    = CMN__STATUT_NOK;
  
  // Valeur de retour de la fonction
  t_CMN__Booleen valRetour = CMN__FAUX;


  //@ RAZ de la liste en sortie
  listeFichier.clear();
  
  //@ SI le nom de fichier ne contient pas de pattern 
  if (pattern == CMN__FAUX)
    {
      //@ Tester avec la fonction stat
      
      //@ SI le fichier existe
      if (stat(nomComplet.c_str(), &strStat) == 0)
	{
	  //@ Ajout du fichier dans la liste en sortie
	  listeFichier.push_back(nomComplet);
	  
	  //@ Mettre a jour la valeur retournee
	  valRetour = CMN__VRAI;
	}
      //@ FIN SI
    }
  //@ SINON le nom de fichier contient un pattern
  else
    {
      t_Chaine        nomFichier("");
      t_Chaine        cheminFichier("");
      DIR*            repertoire = NULL;
      struct dirent*  pDirEnt    = NULL;
      
/* FA 184 */
  struct dirent pResult; // fichier
  int    status;
/* ..FA 184 */
      
      cheminFichier = nomComplet.
	substr(0, nomComplet.rfind(CMN__SEP_REPERTOIRE)+1);
      nomFichier    = nomComplet.
	substr(nomComplet.rfind(CMN__SEP_REPERTOIRE)+1);

      if ((!cheminFichier.empty()) && (!nomFichier.empty()))
	{
	  //@ Ouverture du repertoire
	  repertoire = opendir(cheminFichier.c_str());
	  
	  //@ SI l'acces au repertoire est possible
	  if (repertoire != NULL ) 
	    {
/* FA 184 */
	      //@ POUR tous les fichiers du repertoire
// FT-2765
#if AIX
 	      while(  (status=readdir_r(repertoire,&pResult,&pDirEnt)) ==0 )
#else // LINUX
  	      while( (pDirEnt = readdir(repertoire)) != NULL)
#endif
// ...FT-2765
	      {
/* ..FA 184 */
		  t_Chaine fileName = pDirEnt->d_name;
		  
		  //@ Comparer le nom au pattern
		  //@ SI le nom match le pattern
		  if (fnmatch(nomFichier.c_str(),
			      fileName.c_str(), 
			      FNM_PATHNAME) == 0)
		    {
		      //@ Inserer le fichier dans la liste des
		      //@ fichiers selectionnes
		      listeFichier.push_back(cheminFichier + fileName);
		    }
		  //@ FIN SI
		}
	      //@ FIN POUR tous les fichiers du repertoire
	      
	      //@ Fermer le repertoire
	      closedir(repertoire);
	    }
	  //@ FIN SI l'acces au repertoire est possible
	  
	  //@ SI la liste des fichiers n'est pas vide
	  if (! listeFichier.empty())
	    {
	      //@ Mettre a jour la valeur retournee
	      valRetour = CMN__VRAI;
	    }
	  //@ FIN SI la liste des fichiers n'est pas vide
	}
    }
  //@ FIN SI

  return valRetour;
}


//======================================================================
//					FONCTIONS INTERNES
//======================================================================
