// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/CMN__Fichier.h,v 1.3 2011/11/25 10:17:39 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : CMN
//@ MODULE : CMN__Fichier
//@ 
//@ Commentaire :
//@    Fichier d'entete du module CMN__Fichier qui propose un
//@    ensemble de services concernant les fichiers :
//@     - TrouverFichier: retourne la liste des fichiers correspondant
//@                       a une expression reguliere
//@     - TrouverFichierEx: retourne la liste des infos sur les
//@                         fichiers correspondant a une expression 
//@                         reguliere. Elle permet de specifier une 
//@                         fonction de selection ainsi qu'une 
//@                         fonction de tri
//@     - CopierFichier: copie un fichier dans un autre fichier	
//@ 		
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 12/10/1999 : Serge Bobin
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 14/12/2000 : C.LACAZE
//@ Portage sous systeme AIX 4.3
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Adaptation a l'OPS : ajout de la fonction CMN__RechercherFichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2766(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS2TEC SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef CMN__Fichier_h
#define CMN__Fichier_h

#include <vector>
#include <list>
#include "CMN__Commun.h"
extern "C" {
#include <sys/types.h>
#include <sys/stat.h>
}

//====================================================================*/
// Declaration des define externes
//====================================================================*/
#define CMN__SEP_REPERTOIRE     "/"
#define CMN__CAR_SEP_REPERTOIRE '/'


typedef struct stat CMN__Stat;


// Numero d'erreurs
// FT-2765
static const int CMN__ERR_MAUVAIS_FORMAT = 2000;
static const int CMN__ERR_PERMISSION_REFUSEE = 2001;
// ...FT-2765

// Class encapsulant les infos sur les fichiers
// ============================================
class CMN__InfoFichier
{
public :
	// Constructeur et destructeur
	CMN__InfoFichier() { };
	virtual ~CMN__InfoFichier() {};
	
	// Accesseurs en lecture
	const CMN__Stat & Stat() const { return m_Stat;};
	t_Chaine	 Name() const { return m_Name;};
	// Accesseurs en ecriture
	void Stat(const CMN__Stat & stats) { m_Stat = stats;};
	void Name(const t_Chaine & Name) { m_Name = Name;};

	// Operateur de comparaison 
	bool operator ==(const CMN__InfoFichier& InfoFichier) const 
				{return m_Name == InfoFichier.m_Name;};
	bool operator !=(const CMN__InfoFichier& InfoFichier) const 
				{return m_Name != InfoFichier.m_Name;};
	bool operator <(const CMN__InfoFichier& InfoFichier) const 
				{return m_Name < InfoFichier.m_Name;};
	bool operator >(const CMN__InfoFichier& InfoFichier) const 
				{return m_Name > InfoFichier.m_Name;};
	bool operator >=(const CMN__InfoFichier& InfoFichier) const 
				{return m_Name >= InfoFichier.m_Name;};
	bool operator <=(const CMN__InfoFichier& InfoFichier) const 
				{return m_Name <= InfoFichier.m_Name;};


// Donnees membres
private :
	CMN__Stat m_Stat;
	t_Chaine	 m_Name;
};


typedef vector<CMN__InfoFichier> CMN__ListeInfoFichier;

typedef bool (*t_fnSelect)(void * param,
						 const t_Chaine &Fichier,
						 const CMN__Stat &structStat);

typedef int (*t_fnTri)(const CMN__InfoFichier & , const CMN__InfoFichier &);


//====================================================================*/
// Declaration des fonctions externes
//====================================================================*/

int CMN__RenommerFichier(const t_Chaine& source, const t_Chaine& dest);

t_CMN__Statut CMN__TrouverFichiers(
					const t_Chaine & path,
					const t_Chaine & pattern,
				   	list<t_Chaine> & liste_fichiers,
					int 		NbMaxTentatives=1,
					int		Interval=0);
t_CMN__Statut CMN__TrouverFichiersEx(
				     const t_Chaine &		path,
				     const t_Chaine &		pattern,
				     CMN__ListeInfoFichier &	ListeInfoFichier,
				     t_fnSelect			fnSelect=NULL , 
				     t_fnTri				fnTri=NULL,
				     void *				param=NULL,
				     int 				NbMaxTentatives=1,
				     int				Interval=0);

t_CMN__Statut CMN__CopierFichier(	const t_Chaine & FichierSrc,
					const t_Chaine & FichierDest,
				  	bool bFailIfExist = true);

int CMN__OrdreAlphaFichier(const CMN__InfoFichier & info1, 
			  const CMN__InfoFichier & info2);

bool CMN__PlusVieuxQue(void * ParamDate, 
			  const t_Chaine & Fichier,
			  const CMN__Stat & Stat);

bool CMN__PlusRecentQue(void * ParamDate, 
			  const t_Chaine & Fichier,
			  const CMN__Stat & Stat);

int CMN__TriDateCroissante( const CMN__InfoFichier & info1,
			   const CMN__InfoFichier & info2);

void CMN__CreerArborescence(const t_Chaine & Chemin,mode_t droits=0770) ;


void CMN__RecupererEtatFileSystem(const t_Chaine& Chemin, 
				  double& Taille, 
				  double& Libre );

int CMN__Acces(const t_Chaine & Chemin, int droits = (S_IRUSR | S_IRGRP ));

t_CMN__Booleen CMN__RechercherFichier(const t_Chaine& nomComplet, 
				      t_CMN__Booleen pattern,
				      t_ListeChaine& listeFichier);


//====================================================================*/
// Declaration des variables externes
//====================================================================*/

#endif
