// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/CMN__OPSFile.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : CMN
//@ CLASSE : CMN__OPSFile
//@
//@ Commentaire :
//@    Ce module gere la classe CMN__OPSFile, qui permet de manipuler
//@    les noms de fichiers OPS
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ VERSION : 27/11/2002 : P.GASSIE
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ PM-ID : IA-CR-2100-268-CNE : 10/06/2005 : JL PASCAL
//@ initialisation complete des dates sur 15 caracteres.
//@ See IA-2100-CR-242-THA.
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-3316(DM-2100-OPS) : 18/05/2017 : MPE
//@ PORTAGE OPS SOUS RHEL 6.5 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

// Migration RHEL 6.5
#ifdef LINUX
#include <cstdlib>
#include <cstring>
#endif

#include "CMN__OPSFile.h"
// FT-2765
#define TRUE 1
#define FALSE 0
// ...FT-2765

//
// Definition des constantes
//

const int CMN__OPSFile::POS_PRODTYPE   = 2;
const int CMN__OPSFile::POS_PROCLEVEL  = 3;
const int CMN__OPSFile::POS_SPACECRAFT = 4;
const int CMN__OPSFile::POS_VALSTART   = 5;
const int CMN__OPSFile::POS_VALEND     = 6;
const int CMN__OPSFile::POS_CRE_DATE   = 7;
const int CMN__OPSFile::POS_IDENT = 9;
const int CMN__OPSFile::POS_IDENTCTX = 10;
const int CMN__OPSFile::POS_CENTER = 7;
const int CMN__OPSFile::POS_EDSOFT = 8;
const int CMN__OPSFile::POS_REVSOFT = 9;
const int CMN__OPSFile::POS_EDDATA = 10;
const int CMN__OPSFile::POS_REVDATA = 11;
const int CMN__OPSFile::POS_PROCDATE = 12;
const int CMN__OPSFile::POS_PROCMODE = 13;
const int CMN__OPSFile::POS_FLAG = 14;
const int CMN__OPSFile::POS_SUBDATE = 15;

const int CMN__OPSFile::NBRE_MIN  = 8;

const int CMN__OPSFile::INDICE_PROC_LEVEL = 9;
const int CMN__OPSFile::TAILLE_PROC_LEVEL = 2;
const int CMN__OPSFile::INDICE_START_DATE = 16;
const int CMN__OPSFile::INDICE_STOP_DATE  = 32;
const int CMN__OPSFile::INDICE_PROC_DATE  = 52;
//CR268 initialisation complete des dates sur 15 caracteres
const int CMN__OPSFile::TAILLE_DATE       = 15;

//======================================================================
//					METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CMN__OPSFile
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__OPSFile::CMN__OPSFile()
  : m_NomFichier(""), m_ProdType(""), m_Spacecraft(""), m_ProcLevel(""),
    m_Ident(-1), m_Center(""), m_SoftVersion(0.), m_DataVersion(0.), 
    m_ProcMode(""), m_Flag(""), m_edSoft(0), m_revSoft(0), m_edData(0), 
    m_revData(0)
{
  //@ initialisation des attributs
}

//@ ------------------------------------------------------------------*/
//@ Methode : CMN__OPSFile
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               (ENTREE) nom_fichier : chaine a partir de laquelle le nom
//@                                      du fichier est construit 
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__OPSFile::CMN__OPSFile(const t_Chaine nom_fichier)
  : m_NomFichier(""), m_ProdType(""), m_Spacecraft(""), 
    m_ProcLevel(""), m_Ident(-1), m_Center(""), m_SoftVersion(0.), 
    m_DataVersion(0.), m_ProcMode(""), m_Flag(""), m_edSoft(0), 
    m_revSoft(0), m_edData(0), m_revData(0)
{
  //@ recuperation du nom court du fichier
  m_NomFichier = NomCourt(nom_fichier);

  //@ initialisation des attributs
  this->Initialise();
}

//@ ------------------------------------------------------------------*/
//@ Methode : ~CMN__OPSFile
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__OPSFile::~CMN__OPSFile()
{
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetProdType
//@
//@ Description : Retourne le type de produit
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@               t_Chaine : type de produit
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN__OPSFile::GetProdType()
{
  //@ retourne m_ProdType
  return(m_ProdType);
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetSpacecraft
//@
//@ Description : Retourne le nom du spacecraft
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@               t_Chaine : nom du spacecraft
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN__OPSFile::GetSpacecraft()
{
  //@ retourne m_Spacecraft
  return(m_Spacecraft);
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetValidityStartDate
//@
//@ Description : Retourne la date de debut de validite
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@               t_Date : date de debut de validite
//@
//@ ------------------------------------------------------------------*/
t_Date CMN__OPSFile::GetValidityStartDate()
{
  //@ retourne m_DateValiditeStart
  return(m_DateValiditeStart);
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetValidityEndDate
//@
//@ Description : Retourne la date de fin de validite
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@               t_Date : date de fin de validite
//@
//@ ------------------------------------------------------------------*/
t_Date CMN__OPSFile::GetValidityEndDate()
{
  //@ retourne m_DateValiditeEnd
  return(m_DateValiditeEnd);
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetCreationDate
//@
//@ Description : Retourne la date de creation
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@               t_Date : date de creation
//@
//@ ------------------------------------------------------------------*/
t_Date CMN__OPSFile::GetCreationDate()
{
  //@ retourne la date de creation
  return(m_DateCreation);
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetProcLevel
//@
//@ Description : Retourne le proc level
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@               int : proc level
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN__OPSFile::GetProcLevel()
{
  //@ retourne m_ProcLevel
  return(m_ProcLevel);
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetIdent
//@
//@ Description : Retourne l'identifiant
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@               int : identifiant
//@
//@ ------------------------------------------------------------------*/
int CMN__OPSFile::GetIdent()
{
  //@ retourne m_Ident
  return(m_Ident);
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetNumeroGranule
//@
//@ Description : Retourne le numero de granule du contexte
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@               int : NumeroGranule
//@
//@ ------------------------------------------------------------------*/
int CMN__OPSFile::GetNumeroGranule()
{
  //@ retourne m_Ident
  return(m_IdentCtx);
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetCenter
//@
//@ Description : Retourne le nom du centre
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@               t_Chaine : nom
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN__OPSFile::GetCenter()
{
  //@ retourne m_Center
  return(m_Center);
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetSoftVersion
//@
//@ Description : Retourne le numero de version soft
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@               float : numero de version
//@
//@ ------------------------------------------------------------------*/
float CMN__OPSFile::GetSoftVersion()
{
  //@ retourne m_SoftVersion
  return(m_SoftVersion);
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetDataVersion
//@
//@ Description : Retourne le numero de version du format de donnees
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@               float : numero de version
//@
//@ ------------------------------------------------------------------*/
float CMN__OPSFile::GetDataVersion()
{
  //@ retourne m_DataVersion
  return(m_DataVersion);
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetProcDate
//@
//@ Description : Retourne le proc date
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@               t_Date : date
//@
//@ ------------------------------------------------------------------*/
t_Date CMN__OPSFile::GetProcDate()
{
  //@ retourne m_ProcDate
  return(m_ProcDate);
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetProcMode
//@
//@ Description : Retourne le proc mode
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@               t_Chaine : mode
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN__OPSFile::GetProcMode()
{
  //@ retourne m_ProcMode
  return(m_ProcMode);
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetFlag
//@
//@ Description : Retourne le flag
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@               t_Chaine : flag
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN__OPSFile::GetFlag()
{
  //@ retourne m_Flag
  return(m_Flag);
}
//@ ------------------------------------------------------------------*/
//@ Methode : GetSubDate
//@
//@ Description : Retourne la date de subset
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@               t_Date : date
//@
//@ ------------------------------------------------------------------*/
t_Date CMN__OPSFile::GetSubDate()
{
  //@ retourne m_SubDate
  return(m_SubDate);
}

//@ ------------------------------------------------------------------*/
//@ Methode : Init
//@
//@ Description : Initialisation d'un nom de fichier
//@
//@ Parametres :
//@               (ENTREE) TypeFichier : type du fichier 
//@
//@ Valeur de retour : 
//@               
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::Init(const e_TypeFile& TypeFichier)
{
  //@ enregistrement dans m_NomFichier du modele de nom
  m_NomFichier = MODEL_PRODUCT[TypeFichier];
}

//@ ------------------------------------------------------------------*/
//@ Methode : SetProcLevel
//@
//@ Description : Initialisation du proc level
//@
//@ Parametres :
//@               (ENTREE) procLevel
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::SetProcLevel(const t_Chaine& procLevel)
{
  //@ SI la taille du procLevel est egale a TAILLE_PROC_LEVEL
  if (procLevel.length() == TAILLE_PROC_LEVEL)
    {
      //@ Met a jour m_ProcLevel et m_NomFichier
      m_ProcLevel = procLevel;

      for (int i=0; i<TAILLE_PROC_LEVEL; i++)
        {
          m_NomFichier[INDICE_PROC_LEVEL+i] = procLevel[i];
        }
    }
  //@ FIN SI
}

//@ ------------------------------------------------------------------*/
//@ Methode : SetOrbit
//@
//@ Description : Initialisation du nom du spacecraft
//@
//@ Parametres :
//@               (ENTREE) Orbit : numero orbit
//@
//@ Valeur de retour : 
//@               
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::SetOrbit(const int& Orbit)
{
  char buffer[6];
  sprintf(buffer,"%05d", Orbit);

  t_Chaine chaineTmp(buffer);

  //@ remplacer le champ correspondant
  m_NomFichier.RemplacerValeurAncienne("ooooo",chaineTmp);
}

//@ ------------------------------------------------------------------*/
//@ Methode : SetSpacecraft
//@
//@ Description : Initialisation du nom du spacecraft
//@
//@ Parametres :
//@               (ENTREE) Spacecraft : nom du spacecraft 
//@
//@ Valeur de retour : 
//@               
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::SetSpacecraft(const t_Chaine& Spacecraft)
{
  //@ remplacer le champ correspondant
  m_NomFichier.RemplacerValeurAncienne("sss",Spacecraft);
}

//@ ------------------------------------------------------------------*/
//@ Methode : SetValidityStartDate
//@
//@ Description : Initialisation de la date de debut de validite
//@
//@ Parametres :
//@               (ENTREE) ValidityDate : date
//@
//@ Valeur de retour : 
//@               
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::SetValidityStartDate(const t_Date& ValidityDate)
{
  t_Date dateTmp(ValidityDate);
  //@ remplacer le champ correspondant
  m_NomFichier.RemplacerValeurAncienne("ddddddddddddddd",dateTmp.DateUTC(false,false));
}

//@ ------------------------------------------------------------------*/
//@ Methode : RemplacerValidityStartDate
//@
//@ Description : Remplace la date de debut de validite
//@
//@ Parametres :
//@               (ENTREE) date
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::RemplacerValidityStartDate(const t_Date& date)
{
  t_Date   dateTmp(date);
  t_Chaine strDate = dateTmp.DateUTC(false,false);

  //@ Met a jour m_DateValiditeStart et m_NomFichier
  m_DateValiditeStart = date;

  for (int i=0; i<TAILLE_DATE; i++)
    {
      m_NomFichier[INDICE_START_DATE+i] = strDate[i];
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : ResetValidityStartDate
//@
//@ Description : Reset la date de debut de validite
//@
//@ Parametres :
//@               (ENTREE) date
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::ResetValidityStartDate()
{
  //@ Met a jour m_NomFichier
  for (int i=0; i<TAILLE_DATE; i++)
    {
      m_NomFichier[INDICE_START_DATE+i] = '?';
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : SetValidityEndDate
//@
//@ Description : Initialisation de la date de fin de validite
//@
//@ Parametres :
//@               (ENTREE) ValidityDate : date
//@
//@ Valeur de retour : 
//@               
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::SetValidityEndDate(const t_Date& ValidityDate)
{
  t_Date dateTmp(ValidityDate);
  //@ remplacer le champ correspondant
  m_NomFichier.RemplacerValeurAncienne("eeeeeeeeeeeeeee",dateTmp.DateUTC(false,false));
}


//@ ------------------------------------------------------------------*/
//@ Methode : RemplacerValidityEndDate
//@
//@ Description : Remplace la date de fin de validite
//@
//@ Parametres :
//@               (ENTREE) date
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::RemplacerValidityEndDate(const t_Date& date)
{
  t_Date   dateTmp(date);
  t_Chaine strDate = dateTmp.DateUTC(false,false);

  //@ Met a jour m_DatevaliditeEnd et m_NomFichier
  m_DateValiditeEnd = date;

  for (int i=0; i<TAILLE_DATE; i++)
    {
      m_NomFichier[INDICE_STOP_DATE+i] = strDate[i];
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : ResetValidityEndDate
//@
//@ Description : Reset la date de fin de validite
//@
//@ Parametres :
//@               (ENTREE) date
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::ResetValidityEndDate()
{
  //@ Met a jour m_NomFichier
  for (int i=0; i<TAILLE_DATE; i++)
    {
      m_NomFichier[INDICE_STOP_DATE+i] = '?';
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : SetCenter
//@
//@ Description : Initialisation du nom du centre
//@
//@ Parametres :
//@               (ENTREE) NomCentre : nom du centre
//@
//@ Valeur de retour : 
//@               
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::SetCenter(const t_Chaine& NomCentre)
{
  //@ remplacer le champ correspondant
  m_NomFichier.RemplacerValeurAncienne("cccc",NomCentre);
}

//@ ------------------------------------------------------------------*/
//@ Methode : SetSoftEdition
//@
//@ Description : Initialisation du numero d'edition soft
//@
//@ Parametres :
//@               (ENTREE) SoftEdition : numero d'edition
//@
//@ Valeur de retour : 
//@               
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::SetSoftEdition(const int& SoftEdition)
{
  char buffer[5];
  sprintf(buffer,"%04d",SoftEdition);

  t_Chaine chaineTmp(buffer);

  //@ remplacer le champ correspondant
  m_NomFichier.RemplacerValeurAncienne("hhhh", chaineTmp);
}

//@ ------------------------------------------------------------------*/
//@ Methode : SetSoftRevision
//@
//@ Description : Initialisation du numero de revision soft
//@
//@ Parametres :
//@               (ENTREE) SoftRevision : numero de revision
//@
//@ Valeur de retour : 
//@               
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::SetSoftRevision(const int& SoftRevision)
{
  char buffer[5];
  sprintf(buffer,"%04d",SoftRevision);

  t_Chaine chaineTmp(buffer);

  //@ remplacer le champ correspondant
  m_NomFichier.RemplacerValeurAncienne("jjjj", chaineTmp);
}

//@ ------------------------------------------------------------------*/
//@ Methode : SetDataEdition
//@
//@ Description : Initialisation du numero d'edition du format des donnees
//@
//@ Parametres :
//@               (ENTREE) DataEdition : numero d'edition
//@
//@ Valeur de retour : 
//@               
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::SetDataEdition(const int& DataEdition)
{
  char buffer[5];
  sprintf(buffer,"%04d",DataEdition);

  t_Chaine chaineTmp(buffer);

  //@ remplacer le champ correspondant
  m_NomFichier.RemplacerValeurAncienne("kkkk", chaineTmp);
}

//@ ------------------------------------------------------------------*/
//@ Methode : SetDataRevision
//@
//@ Description : Initialisation du numero de revision du format des donnees
//@
//@ Parametres :
//@               (ENTREE) DataRevision : numero de revision
//@
//@ Valeur de retour : 
//@               
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::SetDataRevision(const int& DataRevision)
{
  char buffer[5];
  sprintf(buffer,"%04d",DataRevision);

  t_Chaine chaineTmp(buffer);

  //@ remplacer le champ correspondant
  m_NomFichier.RemplacerValeurAncienne("llll", chaineTmp);
}    

//@ ------------------------------------------------------------------*/
//@ Methode : SetProcDate
//@
//@ Description : Initialisation de la date creation
//@
//@ Parametres :
//@               (ENTREE) ProcDate : date
//@
//@ Valeur de retour : 
//@               
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::SetProcDate(const t_Date& ProcDate)
{
  t_Date dateTmp(ProcDate);
  //@ remplacer le champ correspondant
  m_NomFichier.RemplacerValeurAncienne("fffffffffffffff", dateTmp.DateUTC(false,false));
}


//@ ------------------------------------------------------------------*/
//@ Methode : RemplacerProcDate
//@
//@ Description : Remplace la date de creation
//@
//@ Parametres :
//@               (ENTREE) date
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::RemplacerProcDate(const t_Date& date)
{
  t_Date   dateTmp(date);
  t_Chaine strDate = dateTmp.DateUTC(false,false);

  //@ Met a jour m_ProcDate et m_NomFichier
  m_ProcDate = date;

  for (int i=0; i<TAILLE_DATE; i++)
    {
      m_NomFichier[INDICE_PROC_DATE+i] = strDate[i];
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : ResetProcDate
//@
//@ Description : Reset la date de creation
//@
//@ Parametres :
//@               (ENTREE) date
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::ResetProcDate()
{
  //@ Met a jour m_NomFichier
  for (int i=0; i<TAILLE_DATE; i++)
    {
      m_NomFichier[INDICE_PROC_DATE+i] = '?';
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : SetMode
//@
//@ Description : Initialisation du mode de production
//@
//@ Parametres :
//@               (ENTREE) Mode : mode
//@
//@ Valeur de retour : 
//@               
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::SetMode(const t_Chaine& Mode)
{
  //@ remplacer le champ correspondant
  m_NomFichier.RemplacerValeurAncienne("n", Mode);
}

//@ ------------------------------------------------------------------*/
//@ Methode : SetFlag
//@
//@ Description : Initialisation du flag
//@
//@ Parametres :
//@               (ENTREE) Flag : flag
//@
//@ Valeur de retour : 
//@               
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::SetFlag(const t_Chaine& Flag)
{
  //@ remplacer le champ correspondant
  m_NomFichier.RemplacerValeurAncienne("o", Flag);
}

//@ ------------------------------------------------------------------*/
//@ Methode : SetSource
//@
//@ Description : Initialisation du nom de la source
//@
//@ Parametres :
//@               (ENTREE) Source : source
//@
//@ Valeur de retour : 
//@               
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::SetSource(const t_Chaine& Source)
{
  //@ remplacer le champ correspondant
  m_NomFichier.RemplacerValeurAncienne("uuuu", Source);
}

//@ ------------------------------------------------------------------*/
//@ Methode : SetSubsetDate
//@
//@ Description : Initialisation de la date subset
//@
//@ Parametres :
//@               (ENTREE) SubsetDate : date
//@
//@ Valeur de retour : 
//@               
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::SetSubsetDate(const t_Date& SubsetDate)
{
  t_Date dateTmp(SubsetDate);
  //@ remplacer le champ correspondant
  m_NomFichier.RemplacerValeurAncienne("ggggggggggggggg", dateTmp.DateUTC(false,false));
}

//@ ------------------------------------------------------------------*/
//@ Methode : SetIdent
//@
//@ Description : Initialisation de l'ident
//@
//@ Parametres :
//@               (ENTREE) Ident : ident
//@
//@ Valeur de retour : 
//@               
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::SetIdent(const int& Ident)
{
  char buffer[6];
  sprintf(buffer,"%05d",Ident);

  t_Chaine IdentTmp(buffer);

  //@ remplacer le champ correspondant
  m_NomFichier.RemplacerValeurAncienne("iiiii", IdentTmp);
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetNom
//@
//@ Description : Retourne le nom du fichier
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@               t_Chaine : nom du fichier
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN__OPSFile::GetNom()
{
  //@ retourne m_NomFichier
  return(m_NomFichier);
}

//======================================================================
//					METHODES PRIVEES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : Initialise
//@
//@ Description : Initialisation des attributs
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::Initialise()
{
  t_Chaine fichierTmp(m_NomFichier);
  t_Chaine mot("");
  int longueurNom = -1;
  bool avhr = FALSE;
  
  //@ determination de la longueur du nom de fichier
  longueurNom = fichierTmp.NombreDeMot("_");
  //@ recuperation du premier mot
  fichierTmp.GetFirstMot("_",mot);
  
  //@ SI fichier AVHR
  if (mot == "AVHR")
  {
    //@ on positionne avhr a true
    avhr = TRUE;
  }
  //@ 
  //@ SI le nom a la longueur suffisante
  if (longueurNom >= NBRE_MIN)
	{
	  // POUR chaque mot du nom du fichier
	  for (int i = 2; i < (longueurNom + 1); i++)
	  {
	    fichierTmp.GetNextMot("_",mot);
	    //@enregistrement du mot
	    EnregistreMot(mot, i, avhr);
	  }
    // FIN POUR
    //@ si fichier AVHR
    if(avhr)
    {
      //@ determination de la version du soft
      char versionSoft[9];
      // bug fix P. Roquet P. Brunel fails on ubuntu 12.04 en gcc -O2
      m_edSoft = 0;
      m_revSoft = 0;
      sprintf(versionSoft,"%d.%d", m_edSoft, m_revSoft);
      m_SoftVersion = atof(versionSoft);
      //@ determination de la version du format de donnees
      char versionData[9];
      sprintf(versionData,"%d.%d", m_edData, m_revData);
      m_DataVersion = atof(versionData);
    }
    //@ FIN SI 
  }
  // FIN SI
}

//@ ------------------------------------------------------------------*/
//@ Methode : EnregistreMot
//@
//@ Description : Enregistrement d'un mot en fonction de sa position
//@               dans le nom du fichier
//@
//@ Parametres :
//@               (ENTREE) mot : partie du nom du fichier
//@               (ENTREE) position : position du mot dans le nom
//@               (ENTREE) avhr : flag pour fichier avhr
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__OPSFile::EnregistreMot(t_Chaine mot,
                                 int position,
                                 bool avhr)
{
  // SI position prod type
  if (position == POS_PRODTYPE)
    {
      // enregistrement du type de produit
      m_ProdType = mot;
    }
  // FIN SI
  // SI position proc level
  if (position == POS_PROCLEVEL)
    {
      // enregistrement du proc level
      m_ProcLevel = mot;
    }
  // FIN SI
  // SI position nom du spacecraft
  if (position == POS_SPACECRAFT)
    {
      // enregistrement du nom du spacecraft
      m_Spacecraft = mot;
    }
  // FIN SI
  // SI position date validite start
  if (position == POS_VALSTART)
    {
      // enregistrement de la date de validite
      m_DateValiditeStart.TransformeDate(mot);
    }
  // FIN SI
  // SI position date validite end
  if (position == POS_VALEND)
    {
      // enregistrement de la date de validite
      m_DateValiditeEnd.TransformeDate(mot);
    }
  // FIN SI
  // SI position date creation et pas fichier avhr
  if ((position == POS_CRE_DATE) && (! avhr))
    {
      // enregistrement de la date de creation
      m_DateCreation.TransformeDate(mot);
    }
  // FIN SI
  // SI position identificateur
  if ((position == POS_IDENT) && (! avhr))
    {
      // enregistrement de l'identificateur 
      m_Ident = atoi(mot.c_str());
    }
  // FIN SI
  // SI position identificateur contexte
  if ((position == POS_IDENTCTX) && (! avhr))
    {
      // enregistrement de l'identificateur 
      m_IdentCtx = atoi(mot.c_str());
    }
  // FIN SI
  // SI position nom du centre
  if((position == POS_CENTER) && avhr)
    {
      //enregistrement du nom
      m_Center = mot;
    }
  // FIN SI
  // SI position numero d'edition soft
  if((position == POS_EDSOFT) && avhr)
    {
      //enregistrement
      m_edSoft = atoi(mot.c_str());
    }
  // FIN SI
  // SI position numero de revision soft
  if((position == POS_REVSOFT) && avhr)
    {
      //enregistrement
      m_revSoft = atoi(mot.c_str());
    }
  // FIN SI
  // SI position numero d'edition du format des donnees
  if((position == POS_EDDATA) && avhr)
    {
      //enregistrement
      m_edData = atoi(mot.c_str());
    }
  // FIN SI
  // SI position numero de revision du format des donnees
  if((position == POS_REVDATA) && avhr)
    {
      //enregistrement
      m_revData = atoi(mot.c_str());
    }
  // FIN SI
  // SI position proc date
  if((position == POS_PROCDATE) && avhr)
    {
      //enregistrement 
      m_ProcDate.TransformeDate(mot);
    }
  // FIN SI
  // SI position proc mode
  if((position == POS_PROCMODE) && avhr)
    {
      //enregistrement 
      m_ProcMode = mot;
    }
  // FIN SI
  // SI position flag
  if((position == POS_FLAG) && avhr)
    {
      //enregistrement 
      m_Flag = mot;
    }
  // FIN SI
  // SI position date subset
  if((position == POS_SUBDATE) && avhr)
    {
      //enregistrement 
      m_SubDate.TransformeDate(mot);
    }
  // FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : NomCourt
//@
//@ Description : Recupere le nom court d'un fichier
//@
//@ Parametres :
//@               (ENTREE) nomFichier : Nom du fichier
//@
//@ Valeur de retour :
//@               Nom court du fichier
//@
//@ ------------------------------------------------------------------*/
t_Chaine CMN__OPSFile::NomCourt(const t_Chaine& nomFichier)
{
  //@ Affecter le nom de fichier a la variable du nom court
  t_Chaine            nomCourt(nomFichier);
  t_Chaine::size_type posDebut = t_Chaine::npos;
  int                 longueur = 0;


  //@ Rechercher le caractere separateur de repertoire
  posDebut = nomFichier.rfind(CMN__SEP_REPERTOIRE);

  //@ SI le caractere a ete trouve
  if (posDebut != t_Chaine::npos)
    {
      posDebut += strlen(CMN__SEP_REPERTOIRE);
      longueur = nomFichier.length() - posDebut;

      //@ Affecter le nom court avec les caracteres suivants
      nomCourt = nomFichier.substr(posDebut, longueur);
    }
  //@ FIN SI


  //@ Retourner le nom court du fichier
  return nomCourt;
}
