// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/CMN__OPSFile.h,v 1.3 2011/11/25 10:17:39 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : CMN
//@ CLASSE : CMN__OPSFile
//@
//@ Commentaire :
//@    Ce module gere la classe CMN__OPSFile, qui permet de manipuler
//@    les noms de fichiers OPS
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ VERSION : 27/11/2002 : P.GASSIE
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef CMN__OPSFile_h
#define CMN__OPSFile_h


//
// Fichiers include
//
#include "CMN__Commun.h"
#include "CMN__Chaine.h"
#include "CMN__Date.h"
#include "CMN__Fichier.h"


static const t_Chaine MODEL_PRODUCT [] = {
  "IASI_xxx_1A_sss_ddddddddddddddd_eeeeeeeeeeeeeee_n_o_ggggggggggggggg",
  "IASI_xxx_1B_sss_ddddddddddddddd_eeeeeeeeeeeeeee_n_o_ggggggggggggggg",
  "IASI_xxx_1C_sss_ddddddddddddddd_eeeeeeeeeeeeeee_n_o_ggggggggggggggg",
  "IASI_VER_01_sss_ddddddddddddddd_eeeeeeeeeeeeeee_n_o_ggggggggggggggg",
  "IASI_ENG_01_sss_ddddddddddddddd_eeeeeeeeeeeeeee_n_o_ggggggggggggggg",
  "IASI_CTX_xx_sss_ddddddddddddddd_xxxxxxxxxxxxxxx_fffffffffffffff_uuuu_xxxxxooooo"
};
  
//
// CLASS DEFINITION
//

class CMN__OPSFile
{
    /*================================================================*/
    
 public:
    
    //
    // Definitions de types
    //
    typedef enum {
        FILE_I1A = 0,
        FILE_I1B = 1,
        FILE_I1C = 2,
        FILE_VER = 3,
        FILE_ENG = 4,
        FILE_CTX = 5
    } e_TypeFile;
    
    //
    // Constructeurs
    //
    CMN__OPSFile();
    CMN__OPSFile(const t_Chaine nom_fichier);
    
    //
    // Destructeur
    //
    virtual ~CMN__OPSFile();
    
    //
    // Methodes publiques
    //
    
    // Extraire les informations d'un nom de fichier
    t_Chaine GetProdType();
    t_Chaine GetProcLevel();
    t_Chaine GetSpacecraft();
    t_Date   GetValidityStartDate();
    t_Date   GetValidityEndDate();
    t_Date   GetCreationDate();
    int      GetIdent();
    int      GetNumeroGranule();
    
    // Methodes associees aux fichiers AVHR
    t_Chaine GetCenter();
    float    GetSoftVersion();
    float    GetDataVersion();
    t_Date   GetProcDate();
    t_Chaine GetProcMode();
    t_Chaine GetFlag();
    t_Date   GetSubDate();
    
    // Construire un nom de fichier
    void Init(const e_TypeFile&);
    void SetProcLevel(const t_Chaine&);
    void SetSpacecraft(const t_Chaine&);
    void SetValidityStartDate(const t_Date&);
    void RemplacerValidityStartDate(const t_Date&);
    void ResetValidityStartDate();
    void SetValidityEndDate(const t_Date&);
    void RemplacerValidityEndDate(const t_Date&);
    void ResetValidityEndDate();
    void SetCenter(const t_Chaine&);
    void SetSoftEdition(const int&);
    void SetSoftRevision(const int&);
    void SetDataEdition(const int&);
    void SetDataRevision(const int&);
    void SetProcDate(const t_Date&);
    void RemplacerProcDate(const t_Date&);
    void ResetProcDate();
    void SetMode(const t_Chaine&);
    void SetFlag(const t_Chaine&);
    void SetSubsetDate(const t_Date&);
    void SetIdent(const int&);
    void SetOrbit(const int&);
    void SetSource(const t_Chaine&);
    t_Chaine GetNom();
  

    /*================================================================*/

protected:
    
    //
    // Methodes protegees
    //
    
    //
    // Attributs proteges
    //


    /*================================================================*/

private:

    //
    // Constantes
    //
    static const int POS_PRODTYPE;
    static const int POS_PROCLEVEL;
    static const int POS_SPACECRAFT;
    static const int POS_VALSTART;
    static const int POS_VALEND;
    static const int POS_CRE_DATE;
    static const int POS_IDENT;
    static const int POS_IDENTCTX;
    static const int POS_CENTER;
    static const int POS_EDSOFT;
    static const int POS_REVSOFT;
    static const int POS_EDDATA;
    static const int POS_REVDATA;
    static const int POS_PROCDATE;
    static const int POS_PROCMODE;
    static const int POS_FLAG;
    static const int POS_SUBDATE;

    static const int NBRE_MIN; 

    static const int INDICE_PROC_LEVEL;
    static const int TAILLE_PROC_LEVEL;
    static const int INDICE_START_DATE;
    static const int INDICE_STOP_DATE;
    static const int INDICE_PROC_DATE;
    static const int TAILLE_DATE;

    //
    // Methodes privees
    //
    void     Initialise();
    void     EnregistreMot(t_Chaine mot, int position, bool avhr);
    t_Chaine NomCourt(const t_Chaine& nomFichier);
  
    //
    // Attributs
    //
    t_Chaine m_NomFichier;
    t_Chaine m_ProdType;
    t_Chaine m_Spacecraft;
    t_Date   m_DateValiditeStart;
    t_Date   m_DateValiditeEnd;
    t_Date   m_DateCreation;
    t_Chaine m_ProcLevel;
    int      m_Ident;
    int      m_IdentCtx;
    t_Chaine m_Center;
    float    m_SoftVersion;
    float    m_DataVersion;
    t_Date   m_ProcDate;
    t_Chaine m_ProcMode;
    t_Chaine m_Flag;
    t_Date   m_SubDate;
    int      m_edSoft;
    int      m_revSoft;
    int      m_edData;
    int      m_revData;
};

//
// Type
//
typedef CMN__OPSFile t_OPSFile;

#endif
