static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/CMN__PGEServices.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : CMN
//@ CLASSE : CMN__PGEServices
//@
//@ Commentaire :
//@    Ce fichier contient la classe CMN__PGEServices, qui gere les
//@    interfaces de type PPF-IASI/MCS. 
//@    Il regroupe les acces a l'API MCS
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : PBR
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 18/11/2002 : PBR
//@ FA-ID : OPS-I00-010 : 18/11/2002 :
//@ Non reception de commandes envoyees quasi-systematiquement
//@ Creation des methodes : ExtractPGFCommand, GetFirstCommand,
//@ GetNextCommand dans CMN__PGEServices.
//@ --------------------------------------------------------------------
//@ VERSION : 03/12/2002 : PBR
//@ FA-ID : OPS-I01-014  03/12/2002 :
//@ MLAInitialisation / MLATerminate : pas d'appel a ces fonctions
//@ par les process n'utilisant pas les services de cette librairie.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-095-CN : 5/02/2004 : JL.PASCAL
//@ une trace pour les STAGE TM est emise en mode debug uniquement
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-159-CN : 26/05/2005 : JL. PASCAL
//@ modif suite a portage linux
//@ fiche de correction correspondante : IA-2100-FC-159-THA
//@ --------------------------------------------------------------------
//@ DM-ID : IA-2100-CR-292-CNE : 10/03/2006 : B.MASSART
//@ The PROCESSING_STATE field of HKTM messages is now an integer (1 means OK
//@ and 2 means FAILURE). See IA-2100-CR-292-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-181-CN : 26/09/2006 : C.MERIGUET
//@ Mise a jour du SpaceCraftID utilise dans les logs a partir de la
//@ variable d'environnement correspondante.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-184-CN : 18/10/2006 : C.MERIGUET
//@ Codage des fonctions reentrantes
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-3251 : 29/05/2017 : M. LANDIECH
//@ OPS doit pouvoir démarer sans le MLA
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//
#include <stdlib.h>
#include <string.h>

#include <math.h>

#include "MLA.h"
#include "JDB__Message.h"
#include "CMN__CGSCst.h"
#include "CMN__OPSProcess.h"
#include "CMN__PGEServices.h"
#include "CMN__PGEServicesAbstract.h"
#include "CMN__PGEServicesAPI.h"
#include "CMN__PGEServicesStub.h"
#include "CFG__Configuration.h"
#include "MP__HKTMStatus.h"
#include "MP__Command.h"
#include "MP__StageTM.h"



//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CMN__PGEServices
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__PGEServices::CMN__PGEServices(e_CMN__Processus processUtilisateur)
{
  t_Chaine instalType;

  m_process = processUtilisateur;

  //@ The constructor retrieves the current installation type in order 
  //@ to initialize the PGF API : nominal or stub
  instalType = CFG__Configuration::GetChaine("OPS_INSTAL_TYPE");

#ifdef DEBUG
  cout << "OPS_INSTAL_TYPE :" << instalType << " "<<OPS_INSTAL_CGS<<endl;
#endif


  if (instalType == OPS_INSTAL_CGS)
     {
      m_InstalType=INSTAL_CGS;
#ifdef DEBUG
      cout << "Install CGS"<<endl;
#endif
      CMN__PGEServicesAPI * ptrPGEServicesAPI;
      ptrPGEServicesAPI= new CMN__PGEServicesAPI(processUtilisateur);

      m_PGEServices = ptrPGEServicesAPI;
     }
  else
     {
      m_InstalType=INSTAL_STANDALONE;
#ifdef DEBUG
      cout << "Install STANDALONE"<<endl;
#endif
      CMN__PGEServicesStub * ptrPGEServicesStub;
      ptrPGEServicesStub= new CMN__PGEServicesStub(processUtilisateur);

      m_PGEServices = ptrPGEServicesStub;
     }

  //@ Initialisation du spacecraft par defaut a NONE
  m_Spacecraft = UNKNOWN_SERVICE;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~CMN__PGEServices
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__PGEServices::~CMN__PGEServices()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : MLAInitialisation
//@
//@ Description : This method initialises the interface between an OPS
//@               processus and the PGF services
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               t_CMN__PGEStatus : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__PGEServices::MLAInitialisation()
{
  t_CMN__PGEStatus status = CMN__STATUT_OK;
  t_Chaine facilityName = CMN_OPS_FACILITY_NAME;
/* FA 181 */
  t_Chaine SpacecraftID("");
  int      i;
/* ..FA 181 */

#ifdef DEBUG
  cout << "Process : "<<m_process<<endl;
#endif
/* FA 181 */
  /* recuperation de la variable d'env SPACECRAFT_ID */
  SpacecraftID = getenv("SPACECRAFT_ID");
  if ( ! SpacecraftID.empty() )
  {
    for (i= 0; i < NbrSpacecraft; i++)
    {
       if (SpacecraftID.c_str()== CMN_CGS_SERVICES[i])
        m_Spacecraft = (e_CMN_CGSSpacecraft) i;
    }
  }
/* ..FA 181 */
  
  if ( (m_process == PROCESS_MP) || (m_process == PROCESS_JDBS) ){
     status= m_PGEServices->MLAInitialisation (facilityName.c_str());
  }
  
  // FT 3251 -- L'OPS doit pouvoir démarrer sans MLA
  status = CMN__STATUT_OK;
  // FT 3251

  return (status);
}


//@ ------------------------------------------------------------------*/
//@ Methode : MLATerminate
//@
//@ Description : This method closes the interface between an OPS
//@               processus and the PGF services
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               t_CMN__PGEStatus : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__PGEServices::MLATerminate()
{
  t_CMN__PGEStatus status;

  if ( (m_process == PROCESS_MP) || (m_process == PROCESS_JDBS) ){
     status = m_PGEServices->MLATerminate();
  }

  return (status);
}


//@ ------------------------------------------------------------------*/
//@ Methode : MLASendFunctionStatus
//@
//@ Description : This method manages MLA sendFunction Status services
//@
//@ Parametres :
//@               (ENTREE) HKTM
//@
//@ Valeur de retour :
//@               t_CMN__PGEStatus : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__PGEServices::MLASendFunctionStatus(MP__HKTMStatus & HKTM)
{
   t_CMN__PGEStatus status;
   timeValue  timeEvt;
   tmParam    HKTMparam;
   CMN__Date  date;
   t_Chaine   paramName;
   t_Chaine   paramValueStr;
   char      encodedValue[PAR_STR_LEN];
   double    temps;
   MP__HKTMStatus::e_TypeStatus HKTMType;
 
   //@ Recuperer la date Courante
   date.DateCourante ();
   temps = date.ConvertirDateEnTemps ();
   timeEvt.sec  = (int)floor(temps);
   timeEvt.usec = (int) ((temps-floor(temps))*OPS_PGE_PRECISION_DATE);

   //@ Extraire de l'objet HKTMStatus les informations a envoyer au MCS
   paramName = HKTM.GetParamName ();
   strncpy (HKTMparam.paramName, paramName.c_str(), PAR_NAM_LEN);
   //FA159
   HKTMparam.paramName [PAR_NAM_LEN-1] ='\0';

   //@ Encoder la valeur du parametres :
   paramValueStr = HKTM.GetValeurChaine ();
   HKTMType = HKTM.GetType();

   //@ Encodage des status stocke sous forme d'entier
   if ( (HKTMType == MP__HKTMStatus::READINESS)   ||
        (HKTMType == MP__HKTMStatus::QUALITY)     ||
        (HKTMType == MP__HKTMStatus::START_ORBIT) ||
        (HKTMType == MP__HKTMStatus::CPU_USAGE)   ||
        (HKTMType == MP__HKTMStatus::PROCESSING_STATE) ) /* CR292 */
   {
     int paramValue = atoi(paramValueStr.c_str());
     
     (void) mlaEncodInt (encodedValue, paramValue);
   }
   //@ Encodage sous forme de string
   else 
   {
     (void) mlaEncodStr (encodedValue, (char*)(paramValueStr.c_str()));
   }

   //@ Renseigner la valeur du parametre
   strncpy (HKTMparam.valueString, encodedValue, PAR_STR_LEN);
//FA159
   HKTMparam.valueString [PAR_STR_LEN-1] ='\0';

#ifdef DEBUG
   cout << "Status value : "<< HKTMparam.valueString <<endl;
#endif       

   //@ Renseigner la validite
   HKTMparam.validity = mla_bool(HKTM.GetValidity());
   
   status = m_PGEServices->MLASendFunctionStatus (&timeEvt, &HKTMparam);

  return (status);
}


//@ ------------------------------------------------------------------*/
//@ Methode : MLASendEvent
//@
//@ Description : This method call the appropriate object in order to
//@               send a log or a trace event message.
//@
//@ Parametres :
//@               (ENTREE) message : Message a envoyer
//@
//@ Valeur de retour :
//@               t_CMN__Statut : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__PGEServices::MLASendEvent (JDB__Message &message)
{
  t_CMN__PGEStatus status;
  double     temps;
  eventData  event;
  logData    trace;
  t_Chaine   leLibelle;
  t_Chaine   msgName;
  // FT-2765
  struct timeval    dateEvt;
  // ...FT-2765
  timeValue  timeEvt;
  
  //@ Recuperer la date du message
  t_Date date = message.GetDate();
  date.LaDateCourantePrecise(3, dateEvt);
  timeEvt.sec = dateEvt.tv_sec;
  timeEvt.usec = dateEvt.tv_usec;

  //@ Recuperer le libelle
  leLibelle = message.GetLibelle ();

  //@ Construire le message Name
  msgName = CMN__Chaine(CMN_OPS_FACILITY_NAME) + 
    CMN__Chaine(CMN_OPS_LOCATION);   

  //@ SI le message est de type Log ALORS
  if (message.GetEventType() == JDB__Message::EVENT)
    {
      //@ Completer le nom du message
      JDB__Message::e_Criticite msgSeverity = message.GetCriticite();
      
      msgName = msgName + 
	CMN_EVENT_SEVERITY[msgSeverity] +
	message.GetFonction() +
	CMN_CGS_SERVICES [m_Spacecraft] +
	message.GetMsgId();
      
      strncpy (event.eventName, msgName.c_str(), EVT_NAM_LEN);
//FA159
      event.eventName [EVT_NAM_LEN-1] ='\0';
      
      //@ Construire le message a l'aide des informations recus
      strncpy (event.eventText, leLibelle.c_str(), EVT_TXT_LEN);
      //FA159
      event.eventText [EVT_TXT_LEN-1] ='\0';

      //@ Envoyer le message de log
      status = m_PGEServices->MLALogEvent (&timeEvt, &event);
    }
  //@ SINON (type TRACE)
  else 
    {
      //@ Completer le nom du message
      msgName = msgName + 
	CMN_EVENT_SEVERITY[JDB__Message::INFORMATION] +
	message.GetFonction() +
	CMN_CGS_SERVICES [m_Spacecraft] +
	message.GetMsgId();
      
      strncpy (trace.traceName, msgName.c_str(), TRA_NAM_LEN);
      //FA159
      trace.traceName [TRA_NAM_LEN-1] ='\0';
      
      //@ Construire le message a l'aide des informations recus
      strncpy (trace.traceText, leLibelle.c_str(), TRA_TXT_LEN);
      //FA159
      trace.traceText [TRA_TXT_LEN-1] ='\0';
      
      //@ Envoyer le message de trace
      status = m_PGEServices->MLALogTrace (&timeEvt, &trace);
    }
  //@ FIN SI
  
  return (status);
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetCommand
//@
//@ Description : This method build a command provided as parameter
//@               in a local liste
//@ Parametres :
//@               (ENTREE) uneCmde : une commande recue
//@
//@ Valeur de retour :
//@               t_CMN__Statut : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__PGEServices::GetCommand (char* uneCmde,
                                              MP__Command& command)
{
  t_CMN__Statut status;
  t_Chaine  laCommande;
  int       nbMot;
  int       iType=0;
  t_Chaine  cmdType;
  t_Chaine  cmdId;
  t_Chaine  cmdWOFilename;

  status = CMN__STATUT_OK;

  //@ Stockage de la chaine extraite en cas d'anomalie
  laCommande = uneCmde;

  //@ Controler la validite du nbre de mot de la commande
  nbMot = laCommande.NombreDeMot (" ");

#ifdef DEBUG
  cout << "la commande : /"<< laCommande<<"/"<<endl;
  cout << "nb mot : " << nbMot <<endl;
#endif       

  if ( (nbMot < 2) || (nbMot > 3) ) {
     status = PGE_INVALID_NB_MOT_CMD;
  }

  //@ Si les controles sont corrects
  if (status == CMN__STATUT_OK) 
    {
      //@ Extraire le premier mot
      laCommande.GetFirstMot(" ", cmdType);

#ifdef DEBUG
      cout << "Type commande : " << cmdType <<endl;
#endif       
      //@ Rechercher le type de la commande passee en parametre
      t_CMN__Booleen typeFound = CMN__FAUX;

      
      while (iType<=CMN__CmdTM::NbrCommandType && typeFound==CMN__FAUX)
	{
          if (cmdType == CMN_PGF_CMD_TYPE [iType]) {
	     typeFound=CMN__VRAI;
	  }
	  else
	    {iType++;}		
	}

      //@ Si le type a ete trouve
      if (typeFound != CMN__FAUX) {
          // Extraction du numero de la commande
          laCommande.GetNextMot(" ", cmdId);
#ifdef DEBUG
      cout << " Cmd Id : " << cmdId <<endl;
#endif 
      
	  //@ S'il y a 3 parametres et que la commande est STEP Alors
          if (iType == CMN__CmdTM::STEP) {
	    if (nbMot==3) {
	     //@ Extraire le mot du Work Order
             laCommande.GetNextMot(" ", cmdWOFilename);
#ifdef DEBUG
      cout << "Work Order : " << cmdWOFilename<<endl;
#endif   
	    }
	    else {
	        status = PGE_INVALID_NB_MOT_CMD;
             // Fin SI 3 parametres
	    }
	  }
	  else {
             if (nbMot==3) {
	        status = PGE_INVALID_NB_MOT_CMD;
	     }
	  }
	// Fin SI Type trouve
	}
        
      else {
	status = PGE_UNKNOWN_CMD_TYPE;
      }

    }
  
  //@ Si les controles sont corrects
  if (status == CMN__STATUT_OK)
    {
      //@ Construire une commande 
      command.SetCommandGroup (CMN__Cmd::CMD);
      command.SetCommandId (atoi(cmdId.c_str()));  
      command.SetCommandType ((CMN__CmdTM::e_CMN__CommandType)iType);
      command.SetCommandStatus (CMN__CmdTM::CODED);
      command.SetWOFilename (cmdWOFilename);

    }
   return (status);
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetFirstCommand
//@
//@ Description : This method provides the first command in the list.
//@
//@ Parametres :
//@               (SORTIE) command : Commande to process
//@               (SORTIE) stdinString : stdin extracted string
//@
//@ Valeur de retour :
//@               t_CMN__Statut : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__PGEServices::GetFirstCommand (MP__Command& command, 
                                                   t_Chaine&    stdinString)
{
  t_CMN__Statut status;
  char*    ptrCmd;
  char     uneCmde [MAX_LINE_SIZE];

/* FA 184*/
   m_lasts = NULL;
  //@ Extraire le 1ier message de commande recupere sur le stdin
  ptrCmd = strtok_r (m_cmdSTDIN, "\n",&m_lasts);
/*.. FA 184*/

  //@ SI une commande a ete recupee sur le stdin
  if (ptrCmd != NULL) {
     status = CMN__STATUT_OK;

     //@ Construire la commande a traiter
     strcpy (uneCmde, ptrCmd);
     uneCmde[strlen(uneCmde)] ='\0';

     stdinString = uneCmde;
     status = GetCommand (uneCmde, command);
  }
  else {
     status = PGE_NO_CMD;
  }

   return (status);
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetNextCommand
//@
//@ Description : This method provides the next command in the list.
//@
//@ Parametres :
//@               (SORTIE) command : Command to process
//@               (SORTIE) stdinString : stdin extracted string
//@
//@ Valeur de retour :
//@               t_CMN__Statut : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__PGEServices::GetNextCommand (MP__Command& command, 
                                                  t_Chaine&    stdinString)
{
  t_CMN__Statut status;
  char*    ptrCmd;
  char     uneCmde [MAX_LINE_SIZE];

/* FA 184*/
  //@ Extraire le  message suivant de commande recupere sur le stdin
  ptrCmd = strtok_r (NULL, "\n",&m_lasts);
/* ..FA 184*/

  //@ SI une commande a ete recupee sur le stdin
  if (ptrCmd != NULL) {
     status = CMN__STATUT_OK;

     //@ Construire la commande a traiter
     strcpy (uneCmde, ptrCmd);
     uneCmde[strlen(uneCmde)] ='\0';

     stdinString = uneCmde;
     status = GetCommand (uneCmde, command);
  }
  else {
     status = PGE_NO_CMD;
  }

   return (status);
}


//@ ------------------------------------------------------------------*/
//@ Methode : ExtractPGFCommand
//@
//@ Description : This method call the appropriate object in order to
//@               retreive a new command.
//@
//@ Parametres :
//@               (SORTIE) stdinString : stdin extracted string
//@
//@ Valeur de retour :
//@               t_CMN__Statut : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__PGEServices::ExtractPGFCommand (t_Chaine&   stdinString)
{
  t_CMN__Statut status;
  bool     cmdATraiter;

  //@ Initialisation de la chaine de caracteres utilisee pour lire le STDIN
  m_cmdSTDIN [0]='\0';

  //@ Recuperation de la commande presente sur le stdin
  status= m_PGEServices->GetPGFCommand (m_cmdSTDIN, &cmdATraiter);
  
  //@ Stockage de la chaine extraite en cas d'anomalie
  stdinString = m_cmdSTDIN;

  //@ S'il n'y a pas de  commande sur le stdin
  if (cmdATraiter == false) {
    status = PGE_NO_CMD;
  }
  // FIN SI commande a traiter
  
   return (status);
}

//@ ------------------------------------------------------------------*/
//@ Methode : SendPGFCommandTM
//@
//@ Description : This method call the appropriate method in order to
//@               send a command TM  message.
//@
//@ Parametres :
//@               (ENTREE) commandTM : Message commandTM
//@
//@ Valeur de retour :
//@               t_CMN__Statut : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__PGEServices::SendPGFCommandTM (MP__Command& commandTM)
{
  t_CMN__Statut status;
  t_Chaine cmdTMStr;
  
  
  //@ Construire le message Command TM a envoyer
  cmdTMStr = commandTM.GetCommandGroupStr ()   + " " + 
    commandTM.GetCommandTypeStr ()             + " " + 
    CMN__Chaine(commandTM.GetCommandId())      + " " +
    CMN__Chaine(commandTM.GetCommandStatus ());
 

  //@ Envoyer le message commandTM au PGF
  status = m_PGEServices->SendPGFCommandTM (cmdTMStr.c_str());
  
  return (status);
}


//@ ------------------------------------------------------------------*/
//@ Methode : SendPGFStageTM
//@
//@ Description : This method call the appropriate method in order to
//@               send a stage TM  message.
//@
//@ Parametres :
//@               (ENTREE) stageTM : Message stageTM
//@
//@ Valeur de retour :
//@               t_CMN__Statut : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut CMN__PGEServices::SendPGFStageTM (MP__StageTM& stageTM)
{
  t_CMN__Statut status;
  t_Chaine stageStr;
  
  //@ Construire le message Stage a envoyer
  stageStr = stageTM.GetCommandGroupStr ()            + " " + 
    stageTM.GetStageIdStr ()                          + " " +
    CMN__Chaine(stageTM.GetCommandId())               + " " +
    CMN_PGF_STAGE_FILE [stageTM.GetStageCategory ()]  + " " +
    stageTM.GetDeliveredFilename ();
  
#ifdef DEBUG
  cout << "Stage => delivered file : " <<stageTM.GetDeliveredFilename ()<<endl;
#endif

  //@ Envoyer le message StageTM au PGF
  status = m_PGEServices->SendPGFStageTM (stageStr.c_str());

  return (status);
}

//======================================================================
//	   		METHODES PRIVEES
//======================================================================
