// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/CMN__PGEServices.h,v 1.3 2011/11/25 10:17:39 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : CMN
//@ CLASSE : CMN__PGEServices
//@
//@ Commentaire :
//@    Cette librairie encapsule l'acces aux sous-systemes en interface
//@    avec l'OPS : MCS et PGF via des librairies (MLA) ou des 
//@    developpements specifiques.
//@    Cette solution permet d'implementer le fonctionnement en
//@    standalone.
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 22/08/2002 : PBR
//@ FA-ID : OPS-I00-010  18/11/2002 :
//@ Non reception de commandes envoyees quasi-systematiquement
//@ Creation des methodes : ExtractPGFCommand, GetFirstCommand,
//@ GetNextCommand dans CMN__PGEServices.
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : PBR
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-184-CN : 18/10/2006 : C.MERIGUET
//@ Codage des fonctions reentrantes
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------


#ifndef CMN__PGEServices_h
#define CMN__PGEServices_h
//
//  Include Files
//
#include "CMN__CGSCst.h"
#include "JDB__Message.h"
#include "CMN__PGEServicesAbstract.h"

//
// EXPORTED TYPES & DATA
//
#define OPS_INSTAL_CGS "CGS"
#define OPS_INSTAL_STANDALONE "STANDALONE"

static const int OPS_PGE_PRECISION_DATE=1000;
static const t_Chaine CMN_PGE_SYMPTOME [] ={"OK", "NO_CMD","ARGUMENT NUMBER", "UNKNOWN COMMAND TYPE"};

// Installation Type values used in the OPS environment file
class CMN__PGEServicesAbstract;
class MP__HKTMStatus;
class MP__Command;
class MP__StageTM;

//
// CLASS DEFINITION
//
class CMN__PGEServices {

/*====================================================================*/
public:


    // Type definitions
    typedef enum {
        INSTAL_CGS =0,
	INSTAL_STANDALONE=1
	}e_InstalType;

    typedef enum {
         // CMN__STATUT_OK = 0
         PGE_NO_CMD=1,
         PGE_INVALID_NB_MOT_CMD=2,
	 PGE_UNKNOWN_CMD_TYPE=3
	}e_PGE_Err; 

   typedef vector<MP__Command> t_ListeCmd;

// Public Methods
    // Constructeur
    CMN__PGEServices(e_CMN__Processus processUtilisateur=PROCESS_NONE);
	
    // Destructeur
    virtual ~CMN__PGEServices();

    // Manage MLA Interface 
    t_CMN__Statut MLAInitialisation ();
    t_CMN__Statut MLATerminate ();

    // Manage HKTM status
    t_CMN__Statut MLASendFunctionStatus (MP__HKTMStatus & HKTM);

    //Manage events : log and trace
    void InitialiseSpacecraft (e_CMN_CGSSpacecraft spacecraft)
                              {m_Spacecraft = spacecraft;};
    t_CMN__Statut MLASendEvent (JDB__Message &message);


    // Extracting PGF command
    t_CMN__Statut ExtractPGFCommand (t_Chaine&  stdinString);
    t_CMN__Statut GetFirstCommand (MP__Command& command, t_Chaine&  stdinString);
    t_CMN__Statut GetNextCommand  (MP__Command& command, t_Chaine&  stdinString);

    // Sending command TM to PGF
    t_CMN__Statut SendPGFCommandTM (MP__Command& commandTM);

    // Sending Stage TM to PGF
    t_CMN__Statut SendPGFStageTM (MP__StageTM& stageTM);

/*====================================================================*/
protected:


/*====================================================================*/
private:


        // Store an stdin extracted command 
        t_CMN__Statut GetCommand (char* uneCmde, MP__Command& command);


	// Installation type which is used to indicate 
	e_InstalType m_InstalType;

        // Process Type : main process or other
        e_CMN__Processus m_process;


	// Gestionnaire des commandes du PGF
	CMN__PGEServicesAbstract *m_PGEServices;

   
        // Spacecraft duquel les donnees traitees par l'OPS proviennent
        e_CMN_CGSSpacecraft m_Spacecraft;

        // Chaine lue sur le stdin
        char  m_cmdSTDIN[MAX_LINE_SIZE];

// FA 184
        char * m_lasts;
//.. FA 184

};

#endif
