static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/CMN__PGEServicesAPI.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : CMN
//@ CLASSE : CMN__PGEServicesAPI
//@
//@ Commentaire :
//@    Cette classe implemente dans le cas du fonctionnement en
//@    STANDALONE l'acces aux API du MCS et PGF
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : PBR
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 18/11/2002 : PBR
//@ FA-ID : OPS-I00-010 : 18/11/2002 :
//@ Non reception de commandes envoyees quasi-systematiquement
//@ Suppression de la commande select.
//@ --------------------------------------------------------------------
//@ VERSION : 03/12/2002 : PBR
//@ FA-ID : OPS-I01-014 : 03/12/2002 :
//@ MLAInitialisation : dans le cas des PPF, appel de mla_Initialisation
//@ avec systematiquement le flag MP_process a false car la reception des
//@ commandes ne s'effectue pas par la methode GetCommande mais 
//@ directement sur le stdin.
//@ --------------------------------------------------------------------
//@ VERSION : 11/05/2004 : PBR
//@ Ajour de la commande select.sur le read pour le rendre non bloquant
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
//  Include Files
//
#include <string.h>

#include <strings.h>
#include <stdio.h>
#include <sys/select.h>
#include <errno.h>
// FT-2765
#include <unistd.h>
// ...FT-2765

#include "CMN__CGSCst.h"
#include "ERR__Exception.h"
#include "CMN__PGEServicesAPI.h"
#include "CMN__OPSProcess.h"


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CMN__PGEServicesAPI
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@                (ENTREE) processUtilisateur : processus qui instancie 
//@                                              un objet de cette classe
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__PGEServicesAPI::CMN__PGEServicesAPI(e_CMN__Processus processUtilisateur)
{

  //@ SI l'instance est creee par le Main Process Alors
  if (processUtilisateur == PROCESS_MP) {
     m_MPflag = mla_true;
  }
  //@ Sinon ne pas initialiser la reception sur le stdin
  else {
     m_MPflag = mla_false;
  }
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~CMN__PGEServicesAPI
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__PGEServicesAPI::~CMN__PGEServicesAPI()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : MLAInitialisation
//@
//@ Description : This method manages MLA library initialisation 
//@               function call
//@
//@ Parametres :
//@               (ENTREE) facilityName
//@               (ENTREE) process : process qui instancie l'objet
//@
//@ Returned Status :
//@               t_CMN__PGEStatus : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__PGEStatus CMN__PGEServicesAPI::MLAInitialisation 
                                                     (const char * facilityName)
{
  int status = MLA_OK;
  char name [CMN_OPS_FACILITY_LEN];

  strncpy (name, facilityName, CMN_OPS_FACILITY_LEN);
  name [CMN_OPS_FACILITY_LEN-1] ='\0';

#ifdef DEBUG
  cout << "mla_initialisation : "<<name<<" "<<m_MPflag<<endl;
#endif       

  status = mla_Initialisation (name, mla_false);

#ifdef DEBUG
  cout << "Status "<<status <<endl;
#endif       

  return (t_CMN__PGEStatus)status;
}


//@ ------------------------------------------------------------------*/
//@ Methode : MLATerminate
//@
//@ Description : This method manages MLA librairy terminaison
//@               function call
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               t_CMN__PGEStatus : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__PGEStatus CMN__PGEServicesAPI::MLATerminate ()
{
  int status = MLA_OK;
  
  status = mla_Terminate ();

  return (t_CMN__PGEStatus)status;
}


//@ ------------------------------------------------------------------*/
//@ Methode : MLASendFunctionStatus
//@
//@ Description : This method manages MLA snedFunction Status services
//@
//@ Parametres :
//@               (ENTREE) timeEvt : Temps OS courant
//@               (ENTREE) HKTMparam 
//@
//@ Valeur de retour :
//@               t_CMN__PGEStatus : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__PGEStatus CMN__PGEServicesAPI::MLASendFunctionStatus 
                                                         (timeValue* timeEvt,
                                                          tmParam*   HKTMparam)
{
   int status = MLA_OK;

   status = mla_SendFunctionStatus (timeEvt, HKTMparam);

   return (t_CMN__PGEStatus)status;
}

//@ ------------------------------------------------------------------*/
//@ Methode : MLALogEvent
//@
//@ Description : This method manages MLA librairy log event function
//@               call
//@
//@ Parameters :
//@               (ENTREE) timeEvt : Temps OS courant
//@               (ENTREE) event : eventName + eventText
//@
//@ Valeur de retour :
//@               t_CMN__PGEStatus : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__PGEStatus CMN__PGEServicesAPI::MLALogEvent (timeValue* timeEvt,
                                                     eventData* event)
{
   int status = MLA_OK;
#ifdef DEBUG
   cout << "CMN__PGEServicesAPI::MLALogEvent: " << event->eventText << endl;
#endif 

   status = mla_LogEvent (timeEvt, event);

  return (t_CMN__PGEStatus)status;
}


//@ ------------------------------------------------------------------*/
//@ Methode : MLALogTrace
//@
//@ Description : This method manages MLA librairy log trace function
//@               call
//@
//@ Parameters :
//@               (ENTREE) timeTrace : Temps OS courant
//@               (ENTREE) trace : traceName + traceText
//@
//@ Valeur de retour :
//@               t_CMN__PGEStatus : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__PGEStatus CMN__PGEServicesAPI::MLALogTrace (timeValue* timeTrace,
                                                     logData* trace)
{
  int  status = MLA_OK;

  status = mla_LogTrace (timeTrace, trace);

  return (t_CMN__PGEStatus)status;
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetPGFCommand
//@
//@ Description : This method extracts the available command from the
//@               stdin queue
//@
//@ Parameters :
//@               (SORTIE) command : Commande recuperee
//@               (SORTIE) validCmd
//@
//@ Valeur de retour :
//@               t_CMN__PGEStatus : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__PGEStatus CMN__PGEServicesAPI::GetPGFCommand (char* command, bool* validCmd)
{
 ERR__Exception exception;   
  int retour;
  int numberOfReadCharacters; 
  t_Chaine entreeStr;

  *validCmd=true;

  // @ Detecter l'arrivee de caracteres sur le STDIN
  
  // Initialiser le timeout d'attente de reception de caracteres sur le stdin
  timeval tv;
  tv.tv_sec = 1;
  tv.tv_usec = 0;
    
  // Initialiser le masque de lecture
  fd_set readList;
  FD_ZERO(&readList);
  FD_SET (0, &readList);
  
  //@ Appel de select
  retour = select(1, &readList, NULL, NULL, &tv);
  
  //@ Si un caracteres a ete recu sur le STDIN
  if (retour == 1) { 
     //@ On lit  la chaine de caracteres sur le stdin
     numberOfReadCharacters = read(STDIN_FILENO,command,MAX_LINE_SIZE);

     if (numberOfReadCharacters > 0){
        command[numberOfReadCharacters] = '\0';
       }	
    else *validCmd=false;
    }
  else *validCmd=false;  

  return (t_CMN__PGEStatus)CMN__STATUT_OK;
}


//@ ------------------------------------------------------------------*/
//@ Methode : SendPGFCommandTM
//@
//@ Description : This method provides the  command TM to the PGF 
//@               from the stdout queue
//@
//@ Parametres :
//@               (ENTREE) commandTM : Command TM
//@
//@ Valeur de retour :
//@               t_CMN__PGEStatus : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__PGEStatus CMN__PGEServicesAPI::SendPGFCommandTM (const char* commandTM)
{

  //@ Envoi de la commande TM
  cout << commandTM << endl;

  return (t_CMN__PGEStatus)CMN__STATUT_OK;
}

//@ ------------------------------------------------------------------*/
//@ Methode : SendPGFStageTM
//@
//@ Description : This method provides the stage TM to the PGF from
//@               the stdout queue
//@
//@ Parametres :
//@               (ENTREE) stageTM : StageTM
//@
//@ Valeur de retour :
//@               t_CMN__PGEStatus : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__PGEStatus CMN__PGEServicesAPI::SendPGFStageTM (const char* stageTM)
{
  //@ Envoi de la stage TM
  cout << stageTM << endl;

  return (t_CMN__PGEStatus)CMN__STATUT_OK;
}
