static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/CMN__PGEServicesStub.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : CMN
//@ CLASSE : CMN__PGEServicesStub
//@
//@ Commentaire :
//@    Cette classe implemente dans le cas du fonctionnement en 
//@    STANDALONE l'acces aux API du MCS et PGF
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : PBR
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 22/08/2002 : PBR
//@ FA-ID : OPS-I00-010 : 18/11/2002 :
//@ Non reception de commandes envoyees quasi-systematiquement
//@ Suppression de la commande select.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-120-CN : 4/02/2004 : JL.PASCAL
//@ modification du message d'erreur en cas de non creation du pipe 
//@ d'entree pour les commandes
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
//  Include Files
//
#include <iostream>
#include <unistd.h>
#include <strings.h>
#include <sys/select.h>
#include <errno.h>
#include <fcntl.h>

#include <stdlib.h>

#include "CMN__PGEServicesStub.h"
#include "CMN__Trace.h"
#include "CMN__Date.h"
#include "CFG__Configuration.h"
#include "CMN__Fichier.h"
#include "ERR__Exception.h"

//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CMN__PGEServicesStub
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@                (ENTREE) processUtilisateur : processus qui instancie 
//@                                              un objet de cette classe
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__PGEServicesStub::CMN__PGEServicesStub(e_CMN__Processus processUtilisateur)
{
  t_Chaine nomFichier;
  CMN__Date date;
  t_Chaine dateText;
  ofstream fichier;

  //@ Recuperer la date courante pour concatener avec les noms de
  //@ fichier
  date.DateCourante ();
  dateText = date.Formater ("%Y%m%d_%H%M%S", 2);

  //@ SI l'instance est creee par le JDBS Alors
  if (processUtilisateur == PROCESS_JDBS) {

     //@ Ouvrir les fichiers dedies stockage :
     //@  - des log events
     nomFichier = CFG__Configuration::GetChaine("STANDALONE_LOG_FILE");
     nomFichier = nomFichier + dateText;
     m_LogFile = new ofstream (nomFichier.c_str());

     //@  - des traces
     nomFichier = CFG__Configuration::GetChaine("STANDALONE_TRACE_FILE");
     nomFichier = nomFichier + dateText;
     m_TraceFile = new ofstream (nomFichier.c_str());

  }


  //@ SI l'instance est creee par le Main Process Alors
  if (processUtilisateur == PROCESS_MP) {
     //@  - des cmdTM et stage TM
     nomFichier = CFG__Configuration::GetChaine("STANDALONE_CMD_TM_FILE");
     nomFichier = nomFichier + dateText;
     m_CmdFile = new ofstream (nomFichier.c_str());

     //@  - des HKTM status
     nomFichier = CFG__Configuration::GetChaine("STANDALONE_HKTM_FILE");
     nomFichier = nomFichier + dateText;
     m_HKTMFile = new ofstream (nomFichier.c_str());

    // Avec le debugger il n'est pas possible de recuperer le STDIN
    // Ouverture d'un pipe
    static int fromPGF;
    t_Chaine commandePipe;
    commandePipe = CFG__Configuration::GetChaine("STDIN_IN_DEBUG");

    fromPGF=open (commandePipe.c_str(), O_NDELAY);

    cout << "Ouverture du pipe d'entree : " << commandePipe.c_str()<<endl;
    if (fcntl(fromPGF, F_SETFL, O_NDELAY) < 0){
      cerr <<"Could not create input pipe for command in standalone configuration"<<endl;
     }
    // Redirection du pipe dans le STDIN
    dup2 (fromPGF, STDIN_FILENO);

  }
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~CMN__PGEServicesStub
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__PGEServicesStub::~CMN__PGEServicesStub()
{
  //@ Fermer les fichiers dedies stockage :
  //@  - des log events
  m_LogFile->close();

  //@  - des traces
  m_TraceFile->close();
  
  //@  - des cmdTM et stage TM
  m_CmdFile->close();
  
  //@  - des HKTM status
  m_HKTMFile->close();

#ifdef DEBUG
    // Fermeture du pipe
    close(STDIN_FILENO);
#endif
}


//@ ------------------------------------------------------------------*/
//@ Methode : MLAInitialisation
//@
//@ Description : This method manages MLA librairy initialisation
//@               function call
//@
//@ Parametres :
//@               (ENTREE) facilityName
//@               (ENTREE) process : process qui instancie l'objet
//@
//@ Valeur de retour :
//@               t_CMN__PGEStatus : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__PGEStatus CMN__PGEServicesStub::MLAInitialisation 
                                                  (const char * facilityName)
{
  // Dans le cas du Stub de mla_Initialise => Ne rien faire
  return (t_CMN__PGEStatus)MLA_OK;
}


//@ ------------------------------------------------------------------*/
//@ Methode : MLATerminate
//@
//@ Description : This method manages MLA librairy terminaison function
//@               call
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               t_CMN__PGEStatus : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__PGEStatus CMN__PGEServicesStub::MLATerminate ()
{
  // Dans le cas du Stub de mla_Terminate => faire un exit
  exit (0);
  
  return (t_CMN__PGEStatus)MLA_OK;
}


//@ ------------------------------------------------------------------*/
//@ Methode : MLASendFunctionStatus
//@
//@ Description : This method manages MLA snedFunction Status services
//@
//@ Parametres :
//@               (ENTREE) timeEvt : Temps OS courant
//@               (ENTREE) HKTMparam 
//@
//@ Valeur de retour :
//@               t_CMN__PGEStatus : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__PGEStatus CMN__PGEServicesStub::MLASendFunctionStatus 
                                                         (timeValue* timeEvt,
                                                          tmParam*   HKTMparam)
{
  int status = MLA_OK;
  double dateSec;
  CMN__Date dateEvt;
  t_Chaine dateStr;


  //@ Formattage de la date en string
  dateSec = (double)timeEvt->sec + (((double)timeEvt->usec)/10);
  dateEvt.ConvertirTempsEnDate(dateSec);
  dateStr = dateEvt.Formater("%Y%m%d_%H%M%S", 2);
  
   
  //@ Ecrire le statut dans le fichier trace dedie
  (*m_HKTMFile)<< dateStr          << " : "
               << HKTMparam->paramName << " => "
               << HKTMparam->valueString << endl;
  
  return (t_CMN__PGEStatus)status;
}


//@ ------------------------------------------------------------------*/
//@ Methode : MLALogEvent
//@
//@ Description : This method manages MLA librairy log event function
//@               call
//@
//@ Parametres :
//@               (ENTREE) timeEvt : Temps OS courant
//@               (ENTREE) event : eventName + eventText
//@
//@ Valeur de retour :
//@               t_CMN__PGEStatus : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__PGEStatus CMN__PGEServicesStub::MLALogEvent (timeValue* timeEvt,
                                                    eventData* event)
{
  int status = MLA_OK;
  double dateSec;
  CMN__Date dateEvt;
  t_Chaine dateStr;


  //@ Formattage de la date en string
  dateSec = (double)timeEvt->sec + (((double)timeEvt->usec)/10);
  dateEvt.ConvertirTempsEnDate(dateSec);
  dateStr = dateEvt.Formater("%Y%m%d_%H%M%S", 2);

   
  //@ Ecrire le status dans le fichier trace dediee
  (*m_LogFile)<< dateStr          << " : "
	      << event->eventName << " => "
	      << event->eventText << endl;
  
  return (t_CMN__PGEStatus)status;
}


//@ ------------------------------------------------------------------*/
//@ Methode : MLALogTrace
//@
//@ Description : This method manages MLA librairy log trace function
//@               call
//@
//@ Parametres :
//@               (ENTREE) timeTrace : Temps OS courant
//@               (ENTREE) trace : traceName + traceText
//@
//@ Valeur de retour :
//@               t_CMN__PGEStatus : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__PGEStatus CMN__PGEServicesStub::MLALogTrace (timeValue* timeTrace,
                                                   logData* trace)
{
  int status = MLA_OK;
  double dateSec;
  CMN__Date dateEvt;
  t_Chaine dateStr;


  //@ Formattage de la date en string
  dateSec = (double)timeTrace->sec + (((double)timeTrace->usec)/10);
  dateEvt.ConvertirTempsEnDate(dateSec);
  dateStr = dateEvt.Formater("%Y%m%d_%H%M%S", 2);
   
  //@ Ecrire le status dans le fichier trace dediee
  (*m_TraceFile)<< dateStr          << " : "
		<< trace->traceName << " => "
		<< trace->traceText << endl;
  
  return (t_CMN__PGEStatus)status;
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetPGFCommand
//@
//@ Description : This method extracts the available command from the
//@               stdin queue
//@
//@ Parameters :
//@               (SORTIE) command : Commande recuperee
//@               (SORTIE) validCmd
//@
//@ Valeur de retour :
//@               t_CMN__PGEStatus : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__PGEStatus CMN__PGEServicesStub::GetPGFCommand (char* command, bool* validCmd)
{
 
 ERR__Exception exception;   
  int retour;
  int numberOfReadCharacters=0; 
  t_Chaine entreeStr;

  *validCmd=true;
 
  //@ On lit  la chaine de caracteres sur le stdin
  numberOfReadCharacters = read(STDIN_FILENO,command,MAX_LINE_SIZE);
  if (numberOfReadCharacters > 0){
     command[numberOfReadCharacters] = '\0';
  }	
  else {
   *validCmd=false;
  }

  return (t_CMN__PGEStatus)CMN__STATUT_OK;
}


//@ ------------------------------------------------------------------*/
//@ Methode : SendPGFCommandTM
//@
//@ Description : This method provides the command TM to the PGF
//@               from the stdout queue
//@
//@ Parametres :
//@               (ENTREE) commandTM : Command TM
//@
//@ Valeur de retour :
//@               t_CMN__PGEStatus : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__PGEStatus CMN__PGEServicesStub::SendPGFCommandTM (const char* commandTM)
{
  CMN__Date date;
  t_Chaine dateText;
  
  //@ Formattage de la date courante en string
  date.DateCourante ();
  dateText = date.Formater ("%Y%m%d_%H%M%S",2);
   
  //@ Ecrire le statut dans le fichier trace dedie
  (*m_CmdFile)<< dateText          << " : "
	      << commandTM << endl;
  cout << commandTM << endl; 

  return (t_CMN__PGEStatus)CMN__STATUT_OK;
}


//@ ------------------------------------------------------------------*/
//@ Methode : SendPGFStageTM
//@
//@ Description : This method provides the stage TM to the PGF from
//@               the stdout queue
//@
//@ Parametres :
//@               (ENTREE) stageTM : StageTM
//@
//@ Valeur de retour :
//@               t_CMN__PGEStatus : Statut d'execution
//@
//@ ------------------------------------------------------------------*/
t_CMN__PGEStatus CMN__PGEServicesStub::SendPGFStageTM (const char* stageTM)
{
  CMN__Date date;
  t_Chaine dateText;
  
  
  //@ Formattage de la date courante en string
  date.DateCourante ();
  dateText = date.Formater ("%Y%m%d_%H%M%S",2);
   
  //@ Ecrire le statut dans le fichier trace dedie
  (*m_CmdFile)<< dateText          << " : "
	      << stageTM << endl;
  cout << stageTM << endl;

  return (t_CMN__PGEStatus)CMN__STATUT_OK;
}
