static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/CMN__ResourceMonitor.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : CMN
//@ CLASSE : CMN__ResourceMonitor
//@
//@ Commentaire :
//@    Cette librairie fournit les services pour calculer la 
//@    consommation CPU instantanee en % d'un processus.
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 03/03/2003 : P. BRANET
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//
#include <cmath>
#include "CMN__Commun.h"
#include "CMN__ResourceMonitor.h"

//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================
//@ ------------------------------------------------------------------*/
//@ Methode : CMN__ResourceMonitor
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__ResourceMonitor::CMN__ResourceMonitor()
{
  // Initialisation de la timezone
  Z.tz_minuteswest = 0;
  Z.tz_dsttime = 0;

  // Mesures des valeurs
  this->GetResourceValue();

  // Memorisation des valeurs initiales
  userT0 = userT1;
  elapsedT0 = elapsedT1;
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetResourceValue
//@
//@ Description : Mesure des valeurs systemes pour le calcul de la
//@               consommation CPU.
//@ Parametres :
//@               Aucun
//@ Valeur de retour :
//@               Aucune
//@ ------------------------------------------------------------------*/
void CMN__ResourceMonitor::GetResourceValue()

{
  rusage R;
  timeval dt;

  // Sauvegarde de la derniere mesure
  userT0 = userT1;
  elapsedT0 = elapsedT1;

  // Mesure de la consommation CPU instantanee
  getrusage(RUSAGE_SELF, &R);
  userT1 =  R.ru_utime.tv_sec  +  1e-6 * R.ru_utime.tv_usec ;

  gettimeofday(&dt,&Z);
  elapsedT1 =  dt.tv_sec  +  1e-6 * dt.tv_usec ;
}


//@ ------------------------------------------------------------------*/
//@ Methode : instantaneousCPU
//@
//@ Description : Calcul de la consommation CPU instantanee a partir 
//@               des mesures faites lors des 2 derniers appels a 
//@               GetResourceValue.
//@ Parametres :
//@               Aucun
//@ Valeur de retour :
//@               pourcentage de consommation CPU (entre 0 et 100)
//@ ------------------------------------------------------------------*/
int CMN__ResourceMonitor::instantaneousCPU()

{
  double elapsed = 0.;

  elapsed = elapsedT1 - elapsedT0;

  // cas ou elapsedT1 concerne le jour suivant de elapsedT0 
  if (elapsed < 0)  
    { 
      elapsed += 86400;
    }

  // consommation CPU en pourcentage 
#ifdef DEBUG
  cout << "CMN__ResourceMonitor::instantaneousCPU(): " << endl;
  cout << "\telapsed time: " << elapsed << endl;
  cout << "\tdifference  : " << (userT1 - userT0) << endl;
  cout << "\tdivision    : " << (100 * (userT1 - userT0) / elapsed)
       << endl;
#endif

  // FT-2765
  return   static_cast<int>(floor ( 100 * (userT1 - userT0) / elapsed  )) ;
  // ...FT-2765
}
