// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/CMN__Trace.h,v 1.3 2011/11/25 10:17:39 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : CMN
//@ MODULE : CMN__Trace
//@
//@ Commentaire :
//@    Fichier d'entete du module CMN__Trace
//@ 	definie les variables et les procedures exportees pour
//@	pour le traitements des traces:
//@		-Initialisation des traces
//@		-procedures de traces
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ Date de creation : 10/03/00
//@ Auteur           : Pierre-Louis Agnetti
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif
#ifndef CMN__Trace_h
#define CMN__Trace_h

//====================================================================*/
// Declaration des define externes
//====================================================================*/

// macros actives en developpement et tests
#ifdef DEBUG

#include <stdlib.h>
#include <stdarg.h>

// Initialisation des options des traces
// niveau de trace, fichier de trace, circulaire ou pas, nombre de lignes en circulaire
#define TRINIT(argc,argv) \
{ \
   CMN__Trace_Init(argc,argv); \
}

// Fin de la gestion des traces
// fermeture du fichier
#define TRCLOSE() \
{ \
   CMN__Trace_Fin(); \
}

// Trace sans parametre autre que la chaine de caractere
// le niveau indique a partir de quel momment cette trace apparait
#define TR0(niveau,chaine) \
{ \
   if (CMN__Trace_NiveauTrace>=niveau) { \
      CMN__Trace_Tracer(__FILE__,__LINE__,chaine); \
   } \
}

// Trace avec un parametre et un format "printf like"
// le niveau indique a partir de quel momment cette trace apparait
#define TR1(niveau,format,p1) \
{ \
   if (CMN__Trace_NiveauTrace>=niveau) { \
      CMN__Trace_Tracer(__FILE__,__LINE__,format,p1,0); \
   } \
}

// Trace avec deux parametres et un format "printf like"
// le niveau indique a partir de quel momment cette trace apparait
#define TR2(niveau,format,p1,p2) \
{ \
   if (CMN__Trace_NiveauTrace>=niveau) { \
      CMN__Trace_Tracer(__FILE__,__LINE__,format,p1,p2,0); \
   } \
}

// Trace avec trois parametres et un format "printf like"
// le niveau indique a partir de quel momment cette trace apparait
#define TR3(niveau,format,p1,p2,p3) \
{ \
   if (CMN__Trace_NiveauTrace>=niveau) { \
      CMN__Trace_Tracer(__FILE__,__LINE__,format,p1,p2,p3,0); \
   } \
}

// Trace avec quatre parametres et un format "printf like"
// le niveau indique a partir de quel momment cette trace apparait
#define TR4(niveau,format,p1,p2,p3,p4) \
{ \
   if (CMN__Trace_NiveauTrace>=niveau) { \
      CMN__Trace_Tracer(__FILE__,__LINE__,format,p1,p2,p3,p4,0); \
   } \
}

// Trace la mention de debut d'une procedure
#define TRDEBUT(procedure) \
{ \
   TR1(3,"DEBUT PROCEDURE %s",procedure); \
}

// Trace la mention de fin d'une procedure
#define TRFIN(procedure) \
{ \
   TR1(3,"FIN PROCEDURE %s",procedure); \
}
// Trace la mention d'erreur d'une procedure
#define TRERR(num_erreur) \
{ \
   TR1(1,"ERREUR %d",num_erreur); \
}

//====================================================================*/
// Declaration des fonctions externes
//====================================================================*/

// Procedures utilisees par les macros TRINIT,TR0,TR1,TR2,TR3,TR4,TRCLOSE

//@ CMN__Trace_Init permet d'initaliser les options de traces
//void CMN__Trace_Init(int argc,char **argv);

//@ CMN__Trace_Tracer permet de tracer dans le fichier de traces
//void CMN__Trace_Tracer(const char *fichier_source,const int numero_ligne,const char *format,...);

//@ CMN__Trace_Fin permet de fermer la gestion des traces
//void CMN__Trace_Fin();

//@ CMN__Trace_malloc permet de tracer et effectuer un malloc
void *CMN__Trace_malloc(size_t size,const char *source,int ligne);

//@ CMN__Trace_realloc permet de tracer et effectuer un realloc
void *CMN__Trace_realloc(void *ptr,size_t size,const char *source,int ligne);

//@ CMN__Trace_free permet de tracer et effectuer un free
void CMN__Trace_free(void *ptr,const char *source,int ligne);

//@ CMN__Trace_Print permet d'ecrire un chaine de caracteres
//void CMN__Trace_Print(const char *buffer);
//====================================================================*/
// Declaration des variables externes
//====================================================================*/

// Variable indiquant le niveau de trace courant
extern int CMN__Trace_NiveauTrace;


#else
// macros inactives en recette et exploitation
#define TRINIT(argc,argv)  {}
#define TR0(niveau,chaine) {}
#define TR1(niveau,format,p1) {}
#define TR2(niveau,format,p1,p2) {}
#define TR3(niveau,format,p1,p2,p3) {}
#define TR4(niveau,format,p1,p2,p3,p4) {}
#define TRCLOSE()            {}
#define TRDEBUT(procedure) {}
#define TRERR(num_erreur) {}
#define TRFIN(procedure) {}

#endif

//@ CMN__Trace_Init permet d'initaliser les options de traces
void CMN__Trace_Init(int argc,char **argv);

//@ CMN__Trace_Tracer permet de tracer dans le fichier de traces
void CMN__Trace_Tracer(const char *fichier_source,const int numero_ligne,const char *format,...);

//@ CMN__Trace_Print permet d'ecrire un chaine de caracteres
void CMN__Trace_Print(const char *buffer);

//@ CMN__Trace_Fin permet de fermer la gestion des traces
void CMN__Trace_Fin();

#endif
#ifdef __cplusplus
}
#endif
