static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/CMN__Verrou.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : MSG
//@ CLASSE : CMN__Verrou
//@
//@ Commentaire :
//@    Ce fichier gere la classe CMN__Verrou
//@    Classe permettant d'avoir une gestion de lock pour des sequences
//@    critiques dans les threads par exemple.
//@   
//@    La gestion des locks est basee sur un semaphore avec au minimum
//@    deux elements a l'interieur:
//@    le dernier element dans le semaphore permet de maintenir un 
//@    compteur d'acces au semaphore.
//@    Le compteur est incremente sur le constructeur et decremente sur
//@    le destructeur.
//@    Lorsque le compteur passe a 0, le semaphore est detruit.
//@    
//@    exemple: P=process T=Thread
//@    P1/T1 >  CMN_Verrou verrou(1); (creation du semaphore avec la cle 1, cpt=1)
//@    P2/T2 >  CMN_Verrou verrou(1); (attachement au semaphore avec la cle 1, cpt=2)
//@    P2/T2 >  verrou.Bloque();
//@    P1/T1 >  verrou.Bloque();
//@    P2/T2 >  verrou.DeBloque();
//@    P1/T1 >  verrou.DeBloque();
//@    P3/T3 >  CMN_Verrou verrou(1); (attachement au semaphore avec la cle 1, cpt=3)
//@    P1/T1 >  delete/free auto verrou (cpt=2)
//@    P3/T3 >  verrou.Bloque();
//@    P3/T3 >  verrou.DeBloque();
//@    P3/T3 >  delete/free auto verrou (cpt=1)
//@    P2/T2 >  delete/free auto verrou (cpt=0 => destruction du semaphore)
//@
//@    NB: la classe permet de gerer un ensemble de verrou en jouant sur le
//@        nombre dans le constructeur et sur le numero sur Bloque/DeBloque
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 13/05/2003 : P.L. AGNETTI
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-144-CN : 21/09/2004 : B. MASSART
//@ Problemes lors du portage d'OPS vers Solaris par Meteo France
//@ Voir IA-2100-FC-144-THA.
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// Standard
#include <sys/sem.h> /* FA144 */

// Projet
#include "ERR__Exception.h"
#include "CMN__Chaine.h"
#include "CMN__Verrou.h"


/* FA144 */
// FT-2765
#ifdef AIX
union semun
{
  int val;
  struct semid_ds *buf;
  unsigned short  *array;
} arg_semun;
#else
union semun
#ifdef HASNT_UNION_SEMUN
{
  int val;
  struct semid_ds *buf;
  unsigned short  *array;
}
#endif
 arg_semun;
#endif
// ...FT-2765

//
// Definition des constantes 
//

//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CMN__Verrou
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Verrou::CMN__Verrou(key_t key,int nb_sems,int semval) 
{   
  ERR__Exception exception;
  CMN__Chaine  parameter;
  struct sembuf sem_ar;

  //@ Memorisation des parametres
  m_key=key;
  //le dernier semaphore utilise pour le comptage pour liberation au dernier utilisateur
  m_nb_sems=nb_sems+1;
  m_semflg=IPC_CREAT|IPC_EXCL|0600;
  m_semval=semval;
  m_semid=-1;
  
  //@ Creation du semaphore
  m_semid = semget(m_key,m_nb_sems,m_semflg);

  /* Save semaphore value */ {
    FILE *fp = fopen( "semaphores.txt", "a" );
    fprintf( fp, "0x%x\n", m_semid );
    fclose( fp );
  }

  //@ Si erreur de creation
  if ( m_semid == -1 ) {
	//@ Si erreur en creation, essayer en access
        //@ Acces du semaphore
        m_semflg=0600;

        m_semid = semget(m_key,m_nb_sems,m_semflg);

        if ( m_semid == -1 ) {
	   //@ Si erreur en acces aussi, tracer l'erreur et lever une exception	
	   exception.SetErrorCode (ERR__Exception::SEM_CREATE_ERROR);
	   THROW(exception);
        }
  }
  // creation sem ok
  else {
    //@ positionner la valeur des semaphores au depart
    for ( int i=0 ; i < m_nb_sems-1 ; i++ ) {
      /* FA144 */
      arg_semun.val = m_semval;
      if ( semctl(m_semid,i,SETVAL,arg_semun) == -1 ) {
	//@ Si erreur Tracer l'erreur et lever une exception	
	parameter = t_Chaine(m_semid);
	exception.SetParametre (parameter);
	parameter = t_Chaine(i);
	exception.SetParametre (parameter);
	exception.SetErrorCode (ERR__Exception::SEM_SET_ERROR);
      }
    }
    //@ positionner la valeur du semaphore compteur a 0
    /* FA144 */
    arg_semun.val = 0;
    if ( semctl(m_semid,m_nb_sems-1,SETVAL,arg_semun) == -1 ) {
	//@ Si erreur Tracer l'erreur et lever une exception	
	parameter = t_Chaine(m_semid);
	exception.SetParametre (parameter);
	parameter = t_Chaine(m_nb_sems-1);
	exception.SetParametre (parameter);
	exception.SetErrorCode (ERR__Exception::SEM_SET_ERROR);
      }
  }



  //@ incrementer le compteur d'utilisation
  sem_ar.sem_num=m_nb_sems-1;
  sem_ar.sem_op=1;
  sem_ar.sem_flg=SEM_UNDO;
  if ( semop(m_semid,&sem_ar,1) == -1 ) {
	//@ Si erreur Tracer l'erreur et lever une exception	
	parameter = t_Chaine(m_semid);
	exception.SetParametre (parameter);
	exception.SetErrorCode (ERR__Exception::SEM_CPT_INC_ERROR);
  }
  
}

//@ ------------------------------------------------------------------*/
//@ Methode : ~CMN__Verrou
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
CMN__Verrou::~CMN__Verrou()
{
  ERR__Exception exception;
  CMN__Chaine  parameter;
  struct sembuf sem_ar;

  //@ decrementer le compteur d'utilisation
  sem_ar.sem_num=m_nb_sems-1;
  sem_ar.sem_op=-1;
  sem_ar.sem_flg=SEM_UNDO;
  if ( semop(m_semid,&sem_ar,1) == -1 ) {
	//@ Si erreur Tracer l'erreur et lever une exception	
	parameter = t_Chaine(m_semid);
	exception.SetParametre (parameter);
	exception.SetErrorCode (ERR__Exception::SEM_CPT_DEC_ERROR);
  }

  //@ liberer le semaphore , si l'instance est la derniere utilisatrice
  // compteur a 0
  /* FA144 */
  arg_semun.val = 0;
  if ( semctl(m_semid,m_nb_sems-1,GETVAL,arg_semun) == 0 ) {
     arg_semun.val = 0;
     if ( semctl(m_semid,0,IPC_RMID,arg_semun) == -1 ) {
	   //@ Si erreur Tracer l'erreur et lever une exception	
	   parameter = t_Chaine(m_semid);
	   exception.SetParametre (parameter);
	   exception.SetErrorCode (ERR__Exception::SEM_DEL_ERROR);
     }
  }

  //@ reinitialiser les donnees membres
  m_key=0;
  m_nb_sems=0;
  m_semflg=0;
  m_semval=0;
  m_semid=-1;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Bloquer
//@
//@ Description : Permet de bloquer un semaphore
//@
//@ Parametres :
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__Verrou::Bloquer()
{
  // Declaration objet exception
  ERR__Exception exception;
  struct sembuf sem_ar;
  CMN__Chaine parameter;

  sem_ar.sem_num=0;
  sem_ar.sem_op=-1;
  sem_ar.sem_flg=0;
  if ( semop(m_semid,&sem_ar,1) == -1 ) {
	//@ Si erreur Tracer l'erreur et lever une exception	
	parameter = t_Chaine(m_semid);
	exception.SetParametre (parameter);
	exception.SetErrorCode (ERR__Exception::SEM_LOCK_ERROR);
  }
  
}




//@ ------------------------------------------------------------------*/
//@ Methode : DeBloquer
//@
//@ Description : Permet de debloquer un semaphore
//@
//@ Parametres :
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__Verrou::DeBloquer()
{
  // Declaration objet exception
  ERR__Exception exception;
  struct sembuf sem_ar;
  CMN__Chaine parameter;

  sem_ar.sem_num=0;
  sem_ar.sem_op=1;
  sem_ar.sem_flg=0;
  if ( semop(m_semid,&sem_ar,1) == -1 ) {
	//@ Si erreur Tracer l'erreur et lever une exception	
	parameter = t_Chaine(m_semid);
	exception.SetParametre (parameter);
	exception.SetErrorCode (ERR__Exception::SEM_UNLOCK_ERROR);
  }
  
}
