// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/ERR__Exception.h,v 1.3 2011/11/25 10:17:39 dpauly Exp $
//@ ---------------------------------------------------------------------------
//@ Composant : CMN
//@ CLASSE : ERR__Exception
//@
//@ Commentaire :
//@    Ce fichier gere la classe ERR__Exception
//@
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 13/05/2003 : P.L. AGNETTI
//@ Gestion des verrous : ajout d'exceptions pour les locks
//@ ---------------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Adaptation a l'OPS : ajout des exceptions liees au parseur XML, 
//@                      au SD et au WOM
//@ ---------------------------------------------------------------------------
//@ VERSION : 29/09/2000 : C.LACAZE
//@  Verification de mot-cles errones
//@ ---------------------------------------------------------------------------
//@ FA-ID : FPA-F11-FA-634-CN : 29/09/2000 :
//@  Ajout d'un numero d'exception emise par VerifierChamp
//@ ---------------------------------------------------------------------------
//@ VERSION : 21/03/2000 : A.MONTMORY
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-166-CN : 13/02/2006 : B.MASSART
//@ An exception is added when string "UMBRA_END" is not found in SVM auxiliary
//@ file. See IA-2100-FC-166-THA.
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

#ifndef ERR__Exception_h
#define ERR__Exception_h

// Fichier include

#include <vector>
#include "CMN__Chaine.h"
#include "ERR__Erreur.h"

// Macro pour lancer une exception (forcement une macro pour expanser
// correctement le __FILE__ et le __LINE__).
#define THROW(x) {x.SetFile(__FILE__);x.SetLine(__LINE__);throw(x); }

class ERR__Exception {

/*====================================================================*/

 public:
	
    //
    // Type
    //
    typedef vector<t_Chaine>	t_ListeParametre;
	
    //
    // Constante
    //
    
    typedef enum {
	// Exception sur le traitement des chaines
	FIELD_NOT_FOUND      = ERR__Erreur::CMN_BASE_ERROR,
	
	// Exception sur le theme lecture/ecriture XML
	INIT_FAILED          = ERR__Erreur::PRS_BASE_ERROR,
	PARSING_FAILED       = ERR__Erreur::PRS_BASE_ERROR+1,
	INVALID_TREE         = ERR__Erreur::PRS_BASE_ERROR+2,
	PARSING_WO_FAILED    = ERR__Erreur::PRS_BASE_ERROR+3,
	PARSING_OBT_FAILED   = ERR__Erreur::PRS_BASE_ERROR+4,
	PARSING_OSV_FAILED   = ERR__Erreur::PRS_BASE_ERROR+5,
	REPORTING_FAILED     = ERR__Erreur::PRS_BASE_ERROR+6,
	DUMPING_FAILED       = ERR__Erreur::PRS_BASE_ERROR+7,
	
	// Exception sur le theme message et connexion
	SOCKET_UNDEFINED     = ERR__Erreur::MSG_BASE_ERROR,
	SOCKET_BROKEN        = ERR__Erreur::MSG_BASE_ERROR+1,
	SOCKET_ERROR         = ERR__Erreur::MSG_BASE_ERROR+2,
	SOCKET_OPEN          = ERR__Erreur::MSG_BASE_ERROR+3,
	UNKNOWN_SERVICE      = ERR__Erreur::MSG_BASE_ERROR+4,
	UNKNOWN_HOST         = ERR__Erreur::MSG_BASE_ERROR+5,
	UNKNOWN_CNX_TYPE     = ERR__Erreur::MSG_BASE_ERROR+6,
	SOCKET_EXIST         = ERR__Erreur::MSG_BASE_ERROR+7,
	CNX_NOT_OPEN         = ERR__Erreur::MSG_BASE_ERROR+8,
	CNX_NOT_FOUND        = ERR__Erreur::MSG_BASE_ERROR+9,
	ADR_NOT_FOUND        = ERR__Erreur::MSG_BASE_ERROR+10,
	UNEXPECTED_MSG       = ERR__Erreur::MSG_BASE_ERROR+11,
	UNEXPECTED_TYPE      = ERR__Erreur::MSG_BASE_ERROR+12,
	SEM_CREATE_ERROR     = ERR__Erreur::MSG_BASE_ERROR+13,
	SEM_SET_ERROR        = ERR__Erreur::MSG_BASE_ERROR+14,
	SEM_DEL_ERROR        = ERR__Erreur::MSG_BASE_ERROR+15,
	SEM_LOCK_ERROR       = ERR__Erreur::MSG_BASE_ERROR+16,
	SEM_UNLOCK_ERROR     = ERR__Erreur::MSG_BASE_ERROR+17,
	SEM_CPT_INC_ERROR     = ERR__Erreur::MSG_BASE_ERROR+18,
	SEM_CPT_DEC_ERROR     = ERR__Erreur::MSG_BASE_ERROR+19,
	
	// Exception sur le theme serveur temps
	UNEXPECTED_ACTION    = ERR__Erreur::SRV_BASE_ERROR+1,
	
	// Exception sur la gestion des messages du journal de bord
	JDB_TOO_MUCH_PARAM   = ERR__Erreur::JDB_BASE_ERROR+1,
	JDB_UKN_CATALOG      = ERR__Erreur::JDB_BASE_ERROR+2,
	JDB_UKN_MESSAGE      = ERR__Erreur::JDB_BASE_ERROR+3,
	JDB_END_FAILED       = ERR__Erreur::JDB_BASE_ERROR+4,
	JDB_BAD_MSG_FORMAT   = ERR__Erreur::JDB_BASE_ERROR+5,
	
	// Exception sur le serveur de journal de bord
	JDBS_OPEN_FAILED     = ERR__Erreur::JDBS_BASE_ERROR+1,
	JDBS_LOCK_FAILED     = ERR__Erreur::JDBS_BASE_ERROR+2,
	JDBS_UNLOCK_FAILED   = ERR__Erreur::JDBS_BASE_ERROR+3,
	
	// Exception sur la gestion des processus
	CANT_GET_PID         = ERR__Erreur::MP_BASE_ERROR,
	CANT_LAUCH_PRO       = ERR__Erreur::MP_BASE_ERROR+1,
	CANT_OPEN_FILE       = ERR__Erreur::MP_BASE_ERROR+2,
	
	// Exception sur le serveur WOM
	WOM_ERROR            = ERR__Erreur::WOM_BASE_ERROR+1,
	WOM_INVALID_WO       = ERR__Erreur::WOM_BASE_ERROR+2,
	WOM_SIGNAL_ERROR     = ERR__Erreur::WOM_BASE_ERROR+3,
	WOM_TMT_NOT_FOUND    = ERR__Erreur::WOM_BASE_ERROR+4,
	WOM_ENV_UNDEFINED    = ERR__Erreur::WOM_BASE_ERROR+5,
	WOM_FILE_OPEN_ERROR  = ERR__Erreur::WOM_BASE_ERROR+6,
	WOM_FILE_WRITE_ERROR = ERR__Erreur::WOM_BASE_ERROR+7,

	//@ Exception sur le serveur SD
	SD_ERROR             = ERR__Erreur::SD_BASE_ERROR+1,
	SD_TMT_START_REFUSED = ERR__Erreur::SD_BASE_ERROR+2,
	SD_TMT_STOP_REFUSED  = ERR__Erreur::SD_BASE_ERROR+3,
	SD_TMT_NOT_FOUND     = ERR__Erreur::SD_BASE_ERROR+4,
	SD_TAC_NOT_FOUND     = ERR__Erreur::SD_BASE_ERROR+5,
	SD_ACT_NOT_FOUND     = ERR__Erreur::SD_BASE_ERROR+6,
	SD_ECH_TAC_NOT_FOUND = ERR__Erreur::SD_BASE_ERROR+7,
	SD_ENV_UNDEFINED     = ERR__Erreur::SD_BASE_ERROR+8,
	SD_FILE_OPEN_ERROR   = ERR__Erreur::SD_BASE_ERROR+9,
	SD_FILE_WRITE_ERROR  = ERR__Erreur::SD_BASE_ERROR+10,

	//@ Exception sur les acces aux donnees
        //   - acces au traitement a executer
	SD_DON_TMT_INVALID   = ERR__Erreur::SD_DON_BASE_ERROR+1,
	SD_DON_TMT_ROOT      = ERR__Erreur::SD_DON_BASE_ERROR+2,
	SD_DON_TMT_SLIDE_ID  = ERR__Erreur::SD_DON_BASE_ERROR+3,
	SD_DON_TMT_DATA_IN   = ERR__Erreur::SD_DON_BASE_ERROR+4,
	SD_DON_TMT_DATA_AUX  = ERR__Erreur::SD_DON_BASE_ERROR+5,
	SD_DON_TMT_DATA_PROD = ERR__Erreur::SD_DON_BASE_ERROR+6,
	SD_DON_TMT_ALLOC     = ERR__Erreur::SD_DON_BASE_ERROR+7,
	SD_DON_TMT_CASDEG    = ERR__Erreur::SD_DON_BASE_ERROR+8,
        //   - acces aux operations a effectuer
	SD_DON_OPER_FAILED   = ERR__Erreur::SD_DON_BASE_ERROR+20,
	SD_DON_OSV_UNLOADED  = ERR__Erreur::SD_DON_BASE_ERROR+21,
	SD_DON_OBT_UNLOADED  = ERR__Erreur::SD_DON_BASE_ERROR+22,
	SD_DON_SVM_NOUMBRAEND= ERR__Erreur::SD_DON_BASE_ERROR+23, /* FA166 */
        //   - acces aux fichiers
	SD_DON_FILE_OPEN     = ERR__Erreur::SD_DON_BASE_ERROR+30,
	SD_DON_FILE_READ     = ERR__Erreur::SD_DON_BASE_ERROR+31,
	SD_DON_FILE_WRITE    = ERR__Erreur::SD_DON_BASE_ERROR+32,
	SD_DON_FILE_UNKNOWN  = ERR__Erreur::SD_DON_BASE_ERROR+33,
	//   - initialisation des traces
	SD_DON_TRACE_INIT_FAILED = ERR__Erreur::SD_DON_BASE_ERROR+40
    } e_Code;
    
    //
    // Constructeur
    //
    ERR__Exception();
    
    //
    // Destructeur
    //
    ~ERR__Exception();
    
   
    //
    // Accesseurs
    //
    
    // Positionne un parametre 
    t_Chaine GetFile() const {return m_File;};	     
    void SetFile(const t_Chaine file) {m_File = file;};	     
    
    t_Chaine GetLine() const {return (m_Line);};	  
    void SetLine(const t_Chaine line) {m_Line = line;};	  
    
    e_Code GetErrorCode() const {return m_ErrorCode;};
    void SetErrorCode (const e_Code ErrorCode) 
	{m_ErrorCode = ErrorCode;};
    
    int GetErrno() const {return m_Errno;};
    void SetErrno (const int Errno) {m_Errno = Errno;};
    
    int GetNbParametre() const {return m_Parametres.size();};
    t_Chaine GetParametre (const int indice) const 
	{if (indice < int(m_Parametres.size())) 
	    return m_Parametres[indice];
	return (""); 
	};
    void SetParametre(const t_Chaine parametre) 
	{m_Parametres.insert(m_Parametres.end(), parametre);};
    void SetDebParametre(const t_Chaine parametre)
	{m_Parametres.insert(m_Parametres.begin(), parametre);};
    
    
    //
    // Methodes publiques
    //
    
    
/*====================================================================*/

 protected:
    
    //
    // Methodes protegees
    //
    
    //
    // Attributs proteges
    //


/*====================================================================*/

 private:
    
    //
    // Methodes privees
    //
    
    //
    // Attributs prives
    //
    t_Chaine         m_File;
    t_Chaine         m_Line;
    e_Code           m_ErrorCode;
    int              m_Errno;
    t_ListeParametre m_Parametres;
};

#endif
