static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/JDB__Callback.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ ---------------------------------------------------------------------------
//@ Composant : CMN
//@ CLASSE : JDB__Callback
//@ 
//@ Commentaire :
//@    Ce module gere la classe JDB__Callback qui fournit les methodes
//@    appeles par le module C JDB_JOURNAL_DE_BORD pour le traitement
//@    d'un event de type LOG et TRACE.
//@ 					
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 16/07/2002 : PBR
//@ Adaptation a l'OPS : rajout de l'Id du message et du type (SetType)
//@                      utilisation de la date courante du noeud. 
//@ ---------------------------------------------------------------------------
//@ VERSION : 18/04/2000 : C.LACAZE
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

//
// Fichiers include
//
#include "CMN__Commun.h"
#include "CMN__Date.h"
#include "ERR__Exception.h"
#include "JDB__Callback.h"
#include "JDB__Message.h"
#include "JDB_JOURNAL_DE_BORD.h"


// Pointeur vers la connexion serveur
// FT-2765
MSG__Connexion*    JDB__Callback::m_ptConnexionSrv = NULL;
// ...FT-2765


//======================================================================
//     		  	METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : JDB__Callback
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
JDB__Callback::JDB__Callback()
{
  //@ Initialisation de la liste de callbacks
  JDB_InitialiserCallback();
  
  //@ Installation des callbacks
  // FT-2765
  JDB_PositionnerCallback (static_cast<char *>("EVT"), JDB__CallbackEVT );
  JDB_PositionnerCallback (static_cast<char *>("TRA"), JDB__CallbackTRA );
  // ...FT-2765
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~JDB__Callback
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
JDB__Callback::~JDB__Callback()
{
}


//======================================================================
//					METHODES PRIVEES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : JDB__CallbackEVT
//@
//@ Description : Callback appelee pour les messages de type Evenement
//@
//@ Parametres :
//@               (ENTREE) msgId : Id du message
//@               (ENTREE) objet : Objet du message
//@               (ENTREE) nivErreur : niveau de criticite du message
//@               (ENTREE) message : libelle du criticite du message
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void JDB__Callback::JDB__CallbackEVT(TYCST__T_pt_char msgId,
                                     TYCST__T_pt_char objet,
                                     TYCST__T_pt_char nivErreur,
                                     const t_JDB_Message message)
{
  //@ Declaration d'un message Journal de bord
  JDB__Message messJdb;
  JDB__Message::e_Criticite criticite;

  messJdb.SetFonction (objet);

  messJdb.SetMsgId (msgId);

  messJdb.SetEventType(JDB__Message::EVENT);

  if( !strcmp (nivErreur, JDB_G_STR_INFO) ){
    criticite = JDB__Message::INFORMATION;
  }else if( !strcmp (nivErreur, JDB_G_STR_WARNING) ){
    criticite = JDB__Message::WARNING;
  }else if( !strcmp (nivErreur, JDB_G_STR_ALERT) ){
    criticite = JDB__Message::ERREUR;
  }else{
    return;
  }
  messJdb.SetCriticite (criticite);
  messJdb.SetLibelle (message);

  //@ Recuperation et positionnement de la date courante
  t_Date date;
  date.DateCourantePrecise();
  messJdb.SetDate (date);

  //@ Serialisation du message Journal de bord
  MSG__Message mess;
  messJdb.Serialiser (mess);
  
  //@ Publication du message
  try 
    {
	  // FT-2765
	  if (m_ptConnexionSrv != NULL)
	  {
		  m_ptConnexionSrv->PublierMessage(mess);
	  }
      // ...FT-2765
    }
  catch (ERR__Exception &exception) {}

}


//@ ------------------------------------------------------------------*/
//@ Methode : JDB__CallbackTRA
//@
//@ Description : Callback appelee pour les messages de type Trace
//@
//@ Parametres :
//@               (ENTREE) msgId : Id du message
//@               (ENTREE) objet : Objet du message
//@               (ENTREE) nivErreur : niveau de criticite du message
//@               (ENTREE) message : libelle du criticite du message
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void JDB__Callback::JDB__CallbackTRA(TYCST__T_pt_char msgId,
                                     TYCST__T_pt_char objet,
                                     TYCST__T_pt_char nivErreur,
                                     const t_JDB_Message message)
{
  //@ Declaration d'un message Journal de bord
  JDB__Message messJdb;
  JDB__Message::e_Criticite criticite;

  messJdb.SetFonction (objet);

  messJdb.SetMsgId (msgId);

  messJdb.SetEventType(JDB__Message::TRACE);

  if( !strcmp (nivErreur, JDB_G_STR_INFO) ){
    criticite = JDB__Message::INFORMATION;
  }else if( !strcmp (nivErreur, JDB_G_STR_WARNING) ){
    criticite = JDB__Message::WARNING;
  }else if( !strcmp (nivErreur, JDB_G_STR_ALERT) ){
    criticite = JDB__Message::ERREUR;
  }else{
    return;
  }
  messJdb.SetCriticite (criticite);
  messJdb.SetLibelle (message);

  //@ Recuperation et positionnement de la date courante
  t_Date date;
  date.DateCourantePrecise();
  messJdb.SetDate (date);

  //@ Serialisation du message Journal de bord
  MSG__Message mess;
  messJdb.Serialiser (mess);
  
  //@ Publication du message
  try 
    {
      // FT-2765
			// Pas d'exception lorsque m_ptConnexionSrv est NULL !
      if (m_ptConnexionSrv != NULL)
			{
			  m_ptConnexionSrv->PublierMessage(mess);
      }
      // ...FT-2765
    }
  catch (ERR__Exception &exception) {}
  
}
