static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/JDB__Client.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ ---------------------------------------------------------------------------
//@ Composant : CMN
//@ CLASSE : JDB__Client
//@ 
//@ Commentaire :
//@    Ce module gere la classe JDB__Client en s'appuyant 
//@    sur le module C JDB_JOURNAL_DE_BORD
//@ 					
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 18/04/2000 : C.LACAZE
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

//
// Fichiers include
//
#include <stdio.h>
#include "CMN__Commun.h"
#include "ERR__Exception.h"
#include "JDB__Client.h"


//======================================================================
//					METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : JDB__Client
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
JDB__Client::JDB__Client()
{
  //@ Initialisation de la liste des parametres
  JDB_InitialiserMessage (&m_ListeParametre);
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~JDB__Client
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
JDB__Client::~JDB__Client()
{
  //@ Liberation de la liste des parametres
  JDB_VideMessage (&m_ListeParametre);
}


//@ ------------------------------------------------------------------*/
//@ Methode : SetParametre
//@
//@ Description : Ajoute un parametre avec sa valeur dans la liste des
//@               parametres
//@
//@ Parametres :
//@               (ENTREE) nom : Nom du parametre.
//@               (ENTREE) valeur : Valeur du parametre.
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void JDB__Client::SetParametre(const t_Chaine& nom, const t_Chaine& valeur)
{
  //@ Ajoute le parametre dans la liste des parametres du message
  e_JDB_Status cr;
  cr = JDB_StockeParametre ((TYCST__T_pt_char) nom.c_str(),
                            (TYCST__T_pt_char) valeur.c_str(),
                            &m_ListeParametre);

  //@ Envoi d'une exception si le nombre maximum de parametres a ete 
  //@ atteint
  if( cr != JDB_Ok ){
    ERR__Exception exception;
    exception.SetErrorCode (ERR__Exception::JDB_TOO_MUCH_PARAM);
    THROW (exception);
  }
}


//@ ------------------------------------------------------------------*/
//@ Methode : EcrireMessage
//@
//@ Description : Envoie le message dont le numero est passe en
//@               parametre au journal de bord
//@
//@ Parametres :
//@               (ENTREE) section : Section contenant le message
//@               (ENTREE) numero : Numero du message
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void JDB__Client::EcrireMessage(int section, int numero) const
{
  //@ Ecriture du message avec les parametres positionnes
  e_JDB_Status cr;
  const t_JDB_pt_ListeParam ptParam = 
    (const t_JDB_pt_ListeParam) &m_ListeParametre;

  cr = JDB_EcrireMessage (section, numero, ptParam );

  switch( cr ){
    case JDB_Ok:
      return;
    break;
    default:
    break;
  }

  //@ Emission d'une exception si erreur d'ecriture
  ERR__Exception exception;
  switch( cr ){
    case JDB_ERR_ACCES_FIC_MES:
      exception.SetErrorCode (ERR__Exception::JDB_UKN_CATALOG);
    break;
    case JDB_ERR_MESSAGE_INEXISTANT:
      exception.SetErrorCode (ERR__Exception::JDB_UKN_MESSAGE);
    break;
    
    case JDB_ERR_SEP_ABSENT:
    case JDB_ERR_LIBELLE_ABSENT:
    case JDB_ERR_ATTRIBUT_ABSENT:
      exception.SetErrorCode (ERR__Exception::JDB_BAD_MSG_FORMAT);
    break;
    default:
    break;
  }
  THROW (exception);
  
}


//@ ------------------------------------------------------------------*/
//@ Methode : Liberer
//@
//@ Description : Libere la liste des parametres du message
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void JDB__Client::Liberer()
{
  //@ Ajoute le parametre dans la liste des parametres du message
  e_JDB_Status cr;
  cr = JDB_VideMessage (&m_ListeParametre);
}


//@ ------------------------------------------------------------------*/
//@ Methode : InitSet
//@
//@ Description : Initialise un set de message en les chargeant une 
//@               fois a vide
//@
//@ Parametres :
//@               (ENTREE) idSet
//@               (ENTREE) msgDeb
//@               (ENTREE) msgFinAucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void JDB__Client::InitSet(int idSet, int msgDeb, int msgFin)
{
  //@ Initialise le set
  JDB__InitSet (idSet,msgDeb,msgFin);
}
