static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/JDB__Message.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ ---------------------------------------------------------------------------
//@ Composant : CMN
//@ CLASSE : JDB__Message
//@
//@ Commentaire :
//@    Cette classe encapsule le message (au sens MSGS) qui contient
//@    l'event (log ou trace) envoye par les processus au JDBS a
//@    travers le MSGS
//@
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 16/07/2002 : PBR
//@ Adaptation a l'OPS : rajout de l'Id du message
//@ ---------------------------------------------------------------------------
//@ VERSION : 26/09/2001 : S. BOBIN
//@ FA-ID : FPA-F11-FA-2344-CN : 20/08/01
//@ troncature du libelle des messages JDB dans le fichier JDB a 200 caracteres
//@   => modification de Synthetiser()
//@ ---------------------------------------------------------------------------
//@ VERSION : 30/03/2000 : C.LACAZE
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

//
// Fichiers include
//
#include "CMN__Commun.h"
#include "CMN__Date.h"
#include "JDB__Message.h"
#include "ERR__Exception.h"
#include "CFG__Configuration.h"


//
// Definition des constantes 
//

// Version Classe
const  t_Chaine JDB__Message::VERSION_CLASSE = "1.0";

// Longueur maximum du libelle
const int JDB__Message::LG_MAX_LIBELLE = 200;	

const char *VALEUR_NOTICE = "NOTICE";
const char *VALEUR_INFORMATION = "INFORMATION";
const char *VALEUR_WARNING = "WARNING";
const char *VALEUR_ERREUR = "ERREUR";

const char *CHAMP_FONCTION = "FONCTION";
const char *CHAMP_DATE = "DATE";
const char *CHAMP_CRITICITE = "CRITICITE";
const char *CHAMP_LIBELLE = "LIBELLE";


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : JDB__Message
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
JDB__Message::JDB__Message()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~JDB__Message
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@
//@ ------------------------------------------------------------------*/
JDB__Message::~JDB__Message()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Deserialiser
//@
//@ Description : Construit un objet de la classe JDB__Message a
//@               partir du contenu du message passe en parametre
//@
//@ Parametres :
//@               (ENTREE) message : message contenant un message JDB
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void JDB__Message::Deserialiser (const MSG__Message& message)
{
  t_Chaine           bloc (message.GetBloc());
  CMN__Configuration config;
  t_Chaine           version;
  t_Chaine           dateText;
 

  //@ SI le type du message est bien evenement Journal de bord
  if ((message.GetType() == MSG__Message::EVT_JDB) ||
      (message.GetType() == MSG__Message::TRA_JDB))
    {
      if (message.GetType() == MSG__Message::EVT_JDB)
	{
	  m_MsgType = EVENT;
	}
      else
	{
	  m_MsgType = TRACE;
	}

    //@ lecture de la version de la classe
    config.ChargerChaine(bloc);

    config.LireValeur("SECTION_ENTETE", "CHAMP_CLASSE", version);

    //@ lecture du champ FONCTION 
    config.LireValeur(CHAMP_FONCTION, "VALEUR", m_Fonction);
    
    //@ lecture de l'attribut date dans la section
    config.LireValeur(CHAMP_DATE, "VALEUR", dateText);
    t_Date date(dateText, FORMAT_DATE);
    m_Date = date;
    
    //@ lecture de l'attribut msg Id dans la section
    config.LireValeur("ID", "VALEUR", m_MsgId);
    
    //@ lecture du champ CRITICITE 
    t_Chaine criticite;
    config.LireValeur(CHAMP_CRITICITE, "VALEUR", criticite);
    if (criticite == VALEUR_INFORMATION)
      {
	m_Criticite = INFORMATION;
      }
    else if (criticite == VALEUR_WARNING)
      {
	m_Criticite = WARNING;
      }
    else
      {
	m_Criticite = ERREUR;
      }

    //@ lecture du champ LIBELLE 
    config.LireValeur(CHAMP_LIBELLE, "VALEUR", m_Libelle);
  }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : Serialiser
//@
//@ Description : Serialise un objet JDB__Message en un paquet reseau
//@               propre a etre envoye
//@
//@ Parametres :
//@               (SORTIE) MSG__Message : Message reseau
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void JDB__Message::Serialiser(MSG__Message& message)
{
  CMN__Configuration config;
  t_Chaine resultat;
   
  //@ Ecriture de la version de la classe
  config.EcrireValeur("SECTION_ENTETE", "CHAMP_CLASSE", VERSION_CLASSE);

  //@ Ecriture de l'attribut fonction 
  config.EcrireValeur(CHAMP_FONCTION, "VALEUR", m_Fonction);

  //@ Ecriture de l'attribut date dans la section
  t_Chaine dateText;
  t_Date date = m_Date;
  dateText = date.Formater(FORMAT_DATE);
  config.EcrireValeur(CHAMP_DATE, "VALEUR", dateText);

  //@ Ecriture de l'id du message
  config.EcrireValeur("ID", "VALEUR", m_MsgId);

  //@ Ecriture de l'attribut criticite 
  t_Chaine criticite = "";
  switch (m_Criticite)
    {
    case INFORMATION :
      criticite = VALEUR_INFORMATION;
      break;
      
    case WARNING :
      criticite = VALEUR_WARNING;
      break;
      
    case ERREUR :
      criticite = VALEUR_ERREUR;
      break;
      
    default:
      break;
    }

  config.EcrireValeur(CHAMP_CRITICITE, "VALEUR", criticite);

  //@ Ecriture de l'attribut libelle 
  config.EcrireValeur(CHAMP_LIBELLE, "VALEUR", m_Libelle);

  //@ Sauvegarde de la serialisation dans une chaine
  config.SauverChaine(resultat);
   
  //@ Sauvegarde du bloc dans le message
  message.SetBloc(resultat.c_str(), resultat.size()+1);
    
  //@ Sauvegarde du type dans le message
  if (m_MsgType == EVENT)
    {
      message.SetType(MSG__Message::EVT_JDB);
    }
  else
    {
      message.SetType(MSG__Message::TRA_JDB);
    }

  //@ Positionnement de l'adresse de publication du message si elle
  //@ n'est pas encore renseignee
  if( message.GetAdresse()[0] == 0 )
    {
      message.SetAdresse (CFG__Configuration::GetAdresse ("JDB_INPUT"));
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : Serialiser
//@
//@ Description : Construit un message a partir de la description de
//@               l'objet courant
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Message contenant la description de l'objet
//@
//@ ------------------------------------------------------------------*/
MSG__Message JDB__Message::Serialiser ()
{
  //@ Declaration d'un message
  MSG__Message message;
  
  //@ Appel de la serialisation
  Serialiser (message);
  
  //@ Retour du message
  return message;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Synthetiser
//@
//@ Description : Genere une section pour l'objet dans le fichier de
//@               configuration
//@
//@ Parametres :
//@               (ENTREE) config : fichier de configuration
//@               (ENTREE) numero : numero de la section
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void JDB__Message::Synthetiser(CMN__Configuration& config, int numero) const
{
  t_Chaine dateText;
  t_Date   date;


  //@ Ecriture de l'attribut fonction dans la section
  config.EcrireValeur (numero, CHAMP_FONCTION, m_Fonction);

  //@ Recuperation et positionnement de la date courante PDS :
  //@ ecriture de la date dans la section
  date.DateCourantePrecise();
  dateText = date.Formater(FORMAT_DATE);
  config.EcrireValeur (numero, CHAMP_DATE, dateText);

  //@ Ecriture de l'attribut fonction Id dans la section
  config.EcrireValeur (numero, "ID", m_MsgId);

  //@ Ecriture de l'attribut criticite dans la section
  t_Chaine criticite;
  switch( m_Criticite){

    case INFORMATION :
      criticite = VALEUR_INFORMATION;
    break;

    case WARNING :
      criticite = VALEUR_WARNING;
    break;

    case ERREUR :
      criticite = VALEUR_ERREUR;
    break;

    default:
    break;
  }
  config.EcrireValeur (numero, CHAMP_CRITICITE, criticite);

  //@ Ecriture de l'attribut libelle en tronquant a gauche
  //@ lorsque le libelle est trop grand
  if ( m_Libelle.size() > LG_MAX_LIBELLE )
    {
      string libelle_tronque(m_Libelle, m_Libelle.size() - LG_MAX_LIBELLE);
      config.EcrireValeur (numero, CHAMP_LIBELLE, libelle_tronque);    
    } 
  else
    {
      config.EcrireValeur (numero, CHAMP_LIBELLE, m_Libelle);    
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : Analyser
//@
//@ Description : Analyse une section du fichier de configuration pour
//@               construire l'objet message
//@
//@ Parametres :
//@               (ENTREE) config : fichier de configuration
//@               (ENTREE) numero : section decrivant le message
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void JDB__Message::Analyser(CMN__Configuration& config, const int numero)
{
  t_Chaine numeroText = t_Chaine(numero);

  //@ Lecture du champ FONCTION dans la section
  config.LireValeur (numeroText, CHAMP_FONCTION, m_Fonction);

  //@ Lecture de l'attribut date dans la section
  t_Chaine dateText;

  config.LireValeur (numeroText, CHAMP_DATE, dateText);
  t_Date date(dateText, FORMAT_DATE);
  m_Date = date;

  //@ Lecture du champ ID dans la section
  config.LireValeur (numeroText, "ID", m_MsgId);

  //@ Lecture du champ CRITICITE dans la section
  t_Chaine criticite;
  config.LireValeur (numeroText, CHAMP_CRITICITE, criticite);
  if( criticite == VALEUR_INFORMATION ){
    m_Criticite = INFORMATION;
  }else if( criticite == VALEUR_WARNING ){
    m_Criticite = WARNING;
  }else{
    m_Criticite = ERREUR;
  }

  //@ Lecture du champ LIBELLE dans la section
  config.LireValeur (numeroText, CHAMP_LIBELLE, m_Libelle);
}


//=====================================================================
//                        METHODES PRIVEES
//=====================================================================
