// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/JDB__Message.h,v 1.3 2011/11/25 10:17:39 dpauly Exp $
//@ ---------------------------------------------------------------------------
//@ Composant : CMN
//@ CLASSE : JDB__Message
//@
//@ Commentaire :
//@    Cette classe encapsule le message (au sens MSGS) qui contient
//@    l'event (log ou trace) envoye par les processus au JDBS a travers
//@    le MSGS.
//@
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 16/07/2002 : PBR
//@ Adaptation a l'OPS : rajout de l'Id du message et du Type
//@ ---------------------------------------------------------------------------
//@ VERSION : 30/03/2000 : C.LACAZE
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

#ifndef JDB__Message_h
#define JDB__Message_h


//
// Fichiers include
//

// STL

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN__Date.h"
#include "CMN__Configuration.h"
#include "CMN__Config.h"
#include "MSG__Message.h"


class JDB__Message {

    /*=================================================================*/

public:

    //
    // Constante
    //
    static const t_Chaine VERSION_CLASSE;
    static const int      LG_MAX_LIBELLE;  // Longueur maximum du libelle 
	
    // Type d'action sur le verrouillage d'un fichier
    typedef enum {
	INFORMATION,
	WARNING,
	ERREUR
    } e_Criticite;

    // Type de message JdB (permet de definir plusieurs types de
    // messages JDB qui sont traites de facon differente par le 
    // destinataire -JDBS-)
    typedef enum {
	EVENT,
	TRACE
    }e_JdbMessageType;
    
    
    //
    // Constructeur
    //
    JDB__Message();
    
    //
    // Destructeur
    //
    ~JDB__Message();
    
    //
    // Accesseurs
    //

    // Type de donnees concernes : Product/Aux Data/Soft : 
    // chaine [P, A, S] extraite du catalogue
    void SetFonction(const t_Chaine& fonction) {m_Fonction = fonction;}
    t_Chaine GetFonction() const {return m_Fonction;}

    void SetDate( t_Date date ) {m_Date = date;}
    t_Date GetDate() const {return m_Date;}
    
    // Information/Warning/Alert
    void SetCriticite(const e_Criticite criticite) {m_Criticite = criticite;}
    e_Criticite GetCriticite() const {return m_Criticite;}
    
    // Corps du message JdB
    void SetLibelle(const t_Chaine& libelle) {m_Libelle = libelle;}
    t_Chaine GetLibelle() const {return m_Libelle;}
    
    // Id du message JDB (chque message a un id unique) : 
    // chaine  extraite du catalogue
    void SetMsgId(const t_Chaine& msgId) {m_MsgId = msgId;}
    t_Chaine GetMsgId() const {return m_MsgId;}
  
    // Type de l'Event : EVENT ou TRACE
    void SetEventType (e_JdbMessageType msgType) {m_MsgType = msgType;}
    e_JdbMessageType GetEventType() const {return m_MsgType;}
    
    
    //
    // Methodes publiques
    //
    void Deserialiser(const MSG__Message& message);
    MSG__Message Serialiser ();
    void Serialiser(MSG__Message &message);
    void Synthetiser(CMN__Configuration& config, int numero) const;
    void Analyser(CMN__Configuration& config, const int numero);


    /*=================================================================*/

 protected:

    //
    // Methodes protegees
    //
    
    //
    // Attributs proteges
    //


    /*=================================================================*/

 private:

    //
    // Methodes privees
    //
    
    //
    // Attributs prives
    //
    
    // Type de donnees concernes : Product/Aux Data/Soft
    t_Chaine m_Fonction;
    
    // Date du message
    t_Date m_Date;
    
    // Niveau de criticite du message :INFO/WARNING/ALERT
    e_Criticite m_Criticite;
    
    // Libelle du message : corps du message
    t_Chaine m_Libelle;
    
    // Message Id 
    t_Chaine m_MsgId;
    
    // Type du message : EVENT ou TRACE
    e_JdbMessageType m_MsgType;
};

#endif
