static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/MP__Command.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ ---------------------------------------------------------------------------
//@ Composant : MP
//@ CLASSE : MP__Command
//@
//@ Commentaire :
//@    Cette classe encapsule et gere le message de type controle
//@    de la production (step/stop/suspend/resume/break) envoye par 
//@    le MP au WOM
//@
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 18/07/2002 : PBR
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

//
// Fichiers include
//
#include "ERR__Exception.h"
#include "CMN__Config.h"
#include "MP__Message.h"
#include "MP__Command.h"


//
// Version Classe
//
const  t_Chaine MP__Command::VERSION_CLASSE = "1.0";


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : MP__Command
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
MP__Command::MP__Command(e_CMN__CommandGroup commandGroup) 
{
  this->SetCommandGroup(commandGroup);
  
  //@ Initialisation des attributs
  m_SDprocessId = -1;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~MP__Command
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
MP__Command::~MP__Command()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Serialiser
//@
//@ Description : Serialisation de l'objet
//@
//@ Parametres :
//@               (ENTREE) typeMessage : type du message CMD ou CMD_TM
//@               (SORTIE) message : Message reseau
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MP__Command::Serialiser(MP__Message::e_TypeMessage typeMessage, 
                             MSG__Message& message)
{
   CMN__Configuration config;
   t_Chaine resultat;
   
   
   //@ Ecriture de la version de la classe
   config.EcrireValeur("SECTION_ENTETE","CHAMP_CLASSE",
                       VERSION_CLASSE);

   //@ Ecriture des informations generales d'une commande :
   //@   - le groupe
   //@   - l'identificateur de la cmde
   config.EcrireValeur("CMDE_GROUP","VALEUR", this->GetCommandGroup());
   config.EcrireValeur("IDENTIFICATEUR","VALEUR",this->GetCommandId());

   //@ Ecriture des informations de la commande :
   //@   - le type de commande
   //@   - le status de la commande
   //@   - le nom du work order
   config.EcrireValeur("CMD_TYPE","VALEUR",this->GetCommandType());
   config.EcrireValeur("CMD_STATUS","VALEUR",this->GetCommandStatus());
   config.EcrireValeur("FILENAME","VALEUR",this->GetWOFilename());

   //@ Ecriture des informations propres au message echange entre 
   //@ le MP et le WOM : l'Id du processus SD
   config.EcrireValeur("ID_SD","VALEUR", m_SDprocessId);

   //@ Sauvegarde de la serialisation dans une chaine
   config.SauverChaine(resultat);
   
   //@ Sauvegarde du bloc dans le message
   message.SetBloc (resultat.c_str(),resultat.size()+1);
    
   //@ Sauvegarde du type dans le message
   message.SetType(typeMessage);
}


//@ ------------------------------------------------------------------*/
//@ Methode : Serialiser
//@
//@ Description : Serialisation de l'objet
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               message : Message reseau
//@
//@ ------------------------------------------------------------------*/
MSG__Message MP__Command::Serialiser(MP__Message::e_TypeMessage typeMessage)
{
  MSG__Message message;
  
  //@ Serialisation de l'objet
  Serialiser(typeMessage, message);
  
  //@ Retourne le message
  return (message);
}


//@ ------------------------------------------------------------------*/
//@ Methode : Deserialiser
//@
//@ Description : Deserialisation de l'objet
//@
//@ Parametres :
//@               (ENTREE) message : message reseau contenant un objet
//@                                  MP__Command serialise
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MP__Command::Deserialiser(const MSG__Message& message)
{
  t_Chaine bloc (message.GetBloc());
  CMN__Configuration config;
  t_Chaine version;
  t_Chaine temp;


  //@ SI le type du message est bien EVT_PRCC_CTRL ou EVT__CMD_TM
  if ((message.GetType() == MP__Message::EVT_PRCSS_CTRL) ||
      (message.GetType() == MP__Message::EVT_CMD_TM))
    {
      //@ lecture de la version de la classe
      config.ChargerChaine(bloc);
      config.LireValeur("SECTION_ENTETE","CHAMP_CLASSE",
			version);

      //@ lecture des informations generales d'une commande :
      //@   - le groupe
      //@   - l'identificateur de la cmde
      config.LireValeur("CMDE_GROUP","VALEUR", temp);
      this->SetCommandGroup((e_CMN__CommandGroup)int(temp));
      config.LireValeur("IDENTIFICATEUR","VALEUR",temp);
      this->SetCommandId((t_CMN__CommandId)int(temp));

      //@ lecture des informations de la commande :
      //@   - le type de commande
      //@   - le status de la commande
      //@   - le nom du work order
      config.LireValeur("CMD_TYPE","VALEUR", temp);
      this->SetCommandType ((e_CMN__CommandType)int(temp));
      config.LireValeur("CMD_STATUS","VALEUR", temp);
      this->SetCommandStatus ((e_CMN__CommandStatus)int(temp));
      config.LireValeur("FILENAME","VALEUR", temp);
      this->SetWOFilename ((t_CMN__WorkOrderFilename)temp);

      //@ lecture des informations propres au message echange entre 
      //@ le MP et le WOM : l'Id du processus SD
      config.LireValeur("ID_SD","VALEUR", temp);
      m_SDprocessId = (e_CMN__CommandType) int(temp);	       
    }
  //@ FIN SI
}
