static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/MP__Dump.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ ---------------------------------------------------------------------------
//@ Composant : MP
//@ CLASSE : MP__Dump
//@
//@ Commentaire :
//@    Cette classe encapsule et gere le message de type fin de 
//@    traitement d'un dump envoye par le processus WOM au MP 
//@    lorsque le traitement d'un dump est termine
//@
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 22/07/2002 : PBR
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

//
// Fichiers include
//
#include "CMN__Commun.h"
#include "ERR__Exception.h"
#include "CFG__Configuration.h"
#include "CMN__Config.h"
#include "MP__Message.h"
#include "MP__Dump.h"


//
// Definition des constantes 
//

// Version Classe
const  t_Chaine MP__Dump::VERSION_CLASSE = "1.0";


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : MP__Dump
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
MP__Dump::MP__Dump() 
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~MP__Dump
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
MP__Dump::~MP__Dump()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Serialiser
//@
//@ Description : Serialisation de l'objet
//@
//@ Parametres :
//@               (SORTIE) message : Message reseau
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MP__Dump::Serialiser(MSG__Message& message)
{
   CMN__Configuration config;
   t_Chaine resultat;
   
   
   //@ Ecriture de la version de la classe
   config.EcrireValeur("SECTION_ENTETE", "CHAMP_CLASSE", VERSION_CLASSE);
		       
   //@ Ecriture de l'etat de fin de traitement du dump courant
   config.EcrireValeur("ETAT_DUMP", "VALEUR", m_EtatFin);
		       
   //@ Sauvegarde de la serialisation dans une chaine
   config.SauverChaine(resultat);
   
   //@ Sauvegarde du bloc dans le message
   message.SetBloc (resultat.c_str(),resultat.size()+1);
    
   //@ Sauvegarde du type dans le message
   message.SetType(MP__Message::EVT_FIN_DUMP);

  //@ Positionnement de l'adresse de publication du message si
  //@ elle n'est pas encore renseignee
  if( message.GetAdresse()[0] == 0 )
    {
      message.SetAdresse(CFG__Configuration::GetAdresse("MP_INPUT"));
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : Serialiser
//@
//@ Description : Serialisation de l'objet
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               message : Message reseau
//@
//@ ------------------------------------------------------------------*/
MSG__Message MP__Dump::Serialiser()
{
  MSG__Message message;
  
  //@ Serialisation de l'objet
  Serialiser(message);
  
  //@ Retourne le message
  return (message);
}


//@ ------------------------------------------------------------------*/
//@ Methode : Deserialiser
//@
//@ Description : Deserialisation de l'objet
//@
//@
//@ Parametres :
//@               (ENTREE) message : message reseau contenant un objet
//@                                  MP__ArretProcessus serialise
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MP__Dump::Deserialiser(const MSG__Message& message)
{
  t_Chaine           bloc(message.GetBloc());
  CMN__Configuration config;
  t_Chaine           version;
  t_Chaine           temp;


  //@ SI le type du message est bien EVT_FIN_DUMP
  if (message.GetType() == MP__Message::EVT_FIN_DUMP)
    {
      //@ lecture de la version de la classe
      config.ChargerChaine(bloc);
      config.LireValeur("SECTION_ENTETE", "CHAMP_CLASSE", version);
      
      //@ lecture de l'etat de fin de traitement du dump
      config.LireValeur("ETAT_DUMP", "VALEUR", temp);
      m_EtatFin = (e_EtatFinDump) int(temp);
    }
  //@ FIN SI
}
