static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/MP__HKTMStatus.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ ---------------------------------------------------------------------------
//@ Composant : MP
//@ CLASSE : MP__HKTMStatus
//@
//@ Commentaire :
//@  Cette classe gere les HKTM status echanges entre les processus
//@  applicatifs et le MP
//@
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 22/07/2002 : PBR
//@ Adaptation a l'OPS
//@ ---------------------------------------------------------------------------
//@ VERSION : 21/03/2000 : A.MONTMORY
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

//
// Fichiers include
//
#include <values.h>

#include "ERR__Exception.h"
#include "MP__HKTMStatus.h"
#include "CMN__Config.h"
#include "CFG__Configuration.h"
#include "MP__Message.h"

//
// Version Classe
//
const  t_Chaine MP__HKTMStatus::VERSION_CLASSE = "1.0";


//==================================================================
//                        METHODES PUBLIQUES
//==================================================================

//@ ---------------------------------------------------------------*/
//@ Methode : MP__HKTMStatus
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ---------------------------------------------------------------*/
MP__HKTMStatus::MP__HKTMStatus (e_ListeStatus statusId)
{
  //@ Initialisation des attributs a des valeurs par defaut
  SetIdStatus (statusId);

}


//@ ---------------------------------------------------------------*/
//@ Methode : ~MP__HKTMStatus
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ---------------------------------------------------------------*/
MP__HKTMStatus::~MP__HKTMStatus()
{
}


//@ ---------------------------------------------------------------*/
//@ Methode : SetIdStatus
//@
//@ Description : Positionner l'Id du Status
//@
//@ Parametres :
//@               (ENTREE) IdStatus
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ---------------------------------------------------------------*/
void MP__HKTMStatus::SetIdStatus (const e_ListeStatus idStatus)
{
  //@ Initialisation des attributs a des valeurs par defaut
  m_IdStatus = idStatus;

  //@ Initialisation du type du Status
  switch (idStatus) {

    case PPFversion:
      m_TypeStatus = VERSION;
    break;

    case OPSReadiness:
      m_TypeStatus = READINESS;
    break;
 
    case L1a_SensingStartTime:
    case L1b_SensingStartTime:
    case L1c_SensingStartTime:
    case ENG_SensingStartTime:
    case VER_SensingStartTime:
      m_TypeStatus = SENSING_START_TIME;
    break;

    case L1a_CurrentSensingTime:
    case L1b_CurrentSensingTime:
    case L1c_CurrentSensingTime:
    case ENG_CurrentSensingTime:
    case VER_CurrentSensingTime:
      m_TypeStatus = CURRENT_SENSING_TIME;
    break;

    case L1a_InstantaneousQuality:
    case L1b_InstantaneousQuality:
    case L1c_InstantaneousQuality:
    case ENG_InstantaneousQuality:
    case VER_InstantaneousQuality:
      m_TypeStatus = QUALITY;
    break;

    case L1a_StartOrbit:
    case L1b_StartOrbit:
    case L1c_StartOrbit:
    case ENG_StartOrbit:
    case VER_StartOrbit:
      m_TypeStatus = START_ORBIT;
    break;

    case L1a_ProcessingState:
    case L1b_ProcessingState:
    case L1c_ProcessingState:
    case ENG_ProcessingState:
    case VER_ProcessingState:
      m_TypeStatus = PROCESSING_STATE;
    break;

    case CPUusage:
      m_TypeStatus = CPU_USAGE;
    break;

    case NONE:
      m_TypeStatus = NO_TYPE;
    break;

    default:
    break;
  
  };


  this->Clear();
}


//@ ---------------------------------------------------------------*/
//@ Methode : Serialiser
//@
//@ Description : Serialisation de l'objet
//@
//@ Parametres :
//@               (SORTIE) message : Message reseau.
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ---------------------------------------------------------------*/
void MP__HKTMStatus::Serialiser( MSG__Message& message)
{
   CMN__Configuration config;
   t_Chaine resultat;
   
   
   //@ Ecriture de la version de la classe
   config.EcrireValeur("SECTION_ENTETE","CHAMP_CLASSE",
                       VERSION_CLASSE);
		       
   //@ Ecriture de l'Id de la HKTM
   config.EcrireValeur("HKTM_ID","VALEUR", m_IdStatus);

   //@ Ecriture du type de status
   config.EcrireValeur("HKTM_TYPE","VALEUR", m_TypeStatus);

   //@ Ecriture de la validite
   config.EcrireValeur("HKTM_VALIDITY","VALEUR", m_validity);

   //@ Ecriture de la disponibilite
   config.EcrireValeur("HKTM_AVAILABILITY","VALEUR", m_available);

   //@ Ecriture du service emetteur des donnees (SPACECRAFT)
   config.EcrireValeur("HKTM_SERVICE","VALEUR", m_service);	

   //@ Ecriture de la valeur chaine Param Name
   config.EcrireValeur("PARAM_NAME","VALEUR",m_ParamName);

   //@ Ecriture de la valeur chaine de la ressource
   config.EcrireValeur("VALEUR_CHAINE","VALEUR",m_ValeurChaine);

   //@ Sauvegarde de la serialisation dans une chaine
   config.SauverChaine(resultat);
   
   //@ Sauvegarde du bloc dans le message
   message.SetBloc (resultat.c_str(),resultat.size()+1);
    
   //@ Sauvegarde du type dans le message
   message.SetType(MP__Message::EVT_HKTM_STATUS);
   
   //@ Positionnement de l'adresse de publication du message si elle 
   //@ n'est pas encore renseignee
   if( message.GetAdresse()[0] == 0 )
     {
       message.SetAdresse (CFG__Configuration::GetAdresse ("MP_INPUT"));
     }  
}


//@ ---------------------------------------------------------------*/
//@ Methode : Serialiser
//@
//@ Description : Serialisation de l'objet
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               message : Message reseau
//@
//@ ---------------------------------------------------------------*/
MSG__Message MP__HKTMStatus::Serialiser()
{
  MSG__Message message;
  
  //@ Serialisation de l'objet
  Serialiser(message);
  
  //@ Retourne le message
  return (message);
}


//@ ---------------------------------------------------------------*/
//@ Methode : Deserialiser
//@
//@ Description : Deserialisation de l'objet
//@
//@
//@ Parametres :
//@              (ENTREE) message : message reseau contenant un objet
//@                                 MP__HKTMStatus serialise.
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ---------------------------------------------------------------*/
void MP__HKTMStatus::Deserialiser(const MSG__Message& message)
{
  t_Chaine bloc (message.GetBloc());
  CMN__Configuration config;
  t_Chaine version;
  t_Chaine temp;

  
  //@ SI le type du message est bien EVT_HKTM_STATUS
  if (message.GetType() == MP__Message::EVT_HKTM_STATUS) 
    {
      //@ lecture de la version de la classe
      config.ChargerChaine(bloc);
      config.LireValeur("SECTION_ENTETE","CHAMP_CLASSE",
                       version);
      
      //@ lecture de l'Id de la HKTM
      config.LireValeur("HKTM_ID","VALEUR", temp);
      // appel a SetIdStatus afin d'initialiser m_TypeStatus
      m_IdStatus = (e_ListeStatus)int(temp);

      //@ lecture du type de status
      config.LireValeur("HKTM_TYPE","VALEUR", temp);
      m_TypeStatus = (e_TypeStatus)int(temp);

      //@ lecture de la validite
      config.LireValeur("HKTM_VALIDITY","VALEUR", temp);
      m_validity=(e_Validity)int(temp);
      
      //@ lecture de la disponibilite
      config.LireValeur("HKTM_AVAILABILITY","VALEUR", temp);
      m_available=(e_Availability)int(temp);

      //@ lecture du service emetteur des donnees (SPACECRAFT)
      config.LireValeur("HKTM_SERVICE","VALEUR", temp);
      m_service=(e_CMN_CGSSpacecraft)int(temp);	
      
      //@ lecture du type de produit en cours de generation
      config.LireValeur("PARAM_NAME","VALEUR", m_ParamName);
      
      //@ lecture de la valeur chaine de la ressource
      config.LireValeur("VALEUR_CHAINE","VALEUR",m_ValeurChaine);
    }
  //@ FIN SI
}
