// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/MP__HKTMStatus.h,v 1.3 2011/11/25 10:17:39 dpauly Exp $
//@ ---------------------------------------------------------------------------
//@ Composant : MP
//@ CLASSE : MP__HKTMStatus
//@
//@ Commentaire :
//@  Cette classe gere les HKTM status echanges entre les processus
//@  applicatifs et le MP.
//@  Les types de HKTM Status geres sont les suivant :
//@  
//@  ----------------------------------------------------------------
//@ |   Types               | Process fournisseur de l'information   |
//@ |-----------------------|----------------------------------------|
//@ | PPF version           |  MP                                    |
//@ |                       |                                        |
//@ | Readiness             |  WOM :                                 |
//@ |                       |       1 (idle) :                       |
//@ |                       |         si liste des comd STEP vide    |
//@ |                       |         ou sur reception de SUSPEND    |
//@ |                       |       2 (working) :                    |
//@ |                       |         si un traitement en cours      |
//@ |                       |       3 (non utilise)                  |
//@ |                       |  MP :                                  |
//@ |                       |       4 (unrecoverable error) :        |
//@ |                       |         si mort d'un processus         |
//@ |                       |                                        |
//@ | Start time            |  SD : apres le calcul de l'overlap     |
//@ |                       |                                        |
//@ | Current sensing time  |  SD                                    |
//@ |                       |                                        |
//@ | Instantaneous quality |  WOM                                   |
//@ |                       |                                        |
//@ | Start Orbit Number    |  SD : apres le calcul de l'overlap     |
//@ |                       |                                        |
//@ | Processing State      |  WOM sur -> reception de STOP          |
//@ | (OK/FAILURE)          |          -> abscence fichier d'entree  |
//@ |                       |          -> reception d'un produit L0  |
//@ |                       |             trop petit.                |
//@ |                       |                                        |
//@ | Cpu Usage             |  SD                                    |
//@  ----------------------------------------------------------------|
//@          
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 11/07/2003 : P. CABANE
//@ FA-ID : IA-2100-FA-11-CN : 11/07/2003 : P. CABANE :
//@ Mise en conformite de l'enumere e_ReadinessState avec l'ICD de l'OPS
//@ ---------------------------------------------------------------------------
//@ VERSION : 05/03/2003 : PBR
//@ FA-ID : IA-2100-FA-11-CN : 05/03/2003 : PBR :
//@ Mise en conformite de l'enumere e_ReadinessState avec l'ICD de l'OPS
//@ ---------------------------------------------------------------------------
//@ VERSION : 22/07/2002 : PBR
//@ Adaptation a l'OPS
//@ ---------------------------------------------------------------------------
//@ VERSION : 21/03/2000 : A.MONTMORY
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-144-CN : 21/09/2004 : B. MASSART
//@ Problemes lors du portage d'OPS vers Solaris par Meteo France
//@ Voir IA-2100-FC-144-THA.
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

#ifndef MP__HKTMStatus_h
#define MP__HKTMStatus_h

//
// Fichier include
//

#include "CMN__Commun.h"
#include "MSG__Message.h"
#include "CMN__CGSCst.h"

#undef OK

class MP__HKTMStatus {

/*=================================================================*/

public:
        //
	// Constante
	//
	static const t_Chaine VERSION_CLASSE;
		
        //
        // Validite de la HKTM (par defaut toujours a VALID);
        typedef enum  {
	   UNVALID = 0,
	   VALID = 1
        } e_Validity;	

        //
        // Disponibilite du status
        typedef enum  {
	   NOT_AVAILABLE= 0,
	   AVAILABLE = 1
        } e_Availability;
		
	//
	// Liste des types de statuts
	typedef enum {
	   VERSION                 = 0,
	   READINESS               = 1,
           SENSING_START_TIME      = 2,
           CURRENT_SENSING_TIME    = 3,
           QUALITY                 = 4,
           START_ORBIT             = 5,
           PROCESSING_STATE        = 6,
           CPU_USAGE               = 7,
           NO_TYPE                 = 8
	} e_TypeStatus;

	// Liste des Id Logique des statuts (cf fichier Status.cfg)
	typedef enum {
	   PPFversion                 = 0,
	   OPSReadiness               = 1,
	   L1a_SensingStartTime       = 2,
	   L1b_SensingStartTime       = 3,
	   L1c_SensingStartTime       = 4,
	   ENG_SensingStartTime       = 5,
	   VER_SensingStartTime       = 6,
	   L1a_CurrentSensingTime     = 7,
	   L1b_CurrentSensingTime     = 8,
	   L1c_CurrentSensingTime     = 9,
	   ENG_CurrentSensingTime     = 10,
	   VER_CurrentSensingTime     = 11,
           L1a_InstantaneousQuality   = 12,
           L1b_InstantaneousQuality   = 13,
           L1c_InstantaneousQuality   = 14,
           ENG_InstantaneousQuality   = 15,
           VER_InstantaneousQuality   = 16,
           L1a_StartOrbit             = 17,
           L1b_StartOrbit             = 18,
           L1c_StartOrbit             = 19,
           ENG_StartOrbit             = 20,
           VER_StartOrbit             = 21,
           L1a_ProcessingState        = 22,
           L1b_ProcessingState        = 23,
           L1c_ProcessingState        = 24,
           ENG_ProcessingState        = 25,
           VER_ProcessingState        = 26,
           CPUusage                   = 27,
           NONE                       = 28
	} e_ListeStatus;

	// Indice des valeurs possibles de OPSReadiness dans CMN_HKTM_READINESS
	typedef enum {
	   IDLE                = 0,
	   WORKING             = 1,
	   RECOVERABLE_ERROR   = 2,
	   UNRECOVERABLE_ERROR = 3 /* FA144 */
	} e_ReadinessState;

	// Indice des valeurs possibles de ProcessingState dans CMN_HKTM_PROCESSING_STATE
	typedef enum {
	   OK      = 0,
	   FAILURE = 1 /* FA144 */
	} e_ProcessingState;
	
        typedef int t_IdStatus;

        //
	// Constructeur
	//
	MP__HKTMStatus (e_ListeStatus statusId=NONE);
	//
	// Destructeur
	//
        virtual ~MP__HKTMStatus();

	//
	// Accesseurs
	//
        e_ListeStatus GetIdStatus() const {return m_IdStatus;};
        void SetIdStatus (const e_ListeStatus idStatus);

        e_TypeStatus GetType () {return m_TypeStatus;};

        e_Validity GetValidity (){return m_validity;};
        void SetValidity (e_Validity validity) {m_validity=validity;};

        e_Availability GetAvailability (){return m_available;};
        void SetAvailability (e_Availability availability)
                             {m_available=availability;};

        e_CMN_CGSSpacecraft GetService (){return m_service;};
        void SetService (e_CMN_CGSSpacecraft service)
             {m_service=service;
	      m_ParamName.RemplacerValeurAncienne ("xxx",CMN_CGS_SERVICES[service]); };

        
        t_Chaine GetParamName()const {return m_ParamName;};
        void SetParamName(const t_Chaine name)  { 
                                       m_ParamName = name;};

        t_Chaine GetValeurChaine() const { return m_ValeurChaine;};
        void SetValeurChaine(const t_Chaine ValeurChaine)  { 
                                       m_ValeurChaine = ValeurChaine;};

//  ----------------------------------------------------------------
// |   Types               |  Valeurs                               |
// |-----------------------|----------------------------------------|
// | PPF version           |  String                                |
// |                       |                                        |
// | Readiness             |  CMN__CGSCst :: CMN_HKTM_READINESS     |
// |                       |                                        |
// | Start time            |  String : YYYMMddHHmmssZ               |
// |                       |                                        |
// | Current sensing time  |  String : YYYMMddHHmmssZ               |
// |                       |                                        |
// | Instantaneous quality |  0..100 (cf definition DA17 �3.2.4)    |
// |                       |                                        |
// | Start Orbit Number    |  number                                |
// |                       |                                        |
// | Processing State      |  CMN__CGSCst::CMN_HKTM_PROCESSING_STATE|
// |                       |                                        |
// | Cpu Usage             |  0..100 (cf definition DA17 �3.2.4)    |
//   ----------------------------------------------------------------


	//
	// Methodes publiques
	//           
        void Clear() {
                         m_validity=VALID;
                         m_available=NOT_AVAILABLE;
                         m_service = UNKNOWN_SERVICE;
                         m_ParamName.erase();
                         m_ValeurChaine.erase();
                     };
	

	void Serialiser (MSG__Message& message);
	MSG__Message Serialiser();
	void Deserialiser(const MSG__Message& message);

/*=================================================================*/
protected:
	//
	// Methodes protegees
	//

	//
	// Attributs proteges
	//

/*=================================================================*/
private:

	//
	// Methodes privees
	//

	//
	// Attributs prives
	//

        // Validite du HKTM Status (par defaut toujours a VALID)
        e_Validity m_validity;

        // Disponibilite du status
        e_Availability  m_available;
	
	// Identificateur unique du status
	e_ListeStatus m_IdStatus;

        // Type du Status
        e_TypeStatus m_TypeStatus;

	// Nom du Parametre
	t_Chaine  m_ParamName;

        // Service
        e_CMN_CGSSpacecraft m_service;

        // Valeur du Status
	t_Chaine m_ValeurChaine;
	
};

#endif
