static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/MP__Spacecraft.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ ---------------------------------------------------------------------------
//@ Composant : MP
//@ CLASSE : MP__Spacecraft
//@ 
//@ Commentaire :
//@    Ce module gere la classe MP__Spacecraft
//@
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 22/07/2002 : PBR
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

//
// Fichiers include
//
#include "ERR__Exception.h"
#include "CMN__Config.h"
#include "MP__Message.h"
#include "MP__Spacecraft.h"


//
// Version Classe
//
const  t_Chaine MP__Spacecraft::VERSION_CLASSE = "1.0";


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : MP__Spacecraft
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
MP__Spacecraft::MP__Spacecraft() 
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~MP__Spacecraft
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
MP__Spacecraft::~MP__Spacecraft()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Serialiser
//@
//@ Description : Serialisation de l'objet
//@
//@ Parametres :
//@               (SORTIE) message : Message reseau.
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MP__Spacecraft::Serialiser( MSG__Message& message)
{
   CMN__Configuration config;
   t_Chaine resultat;
   
   
   //@ Ecriture de la version de la classe
   config.EcrireValeur("SECTION_ENTETE","CHAMP_CLASSE",
                       VERSION_CLASSE);
		       
   //@ Ecriture du type SPACECRAFT
   config.EcrireValeur("SPACECRAFT","VALEUR",m_Spacecraft);
		       
   //@ Sauvegarde de la serialisation dans une chaine
   config.SauverChaine(resultat);
   
   //@ Sauvegarde du bloc dans le message
   message.SetBloc (resultat.c_str(),resultat.size()+1);
    
   //@ Sauvegarde du type dans le message
   message.SetType(MP__Message::EVT_SPACECRAFT);
}


//@ ------------------------------------------------------------------*/
//@ Methode : Serialiser
//@
//@ Description : Serialisation de l'objet
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               message : Message reseau
//@
//@ ------------------------------------------------------------------*/
MSG__Message MP__Spacecraft::Serialiser()
{
  MSG__Message message;
  
  //@ Serialisation de l'objet
  Serialiser(message);
  
  //@ Retourne le message
  return (message);
}


//@ ------------------------------------------------------------------*/
//@ Methode : Deserialiser
//@
//@ Description : Deserialisation de l'objet
//@
//@ Parametres :
//@               (ENTREE) message : message reseau contenant un objet
//@                                  MP__Spacecraft serialise.
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MP__Spacecraft::Deserialiser(const MSG__Message& message)
{
  t_Chaine bloc (message.GetBloc());
  CMN__Configuration config;
  t_Chaine version;
  t_Chaine temp;
  
  //@ SI le type du message est bien EVT_SPACECRAFT
  if (message.GetType() == MP__Message::EVT_SPACECRAFT) 
    {
      
      //@ lecture de la version de la classe
      config.ChargerChaine(bloc);
      config.LireValeur("SECTION_ENTETE","CHAMP_CLASSE",
			version);
      
      //@ lecture du Spacecraft
      config.LireValeur("SPACECRAFT","VALEUR", temp);
      m_Spacecraft = (e_CMN_CGSSpacecraft) int(temp);
  }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : SetSpacecraft
//@
//@ Description : This method stores the spacecraft according to the
//@               string given as input parameter
//@
//@ Parametres :
//@               (ENTREE) spacecraft name
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK si l'attribut a ete mis a jour 
//@               CMN__STATUT_NOK sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut MP__Spacecraft::SetSpacecraft(t_Chaine spacecraft)
{
  int iType=0;
  t_CMN__Booleen typeFound = CMN__FAUX;


  //@ TANT QUE service type not found
  while (iType<NbrSpacecraft && typeFound==CMN__FAUX)
    {
      //@ Compare type enum to spacecraft type given as parameter
      if (spacecraft == CMN_CGS_SERVICES [iType])
	{
	  m_Spacecraft=(e_CMN_CGSSpacecraft)iType;
	  typeFound=CMN__VRAI;
	}
      iType++;		
    }
  //@ FIN TANT QUE

  //@ Compute status execution according to finding result
  if (typeFound==CMN__VRAI)
    {return CMN__STATUT_OK;}
  else
    {return CMN__STATUT_NOK;}
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetSpacecraftStr
//@
//@ Description : Provide the current spcacraft in ASCII format
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Spacecraft in ASCII format
//@
//@ ------------------------------------------------------------------*/
t_Chaine MP__Spacecraft::GetSpacecraftStr()
{
  return CMN_CGS_SERVICES [m_Spacecraft];
}
