static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/MP__StageTM.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ ---------------------------------------------------------------------------
//@ Composant : MP
//@ CLASSE : MP__StageTM
//@
//@ Commentaire :
//@    Cette classe encapsule et gere les informations echangees entre
//@    le MP et le WOM liees aux fichiers de donnees produits.
//@
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 22/07/2002 : PBR
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

//
// Fichiers include
//
#include "ERR__Exception.h"
#include "CMN__Config.h"
#include "CFG__Configuration.h"
#include "MP__Message.h"
#include "MP__StageTM.h"


//
// Definition des constantes 
//

// Version Classe
const t_Chaine MP__StageTM::VERSION_CLASSE = "1.0";

// Adresse d'envoi
const t_Chaine MP__StageTM::ADR_INPUT      = "MP_INPUT";


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : Serialiser
//@
//@ Description : Serialisation de l'objet
//@
//@ Parametres :
//@               (SORTIE) message : Message reseau
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MP__StageTM::Serialiser(MSG__Message& message)
{
   CMN__Configuration config;
   t_Chaine resultat;
   
   
   //@ Ecriture de la version de la classe
   config.EcrireValeur("SECTION_ENTETE","CHAMP_CLASSE",
                       VERSION_CLASSE);

   //@ Ecriture des informations generales d'une commande :		       
   //@   - le groupe
   //@   - l'identificateur de la cmde
   config.EcrireValeur("CMDE_GROUP","VALEUR", this->GetCommandGroup());
   config.EcrireValeur("IDENTIFICATEUR","VALEUR",this->GetCommandId());

   //@ Ecriture des informations de la commande :
   //@   - l'identificateur de la commande StageTM
   //@   - la category du produit
   //@   - le nom du fichier produit
   config.EcrireValeur("STAGE_ID","VALEUR",this->GetStageCategory());
   config.EcrireValeur("STAGE_CAT","VALEUR",this->GetStageId());
   config.EcrireValeur("FILENAME","VALEUR",this->GetDeliveredFilename());

   //@ Sauvegarde de la serialisation dans une chaine
   config.SauverChaine(resultat);
   
   //@ Sauvegarde du bloc dans le message
   message.SetBloc (resultat.c_str(),resultat.size()+1);
    
   //@ Sauvegarde du type dans le message
   message.SetType(MP__Message::EVT_STAGE_TM);

  //@ Positionnement de l'adresse de publication du message si
  //@ elle n'est pas encore renseignee
  if( message.GetAdresse()[0] == 0 )
    {
      message.SetAdresse(CFG__Configuration::GetAdresse(ADR_INPUT));
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : Serialiser
//@
//@ Description : Serialisation de l'objet
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               message : Message reseau
//@
//@ ------------------------------------------------------------------*/
MSG__Message MP__StageTM::Serialiser()
{
  MSG__Message message;
  
  //@ Serialisation de l'objet
  Serialiser(message);
  
  //@ Retourne le message
  return (message);
}


//@ ------------------------------------------------------------------*/
//@ Methode : Deserialiser
//@
//@ Description : Deserialisation de l'objet
//@
//@ Parametres :
//@               (ENTREE) message : message reseau contenant un objet
//@                                  MP__Command serialise
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MP__StageTM::Deserialiser(const MSG__Message& message)
{
  t_Chaine bloc (message.GetBloc());
  CMN__Configuration config;
  t_Chaine version;
  t_Chaine temp;

  //@ SI le type du message est bien EVT_STAGE_TM
  if (message.GetType() == MP__Message::EVT_STAGE_TM)
    {
      //@ lecture de la version de la classe
      config.ChargerChaine(bloc);
      config.LireValeur("SECTION_ENTETE","CHAMP_CLASSE",
                         version);
      //@ lecture des informations generales d'une commande :
      //@   - le groupe
      //@   - l'identificateur de la cmde
      config.LireValeur("CMDE_GROUP","VALEUR", temp);
      this->SetCommandGroup((e_CMN__CommandGroup)int(temp));
      config.LireValeur("IDENTIFICATEUR","VALEUR",temp);
      this->SetCommandId((t_CMN__CommandId)int(temp));

      //@ lecture des informations de la commande :
      //@   - l'identificateur de la commande StageTM
      //@   - la category du produit
      //@   - le nom du fichier produit
      config.LireValeur("STAGE_ID","VALEUR", temp);
      this->SetStageCategory ((e_CMN__StageCategory)int(temp));
      config.LireValeur("STAGE_CAT","VALEUR", temp);
      this->SetStageId ((e_CMN__StageId)int(temp));
      config.LireValeur("FILENAME","VALEUR", temp);
      this->SetDeliveredFilename ((t_CMN__DeliveredFile)temp);
    }
  //@ FIN SI
}
