static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/SD__Init.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ ---------------------------------------------------------------------------
//@ Composant : CMN
//@ CLASSE : SD__Init
//@
//@ Commentaire :
//@    Ce fichier contient la classe SD__Init, qui permet de gerer le
//@    message de fin d'initialisation du composant SD
//@
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

//
// Fichiers include
//

// STL

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN__Config.h"
#include "CFG__Configuration.h"
#include "MP__Message.h"
#include "SD__Message.h"
#include "SD__Init.h"


//
// Definition des constantes
//
const t_Chaine SD__Init::VERSION_CLASSE = "1.0";
const t_Chaine SD__Init::SECTION_ENTETE = "SECTION_ENTETE";
const t_Chaine SD__Init::CHAMP_CLASSE   = "CHAMP_CLASSE";
const t_Chaine SD__Init::CHAMP_ETAT     = "ETAT";
const t_Chaine SD__Init::VALEUR         = "VALEUR";
const t_Chaine SD__Init::ADR_INPUT      = "WOM_INPUT";


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD__Init
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD__Init::SD__Init()
{
  m_Etat = SD_INIT_OK;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD__Init
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD__Init::~SD__Init()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Deserialiser
//@
//@ Description : Construit un objet de la classe SD__Init a partir
//@               du contenu du message passe en parametre
//@
//@ Parametres :
//@               (ENTREE) message : Message contenant l'objet
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD__Init::Deserialiser(const MSG__Message& message)
{
  t_Chaine           bloc(message.GetBloc());
  CMN__Configuration config;
  t_Chaine           version("");
  t_Chaine           temp("");
  

  //@ SI le type du message est bien evenement INIT_SD
  if (message.GetType() == SD__Message::EVT_INIT_SD)
    {
      //@ lecture de la version de la classe
      config.ChargerChaine(bloc);
      
      //@ lecture de la version de la classe
      config.ChargerChaine(bloc);
      config.LireValeur(SECTION_ENTETE, CHAMP_CLASSE, version);
      
      //@ lecture de l'etat de l'initialisation
      config.LireValeur(CHAMP_ETAT, VALEUR, temp);
      m_Etat = (e_ListeStatus) ((int)temp);
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : Serialiser
//@
//@ Description : Construit un message a partir de la description de
//@               l'objet courant
//@
//@ Parametres :
//@               (SORTIE) MSG__Message : Message reseau
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD__Init::Serialiser(MSG__Message& message)
{
  CMN__Configuration config;
  t_Chaine           resultat("");


  //@ Ecriture de la version de la classe
  config.EcrireValeur(SECTION_ENTETE, CHAMP_CLASSE, VERSION_CLASSE);

  //@ Ecriture du type de traitement
  config.EcrireValeur(CHAMP_ETAT, VALEUR, m_Etat);
   
  //@ Sauvegarde de la serialisation dans une chaine
  config.SauverChaine(resultat);
   
  //@ Sauvegarde du bloc dans le message
  message.SetBloc(resultat.c_str(), resultat.size()+1);
    
  //@ Sauvegarde du type dans le message
  message.SetType(SD__Message::EVT_INIT_SD);

  //@ Positionnement de l'adresse de publication du message si
  //@ elle n'est pas encore renseignee
  if( message.GetAdresse()[0] == 0 )
    {
      message.SetAdresse(CFG__Configuration::GetAdresse(ADR_INPUT));
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : Serialiser
//@
//@ Description : Construit un message a partir de la description de
//@               l'objet courant
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Message contenant la description de l'objet
//@
//@ ------------------------------------------------------------------*/
MSG__Message SD__Init::Serialiser()
{
  //@ Declaration d'un message
  MSG__Message message;
  
  //@ Appel de la serialisation
  Serialiser(message);
  
  //@ Retour du message
  return message;
}
