// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/SD__Message.h,v 1.3 2011/11/25 10:17:39 dpauly Exp $
//@ ---------------------------------------------------------------------------
//@ Composant : CMN
//@ CLASSE : SD__Message
//@
//@ Commentaire :
//@    Fichier d'interface de la classe SD__Message, qui definit les
//@    types de messages echanges entre le WOM et le SD et lies aux
//@    traitements executes par le SD. Cette classe definit aussi les
//@    types de messages JDB geres par le SD
//@
//@ ---------------------------------------------------------------------------
//@ HISTORIQUE
//@ ---------------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Creation du fichier
//@ ---------------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-071-CN : 10/11/2003 : JL.PASCAL
//@ l'OPS ne detecte pas les absences de donnees si des dummy les remplacent 
//@ des messages d'information sont donc ajoutes a la detection et a la creation
//@ IdentifierLignes et InitLecture
//@ fiche de correction correspondante : IA-2100-FC-071-THA
//@ ---------------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-144-CN : 21/09/2004 : B. MASSART
//@ Problemes lors du portage d'OPS vers Solaris par Meteo France
//@ Voir IA-2100-FC-144-THA.
//@ ---------------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-271-CN : 22/06/2005 : JL.PASCAL
//@ prise en compte du mode transition
//@ fiche de correction correspondante : IA-2100-CR-271-THA
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-2100-CR-292-CNE : 10/03/2006 : B.MASSART
//@ Function SD_ALG_AVHRR_GetSubSatLatLon is added to set up 
//@ MPHR_SUBSAT_LATITUDE/LONGITUDE_START/END fields in Level1 products.
//@ See IA-2100-CR-292-THA.
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ ---------------------------------------------------------------------------
//@ DM-ID : IA-FT-3353(DM-2100-OPS) : 22/05/2017 : MPE
//@ Gestion Dummy MDR et MDR en overlap => Msg et sortie en erreur de l'OPS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ ---------------------------------------------------------------------------

#ifndef SD__Message_h
#define SD__Message_h

//
// Fichiers include
//

// STL

// Standard

// Projet
#include "CMN__Commun.h"
#ifdef __cplusplus
#include "MSG__Message.h"
#endif

//
// Liste des ensembles de messages EVT ou TRA disponibles pour les
// Classes et Modules du process SD
//
// Rappel : un ensemble contient au maximum 99 messages
//          les messages sont definis dans FRW/src/SD.msg
//

    typedef enum {
      // Set pour les packages qui implementent l'architecture d'accueil des
      // algorithmes; packages : ACT, EVT, FRW, GES, SRV, TAC
      SD_SET = 1,

      // Set pour les classes du package DON qui gerent les donnees Statiques :
      // SD_DON_BanqueSpectrale, SD_DON_ConfAutre, SD_DON_ConfStable, SD_DON_Contexte,
      // SD_DON_GestConfAutre, SD_DON_SVM, D_DON_OSV, SD_DON_OBTUTC, 
      // SD_DON_InventaireConf
      SD_CONF_SET = 2,

      // Set pour les classes du package DON qui gerent les donnees d'entree :
      // SD_DON_GranuleAVHRR, SD_DON_GranuleN0, SD_DON_LigneAVHRR, SD_DON_LigneN0,
      // SD_DON_PaquetN0
      SD_DATA_SET = 3,

      // Set pour les classes du package DON qui gerent les produits generes et
      // les donnees de facon generales : SD_DON_GestDonnees, SD_DON_Traitement,
      // SD_DON_CasDegLigne, SD_DON_GestCasDeg, SD_DON_GestDonnees,
      // SD_DON_GestGranule, SD_DON_Granule, SD_DON_GranuleN1, SD_DON_LigneN1
      SD_PRODUCT_SET = 4,

      // Set pour les algorithmes de la classe : SD_ALG_AVHRR
      SD_ALG_AVHRR_SET = 5,

      // Set pour les algorithmes de la classe : SD_ALG_ISRFEM
      SD_ALG_ISRFEM_SET = 6,

      // Set pour les algorithmes de la classe : SD_ALG_Image
      SD_ALG_IMAGE_SET = 7,

      // Set pour les algorithmes de la classe : SD_ALG_Monitoring
      SD_ALG_MONITORING_SET = 8,

      // Set pour les algorithmes de la classe : SD_ALG_Spectre
      SD_ALG_SPECTRE_SET = 9

    } e_ListeSDSetMessageJDB;

#ifdef __cplusplus

class SD__Message {

    /*================================================================*/

 public:
    
    ////////////////////////////////
    // Type definitions
    ////////////////////////////////

    // Type des messages utilises par le SD
    typedef enum {
        // Envoye par SD:
        EVT_INIT_SD      = MSG__Message::APPLICATIF_SD + 101, 
        // Recu et envoye par SD:
        EVT_TMT_SD       = MSG__Message::APPLICATIF_SD + 102, 
        // Recu par SD:
        EVT_ARRET_TMT_SD = MSG__Message::APPLICATIF_SD + 103 /* FA144 */
    } e_TypeMessage;
    
    //
    // Type des messages JDB utilises par le SD
    //
    typedef enum {
        MSG_SD_GENERIC               =  1,
        MSG_SD_START                 =  2,
        MSG_SD_STOP_OK               =  3,
        MSG_SD_STOP_ON_ERROR         =  4,
        MSG_UNEXPECTED_MSG           =  5,
        MSG_UNEXPECTED_EXP           =  6,
        MSG_UNEXPECTED_TYP           =  7,
        MSG_SD_MESSAGE               =  8,
        MSG_SD_ERROR                 =  9,
        MSG_SD_TMT_START_REFUSED     =  11,
        MSG_SD_TMT_STOP_REFUSED      =  12,
        MSG_SD_TMT_NOT_FOUND         =  13,
        MSG_SD_TMT_BEGIN             =  14,
        MSG_SD_TMT_END               =  15,
        MSG_SD_TMT_ENDED_ON_ERROR    =  16,
        MSG_SD_TAC_NOT_FOUND         =  17,
        MSG_SD_TAC_BEGIN             =  18,
        MSG_SD_TAC_END               =  19,
        MSG_SD_TAC_ERROR             =  20,
        MSG_SD_ACT_NOT_FOUND         =  21,
        MSG_SD_ACT_BEGIN             =  22,
        MSG_SD_ACT_END               =  23,
        MSG_SD_ECH_TAC_NOT_FOUND     =  24,
        MSG_SD_FILE_READ_ERROR       =  25,
        MSG_SD_FILE_WRITE_ERROR      =  26,
        MSG_SD_FILE_OPEN_ERROR       =  27,
        MSG_SD_ENV_UNDEFINED         =  28,
        MSG_GES_EXCEPTION_RAISED     =  29,
        MSG_FIN                      =  30 // to mark the set end
    } e_TypeMessageJDB;

    //
    // Liste des messages JDB du set  SD_CONF_SET
    //
    typedef enum {
        MSG_CONF_LOAD_ERR            =  1,
        MSG_CONF_LOAD_OK             =  2,
        MSG_CONF_NO_FILE             =  3,
	MSG_CONF_WRITE_OK	     =  4,
	MSG_CONF_WRITE_KO	     =  5,
	MSG_CONF_PTSI_DIFF	     =  6,
        MSG_CONF_SVM_UNAVAILABLE     =  7,
	MSG_CONF_VERSION_ERROR       =  8,
	MSG_CONF_IDCS_DIFF	     =  9,
	MSG_CONF_IDBS_DIFF	     =  10
    } e_CONFMessageJDB;

    // Liste des messages JDB du set  SD_DATA_SET
    //
    typedef enum {
        MSG_DATA_LOAD_AVHRR_ERR      =  1,
        MSG_DATA_START_DATE_INFO     =  2,
        MSG_DATA_AVHRR_NOT_USED      =  3,
        MSG_DATA_OBTUTC_UNAVAILABLE  =  4,
        MSG_DATA_OSV_DEFAULT         =  5,
        MSG_DATA_MODE_HAS_CHANGED    =  6,
        MSG_DUMMY_DATA_DETECTED      =  7,
        MSG_DUMMY_DATA_GENERATED      =  8,
        MSG_DATA_TRANSITION_DETECTED      =  9, //CR271
        MSG_DUMMY_NEXT_MDR_INCONSITENT  = 10 // FT 3353
    } e_DATAMessageJDB;

    // Liste des messages JDB du set  SD_PRODUCT_SET
    //
    typedef enum {
        MSG_PROD_CAS_DEG             =  1,
	MSG_PROD_CAS_PERMANENT	     =  2,
	MSG_PROD_CONF_CHARGEE	     =  3
    } e_PRODMessageJDB;

    // Listes des messages JDB des sets ALGO sont definies en fin de fichier
    // en dehors de la zone C++
 
    ////////////////////////////////
    // Constructeur
    ////////////////////////////////
    SD__Message();
    
    ////////////////////////////////
    // Destructeur
    ////////////////////////////////
    virtual ~SD__Message();

    ////////////////////////////////
    // Accesseurs
    ////////////////////////////////

    ////////////////////////////////
    // Methodes publiques
    ////////////////////////////////

    
    /*================================================================*/

 protected:

    ////////////////////////////////
    // Methodes protegees
    ////////////////////////////////

    ////////////////////////////////
    // Attributs proteges
    ////////////////////////////////


    /*================================================================*/

 private:
    
    ////////////////////////////////
    // Methodes privees
    ////////////////////////////////

    ////////////////////////////////
    // Attributs prives
    ////////////////////////////////
};
#endif

    // Liste des messages JDB du set  SD_ALG_AVHRR_SET
    //
    typedef enum {
        MSG_ALG_AVHRR_41_CCS_ALRT_TRAIT_CANAUX = 1, 
       	MSG_ALG_AVHRR_41_CCS_ALRT_NB_PIXAV_NUL = 2,
       	MSG_ALG_AVHRR_41_CCS_ALRT = 3,
       	MSG_ALG_AVHRR_40_IAC_ALRT = 4,
       	MSG_ALG_AVHRR_44_GEO_ALRT = 5,
        MSG_ALG_AVHRR_GetSubSatLatLon_ALRT = 6 /* CR292 */
    } e_AVHRRMessageJDB;

    // Liste des messages JDB du set  SD_ALG_ISRFEM_SET
    //
    typedef enum {
        MSG_ISRF_XXX                 =  1,
        MSG_ISRF_YYY                 =  2,
        MSG_ISRF_ZZZ                 =  3
    } e_ISFRMessageJDB;

    // Liste des messages JDB du set  SD_ALG_IMAGE_SET
    //
    typedef enum {
        MSG_IMAG_XXX                 =  1,
        MSG_IMAG_YYY                 =  2,
        MSG_IMAG_ZZZ                 =  3
    } e_IMAGMessageJDB;

    // Liste des messages JDB du set  SD_ALG_MONITORING_SET
    //
    typedef enum {
        MSG_MONI_XXX                 =  1,
        MSG_MONI_YYY                 =  2,
        MSG_MONI_ZZZ                 =  3
    } e_MONIMessageJDB;


    // Liste des messages JDB du set  SD_ALG_SPECTRE_SET
    //
    typedef enum {
        MSG_ALG_SPECTRE_30_FTB_ALRT_MOINDRES_CARRES         = 1, 
        MSG_ALG_SPECTRE_35_S1B_ALRT_INIT_SPLINE             = 2, 
        MSG_ALG_SPECTRE_35_S1B_ALRT_INTERP_SPLINE           = 3, 
        MSG_ALG_SPECTRE_37_S1C_ALRT_FCT_AMORTISST           = 4, 
        MSG_ALG_SPECTRE_37_S1C_ALRT_ZERO_PADDING            = 5, 
        MSG_ALG_SPECTRE_37_S1C_ALRT_INIT_FFT_DIRECTE        = 6, 
        MSG_ALG_SPECTRE_37_S1C_ALRT_CALCUL_FFT_DIRECTE      = 7, 
        MSG_ALG_SPECTRE_37_S1C_ALRT_PREPARE_INTERFEROGRAM   = 8, 
        MSG_ALG_SPECTRE_37_S1C_ALRT_INIT_FFT_INVERSE        = 9, 
        MSG_ALG_SPECTRE_37_S1C_ALRT_CALCUL_FFT_INVERSE      = 10,
        MSG_ALG_SPECTRE_37_S1C_ALRT_ARRANGT_SPECTRE_1C      = 11, 
        MSG_ALG_SPECTRE_XX_INITPROD_ALRT_INTERP_FCS         = 12, 
        MSG_ALG_SPECTRE_XX_INITPROD_ALRT_INTERP_PLANCKBBT   = 13, 
        MSG_ALG_SPECTRE_XX_INITPROD_ALRT_INTERP_FCS_SURECH  = 14
    } e_SPECMessageJDB;

#endif
