// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/ENV/src/FMES_FICHIER_MESSAGES.c,v 1.3 2011/11/25 10:17:39 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : ENV
//@ MODULE : FMES_FICHIER_MESSAGES
//@
//@ Commentaire :
//@    Fichier contenant le module FMES_FICHIER_MESSAGES
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 05/03/1997 : THALES-IS
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL

// Standard
// FT-2765
#include <string.h>
// ...FT-2765
#include <nl_types.h>
#include <locale.h>

// Projet
#include "FMES_FICHIER_MESSAGES.h"
#include "FPARA_FICHIER_PARAMETRES.h"
#include "TYCST.h"


//
// Definition des constantes 
//

//=====================================================================
//                        TYPES INTERNES
//=====================================================================

// Type definissant les etats possibles de l'objet
typedef enum {
  FMES_NonInitialise  = 0,
  FMES_Initialise      = 1
} e_FMES_Etat;


//=====================================================================
//                        VARIABLES STATIQUES
//=====================================================================

// Descripteur du fichier catalogue
static nl_catd s_FMES_Catfd = (nl_catd) -1;

// Descripteur du fichier catalogue
static e_FMES_Etat s_FMES_Etat = FMES_NonInitialise;


//======================================================================
// Declaration des fonctions internes
//======================================================================


//======================================================================
// Declaration des variables globales
//======================================================================


//======================================================================
//					FONCTIONS EXTERNES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Procedure : LireMessage 
//@
//@ Description : Grace au numero de sous-ensemble de messages et au
//@               numero de message precises, cette fonction retourne
//@               un pointeur sur un buffer interne contenant le
//@               message associe, stocke dans le buffer interne
//@               contenant le message associe, stocke dans le
//@               catalogue
//@
//@ Parametres :
//@               (ENTREE) id_message_set
//@               (ENTREE) id_message
//@               (SORTIE) message
//@
//@ Valeur de retour :
//@               e_FMES_Status
//@
//@ ------------------------------------------------------------------*/
extern
e_FMES_Status
FMES_LireMessage(const TYCST__T_int2 id_message_set, 
		 const TYCST__T_int2 id_message, 
		 TYCST__T_pt_pt_char message)
{
  // Status retour fonction
  e_FMES_Status retour;

  // message par defaut
  static TYCST__T_char msg_def[] = "MESSAGE PAR DEFAUT";

 
  //@ Initialiser status de retour a NOK
  retour = FMES_Nok ;
  
  //@ Recuperer dans message le numero message id_message du 
  //@ sous-ensemble de messages id_message_set dans le descripteur
  //@ du fichier catalogue
  
  //@ Appel fonction NLS "catgets"
  *message =catgets(s_FMES_Catfd ,id_message_set ,id_message ,msg_def);
  
  //@ SI lecture message reussie ALORS
  //@     Initialiser status de retour a OK
  //@ FIN SI
  if ( strcmp(*message ,msg_def ) != 0 ) retour =  FMES_Ok;
  
  
  //@ Retourner status de retour
  return(retour);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : InitSet
//@
//@ Description : Initialise les messages du catalogue
//@
//@ Parametres :
//@               (ENTREE) id_message_set
//@               (ENTREE) id_debut
//@               (ENTREE) id_fin
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
extern
void
FMES_InitSet(const TYCST__T_int2 id_message_set, 
	     const TYCST__T_int2 id_debut, 
	     const TYCST__T_int2 id_fin) 
{
  // Message par defaut
  static TYCST__T_char msg_def[] = "MESSAGE PAR DEFAUT";
  
  char *message;
  int i; 
  

  //@ POUR tous les messages du SET
  for ( i = id_debut; i < id_fin; i++) 
    {
      //@ Les lire une fois pour initialiser le catalogue
      message =catgets(s_FMES_Catfd ,id_message_set ,i,msg_def);
    }
  //@ FIN POUR
}


//@ ------------------------------------------------------------------*/
//@ Procedure : Initialiser
//@
//@ Description : Initialise le catalogue
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               e_FMES_Status
//@
//@ ------------------------------------------------------------------*/
extern
e_FMES_Status
FMES_Initialiser(void)
{
  // Status retour fonction
  e_FMES_Status retour = FMES_Ok;

  // Nom du catalogue de messages
  TYCST__T_pt_char nom_catal;

  // Retour fonctions objet FPARA
  FPARA__T_status ret_fpara = FPARA__C_ok;
  
 
  //@ SI etat=non initialise ALORS
  //@   Lire dans l'environnement le nom du catalogue de messages
  //@   SI lecture nom catalogue erronee ALORS
  //@     erreur lecture nom catalogue
  //@   SINON
  //@     SI le catalogue n'a pas deja ete ouvert ALORS
  //@       Ouvrir le catalogue et le charger totalement en memoire
  //@       Appel fonction NLS "catopen" avec nom du catalogue en
  //@       parametre
  //@       SI ouverture catalogue reussie ALORS
  //@           Initialiser status de retour a OK
  //@           Conserver le descripteur du fichier catalogue
  //@       FIN SI
  //@     SINON
  //@       erreur catalogue deja ouvert
  //@     FIN SI
  //@   FIN SI
  //@   SI pas d'erreur de traitement ALORS etat = initialise
  //@ FIN SI
  //@ Retourner status de retour
  
  if ( s_FMES_Etat == FMES_NonInitialise )
    {
      // La langue des messages est definie a partir des variables
      // d'environnement
      setlocale(LC_MESSAGES,"");
#ifdef IDEBUG
      ret_fpara = FPARA__Lire_valeur("DEBUG_CAT" ,&nom_catal ,NULL);
#else
      ret_fpara = FPARA__Lire_valeur(FMES_CLE_CAT_MESS ,&nom_catal ,NULL);
#endif     

      if ( ret_fpara != FPARA__C_ok ) retour = FMES_LectCatKo ;
      else
	{
	  if ( s_FMES_Catfd == (nl_catd) -1 )
	    {
	      s_FMES_Catfd = catopen(nom_catal ,NL_CAT_LOCALE);
	      if ( s_FMES_Catfd == (nl_catd) -1 ) retour = FMES_OpenCatKo ;
	    }
	  else   retour = FMES_CatDejaOuvert;
	}
      if ( retour == FMES_Ok ) s_FMES_Etat = FMES_Initialise ;
    }
  
  return(retour);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : Terminer
//@
//@ Description : Termine une session d'acces au catalogue des
//@               messages en fermant celui-ci et en liberant la
//@               memoire utilisee par celui-ci
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               e_FMES_Status
//@
//@ ------------------------------------------------------------------*/
extern
e_FMES_Status
FMES_Terminer(void)
{
  //@ SI etat = initialise  ALORS
  if ( s_FMES_Etat == FMES_Initialise )
    {
      //@ Fermer le catalogue
      //@ Appel de la fonction NLS "catclose" avec le descripteur
      //@ du fichier catalogue
      //@ Reinit descripteur du fichier catalogue a -1
      //@ Etat = non initialise
      catclose(s_FMES_Catfd);
      s_FMES_Catfd = (nl_catd) -1;
      s_FMES_Etat = FMES_NonInitialise ;
    }
  //@ FIN SI
  
  //@ Retourner status de retour OK
  return(FMES_Ok); 
}


//======================================================================
//					FONCTIONS INTERNES
//======================================================================
