// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/ENV/src/FPARA_FICHIER_PARAMETRES.c,v 1.3 2011/11/25 10:17:39 dpauly Exp $
//@ --------------------------------------------------------------------/
//@ Composant : ENV
//@ MODULE : FPARA_FICHIER_PARAMETRES
//@
//@ Commentaire :
//@    Fichier contenant le module FPARA_FICHIER_PARAMETRES
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 05/03/1997 : THALES-IS
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL

// Standard
#include <stdio.h>
// FT-2765
#include <string.h>
// ...FT-2765

// Projet
#include "FPARA_FICHIER_PARAMETRES.h"


//
// Definition des constantes 
//

// Chaine de caractere definissant un parametre de type ENTIER signe code
// sur 2 octets
#define FPARA__C_chaine_int2 "int2"

// Chaine de caractere definissant un parametre de type ENTIER signe code
// sur 4 octets
#define FPARA__C_chaine_int4 "int4"

// Chaine de caractere definissant un parametre de type REEL signe code
// sur 4 octets 
#define FPARA__C_chaine_real4 "real4"

// Chaine de caractere definissant un parametre de type REEL signe code
// sur 8 octets
#define FPARA__C_chaine_real8 "real8"

// Chaine de caractere definissant un parametre de type CHAINE
#define FPARA__C_chaine_string "string"

// Chaine de caractere definissant un parametre de type BOOLEEN code sur
// 1 octet
#define FPARA__C_chaine_boolean "boolean"

// Chaine de caractere definissant un parametre de type ENTIER non signe
// code sur 2 octets
#define FPARA__C_chaine_uint2 "uint2"

// Chaine de caractere definissant un parametre de type ENTIER non signe
// code sur 4 octets
#define FPARA__C_chaine_uint4 "uint4"

// Suffixe rajoute au nom d'un parametre pour recuperer le type d'un
// parametre
#define FPARA__C_suffix_type "_TYPE"


//======================================================================
// Declaration des types internes
//======================================================================


//======================================================================
// Declaration des variables globales
//======================================================================


//======================================================================
// Declaration des fonctions internes
//======================================================================

static
FPARA__T_status
FPARA__Atoi_type(
		 const TYCST__T_pt_char chaine_type_param, 
		 FPARA__T_pt_type_param type_param
		 );


//======================================================================
//					FONCTIONS EXTERNES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Procedure : Lire_valeur
//@
//@ Description : L'operation permet de lire valeur d'un parametre
//@               statique, et son type.
//@               La valeur et le type sont lus dans les variables
//@               d'environnement initialisees dans le fichier
//@               parametre du client
//@
//@ Parametres :
//@               (ENTREE) nom_param
//@               (SORTIE) val_param
//@               (SORTIE) type_param
//@
//@ Valeur de retour :
//@               FPARA__T_status
//@
//@ ------------------------------------------------------------------*/
extern
FPARA__T_status
FPARA__Lire_valeur(const TYCST__T_pt_char nom_param,
		   FPARA__T_pt_val_param val_param,
		   FPARA__T_pt_type_param type_param)
{
  // Chaine type de parametre
  TYCST__T_pt_char  chaine_type_param; 

  // Chaine valeur du type de parametre
  TYCST__T_pt_char  chaine_val_type_param;  

  // Status de retour de la fonction
  FPARA__T_status   status;	

  
  status = FPARA__C_ok;

  //@ Lire la valeur de la variable d'environnement "nom_param"
  //@ SI nom_param inexistant
  if ( (*val_param = (FPARA__T_val_param)getenv(nom_param)) == NULL ) 
    {
      //@ Erreur parametre inconnu
      status = FPARA__C_err_param_inconnu;
    }  
  //@ FIN SI

  //@ SI l'appelant veut connaitre le type du parametre
  if ( (status == FPARA__C_ok) && (type_param != NULL))
    {
      //@ Reservation memoire pour la chaine a lire
      chaine_type_param = (TYCST__T_pt_char) TYCST__MALLOC(strlen(nom_param) +
							   strlen(FPARA__C_suffix_type) +1);
      
      //@ SI allocation reussie
      if (chaine_type_param != NULL ) 
	{
	  //@ Constitution de la chaine a lire
	  sprintf(chaine_type_param,"%s%s",nom_param,FPARA__C_suffix_type);
       
	  //@ Lire la valeur de la variable d'environnement
	  //@ "nom_param"_TYPE

	  //@ SI nom_param_TYPE inexistant
	  if ( (chaine_val_type_param = 
		(TYCST__T_pt_char)getenv(chaine_type_param)) == NULL )
	    {
	      //@ Erreur type absent
	      status=FPARA__C_err_type_absent; 
	    } 
	  //@ SINON nom_param_TYPE existe
	  else
	    {
	      //@ Convertir la chaine type en une valeur entiere
	      status=FPARA__Atoi_type(chaine_val_type_param,type_param) ;
	    }
	  //@ FIN SI

	  //@ Liberation de la chaine a lire
	  free(chaine_type_param);
	}
      //@ SINON erreur allocation
      else
	{
	  //@ Erreur allocation memoire impossible
	  status=FPARA__C_err_alloc; 
	}
      //@ FIN SI  allocation
    }
  //@ FIN SI l'appelant veut connaitre le type du parametre
  
  //@ Retourner status
  return(status);
  
}


//======================================================================
//					FONCTIONS INTERNES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Procedure : Atoi_type
//@
//@ Description : Convertir une chaine de caractere definissant le
//@               type d'un parametre en une valeur entiere
//@
//@ Parametres :
//@               (ENTREE) chaine_type_param
//@               (SORTIE) type_param
//@
//@ Valeur de retour :
//@               FPARA__T_status
//@
//@ ------------------------------------------------------------------*/
static
FPARA__T_status
FPARA__Atoi_type(const TYCST__T_pt_char chaine_type_param,
		 FPARA__T_pt_type_param type_param)
{
  typedef struct {
    TYCST__T_pt_char    chaine_type; // Type sous la forme d'une chaine
    FPARA__T_type_param type;        // Type sous la forme d'un entier
  } T_conv ;	// Type conversion
  

  // Tableau de conversion
  static T_conv tab_conv[] = { 	
    {FPARA__C_chaine_int2,FPARA__C_int2},
    {FPARA__C_chaine_int4,FPARA__C_int4},
    {FPARA__C_chaine_real4,FPARA__C_real4},
    {FPARA__C_chaine_real8,FPARA__C_real8},
    {FPARA__C_chaine_string,FPARA__C_string},
    {FPARA__C_chaine_boolean,FPARA__C_boolean},
    {FPARA__C_chaine_uint2,FPARA__C_uint2},
    {FPARA__C_chaine_uint4,FPARA__C_uint4}
  };
  
  FPARA__T_status status;	// Status de retour de la fonction
  TYCST__T_int2    i;		// Indice de boucle
  TYCST__T_boolean trouve;	// Vrai si le type a ete trouve
  

  status = FPARA__C_ok;
   
  //@ Rechercher la chaine type de parametre dans la table de
  //@ conversion
  i=0;
  trouve = TYCST__C_faux;
  while ( (! trouve) && (i < TYCST__NB_ELT(tab_conv)) )
    {
      trouve = !strcmp(tab_conv[i].chaine_type, chaine_type_param);
      if ( ! trouve ) 
	{
	  i++;
	}
    }

  //@ SI la chaine type existe dans la table de conversion
  if ( trouve ) 
    {
      //@ Recuperer le type de parametre  dans la table
      *type_param = tab_conv[i].type; 
    }
  //@ SINON la chaine n'existe pas
  else 
    {
      //@ Erreur type inconnu
      status = FPARA__C_err_type_inconnu;
    }
  //@ FIN SI
   
  return(status); 
}

