//@ $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/ENV/src/JDB_JOURNAL_DE_BORD.c,v 1.3 2011/11/25 10:17:39 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : ENV
//@ MODULE : JDB_JOURNAL_DE_BORD
//@
//@ Commentaire :
//@    Fichier contenant le module JDB_JOURNAL_DE_BORD
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 05/03/1997 : Serge Bobin
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 16/07/2002 : PBR
//@ Adaptation a l'OPS : rajout de l'Id du message comme parametre
//@                      d'appel des callback
//@                      JDB_InitialiserCallback, PositionnerCallback
//@                      JDB__Ecrire_msg_fmt, JDB__Extraire_attributs
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-159-CN : 26/05/2005 : JL. PASCAL
//@ modif suite a portage linux
//@ fiche de correction correspondante : IA-2100-FC-159-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-184-CN : 18/10/2006 : C.MERIGUET
//@ Codage des fonctions reentrantes
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------
//
// Fichiers include
//

// STL

// Standard
#include <stdio.h>
#include <stdlib.h> 
#include <string.h>
#include <time.h>
#include <locale.h>

// Projet
#include "JDB_JOURNAL_DE_BORD.h"
#include "FMES_FICHIER_MESSAGES.h"
#include "FPARA_FICHIER_PARAMETRES.h"
#include "TYCST.h"


//
// Definition des constantes 
//

// Longueur maximum d'une date ascii
#define JDB_LG_MAX_DATE_ASCII           30

// Taille maximum par defaut du fichier de trace (octets)
#define JDB_TAILLE_MAX_FIC_TRACE_DEFAUT 50000

// Taille maximum par defaut du fichier de trace (octets)
#define JDB_PARAM_TAILLE_MAX_FIC_TRACE  "JDB_TAILLE_MAX_FIC_TRACE"


//=====================================================================
//                        TYPES INTERNES
//=====================================================================

// Type date ascii
typedef TYCST__T_char t_JDB_DATE_ASCII[JDB_LG_MAX_DATE_ASCII +1] ;


//=====================================================================
//                        VARIABLES STATIQUES
//=====================================================================

// Liste des parametres communs a plusieurs messages JDB
static
t_JDB_ListeParam s_JDB_ParamCommun;

// Liste des noms logiques et de leur callback associes
static
t_JDB_ListeCallback *s_JDB_ListCallback=NULL;


//======================================================================
// Declaration des fonctions internes
//======================================================================

static
e_JDB_Status
JDB_Substituer(
	       const TYCST__T_pt_char chaine1,
	       const TYCST__T_pt_char chaine2,
	       const TYCST__T_pt_char chaine3,
	       TYCST__T_pt_char chaine4
	       );

static
e_JDB_Status
JDB__Extraire_attributs(
	t_JDB_Message        message,
        TYCST__T_pt_pt_char  msgId,
        TYCST__T_pt_pt_char  grav,
	TYCST__T_pt_pt_char  objet,
        TYCST__T_pt_pt_char  list_dest,
	TYCST__T_pt_pt_char  libelle
        );

static
int
JDB__Comparer_nom_param(
			const void *,
			const void *
			);


static
e_JDB_Status
JDB__Ecrire_fichier(
	const t_JDB_Message 	  message
	);

static
e_JDB_Status
JDB__Modifier_param(
	const t_JDB_PtParam old_param,
	const t_JDB_PtParam new_param
	);

static
e_JDB_Status
JDB__Ajouter_param(
	const t_JDB_PtParam 	    new_param,
	const t_JDB_pt_ListeParam pt_liste
	);

static
e_JDB_Status
JDB__Ecrire_msg_fmt(
	const t_JDB_Message    message,
        const TYCST__T_pt_char msgId,
        const TYCST__T_pt_char grav,	
	const TYCST__T_pt_char objet,
        const TYCST__T_pt_char list_dest
	);

static
e_JDB_Status
JDB__Maj_param_msg(
	const TYCST__T_pt_char mess_in, 
	const t_JDB_pt_ListeParam liste_param,
	TYCST__T_pt_char mess_out 
	);

static
e_JDB_Status
JDB__Dater(
	   const TYCST__T_pt_char      mess_in, 
	   TYCST__T_pt_char      mess_out
	   );


//======================================================================
// Declaration des variables globales
//======================================================================


//======================================================================
//					FONCTIONS EXTERNES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Procedure : InitialiserMessage
//@
//@ Description : Initialise un message vide
//@
//@ Parametres :
//@               (SORTIE) messageJDB
//@
//@ Valeur de retour :
//@               e_JDB_Status
//@
//@ ------------------------------------------------------------------*/
extern
e_JDB_Status
JDB_InitialiserMessage(t_JDB_pt_ListeParam messageJDB)
{
  TYCST__T_int2 compteur;
  e_JDB_Status codeRetour = JDB_Ok;


  //@ Initialise le nombre de parametres du message JDB
  messageJDB->nb = 0;
  
  //@ Initialise la liste des parametres
  for (compteur=0; compteur<JDB_NB_MAX_PARAM; compteur++)
    {
      messageJDB->tab[compteur].nom=NULL;
      messageJDB->tab[compteur].valeur=NULL;
    }

  return(codeRetour);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : StockerParametres
//@
//@ Description : Stocke un nouveau parametre (nom,valeur)
//@
//@ Parametres :
//@               (ENTREE) NomParametre : Nom du parametre a stocker
//@               (ENTREE) ValeurParametre : Valeur du parametre a
//@                                          stocker
//@               (SORTIE) MsgJdb : Message JDB mis a jour avec le
//@                                 nouveau parametre
//@
//@ Valeur de retour :
//@               e_JDB_Status
//@
//@ ------------------------------------------------------------------*/
extern
e_JDB_Status
JDB_StockeParametre( TYCST__T_pt_char NomParametre,
		     TYCST__T_pt_char ValeurParametre,
		     t_JDB_pt_ListeParam MsgJdb)
{
  TYCST__T_int4 numeroParametre;
  e_JDB_Status codeRetour = JDB_Ok;
  
  
  //@ Recuperation du numero du parametre
  numeroParametre = MsgJdb->nb;
  
  //@ Test si on peut encore stocker un parametre
  if (numeroParametre == JDB_NB_MAX_PARAM)
    {
      codeRetour= JDB_ERR_TROP_DE_PARAMETRES;
      return(codeRetour);
    }
  
  //@ Stockage du nom du parametre
  MsgJdb->tab[numeroParametre].nom = 
    (TYCST__T_char *) TYCST__MALLOC ((strlen(NomParametre)+1));
  strncpy(MsgJdb->tab[numeroParametre].nom, 
          NomParametre, 
          strlen(NomParametre));
  MsgJdb->tab[numeroParametre].nom[strlen(NomParametre)] = '\0';
  
  //@ Stockage de la valeur du parametre
  MsgJdb->tab[numeroParametre].valeur = 
    (TYCST__T_char *) TYCST__MALLOC ((strlen(ValeurParametre)+1));
  strncpy(MsgJdb->tab[numeroParametre].valeur, 
	  ValeurParametre,
	  strlen(ValeurParametre));
  MsgJdb->tab[numeroParametre].valeur[strlen(ValeurParametre)] = '\0';

  
  //@ Incrementation du nombre de parametres
  MsgJdb->nb++;
  
  
  return (codeRetour);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : VideMessage
//@
//@ Description : Vide le message (parametres)
//@
//@ Parametres :
//@               (ENTREE/SORTIE) MsgJdb
//@
//@ Valeur de retour :
//@               e_JDB_Status
//@
//@ ------------------------------------------------------------------*/
extern
e_JDB_Status
JDB_VideMessage(t_JDB_pt_ListeParam MsgJdb)
{
  e_JDB_Status codeRetour = JDB_Ok;
  TYCST__T_int4 nbreParametres;
  TYCST__T_int4 compteur;


  //@ Recuperation du nombre de parametres
  nbreParametres = MsgJdb->nb;
  
  //@ Vide tous les parametres
  for (compteur=0; compteur<nbreParametres; compteur++)
    {
      TYCST__FREE(MsgJdb->tab[compteur].nom);
      TYCST__FREE(MsgJdb->tab[compteur].valeur);
    }
  
  //@ Le compteur est mis a zero
  MsgJdb->nb = 0;

  return (codeRetour);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : EcrireMessage
//@
//@ Description : Permet d'identifier la partie textuelle d'un
//@               message applicatif repertorie dans un fichier
//@               et de completer celui-ci en prenant en compte
//@               si necessaire des parametres dynamiques, puis
//@               de l'envoyer au Journal de bord via un 
//@               Datagroup specifique
//@
//@ Parametres :
//@               (ENTREE) id_ensemble
//@               (ENTREE) id_mess
//@               (ENTREE) pt_liste_param
//@
//@ Valeur de retour :
//@               e_JDB_Status
//@
//@ ------------------------------------------------------------------*/
extern
e_JDB_Status
JDB_EcrireMessage(const TYCST__T_int2 id_ensemble, 
		  const TYCST__T_int2 id_mess, 
		  const t_JDB_pt_ListeParam pt_liste_param)
{
  static    t_JDB_Message     	last_message=""; // Dernier message recu
  
  TYCST__T_pt_char   	message_b;	// Message brut
  t_JDB_Message         copie_message_b;// Copie de message brute
  t_JDB_Message     	message_f1;	// Message formate
  t_JDB_Message     	message_f2;	// Message formate
  t_JDB_Message     	message_f3;	// Message formate
  e_FMES_Status     	fmes_status; 	// Status des operation de FMES
  FPARA__T_status     	fpara_status; 	// Status des operation de FPARA
  e_JDB_Status      	status; 	// Status de retour de l'operation
  FPARA__T_val_param 	valeur;		// Valeur d'un parametre
  TYCST__T_pt_char      msgId;          // Identificateur du message
  TYCST__T_pt_char      grav;		// Gravite du message
  TYCST__T_pt_char 	dest;		// Destination
  TYCST__T_pt_char      obj;            // Objet
  TYCST__T_pt_char   	libelle;	// Libelle
  static TYCST__T_boolean first=TYCST__C_vrai;// Vrai si premier passage
  
  
  status = JDB_Ok;
  
  //@ SI premier appel
  if ( first ) 
    {
      //@ Initialiser l'acces au catalogue des messages
      if ((fmes_status=FMES_Initialiser()) != FMES_Ok ) 
	{
	  status = JDB_ERR_ACCES_FIC_MES;
	}
      
      //@ SI pas d'erreur */
      if ( status == JDB_Ok ) 
	{
	  //@ Premier appel = faux
	  first = TYCST__C_faux;
	}
      //@ FIN SI
    }
  //@ FIN SI premier appel
  
  //@ SI pas d'erreur
  if ( status == JDB_Ok )
    {
      //@ Recuperer le texte du message brut
      fmes_status=FMES_LireMessage(id_ensemble, id_mess, &message_b);
      
      //@ SI recuperation impossible
      if (fmes_status != FMES_Ok ) 
	{
	  //@ Erreur message inexistant
	  status = JDB_ERR_MESSAGE_INEXISTANT ;
	}
      //@ SINON
      else 
	{
	  //@ Faire une copie du message brut
	  strncpy(copie_message_b,message_b,sizeof(copie_message_b)-1);
	  copie_message_b[sizeof(copie_message_b)-1] = '\0';
	}
      //@ FIN SI recuperation impossible
    }
  //@ FIN SI pas d'erreur
  
  //@ SI pas d'erreur
  if ( status == JDB_Ok ) 
    {
      //@ Analyser message brute pour en extraire les infos :
      //@ - Id message
      //@ - gravite ( WARNING, ALERT, INFO, NOTICE )
      //@ - destination ( FICHIER, NONE, CSF )
      //@ - libelle
      status = JDB__Extraire_attributs(copie_message_b,
				       &msgId, &grav, &obj, &dest, &libelle);
    }
  //@ FIN SI pas d'erreur
  
  //@ SI pas d'erreur
  if ( status == JDB_Ok ) 
    {
      //@ Remplacer dans le message toute occurence d'un param
      //@ commun par sa valeur
      JDB__Maj_param_msg(libelle, &s_JDB_ParamCommun, message_f1);

      //@ Remplacer dans le message toute occurence d'un param
      //@ specifique, par sa valeur
      JDB__Maj_param_msg(message_f1, pt_liste_param, message_f2);

      //@ SI le message formate est different du dernier message
      //@ recu
      if ( strcmp(message_f2,last_message) )
	{
	  //@ Le message courant devient le dernier message
	  strcpy(last_message,message_f2);
	  
	  //@ Dater le message
	  JDB__Dater(message_f2,message_f3); 
	
	  //@ Ecrire le message formate en tenant compte 
	  //@ des infos extraites
	  JDB__Ecrire_msg_fmt(message_f3, msgId,grav,obj,dest);
	}
      //@ SINON on poubellise le message
      //@ FIN SI
    }
  //@ FIN SI pas d'erreur

  fflush(stderr);

  //@ Retourner le status
  return (status);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : InitialiserCallback
//@
//@ Description : Initialiser les couples (nom logique,callback)
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               e_JDB_Status
//@
//@ ------------------------------------------------------------------*/
extern
e_JDB_Status
JDB_InitialiserCallback()
{
  e_JDB_Status codeRetour = JDB_Ok;
  TYCST__T_int4 compteur;


  //@ Allocation de la place necessaire
  s_JDB_ListCallback = 
    (t_JDB_ListeCallback *) TYCST__MALLOC (sizeof(t_JDB_ListeCallback));

  //@ Initialiser le nombre de couples (nom logique,callback)
  s_JDB_ListCallback->NbCallback=0;

  //@ Initialise la liste des couples (nom logique,callback)
  for (compteur=0; compteur<JDB_NB_MAX_NOM_LOGIQUE_CALLBACK; compteur++)
    {
      s_JDB_ListCallback->tab[compteur].nomLogique = NULL;
      s_JDB_ListCallback->tab[compteur].callback = 
	(TYCST__T_void (*)(TYCST__T_pt_char,
			   TYCST__T_pt_char,
			   TYCST__T_pt_char,
			   const t_JDB_Message)) 0;
    }


  return(codeRetour);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : PositionnerCallback
//@
//@ Description : Associe un callback a un nom logique
//@
//@ Parametres :
//@               (ENTREE) NomLogique : Destination dans un fichier
//@                                     message
//@               (ENTREE) Callback : Callback associe au nom logique
//@
//@ Valeur de retour :
//@               e_JDB_Status
//@
//@ ------------------------------------------------------------------*/
extern
e_JDB_Status
JDB_PositionnerCallback(TYCST__T_pt_char NomLogique, 
			TYCST__T_void (*Callback)(TYCST__T_pt_char,
						  TYCST__T_pt_char,
						  TYCST__T_pt_char,
						  const t_JDB_Message))
{
  e_JDB_Status codeRetour = JDB_Ok;
  TYCST__T_int4 numeroCallback;


  //@ Recuperation du numero du callback
  numeroCallback=s_JDB_ListCallback->NbCallback;

  //@ Stockage du nom logique
  s_JDB_ListCallback->tab[numeroCallback].nomLogique=
    (TYCST__T_char *) TYCST__MALLOC ((strlen(NomLogique)+1));
  strncpy(s_JDB_ListCallback->tab[numeroCallback].nomLogique,
	  NomLogique,
	  strlen(NomLogique));
  s_JDB_ListCallback->tab[numeroCallback].nomLogique[strlen(NomLogique)]
    = '\0';


  //@ Stockage du callback
  s_JDB_ListCallback->tab[numeroCallback].callback = Callback;

  //@ Mise a jour du nombre de callback
  s_JDB_ListCallback->NbCallback++;
  

  return(codeRetour);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : InitSet
//@
//@ Description : Cette operation permet d'initialiser un set de
//@               message
//@
//@ Parametres :
//@               (ENTREE) idSet
//@               (ENTREE) msgDeb
//@               (ENTREE) msgFin
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
extern
void
JDB__InitSet(int idSet, int msgDeb, int msgFin)
{
  //@ Initialisation du catalogue
  FMES_Initialiser();
  
  //@ Initialisation du set
  FMES_InitSet((TYCST__T_int2)idSet, (TYCST__T_int2)msgDeb,
	       (TYCST__T_int2)msgFin); 
} 


//======================================================================
//					FONCTIONS INTERNES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Procedure : Substituer
//@
//@ Description : Cette operation permet de substituer la chaine
//@               "chaine1" par la chaine "chaine2" dans la chaine
//@               "chaine3" en placant le resultat de la substitution
//@               dans la chaine "chaine4"
//@
//@ Parametres :
//@               (ENTREE) chaine1
//@               (ENTREE) chaine2
//@               (ENTREE) chaine3
//@               (SORTIE) chaine4
//@
//@ Valeur de retour :
//@               e_JDB_Status
//@
//@ ------------------------------------------------------------------*/
e_JDB_Status
JDB_Substituer(const TYCST__T_pt_char chaine1,
	       const TYCST__T_pt_char chaine2,
	       const TYCST__T_pt_char chaine3,
	       TYCST__T_pt_char chaine4)
{
  TYCST__T_pt_char pt;      // Pointeur de parcours de chaine3
  TYCST__T_pt_char pt_cur;  // Pointeur sur la chaine chaine1 dans chaine3
  e_JDB_Status status;	    // Status de retour de la fonction
  

  status = JDB_Ok;

  //@ SI chaine1 est vide
  if ( strlen (chaine1) == 0 )
    {
      //@ Rien a faire
      return(JDB_Ok);
    }
  //@ FIN SI

  chaine4[0] = '\0';
  pt=chaine3;
  
  //@ Rechercher chaine1 dans chaine3
  pt_cur=strstr(pt,chaine1);

  //@ SI non trouvee
  if ( pt_cur == NULL ) 
    {
      //@ Copier chaine3 dans chaine4
      strcpy(chaine4,chaine3);
      status = JDB_Ok;
   }
  //@ SINON
  else 
    {
      //@ TANT QUE trouve
      while ( pt_cur != NULL )
	{
	  //@ Ajouter les caracteres, de pt (chaine3) jusqu'a chaine1
	  //@ dans chaine4
          strncat(chaine4 , pt , pt_cur - pt );
   
	  //@ Ajouter chaine2 a chaine4
          strcat(chaine4,chaine2);
	  
	  //@ Incrementer pt jusqu'a chaine1 + lg(chaine1)
          pt = pt_cur + strlen(chaine1);
	  
	  //@ Rechercher chaine1 dans pt
          pt_cur=strstr(pt,chaine1);
	}
      //@ FIN TANT QUE

      //@ Recopier les caracteres restant dans chaine4
      strcat(chaine4,pt);
    }
  //@ FIN SI
  
  return (status);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : Ecrire_msg_fmt
//@
//@ Description : Ecrire un message formate en tenant compte des infos
//@               suivantes:
//@                 - gravite =  WARNING, ALERT, INFO, NOTICE
//@                 - destination =  CSF, FICHIER, NONE
//@
//@ Parametres :
//@               (ENTREE) message : Message formate
//@               (ENTREE) msgId : Identificateur message
//@               (ENTREE) grav : Gravite du message
//@               (ENTREE) objet : Objet du message
//@               (ENTREE) dest : Destination
//@
//@ Valeur de retour :
//@               e_JDB_Status
//@
//@ ------------------------------------------------------------------*/
static
e_JDB_Status
JDB__Ecrire_msg_fmt(const t_JDB_Message message,
		    const TYCST__T_pt_char msgId,
		    const TYCST__T_pt_char grav,
		    const TYCST__T_pt_char objet, 
		    const TYCST__T_pt_char dest	)
{
   TYCST__T_pt_char cur_dest;	// Destination courante
   TYCST__T_pt_char niv_erreur;
   e_JDB_Status status;	        // Status de retour de la fonction
   TYCST__T_pt_char dest1;
   TYCST__T_int4 compteur;
       

   status = JDB_Ok;
   dest1=dest;

   //@ Conversion de la chaine gravite en numero de message CSF
   if ( !strcmp ( grav, JDB_G_STR_INFO )) {
     niv_erreur   = JDB_G_STR_INFO;
   }
   else if ( !strcmp ( grav, JDB_G_STR_WARNING )) { 
     niv_erreur   = JDB_G_STR_WARNING;
   }
   else if ( !strcmp ( grav, JDB_G_STR_NOTICE )) { 
     niv_erreur   = JDB_G_STR_NOTICE;
   }
   else if ( !strcmp ( grav, JDB_G_STR_ALERT )) { 
     niv_erreur   = JDB_G_STR_ALERT;
   }
   else {
     niv_erreur=JDB_G_STR_UNKNOWN;
   }
   
#ifdef OPS_DEBUG
   printf ("%s %s\n", niv_erreur, message);
#else
   //@ SI la liste des callback n'est pas initialisee
   if (!s_JDB_ListCallback) 
     {
       //@ Retour erreur
       status = JDB_ERR_LISTE_CALLBACK_NON_INITIALISE;
     }
   //@ SINON
   else
     {
       //@ POUR toutes les destinations
       //FA159
       char* lasts;
       while ( ((cur_dest =(TYCST__T_pt_char) strtok_r(dest1,JDB_SEP_LIST_ATTR,&lasts))
		!= NULL) && (status == JDB_Ok) ) 
	 {
	   compteur=0;
    
	   //@ TANT QUE la destination ne correspond pas a
	   //@ un nom logique stocke et que le tous les noms
	   //@ logiques ont ete consultes
       //FA159
//	   while ((strcmp(cur_dest,s_JDB_ListCallback->tab[compteur].nomLogique)
//		   != 0) && (compteur < s_JDB_ListCallback->NbCallback))
//	     {
//	       //@ Incrementer le compteur
//	       compteur++;
//	     }
	   while (compteur < s_JDB_ListCallback->NbCallback)
	     {
	     	if (strcmp(cur_dest,s_JDB_ListCallback->tab[compteur].nomLogique) == 0) break; 
	       //@ Incrementer le compteur
	       compteur++;
	     }
	   //@ FIN TANT QUE
	 
	   //@ SI la destination est non trouvee
	   if (compteur == s_JDB_ListCallback->NbCallback)
	     {
	       //@ Retour erreur
	       status = JDB_ERR_DESTINATION_NON_TROUVE;
	     }
	   //@ SINON
	   else
	     {
	       //@ Appeler le callback associe a la destination
	       s_JDB_ListCallback->tab[compteur].callback(msgId,
							  objet,
							  niv_erreur,
							  message);
	     }
	   //@ FIN SI
	   
	   dest1=NULL;
	 }
       //@ FIN POUR toutes les destinations
     }
   //@ FIN SI
#endif

   return(status);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : Extraire_attributs
//@
//@ Description : Analyser un message brute pour en extraire les
//@               attributs:
//@                 - Identificateur du message
//@                 - Gravite
//@                 - Surveillance
//@                 - Destination
//@
//@ Parametres :
//@               (ENTREE/SORTIE) message_in : Message en entree
//@               (SORTIE) msgId : Identificateur du msg
//@               (SORTIE) grav : Gravite du message
//@               (SORTIE) objet : Objet du message
//@               (SORTIE) list_dest : Liste de destination
//@               (SORTIE) libelle : Libelle du message
//@
//@ Valeur de retour :
//@               e_JDB_Status
//@
//@ ------------------------------------------------------------------*/
static
e_JDB_Status
JDB__Extraire_attributs(t_JDB_Message  message_in,
			TYCST__T_pt_pt_char  msgId,
			TYCST__T_pt_pt_char  grav,
			TYCST__T_pt_pt_char  objet,
			TYCST__T_pt_pt_char  list_dest,
			TYCST__T_pt_pt_char  libelle)
{
   e_JDB_Status    status;	// Status de retour de la fonction
   TYCST__T_pt_char token;	// Partie du message
   TYCST__T_pt_char attributs;	// Attributs du message

/* FA 184*/
   char * lasts;
   //@ Separer la partie attribut de la partie libelle
   attributs = strtok_r(message_in,JDB_SEP_ATTR_LIBELLE,&lasts);
/* ..FA 184*/

   status = JDB_Ok;

   //@ SI la partie attribut est vide
   if ( attributs == NULL ) 
     {
       //@ Retour erreur
       status = JDB_ERR_SEP_ABSENT;
     }
   //@ SINON
   else 
     {

/* FA 184*/
       *libelle=strtok_r(NULL,JDB_SEP_ATTR_LIBELLE,&lasts);
            
       //@ Extraire les attributs
       *msgId = strtok_r(attributs,JDB_SEP_TYPE_ATTR,&lasts);
       *grav = strtok_r(NULL,JDB_SEP_TYPE_ATTR,&lasts);
       *objet = strtok_r(NULL,JDB_SEP_TYPE_ATTR,&lasts);
       *list_dest = strtok_r(NULL,JDB_SEP_TYPE_ATTR,&lasts);
/* ..FA 184*/
       
       //@ SI le libelle n'est pas present
       if (*libelle == NULL )
	 {
	   //@ Retour erreur
	   status = JDB_ERR_LIBELLE_ABSENT;
	 }
       //@ FIN SI
      
       //@ SI pas d'erreur
       if ( status == JDB_Ok ) 
	 {
	   //@ Tester si tous les types d'attributs sont presents
	   if ((*grav == NULL) || (*list_dest == NULL)) 
	     {
	       status = JDB_ERR_ATTRIBUT_ABSENT;
	     }
	 }
       //@ FIN SI
       
       fflush(stderr);
       
     }
   //@ FIN SI

   //@ Retourner le status de la fonction
   return (status);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : Ecrire_fichier
//@
//@ Description : Ecrire un message dans un fichier de trace
//@
//@ Parametres :
//@               (ENTREE) message
//@
//@ Valeur de retour :
//@               e_JDB_Status
//@
//@ ------------------------------------------------------------------*/
static
e_JDB_Status
JDB__Ecrire_fichier(const t_JDB_Message 	  message)
{
  e_JDB_Status           status;	   // Status de retour de la fonction
  FPARA__T_status         fpara_status;   // Status de retour de FPARA
  FPARA__T_val_param 	   nom_fic ;       // Nom du fichier trace
  FPARA__T_val_param 	   str_taille_max ;// Chaine taille max du fichier trace
  static TYCST__T_int4    taille_max ;	   // Taille max du fichier trace
  
  static FILE             *pf;           // Pointeur fichier
  static TYCST__T_boolean first=TYCST__C_vrai;	// Vrai si premier passage dans la fonction
  
  
  status = JDB_Ok;
  
  //@ Lire le parametre "FICHIER_TRACE"
  fpara_status=FPARA__Lire_valeur(JDB_PARAM_FICHIER_TRACE, &nom_fic, NULL );

  //@ SI erreur de lecture
  if ( fpara_status !=  FPARA__C_ok ) 
    {
      //@ Sortie sur stdout
      pf = stdout;
      first = TYCST__C_faux;
    }
  //@ SINON
  else
    {
      //@ SI premiere fois
      if ( first )
	{
	  //@ Lire la taille maximum du fichier
          fpara_status=FPARA__Lire_valeur(JDB_PARAM_TAILLE_MAX_FIC_TRACE,
                                          &str_taille_max, NULL );
          if ( fpara_status !=  FPARA__C_ok )
	    {
	      taille_max = JDB_TAILLE_MAX_FIC_TRACE_DEFAUT;
	    }
          else
	    {
	      taille_max = TYCST__STRTOINT4(str_taille_max);
	    }

          //@ Ouverture du fichier trace
          pf = fopen(nom_fic,"w");

	  //@ SI erreur d'ouverture
          if ( pf == NULL ) 
	    {
	      //@ Retourner l'erreur
	      status = JDB_ERR_OUVERTURE_FICHIER;
	    }
	  //@ SINON
          else
	    {
	      first = TYCST__C_faux;
	      
	      //@ Non bufferisation des ecritures
	      setbuf(pf,NULL);
	    }
	  //@ FIN SI
	}
      //@ FIN SI
    }
  //@ FIN SI

  //@ SI pas d'erreur
  if ( status == JDB_Ok )
    {
      //@ SI on ecrit dans autre chose que la sortie standard
      if ( pf != stdout )
	{
	  //@ SI on va depasser la taille max
	  if ( ftell(pf) + strlen(message) > taille_max ) 
	    {
	      //@ Se repositionner au debut du fichier
	      fseek(pf, SEEK_SET,0);
	    } 
	  //@ FIN SI
	}
      //@ FIN SI stdout
      
      //@ Ecrire le message dans le fichier
      fprintf(pf,"%s\n",message);
   }
  //@ FIN SI
  
   return (status);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : Modifier_param
//@
//@ Description : Modifier la valeur d'un parametre
//@
//@ Parametres :
//@               (ENTREE) old_param
//@               (ENTREE) new_param
//@
//@ Valeur de retour :
//@               e_JDB_Status
//@
//@ ------------------------------------------------------------------*/
static
e_JDB_Status
JDB__Modifier_param(const t_JDB_PtParam old_param,
		    const t_JDB_PtParam new_param)
{
  e_JDB_Status  status;	    // Status de retour de la fonction
  TYCST__T_pt_char pt_val;  // Valeur d'un parametre
  

  status = JDB_Ok;
  
  //@ SI longueur de la nouvelle valeur > a l'ancienne
  if ( strlen(new_param->valeur) > strlen(old_param->valeur) ) 
    {
      //@ Retailler l'espace de l'ancienne valeur
      pt_val=(TYCST__T_pt_char)TYCST__REALLOC (old_param->valeur,
					       strlen( new_param->valeur ) +1);
      
      //@ SI erreur de retaillage
      if (pt_val== NULL)
	{ 
	  //@ Erreur pas assez de memoire
	  status = JDB_ERR_PAS_ASSEZ_DE_MEMOIRE;
	}
      //@ SINON retaillage ok
      else
	{
	  //@ Prise en compte du nouvel espace memoire
	  old_param->valeur=pt_val;
	}
      //@ FIN SI erreur de retaillage
   }
  //@ FIN SI longueur de la nouvelle valeur > a l'ancienne

   //@ SI pas d'erreur
   if ( status == JDB_Ok ) 
     {
       //@ Copier la nouvelle valeur a la place de l'ancienne
       strcpy(old_param->valeur, new_param->valeur);
     }
   //@ FIN SI pas d'erreur

   return(status);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : Ajouter_param
//@
//@ Description : Ajouter un nouveau parametre dans une liste
//@
//@ Parametres :
//@               (ENTREE) new_param
//@               (ENTREE) pt_liste
//@
//@ Valeur de retour :
//@               e_JDB_Status
//@
//@ ------------------------------------------------------------------*/
static
e_JDB_Status
JDB__Ajouter_param(const t_JDB_PtParam new_param,
		   const t_JDB_pt_ListeParam pt_liste)
{
   e_JDB_Status    status;	// Status de retour de la fonction
   TYCST__T_pt_char nom;	// Nom du parametre
   TYCST__T_pt_char valeur;	// Valeur du parametre


   status = JDB_Ok;

   //@ Tester la coherence des arguments
   if ( pt_liste == NULL )
     {
       return(JDB_ERR_ARG_INVALIDE);
     }

   //@ SI le nombre maximum de parametres dans la liste n'est pas
   //@ deja atteint
   if ( pt_liste->nb < TYCST__NB_ELT(pt_liste->tab) )
     {
       //@ Reserver memoire pour le nom et la valeur
       nom    = (TYCST__T_pt_char) TYCST__MALLOC(strlen(new_param->nom) +1 );
       valeur = (TYCST__T_pt_char) TYCST__MALLOC(strlen(new_param->valeur) +1 );
       
       //@ SI erreur allocation
       if (( nom == NULL ) || ( valeur == NULL ))
	 {
	   //@ Erreur pas assez de memoire
	   status = JDB_ERR_PAS_ASSEZ_DE_MEMOIRE;
	 }
       //@ SINON allocation reussie
       else 
	 {
	   //@ Copier le nom et la valeur dans l'espace alloue
	   strcpy(nom,new_param->nom);
	   strcpy(valeur,new_param->valeur);
	   
	   //@ Ajouter le parametre en fin de liste
	   pt_liste->tab[pt_liste->nb].nom    = nom;
	   pt_liste->tab[pt_liste->nb].valeur = valeur;
	   pt_liste->nb++;
	   
	   //@ Trier la liste
	   qsort(pt_liste->tab,pt_liste->nb,
		 sizeof(pt_liste->tab[0]),JDB__Comparer_nom_param);
	 }
       //@ FIN SI erreur allocation
   }
   //@ SINON nombre maximum de parametre deja atteint
   else 
     {
       //@ Erreur trop de parametres
       status = JDB_ERR_TROP_DE_PARAMETRES;
     }
   //@ FIN SI nombre maximunm de parametres


   return(status);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : Comparer_nom_param
//@
//@ Description : Comparer 2 parametres.
//@               Retourne 1 valeur > 0 si param_1->nom > param2->nom
//@               Retourne 1 valeur < 0 si param_1->nom < param2->nom
//@               Retourne 1 valeur = 0 si param_1->nom > param2->nom
//@
//@ Parametres :
//@               (ENTREE) param_1
//@               (ENTREE) param_2
//@
//@ Valeur de retour :
//@               TYCST__T_int4
//@
//@ ------------------------------------------------------------------*/
static
TYCST__T_int4
JDB__Comparer_nom_param(const void *param_1,
			const void *param_2)
{
  //@  Comparer les noms des 2 parametres en utilisant l'ordre ascii
  //@  Retourner le resultat
  return(strcmp(((const t_JDB_PtParam)param_1)->nom,
		((const t_JDB_PtParam)param_2)->nom));
}


//@ ------------------------------------------------------------------*/
//@ Procedure : Maj_param_msg
//@
//@ Description : Cette operation permet de remplacer toute occurence
//@               d'un parametre de la liste "liste_param" par sa
//@               valeur et de placer le resultat dans "mess_out"
//@
//@ Parametres :
//@               (ENTREE) mess_in
//@               (ENTREE) pt_liste_param
//@               (SORTIE) mess_out
//@
//@ Valeur de retour :
//@               e_JDB_Status
//@
//@ ------------------------------------------------------------------*/
static
e_JDB_Status
JDB__Maj_param_msg(const TYCST__T_pt_char mess_in, 
		   const t_JDB_pt_ListeParam pt_liste_param,
		   TYCST__T_pt_char mess_out)
{
   static t_JDB_Message mess_tmp1;	// Message temporaire
   static t_JDB_Message mess_tmp2;	// Message temporaire
   TYCST__T_pt_char      pt_mess_1;	// Pointeur message 1
   TYCST__T_pt_char      pt_mess_2;	// Pointeur message 2
   TYCST__T_pt_char      pt_mess_3;	// Pointeur message 3
   e_JDB_Status         status;	        // Status de retour de la fonction
   TYCST__T_int4         i;		// Indice de boucle
   

   status = JDB_Ok;

   //@ SI il existe une liste de parametres
   if ( pt_liste_param != NULL )
     {
       //@ Tester si le nb de parametres dans la liste est acceptable
      if ((pt_liste_param->nb < 0) || (pt_liste_param->nb > JDB_NB_MAX_PARAM))
	{
	  return(JDB_ERR_ARG_INVALIDE);
	}
   
      //@ Initialiser les messages temporaires avec le message
      //@ en entree
      strcpy(mess_tmp1,mess_in);
      strcpy(mess_tmp2,mess_in);
      strcpy(mess_out,mess_in);
      pt_mess_1 = mess_tmp1;
      pt_mess_2 = mess_tmp2;
   
      //@ POUR tous les parametres de la liste
      for ( i=0 ; i< pt_liste_param->nb; i++) 
	{
	  //@ Substituer le parametre courant par sa valeur
	  JDB_Substituer(pt_liste_param->tab[i].nom,   
			 pt_liste_param->tab[i].valeur,   
			 pt_mess_1,pt_mess_2);
	  
	  //@ Recopie du message obtenu dans le message en sortie
	  strcpy(mess_out,pt_mess_2);

	  //@ Swapper les buffers
	  pt_mess_3 = pt_mess_2;
	  pt_mess_2 = pt_mess_1;
	  pt_mess_1 = pt_mess_3;
	}
      //@ FIN POUR
   }
   //@ SINON pas de liste de parametres
   else 
     {
       //@ Recopier le message en entree dans le message en sortie
      strcpy(mess_out, mess_in);
   }

   //@ Retourner le status de retour
   return(status);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : Dater
//@
//@ Description : Cette operation permet de dater un message en
//@               rajoutant la date en tete du message selon le
//@               format indique par la variable d'environnement
//@               JDB_FORMAT_DATE.
//@               Se reporter au manuel de cftime(3C) pour
//@               connaitre les formats de conversion disponibles.
//@               La conversion %C est utilisee par defaut.
//@               Cette operation est sensible a la variable
//@               LC_TIME et prend donc en compte la langue lors
//@               de la conversion
//@
//@ Parametres :
//@               (ENTREE) mess_in
//@               (SORTIE) mess_out
//@
//@ Valeur de retour :
//@               e_JDB_Status
//@
//@ ------------------------------------------------------------------*/
static
e_JDB_Status
JDB__Dater(const TYCST__T_pt_char mess_in, 
	   TYCST__T_pt_char mess_out)
{
  e_JDB_Status        status;	        // Status de retour de la fonction
  FPARA__T_status      fpara_status;	// Status de retour de la fonction
  static FPARA__T_val_param   format;  // Format de la date
  time_t		date;           // Date courante
  t_JDB_DATE_ASCII	date_ascii;     // Date ascii        
  struct tm *		date_tm;	// Date dans une structure temps
  static TYCST__T_boolean	first = TYCST__C_vrai;// Vrai si premier appel
  
/* FA 184 */
      struct tm * CurrentTime;
/* ..FA 184*/
  
  status = JDB_Ok;
  
  // SI premiere fois
  if ( first ) 
    {
      first = TYCST__C_faux;
      
      //@ Prendre en compte la langue choisie pour la date
      setlocale(LC_TIME,"");
      
      //@ Recuperer le format de la date
      fpara_status = FPARA__Lire_valeur(JDB_PARAM_FORMAT_DATE, &format, NULL) ;
      if ( fpara_status != JDB_Ok )
	{
	  format = NULL;
	}
    }
  //@ FIN SI premiere fois
   
  //@ SI le format n'est pas nul
  if (format != NULL)
    {
      //@ Lire la date courante
      date = time(NULL);
/* FA 184 */
      date_tm = localtime_r(&date,CurrentTime);
/* ..FA 184*/

      //@ Convertir la date selon le format choisi
      if (strftime(date_ascii,sizeof(date_ascii)-1,format, date_tm) == 0)
	{
	  status = JDB_Nok;
	} 
      else
	{
	  //@ Concatener la date et le libelle du message
	  sprintf(mess_out,"%s %s",date_ascii,mess_in);
	}
    } 
  else
    {
      sprintf(mess_out,"%s",mess_in);
    }
  //@ FIN SI

  //@ Retourner le status de retour
  return(status);
}
