// $Header: /data/CVS/iasi/home/ops_gc/cvs-repository/OPS/CMN/src/MLA.h,v 1.3 2002/10/28 14:00:05 pbr Exp $
//@ ------------------------------------------------------------------*/
//@ Composant : CMN
//@ CLASSE : MLA
//@
//@ Commentaire :
//@    Ce fichier regroupe les constantes et les types utilises par les
//@    librairies MLA (en attendant la vrai librairie).
//@    
//@ ------------------------------------------------------------------*/
//@ HISTORIQUE
//@ VERSION : 08/07/2002 : P. BRANET
//@ Creation du fichier
//@
//@ Origine     Date        Auteur   Commentaire
//@ --------------------------------------------
//@ FA-ID :<N>  ../../..    ...
//@
//@ FIN-HISTORIQUE
//@ Pascal Brunel 21/11/2013 augmentation de la longueur des messages
//@ ------------------------------------------------------------------*/

#ifndef MLA_h
#define MLA_h


#ifdef __cplusplus
extern "C" {
#endif

/* Message protocol field seperator */

#define FIELD_SEPARATOR "|"

/* MLA_ERROR CODES DEFINITION - RETURN STATUSES */

/* Return OK*/

#define MLA_OK 0

/* Decoding Error */
#define DECOD_ERROR -1

#define NOTHING -1
#define NOT_CONNECTED -1
#define CONNECTED 0
#define NEW_CONNECTION 1

#define MLA_ERROR -1

#define TC_LOST -2
#define BACKLOG 1

typedef enum logique {mla_false=0, mla_true=1} mla_bool;

typedef struct timevalue {
	int sec;
	int usec;
} timeValue;



//
/* RETURN CODES DEFINITION - RETURN STATUSES */

/* Trying to use a function before API initialization or after termination */
/* Or error initializing the API */
#define API_NOT_INIT -1

/* Trying to initialize the API that is already initialized (Better try to Abort !!) */
#define API_INIT -2

/* Using an illegan facility name */
#define UNKNOWN_FACILITY -3

/* Future usage = Not able to connect to MLA */
/* Current usage = Unable to open a file for API I/O */
#define CONNECT_NOK -4

/* Future usage = Not able to disconnect from MLA */
/* Current usage = Unable to close a file for API I/O */
#define DISCONNECT_NOK -5

/* Unable to get environment variables variable */
#define ENV_VAR_NOK -6

/* Trying to use GetCmd or SendAck while not declared as a Main Process */
#define NOT_MP -7

/* No Command to process at this time */
#define API_NO_COMMAND -8

/* Illegal input to SendFunctionStatus or WriteLog 		*/
/* Length (paramName) > 35					*/
/* Length (eventText) > 80					*/
/* Only 0-9 a-z A-Z authorized in paramName and eventName	*/
/* use of '|' is prohibited in event string			*/

#define ILLEGAL_INPUT -9

#define TT_SCHEDULED  1 
#define TT_STARTED  2
#define TT_COMPLETED 3
#define TT_REJECTED 4
#define TT_ABORTED 5
#define IC_COMPLETED 10
#define IC_ABORTED 11

#define LENSTRG 1024 /* Maximum of the sent buffer */

#define EVT_NAM_LEN 31 /* Maximum of the event name length */
#define EVT_TXT_LEN 256 /* Maximum length of the event text */

#define TRA_NAM_LEN 31 /* Maximum of the trace name length */
#define TRA_TXT_LEN 256 /* Maximum length of the trace text */

#define PAR_NAM_LEN 35 /* Maximum of the param name length */
#define PAR_STR_LEN 256 /* Maximum of the param text arg length */

#define CMD_NAM_LEN 35 /* Maximum of the command name length */
#define CMD_STR_LEN 33 /* Maximum of the command arg length */
#define CMD_ARG_NBR 15 /* Maximum number of command arguments */

#define CMD_MAX_ELEM 24  /* Maximum element number in the TC string */

#define TIMEOUT 5 /* delay to execute this function */

/* SOCKETS COMMUNICATION */
#define BUFSIZE 200
#define NORMAL 0

/* Authorized interval times to accept TLM and EVT in seconds */
#define TIME_BEFORE 10
#define TIME_AFTER 3

/* Win32 or Unix Compilation options */
#ifdef WIN32
    #define MLA_EWOULDBLOCK WSAEWOULDBLOCK
    #define MLA_ENOTCONN WSAENOTCONN
    #ifdef BUILD_DLL
        #define EXPORT __declspec(dllexport)
    #else
        #define EXPORT __declspec(dllimport)
    #endif
#else
    #define MLA_EWOULDBLOCK EWOULDBLOCK
    #define MLA_ENOTCONN ENOTCONN
    #define EXPORT extern
#endif

typedef struct telem {
	char paramName[PAR_NAM_LEN];
	mla_bool validity;
	char valueString[PAR_STR_LEN];
} tmParam;

typedef struct event {
	char eventName[EVT_NAM_LEN ];
	char eventText[EVT_TXT_LEN];
} eventData;

typedef struct trace {
	char traceName[TRA_NAM_LEN ];
	char traceText[TRA_TXT_LEN];
} logData;

typedef struct cmd {
	int cmdNbr;
	char cmd[CMD_NAM_LEN];
	timeValue tBegin;
	timeValue tEnd;
	int iter;
	char cmdData[CMD_ARG_NBR][CMD_STR_LEN + 2];
} tcCmd;

//
// EXPORTED TYPES & DATA
//

extern int mla_Initialisation (char*, mla_bool);
extern int mla_Terminate ();
extern int mla_SendFunctionStatus (timeValue*, tmParam*);
extern int mla_LogEvent (timeValue*, eventData*);
extern int mla_LogTrace (timeValue*, logData*);
extern int mla_GetCommand (tcCmd *);
extern int mla_SendAck (int, int);

/* Specific encoding functions */
extern int mlaEncodStr (char*, char*);
extern int mlaEncodInt (char*, int);
extern int mlaEncodBool (char*, mla_bool);
extern int mlaEncodFloat (char*, float);
extern int mlaEncodTime (char*, timeValue*);


/* Specific Decoding functions */
extern int mlaDecodStr (char*, char*);
extern int mlaDecodInt (char*, int*);
extern int mlaDecodBool (char*, mla_bool*);
extern int mlaDecodFloat (char*, float*);
extern int mlaDecodTime (char*, timeValue*);


#ifdef __cplusplus
}
#endif



#endif
