#ifdef HASNT_SETENV

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "env.h"

/**
 * Environment variables management for Solaris
 * the man page of putenv is clear :
 * int putenv(char *string);
 *    The string argument should not be an automatic variable.  It
 *    should  be  declared static if it is declared within a func-
 *    tion because it cannot be automatically declared.  A  poten-
 *    tial  error  is  to  call  putenv()  with  a  pointer  to an
 *    automatic variable as the argument and to then exit the cal-
 *    ling function while string is still part of the environment.
 *
 * This means we have to manage memory ourselves. That's what the 
 * following code does.
 * FIXME: not MT safe
 */

typedef struct env_buffer {
  char* buffer;
  char* name;  
  struct env_buffer *next;
} env_buffer;

env_buffer *get_env_buffer() {
  env_buffer* eb;
  eb = ( env_buffer* )malloc( sizeof( env_buffer ) );
  if( ! eb ) { return NULL; }
  eb->next = NULL;
  eb->name = NULL;
  eb->buffer = NULL;
  return eb;
}

void free_env_buffer( env_buffer *eb ) {
  free( eb->buffer ); 
  free( eb->name ); 
  free( eb );
}

static env_buffer *EB = 0;

int setenv( const char *name, const char *value, int overwrite ) {
  env_buffer* eb;
  eb = EB;
  while( eb && strcmp( eb->name, name ) ) {
    eb = eb->next;
  }
  if( ! eb ) {
    eb = get_env_buffer();
    if( ! eb ) { return -1; }
    
    eb->name = ( char* )malloc( strlen( name ) + 1 );
    if( ! eb->name ) { return -1; }
    
    strcpy( eb->name, name );
    eb->next = EB;
    EB = eb;
  } else {
    if( ! overwrite ) { return 0; }
  }

  eb->buffer = ( char* )realloc( eb->buffer, strlen( name ) + strlen( value ) + 2 );
  /* + 2 is for the "=" and the terminating "\0" */
  if( ! eb->buffer ) { return -1; }
  
  sprintf( eb->buffer, "%s=%s", name, value );
  
  if( putenv( eb->buffer ) ) { return -1; };
  
  return 0;  
}


void unsetenv( const char *name ) {
  setenv( name, "", 1 );
}


#endif
