#ifndef __FORTRAN_LOCK_H
#define __FORTRAN_LOCK_H

#include <stdio.h>
#include <pthread.h>


#ifdef FORTRAN_NOT_THREADSAFE

#ifdef FORTRAN_NOT_THREADSAFE_VERBOSE

#define FORTRAN_LOCK() \
  do {                                                                                   \
    printf( "THREAD 0x%x => FORTRAN_LOCK " __FILE__ ":%d\n", pthread_self(), __LINE__ ); \
    fortran_lock();                                                                      \
    printf( "THREAD 0x%x <= FORTRAN_LOCK " __FILE__ ":%d\n", pthread_self(), __LINE__ ); \
  } while( 0 )

#define FORTRAN_UNLOCK() \
  do {                                                                                     \
    printf( "THREAD 0x%x => FORTRAN_UNLOCK " __FILE__ ":%d\n", pthread_self(), __LINE__ ); \
    fortran_unlock();                                                                      \
    printf( "THREAD 0x%x <= FORTRAN_UNLOCK " __FILE__ ":%d\n", pthread_self(), __LINE__ ); \
  } while( 0 )

#else

#define FORTRAN_LOCK() \
  do {                                                                                   \
    fortran_lock();                                                                      \
  } while( 0 )

#define FORTRAN_UNLOCK() \
  do {                                                                                     \
    fortran_unlock();                                                                      \
  } while( 0 )

#endif

#else

#define FORTRAN_LOCK() \
  do { } while( 0 )

#define FORTRAN_UNLOCK() \
  do { } while( 0 )

#endif

int fortran_lock();
int fortran_unlock();

#endif
