#ifdef OPS_USE_AAPP
	subroutine ops_earth_to_metop(
     >	   pos, vel, att_mode, att, 
     >	   nbpts, lat, lon, azim, elev, ierr) 
	
!+
!NOM/NAME  %M% version %I% on %G%        
!
!BUT/PURPOSE 
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!DESCRIPTION 
!	FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!      
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Apr 2004      no          A. Marsouin P.Brunel
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
	implicit none                                                     
!
!ARGUMENTS
!
!       Rs is the spacecraft fixed reference frame
!       Ra is the nominal attitude reference frame
!       Rg is the Greenwich reference frame
!
! input 
	double precision pos(3)		! satellite position in Rg  (km)
	double precision vel(3)		! satellite velocity in Rg  (km/s)
        Integer att_mode		! attitude mode
	double precision att(3,3)	! attitude error matrix		(rad)
	integer nbpts 			! number of points
	Double Precision lat(nbpts)	! latitudes 			(rad)
	Double Precision lon(nbpts)	! longitudes 			(rad)
! output
	Double Precision elev(nbpts)	! elevation angles in Metop ref.frame	(rad)
	Double Precision azim(nbpts)	! azimut angles	in Metop ref.frame	(rad)
	integer ierr			! return code 
!		0 = OK   
!  		9 = internal error, attitude mode not available
!		1 = some erroneous latitude values

!DECLARATIONS
!
! identification variable for "what"
      character*62 sccsid     
!
      data sccsid
     &/'%Z% %M% version %I% on %G%'/       
!
! Local parameters
!
	double precision pi	! pi
	parameter ( pi = 3.14159265358979311600D0 )
	
! Local variables	
	integer i			! loop variable
	double precision Mat_Ra_Rg(3,3)	! Ra to Rg conversion matrix
	Double Precision om_g(3)	! array of viewed points in Rg
	double precision sm_s(3)	! satellite - viewed point in Rs
	double precision projec		! intermediate variable
! 
!
!-
        ! calculates the Ra <--> Rg transformation
        !--------------------------------------------
        Call snagre(att_mode, pos, vel, Mat_Ra_Rg, ierr)
        If( ierr .Ne. 0 ) Then
           Call ml_wti('error','snagre return code',ierr)
           Call ml_wti('error','snagre input att mode ',att_mode)
           Return
        Endif
        
        Do i=1,nbpts
        	  
        ! calculates the viewed point cartesian coordinates in Rg
        !--------------------------------------------------------
          Call lalocart(1,lat(i),lon(i),om_g,ierr)
          If( ierr .Ne. 0 ) Then
            Call ml_wti('error','lalocart return code',ierr)
          Endif
                
          ! calculates the viewing vector, satellite - viewed point, in Rs
          !----------------------------------------------------------------
          Call viewvect(pos, Mat_Ra_Rg, att, om_g, sm_s)
          
          ! calculates the elevation and azimuth angles in Metop ref.frame
          !----------------------------------------------------------------
          projec = dsqrt(sm_s(2)*sm_s(2) + sm_s(3)*sm_s(3))
          elev(i) = datan2( sm_s(1) , projec )
          azim(i) = datan2( sm_s(3) , sm_s(2) ) + pi

	Enddo
	
        Return

	end
#endif
