#ifdef OPS_USE_AAPP
       subroutine ops_metop_to_earth(   
     >	   time, pos, vel, att_mode, att, 
     >	   nbpts, azim, elev, 
     >	   lat, lon, Elev_sat, Azim_sat, Elev_sol, Azim_sol,
     >     EarthSatelDist, ierr)
!+
!NOM/NAME  %M% version %I% on %G%        
!
!BUT/PURPOSE 
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!DESCRIPTION 
!	FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!      
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Apr 2004      no          A. Marsouin P.Brunel
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
	implicit none                                                     
!
!ARGUMENTS
!
!       Rs is the spacecraft fixed reference frame
!       Ra is the nominal attitude reference frame
!       Rg is the Greenwich reference frame
!
! input 
	Double Precision time		! julian instant 		(julian day)
	double precision pos(3)		! satellite position at time in Rg  (km)
	double precision vel(3)		! satellite velocity at time in Rg  (km/s)
        Integer att_mode		! attitude mode
	double precision att(3,3)	! attitude error matrix		(rad)
	integer nbpts 			! number of points
	Double Precision elev(nbpts)	! elevation angles in Metop ref.frame	(rad)
	Double Precision azim(nbpts)	! azimut angles	in Metop ref.frame	(rad)
        
! output		
	Double Precision lat(nbpts)		! latitudes 			(rad)
	Double Precision lon(nbpts)		! longitudes 			(rad)
	Double Precision Elev_sat(nbpts)	! satellite elevation angles	(rad)
	Double Precision Azim_sat(nbpts)	! satellite azimut angles	(rad)
	Double Precision Elev_sol(nbpts) 	! sun elevation angles		(rad)
	Double Precision Azim_sol(nbpts) 	! sun azimut angles		(rad)
	Double Precision EarthSatelDist(nbpts)	! Orbit radius                  (m) 	
	Integer ierr				! return code 
!  		0 = OK  
!  		9 = internal error, attitude mode not available

!DECLARATIONS
!
! identification variable for "what"
      character*62 sccsid     
!
      data sccsid
     &/'%Z% %M% version %I% on %G%'/       
!
! Local parameters
	double precision pis2		! pis2
	parameter ( pis2 = 3.14159265358979311600D0/2.d0 )
!
! Local variables	(distances in km, angles in radians)
	integer i			! loop variable
	double precision Mat_Ra_Rg(3,3)	! Ra to Rg conversion matrix
	double precision smu_s(3)	! satellite - viewed point unit vector in Rs
	Double Precision om_g(3)	! viewed point in Rg
	Double Precision satzen		! satellite zenith angle 	
	Double Precision sunzen		! sun zenith angle 	   	
	Double Precision sun_g(3)	! sun position in Rg
	Double Precision dist		! sun-Earth distance
! 
!
!-
        ! calculates the Ra <--> Rg transformation
        !--------------------------------------------
        Call snagre(att_mode, pos, vel, Mat_Ra_Rg, ierr)
        If( ierr .Ne. 0 ) Then
           Call ml_wti('error','snagre return code',ierr)
           Call ml_wti('error','snagre input att mode ',att_mode)
           Return
        Endif
        
        EarthSatelDist(1) =  dsqrt( pos(1) * pos(1) + pos(2) * pos(2)
     &                   + pos(3) * pos(3) ) * 1000.
	  
        Do i=1,nbpts
        
	   !  calculates the satellite - viewed point unit vector in Rs
	   !---------------------------------------------------------------
           smu_s(1) =   dsin(elev(i))
           smu_s(2) = - dcos(elev(i)) * dcos(azim(i))
           smu_s(3) = - dcos(elev(i)) * dsin(azim(i))

              
	   !  calculates the viewing vector, satellite - viewed point, in Rg
	   !---------------------------------------------------------------
	   Call earthpix( pos, att, Mat_Ra_Rg, smu_s, om_g, ierr)
	   If( ierr .Ne. 0 ) Then
	      Call ml_wti('error','earthpix return code',ierr)
	      write(0,*) 'att mode ',att_mode
	      write(0,*) 'pos vel ',pos,vel
	      write(0,*) 'vecteur smu_s ',smu_s
	      Return
	   End If

           ! calculates the viewed point latitude and longitude
           !---------------------------------------------------
	   Call  cartlalo(1, om_g, lat(i), lon(i))

           ! satellite zenith and azimut angles of the viewed point
           !---------------------------------------------------
	   Call zenazi(1, lat(i), lon(i), pos, 
     &                    satzen, Azim_sat(i), dist)
           
           ! sun zenith and azimut angles of the viewed point
           !-------------------------------------------------
	   Call sungrw(time, sun_g, dist)
	   Call zenazi(1, lat(i), lon(i), sun_g, 
     &                    sunzen, Azim_sol(i), dist)
     
     	   Elev_sat(i) = pis2 - satzen
     	   Elev_sol(i) = pis2 - sunzen

	   EarthSatelDist(i) = EarthSatelDist(1) 
	Enddo
	
        Return

	end
#endif
