#ifdef OPS_USE_AAPP
	subroutine ops_sp_read(sp_unit,sp_name,ierr)
!+
!NOM/NAME  %M% version %I% on %G%        
!
!BUT/PURPOSE 
!
!DESCRIPTION 
!	FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!   
!	return code 0=OK  
!             1=open fails	(fatal)
!             3=not enough storage	(fatal)      	
!             4=IO error		(fatal)  
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 April 2004     no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
	implicit none                                                     
!     
!ARGUMENTS
        integer sp_unit                 ! Logical unit for satpos file
	character*256 sp_name	! input filename 
	integer ierr		! return code 0=OK  
				! 1=open fails          (fatal)
 				! 3=not enough storage  (fatal)
				! 4=IO error            (fatal)
!	
!DECLARATIONS
!
! identification variable for "what"
      character*62 sccsid     
!
      data sccsid
     &/'%Z% %M% version %I% on %G%'/       
!
! Local parameters
	double precision pi
	parameter ( pi = 3.14159265358979311600d0)
	double precision cdr
	parameter (cdr=pi/180.d0)
!
!
! Commons
! Satpos common
#include "c_satpos.h"
!
! Functions
!
! Local variables
	integer ios			! I/O status variable
	integer nbsteps			! total number of steps of satpos file
	integer step1			! first step to store
	integer step2			! last step to store
	integer istep			! loop step number
	integer fstep			! file step number
	integer nsp			! number of step stored
	character*8 dummyst		! dummy string
	character*22 ch_stdate		! file start date string
	character*22 ch_epoch		! epoch time string
! 
!-
!	No rewind
!
	dummyst='********'
	ios=0
        ierr = 0

        ! Open Satpos file
	open (unit=sp_unit,file=sp_name,status='old',iostat=ios)
	if( ios.ne.0 ) then
	  call ml_wti('error',
     &       'while openning the satpos file I/O status',ios)
	  call ml_wt('error','filename '//sp_name)
	    ierr = 1
	    return
	endif
	  call ml_wt('info','opens the satpos file')
	  call ml_wt('info',sp_name)

!	finds the satpos target
	do while (ios.eq.0 .and. dummyst(1:7).ne.'#SATPOS' )
	  read(sp_unit,'(a)',iostat=ios) dummyst
	end do

        if(ios.ne.0) then
	    call ml_wt('error','while decoding the SATPOS file'//
     &        'no #SATPOS target')
	    call ml_wt('error','satpos file maybe empty')
	    ierr = 4
	    return
         endif

!	reads the header and verifies the satellites names
	read(sp_unit,'(20x,a)') sp_satnam
	read(sp_unit,'(20x,a)') sp_grsnam
	read(sp_unit,'(20x,a)') ch_stdate
	call xicj(ch_stdate,sp_fstart)
	read(sp_unit,'(20x,f6.2)') sp_nbdays
	read(sp_unit,'(20x,f6.2)') sp_dt
	read(sp_unit,'(20x,a)') sp_bultyp
	read(sp_unit,'(20x,i2)') sp_bulsear
	read(sp_unit,'(20x,a)') sp_bulfilna
	
	read(sp_unit,'(20x,a)') ch_epoch
	call xicj(ch_epoch,sp_epoch)
	read(sp_unit,'(20x,f10.4)') sp_semaxis
	read(sp_unit,'(20x,f10.9)') sp_eccent
	read(sp_unit,'(20x,f10.5)') sp_inclin
	read(sp_unit,'(20x,f10.5)') sp_argper
	read(sp_unit,'(20x,f10.5)') sp_rigasc
	read(sp_unit,'(20x,f10.5)') sp_meanano
	if( sp_semaxis .gt. 10000.) then
	   read(sp_unit,'(20x,3(f12.4,1x))') sp_x, sp_y, sp_z
	else
	   read(sp_unit,'(20x,3(f10.4,1x))') sp_x, sp_y, sp_z
	endif
	read(sp_unit,'(20x,3(f10.7,1x))') sp_vx, sp_vy, sp_vz
	read(sp_unit,'(20x,f10.4)') sp_latstat
	read(sp_unit,'(20x,f10.4)') sp_lonstat
	read(sp_unit,'(20x,f10.4)') sp_altstat
	read(sp_unit,'(20x,f10.4)') sp_sitestat
	read(sp_unit,'(a)',iostat=ios) dummyst
	sp_inclin = sp_inclin * cdr
	sp_argper = sp_argper * cdr
	sp_rigasc = sp_rigasc * cdr
	sp_meanano = sp_meanano * cdr
	sp_latstat = sp_latstat * cdr
	sp_lonstat = sp_lonstat * cdr
	sp_sitestat = sp_sitestat * cdr


!	number of steps in satpos file
	nbsteps = (sp_nbdays * 86400.d0) / sp_dt + 1

!	computes first step, last step and number of steps to read
!	takes one more step than necessary on each side
        step1 = 1
        step2 = nbsteps
	sp_nbli = step2 - step1 + 1

	if(sp_nbli.gt.nspmax) then
	  call ml_wt('error','not enough storage in c_satpos common')
	  call ml_wti('error','maximum satpos records allowed',nspmax)
	  call ml_wti('error','satpos records needed',sp_nbli)
	  ierr = 3
	  return
	endif

	if(step1.gt.1) then
	  do istep=1,step1-1
	    read(sp_unit,'(a)',iostat=ios) dummyst
	  end do
	endif

!	read and store the values
	nsp = 0
	do istep=step1,step2
	  nsp = nsp+1
	  if( sp_semaxis .gt. 10000.) then
	  read(sp_unit,'(i8,3f12.3,3f10.6,i6,2i4)',iostat=ios)
     &                  fstep, 
     &                  sp_pos(1,nsp), sp_pos(2,nsp), sp_pos(3,nsp),
     &                  sp_vel(1,nsp), sp_vel(2,nsp), sp_vel(3,nsp),
     &                  sp_orbnum(nsp),
     &                  sp_satlight(nsp), sp_satvis(nsp)
	  else
	  read(sp_unit,'(i8,3f10.3,3f10.6,i6,2i4)',iostat=ios)
     &                  fstep, 
     &                  sp_pos(1,nsp), sp_pos(2,nsp), sp_pos(3,nsp),
     &                  sp_vel(1,nsp), sp_vel(2,nsp), sp_vel(3,nsp),
     &                  sp_orbnum(nsp),
     &                  sp_satlight(nsp), sp_satvis(nsp)
	  end if
	  if(fstep.ne.istep .or. ios.ne.0) then
	    call ml_wti('error',
     &                  'I/O error while reading satpos file',ios)
	    ierr=4
	    return
	  endif
	end do

!	compute the real time of the common data
	sp_jstart = sp_fstart + (step1-1) *sp_dt / 86400.d0
	sp_dt2 = sp_dt * sp_dt
	sp_dt3 = sp_dt2 * sp_dt

        ! Close Satpos file
	close (unit=sp_unit,iostat=ios)
	if( ios.ne.0 ) then
	  call ml_wti('warning',
     &       'while closing the satpos file I/O status',ios)
	  call ml_wt('warning','filename '//sp_name)
          return
	endif

	return
	end
#endif
