#include "spline-lite.h"

/*
 *
 *    CUBIST, VERSION OF 17 JUL 81
 *    CALLED BY 'SPLINE'
 *    CUBIC SPLINE GENERATOR BY W.HIBBARD, MOD'D BY H.WOOLF.  SECOND DERIV.
 *    NOT CONTINUOUS. THE DEGREES OF FREEDOM GAINED ARE USED IN AN
 *    ATTEMPT TO AVOID  OSCILLATIONS.
 *    FIT TO THE POINTS (X(I),Y(I)) I=1,...,N .
 */
      

static void cubist( int n, double *x, double *y, double *as ) {
  int m, i;
  double t, w, z, s, u, v;
  double zs, zq, ws, wq;
  double aa, ba, ca, da;
  
  m = n - 1;
  
  for( i = 0; i < m; i++ ) {
    if( i == 0 )
      goto l110;
    t = ( y[i+1] - y[i-1] ) / ( x[i+1] - x[i-1] );
    goto l120;
l110:
    w = ( y[1] + y[2] ) / 2.0;  
    z = ( x[1] + x[2] ) / 2.0;
    t = ( w - y[0] ) / ( z - x[0] );
    t = 2.0 * ( y[1] - y[0] ) / ( x[1] - x[0] ) - t;
l120:
    if( i == m - 1 )
      goto l130;
    s = ( y[i+2] - y[i] ) / ( x[i+2] - x[i] );
    goto l140;
l130:
    w = ( y[n-2] + y[n-3] ) / 2.0;
    z = ( x[n-2] + x[n-3] ) / 2.0;
    s = ( y[n-1] - w )/( x[n-1] - z );
    s = 2.0 * ( y[n-1] - y[n-2] ) / ( x[n-1] - x[n-2] ) - s;
l140:
    u = y[i+1];
    v = y[i];
    w = ( x[i+1] + x[i] )/ 2.0;
    z = ( x[i+1] - x[i] )/ 2.0;
    zs = z * z; 
    zq = z * zs; 
    ws = w * w;
    wq = w * ws; 
    aa =  .5 * ( u + v ) -  .25 * z * ( s - t );
    ba = .75 * ( u - v ) / z  - .25 * ( s + t );
    ca = .25 * ( s - t ) / z;
    da = .25 * ( s + t ) / zs - .25 * ( u - v ) / zq;
    as[0+5*i] = aa - ba * w + ca * ws - da * wq;
    as[1+5*i] = ba - 2.0 * ca * w + 3.0 * da * ws;
    as[2+5*i] = ca - 3.0 * da * w;
    as[3+5*i] = da;
    as[4+5*i] = 0.;
  }

}




/*
 *
 *    SPLINE, VERSION OF 17 JUL 81
 *    SUBROUTINE SPLINE(XI,YI,XO,YO,AS,NI,NO,II)
 *    DOUBLE PRECISION XI(NI)
 *    DOUBLE PRECISION YI(NI)
 *    DOUBLE PRECISION XO(NO)
 *    DOUBLE PRECISION YO(NO)
 *    DOUBLE PRECISION AS(5,NI)
 * 
 */

void spline_spline( 
  double *xi, double *yi, double *xo, double *yo, 
  double *as, int ni, int no, int ii 
) {
  int nim1, i, j, k, l, m;
  double xm, xn, xx;
  double c[4];
  
  if( ii == 1 )
    cubist( ni, xi, yi, as );

  nim1 = ni;
  m = 0;
  
  for( j = 0; j < no; j++ ) {
    xx = xo[j];
    for( i = m; i < nim1; i++ ) {
      l = i;
      xm = xi[i];
      xn = xi[i+1];
      if( xx == xm )
        goto l120;
      if( xx == xn )
        goto l110;
      if( ( xx > xm ) && ( xx < xn ) )
        goto l130;
    }
l110:
    l = l + 1;
l120:
    yo[j] = yi[l];
    m = l;
    goto l150;
l130:    
    for( k = 0; k < 4; k++ ) 
      c[k] = as[ k + 5 * l ];
    yo[j] = c[0] + xx * ( c[1] + xx * ( c[2] + xx * c[3] ) );
    m = l;
l150:
    continue;
  }
  
  

}


  


