// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/JDBS/src/JDBS__GestionnaireActivite.cc,v 1.3 2011/11/25 10:17:44 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : JDBS
//@ CLASSE : JDBS__GestionnaireActivite
//@
//@ Commentaire :
//@    Ce fichier gere la classe JDBS__GestionnaireActivite
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 30/03/2000 : C.LACAZE
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 14/12/2000 : C.LACAZE
//@  Portage sous systeme AIX 4.3
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-3251 : 29/05/2017 : M. LANDIECH
//@ OPS doit pouvoir démarer sans le MLA
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL
#include <cstdio>

// Standard
#include <fcntl.h>
#include <fstream>
// FT-2765
#ifdef AIX
#include <sys/flock.h>
#endif
// ...FT-2765

// Projet
#include "CMN__Commun.h"
#include "ERR__Exception.h"
#include "CMN__Config.h"
#include "CMN__Fichier.h"
#include "CFG__Configuration.h"
#include "MSG__Abonnement.h"
#include "JDBS__GestionnaireActivite.h"
#include "MP__Message.h"
#include "MP__Spacecraft.h"

//
// Definition des constantes 
//
const double JDBS__PRECISION = 2.;


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : JDBS__GestionnaireActivite
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
JDBS__GestionnaireActivite::JDBS__GestionnaireActivite ()
{
  //@ Positionne le flag de fin
  m_FlagFin = false;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~JDBS__GestionnaireActivite
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
JDBS__GestionnaireActivite::~JDBS__GestionnaireActivite()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Initialiser
//@
//@ Description : Initialisation de l'instance du PGE services actif
//@
//@ Parametres :
//@               (ENTREE) ptPGEServices : Adresse de l'instance du
//@               PGE services
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void JDBS__GestionnaireActivite::Initialiser(CMN__PGEServices* ptPGEServices)
{
  //@ Initialiser l'instance du PGE services
  m_ptPGEServices = ptPGEServices;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ConnecterServeur
//@
//@ Description : Permet de se connecter au bus MSG en etablissant une
//@               connexion avec le serveur
//@
//@ Parametres :
//@               (ENTREE) host : Nom du hote sur lequel attend le 
//@                               serveur
//@               (ENTREE) service : Numero de port de la connexion
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void JDBS__GestionnaireActivite::ConnecterServeur(char *host, char *service)
{
  MSG__Message message;
  
  
  //@ Ouvrir la connexion du type cliente
  m_ConnexionServeur.Ouvrir(host,service, 
                            MSG__Connexion::CNX_CLIENTE);
  
  //@ S'abonner aux adresses du processus 
  MSG__Abonnement abonnement;
  abonnement.SetListeAdresse
  (CFG__Configuration::GetListeAdresse("LISTE_JDB"));
  abonnement.SetAction (MSG__Abonnement::ABONNER);
  abonnement.Serialiser(message);
  m_ConnexionServeur.PublierMessage(message); 		     
  message.SetAdresse ("");
  }

//@ ------------------------------------------------------------------*/
//@ Methode : EnvoyerMessage
//@
//@ Description : Permet d'envoyer un message sur le bus MSG
//@
//@ Parametres :
//@               (ENTREE) message : Message a envoyer
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void JDBS__GestionnaireActivite::EnvoyerMessage(MSG__Message& message)
{
   //@ Publication du message
   m_ConnexionServeur.PublierMessage(message); 
}

//@ ------------------------------------------------------------------*/
//@ Methode : Ecouter
//@
//@ Description : Permet d'ecouter la connexion avec le serveur et de 
//@               dispatcher les messages
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void JDBS__GestionnaireActivite::Ecouter()
{
  //@ Liste des descripteurs actifs
  MSG__Connexion::t_ListeDescripteur descripteurActif;
  MSG__Message newMessage;

  //@ Appel bloquant scruter sur la connexion serveur
  m_ConnexionServeur.Scruter (descripteurActif);
   
  //@ SI il y a un descripteur descripteur actif
  if (descripteurActif.size() == 1)
    {
      //@ Lire le message
      m_ConnexionServeur.LireMessage (newMessage);
      
      //@ Traiter le message
      TraiterMessage (newMessage);	    		
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : TraiterMessage
//@
//@ Description : Permet de traiter les differents types de messages
//@
//@
//@ Parametres :
//@               (ENTREE) message : Message recu
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void JDBS__GestionnaireActivite::TraiterMessage(const MSG__Message& message)
{
  ERR__Exception exception;
  JDB__Message messageJDB;
  TES__Evenement evenementTimer;
  t_CMN__Statut status = CMN__STATUT_OK;
  MP__Spacecraft currentSpacecraft;


  //@ SELON le type du message
  switch (message.GetType()) 
    {
    //@ Si c'est un message journal de bord event ou trace
    case MSG__Message::EVT_JDB :
    case MSG__Message::TRA_JDB :
      
      //@ On construit un objet message journal de bord
      messageJDB.Deserialiser(message);

      //@ Le message est envoye au journal de bord
      status = m_ptPGEServices->MLASendEvent (messageJDB);

// FT 3251 -- OPS doit fonctionner sans MLA
//      //@ SI l'envoi du message s'est mal deroule
//      if (status == CMN__STATUT_NOK)
//	{
//	  //@ On envoie une exception JDB_END_FAILED
//	  exception.SetErrorCode (ERR__Exception::JDB_END_FAILED);
//	  THROW (exception);
//	}
//      //@ FIN SI
// Fin -- FT 3251

      break;

    //@ Si c'est un evenement FIN
    case MSG__Message::FIN :
     
      //@ On positionne le flag de fin d'execution
      Terminer(true);
      break;

    //@ Si c'est un evenement SPACECRAFT
    case MP__Message::EVT_SPACECRAFT :
      //@ Construction d'un objet MP__Spacecraft a partir du message
      currentSpacecraft.Deserialiser (message);

      //@ Mettre a jour le Spacecraft pour l'envoi des log/trace events
      m_ptPGEServices->InitialiseSpacecraft(currentSpacecraft.GetSpacecraft());

    break;

    //@ Defaut:
    default :
      //@ Par defaut, on envoie une exception UNEXCEPTED_MSG
      exception.SetErrorCode (ERR__Exception::UNEXPECTED_MSG);
      THROW (exception);   
      break;
    }
  //@ FIN SELON
}


//@ ------------------------------------------------------------------*/
//@ Methode : Terminer
//@
//@ Description : Permet de terminer la gestion des activites du 
//@               serveur
//@
//@ Parametres :
//@               (ENTREE) flag : Indicateur de fin
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void JDBS__GestionnaireActivite::Terminer(bool flagFin)
{
  //@ Memorisation de l'etat de fin du gestionnaire
  m_FlagFin = flagFin;
}


//=====================================================================
//                        METHODES PRIVEES
//=====================================================================
