// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/JDBS/src/JDBS__JDBServer.cc,v 1.3 2011/11/25 10:17:44 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : JDBS
//@ CLASSE : JDBS__JDBServer
//@
//@ Commentaire :
//@    Ce fichier gere la classe JDBS__JDBServer
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 31/03/2000 : C.LACAZE
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : PBR
//@ Adaptation a l'OPS	 
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
#include <stdlib.h>
//

// STL

// Standard

// Projet
#include "CMN__Commun.h"
#include "CFG__Configuration.h"
#include "CMN__Securite.h"
#include "CMN__Version.h"
#include "ERR__Exception.h"
#include "CMN__PGEServices.h"
#include "JDBS__JDBServer.h"
#include "CMN__OPSProcess.h"

//
// Definition des constantes
//


static void JDBS__Terminate ( int Signal );
static void JDBS__Exit ( int Signal );

//=====================================================================
//                        VARIABLES STATIQUES
//=====================================================================

//@ Serveur de journal de bord
static JDBS__JDBServer   S_Serveur;

// Acces aux services PGE
static CMN__PGEServices* S_PGEServices;


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : JDBS__JDBServer
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
JDBS__JDBServer::JDBS__JDBServer()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~JDBS__JDBServer
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
JDBS__JDBServer::~JDBS__JDBServer()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Initialiser
//@
//@ Description : Permet d'initialiser les traitements du serveur de
//@               journal de bord
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void JDBS__JDBServer::Initialiser()
{
  //@ Lecture du parametrage (nom Host et numero de service)
  t_Chaine host = CFG__Configuration::GetAdresse("MSG_SERVER_HOST");
  t_Chaine service = CFG__Configuration::GetAdresse("MSG_SERVER_PORT");
  
  //@ Initialise le gestionnaire d'activite
  m_GestionnaireActivite.Initialiser(S_PGEServices);
  
  //@ Creer la connexion serveur en appelant le gestionnaire
  //@ d'activite
  m_GestionnaireActivite.ConnecterServeur((char *)host.c_str(), 
					  (char *)service.c_str());
}


//@ ------------------------------------------------------------------*/
//@ Methode : Lancer
//@
//@ Description : Permet de lancer la boucle de scrutation des
//@               evenements
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void JDBS__JDBServer::Lancer()
{
  //@ TANT QUE pas fin
  while (!m_GestionnaireActivite.Terminer()) 
    {
      //@ Scruter les evenements
      m_GestionnaireActivite.Ecouter();
    }
  //@ FIN TANT QUE
}


//@ ------------------------------------------------------------------*/
//@ Methode : Terminer
//@
//@ Description : Permet de terminer l'execution du serveur de journal
//@               de bord
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void JDBS__JDBServer::Terminer(MP__ArretProcess::e_EtatFinProcessus
			       etat)
{
  MP__ArretProcess msgArret;
  MSG__Message     message;


  //@ Construction d'un message de type ARRET PROCESSUS
  msgArret.SetProcessus(PROCESS_JDBS);
  msgArret.SetEtatFin(etat);
  msgArret.Serialiser(message);

  //@ Envoi du message
  m_GestionnaireActivite.EnvoyerMessage(message);

  //@ Terminaison du gestionnaire d'activite
  m_GestionnaireActivite.Terminer (true);
}


//@ ------------------------------------------------------------------*/
//@ Methode : main
//@
//@ Description : Programme principal du serveur de journal de bord
//@
//@ Parametres :
//@               (ENTREE) argc : Nombre de parametres
//@               (ENTREE) argv : Tableau des parametres
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
main(int argc, char *argv[])
{
  t_CMN__PGEStatus status = CMN__STATUT_OK;
 

  //@ Traitement des options d'execution
  CMN__Version_Trace(argc, argv);

  //@ Declaration du demarrage au MCS
  S_PGEServices = new CMN__PGEServices(PROCESS_JDBS);
  status = S_PGEServices->MLAInitialisation();  
   
  //@ SI le status de l'initialisation du MCS est OK
  if (status == CMN__STATUT_OK) 
    {
      //@ Capture des exceptions
      try 
	{
	  //@ Installation du module de gestion de la securite
	  CMN__Securite_Init(JDBS__Exit, JDBS__Terminate);
	  
	  //@ Initialisation du serveur
	  S_Serveur.Initialiser();
	  
	  //@ Lancer la boucle infinie de scrutation des evenements
	  S_Serveur.Lancer();	 	    

	  //@ On est sorti de la boucle: on termine
	  S_Serveur.Terminer(MP__ArretProcess::FIN_OK);
	  
          //@ On signale au MCS la fin du processus (fait exit)
	  S_PGEServices->MLATerminate();	 
	} 

      //@ Exploitation des exceptions  
      catch (ERR__Exception &exception) 
	{
	  //@ Impression des exceptions
	  cout << "Exception (JDBS): " << endl;
	  cout << "\tFile : " << exception.GetFile() << endl;
	  cout << "\tLine : " << exception.GetLine() << endl;
	  cout << "\tError: " << exception.GetErrorCode() << endl;
	  cout << "\tErrno: " << exception.GetErrno() << endl;
	  for (int i=0; i<exception.GetNbParametre(); i++)
	    {
	      cout << "\tInfo : " << exception.GetParametre(i) << endl;
	    }
	    
	  //@ On essaye de terminer
          S_Serveur.Terminer(MP__ArretProcess::FIN_NOK);
	   
	  //@ On signale au MCS la fin du processus (fait exit)
          S_PGEServices->MLATerminate();	 
	}
    } 
  //@ FIN SI status ok 
  
  delete S_PGEServices;
  
  //@ Erreur initialisation MCS exit(-1)
  exit (-1);
}


//=====================================================================
//                        METHODES PRIVEES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : JDBS__Terminate
//@
//@ Description : Procedure notifiee sur le signal SIGTERM. Arrete le
//@               serveur de Journal de Bord (arret dit rapide)
//@
//@ Parametres :
//@               (ENTREE) signal : Numero du signal invoquant cette
//@                                 methode
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void JDBS__Terminate(int Signal)
{
  //@ Terminaison du serveur de journal de bord
  S_Serveur.Terminer(MP__ArretProcess::FIN_NOK);
  
  //@ On signale au MCS la fin du processus (fait exit)
  S_PGEServices->MLATerminate();
}


//@ ------------------------------------------------------------------*/
//@ Methode : JDBS__Exit
//@
//@ Description : Procedure notifiee sur un signal autre que SIGTERM.
//@               Arrete le serveur de Journal de Bord (arret dit
//@               long)
//@
//@ Parametres :
//@               (ENTREE) signal : Numero du signal invoquant cette
//@                                 methode
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void JDBS__Exit(int Signal)
{
  //@ Terminaison du serveur
  exit (0);
}
