// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/MP/src/MP__GestionnaireActivite.cc,v 1.3 2011/11/25 10:17:44 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : MP
//@ CLASSE : MP__GestionnaireActivite
//@
//@ Commentaire :
//@    Ce fichier gere la classe MP__GestionnaireActivite.
//@    Cette classe fournit les services de gestion de l'ensemble des
//@    activites (fonctionnalite) mises en oeuvre par le processus MP.
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 21/03/2000 : A.MONTMORY
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ SIF-00-016  31/07/00 C.LACAZE
//@ Erreur de pattern sur la scrutation des petits produits.
//@ --------------------------------------------------------------------
//@ SIF-00-006  24/07/00 A.MONTMORY   
//@ Dans RenvoyerStatusSI supprimer le message il est envoye par SIP.
//@ --------------------------------------------------------------------
//@ SIF-00-007  24/07/00 A.MONTMORY   
//@ Dans RenvoyerRapportSI detruire le rapport apres envoie.
//@ --------------------------------------------------------------------
//@ SIF-00-038  30/08/00 C.LACAZE
//@ Dans TraiterMessage, les status de ressources indiquant les places disques
//@ disponibles doivent etre exprimes en Mo. La valeur de SizeFree exprimee en
//@ octets est donc divisee par 1024*1024.
//@ --------------------------------------------------------------------
//@ SIF-00-057  06/09/00 A.MONTMORY
//@ Dans RenvoyerRapportSI Ne pas detruire le rapport en cas d'erreur
//@ d'emission.
//@ --------------------------------------------------------------------
//@ VERSION : 14/12/2000 : C.LACAZE
//@  Portage sous systeme AIX 4.3
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : PBR
//@ Adapation a l'OPS-IASI
//@ --------------------------------------------------------------------
//@ VERSION : 22/11/2002 : PBR
//@ FA-ID : OPS-I00-010  18/11/2002 :
//@ ReceptionnerCommande : modification traitement des commandes recues
//@ suite a la creation desmethodes : ExtractPGFCommand, GetFirstCommand,
//@ GetNextCommand dans CMN__PGEServices.
//@ Creation de la methode EnvoyerMsgAnomalieCmde
//@ --------------------------------------------------------------------
//@ VERSION : 03/12/2002 : PBR
//@ FA-ID : OPS-I00-01  02/12/2002 :
//@ TraiterCmdTM : ne pas renvoyer au PGF le compte rendu d'execution
//@  d'une commande lorsqu'il s'agit d'une commande interne
//@ (CMN__OPS_INTERNAL_CMD).
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-3251 : 29/05/2017 : M. LANDIECH
//@ Gestion du process SD via son nom court pour que
//@ les commandes SUSPEND et RESUME du PGF puissent marcher
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//
#include <unistd.h>
#include <cstdio>
#include <values.h>
#include "ERR__Exception.h"
#include "CMN__Fichier.h"
#include "MSG__Abonnement.h"
#include "CFG__Configuration.h"
#include "MP__GestionnaireActivite.h"
#include "CMN__Date.h"
#include "JDB__Callback.h"
#include "JDB__Client.h"
#include "CMN__CGSCst.h"
#include "CMN__OPSProcess.h"
#include "MP__Spacecraft.h"
#include "CMN__CGSCst.h"
#include "CMN__OPSProcess.h"

//
// Definition des constantes 
//
static const int NTRACE = 1;

static const double MP__1MB = (1024. * 1024.);

//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : MP__GestionnaireActivite
//@
//@ Description : Constructeur
//@
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@
//@ ------------------------------------------------------------------*/

MP__GestionnaireActivite::MP__GestionnaireActivite() 
  : m_FlagFin(false), m_ErreurIrrecuperable(false),  p_PGEServices(0)
{
   //@ Positionne le flag de fin
}

//@ ------------------------------------------------------------------*/
//@ Methode : ~MP__GestionnaireActivite
//@
//@ Description : Destructeur
//@
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@
//@ ------------------------------------------------------------------*/

MP__GestionnaireActivite::~MP__GestionnaireActivite()
{
  //@ Detruire l'instance des PGE services
  if (p_PGEServices != NULL)
    {
      delete p_PGEServices;
    }
}

//@ ------------------------------------------------------------------*/
//@ Methode : SetPGEservices
//@
//@ Description : Construire une instance des PGE services
//@
//@ Parametres :
//@               (ENTREE) IsMainProcess : indicateur du type de process 
//@                                        courant
//@ Valeur de retour :
//@               Aucune
//@
//@
//@ ------------------------------------------------------------------*/

void MP__GestionnaireActivite::SetPGEservices (bool IsMainProcess)
{
  p_PGEServices = new CMN__PGEServices (PROCESS_MP);
}

//@ ------------------------------------------------------------------*/
//@ Methode : ConnecterServeur
//@
//@ Description : permet de creer une connection SERVEUR sur laquelle
//@               il attend les connexions clientes.
//@
//@
//@ Parametres :
//@               (ENTREE) service : Numero de port de la connexion. 
//@               (ENTREE) Type : Type de la connexion
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/


void MP__GestionnaireActivite::ConnecterServeur(
                                 char *host, char *service)
{
#ifdef DEBUG
  cout<< host <<endl;
  cout<<service<<endl;
#endif
   //@ Ouvrir la connexion du type cliente
   m_ConnexionServeur.Ouvrir(host, service, 
                             MSG__Connexion::CNX_CLIENTE);
			        
   //@ S'abonner aux adresses du processus 
   MSG__Abonnement abonnement;
   MSG__Message message;
   abonnement.SetListeAdresse
   (CFG__Configuration::GetListeAdresse("LISTE_MP"));
   abonnement.SetAction (MSG__Abonnement::ABONNER);
   abonnement.Serialiser(message);
   m_ConnexionServeur.PublierMessage(message); 
 
   //@ Installer les callback qui permettent d'envoyer les msg JDB
   static JDB__Callback msgCallback;
   msgCallback.SetConnexion(&m_ConnexionServeur);
	
   //@ Initialiser les status par defaut
#ifdef DEBUG
   cout << t_Chaine(OPS_RELEASE)<<endl;
#endif
   MP__HKTMStatus opsRelease = m_GestionnaireStatus.InitialiserStatus();
   
   //@ Renvoi le status "Software realease"
   RenvoyerStatusRes(opsRelease);
   
  // Envoie un message JDB
  JDB__Client messageJDB;
  //@ Positione la version
  // La valeur de OPS_RELEASE est passe par le Makefile
  messageJDB.SetParametre ("$VERSION", t_Chaine(OPS_RELEASE) );
  messageJDB.EcrireMessage (MP__Message::MP_SET,MP__Message::MSG_VERSION);
  messageJDB.Liberer ();
 		     
}

//@ ------------------------------------------------------------------*/
//@ Methode : Ecouter
//@
//@ Description : Permet d'ecouter les descripteurs geres dans la
//@               connexion de type SERVEUR et de dispatcher les 
//@               messages.
//@
//@
//@ Parametres :
//@               aucun
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/


void MP__GestionnaireActivite::Ecouter()
{
   //@ Liste des descripteurs actifs
   MSG__Connexion::t_ListeDescripteur descripteurActif;
   MSG__Message newMessage;

   //@ Appel bloquant scruter sur la connexion serveur
   m_ConnexionServeur.Scruter (descripteurActif);
   
   //@ Si il y a un descripteur descripteur actif
   if (descripteurActif.size() == 1) {
   
   //@ Lire le message
      m_ConnexionServeur.LireMessage (newMessage);
       
   //@ Traiter le message
      TraiterMessage (newMessage);
   }
}

//@ ------------------------------------------------------------------*/
//@ Methode : LancerProcessService
//@
//@ Description : Permet de lancer tous les processus permanents decrit 
//@               dans la configuration
//@
//@ Parametres :
//@               aucun   
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/

void MP__GestionnaireActivite::LancerProcessService   ()
{
  t_ListeChaine listeProcess;
  //@ Lecture de la liste des processus a lancer
  listeProcess = CFG__Configuration::GetListeChaine("PROC_SERVICE");

  //@ Pour tous les processus de la liste
   t_ListeChaine::iterator pInfo;
   for ( pInfo = listeProcess.begin(); 
	 pInfo !=  listeProcess.end();
         pInfo++) {
  //@ Lancer les processus
     LancerProcess ((*pInfo), "");
     
  //@ Attendre 5 secondes entre chaque lancement
     sleep(5); 	 
   }
 
}

//@ ------------------------------------------------------------------*/
//@ Methode : LancerProcessApplicatif
//@
//@ Description : Permet de lancer tous les processus applicatifs decrits
//@               dans la configuration
//@
//@ Parametres :
//@               aucun   
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/

void MP__GestionnaireActivite::LancerProcessApplicatifs ()
{
  t_ListeChaine listeProcess;
  //@ Lecture de la liste des processus a lancer
  listeProcess = CFG__Configuration::GetListeChaine("PROC_APPLICATIFS");

  //@ Pour tous les processus de la liste
   t_ListeChaine::iterator pInfo;
   for ( pInfo = listeProcess.begin(); 
	 pInfo !=  listeProcess.end();
         pInfo++) {
  //@ Lancer les processus
     LancerProcess ((*pInfo), "");
   }
 
}
//@ ------------------------------------------------------------------*/
//@ Methode : StopperProcessPermanent
//@
//@ Description : Permet de stopper tous les processus permanent decrit 
//@               dans la configuration
//@
//@ Parametres :
//@               aucun   
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/

void MP__GestionnaireActivite::StopperProcessPermanent ()
{

  //@ Pour tous les processus de la liste
   t_ListeProcess::iterator pInfo;
   int iterator = m_ListeProcess.size();

  // Envoie un message JDB
   JDB__Client message;
   message.EcrireMessage (MP__Message::MP_SET,MP__Message::MSG_ARRET_OPS);
   message.Liberer ();
   
   //@ On parsours la liste a l'envers pour arreter le Serveur de 
   //@ message en dernier
   pInfo = m_ListeProcess.end();
   pInfo--;
   for ( int i = 0; 
	 i < iterator;
         pInfo--,i++) {
	 
  //@ Stopper les processus
     StopperProcess (pInfo->GetNomProcess(),pInfo->GetPid());
     
  //@ Attendre 5 secondes entre chaque arret
     sleep(5); 	 
   }
 	 
}


//@ ------------------------------------------------------------------*/
//@ Methode : InitialiserEvtTimer
//@
//@ Description : Permet d'initialiser les evenements timers utilise par 
//@               le main process : deux evenements timer sont utilises
//@               par le MP :
//@               => pour declencher l'envoi periodique des HKTM status,
//@               => pour structer la reception d'une commande.
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/

void MP__GestionnaireActivite::InitialiserEvtTimer ()

{
  //@ Programme l'evenement timer qui cadence la scrutation des
  //@ fichiers instructions
  TES__Evenement evenement;
  CMN__Date laDate;
  
  //@ Recuperer la date courante pour initialiser la scrutation
  laDate.DateCourante ();
  t_DateNum StartDate = laDate.ConvertirDateEnTemps() + 10;
  
 
  //@ Recuperer la periode d'envoi des status de resources
  t_DateNum periodeHKTM = CFG__Configuration::GetNumerique("HKTM_PERIODICITY");

  //@ Definition d'un evenement timer 
  //@  => pour declencher l'envoie periodique des status de ressources
  MSG__Message message;
  evenement.SetAction (TES__Evenement::ACTION_ADD);
  // FT-2765
  evenement.SetPeriode (static_cast<int>(periodeHKTM));
  // ...FT-2765
  evenement.SetStartDate (StartDate);
  evenement.SetNomLogique ("EVT_TIMER_STATUS");
  evenement.SetTypeLogique (MP__Message::EVT_TIMER_STATUS);
  evenement.SetAdresse (CFG__Configuration::GetAdresse("MP_INPUT"));
  
  //@ Serialisation de l'evemenent timer EVT_TIMER_STATUS
  message = evenement.Serialiser();

  //@ Publier l'evenement EVT_TIMER_STATUS
  m_ConnexionServeur.PublierMessage (message);

  //@  => pour pour structer la reception d'une commande.

  //@ Recuperer la periode de scrutation
  t_DateNum periodeCMD = CFG__Configuration::GetNumerique("CMD_PERIODICITY");

  evenement.SetAction (TES__Evenement::ACTION_ADD);
  // FT-2765
  evenement.SetPeriode (static_cast<int>(periodeCMD));
  // ...FT-2765
  evenement.SetStartDate (StartDate);
  evenement.SetNomLogique ("EVT_TIMER_CMD");
  evenement.SetTypeLogique (MP__Message::EVT_TIMER_CMD);
  evenement.SetAdresse (CFG__Configuration::GetAdresse("MP_INPUT"));

  //@ Serialisation de l'evemenent timer EVT_TIMER_CMD
  message = evenement.Serialiser();

  //@ Publier l'evenement EVT_TIMER_CMD
  m_ConnexionServeur.PublierMessage (message);
}



//=====================================================================
//                        METHODES PRIVEES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : ManageException
//@
//@ Description : Permet de traiter les differents types d'exception.
//@
//@
//@ Parametres :
//@               (ENTREE) exception : exception recu. 
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/

void MP__GestionnaireActivite::ManageException(ERR__Exception exception)
{
   JDB__Client message;
   //@ Traiter les exceptions (erreur semantique dans une SI)
   //@ Selon le type d'erreur
   switch (exception.GetErrorCode()) {
   //@ Si c'est une erreur dans SI_NIVEAU2	  
     //    case ERR__Exception::BAD_ADRNAMS :
  //      message.SetParametre ("$ADR", exception.GetParametre(0));
  //      message.EcrireMessage (MP__Message::MP_SET,MP__Message::MSG_BAD_ADR);
     //   break;
      default :
   //@ defaut envoyer un message unexpected exception	    
        message.SetParametre ("$FILE", exception.GetFile());
        message.SetParametre ("$LINE", exception.GetLine());
        message.EcrireMessage (MP__Message::MP_SET,MP__Message::MSG_UNEXPECTED_EXP);
      break;
   }

}

//@ ------------------------------------------------------------------*/
//@ Methode : TraiterMessage
//@
//@ Description : Permet de traiter les differents types de messages.
//@
//@
//@ Parametres :
//@               (ENTREE) connexionCliente : conexion sur laquelle
//@                a ete recue le message.
//@               (ENTREE) message : message recu. 
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/
void MP__GestionnaireActivite::TraiterMessage(MSG__Message& message)
{
  TES__Evenement    evtTimer;

  ERR__Exception    exception;
  
  MP__HKTMStatus    status;
  MP__StageTM       stageTM;
  MP__Command       cmdTM;
  MP__Dump          finDump;
  MP__ArretProcess  mortProcess;
  MP__Spacecraft    currentSpacecraft;


#ifdef INVESTIGATION
  //@ INVESTIGATION: Envoyer un message de type JDB MSG_MP_MESSAGE
  JDB__Client messageJDB;
  messageJDB.SetParametre("$MSG", message.GetType());
  messageJDB.EcrireMessage(MP__Message::MP_SET,
			   MP__Message::MSG_MP_MESSAGE);
  messageJDB.Liberer();
#endif

   //@ En captant les exceptions
   try { 
   
     //@ Selon le type du message
     switch (message.GetType()) {

        //@ Si c'est un evenement TIMER
       case MSG__Message::EVT_TIMER :
     
         //@ Alors on construit un objet timer
         evtTimer.Deserialiser(message);
       break;
 
       //@ Si c'est un message de type pipeline/operational status
       case MP__Message::EVT_HKTM_STATUS :

	 //@ Construction d'un objet MP__HKTMStatus a partir du message
         status.Deserialiser (message);

         //@ Mettre a jour la collection des status
         MettreAJourStatusRes(status);   
       break;

       //@ Si c'est un message de type StageTM
       case MP__Message::EVT_STAGE_TM :
	 //@ Construction d'un objet MP__CmdTM a partir du message
         stageTM.Deserialiser (message);

         //@ Prevenir le PGF de la mise a disposition du fichier
         TraiterStageTM (stageTM);
       break; 

       //@ Si c'est un message de type CmdTM
       case MP__Message::EVT_CMD_TM :
 	 //@ Construction d'un objet MP__CmdTM a partir du message
        cmdTM.Deserialiser (message);
 
	//@ Renvoyer le Compte rendu d'execution de le Cmde au PGF
        TraiterCmdTM (cmdTM);
       break;

       //@ Si c'est un message de type mort d'un processus
       case MP__Message::EVT_ARRET_PROCESSUS :
	 //@ Construction d'un objet MP__ArretProcessus a partir du message
          mortProcess.Deserialiser (message);

         //@ Traiter la mort d'un porcessus
         TraiterMortProcessus (mortProcess);
       break;

       //@ Si c'est un message de type fin de traitement du dump
       //@ envoyer par le WOM sur fin normale ou en fin de traitement d'une
       //@ commande STOP
       case MP__Message::EVT_FIN_DUMP :
	 //@ Construction d'un objet MP__Dump a partir du message
        finDump.Deserialiser (message);

         //@ Traiter la fin de traitement d'un dump
         TraiterFinDump (finDump);
       break;

       //@ Si c'est un message de type Spacecraft
       case MP__Message::EVT_SPACECRAFT:
	 //@ Construction d'un objet MP__Spacecraft a partir du message
        currentSpacecraft.Deserialiser (message);

	//@ Mettre a jour les HKTM status.
         TraiterSpacecraft (currentSpacecraft);

       break;

       //@ Si c'est un message inconnu
       default :
       //@ par defaut on envoie une exception UNEXCEPTED_MSG
          exception.SetErrorCode (ERR__Exception::UNEXPECTED_MSG);
          THROW (exception);   
       break;
     }

     //@ Examem des actions contenues dans l'evenement timer eventuel
     //@ on peut avoir deux types logiques : scrutation de la reception 
     //@ d'une commande ou envoie des HKTM status
     switch (evtTimer.GetTypeLogique()) {
        //@ Si c'est un evenement timer de type scrutation commande
       case MP__Message::EVT_TIMER_CMD :
         //@ Scruter la reception d'une commande sur le stdin
         ReceptionnerCommande ();   
       break;

       //@ Si c'est un evenement timer de type envoi periodique HKTM status 
       case MP__Message::EVT_TIMER_STATUS :
         //@ Renvoyer tous les status HKTM au MCS
         RenvoyerAllStatus ();   
       break;

       //@ Si type UNKNOWN on ne fait rien (evenement vide)
       case MSG__Message::UNKNOWN :
       break;
       
       default :
       //@ par defaut on envoie une exception UNEXPECTED_TYPE
         exception.SetErrorCode (ERR__Exception::UNEXPECTED_TYPE);
         THROW (exception);   
       break;
     }


   } catch (ERR__Exception &exception) {
   // Exploiter les exceptions
      ManageException(exception);
   }  

}


//@ ------------------------------------------------------------------*/
//@ Methode : RenvoyerStatusRes
//@
//@ Description : Permet de renvoyer le status de ressources passe en 
//@               parametre.
//@
//@
//@ Parametres :
//@               (ENTREE) HKTM : status HKTM a renvoyer
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/

void MP__GestionnaireActivite::RenvoyerStatusRes (MP__HKTMStatus&  HKTM)

{
  t_CMN__PGEStatus  status;


#ifdef DEBUG
  cout <<HKTM.GetIdStatus()<<endl;
  cout << HKTM.GetValeurChaine()<<endl;

//  TR2 (NTRACE,"ENVOIE STATUS RESSOURCE %d VALEUR %s",HKTM.GetIdStatus(),
//         HKTM.GetValeurChaine());
#endif

  if (HKTM.GetAvailability() == MP__HKTMStatus::AVAILABLE) {
     //@ Envoyer le status avec MLASendFunctionStatus 
     status = p_PGEServices->MLASendFunctionStatus (HKTM);
    }
}

//@ ------------------------------------------------------------------*/
//@ Methode : RenvoyerAllStatus
//@
//@ Description : Permet de renvoyer tous les status de ressources connus
//@               du gestionnaire.
//@
//@
//@ Parametres :
//@               aucun
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/


void MP__GestionnaireActivite::RenvoyerAllStatus()
{

  MP__GestionnaireStatus::t_ListeStatus listeStatus;
  
  //@ Recuperer la liste des status du gestionnaire
  listeStatus = m_GestionnaireStatus.GetListeStatus();  

  //@ Envoie d'un message JDB
  JDB__Client message;
  message.EcrireMessage (MP__Message::MP_SET,MP__Message::MSG_STATUS_RES);
  message.Liberer ();
  
  //@ Pour tous les status de la liste
  MP__GestionnaireStatus::t_ListeStatus::iterator pInfo;
  for ( pInfo = listeStatus.begin(); 
	pInfo !=  listeStatus.end();
        pInfo++) {
	
  //@ Recuperer le status courant.
     MP__HKTMStatus temp = (*pInfo).second;
       
  //@ et le renvoyer 
     RenvoyerStatusRes(temp);	
  }	
}


//@ ------------------------------------------------------------------*/
//@ Methode : EnvoyerMsgAnomalieCmde
//@
//@ Description : Renvoie un log event pour avertir d'une anomalie sur
//@               la commande recue
//@
//@ Parametres :
//@               (ENTREE) status : status d'extraction de la commande
//@               (ENTREE) stdinString : stdin extracted string
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/
void MP__GestionnaireActivite::EnvoyerMsgAnomalieCmde(t_CMN__Statut status,
                                                       t_Chaine     &chaineExtraite)
                              
{
   //@ Renvoyer un Log Event
   JDB__Client messageJDB;

#ifdef DEBUG
   cout << "cmde en anomalie  :" << chaineExtraite.c_str()<<endl;
   cout << "symptome          :" << CMN_PGE_SYMPTOME[status].c_str()<<endl;
#endif

   if (status != CMN__PGEServices::PGE_NO_CMD) {
      messageJDB.SetParametre ("$CMD", chaineExtraite);
      messageJDB.SetParametre ("$SYMPTOME", CMN_PGE_SYMPTOME[status]);
      messageJDB.EcrireMessage (MP__Message::MP_SET,
                                MP__Message::MSG_INVALID_CMD);
      messageJDB.Liberer ();
   }
}

//@ ------------------------------------------------------------------*/
//@ Methode : ReceptionnerCommande
//@
//@ Description : Permet de receptionner une commande du PGF envoye sur 
//@               le stdin .
//@
//@ Parametres :
//@               aucun
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/

void MP__GestionnaireActivite::ReceptionnerCommande()
                              
{
  t_CMN__Statut status;
  t_Chaine    chaineExtraite;
  MP__Command    command;

  //@ Extraire les donnees recues sur le stdin
  status = p_PGEServices->ExtractPGFCommand (chaineExtraite);

  //@ Si l'extraction est correcte
  if (status == CMN__STATUT_OK){

     //@ Recuperer la 1iere commande
     status = p_PGEServices->GetFirstCommand (command, chaineExtraite);

     //@ Tant que une commande est disponible sur le sdin
    while (status != CMN__PGEServices::PGE_NO_CMD)  {
     
        //@ Si le status d'extraction de la commande est OK
        if (status == CMN__STATUT_OK) {
          //@ Renvoyer un message
          //@ Traiter la commande recu :
          TraiterCommandeRecu (command);
        }
       // SINON SI le status d'extraction de la commande est NOK
      else if (status != CMN__PGEServices::PGE_NO_CMD) {
	//@ Renvoyer un message JdB
        EnvoyerMsgAnomalieCmde(status, chaineExtraite);
      }

        //@ Recuperer la commande suivante
        status = p_PGEServices->GetNextCommand (command, chaineExtraite);
    }
   // FIN WHILE
  }
  //@ SINON extraction incorrecte
  else {
      //@ Renvoyer un message JdB
      EnvoyerMsgAnomalieCmde(status, chaineExtraite);
  }
  //@ FINSI extraction correcte

}

//@ ------------------------------------------------------------------*/
//@ Methode : TraiterCommandeRecu
//@
//@ Description : Traite la commande extraite du STDIN
//@
//@ Parametres :
//@               (ENTREE) cmdTM : message envoye par le WOM
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/

void MP__GestionnaireActivite::TraiterCommandeRecu(MP__Command&    command)

{
  bool cmdValid = false;

  //@ Dans le cas d'un SUSPEND ou d'un RESUME
  if ( (command.GetCommandType () == CMN__CmdTM::SUSPEND) ||
       (command.GetCommandType () == CMN__CmdTM::RESUME) ) {
      //@ Completer avec l'Id du processus SD :
      //@ on parsours la liste des processus pour trouver le SD
      int iterator = m_ListeProcess.size();
      t_ListeProcess::iterator pInfo;
      pInfo = m_ListeProcess.begin();
      for ( int i = 0; 
            i < iterator;
            pInfo++,i++) {
#ifdef DEBUG
	cout <<"process : "<< pInfo->GetNomProcess() << " "<< pInfo->GetPid()<<endl;
#endif
         //@ SI l'Id du process SD n'a pas ete trouve 
         // (processus non lance) 
         if (pInfo->GetNomProcess() == CMN_NOM_PROCESS_SD_COURT) {
             cmdValid = true;
             command.SetProcessId ((MP__Command::t_MP__ProcessId)pInfo->GetPid());
#ifdef DEBUG
             cout << "Id CMD :" << command.GetCommandId()<<endl;
             cout << "Id SD  :" << command.GetProcessId()<<endl;
#endif
         }
         // FIN SI Id du Processus SD
      }
      // FIN FOR

  }
  else if (command.GetCommandType () == CMN__CmdTM::STOP) {
      //@ Supprimer le timer de scrutation des commandes sur le stdin
      TES__Evenement evenementTimer;
      MSG__Message message;
      evenementTimer.SetAction (TES__Evenement::ACTION_DELETE);
      evenementTimer.SetNomLogique ("EVT_TIMER_STATUS");
      message = evenementTimer.Serialiser();
      m_ConnexionServeur.PublierMessage (message);
      message.SetAdresse ("");

      cmdValid = true;
  }
      
  else
     {cmdValid = true;}
     //@ FIn SI cas d'un SUSPEND ou d'un RESUME ou un STOP

  //@ Si la commande est valide Alors
  if (cmdValid == true) {
     //@ Fabriquer un message a destination du WOM
     MSG__Message message;
     command.Serialiser (MP__Message::EVT_PRCSS_CTRL, message);
     message.SetAdresse(CFG__Configuration::GetAdresse ("WOM_INPUT"));

     //@  => envoyer le message
     m_ConnexionServeur.PublierMessage (message);
  }
  //@ Sinon
  else {
     //@  Refuser la commande (Rejected)
     command.SetCommandGroup  (CMN__Cmd::ACK);
     command.SetCommandStatus (CMN__CmdTM::REJECTED);
     TraiterCmdTM (command);        

     //@ Renvoyer un Log Event
     JDB__Client messageJDB;
     messageJDB.SetParametre ("$TYPE", 
                   CMN_PGF_CMD_TYPE [(int)(command.GetCommandType())]);
     messageJDB.SetParametre ("$NUM", (CMN__Chaine(command.GetCommandId())));
     messageJDB.SetParametre ("$STATE", 
                      CMN_PGF_CMD_STATE [(int)(command.GetCommandStatus ())]);
     messageJDB.EcrireMessage (MP__Message::MP_SET,
                            MP__Message::MSG_CMD_PROCESSING_STATE);
     messageJDB.Liberer ();
  }
  //@ FIN SI la commande est valide

}
//@ ------------------------------------------------------------------*/
//@ Methode : TraiterCmdTM
//@
//@ Description : Permet de renvoyer le status d'execution d'une 
//@               commande .
//@
//@ Parametres :
//@               (ENTREE) cmdTM : message envoye par le WOM
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/

void MP__GestionnaireActivite::TraiterCmdTM (MP__Command& cmdTM)

{
  t_CMN__PGEStatus status;

#ifdef DEBUG
   cout << "Reception d'une CMD-TM  => " <<endl;
   cout << " Id       : "<< cmdTM.GetCommandId () <<endl;
   cout << " Type     : "<< cmdTM.GetCommandTypeStr() <<endl;
   cout << " Status   : "<< cmdTM.GetCommandStatus()<<endl;
   cout << " FileName : "<< cmdTM.GetWOFilename()<<endl;
#endif



  //@ Si la commande est de type STOP ou STEP
  //@ et l'etat est COMPLETED ou ABORTED Alors
  if ( ( (cmdTM.GetCommandType() == CMN__CmdTM::STOP)  ||
         (cmdTM.GetCommandType() == CMN__CmdTM::STEP)  ) &&
       ( (cmdTM.GetCommandStatus () == CMN__CmdTM::COMPLETED) ||
         (cmdTM.GetCommandStatus () == CMN__CmdTM::ABORTED) )) {
     //@ Renvoyer les HKTM status
     RenvoyerAllStatus();
  }

  //@ Si la commande est de type START  Alors
  if (cmdTM.GetCommandType() == CMN__CmdTM::START) {
     //@ Completer avec l'Id de la commande Start
     cmdTM.SetCommandId (m_StartId);

     //@ Si l'etat est CODED Alors
     if  (cmdTM.GetCommandStatus () == CMN__CmdTM::CODED)  {
        m_StartState =CMN__CmdTM::CODED;
        //@ Renvoyer les HKTM status
        RenvoyerAllStatus();
     }
     else {
       m_StartState =CMN__CmdTM::REJECTED;

       //@ Etat est REJECTED (les etats COMPLETED/ABPORTED sont deduits du
       //@ message FinDump
       //@ Supprimer le timer de scrutation des commandes sur le stdin
       TES__Evenement evenementTimer;
       MSG__Message message;
       evenementTimer.SetAction (TES__Evenement::ACTION_DELETE);
       evenementTimer.SetNomLogique ("EVT_TIMER_STATUS");
       message = evenementTimer.Serialiser();
       m_ConnexionServeur.PublierMessage (message);
       message.SetAdresse ("");
     }
  }


  //@ S'il ne s'agit pas d'une commande interne Alors
  if (cmdTM.GetCommandId() != CMN__OPS_INTERNAL_CMD)
  {
    //@ Envoyer le compte rendu d'execution de la commande avec SendPGFCommandTM
    status = p_PGEServices->SendPGFCommandTM (cmdTM);
  
    //@ Envoyer un message de Log
    JDB__Client messageJDB;
    messageJDB.SetParametre ("$TYPE", 
			     CMN_PGF_CMD_TYPE [(int)(cmdTM.GetCommandType())]);
    messageJDB.SetParametre ("$NUM", 
			     (CMN__Chaine(cmdTM.GetCommandId())).c_str());
    messageJDB.SetParametre ("$STATE", 
			     CMN_PGF_CMD_STATE [(int)(cmdTM.GetCommandStatus ())]);
    messageJDB.EcrireMessage (MP__Message::MP_SET,
			      MP__Message::MSG_CMD_PROCESSING_STATE);
    messageJDB.Liberer ();
  }
}
			       
//@ ------------------------------------------------------------------*/
//@ Methode : TraiterStageTM
//@
//@ Description : Permet de prevenir le PGF de la mise a disposition d'un 
//@               fichier.
//@
//@ Parametres :
//@               (ENTREE) stageTM : message envoye par le WOM
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/

void MP__GestionnaireActivite::TraiterStageTM (MP__StageTM& stageTM)
{
  t_CMN__PGEStatus status;

#ifdef DEBUG
   cout << "Reception d'un STAGE-TM  => " <<endl;
   cout << " Id       : "<< stageTM.GetCommandId () <<endl;
   cout << " Category : "<< stageTM.GetStageCategory() <<endl;
   cout << " Stage Id : "<< stageTM.GetStageId()<<endl;
   cout << " FileName : "<< stageTM.GetDeliveredFilename()<<endl;
#endif

  //@ Envoyer le stageTM au PGF 
  status = p_PGEServices->SendPGFStageTM (stageTM);

}

//@ ------------------------------------------------------------------*/
//@ Methode : TraiterSpacecraft
//@
//@ Description : Cette methode a en charge de mettre a jour le champ
//@               Service des HKTM status stockes par l'objet
//@               GestionnaireStatus.
//@
//@ Parametres :
//@               (ENTREE) currentSpacecraft : message envoye par le SD
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/
void MP__GestionnaireActivite::TraiterSpacecraft 
                                   (const MP__Spacecraft& currentSpacecraft)

{
  MP__GestionnaireStatus::t_ListeStatus listeStatus;

  //@ Recuperer la liste des status du gestionnaire
  listeStatus = m_GestionnaireStatus.GetListeStatus();  

  //@ Pour tous les status de la liste
  MP__GestionnaireStatus::t_ListeStatus::iterator pInfo;
  for ( pInfo = listeStatus.begin(); 
	pInfo !=  listeStatus.end();
        pInfo++) {
	
     //@ Initialiser le champ Service
     MP__HKTMStatus status = (*pInfo).second;
     status.SetService (currentSpacecraft.GetSpacecraft());

     //@ Inserer le status dans la collection
     m_GestionnaireStatus.SetStatus(status.GetIdStatus(), status);

  }	
}

//@ ------------------------------------------------------------------*/
//@ Methode : TraiterFinDump
//@
//@ Description : Traite le message indiquant la fin de traitement d'un
//@               dump. 
//@
//@ Parametres :
//@               (ENTREE) finDump : message envoye par le WOM
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/

void MP__GestionnaireActivite::TraiterFinDump(
                               const MP__Dump& finDump)
{
   MP__Command      cmdStart;
   t_CMN__PGEStatus status;
   CMN__CmdTM::e_CMN__CommandStatus startState;

#ifdef DEBUG
   cout << "Reception d'un Fin Dump  => " <<endl;
   cout << " Etat       : "<< finDump.GetEtatFinDump() <<endl;
#endif

  //@ Renvoyer le compte rendu d'execution du Start :
   cmdStart.SetCommandGroup (CMN__Cmd::ACK);
   cmdStart.SetCommandType  (CMN__CmdTM::START);
   cmdStart.SetCommandId    (m_StartId);

  //@ Si la fin est normale Alors
  if (finDump.GetEtatFinDump () == MP__Dump::FIN_OK) {
      //@ Etat commande Start =  COMPLETED
      startState = CMN__CmdTM::COMPLETED;

  }
  //@ Sinon
  else {
       //@ Etat commande Start =  ABORTED
       startState = CMN__CmdTM::ABORTED;

       //@ Supprimer le timer de scrutation des commandes sur le stdin
       TES__Evenement evenementTimer;
       MSG__Message message;
       evenementTimer.SetAction (TES__Evenement::ACTION_DELETE);
       evenementTimer.SetNomLogique ("EVT_TIMER_STATUS");
       message = evenementTimer.Serialiser();
       m_ConnexionServeur.PublierMessage (message);
       message.SetAdresse ("");
  }

  //@ Si le Start est tr&ite (n'a pas ete rejete) ALORS
  if (m_StartState == CMN__CmdTM::CODED) {
     //@ Renvoyer l'etat d'execution de la commande Start
     cmdStart.SetCommandStatus (startState);

     //@ Envoyer le compte rendu d'execution de la commande au PGF
     status = p_PGEServices->SendPGFCommandTM (cmdStart);

     //@ Envoyer un message de Log
     JDB__Client messageJDB;
     messageJDB.SetParametre ("$TYPE", 
                              CMN_PGF_CMD_TYPE [(int)(CMN__CmdTM::START)]);
     messageJDB.SetParametre ("$NUM", (CMN__Chaine (m_StartId)).c_str());
     messageJDB.SetParametre ("$STATE", 
                        CMN_PGF_CMD_STATE [(int)(startState)]);
     messageJDB.EcrireMessage (MP__Message::MP_SET,
                            MP__Message::MSG_CMD_PROCESSING_STATE);
     messageJDB.Liberer ();
  }

  //@ Positionne le flag de fin
   m_FlagFin = true;

  //@ Arreter le Systeme
  Terminer (true);
}

//@ ------------------------------------------------------------------*/
//@ Methode : TraiterMortProcessus
//@
//@ Description : Cette procedure traite la mort d'un processus.
//@
//@ Parametres :
//@               (ENTREE) mortProcess : message envoye par le process
//@                                      avant son arret.
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/
void MP__GestionnaireActivite::TraiterMortProcessus(
                                const MP__ArretProcess& mortProcess)
{
  MP__Command  cmdStop;

#ifdef DEBUG
  cout << "Reception d'un Mort Process  => " << endl;
  cout << " Processus  : "<< mortProcess.GetProcessus() << endl;
  cout << " Etat       : "<< mortProcess.GetEtatArret() << endl;
#endif
  
  //@ Si on n'est pas en phase d'arret du systeme :
  //@ c'est une mort inattendu d'un processus Alors
  if (m_FlagFin == false) 
    {
      //@ Renvoyer l'Operational Status Readiness a Unrecoverable Error
      MP__HKTMStatus OpStatus;
      OpStatus.SetIdStatus(MP__HKTMStatus::OPSReadiness);
      OpStatus.SetValidity(MP__HKTMStatus::VALID);
      OpStatus.SetAvailability(MP__HKTMStatus::AVAILABLE);
      OpStatus.SetValeurChaine(CMN_HKTM_READINESS[MP__HKTMStatus::
                                                  UNRECOVERABLE_ERROR]);
      RenvoyerUnStatus(OpStatus);
      
      //@ Mettre a jour le temoin d'erreur irrecuperable
      m_ErreurIrrecuperable = true;

      //@ S'il s'agit d'un processus vital au fonctionnement de l'architecture :
      //@ tel que MSGS 
      if (mortProcess.GetProcessus () == PROCESS_MSGS) 
        {
          //@ Plus aucune communication inter-process
          
          //@ Arreter le MP
          Terminer (true);
        }
      //@ S'il s'agit du WOM Alors
      else if (mortProcess.GetProcessus () == PROCESS_WOM) 
        {
          //@ Envoyer un Log Event
          JDB__Client messageJDB;
          messageJDB.SetParametre ("$NAME", 
                                   CMN_PROCESS_NAME[(int)(mortProcess.
                                                          GetProcessus())]);
          
          messageJDB.EcrireMessage (MP__Message::MP_SET,
                                    MP__Message::MSG_MORT_PROCESS);
          messageJDB.Liberer ();

          //@ Arreter le systeme immediatement : envoyer un message de fin
          Terminer (true);
        }
      //@ S'il sagit du processus TES, JDBS ou SD Alors
      else if ( (mortProcess.GetProcessus () == PROCESS_TES)  ||
                (mortProcess.GetProcessus () == PROCESS_JDBS) ||
                (mortProcess.GetProcessus () == PROCESS_SD)  )  
        {
          //@ Envoyer une commande STOP au WOM
          cmdStop.SetCommandGroup (CMN__Cmd::CMD);
          cmdStop.SetCommandId (int(CMN__OPS_INTERNAL_CMD));
          cmdStop.SetCommandType  (CMN__CmdTM::STOP);
          cmdStop.SetCommandStatus(CMN__CmdTM::CODED);
          
          //@ Fabriquer un message a destination du WOM
          MSG__Message message;
          cmdStop.Serialiser (MP__Message::EVT_PRCSS_CTRL, message);
          message.SetAdresse(CFG__Configuration::GetAdresse ("WOM_INPUT"));
          
          //@  => envoyer le message
          m_ConnexionServeur.PublierMessage (message);
          
          //@ Envoyer un Log Event
          JDB__Client messageJDB;
          messageJDB.SetParametre ("$NAME", 
                                   CMN_PROCESS_NAME[(int)(mortProcess.
                                                          GetProcessus())]);
          
          messageJDB.EcrireMessage (MP__Message::MP_SET,
                                    MP__Message::MSG_MORT_PROCESS);
          messageJDB.Liberer ();
        }
    } 
  //@ Sinon
  //@ Ne rien faire : laisser la phase d'arret se poursuivre
}

//@ ------------------------------------------------------------------*/
//@ Methode : MettreAJourStatusRes
//@
//@ Description : Permet de mettre a jour un status de ressources dans 
//@               le gestionnaire.
//@
//@ Parametres :
//@               (ENTREE) status : status a metttre a jour
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/


void MP__GestionnaireActivite::MettreAJourStatusRes(MP__HKTMStatus& status)
{

#ifdef DEBUG
  cout << "Mettre a jour HKTM Status  => " << endl;
  cout << " Id      : "<< status.GetIdStatus () << endl;
  cout << " Type    : "<< status.GetType () << endl;
  cout << " Valid   : "<< status.GetValidity () << endl;
  cout << " Avail   : "<< status.GetAvailability () << endl;
  cout << " Service : "<< status.GetService () << endl;
  cout << " Name    : "<< status.GetParamName() << endl;
  cout << " Valeur  : "<< status.GetValeurChaine () << endl;
#endif

  //@ Ajouter le status dans le gestionnaire sauf si le temoin d'erreur irrecuperable est 
  //@ possitionne ET s'il s'agit d'une HKTM readiness 
  if (! ( (m_ErreurIrrecuperable == true) && 
          (status.GetIdStatus() == MP__HKTMStatus::OPSReadiness) ) )
  {
    //@ Ajouter le status dans le gestionnaire
    m_GestionnaireStatus.AjouterStatus(status);
  }
}


//@ ------------------------------------------------------------------*/
//@ Methode : RenvoyerUnStatus
//@
//@ Description : Met a jour et renvoit un status.
//@
//@
//@ Parametres :
//@               (ENTREE) status : status. 
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/


void MP__GestionnaireActivite::RenvoyerUnStatus(MP__HKTMStatus& statusAEnvoyer)

{
  t_CMN__PGEStatus retour;
  MP__HKTMStatus   status;

  //@ Mettre a jour la collection des status
  MettreAJourStatusRes(statusAEnvoyer);   

  //@ Recuperer le status dans le gestionnaire
  m_GestionnaireStatus.GetStatus(statusAEnvoyer.GetIdStatus(), status);

  //@ SI le HKTM status est disponible ALORS
  if (status.GetAvailability() == MP__HKTMStatus::AVAILABLE){
     //@ Envoie le status de la sous instruction au MCS
     retour = p_PGEServices->MLASendFunctionStatus(status);
  
     if (retour != CMN__STATUT_OK) {
        JDB__Client messageJDB;
        messageJDB.EcrireMessage (MP__Message::MP_SET,
                                  MP__Message::MSG_ENVOI_HKTM_NOK);
        messageJDB.Liberer ();
     }
  }

}


//@ ------------------------------------------------------------------*/
//@ Methode : LancerProcess
//@
//@ Description : Permet de lancer un process designe par son nom logique
//@               le nom logique doit correspondre a un shell 
//@               <NomLogique>.sh
//@
//@ Parametres :
//@               (ENTREE) nomProcess : nom logique du process a lancer  
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/

void MP__GestionnaireActivite::LancerProcess (t_Chaine nomProcess,
                                              t_Chaine arg)

{
  t_Chaine                commande;
  pid_t                   pid=0;
  FILE*                   fp = NULL;
  char                    sz_pid[32] = {0};
  ERR__Exception          exception;

  //@ creation de la commande a executer
  commande = CFG__Configuration::GetRepertoire("CMD_DIR") + "/"+ nomProcess + ".sh";

  //@ ajout des parametres a la commande
  if (arg != "")
    {
      commande = commande + " " + arg;
    }

#ifdef DEBUG
//    TR1(NTRACE,"LANCEMENT du PROCESS %s ",commande)
#endif

  //@ Lancement du process a travers un pipe 
  if ( (fp=popen((char*)commande.c_str(),"r"))  !=  0) {
  
  //@ recuperation du pid par lecture dans le pipe
     char *pidChaine = fgets(sz_pid,sizeof(sz_pid),fp);
#ifdef DEBUG
     cout << string(commande) << " : "<< pidChaine <<endl;
#endif

     if ( pidChaine ) {
        pid = pid_t (t_Chaine(sz_pid));
     }
     pclose(fp);
  //@ Si le PID est correct alors on enregistre le processus dans la collection
     if ( pid > 0) {
       MP__UnProcess unProcess;
       int ind;

       unProcess.SetPid(pid);
       unProcess.SetNomProcess (nomProcess);
       for( ind=0; ind < m_ListeProcess.size(); ind++ ) {
         if ( m_ListeProcess[ind].GetNomProcess() == nomProcess ) {
#ifdef DEBUG
	   cout << "Le process " << nomProcess << " existe deja\n";
#endif
	   break;
	 }
       }
       if ( ind == m_ListeProcess.size() ) {
#ifdef DEBUG
         cout << "Insertion process " << nomProcess << "\n";
#endif
         m_ListeProcess.insert(m_ListeProcess.end(), unProcess);
       }

#ifdef UNDEF
       //@ Envoie d'un message JDB
       JDB__Client message;
       message.SetParametre ("$PROCESS", nomProcess);
       message.SetParametre ("$ACTION", "STARTED");
       message.EcrireMessage (MP__Message::MP_SET,
                              MP__Message::MSG_PROCESS_ACTION);
       message.Liberer ();
#endif

  //@ Si le pid n'est pas correcte alors on envoie une exception CANT_GET_PID
     } else {
       exception.SetErrorCode (ERR__Exception::CANT_GET_PID);
       THROW (exception);   
     }
  } else {   
  //@ Si le lancement a echoue alors on envoie une exception CANT_LAUCH_PRO
    exception.SetErrorCode (ERR__Exception::CANT_LAUCH_PRO);
    THROW (exception);   
  }

}

//@ ------------------------------------------------------------------*/
//@ Methode : StopperProcess
//@
//@ Description : Permet de stopper un processus 
//@
//@ Parametres :
//@               (ENTREE) nomProcess : nomLogique du processus a arreter  
//@               (ENTREE) pid : pid du processus a arreter (non utilise) 
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/

void MP__GestionnaireActivite::StopperProcess (t_Chaine nomProcess, pid_t pid)

{

   //@ Envoie d'un message de fin a tous les processus
   MSG__Message messageFin;
   messageFin.SetType (MSG__Message::FIN);
   messageFin.SetAdresse(CFG__Configuration::GetAdresse(nomProcess+"_FIN"));
   m_ConnexionServeur.PublierMessage (messageFin);

}
