// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/MP/src/MP__GestionnaireActivite.h,v 1.3 2011/11/25 10:17:44 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : MP
//@ CLASSE : MP__GestionnaireActivite
//@
//@ Commentaire :
//@    Ce fichier gere la classe MP__GestionnaireActivite.
//@    Cette classe fournit les services de gestion de l'ensemble des
//@    activites (fonctionnalite) mises en oeuvre par le processus MP.
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 21/03/2000 : A.MONTMORY
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : PBR
//@ Adapation a l'OPS-IASI
//@ --------------------------------------------------------------------
//@ VERSION : 22/11/2002 : PBR
//@ FA-ID : OPS-I00-010  18/11/2002 :
//@ ReceptionnerCommande : modification traitement des commandes recues 
//@ suite a la creation desmethodes : ExtractPGFCommand, GetFirstCommand,
//@ GetNextCommand dans CMN__PGEServices.
//@ Creation de la methode EnvoyerMsgAnomalieCmde
//@ --------------------------------------------------------------------
//@ VERSION : 03/12/2002 : PBR
//@ FA-ID : OPS-I00-013  02/12/2002 :
//@ Etant donne que la constante CMN__OPS_INTERNAL_CMD est utilisee par
//@ plusieurs classes, son stockage est deplace dans CMN__Cmd.h
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef MP__GestionnaireActivite_h
#define MP__GestionnaireActivite_h

//
// Fichier include
//

#include "CMN__Commun.h"
#include "MP__Message.h"
#include "MSG__Connexion.h"
#include "TES__Evenement.h"
#include "CMN__PGEServices.h"
#include "CMN__Cmd.h"
#include "MP__Command.h"
#include "MP__StageTM.h"
#include "MP__GestionnaireStatus.h"
#include "MP__HKTMStatus.h"
#include "MP__UnProcess.h"
#include "MP__Dump.h"
#include "MP__ArretProcess.h"
#include "MP__Spacecraft.h"

class MP__GestionnaireActivite {

/*====================================================================*/

public:
        //
        // Liste des processus
        //
        typedef vector<MP__UnProcess> 
	        t_ListeProcess;

	
        //
	// Constructeur
	//
	MP__GestionnaireActivite();
	
	//
	// Destructeur
	//
	~MP__GestionnaireActivite();

	//
	// Accesseurs
	MSG__Connexion GetConnexion() const {
	                  return m_ConnexionServeur;
	               };

        void SetPGEservices (bool IsMainProcess);
        CMN__PGEServices* GetPGEservices () {return p_PGEServices;};
	

        void SetIdStart (int IdCmd) 
                         {m_StartId=(CMN__Cmd::t_CMN__CommandId)IdCmd;};


	//
	//
	// Methodes publiques
	//
	void ConnecterServeur(char *host, char *service);
	void Ecouter();
        void Terminer( bool flagFin) { m_FlagFin = flagFin; };
	bool Terminer () { return m_FlagFin; };
	
        // Initialiser les evenements timers
	void InitialiserEvtTimer();
	
	// Lancer tous les processus permanent
        void LancerProcessService();
        void LancerProcessApplicatifs();
	
	// Stopper tous les process
	void StopperProcessPermanent();

	/*===================================================================*/
protected:
	//
	// Methodes protegees
	//

	//
	// Attributs proteges
	//

	/*===================================================================*/
private:
	//
	// Methodes privees
	//
	void TraiterMessage(MSG__Message& message);
	void ManageException(ERR__Exception exception);
	
	// Renvoyer un/tous les status
	void RenvoyerUnStatus (MP__HKTMStatus& statusAEnvoyer);	
        void RenvoyerAllStatus ();
	// Renvoi un status de la liste GestionnaireStatus
	void RenvoyerStatusRes (MP__HKTMStatus& HKTM);

        // Receptionner une commande en provenance du PGF
        void ReceptionnerCommande();
        void TraiterCommandeRecu(MP__Command&    command);
        void EnvoyerMsgAnomalieCmde(t_CMN__Statut status, t_Chaine& chaineExtraite);

        // Traiter un compte rendu d'execution d'une commande du PGF
        void TraiterCmdTM (MP__Command& cmdTM);

        // Traiter le message de mise a disposition de donnees pour le PGF
        void TraiterStageTM (MP__StageTM& stageTM);

        // Traiter le message de fin de traitement d'un dump
        void TraiterFinDump(const MP__Dump& finDump);

        // Traiter l'evenement de mort d'un processus
        void TraiterMortProcessus(const MP__ArretProcess& mortProcess);

	// Mettre a jour un status de ressources
	void MettreAJourStatusRes (MP__HKTMStatus& status);

	// Initialiser le champs Service des HKTM Status
        void TraiterSpacecraft (const MP__Spacecraft& currentSpacecraft);
			
	// Lancer un process
	void LancerProcess(t_Chaine nomProcess, t_Chaine arg);
	 	
	// Stopper un process
	void StopperProcess(t_Chaine nomProcess, pid_t pid);
	
	//
	// Attributs prives
	//
	
	// Gestionnaire des status
	MP__GestionnaireStatus m_GestionnaireStatus;
	
	// Flag de fin
	bool m_FlagFin;
	
	// Flag d'erreur irrecuperable
	bool m_ErreurIrrecuperable;

	// Liste des processus lances par le MP
	t_ListeProcess  m_ListeProcess;
	
	// Connexion avec le serveur
	MSG__Connexion m_ConnexionServeur;

        // Gestionnaire d'acces aux PGE services
	CMN__PGEServices*  p_PGEServices;

	// Start Id
	CMN__Cmd::t_CMN__CommandId m_StartId;

        // Start command Status
        CMN__CmdTM::e_CMN__CommandStatus m_StartState;

};

#endif
