// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/MP/src/MP__GestionnaireStatus.cc,v 1.3 2011/11/25 10:17:44 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : MP
//@ CLASSE : MP__GestionnaireStatus
//@
//@ Commentaire :
//@    Cette classe fournit les services de gestion de la liste 
//@    des HKTM status a transmettre au MCS.
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 21/03/2000 : A.MONTMORY
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ SIF-00-038 A.MONTMORY 05/09/2000
//@ Dans AjouterStatus verifier si l'applicaion a positionne des valeurs
//@ significatives sinon prendre les valeurs du gestionnaire qui viennent
//@ du fichier de configuration.
//@ --------------------------------------------------------------------
//@ VERSION : 28/08/2002 : PBR
//@ Adapation a l'OPS-IASI
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

#include <values.h>
#include "ERR__Exception.h"
#include "CMN__Config.h"
#include "CFG__Configuration.h"
#include "MP__GestionnaireStatus.h"

//
// Definition des constantes 
//
static const int NTRACE = 1;


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : MP__GestionnaireStatus
//@
//@ Description : Constructeur
//@
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@
//@ ------------------------------------------------------------------*/

MP__GestionnaireStatus::MP__GestionnaireStatus()
{
}

//@ ------------------------------------------------------------------*/
//@ Methode : ~MP__GestionnaireStatus
//@
//@ Description : Destructeur
//@
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@
//@ ------------------------------------------------------------------*/

MP__GestionnaireStatus::~MP__GestionnaireStatus()
{
}

//@ ------------------------------------------------------------------*/
//@ Methode : InitialiserStatus
//@
//@ Description : permet de charger tous les statuts HKTM traites par 
//@               l'OPS dans la liste geree par le Gestionnaire 
//@               et de les initaliser avec leur valeur par defaut 
//@               a partir du fichier Status.cfg
//@
//@ Parametres :
//@               aucun.
//@
//@ Valeur de retour :
//@               statusVersion : version logicielle de l'OPS
//@
//@ ------------------------------------------------------------------*/

MP__HKTMStatus MP__GestionnaireStatus::InitialiserStatus()

{
  CMN__Configuration config;
  MP__HKTMStatus     status;
  
  //@ Il faut charger le fichier qui coontient les status par defaut
  t_Chaine fichier = CFG__Configuration::GetFichier("MP_FICHIER_STATUS");
//  TR1(NTRACE,"NOM DU FICHIER STATUS %s",fichier);
  config.ChargerFichier(fichier);
  
  //@ Pour tous les status definis dans le fichier
  int nombre=0;
  config.NombreDeSections(&nombre);
  t_Chaine nomSection;

  
  for (int i=0; i<nombre;i++) {
  //@ Nom de la section = STATUS<I>
    nomSection = "STATUS" + t_Chaine(i);

  //@ Lecture de l'id logique du status
    t_Chaine temp;
    config.LireValeur (nomSection,"ID_LOGIQUE",temp);

    status.SetIdStatus (MP__HKTMStatus::e_ListeStatus(int(temp)));

  //@ Lecture du nom du parametre
    config.LireValeur (nomSection,"PARAM_NAME",temp);
    status.SetParamName (temp);

  //@ Lecture de la Validite du status
    config.LireValeur (nomSection,"VALIDITE",temp);
    status.SetValidity (MP__HKTMStatus::e_Validity(int(temp)));

  //@ Lecture de la disponibilite du status
    config.LireValeur (nomSection,"DISPONIBILITE",temp);
    status.SetAvailability (MP__HKTMStatus::e_Availability(int(temp)));

  //@ Lecture de la valeur chaine du status
    config.LireValeur (nomSection,"VALEUR_CHAINE",temp);
    status.SetValeurChaine (temp);

  //@ Ajouter le status dans le gestionnaire
    AjouterStatus(status);
  }


  //@ Forme le status de la version du logiciel
  MP__HKTMStatus statusVersion (MP__HKTMStatus::PPFversion);
  GetStatus(MP__HKTMStatus::PPFversion,statusVersion);

  //@ La valeur de OPS_RELEASE est passe par le Makefile
  statusVersion.SetAvailability (MP__HKTMStatus::AVAILABLE);
  statusVersion.SetValeurChaine(t_Chaine(OPS_RELEASE));
  AjouterStatus(statusVersion);
  
  return (statusVersion);
}

//@ ------------------------------------------------------------------*/
//@ Methode : AjouterStatus
//@
//@ Description : permet d'ajouter un status dans la liste du gestionnaire.
//@
//@
//@ Parametres :
//@               (ENTREE) status : Status a rajouter.
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/


void MP__GestionnaireStatus::AjouterStatus(
                             MP__HKTMStatus &status)
{
   
   MP__HKTMStatus oldStatus;
   
   //@ Rechercher si le status a ajouter fait parti de la liste du
   //@ gestionnaire
   GetStatus(status.GetIdStatus(), oldStatus);
   
   //@ Si le status fait deja parti de la liste qui est fabrique
   //@ au chargement du fichier de configuration (status.cfg)
   //@ => A la reception d'un status (hors initialisation)
   if(oldStatus.GetParamName() != "" ) {
     //@ le ParamName est celui stocke dans la liste
     status.SetParamName(oldStatus.GetParamName());
     
     //@ Des le 1ier stockage la DISPONIBILITE est positionne a VRAI 
     status.SetAvailability(MP__HKTMStatus::AVAILABLE);   
    }

   //@ Si la valeur chaine n'est pas positionne alors prendre la valeur
   //@ du gestionnaire
   if (status.GetValeurChaine() == "NOT_USED") 
        status.SetValeurChaine(oldStatus.GetValeurChaine());

   //@ Mettre a jour la VALIDITE qui est un champ obligatoire 
   //@ dans le status fourni
   status.SetValidity(oldStatus.GetValidity());

   //@ Inserer le status dans la collection
   m_ListeStatus[status.GetIdStatus()] = status;
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetStatus
//@
//@ Description : permet de retrouver le status concerne.
//@
//@
//@ Parametres :
//@               (ENTREE) idStatus : identificateur du status
//@               (SORTIE) status : status trouve
//@
//@ Valeur de retour :
//@                aucune
//@ ------------------------------------------------------------------*/


void MP__GestionnaireStatus::GetStatus(
		    const MP__HKTMStatus::t_IdStatus idStatus, 
		    MP__HKTMStatus& status)
{
   //@ Retrouver le status dans le gestionnaire
   status = m_ListeStatus[idStatus];
}

//@ ------------------------------------------------------------------*/
//@ Methode : SetStatus
//@
//@ Description : permet d'inserer un status
//@
//@ Parametres :
//@               (ENTREE) idStatus : identificateur du status
//@               (ENTREE) status : Status a inserer dans la liste
//@
//@ Valeur de retour :
//@               aucune
//@ ------------------------------------------------------------------*/

void MP__GestionnaireStatus::SetStatus(
		    const MP__HKTMStatus::t_IdStatus idStatus, 
		    const MP__HKTMStatus& status)

{
   //@ Inserer le status dans la collection
   m_ListeStatus[idStatus] = status;
}
