// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/MP/src/MP__MainProcess.cc,v 1.3 2011/11/25 10:17:44 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : MP
//@ CLASSE : MP__MainProcess
//@
//@ Commentaire :
//@    Ce fichier encapsule le main et les services de hauts niveaux
//@    du processus informatique MP (main process).
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 21/03/2000 : A.MONTMORY
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : PBR
//@ Adaptation a l'OPS	 
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-114-CN : 24/03/2004 : JL. PASCAL
//@ nettoyage du repertoire tmp au demarrage
//@ fiche de correction correspondante : IA-2100-FC-114-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-159-CN : 26/05/2005 : JL. PASCAL
//@ modif suite a portage linux
//@ fiche de correction correspondante : IA-2100-FC-159-THA
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------/

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/signal.h>
#include "CMN__PGEServices.h"
#include "ERR__Exception.h"
#include "MP__MainProcess.h"
#include "JDB__Callback.h"
#include "TES__Evenement.h"
#include "CMN__Securite.h"
#include "CFG__Configuration.h"
#include "CMN__Trace.h"

//=====================================================================
//                        DONNEES STATIQUES
//=====================================================================

static MP__MainProcess s_MPServeur;
//FA 114
//FA 159
static const t_Chaine CDE_CLEAN_BEG   = "/bin/sh -c 'rm -f ";
// FT-2765
static const t_Chaine CDE_CLEAN_END1   = "/*LOG* 2>/dev/null'";
static const t_Chaine CDE_CLEAN_END2   = "/*TRA* 2>/dev/null'";
static const t_Chaine CDE_CLEAN_END3   = "/*HKTM* 2>/dev/null'";
static const t_Chaine CDE_CLEAN_END4   = "/*TLM* 2>/dev/null'";
// ...FT2765

//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Procedure : MP__Terminer
//@
//@ Description : Arret du MP
//@
//@
//@ Parametres :
//@               (ENTREE) signal: numero du signal
//@
//@ Valeur de retour :
//@               Aucune
//@
//@
//@ ------------------------------------------------------------------*/

void MP__Terminer (int signal)
{
  CMN__PGEServices*  p_PGEServices=NULL;

  //@ Appel de la methode terminer du serveur
  s_MPServeur.Terminer();

  //@ Purge des fichiers geres par le sous-systeme
  //Chaine commande;
  //mmande = CFG__Configuration::GetChaine("COMMAND_PURGE");
  //stem (commande.c_str());
#ifdef DEBUG
  //ut << "Commande: " << commande << endl;
#endif

   //@ Initialiser l'instance de PGE services
   p_PGEServices = s_MPServeur.GetPGEservices();

   //@ Prevenir le MCS de l'arret du MP (Terminate)
   (void)p_PGEServices->MLATerminate ();	 

}

//@ ------------------------------------------------------------------*/
//@ Procedure : MP__Exit
//@
//@ Description : Arret immediat du MP
//@
//@ Parametres :
//@               (ENTREE) signal: numero du signal
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/

void MP__Exit (int signal)
{
  exit (0);
}

//@ ------------------------------------------------------------------*/
//@ Methode : MP__MainProcess
//@
//@ Description : Constructeur
//@
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@
//@ ------------------------------------------------------------------*/

MP__MainProcess::MP__MainProcess()
{
  //@ Construire une instance des PGE services
  m_GestionnaireActivite.SetPGEservices (true);
}

//@ ------------------------------------------------------------------*/
//@ Methode : ~MP__MainProcess
//@
//@ Description : Destructeur
//@
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@
//@ ------------------------------------------------------------------*/

MP__MainProcess::~MP__MainProcess()
{
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetPGEservices
//@
//@ Description : Recuperer le pointeur de l'instance des PGE services
//@
//@ Parametres :
//@               (SORTIE)pPGEservices  : pointeur sur l'instance de PGS 
//@                                       services courant
//@ Valeur de retour :
//@               Aucune
//@
//@
//@ ------------------------------------------------------------------*/

CMN__PGEServices* MP__MainProcess::GetPGEservices ()

{
  //@ Recuperer le pointeurl'adresse de l'instance des PGE services
  return (m_GestionnaireActivite.GetPGEservices ());

}
//@ ------------------------------------------------------------------*/
//@ Methode : Initialiser
//@
//@ Description : permet d'initialiser les traitements du serveur de
//@               message.
//@
//@ Parametres :
//@               (ENTREE) IdCmdStart : Id de la commande start
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/


void MP__MainProcess::Initialiser(int IdCmdStart)
{

   //@ Initialiser l'Id de la commande Start et le Mode
   m_GestionnaireActivite.SetIdStart (IdCmdStart);

   //@ Lecture du parametrage (nom Host et numero de service)
   t_Chaine host = CFG__Configuration::GetAdresse("MSG_SERVER_HOST");
   t_Chaine service = CFG__Configuration::GetAdresse("MSG_SERVER_PORT");

   //@ Installer les callback qui permettent d'envoyer les msg JDB
   JDB__Callback msgCallback;

   //@ Lancer tous les processus de service
   m_GestionnaireActivite.LancerProcessService();


#ifdef DEBUG
   //cout << "ATTENTE 20 sec POUR CAPTURE AVEC LE DEBUG " << getpid() << endl;
   //sleep (20);
#endif
  //@ Creer la connexion serveur en appelant le gestionnaire activite
   m_GestionnaireActivite.ConnecterServeur((char*)host.c_str(),
                                                (char*)service.c_str());
   
   //@ Lancer tous les processus applicatifs
   m_GestionnaireActivite.LancerProcessApplicatifs();
					
   //@ Initialiser les evenements timers utilises par le MP
   m_GestionnaireActivite.InitialiserEvtTimer();

}


//@ ------------------------------------------------------------------*/
//@ Methode : Lancer
//@
//@ Description : permet de lancer la boucle de scrutation des evenements
//@
//@
//@ Parametres :
//@               aucune
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/

void MP__MainProcess::Lancer()
{
  
   //@ Tantque pas fin
   while (!m_GestionnaireActivite.Terminer()) {
   
   //@ scruter les evenements
      m_GestionnaireActivite.Ecouter();
   }

}

//@ ------------------------------------------------------------------*/
//@ Methode : Terminer
//@
//@ Description : permet de terminer l'execution du serveur de message.
//@
//@ Parametres :
//@               aucune   
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/

void MP__MainProcess::Terminer()
{

  //@ Stopper tous les process permanents
  m_GestionnaireActivite.StopperProcessPermanent ();

  //@ Fermeture fichier TRACE
  CMN__Trace_Fin();
  TRCLOSE ();
  
}


//@ ------------------------------------------------------------------*/
//@ Methode : main
//@
//@ Description : programme principale du Main Process.
//@
//@ Parametres :
//@               aucune   
//@
//@ Valeur de retour :
//@               aucune
//@
//@ ------------------------------------------------------------------*/

// FT-2765
int main (int argc, char *argv[])
// ...FT-2765
{
  t_Chaine           idCommandStr;
  int                idCmdStart;
  t_CMN__Statut      status;
  t_Chaine           logMsg;
  t_Chaine           internalTrace;
  CMN__PGEServices*  p_PGEServices=NULL;
  t_Chaine argument;
//FA 114
  // Commande systeme
  t_Chaine       commande("");
  t_Chaine       repertoireTmp("");
  //@ Nettoyage du repertoire tmp (tmp)
  repertoireTmp = CFG__Configuration::GetRepertoire("TMP_DIR");
  //@ on prend des precautions
  if ( repertoireTmp != "")
   {
    commande = CDE_CLEAN_BEG + repertoireTmp + CDE_CLEAN_END1;
    system(commande.c_str());
    commande = CDE_CLEAN_BEG + repertoireTmp + CDE_CLEAN_END2;
    system(commande.c_str());
    commande = CDE_CLEAN_BEG + repertoireTmp + CDE_CLEAN_END3;
    system(commande.c_str());
    commande = CDE_CLEAN_BEG + repertoireTmp + CDE_CLEAN_END4;
    system(commande.c_str());
  }

   //@ Le MP recoit en entre 3 arguments  :
   //@   -  IdCommand
   //@   -  working root directory => stocke dans une variable d'environnement
   //@   -  mode : Nominal/Debug   => stocke dans une variable d'environnement

  //@ Preparation du fichier de Log du processus
  // FT-2765
  char *myargv[] = {
		  argv[1] ,
		  static_cast<char *>("-t"),
		  static_cast<char *>("MP.log"),
		  static_cast<char *>("-n"),
		  static_cast<char *>("4")
  };
  // ...FT-2765
  int   myargc   = 5;
  CMN__Trace_Init( myargc , myargv );  

   //@ Controle des arguments
   if (argc < 3) {
      internalTrace = "Usage : <IdCommand> <working root directory> <mode>";
      CMN__Trace_Print(internalTrace.c_str());
      internalTrace = "Number of arg : " + argc;
      CMN__Trace_Print(internalTrace.c_str());

   }  else {
      internalTrace = "MP is running";
      CMN__Trace_Print(internalTrace.c_str());
      } 

   switch (argc){
      case 3 :
        argument = argv[3];
        internalTrace = "Mode                   : ["  + argument +"]";
        CMN__Trace_Print(internalTrace.c_str());       

      case 2 :
        argument = argv[2];
        internalTrace = "Working root directory : [" + argument +"]";
        CMN__Trace_Print(internalTrace.c_str());       

      case 1 :
        argument = argv[1];
        internalTrace = "IdCommand              : [" + argument +"]";
        CMN__Trace_Print(internalTrace.c_str());       
	break;

      case 0 :
        internalTrace = "No argument is available !";
        CMN__Trace_Print(internalTrace.c_str());       
	break;
    }


   //@ Si le nombre d'argumet est incorrect
   if (argc < 3) {
      //@ Fermer le fichier de trace
      CMN__Trace_Fin();

      //@ Arreter l'OPS
      exit (1);
   }

   // Initialisation fichier trace
   TRINIT( argc , argv );

   //@ Initialiser l'instance de PGE services
   p_PGEServices = s_MPServeur.GetPGEservices();

   //@ Declaration du demarrage au MCS
   status = p_PGEServices->MLAInitialisation ();  

   //@ Si le status de l'initisalition du MCS est OK
   if (status == 0) {
          
       //@ Capture des exceptions
       try {
     	   
           //@ Positionner l'environnement pour la securite
//FA159
//           CMN__Securite_Init (MP__Exit, MP__Terminer);
          CMN__Securite_Init ( MP__Terminer,MP__Exit);

           //@ Initialisation du serveur de message avec l'Id cmde Start
           idCommandStr = argv[1];
           idCmdStart = int(idCommandStr);
           s_MPServeur.Initialiser(idCmdStart);

           //@ Lancer la boucle infini de scrutation des evenements
           s_MPServeur.Lancer();	 	    

           //@ On est sorti de la boucle on termine l'execution du processus
           s_MPServeur.Terminer();
	   
 	 
           //@ on signale au MCS la fin du processus (fait exit)
           status = p_PGEServices->MLATerminate ();	 
	     
       } 
       //@ Exploitation des exception    
       catch (ERR__Exception &exception) {
      
          //@ Impression des exceptions
          cout << exception.GetFile() << endl;
	  cout << exception.GetLine() << endl;
  	  cout << exception.GetErrorCode() << endl;
  	  cout << exception.GetErrno() << endl;
          //@ On essaye de terminer
          s_MPServeur.Terminer();  
	  

           //@ on signale au MCS la fin du processus (fait exit)
          status = p_PGEServices->MLATerminate ();	 
       }

   } // fin status ok 

  
   //@ erreur initialisation MCS exit(-1)
   exit (-1);
}
