// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/MSG/src/MSG__Abonnement.cc,v 1.3 2011/11/25 10:17:44 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : MSG
//@ CLASSE : MSG__Abonnement
//@
//@ Commentaire :
//@    Ce fichier gere la classe MSG__Abonnement
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 21/03/2000 : A.MONTMORY
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 14/12/2000 : C.LACAZE
//@  Portage sous systeme AIX 4.3
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL
#include <iostream>
#include <iterator>
#include <algorithm>
#include <sstream>

// Standard
#include <unistd.h>

// Projet
#include "ERR__Exception.h"
#include "MSG__Abonnement.h"
#include "CMN__Config.h"


//
// Definition des constantes 
//

// Version Classe
const  t_Chaine MSG__Abonnement::VERSION_CLASSE = "1.0";


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : MSG__Abonnement
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
MSG__Abonnement::MSG__Abonnement() : m_Action(UNKNOWN)
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~MSG__Abonnement
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
MSG__Abonnement::~MSG__Abonnement()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Serialiser
//@
//@ Description : Serialisation de l'objet
//@
//@ Parametres :
//@               (SORTIE) message : Message reseau
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSG__Abonnement::Serialiser(MSG__Message& message)
{
  typedef ostream_iterator<t_Chaine> t_ListeOutput;
  CMN__Configuration config;
  ostringstream      outString;
  t_Chaine           resultat("");

  
  t_ListeOutput out (outString,",");
  
  //@ Ecriture de la version de la classe
  config.EcrireValeur("SECTION_ENTETE","CHAMP_CLASSE",
                      VERSION_CLASSE);

  //@ Si la valeur de UNIQUE Adresse est vide alors en affecter une
  if (m_UniqueAdresse == "") m_UniqueAdresse = "UNIQUE"+ t_Chaine(getpid());       
  //@ Ecriture de la valeur UniqueAdresse
  config.EcrireValeur("UNIQUE_DG","VALEUR",m_UniqueAdresse);
  
  //@ Ecriture de l'action demandee
  config.EcrireValeur("ACTION","VALEUR",m_Action);
  
  //@ Boucle sur les adresses
  config.EcrireValeur("ADRESSES","NB",(int)m_ListeAdresse.size());
  
  //@ Utilisation d'un "algorithm" de la STL
  copy (m_ListeAdresse.begin(),m_ListeAdresse.end(),out);
  outString << ends;
  config.EcrireValeur("ADRESSES","ADRESSE",outString.str());
  
  //@ Sauvegarde de la serialisation dans une chaine
  config.SauverChaine(resultat);
  
  //@ Sauvegarde du bloc dans le message
  message.SetBloc (resultat.c_str(),resultat.size()+1);
  
  //@ Sauvegarde du type dans le message
  message.SetType(MSG__Message::ABONNEMENT);
  
  //@ Sauvegarde adresse NONE car destinee au Serveur MSGS
  message.SetAdresse("SERVEUR");
}


//@ ------------------------------------------------------------------*/
//@ Methode : Serialiser
//@
//@ Description : Serialisation de l'objet
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               message : Message reseau
//@
//@ ------------------------------------------------------------------*/
MSG__Message MSG__Abonnement::Serialiser()
{
  MSG__Message message;
  
  //@ Serialisation de l'objet
  Serialiser(message);
  
  //@ Retourne le message
  return (message);
}


//@ ------------------------------------------------------------------*/
//@ Methode : Deserialiser
//@
//@ Description : Deserialisation de l'objet
//@
//@ Parametres :
//@               (ENTREE) message : Message reseau contenant un objet
//@                                  MSG__Abonnement serialise
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSG__Abonnement::Deserialiser(const MSG__Message& message)
{
  t_Chaine           bloc (message.GetBloc());
  CMN__Configuration config;
  t_Chaine           version("");
  t_Chaine           nbAdresseChaine("");
  t_Chaine           Adresses("");
  t_ListeChaineTriee listeAdresse;
  t_Chaine           temp("");


  //@ SI le type du message est bien ABONNEMENT
  if (message.GetType() == MSG__Message::ABONNEMENT) 
    {
      //@ Lecture de la version de la classe
      config.ChargerChaine(bloc);
      config.LireValeur("SECTION_ENTETE","CHAMP_CLASSE",
                        version);

      //@ Lecture de la valeur de UNIQUE Adresse 
      config.LireValeur("UNIQUE_DG","VALEUR",m_UniqueAdresse);

      //@ Action demandee
      config.LireValeur("ACTION","VALEUR",temp);
      m_Action = (MSG__Abonnement::e_TypeAction)int(temp);
     
      //@ Nombre d'adresse
      config.LireValeur("ADRESSES","NB",nbAdresseChaine);
      int nbAdresse = nbAdresseChaine;
      config.LireValeur("ADRESSES","ADRESSE",Adresses);

      //@ Boucle sur les adresses
      int pos = 0;
      for (int i = 0; i < nbAdresse; i++) 
        {
          int posSeparateur = Adresses.find (",",pos);
          listeAdresse.insert(
                              Adresses.substr(pos, posSeparateur - pos));
          pos = posSeparateur + 1;     
        }
      
      //@ Si la taille de la listeAdresse est egale au nombre 
      //@ attendu alors on l'affecte dans l'objet
      if (listeAdresse.size() == nbAdresse) 
        {
          SetListeAdresse(listeAdresse);
        }
    }
  //@ FIN SI
}
