// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/MSG/src/MSG__Connexion.cc,v 1.3 2011/11/25 10:17:44 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : MSG
//@ CLASSE : MSG__Connexion
//@
//@ Commentaire :
//@    Ce fichier gere la classe MSG__Connexion
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 21/03/2000 : A.MONTMORY
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 14/12/2000 : C.LACAZE
//@  Portage sous systeme AIX 4.3
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL

// Standard
#include <strings.h>
#include <sys/select.h>
#include <errno.h>

// Projet
#include "CMN__Commun.h"
#include "ERR__Exception.h"
#include "MSG__Connexion.h"


//
// Definition des constantes 
//
const int MSG__Message::MSG_TAILLE_MAX=65535;


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : MSG__Connexion
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
MSG__Connexion::MSG__Connexion() : m_MaxDesc(0)
{   
  //@ Type de la connexion UNKNOWN
  m_Type = CNX_UNKNOWN;
  
  //@ Masque de socket a zero
  FD_ZERO(&m_MasqueSockets);
  
  //@ Structure timeout a l'infini (1 milliard de seconde)
  // FT-2765
#ifdef AIX
  m_Timeout.tv_sec = 1000000000;
#else
  m_Timeout.tv_sec = 1000000;
#endif
  // ...FT-2765
  m_Timeout.tv_usec = 0;

  //@ pas de verrou par defaut
  m_Verrou = NULL;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~MSG__Connexion
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
MSG__Connexion::~MSG__Connexion()
{
  //@ liberer le verrou si il existe
  if ( m_Verrou ) delete m_Verrou;
}

//@ ------------------------------------------------------------------*/
//@ Methode : GererVerrou
//@
//@ Description : Permet de mettre en marche la gestion de verrou
//@               pour les PublierMessage en multithreading.
//@
//@ Parametres :
//@               Aucun
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSG__Connexion::GererVerrou()
{
	if ( !m_Verrou ) m_Verrou = new CMN__Verrou();
}

//@ ------------------------------------------------------------------*/
//@ Methode : Ouvrir
//@
//@ Description : Permet d'ouvrir une connexion
//@
//@ Parametres :
//@               (ENTREE) host : Nom de la machine ou se connecter
//@               (ENTREE) service : Numero de port de la connexion
//@               (ENTREE) type : Type de la connexion
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSG__Connexion::Ouvrir(char *host, char *service, e_TypeConnexion type)
{
  // Declaration objet exception
  ERR__Exception exception;
  

  //@ SI l'objet socket n'existe pas
  if (GetSocket().GetSocketDescripteur() == MSG__Socket::SOCKET_UNKNOWN )
    {
      //@ On ouvre le socket
      m_Socket.Ouvrir();

      //@ On initialise le descripteur
      MSG__Descripteur descripteur;
      
      //@ On initialise le type de la connexion
      SetType(type);
      
      //@ SELON le type de la connexion   
      switch (type)
	{
	//@ Si type = CNX_SERVEUR alors on se met a l'ecoute
	case CNX_SERVEUR :
	  m_Socket.Ecouter(host, service);
	  
	  //@ Ajouter le descripteur dans le masque
	  descripteur.SetType(MSG__Descripteur::SOC_SERVEUR);
	  descripteur.SetDescripteur(m_Socket.GetSocketDescripteur());
	  SetMasque(descripteur);	          
	  break;
	  
	//@ Si type = CNX_CLIENTE alors on se connecte au serveur
	case CNX_CLIENTE :
	  //@ Ajouter le descripteur dans le masque
	  descripteur.SetType(MSG__Descripteur::SOC_CLIENT);
	  descripteur.SetDescripteur(m_Socket.GetSocketDescripteur());
	  SetMasque(descripteur);	          
	  m_Socket.Connecter(host, service);
	  break;       	
	   
	//@ Defaut: type inconnu
	default : 
	  //@ On envoie une exception UNKNOWN_CNX_TYPE
	  exception.SetErrorCode (ERR__Exception::UNKNOWN_CNX_TYPE);
	  THROW (exception);   
	  break;
	}
      //@ FIN SELON
    } 
  //@ SINON
  else
    {
      //@ On envoie une exception SOCKET_EXIST
      exception.SetErrorCode (ERR__Exception::SOCKET_EXIST);
      THROW (exception);   
    }
  //@ FIN SI 
}


//@ ------------------------------------------------------------------*/
//@ Methode : Fermer
//@
//@ Description : Permet de fermer une connexion
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSG__Connexion::Fermer()
{
  // Declaration objet exception
  ERR__Exception exception;
  
  
  //@ SI l'objet socket existe 
  if (GetSocket().GetSocketDescripteur() != MSG__Socket::SOCKET_UNKNOWN)
    {
      //@ On ferme le socket
      m_Socket.Fermer();
    } 
  //@ FIN SI
  
  //@ Le type de la connexion est mis a inconnu
  m_Type = CNX_UNKNOWN;
}


//@ ------------------------------------------------------------------*/
//@ Methode : PublierMessage 
//@
//@ Description : Permet de publier un message
//@
//@ Parametres :
//@               (ENTREE) message : Message a diffuser
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSG__Connexion::PublierMessage( MSG__Message& message)
{
  ERR__Exception exception;
  MSG__Message::t_Entete entete;
  int longueurMsg = message.GetLongueur();
  char *bloc = message.GetBloc();
  int tailleMax = message.GetTailleMax();
  
  
  //@ SI le type de la connexion est connu
   if (GetType() != CNX_UNKNOWN)
     {
       //@ Il faut eventuellement decouper le message
       int nbBloc = (longueurMsg + tailleMax -1)/tailleMax;
       message.SetNbBloc(nbBloc);

       //@ verrou pour la gestion du multithreading
       if ( m_Verrou ) m_Verrou->Bloquer();
       //@ On envoie l'entete sur le socket
       entete = message.GetEntete();
       m_Socket.Envoyer((void *) &entete, sizeof(entete));
       
       //@ POUR tous les blocs
       int offset = 0;
       for (int i = 0; i < nbBloc ; i++)
	 {
	   //@ On envoie le corps du message sur le socket
	   m_Socket.Envoyer((void *) (bloc+offset), 
			    MIN(longueurMsg,tailleMax));

	   //@ On avance les pointeurs		  
	   offset += tailleMax;
	   longueurMsg -= tailleMax;  
	 }
       //@ FIN POUR tous les blocs
       //@ verrou pour la gestion du multithreading
       if ( m_Verrou ) m_Verrou->DeBloquer();
     }
   //@ SINON
   else
     {
       //@ On envoie une exception CNX_NOT_OPEN
       exception.SetErrorCode (ERR__Exception::CNX_NOT_OPEN);
       THROW (exception);   
     }
   //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : LireMessage 
//@
//@ Description : Permet de lire un message sur une adresse logique
//@
//@ Parametres :
//@               (SORTIE) message : Message lu
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSG__Connexion::LireMessage(MSG__Message& message)
{
  // Declaration objet exception
  ERR__Exception exception;

  int taille;
  MSG__Message::t_Entete entete;
  char *bloc;
  
  
  //@ SI le type de la connexion est connu
  if (GetType() != CNX_UNKNOWN)
    {
      //@ On lit l'entete sur le socket
      int offset = 0;
      int longueurEntete = sizeof(entete);
      while (longueurEntete) 
	{
	  taille = m_Socket.Recevoir((void *)((char*)&entete+offset),
				     longueurEntete);

	  offset += taille;
	  longueurEntete -= taille;  
	}
      
      //@ Analyse de l'entete
      message.SetEntete(entete);
      int longueurMsg = message.GetLongueur();
      int tailleMax = message.GetTailleMax();

      //@ Allocation du paquet de reception
      message.SetBloc(0,longueurMsg);
      bloc = message.GetBloc();
      
      //@ TANT QUE le message n'est pas entierement lu
      offset = 0;
      while (longueurMsg) 
	{
	  //@ Reception d'un bloc
	  taille = m_Socket.Recevoir((void *) (bloc+offset), 
				     MIN(longueurMsg,tailleMax));

	  //@ On avance les pointeurs		  
	  offset += taille;
	  longueurMsg -= taille;  
	}
      //@ FIN TANT QUE
    }
  //@ SINON
  else 
    {
      //@ On envoie une exception CNX_NOT_OPEN
      exception.SetErrorCode (ERR__Exception::CNX_NOT_OPEN);
      THROW (exception);   
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : SetMasque 
//@
//@ Description : Permet de rajouter un descripteur dans le masque de
//@               la connexion
//@
//@ Parametres :
//@               (ENTREE) descripteur : Descripteur de socket
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSG__Connexion::SetMasque(const MSG__Descripteur& descripteur)
{
  //@ SI le descripteur n'est pas deja positionne
  if (!FD_ISSET(descripteur.GetDescripteur(),&m_MasqueSockets))
    {
      //@ On ajoute le descripteur de socket au masque
      FD_SET (descripteur.GetDescripteur(),&m_MasqueSockets);
    
      //@ On rajoute l'objet descripteur dans la liste
      m_ListeDescripteur.insert(descripteur);
      
      //@ Stockage du max du numero de descripteur (pour select)
      m_MaxDesc = MAX(m_MaxDesc,descripteur.GetDescripteur()); 
    }
  //@ FIN SI   
}


//@ ------------------------------------------------------------------*/
//@ Methode : SubMasque 
//@
//@ Description : Permet de supprimer un descripteur dans le masque
//@               de la connexion
//@
//@ Parametres :
//@               (ENTREE) descripteur : Descripteur de socket
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSG__Connexion::SubMasque( const MSG__Descripteur& descripteur )
{
  //@ SI le descripteur est deja positionne
  if (FD_ISSET(descripteur.GetDescripteur(),&m_MasqueSockets))
    {
      //@ On ajoute le descripteur de socket au masque
      FD_CLR (descripteur.GetDescripteur(),&m_MasqueSockets);
      
      //@ On rajoute l'objet descripteur dans la liste
      m_ListeDescripteur.erase(descripteur);
   }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : Scruter 
//@
//@ Description : Permet de detecter une activite sur un des sockets
//@               du masque de la connexion
//@
//@ Parametres :
//@               (SORTIE) descripteurActif : Liste des sockets actifs
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSG__Connexion::Scruter(t_ListeDescripteur& descripteurActif)
{
   // Declaration objet exception
   ERR__Exception exception;

   int retour;
   int taille;
   fd_set sauvList;
   fd_set writeList;
   fd_set exceptList;
   struct timeval selectTime;
     
   
   //@ RAZ des masques
   FD_ZERO(&writeList);
   FD_ZERO(&exceptList);

   //@ SI le type de la connexion est connu
   if (GetType() != CNX_UNKNOWN) 
     {
       //@ Sauvegarde du masque (il est modifie par select)
       sauvList = m_MasqueSockets;
       selectTime = m_Timeout;
      
       //@ Appel select
       retour = select(m_MaxDesc+1,&sauvList, &writeList,
		       &exceptList, &selectTime);

       //@ SELON retour
       switch (retour)
	 {
	 //@ Si retour = -1: erreur systeme
	 case -1 :
	   //@ Emission exception SOCKET_ERROR
	   exception.SetErrorCode (ERR__Exception::SOCKET_ERROR);
	   exception.SetErrno (errno);
           THROW (exception);
	   break;
	   
	 //@ Si retour = 0: timeout	 
	 case 0 : 
	   //@ On ne fait rien 
	   //@ descripteurActif.size() = 0 indique
	   // que l'on est sorti en timeout
	   break;
	   
	 //@ Defaut:
         default : 
	   //@ On construit la liste descripteurActif des
	   //@ sockets actifs

	   //@ POUR tous les descripteurs presents dans la
	   //@ liste m_ListeDescripteur
	   t_ListeDescripteur::iterator pInfo;
	   for ( pInfo = m_ListeDescripteur.begin(); 
		 pInfo !=  m_ListeDescripteur.end();
		 pInfo++) 
	     {
	       //@ Si le socket est actif (FD_ISSET),
	       //@ le rajouter dans la liste descripteurActif
	       if (FD_ISSET((*pInfo).GetDescripteur (),&sauvList))
		 {
		   descripteurActif.insert(*pInfo);
		 }
	     }
	   //@ FIN POUR
	 }      
       //@ FIN SELON
     } 
   //@ SINON
   else
     {
       //@ On envoie une exception CNX_NOT_OPEN
       exception.SetErrorCode (ERR__Exception::CNX_NOT_OPEN);
       THROW (exception);   
     }
   //@ FIN SI
}   


//@ ------------------------------------------------------------------*/
//@ Methode : SetTimeout 
//@
//@ Description : Permet de positionner un timeout utilise par
//@                scruter; 0 = timeout infini
//@
//@ Parametres :
//@               (ENTREE) timeout : Valeur de timeout
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSG__Connexion::SetTimeout(const double timeout)
{
  //@ Affecte le parametre timeout dans m_Timeout
  m_Timeout.tv_sec = (time_t) timeout; 
  m_Timeout.tv_usec = long ((timeout - (double) m_Timeout.tv_sec) * 1000000);  
}
