// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/MSG/src/MSG__Connexion.h,v 1.3 2011/11/25 10:17:44 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : MSG
//@ CLASSE : MSG__Connexion
//@
//@ Commentaire :
//@    Ce fichier gere la classe MSG__Connexion
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 21/03/2000 : A.MONTMORY
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef MSG__Connexion_h
#define MSG__Connexion_h

//
// Fichiers include
//
// FT-2765
#include <sys/time.h>
// ...FT-2765
// STL
#include <set>

// Standard

// Projet
#include "MSG__Socket.h"
#include "MSG__Message.h"
#include "CMN__Verrou.h"


//
// Classe MSG__Descripteur pour decrire un descripteur de socket
//

class MSG__Descripteur {

    /*=================================================================*/

 public:
    
    //
    // Type de descripteur EXTERNE ou MSG
    //
    typedef enum {
	SOC_UNKNOWN = 0,
	SOC_CLIENT = 1,
	SOC_SERVEUR = 2,
	SOC_EXTERNAL = 3 	
    } e_TypeSocket;
	
    //
    // Constructeur
    //
    MSG__Descripteur() : m_Descripteur(0),m_TypeSocket(SOC_UNKNOWN){};
    MSG__Descripteur(MSG__Socket::t_SocketDescripteur Descripteur,
		     e_TypeSocket Type) : m_Descripteur(Descripteur),
	m_TypeSocket(Type) {};

    //
    // Destructeur
    //
    ~MSG__Descripteur() {};
    
    //
    // Accesseurs
    //
    MSG__Socket::t_SocketDescripteur GetDescripteur () const
	{ return m_Descripteur;};
    e_TypeSocket GetType () const { return m_TypeSocket;};
    void SetDescripteur (const MSG__Socket::t_SocketDescripteur Descripteur)
	{m_Descripteur = Descripteur;};
    void SetType (const e_TypeSocket Type) {m_TypeSocket = Type;};
    bool operator< (const MSG__Descripteur& desc) const 
	{return m_Descripteur < desc.GetDescripteur();};	   


    /*=================================================================*/

protected:
    
    //
    // Methodes protegees
    //

    //
    // Attributs proteges
    //


    /*=================================================================*/

 private:

    //
    // Methodes privees
    //
    
    //
    // Attributs prives
    //
    MSG__Socket::t_SocketDescripteur m_Descripteur;
    e_TypeSocket m_TypeSocket;
};


class MSG__Connexion {

    /*=================================================================*/

 public:
    
    //
    // Type de la connexion
    //
    typedef enum {
	CNX_UNKNOWN = 0,
	CNX_SERVEUR = 1,
	CNX_CLIENTE =2 	
    } e_TypeConnexion;
	
    //
    // Liste des descripteurs
    //
    typedef set<MSG__Descripteur, less<MSG__Descripteur> > t_ListeDescripteur;
    
    //
    // Constructeur
    //
    MSG__Connexion();
    
    //
    // Destructeur
    //
    ~MSG__Connexion();
    
    //
    // Accesseurs
    //
    e_TypeConnexion GetType() const {return m_Type;};
    void SetType (const e_TypeConnexion type) {m_Type = type;};
    MSG__Socket GetSocket () const {return m_Socket;};
    void SetSocket ( MSG__Socket socket) {m_Socket = socket;};
    
    //
    // Methodes publiques
    //
    void GererVerrou();
    void Ouvrir(char *host, char *service, e_TypeConnexion type);
    void Fermer();
    void PublierMessage( MSG__Message& message);
    void LireMessage(MSG__Message& message);
    void SetMasque(const MSG__Descripteur& descripteur);
    void SubMasque(const MSG__Descripteur& descripteur);
    void Scruter(t_ListeDescripteur& descripteurActif);
    void SetTimeout(const double timeout);


    /*=================================================================*/

 protected:

    //
    // Methodes protegees
    //

    //
    // Attributs proteges
    //
    
	
    /*=================================================================*/

 private:
    
    //
    // Methodes privees
    //
    
    //
    // Attributs prives
    //
	
    // Pointeur sur l'objet socket
    MSG__Socket        m_Socket;
    
    // Type de la connexion (SERVEUR ou CLIENTE)
    e_TypeConnexion    m_Type;
	
    // Masque des sockets utilise par Scruter()
    fd_set             m_MasqueSockets;
	
    // Liste des descripteurs positionnes dans le masque
    t_ListeDescripteur m_ListeDescripteur;
	
    // Timeout eventuel utilise par scruter
    struct timeval     m_Timeout;
    
    // Max des descripteurs (pour select)
    int                m_MaxDesc;

    // verrou pour le multithreading
    CMN__Verrou *m_Verrou;
};

#endif
