// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/MSG/src/MSG__Message.h,v 1.3 2011/11/25 10:17:44 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : MSG
//@ CLASSE : MSG__Message
//@
//@ Commentaire :
//@    Ce fichier gere la classe MSG__Message
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 21/03/2000 : A.MONTMORY
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 16/07/2002 : PBR
//@ Adaptation a l'OPS : rajout du type de message TRA_JDB et APPLICATIF_XX
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-3316(DM-2100-OPS) : 18/05/2017 : MPE
//@ PORTAGE OPS SOUS RHEL 6.5 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef MSG__Message_h
#define MSG__Message_h

//
// Fichiers include
//

// STL

// Standard

// Migration RHEL 6.5
#ifdef LINUX
#include <cstring>
#endif

// Projet
#include "CMN__Commun.h"


static const int MSG_TAILLE_ADR = 32;

class MSG__Message {

    /*=================================================================*/

public:
        //
	// Type de message defini par l'architecture (ABONNEMENT, FES, 
	// TES etc..), les messages aplicatifs doivent etre d'un numero
	// superieur a MSG_APPLICATIF
	//
        typedef enum {
	   BASE_MSG = 0,
           UNKNOWN = BASE_MSG,
	   ABONNEMENT = BASE_MSG+1,
	   FIN = BASE_MSG+2,
	   EVT_TIMER = BASE_MSG+3,
	   EVT_JDB = BASE_MSG+4,
           TRA_JDB = BASE_MSG+5,
           EVT_PROCESS = BASE_MSG+6,
	   APPLICATIF_MP = BASE_MSG+200,
	   APPLICATIF_TES = BASE_MSG+300,
	   APPLICATIF_WOM = BASE_MSG+400,
	   APPLICATIF_JDBS = BASE_MSG+500,
	   APPLICATIF_SD = BASE_MSG+600
	} t_TypeMessage;     
  
    //
    // Type
    //
    typedef int t_Type;
    
    //
    // Constantes
    //
    static const int MSG_TAILLE_MAX;

    typedef struct {
	// Taille max du message
	int tailleMax;
	
	// Longueur du paquet d'octet qui suit entete
	int longueur;
	
	// Type du message recu  
	t_Type type;
	
	// Nombre de bloc a lire pour recontituer le message complet  
	int nbBloc;
	
	// Adresse du message  
	char adresse[MSG_TAILLE_ADR];
    } t_Entete;
	
	
    //
    // Constructeur
    //
    MSG__Message():m_Bloc(0) {
	m_Entete.tailleMax = MSG_TAILLE_MAX;
	m_Entete.nbBloc = 0;
	m_Entete.longueur = 0;
	m_Entete.type = 0;
	m_Entete.adresse[0] = 0;
    };

    //
    // Constructeur copy
    //
    MSG__Message(const MSG__Message& old) {
	m_Entete.tailleMax = old.m_Entete.tailleMax;
	m_Entete.nbBloc = old.m_Entete.nbBloc;
	m_Entete.longueur = old.m_Entete.longueur;
	m_Entete.type = old.m_Entete.type;
	memcpy (m_Entete.adresse,
		old.m_Entete.adresse,
		sizeof(m_Entete.adresse));
	if (old.m_Entete.longueur)
	    m_Bloc = new char [old.m_Entete.longueur];
	memcpy(m_Bloc,old.m_Bloc,old.m_Entete.longueur);
    };
    
    //
    // operateur =
    //
    MSG__Message& operator=(const MSG__Message& old) {
	if (&old != this) {
	    if (m_Bloc) delete [] m_Bloc;                         
	    m_Entete.tailleMax = old.m_Entete.tailleMax;
	    m_Entete.nbBloc = old.m_Entete.nbBloc;
	    m_Entete.longueur = old.m_Entete.longueur;
	    m_Entete.type = old.m_Entete.type;
	    memcpy (m_Entete.adresse,
		    old.m_Entete.adresse,
		    sizeof(m_Entete.adresse));
	    if (old.m_Entete.longueur)
		m_Bloc = new char [old.m_Entete.longueur];
	    memcpy(m_Bloc,old.m_Bloc,old.m_Entete.longueur);
	}
	return (*this);
    };
    
    //
    // Destructeur
    //
    ~MSG__Message() { if (m_Bloc) delete [] m_Bloc;};
    
    //
    // Accesseurs
    //
    t_Type GetType() const {return m_Entete.type;};
    void SetType(const t_Type type) {m_Entete.type = type;};
    int GetNbBloc() const {return m_Entete.nbBloc;};
    void SetNbBloc(const int nbBloc) {m_Entete.nbBloc = nbBloc;};
    int GetLongueur() const {return m_Entete.longueur;};
    int GetTailleMax() const {return m_Entete.tailleMax;};
    char * GetBloc() const {return m_Bloc;};
    void SetBloc (const char * bloc, int taille) { 
	if (m_Bloc) delete [] m_Bloc;
	m_Bloc = new char [taille];
	if (bloc) memcpy (m_Bloc,bloc,taille);
	m_Entete.longueur = taille;
    };
    t_Chaine GetAdresse() const {return t_Chaine(m_Entete.adresse);};
    void SetAdresse (t_Chaine adresse) { 
	strncpy( m_Entete.adresse,
		 adresse.c_str(),MSG_TAILLE_ADR-1);
    };			 
    t_Entete GetEntete() const {return m_Entete;};
    void SetEntete(const t_Entete entete) {m_Entete = entete;};
    

    //
    // Methodes publiques
    //

    
    /*=================================================================*/

 protected:
    
    //
    // Methodes protegees
    //
    
    //
    // Attributs proteges
    //

	
    /*=================================================================*/

 private:
    
    //
    // Methodes privees
    //

    //
    // Attributs prives
    //
    
    // Entete du message
    //
    t_Entete m_Entete;
    
    // Paquet d'octet contenant l'information
    char* m_Bloc;
};

#endif
