// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/MSG/src/MSG__Socket.cc,v 1.3 2011/11/25 10:17:44 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : MSG
//@ CLASSE : MSG__Socket
//@
//@ Commentaire :
//@    Ce fichier gere la classe MSG__Socket
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ VERSION : 21/03/2000 : A.MONTMORY
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-159-CN : 26/05/2005 : JL. PASCAL
//@ modif suite a portage linux
//@ fiche de correction correspondante : IA-2100-FC-159-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-184-CN : 18/10/2006 : C.MERIGUET
//@ Codage des fonctions reentrantes
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-3316(DM-2100-OPS) : 18/05/2017 : MPE
//@ PORTAGE OPS SOUS RHEL 6.5 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL

// Standard
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/socketvar.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <signal.h>
// Migration RHEL 6.5
#ifdef LINUX
#include <cstdlib>
#include <cstring>
#endif


// FT-2765
#include <pthread.h>

#ifdef HASNT_INADDR_NONE
#define INADDR_NONE 0xffffffffU
#endif
// ...FT-2765

// Projet
#include "ERR__Exception.h"
#include "CMN__Commun.h"
#include "MSG__Socket.h"


//
// Definition des constantes 
//
const int MSG__Socket::SOCKET_UNKNOWN = -1;


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : MSG__Socket
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
MSG__Socket::MSG__Socket()
{
  // Initialisation descripteur socket
  m_SocketDescripteur = (t_SocketDescripteur) SOCKET_UNKNOWN;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~MSG__Socket
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
MSG__Socket::~MSG__Socket()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Envoyer
//@
//@ Description : Permet d'envoyer un paquet d'octet
//@
//@ Parametres :
//@               (ENTREE) void* paquet : Paquet d'octet a envoyer
//@               (ENTREE) int   lg     : Longueur du paquet   
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSG__Socket::Envoyer(void *paquet, int lg)
{
  // Declaration objet exception
  ERR__Exception exception;

  int retour;
  struct sigaction action;
  struct sigaction old_action;
  
  
  //@ SI le descripteur de socket est ouvert
  if (GetSocketDescripteur() != SOCKET_UNKNOWN)
    {
      //@ On a besoin de bloquer le signal SIGPIPE
      action.sa_handler = SIG_IGN;
      sigemptyset(&action.sa_mask);
      action.sa_flags = 0;
      sigaction(SIGPIPE, &action, &old_action);
      
      //@ On envoie le message
      retour = send (GetSocketDescripteur(),paquet,(size_t) lg, 0);
      
      //@ On restaure le handler de l'utilisateur   
      sigaction(SIGPIPE, &old_action, NULL);
      
      //@ SI retour = -1: erreur systeme 
      if (retour == -1) 
	{
	  //@ Emission exception SOCKET_ERROR
	  exception.SetErrorCode (ERR__Exception::SOCKET_ERROR);
	  exception.SetErrno (errno);
	  THROW(exception);   
	}
      //@ FIN SI
    } 
  //@ SINON
  else
    {
      //@ On envoie une exception SOCKET_UNDEFINED
      exception.SetErrorCode (ERR__Exception::SOCKET_UNDEFINED);
      THROW (exception);   
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : Recevoir
//@
//@ Description : Permet de recevoir un paquet sur le socket
//@
//@ Parametres :
//@               (ENTREE) void* paquet : Paquet d'octet a recevoir 
//@                                       alloue a lg octet
//@               (ENTREE) int   lg    : Longueur du paquet   
//@
//@ Valeur de retour :
//@               Longueur effectivement lue
//@
//@ ------------------------------------------------------------------*/
int MSG__Socket::Recevoir(void *paquet, int lg)
{
  // Declaration objet exception
   ERR__Exception exception;

   int retour = 0;


   //@ SI le descripteur de socket est ouvert
   if (GetSocketDescripteur() != SOCKET_UNKNOWN) 
     {
       //@ Reception du message
       retour = recv (GetSocketDescripteur(),paquet,(size_t) lg, 0);
    
       //@ SELON retour
       switch (retour)
	 {
	 //@ Si retour = 0: la connexion est coupee
	 case 0 : 
	   //@ Emission exception SOCKET_BROKEN
	   exception.SetErrorCode (ERR__Exception::SOCKET_BROKEN);
	   THROW (exception); 
	   break;
	 
	 //@ Si retour = -1: erreur systeme   
	 case -1 :
	   //@ Emission exception SOCKET_ERROR 
	   exception.SetErrorCode (ERR__Exception::SOCKET_ERROR);
	   exception.SetErrno (errno);
	   THROW (exception);
	   break;

	 //@ Defaut:
	 default : 
	   //@ On retourne la longueur lue.	
	   return retour;      
	 }
       //@ FIN SELON
     }
   //@ SINON
   else
     {
       //@ On envoie une exception SOCKET_UNDEFINED
       exception.SetErrorCode (ERR__Exception::SOCKET_UNDEFINED);
       THROW (exception);   
     }
   //@ FIN SI
   
   return retour;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Ecouter
//@
//@ Description : Permet d'ecouter une activite sur le socket
//@
//@ Parametres :
//@               (ENTREE) host : Nom de la machine ou se connecter
//@               (ENTREE) service : Numero de port de la connexion 
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSG__Socket::Ecouter(char *host, char *service)
{
   // Declaration objet exception
   ERR__Exception exception;

   int retour;
   struct sockaddr_in sin;
   

   //@ SI le descripteur de socket est ouvert
   if (GetSocketDescripteur() != SOCKET_UNKNOWN)
     {
       //@ Positionner dans la structure sockaddr_in l'adresse
       //@ ou se connecter
       memset((void*)&sin, 0, sizeof(sin)); //  Mise a zero 
       sin.sin_family = AF_INET;   //  Famille Internet
       sin.sin_addr.s_addr = INADDR_ANY;
       
       //@ Recherche du numero de port
       sin.sin_port = htons(atoi(service));
       
       //@ SI le service n'est pas un numero de port
       if (!sin.sin_port) 
	 {
	   //@ On le recherche dans /etc/service 
    	   /* FA 184 */
#ifdef AIX
	   struct servent iservice;
	   struct servent_data service_data;
	   retour = getservbyname_r(service, "tcp",&iservice,&service_data);
	   //@ Si le service est inconnu alors 
	   //@ exception UNKNOWN_SERVICE
	   if (0 == retour)
	     {
	       sin.sin_port = iservice.s_port;
	     }
	   else
	     {
	       exception.SetErrorCode (ERR__Exception::UNKNOWN_SERVICE);
	       THROW (exception);            
	     }
#else // LINUX
	   // FT-2765/FT-2766
       // getservbyname_r does not exist everywhere
   	   struct servent *iservice = NULL;
       static pthread_mutex_t getservbyname_mutex = PTHREAD_MUTEX_INITIALIZER;

       pthread_mutex_lock( &getservbyname_mutex );
   	   iservice = getservbyname( service, "tcp" );

   	   //@ Si le service est inconnu alors
   	   //@ exception UNKNOWN_SERVICE
   	   if (iservice)
   	     {
   	       sin.sin_port = iservice->s_port;
                  pthread_mutex_unlock( &getservbyname_mutex );
   	     }
   	   else
   	     {
                  pthread_mutex_unlock( &getservbyname_mutex );
   	       exception.SetErrorCode (ERR__Exception::UNKNOWN_SERVICE);
   	       THROW (exception);
   	     }
	   // ...FT-2765/FT-2766
#endif
      	/* ..FA 184 */
	   
	 }
       //@ FIN SI
       
       //@ Association de l'adresse au socket avec l'appel bind
       retour = bind(GetSocketDescripteur(), (struct sockaddr *)
		     & sin, sizeof(sin));  	
       
       //@ On effectue un appel listen
       retour = listen (GetSocketDescripteur(),SOMAXCONN);

       //@ Si retour = -1: erreur systeme
     } 
   //@ SINON
   else 
     {
       //@ On envoie une exception SOCKET_UNDEFINED
       exception.SetErrorCode (ERR__Exception::SOCKET_UNDEFINED);
       THROW (exception);   
     }
   //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : Ouvrir
//@
//@ Description : Permet d'ouvrir un socket
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSG__Socket::Ouvrir()
{
   // Declaration objet exception
   ERR__Exception exception;

   int retour;
   int on = 1;


   //@ SI le descripteur de socket n'est pas ouvert
   if (GetSocketDescripteur() == SOCKET_UNKNOWN) 
     {
       //@ On effectue un appel socket
       retour = socket(AF_INET, SOCK_STREAM, 0);
       
       //@ SELON retour
       switch (retour)
	 {
	 //@ Si retour = -1: erreur systeme
	 case -1 :
	   //@ Emission exception SOCKET_ERROR
	   exception.SetErrorCode (ERR__Exception::SOCKET_ERROR);
	   exception.SetErrno (errno);
	   THROW (exception);
	   break;
	   
	 //@ Defaut:
	 default : 
	   //@ On enregistre le socket descripteur dans
	   //@ m_SocketDescripteur
	   m_SocketDescripteur = retour;      
	   
	   //@ On positionne l'option SO_REUSEADDR sur le socket, 
	   //@ pour pouvoir le reutiliser
	   setsockopt(retour, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));  
	 }
       //@ FIN SELON
     } 
   //@ SINON
   else
     {
       //@ On envoie une exception SOCKET_OPEN
       exception.SetErrorCode (ERR__Exception::SOCKET_OPEN);
       THROW (exception);   
     }
   //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : Fermer
//@
//@ Description : Permet de fermer un socket
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSG__Socket::Fermer()
{
  // Declaration objet exception
  ERR__Exception exception;

  int retour;
   
  
  //@ SI le descripteur de socket est ouvert
  if (GetSocketDescripteur() != SOCKET_UNKNOWN)
    {
      //@ on le ferme et il devient indefini
      close(GetSocketDescripteur());
      m_SocketDescripteur = SOCKET_UNKNOWN;
      
    } 
  //@ SINON 
  else
    {
      //@ On envoie une exception SOCKET_UNDEFINED
      exception.SetErrorCode (ERR__Exception::SOCKET_UNDEFINED);
      THROW (exception);   
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : Accepter 
//@
//@ Description : Permet d'acepter une connexion sur un socket sur 
//@               lequel on etait a l'ecoute
//@
//@ Parametres :
//@               (SORTIE) newSocket : Nouvel objet socket cree suite
//@                                    a l'acceptation
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSG__Socket::Accepter(MSG__Socket& newSocket)
{
  // Declaration objet exception
  ERR__Exception exception;

  int retour;
  struct sockaddr_in address;
  //FA159
//  size_t address_length;
  socklen_t address_length;
  address_length = sizeof(struct sockaddr_in);
  
  //@ SI le descripteur de socket est ouvert
  if (GetSocketDescripteur() != SOCKET_UNKNOWN) 
    {
      //@ On accepte la connexion ce qui rend un nouveau socket
      retour = accept(m_SocketDescripteur,
		      (struct sockaddr *) & address,
		      &address_length);
      
      //@ SELON retour
      switch (retour)
	{
	//@ Si retour = -1: erreur systeme
	case -1 :
	  //@ Emission exception SOCKET_ERROR
	  exception.SetErrorCode (ERR__Exception::SOCKET_ERROR);
	  exception.SetErrno (errno);
	  THROW (exception);
	  break;
	  
	//@ Defaut:	
	default : 
	  //@ On cree un nouvel objet socket avec le nouveau
	  //@ descripteur retourne par accept
	  newSocket.SetSocketDescripteur (retour);
	  break;
	}      
      //@ FIN SELON   
    }
  //@ SINON
  else
    {
      //@ On envoie une exception SOCKET_UNDEFINED
      exception.SetErrorCode (ERR__Exception::SOCKET_UNDEFINED);
      THROW (exception);      
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : Connecter 
//@
//@ Description : Permet de se connecter sur un socket serveur qui est
//@               en listen
//@
//@ Parametres :
//@               (ENTREE) host : Nom de la machine ou se connecter
//@               (ENTREE) service : Numero de port de la connexion 
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSG__Socket::Connecter(char *host, char *service)
{
  // Declaration objet exception
  ERR__Exception exception;
  
  int retour;
  struct sockaddr_in sin;
/* FA 184 */
// FT-2765
#ifdef AIX
  struct servent iservice;
  struct hostent ihost;
  struct hostent_data host_data;
#else // LINUX
  struct servent *iservice;
  struct hostent *ihost;
#endif
// ...FT-2765
/* ..FA 184 */

  //@ SI le descripteur de socket est ouvert
  if (GetSocketDescripteur() != SOCKET_UNKNOWN)
    {
      //@ Positionner dans la structure sockaddr_in l'adresse 
      //@ ou se connecter
      memset((void *)&sin, 0, sizeof(sin)); //  Mise a zero 
      sin.sin_family = AF_INET;   //  Famille Internet
      
      //@ Recherche du numero de port
      sin.sin_port = htons(atoi(service));
      
      //@ SI le service n'est pas un numero de port
      if (!sin.sin_port)
	{
/* FA 184 */
// FT-2765
#ifdef AIX
       struct servent iservice;
	   struct servent_data service_data;
	   retour = getservbyname_r(service, "tcp",&iservice,&service_data);
	   //@ Si le service est inconnu alors 
	   //@ exception UNKNOWN_SERVICE
	   if (0 == retour)
	     {
	       sin.sin_port = iservice.s_port;
	     }
	   else
	     {
	       exception.SetErrorCode (ERR__Exception::UNKNOWN_SERVICE);
	       THROW (exception);            
	     }
#else // LINUX
        static pthread_mutex_t getservbyname_mutex = PTHREAD_MUTEX_INITIALIZER;

 	   pthread_mutex_lock( &getservbyname_mutex );
 	   iservice = getservbyname( service, "tcp" );

 	  	//@ Si le service est inconnu alors
 	  	//@ exception UNKNOWN_SERVICE
 	  	if (iservice)
 	  	   {
 	  	     sin.sin_port = iservice->s_port;
 	         pthread_mutex_unlock( &getservbyname_mutex );
 	  	   }
 	  	 else
 	  	   {
 	         pthread_mutex_unlock( &getservbyname_mutex );
 	  	     exception.SetErrorCode (ERR__Exception::UNKNOWN_SERVICE);
 	  	     THROW (exception);
 	  	   }
#endif
// FT-2765
/* ..FA 184 */
	}
      //@ FIN SI

      //@ Recherche de l'adresse IP de host 
      sin.sin_addr.s_addr = inet_addr(host);
      if (sin.sin_addr.s_addr == INADDR_NONE) 
	{
	  //@ Si host n'est pas une adresse IP mais un nom, on le 
	  //@ recherche dans (/etc/host, dns )
/* FA 184 */
// FT-2765
#ifdef AIX
	  retour = gethostbyname_r(host,&ihost,&host_data);
	  //@ Si le host est inconnu alors exception UNKNOWN_HOST   
	  if (0 == retour) 
	    {
	      memcpy(&sin.sin_addr, ihost.h_addr, ihost.h_length);
	    } 
	  else 
	    {
	      exception.SetErrorCode (ERR__Exception::UNKNOWN_HOST);
	      THROW (exception);            
	    }
#else // LINUX
      static pthread_mutex_t gethostbyname_mutex = PTHREAD_MUTEX_INITIALIZER;
      pthread_mutex_lock( &gethostbyname_mutex );

	  ihost = gethostbyname(host);

	  //@ Si le host est inconnu alors exception UNKNOWN_HOST
	  if (ihost)
	    {
	      memcpy(&sin.sin_addr, ihost->h_addr, ihost->h_length);
          pthread_mutex_unlock( &gethostbyname_mutex );
	    }
	  else
	    {
          pthread_mutex_unlock( &gethostbyname_mutex );
 	      exception.SetErrorCode (ERR__Exception::UNKNOWN_HOST);
	      THROW (exception);
	    }
#endif
// FT-2765
/* ..FA 184 */
	}

      //@ Se connecter avec l'appel connect
      retour = connect(m_SocketDescripteur,
		       (struct sockaddr *) & sin,
		       sizeof(sin));

      //@ SELON retour
      switch (retour)
	{
	//@ Si retour = -1: erreur systeme
	case -1 :
	  //@ Emission exception SOCKET_ERROR
	  exception.SetErrorCode (ERR__Exception::SOCKET_ERROR);
	  exception.SetErrno (errno);
	  THROW (exception);
	  break;
	  
	//@ Defaut: aucun traitement	
	default : 
	  break;
	}
      //@ FIN SELON
    }
  //@ SINON
  else
    {
      //@ On envoie une exception SOCKET_UNDEFINED
      exception.SetErrorCode (ERR__Exception::SOCKET_UNDEFINED);
      THROW (exception);      
    }
  //@ FIN SI
}
