// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/MSGS/src/MSGS__GestionnaireActivite.cc,v 1.3 2011/11/25 10:17:44 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : MSG
//@ CLASSE : MSGS__GestionnaireActivite
//@
//@ Commentaire :
//@    Ce fichier gere la classe MSGS__GestionnaireActivite
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 21/03/2000 : A.MONTMORY
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL

// Standard

// Projet
#include "ERR__Exception.h"
#include "MSG__Abonnement.h"
#include "MSGS__GestionnaireActivite.h"


//
// Definition des constantes 
//


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : MSGS__GestionnaireActivite
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
MSGS__GestionnaireActivite::MSGS__GestionnaireActivite()
{
  //@ Positionne le flag de fin
  m_FlagFin = false;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~MSGS__GestionnaireActivite
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
MSGS__GestionnaireActivite::~MSGS__GestionnaireActivite()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : CreerConnexionServeur
//@
//@ Description : Permet de creer une connection SERVEUR sur laquelle
//@               il attend les connexions clientes
//@
//@ Parametres :
//@               (ENTREE) host : Numero de port de la connexion
//@               (ENTREE) service : Type de la connexion
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSGS__GestionnaireActivite::CreerConnexionServeur(char *host, char *service)
{
  //@ Ouvrir la connexion du type serveur (qui se met en ecoute)
  m_ConnectionServeur.Ouvrir(host, service, 
                             MSG__Connexion::CNX_SERVEUR);
  
  //@ Ajouter la connexion dans le gestionnaire
  m_Connexions.AjouterConnexion(m_ConnectionServeur);			     
}


//@ ------------------------------------------------------------------*/
//@ Methode : Ecouter
//@
//@ Description : Permet d'ecouter les descripteurs geres dans la
//@               connexion de type SERVEUR et de dispatcher les 
//@               messages
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSGS__GestionnaireActivite::Ecouter()
{
  //@ Liste des descripteurs actifs
  MSG__Connexion::t_ListeDescripteur descripteurActif;

  MSG__Socket newSocket;
  MSG__Connexion connexionCliente;
  MSG__Descripteur newDescripteur;
  MSG__Message newMessage;


  //@ Appel bloquant scruter sur la connexion serveur
  m_ConnectionServeur.Scruter (descripteurActif);
  
  //@ POUR tous les descripteurs actifs
  MSG__Connexion::t_ListeDescripteur::iterator pInfo;
  for ( pInfo = descripteurActif.begin(); 
	pInfo !=  descripteurActif.end();
	pInfo++) 
    {
      //@ SELON le type de descripteur actif
      MSG__Descripteur newDescripteur;
      switch ((*pInfo).GetType())
	{
	//@ Si type = SOC_SERVEUR  
	case MSG__Descripteur::SOC_SERVEUR :
	  
	  //@ alors on accepte la connexion cliente 
	  m_ConnectionServeur.GetSocket().Accepter(newSocket);
	  
	  //@ on construit un objet connexion CLIENTE   
	  connexionCliente.SetType(MSG__Connexion::CNX_CLIENTE);
	  connexionCliente.SetSocket(newSocket); 
	  //@ on ajoute cette connexion dans le gestionnaire
	  m_Connexions.AjouterConnexion(connexionCliente);
	  
	  //@ et on ajoute le descripteur du nouveau socket dans
	  //@ le masque de la connexion serveur
	  newDescripteur.SetDescripteur(newSocket.GetSocketDescripteur());
	  newDescripteur.SetType (MSG__Descripteur::SOC_CLIENT);
	  m_ConnectionServeur.SetMasque (newDescripteur);
	  break;
	  
	//@ Si type = SOC_CLIENT alors il faut traiter le message  
	case MSG__Descripteur::SOC_CLIENT :
	  
	  //@ il faut retrouver la connexion active a partir
	  //@ du descripteur
	  m_Connexions.GetCnxParDescripteur((*pInfo).GetDescripteur(),
					    connexionCliente);

	  //@ lire le message sur la connexion
	  //@ on capte les exceptions pour eliminer les connexions
	  //@ qui sont terminees (le client a fait exit)

	  //@ En captant les exceptions
	  try 
	    {
	      //@ Lire le message
              connexionCliente.LireMessage (newMessage);
	      
	      //@ Traiter le message
              TraiterMessage (connexionCliente,newMessage);
	      
             }
	  
	  //@ Traiter les exceptions
	  catch (ERR__Exception &exception) 
	    {
	      //@ Supprimer la connexion du gestionnaire
	      SupprimerConnexion(connexionCliente);		  
	    }
	  break;
	  
	//@ Defaut:
	default :
	  //@ Par defaut, on envoie une exception UNKNOWN_CNX_TYPE
	  ERR__Exception exception;
	  exception.SetErrorCode (ERR__Exception::UNKNOWN_CNX_TYPE);
	  THROW (exception);
	  break;   
	}
      //@ FIN SELON
    }
  //@ FIN POUR
}


//@ ------------------------------------------------------------------*/
//@ Methode : SupprimerConnexion
//@
//@ Description : Permet de supprimer une connexion inactive des
//@               differents gestionnaires
//@
//@ Parametres :
//@               (ENTREE) connexionCliente : Connexion a supprimer
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSGS__GestionnaireActivite::SupprimerConnexion(MSG__Connexion& connexionCliente)
{
  //@ Recuperer la liste des adresses associees a cette cnx
  t_ListeChaineTriee listeAdresse = m_Connexions.GetAdresses (connexionCliente);
   	 
  //@ POUR toutes les adresses de la liste
  t_ListeChaineTriee::iterator pInfo;
  for ( pInfo = listeAdresse.begin(); 
	pInfo !=  listeAdresse.end();
	pInfo++) 
    {
      MSGS__Adresse adresse;
      
      //@ Retrouver adresse dans le gestionnaire
      if (m_Adresses.ExisteAdresse(*pInfo,adresse)) 
	{
	  //@ Enlever la connexion associee a l'adresse dans le
	  //@ gestionnaire
	  adresse.EnleverCnxDiffusion(connexionCliente);

	  //@ Mettre a jour l'adresse dans le gestionnaire
	  m_Adresses.AjouterAdresse (adresse.GetName(),adresse);
	}	  
    }
  //@ FIN POUR
  
  //@ Supprimer le descripteur du masque de la connexion serveur
  MSG__Descripteur descripteur;
  descripteur.SetDescripteur(connexionCliente.GetSocket().GetSocketDescripteur());
  descripteur.SetType (MSG__Descripteur::SOC_CLIENT);
  m_ConnectionServeur.SubMasque (descripteur);			     
  
  //@ Supprimer la connexion du gestionnaire des connexions
  m_Connexions.EnleverConnexion(connexionCliente);
  
  //@ Fermer la connexion
  connexionCliente.Fermer();
}


//@ ------------------------------------------------------------------*/
//@ Methode : TraiterMessage
//@
//@ Description : Permet de traiter les differents types de messages
//@               recus
//@
//@ Parametres :
//@               (ENTREE) connexionCliente : Connexion sur laquelle
//@                                           a ete recu le message
//@               (ENTREE) message : Message recu
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSGS__GestionnaireActivite::TraiterMessage(const MSG__Connexion& connexionCliente,
						MSG__Message& message)
{
  MSG__Abonnement abonnement;
  
  //@ SELON le type du message
  switch (message.GetType())
    {
    //@ Si c'est un message abonnement
    case MSG__Message::ABONNEMENT :
      
      //@ Alors on construit un objet abonnement
      abonnement.Deserialiser(message);
      
      //@ SI l'action demandee est s'ABONNER
      if (abonnement.GetAction() == MSG__Abonnement::ABONNER) 
	{
	  //@ On s'abonne sur la liste contenue dans l'abonnement
	  Abonner (connexionCliente,abonnement.GetListeAdresse());
	}
      //@ SINON
      else
	{
	  //@ On se desabonne de la liste contenue dans l'abonnement
          Desabonner (connexionCliente,abonnement.GetListeAdresse());
	}
      //@ FIN SI
      break;
     
    //@ Si c'est un message fin d'execution
    case MSG__Message::FIN :
      //@ SI l'adresse est SERVEUR alors
      if (message.GetAdresse() == "SERVEUR") 
	{
	  //@ On positionne le flag de fin d'execution 
	  Terminer(true);
	} 
      //@ SINON
      else 
	{
	  //@ On route le message pour les clients
	  RouterMessage(message);    
	}
      //@ FIN SI
      break;

    //@ Defaut:
    default :
      //@ Par defaut, on route les messages
      RouterMessage(message);
    }
  //@ FIN SELON
}


//@ ------------------------------------------------------------------*/
//@ Methode : Abonner
//@
//@ Description : Permet de gerer l'abonnement
//@
//@ Parametres :
//@               (ENTREE) connexionCliente : Connexion sur laquelle
//@                                           a ete recu le message
//@               (ENTREE) listeAdresse : Liste des adresses
//@                                       auxquelles il faut s'abonner
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSGS__GestionnaireActivite::Abonner(const MSG__Connexion& connexionCliente,
					 t_ListeChaineTriee listeAdresse)
{
  //@ POUR toutes les adresses de la liste
  t_ListeChaineTriee::iterator pInfo;

  for ( pInfo = listeAdresse.begin(); 
	pInfo !=  listeAdresse.end();
	pInfo++) 
    {
      MSGS__Adresse adresse;
      adresse.SetName(*pInfo); 

      //@ Retrouver adresse dans le gestionnaire
      if (m_Adresses.ExisteAdresse(*pInfo)) 
	{
	  adresse = m_Adresses.GetAdresse(*pInfo);
	}	
  
      adresse.AjouterCnxDiffusion(connexionCliente);

      //@ Ajouter l'adresse dans le gestionnaire
      m_Adresses.AjouterAdresse (adresse.GetName(),adresse);
    }
  //@ FIN POUR
 
  //@ Ajouter la liste des adresses pour la connexion
  m_Connexions.SetAdresses (connexionCliente,listeAdresse);
}


//@ ------------------------------------------------------------------*/
//@ Methode : Desabonner
//@
//@ Description : Permet de gerer le desabonnement
//@
//@ Parametres :
//@               (ENTREE) connexionCliente : Connexion sur laquelle
//@                                           a ete recu le message
//@               (ENTREE) listeAdresse : Liste des adresses
//@                                       auxquelles il faut se
//@                                       desabonner
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSGS__GestionnaireActivite::Desabonner(const MSG__Connexion& connexionCliente,
					    t_ListeChaineTriee listeAdresse)
{
  // Adresse logique
  MSGS__Adresse adresse;


  //@ POUR toutes les adresses de la liste
  t_ListeChaineTriee::iterator pInfo;
  for ( pInfo = listeAdresse.begin(); 
	pInfo !=  listeAdresse.end();
	pInfo++) 
    {
      //@ Retrouver adresse dans le gestionnaire
      MSGS__Adresse adresse;
      adresse = m_Adresses.GetAdresse(*pInfo);
       
      //@ Enlever la connexion dans l'adresse
      adresse.EnleverCnxDiffusion(connexionCliente);
      
      //@ Si l'adresse ne contient plus de connexion alors l'enlever
      //@ du gestionnaire
      if (adresse.EstVide()) 
	{
	  m_Adresses.EnleverAdresse(adresse.GetName());
	}    
    }
  //@ FIN POUR
  
  //@ Enlever la liste des adresses pour la connexion
  m_Connexions.SubAdresses (connexionCliente,listeAdresse);
}


//@ ------------------------------------------------------------------*/
//@ Methode : RouterMessage
//@
//@ Description : Permet de gerer le routage des messages
//@
//@ Parametres :
//@               (ENTREE) message : Message a router
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSGS__GestionnaireActivite::RouterMessage(MSG__Message& message)
{
  // Adresse logique de diffusion
  MSGS__Adresse::t_Adresse adresseName = message.GetAdresse();
  MSGS__Adresse adresse;
   

  //@ Rechercher l'adresse dans le gestionnaire
  //@ SI l'adresse existe
  if (m_Adresses.ExisteAdresse(adresseName,adresse)) 
    {
      //@ Retourner la liste des connexions attachees a cette adresse
      MSGS__Adresse::t_ListeCnx listeCnx = adresse.GetListeCnx();
      
      //@ POUR toute les connexions
      MSGS__Adresse::t_ListeCnx::iterator  pConnexion;
      for ( pConnexion = listeCnx.begin(); 
	    pConnexion !=  listeCnx.end();
	    pConnexion++) 
	{
	  //@ Publier le message sur la connexion courante
	  MSG__Connexion connect = *pConnexion;

	  //@ On peut avoir une exception car on peut ecrire sur un
	  //@ socket alors que le client est parti
	  try 
	    {
	      connect.PublierMessage(message); 		  
	    }

	  //@ Traiter les exceptions
	  catch (ERR__Exception &exception) 
	    {
	      //@ Supprimer la connexion du gestionnaire
	      SupprimerConnexion(connect);		  
	    }
	}
      //@ FIN POUR
    }
  //@ FIN SI
}
