// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/MSGS/src/MSGS__GestionnaireAdresse.cc,v 1.3 2011/11/25 10:17:44 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : MSG
//@ CLASSE : MSGS__GestionnaireAdresse
//@
//@ Commentaire :
//@    Ce fichier gere la classe MSGS__GestionnaireAdresse
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ VERSION : 21/03/2000 : A.MONTMORY
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL

// Standard

// Projet
#include "ERR__Exception.h"
#include "MSGS__GestionnaireAdresse.h"


//
// Definition des constantes 
//


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : MSGS__GestionnaireAdresse
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
MSGS__GestionnaireAdresse::MSGS__GestionnaireAdresse()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~MSGS__GestionnaireAdresse
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
MSGS__GestionnaireAdresse::~MSGS__GestionnaireAdresse()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ExisteAdresse
//@
//@ Description : Permet de tester l'existence d'une Adresse a partir
//@               de son nom
//@
//@ Parametres :
//@               (ENTREE) name : Adresse
//@
//@ Valeur de retour :
//@               Booleen indiquant l'existence de l'adresse
//@
//@ ------------------------------------------------------------------*/
bool MSGS__GestionnaireAdresse::ExisteAdresse(const MSGS__Adresse::t_Adresse name)
{
  //@ Trouver l'adresse comportant le descripteur
  t_ListeAdresse::iterator pInfo; 
  pInfo = m_ListeAdresse.find(name);
  
  //@ SI non trouvee
  if (pInfo == m_ListeAdresse.end()) 
    {
      //@ Retourner FAUX
      return (false);
    } 
  //@ SINON
  else
    {
      //@ Retourner VRAI
      return (true);
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : ExisteAdresse
//@
//@ Description : Permet de tester l'existence d'une adresse a partir
//@               de son nom et de renvoyer sa valeur
//@
//@ Parametres :
//@               (ENTREE) name : Nom de l'adresse
//@               (SORTIE) adresse : Adresse trouvee
//@
//@ Valeur de retour :
//@               Booleen indiquant l'existence de l'adresse
//@
//@ ------------------------------------------------------------------*/
bool MSGS__GestionnaireAdresse::ExisteAdresse(const MSGS__Adresse::t_Adresse name,
					      MSGS__Adresse& adresse)
{
  //@ Trouver l'adresse comportant le descripteur 
  t_ListeAdresse::iterator pInfo; 
  pInfo = m_ListeAdresse.find(name);
   
  //@ SI non trouvee
  if (pInfo == m_ListeAdresse.end())
    {
      //@ Retourner FAUX
      return (false);
   }
  //@ SINON
  else
    {
      //@ Mettre a jour l'adresse
      adresse = (*pInfo).second;

      //@ Retourner VRAI
      return (true);
   }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetAdresse
//@
//@ Description : Permet de retrouver une adresse a partir de son
//@               nom
//@
//@ Parametres :
//@               (ENTREE) name : Nom de l'adresse
//@
//@ Valeur de retour :
//@               adresse : Objet adresse trouve
//@
//@ ------------------------------------------------------------------*/
MSGS__Adresse& MSGS__GestionnaireAdresse::GetAdresse(const MSGS__Adresse::t_Adresse name)
{
   // Declaration objet exception
   ERR__Exception exception;
 

   //@ Trouver l'adresse comportant le descripteur 
   t_ListeAdresse::iterator pInfo; 
   pInfo = m_ListeAdresse.find(name);
   
  //@ SI non trouvee
   if (pInfo == m_ListeAdresse.end())
     {
       //@ Envoyer une exception ADR_NOT_FOUND
       exception.SetErrorCode (ERR__Exception::ADR_NOT_FOUND);
       THROW (exception);   
     }
   //@ SINON
   else
     {
       //@ Retourner l'adresse
       return ((*pInfo).second);
   }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : AjouterAdresse
//@
//@ Description : Permet d'ajouter une adresse dans le gestionnaire
//@
//@ Parametres :
//@               (ENTREE) name : Nom de l'adresse (clef d'acces)
//@               (ENTREE) adresse : Objet adresse a ajouter
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSGS__GestionnaireAdresse::AjouterAdresse(MSGS__Adresse::t_Adresse name, 
                                               const MSGS__Adresse &adresse)
{
  //@ Ajouter l'adresse dans le gestionnaire
  m_ListeAdresse[name] = adresse;
}


//@ ------------------------------------------------------------------*/
//@ Methode : EnleverAdresse
//@
//@ Description : Permet d'enlever une adresse du gestionnaire
//@
//@ Parametres :
//@               (ENTREE) name : Nom de l'adresse (clef d'acces)
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSGS__GestionnaireAdresse::EnleverAdresse(const MSGS__Adresse::t_Adresse name)
{
  //@ Enlever l'adresse du gestionnaire
  m_ListeAdresse.erase(name);
}
