// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/MSGS/src/MSGS__GestionnaireConnexion.cc,v 1.3 2011/11/25 10:17:44 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : MSG
//@ CLASSE : MSGS__GestionnaireConnexion
//@
//@ Commentaire :
//@    Ce fichier gere la classe MSGS__GestionnaireConnexion
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ VERSION : 21/03/2000 : A.MONTMORY
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL

// Standard

// Projet
#include "ERR__Exception.h"
#include "MSGS__GestionnaireConnexion.h"


//
// Definition des constantes 
//


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : MSGS__GestionnaireConnexion
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
MSGS__GestionnaireConnexion::MSGS__GestionnaireConnexion()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~MSGS__GestionnaireConnexion
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
MSGS__GestionnaireConnexion::~MSGS__GestionnaireConnexion()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetCnxParDescripteur
//@
//@ Description : Permet de retrouver une connexion a partir de son
//@               descripteur de socket
//@
//@ Parametres :
//@               (ENTREE) descripteur : Numero de descripteur
//@               (SORTIE) connexion
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSGS__GestionnaireConnexion::GetCnxParDescripteur(const MSG__Socket::t_SocketDescripteur descripteur,
						       MSG__Connexion& connexion)
{
   // Declaration objet exception
   ERR__Exception exception;


   //@ Trouver la connexion comportant le descripteur
   t_ListeConnexion::iterator pInfo; 
   pInfo = m_ListeConnexion.find(descripteur);
   
   //@ SI non trouvee
   if (pInfo == m_ListeConnexion.end()) 
     {
       //@ Envoyer une exception CNX_NOT_FOUND
       exception.SetErrorCode (ERR__Exception::CNX_NOT_FOUND);
       THROW (exception);   
     } 
   //@ SINON   
   else 
     {
       MSGS__PaireCnxAdresse paire; 
       paire = (*pInfo).second;

       //@ Mettre a jour le parametre connexion
       connexion = paire.GetConnexion();
     }
   //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetAdresses
//@
//@ Description : Permet de retrouver la liste d'adresse associee a
//@               une connexion
//@
//@ Parametres :
//@               (ENTREE) connexion : Connexion geree dans le 
//@                                    gestionnaire
//@
//@ Valeur de retour :
//@               Liste d'adresse
//@
//@ ------------------------------------------------------------------*/
t_ListeChaineTriee MSGS__GestionnaireConnexion::GetAdresses(const MSG__Connexion& connexion )
{
   MSGS__PaireCnxAdresse paire;
   
   //@ Ajouter la connexion dans le gestionnaire
   paire = m_ListeConnexion[connexion.GetSocket().GetSocketDescripteur()];

   return paire.GetListe(); 
}


//@ ------------------------------------------------------------------*/
//@ Methode : SetAdresses
//@
//@ Description : Permet d'enrichir la liste d'adresse associee a
//@               une connexion
//@
//@ Parametres :
//@               (ENTREE) connexion : Connexion geree dans le 
//@                                    gestionnaire
//@               (ENTREE) listeAdresse : Liste des adresses a
//@                                       positionner
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSGS__GestionnaireConnexion::SetAdresses(const MSG__Connexion& connexion, 
					      t_ListeChaineTriee listeAdresse )
{
  MSGS__PaireCnxAdresse paire;
  t_ListeChaineTriee listeExistante;
   
  //@ Rechercher la connexion dans le gestionnaire
  paire = m_ListeConnexion[connexion.GetSocket().GetSocketDescripteur()];
  listeExistante = paire.GetListe();
   
  //@ POUR toutes les adresses de la liste
  t_ListeChaineTriee::iterator pInfo;
  for ( pInfo = listeAdresse.begin(); 
	pInfo !=  listeAdresse.end();
	pInfo++) 
    {
      //@ SI elle n'existe pas
      if (listeExistante.find(*pInfo) == listeExistante.end()) 
	{
	  //@ Rajouter l'adresse
	  listeExistante.insert(*pInfo);
	}
      //@ FIN SI
    }
  //@ FIN POUR
  
  //@ Reaffecter la liste 
  paire.SetListe(listeExistante);
  
  //@ Reaffecter la connexion
  m_ListeConnexion[connexion.GetSocket().GetSocketDescripteur()] = paire;
}


//@ ------------------------------------------------------------------*/
//@ Methode : SubAdresses
//@
//@ Description : Permet de supprimer des adresses de la liste
//@               d'adresse associee a une connexion
//@
//@ Parametres :
//@               (ENTREE) connexion : Connexion geree dans le 
//@                                    gestionnaire
//@               (ENTREE) listeAdresse : Liste des adresses a
//@                                       supprimer
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSGS__GestionnaireConnexion::SubAdresses(const MSG__Connexion& connexion, 
					      t_ListeChaineTriee listeAdresse )
{
  MSGS__PaireCnxAdresse paire;
  t_ListeChaineTriee listeExistante;
  
  
  //@ Rechercher la connexion dans le gestionnaire
  paire = m_ListeConnexion[connexion.GetSocket().GetSocketDescripteur()];
  listeExistante = paire.GetListe();
  
  //@ POUR toutes les adresses de la liste
  t_ListeChaineTriee::iterator pInfo;
  for ( pInfo = listeAdresse.begin(); 
	pInfo !=  listeAdresse.end();
	pInfo++)
    {
      //@ SI l'adresse existe
      if (listeExistante.find(*pInfo) != listeExistante.end()) 
	{
	  //@ Supprimer l'adresse
	  listeExistante.erase(*pInfo);
	}
      //@ FIN SI
    }
  //@ FIN POUR
   
  //@ Reaffecter la liste 
  paire.SetListe(listeExistante);
  
  //@ Reaffecter la connexion
  m_ListeConnexion[connexion.GetSocket().GetSocketDescripteur()] = paire;
}


//@ ------------------------------------------------------------------*/
//@ Methode : AjouterConnexion
//@
//@ Description : Permet d'ajouter une connexion dans le gestionnaire
//@
//@ Parametres :
//@               (ENTREE) connexion : Connexion a ajouter   
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSGS__GestionnaireConnexion::AjouterConnexion(const MSG__Connexion &connexion)
{
  MSGS__PaireCnxAdresse paire;
  
  paire.SetConnexion(connexion);

  //@ Ajouter la connexion dans le gestionnaire
  m_ListeConnexion[connexion.GetSocket().GetSocketDescripteur()] = paire;
}


//@ ------------------------------------------------------------------*/
//@ Methode : EnleverConnexion
//@
//@ Description : Permet d'enlever une connexion du gestionnaire
//@
//@ Parametres :
//@               (ENTREE) connexion : Connexion a enlever   
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSGS__GestionnaireConnexion::EnleverConnexion(const MSG__Connexion &connexion)
{
  //@ Enlever la connexion dans le gestionnaire
  m_ListeConnexion.erase( connexion.GetSocket().GetSocketDescripteur() );
}
