// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/MSGS/src/MSGS__ServeurMessage.cc,v 1.3 2011/11/25 10:17:44 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : MSG
//@ CLASSE : MSGS__ServeurMessage
//@
//@ Commentaire :
//@    Ce fichier gere la classe MSGS__ServeurMessage
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 21/03/2000 : A.MONTMORY
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
#include <stdlib.h>
//

// STL

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN__Version.h"
#include "CMN__OPSProcess.h"
#include "CMN__PGEServices.h"
#include "CFG__Configuration.h"
#include "ERR__Exception.h"
#include "MSGS__ServeurMessage.h"


//
// Definition des constantes 
//


//=====================================================================
//                        VARIABLES STATIQUES
//=====================================================================

// Serveur de message
static MSGS__ServeurMessage S_Serveur;

// Acces aux services PGE
static CMN__PGEServices     S_PGEServices(PROCESS_MSGS);


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : MSGS__ServeurMessage
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
MSGS__ServeurMessage::MSGS__ServeurMessage()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~MSGS__ServeurMessage
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
MSGS__ServeurMessage::~MSGS__ServeurMessage()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Initialiser
//@
//@ Description : Permet d'initialiser les traitements du serveur de
//@               message
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSGS__ServeurMessage::Initialiser()
{
  //@ Lecture du parametrage (nom Host et numero de service)
  t_Chaine host = CFG__Configuration::GetAdresse("MSG_SERVER_HOST");
  t_Chaine service = CFG__Configuration::GetAdresse("MSG_SERVER_PORT");
  
  //@ Creer la connexion serveur en appelant le gestionnaire 
  //@ d'activite
  m_GestionnaireActivite.CreerConnexionServeur((char*)host.c_str(),
					       (char*)service.c_str());
}


//@ ------------------------------------------------------------------*/
//@ Methode : Lancer
//@
//@ Description : Permet de lancer la boucle de scrutation des
//@               evenements
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSGS__ServeurMessage::Lancer()
{
  //@ TANT QUE pas fin
  while (!m_GestionnaireActivite.Terminer()) 
    {
      //@ Scruter les evenements
      m_GestionnaireActivite.Ecouter();
    }
  //@ FIN TANT QUE
}


//@ ------------------------------------------------------------------*/
//@ Methode : Terminer
//@
//@ Description : Permet de terminer l'execution du serveur de message
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void MSGS__ServeurMessage::Terminer()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : main
//@
//@ Description : Programme principal du serveur de message
//@
//@ Parametres :
//@               (ENTREE) argc : Nombre de parametres
//@               (ENTREE) argv : Tableau des parametres
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
main (int argc, char *argv[])
{
  t_CMN__PGEStatus status = CMN__STATUT_OK;
  

  //@ Traitement des options d'execution
  CMN__Version_Trace(argc, argv);

  //@ Declaration du demarrage au MCS
  status = S_PGEServices.MLAInitialisation();  
   
  //@ SI le status de l'initialisation du MCS est OK
  if (status == CMN__STATUT_OK) 
    {
      //@ Capture des exceptions
      try 
	{
	  //@ Initialisation du serveur de message
	  S_Serveur.Initialiser();
	  
	  //@ Lancer la boucle infinie de scrutation des evenements
	  S_Serveur.Lancer();	 	    
	  
	  //@ On est sorti de la boucle: on termine
	  S_Serveur.Terminer();
	  
          //@ On signale au MCS la fin du processus (fait exit)
	  S_PGEServices.MLATerminate();	 
        } 

      
      //@ Exploitation des exceptions    
      catch (ERR__Exception &exception) 
	{
	  //@ Impression des exceptions
	  cout << "Exception (MSGS): " << endl;
	  cout << "\tFile : " << exception.GetFile() << endl;
	  cout << "\tLine : " << exception.GetLine() << endl;
	  cout << "\tError: " << exception.GetErrorCode() << endl;
	  cout << "\tErrno: " << exception.GetErrno() << endl;
	  for (int i=0; i<exception.GetNbParametre(); i++)
	    {
	      cout << "\tInfo : " << exception.GetParametre(i) << endl;
	    }
	    
	  //@ On essaye de terminer
          S_Serveur.Terminer ();
	   
	  //@ On signale au MCS la fin du processus (fait exit)
          S_PGEServices.MLATerminate();	 
	} 
    }
  //@ FIN SI status ok 


  //@ Erreur initialisation MCS exit(-1)
  exit(-1);
}
