// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/ACT/src/SD_ACT_Action.cc,v 1.3 2011/11/25 10:17:44 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_ACT
//@ CLASSE : SD_ACT_Action
//@ 
//@ Commentaire :
//@    Ce module gere la classe SD_ACT_Action
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 09/09/1999 : SAS
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FAID<SMM-M4-FA-89-CN>    21/01/00    GRT    Modification du message compte rendu    
//@ --------------------------------------------------------------------
//@ FAID<SMM-M4-FA-76-CN>    24/01/00    Rajout du producteur dans journaliserWarning
//@                                      et JournaliserAlarme
//@ --------------------------------------------------------------------
//@ FAID<SMM-M4-FA-91-CN>    26/01/00    Appel de JournaliserWarning, on n'utilise plus JournaliserAnomalie
//@ --------------------------------------------------------------------
//@ FAID<SMM-M4-FA-61-CN>    27/01/00    Ajout d'un statut d'action
//@ --------------------------------------------------------------------
//@ FAID<SMM-M4-FA-???-CN>   14/11/00    Rajout des alarmes quand le nombre de fichiers TAI-TUC est diffirent de 1
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-2321-CN  30/08/01    GRT  Rajout de la fonction ModifierPrivilegesDuProcessus
//@                                     Verouillage lors de la creation des commandes 
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-2334-CN  30/08/01    GRT  Modification du message debut d'action et fin d'action pour les fichiers renommes
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-2555-CN   12/01/02 GRT Rajout des methodes TracerDebutCollectif et TracerFinCollectif
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Adaptation a l'OPS : ajout des parametres de simulation
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------


//
// Fichiers include
//

// STL
#include <iostream>

// Standard

// Projet
#include "CMN__Commun.h"
#include "SD_TAC_Tache.h"
#include "SD_FRW_ApplicationServeurDonnees.h"
#include "SD_ACT_Action.h"


//
// Definition des constantes
//
const t_Chaine SD_ACT_Action::LIBELLE_DEBUT_ACTION = "Debut de l'action: ";
const t_Chaine SD_ACT_Action::LIBELLE_FIN_ACTION   = "Fin de l'action: ";
const int      SD_ACT_Action::NB_TYPE_TM           = 5;


//======================================================================
//                                      METHODES PUBLIQUES 
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD_ACT_Action
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_ACT_Action::SD_ACT_Action()
  : m_Tache(0), m_NomAction(""), m_PeutEtreEnchainee(CMN__FAUX), 
    m_StatutAction(CMN__STATUT_OK), m_Simulation(-1)
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : SD_ACT_Action
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               (ENTREE) tache : Tache pere de l'action
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_ACT_Action::SD_ACT_Action(SD_TAC_Tache* tache)
  : m_Tache(0), m_NomAction(""), m_PeutEtreEnchainee(CMN__FAUX), 
    m_StatutAction(CMN__STATUT_OK), m_Simulation(-1)
{
  //@ Memoriser la tache parente de l'action
  m_Tache             = tache;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_ACT_Action
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_ACT_Action::~SD_ACT_Action()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Tache
//@
//@ Description : Accesseur a la tache associee a l'action
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Tache associee a l'action
//@
//@ ------------------------------------------------------------------*/
SD_TAC_Tache* SD_ACT_Action::Tache()
{
  return m_Tache;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Executer
//@
//@ Description : Traitement principal de l'action
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_ACT_Action::Executer()
{
  //@ Methode virtuelle a surcharger dans les classes derivees

  //@ Invoquer la verification des symptomes une fois l'action
  //@ executee
  VerifierSymptome();
}


//@ ------------------------------------------------------------------*/
//@ Methode : VerifierSymptome
//@
//@ Description : Verifie si les symptomes associes a l'action sont
//@               en erreur ou non
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_ACT_Action::VerifierSymptome()
{
  //@ Methode virtuelle a surcharger dans les classes derivees
  //@ Les symptomes controles sont specifiques a l'action
        
  m_PeutEtreEnchainee = CMN__VRAI;
}


//@ ------------------------------------------------------------------*/
//@ Methode : PeutEtreEnchainee
//@
//@ Description : Indique si l'action en cours peut etre enchainee 
//@               par la suivante
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Booleen : CMN__VRAI si l'action peut etre enchainee
//@                         CMN__FAUX sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Booleen SD_ACT_Action::PeutEtreEnchainee()
{
  //@ L'action est terminee si les instructions qui la
  //@ composent se sont completement deroulees
  return m_PeutEtreEnchainee;
}


//@ ------------------------------------------------------------------*/
//@ Methode : TracerDebut
//@
//@ Description : Tracer le debut de l'action
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_ACT_Action::TracerDebut()
{
  t_Chaine libelle(LIBELLE_DEBUT_ACTION);

  libelle += m_NomAction;
  JournaliserEvenement(libelle, SD__Message::MSG_SD_ACT_BEGIN);
}

        
//@ ------------------------------------------------------------------*/
//@ Methode : TracerFin
//@
//@ Description : Tracer la fin de l'action
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_ACT_Action::TracerFin()
{
  t_Chaine libelle(LIBELLE_FIN_ACTION);

  libelle += m_NomAction;
  JournaliserEvenement(libelle, SD__Message::MSG_SD_ACT_END);
}


//@ ------------------------------------------------------------------*/
//@ Methode : JournaliserEvenement
//@
//@ Description : Journalise un evenement
//@
//@ Parametres :
//@               (ENTREE) libelle : Libelle du message
//@               (ENTREE) typeEvt : Type d'evenement
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_ACT_Action::JournaliserEvenement(t_Chaine libelle,
                                         SD__Message::
                                         e_TypeMessageJDB typeEvt)
{
  CMN_DAT_Evenement evenement;
        
  //@ Acceder au journal de bord et journaliser l'evenement
  evenement.Identifiant(Tache()->Application()->
                        AllouerChronoEvenement());
  evenement.TypeEvenement(typeEvt);
  evenement.Fonction(Tache()->NomTache());
  evenement.Libelle(libelle.c_str());
  evenement.Prm1(Tache()->NomTache());
  evenement.Prm2(m_NomAction);
        
  Tache()->Application()->Evenements()->JournaliserEvenement(evenement);
}


//@ ------------------------------------------------------------------*/
//@ Methode : EnvoyerSensingTime
//@
//@ Description : Permet d'envoyer une telemesure SensingTime
//@
//@ Parametres :
//@               (ENTREE) sensingTime : SensingTime a envoyer
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_ACT_Action::EnvoyerSensingTime(t_TableauDate& sensingTime)
{
  const MP__HKTMStatus::e_ListeStatus typeCurrentSensingTime[] = 
    {MP__HKTMStatus::L1a_CurrentSensingTime,
     MP__HKTMStatus::L1b_CurrentSensingTime,
     MP__HKTMStatus::L1c_CurrentSensingTime,
     MP__HKTMStatus::ENG_CurrentSensingTime,
     MP__HKTMStatus::VER_CurrentSensingTime};


  //@ SI le nombre d'elements du tableau sensingTime n'est pas 
  //@ superieur au nombre maximum de type de telemesures
  if (sensingTime.size() <= NB_TYPE_TM)
    {
      //@ POUR tous les elements du tableau sensingTime
      for (int i=0; i<sensingTime.size(); i++)
        {
          //@ Appel a la fonction 
          //@ SD_FRW_ApplicationServeurDonnees::EnvoyerTM()
          Tache()->Application()->EnvoyerTM(typeCurrentSensingTime[i], 
                                            sensingTime[i].DateUTC());
        }
      //@ FIN POUR tous les elements du tableau sensingTime
    }
  //@ FIN SI
}


//======================================================================
//                                      METHODES PRIVEES
//======================================================================
