//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/ALG/src/SD_ALG_ISRFEM.c,v 1.5 2013/06/18 12:26:33 lcaspar Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_ALG
//@ MODULE : SD_ALG_ISRFEM
//@
//@ Comment :
//@    Code file for the  SD_ALG_ISRFEM class which includes the
//@    algorithm used to compute ISRFEM processing chain
//@
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 11/02/2003 : [reference :] Mathieu JUNG
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-061-CN : 10/10/2003 : B. MASSART
//@ l'ecart entre les decalages spectraux de THALES-IS et ceux de NOVELTIS 
//@ depasse le seuil de 0.01 cm-1. L'ecart entre les indices de qualite 
//@ des decalages spectraux de THALES-IS et ceux de NOVELTIS depasse le seuil 
//@ de 0.01. L'erreur provient du pseudo-code livre par le CNES ou le calcul 
//@ de l'indice de l'harmonique choisie pour le calcul du dephasage entre 
//@ le spectre mesure et le spectre synthetique fait intervenir 
//@ (NsFFT-1)/NsSSR au lieu de NsFFT/NsSSR (NsFFT, le nombre d'echantillons 
//@ des FFT, est toujours pair). Correction dans SD_ALG_23_SSD :
//@ NabsHar = (Nhar + 1) * ((NsFFT - 1) / NsSSR) + (NsFFT/2 + 1); devient
//@ NabsHar = (Nhar + 1) * (NsFFT / NsSSR) + (NsFFT/2 + 1);
//@ fiche de correction correspondante : IA-2100-FC-061-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-094-CN : 16/10/2003 : B. MASSART
//@ coquille dans le test de validite du resultat de l'appel a la fonction 
//@ SD_BAS_Extremum2D dans SD_ALG_ISRFEM.c: SD_ALG_24_IAX
//@ fiche de correction correspondante : IA-2100-FC-094-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-081-CN : 21/10/2003 : B. MASSART
//@ coquille dans le test de validite du resultat de l'appel a la fonction 
//@ SD_BAS_Extremum2D dans SD_ALG_ISRFEM.c: SD_ALG_24_IAX
//@ fiche de correction correspondante : IA-2100-FC-081-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-082-CN : 22/10/2003 : B. MASSART
//@ coquille dans le test de validite du resultat de l'appel a la fonction 
//@ SD_BAS_Extremum2D dans SD_ALG_ISRFEM.c: SD_ALG_24_IAX
//@ fiche de correction correspondante : IA-2100-FC-082-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-084-CN : 13/11/2003 : JL. PASCAL
//@ optimisation du code ( les initialisation de FFT sont faite une fois par ligne );
//@ dans SOS et SSD
//@ optimisation de transform ipsf
//@ fiche de correction correspondante : IA-2100-FC-084-THA
//@ --------------------------------------------------------------------
//@ DM-ID : PM4 : 31/03/2003 : B. MASSART
//@ -ajout SD_ALG_FAXAXERES_DEFAUT
//@ -indice SN de reference = 15 pour date UTC de la ligne dans 21_SSS
//@ -normalisation de Dist par SD_BAS_PN dans 24_IAX
//@ -correction AxeQual dans 24_IAX
//@ -la conversion en degres de ENG_GIAXAXEY, ENG_GIAXAXEZ, ENG_GIAXAXEQUAL, 
//@  ENG_GFAXAXEY, ENG_GFAXAXEZ, ENG_GFAXAXERES se fait en fin de traitement
//@ -changement test pour FlagAxeNonQual dans 25_FAX
//@ -normalisation de DistAxe dans 25_FAX
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-195-CN : 19/12/2006 : C. MERIGUET
//@ Corrections "coquilles" dans 22_SOS: multiplication d'un entier par 
//@ 0.5 + boucle for (1 a Nslast)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-2100-CR-329-CNE : 06/03/2007 : C. MERIGUET
//@ Calcul du residu sur le granule + duree d'oubli (ILS+FLS)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-2100-CR-348-CNE : 01/06/2007 : S. BRUNEL
//@ 23_SSD : Calcul de WnShiftQual (sature a 1)
//@ 25_FAX : Calcul de l'indice de qualite de l'axe interfero affine
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-220-CN : 25/06/2007 : C. MERIGUET / S. BRUNEL
//@ 23_SSD : Calcul de WnShiftQual (sature a 1), restitution de ligne de
//@ code d'initialisation de WnShiftQual
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2520(DM-2100-OPS) : 19/12/2008 : S. BRUNEL
//@ Rendre les variables WnShiftMin et WnShiftMax dependantes 
//@ du numero de pixel (Day 2,point 1) 
//@ Attention : Modification provisoire pour prendre en compte dans 21_SSS 
//@ le changement de taille de WnShiftMin et WnShiftMax dans le fichier 
//@ de configuration BRD.
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2524(DM-2100-OPS) : 11/05/2009 : S. BRUNEL
//@ Ajouter un indice et un flag dans l'algorithme 22_SSD (Day 2,point 5)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2520(DM-2100-OPS) : 28/05/2009 : S. BRUNEL
//@ Rendre les variables WnShiftMin et WnShiftMax dependantes 
//@ du numero de pixel (Day 2,point 1) 
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2629(DM-2100-OPS) : 09/06/2009 : S. BRUNEL
//@ Suppression temporaire de la condition ( nit < NItera ) 
//@ dans la levee de GSsdSpectralShiftNonQual
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2630(DM-2100-OPS) : 10/06/2009 : S. BRUNEL
//@ Creation d'une trace dans SD.log si SSD ne converge pas
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2634(FA-2100-OPS) : 11/06/2009 : S. BRUNEL
//@ Declaration des informations de phase de type errone dans 23 SSD
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2652(DM-2100-OPS) : 08/07/2009 : S. BRUNEL
//@ Suppression des verrous temporaires assurant la non regression
//@ entre V4-2-5 et V4-2
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-3095(DM-2100-OPS) : 13/03/2013 : P. JULIAN
//@ Amelioration de la position de l axe filtre SD_ALG_25_FAX
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-3117(FA-2100-OPS) : 17/06/2013 : P. JULIAN
//@ Anomalie sur la recuperation du CCD
//@ --------------------------------------------------------------------
//@FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include "SD_ALG_ISRFEM.h"

#include <math.h>
#include "SD_BAS_OperateurBase.h"
#include "SD_BAS_LibESSL.h"

#ifdef INVESTIGATION
#include "SD_BAS_Traces.h"
#endif


//declaration des fonctions importees

/*====================================================================*/
// Local Constant Declaration
/*====================================================================*/
static char *SD_BAS_MSG_ERR_APODISATION="Error during apodisation";
static char *SD_BAS_MSG_ERR_PREPARATION_FFT="Error during FFT permutation";
static char *SD_BAS_MSG_ERR_NFIRST_NLAST="Nfirst or Nlast value error";
static char *SD_BAS_MSG_ERR_SPLINE="Error during spline";
static char *SD_BAS_MSG_ERR_MOINDRES_CARRES="Error during MoindreCarre1D";
static char *SD_BAS_MSG_IPSF="Error during IPSF";

// FT-2765
static const double PRECISION_AJUSTEMENT_MC=0.0001;
// ...FT-2765

/* DM 11/03/2004 */
const double SD_ALG_FAXAXERES_DEFAUT = -2.0;

/*====================================================================*/
// Local types Declaration
/*====================================================================*/

//======================================================================
// Internal fonctions Declaration
//======================================================================

//======================================================================
// Global variables Definition
//======================================================================
const int   SD_ALG_INIT_FFT         = 1;
const int   SD_ALG_CALCUL_FFT       = 0;
const int   SD_ALG_INIT_SPLINE      = 0;
const int   SD_ALG_CALCUL_SPLINE    = 4;
#define SD_ALG_MAXDEGFAX    20

//======================================================================
//                  FONCTIONS EXTERNES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_22_SOS
//@
//@ Description : algorithme de surechantillonnage d'un spectre
//@ utilise dans la chaine ISRFEM et dans la chaine produit
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  ctp : variables temporaires
//@         (ENTREE)  ctx_appel : ISRFEM ou PRODUIT
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (ENTREE)  DataGenGranule : Donnees generales au granule
//@         (Entree)  GSpectInit : spectre decode reel
//@         (Sortie)  InfoGen : info generale issues de l'algo
//@         (Sortie)  GSpectReech : spectre reechantillone
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_22_SOS(
  t_SD_BAS_DataThread *       ctx,
  t_SD_BAS_DataThread22SOS *  ctp,
  int                         ctx_appel,
  t_SD_BAS_ConfAutre *        ConfAutre,
  t_SD_BAS_DataGen  *         DataGenGranule,
  double *                    GSpectInit,
  t_SD_BAS_22_SOS_Sortie *    InfoGen, /* ( ceci peut  etre detaille) */
  double *                    GSpectReech)

{
  /* Variables locales */
  /*-------------------*/
  static char   tcProcName_F[] = "SD_ALG_22_SOS";
  t_CMN__Statut cr_op = CMN__STATUT_OK;

  //@ (quel est l'interet de prevoir une FFT par morceau: cette possibilite
  //@ est abandonnee)
  //@ selon le contexte d'appel (ISRFEM ou produit) les nombres d'ondes
  //@ sont precalcules dans t_SD_BAS_DataGen

  int indFS;              // Indice de la fenetre spectrale dans laquelle on travaille
  int nFFT;               // Dimension de la transformee de Fourier
  int nSpline;            // Dimension de l'interpolation par Spline
  int nFFT_2;             // Dimension de la moitie de la FFT pour ESSL
  int surechFFT;          // Facteur de surechantillonnage FFT
  int surechSpline;       // Facteur de surechantillonnage spline
  int sigmaSpectre;       // sigma de la fenetre d'apodisation du spectre
  int sigmaInterf;        // sigma de la fenetre d'apodisation de la FFT
  int Nfirst;             // Numero du premier echantillon de la partie a traiter
  int Nlast;              // Numero du dernier echantillon de la partie a traiter
  int nbpts;              // Nombre de points a traiter
  double deltanu;         // Largeur d'un echantillon spectral
  double *CosPhi;         // Cosinus du phi dephasage
  double *SinPhi;         // Sinus du phi dephasage

  // Pour la FFT
  double scale;
  int naux1, naux2;       // Taille des tableaux auxiliaires de la FFT
  int nzg;                // Nombre de zeros inseres a gauche avant la FFT

  // Compteurs de boucle
  int ns;                 // Compteur de boucle sur le dephasage
  int i;

  // Pour le calcul du dephasage
  double x;               // Partie reelle du dephasage
  double y;               // Partie imaginaire du dephasage
  double sinT;            // Sinus de la phase
  double cosT;            // Cosinus de la phase

  // Gestion de erreurs
  t_CMN__Statut irc;      // Statuts de sortie des fonctions appelees
  char * msg_err;         // Message d'erreur
  int num_lig;

  // Pointeurs de tableaux divers
  double *pt1, *pt2;
  dcmplx *pt1C, *pt2C;
  // optimisation
 double temp;


  /* Initialiser les variables locales */
  /*-----------------------------------*/
  //@ selon le contexte d'appel les parametres issus des fichiers de conf varient
  if(ctx_appel == ISRFEM)
  {
    indFS           = ConfAutre->IDefSsdSpectralWindowOPS;
    nFFT            = ConfAutre->IWnpNsFFT[IND1(indFS)];
    nFFT_2          = nFFT/2 + 1;
    surechFFT       = ConfAutre->IDefSsdOSFftFactor[IND1(indFS)];
    surechSpline    = ConfAutre->IDefSsdOSSplineFactor[IND1(indFS)];
    sigmaSpectre    = ConfAutre->IDefSsdSigS[IND1(indFS)];
    sigmaInterf     = ConfAutre->IDefSsdSigI[IND1(indFS)];
    Nfirst          = ConfAutre->IDefSsdNsfirstSW[IND1(indFS)];
    Nlast           = ConfAutre->IDefSsdNslastSW[IND1(indFS)];
    nbpts           = Nlast - Nfirst + 1;
    deltanu         = ConfAutre->IDefSpectrDWn;
    CosPhi          = DataGenGranule->CosDephasageISRFEM;
    SinPhi          = DataGenGranule->SinDephasageISRFEM;
  }

  if(ctx_appel == PRODUIT)
  {
    nFFT            = ConfAutre->IDefS1bNsFFT;
    nFFT_2          = nFFT/2 + 1;
    surechFFT       = ConfAutre->IDefS1bOSFftFactor;
    surechSpline    = ConfAutre->IDefS1bOSSplineFactor;
    sigmaSpectre    = ConfAutre->IDefS1bSigS;
    sigmaInterf     = ConfAutre->IDefS1bSigI;
    Nfirst          = ConfAutre->IDefNsfirst[SD_BAS_B1];
    Nlast           = ConfAutre->IDefNslast[SD_BAS_B3];
    nbpts           = Nlast - Nfirst + 1;
    deltanu         = ConfAutre->IDefSpectrDWn;
    CosPhi          = DataGenGranule->CosDephasagePRODUIT;
    SinPhi          = DataGenGranule->SinDephasagePRODUIT;
  }

  /* Corps de l'operation */
  /*----------------------*/
#ifdef OPS_DEBUG
  printf ("Debut de %s\n", tcProcName_F);
#endif

#ifdef OPS_DEBUG
  if(ctx_appel == ISRFEM) printf("Chaine ISRFEM\n");
  if(ctx_appel == PRODUIT) printf("Chaine PRODUIT\n");
  printf("Fin des initialisatons\n");
  printf("Donnees d'initialisation\n");
  printf("   indFS\t=%d\n",indFS);
  printf("   nFFT\t=%d\n",nFFT);
  printf("   surechFFT\t=%d\n",surechFFT);
  printf("   surechSpline\t=%d\n",surechSpline);
  printf("   sigmaSpectre\t=%d\n",sigmaSpectre);
  printf("   sigmaInterf\t=%d\n",sigmaInterf);
  printf("   Nfirst\t=%d\n",Nfirst);
  printf("   Nlast\t=%d\n",Nlast);
  printf("   deltanu\t=%lf\n",deltanu);
#endif

  //@ Interception des erreurs sur les arguments
  if(Nfirst > Nlast)
  {
    cr_op   = SD_BAS_ISRFEM_ERR_NFIRST_NLAST;
    msg_err = (char *)SD_BAS_MSG_ERR_NFIRST_NLAST;
    num_lig = __LINE__;
    goto fin;
  }

  if(Nfirst < 0)
  {
    cr_op   = SD_BAS_ISRFEM_ERR_NFIRST_NLAST;
    msg_err = (char *)SD_BAS_MSG_ERR_NFIRST_NLAST;
    num_lig = __LINE__;
    goto fin;
  }

  //@ Preparation de la structure de sortie
  InfoGen->GSosSsdOSFActor = 1;

  //@ Test sur les interpolateurs
  //@ Si OSFftFactor > 1, alors on interpole par fft
  //=========================================================================
  if(surechFFT > 1)
  {
    //@ Amortissement (SigS) des extremites du spectre
    //-------------------------------------------------
    // Calcul de la fenetre d'amortissement

    irc = SD_BAS_FctAmortissement(
      nbpts, 
      sigmaSpectre, 
      deltanu, 
      ctp->fctAmortissement, 
      ctp->noyAmortissementS);

    if(irc != CMN__STATUT_OK)
    {
      cr_op   = SD_BAS_ISRFEM_ERR_APODISATION;
      msg_err = (char *)SD_BAS_MSG_ERR_APODISATION;
      num_lig = __LINE__;
      goto fin;
    }

    // Multiplication du spectre par la fenetre et copie pour ne pas l'ecraser
    for(i = 1 ; i <= nbpts ; i++)
    {
      ctp->GSpectInit_cp[IND1(i)] = 
        GSpectInit[IND1(i)]*ctp->fctAmortissement[IND1(i)];
    }


#ifdef OPS_DEBUG
  printf("Spectre amorti :\n");
  for(i=1; i<=nbpts; i++) printf("%le\t",ctp->GSpectInit_cp[IND1(i)]);
  printf("\n\n");
#endif

    // On noie le spectre de taille nbpts dans un tableau de taille NFFT
    // Rearrangement avant la FFT
/* tmp 
    irc = SD_BAS_PreparationFFTR(
      ctx, 
      ctp->GSpectInit_cp, 
      Nfirst, 
      Nlast, 
      ctp->GSpecArr, 
      nFFT);
*/
    irc = SD_BAS_PreparationFFTR(
      ctx, 
      ctp->GSpectInit_cp, 
      1, 
      nbpts, 
      ctp->GSpecArr, 
      nFFT);

    if(irc != CMN__STATUT_OK)
    {
      cr_op   = SD_BAS_ISRFEM_ERR_PREPARATION_FFT;
      msg_err = (char *)SD_BAS_MSG_ERR_PREPARATION_FFT;
      num_lig = __LINE__;
      goto fin;
    }

#ifdef OPS_DEBUG
  printf("Spectre amorti rearrange :\n");
  for(i=0; i<nFFT; i++) printf("%le\n",ctp->GSpecArr[i]);
  printf("\n\n");
#endif

    //@ Transformee de Fourier directe du spectre reel Spect(n) -> STF(m)
    //---------------------------------------------------------------------
  if(nFFT <= 2048)
        {
            naux1 = 22000;                          // cf doc lib ESSL p. 738 730 (max dcft drcft)
            naux2 = 20000;
        }
   else
        {
            naux1 = (int)(20000 + 2.28*nFFT);       // cf doc lib ESSL p. 738 730 (max dcft drcft)
            naux2 = (int)(20000 + 2.28*nFFT);
        }

#ifdef OPS_DEBUG
  printf(" Initialisation de la FFT :\n");
  printf("  nFFT=%d \t naux1=%d \t naux2=%d\n", nFFT, naux1, naux2);
#endif

    //@ Initialisation de la FFT
    scale = 1.0;        // Pour la FFT directe
// optimisation de calcul des FFT
  if (ctp->CalculFFTSOS == 0)
   {
    irc = SD_BAS_FFT1DirDR2C(
      ctx, 
      SD_ALG_INIT_FFT, 
      &nFFT, 
      1, 
      ctp->GSpecArr, 
      ctp->aux1, 
      ctp->aux2, 
      naux1, 
      naux2, 
      scale, 
      ctp->sTF);

    if(irc != CMN__STATUT_OK)
    {
      cr_op   = irc;
      num_lig = __LINE__;
      goto fin;
    }
   }
#ifdef OPS_DEBUG
  printf("FFT initialisee :\n");
#endif

    //@ Calcul de la FFT => LibEssl ne renvoie que la moitie des coeffs en raison de la symetrie Hermitienne
    irc = SD_BAS_FFT1DirDR2C(
      ctx, 
      SD_ALG_CALCUL_FFT, 
      &nFFT, 
      1, 
      ctp->GSpecArr, 
      ctp->aux1, 
      ctp->aux2, 
      naux1, 
      naux2, 
      scale, 
      ctp->sTF);

    if(irc != CMN__STATUT_OK)
    {
      cr_op   = irc;
      num_lig = __LINE__;
      goto fin;
    }

#ifdef OPS_DEBUG
  printf("FFT effectuee :\n");
#endif

    //@ Duplication du spectre par symetrie hermitienne pour se ramener a un spectre centre
    //======================================================================================
    // Frequences positives
// jlp
    for(i = 0 ; i < nFFT_2-1 ; i++)
    {
      RE(ctp->sTFdup[nFFT_2-1+i]) = RE(ctp->sTF[i]);
      IM(ctp->sTFdup[nFFT_2-1+i]) = IM(ctp->sTF[i]);
    }

    // fe/2
    RE(ctp->sTFdup[0]) = RE(ctp->sTF[nFFT_2-1]);
    IM(ctp->sTFdup[0]) = IM(ctp->sTF[nFFT_2-1]);

    // Frequences negatives : symetrie hermitienne = partie RE paire, partie IM impaire
    for(i = 1 ; i < nFFT_2-1 ; i++)
    {
      RE(ctp->sTFdup[i]) = RE(ctp->sTF[nFFT_2-1-i]);
      IM(ctp->sTFdup[i]) = -IM(ctp->sTF[nFFT_2-1-i]);
    }


#ifdef OPS_DEBUG
  printf("\nSymetrie hermitienne realisee :\n");
  for(i = 0 ; i < nFFT ; i++)
  {
    printf("(%le,%le)\n", RE(ctp->sTFdup[i]), IM(ctp->sTFdup[i]));
  }
  printf("\n");
#endif

    //@ Amortissement (SigI) des extremites de STF(n)
    //-------------------------------------------------
    // Calcul de la fenetre d'amortissement
    irc = SD_BAS_FctAmortissement(
      nFFT, 
      sigmaInterf, 
      deltanu, 
      ctp->fctAmortissement, 
      ctp->noyAmortissementI);

    if(irc != CMN__STATUT_OK)
    {
      cr_op   = irc;
      num_lig = __LINE__;
      goto fin;
    }


    for(i = 0 ; i < nFFT ; i++)
    {
      RE(ctp->sTFdup[i]) *= ctp->fctAmortissement[i];
      IM(ctp->sTFdup[i]) *= ctp->fctAmortissement[i];
    }


#ifdef OPS_DEBUG
  printf("\nAmortissement realise :\n");
  for(i = 0 ; i < nFFT ; i++)
  {
    printf("(%le,%le)\n", RE(ctp->sTFdup[i]), IM(ctp->sTFdup[i]));
  }
  printf("\n");
#endif

#ifdef OPS_DEBUG
  printf("Debut de boucle sur OSFftFactor\n");
#endif

    //@ Boucle sur le facteur de surechantillonnage ns varie de 1 a OSFftFactor
    //-------------------------------------------------------------------------
    for(ns = 1 ; ns <= surechFFT ; ns++)
    {
#ifdef OPS_DEBUG
  printf("ns = %d\n==============================\n",ns);
#endif

      pt1     = SinPhi + IND1(ns)*nFFT;
      pt2     = CosPhi + IND1(ns)*nFFT;
      pt1C    = ctp->sTFdup;
      pt2C    = ctp->sTFt;

      //@ Mutiplication du spectre par le dephasage ad hoc
      for(i = 0 ; i < nFFT ; i++)
      {
        sinT = *pt1;
        cosT = *pt2;
        pt1++;
        pt2++;

        x = RE(*pt1C);
        y = IM(*pt1C);
        pt1C++;

        RE(*pt2C) = x*cosT - y*sinT;
        IM(*pt2C) = y*cosT + x*sinT;
        pt2C++;

#ifdef OPS_DEBUG
  printf("(%d %le %le %lf %lf %le %le)\n",
    i, x, y, sinT, cosT, RE(ctp->sTFt[IND1(i)]), IM(ctp->sTFt[IND1(i)]));
#endif
     }

#ifdef OPS_DEBUG
  printf("\n\n");
#endif

      //@ Rearrangement du spectre
      irc = SD_BAS_PreparationFFTC(ctx, ctp->sTFt, 1, nFFT, ctp->sTFt2,nFFT);
      if(irc != CMN__STATUT_OK)
      {
        cr_op   = SD_BAS_ISRFEM_ERR_PREPARATION_FFT;
        msg_err = (char *)SD_BAS_MSG_ERR_PREPARATION_FFT;
        num_lig = __LINE__;
        goto fin;
      }

      //@ Transformee de Fourier inverse de l'interferogramme dephase complexe
      //@ vers un spectre reelSns(n) = Partie-Reelle[TF-1 (STF(m).exp(-i.dphi(m))]

      //initialisation de la FFT
// optimisation de calcul des FFT
       scale = 1.0/nFFT;        // Pour la FFT inverse
     if (ctp->CalculFFTSOS == 0) 
       {
       if(ns == 1) 
        {
        irc = SD_BAS_FFT1DirInvDC2C(
          ctx, 
          -1, 
          SD_ALG_INIT_FFT, 
          &nFFT, 
          1, 
          ctp->sTFt2, 
          ctp->aux12, 
          ctp->aux22, 
          naux1, 
          naux2,
          scale,ctp->GSpecArrC);

       if(irc != CMN__STATUT_OK)
         {
         cr_op   = irc;
         num_lig = __LINE__;
         goto fin;
         }
       }
      ctp->CalculFFTSOS =1;
      }

      // Calcul
      irc = SD_BAS_FFT1DirInvDC2C(
        ctx, 
        -1, 
        SD_ALG_CALCUL_FFT, 
        &nFFT, 
        1, 
        ctp->sTFt2, 
        ctp->aux12, 
        ctp->aux22, 
        naux1, 
        naux2,
        scale, 
        ctp->GSpecArrC);

      if(irc != CMN__STATUT_OK)
      {
        cr_op   = irc;
        num_lig = __LINE__;
        goto fin;
      }

      //@ Recuperation de la partie reelle
      for(i = 0 ; i < nFFT ; i++)
      {
        ctp->GSpecArr[i] = RE(ctp->GSpecArrC[i]);
      }

      //@ Rearrangement du spectre
      irc = SD_BAS_PreparationFFTR(
        ctx, 
        ctp->GSpecArr, 
        1, 
        nFFT, 
        ctp->GSpecArr2, 
        nFFT);

      if(irc != CMN__STATUT_OK)
      {
        cr_op   = SD_BAS_ISRFEM_ERR_PREPARATION_FFT;
        msg_err = (char *)SD_BAS_MSG_ERR_PREPARATION_FFT;
        num_lig = __LINE__;
        goto fin;
      }

      //@ Positionnement des echantillons spectraux
      //@ S(ns + (n-1) . OSFftFactor) = Sns(n)
      // *** On intercale dans le spectre surechantillonne en supprimant les 0 ajoutes ***
      i = 0;

// FA 195 : division par  remplace multiplication par 0.5
      nzg = (nFFT - nbpts + 1) /2 ;
// .. FA 195
      while (ns - 1 + i * surechFFT < (Nlast - Nfirst) * surechFFT + 1)
      {
        ctp->GSpectReechFFT[ns - 1 + i * surechFFT] = ctp->GSpecArr2[nzg + i];
        i++;
      }
    }
    //@ Fin de boucle sur les dephasages
    InfoGen->GSosSsdOSFActor *= surechFFT;
  }
  //@ Fin de boucle de surechantillonnage par FFT
  
  //@ Sinon S(n) = Spect(n)
  else
  {
// FA 195: boucle de 1 a (Nlast-Nfirst) au lieu de 1 a Nlast
    for(i = 1; i <= (Nlast-Nfirst); i++)
// .. FA 195
    {
      ctp->GSpectReechFFT[IND1(i)] = GSpectInit[IND1(i)];
    }
  }
  //@ Fin d'interpolation par fft
#ifdef OPS_DEBUG
  printf("Fin de boucle sur OSFftFactor\n");
#endif

  //@ Calcul de la base des nombres d'onde
  //@ Si surech FFT < 1 on prend 1
  temp = (double) deltanu / (double)InfoGen->GSosSsdOSFActor;
  for(i = 0 ; i < (Nlast - Nfirst) * InfoGen->GSosSsdOSFActor + 1 ; i++)
  {
    ctp->wn[i] = (Nfirst - 1) * deltanu + i * temp;
  }


  //@ Si OSsplineFactor > 1, alors on interpole par spline cubique
  //@ Interpolation par spline cubique(Wn(n), S(n)) -> (OSWn(m), OSSpect(m))
  //=========================================================================
  if(surechSpline > 1)
  {
    //@ Calcul des abscisses frequentielles desirees
    //@ (OSWn(m) en tenant compte du surechantillonnage par fft s'il a eu lieu
    //@ pour calculer les abscisses du spectre en entree Wn(n)
    //@ Pour tout n I [ 1, (Nslast-Nsfirst) * OSFftFactor * OSsplineFactor+ 1 ]
    //@ OSWn(n) = (Nsfirst-1)*DwnSpect + (n-1)*DwnSpect/(OSFftFactor* OSsplineFactor)
    nSpline = (Nlast - Nfirst) * InfoGen->GSosSsdOSFActor * surechSpline+1;

    for(i = 0 ; i < nSpline ; i++)
    {
      ctp->oswn[i] = (Nfirst - 1) * deltanu + 
        i * deltanu / (InfoGen->GSosSsdOSFActor * surechSpline);
    }

    //@ Interpolation par spline : Initialisation de la fonction
    irc = SD_BAS_InterpSpline1D(
      SD_ALG_INIT_SPLINE,
      (Nlast-Nfirst) * InfoGen->GSosSsdOSFActor + 1, 
      ctp->wn, 
      ctp->GSpectReechFFT, 
      ctp->y2, 
      nSpline, 
      ctp->oswn, 
      GSpectReech);

    if(irc != CMN__STATUT_OK)
    {
      cr_op   = SD_BAS_ISRFEM_ERR_SPLINE;
      msg_err = (char *)SD_BAS_MSG_ERR_SPLINE;
      num_lig = __LINE__;
      goto fin;
    }

#ifdef OPS_DEBUG
  printf("Spline initialise : facteur de surechantillonage = %i\n", 
    surechSpline);
#endif
    //@ Interpolation par spline : Calcul
    irc = SD_BAS_InterpSpline1D(
      SD_ALG_CALCUL_SPLINE, 
      (Nlast-Nfirst) * InfoGen->GSosSsdOSFActor + 1, 
      ctp->wn,
      ctp->GSpectReechFFT, 
      ctp->y2, 
      nSpline, 
      ctp->oswn, 
      GSpectReech);

    if(irc != CMN__STATUT_OK)
    {
      cr_op   = SD_BAS_ISRFEM_ERR_SPLINE;
      msg_err = (char *)SD_BAS_MSG_ERR_SPLINE;
      num_lig = __LINE__;
      goto fin;
    }

#ifdef OPS_DEBUG
  printf("Spline calcule %i\n",SD_ALG_CALCUL_SPLINE);
#endif
    InfoGen->GSosSsdOSFActor *= surechSpline;
  }

  // Remplissage du spectre de sortie dans le cas ou surechSpline <= 1
  else
  {
#ifdef OPS_DEBUG
  printf("Spectre reechantillonne\n=========================\n");
#endif
    for(i = 0 ; i < (Nlast - Nfirst) * InfoGen->GSosSsdOSFActor + 1; i++)
    {
      GSpectReech[i] = ctp->GSpectReechFFT[i];
#ifdef OPS_DEBUG
  printf("%le\t",GSpectReech[i]);
#endif
    }
  }
  //@ Fin d'interpolation spline cubique

  //@ Remplissage  de la structure de sortie de 22_SOS
  //@ OSNsfirst = InfoGen.GSosSsdOSFActor . (Nsfirst - 1) + 1
  //@ OSNslast = InfoGen.GSosSsdOSFActor . (Nslast - 1) + 1
  //@ Largeur spectrale d'un echantillon
  //@ DWnOSSpect = DWnSpect / InfoGen.GSosSsdOSFActor

  InfoGen->GSosSsdOSNsFirstSpect = InfoGen->GSosSsdOSFActor * (Nfirst - 1) + 1;
  InfoGen->GSosSsdOSNsLastSpect  = InfoGen->GSosSsdOSFActor * (Nlast - 1) + 1;
// jlp
  InfoGen->GSosSsdDWnOSSpect     = (double)deltanu/(double)InfoGen->GSosSsdOSFActor;

  // On trace ici toutes les variables en sortie
#ifdef INVESTIGATION
  // Nfirst
  SD_BAS_TRACEENTIER1D(
    tcProcName_F, 
    &ctx->ctxtrt, 
    "GSosSsdOSNsFirstSpect", 
    1, 
    1, 
    &InfoGen->GSosSsdOSNsFirstSpect, 
    ctx);

  // Nlast
  SD_BAS_TRACEENTIER1D(
    tcProcName_F, 
    &ctx->ctxtrt, 
    "GSosSsdOSNsLastSpect", 
    1, 
    1, 
    &InfoGen->GSosSsdOSNsLastSpect, 
    ctx);

  // Pas frequentiel
  SD_BAS_TRACEREEL1D(
    tcProcName_F, 
    &ctx->ctxtrt, 
    "GSosSsdDWnOSSpect", 
    1, 
    1, 
    &InfoGen->GSosSsdDWnOSSpect, 
    ctx);

  // Spectre
  // jlp
 SD_BAS_TRACEFICHIERREEL (
    tcProcName_F, 
    &(ctx->ctxtrt), 
    "GSpectReech",
    1, 
    (Nlast-Nfirst) * InfoGen->GSosSsdOSFActor + 1,
    GSpectReech, 
    ctx);
#endif

  fin:
  // Traitement des erreur s
  //================================================================
  if ( cr_op != CMN__STATUT_OK )
  {
    // mise a jour de la variable ctx->cr.errinterne
    SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
    SD_BAS_ERR_SET_ISRFEM,
    cr_op,
    tcProcName_F,
    num_lig,
    (char *) ctx->cr.errinterne.libelle);

    // on imprime le message d'erreur sur la sortie standard
#ifdef OPS_DEBUG
  SD_BAS_PRINT_ERR_INT(
    ctx->cr.errinterne.nomfonction,
    ctx->cr.errinterne.numligne,
    ctx->cr.errinterne.libelle);
#endif
  }

// sortie de la fonction
return(cr_op);
} /* fin de SD_ALG_22_SOS */


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_23_SSD
//@
//@ Description : algorithme de determination
//@ des decalage spectraux
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  ctp : variables temporaires
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (ENTREE)  ConfStable : fichier de conf stable
//@         (ENTREE)  InfoGen : info generales issues de l'algo SOS
//@         (ENTREE)  GSosSsdSpect : spectre reechantillone issue de l'algo SOS
//@         (Sortie)  InfoDet : info det du produit ENG
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_23_SSD(
        t_SD_BAS_DataThread * ctx,
        t_SD_BAS_DataThreadImageISRFEM *ctp,
        t_SD_BAS_ConfAutre *  ConfAutre,
        t_SD_BAS_ConfStable *  ConfStable,
        t_SD_BAS_22_SOS_Sortie * InfoGen,
        double * GSosSsdSpect,
        t_SD_BAS_Liste_Var InfoDet)

{
  static char tcProcName_F[] = "SD_ALG_23_SSD";
  t_CMN__Statut cr_op=CMN__STATUT_OK;


  // Recuperation des infos contenues dans les structure
  int         indFS;          // Indice de la fenetre spectrale dans laquelle on travaille
  int         SN;             // Numero de sous cycle
  int PN;                     // Pixel Number
  double      DWnSWref;
  double      WnSWref;
  int         NsSSR;
  int         NsSW;
  double      ModulNat;
  double      *WnSW;
  int         NsFFT;
  //int         Nharmonic;      // ?? A quoi sert-ce ?
  double      PhaseNat;
  int         Nhar;
  int         Sign;
  int         NItera;
  int         SigG;
  double      ModulCutoff;
  int         Nsfirst;
  int         Nslast;
  double      DWnSpec;

  // FT-2524
  double      SsdConverCutoff;    // Seuil de convergence du calcul de la phase ConverCutOff
  // ...FT-2524

  // Variables internes
  double Spectcalcoef;        // Coefficient de calibration spectrale
  int    NabsHar;             // Abscisse de l'harmonique
  double Modul;               // Module de l'harmonique
  double Phase;               // Phase de l'harmonique
  double WnShift;             // Decalage spectral
  double WnShiftQual;         // Indice de qualite
  unsigned int FlagNonQual;         // Test de qualite du module de l'harmonique

  double x;                   // Partie reelle d'un complexe
  double y;                   // Partie imaginaire d'un complexe

  // FT-2524
  // FT-2634
  double PhasePrevious;
  // ...FT-2634
  int stopIter;  
  // ...FT-2524

  // Pour la FFT
  double scale;
  int nFFT_2;             // Dimension de la moitie de la FFT pour ESSL
  int naux1, naux2;       // Taille des tableaux auxiliaires de la FFT

  // Gestion de erreurs
  char *msg_err;          // Message d'erreur
  int num_lig;            // Ligne d'erreur


  // Compteurs
  int i;
  int n;                      // Nombre d'echantillons
  int nit;                    // Nombre d'iterations

  // Statuts de sortie des fonctions appelees
  t_CMN__Statut irc;


#ifdef OPS_DEBUG
  printf("****** Algorithme 23_SSD ******\n*******************************\n");
  printf("\n");
#endif

  // Initialisations
  indFS           = ConfAutre->IDefSsdSpectralWindowOPS;
  PN              = ctx->ctxtrt.PN;
  SN              = ctx->ctxtrt.SN;

  Nhar            = ConfAutre->IDefSsdNhar[IND1(indFS)];
  DWnSWref        = ConfAutre->IWnpDWnSWref[IND1(indFS)];
  WnSWref         = ConfAutre->IWnpWnSWref[IND1(indFS)];
  NsSSR           = ConfAutre->IWnpNsSSR[IND1(indFS)];
  NsSW            = ConfAutre->IWnpNsSW[IND1(indFS)];
  WnSW            = &ConfStable->IWnpWnSW[IND2(1,indFS,SD_BAS_NS_SW)];
  NsFFT           = ConfAutre->IWnpNsFFT[IND1(indFS)];
  ModulNat        = ConfAutre->IWnpModulNat[IND2(Nhar+1,indFS,SD_BAS_SB)];
  PhaseNat        = ConfAutre->IWnpPhaseNat[IND2(Nhar+1,indFS,SD_BAS_SB)];
  Sign            = ConfAutre->IDefWnpSign[IND1(indFS)];
  NItera          = ConfAutre->IDefSsdNItera;
  SigG            = ConfAutre->IDefWnpSigG[IND1(indFS)];
  ModulCutoff     = ConfAutre->IDefSsdModulCutoff[IND2(Nhar+1,indFS,SD_BAS_SB)];
  // FT-2524
  SsdConverCutoff    = ConfStable->IDefSsdConverCutoff;
  // ...FT-2524
  Nsfirst         = InfoGen->GSosSsdOSNsFirstSpect;
  Nslast          = InfoGen->GSosSsdOSNsLastSpect;
  DWnSpec         = InfoGen->GSosSsdDWnOSSpect;

  nFFT_2          = NsFFT/2 + 1;
  // Fin des initialisations

#ifdef OPS_DEBUG
  printf("Fin des initialisations\n");
  printf("Nsfirst : %d\n",Nsfirst);
  printf("Nslast : %d\n",Nslast);
  printf("NsSW : %d\n",NsSW);
  printf("Nb Iterations : %d\n",NItera);
  printf("\n");
#endif

    //@ Calcul des positions spectrales nominales
    for(n=Nsfirst; n<= Nslast; n++)
    {
        //@ Wno(n) = (n-1).DWnSpect pour n de Nsfirst a Nslast
        ctp->Wno[n-Nsfirst] = (n-1)*DWnSpec;
    }

    #ifdef OPS_DEBUG
        printf("Positions spectrales nominales\n");
        for(n=Nsfirst; n<= Nslast; n++)
        {
            printf("(%d,%le)\t",n,ctp->Wno[n-Nsfirst]);
        }
        printf("\n");
    #endif

    //@ Initialisation du coefficient de calibration spectrale de la fenetre Spectcalcoef = 1
    Spectcalcoef = 1.0;

    // FT-2524
    //@ Initialisation de la phase precedente PhasePrevious = 0
    PhasePrevious = 0;      
    // ...FT-2524

    //@ Calcul de la fenetre d'amortissement
    irc = SD_BAS_FctAmortissement(NsSW,SigG,DWnSpec,ctp->fctAmortissement,ctp->noyAmortissementS);
    if(irc != CMN__STATUT_OK)
    {
        cr_op   = SD_BAS_ISRFEM_ERR_APODISATION;
        msg_err = (char *)SD_BAS_MSG_ERR_APODISATION;
        num_lig = __LINE__;

        goto fin;
    }

    //@ Boucle sur le nombre d'iteration nit varie de 1 a NItera
    // FT-2524
    stopIter = 0; 
    for(nit=1; (nit<=NItera) && (!stopIter); nit++)
    {
    // ...FT-2524
        //@ Calibration spectrale des positions des echantillons
        for(n=Nsfirst; n<= Nslast; n++)
        {
            //@ Wn(n) = Wno(n).SpectCalcoef pour tous les echantillons de Nsfirst a Nslast
            ctp->Wn[n-Nsfirst] = ctp->Wno[n-Nsfirst]*Spectcalcoef;
        }

        #ifdef OPS_DEBUG
            printf("\n\nITERATION no %i\n",nit);
            printf("===============================================\n",nit);
            printf("Positions spectrales calibrees\n");
            for(n=Nsfirst; n<= Nslast; n++)
            {
                printf("(%d,%le)\t",n,ctp->Wn[n-Nsfirst]);
            }
            printf("\n");
        #endif

        //@ Calcul des radiances sur la base des pseudo-nombres d'ondes par interpolation
        //@ spline cubique a partir de la base des nombres d'ondes calibres
        //@ (Spect(n) sur la base Wn(n)) donnent (RdSW(m) sur la base WnSW(m))

        //@ Interpolation par spline : Initialisation de la fonction
        #ifdef OPS_DEBUG
            printf("Interpolation des radiances par spline\n");
        #endif

        irc = SD_BAS_InterpSpline1D(SD_ALG_INIT_SPLINE,Nslast - Nsfirst + 1,ctp->Wn,
                                    GSosSsdSpect,ctp->y2ssd,
                                    NsSW,WnSW,ctp->RdSW);
        if(irc != CMN__STATUT_OK)
        {
            cr_op   = SD_BAS_ISRFEM_ERR_SPLINE;
            msg_err = (char *)SD_BAS_MSG_ERR_SPLINE;
            num_lig = __LINE__;

            goto fin;
        }

        //@ Interpolation par spline : Calcul
        irc = SD_BAS_InterpSpline1D(SD_ALG_CALCUL_SPLINE,Nslast - Nsfirst + 1,ctp->Wn,
                                    GSosSsdSpect,ctp->y2ssd,
                                    NsSW,WnSW,ctp->RdSW);
        if(irc != CMN__STATUT_OK)
        {
            cr_op   = SD_BAS_ISRFEM_ERR_SPLINE;
            msg_err = (char *)SD_BAS_MSG_ERR_SPLINE;
            num_lig = __LINE__;

            goto fin;
        }

        //Affichage des radiances et des radiances interpolees
        #ifdef OPS_DEBUG
            printf("Radiances avant interpolation\n");
            for(i=0; i<(Nslast -Nsfirst + 1); i++)
            {
                printf("(%le,%le)\t",ctp->Wn[i],GSosSsdSpect[i]);
            }
            printf("\n");
            printf("Radiances apres interpolation\n");
            for(i=0; i<NsSW; i++)
            {
                printf("(%le,%le)\t",WnSW[i],ctp->RdSW[i]);
            }
            printf("\n");
        #endif


        //@ Amortissement des extremites de la fenetre spectrale.
        //@ On applique la methode de "windowing" pour amortir les extremites
        //@ de la fenetre en utilisant une gaussienne de demi-largeur SigG.

        for(i=0; i<NsSW; i++)
        {
            ctp->RdSW[i] *= ctp->fctAmortissement[i];
        }

        #ifdef OPS_DEBUG
            printf("Radiances amorties\n");
            for(i=0; i<NsSW; i++)
            {
                printf("%le\t",ctp->RdSW[i]);
            }
            printf("\n");
        #endif

        //@ On noie les radiances un tableau de taille NsFFT
        // Rearrangement avant la FFT
        irc = SD_BAS_PreparationFFTR(ctx,ctp->RdSW,1,NsSW,ctp->RdSW2,NsFFT);
        if(irc != CMN__STATUT_OK)
        {
            cr_op   = SD_BAS_ISRFEM_ERR_PREPARATION_FFT;
            msg_err = (char *)SD_BAS_MSG_ERR_PREPARATION_FFT;
            num_lig = __LINE__;

            goto fin;
        }

        #ifdef OPS_DEBUG
            printf("Tableau des radiances rearrange avant FFT\n");
            for(i=0; i<NsFFT; i++)
            {
                printf("%le\t",ctp->RdSW2[i]);
            }
            printf("\n");
        #endif



        //@ Transformee de Fourier complexe des radiances de la fenetre spectrale FFTSW(n)

        //---------------------------------------------------------------------
  if(NsFFT <= 2048)
        {
            naux1 = 22000;                          // cf doc lib ESSL p. 738 730 (max dcft drcft)
            naux2 = 20000;
        }
   else
        {
            naux1 = (int)(20000 + 2.28*NsFFT);       // cf doc lib ESSL p. 738 730 (max dcft drcft)
            naux2 = (int)(20000 + 2.28*NsFFT);
        }

        //@ Initialisation de la FFT
        scale = 1.0;        // Pour la FFT directe

        if (ctp->CalculFFTSSD == 0)
        {
          irc = SD_BAS_FFT1DirDR2C(ctx,SD_ALG_INIT_FFT,&NsFFT,1,ctp->RdSW2,ctp->aux1,ctp->aux2,naux1,naux2,scale,ctp->sTF);
          if(irc != CMN__STATUT_OK)
          {
            cr_op   = irc;
            num_lig = __LINE__;
            goto fin;
          }
	  ctp->CalculFFTSSD = 1; 
	}


        //@ Calcul de la FFT => LibEssl ne renvoie que la moitie des coeffs en raison de la symetrie Hermitienne
        irc = SD_BAS_FFT1DirDR2C(ctx,SD_ALG_CALCUL_FFT,&NsFFT,1,ctp->RdSW2,ctp->aux1,ctp->aux2,naux1,naux2,scale,ctp->sTF);
        if(irc != CMN__STATUT_OK)
        {
            cr_op   = irc;
            num_lig = __LINE__;

            goto fin;
        }

        //@ Duplication du spectre par symetrie hermitienne pour se ramener a un spectre centre
        //======================================================================================
        // Frequences positives
// jlp
        for(i=0; i<nFFT_2-1; i++)
        {
            RE(ctp->sTFdup[nFFT_2-1+i]) = RE(ctp->sTF[i]);
            IM(ctp->sTFdup[nFFT_2-1+i]) = IM(ctp->sTF[i]);
        }

        // fe/2
        RE(ctp->sTFdup[0]) = RE(ctp->sTF[nFFT_2-1]);
        IM(ctp->sTFdup[0]) = IM(ctp->sTF[nFFT_2-1]);

        // Frequences negatives : symetrie hermitienne = partie RE paire, partie IM impaire
        for(i=1; i<nFFT_2-1; i++)
        {
            RE(ctp->sTFdup[i]) = RE(ctp->sTF[nFFT_2-1-i]);
            IM(ctp->sTFdup[i]) = -IM(ctp->sTF[nFFT_2-1-i]);
        }

        #ifdef OPS_DEBUG
            printf("Transformee de Fourier Complexe des radiances\n");
            for(i=0; i<NsFFT; i++)
            {
                printf("(%le;%le)\t",RE(ctp->sTFdup[i]),IM(ctp->sTFdup[i]));
            }
            printf("\n");
        #endif


        //@ Abscisse de l'harmonique
        NabsHar = (Nhar + 1) * (NsFFT / NsSSR) + (NsFFT/2 + 1);

        #ifdef OPS_DEBUG
            printf("Abscisse de l'harmonique\n");
            printf("%lf\n",NabsHar);
        #endif

        //@ Module de l'harmonique    Modul =  cf DA7
        x = RE(ctp->sTFdup[IND1(NabsHar)]);
        y = IM(ctp->sTFdup[IND1(NabsHar)]);
        Modul = sqrt(x*x + y*y);

        #ifdef OPS_DEBUG
            printf("Module de l'harmonique\n");
            printf("%le\n",Modul);
        #endif

        //@ Phase de l'harmonique
        Phase = atan2(y*Sign,x*Sign);

        #ifdef OPS_DEBUG
            printf("Phase de l'harmonique\n");
            printf("%le\n",Phase);
        #endif

        //@ Calcul du coefficient de calibration spectral
        Spectcalcoef *= 1 + ( (Phase - PhaseNat)*DWnSWref )/( 2*M_PI*WnSWref*(Nhar+1) );
        #ifdef OPS_DEBUG
            printf("coefficient de calibration spectral\n");
            printf("%le\n",Spectcalcoef);
        #endif
        // FT-2524
        if (abs(Phase - PhasePrevious) < SsdConverCutoff)
        {
          stopIter = 1;
        }
        // FT-2630
        else
        {
          PhasePrevious = Phase;
        }
        // ...FT-2630
        // ...FT-2524
    }
    //@ Fin de boucle iterative

    // FT-2630
    if (!stopIter)
    {
      // pas de convergence
      printf("23_SSD (No convergence) : LN=%d PN=%d SN=%d Phase=%1.15f, PhasePrevious=%1.15f, ConverCutoff=%1.15f\n",
                InfoDet.tab_var[ENG_DATA_PX].val.pui4[14*SD_BAS_PN*SD_BAS_SNOT + IND2(PN,SN,SD_BAS_PN)], // Ligne DPS
                PN,
                SN,
                Phase,
                PhasePrevious,
                SsdConverCutoff
            );
    }
    // ...FT-2630
    //@ Indice de qualite
    // FA 220
    // restitution ligne supprime du patch3-6-7
    WnShiftQual = Modul/ModulNat;
    // ...FA 220
    // CR 348
    //@ Si WnShiftQual > 1 alors WnShiftQual=1/ WnShiftQual;
    if(WnShiftQual > 1.0) WnShiftQual = 1.0;
    // ...CR 348

    #ifdef OPS_DEBUG
        printf("Indice de qualite\n");
        printf("%le\n",WnShiftQual);
    #endif

    //@ Test de qualite du module de l'harmonique desiree
    //@ Si Modul > Modulcutoff(NHar) alors FlagNonQual = non
    // CR 348
    // FT-2524
    // FT-2629
    // FT-2652
    if ((Modul > (ModulCutoff*ModulNat)) && (nit < NItera)) FlagNonQual=CMN__NON;
    // ...FT-2652
    // ...FT-2629
    // ...FT-2524
    // ...CR 348
    
    //@ Sinon FlagNonQual =oui
    else FlagNonQual=CMN__OUI;
    // FT-2524
    if (nit >= NItera)
    {
      // ConverFlag = CMN__NON;
      InfoDet.tab_var[ENG_GSSDCONVERFLAG].val.pui4[IND2(PN,SN,SD_BAS_PN)] = CMN__NON;
    }
    else
    {
      // ConverFlag = CMN__OUI;
      InfoDet.tab_var[ENG_GSSDCONVERFLAG].val.pui4[IND2(PN,SN,SD_BAS_PN)] = CMN__OUI;
    }
    // ...FT-2524

    #ifdef OPS_DEBUG
        printf("Test de qualite\n");
        printf("%d\n",FlagNonQual);
    #endif

    //@ Calcul du decalage spectral
    WnShift = WnSWref * (Spectcalcoef - 1);

    #ifdef OPS_DEBUG
        printf("decalage spectral\n");
        printf("%le\n",WnShift);
    #endif

    // Recopie des sorties dans la listvar InfoDet
    // Remarque ... les tab var en question sont des tableaux et je ne sais pas ou copier
    // les resultats !
    InfoDet.tab_var[ENG_GSSDWNSHIFT].val.pr8[IND2(PN,SN,SD_BAS_PN)]                     = WnShift;
    InfoDet.tab_var[ENG_GSSDWNSHIFTQUAL].val.pr8[IND2(PN,SN,SD_BAS_PN)]                 = WnShiftQual;
    InfoDet.tab_var[ENG_GSSDFLAGSPECTRALSHIFTNONQUAL].val.pui4[IND2(PN,SN,SD_BAS_PN)]   = FlagNonQual;

    // On trace ici toutes les variables en sortie
    #ifdef INVESTIGATION
       // Decalage spectral
        SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"WnShift",1,1,&WnShift,ctx);
        // Flag de qualite
        // FlagUnderOverFlow
        SD_BAS_TRACE(tcProcName_F, "FlagNonQual",
                     1, 1, 1,
                     TRACE_UINT4, TRACE_ASC,
                     &FlagNonQual, ctx);

        // Indice de qualite
        SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"WnShiftQual",1,1,&WnShiftQual,ctx);
    #endif

fin:

    // Traitement des erreurs
    //================================================================
    if ( cr_op != CMN__STATUT_OK )
    {
        // mise a jour de la variable ctx->cr.errinterne
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
        SD_BAS_ERR_SET_ISRFEM,
        cr_op,
        tcProcName_F,
        num_lig,
        msg_err);

        // on imprime le message d'erreur sur la sortie standard
        #ifdef OPS_DEBUG
            SD_BAS_PRINT_ERR_INT(ctx->cr.errinterne.nomfonction,
                                 ctx->cr.errinterne.numligne,
                                 ctx->cr.errinterne.libelle);
        #endif
    }

// sortie de la fonction


// sortie de la fonction
return(cr_op);

}

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_21_SSS
//@
//@ Description : algorithme de selection des decalages spectraux
//@ des decalage spectraux
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  ctp : variables temporaires
//@         (ENTREE)  t : date UTC de la ligne (SN=15) /* DM 11/03/2004 */
//@         (ENTREE)  t_FE : date UTC de fin de l'eclipse precedente
//@         (ENTREE)  GIrcImg : image calibree
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (ENTREE)  ConfStable : fichier de conf stable
//@         (ENTREE)  ListeInfoN0 : Liste des paquets de N0 PX
//@                   pour la ligne ( on choisira ceux de PN
//@                   et de CCD ( SN variant))
//@                   cette info est dans InfoDet1ABC
//@         (ENTREE)  NbPaquetsPX : nb paquets du produit N0
//@         (Entree/Sortie)  InfoDetEng : info det du produit ENG
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_21_SSS(
        t_SD_BAS_DataThread * ctx,
        t_SD_BAS_DataThreadImageISRFEM *ctp,
        t_SD_BAS_SCDSTime t,
        t_SD_BAS_SCDSTime t_FE,
        double * GIrcImg,
        t_SD_BAS_ConfAutre *  ConfAutre,
        t_SD_BAS_ConfStable *  ConfStable,
       	t_SD_BAS_PaquetN0** ListePaquetPX, 
       	int NbPaquetsPX, 
        t_SD_BAS_Liste_Var * InfoDetEng
            )
{
static char tcProcName_F[] = "SD_ALG_21_SSS";
t_CMN__Statut cr_op=CMN__STATUT_OK;
// Entree dans la fonction

    // Gestion de erreurs
    t_CMN__Statut irc;
    char *msg_err;          // Message d'erreur
    int num_lig;

    // Recuperation des infos contenues dans les structure
    int                 PN;                     // Pixel Number
    int                 SN;                     // Nombre de souscycles
    int                 CCD;                    // Direction du coin cuble

    double              Period;
    double              *WnShift;
    double              *WnShiftQual;
    unsigned int        *FlagNonQual;
    int                 ColDeb;
    int                 ColFin;
    int                 LinDeb;
    int                 LinFin;
    double              IISRot;
    double              *IISY;
    double              *IISZ;
    double              IISDY;
    double              IISDZ;
    int                 NbLin;
    int                 NbCol;
    int                 Osf;
    double              *Ypsf;
    double              *Zpsf;
    double              *WgtPsf;
    double              WnShiftMin;
    double              WnShiftMax;
    double              ImagStdCutoff;
    double              *OffsetSondIISModel;
    unsigned int        Scd;


    // Variables internes
    double              WnShiftWgt;
    double              WnShiftSum;
    double              Nsel;
    double              OffsetSondIIS1;
    double              OffsetSondIIS2;
    double              ModelCCD1[SD_BAS_MODEL];
    double              ModelCCD2[SD_BAS_MODEL];

    double DY;
    double DZ;
    double ImagMean;
    double Variance;
    double ImagStd;
    double Surf;
    int    SNmin;
    int    SNmax;
    int    modePdv;
    t_CMN__Booleen ScdTrouve;
    int    PNCourant;

    // Sorties
    unsigned int FlagDateNOK    = CMN__NON;
    double WnShiftMean          = 0.0;
    unsigned int FlagNonSel     = CMN__NON;
    double WnShiftMeanQual      = 0.0;


    // Compteurs
    int il,ic;                  // Indices ligne et colonne
    int ct;                     // Pour les indices internes en C
    int PaquetPX;               // Pour acceder a Scd(SN)


#ifdef OPS_DEBUG
    printf("*** Algorithme 21_SSS ***\n====================================\n");
    printf("Entree dans la fonction \n");
#endif


    //@ Initialisations
    PN                  = ctx->ctxtrt.PN;
    CCD                 = ctx->ctxtrt.CCD + 1;              // Pour pouvoir utiliser les macros


    NbLin               = ConfStable->IDefPsfSondNbLin;
    NbCol               = ConfStable->IDefPsfSondNbCol;
    Osf                 = ConfStable->IDefPsfSondOverSampFactor;
    WgtPsf              = ConfStable->IDefPsfSondWgt;
    Ypsf                = ConfStable->IDefPsfSondY;
    Zpsf                = ConfStable->IDefPsfSondZ;
    // FT-2520
    WnShiftMax          = ConfAutre->IDefSssWnShiftMax[IND1(PN)];
    WnShiftMin          = ConfAutre->IDefSssWnShiftMin[IND1(PN)];
    // ..FT-2520
    ImagStdCutoff       = ConfAutre->IDefSssStdImagCutoff;
    Period              = ConfAutre->IDefMETOPOrbitPeriod;
    ColDeb              = ConfAutre->IDefIISColDeb[IND1(PN)];
    ColFin              = ConfAutre->IDefIISColFin[IND1(PN)];
    LinDeb              = ConfAutre->IDefIISLinDeb[IND1(PN)];
    LinFin              = ConfAutre->IDefIISLinFin[IND1(PN)];
    IISRot              = ConfAutre->IDefIISRot;
    IISY                = ConfAutre->IDefIISY;
    IISZ                = ConfAutre->IDefIISZ;
    IISDY               = ConfAutre->IDefIISDY;
    IISDZ               = ConfAutre->IDefIISDZ;
    OffsetSondIISModel  = ConfAutre->IDefOffsetSondIISModel;

    WnShift     = InfoDetEng->tab_var[ENG_GSSDWNSHIFT].val.pr8;
    WnShiftQual = InfoDetEng->tab_var[ENG_GSSDWNSHIFTQUAL].val.pr8;
    FlagNonQual = InfoDetEng->tab_var[ENG_GSSDFLAGSPECTRALSHIFTNONQUAL].val.pui4;

#ifdef OPS_DEBUG
    printf("Fin des initialisations : \n");
    printf("   ColDeb : %d\n",ColDeb);
    printf("   ColFin: %d\n",ColFin);
    printf("   LinDeb : %d\n",LinDeb);
    printf("   LinFin: %d\n",LinFin);
#endif


    //@ WnShiftWgt=0 WnShiftSum=0 Nsel=0
    WnShiftWgt = 0.0;
    WnShiftSum = 0.0;
    Nsel = 0.0;

    //@ Estimation de l'offset sondeur-IIS par la le modele orbital
    // Recuperation des coefficients du modele
    for(il=1; il<=SD_BAS_MODEL; il++)
    {
      ModelCCD1[IND1(il)] = 
	       OffsetSondIISModel[IND3(il,1,PN,SD_BAS_MODEL,SD_BAS_CCD)];
      ModelCCD2[IND1(il)] = 
	       OffsetSondIISModel[IND3(il,2,PN,SD_BAS_MODEL,SD_BAS_CCD)];
    }

    OffsetSondIIS1 = SD_BAS_EvolutionModele(t,t_FE,Period,ModelCCD1);
    OffsetSondIIS2 = SD_BAS_EvolutionModele(t,t_FE,Period,ModelCCD2);


#ifdef OPS_DEBUG
    printf("Estimation de l'offset sondeur-IIS par le modele orbital\n");
    printf("Modele 1\n");
    for(il=0; il<SD_BAS_MODEL; il++)
    {
        printf("%lf\t",ModelCCD1[il]);
    }
    printf("\n");

    printf("Modele 2\n");
    for(il=0; il<SD_BAS_MODEL; il++)
    {
        printf("%lf\t",ModelCCD2[il]);
    }
    printf("\n");

    printf("   OffsetSondIIS1 : %lf\n",OffsetSondIIS1);
    printf("   OffsetSondIIS2 : %lf\n",OffsetSondIIS2);
#endif
    //@ Si ( abs(t-t_FE) < 8*Period ou OffsetSondIISModel(1,6)=0 )
    //@ Alors FlagDateNOK = NON
    //@ Sinon FlagDateNOK = OUI
    //@ Fin de Si

    if( (abs(t.day*86400000 - t_FE.day*86400000 + t.millis - t_FE.millis) < 8000*Period) ||
       	(SD_BAS_MacEGAUX_D(OffsetSondIISModel[IND3(6,1,PN,SD_BAS_MODEL,SD_BAS_CCD)],0.0)) )
    {
        FlagDateNOK = CMN__NON;
    }
    else
    {
        FlagDateNOK = CMN__OUI;
    }


#ifdef OPS_DEBUG
    printf("   FlagDateNOK : %i\n",FlagDateNOK);
#endif


    //@ Transformation des vecteurs IISY et IISZ en matrices IISY(ligne,colonne)
    //@ et IISZ(ligne,colonne)
    for(il=LinDeb, ct=0; il<=LinFin; il++)
    {
        for(ic=ColDeb; ic<=ColFin; ic++,ct++)
        {
            ctp->Y[ct] = IISY[IND1(ic)]*cos(IISRot) - IISZ[IND1(il)]*sin(IISRot);
            ctp->Z[ct] = IISY[IND1(ic)]*sin(IISRot) + IISZ[IND1(il)]*cos(IISRot);
        }
    }

    DY = IISDY*cos(IISRot) - IISDZ*sin(IISRot);
    DZ = IISDY*sin(IISRot) + IISDZ*cos(IISRot);

#ifdef OPS_DEBUG
    printf("   DY : %lf\n",DY);
    printf("   DZ : %lf\n",DZ);
#endif



    //@ Transformation de l?IPSF sondeur dans le raster Imageur IIS  restreint a
    //@ la partie utile de l?Ipsf  de (LinDeb,ColDeb) a (LinFin,ColFin)
    //@ avec prise en compte de l?offset sondeur imageur
    //@ [ ctp->IISWgtIpsf, ImagYIpsfunused, ImagZIpsfunused ] = TransformIpsf( NbLin, NbCol
    //@  , Ypsf, Zpsf, Wgtpsf, LinDeb, ColDeb,LinFin,ColFin, IISY, IISZ, IISDY, IISDZ, OffsetSondIIS, Osf)
    //@ la fonction transform IPSF peut etre precalculee ( par granule ou par ligne ?)

    irc = SD_ALG_TransformIpsf(
      ctx, 
      ctp, 
      NbLin, 
      NbCol, 
      Ypsf, 
      Zpsf, 
      WgtPsf, 
      LinDeb, 
      ColDeb,
      LinFin, 
      ColFin, 
      ctp->Y, 
      ctp->Z, 
      DY, 
      DZ, 
      OffsetSondIIS1, 
      OffsetSondIIS2,
      Osf);
      
    if(irc != CMN__STATUT_OK)
    {
        cr_op   = SD_BAS_ISRFEM_ERR_IPSF;
        msg_err = (char *)SD_BAS_MSG_IPSF;
        num_lig = __LINE__;

        goto fin;
    }


    //@ Surf = (LinFin ? LinDeb + 1) * (ColFin ? ColDeb + 1)
    Surf = (double)((LinFin - LinDeb + 1) * (ColFin - ColDeb + 1));

#ifdef OPS_DEBUG
    printf("   Surf : %lf\n", Surf);
    printf("   CCD : %d\n", CCD);
#endif

    //@ Boucle conditionnelle sur les sous-cycles Sn si CCD=Scd(Sn) alors
    /* Definir le mode de fonctionnement */
    modePdv = InfoDetEng->tab_var[ENG_MODE].val.pui4[0];
    
    /* Definir SNmin et SNmax en fonction du mode de fonctionnement */
    /* SI modePdv = external_calibration */
    if(modePdv == 1)
    {
      /* SNmin = 3 et SNmax = 29 */
      SNmin = 3;
      SNmax = 29;
    }
    /* SINON (mode normal ou auxiliary) */
    else
    {
      /* SNmin = 1 et SNmax = 30 */
      SNmin = 1;
      SNmax = 30;
    }
    /* FIN SI */
    
    ScdTrouve = CMN__FAUX;
    SN = 0;
    PNCourant = 0;
    for (PaquetPX = 1 ; (PaquetPX <= NbPaquetsPX) && (SN <= SNmax) ; PaquetPX++)
    {
      SN =
        ListePaquetPX[IND1(PaquetPX)]->InfoGen.tab_var[N0_SN].val.pui4[0];
      PNCourant = 
        ListePaquetPX[IND1(PaquetPX)]->InfoGen.tab_var[N0_PN].val.pui4[0];
      
      if ( (SN >= SNmin) && (PNCourant == PN) )
      {
	Scd = ListePaquetPX[IND1(PaquetPX)]->InfoDet.tab_var[PX_ICD].val.pui4[0]+1;
	ScdTrouve = CMN__VRAI;
      }
      
      if ( (ScdTrouve == CMN__VRAI) && (CCD == Scd) )
      {
#ifdef OPS_DEBUG
    printf("   FlagNonQual : %d\n", FlagNonQual[IND2(PN,SN,SD_BAS_PN)]);
    printf("   WnShiftMin : %lf\n", WnShiftMin);
    printf("   WnShift : %lf\n", WnShift[IND2(PN,SN,SD_BAS_PN)]);
    printf("   WnShiftMax : %lf\n", WnShiftMax);
#endif
	
	ScdTrouve = CMN__FAUX;
	
        //@   Si FlagNonQual(Sn)=Non et si WnShiftMin < Wnshift(Sn) < WnShiftMax alors
        if(    FlagNonQual[IND2(PN,SN,SD_BAS_PN)]==CMN__NON
            && WnShiftMin  < WnShift[IND2(PN,SN,SD_BAS_PN)]
            && WnShift[IND2(PN,SN,SD_BAS_PN)] < WnShiftMax
          )
        {
          //@ Calcul de la valeur moyenne de l'image intra-pixel sondeur ( cf DA7)
          ImagMean = 0.0;
          for(il=LinDeb, ct=0; il<=LinFin; il++)
          {
            for(ic=ColDeb; ic<=ColFin; ic++, ct++)
            {
	      ImagMean += GIrcImg[ct + IND1(SN)*SD_BAS_IMCO*SD_BAS_IMLI];
	    }
	  }
	  ImagMean /=Surf;

#ifdef OPS_DEBUG
    printf("   ImagMean : %lf\n",ImagMean);
#endif

          //@     Calcul de la variance intra-pixel sondeur (cf DA7)
          Variance = 0.0;
	  for(il=LinDeb, ct=0; il<=LinFin; il++)
	  {
	    for(ic=ColDeb; ic<=ColFin; ic++, ct++)
	    {
	      Variance += 
	        (GIrcImg[ct + IND1(SN)*SD_BAS_IMCO*SD_BAS_IMLI] - ImagMean) * 
                (GIrcImg[ct + IND1(SN)*SD_BAS_IMCO*SD_BAS_IMLI] - ImagMean) *
                ctp->IISWgtIpsf[ct];
            }
          }

          ImagStd = sqrt(Variance );

#ifdef OPS_DEBUG
    printf("   Variance : %lf\n",Variance);
    printf("   ImagStd : %lf\n",ImagStd);
#endif

          //@ Si ImagStd < ImagStdCutoff alors
          if(ImagStd < ImagStdCutoff)
          {
	    Nsel = Nsel + 1;
	    WnShiftSum = WnShiftSum + WnShift[IND2(PN,SN,SD_BAS_PN)] * WnShiftQual[IND2(PN,SN,SD_BAS_PN)];
	    WnShiftWgt = WnShiftWgt + WnShiftQual[IND2(PN,SN,SD_BAS_PN)];
	  }
          //@ Fin de si (test de variabilite sur l?image)
        }
        //@ Fin de si SN valide
      }
      //@ Fin si CCD = Scd
    }
    //@ Fin de boucle conditionnelle sur les sous-cycle de direction CCD

    //@ Si Nsel > 0 alors
    if(Nsel > 0)
    {
        //@ WnShiftMean=WnShiftSum/WnShiftWgt
        WnShiftMean = WnShiftSum/WnShiftWgt;
        //@ FlagNonSel=NON
        FlagNonSel = CMN__NON;
        //@ WnShiftMeanQual=WnShiftWgt / Nsel
        WnShiftMeanQual = WnShiftWgt / Nsel;
    }
    //@ Sinon
    else
    {
        //@ FlagNonSel=OUI
        FlagNonSel = CMN__OUI;
        //@ WnShiftMeanQual=WnShiftWgt
        WnShiftMeanQual = WnShiftWgt;
    }
    //@ Fin de si


    // Recopie des sorties dans la listvar InfoDet
    // Remarque ... les tab var en question sont des tableaux et je ne sais pas ou copier
    // les resultats !
    *(InfoDetEng->tab_var[ENG_GSSSFLAGDATENOK].val.pui4) = FlagDateNOK;
    InfoDetEng->tab_var[ENG_GSSSWNSHIFTMEAN].val.pr8[IND2(PN,CCD,SD_BAS_PN)] = WnShiftMean;
    InfoDetEng->tab_var[ENG_GSSSWNSHIFTMEANQUAL].val.pr8[IND2(PN,CCD,SD_BAS_PN)] = WnShiftMeanQual;
    InfoDetEng->tab_var[ENG_GSSSFLAGNONSELPX].val.pui4[IND2(PN,CCD,SD_BAS_PN)] = FlagNonSel;

#ifdef OPS_DEBUG
    printf("\nValeurs retournees\n---------------------\n");
    printf("   WnShiftMean : %lf\n",WnShiftMean);
    printf("   WnShiftMeanQual : %lf\n",WnShiftMeanQual);
    printf("   FlagNonSel : %d\n",FlagNonSel);
    printf("   FlagDateNOK : %d\n",FlagDateNOK);
    printf("Fin de la fonction 21_SSS\n");
#endif


// on trace ici si besoin toutes les variables en sortie
#ifdef INVESTIGATION
    // Decalage spectral moyen pour la ligne
    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"WnShiftMean",1,1,&WnShiftMean,ctx);
    // Flag de selection des determinations des decalages spectraux
    SD_BAS_TRACE(tcProcName_F, "FlagNonSel",
                 1, 1, 1,
                 TRACE_UINT4, TRACE_ASC,
                 &FlagNonSel, ctx);
    // Indice de qualite de la valeur moyenne
    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"WnShiftMeanQual",1,1,&WnShiftMeanQual,ctx);
    // Flag indicateur d'une incoherence des dates
    SD_BAS_TRACE(tcProcName_F, "FlagDateNOK",
                 1, 1, 1,
                 TRACE_UINT4, TRACE_ASC,
                 &FlagDateNOK, ctx);
#endif


fin:

    // Traitement des erreurs
    //================================================================
    if ( cr_op != CMN__STATUT_OK )
    {
        // mise a jour de la variable ctx->cr.errinterne
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
        SD_BAS_ERR_SET_ISRFEM,
        cr_op,
        tcProcName_F,
        num_lig,
        msg_err);

        // on imprime le message d'erreur sur la sortie standard
        #ifdef OPS_DEBUG
            SD_BAS_PRINT_ERR_INT(ctx->cr.errinterne.nomfonction,
                                 ctx->cr.errinterne.numligne,
                                 ctx->cr.errinterne.libelle);
        #endif
    }


// sortie de la fonction
return(cr_op);


}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_24_IAX
//@
//@ Description : algorithme de determination de l'axe interferometrique
//@ instantanne
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  ctp : variables temporaires
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (ENTREE)  BanqueSpectrale : pointeur sur le fichier BanqueSpectrale
//@         (ENTREE)  NbISPPX : nombre des paquets de N0 PX
//@         (ENTREE)  ListeInfoN0 : Liste des paquets de N0 PX
//@                 pour la ligne (on choisira ceux de CCD
//@         (Entree/Sortie)  InfoDetEng : info det du produit ENG
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_24_IAX(
        t_SD_BAS_DataThread * ctx,
        t_SD_BAS_DataThreadImageISRFEM *ctp,
        t_SD_BAS_ConfAutre *  ConfAutre,
        t_SD_BAS_BanqueSpectrale * BanqueSpectrale,
        //int NbISPPX,
        // t_SD_BAS_Liste_Var  * ListeInfoN0,
        t_SD_BAS_Liste_Var  * InfoDetEng
            )
{
static char tcProcName_F[] = "SD_ALG_24_IAX";
t_CMN__Statut cr_op=CMN__STATUT_OK;

// Entree dans la fonction


    // Gestion de erreurs
    t_CMN__Statut irc;
    char *msg_err;          // Message d'erreur
    int num_lig;

    // Recuperation des infos contenues dans les structure
    int                 indFS;              // Indice de la fenetre spectrale dans laquelle on travaille
    int                 CCD;                // Direction du coin cuble

    double          PdsPix[SD_BAS_PN];
    double          *WnShiftMean;
    unsigned int    *FlagNonSel;
    int             NbLin;
    int             NbCol;
    double          *GridY;
    double          *GridZ;
    double          *GridWnShift;


    // Sorties
    double AxeYInstant;
    double AxeZInstant;
    int FlagAxeNonQual;
    double AxeQual = 0.;
    double AxeRes  = 0.;


    // Variables internes
    double DistPix[SD_BAS_PN];
    int nlin0;              // Ligne du point de la grille de distance minimum
    int ncol0;              // Colonne du point de la grille de distance minimum
    int nlinm;              // Ligne du point de la grille juste avant
    int ncolm;              // Colonne du point de la grille juste avant
    int nlinp;              // Ligne du point de la grille juste apres
    int ncolp;              // Colonne du point de la grille juste apres
    double MinLin;
    double MinCol;
    double DistMin;
    double minDist;
    double DY;
    double DZ;
    double sum;
    double FitQual;

    // Pour l'estimation d'extremum 2D par moindres carres + paraboloide
    int nbeq = 9;           // Nombre d'equations
    int degre = 5;          // Degre du polynome d'ajustement
    double a[54];           // Matrice A 6x9 de la resolution de AX = B (9 equations,6inconnues)
    double b[9];            // Vecteur B 9 de la resolution de AX = B
    double tabTmpMC[9];     // Tableau temporaire pour moindres carres
    double x[6];            // Coeffs du paraboloide ajuste
    double eps;             // Moyenne quadratique des residus d'ajustement
    double res[4];          // Resultat


    // Compteurs
    int il, ic;
    int nl, nc;
    int ct;                 // Compteur de boucle pour les tableaux C
    int npix;

#ifdef OPS_DEBUG
    printf("*** Algorithme 24_IAX ***\n====================================\n");
    printf("Entree dans la fonction \n");
#endif

    //@ Initialisations
    indFS           = ConfAutre->IDefSsdSpectralWindowOPS;
    CCD             = ctx->ctxtrt.CCD + 1;          // Pour pouvoir utiliser les macros
    WnShiftMean     = InfoDetEng->tab_var[ENG_GSSSWNSHIFTMEAN].val.pr8;
    FlagNonSel      = InfoDetEng->tab_var[ENG_GSSSFLAGNONSELPX].val.pui4;
    NbLin           = BanqueSpectrale->IDefSdbGridNbLin;
    NbCol           = BanqueSpectrale->IDefSdbGridNbCol;
    GridY           = BanqueSpectrale->IDefSdbGridY;
    GridZ           = BanqueSpectrale->IDefSdbGridZ;
    GridWnShift     = BanqueSpectrale->ISdbGridWnShift;
    AxeQual         = 0.0;
    AxeRes          = 0.0;

#ifdef OPS_DEBUG
    printf("Fin des initialisations \n");
#endif

    //@ Pour chacun des pixels npix = 1 a 4
    for(npix=1; npix<=SD_BAS_PN; npix++)
    {
        PdsPix[IND1(npix)] = ConfAutre->IDefPdsPix[IND1(npix)];
        
        //@   Si FlagNonSel(npix) = oui alors pdspix(npix) = 0
        if(FlagNonSel[IND2(npix,CCD,SD_BAS_PN)] == CMN__OUI)
        {
            PdsPix[IND1(npix)] = 0.0;
        }
    }


    //@ Pour chaque point de la grille (nlin,ncol) on calcule le carre de
    //@  la distance entre les decalages spectraux mesures et calcules
    for(il=1, ct=0; il<=NbLin; il++)
    {
        for(ic=1; ic<=NbCol; ic++, ct++)
        {
            //@   Pour chaque pixel npix de 1 a Npix
            for(npix=1; npix<=SD_BAS_PN; npix++)
            {
                //@     DistPix(npix)=( WnShiftMean(npix)- GridWnShift(nlin,ncol,npix))**2
                DistPix[IND1(npix)] =  (WnShiftMean[IND2(npix,CCD,SD_BAS_PN)] - GridWnShift[IND5(npix,CCD,indFS,ic,il,SD_BAS_PN,SD_BAS_CDX,SD_BAS_WNP_NB_SW,SD_BAS_SDB_NB_COL)])
                                     * (WnShiftMean[IND2(npix,CCD,SD_BAS_PN)] - GridWnShift[IND5(npix,CCD,indFS,ic,il,SD_BAS_PN,SD_BAS_CDX,SD_BAS_WNP_NB_SW,SD_BAS_SDB_NB_COL)]);
            }
            //@   Fin de boucle sur les pixels

            //@   Dist(nlin,ncol) = Sigma pixels (PdsPix(npix).DistPix(npix))
            ctp->Dist[ct]=0.0;
            for(npix=1; npix<=SD_BAS_PN; npix++)
            {
                ctp->Dist[ct] += PdsPix[IND1(npix)] * DistPix[IND1(npix)];
            }
            /* PM4 */
            ctp->Dist[ct] = ctp->Dist[ct] / (double) SD_BAS_PN;
        }
    }
    //@ Fin de boucle sur les points de grille

    //@ Recherche du point de grille de distance minimum nlin0 et ncol0
    //@  tels que Dist(nlin0,ncol0)=min(Dist(nlin,ncol))
    nlin0=1;
    ncol0=1;
    minDist=ctp->Dist[0];
    for(il=1, ct=0; il<=NbLin; il++)
    {
        for(ic=1; ic<=NbCol; ic++, ct++)
        {
            if(ctp->Dist[ct] < minDist)
            {
                nlin0=il;
                ncol0=ic;
                minDist = ctp->Dist[ct];
            }
        }
    }
#ifdef OPS_DEBUG
    printf("Point de la grille de distance minimum\n");
    printf("   nlin0 : %d\n",nlin0);
    printf("   ncol0 : %d\n",ncol0);
#endif

    //@ Si (nlin0, ncol0 ) n?est pas sur un bord de la grille alors
    //@  [MinLin,MinCol,DistMin,FitQual,FlagError]=Recherche d?extremum(Dist,nlin0,ncol0)
    //@     appel SD_BAS_Extremum2D (Minimum)
    //@ Fin de Si
    if(nlin0!=1 && ncol0!=1 && nlin0!=NbLin && ncol0!=NbCol)
    {
        //@ Ajustement de paraboloide resolution d'un systeme sousresolu AX = B
        //@ Calcul de A (ordre FORTRAN)
        nlinm = nlin0 - 1;
        nlinp = nlin0 + 1;
        ncolm = ncol0 - 1;
        ncolp = ncol0 + 1;

        for(ic=0; ic<9; ic++)
        {
            if(ic==0){nl = -1; nc = -1;};
            if(ic==1){nl = 0; nc = -1;};
            if(ic==2){nl = 1; nc = -1;};
            if(ic==3){nl = -1; nc = 0;};
            if(ic==4){nl = 0; nc = 0;};
            if(ic==5){nl = 1; nc = 0;};
            if(ic==6){nl = -1; nc = 1;};
            if(ic==7){nl = 0; nc = 1;};
            if(ic==8){nl = 1; nc = 1;};

            for(il=0; il<6; il++)
            {
                if(il==0) a[il*9 + ic] = 1.0;
                if(il==1) a[il*9 + ic] = nl;
                if(il==2) a[il*9 + ic] = nc;
                if(il==3) a[il*9 + ic] = nl*nl;
                if(il==4) a[il*9 + ic] = nc*nc;
                if(il==5) a[il*9 + ic] = nl*nc;
            }
        }


        //@ Calcul de B
        b[0] = ctp->Dist[IND2(ncolm,nlinm,NbCol)];
        b[1] = ctp->Dist[IND2(ncolm,nlin0,NbCol)];
        b[2] = ctp->Dist[IND2(ncolm,nlinp,NbCol)];
        b[3] = ctp->Dist[IND2(ncol0,nlinm,NbCol)];
        b[4] = ctp->Dist[IND2(ncol0,nlin0,NbCol)];
        b[5] = ctp->Dist[IND2(ncol0,nlinp,NbCol)];
        b[6] = ctp->Dist[IND2(ncolp,nlinm,NbCol)];
        b[7] = ctp->Dist[IND2(ncolp,nlin0,NbCol)];
        b[8] = ctp->Dist[IND2(ncolp,nlinp,NbCol)];

#ifdef OPS_DEBUG
    printf("   Matrice A\n");
    for(il=0; il<6; il++)
    {
        for(ic=0; ic<9; ic++)
        {
            printf("%4.2lf \t",a[il*9+ic]);
        }
        printf("\n");
    }

    printf("   \nVecteur B\n");
    for(ic=0; ic<9; ic++)
    {
        printf("%4.2lf \t",b[ic]);
    }
    printf("\n\n");
    printf("Ajustement du paraboloide par moindres carres\n");
#endif

        //@ Calcul des moindres carres
        irc = SD_BAS_MoindresCarres1D(
	         ctx, 
	         nbeq, 
	         degre + 1, 
	         a, 
	         b, 
	         tabTmpMC, 
	         PRECISION_AJUSTEMENT_MC, 
	         x, 
	         &eps);
	  
        if(irc != CMN__STATUT_OK)
        {
#ifdef OPS_DEBUG
    printf("L'ajustement a echoue\n");
#endif
            cr_op   = SD_BAS_ISRFEM_ERR_MOINDRES_CARRES;
            msg_err = (char *)SD_BAS_MSG_ERR_MOINDRES_CARRES;
            num_lig = __LINE__;

            goto fin;
        }
#ifdef OPS_DEBUG
    printf("Ajustement OK\n");
    printf("Recherche d'extremum\n");
#endif

        //@ Recherche de l'extremum du paraboloide
        irc = SD_BAS_Extremum2D (x,eps,res,0,0);
    }

    //@ Si FlagError ou si (nlin0,ncol0) est sur un bord de la grille alors
    if(nlin0==1 || ncol0==1 || nlin0==NbLin || ncol0==NbCol || irc != CMN__STATUT_OK)
    {
#ifdef OPS_DEBUG
    if(irc == CMN__STATUT_OK)
        printf("Extremum en bord de grille\n");
    if(irc != CMN__STATUT_OK)
        printf("La recherche d'extremum a echoue\n");
#endif
        //@   FlagAxeNonQual=oui
        FlagAxeNonQual = CMN__OUI;
        //@   AxeYInstant = GridY(ncol0)
        AxeYInstant = GridY[IND2(CCD,ncol0,SD_BAS_CDX)];
        //@   AxeZInstant = GridZ(nlin0)
        AxeZInstant = GridZ[IND2(CCD,nlin0,SD_BAS_CDX)];
    }
    //@ Sinon
    else
    {
      //@ Pas de la grille
      //@   DY=GridY(ncol0+1)-GridY(ncol0-1)/2
      //@   DZ=GridZ(nlin0-1)-GridZ(nlin0+1)/2
      DY=(GridY[IND2(CCD,ncol0+1,SD_BAS_CDX)]-GridY[IND2(CCD,ncol0-1,SD_BAS_CDX)]) / 2.0;
      DZ=(GridZ[IND2(CCD,nlin0+1,SD_BAS_CDX)]-GridZ[IND2(CCD,nlin0-1,SD_BAS_CDX)]) / 2.0;

#ifdef OPS_DEBUG
    printf("   Pas de la grille DY : %lf\n",DY);
    printf("   Pas de la grille DZ : %lf\n",DZ);
#endif

#ifdef OPS_DEBUG
    printf("Extremum approche par une paraboloide\n");
    printf("   Ligne Min : %d\n",res[0]);
    printf("   Colonne Min : %d\n",res[1]);
    printf("   Distance correspondante : %d\n",res[2]);
    printf("   Erreur d'approximation : %d\n",res[3]);
#endif
        // Recuperation des valeurs
        MinLin = res[0];
        MinCol = res[1];
        DistMin = res[2];

        FitQual = res[3];
        //@   FlagAxeNonQual=non
        FlagAxeNonQual = CMN__NON;
        //@   AxeYInstant = GridY(ncol0)+MinCol*DY
        AxeYInstant = GridY[IND2(CCD,ncol0,SD_BAS_CDX)]+MinCol*DY;
        //@   AxeZInstant = GridZ(nlin0)+MinLin*DZ
        AxeZInstant = GridZ[IND2(CCD,nlin0,SD_BAS_CDX)]+MinLin*DZ;

        //@ Indices de qualite de la determination
        //@   sur la position du minimum
        /* DM 11/03/2004 */
        AxeQual = FitQual * sqrt(fabs(DY * DZ));
        
        //@   sur la conformite des mesures de shift avec la modelisation
        AxeRes = DistMin;
    }

    //@ Si Sigma( pdspix(npix)) < 3 alors FlagAxeNonQual=oui
    sum = 0.0;
    for(npix=1; npix<=4; npix++)
    {
        sum += PdsPix[IND1(npix)];
    }
    
    if(sum < 3.0)
        FlagAxeNonQual = CMN__OUI;


    // Recopie des sorties dans la listvar InfoDet

    InfoDetEng->tab_var[ENG_GIAXAXEY].val.pr8[IND1(CCD)]              = AxeYInstant;
    InfoDetEng->tab_var[ENG_GIAXAXEZ].val.pr8[IND1(CCD)]              = AxeZInstant;
    InfoDetEng->tab_var[ENG_GIAXAXEQUAL].val.pr8[IND1(CCD)]           = AxeQual;
    InfoDetEng->tab_var[ENG_GIAXAXERES].val.pr8[IND1(CCD)]            = AxeRes;
    InfoDetEng->tab_var[ENG_GIAXFLAGAXENONQUAL].val.pui4[IND1(CCD)]   = FlagAxeNonQual;

// On trace ici toutes les variables en sortie
#ifdef INVESTIGATION
    // Coord Y de l'axe instantane
    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"AxeYInstant",1,1,&AxeYInstant,ctx);
    // Coord Z de l'axe instantane
    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"AxeZInstant",1,1,&AxeZInstant,ctx);
    // Index de qualite de la position
    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"AxeQual",1,1,&AxeQual,ctx);
    // Index de vraissemblance de l'ecart mesure/modele
    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"AxeRes",1,1,&AxeRes,ctx);
    // Flag de qualite
    SD_BAS_TRACEENTIER1D(tcProcName_F,&ctx->ctxtrt,"FlagAxeNonQual",1,1,&FlagAxeNonQual,ctx);

#endif


fin:

    // Traitement des erreurs
    //================================================================
    if ( cr_op != CMN__STATUT_OK )
    {
        // mise a jour de la variable ctx->cr.errinterne
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
        SD_BAS_ERR_SET_ISRFEM,
        cr_op,
        tcProcName_F,
        num_lig,
        msg_err);

        // on imprime le message d'erreur sur la sortie standard
        #ifdef OPS_DEBUG
            SD_BAS_PRINT_ERR_INT(ctx->cr.errinterne.nomfonction,
                                 ctx->cr.errinterne.numligne,
                                 ctx->cr.errinterne.libelle);
        #endif
    }

// sortie de la fonction
return(cr_op);
}

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_25_FAX
//@
//@ Description : algorithme de filtrage de l'axe interferometrique
//@ remarque importante : dans le cas des modes intermediaires ou
//@ complet on se base ici toujours sur le contexte courant:
//@ si 0-1C alors le contexte a ete enrichi auparavant et contient l'axe
//@         instantanne du granule courant
//@ si 1A-1C alors le contexte a ete lu auparavant et on considere que
//@         les derniers elts de l'historique correspondent
//@         au granule courant
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  ctp : variables temporaires
//@         (ENTREE)  contextecourant : pointeur sur le contexte
//@                     courant configuration (avec historique)
//@         (ENTREE)  nbligne : nombre de ligne du granule traite
//@         (ENTREE)  tF : date de la premiere ligne du granule
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (Entree/Sortie)  InfoDetEng : tableau de pointeurs
//@         sur les info det du produit ENG (ordre chrono)
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_25_FAX(
        t_SD_BAS_DataThread * ctx,
        t_SD_BAS_DataThreadImageISRFEM *ctp,
        t_SD_BAS_Contexte * contextecourant,
        int nbligne,
        t_SD_BAS_SCDSTime tF,
        t_SD_BAS_ConfAutre *  ConfAutre,
        // FT-3095
  		t_SD_BAS_ConfStable * ConfStable,
  	    // ... FT-3095       
        t_SD_BAS_Liste_Var *InfoDetEng
            )
{
  static char tcProcName_F[] = "SD_ALG_25_FAX";
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  // Entree dans la fonction a partir du contexte courant

  // Gestion de erreurs
  t_CMN__Statut irc;
  char *msg_err;          // Message d'erreur
  int num_lig;


  // Recuperation des infos contenues dans les structure
  int     CCD;                 //@ Direction du coin de cube
  int     ILS;                 //@ Nombre de lignes du granule courant
  int     FLS;                 //@ Nombre de lignes du granule precedent
  double  t0;                  //@ Duree d'oubli du filtrage de l'axe interferometrique
  int     Ndeg;                //@ Degre du polynome de regression
  double  CutoffRms;           //@ Seuil de la valeur Rms de la regression

  // Sortie
  unsigned int    FlagAxeNonQual;       //@ Flag de qualite de la position filtree de l'axe interferometrique
  double          AxeRes;               //@ Index de qualite du filtrage


  // Variables internes
  double DistAxe;
  // CR 348
  double SommeW;
  // ...CR 348
  double DistAxeRms;
  double S; 
  double    y[SD_ALG_MAXDEGFAX+1];   // variable contenant les coef des polynomes
  double    z[SD_ALG_MAXDEGFAX+1];   // variable contenant les coef des polynomes
  int       j;

  // Pour l'ajustement par moindres carres
  double eps;          // Precision de l'ajustement
  double LineTimeS;    // Date de la ligne en cours convertie en secondes
  double Ttmp;        // Pour le calcul de (t-tF)/t0

  // Compteurs
  int i;
  int il, ic;
  int nbpts; // nombre de points avant la duree d'oublie

// pour les grands nombres
  t_SD_BAS_SCDSTime dateLignetemp;
  double trelatif,tref;
  double val;
  int k;
  /* DM 11/03/2004 */
  int N; /* nb de points tels que FlagAxeNonQualInstant = NON */

//#ifdef OPS_DEBUG
    printf("*** Algorithme 25_FAX ***\n====================================\n");
    printf("Entree dans la fonction \n");
//#endif

  //@ Initialisations
  CCD             = ctx->ctxtrt.CCD+1;            // Pour pouvoir utiliser les macros
  ILS             = nbligne;
  FLS             = MIN(ConfAutre->IDefFaxFLS, fabs(contextecourant->GIaxNbElt - ILS));
  t0              = ConfAutre->IDefFaxT0;
  Ndeg            = ConfAutre->IDefFaxRegressDegre;
  CutoffRms       = ConfAutre->IDefFaxRegressRmsCutoff;
  nbpts		  	  = 0;
  // FT-3095
  FlagAxeNonQual = CMN__NON;
  // ... FT-3095
  
#ifdef OPS_DEBUG
    printf("   ILS : %d\n",ILS);
    printf("   FLS : %d\n",FLS);
#endif

  //@ Initialisations des valeurs instantanees SERIE EN COURS + SERIE PRECEDENTE
  for(i=0; i<ILS+FLS; i++)
  {
    ctp->AxeYInstant[i]            = contextecourant->GIaxHistAxeInterf[i*SD_BAS_CCD+IND1(CCD)].GiaxAxeYElt;
    ctp->AxeZInstant[i]            = contextecourant->GIaxHistAxeInterf[i*SD_BAS_CCD+IND1(CCD)].GiaxAxeZElt;
    ctp->AxeQualInstant[i]         = contextecourant->GIaxHistAxeInterf[i*SD_BAS_CCD+IND1(CCD)].GiaxAxeQualElt;
    ctp->FlagAxeNonQualInstant[i]  = contextecourant->GIaxHistAxeInterf[i*SD_BAS_CCD+IND1(CCD)].GiaxFlagAxeNonQual;
  }

#ifdef OPS_DEBUG
  for(i=0; i<ILS+FLS; i++)
  {
    printf("AxeY(t),AxeZ(t),Qual(t),FlagNQ(t) : (%lf,%lf,%lf,%i)\n",
    ctp->AxeYInstant[i],ctp->AxeZInstant[i],ctp->AxeQualInstant[i],ctp->FlagAxeNonQualInstant[i]);
  }
  printf("Fin des initialisations \n");
#endif

  //5.0 PM 3
  // si le nombre de points disponible dans l'intervalle ( t-t0) est inferiaur ou egal a 2*Ndeg
  //@ Ajustement au sens des moindres carres par un polynome de degre Ndeg
  //@  de la serie temporelle des determinations instantanees de longueur
  //@ FLS+ILS avec ponderation a zero des mesures de mauvaise qualite
  //@ (i.e. FlagAxeNonQual(t)=OUI) et avec un poids w(t) = AxeQual(t)*exp(-(t-tF)/t0)
  //@  pour les autres mesures.(tF est la date de la derniere mesure de la serie ILS )

  // recherche d'une date de reference proche pour limiter la taille des nombre manipules
  for (i = 0; i < ILS + FLS; i++)
  {
    if (ctp->FlagAxeNonQualInstant[i] == CMN__NON)
    {
      dateLignetemp.day=contextecourant->GIaxHistAxeInterf[i*SD_BAS_CCD+IND1(CCD)].GiaxDateElt.day;
      dateLignetemp.millis=contextecourant->GIaxHistAxeInterf[i*SD_BAS_CCD+IND1(CCD)].GiaxDateElt.millis;
      tref = CMN__DATE_CONV_SCDS_2_SEC(dateLignetemp);
    }
  }
  
  //@ Calcul des matrices A et B
  for(ic=0; ic<ILS+FLS; ic++)
  {
    // Recuperation de la date qui correspond a la ligne en cours et conversion en secondes
    LineTimeS = 
      contextecourant->GIaxHistAxeInterf[ic*SD_BAS_CCD+IND1(CCD)].GiaxDateElt.day*86400
      + contextecourant->GIaxHistAxeInterf[ic*SD_BAS_CCD+IND1(CCD)].GiaxDateElt.millis/1000.0;

    Ttmp = (LineTimeS - (tF.day*86400 + tF.millis/1000.0))/t0;

    //5.0 PM 3
    // on compte le nombre de points dans la duree d'oubli
    if (LineTimeS >  (tF.day*86400 + tF.millis/1000.0 -t0) && ctp->FlagAxeNonQualInstant[ic] == CMN__NON )
    {
      nbpts++;
    }
#ifdef OPS_DEBUG
    printf("Date de la ligne %d : %lf\n",ic,LineTimeS);
#endif

    // pour les grands nombres
    trelatif = LineTimeS - tref;
    
    for(il=0; il<Ndeg+1; il++)
    {
      // pour les grands nombres
      for  ( k = 0 , val = 1 ; k < il ; k++) val = val * trelatif;
      ctp->A[il*(ILS+FLS)+ic] = val;
    }
    
    // Calcul du coefficient B correspondant
    if(ctp->FlagAxeNonQualInstant[ic]==CMN__OUI)
    {
      ctp->w[ic] = 0.0;
    }
    else
    {
      ctp->w[ic] = sqrt (exp(Ttmp) / ctp->AxeQualInstant[ic]);
    }
    
    for(il=0; il<Ndeg+1; il++)
    {
      /*  on applique ici aussi la matrice des poids */
      ctp->A[il*(ILS+FLS)+ic] = ctp->A[il*(ILS+FLS)+ic]*ctp->w[ic];
    }
         
    ctp->B[ic] = ctp->AxeYInstant[ic]*ctp->w[ic];
    ctp->C[ic] = ctp->AxeZInstant[ic]*ctp->w[ic];	 
  }

#ifdef OPS_DEBUG
  printf("Ajustement par moindres carres d'un polynome de degre %d\n",Ndeg);
  printf("   Matrice A\n");
  for(il=0; il<Ndeg+1; il++)
  {
    for(ic=0; ic<ILS+FLS; ic++)
    {
      printf("%lf\t",ctp->A[il*(ILS+FLS)+ic]);
    }
    printf("\n");
  }
  printf("   \nVecteur B\n");
  for(ic=0; ic<ILS+FLS; ic++)
  {
    printf("%lf\t",ctp->B[ic]);
  }
  printf("\n\n");
#endif

  /* DM 11/03/2004 */
  if ( nbpts <= 2*(Ndeg+1) )
  {
    // FT-3095
    for (i = 0; i < ILS; i++)  
    {
      // FT-3117
      ctp->AxeYFiltered[i]=ConfStable->IDefFaxGuessY[IND1(CCD)];
      ctp->AxeZFiltered[i]=ConfStable->IDefFaxGuessZ[IND1(CCD)];
      // ... FT-3117 
    }
  	// ... FT-3095
  	
    // pour differentier du cas ou le filtrage a lieu
    AxeRes=-1.;   
  }
  else
  {
    //@ Calcul de la serie temporelle filtree de longueur ILS par le polynome
    //@ d'ajustement AxeYFiltered(t),AxeZFiltered(t)
    //@  appel SD_BAS_MoindresCarres1DNdeg
#ifdef OPS_DEBUG
  printf("Appel SD_BAS_MoindresCarres1D sur les lignes \n");
#endif
    
    irc =  SD_BAS_MoindresCarres1D(ctx,ILS+FLS,Ndeg+1,ctp->A,ctp->B,ctp->tabTmpMC,CutoffRms,y,&eps);
    if(irc != CMN__STATUT_OK)
    {
#ifdef OPS_DEBUG
  printf("SD_BAS_MoindresCarres1D sur les lignes a echoue\n");
#endif
      cr_op   = SD_BAS_ISRFEM_ERR_MOINDRES_CARRES;
      msg_err = (char *)SD_BAS_MSG_ERR_MOINDRES_CARRES;
      num_lig = __LINE__;
      goto fin;
    }
    
    // CR 329
    for (i = 0; i < ILS + FLS; i++)  
    // FIN CR 329
    {
      // Recuperation de la date qui correspond a la ligne en cours et conversion en secondes
      LineTimeS = 
        contextecourant->GIaxHistAxeInterf[i*SD_BAS_CCD+IND1(CCD)].GiaxDateElt.day*86400
        + contextecourant->GIaxHistAxeInterf[i*SD_BAS_CCD+IND1(CCD)].GiaxDateElt.millis/1000.0;
      ctp->AxeYFiltered[i]=0.;
    
      // pour les grands nombres
      trelatif = LineTimeS - tref;
    
      for (j = 0 ; j < (Ndeg + 1) ; j++)
      {
        // pour les grands nombres
        for  ( k = 0 , val = 1 ; k < j ; k++) val = val * trelatif;
        ctp->AxeYFiltered[i] = ctp->AxeYFiltered[i] + y[j] * val;
      }
    }
  
#ifdef OPS_DEBUG
  printf("Appel SD_BAS_MoindresCarres1D sur les colonnes \n");
#endif
     
    irc =  SD_BAS_MoindresCarres1D(ctx,ILS+FLS,Ndeg+1,ctp->A,ctp->C,ctp->tabTmpMC,CutoffRms,z,&eps);
    if(irc != CMN__STATUT_OK)
    {
#ifdef OPS_DEBUG
  printf("SD_BAS_MoindresCarres1D sur les colonnes a echoue\n");
#endif
      cr_op   = SD_BAS_ISRFEM_ERR_MOINDRES_CARRES;
      msg_err = (char *)SD_BAS_MSG_ERR_MOINDRES_CARRES;
      num_lig = __LINE__;
      goto fin;
    }

    // CR 329
    for (i = 0; i < ILS + FLS; i++)  
    // FIN CR 329
    {
      // Recuperation de la date qui correspond a la ligne en cours et conversion en secondes
      LineTimeS = 
        contextecourant->GIaxHistAxeInterf[i*SD_BAS_CCD+IND1(CCD)].GiaxDateElt.day*86400
        + contextecourant->GIaxHistAxeInterf[i*SD_BAS_CCD+IND1(CCD)].GiaxDateElt.millis/1000.0;
      ctp->AxeZFiltered[i]=0.;
      
      // pour les grands nombres
      trelatif=LineTimeS-tref;
      
      for (j = 0 ; j < (Ndeg + 1) ; j++)
      {
        for  ( k = 0 , val = 1 ; k < j ; k++) val = val * trelatif;
        ctp->AxeZFiltered[i] = ctp->AxeZFiltered[i] + z[j] * val;
      }
    }

    //@ Calcul de la valeur RMS de la distance entre la serie temporelle instantanee
    //@ de longueur ILS et le polynome d'ajustement DistAxeRms :
    //@  cf DA 7
    DistAxe = 0.0;
    
    // CR 348
    SommeW = 0.0;
    // ...CR 348
    
    /* DM 11/03/2004 */
    N = 0;
    // CR 329
    for (ic = 0; ic < ILS + FLS; ic++)  
    // FIN CR 329
    {
      if(ctp->FlagAxeNonQualInstant[ic]==CMN__NON)
      {
	// CR 348
	        DistAxe += 
          ( (ctp->AxeYFiltered[ic] - ctp->AxeYInstant[ic])*(ctp->AxeYFiltered[ic] - ctp->AxeYInstant[ic])
          + (ctp->AxeZFiltered[ic] - ctp->AxeZInstant[ic])*(ctp->AxeZFiltered[ic] - ctp->AxeZInstant[ic]) ) * ctp->w[ic]*ctp->w[ic];
	  
	SommeW += ctp->w[ic]*ctp->w[ic];
	// ...CR348        
        /* DM 11/03/2004 */
        N = N + 1;
      }
    }

    // CR 348 
    if(SommeW > 0.0)
    {
    	DistAxe = DistAxe / SommeW;
    } //if
    // ...CR348

    /* DM 11/03/2004 */
    if (N > 0)
    {
      DistAxeRms = sqrt(DistAxe / (double) N);
    }
    else
    {
      DistAxeRms = SD_ALG_FAXAXERES_DEFAUT;
    }

//#ifdef OPS_DEBUG
  printf("   DistAxeRms = %lf\n",DistAxeRms);
//#endif

    //@  Test de qualite :
    
    // FT-3095
  	//@ Si DistAxeRms > CutoffRms, alors on renseigne l axe filtre par sa valeur par defaut
    if(DistAxeRms > CutoffRms)
    {
      for (i = 0; i < ILS; i++)  
      {
        ctp->AxeYFiltered[i]=ConfStable->IDefFaxGuessY[IND1(CCD)];
     	ctp->AxeZFiltered[i]=ConfStable->IDefFaxGuessZ[IND1(CCD)]; 
      }
      // pour differentier du cas ou le filtrage a lieu
      AxeRes=-2.; 
    } 
    else
    {
      //@ Indicateur de la qualite du filtrage :
      //@ S =  cf DA7
      //@ AxeRes = DistAxeRms / sqrt (S-Ndeg-1) (version originale=
      //@ AxeRes = DistAxeRms (apres correction demandee par le CNES)
    
      AxeRes = DistAxeRms;
    }  
  	// ... FT-3095
  	
#ifdef OPS_DEBUG
  printf("   FlagAxeNonQual = %d\n",FlagAxeNonQual);
#endif

  }
  // fin du test sur le nombre de points 


#ifdef OPS_DEBUG
  printf("   AxeRes = %d\n",AxeRes);
#endif

  // Recopie des sorties dans la lignes correspondantes
  for(ic=0; ic<ILS; ic++)
  {
    // recuperation du CCD
    // Remarque : cet algo est inclus dans une boucle de plus haut niveau sur le CCD. Toutes les
    // lignes ont donc le meme CCD a priori et on peut factoriser cette recuperation en dehors
    // de cette boucle
    InfoDetEng[nbligne-1-ic].tab_var[ENG_GFAXAXEY].val.pr8[IND1(CCD)]              = ctp->AxeYFiltered[ic];
    InfoDetEng[nbligne-1-ic].tab_var[ENG_GFAXAXEZ].val.pr8[IND1(CCD)]              = ctp->AxeZFiltered[ic];
    InfoDetEng[nbligne-1-ic].tab_var[ENG_GFAXFLAGAXENONQUAL].val.pui4[IND1(CCD)]   = FlagAxeNonQual;
    InfoDetEng[nbligne-1-ic].tab_var[ENG_GFAXAXERES].val.pr8[IND1(CCD)]            = AxeRes;
  }


// On trace ici toutes les variables en sortie
#ifdef INVESTIGATION
  // Coord Y de l'axe instantane
  SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"AxeYFiltered",ILS,1,ctp->AxeYFiltered,ctx);
  // Coord Z de l'axe instantane
  SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"AxeZFiltered",ILS,1,ctp->AxeZFiltered,ctx);
  // Index de vraissemblance de l'ecart mesure/modele
  SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"AxeRes",1,1,&AxeRes,ctx);
  // Flag de qualite
  SD_BAS_TRACE(
    tcProcName_F, 
    "FlagAxeNonQual",
    1, 
    1, 
    1,
    TRACE_UINT4, 
    TRACE_ASC,
    &FlagAxeNonQual, 
    ctx);
#endif


fin:
  // Traitement des erreurs
  //================================================================
  if ( cr_op != CMN__STATUT_OK )
  {
    // mise a jour de la variable ctx->cr.errinterne
    SD_BAS_TEST_ERR_INT(
      ctx->cr.errinterne,
      SD_BAS_ERR_SET_ISRFEM,
      cr_op,
      tcProcName_F,
      num_lig,
      msg_err);

    // on imprime le message d'erreur sur la sortie standard
#ifdef OPS_DEBUG
  SD_BAS_PRINT_ERR_INT(
    ctx->cr.errinterne.nomfonction,
    ctx->cr.errinterne.numligne,
    ctx->cr.errinterne.libelle);
#endif
    }

// sortie de la fonction
return(cr_op);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_43_ISF
//@
//@ Description : Interpolation des fonction spectrales
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  ctp : variables temporaires
//@         (ENTREE)  BanqueSpectrale : pointeur sur le
//@                 fichier BanqueSpectrale
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (Entree/Sortie)  InfoDetEng : info  det du produit ENG
//@         (Sortie)  InfoDet1ABC : info det du produit 1ABC
//@         (Sortie)  InfoCalcisp : info generales calculees sur un isp
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_43_ISF(
        t_SD_BAS_DataThread * ctx,
        t_SD_BAS_DataThreadProduit  *ctp,
        t_SD_BAS_BanqueSpectrale * BanqueSpectrale,
        t_SD_BAS_ConfAutre *  ConfAutre,
        t_SD_BAS_Liste_Var * InfoDetEng,
        t_SD_BAS_Liste_Var * InfoDet1ABC
            )
{
static char tcProcName_F[] = "SD_ALG_43_ISF";
t_CMN__Statut cr_op=CMN__STATUT_OK;
// Entree dans la fonction

    // Gestion de erreurs
    t_CMN__Statut irc;
    char *msg_err;          // Message d'erreur
    int num_lig;


    // Recuperation des infos contenues dans les structure
    //@ Entree
    int     PN;
    int     CCD;

    int     NbLin;              //@ Nombre de lignes de la grille des positions de l'axe interferometrique
    int     NbCol;              //@ Nombre de colonnes de la grille des positions de l'axe interferometrique
    double  *GridY;             //@ Coordonnees angulaires des points de la grille selon Y
    double  *GridZ;             //@ Coordonnees angulaires des points de la grille selon Z
    int     NsfirstSpectralTab; //@ Numero d'echantillon de la premiere fonction d'instrument
    int     NslastSpectralTab;  //@ Numero d'echantillon de la derniere fonction d'instrument
    //double  DWnSpectralTab;     //@ Pas d'echantillonnage des fonctions spectrales
    int     NsOpd;              //@ Nombre de points de discretisation de l'Opd
    double  OpdMax;             //@ Difference de marche optique maximum representee par la fonction d'auto-apodisation
    double  *FcsTab;            //@ Fonctions de calibration spectrales de la banque de donnees spectrales
    cmplx  *AfTab;             //@ Fonctions d'apodisation de la banque de donnees spectrales
    double  AxeY;               //@ Coordonnees y filtree de l'axe interferometrique
    double  AxeZ;               //@ Coordonnees z filtree de l'axe interferometrique
    int     NbSWwidth;          //@ Demi-largeur des fenetres d'apodisation
    int     NsErase;            //@ Nombre d'echantillons mis a 0 dans la fonction d'apodisation

    //@ Sorties
    int     LinOrigin;          //@ Origine en ligne de grille de l'interpolation dans la BD spectrale
    int     ColOrigin;          //@ Origine en colonne de grille de l'interpolation dans la BD spectrale
    double  Pds1;               //@ Poids du point 1 de l'interpolation
    double  Pds2;               //@ Poids du point 2 de l'interpolation
    double  Pds3;               //@ Poids du point 3 de l'interpolation
    double  Pds4;               //@ Poids du point 4 de l'interpolation
    unsigned int FlagPdsNonValid;


    //@ Internes
    double Surf;
    int nl;
    int nc;
    dcmplx *Af0;
    double *REtmp;
    double *IMtmp;

    //@ Compteurs
    int il;             // Compteur ligne
    int ic;             // Compteur colonne
    int m;
    int n;              // Nombres d'onde
    int l;

#ifdef OPS_DEBUG
    printf("*** Algorithme 43_ISF ***\n====================================\n");
    printf("Entree dans la fonction \n");
#endif

    //@ Initialisations
    PN                  = ctx->ctxtrt.PN;
    CCD                 = ctx->ctxtrt.CCD + 1;          // Pour pouvoir utiliser les macros

    NbLin               = BanqueSpectrale->IDefSdbGridNbLin;
    NbCol               = BanqueSpectrale->IDefSdbGridNbCol;
    GridY               = BanqueSpectrale->IDefSdbGridY;
    GridZ               = BanqueSpectrale->IDefSdbGridZ;
    NsfirstSpectralTab  = BanqueSpectrale->IDefSafNsfirst;
    NslastSpectralTab   = BanqueSpectrale->IDefSafNslast;
    //DWnSpectralTab      = BanqueSpectrale->IDefSafDWn;
    NsOpd               = BanqueSpectrale->IDefSafNsOpd;
    OpdMax              = BanqueSpectrale->IDefSafOpdMax;
    FcsTab              = BanqueSpectrale->ISdbGridFcs;
    AfTab               = BanqueSpectrale->ISdbGridAf;

    AxeY                = InfoDetEng->tab_var[ENG_GFAXAXEY].val.pr8[IND1(CCD)];
    AxeZ                = InfoDetEng->tab_var[ENG_GFAXAXEZ].val.pr8[IND1(CCD)];
    NbSWwidth           = ConfAutre->IDefS1cNbSWwidth;
    NsErase             = ConfAutre->IDefS1cNsErase;
    LinOrigin=0;
    ColOrigin=0;

#ifdef OPS_DEBUG
    printf("Fin des initialisations \n");
#endif

    //@ Si (AxeY, AxeZ) est a l?interieur de la grille c?est a dire :
    //@   GridZ(NbLin) < AxeZ <= GridZ(1)et GridY(1) <= AxeY < GridY(NbCol)
    if(     GridZ[IND2(CCD,NbLin,SD_BAS_CDX)] < AxeZ && AxeZ <= GridZ[IND2(CCD,1,SD_BAS_CDX)]
        &&  GridY[IND2(CCD,1,SD_BAS_CDX)] <= AxeY && AxeY < GridY[IND2(CCD,NbCol,SD_BAS_CDX)]
       )
    {
        //@   FlagPdsNonValid=NON
        FlagPdsNonValid = CMN__NON;

        //@   Recherche du point de la grille definissant le coin haut gauche du pave
        //@   contenant la position de l?axe interferometrique (nl,nc) tel que :
        //@   GridZ(nl+1) < AxeZ <= GridZ(nl)et GridY(nc) <= AxeY < GridY(nc+1)
        //@   LinOrigin=nl ColOrigin=nc
        for(il=1; il<=NbLin; il++)
        {
            for(ic=1; ic<=NbCol; ic++)
            {
                if(     GridZ[IND2(CCD,il+1,SD_BAS_CDX)] < AxeZ && AxeZ <= GridZ[IND2(CCD,il,SD_BAS_CDX)]
                    &&  GridY[IND2(CCD,ic,SD_BAS_CDX)] <= AxeY && AxeY < GridY[IND2(CCD,ic+1,SD_BAS_CDX)]
                  )
                {
                    nl = il;
                    nc = ic;

                }
            }
        }

        LinOrigin = nl;
        ColOrigin = nc;
        //@   Calcul des poids d?interpolation si FlagPdsNonValid=NON
        //@     Surf=( GridY(nc+1)-GridY(nc))*( GridZ(nl)-GridZ(nl+1))
        Surf=( GridY[IND2(CCD,nc+1,SD_BAS_CDX)]-GridY[IND2(CCD,nc,SD_BAS_CDX)])*( GridZ[IND2(CCD,nl,SD_BAS_CDX)]-GridZ[IND2(CCD,nl+1,SD_BAS_CDX)]);
        //@     Pds1=(GridY(nc+1)-AxeY)*(AxeZ-GridZ(nl+1))/Surf
        Pds1=(GridY[IND2(CCD,nc+1,SD_BAS_CDX)]-AxeY)*(AxeZ-GridZ[IND2(CCD,nl+1,SD_BAS_CDX)])/Surf;
        //@     Pds2=(AxeY-GridY(nc))*(AxeZ-GridZ(nl+1)) /Surf
        Pds2=(AxeY-GridY[IND2(CCD,nc,SD_BAS_CDX)])*(AxeZ-GridZ[IND2(CCD,nl+1,SD_BAS_CDX)]) /Surf;
        //@     Pds3=( AxeY-GridY(nc))*(GridZ(nl)-AxeZ) /Surf
        Pds3=( AxeY-GridY[IND2(CCD,nc,SD_BAS_CDX)])*(GridZ[IND2(CCD,nl,SD_BAS_CDX)]-AxeZ) /Surf;
        //@     Pds4=(GridY(nc+1)-AxeY)*( GridZ(nl)-AxeZ) /Surf
        Pds4=(GridY[IND2(CCD,nc+1,SD_BAS_CDX)]-AxeY)*( GridZ[IND2(CCD,nl,SD_BAS_CDX)]-AxeZ) /Surf;
    }
    //@ Sinon
    else
    {
       //@   FlagPdsNonValid=OUI
       FlagPdsNonValid = 1;
       //@   Pds1=1 Pds2=Pds3=Pds4=0
       Pds1 = 1.0;
       Pds2 = 0.0;
       Pds3 = 0.0;
       Pds4 = 0.0;
    }
    //@ Fin de si

#ifdef OPS_DEBUG
    printf("Calcul des poids \n");
    printf("   Poids 1 : %lf \n",Pds1);
    printf("   Poids 2 : %lf \n",Pds2);
    printf("   Poids 3 : %lf \n",Pds3);
    printf("   Poids 4 : %lf \n",Pds4);
#endif


    // On factorise le calcul de la base pour gagner du temps
    //===========================================================
    //@   Base des opd tabulees pour tout m=1 a NsOpd
    for(m=1; m<=NsOpd; m++)
    {
        //@     OpdTab(m)=(m-NsOpd/2+1)*OpdMax/(NsOpd-1)
      // DM ALGO 5.1
        ctp->OpdTab[IND1(m)]=(double)(m-NsOpd/2-1)*OpdMax/(double)(NsOpd-1);

    }

    //@   Base des opd pour S1c pour tout l = 1 a 2*NbSWwidth+1
    for(l=1; l<=2*NbSWwidth+1; l++)
    {
        //@     Opd(l)=(l-NbSWwidth+1)*OpdMax/(2*NbSWwidth)
        ctp->Opd[IND1(l)]=(double)(l-(NbSWwidth+1))*OpdMax/(double)(2*NbSWwidth);
    }

#ifdef OPS_DEBUG
    printf("Base des opd tabulees  : NsOpd = %d\n",NsOpd);
    for(m=1; m<=NsOpd; m++)
    {
        printf("\t%lf",ctp->OpdTab[IND1(m)]);
    }
    printf("\n");

    printf("Base des opd tabulees : NbSWwidth = %d\n",NbSWwidth);

    for(l=1; l<=2*NbSWwidth+1; l++)
    {
        printf("\t%lf",ctp->Opd[IND1(l)]);
    }
    printf("\n");
#endif



#ifdef OPS_DEBUG
    printf("NsfirstSpectralTab = %d\n",NsfirstSpectralTab);
    printf("NslastSpectralTab = %d\n",NslastSpectralTab);
#endif



    //@ Interpolation pour tous les nombres d'onde tabules n de NsfirstSpectralTab
    //@ a NslastFcsTab
    for(n=NsfirstSpectralTab; n<=NslastSpectralTab; n++)
    {
#ifdef OPS_DEBUG
    printf("Boucle sur les nombres d'onde n=%d\n",n);
    printf("FlagPdsNonValid = %d\n",FlagPdsNonValid);
#endif

        //@ Si FlagPdsNonValid=NON
        if(FlagPdsNonValid==CMN__NON)
        {
            for(m=1; m<=NsOpd; m++)
            {
                //@   Af0(m,n)= AfTab(m,n,nl,nc)*Pds1+ AfTab(m,n,nl,nc+1)*Pds2
                //@     + AfTab(m,n,nl+1,nc+1)*Pds3+ AfTab(m,n,nl+1,nc)*Pds4

                // Partie reelle
                RE(ctp->Af0[IND1(m)]) =   RE(AfTab[IND6(m,n-NsfirstSpectralTab+1,PN,CCD,nc,nl,SD_BAS_SAF_NS_OPD,SD_BAS_SDB_NFSMAX,SD_BAS_PN,SD_BAS_CDX,SD_BAS_SDB_NB_COL)])*Pds1
                                        + RE(AfTab[IND6(m,n-NsfirstSpectralTab+1,PN,CCD,nc+1,nl,SD_BAS_SAF_NS_OPD,SD_BAS_SDB_NFSMAX,SD_BAS_PN,SD_BAS_CDX,SD_BAS_SDB_NB_COL)])*Pds2
                                        + RE(AfTab[IND6(m,n-NsfirstSpectralTab+1,PN,CCD,nc+1,nl+1,SD_BAS_SAF_NS_OPD,SD_BAS_SDB_NFSMAX,SD_BAS_PN,SD_BAS_CDX,SD_BAS_SDB_NB_COL)])*Pds3
                                        + RE(AfTab[IND6(m,n-NsfirstSpectralTab+1,PN,CCD,nc,nl+1,SD_BAS_SAF_NS_OPD,SD_BAS_SDB_NFSMAX,SD_BAS_PN,SD_BAS_CDX,SD_BAS_SDB_NB_COL)])*Pds4;


                // Partie imaginaire
                IM(ctp->Af0[IND1(m)]) =   IM(AfTab[IND6(m,n-NsfirstSpectralTab+1,PN,CCD,nc,nl,SD_BAS_SAF_NS_OPD,SD_BAS_SDB_NFSMAX,SD_BAS_PN,SD_BAS_CDX,SD_BAS_SDB_NB_COL)])*Pds1
                                        + IM(AfTab[IND6(m,n-NsfirstSpectralTab+1,PN,CCD,nc+1,nl,SD_BAS_SAF_NS_OPD,SD_BAS_SDB_NFSMAX,SD_BAS_PN,SD_BAS_CDX,SD_BAS_SDB_NB_COL)])*Pds2
                                        + IM(AfTab[IND6(m,n-NsfirstSpectralTab+1,PN,CCD,nc+1,nl+1,SD_BAS_SAF_NS_OPD,SD_BAS_SDB_NFSMAX,SD_BAS_PN,SD_BAS_CDX,SD_BAS_SDB_NB_COL)])*Pds3
                                        + IM(AfTab[IND6(m,n-NsfirstSpectralTab+1,PN,CCD,nc,nl+1,SD_BAS_SAF_NS_OPD,SD_BAS_SDB_NFSMAX,SD_BAS_PN,SD_BAS_CDX,SD_BAS_SDB_NB_COL)])*Pds4;
            }

            //@   Fcs(n)=FcsTab(n,nl,nc)*Pds1+ FcsTab(n,nl,nc+1)*Pds2
            //@     + FcsTab(n,nl+1,nc+1)*Pds3+ FcsTab(n,nl+1,nc)*Pds4
            ctp->Fcs[IND3(n-NsfirstSpectralTab+1,PN,CCD,SD_BAS_SDB_NFSMAX,SD_BAS_PN)] =    FcsTab[IND5(n-NsfirstSpectralTab+1,PN,CCD,nc,nl,SD_BAS_SDB_NFSMAX,SD_BAS_PN,SD_BAS_CDX,SD_BAS_SDB_NB_COL)]*Pds1
                                             +  FcsTab[IND5(n-NsfirstSpectralTab+1,PN,CCD,nc+1,nl,SD_BAS_SDB_NFSMAX,SD_BAS_PN,SD_BAS_CDX,SD_BAS_SDB_NB_COL)]*Pds2
                                             +  FcsTab[IND5(n-NsfirstSpectralTab+1,PN,CCD,nc+1,nl+1,SD_BAS_SDB_NFSMAX,SD_BAS_PN,SD_BAS_CDX,SD_BAS_SDB_NB_COL)]*Pds3
                                             +  FcsTab[IND5(n-NsfirstSpectralTab+1,PN,CCD,nc,nl+1,SD_BAS_SDB_NFSMAX,SD_BAS_PN,SD_BAS_CDX,SD_BAS_SDB_NB_COL)]*Pds4;
        }
        //@ Sinon
        else
        {
            //@   Fcs(n)=FcsTab(n,NbLin/2+1,NbCol/2+1)
            //@   Af0(NsOpd,n)=AfTab(NsOpd,n,NbLin/2+1,NbCol/2+1)
            ctp->Fcs[IND3(n-NsfirstSpectralTab+1,PN,CCD,SD_BAS_SDB_NFSMAX,SD_BAS_PN)] = FcsTab[IND5(n-NsfirstSpectralTab+1,PN,CCD,NbCol/2+1,NbLin/2+1,SD_BAS_SDB_NFSMAX,SD_BAS_PN,SD_BAS_CDX,SD_BAS_SDB_NB_COL)];

            for(m=1; m<=NsOpd; m++)
            {
                RE(ctp->Af0[IND1(m)]) = RE(AfTab[IND6(m,n-NsfirstSpectralTab+1,PN,CCD,NbCol/2+1,NbLin/2+1,SD_BAS_SAF_NS_OPD,SD_BAS_SDB_NFSMAX,SD_BAS_PN,SD_BAS_CDX,SD_BAS_SDB_NB_COL)]);
                IM(ctp->Af0[IND1(m)]) = IM(AfTab[IND6(m,n-NsfirstSpectralTab+1,PN,CCD,NbCol/2+1,NbLin/2+1,SD_BAS_SAF_NS_OPD,SD_BAS_SDB_NFSMAX,SD_BAS_PN,SD_BAS_CDX,SD_BAS_SDB_NB_COL)]);
            }
        }


        //@ Re-echantillonnage des fonctions d'apodisation pour leur application dans
        //@ l?algorithme S1c

        //@   Interpolation spline cubique
        //@    (OpdTab(m), Af0(m,n)) donne (Opd(l), Af(l,n))
        //@     appel SD_BAS_InterpSpline1D

        //@ Interpolation par spline : Initialisation de la fonction
        //@ Separation des parties reelles et imaginaires

        // Preparation des variables temporaires
        Af0     = ctp->Af0;
        REtmp   = ctp->REtmp;
        IMtmp   = ctp->IMtmp;
        for(m=0; m<NsOpd; m++)
        {
           *REtmp = RE(*Af0);
           *IMtmp = IM(*Af0);
           Af0++;
           REtmp++;
           IMtmp++;
        }


        //@ Interpolation par spline : Initialisation partie reelle
        irc = SD_BAS_InterpSpline1D(SD_ALG_INIT_SPLINE,NsOpd,ctp->OpdTab,
                                    ctp->REtmp,ctp->y2isf,
                                    2*NbSWwidth+1,ctp->Opd,ctp->REint);

        if(irc != CMN__STATUT_OK)
        {
            cr_op   = SD_BAS_ISRFEM_ERR_SPLINE;
            msg_err = (char *)SD_BAS_MSG_ERR_SPLINE;
            num_lig = __LINE__;
            goto fin;
        }


        //@ Interpolation par spline : Calcul partie reelle
        irc = SD_BAS_InterpSpline1D(SD_ALG_CALCUL_SPLINE,NsOpd,ctp->OpdTab,
                                    ctp->REtmp,ctp->y2isf,
                                    2*NbSWwidth+1,ctp->Opd,ctp->REint);

        if(irc != CMN__STATUT_OK)
        {
            cr_op   = SD_BAS_ISRFEM_ERR_SPLINE;
            msg_err = (char *)SD_BAS_MSG_ERR_SPLINE;
            num_lig = __LINE__;
            goto fin;
        }

        irc = SD_BAS_InterpSpline1D(SD_ALG_INIT_SPLINE,NsOpd,ctp->OpdTab,
                                    ctp->IMtmp,ctp->y2isf,
                                    2*NbSWwidth+1,ctp->Opd,ctp->IMint);

        if(irc != CMN__STATUT_OK)
        {
            cr_op   = SD_BAS_ISRFEM_ERR_SPLINE;
            msg_err = (char *)SD_BAS_MSG_ERR_SPLINE;
            num_lig = __LINE__;
            goto fin;
        }

        //@ Interpolation par spline : Calcul partie imaginaire
        irc = SD_BAS_InterpSpline1D(SD_ALG_CALCUL_SPLINE,NsOpd,ctp->OpdTab,
                                     ctp->IMtmp,ctp->y2isf, 
                                    2*NbSWwidth+1,ctp->Opd,ctp->IMint);

        if(irc != CMN__STATUT_OK) 
        {
            cr_op   = SD_BAS_ISRFEM_ERR_SPLINE;
            msg_err = (char *)SD_BAS_MSG_ERR_SPLINE;
            num_lig = __LINE__;
            goto fin;
        }

        // Refabrication de Af
        for(m=1; m<=2*NbSWwidth+1; m++)
        {
           RE(ctp->Af[IND4(m,(n-NsfirstSpectralTab+1),PN,CCD,(2*NbSWwidth+1),SD_BAS_SDB_NFSMAX,SD_BAS_PN)]) = ctp->REint[IND1(m)];
           IM(ctp->Af[IND4(m,(n-NsfirstSpectralTab+1),PN,CCD,(2*NbSWwidth+1),SD_BAS_SDB_NFSMAX,SD_BAS_PN)]) = ctp->IMint[IND1(m)];
        }

        //@   Mise a zero des extremitees
        //@   Af(l,n)=0 pour l=1 a NsErase
        for(l=1; l<=NsErase; l++)
        {
            RE(ctp->Af[IND4(l,(n-NsfirstSpectralTab+1),PN,CCD,(2*NbSWwidth+1),SD_BAS_SDB_NFSMAX,SD_BAS_PN)])=0.0;
            IM(ctp->Af[IND4(l,(n-NsfirstSpectralTab+1),PN,CCD,(2*NbSWwidth+1),SD_BAS_SDB_NFSMAX,SD_BAS_PN)])=0.0;
        }
        //@   et pour l=2*NbSWwidth-NsErase+1 a 2*NbSWwidth
        for(l=2*NbSWwidth-NsErase+2; l<=2*NbSWwidth+1; l++)
        {
            RE(ctp->Af[IND4(l,(n-NsfirstSpectralTab+1),PN,CCD,(2*NbSWwidth+1),SD_BAS_SDB_NFSMAX,SD_BAS_PN)])=0.0;
            IM(ctp->Af[IND4(l,(n-NsfirstSpectralTab+1),PN,CCD,(2*NbSWwidth+1),SD_BAS_SDB_NFSMAX,SD_BAS_PN)])=0.0;
        }
    }

#ifdef OPS_DEBUG
    printf("Fin des calculs, sauvegarde des sorties \n");
#endif

     // Recopie des sorties dans les listvar ad hoc
     InfoDet1ABC->tab_var[N1C_GISFLINORIGIN].val.pui4[IND1(CCD)]             = LinOrigin;
     InfoDet1ABC->tab_var[N1C_GISFCOLORIGIN].val.pui4[IND1(CCD)]             = ColOrigin;
     InfoDet1ABC->tab_var[N1C_GISFPDS1].val.pr8[IND1(CCD)]                   = Pds1;
     InfoDet1ABC->tab_var[N1C_GISFPDS2].val.pr8[IND1(CCD)]                   = Pds2;
     InfoDet1ABC->tab_var[N1C_GISFPDS3].val.pr8[IND1(CCD)]                   = Pds3;
     InfoDet1ABC->tab_var[N1C_GISFPDS4].val.pr8[IND1(CCD)]                   = Pds4;
     InfoDetEng->tab_var[ENG_GISFFLAGPDSNONVALID].val.pui4[IND1(CCD)]         = FlagPdsNonValid;


// On trace ici toutes les variables en sortie
#ifdef INVESTIGATION
    // Ligne coin haut gauche du pave contenant l'axe interferometrique
    SD_BAS_TRACEENTIER1D(tcProcName_F,&ctx->ctxtrt,"LinOrigin",1,1,&LinOrigin,ctx);
    // Colonne coin haut gauche du pave contenant l'axe interferometrique
    SD_BAS_TRACEENTIER1D(tcProcName_F,&ctx->ctxtrt,"ColOrigin",1,1,&ColOrigin,ctx);
    // Poids du point 1 de l'interpolation
    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"Pds1",1,1,&Pds1,ctx);
    // Poids du point 2 de l'interpolation
    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"Pds2",1,1,&Pds2,ctx);
    // Poids du point 3 de l'interpolation
    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"Pds3",1,1,&Pds3,ctx);
    // Poids du point 4 de l'interpolation
    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"Pds4",1,1,&Pds4,ctx);
    // Flag de validite des poids
    SD_BAS_TRACE(tcProcName_F, "FlagPdsNonValid",
                 1, 1, 1,
                 TRACE_UINT4, TRACE_ASC,
                 &FlagPdsNonValid, ctx);
     // Nombres d'ondes
     SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"Fcs",NslastSpectralTab-NsfirstSpectralTab+1,1,&ctp->Fcs[IND3(1,PN,CCD,SD_BAS_SDB_NFSMAX,SD_BAS_PN)],ctx);
#endif


fin:

    // Traitement des erreurs
    //================================================================
    if ( cr_op != CMN__STATUT_OK )
    {
        // mise a jour de la variable ctx->cr.errinterne
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
        SD_BAS_ERR_SET_ISRFEM,
        cr_op,
        tcProcName_F,
        num_lig,
        msg_err);

        // on imprime le message d'erreur sur la sortie standard
        #ifdef OPS_DEBUG
            SD_BAS_PRINT_ERR_INT(ctx->cr.errinterne.nomfonction,
                                 ctx->cr.errinterne.numligne,
                                 ctx->cr.errinterne.libelle);
        #endif
    }

// sortie de la fonction



// sortie de la fonction
return(cr_op);

}


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_TransformIpsf
//@
//@ Description : calcule la valeur du poids de l'IPSF sondeur correspondant
//@ a chaque pixel d'imageur.
//@
//@
//@ Parameters :
//@         (ENTREE)
//@         (Sortie)
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_TransformIpsf(
    t_SD_BAS_DataThread * ctx,
    t_SD_BAS_DataThreadImageISRFEM *ctp,
    int                 NbLin,
    int                 NbCol,
    double              *Ypsf,
    double              *Zpsf,
    double              *WgtPsf,
    int                 LinDeb,
    int                 ColDeb,
    int                 LinFin,
    int                 ColFin,
    double              *IISY,
    double              *IISZ,
    double              ImgDY,
    double              ImgDZ,
    double              OffsetSondIIS1,
    double              OffsetSondIIS2,
    int                 Osf
)
{
    t_CMN__Statut cr_op=CMN__STATUT_OK;

    int PN;
    int CpsfDeb;
    int CpsfFin;
    int LpsfDeb;
    int LpsfFin;
    int NbColpsf;
    double SurfPixImg;
    double SumWgtpsf;
    double SurfSamp;
    double Zspl;
    double Yspl;
    double DYpsf;
    double DZpsf;
    double Norm;
    double Ymin,Ymax,Zmin,Zmax;
    double Csty,Cstz;
    double Invnorm;
    double  Zsplinit;
    double  Ysplinit;
    double  InvDYpsf;
    double  InvDZpsf;

    // Compteurs de boucle
    int il,ic;
    int jl,jc;
    int kl,kc;
    int ct;

    // Initialisations
    PN                  = ctx->ctxtrt.PN;


#ifdef OPS_DEBUG
    printf("Debut de la fonction transform IPSF\n----------------------------------\n");
#endif


    DYpsf = Ypsf[IND2(2,PN,SD_BAS_NB_COL_SOND)] - Ypsf[IND2(1,PN,SD_BAS_NB_COL_SOND)];
    DZpsf = Zpsf[IND2(2,PN,SD_BAS_NB_LIN_SOND)] - Zpsf[IND2(1,PN,SD_BAS_NB_LIN_SOND)];
//optimisation
    Csty=DYpsf/(double)Osf;
    Cstz=DZpsf/(double)Osf;

    //@ Initialisationd de la normalisation
    Norm = 0.0;

    //@ Application de l'offset sondeur/imageur
    for(il=LinDeb, ct=0; il<=LinFin; il++)
    {
        for(ic=ColDeb; ic<=ColFin; ic++, ct++)
        {
//            ctp->ImgY[ct] = IISY[ct] + OffsetSondIIS1;
//            ctp->ImgZ[ct] = IISZ[ct] + OffsetSondIIS2;
//  Version 5.0
            ctp->ImgY[ct] = IISY[ct] - OffsetSondIIS1;
            ctp->ImgZ[ct] = IISZ[ct] - OffsetSondIIS2;
        }
    }


//optimisation
    InvDYpsf=1/DYpsf;
    InvDZpsf=1/DZpsf;
    for(il=LinDeb, ct=0; il<=LinFin; il++)
    {
        for(ic=ColDeb; ic<=ColFin; ic++, ct++)
        {
            //@ Calcul du domaine Ipsf associe au pixel imageur
//            CpsfDeb = NINT( MAX(   1.0, 1.0 + floor( (ctp->ImgY[ct]-fabs(ImgDY)/2 - Ypsf[IND2(1,PN,SD_BAS_NB_COL_SOND)])/DYpsf )));
            CpsfDeb = NINT( MAX(   1.0, 1.0 + floor( (ctp->ImgY[ct]-fabs(ImgDY)*0.5 - Ypsf[IND2(1,PN,SD_BAS_NB_COL_SOND)])*InvDYpsf )));
//            LpsfDeb = NINT( MAX(   1.0, 1.0 + floor( (ctp->ImgZ[ct]-fabs(ImgDZ)/2 - Zpsf[IND2(1,PN,SD_BAS_NB_LIN_SOND)])/DZpsf )));
            LpsfDeb = NINT( MAX(   1.0, 1.0 + floor( (ctp->ImgZ[ct]-fabs(ImgDZ)*0.5 - Zpsf[IND2(1,PN,SD_BAS_NB_LIN_SOND)])*InvDZpsf )));
//            CpsfFin = NINT( MIN( NbCol, 1.0 + ceil(  (ctp->ImgY[ct]+fabs(ImgDY)/2 - Ypsf[IND2(1,PN,SD_BAS_NB_COL_SOND)])/DYpsf )));
            CpsfFin = NINT( MIN( NbCol, 1.0 + ceil(  (ctp->ImgY[ct]+fabs(ImgDY)*0.5 - Ypsf[IND2(1,PN,SD_BAS_NB_COL_SOND)])*InvDYpsf )));
//            LpsfFin = NINT( MIN( NbLin, 1.0 + ceil(  (ctp->ImgZ[ct]+fabs(ImgDZ)/2 - Zpsf[IND2(1,PN,SD_BAS_NB_LIN_SOND)])/DZpsf )));
            LpsfFin = NINT( MIN( NbLin, 1.0 + ceil(  (ctp->ImgZ[ct]+fabs(ImgDZ)*0.5 - Zpsf[IND2(1,PN,SD_BAS_NB_LIN_SOND)])*InvDZpsf )));
            NbColpsf = CpsfFin - CpsfDeb + 1;

            //@ Initialisation de l'integration sur le domaine du pixel imageur
            SurfPixImg = 0.0;
            SumWgtpsf = 0.0;
            //precalculs
//optimisation
            Ymin = (ctp->ImgY[ct] - fabs(ImgDY)*0.5);
            Ymax = (ctp->ImgY[ct] + fabs(ImgDY)*0.5);
            Zmin = (ctp->ImgZ[ct] - fabs(ImgDZ)*0.5);
            Zmax = (ctp->ImgZ[ct] + fabs(ImgDZ)*0.5);

            //@ Calcul de la surface et du poids associes aux points du domaine Ipsf utile
            for(jl=LpsfDeb; jl<=LpsfFin; jl++)
            {
//optimisation
                Zsplinit= Zpsf[IND2(jl,PN,SD_BAS_NB_LIN_SOND)]  - ((double)Osf*0.5 + 0.5) * Cstz;
                for(jc=CpsfDeb; jc<=CpsfFin; jc++)
                {
                    //@ Initialisation
                   SurfSamp = 0.0;

//optimisation
                   Ysplinit= Ypsf[IND2(jc,PN,SD_BAS_NB_COL_SOND)]  - ((double)Osf*0.5 + 0.5) * Csty;
                   Zspl= Zsplinit;
                   for(kl=1; kl<=Osf; kl++)
                   {

                       Zspl=  Zspl + Cstz;
//optimisation
//                         if(    (ctp->ImgZ[ct] - fabs(ImgDZ)/2.0) <= Zspl
//                           && (Zspl <= ctp->ImgZ[ct] + fabs(ImgDZ)/2.0) )
                       if( (Zmin <= Zspl) && (Zspl <= Zmax) )
                       {
//optimisation
                        Yspl= Ysplinit;
//                        Zspl = Zpsf[IND2(jl,PN,SD_BAS_NB_LIN_SOND)] + (kl-(Osf/2.0 + 0.5))*DZpsf/Osf;
                            for(kc=1; kc<=Osf; kc++)
                            {
//optimisation
//                                Yspl = Ypsf[IND2(jc,PN,SD_BAS_NB_COL_SOND)] + (kc-(Osf/2.0+0.5))*DYpsf/Osf;
                                Yspl= Yspl + Csty;

//optimisation
//                                if(    (ctp->ImgY[ct] - fabs(ImgDY)/2.0) <= Yspl
//                                        && (Yspl <= ctp->ImgY[ct] + fabs(ImgDY)/2.0) )
                                if( (Ymin <= Yspl) && (Yspl <= Ymax) )
                                {
                                  SurfSamp += 1.0;
                                }
                            }
                        }
                    }
                    SurfPixImg = SurfPixImg + SurfSamp;
                    SumWgtpsf = SumWgtpsf + SurfSamp * WgtPsf[IND1(jl)*SD_BAS_NB_COL_SOND + IND1(jc)];
                }
            }

            //@ Calcul du poids Ipsf associe a chaque pixel de l'imageur
            if(!SD_BAS_MacEGAUX_D(SurfPixImg,0.0))
            {
                ctp->IISWgtIpsf[ct] = SumWgtpsf / SurfPixImg;
            }
            else
            {
                ctp->IISWgtIpsf[ct] = 0.0;
            }

            Norm = Norm + ctp->IISWgtIpsf[ct];
        }
    }

#ifdef OPS_DEBUG
    printf("   Coefficient de normalisation des poids : %lf \n",Norm);
#endif

    //@ Calcul du poids normalise
//  optimisation
    Invnorm=1./Norm;
    for(il=LinDeb, ct=0; il<=LinFin; il++)
    {
        for(ic=ColDeb; ic<=ColFin; ic++, ct++)
        {
//            ctp->IISWgtIpsf[ct] /= Norm;
            ctp->IISWgtIpsf[ct] *= Invnorm;
        }
   }
    // Fin de la fonction TransformIpsf
#ifdef OPS_DEBUG
    printf("Fin de la fonction transform IPSF\n----------------------------------\n");
#endif

return cr_op;
}


//======================================================================
//                  FONCTIONS INTERNES
//======================================================================
