//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/ALG/src/SD_ALG_ISRFEM.h,v 1.4 2013/03/29 16:55:26 lcaspar Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_ALG
//@ MODULE : SD_ALG_ISRFEM
//@
//@ Comment :
//@    include file for the SD_ALG_ISRFEM class which implements the
//@    algorithm used to compute IASI L1 ISRFEM chain
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ VERSION : 11/02/2003 : [reference :] Mathieu JUNG
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ PM-ID : PM4 : 31/03/2004 : B. MASSART
//@ SD_ALG_21_SSS:
//@ -date UTC de la ligne (SN=15)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-3095(DM-2100-OPS) : 27/03/2013 : P. JULIAN
//@ Amelioration de la position de l axe filtre SD_ALG_25_FAX
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_ALG_ISRFEM_h
#define SD_ALG_ISRFEM_h


// inserer ici les fichiers include
#include <stdio.h>       /* entree/sortie */
#include <stdlib.h>      /* alloc memoire, ...*/
#include <string.h>      /* chaines de carateres */
#include <unistd.h>      /* gestion pipes, ... */
#include <time.h>        /* gestion de la date */
#include <sys/param.h>   /* constantes systeme */

#include "SD_BAS_Global.h"
#include "SD_BAS_BanqueSpectrale.h"
#include "SD_BAS_Niveau1.h"
#include "SD_BAS_Niveau0.h"
#include "SD_BAS_Contexte.h"
#include "SD_BAS_ConfAutre.h"
#include "SD_BAS_ConfStable.h"
#include "SD_BAS_Produits.h"


/*====================================================================*/
//   External define  Declarations
/*====================================================================*/
#define ERR_BASE_ISRFEM                         SD_ALG_ERR_BASE_ISRFEM
#define SD_BAS_ISRFEM_ERR_APODISATION           (ERR_BASE_ISRFEM+1)
#define SD_BAS_ISRFEM_ERR_PREPARATION_FFT       (ERR_BASE_ISRFEM+2)
#define SD_BAS_ISRFEM_ERR_NFIRST_NLAST          (ERR_BASE_ISRFEM+3)
#define SD_BAS_ISRFEM_ERR_SPLINE                (ERR_BASE_ISRFEM+4)
#define SD_BAS_ISRFEM_ERR_MOINDRES_CARRES       (ERR_BASE_ISRFEM+5)
#define SD_BAS_ISRFEM_ERR_IPSF                  (ERR_BASE_ISRFEM+6)

#define SD_BAS_ERR_SET_ISRFEM                   1

/*====================================================================*/
// Constant Declaration
/*====================================================================*/
// Valeurs de ctx_appel pour SD_ALG_22_SOS
// FT-2765
static const int ISRFEM    =0;     // Quand appele par ISRFEM
static const int PRODUIT   =1;     // Qaudn appele par Produits
// ...FT-2765



/*====================================================================*/
// Exported types Declaration
/*====================================================================*/
// on mentionne ici toutes les donnees en sortie de 22_SOS hors spectre
typedef struct
    {
    int GSosSsdOSFActor;
    int GSosSsdOSNsFirstSpect;  /* GS1aOSNsfirst ds la chaine PRODUIT */
    int GSosSsdOSNsLastSpect;   /* GS1aOSNslast ds la chaine PRODUIT */
    double GSosSsdDWnOSSpect;   /* GS1aDWnOSSpect ds la chaine PRODUIT */
    } t_SD_BAS_22_SOS_Sortie;

/*====================================================================*/
// External variables Declaration
/*====================================================================*/

/*====================================================================*/
//                  FONCTIONS EXTERNES
//======================================================================

#ifdef __cplusplus
extern "C" {
#endif

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_22_SOS
//@
//@ Description : algorithme de surechantillonnage d'un spectre
//@ utilise dans la chaine ISRFEM et dans la chaine produit
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  ctp : variables temporaires
//@         (ENTREE)  ctx_appel : ISRFEM ou PRODUIT
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (ENTREE)  DataGenGranule : Donnees generales au granule
//@         (Entree)  GSpectInit : spectre decode reel
//@         (Sortie)  InfoGen : info generale issues de l'algo
//@         (Sortie)  GSpectReech : spectre reechantillone
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_22_SOS(
        t_SD_BAS_DataThread * ctx,
        t_SD_BAS_DataThread22SOS *ctp,
        int ctx_appel,
        t_SD_BAS_ConfAutre *  ConfAutre,
        t_SD_BAS_DataGen  * DataGenGranule,
        double * GSpectInit,
        t_SD_BAS_22_SOS_Sortie  *InfoGen, // ( ceci peut  etre detaille)
        double * GSpectReech
            )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_23_SSD
//@
//@ Description : algorithme de determination
//@ des decalage spectraux
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  ctp : variables temporaires
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (ENTREE)  ConfStable : fichier de conf stable
//@         (ENTREE)  InfoGen : info generales issues de l'algo SOS
//@         (ENTREE)  GSosSsdSpect : spectre reechantillone issue de l'algo SOS
//@         (Sortie)  InfoDet : info det du produit ENG
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_23_SSD(
        t_SD_BAS_DataThread * ctx,
        t_SD_BAS_DataThreadImageISRFEM *ctp,
        t_SD_BAS_ConfAutre *  ConfAutre,
        t_SD_BAS_ConfStable *  ConfStable,
        t_SD_BAS_22_SOS_Sortie * InfoGen,
        double * GSosSsdSpect,
        t_SD_BAS_Liste_Var InfoDet
            )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_21_SSS
//@
//@ Description : algorithme de selection des decalages spectraux
//@ des decalage spectraux
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  ctp : variables temporaires
//@         (ENTREE)  t : date UTC de la ligne (SN=15) /* DM 11/03/2004 */
//@         (ENTREE)  t_FE : date UTC de fin de l'eclipse precedente
//@         (ENTREE)  GIrcImg : image calibree
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (ENTREE)  ConfStable : fichier de conf stable
//@         (ENTREE)  NbISPPX : nombre des paquets de N0 PX
//@         (ENTREE)  ListeInfoN0 : Liste des paquets de N0 PX
//@                   pour la ligne ( on choisira ceux de PN
//@                   et de CCD ( SN variant))
//@                   cette info est dans InfoDet1ABC
//@         (ENTREE)  NbPaquetsPX : nb paquets du produit N0
//@         (ENTREE)  InfoDet1ABC : info det du produit 1ABC
//@         (Entree/Sortie)  InfoDetEng : info det du produit ENG
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_21_SSS(
        t_SD_BAS_DataThread * ctx,
        t_SD_BAS_DataThreadImageISRFEM *ctp,
        t_SD_BAS_SCDSTime t,
        t_SD_BAS_SCDSTime t_FE,
        double * GIrcImg,
        t_SD_BAS_ConfAutre *  ConfAutre,
        t_SD_BAS_ConfStable *  ConfStable,
	       t_SD_BAS_PaquetN0** ListePaquetPX, 
       	int NbPaquetsPX, 
        t_SD_BAS_Liste_Var * InfoDetEng
            )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_24_IAX
//@
//@ Description : algorithme de determination de l'axe interferometrique
//@ instantanne
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  ctp : variables temporaires
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (ENTREE)  BanqueSpectrale : pointeur sur le fichier BanqueSpectrale
//@         (ENTREE)  NbISPPX : nombre des paquets de N0 PX
//@         (ENTREE)  ListeInfoN0 : Liste des paquets de N0 PX
//@                 pour la ligne (on choisira ceux de CCD
//@         (Entree/Sortie)  InfoDetEng : info det du produit ENG
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_24_IAX(
        t_SD_BAS_DataThread * ctx,
        t_SD_BAS_DataThreadImageISRFEM *ctp,
        t_SD_BAS_ConfAutre *  ConfAutre,
        t_SD_BAS_BanqueSpectrale * BanqueSpectrale,
       // int NbISPPX,
        // t_SD_BAS_Liste_Var  * ListeInfoN0,
        t_SD_BAS_Liste_Var  * InfoDetEng
            )
;

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_25_FAX
//@
//@ Description : algorithme de filtrage de l'axe interferometrique
//@ remarque importante : dans le cas des modes intermediaires ou
//@ complet on se base ici toujours sur le contexte courant:
//@ si 0-1C alors le contexte a ete enrichi auparavant et contient l'axe
//@         instantanne du granule courant
//@ si 1A-1C alors le contexte a ete lu auparavant et on considere que
//@         les derniers elts de l'historique correspondent
//@         au granule courant
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  ctp : variables temporaires
//@         (ENTREE)  contextecourant : pointeur sur le contexte
//@                     courant configuration (avec historique)
//@         (ENTREE)  nbligne : nombre de ligne du granule traite
//@         (ENTREE)  tF : date de la premiere ligne du granule
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (ENTREE)  ConfStable : fichier de conf stable
//@         (Entree/Sortie)  InfoDetEng : tableau de pointeurs
//@         sur les info det du produit ENG (ordre chrono)
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_25_FAX(
        t_SD_BAS_DataThread * ctx,
        t_SD_BAS_DataThreadImageISRFEM *ctp,
        t_SD_BAS_Contexte * contextecourant,
        int nbligne,
        t_SD_BAS_SCDSTime tF,
        t_SD_BAS_ConfAutre *  ConfAutre,
        // FT-3095
  		t_SD_BAS_ConfStable * ConfStable,
  	    // ... FT-3095
        t_SD_BAS_Liste_Var *  InfoDetEng
            )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_43_ISF
//@
//@ Description : Interpolation des fonction spectrales
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  ctp : variables temporaires
//@         (ENTREE)  BanqueSpectrale : pointeur sur le
//@                 fichier BanqueSpectrale
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (Entree/Sortie)  InfoDetEng : info  det du produit ENG
//@         (Sortie)  InfoDet1ABC : info det du produit 1ABC
//@         (Sortie)  InfoCalcisp : info generales calculees sur un isp
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_43_ISF(
        t_SD_BAS_DataThread * ctx,
        t_SD_BAS_DataThreadProduit *ctp,
        t_SD_BAS_BanqueSpectrale * BanqueSpectrale,
        t_SD_BAS_ConfAutre *  ConfAutre,
        t_SD_BAS_Liste_Var * InfoDetEng,
        t_SD_BAS_Liste_Var * InfoDet1ABC
            )
;


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_TransformIpsf
//@
//@ Description : calcule la valeur du poids de l'IPSF sondeur correspondant
//@ a chaque pixel d'imageur.
//@
//@ Parameters :
//@ (ENTREE) ctx : contexte du traitement
//@ (ENTREE/SORTIE) ctp : structure d'accueil des sorties ImgY, ImgZ et IISWgtIpsf
//@ (ENTREE) NbLin : nb de lignes des discretisation de l'IPSF
//@ (ENTREE) NbCol : nb de colonnes des discretisation de l'IPSF
//@ (ENTREE) Ypsf : coordonnee angulaire nominale en Y
//@ (ENTREE) Zpsf : coordonnee angulaire nominale en Z
//@ (ENTREE) WgtPsf : poids associe aux points de l'IPSF
//@ (ENTREE) LinDeb : domaine de definition de l'imageur IIS
//@ (ENTREE) ColDeb
//@ (ENTREE) LinFin
//@ (ENTREE) ColFin
//@ (ENTREE) IISY : coordonnee angulaire nominale en Y de l'IIS
//@ (ENTREE) IISZ : coordonnee angulaire nominale en Z de l'IIS
//@ (ENTREE) ImgDY : dimensions angulaires d'un pixel
//@ (ENTREE) ImgDZ
//@ (ENTREE) OffsetSondIIS1 : estimation de l'offset sondeur par le modele orbital
//@ (ENTREE) OffsetSondIIS2
//@ (ENTREE) Osf : facteur de surechantillonnage de l'IPSF
//@       
//@ Pour 41_CCS, la fonction est appelee par le module GES qui recopie membre a 
//@ membre 
//@ t_SD_BAS_DataThreadImageISRFEM->ImgY, 
//@ t_SD_BAS_DataThreadImageISRFEM->ImgZ, 
//@ t_SD_BAS_DataThreadImageISRFEM->IISWgtIpsf 
//@ respectivement dans 
//@ t_SD_BAS_DataThreadProduit->ImgY, 
//@ t_SD_BAS_DataThreadProduit->ImgZ, 
//@ t_SD_BAS_DataThreadProduit->IISWgtIpsf
//@
//@ Pour l'appel initial dans la fonction IpsfToAvhrr de 41_CCS :
//@ int                 NbLin = IDefPsfSondNbLin
//@ int                 NbCol = IDefPsfSondNbCol
//@ double              *Ypsf = IDefPsfSondY(PN,NbCol)
//@ double              *Zpsf = IDefPsfSondZ(PN,NbCol)
//@ double              *WgtPsf = IDefPsfSondWgt[nl=1..NbLin,nc=1..NbCol
//@ int                 LinDeb = 1
//@ int                 ColDeb = 1
//@ int                 LinFin = NbLinAvFov
//@   NbLinAvFov = LinAvFovEnd - LinAvFovFirst + 1
//@     LinAvFovFirst = PartieEntiere{ Min{ PsfCornerAvLin }}
//@     LinAvFovEnd = PartieEntiere{ Max{ PsfCornerAvLin }} + 1
//@     Si (OffsetIISAvhrr(2) > 0) alors
//@       LinAvFovEnd = LinAvFovEnd + PartieEntiere{OffsetIISAvhrr(2)} +1
//@     Sinon
//@       LinAvFovFirst = LinAvFovFirst - PartieEntiere{OffsetIISAvhrr(2)} -1
//@     Fin de si
//@       [PsfCornerAvLin, PsfCornerAvCol] = GeomMetopAvhrr{ IasiTime, PsfCornerMetop }
//@         IasiTime = GEPSDatIasi(SN,LN)
//@         PsfCornerMetop = GeomIasiMetop{ MatRot, ScanAngle, IpsfCornerY, IpsfCornerZ }
//@           MatRot = IDefIASIMETOPMatRot
//@           ScanAngle = IDEFIASIScanAngle(SP)
//@           IpsfCornerY[1] = IpsfCornerY[4] = Min{ YPsf } - DYpsf/2 + OffsetSondIIS(1)
//@           IpsfCornerY[2] = IpsfCornerY[3] = Max{ YPsf } + DYpsf/2 + OffsetSondIIS(1)
//@             YPsf = IDefPsfSondY(PN)
//@             DYpsf = YPsf(2) - YPsf(1)
//@             [ FlagDateNOK, OffsetSondIIS(Ipsf,2), OffsetSondAvhrr(Ipsf,2) ] = CalcOffset{ IasiTime, ScanAngle, MatRot, OffsetSondIISModel, t_FE, Period, OffsetIISAvhrr }
//@               OffsetSondIISModel = IDefOffsetSondIISModel(PN)
//@               t_FE = GEPSEndEclipseTime(LN-1)
//@               Period = IDefMETOPOrbitPeriod
//@               OffsetIISAvhrr = GIacOffsetIISAvhrr(SN,LN)
//@           IpsfCornerZ[1] = IpsfCornerZ[2] = Min{ ZPsf } - DZpsf/2 + OffsetSondIIS(2)
//@           IpsfCornerZ[3] = IpsfCornerZ[4] = Max{ ZPsf } + DZpsf/2 + OffsetSondIIS(2)
//@             ZPsf = IDefPsfSondZ(PN)
//@             DZpsf = DZpsf = ZPsf(2) - ZPsf(1)
//@       OffsetIISAvhrr = GIacOffsetIISAvhrr(SN,LN)
//@ int                 ColFin = NbColAvFov
//@   NbColAvFov = ColAvFovEnd - ColAvFovFirst + 1
//@     ColAvFovFirst = PartieEntiere{ Min{ PsfCornerAvCol }} 
//@     ColAvFovEnd = PartieEntiere{ Max{ PsfCornerAvCol }} + 1
//@     Si (OffsetIISAvhrr(1) > 0) alors
//@       ColAvFovEnd = ColAvFovEnd + PartieEntiere{OffsetIISAvhrr(1)} +1
//@     Sinon
//@       ColAvFovFirst = ColAvFovFirst - PartieEntiere{OffsetIISAvhrr(1)} -1
//@     Fin de si
//@ double              *IISY = ImgAvY
//@ double              *IISZ = ImgAvZ
//@   Boucle sur les lignes de l'imagette Avhrr utile( il = 1 a nls )
//@     Boucle sur les colonnes de l'imagette Avhrr utile ( ic = 1 a ncs)
//@       ImgAvY(il,ic) = Intlin2D{ImgAvYSubG,ls,cs,nls,ncs,il,ic}
//@       ImgAvZ(il,ic) = Intlin2D{ImgAvZSubG,ls,cs,nls,ncs,il,ic}
//@     Fin de boucle sur les colonnes Avhrr
//@   Fin de boucle sur les lignes Avhrr
//@     nls = NbLinAvFov
//@     ncs = NbColAvFov
//@     ls = {LinAvFovFirst, LinAvFovFirst+dl, LinAvFovFirst+2*dl, ..., LinAvFovEnd}
//@     cs = {ColAvFovFirst, ColAvFOvFirst+dc, ColAvFovFirst+2*dc,..., ColAvFovEnd}
//@     Boucle sur les lignes ( il = 1 a nls )
//@       Boucle sur les colonnes ( ic = 1 a ncs)
//@         ImgAvLinSubG(il,ic) = IntToReal{ ls(il) }
//@         ImgAvColSubG(il,ic) = IntToReal{ cs(ic) }
//@         ImgAvMetopSubG(il,ic) = GeomAvhrrMetop{ IasiTime, ImgAvLinSubG(il,ic) , ImgAvColSubG(il,ic) }
//@         [ImgAvYSubG(il,ic), ImgAvZSubG(il,ic)] = GeomMetopIasi{ MatRot, ScanAngle, ImgAvMetopSubG(il,ic) }
//@       Fin de boucle sur les colonnes 
//@     Fin de boucle sur les lignes 
//@ double              ImgDY = ImgAvDY
//@ double              ImgDZ = ImgAvDZ
//@   Si abs{ ImgAvZ(1,2) - ImgAvZ(1,1)} < abs{ ImgAvY(1,2) - ImgAvY(1,1)} alors
//@     ImgAvDY = [ ImgAvY(1,NbColAvFov) - ImgAvY(1,1) 
//@       + ImgAvY(NbLinAvFov,NbColAvFov) - ImgAvY(NbLinAvFov,1) ]
//@       / [ 2 NbColAvFov ]
//@     ImgAvDZ = [ ImgAvZ(NbLinAvFov,1) - ImgAvZ(1,1) 
//@       + ImgAvZ(NbLinAvFov,NbColAvFov) - ImgAvZ(1,NbColAvFov) ]
//@       / [ 2 NbLinAvFov ]
//@   Sinon
//@     ImgAvDY = [ ImgAvY(NbLinAvFov,1) - ImgAvY(1,1) 
//@       + ImgAvY(NbLinAvFov,NbColAvFov) - ImgAvY(1,NbColAvFov) ]
//@       / [ 2 NbLinAvFov ]
//@     ImgAvDZ = [ ImgAvZ(1,NbColAvFov) - ImgAvZ(1,1) 
//@       + ImgAvZ(NbLinAvFov,NbColAvFov) - ImgAvZ(NbLinAvFov,1) ]
//@       / [ 2 NbColAvFov ]
//@   Fin de si
//@ double              OffsetSondIIS1 = OffsetSondAvhrr(1)
//@ double              OffsetSondIIS2 = OffsetSondAvhrr(2)
//@ int                 Osf = OverSampFactor
//@   OverSampFactor = IDefPsfSondOverSampFactor
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_TransformIpsf(
    t_SD_BAS_DataThread * ctx,
    t_SD_BAS_DataThreadImageISRFEM *ctp,
    int                 NbLin,
    int                 NbCol,
    double              *Ypsf,
    double              *Zpsf,
    double              *WgtPsf,
    int                 LinDeb,
    int                 ColDeb,
    int                 LinFin,
    int                 ColFin,
    double              *IISY,
    double              *IISZ,
    double              ImgDY,
    double              ImgDZ,
    double              OffsetSondIIS1,
    double              OffsetSondIIS2,
    int                 Osf
);

//======================================================================
#ifdef __cplusplus
}
#endif

#endif
