//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/ALG/src/SD_ALG_Image.c,v 1.3 2011/11/25 10:17:44 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_ALG
//@ MODULE : SD_ALG_Image
//@
//@ Comment :
//@    Code file for the  SD_ALG_Image class which includes the
//@    Image processing chain algorithms
//@
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ 27/11/2002 : Jean-Luc PASCAL
//@ Procedure : SD_ALG_XX_Initprod et SD_ALG_XX_Libere deplacees dans SD_ALG_Spectre.c
//@ 20/03/2003 : Benjamin MASSART
//@ TU OK
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2522(DM-2100-OPS) : 01/12/2008 : S. BRUNEL
//@ Ajout de variables dans les produits L1C et ENG  (Day 2,point 3)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <stdio.h>

#include "SD_ALG_Image.h"

#ifdef INVESTIGATION
#include "SD_BAS_Traces.h"
#endif

/*====================================================================*/
/* Local Constant Declaration                                         */
/*====================================================================*/
#ifdef OPS_DEBUG
static const char REP_TU[] = "SD/ALG/tu/TU_SD_ALG_Image";
#endif

/*====================================================================*/
/* Local types Declaration                                            */
/*====================================================================*/

/*====================================================================*/
/* Internal fonctions Declaration                                     */
/*====================================================================*/

/*====================================================================*/
/* Global variables Definition                                        */
/*====================================================================*/
#define SD_ALG_IMAGE_LG_LIBELLE_TRACES  60
#define INDICE_EQUAL_COUNT_OLD_COURANT    0
#define INDICE_EQUAL_COUNT_OLD_PRECEDENT  1

/*====================================================================*/
/*					FONCTIONS EXTERNES                                        */
/*====================================================================*/

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_38_ICC
//@
//@ Description : calcul des coef de calibration radiometrique
//@ Les coefficients de calibration radiometrique des images sont obtenus a 
//@ partir des images visees corps noir. Les detecteurs et la chaine 
//@ d'acquisitionsont supposes lineaires en energie
//@
//@ Parameters :
//@			(ENTREE)  ctx : contexte de l'etape (numero de ligne traitee LN)
//@			(ENTREE)  conf_autre : fichier de conf autre
//@			(ENTREE)  conf_stable : fichier de conf stable
//@     (ENTREE)  bbt : temperature filtree du corps noir chaud calculee par 30_FTB
//@               pour la ligne LN-1
//@			(ENTREE)  image_cs_35 : image espace froid ( SN=35) (LN-1) 
//@               taille SD_BAS_IMCO * SD_BAS_IMLI
//@			(ENTREE)  image_cs_36 : image espace froid ( SN=36) (LN-1)
//@               taille SD_BAS_IMCO * SD_BAS_IMLI
//@			(ENTREE)  image_cs_32 : image corps noir chaud ( SN=32) (LN-1)
//@               taille SD_BAS_IMCO * SD_BAS_IMLI
//@			(ENTREE)  image_cs_33 : image corps noir chaud ( SN=33) (LN-1)
//@               taille SD_BAS_IMCO * SD_BAS_IMLI
//@			(ENTREE)  BIIS_equal_count : compteur equaliser (LN-1)
//@			(ENTREE)  BIIS_equal_count_old : compteur equaliser (LN-2)
//@     (ENTREE/SORTIE) rad_cal_offset_imag : coefficient de calibration radiometrique
//@                     IIS (filtre)
//@     (ENTREE/SORTIE) rad_cal_slope_imag :  coefficient de calibration radiometrique
//@                     IIS (filtre)
//@			(SORTIE)  contexte_courant : pointeur sur le contexte
//@               courant configuration
//@			(SORTIE)  info_det_ENG : info det du produit ENG de ligne (LN-1)
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_38_ICC (
  t_SD_BAS_DataThread *         ctx, 
  t_SD_BAS_DataThreadImageDeb * data_thread_image, 
  t_SD_BAS_ConfAutre *          conf_autre,
  t_SD_BAS_ConfStable *         conf_stable,
  const double                  bbt, 
  double *                      image_cs_35,
  double *                      image_cs_36,
  double *                      image_bb_32,
  double *                      image_bb_33,
  const int                     BIIS_equal_count,
  const int                     BIIS_equal_count_old,
  double *                      rad_cal_offset_imag, 
  double *                      rad_cal_slope_imag, 
  t_SD_BAS_Contexte *           contexte_courant,
  t_SD_BAS_Liste_Var            info_det_ENG)

{
  /* Variables locales */
  /*-------------------*/
  char              tcProcName_F[] = "SD_ALG_38_ICC";
  char              LibelleTraces[SD_ALG_IMAGE_LG_LIBELLE_TRACES + 1];
  t_CMN__Statut     CodeRetour = CMN__STATUT_OK;
  
  /* Indice du premier echantillon du filtre spectral de l'imageur */
  int               Nsfirst;
  /* Indice du dernier echantillon du filtre spectral de l'imageur */
  int               Nslast;
  /* Nb d'echantillons du filtre spectral imageur */
  int               TailleWn;
  /* Nb de cycles de filtrage des coefficients de calibration radiometrique de */
  /* l'IIS */
  int               NbCycleFilterCalCoef;
  /* Vecteur des nb d'onde des echantillons spectraux */
  double *          Wn;
  /* Radiance du corps noir integree sur tous les echantillons spectraux */
  double            RadBB;
  /* Reponse spectrale de l'imageur IIS */
  double *          ImagFilter;
  /* Indice des echantillons spectraux */
  int               n;
  /* Largeur d'un echantillon du filtre spectral de l'imageur */
  double            DWnFilter;
  /* Coefficient de calibration d'ordre 1 (variable temporaire) */
  /* pour la ligne LN-1 */
  double *          Slope;
  /* Image de la visee de l'espace froid (variable temporaire) */
  /* pour la ligne LN-1 */
  double *          Offset;
  /* Nb de lignes et de colonnes des images IIS */
  int               NbLin;
  int               NbCol;
  int               TailleImages;
  /* Indices */
  int               nl;
  int               nc;
  /* Table des pixels morts */
  int *             IISDeadPix;
  /* Coefficient de ponderation */
  double            w;
  /* Numeros de l'image d'egalisation courant pour un scan */
  int               EqualCount;
  /* Numeros de l'image d'egalisation precedant pour un scan */
  int               EqualCountOld;
  /* Coefficients de calibration radiometrique arrondis a la valeur entiere */
  /* la plus proche */
  /* Variables d'entree issues de la ligne LN-2 et mises a jour */
  /* pour la ligne LN-1 */
  double *          RadCalOffsetImagCtx;
  double *          RadCalSlopeImagCtx;
// jlp
  double *    RadCalOffsetImag;
  double *             RadCalSlopeImag;
  /* Flag de reinitialisation des coefficients de calibration */
  /* pour la ligne LN-1 */
  t_CMN__Booleen    FlagInit;
#ifdef INVESTIGATION
  char              ChaineFlagInit[16]; /* chaine de caracteres associee pour les traces */
#endif
  
#ifdef OPS_DEBUG
  FILE *        Fd;
  char          Fichier[256];
#endif

  /* Initialiser les variables locales */
  /*-----------------------------------*/
  Nsfirst = conf_stable->IDefIISSrfNsfirst;
  Nslast = conf_stable->IDefIISSrfNslast;
  TailleWn = Nslast - Nsfirst + 1;

  NbLin = conf_autre->IDefIISNbLin;
  NbCol = conf_autre->IDefIISNbCol;
  TailleImages = NbLin * NbCol;

  Wn = data_thread_image->Wn;
  
  Slope = data_thread_image->Slope;
  
  Offset = data_thread_image->Offset;
  
  NbCycleFilterCalCoef = conf_autre->IDefIccNbCycle;
  
  DWnFilter = conf_stable->IDefIISSrfDWn;
  ImagFilter = conf_stable->IDefIISSrf;

  IISDeadPix = conf_stable->IDefDptIISDeadPix;
  
  EqualCount = BIIS_equal_count;
  EqualCountOld = BIIS_equal_count_old;
  
  RadBB = 0.0;
   
  /* Valeurs reelles des coeff de calibration enregistrees dans le contexte et */
  /* mises a jour entre chaque ligne */
  RadCalSlopeImagCtx  = contexte_courant->GIccRadCalSlopeImag;
  RadCalOffsetImagCtx = contexte_courant->GIccRadCalOffsetImag;

  /* coeff arrondi a l'entier le plus proche pour la sortie dans le produit N1 */
 // jlp
  RadCalSlopeImag = info_det_ENG.tab_var[ENG_GICCRADCALSLOPEIMAG].val.pr8;
  RadCalOffsetImag = info_det_ENG.tab_var[ENG_GICCRADCALOFFSETIMAG].val.pr8;

/* Corps de l'operation */
/*----------------------*/
#ifdef OPS_DEBUG
  printf ("Debut de %s\n", tcProcName_F);
#endif

#ifdef OPS_DEBUG
  printf (" DWnFilter=%.4lf \t Nsfirst=%d \t Nslast=%d \t TailleWn=%d\n", 
    DWnFilter, Nsfirst, Nslast, TailleWn);
  printf (" NbLin=%d \t NbCol=%d\n", NbLin, NbCol);
  printf (" NbCycleFilterCalCoef=%d\n", NbCycleFilterCalCoef);
  printf (" BBT=%.4lf\n", bbt);
  printf (" EqualCount=%d \t EqualCountOld=%d\n", EqualCount, EqualCountOld);

  sprintf (Fichier, "%s/%s/sorties/%s_rad_cal_offset_imag.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (n = 1 ; n <= TailleImages ; n++)
  {
    fprintf (Fd, "%.4le\n", rad_cal_offset_imag[IND1(n)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

  sprintf (Fichier, "%s/%s/sorties/%s_rad_cal_slope_imag.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (n = 1 ; n <= TailleImages ; n++)
  {
    fprintf (Fd, "%.4le\n", rad_cal_slope_imag[IND1(n)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
#endif

  //@ Si (FlagIISCalibMiss(SN=33)=OUI ou FlagIISCalibMiss(SN=36)=OUI) 
  //@ pas de mise a jour : ceci est pris en compte dans la methode appelante
  //@ SD_GES_ChaineImageDeb::Executer
  //@ Sinon 

  //@ Calcul des coefficients de calibration instantanes
  //@ Calcul de la radiance de Planck pour tous les echantillons du filtre 
  //@ spectral n de Nsfirst a Nslast
  for (n = Nsfirst ; n <= Nslast ; n++)
  {
    //@ Wn(n)=(n-1). DWnFilter
    Wn[n - Nsfirst] = (n - 1) * DWnFilter;
    
    //@ RadBB=Somme(n ImagFilter(n).Planck(Wn(n),BBT(LN-1)))
    RadBB = RadBB + ImagFilter[n - Nsfirst] * SD_BAS_Planck(
      bbt, 
      Wn[n - Nsfirst], 
      conf_autre->IDefPlkPK1, 
      conf_autre->IDefPlkPK2);
  }

#ifdef OPS_DEBUG
  printf (" RadBB=%.4le\n", RadBB);
#endif

  //@ Pour tous les points de l'image (nl, nc)
  for (nl = 1; nl <= NbLin ; nl++)
  {
    for (nc = 1; nc <= NbCol; nc++)
    {
      //@ Offset (nl,nc)=ImageCS_36(nl,nc)
      Offset[IND2(nc, nl, NbCol)] = image_cs_36[IND2(nc, nl, NbCol)];
      
      //@ Si (IISDeadPix(nl,nc)=NON) alors	
      if (IISDeadPix[IND2(nc, nl, NbCol)] == (int) CMN__NON)
      {
        //@ Slope(nl,nc)=RadBB / (ImageBB_33(nl,nc) - ImageCS_36(nl,nc))
        Slope[IND2(nc, nl, NbCol)] = 
          RadBB / (image_bb_33[IND2(nc, nl, NbCol)] - 
                   image_cs_36[IND2(nc, nl, NbCol)]);
      }
      
      //@ Si (IISDeadPix(nl,nc)=OUI) alors	
      else
      {
        //@ Slope(nl,nc)=0
        Slope[IND2(nc, nl, NbCol)] = 0.0;
      }

      //@ Calcul des coefficients de calibration filtres
      //@ w = 1 / NbCycleFilterCalCoef
      w = 1.0 / (double)NbCycleFilterCalCoef;
      
      //@ Si EqualCount=EqualCountOld Alors
      if (EqualCount == EqualCountOld)
      {
        //@ Mise a jour des coefficients filtres
        
        //@ RadCalOffsetImag(nl,nc) = (1-w)* RadCalOffsetImag(nl,nc) + 
        //@                           w*Offset(nl,nc) 
        rad_cal_offset_imag[IND2(nc, nl, NbCol)] = 
          (1 - w) * (rad_cal_offset_imag[IND2(nc, nl, NbCol)]) + 
          w * (Offset[IND2(nc, nl, NbCol)]);
// jlp
        RadCalOffsetImag[IND2(nc, nl, NbCol)] = 
          rad_cal_offset_imag[IND2(nc, nl, NbCol)];

        //@ RadCalSlopeImag(nl,nc) = (1-w)* RadCalSlopeImag(nl,nc) + 
        //@                          w*Slope(nl,nc)
        rad_cal_slope_imag[IND2(nc, nl, NbCol)] = 
          (1 - w) * (rad_cal_slope_imag[IND2(nc, nl, NbCol)]) + 
          w * (Slope[IND2(nc, nl, NbCol)]);
// jlp
        RadCalSlopeImag[IND2(nc, nl, NbCol)] = 
          rad_cal_slope_imag[IND2(nc, nl, NbCol)];

        //@ FlagInit = NON
        FlagInit = CMN__NON;
      } /* fin si EqualCount=EqualCountOld */
      
      //@ Sinon 
      else
      {
        //@ Initialisation des coefficients filtres
        
        //@   RadCalOffsetImag(nl,nc) = Offset(nl,nc) 
        rad_cal_offset_imag[IND2(nc, nl, NbCol)] = 
          Offset[IND2(nc, nl, NbCol)];
// jlp
        RadCalOffsetImag[IND2(nc, nl, NbCol)] = 
          rad_cal_offset_imag[IND2(nc, nl, NbCol)];
        //@   RadCalSlopeImag(nl,nc) = Slope(nl,nc)
        rad_cal_slope_imag[IND2(nc, nl, NbCol)] = 
          Slope[IND2(nc, nl, NbCol)];
// jlp
        RadCalSlopeImag[IND2(nc, nl, NbCol)] = 
          rad_cal_slope_imag[IND2(nc, nl, NbCol)];

        //@   FlagInit = OUI
        FlagInit = CMN__OUI;
      } /* fin si EqualCount != EqualCountOld */
      
      //@ Mise a jour du contexte
      RadCalOffsetImagCtx[IND2(nc, nl, NbCol)] = 
        rad_cal_offset_imag[IND2(nc, nl, NbCol)];
      RadCalSlopeImagCtx[IND2(nc, nl, NbCol)] = 
        rad_cal_slope_imag[IND2(nc, nl, NbCol)];

    } /* fin boucle sur les lignes */
  } /* fin boucle sur les colonnes */
  //@ Fin de la boucle sur les pixels
  
  //@ Enregistrement du produit
  *(info_det_ENG.tab_var[ENG_GICCFLAGINIT].val.pui4) = (unsigned int)FlagInit;
  
  //@ le contexte est mis a jour
  /* on met a jour le compteur d'egalisation */
  contexte_courant->BIISEqualCountOld[INDICE_EQUAL_COUNT_OLD_COURANT] = EqualCount;
  /* les coeff de calibration ont deja ete mis a jour */

  /* on trace ici si besoin toutes les variables en sortie */
#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_IISDeadPix.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (n = 1 ; n <= TailleWn ; n++)
  {
    fprintf (Fd, "%.4le\n", ImagFilter[IND1(n)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

  sprintf (Fichier, "%s/%s/sorties/%s_IISDeadPix.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (n = 1 ; n <= TailleImages ; n++)
  {
    fprintf (Fd, "%.4lf\n", IISDeadPix[IND1(n)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

  sprintf (Fichier, "%s/%s/sorties/%s_image_cs_36.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (n = 1 ; n <= TailleImages ; n++)
  {
    fprintf (Fd, "%.4lf\n", image_cs_36[IND1(n)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

  sprintf (Fichier, "%s/%s/sorties/%s_image_bb_33.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (n = 1 ; n <= TailleImages ; n++)
  {
    fprintf (Fd, "%.4lf\n", image_bb_33[IND1(n)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

  sprintf (Fichier, "%s/%s/sorties/%s_Offset.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (n = 1 ; n <= TailleImages ; n++)
  {
    fprintf (Fd, "%.4le\n", Offset[IND1(n)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

  sprintf (Fichier, "%s/%s/sorties/%s_Slope.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (n = 1 ; n <= TailleImages ; n++)
  {
    fprintf (Fd, "%.4le\n", Slope[IND1(n)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

  sprintf (Fichier, "%s/%s/sorties/%s_RadCalOffsetImagCtx.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (n = 1 ; n <= TailleImages ; n++)
  {
    fprintf (Fd, "%.4le\n", RadCalOffsetImagCtx[IND1(n)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

  sprintf (Fichier, "%s/%s/sorties/%s_RadCalSlopeImagCtx.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (n = 1 ; n <= TailleImages ; n++)
  {
    fprintf (Fd, "%.4le\n", RadCalSlopeImagCtx[IND1(n)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

  sprintf (Fichier, "%s/%s/sorties/%s_RadCalOffsetImag.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (n = 1 ; n <= TailleImages ; n++)
  {
    fprintf (Fd, "%u\n", RadCalOffsetImag[IND1(n)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

  sprintf (Fichier, "%s/%s/sorties/%s_RadCalSlopeImag.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (n = 1 ; n <= TailleImages ; n++)
  {
    fprintf (Fd, "%d\n", RadCalSlopeImag[IND1(n)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
#endif

#ifdef INVESTIGATION
  /* Ecrire les coefficients de calibration radiometrique */
  sprintf (LibelleTraces, "GIccRadCalSlopeImag");
  SD_BAS_TRACE (
    tcProcName_F, 
    LibelleTraces,
    NbLin, 
    NbCol, 
    1, 
    TRACE_REAL8, 
    TRACE_BIN, 
    RadCalSlopeImag, 
    ctx);

  sprintf (LibelleTraces, "GIccRadCalOffsetImag");
  SD_BAS_TRACE (
    tcProcName_F, 
    LibelleTraces, 
    NbLin, 
    NbCol, 
    1, 
    TRACE_REAL8, 
    TRACE_BIN, 
    RadCalOffsetImag, 
    ctx);
  
  if (FlagInit == CMN__OUI)
  {
    strcpy(ChaineFlagInit, "OUI");
  }
  else
  {
    if (FlagInit == CMN__NON)
    {
      strcpy(ChaineFlagInit, "OUI");
    }
    else
    {
      strcpy(ChaineFlagInit, "NON DEFINI");
    }
  }

  sprintf (LibelleTraces, "GIccFlagInit");
  SD_BAS_TRACE(
    tcProcName_F, 
    LibelleTraces, 
    1, 
    strlen (ChaineFlagInit), 
    1,
    TRACE_STRING, 
    TRACE_ASC, 
    ChaineFlagInit, 
    ctx);
#endif /* fin mode INVESTIGATION */

/* sortie de la fonction */
fin :
#ifdef OPS_DEBUG
  printf ("Fin de %s\n", tcProcName_F);
#endif
return(CodeRetour); 
} /* fin de SD_ALG_38_ICC */


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_39_IRC
//@
//@ Description : calibration radiometrique des images de niveau 0
//@
//@
//@ Parameters :
//@			(ENTREE)          ctx : contexte de l'etape 
//@                       (numero de ligne traitee LN et SN)
//@			(ENTREE)          conf_autre : fichier de conf autre
//@			(ENTREE)          conf_stable : fichier de conf stable
//@                       de calibration radiometriques filtres (38_ICC)
//@                       dimension SD_BAS_IMLI * SD_BAS_IMCO
//@			l'appelant est issue de SD_DON_GestDonnees::EncodeDecodeLigneS1
//@     (ENTREE)          image : image decodee a calibrer
//@                       taille = SD_BAS_IMCO * SD_BAS_IMLI
//@     (ENTREE)          rad_cal_offset_imag : sortie de 38_ICC
//@     (ENTREE)          rad_cal_slope_imag : sortie de 38_ICC
//@			(SORTIE)          image_calibree : image calibree (LN)
//@                       taille = SD_BAS_IMCO * SD_BAS_IMLI
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_39_IRC(
  t_SD_BAS_DataThread * ctx, 
  t_SD_BAS_ConfAutre *  conf_autre,
  t_SD_BAS_ConfStable * conf_stable,
  double *              image, 
  double *              rad_cal_offset_imag, 
  double *              rad_cal_slope_imag, 
  t_SD_BAS_Liste_Var *InfoDet1ENG,
  double *              image_calibree)

{
  /* Variables locales */
  /*-------------------*/
  char tcProcName_F[] = "SD_ALG_39_IRC";
  char              LibelleTraces[SD_ALG_IMAGE_LG_LIBELLE_TRACES + 1];
  t_CMN__Statut     CodeRetour = CMN__STATUT_OK;
  
  /* Nb de lignes et de colonnes des images IIS */
  int               NbLin;
  int               NbCol;
  int               TailleImage;
  /* Indices */
  int               nl, nc;
  /* Table des pixels morts */
  int *             IISDeadPix;

#ifdef OPS_DEBUG
  FILE *        Fd;
  char          Fichier[256];
  int           n;
#endif
  //jlp calcul variance
  double VarImagIIS,Xbar;
  int nbpts,SN;
  VarImagIIS = 0.0;
  Xbar=0.0;
  nbpts=0;
  SN              = ctx->ctxtrt.SN;

  /* Initialiser les variables locales */
  /*-----------------------------------*/
  NbLin = conf_autre->IDefIISNbLin;
  NbCol = conf_autre->IDefIISNbCol;
  TailleImage = NbCol * NbLin;
  IISDeadPix = conf_stable->IDefDptIISDeadPix;
  
/* Corps de l'operation */
/*----------------------*/
#ifdef OPS_DEBUG
  printf ("Debut de %s\n", tcProcName_F);
#endif

#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_image_decodee.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (n = 1 ; n <= TailleImage ; n++)
  {
    fprintf (Fd, "%.4lf\n", image[IND1(n)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
#endif
  
  //@ on commence ici  par decoder le train de bit DataDecode
  //@ cf DA22 + FD
  //@ NON : le decodage est fait par l'appelant => data_decode est l'image 
  //@ a calibrer
  
  //@ Pour tous les points de l'image (nl,nc) de (1,1) a (NbLin,NbCol)
  for (nl = 1; nl <= NbLin ; nl++)
  {
    for (nc = 1; nc <= NbCol; nc++)
    {
      //@ Si IISDeadPix(nl,nc)=non
      if (IISDeadPix[IND2(nc, nl, NbCol)] == (int) CMN__NON)
      {
        //@ Image(nl,nc)=RadCalSlopeImag(nl,nc).[Image(nl,nc)-RadCalOffsetImag(nl,nc)]
        image_calibree[IND2(nc, nl, NbCol)] = 
          rad_cal_slope_imag[IND2(nc, nl, NbCol)] * 
          (image[IND2(nc, nl, NbCol)] - rad_cal_offset_imag[IND2(nc, nl, NbCol)]);
//jlp calcul varimage iis dans IRC
          Xbar  += image_calibree[IND2(nc, nl, NbCol)];
	  nbpts++;
      }

      //@ Sinon
      else
      {
        //@ Image(nl,nc)=0 
        image_calibree[IND2(nc, nl, NbCol)] = 0.0;
      } /* fin si IISDeadPix(nl,nc)=non */
    } /* fin boucle sur les colonnes de l'image */
  } /* fin boucle sur les lignes de l'image */

//jlp calcul varimage iis dans IRC
    if (nbpts != 0)
      {
      Xbar /= (double)(nbpts);  
      }
   for (nl = 1; nl <= NbLin ; nl++)
     {
     for (nc = 1; nc <= NbCol; nc++)
       {
         if (IISDeadPix[IND2(nc, nl, NbCol)] == (int) CMN__NON)
 	  { 
          VarImagIIS  += (image_calibree[IND2(nc, nl, NbCol)] - Xbar)*(image_calibree[IND2(nc, nl, NbCol)] - Xbar);
	  }
      }
    }
    if (nbpts != 0)
     {
      VarImagIIS = sqrt(VarImagIIS/(double)(nbpts));
      }
    InfoDet1ENG->tab_var[ENG_GIACVARIMAGIIS].val.pr8[IND1(SN)]                       = VarImagIIS;
    // FT-2522
    InfoDet1ENG->tab_var[ENG_GIACAVGIMAGIIS].val.pr8[IND1(SN)] = Xbar;
    // ..FT-2522
#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_image_calibree.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (n = 1 ; n <= TailleImage ; n++)
  {
    fprintf (Fd, "%.4lf\n", image_calibree[IND1(n)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
#endif

  /* on trace ici si besoin toutes les variables en sortie */
#ifdef INVESTIGATION
  /* Ecrire l'image calibree radiometriquement */
  sprintf (LibelleTraces, "GIrcImage");
  SD_BAS_TRACE (
    tcProcName_F, 
    LibelleTraces,
    NbLin, 
    NbCol, 
    1, 
    TRACE_REAL8, 
    TRACE_BIN, 
    image_calibree, 
    ctx);
#endif /* fin du mode INVESTIGATION */


fin :
/* sortie de la fonction */
#ifdef OPS_DEBUG
  printf ("Fin de %s\n", tcProcName_F);
#endif
return(CodeRetour); 

}

//======================================================================
//					FONCTIONS INTERNES  
//======================================================================


