//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/ALG/src/SD_ALG_Image.h,v 1.3 2011/11/25 10:17:44 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_ALG
//@ MODULE : SD_ALG_Image
//@
//@ Comment :
//@    include file for the SD_ALG_Image class which includes the
//@    Image processing chain algorithms
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ 27/11/2002 : Jean-Luc PASCAL
//@ Procedure : SD_ALG_XX_Initprod deplacee dans SD_ALG_Spectre.h
//@ 20/03/2003 : Benjamin MASSART
//@ TU OK
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_ALG_Image_h
#define SD_ALG_Image_h


// inserer ici les fichiers include 
#include <stdio.h>       /* entree/sortie */
#include <stdlib.h>      /* alloc memoire, ...*/
#include <string.h>      /* chaines de carateres */
#include <unistd.h>      /* gestion pipes, ... */
#include <time.h>        /* gestion de la date */
#include <sys/param.h>   /* constantes systeme */

#include "CMN__Commun.h"
#include "SD_BAS_Global.h"
#include "SD_BAS_Niveau1.h"
#include "SD_BAS_Niveau0.h"
#include "SD_BAS_Contexte.h"
#include "SD_BAS_ConfAutre.h"
#include "SD_BAS_ConfStable.h"
#include "SD_BAS_OperateurBase.h"

/*====================================================================*/
/*   External define  Declarations                                    */
/*====================================================================*/
#define ERR_BASE_IMAGE                          SD_BAS_ERR_BASE_IMAGE
#define SD_BAS_ERR_SET_IMAGE                    1

/*====================================================================*/
/* Constant Declaration                                               */
/*====================================================================*/


/*====================================================================*/
/* Exported types Declaration                                         */
/*====================================================================*/
/* Liste des codes d'erreur retournes par les algo du module */

/*====================================================================*/
/* External variables Declaration                                     */
/*====================================================================*/

#ifdef __cplusplus
extern "C" {
#endif
/*====================================================================*/
/*					FONCTIONS EXTERNES                                        */
/*====================================================================*/

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_38_ICC
//@
//@ Description : calcul des coef de calibration radiometrique
//@ Les coefficients de calibration radiometrique des images sont obtenus a 
//@ partir des images visees corps noir. Les detecteurs et la chaine 
//@ d'acquisitionsont supposes lineaires en energie
//@
//@ Parameters :
//@			(ENTREE)  ctx : contexte de l'etape (numero de ligne traitee LN)
//@			(ENTREE)  conf_autre : fichier de conf autre
//@			(ENTREE)  conf_stable : fichier de conf stable
//@     (ENTREE)  bbt : temperature filtree du corps noir chaud calculee par 30_FTB
//@               pour la ligne LN-1
//@			(ENTREE)  image_cs_35 : image espace froid ( SN=35) (LN-1) 
//@               taille SD_BAS_IMCO * SD_BAS_IMLI
//@			(ENTREE)  image_cs_36 : image espace froid ( SN=36) (LN-1)
//@               taille SD_BAS_IMCO * SD_BAS_IMLI
//@			(ENTREE)  image_cs_32 : image corps noir chaud ( SN=32) (LN-1)
//@               taille SD_BAS_IMCO * SD_BAS_IMLI
//@			(ENTREE)  image_cs_33 : image corps noir chaud ( SN=33) (LN-1)
//@               taille SD_BAS_IMCO * SD_BAS_IMLI
//@			(ENTREE)  BIIS_equal_count : compteur equaliser (LN-1)
//@			(ENTREE)  BIIS_equal_count_old : compteur equaliser (LN-2)
//@     (ENTREE/SORTIE) rad_cal_offset_imag : coefficient de calibration radiometrique
//@                     IIS (filtre)
//@     (ENTREE/SORTIE) rad_cal_slope_imag :  coefficient de calibration radiometrique
//@                     IIS (filtre)
//@			(SORTIE)  contexte_courant : pointeur sur le contexte
//@               courant configuration
//@			(SORTIE)  info_det_ENG : info det du produit ENG de ligne (LN-1)
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_38_ICC (
  t_SD_BAS_DataThread *         ctx, 
  t_SD_BAS_DataThreadImageDeb * data_thread_image, 
  t_SD_BAS_ConfAutre *          conf_autre,
  t_SD_BAS_ConfStable *         conf_stable,
  const double                  bbt, 
  double *                      image_cs_35,
  double *                      image_cs_36,
  double *                      image_bb_32,
  double *                      image_bb_33,
  const int                     BIIS_equal_count,
  const int                     BIIS_equal_count_old,
  double *                      rad_cal_offset_imag, 
  double *                      rad_cal_slope_imag, 
  t_SD_BAS_Contexte *           contexte_courant,
  t_SD_BAS_Liste_Var            info_det_ENG)
;


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_39_IRC
//@
//@ Description : calibration radiometrique des images de niveau 0
//@
//@
//@ Parameters :
//@			(ENTREE)          ctx : contexte de l'etape 
//@                       (numero de ligne traitee LN et SN)
//@			(ENTREE)          conf_autre : fichier de conf autre
//@			(ENTREE)          conf_stable : fichier de conf stable
//@                       de calibration radiometriques filtres (38_ICC)
//@                       dimension SD_BAS_IMLI * SD_BAS_IMCO
//@			l'appelant est issue de SD_DON_GestDonnees::EncodeDecodeLigneS1
//@     (ENTREE)          image : image decodee a calibrer
//@                       taille = SD_BAS_IMCO * SD_BAS_IMLI
//@     (ENTREE)          rad_cal_offset_imag : sortie de 38_ICC
//@     (ENTREE)          rad_cal_slope_imag : sortie de 38_ICC
//@			(SORTIE)          image_calibree : image calibree (LN)
//@                       taille = SD_BAS_IMCO * SD_BAS_IMLI
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_39_IRC(
  t_SD_BAS_DataThread * ctx, 
  t_SD_BAS_ConfAutre *  conf_autre,
  t_SD_BAS_ConfStable * conf_stable,
  double *              image, 
  double *              rad_cal_offset_imag, 
  double *              rad_cal_slope_imag, 
  t_SD_BAS_Liste_Var *InfoDet1ENG,
  double *              image_calibree)
;


//======================================================================
#ifdef __cplusplus
}
#endif


#endif
