//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/ALG/src/SD_ALG_Monitoring.c,v 1.3 2011/11/25 10:17:44 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_ALG
//@ MODULE : SD_ALG_Monitoring
//@
//@ Comment :
//@    Code file for the  SD_ALG_Monitoring class which includes the
//@    Monitoring algorithm for IASI L1 chain
//@
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 04/03/2003 : Pierre DHERETE
//@ Versions 100_EXS, 110_DPT, 111_MCX stables
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-060-CN : B. MASSART
//@ SD_ALG_111_MCX : modifications pour que les flags ENG_MMCXFLAGNOISECALRAD
//@ et ENG_MMCXFLAGBIASCALRAD soient leves dans le produit engineering si 
//@ FlagNoiseCalRad et FlagBiasCalRad valent au moins une fois VRAI dans 
//@ la boucle sur les plages spectrales quelle que soit la bande spectrale 
//@ consideree.
//@ --------------------------------------------------------------------
//@ DM-ID :IA-CR-0000-177-CNE : 31/03/2004 : B. MASSART
//@ SD_ALG_45_QIS:
//@ -suppression de FlagQualProduit
//@ -changement dimensions de GOPSFltIsrfemOff
//@ --------------------------------------------------------------------
//@ DM-ID :IA-CR-0000-234-CNE : 31/03/2004 : B. MASSART
//@ SD_ALG_45_QIS:
//@ -changement tests logiques pour ENG_GQISFLAGQUAL
//@ --------------------------------------------------------------------
//@ DM-ID : PM4 : 31/03/2004 : B. MASSART
//@ SD_ALG_45_QIS:
//@ -prise en compte de SP(SN)
//@ -date IasiTime a SN=15
//@ -correction syntaxe ENG_GOPSFLTISRFEMOFF
//@ -correction syntaxe N1C_GQISQUALINDEXIIS
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-132-CN : 29/07/2004 : P.CABANE
//@ AxeRes used but not initialized in SD_ALG_45_QIS
//@ fiche de correction correspondante : IA-2100-FC-132-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-143-CN : 21/09/2004 : B.MASSART
//@ L'OPS s'arrete sur l'initialisation de VIADRENG_MEXSSMIN avec le contexte 
//@ a froid dans SD_GES_ChaineProd0_1C::Corpstraitement lorsque la 10ieme ligne 
//@ est un DMDR. On opte pour une gestion des VIADR_DUMMY similaire a celle des 
//@ VIADR_ENG valides. Voir IA-2100-FC-143-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-156-CN : 30/05/2005 : B MASSART
//@ GQualIndexSpect=2, au lieu de 1 en mode calibration externe ne ciblant 
//@ pas la Terre. L'indice SP calcule dans 45_QIS valait toujours 0 au moment 
//@ de calculer GqualIndexSpect puisque la derniere valeur de SN dans la boucle 
//@ correspond au mouvement du miroir de scan (SN=30). On rajoute la variable 
//@ SPmem qui memorise la valeur de SP si SN est dans l'intervalle [3;29].
//@ Fiche de correction correspondante : IA-2100-FC-156-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-157-CN : 30/05/2005 : B MASSART
//@ L'analyse du code de 45_QIS montre que GQisQualIndexSpect n'est rempli que 
//@ si GQisFlagQual est bon. Ce test n'est pas specifie dans le dossier de 
//@ definition des algos. Le test est supprime.
//@ Fiche de correction correspondante : IA-2100-FC-157-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-158-CN : 30/05/2005 : JL PASCAL
//@ L'OPS genere des resulta VIADRENG_MEXSSMIN et MAX errone et qui  
//@ dependent du nombre de thread 
//@ Voir IA-2100-FC-158-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-271-CN : 22/06/2005 : JL.PASCAL
//@ prise en compte du mode transition
//@ fiche de correction correspondante : IA-2100-CR-271-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-169-CN : 27/02/2006 : B.MASSART
//@ Syntax correction for IisInHomPix. See IA-2100-FC-169-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-182-CN : 30/10/2006 : R.MORENO
//@ Pb de geolocalisation non traces dans le CG + levee de flags
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-187-CN : 21/11/2006 : R.MORENO
//@ Calcul du MEXSSMIN
//@ --------------------------------------------------------------------
//@ DM-ID : IA-2100-CR-332-CNE : 07/03/2007 : S. BRUNEL
//@ Inversion de la fraction PixQual
//@ --------------------------------------------------------------------
//@ DM-ID : IA-2100-CR-335-CNE : 07/03/2007 : S. BRUNEL
//@ Modification du flag general. Comparer la valeur absolue
//@ de BiasCalRad a BiasCalRadCutoff
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-210-CN : 13/03/2007 : S. BRUNEL
//@ Regression des FA156 et FA157, on remet le test sur GQISFLAGQUAL 
//@ pour le calcul des moyennes quadratiques, par contre le champ 
//@ GQISQUALINDEXSPECT ne depend pas du flag.
//@ --------------------------------------------------------------------
//@ DM-ID : IA-2100-CR-348-CNE : 01/06/2007 : S. BRUNEL
//@ 111_MCX : MMcxCoeffCalRad passe en NeDT a 280 K
//@ 45_QIS : Modification du calcul des indices de qualite 
//@ GQisQualIndex, GQISQualIndexRad et GQISQualIndexSpec
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-1767-CN (rev 0): 29/06/2007 : S. BRUNEL
//@ 45_QIS : Leve le flag ENG_GQISFLAGQUAL si un des flags GQISFLAGQUAL
//@ des produits L1A, L1B, L1C est leve (cf CR 349).
//@ --------------------------------------------------------------------
//@ FA-ID :IA-FT-2375 : 11/09/2007 : S. BRUNEL
//@ ELSE ... manquant dans 45_QIS
//@ --------------------------------------------------------------------
//@ DM-ID :IA-FT-2373 : 11/09/2007 : S. BRUNEL
//@ Modification du calcul du GQisQualIndexLoc moyen 
//@ pour L1C dans 45_QIS
//@ --------------------------------------------------------------------
//@ FA-ID :IA-FT-1767 (rev 1): 11/09/2007 : S. BRUNEL
//@ ENG_GQISFLAGQUAL non leve en cas d'overflow (MODIFICATION DE L'ACTION)
//@ --------------------------------------------------------------------
//@ DM-ID :IA-2100-FT-2447 : 21/04/2008 : S. BRUNEL
//@ Nouvelle modification du calcul du GQisQualIndexLoc moyen pour L1C 
//@ dans 45_QIS
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FT-2550 : 09/12/2008 : S. BRUNEL
//@ FA SGC CNES : incidents industriels 
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FT-2551 : 10/12/2008 : S. BRUNEL
//@ Regression observee V4-1 (probleme de mise en conf) 
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2525(DM-2100-OPS) : 19/12/2008 : S. BRUNEL
//@ Modification du GQisFlagQual, ajout d'informations sur la cause de
//@ la rejection (Day 2,point 6)
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2557(FA-2100-OPS) : 02/02/2009 : S. BRUNEL
//@ Non conformite des produits generes par la V4-2-1 a la PFS Day-2
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2523(DM-2100-OPS) : 06/02/2008 : S. BRUNEL
//@ Include BArcImagMeanRMS dans le produit ENG (Day 2,point 4)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2526(DM-2100-OPS) : 23/02/2009 : S. BRUNEL
//@ Include les differences de NZPD interpixel dans le produit 
//@ ENG (Day 2,point 7)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2523(DM-2100-OPS) : 06/03/2009 : S. BRUNEL
//@ Include BArcImagMeanRMS dans le produit ENG (Day 2,point 4)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2525(DM-2100-OPS) : 06/03/2009 : S. BRUNEL
//@ Modification du GQisFlagQual, ajout d'informations sur la cause de
//@ la rejection (Day 2,point 6)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2526(DM-2100-OPS) : 06/03/2009 : S. BRUNEL
//@ Include les differences de NZPD interpixel dans le produit 
//@ ENG (Day 2,point 7)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2527(DM-2100-OPS) : 01/12/2008 : S. BRUNEL
//@ Permettre le calcul des spectres dans les bandes B1 & B2 meme
//@ lorsque un spike affecte la bande B3 (Day 2,point 8)
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2566(FA-2100-OPS) : 26/03/2009 : S. BRUNEL
//@ Probleme de recopie de GQisFlagQualDetailed dans le produit L1C
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2574(DM-2100-OPS) : 16/04/2009 : S. BRUNEL
//@ Modification de la valeur d'initialisation des bits spare de 
//@ GQisFlagQualDetailed (1=>0)
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2596(FA-2100-OPS) : 17/04/2009 : S. BRUNEL
//@ 45_QIS : GOPSFlagSpikNZpdBzpd est faux au retour de sub_45_QIS 
//@ si la bande de determination du NZPD n'est pas la bande 3
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2638(FA-2100-OPS) : 17/06/2009 : S. BRUNEL
//@ Valeurs erronees de MHipNZpdInterPixel pour les SN 32,33,35 et 36
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2573(DM-2100-OPS) : 16/04/2009 : S. BRUNEL
//@ Modification de la valeur d'initialisation de la variable
//@ MHipNZpdInterpixel (0=>valeur minimum codable)
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2648(FA-2100-OPS) : 30/06/2009 : S. BRUNEL
//@ MHipFlagInterPixNZpdNonQual n'est pas PN dependant
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2749(DM-2100-OPS) : 22/03/2010 : S. BRUNEL
//@ Mise en coherence de 45_QIS vis a vis des flags de qualite FTB
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2771(FA-2100-OPS) : 18/05/2010 : S. BRUNEL
//@ Trace GOPSFltBBTMiss non supprimee dans SD_ALG_Monitoring.cc
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-3296(FA-2100-OPS) : 18/05/2017 : MLH
//@ Modification du controle GQisFlagDetailed_Hardware en cas de PX
//@ manquant, leve uniquement si CSQ, SQ1 ou SQ2 leve
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------


#include <stdio.h>
// FT-2765
#include <limits.h>
#include "SD_BAS_SwapEndian.h"
// ...FT-2765

#include "SD_ALG_Monitoring.h"
#include "SD_BAS_Conversion.h"

#ifdef INVESTIGATION
#include "SD_BAS_Traces.h"
#endif

//# declaration des fonctions importees

/*====================================================================*/
// Local Constant Declaration
/*====================================================================*/
static char *SD_ALG_MSG_ERR_NFIRST_NLAST="First index > Last index";
static char *SD_ALG_MSG_ERR_NFIRST_NEG="First index < 0";
static char *SD_ALG_MSG_ERR_NB_ECH_SPECT="Number of spectral samples < 1";
// CR 332
static char *SD_ALG_MSG_ERR_CONF="Taille image nulle";
// FIN CR 332

#ifdef OPS_DEBUG
static const char REP_TU[] = "SD/ALG/tu/TU_SD_ALG_Monitoring";
#endif

/*====================================================================*/
// Local types Declaration
/*====================================================================*/
//======================================================================
// Internal fonctions Declaration
//======================================================================

//======================================================================
// Global variables Definition
//======================================================================



//======================================================================
//                  FONCTIONS EXTERNES
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_100_EXS
//@
//@ Description : Mise a jour des temperatures de brillance min et max
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape numero de ligne sn pn traitee
//@         (ENTREE)  contexte_courant : contexte des granules
//@         (ENTREE)  conf_autre : fichier de conf autre
//@         (ENTREE/SORTIE) data_thread_produit : pour les tableaux temporaires
//@         (ENTREE)  produit_techno : produit ENG dans lequel
//@                   se trouvent les spectres min et max 
//@                   sousechantillonnes d'un facteur 16 (550 echantillons)
//@         (ENTREE)  spectre : spectre decode pour un ISP
//@                   taille = conf_autre->IDefNslast[SD_BAS_B3] - 
//@                            conf_autre->IDefNsfirst[SD_BAS_B1] + 1
//@         (SORTIE)  spectre_froid : spectre froid mis a jour
//@                   taille = SD_BAS_SPCT_COD
//@         (SORTIE)  spectre_chaud : spectre froid mis a jour
//@                   taille = SD_BAS_SPCT_COD
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_100_EXS(
  t_SD_BAS_DataThread *        ctx,
  t_SD_BAS_Contexte *          contexte_courant,
  t_SD_BAS_ConfAutre *         conf_autre,
  t_SD_BAS_DataThreadProduit * data_thread_produit, 
//  t_SD_BAS_Liste_Var           produit_techno, 
  t_SD_BAS_LigneN1 *   		LigneVIADRENG,
  double *                     spectre) 
//  unsigned int *               spectre_froid,
//  unsigned int *               spectre_chaud)

{
  /* Variables locales */
  /*-------------------*/
  static char tcProcName_F[] = "SD_ALG_100_EXS";
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  char*     msg_err;          // Message d'erreur

  /* Largeur d'un echantillon spectral */
  double    DWnSpect;
  /* Vecteur des nb d'ondes */
  double    Wn;
  /* Indice du premier echantillon du spectre complet */
  int       Nsfirst;
  /* Indice du dernier echantillon du spectre complet */
  int       Nslast;
  /* Nb d'echantillons du spectre complet */
  int       TailleSpectre;
  /* Nb d'echantillons des spectres chaud et froid */
  int       TailleSpectresExtremes;
  /* Nombre d'echantillons par plage spectrale des spectres chaud et froid */
  double    NbNsSeg;
  /* Contantes de la loi de Planck */
  double    K1;
  double    K2;
  /* Temperature de brillance calculee a partir du spectre complet */
  double    TemperatureBrillance;
  /* Compteurs */
  int       n, nseg;
//FA158
  int lnviadr;
#ifdef OPS_DEBUG
  FILE *            Fd;
  char              Fichier[256];
  int Sousech,m;
#endif
// FA 187
    int numDerGran;
    int numGranCour;


    numDerGran = (contexte_courant->GCTXNumDerLig[0]-1)/PERIODE_VIADR;
    numGranCour = (contexte_courant->GCTXNumDerLig[1] + ctx->ctxtrt.LN -1)/PERIODE_VIADR;
// ..FA 187

  /* Initialiser les variables locales */
  /*-----------------------------------*/
  DWnSpect = conf_autre->IDefSpectrDWn;
  Nsfirst = conf_autre->IDefNsfirst[SD_BAS_B1];
  Nslast = conf_autre->IDefNslast[SD_BAS_B3];
  TailleSpectre = Nslast - Nsfirst + 1;
  TailleSpectresExtremes = SD_BAS_SPCT_COD;
  NbNsSeg = SD_BAS_UDR_SPLG_COD;
  K1 = conf_autre->IDefPlkPK1;
  K2 = conf_autre->IDefPlkPK2;
  //FA158
  lnviadr =1 +((contexte_courant->GCTXNumDerLig[1] +ctx->ctxtrt.LN-1)%PERIODE_VIADR);
  /* Corps de l'operation */
  /*----------------------*/
#ifdef OPS_DEBUG
  printf ("Debut de %s\n", tcProcName_F);
#endif
  
#ifdef OPS_DEBUG
  printf (" DWnSpect=%.4le\tNsfirst=%d\tNslast=%d\tTailleSpectre=%d\n", 
    DWnSpect, Nsfirst, Nslast, TailleSpectre);
  printf (" TailleSpectresExtremes=%d\tSousech=%.1lf\n", 
    TailleSpectresExtremes, Sousech);
#endif
  
  //@ Mise a jour des spectres chaud et froid
  for (n = Nsfirst ; n <= Nslast ; n++)
   {
    //@ Indice de plage spectrale
    //@ nseg = floor ((n - Nsfirst) / NbNsSeg
    nseg = floor ((n - Nsfirst) / NbNsSeg) + 1;
    
    //@ Nombre d'onde de l'echantilon spectral
    //@ Wn = (n-1)*DWnSpect
    Wn = (n - 1) * DWnSpect;
    
    //@ appel SD_BAS_PlanckInverse
    //@ TB = Planck-1 (cf DA7)
    //jlp DM ALGO
    if ( spectre[n - Nsfirst] > SD_BAS_Planck(100.0,Wn,K1,K2))
    {
      TemperatureBrillance = SD_BAS_PlanckInv (
        spectre[n - Nsfirst], Wn, K1, K2);
    }
    else
    {
      TemperatureBrillance = 100.0;
    }
    
    //@ TB = Round(TB-100)
    // 5.0 PM 3
    TemperatureBrillance = TemperatureBrillance - 100.0;
    
    /* FA143 */
// FA 187
    if (   // cas ou la  ligne appartient au dernier VIADR qui est  ecrit
           ( (numGranCour == numDerGran) && (contexte_courant->GCTXNumDerLig[0]%PERIODE_VIADR == 0)  )
	   // cas ou la  ligne n'appartient pas au dernier VIADR 
        || (numGranCour != numDerGran) )
//..FA 187
    /* FIN FA143 */
     {
  // dans le cas ou on est en cours granule, on travaille avec le VIADR
//FA158 mise a jour d'une variable par ligne
     LigneVIADRENG->InfoDet.tab_var[VIADRENG_MEXSSMIN].val.pui4[IND2(nseg,lnviadr,SD_BAS_SPCT_COD)] =  
     MIN(LigneVIADRENG->InfoDet.tab_var[VIADRENG_MEXSSMIN].val.pui4[IND2(nseg,lnviadr,SD_BAS_SPCT_COD)],floor(TemperatureBrillance));   
//     LigneVIADRENG->InfoDet.tab_var[VIADRENG_MEXSSMIN].val.pui4[IND1(nseg)] =  MIN(LigneVIADRENG->InfoDet.tab_var[VIADRENG_MEXSSMIN].val.pui4[IND1(nseg)],floor(TemperatureBrillance));   
//FA158 mise a jour d'une variable par ligne
     LigneVIADRENG->InfoDet.tab_var[VIADRENG_MEXSSMAX].val.pui4[IND2(nseg,lnviadr,SD_BAS_SPCT_COD)] =  
     MAX(LigneVIADRENG->InfoDet.tab_var[VIADRENG_MEXSSMAX].val.pui4[IND2(nseg,lnviadr,SD_BAS_SPCT_COD)],ceil(TemperatureBrillance));   
//     LigneVIADRENG->InfoDet.tab_var[VIADRENG_MEXSSMAX].val.pui4[IND1(nseg)] =  MAX(LigneVIADRENG->InfoDet.tab_var[VIADRENG_MEXSSMAX].val.pui4[IND1(nseg)],ceil(TemperatureBrillance));   
     }
    else
     {
  // dans le cas ou on est en fin de granule, on travaille avec le contexte
//FA158 mise a jour d'une variable par ligne
     contexte_courant->MExsSmin[IND2(nseg,lnviadr,SD_BAS_SPCT_COD)] = MIN(contexte_courant->MExsSmin[IND2(nseg,lnviadr,SD_BAS_SPCT_COD)],floor(TemperatureBrillance));
//     contexte_courant->MExsSmin[IND1(nseg)] = MIN(contexte_courant->MExsSmin[IND1(nseg)],floor(TemperatureBrillance));
//FA158 mise a jour d'une variable par ligne
     contexte_courant->MExsSmax[IND2(nseg,lnviadr,SD_BAS_SPCT_COD)] = MAX(contexte_courant->MExsSmax[IND2(nseg,lnviadr,SD_BAS_SPCT_COD)],ceil(TemperatureBrillance));
//     contexte_courant->MExsSmax[IND1(nseg)] = MAX(contexte_courant->MExsSmax[IND1(nseg)],ceil(TemperatureBrillance));
     }
  }
  //@ le VIADR est mis a jour toutes les 10 lignes par l'appelant en utilisant
  //@ les donnees du contexte
  
  // ================================================================================
  // on trace ici si besoin toutes les variables en sortie
  // ================================================================================
#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_spectre.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (n = 1 ; n <= TailleSpectre ; n++)
  {
    fprintf (Fd, "%.4le\n", spectre[IND1(n)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

  sprintf (Fichier, "%s/%s/sorties/%s_SpectreFroid.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (m = 1 ; m <= TailleSpectresExtremes ; m++)
  {
    fprintf (Fd, "%.4le\n", contexte_courant->MExsSmin[IND1(m)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
  
  sprintf (Fichier, "%s/%s/sorties/%s_SpectreChaud.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (m = 1 ; m <= TailleSpectresExtremes ; m++)
  {
    fprintf (Fd, "%.4le\n", contexte_courant->MExsSmax[IND1(m)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

#endif

#ifdef INVESTIGATION
  // SpectreFroid
  SD_BAS_TRACE (
    tcProcName_F, 
    "MExsSmin", 
    1, 
    TailleSpectresExtremes, 
    1, 
    TRACE_REAL8, 
    TRACE_BIN, 
    contexte_courant->MExsSmin, 
    ctx);

  // SpectreChaud
  SD_BAS_TRACE (
    tcProcName_F, 
    "MExsSmax", 
    1, 
    TailleSpectresExtremes, 
    1, 
    TRACE_REAL8, 
    TRACE_BIN, 
    contexte_courant->MExsSmax, 
    ctx);
#endif

fin:

  // ================================================================================
  // Traitement des erreurs =========================================================
  // ================================================================================
  if( cr_op != CMN__STATUT_OK )
  {
    // mise a jour de la variable ctx->cr.errinterne
    SD_BAS_TEST_ERR_INT (
      ctx->cr.errinterne, 
      SD_ALG_ERR_SET_MONITORING, 
      cr_op,
      tcProcName_F, 
      __LINE__, 
      msg_err);

  }
  // ================================================================================

  // sortie de la fonction
  return( cr_op);
}

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_110_DPT
//@
//@ Description : calcul table pixel morts
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@          numero de ligne traitee
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (ENTREE)  ConfStable : fichier de conf stable
//@         (ENTREE)  ImgCS_36 : image espace froid ( SN=36)
//@         (Entree/Sortie)  InfoDet : info det du produit ENG
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_110_DPT(
        t_SD_BAS_DataThread  *ctx,
        t_SD_BAS_Contexte    *contexte_courant,
        t_SD_BAS_ConfAutre   *ConfAutre,
        t_SD_BAS_ConfStable  *ConfStable,
        double               *ImgCS_36,
        double               *VarImag,
        t_SD_BAS_Liste_Var   InfoDet_ENG
//        t_SD_BAS_Liste_Var   InfoDet_VIADRENG
)
{

  static char tcProcName_F[]= "SD_ALG_110_DPT";
  t_CMN__Statut cr_op= CMN__STATUT_OK;
  // CR 332
  char *msg_err;
  // FIN CR 332

  // ================================================================================
  //@ Variables d'entrees et de sorties (Noms locaux) ===============================
  // ================================================================================
  int          NbLin;               // Nombre de lignes de l'image
  int          NbCol;               // Nombre de colonnes de l'image
  double      *RadCalSlopeImag;    // Coefficients de calibration radiometrique (pentes)
  double      *RadCalOffsetImag;   // Coefficients de calibration radiometrique (offsets)
  double       *VarImagIN;          // Pseudo-variance de l'image espace froid de la ligne precedente (LN-2)
  double       *ImageCS_36;         // Image de la visee de l'espace froid (SN=36)
  int          NbCycleFilter;       // Nombre de cycles de filtrage
  int          *IISDeadPix;         // Table des pixels morts de l'imageur
  double       IISGainCoeffCutoff;  // Perte de gain par rapport a la moyenne admissible
  double       IISVarianceCutoff;   // Seuil de pseudo-variance de l'image (valeur max admissible)
  int          IISInHomPixMax;      // Nombre maximum de declaration en mauvaise sante
  unsigned int flagInit;            // Indicateur de reinitialisation de l'image d'offset
  // Sorties
   int *IISBadHealthPix;    // Table des pixels en mauvaise sante
  int          *IISInHomPix;        // Table des compteurs de pixels inhomogenes
  double       *VarImagOUT;         // Pseudo-variance de l'image espace froid de la ligne precedente (LN-1)
  double       VarImagMax;          // Variance maximale
  double       VarImagMean;         // Vamriance moyenne
  double       PixQual;             // Qualite du pixel
  // ================================================================================


  // ================================================================================
  //@ Variables de la fonction ======================================================
  // ================================================================================
  int          imasize;
  double          *RadCalSlopeImagCurrent, *RadCalSlopeImagEnd;
  double       *VarImagOUTCurrent, *VarImagOUTEnd;
  double       *VarImagINCurrent;
  int          *IISDeadPixCurrent;
  double *RadCalOffsetImagCurrent;
   int *IISBadHealthPixCurrent;
  double       *ImageCS_36Current;
  int          *IISInHomPixCurrent;
  double       w;
  double       K;
  double       diff;
  double       SlopeMoy;
  double       NbDeadPix;
  double       NbInHomPix;
  int          i;
  int nbpix;
  // ================================================================================


  // Entree dans la fonction

  // ================================================================================
  // Initialisation des variables d'entrees/sorties =================================
  // ================================================================================
  NbLin=              ConfAutre->IDefIISNbLin;
  NbCol=              ConfAutre->IDefIISNbCol;
  RadCalSlopeImag=    InfoDet_ENG.tab_var[ENG_GICCRADCALSLOPEIMAG].val.pr8;
  RadCalOffsetImag=   InfoDet_ENG.tab_var[ENG_GICCRADCALOFFSETIMAG].val.pr8;
  VarImagIN=          VarImag; //(contexte->MDptVarImag)  (LN-2)
  ImageCS_36=         ImgCS_36;
  NbCycleFilter=      ConfAutre->IDefDptNbCycleFilter;
  IISDeadPix=         ConfStable->IDefDptIISDeadPix;
  IISGainCoeffCutoff= ConfAutre->IDefDptGainCoeffCutoff ;
  IISVarianceCutoff=  ConfAutre->IDefDptIISVarianceCutoff;
  IISInHomPixMax=     ConfAutre->IDefDptIISInHomPixMax;
  flagInit=           *(InfoDet_ENG.tab_var[ENG_GICCFLAGINIT].val.pui4);
  // Sorties (vecteurs)
  VarImagOUT=         (contexte_courant->MDptVarImag);               // Pas d'offset car (LN-1)
  IISInHomPix=        contexte_courant->MDptIisInHomPix; /* FA169 */
  IISBadHealthPix=    contexte_courant->MDptIisBadHealthPix;
//  IISBadHealthPix=    InfoDet_VIADRENG.tab_var[VIADRENG_MDPTIISBADHEALTHPIX].val.pui4;
  // ================================================================================


  // ================================================================================
  // Initialisations diverses =======================================================
  // ================================================================================
  //@ Taille de l'image
  imasize= NbLin * NbCol;

  //@ le contexte est ici mis a jour
  //@ Offset d'une image (imasize) car (LN-2)
  for( i=imasize; i < 2*imasize; i++)
    contexte_courant->MDptVarImag[i]= VarImagIN[i-imasize];

  //@ w = 1 / NbCycleFilter
  w= 1.0 / (double)NbCycleFilter;

  //@ K = 1 / IISGainCoeffCutoff
  K= 1.0 / IISGainCoeffCutoff;
  // ================================================================================



  // ================================================================================
  // Mise a zero de l'image =========================================================
  // ================================================================================
  if( flagInit == CMN__OUI)
  {
    //@ if (giccflaginit = oui) Alors  Pour tous les pixels image (nl,nc) VarImag(nl,nc)=0
    VarImagOUTCurrent= VarImagOUT;
    VarImagOUTEnd= VarImagOUT + imasize;
    while( VarImagOUTCurrent < VarImagOUTEnd)
    {
      (*VarImagOUTCurrent)= 0.0;
      VarImagOUTCurrent++;
    }
  }
  // ================================================================================


  // ================================================================================
  // Calcul de SlopeMoy =============================================================
  // ================================================================================
  //@ Calcul de la valeur moyenne SlopeMoy sur l'image RadCalSlopeImag
  //@ pourles pixels (nl,nc) a priori en bonne sante (i.e. IISDeadPix(nl,nc)=NON)

  //@ Initialisation des pointeurs en debut de tableaux
  RadCalSlopeImagCurrent=  RadCalSlopeImag;
  RadCalSlopeImagEnd=      RadCalSlopeImag + imasize;
  SlopeMoy=                0.0;
  nbpix=0;
  IISDeadPixCurrent=IISDeadPix;
  //@ Sommation sur les pixels
  while( RadCalSlopeImagCurrent < RadCalSlopeImagEnd)
  {
    if( (*IISDeadPixCurrent) == CMN__NON)
     {
     SlopeMoy += (double)(*RadCalSlopeImagCurrent);
     nbpix++;
     }
    RadCalSlopeImagCurrent++;
//    IISDeadPixCurrent=IISDeadPixCurrent+1;
  }
  //@ Moyenne
  SlopeMoy /= (double)nbpix;
  // ================================================================================


  // ================================================================================
  // Evaluation =====================================================================
  // ================================================================================
  //@ Initialisation des pointeurs en debut de tableaux
  IISDeadPixCurrent=       IISDeadPix;
  RadCalSlopeImagCurrent=  RadCalSlopeImag;
  RadCalOffsetImagCurrent= RadCalOffsetImag;
  IISBadHealthPixCurrent=  IISBadHealthPix;
  ImageCS_36Current=       ImageCS_36;
  IISInHomPixCurrent=      IISInHomPix;
  VarImagINCurrent=        VarImagIN;

  VarImagOUTCurrent=       VarImagOUT;
  VarImagOUTEnd=           VarImagOUT + imasize;
  nbpix=0;

  while( VarImagOUTCurrent < VarImagOUTEnd)
  {
    //@ Pour chaque pixel image (nl,nc) tel que IISDeadPix(nl,nc)=NON
  
    nbpix++;
    if( (*IISDeadPixCurrent) == CMN__NON)
    {
      //@   IISBadHealthPix(nl,nc)=NON
      (*IISBadHealthPixCurrent)= CMN__NON;

      //@   Premier critere de mauvaise sante : le gain
      //@   Si RadCalSlopeImag (nl,nc)>SlopeMoy*K Alors IISBadHealthPix(nl,nc)=OUI
      if( (*RadCalSlopeImagCurrent) > (SlopeMoy * K))    (*IISBadHealthPixCurrent)= CMN__OUI;

      //@   Deuxieme critere de mauvaise sante : le bruit (mesure sur l'image CS, SN=36)
      //@   VarImag(nl,nc)=   (1-w)*VarImag(nl,nc)
      //@                       + w * (ImageCS_36(nl,nc) - RadCalOffsetImag(nl,nc))^2
      diff= (*ImageCS_36Current) - (*RadCalOffsetImagCurrent);
      (*VarImagOUTCurrent)= (1.0 - w) * (*VarImagINCurrent)  +  w * (diff * diff);

      //@   Si VarImag(nl,nc) > IISVarianceCutoff Alors    IISBadHealthPix(nl,nc) = OUI
      if( *VarImagOUTCurrent > IISVarianceCutoff)    (*IISBadHealthPixCurrent)= CMN__OUI;

      //@   Si IISBadHealthPix(nl,nc) = OUI Alors
      if( (*IISBadHealthPixCurrent) == CMN__OUI)
      {
        //@     IISInHomPix(nl,nc) = IISInHomPix(nl,nc) + 1
        (*IISInHomPixCurrent)++;

        //@     Si IISInHomPix(nl,nc) < IISInHomPixMax Alors     IISBadHealthPix(nl,nc) = NON
        if( (*IISInHomPixCurrent) < IISInHomPixMax)   (*IISBadHealthPixCurrent)= CMN__NON;
      }
      else
      {
        //@     Sinon    IISInHomPix(nl,nc) = 0
        (*IISInHomPixCurrent)= 0;
      }
    } //@ !(*IISDeadPixCurrent)

    VarImagINCurrent++;
    VarImagOUTCurrent++;
    IISDeadPixCurrent++;
    RadCalSlopeImagCurrent++;
    RadCalOffsetImagCurrent++;
    IISBadHealthPixCurrent++;
    ImageCS_36Current++;
    IISInHomPixCurrent++;
  }  //@ Fin de boucle sur les pixels
  // ================================================================================


  // ================================================================================
  // Calcul des statistiques ========================================================
  // ================================================================================
  //@ VarImagMax = Max(VarImag(nl,nc))
  //@ VarImagMean = Mean(VarImag(nl,nc))
  //@ Indice de qualite de l'imageur
  //@   Calcul du nombre de pixels morts NbDeadPix
  //@   Calcul du nombre de pixels inhomogenes NbInHomPix
  IISDeadPixCurrent=  IISDeadPix;
  IISInHomPixCurrent= IISInHomPix;
  IISBadHealthPixCurrent=  IISBadHealthPix;
  nbpix=0;

  VarImagOUTCurrent=  VarImagOUT;
  VarImagOUTEnd=      VarImagOUT + (NbLin*NbCol);
  VarImagMax=         0.0;
  VarImagMean=        0.0;
  NbDeadPix=          0.0;
  NbInHomPix=         0.0;

  while( VarImagOUTCurrent < VarImagOUTEnd)
  {
    if ((*IISDeadPixCurrent)  == CMN__NON && (*IISBadHealthPixCurrent) == CMN__NON)
     {
      VarImagMean += (*VarImagOUTCurrent);
      VarImagMax= MAX( VarImagMax, *VarImagOUTCurrent);
      nbpix++;
    }
     if( (*IISDeadPixCurrent)  == CMN__OUI )    NbDeadPix++;
    if( (*IISInHomPixCurrent) > 0 )    NbInHomPix++;

    VarImagOUTCurrent++;
    IISDeadPixCurrent++;
    IISInHomPixCurrent++;
    IISBadHealthPixCurrent++;
  }
  VarImagMean /= (double)nbpix;

  //@   Calcul de la fraction de pixels vivants
  //@    PixQual=(NbLin*NbCol)/(NbLin*NbCol-NbDeadPix-NbInHomPix)
  // CR 332
  if (imasize != 0) 
  {
  	PixQual= ((double)imasize - NbDeadPix - NbInHomPix) / (double)imasize;
  } else {
  	cr_op = CMN__STATUT_NOK;
	msg_err = (char *)SD_ALG_MSG_ERR_CONF;
    	// mise a jour de la variable ctx->cr.errinterne
    	SD_BAS_TEST_ERR_INT (
      	ctx->cr.errinterne, 
      	SD_ALG_ERR_SET_MONITORING, 
      	cr_op,
      	tcProcName_F, 
      	__LINE__, 
      	msg_err);
  }
  // FIN CR 332
  // ================================================================================

  *(InfoDet_ENG.tab_var[ENG_MDPTVARIMAGMAX].val.pr8)=  VarImagMax;
  *(InfoDet_ENG.tab_var[ENG_MDPTVARIMAGMEAN].val.pr8)= VarImagMean;
  *(InfoDet_ENG.tab_var[ENG_MDPTPIXQUAL].val.pr8)=     PixQual;

  //@ le VIADR est mis a jour toutes les 10 lignes par l'appelant en utilisant
  //@   les donnees du contexte
  // ================================================================================
  // on trace ici si besoin toutes les variables en sortie
  // ================================================================================
#ifdef INVESTIGATION
  SD_BAS_TRACE( tcProcName_F,    "MDptIISBadHealthPix(LN-1)",
                NbLin,               NbCol,   1,
                TRACE_INT4,     TRACE_BIN,
                IISBadHealthPix, ctx);

  SD_BAS_TRACE( tcProcName_F, "MDptIISInHomPix",
                NbLin,            NbCol,   1,
                TRACE_INT4,   TRACE_BIN,
                IISInHomPix,  ctx);

  SD_BAS_TRACE( tcProcName_F, "MDptVarImag(LN-1)",
                NbLin,            NbCol,   1,
                TRACE_REAL8,  TRACE_BIN,
                VarImagOUT,   ctx);

  SD_BAS_TRACE( tcProcName_F,  "VarImagMax",
                1,   1,   1,
                TRACE_REAL8, TRACE_ASC,
                &VarImagMax,  ctx);
  SD_BAS_TRACE( tcProcName_F, "VarImagMean",
                1,   1,   1,
                TRACE_REAL8,  TRACE_ASC,
                &VarImagMean,  ctx);
  SD_BAS_TRACE( tcProcName_F, "PixQual",
                1,   1,   1,
                TRACE_REAL8,  TRACE_ASC,
                &PixQual,      ctx);
#endif
  // ================================================================================


fin:
  // sortie de la fonction
  return( cr_op);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_111_MCX
//@
//@ Description : surveillance de la calibration radiometrique
//@       complexe.
//@ remarque: on peut ici envisager de traiter toutes les bandes en un appel
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@          numero de ligne sn pn sb traitee
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (Entree)  InfoN0 :  paquet de N0 PX
//@         (Sortie)  InfoDet : info det du produit ENG
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_111_MCX(
        t_SD_BAS_DataThread  *ctx,
        t_SD_BAS_ConfAutre   *ConfAutre,
        t_SD_BAS_Liste_Var   InfoN0,
        t_SD_BAS_Liste_Var   InfoDet_ENG,
        double               *Spect       // Spectre calibre niveau 0 Spect=GDocSpect( PN,SN,LN )
)
{
  static char tcProcName_F[]= "SD_ALG_111_MCX";
  t_CMN__Statut cr_op= CMN__STATUT_OK;
  // Entree dans la fonction

  // ================================================================================
  //@ Variables d'entrees et de sorties (Noms locaux) ===============================
  // ================================================================================
  //Entrees :
  int    SpectralBand;      // Spectral Band : numero de la bande spectrale entre 1 et 3
  int    PixelNumber;       // Numero de pixel
  int    SubCycle;          // Numero de sous-cycle
  int    NbSeg;             // Nombre de plages spectrales NbSeg=IDefArcNbSeg( SB )
  int    NbNsSeg;           // Nombre d'echantillons spectraux par plage spectrale NbNsSeg=IDefArcNbNsSeg( SB )
  double *ImagMean;         // Valeur moyenne de la partie imaginaire residuelle par plage spectrale ImagMean=BArcImagMean( n=1..NbSeg,SB,PN,SN,LN )
  double *ImagRMS;          // Valeur RMS de la partie imaginaire residuelle par plage spectrale ImagRMS=BArcImagRMS(n=1..NbSeg,SB,PN,SN,LN )
//Not used  double *ImagMeanRMS;      // Valeur RMS de la partie imaginaire residuelle par bande spectrale ImagMeanRMS=BArcImagMeanRMS(n=1..NbSeg,SB,PN,SN,LN )
  int    Nsfirst;           // Numero du premier echantillon spectral de la bande Nsfirst=IDefNsfirst( SB )
  int    Nslast;            // Numero du dernier echantillon spectral de la bande Nslast=IDefNslast( SB )
  double DWnSpect;          // Largeur d'un echantillon spectral DWnSpect=IDefSpectrDWn
  double NoiseCalRadCutoff; // Seuil du bruit de calibration radiometrique de la bande NoiseCalRadCutoff=IDefMcxNoiseCalRadCutoff( SB )
  double BiasCalRadCutoff;  // Seuil de biais de calibration radiometrique de la bande BiasCalRadCutoff=IDefMcxBiasCalRadCutoffS( SB )
//Not used  double ColdSpect;         // Radiance nominale du spectre de calibration CS ColdSpect=IDefMcxColdSpectrum( SB )
//Not used  double Dphi;              // Ecart de phase nominal entre les spectres de calibration CS et BB Dphi=IDefMcxDphi( SB )

  //Sorties :
  double       NoiseCalRad[SD_BAS_TAILLE_NB_SI_B]; // Spectre de bruit de calibration radiometrique en NedT a 280K NoiseCalRad=MMcxNoiseCalRad(28, PN, SN, LN)
  double       BiasCalRad[SD_BAS_TAILLE_NB_SI_B];  // Spectre de biais de calibration radiometrique en NedT a 280K BiasCalRad=MMcxBiasCalRad(28, PN, SN, LN)
  unsigned int FlagNoiseCalRad = CMN__NON;   // Flag de depassement du seuil de bruit de calibration radiometrique FlagNoiseCalRad=MMcxFlagNoiseCalRad(PN, SN, LN)
  unsigned int FlagBiasCalRad = CMN__NON;    // Flag de depassement du seuil de biais de calibration radiometrique FlagBiasCalRad=MMcxFlagBiasCalRad(PN, SN, LN)
  double       CoefCalRad;        // Coefficient d'erreur de calibration radiometrique CoefCalRad=MMcxCoeffCalRad(SB, PN, SN, LN)
  // ================================================================================


  // ================================================================================
  //@ Variables de la fonction ======================================================
  // ================================================================================
  char   *msg_err;          // Message d'erreur
  double Coef;
  double Wn0;
  int    ind;
  int    m, n, n0;
  int    i;
  double Plank1, Plank2;
  double Sum, SumSpect, DPlk;
  double Wn[SD_BAS_TAILLE_NB_SI_B];
  double RSc[SD_BAS_TAILLE_NB_SI_B];
  double Phin[SD_BAS_TAILLE_NB_SI_B];
  double NCalRad[SD_BAS_TAILLE_NB_SI_B], *NCR;
  double Phib[SD_BAS_TAILLE_NB_SI_B];
  double BCalRad[SD_BAS_TAILLE_NB_SI_B], *BCR;
  int    LN;
  int    SN;
  int    PN;
  int    SB;
  int    NbSegSB;
  int    OffsetPtrCalRad;
  int    TabNbNsSeg[SD_BAS_NBSEGMAX];
  t_CMN__Booleen NbSegMaxTrouve;
  // ================================================================================


  // ================================================================================
  // Initialisation des variables d'entrees/sorties =================================
  // ================================================================================
  LN = ctx->ctxtrt.LN;
  SN = ctx->ctxtrt.SN;
  PN = ctx->ctxtrt.PN;
  SB = ctx->ctxtrt.SB;
  OffsetPtrCalRad = 0;
  NbSegSB = 0;
  for (i = 0 ; i < SD_BAS_SB ; i++)
  {
    NbSegSB = NbSegSB + ConfAutre->IDefArcNbSeg[i];
  }
  
  SpectralBand=      IND1(ctx->ctxtrt.SB);
  PixelNumber=       ctx->ctxtrt.PN;
  SubCycle=          ctx->ctxtrt.SN;
  NbSeg=             ConfAutre->IDefArcNbSeg[SpectralBand];
  NbNsSeg=           ConfAutre->IDefArcNbNsSeg[SpectralBand];
  if( SpectralBand == SD_BAS_B1)
  {
    ImagMean=     InfoN0.tab_var[PX_BARCIMAGMEANB1].val.pr8;
    ImagRMS=      InfoN0.tab_var[PX_BARCIMAGRMSB1].val.pr8;
//Unused    ImagMeanRMS=  InfoN0.tab_var[PX_BARCIMAGMEANRMSB1].val.pr8;
  }
  if( SpectralBand == SD_BAS_B2)
  {
    ImagMean=     InfoN0.tab_var[PX_BARCIMAGMEANB2].val.pr8;
    ImagRMS=      InfoN0.tab_var[PX_BARCIMAGRMSB2].val.pr8;
//Unused    ImagMeanRMS=  InfoN0.tab_var[PX_BARCIMAGMEANRMSB2].val.pr8;
  }
  if( SpectralBand == SD_BAS_B3)
  {
    ImagMean=     InfoN0.tab_var[PX_BARCIMAGMEANB3].val.pr8;
    ImagRMS=      InfoN0.tab_var[PX_BARCIMAGRMSB3].val.pr8;
//Unused    ImagMeanRMS=  InfoN0.tab_var[PX_BARCIMAGMEANRMSB3].val.pr8;
  }
  Nsfirst=           ConfAutre->IDefNsfirst[SpectralBand];
  Nslast=            ConfAutre->IDefNslast[SpectralBand];
  DWnSpect=          ConfAutre->IDefSpectrDWn;
  NoiseCalRadCutoff= ConfAutre->IDefMcxNoiseCalRadCutoff[SpectralBand];
  BiasCalRadCutoff=  ConfAutre->IDefMcxBiasCalRadCutoff[SpectralBand];
//Unused  ColdSpect=         ConfAutre->IDefMcxColdSpectrum[SpectralBand*SD_BAS_TAILLE_NB_SI_B];
//Unused  Dphi=              ConfAutre->IDefMcxDphi[SpectralBand*SD_BAS_TAILLE_NB_SI_B];
  // ================================================================================


  // ================================================================================
  //@ Controle des bornes ===========================================================
  // ================================================================================
  if( Nsfirst > Nslast)
  {
    cr_op= SD_ALG_MONITORING_ERR_NFIRST_NLAST;
    msg_err= (char *)SD_ALG_MSG_ERR_NFIRST_NLAST;

    goto fin;
  }

  if( Nsfirst < 0)
  {
    cr_op= SD_ALG_MONITORING_ERR_NFIRST_NEG;
    msg_err= (char *)SD_ALG_MSG_ERR_NFIRST_NEG;

    goto fin;
  }
  // ================================================================================

  // ================================================================================
  //@ Controle du nombre d'echantillons spectraux par plage spectrale ===============
  // ================================================================================
  if( NbNsSeg < 1)
  {
    cr_op= SD_ALG_MONITORING_ERR_NB_ECH_SPECT;
    msg_err= (char *)SD_ALG_MSG_ERR_NB_ECH_SPECT;

    goto fin;
  }
  
  for (m= 0; m < SD_BAS_NBSEGMAX; m++)
  {
    TabNbNsSeg[m] = 0;
  }
  
  NbSegMaxTrouve = CMN__FAUX;
  for (m= 1; (m <= NbSeg) && (NbSegMaxTrouve == CMN__FAUX); m++)
  {
    if (NbNsSeg * m <= Nslast - Nsfirst + 1)
    {
      TabNbNsSeg[IND1(m)] = NbNsSeg;
    }
    else
    {
      TabNbNsSeg[IND1(m)] = Nslast - Nsfirst + 1 - (m-1) * NbNsSeg;
      NbSegMaxTrouve = CMN__VRAI;
    }
  }
  NbSeg = m - 1;
  // ================================================================================

  //@ Coef = 0
  Coef= 0.0;

  Plank1= ConfAutre->IDefPlkPK1;
  Plank2= ConfAutre->IDefPlkPK2;

#ifdef DEBUG
  printf( "Nsfirst= %d\n", Nsfirst);
  printf( "Nslast= %d\n", Nslast);
  printf( "NbSeg= %d\n", NbSeg);
  printf( "NbNsSeg= %d\n", NbNsSeg);
#endif

//@ Boucle sur les plages spectrales  m=1 a NbSeg
  for( m= 1; m <= NbSeg; m++)
  {
    //@   Calcul du nombre d'onde central de la plage spectrale m (cf DA7)
    n0=  (m - 1) * NbNsSeg;
    Wn0= ( (double)Nsfirst - 1.0 + (double)n0) * DWnSpect;
    Sum= 0.0;
    SumSpect= 0.0;
    for( n= 1; n <= TabNbNsSeg[IND1(m)] ; n++)
    {
      ind= n + n0 + Nsfirst - ConfAutre->IDefNsfirst[SD_BAS_B1];
      Sum += (double) n;
      SumSpect += Spect[IND1(ind)];
    }

    Wn[IND1(m)]= Wn0 + (DWnSpect / (double)TabNbNsSeg[IND1(m)]) * Sum;
    
    //@   Calcul de la valeur moyenne du spectre de la plage spectrale m (cf DA7)
    RSc[IND1(m)]= SumSpect / TabNbNsSeg[IND1(m)];

    //@   Calcul de la phase residuelle caracteristique du bruit radiometrique
    //@     Phin(m) = Arctangente(ImagRMS(m)/RSc(m))
    Phin[IND1(m)]= atan( ImagRMS[IND1(m)] / RSc[IND1(m)]);

    //@   Calcul de l'erreur de calibration radiometrique
    //@     NCalRad(m)=RSc(m).(1-cos(Phin(m)))
    NCalRad[IND1(m)]= RSc[IND1(m)] * (1.0 - cos( Phin[IND1(m)]));

    //@   Conversion de l'erreur de calibration radiometrique en NedT a 280 K
    //@     NoiseCalRad(m)=NCalRad(m)/(Derivee de Planck a Wn(m) pour T=280 K)
// jlp
    DPlk= SD_BAS_PlanckDeriv( (double)280, Wn[IND1(m)], Plank1, Plank2);
    NoiseCalRad[IND1(m)]= NCalRad[IND1(m)] / DPlk;

    //@   Flag de depassement du seuil de bruit
    //@     Si NoiseCalRad(m) > NoiseCalRadCutoff) alors FlagNoiseCalRad=OUI
    if( NoiseCalRad[IND1(m)] > NoiseCalRadCutoff)
      FlagNoiseCalRad= CMN__OUI;

    //@   Calcul de la phase residuelle caracteristique du biais  radiometrique
    //@     Phib(m) = Arctangente(ImagMean(m)/RSc(m))
    Phib[IND1(m)]= atan( ImagMean[IND1(m)] / RSc[IND1(m)]);

    //@   Calcul de l'erreur de biais de calibration radiometrique
    //@     BCalRad(m)=RSc(m).(1-cos(Phib(m)))
    BCalRad[IND1(m)]= RSc[IND1(m)] * (1.0 - cos( Phib[IND1(m)]));

    //@   Conversion de l'erreur de biais de calibration radiometrique en NedT a 280 K
    //@    BiasCalRad(m)=BCalRad(m)/(Derivee de Planck a Wn(m) pour T=280 K)
    BiasCalRad[IND1(m)]= BCalRad[IND1(m)] / DPlk;

    //@   Flag de depassement du seuil de biais de calibration radiometrique
    //@     Si BiasCalRad > BiasCalRadCutoff) alors FlagBiasCalRad=OUI
    // CR 335
    if( fabs(BiasCalRad[IND1(m)]) > BiasCalRadCutoff)
      FlagBiasCalRad= CMN__OUI;
    // ...CR 335
    //@     Coef = Coef + cos(Phib(m))

    // CR 348
    Coef += BiasCalRad[IND1(m)];
    // ...CR 348
    
    #ifdef DEBUG
      printf( "========================================\n");
      printf( "m = %d\n", m);
      printf( "n0= %d\n", n0);
      printf( "Wn0= %le\n", Wn0);
      printf( "SumSpect= %le\n", SumSpect);
      printf( "Wn[m]= %le\n", Wn[IND1(m)]);
      printf( "RSc[m]= %le\n", RSc[IND1(m)]);
      printf( "ImagRMS[m]= %le\n", ImagRMS[IND1(m)]);
      printf( "Phin[m]= %le\n", Phin[IND1(m)]);
      printf( "NCalRad[m]= %le\n", NCalRad[IND1(m)]);
      printf( "NoiseCalRad[m]= %le\n", NoiseCalRad[IND1(m)]);
      printf( "Phib[m]= %le\n", Phib[IND1(m)]);
      printf( "BCalRad[m]= %le\n", BCalRad[IND1(m)]);
      printf( "BiasCalRad[m]= %le\n", BiasCalRad[IND1(m)]);
      printf( "Coef= %le\n", Coef);
    #endif
}  //@ Fin de boucle sur les plages spectrales

  //@ CoefCalRad = 1 - Coef / NbSeg
  // CR 348
  CoefCalRad= Coef / ((double)NbSeg);
  // ...CR 348
#ifdef DEBUG
  printf( "CoefCalRad= %f\n", CoefCalRad);
#endif

  // ================================================================================
  // Affectation de variables de sorties ============================================
  // ================================================================================
  NCR= InfoDet_ENG.tab_var[ENG_MMCXNOISECALRAD].val.pr8;
  BCR= InfoDet_ENG.tab_var[ENG_MMCXBIASCALRAD].val.pr8;
  switch (SB)
  {
    case 1:
    {
      OffsetPtrCalRad = (SN-1)*NbSegSB*SD_BAS_PN + (PN-1)*NbSegSB;
    }
    break;
    
    case 2:
    {
      OffsetPtrCalRad = 
        (SN-1)*NbSegSB*SD_BAS_PN + 
        (PN-1)*NbSegSB + 
        ConfAutre->IDefArcNbSeg[SD_BAS_B1];
    }
    break;
    
    case 3:
    {
      OffsetPtrCalRad = 
        (SN-1)*NbSegSB*SD_BAS_PN + 
        (PN-1)*NbSegSB + 
        ConfAutre->IDefArcNbSeg[SD_BAS_B1] + 
        ConfAutre->IDefArcNbSeg[SD_BAS_B2];
    }
    break;
    
    default:
    {
      OffsetPtrCalRad = 0;
    }
    break;
  }
  
  for (i = 0 ; i < OffsetPtrCalRad ; i++)
  {
    NCR++;
    BCR++;
  }
  
  for( m= 1; m <= NbSeg; m++)
  {
    *NCR = NoiseCalRad[IND1(m)];
    *BCR = BiasCalRad[IND1(m)];

    NCR++;
    BCR++;    
  }
  
  i= IND2( PixelNumber, SubCycle, SD_BAS_PN);
  /* SI on analyse la premiere bande spectrale ALORS */
  if (SpectralBand == 0)
  {
    /* On initialise les champs */
    InfoDet_ENG.tab_var[ENG_MMCXFLAGNOISECALRAD].val.pui4[i]= CMN__NON;
    InfoDet_ENG.tab_var[ENG_MMCXFLAGBIASCALRAD].val.pui4[i]=  CMN__NON;
  }
  
  if (FlagNoiseCalRad == CMN__OUI)
    InfoDet_ENG.tab_var[ENG_MMCXFLAGNOISECALRAD].val.pui4[i]= FlagNoiseCalRad;
  if (FlagBiasCalRad == CMN__OUI)
    InfoDet_ENG.tab_var[ENG_MMCXFLAGBIASCALRAD].val.pui4[i]=  FlagBiasCalRad;
  
  i= IND3( SpectralBand + 1, PixelNumber, SubCycle, SD_BAS_SB, SD_BAS_PN);
  InfoDet_ENG.tab_var[ENG_MMCXCOEFFCALRAD].val.pr8[i]= CoefCalRad;
  // ================================================================================

  // ================================================================================
  // on trace ici si besoin toutes les variables en sortie
  // ================================================================================
#ifdef INVESTIGATION
  SD_BAS_TRACE( tcProcName_F,  "NoiseCalRad",
                1,   NbSeg,   1,
                TRACE_REAL8,   TRACE_BIN,
                &NoiseCalRad,  ctx);
  SD_BAS_TRACE( tcProcName_F, "BiasCalRad",
                1,   NbSeg,   1,
                TRACE_REAL8,  TRACE_BIN,
                &BiasCalRad,  ctx);

  SD_BAS_TRACE( tcProcName_F,  "FlagNoiseCalRad",
                1,   1,   1,
                TRACE_UINT4, TRACE_ASC,
                &FlagNoiseCalRad,  ctx);

  SD_BAS_TRACE( tcProcName_F,  "FlagBiasCalRad",
                1,   1,   1,
                TRACE_UINT4, TRACE_ASC,
                &FlagBiasCalRad,  ctx);

  SD_BAS_TRACE( tcProcName_F, "CoefCalRad",
                1,   1,   1,
                TRACE_REAL8,  TRACE_ASC,
                &CoefCalRad,      ctx);
#endif
  // ================================================================================

fin:
  //@ le contexte est ici mis a jour
  // ================================================================================
  // Traitement des erreurs =========================================================
  // ================================================================================
  if( cr_op != CMN__STATUT_OK )
  {
    // mise a jour de la variable ctx->cr.errinterne
    SD_BAS_TEST_ERR_INT( ctx->cr.errinterne, SD_ALG_ERR_SET_MONITORING, cr_op,
                         tcProcName_F,       __LINE__,                  msg_err);

  }
  // ================================================================================

  // sortie de la fonction
  return( cr_op);
}

// FT-2525 / FT-2527
//@ ------------------------------------------------------------------*/
//@ Procedure : sub_45_QIS
//@
//@ Description : Detection d'un spike uniquement dans la bande 
//@               spectrale de determination du NZPD
//@ 
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape numero de ligne traitee
//@         (ENTREE)  ConfStable : fichier de conf stable
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (ENTREE/SORTIE) InfoDetENG : info det du produit ENG
//@         (ENTREE)  sn: scan number
//@         (ENTREE)  pn: pixel number
//@         (SORTIE)  FlagSpik : flag de spike dans la bande de determination du NZPD
//@         (SORTIE)  BSpkFlagSpikB1 : flag BSpkFlagSpik for band B1
//@         (SORTIE)  BSpkFlagSpikB2 : flag BSpkFlagSpik for band B2
//@         (SORTIE)  BSpkFlagSpikB3 : flag BSpkFlagSpik for band B3
//@         (SORTIE)  BSpkFlagSpikZpdBzpd :flag BSpkFlagSpik for band ZpdBzpd
//@         (SORTIE)  BZpdFlagNZpdNonQual :
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/


t_CMN__Statut sub_45_QIS(
  t_SD_BAS_DataThread *ctx,
  t_SD_BAS_ConfStable *ConfStable,
  t_SD_BAS_ConfAutre  *ConfAutre,
  t_SD_BAS_Liste_Var  *InfoDetENG,
  unsigned int sn,
  unsigned int pn,
  unsigned int *GOPSFlagSpikNZpdBzpd,
  unsigned int *BSpkFlagSpikB1,
  unsigned int *BSpkFlagSpikB2,
  unsigned int *BSpkFlagSpikB3,
  unsigned int *BSpkFlagSpikZpdBzpd,
  unsigned int *BZpdFlagNZpdNonQual,
  unsigned int *BArcFlagCalSpectNonQualB1,
  unsigned int *BArcFlagCalSpectNonQualB2,
  unsigned int *BArcFlagCalSpectNonQualB3        
  )
{
  static char tcProcName_F[] = "sub_45_QIS";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
 
  // declarations
  int                 ZpdBzpd;       // Spectral band ID for NZPD determination
  unsigned int        FlagMultiBand; // switch on/off for spectra avaibility 
  int                 SB;            // compteur sur les bandes spectrales
  unsigned int        FlagInterPixNzpdNonQual;
  unsigned int        FlagExternCalMode;          //@ Indicateur de mode calibration externe
  unsigned int        tmp;    
  // FT-2596
  int                 state;
  // ...FT-2596


  // initialisations
  ZpdBzpd             = ConfAutre->IDefZpdBzpd; 
  FlagMultiBand       = ConfStable->IDefFlagMultiBand;
  
  FlagExternCalMode   = *(InfoDetENG->tab_var[ENG_INSTRUMENT_MODE].val.pui4);
  
  // FT-2765
  //@ word 151 from N0
  tmp = InfoDetENG->tab_var[ENG_DATA_PX].val.pui4[147*SD_BAS_PN*SD_BAS_SNOT + IND2(pn,sn,SD_BAS_PN)];
  //@ BSpkFlagSpik B1 = bit 0 ; BSpkFlagSpik B2 = bit 1; BSpkFlagSpik B3 = bit 2
  *BSpkFlagSpikB1 = SD_BAS_Is_BitSeti( (unsigned short)tmp, 0);
  *BSpkFlagSpikB2 = SD_BAS_Is_BitSeti( (unsigned short)tmp, 1);
  *BSpkFlagSpikB3 = SD_BAS_Is_BitSeti( (unsigned short)tmp, 2);
  *BSpkFlagSpikZpdBzpd = SD_BAS_Is_BitSeti((unsigned short)tmp,ZpdBzpd-1);
  //@ bit 3
  *BZpdFlagNZpdNonQual = SD_BAS_Is_BitSeti((unsigned short)tmp,3);
  //@ BArcFlagCalSpectNonQual B1 = bit 7, B2 = bit 8, B3 = bit 9
  *BArcFlagCalSpectNonQualB1 = SD_BAS_Is_BitSeti((unsigned short)tmp, 7); // B1
  *BArcFlagCalSpectNonQualB2 = SD_BAS_Is_BitSeti((unsigned short)tmp, 8); // B2
  *BArcFlagCalSpectNonQualB3 = SD_BAS_Is_BitSeti((unsigned short)tmp, 9); // B3
  // ...FT-2765

  // MHipFlaginterPixNzpdNonQual
  FlagInterPixNzpdNonQual =
  InfoDetENG->tab_var[ENG_MHIPFLAGINTERPIXNZPDNONQUAL].val.pui4[IND2(pn,sn,SD_BAS_PN)];

  *GOPSFlagSpikNZpdBzpd = CMN__NON;
  // FT-2596
  state = 0;
  // ...FT-2596

  // corps de la fonction
  //@ Mise a jour de l'indice de qualite en cas de recuperation de bande (spike)
  if (FlagMultiBand == CMN__OUI) 
  {
    if ((FlagExternCalMode%128 == SD_BAS_NORMAL) || 
        ((FlagExternCalMode%128 == SD_BAS_EXTERNAL_CALIBRATION) && (sn != 1) &&
        (sn != 2) && (sn !=30)))
    {
       //@ Mise a jour de l'indice de qualite dans le cas de detection d'un 
       //@ spike uniquement dans la bande spectrale de determination du NZPD
       if (*BSpkFlagSpikZpdBzpd == CMN__NON)
       {
         //@
         if ((*BZpdFlagNZpdNonQual == CMN__OUI) && (FlagInterPixNzpdNonQual == CMN__NON))
         {
           //@ B1
           if ((SD_BAS_B1 != IND1(ZpdBzpd)) && (*BSpkFlagSpikB1 == CMN__OUI) && (*BArcFlagCalSpectNonQualB1 == CMN__OUI))
           {
             InfoDetENG->tab_var[ENG_GQISFLAGQUAL].val.pui4[IND3(SD_BAS_B1+1,pn,sn,SD_BAS_SB,SD_BAS_PN)] = CMN__NON;
             // FT-2596
             state++;
             // ...FT-2596
           }
           //@ B2
           if ((SD_BAS_B2 != IND1(ZpdBzpd)) && (*BSpkFlagSpikB2 == CMN__OUI) && (*BArcFlagCalSpectNonQualB2 == CMN__OUI))
           {
             InfoDetENG->tab_var[ENG_GQISFLAGQUAL].val.pui4[IND3(SD_BAS_B2+1,pn,sn,SD_BAS_SB,SD_BAS_PN)] = CMN__NON;
             // FT-2596
             state++;
             // ...FT-2596
           }
           //@ B3
           if ((SD_BAS_B3 != IND1(ZpdBzpd)) && (*BSpkFlagSpikB3 == CMN__OUI) && (*BArcFlagCalSpectNonQualB3 == CMN__OUI))
           {
             InfoDetENG->tab_var[ENG_GQISFLAGQUAL].val.pui4[IND3(SD_BAS_B3+1,pn,sn,SD_BAS_SB,SD_BAS_PN)] = CMN__NON;
             // FT-2596
             state++;
             // ...FT-2596
           }
           // FT-2596
           if (state == 2)
           {
             *GOPSFlagSpikNZpdBzpd = CMN__OUI;
           }
           // ...FT-2596
         }
       }
    }
  } // Fin si FlagMultiBand
  
  return(cr_op);
}
// ...FT-2525 / ...FT-2527

// FT-2525 / FT-2527
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_45_QIS
//@
//@ Description : Calcul des Flags et indices de qualite
//@       
//@ remarque: on peut ici envisager de traiter toutes les bandes en un appel
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape numero de ligne traitee
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (ENTREE)  NbISPPX : nombre des paquets de N0 PX
//@         (ENTREE)  ListeInfoN0PX : Liste des paquets de N0 PX pour la ligne
//@         (ENTREE)  NbISPPX : nombre des paquets de N0 IP
//@         (ENTREE)  ListeInfoN0IP : Liste des paquets de N0 IP pour la ligne
//@         (ENTREE/SORTIE)  InfoDetENG : info det du produit ENG
//@         (ENTREE/SORTIE)  InfoDetENG : info det du produit N1
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_45_QIS(
        t_SD_BAS_DataThread *ctx,
        t_SD_BAS_ConfStable  *ConfStable,
        t_SD_BAS_ConfAutre   *ConfAutre,
        t_SD_BAS_Liste_Var  *InfoDetENG,
        t_SD_BAS_Liste_Var * InfoDetN1ABC
)
// ...FT-2525 / ...FT2527
{
  static char tcProcName_F[] = "SD_ALG_45_QIS";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  // Entree dans la fonction


    // Entrees
    int                 IasiTime;

    double              *CoeffCalRad;              //@ Coefficient d'erreur de la calibration radiometrique
    double              CovarEigenValCalRad;        //@ Indice de bruit associe a la calibration radiometrique
    double              CovarEigenValSpect;         //@ Indice de bruit associe a la calibration spectrale
    double              TrNom;                      //@ Trace de la matrice de covariance nominale
    double              TrCalRad;                   //@ Trace de la partie calibration radiometrique de la matrice de covariance
    double              TrSpect;                    //@ Trace de la partie calibration spectrale de la matrice de covariance
    double              IISNoiseNom;                //@ Bruit radiometrique de l'IIS moyen nominal
    double              IISVarImagMean;             //@ Valeur moyenne de la pseudo-variance dans l'image IIS
    unsigned int        SysTECSondQual;             //@ Indice de qualite systeme-TEC du sondeur
    unsigned int        SysTECIISQual;              //@ Indice de qualite systeme-TEC de l'IIS
    unsigned int        FlagExternCalMode;          //@ Indicateur de mode calibration externe
    double              DurFltIsrfOff;              //@ Duree maximum du cas degrade " chaine ISRF-EM non activee "
    double              DurFltBBTMiss;              //@ Duree maximum du cas degrade " pas de temperature BB "
    double              DurFltImgBBMiss;            //@ Duree maximum du cas degrade " pas d'image de calibration BB "
    double              DurFltImgCsMiss;            //@ Duree maximum du cas degrade " pas d'image de calibration CS "
    double              DurFltRadAvhrMiss;          //@ Duree maximum du cas degrade " pas de radiances Avhrr "
    double              DurImgEWMiss;

    // FT-2525
    unsigned int        BSpkFlagSpikB1;             //@ flag BSpkFlagSpik for band B1
    unsigned int        BSpkFlagSpikB2;             //@ flag BSpkFlagSpik for band B2
    unsigned int        BSpkFlagSpikB3;             //@ flag BSpkFlagSpik for band B3
    unsigned int        BSpkFlagSpikZpdBzpd;        //@ flag BSpkFlagSpik for band ZpdBzpd
    unsigned int        BZpdFlagNZpdNonQual;        
    unsigned int        FlagInterPixNzpdNonQual;
    // FT-2527
    unsigned int        GOPSFlagSpikNZpdBzpd;
    unsigned int        BArcFlagCalSpectNonQualB1;
    unsigned int        BArcFlagCalSpectNonQualB2;
    unsigned int        BArcFlagCalSpectNonQualB3;       
    // ..FT-2527
    // ..FT-2525
    unsigned int        BIMSCSQ;
    unsigned int        BIMSSSQ1;
    unsigned int        BIMSSSQ2;
    unsigned int        BBofFlagSpecNonQual;
    unsigned int        *GDocFlagUnderOverFlow;
    unsigned int        *GFaxAxeNonQual;
    unsigned int        *GIsfFlagPdsNonValid;
    unsigned int        *GIacFlagCoregNonQual;
    unsigned int        *GIacFlagCoregNonValid;
    unsigned int        GHecFlagDateNOK;
    unsigned int        *MMcxFlagBiasCalRad;
    unsigned int        *MMcxFlagNoiseCalRad;

    unsigned int        *GOPSFltIsrfemOff;
    // FT-2749
    unsigned int        GFtbFlagBBTNonQual;
    // ...FT-2749
    unsigned int        GOPSFltImgBBMiss;
    unsigned int        GOPSFltImgCSMiss;
    unsigned int        *GOPSFltRadAvhrrMiss;
    // FA 182
    unsigned int        *GOPSFlaGeoAvhrrMiss;
    // ..FA 182
    unsigned int        *GOPSFlagPixMiss;
    unsigned int        *GOPSFltImgEWMiss;
    // FT-2525
    unsigned int        *GOPSFltBandMiss;
    // ..FT-2525

    int                 GOPSDatIsrfemOff;
    int                 GOPSDatBBTMiss;
    int                 GOPSDatImgBBMiss;
    int                 GOPSDatImgCSMiss;
    int                 GOPSDatRadAvhrrMiss;
    int                 GOPSDatImgEWMiss;
    // FT-2525
    int                 ZpdBzpd;        // Spectral band ID for NZPD determination
    unsigned int      FlagMultiBand;  // switch on/off for spectra avaibility 
    // ..FT-2525

    // Sorties
    double              QualIndexRadAvg;            //@ Indice de qualite radiometrique des produits sondeur (moyen)
    double              QualIndexSpect;             //@ Indice de qualite spectrale des produits sondeur
    double              QualIndexLocAvg;            //@ Indice de qualite geometrique des produits sondeur (moyen)
    double              QualIndexIISAvg;            //@ Indice de qualite de l'image IIS dans les produits 1C (moyen)
    double              QualIndexAvg;               //@ Indice de qualite general des produits sondeur (moyen)


    // Internes
    unsigned int        tmp;                        //@ Pour la recuperation des flags dans le produit;
    /* PM4 CR177 */
    /* unsigned int        FlagQualProduit; */
    double              CoeffMoy;
    double              Tr;
    double              AxeRes;                     //@ Qualite du filtrage de l'axe interferometrique
    double              AxeRes0;                    //@ Qualite du filtrage de l'axe interferometrique pour CCD=0
    double              AxeRes1;                    //@ Qualite du filtrage de l'axe interferometrique pour CCD=1

    // Compteurs
    int                 SN;                         //@ Compteur sur les SN
    /* DM 11/03/2004 */
    int                 SP;                         //@ Compteur sur les SP

    // FT-2525   
    int                 SB;                         //@ compteur sur les bandes spectrales
    unsigned short   FlagQualDetailed;           //@ 
    // ..FT-2525
    
    /* FA156 */
    int                 SPmem;                      //@ Compteur sur les SP
    /* FIN FA156 */
    
    int                 PN;                         //@ Compteur sur les PN
    int                 CD;                         //@ Indicateur de CCD
    int                 ct;
    /* CR 2373 */
    int                 ct_loc;
    /* ... CR 2373 */
   
 
    //@ Initialisations
    SN                  = ctx->ctxtrt.SN;               // Pour pouvoir utiliser les macros
    
    /* FA156 */
    SPmem = 0;
    /* FIN FA156 */
    
    // FT-2574 
    // FT-2525
    FlagQualDetailed = 0x1FFF;   //@  FlagQualDetailed = non sauf sur bits 13,14,15                   
    // ..FT-2525
    // ...FT-2574 
    
    /* DM 11/03/2004 */
    IasiTime            = NINT( CMN__DATE_CONV_SCDS_2_SEC(InfoDetENG->tab_var[ENG_GEPSDATIASI].val.pdate[IND1(SD_BAS_SNREFDATELIGNE)]));

    CoeffCalRad        = InfoDetENG->tab_var[ENG_MMCXCOEFFCALRAD].val.pr8;
    CovarEigenValCalRad = ConfAutre->IDefCovarEigenValCalRad;
    CovarEigenValSpect  = ConfAutre->IDefCovarEigenValSpect;
    TrNom               = ConfAutre->IDefTrnom;
    TrCalRad            = ConfAutre->IDefTrCalRad;
    TrSpect             = ConfAutre->IDefTrSpect;
    IISNoiseNom         = ConfAutre->IDefIISNoiseNom;
    SysTECSondQual      = ConfAutre->IDefSysTECSondQual;
    SysTECIISQual       = ConfAutre->IDefSysTECIISQual;
    DurFltIsrfOff       = ConfAutre->IDefDurFltIsrfemOff;
    DurFltBBTMiss       = ConfAutre->IDefDurBBTMiss;
    DurFltImgBBMiss     = ConfAutre->IDefDurImgBBMiss;
    DurFltImgCsMiss     = ConfAutre->IDefDurImgCSMiss;
    DurFltRadAvhrMiss   = ConfAutre->IDefDurRadAvhrrMiss;
    DurImgEWMiss        = ConfAutre->IDefDurImgEWMiss;

    // FT-2525
    ZpdBzpd             = ConfAutre->IDefZpdBzpd;
    FlagMultiBand       = ConfStable->IDefFlagMultiBand;
    // ..FT-2525

    IISVarImagMean      = *(InfoDetENG->tab_var[ENG_MDPTVARIMAGMEAN].val.pr8);
    FlagExternCalMode   = *(InfoDetENG->tab_var[ENG_INSTRUMENT_MODE].val.pui4);

    // Flags de sortie des algos
    GDocFlagUnderOverFlow   = InfoDetENG->tab_var[ENG_GDOCFLAGUNDEROVERFLOW].val.pui4;
    GFaxAxeNonQual          = InfoDetENG->tab_var[ENG_GFAXFLAGAXENONQUAL].val.pui4;
    GIsfFlagPdsNonValid     = InfoDetENG->tab_var[ENG_GISFFLAGPDSNONVALID].val.pui4;
    GIacFlagCoregNonQual    = InfoDetENG->tab_var[ENG_GIACFLAGCOREGNONQUAL].val.pui4;
    GIacFlagCoregNonValid   = InfoDetENG->tab_var[ENG_GIACFLAGCOREGNONVALID].val.pui4;
    GHecFlagDateNOK         = *(InfoDetENG->tab_var[ENG_GHECFLAGDATENOK].val.pui4);
    MMcxFlagBiasCalRad      = InfoDetENG->tab_var[ENG_MMCXFLAGBIASCALRAD].val.pui4;
    MMcxFlagNoiseCalRad     = InfoDetENG->tab_var[ENG_MMCXFLAGNOISECALRAD].val.pui4;
    GOPSFltIsrfemOff        = InfoDetENG->tab_var[ENG_GOPSFLTISRFEMOFF].val.pui4;
    // FT-2749
    GFtbFlagBBTNonQual      = InfoDetENG->tab_var[ENG_GFTBFLAGBBTNONQUAL].val.pui4[0];
    // ...FT-2749
    GOPSFltImgBBMiss        = *(InfoDetENG->tab_var[ENG_GOPSFLTIMGBBMISS].val.pui4);
    GOPSFltImgCSMiss        = *(InfoDetENG->tab_var[ENG_GOPSFLTIMGCSMISS].val.pui4);
    GOPSFltRadAvhrrMiss     = InfoDetENG->tab_var[ENG_GOPSFLTRADAVHRRMISS].val.pui4;
   // FA 182
    GOPSFlaGeoAvhrrMiss     = InfoDetENG->tab_var[ENG_GOPSFLAGEOAVHRRMISS].val.pui4;
   // ..FA 182

    GOPSFlagPixMiss         = InfoDetENG->tab_var[ENG_GOPSFLAPIXMISS].val.pui4;
    GOPSFltImgEWMiss        = InfoDetENG->tab_var[ENG_GOPSFLTIMGEWMISS].val.pui4;
    // FT-2525
    GOPSFltBandMiss         = InfoDetENG->tab_var[ENG_GOPSFLTBANDMISS].val.pui4;
    // ..FT-2525

    GOPSDatIsrfemOff        = NINT( CMN__DATE_CONV_SCDS_2_SEC(*(InfoDetENG->tab_var[ENG_GOPSDATISRFEMOFF].val.pdate)));
    GOPSDatBBTMiss          = NINT( CMN__DATE_CONV_SCDS_2_SEC(*(InfoDetENG->tab_var[ENG_GOPSDATBBTMISS].val.pdate)));
    GOPSDatImgBBMiss        = NINT( CMN__DATE_CONV_SCDS_2_SEC(*(InfoDetENG->tab_var[ENG_GOPSDATIMGBBMISS].val.pdate)));
    GOPSDatImgCSMiss        = NINT( CMN__DATE_CONV_SCDS_2_SEC(*(InfoDetENG->tab_var[ENG_GOPSDATIMGCSMISS].val.pdate)));
    GOPSDatRadAvhrrMiss     = NINT( CMN__DATE_CONV_SCDS_2_SEC(*(InfoDetENG->tab_var[ENG_GOPSDATRADAVHRRMISS].val.pdate)));
    GOPSDatImgEWMiss        = NINT( CMN__DATE_CONV_SCDS_2_SEC(*(InfoDetENG->tab_var[ENG_GOPSDATIMGEWMISS].val.pdate)));

   // CR 348 - Deplacement du code
    AxeRes0 = InfoDetENG->tab_var[ENG_GFAXAXERES].val.pr8[0];
    AxeRes1 = InfoDetENG->tab_var[ENG_GFAXAXERES].val.pr8[1];
    
    /* FA156 */
   //CR271 on traite le cas de transition
    if ( (FlagExternCalMode%128 == SD_BAS_EXTERNAL_CALIBRATION) &&
         ( (SPmem == 31) ||
           (SPmem == 32) ||
           (SPmem == 33) ||
           (SPmem == 35) ||
           (SPmem == 36)
         )
       )
    /* FIN FA156 */
    {
      QualIndexSpect = 1.0;
    }
    else
    {
      // CR 348
      QualIndexSpect = sqrt((AxeRes0*AxeRes0*TrSpect*TrSpect + AxeRes1*AxeRes1*TrSpect*TrSpect)/2.0 + CovarEigenValSpect*CovarEigenValSpect) / CovarEigenValSpect;
        // ...CR 348
    }
     // ...CR 348 - Deplacement du code 
    
    //@ Boucle sur les souscycles
    for(SN=1; SN<=SD_BAS_SNOT; SN++)
    {
      //@ Recherche de CD=CCD(SN)
      CD = InfoDetENG->tab_var[ENG_GEPS_CCD].val.pui4[IND1(SN)] + 1;
      
      /* DM 11/03/2004 */
      SP = InfoDetENG->tab_var[ENG_GEPS_SP].val.pi4[IND1(SN)];
      
      /* FA156 */
      if ( (SN>2) && (SN<SD_BAS_SNOT) )
      {
        SPmem = SP;
      }
      /* FIN FA156 */
      
      //@ Boucle sur les pixels
      for(PN=1; PN<=4; PN++)
      {
        //@ Lecture des bits B10, B13, B14 du word 13 correspondant a PN et SN
        tmp = InfoDetENG->tab_var[ENG_DATA_PX].val.pui4[9*SD_BAS_PN*SD_BAS_SNOT + IND2(PN,SN,SD_BAS_PN)];
        // FT-2765
        BIMSCSQ= SD_BAS_Is_BitSeti((unsigned short)tmp,10);
        BIMSSSQ1 = SD_BAS_Is_BitSeti((unsigned short)tmp,12);
        BIMSSSQ2 = SD_BAS_Is_BitSeti((unsigned short)tmp,13);
        // ...FT-2765
        // FT-2523
        //@ Lecture du bit B2 du word 150 correspondant a PN et SN
        tmp = InfoDetENG->tab_var[ENG_DATA_PX].val.pui4[146*SD_BAS_PN*SD_BAS_SNOT + IND2(PN,SN,SD_BAS_PN)];
        // FT-2765
        BBofFlagSpecNonQual = SD_BAS_Is_BitSeti( (unsigned short)tmp,2);
        // ...FT-2765
        // ..FT-2523
        
        //@ Condition sur le mode calibration
        //@ Si on est en mode de calibration
#ifdef OPS_DEBUG
        printf("Boucle SN=%d PN=%d\n---------------------\n",SN,PN);
        printf("   FlagExternCalMode=%d\\nn",FlagExternCalMode);
        printf("   BIMSCSQ=%d\n",BIMSCSQ);
        printf("   BIMSSSQ1=%d\n",BIMSSSQ1);
        printf("   BIMSSSQ2=%d\n",BIMSSSQ2);
        printf("   BBofFlagSpecNonQual=%d\n",BBofFlagSpecNonQual);
        printf("   GOPSFlagPixMiss=%d\n",GOPSFlagPixMiss[IND2(PN,SN,SD_BAS_PN)]);
        printf("   GDocFlagUnderOverFlow=%d\n",GDocFlagUnderOverFlow[IND2(PN,SN,SD_BAS_PN)]);
        printf("   GFaxAxeNonQual=%d\n",GFaxAxeNonQual[IND1(CD)]);
        printf("   GIsfFlagPdsNonValid=%d\n",GIsfFlagPdsNonValid[IND1(CD)]);
        printf("   GIacFlagCoregNonQual=%d\n",GIacFlagCoregNonQual[IND1(SN)]);
        printf("   GIacFlagCoregNonValid=%d\n",GIacFlagCoregNonValid[IND1(SN)]);
        printf("   MMcxFlagBiasCalRad=%d\n",MMcxFlagBiasCalRad[IND2(PN,SN,SD_BAS_PN)]);
        printf("   MMcxFlagNoiseCalRad=%d\n",MMcxFlagNoiseCalRad[IND2(PN,SN,SD_BAS_PN)]);
        printf("   GHecFlagDateNOK=%d\n",GHecFlagDateNOK);
        /* PM4 CR177 */
        printf("   GOPSFltIsrfemOff=%d\n",GOPSFltIsrfemOff[IND1(SN)]);
        printf("   IasiTime=%f\tGOPSDatIsrfemOff=%f\tDurFltIsrfOff=%f\n",IasiTime,GOPSDatIsrfemOff,DurFltIsrfOff);
        // FT-2771
        printf("   GFtbFlagBBTNonQual=%d\n",GFtbFlagBBTNonQual);
        // ...FT-2771
        printf("   IasiTime=%f\tGOPSDatBBTMiss=%f\tDurFltBBTMiss=%f\n",IasiTime,GOPSDatBBTMiss,DurFltBBTMiss);
        printf("   GOPSFltImgBBMiss=%d\n",GOPSFltImgBBMiss);
        printf("   IasiTime=%f\tGOPSDatImgBBMiss=%f\tDurFltImgBBMiss=%f\n",IasiTime,GOPSDatImgBBMiss,DurFltImgBBMiss);
        printf("   GOPSFltImgCSMiss=%d\n",GOPSFltImgCSMiss);
        printf("   IasiTime=%f\tGOPSDatImgCSMiss=%f\tDurFltImgCsMiss=%f\n",IasiTime,GOPSDatImgCSMiss,DurFltImgCsMiss);
        printf("   GOPSFltRadAvhrrMiss=%d\n",GOPSFltRadAvhrrMiss[IND1(SN)]);
        printf("   IasiTime=%f\tGOPSDatRadAvhrrMiss=%f\tDurFltRadAvhrMiss=%f\n",IasiTime,GOPSDatRadAvhrrMiss,DurFltRadAvhrMiss);
        printf("   GOPSFltImgEWMiss=%d\n",GOPSFltImgEWMiss[IND1(SN)]);
        printf("   IasiTime=%f\tGOPSDatImgEWMiss=%f\tDurImgEWMiss=%f\n",IasiTime,GOPSDatImgEWMiss,DurImgEWMiss);
        printf("   GOPSFlaGeoAvhrrMiss=%d\n", GOPSFlaGeoAvhrrMiss[IND1(SN)] );
#endif

        // FT-2765
         if ( BIMSCSQ                                      == CMN__ERREUR )
           printf( "*> BIMSCSQ                                      == CMN__ERREUR \n" );
         if ( BIMSSSQ1                                     == CMN__ERREUR )
           printf( "*> BIMSSSQ1                                     == CMN__ERREUR \n" );
         if ( BIMSSSQ2                                     == CMN__ERREUR )
           printf( "*> BIMSSSQ2                                     == CMN__ERREUR \n" );
         if ( BBofFlagSpecNonQual                          == CMN__ERREUR )
           printf( "*> BBofFlagSpecNonQual                          == CMN__ERREUR \n" );
         if ( GOPSFlagPixMiss[IND2(PN,SN,SD_BAS_PN)]       == CMN__OUI )
           printf( "*> GOPSFlagPixMiss[IND2(PN,SN,SD_BAS_PN)]       == CMN__OUI \n" );
         if ( GDocFlagUnderOverFlow[IND2(PN,SN,SD_BAS_PN)] == CMN__OUI )
           printf( "*> GDocFlagUnderOverFlow[IND2(PN,SN,SD_BAS_PN)] == CMN__OUI \n" );
         if ( GFaxAxeNonQual[IND1(CD)]                     == CMN__OUI )
           printf( "*> GFaxAxeNonQual[IND1(CD)]                     == CMN__OUI \n" );
         if ( GIsfFlagPdsNonValid[IND1(CD)]                == CMN__OUI )
           printf( "*> GIsfFlagPdsNonValid[IND1(CD)]                == CMN__OUI \n" );
         if ( MMcxFlagBiasCalRad[IND2(PN,SN,SD_BAS_PN)]    == CMN__OUI )
           printf( "*> MMcxFlagBiasCalRad[IND2(PN,SN,SD_BAS_PN)]    == CMN__OUI \n" );
         if ( MMcxFlagNoiseCalRad[IND2(PN,SN,SD_BAS_PN)]   == CMN__OUI )
           printf( "*> MMcxFlagNoiseCalRad[IND2(PN,SN,SD_BAS_PN)]   == CMN__OUI \n" );
         if ( GHecFlagDateNOK                              == CMN__OUI )
           printf( "*> GHecFlagDateNOK                              == CMN__OUI \n" );
         if ( (GOPSFltIsrfemOff[IND1(SN)]                  == CMN__OUI && (IasiTime - GOPSDatIsrfemOff    > DurFltIsrfOff)) )
           printf( "*> (GOPSFltIsrfemOff[IND1(SN)]                  == CMN__OUI && (IasiTime - GOPSDatIsrfemOff    > DurFltIsrfOff)) \n" );
         if ( GFtbFlagBBTNonQual                           == CMN__OUI )
           printf( "*> GFtbFlagBBTNonQual                           == CMN__OUI \n" );
         if ( (GOPSFltImgBBMiss                            == CMN__OUI && (IasiTime - GOPSDatImgBBMiss    > DurFltImgBBMiss)) )
           printf( "*> (GOPSFltImgBBMiss                            == CMN__OUI && (IasiTime - GOPSDatImgBBMiss    > DurFltImgBBMiss)) \n" );
         if ( (GOPSFltImgCSMiss                            == CMN__OUI && (IasiTime - GOPSDatImgCSMiss    > DurFltImgCsMiss)) )
           printf( "*> (GOPSFltImgCSMiss                            == CMN__OUI && (IasiTime - GOPSDatImgCSMiss    > DurFltImgCsMiss)) \n" );
         if ( (GOPSFltImgEWMiss[IND1(SN)]                  == CMN__OUI && (IasiTime - GOPSDatImgEWMiss    > DurImgEWMiss)) )
           printf( "*> (GOPSFltImgEWMiss[IND1(SN)]                  == CMN__OUI && (IasiTime - GOPSDatImgEWMiss    > DurImgEWMiss)) \n" );
         if ( (GOPSFltRadAvhrrMiss[IND1(SN)]               == CMN__OUI && (IasiTime - GOPSDatRadAvhrrMiss > DurFltRadAvhrMiss)) )
           printf( "*> (GOPSFltRadAvhrrMiss[IND1(SN)]               == CMN__OUI && (IasiTime - GOPSDatRadAvhrrMiss > DurFltRadAvhrMiss)) \n" );
         if ( (GOPSFlaGeoAvhrrMiss[IND1(SN)]               == CMN__OUI ) )
           printf( "*> (GOPSFlaGeoAvhrrMiss[IND1(SN)]               == CMN__OUI ) \n" );
         // ...FT-2765
         /* PM4 CR177 */
        //@ SI SN non valide ALORS */
        //CR271 on traite le cas de transition
        if(FlagExternCalMode%128 == SD_BAS_EXTERNAL_CALIBRATION && (SN==1 || SN==2 || SN==30))
        {
          //@ Flag de qualite des produits = NON
          /* PM4 CR234 inversion logique CMN__OUI a la place de CMN__NON */
          // FT-2525
          InfoDetENG->tab_var[ENG_GQISFLAGQUAL].val.pui4[IND3(ZpdBzpd,PN,SN,SD_BAS_SB,SD_BAS_PN)] = CMN__OUI;
          // ..FT-2525
        }
        //@ Sinon : calcul du flag de qualite des produits
        else
        {
          /* Prise en compte de CR211 : CMN__ERREUR */
          /* DM 11/03/2004 */
          //CR271 on traite le cas de transition
          if ( (FlagExternCalMode%128 == SD_BAS_EXTERNAL_CALIBRATION) &&
               ( (SP == 31) ||
                 (SP == 32) ||
                 (SP == 33) ||
                 (SP == 35) ||
                 (SP == 36)
               )
             )
          {
            if ( BIMSCSQ                                      == CMN__ERREUR
              || BIMSSSQ1                                     == CMN__ERREUR
              || BIMSSSQ2                                     == CMN__ERREUR
              || BBofFlagSpecNonQual                          == CMN__ERREUR
              || GOPSFlagPixMiss[IND2(PN,SN,SD_BAS_PN)]       == CMN__OUI
              || GDocFlagUnderOverFlow[IND2(PN,SN,SD_BAS_PN)] == CMN__OUI
              || MMcxFlagBiasCalRad[IND2(PN,SN,SD_BAS_PN)]    == CMN__OUI
              || MMcxFlagNoiseCalRad[IND2(PN,SN,SD_BAS_PN)]   == CMN__OUI
              || GHecFlagDateNOK                              == CMN__OUI
              // FT-2749
              || (GFtbFlagBBTNonQual                          == CMN__OUI)
              // ...FT-2749
              || (GOPSFltImgBBMiss                            == CMN__OUI && (IasiTime - GOPSDatImgBBMiss    > DurFltImgBBMiss))
              || (GOPSFltImgCSMiss                            == CMN__OUI && (IasiTime - GOPSDatImgCSMiss    > DurFltImgCsMiss))
              || (GOPSFltImgEWMiss[IND1(SN)]                  == CMN__OUI && (IasiTime - GOPSDatImgEWMiss    > DurImgEWMiss))
               )
            {
              /* PM4 CR234 */
             // FT-2525
              InfoDetENG->tab_var[ENG_GQISFLAGQUAL].val.pui4[IND3(ZpdBzpd,PN,SN,SD_BAS_SB,SD_BAS_PN)] = CMN__OUI;
             // .. FT-2525
            }
            else
            {
              /* PM4 CR234 */
             // FT-2525
              InfoDetENG->tab_var[ENG_GQISFLAGQUAL].val.pui4[IND3(ZpdBzpd,PN,SN,SD_BAS_SB,SD_BAS_PN)] = CMN__NON;
             // .. FT-2525
            }
          } /* fin mode calibration externe corps noir */
          else
          {
            if ( BIMSCSQ                                      == CMN__ERREUR
              || BIMSSSQ1                                     == CMN__ERREUR
              || BIMSSSQ2                                     == CMN__ERREUR
              || BBofFlagSpecNonQual                          == CMN__ERREUR
              || GOPSFlagPixMiss[IND2(PN,SN,SD_BAS_PN)]       == CMN__OUI
              || GDocFlagUnderOverFlow[IND2(PN,SN,SD_BAS_PN)] == CMN__OUI
              || GFaxAxeNonQual[IND1(CD)]                     == CMN__OUI
              || GIsfFlagPdsNonValid[IND1(CD)]                == CMN__OUI
	          // CR 335
              // || GIacFlagCoregNonQual[IND1(SN)]               == CMN__OUI
              // || GIacFlagCoregNonValid[IND1(SN)]              == CMN__OUI
	          // FIN CR 335
              || MMcxFlagBiasCalRad[IND2(PN,SN,SD_BAS_PN)]    == CMN__OUI
              || MMcxFlagNoiseCalRad[IND2(PN,SN,SD_BAS_PN)]   == CMN__OUI
              || GHecFlagDateNOK                              == CMN__OUI
              || (GOPSFltIsrfemOff[IND1(SN)]                  == CMN__OUI && (IasiTime - GOPSDatIsrfemOff    > DurFltIsrfOff))
              // FT-2749
              || (GFtbFlagBBTNonQual                          == CMN__OUI)
              // ...FT-2749
              || (GOPSFltImgBBMiss                            == CMN__OUI && (IasiTime - GOPSDatImgBBMiss    > DurFltImgBBMiss))
              || (GOPSFltImgCSMiss                            == CMN__OUI && (IasiTime - GOPSDatImgCSMiss    > DurFltImgCsMiss))
              || (GOPSFltImgEWMiss[IND1(SN)]                  == CMN__OUI && (IasiTime - GOPSDatImgEWMiss    > DurImgEWMiss))
              || (GOPSFltRadAvhrrMiss[IND1(SN)]               == CMN__OUI && (IasiTime - GOPSDatRadAvhrrMiss > DurFltRadAvhrMiss))
              // FA 182
              || (GOPSFlaGeoAvhrrMiss[IND1(SN)]               == CMN__OUI )
              // ..FA 182
               )
            {
              /* PM4 CR234 */
              // FT-2525
              InfoDetENG->tab_var[ENG_GQISFLAGQUAL].val.pui4[IND3(ZpdBzpd,PN,SN,SD_BAS_SB,SD_BAS_PN)] = CMN__OUI;
              // .. FT-2525
            }
            /* FA 2375 */
            else
            {
              // FT-2525
              InfoDetENG->tab_var[ENG_GQISFLAGQUAL].val.pui4[IND3(ZpdBzpd,PN,SN,SD_BAS_SB,SD_BAS_PN)] = CMN__NON;
              // .. FT-2525
            }
            /* ... FA 2375 */
	    
          } /* fin mode normal ou mode calibration externe != corps noir */
          
          //@ Calcul des indices de qualite
          // CR 348
	        CoeffMoy = sqrt( (
	  	        CoeffCalRad[IND3(1,PN,SN,SD_BAS_SB,SD_BAS_PN)]*CoeffCalRad[IND3(1,PN,SN,SD_BAS_SB,SD_BAS_PN)]
          	  + CoeffCalRad[IND3(2,PN,SN,SD_BAS_SB,SD_BAS_PN)]*CoeffCalRad[IND3(2,PN,SN,SD_BAS_SB,SD_BAS_PN)]
          	  + CoeffCalRad[IND3(3,PN,SN,SD_BAS_SB,SD_BAS_PN)]*CoeffCalRad[IND3(3,PN,SN,SD_BAS_SB,SD_BAS_PN)]
	            )/3.0 );
          InfoDetENG->tab_var[ENG_GQISQUALINDEXRAD].val.pr8[IND2(PN,SN,SD_BAS_PN)] = 
	             sqrt(CoeffMoy*CoeffMoy + CovarEigenValCalRad*CovarEigenValCalRad)/CovarEigenValCalRad;
	        // ...CR 348

          //CR271 on traite le cas de transition
          if ((GIacFlagCoregNonValid[IND1(SN)] == CMN__OUI) ||
		           // CR 335
		           (GIacFlagCoregNonQual[IND1(SN)] == CMN__OUI) ||
               // FIN CR 335	  
               // FA 182
	             (GOPSFlaGeoAvhrrMiss[IND1(SN)] == CMN__OUI )  || 
               // ..FA 182
               ( (FlagExternCalMode%128 == SD_BAS_EXTERNAL_CALIBRATION) &&
                 ( (SP == 31) ||
                   (SP == 32) ||
                   (SP == 33) ||
                   (SP == 35) ||
                   (SP == 36)
                 )
               )
             )
          {
            InfoDetENG->tab_var[ENG_GQISQUALINDEXLOC].val.pr8[IND2(PN,SN,SD_BAS_PN)] = 5;
          }
          else
          {
            InfoDetENG->tab_var[ENG_GQISQUALINDEXLOC].val.pr8[IND2(PN,SN,SD_BAS_PN)] = InfoDetENG->tab_var[ENG_GIACPOSMAXQUAL].val.pr8[IND1(SN)];
          }
          
          InfoDetENG->tab_var[ENG_GQISQUALINDEXIIS].val.pr8[IND1(SN)] = sqrt(IISVarImagMean) / IISNoiseNom;
          
          /* FA132 */
          AxeRes = InfoDetENG->tab_var[ENG_GFAXAXERES].val.pr8[IND1(CD)];
	        // CR 348
          Tr =  InfoDetENG->tab_var[ENG_GQISQUALINDEXRAD].val.pr8[IND2(PN,SN,SD_BAS_PN)]
                * QualIndexSpect;
          InfoDetENG->tab_var[ENG_GQISQUALINDEX].val.pr8[IND2(PN,SN,SD_BAS_PN)] = Tr;
      	  // ...CR 348

        }/* fin si SN valide */
         
        // FT-2525

        //@ Copie du flag de qualite generale du produit (FlagQual) dans les deux autres bandes
        if (SD_BAS_B1 != IND1(ZpdBzpd))
        {
          InfoDetENG->tab_var[ENG_GQISFLAGQUAL].val.pui4[IND3(SD_BAS_B1+1,PN,SN,SD_BAS_SB,SD_BAS_PN)] =
            InfoDetENG->tab_var[ENG_GQISFLAGQUAL].val.pui4[IND3(ZpdBzpd,PN,SN,SD_BAS_SB,SD_BAS_PN)];
        }    
        if (SD_BAS_B2 != IND1(ZpdBzpd))
        {
          InfoDetENG->tab_var[ENG_GQISFLAGQUAL].val.pui4[IND3(SD_BAS_B2+1,PN,SN,SD_BAS_SB,SD_BAS_PN)] =
            InfoDetENG->tab_var[ENG_GQISFLAGQUAL].val.pui4[IND3(ZpdBzpd,PN,SN,SD_BAS_SB,SD_BAS_PN)];
        }    
        if (SD_BAS_B3 != IND1(ZpdBzpd))
        {
          InfoDetENG->tab_var[ENG_GQISFLAGQUAL].val.pui4[IND3(SD_BAS_B3+1,PN,SN,SD_BAS_SB,SD_BAS_PN)] =
            InfoDetENG->tab_var[ENG_GQISFLAGQUAL].val.pui4[IND3(ZpdBzpd,PN,SN,SD_BAS_SB,SD_BAS_PN)];
        }    
        // FT-2527
        cr_op = sub_45_QIS(
                ctx, 
                ConfStable, 
                ConfAutre, 
                InfoDetENG, 
                SN, 
                PN, 
                &GOPSFlagSpikNZpdBzpd,
                &BSpkFlagSpikB1,
                &BSpkFlagSpikB2,
                &BSpkFlagSpikB3,
                &BSpkFlagSpikZpdBzpd,
                &BZpdFlagNZpdNonQual,
                &BArcFlagCalSpectNonQualB1,
                &BArcFlagCalSpectNonQualB2,
                &BArcFlagCalSpectNonQualB3        
                );
        // ..FT-2527
 
        //@ Calcul du flag detaille de la qualite generale du produit (FlagQualDetailed)
        if ( (FlagExternCalMode%128 == SD_BAS_EXTERNAL_CALIBRATION) && ((SN == 1) || (SN == 2) || (SN == 30)) )
        {
          // Tous les flags = CMN__OUI
          InfoDetENG->tab_var[ENG_GQISFLAGQUALDETAILED].val.pui4[IND2(PN,SN,SD_BAS_PN)] = 0xFFFF;
        }
        else
        {
          if ( (FlagExternCalMode%128 == SD_BAS_EXTERNAL_CALIBRATION) &&
                 ( (SP == 31) || (SP == 32) || (SP == 33) || (SP == 35) || (SP == 36) ) )
          {
            //@ Generation de l'indicateur de qualite FlagQualDetailed(PN,SN) sur visees corps noirs (en ExternalCal)
            //@ Affectation du bit 7 (Calibration spectrale)
            //FlagQualDetailed |= 1 << 7;   // 1 = CMN__OUI
          }
          else
          {
            //@ Generation de l'indicateur qualite FlagQualDetailed(PN,SN) sur visees atmospheriques (Ext.Cal ou Normal OP.)
            //@ Affectation du bit 7 (Calibration spectrale)
            if ((GFaxAxeNonQual[IND1(CD)] == CMN__OUI) ||
                (GIsfFlagPdsNonValid[IND1(CD)] == CMN__OUI) ||
                ((GOPSFltIsrfemOff[IND1(SN)] == CMN__OUI) && (IasiTime - GOPSDatIsrfemOff    > DurFltIsrfOff)))
            {
              FlagQualDetailed |= 1 << 7;   // CMN__OUI
            }
            else
            {
              FlagQualDetailed &= ~(1 << 7);   // CMN__NON   
            }
          } 
          // Flags commun a la Generation de l'indicateur de qualite FlagQualDetailed(PN,SN) sur visees corps noirs (en ExternalCal)
          //@ et a la generation de l'indicateur qualite FlagQualDetailed(PN,SN) sur visees atmospheriques (Ext.Cal ou Normal OP.)

          //@ Affectation du bit 0 (Hardware)
          if ( (GOPSFlagPixMiss[IND2(PN,SN,SD_BAS_PN)] == CMN__NON) && 		//@ ..FT-3296 -- Evite la levée du flag en cas de pixel manquant
          			((BIMSCSQ == CMN__ERREUR) || (BIMSSSQ1 == CMN__ERREUR) || (BIMSSSQ2 == CMN__ERREUR))) 
          {
            FlagQualDetailed |= 1 << 0;   // CMN__OUI
          }
          else
          {
            FlagQualDetailed &= ~(1 << 0);   // CMN__NON
	      }
      
          //@ Affectation du bit 1 (Spike B1)
          if (BSpkFlagSpikB1 == CMN__ERREUR) 
          {
            FlagQualDetailed |= 1 << 1;   // CMN__OUI   
          }
          else
          {
            FlagQualDetailed &= ~(1 << 1);   // CMN__NON   
          }
        
          //@ Affectation du bit 2 (Spike B2)
          if (BSpkFlagSpikB2 == CMN__ERREUR)
          {
            FlagQualDetailed |= 1 << 2;   // CMN__OUI   
          }
          else
          {
            FlagQualDetailed &= ~(1 << 2);   // CMN__NON   
          }

          //@ Affectation du bit 3 (Spike B3)
          if (BSpkFlagSpikB3 == CMN__ERREUR) 
          {
            FlagQualDetailed |= 1 << 3;   // CMN__OUI   
          }
          else
          {
            FlagQualDetailed &= ~(1 << 3);   // CMN__NON   
          }
        
          //@ Affectation du bit 4 (Calibration radiometrique)
          if ((BZpdFlagNZpdNonQual == CMN__ERREUR) || 
              (BArcFlagCalSpectNonQualB1 == CMN__ERREUR ) ||
              (BArcFlagCalSpectNonQualB2 == CMN__ERREUR ) ||
              (BArcFlagCalSpectNonQualB3 == CMN__ERREUR )) 
          {
            FlagQualDetailed |= 1 << 4;   // CMN__OUI   
          }
          else
          {
            FlagQualDetailed &= ~(1 << 4);   // CMN__NON   
          }

          //@ Affectation du bit 5 (Qualite bord)
          if (BBofFlagSpecNonQual == CMN__ERREUR) 
          {
            FlagQualDetailed |= 1 << 5;   // CMN__OUI   
          }
          else
          {
            FlagQualDetailed &= ~(1 << 5);   // CMN__NON   
          }
        
          //@ Affectation du bit 6 (Over/Underflows)
          if (GDocFlagUnderOverFlow[IND2(PN,SN,SD_BAS_PN)] == CMN__OUI) 
          {
            FlagQualDetailed |= 1 << 6;   // CMN__OUI   
          }
          else
          {
            FlagQualDetailed &= ~(1 << 6);   // CMN__NON   
          }
            
          //@ Affectation du bit 8 (Post-calibration radiometrique)
          if ((InfoDetENG->tab_var[ENG_GFTBFLAGBBTNONQUAL].val.pui4[0] == CMN__OUI) || 
              (InfoDetENG->tab_var[ENG_GHECFLAGDATENOK].val.pui4[0] == CMN__OUI))  
          {
            FlagQualDetailed |= 1 << 8;   // CMN__OUI   
          }
          else
          {
            FlagQualDetailed &= ~(1 << 8);   // CMN__NON   
          }
        
          //@ Affectation du bit 9 (Qualite generale)
          if ((InfoDetENG->tab_var[ENG_GQISFLAGQUAL].val.pui4[IND3(SD_BAS_B1+1,PN,SN,SD_BAS_SB,SD_BAS_PN)] == CMN__OUI) ||
              (InfoDetENG->tab_var[ENG_GQISFLAGQUAL].val.pui4[IND3(SD_BAS_B2+1,PN,SN,SD_BAS_SB,SD_BAS_PN)] == CMN__OUI) ||
              (InfoDetENG->tab_var[ENG_GQISFLAGQUAL].val.pui4[IND3(SD_BAS_B3+1,PN,SN,SD_BAS_SB,SD_BAS_PN)] == CMN__OUI)) 
          {
            FlagQualDetailed |= 1 << 9;   // CMN__OUI   
          }
          else
          {
            FlagQualDetailed &= ~(1 << 9);   // CMN__NON   
          }
        
          //@ Affectation du bit 10 (Donnee sondeur manquante)
	      if (GOPSFlagPixMiss[IND2(PN,SN,SD_BAS_PN)] == CMN__OUI)
          {
            FlagQualDetailed |= 1 << 10;   // CMN__OUI   
          }
          else
          {
            FlagQualDetailed &= ~(1 << 10);   // CMN__NON   
          }
        
          //@ Affectation du bit 11 (Donnee IIS manquante)
          if ((GOPSFltImgEWMiss[IND1(SN)] == CMN__OUI) && 
              (IasiTime - GOPSDatImgEWMiss    > DurImgEWMiss)) 
          {
            FlagQualDetailed |= 1 << 11;   // CMN__OUI   
          }
          else
          {
            FlagQualDetailed &= ~(1 << 11);   // CMN__NON   
          }
        
          //@ Affectation du bit 12 (Donnee Avhrr manquante)
          if (((GOPSFltRadAvhrrMiss[IND1(SN)] == CMN__OUI) && 
               (IasiTime - GOPSDatRadAvhrrMiss > DurFltRadAvhrMiss)) ||
              (GOPSFlaGeoAvhrrMiss[IND1(SN)] == CMN__OUI))
          {
            FlagQualDetailed |= 1 << 12;   // CMN__OUI   
          }
          else
          {
            FlagQualDetailed &= ~(1 << 12);   // CMN__NON   
          }
          InfoDetENG->tab_var[ENG_GQISFLAGQUALDETAILED].val.pui4[IND2(PN,SN,SD_BAS_PN)] = FlagQualDetailed; 
          // FT-2566
          InfoDetN1ABC->tab_var[N1C_GQISFLAGQUALDETAILED].val.pui4[IND2(PN,SN,SD_BAS_PN)] = FlagQualDetailed; 
          // ...FT-2566
        }
        // ..FT-2525
      }//@ Fin de boucle sur les pixels
    }//@ Fin de boucle sur les souscycles
    
    //@ Calcul des indices de qualite moyens qui sont inseres dans les produits 1a, 1b, 1c

    //@ QualIndexRadAvg = moyenne quadratique des QualIndexRad(PN,SN) tq FlagQual(IDefZpdBzpd,PN,SN)=NON)
    //@ QualIndexAvg = moyenne quadratique des QualIndex(PN,SN) tq FlagQual(PN,SN)=NON)
    /* CR 2373 */
    //@ QualIndexLocAvg = moyenne quadratique des QualIndexLoc(PN,SN) tq (FlagQual(IDefZpdBzpd,PN,SN)=NON 
    //@                   et FlagCoregNonValid(SN)=NON et FlagCoregNonQual(SN)=NON)
    /* ... CR 2373 */
    //@ QualIndexIISAvg = moyenne quadratique des QualIndexIIS(SN) tq FlagQual(IDefZpdBzpd,PN,SN)=NON)
    QualIndexRadAvg = 0.0;
    QualIndexAvg    = 0.0;
    QualIndexLocAvg = 0.0;
    QualIndexIISAvg = 0.0;
    ct              = 0;
    /* CR 2373 */
    ct_loc          = 0;
    /* ... CR 2373 */
   
    /* PM4 CR177 */
    /* FlagQualProduit = CMN__OUI;*/
    
    //@ Boucle sur les souscycles
    for(SN=1; SN<=30; SN++)
    {
      //@ Boucle sur les pixels
      for(PN=1; PN<=4; PN++)
      {
	      // FA 210
        // FT-2525
        if (InfoDetENG->tab_var[ENG_GQISFLAGQUAL].val.pui4[IND3(ZpdBzpd,PN,SN,SD_BAS_SB,SD_BAS_PN)] == CMN__NON) 
        {
          // ..FT-2525
        
          /* FA157 */
          ct++;
          QualIndexRadAvg += InfoDetENG->tab_var[ENG_GQISQUALINDEXRAD].val.pr8[IND2(PN,SN,SD_BAS_PN)]
                             * InfoDetENG->tab_var[ENG_GQISQUALINDEXRAD].val.pr8[IND2(PN,SN,SD_BAS_PN)];
          QualIndexAvg    += InfoDetENG->tab_var[ENG_GQISQUALINDEX].val.pr8[IND2(PN,SN,SD_BAS_PN)]
                           * InfoDetENG->tab_var[ENG_GQISQUALINDEX].val.pr8[IND2(PN,SN,SD_BAS_PN)];

          /* CR 2373 */
          if ( (GIacFlagCoregNonValid[IND1(SN)] == CMN__NON) &&
		            (GIacFlagCoregNonQual[IND1(SN)] == CMN__NON) ) 
          {
              ct_loc++;
              QualIndexLocAvg += InfoDetENG->tab_var[ENG_GQISQUALINDEXLOC].val.pr8[IND2(PN,SN,SD_BAS_PN)]
                               * InfoDetENG->tab_var[ENG_GQISQUALINDEXLOC].val.pr8[IND2(PN,SN,SD_BAS_PN)];
          }
          /* ... CR 2373 */
          QualIndexIISAvg += InfoDetENG->tab_var[ENG_GQISQUALINDEXIIS].val.pr8[IND1(SN)]
                             * InfoDetENG->tab_var[ENG_GQISQUALINDEXIIS].val.pr8[IND1(SN)];
	      }
        InfoDetENG->tab_var[ENG_GQISQUALINDEXSPECT].val.pr8[IND2(PN,SN,SD_BAS_PN)] = QualIndexSpect;
        /* FIN FA157 */
	      // FIN FA 210
        
		    /* FA 1767 Rev 1 */
        /* PM4 CR177 */
        // FT-2525
        InfoDetN1ABC->tab_var[N1C_GQISFLAGQUAL].val.pui4[IND3(1,PN,SN,SD_BAS_SB,SD_BAS_PN)] =
            InfoDetENG->tab_var[ENG_GQISFLAGQUAL].val.pui4[IND3(1,PN,SN,SD_BAS_SB,SD_BAS_PN)];
        InfoDetN1ABC->tab_var[N1C_GQISFLAGQUAL].val.pui4[IND3(2,PN,SN,SD_BAS_SB,SD_BAS_PN)] = 
            InfoDetENG->tab_var[ENG_GQISFLAGQUAL].val.pui4[IND3(2,PN,SN,SD_BAS_SB,SD_BAS_PN)];
        InfoDetN1ABC->tab_var[N1C_GQISFLAGQUAL].val.pui4[IND3(3,PN,SN,SD_BAS_SB,SD_BAS_PN)] = 
            InfoDetENG->tab_var[ENG_GQISFLAGQUAL].val.pui4[IND3(3,PN,SN,SD_BAS_SB,SD_BAS_PN)];
        // ..FT-2525
        /* ...FA 1767 Rev 1 */

      }//@ Fin de boucle sur les pixels
    }//@ Fin de boucle sur les souscycles
    /* CR 2373 */
    if(ct == 0)
    {
        QualIndexRadAvg = 100.0;
        QualIndexAvg    = 100.0;
        QualIndexIISAvg = 100.0;
    }
    else
    {
        QualIndexRadAvg = sqrt(QualIndexRadAvg/ct);
        QualIndexAvg    = sqrt(QualIndexAvg/ct);
        QualIndexIISAvg = sqrt(QualIndexIISAvg/ct);
    }

    if(ct_loc == 0)
    {
    	// FT-2447
      QualIndexLocAvg = 5.0;
	    // FT-2447
    }
    else
    {
        QualIndexLocAvg = sqrt(QualIndexLocAvg/ct_loc);
    }
    /* ... CR 2373 */

    // Ecriture des sorties dans les structures adequates

    // ENG_GQISFLAGQUAL     -> fait en cours de code
    // ENG_GQISQUALINDEXRAD -> fait en cours de code
    // ENG_GQISQUALINDEX    -> fait en cours de code
    // ENG_GQISQUALINDEXLOC -> fait en cours de code
    // ENG_GQISQUALINDEXIIS -> fait en cours de code
    // ENG_GQISQUALINDEXSPECT -> fait en cours de code

    *(InfoDetN1ABC->tab_var[N1C_GQISQUALINDEXRAD].val.pr8)       = QualIndexRadAvg;
    *(InfoDetN1ABC->tab_var[N1C_GQISQUALINDEXLOC].val.pr8)       = QualIndexLocAvg;
    *(InfoDetN1ABC->tab_var[N1C_GQISQUALINDEXIIS].val.pr8)       = QualIndexIISAvg;
    *(InfoDetN1ABC->tab_var[N1C_GQISQUALINDEX].val.pr8)          = QualIndexAvg;
    /* PM4 CR177 */
    /* *(InfoDetN1ABC->tab_var[N1C_GQISFLAGQUAL].val.pui4)          = FlagQualProduit; */
    *(InfoDetN1ABC->tab_var[N1C_GQISQUALINDEXSPECT].val.pr8)     = QualIndexSpect;
    *(InfoDetN1ABC->tab_var[N1C_GQISSYSTECIISQUAL].val.pui4)      = SysTECIISQual;
    *(InfoDetN1ABC->tab_var[N1C_GQISSYSTECSONDQUAL].val.pui4)     = SysTECSondQual;

#ifdef INVESTIGATION
    // On trace l'ensemble des sorties

        // ENG_GQISFLAGQUAL
        SD_BAS_TRACE(tcProcName_F, "ENG_GQISFLAGQUAL",
                     SD_BAS_PN, SD_BAS_SNOT, 1,
                     TRACE_UINT4, TRACE_ASC,
                     InfoDetENG->tab_var[ENG_GQISFLAGQUAL].val.pui4, ctx);

        // ENG_GQISFLAGQUALDETAILED
        SD_BAS_TRACE(tcProcName_F, "ENG_GQISFLAGQUALDETAILED",
                     SD_BAS_PN, SD_BAS_SNOT, 1,
                     TRACE_UINT4, TRACE_ASC,
                     InfoDetENG->tab_var[ENG_GQISFLAGQUALDETAILED].val.pui4, ctx);

        // ENG_GQISQUALINDEXRAD
        SD_BAS_TRACE(tcProcName_F, "ENG_GQISQUALINDEXRAD",
                     SD_BAS_PN, SD_BAS_SNOT, 1,
                     TRACE_REAL8, TRACE_ASC,
                     InfoDetENG->tab_var[ENG_GQISQUALINDEXRAD].val.pr8, ctx);

        // ENG_GQISQUALINDEX
        SD_BAS_TRACE(tcProcName_F, "ENG_GQISQUALINDEX",
                     SD_BAS_PN, SD_BAS_SNOT, 1,
                     TRACE_REAL8, TRACE_ASC,
                     InfoDetENG->tab_var[ENG_GQISQUALINDEX].val.pr8, ctx);

        // ENG_GQISQUALINDEXLOC
        SD_BAS_TRACE(tcProcName_F, "ENG_GQISQUALINDEXLOC",
                     SD_BAS_PN, SD_BAS_SNOT, 1,
                     TRACE_REAL8, TRACE_ASC,
                     InfoDetENG->tab_var[ENG_GQISQUALINDEXLOC].val.pr8, ctx);

        // ENG_GQISQUALINDEXIIS
        SD_BAS_TRACE(tcProcName_F, "ENG_GQISQUALINDEXIIS",
                     SD_BAS_SNOT, 1, 1,
                     TRACE_REAL8, TRACE_ASC,
                     InfoDetENG->tab_var[ENG_GQISQUALINDEXIIS].val.pr8, ctx);

        // N1AB_GQISQUALINDEXRAD
        SD_BAS_TRACE(tcProcName_F, "N1AB_GQISQUALINDEXRAD",
                     1, 1, 1,
                     TRACE_REAL8, TRACE_ASC,
                     InfoDetN1ABC->tab_var[N1C_GQISQUALINDEXRAD].val.pr8, ctx);

        // N1AB_GQISQUALINDEXLOC
        SD_BAS_TRACE(tcProcName_F, "N1AB_GQISQUALINDEXLOC",
                     1, 1, 1,
                     TRACE_REAL8, TRACE_ASC,
                     InfoDetN1ABC->tab_var[N1C_GQISQUALINDEXLOC].val.pr8, ctx);

        // N1AB_GQISQUALINDEIIS
        SD_BAS_TRACE(tcProcName_F, "N1AB_GQISQUALINDEXIIS",
                     1, 1, 1,
                     TRACE_REAL8, TRACE_ASC,
                     InfoDetN1ABC->tab_var[N1C_GQISQUALINDEXIIS].val.pr8, ctx);

        // N1AB_GQISQUALINDEX
        SD_BAS_TRACE(tcProcName_F, "N1AB_GQISQUALINDEX",
                     1, 1, 1,
                     TRACE_REAL8, TRACE_ASC,
                     InfoDetN1ABC->tab_var[N1C_GQISQUALINDEX].val.pr8, ctx);

        // N1C_GQISFLAGQUAL
        SD_BAS_TRACE(tcProcName_F, "N1C_GQISFLAGQUAL",
                     SD_BAS_PN, SD_BAS_SNOT, 1,
                     TRACE_UINT4, TRACE_ASC,
                     InfoDetN1ABC->tab_var[N1C_GQISFLAGQUAL].val.pui4, ctx);

        // N1C_GQISFLAGQUALDETAILED
        SD_BAS_TRACE(tcProcName_F, "N1C_GQISFLAGQUALDETAILED",
                     SD_BAS_PN, SD_BAS_SNOT, 1,
                     TRACE_UINT4, TRACE_ASC,
                     InfoDetENG->tab_var[N1C_GQISFLAGQUALDETAILED].val.pui4, ctx);
#endif

fin:
  // sortie de la fonction
  return( cr_op);
}

// FT-2526
//@ --------------------------------------------------------------------------*/
//@ Procedure : SD_ALG_46_HIP
//@
//@ Description : Homogeneite inter-pixel du NZPD
//@ 
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape numero de ligne traitee
//@         (ENTREE)  ConfStable : fichier de conf sol (GRD)
//@         (ENTREE)  InfoDetVER : info det2 du produit N1
//@         (ENTREE/SORTIE)  InfoDetENG : info det du produit N1
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ --------------------------------------------------------------------------*/

t_CMN__Statut
SD_ALG_46_HIP(
        t_SD_BAS_DataThread *ctx,
        t_SD_BAS_ConfStable *ConfStable,
        void *InfoDetVER,
        t_SD_BAS_Liste_Var *InfoDetENG
)
{
  static char tcProcName_F[] = "SD_ALG_46_HIP";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  // Entree dans la fonction


  int                PN;                 //@ Compteur sur les PN
  int                CD;                 //@ Indicateur de CCD
  int                SN;                 //@ compteur sur les SN
  int                SNmin;
  int                SNmax;
  int                compteur = 0;
  int                compteur32 = 0;
  int                compteur33 = 0;
  int                compteur35 = 0;
  int                compteur36 = 0;
  int                index;
  int                indexPixRef;
  int                *InterPixNZpd;        //@ Vecteur nominal des differences NZPD inter-pixel
  int                PixRef;               //@ pixel de reference
  unsigned int     FlagExternCalMode;      //@ Indicateur de mode calibration externe
  unsigned short int *ptrInfoDetVER;
  unsigned short int word1;
  unsigned short int word2;
  
  // initialisations
  InterPixNZpd       = ConfStable->IDefInterPixNZpd;
  PixRef             = ConfStable->IDefPixRef; 
  FlagExternCalMode  = InfoDetENG->tab_var[ENG_INSTRUMENT_MODE].val.pui4[0];

  if (FlagExternCalMode%128 == SD_BAS_NORMAL)
  {
    SNmin = 1;
    SNmax = 30;
  }
  else
  {
    SNmin = 3;
    SNmax = 29;
  }   
  //@ initialisations des donnees en sortie
  for (SN=1; SN<=(SD_BAS_SNOT+4); SN++)
  {
    for (PN=1; PN <= SD_BAS_PN; PN++)
    {
      // FT-2573
      InfoDetENG->tab_var[ENG_MHIPNZPDINTERPIXEL].val.pui4[IND2(PN,SN,SD_BAS_PN)] = SHRT_MIN; // -32764
      // ...FT-2573
      // FT-2648
      InfoDetENG->tab_var[ENG_MHIPFLAGINTERPIXNZPDNONQUAL].val.pui4[IND2(PN,SN,SD_BAS_PN)] = CMN__OUI;
      // ...FT-2648
    }
  }

  //@ calcul sur les PX : SN 1 ... 30 ou 3 ... 29
  for (SN=SNmin; SN <= SNmax; SN++) 
  {
    // CD=CCD(SN)
    CD = InfoDetENG->tab_var[ENG_GEPS_CCD].val.pui4[IND1(SN)] + 1;
    compteur = 0;
    
    for (PN = 1; PN <= SD_BAS_PN; PN++)
    {
      // Calcul des differences inter-pixel - BZpdNZpd = Word 29 d'un paquet PX du N0
      InfoDetENG->tab_var[ENG_MHIPNZPDINTERPIXEL].val.pui4[IND2(PN,SN,SD_BAS_PN)] =
          InfoDetENG->tab_var[ENG_DATA_PX].val.pui4[25*SD_BAS_PN*SD_BAS_SNOT + IND2(PN,SN,SD_BAS_PN)]-
          InfoDetENG->tab_var[ENG_DATA_PX].val.pui4[25*SD_BAS_PN*SD_BAS_SNOT + IND2(PixRef,SN,SD_BAS_PN)]; 

      //@ Test par rapport a la valeur nominal
      if (InfoDetENG->tab_var[ENG_MHIPNZPDINTERPIXEL].val.pui4[IND2(PN,SN,SD_BAS_PN)] != InterPixNZpd[IND2(PN,CD,SD_BAS_PN)]) 
      {
        compteur++; 
      }
    }
    if (compteur == 0)
    {
      // FT-2648
      InfoDetENG->tab_var[ENG_MHIPFLAGINTERPIXNZPDNONQUAL].val.pui4[IND2(1,SN,SD_BAS_PN)] = CMN__NON;
      // ...FT-2648
    }   

  }
          
  //@ calcul pour les SN 32,33,35,36 dans le paquet AP (cf IMVD Issue 2.10)
  // FT-2638
  //@ word  71 = PX1 ; PX1 + 0= SN32; PX1 + 15 = SN33; PX1 + 45 = SN35; PX1 + 60 = SN36 ; SNx+2(B0)=CD; SNx+10=BZPDNZPD
  //@ word 131 = PX2 ; PX2 + 0= SN32; PX2 + 15 = SN33; PX2 + 45 = SN35; PX2 + 60 = SN36 ; SNx+2(B0)=CD; SNx+10=BZPDNZPD
  //@ word 191 = PX3 ; PX3 + 0= SN32; PX3 + 15 = SN33; PX3 + 45 = SN35; PX3 + 60 = SN36 ; SNx+2(B0)=CD; SNx+10=BZPDNZPD
  //@ word 251 = PX4 ; PX4 + 0= SN32; PX4 + 15 = SN33; PX4 + 45 = SN35; PX4 + 60 = SN36 ; SNx+2(B0)=CD; SNx+10=BZPDNZPD
  //@
  ptrInfoDetVER      = (unsigned short int *)InfoDetVER + IND1(71); // pour travailler sur les mots du N0 -
  // ...FT-2638 
  compteur32 = 0;
  compteur33 = 0;
  compteur35 = 0;
  compteur36 = 0;
  for (PN = 1; PN <= SD_BAS_PN; PN++)
  {
    // CD=CCD(SN)
    index = (PN-1)*60+2;
    // FT-2765
    word1 = ptrInfoDetVER[index];
    SWAP_INT_BUFFER(&word1, sizeof(word1), 1);
    CD = ((word1 & 0x8000) >> 15) + 1; // bit 0 = CD (Cube direction)
    // ...FT-2765
	
    // Calcul des differences inter-pixel pour SN = 32
    index = (PN-1)*60+10;
    indexPixRef = (PixRef-1)*60+10;
    
    // FT-2765
    word1 = ptrInfoDetVER[index];
    SWAP_INT_BUFFER(&word1, sizeof(word1), 1);
	word2 = ptrInfoDetVER[indexPixRef];
    SWAP_INT_BUFFER(&word2, sizeof(word2), 1);
    InfoDetENG->tab_var[ENG_MHIPNZPDINTERPIXEL].val.pui4[IND2(PN,SD_BAS_SNOT+1,SD_BAS_PN)] = word1 - word2;
    // ...FT-2765
    //@ Test par rapport a la valeur nominal
    if (InfoDetENG->tab_var[ENG_MHIPNZPDINTERPIXEL].val.pui4[IND2(PN,SD_BAS_SNOT+1,SD_BAS_PN)] != InterPixNZpd[IND2(PN,CD,SD_BAS_PN)]) 
    {
      compteur32++; 
    }
	
    // CD=CCD(SN)
    index = (PN-1)*60+2+15;
    // FT-2765
    word1 = ptrInfoDetVER[index];
    SWAP_INT_BUFFER(&word1, sizeof(word1), 1);
    CD = ((word1 & 0x8000) >> 15) + 1; // bit 0 = CD (Cube direction)
    // ...FT-2765

    // Calcul des differences inter-pixel pour SN = 33
    index = (PN-1)*60+10+15;
    indexPixRef = (PixRef-1)*60+10+15;
    // FT-2765
    word1 = ptrInfoDetVER[index];
    SWAP_INT_BUFFER(&word1, sizeof(word1), 1);
	word2 = ptrInfoDetVER[indexPixRef];
    SWAP_INT_BUFFER(&word2, sizeof(word2), 1);
    InfoDetENG->tab_var[ENG_MHIPNZPDINTERPIXEL].val.pui4[IND2(PN,SD_BAS_SNOT+2,SD_BAS_PN)] = word1 - word2;
    // ...FT-2765
    //@ Test par rapport a la valeur nominal
    if (InfoDetENG->tab_var[ENG_MHIPNZPDINTERPIXEL].val.pui4[IND2(PN,SD_BAS_SNOT+2,SD_BAS_PN)] != InterPixNZpd[IND2(PN,CD,SD_BAS_PN)]) 
    {
      compteur33++; 
    }
	
    // CD=CCD(SN)
    index = (PN-1)*60+2+30;
    // FT-2765
    word1 = ptrInfoDetVER[index];
    SWAP_INT_BUFFER(&word1, sizeof(word1), 1);
    CD = ((word1 & 0x8000) >> 15) + 1; // bit 0 = CD (Cube direction)
    // ...FT-2765
	
    // Calcul des differences inter-pixel pour SN = 35
    index = (PN-1)*60+10+30;
    indexPixRef = (PixRef-1)*60+10+30;
    // FT-2765
    word1 = ptrInfoDetVER[index];
    SWAP_INT_BUFFER(&word1, sizeof(word1), 1);
	word2 = ptrInfoDetVER[indexPixRef];
    SWAP_INT_BUFFER(&word2, sizeof(word2), 1);
    InfoDetENG->tab_var[ENG_MHIPNZPDINTERPIXEL].val.pui4[IND2(PN,SD_BAS_SNOT+3,SD_BAS_PN)] = word1 - word2;
    // ...FT-2765
    //@ Test par rapport a la valeur nominal
    if (InfoDetENG->tab_var[ENG_MHIPNZPDINTERPIXEL].val.pui4[IND2(PN,SD_BAS_SNOT+3,SD_BAS_PN)] != InterPixNZpd[IND2(PN,CD,SD_BAS_PN)]) 
    {
      compteur35++; 
    }

    // CD=CCD(SN)
    index = (PN-1)*60+2+45;
    // FT-2765
    word1 = ptrInfoDetVER[index];
    SWAP_INT_BUFFER(&word1, sizeof(word1), 1);
    CD = ((word1 & 0x8000) >> 15) + 1; // bit 0 = CD (Cube direction)
    // ...FT-2765
	
    // Calcul des differences inter-pixel pour SN = 36
    index = (PN-1)*60+10+45;
    indexPixRef = (PixRef-1)*60+10+45;
    // FT-2765
    word1 = ptrInfoDetVER[index];
    SWAP_INT_BUFFER(&word1, sizeof(word1), 1);
	word2 = ptrInfoDetVER[indexPixRef];
    SWAP_INT_BUFFER(&word2, sizeof(word2), 1);
    InfoDetENG->tab_var[ENG_MHIPNZPDINTERPIXEL].val.pui4[IND2(PN,SD_BAS_SNOT+4,SD_BAS_PN)] = word1 - word2;
    // ...FT-2765
    //@ Test par rapport a la valeur nominal
    if (InfoDetENG->tab_var[ENG_MHIPNZPDINTERPIXEL].val.pui4[IND2(PN,SD_BAS_SNOT+4,SD_BAS_PN)] != InterPixNZpd[IND2(PN,CD,SD_BAS_PN)]) 
    {
      compteur36++; 
    }
  }
  if (compteur32 == 0)
  {
    // FT-2648
    InfoDetENG->tab_var[ENG_MHIPFLAGINTERPIXNZPDNONQUAL].val.pui4[IND2(1,(SD_BAS_SNOT+1),SD_BAS_PN)] = CMN__NON;
    // ...FT-2648
  }   
  if (compteur33 == 0)
  {
    // FT-2648
    InfoDetENG->tab_var[ENG_MHIPFLAGINTERPIXNZPDNONQUAL].val.pui4[IND2(1,(SD_BAS_SNOT+2),SD_BAS_PN)] = CMN__NON;
    // ...FT-2648
  }   
  if (compteur35 == 0)
  {
    // FT-2648
    InfoDetENG->tab_var[ENG_MHIPFLAGINTERPIXNZPDNONQUAL].val.pui4[IND2(1,(SD_BAS_SNOT+3),SD_BAS_PN)] = CMN__NON;
    // ...FT-2648
  }   
  if (compteur36 == 0)
  {
    // FT-2648
    InfoDetENG->tab_var[ENG_MHIPFLAGINTERPIXNZPDNONQUAL].val.pui4[IND2(1,(SD_BAS_SNOT+4),SD_BAS_PN)] = CMN__NON;
    // ...FT-2648
  }   
  // FT-2648
  for (SN=1; SN<=(SD_BAS_SNOT+4); SN++)
  {
    for (PN=2; PN <= SD_BAS_PN; PN++)
    {
      InfoDetENG->tab_var[ENG_MHIPFLAGINTERPIXNZPDNONQUAL].val.pui4[IND2(PN,SN,SD_BAS_PN)] = 
              InfoDetENG->tab_var[ENG_MHIPFLAGINTERPIXNZPDNONQUAL].val.pui4[IND2(1,SN,SD_BAS_PN)];
    }
  }
  // ...FT-2648


  // sortie de la fonction
  return(cr_op);
}
// FT-2526
//======================================================================
//                  FONCTIONS INTERNES
//======================================================================


