//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/ALG/src/SD_ALG_Monitoring.h,v 1.3 2011/11/25 10:17:44 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_ALG
//@ MODULE : SD_ALG_Monitoring
//@
//@ Comment :
//@    include file for the SD_ALG_Monitoring class which includes the
//@    Monitoring algorithm for IASI L1 chain
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 04/03/2003 : Pierre DHERETE
//@ Versions 100_EXS, 110_DPT, 111_MCX stables
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2525(DM-2100-OPS) : 06/03/2009 : S. BRUNEL
//@ Modification du GQisFlagQual, ajout d'informations sur la cause de
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2526(DM-2100-OPS) : 06/03/2009 : S. BRUNEL
//@ Include les differences de NZPD interpixel dans le produit 
//@ ENG (Day 2,point 7)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2527(DM-2100-OPS) : 06/03/2009 : S. BRUNEL
//@ Permettre le calcul des spectres dans les bandes B1 & B2 meme
//@ lorsque un spike affecte la bande B3 (Day 2,point 8)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_ALG_Monitoring_h

#define SD_ALG_Monitoring_h


// inserer ici les fichiers include
#include <stdio.h>       /* entree/sortie */
#include <stdlib.h>      /* alloc memoire, ...*/
#include <string.h>      /* chaines de carateres */
#include <unistd.h>      /* gestion pipes, ... */
#include <time.h>        /* gestion de la date */
#include <sys/param.h>   /* constantes systeme */
#include <math.h>        /* Fonctions et constantes math */

#include "CMN__Commun.h"
#include "SD_BAS_Global.h"
#include "SD_BAS_Contexte.h"
#include "SD_BAS_ConfAutre.h"
#include "SD_BAS_Produits.h"
#include "SD_BAS_ConfStable.h"
#include "SD_BAS_OperateurBase.h"
#include "SD_BAS_Niveau1.h"
#include "SD_BAS_Niveau0.h"

/*====================================================================*/
//   External define  Declarations
/*====================================================================*/
#define SD_ALG_ERR_SET_MONITORING           1
#define ERR_BASE_MONITORING                 SD_ALG_ERR_BASE_MONITORING
#define SD_ALG_MONITORING_ERR_NFIRST_NLAST  (ERR_BASE_MONITORING+1)
#define SD_ALG_MONITORING_ERR_NFIRST_NEG    (ERR_BASE_MONITORING+2)
#define SD_ALG_MONITORING_ERR_NB_ECH_SPECT  (ERR_BASE_MONITORING+3)

/*====================================================================*/
// Constant Declaration
/*====================================================================*/
//const int  xxx_yyy_MACONSTANTE;

/*====================================================================*/
// Exported types Declaration
/*====================================================================*/
/*
  typedef struct
    {
     int itest;
     double dtest;
     float ftest;
     char  ctest;
     char *pctest;
     double *pdtest;
     int    *pitest;
    } t_xxx_yyy_Montype;
*/
/*====================================================================*/
// External variables Declaration
/*====================================================================*/
//int xxx_yyy_MAVARIABLEGLOBALE

#ifdef __cplusplus
extern "C" {
#endif

/*====================================================================*/
//                  FONCTIONS EXTERNES
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_100_EXS
//@
//@ Description : Mise a jour des temperatures de brillance min et max
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape numero de ligne sn pn traitee
//@         (ENTREE)  contexte_courant : contexte des granules
//@         (ENTREE)  conf_autre : fichier de conf autre
//@         (ENTREE/SORTIE) data_thread_produit : pour les tableaux temporaires
//@         (ENTREE)  produit_techno : produit ENG dans lequel
//@                   se trouvent les spectres min et max 
//@                   sousechantillonnes d'un facteur 16 (550 echantillons)
//@         (ENTREE)  spectre : spectre decode pour un ISP
//@                   taille = conf_autre->IDefNslast[SD_BAS_B3] - 
//@                            conf_autre->IDefNsfirst[SD_BAS_B1] + 1
//@         (SORTIE)  spectre_froid : spectre froid mis a jour
//@                   taille = SD_BAS_SPCT_COD
//@         (SORTIE)  spectre_chaud : spectre froid mis a jour
//@                   taille = SD_BAS_SPCT_COD
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_100_EXS(
  t_SD_BAS_DataThread *        ctx,
  t_SD_BAS_Contexte *          contexte_courant,
  t_SD_BAS_ConfAutre *         conf_autre,
  t_SD_BAS_DataThreadProduit * data_thread_produit, 
//  t_SD_BAS_Liste_Var           produit_techno, 
  t_SD_BAS_LigneN1 *   		LigneVIADRENG,
  double *                     spectre) 
//  unsigned int *               spectre_froid,
//  unsigned int *               spectre_chaud)
;


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_110_DPT
//@
//@ Description : calcul table pixel morts
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@          numero de ligne traitee
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (ENTREE)  ConfStable : fichier de conf stable
//@         (ENTREE)  ImgCS_36 : image espace froid ( SN=36)
//@         (Entree/Sortie)  InfoDet : info det du produit ENG
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_110_DPT(
        t_SD_BAS_DataThread  *ctx,
    t_SD_BAS_Contexte    *contexte_courant,
        t_SD_BAS_ConfAutre   *ConfAutre,
        t_SD_BAS_ConfStable  *ConfStable,
        double               *ImgCS_36,
    double               *VarImag,
        t_SD_BAS_Liste_Var   InfoDet_ENG
//        t_SD_BAS_Liste_Var   InfoDet_VIADRENG
);


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_111_MCX
//@
//@ Description : surveillance de la calibration radiometrique
//@       complexe.
//@ remarque: on peut ici envisager de traiter toutes les bandes en un appel
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@          numero de ligne sn pn sb traitee
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (Entree)  InfoN0 :  paquet de N0 PX
//@         (Sortie)  InfoDet : info det du produit ENG
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_111_MCX(
        t_SD_BAS_DataThread  *ctx,
        t_SD_BAS_ConfAutre   *ConfAutre,
        t_SD_BAS_Liste_Var   InfoN0,
        t_SD_BAS_Liste_Var   InfoDet,
    double               *Spect            // Spectre calibre niveau 0 Spect=GDocSpect( PN,SN,LN )
);

// FT-2525 / FT-2527
//@ ------------------------------------------------------------------*/
//@ Procedure : sub_45_QIS
//@
//@ Description : Detection d'un spike uniquement dans la bande 
//@               spectrale de determination du NZPD
//@       
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape numero de ligne traitee
//@         (ENTREE)  ConfStable : fichier de conf stable
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (ENTREE/SORTIE) InfoDetENG : info det du produit ENG
//@         (ENTREE)  sn: scan number
//@         (ENTREE)  pn: pixel number
//@         (SORTIE)  FlagSpik : flag de spike dans la bande de determination du NZPD
//@         (SORTIE)  BSpkFlagSpikB1 : flag BSpkFlagSpik for band B1
//@         (SORTIE)  BSpkFlagSpikB2 : flag BSpkFlagSpik for band B2
//@         (SORTIE)  BSpkFlagSpikB3 : flag BSpkFlagSpik for band B3
//@         (SORTIE)  BSpkFlagSpikZpdBzpd :flag BSpkFlagSpik for band ZpdBzpd
//@         (SORTIE)  BZpdFlagNZpdNonQual :
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut sub_45_QIS(
  t_SD_BAS_DataThread *ctx,
  t_SD_BAS_ConfStable *ConfStable,
  t_SD_BAS_ConfAutre  *ConfAutre,
  t_SD_BAS_Liste_Var  *InfoDetENG,
  unsigned int sn,
  unsigned int pn,
  unsigned int *GOPSFlagSpikNZpdBzpd,
  unsigned int *BSpkFlagSpikB1,
  unsigned int *BSpkFlagSpikB2,
  unsigned int *BSpkFlagSpikB3,
  unsigned int *BSpkFlagSpikZpdBzpd,
  unsigned int *BZpdFlagNZpdNonQual,
  unsigned int *BArcFlagCalSpectNonQualB1,
  unsigned int *BArcFlagCalSpectNonQualB2,
  unsigned int *BArcFlagCalSpectNonQualB3        
);
// ...FT-2525 / ...FT-2527

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_45_QIS
//@
//@ Description : Calcul des Flags et indices de qualite
//@       .
//@ remarque: on peut ici envisager de traiter toutes les bandes en un appel
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape numero de ligne traitee
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (ENTREE)  NbISPPX : nombre des paquets de N0 PX
//@         (ENTREE)  ListeInfoN0PX : Liste des paquets de N0 PX pour la ligne
//@         (ENTREE)  NbISPPX : nombre des paquets de N0 IP
//@         (ENTREE)  ListeInfoN0IP : Liste des paquets de N0 IP pour la ligne
//@         (ENTREE/SORTIE)  InfoDetENG : info det du produit ENG
//@         (ENTREE/SORTIE)  InfoDetENG : info det du produit N1
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
// FT-2525
t_CMN__Statut SD_ALG_45_QIS(
        t_SD_BAS_DataThread *ctx,
        t_SD_BAS_ConfStable  *ConfStable,
        t_SD_BAS_ConfAutre   *ConfAutre,
        t_SD_BAS_Liste_Var  *InfoDetENG,
        t_SD_BAS_Liste_Var * InfoDetN1ABC
);
// ...FT-2525

// FT-2526
//@ --------------------------------------------------------------------------*/
//@ Procedure : SD_ALG_46_HIP
//@
//@ Description : Homogeneite inter-pixel du NZPD
//@ 
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape numero de ligne traitee
//@         (ENTREE)  ConfStable : fichier de conf sol (GRD)
//@         (ENTREE)  InfoDetVER : info det2 du produit N1
//@         (ENTREE/SORTIE)  InfoDetENG : info det du produit N1
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ --------------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_46_HIP(
        t_SD_BAS_DataThread *ctx,
        t_SD_BAS_ConfStable *ConfStable,
        void *InfoDetVER,
        t_SD_BAS_Liste_Var *InfoDetENG
);
// ...FT-2526

//======================================================================
#ifdef __cplusplus
}
#endif


#endif
