//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/ALG/src/SD_ALG_Spectre.c,v 1.3 2011/11/25 10:17:44 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_ALG
//@ MODULE : SD_ALG_Spectre 
//@
//@ Comment :
//@    Code file for the  SD_ALG_Spectre class which includes the
//@    algorithm used to compute IASI L1 spectra
//@
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 29/01/2003 : Olivier DUFFEZ
//@ Premiere version qui fonctionne pour 20_DOC
//@ --------------------------------------------------------------------
//@ VERSION : 27/11/2002 : Jean-Luc PASCAL
//@ Procedure : SD_ALG_XX_Initprod deplacee de SD_ALG_Image.c
//@ --------------------------------------------------------------------
//@ VERSION : 20/03/2003 : Benjamin MASSART
//@ TU OK
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-084-CN : 13/11/2003 : JL. PASCAL
//@ optimisation du code ( les init de fft  sont fait une fois par ligne );
//@ fiche de correction correspondante : IA-2100-FC-084-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-059-CN : 13/11/2003 : B. MASSART
//@ Function SD_ALG_20_DOC:definition of number max of positions of underflows 
//@ and overflows written in the engineering product. See IA-2100-FC-059-THA.
//@ --------------------------------------------------------------------
//@ PM4 : 31/03/2004 : B. MASSART
//@ 30_FTB:
//@ -ajout SD_ALG_FTBBBTRES_DEFAUT
//@ -modif test pour calcul GFtbFilteredBBT
//@ -normalisation de Dist
//@ SD_ALG_XX_Initprod:
//@ -date UTC de la ligne SN=15
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include "SD_ALG_Spectre.h"

#include <stdio.h>
#include <math.h>

#include "CMN__Commun.h"

#ifdef INVESTIGATION
#include "SD_BAS_Traces.h"
#endif

/*====================================================================*/
/* Local Constant Declaration */
/*====================================================================*/
static const int   SD_ALG_INIT_SPLINE      = 0;
static const int   SD_ALG_CALCUL_SPLINE    = 4;

static const int NB_BITS_OCTET       = 8;
static const int SD_ALG_INIT_FFT     = 1;
static const int SD_ALG_CALCUL_FFT   = 0;
static const double SD_ALG_BBT_REF   = 293.;

#ifdef OPS_DEBUG
static const char REP_TU[] = "SD/ALG/tu/TU_SD_ALG_Spectre";
#endif

/* DM 11/03/2004 */
const double SD_ALG_FTBBBTRES_DEFAUT = -2.0;

/*====================================================================*/
/* Local types Declaration  */
/*====================================================================*/
typedef enum
{
  SD_ALG_SPECTRE_30_FTB_ALRT_MOINDRES_CARRES        = ERR_BASE_SPECTRE + 1, 
  SD_ALG_SPECTRE_35_S1B_ALRT_INIT_SPLINE            = ERR_BASE_SPECTRE + 2, 
  SD_ALG_SPECTRE_35_S1B_ALRT_INTERP_SPLINE          = ERR_BASE_SPECTRE + 3, 
  SD_ALG_SPECTRE_37_S1C_ALRT_FCT_AMORTISST          = ERR_BASE_SPECTRE + 4, 
  SD_ALG_SPECTRE_37_S1C_ALRT_ZERO_PADDING           = ERR_BASE_SPECTRE + 5, 
  SD_ALG_SPECTRE_37_S1C_ALRT_INIT_FFT_DIRECTE       = ERR_BASE_SPECTRE + 6, 
  SD_ALG_SPECTRE_37_S1C_ALRT_CALCUL_FFT_DIRECTE     = ERR_BASE_SPECTRE + 7, 
  SD_ALG_SPECTRE_37_S1C_ALRT_PREPARE_INTERFEROGRAM  = ERR_BASE_SPECTRE + 8, 
  SD_ALG_SPECTRE_37_S1C_ALRT_INIT_FFT_INVERSE       = ERR_BASE_SPECTRE + 9, 
  SD_ALG_SPECTRE_37_S1C_ALRT_CALCUL_FFT_INVERSE     = ERR_BASE_SPECTRE + 10, 
  SD_ALG_SPECTRE_37_S1C_ALRT_ARRANGT_SPECTRE_1C     = ERR_BASE_SPECTRE + 11, 
  SD_ALG_SPECTRE_XX_INITPROD_ALRT_INTERP_FCS        = ERR_BASE_SPECTRE + 12, 
  SD_ALG_SPECTRE_XX_INITPROD_ALRT_INTERP_PLANCKBBT  = ERR_BASE_SPECTRE + 13, 
  SD_ALG_SPECTRE_XX_INITPROD_ALRT_INTERP_FCS_SURECH = ERR_BASE_SPECTRE + 14
} e_CodeRetour;

//======================================================================
// Internal fonctions Declaration
//======================================================================

//======================================================================
// Global variables Definition
//======================================================================
/* Pour la gestion des messages */
char            SD_ALG_SPECTRE_ValeurParam[SD_ALG_SPECTRE_LG_VALEUR_PARAM];
char            SD_ALG_SPECTRE_Libelle_Erreur[SD_BAS_LGLIBERR];
int             SD_ALG_SPECTRE_MsgSet;
int             SD_ALG_SPECTRE_MsgNum;

//======================================================================
//          FONCTIONS EXTERNES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_20_DOC
//@
//@ Description : algorithme de decodage des spectres de niveau 0 a 
//@ appeler apres DecodeDataLigne0
//@ pour les modes intermediaires le decodage est effectue directement dans 
//@ SD_DON_GestDonnees::EncodeDecodeLigneS1 et cette fonction n'est pas appelee
//@
//@
//@ Parameters :
//@     (ENTREE)  ctx : contexte :numero de ligne SN et PN traite
//@     (ENTREE)  conf_autre : fichier de conf autre
//@     (ENTREE)  ctx_appel : identificateur de la chaine 
//@                           (SD_BAS_ISRFEM ou SD_BAS_PRODUIT)
//@     (ENTREE)  CodSpect  : spectre code (flot continu de bits, apres
//@                           rearrangement par SD_BAS_DecodeSpectreN0)
//@                           donnees decodees allouee par
//@                           l'appelant et issue de 
//@                           SD_DON_GestDonnees::EncodeDecodeLigneS1
//@                           (en principe les mots on ete rearranges et 
//@                           compactes)(AC)
//@     (SORTIE)  Spect       : spectre decode pour un ISP (GDocFen(PN,SN,LN)
//@     (SORTIE)  InfoDet1ENG : info det du produit ENG 
//@     (SORTIE)  validite_bandes_spectrales : contient SD_BAS_SB booleens 
//@               qui indiquent si chaque bande spectrale contient des overflow 
//@               ou des underflow
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_20_DOC(
    t_SD_BAS_DataThread * ctx,
    t_SD_BAS_ConfAutre *  conf_autre,
    int                   ctx_appel,
    const char *          CodSpect,
    double *              Spect,
    t_SD_BAS_Liste_Var *  InfoDet1ENG,
    t_CMN__Booleen *      validite_bandes_spectrales)

{
  /* Variables locales */
  /*-------------------*/
  static char tcProcName_F[] = "SD_ALG_20_DOC";
  t_CMN__Statut CodeRetour = CMN__STATUT_OK;

  /* Indice de la fenetre spectrale dans laquelle on travaille */
  int           indFS;
  /* Indice du premier echantillon spectral */
  int           Nsfirst;
  /* Indice du dernier echantillon spectral */
  int           Nslast;
  /* Nb d'echantillons du spectre */
  int           TailleSpectre;
  /* Indice du premier echantillon spectral dans la chaine ISRFEM */
  int           SWNsfirst;
  /* Indice du dernier echantillon spectral dans la chaine ISRFEM */
  int           SWNslast;
  /* numero de l'indice de l'echantillon courant */
  unsigned int  n;
  /* numero de segment de codage du spectre */
  unsigned int  nseg;
  /* valeur de l'echantillon extraite du train de bits */
  unsigned int  Cod;
  /* nombre de bits a transmettre par plage spectrale */
  unsigned int  NbBitsSeg;
  /* indice de bit pour 1 echantillon Cod */
  unsigned int  iBit;
  /* indice de bit dans CodSpect */
  unsigned int  indiceBit;
  /* indice d'octet dans CodSpect */
  unsigned int  indiceOctet;
  /* flag */
  unsigned int  V;
  /* facteur d'echelle */
  double        Scale;
  /* facteur d'offset */
  double        Offset;
  /* position du miroir de scan */
  int           SP;
  /* indice de la table de codage */
  int           CTN;
  /* numero du premier echantillon de la premiere plage */
  int           NsegfirstNsfirst;
  /* nb de plages spectrales couvrant [Nsfirst;Nslast] */
  unsigned int  NbSeg;
  /* nb d'echantillons par plage */
  int           NbNsSeg;
  /* parametres de mise a l'echelle */
  double        TabScaleM;
  double        TabScaleC;
  double        TabOffsetM;
  double        TabOffsetC;
  /* facteur d'echelle par plage */
  int *         TabScale;
  /* offset par plage */
  int *         TabOffset;
  /* nb de bits codant un echantillon par plage */
  int *         TabNbBit;
  /* Nombre d'underflow */
  int           NbUnderFlow;
  /* Position des underflow */
  int *         PosUnderFlow;
  /* Nombre d'overflow */
  int           NbOverFlow;
  /* Position des overflow */
  int *         PosOverFlow;
  /* indice de bande spectrale */
  int           iSB;
  /* indices des bornes des bandes spectrales */
  int           NsfirstSB[SD_BAS_SB];
  int           NslastSB[SD_BAS_SB];
  
#ifdef OPS_DEBUG
  FILE *            Fd;
  char              Fichier[256];
#endif

  /* Initialiser les variables locales */
  /*-----------------------------------*/
  Nsfirst = conf_autre->IDefNsfirst[SD_BAS_B1];
  Nslast  = conf_autre->IDefNslast[SD_BAS_B3];
  TailleSpectre = Nslast - Nsfirst + 1;
  
  if (ctx_appel == SD_BAS_ISRFEM) 
  {
    indFS = conf_autre->IDefSsdSpectralWindowOPS;
    SWNsfirst = conf_autre->IDefSsdNsfirstSW[IND1(indFS)];
    SWNslast  = conf_autre->IDefSsdNslastSW[IND1(indFS)];
    TailleSpectre = SWNslast - SWNsfirst + 1;
  }

  SP = (InfoDet1ENG->tab_var[ENG_GEPS_SP].val.pi4[IND1(ctx->ctxtrt.SN)]);
  CTN = 1;
  if (SP == 32)
  {
    CTN = 2;
  }
  if ( (SP == 31) || (SP == 33) )
  {
    CTN = 3;
  }
  if ( (SP == 34) || (SP == 35) )
  {
    CTN = 4;
  }
  
  NsegfirstNsfirst = conf_autre->IDefNSegNsfirst;
  NbNsSeg = conf_autre->IDefCtcNbNsSeg;
  NbSeg = (int)((double)(Nslast - NsegfirstNsfirst) / (double)NbNsSeg) + 1;
  
  TabScaleM = conf_autre->ICtcTabScaleM[IND1(CTN)];
  TabScale = conf_autre->ICtcTabScale;
  TabScaleC = conf_autre->ICtcTabScaleC[IND1(CTN)];

  TabOffsetM = conf_autre->ICtcTabOffsetM[IND1(CTN)];
  TabOffset = conf_autre->ICtcTabOffset;
  TabOffsetC = conf_autre->ICtcTabOffsetC[IND1(CTN)];

  TabNbBit = conf_autre->ICtcTabNbBit;

  PosUnderFlow = InfoDet1ENG->tab_var[ENG_GDOCPOSUNDERFLOW].val.pi4;
  PosOverFlow = InfoDet1ENG->tab_var[ENG_GDOCPOSOVERFLOW].val.pi4;
  
  for (iSB = 0 ; iSB < SD_BAS_SB ; iSB++)
  {
    NsfirstSB[iSB] = conf_autre->IDefNsfirst[iSB];
    NslastSB[iSB] = conf_autre->IDefNslast[iSB];
    validite_bandes_spectrales[iSB] = CMN__VRAI;
    PosUnderFlow[IND3(iSB+1,ctx->ctxtrt.PN,ctx->ctxtrt.SN,SD_BAS_SB,SD_BAS_PN)]=0;
    PosOverFlow[IND3(iSB+1,ctx->ctxtrt.PN,ctx->ctxtrt.SN,SD_BAS_SB,SD_BAS_PN)]=0;
//    PosUnderFlow[iSB] = 0;
//    PosOverFlow[iSB] = 0;
  }

  /* Corps de l'operation */
  /*----------------------*/
#ifdef OPS_DEBUG
  printf ("Debut de %s\n", tcProcName_F);
#endif

#ifdef OPS_DEBUG
  printf (" ctx_appel=%d \t SD_BAS_ISRFEM=%d \t SD_BAS_PRODUIT=%d\n", 
    ctx_appel, SD_BAS_ISRFEM, SD_BAS_PRODUIT);
  printf (" Nsfirst=%d \t Nslast=%d \t TailleSpectre=%d\n", 
    Nsfirst, Nslast, TailleSpectre);
  if (ctx_appel == SD_BAS_ISRFEM) 
  {
    printf (" SWNsfirst=%d \t SWNslast=%d\n", SWNsfirst, SWNslast);
  }
  printf (" SP=%d \t CTN=%d\n", SP, CTN);
  printf (" NbSeg=%d \t NsegfirstNsfirst=%d \t NbNsSeg=%d\n", 
    NbSeg, NsegfirstNsfirst, NbNsSeg);
  printf (" TabScaleM=%.8le \t TabScaleC=%.8le\n", 
    TabScaleM, TabScaleC);
  printf (" TabOffsetM=%.8le \t TabOffsetC=%.8le\n", 
    TabOffsetM, TabOffsetC);
#endif

  //@ on commence ici par reorganiser et par decoder le train de bit DataDecode
  //@ cf DA22 12.3 + FD
  //@ Ceci a deja ete realise dans la fonction SD_BAS_DecodeSpectreN0

  //@ Initialisation du nombre de mots en under/overflow
  //@ NbOverFlow = 0 et NbUnderFlow = 0
  NbUnderFlow = 0;
  NbOverFlow = 0;

  //@ Initialisation des indices de bit et d'octet dans le tableau d'octets CodSpect
  indiceBit   = 0;
  indiceOctet = 0;

  //@ Debut de boucle sur les echantillons n de Nsfirst a Nslast
#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_OffsetScale.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  fprintf (Fd, "Offset\tScale\n");
  for (n = 1 ; n <= NbSeg ; n++)
  {
    fprintf (Fd, "%d\t%d\t%d\n", n, 
      TabOffset[IND2(n, CTN, SD_BAS_SPCT_COD)], 
      TabScale[IND2(n, CTN, SD_BAS_SPCT_COD)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

  sprintf (Fichier, "%s/%s/sorties/%s_Spect.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  if (Fd == (FILE*)NULL)
  {
    printf ("Impossible d'ouvrir le fichier suivant:\n %s\n", Fichier);
    goto fin;
  }
  fprintf (Fd, "n\tBCodSpect\tSpect decode\tOffset\tScale\n");
#endif
  for (n = Nsfirst; n <= Nslast; n++)
  {
    //@ nseg = PartieEntiere((n - NsegfirstNsfirst) / NbNsSeg) + 1 (definition)
    // la partie entiere est realisee par le cast implicite en 'int'
    nseg = (int)((double)(n - NsegfirstNsfirst) / (double)NbNsSeg) + 1;
    
    //@ Scale = TabScaleM * Tabscale(nseg) + TabScaleC
    // ICtcTabScale est suppose SD_BAS_PN lignes de SD_BAS_SPCT_COD valeurs
    Scale = TabScaleM * TabScale[IND2(nseg, CTN, SD_BAS_SPCT_COD)] + TabScaleC;
    
    //@ Offset = TabOffsetM * Taboffset(nseg) + TabOffsetC
    // ICtcTabOffset est suppose SD_BAS_PN lignes de SD_BAS_SPCT_COD valeurs
    Offset = TabOffsetM * TabOffset[IND2(nseg, CTN, SD_BAS_SPCT_COD)] + TabOffsetC;

    // pour simplifier l'ecriture :
    // NbBitsSeg = nombre de bits a transmettre par plage spectrale
    NbBitsSeg = TabNbBit[IND2(nseg, CTN, SD_BAS_SPCT_COD)];
    
    //@ Cod formule dans DA7. On decode les NbBitsSeg bits suivants dans CodSpect
    Cod = 0;
    for (iBit = 0; iBit < NbBitsSeg; iBit++)
    {
      // Cod += (extraction du bit courant dans CodSpect) * (2**iBit)
      V = CodSpect[indiceOctet] & (128 >> indiceBit);
      if (V)
      {
        Cod = Cod + (1 << iBit);
      }

      // mise a jour de l'indice de bit
      indiceBit ++;

      // mise a jour de l'indice d'octet
      if (indiceBit == NB_BITS_OCTET)
      {
        indiceBit = 0;
        indiceOctet++;
      }
    }
    
    //@ test de saturation
    //@ si Cod=0
    if (Cod == 0) 
    {
      //@ alors NbUnderFlow = NbUnderFlow + 1
      NbUnderFlow = NbUnderFlow + 1;

      //@ et reperage de la position des echantillons faux
      //@ PosUnderFlow(NbUnderFlow)= n
      /* ATTENTION : PosUnderFlow est de dimension 3xSNxPN, 3=SD_BAS_SB */
      /* on compte le nb d'underflow dans chaque bande spectrale */
      for (iSB = 0 ; iSB < SD_BAS_SB ; iSB++)
      {
        if ( (n >= NsfirstSB[iSB]) && (n <= NslastSB[iSB]) )
        {
          validite_bandes_spectrales[iSB] = CMN__FAUX;
          if (NbUnderFlow <= SD_BAS_NBPOSUNDERFLOWMAX)
          {
            PosUnderFlow[IND3(NbUnderFlow,ctx->ctxtrt.PN,ctx->ctxtrt.SN,SD_BAS_SB,SD_BAS_PN)] = n;
          } /* fin IF */
        } /* fin IF */
      } /* fin FOR */
    } /* fin IF */
    //@ si Cod = 2**TabNbBit(nseg) - 1  
    else if ( Cod == ((1 << NbBitsSeg) - 1) )
    {
      //@ alors NbOverFlow = NbOverFlow + 1 
      NbOverFlow = NbOverFlow + 1;

      //@ et reperage de la position des echantillons faux  
      //@ PosOverFlow(NbOverFlow) = n 
      for (iSB = 0 ; iSB < SD_BAS_SB ; iSB++)
      {
        if ( (n >= NsfirstSB[iSB]) && (n <= NslastSB[iSB]) )
        {
          validite_bandes_spectrales[iSB] = CMN__FAUX;
          if (NbOverFlow <= SD_BAS_NBPOSOVERFLOWMAX)
          {
            PosOverFlow[IND3(NbOverFlow,ctx->ctxtrt.PN,ctx->ctxtrt.SN,SD_BAS_SB,SD_BAS_PN)] = n;
          } /* fin IF */
        } /* fin IF */
      } /* fin FOR */
    } /* fin IF */
    //@ fin de test 

    //@ decodage de Cod
    //@ Spect(n) = (Cod - 1 + Offset) / Scale
    
    /* Cas de la chaine ISRFEM */
    if (ctx_appel == SD_BAS_ISRFEM) 
    {
      if ( (n >= SWNsfirst) && (n <= SWNslast) )
      {
        Spect[n - SWNsfirst] = (double)(Cod - 1 + Offset) / (double)Scale;
      }
    }
    
    /* Cas de la chaine PRODUIT */
    if (ctx_appel == SD_BAS_PRODUIT)
    {
      Spect[n - Nsfirst] = (double)(Cod - 1 + Offset) / (double)Scale;
    }
#ifdef OPS_DEBUG
  if ( (n < Nsfirst + 2) || (n > Nslast - 2) )
  {
    printf (" n = %d ; Cod = %d \t NbBitsSeg = %d \t TabScale = %d \t TabOffset = %d\n", 
      n, Cod, NbBitsSeg, 
      TabScale[IND2(nseg, CTN, SD_BAS_SPCT_COD)], 
      TabOffset[IND2(nseg, CTN, SD_BAS_SPCT_COD)]);
    printf ("\t Scale = %.4le ; Offset = %.4le ; Spect = %.4le\n", 
      Scale, Offset, Spect[n - Nsfirst]);
  }

  fprintf (Fd, "%d\t%d\t%.7le\t%d\t%d\n", 
    n - Nsfirst + 1, Cod, Spect[n - Nsfirst], 
    TabOffset[IND2(nseg, CTN, SD_BAS_SPCT_COD)], 
    TabScale[IND2(nseg, CTN, SD_BAS_SPCT_COD)]);
#endif

  } //@ fin de la boucle sur les echantillons n de Nsfirst a Nslast
  
#ifdef OPS_DEBUG
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

  for (iSB = 0 ; iSB < SD_BAS_SB ; iSB++)
  {
    printf (" validite_bandes_spectrales[SB=%d]=%d \t Vrai=%d \t FAUX=%d\n", 
      iSB + 1, validite_bandes_spectrales[iSB], CMN__VRAI, CMN__FAUX);
  }
  printf (" NbUnderFlow=%d \t NbOverFlow=%d\n", 
    NbUnderFlow, NbOverFlow);
#endif

  //@ Rapport d'analyse de qualite (nombre de mots faux et positions)
 (InfoDet1ENG->tab_var[ENG_GDOCNBUNDERFLOW].val.pi4[IND2(ctx->ctxtrt.PN,ctx->ctxtrt.SN,SD_BAS_PN)]) = NbUnderFlow;
 (InfoDet1ENG->tab_var[ENG_GDOCNBOVERFLOW].val.pi4[IND2(ctx->ctxtrt.PN,ctx->ctxtrt.SN,SD_BAS_PN)]) = NbOverFlow;
  
  //@ Flag d'over/underflow
  //@ Si NbOverFlow = 0 et NbUnderFlow = 0  alors
  if ( (NbUnderFlow == 0) && (NbOverFlow == 0) )
  {
    //@ FlagUnderOverFlow=non
    (InfoDet1ENG->tab_var[ENG_GDOCFLAGUNDEROVERFLOW].val.pui4[IND2(ctx->ctxtrt.PN,ctx->ctxtrt.SN,SD_BAS_PN)]) = CMN__FAUX;
  }
  else
  {
    //@ Sinon FlagUnderOverFlow=oui
    (InfoDet1ENG->tab_var[ENG_GDOCFLAGUNDEROVERFLOW].val.pui4[IND2(ctx->ctxtrt.PN,ctx->ctxtrt.SN,SD_BAS_PN)]) = CMN__VRAI;
  }

  // On trace ici toutes les variables en sortie
#ifdef INVESTIGATION
    // NbUnderFlow
    SD_BAS_TRACE(
      tcProcName_F, 
      "ENG_GDOCNBUNDERFLOW",
      1, 
      1, 
      1,
      TRACE_INT4, 
      TRACE_ASC,
      &InfoDet1ENG->tab_var[ENG_GDOCNBUNDERFLOW].val.pi4[IND2(ctx->ctxtrt.PN,ctx->ctxtrt.SN,SD_BAS_PN)], 
      ctx);

    // PosUnderFlow
    SD_BAS_TRACE(
      tcProcName_F, 
      "ENG_GDOCPOSUNDERFLOW",
      1, 
      SD_BAS_SB, 
      1,
      TRACE_INT4, 
      TRACE_ASC,
      &InfoDet1ENG->tab_var[ENG_GDOCPOSUNDERFLOW].val.pi4[IND3(1,ctx->ctxtrt.PN,ctx->ctxtrt.SN,SD_BAS_SB,SD_BAS_PN)], 
      ctx);

    // NbOverFlow
    SD_BAS_TRACE(
      tcProcName_F, 
      "ENG_GDOCNBOVERFLOW",
      1, 
      1, 
      1,
      TRACE_INT4, 
      TRACE_ASC,
      &InfoDet1ENG->tab_var[ENG_GDOCNBOVERFLOW].val.pi4[IND2(ctx->ctxtrt.PN,ctx->ctxtrt.SN,SD_BAS_PN)], 
      ctx);

    // PosOverFlow
    SD_BAS_TRACE(
      tcProcName_F, 
      "ENG_GDOCPOSOVERFLOW",
      1, 
      SD_BAS_SB, 
      1,
      TRACE_INT4, 
      TRACE_ASC,
      &InfoDet1ENG->tab_var[ENG_GDOCPOSOVERFLOW].val.pi4[IND3(1,ctx->ctxtrt.PN,ctx->ctxtrt.SN,SD_BAS_SB,SD_BAS_PN)], 
      ctx);

    // Spect
    SD_BAS_TRACE(
      tcProcName_F, 
      "Spect",        
      1, 
      TailleSpectre, 
      1, 
      TRACE_REAL8, 
      TRACE_BIN, 
      Spect, 
      ctx);
    
    // FlagUnderOverFlow
    SD_BAS_TRACE(
      tcProcName_F, 
      "ENG_GDOCFLAGUNDEROVERFLOW",
      1, 
      1, 
      1,
      TRACE_INT4, 
      TRACE_ASC,
      &InfoDet1ENG->tab_var[ENG_GDOCFLAGUNDEROVERFLOW].val.pi4[IND2(ctx->ctxtrt.PN,ctx->ctxtrt.SN,SD_BAS_PN)], 
      ctx);
			
#endif

  fin:
  // sortie de la fonction 
#ifdef OPS_DEBUG
  printf ("Fin de %s\n", tcProcName_F);
#endif
  return(CodeRetour); 
} /* fin de SD_ALG_20_DOC */


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_09_PLK
//@
//@ Description : calcul de la fonction de planck 
//@ on utilise ici les valeurs de la ligne courante et pas celle de la
//@ ligne precedente (cf FD51 AC)
//@
//@
//@ Parameters :
//@     (ENTREE)  ctx               : contexte de l'etape
//@     (ENTREE)  conf_autre         : fichier de conf autre
//@     (ENTREE)  BBT               : temperature BBT (LN -1)
//@     (Sortie)  dataThreadProduit : info generales calculees sur la ligne (N-1)
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_09_PLK(
    t_SD_BAS_DataThread *         ctx, 
    t_SD_BAS_ConfAutre *          conf_autre,
    double                        BBT, 
    t_SD_BAS_DataThreadProduit *  data_thread_produit)

{
  /* Variables locales */
  /*-------------------*/
  static char   tcProcName_F[] = "SD_ALG_09_PLK";
  t_CMN__Statut CodeRetour = CMN__STATUT_OK;
  
  int           indicePlanck;   /* indice du tableau de sortie */
  int           iCalcul;   /* indice de la boucle principale de calcul */
  int           iInterpolation; /* indice de la boucle d'interpolation */
  int           TaillePlanckBBT; /* nb de pts de calcul de la fonction de Planck */
  int           Nboucles;  /* nb de boucles principales de calcul */
  int           Nboucles2; /* nb de boucles d'interpolation */
  double        DWnSpect;  /* echantillonnage spectral nominal du spectre mesure */
  int           Nsfirst; /* indice du premier echantillon spectral */
  int           Nslast; /* indice du dernier echantillon spectral */
  double        N; /* multiplicateur pour le calcul du pas de recurrence */
  double        K1; /* constante pour les calculs */
  double        K2; /* constante pour les calculs */
  double*       PlanckBBT; /* pointeur sur le tableau de sortie,
                              de taille (NsLast-NsFirst+1), au + egale a SD_BAS_SS0 */
  double        DeltaWn;   /* constante pour les calculs */
  double        Beta;      /* constante pour les calculs */
  double        Gamma;     /* constante pour les calculs */
  double        Wn0;       /* nombre d'onde pour le premier echantillon */
  double        Wn;        /* nombre d'onde */
  double        Wn2;       /* nombre d'onde */
  double        G;         /* variable temporaire de calcul */
  double        F;         /* variable temporaire de calcul */
  double        A;         /* variable temporaire de calcul */
  double        B;         /* variable temporaire de calcul */
  double        Da;        /* variable temporaire de calcul */

#ifdef OPS_DEBUG
  FILE *        Fd;
  char          Fichier[256];
#endif

  /* Initialiser les variables locales */
  /*-----------------------------------*/
  DWnSpect = conf_autre->IDefSpectrDWn;
  K1 = conf_autre->IDefPlkPK1;
  K2 = conf_autre->IDefPlkPK2;
  N = conf_autre->IDefPlkN;
  Nsfirst = conf_autre->IDefNsfirst[SD_BAS_B1];
  Nslast = conf_autre->IDefNslast[SD_BAS_B3];
  TaillePlanckBBT = Nslast - Nsfirst + 1;
  Nboucles = (int)ceil((double)TaillePlanckBBT /(double) N);
//  Nboucles = NINT(TaillePlanckBBT / N);
  PlanckBBT = data_thread_produit->PlanckBBT;
  DeltaWn = N * DWnSpect;
  Beta = exp(K2 * DeltaWn / BBT);
  Gamma = Beta - 1.0;
  Wn0 = (Nsfirst - 1) * DWnSpect;
  indicePlanck = 1;

  /* Corps de l'operation */
  /*----------------------*/
#ifdef OPS_DEBUG
  printf ("Debut de %s\n", tcProcName_F);
#endif

#ifdef OPS_DEBUG
  printf (" N        = %.4le\n", N);
  printf (" NINT(N)  = %d\n", NINT(N));
  printf (" Nsfirst  = %d \t Nslast = %d \t TaillePlanckBBT = %d\n", 
    Nsfirst, Nslast, TaillePlanckBBT);
  printf (" Nboucles = %d\n", Nboucles);
  printf (" BBT      = %.4le\n", BBT);
  printf (" DWnSpect = %.4le\n", DWnSpect);
  printf (" DeltaWn  = %.4le\n", DeltaWn);
  printf (" Wn0      = %.4le\n", Wn0);
  printf (" K1       = %.4le\n", K1);
  printf (" K2       = %.4le\n", K2);
#endif

  //@ Initialisation des nombres d'onde --------------------------------------
  //@ Wn = Wn0 - DeltaWn
  Wn = Wn0 - DeltaWn;

  //@ G = Exp( K2 . Wn / BBT ) - 1
  G = exp(K2 * Wn / BBT) - 1.0;

  //@ Wn = Wn + DeltaWn
  Wn = Wn + DeltaWn;

  //@ F = K1 . Wn**3
  F = K1 * Wn * Wn * Wn;

  //@ G = Beta . G + Gamma
  G = Beta * G + Gamma;

  //@ A = F / G
  A = F / G;

  //@ Boucle sur les points de calcul ----------------------------------------
  //@ tous les DeltaWn de Nsfirst a Nslast par pas de N 
#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_PlanckBBT.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  if (Fd == NULL)
  {
    printf (" Impossible d'ouvrir le fichier suivant:\n %s\n", Fichier);
    goto fin;
  }
  fprintf(Fd, "Wn\t09_PLK\tPlanck\n");
#endif

  for (iCalcul = 1 ; iCalcul <= Nboucles ; iCalcul++)
  {
    //@ Wn' = Wn
    Wn2 = Wn;

    //@ Wn = Wn + DeltaWn
    Wn = Wn + DeltaWn;

    //@ F = K1 . Wn**3
    F = K1 * Wn * Wn * Wn;

    //@ G = Beta . G + Gamma
    G = Beta * G + Gamma;

    //@ B = F / G
    B = F / G;

    //@ Da = ( B - A ) / N
    Da = (B - A) / N;

    //@ PlanckBBT(Wn', BBT) = A
    PlanckBBT[IND1(indicePlanck)] = A;

#ifdef OPS_DEBUG
  fprintf(Fd, "%.6e\t%.6e\t%.6e\n",
    Wn2, PlanckBBT[IND1(indicePlanck)], SD_BAS_Planck(BBT, Wn2, K1, K2));
#endif
    indicePlanck++;

    //@ Boucle sur les points d'interpolation --------------------------------
    if ( NINT(N * iCalcul) > TaillePlanckBBT )
    {
      Nboucles2 = TaillePlanckBBT % NINT(N) - 1;
    }
    else
    {
      Nboucles2 = NINT(N) - 1;
    }

    //@ tous les DWnSpect de 1 a N-1
    for (iInterpolation = 1 ; iInterpolation <= Nboucles2 ; iInterpolation++)
    {
      //@ Wn' = Wn' + DWnSpect
      Wn2 = Wn2 + DWnSpect;

      //@ PlanckBBT( Wn' , BBT ) = PlanckBBT( Wn' - DWnSpect , BBT ) + Da
      PlanckBBT[IND1(indicePlanck)] = PlanckBBT[IND1(indicePlanck - 1)] + Da;
#ifdef OPS_DEBUG
  fprintf(Fd, "\t%.6e\t%.6e\t%.6e\n",
    Wn2, PlanckBBT[IND1(indicePlanck)], SD_BAS_Planck(BBT, Wn2, K1, K2));
#endif
      indicePlanck++;

    } //@ Fin de boucle d'interpolation

    //@ A = B
    A = B;

  } //@ Fin de boucle principale
#ifdef OPS_DEBUG
  printf (" Nboucles2 = %d \t indicePlanck = %d\n", Nboucles2, indicePlanck);
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
#endif

  /* on trace ici si besoin toutes les variables en sortie */
#ifdef INVESTIGATION
  // Spectre de la fonction de Planck a la temperature BBT
  SD_BAS_TRACE(tcProcName_F, "PlanckBBT",
               1, conf_autre->IDefNslast[SD_BAS_B3], 1,
               TRACE_REAL8, TRACE_BIN,
               PlanckBBT, ctx);
#endif

  // sortie de la fonction 
  fin:
#ifdef OPS_DEBUG
  printf ("Fin de %s\n", tcProcName_F);
#endif
  return (CodeRetour); 
} /* fin de SD_ALG_09_PLK */


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_30_FTB
//@
//@ Description : filtrage de la temperature du corps noir
//@
//@
//@ Parameters :
//@     (ENTREE)  ctx             : contexte de l'etape
//@     (ENTREE)  NbLignes        : nb de lignes du granule traite
//@     (ENTREE)  ContexteCourant : pointeur sur le contexte courant conf.
//@     (ENTREE)  conf_autre       : fichier de conf autre
//@     (ENTREE)  data_thread_produit : infos generales calculees sur la ligne (SN)
//@     (SORTIE)  DataGenGranule  : Donnees generales au granule
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_30_FTB(
    t_SD_BAS_DataThread *        ctx, 
    int                          NbLignes, 
    t_SD_BAS_Contexte *          ContexteCourant,
    t_SD_BAS_ConfAutre *         conf_autre,
    t_SD_BAS_DataThreadProduit * data_thread_produit,
    t_SD_BAS_DataGen  *          data_gen_granule)

{
  /* Variables locales */
  /*-------------------*/
  static char tcProcName_F[] = "SD_ALG_30_FTB";
  t_CMN__Statut CodeRetour = CMN__STATUT_OK;

  int       ILS;             // nb de lignes du granule courant
  int       FLS;             // nb de lignes necessaires au filtrage temporel
  double    t;               // Date de la ligne en cours convertie en s
  double    t0;              // Duree d'oubli
  double    tF;              // Date de la derniere mesure de la serie ILS
  int       Ndeg;            // Degre du polynome de regression
  double    CutoffRms;       // Seuil de la valeur Rms de la regression
  double    BBTRes;          // Precision de l'ajustement
  double *  GFtbFilteredBBT; // de taille nblignes granule
  int       i;              // variable de boucle sur les lignes
  int       j;              // variable de boucle sur les colonnes
  double *  x;               // variable contenant les coef des polynomes
  int nbpts; // nombre de points avant la duree d'oublie
  double    Dist;
  t_SD_BAS_SCDSTime dateLignetemp;
  double trelatif,tref;
  double val;
  int k;
  t_CMN__Booleen Trouve;
  
  /* DM 11/03/2004 */
  int N; /* nb de points tels que GBBTMissing = NON */
 
#ifdef OPS_DEBUG
  FILE *    Fd;
  char      Fichier[256];
#endif

  //@ Initialisations
  ILS             = NbLignes;
  FLS             = MIN(conf_autre->IDefFtbFLS, fabs(ContexteCourant->GBBTNbElt - ILS));
//  FLS             = MIN(conf_autre->IDefFtbFLS, ContexteCourant->GBBTNbElt - ILS);
  t0              = conf_autre->IDefFtbT0;
  Trouve = CMN__FAUX;
  for (i = 0 ; (i < ILS) && (Trouve == CMN__FAUX) ; i++)
  {
    if (ContexteCourant->GBBTHistTempCN[i].GBBTMissing == CMN__NON)
    {
      tF = CMN__DATE_CONV_SCDS_2_SEC(ContexteCourant->GBBTHistTempCN[i].GBBTDateElt);
      Trouve = CMN__VRAI;
    }
  }
  Ndeg            = conf_autre->IDefFtbRegressDegre;
  CutoffRms       = conf_autre->IDefFtbRegressRmsCutoff;
  GFtbFilteredBBT = data_gen_granule->GFtbFilteredBBT;
  x               = data_thread_produit->X;
  nbpts		  = 0;
  BBTRes	  = 0;

  /* Corps de l'operation */
  /*----------------------*/
#ifdef OPS_DEBUG
  printf ("Debut de %s\n", tcProcName_F);
#endif

#ifdef OPS_DEBUG
  printf (" ILS = %d \t FLS = %d \t Ndeg = %d\n", ILS, FLS, Ndeg);
  printf (" t0 = IDefFtbT0 = %.4lf \t tF = %.4lf\n", t0, tF);
#endif

  //@ Ajustement au sens des moindres carres par un polynome de degre Ndeg 
  //@ de la serie temporelle des BIMSBBT de longueur FLS+ILS avec un poids w(t)
  //@ Si (BBTMissing(t) = non) Alors w(t) = exp( -(t-tF)/t0 ) 
  //@ Sinon  w(t) = 0( tF est la date de la derniere mesure de la serie ILS )

// calcul d'une date de reference
  Trouve = CMN__FAUX;
  for (i = 0; (i < ILS + FLS) && (Trouve == CMN__FAUX) ; i++)
  {
    if (ContexteCourant->GBBTHistTempCN[ILS-1-i].GBBTMissing == CMN__NON)
    {
      dateLignetemp.day=ContexteCourant->GBBTHistTempCN[ILS-1-i].GBBTDateElt.day;
      dateLignetemp.millis=ContexteCourant->GBBTHistTempCN[ILS-1-i].GBBTDateElt.millis;
      tref = CMN__DATE_CONV_SCDS_2_SEC(dateLignetemp);
      Trouve = CMN__VRAI;
    }
  }
  
  //@ Calcul des matrices A et B 
  for (i = 0; i < ILS + FLS; i++)
  {
    /* Recuperation de la date qui correspond a la ligne en cours et */
    /* conversion en secondes */
    /* ATTENTION : les dates les plus recentes correspondent aux indices */
    /* les plus petits */
    t = CMN__DATE_CONV_SCDS_2_SEC(ContexteCourant->GBBTHistTempCN[i].GBBTDateElt);
//5.0 PM 3
// on compte le nombre de points dans la duree d'oubli
    if (t >  (tF -t0) && ContexteCourant->GBBTHistTempCN[i].GBBTMissing == CMN__NON)
    {
      nbpts++;
    }
    

    /* Calcul du coefficient B correspondant (B = BIMSBBT * w) */
    if (ContexteCourant->GBBTHistTempCN[i].GBBTMissing == CMN__OUI)
    {
      /* Remplissage de la matrice A */
      for (j = 0; j < Ndeg + 1 ; j++)
      {
        /* on applique ici aussi la matrice des poids = 0 */
        data_thread_produit->A[j * (ILS + FLS) + i] = 0.;
      } /* fin de boucle sur le nombre d'inconnues */
      
      /* il y a absence de mesure */
      data_thread_produit->B[i] = 0.0;
    }
    else
    {
      // on se positionne en temps relatif
      trelatif=t-tref;
      
      /* Remplissage de la matrice A */
      for (j = 0; j < Ndeg + 1 ; j++)
      {
        /* on applique ici aussi la matrice des poids */
//        data_thread_produit->A[j * (ILS + FLS) + i] = pow(t, (double)(j))* exp(-(t - tF) / t0);
        for  (k=0,val=1;k<j;k++) val = val * trelatif;
        data_thread_produit->A[j * (ILS + FLS) + i] = val * sqrt(exp((t - tF) / t0));
      } /* fin de boucle sur le nombre d'inconnues */
      /* vecteur des mesures : B = BIMSBBT * w */
      data_thread_produit->B[i] = 
        (ContexteCourant->GBBTHistTempCN[i].BIMSBBT) * sqrt(exp((t - tF) / t0));
    }

  } /* fin de la boucle sur le nombre d'equations */

#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_matriceA.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  printf ("Fichier=\n%s\n", Fichier);
  Fd = fopen(Fichier, "wt");
  for (i = 0; i < ILS + FLS ; i++)
  {
    for (j = 0; j < Ndeg + 1 ; j++)
    {
      fprintf (Fd, "%.4le\t", data_thread_produit->A[j * (ILS + FLS) + i]);
    }
    fprintf (Fd, "\n");
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

  sprintf (Fichier, "%s/%s/sorties/%s_matriceB.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  printf ("Fichier=\n%s\n", Fichier);
  Fd = fopen(Fichier, "wt");
  for (i = 0; i < ILS + FLS; i++)
  {
    printf ("B[%d] = %.4le\n", i, data_thread_produit->B[i]);
    fprintf (Fd, "%.4lf\n", data_thread_produit->B[i]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
#endif

//5.0 PM 3
// si le nombre de points disponible dans l'intervalle ( t-t0) est inferiaur ou egal a 2*Ndeg
  /* DM 11/03/2004 */
  if ( nbpts <= 2*(Ndeg+1) )
   {
    data_gen_granule->GFfbFLagBBTNonQual = CMN__OUI;
    // pour differentier du cas ou le filtrage a lieu
    BBTRes=-1;
    for (i = 0; i < ILS; i++)  
     {
    GFtbFilteredBBT[i]=SD_ALG_BBT_REF; // 293 K
     }
   }
 else
  {
  //@ Calcul de la serie temporelle filtree de longueur ILS 
  //@ par le polynome d'ajustement GFtbFilteredBBT
  //@ Calcul de la valeur RMS du residu d'ajustement sur la serie ILS
  //@ (indicateur de qualite)
  //@ et pour les termes tels que BBTMissing = NON
  //@ BBTRes = cf DA7 
  //@ appel SD_BAS_Moindres_Carres1D
  CodeRetour = SD_BAS_MoindresCarres1D(
    ctx,                   /* gestion des erreurs */
    ILS + FLS,             /* nb d'equations */
    Ndeg + 1,              /* nb d'inconnues = degre du polynome + 1 */
    data_thread_produit->A,        /* matrice A (entree) */
    data_thread_produit->B,        /* vecteur B (entree) dimension ILS + FLS */
    data_thread_produit->tabTmpMC, /* tableau temporaire pour libESSL */
    CutoffRms,             /* tolerance d'erreur sur l'estimation */
    x,
    &BBTRes);              /* RMS du residu d'ajustement */

  SD_BAS_THROW(
    CodeRetour, SD_ALG_SPECTRE_30_FTB_ALRT_MOINDRES_CARRES, 
    CodeRetour, BAS_CR_NEQ, CMN__STATUT_OK, "");

  /* Calcul de la temperature filtree du corps noir */
  for (i = 0; i < ILS; i++)  
  {
    /* Recuperation de la date qui correspond a la ligne en cours et */
    /* conversion en secondes */
    /* ATTENTION : les dates les plus recentes correspondent aux indices */
    /* les plus petits */
    t = CMN__DATE_CONV_SCDS_2_SEC(ContexteCourant->GBBTHistTempCN[i].GBBTDateElt);
    GFtbFilteredBBT[i]=0.;
// on travaille en temps relatif
    trelatif=t-tref;
    for (j = 0 ; j < Ndeg + 1 ; j++)
    {
//     GFtbFilteredBBT[i] = GFtbFilteredBBT[i] + x[j] * pow(t,(double)(j));
     for  (k=0,val=1;k<j;k++) val=val*trelatif;
     GFtbFilteredBBT[i] = GFtbFilteredBBT[i] + x[j] * val;
    }
  }
  Dist=0.0;
    
  /* DM 11/03/2004 */
  N = 0;
  for(i=0; i<ILS; i++)
  {
    if (ContexteCourant->GBBTHistTempCN[i].GBBTMissing == CMN__NON)
    {
    Dist += 
      (GFtbFilteredBBT[i] - ContexteCourant->GBBTHistTempCN[i].BIMSBBT)*(GFtbFilteredBBT[i] -
      ContexteCourant->GBBTHistTempCN[i].BIMSBBT);
      
      /* DM 11/03/2004 */
      N = N + 1;
    }
  }

  /* DM 11/03/2004 */
  if (N > 0)
  {
    BBTRes = sqrt(Dist / (double) N);
  }
  else
  {
    BBTRes = SD_ALG_FTBBBTRES_DEFAUT;
  }
  
#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_GFtbFilteredBBT.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  printf ("Fichier=\n%s\n", Fichier);
  Fd = fopen(Fichier, "wt");
  for (i = 0 ; i < ILS + FLS ; i++)
  {
    fprintf (Fd, "%.4le\n", GFtbFilteredBBT[i]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

  printf (" GFtbBBTRes = %.4le \t CutoffRms = %.4le\n", BBTRes, CutoffRms);
#endif

  data_gen_granule->GFtbBBTRes = BBTRes;

  //@  Test de qualite :
  //@ Si BBTRes > CutoffRms,
  if (BBTRes > CutoffRms)
  {
    //@ alors FlagBBTNonQual = oui
    data_gen_granule->GFfbFLagBBTNonQual = CMN__OUI;
  }
  else
  {
    //@ Sinon FlagBBTNonQual = non
    data_gen_granule->GFfbFLagBBTNonQual = CMN__NON;
  }
 }
 // fin du test sur le nombre de points 


#ifdef OPS_DEBUG
  printf (" GFfbFLagBBTNonQual = %d \t CMN__OUI = %d\n", 
    data_gen_granule->GFfbFLagBBTNonQual, CMN__OUI);
#endif

  //@ le contexte est mis a jour (ajout d'un element dans l'historique)
  //@ par l'appelant 
  //@ et on stocke dans data_thread_produitne les temp filtrees
  data_gen_granule->GFtbBBTRes = BBTRes;

  // on trace ici si besoin toutes les variables en sortie
#ifdef INVESTIGATION
  // GFtbFilteredBBT
  SD_BAS_TRACE(
    tcProcName_F, 
    "GFtbFilteredBBT",
    1, 
    ILS,
    1,
    TRACE_REAL8, 
    TRACE_ASC,
    GFtbFilteredBBT, 
    ctx);

  // GFtbBBTRes
  SD_BAS_TRACE(
    tcProcName_F, 
    "GFtbBBTRes",
    1, 
    1, 
    1,
    TRACE_REAL8, 
    TRACE_ASC,
    &BBTRes, 
    ctx);

  // FlagBBTNonQual
  SD_BAS_TRACE(
    tcProcName_F, 
    "FlagBBTNonQual",
    1, 
    1, 
    1,
    TRACE_INT4, 
    TRACE_ASC,
    &(data_gen_granule->GFfbFLagBBTNonQual), 
    ctx);
#endif

fin:
  /* sortie de la fonction */

  /* Switch a completer si il y a des parametres dans un message */
  /* Definir un numero specifique a chaque cas d'erreur :*/
  /*  #define SD_ALG_SPECTRE_<ALGO>_[ALRT|INFO|WRNG]_<LIBELLE> (SD_ALG_ERR_BASE_SPECTRE+1) */
  /*  dans SD_ALG_Spectre.h */
  switch (CodeRetour)
  {
    case SD_ALG_SPECTRE_30_FTB_ALRT_MOINDRES_CARRES:
    {
#ifdef OPS_DEBUG
  printf (" Erreur d'appel a SD_BAS_MoindresCarres1D.\n");
#endif
      /* "ERROR ON LEAST SQUARE FITTING OF BLACK BODY TEMPERATURE" defini dans SD.msg */
      SD_ALG_SPECTRE_MsgSet = SD_ALG_SPECTRE_SET; /* defini dans SD__Message.h */
      SD_ALG_SPECTRE_MsgNum = MSG_ALG_SPECTRE_30_FTB_ALRT_MOINDRES_CARRES; /* defini dans l'enumere e_SPECMessageJDB de SD__Message.h */
      CreerMessage (
        SD_ALG_SPECTRE_MsgSet, 
        SD_ALG_SPECTRE_MsgNum, 
        &(ctx->cr.msgJDB));
      CodeRetour = CMN__STATUT_NOK;
    }
    break;

    default :
    {
      if (CodeRetour != CMN__STATUT_OK)
      {
        CodeRetour = CMN__STATUT_NOK;
      }
    }
  } /* fin switch */
    
#ifdef OPS_DEBUG
  printf ("Fin de %s\n", tcProcName_F);
#endif
  return(CodeRetour); 
} /* fin de SD_ALG_30_FTB */


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_31_SCR
//@
//@ Description : calibration spectrale du coef de calibration radiometrique
//@ on utilise ici les valeurs de la ligne courante et le BBT de la
//@ ligne precedente 
//@
//@
//@ Parameters :
//@     (ENTREE)  ctx : contexte de l'etape
//@     (ENTREE)  conf_autre : fichier de conf autre
//@     (ENTREE)  spectre_decode : spectre decode pour un ISP (GDocSpect)
//@                                pour la ligne LN
//@     (ENTREE)  data_thread_produit : info generales de les lignes LN et LN-1
//@     (SORTIE)  spect : spectre corrige (GScrSpect) pour la ligne LN
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_31_SCR(
                t_SD_BAS_DataThread *         ctx, 
                t_SD_BAS_ConfAutre *          conf_autre,
                t_SD_BAS_DataThreadProduit *  data_thread_produit, 
                double *                      spectre_decode, 
                double *                      spectre_calibre)

{
  /* Variables locales */
  /*-------------------*/
  char          tcProcName_F[] = "SD_ALG_31_SCR";
  char          LibelleTraces[SD_ALG_SPECTRE_LG_LIBELLE_TRACES + 1];
  t_CMN__Statut CodeRetour = CMN__STATUT_OK;
  /* Numero du premier echantillon spectral */
  int           Nsfirst;
  /* Numero du dernier echantillon spectral */
  int           Nslast;
  /* Nombre d'echantillons spectraux */
  int           TailleSpectreCalibre;
  /* Fonction de Planck utilisee a bord pour la ligne precedente (cf 09_PLK) */
  double *      PlanckBBT;
  /* Fonction de Planck interpollee sur la base des nb d'onde nominaux */
  /* calibres spectralement (cf XX_InitProd) */
  double *      B1;
  /* Compteur */
  int           n;

#ifdef OPS_DEBUG
  FILE *            Fd;
  char              Fichier[256];
#endif
  int     PN;
  int     CCD;

  //@ Initialisations jlp
  PN                  = ctx->ctxtrt.PN;
  CCD                 = ctx->ctxtrt.CCD + 1;          // Pour pouvoir utiliser les macros

  /* Initialiser les variables locales */
  /*-----------------------------------*/
  Nsfirst = conf_autre->IDefNsfirst[SD_BAS_B1];
  Nslast = conf_autre->IDefNslast[SD_BAS_B3];
  TailleSpectreCalibre = Nslast - Nsfirst + 1;

  PlanckBBT = data_thread_produit->PlanckBBT; /* ligne LN-1 */
  B1 = data_thread_produit->B1;               /* ligne LN-1 */

  /* Corps de l'operation */
  /*----------------------*/
#ifdef OPS_DEBUG
  printf ("Debut de %s\n", tcProcName_F);
#endif

  //@----------- les calculs suivants ont ete precalcule dans XX_InitProd
  //@ Calcul des nb d'onde des fonctions de calibration spectrale d'indice 
  //@ compris entre NsfirstFcsTab et NslastFcsTab
  //@ WnFcsTab(m) = (m - 1) * DWnFcsTab;
  
  //@ Calcul des positions nominales des echantillons d'indice compris 
  //@ entre Nsfirst et Nslast
  //@ Wn0(n) = (n - 1) * DWnSpect;

  //@ Interpolation lineaire des fonctions de calibration spectrale
  //@ [WnFcsTab, FcsTab] donnent [Wn0, FcsInterpolee]
  //@ appel SD_BAS_InterpLineaire1D
  
  //@ Calibration spectrales des positions nominales des echantillons 
  //@ WnSpect : Vecteur des nb d'onde nominaux calibres
  //@ taille conf_autre->IDefNslast[SB=3] - conf_autre->IDefNsfirst[SB=1] + 1
  //@ occurence : indice de ligne du granule = LN  
  //@ WnSpect(n) = Wn0(n) * FcsInterpolee(n);

  //@ Calcul de la fonction de Planck interpolee B1 aux nb d'onde WnSpect
  //@ B1 : Valeurs de la fontion de Planck calibree pour la ligne precedente
  //@ taille conf_autre->IDefNslast[SB=3]-conf_autre->IDefNsfirst[SB=1]+1
  //@ occurence : indice de ligne du granule = LN-1
  //@ [Wn0,PlanckBBT] donnent [WnSpect, B1]
  //@ appel a SD_BAS_InterpLineaire1D
  //@ ------

  //@ correction radiometrique
  //@ Spect(n) = Spect(n) . B1(n) / PlanckBBT(n)
  // jlp
  for (n = 1 ; n <= TailleSpectreCalibre ; n++)
  {
    spectre_calibre[IND1(n)] = 
      spectre_decode[IND1(n)] * B1[IND3(n,PN,CCD,(conf_autre->IDefNslast[SD_BAS_B3] - conf_autre->IDefNsfirst[SD_BAS_B1] + 1),SD_BAS_PN)] 
      / PlanckBBT[IND1(n)];
  }
  
#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_spectre_decode.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (n = 1 ; n <= TailleSpectreCalibre ; n++)
  {
    fprintf (Fd, "%.4le\n", spectre_decode[IND1(n)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

  sprintf (Fichier, "%s/%s/sorties/%s_spectre_calibre.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (n = 1 ; n <= TailleSpectreCalibre ; n++)
  {
    fprintf (Fd, "%.4le\n", spectre_calibre[IND1(n)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
#endif

  /* on trace ici si besoin toutes les variables en sortie */
#ifdef INVESTIGATION
  /* ecrire le spectre calibre radiometriquement dans un fichier binaire */
  sprintf (LibelleTraces, "GScrSpect");
 
 // jlp
  SD_BAS_TRACEFICHIERREEL (
    tcProcName_F, 
    &(ctx->ctxtrt), 
    LibelleTraces, 
    1, 
    TailleSpectreCalibre,
    spectre_calibre, 
    ctx);
#endif

fin:
/* sortie de la fonction */
#ifdef OPS_DEBUG
  printf ("Fin de %s\n", tcProcName_F);
#endif
return(CodeRetour); 
}


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_32_HEC
//@
//@ Description : correction d'emissivite du corps noir
//@ on utilise ici le spectre  de la ligne courante corrrige par les info
//@ generales de la ligne precedente. 
//@ Ces infos sont accessibles dans la structure t_SD_BAS_DataThreadProduit
//@
//@
//@ Parameters :
//@     (ENTREE)  ctx : contexte de l'etape
//@     (ENTREE)  conf_autre : fichier de conf autre
//@     (ENTREE/SORTIE)  data_thread_produit : info generales des lignes 
//@                                            LN et LN-1
//@     (ENTREE)  spectre_calibre : spectre calibre radiometriquement pour 
//@                                 LN, SN, PN (GScrSpect)
//@     (SORTIE)  spectre_corrige: spectre corrige de l'emissivite du corps 
//@                                noir pour LN SN PN (GHecSpect)
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_32_HEC(
                t_SD_BAS_DataThread *         ctx, 
                t_SD_BAS_ConfAutre *          conf_autre,
                t_SD_BAS_DataThreadProduit *  data_thread_produit,
                double *                      spectre_calibre,
                double *                      spectre_corrige_emissivite)

{
  /* Variables locales */
  /*-------------------*/
  char          tcProcName_F[] = "SD_ALG_32_HEC";
  char          LibelleTraces[SD_ALG_SPECTRE_LG_LIBELLE_TRACES + 1];
  t_CMN__Statut CodeRetour = CMN__STATUT_OK;

  /* Numero du premier echantillon spectral */
  int           Nsfirst;
  /* Numero du dernier echantillon spectral */
  int           Nslast;
  /* Nombre d'echantillons spectraux */
  int           TailleSpectreCorrige;
  /* Radiance emise par le corps noir chaud pour les echantillons dont */
  /* l'indice est compris entre Nsfirst et Nslast (cf XX_InitProd) */
  double *      Bbb;
  /* Fonction de Planck interpollee sur la base des nb d'onde nominaux */
  /* calibres spectralement (cf XX_InitProd) */
  double *      B1;
  /* compteur */
  int           n;

#ifdef OPS_DEBUG
  FILE *            Fd;
  char              Fichier[256];
#endif

  int     PN;
  int     CCD;

  //@ Initialisations jlp
  PN                  = ctx->ctxtrt.PN;
  CCD                 = ctx->ctxtrt.CCD + 1;          // Pour pouvoir utiliser les macros
  /* Initialiser les variables locales */
  /*-----------------------------------*/
  Nsfirst = conf_autre->IDefNsfirst[SD_BAS_B1];
  Nslast = conf_autre->IDefNslast[SD_BAS_B3];
  TailleSpectreCorrige = Nslast - Nsfirst + 1;
  Bbb = data_thread_produit->Bbb;
  B1 = data_thread_produit->B1;

  /* Corps de l'operation */
  /*----------------------*/
#ifdef OPS_DEBUG
  printf ("Debut de %s\n", tcProcName_F);
#endif

  //@----------- les calculs suivants ont ete precalcule dans XX_InitProd
  //@ Calcul des nb d'onde des fonctions de calibration spectrale d'indice 
  //@ compris entre NsfirstFcsTab et NslastFcsTab
  //@ WnFcsTab(m) = (m - 1) * DWnFcsTab;

  //@ Calcul des positions nominales des echantillons d'indice compris 
  //@ entre Nsfirst et Nslast
  //@ Wn0(n) = (n - 1) * DWnSpect;

  //@ Interpolation lineaire des fonctions de calibration spectrale
  //@ [WnFcsTab, FcsTab] donnent [Wn0, FcsInterpolee]
  //@ appel SD_BAS_InterpLineaire1D

  //@ Calibration spectrales des positions nominales des echantillons 
  //@ WnSpect : Vecteur des nb d'onde nominaux calibres
  //@ taille conf_autre->IDefNslast[SB=3] - conf_autre->IDefNsfirst[SB=1] + 1
  //@ occurence : indice de ligne du granule = LN  
  //@ WnSpect(n) = Wn0(n) * FcsInterpolee(n);

  //@----------- les calculs suivants ont ete precalcule dans XX_InitProd
  //@ Calcul de la temperature des surfaces radiatives
  //@ Pour i entre 1 et NbRrs
  //@   Trs(i) = Modele-Orbital{t, t_FE, Period, TRrsModel(i)}
  //@ FinPour
  //@ appel SD_BAS_EvolutionModele

  //@ Si ( (abs(t-t_FE) < 8*Period) OU 
  //@@     ( (TrsModel(1)=0) ET
  //@        (TrsModel(2)=0) ET
  //@        (TrsModel(3)=0) ET
  //@        (TrsModel(4)=0) ET
  //@        (TrsModel(5)=0) ET
  //@        (TrsModel(6)=0) ) )
  //@   Alors FlagDateNOK = NON
  //@   Sinon FlagDateNOK = OUI
  //@ FinSi

  //@ Calcul de la radiance emise par le corps noir chaud pour les echantillons 
  //@ spectraux n allant de Nsfirst a Nslast
  //@ appel SD_BAS_Planck
  //@   Bbb(n) = BBEmissivity(n). Planck(WnSpect(n), BBT)+ 
  //@   (cf DA7) Rrs(i) . Planck(WnSpect(n), TRrs(i))
  //@ ------

  //@ Correction radiometrique
  //@   Spect(n) = Spect(n) . Bbb(n) / B1(n)
// jlp
  for (n = 1 ; n <= TailleSpectreCorrige ; n++)
  {
    spectre_corrige_emissivite[IND1(n)] = 
      spectre_calibre[IND1(n)] * Bbb[IND3(n,PN,CCD,(conf_autre->IDefNslast[SD_BAS_B3] - conf_autre->IDefNsfirst[SD_BAS_B1] + 1),SD_BAS_PN)] 
      / B1[IND3(n,PN,CCD,(conf_autre->IDefNslast[SD_BAS_B3] - conf_autre->IDefNsfirst[SD_BAS_B1] + 1),SD_BAS_PN)];
  }

#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_spectre_calibre.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (n = 1 ; n <= TailleSpectreCorrige ; n++)
  {
    fprintf (Fd, "%.4le\n", spectre_calibre[IND1(n)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

  sprintf (Fichier, "%s/%s/sorties/%s_spectre_corrige_emissivite.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (n = 1 ; n <= TailleSpectreCorrige ; n++)
  {
    fprintf (Fd, "%.4le\n", spectre_corrige_emissivite[IND1(n)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
#endif

  /* on trace ici si besoin toutes les variables en sortie */
#ifdef INVESTIGATION
  /* ecrire le spectre calibre radiometriquement dans un fichier binaire */
  sprintf (LibelleTraces, "GHecSpect");
 // jlp
  SD_BAS_TRACEFICHIERREEL (
    tcProcName_F, 
    &(ctx->ctxtrt), 
    LibelleTraces, 
    1,
    TailleSpectreCorrige, 
    spectre_corrige_emissivite , 
    ctx);
#endif

fin:
// sortie de la fonction 
#ifdef OPS_DEBUG
  printf ("Fin de %s\n", tcProcName_F);
#endif
return(CodeRetour);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_33_SME
//@
//@ Description : estimation de la temperature du miroir de balayage 
//@
//@
//@ Parameters :
//@     (ENTREE)  ctx : contexte de l'etape
//@        numero de ligne 
//@     (ENTREE)  contextecourant : pointeur sur le contexte
//@                     courant configuration
//@     (ENTREE)  t : date UTC de reference de la ligne LN-1
//@     (ENTREE)  t_FE : date UTC de fin de l'eclipse
//@        precedant la ligne LN-1
//@     (ENTREE)  conf_autre : fichier de conf autre
//@     (ENtree/Sortie)  InfoDetEng : info det du produit ENG (LN-1)
//@   (on choisira la date UTC de debut recuperee dans InfoDetEng)
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_33_SME(
    t_SD_BAS_DataThread * ctx, 
    t_SD_BAS_SCDSTime     t,
    t_SD_BAS_SCDSTime     t_FE,
    t_SD_BAS_ConfAutre * conf_autre,
    t_SD_BAS_Liste_Var *  InfoDetEng, 
    double *              t_scan)

{
  /* Variables locales */
  /*-------------------*/
 static char   tcProcName_F[] = "SD_ALG_33_SME";
  t_CMN__Statut CodeRetour = CMN__STATUT_OK;
  
  /* Periode orbitale nominale de METOP (periode nodale) en secondes */
  double        Period;
  /* Modele de temperature des surfaces radiatives */
  double *      TScanModel;

  /* Initialiser les variables locales */
  /*-----------------------------------*/
  Period = conf_autre->IDefMETOPOrbitPeriod;
  TScanModel = conf_autre->IDefSmeTScanModel;

  /* Corps de l'operation */
  /*----------------------*/
#ifdef OPS_DEBUG
  printf ("Debut de %s\n", tcProcName_F);
#endif
  
  //@ appel SD_BAS_EvolutionModele
  //@ Tscan = Modele-Orbital{t, t_FE, Period, TscanModel}
  *t_scan = SD_BAS_EvolutionModele(t, t_FE, Period, TScanModel);
#ifdef OPS_DEBUG
  printf("Tscan = %f\n", *t_scan);
#endif
   *(InfoDetEng->tab_var[ENG_GSMETSCAN].val.pr8)=*t_scan;
  //@ Si (abs(t-t_FE) < 8 * Period ou TscanModel(6)=0 )
  if( ( fabs((t.day - t_FE.day) * NB_MILLIS_JOUR + t.millis - t_FE.millis)
        < NINT(8.0 * Period * NB_MILLIS_SEC) ) ||
      SD_BAS_MacEGAUX_D(conf_autre->IDefSmeTScanModel[IND1(6)], 0.0) )
  {
    //@ Alors FlagDateNOK = NON
    *(InfoDetEng->tab_var[ENG_GSMEFLAGDATENOK].val.pui4) = CMN__NON;
  }
  else
  {
    //@ Sinon FlagDateNOK = OUI
    *(InfoDetEng->tab_var[ENG_GSMEFLAGDATENOK].val.pui4) = CMN__OUI;
  }
  //@ Fin Si

  // on trace ici si besoin toutes les variables en sortie
#ifdef INVESTIGATION
  // Tscan
  SD_BAS_TRACE(tcProcName_F, "ENG_GSMETSCAN",
               1, 1, 1,
               TRACE_REAL8, TRACE_ASC,
               InfoDetEng->tab_var[ENG_GSMETSCAN].val.pr8, ctx);

  // FlagDateNOK
  SD_BAS_TRACE(tcProcName_F, "ENG_GSMEFLAGDATENOK",
               1, 1, 1,
               TRACE_INT4, TRACE_ASC,
               InfoDetEng->tab_var[ENG_GSMEFLAGDATENOK].val.pui4, ctx);
#endif

fin:
  // sortie de la fonction 
#ifdef OPS_DEBUG
  printf ("Fin de %s\n", tcProcName_F);
#endif
  return(CodeRetour); 
} /* fin de SD_ALG_33_SME */


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_34_SMC
//@
//@ Description : correction angulaire de reflectivite et de polarisation 
//@ du miroir de balayage
//@
//@
//@ Parameters :
//@     (ENTREE)  ctx : contexte de l'etape
//@     (ENTREE)  conf_stable : fichier de conf stable
//@     (ENTREE)  conf_autre : fichier de cof autre
//@     (ENTREE)  data_thread_produit : info generales calculees sur la ligne 
//@                                     LN et LN-1
//@     (ENTREE)  t_scan : temperature du miroir de balayage pour la ligne LN-1
//@                        calculee par 33_SME
//@     (ENTREE)  spectre_corrige_emissivite : spectre corrige de l'emissivite 
//@               du corps noir pour LN SN PN (GHecSpect)
//@               taille = Nslast - Nsfirst + 1;
//@                 Nsfirst = conf_autre->IDefNsfirst[SD_BAS_B1];
//@                 Nslast  = conf_autre->IDefNslast[SD_BAS_B3];
//@     (ENTREE)  InfoDet1ENG : info det du produit ENG 
//@     (SORTIE)  spectre_corrige_reflectivite : spectre 1A pour LN SN PN
//@               corrige de la reflectivite et de la polarisation du miroir
//@               de balayage (GSmcSpect)
//@               taille = Nslast - Nsfirst + 1;
//@                 Nsfirst = conf_autre->IDefNsfirst[SD_BAS_B1];
//@                 Nslast  = conf_autre->IDefNslast[SD_BAS_B3];
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_34_SMC(
    t_SD_BAS_DataThread *         ctx, 
    t_SD_BAS_ConfStable *         conf_stable,
    t_SD_BAS_ConfAutre *          conf_autre, 
    t_SD_BAS_DataThreadProduit *  data_thread_produit,
    t_SD_BAS_Liste_Var *          InfoDet1ENG, 
    double                        t_scan,
    double *                      spectre_corrige_emissivite, 
    double *                      spectre_corrige_reflectivite)

{
  /* Variables locales */
  /*-------------------*/
  char      tcProcName_F[] = "SD_ALG_34_SMC";
  char      LibelleTraces[SD_ALG_SPECTRE_LG_LIBELLE_TRACES + 1];
  t_CMN__Statut CodeRetour = CMN__STATUT_OK;

  /* Numero du premier echantillon spectral */
  int       Nsfirst;
  /* Numero du dernier echantillon spectral */
  int       Nslast;
  /* Nombre d'echantillons spectraux */
  int       TailleSpectreCorrige;
  /* coeff de reflectivite aux positions du miroir de balayage pour */
  /* la calibration du corps noir chaud */
  double *  Rbb;
  /* coeff de reflectivite aux positions du miroir de balayage pour */
  /* la calibration du corps noir chaud */
  double *  Rcs;
  /* coeff de reflectivite aux positions du miroir de balayage pour */
  /* la mesure courante du spectre */
  double *  Rew;
  /* Vecteur des nb d'onde nominaux calibres pour la ligne LN */
  /* calcule par XX_IniProd */
  double *  WnSpect;
  /* Radiance emise par le corps noir chaud pour les echantillons dont */
  /* l'indice est compris entre Nsfirst et Nslast */
  /* calcule par XX_InitProd */
  double *  Bbb;
  /* Compteur */
  int       n;

#ifdef OPS_DEBUG
  FILE *    Fd;
  char      Fichier[256];
  int       i;
#endif
  int     SP;
  int     PN;
  int     CCD;

  //@ Initialisations jlp
  SP = InfoDet1ENG->tab_var[ENG_GEPS_SP].val.pi4[IND1(ctx->ctxtrt.SN)];
  PN                  = ctx->ctxtrt.PN;
  CCD                 = ctx->ctxtrt.CCD + 1;          // Pour pouvoir utiliser les macros
  /* Initialiser les variables locales */
  /*-----------------------------------*/
  Rbb = conf_stable->ISmaRbb;
  Rcs = conf_stable->ISmaRcs;
  Rew = &(conf_stable->ISmaRew[IND2(1,SP,SD_BAS_SS1)]);
  WnSpect = data_thread_produit->WnSpect;
  Bbb = data_thread_produit->Bbb;
  Nsfirst = conf_autre->IDefNsfirst[SD_BAS_B1];
  Nslast  = conf_autre->IDefNslast[SD_BAS_B3];
  TailleSpectreCorrige = Nslast - Nsfirst + 1;

  /* Corps de l'operation */
  /*----------------------*/
#ifdef OPS_DEBUG
  printf ("Debut de %s\n", tcProcName_F);
#endif

#ifdef OPS_DEBUG
  printf (" Nsfirst=%d \t Nslast=%d \t TailleSpectreCorrige=%d\n", 
    Nsfirst, Nslast, TailleSpectreCorrige);
#endif

  //@----------- les calculs suivants ont ete precalcule dans XX_InitProd
  //@ Calcul des nb d'onde des fonctions de calibration spectrale d'indice 
  //@ compris entre NsfirstFcsTab et NslastFcsTab
  //@ WnFcsTab(m) = (m - 1) * DWnFcsTab;
  
  //@ Calcul des positions nominales des echantillons d'indice compris 
  //@ entre Nsfirst et Nslast
  //@ Wn0(n) = (n - 1) * DWnSpect;

  //@ Interpolation lineaire des fonctions de calibration spectrale
  //@ [WnFcsTab, FcsTab] donnent [Wn0, FcsInterpolee]
  //@ appel SD_BAS_InterpLineaire1D
  
  //@ Calibration spectrales des positions nominales des echantillons 
  //@ WnSpect : Vecteur des nb d'onde nominaux calibres
  //@ taille conf_autre->IDefNslast[SB=3] - conf_autre->IDefNsfirst[SB=1] + 1
  //@ occurence : indice de ligne du granule = LN  
  //@ WnSpect(n) = Wn0(n) * FcsInterpolee(n);
  //@----------- 
  
  //@ Correction angulaire de reflectivite et de  polarisation du miroir 
  //@ de balayage
  //@   Spect(n)=Spect(n)- ( Rbb(n)-Rcs(n) ) . Spect(n) 
  //@  *(Planck(WnSpect(n),TScan(NLigne-1))- Bbb(n,NLigne-1)) / Bbb(n,NLigne-1)
  //@  - (Rcs(n)-Rew(n)) . (Planck(WnSpect(n),TScan(NLigne-1))-Spect(n))

// jlp attention ici il faudrait normalement la bbb de la ligne N-1
  for (n = 1 ; n <= TailleSpectreCorrige ; n++)
  {
    spectre_corrige_reflectivite[IND1(n)] = 
      spectre_corrige_emissivite[IND1(n)] - 
      ((Rbb[IND1(n)] - Rcs[IND1(n)]) * spectre_corrige_emissivite[IND1(n)] * 
       (SD_BAS_Planck(
         t_scan, 
         WnSpect[IND3(n,PN,CCD,(conf_autre->IDefNslast[SD_BAS_B3] - conf_autre->IDefNsfirst[SD_BAS_B1] + 1),SD_BAS_PN)], 
         conf_autre->IDefPlkPK1, 
         conf_autre->IDefPlkPK2) - Bbb[IND3(n,PN,CCD,(conf_autre->IDefNslast[SD_BAS_B3] - conf_autre->IDefNsfirst[SD_BAS_B1] + 1),SD_BAS_PN)]
       ) / Bbb[IND3(n,PN,CCD,(conf_autre->IDefNslast[SD_BAS_B3] - conf_autre->IDefNsfirst[SD_BAS_B1] + 1),SD_BAS_PN)]
      ) - (Rcs[IND1(n)] - Rew[IND1(n)]) * 
      (SD_BAS_Planck(
         t_scan, 
         WnSpect[IND3(n,PN,CCD,(conf_autre->IDefNslast[SD_BAS_B3] - conf_autre->IDefNsfirst[SD_BAS_B1] + 1),SD_BAS_PN)], 
         conf_autre->IDefPlkPK1, 
         conf_autre->IDefPlkPK2) - spectre_corrige_emissivite[IND1(n)]
      );
  }

#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_spectre_corrige_emissivite.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (i = 1 ; i <= TailleSpectreCorrige ; i++)
  {
    fprintf (Fd, "%.4le\n", spectre_corrige_emissivite[IND1(i)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

  sprintf (Fichier, "%s/%s/sorties/%s_Rbb.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (i = 1 ; i <= TailleSpectreCorrige ; i++)
  {
    fprintf (Fd, "%.4le\n", Rbb[IND1(i)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

  sprintf (Fichier, "%s/%s/sorties/%s_Rcs.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (i = 1 ; i <= TailleSpectreCorrige ; i++)
  {
    fprintf (Fd, "%.4le\n", Rcs[IND1(i)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

  sprintf (Fichier, "%s/%s/sorties/%s_Rew.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (i = 1 ; i <= TailleSpectreCorrige ; i++)
  {
    fprintf (Fd, "%.4le\n", Rew[IND1(i)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

  sprintf (Fichier, "%s/%s/sorties/%s_SD_BAS_Planck.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (i = 1 ; i <= TailleSpectreCorrige ; i++)
  {
    fprintf (Fd, "%.4le\n", 
      SD_BAS_Planck(
         t_scan, 
         WnSpect[IND1(i)], 
         conf_autre->IDefPlkPK1, 
         conf_autre->IDefPlkPK2));
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
#endif

  /* on trace ici si besoin toutes les variables en sortie */
#ifdef INVESTIGATION
  /* ecrire le spectre corrige de la reflectivite et de la polarisation du */
  /* miroir de balayage */
  sprintf (LibelleTraces, "GSmcSpect");
 // jlp
  SD_BAS_TRACEFICHIERREEL (
    tcProcName_F, 
    &(ctx->ctxtrt), 
    LibelleTraces, 
    1,
    TailleSpectreCorrige, 
    spectre_corrige_reflectivite, 
    ctx);
#endif

fin:
// sortie de la fonction 
#ifdef OPS_DEBUG
  printf ("Fin de %s\n", tcProcName_F);
#endif
return(CodeRetour);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_35_S1B
//@
//@ Description : re-echantillonnage du spectre 
//@
//@
//@ Parameters :
//@     (ENTREE)  ctx : contexte de l'etape
//@     (ENTREE)  conf_autre : fichier de conf autre
//@     (ENTREE)  data_thread_produit : info generales calculees sur la ligne 
//@                                     LN et LN-1
//@     (ENTREE)  sortie_22_SOS : info generales issues de l'algo SOS
//@     (ENTREE)  spectre_1a_surechantillonne : spectre 1A (GS1aOSSpect)
//@               taille TailleWnOSSpect = OSNslast - OSNsfirst + 1;
//@               OSNsfirst = sortie_22_SOS->GSosSsdOSNsFirstSpect;
//@               OSNslast = sortie_22_SOS->GSosSsdOSNsLastSpect;
//@     (Sortie)  spectre_1b : spectre 1B issu de LN SN PN (GS1bSpect)
//@               taille TailleWnSpect1b = Nslast1b - Nsfirst1b + 1;
//@               Nsfirst1b = conf_autre->IDefNsfirst1b;
//@               Nslast1b = conf_autre->IDefNslast1b;
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_35_S1B(
    t_SD_BAS_DataThread *         ctx, 
    t_SD_BAS_ConfAutre *          conf_autre,
    t_SD_BAS_DataThreadProduit *  data_thread_produit,
    t_SD_BAS_22_SOS_Sortie *      sortie_22_SOS,     
    double *                      spectre_1a_surechantillonne,
    double *                      spectre_1b)

{
  /* Variables locales */
  /*-------------------*/
#ifdef INVESTIGATION
  char          LibelleTraces[SD_ALG_SPECTRE_LG_LIBELLE_TRACES + 1];
#endif
  char          tcProcName_F[] = "SD_ALG_35_S1B";
  t_CMN__Statut CodeRetour = CMN__STATUT_OK;

  /* Numero du premier echantillon du spectre surechantillonne */
  int       OSNsfirst;
  /* Numero du dernier echantillon du spectre surechantillonne */
  int       OSNslast;
  /* Nombre d'echantillons du spectre surechantillonne */
  int       TailleWnOSSpect;
  /* Base des nombres d'onde du spectre surechantillonne */
  double *  WnOSSpect;
  /* Numero du premier echantillon du spectre de niveau 1B */
  int       Nsfirst1b;
  /* Numero du dernier echantillon du spectre de niveau 1B */
  int       Nslast1b;
  /* Nombre d'echantillons du spectre de niveau 1B */
  int       TailleWnSpect1b;
  /* Base des nombres d'onde du spectre de niveau 1B */
  double *  WnSpect1b;
  /* Coefficients de la fonction Spline cubique */
  // FT-2765
  AUX_SD_BAS_InterpSpline1D *  CoeffSpline;
  // ...FT-2765
  int     PN;
  int     CCD;

  //@ Initialisations jlp
  PN                  = ctx->ctxtrt.PN;
  CCD                 = ctx->ctxtrt.CCD + 1;          // Pour pouvoir utiliser les macros

  /* Initialiser les variables locales */
  /*-----------------------------------*/
  OSNsfirst = sortie_22_SOS->GSosSsdOSNsFirstSpect;
  OSNslast = sortie_22_SOS->GSosSsdOSNsLastSpect;
  TailleWnOSSpect = OSNslast - OSNsfirst + 1;
  WnOSSpect = data_thread_produit->WnOSSpect;

  Nsfirst1b = conf_autre->IDefNsfirst1b;
  Nslast1b = conf_autre->IDefNslast1b;
  TailleWnSpect1b = Nslast1b - Nsfirst1b + 1;
  WnSpect1b = data_thread_produit->WnSpect1b;

  CoeffSpline = data_thread_produit->CoeffSpline;

  /* Corps de l'operation */
  /*----------------------*/
#ifdef OPS_DEBUG
  printf ("Debut de %s\n", tcProcName_F);
#endif

  //@----------- les calculs suivants ont ete precalcule dans XX_InitProd
  //@ Calcul des positions nominales des echantillons 1B d'indice compris 
  //@ entre Nsfirst1b et Nslast1b
  //@ WnSpect1b(n) = (n - 1) * DWnSpect1b;
  
  //@ Calcul des nb d'onde des fonctions de calibration spectrale d'indice 
  //@ compris entre NsfirstFcsTab et NslastFcsTab
  //@ WnFcsTab(m) = (m - 1) * DWnFcsTab;

  //@ WnOSSpect : calibration spectrale du vecteur des nb d'onde du spectre 
  //@ surechantillonne d'indice compris entre OSNsfirst et OSNslast 
  //@ WnOSSpect(n) = (n - 1) * DWnOSSpect;

  //@ Interpolation de la fonction de calibration spectrale sur la base 
  //@ surechantillonnee
  //@ [WnFcsTab(n), FcsTab(n)] donnent  [WnOSSpect(m), OSFcs(m)] 
  //@  appel SD_BAS_InterpLineaire1D

  //@ Calibration spectrale de la base surechantillonnee
  //@  pour n variant de OSNsfirst a OSNslast
  //@   WnOSSpect(n) = WnOSSpect(n) . OSFcs(n)
  //@----------- 
  
  //@ Extraction par interpolation spline cubique des echantillons spectraux 
  //@ sur la base d'echantillonnage du spectre de niveau 1b
  //@   [WnOSSpect(n), OSSpect(n)] donnent  [WnSpect(n), Spect(n)] 
  //@ par interpolation spline cubique
  //@  appel SD_BAS_InterpSpline1D
  
  //@ Interpolation par spline : initialisation de la fonction
// jlp
  CodeRetour = SD_BAS_InterpSpline1D (
                SD_ALG_INIT_SPLINE, 
                TailleWnOSSpect, 
                &WnOSSpect[IND3(1,PN,CCD,(data_thread_produit->GSosSsdOSNsLastSpect - data_thread_produit->GSosSsdOSNsFirstSpect + 1),SD_BAS_PN)], 
                spectre_1a_surechantillonne, 
                CoeffSpline, 
                TailleWnSpect1b, 
                WnSpect1b, 
                spectre_1b);
  
  SD_BAS_THROW(
    CodeRetour, SD_ALG_SPECTRE_35_S1B_ALRT_INIT_SPLINE, 
    CodeRetour, BAS_CR_NEQ, CMN__STATUT_OK, "");

  //@ Interpolation par spline : calcul
  CodeRetour = SD_BAS_InterpSpline1D (
                SD_ALG_CALCUL_SPLINE, 
                TailleWnOSSpect, 
                &WnOSSpect[IND3(1,PN,CCD,(data_thread_produit->GSosSsdOSNsLastSpect - data_thread_produit->GSosSsdOSNsFirstSpect + 1),SD_BAS_PN)], 
                spectre_1a_surechantillonne, 
                CoeffSpline, 
                TailleWnSpect1b, 
                WnSpect1b, 
                spectre_1b);

  SD_BAS_THROW(
    CodeRetour, SD_ALG_SPECTRE_35_S1B_ALRT_INTERP_SPLINE, 
    CodeRetour, BAS_CR_NEQ, CMN__STATUT_OK, "");

  /* on trace ici si besoin toutes les variables en sortie */
#ifdef INVESTIGATION
  /* ecrire le spectre 1B dans un fichier binaire */
  sprintf (LibelleTraces, "GS1bSpect");
 // jlp
  SD_BAS_TRACE (
    tcProcName_F, 
    LibelleTraces,
    1, 
    TailleWnSpect1b, 
    1, 
    TRACE_REAL8, 
    TRACE_BIN, 
    spectre_1b, 
    ctx);
#endif
  
fin:
// sortie de la fonction 
  switch (CodeRetour)
  {
    case SD_ALG_SPECTRE_35_S1B_ALRT_INIT_SPLINE:
    {
#ifdef OPS_DEBUG
  printf (" Erreur d'initialisation de SD_BAS_InterpSpline1D.\n");
#endif
      /* "ERROR ON INITIALIZATION OF THE SPLINE INTERPOLATION OF SPECTRUM 1A" defini dans SD.msg */
      SD_ALG_SPECTRE_MsgSet = SD_ALG_SPECTRE_SET; /* defini dans SD__Message.h */
      SD_ALG_SPECTRE_MsgNum = MSG_ALG_SPECTRE_35_S1B_ALRT_INIT_SPLINE; /* defini dans l'enumere e_SPECMessageJDB de SD__Message.h */
      CreerMessage (
        SD_ALG_SPECTRE_MsgSet, 
        SD_ALG_SPECTRE_MsgNum, 
        &(ctx->cr.msgJDB));
      CodeRetour = CMN__STATUT_NOK;
    }
    break;
    
    case SD_ALG_SPECTRE_35_S1B_ALRT_INTERP_SPLINE:
    {
#ifdef OPS_DEBUG
  printf (" Erreur d'interpolation par SD_BAS_InterpSpline1D.\n");
#endif
      /* "ERROR ON THE SPLINE INTERPOLATION OF SPECTRUM 1A" defini dans SD.msg */
      SD_ALG_SPECTRE_MsgSet = SD_ALG_SPECTRE_SET; /* defini dans SD__Message.h */
      SD_ALG_SPECTRE_MsgNum = MSG_ALG_SPECTRE_35_S1B_ALRT_INIT_SPLINE; /* defini dans l'enumere e_SPECMessageJDB de SD__Message.h */
      CreerMessage (
        SD_ALG_SPECTRE_MsgSet, 
        SD_ALG_SPECTRE_MsgNum, 
        &(ctx->cr.msgJDB));
      CodeRetour = CMN__STATUT_NOK;
    }
    break;

    default :
    {
      if (CodeRetour != CMN__STATUT_OK)
      {
        CodeRetour = CMN__STATUT_NOK;
      }
    }
  } /* fin switch */
    
#ifdef OPS_DEBUG
  printf ("Fin de %s\n", tcProcName_F);
#endif
return(CodeRetour); 
}

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_37_S1C
//@
//@ Description : Apodisation  du spectre 
//@
//@
//@ Parameters :
//@     (ENTREE)  ctx : contexte de l'etape numero de ligne LN, SN et PN
//@     (ENTREE)  BanqueSpectrale : pointeur sur le fichier BanqueSpectrale
//@     (ENTREE)  conf_autre : fichier de conf autre
//@     (ENTREE)  data_thread_produit : info generales calculees sur la ligne LN
//@               dont Af (GIsfAf)
//@     (ENTREE)  spectre_1b: spectre 1B issu de LN SN PN (GS1bSpect)
//@               taille TailleWnSpect1b = Nslast1b - Nsfirst1b + 1;
//@               Nsfirst1b = conf_autre->IDefNsfirst1b;
//@               Nslast1b = conf_autre->IDefNslast1b;
//@     (SORTIE)  spectre_1c : spectre 1C issu de LN SN PN (GS1cSpect)
//@               taille TailleWnSpect1b = Nslast1b - Nsfirst1b + 1;
//@               Nsfirst1b = conf_autre->IDefNsfirst1b;
//@               Nslast1b = conf_autre->IDefNslast1b;
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_37_S1C(
    t_SD_BAS_DataThread *         ctx, 
    t_SD_BAS_BanqueSpectrale *    banque_spectrale,
    t_SD_BAS_ConfAutre *          conf_autre,
    t_SD_BAS_DataThreadProduit *  data_thread_produit,
    double *                      spectre_1b,
    double *                      spectre_1c)

{
  /* Variables locales */
  /*-------------------*/
#ifdef INVESTIGATION
  char          LibelleTraces[SD_ALG_SPECTRE_LG_LIBELLE_TRACES + 1];
#endif
  char          tcProcName_F[] = "SD_ALG_37_S1C";
  t_CMN__Statut CodeRetour = CMN__STATUT_OK;

  /* Precedent echantillon spectral correspondant au point d'application de */
  /* la fonction d'apodisation */
  int           nWn1;
  /* Echantillon spectral correspondant au point d'application de */
  /* la fonction d'apodisation */
  int           nWn2;
  /* Position en nb d'onde des fonctions d'apodisation pour n allant de */
  /* NsfirstAfTab a NslastAfTab */
  double *      WnAf;
  /* Indice des fonctions d'apodisation */
  int           naf;
  /* Largeur d'un echantillon du spectre 1B */
  double        DWnSpect1b;
  /* Numero du premier echantillon du spectre de niveau 1B */
  int           Nsfirst1b;
  /* Numero du dernier echantillon du spectre de niveau 1B */
  int           Nslast1b;
  /* Nombre d'echantillons du spectre de niveau 1B */
  int           TailleWnSpect1b;
  /* Spectre 1B prolonge par continuite pour gerer l'extraction de la 1ere et */
  /* de la derniere fenetre spectrale */
  double *      Spectre1bProlonge;
  /* Numero du premier echantillon du spectre de niveau 1B prolonge */
  int           Nsfirst1bProlonge;
  /* Numero du dernier echantillon du spectre de niveau 1B prolonge */
  int           Nslast1bProlonge;
  /* Nombre d'echantillons du spectre de niveau 1B prolonge */
  int           TailleWnSpect1bProlonge;
  /* Nombre de zeros qui completent une fenetre spectrale a gauche */
  int           NbZerosGauche;
  /* Largeur d'un echantillon du spectre 1C */
  double        DWnSpect1c;
  /* Numero du premier echantillon du spectre de niveau 1C */
  int           Nsfirst1c;
  /* Numero du dernier echantillon du spectre de niveau 1C */
  int           Nslast1c;
  /* Nombre d'echantillons du spectre de niveau 1C */
  int           TailleWnSpect1c;
  /* Nb d'echantillons compris dans la demie largeur de la fenetre spectrale */
  int           NbSWwidth;
  /* Nombre d'echantillons de la fenetre spectrale */
  int           TailleFenetreSpectrale;
  /* Demie largeur a mi-hauteur de la Gaussienne d'amortissement du spectre */
  int           SigS;
  /* Numero de la premiere fonction spectrale */
  int           NsfirstAfTab;
  /* Numero de la derniere fonction spectrale */
  int           NslastAfTab;
#ifdef OPS_DEBUG
  /* Nb de fonctions d'apodisation spectrale */
  int           TailleWnAfTab;
#endif
  /* Fonction Gaussienne d'amortissement de la fenetre spectrale */
  double *      FctAmortissement;
  /* Tableau temporaire du noyau pour le calcul de la fonction d'amortissement */
  double *      G;
  /* Fenetre spectrale limitee a 2*conf_autre->IDefS1cNbSWwidth+1 echantillons */
  double *      S;
  /* Parametre d'initialisation des FFT */
  double        Scale;
  /* Fenetre spectrale completee par des zeros pour atteindre pour atteindre */
  /* une taille qui soit une puissance de 2 */
  double *      SpectrePadde;
  /* Moitie de la TF du spectre reel renvoyee par la librairie ESSL */
  /* (cf symetrie Hermitienne) */
  dcmplx *      MoitieTFSpectrePadde;
  /* Transformee du spectre apres apodisation et zero-padding */
  dcmplx *      TFSpectrePadde;
  /* FFT de la fenetre spectrale 1B apodisee */
  dcmplx *      SAtf;
  dcmplx *      SAtfArrangee;
  /* Fonctions d'apodisation calculees par 43_ISF */
  dcmplx *      Af;
  /* FFT inverse de SAtf */
  dcmplx *      TFInvSAtf;
  /* Partie reelle de la FFT inverse de SAtf */
  double *      ReTFInvSAtf;
  /* Partie reelle de la FFT inverse de SAtf apres rearrangement des coeff */
  double *      SA2Padde;
  /* Fenetres spectrales successives du spectre 1C */
  double *      SA1;
  double *      SA2;
  /* Pointeur sur SA1 */
  double *      PtrSA1;
  /* Pointeur sur le premier echantillon de la fenetre spectrale du spectre 1B */
  double *      Spect1bShift;
  /* Indice du premier echantillon de la fenetre spectrale */
  int           Nfirst;
  /* Indice du dernier echantillon de la fenetre spectrale */
  int           Nlast;
  /* Taille du spectre en entree de la FFT */
  int           nFFT;
  /* nFFT / 2 + 1 */
  int           nFFT_2;
  /* Tailles des tableaux auxiliaires utilises pour le calcul des FFT */
  int           naux1;
  int           naux2;
  /* Poids de la combinaison lineaire des spectres SA1 et SA2 */
  double        Alpha;
  double        Beta;
  /* Compteurs */
  int           i, m, n;

#ifdef OPS_DEBUG
  FILE *        Fd;
  char          Fichier[256];
  /* Largeur d'un echantillon de base des nb d'onde des fonctions d'apodisation spectrale */
  double        DWnAfTab;
#endif
  int     PN;
  int     CCD;

  //@ Initialisations jlp
  PN                  = ctx->ctxtrt.PN;
  CCD                 = ctx->ctxtrt.CCD + 1;          // Pour pouvoir utiliser les macros

  /* Initialiser les variables locales */
  /*-----------------------------------*/
  Af = data_thread_produit->Af;
  NsfirstAfTab = banque_spectrale->IDefSafNsfirst;
  NslastAfTab = banque_spectrale->IDefSafNslast;
#ifdef OPS_DEBUG
  TailleWnAfTab = NslastAfTab - NsfirstAfTab + 1;
#endif
  WnAf = data_thread_produit->WnAf;

  DWnSpect1b = conf_autre->IDefSpectDWn1b;
  Nsfirst1b = conf_autre->IDefNsfirst1b;
  Nslast1b = conf_autre->IDefNslast1b;
  TailleWnSpect1b = Nslast1b - Nsfirst1b + 1;
  
  Spectre1bProlonge = data_thread_produit->Spectre1bProlonge;

#ifdef INVESTIGATION
  DWnSpect1c = conf_autre->IDefSpectDWn1b;
  Nsfirst1c = conf_autre->IDefNsfirst1b;
  Nslast1c = conf_autre->IDefNslast1b;
  TailleWnSpect1c = Nslast1c - Nsfirst1c + 1;
#endif
  
  NbSWwidth = conf_autre->IDefS1cNbSWwidth;
  TailleFenetreSpectrale = 2 * NbSWwidth;
  SigS = conf_autre->IDefS1cSigS;
  FctAmortissement = data_thread_produit->FctAmortissement;
  G = data_thread_produit->G;
  S = data_thread_produit->S;
  SpectrePadde = data_thread_produit->SpectrePadde;
  MoitieTFSpectrePadde = data_thread_produit->MoitieStf;
  TFSpectrePadde = data_thread_produit->Stf;
  SAtf = data_thread_produit->SAtf;
  SAtfArrangee = data_thread_produit->SAtfArrangee;
  TFInvSAtf = data_thread_produit->TFInvSAtf;
  ReTFInvSAtf = data_thread_produit->ReTFInvSAtf;
  SA2Padde = data_thread_produit->SA2Padde;
  SA1 = data_thread_produit->SA1;
  SA2 = data_thread_produit->SA2;
  
  Nsfirst1bProlonge = Nsfirst1b - NbSWwidth;
  Nslast1bProlonge = Nslast1b + NbSWwidth;
  TailleWnSpect1bProlonge = Nslast1bProlonge - Nsfirst1bProlonge + 1;

  Nfirst = 1;
  Nlast = TailleFenetreSpectrale;

  nFFT = SD_BAS_CalculPuissance2Sup (TailleFenetreSpectrale);
  nFFT_2 = nFFT / 2 + 1;

  NbZerosGauche = (nFFT - TailleFenetreSpectrale + 1) / 2;

#ifdef OPS_DEBUG
  DWnAfTab = banque_spectrale->IDefSafDWn;
#endif

  /* Corps de l'operation */
  /*----------------------*/
#ifdef OPS_DEBUG
  printf ("Debut de %s\n", tcProcName_F);
#endif

#ifdef OPS_DEBUG
  printf (" NbSWwidth=%d\n", NbSWwidth);
  printf (" Nsfirst1b=%d\t\tNslast1b=%d\t\tTailleWnSpect1b=%d\tDWnSpect1b=%.2e\n", 
    Nsfirst1b, Nslast1b, TailleWnSpect1b, DWnSpect1b);
  printf (" Nsfirst1c=%d\t\tNslast1c=%d\t\tTailleWnSpect1c=%d\tDWnSpect1c=%.2e\n", 
    Nsfirst1c, Nslast1c, TailleWnSpect1c, DWnSpect1c);
  printf (" NsfirstAfTab=%d\tNslastAfTab=%d\t\tTailleWnAfTab=%d\tDWnAfTab=%.2e\n", 
    NsfirstAfTab, NslastAfTab, TailleWnAfTab, DWnAfTab);
  printf (" TailleFenetreSpectrale=%d\n", TailleFenetreSpectrale);
  printf (" SigS=%.4le\n", SigS);
  printf (" Nsfirst1bProlonge=%d \t Nslast1bProlonge=%d \t TailleWnSpect1bProlonge=%d\n", 
    Nsfirst1bProlonge, Nslast1bProlonge, TailleWnSpect1bProlonge);
  printf (" nFFT=%d\n", nFFT);
  printf (" IDefS1bNsFFT=%d\n", conf_autre->IDefS1bNsFFT);
  printf (" nFFT_2=%d\n", nFFT_2);
  printf (" NbZerosGauche=%d\n", NbZerosGauche);
#endif

  //@----------- les calculs suivants ont ete precalcule dans XX_InitProd
  //@ Position en nombre d'onde des fonctions d'apodisation 
  //@ n de NsfirstAfTab a NslastAfTab
  //@ WnAf(n) = (n-1) . DWnAfTab
  //@---
  
  //@ Prolongement par continuite de 2*NbSWwidth echantillons de part et d'autre
  //@ du spectre 1B
#ifdef OPS_DEBUG
  printf (" spectre_1b[1]=%.4le\t", spectre_1b[IND1(1)]);
  printf (" spectre_1b[%d]=%.4le\n", 
    TailleWnSpect1b, spectre_1b[IND1(TailleWnSpect1b)]);
#endif

  for (i = 1 ; i <= TailleWnSpect1bProlonge ; i++)
  {
    if (i <= NbSWwidth)
    {
      Spectre1bProlonge[IND1(i)] = spectre_1b[IND1(1)];
    }
    else
    {
      if (i > TailleWnSpect1b + NbSWwidth)
      {
        Spectre1bProlonge[IND1(i)] = spectre_1b[IND1(TailleWnSpect1b)];
      }
      else
      {
        Spectre1bProlonge[IND1(i)] = spectre_1b[IND1(i - NbSWwidth)];
      }
    }
  }

#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_spectre_1b.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (i = 1 ; i <= TailleWnSpect1b ; i++)
  {
    fprintf (Fd, "%.4le\n", spectre_1b[IND1(i)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

  sprintf (Fichier, "%s/%s/sorties/%s_Spectre1bProlonge.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (i = 1 ; i <= TailleWnSpect1bProlonge ; i++)
  {
    fprintf (Fd, "%.4le\n", Spectre1bProlonge[IND1(i)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
#endif

  //@ Recherche de l'echantillon spectral correspondant au point d'application
  //@  de la premiere fonction d'auto-apodisation 
  //@   nWn2 = WnAf(NsfirstAfTab) / DWnSpect + 1
  nWn2 = NINT(WnAf[NsfirstAfTab - NsfirstAfTab] / DWnSpect1b) + 1;

  //@ Calcul de la fonction d'amortissement a appliquer sur chaque fenetre
  //@ extraite du spectre 1B prolonge
  CodeRetour = SD_BAS_FctAmortissement (
                TailleFenetreSpectrale, 
                SigS, 
                DWnSpect1b, 
                FctAmortissement, 
                G);
  
  SD_BAS_THROW(
    CodeRetour, SD_ALG_SPECTRE_37_S1C_ALRT_FCT_AMORTISST, 
    CodeRetour, BAS_CR_NEQ, CMN__STATUT_OK, "");
                
#ifdef OPS_DEBUG
  sprintf (Fichier,  "%s/%s/sorties/%s_FctAmortissement.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (i = 1 ; i <= TailleFenetreSpectrale ; i++)
  {
    fprintf (Fd, "%.4le\n", FctAmortissement[IND1(i)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
#endif

  //@ Boucle sur les fonctions d'auto-apodisation indicees par le numero
  //@ du nombre d'onde naf = NsfirstAfTab a NslastAfTab
  for (naf = NsfirstAfTab ; naf <= NslastAfTab ; naf++)
  {
    //@ Recherche de l'echantillon spectral correspondant au point 
    //@ d'application de la fonction d'apodisation naf
    //@ nWn1 = nWn2
    //@ nWn2 = WnAf(naf) / DWnSpect + 1
    nWn1 = nWn2;  /* on memorise la position precedente du nb d'onde pour */
                  /* l'interpolation lineaire */
    nWn2 = NINT(WnAf[naf - NsfirstAfTab] / DWnSpect1b) + 1;
    
    //@   Extraction de la fenetre spectrale avec gestion des bords de spectre
    //@     S(1) = Spect1b(nWn2 - NbSWwidth)
    //@     a
    //@     S(2NbSWwidth) = Spect1b(nWn2 + NbSWwidth-1)
    //@   Amortissement des bords de la fenetre spectrale par une gaussienne de 
    //@   demi-largeur  a mi-hauteur SigS
    //@     appel SD_BAS_Amortissement
    Spect1bShift = Spectre1bProlonge + nWn2 - NbSWwidth - Nsfirst1bProlonge;

    for (i = 1 ; i <= TailleFenetreSpectrale ; i++)
    {
      S[IND1(i)] = Spect1bShift[IND1(i)] * FctAmortissement[IND1(i)];
    }
    
#ifdef OPS_DEBUG
    if (naf == NsfirstAfTab)
    {
      printf (" naf=%d \t NsfirstAfTab=%d \t nWn1=%d \t nWn2=%d\n", 
        naf, NsfirstAfTab, nWn1, nWn2);
      
      sprintf (Fichier, "%s/%s/sorties/%s_Spect1bShift.txt", 
        getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
      Fd = fopen (Fichier, "wt");
      for (i = 1 ; i <= TailleFenetreSpectrale ; i++)
      {
        fprintf (Fd, "%.4le\n", Spect1bShift[IND1(i)]);
      }
      fclose (Fd);
      printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

      sprintf (Fichier, "%s/%s/sorties/%s_S.txt", 
        getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
      Fd = fopen (Fichier, "wt");
      for (i = 1 ; i <= TailleFenetreSpectrale ; i++)
      {
        fprintf (Fd, "%.4le\n", S[IND1(i)]);
      }
      fclose (Fd);
      printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
    }  
#endif

    //@ Transformee de Fourier directe de S centree sur nWn2 et 
    //@ de largeur 2.NbSWwidth+1
    //@ Stf(m) = TF[S(n)]
    //@   appel SD_BAS_FFT1DR2C 
    
    /* On noie le spectre de taille nbpts dans un tableau de taille nFFT */
    /* tel que nFFT soit la puissance de 2 immediatement superieure a */
    /* la taille de la fenetre spectrale */
    /* Rearrangement avant la FFT */    
    CodeRetour = SD_BAS_PreparationFFTR(
                  ctx, 
                  S, 
                  Nfirst, 
                  Nlast, 
                  SpectrePadde, 
                  nFFT);
    
    sprintf (SD_ALG_SPECTRE_ValeurParam, "%d", nWn2);
    SD_BAS_THROW(
      CodeRetour, SD_ALG_SPECTRE_37_S1C_ALRT_ZERO_PADDING, 
      CodeRetour, BAS_CR_NEQ, CMN__STATUT_OK, "");

#ifdef OPS_DEBUG
    if (naf == NsfirstAfTab)
    {
      sprintf (Fichier, "%s/%s/sorties/%s_SpectrePadde.txt", 
        getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
      Fd = fopen (Fichier, "wt");
      for (i = 1 ; i <= nFFT ; i++)
      {
        fprintf (Fd, "%.4le\n", SpectrePadde[IND1(i)]);
      }
      fclose (Fd);
      printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
    }
#endif

  if(nFFT <= 2048)
        {
            naux1 = 22000;                          // cf doc lib ESSL p. 738 730 (max dcft drcft)
            naux2 = 20000;
        }
   else
        {
            naux1 = (int)(20000 + 2.28*nFFT);       // cf doc lib ESSL p. 738 730 (max dcft drcft)
            naux2 = (int)(20000 + 2.28*nFFT);
        }

    //@ Initialisation de la FFT reel->complexe
    Scale = 1.0;        /* Pour la FFT directe */
   if (data_thread_produit->CalculFFTS1C == 0 )
     {
     CodeRetour = SD_BAS_FFT1DirDR2C(
                  ctx, 
                  SD_ALG_INIT_FFT, 
                  &nFFT, 
                  1, 
                  SpectrePadde, 
                  data_thread_produit->aux1, 
                  data_thread_produit->aux2, 
                  naux1, 
                  naux2, 
                  Scale, 
                  MoitieTFSpectrePadde);
    
     SD_BAS_THROW(
      CodeRetour, SD_ALG_SPECTRE_37_S1C_ALRT_INIT_FFT_DIRECTE, 
      CodeRetour, BAS_CR_NEQ, CMN__STATUT_OK, "");
      }

#ifdef OPS_DEBUG
    if (naf == NsfirstAfTab)
    {
      printf(" FFT initialisee\n");
    }
#endif
    
    //@ Calcul de la FFT => LibEssl ne renvoie que la moitie des coeffs en 
    //@ raison de la symetrie Hermitienne
    CodeRetour = SD_BAS_FFT1DirDR2C(
                  ctx, 
                  SD_ALG_CALCUL_FFT, 
                  &nFFT, 
                  1, 
                  SpectrePadde, 
                  data_thread_produit->aux1, 
                  data_thread_produit->aux2, 
                  naux1, 
                  naux2, 
                  Scale, 
                  MoitieTFSpectrePadde);

    SD_BAS_THROW(
      CodeRetour, SD_ALG_SPECTRE_37_S1C_ALRT_CALCUL_FFT_DIRECTE, 
      CodeRetour, BAS_CR_NEQ, CMN__STATUT_OK, "");

#ifdef OPS_DEBUG
    if (naf == NsfirstAfTab)
    {
      printf(" FFT effectuee\n");
      sprintf (Fichier, "%s/%s/sorties/%s_MoitieTFSpectrePadde.txt", 
        getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
      Fd = fopen (Fichier, "wt");
      for (i = 1 ; i <= nFFT_2 ; i++)
      {
        fprintf (Fd, "%.4le\t%.4le\n", 
          RE(MoitieTFSpectrePadde[IND1(i)]), IM(MoitieTFSpectrePadde[IND1(i)]));
      }
      fclose (Fd);
      printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
    }
#endif
    
    //@ Duplication du spectre par symetrie hermitienne pour se ramener a un 
    //@ spectre centre
    /* Frequences positives */
    for(i = 1; i <= nFFT_2 - 1 ; i++)
    {
      RE(TFSpectrePadde[IND1(nFFT_2) + IND1(i)]) = RE(MoitieTFSpectrePadde[IND1(i)]);
      IM(TFSpectrePadde[IND1(nFFT_2) + IND1(i)]) = IM(MoitieTFSpectrePadde[IND1(i)]);
    }

    /* fe/2 */
    IM(TFSpectrePadde[IND1(1)]) = IM(MoitieTFSpectrePadde[IND1(nFFT_2)]);
    RE(TFSpectrePadde[IND1(1)]) = RE(MoitieTFSpectrePadde[IND1(nFFT_2)]);

    /* Frequences negatives : */
    /* symetrie hermitienne = partie RE paire, partie IM impaire */
    for(i = 2; i <= nFFT_2 - 1; i++)
    {
      RE(TFSpectrePadde[IND1(i)]) = 
        RE(MoitieTFSpectrePadde[IND1(nFFT_2) - IND1(i)]);
      IM(TFSpectrePadde[IND1(i)]) = 
        -IM(MoitieTFSpectrePadde[IND1(nFFT_2) - IND1(i)]);
    }

#ifdef OPS_DEBUG
    if (naf == NsfirstAfTab)
    {
      sprintf (Fichier, "%s/%s/sorties/%s_TFSpectrePadde.txt", 
        getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
      Fd = fopen (Fichier, "wt");
      for (i = 1 ; i <= nFFT ; i++)
      {
        fprintf (Fd, "%.4le\t%.4le\n", 
          RE(TFSpectrePadde[IND1(i)]), IM(TFSpectrePadde[IND1(i)]));
      }
      fclose (Fd);
      printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);

      sprintf (Fichier, "%s/%s/sorties/%s_Af.txt", 
        getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
      Fd = fopen (Fichier, "wt");
      for (i = 1 ; i <= nFFT ; i++)
      {
        if ( (i <= NbZerosGauche) || (i > TailleFenetreSpectrale + NbZerosGauche) )
        {
          fprintf (Fd, "%.4le\t%.4le\n", 0.0, 0.0);
        }
        else
        {
          fprintf (Fd, "%.4le\t%.4le\n", 
            RE(Af[IND2(i, naf - NsfirstAfTab + 1, TailleFenetreSpectrale+1)]), 
            IM(Af[IND2(i, naf - NsfirstAfTab + 1, TailleFenetreSpectrale+1)]));
        }
      }
      fclose (Fd);
      printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
    }
#endif
    
    //@ Produit par la fonction d'apodisation
    //@   SAtf(m) = Stf(m).Af(m, naf)
    for (m = 1 ; m <= nFFT ; m++)
    {
      if ( (m <= NbZerosGauche) || (m > TailleFenetreSpectrale + NbZerosGauche) )
      {
        /* Pour eviter de faire du zero-padding sur Af(naf) */
        RE(SAtf[IND1(m)]) = 0.0;
        IM(SAtf[IND1(m)]) = 0.0;
      }
      else
      {
        RE(SAtf[IND1(m)]) = 
          RE(TFSpectrePadde[IND1(m)]) * 
          RE(Af[IND4(m - NbZerosGauche, naf - NsfirstAfTab + 1,PN,CCD,TailleFenetreSpectrale+1,SD_BAS_SDB_NFSMAX,SD_BAS_PN)]) - 
          IM(TFSpectrePadde[IND1(m)]) * 
          IM(Af[IND4(m - NbZerosGauche, naf - NsfirstAfTab + 1,PN,CCD,TailleFenetreSpectrale+1,SD_BAS_SDB_NFSMAX,SD_BAS_PN)]);
        IM(SAtf[IND1(m)]) = 
          IM(TFSpectrePadde[IND1(m)]) * 
          RE(Af[IND4(m - NbZerosGauche, naf - NsfirstAfTab + 1,PN,CCD, TailleFenetreSpectrale+1,SD_BAS_SDB_NFSMAX,SD_BAS_PN)]) + 
          RE(TFSpectrePadde[IND1(m)]) * 
          IM(Af[IND4(m - NbZerosGauche, naf - NsfirstAfTab + 1,PN,CCD, TailleFenetreSpectrale+1,SD_BAS_SDB_NFSMAX,SD_BAS_PN)]);
      }
    }

#ifdef OPS_DEBUG
    if (naf == NsfirstAfTab)
    {
     sprintf (Fichier, "%s/%s/sorties/%s_SAtf.txt", 
        getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
      Fd = fopen (Fichier, "wt");
      for (m = 1 ; m <= nFFT ; m++)
      {
        fprintf (Fd, "%.4le\t%.4le\n", 
          RE(SAtf[IND1(m)]), IM(SAtf[IND1(m)]));
      }
      fclose (Fd);
      printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
    }
#endif
    
    //@ Transformee de Fourier inverse de SAtf
    //@   appel SD_BAS_FFTInv1DC2R 
    //@   SA2(n) = Partie Reelle[TF-1[SAtf(m)]]
    //@ Rearrangement du spectre
    CodeRetour = SD_BAS_PreparationFFTC(
                  ctx, 
                  SAtf, 
                  1, 
                  nFFT, 
                  SAtfArrangee, 
                  nFFT);

    SD_BAS_THROW(
      CodeRetour, SD_ALG_SPECTRE_37_S1C_ALRT_PREPARE_INTERFEROGRAM, 
      CodeRetour, BAS_CR_NEQ, CMN__STATUT_OK, "");
    
#ifdef OPS_DEBUG
    if (naf == NsfirstAfTab)
    {
      printf (" Preparation FFT inverse terminee\n");
      sprintf (Fichier, "%s/%s/sorties/%s_SAtfArrangee.txt", 
        getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
      Fd = fopen (Fichier, "wt");
      for (i = 1 ; i <= nFFT ; i++)
      {
        fprintf (Fd, "%.4le\t%.4le\n", 
          SAtfArrangee[IND1(i)]);
      }
      fclose (Fd);
      printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
    }
#endif

    //@ Initialisation de la FFT inverse complexe->complexe
    Scale = 1.0/nFFT; /* Pour la FFT inverse */
    if (data_thread_produit->CalculFFTS1C == 0 )
     {
     CodeRetour = SD_BAS_FFT1DirInvDC2C(
                  ctx, 
                  -1, 
                  SD_ALG_INIT_FFT, 
                  &nFFT, 
                  1, 
                  SAtfArrangee,
                  data_thread_produit->aux12, 
                  data_thread_produit->aux22, 
                  naux1, 
                  naux2,
                  Scale, 
                  TFInvSAtf);

     SD_BAS_THROW(
      CodeRetour, SD_ALG_SPECTRE_37_S1C_ALRT_INIT_FFT_INVERSE,  
      CodeRetour, BAS_CR_NEQ, CMN__STATUT_OK, "");
      
      data_thread_produit->CalculFFTS1C = 1;
      }

#ifdef OPS_DEBUG
    if (naf == NsfirstAfTab)
    {
      printf (" FFT inverse initialisee\n");
    }
#endif
    
    //@ FFT inverse complexe->complexe
    CodeRetour = SD_BAS_FFT1DirInvDC2C(
                  ctx, 
                  -1, 
                  SD_ALG_CALCUL_FFT, 
                  &nFFT, 
                  1, 
                  SAtfArrangee,
                  data_thread_produit->aux12, 
                  data_thread_produit->aux22, 
                  naux1, 
                  naux2, 
                  Scale, 
                  TFInvSAtf);

    SD_BAS_THROW(
      CodeRetour, SD_ALG_SPECTRE_37_S1C_ALRT_CALCUL_FFT_INVERSE,  
      CodeRetour, BAS_CR_NEQ, CMN__STATUT_OK, "");
    
#ifdef OPS_DEBUG
    if (naf == NsfirstAfTab)
    {
      sprintf (Fichier, "%s/%s/sorties/%s_TFInvSAtf.txt", 
        getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
      Fd = fopen (Fichier, "wt");
      for (i = 1 ; i <= nFFT ; i++)
      {
        fprintf (Fd, "%.4le\t%.4le\n", 
          RE(TFInvSAtf[IND1(i)]), IM(TFInvSAtf[IND1(i)]));
      }
      fclose (Fd);
      printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
    }
#endif
    
    //@ Recuperation de la partie reelle
    for(i = 1; i <= nFFT; i++)
    {
        ReTFInvSAtf[IND1(i)] = RE(TFInvSAtf[IND1(i)]);
    }

    //@ Rearrangement du spectre
    CodeRetour = SD_BAS_PreparationFFTR(
                  ctx, 
                  ReTFInvSAtf, 
                  1, 
                  nFFT, 
                  SA2Padde, 
                  nFFT);
    
    SD_BAS_THROW(
      CodeRetour, SD_ALG_SPECTRE_37_S1C_ALRT_ARRANGT_SPECTRE_1C,  
      CodeRetour, BAS_CR_NEQ, CMN__STATUT_OK, "");

#ifdef OPS_DEBUG
    if (naf == NsfirstAfTab)
    {
      sprintf (Fichier, "%s/%s/sorties/%s_SA2Padde.txt", 
        getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
      Fd = fopen (Fichier, "wt");
      for (i = 1 ; i <= nFFT ; i++)
      {
        fprintf (Fd, "%.4le\n", SA2Padde[IND1(i)]);
      }
      fclose (Fd);
      printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
    }
#endif
    
    //@ Extraction du spectre apodise
    for (i = 1 ; i <= TailleFenetreSpectrale ; i++)
    {
      SA2[IND1(i)] = SA2Padde[IND1(i + NbZerosGauche)];
    }

#ifdef OPS_DEBUG
    if (naf == NsfirstAfTab)
    {
      sprintf (Fichier, "%s/%s/sorties/%s_SA2.txt", 
        getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
      Fd = fopen (Fichier, "wt");
      for (i = 1 ; i <= TailleFenetreSpectrale ; i++)
      {
        fprintf (Fd, "%.4le\n", SA2[IND1(i)]);
      }
      fclose (Fd);
      printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
    }
#endif
    
    //@ Si naf different de NsfirstAfTab  alors :
    if (naf != NsfirstAfTab)
    {
      //@ Calcul des points du spectre entre nWn1 et nWn2
      for (n = nWn1 ; n <= nWn2 ; n++)
      {
        //@ Poids de la combinaison lineaire
        //@ b = (n - nWn1) / (nWn2 - nWn1)
        //@ a = 1 - b
        Beta = (double)(n - nWn1) / (double)(nWn2 - nWn1);
        Alpha = 1.0 - Beta;

        spectre_1c[IND1(n - Nsfirst1b + 1)] = 
          Alpha * SA1[IND1(n - nWn1 + NbSWwidth + 1)] + 
          Beta  * SA2[IND1(n - nWn2 + NbSWwidth + 1)];
      } //@ Fin de boucle sur les points de nWn1 a nWn2
    } //@ Fin de test sur naf

    //@ Transfert des resultats
    //@  SA1(n) = SA2(n)
    /* Plutot que de faire une recopie qui prend du temps, */
    /* on permute les pointeurs et on ecrase SA1 a l'iteration suivante */
    PtrSA1 = SA1;
    SA1 = SA2;
    SA2 = PtrSA1;

  } //@ Fin de boucle sur les fonctions d'apodisation

#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_Spectre1c.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  printf ("Fichier=\n%s\n", Fichier);
  Fd = fopen(Fichier, "wt");
  for (i = 1 ; i <= TailleWnSpect1c ; i++)
  {
    fprintf (Fd, "%.4le\n", spectre_1c[IND1(i)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
#endif

  /* on trace ici si besoin toutes les variables en sortie */
#ifdef INVESTIGATION
  /* ecrire le spectre de niveau 1C dans un fichier binaire */
 // jlp
  sprintf (LibelleTraces, "GS1cSpect");
  SD_BAS_TRACE (
    tcProcName_F, 
    LibelleTraces,
    1, 
    TailleWnSpect1c, 
    1, 
    TRACE_REAL8, 
    TRACE_BIN, 
    spectre_1c, 
    ctx);
#endif

fin:
// sortie de la fonction 
  switch (CodeRetour)
  {
    case SD_ALG_SPECTRE_37_S1C_ALRT_FCT_AMORTISST:
    {
#ifdef OPS_DEBUG
  printf (" echec du calcul de la fonction d'amortissement.\n");
#endif
      /* "ERROR ON COMPUTATION OF THE ABSORPTION FUNCTION" defini dans SD.msg */
      SD_ALG_SPECTRE_MsgSet = SD_ALG_SPECTRE_SET; /* defini dans SD__Message.h */
      SD_ALG_SPECTRE_MsgNum = MSG_ALG_SPECTRE_37_S1C_ALRT_FCT_AMORTISST; /* defini dans l'enumere e_SPECMessageJDB de SD__Message.h */
      CreerMessage (
        SD_ALG_SPECTRE_MsgSet, 
        SD_ALG_SPECTRE_MsgNum, 
        &(ctx->cr.msgJDB));
      CodeRetour = CMN__STATUT_NOK;
    }
    break;

    case SD_ALG_SPECTRE_37_S1C_ALRT_ZERO_PADDING:
    {
#ifdef OPS_DEBUG
  printf (" Echec du zero-padding du spectre 1B.\n");
#endif
      /* "ERROR ON ZERO-PADDING THE SPECTRAL WINDOW CENTERED ON $WAVENUMBER" defini dans SD.msg */
      SD_ALG_SPECTRE_MsgSet = SD_ALG_SPECTRE_SET; /* defini dans SD__Message.h */
      SD_ALG_SPECTRE_MsgNum = MSG_ALG_SPECTRE_37_S1C_ALRT_FCT_AMORTISST; /* defini dans l'enumere e_SPECMessageJDB de SD__Message.h */
      CreerMessage (
        SD_ALG_SPECTRE_MsgSet, 
        SD_ALG_SPECTRE_MsgNum, 
        &(ctx->cr.msgJDB));
      
      (void) StockerParametreMessage (
              "WAVENUMBER", 
              SD_ALG_SPECTRE_ValeurParam, 
              &(ctx->cr.msgJDB));

      CodeRetour = CMN__STATUT_NOK;
    }
    break;
    
    case SD_ALG_SPECTRE_37_S1C_ALRT_INIT_FFT_DIRECTE:
    {
#ifdef OPS_DEBUG
  printf (" Erreur lors de l'initialisation de la FFT par SD_BAS_FFT1DirDR2C\n");
#endif
      /* "ERROR ON INTIALIZATION OF THE FFT COMPUTATION" defini dans SD.msg */
      SD_ALG_SPECTRE_MsgSet = SD_ALG_SPECTRE_SET; /* defini dans SD__Message.h */
      SD_ALG_SPECTRE_MsgNum = MSG_ALG_SPECTRE_37_S1C_ALRT_INIT_FFT_DIRECTE; /* defini dans l'enumere e_SPECMessageJDB de SD__Message.h */
      CreerMessage (
        SD_ALG_SPECTRE_MsgSet, 
        SD_ALG_SPECTRE_MsgNum, 
        &(ctx->cr.msgJDB));
      CodeRetour = CMN__STATUT_NOK;
    }
    break;

    case SD_ALG_SPECTRE_37_S1C_ALRT_CALCUL_FFT_DIRECTE:
    {
#ifdef OPS_DEBUG
  printf (" Echec du calcul de FFT directe.\n");
#endif
      /* "ERROR ON THE DIRECT FFT COMPUTATION" defini dans SD.msg */
      SD_ALG_SPECTRE_MsgSet = SD_ALG_SPECTRE_SET; /* defini dans SD__Message.h */
      SD_ALG_SPECTRE_MsgNum = MSG_ALG_SPECTRE_37_S1C_ALRT_CALCUL_FFT_DIRECTE; /* defini dans l'enumere e_SPECMessageJDB de SD__Message.h */
      CreerMessage (
        SD_ALG_SPECTRE_MsgSet, 
        SD_ALG_SPECTRE_MsgNum, 
        &(ctx->cr.msgJDB));
      CodeRetour = CMN__STATUT_NOK;
    }
    break;

    case SD_ALG_SPECTRE_37_S1C_ALRT_PREPARE_INTERFEROGRAM:
    {
#ifdef OPS_DEBUG
  printf (" Echec du rearrangement de l'interferogramme avant FFT inverse.\n");
#endif
      /* "ERROR ON SETTING UP THE INTERFEROGRAM BEFORE INVERSE FFT" defini dans SD.msg */
      SD_ALG_SPECTRE_MsgSet = SD_ALG_SPECTRE_SET; /* defini dans SD__Message.h */
      SD_ALG_SPECTRE_MsgNum = MSG_ALG_SPECTRE_37_S1C_ALRT_PREPARE_INTERFEROGRAM; /* defini dans l'enumere e_SPECMessageJDB de SD__Message.h */
      CreerMessage (
        SD_ALG_SPECTRE_MsgSet, 
        SD_ALG_SPECTRE_MsgNum, 
        &(ctx->cr.msgJDB));
      CodeRetour = CMN__STATUT_NOK;
    }
    break;

    case SD_ALG_SPECTRE_37_S1C_ALRT_INIT_FFT_INVERSE:
    {
#ifdef OPS_DEBUG
  printf (" Echec de l'initialisation de la FFT inverse de la FFT du spectre 1B apodisee.\n");
#endif
      /* "ERROR ON INITIALIZATION OF THE INVERSE FFT" defini dans SD.msg */
      SD_ALG_SPECTRE_MsgSet = SD_ALG_SPECTRE_SET; /* defini dans SD__Message.h */
      SD_ALG_SPECTRE_MsgNum = MSG_ALG_SPECTRE_37_S1C_ALRT_INIT_FFT_INVERSE; /* defini dans l'enumere e_SPECMessageJDB de SD__Message.h */
      CreerMessage (
        SD_ALG_SPECTRE_MsgSet, 
        SD_ALG_SPECTRE_MsgNum, 
        &(ctx->cr.msgJDB));
      CodeRetour = CMN__STATUT_NOK;
    }

    case SD_ALG_SPECTRE_37_S1C_ALRT_CALCUL_FFT_INVERSE:
    {
#ifdef OPS_DEBUG
  printf (" Echec de la FFT inverse de la FFT du spectre 1B apodisee.\n");
#endif
      /* "ERROR ON COMPUTATION OF THE INVERSE FFT" defini dans SD.msg */
      SD_ALG_SPECTRE_MsgSet = SD_ALG_SPECTRE_SET; /* defini dans SD__Message.h */
      SD_ALG_SPECTRE_MsgNum = MSG_ALG_SPECTRE_37_S1C_ALRT_CALCUL_FFT_INVERSE; /* defini dans l'enumere e_SPECMessageJDB de SD__Message.h */
      CreerMessage (
        SD_ALG_SPECTRE_MsgSet, 
        SD_ALG_SPECTRE_MsgNum, 
        &(ctx->cr.msgJDB));
      CodeRetour = CMN__STATUT_NOK;
    }

    case SD_ALG_SPECTRE_37_S1C_ALRT_ARRANGT_SPECTRE_1C:
    {
#ifdef OPS_DEBUG
  printf (" Echec du rearrangement du spectre 1B apodise.\n");
#endif
      /* "ERROR ON ARRANGING THE INVERSE FFT" defini dans SD.msg */
      SD_ALG_SPECTRE_MsgSet = SD_ALG_SPECTRE_SET; /* defini dans SD__Message.h */
      SD_ALG_SPECTRE_MsgNum = MSG_ALG_SPECTRE_37_S1C_ALRT_ARRANGT_SPECTRE_1C; /* defini dans l'enumere e_SPECMessageJDB de SD__Message.h */
      CreerMessage (
        SD_ALG_SPECTRE_MsgSet, 
        SD_ALG_SPECTRE_MsgNum, 
        &(ctx->cr.msgJDB));
      CodeRetour = CMN__STATUT_NOK;
    }

    default :
    {
      if (CodeRetour != CMN__STATUT_OK)
      {
        CodeRetour = CMN__STATUT_NOK;
      }
    }
  } /* fin switch */

#ifdef OPS_DEBUG
  printf ("Fin de %s\n", tcProcName_F);
#endif
return(CodeRetour);
}

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_XX_Initprod
//@
//@ Description : algorithme commun a une ligne IASI et realise au debut
//@ de la chaine produit ( apres 43_ISF, 09_PLK, 33_SME)
//@
//@
//@ Parameters :
//@     (ENTREE)  ctx : contexte de l'etape
//@     (ENTREE)  conf_autre : fichier de conf autre
//@     (ENTREE)  banque_spectrale : fichier banque spectrale
//@     (ENTREE/SORTIE)  data_thread_produit : info generales calculees sur les 
//@                                            lignes LN et LN-1
//@     (ENTREE)  sortie_22_SOS : info generales issues de l'algo SOS
//@     (ENTREE)  t : date UTC de la ligne (SN=15)   /* DM 11/03/2004 */
//@     (ENTREE)  t_FE : date de fin d'eclipse pour la ligne LN 
//@                      noter que cette date est anterieure a la date du 1er
//@                      scan de la ligne LN
//@     (ENTREE)  BBT : temperature du corps noir (LN-1)
//@     (SORTIE)  flag_date_NOK : flag indicateur d'une incoherence des dates 
//@                               (LN-1)
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_XX_Initprod(
  t_SD_BAS_DataThread *         ctx, 
  t_SD_BAS_ConfAutre *          conf_autre,
  t_SD_BAS_BanqueSpectrale *    banque_spectrale,
  t_SD_BAS_DataThreadProduit *  data_thread_produit, 
  t_SD_BAS_22_SOS_Sortie *      sortie_22_SOS, 
  t_SD_BAS_SCDSTime             t, 
  t_SD_BAS_SCDSTime             t_FE, 
  double                        BBT, 
  t_CMN__Booleen *              flag_date_NOK)

{
  /* Variables locales */
  /*-------------------*/
  char              tcProcName_F[] = "SD_ALG_XX_InitProd";
  t_CMN__Statut     CodeRetour = CMN__STATUT_OK;
#ifdef INVESTIGATION
  char              LibelleTraces[SD_ALG_SPECTRE_LG_LIBELLE_TRACES + 1];
#endif
  
  /* Vecteur des nb d'onde des fonction de calibration spectrale */
  double *          WnFcsTab;
  /* Vecteur des nb d'onde nominaux */
  double *          Wn0;
  /* Vecteur des nb d'onde nominaux calibres */
  double *          WnSpect;
  /* Fonction de calibration spectrale */
  double *          FcsTab;
  /* Fonction de calibration spectrale interpolee sur Wn0 */
  double *          FcsInterpolee;
  /* Fonction de calibration spectrale interpolee sur WnOSSpect*/
  double *          OSFcsInterpolee;
  /* Largeur d'un echantillon spectral de la fonction de calibration spectrale */
  double            DWnFcsTab;
  /* Indice du premier echantillon de la premiere fonction d'instrument */
  int               NsfirstFcsTab;
  /* Indice du dernier echantillon de la premiere fonction d'instrument */
  int               NslastFcsTab;
  /* Nb d'echantillons de la premiere fonction d'instrument */
  int               TailleWnFcsTab;
  /* Largeur nominale d'un echantillon spectral */
  double            DWnSpect;
  /* Largeur d'un echantillon spectral de niveau 1B */
  double            DWnSpect1b;
  /* Indice du premier echantillon nominal du spectre decode */
  int               Nsfirst;
  /* Indice du dernier echantillon nominal du spectre decode */
  int               Nslast;
  /* Nb d'echantillons nominaux */
  int               TailleWn0;
  /* Indice du premier echantillon du spectre 1B */
  int               Nsfirst1b;
  /* Indice du dernier echantillon du spectre 1B */
  int               Nslast1b;
  /* Nb d'echantillons du spectre 1B */
  int               TailleWnSpect1b;
  /* Vecteur des nb d'onde du spectre 1B */
  double *          WnSpect1b;
  /* Largeur d'un echantillon spectral surechantillonne */
  double            DWnOSSpect;
  /* Indice du premier echantillon du spectre surechantillonne */
  int               OSNsfirst;
  /* Indice du dernier echantillon du spectre surechantillonne */
  int               OSNslast;
  /* Nb d'echantillons du spectre surechantillonne */
  int               TailleWnOSSpect;
  /* Vecteur des nb d'onde surechantillonnes */
  double *          WnOSSpect;
  /* Largeur d'un echantillon de base des nb d'onde des fonctions d'apodisation spectrale */
  double            DWnAfTab;
  /* Indice de la premiere fonction d'apodisation spectrale */
  int               NsfirstAfTab;
  /* Indice de la derniere fonction d'apodisation spectrale */
  int               NslastAfTab;
#ifdef OPS_DEBUG
  /* Nb de fonctions d'apodisation spectrale */
  int               TailleWnAfTab;
#endif
  /* Vecteur des positions en nb d'ondes des fonction d'apodisation spectrale */
  double *          WnAf;
  /* Fonction de Planck calibree pour la ligne precedente */
  double *          B1;
  /* Radiance des surfaces radiatives reflechies */
  double            RadianceReflechie;
  /* Nb de surfaces radiatives reflechies par le corps noir */
  int               NbRrs;
  /* Fraction de l'emission des surfaces radiatives reflechies */
  double *          RRrs;
  /* Modele de temperature des surfaces radiatives */
  double *          TRrsModel;
  /* Temperature des surfaces radiatives */
  double *          TRrs;
  /* Indentificateurs de nullite des premiers parametres du modele de temperature des surfaces radiatives */
  t_CMN__Booleen    ParamModeleNuls;
  /* Periode de revlution du satellite */
  double            Period;
  /* Radiance emise par le corps noir chaud */
  double *          Bbb;
  /* Emissivite spectrale du corps noir */
  double *          BBEmissivity;
  /* Compteurs */
  int               i, j, m, n;
  int     PN;
  int     CCD;

#ifdef OPS_DEBUG
  FILE *            Fd;
  char              Fichier[256];
#endif

    //@ Initialisations
    PN                  = ctx->ctxtrt.PN;
    CCD                 = ctx->ctxtrt.CCD + 1;          // Pour pouvoir utiliser les macros
  /* Initialiser les variables locales */
  /*-----------------------------------*/
  /* entrees de XX_InitProd */
  DWnFcsTab = banque_spectrale->IDefSafDWn;
  NsfirstFcsTab = banque_spectrale->IDefSafNsfirst;
  NslastFcsTab = banque_spectrale->IDefSafNslast;
  TailleWnFcsTab = NslastFcsTab - NsfirstFcsTab + 1;

  DWnSpect = conf_autre->IDefSpectrDWn;
  Nsfirst = conf_autre->IDefNsfirst[SD_BAS_B1];
  Nslast = conf_autre->IDefNslast[SD_BAS_B3];
  TailleWn0 = Nslast - Nsfirst + 1;

// jlp //
  FcsTab = data_thread_produit->Fcs;                      /* ligne LN */

  NbRrs = conf_autre->IDefBBNbRrs;
  RRrs = conf_autre->IDefBBRRs;
  TRrsModel = conf_autre->IDefHecTRrsModel;
  Period = conf_autre->IDefMETOPOrbitPeriod;
  ParamModeleNuls = CMN__VRAI;

  DWnSpect1b = conf_autre->IDefSpectDWn1b;
  Nsfirst1b = conf_autre->IDefNsfirst1b;
  Nslast1b = conf_autre->IDefNslast1b;
  TailleWnSpect1b = Nslast1b - Nsfirst1b + 1;

  DWnOSSpect = sortie_22_SOS->GSosSsdDWnOSSpect;
  OSNsfirst = sortie_22_SOS->GSosSsdOSNsFirstSpect;
  OSNslast = sortie_22_SOS->GSosSsdOSNsLastSpect;
  TailleWnOSSpect = OSNslast - OSNsfirst + 1;

  DWnAfTab = banque_spectrale->IDefSafDWn;
  NsfirstAfTab = banque_spectrale->IDefSafNsfirst;
  NslastAfTab = banque_spectrale->IDefSafNslast;
#ifdef OPS_DEBUG
  TailleWnAfTab = NslastAfTab - NsfirstAfTab + 1;
#endif

  BBEmissivity = conf_autre->IDefBBEmissivity;
  RadianceReflechie = 0.0;

  /* sortie de XX_InitProd */
  WnFcsTab = data_thread_produit->WnFcsTab;               /* ligne LN */
  Wn0 = data_thread_produit->Wn0;                         /* ligne LN */
// jlp 
  FcsInterpolee = data_thread_produit->FcsInterpolee;     /* ligne LN */
// jlp // 
  WnSpect = data_thread_produit->WnSpect;                 /* ligne LN */
// jlp // //
  B1 = data_thread_produit->B1;                           /* ligne LN-1 */
  TRrs = data_thread_produit->TRrs;                       /* ligne LN-1 */
// jlp // // 
  Bbb = data_thread_produit->Bbb;                         /* ligne LN-1 */
  WnSpect1b = data_thread_produit->WnSpect1b;             /* ligne LN */
// jlp //
  WnOSSpect = data_thread_produit->WnOSSpect;             /* ligne LN */
// jlp
  OSFcsInterpolee = data_thread_produit->OSFcsInterpolee; /* ligne LN */
  WnAf = data_thread_produit->WnAf;                       /* ligne LN */

  /* Corps de l'operation */
  /*----------------------*/
#ifdef OPS_DEBUG
  printf ("Debut de %s\n", tcProcName_F);
#endif
  
#ifdef OPS_DEBUG
  printf (" DWnFcsTab=%.2e\tNsfirstFcsTab=%d\tNslastFcsTab=%d\tTailleWnFcsTab=%d\n", 
    DWnFcsTab, NsfirstFcsTab, NslastFcsTab, TailleWnFcsTab);
  printf (" DWnSpect=%.2e\tNsfirst=%d\t\tNslast=%d\t\tTailleWn0=%d\n", 
    DWnSpect, Nsfirst, Nslast, TailleWn0);
  printf (" DWnSpect1b=%.2e\tNsfirst1b=%d\t\tNslast1b=%d\t\tTailleWnSpect1b=%d\n", 
    DWnSpect1b, Nsfirst1b, Nslast1b, TailleWnSpect1b);
  printf (" DWnOSSpect=%.2e\tOSNsfirst=%d\t\tOSNslast=%d\t\tTailleWnOSSpect=%d\n", 
    DWnOSSpect, OSNsfirst, OSNslast, TailleWnOSSpect);
  printf (" DWnAfTab=%.2e\tNsfirstAfTab=%d\t\tNslastAfTab=%d\t\tTailleAfTab=%d\n", 
    DWnAfTab, NsfirstAfTab, NslastAfTab, TailleWnAfTab);
  printf (" NbRrs=%d\n", NbRrs);
#endif

  /***************************************************************************/
  /* Liste des variables initialisees par 43_ISF                             */
  /***************************************************************************/
  /* occurence : indice de ligne du granule = LN */
  /*
  double *Fcs;    //taille BanqueSpectrale->IDefSafNslast - BanqueSpectrale->IDefSafNsfirst +1
  double *y2isf;  //taille 4*(BanqueSpectrale->IDefSafNsOpd)
  double *OpdTab; //taille BanqueSpectrale->IDefSafNsOpd;
  double *Opd;    //taille 2*conf_autre->IDefS1cNbSWwidth + 1
  dcmplx *Af;     //taille (BanqueSpectrale->IDefSafNslast - BanqueSpectrale->IDefSafNsfirst +1) * (2*conf_autre->IDefS1cNbSWwidth + 1)
  dcmplx *Af0;    //taille BanqueSpectrale->IDefSafNsOpd
  double *REtmp;  //taille BanqueSpectrale->IDefSafNsOpd
  double *IMtmp;  //taille BanqueSpectrale->IDefSafNsOpd
  double *REint;  //taille 2*conf_autre->IDefS1cNbSWwidth + 1
  double *IMint;  //taille 2*conf_autre->IDefS1cNbSWwidth + 1
  */

  /***************************************************************************/
  /* Liste des variables initialisees par 09_PLK                             */
  /***************************************************************************/
  /* data_thread_produit_LN_1->PlanckBBT : Fonction de Planck utilisee */
  /* a bord pour la ligne precedente */
  /* taille conf_autre->IDefNslast[SB=3]-conf_autre->IDefNsfirst[SB=1]+1 */
  /* occurence : indice de ligne du granule = LN-1 */
  
  /***************************************************************************/
  /* Liste des variables initialisees par XX_InitProd (XX_Init dans la doc)  */
  /***************************************************************************/

  /* Variables initialement utilisees pour 31_SCR, 32_HEC, 34_SMC */
  /*--------------------------------------------------------------*/
  //@ Calcul des nb d'onde des fonctions de calibration spectrale d'indice 
  //@ compris entre NsfirstFcsTab et NslastFcsTab
  for (m = NsfirstFcsTab ; m <= NslastFcsTab ; m++)
  {
    WnFcsTab[m - NsfirstFcsTab] = (m - 1) * DWnFcsTab;
  }
  
#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_WnFcsTab.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (i = 1 ; i <= TailleWnFcsTab ; i++)
  {
    fprintf (Fd, "%.4le\n", WnFcsTab[IND1(i)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
#endif

  //@ Calcul des positions nominales des echantillons d'indice compris 
  //@ entre Nsfirst et Nslast
  for (n = Nsfirst ; n <= Nslast ; n++)
  {
    Wn0[n - Nsfirst] = (n - 1) * DWnSpect;
  }

#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_Wn0.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (i = 1 ; i <= TailleWn0 ; i++)
  {
    fprintf (Fd, "%.4le\n", Wn0[IND1(i)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
#endif

  //@ Interpolation lineaire des fonctions de calibration spectrale
// jlp
  CodeRetour = SD_BAS_InterpLineaire1D (
                TailleWnFcsTab, 
                WnFcsTab, 
                &FcsTab[IND3(1,PN,CCD,SD_BAS_SDB_NFSMAX,SD_BAS_PN)], 
                data_thread_produit->TabAuxESSLTailleWn0, 
                TailleWn0, 
                Wn0,
                &FcsInterpolee[IND3(1,PN,CCD,(conf_autre->IDefNslast[SD_BAS_B3] - conf_autre->IDefNsfirst[SD_BAS_B1] + 1),SD_BAS_PN)]);
  
  SD_BAS_THROW(
    CodeRetour, SD_ALG_SPECTRE_XX_INITPROD_ALRT_INTERP_FCS,  
    CodeRetour, BAS_CR_NEQ, CMN__STATUT_OK, "");
  
#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_FcsInterpolee.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (i = 1 ; i <= TailleWn0 ; i++)
  {
    fprintf (Fd, "%.4le\n", FcsInterpolee[IND1(i)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
#endif

  //@ WnSpect : Vecteur des nb d'onde nominaux calibres
  //@ taille conf_autre->IDefNslast[SB=3] - conf_autre->IDefNsfirst[SB=1] + 1
  //@ occurence : indice de ligne du granule = LN  
  for (n = 1 ; n <= TailleWn0 ; n++)
  {
    WnSpect[IND3(n,PN,CCD,(conf_autre->IDefNslast[SD_BAS_B3] - conf_autre->IDefNsfirst[SD_BAS_B1] + 1),SD_BAS_PN)] = 
    Wn0[IND1(n)] * FcsInterpolee[IND3(n,PN,CCD,(conf_autre->IDefNslast[SD_BAS_B3] - conf_autre->IDefNsfirst[SD_BAS_B1] + 1),SD_BAS_PN)];
  }
  
#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_WnSpect.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (i = 1 ; i <= TailleWn0 ; i++)
  {
    fprintf (Fd, "%.4le\n", WnSpect[IND1(i)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
#endif

  /* Variables initialement utilisees pour 31_SCR */
  /*--------------------------------------------------------*/
  /* B1 : Valeurs de la fontion de Planck calibree pour la ligne precedente */
  /* taille conf_autre->IDefNslast[SB=3]-conf_autre->IDefNsfirst[SB=1]+1 */
  /* occurence : indice de ligne du granule = LN-1 */

  //@ Calcul de la fonction de Planck interpolee B1 aux nb d'onde WnSpect
  CodeRetour = SD_BAS_InterpLineaire1D (
                TailleWn0, 
                Wn0, 
                data_thread_produit->PlanckBBT, 
                data_thread_produit->TabAuxESSLTailleWn0, 
                TailleWn0, 
                &WnSpect[IND3(1,PN,CCD,(conf_autre->IDefNslast[SD_BAS_B3] - conf_autre->IDefNsfirst[SD_BAS_B1] + 1),SD_BAS_PN)], 
                &B1[IND3(1,PN,CCD,(conf_autre->IDefNslast[SD_BAS_B3] - conf_autre->IDefNsfirst[SD_BAS_B1] + 1),SD_BAS_PN)]);

  SD_BAS_THROW(
    CodeRetour, SD_ALG_SPECTRE_XX_INITPROD_ALRT_INTERP_PLANCKBBT,  
    CodeRetour, BAS_CR_NEQ, CMN__STATUT_OK, "");
  
#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_B1.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (i = 1 ; i <= TailleWn0 ; i++)
  {
    fprintf (Fd, "%.4le\n", B1[IND1(i)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
#endif

  /* Variables initialement utilisees pour 32_HEC */
  /*----------------------------------------------*/
  //@ Calcul de la temperature des surfaces radiatives
  for (i = 1 ; i <= NbRrs ; i++)
  {
    TRrs[IND1(i)] = SD_BAS_EvolutionModele(
                    t,
                    t_FE,
                    Period, 
                    &TRrsModel[IND2(1,i,SD_BAS_MODEL)]);
    
    for (j = 1 ; j <= SD_ALG_SPECTRE_NB_PARAM_MODELE_ORBITAL ; j++)
    {
      if (TRrsModel[IND2(j,i,SD_BAS_MODEL)] != 0)
      {
        ParamModeleNuls = CMN__FAUX;
      }
    } /* fin boucle sur les param du modele orbital */
  } /* fin boucle sur les surfaces radiatives */
  
  //@ Definition du flag indicateur d'une incoherence des dates
  if ( (CMN__DATE_DIFF_SCDS_2_SEC(T, t_FE) < NINT(8.0 * Period)) ||
       (ParamModeleNuls == CMN__VRAI) )
  {
    *flag_date_NOK = CMN__NON;
  }
  else
  {
    *flag_date_NOK = CMN__OUI;
  }
  
  //@ Bbb : Radiance emise par le corps noir chaud pour les echantillons dont
  //@ l'indice est compris entre Nsfirst et Nslast
  /* taille conf_autre->IDefNslast[SB=3]-conf_autre->IDefNsfirst[SB=1]+1 */
  /* occurence : indice de ligne du granule = LN */
  for (n = 1 ; n <= TailleWn0 ; n++)
  {
    RadianceReflechie = 0.0;
    for ( i = 1 ; i <= NbRrs ; i++)
    {
      RadianceReflechie = RadianceReflechie + 
                          RRrs[IND1(i)] * SD_BAS_Planck(
                                          TRrs[IND1(i)], 
                                          WnSpect[IND3(n,PN,CCD,(conf_autre->IDefNslast[SD_BAS_B3] - conf_autre->IDefNsfirst[SD_BAS_B1] + 1),SD_BAS_PN)], 
                                          conf_autre->IDefPlkPK1, 
                                          conf_autre->IDefPlkPK2);
                                          
    }
    Bbb[IND3(n,PN,CCD,(conf_autre->IDefNslast[SD_BAS_B3] - conf_autre->IDefNsfirst[SD_BAS_B1] + 1),SD_BAS_PN)] = BBEmissivity[IND1(n)] * 
                   SD_BAS_Planck(
                      BBT, 
                      WnSpect[IND3(n,PN,CCD,(conf_autre->IDefNslast[SD_BAS_B3] - conf_autre->IDefNsfirst[SD_BAS_B1] + 1),SD_BAS_PN)], 
                      conf_autre->IDefPlkPK1, 
                      conf_autre->IDefPlkPK2) + 
                      RadianceReflechie*(1.-BBEmissivity[IND1(n)]); 
  } /* FIN POUR n = Nsfirst a Nslast */

#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_Bbb.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (i = 1 ; i <= TailleWn0 ; i++)
  {
    fprintf (Fd, "%.4le\n", Bbb[IND1(i)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
#endif

  /* Variables initialement utilisees pour 35_S1B */
  /*----------------------------------------------*/
  //@ Calcul des positions nominales des echantillons 1B d'indice compris 
  //@ entre Nsfirst1b et Nslast1b
  for (n = Nsfirst1b ; n <= Nslast1b ; n++)
  {
    WnSpect1b[n - Nsfirst1b] = (n - 1) * DWnSpect1b;
  }

#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_WnSpect1b.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (i = 1 ; i <= TailleWnSpect1b ; i++)
  {
    fprintf (Fd, "%.4le\n", WnSpect1b[IND1(i)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
#endif

  //@ WnOSSpect : calibration spectrale du vecteur des nb d'onde du spectre 
  //@ surechantillonne d'indice compris entre OSNsfirst et OSNslast 
  /* taille SD_BAS_22_SOS_Sortie->GSosSsdOSNsLastSpect - */
  /*       SD_BAS_22_SOS_Sortie->GSosSsdOSNsFirstSpect + 1 */
  /* occurence : indice de ligne du granule = LN */
// jlp
  for (n = OSNsfirst ; n <= OSNslast ; n++)
  {
   WnOSSpect[IND3(n - OSNsfirst+1,PN,CCD,(data_thread_produit->GSosSsdOSNsLastSpect - data_thread_produit->GSosSsdOSNsFirstSpect + 1),SD_BAS_PN)] = (n - 1) * DWnOSSpect;
  }
  
  //@ Interpolation de la fonction de calibration spectrale sur la base 
  //@ surechantillonnee
// jlp
  CodeRetour = SD_BAS_InterpLineaire1D (
                TailleWnFcsTab, 
                WnFcsTab, 
                &FcsTab[IND3(1,PN,CCD,SD_BAS_SDB_NFSMAX,SD_BAS_PN)], 
                data_thread_produit->TabAuxESSLTailleWnOSSpect, 
                TailleWnOSSpect, 
                &WnOSSpect[IND3(1,PN,CCD,(data_thread_produit->GSosSsdOSNsLastSpect - data_thread_produit->GSosSsdOSNsFirstSpect + 1),SD_BAS_PN)], 
                &OSFcsInterpolee[IND3(1,PN,CCD,(data_thread_produit->GSosSsdOSNsLastSpect - data_thread_produit->GSosSsdOSNsFirstSpect + 1),SD_BAS_PN)]);
                
  SD_BAS_THROW(
    CodeRetour, SD_ALG_SPECTRE_XX_INITPROD_ALRT_INTERP_FCS_SURECH, 
    CodeRetour, BAS_CR_NEQ, CMN__STATUT_OK, "");

#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_OSFcsInterpolee.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (i = 1 ; i <= TailleWnOSSpect ; i++)
  {
    fprintf (Fd, "%.4le\n", OSFcsInterpolee[IND1(i)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
#endif

  for (n = 1 ; n <= TailleWnOSSpect ; n++)
  {
    WnOSSpect[IND3(n,PN,CCD,(data_thread_produit->GSosSsdOSNsLastSpect - data_thread_produit->GSosSsdOSNsFirstSpect + 1),SD_BAS_PN)] =
     WnOSSpect[IND3(n,PN,CCD,(data_thread_produit->GSosSsdOSNsLastSpect - data_thread_produit->GSosSsdOSNsFirstSpect + 1),SD_BAS_PN)] * OSFcsInterpolee[IND3(n,PN,CCD,(data_thread_produit->GSosSsdOSNsLastSpect - data_thread_produit->GSosSsdOSNsFirstSpect + 1),SD_BAS_PN)];
  }

#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_WnOSSpect.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (i = 1 ; i <= TailleWnOSSpect ; i++)
  {
    fprintf (Fd, "%.4le\n", WnOSSpect[IND1(i)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
#endif

  /* Variables initialement utilisees pour 37_S1C */
  /*----------------------------------------------*/
  //@ WnAf : vecteur des nb d'onde des fonctions d'apodisation d'indice 
  //@ compris entre NsfirstAfTab et NslastAfTab
  /* taille BanqueSpectrale->IDefSafNslast - BanqueSpectrale->IDefSafNsfirst + 1 */
  /* occurence : indice de ligne du granule = LN */
  for (n = NsfirstAfTab ; n <= NslastAfTab ; n++)
  {
    WnAf[n - NsfirstAfTab] = (n - 1) * DWnAfTab;
  }

#ifdef OPS_DEBUG
  sprintf (Fichier, "%s/%s/sorties/%s_WnAf.txt", 
    getenv("OPS_SRC_HOME"), REP_TU, tcProcName_F);
  Fd = fopen (Fichier, "wt");
  for (i = 1 ; i <= TailleWnAfTab ; i++)
  {
    fprintf (Fd, "%.4le\n", WnAf[IND1(i)]);
  }
  fclose (Fd);
  printf (" Ecriture OK du fichier suivant:\n %s\n", Fichier);
#endif

  /***************************************************************************/
  /* Liste des variables initialisees par TransformIpsf                      */
  /* TransformIpsf n'est pas factorisee dans XX_Init : cela impliquerait     */
  /* d'appeler XX_Init avant 21_SSS qui precede l'appel a 43_CCS or on a     */
  /* besoin de 43_ISF pour factoriser les calculs de la chaine PRODUIT       */
  /***************************************************************************/
  /*double * ImgY;     */
  /*double * ImgZ;     */
  /*double * ImgWgtpsf;*/

  /* on trace ici si besoin toutes les variables en sortie */
#ifdef INVESTIGATION
  /* ecrire WnFcsTab, nb d'onde des fonctions de calibration spectrale, */
  /* dans un fichier binaire */
  sprintf (LibelleTraces, "WnFcsTab");
  SD_BAS_TRACE (
    tcProcName_F, 
    LibelleTraces, 
    TailleWnFcsTab, 
    1, 
    1, 
    TRACE_REAL8, 
    TRACE_BIN, 
    WnFcsTab, 
    ctx);

  /* ecrire Wn0, positions nominales des echantillons, dans un fichier binaire */
  sprintf (LibelleTraces, "Wn0");
  SD_BAS_TRACE (
    tcProcName_F, 
    LibelleTraces, 
    TailleWn0, 
    1,
    1, 
    TRACE_REAL8, 
    TRACE_BIN, 
    Wn0, 
    ctx);
  
  /* ecrire FcsInterpolee la fonction de calibration spectrale interpolee sur */
  /* Wn0 dans un fichier binaire */
  sprintf (LibelleTraces, "FcsInterpolee");
  SD_BAS_TRACE (
    tcProcName_F, 
    LibelleTraces, 
    TailleWn0, 
    1,
    1, 
    TRACE_REAL8, 
    TRACE_BIN, 
    FcsInterpolee, 
    ctx);
  
  /* ecrire le vecteur des nb d'onde nominaux calibres WnSpect */
  /* dans un fichier binaire */
  sprintf (LibelleTraces, "WnSpect");
  SD_BAS_TRACE (
    tcProcName_F, 
    LibelleTraces, 
    TailleWn0, 
    1,
    1, 
    TRACE_REAL8, 
    TRACE_BIN, 
    WnSpect, 
    ctx);
  
  /* ecrire B1, fonction de Planck interpolee sur la base WnSpect, */
  /* dans un fichier binaire */
  sprintf (LibelleTraces, "B1");
  SD_BAS_TRACE (
    tcProcName_F, 
    LibelleTraces, 
    TailleWn0, 
    1,
    1, 
    TRACE_REAL8, 
    TRACE_BIN, 
    B1, 
    ctx);
  
  /* ecrire TRrs, les temperatures des surfaces radiatives, */
  /* dans un fichier binaire */
  sprintf (LibelleTraces, "TRrs");
  SD_BAS_TRACE (
    tcProcName_F, 
    LibelleTraces, 
    NbRrs, 
    1,
    1, 
    TRACE_REAL8, 
    TRACE_ASC, 
    TRrs, 
    ctx);
  
  /* ecrire la radiance emise par le corps noir chaud pour les echantillons */
  /* dont l'indice est compris entre Nsfirst et Nslast dans un fichier binaire */
  sprintf (LibelleTraces, "Bbb");
  SD_BAS_TRACE (
    tcProcName_F, 
    LibelleTraces, 
    TailleWn0, 
    1,
    1, 
    TRACE_REAL8, 
    TRACE_BIN, 
    Bbb, 
    ctx);
  
  /* ecrire WnSpect1B, vecteur des positions nominales des echantillons 1B, */
  /* dans un fichier binaire */
  sprintf (LibelleTraces, "WnSpect1b");
  SD_BAS_TRACE (
    tcProcName_F, 
    LibelleTraces, 
    TailleWnSpect1b, 
    1,
    1, 
    TRACE_REAL8, 
    TRACE_BIN, 
    WnSpect1b, 
    ctx);
  
  /* ecrire WnOSSpect, base d'echantillonnage du spectre surechantillonne, */
  /* dans un fichier binaire */
  sprintf (LibelleTraces, "WnOSSpect");
  SD_BAS_TRACE (
    tcProcName_F, 
    LibelleTraces, 
    TailleWnSpect1b, 
    1,
    1, 
    TRACE_REAL8, 
    TRACE_BIN, 
    WnOSSpect, 
    ctx);
  
#endif /* fin mode INVESTIGATION */

/* sortie de la fonction  */
fin :
  switch (CodeRetour)
  {
    case SD_ALG_SPECTRE_XX_INITPROD_ALRT_INTERP_FCS:
    {
#ifdef OPS_DEBUG
  printf (" Echec de l'interpolation linaire 1D de FcsTab sur Wn0.\n");
#endif
      /* "ERROR ON LINEAR INTERPOLATION OF THE SPECTRAL CALIBRATION FUNCTION" defini dans SD.msg */
      SD_ALG_SPECTRE_MsgSet = SD_ALG_SPECTRE_SET; /* defini dans SD__Message.h */
      SD_ALG_SPECTRE_MsgNum = MSG_ALG_SPECTRE_XX_INITPROD_ALRT_INTERP_FCS; /* defini dans l'enumere e_SPECMessageJDB de SD__Message.h */
      CreerMessage (
        SD_ALG_SPECTRE_MsgSet, 
        SD_ALG_SPECTRE_MsgNum, 
        &(ctx->cr.msgJDB));
      CodeRetour = CMN__STATUT_NOK;
    }
    break;

    case SD_ALG_SPECTRE_XX_INITPROD_ALRT_INTERP_PLANCKBBT:
    {
#ifdef OPS_DEBUG
  printf (" Echec de l'interpolation linaire 1D de la PlanckBBT.\n");
#endif
      /* "ERROR ON LINEAR INTERPOLATION OF THE PLANCK FUNCTION" defini dans SD.msg */
      SD_ALG_SPECTRE_MsgSet = SD_ALG_SPECTRE_SET; /* defini dans SD__Message.h */
      SD_ALG_SPECTRE_MsgNum = MSG_ALG_SPECTRE_XX_INITPROD_ALRT_INTERP_PLANCKBBT; /* defini dans l'enumere e_SPECMessageJDB de SD__Message.h */
      CreerMessage (
        SD_ALG_SPECTRE_MsgSet, 
        SD_ALG_SPECTRE_MsgNum, 
        &(ctx->cr.msgJDB));
      CodeRetour = CMN__STATUT_NOK;
    }
    break;

    case SD_ALG_SPECTRE_XX_INITPROD_ALRT_INTERP_FCS_SURECH:
    {
#ifdef OPS_DEBUG
  printf (" Echec de l'interpolation linaire 1D de la fonction de calibration spectrale sur la base des nb d'ondes surechantillonnee.\n");
#endif
      /* "ERROR ON LINEAR INTERPOLATION OF THE OVER SAMPLED SPECTRAL CALIBRATION FUNCTION" defini dans SD.msg */
      SD_ALG_SPECTRE_MsgSet = SD_ALG_SPECTRE_SET; /* defini dans SD__Message.h */
      SD_ALG_SPECTRE_MsgNum = MSG_ALG_SPECTRE_XX_INITPROD_ALRT_INTERP_FCS_SURECH; /* defini dans l'enumere e_SPECMessageJDB de SD__Message.h */
      CreerMessage (
        SD_ALG_SPECTRE_MsgSet, 
        SD_ALG_SPECTRE_MsgNum, 
        &(ctx->cr.msgJDB));
      CodeRetour = CMN__STATUT_NOK;
    }
    break;
    
    default :
    {
      if (CodeRetour != CMN__STATUT_OK)
      {
        CodeRetour = CMN__STATUT_NOK;
      }
    }
  } /* fin switch */

#ifdef OPS_DEBUG
  printf ("Fin de %s\n", tcProcName_F);
#endif
return(CodeRetour);
}

//======================================================================
//          FONCTIONS INTERNES
//======================================================================



