//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/ALG/src/SD_ALG_Spectre.h,v 1.3 2011/11/25 10:17:44 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_ALG
//@ MODULE : SD_ALG_Spectre
//@
//@ Comment :
//@    include file for the SD_ALG_Spectre class which implements the
//@    algorithm used to compute IASI L1 spectra
//@
///@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 29/01/2003 : Olivier DUFFEZ
//@ Premiere version qui fonctionne pour 20_DOC
//@ --------------------------------------------------------------------
//@ VERSION : 27/11/2002 : Jean-Luc PASCAL
//@ Procedure : SD_ALG_XX_Initprod deplacee de SD_ALG_Image.c
//@ --------------------------------------------------------------------
//@ VERSION : 20/03/2003 : Benjamin MASSART
//@ TU OK
//@ --------------------------------------------------------------------
//@ PM-ID : PM4 : 31/03/2004 : B. MASSART
//@ SD_ALG_XX_Initprod:
//@ -date UTC de la ligne (SN=15)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_ALG_Spectre_h
#define SD_ALG_Spectre_h


// inserer ici les fichiers include
#include <stdio.h>       /* entree/sortie */
#include <stdlib.h>      /* alloc memoire, ...*/
#include <string.h>      /* chaines de carateres */
#include <unistd.h>      /* gestion pipes, ... */
#include <time.h>        /* gestion de la date */
#include <sys/param.h>   /* constantes systeme */
#include <math.h>        /* fonctions mathematiques */


#include "CMN__Commun.h"
#include "SD_BAS_Global.h"
#include "SD_BAS_OperateurBase.h"
#include "SD_BAS_ConfAutre.h"
#include "SD_BAS_Produits.h"
#include "SD_BAS_Niveau1.h"
#include "SD_BAS_Contexte.h"
#include "SD_BAS_BanqueSpectrale.h"
#include "SD_ALG_ISRFEM.h"
#include "SD_BAS_LibESSL.h"
#include "SD_BAS_ConfStable.h"
#include "ERR__Erreur.h"
#include "SD__Message.h"


/*====================================================================*/
//   External define  Declarations
/*====================================================================*/
#define ERR_BASE_SPECTRE                  1370 /* SD_ALG_ERR_BASE_SPECTRE = 1360 est defini dans ERR_Erreur.h */
#define SD_ALG_SPECTRE_LG_LIBELLE_TRACES  60
#define SD_ALG_SPECTRE_LG_VALEUR_PARAM    20


/*====================================================================*/
// Constant Declaration
/*====================================================================*/
static const int SD_ALG_SPECTRE_NB_PARAM_MODELE_ORBITAL = 6;

/*====================================================================*/
// Exported types Declaration
/*====================================================================*/

/*====================================================================*/
// External variables Declaration
/*====================================================================*/

/*====================================================================*/
//                  FONCTIONS INTERNES
//======================================================================

/*====================================================================*/
//                  FONCTIONS EXTERNES
//======================================================================

#ifdef __cplusplus
extern "C" {
#endif

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_20_DOC
//@
//@ Description : algorithme de decodage des spectres de niveau 0 a 
//@ appeler apres DecodeDataLigne0
//@ pour les modes intermediaires le decodage est effectue directement dans 
//@ SD_DON_GestDonnees::EncodeDecodeLigneS1 et cette fonction n'est pas appelee
//@
//@
//@ Parameters :
//@     (ENTREE)  ctx : contexte :numero de ligne SN et PN traite
//@     (ENTREE)  conf_autre : fichier de conf autre
//@     (ENTREE)  ctx_appel : identificateur de la chaine 
//@                           (SD_BAS_ISRFEM ou SD_BAS_PRODUIT)
//@     (ENTREE)  CodSpect  : spectre code (flot continu de bits, apres
//@                           rearrangement par SD_BAS_DecodeSpectreN0)
//@                           donnees decodees allouee par
//@                           l'appelant et issue de 
//@                           SD_DON_GestDonnees::EncodeDecodeLigneS1
//@                           (en principe les mots on ete rearranges et 
//@                           compactes)(AC)
//@     (SORTIE)  Spect       : spectre decode pour un ISP (GDocFen(PN,SN,LN)
//@     (SORTIE)  InfoDet1ENG : info det du produit ENG 
//@     (SORTIE)  validite_bandes_spectrales : contient SD_BAS_SB booleens 
//@               qui indiquent si chaque bande spectrale contient des overflow 
//@               ou des underflow
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_20_DOC(
    t_SD_BAS_DataThread * ctx,
    t_SD_BAS_ConfAutre *  conf_autre,
    int                   ctx_appel,
    const char *          CodSpect,
    double *              Spect,
    t_SD_BAS_Liste_Var *  InfoDet1ENG,
    t_CMN__Booleen *      validite_bandes_spectrales)
;

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_09_PLK
//@
//@ Description : calcul de la fonction de planck
//@ on utilise ici les valeurs de la ligne courante et pas celle de la
//@ ligne precedente (cf FD51 AC)
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@                   numlignes : numero de ligne
//@         (ENTREE)  conf_autre : fichier de conf autre
//@         (ENTREE)  BBT : temperature BBT (LN -1)
//@         (Sortie)  InfoCalcLigne : info generales calculees sur la ligne (N-1)
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_09_PLK(
    t_SD_BAS_DataThread *         ctx, 
    t_SD_BAS_ConfAutre *          conf_autre,
    double                        BBT, 
    t_SD_BAS_DataThreadProduit *  dataThreadProduit
    )
;


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_30_FTB
//@
//@ Description : filtrage de la temperature du corps noir
//@
//@
//@ Parameters :
//@     (ENTREE)  ctx             : contexte de l'etape
//@     (ENTREE)  NbLignes        : nb de lignes du granule traite
//@     (ENTREE)  ContexteCourant : pointeur sur le contexte courant conf.
//@     (ENTREE)  conf_autre       : fichier de conf autre
//@     (ENTREE)  InfoCalcLig     : infos generales calculees sur la ligne (SN)
//@     (SORTIE)  DataGenGranule  : Donnees generales au granule
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_30_FTB(
    t_SD_BAS_DataThread *        ctx, 
    int                          NbLignes, 
    t_SD_BAS_Contexte *          ContexteCourant,
    t_SD_BAS_ConfAutre *         conf_autre,
    t_SD_BAS_DataThreadProduit * InfoCalcLig,
    t_SD_BAS_DataGen  *          DataGenGranule)
;


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_31_SCR
//@
//@ Description : calibration spectrale du coef de calibration radiometrique
//@ on utilise ici les valeurs de la ligne courante et le BBT de la
//@ ligne precedente
//@
//@
//@ Parameters :
//@     (ENTREE)  ctx : contexte de l'etape
//@     (ENTREE)  conf_autre : fichier de conf autre
//@     (ENTREE)  spectre_decode : spectre decode pour un ISP (GDocSpect)
//@                                pour la ligne LN
//@     (ENTREE)  data_thread_produit : info generales de les lignes LN et LN-1
//@     (SORTIE)  spectre_calibre : spectre corrige (GScrSpect) pour la ligne LN
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_31_SCR(
                t_SD_BAS_DataThread *         ctx, 
                t_SD_BAS_ConfAutre *          conf_autre,
                t_SD_BAS_DataThreadProduit *  data_thread_produit, 
                double *                      spectre_decode, 
                double *                      spectre_calibre)
;


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_32_HEC
//@
//@ Description : correction d'emissivite du corps noir
//@ on utilise ici le spectre  de la ligne courante corrrige par les info
//@ generales de la ligne precedente. 
//@ Ces infos sont accessibles dans la structure t_SD_BAS_DataThreadProduit
//@
//@
//@ Parameters :
//@     (ENTREE)  ctx : contexte de l'etape
//@     (ENTREE)  conf_autre : fichier de conf autre
//@     (ENTREE/SORTIE)  data_thread_produit : info generales des lignes 
//@                                            LN et LN-1
//@     (ENTREE)  spectre_calibre : spectre calibre radiometriquement pour 
//@                                 LN, SN, PN (GScrSpect)
//@     (SORTIE)  spectre_corrige: spectre corrige de l'emissivite du corps 
//@                                noir pour LN SN PN (GHecSpect)
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_32_HEC(
                t_SD_BAS_DataThread *         ctx, 
                t_SD_BAS_ConfAutre *          conf_autre,
                t_SD_BAS_DataThreadProduit *  data_thread_produit,
                double *                      spectre_calibre,
                double *                      spectre_corrige)
;


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_33_SME
//@
//@ Description : estimation de la temperature du miroir de balayage 
//@
//@
//@ Parameters :
//@     (ENTREE)  ctx : contexte de l'etape
//@        numero de ligne 
//@     (ENTREE)  contextecourant : pointeur sur le contexte
//@                     courant configuration
//@     (ENTREE)  t : date UTC de reference de la ligne LN-1
//@     (ENTREE)  t_FE : date UTC de fin de l'eclipse
//@        precedant la ligne LN-1
//@     (ENTREE)  conf_autre : fichier de conf autre
//@     (ENtree/Sortie)  InfoDetEng : info det du produit ENG (LN-1)
//@   (on choisira la date UTC de debut recuperee dans InfoDetEng)
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_33_SME(
    t_SD_BAS_DataThread * ctx, 
    t_SD_BAS_SCDSTime     t,
    t_SD_BAS_SCDSTime     t_FE,
    t_SD_BAS_ConfAutre *  conf_autre,
    t_SD_BAS_Liste_Var *  InfoDetEng, 
    double *              t_scan)
;


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_34_SMC
//@
//@ Description : correction angulaire de reflectivite et de polarisation 
//@ du miroir de balayage
//@
//@
//@ Parameters :
//@     (ENTREE)  ctx : contexte de l'etape
//@     (ENTREE)  conf_stable : fichier de conf stable
//@     (ENTREE)  conf_autre : fichier de cof autre
//@     (ENTREE)  info_det_eng : info detaillee du produit ENG (LN-1) 
//@               dont GsmeTscan ( remplacer par la valeur directement)
//@     (ENTREE)  data_thread_produit : info generales calculees sur la ligne 
//@                                     LN et LN-1
//@     (ENTREE)  spectre_corrige_emissivite : spectre corrige de l'emissivite 
//@               du corps noir pour LN SN PN (GHecSpect)
//@     (ENTREE)  InfoDet1ENG : info det du produit ENG 
//@     (SORTIE)  spectre_corrige_reflectivite : spectre 1A pour LN SN PN
//@               corrige de la reflectivite et de la polarisation du miroir
//@               de balayage (GSmcSpect)
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_34_SMC(
    t_SD_BAS_DataThread *         ctx, 
    t_SD_BAS_ConfStable *         conf_stable,
    t_SD_BAS_ConfAutre *          conf_autre, 
    t_SD_BAS_DataThreadProduit *  data_thread_produit,
    t_SD_BAS_Liste_Var *          InfoDet1ENG, 
    double                        t_scan,
    double *                      spectre_corrige_emissivite, 
    double *                      spectre_corrige_reflectivite)
;


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_35_S1B
//@
//@ Description : re-echantillonnage du spectre 
//@
//@
//@ Parameters :
//@     (ENTREE)  ctx : contexte de l'etape
//@     (ENTREE)  conf_autre : fichier de conf autre
//@     (ENTREE)  data_thread_produit : info generales calculees sur la ligne 
//@                                     LN et LN-1
//@     (ENTREE)  sortie_22_SOS : info generales issues de l'algo SOS
//@     (ENTREE)  spectre_1a_surechantillonne : spectre 1A (GS1aOSSpect)
//@               taille OSNslast - OSNsfirst + 1
//@               OSNsfirst = Sortie22SOS->GSosSsdOSNsFirstSpect;
//@               OSNslast = Sortie22SOS->GSosSsdOSNsLastSpect;
//@               t_SD_BAS_22_SOS_Sortie *       Sortie22SOS;
//@     (Sortie)  spectre_1b : spectre 1B issu de LN SN PN (GS1bSpect)
//@               taille Nslast1b - Nsfirst1b + 1
//@               Nsfirst1b = conf_autre->IDefNsfirst1b;
//@               Nslast1b = conf_autre->IDefNslast1b;
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_35_S1B(
    t_SD_BAS_DataThread *         ctx, 
    t_SD_BAS_ConfAutre *          conf_autre,
    t_SD_BAS_DataThreadProduit *  data_thread_produit,
    t_SD_BAS_22_SOS_Sortie *      sortie_22_SOS,     
    double *                      spectre_1a_surechantillonne,
    double *                      spectre_1b)
;


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_37_S1C
//@
//@ Description : Apodisation  du spectre 1B
//@
//@
//@ Parameters :
//@     (ENTREE)  ctx : contexte de l'etape numero de ligne LN, SN et PN
//@     (ENTREE)  banque_spectrale : pointeur sur le fichier BanqueSpectrale
//@     (ENTREE)  conf_autre : fichier de conf autre
//@     (ENTREE)  data_thread_produit : info generales calculees sur la ligne LN
//@               dont Af (GIsfAf)
//@     (ENTREE)  spectre_1b: spectre 1B issu de LN SN PN (GS1bSpect)
//@               taille Nslast1b - Nsfirst1b + 1
//@               Nsfirst1b = conf_autre->IDefNsfirst1b;
//@               Nslast1b = conf_autre->IDefNslast1b;
//@     (SORTIE)  spectre_1c : spectre 1C issu de LN SN PN (GS1cSpect)
//@               taille Nslast1b - Nsfirst1b + 1 (idem que spectre_1b)
//@               Nsfirst1b = conf_autre->IDefNsfirst1b;
//@               Nslast1b = conf_autre->IDefNslast1b;
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_37_S1C(
    t_SD_BAS_DataThread *         ctx, 
    t_SD_BAS_BanqueSpectrale *    banque_spectrale,
    t_SD_BAS_ConfAutre *          conf_autre,
    t_SD_BAS_DataThreadProduit *  data_thread_produit,
    double *                      spectre_1b,
    double *                      spectre_1c)
;


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_XX_Initprod
//@
//@ Description : algorithme commun a une ligne IASI et realise au debut
//@ de la chaine produit ( apres 43_ISF, 09_PLK, 33_SME)
//@
//@
//@ Parameters :
//@     (ENTREE)  ctx : contexte de l'etape
//@     (ENTREE)  conf_autre : fichier de conf autre
//@     (ENTREE)  banque_spectrale : fichier banque spectrale
//@     (ENTREE)  contexte_courant : pointeur sur le contexte
//@                                  de courant configuration
//@     (ENTREE/SORTIE)  data_thread_produit : info generales calculees sur les 
//@                                            lignes LN et LN-1
//@     (ENTREE)  sortie_22_SOS : info generales issues de l'algo SOS
//@     (ENTREE)  t : date UTC de la ligne (SN=15) /* DM 11/03/2004 */
//@     (ENTREE)  t_FE : date de fin d'eclipse pour la ligne LN 
//@                      noter que cette date est anterieure a la date du 1er
//@                      scan de la ligne LN
//@     (ENTREE)  BBT : temperature du corps noir (LN-1)
//@     (SORTIE)  flag_date_NOK : flag indicateur d'une incoherence des dates 
//@                               (LN-1)
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_XX_Initprod(
  t_SD_BAS_DataThread *         ctx, 
  t_SD_BAS_ConfAutre *          conf_autre,
  t_SD_BAS_BanqueSpectrale *    banque_spectrale,
  t_SD_BAS_DataThreadProduit *  data_thread_produit, 
  t_SD_BAS_22_SOS_Sortie *      sortie_22_SOS, 
  t_SD_BAS_SCDSTime             t, 
  t_SD_BAS_SCDSTime             t_FE, 
  double                        BBT, 
  t_CMN__Booleen *              flag_date_NOK)
;

#ifdef __cplusplus
}
#endif

#endif
