//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/BAS/src/SD_BAS_AVHRR.c,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_BAS2
//@ MODULE : SD_BAS_AVHRR
//@
//@ Comment :
//@    Code file for the  SD_BAS_Niveau0 class which includes the
//@    acces to the AVHRR L2 product  :
//@	this is done thanks to SD_BAS_Conversion and SD_BAS_Produits
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 28/01/2003 : [reference :] ott
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-065-CN : 30/09/2003 : B. MASSART
//@ on initialise la taille de la liste des variables detaillees InfoDet
//@ dans SD_BAS_LireMDRAVHRR pour eviter une erreur lors de la liberation 
//@ de la memoire. Corrige egalement IA-2100-FA-076-CN.
//@ --------------------------------------------------------------------
//@ FA-ID :IA--CR-2100-257-CNE : 1/02/2005 : JL PASCAL
//@ Mise a jour des facteurs de decodage canal 3a 3b : 4
//@ Mise a jour de l'ordre de stockage des lattitude longitude
//@ dans le produit
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-159-CN : 26/05/2005 : JL. PASCAL
//@ modif suite a portage linux
//@ fiche de correction correspondante : IA-2100-FC-159-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA--CR-2100-269-CNE : 08/06/2005 : JL PASCAL
//@ test du nombre de pixel AVHRR et nombre de point de geoloc
//@ fiche de correction correspondante : IA-2100-FC-269-THA
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-3358(DM-2100-OPS) : 13/04/2017 : MLA
//@ Pour produits > 2Go, il faut utiliser des long long pour stocker la position du curseur 
//@ et les fonctions fseeko et ftello
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------


#include <stdio.h>
#include "SD_BAS_AVHRR.h"
#include <math.h>

// FT-2765
#include "SD_BAS_SwapEndian.h"
// ...FT-2765

// declaration des fonctions importees

//*====================================================================*/
//* Local Constant Declaration 
//*====================================================================*/
const char * SD_BAS_BLOC_DONGEN0 = "DONGEN";	 		//* nom du bloc DONGEN */
const char * SD_BAS_BLOC_MDR = "AVHRR_MDR";	 		//* nom du bloc DONDET */
//CR269
const char * SD_BAS_BLOC_SPHR = "AVHRR_SPHR";	 		//* nom du bloc DONDET */
//const int SD_BAS_TAILLE_MAX_MDR1B = 26634;
// version 6.3 penser a modifier le fichier SD_BAS_PRODUIT.c et SD_BAS_AVHRR.h et SD_BAS_AVHRR.env
const int SD_BAS_TAILLE_MAX_MDR1B = 26644;
const char * SD_BAS_BLOC_GRH = "GRH";  	 	//* nom du bloc GRH */
const int SD_BAS_AVHRR_OFFSET_IMAGE = 24;    // position des images dans les MDR 1B
//const int SD_BAS_AVHRR_OFFSET_EARTH_LOCATION = 21372;// position des informations de geolocalisation dans les MDR 1B
/* cvs? CR257 */
const int SD_BAS_AVHRR_OFFSET_EARTH_LOCATION = 21380;// position des informations de geolocalisation dans les MDR 1B
const int SD_BAS_AVHRR_IMAGE_SF_3a3b = 4;// facteur a appliquer aux images (10^(- SF))
const int SD_BAS_AVHRR_INDICE_3a3b = 2;// facteur a appliquer aux images (10^(- SF))
const int SD_BAS_AVHRR_IMAGE_SF = 2;// facteur a appliquer aux images (10^(- SF))
const int SD_BAS_AVHRR_GEOLOC_SF = 4;// facteur a appliquer aux informations de geolocalisation (10^(- SF))
const int SD_BAS_SIZEOF_DATA = 2;
//CR269
const int SD_BAS_AVHRR_TAILLE_SPHR = 123;// taille sphr
const int SD_BAS_AVHRR_OFFSET_SPHR = 3327;// offset depuis le debut du produit


//#define SD_BAS_SIZE_OF_DATA_GEOLOC 102 


static char* SD_BAS_MSG_VERSION_AVHRR = "INVALID AVHRR VERSION";
static char* SD_BAS_MSG_LIREMPHRAVHRR = "INVALID AVHRR MPHR";
static char* SD_BAS_MSG_LIREIPRAVHRR  = "INVALID AVHRR IPR";
//CR269
static char* SD_BAS_MSG_TAILLE_AVHRR  = "AVHRR INVALID ARRAY SIZE : %d found ";


//*====================================================================*/
//* Local types Declaration  
//*====================================================================*/
//======================================================================
// Internal fonctions Declaration
//======================================================================

//======================================================================
// Global variables Definition
//======================================================================
//======================================================================
//					FONCTIONS EXTERNES
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LireMPHRAVHRR
//@
//@ Description : fonction de lecture du MPHR d'un produit AVHRR
//@  cette operation generique s'applique aux produits AVHRR de N2 
//@
//@
//@ Parameters :
//@			(ENTREE)  descripteur  : descripteur du produit
//@				  utilise par les fonctions d'acces
//@			(SORTIE)  mphr  : liste des variables lues dans le MPHR
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LireMPHRAVHRR(
		t_SD_BAS_Fd * Descripteur,
		t_SD_BAS_MPHR * mphr,
		t_SD_BAS_DataThread *ctx
			)
			
{
  static const char tcProcName_F[] = "SD_BAS_LireMPHRAVHRR";
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  char * msg_err = NULL;
  int num_lig = 0;
  //CR269 buffer pour le SPHR du rpoduit
  t_SD_BAS_Liste_Var sphr;
  int nb_element;
  /* buffer de lecture */
  char *bloc;
  /* retour de fonction */
  int ret;
    /* longueur des donnees lues */
  int lg = 0;
  /* nom du bloc a ecrire */
  char nom_bloc[SD_BAS_LG_NOM_BLOC];
  t_SD_BAS_status_d status_d;
  char msg[1024];


  // on appelle la fonction SD_BAS_LireMPHR afin 
  // de lire le MPHR
  cr_op = SD_BAS_LireMPHR(Descripteur,mphr,ctx);
  if (cr_op != CMN__STATUT_OK)
    {
      cr_op   = SD_BAS_ERR_BAS_NIVEAU;
      msg_err = SD_BAS_MSG_LIREMPHRAVHRR;
      num_lig = __LINE__;
      goto fin;
    }

  // on verifie que la version lue dans le MPH est compatible avec la version
  // de la couche : Descripteur->version <= SD_BAS_AVHRR_VERSION
  // sinon on emmet une erreur et on sort
  if(( Descripteur->version.major > SD_BAS_AVHRR_VERSION.major) ||
     Descripteur->version.major == SD_BAS_AVHRR_VERSION.major && 
     Descripteur->version.minor > SD_BAS_AVHRR_VERSION.minor)
    {
      cr_op   = SD_BAS_ERR_VERSION_AVHRR;
      msg_err = SD_BAS_MSG_VERSION_AVHRR;
      num_lig = __LINE__;     
      goto fin;
    }


  // on stocke en interne le nombre de MDR
  Descripteur->nombreMDR =  *(mphr->listevar.tab_var[MPHR_TOTAL_MDR].val.pui4);

  //CR269 on s'alloue un buffer pour le SPHR du rpoduit
    if( SD_BAS_MacMALLOC(bloc,char,SD_BAS_AVHRR_TAILLE_SPHR) == FALSE)
           {
	     cr_op = SD_BAS_ERR_BAS_NIVEAU;
	     msg_err = SD_BAS_MSG_ALLOC;
	     num_lig = __LINE__;
	     goto fin;
           }
  
  //CR269 on se place prealablement a l'offset de SD_BAS_AVHRR_OFFSET_SPHR
     ret = fseeko(Descripteur->fd ,SD_BAS_AVHRR_OFFSET_SPHR , SEEK_SET);
     
     if(ret == -1)
       {
	 cr_op = SD_BAS_ERR_BAS_NIVEAU;
	 msg_err = SD_BAS_MSG_SEEK;
	 num_lig = __LINE__;
	 goto fin;
       }
  
  //CR269 on lit dans le fichier une taille de SPHR dans le cas ou ce n'est pas un product model
   nb_element=1;
   ret = fread(bloc, SD_BAS_AVHRR_TAILLE_SPHR, nb_element, Descripteur->fd);
   if(ret != nb_element)
       {
	 cr_op = SD_BAS_ERR_BAS_NIVEAU;
	 msg_err = SD_BAS_MSG_READ;
	 num_lig = __LINE__;
	 goto fin;
       }
 
  //CR269 on decommune le buffer precedent avec utilisation d'un bloc specifique
   strcpy(nom_bloc, SD_BAS_BLOC_SPHR);
   status_d=DECOM__Decommuter(bloc,
                                 lg,
				 nom_bloc,"",
                                 &sphr);
   SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_DECOMMUTER ,__LINE__);  

  //CR269 on libere le buffer precedent un buffer pour le SPHR du rpoduit
  SD_BAS_MacFREE(bloc);
 
  //CR269 on controle que la taille en nombre de pixel est correcte
  if (sphr.tab_var[AVHRR_EARTH_VIEW_PER_SCAN_LINE].val.pi4[0] != SD_BAS_NPI_AVHRR)
    {
      cr_op   = SD_BAS_ERR_BAS_NIVEAU;
      sprintf(msg,SD_BAS_MSG_TAILLE_AVHRR,sphr.tab_var[AVHRR_EARTH_VIEW_PER_SCAN_LINE].val.pui4[0]);
      msg_err=msg;
      num_lig = __LINE__;     
      goto fin;
    }
    

  //CR269 on controle que la taille en nombre de points de geoloc est correcte
  if (sphr.tab_var[AVHRR_NAV_SAMPLE_RATE].val.pi4[0] != SD_BAS_SAMPLE_OF_DATA_GEOLOC)
    {
      cr_op   = SD_BAS_ERR_BAS_NIVEAU;
      sprintf(msg,SD_BAS_MSG_TAILLE_AVHRR,sphr.tab_var[AVHRR_NAV_SAMPLE_RATE].val.pui4[0]);
      msg_err=msg;
      num_lig = __LINE__;     
      goto fin;
    }
  
  //CR269 on libere la listevar du sphr
  SD_BAS_LibereListeVar(LECTURE, &sphr);

  // SI c'est le premier appel (etat INIT) on initialise l'offset des MDR
  if (Descripteur->etat == INIT)
    {
      //calcul de l'offset des MDR
      cr_op = SD_BAS_LireIPR(Descripteur,
                             ctx);
      if (cr_op != CMN__STATUT_OK)
        {
          cr_op   = SD_BAS_ERR_BAS_NIVEAU;
          msg_err = SD_BAS_MSG_LIREIPRAVHRR;
          num_lig = __LINE__;
          goto fin;
        }

      
      Descripteur->offsetMDRSuivant = Descripteur->offsetMDR;
      
      // on passe a etat ENCOURS
      Descripteur->etat = ENCOURS;
    }
  // FinSI


 fin:
  // sortie de la fonction 
  if ( cr_op != CMN__STATUT_OK)
    {
      /* mise a jour de la variable ctx->cr.errinterne */
      SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                          SD_BAS_ERR_SET_AVHRR,
                          SD_BAS_ERR_NUM_JDB,
                          tcProcName_F,
                          num_lig,
                          msg_err);
    }
  
  return(cr_op); 
  
}

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LireMDRGenAVHRR
//@
//@ Description : fonction de lecture des donnees gen des IPR d'un produit 
//@  cette operation generique s'applique aux produits AVHRR 
//@  on lit ici le grh et quelques info detaillees  ( soit sequentiellement)
//@  soit a partir d'une adresse
//@ 
//@ SD_BAS_LireMPHRAVHRR doit avoir ete appele avant
//@
//@ Parameters :
//@			(ENTREE)  init  : contexte d'initialisation
//@				init = INIT ( on lit le suivant )
//@				init = NORMAL ( on lit a l'adresse indique )
//@			(ENTREE)  offset  : offset pour le mode normal
//@			(ENTREE)  descripteur  : descripteur du produit
//@				  utilise par les fonctions d'acces
//@			(SORTIE)  dongen  : liste des variables lues dans le MDR
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LireMDRGenAVHRR(
		int init,
		int offset,
		t_SD_BAS_Fd * Descripteur,
		t_SD_BAS_Liste_Var * dongen,
		t_SD_BAS_DataThread *ctx
    )
{
  static const char tcProcName_F[] = "SD_BAS_LireMDRGenAVHRR";
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  char * bloc = NULL;
  int lg = 0;
  t_SD_BAS_Liste_Var listevar;
  unsigned int version = 0;
  unsigned int subclass = 0;
  off_t taille_rec = 0;
  int num_lig = 0;
  int ret = 0;
  size_t nbElement = 0;
  off_t  offset_vrai = 0;
  t_SD_BAS_status_d status_d;
  DECOM__T_val_var   valeur;
  char * msg_err = NULL;
 

// on verifie que l'etat est a INIT ou ENCOURS 
// et que le mode est lecture 
  if ( ((Descripteur->etat == INIT) || (Descripteur->etat == ENCOURS))
       && (Descripteur->mode == LECTURE))
    {

// si INIT
//   offset = offset MDR courant
      if (init == INIT)
        offset_vrai = Descripteur->offsetMDRSuivant;

// sinon NORMAL
//   offset = parametre
      else 
        offset_vrai = offset;


//   on alloue un tableau de char de taille SD_BAS_TAILLE_GRH

      if( SD_BAS_MacMALLOC(bloc,char,SD_BAS_TAILLE_GRH) == FALSE)
        {
          cr_op = SD_BAS_ERR_BAS_NIVEAU;
          msg_err = SD_BAS_MSG_ALLOC;
          num_lig = __LINE__;
          goto fin;
        }


//   on recupere un bloc correspondant a une taille de SD_BAS_TAILLE_GRH dans le
//   fichier prealablement ouvert a l'offset du MDR courant 
      ret =fseeko(Descripteur->fd, offset_vrai, SEEK_SET);
      if(ret == -1)
        {
          cr_op = SD_BAS_ERR_BAS_NIVEAU;
          num_lig = __LINE__;
          msg_err = SD_BAS_MSG_SEEK;
          goto fin;   
        }     

      
      nbElement = fread(bloc, SD_BAS_TAILLE_GRH, 1, Descripteur->fd);
      if (nbElement != 1)
        {
          cr_op = SD_BAS_ERR_BAS_NIVEAU;
          num_lig = __LINE__;
          msg_err = SD_BAS_MSG_READ;
          goto fin;   
        }     


//   on appelle la couche de decodage afin de lire le contenu du GRH (SD_BAS_DecommuteGRH) 
//   DECOM__Decommuter ( bloc SD_BAS_BLOC_GRH fic SD_BAS_NOM_FIC_PROD)
      status_d=DECOM__Decommuter(bloc,lg ,SD_BAS_BLOC_GRH,"",&listevar );
      SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_DECOMMUTER, __LINE__);

      
      SD_BAS_MacFREE(bloc);
      

//   on recupere ici la version du record ( champ record_subclass_version ) 
      version = *(listevar.tab_var[GRH_RECORD_SUBCLASS_VERSION].val.pui4);

//   on recupere ici la taille du record 
      taille_rec = *(listevar.tab_var[GRH_RECORD_SIZE].val.pui4);


//   et on verifie que la version est < SD_BAS_IPR_VERSION
      if( version <= SD_BAS_MDR_AVHRR_VERSION)
        {         

          // on initialise dongen
          dongen->nb_var = 0;
          dongen->tab_var = NULL;
          


//   on recupere ici la tpe la sous classe du record ( champ record_subclass )
          subclass = *(listevar.tab_var[GRH_RECORD_SUBCLASS].val.pui4);
          
         



//   on ajoute dans la liste des variables decommutees la taille du record

          status_d=DECOM__Ajouter_variable (dongen,"","AVHRR_LONGUEURREC",
                                            DECOM__C_uint4,1,valeur);
          SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_DECOMMUTER, __LINE__);
// on alloue dans la liste de variable la taille de la valeur

          ret = SD_BAS_MacMALLOC(dongen->tab_var[AVHRR_LONGUEURREC].val.pui4, unsigned int, 1);
          if( ret == FALSE )
          
            {
              cr_op = SD_BAS_ERR_BAS_NIVEAU;
              num_lig = __LINE__;
              msg_err = SD_BAS_MSG_ALLOC;
              goto fin;
            }

// on affecte la valeur

          *dongen->tab_var[AVHRR_LONGUEURREC].val.pui4 = taille_rec;





//   on ajoute dans la liste des variables decommutees offsetMDRsuivant
          status_d=DECOM__Ajouter_variable (dongen,"","AVHRR_OFFSET",
                                            DECOM__C_uint4,1,valeur);
          SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_AJOUTER, __LINE__);


// on alloue dans la liste de variable la taille de la valeur

          ret = SD_BAS_MacMALLOC(dongen->tab_var[AVHRR_OFFSET].val.pui4, unsigned int, 1);
          if( ret == FALSE )

            {
              cr_op = SD_BAS_ERR_BAS_NIVEAU;
              num_lig = __LINE__;
              msg_err = SD_BAS_MSG_ALLOC;
              goto fin;   
            }     

// on affecte la valeur

          *dongen->tab_var[AVHRR_OFFSET].val.pui4 =
            Descripteur->offsetMDRSuivant;



//   on ajoute dans la liste des variables decommutees la validite du record
          status_d=DECOM__Ajouter_variable (dongen,"","AVHRR_VALIDREC",
                                            DECOM__C_uint4,1,valeur);
          SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_AJOUTER, __LINE__);

// on alloue dans la liste de variable la taille de la valeur

          ret = SD_BAS_MacMALLOC(dongen->tab_var[AVHRR_VALIDREC].val.pui4,unsigned int, 1 );
          if( ret == FALSE )

            {
              cr_op = SD_BAS_ERR_BAS_NIVEAU;
              num_lig = __LINE__;
              msg_err = SD_BAS_MSG_ALLOC;
              goto fin;
            }


// on affecte la valeur suivant le type de l'instrument

//   si intrument == dummy
          if(*listevar.tab_var[GRH_INSTRUMENT_GROUP].val.pui4 == IN_DUMMY)
//    on ajoute un type de donnee dans la liste de variable qui vaut invalide
            *dongen->tab_var[AVHRR_VALIDREC].val.pui4 = INVALIDE;

//   sinon on ajoute un type de donnee dans la liste de variable qui vaut valide
          else
            *dongen->tab_var[AVHRR_VALIDREC].val.pui4 = VALIDE;




//   on ajoute dans la liste des variables decommutees offsetMDRsuivant
          status_d=DECOM__Ajouter_variable (dongen,"","DATEDEB",
                                            DECOM__C_date,1,valeur);
          SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_AJOUTER, __LINE__);


// on alloue dans la liste de variable la taille de la valeur

          ret = SD_BAS_MacMALLOC(dongen->tab_var[AVHRR_DATEDEB].val.pdate, t_SD_BAS_SCDSTime, 1 );
          if( ret == FALSE )
          
            {
              cr_op = SD_BAS_ERR_BAS_NIVEAU;
              num_lig = __LINE__;
              msg_err = SD_BAS_MSG_ALLOC;
              goto fin;
            }

// on affecte la valeur

          dongen->tab_var[AVHRR_DATEDEB].val.pdate->day = 
            listevar.tab_var[GRH_RECORD_START_TIME].val.pdate->day;
          dongen->tab_var[AVHRR_DATEDEB].val.pdate->millis = 
            listevar.tab_var[GRH_RECORD_START_TIME].val.pdate->millis;


//   on ajoute dans la liste des variables decommutees offsetMDRsuivant
          status_d=DECOM__Ajouter_variable (dongen,"","DATEFIN",
                                            DECOM__C_date,1,valeur);
          SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_AJOUTER, __LINE__);


// on alloue dans la liste de variable la taille de la valeur

          ret = SD_BAS_MacMALLOC(dongen->tab_var[AVHRR_DATEFIN].val.pdate, t_SD_BAS_SCDSTime, 1 );
          if( ret == FALSE )
            {
              cr_op = SD_BAS_ERR_BAS_NIVEAU;
              num_lig = __LINE__;
              msg_err = SD_BAS_MSG_ALLOC;
              goto fin;
            }

// on affecte la valeur

          dongen->tab_var[AVHRR_DATEFIN].val.pdate->day = 
            listevar.tab_var[GRH_RECORD_STOP_TIME].val.pdate->day;
          dongen->tab_var[AVHRR_DATEFIN].val.pdate->millis = 
            listevar.tab_var[GRH_RECORD_STOP_TIME].val.pdate->millis;


//   on ajoute dans la liste des variables decommutees la validite du record
          status_d=DECOM__Ajouter_variable (dongen,"","AVHRR_FLAG",
                                            DECOM__C_uint4,1,valeur);
          SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_AJOUTER, __LINE__);

// on alloue dans la liste de variable la taille de la valeur

          ret = SD_BAS_MacMALLOC(dongen->tab_var[AVHRR_FLAG].val.pui4,unsigned int, 1 );
          if( ret == FALSE )
	    
            {
              cr_op = SD_BAS_ERR_BAS_NIVEAU;
              num_lig = __LINE__;
              msg_err = SD_BAS_MSG_ALLOC;
              goto fin;
            }

// on teste la date UTC par rapport a la date UTC precedente et on
// retourne une erreur si date UTC <= date UTC precedente
// on renseigne dateSCDSprec
	  
	  if(dongen->tab_var[AVHRR_DATEDEB].val.pdate->day < Descripteur->dateSCDSprec.day 
	     ||(dongen->tab_var[AVHRR_DATEDEB].val.pdate->day == Descripteur->dateSCDSprec.day 
	     && dongen->tab_var[AVHRR_DATEDEB].val.pdate->millis < Descripteur->dateSCDSprec.millis ))
	    *(dongen->tab_var[AVHRR_FLAG].val.pui4) = INVALIDE;
	     
	  else
	    *(dongen->tab_var[AVHRR_FLAG].val.pui4) = VALIDE;

     



        }
// sinon on sort en erreur
else
    {
      cr_op   = SD_BAS_ERR_VERSION_AVHRR;
      msg_err = SD_BAS_MSG_VERSION_AVHRR;
      num_lig = __LINE__;     
      goto fin;
    }





// on met a jour offsetMDRsuivant = offsetMDRsuivant + record_size
  Descripteur->offsetMDRSuivant =  Descripteur->offsetMDRSuivant + taille_rec;
  
      SD_BAS_LibereListeVar(ECRITURE, &listevar);
    }
//    fin si




// 
fin:
// sortie de la fonction 

  if ( cr_op != CMN__STATUT_OK )
    {
     /* mise a jour de la variable ctx->cr.errinterne */
      SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                         SD_BAS_ERR_SET_AVHRR,
                         SD_BAS_ERR_NUM_JDB,
                         tcProcName_F,
                         num_lig,
                         msg_err);


   }

  return(cr_op); 

}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LireMDRAVHRR
//@
//@ Description : fonction de lecture  des MDR  d'un produit AVHRR 1B
//@  cette operation s'applique aux produits AVHRR
//@  on lit ici les info detaillees 
//@  on lit ici les info detaillees  en considerant que l'offset est renseigne
//@
//@
//@ Parameters :
//@			(ENTREE)  descripteur  : descripteur du produit
//@				  utilise par les fonctions d'acces
//@			(SORTIE)  mdr  : liste des variables lues dans le MDR
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LireMDRAVHRR(
		t_SD_BAS_Fd * Descripteur,
		t_SD_BAS_LigneAVHRR * mdr,
    t_SD_BAS_DataThread *ctx
			)
{
  static const char tcProcName_F[] = "SD_BAS_LireMDRAVHRR";
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  int lg = 0;
  char *bloc = NULL;
  int init = 0;
  unsigned int offset = 0;
  int num_lig = 0;
  int ret = 0;
  size_t nbElement = 0;
  t_SD_BAS_status_d status_d;
  char * msg_err = NULL;
  short int image[SD_BAS_NPI_AVHRR * SD_BAS_NBA_AVHRR];
  int geoloc[2 * SD_BAS_SIZE_OF_DATA_GEOLOC];
  int i = 0;
  e_BAS_ValidRec     validite;
  int j;
  double factim;
  double invfactim;
/* cvs? CR257 */
  double factim3a3b;
  double invfactim3a3b;
  double factgeo;
  double invfactgeo;
  double invfactthermique;

// on verifie que l'etat est a INIT ou ENCOURS
// et que le mode est lecture 
  if ( ((Descripteur->etat == INIT) || (Descripteur->etat == ENCOURS))
       && (Descripteur->mode == LECTURE))
    {


// on initialise la structure mdr->InfoGen par appel a SD_BAS_LireMDRGen (init
// =1)
     init =1;
     SD_BAS_LireMDRGenAVHRR(init,
                            Descripteur->offsetMDRSuivant,
                            Descripteur,
                            &mdr->InfoGene,
                            ctx);
    if ( cr_op != CMN__STATUT_OK )
    {
      /* affectation des variables de gestion des erreurs */
      num_lig = __LINE__;
      goto fin;
    }

   /* on initialise la taille de la liste des variables detaillees InfoDet */
   mdr->InfoDet.nb_var = 0;

// on recupere dans la liste des variables decommutees offsetMDRsuivant
    offset = *mdr->InfoGene.tab_var[AVHRR_OFFSET].val.pui4;
   /* ainsi que la validite du record */
    validite = *(mdr->InfoGene.tab_var[AVHRR_VALIDREC].val.pui4);


   /* si le record n'est pas invalide */
    if ( validite != INVALIDE )
    {
// on s'alloue un tableau de char de longueur taille du record que l'on stocke dans .
    if( SD_BAS_MacMALLOC(bloc, char, SD_BAS_TAILLE_MAX_MDR1B) == FALSE)
      {
        cr_op = SD_BAS_ERR_BAS_NIVEAU;
        msg_err = SD_BAS_MSG_ALLOC;
        num_lig = __LINE__;
        goto fin;
      }

// on recupere un bloc correspondant a une taille du record dans le
// fichier prealablement ouvert a l'offset du MDR courant (hors GRH)
    ret = fseeko(Descripteur->fd, offset, SEEK_SET);
    if(ret == -1)
      {
        cr_op = SD_BAS_ERR_BAS_NIVEAU;
        msg_err = SD_BAS_MSG_SEEK;
        num_lig = __LINE__;
        goto fin;
      }
    
    nbElement = fread(bloc, SD_BAS_TAILLE_MAX_MDR1B, 1, Descripteur->fd);
    if (nbElement != 1)
      {
        cr_op = SD_BAS_ERR_BAS_NIVEAU;
        msg_err = SD_BAS_MSG_READ;
        num_lig = __LINE__;
        goto fin;
      }

// son appelle la couche de decodage afin de lire 
// le contenu du MDR  ( hors image)
// DECOM__Decommuter ( bloc SD_BAS_BLOC_MDR P fic SD_BAS_NOM_FIC_PRODAVHRR)
// on stocke ceci dans le champ InfoDet;

  status_d=DECOM__Decommuter(bloc,lg ,SD_BAS_BLOC_MDR,"",&mdr->InfoDet );
  SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_DECOMMUTER, __LINE__);

  //on teste AVHRR channel pour savoir s'il faut recuperer le channel 3a ou 3b

#ifdef OPS_DEBUG
  printf("AVHRR channel utilise (3a/3b) : %d\n",
         *mdr->InfoDet.tab_var[AVHRR_CHANNEL].val.pui4);
#endif


  /* on recopie les images dans la structure t_SD_BAS_ImageAVHRR */
 
      memcpy((char*)(image),
             &bloc[SD_BAS_AVHRR_OFFSET_IMAGE],
             SD_BAS_NPI_AVHRR * SD_BAS_NBA_AVHRR * SD_BAS_SIZEOF_DATA);
      // FT-2765
      SWAP_INT_BUFFER( image, SD_BAS_SIZEOF_DATA, SD_BAS_NPI_AVHRR * SD_BAS_NBA_AVHRR );
      // ...FT-2765

    // precalcul des scale facteurs 
   factim=1.;
    for (j=0;j<SD_BAS_AVHRR_IMAGE_SF;j++)
           factim=factim*10.;
    invfactim=1./factim;
   factgeo=1.;
    for (j=0;j<SD_BAS_AVHRR_GEOLOC_SF;j++)
           factgeo=factgeo*10.;
    invfactgeo=1./factgeo;
   invfactthermique=1./100000.;
/*  CR257 */
   factim3a3b=1.;
    for (j=0;j<SD_BAS_AVHRR_IMAGE_SF_3a3b;j++)
           factim3a3b=factim3a3b*10.;
    invfactim3a3b=1./factim3a3b;
    


  //on decode les images en applicant le scale factor
  for(i=0; i<SD_BAS_NPI_AVHRR*SD_BAS_NBA_AVHRR; i++)
    mdr->Image.im[i] = (float)((double)image[i] * invfactim);
//    mdr->Image.im[i] = (float)((double)image[i] * pow(10. ,(double) -SD_BAS_AVHRR_IMAGE_SF));
   
/*  CR257 */
  //on modifie alors le canal 3a3b
  for(i=0; i<SD_BAS_NPI_AVHRR; i++)
    mdr->Image.im[SD_BAS_NPI_AVHRR*SD_BAS_AVHRR_INDICE_3a3b+i] = (float)((double)image[SD_BAS_NPI_AVHRR*SD_BAS_AVHRR_INDICE_3a3b+i] * invfactim3a3b);
    
// en decodant les informations radiometriques ( scale factor)
// on recopie (Les Info de geolocalisation) depuis le tableau de char

//DCR 79
// conversion en W des canaux 3b 4 et 5 
  for(i=0; i<SD_BAS_NBA_AVHRR; i++)
    {
    if (i > 2 || ( i == 2 && *mdr->InfoDet.tab_var[AVHRR_CHANNEL].val.pui4== SELECTED_CHANNEL_3B ))
      {
      for(j=0; j<SD_BAS_NPI_AVHRR; j++)
      mdr->Image.im[i*SD_BAS_NPI_AVHRR+j] =mdr->Image.im[i*SD_BAS_NPI_AVHRR+j]*invfactthermique ;
      }
     }

  /* on recopie 2 * (SD_SIZE_OF_DATA_GEOLOC * 4) octets depuis le tableau de charcorrespondant aux latitudes et aux longitudes */
  memcpy(geoloc, &bloc[SD_BAS_AVHRR_OFFSET_EARTH_LOCATION],  2 * SD_BAS_SIZE_OF_DATA_GEOLOC * sizeof(int));
  // FT-2765
  SWAP_INT_BUFFER( geoloc, sizeof(int), 2 * SD_BAS_SIZE_OF_DATA_GEOLOC );
  // ...FT-2765

  /* on decode les latitudes  et les longitudes en applicant le scale factor */
  for(i=0; i<SD_BAS_SIZE_OF_DATA_GEOLOC; i++)
  {
/*  CR257 */
    mdr->Image.latitude[i]  = (double)geoloc[2*i] * invfactgeo;
    mdr->Image.longitude[i] = (double)geoloc[2*i+1] * invfactgeo;
//modif PM3
//    mdr->Image.latitude[i]  = (double)geoloc[i] * invfactgeo;
//    mdr->Image.longitude[i] = (double)geoloc[i+SD_BAS_SIZE_OF_DATA_GEOLOC] * invfactgeo;
//    mdr->Image.latitude[i]  = (double)geoloc[2*i] * pow(10. ,(double) -SD_BAS_AVHRR_GEOLOC_SF);
//    mdr->Image.longitude[i] = (double)geoloc[2*i+1] * pow(10. ,(double) -SD_BAS_AVHRR_GEOLOC_SF);
  }
 
  /* on libere le buffer alloue */
  SD_BAS_MacFREE(bloc);

    } /* fin du cas ou le record est valide */
  }/* fin du test sur l'etat et le mode */


fin:
// sortie de la fonction 
  if ( cr_op != CMN__STATUT_OK )
     {
     /* mise a jour de la variable ctx->cr.errinterne */
     SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                         SD_BAS_ERR_SET_AVHRR,
                         SD_BAS_ERR_NUM_JDB,
                         tcProcName_F,
                         num_lig,
                         msg_err);


   }


 return(cr_op);

 
}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LibereMDRAVHRR
//@
//@ Description : fonction de liberation des MDR  d'un produit 
//@  cette operation s'applique aux produits AVHRR  
//@  on desallouie ici toute la place allouee precedenment 
//@
//@
//@ Parameters :
//@			(ENTREE/SORTIE)  mdr  : liste des variables lues dans le MDR
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LibereMDRAVHRR(
				    t_SD_BAS_LigneAVHRR * mdr
				    )
{

t_CMN__Statut cr_op=CMN__STATUT_OK;



// on libere la liste dans la structure mdr->InfoGen par appel a SD_BAS_LibereListeVar
//( liberation globale)
 SD_BAS_LibereListeVar(LECTURE, &mdr->InfoGene);
 
// on libere la liste dans la structure mdr->InfoDet par appel a SD_BAS_LibereListeVar
//( liberation globale)
 DECOM__Liberer_Stockage_liste_var(&mdr->InfoDet);
 

fin:
// sortie de la fonction 
  return(cr_op);

}
//======================================================================
//					FONCTIONS INTERNES
//======================================================================

