//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/BAS/src/SD_BAS_BanqueSpectrale.h,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Component : SD_BAS
//@ MODULE : SD_BAS_BanqueSpectrale
//@
//@ Comment :
//@    include file used to define Spectral Dase configuration file
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_BAS_BanqueSpectrale_h
#define SD_BAS_BanqueSpectrale_h

/* librairies C de bases utilisees par les modules du projet */
#include <stdio.h>       /* entree/sortie */
#include <stdlib.h>      /* alloc memoire, ...*/
#include <string.h>      /* chaines de carateres */
#include <unistd.h>      /* gestion pipes, ... */
#include <time.h>        /* gestion de la date */
#include <sys/param.h>   /* constantes systeme */

// inserer ici les fichiers include 
#include "SD_BAS_Global.h"
#include "ERR__Erreur.h"

/*====================================================================*/
/*   External define  Declarations                                    */
/*====================================================================*/

/*-- constantes pour dimensionner les tableaux --*/
#define SD_BAS_CDX        2
#define SD_BAS_SDB_NB_COL 11
#define SD_BAS_SDB_NB_LIG 11
#define SD_BAS_SAF_NS_OPD 301
#define SD_BAS_SDB_NFSMAX 142
#define SD_BAS_WNP_NB_SW  10


//====================================================================*/
// Exported types Declaration
//====================================================================*/
// structure contenant les informations contenues dans la base de donnees
// spectrale
// les tableaux multi-dimensionnels sont declares comme des tableaux 1D
// ordre des dimensions = les indices qui varient le + vite a gauche
typedef struct
	{
    /* Issue Number of the format of the configuration file */
    int       IDefIssueIcd;

    /* Revision Number of the format of the configuration file */
    int       IDefRevisionIcd;

    /* Identification of this spectral database */
    int       IDefSdbID;

    /* Date of the validity period of the IASI I1 PPS spectral database */
    t_SD_BAS_SCDSTime IDefSdbDate;

    /* Number of the METOP satellite */
    int       IDefMETOPNumber;

    /* Spectral width of the spectral functions sampling */
    double    IDefSafDWn;

    /* Number of the sample corresponding to the first instrument function */
    int       IDefSafNsfirst;

    /* Number of the sample corresponding to the last instrument function */
    int       IDefSafNslast;

    /* Number of points for the sampling of the OPD */
    int       IDefSafNsOpd;

    /* Maximum of optical path difference represented by the self-apodization function */
    double    IDefSafOpdMax;

    /* Number of columns of the grid of interferometric axis positions */
    int       IDefSdbGridNbCol;

    /* Number of lines of the grid of interferometric axis positions */
    int       IDefSdbGridNbLin;

    /* Y angular coordinates of the grid points */
    double    IDefSdbGridY[SD_BAS_CDX*SD_BAS_SDB_NB_COL];

    /* Z angular coordinates of the grid points */
    double    IDefSdbGridZ[SD_BAS_CDX*SD_BAS_SDB_NB_LIG];

    /* Apodisation functions in the spectral database */
    cmplx     ISdbGridAf[SD_BAS_SAF_NS_OPD*SD_BAS_SDB_NFSMAX*SD_BAS_PN*SD_BAS_CDX*SD_BAS_SDB_NB_COL*SD_BAS_SDB_NB_LIG];

    /* Spectral calibration functions in the spectral database */
    double    ISdbGridFcs[SD_BAS_SDB_NFSMAX*SD_BAS_PN*SD_BAS_CDX*SD_BAS_SDB_NB_COL*SD_BAS_SDB_NB_LIG];

    /* Spectral shifts computed at the grid points */
    double    ISdbGridWnShift[SD_BAS_PN*SD_BAS_CDX*SD_BAS_WNP_NB_SW*SD_BAS_SDB_NB_COL*SD_BAS_SDB_NB_LIG];

	} t_SD_BAS_BanqueSpectrale;


#ifdef __cplusplus
extern "C" {
#endif

//====================================================================*/
// External variables Declaration
//====================================================================*/
// FT-2765
static const t_SD_BAS_VERSION SD_BAS_BANQUE_SPECTRALE_VERSION={1,6};
// ...FT-2765
//====================================================================*/
// External fonctions Declaration
//====================================================================*/

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_ChargerBS
//@
//@ Description : ouverture lecture et fermeture du fichier de banque
//@ spectrale indique en argument
//@
//@
//@ Parameters :
//@			(ENTREE)  Fichier : nom du fichier de bande spectrale
//@			(ENTREE/SORTIE)  BS : banque spectrale lue
//@  			(ENTREE)  ctx : contexte de l'etape
//@
//@ Return Status :
//@			t_CMN__Statut
//@			CMN__STATUT_OK si la fonction a reussi
//@		        SD_BAS_BS_ERR_OUVERTURE si on ne peut pas ouvrir le fichier
//@                     SD_BAS_BS_ERR_FERMETURE si on ne peut pas fermer le fichier
//@                     SD_BAS_BS_ERR_LECTURE   si on ne peut lire
//@                     SD_BAS_VERSION si la longueur du fichier ou le numero
//@                     d'enregistrement sont different de ceux predefinis
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_ChargerBS(
		char * Fichier,
		t_SD_BAS_BanqueSpectrale * BS,
		t_SD_BAS_DataThread *ctx
			);

#ifdef __cplusplus
}
#endif

#endif

