// $Header $
//@ --------------------------------------------------------------------
//@ Composant : SD_BAS
//@ MODULE : SD_BAS_CompteRendu
//@
//@ Commentaire :
//@    Le module SD_BAS_CompteRendu gere les comptes rendus 
//@    d'execution et les messages emis par les procedures C
//@    dans l'environnement du CGS.
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 17/12/2002 : PBR :
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//*====================================================================*/
//* Local Constant Declaration
//*====================================================================*/


//*====================================================================*/
//* Local types Declaration
//*====================================================================*/
//
// Fichiers include
//
#include <string.h>
#include "JDB_JOURNAL_DE_BORD.h"
#include "SD_BAS_CompteRendu.h"

//======================================================================
//                  FONCTIONS EXTERNES
//======================================================================

//@ ---------------------------------------------------------------*/
//@ Procedure : InitialiserCompteRendu 
//@
//@ Description : Cette procedure fait appel aux procedures:
//@                 - InitialiserListeMessages
//@                 - InitialiserListeErreurs
//@
//@ (NE DOIT ETRE UTILISEE QUE PAR LA FONCTION C++ QUI DECLARE
//@  LA VARIABLE SD_BAS_DataThread)
//@
//@ Parametres :
//@               (ENTREE/SORTIE) compteRendu : Compte rendu
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ---------------------------------------------------------------*/
void InitialiserCompteRendu (t_SD_BAS_CompteRendu* compteRendu)
{
  InitialiserListeMessages(&(compteRendu->msgJDB));
  InitialiserListeErreurs(&(compteRendu->errinterne));
}

//@ ---------------------------------------------------------------*/
//@ Procedure : InitialiserListeMessages
//@
//@ Description : Cette procedure vide les structures de stockage
//@               des messages 
//@    
//@ (NE DOIT ETRE UTILISEE QUE PAR LA FONCTION C++ QUI DECLARE
//@  LA VARIABLE SD_BAS_DataThread)
//@
//@ Parametres :
//@               (ENTREE/SORTIE) liste_messages : la liste des messages
//@
//@ Valeur de retour :
//@               aucun
//@
//@ ---------------------------------------------------------------*/
void InitialiserListeMessages (t_SD_BAS_ListeMessage * liste_messages)
{
  liste_messages->nb = 0;
}


//@ ---------------------------------------------------------------*/
//@ Procedure : InitialiserListeErreurs 
//@
//@ Description : Cette procedure vide les structures de stockage
//@               des messages 
//@    
//@ (NE DOIT ETRE UTILISEE QUE PAR LA FONCTION C++ QUI DECLARE
//@  LA VARIABLE SD_BAS_DataThread)
//@
//@ Parametres :
//@               (ENTREE/SORTIE) liste_erreurs : la liste des erreurs
//@
//@ Valeur de retour :
//@               aucun
//@
//@ ---------------------------------------------------------------*/
void InitialiserListeErreurs (t_SD_BAS_ERREUR * liste_erreurs)

{
  int i;

  liste_erreurs->nb = 0;
  for (i=0 ; i < SD_BAS_NB_MAX_MSG ; i++)
  {
    liste_erreurs->numligne[i] = 0;
    sprintf (liste_erreurs->nomfonction[i], "");
    sprintf (liste_erreurs->libelle[i], "");
  }
}


//@ ---------------------------------------------------------------*/
//@ Procedure : CreerMessage
//@
//@ Description : Cette procedure initialise les caracteristique du
//@               message a stocker. 
//@
//@ Parametres :
//@               (ENTREE) set : set de stockage du message
//@               (ENTREE) num : numero dans le set de stockage du message
//@               (ENTREE/SORTIE) liste : la liste des messages
//@
//@ Valeur de retour :
//@               aucun
//@
//@ ---------------------------------------------------------------*/
void CreerMessage (int set, 
                    int num,
                    t_SD_BAS_ListeMessage* liste)
{
  // Rajouter un message dans la liste
  if ( liste->nb < SD_BAS_NB_MAX_MSG )
  {
    liste->nb++;

    // Mettre a jour son numero
    liste->msg [(liste->nb)-1].set = set;
    liste->msg [(liste->nb)-1].num = num;

    // Initialiser les parametres du message
    (void) JDB_InitialiserMessage(&(liste->msg [(liste->nb)-1].param));
  }
}

//@ ---------------------------------------------------------------*/
//@ Procedure : StockerParametreMessage
//@
//@ Description : Cette procedure stocke les parametres du message.
//@
//@ Parametres :
//@               (ENTREE) set : set de stockage du message
//@               (ENTREE) num : numero dans le set de stockage du message
//@               (ENTREE/SORTIE) liste : la liste des messages
//@
//@ Valeur de retour :
//@               aucun
//@
//@ ---------------------------------------------------------------*/
e_JDB_Status StockerParametreMessage (char* nomParametre,
	                                char* valeurParametre,
                                         t_SD_BAS_ListeMessage* liste)

{

  //@ Ajouter le parametre dans la liste des parametres du message
  e_JDB_Status cr;

  cr = JDB_StockeParametre (nomParametre,
                            valeurParametre,
                            &(liste->msg [(liste->nb)-1].param));

  return (cr);
}

//@ ---------------------------------------------------------------*/
//@ Fonction : NbreMessage
//@
//@ Description : Cette fonction retourne le nombre de messages stocke.
//@
//@ Parametres :
//@               (ENTREE) liste : la liste des messages
//@
//@ Valeur de retour :
//@               aucun
//@
//@ ---------------------------------------------------------------*/
int NbreMessage (t_SD_BAS_ListeMessage* liste)

{
  return (liste->nb);
}


//@ ---------------------------------------------------------------*/
//@ Fonction : NbreErreurs
//@
//@ Description : Cette fonction retourne le nombre d'erreurs stockees.
//@
//@ Parametres :
//@               (ENTREE) liste : la liste des erreurs
//@
//@ Valeur de retour :
//@               aucun
//@
//@ ---------------------------------------------------------------*/
int NbreErreurs (t_SD_BAS_ERREUR* liste)

{
  return (liste->nb);
}


//@ ---------------------------------------------------------------*/
//@ Procedure : EnvoyerMessagesStockes
//@
//@ Description : Cette procedure envoie les messages stockes
//@               
//@    
//@ (NE DOIT ETRE UTILISEE QUE PAR UNE FONCTION C++ QUI DECLARE
//@  LA VARIABLE SD_BAS_DataThread)
//@
//@ Parametres :
//@               (ENTREE/SORTIE) liste_messages : la liste des messages
//@
//@ Valeur de retour :
//@               aucun
//@
//@ ---------------------------------------------------------------*/
void EnvoyerMessagesStockes (
  t_SD_BAS_ListeMessage*  liste_messages)
{
  short int set = 0;
  short int num = 0;
  int       i   = 0;

  // Pour tous les messages stockes
  for(i=0; i<NbreMessage (liste_messages); i++)
    {
      set = (short int) liste_messages->msg[i].set;
      num = (short int) liste_messages->msg[i].num;

      // Envoyer Message
      (void) JDB_EcrireMessage(set, 
                               num, 
                               &(liste_messages->msg[i].param));
      (void) JDB_VideMessage (&(liste_messages->msg[i].param));
    }

  InitialiserListeMessages (liste_messages);
}


//@ ---------------------------------------------------------------*/
//@ Procedure : EnvoyerErreursStockees
//@
//@ Description : Cette procedure envoie les erreurs stockees
//@               
//@    
//@ (NE DOIT ETRE UTILISEE QUE PAR UNE FONCTION C++ QUI DECLARE
//@  LA VARIABLE SD_BAS_DataThread)
//@
//@ Parametres :
//@               (ENTREE/SORTIE) liste_erreurs : la liste des erreurs
//@
//@ Valeur de retour :
//@               aucun
//@
//@ ---------------------------------------------------------------*/
void EnvoyerErreursStockees (
  t_SD_BAS_ERREUR*  liste_erreurs)
{
  int i;

  // Pour tous les messages stockes
  for(i=0; i<liste_erreurs->nb; i++)
    {
      // Envoyer Message
      fprintf(stderr, "**fonction %s ligne %d \n \tAnomalie : %s\n", 
        liste_erreurs->nomfonction[i], 
        liste_erreurs->numligne[i], 
        liste_erreurs->libelle[i]);
    }

  InitialiserListeErreurs (liste_erreurs);
}


//@ ---------------------------------------------------------------*/
//@ Procedure : EcrireErreursStockees
//@
//@ Description : Cette procedure envoie les erreurs stockees dans un fichier
//@               
//@    
//@
//@ Parametres :
//@               (ENTREE)        fp : descripteur du fichier
//@               (ENTREE/SORTIE) liste_erreurs : la liste des erreurs
//@
//@ Valeur de retour :
//@               aucun
//@
//@ ---------------------------------------------------------------*/
void EcrireErreursStockees (
  FILE *            fp, 
  t_SD_BAS_ERREUR*  liste_erreurs)
{
  int i;

  // Pour tous les messages stockes
  for(i=0; i<liste_erreurs->nb; i++)
    {
      // Envoyer Message
      fprintf(fp, "**fonction %s ligne %d \n \tAnomalie : %s\n", 
        liste_erreurs->nomfonction[i], 
        liste_erreurs->numligne[i], 
        liste_erreurs->libelle[i]);
    }

  InitialiserListeErreurs (liste_erreurs);
}


//@ ---------------------------------------------------------------*/
//@ Procedure : RecupererDernierMessageStocke
//@
//@ Description : Cette procedure renvoie le dernier message Stocke
//@               
//@    
//@ (NE DOIT ETRE UTILISEE QUE PAR UNE FONCTION C++ QUI DECLARE
//@  LA VARIABLE SD_BAS_DataThread)
//@ (DOIT ETRE APPELEE AVANT EnvoyerMessagesStockes())
//@
//@ Parametres :
//@               (ENTREE) liste : la liste des messages
//@               (SORTIE) message : intitule du dernier message
//@
//@ Valeur de retour :
//@               aucun
//@
//@ ---------------------------------------------------------------*/
void RecupererDernierMessageStocke(t_SD_BAS_ListeMessage* liste, 
                                   t_ChaineFixe message)
{
  int indiceDernierMessage = NbreMessage(liste) - 1;
  int tailleMessage        = sizeof(t_ChaineFixe);

#ifdef DEBUG
  int j = 0;

  printf("RecupererDernierMessageStocke: nb msg: %d (dernier=%d)\n", 
         (liste->msg[indiceDernierMessage].param.nb), 
         indiceDernierMessage);

  for (j=0; j<(liste->msg[indiceDernierMessage].param.nb); j++)
    { 
      printf("RecupererDernierMessageStocke: nom   : %s\n", 
             liste->msg[indiceDernierMessage].param.tab[j].nom);
      printf("RecupererDernierMessageStocke: valeur: %s\n",
             liste->msg[indiceDernierMessage].param.tab[j].valeur);
    }
#endif

  strncpy(message, 
          liste->msg[indiceDernierMessage].param.tab[0].valeur, 
          tailleMessage);
  message[tailleMessage-1] = '\0';
}


//@ ---------------------------------------------------------------*/
//@ Procedure : RecupererPremiereErreurStockee
//@
//@ Description : Cette procedure renvoie la premiere erreur stockee
//@               
//@    
//@ (NE DOIT ETRE UTILISEE QUE PAR UNE FONCTION C++ QUI DECLARE
//@  LA VARIABLE SD_BAS_DataThread)
//@ (DOIT ETRE APPELEE AVANT EnvoyerMessagesStockes())
//@
//@ Parametres :
//@               (ENTREE) liste_erreurs : la liste des erreurs
//@               (SORTIE) nom_fonction : nom de la procedure ou a eu lieu 
//@                        la premiere erreur
//@               (SORTIE) num_ligne : numero de ligne dans le code
//@               (SORTIE) message : libelle de l'erreur
//@
//@ Valeur de retour :
//@               aucun
//@
//@ ---------------------------------------------------------------*/
void RecupererPremiereErreurStockee(
  t_SD_BAS_ERREUR*  liste_erreurs, 
  t_ChaineFixe      nom_fonction, 
  int*              num_ligne, 
  t_ChaineFixe      message)

{
  int IndicePremiereErreur = 0;
  
  sprintf (nom_fonction, "%s", liste_erreurs->nomfonction[IndicePremiereErreur]);
  *num_ligne = liste_erreurs->numligne[IndicePremiereErreur];
  sprintf (message, liste_erreurs->libelle[IndicePremiereErreur]);
}


//@ ---------------------------------------------------------------*/
//@ Procedure : LibererCompteRendu 
//@
//@ Description : Cette procedure purge les parametres stockes
//@               
//@ Parametres :
//@               (ENTREE/SORTIE) liste : la liste des messages
//@
//@ Valeur de retour :
//@               aucun
//@
//@ ---------------------------------------------------------------*/
void LibererCompteRendu (
  t_SD_BAS_ListeMessage * liste_messages)
{
  int i;

  // Initialiser la liste des messages
  for (i=0; i<liste_messages->nb; i++)
  {
    (void) JDB_VideMessage(&(liste_messages->msg[i].param));
  }
}

