// $Header $
//@ --------------------------------------------------------------------
//@ Composant : SD_BAS
//@ MODULE : SD_BAS_CompteRendu
//@
//@ Commentaire :
//@    Le module SD_BAS_CompteRendu gere les comptes rendus 
//@    d'execution et les messages emis par les procedures C
//@    dans l'environnement du CGS.
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 17/12/2002 : PBR :
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-144-CN : 21/09/2004 : B. MASSART
//@ Problemes lors du portage d'OPS vers Solaris par Meteo France
//@ Voir IA-2100-FC-144-THA.
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_BAS_CompteRendu_h
#define SD_BAS_CompteRendu_h

//
// Fichiers include
//
#include "CMN__Commun.h"
#include "JDB_JOURNAL_DE_BORD.h"
#include "SD__Message.h"

//
// Definition des constantes 
//

#define SD_BAS_NB_MAX_MSG     20      /* Nbre max de messages stockes dans une liste de msg JDB*/
#define SD_BAS_LGNOMPROCEDURE 60      /* longueur max des noms de fonctions */
#define SD_BAS_LGLIBERR       255     /* longueur des libelles erreur */

//==============================================================
// Declaration des MACRO
//==============================================================

// --------------------------------------------------------------
// Macro :  SD_BAS_PRINT_ERR_INT
//
// Description : Cette macro n'est utilisable qu'en mode debug :
//               elle affiche sur la sortie standard un message
//               construit a l'aide des parametres d'entrees
// Parametres :
//               (ENTREE) nomfonction 
//               (ENTREE) numligne 
//               (ENTREE) libelle
// Prerequis :
//               aucun
// --------------------------------------------------------------

#ifdef OPS_DEBUG
#define SD_BAS_PRINT_ERR_INT(nomfonction, numligne, libelle)\
 {\
   printf("**fonction %s ligne %d \n \tErreur interne : %s\n", nomfonction, numligne, libelle);\
 }
#endif


// --------------------------------------------------------------
// Macro :  SD_BAS_TEST_ERR_INT
//
// Description : Cette macro permet de traiter une anomalie :
//               stockage des parametres de l'anomalie
//               et construction d'un message JdB a emettre
// Parametres :
//               (t_SD_BAS_ERREUR) errinterne 
//               (int/int) set/num : numero du message Jdb a construire
//                          parametres non utilises par la macro
//               (char *) fct : nom de la procedure a l'origine de l'anomalie
//               (char *) lig : numero de ligne dans la procedure
//               (char *) msg : chaine de caracteres
// Prerequis :
//               une variable de type (t_SD_BAS_ORIGINE)et de nom ctx 
//               doit etre defini par la procedure utilisant la macro
// --------------------------------------------------------------

/* FA144 */
#define SD_BAS_TEST_ERR_INT(errinterne, set, num, fct, lig, msg)\
{\
  if (errinterne.nb < SD_BAS_NB_MAX_MSG) \
    { \
      CreerMessage (SD_SET, 1, &(ctx->cr.msgJDB));\
      StockerParametreMessage ("$LIBELLE", msg, &(ctx->cr.msgJDB));\
      strncpy(errinterne.nomfonction[errinterne.nb], fct,SD_BAS_LGNOMPROCEDURE);\
      errinterne.nomfonction[errinterne.nb][SD_BAS_LGNOMPROCEDURE - 1] = '\0';\
      errinterne.numligne[errinterne.nb] = lig;\
      strncpy(errinterne.libelle[errinterne.nb], msg,SD_BAS_LGLIBERR);\
      errinterne.libelle[errinterne.nb][SD_BAS_LGLIBERR -1 ] = '\0';\
      errinterne.nb++;\
      fprintf(stderr,\
              "**fonction %s ligne %d \n \tAnomalie : %s\n", \
              fct, lig, msg);\
    } \
}

// --------------------------------------------------------------
// Macro :  SD_BAS_THROW
//
// Description : Cette macro encapsule le traitement d'un compte rendu.
//               Puis se debranche sur l'etiquette fin.
//               Renvoi de "code_cr" si la valeur de "cr_recu" EQ/NEQ a 
//               une valeur de cr "test_cr".
// Parametres :
//               (int) cr_op : compte rendu de la procedure
//               (t_CMN__Statut/int) code_cr : code cr a renvoyer par la procedure
//               (int) cr_recu : valeur de cr recu
//               (e_BAS_TestCR) test
//               (int) test_cr : valeur de cr a testerrecu
// Prerequis :
//               [ctx] une variable de type (t_SD_BAS_DataThread)et de nom ctx 
//                     doit etre defini par la procedure utilisant la macro
//               [tcProcName_F] : nom de la procedure courante
// --------------------------------------------------------------

/* FA144 */
#define SD_BAS_THROW(cr_op, code_cr, cr_recu, test, test_cr, msg) \
  if ( ((test == BAS_CR_EQ)  && (cr_recu == test_cr)) || \
       ((test == BAS_CR_NEQ) && (cr_recu != test_cr)))   \
  { \
    if (ctx->cr.errinterne.nb < SD_BAS_NB_MAX_MSG) \
    { \
       cr_op = code_cr;\
       ctx->cr.errinterne.numligne[ctx->cr.errinterne.nb] = __LINE__;\
       strncpy(ctx->cr.errinterne.nomfonction[ctx->cr.errinterne.nb], tcProcName_F, SD_BAS_LGNOMPROCEDURE);\
       ctx->cr.errinterne.nomfonction[ctx->cr.errinterne.nb][SD_BAS_LGNOMPROCEDURE-1] = '\0';\
       strncpy(ctx->cr.errinterne.libelle[ctx->cr.errinterne.nb], msg, SD_BAS_LGLIBERR);\
       ctx->cr.errinterne.libelle[ctx->cr.errinterne.nb][SD_BAS_LGLIBERR-1] = '\0';\
       ctx->cr.errinterne.nb++;\
    }\
    goto fin; \
  }


// --------------------------------------------------------------
// Macro :  SD_BAS_CATCH
//
// Description : Cette macro encapsule l'etiquette fin.

// Parametres :
//               aucun
// Prerequis :
//               un goto fin ou l'utilisation de SD_BAS_THROW
// --------------------------------------------------------------
#define SD_BAS_CATCH fin :


//==============================================================
// Declaration des types externes
//==============================================================


//--------------------------------------------------------------------*/
// type definissant les informations utiles pour les messages  d'erreur
// algo a utiliser dans les modules de haut niveau
//--------------------------------------------------------------------*/
typedef struct
{
  int nb; /* nombre d'elements ecrits dans numligne et nomfonction */
  char libelle[SD_BAS_NB_MAX_MSG][SD_BAS_LGLIBERR]; /* libelle de l'erreur de bas niveau */
  int numligne[SD_BAS_NB_MAX_MSG]; /* numero de la ligne du fichier ou l'erreur est emise */
  char nomfonction[SD_BAS_NB_MAX_MSG][SD_BAS_LGNOMPROCEDURE]; /* nom de la fonction ou l'erreur est emise */
} t_SD_BAS_ERREUR;


//--------------------------------------------------------------------*/
// Type definissant un message JDB par son set, son numero dans le set
// et la liste de ses parametres
//--------------------------------------------------------------------*/
typedef struct {
    int               set;
    int               num;
    t_JDB_ListeParam  param;
} t_SD_BAS_ParamMessage;

//--------------------------------------------------------------------*/
// Type definissant la liste des messages JDB
//--------------------------------------------------------------------*/
typedef struct {
    int      	                nb;
    t_SD_BAS_ParamMessage 	msg [SD_BAS_NB_MAX_MSG];
} t_SD_BAS_ListeMessage;


//--------------------------------------------------------------------*/
// type definissant un compte rendu
//--------------------------------------------------------------------*/
typedef struct
    {
    t_SD_BAS_ERREUR       errinterne;    // numero de l'erreur interne a emettre
    t_SD_BAS_ListeMessage msgJDB;        // Liste des messages a emettre
    } t_SD_BAS_CompteRendu;

//--------------------------------------------------------------------*/
// Enumere de test EQ (equal), NEQ ( not Equal)
//--------------------------------------------------------------------*/
typedef enum
    {
    BAS_CR_EQ  = 1,
    BAS_CR_NEQ = 2
    } e_BAS_TestCR;

//===================================================================
// Declaration des fonctions externes
//===================================================================
#ifdef __cplusplus
extern "C" {
#endif
//@ ---------------------------------------------------------------*/
//@ Procedure : InitialiserCompteRendu 
//@
//@ Description : Cette procedure fait appel aux procedures:
//@                 - InitialiserListeMessages
//@                 - InitialiserListeErreurs
//@
//@ Parametres :
//@               (ENTREE/SORTIE) compteRendu : Compte rendu
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ---------------------------------------------------------------*/
void InitialiserCompteRendu (t_SD_BAS_CompteRendu* compteRendu);

//@ ---------------------------------------------------------------*/
//@ Procedure : InitialiserListeMessage
//@
//@ Description : Cette procedure vide les structures de stockage
//@               des messages. 
//@
//@ Parametres :
//@               (ENTREE/SORTIE) liste : la liste des messages
//@
//@ Valeur de retour :
//@               aucun
//@
//@ ---------------------------------------------------------------*/
void InitialiserListeMessages (t_SD_BAS_ListeMessage * liste_messages);
 

//@ ---------------------------------------------------------------*/
//@ Procedure : InitialiserListeErreurs 
//@
//@ Description : Cette procedure vide les structures de stockage
//@               des messages 
//@    
//@ (NE DOIT ETRE UTILISEE QUE PAR LA FONCTION C++ QUI DECLARE
//@  LA VARIABLE SD_BAS_DataThread)
//@
//@ Parametres :
//@               (ENTREE/SORTIE) liste_erreurs : la liste des erreurs
//@
//@ Valeur de retour :
//@               aucun
//@
//@ ---------------------------------------------------------------*/
void InitialiserListeErreurs (t_SD_BAS_ERREUR * liste_erreurs);

//@ ---------------------------------------------------------------*/
//@ Procedure : LibererCompteRendu 
//@
//@ Description : Cette procedure purge les parametres stockes
//@               
//@ Parametres :
//@               (ENTREE/SORTIE) liste : la liste des messages
//@
//@ Valeur de retour :
//@               aucun
//@
//@ ---------------------------------------------------------------*/
void LibererCompteRendu (t_SD_BAS_ListeMessage* liste);

//@ ---------------------------------------------------------------*/
//@ Procedure : CreerMessage
//@
//@ Description : Cette procedure initialise les caracteristique du
//@               message a stocker. 
//@
//@ Parametres :
//@               (ENTREE) set : set de stockage du message
//@               (ENTREE) num : numero dans le set de stockage du message
//@               (ENTREE/SORTIE) liste : la liste des messages
//@
//@ Valeur de retour :
//@               aucun
//@
//@ ---------------------------------------------------------------*/
void CreerMessage (int set, 
                   int num,
                   t_SD_BAS_ListeMessage* liste);

//@ ---------------------------------------------------------------*/
//@ Procedure : StockerParametreMessage
//@
//@ Description : Cette procedure stocke les parametres du message.
//@
//@ Parametres :
//@               (ENTREE) set : set de stockage du message
//@               (ENTREE) num : numero dans le set de stockage du message
//@               (ENTREE/SORTIE) liste : la liste des messages
//@
//@ Valeur de retour :
//@               e_JDB_Status = compte rendu d'execution
//@
//@ ---------------------------------------------------------------*/
e_JDB_Status StockerParametreMessage (char* nomParametre,
	                                char* valeurParametre,
                                        t_SD_BAS_ListeMessage* liste);

//@ ---------------------------------------------------------------*/
//@ Fonction : NbreMessage
//@
//@ Description : Cette fonction retourne le nombre de messages stocke.
//@
//@ Parametres :
//@               (ENTREE) liste : la liste des messages
//@
//@ Valeur de retour :
//@               aucun
//@
//@ ---------------------------------------------------------------*/
int NbreMessage (t_SD_BAS_ListeMessage* liste);

//@ ---------------------------------------------------------------*/
//@ Fonction : NbreErreurs
//@
//@ Description : Cette fonction retourne le nombre d'erreurs stocke.
//@
//@ Parametres :
//@               (ENTREE) liste : la liste des messages
//@
//@ Valeur de retour :
//@               aucun
//@
//@ ---------------------------------------------------------------*/
int NbreErreurs (t_SD_BAS_ERREUR* liste);

//@ ---------------------------------------------------------------*/
//@ Procedure : EnvoyerMessagesStocker 
//@
//@ Description : Cette procedure envoie les messages Stockees
//@               
//@    
//@ (NE DOIT ETRE UTILISEE QUE PAR UNE FONCTION C++ QUI DECLARE
//@  LA VARIABLE SD_BAS_DataThread)
//@
//@ Parametres :
//@               (ENTREE/SORTIE) liste : la liste des messages
//@
//@ Valeur de retour :
//@               aucun
//@
//@ ---------------------------------------------------------------*/
void EnvoyerMessagesStockes (t_SD_BAS_ListeMessage* liste);

//@ ---------------------------------------------------------------*/
//@ Procedure : EnvoyerErreursStockees
//@
//@ Description : Cette procedure envoie les erreurs stockees
//@               
//@    
//@ (NE DOIT ETRE UTILISEE QUE PAR UNE FONCTION C++ QUI DECLARE
//@  LA VARIABLE SD_BAS_DataThread)
//@
//@ Parametres :
//@               (ENTREE/SORTIE) liste_erreurs : la liste des erreurs
//@
//@ Valeur de retour :
//@               aucun
//@
//@ ---------------------------------------------------------------*/
void EnvoyerErreursStockees (t_SD_BAS_ERREUR*  liste_erreurs);


//@ ---------------------------------------------------------------*/
//@ Procedure : EcrireErreursStockees
//@
//@ Description : Cette procedure envoie les erreurs stockees dans un fichier
//@               
//@    
//@
//@ Parametres :
//@               (ENTREE)        fp : descripteur du fichier
//@               (ENTREE/SORTIE) liste_erreurs : la liste des erreurs
//@
//@ Valeur de retour :
//@               aucun
//@
//@ ---------------------------------------------------------------*/
void EcrireErreursStockees (
  FILE *            fp, 
  t_SD_BAS_ERREUR*  liste_erreurs)
;

//@ ---------------------------------------------------------------*/
//@ Procedure : RecupererDernierMessageStocke
//@
//@ Description : Cette procedure renvoie le dernier message Stocke
//@               
//@    
//@ (NE DOIT ETRE UTILISEE QUE PAR UNE FONCTION C++ QUI DECLARE
//@  LA VARIABLE SD_BAS_DataThread)
//@
//@ Parametres :
//@               (ENTREE) liste : la liste des messages
//@               (SORTIE) message : intitule du dernier message
//@
//@ Valeur de retour :
//@               aucun
//@
//@ ---------------------------------------------------------------*/
void RecupererDernierMessageStocke(t_SD_BAS_ListeMessage* liste,
                                   t_ChaineFixe message);

#ifdef __cplusplus
}
#endif

#endif
