//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/BAS/src/SD_BAS_ConfAutre.c,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_BAS
//@ MODULE : SD_BAS_ConfAutre
//@
//@ Comment :
//@    Code file for the  SD_BAS_ConfAutre class which encapsulates the
//@    acces to the context file   :
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 31/01/2003 : [reference :] ott
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2520(DM-2100-OPS) : 19/12/2008 : S. BRUNEL
//@ Rendre les variables WnShiftMin et WnShiftMax dependante du numero de 
//@ pixel (Day 2,point 1)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2527(DM-2100-OPS) : 19/12/2008 : S. BRUNEL
//@ Permetrre le calcul des spectres dans les bandes B1 & B2 meme lorsque un
//@ spike affecte la bande B3 (Day 2,point 8)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------


#include <stdio.h>
#include "SD_BAS_ConfAutre.h"

// FT-2765
#include "SD_BAS_SwapEndian.h"
// ...FT-2765

/*====================================================================*/
// Local Constant Declaration 
/*====================================================================*/

/* macro de detection d'erreur sur les fread() */
#define SD_BAS_CA_ERR_FREAD(ret, nb_octets)\
{\
   if ((ret) != (nb_octets))\
   {\
     cr_op   = SD_BAS_ERR_BAS_NIVEAU;\
     msg_err = SD_BAS_MSG_READ;\
     num_lig = __LINE__;\
     goto fin;\
   }\
}

// FT-2765
static const int SD_BAS_SIZE_INT4 = 4;
static const int SD_BAS_SIZE_REAL64 = 8;

static char * SD_BAS_MSG_VERSION_CA = "INVALID OTHER CONFIGURATION FILE VERSION";
// ...FT-2765

/*====================================================================*/
// Local types Declaration  
/*====================================================================*/
//======================================================================
// Internal fonctions Declaration
//======================================================================
//======================================================================
// Global variables Definition
//======================================================================


//======================================================================
//					FONCTIONS EXTERNES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_ChargerConfAutre
//@
//@ Description : ouverture lecture et fermeture du fichier de configuration  
//@ autre indique en argument
//@
//@
//@ Parameters :
//@			(ENTREE)  Fichier : nom du fichier de configuration 
//@			(ENTREE/SORTIE)  Conf : configuration lu 
//@  			(ENTREE)  ctx : contexte de l'etape                   
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@		        SD_BAS_ERR_OUVERTURE si on ne peut pas ouvrir le fichier
//@                     SD_BAS_ERR_FERMETURE si on ne peut pas fermer le fichier
//@                     SD_BAS_ERR_LECTURE   si on ne peut lire
//@                     SD_BAS_VERSION si la longueur du fichier ou le numero
//@                     d'enregistrement son different de ceux predefinis
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_ChargerConfAutre(
		char * Fichier,
		t_SD_BAS_ConfAutre * Conf,
		t_SD_BAS_DataThread *ctx
			)
{
  static const char tcProcName_F[] = "SD_BAS_ChargerConfAutre";
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  int ret = 0;
  size_t nbElement = 0;
  int num_lig = 0;
  char *msg_err = NULL;
  int buff = 0;
  FILE * ConfAutre = NULL;
  

  //@ ouverture du fichier

  ConfAutre = fopen ( Fichier , "rb");

  if( ConfAutre == NULL)
    {
      cr_op   = SD_BAS_ERR_BAS_NIVEAU;
      msg_err = SD_BAS_MSG_OPEN;
      num_lig = __LINE__;
      goto fin;
    }


  //@ Lecture des donnees

  /* lecture  IDefIssueIcd  */
  nbElement = fread(&Conf->IDefIssueIcd, SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefIssueIcd,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture   IDefRevisionIcd */
  nbElement = fread(&Conf->IDefRevisionIcd, SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefRevisionIcd,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture IDefPTSI   */
  nbElement = fread(&Conf->IDefPTSI, SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefPTSI,SD_BAS_SIZE_INT4,1);
  // ...FT-2765
 
  /* lecture IDefConf   */
  nbElement = fread(&Conf->IDefConf, SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefConf,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture IDefSdbID   */
  nbElement = fread(&Conf->IDefSdbID,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefSdbID,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture IDefStableparamID   */
  nbElement = fread(&Conf->IDefStableparamID,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefStableparamID,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture IDefParamDate.day   */
  nbElement = fread(&buff, SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&buff,SD_BAS_SIZE_INT4,1);
  // ...FT-2765
  Conf->IDefParamDate.day = buff;

  /* lecture IDefParamDate.millis   */
  nbElement = fread(&buff,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&buff,SD_BAS_SIZE_INT4,1);
  // ...FT-2765
  Conf->IDefParamDate.millis = buff;

  /* lecture  IDefMETOPNumber  */
  nbElement = fread(&Conf->IDefMETOPNumber, SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefMETOPNumber,SD_BAS_SIZE_INT4,1);
  // ...FT-2765
 
  /* lecture  ICtcNbSeg  */
  // jlp
  nbElement = fread(Conf->ICtcNbSeg ,SD_BAS_SIZE_INT4, SD_BAS_CTN, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_CTN);
	// FT-2765
  SWAP_INT_BUFFER(Conf->ICtcNbSeg,SD_BAS_SIZE_INT4,SD_BAS_CTN);
  // ...FT-2765

  /* lecture  IDefNSegNsfirst  */
  nbElement = fread(&Conf->IDefNSegNsfirst, SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefNSegNsfirst,SD_BAS_SIZE_INT4,1);
  // ...FT-2765
 
  /* lecture  ICtcTabNbBit  */
  // jlp
  nbElement = fread(Conf->ICtcTabNbBit ,SD_BAS_SIZE_INT4, SD_BAS_SPCT_COD * SD_BAS_CTN, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_SPCT_COD * SD_BAS_CTN);
	// FT-2765
  SWAP_INT_BUFFER(Conf->ICtcTabNbBit,SD_BAS_SIZE_INT4,SD_BAS_SPCT_COD*SD_BAS_CTN);
  // ...FT-2765
  
  /* lecture  ICtcTabOffset  */
  // jlp
  nbElement = fread(Conf->ICtcTabOffset ,SD_BAS_SIZE_INT4, SD_BAS_SPCT_COD * SD_BAS_CTN, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_SPCT_COD * SD_BAS_CTN);
	// FT-2765
  SWAP_INT_BUFFER(Conf->ICtcTabOffset,SD_BAS_SIZE_INT4,SD_BAS_SPCT_COD*SD_BAS_CTN);
  // ...FT-2765

  /* lecture  ICtcTabOffsetC  */
  // jlp
  nbElement = fread(Conf->ICtcTabOffsetC ,SD_BAS_SIZE_REAL64,  SD_BAS_CTN, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_CTN);
	// FT-2765
  SWAP_INT_BUFFER(Conf->ICtcTabOffsetC,SD_BAS_SIZE_REAL64,SD_BAS_CTN);
  // ...FT-2765

  /* lecture  ICtcTabOffsetM  */
  // jlp
  nbElement = fread(Conf->ICtcTabOffsetM ,SD_BAS_SIZE_REAL64,  SD_BAS_CTN, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_CTN);
	// FT-2765
  SWAP_INT_BUFFER(Conf->ICtcTabOffsetM,SD_BAS_SIZE_REAL64,SD_BAS_CTN);
  // ...FT-2765

  /* lecture  ICtcTabScale  */
  nbElement = fread(Conf->ICtcTabScale ,SD_BAS_SIZE_INT4, SD_BAS_CTN * SD_BAS_SPCT_COD, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_CTN * SD_BAS_SPCT_COD);
	// FT-2765
  SWAP_INT_BUFFER(Conf->ICtcTabScale,SD_BAS_SIZE_INT4,SD_BAS_CTN*SD_BAS_SPCT_COD);
  // ...FT-2765
     
  /* lecture  ICtcTabScaleC */
  nbElement = fread(Conf->ICtcTabScaleC ,SD_BAS_SIZE_REAL64,  SD_BAS_CTN, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_CTN);
	// FT-2765
  SWAP_INT_BUFFER(Conf->ICtcTabScaleC,SD_BAS_SIZE_REAL64,SD_BAS_CTN);
  // ...FT-2765

  /* lecture  ICtcTabScaleM  */
  nbElement = fread(Conf->ICtcTabScaleM ,SD_BAS_SIZE_REAL64,  SD_BAS_CTN, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_CTN);
	// FT-2765
  SWAP_INT_BUFFER(Conf->ICtcTabScaleM,SD_BAS_SIZE_REAL64,SD_BAS_CTN);
  // ...FT-2765
 
  /* lecture  IDefArcNbNsSeg  */
  nbElement = fread(Conf->IDefArcNbNsSeg  ,SD_BAS_SIZE_INT4,  SD_BAS_SB, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_SB);
	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefArcNbNsSeg,SD_BAS_SIZE_INT4,SD_BAS_SB);
  // ...FT-2765

  /* lecture  IDefArcNbSeg  */
  nbElement = fread(Conf->IDefArcNbSeg  ,SD_BAS_SIZE_INT4,  SD_BAS_SB, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_SB);
	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefArcNbSeg,SD_BAS_SIZE_INT4,SD_BAS_SB);
  // ...FT-2765

  /* lecture  IDefBBEmissivity  */
  nbElement = fread(Conf->IDefBBEmissivity  ,SD_BAS_SIZE_REAL64,  SD_BAS_SS1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_SS1);
	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefBBEmissivity,SD_BAS_SIZE_REAL64,SD_BAS_SS1);
  // ...FT-2765

  /* lecture IDefBBNbRrs   */
  nbElement = fread(&Conf->IDefBBNbRrs  ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefBBNbRrs,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture  IDefBBRRs  */
  nbElement = fread(Conf->IDefBBRRs ,SD_BAS_SIZE_REAL64,  SD_BAS_NB_SURF_RAD, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_NB_SURF_RAD);
	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefBBRRs,SD_BAS_SIZE_REAL64,SD_BAS_NB_SURF_RAD);
  // ...FT-2765

  /* lecture  IDefCcsChannelId  */
  nbElement = fread(Conf->IDefCcsChannelId  ,SD_BAS_SIZE_INT4,  SD_BAS_NBK, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_NBK);
	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefCcsChannelId,SD_BAS_SIZE_INT4,SD_BAS_NBK);
  // ...FT-2765

  /* lecture   IDefCcsChannelNoise */
  nbElement = fread(Conf->IDefCcsChannelNoise  ,SD_BAS_SIZE_REAL64, SD_BAS_NBK, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_NBK);
	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefCcsChannelNoise,SD_BAS_SIZE_REAL64,SD_BAS_NBK);
  // ...FT-2765

  /* lecture  IDefCcsChannelWn  */
  nbElement = fread(Conf->IDefCcsChannelWn  ,SD_BAS_SIZE_REAL64, SD_BAS_NBK, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_NBK);
	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefCcsChannelWn,SD_BAS_SIZE_REAL64,SD_BAS_NBK);
  // ...FT-2765

  /* lecture  IDefCcsChannelWgt  */
  nbElement = fread(Conf->IDefCcsChannelWgt  ,SD_BAS_SIZE_INT4, SD_BAS_NBK * SD_BAS_CCD, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_NBK * SD_BAS_CCD);
	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefCcsChannelWgt,SD_BAS_SIZE_INT4,SD_BAS_NBK*SD_BAS_CCD);
  // ...FT-2765

  /* lecture   IDefCcsDimClassSpace */
  nbElement = fread(&Conf->IDefCcsDimClassSpace  ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefCcsDimClassSpace,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture  IDefCcsDistAgloInit  */
  nbElement = fread(&Conf->IDefCcsDistAgloInit  ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefCcsDistAgloInit,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture  IDefCcsDistMax  */
  nbElement = fread(&Conf->IDefCcsDistMax  ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefCcsDistMax,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture IDefCcsGlueCutoff   */
  nbElement = fread(&Conf->IDefCcsGlueCutoff ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefCcsGlueCutoff,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture IDefCcsImgCILinMax   */
  nbElement = fread(&Conf->IDefCcsImgCILinMax ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefCcsImgCILinMax,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture  IDefCcsImgCIColMax  */
  nbElement = fread(&Conf->IDefCcsImgCIColMax ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefCcsImgCIColMax,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture  IDefCcsIterConvCutoff  */
  nbElement = fread(&Conf->IDefCcsIterConvCutoff ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefCcsIterConvCutoff,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture IDefCcsMinIterNCCutoff   */
  nbElement = fread(&Conf->IDefCcsMinIterNCCutoff ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefCcsMinIterNCCutoff,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture  IDefCcsMode  */
  nbElement = fread(&Conf->IDefCcsMode ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefCcsMode,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture  IDefCcsNbClusMax  */
  nbElement = fread(&Conf->IDefCcsNbClusMax ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefCcsNbClusMax,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture IDefCcsNbIterMax   */
  nbElement = fread(&Conf->IDefCcsNbIterMax ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefCcsNbIterMax,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture  IDefCcsNbNonClassifMax  */
  nbElement = fread(&Conf->IDefCcsNbNonClassifMax ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefCcsNbNonClassifMax,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture  IDefCcsRadToTb  */
  nbElement = fread(&Conf->IDefCcsRadToTb ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefCcsRadToTb,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture  IDefCcsSortingChannel  */
  nbElement = fread(&Conf->IDefCcsSortingChannel ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefCcsSortingChannel,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture  IDefCcsSplitCutoff  */
  nbElement = fread(&Conf->IDefCcsSplitCutoff ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefCcsSplitCutoff,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture IDefCovarEigenValCalRad   */
  nbElement = fread(&Conf->IDefCovarEigenValCalRad ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefCovarEigenValCalRad,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture IDefCovarEigenValSpect   */
  nbElement = fread(&Conf->IDefCovarEigenValSpect ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefCovarEigenValSpect,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture  IDefCtcNbNsSeg  */
  nbElement = fread(&Conf->IDefCtcNbNsSeg ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefCtcNbNsSeg,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture  IDefCovarMatEigenVal1b  */
  // jlp
  nbElement = fread(Conf->IDefCovarMatEigenVal1b ,SD_BAS_SIZE_REAL64, SD_BAS_COVAR_DIM * SD_BAS_CCD, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_COVAR_DIM * SD_BAS_CCD);
	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefCovarMatEigenVal1b,SD_BAS_SIZE_REAL64,SD_BAS_COVAR_DIM*SD_BAS_CCD);
  // ...FT-2765

  /* lecture  IDefCovarMatEigenVal1c  */
  // jlp
  nbElement = fread(Conf->IDefCovarMatEigenVal1c ,SD_BAS_SIZE_REAL64, SD_BAS_COVAR_DIM * SD_BAS_CCD, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_COVAR_DIM * SD_BAS_CCD);
	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefCovarMatEigenVal1c,SD_BAS_SIZE_REAL64,SD_BAS_COVAR_DIM*SD_BAS_CCD);
  // ...FT-2765

  /* lecture  IDefDptGainCoeffCutoff  */
  nbElement = fread(&Conf->IDefDptGainCoeffCutoff ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefDptGainCoeffCutoff,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture  IDefDptIISInHomPixMax  */
  nbElement = fread(&Conf->IDefDptIISInHomPixMax ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefDptIISInHomPixMax,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture   IDefDptIISVarianceCutoff */
  nbElement = fread(&Conf->IDefDptIISVarianceCutoff ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefDptIISVarianceCutoff,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture  IDefDptNbCycleFilter  */
  nbElement = fread(&Conf->IDefDptNbCycleFilter ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefDptNbCycleFilter,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture  IDefDurFltIsrfemOff  */
  nbElement = fread(&Conf->IDefDurFltIsrfemOff ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefDurFltIsrfemOff,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture IDefDurBandMiss   */
  nbElement = fread(&Conf->IDefDurBandMiss ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefDurBandMiss,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture  IDefDurBBTMiss  */
  nbElement = fread(&Conf->IDefDurBBTMiss ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefDurBBTMiss,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765


  /* lecture IDefDurImgEWMiss   */
  nbElement = fread(&Conf->IDefDurImgEWMiss ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefDurImgEWMiss,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture  IDefDurImgBBMiss  */
  nbElement = fread(&Conf->IDefDurImgBBMiss ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefDurImgBBMiss,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture  IDefDurImgCSMiss  */
  nbElement = fread(&Conf->IDefDurImgCSMiss ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefDurImgCSMiss,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture  IDefDurRadAvhrrMiss  */
  nbElement = fread(&Conf->IDefDurRadAvhrrMiss ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefDurRadAvhrrMiss,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture  IDefFaxFLS  */
  nbElement = fread(&Conf->IDefFaxFLS ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefFaxFLS,SD_BAS_SIZE_INT4,1);
  // ...FT-2765
 
  /* lecture  IDefFaxRegressDegre  */
  nbElement = fread(&Conf->IDefFaxRegressDegre ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefFaxRegressDegre,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture IDefFaxRegressRmsCutoff   */
  nbElement = fread(&Conf->IDefFaxRegressRmsCutoff ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefFaxRegressRmsCutoff,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture   IDefFaxT0 */
  nbElement = fread(&Conf->IDefFaxT0 ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefFaxT0,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture IDefFtbFLS   */
  nbElement = fread(&Conf->IDefFtbFLS ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefFtbFLS,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture    */
  nbElement = fread(&Conf->IDefFtbRegressDegre ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefFtbRegressDegre,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture  IDefFtbRegressDegre  */
  nbElement = fread(&Conf->IDefFtbRegressRmsCutoff ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefFtbRegressRmsCutoff,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture  IDefFtbT0  */
  nbElement = fread(&Conf->IDefFtbT0 ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefFtbT0,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture  IDefGeoIISSubGridSize  */
  nbElement = fread(&Conf->IDefGeoIISSubGridSize ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefGeoIISSubGridSize,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture IDefGeoIISSubGridNY   */
  nbElement = fread(Conf->IDefGeoIISSubGridNY ,SD_BAS_SIZE_INT4, SD_BAS_SGI_LI , ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_SGI_LI);
	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefGeoIISSubGridNY,SD_BAS_SIZE_INT4,SD_BAS_SGI_LI);
  // ...FT-2765

  /* lecture  IDefGeoIISSubGridNZ  */
  nbElement = fread(Conf->IDefGeoIISSubGridNZ ,SD_BAS_SIZE_INT4, SD_BAS_SGI_CO , ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_SGI_CO);
	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefGeoIISSubGridNZ,SD_BAS_SIZE_INT4,SD_BAS_SGI_CO);
  // ...FT-2765

  /* lecture IDefGranuleSizeDefault   */
  nbElement = fread(&Conf->IDefGranuleSizeDefault ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefGranuleSizeDefault,SD_BAS_SIZE_INT4,1);
  // ...FT-2765
 
  /* lecture    */
  // jlp
  nbElement = fread(Conf->IDefHecTRrsModel ,SD_BAS_SIZE_REAL64, SD_BAS_NB_SURF_RAD*SD_BAS_MODEL , ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_NB_SURF_RAD*SD_BAS_MODEL);
	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefHecTRrsModel,SD_BAS_SIZE_REAL64,SD_BAS_NB_SURF_RAD*SD_BAS_MODEL);
  // ...FT-2765

  /* lecture  IDefHecTRrsModel  */
  nbElement = fread(&Conf->IDefIacCorrelCutoff ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefIacCorrelCutoff,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture    */
  nbElement = fread(&Conf->IDefIacDCol ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefIacDCol,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture  IDefIacDCol  */
  nbElement = fread(&Conf->IDefIacDLin ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefIacDLin,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture IDefIacPosMaxCutoff   */
  nbElement = fread(&Conf->IDefIacPosMaxCutoff ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefIacPosMaxCutoff,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture IDefIASIMETOPMatRot   */
  // jlp
  nbElement = fread(Conf->IDefIASIMETOPMatRot ,SD_BAS_SIZE_REAL64, SD_BAS_DIM_ROT_MATRICE * SD_BAS_DIM_ROT_MATRICE , ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_DIM_ROT_MATRICE * SD_BAS_DIM_ROT_MATRICE);
	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefIASIMETOPMatRot,SD_BAS_SIZE_REAL64,SD_BAS_DIM_ROT_MATRICE*SD_BAS_DIM_ROT_MATRICE);
  // ...FT-2765

  /* lecture IDefIASIScanAngle   */
  // jlp
  nbElement = fread(Conf->IDefIASIScanAngle ,SD_BAS_SIZE_REAL64,  SD_BAS_SP , ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_SP);
	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefIASIScanAngle,SD_BAS_SIZE_REAL64,SD_BAS_SP);
  // ...FT-2765

  /* lecture IDefIccNbCycle   */
  nbElement = fread(&Conf->IDefIccNbCycle ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefIccNbCycle,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture  IDefIISAVHRROffsetGuess  */
  // jlp
  nbElement = fread(Conf->IDefIISAVHRROffsetGuess ,SD_BAS_SIZE_REAL64, SD_BAS_SP * SD_BAS_CCD, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_SP * SD_BAS_CCD);
	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefIISAVHRROffsetGuess,SD_BAS_SIZE_REAL64,SD_BAS_SP*SD_BAS_CCD);
  // ...FT-2765

  /* lecture  IDefIISColDeb  */
  nbElement = fread(Conf->IDefIISColDeb ,SD_BAS_SIZE_INT4, SD_BAS_PN , ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_PN);
	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefIISColDeb,SD_BAS_SIZE_INT4,SD_BAS_PN);
  // ...FT-2765

  /* lecture  IDefIISColFin  */
  nbElement = fread(Conf->IDefIISColFin,SD_BAS_SIZE_INT4, SD_BAS_PN , ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_PN);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefIISColFin,SD_BAS_SIZE_INT4,SD_BAS_PN);
  // ...FT-2765

  /* lecture IDefIISDY   */
  nbElement = fread(&Conf->IDefIISDY ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefIISDY,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture  IDefIISDZ  */
  nbElement = fread(&Conf->IDefIISDZ ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefIISDZ,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture IDefIISLinDeb   */
  nbElement = fread(Conf->IDefIISLinDeb ,SD_BAS_SIZE_INT4, SD_BAS_PN , ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_PN);
	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefIISLinDeb,SD_BAS_SIZE_INT4,SD_BAS_PN);
  // ...FT-2765

  /* lecture IDefIISLinFin   */
  nbElement = fread(Conf->IDefIISLinFin ,SD_BAS_SIZE_INT4, SD_BAS_PN , ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_PN);
	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefIISLinFin,SD_BAS_SIZE_INT4,SD_BAS_PN);
  // ...FT-2765

  /* lecture IDefIISNbCol   */
  nbElement = fread(&Conf->IDefIISNbCol ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefIISNbCol,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture  IDefIISNbLin  */
  nbElement = fread(&Conf->IDefIISNbLin ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefIISNbLin,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture IDefIISNoiseNom   */
  nbElement = fread(&Conf->IDefIISNoiseNom ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefIISNoiseNom,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture   IDefIISRot */
  nbElement = fread(&Conf->IDefIISRot ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefIISRot,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture  IDefIISY  */
  nbElement = fread(Conf->IDefIISY ,SD_BAS_SIZE_REAL64, SD_BAS_IMLI , ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_IMLI);
	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefIISY,SD_BAS_SIZE_REAL64,SD_BAS_IMLI);
  // ...FT-2765

  /* lecture  IDefIISZ  */
  nbElement = fread(Conf->IDefIISZ ,SD_BAS_SIZE_REAL64, SD_BAS_IMCO , ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_IMCO);
	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefIISZ,SD_BAS_SIZE_REAL64,SD_BAS_IMCO);
  // ...FT-2765

  /* lecture  IDefMcxColdSpectrum  */
  // jlp
  nbElement = fread(Conf->IDefMcxColdSpectrum ,SD_BAS_SIZE_REAL64, SD_BAS_TAILLE_NB_SI_B * SD_BAS_SB, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B * SD_BAS_SB);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefMcxColdSpectrum,SD_BAS_SIZE_REAL64,SD_BAS_SB);
  // ...FT-2765

  /* lecture  IDefMcxDphi  */
  // jlp
  nbElement = fread(Conf->IDefMcxDphi ,SD_BAS_SIZE_REAL64, SD_BAS_TAILLE_NB_SI_B * SD_BAS_SB , ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B * SD_BAS_SB);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefMcxDphi,SD_BAS_SIZE_REAL64,SD_BAS_TAILLE_NB_SI_B*SD_BAS_SB);
  // ...FT-2765
 

  /* lecture IDefMcxBiasCalRadCutoff   */
  nbElement = fread(Conf->IDefMcxBiasCalRadCutoff ,SD_BAS_SIZE_REAL64, SD_BAS_SB , ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_SB);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefMcxBiasCalRadCutoff,SD_BAS_SIZE_REAL64,SD_BAS_SB);
  // ...FT-2765

  /* lecture  IDefMcxNoiseCalRadCutoff  */
  nbElement = fread(Conf->IDefMcxNoiseCalRadCutoff ,SD_BAS_SIZE_REAL64, SD_BAS_SB , ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_SB);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefMcxNoiseCalRadCutoff,SD_BAS_SIZE_REAL64,SD_BAS_SB);
  // ...FT-2765

  /* lecture  IDefMETOPOrbitPeriod  */
  nbElement = fread(&Conf->IDefMETOPOrbitPeriod ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefMETOPOrbitPeriod,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture  IDefNsfirst  */
  nbElement = fread(Conf->IDefNsfirst ,SD_BAS_SIZE_INT4, SD_BAS_SB , ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_SB);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefNsfirst,SD_BAS_SIZE_INT4,SD_BAS_SB);
  // ...FT-2765

  /* lecture  IDefNslast  */
  nbElement = fread(Conf->IDefNslast ,SD_BAS_SIZE_INT4, SD_BAS_SB , ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_SB);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefNslast,SD_BAS_SIZE_INT4,SD_BAS_SB);
  // ...FT-2765

  /* lecture  IDefOffsetSondIISModel  */
  // jlp
  nbElement = fread(Conf->IDefOffsetSondIISModel, SD_BAS_SIZE_REAL64, SD_BAS_MODEL * SD_BAS_CCD *SD_BAS_PN, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_MODEL * SD_BAS_CCD *SD_BAS_PN);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefOffsetSondIISModel,SD_BAS_SIZE_REAL64,SD_BAS_MODEL*SD_BAS_CCD*SD_BAS_PN);
  // ...FT-2765

  /* lecture  IDefPdsPix  */
  nbElement = fread(Conf->IDefPdsPix ,SD_BAS_SIZE_REAL64, SD_BAS_PN , ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_PN);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefPdsPix,SD_BAS_SIZE_REAL64,SD_BAS_PN);
  // ...FT-2765

  /* lecture    */
  nbElement = fread(&Conf->IDefPlkN ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefPlkN,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture IDefPlkN   */
  nbElement = fread(&Conf->IDefPlkPK1 ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefPlkPK1,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture IDefPlkPK2   */
  nbElement = fread(&Conf->IDefPlkPK2 ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefPlkPK2,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture IDefPsfSondBaryCentreY   */
  nbElement = fread(Conf->IDefPsfSondBaryCentreY ,SD_BAS_SIZE_REAL64, SD_BAS_PN , ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_PN);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefPsfSondBaryCentreY,SD_BAS_SIZE_REAL64,SD_BAS_PN);
  // ...FT-2765

  /* lecture  IDefPsfSondBaryCentreZ  */
  nbElement = fread(Conf->IDefPsfSondBaryCentreZ ,SD_BAS_SIZE_REAL64, SD_BAS_PN , ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_PN);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefPsfSondBaryCentreZ,SD_BAS_SIZE_REAL64,SD_BAS_PN);
  // ...FT-2765

  /* lecture IDefQisAvgRadNoise   */
  // jlp
  nbElement = fread(Conf->IDefQisAvgRadNoise ,SD_BAS_SIZE_REAL64, SD_BAS_PN * SD_BAS_SB , ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_PN * SD_BAS_SB);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefQisAvgRadNoise,SD_BAS_SIZE_REAL64,SD_BAS_PN*SD_BAS_SB);
  // ...FT-2765
 
  /* lecture  IDefS1bNsFFT  */
  nbElement = fread(&Conf->IDefS1bNsFFT ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefS1bNsFFT,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture  IDefS1bOSFftFactor  */
  nbElement = fread(&Conf->IDefS1bOSFftFactor ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefS1bOSFftFactor,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture  IDefS1bOSSplineFactor  */
  nbElement = fread(&Conf->IDefS1bOSSplineFactor ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefS1bOSSplineFactor,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture  IDefS1bSigI  */
  nbElement = fread(&Conf->IDefS1bSigI ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefS1bSigI,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture IDefS1bSigS   */
  nbElement = fread(&Conf->IDefS1bSigS ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefS1bSigS,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture IDefS1cNbSWwidth   */
  nbElement = fread(&Conf->IDefS1cNbSWwidth ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefS1cNbSWwidth,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture IDefS1cNsErase   */
  nbElement = fread(&Conf->IDefS1cNsErase ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefS1cNsErase,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture   IDefS1cSigS */
  nbElement = fread(&Conf->IDefS1cSigS ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefS1cSigS,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture IDefSmeTScanModel   */
  nbElement = fread(Conf->IDefSmeTScanModel ,SD_BAS_SIZE_REAL64, SD_BAS_MODEL , ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_MODEL);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefSmeTScanModel,SD_BAS_SIZE_REAL64,SD_BAS_MODEL);
  // ...FT-2765

  /* lecture IDefSpectDWn1b   */
  nbElement = fread(&Conf->IDefSpectDWn1b ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefSpectDWn1b,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture  IDefNsfirst1b  */
  nbElement = fread(&Conf->IDefNsfirst1b ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefNsfirst1b,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture IDefNslast1b   */
  nbElement = fread(&Conf->IDefNslast1b ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefNslast1b,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture IDefSpectrDWn   */
  nbElement = fread(&Conf->IDefSpectrDWn ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefSpectrDWn,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture IDefSsdModulCutoff   */
  // jlp
  nbElement = fread(Conf->IDefSsdModulCutoff ,SD_BAS_SIZE_REAL64, SD_BAS_TAILLE_NB_SI_B * SD_BAS_SB, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B * SD_BAS_SB);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefSsdModulCutoff,SD_BAS_SIZE_REAL64,SD_BAS_TAILLE_NB_SI_B*SD_BAS_SB);
  // ...FT-2765
 
  /* lecture  IDefSsdNhar  */
  // jlp
  nbElement = fread(Conf->IDefSsdNhar ,SD_BAS_SIZE_INT4, SD_BAS_TAILLE_NB_SI_B, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefSsdNhar,SD_BAS_SIZE_INT4,SD_BAS_TAILLE_NB_SI_B);
  // ...FT-2765

  /* lecture IDefSsdNItera   */
  nbElement = fread(&Conf->IDefSsdNItera ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefSsdNItera,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture  IDefSsdNsfirstSW  */
  nbElement = fread(Conf->IDefSsdNsfirstSW ,SD_BAS_SIZE_INT4, SD_BAS_TAILLE_NB_SI_B, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefSsdNsfirstSW,SD_BAS_SIZE_INT4,SD_BAS_TAILLE_NB_SI_B);
  // ...FT-2765

  /* lecture  IDefSsdNslastSW  */
  nbElement = fread(Conf->IDefSsdNslastSW ,SD_BAS_SIZE_INT4, SD_BAS_TAILLE_NB_SI_B, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefSsdNslastSW,SD_BAS_SIZE_INT4,SD_BAS_TAILLE_NB_SI_B);
  // ...FT-2765

  /* lecture  IDefSsdOSFftFactor  */
  nbElement = fread(Conf->IDefSsdOSFftFactor,SD_BAS_SIZE_INT4, SD_BAS_TAILLE_NB_SI_B, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefSsdOSFftFactor,SD_BAS_SIZE_INT4,SD_BAS_TAILLE_NB_SI_B);
  // ...FT-2765

  /* lecture IDefSsdOSSplineFactor   */
  nbElement = fread(Conf->IDefSsdOSSplineFactor,SD_BAS_SIZE_INT4, SD_BAS_TAILLE_NB_SI_B, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefSsdOSSplineFactor,SD_BAS_SIZE_INT4,SD_BAS_TAILLE_NB_SI_B);
  // ...FT-2765

  /* lecture  IDefSsdSigI  */
  nbElement = fread(Conf->IDefSsdSigI,SD_BAS_SIZE_INT4, SD_BAS_TAILLE_NB_SI_B, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefSsdSigI,SD_BAS_SIZE_INT4,SD_BAS_TAILLE_NB_SI_B);
  // ...FT-2765

  /* lecture IDefSsdSigS   */
  nbElement = fread(Conf->IDefSsdSigS,SD_BAS_SIZE_INT4, SD_BAS_TAILLE_NB_SI_B, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefSsdSigS,SD_BAS_SIZE_INT4,SD_BAS_TAILLE_NB_SI_B);
  // ...FT-2765

  /* lecture IDefSsdSpectralWindowOPS   */
  nbElement = fread(&Conf->IDefSsdSpectralWindowOPS ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefSsdSpectralWindowOPS,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture  IDefSssStdImagCutoff  */
  nbElement = fread(&Conf->IDefSssStdImagCutoff ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefSssStdImagCutoff,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* FT-2520 - Version 1.7 */
  /* lecture  IDefSssWnShiftMax  */
  nbElement = fread(Conf->IDefSssWnShiftMax, SD_BAS_SIZE_REAL64, SD_BAS_PN, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_PN);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefSssWnShiftMax,SD_BAS_SIZE_REAL64,SD_BAS_PN);
  // ...FT-2765
  
  /* lecture  IDefSssWnShiftMin  */
  nbElement = fread(Conf->IDefSssWnShiftMin, SD_BAS_SIZE_REAL64, SD_BAS_PN, ConfAutre);
  // FT-2765
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_PN);
  SWAP_INT_BUFFER(Conf->IDefSssWnShiftMin,SD_BAS_SIZE_REAL64,SD_BAS_PN);
  // ...FT-2765
  /* ..FT-2520 */

  /* lecture  IDefSysTECIISQual  */
  nbElement = fread(&Conf->IDefSysTECIISQual ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefSysTECIISQual,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture   IDefSysTECSondQual */
  nbElement = fread(&Conf->IDefSysTECSondQual ,SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefSysTECSondQual,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* lecture IDefTrCalRad   */
  nbElement = fread(&Conf->IDefTrCalRad ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefTrCalRad,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture IDefTrnom   */
  nbElement = fread(&Conf->IDefTrnom ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefTrnom,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture IDefTrSpect   */
  nbElement = fread(&Conf->IDefTrSpect ,SD_BAS_SIZE_REAL64, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefTrSpect,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* lecture  IDefWnpNharmonic  */
  nbElement = fread(Conf->IDefWnpNharmonic,SD_BAS_SIZE_INT4, SD_BAS_TAILLE_NB_SI_B, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefWnpNharmonic,SD_BAS_SIZE_INT4,SD_BAS_TAILLE_NB_SI_B);
  // ...FT-2765

  /* lecture  IDefWnpSigG  */
  nbElement = fread(Conf->IDefWnpSigG,SD_BAS_SIZE_INT4, SD_BAS_TAILLE_NB_SI_B, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefWnpSigG,SD_BAS_SIZE_INT4,SD_BAS_TAILLE_NB_SI_B);
  // ...FT-2765

  /* lecture   IDefWnpSign */
  nbElement = fread(Conf->IDefWnpSign,SD_BAS_SIZE_INT4, SD_BAS_TAILLE_NB_SI_B, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefWnpSign,SD_BAS_SIZE_INT4,SD_BAS_TAILLE_NB_SI_B);
  // ...FT-2765

  /* lecture  IWnpDWnSWref  */
  nbElement = fread(Conf->IWnpDWnSWref,SD_BAS_SIZE_REAL64, SD_BAS_TAILLE_NB_SI_B, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IWnpDWnSWref,SD_BAS_SIZE_REAL64,SD_BAS_TAILLE_NB_SI_B);
  // ...FT-2765

  /* lecture  IWnpModulNat  */
  // jlp
  nbElement = fread(Conf->IWnpModulNat ,SD_BAS_SIZE_REAL64, SD_BAS_TAILLE_NB_SI_B * SD_BAS_SB, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B * SD_BAS_SB);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IWnpModulNat,SD_BAS_SIZE_REAL64,SD_BAS_TAILLE_NB_SI_B*SD_BAS_SB);
  // ...FT-2765
 
  /* lecture IWnpNsFFT   */
  nbElement = fread(Conf->IWnpNsFFT,SD_BAS_SIZE_INT4, SD_BAS_TAILLE_NB_SI_B, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IWnpNsFFT,SD_BAS_SIZE_INT4,SD_BAS_TAILLE_NB_SI_B);
  // ...FT-2765

  /* lecture  IWnpNsSSR  */
  nbElement = fread(Conf->IWnpNsSSR,SD_BAS_SIZE_INT4, SD_BAS_TAILLE_NB_SI_B, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IWnpNsSSR,SD_BAS_SIZE_INT4,SD_BAS_TAILLE_NB_SI_B);
  // ...FT-2765

  /* lecture  IWnpNsSW  */
  nbElement = fread(Conf->IWnpNsSW,SD_BAS_SIZE_INT4, SD_BAS_TAILLE_NB_SI_B, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IWnpNsSW,SD_BAS_SIZE_INT4,SD_BAS_TAILLE_NB_SI_B);
  // ...FT-2765

  /* lecture IWnpPhaseNat   */
  // jlp
  nbElement = fread(Conf->IWnpPhaseNat ,SD_BAS_SIZE_REAL64, SD_BAS_TAILLE_NB_SI_B * SD_BAS_SB, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B * SD_BAS_SB);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IWnpPhaseNat,SD_BAS_SIZE_REAL64,SD_BAS_TAILLE_NB_SI_B*SD_BAS_SB);
  // ...FT-2765
 
  /* lecture  IWnpWnSWref  */
  nbElement = fread(Conf->IWnpWnSWref,SD_BAS_SIZE_REAL64,  SD_BAS_TAILLE_NB_SI_B, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IWnpWnSWref,SD_BAS_SIZE_REAL64,SD_BAS_TAILLE_NB_SI_B);
  // ...FT-2765

  /* FT-2527 - Version 1.7 */
  /* IDefZpdBzpd */
  nbElement = fread(&Conf->IDefZpdBzpd, SD_BAS_SIZE_INT4, 1, ConfAutre);
  SD_BAS_CA_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefZpdBzpd,SD_BAS_SIZE_INT4,1);
  // ...FT-2765
  /* ..FT-2527 */

  //@ Fermeture du fichier
  ret = fclose(ConfAutre);
  if (ret == EOF)
    {
      cr_op   = SD_BAS_ERR_BAS_NIVEAU;
      msg_err = SD_BAS_MSG_CLOSE;
      num_lig = __LINE__;
      goto fin;
    }


  //@ Verification version revision
  if(Conf->IDefIssueIcd > SD_BAS_CONFIGURATION_AUTRE_VERSION.major ||
     (Conf->IDefIssueIcd ==  SD_BAS_CONFIGURATION_AUTRE_VERSION.major &&  
      Conf->IDefRevisionIcd > SD_BAS_CONFIGURATION_AUTRE_VERSION.minor))
    {
      cr_op   = SD_BAS_ERR_VERSION_CA;
      msg_err = SD_BAS_MSG_VERSION_CA;
      num_lig = __LINE__;
      goto fin;
    }
 

 fin:

  if (cr_op != CMN__STATUT_OK)
    {
      SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                          SD_BAS_ERR_SET_ConfAutre,
                          SD_BAS_ERR_NUM_JDB,
                          tcProcName_F,
                          num_lig,
                          msg_err);
    }


  return (cr_op);
}
