//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_BAS
//@ MODULE : SD_BAS_ConfStable
//@
//@ Comment :
//@    Code file for the  SD_BAS_ConfStable class which encapsulates the
//@    acces to the context file   :
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 06/02/2003 : [reference :] ott
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2521(DM-2100-OPS) : 19/12/2008 : S. BRUNEL
//@ Calcul de 4 pseudo-canaux AVHRR (Day 2,point 2)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2524(DM-2100-OPS) : 19/12/2008 : S. BRUNEL
//@ Ajouter un indice et un flag dans l'algorithme 22_SSD (Day 2,point 5)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2526(DM-2100-OPS) : 19/12/2008 : S. BRUNEL
//@ Include les differences de NZPD interpixel dans le produit 
//@ ENG (Day 2,point 7)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2527(DM-2100-OPS) : 19/12/2008 : S. BRUNEL
//@ Permetrre le calcul des spectres dans les bandes B1 & B2 meme lorsque un
//@ spike affecte la bande B3 (Day 2,point 8)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2528(DM-2100-OPS) : 19/12/2008 : S. BRUNEL
//@ Ajouter les informations sur les nuages et la surface obtenue
//@ par AVHRR dans le produit L1C (Day 2,point 9)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2529(DM-2100-OPS) : 19/12/2008 : S. BRUNEL
//@ Implementation de valeurs par defaut pour l'axe interferometrique dans 
//@ le fichier de "contexte a froid" (Day 2,point 10)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <stdio.h>
#include "SD_BAS_ConfStable.h"

// FT-2765
#include "SD_BAS_SwapEndian.h"
// ...FT-2765

/*====================================================================*/
// Local Constant Declaration 
/*====================================================================*/

/* macro de detection d'erreur sur les fread() */
#define SD_BAS_CS_ERR_FREAD(ret, nb_octets)\
{\
   if ((ret) != (nb_octets))\
   {\
     cr_op   = SD_BAS_ERR_BAS_NIVEAU;\
     msg_err = SD_BAS_MSG_READ;\
     num_lig = __LINE__;\
     goto fin;\
   }\
}

// FT-2765
static const int SD_BAS_SIZE_INT4 = 4;
static const int SD_BAS_SIZE_INT2 = 2;
static const int SD_BAS_SIZE_REAL64 = 8;
static const int SD_BAS_SIZE_BOOLEAN = 1;

static char * SD_BAS_MSG_VERSION_CS = "INVALID STABLE CONFIGURATION FILE VERSION";
// ...FT-2765

/*====================================================================*/
// Local types Declaration  
/*====================================================================*/
//======================================================================
// Internal fonctions Declaration
//======================================================================
//======================================================================
// Global variables Definition
//======================================================================


//======================================================================
//					FONCTIONS EXTERNES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LireConfStable
//@
//@ Description : ouverture lecture et fermeture du fichier de configuration  
//@ autre indique en argument
//@
//@
//@ Parameters :
//@			(ENTREE)  Fichier : nom du fichier de configuration 
//@			(ENTREE/SORTIE)  Conf : configuration lu 
//@  			(ENTREE)  ctx : contexte de l'etape                   
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@		        SD_BAS_ERR_OUVERTURE si on ne peut pas ouvrir le fichier
//@                     SD_BAS_ERR_FERMETURE si on ne peut pas fermer le fichier
//@                     SD_BAS_ERR_LECTURE   si on ne peut lire
//@
//@                     SD_BAS_VERSION si la longueur du fichier ou le numero
//@                     d'enregistrement son different de ceux predefinis
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_ChargerConfStable(
		char * Fichier,
		t_SD_BAS_ConfStable * Conf,
		t_SD_BAS_DataThread *ctx
			)
{
  static const char tcProcName_F[] = "SD_BAS_ChargerConfStable";
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  int ret = 0;
  size_t nbElement = 0;
  int num_lig = 0;
  char *msg_err = NULL;
  int buff = 0;
  FILE * ConfStable = NULL;
  // jlp
  char c;
  int *p;
  int i;


  //@ ouverture du fichier en lecture
  ConfStable = fopen ( Fichier , "rb");

  if( ConfStable == NULL)
    {
      cr_op   = SD_BAS_ERR_BAS_NIVEAU;
      msg_err = SD_BAS_MSG_OPEN;
      num_lig = __LINE__;
      goto fin;
    } 


  //@ Lecture des donnees

  /* Lecture IDefIssueIcd */
  nbElement = fread(&Conf->IDefIssueIcd,SD_BAS_SIZE_INT4, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefIssueIcd,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* Lecture IDefRevisionIcd   */
  nbElement = fread(&Conf->IDefRevisionIcd,SD_BAS_SIZE_INT4, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefRevisionIcd,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* Lecture  IDefStableParamID  */
  nbElement = fread(&Conf->IDefStableParamID,SD_BAS_SIZE_INT4, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefStableParamID,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* Lecture  IDefStableParamDate.day  */
  nbElement = fread(&buff, SD_BAS_SIZE_INT4, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&buff,SD_BAS_SIZE_INT4,1);
  // ...FT-2765
  Conf->IDefStableParamDate.day = buff;
  
  /* Lecture  IDefStableParamDate.millis  */
  nbElement = fread(&buff, SD_BAS_SIZE_INT4, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&buff,SD_BAS_SIZE_INT4,1);
  // ...FT-2765
  Conf->IDefStableParamDate.millis = buff;

  /* Lecture  IDefMETOPNumber  */
  nbElement = fread(&Conf->IDefMETOPNumber,SD_BAS_SIZE_INT4, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefMETOPNumber,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* Lecture IDefAvhrrMarginBefore   */
  nbElement = fread(&Conf->IDefAvhrrMarginBefore, SD_BAS_SIZE_INT4, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefAvhrrMarginBefore,SD_BAS_SIZE_INT4,1);
  // ...FT-2765
  
  /* Lecture IDefAvhrrMarginAfter   */
  nbElement = fread(&Conf->IDefAvhrrMarginAfter, SD_BAS_SIZE_INT4, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefAvhrrMarginAfter,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* Lecture IDefDptIISDeadPix   */
  // jlp
  p = Conf->IDefDptIISDeadPix;
  for (i=0; i<SD_BAS_IMCO * SD_BAS_IMLI; i++)
    {
      nbElement = fread(&c, SD_BAS_SIZE_BOOLEAN, 1, ConfStable);
      //  nbElement = fread(&Conf->IDefDptIISDeadPix, SD_BAS_SIZE_BOOLEAN, SD_BAS_IMCO * SD_BAS_IMLI, ConfStable);
      SD_BAS_CS_ERR_FREAD(nbElement, 1);
	    // FT-2765
      SWAP_INT_BUFFER(&c,SD_BAS_SIZE_BOOLEAN,1);
      // ...FT-2765

      *p = (int)c;
      p++;
    }
  

  /* Lecture  IDefIISNeDT  */
  // jlp
  nbElement = fread(Conf->IDefIISNeDT,SD_BAS_SIZE_REAL64, SD_BAS_IMCO * SD_BAS_IMLI, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, SD_BAS_IMCO * SD_BAS_IMLI);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefIISNeDT,SD_BAS_SIZE_REAL64,SD_BAS_IMCO*SD_BAS_IMLI);
  // ...FT-2765
  

  /* Lecture  IDefIISSrf  */
  // jlp
  nbElement = fread(Conf->IDefIISSrf,SD_BAS_SIZE_REAL64,  SD_BAS_IIS_MAX, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, SD_BAS_IIS_MAX);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefIISSrf,SD_BAS_SIZE_REAL64,SD_BAS_IIS_MAX);
  // ...FT-2765

  /* Lecture  IDefIISSrfDWn  */
  nbElement = fread(&Conf->IDefIISSrfDWn,SD_BAS_SIZE_REAL64, 1 , ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefIISSrfDWn,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* Lecture IDefIISSrfNsfirst   */
  nbElement = fread(&Conf->IDefIISSrfNsfirst,SD_BAS_SIZE_INT4, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefIISSrfNsfirst,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* Lecture  IDefIISSrfNslast  */
  nbElement = fread(&Conf->IDefIISSrfNslast,SD_BAS_SIZE_INT4, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefIISSrfNslast,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* Lecture  IDefPsfSondNbCol  */
  nbElement = fread(&Conf->IDefPsfSondNbCol,SD_BAS_SIZE_INT4, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefPsfSondNbCol,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* Lecture IDefPsfSondNbLin   */
  nbElement = fread(&Conf->IDefPsfSondNbLin ,SD_BAS_SIZE_INT4, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);  
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefPsfSondNbLin,SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* Lecture  IDefPsfSondOverSampFactor  */
  nbElement = fread(&Conf->IDefPsfSondOverSampFactor,SD_BAS_SIZE_INT4, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1); 
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefPsfSondOverSampFactor,SD_BAS_SIZE_INT4,1);
  // ...FT-2765
 
  /* Lecture  IDefPsfSondWgt  */
  // jlp
  nbElement = fread(Conf->IDefPsfSondWgt,SD_BAS_SIZE_REAL64, SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND * SD_BAS_PN, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND * SD_BAS_PN); 
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefPsfSondWgt,SD_BAS_SIZE_REAL64,SD_BAS_NB_COL_SOND*SD_BAS_NB_LIN_SOND*SD_BAS_PN);
  // ...FT-2765

  /* Lecture  IDefPsfSondY  */
  // jlp
  nbElement = fread(Conf->IDefPsfSondY,SD_BAS_SIZE_REAL64, SD_BAS_NB_COL_SOND * SD_BAS_PN, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, SD_BAS_NB_COL_SOND * SD_BAS_PN);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefPsfSondY,SD_BAS_SIZE_REAL64,SD_BAS_NB_COL_SOND*SD_BAS_PN);
  // ...FT-2765

  /* Lecture  IDefPsfSondZ  */
  // jlp
  nbElement = fread(Conf->IDefPsfSondZ,SD_BAS_SIZE_REAL64, SD_BAS_NB_COL_SOND * SD_BAS_PN, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, SD_BAS_NB_COL_SOND * SD_BAS_PN);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefPsfSondZ,SD_BAS_SIZE_REAL64,SD_BAS_NB_COL_SOND*SD_BAS_PN);
  // ...FT-2765
  
  /* Lecture  ISmaRbb  */
  // jlp
  nbElement = fread(Conf->ISmaRbb,SD_BAS_SIZE_REAL64,  SD_BAS_SS1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, SD_BAS_SS1);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->ISmaRbb,SD_BAS_SIZE_REAL64,SD_BAS_SS1);
  // ...FT-2765

  /* Lecture   ISmaRcs */
  // jlp
  nbElement = fread(Conf->ISmaRcs,SD_BAS_SIZE_REAL64,  SD_BAS_SS1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, SD_BAS_SS1);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->ISmaRcs,SD_BAS_SIZE_REAL64,SD_BAS_SS1);
  // ...FT-2765

  /* Lecture ISmaRew   */
  // jlp
  nbElement = fread(Conf->ISmaRew,SD_BAS_SIZE_REAL64, SD_BAS_SS1 * SD_BAS_SN , ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, SD_BAS_SS1 * SD_BAS_SN);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->ISmaRew,SD_BAS_SIZE_REAL64,SD_BAS_SS1*SD_BAS_SN);
  // ...FT-2765
 
  /* Lecture  ISmaRNs  */
  nbElement = fread(&Conf->ISmaRNs ,SD_BAS_SIZE_INT4, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);  
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->ISmaRNs,SD_BAS_SIZE_INT4,1);
  // ...FT-2765
 
  /* Lecture  IWnpWnSW  */
  // jlp
  nbElement = fread(Conf->IWnpWnSW,SD_BAS_SIZE_REAL64, SD_BAS_NS_SW * SD_BAS_TAILLE_NB_SI_B , ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, SD_BAS_NS_SW * SD_BAS_TAILLE_NB_SI_B);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IWnpWnSW,SD_BAS_SIZE_REAL64,SD_BAS_NS_SW*SD_BAS_TAILLE_NB_SI_B);
  // ...FT-2765
 
  /* Lecture  IDefDeltaAvhrrDTbefore  */
  // jlp
  nbElement = fread(Conf->IDefDeltaAvhrrDTbefore,SD_BAS_SIZE_REAL64,  SD_BAS_SNOT, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, SD_BAS_SNOT);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefDeltaAvhrrDTbefore,SD_BAS_SIZE_REAL64,SD_BAS_SNOT);
  // ...FT-2765

  /* Lecture  IDefDeltaAvhrrDTafter  */
  // jlp
  nbElement = fread(Conf->IDefDeltaAvhrrDTafter,SD_BAS_SIZE_REAL64, SD_BAS_SNOT, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, SD_BAS_SNOT);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefDeltaAvhrrDTafter,SD_BAS_SIZE_REAL64,SD_BAS_SNOT);
  // ...FT-2765

  /* Lecture  IDefDeltaAvhrrColLeft  */
  // jlp
  nbElement = fread(Conf->IDefDeltaAvhrrColLeft ,SD_BAS_SIZE_INT4,  SD_BAS_SNOT, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, SD_BAS_SNOT); 
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefDeltaAvhrrColLeft,SD_BAS_SIZE_INT4,SD_BAS_SNOT);
  // ...FT-2765

  /* Lecture  IDefDeltaAvhrrColRight  */
  // jlp
  nbElement = fread(Conf->IDefDeltaAvhrrColRight ,SD_BAS_SIZE_INT4, SD_BAS_SNOT, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, SD_BAS_SNOT);  
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefDeltaAvhrrColRight,SD_BAS_SIZE_INT4,SD_BAS_SNOT);
  // ...FT-2765

  /* Lecture  IDefScaleSondNbScale  */
  nbElement = fread(&Conf->IDefScaleSondNbScale ,SD_BAS_SIZE_INT2, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);  
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefScaleSondNbScale,SD_BAS_SIZE_INT2,1);
  // ...FT-2765

  /* Lecture   IDefScaleSondNsfirst */
  // jlp
  nbElement = fread(Conf->IDefScaleSondNsfirst ,SD_BAS_SIZE_INT2, SD_BAS_TAILLE_NB_SI_B , ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B);  
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefScaleSondNsfirst,SD_BAS_SIZE_INT2,SD_BAS_TAILLE_NB_SI_B);
  // ...FT-2765

  /* Lecture  IDefScaleSondNslast  */
  // jlp
  nbElement = fread(Conf->IDefScaleSondNslast ,SD_BAS_SIZE_INT2, SD_BAS_TAILLE_NB_SI_B , ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B);  
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefScaleSondNslast,SD_BAS_SIZE_INT2,SD_BAS_TAILLE_NB_SI_B);
  // ...FT-2765

  /* Lecture  IDefScaleSondScaleFactor  */
  // jlp
  nbElement = fread(Conf->IDefScaleSondScaleFactor ,SD_BAS_SIZE_INT2, SD_BAS_TAILLE_NB_SI_B , ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B);  
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefScaleSondScaleFactor,SD_BAS_SIZE_INT2,SD_BAS_TAILLE_NB_SI_B);
  // ...FT-2765

  /* Lecture IDefScaleIISScaleFactor   */
  nbElement = fread(&Conf->IDefScaleIISScaleFactor ,SD_BAS_SIZE_INT2, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);  
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefScaleIISScaleFactor,SD_BAS_SIZE_INT2,1);
  // ...FT-2765

  /* version 1.6 */
  /* Lecture IDefScaleSondNbScale1a */
  nbElement = fread(&Conf->IDefScaleSondNbScale1a, SD_BAS_SIZE_INT2, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1); 
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefScaleSondNbScale1a,SD_BAS_SIZE_INT2,1);
  // ...FT-2765
  
  /* Lecture IDefScaleSondNsfirst1a */
  nbElement = fread(Conf->IDefScaleSondNsfirst1a, SD_BAS_SIZE_INT2, SD_BAS_TAILLE_NB_SI_B, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefScaleSondNsfirst1a,SD_BAS_SIZE_INT2,SD_BAS_TAILLE_NB_SI_B);
  // ...FT-2765

  /* Lecture IDefScaleSondNslast1a */
  nbElement = fread(Conf->IDefScaleSondNslast1a, SD_BAS_SIZE_INT2, SD_BAS_TAILLE_NB_SI_B, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefScaleSondNslast1a,SD_BAS_SIZE_INT2,SD_BAS_TAILLE_NB_SI_B);
  // ...FT-2765

  /* Lecture IDefScaleSondSacleFactor1a */
  nbElement = fread(Conf->IDefScaleSondScaleFactor1a, SD_BAS_SIZE_INT2, SD_BAS_TAILLE_NB_SI_B, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, SD_BAS_TAILLE_NB_SI_B);
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefScaleSondScaleFactor1a,SD_BAS_SIZE_INT2,SD_BAS_TAILLE_NB_SI_B);
  // ...FT-2765

  /* Lecture IDefAvhrrSubGridDl */
  nbElement = fread(&Conf->IDefAvhrrSubGridDl, SD_BAS_SIZE_INT2, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefAvhrrSubGridDl,SD_BAS_SIZE_INT2,1);
  // ...FT-2765
  
  /* Lecture IDefAvhrrSubGridDc */
  nbElement = fread(&Conf->IDefAvhrrSubGridDc, SD_BAS_SIZE_INT2, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefAvhrrSubGridDc,SD_BAS_SIZE_INT2,1);
  // ...FT-2765

  /* Lecture IDefIacAvhrrChannel */
  nbElement = fread(&Conf->IDefIacAvhrrChannel, SD_BAS_SIZE_INT2, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefIacAvhrrChannel,SD_BAS_SIZE_INT2,1);
  // ...FT-2765

  /* FT-2524 - Version 1.7 - IDefSsdConverCutoff */
  nbElement = fread(&Conf->IDefSsdConverCutoff, SD_BAS_SIZE_REAL64, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);      
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefSsdConverCutoff,SD_BAS_SIZE_REAL64,1);
  // ...FT-2765
  /* ..FT-2524 */
  
  /* FT-2527 - Version 1.7 */
  /* IDefFlagMultiBand */
  nbElement = fread(&Conf->IDefFlagMultiBand, SD_BAS_SIZE_INT4, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);      
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefFlagMultiBand,SD_BAS_SIZE_INT4,1);
  // ...FT-2765
  /* ..FT-2527 */
  
  /* FT-2529 - Version 1.7 */
   /* IDefFaxGuessY */
  nbElement = fread(Conf->IDefFaxGuessY, SD_BAS_SIZE_REAL64, 2, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 2);   
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefFaxGuessY,SD_BAS_SIZE_REAL64,2);
  // ...FT-2765
  
  /* IDefFaxGuessZ */
  nbElement = fread(Conf->IDefFaxGuessZ, SD_BAS_SIZE_REAL64, 2, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 2);   
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefFaxGuessZ,SD_BAS_SIZE_REAL64,2);
  // ...FT-2765
  /* ..FT-2529 */
  
  /* FT-2526 - Version 1.7 */
  /* IDefPixRef */
  nbElement = fread(&Conf->IDefPixRef, SD_BAS_SIZE_INT4, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);      
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefPixRef,SD_BAS_SIZE_INT4,1);
  // ...FT-2765
  
  /* IDefInterPixNZpd */
  nbElement = fread(Conf->IDefInterPixNZpd, SD_BAS_SIZE_INT4, SD_BAS_PN * 2, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, SD_BAS_PN * 2);      
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefInterPixNZpd,SD_BAS_SIZE_INT4,SD_BAS_PN*2);
  // ...FT-2765
  /* ..FT-2526 */

  /* FT-2529 - Version 1.7 */
  /*IDefEUMAvhrrCldErrorCutoff */
  nbElement = fread(&Conf->IDefEUMAvhrrCldErrorCutoff, SD_BAS_SIZE_INT4, 1, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 1);      
 	// FT-2765
  SWAP_INT_BUFFER(&Conf->IDefEUMAvhrrCldErrorCutoff,SD_BAS_SIZE_INT4,1);
  // ...FT-2765
  /* ..FT-2529 */
  
  /* FT-2521 - Version 1.7 */
  /* IDefAvhrrNsfirst */
  nbElement = fread(Conf->IDefAvhrrNsfirst, SD_BAS_SIZE_INT4, 2, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 2);      
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefAvhrrNsfirst,SD_BAS_SIZE_INT4,2);
  // ...FT-2765

  /*IDefAvhrrNslast */
  nbElement = fread(Conf->IDefAvhrrNslast, SD_BAS_SIZE_INT4, 2, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 2);      
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefAvhrrNslast,SD_BAS_SIZE_INT4,2);
  // ...FT-2765
  
  /*IDefAvhrrWgt */
  nbElement = fread(Conf->IDefAvhrrWgt, SD_BAS_SIZE_REAL64, 2*1500, ConfStable);
  SD_BAS_CS_ERR_FREAD(nbElement, 2*1500);      
 	// FT-2765
  SWAP_INT_BUFFER(Conf->IDefAvhrrWgt,SD_BAS_SIZE_REAL64,2*1500);
  // ...FT-2765
  /* ..FT-2521 */

  //@ Fermeture du fichier
  ret = fclose(ConfStable);
  if(ret == EOF) 
    {
      cr_op   = SD_BAS_ERR_BAS_NIVEAU;
      msg_err = SD_BAS_MSG_CLOSE;
      num_lig = __LINE__;
      goto fin;
    }


  //verification version revision
  if(Conf->IDefIssueIcd > SD_BAS_CONFIGURATION_STABLE_VERSION.major ||
     (Conf->IDefIssueIcd ==  SD_BAS_CONFIGURATION_STABLE_VERSION.major && 
      Conf->IDefRevisionIcd > SD_BAS_CONFIGURATION_STABLE_VERSION.minor))
    {
      cr_op   = SD_BAS_ERR_VERSION_CS;
      msg_err = SD_BAS_MSG_VERSION_CS;
      num_lig = __LINE__;
      goto fin;
    }


 fin:
 
  if (cr_op != CMN__STATUT_OK)
    {
      SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                          SD_BAS_ERR_SET_ConfStable,
                          SD_BAS_ERR_NUM_JDB,
                          tcProcName_F,
                          num_lig,
                          msg_err);
    }


  return (cr_op);
}
