//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/BAS/src/SD_BAS_ConfStable.h,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Component : SD_BAS
//@ MODULE : SD_BAS_ConfStable
//@
//@ Comment :
//@    include file used to define stableconfiguration file
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 06/02/2003 : [reference :] ott
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2521(DM-2100-OPS) : 19/12/2008 : S. BRUNEL
//@ Calcul de 4 pseudo-canaux AVHRR (Day 2,point 2)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2524(DM-2100-OPS) : 19/12/2008 : S. BRUNEL
//@ Ajouter un indice et un flag dans l'algorithme 22_SSD (Day 2,point 5)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2526(DM-2100-OPS) : 19/12/2008 : S. BRUNEL
//@ Include les differences de NZPD interpixel dans le produit 
//@ ENG (Day 2,point 7)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2527(DM-2100-OPS) : 19/12/2008 : S. BRUNEL
//@ Permetrre le calcul des spectres dans les bandes B1 & B2 meme lorsque un
//@ spike affecte la bande B3 (Day 2,point 8)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2528(DM-2100-OPS) : 19/12/2008 : S. BRUNEL
//@ Ajouter les informations sur les nuages et la surface obtenue
//@ par AVHRR dans le produit L1C (Day 2,point 9)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2529(DM-2100-OPS) : 19/12/2008 : S. BRUNEL
//@ Implementation de valeurs par defaut pour l'axe interferometrique dans 
//@ le fichier de "contexte a froid" (Day 2,point 10)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------



#ifndef SD_BAS_ConfStable_h
#define SD_BAS_ConfStable_h


/* librairies C de bases utilisees par les modules du projet */
#include <stdio.h>       /* entree/sortie */
#include <stdlib.h>      /* alloc memoire, ...*/
#include <string.h>      /* chaines de carateres */
#include <unistd.h>      /* gestion pipes, ... */
#include <time.h>        /* gestion de la date */
#include <sys/param.h>   /* constantes systeme */
#include <fcntl.h>

// inserer ici les fichiers include 
#include "SD_BAS_Global.h"
#include "ERR__Erreur.h"

/*====================================================================*/
//   External define  Declarations
/*====================================================================*/

/* FT-2521,FT-2524,FT-2526,FT-2527,FT-2528,FT-2529 */
// FT-2765
static const int SD_BAS_TAILLE_CS = 3356760;
// ...FT-2765
/* ..FT-2521,FT-2524,FT-2526,FT-2527,FT-2528,FT-2529 */



#define ERR_BASE_CONFSTABLE SD_BAS_ERR_BASE_ConfStable
#define SD_BAS_ERR_VERSION_CS (ERR_BASE_CONFSTABLE +1)


// FT-2765
static const int SD_BAS_ERR_SET_ConfStable = 1;
// ...FT-2765

//===================================================================*/
// Exported types Declaration  
//====================================================================*/
// structure contenant les informations contenue dans la base de donnees
// spectrale 



typedef struct 
	{
// il faut definir ici l'ensemble des champs lu dans le fichier 
// apres decodage	
	  int IDefIssueIcd;
	  int IDefRevisionIcd;
	  int  IDefStableParamID;
	  t_SD_BAS_SCDSTime IDefStableParamDate;
	  int IDefMETOPNumber;
	  int IDefAvhrrMarginBefore;
	  int IDefAvhrrMarginAfter;
	  int  IDefDptIISDeadPix[SD_BAS_IMLI * SD_BAS_IMCO];
	  double IDefIISNeDT[SD_BAS_IMLI * SD_BAS_IMCO];
	  double IDefIISSrf[SD_BAS_IIS_MAX];
	  double IDefIISSrfDWn;
	  int IDefIISSrfNsfirst;
	  int IDefIISSrfNslast;
//	  int IDefIISSrfNs;
	  int IDefPsfSondNbCol;
	  int IDefPsfSondNbLin;
	  int IDefPsfSondOverSampFactor;
	  double IDefPsfSondWgt[SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND * SD_BAS_PN];
	  double IDefPsfSondY[SD_BAS_NB_COL_SOND * SD_BAS_PN];
	  double IDefPsfSondZ[SD_BAS_NB_LIN_SOND * SD_BAS_PN];
	  double ISmaRbb[SD_BAS_SS1];
	  double ISmaRcs[SD_BAS_SS1];
	  double ISmaRew[SD_BAS_SS1 * SD_BAS_SN];
	  int ISmaRNs;
	  double IWnpWnSW[SD_BAS_NS_SW * SD_BAS_TAILLE_NB_SI_B];
	  double IDefDeltaAvhrrDTbefore[SD_BAS_SNOT];
	  double IDefDeltaAvhrrDTafter[SD_BAS_SNOT];
	  int IDefDeltaAvhrrColLeft[SD_BAS_SNOT];
	  int IDefDeltaAvhrrColRight[SD_BAS_SNOT];
	  short int IDefScaleSondNbScale;
	  short int IDefScaleSondNsfirst[SD_BAS_TAILLE_NB_SI_B];
	  short int IDefScaleSondNslast[SD_BAS_TAILLE_NB_SI_B];
	  short int IDefScaleSondScaleFactor[SD_BAS_TAILLE_NB_SI_B];
	  short int IDefScaleIISScaleFactor;
// version 1.6
      short int IDefScaleSondNbScale1a;
      short int IDefScaleSondNsfirst1a[SD_BAS_TAILLE_NB_SI_B];
      short int IDefScaleSondNslast1a[SD_BAS_TAILLE_NB_SI_B];
      short int IDefScaleSondScaleFactor1a[SD_BAS_TAILLE_NB_SI_B];
      short int IDefAvhrrSubGridDl;
      short int IDefAvhrrSubGridDc;
      short int IDefIacAvhrrChannel;

      // FT-2524 - Version 1.7
      double IDefSsdConverCutoff;
      // ..FT-2524
      // FT-2527 - Version 1.7
      int IDefFlagMultiBand;
      // ..FT-2527
      // FT-2529 - Version 1.7
      double IDefFaxGuessY[2];
      double IDefFaxGuessZ[2];
      // ..FT-2529
      // FT-2526 - Version 1.7
      int IDefPixRef;
      int IDefInterPixNZpd[SD_BAS_PN * 2];
      // ..FT-2526
      // FT-2529 - Version 1.7
      int IDefEUMAvhrrCldErrorCutoff;
      // ..FT-2529
      // FT-2521 - Version 1.7
      int IDefAvhrrNsfirst[2];
      int IDefAvhrrNslast[2];
      double IDefAvhrrWgt[2 * 1500];
      // ..FT-2521

	} t_SD_BAS_ConfStable; 


//====================================================================*/
// External variables Declaration 
//====================================================================*/
//version de l'ICD utilise pour le format du fichier de conf stable
/* ..FT-2521,FT-2524,FT-2526,FT-2527,FT-2528,FT-2529 */
// FT-2765
static const t_SD_BAS_VERSION SD_BAS_CONFIGURATION_STABLE_VERSION={1,7};
// ...FT-2765
/* FT-2521,FT-2524,FT-2526,FT-2527,FT-2528,FT-2529 */

//====================================================================*/
// External fonctions Declaration 
//====================================================================*/

#ifdef __cplusplus
extern "C" {
#endif
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_ChargerConfStable
//@
//@ Description : ouverture lecture et fermeture du fichier de configuration  
//@ autre indique en argument
//@
//@
//@ Parameters :
//@			(ENTREE)  Fichier : nom du fichier de configuration 
//@			(ENTREE/SORTIE)  Conf : configuration lu 
//@  			(ENTREE)  ctx : contexte de l'etape                   
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@		        SD_BAS_ERR_OUVERTURE si on ne peut pas ouvrir le fichier
//@                     SD_BAS_ERR_FERMETURE si on ne peut pas fermer le fichier
//@                     SD_BAS_ERR_LECTURE   si on ne peut lire
//@                     SD_BAS_VERSION si la longueur du fichier ou le numero
//@                     d'enregistrement son different de ceux predefinis
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_ChargerConfStable(
		char * Fichier,
		t_SD_BAS_ConfStable * Conf,
		t_SD_BAS_DataThread *ctx
		);

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_EcritConfStable
//@
//@ Description : ouverture lecture et fermeture du fichier de configuration  
//@ autre indique en argument
//@
//@
//@ Parameters :
//@			(ENTREE)  Fichier : nom du fichier de configuration 
//@			(ENTREE)  Conf : configuration lu 
//@  			(ENTREE)  ctx : contexte de l'etape                   
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@		        SD_BAS_ERR_OUVERTURE si on ne peut pas ouvrir le fichier
//@                     SD_BAS_ERR_FERMETURE si on ne peut pas fermer le fichier
//@                     SD_BAS_ERR_ECRITURE   si on ne peut pas ecrire
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_EcritConfStable(
		char * Fichier,
		t_SD_BAS_ConfStable * Conf,
		t_SD_BAS_DataThread *ctx
		);




#ifdef __cplusplus
}
#endif


#endif
