//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/BAS/src/SD_BAS_Contexte.c,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_BAS
//@ MODULE : SD_BAS_Contexte
//@
//@ Comment :
//@    Code file for the  SD_BAS_Contexte class which encapsulates the
//@    acces to the context file   :
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 13/10/2002 : [reference :] ott
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-169-CN : 27/02/2006 : B.MASSART
//@ Patch V3-3-5 is integrated. Type of several variables in context is 
//@ modified. CTX version is now 1.2. See IA-2100-FC-169-THA.
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <stdio.h>

#include "SD_BAS_Contexte.h"
// declaration des fonctions importees

// FT-2765
#include "SD_BAS_SwapEndian.h"
// ...FT-2765

/*====================================================================*/
// Local Constant Declaration
/*====================================================================*/

/* MACRO testant le retour des fonction d'acces au fichier */
// FT-2765
#define SD_BAS_ERREUR(cr,errnum,msg,lg)\
{\
   if(cr <= 0)\
     {\
      printf( __FILE__ ":%d errnum = %d, %s\n", __LINE__, errnum, msg ); \
      fflush( stdout ); \
      cr_op=errnum;\
      num_lg=lg;\
      msg_err = msg;\
      goto fin;\
     }\
}
// ...FT-2765

/* definition des messages d'erreurs */

/* erreur de version */
static char* SD_BAS_MSG_VERSION_CTX    = "INVALID CONTEXT FILE VERSION";
/* erreur d'integrite */
static char* SD_BAS_MSG_INTEGRITE      = "CONTEXT FILE IS USING";
/* erreur de lecture du contexte */
static char* SD_BAS_MSG_LIRECONTEXTE   = "INVALID CONTEXT FILE";
/* erreur d'ecriture du contexte */
static char* SD_BAS_MSG_ECRIRECONTEXTE = "CONTEXT FILE WRITE FAILED";

// version de l'ICD utilise pour le format du fichier de conf autre
// en principe si le fichier a une version () <= alors
// la couche est capable de lire la donnee
// version de l'OPS en debut de vie
static const t_SD_BAS_VERSION SD_BAS_CTXVERSION={1,2}; /* FA169 */


/*====================================================================*/
// Local types Declaration
/*====================================================================*/
//======================================================================
// Internal fonctions Declaration
//======================================================================
t_CMN__Statut SD_BAS_LireCorpsCtx(
                 int fd,
                 t_SD_BAS_Contexte * Ctx,
                 t_SD_BAS_DataThread *ctx
                 );



t_CMN__Statut SD_BAS_EcrireCorpsCtx(
                 int fd,
                 t_SD_BAS_Contexte * Ctx,
                 t_SD_BAS_DataThread *ctx
                 );

//======================================================================
// Global variables Definition
//======================================================================


//======================================================================
//                  FONCTIONS EXTERNES
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LireCtx
//@
//@ Description : ouverture lecture et fermeture du fichier de contexte
//@ indique en argument
//@
//@
//@ Parameters :
//@         (ENTREE)  Fichier : nom du fichier de contexte
//@         (ENTREE/SORTIE)  Ctx : contexte lu
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LireCtx(
  char * Fichier,
  t_SD_BAS_Contexte * Ctx,
  t_SD_BAS_DataThread *ctx)
{
  static const char tcProcName_F[] = "SD_BAS_LireCtx";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  int fd;
  int flag;
  int cr;
  unsigned short int integ;
  int i;
  char *msg_err;
  int num_lg;

  unsigned short int BufferUnsignedShortInt;
  short int          BufferShortInt;
  unsigned int       BufferUnsignedInt;
  int                BufferInt;
  
  int TailleUnsignedShortInt = 2;
  int TailleShortInt = 2;
  int TailleUnsignedInt = 4;
  int TailleInt = 4;
  int TailleDouble = 8;
  
  // Entree dans la fonction
  //@  ouverture du fichier en lecture
  flag = O_RDONLY;
  
  //@ ouverture du fichier
  fd = open(Fichier,flag);
  if(fd == -1)
  {
    cr_op = SD_BAS_ERR_BAS_NIVEAU;
    msg_err =SD_BAS_MSG_OPEN;
    num_lg = __LINE__;
    goto fin;
  }
  
  //@ lecture de l'indicateur d'integrite et test
  cr = read(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ , __LINE__);
	// FT-2765
  SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
  // ...FT-2765
  integ = BufferUnsignedShortInt;
  
  //@ test sur l'indicateur d'integrite
  if(0 == integ)
  {
    cr_op = SD_BAS_ERR_INTEGRITE;
    msg_err = SD_BAS_MSG_INTEGRITE;
    num_lg = __LINE__;
    goto fin;
  }
  
  /* lecture de la version OPS */
  cr = read(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
	// FT-2765
  SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
  // ...FT-2765
  Ctx->GCTXVersOPS.major = (int)BufferUnsignedShortInt;

  /* lecture de la version OPS */
  cr = read(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
	// FT-2765
  SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
  // ...FT-2765
  Ctx->GCTXVersOPS.minor = (int)BufferUnsignedShortInt;

  //@ lecture de la version du fichier CTX
  cr = read(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
	// FT-2765
  SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
  // ...FT-2765
  Ctx->GCTXVersCTX.major = (int)BufferUnsignedShortInt;

  cr = read(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
	// FT-2765
  printf( "BufferUnsignedShortInt: %d \n",  (int)BufferUnsignedShortInt);
  SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
  // ...FT-2765
  Ctx->GCTXVersCTX.minor = (int)BufferUnsignedShortInt;

  printf( "GCTXVersCTX.minor: %d \n", Ctx->GCTXVersCTX.minor );
  printf( "GCTXVersCTX.major: %d \n", Ctx->GCTXVersCTX.major );
  printf( "SD_BAS_CTXVERSION.minor: %d \n", SD_BAS_CTXVERSION.minor );
  printf( "SD_BAS_CTXVERSION.major: %d \n", SD_BAS_CTXVERSION.major );
  printf( "BufferUnsignedShortInt: %d \n",  (int)BufferUnsignedShortInt);

  

  
  //@  controle de coherence entre la version CTX
  //@  du fichier (vf ) et la version de la couche de lecture (vl) : vl >= vf
  if (Ctx->GCTXVersCTX.major > SD_BAS_CTXVERSION.major || 
      ((Ctx->GCTXVersCTX.major == SD_BAS_CTXVERSION.major) && 
       (Ctx->GCTXVersCTX.minor > SD_BAS_CTXVERSION.minor)))
  {
    cr_op = SD_BAS_ERR_VERSION;
    msg_err = SD_BAS_MSG_VERSION_CTX;
    num_lg = __LINE__;
    goto fin;
  }

  //@ lecture de GCTXNatCTX
  cr = read(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
	// FT-2765
  SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
  // ...FT-2765
  Ctx->GCTXNatCTX = (int)BufferUnsignedShortInt;

  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    // jlp
    //@ lecture de GEPSDatIASI (day)
    cr = read(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
  	// FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    Ctx->GEPSDatIASI[i].day = BufferUnsignedShortInt;
    
    //@ lecture de GEPSDatIASI (millis)
    cr = read(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
	  // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    Ctx->GEPSDatIASI[i].millis = BufferUnsignedInt;
  }

  //@ lecture de BIMSBBT
  cr = read(fd, &Ctx->BIMSBBT, SD_BAS_NB_VAL * TailleDouble);
	// FT-2765
  SWAP_INT_BUFFER(&Ctx->BIMSBBT,TailleDouble,SD_BAS_NB_VAL);
  // ...FT-2765
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);

  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ lecture de GCTXNumDerLig
    cr = read(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
	  // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    Ctx->GCTXNumDerLig[i] = (int)BufferUnsignedInt;
    if (Ctx->GCTXNumDerLig[i] < 0)
    {
      cr_op = SD_BAS_ERR_VERSION;
      msg_err = SD_BAS_MSG_LIRECONTEXTE;
      num_lg = __LINE__;
      goto fin;
    }
  }

  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ lecture de PTSI
    cr = read(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
  	// FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    Ctx->PTSI[i] = (int)BufferUnsignedShortInt;
  }
  
  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ lecture de BIISEqualCountOld
    cr = read(fd, &BufferShortInt, TailleShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
	  // FT-2765
    SWAP_INT_BUFFER(&BufferShortInt,TailleShortInt,1);
    // ...FT-2765
    Ctx->BIISEqualCountOld[i] = (int)BufferShortInt;
  }
  
  //@ lecture de GIaxNbElt
  cr = read(fd, &BufferInt, TailleInt);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
	// FT-2765
  SWAP_INT_BUFFER(&BufferInt,TailleInt,1);
  // ...FT-2765
  Ctx->GIaxNbElt = BufferInt;

  for(i=0; i < SD_BAS_CCD * Ctx->GIaxNbElt ; i++)
  {
// jlp  
    //@ lecture de GiaxDateElt
    cr = read(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
	  // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    Ctx->GIaxHistAxeInterf[i].GiaxDateElt.day = BufferUnsignedShortInt;
         
    //@ lecture de GiaxDateElt
    cr = read(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
	  // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    Ctx->GIaxHistAxeInterf[i].GiaxDateElt.millis = BufferUnsignedInt;

    //@ lecture de GiaxAxeYElt
    cr = read(fd, &Ctx->GIaxHistAxeInterf[i].GiaxAxeYElt, TailleDouble);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
	  // FT-2765
    SWAP_INT_BUFFER(&Ctx->GIaxHistAxeInterf[i].GiaxAxeYElt,TailleDouble,1);
    // ...FT-2765

    //@ lecture de GiaxAxeZElt
    cr = read(fd, &Ctx->GIaxHistAxeInterf[i].GiaxAxeZElt, TailleDouble);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
	  // FT-2765
    SWAP_INT_BUFFER(&Ctx->GIaxHistAxeInterf[i].GiaxAxeZElt,TailleDouble,1);
    // ...FT-2765

    //@ lecture de GiaxAxeQualElt
    cr = read(fd, &Ctx->GIaxHistAxeInterf[i].GiaxAxeQualElt, TailleDouble);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
	  // FT-2765
    SWAP_INT_BUFFER(&Ctx->GIaxHistAxeInterf[i].GiaxAxeQualElt,TailleDouble,1);
    // ...FT-2765

    //@ lecture de GiaxFlagAxeNonQual
    cr = read(
      fd, 
      &Ctx->GIaxHistAxeInterf[i].GiaxFlagAxeNonQual, 
      TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);     
	  // FT-2765
    SWAP_INT_BUFFER(&Ctx->GIaxHistAxeInterf[i].GiaxFlagAxeNonQual,TailleUnsignedShortInt,1);
    // ...FT-2765
  }
  
  //@ lecture de MExsSmin
  cr = read(fd, Ctx->MExsSmin, SD_BAS_SPCT_COD * TailleDouble);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
  // FT-2765
  SWAP_INT_BUFFER(Ctx->MExsSmin,TailleDouble,SD_BAS_SPCT_COD);
  // ...FT-2765
  
  //@ lecture de MExsSmax
  cr = read(fd, Ctx->MExsSmax,SD_BAS_SPCT_COD * TailleDouble);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
  // FT-2765
  SWAP_INT_BUFFER(Ctx->MExsSmax,TailleDouble,SD_BAS_SPCT_COD);
  // ...FT-2765

  //@ lecture de GBBTNbElt
  cr = read(fd, &BufferUnsignedInt, TailleUnsignedInt);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
  // FT-2765
  SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
  // ...FT-2765
  Ctx->GBBTNbElt = (int)BufferUnsignedInt;
  if (Ctx->GBBTNbElt < 0)
  {
    cr_op = SD_BAS_ERR_VERSION;
    msg_err = SD_BAS_MSG_LIRECONTEXTE;
    num_lg = __LINE__;
    goto fin;
  }

  /* Lecture du reste des donnees */
  cr_op = SD_BAS_LireCorpsCtx(fd, Ctx, ctx);
  
  if (cr_op != CMN__STATUT_OK)
  {
    cr_op   = SD_BAS_ERR_BAS_NIVEAU;
    msg_err = SD_BAS_MSG_LIRECONTEXTE;
    num_lg  = __LINE__;
    goto fin;
  }
  
  //@ fermeture du fichier
  cr = close(fd);
  
  /* Pour compatibilite avec SD_BAS_ERREUR */
  if(cr == 0) cr=1;
  
  SD_BAS_ERREUR(cr,SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_CLOSE, __LINE__);
  
  // on trace ici si besoin toutes les variables en sortie

fin:

  if(cr_op != CMN__STATUT_OK)
  {
    /* mise a jour du contexte d'erreur */
    SD_BAS_TEST_ERR_INT(
      ctx->cr.errinterne,
      SD_BAS_ERR_SET_Contexte,
      SD_BAS_ERR_NUM_JDB,
      tcProcName_F,
      num_lg,
      msg_err);
  }
  
  return(cr_op);
}

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_EcrireCtx
//@
//@ Description : ouverture lecture et fermeture du fichier de contexte
//@ indique en argument
//@
//@
//@ Parameters :
//@         (ENTREE)  Fichier : nom du fichier de contexte
//@         (ENTREE)  Ctx : contexte lu
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_EcrireCtx(
  char * Fichier,
  t_SD_BAS_Contexte * Ctx,
  t_SD_BAS_DataThread *ctx)
{
  static const char tcProcName_F[] = "SD_BAS_EcrireCtx";
  t_CMN__Statut cr_op=CMN__STATUT_OK;

  int    flags;            /* flag utilise pour la fonction open           */
  int    mode;             /* valeur du umask utilise pou la fonction open */
  int    fd;               // file descriptor du fichier lum

  unsigned short int indicateur;
  
  unsigned short int BufferUnsignedShortInt;
  short int          BufferShortInt;
  unsigned int       BufferUnsignedInt;
  int                BufferInt;
  // FT-2765
  double             BufferDouble;
  // FT-2765
  
  int TailleUnsignedShortInt = 2;
  int TailleShortInt = 2;
  int TailleUnsignedInt = 4;
  int TailleInt = 4;
  int TailleDouble = 8;
  
  int cr;
  int i;
  int num_lg;
  char *msg_err;
  
  // Entree dans la fonctiong
  //@ ouverture du fichier
  flags = O_RDWR|O_CREAT;
  mode  = 0777;

  fd = open(Fichier,flags,mode);
  if(fd == -1)
  {
    cr_op = SD_BAS_ERR_BAS_NIVEAU;
    msg_err =SD_BAS_MSG_OPEN;
    num_lg=__LINE__;
    goto fin;
  }
  
  //@ ecriture de  la version de la couche de lecture
  //@ et  de l'indicateur d'integrite KO
  indicateur = 0;
  BufferUnsignedShortInt = indicateur;
  // FT-2765
  SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
  // ...FT-2765
  cr=write(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);

  //@ ecriture de  GCTXVersOPS
  BufferUnsignedShortInt = (unsigned short int)Ctx->GCTXVersOPS.major;
  // FT-2765
  SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
  // ...FT-2765
  cr = write(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  
  //@ ecriture de GCTXVersOPS
  BufferUnsignedShortInt = (unsigned short int)Ctx->GCTXVersOPS.minor;
  // FT-2765
  SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
  // ...FT-2765
  cr = write(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  
  //@ ecriture de GCTXVersCTX
  BufferUnsignedShortInt = (unsigned short int)Ctx->GCTXVersCTX.major;
  // FT-2765
  SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
  // ...FT-2765
  cr = write(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  
  //@ ecriture de GCTXVersCTX
  BufferUnsignedShortInt = (unsigned short int)Ctx->GCTXVersCTX.minor;
  // FT-2765
  SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
  // ...FT-2765
  cr = write(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);

  //@ ecriture de GCTXNatCTX
  BufferUnsignedShortInt = (unsigned short int)Ctx->GCTXNatCTX;
  // FT-2765
  SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
  // ...FT-2765
  cr = write(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);

  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ ecriture de GEPSDatIASI
    BufferUnsignedShortInt = (unsigned short int)Ctx->GEPSDatIASI[i].day;
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
    
    //@ ecriture de GEPSDatIASI
    BufferUnsignedInt = (unsigned int)Ctx->GEPSDatIASI[i].millis;
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }
  
  //@ ecriture de BIMSBBT
  // FT-2765
  for (i=0; i < SD_BAS_NB_VAL; i++)
  {
    BufferDouble = Ctx->BIMSBBT[i];
    SWAP_INT_BUFFER(&BufferDouble,TailleDouble,1);
    cr = write(fd, &BufferDouble, TailleDouble);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }
  // ...FT-2765

  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ ecriture de GCTXNumDerLig
    BufferUnsignedInt = (unsigned int)Ctx->GCTXNumDerLig[i];
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }

  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ ecriture de PTSI
    BufferUnsignedShortInt = (unsigned short int)Ctx->PTSI[i];
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }

  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ ecriture de BIISEqualCountOld
    BufferShortInt = (short int)Ctx->BIISEqualCountOld[i];
    // FT-2765
    SWAP_INT_BUFFER(&BufferShortInt,TailleShortInt,1);
    // ...FT-2765
    cr = write(fd, &BufferShortInt, TailleShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }

  //@ ecriture de GIaxNbElt
  BufferInt = Ctx->GIaxNbElt;
  // FT-2765
  SWAP_INT_BUFFER(&BufferInt,TailleInt,1);
  // ...FT-2765
  cr = write(fd, &BufferInt, TailleInt);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);

  for(i=0;i<Ctx->GIaxNbElt * SD_BAS_CCD;i++)
  {
    //@ ecriture de GiaxDateElt
    BufferUnsignedShortInt = 
      (unsigned short int)Ctx->GIaxHistAxeInterf[i].GiaxDateElt.day;
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
    
    //@ ecriture de GiaxDateElt
    BufferUnsignedInt = 
      (unsigned int )Ctx->GIaxHistAxeInterf[i].GiaxDateElt.millis;
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
    
    //@ ecriture de GiaxAxeYElt
    // FT-2765
    BufferDouble = Ctx->GIaxHistAxeInterf[i].GiaxAxeYElt;
    SWAP_INT_BUFFER(&BufferDouble,TailleDouble,1);
    cr = write(fd,&BufferDouble, TailleDouble);
    // ...FT-2765
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
    
    //@ ecriture de GiaxAxeZElt
    // FT-2765
    BufferDouble = Ctx->GIaxHistAxeInterf[i].GiaxAxeZElt;
    SWAP_INT_BUFFER(&BufferDouble,TailleDouble,1);
    cr = write(fd,&BufferDouble, TailleDouble);
    // ...FT-2765
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
    
    //@ ecriture de GiaxAxeQualElt
    // FT-2765
    BufferDouble = Ctx->GIaxHistAxeInterf[i].GiaxAxeQualElt;
    SWAP_INT_BUFFER(&BufferDouble,TailleDouble,1);
    cr = write(fd,&BufferDouble, TailleDouble);
    // ...FT-2765
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
    
    //@ ecriture de GiaxFlagAxeNonQual
    // FT-2765
    BufferUnsignedShortInt = Ctx->GIaxHistAxeInterf[i].GiaxFlagAxeNonQual;
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    cr = write(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
    // ...FT-2765
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }

  //@ ecriture de MExsSmin
  // FT-2765
  for (i=0; i<SD_BAS_SPCT_COD; i++)
  {
    BufferDouble = Ctx->MExsSmin[i];
    SWAP_INT_BUFFER(&BufferDouble,TailleDouble,1);
    cr = write(fd,&BufferDouble, TailleDouble);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }
  // ...FT-2765
  
  //@ ecriture de MExsSmax
  // FT-2765
  for (i=0; i<SD_BAS_SPCT_COD; i++)
  {
    BufferDouble = Ctx->MExsSmax[i];
    SWAP_INT_BUFFER(&BufferDouble,TailleDouble,1);
    cr = write(fd,&BufferDouble, TailleDouble);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }
  // ...FT-2765
  
  //@ ecriture de GBBTNbElt
  BufferUnsignedInt = (unsigned int)Ctx->GBBTNbElt;
  // FT-2765
  SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
  // ...FT-2765
  cr = write(fd, &BufferUnsignedInt, TailleUnsignedInt);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  
  /* Ecrire le reste des donnees */
  cr_op = SD_BAS_EcrireCorpsCtx(fd, Ctx, ctx);
  
  if (cr_op != CMN__STATUT_OK)
  {
    cr_op   = SD_BAS_ERR_BAS_NIVEAU;
    msg_err = SD_BAS_MSG_ECRIRECONTEXTE;
    num_lg  = __LINE__;
    goto fin;
  }

  //@ mise a jour  de l'indicateur d'integrite
  lseek(fd,0L,0);
  indicateur = 1;
  BufferUnsignedShortInt = indicateur;
  // FT-2765
  SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
  // ...FT-2765
  cr = write(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);

  //@ fermeture du fichier
  cr = close(fd);

  /* Pour compatibilite avec SD_BAS_ERREUR */
  if(cr == 0) cr =1;
 
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_CLOSE, __LINE__);
  
  // on trace ici si besoin toutes les variables en sortie

fin:

  if(cr_op != CMN__STATUT_OK)
  {
    SD_BAS_TEST_ERR_INT(
      ctx->cr.errinterne,
      SD_BAS_ERR_SET_Contexte,
      SD_BAS_ERR_NUM_JDB,
      tcProcName_F,
      num_lg,
      msg_err);

    /* on imprime le message d'erreur sur la sortie standard */
#ifdef DEBUG
     SD_BAS_PRINT_ERR_INT(
       ctx->cr.errinterne.nomfonction,
       ctx->cr.errinterne.numligne,
       ctx->cr.errinterne.libelle);
#endif
  }

  return(cr_op);
}



//======================================================================
//                  FONCTIONS INTERNES
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LireCorpsCtx
//@
//@ Description : lecture du fichier de contexte
//@ indique en argument
//@
//@
//@ Parameters :
//@         (ENTREE) fd  : descripteur du fichier ce contexte
//@         (ENTREE/SORTIE)  Ctx : contexte lu
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LireCorpsCtx(
  int fd,
  t_SD_BAS_Contexte * Ctx,
  t_SD_BAS_DataThread *ctx)
{
  static const char tcProcName_F[] = "SD_BAS_LireCorpCtx";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  int cr;
  int i;
  char *msg_err;
  int num_lg;
  
  unsigned short int BufferUnsignedShortInt;
  short int          BufferShortInt;
  unsigned int       BufferUnsignedInt;
  int                BufferInt;
  
  int TailleUnsignedShortInt = 2;
  int TailleShortInt = 2;
  int TailleUnsignedInt = 4;
  int TailleInt = 4;
  int TailleDouble = 8;
  
  /* Lecture des donnees */
  for(i=0;i< Ctx->GBBTNbElt;i++)
  {
// jlp
    //@ lecture de GBBTDateElt
    cr = read(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
	  // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    Ctx->GBBTHistTempCN[i].GBBTDateElt.day = BufferUnsignedShortInt;
    
    //@ lecture de GBBTDateElt
    cr = read(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU,SD_BAS_MSG_READ, __LINE__);
	  // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    Ctx->GBBTHistTempCN[i].GBBTDateElt.millis = BufferUnsignedInt;

    //@ lecture de 
    cr = read(fd, &Ctx->GBBTHistTempCN[i].BIMSBBT, TailleDouble);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
	  // FT-2765
    SWAP_INT_BUFFER(&Ctx->GBBTHistTempCN[i].BIMSBBT,TailleDouble,1);
    // ...FT-2765

    //@ lecture de GBBTMissing
    cr = read(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU,SD_BAS_MSG_READ, __LINE__);
	  // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    Ctx->GBBTHistTempCN[i].GBBTMissing = (int)BufferUnsignedShortInt;
  }

  //@ lecture de GIccRadCalSlopeImag
  cr = read(
    fd, 
    Ctx->GIccRadCalSlopeImag, 
    TailleDouble * SD_BAS_NB_VAL * SD_BAS_IMLI * SD_BAS_IMCO);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
  // FT-2765
  SWAP_INT_BUFFER(Ctx->GIccRadCalSlopeImag,TailleDouble,SD_BAS_NB_VAL*SD_BAS_IMLI*SD_BAS_IMCO);
  // ...FT-2765

  //@ lecture de GIccRadCalOffsetImag
  cr = read(
    fd, 
    Ctx->GIccRadCalOffsetImag, 
    TailleDouble * SD_BAS_NB_VAL *SD_BAS_IMLI * SD_BAS_IMCO );
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
  // FT-2765
  SWAP_INT_BUFFER(Ctx->GIccRadCalOffsetImag,TailleDouble,SD_BAS_NB_VAL*SD_BAS_IMLI*SD_BAS_IMCO);
  // ...FT-2765

  //@ lecture de GIacOffsetIISAVHRRLin
  cr = read(fd, Ctx-> GIacOffsetIISAVHRRLin, SD_BAS_NB_VAL * TailleDouble);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
  // FT-2765
  SWAP_INT_BUFFER(Ctx-> GIacOffsetIISAVHRRLin,TailleDouble,SD_BAS_NB_VAL);
  // ...FT-2765

  //@ lecture de GIacOffsetIISAVHRRCol
  cr = read(fd, Ctx-> GIacOffsetIISAVHRRCol,SD_BAS_NB_VAL * TailleDouble);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
  // FT-2765
  SWAP_INT_BUFFER(Ctx-> GIacOffsetIISAVHRRCol,TailleDouble,SD_BAS_NB_VAL);
  // ...FT-2765

  for(i = 0; i < SD_BAS_NB_VAL *SD_BAS_IMCO * SD_BAS_IMLI; i++)
  {
    //@ lecture de MDptIisInHomPix
    cr = read(fd, &BufferInt, TailleInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
    // FT-2765
    SWAP_INT_BUFFER(&BufferInt,TailleInt,1);
    // ...FT-2765
    Ctx->MDptIisInHomPix[i] = BufferInt;     
  }
  
  for(i = 0; i < SD_BAS_NB_VAL *SD_BAS_IMCO * SD_BAS_IMLI; i++)
  {
    //@ lecture de MDptIisBadHealthPix
    cr = read(fd, &BufferInt, TailleInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
    // FT-2765
    SWAP_INT_BUFFER(&BufferInt,TailleInt,1);
    // ...FT-2765
    Ctx->MDptIisBadHealthPix[i] = BufferInt;
  }
  
  //@ lecture de MDptVarImag
  cr = read(
    fd, 
    &Ctx->MDptVarImag, 
    SD_BAS_NB_VAL * SD_BAS_IMCO * SD_BAS_IMLI * TailleDouble);
  SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
  // FT-2765
  SWAP_INT_BUFFER(&Ctx->MDptVarImag,TailleDouble,SD_BAS_NB_VAL*SD_BAS_IMLI*SD_BAS_IMCO);
  // ...FT-2765
  
  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ lecture de GOPSDatIsrfemOff
    cr = read(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    Ctx->GOPSDatIsrfemOff[i].day = BufferUnsignedShortInt;

    //@ lecture de GOPSDatIsrfemOff
    cr = read(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    Ctx->GOPSDatIsrfemOff[i].millis = BufferUnsignedInt;
  }
  
  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ lecture de GOPSDatBandMiss
    cr = read(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    Ctx->GOPSDatBandMiss[i].day = BufferUnsignedShortInt;

    //@ lecture de GOPSDatBandMiss
    cr = read(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    Ctx->GOPSDatBandMiss[i].millis = BufferUnsignedInt;
  }
  
  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ lecture de GOPSDatBBTMiss
    cr = read(fd,&BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    Ctx->GOPSDatBBTMiss[i].day = BufferUnsignedShortInt;

    //@ lecture de GOPSDatBBTMiss
    cr = read(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    Ctx->GOPSDatBBTMiss[i].millis = BufferUnsignedInt;
  }

  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ lecture de GOPSDatImgEWMiss
    cr = read(fd,&BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    Ctx->GOPSDatImgEWMiss[i].day = BufferUnsignedShortInt;

    //@ lecture de GOPSDatImgEWMiss
    cr = read(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    Ctx->GOPSDatImgEWMiss[i].millis = BufferUnsignedInt;
  }
  
  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ lecture de GOPSDatImgBBMiss
    cr = read(fd,&BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    Ctx->GOPSDatImgBBMiss[i].day = BufferUnsignedShortInt;

    //@ lecture de GOPSDatImgBBMiss
    cr = read(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    Ctx->GOPSDatImgBBMiss[i].millis = BufferUnsignedInt;
  }

  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ lecture de GOPSDatImgCSMiss
    cr = read(fd,&BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    Ctx->GOPSDatImgCSMiss[i].day = BufferUnsignedShortInt;

    //@ lecture de GOPSDatImgCSMiss
    cr = read(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    Ctx->GOPSDatImgCSMiss[i].millis = BufferUnsignedInt;
  }
  
  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ lecture de GOPSDatRadAvhrrMiss
    cr = read(fd,&BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    Ctx->GOPSDatRadAvhrrMiss[i].day = BufferUnsignedShortInt;

    //@ lecture de GOPSDatRadAvhrrMiss
    cr = read(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    Ctx->GOPSDatRadAvhrrMiss[i].millis = BufferUnsignedInt;
  }
  
  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ lecture de GCTXNumGranule
    cr = read(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_READ, __LINE__);
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    Ctx->GCTXNumGranule[i] = (int)BufferUnsignedInt;
    if (Ctx->GCTXNumGranule[i] < 0)
    {
      cr_op = SD_BAS_ERR_VERSION;
      msg_err = SD_BAS_MSG_LIRECONTEXTE;
      num_lg = __LINE__;
      goto fin;
    }
  }

fin:

  if(cr_op != CMN__STATUT_OK)
  {
    /* mise a jour du contexte d'erreur */
    SD_BAS_TEST_ERR_INT(
      ctx->cr.errinterne,
      SD_BAS_ERR_SET_Contexte,
      SD_BAS_ERR_NUM_JDB,
      tcProcName_F,
      num_lg,
      msg_err);
  }

  return(cr_op);
}

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_EcrireCorpsCtx
//@
//@ Description : Ecriture du fichier de contexte
//@ indique en argument
//@
//@
//@ Parameters :
//@         (ENTREE) fd  : descripteur du fichier ce contexte
//@         (ENTREE)  Ctx : contexte lu
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_EcrireCorpsCtx(
  int fd,
  t_SD_BAS_Contexte * Ctx,
  t_SD_BAS_DataThread *ctx)
{
  static const char tcProcName_F[] = "SD_BAS_EcrireCorpCtx";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  int cr;
  int i;
  int num_lg;
  char *msg_err;
  
  unsigned short int BufferUnsignedShortInt;
  short int          BufferShortInt;
  unsigned int       BufferUnsignedInt;
  int                BufferInt;
  // FT-2765
  double             BufferDouble;
  // ...FT-2765
  
  int TailleUnsignedShortInt = 2;
  int TailleShortInt = 2;
  int TailleUnsignedInt = 4;
  int TailleInt = 4;
  int TailleDouble = 8;
  
  for(i=0;i<Ctx->GBBTNbElt;i++)
  {
    //@ ecriture de GBBTDateElt
    BufferUnsignedShortInt = 
      (unsigned short int)Ctx->GBBTHistTempCN[i].GBBTDateElt.day;
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);

    //@ ecriture de GBBTDateElt
    BufferUnsignedInt = 
      (unsigned int)Ctx->GBBTHistTempCN[i].GBBTDateElt.millis;
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);

    //@ ecriture de BIMSBBT
    // FT-2765
    BufferDouble = Ctx->GBBTHistTempCN[i].BIMSBBT;
    SWAP_INT_BUFFER(&BufferDouble,TailleDouble,1);
    cr = write(fd, &BufferDouble, TailleDouble);
    // ...FT-2765
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);

    //@ ecriture de GBBTMissing
    BufferUnsignedShortInt = 
      (unsigned short int)Ctx->GBBTHistTempCN[i].GBBTMissing; /* modulo 65536 */
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }

  //@ ecriture de GIccRadCalSlopeImag
  // FT-2765
  for (i=0; i<(SD_BAS_NB_VAL*SD_BAS_IMCO*SD_BAS_IMLI); i++)
  {
    BufferDouble = Ctx->GIccRadCalSlopeImag[i];
    SWAP_INT_BUFFER(&BufferDouble,TailleDouble,1);
    cr = write(fd, &BufferDouble, TailleDouble);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }
  // ...FT-2765

  //@ ecriture de GIccRadCalOffsetImag
  // FT-2765
  for (i=0; i<(SD_BAS_NB_VAL*SD_BAS_IMCO*SD_BAS_IMLI); i++)
  {
    BufferDouble = Ctx->GIccRadCalOffsetImag[i];
    SWAP_INT_BUFFER(&BufferDouble,TailleDouble,1);
    cr = write( fd, &BufferDouble, TailleDouble);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }
  // ...FT-2765

  //@ ecriture de GIacOffsetIISAVHRRLin
  // FT-2765
  for (i=0; i<SD_BAS_NB_VAL; i++)
  {
    BufferDouble = Ctx->GIacOffsetIISAVHRRLin[i];
    SWAP_INT_BUFFER(&BufferDouble,TailleDouble,1);
    cr = write( fd, &BufferDouble, TailleDouble);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }
  // ...FT-2765

  //@ ecriture de GIacOffsetIISAVHRRCol
  // FT-2765
  for (i=0; i<SD_BAS_NB_VAL; i++)
  {
    BufferDouble = Ctx->GIacOffsetIISAVHRRCol[i];
    SWAP_INT_BUFFER(&BufferDouble,TailleDouble,1);
    cr = write( fd, &BufferDouble, TailleDouble);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }
  // ...FT-2765

  for(i = 0; i < SD_BAS_NB_VAL * SD_BAS_IMCO * SD_BAS_IMLI; i++)
  {
    //@ ecriture de MDptIisInHomPix
    BufferInt = Ctx->MDptIisInHomPix[i];
    // FT-2765
    SWAP_INT_BUFFER(&BufferInt,TailleInt,1);
    // ...FT-2765
    cr = write(fd, &BufferInt, TailleInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }
  
  for(i=0;i< SD_BAS_NB_VAL * SD_BAS_IMCO * SD_BAS_IMLI;i++)
  {
    //@ ecriture de MDptIisBadHealthPix
    BufferInt = Ctx->MDptIisBadHealthPix[i];
    // FT-2765
    SWAP_INT_BUFFER(&BufferInt,TailleInt,1);
    // ...FT-2765
    cr = write(fd, &BufferInt, TailleInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }

  //@ ecriture de MDptVarImag
  // FT-2765
  for (i=0; i<(SD_BAS_NB_VAL*SD_BAS_IMCO*SD_BAS_IMLI); i++)
  {
    BufferDouble = Ctx->MDptVarImag[i];
    SWAP_INT_BUFFER(&BufferDouble,TailleDouble,1);
    cr = write( fd, &BufferDouble, TailleDouble);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }
  // ...FT-2765

  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ ecriture de GOPSDatIsrfemOff
    BufferUnsignedShortInt = (unsigned short int )Ctx->GOPSDatIsrfemOff[i].day;
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
    
    //@ ecriture de GOPSDatIsrfemOff
    BufferUnsignedInt=(unsigned int)Ctx->GOPSDatIsrfemOff[i].millis;
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedInt,TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }

  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ ecriture de GOPSDatBandMiss
    BufferUnsignedShortInt = (unsigned short int)Ctx->GOPSDatBandMiss[i].day;
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
    
    //@ ecriture de GOPSDatBandMiss
    BufferUnsignedInt = (unsigned int)Ctx->GOPSDatBandMiss[i].millis;
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }

  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ ecriture de GOPSDatBBTMiss
    BufferUnsignedShortInt = (unsigned short int)Ctx->GOPSDatBBTMiss[i].day;
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
    
    //@ ecriture de GOPSDatBBTMiss
    BufferUnsignedInt = (unsigned int)Ctx->GOPSDatBBTMiss[i].millis;
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }

  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ ecriture de GOPSDatImgEWMiss
    BufferUnsignedShortInt = (unsigned short int)Ctx->GOPSDatImgEWMiss[i].day;
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
    
    //@ ecriture de GOPSDatImgEWMiss
    BufferUnsignedInt = (unsigned int)Ctx->GOPSDatImgEWMiss[i].millis;
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }

  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ ecriture de GOPSDatImgBBMiss
    BufferUnsignedShortInt = (unsigned short int)Ctx->GOPSDatImgBBMiss[i].day;
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
    
    //@ ecriture de GOPSDatImgBBMiss
    BufferUnsignedInt = (unsigned int)Ctx->GOPSDatImgBBMiss[i].millis;
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }

  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ ecriture de  GOPSDatImgCSMiss
    BufferUnsignedShortInt = (unsigned short int)Ctx->GOPSDatImgCSMiss[i].day;
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
    
    //@ ecriture de  GOPSDatImgCSMiss
    BufferUnsignedInt = (unsigned int)Ctx->GOPSDatImgCSMiss[i].millis;
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }

  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ ecriture de  GOPSDatRadAvhrrMiss
    BufferUnsignedShortInt = 
      (unsigned short int)Ctx->GOPSDatRadAvhrrMiss[i].day;
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedShortInt,TailleUnsignedShortInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedShortInt, TailleUnsignedShortInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
    
    //@ ecriture de  GOPSDatRadAvhrrMiss
    BufferUnsignedInt = (unsigned int)Ctx->GOPSDatRadAvhrrMiss[i].millis;
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }

  for(i=0;i<SD_BAS_NB_VAL;i++)
  {
    //@ ecriture de  GCTXNumGranule
    BufferUnsignedInt = (unsigned int)Ctx->GCTXNumGranule[i];
    // FT-2765
    SWAP_INT_BUFFER(&BufferUnsignedInt,TailleUnsignedInt,1);
    // ...FT-2765
    cr = write(fd, &BufferUnsignedInt, TailleUnsignedInt);
    SD_BAS_ERREUR(cr, SD_BAS_ERR_BAS_NIVEAU, SD_BAS_MSG_WRITE, __LINE__);
  }

fin:

  if(cr_op != CMN__STATUT_OK)
  {
    /* mise a jour du contexte d'erreur */
    SD_BAS_TEST_ERR_INT(
      ctx->cr.errinterne,
      SD_BAS_ERR_SET_Contexte,
      SD_BAS_ERR_NUM_JDB,
      tcProcName_F,
      num_lg,
      msg_err);
  }

  return(cr_op);
}
