// $header: $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_BAS
//@ MODULE : SD_BAS_Conversion
//@
//@ Comment :
//@    Code file for the  SD_BAS_Conversion class which encapsulates the
//@    conversion facility for IASI OPS  :
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include "SD_BAS_Conversion.h"
// FT-2765
#include "SD_BAS_SwapEndian.h"
// ...FT-2765

/*====================================================================*/
// Local Constant Declaration 
/*====================================================================*/
/*====================================================================*/
// Local types Declaration  
/*====================================================================*/
//======================================================================
// Internal fonctions Declaration
//======================================================================
//======================================================================
// Global variables Definition
//======================================================================


//======================================================================
//					FONCTIONS EXTERNES
//======================================================================

// FT-2765
//@ ------------------------------------------------------------------*/
//@ Procedure :  SD_BAS_Is_BitSeti
//@
//@ Description : Retourne l'etat d'un bit dans un mot de 16 bits
//@               cette fonction s'applique au mots de niveau 0
//@
//@ Parameters :   word : mot sr 16 bits
//@                bit : numero du bit a extraire 0 -> 15
//@
//@
//@ Return Status :
//@                  1 si le bit est a 1
//@                  0 si le bit est a 0
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_Is_BitSeti( unsigned short word,int bit)
{
  unsigned short mask=1;
  int i;
  unsigned short val;

  // calcul du masque
  for(i=0;i<(15-bit);i++)
    mask = 2* mask;

 //on aplique la masque
  val = (word & mask);

  /* valeur de retour */
  if (val == 0 )
    /* le bit est a 0 */
    return(0);
  else
    /* le bit est a 1 */
    return(1);
}
// ...FT-2765

//@ ------------------------------------------------------------------*/
//@ Procedure :  SD_BAS_Is_BitSet
//@
//@ Description : Retourne l'etat d'un bit dans un mot de 16 bits
//@               cette fonction s'applique au mots de niveau 0 
//@               
//@ Parameters :   word : mot sr 16 bits
//@                bit : numero du bit a extraire 0 -> 15
//@		 
//@
//@ Return Status : 
//@                  1 si le bit est a 1
//@                  0 si le bit est a 0
//@
//@ ------------------------------------------------------------------*/ 
t_CMN__Statut SD_BAS_Is_BitSet( unsigned short word,int bit)
{
  unsigned short mask=1;
  int i;
  unsigned short val;

  // FT-2765
  SWAP_INT_BUFFER( &word, 2, 1 );
  // ...FT-2765
 
  // calcul du masque 
  for(i=0;i<(15-bit);i++) 
    mask = 2* mask;

 //on aplique la masque
  val = (word & mask);
 
  /* valeur de retour */
  if (val == 0 )
    /* le bit est a 0 */
    return(0);
  else
    /* le bit est a 1 */
    return(1);
}

#ifdef ffff
//@ ------------------------------------------------------------------*/
//@ Procedure :  SD_BAS_IsBitSet
//@
//@ Description : Renvoie la valeur du nombre represente par les bits
//@               bit1 a bit2 d'un mot de 16 bits
//@               La numerotation des bits est la suivante :
//@                    B0 (MSB) ..... B15 (LSB)
//@               Exemple : word = 0001100110101101 (=6573 en decimal)
//@                         bit1 = 5
//@                         bit2 = 12
//@                    =>  *val  = 00110101000 (= 53 en decimal)
//@
//@ Parameters :
//@     (ENTREE)  word : mot de 16 bits dont on veut extraire une valeur
//@     (ENTREE)  bit1 : indice du bit de poids fort de la valeur a extraire
//@     (ENTREE)  bit2 : indice du bit de poids faible de la valeur a extraire
//@     (SORTIE)  val  : valeur extraite
//@
//@ Return Status : 
//@                  CMN__STATUT_OK  si aucune erreur
//@                  CMN__STATUT_NOK si erreur rencontree
//@
//@ ------------------------------------------------------------------*/ 

t_CMN__Statut SD_BAS_ExtraireValeur(
  const unsigned short  word,
  const unsigned short  bit1,
  const unsigned short  bit2,
        unsigned short* val
  )
{
  unsigned short word_decale;
  t_CMN__Statut retour;
 
  /* on verifie d'abord si les parametres d'entree sont dans la bonne
     plage de valeurs */
  if ( (bit1 > bit2) || (bit1 < 0) || (bit1 > 15) || (bit2 < 0) || (bit2 > 15) )
  {
    *val   = 0;
    retour = CMN__STATUT_NOK;
  }
  else
  {
    /* on commence par decaler le mot d'entree vers la droite */
    word_decale = word >> (15 - bit2);
    /* on fait ensuite un & logique avec un masque pour ne garder que
       les bits de poids faible */
    *val        = word_decale & ((1 << (bit2 - bit1 + 1)) - 1);
    /* retour OK */
    retour      = CMN__STATUT_OK;
  }

  return(retour);
}
#endif
//@ ------------------------------------------------------------------*/
//@ Procedure :  SD_BAS_IsBitSet
//@
//@ Description : Renvoie la valeur du nombre represente par les bits
//@               bit1 a bit2 d'un mot de 16 bits
//@               La numerotation des bits est la suivante :
//@                    B0 (MSB) ..... B15 (LSB)
//@               Exemple : word = 0001100110101101 (=6573 en decimal)
//@                         bit1 = 5
//@                         bit2 = 12
//@                    =>  *val  = 00110101000 (= 53 en decimal)
//@
//@ Parameters :
//@     (ENTREE)  word : mot de 16 bits dont on veut extraire une valeur
//@     (ENTREE)  bit1 : indice du bit de poids fort de la valeur a extraire
//@     (ENTREE)  bit2 : indice du bit de poids faible de la valeur a extraire
//@     (SORTIE)  val  : valeur extraite
//@
//@ Return Status : 
//@                  CMN__STATUT_OK  si aucune erreur
//@                  CMN__STATUT_NOK si erreur rencontree
//@
//@ ------------------------------------------------------------------*/ 
t_CMN__Statut SD_BAS_ExtraireValeur (
  const unsigned word,
  const unsigned short  bit1,
  const unsigned short  bit2,
        unsigned * val
  )
{

  t_CMN__Statut retour;
   unsigned  word_decale;
 

  
  /* on verifie d'abord si les parametres d'entree sont dans la bonne
     plage de valeurs */
  // FT-2765
#ifdef AIX
  if ( (bit1 > bit2) || (bit1 < 0) || (bit1 > 31) || (bit2 < 0) || (bit2 > 31) )
#else // LINUX
  if ( (bit1 > bit2)  /* || (bit1 < 0) */ || (bit1 > 31) /* || (bit2 < 0) */ || (bit2 > 31) )
#endif
  // ...FT-2765
  {
    *val   = 0;
    retour = CMN__STATUT_NOK;
  }
  else
  {
    /* on commence par decaler le mot d'entree vers la droite */    

    word_decale = word >> bit1;

  /* on fait ensuite un & logique avec un masque pour ne garder que
       les bits de poids faible */
    *val        = word_decale & ((1 << (bit2 - bit1 +1)) - 1);

   /* retour OK */
    retour      = CMN__STATUT_OK;
  }
  return(retour);
}
