/*$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/BAS/src/SD_BAS_DECOM_lex.l,v 1.3 2011/11/25 10:17:45 dpauly Exp $*/
%{
/*@ ------------------------------------------------------------------*/
/*@ Composant : SD */
/*@ Sous-composant : SD_BAS */
/*@ MODULE : SD_BAS_DECOM */
/*@ */
/*@ Comment : */
/*@    lex code file for the  SD_BAS_DECOM class which includes the */
/*@    commutation/ decommutation rules   */
/*@ */
/*@ ------------------------------------------------------------------*/
/*@ HISTORIQUE */
/*@ VERSION : jj/mm/aaaa : [reference :] Auteur */
/*@ <texte libre permettant de commenter le pourquoi et le comment  */
/*@ <version du fichier> */
/*@ */
/*@ FA-ID :<N> : <date> : <commentaire libre> */
/*@ <eventuellement sur plusieurs lignes> */
/*@ FA-ID :<N> : <date> : <commentaire libre> */
/*@ <eventuellement sur plusieurs lignes> */
/*@ DM-ID :<N> : <date> : <commentaire libre> */
/*@ <eventuellement sur plusieurs lignes> */
/*@ */
/*@FIN-HISTORIQUE */
/*@ ------------------------------------------------------------------*/
/*****************************************************************************/
/*                                  EURIDIS                                  */
/*                         REALISATION PAR SYSECA SA                         */
/*****************************************************************************/
/* SYSTEME           :  Services Communs                                     */
/* NOM DU FICHIER    : DECOM_lex.l                                                   */
/* OBJET HOOD        : DECOM_DECOMMUTATION                                   */
/* NOM ABREGE        : DECOM                                                 */
/* TYPE DU FICHIER   : INTERNAL                                              */
/* LANGAGE           : lex                                                   */
/* AUTEUR            : A.MONTMORY                                            */
/* DATE DE CREATION  : 05/16/97                                                   */
/*****************************************************************************/
/* MACHINE CIBLE     : SUN                                                   */
/* OS UNIX           : SOLARIS 2.5                                           */
/* COMPILATEUR       : xlc                                                   */
/* LOGICIELS DE BASE :                                                       */
/* PROGICIELS        :                                                       */
/*****************************************************************************/
/* VERSION           : 1.4                                                   */
/* DATE VERSION      : 97/05/16                                                   */
/* AUTEUR VERSION    :                                                       */
/*****************************************************************************/
/* MODIFICATIONS     :                                                       */
/*****************************************************************************/


/* FIN CARTOUCHE */

/*****************************************************************************/
/* Defintion des tokens du langage de description des blocs.                 */
/*****************************************************************************/
/* ROLE      : Permet de charger la description de tous les blocs . Cette    */
/*           : description est defini dans un fichier ASCII dont le format   */
/*           : est le suivant :                                              */
/*           : VERSION   <NUM_VERSION>                                       */
/*           : [ BLOC BRUT                                                   */
/*           : NOM  <NOM_BLOC>                                               */
/*           : LG      <LONGUEUR_BLOC>                                       */
/*           : INIT     <VAL_INIT>                                           */
/*           : VAR                                                           */
/*           : NOM      <NOM_VARIABLE>                                       */
/*           : OFFSET <OFFSET>                                               */
/*           : LG          <LONGUEUR_VAR>                                    */
/*           : DECOM_CONV <TYPE_CONV>                                        */
/*           : PARAM_DECOM_CONV [  <PARAM_CONV>]*                            */
/*           : RECOM_CONV <TYPE_CONV>                                        */
/*           : PARAM_RECOM_CONV [  <PARAM_CONV>]*                            */
/*           : FINVAR                                                        */
/*           : FINBLOC ]*                                                    */
/*           : [ BLOC ASCII                                                  */
/*           : SEP (<SEPARATEUR>)                                            */
/*           : INIT     <VAL_INIT>                                           */
/*           : VAR                                                           */
/*           : NOM      <NOM_VARIABLE>                                       */
/*           : RANG      <RANG_VARIABLE>                                     */
/*           : DECOM_CONV <TYPE_CONV>                                        */
/*           : PARAM_DECOM_CONV [  <PARAM_CONV>]*                            */
/*           : RECOM_CONV <TYPE_CONV>                                        */
/*           : PARAM_RECOM_CONV [  <PARAM_CONV>]*                            */
/*           : FINVAR                                                        */
/*           : FINBLOC]*                                                     */
/*           : Les informations utiles sont entre <>                         */
/*           : Les mots cles ne sont pas entre <>                            */
/*           : Toute information entre [ ] et suivi de * peut apparaitre     */
/*           : en n exemplaire                                               */
/*           : ( n>= 0)                                                      */
/*           : <NUM_VERSION>     : numero de version du fichier              */
/*           : <NOM_BLOC>        : nom du bloc                               */
/*           : <LONGUEUR_BLOC>   : longueur du bloc en octets                */
/*           : <VAL_INIT>        : valeur d'initialisation des octets du     */
/*           :  bloc (en hexa)                                               */
/*           : <NOM_VARIABLE>    : nom d'une variable                        */
/*           : <OFFSET>          : deplacement en bits par rapport au        */
/*           :                     debut du bloc                             */
/*           : <LONGUEUR_VAR>    : longueur en bits des donnees dans le bloc */
/*           : <TYPE_CONV>       : Type de conversion                        */
/*           : <PARAM_CONV>      : Parametres de la conversion (entiers)     */
/*           : <SEPARATEUR>      : Caractere separateur de champs dans un    */
/*           :                     bloc ASCII                                */
/*           : <RANG_VARIABLE>   : Rang d'une variable dans un bloc ASCII .  */
/*           :                     La premiere variable commence au rang 0.  */
/*****************************************************************************/
%}

%%

"VERSION"       { /* Indentification de la version interne du fichier.       */
                  return T_VERSION; 
                }
"BLOC"		{ /* Indentification du token "BLOC" qui permet de definir   */
                  /* le debut d'une section bloc.                            */
			return T_BLOC;
		}
"BINAIRE"	{ /* Indentification du token "BINAIRE" qui caracterise un      */
		  /* bloc binaire.                                           */
			return T_BRUT;
		}
"ASCII"		{ /* Indentification du token "ASCII" qui caracterise un     */
		  /* bloc dont les variables sont codees en ASCII.           */
			return T_ASCII;
		}
"NOM"		{ /* Indentification du token "NOM" qui permet de definir    */
                  /* un nom de bloc ou de variable.                          */
			return T_NOM;
		}
"LG"		{ /* Indentification du token "LG" qui permet de definir     */
                  /* la longueur en octet d'un bloc ou d'une variable        */
                  /* (en bit).                                               */
			return T_LG;
		}
"INIT"		{ /* Indentification du token "INIT" qui permet de definir   */
                  /* le pattern de remplissage d'un bloc.                    */
			return T_INIT;
		}
"VAR"		{ /* Indentification du token "VAR" qui permet de definir    */
                  /* le debut d'une section variable.                        */
			return T_VAR;
		}
"OFFSET"	{ /* Indentification du token "OFFSET" qui permet de definir */
                  /* l'offset en bit d'une variable par rapport au debut     */
                  /* du bloc.                                                */
			return T_OFFSET;
		}
"BIT"	{ /* Indentification du token "BIT" qui permet de definir */
                  /* l'unite en bit d'une variable par rapport au debut     */
                  /* du bloc.                                                */
			return T_BIT;
		}
"DIM"	        { /* Indentification du token "OFFSET" qui permet de definir */
                  /* l'offset en bit d'une variable par rapport au debut     */
                  /* du bloc.                                                */
			return T_DIM;
		}

"CONV"	        { /* Indentification du token "CONV" qui permet de     */
                  /* definir la fonction de conversion pour la               */
                  /* decommutation d'une varaible.                           */
			return T_CONV;
		}
"PARAM"	        { /* Indentification du token "PARAM" qui */
                  /* permet de definir la liste des parametres de la         */
                  /* fonction de decommutation.                              */
			return T_PARAM_CONV;
		}
"RANG"	        { /* Indentification du token "RANG" qui permet de           */
                  /* definir la position d'une variable dans un bloc ascii.  */
			return T_RANG;
		}
"SEP"		{ /* Indentification du token "SEP" qui permet de            */
                  /* definir le separateur dans un bloc ascii.               */
                  	return T_SEP;
    		}
'[ ,;_=]'|'\\0'        { /* separateur possible ',' ou ' ' ou ';' ou '_' ou '=' */
                        if (strcmp(yytext,"'\\0'") == 0) {
                          yylval.sep = '\0';
                        } else {
                          yylval.sep = yytext[1];
                        }
			return T_TYPE_SEP;
		}   		
"FINVAR"	{ /* Indentification du token "FINVAR" qui permet de definir  */
                  /* la fin d'une section variable.                           */
			return T_FINVAR;
		}
"FINBLOC"	{ /* Indentification du token "FINBLOC" qui permet de definir */
                  /* la fin d'une section bloc.                               */
			return T_FINBLOC;
		}
[a-zA-Z_][a-zA-Z0-9_]*   { /* Indentificateur (nom de variable ou de bloc)             */
			strcpy (yylval.sval,yytext);
			return T_IDENT;
		}
-?[0-9]+        { /* les nombres entiers.                                     */
		        yylval.ival = TYCST__STRTOINT4 (yytext);
			return T_INT;
		}
-?[0-9]*\.[0-9]+ { /* les nombres flottants.                                  */
			yylval.fval = TYCST__STRTOREAL8 (yytext);
			return T_FLOAT;
		}
-?[0-9]*\.?[0-9]+[eE]-?[0-9]+ { /* les nombres flottants au format x.xxxxE-xx.                                  */
			yylval.fval = TYCST__STRTOREAL8 (yytext);
			return T_FLOAT;
		}
#.*[\n]         { /* le commentaire sur une ligne qui commence par #*/
		   /* return T_COMMENT */;
		}
%%
