//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/BAS/src/SD_BAS_Global.h,v 1.5 2013/03/29 16:55:26 lcaspar Exp $
//@ --------------------------------------------------------------------
//@ Component : SD_BAS
//@ MODULE : SD_BAS_Global
//@
//@ Comment :
//@    include file used by all the class of the SD jlp
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 04/09/2003 : [reference :] Benjamin MASSART
//@ cette version prend en compte les evolutions du code pour une gestion
//@ dynamique de l'ecriture des IPR dans les produits de niveau 1 suite
//@ aux modifications apparues dans la version 6.4 du GPFS et la version
//@ 6.5 du IASI Level 1 Product Format Specification
//@ --------------------------------------------------------------------
//@ DM-ID : PM-3 : 04/09/2003 :
//@ definition du nombre max d'IPR a ecrire dans un produit de niveau 1
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-066-CN : 7/11/2003 : JL.PASCAL
//@ le spectre est vide dans le cas de lignes dummy a cause d'un mauvais indicage des lignes
//@ le nombre de MDR non dummy est calcule dans  SD_GES_FiltrageAxeInterf
//@ et utilise dans SD_GES_ChaineProdxx_xx
//@ (cf IA-2100-FC-066-THA)
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-084-CN : 13/11/2003 : JL. PASCAL
//@ optimisation du code ( les initialisation de FFT sont faite une fois par ligne );
//@ fiche de correction correspondante : IA-2100-FC-084-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-059-CN : 13/11/2003 : B. MASSART
//@ definition of number max of positions of underflows 
//@ and overflows written in the engineering product. See IA-2100-FC-059-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-080-CN : 18/11/2003 : JL.PASCAL
//@ lors du mode calibration externe en visee NADIR un cas CG3 est leve de maniere incorrecte
//@ le pb est du a l'iddentification de la visee nadir (SP =34)
//@ un enumere est ajoute  SD_BAS_SP_EARTH_NADIR
//@ fiche de correction correspondante : IA-2100-FC-080-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-111-CN : 09/02/2004 : B. MASSART
//@ ajout des noms de fichier tronques (15 premiers caracteres) pour les 
//@ produits de niveau 1 pour qu'ils soient traces dans le fichier report
//@ --------------------------------------------------------------------
//@ DM-ID : PM4 : 31/03/2004 : B. MASSART
//@ -ajout SD_BAS_SNREFDATELIGNE
//@ -ajour enumere e_SD_BAS_GEPSIdConf_FLAG
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-128-CN : 07/08/2004 : B. MASSART
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-133-CN : 05/08/2004 : B. MASSART
//@ correction of N1C_GGEOIISLOC index in Corpstraitement 
//@ to store ENG_CGEOSUBSATELLITEPOSITION. See IA-2100-FC-133-CN.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-158-CN : 30/06/2005 : JL PASCAL
//@ L'OPS genere des resulta VIADRENG_MEXSSMIN et MAX errone et qui  
//@ dependent du nombre de thread 
//@ Voir IA-2100-FC-158-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-271-CN : 22/06/2005 : JL.PASCAL
//@ prise en compte du mode transition
//@ fiche de correction correspondante : IA-2100-CR-271-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-171-CN : 09/03/2006 : B. MASSART
//@ Spacecraft ID is defined as an environment variable. Processing level in ENG
//@ and VER file name and MPHR is set to 01. See IA-2100-FA-171-THA.
//@ --------------------------------------------------------------------
//@ PM-ID : IA-2100-CR-292-CNE : 10/03/2006 : B.MASSART
//@ Function SD_ALG_AVHRR_GetSubSatLatLon is added to set up 
//@ MPHR_SUBSAT_LATITUDE/LONGITUDE_START/END fields in Level1 products.
//@ See IA-2100-CR-292-THA.
//@ --------------------------------------------------------------------
//@ PM-ID : IA-2100-CR-290-CNE : 30/03/2006 : B. GUENDE
//@ Comments updated.
//@ See IA-2100-CR-290-THA
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-193-CN : 9/11/2006 : R.MORENO
//@ Levee de CG pour les images associees aux visees da calibration CS
//@ et BB
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2521(DM-2100-OPS) : 27/04/2009 : S. BRUNEL
//@ Calcul de 4 pseudo-canaux AVHRR (Day 2, point 2)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2939(DM-2100-OPS) : 12/12/2011 : L. CASPAR
//@ Mise en place de conditions de levee du cas degarde 9 alternatives pour 
//@ correction de la NCR 12303
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef  SD_BAS_Global_h
#define  SD_BAS_Global_h


/* librairies C de bases utilisees par les modules du projet */
#include <stdio.h>       /* entree/sortie */
#include <stdlib.h>      /* alloc memoire, ...*/
#include <string.h>      /* chaines de carateres */
#include <unistd.h>      /* gestion pipes, ... */
#include <time.h>        /* gestion de la date */
#include <sys/param.h>   /* constantes systeme */
#include "SD_BAS_OPS_Essl.h"
// FT-2765
#include "SD_BAS_AUXLibESSL.h"
// ...FT-2765

#include "CMN__Commun.h"
#include "ERR__Erreur.h"
#include "CMN__Date.h"
#include "SD_BAS_CompteRendu.h"

#define TRUE 1
#define FALSE 0

/*====================================================================*/
/*   External define  Declarations */
/*====================================================================*/
//--------------------------------------------------------------------*/
//* Macros permettant d'acceder aux elements d'un tableau en utilisant */
//* un mode de rangement de type fortran ( indices commencant a 1 et
//* variants de la gauche vers le droite */
//--------------------------------------------------------------------*/
// jlp


// FT-2765
#ifdef OPS_CHECK_IND

static int ops_check_ind( const char *file, int line ) {
  printf( "Index out of bounds at %s:%d\n", file, line );
  return 0;
}

#define _ops_check0( i ) ( ( i ) >= 1 )
#define _ops_check1( i, dimi ) ( ( ( i ) >= 1 ) && ( ( i ) <= ( dimi ) ) )

#define IND1(i) ( _ops_check0( i ) \
              ? (i-1)            \
              : ops_check_ind( __FILE__, __LINE__ ) )

#define IND2(i,j,dimi) ( _ops_check1( i, dimi ) && _ops_check0( j ) \
              ? ((j-1)*(dimi)+(i-1))                       \
                     : ops_check_ind( __FILE__, __LINE__ ) )

#define IND3(i,j,k,dimi,dimj) ( _ops_check1( i, dimi ) && _ops_check1( j, dimj ) && _ops_check0( k ) \
                            ? ((k-1)*(dimi)*(dimj)+(j-1)*(dimi)+(i-1))                             \
                            : ops_check_ind( __FILE__, __LINE__ ) )

#define IND4(i,j,k,l,dimi,dimj,dimk) ( _ops_check1( i, dimi ) && _ops_check1( j, dimj ) &&                 \
                                     _ops_check1( k, dimk ) && _ops_check0( l )                          \
                                   ? ((l-1)*(dimi)*(dimj)*(dimk)+(k-1)*(dimi)*(dimj)+(j-1)*(dimi)+(i-1)) \
                                   : ops_check_ind( __FILE__, __LINE__ ) )

#define IND5(i,j,k,l,m,dimi,dimj,dimk,diml) ( _ops_check1( i, dimi ) && _ops_check1( j, dimj ) &&                  \
                                            _ops_check1( k, dimk ) && _ops_check1( l, diml ) && _ops_check0( m ) \
                                          ? ((m-1)*(dimi)*(dimj)*(dimk)*(diml)+(l-1)*(dimi)*(dimj)*(dimk)+(k-1)* \
                                            (dimi)*(dimj)+(j-1)*(dimi) + (i-1))                                  \
                                          : ops_check_ind( __FILE__, __LINE__ ) )

#define IND6(i,j,k,l,m,n,dimi,dimj,dimk,diml,dimm) ( _ops_check1( i, dimi ) && _ops_check1( j, dimj ) && _ops_check1( k, dimk ) && \
                                                   _ops_check1( l, diml ) && _ops_check1( m, dimm ) && _ops_check0( n )          \
                                                 ? ((n-1)*(dimi)*(dimj)*(dimk)*(diml)*(dimm)+(m-1)*(dimi)*(dimj)*(dimk)*(diml)+  \
                                                   (l-1)*(dimi)*(dimj)*(dimk)+(k-1)*(dimi)*(dimj) + (j-1)*(dimi) + (i-1))        \
                                                 : ops_check_ind( __FILE__, __LINE__ ) )


#else
#define IND1(i) (i-1)
#define IND2(i,j,dimi) ((j-1)*(dimi)+(i-1))
#define IND3(i,j,k,dimi,dimj) ((k-1)*(dimi)*(dimj)+(j-1)*(dimi)+(i-1))
#define IND4(i,j,k,l,dimi,dimj,dimk) ((l-1)*(dimi)*(dimj)*(dimk)+(k-1)*(dimi)*(dimj)+(j-1)*(dimi)+(i-1))
#define IND5(i,j,k,l,m,dimi,dimj,dimk,diml) ((m-1)*(dimi)*(dimj)*(dimk)*(diml)+(l-1)*(dimi)*(dimj)*(dimk)+(k-1)*(dimi)*(dimj)+(j-1)*(dimi) + (i-1))
#define IND6(i,j,k,l,m,n,dimi,dimj,dimk,diml,dimm) ((n-1)*(dimi)*(dimj)*(dimk)*(diml)*(dimm)+(m-1)*(dimi)*(dimj)*(dimk)*(diml)+(l-1)*(dimi)*(dimj)*(dimk)+(k-1)*(dimi)*(dimj) + (j-1)*(dimi) + (i-1))
#endif
// ...FT-2765


//--------------------------------------------------------------------*/
/* Macro permettant l'appel a malloc avec gestion d'erreur */
/* La macro a ete modifiee afin d'initialiser avec des octets = 0 la zone allouee */
//--------------------------------------------------------------------*/

#define SD_BAS_MacMALLOC(p,T,n)   ( (p = (T*)calloc((size_t)n,(size_t)(sizeof(T))))? \
                 TRUE : FALSE)


//--------------------------------------------------------------------*/
/* Macro permettant l'appel a realloc avec gestion d'erreur */
//--------------------------------------------------------------------*/

#define SD_BAS_MacREALLOC(p,T,n)  ( (p = (T*)realloc((void *)p, (size_t)(sizeof(T)*(n))))? \
                         TRUE : FALSE)

//--------------------------------------------------------------------*/
/* Macro permettant l'appel a free avec reinitialisation du pointeur */
//--------------------------------------------------------------------*/

#define SD_BAS_MacFREE(p) { if (p) {free((void *)(p)); p = NULL;} }



/*--------------------------------------------------------------------*/
/* Macro de test d'egalite relative de doubles a une precision pres   */
/* Utilisation :                                                      */
/*       if (SD_BAS_MacEGAUX_D(reel1,reel2) == TRUE)                  */
/*     {                                                              */
/*     ...                                                            */
/*     }                                                              */
/* SD_BAS_PRECISION_MACHINE est la puissance immediatement superieure */
/* a 2^MACHEP ou MACHEP est le plus petit exposant tel que 2^MACHEP   */
/* ajoute a 1.0 est different de 1.0 (cf Numerical Recipes)           */
/* Conformement a la MP du CNES RNC-CNES-Q-80-518, on definit         */
/* la precision relative de calcul scientifique en multipliant        */
/* la precision machine par un facteur d'echelle > 1                  */
/*--------------------------------------------------------------------*/

static const double SD_BAS_PRECISION_MACHINE = 1.0E-15; /* pour les doubles */
static const double SD_BAS_ECHELLE = 1.0E1;

#define SD_BAS_MacEGAUX_D(reel1,reel2)   \
     ( fabs((reel1) - (reel2)) <= \
       SD_BAS_ECHELLE * SD_BAS_PRECISION_MACHINE * fabs(reel2) )


//--------------------------------------------------------------------*/
/* constantes globales a IASI */
//--------------------------------------------------------------------*/
static const int SD_BAS_B1 = 0;         // indice de la bande spectrale B1
static const int SD_BAS_B2 = 1;         // indice de la bande spectrale B2
static const int SD_BAS_B3 = 2;         // indice de la bande spectrale B3
#define SD_BAS_SB 3               // number of spectral band
#define SD_BAS_SN 37              // step number

#define SD_BAS_SN_IP_SUP_1  32    // SN IP
#define SD_BAS_SN_IP_SUP_2  33    // SN IP
#define SD_BAS_SN_IP_SUP_3  35    // SN IP
#define SD_BAS_SN_IP_SUP_4  36    // SN IP

// FA 193
#define SD_BAS_SN_IP_SUP_BB SD_BAS_SN_IP_SUP_2  // SN de l'IP BB a utiliser (les 4 descendent)
#define SD_BAS_SN_IP_SUP_CS SD_BAS_SN_IP_SUP_4  // SN de l'IP CS a utiliser (les 4 descendent)
//..FA 193

#define SD_BAS_SP           37    // scan position
#define SD_BAS_SP_EARTH_MIN  1
#define SD_BAS_SP_EARTH_MAX 30
#define SD_BAS_SP_EARTH_NADIR 34

// FT-2521
#define SD_BAS_PS 4               // nombre de pseudo-canaux
// ...FT-2521

#define SD_BAS_PN 4               // pixel number

static const double SD_BAS_DUREE_LGN = 8.;  // duree d'une ligne IASI (en s)
#define SD_BAS_SNOT 30            // observationnal target in normal mode
#define SD_BAS_SNOT_BIS  (SD_BAS_SNOT + 4)
#define SD_BAS_CALIB_EXT_SN_MIN 3 // observationnal target in calib mode
#define SD_BAS_CALIB_EXT_SN_MAX 29
#define SD_BAS_NOBT (double)(4294967296.0) // number of OBT values (2^32=4294967296)
#define SD_BAS_MAXBA 3600         // number of samples in one IASI band
#define SD_BAS_SS0 3600*3         // number of samples in IASI 0 spectra
#define SD_BAS_SS1 8500           //  number of sample in  IASI 1A1B1C spectra )
#define SD_BAS_SS  8700           //  number of sample in an IASI spectrum
#define SD_BAS_MAXECH_ISFREM 150  // maximum number IASI spectra ( 1B 1C)
#define SD_BAS_IMLI 64            // number of line for IASI imager pixel
#define SD_BAS_IMCO 64            // number of column for IASI imager pixel
#define SD_BAS_AMLI 100           // number of line for AVHRR imager pixel
#define SD_BAS_AMCO 100           // number of column for AVHRR imager pi
#define SD_BAS_CCD 2              // number of Corner Cube Direction
#define SD_BAS_NCL 7              // number of class for FOV sounder Analysis
#define SD_BAS_NCL_MAX 20         // number of class for FOV MAX sounder Analysis
#define SD_BAS_NBK 6              // number of AVHRR channel
#define SD_BAS_TAILLE_NB_SI_B1 8  // number of Spectral Intervals for B1
#define SD_BAS_TAILLE_NB_SI_B2 10 // number of Spectral Intervals for B2
#define SD_BAS_TAILLE_NB_SI_B3 10 // number of Spectral Intervals for B3
#define SD_BAS_TAILLE_NB_SI    27 // total number of Spectral Intervals
#define SD_BAS_SGI 5*5            // number of pixels of the subgrid of the imager
#define SD_BAS_NLI_MAX_UTIL_AVHRR 160     // number of lines in one strip AVHRR for one line IASI
#define SD_BAS_NPI_AVHRR 2048     // number of pixel in one band AVHRR
#define SD_BAS_NBA_AVHRR 5        // number of AVHRRband
#define SD_BAS_NIM 28             // Number of  samples used to represent the imaginary part of the IASI spectrum
#define SD_BAS_CTN 4              // number of spectral coding table 

#define SD_BAS_TAILLE_NB_SI_B 10 // number of Spectral sample
#define SD_BAS_DIM_ROT_MATRICE 3
#define SD_BAS_MODEL           200 //number of model value
#define SD_BAS_SGI_LI 5 // number of line of the subgrid of the imager
#define SD_BAS_SGI_CO 5// number of column of the subgrid of the imager

/* FA133 */
#define SD_BAS_SGI_CTR_INDEX 12 //pixel number of the SGI center (start at 0)

// FT-2939
#define SD_BAS_DELTA_TIME_MAX_NIGHT_TRANSITION_CG9 30 // delta time in seconds around night=>day or day=>night transition for clouds coverage   
// ..FT-2939

#define SD_BAS_AVHRR_NOISE 20
#define SD_BAS_AVHRR_LINE_PERIOD (1000./6.) // period of a AVHRR line in milliseconds
#define SD_BAS_SPCT_COD 550 //numner od on_board spectra coding
#define SD_BAS_UDR_SPLG_COD 16.0 // under sampling factor of on_board spectra coding
#define SD_BAS_COVAR_DIM 100

#define SD_BAS_IIS_MAX 100
#define SD_BAS_NB_COL_SOND 100
#define SD_BAS_NB_LIN_SOND 100
#define SD_BAS_NS_SW 4096
#define SD_BAS_VERIF 4320

#define SD_BAS_DEUX 2
#define SD_BAS_QUATRE 4
#define SD_BAS_CINQ 5

#define SD_BAS_NB_SURF_RAD  10 // Number of radiative surfaces seen by the black body

/* nombre max d'IPR susceptibles d'etre ecrits dans un produit de niveau 1 */
#define SD_BAS_NB_MAX_IPR 150

#define SD_BAS_NBPOSOVERFLOWMAX 3  /* number max of positions of overflows
                                      written in the engineering product */
#define SD_BAS_NBPOSUNDERFLOWMAX 3 /* number max of positions of underflows
                                      written in the engineering product */
#define SD_BAS_NBSEGMAX 256 /* number max of segments in a spectral band */

/* DM 11/03/2004 */
#define SD_BAS_SNREFDATELIGNE 15 /* indice de scan pour dater une ligne */

static const int PERIODE_VIADR = 10; // Period of VIADR vs MDR 
//FA158
#define  SD_BAS_PERIODE_VIADR 10 // Period of VIADR vs MDR 

//--------------------------------------------------------------------*/
/* constantes globales a L'OPS */
//--------------------------------------------------------------------*/
// Valeurs de ctx_appel pour SD_ALG_22_SOS SD_ALG_20_DOC
static const int SD_BAS_ISRFEM    = 0;     // Quand appele par ISRFEM
static const int SD_BAS_PRODUIT   = 1;     // Qaudn appele par PRODUIT

#define SD_BAS_NB_CGXX 15   // nombre de cas degrades
/* CR290 */
// SD_BAS_NB_CGXX doit etre coherent avec CMN__NB_CGXX
/* Fin CR290 */
#define SD_BAS_NB_THREAD 10 // nombre de thread maximum utilises

/* variables d'environnement globales a l'OPS */


/*--------------------------------------------------------------------*/
/* constantes de gestion des erreurs de SD/BAS                        */
/* toutes les erreurs de bas niveau ont le meme numero d'erreur       */
/*--------------------------------------------------------------------*/
static const int SD_BAS_ERR_NUM_JDB = 1;
static const int SD_BAS_ERR_NON_DEF    = -1;
static const int SD_BAS_ERR_BAS_NIVEAU = (SD_BAS_ERR_BASE + 1);
// FT-2765
static char *SD_BAS_MSG_ALLOC   = (char *)"ALLOCATION ERROR";
static char *SD_BAS_MSG_OPEN    = (char *)"CANNOT OPEN FILE";
static char *SD_BAS_MSG_CLOSE   = (char *)"CANNOT CLOSE FILE";
static char *SD_BAS_MSG_READ    = (char *)"READ FILE ERROR";
static char *SD_BAS_MSG_WRITE   = (char *)"WRITE FILE ERROR";
static char *SD_BAS_MSG_SEEK    = (char *)"SEEK FILE ERROR";
// ...FT-2765

//====================================================================*/
// Exported types Declaration
//====================================================================*/

#define SD_BAS_LGNOMFICHIER MAXPATHLEN     /* longueur des noms des fichiers */
#define SD_BAS_LGLIBERR 255     /* longueur des libelles erreur */
#define SD_BAS_LGNOMPRODUITL1TRONQUE 15

/* FA171 */
#define NomProdVERTronque "IASI_VER_01_"
#define NomProdENGTronque "IASI_ENG_01_"
#define NomProd1ATronque  "IASI_xxx_1A_"
#define NomProd1BTronque  "IASI_xxx_1B_"
#define NomProd1CTronque  "IASI_xxx_1C_"
#define NomProdCTXTronque "IASI_CTX_xx_"

/* Nom complet (path+nom) d'un fichier de donnees*/
typedef char t_SD_BAS_NomFichier[SD_BAS_LGNOMFICHIER+1];

//******** definitions liees a l'instrument IASI
//--------------------------------------------------------------------*/
// type definissant le mode de prise de vue
//--------------------------------------------------------------------*/
typedef enum
    {
    SD_BAS_AUXILIARY            = 120, /* CR 271 */
    SD_BAS_NORMAL               = 33, /* CR 271 */
    SD_BAS_EXTERNAL_CALIBRATION = 34 /* CR 271 */
    } e_BAS_ModePdv;
//--------------------------------------------------------------------*/
// type definissant la transition entre 2 modes
//--------------------------------------------------------------------*/
typedef enum
    {
    SD_BAS_STABLE            = 1, /* CR 271 */
    SD_BAS_TRANSITION               = 0 /* CR 271 */
    } e_BAS_Transition;


//--------------------------------------------------------------------*/
// type definissant le type de traitement
//--------------------------------------------------------------------*/
typedef enum
    {
    GRANULE=0,
    DUMP=1
    } e_BAS_ModeTrt;

//--------------------------------------------------------------------*/
// type definissant l position du traitement
//--------------------------------------------------------------------*/
typedef enum
        {
    FIRST=1,
    MIDDLE=2,
    LAST=3
    } e_BAS_PositionTrt;

//--------------------------------------------------------------------*/
// type definissant le mode plateforme
//--------------------------------------------------------------------*/
typedef enum
    {
    PTF_NORMAL=1,
    PTF_HORS_PLAN=2,
    PTF_PLAN=3
    } e_BAS_ModePlateforme;

// ***** Definitions utilisees dans les differents modules
typedef struct
{
  double dX[2];
} t_SD_BAS_Vecteur2;    /* Vecteur a deux composantes */
                        /* Pour un vecteur V de type SD_BAS_Vecteur, V.dX[i] correspond
                        a la ieme composante */
typedef struct
{
  t_SD_BAS_Vecteur2 aV[2];
} t_SD_BAS_Matrice2x2;      /* Matrice de 2 lignes et 2 colonnes */
                            /* Pour une matrice M de type SD_BAS_Matrice3x3, M.aV[j].dX[i] correspond a
                            l'element de la ligne i et de la colonne j */


typedef struct
{
  double dX[3];
} t_SD_BAS_Vecteur3;    /* Vecteur a trois composantes */
                        /* Pour un vecteur V de type SD_BAS_Vecteur,
                        V.dX[i] correspond a la ieme composante */
typedef struct
{
  t_SD_BAS_Vecteur3 aV[3];
} t_SD_BAS_Matrice3x3;      /* Matrice de 3 lignes et 3 colonnes */
                            /* Pour une matrice M de type SD_BAS_Matrice3x3, M.aV[j].dX[i]
                            correspond a             l'element de la ligne i et de la colonne j */


//******** definitions lies aux formats des donnees IASI
//--------------------------------------------------------------------*/
/* Dates au format IASI */
//--------------------------------------------------------------------*/
/* dates UTC Jour Julien 2000 (cf. CMN__Date.h) */
typedef t_SCDSTime t_SD_BAS_SCDSTime;
typedef t_LCDSTime t_SD_BAS_LCDSTime;

/* date OBT (On Board Time) = nb de 1/256 secondes sur 32 bits */
typedef unsigned int t_SD_BAS_OBTTime;

typedef struct
    {
    unsigned short int year; // year
    unsigned short int month; // month
    unsigned short int day; // day
    unsigned short int hour; // hour
    unsigned short int min; // minute
    unsigned short int sec; // seconds
    unsigned int millis; // number of ms
    } t_SD_BAS_GeneralTime;

//--------------------------------------------------------------------*/
/* modele d'evolution  au format IASI */
//--------------------------------------------------------------------*/
typedef struct
    {
    t_SD_BAS_SCDSTime T0;
    double a0;
    double a1;
    double b1;
    int n;
    double g[194];
    } t_SD_BAS_ModeleEvolution;

//--------------------------------------------------------------------*/
/* liste des flag constituant une partie de GEPSIdConf                */
//--------------------------------------------------------------------*/
typedef enum
{
  SD_BAS_CG01_GOPSFLAPIXMISS = 0, 
  SD_BAS_CG02_GOPSFLADATAGAP = 1, 
  SD_BAS_CG03_GOPSFLTLISFERMOFF = 2, 
  SD_BAS_CG04_GOPSFLTBANDMISS = 3, 
  SD_BAS_CG05_GOPSFLTBBTMISS = 4, 
  SD_BAS_CG06_GOPSFLTIMGEWMISS = 5, 
  SD_BAS_CG07_GOPSFLTIMGBBMISS = 6, 
  SD_BAS_CG08_GOPSFLTIMGCSMISS = 7, 
  SD_BAS_CG11_GOPSFLAGPACKETVPMISS = 8, 
  SD_BAS_CG12_GOPSFLAGPACKETAPMISS = 9, 
  SD_BAS_CG13_GOPSFLAGPACKETPXMISS = 10, 
  SD_BAS_CG14_GOPSFLAGPACKETIPMISS = 11, 
  SD_BAS_CG15_GOPSFLAGEOAVHRRMISS = 12
} e_SD_BAS_GEPSIdConf_FLAG;


//******** definitions interne a la chaine OPS
//--------------------------------------------------------------------*/
// type definissant les donnees generales au granule
//--------------------------------------------------------------------*/
typedef struct
{
  // on mentionne ici toutes les donnees precalculees par
  // l'algo YY_CONF pour le granule ( cf SD_DON_DonGenGranule)
  double *  WnShift;                  //[SD_BAS_SN]
  int *     FlagNomQual;              //[SD_BAS_SN]
//  double *  WnFcsTab;                 //[fichier BS]
  double *  CosDephasagePRODUIT;         //[(3*SD_BAS_MAXBA+1)*ossfftfactor],
                                      // dim cf fichier de conf
  double *  CosDephasageISRFEM;          //[(SD_BAS_MAXECH_ISFREM+1)*ossfftfactor],

  double *  SinDephasagePRODUIT;         //[(3*SD_BAS_MAXBA+1)*ossfftfactor],
                                      // dim cf fichier de conf
  double *  SinDephasageISRFEM;          //[(SD_BAS_MAXECH_ISFREM+1)*ossfftfactor],
                                      // dim cf fichier de conf
                                      // dim cf fichier de conf


  // 30_FTB
  double *  GFtbFilteredBBT;          // de taille SD_BAS_NB_HIST_BBT
  int       GFfbFLagBBTNonQual;
  double    GFtbBBTRes;

  // donnees generales
  int       NbThread;                 // nombre de thread de calculs utilises
                                      //( issu appel ApplicationServeurdonnees

  int       NbLignesNonDummy;          // nombre de MDR non dummy dans les  produits



} t_SD_BAS_DataGen;

//--------------------------------------------------------------------*/
// type definissant les informations sur la donnee traitee
//--------------------------------------------------------------------*/
typedef struct
    {
    int LN; // numero de la ligne ( depuis le debut du granule )
    int SN; // numero du step number ( 0 si non utilise )
    int PN; // numero du pixel ( 0 si non utilise )
    int CCD;// numero diraction du coin de cube ( 0 si non utilise )
    int SB; // numero de la bande ( 0 si non utilise )
    int Thread; // numero du thread dans lequel le calcul est effectue
    } t_SD_BAS_ORIGINE;

//------------------------------------------------------------------*/
// type definissant les donnees internes  pour les meesage Metop
// 
//--------------------------------------------------------------------*/
typedef char  t_SD_BAS_MetopMSG[256];
//------------------------------------------------------------------*/
// type definissant les donnees internes  a un thread utilisees lors du
// traitement d'une etape
//
//--------------------------------------------------------------------*/
typedef struct
    {
    t_SD_BAS_ORIGINE ctxtrt; // contexte du traitement ( SN,PN ...);
    t_SD_BAS_CompteRendu cr;
    t_SD_BAS_MetopMSG  * ptmsg;
// on mentionne ici toutes les donnees temporaires utilisees par les algorithmes
// base ( spline fft ...) il est de la responsabilite des developpeurs algo de
// dimensionner ci-dessous les tailles necessaires qui seront allouees
// si-possible par les methodes de SD_GES
//*** attention !!!!!! note pour les developpeurs SD_ALG
//***** si possible il faut mettre les info qui suivent dans les structures par chaine
//*** fin attention !!!!!!
// SD_BAS_MetopGeod de taille (AC) ( a decliner par algo  appelant)
//    double * x; // tableau utilise pour stocker les coord metop dans 44_geo
//    double * y; // tableau utilise pour stocker les coord metop dans 44_geo
//    double * lat; // tableau utilise pour stocker les lat dans 44_geo
//    double * lon; // tableau utilise pour stocker les lon dans 44_geo
//    double * Ang_mes1; // tableau utilise pour stocker les angles mes dans 44_geo
//    double * Ang_mes2; // tableau utilise pour stocker les angles mes dans 44_geo
//    double * Ang_sol1; // tableau utilise pour stocker les angles solaire dans 44_geo
//    double * Ang_sol2; // tableau utilise pour stocker les angles solaire dans 44_geo
// SD_BAS_InterpSpline1D  de taille ???(AD) ( a decliner par algo  appelant de spline )
//    double *xe; // tableau utilise pour les interp spline .......
//    double *ye; // tableau utilise pour les interp spline .......
//    double *c; // tableau utilise pour les interp spline (AC) .......
//    double *xs; // tableau utilise pour les interp spline .......
//    double *ys; // tableau utilise pour les interp spline .......
// SD_BAS_InterpLineaire1D  de taille ???(AC) ( a decliner par algo  appelant  )
//    double *auxi; // tableau utilise pour les interp lin .......
// SD_BAS_InterpLineaire2D  de taille ???(AC) ( a decliner par algo  appelant  )
//    double *ratio; //  .......
// SD_BAS_ValVectPropre  de taille 2*nbpts
//    double *auxv; //  .......
// SD_BAS_FctAmortissement  de taille ??? ( a decliner par algo  appelant  )
//    double *fctAmortissement;   // Fonction d'amortissement de la taille max du spectre a amortir
//    double *noyAmortissement;   // Noyau gaussien pour l'amortissement de taille n ou n represente
                                // le nombre de points correspondant a sigma
// SD_BAS_FFT1DirDR2C et SD_BAS_FFT1InvDC2R de taille ???(AC) ( a decliner par algo  appelant
//de fft )
//    double *xr; // tableau utilise pour les FFT .......
//    double *yc; // tableau utilise pour les FFT .......
//    double *xc; // tableau utilise pour les FFT .......
//    double *yr; // tableau utilise pour les FFT .......
//    double *aux1;               // 20000 + 1.64nfft
//    double *aux2;               // 20000 + 1.14nfft

// SD_BAS_PermuteCirculairement ( a decliner par algo  appelant)
//    double * const pdtabf;          /* bloc initial a permuter */
//    double * const bloc_tempo;      /* bloc  temporaire de la taille du signal a permuter */
    } t_SD_BAS_DataThread;

/*--------------------------------------------------------------------*/
/* type definissant les donnees internes  a une chaine  utilisees     */
/* lors du traitement d'une etape ( donc dans un thread )             */
/* il faut y ajouter les tableaux temporaires                         */
/*--------------------------------------------------------------------*/
/* les donnees ci-dessous doivent etre allouees dynamiquement au      */
/* debut de l'etape (paquetage GES) puis liberees en fin              */
typedef struct
{
  /***************************************************************************/
  /* Liste des variables initialisees par 38_IRC                             */
  /***************************************************************************/
  /* Vecteur des nb d'onde des echantillons spectraux */
  /* Taille conf_stable->IDefIISSrfNslast - conf_stable->IDefIISSrfNsfirst + 1 */
  double *          Wn;

  /* Matrice des coeff de calibration instantanes */
  /* Taille conf_autre->IDefIISNbCol * conf_autre->IDefIISNbLin */
  double *          Slope;
  double *          Offset;
} t_SD_BAS_DataThreadImageDeb;

typedef struct
    {
        // Pour SD_ALG_22_S0S
        //========================================================================
        // dans toute  cette structure, indFS       = ConfAutre->IDefSsdSpectralWindowOPS;
        //                              nFFT        = max(ConfAutre->IWnpNsFFT[indFS] , ConfAutre->IDefS1bNsFFT)
        //                              surechFFT   = max(ConfAutre->IDefSsdOSFftFactor[indFS] ,ConfAutre->IDefS1bOSFftFactor)
        //                              surechSpline= max(ConfAutre->IDefSsdOSSplineFactor[indFS] , ConfAutre->IDefS1bOSSplineFactor)
        //                              sigmaSpectre= max(ConfAutre->IDefSsdSigS[indFS],ConfAutre->IDefS1bSigS)
        //                              sigmaInterf = max(ConfAutre->IDefSsdSigI[indFS],ConfAutre->IDefS1bSigI)
        //                              Nfirst      = min(ConfAutre->IDefSsdNsfirstSW[indFS],ConfAutre->IDefNsfirst1b)
        //                              Nlast       = max(ConfAutre->IDefSsdNslastSW[indFS],ConfAutre->IDefNslast1b)

        int CalculFFTSOS;          // pour optimisation du calcul des fft
        double *GSpectInit_cp;          // taillle Nlast
        double *GSpecArr;               // Taille nFFT;
        dcmplx *GSpecArrC;              // Taille ConfAutre->IWnpNsFFT[indFS];
        double *GSpecArr2;              // Taille nFFT;
        double *fctAmortissement;       // Taille nFFT;
        double *fctAmortissementArr;    // Taille nFFT;

        double *noyAmortissementS;      // Taille 6*sigmaSpectre + 1;
        double *noyAmortissementI;      // Taille 6*sigmaInterf + 1;

        // Pour les FFT ( avec optimisation )
        // FT-2765
        AUX_SD_BAS_FFT1DirDR2C *aux1;
        // ...FT-2765
        double *aux2;
        // FT-2765
        AUX_SD_BAS_FFT1DirInvDC2C *aux12; /* taille naux1 */
        // ...FT-2765
        double *aux22; /* taille naux2 */
        //if(nFFT <= 2048)
        //{
        //    naux1 = 22000;                          // cf doc lib ESSL p. 738 730 (max dcft drcft)
        //    naux2 = 20000;
        //}
        //else
        //{
        //    naux1 = (int)(20000 + 2.28*nFFT);       // cf doc lib ESSL p. 738 730 (max dcft drcft)
        //    naux2 = (int)(20000 + 2.28*nFFT);
        //}

        dcmplx *sTF;                // Taille nFFT
        dcmplx *sTFt;               // Taille nFFT
        dcmplx *sTFt2;              // Taille nFFT
        dcmplx *sTFdup;             // Taille nFFT
        double *GSpectReechFFT;     // Taille (Nlast-Nfirst + 1)*surechFFT+1
        double *wn;                 // Taille (Nlast-Nfirst + 1)*surechFFT+1
        double *oswn;               // Taille (Nlast-Nfirst + 1)*surechFFT*surechSpline+1;
        // FT-2765
        AUX_SD_BAS_InterpSpline1D *y2;// Taille 4*((Nlast-Nfirst+1)*surechFFT+1))
        // ...FT-2765

    } t_SD_BAS_DataThread22SOS;

/* la definition de cette structure ne doit etre faite qu'en C */
/* le type dcmplx est defini dans ESSL.h */
typedef struct
    {
        // dans toute  cette structure, indFS       = ConfAutre->IDefSsdSpectralWindowOPS;
        //                              nFFT        = max(ConfAutre->IWnpNsFFT[indFS] , ConfAutre->IDefS1bNsFFT)
        //                              surechFFT   = max(ConfAutre->IDefSsdOSFftFactor[indFS] ,ConfAutre->IDefS1bOSFftFactor)
        //                              surechSpline= max(ConfAutre->IDefSsdOSSplineFactor[indFS] , ConfAutre->IDefS1bOSSplineFactor)
        //                              sigmaSpectre= max(ConfAutre->IDefSsdSigS[indFS],ConfAutre->IDefS1bSigS)
        //                              sigmaInterf = max(ConfAutre->IDefSsdSigI[indFS],ConfAutre->IDefS1bSigI)
        //                              Nfirst      = min(ConfAutre->IDefSsdNsfirstSW[indFS],ConfAutre->IDefNsfirst1b)
        //                              Nlast       = max(ConfAutre->IDefSsdNslastSW[indFS],ConfAutre->IDefNslast1b)

        int CalculFFTSSD;          // pour optimisation du calcul des fft

        // Pour SD_ALG_23_SSD (initialise par 39_IRC (sans qu'il le sache))
        //========================================================================
         double *GIrcImg;       // taille (SD_BAS_IMCO*SD_BAS_IMLI*SD_BAS_SN)
        // Pour SD_ALG_23_SSD
        //========================================================================
        // Infogen = structure de sortie de 22_SOS APRES execution de 22_SOS

        double *fctAmortissement;       // Taille ((Nlast-Nfirst + 1)*surechFFT*surechSpline+1);
        double *noyAmortissementS;      // Taille 6*sigmaSpectre + 1;

        // Pour les FFT
        dcmplx *sTF;                // Taille nFFT
        dcmplx *sTFdup;             // Taille nFFT
        // FT-2765
        AUX_SD_BAS_FFT1DirDR2C *aux1;
        // ...FT-2765
        double *aux2;

        //if(nFFT <= 4096)
        //{
        //    naux1 = 22000;                          // cf doc lib ESSL p. 738
        //    naux2 = 20000;
        //}
        //else
        //{
        //    naux1 = (int)(20000 + 1.64*nFFT);       // cf doc lib ESSL p. 738
        //    naux2 = (int)(20000 + 1.14*nFFT);
        //}


        double *Wno;                // Taille ((Nlast-Nfirst + 1)*surechFFT*surechSpline + 1)
        double *Wn;                 // Taille ((Nlast-Nfirst + 1)*surechFFT*surechSpline + 1)
        // FT-2765
        AUX_SD_BAS_InterpSpline1D *y2ssd; // Taille  4*((Nlast-Nfirst + 1)*surechFFT*surechSpline + 1);
        // ...FT-2765
        double *RdSW;               // Taille nFFT
        double *RdSW2;               // Taille nFFT



        // Pour SD_ALG_21_SSS
        //========================================================================
        //ColDeb  = Max(ConfAutre->IDefIISColDeb);
        //ColFin  = Max(ConfAutre->IDefIISColFin);
        //LinDeb  = Max(ConfAutre->IDefIISLinDeb);
        //LinFin  = Max(ConfAutre->IDefIISLinFin);
        double *Y;              // Taille (LinFin-LinDeb+1)*(ColFin-ColDeb+1)
        double *Z;              // Taille (LinFin-LinDeb+1)*(ColFin-ColDeb+1)
        double *ImgY;           // Taille (LinFin-LinDeb+1)*(ColFin-ColDeb+1)
        double *ImgZ;           // Taille (LinFin-LinDeb+1)*(ColFin-ColDeb+1)
        double *IISWgtIpsf;     // Taille (LinFin-LinDeb+1)*(ColFin-ColDeb+1)

        // Pour SD_ALG_24_IAX
        //========================================================================
        //NbLin           = BanqueSpectrale->IDefSdbGridNbLin;
        //NbCol           = BanqueSpectrale->IDefSdbGridNbCol;
        double *Dist;        // Taille NbLin*NbCol

        // Pour SD_ALG_25_FAX
        //========================================================================
        // Pour la resolution de AX = B
        double *A;        // (ConfAutre->IDefFaxRegressDegre+1)*NbLignesIASI+ConfAutre[0]->IDefFaxFLS
        double *B;        // NbLignesIASI+ConfAutre[0]->IDefFaxFLS
        double *C;        // NbLignesIASI+ConfAutre[0]->IDefFaxFLS
        double *X;        // NbLignesIASI+ConfAutre[0]->IDefFaxFLS
        double *w;        // NbLignesIASI+ConfAutre[0]->IDefFaxFLS
        double  *AxeYInstant;           //@ taille NbLignesIASI+ConfAutre[0]->IDefFaxFLS
        double  *AxeZInstant;           //@ taille NbLignesIASI+ConfAutre[0]->IDefFaxFLS
        double  *AxeQualInstant;        //@ taille NbLignesIASI+ConfAutre[0]->IDefFaxFLS
        unsigned short int *FlagAxeNonQualInstant; //@ taille NbLignesIASI+ConfAutre[0]->IDefFaxFLS
        double  *AxeYFiltered;          //@ taille NbLignesIASI+ConfAutre[0]->IDefFaxFLS
        double  *AxeZFiltered ;         //@ taille NbLignesIASI+ConfAutre[0]->IDefFaxFLS
        double *tabTmpMC;   // Taille Max{ConfAutre->IDefFaxRegressDegre+1 , NbLignesIASI+ConfAutre[0]->IDefFaxFLS}
        // Max(contextecourant->GIaxNbElt) = 1600 d'apres JLP

    } t_SD_BAS_DataThreadImageISRFEM;

    typedef struct
    {
    double *inutile;               // a supprimer lors du premier ajout
    } t_SD_BAS_DataThreadFiltrageAxeInterf;


/*****************************************************************************/
/* Structure t_SD_BAS_DataThreadProduit :                                    */
/* - cette structure rassemble des variables utilisees par les algorithmes   */
/*   de la chaine produit suivants :                                         */
/*   43_ISF (sert d'interface entre la chaine ISRFEM et la chaine PRODUIT)   */
/*   09_PLK                                                                  */
/*   30_FTB                                                                  */
/*   31_SCR                                                                  */
/*   32_HEC                                                                  */
/*   34_SMC                                                                  */
/*   35_S1B                                                                  */
/*   37_S1C                                                                  */
/* - toutes les variables se sont vues allouer de la memoire en amont        */
/*****************************************************************************/
typedef struct
{
  int CalculFFTS1C;          // pour optimisation du calcul des fft
  int GSosSsdOSNsLastSpect;  //taille des tableaux ci dessous
  int GSosSsdOSNsFirstSpect; //taille des tableaux ci dessous;
  /***************************************************************************/
  /* Liste des variables initialisees par 43_ISF                             */
  /***************************************************************************/
  // Pour le spline
  double *Fcs;    //taille BanqueSpectrale->IDefSafNslast - BanqueSpectrale->IDefSafNsfirst +1
  // FT-2765
  AUX_SD_BAS_InterpSpline1D *y2isf;  //taille 4*(BanqueSpectrale->IDefSafNsOpd)
  // ...FT-2765
  double *OpdTab; //taille BanqueSpectrale->IDefSafNsOpd;
  double *Opd;    //taille 2*ConfAutre->IDefS1cNbSWwidth + 1
  dcmplx *Af;     //taille (BanqueSpectrale->IDefSafNslast - BanqueSpectrale->IDefSafNsfirst +1) * (2*ConfAutre->IDefS1cNbSWwidth+1)
  dcmplx *Af0;    //taille BanqueSpectrale->IDefSafNsOpd
  double *REtmp;  //taille BanqueSpectrale->IDefSafNsOpd
  double *IMtmp;  //taille BanqueSpectrale->IDefSafNsOpd
  double *REint;  //taille 2*ConfAutre->IDefS1cNbSWwidth + 1
  double *IMint;  //taille 2*ConfAutre->IDefS1cNbSWwidth + 1


  /***************************************************************************/
  /* Liste des variables initialisees par 09_PLK                             */
  /***************************************************************************/
  /* Fonction de Planck utilisee a bord pour la ligne precedente */
  /* taille ConfAutre->IDefNslast[SB=3]-ConfAutre->IDefNsfirst[SB=1]+1 */
  double *          PlanckBBT; /* LN-1 */


  /***************************************************************************/
  /* Liste des variables initialisees par 30_FTB                             */
  /***************************************************************************/
  // Pour la resolution de AX = B
  double *A;          // (ConfAutre->IDefFtbRegressDegre+1)*(nombreLignes+ConfAutre->IDefFaxFLS)
  double *B;          // (nombreLignes+ConfAutre->IDefFaxFLS)
  double *tabTmpMC;   // Taille Max{ConfAutre->IDefFtbRegressDegre+1 , (nombreLignes+ConfAutre->IDefFaxFLS)}
  double *X;          // taille ConfAutre->IDefFtbRegressDegre + 1

  /***************************************************************************/
  /* Liste des variables initialisees par XX_InitProd (XX_Init dans la doc)  */
  /***************************************************************************/
  /* Zone de travail pour l'interpolation lineaire 1D de la librairie ESSL */
  /* taille = conf_autre->IDefNslast[SD_BAS_B3] - conf_autre->IDefNsfirst[SD_BAS_B1] + 1 */
  // FT-2765
  AUX_SD_BAS_InterpLineaire1D *TabAuxESSLTailleWn0;
  // ...FT-2765
  /* Zone de travail pour l'interpolation lineaire 1D de la librairie ESSL */
  /* taille = t_SD_BAS_22_SOS_Sortie->GSosSsdOSNsLastSpect - t_SD_BAS_22_SOS_Sortie->GSosSsdOSNsFirstSpect + 1 */
  // FT-2765
  AUX_SD_BAS_InterpLineaire1D *TabAuxESSLTailleWnOSSpect;
  // FT-2765

  /* Variables initialement utilisees pour 31_SCR, 32_HEC, 34_SMC */
  /*--------------------------------------------------------------*/
  /* Vecteur des nb d'onde des fonctions de calibration spectrale tabulees */
  /* entre NsfirstFcsTab et NslastFcsTab */
  /* taille banque_spectrale->IDefSafNslast - banque_spectrale->IDefSafNsfirst + 1 */
  /* occurence : indice de ligne du granule = LN */
  double *          WnFcsTab;
  /* Vecteur des positions nominales des echantillons entre Nsfirst et Nslast */
  /* taille conf_autre->IDefNslast[SD_BAS_B3] - conf_autre->IDefNsfirst[SD_BAS_B1] + 1 */
  /* occurence : indice de ligne du granule = LN */
  double *          Wn0;
  /* Fonction de calibration spectrale interpolee lineairement */
  /* taille conf_autre->IDefNslast[SD_BAS_B3] - conf_autre->IDefNsfirst[SD_BAS_B1] + 1 */
  /* occurence : indice de ligne du granule = LN */
  double *          FcsInterpolee;
  /* Vecteur des nb d'onde nominaux calibres */
  /* taille ConfAutre->IDefNslast[SB=3] - ConfAutre->IDefNsfirst[SB=1] + 1 */
  /* occurence : indice de ligne du granule = LN */
  double *          WnSpect;

  /* Variables initialement utilisees pour 31_SCR */
  /*----------------------------------------------*/
  /* Valeurs de la fontion de Planck calibree pour la ligne precedente */
  /* taille ConfAutre->IDefNslast[SB=3]-ConfAutre->IDefNsfirst[SB=1]+1 */
  /* occurence : indice de ligne du granule = LN-1 */
  double *          B1;

  /* Variables initialement utilisees pour 32_HEC */
  /*----------------------------------------------*/
  /* Temperature des surfaces radiatives */
  /* taille conf_autre->IDefBBNbRrs */
  /* occurence : indice de ligne du granule = LN-1 */
  double *          TRrs;
  /* Radiance emise par le corps noir chaud pour les echantillons dont */
  /* l'indice est compris entre Nsfirst et Nslast */
  /* taille ConfAutre->IDefNslast[SB=3]-ConfAutre->IDefNsfirst[SB=1]+1 */
  /* occurence : indice de ligne du granule = LN-1 */
  double *          Bbb;

  /* Variables initialement utilisees pour 35_S1B */
  /*----------------------------------------------*/
  /* Vecteur des nb d'onde nominaux 1b calibres */
  /* taille ConfAutre->IDefSpectNslast1b[SB=3] - ConfAutre->IDefSpectNsfirst1b[SB=1] + 1 */
  /* occurence : indice de ligne du granule = LN */
  double *          WnSpect1b;

  /* Fonction de calibration spectrale interpolee lineairement */
  /* taille sortie_22_SOS->GSosSsdOSNsLastSpect - sortie_22_SOS->GSosSsdOSNsFirstSpect + 1 */
  double *          OSFcsInterpolee;

  /* Vecteur des nb d'onde du spectre surechantillonne d'indice compris entre */
  /* OSNsfirst et OSNslast */
  /* taille SD_BAS_22_SOS_Sortie->GSosSsdOSNsLastSpect - */
  /*       SD_BAS_22_SOS_Sortie->GSosSsdOSNsFirstSpect + 1 */
  /* occurence : indice de ligne du granule = LN */
  double *          WnOSSpect;

  /* Variables initialement utilisees pour 37_S1C */
  /*----------------------------------------------*/
  /* Vecteur des nb d'onde des fonctions d'apodisation d'indice compris entre NsfirstAfTab et */
  /* NslastAfTab */
  /* taille BanqueSpectrale->IDefSafNslast - BanqueSpectrale->IDefSafNsfirst + 1 */
  /* occurence : indice de ligne du granule = LN */
  double *          WnAf;
  /* Spectre 1B prolonge par continuite aux extremites */
  /* taille ConfAutre->IDefNslast1b - ConfAutre->IDefNsfirst1b + 1 + 2 * conf_autre->IDefS1cNbSWwidth */
  double *          Spectre1bProlonge;
  /* Fonction d'amortissement de la fenetre spectrale */
  /* taille 2*conf_autre->IDefS1cNbSWwidth+1 */
  /* occurence : indice de ligne du granule = LN */
  double *          FctAmortissement;
  /* Tableau des coeff necessaires a l'interpolation par Spline cubique */
  /* taille 4*(t_SD_BAS_22_SOS_Sortie->GSosSsdOSNsLastSpect - t_SD_BAS_22_SOS_Sortie->GSosSsdOSNsFirstSpect + 1) */
  // FT-2765
  AUX_SD_BAS_InterpSpline1D *CoeffSpline;
  // ...FT-2765
  /* Tableau temporaire du noyau pour le calcul de la fonction d'amortissement */
  /* taille 6*conf_autre->IDefS1cSigS+1 */
  double *          G;
  /* Fenetre spectrale du spectre 1B */
  /* taille 2*conf_autre->IDefS1cNbSWwidth+1 */
  double *          S;
  /* Fenetre spectrale completee par des zeros pour atteindre pour atteindre */
  /* une taille qui soit une puissance de 2 */
  /* taille nFFT = SD_BAS_CalculPuissance2Sup (TailleFenetreSpectrale) */
  /* TailleFenetreSpectrale = 2*conf_autre->IDefS1cNbSWwidth + 1 */
  double *          SpectrePadde;
  /* Pour les FFT                                                 */
  /* les tailles naux1 et naux2 sont definies comme suit :        */
  /*  if(nFFT <= 2048)                                            */
  /*  {                                                           */
  /*    naux1 = 22000; cf doc lib ESSL p. 738, 731                */
  /*    naux2 = 20000;                                            */
  /*  }                                                           */
  /*  else                                                        */
  /*  {                                                           */
  /*    naux1 = (int)(20000 + 2.28*nFFT); cf doc lib ESSL p. 738  */
  /*    naux2 = (int)(20000 + 2.28*nFFT);                         */
  /*  }                                                           */
  // FT-2765
    AUX_SD_BAS_FFT1DirDR2C *aux1; /* taille naux1 */
    // ...FT-2765
    double *				aux2; /* taille naux2 */
    // FT-2765/FT-2766
    AUX_SD_BAS_FFT1DirInvDC2C *aux12; /* taille naux1 */
    // ...FT-2765
  double *          aux22; /* taille naux2 */
  /* Transformee du spectre apres apodisation et zero-padding */
  /* taille nFFT = SD_BAS_CalculPuissance2Sup (TailleFenetreSpectrale) */
  /* TailleFenetreSpectrale = 2*conf_autre->IDefS1cNbSWwidth + 1 */
  dcmplx *          Stf;
  /* Moitie de la TF du spectre reel renvoyee par la librairie ESSL */
  /* (cf symetrie Hermitienne) */
  /* taille (1/2) * nFFT + 1 */
  /* nFFT = SD_BAS_CalculPuissance2Sup (TailleFenetreSpectrale) */
  /* TailleFenetreSpectrale = 2*conf_autre->IDefS1cNbSWwidth + 1 */
  dcmplx *          MoitieStf;
  /* FFT de la fenetre spectrale 1B apodisee */
  /* taille nFFT = SD_BAS_CalculPuissance2Sup (TailleFenetreSpectrale) */
  /* TailleFenetreSpectrale = 2*conf_autre->IDefS1cNbSWwidth + 1 */
  dcmplx *          SAtf;
  /* FFT de la fenetre spectrale 1B apodisee rearrangee*/
  /* taille nFFT = SD_BAS_CalculPuissance2Sup (TailleFenetreSpectrale) */
  /* TailleFenetreSpectrale = 2*conf_autre->IDefS1cNbSWwidth + 1 */
  dcmplx *          SAtfArrangee;
  /* FFT inverse de SAtf */
  /* taille nFFT = SD_BAS_CalculPuissance2Sup (TailleFenetreSpectrale) */
  /* TailleFenetreSpectrale = 2*conf_autre->IDefS1cNbSWwidth + 1 */
  dcmplx *          TFInvSAtf;
  /* Partie reelle de la FFT inverse de SAtf */
  /* taille nFFT = SD_BAS_CalculPuissance2Sup (TailleFenetreSpectrale) */
  /* TailleFenetreSpectrale = 2*conf_autre->IDefS1cNbSWwidth + 1 */
  double *          ReTFInvSAtf;
  /* Spectre apodise temporaire */
  /* taille nFFT = SD_BAS_CalculPuissance2Sup (TailleFenetreSpectrale) */
  /* TailleFenetreSpectrale = 2*conf_autre->IDefS1cNbSWwidth + 1 */
  double *          SA2Padde;
  /* Partie reelle de la FFT inverse de SAtf apres rearrangement des coeff */
  /* taille 2*conf_autre->IDefS1cNbSWwidth + 1 */
  double *          SA1;
  double *          SA2;


  /***************************************************************************/
  /* Liste des variables initialisees pour 41_CCS                            */
  /***************************************************************************/
  // 41_CCS
  unsigned int  *AvhrrChannelState;     // unsigned int( SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NBK)

  // 41_CCS ; SD_ALG_IpsfToIIS
  double        *ImgYpsf;               // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  double        *ImgZpsf;               // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  double        *ImgWgtpsf;             // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)

  // 41_CCS ; SD_ALG_InitRad
  int           *IdPix;                 // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)

  // SD_ALG_NueesDynamiques
  float         *RadNW;                 // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)

  // SD_ALG_NueesDynamiques ; SD_ALG_IterNuDyn ; SD_ALG_GestionProto
  double        *Dist;                  // double( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NCL)

  // SD_ALG_IpsfToIIS
  double        *ImgY0;                 // double( SD_BAS_IMCO * SD_BAS_IMLI)
  double        *ImgZ0;                 // double( SD_BAS_IMCO * SD_BAS_IMLI)
  double        *ImgY;                  // double( SD_BAS_IMCO * SD_BAS_IMLI)
  double        *ImgZ;                  // double( SD_BAS_IMCO * SD_BAS_IMLI)

  // SD_ALG_AnaRadIntoFov
  float         *RadVectFov;            // float( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NBK)
  int           *IdPixFov;              // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  int           *Index;                 // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)

  // SD_ALG_InitRad
  float         *RadN;                  // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
  
  // SD_ALG_InitProto
  unsigned int  *UsedPixTest;           // unsigned int(SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  double        *DistZero;              // double( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  double        *Dist1;                 // double( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)

  // SD_ALG_GestionProto ; SD_ALG_GestionProtoPost
  int           *IdPixAglo;             // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)

  // SD_ALG_StatClas
  float         *ImgIJ;                 // float( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)

  float               *Y;        // Image AVHRR Taille= SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NBK
  /***************************************************************************/
  /* FIN Liste des variables initialisees pour 41_CCS                        */
  /***************************************************************************/



  /* Variables initialement utilisees pour 40_IAC */
  /*----------------------------------------------*/
    // Dl              = ConfAutre->IDefIacDLin;
    // Dc              = ConfAutre->IDefIacDCol;
   double              *Ybar;     // Taille 2*(Dl+1)
   double              *Y2bar;    // Taille 2*(Dl+1)
   double              *R;        // Taille 4*(Dl+1)*(Dc+1)


  /* Variables initialement utilisees par 44_GEO */
  /*----------------------------------------------*/
  //  Ssize  = ConfAutre->IDefGeoIISSubGridSize;
  double *SgIISY;               // Taille Ssize*Ssize
  double *SgIISZ;               // Taille Ssize*Ssize
  double *SgIISRM;              // Taille 2*Ssize*Ssize
  double *lat;                  // Taille Ssize*Ssize
  double *lon;                  // Taille Ssize*Ssize
  double *Ang_mesY;             // Taille Ssize*Ssize
  double *Ang_mesZ;             // Taille Ssize*Ssize
  double *Azim_sol;             // Taille Ssize*Ssize
  double *Elev_sol;             // Taille Ssize*Ssize
  double *IISCornerAvhrrLin;    // Taille Ssize*Ssize
  double *IISCornerAvhrrCol;    // Taille Ssize*Ssize
  double *LocIasiAvhrrLin;      // Taille Ssize*Ssize
  double *LocIasiAvhrrCol;      // Taille Ssize*Ssize
  double *EarthSatDist;         // Taille Ssize*Ssize

  /* Variables initialement utilisees par 100_EXS */
  /*----------------------------------------------*/
  /* Temperature de brillance min */
  /* taille = SD_BAS_SPCT_COD */
  double *  SpectreFroid;
  /* Temperature de brillance max */
  /* taille = SD_BAS_SPCT_COD */
  double *  SpectreChaud;

} t_SD_BAS_DataThreadProduit;

//--------------------------------------------------------------------*/
// type definissant les donnees statistiques utilises pour les
// record generaux du produit de sortie
//--------------------------------------------------------------------*/
typedef struct
    {
    int count_degraded_inst_mdr;
    int count_degraded_proc_mdr;
    int count_degraded_inst_mdr_blocks;
    int count_degraded_proc_mdr_blocks;
    
    /* CR292 */
    // Champs initialises par SD_ALG_AVHRR_GetSubSatLatLon pour preparer
    // la mise a jour des champs du MPHR N1
    // MPHR_SUBSAT_LATITUDE_START
    // MPHR_SUBSAT_LATITUDE_END
    // MPHR_SUBSAT_LONGITUDE_START
    // MPHR_SUBSAT_LONGITUDE_END
    double subsat_latitude_start;
    double subsat_longitude_start;
    double subsat_latitude_end;
    double subsat_longitude_end;
    /* FIN CR292 */
    
// a completer en renseignant ici les champs indique dans le fichier xls de
// jlp  ces champs sont mis a jour par le traitement
// en particulier nombre de MDR corrupted
// d'autres champs peuvent etre geres en interne par l'objet SD_DON_GranuleN1
// nombre de record , date du produit , nom du produit et initialise des le
// debut ou a la fin ( date de fin de tt)
    } t_SD_BAS_StatMphr;

//--------------------------------------------------------------------*/
// type definissant la version
//--------------------------------------------------------------------*/
typedef struct
    {
    int major;
    int minor;
    } t_SD_BAS_VERSION;
//====================================================================*/
// External variables Declaration
//====================================================================*/
// version de l'OPS utilise pour l'ecriture dans les different fichiers
// version de l'OPS en debut de vie
static const t_SD_BAS_VERSION SD_BAS_OPSVERSION={1,0};

//====================================================================*/
// External fonctions Declaration
//====================================================================*/


#endif

