// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/BAS/src/SD_BAS_LibESSL.h,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_BAS
//@ MODULE : SD_BAS_LibESSL
//@
//@ Comment :
//@    Include file for the  SD_BAS_LibESSL class which encapsulates the
//@    acces to the IBM ESSL librarie used for   :
//@ - Fast Fourier Transform.
//@ - Spline/linear Interpolation.
//@     - least Square approximation.
//@ - .....
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_BAS_LibESSL_h
#define SD_BAS_LibESSL_h

// Pour initialiser la gestion des erreurs dans ESSL
#ifndef __ESVERR
#define __ESVERR        /* Doit etre defini avant le include <essl.h> */
#endif
//#include "essl.h"        // Deja incluse via SD_BAS_Global.h et
                            // n'aime pas les redondances...
extern int enotrm();        // Pour la gestion des erreurs

// inserer ici les fichiers include
#include <stdio.h>       /* entree/sortie */
#include <stdlib.h>      /* alloc memoire, ...*/
#include <string.h>      /* chaines de carateres */
#include <unistd.h>      /* gestion pipes, ... */
#include <time.h>        /* gestion de la date */
#include <sys/param.h>   /* constantes systeme */
#include <math.h>

#include "SD_BAS_Global.h"
#include "CMN__Commun.h"
#include "ERR__Erreur.h"
#include "SD_BAS_AUXLibESSL.h"


/*====================================================================*/
/*   External define  Declarations                                    */
/*====================================================================*/
#define ERR_BASE_ESSL                        SD_BAS_ERR_BASE_ESSL
#define SD_BAS_LibESSL_ERR_TAILLE_FFT       (ERR_BASE_ESSL+1)
#define SD_BAS_LibESSL_ERR_CONVERG          (ERR_BASE_ESSL+2)
#define SD_BAS_LibESSL_ERR_NFIRST           (ERR_BASE_ESSL+3)
#define SD_BAS_LibESSL_ERR_NLAST            (ERR_BASE_ESSL+4)
#define SD_BAS_LibESSL_ERR_ALLOCATION       (ERR_BASE_ESSL+5)
#define SD_BAS_LibESSL_ERR_DGESVF           (ERR_BASE_ESSL+6)

#define SD_BAS_ERR_SET_ESSL                 1

/*====================================================================*/
/* Constant Declaration                                               */
/*====================================================================*/

/*====================================================================*/
/* Exported types Declaration                                         */
/*====================================================================*/

/*====================================================================*/
/* External variables Declaration                                     */
/*====================================================================*/

/*====================================================================*/
//                  FONCTIONS EXTERNES
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_FFT1DirDR2C
//@
//@ Description : transformee de fourier directe 1D ( reel vers complexe)
//@ pour des pb d'efficacite il est souhaitable ici de regrouper le calcul de
//@ plusieurs FFT . si ce n'est pas possible utiliser l' argument d'appel Init
//@
//@
//@ Parameters :
//@         (ENTREE)  init : initialisation ou calcul (1 ou 0)
//@         (ENTREE)  nbpts : nombre de points de la FFT a traiter
//@         (ENTREE)  nbfft : nombre de FFT a traiter
//@         (ENTREE)  xr : input constitue de plusieurs tableaux
//@                 de reels contigus
//@         (ENTREE)  aux1 : tableau temporaire
//@         (ENTREE)  aux2 : tableau temporaire
//@         (ENTREE)  naux1 : taille du tableau temporaire aux1
//@         (ENTREE)  naux2 : taille du tableau temporaire aux2
//@         (ENTREE)  scale : facteur de normalisation
//@         (ENTREE)  yc : output constitue de plusieurs tableaux
//@                 de complexes contigus
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_FFT1DirDR2C(
        t_SD_BAS_DataThread * ctx,
        int  init,
        int  *nbpts,
        int  nbfft,
        double * xr,
        // FT-2765
        AUX_SD_BAS_FFT1DirDR2C * aux1,
        // ...FT-2765
        double * aux2,
        int naux1,
        int naux2,
        double  scale,
        dcmplx *yc
            )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_FFT1InvDC2R
//@
//@ Description : transformee de fourier inverse 1D (  complexe vers reel)
//@ pour des pb d'efficacite il est souhaitable ici de regrouper le calcul de
//@ plusieurs FFT . si ce n'est pas possible utiliser l' argument d'appel Init
//@
//@
//@ Parameters :
//@         (ENTREE)  init : initialisation ou calcul (1 ou 0)
//@         (ENTREE)  nbpts : nombre de points de la FFT a traiter
//@         (en tenant compte de la symetrie)
//@         (ENTREE)  nbfft : nombre de FFT a traiter
//@         (ENTREE)  xc : input constitue de plusieurs tableaux
//@                 de complexes contigus
//@         (ENTREE)  aux1 : tableau temporaire
//@         (ENTREE)  aux2 : tableau temporaire
//@         (ENTREE)  naux1 : taille du tableau temporaire aux1
//@         (ENTREE)  naux2 : taille du tableau temporaire aux2
//@         (ENTREE)  scale : facteur de normalisation
//@         (ENTREE)  yr : output constitue de plusieurs tableaux
//@                 de complexes contigus
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_FFT1InvDC2R(
        t_SD_BAS_DataThread * ctx,
        int  init,
        int  *nbpts,
        int  nbfft,
        dcmplx * xc,
        // FT-2765
        AUX_SD_BAS_FFT1InvDC2R * aux1,
        // ...FT-2765
        double * aux2,
        int naux1,
        int naux2,
        double  scale,
        double *yr
            )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_FFT1DirDC2C
//@
//@ Description : transformee de fourier directe 1D ( complexe vers complexe)
//@ pour des pb d'efficacite il est souhaitable ici de regrouper le calcul de
//@ plusieurs FFT . si ce n'est pas possible utiliser l' argument d'appel Init
//@
//@
//@ Parameters :
//@         (ENTREE)  isign : sens de la FFT (1=directe, -1=inverse)
//@         (ENTREE)  init : initialisation ou calcul (1 ou 0)
//@         (ENTREE)  nbpts : nombre de points de la FFT a traiter
//@         (ENTREE)  nbfft : nombre de FFT a traiter
//@         (ENTREE)  xr : input constitue de plusieurs tableaux
//@                 de reels contigus
//@         (ENTREE)  aux1 : tableau temporaire
//@         (ENTREE)  aux2 : tableau temporaire
//@         (ENTREE)  naux1 : taille du tableau temporaire aux1
//@         (ENTREE)  naux2 : taille du tableau temporaire aux2
//@         (ENTREE)  scale : facteur de normalisation
//@         (ENTREE)  yc : output constitue de plusieurs tableaux
//@                 de complexes contigus
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_FFT1DirInvDC2C(
        t_SD_BAS_DataThread * ctx,
        int isign,
        int  init,
        int  *nbpts,
        int  nbfft,
        dcmplx * xc,
        // FT-2765
        AUX_SD_BAS_FFT1DirInvDC2C * aux1,
        // ...FT-2765
        double * aux2,
        int naux1,
        int naux2,
        double  scale,
        dcmplx *yc
            )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_MoindresCarres1D
//@
//@ Description : resolution par moindres carres d'un systeme lineaire
//@ de type A X = B
//@ les lignes ont ete constitues et allouees par l'appelant ainsi que
//@ les vecteurs b et s
//@
//@ Parameters :
//@         (ENTREE)  ctx : gestion des erreurs
//@         (ENTREE)  nbeq : nombre d'equations = nombre de lignes de A
//@         (ENTREE)  ninc : nombre d'inconnues = nombre de colonnes de A
//@                         (dans la cas de l'ajustement d'un polynome de
//@                         degre N, ninc = N+1)
//@         (ENTREE)  a : input constitue d'une matrice nbeq*ninc
//@                       de double contigus ranges par colonnes
//@                       (ordre fortran)
//@         (ENTREE)  b : input constitue d'un vecteur nbeq
//@                       de double contigus
//@         (SORTIE)  s : vecteur de taille ninc contenant les valeurs
//@                       singulieres de A (alloue par l'appelant)
//@         (ENTREE)  tau : tolerance d'erreur sur l'estimation
//@         (SORTIE)  x : resultat constitue d'un vecteur de taille ninc
//@                       de double contigus
//@         (SORTIE)  rmse : RMSE de l'ajustement
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_MoindresCarres1D(
        t_SD_BAS_DataThread * ctx,
        int  nbeq,
        int  ninc,
        double * a,
        double * b,
        double * s,
        double tau,
        double * x,
        double * rmse
            )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_InterpSpline1D
//@
//@ Description : interpolation spline d'une serie de valeurs 1D triees
//@ par ordre croissant
//@ cette fonction doit etre appelee une premiere fois avec init=0
//@ puis  elle est appelee pour des entrees similaires
//@ avec init=4 pour reutiliser c (AC)
//@
//@ Parameters :
//@         (ENTREE)  init : initialisation ou calcul (0 ou 4)
//@         (ENTREE)  ne : nombre de points en entree
//@         (ENTREE)  xe : abscisse des points en entree
//@         (ENTREE)  ye : ordonnees des points en entree
//@         (ENTREE/SORTIE)  c : tableau des coef precalcules de taille 4*ne
//@         (ENTREE)  ns : nombre de points en sortie
//@         (ENTREE)  xs : abscisse des points en sortie
//@         (SORTIE)  ys : ordonnees des points en sortie
//@
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_InterpSpline1D(
        int  init,
        int  ne,
        double * xe,
        double * ye,
        // FT-2765
        AUX_SD_BAS_InterpSpline1D * c,
        // ...FT-2765
        int  ns,
        double * xs,
        double * ys
            )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_InterpSpline2D
//@
//@ Description : interpolation spline d'une serie de valeurs 2D triees
//@ par ordre croissant pour un point
//@
//@ Parameters :
//@         (ENTREE)  ne : nombre de points de x1 et de x2 en entree
//@         (ENTREE)  xe1 : abscisse des points en entree
//@         (ENTREE)  xe2 : abscisse des points en entree 
//@         (ENTREE)  ye : ordonnees des points en entree ranges par colonne (ne*ne)
//@         (ENTREE)  nc : nombre de points de c
//@         (ENTREE/SORTIE)  c : tableau des coef precalcules de taille (11ne + 3)
//@         (ENTREE)  xs1 : abscisse du point en sortie
//@         (ENTREE)  xs2 : abscisse du point en sortie
//@         (SORTIE)  ys : ordonnees du point en sortie
//@
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_InterpSpline2D(
        int  ne,
        double * xe1,
        double * xe2,
        double * ye,
        int  nc,
        double * c,
        double * xs1,
        double * xs2,
        double * ys
            )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_InterpLineaire1D
//@
//@ Description : interpolation lineaire d'une serie de valeurs 1D triees
//@ par ordre croissant
//@
//@ Parameters :
//@         (ENTREE)  ne : nombre de points en entree
//@         (ENTREE)  xe : abscisse des points en entree
//@         (ENTREE)  ye : ordonnees des points en entree
//@         (ENTREE/SORTIE)  auxi : tableau de travail
//@         (ENTREE)  ns : nombre de points en sortie
//@         (ENTREE)  xs : abscisse des points en sortie
//@         (SORTIE)  ys : ordonnees des points en sortie
//@
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_InterpLineaire1D(
        int  ne,
        double * xe,
        double * ye,
        // FT-2765
        AUX_SD_BAS_InterpLineaire1D * auxi,
        // ...FT-2765
        int  ns,
        double * xs,
        double * ys
            )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_InterpBiLineaire2D
//@
//@ Description : interpolation bilineaire d'une serie de valeurs 2D triees
//@ par ordre croissant
//@
//@ ATTENTION !! si les xs et ys ne sont pas compris dans les intervalles
//@ [xemin,xemax] et [yemin,yemax] les zs correspondants sont extrapoles
//@
//@ Parameters :
//@         (ENTREE)  init : initialisation ou calcul (1 ou 0)
//@         (ENTREE)  nxe : nombre de colonnes de l'image de ref
//@         (ENTREE)  nye : nombre de lignes de l'image de ref
//@         (ENTREE)  xe : coord x pour l'image de ref( trie)
//@         (ENTREE)  ye : coord y pour l'image de ref ( trie)
//@         (ENTREE)  ze : radiometrie de l'imag de ref : tableau (nxe*nye) range par
//@                         lignes (ligne 1,ligne 2, ... , ligne nye)
//@         (ENTREE)  nxs : nombre de colonnes de l'imagette a reechantillonner (nblig, nbcol)
//@         (ENTREE)  nys : nombre de lignes de l'imagette a reechantillonner (nblig, nbcol)
//@         (ENTREE)  xs : coord x pour l'imagette a reechatillonner( trie)
//@         (ENTREE)  ys : coord y pour l'imagette a reechatillonner( trie)
//@         (ENTREE/sortie)  ratio : ratio precalcules taille = 2*(nxs+nys)
//@                                     (pour t et u et les indices)
//@         (SORTIE)  zs : radiometrie de l'imag reechantillonnee : tableau (nxs*nys)
//@
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_InterpLineaire2D(
        t_SD_BAS_DataThread * ctx,
        int init,
        int   nxe,
        int   nye,
        double * xe,
        double * ye,
        double * ze,
        int  nxs,
        int  nys,
        double * xs,
        double * ys,
        double * zs,
        double *ratio
            )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_ValVectPropre
//@
//@ Description : calcul des valeurs propres et des vecteurs propres
//@ d'une matrice carree
//@
//@
//@ Parameters :
//@         (ENTREE)  nbpts : nombre de pts
//@         (ENTREE)  a : matrice
//@         (SORTIE)  auxv : tableau temporaire
//@         (SORTIE) valpropre : valeurs propres
//@         (SORTIE) vecpropre : vecteurs propres
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_ValVectPropre(
        t_SD_BAS_DataThread * ctx,
        int   nbpts,
        double * a,
        double * auxv,
        dcmplx * valpropre,
        dcmplx * vecpropre
        )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_PreparationFFTR
//@
//@ Description : Preparation d'un signal reel en vue de l'injecter dans
//@               l'un des algorithmes de FFT precedents
//@
//@
//@ Parameters :
//@         (ENTREE)  x : signal a preparer
//@         (ENTREE)  nfirst : position du premier point de x dans le spectre
//@                             originel centre
//@         (ENTREE)  nlast : position du dernier point de x dans le spectre
//@                             originel centre
//@         (SORTIE)  y : signal prepare, alloue par l'appelant
//@         (ENTREE)  nFFT : taille de ny
//@         (ENTREE)  xpadde : tableau temporaire de taille ny pour le calcul
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_PreparationFFTR(
        t_SD_BAS_DataThread * ctx,
        double *x,
        int nfirst,
        int nlast,
        double *y,
        int nFFT
)
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_PreparationFFTC
//@
//@ Description : Preparation d'un signal COMPLEXE en vue de l'injecter dans
//@               l'un des algorithmes de FFT precedents
//@
//@
//@ Parameters :
//@         (ENTREE)  x : signal a preparer
//@         (ENTREE)  nfirst : position du premier point de x dans le spectre
//@                             originel centre. nfirst min vaut 1
//@         (ENTREE)  nlast : position du dernier point de x dans le spectre
//@                             originel centre. nlast min vaut 1
//@         (SORTIE)  y : signal prepare, alloue par l'appelant
//@         (ENTREE)  nFFT : taille de ny
//@         (ENTREE)  xpadde : tableau temporaire de taille ny pour le calcul
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_PreparationFFTC(
        t_SD_BAS_DataThread * ctx,
        dcmplx *x,
        int nfirst,
        int nlast,
        dcmplx *y,
        int nFFT
)
;

//======================================================================
#endif
