//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/BAS/src/SD_BAS_LibMath.c,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_BAS
//@ MODULE : SD_BAS_LibMath
//@
//@ Comment :
//@    Code file for the  SD_BAS_LibESSL class which implements
//@    standard mathematical methods   :
//@ - Matrix operations.
//@ - .....
//@ these methods are hand made in order to facilitate the
//@ portabilite of the software
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 08/01/2003 : [reference :] Mathieu Jung
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <stdio.h>
#include <math.h>

#include "SD_BAS_LibMath.h"

/*====================================================================*/
// Local Constant Declaration
/*====================================================================*/
const double SD_BAS_Epsilon=1e-10;

/*====================================================================*/
// Local types Declaration
/*====================================================================*/
//======================================================================
// Internal fonctions Declaration
//======================================================================

//======================================================================
// Global variables Definition
//======================================================================


//======================================================================
//                  FONCTIONS EXTERNES
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_ProduitMatriceVecteur
//@
//@ Description : Calcule le produit d'une matrice carree 3x3 par un
//@ vecteur a 3 dimensions
//@
//@
//@ Parameters :
//@         (ENTREE)  paM_E : Matrice membre gauche du produit
//@         (ENTREE)  paA_E : Vecteur membre droit du produit
//@         (SORTIE)  paB_S : Vecteur resultat du produit
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_ProduitMatriceVecteur (
  t_SD_BAS_Matrice3x3 *paM_E,         /* Matrice membre gauche du produit */
  t_SD_BAS_Vecteur3    *paA_E,        /* Vecteur membre droit du produit */
  t_SD_BAS_Vecteur3    *paB_S         /* Vecteur resultat du produit */
  )
{
t_CMN__Statut cr_op=CMN__STATUT_OK;

//@  for(iIndLig=0;iIndLig<3;iIndLig++)
//@      {
//@        paB_S->dX[iIndLig] = 0.0;
//@        for(iIndCol=0;iIndCol<3;iIndCol++)
//@     {
//@       paB_S->dX[iIndLig] = paB_S->dX[iIndLig] +
//@         (paM_E->aV[iIndCol].dX[iIndLig] * paA_E->dX[iIndCol]);
//@     }
//@      }

    int iIndLig;        // Compteur sur les lignes
    int iIndCol;        // Compteur sur les colonnes


    for(iIndLig=0;iIndLig<3;iIndLig++)
    {
        paB_S->dX[iIndLig] = 0.0;
        for(iIndCol=0;iIndCol<3;iIndCol++)
        {
            paB_S->dX[iIndLig] =    paB_S->dX[iIndLig]
                                 + (paM_E->aV[iIndCol].dX[iIndLig] * paA_E->dX[iIndCol]);
        }
    }

// sortie de la fonction
return(cr_op);
}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_TransposerMatrice
//@
//@ Description : Calculer la transposee d'une matrice 3 par 3 a
//@                 coefficients reels (double precision).
//@                 L'appel a SD_BAS_TransposerMatrice s'ecrit :
//@         SD_BAS_TransposerMatrice(aMatrice,&aTransposee)
//@     Note : Il appartient a l'utilisateur d'allouer et de desallouer les matrices
//@            passees en parametre de la fonction.
//@
//@
//@ Parameters :
//@         (ENTREE)  aMatrice_E : Matrice a transposer
//@         (SORTIE)  paTransposee_S : Tranposee de la matrice
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_TransposerMatrice(
  t_SD_BAS_Matrice3x3*          aMatrice_E,                  /* Matrice a transposer    */
  t_SD_BAS_Matrice3x3*          paTransposee_S               /* Tranposee de la matrice */
  )

{
t_CMN__Statut cr_op=CMN__STATUT_OK;

  /* Boucle sur les lignes*/
//@  for (iLigne=0; iLigne<aMatrice_E.iNbLignes ; iLigne++)
//@     {
//@       /* Boucle sur les colonnes*/
//@       for (iColonne=0; iColonne<aMatrice_E.iNbColonnes ; iColonne++)
//@     {
//@       /* Transposition des coefficients */
//@       paTransposee_S->ppdValeurs[iColonne][iLigne] = aMatrice_E.ppdValeurs[iLigne][iColonne];
//@     }
//@       /* Fin Boucle sur les colonnes*/
//@     }
  /* Fin Boucle sur les lignes*/

    int iLigne;     // Compteur sur les lignes
    int iColonne;   // Compteur sur les colonnes

      /* Boucle sur les lignes*/
    for (iLigne=0; iLigne<3 ; iLigne++)
    {
        /* Boucle sur les colonnes*/
        for (iColonne=0; iColonne<3; iColonne++)
        {
            /* Transposition des coefficients */
            paTransposee_S->aV[iColonne].dX[iLigne] = aMatrice_E->aV[iLigne].dX[iColonne];
        }
        /* Fin Boucle sur les colonnes*/
    }
    /* Fin Boucle sur les lignes*/


// sortie de la fonction
return(cr_op);
}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_NormaliseVecteur
//@
//@ Description : Normalise un  vecteur de 3 elements
//@
//@
//@ Parameters :
//@         (ENTREE)  paVecteur_E : vecteur initial
//@         (SORTIE)  paVecteur_S : vecteur normalise
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_NormaliseVecteur(
  t_SD_BAS_Vecteur3    *paVecteur_E,          /* Vecteur non normalise */
  t_SD_BAS_Vecteur3    *paVecteur_S       /* Vecteur  normalise */
  )

{
t_CMN__Statut cr_op=CMN__STATUT_OK;

//@ trivial
    double maxv=fabs(paVecteur_E->dX[0]);
    int i;

    //@ recherche du max
    if (fabs(paVecteur_E->dX[1]) > maxv) maxv = fabs(paVecteur_E->dX[1]);
    if (fabs(paVecteur_E->dX[2]) > maxv) maxv = fabs(paVecteur_E->dX[2]);

    //@ Normalisation
    for(i=0; i<3; i++)
        paVecteur_S->dX[i] = paVecteur_E->dX[i]/maxv;

    //@ Cas ou tous les elements sont nuls
    if(SD_BAS_MacEGAUX_D(maxv,0.0))
        cr_op = CMN__STATUT_NOK;

// sortie de la fonction
return(cr_op);
}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_MatriceRotation
//@
//@ Description : Initialise une matrice de rotation d'angle alpha
//@               Autour de l'axe Y
//@
//@
//@ Parameters :
//@         (ENTREE)  dalpha : angle de la matrice ( en degre)
//@         (SORTIE)  paMatriceRot_S : Tranposee de la matrice
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_MatriceRotation(
  double  dalpha,           // angle    double(1)
  double *paMatriceRot_S    // matrice  double( SD_BAS_DIM_ROT_MATRICE * SD_BAS_DIM_ROT_MATRICE)
)
{
t_CMN__Statut cr_op=CMN__STATUT_OK;

  //@ La matrice de rotation est : | cos(alpha)  0  -sin(alpha) |
  //                               |     0       1       0      |
  //                               | sin(alpha)  0   cos(alpha) |
  paMatriceRot_S[IND2(1,1,SD_BAS_DIM_ROT_MATRICE)]=  cos( dalpha);
  paMatriceRot_S[IND2(2,1,SD_BAS_DIM_ROT_MATRICE)]=  0.0;
  paMatriceRot_S[IND2(3,1,SD_BAS_DIM_ROT_MATRICE)]= -sin( dalpha);
  paMatriceRot_S[IND2(1,2,SD_BAS_DIM_ROT_MATRICE)]=  0.0;
  paMatriceRot_S[IND2(2,2,SD_BAS_DIM_ROT_MATRICE)]=  1.0;
  paMatriceRot_S[IND2(3,2,SD_BAS_DIM_ROT_MATRICE)]=  0.0;
  paMatriceRot_S[IND2(1,3,SD_BAS_DIM_ROT_MATRICE)]=  sin( dalpha);
  paMatriceRot_S[IND2(2,3,SD_BAS_DIM_ROT_MATRICE)]=  0.0;
  paMatriceRot_S[IND2(3,3,SD_BAS_DIM_ROT_MATRICE)]=  cos( dalpha);

// sortie de la fonction
return(cr_op);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_InverserMatrice3x3
//@
//@ Description : inverse une matrice
//@
//@
//@ Parameters :
//@         (ENTREE)  aMatrice_E : matrice a inverser ( en degre)
//@         (SORTIE)  paInverse_S : Inverse de la matrice
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_InverserMatrice3x3(
            t_SD_BAS_Matrice3x3* Matrice_E, /* Matrice a inverser   */
            t_SD_BAS_Matrice3x3* Inverse_S  /* Inverse de la matrice */
                           )
{
t_CMN__Statut cr_op=CMN__STATUT_OK;

    t_SD_BAS_Matrice3x3 aCofacteurs;
    double dDeterminant;


//@  /* Calcul des coefficients de la matrice des cofacteurs */
  aCofacteurs.aV[0].dX[0] =   (Matrice_E->aV[1].dX[1])*(Matrice_E->aV[2].dX[2]) -
                              (Matrice_E->aV[1].dX[2])*(Matrice_E->aV[2].dX[1]);

  aCofacteurs.aV[1].dX[0] = - (Matrice_E->aV[0].dX[1])*(Matrice_E->aV[2].dX[2]) +
                              (Matrice_E->aV[0].dX[2])*(Matrice_E->aV[2].dX[1]);

  aCofacteurs.aV[2].dX[0] =   (Matrice_E->aV[0].dX[1])*(Matrice_E->aV[1].dX[2]) -
                              (Matrice_E->aV[0].dX[2])*(Matrice_E->aV[1].dX[1]);

  aCofacteurs.aV[0].dX[1] = - (Matrice_E->aV[1].dX[0])*(Matrice_E->aV[2].dX[2]) +
                              (Matrice_E->aV[1].dX[2])*(Matrice_E->aV[2].dX[0]);

  aCofacteurs.aV[1].dX[1] =   (Matrice_E->aV[0].dX[0])*(Matrice_E->aV[2].dX[2]) -
                              (Matrice_E->aV[0].dX[2])*(Matrice_E->aV[2].dX[0]);

  aCofacteurs.aV[2].dX[1] = - (Matrice_E->aV[0].dX[0])*(Matrice_E->aV[1].dX[2]) +
                              (Matrice_E->aV[0].dX[2])*(Matrice_E->aV[1].dX[0]);

  aCofacteurs.aV[0].dX[2] =   (Matrice_E->aV[1].dX[0])*(Matrice_E->aV[2].dX[1]) -
                              (Matrice_E->aV[1].dX[1])*(Matrice_E->aV[2].dX[0]);

  aCofacteurs.aV[1].dX[2] = - (Matrice_E->aV[0].dX[0])*(Matrice_E->aV[2].dX[1]) +
                              (Matrice_E->aV[0].dX[1])*(Matrice_E->aV[2].dX[0]);

  aCofacteurs.aV[2].dX[2] =   (Matrice_E->aV[0].dX[0])*(Matrice_E->aV[1].dX[1]) -
                              (Matrice_E->aV[0].dX[1])*(Matrice_E->aV[1].dX[0]);

//@  /* Calcul du determinant */
  dDeterminant =  (Matrice_E->aV[0].dX[0])*(aCofacteurs.aV[0].dX[0])
                + (Matrice_E->aV[1].dX[0])*(aCofacteurs.aV[1].dX[0])
                + (Matrice_E->aV[2].dX[0])*(aCofacteurs.aV[2].dX[0]);


  if(SD_BAS_MacEGAUX_D(dDeterminant,0.0))
  {
        cr_op=CMN__STATUT_NOK;
        goto fin;
  }


//@  /* Calcul de l'inverse de la matrice */
  Inverse_S->aV[0].dX[0] = (aCofacteurs.aV[0].dX[0])/dDeterminant;
  Inverse_S->aV[1].dX[0] = (aCofacteurs.aV[0].dX[1])/dDeterminant;
  Inverse_S->aV[2].dX[0] = (aCofacteurs.aV[0].dX[2])/dDeterminant;
  Inverse_S->aV[0].dX[1] = (aCofacteurs.aV[1].dX[0])/dDeterminant;
  Inverse_S->aV[1].dX[1] = (aCofacteurs.aV[1].dX[1])/dDeterminant;
  Inverse_S->aV[2].dX[1] = (aCofacteurs.aV[1].dX[2])/dDeterminant;
  Inverse_S->aV[0].dX[2] = (aCofacteurs.aV[2].dX[0])/dDeterminant;
  Inverse_S->aV[1].dX[2] = (aCofacteurs.aV[2].dX[1])/dDeterminant;
  Inverse_S->aV[2].dX[2] = (aCofacteurs.aV[2].dX[2])/dDeterminant;

fin :


// sortie de la fonction
return(cr_op);
}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_PermuteCirculairement
//@
//@ Description : SD_BAS_PermuteCirculairement un  tableau 1D
//@  : Cette operation permute circulairement le filtre initial , ceci est realise    */
//@  : physiquement en memoire par recopie des partie reelles et imaginaires          */
//@    attention ici tout est optimise pour des reels simple precision ( float )      */
//@
//@/*CONTEXTE D'APPEL : il est necessaire de prevoir ici un bloc temporaire de meme taille */
//@/* que le bloc a traiter et alloue par l'appelant                                        */
//@
//@ Parameters :
//@         (ENTREE)  nbpts : nombre de points
//@         (ENTREE)  comp_init : point chaud initial
//@         (ENTREE)  comp_final : point chaud final
//@         (ENTREE)  bloc_tempo : bloc temporaire
//@         (ENTREE/SORTIE)  pdtabf : tableau initial a permuter
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_PermuteCirculairement(
    int const nbpts ,               /* [E]nb de pts */
    int const comp_init ,           /* [E]composante  du point initial */
    int const comp_final ,          /* [E]composante  du point final */
    double * const bloc_tempo,      /* [E] bloc  temporaire */
    double * const pdtabf           /* [E/S] bloc initial a permuter */
  )

{
t_CMN__Statut cr_op=CMN__STATUT_OK;

    // Initialisations
    int sc;         // Decalage ramene a un decalage > 0 sur ]0, npts[
    int i;          // Compteur de boucle
    double *dpt;    // Pointeur tamporaire

    //@ Calcul du decalage
    int decalage = comp_final - comp_init;

    // @ Le traitement n'est fait que s'il y a lieu
    if(decalage%nbpts != 0)
    {
        // On ramene le decalage a l'intervalle ]0, npts[
        if(decalage < 0) sc = ((decalage%nbpts)+nbpts)%nbpts;
        else sc=decalage;

        dpt = pdtabf;
        for(i=0; i<nbpts; i++)
        {
            *(bloc_tempo + ((i+sc)%nbpts) ) = *dpt;
            dpt++;
        }

        // On recopie le buffer tamporaire dans le tableau d'entree
        for(i=0; i<nbpts; i++)
            *(pdtabf+i) = *(bloc_tempo + i);
    }



// sortie de la fonction
return(cr_op);
}

// prevoir la multiplication de complexes
//======================================================================
//                  FONCTIONS INTERNES
//======================================================================

