//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/BAS/src/SD_BAS_LibMath.h,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_BAS
//@ MODULE : SD_BAS_LibMath
//@
//@ Comment :
//@    Include file for the  SD_BAS_LibESSL class which implements
//@    standard mathematical methods   :
//@ - Matrix operations.
//@ - .....
//@ these methods are hand made in order to facilitate the
//@ portabilite of the software
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_BAS_LibMath_h
#define SD_BAS_LibMath_h


// inserer ici les fichiers include
#include <stdio.h>       /* entree/sortie */
#include <stdlib.h>      /* alloc memoire, ...*/
#include <string.h>      /* chaines de carateres */
#include <unistd.h>      /* gestion pipes, ... */
#include <time.h>        /* gestion de la date */
#include <sys/param.h>   /* constantes systeme */

#include "CMN__Commun.h"
#include "SD_BAS_Global.h"

/*====================================================================*/
//   External define  Declarations
/*====================================================================*/

#define ERR_BASE_MATH           CMN_GLB_ERR_BASE_LibMath
#define SD_BAS_LibMath_ERR_NONE         (ERR_BASE_MATH+0)
#define SD_BAS_ERR_POINT_HORS_GRILLE    (ERR_BASE_MATH+1)
#define SD_BAS_ERR_CALCUL               (ERR_BASE_MATH+2)
#define SD_BAS_ERR_NON_TROUVE           (ERR_BASE_MATH+3)

/*====================================================================*/
// Constant Declaration
/*====================================================================*/

/*====================================================================*/
// Exported types Declaration
/*====================================================================*/
/********************************************************************************
  Definition des types exportes
 *******************************************************************************/


/*====================================================================*/
// External variables Declaration
/*====================================================================*/

//====================================================================*/
//                  FONCTIONS EXTERNES
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_ProduitMatriceVecteur
//@
//@ Description : Calcule le produit d'une matrice carree 3x3 par un
//@ vecteur a 3 dimensions
//@
//@
//@ Parameters :
//@         (ENTREE)  paM_E : Matrice membre gauche du produit
//@         (ENTREE)  paA_E : Vecteur membre droit du produit
//@         (SORTIE)  paB_S : Vecteur resultat du produit
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_ProduitMatriceVecteur (
  t_SD_BAS_Matrice3x3 *paM_E,         /* Matrice membre gauche du produit */
  t_SD_BAS_Vecteur3    *paA_E,        /* Vecteur membre droit du produit */
  t_SD_BAS_Vecteur3    *paB_S         /* Vecteur resultat du produit */
  )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_TransposerMatrice
//@
//@ Description : Calculer la transposee d'une matrice 3 par 3 a
//@                 coefficients reels (double precision).
//@                 L'appel a SD_BAS_TransposerMatrice s'ecrit :
//@         SD_BAS_TransposerMatrice(aMatrice,&aTransposee)
//@     Note : Il appartient a l'utilisateur d'allouer et de desallouer les matrices
//@            passees en parametre de la fonction.
//@
//@
//@ Parameters :
//@         (ENTREE)  aMatrice_E : Matrice a transposer
//@         (SORTIE)  paTransposee_S : Tranposee de la matrice
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_TransposerMatrice(
  t_SD_BAS_Matrice3x3*          aMatrice_E,                  /* Matrice a transposer    */
  t_SD_BAS_Matrice3x3*          paTransposee_S               /* Tranposee de la matrice */
  )

;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_NormaliseVecteur
//@
//@ Description : Normalise un  vecteur de 3 elements
//@
//@
//@ Parameters :
//@         (ENTREE)  paVecteur_E : vecteur initial
//@         (SORTIE)  paVecteur_S : vecteur normalise
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_NormaliseVecteur(
  t_SD_BAS_Vecteur3    *paVecteur_E,          /* Vecteur non normalise */
  t_SD_BAS_Vecteur3    *paVecteur_S       /* Vecteur  normalise */
  )

;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_MatriceRotation
//@
//@ Description : Initialise une matrice de rotation d'angle alpha
//@
//@
//@ Parameters :
//@         (ENTREE)  dalpha : angle de la matrice ( en degre)
//@         (SORTIE)  paMatriceRot_S : Tranposee de la matrice
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_MatriceRotation(
        double           dalpha,                  /* angle    */
        double           *paMatriceRot_S          /*  matrice */
  )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_InverserMatrice3x3
//@
//@ Description : inverse une matrice
//@
//@
//@ Parameters :
//@         (ENTREE)  aMatrice_E : matrice a inverser ( en degre)
//@         (SORTIE)  paInverse_S : Inverse de la matrice
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_InverserMatrice3x3(
            t_SD_BAS_Matrice3x3* Matrice_E, /* Matrice a inverser   */
            t_SD_BAS_Matrice3x3* Inverse_S  /* Inverse de la matrice */
                           )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_PermuteCirculairement
//@
//@ Description : SD_BAS_PermuteCirculairement un  tableau 1D
//@  : Cette operation permute circulairement le filtre initial , ceci est realise    */
//@  : physiquement en memoire par recopie des partie reelles et imaginaires          */
//@    attention ici tout est optimise pour des reels simple precision ( float )      */
//@
//@/*CONTEXTE D'APPEL : il est necessaire de prevoir ici un bloc temporaire de meme taille */
//@/* que le bloc a traiter et alloue par l'appelant                                        */
//@
//@ Parameters :
//@         (ENTREE)  nbpts : nombre de points
//@         (ENTREE)  comp_init : point chaud initial
//@         (ENTREE)  comp_final : point chaud final
//@         (ENTREE)  bloc_tempo : bloc temporaire
//@         (ENTREE/SORTIE)  pdtabf : tableau initial a permuter
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_PermuteCirculairement(
    int const nbpts ,   /* [E]nb de pts */
    int const comp_init ,   /* [E]composante  du point initial */
    int const comp_final ,  /* [E]composante  du point final */
    double * const bloc_tempo,/* [E] bloc  temporaire */
    double * const pdtabf   /* [E/S] bloc initial a permuter */
  )

;

// prevoir la multiplication de complexes
//======================================================================
#endif
