//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/BAS/src/SD_BAS_Niveau0.c,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_BAS
//@ MODULE : SD_BAS_Niveau0
//@
//@ Comment :
//@    Code file for the  SD_BAS_Niveau0 class which includes the
//@    acces to the IASI L0 product  :
//@ this is done thanks to SD_BAS_Conversion and SD_BAS_Produits
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ VERSION : 20/12/2002 : Olivier Duffez
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-015-CN : 24/11/2003 : JL.PASCAL
//@ des warning sont emis lors de la compilation de l'OPS
//@ (cf IA-2100-FC-015-THA)
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-271-CN : 22/06/2005 : JL.PASCAL
//@ prise en compte du mode transition
//@ fiche de correction correspondante : IA-2100-CR-271-THA
//@ --------------------------------------------------------------------
//@ PM-ID :IA-2100-CR-301-CN : 03/07/2006 : B. MASSART
//@ PTSI is now read in the first PX valid record in N0 product to select the 
//@ BRD file. Before CR301, PTSI was read in the first CTX file where its 
//@ value is 0. (patch3-5-5). See IA-2100-CR-301-THA.
//@ --------------------------------------------------------------------
//@ MODIFIED FOR PATCH3-5-5 : 05/Jul/06
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-162-CN : 09/2006 : C.MERIGUET
//@ Gestion des IPR en mode DUMP
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-203-CN : 29/01/2007 : R.MORENO
//@ Pb de recherche du ptsi lorsque le produit commence par un AP.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-203-CN : 21/02/2007 : C.MERIGUET
//@ Pb de recherche du ptsi lorsque le produit commence par un AP.
//@ fiche de correction correspondante : IA-2100-CR-203bis-THA
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2512 (FA-2100-OPS) : 17/11/2008 : S. BRUNEL
//@ Fuites memoires signalees par EUMETSAT (EUM.EPS.NCR.3627.1)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-3358(DM-2100-OPS) : 13/04/2017 : MLA
//@ Pour produits > 2Go, il faut utiliser des long long pour stocker la position du curseur 
//@ et les fonctions fseeko et ftello
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <stdio.h>

#include "SD_BAS_Niveau0.h"
#include "SD_BAS_DECOM_DECOMMUTATION.h"
// FT-2765
#include "SD_BAS_SwapEndian.h"
// ...FT-2765

// declaration des fonctions importees

//*====================================================================*/
//* Local Constant Declaration 
//*====================================================================*/
static const int NB_BITS_BYTE = 8;  /* nombre de bits dans un octet */
static const int NB_BITS_WORD = 16; /* nombre de bits dans un mot */
static const int NB_WORDS_OFFSET_IMAGE = 26; /* offset des images dans les paquets IP */
static const int NB_WORDS_OFFSET_SPECTRE = 161; /* offset des spectres dans les paquets PX */
static const int SD_BAS_TAILLE_ENTETE = 26; /* taille des entetes de record */

static const char* SD_BAS_BLOC_DONGEN_TYPE = "SD_BAS_BLOC_DONGEN_TYPE";/* nom du bloc DONGEN TYPE de PAQUET */
static const char* SD_BAS_BLOC_DONGEN   = "SD_BAS_BLOC_DONGEN";   /* nom du bloc DONGEN */
static const char* SD_BAS_BLOC_DONGENAP = "SD_BAS_BLOC_DONGENAP"; /* nom du bloc DONGEN AP*/
static const char* SD_BAS_BLOC_DONGENVP = "SD_BAS_BLOC_DONGENVP"; /* nom du bloc DONGEN VP*/
static const char* SD_BAS_BLOC_DONGEN_DUMMY = "SD_BAS_BLOC_DONGEN_DUMMY"; /* nom du bloc DONGEN VP*/

static const char* SD_BAS_BLOC_DONDETPX = "SD_BAS_BLOC_DONDETPX"; /* nom du bloc DONDET PX*/
static const char* SD_BAS_BLOC_DONDETIP = "SD_BAS_BLOC_DONDETIP"; /* nom du bloc DONDET IP*/
static const char* SD_BAS_BLOC_DONDETAP = "SD_BAS_BLOC_DONDETAP"; /* nom du bloc DONDET AP*/

static const int SD_BAS_PACKET_PRIMARY_HEADER_LENGTH = 6; /* octets */

static const int SD_BAS_OFFSET_PTSI_PX = 58; /* CR301 */

/* numeros des erreurs du module */
#define SD_BAS_ERR_LECTURE_TYPE_PAQUET (SD_BAS_ERR_BASE_Niveau0 + 0)
#define SD_BAS_ERR_TYPE_PAQUET         (SD_BAS_ERR_BASE_Niveau0 + 1)
#define SD_BAS_ERR_DATEUTC             (SD_BAS_ERR_BASE_Niveau0 + 2)
#define SD_BAS_ERR_LONGUEUR_PX         (SD_BAS_ERR_BASE_Niveau0 + 3)
#define SD_BAS_ERR_LONGUEUR_IP         (SD_BAS_ERR_BASE_Niveau0 + 4)
#define SD_BAS_ERR_BPP_IP              (SD_BAS_ERR_BASE_Niveau0 + 5)
#define SD_BAS_ERR_VERSION             (SD_BAS_ERR_BASE_Niveau0 + 6)
#define SD_BAS_ERR_LIREMDRGENN0        (SD_BAS_ERR_BASE_Niveau0 + 7)
#define SD_BAS_ERR_LIBERELISTEVAR      (SD_BAS_ERR_BASE_Niveau0 + 8)
#define SD_BAS_ERR_LIREMPHRN0          (SD_BAS_ERR_BASE_Niveau0 + 9)
#define SD_BAS_ERR_LIREIPRN0           (SD_BAS_ERR_BASE_Niveau0 + 10)

/* libelles des messages d'erreur du module */
static char* SD_BAS_MSG_VERSION   = "INVALID MPHR VERSION";
static char* SD_BAS_MSG_LIREMPHRN0     = "INVALID LEVEL_0 MPHR";
static char* SD_BAS_MSG_LIREIPRN0     = "INVALID LEVEL_0 IPR";
static char* SD_BAS_MSG_VERSION_MDR    = "INVALID MDR VERSION";
static char* SD_BAS_MSG_LECTURE_TYPE_PAQUET = "INVALID TYPE OF PACKET";
static char* SD_BAS_MSG_TYPE_PAQUET_PX = "SPECTRUM DECODING IMPOSSIBLE: THE TYPE OF PACKET SHOULD BE PX";
static char* SD_BAS_MSG_LONGUEUR_PX    = "SPECTRUM DECODING IMPOSSIBLE: THE SIZE OF PACKET IS TOO LARGE";
static char* SD_BAS_MSG_TYPE_PAQUET_IP = "IMAGE DECODING IMPOSSIBLE: THE TYPE OF PACKET SHOULD BE IP";
static char* SD_BAS_MSG_LONGUEUR_IP    = "IMAGE DECODING IMPOSSIBLE: THE SIZE OF IMAGE IS TOO LARGE";
static char* SD_BAS_MSG_BPP_IP         = "IMAGE DECODING IMPOSSIBLE: INVALID BIT DEPTH";
static char* SD_BAS_MSG_LIREMDRGENN0   = "INVALID LEVEL_0 MDR";
static char* SD_BAS_MSG_LIBERELISTEVAR = "LIST VAR FREE FAILED";

/* CR301 */
static char* SD_BAS_MSG_GRH_RECORD_EMPTY = "N0 GRH RECORD EMPTY";
static char* SD_BAS_MSG_N0PTSI_NOT_FOUND = "N0 FIRST PTSI NOT FOUND";

const int SD_BAS_ERR_SET_Niveau0 = 1;

//*====================================================================*/
//* Local types Declaration  
//*====================================================================*/

typedef enum
{
  BPP8  = 8,  /*  8 bits par pixel */
  BPP10 = 10, /* 10 bits par pixel */
  BPP12 = 12  /* 12 bits par pixel */
} e_SD_BAS_BPP;

// CR301 Caracteristique d'un record suffisantes pour acceder au PTSI
typedef struct 
{
  short int    type;
  short int    subclass;
  unsigned int offset;
  unsigned int taille;
}t_DescProd;

//======================================================================
// Internal fonctions Declaration
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_InverserMot
//@
//@ Description : Inverse tous les bits d'un mot de 16 bits
//@               (le bit 0 devient le bit 15, etc.)
//@
//@ Parameters :
//@     (ENTREE) motIn  : mot de 16 bits en entree
//@     (SORTIE) motOut : mot de 16 bits en entree
//@
//@ Return Status : aucun
//@
//@ ------------------------------------------------------------------*/
// FT-2765
static void SD_BAS_InverserMot(
    const unsigned short   motIn,
          unsigned short * motOut
      );
// ...FT-2765

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_TypePaquet
//@
//@ Description : Retourne le type de paquet en fonction de l'APID
//@
//@ Parameters :
//@     (ENTREE) APID : Application Process ID
//@     (ENTREE/SORTIE) ctx   : contexte de gestion des erreurs
//@
//@ Return Status : aucun
//@
//@ ------------------------------------------------------------------*/
static e_SD_BAS_TypeN0 SD_BAS_TypePaquet(
  const int             APID,
  t_SD_BAS_DataThread * ctx
  );


//======================================================================
// Global variables Definition
//======================================================================

//======================================================================
//          FONCTIONS EXTERNES
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LireMPHRN0
//@
//@ Description : fonction de lecture du MPHR d'un produit 
//@  cette operation generique s'applique aux produits IASI de N0 
//@
//@
//@ Parameters :
//@     (ENTREE)  descripteur  : descripteur du produit
//@         utilise par les fonctions d'acces
//@     (SORTIE)  mphr  : liste des variables lues dans le MPHR
//@			(ENTREE/SORTIE)  ctx  : contexte de gestion des erreurs
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LireMPHRN0(
    t_SD_BAS_Fd *         Descripteur,
    t_SD_BAS_MPHR *       mphr,
    t_SD_BAS_DataThread * ctx
      )
{
  static const char tcProcName_F[] = "SD_BAS_LireMPHRN0";
  t_CMN__Statut cr_op = CMN__STATUT_OK; /* variable de retour de la fonction */
  int num_lig;             /* num de la ligne ou l'erreur est detectee */
  char* msg_err;           /* pointeur vers le message d'erreur */
  
  /* on appelle la fonction SD_BAS_LireMPHR afin de lire le MPHR */
  cr_op = SD_BAS_LireMPHR(Descripteur, mphr, ctx);
  if ( cr_op != CMN__STATUT_OK )
    {
      cr_op   = SD_BAS_ERR_LIREMPHRN0;
      msg_err = SD_BAS_MSG_LIREMPHRN0;
      num_lig = __LINE__;
      goto fin;
    }

  /* on verifie que la version lue dans le MPH est compatible avec la version
     de la couche : Descripteur->version <= SD_BAS_IASIN0_VERSION
     sinon on emet une erreur et on sort */
  if( (Descripteur->version.major > SD_BAS_IASIN0_VERSION.major) ||
      ( (Descripteur->version.major == SD_BAS_IASIN0_VERSION.major) &&
         Descripteur->version.minor  > SD_BAS_IASIN0_VERSION.minor ) )
  {
    /* affectation des variables de gestion des erreurs */
    cr_op   = SD_BAS_ERR_VERSION;
    msg_err = SD_BAS_MSG_VERSION;
    num_lig = __LINE__;
    goto fin;
  }

  /* on stocke en interne le nombre de MDR */
  Descripteur->nombreMDR = *(mphr->listevar.tab_var[MPHR_TOTAL_MDR].val.pui4);

  /* Si c'est le premier appel (etat INIT) */
  if ( Descripteur->etat == INIT )
  {
    /* on lit les IPR par appel a SD_BAS_LireIPR */
    cr_op = SD_BAS_LireIPR(Descripteur, ctx);
    if (cr_op != CMN__STATUT_OK)
      {
        cr_op   = SD_BAS_ERR_LIREIPRN0;
        msg_err = SD_BAS_MSG_LIREIPRN0;
        num_lig = __LINE__;
        goto fin;
      }

    /* on passe a etat ENCOURS */
    Descripteur->etat = ENCOURS;
    /* FinSI */
  }

  /* fin de la fonction : gestion des erreurs */
 fin:
  /* on ne met a jour la structure ctx que si une erreur a ete detectee
     et qu'elle n'a pas encore ete stockee dans ctx */
  if (cr_op != CMN__STATUT_OK) 
    {
      /* mise a jour de la variable ctx->cr.errinterne */
      SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                          SD_BAS_ERR_SET_Niveau0,
                          SD_BAS_ERR_NUM_JDB,
                          tcProcName_F,
                          num_lig,
                          msg_err);
  }

  /* sortie de la fonction */
  return(cr_op); 
}

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LireMDRGenN0
//@
//@ Description : fonction de lecture des donnees gen des IPR d'un produit 
//@  cette operation generique s'applique aux produits IASI de N0 
//@  on lit ici le grh et quelques info detaillees ( soit sequentiellement)
//@  soit a partir d'une adresse
//@
//@ SD_BAS_LireMPHRN0 doit avoir ete appele avant
//@
//@ Parameters :
//@     (ENTREE)         init  : contexte d'initialisation
//@                           init = INIT ( on lit le suivant )
//@                           init = NORMAL ( on lit a l'adresse indiquee )
//@     (ENTREE)         offset  : offset pour le mode normal
//@     (ENTREE/SORTIE)  descripteur  : descripteur du produit
//@                           utilise par les fonctions d'acces
//@     (ENTREE/SORTIE)  ctx  : contexte de gestion des erreurs
//@     (SORTIE)         dongen  : liste des variables lues dans le MDR
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LireMDRGenN0(
    const e_BAS_CtxInit   init,
    const int             offset,
    t_SD_BAS_Fd *         Descripteur,
    t_SD_BAS_DataThread * ctx,
    t_SD_BAS_Liste_Var *  dongen
      )
{
  static const char        tcProcName_F[] = "SD_BAS_LireMDRGenN0";
  t_CMN__Statut            cr_op = CMN__STATUT_OK;
  int num_lig;             /* num de la ligne ou l'erreur est detectee */
  char* msg_err;           /* pointeur vers le message d'erreur */
  char*                    bloc = NULL; /* bloc de donnees binaires lues dans le fichier */
  off_t                    vrai_offset;
  int                      lg = 0;
  t_SD_BAS_Liste_Var       grh;
  int                      nb_octets_record = 0;
  int                      validerec;   /* indicateur de validite du record */
  e_SD_BAS_TypeN0          type_paquet; /* type de paquet : PX, IP, VP ou AP */
  DECOM__T_val_var         val_var;
  static t_SD_BAS_SCDSTime dateUTCprec = {0, 0}; 
  t_SD_BAS_status_d        status_d;
  int                      nb_elements; /* valeur de retour de fonction */
  int                      ret;         /* valeur de retour de fonction */
  int                      i; /*indice de boucle */

 
//  initialisation
  dongen->tab_var=NULL;
  dongen->nb_var=0;

#ifdef INVESTIGATION
  printf("debut de SD_BAS_LireMDRGenN0\n");
  printf("etat = %d \t mode = %d \n",Descripteur->etat,Descripteur->mode);
#endif 

  
  /* initialisation de la dateUTCprec */
  if ( Descripteur->offsetMDRSuivant == Descripteur->offsetMDR )
  {
    dateUTCprec.day    = 0;
    dateUTCprec.millis = 0;
  }
  
  /* on verifie que l'etat est a INIT ou ENCOURS et que le mode est lecture */
  if ( ( (Descripteur->etat == INIT) || (Descripteur->etat == ENCOURS) ) &&
       ( Descripteur->mode == LECTURE ) )
  {
    /* si etat NORMAL */
    if ( init == CTX_NORMAL )
    {
      /* offset = offset MDR suivant */
      vrai_offset = Descripteur->offsetMDRSuivant;
    }
    /* sinon INIT */
    else
    {
      /* offset = parametre de la fonction */
      vrai_offset = offset;
    }

    /* on alloue un tableau de char de taille SD_BAS_TAILLE_GRH */
    if ( SD_BAS_MacMALLOC(bloc, char, SD_BAS_TAILLE_GRH) == FALSE )
    {
      /* affectation des variables de gestion des erreurs */
      cr_op   = SD_BAS_ERR_BAS_NIVEAU;
      msg_err = SD_BAS_MSG_ALLOC;
      num_lig = __LINE__;
      goto fin;
    }

    /* positionnement du fichier a l'offset du MDR courant */
    cr_op = fseeko(Descripteur->fd, vrai_offset, SEEK_SET);
    if ( cr_op != 0)
    {
      /* affectation des variables de gestion des erreurs */
      cr_op   = SD_BAS_ERR_BAS_NIVEAU;
      msg_err = SD_BAS_MSG_SEEK;
      num_lig = __LINE__;
      goto fin;
    }

    /* on recupere un bloc correspondant a une taille de SD_BAS_TAILLE_GRH */
    nb_elements = 1;
    ret = fread(bloc, SD_BAS_TAILLE_GRH, nb_elements, Descripteur->fd);
    if ( ret != nb_elements )
    {
      /* affectation des variables de gestion des erreurs */
      cr_op   = SD_BAS_ERR_BAS_NIVEAU;
      msg_err = SD_BAS_MSG_READ;
      num_lig = __LINE__;
      goto fin;
    }

    /* on appelle la couche de decodage afin de lire le contenu du GRH (SD_BAS_DecommuteGRH) */
    status_d = SD_BAS_DecommuteGRH(&grh, bloc);
    SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_DECOMMUTER, __LINE__);

    /* liberation du tableau */
    SD_BAS_MacFREE(bloc);

    /* on recupere ici la taille du record ( champ record_size ) */
    nb_octets_record = *(grh.tab_var[GRH_RECORD_SIZE].val.pui4);

    /* si intrument == dummy */
    if ( *(grh.tab_var[GRH_INSTRUMENT_GROUP].val.pui4) == IN_DUMMY )
    {
      //CR271
      for(i=0; i<N0_MODE_TRANSITION+1; i++)
	{
	  status_d = DECOM__Ajouter_variable (dongen, "", "VIDE",
					      DECOM__C_uint4, 1, val_var);
	  SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_AJOUTER, __LINE__);
	  if (SD_BAS_MacMALLOC(dongen->tab_var[i].val.pui4, unsigned int, 1) == FALSE)
	    {
	      cr_op   = SD_BAS_ERR_BAS_NIVEAU;
	      msg_err = SD_BAS_MSG_ALLOC;
	      num_lig = __LINE__;
	      goto fin;
	    }
	}
      /* on ajoutera juste un type de donnee dans la liste de variables qui vaut INVALIDE */
      validerec = INVALIDE;
    }
    /* sinon */
    else
    {
      /* on ajoutera juste un type de donnee dans la liste de variables qui vaut VALIDE */
      validerec = VALIDE;

      /* on recupere ici la version du record ( champ record_subclass_version ) */
      /* et on verifie que la version est < SD_BAS_MPR_VERSION */
      if ( *(grh.tab_var[GRH_RECORD_SUBCLASS_VERSION].val.pui4) > (SD_BAS_MDR_VERSION_N0) )
      {
        /* affectation des variables de gestion des erreurs */
        cr_op   = SD_BAS_ERR_VERSION;
        msg_err = SD_BAS_MSG_VERSION_MDR;
        num_lig = __LINE__;
        goto fin;
      }


      /* on alloue un tableau de char de taille recuperee dans le GRH */
      if ( SD_BAS_MacMALLOC(bloc, char, nb_octets_record) == FALSE )
      {
        /* affectation des variables de gestion des erreurs */
        cr_op   = SD_BAS_ERR_BAS_NIVEAU;
        msg_err = SD_BAS_MSG_ALLOC;
        num_lig = __LINE__;
        goto fin;
      }

      /* on se repositionne a l'offset du MDR courant */
      cr_op = fseeko(Descripteur->fd, vrai_offset, SEEK_SET);
      if ( cr_op != 0)
      {
        /* affectation des variables de gestion des erreurs */
        cr_op   = SD_BAS_ERR_BAS_NIVEAU;
        msg_err = SD_BAS_MSG_SEEK;
        num_lig = __LINE__;
        goto fin;
      }
 
      /* on lit dans le fichier ce buffer a partir de l'adresse du MDR suivant */
      nb_elements = 1;
      ret = fread(bloc, nb_octets_record, nb_elements, Descripteur->fd);
      if ( ret != nb_elements )
      {
        /* affectation des variables de gestion des erreurs */
        cr_op   = SD_BAS_ERR_BAS_NIVEAU;
        msg_err = SD_BAS_MSG_READ;
        num_lig = __LINE__;
        goto fin;
      }

      /* attention : la position des donnees ("offset") ( word i ) correspond a la spec
         de niveau 0 : le buffer en memoire commence lui a une position
         differente ( adresse word i de la spec N0 = 26 octets ( entete) + 2*(i-1) ) */

      /* on appelle la couche de decodage afin de decoder le word 1 pour determiner l'APID */
      /* DECOM__Decommuter ( bloc DONGENAP fic SD_BAS_NOM_FIC_PROD) */
      status_d = DECOM__Decommuter(bloc, lg, SD_BAS_BLOC_DONGEN_TYPE, "", dongen);
      SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_DECOMMUTER, __LINE__);

      /* pour l'instant ce qui est stocke dans *(dongen->tab_var[N0_TYPE].val.pui4) */
      /* est l'APID */
      /* le type de paquet est deduit de l'APID, que l'on va stocker dans
         *(dongen->tab_var[N0_TYPE].val.pui4) */
      type_paquet = SD_BAS_TypePaquet(*(dongen->tab_var[N0_TYPE].val.pui4), ctx);
      SD_BAS_LibereListeVar(LECTURE, dongen);
      
      /* si paquet AP */
      if ( type_paquet == AP )
      {
        /* on appelle la couche de decodage afin de lire le contenu du MDR */
        /* DECOM__Decommuter ( bloc DONGENAP fic SD_BAS_NOM_FIC_PROD) */
        status_d = DECOM__Decommuter(bloc, lg, SD_BAS_BLOC_DONGENAP, "", dongen);
        SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_DECOMMUTER, __LINE__);
      }
      /* sinon si paquet VP */
      else if ( type_paquet == VP )
      {
        /* on appelle la couche de decodage afin de lire le contenu du MDR */
        /* DECOM__Decommuter ( bloc DONGENAP fic SD_BAS_NOM_FIC_PROD) */
        status_d = DECOM__Decommuter(bloc, lg, SD_BAS_BLOC_DONGENVP, "", dongen);
        SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_DECOMMUTER, __LINE__);
      }
      /* sinon */
      else
      {
        /* on appelle la couche de decodage afin de lire le contenu du MDR
           DECOM__Decommuter ( bloc DONGEN fic SD_BAS_NOM_FIC_PROD) */
        status_d = DECOM__Decommuter(bloc, lg, SD_BAS_BLOC_DONGEN, "", dongen);
        SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_DECOMMUTER, __LINE__);
      }
      /* fin si */

      /* liberation du tableau */
      SD_BAS_MacFREE(bloc);



      /* mise a jour du type de paquet (pour l'instant c'est l'APID qui est stocke) */
      *(dongen->tab_var[N0_TYPE].val.pui4) = type_paquet;


      /* mise a jour de la taille du packet data du N0 a lire pour fabriquer  */
      /* le produit de verification. La longueur ecrite dans le N0 correspond */
      /* a l'indice SSC (Source Sequence Count) de chaque APID (Application   */
      /* Process IDentifier) = taille du Packet Data Field - 1                */
      /* Le premier paquet doit etre indice a 0 d'ou le -1                    */
      *(dongen->tab_var[N0_LONGUEURDATA].val.pui4) = 
        *(dongen->tab_var[N0_LONGUEURDATA].val.pui4) +
        SD_BAS_PACKET_PRIMARY_HEADER_LENGTH + 1;

      /* fin du test sur le type d'instrument */
    }


    /* on ajoute dans la liste des variables decommutees la taille du record */
    status_d = DECOM__Ajouter_variable (dongen, "", "N0_LONGUEURREC",
                                        DECOM__C_uint4, 1, val_var);
    SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_AJOUTER, __LINE__);
    if (SD_BAS_MacMALLOC(dongen->tab_var[N0_LONGUEURREC].val.pui4, unsigned int, 1)
        == FALSE)
    {
      cr_op   = SD_BAS_ERR_BAS_NIVEAU;
      msg_err = SD_BAS_MSG_ALLOC;
      num_lig = __LINE__;
      goto fin;
    }
    *(dongen->tab_var[N0_LONGUEURREC].val.pui4) = nb_octets_record;

    /* on ajoute  dans la liste des variables decommutees l'indicateur de validite */
    status_d = DECOM__Ajouter_variable (dongen, "", "N0_VALIDREC",
                                        DECOM__C_uint4, 1, val_var);
    SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_AJOUTER, __LINE__);
    if (SD_BAS_MacMALLOC(dongen->tab_var[N0_VALIDREC].val.pui4, unsigned int, 1)
        == FALSE)
    {
      cr_op   = SD_BAS_ERR_BAS_NIVEAU;
      msg_err = SD_BAS_MSG_ALLOC;
      num_lig = __LINE__;
      goto fin;
    }
    *(dongen->tab_var[N0_VALIDREC].val.pui4) = validerec;

    /* on ajoute dans la liste des variables decommutees l'offset */
    status_d = DECOM__Ajouter_variable (dongen, "", "N0_OFFSET",
                                        DECOM__C_uint4, 1, val_var);
    SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_AJOUTER, __LINE__);
    if (SD_BAS_MacMALLOC(dongen->tab_var[N0_OFFSET].val.pui4, unsigned int, 1)
        == FALSE)
    {
      cr_op   = SD_BAS_ERR_BAS_NIVEAU;
      msg_err = SD_BAS_MSG_ALLOC;
      num_lig = __LINE__;
      goto fin;
    }
    *(dongen->tab_var[N0_OFFSET].val.pui4) = vrai_offset;
   
     /* on ajoute dans la liste des variables decommutees la date de debut */
    status_d = DECOM__Ajouter_variable (dongen, "", "N0_DATEDEB",
                                        DECOM__C_date, 1, val_var);
    SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_AJOUTER, __LINE__);

    if (SD_BAS_MacMALLOC(dongen->tab_var[N0_DATEDEB].val.pdate, t_SD_BAS_SCDSTime, 1)
        == FALSE)
      {
	cr_op   = SD_BAS_ERR_BAS_NIVEAU;
	msg_err = SD_BAS_MSG_ALLOC;
	num_lig = __LINE__;
	goto fin;
      }
    
    dongen->tab_var[N0_DATEDEB].val.pdate->day = grh.tab_var[GRH_RECORD_START_TIME].val.pdate->day;
    dongen->tab_var[N0_DATEDEB].val.pdate->millis = grh.tab_var[GRH_RECORD_START_TIME].val.pdate->millis;

    /* on ajoute dans la liste des variables decommutees la date de debut */
    status_d = DECOM__Ajouter_variable (dongen, "", "N0_DATEFIN",
                                        DECOM__C_date, 1, val_var);
    SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_AJOUTER, __LINE__);

    if (SD_BAS_MacMALLOC(dongen->tab_var[N0_DATEFIN].val.pdate, t_SD_BAS_SCDSTime, 1)
        == FALSE)
      {
	cr_op   = SD_BAS_ERR_BAS_NIVEAU;
	msg_err = SD_BAS_MSG_ALLOC;
	num_lig = __LINE__;
	goto fin;
      }
    
    dongen->tab_var[N0_DATEFIN].val.pdate->day = grh.tab_var[GRH_RECORD_STOP_TIME].val.pdate->day;
    dongen->tab_var[N0_DATEFIN].val.pdate->millis = grh.tab_var[GRH_RECORD_STOP_TIME].val.pdate->millis;
         
  /* on teste la date UTC par rapport a la date UTC precedente et on 
     retourne une erreur si date UTC <= date UTC precedente */
    if ( (dongen->tab_var[N0_DATEUTC].val.pdate->day    <  dateUTCprec.day) ||
      ((dongen->tab_var[N0_DATEUTC].val.pdate->day    == dateUTCprec.day) &&
       (dongen->tab_var[N0_DATEUTC].val.pdate->millis <  dateUTCprec.millis)) )
    {
    /* affectation des variables de gestion des erreurs */
      validerec = INCOHERENT;
//    cr_op   = SD_BAS_ERR_DATEUTC;
//    msg_err = SD_BAS_MSG_DATEUTC;
//    num_lig = __LINE__;
//    goto fin;
    }
  /* mise a jour de la valeur de la date UTC precedente */
  dateUTCprec = *(dongen->tab_var[N0_DATEUTC].val.pdate);

  /* on renseigne dateOBTprec */
  Descripteur->dateOBTprec = *(dongen->tab_var[N0_DATEOBT].val.pui4);


  /* on libere le grh lu */
  SD_BAS_LibereListeVar(LECTURE, &grh);

  } /* fin du test sur l'etat et le mode */
  else
  {
    /* affectation des variables de gestion des erreurs */
    cr_op   = SD_BAS_ERR_LIREMDRGENN0;
    msg_err = SD_BAS_MSG_LIREMDRGENN0;
    num_lig = __LINE__;
    goto fin;
  }



  /* fin de la fonction : gestion des erreurs */
  fin:

  /* on met a jour offsetMDRsuivant = offsetMDRsuivant + record_size */
  Descripteur->offsetMDRSuivant += nb_octets_record;
 
  /* on ne met a jour la structure ctx que si une erreur a ete detectee
     et qu'elle n'a pas encore ete stockee dans ctx */
  if ( cr_op != CMN__STATUT_OK)
  {
    /* mise a jour de la variable ctx->cr.errinterne */
    SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
      SD_BAS_ERR_SET_Niveau0,
      SD_BAS_ERR_NUM_JDB,
      tcProcName_F,
      num_lig,
      msg_err);
  }

  /* sortie de la fonction */
  return(cr_op); 
}

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LireMDRN0
//@
//@ Description : fonction de lecture  des MDR de type PX IP AP VP d'un produit 
//@  cette operation s'applique aux produits IASI de N0 
//@  on lit ici les info detaillees a partir de l'offset
//@
//@
//@ Parameters :
//@     (ENTREE)  descripteur  : descripteur du produit
//@         utilise par les fonctions d'acces
//@     (ENTREE)  offset  : offset issu de la lecture des donnees generales
//@     (ENTREE/SORTIE)  ctx  : contexte de gestion des erreurs
//@     (SORTIE)  mdr  : liste des variables lues dans le MDR
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LireMDRN0(
    t_SD_BAS_Fd *       Descripteur,
    const int           offset,
		t_SD_BAS_DataThread * ctx,
    t_SD_BAS_PaquetN0 * mdr
    )
{
  static const char tcProcName_F[] = "SD_BAS_LireMDRN0";
  t_CMN__Statut      cr_op = CMN__STATUT_OK;
  int num_lig;             /* num de la ligne ou l'erreur est detectee */
  char* msg_err;           /* pointeur vers le message d'erreur */
  t_SD_BAS_status_d  status_d;
  int                ret;         /* valeur de retour de fonction */
  int                lg = 0;
  int                nb_octets_record;
  e_BAS_ValidRec     validite;
  e_SD_BAS_TypeN0    type_paquet;
  int nb_elements;
  char * bloc;

  /* on verifie que l'etat est a INIT ou ENCOURS et que le mode est lecture */
  if ( ( (Descripteur->etat == INIT) || (Descripteur->etat == ENCOURS) ) &&
       ( Descripteur->mode == LECTURE ) )
  {
    /* on initialise la structure mdr->InfoGen par appel a SD_BAS_LireMDRGenN0 */
    /* (init = NORMAL) en specifiant l'adresse du MDR a lire */
    cr_op = SD_BAS_LireMDRGenN0( CTX_INIT, offset, Descripteur, ctx, &(mdr->InfoGen) );
    if ( cr_op != CMN__STATUT_OK )
    {
      /* affectation des variables de gestion des erreurs */
      cr_op   = SD_BAS_ERR_LIREMDRGENN0;
      msg_err = SD_BAS_MSG_LIREMDRGENN0;
      num_lig = __LINE__;
      goto fin;
    }

    /* on recupere dans la liste des variables decommutees la taille du tableau de data */
    nb_octets_record = *(mdr->InfoGen.tab_var[N0_LONGUEURREC].val.pui4);

    /* ainsi que la validite du record */
    validite = *(mdr->InfoGen.tab_var[N0_VALIDREC].val.pui4);

    /* si le record n'est pas invalide */
    if ( validite != INVALIDE )
    {
 
   /* on s'alloue un tableau de char de longueur taille data du record */
      if ( SD_BAS_MacMALLOC(bloc, char, nb_octets_record) == FALSE )
      {
        cr_op   = SD_BAS_ERR_BAS_NIVEAU;
        msg_err = SD_BAS_MSG_ALLOC;
        num_lig = __LINE__;
        goto fin;
      }

      // on recupere un bloc correspondant a une taille data du record dans le
      // fichier prealablement ouvert a l'offset du MDR courant + 26 octets
      cr_op = fseeko(Descripteur->fd, offset, SEEK_SET);
      if ( cr_op != 0)
      {
        /* affectation des variables de gestion des erreurs */
        cr_op   = SD_BAS_ERR_BAS_NIVEAU;
        msg_err = SD_BAS_MSG_SEEK;
        num_lig = __LINE__;
        goto fin;
      }

      nb_elements = 1;
      ret = fread(bloc, nb_octets_record, nb_elements, Descripteur->fd);
      if ( ret != nb_elements )
      {
        cr_op   = SD_BAS_ERR_BAS_NIVEAU;
        msg_err = SD_BAS_MSG_READ;
        num_lig = __LINE__;
        goto fin;
      }

      // selon le type de paquet (IP PX AP) on appelle la couche de decodage afin de lire 
      // le contenu du MDR  
      // DECOM__Decommuter ( bloc SD_BAS_BLOC_ISP_PX ou IP ou AP fic SD_BAS_NOM_FIC_PROD)
      // on stocke ceci dans le champ InfoDet;
      type_paquet = (e_SD_BAS_TypeN0)*(mdr->InfoGen.tab_var[N0_TYPE].val.pui4);
 

      switch ( type_paquet )
      {
        /* paquet spectre */
      case PX:
#ifdef OPS_DEBUG
        printf("paquet PX\n");
#endif
        status_d = DECOM__Decommuter(bloc,
                                     lg,
                                     SD_BAS_BLOC_DONDETPX,
                                     "",
                                     &(mdr->InfoDet) );
        SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_DECOMMUTER, __LINE__);
        break;
      
      /* paquet image */
      case IP:
#ifdef OPS_DEBUG
        printf("paquet IP\n");
#endif
        status_d = DECOM__Decommuter(bloc,
                                     lg,
                                     SD_BAS_BLOC_DONDETIP,
                                     "",
                                     &(mdr->InfoDet) );
        SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_DECOMMUTER, __LINE__);
        break;

      /* paquet de verification */
      case VP:
#ifdef OPS_DEBUG
        printf("paquet VP\n");
#endif
        /* ********************** rien a decoder !!! ********************** */
        break;

      /* paquet auxiliaire */
      case AP:
#ifdef OPS_DEBUG
        printf("paquet AP\n");
#endif
        status_d = DECOM__Decommuter(bloc,
                                     lg,
                                     SD_BAS_BLOC_DONDETAP,
                                     "",
                                     &(mdr->InfoDet) );
        SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_DECOMMUTER, __LINE__);
        break;

      default:
        cr_op   = SD_BAS_ERR_LECTURE_TYPE_PAQUET;
        msg_err = SD_BAS_MSG_LECTURE_TYPE_PAQUET;
        num_lig = __LINE__;
        goto fin;
      } /* fin du switch */


      /* on s'alloue un tableau de char de longueur taille data du record */
      /* que l'on stocke dans data de mdr->InfoCod */
      if ( SD_BAS_MacMALLOC(mdr->InfoCod.data, char, nb_octets_record - SD_BAS_TAILLE_ENTETE) == FALSE )
      {
        cr_op   = SD_BAS_ERR_BAS_NIVEAU;
        msg_err = SD_BAS_MSG_ALLOC;
        num_lig = __LINE__;
        goto fin;
      }

      /* on recopie les infos utiles dons mdr->infocod sans l'entete */
      memcpy(mdr->InfoCod.data, bloc + SD_BAS_TAILLE_ENTETE, nb_octets_record - SD_BAS_TAILLE_ENTETE);

      /* on libere bloc */
      SD_BAS_MacFREE(bloc);

      /* Mise a jour du type de paquet*/
      mdr->type = type_paquet;

    } /* fin du cas ou le record est valide */

  } /* fin du test sur l'etat et le mode */

  /* fin de la fonction : gestion des erreurs */
  fin:
  /* on ne met a jour la structure ctx que si une erreur a ete detectee
     et qu'elle n'a pas encore ete stockee dans ctx */
  if (cr_op != CMN__STATUT_OK)
  {
    /* mise a jour de la variable ctx->cr.errinterne */
    SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
      SD_BAS_ERR_SET_Niveau0,
      SD_BAS_ERR_NUM_JDB,
      tcProcName_F,
      num_lig,
      msg_err);

  }

  /* sortie de la fonction */
  return(cr_op); 
}

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LibereMDRN0
//@
//@ Description : fonction de liberation des MDR  d'un produit 
//@  cette operation s'applique aux produits IASI de N0 
//@  on desalloue ici toute la place allouee precedenment 
//@
//@
//@ Parameters :
//@     (ENTREE/SORTIE)  ctx  : contexte de gestion des erreurs
//@     (ENTREE/SORTIE)  mdr  : liste des variables lues dans le MDR
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LibereMDRN0(
				 t_SD_BAS_DataThread * ctx,
				 t_SD_BAS_PaquetN0 *   mdr
				 )
{
  static const char tcProcName_F[] = "SD_BAS_LibereMDRN0";
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  int num_lig;             /* num de la ligne ou l'erreur est detectee */
  char* msg_err;           /* pointeur vers le message d'erreur */


    /* on libere la liste dans la structure mdr->InfoGen par appel a SD_BAS_LibereListeVar
      (liberation globale) */
    cr_op = SD_BAS_LibereListeVar(LECTURE, &(mdr->InfoGen));
    if ( cr_op != CMN__STATUT_OK )
    {
      /* affectation des variables de gestion des erreurs */
      cr_op   = SD_BAS_ERR_LIBERELISTEVAR;
      msg_err = SD_BAS_MSG_LIBERELISTEVAR;
      num_lig = __LINE__;
      goto fin;
    }
    
    /* on libere la liste dans la structure mdr->InfoDet par appel a SD_BAS_LibereListeVar
      (liberation globale) */
    if(mdr->InfoDet.nb_var != 0)
      {
	cr_op = SD_BAS_LibereListeVar(LECTURE, &(mdr->InfoDet));
	if ( cr_op != CMN__STATUT_OK )
	  {
	    /* affectation des variables de gestion des erreurs */
	    cr_op   = SD_BAS_ERR_LIBERELISTEVAR;
	    msg_err = SD_BAS_MSG_LIBERELISTEVAR;
	    num_lig = __LINE__;
	    goto fin;
	  }
      }
    /* on libere le tableau data dans la structure mdr->Infocod */
    SD_BAS_MacFREE(mdr->InfoCod.data);


  /* fin de la fonction : gestion des erreurs */
  fin:
  /* on ne met a jour la structure ctx que si une erreur a ete detectee
     et qu'elle n'a pas encore ete stockee dans ctx */
  if (cr_op != CMN__STATUT_OK)
  {
    /* mise a jour de la variable ctx->cr.errinterne */
    SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
      SD_BAS_ERR_SET_Niveau0,
      SD_BAS_ERR_NUM_JDB,
      tcProcName_F,
      num_lig,
      msg_err);

  }

  /* sortie de la fonction */
  return(cr_op); 
}

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_DecodeSpectreN0
//@
//@ Description : fonction de decodage du spectre de N0  
//@  cette operation s'applique aux produits IASI de N0 
//@   la place allouee precedemment est allouee par l'appelant
//@
//@ Parameters :
//@     (ENTREE)        mdr      : contenu du MDR
//@     (ENTREE/SORTIE) ctx      : contexte de gestion des erreurs
//@     (SORTIE)        spectre  : contenu du spectre
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_DecodeSpectreN0(
				     t_SD_BAS_PaquetN0 *   mdr,
				     t_SD_BAS_DataThread * ctx,
				     char *                spectre
				     )
{
  static const char tcProcName_F[] = "SD_BAS_DecodeSpectreN0";
  t_CMN__Statut    cr_op = CMN__STATUT_OK;
  int              num_lig;      /* num de la ligne ou l'erreur est detectee */
  char *           msg_err;      /* pointeur vers le message d'erreur */
  char * p_spectre = NULL;       /* pointeur de parcours du spectre de sortie */
  unsigned short * p_words   = NULL; /* pointeur de parcours des mots de 16 bits */
  unsigned short   mot16;        /* variable temporaire de calcul (sur 16 bits) */
  int              nb_bits;      /* nombre de bits a lire */
  int              nb_mots;      /* nombre de mots de 16 bits a lire */
  int              iMot;         /* variable de boucle sur les mots de 16 bits */
  unsigned short int* tampon = NULL;
  /* on verifie que le type est PX */
  if ( *(mdr->InfoGen.tab_var[N0_TYPE].val.pui4) != PX)
    {
      
#ifdef OPS_DEBUG
      printf("Erreur dans SD_BAS_DecodeSpectreN0 : type paquet = %d au lieu de %d\n",
             *(mdr->InfoGen.tab_var[N0_TYPE].val.pui4),
             PX);
#endif
      
      cr_op   = SD_BAS_ERR_TYPE_PAQUET;
      msg_err = SD_BAS_MSG_TYPE_PAQUET_PX;
      num_lig = __LINE__;
      goto fin;
    }

  /* on travaille ici directement sur le tableau data dans la structure mdr->InfoCod */
  
  /* le nombre d'echantillons est issu de la variable SD_BAS_SS0 */
  /* la taille d'un echantillon est issue de PX_IDEFCODNBBITMAX */
  /* on verifie que PX_IDEFCODNBBITMAX * SD_BAS_SS0 + 160word<longueur du tableau Data */
  /* sinon erreur de taille */
  nb_bits = *(mdr->InfoDet.tab_var[PX_IDEFCODNBBITMAX].val.pui4);
  
  if ( nb_bits > (*(mdr->InfoGen.tab_var[N0_LONGUEURDATA].val.pui4)) * NB_BITS_BYTE )
    {
      cr_op   = SD_BAS_ERR_LONGUEUR_PX;
      msg_err = SD_BAS_MSG_LONGUEUR_PX;
      num_lig = __LINE__;
      goto fin;
    }

  /* calcul du nombre de mots (le dernier etant eventuellement partiellement rempli) */
  nb_mots = nb_bits / NB_BITS_WORD;
  /* on cherche le multiple sup */
  if ((nb_bits % NB_BITS_WORD) != 0)
  {
    nb_mots ++;
  }

  /* initialisation des pointeurs de parcours */
  /* on travaille a partir du word NB_WORDS_OFFSET_SPECTRE = 161 */
  p_spectre = spectre;

#ifdef COMPIL_INSURE
  tampon = (unsigned short int*) malloc(nb_mots*2);
  memset(tampon, 0, nb_mots);
  memcpy(tampon, ((unsigned short*)(mdr->InfoCod.data)) + (NB_WORDS_OFFSET_SPECTRE - 1), nb_bits/8);
  p_words = tampon;
#else
  p_words   = ((unsigned short*)(mdr->InfoCod.data)) + (NB_WORDS_OFFSET_SPECTRE - 1) ;
#endif

  /* pour chaque mot de 16 bits */

  for (iMot=0; iMot < nb_mots ; iMot++)
  {

    
    /* le spectre est stocke par mots de 16 bits dont les bits sont inverses */
    /* il faut donc inverser les bits (0 => 15, 1 => 14, etc.) */
    /* cf p 37 DA22 */
    SD_BAS_InverserMot(*p_words, &mot16);

    /* recopie des 2 octets du mot de 16 bits dans le tableau du spectre */
    /* octet de poids fort du mot de 16 bit inverse */
    *p_spectre = 0x00FF & (mot16 >> NB_BITS_BYTE);
    p_spectre ++;

    /* octet de poids faible du mot de 16 bit inverse */
    *p_spectre = 0x00FF & mot16;
    p_spectre ++;

    /* incrementation du pointeur d'entree */
    p_words++;

    /* fin pour chaque mot de 16 bits */
  }

  // FT-2765
  SWAP_INT_BUFFER( spectre, 2, nb_mots );
  // ...FT-2765

#ifdef COMPIL_INSURE
  free(tampon);
#endif

  /* fin de la fonction : gestion des erreurs */
  fin:
  /* on ne met a jour la structure ctx que si une erreur a ete detectee
     et qu'elle n'a pas encore ete stockee dans ctx */
  if (cr_op != CMN__STATUT_OK)

  {
    /* mise a jour de la variable ctx->cr.errinterne */
    SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
      SD_BAS_ERR_SET_Niveau0,
      SD_BAS_ERR_NUM_JDB,
      tcProcName_F,
      num_lig,
      msg_err);

  }

  /* sortie de la fonction */
  return(cr_op); 
}

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_DecodeImageN0
//@
//@ Description : fonction de decodage d'une image de N0  
//@  cette operation s'applique aux produits IASI de N0 
//@   la place allouee precedenment est allouee par l'appelant
//@
//@
//@ Parameters :
//@     (ENTREE)        mdr   : contenu du MDR
//@     (ENTREE/SORTIE) ctx   : contexte de gestion des erreurs
//@     (ENTREE/SORTIE) image : contenu de l'image
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_DecodeImageN0(
				   t_SD_BAS_PaquetN0 *   mdr,
				   t_SD_BAS_DataThread * ctx,
				   double *              image
				   )
{
  static const char    tcProcName_F[] = "SD_BAS_DecodeImageN0";
  t_CMN__Statut        cr_op = CMN__STATUT_OK;
  int                  num_lig;        /* num de la ligne ou l'erreur est detectee */
  char*                msg_err;        /* pointeur vers le message d'erreur */
  int                  iPixel;         /* variable de boucle sur les pixels */
  int                  iPaquet;        /* variable de boucle sur les paquets d'octets */
  int                  iEq;            /* variable de boucle pour lire le compteur d'eg. */
  int                  nb_bits;        /* nombre de bits a lire */
  e_SD_BAS_BPP         nb_bits_pixel;  /* nombre de bits par pixel */
  int                  nb_pixels;      /* nombre de pixels */
  int                  nb_paquets;     /* nombre de paquets de pixels */
  char *               p_char  = NULL; /* pointeur de parcours du tableau d'octets entree */
  unsigned short int * p_ushort = NULL;/* pointeur de parcours du tableau d'octets entree */  
  double *             p_image = NULL; /* pointeur de parcours de l'image de sortie */
  int                  eq_counter = 0; /* compteur d'egalisation entier sur 4 octets */
  int                  val_pixel = 0;  /* valeur d'un pixel, stocke dans un entier */
  // FT-2765
  unsigned short int i_ushort_0;
  unsigned short int i_ushort_1;
  unsigned short int i_ushort_2;
  unsigned short int i_ushort_3;
  unsigned short int i_ushort_4;
  // ...FT-2765

  /* on verifie que le type = IP */
  if ( *(mdr->InfoGen.tab_var[N0_TYPE].val.pui4) != IP)
    {

#ifdef OPS_DEBUG
      printf("Erreur dans SD_BAS_DecodeImageN0 : type paquet = %d au lieu de %d\n",
             *(mdr->InfoGen.tab_var[N0_TYPE].val.pui4),
             IP);
#endif

      cr_op   = SD_BAS_ERR_TYPE_PAQUET;
      msg_err = SD_BAS_MSG_TYPE_PAQUET_IP;
      num_lig = __LINE__;
      goto fin;
    }

  /* on travaille ici directement sur le tableau data dans la structure mdr->InfoCod */
    /* le nombre d'echantillons est issu de IP_IISSAMPLE */
  /* le nombre de bits de chaque echantillon est issu de IP_IISSIZE */
  
  /* on verifie que (SD_BAS_IMLI*SD_BAS_IMCO+4)*IP_IISSIZE + 26word<longueur du tableau Data */
  /* sinon erreur de taille */
  /* une image IASI contient un entete de 4 echantillons (compteur d'egalisation) */
  /* SD_BAS_IMLI   = nombre de lignes   d'une image IASI */
  /* SD_BAS_IMCO   = nombre de colonnes d'une image IASI */
  /* nb_bits_pixel = nombre de bits de chaque pixel d'une image IASI (8, 10 ou 12) */
  nb_bits_pixel  = (e_SD_BAS_BPP)(*(mdr->InfoDet.tab_var[IP_IISSIZE].val.pui4));
  nb_pixels      = SD_BAS_IMLI * SD_BAS_IMCO;
  nb_bits        = (nb_pixels + 4) * nb_bits_pixel;
  
  if ( nb_bits > (*(mdr->InfoGen.tab_var[N0_LONGUEURDATA].val.pui4) * NB_BITS_BYTE) )
    {
      cr_op   = SD_BAS_ERR_LONGUEUR_IP;
      msg_err = SD_BAS_MSG_LONGUEUR_IP;
      num_lig = __LINE__;
      goto fin;
  }
  
  
  /* on travaille a partir du word NB_WORDS_OFFSET_IMAGE = 26 */
  p_char  =  mdr->InfoCod.data + ((NB_WORDS_OFFSET_IMAGE - 1) * NB_BITS_WORD/NB_BITS_BYTE);
  //  p_char  = mdr->InfoCod.data;

  /* on positionne un pointeur de parcours sur le debut du tableau image de sortie */
  p_image = image;

  /* initialisation du compteur d'egalisation */
  eq_counter = 0;

  /* on distingue 3 cas selon la taille de chaque pixel en nb de bits */
  switch ( nb_bits_pixel )
  {
    /*---------------------- 8 bits par pixel -------------------------------*/
    case BPP8:
      /* lecture du compteur d'egalisation sur 4 octets */
      /* on suppose que le 1er octet lu est l'octet de poids fort */
      for (iEq=0; iEq<4; iEq++)
      {
        eq_counter = eq_counter * 256 + (*p_char);
        p_char++;
      }

      /* on stocke la valeur du compteur dans un double, au debut du tableau image */
      *p_image = (double)(eq_counter);
#ifdef OPS_DEBUG
        printf("compteur d'egalisation = 0x%x (=%f)\n", eq_counter, *p_image);
#endif
      p_image++;

      /* pour chaque pixel */
      for (iPixel=0; iPixel < nb_pixels; iPixel++)
      {
        /* on extrait 8 bits du buffer */
        /* on stocke la valeur dans le double en sortie (stockage en colonne) */
        *(p_image++) = (double)*(p_char++);
      /* fin pour */
      }
      break;

    /*---------------------- 10 bits par pixel ------------------------------*/
    case BPP10:
      /* lecture du compteur d'egalisation : 32 bits repartis sur 5 octets */
      /* on suppose que le 1er octet lu est l'octet de poids fort */
      eq_counter = p_char[0];

      /* les 8 bits suivants sont repartis
         en 6 derniers (masque 0x3F) + 2 premiers (masque 0xC0) */
      eq_counter = eq_counter * 256
        + ((p_char[1] & 0x3F) << 2)
        + ((p_char[2] & 0xC0) >> 6);

      /* les 8 bits suivants sont repartis
         en 4 derniers (masque 0x0F) + 4 premiers (masque 0xF0) */
      eq_counter = eq_counter * 256
        + ((p_char[2] & 0x0F) << 4)
        + ((p_char[3] & 0xF0) >> 4);

      /* les 8 bits suivants sont repartis
         en 2 derniers (masque 0x03) + 6 premiers (masque 0xFC) */
      eq_counter = eq_counter * 256
        + ((p_char[3] & 0x03) << 6)
        + ((p_char[4] & 0xFC) >> 2);

      /* on stocke la valeur du compteur dans un double, au debut du tableau image */
      *p_image = (double)(eq_counter);
#ifdef OPS_DEBUG
        printf("compteur d'egalisation = 0x%x (=%f)\n", eq_counter, *p_image);
#endif
      p_image++;

      /* on saute les 4 echantillons (compteur d'egalisation) */
      /* le nb d'octets a sauter est 4 * BPP10 / NB_BITS_BYTE = 5 */
      p_char += 5;

	     p_ushort = (unsigned short int *)p_char;
	 
     	// on va lire les pixels par paquets de 8 pixels = 80 bits = 10 octets = 5 shortint
      for (iPaquet=0; iPaquet < nb_pixels/8; iPaquet++)
      {
      	// FT-2765
      	i_ushort_0 = p_ushort[0];
      	i_ushort_1 = p_ushort[1];
      	i_ushort_2 = p_ushort[2];
      	i_ushort_3 = p_ushort[3];
      	i_ushort_4 = p_ushort[4];

      	SWAP_INT_BUFFER( &i_ushort_0, 2, 1 );
      	SWAP_INT_BUFFER( &i_ushort_1, 2, 1 );
      	SWAP_INT_BUFFER( &i_ushort_2, 2, 1 );
      	SWAP_INT_BUFFER( &i_ushort_3, 2, 1 );
      	SWAP_INT_BUFFER( &i_ushort_4, 2, 1 );

      	// 1er pixel : 10 premiers bits
          val_pixel = ((i_ushort_0 & 0xFFC0) >> 6);
          *(p_image++) = (double)val_pixel;

          // 2eme pixel : 6 derniers bits (masque 0x003F) + 4 premiers bits (masque 0xF000)
         	val_pixel = ((i_ushort_0 & 0x003F) << 4) + ((i_ushort_1 & 0xF000) >> 12);
         	*(p_image++) = (double)val_pixel;

         	// 3eme pixel : Bits 5 a 14 (masque 0x0FFC)
         	val_pixel = ((i_ushort_1 & 0x0FFC) >> 2);
         	*(p_image++) = (double)val_pixel;

         	// 4eme pixel : 2 derniers bits (masque 0x0003) + 8 premiers bits (masque 0xFF00)
         	val_pixel = ((i_ushort_1 & 0x0003) << 8) + ((i_ushort_2 & 0xFF00) >> 8);
         	*(p_image++) = (double)val_pixel;

         	// 5eme pixel : 8 derniers bits (masque 0x00FF) + 2 premiers bits (masque 0xC000)
         	val_pixel = ((i_ushort_2 & 0x00FF) << 2 ) + ((i_ushort_3 & 0xC000) >> 14);
         	*(p_image++) = (double)val_pixel;


         	// 6eme pixel : Bits 3 a 12 (masque 0x3FF0)
         	val_pixel = ((i_ushort_3 & 0x3FF0) >> 4 ) ;
         	*(p_image++) = (double)val_pixel;

         	// 7eme pixel : 4 derniers bits (masque 0x000F) + 6 premiers bits (masque 0xFC00)
         	val_pixel = ((i_ushort_3 & 0x000F) << 6 ) + ((i_ushort_4 & 0xFC00) >> 10);
         	*(p_image++) = (double)val_pixel;

         	// 8eme pixel : 10 derniers bits (masque 0x03FF)
         	val_pixel = i_ushort_4 & 0x03FF;
         	*(p_image++) = (double)val_pixel;
         	// ...FT-2765

        // on decale le pointeur d'entree de 5 shortint 
       	p_ushort += 5;		

      // fin pour
     	}
     	break; 	


    /*---------------------- 12 bits par pixel ------------------------------*/
    case BPP12:
      /* lecture du compteur d'egalisation : 32 bits repartis sur 6 octets */
      /* on suppose que le 1er octet lu est l'octet de poids fort */
      eq_counter = p_char[0];

      /* les 8 bits suivants sont repartis
         en 4 derniers (masque 0x0F) + 4 premiers (masque 0xF0) */
      eq_counter = eq_counter * 256
        + ((p_char[1] & 0x0F) << 4)
        + ((p_char[2] & 0xF0) >> 4);

      /* les 8 bits suivants sont cales sur l'octet */
      eq_counter = eq_counter * 256 + p_char[3];

      /* les 8 bits suivants sont repartis
         en 4 derniers (masque 0x0F) + 4 premiers (masque 0xF0) */
      eq_counter = eq_counter * 256
        + ((p_char[4] & 0x0F) << 4)
        + ((p_char[5] & 0xF0) >> 4);

      /* on stocke la valeur du compteur dans un double, au debut du tableau image */
      *p_image = (double)(eq_counter);
#ifdef OPS_DEBUG
        printf("compteur d'egalisation = 0x%x (=%f)\n", eq_counter, *p_image);
#endif
      p_image++;

      /* on saute les 4 echantillons (compteur d'egalisation) */
      /* le nb d'octets a sauter est 4 * BPP12 / NB_BITS_BYTE = 6 */
      p_char += 6;

     	p_ushort = (unsigned short int *)p_char;  
	    
      // on va lire les pixels par paquets de 4 pixels = 6 octets 
      nb_paquets = nb_pixels / 4;     

      for (iPaquet=0; iPaquet < nb_paquets; iPaquet++)
      {
          // FT-2765
      	  i_ushort_0 = p_ushort[0];
      	  i_ushort_1 = p_ushort[1];
      	  i_ushort_2 = p_ushort[2];

      	  SWAP_INT_BUFFER( &i_ushort_0, 2, 1 );
      	  SWAP_INT_BUFFER( &i_ushort_1, 2, 1 );
      	  SWAP_INT_BUFFER( &i_ushort_2, 2, 1 );

      	  // 1er pixel : 12 premier bits(masque 0xFFF0)
          val_pixel = ((i_ushort_0 & 0xFFF0) >> 4);
          *(p_image++) = (double)val_pixel;

          // 2eme pixel : 4 derniers bits (masque 0x000F) + 8 premiers bits
          val_pixel = ((i_ushort_0 & 0x000F) << 8) + ((i_ushort_1 & 0xFF00) >> 8);
          *(p_image++) = (double)val_pixel;

          // 3eme pixel : 8 derniers bits (masque 0x00FF) + 4 premiers bits
          val_pixel = ((i_ushort_1 & 0x00FF) << 4) + ((i_ushort_2 & 0xF000) >> 12);
          *(p_image++) = (double)val_pixel;

          // 4eme pixel : 12 derniers bits (masque 0x0FFF)
          val_pixel = i_ushort_2 & 0x0FFF;
          *(p_image++) = (double)val_pixel;
          // ...FT-2765

        // on decale le pointeur d'entree de 6 octets = 3 shortint
        p_ushort += 3;     

      // fin pour 
      }
      break;      


    /*---------------------- autre cas : erreur -----------------------------*/
  default:
    cr_op   = SD_BAS_ERR_BPP_IP;
    msg_err = SD_BAS_MSG_BPP_IP;
    num_lig = __LINE__;
    goto fin;
  }

  /* fin de la fonction : gestion des erreurs */
 fin:
  
  /* on ne met a jour la structure ctx que si une erreur a ete detectee
     et qu'elle n'a pas encore ete stockee dans ctx */
  if (cr_op != CMN__STATUT_OK)
    {
      /* mise a jour de la variable ctx->cr.errinterne */
      SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                          SD_BAS_ERR_SET_Niveau0,
                          SD_BAS_ERR_NUM_JDB,
                          tcProcName_F,
                          num_lig,
                          msg_err);
    }

  /* sortie de la fonction */
  return(cr_op); 
}


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_TypeN0
//@
//@ Description : fonction externe determinant le type des records de NO
//@  cette operation s'applique aux produits IASI de N0 
//@
//@
//@ Parameters :
//@     (ENTREE)        bloc  : un paquet de niveau 0
//@     (ENTREE/SORTIE) ctx   : contexte de gestion des erreurs
//@
//@ Return Status : 
//@                    type   : type du record (AP VA IP PX )
//@
//@ ------------------------------------------------------------------*/
int SD_BAS_TypeN0(char * bloc, t_SD_BAS_DataThread *ctx)
{

  static const char tcProcName_F[] = "SD_BAS_TypeN0";
  t_CMN__Statut cr_op   = CMN__STATUT_OK;
  int lg = 0;
  t_SD_BAS_Liste_Var dongen;
  t_SD_BAS_status_d  status_d;
  int num_lig;                  /* num de la ligne ou l'erreur est detectee */
  char* msg_err;                /* pointeur vers le message d'erreur */
  int paquet;


  /* on decommute l'APID */
  status_d = DECOM__Decommuter(bloc, lg, SD_BAS_BLOC_DONGEN_TYPE, "", &dongen);
  SD_BAS_TEST_DECOM(status_d, SD_BAS_ERR_DECOMMUTER, __LINE__);

  /* on deduit le type & partir de l'APID */
  paquet = SD_BAS_TypePaquet(*(dongen.tab_var[N0_TYPE].val.pui4), ctx);

  /* on libere dongen      (liberation globale) */
  DECOM__Liberer_Stockage_liste_var(&dongen);
  
  
 fin:
  /* on ne met a jour la structure ctx que si une erreur a ete detectee
     et qu'elle n'a pas encore ete stockee dans ctx */
  if ( cr_op != CMN__STATUT_OK)
   {
    /* mise a jour de la variable ctx->cr.errinterne */
    SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
      SD_BAS_ERR_SET_Niveau0,
      SD_BAS_ERR_NUM_JDB,
      tcProcName_F,
      num_lig,
      msg_err);

  }

  /* sortie de la fonction */
  return(paquet);  
}


//======================================================================
//          FONCTIONS INTERNES  
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_InverserMot
//@
//@ Description : Inverse tous les bits d'un mot de 16 bits
//@               (le bit 0 devient le bit 15, etc.)
//@
//@ Parameters :
//@     (ENTREE) motIn  : mot de 16 bits en entree
//@     (SORTIE) motOut : mot de 16 bits en entree
//@
//@ Return Status : aucun
//@
//@ ------------------------------------------------------------------*/
static void SD_BAS_InverserMot(
    const unsigned short   motIn,
          unsigned short * motOut
      )
{
  int              iBit;    /* variable de boucle sur les bits du mot */
  unsigned short   masque0; /* masque utilise pour renverser les bits du mot */
  unsigned short   masque1; /* masque utilise pour renverser les bits du mot */
  unsigned short   v;       /* mot temporaire */

  /* le 1er masque est initialise par un 1 suivi de 15 zeros */
  masque0 = 0x8000;

  /* le second masque est initialise a 1 */
  masque1 = 0x0001;

  /* le mot renverse est initialise a 0 */
  *motOut = 0;

  /* boucle sur les 16 bits du mot */
  for (iBit=0; iBit<NB_BITS_WORD; iBit++)
  {
    /* premier masque */
    v = motIn & masque1;

    /* si le bit est egal a 1 on l'ajoute */
    if ( v ) /* de lapin :-) */
    {
      *motOut |= masque0;
    }

    /* mise a jour des masques */
    masque0 >>= 1;
    masque1 <<= 1;
  }
}

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_TypePaquet
//@
//@ Description : Retourne le type de paquet en fonction de l'APID
//@
//@ Parameters :
//@     (ENTREE) APID : Application Process ID
//@     (ENTREE/SORTIE) ctx   : contexte de gestion des erreurs
//@
//@ Return Status : aucun
//@
//@ ------------------------------------------------------------------*/
static e_SD_BAS_TypeN0 SD_BAS_TypePaquet(
  const int             APID,
  t_SD_BAS_DataThread * ctx
  )
{
  static const char tcProcName_F[] = "SD_BAS_TypePaquet";
  t_CMN__Statut cr_op   = CMN__STATUT_OK;
  int num_lig;                  /* num de la ligne ou l'erreur est detectee */
  char* msg_err;                /* pointeur vers le message d'erreur */
  e_SD_BAS_TypeN0 type_paquet;  /* variable retournee par la fonction */

  switch (APID)
  {
    case APID_PX1:
    case APID_PX2:
    case APID_PX3:
    case APID_PX4:
      type_paquet = PX;
    break;
    
    case APID_IP:
      type_paquet = IP;
      break;

    case APID_VP:
      type_paquet = VP;
      break;

    case APID_AP:
      type_paquet = AP;
      break;

    default:

      /* affectation des variables de gestion des erreurs */
      cr_op   = SD_BAS_ERR_LECTURE_TYPE_PAQUET;
      msg_err = SD_BAS_MSG_LECTURE_TYPE_PAQUET;
      num_lig = __LINE__;
      goto fin;
  }

  /* fin de la fonction : gestion des erreurs */
  fin:
  /* on ne met a jour la structure ctx que si une erreur a ete detectee
     et qu'elle n'a pas encore ete stockee dans ctx */
  if (cr_op != CMN__STATUT_OK)
   {
    /* mise a jour de la variable ctx->cr.errinterne */
    SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
      SD_BAS_ERR_SET_Niveau0,
      SD_BAS_ERR_NUM_JDB,
      tcProcName_F,
      num_lig,
      msg_err);


  }

  /* sortie de la fonction */
  return type_paquet;
}

//@ ------------------------------------------------------------------*/
//@ Methode : SD_BAS_GetFirstPtsiN0 CR301
//@
//@ Description : Lit la valeur du PTSI du premier paquet N0. Cette valeur
//@               remplace celle lue dans le fichier de contexte pour
//@               definir le PTSI des parametres de configuration bord BRD
//@
//@ Parametres :
//@               (ENTREE) N0_product_name
//@
//@ Valeur retour :   int  FirstPtsiN0 issu du fichier N0
//@
//@ ------------------------------------------------------------------*/
/* FA 162 */
int SD_BAS_GetFirstPtsiN0(const t_SD_BAS_NomProduit N0_product_name,
                          const e_BAS_ModeTrt ModeTraitement )
/* .. FA 162*/
{
  t_CMN__Statut       cr_op = CMN__STATUT_OK;
  int                 num_lig;
  char               *msg_err;
  
  t_SD_BAS_Fd         ProduitN0;
  t_SD_BAS_DataThread ctx;  
  t_CMN__Booleen      ficOuvert = CMN__FAUX;
  unsigned int        OffsetPaquet = 0;
  t_CMN__Booleen      PremierPaquetPXValideTrouve = CMN__FAUX;
  char                Buffer[28];
  t_SD_BAS_status_d   Status;
  t_SD_BAS_Liste_Var  Grh;

  // FA 203
  t_SD_BAS_Liste_Var  dongen;
  // FIN FA 203	
  t_DescProd          Record;
  int                 Nb = 0;
  t_CMN__Booleen      PaquetValide = CMN__FAUX;
  unsigned int        OffsetPremierPaquetPXValide = 0;
  unsigned short      sBloc = 0;
  unsigned int        FirstPtsiN0 = 0;
  unsigned int        TailleProduit = 0;
  t_SD_BAS_MPHR       MphrN0;
  char               *NomRecord[]={"MPHR","SPHR","IPR","GEADR","GIADR","VEADR","VIADR","MDR"};

//  FILE               *FdN0;
  
#ifdef OPS_DEBUG
  // Nom de la methode
  static const char   tcProcName_F[] = "SD_BAS_GetFirstPtsiN0";
#endif
  
#ifdef OPS_DEBUG
  printf ("\n%s\n", tcProcName_F);
  printf ("CR301 produit N0 : \n%s\n", N0_product_name);
#endif
  
  //@ Determiner l'offset du premier paquet PX valide
  {
    //@ Ouvrir le produit de niveau 0
/* FA 162 */
    cr_op = SD_BAS_OuvrirProduit(
      LECTURE, 
      NA_IASIL0,
      // FT-2765
      (char *)N0_product_name,
      // ...FT-2765
      &ProduitN0, 
      &ctx,
      ModeTraitement);
/* .. FA 162*/
    //@ SI l'ouverture du produit N0 a genere une erreur
    if (cr_op != CMN__STATUT_OK)
    {
#ifdef OPS_DEBUG
      printf ("%s : Erreur OuvrirProduit: \n" , tcProcName_F);
      printf ("%s\n", N0_product_name);
#endif
      cr_op   = SD_BAS_ERR_BAS_NIVEAU;
      num_lig = __LINE__;
      msg_err = SD_BAS_MSG_OPEN;
      goto fin;
    } //@ FIN SI l'ouverture du produit N0 a genere une erreur

#ifdef OPS_DEBUG
    printf ("%s: OuvrirProduit: \n", tcProcName_F);
    printf ("%s\n", N0_product_name);
#endif

    //@ Le produit N0 est ouvert
    ficOuvert = CMN__VRAI;    

    /* Lire le nombre de MDR dans le MPHR N0 */
    cr_op = SD_BAS_LireMPHRN0(&ProduitN0, &MphrN0, &ctx);
    if (CMN__STATUT_OK != cr_op)
    {
      cr_op   = SD_BAS_ERR_BAS_NIVEAU;
      num_lig = __LINE__;
      msg_err = SD_BAS_MSG_LIREMPHRN0;
      goto fin;
    }

#ifdef DEBUG
    printf ("Nb MDR = %d\n", ProduitN0.nombreMDR);
    printf ("Offset premier MDR    : %d\n", ProduitN0.offsetMDR);
    printf ("Offset MDR suivant    : %d\n", ProduitN0.offsetMDRSuivant);
#endif

    //@ Determiner la taille du produit N0
    cr_op = fseeko (ProduitN0.fd,0,SEEK_END);
    if(cr_op == -1)
    {
      cr_op   = SD_BAS_ERR_BAS_NIVEAU;
      num_lig = __LINE__;
      msg_err = SD_BAS_MSG_SEEK;
      goto fin;
    }
    TailleProduit = ftello (ProduitN0.fd);

    Record.type     = 0;
    Record.subclass = 0;
    Record.offset   = 0;
    Record.taille   = 0;


    // FA 203 , nouvelle correction
   
    OffsetPaquet = ProduitN0.offsetMDR;
    
    while ( (OffsetPaquet<TailleProduit) && 
            (CMN__FAUX == PremierPaquetPXValideTrouve) )
    {
       cr_op = SD_BAS_LireMDRGenN0(CTX_INIT,
    				OffsetPaquet,
    				&ProduitN0,
    				&ctx,
    				&dongen);
       if ( cr_op != CMN__STATUT_OK )
       {
      /* affectation des variables de gestion des erreurs */
      cr_op   = SD_BAS_ERR_LIREMDRGENN0;
      msg_err = SD_BAS_MSG_LIREMDRGENN0;
      num_lig = __LINE__;
      goto fin;
       }

       Record.type     = TR_MDR;
       Record.subclass = (short)*(dongen.tab_var[N0_TYPE].val.pui4);
       Record.offset   = *(dongen.tab_var[N0_OFFSET].val.pui4);
       Record.taille   = *(dongen.tab_var[N0_LONGUEURREC].val.pui4);

       if (PX == Record.subclass)
       {
           OffsetPremierPaquetPXValide = Record.offset;
           PremierPaquetPXValideTrouve = CMN__VRAI;
           //@ Se positionner a l'adresse du PTSI
           cr_op = fseeko (
                ProduitN0.fd, 
                OffsetPremierPaquetPXValide + SD_BAS_OFFSET_PTSI_PX, 
                SEEK_SET);
           if (cr_op == -1)
           {
                cr_op   = SD_BAS_ERR_BAS_NIVEAU;
                msg_err = SD_BAS_MSG_SEEK;
                num_lig = __LINE__;
                goto fin;
           }

           //@ Lire le PTSI dans le produit de niveau 0
           cr_op = fread(
                &sBloc, 
                sizeof (unsigned short), 
                1, 
                ProduitN0.fd);
           if(cr_op != 1)
           {
                cr_op   = SD_BAS_ERR_BAS_NIVEAU;
                msg_err = SD_BAS_MSG_READ;
                num_lig = __LINE__;
                goto fin;
           }
           // FT-2765
           SWAP_INT_BUFFER( &sBloc, 2, 1 );
           // ...FT-2765
           FirstPtsiN0 = (int) sBloc;              
   
       }//@ FIN SI Type paquet PX
       
       OffsetPaquet = OffsetPaquet + Record.taille;
      	
       // FT-2512
       DECOM__Liberer_liste_var(&dongen);
       // ..FT-2512
    }//@Fin TANT QUE
    // FIN FA 203	    

    //@ Fermer le produit de niveau 0
    if (CMN__VRAI == ficOuvert)
    {
/* FA 162 */
      cr_op = SD_BAS_FermerProduit(
        LECTURE, 
        &ProduitN0, 
        &ctx,
	ModeTraitement);
/* .. FA 162*/
    }
  } //@ FIN Determiner l'offset du premier paquet PX valide

  if (CMN__FAUX == PremierPaquetPXValideTrouve)
  {
    cr_op   = SD_BAS_ERR_BAS_NIVEAU;
    msg_err = SD_BAS_MSG_N0PTSI_NOT_FOUND;
    num_lig = __LINE__;
    goto fin;
  }  

fin:  
  if (cr_op != CMN__STATUT_OK)
  {
#ifdef OPS_DEBUG
    //@ Impression des exceptions
    printf ("\nException in %s\n", tcProcName_F);
#endif
    //@ SI le fichier est ouvert
    if (ficOuvert == CMN__VRAI)
    {
      //@ Fermeture du fichier de N0 en lecture
/* FA 162 */
      SD_BAS_FermerProduit(LECTURE, &ProduitN0, &ctx,ModeTraitement);
/* .. FA 162*/
    }
    //@ FIN SI le fichier est ouvert
  }
  // FT-2512
  SD_BAS_LibereListeVar(ECRITURE, &MphrN0.grh);
  SD_BAS_LibereListeVar(ECRITURE, &MphrN0.listevar);
  // ..FT-2512 

  return (FirstPtsiN0);
} // FIN SD_BAS_GetFirstPtsiN0 CR301
