//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/BAS/src/SD_BAS_Niveau0.h,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Component : SD_BAS
//@ MODULE : SD_BAS_Niveau0
//@
//@ Comment :
//@    include file for the SD_BAS_Niveau0 class
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 20/12/2002 : Olivier Duffez
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID :IA-CR-2100-246-CNE : 06/07/2004 : B. MASSART
//@ word # 16 in level 0 PX packets is no more read as part of PTSI number
//@ --------------------------------------------------------------------
//@ DM-ID :IA-CR-2100-243-CNE : 02/08/2004 : B. MASSART
//@ Line identification uses N0_LN in ISP packets
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-148-CN : 10/11/2004 : B.MASSART
//@ Mise a jour de SD_BAS_IASIN0_VERSION = 6.6. Voir IA-2100-FC-148-THA.
//@ --------------------------------------------------------------------
//@ MODIFIED 14 Juin 2006 for WorkAround-V3.5.4
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-271-CN : 22/06/2005 : JL.PASCAL
//@ prise en compte du mode transition
//@ fiche de correction correspondante : IA-2100-CR-271-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-270-CN : 24/06/2005 : JL.PASCAL
//@ prise en compte des SN manquant
//@ fiche de correction correspondante : IA-2100-CR-270-THA
//@ --------------------------------------------------------------------
//@ PM-ID :IA-2100-CR-301-CN : 03/07/2006 : B. MASSART
//@ PTSI is now read in the first PX valid record in N0 product to select the 
//@ BRD file. Before CR301, PTSI was read in the first CTX file where its 
//@ value is 0. (patch3-5-5). See IA-2100-CR-301-THA.
//@ --------------------------------------------------------------------
//@ MODIFIED FOR PATCH3-5-5 : 05/Jul/06
//@ --------------------------------------------------------------------
//@ DM-ID :IA-CR-2100-306-CNE : 15/09/2006 : C.MERIGUET
//@ Mise a jour de SD_BAS_IASIN0_VERSION, la version passe a 10.0
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-162-CN : 09/2006 : C.MERIGUET
//@ Gestion des IPR en mode DUMP
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_BAS_Niveau0_h
#define SD_BAS_Niveau0_h

// inserer ici les fichiers include 
#include "SD_BAS_Global.h"
#include "SD_BAS_Produits.h"


//====================================================================*/
//   External define  Declarations
//====================================================================*/
//====================================================================*/
// Constant Declaration 
//====================================================================*/
#define SD_BAS_MAXLENGTH_SPECTN0 (4319*2)     // taille max d'un spectre N0 encode
//CR270
#define SD_BAS_DUREE_ASE_SPECTN0 (216)     // duree d'un ensemble compose de 4 PX et d'un IP

//--------------------------------------------------------------------*/
// type definissant le type d'APID 
//--------------------------------------------------------------------*/
typedef enum 
{
  APID_PX1 = 130, /* Spectrum packet pixel 1 */
  APID_PX2 = 135, /* Spectrum packet pixel 2 */
  APID_PX3 = 140, /* Spectrum packet pixel 3 */
  APID_PX4 = 145, /* Spectrum packet pixel 4 */
  APID_IP  = 150, /* image packet */
  APID_VP  = 160, /* verification packet */
  APID_AP  = 180  /* auxiliary packet */
} e_SD_BAS_APID;

//====================================================================*/
// Exported types Declaration  
//====================================================================*/

// les valeurs doivent etre au depart initialisees a 0 et les flags a faux

//--------------------------------------------------------------------*/
// type definissant le type de paquet N0 
//--------------------------------------------------------------------*/
typedef enum 
{
  PX,
  IP,
  VP,
  AP
} e_SD_BAS_TypeN0;

/*--------------------------------------------------------------------
  structure contenant les informations generales issues d'un MDR de N0 
  cette structure est retournee par la fonction de parcours rapide
  --------------------------------------------------------------------*/
/* attention l'ordre suivant doit correspondre a l'ordre dans le fichier
   ASCII de decommutation */
typedef enum
{
  N0_TYPE,
  N0_LONGUEURDATA,
  N0_PN,
  N0_DATEUTC,
  N0_DATEOBT,
  N0_SN,
  N0_SP,
  N0_PTSI,
  N0_ONBOARD_SOFTWARE_VERSION, 
  N0_LN, /* CR243 */
  N0_MODE,
  N0_MODE_TRANSITION, /*CR271 */
  N0_LONGUEURREC,
  N0_VALIDREC,
  N0_OFFSET,
  N0_DATEDEB,
  N0_DATEFIN
} e_SD_BAS_IND_DonGenN0;
//typedef struct 
//{
//  int LongueurRec;    /* longueur du record (record size du GRH) */
//  e_BAS_ValidRec validRec; // validite du record (VALIDE OU INVALIDE) rajoutee apres lecture du GRH
                        // cf champ INSTRUMENT du GRH)
//  unsigned int  offset; // offset du MDR  depuis le debut du produit
        // utilise par les fonctions de lecture detaillees
        // cette valeur est renseignee lors de la lecture rapide
        // (elle n'est pas issue du produit mais ajoutee lors de la lecture) 
//  e_BAS_TypeN0 type;  /* type d'ISP  (word 1) */
//  int Longueurdata;   /* longueur du tableau Data (word 3) */
//  int pn;             /* de 1 a 4 dans le cas des paquets PX : 0 sinon (word 21) */
                        /* de 1 a 5 dans le cas des paquets VP (word 21) */
//  t_SD_BAS_SCDSTime dateUTC; /* date UTC (word 4 a 7)  */
//  t_SD_BAS_OBTTime dateOBT;  /* date OBT (word 8 a 10) */
//  int sn;             /* compris entre 1 et 37 (step number) (word 12 sauf AP) */
//  int sp;             /* compris entre 0 et 36 (scan number) (word 12 sauf AP) */
//  int PTSI;           /* parameter table set identifier (word 16 et 17 sauf AP 13 et 14) */
//  e_BAS_ModePdv mode; /* mode de prise de vue (word 19 sauf pour le paquet AP) */
//} t_SD_BAS_DonGenN0; 

//--------------------------------------------------------------------*/
// structure contenant les informations codees d'un IPR de N0 
// cette structure est retournee par les fonctions de lecture detaillee
//--------------------------------------------------------------------*/
typedef struct 
{
  int longueur; // longueur du tableau Data (word 3)
  char * data; // adresse du MDR utilise par les fonctions de lecture detaillee
      // on stocke ici l'ensemble des paquets 
      // PX ( word 1 - longueur )
      // IP ( word 1 - longueur )
      // VP ( word 1 - longueur ) 
      // AP ( word 1 - longueur )
      // ceci est utilise lors de la constitution du produit en sortie
      // ex pour les paquet VP ceci est egalement utilise lors du
      // decodage a la volee
} t_SD_BAS_DonCodN0; 

//--------------------------------------------------------------------*/
// type definissant les resultats specifiques contenus dans les IPR de type PX
// cette structure est retournee par les fonctions de lecture detaillee
//--------------------------------------------------------------------*/
// attention l'ordre suivant doit correspondre a l'ordre dans le fichier
// ASCII de decommutation
typedef enum
{
  PX_ICD,
  PX_NV,
  PX_LN,
  PX_SQIS,
  PX_NZPDQI,
  PX_BARCIMAGMEANB1,
  PX_BARCIMAGMEANB2,
  PX_BARCIMAGMEANB3,
  PX_BARCIMAGRMSB1,
  PX_BARCIMAGRMSB2,
  PX_BARCIMAGRMSB3,
  PX_BARCIMAGMEANRMSB1,
  PX_BARCIMAGMEANRMSB2,
  PX_BARCIMAGMEANRMSB3,
  PX_SPECDATAAREA,
  PX_IDEFCODNBBITMAX
} e_SD_BAS_IND_IPRN0_PX;
//typedef struct 
//  {
//  int cd;                   // cube direction  0 ou 1 (word 13 B0)
//  char  nv;                 // indicateur de qualite (word 13 B8 a B15)
//  int ln;                   // no de ligne depuis le passage en mode operationnel (word 18)
//  short int sqis;           // indicateur de qualite (word 20)
//  int nzpdqi;               // NZPD indicateur de qualite (word 30 31)
//  float barcimagmeanB1[16]; // mean value of imaginary part B1 (word 32 47)
//  float barcimagmeanB2[20]; // mean value of imaginary part B2 (word 48 67)
//  float barcimagmeanB3[20]; // mean value of imaginary part B3 (word 68 87)
//  float barcimagrmsB1[16];  // mean value of square of imaginary part B1 (word 88 103)
//  float barcimagrmsB2[20];  // mean value of square of imaginary part B2 (word 104 123)
//  float barcimagrmsB3[20];  // mean value of square of imaginary part B3 (word 124 143)
//  float barcimagmeanrmsB1;  // mean value of mean of square of imaginary part B1 (word 144 145)
//  float barcimagmeanrmsB2;  // mean value of mean of square of imaginary part B2 (word 146 147)
//  float barcimagmeanrmsB3;  // mean value of mean of square of imaginary part B3 (word 148 149)
//  int   specdataarea;       // spectrum status area ( word 150-151)
//  int   idefcodnbbitmax;    // nombre de bits maxi du spectre encode ( word 155-156)
                              // de 0 a 32 bits) ceci associe au word 3 permet
                              // de determiner le nbre d'elements dans le spectre
                              // n = (longueur - 160 )(idefcodnbbitmax%16+ 1)
//  t_SD_BAS_Liste_Var Listevar;  
//  } t_SD_BAS_DonDetIPRN0_PX; 

//--------------------------------------------------------------------*/
// type definissant les resultats specifiques contenus dans les IPR de type IP
// cette structure est retournee par les fonctions de lecture detaillee
//--------------------------------------------------------------------*/
// attention l'ordre suivant doit correspondre a l'ordre dans le fichier
// ASCII de decommutation
typedef enum
{
  IP_CD,
  IP_NV,
  IP_LN,
  IP_SQIS,
  IP_IISSIZE,
  IP_NBSAMPLE
} e_SD_BAS_IND_IPRN0_IP;
//typedef struct 
//  {
//  int       cd;             // cube direction  0 ou 1 (word 13 B0)
//  char      nv;             // indicateur de qualite (word 13 B8 a B15)
//  int       ln;             // numero de ligne depuis le passage en mode operationnel (word 18)
//  short int sqis;           // indicateur de qualite (word 20)
//  int       iissize;        // nombre de bits par echantillon IIS (word 21 B0-B7)
//  int       nbsample;       // nombre d'echantillons IIS (word 22)
//  t_SD_BAS_Liste_Var Listevar;  
//  } t_SD_BAS_DonDetIPRN0_IP; 

//--------------------------------------------------------------------*/
// type definissant les resultats specifiques contenus dans les IPR de type AP
// cette structure est retournee par les fonctions de lecture detaillee
//--------------------------------------------------------------------*/
// attention l'ordre suivant doit correspondre a l'ordre dans le fichier
// ASCII de decommutation
typedef enum
{
  AP_BBT,
  AP_LN,
  AP_SQIS,
  AP_OMT_NV,
  AP_BCODSPECTVERIF,
  AP_OPER_DATA_WORD33,
  AP_OPER_DATA_WORD135,
  AP_OPER_DATA_WORD136
} e_SD_BAS_IND_IPRN0_AP;
//typedef struct 
//  {
//  unsigned int bbt;               // black body temperature (mK) (word 15 16)
//  int          ln;                // n de ligne depuis le passage en mode operationnel (word 18)
//  short int    sqis;              // indicateur de qualite (word 19)
//  short int    omt_nv;            // indicateur de qualite (word 28)
//  short int    bcodspectverif;    // numero de l'interf de verification  (word 35)
//  short int    oper_data_word33;  // mot qualite pour SN 33  (word 88)
//  short int    oper_data_word135; // mot qualite pour SN 35  (word 103)
//  short int    oper_data_word136; // mot qualite pour SN 36  (word 118)
//  t_SD_BAS_Liste_Var Listevar;  
//  } t_SD_BAS_DonDetIPRN0AP; 

//--------------------------------------------------------------------*/
// type definissant les resultats specifiques contenus dans les IPR de type VP
//--------------------------------------------------------------------*/
//a priori ici aucun decodage n'est necessaire

//--------------------------------------------------------------------*/
// structure contenant les informations utiles  d'un IPR de N0 
// cette structure est retournee par les fonctions de lecture detaillee
// cette fonction est appelee apres la fonction de parcours rapide
// en entree on indique l'offset du record a lire dans le produit et le type d'ISP
// sequence d'appel:
// 1 : lecture rapide
// 2 : lecture detaillee selon le type
// 3 :   allocation memoire en fonction de la longueur pour InfoCod (interne)
// 4 :   allocation memoire en fonction du type pour InfoDet (interne)
// 5 : liberation des donnees allouees (fonction specifique) a mettre dans le destructeur
//--------------------------------------------------------------------*/
typedef struct 
{
  e_SD_BAS_TypeN0 type;   // type d'ISP  (word 1)
  t_SD_BAS_Liste_Var InfoGen;// info generales
  //  t_SD_BAS_DonGenN0 InfoGen; // info generales
  // ceci est rempli par la fonction de lecture detaillee
  t_SD_BAS_DonCodN0 InfoCod; // info non decodees issus de la fonction
  // de lecture detaillee
  t_SD_BAS_Liste_Var InfoDet; 
  // ceci est  alloue par les fonctions de lecture detaillee
  // ou d'init pour ecriture
  //  void * InfoDet;      // info decodees
  // pointeur sur t_SD_BAS_DonDetIPRN0PX
  // ou t_SD_BAS_DonDetIPRN0IP 
  //ou t_SD_BAS_DonDetIPRN0AP selon
  // le type d'ISP 
  // ceci est rempli par la fonction de lecture detaillee
  // lors de l'utilisation dans les algo  on demande les ISP par type
  // le cast  peut donc se faire de maniere systematique
} t_SD_BAS_PaquetN0; 


#ifdef __cplusplus
extern "C" {
#endif

//====================================================================*/
// External variables Declaration 
//====================================================================*/
// version du produit L0 de l'OPS en debut de vie
//const t_SD_BAS_VERSION SD_BAS_IASIN0_VERSION={1,0};
// TODO : attention a remettre le numero de version {1,0}
//CR306
// FT-2765
static const t_SD_BAS_VERSION SD_BAS_IASIN0_VERSION={10,0};
// ...FT-2765
//..CR306
// FT-2765
static const int SD_BAS_MDR_VERSION_N0=1;
// ...FT-2765

//====================================================================*/
// External fonctions Declaration 
//====================================================================*/
//          FONCTIONS EXTERNES
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LireMPHRN0
//@
//@ Description : fonction de lecture du MPHR d'un produit 
//@  cette operation generique s'applique aux produits IASI de N0 
//@
//@
//@ Parameters :
//@     (ENTREE)  descripteur  : descripteur du produit
//@         utilise par les fonctions d'acces
//@     (SORTIE)  mphr  : liste des variables lues dans le MPHR
//@     (ENTREE/SORTIE)  ctx  : contexte de gestion des erreurs
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LireMPHRN0(
    t_SD_BAS_Fd *         Descripteur,
    t_SD_BAS_MPHR *       mphr,
    t_SD_BAS_DataThread * ctx
      )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LireMDRGenN0
//@
//@ Description : fonction de lecture des donnees gen des IPR d'un produit 
//@  cette operation generique s'applique aux produits IASI de N0 
//@  on lit ici le grh et quelques info detaillees  ( soit sequentiellement)
//@  soit a partir d'une adresse
//@
//@ SD_BAS_LireMPHRN0 doit avoir ete appele avant
//@
//@ Parameters :
//@     (ENTREE)         init  : contexte d'initialisation
//@                           init = INIT ( on lit le suivant )
//@                           init = NORMAL ( on lit a l'adresse indiquee )
//@     (ENTREE)         offset  : offset pour le mode normal
//@     (ENTREE/SORTIE)  descripteur  : descripteur du produit
//@                           utilise par les fonctions d'acces
//@     (SORTIE)         dongen  : liste des variables lues dans le MDR
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LireMDRGenN0(
    const e_BAS_CtxInit   init,
    const int             offset,
    t_SD_BAS_Fd *         Descripteur,
		t_SD_BAS_DataThread * ctx,
    t_SD_BAS_Liste_Var *  dongen
      )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LireMDRN0
//@
//@ Description : fonction de lecture  des MDR de type PX IP AP VP d'un produit 
//@  cette operation s'applique aux produits IASI de N0 
//@  on lit ici les info detaillees a partir de l'offset
//@
//@
//@ Parameters :
//@     (ENTREE)  descripteur  : descripteur du produit
//@         utilise par les fonctions d'acces
//@     (ENTREE)  offset  : offset issu de la lecture des donnees generales
//@     (ENTREE/SORTIE)  ctx  : contexte de gestion des erreurs
//@     (SORTIE)  mdr  : liste des variables lues dans le MDR
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LireMDRN0(
    t_SD_BAS_Fd *       Descripteur,
    const int           offset,
		t_SD_BAS_DataThread * ctx,
    t_SD_BAS_PaquetN0 * mdr
      );

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LibereMDRN0
//@
//@ Description : fonction de liberation des MDR  d'un produit 
//@  cette operation s'applique aux produits IASI de N0 
//@  on desalloue ici toute la place allouee precedenment 
//@
//@
//@ Parameters :
//@     (ENTREE)  descripteur : descripteur du produit
//@                             utilise par les fonctions d'acces
//@     (ENTREE/SORTIE)  ctx  : contexte de gestion des erreurs
//@     (ENTREE/SORTIE)  mdr  : liste des variables lues dans le MDR
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LibereMDRN0(
				 t_SD_BAS_DataThread * ctx,
				 t_SD_BAS_PaquetN0 *   mdr
				 )
       ;

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_DecodeSpectreN0
//@
//@ Description : fonction de decodage du spectre de N0  
//@  cette operation s'applique aux produits IASI de N0 
//@   la place allouee precedenment est allouee par l'appelant
//@
//@
//@ Parameters :
//@     (ENTREE)        mdr      : contenu du MDR
//@     (ENTREE/SORTIE) ctx      : contexte de gestion des erreurs
//@     (SORTIE)        spectre  : contenu du spectre
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_DecodeSpectreN0(
    t_SD_BAS_PaquetN0 *   mdr,
		t_SD_BAS_DataThread * ctx,
    char *                spectre
      )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_DecodeImageN0
//@
//@ Description : fonction de decodage d'une image de N0  
//@  cette operation s'applique aux produits IASI de N0 
//@   la place allouee precedenment est allouee par l'appelant
//@
//@
//@ Parameters :
//@     (ENTREE)        mdr   : contenu du MDR
//@     (ENTREE/SORTIE) ctx   : contexte de gestion des erreurs
//@     (ENTREE/SORTIE) image : contenu de l'image
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_DecodeImageN0(
    t_SD_BAS_PaquetN0 *   mdr,
		t_SD_BAS_DataThread * ctx,
    double *              image
      )
;


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_TypeN0
//@
//@ Description : fonction externe determinant le type des records de NO
//@  cette operation s'applique aux produits IASI de N0 
//@
//@
//@ Parameters :
//@     (ENTREE)        bloc  : un paquet de niveau 0
//@     (ENTREE/SORTIE) ctx   : contexte de gestion des erreurs
//@
//@ Return Status : 
//@                    type   : type du record (AP VA IP PX )
//@
//@ ------------------------------------------------------------------*/
  int SD_BAS_TypeN0(char * bloc, t_SD_BAS_DataThread *ctx);

//@ ------------------------------------------------------------------*/
//@ Methode : SD_BAS_GetFirstPtsiN0 CR301
//@
//@ Description : Lit la valeur du PTSI du premier paquet N0. Cette valeur
//@               remplace celle lue dans le fichier de contexte pour
//@               definir le PTSI des parametres de configuration bord BRD
//@
//@ Parametres :
//@               (ENTREE) N0_product_name
//@
//@ Valeur retour :   int  FirstPtsiN0 issu du fichier N0
//@
//@ ------------------------------------------------------------------*/
/* FA 162 */
int SD_BAS_GetFirstPtsiN0(const t_SD_BAS_NomProduit N0_product_name,
                          const e_BAS_ModeTrt ModeTraitement );
/* ..FA 162 */


//======================================================================
#ifdef __cplusplus
}
#endif


#endif
