//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/BAS/src/SD_BAS_OperateurBase.h,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_BAS
//@ MODULE : SD_BAS_OperateurBase
//@
//@ Comment :
//@    Code file for the  SD_BAS_OperateurBase class which implements
//@    specific mathematical methods   :
//@ - Plank functions.
//@ - .....
//@ these methods are hand made in order to facilitate the
//@ portabilite of the software
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_BAS_OperateurBase_h
#define SD_BAS_OperateurBase_h


// inserer ici les fichiers include
#include <stdio.h>       /* entree/sortie */
#include <stdlib.h>      /* alloc memoire, ...*/
#include <string.h>      /* chaines de carateres */
#include <unistd.h>      /* gestion pipes, ... */
#include <time.h>        /* gestion de la date */
#include <sys/param.h>   /* constantes systeme */


#include "CMN__Commun.h"
#include "SD_BAS_Global.h"
#include "ERR__Erreur.h"
#include "SD_BAS_LibESSL.h"

/*====================================================================*/
//   External define  Declarations
/*====================================================================*/
#define ERR_BASE_OPBASE                     SD_BAS_ERR_BASE_OperateurBase
#define SD_BAS_OperateurBase_ERR_NONE       (ERR_BASE_OPBASE+0)
#define SD_BAS_PAS_EXTREMUM                 (ERR_BASE_OPBASE+1)
#define SD_BAS_HORS_GRILLE                  (ERR_BASE_OPBASE+2)

/* Nombre de jours dans le mois de janvier */
#define JOURS_EN_JANVIER              31.0
#define JOURS_EN_MARS                 31.0
#define JOURS_EN_AVRIL                30.0
#define JOURS_EN_MAI                  31.0
#define JOURS_EN_JUIN                 30.0
#define JOURS_EN_JUILLET              31.0
#define JOURS_EN_AOUT                 31.0
#define JOURS_EN_SEPTEMBRE            30.0
#define JOURS_EN_OCTOBRE              31.0
#define JOURS_EN_NOVEMBRE             30.0
#define JOURS_EN_DECEMBRE             31.0

/*====================================================================*/
// Constant Declaration
/*====================================================================*/

/*====================================================================*/
// Exported types Declaration
typedef enum
{
  mois_inconnu   = 0,
  mois_janvier   = 1,
  mois_fevrier   = 2,
  mois_mars      = 3,
  mois_avril     = 4,
  mois_mai       = 5,
  mois_juin      = 6,
  mois_juillet   = 7,
  mois_aout      = 8,
  mois_septembre = 9,
  mois_octobre   = 10,
  mois_novembre  = 11,
  mois_decembre  = 12
} e_SD_BAS_Mois;

/* Decompte des jours des mois d'une annee */
static double jours_par_mois[13] = 
{
  0.0,                         /* mois inconnu */
  JOURS_EN_JANVIER,
  28.0,                        /* en fevrier, c'est variable */
  JOURS_EN_MARS,
  JOURS_EN_AVRIL,
  JOURS_EN_MAI,
  JOURS_EN_JUIN,
  JOURS_EN_JUILLET,
  JOURS_EN_AOUT,
  JOURS_EN_SEPTEMBRE,
  JOURS_EN_OCTOBRE,
  JOURS_EN_NOVEMBRE,
  JOURS_EN_DECEMBRE
};

/*====================================================================*/

/*====================================================================*/
// External variables Declaration
/*====================================================================*/

/*====================================================================*/
//                  FONCTIONS EXTERNES
//======================================================================

#ifdef __cplusplus
extern "C" {
#endif

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_Planck
//@
//@ Description : Calcule la fonction de planck pour  une temperature T
//@ et un nombre d'onde nu
//@
//@
//@ Parameters :
//@         (ENTREE)  T : temperature du corps noir ( oK)
//@         (ENTREE)  nu: nombre d'onde ( m-1)
//@         (ENTREE)  IDefPlkPK1: Precalcul de 2*h*c*c
//@         (ENTREE)  IDefPlkPK2: Precalcul de h*c/k
//@
//@ Return Status :
//@         B : luminance ( w/(m2 sr m-1)
//@
//@ ------------------------------------------------------------------*/
double SD_BAS_Planck (
    double const T,     /* temperature */
    double const nu,    /* nombre d'onde */
    double IDefPlkPK1,  /* Precalcul de 2*h*c*c */
    double IDefPlkPK2   /* Precalcul de h*c/k */
    )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_PlanckInv
//@
//@ Description : Calcule l'inverse de la fonction de planck pour  une
//@ luminance B et un nombre d'onde nu
//@
//@
//@ Parameters :
//@         (ENTREE)  B : luminance ( w/(m2 sr m-1)
//@         (ENTREE)  nu: nombre d'onde ( m-1)
//@         (ENTREE)  IDefPlkPK1: Precalcul de 2*h*c*c
//@         (ENTREE)  IDefPlkPK2: Precalcul de h*c/k
//@
//@ Return Status :
//@          T : temperature du corps noir ( oK)
//@
//@ ------------------------------------------------------------------*/
double SD_BAS_PlanckInv (
    double const B,         /* luminance */
    double const nu,    /* nombre d'onde */
    double IDefPlkPK1,  /* Precalcul de 2*h*c*c */
    double IDefPlkPK2   /* Precalcul de h*c/k */
    )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_PlanckDeriv
//@
//@ Description : Calcule la derivee de la fonction de planck par
//@ rapport a la temp pour temperature T et un nombre d'onde nu
//@
//@
//@ Parameters :
//@         (ENTREE)  T : temperature du corps noir ( oK)
//@         (ENTREE)  nu: nombre d'onde ( m-1)
//@         (ENTREE)  IDefPlkPK1: Precalcul de 2*h*c*c
//@         (ENTREE)  IDefPlkPK2: Precalcul de h*c/k
//@
//@ Return Status :
//@         dBdt : derivee de planck  ( w/(m2 sr m-1 T )
//@
//@ ------------------------------------------------------------------*/
double SD_BAS_PlanckDeriv (
    double const T,       /* temperature */
    double const nu,          /* nombre d'onde */
    double IDefPlkPK1,  /* Precalcul de 2*h*c*c */
    double IDefPlkPK2   /* Precalcul de h*c/k */
    )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_FctAmortissement
//@
//@ Description : calcul de la fonction d'ammortissement des bord
//@ d'une fenetre spectrale avant TF
//@ remarque : cette fonction allouant de la memoire elle ne doit pas etre
//@     utilisee de maniere intensive sous peine de pb de perf
//@
//@ Parameters :
//@         (ENTREE)  nbpts: nombre de points du signal a amortir
//@         (ENTREE)  n: amortisement sigma exprime en nombre de points
//@         (ENTREE)  deltanu: largeur d'un echantillon
//@         (sortie)  amort : fonction d'ammortissement
//@         ( allouee par l'appelant)
//@         (ENTREE)  G: tableau temporaire de taille 6*n + 1
//@
//@ Return Status :
//@         t_CMN__Statut
//@             CMN__STATUT_OK si la fonction a reussi
//@             CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut  SD_BAS_FctAmortissement (
    int const nbpts,    /* nombre de points du signal a amortir */
    int const n,        /* nombre de points pour l'amort sigmas*/
    double const deltanu,   /* largeur d'un echantillon */
    double *amort,      /* fonction d'ammortissement */
    double *G               /* Fonction d'amortissement G(nu) */
    )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_EvolutionModele
//@
//@ Description : calcul de l'evolution d'une variable en fonction de la
//@       date courante
//@ remarque : cette fonction allouant de la memoire elle ne doit pas etre
//@     utilisee de maniere intensive sous peine de pb de perf
//@
//@ Parameters :
//@         (ENTREE)  T: date courante
//@         (ENTREE)  Tfe: date de fin de l'eclipse precedente
//@         (ENTREE)  Ts: periode orbitale en SECONDES
//@         (ENTREE)  Model: modele d'evolution
//@
//@ Return Status :
//@         x : valeur evaluee
//@
//@ ------------------------------------------------------------------*/
double  SD_BAS_EvolutionModele (
     t_SD_BAS_SCDSTime T,
     t_SD_BAS_SCDSTime Tfe,
     double Ts,
     double *Model
    )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_Extremum2D
//@
//@ Description : calcul de la position de l'extremum d'une variable
//@ dont on connait la valeur en 9 points
//@ remarque : cette fonction doit etre appelee apres resolution du systeme
//@ lineaire par moindres carres ( fonction SD_BAS_MoindresCarres1D)
//@ remarque : on suppose que les points de la grille sont numerote de la
//@ maniere suivante ( de 0 a 2 pour x et de 0 a 2 pour y) ceci est
//@ utilise lors de la resolution par moindres carre de SD_BAS_MoindresCarres1D
//@
//@ Parameters :
//@         (ENTREE)  x : vecteur de 6 doubles contigus x1,x2,x3,x4,x5,x6
//@                     coefficients du paraboloide ajuste.
//@         (ENTREE)  eps : moyenne quadratique des residus d'ajustement
//@         (SORTIE)  res : vecteur de 4 doubles alloue par l'appelant qui contiendra
//@                         en sortie :
//@                             - Ymax
//@                             - Zmax
//@                             - valueur max
//@                             - Qualite de l'estimateur
//@
//@         (ENTREE)  dcmax : decalage central entier 
//@         (ENTREE)  dlmax : decalage central entier 
//@ Return Status :
//@         SD_BAS_PAS_EXTREMUM
//@         SD_BAS_HORS_GRILLE
//@         CMN__STATUT_OK si la fonction a reussi
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut  SD_BAS_Extremum2D (
        double * x,
        double eps,
        double *res,
	int dcmax,
	int dlmax
    )
;


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_CalculPuissance2Sup
//@
//@ Description : calcule la puissance de 2 immediatement superieure ou egale
//@
//@
//@ Parameters :
//@     (ENTREE)  : valeur entiere dont on cherche la puissance de 2 
//@                 immediatement superieure ou egale
//@
//@ Return Status : 
//@ aucun
//@
//@ ------------------------------------------------------------------*/
int SD_BAS_CalculPuissance2Sup (int x);

//@ ------------------------------------------------------------------
//@ Methode : SD_BAS_ConvertirDateJourJulien
//@
//@ Description : Convertit une date au format en jour Julien
//@ (ENTREE)  date : date au format t_SD_BAS_GeneralTime
//@ (SORTIE)  jour_julien : date en jours juliens (0.0 au 1/1/2000 00:00:00.0)
//@
//@ Parametres :
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------
int SD_BAS_ConvertirDateJourJulien(
  t_SD_BAS_GeneralTime * date, 
  double *               jour_julien);
  

//@ ------------------------------------------------------------------
//@ Methode : SD_BAS_EstBissextile
//@
//@ Description : teste si une annee est bissextile
//@ (ENTREE)  annee 
//@
//@ Parametres :
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------
t_CMN__Booleen SD_BAS_EstBissextile(const int annee);

#ifdef __cplusplus
}
#endif

//======================================================================
#endif
