//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/BAS/src/SD_BAS_OperateurLoc.h,v 1.4 2012/07/10 16:49:16 lcaspar Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_BAS
//@ MODULE : SD_BAS_OperateurLoc
//@
//@ Comment :
//@    Code file for the  SD_BAS_OperateurLoc class which includes the
//@    geometrical algorithms used to transform the coordinates  :
//@ - from METOP  to earth
//@ - from earth to METOP
//@     -.....
//@ this is done thanks to Metop libraries which are non-thread-safe
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2997(DM-2100-OPS) : 12/06/2012 : L. CASPAR
//@ Ajout de fonctions pour calculer la geolocalisation sans l'AVHRR
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_BAS_OperateurLoc_h
#define SD_BAS_OperateurLoc_h

// inserer ici les fichiers include
#include <stdio.h>       /* entree/sortie */
#include <stdlib.h>      /* alloc memoire, ...*/
#include <string.h>      /* chaines de carateres */
#include <unistd.h>      /* gestion pipes, ... */
#include <time.h>        /* gestion de la date */
#include <sys/param.h>   /* constantes systeme */

#include "SD_BAS_Global.h"
#include "SD_BAS_AVHRR.h"
#include "CMN__Commun.h"
#include "ERR__Erreur.h"

/*====================================================================*/
/*   External define  Declarations */
/*====================================================================*/
#ifdef ERR_BASE
#undef ERR_BASE
#endif
#define ERR_BASE                                    SD_BAS_ERR_BASE_OperateurLoc
//CR 232
//#define SD_BAS_ERR_BASE_OperateurLocs_ERR_NONE      (ERR_BASE+0)
#define SD_BAS_ERR_MP_ORBIT                         (ERR_BASE+1)
#define SD_BAS_ERR_MP_TARGET                        (ERR_BASE+2)
#define SD_BAS_ERR_MP_STAVIS                        (ERR_BASE+3)
#define SD_BAS_OpLoc_ERR_INTERP                     (ERR_BASE+4)
#define SD_BAS_OPLOC_ERR_METOP2GEOD                 (ERR_BASE+5)
#define SD_BAS_OPLOC_ERR_GEOD2AVHRR                 (ERR_BASE+6)
#define SD_ALG_OPLOC_ERR_ALLOCATION                 (ERR_BASE+7)
#define SD_BAS_OPLOC_ERR_AVHRR2GEOD                 (ERR_BASE+8)
#define SD_BAS_OPLOC_ERR_GEOD2METOP                 (ERR_BASE+9)
//CR 232
#define SD_BAS_MSG_METOPENV                 (ERR_BASE+0)
#define SD_BAS_ERR_SET_OpLoc 1
/*====================================================================*/
/* Constant Declaration */
/*====================================================================*/

/*====================================================================*/
/* Exported types Declaration  */
/*====================================================================*/

/*====================================================================*/
/* External variables Declaration */
/*====================================================================*/

/*====================================================================*/
//                  FONCTIONS EXTERNES
//======================================================================
#ifdef __cplusplus
extern "C" {
#endif
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_MetopGeod
//@
//@ Description : conversion des coordonnees du referentiel Metop
//@     vers le referentiel geodesique (lat, lon)
//@ cette fonction ne peut etre appelee qu'apres la reservation de la
//@ librairie Metop (SD_BAS_Reserver)
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         ( numero de ligne SN traite
//@         (ENTREE)  IasiTime[2] : date du SN traite
//@         (ENTREE)  MphrN0 :  MPHR produit N0
//@         (ENTREE)  nbpts : nombre de points a traiter
//@         (ENTREE)  azim : coordonnees des pts dans le repere METOP (azim)
//@         (ENTREE)  elev : coordonnees des pts dans le repere METOP (elev)
//@         (SORTIE)  lat : latitude
//@         (SORTIE)  lon : longitude
//@         (SORTIE)  Ang_mesY : angles de mesure METOP
//@         (SORTIE)  Ang_mesZ : angles de mesure METOP
//@         (SORTIE)  Azim_sol : angles solaire
//@         (SORTIE)  Elev_sol : angles solaire
//@         (SORTIE)  EarthSatelDist : disance Terre/Satellite
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_MetopGeod(
        t_SD_BAS_DataThread * ctx,
        t_SD_BAS_SCDSTime * IasiTime,
        t_SD_BAS_MPHR * MphrN0,
        int  nbpts,
        double * azim,
        double * elev,
        double * lat,
        double * lon,
        double * Ang_mesY,
        double * Ang_mesZ,
        double * Azim_sol,
        double * Elev_sol,
        double * EarthSatelDist
            )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_GeodMetop
//@
//@ Description : conversion des coordonnees du referentiel Metop
//@     vers le referentiel geodesique (lat, lon)
//@ cette fonction ne peut etre appelee qu'apres la reservation de la
//@ librairie Metop (SD_BAS_Reserver)
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  IasiTime[2] : date du SN traite
//@         (ENTREE)  MphrN0 :  MPHR produit N0
//@         (ENTREE)  nbpts : nombre de points a traiter
//@         (ENTREE)  lat : latitude
//@         (ENTREE)  lon : longitude
//@         (SORTIE)  azim : coordonnees des pts dans le repere METOP
//@         (SORTIE)  elev : coordonnees des pts dans le repere METOP
//@
//@ Return Status :
//@         t_CMN__Statut
//@         SD_BAS_STATUT_OK si la fonction a reussi
//@         SD_BAS_STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_GeodMetop(
        t_SD_BAS_DataThread * ctx,
        t_SD_BAS_SCDSTime * IasiTime,
        t_SD_BAS_MPHR * MphrN0,
        int  nbpts,
        double * lat,
        double * lon,
        double * azim,
        double * elev
            )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_GeodAVHRR
//@
//@ Description : recherche dans le bandeau AVHRR des pixels correspondants
//@     aux coordonnees geodesiques (lat, lon)
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  nbpts : nombre de points a traiter
//@         (ENTREE)  lat : latitude
//@         (ENTREE)  lon : longitude
//@         (ENTREE)  indicedeb : indice de la premiere ligne AVHRR utile
//@         (ENTREE)  indicefin : indice de la derniere ligne AVHRR utile
//@         (ENTREE)  BandeauAVHRR : bandeau AVHRR charge en memoire
//@         (ENTREE)  nbligbandeau : nombre de lignes total du bandeau
//@         (ENTREE)  coldeb : numero de col du premier pixel AVHRR a analyser
//@         (ENTREE)  colfin : numero de col du dernier pixel AVHRR a analyser
//@         (SORTIE)  col : numero de col du pixel AVHRR trouve (fractionnel)
//@         (SORTIE)  lig : numero de lig du pixel AVHRR trouve (fractionnel)
//@         (SOrtie)  date : date de la ligne AVHRR du pixel trouve
//@
//@ Return Status :
//@         t_CMN__Statut
//@         SD_BAS_STATUT_OK si la fonction a reussi
//@         SD_BAS_STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_GeodAVHRR(
        t_SD_BAS_DataThread * ctx,
        int  nbpts,
        double * lat,
        double * lon,
        int indicedeb,
        int indicefin,
        t_SD_BAS_LigneAVHRR** BandeauAVHRR,
        int nbligbandeau,
        int coldeb,
        int colfin,
        double * col,
        double * lig,
        t_SD_BAS_SCDSTime * date
            )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_AVHRRGeod
//@
//@ Description : conversion de coordonnees AVHRR fractionnaires en
//@               coordonnees geodesiques lat,lon)
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  nbpts : nombre de points a traiter
//@         (ENTREE)  BandeauAVHRR : bandeau AVHRR charge en memoire
//@         (ENTREE)  nbligbandeau : nombre de lignes total du bandeau
//@         (ENTREE)  col : colonnes fractionnaires des pixels AVHRR
//@         (ENTREE)  lig : lignes fractionnaires des pixels AVHRR
//@         (SORTIE)  lat : latitudes correspondant aux points AVHRR
//@         (SORTIE)  lon : longitudes correspondant aux points AVHRR
//@
//@ Return Status :
//@         t_CMN__Statut
//@         SD_BAS_STATUT_OK si la fonction a reussi
//@         SD_BAS_STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_AVHRRGeod(
        t_SD_BAS_DataThread * ctx,
        int  nbpts,
        t_SD_BAS_LigneAVHRR** BandeauAVHRR,
        int nbligbandeau,
        double * col,
        double *lig,
        double *lat,
        double *lon
            )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_MetopAVHRR
//@
//@ Description : Conversion de coordonnees Metop -> raster AVHRR
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape : gestion des erreurs
//@         (ENTREE)  IasiTime[2] : date du SN traite
//@         (ENTREE)  Mphr :  MPHR produit N0
//@         (ENTREE)  nbpts : nombre de points a traiter
//@         (ENTREE)  azim[nbpts] : coordonnees des pts dans le repere METOP (azim)
//@         (ENTREE)  elev[nbpts] : coordonnees des pts dans le repere METOP (elev)
//@         (ENTREE)  indicedeb : indice de la premiere ligne AVHRR utile
//@         (ENTREE)  indicefin : indice de la derniere ligne AVHRR utile
//@         (ENTREE)  BandeauAVHRR : bandeau AVHRR charge en memoire
//@         (ENTREE)  nbligbandeau : nombre de lignes total du bandeau
//@         (ENTREE)  AvhrrColLeft : numero de col du premier pixel AVHRR a analyser
//@         (ENTREE)  AvhrrColRight : numero de col du dernier pixel AVHRR a analyser
//@         (SORTIE)  lat : latitude
//@         (SORTIE)  lon : longitude
//@         (SORTIE)  Ang_mesY : angles de mesure METOP
//@         (SORTIE)  Ang_mesZ : angles de mesure METOP
//@         (SORTIE)  Azim_sol : angles solaire
//@         (SORTIE)  Elev_sol : angles solaire
//@         (SORTIE)  ColAVHRR : Colonnes AVHRR correspondant aux points metop (fraction de pixel)
//@         (SORTIE)  LigAVHRR : Lignes AVHRR correspondant aux points metop (fraction de pixel)
//@         (SORTIE)  EarthSatelDist : disance Terre/Satellite
//@
//@ Return Status :
//@         t_CMN__Statut
//@         SD_BAS_STATUT_OK si la fonction a reussi
//@         SD_BAS_STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_MetopAVHRR
(
    t_SD_BAS_DataThread * ctx,
    t_SD_BAS_SCDSTime * IasiTime,
    t_SD_BAS_MPHR * MphrN0,
    int  nbpts,
    double * azim,
    double * elev,
    int indicedeb,
    int indicefin,
    t_SD_BAS_LigneAVHRR** BandeauAVHRR,
    int nbligbandeau,
    int AvhrrColLeft,
    int AvhrrColRight,
    double *lat,              // Sortie SD_BAS_MetopGeod
    double *lon,              // Sortie SD_BAS_MetopGeod
    double *Ang_mesY,         // Sortie SD_BAS_MetopGeod
    double *Ang_mesZ,         // Sortie SD_BAS_MetopGeod
    double *Azim_sol,         // Sortie SD_BAS_MetopGeod
    double *Elev_sol,         // Sortie SD_BAS_MetopGeod
    double *ColAVHRR,
    double *LigAVHRR,
    double *EarthSatelDist
);
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_AVHRRMetop
//@
//@ Description : Conversion de coordonnees AVHRR (fractionnaires) -> Metop
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  IasiTime[2] : date du SN traite
//@         (ENTREE)  MphrN0 :  MPHR produit N0
//@         (ENTREE)  nbpts : nombre de points a traiter
//@         (ENTREE)  BandeauAVHRR : bandeau AVHRR charge en memoire
//@         (ENTREE)  nbligbandeau : nombre de lignes total du bandeau
//@         (ENTREE)  col : colonnes fractionnaires des pixels AVHRR
//@         (ENTREE)  lig : lignes fractionnaires des pixels AVHRR
//@         (SORTIE)  azim : azim Metop
//@         (SORTIE)  elev : elev Metop
//@
//@ Return Status :
//@         t_CMN__Statut
//@         SD_BAS_STATUT_OK si la fonction a reussi
//@         SD_BAS_STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_AVHRRMetop
(
    t_SD_BAS_DataThread * ctx,
    t_SD_BAS_SCDSTime * IasiTime,
    t_SD_BAS_MPHR * MphrN0,
    int  nbpts,
    t_SD_BAS_LigneAVHRR** BandeauAVHRR,
    int nbligbandeau,
    double * col,
    double * lig,
    double * azim,
    double * elev
);

// FT-2997
	//@ ------------------------------------------------------------------*/
	//@ Procedure : SD_BAS_MetopAVHRR_SansAVHRR
	//@
	//@ Description : Conversion de coordonnees Metop -> raster AVHRR
	//@
	//@ Parameters :
	//@         (ENTREE)  ctx : contexte de l'etape : gestion des erreurs
	//@         (ENTREE)  nbpts : nombre de points a traiter
	//@         (ENTREE)  IasiTime[2] : date du SN traite
	//@         (ENTREE)  MphrN0 :  MPHR produit N0
	//@         (ENTREE)  azimuth : coordonnees des pts dans le repere METOP (azim)
	//@         (ENTREE)  elevation : coordonnees des pts dans le repere METOP (elev)
	//@         (ENTREE)  TimeFirst : date de debut (IasiTime - DeltaTime)
	//@         (ENTREE)  TimeLast : date de fin (IasiTime + DeltaTime)
	//@         (ENTREE)  DateFirstLineAvhrr : nombre de lignes total du bandeau
	//@         (SORTIE)  AvhrrCol[nbpts] : Colonnes AVHRR correspondant aux points metop (fraction de pixel)
	//@         (SORTIE)  AvhrrLin[nbpts] : Lignes AVHRR correspondant aux points metop (fraction de pixel)
	//@
	//@ Return Status :
	//@         t_CMN__Statut
	//@         SD_BAS_STATUT_OK si la fonction a reussi
	//@         SD_BAS_STATUT_NOK si la fonction a echoue
	//@
	//@ ------------------------------------------------------------------*/
	t_CMN__Statut SD_BAS_MetopAVHRR_SansAVHRR
	(
	    // Entrees
	    t_SD_BAS_DataThread * ctx,
	    int  nbpts,
	    t_SD_BAS_SCDSTime * IasiTime,
	    t_SD_BAS_MPHR * MphrN0,
    	double * azimuth,
    	double * elevation,
	    double TimeFirst,
	    double TimeLast,
	    double DateFirstLineAvhrr,
	    
	    // Sorties
	    double * AvhrrLin, 
	    double * AvhrrCol   
	);
	//@ ------------------------------------------------------------------*/
	//@ Procedure : SD_BAS_AVHRRMetop_SansAVHRR
	//@
	//@ Description : Conversion de coordonnees AVHRR (fractionnaires) -> Metop
	//@
	//@ Parameters :
	//@         (ENTREE)  ctx : contexte de l'etape
	//@         (ENTREE)  nbpts : nombre de points a traiter
	//@         (ENTREE)  IasiTime[2] : date du SN traite
	//@         (ENTREE)  MphrN0 :  MPHR produit N0
	//@         (ENTREE)  AvhrrLin[nbpts] : Lignes AVHRR correspondant aux points metop (fraction de pixel)
	//@         (ENTREE)  AvhrrCol[nbpts] : Colonnes AVHRR correspondant aux points metop (fraction de pixel)
	//@         (ENTREE)  DateFirstLineAvhrr : nombre de lignes total du bandeau
	//@         (SORTIE)  azim : azim Metop
	//@         (SORTIE)  elev : elev Metop
	//@
	//@ Return Status :
	//@         t_CMN__Statut
	//@         SD_BAS_STATUT_OK si la fonction a reussi
	//@         SD_BAS_STATUT_NOK si la fonction a echoue
	//@
	//@ ------------------------------------------------------------------*/
	t_CMN__Statut SD_BAS_AVHRRMetop_SansAVHRR
	(
	    // Entrees
	    t_SD_BAS_DataThread * ctx,
	    int  nbpts,
	    t_SD_BAS_SCDSTime * IasiTime,
	    t_SD_BAS_MPHR * MphrN0,
	    double * AvhrrLin, 
	    double * AvhrrCol,   
	    double DateFirstLineAvhrr,
	    // Sorties
    	double * azim,
    	double * elev
	);
// ...FT-2997	
	
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_InitialiserOperateur
//@
//@ Description : cette fonction permet de creer les mutex utiles pour rendre
//@     threadsafe metop_pointing (bloquante)
//@     afin de la rendre thread safe
//@
//@
//@ Parameters :  aucun
//@
//@ Return Status :
//@         t_CMN__Statut
//@         SD_BAS_STATUT_OK si la fonction a reussi
//@         SD_BAS_STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_InitialiserOperateur(
  t_SD_BAS_DataThread * ctx);
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_DetruireOperateur
//@
//@ Description : cette fonction permet de detruire les mutex utiles pour rendre
//@     threadsafe metop_pointing (bloquante)
//@     afin de la rendre thread safe
//@
//@
//@ Parameters :  aucun
//@
//@ Return Status :
//@         t_CMN__Statut
//@         SD_BAS_STATUT_OK si la fonction a reussi
//@         SD_BAS_STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_DetruireOperateur(
    void
            )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_ReserverOperateur
//@
//@ Description : cette fonction permet de reserver l'acces a la librairie
//@     threadsafe metop_pointing (bloquante)
//@     afin de la rendre thread safe
//@
//@
//@ Parameters :  aucun
//@
//@ Return Status :
//@         t_CMN__Statut
//@         SD_BAS_STATUT_OK si la fonction a reussi
//@         SD_BAS_STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_ReserverOperateur(
    void
            )
;

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LibererOperateur
//@
//@ Description : cette fonction permet de liberer l'acces a la librairie
//@     threadsafe metop_pointing (bloquante)
//@     afin de la rendre thread safe
//@
//@
//@ Parameters :  aucun
//@
//@ Return Status :
//@         t_CMN__Statut
//@         SD_BAS_STATUT_OK si la fonction a reussi
//@         SD_BAS_STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LibererOperateur(
      void      )
;

//======================================================================
#ifdef __cplusplus
}
#endif

#endif
