//$Header: /home/ops_gc/cvs-repository/OPS/SD/BAS/src/SD_BAS_Produits.c,v 1.39 2003/02/21 17:12:06 ott 
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_BAS
//@ MODULE : SD_BAS_Produits
//@
//@ Comment :
//@    Code file for the  SD_BAS_Produits class which includes the
//@    acces to the generic parts of the product  :
//@	- IASI L0
//@	- IASI L1
//@     - AVHRR L0
//@	this is done thanks to SD_BAS_Conversion
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 13/12/2002 : [reference :] Ott
//@ --------------------------------------------------------------------
//@ VERSION : 04/09/2003 : [reference :] Benjamin MASSART
//@ cette version prend en compte les evolutions du code pour une gestion
//@ dynamique de l'ecriture des IPR dans les produits de niveau 1 suite
//@ aux modifications apparues dans la version 6.4 du GPFS et la version
//@ 6.5 du IASI Level 1 Product Format Specification
//@ --------------------------------------------------------------------
//@ DM-ID : PM-3 : 04/09/2003 :
//@ -SD_BAS_TAILLE_IPR definie dans SD_Produit.h pour etre accessible depuis
//@  SD_DON_GranuleN1::DecrireIPR()
//@ -SD_BAS_OFFSET_IPR_N1 definie dans SD_Produit.h pour etre accessible depuis
//@  SD_DON_GranuleN1::DecrireIPR()
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-126-CN : 01/09/2004 : P.CABANE
//@ Reading of the software has been updated
//@ fiche de correction correspondante : IA-2100-FC-126-THA
//@ --------------------------------------------------------------------
//@ PM-ID :IA-2100-CR-301-CN : 03/07/2006 : B. MASSART
//@ SD_BAS_DecommuteGRH return status corrected. (patch3-5-5). 
//@ See IA-2100-CR-301-THA.
//@ --------------------------------------------------------------------
//@ MODIFIED FOR PATCH3-5-5 : 05/Jul/06
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-162-CN : 09/2006 : C.MERIGUET
//@ Gestion des IPR en mode DUMP
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-3358(DM-2100-OPS) : 13/04/2017 : MLA
//@ Pour produits > 2Go, il faut utiliser des long long pour stocker la position du curseur 
//@ et les fonctions fseeko et ftello
//@ --------------------------------------------------------------------
//@FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <stdio.h>
#include <unistd.h>

// declaration des fonctions importees
#include "SD_BAS_Produits.h"

/*====================================================================*/
/* Local Constant Declaration                                         */
/*====================================================================*/



const char* SD_BAS_NOM_FIC_PROD ="SD_BAS_Produits.env";	 /* fichier descripteur de contenu */
const char* SD_BAS_NOM_FIC_N0 = "SD_BAS_Niveau0.env";
const char* SD_BAS_NOM_FIC_N1 = "SD_BAS_Niveau1.env";
const char* SD_BAS_NOM_FIC_AVHRR = "SD_BAS_AVHRR.env";
const char* SD_BAS_NOM_FIC_MPHR = "SD_BAS_MPHR_FIELD.model";    /* fichier definissant les champs ASCII des MPHR */
const char* SD_BAS_NOM_FIC_INTER = "SD_BAS_Niveau1.tmp";
const char* SD_BAS_BLOC_MPHR = "MPHR";	 	/* nom du bloc MPHR */
const int SD_BAS_TAILLE_MPHR = 3307;	 	/* taille du MPHR */
const int SD_BAS_OFFSET_MPHR = 0;	       	/* taille du MPHR */
const char* SD_BAS_BLOC_PRODMOD = "PRODMOD";   	/* nom du bloc PRODMOD */
const int SD_BAS_TAILLE_PRODMOD = 3287;	 	/* taille du PRODMOD */
// FT-2765
static const char* SD_BAS_BLOC_GRH  ="GRH"; 	 	/* nom du bloc GRH */
// ...FT-2765
const char* SD_BAS_BLOC_IPRGIADR = "IPRGIADR";  /* nom du bloc IPR VIADR */
const char* SD_BAS_BLOC_IPRVIADR = "IPRVIADR"; 	/* nom du bloc IPR VIADR */
const char* SD_BAS_BLOC_IPRMDR = "IPRMDR";     	/* nom du bloc IPR MDR */
//const int SD_BAS_OFFSET_IPR_N0 = 3675;		/* offset des IPR */
const int SD_BAS_OFFSET_IPR_N0 = 3307;		/* offset des IPR */
const int SD_BAS_OFFSET_IPR_AVHRR = 3450;	/* offset des IPR */
//version 6.3 
//const int SD_BAS_OFFSET_IPR_AVHRR = 3437;	/* offset des IPR */
const int SD_BAS_TAILLE_DUMMY = 21;	 	/* taille des DMDR */
// FT-2765
static const char* SD_BAS_BLOC_DUMMY = "DUMMY"; 	/* nom du bloc dmdr */
// ...FT-2765
const char* SD_BAS_BLOC_IPR = "IPR";                 /* nom du bloc IPR */
#define SD_BAS_NB_GRH_FIELD 7
#define SD_BAS_NB_IPR_FIELD 4
#define SD_BAS_NB_DATE 2



/* taille du buffer de recopie de fichier */
const int SD_BAS_TAILLE_BLOC_MAX = 100000000;

/* variable d'environnement contenant le repertoire des fichier .env */
char * REP_CONF_DIR          = "REP_CONF_DIR"; 
/*  variable d'environnement contenant le repertoire des fichier temporaire */
char * REP_TEMP_DIR          = "REP_DATA_TMP_DIR";
char * SD_BAS_SEP_REPERTOIRE = "/";

/* message d'erreur */
char *SD_BAS_MSG_REMOVE = "REMOVE FILE ERROR";
char *SD_BAS_MSG_MISSING_FILE ="MISSING CONFIGURATION FILES";
char *SD_BAS_MSG_ENVIRONNEMENT = "MISSING ENVIRONNEMENT VARIABLE";

char * TypeAcces[]= { "rb","wb","rb+","wb"};

/*====================================================================*/
/* Local types Declaration                                            */
/*====================================================================*/
//======================================================================
// Internal fonctions Declaration
//======================================================================

//======================================================================
// Global variables Definition
//======================================================================

static int fichier_descripteur = NON_CHARGE;

//======================================================================
//					FONCTIONS EXTERNES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_OuvrirProduit
//@
//@ Description : fonction d'ouverture du produit en lecturqe ou en ecriture
//@  cette operation generique s'applique aux produits IASI de N0 N1 et AVHRR
//@ remarque : cette fonction peut s'appliquer aux productmodel
//@
//@
//@ Parameters :
//@			(ENTREE)  mode  : mode d'ouverture (LECTURE, ECRITURE, APPEND)
//@			(ENTREE)  nature  : nature du produit (IASI, AVHRR, PRODMODEL)
//@			(ENTREE)  nom  : nom du produit
//@			(SORTIE)  descripteur  : descripteur du produit
//@				  utilise par les fonctions d'acces
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
/* FA 162 */
t_CMN__Statut SD_BAS_OuvrirProduit(
                                   e_BAS_TypeAcces mode, 
                                   e_BAS_Nature nature,
                                   t_BAS_NAMEPROD nom,
                                   t_SD_BAS_Fd * Descripteur,
                                   t_SD_BAS_DataThread *ctx,
				   const e_BAS_ModeTrt modeTraitement
                                   )

/* ..FA 162 */
{
  static const char tcProcName_F[] = "SD_BAS_OuvrirProduit";
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  int num_lig = 0;
  t_SD_BAS_status_d status;
  int ret = 0;
  off_t taille_mdr = 0;
  off_t tailleFic = 0;
  char *msg_err = NULL;
  char           fichierConfiguration[SD_BAS_NAMEPROD];
  char *repertoire = NULL;

  // FT-2765
  memset( Descripteur, '\0', sizeof( t_SD_BAS_Fd ) );
  // ...FT-2765

  // ouvrir le fichier dans le mode demande fopen ("rb", "wb", "ab")
  Descripteur->fd = fopen(nom, TypeAcces[mode]);

  // si erreur sortir en indiquant l'erreur dans la structure t_SD_BAS_Fd
  if( Descripteur->fd ==NULL)
    {
      cr_op = SD_BAS_ERR_BAS_NIVEAU;
      num_lig = __LINE__;
      msg_err = SD_BAS_MSG_OPEN;
      goto fin;
    }
  
  // initialiser la structure t_SD_BAS_Fd avec le nom du  fichier 
  // le file descripteur et le mode d'acces et la nature
  strcpy(Descripteur->nom,nom);
  Descripteur->mode = mode;
  Descripteur->nature = nature;
  
  // initialiser l'etat a INIT
  Descripteur->etat = INIT;  

   

  // si mode APPEND_INIT
  if(mode == APPEND_INIT)
    {
           
      /* on cree un fichier temporaire ou seront ecrit les MDRs */
      
      /* on recupere le repertoire des ficiers temporaire */
      if((repertoire=getenv(REP_TEMP_DIR))==NULL)
	{
	  cr_op = SD_BAS_ERR_BAS_NIVEAU;
	  num_lig = __LINE__;
	  msg_err = SD_BAS_MSG_ENVIRONNEMENT;
          goto fin;
	}
      

/* FA 162 */
      if ( (DUMP == modeTraitement) && (Descripteur->nature == NA_IASIL1ENG) )
      {
         /* on cree un fichier temporaire pour stocker les VIADR */
         sprintf(Descripteur->nom_viadr_tmp, 
                "%s%s%d%d%s", 
                repertoire, 
                SD_BAS_SEP_REPERTOIRE, 
                Descripteur->nature, 
                getpid(), 
                "viadr.tmp");

         /* on ouvre le fichier temporaire */
         /* (ecriture en mode APPEND_INIT) */
         Descripteur->fd_viadr_tmp = fopen(Descripteur->nom_viadr_tmp, TypeAcces[mode]);
      
         // si erreur sortir en indiquant l'erreur dans la structure t_SD_BAS_Fd
         if( Descripteur->fd_viadr_tmp == NULL)
         {
	   cr_op = SD_BAS_ERR_BAS_NIVEAU;
	   num_lig = __LINE__;
	   msg_err = SD_BAS_MSG_OPEN; 
	   goto fin;
         }
	 
      }/* fin si fichier ENG*/	
/* ..FA 162 */
      
      /* on cree le nom du fichier temporaire avec le pid de procesus et la nature du produit */
      sprintf(Descripteur->nom_sec, 
              "%s%s%d%d%s", 
              repertoire, 
              SD_BAS_SEP_REPERTOIRE, 
              Descripteur->nature, 
              getpid(), 
              SD_BAS_NOM_FIC_INTER);

      /* on ouvre le fichier temporaire */
      /* (ecriture en mode APPEND_INIT) */
      Descripteur->fd_sec = fopen(Descripteur->nom_sec, TypeAcces[mode]);
      
      // si erreur sortir en indiquant l'erreur dans la structure t_SD_BAS_Fd

      if( Descripteur->fd_sec == NULL)
	{
	  cr_op = SD_BAS_ERR_BAS_NIVEAU;
	  num_lig = __LINE__;
	  msg_err = SD_BAS_MSG_OPEN; 
	  goto fin;
	}
      
    }    

  /* si mode APPEND */
  if(mode == APPEND)
    {
      /* on recupere le repertoire des ficiers temporaire */
      if((repertoire=getenv(REP_TEMP_DIR))==NULL)
	{
	  cr_op = SD_BAS_ERR_BAS_NIVEAU;
	  num_lig = __LINE__;
	  msg_err = SD_BAS_MSG_ENVIRONNEMENT;
          goto fin;
	}

/* FA 162 */
      if ( (DUMP == modeTraitement) && (Descripteur->nature == NA_IASIL1ENG) )
      {
         /* on cree un fichier temporaire pour stocker les VIADR */
         sprintf(Descripteur->nom_viadr_tmp, 
                "%s%s%d%d%s", 
                repertoire, 
                SD_BAS_SEP_REPERTOIRE, 
                Descripteur->nature, 
                getpid(), 
                "viadr.tmp");

         /* on ouvre le fichier temporaire */
         /* (ecriture en mode APPEND_INIT) */
         Descripteur->fd_viadr_tmp = fopen(Descripteur->nom_viadr_tmp, TypeAcces[mode]);
      
         // si erreur sortir en indiquant l'erreur dans la structure t_SD_BAS_Fd
         if( Descripteur->fd_viadr_tmp == NULL)
         {
	   cr_op = SD_BAS_ERR_BAS_NIVEAU;
	   num_lig = __LINE__;
	   msg_err = SD_BAS_MSG_OPEN; 
	   goto fin;
         }
	 
      }/* fin si fichier ENG*/	
/* ..FA 162 */
      /* on cree le nom du fichier temporaire avec le pid de procesus et la nature du produit */
      sprintf(Descripteur->nom_sec,
              "%s%s%d%d%s", 
              repertoire, 
              SD_BAS_SEP_REPERTOIRE, 
              Descripteur->nature, 
              getpid(), 
              SD_BAS_NOM_FIC_INTER);
      

      /* on ouvre le fichier temporaire */
      /*  en mode APPEND) */
      Descripteur->fd_sec = fopen(Descripteur->nom_sec, TypeAcces[mode]);

      // si erreur sortir en indiquant l'erreur dans la structure t_SD_BAS_Fd     
      if( Descripteur->fd_sec == NULL)
	{
	  cr_op = SD_BAS_ERR_BAS_NIVEAU;
	  num_lig = __LINE__;
	  msg_err = SD_BAS_MSG_OPEN; 
	  goto fin;
	}


      /* on appele LireIPR pour mettre a jour les offsets du descripteur */
       cr_op = SD_BAS_LireIPR(Descripteur, ctx); 
       if(cr_op != CMN__STATUT_OK) 
         {
           goto fin;
         }
      /* on recupere la taille du fichier principal */
      tailleFic = ftello(Descripteur->fd);
      Descripteur->offsetVIADRSuivant = tailleFic;
     
      /* on force les offset des VIADR et MDR */
      /* on ecrit les viadr supplementaires en fin de fichier principal */

      ret = fseeko(Descripteur->fd, 0, SEEK_END);
      if(ret == -1)
	{
	  cr_op = SD_BAS_ERR_BAS_NIVEAU;
	  num_lig = __LINE__;
	  msg_err = SD_BAS_MSG_SEEK;
	  goto fin;
	}

      /* on recupere la taille du fichier secondaire */
      taille_mdr = ftello(Descripteur->fd);
      Descripteur->offsetVIADRSuivant = taille_mdr;


      /* on ecrit les MDR supplementaires en fin de fichier secondaire */
      ret = fseeko(Descripteur->fd_sec, 0, SEEK_END);
      if(ret == -1)
	{
	  cr_op = SD_BAS_ERR_BAS_NIVEAU;
	  num_lig = __LINE__;
	  msg_err = SD_BAS_MSG_SEEK;
	  goto fin;
	}

      /* on recupere la taille du fichier secondaire */
      taille_mdr = ftello(Descripteur->fd_sec);
      Descripteur->offsetMDRSuivant = taille_mdr;
 
      
    } /* fin mode append */

  if (mode == LECTURE  ||  mode == ECRITURE || mode == APPEND_INIT)
    {   
      // si mode lecture
      //   initialiser les offset les numero et les nombres a 0
      // si mode creation
      //   initialiser les offset les numero et les nombres a 0
     
      Descripteur->offsetGAD = 0;
      Descripteur->offsetGADSuivant = 0;
     
      Descripteur->offsetVIADR = 0;
      Descripteur->offsetVIADRSuivant = 0;
      Descripteur->nombreVIADR = 0;

      Descripteur->offsetMDR = 0;
      Descripteur->offsetMDRSuivant = 0;
      Descripteur->nombreMDR = 0; 
    }
  
 
  /* on recupere le repertoire des fichiers de configuration */
  if((repertoire=getenv(REP_CONF_DIR))==NULL)
    {
      cr_op = SD_BAS_ERR_BAS_NIVEAU;
      num_lig = __LINE__;
      msg_err = SD_BAS_MSG_ENVIRONNEMENT;
      goto fin;
    }


  // on verifie que les fichiers descripteurs  n'ont pas ete  deja charge
  if(fichier_descripteur == NON_CHARGE )
    {
      fichier_descripteur = CHARGE;

      // Ouvrir le fichier de configuration de l'application
      sprintf(fichierConfiguration, 
              "%s%s%s", 
              repertoire, 
              SD_BAS_SEP_REPERTOIRE, 
              SD_BAS_NOM_FIC_PROD);

      //on teste l'existence du fichier
      if( access(fichierConfiguration, R_OK) == -1)
        {
          num_lig = __LINE__;
          msg_err = SD_BAS_MSG_MISSING_FILE;
          cr_op = SD_BAS_ERR_MISSING_FILE;
          goto fin;
        }

      status = DECOM__Charger(fichierConfiguration);
      SD_BAS_TEST_DECOM(status, SD_BAS_ERR_CHARGER ,__LINE__);
    
      /* on ouvre les fichiers de conf de niveau 0 */
      sprintf(fichierConfiguration, 
              "%s%s%s", 
              repertoire, 
              SD_BAS_SEP_REPERTOIRE, 
              SD_BAS_NOM_FIC_N0);

      //on teste l'existence du fichier
      if( access(fichierConfiguration, R_OK) == -1)
        {
          num_lig = __LINE__;
          msg_err = SD_BAS_MSG_MISSING_FILE;
          cr_op = SD_BAS_ERR_MISSING_FILE;
          goto fin;
        }


      status = DECOM__Charger(fichierConfiguration);
      SD_BAS_TEST_DECOM(status,SD_BAS_ERR_CHARGER,__LINE__);

      // on met a jour l'indicateur du fichier descripteur
   
      /* on ouvre les fichiers de conf de niveau 1 */
      sprintf(fichierConfiguration, 
              "%s%s%s", 
              repertoire, 
              SD_BAS_SEP_REPERTOIRE, 
              SD_BAS_NOM_FIC_N1);

      //on teste l'existence du fichier
      if( access(fichierConfiguration, R_OK) == -1)
        {
          num_lig = __LINE__;
          msg_err = SD_BAS_MSG_MISSING_FILE;
          cr_op = SD_BAS_ERR_MISSING_FILE;
          goto fin;
        }

      status = DECOM__Charger(fichierConfiguration);
      SD_BAS_TEST_DECOM(status,SD_BAS_ERR_CHARGER,__LINE__);

     

      /* on ouvre les fichiers de conf des produits avhrr  */
      sprintf(fichierConfiguration, 
              "%s%s%s", 
              repertoire, 
              SD_BAS_SEP_REPERTOIRE, 
              SD_BAS_NOM_FIC_AVHRR);

     
      //on teste l'existence du fichier
      if( access(fichierConfiguration, R_OK) == -1)
        {
          num_lig = __LINE__;
          msg_err = SD_BAS_MSG_MISSING_FILE;
          cr_op = SD_BAS_ERR_MISSING_FILE;
          goto fin;
        }
    

      // on charge SD_BAS_AVHRR.env
      status = DECOM__Charger(fichierConfiguration);
      SD_BAS_TEST_DECOM(status,SD_BAS_ERR_CHARGER,__LINE__);
    }


  // on initialise l'indicateur de coherence temporelle
  Descripteur->dateSCDSprec.day = 0;
  Descripteur->dateSCDSprec.millis = 0; 


 fin:

  // sortie de la fonction 
  if ( cr_op != CMN__STATUT_OK )
    {
      SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                          SD_BAS_ERR_SET_Produits,
                          SD_BAS_ERR_NUM_JDB,
                          tcProcName_F,
                          num_lig,
                          msg_err);
    }

  return(cr_op); 
}


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_FermerProduit
//@
//@ Description : fonction de fermeture du produit en lecture ou en ecriture
//@  cette operation generique s'applique aux produits IASI de N0 N1 et AVHRR
//@ remarque : cette fonction peut s'appliquer au productmodel
//@
//@
//@ Parameters :
//@			(ENTREE)  descripteur  : descripteur du produit
//@				  utilise par les fonctions d'acces
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
/* FA 162 */
t_CMN__Statut SD_BAS_FermerProduit(
                                   e_BAS_TypeAcces mode, 
				   t_SD_BAS_Fd * Descripteur,
                                   t_SD_BAS_DataThread *ctx,
				   const e_BAS_ModeTrt modeTraitement
                                   )
/* .. FA 162 */    

{
  /* nom de la fonction */
  static const char tcProcName_F[] = "SD_BAS_FermerProduit";
  /* retour de la fonction */
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  /* retour de fonction */
  int ret = 0;
  /* elements lus ou ecrits */
  size_t nbElement = 0;
  /* numero de ligne pour le contexte d'erreur */
  int num_lig = 0; 
  /* bloc recopie en mode append */
  char *mdr = NULL; 
  /* taille du bloc */
  // FT-2765
  off_t  taille_mdr = 0;
  // ...FT-2765
  /* message d'erreur pour le contexte d'erreur */
  char *msg_err = NULL;
  /* taille des blocs recopie */
  off_t  taille = 0;
/* FA 162 */
  /* bloc recopie en mode append */
  char *viadr = NULL; 
  /* taille du bloc */
  off_t taille_viadr = 0;
 
/* ..FA 162 */

  /* on verifie que l'etat est a INIT ou ENCOURS */
  if( Descripteur->etat == INIT || Descripteur->etat == ENCOURS)
    {
      /* si mode append */
      if(mode == APPEND)
        {
          /* on ferme le fichier secondaire */
          ret = fclose( Descripteur->fd_sec );
	 
          /* si erreur sortir en indiquant l'erreur dans la structure t_SD_BAS_Fd*/
          if(ret == EOF)
            {
              cr_op = SD_BAS_ERR_BAS_NIVEAU;
              num_lig = __LINE__;
              msg_err = SD_BAS_MSG_CLOSE;
              goto fin;
            }     
/* FA 162 */
           /** on ferme aussi le fichier tempo des viadr **/
           if ( (DUMP == modeTraitement) && (Descripteur->nature == NA_IASIL1ENG) )
           {
               ret = fclose( Descripteur->fd_viadr_tmp );	 
               /* si erreur sortir en indiquant l'erreur dans la structure t_SD_BAS_Fd*/
               if(ret == EOF)
               {
                  cr_op = SD_BAS_ERR_BAS_NIVEAU;
                  num_lig = __LINE__;
                  msg_err = SD_BAS_MSG_CLOSE;
                  goto fin;
               }     
	      
           }
	  
/* ..FA 162 */

        }/* fin mode append */


      /* si mode append_close */
      if(mode == APPEND_CLOSE)
        {         
/* FA 162 */
           /** ecriture des VIADR si fichier ENG **/
           if ( (DUMP == modeTraitement) && (Descripteur->nature == NA_IASIL1ENG) )
           {
	      /* on se place d'abord en fin de fichier secondaire */
              ret = fseeko(Descripteur->fd_viadr_tmp, 0, SEEK_END);
              if(ret == -1)
              {
                 cr_op = SD_BAS_ERR_BAS_NIVEAU;
                 num_lig = __LINE__;
                 msg_err = SD_BAS_MSG_SEEK;
                 goto fin;
              }
         
              /* on recupere la taille du fichier */
              taille_viadr = ftello(Descripteur->fd_viadr_tmp);

              /* on se  replace en debut de fichier secondaire */
              ret = fseeko(Descripteur->fd_viadr_tmp, 0, SEEK_SET);
              if(ret == -1)
              {
                 cr_op = SD_BAS_ERR_BAS_NIVEAU;
                 num_lig = __LINE__;
                 msg_err = SD_BAS_MSG_SEEK;
                 goto fin;
              }

              /* on se place a la fin du fichier principal*/
              ret = fseeko(Descripteur->fd, 0, SEEK_END);
              if(ret == -1)
              {
                 cr_op = SD_BAS_ERR_BAS_NIVEAU;
                 num_lig = __LINE__;
                 msg_err = SD_BAS_MSG_SEEK;
                 goto fin;
              }
	 
              /* on alloue un buffer de la taille SD_BAS_TAILLE_BLOC_MAX */
              if(SD_BAS_MacMALLOC(viadr, char, SD_BAS_TAILLE_BLOC_MAX)==FALSE)
              {
                 cr_op = SD_BAS_ERR_BAS_NIVEAU;
                 num_lig = __LINE__;
                 msg_err = SD_BAS_MSG_ALLOC;
                 goto fin;
              }


              /* on recopie le fichier temporaire en fin du fichier principal */
              /* par bloc de taille SD_BAS_TAILLE_BLOC_MAX */

              while(taille_viadr >  0)
              {
                 /* si on lire des blocs entier */
                 if(taille_viadr > SD_BAS_TAILLE_BLOC_MAX)
                 {
                   taille = SD_BAS_TAILLE_BLOC_MAX;
                 }
                 /* sinon */
                 else
                 {
                   taille = taille_viadr;
                 }
	   
	     
                 /* on lit les VIADR dans le fichier temporaire  */
                 nbElement = fread(viadr, taille, 1, Descripteur->fd_viadr_tmp);
                 if (nbElement != 1)
                 {
                   cr_op = SD_BAS_ERR_BAS_NIVEAU;
                   num_lig = __LINE__;
                   msg_err = SD_BAS_MSG_READ;
                   goto fin;
                 }
	     
                 /* on les recopie dans le fichier principal */
                 nbElement = fwrite(viadr, taille, 1, Descripteur->fd);
                 if (nbElement != 1)
                 {
                   cr_op = SD_BAS_ERR_BAS_NIVEAU;
                   num_lig = __LINE__;
                   msg_err = SD_BAS_MSG_WRITE;
                   goto fin;
                 }

	     
                 /* on decremente la taille de viadr a lire */
                 taille_viadr = taille_viadr - SD_BAS_TAILLE_BLOC_MAX;
              } /* fin while */

          /* on libere le buffer */
          SD_BAS_MacFREE(viadr);

          /* fermeture du fichier temporaire  */
          ret = fclose( Descripteur->fd_viadr_tmp );
          /* si erreur sortir en indiquant l'erreur dans la structure t_SD_BAS_Fd*/
          if(ret == EOF)
            {
              cr_op = SD_BAS_ERR_BAS_NIVEAU;
              num_lig = __LINE__;
              msg_err = SD_BAS_MSG_CLOSE;
              goto fin;
            }     
	 

          /* on supprime le fichier secondaire */
          ret = remove(Descripteur->nom_viadr_tmp);
          if(ret != 0)
            {
              cr_op = SD_BAS_ERR_REMOVE;
              num_lig = __LINE__;
              msg_err = SD_BAS_MSG_REMOVE;
            }
	      
           }/* fin si fichier ENG */
	  
	  /** Ecriture des MDR **/
/* ..FA 162 */

	  /* on se place d'abord en fin de fichier secondaire */
          ret = fseeko(Descripteur->fd_sec, 0, SEEK_END);
          if(ret == -1)
            {
              cr_op = SD_BAS_ERR_BAS_NIVEAU;
              num_lig = __LINE__;
              msg_err = SD_BAS_MSG_SEEK;
              goto fin;
            }
         
          /* on recupere la taille du fichier */
          taille_mdr = ftello(Descripteur->fd_sec);

          /* on se  replace en debut de fichier secondaire */
          ret = fseeko(Descripteur->fd_sec, 0, SEEK_SET);
          if(ret == -1)
            {
              cr_op = SD_BAS_ERR_BAS_NIVEAU;
              num_lig = __LINE__;
              msg_err = SD_BAS_MSG_SEEK;
              goto fin;
            }

          /* on se place a la fin du fichier principal*/
          ret = fseeko(Descripteur->fd, 0, SEEK_END);
          if(ret == -1)
            {
              cr_op = SD_BAS_ERR_BAS_NIVEAU;
              num_lig = __LINE__;
              msg_err = SD_BAS_MSG_SEEK;
              goto fin;
            }
	 
          /* on alloue un buffer de la taille SD_BAS_TAILLE_BLOC_MAX */
          if(SD_BAS_MacMALLOC(mdr, char, SD_BAS_TAILLE_BLOC_MAX)==FALSE)
            {
              cr_op = SD_BAS_ERR_BAS_NIVEAU;
              num_lig = __LINE__;
              msg_err = SD_BAS_MSG_ALLOC;
              goto fin;
            }


          /* on recopie le fichier intermediaires en fin du fichier principal */
          /* par bloc de taille SD_BAS_TAILLE_BLOC_MAX */

          while(taille_mdr >  0)
            {
              /* si on lire des blocs entier */
              if(taille_mdr > SD_BAS_TAILLE_BLOC_MAX)
                {
                  taille = SD_BAS_TAILLE_BLOC_MAX;
                }
              /* sinon */
              else
                {
                  taille = taille_mdr;
                }
	   
	     
              /* on lit les MDR dans le fichier secondaire */
              nbElement = fread(mdr, taille, 1, Descripteur->fd_sec);
              if (nbElement != 1)
                {
                  cr_op = SD_BAS_ERR_BAS_NIVEAU;
                  num_lig = __LINE__;
                  msg_err = SD_BAS_MSG_READ;
                  goto fin;
                }
	     
              /* on les recopie dans le fichier principal */
              nbElement = fwrite(mdr, taille, 1, Descripteur->fd);
              if (nbElement != 1)
                {
                  cr_op = SD_BAS_ERR_BAS_NIVEAU;
                  num_lig = __LINE__;
                  msg_err = SD_BAS_MSG_WRITE;
                  goto fin;
                }

	     
              /* on decremente la taille de mdr a lire */
              taille_mdr = taille_mdr - SD_BAS_TAILLE_BLOC_MAX;
            }

          /* on libere le buffer */
          SD_BAS_MacFREE(mdr);

          /* fermeture du fichier intermediaire */
          ret = fclose( Descripteur->fd_sec );
          /* si erreur sortir en indiquant l'erreur dans la structure t_SD_BAS_Fd*/
          if(ret == EOF)
            {
              cr_op = SD_BAS_ERR_BAS_NIVEAU;
              num_lig = __LINE__;
              msg_err = SD_BAS_MSG_CLOSE;
              goto fin;
            }     
	 

          /* on supprime le fichier secondaire */
          ret = remove(Descripteur->nom_sec);
          if(ret != 0)
            {
              cr_op = SD_BAS_ERR_REMOVE;
              num_lig = __LINE__;
              msg_err = SD_BAS_MSG_REMOVE;
            }
        }


      /* fermeture du fichier principal */
      ret = fclose( Descripteur->fd );

      /* si erreur sortir en indiquant l'erreur dans la structure t_SD_BAS_Fd */
      if(ret == EOF)
        {
          cr_op = SD_BAS_ERR_BAS_NIVEAU;
          num_lig = __LINE__;
          msg_err = SD_BAS_MSG_CLOSE;
          goto fin;
        }
    }


 fin:

  // sortie de la fonction 
  if ( cr_op != CMN__STATUT_OK )
    {
      /* on enregistre l'erreur */
      SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                          SD_BAS_ERR_SET_Produits,
                          SD_BAS_ERR_NUM_JDB,
                          tcProcName_F,
                          num_lig,
                          msg_err);    
    }

  return(cr_op); 
}


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LireMPHR
//@
//@ Description : fonction de lecture du MPHR d'un produit 
//@  cette operation generique s'applique aux produits IASI de N0 N1 et AVHRR
//@ remarque : cette fonction peut s'appliquer au productmodel
//@
//@
//@ Parameters :
//@			(ENTREE)  descripteur  : descripteur du produit
//@				  utilise par les fonctions d'acces
//@			(SORTIE)  mphr  : liste des variables lues dans le MPHR
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LireMPHR(
		t_SD_BAS_Fd * Descripteur,
		t_SD_BAS_MPHR * mphr,
		t_SD_BAS_DataThread *ctx
			)
{
  static const char tcProcName_F[] = "SD_BAS_LireMPHR";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  char *fic = NULL;
  char nom_bloc[DECOM__C_lg_nom_bloc ];
  int ret = 0;
  size_t nbElement = 0;
  t_SD_BAS_status_d status_d;
  int lg = 0;
  int num_lig = 0;
  char *msg_err = NULL;

  // on verifie que l'etat est a INIT ou ENCOURS 
  // et que le mode est lecture ou append
  if ( ((Descripteur->etat == INIT) || (Descripteur->etat == ENCOURS))
       &&( (Descripteur->mode == LECTURE) || (Descripteur->mode == APPEND)))
    {
      
#ifdef PROD_MOD_SANS_ENTETE_BINAIRE
      // si type different de producmodel 
      
      if (Descripteur->nature != NA_PRODMODEL )
        {
#endif
          //   on alloue un tableau de char de taille SD_BAS_TAILLEMPHR
          if( SD_BAS_MacMALLOC(fic,char,SD_BAS_TAILLE_MPHR) == FALSE)
            {
              cr_op = SD_BAS_ERR_BAS_NIVEAU;
              num_lig = __LINE__;
              msg_err = SD_BAS_MSG_ALLOC;
              goto fin;
            }
          
          /* on se positionne a la position SD_BAS_OFFSET_MPHR */
          ret = fseeko(Descripteur->fd,SD_BAS_OFFSET_MPHR,SEEK_SET);
          if(ret == -1)
            {
              cr_op = SD_BAS_ERR_BAS_NIVEAU;
              num_lig = __LINE__;
              msg_err = SD_BAS_MSG_SEEK;
              goto fin;
            }	
          
          //   on recupere un bloc correspondant a une taille de SD_BAS_TAILLEMPHR dans le
          //   fichier prealablement ouvert
          nbElement = fread(fic,SD_BAS_TAILLE_MPHR,1,Descripteur->fd);
          if(nbElement != 1)
            {
              cr_op = SD_BAS_ERR_BAS_NIVEAU;
              num_lig = __LINE__;
              msg_err = SD_BAS_MSG_READ;
              goto fin;
            }
	   
          
          //   on recopie dans le nom du bloc SD_BAS_BLOC_MPHR
          strcpy(nom_bloc,SD_BAS_BLOC_MPHR);
          
          //   on appelle la couche de decodage afin de lire le contenu du GRH 
          //   DECOM__Decommuter ( fic SD_BAS_NOM_FIC_PROD bloc SD_BAS_BLOC_GRH ou SD_BAS_BLOC_PRODMOD
          
          status_d = SD_BAS_DecommuteGRH(&mphr->grh, fic);
          SD_BAS_TEST_DECOM(status_d,SD_BAS_ERR_DECOMMUTER,__LINE__);
          
          
#ifdef PROD_MOD_SANS_ENTETE_BINAIRE
        }
      
      // sinon
      else
        { 
          //   on alloue un tableau de char de taille SD_BAS_TAILLEPRODMOD
          if( SD_BAS_MacMALLOC(fic,char,SD_BAS_TAILLE_PRODMOD) == FALSE)
            {
              cr_op = SD_BAS_ERR_BAS_NIVEAU;
              num_lig = __LINE__;
              msg_err = SD_BAS_MSG_ALLOC;
              goto fin;
            }
          
          
          //   on recupere un bloc correspondant a une taille de SD_BAS_TAILLEPRODMOD dans le
          //   fichier prealablement ouvert
          nbElement = fread(fic,SD_BAS_TAILLE_PRODMOD,1,Descripteur->fd);
          if (nbElement != 1)
            {
              cr_op = SD_BAS_ERR_BAS_NIVEAU;
              num_lig = __LINE__;
              msg_err = SD_BAS_MSG_READ;
              goto fin;
            }
          
          
          //   on recopie dans le nom du bloc SD_BAS_BLOC_PRODMOD
          strcpy(nom_bloc,SD_BAS_BLOC_PRODMOD);
          lg = SD_BAS_TAILLE_PRODMOD;
          // finsi
        } 
#endif
      
      
      // on appelle la couche de decodage afin de lire le contenu du MPH ou prodmod
      // DECOM__Decommuter ( fic SD_BAS_NOM_FIC_PROD bloc SD_BAS_BLOC_MPHR ou SD_BAS_BLOC_PRODMOD
      // on recopie l'adresse retournee dans t_SD_BAS_Liste_Var
      //   et liberation des variables SD_BAS_LibereVariable
      
     status_d=DECOM__Decommuter(fic,lg,nom_bloc,"",& mphr->listevar);
     SD_BAS_TEST_DECOM(status_d,SD_BAS_ERR_DECOMMUTER,__LINE__);
       
     if( status_d != DECOM__C_ok)
       {
         DECOM__Liberer_Stockage_liste_var(&mphr->listevar);	 
         DECOM__Liberer_Stockage_liste_var(&mphr->grh);
       }
     
     SD_BAS_TEST_DECOM(status_d,SD_BAS_ERR_DECOMMUTER,__LINE__);
       

     // la description des bloc est laissee en memoire
     // on libere en memoire le tableau de char
     SD_BAS_MacFREE(fic);
     
     // on renseigne la valeur de la version a partir du champ FORMAT_VERSION 
     // dans la structure Descripteur
     Descripteur->version.major = *(mphr->listevar.tab_var[MPHR_FORMAT_MAJOR_VERSION].val.pi4);
     Descripteur->version.minor = *( mphr->listevar.tab_var[MPHR_FORMAT_MINOR_VERSION].val.pi4);
     
    }//fin si
  

 fin:
  
  // sortie de la fonction 
  //if (status != MO_OK) cr_op=CMN__STATUT_NOK;
  if (cr_op != CMN__STATUT_OK)
    {
      SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                          SD_BAS_ERR_SET_Produits,
                          SD_BAS_ERR_NUM_JDB,
                          tcProcName_F,
                          num_lig,
                          msg_err);
    }

  return (cr_op); 
}


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_EcrireMPHR
//@
//@ Description : fonction d'ecriture du MPHR d'un produit 
//@  cette operation generique s'applique aux produits IASI de N0 N1 et AVHRR
//@
//@
//@ Parameters :
//@			(ENTREE)  descripteur  : descripteur du produit
//@				  utilise par les fonctions d'acces
//@			(ENTREE)  mphr  : liste des variables a ecrire dans le MPHR
//@			(ENTREE)  grh  : liste des variables a ecrire dans le MPHR
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_EcrireMPHR(
		t_SD_BAS_Fd * Descripteur,
		t_SD_BAS_MPHR * mphr,
		t_SD_BAS_DataThread *ctx
			)
{
  /* nom de la fonction */
  static const char tcProcName_F[] = "SD_BAS_EcrireMPHR";
  /* retour de la fonction */
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  /* donnees lues dans le fichier:MPHR */
  char *fic = NULL;
  /* donnees lues dans le fichier:GRH */
  char * grh = NULL;
  /* retour de fonction */
  int ret = 0;
  /* elements lus ou ecrits */
  size_t nbElement = 0;
  /* longueur du bloc ecrit*/
  int lg = 0;
   /* retour des fonctions de commutation */
  t_SD_BAS_status_d status_d;
  /* message d'erreur pour le contexte d'erreur */
  char *msg_err = NULL;
  /* numero de ligne pour le contexte d'erreur */
  int num_lig = 0; 
  /* indice de boucle */
  int i = 0;  
  /* descripteur du fichier contenant les champs du MPHR */
  FILE *champ = NULL;
  /* variable de travail */
  char buffer;
  /* path des fichiers de configurations */
  char  fichierConfiguration[SD_BAS_NAMEPROD]; 
  /* repertoire des fichiers de configurations */
  char * repertoire = NULL; 



  /* debut de la fonction*/
  
  /* on verifie que l'etat est a INIT ou ENCOURS */
  /* et que le mode est ecriture ou en append */
  if ( (Descripteur->etat == INIT || Descripteur->etat == ENCOURS)
       &&( Descripteur->mode == ECRITURE || Descripteur->mode == APPEND || Descripteur->mode == APPEND_INIT))
    {

      /* le GRH a ete alloue lors de la lecture du product model donc on affecte directement les valeurs */

      /*  on initialise la classe du record (TR_MPHR) */
      *(mphr->grh.tab_var[GRH_RECORD_CLASS].val.pui4)=TR_MPHR;
      
      /* FA126 */
      /* on initialise l'instrument dans le GRH(IN_GENERIC)  */
      *(mphr->grh.tab_var[GRH_INSTRUMENT_GROUP].val.pui4) = IN_GENERIC;
      /* FIN FA126 */
      
      /* on initialise la sous classe du record (0) */
      *(mphr->grh.tab_var[GRH_RECORD_SUBCLASS].val.pui4)=0;
      
      /* on initialise la taille du record dans le GRH (SD_BAS_TAILLE_MPHR) */
      *(mphr->grh.tab_var[GRH_RECORD_SIZE].val.pui4)=SD_BAS_TAILLE_MPHR;

      
      /* on appelle la couche de decodage afin de commuter le contenu du MPH */
      /* DECOM__Commuter ( fic SD_BAS_NOM_FIC_PROD bloc SD_BAS_BLOC_MPHR) */
      status_d=DECOM__Recommuter (SD_BAS_BLOC_MPHR, mphr->listevar , &fic,&lg);
      /* si la commutation echoue on emet une erreur */
      SD_BAS_TEST_DECOM(status_d,SD_BAS_ERR_RECOMMUTER,__LINE__);
      
      
      /* on commute le GRH contenu dans le produit (fonction interne) */
      /* apres init des champs internes */
      /* DECOM__Commuter ( fic SD_BAS_NOM_FIC_PROD bloc SD_BAS_BLOC_GRH); */
      
      
      /* status_d=DECOM__Recommuter (SD_BAS_BLOC_GRH,mphr->grh, &grh,&lg); */
      /* SD_BAS_TEST_DECOM(status_d,SD_BAS_ERR_RECOMMUTER,__LINE__); */
      status_d=SD_BAS_CommuteGRH(mphr->grh,&grh);
      /* si la commutation echoue on emet une erreur */
      SD_BAS_TEST_DECOM(status_d,SD_BAS_ERR_RECOMMUTER,__LINE__);
      
      
      /* creation du path */
      /* on recupere le repertoire des fichiers de configurations */
      if((repertoire=getenv(REP_CONF_DIR))==NULL)
        {
          cr_op = SD_BAS_ERR_BAS_NIVEAU;
          num_lig = __LINE__;
          msg_err = SD_BAS_MSG_ENVIRONNEMENT;
          goto fin;
        }
      
      sprintf(fichierConfiguration, "%s%s%s", repertoire, SD_BAS_SEP_REPERTOIRE, SD_BAS_NOM_FIC_MPHR);
 
 
      /* ouverture du fichier decrivant les noms des champs ASCII */
      champ = fopen(fichierConfiguration, "rb");
      if (champ == NULL)
        {
          cr_op = SD_BAS_ERR_BAS_NIVEAU;
          num_lig = __LINE__;
          msg_err = SD_BAS_MSG_OPEN;
          goto fin;
        }


      /* on se positionne a la position SD_BAS_OFFSET_MPHR */
      ret = fseeko(Descripteur->fd,SD_BAS_OFFSET_MPHR,SEEK_SET);
      if(ret == -1)
        {
          cr_op = SD_BAS_ERR_BAS_NIVEAU;
          num_lig = __LINE__;
          msg_err = SD_BAS_MSG_SEEK;
          goto fin;
        }

      /* ajout des noms de champs au record MPHR */
      
      /* pour chaque caractere du MPHR */
      for(i=0; i<SD_BAS_TAILLE_MPHR; i++)
        {
          /* on lit le fichier de configuration */
          nbElement = fread(&buffer,sizeof(char), 1, champ);
          if(nbElement != 1)
            {
              cr_op = SD_BAS_ERR_BAS_NIVEAU;
              num_lig = __LINE__;
              msg_err = SD_BAS_MSG_READ;
              goto fin;
            }


          /* si different de 'x' */
          if (buffer != 'x')
            {
              /* on recopie le nom du champ */
              nbElement = fwrite(&buffer, sizeof(char), 1, Descripteur->fd);
              if(nbElement != 1)
                {
                  cr_op = SD_BAS_ERR_BAS_NIVEAU;
                  num_lig = __LINE__;
                  msg_err = SD_BAS_MSG_WRITE;
                  goto fin;
                }
            } 
          /* sinon */
          else
            {
              /* on recopie la valeur du champ */
              if (i<SD_BAS_TAILLE_GRH)
                {
                  /* depuis le bloc ascii du  GRH */
                  nbElement = fwrite(&grh[i], sizeof(char), 1, Descripteur->fd);
                }
              else
                {
                  /* ou depuis le bloc ascii du MPHR */
                  nbElement = fwrite(&fic[i], sizeof(char), 1, Descripteur->fd);
                }

              if(nbElement != 1)
                {
                  cr_op = SD_BAS_ERR_BAS_NIVEAU;
                  num_lig = __LINE__;
                  msg_err = SD_BAS_MSG_WRITE;
                  goto fin;
                }
            }

          /* fin pour */
        }

      
      /* fermeture du fichier decrivant les noms des champs ASCII */
      ret = fclose(champ);
      if(ret == EOF)
        {
          cr_op = SD_BAS_ERR_BAS_NIVEAU;
          num_lig = __LINE__;
          msg_err = SD_BAS_MSG_CLOSE;
          goto fin;
        }
      
      
      /* on libere le tableau de char de taille SD_BAS_TAILLEMPHR */
      SD_BAS_MacFREE(fic);

      /* on libere le tableau de char de taille SD_BAS_TAILLEGRH */
      SD_BAS_MacFREE(grh);
    }
  /* fin si */
  
 fin:
  
  /* si on detecte une erreur */
  if (cr_op != CMN__STATUT_OK)
    {
      /* on enregistre l'erreur */
      SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                          SD_BAS_ERR_SET_Produits,
                          SD_BAS_ERR_NUM_JDB,
                          tcProcName_F,
                          num_lig,
                          msg_err);
    }

  /* on retourne le status */ 
  return(cr_op); 
}


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LireIPR
//@
//@ Description : fonction de lecture des IPR d'un produit 
//@  cette operation generique s'applique aux produits IASI de N0 N1 et AVHRR
//@  on lit ici le gru et l'IPR
//@
//@
//@ Parameters :
//@			(ENTREE)  descripteur  : descripteur du produit
//@				  utilise par les fonctions d'acces
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LireIPR(
		t_SD_BAS_Fd * Descripteur,
		t_SD_BAS_DataThread *ctx
			)
{

  static const char tcProcName_F[] = "SD_BAS_LireIPR";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  char * fic = NULL;
  int ret = 0;
  size_t nbElement = 0;
  int lg = 0;
  t_SD_BAS_status_d status_d;
  int num_lig = 0;
  char *msg_err = NULL;
  t_SD_BAS_IPR ipr;
  int i = 0;
  t_CMN__Booleen first_giadr = CMN__VRAI;
  t_CMN__Booleen first_viadr = CMN__VRAI; 
  t_CMN__Booleen first_mdr = CMN__VRAI; 
  e_BAS_TypeRec type_ipr = TR_IPR;
  e_BAS_TypeRec type_pointe;
  off_t offset_ipr = 0;
  off_t taille_fic = 0;


  // on verifie que l'etat est a INIT ou ENCOURS 
  // et que le mode est lecture ou append
  if ( (Descripteur->etat == INIT || Descripteur->etat == ENCOURS)
       &&( Descripteur->mode == LECTURE || Descripteur->mode == APPEND))
  {
    switch(Descripteur->nature)
    {
      /* si produit de niveau 0 */
      case NA_IASIL0:
        offset_ipr = SD_BAS_OFFSET_IPR_N0;
      break;
      /* si produit avhrr */
      case NA_AVHRR:
        offset_ipr = SD_BAS_OFFSET_IPR_AVHRR;
      break;
      /* sinon produit de niveeau 1 */
      default:
        offset_ipr = SD_BAS_OFFSET_IPR_N1;
      break;
    }

    // on alloue un tableau de char de taille SD_BAS_TAILLE_IPR
    if( SD_BAS_MacMALLOC(fic,char,SD_BAS_TAILLE_IPR) == FALSE)
    {
      cr_op = SD_BAS_ERR_BAS_NIVEAU;
      num_lig = __LINE__;
      msg_err = SD_BAS_MSG_ALLOC;
      goto fin;
    }

    /* on recupere la taille du fichier */
    ret = fseeko(Descripteur->fd, 0, SEEK_END);
    if(ret == -1)
    {
      cr_op = SD_BAS_ERR_BAS_NIVEAU;
      num_lig = __LINE__;
      msg_err = SD_BAS_MSG_SEEK;
      goto fin;
    }
 
    taille_fic = ftello(Descripteur->fd);

    i=0;
    while(type_ipr == TR_IPR && offset_ipr < taille_fic)
    {
      // on recupere un bloc correspondant a une taille de SD_BAS_TAILLE_IPR dans le
      // fichier prealablement ouvert a l'offset SD_BAS_OFFSET_IPR
      ret=fseeko(Descripteur->fd, offset_ipr, SEEK_SET);
      if(ret == -1)
      {
        cr_op = SD_BAS_ERR_BAS_NIVEAU;
	num_lig = __LINE__;
	msg_err = SD_BAS_MSG_SEEK;
	goto fin;
      }
      
      nbElement = fread(fic, SD_BAS_TAILLE_IPR, 1, Descripteur->fd);
      if (nbElement != 1)
      {
        cr_op = SD_BAS_ERR_BAS_NIVEAU;
	num_lig = __LINE__;
	msg_err = SD_BAS_MSG_READ;
	goto fin;
      }
          
      // on appelle la couche de decodage afin de lire le contenu du GRH  (fonction interne)
      // DECOM__Decommuter ( bloc SD_BAS_BLOC_IPR fic SD_BAS_NOM_FIC_PROD)
      // on recopie l'adresse retournee dans iprviadr  
      status_d = SD_BAS_DecommuteGRH(&ipr.grh ,fic);
      SD_BAS_TEST_DECOM(status_d,SD_BAS_ERR_DECOMMUTER,__LINE__);   

      // on appelle la couche de decodage afin de lire le contenu du IPR 
      // DECOM__Decommuter ( bloc SD_BAS_BLOC_IPR fic SD_BAS_NOM_FIC_PROD)
      status_d=DECOM__Decommuter(fic,lg,SD_BAS_BLOC_IPR,"", &ipr.ipr);
      SD_BAS_TEST_DECOM(status_d,SD_BAS_ERR_DECOMMUTER,__LINE__);   
     
      type_ipr =  *(ipr.grh.tab_var[GRH_RECORD_CLASS].val.pui4);
      type_pointe = *( ipr.ipr.tab_var[IPR_TARGET_RECORD_CLASS].val.pui4);
      
      if (type_ipr == TR_IPR)
      {
        /* si l'IPR pointele premier GIADR */
	if( type_pointe == TR_GIADR && first_giadr == CMN__VRAI)
	{
	  /* on  initialise les offset dans la structure Descripteur */
	  Descripteur->offsetGAD = *( ipr.ipr.tab_var[IPR_TARGET_RECORD_OFFSET].val.pui4);
	  Descripteur->offsetGADSuivant = *( ipr.ipr.tab_var[IPR_TARGET_RECORD_OFFSET].val.pui4);
	  
	  /* on met a lour l'indicateur sur le premier GIADR */
	  first_giadr = CMN__FAUX;
	} 
 
        /* si l'IPR pointele premier VIADR */
        if( type_pointe == TR_VIADR && first_viadr == CMN__VRAI)
        {
          /* on  initialise les offset dans la structure Descripteur */
	  Descripteur->offsetVIADR = *( ipr.ipr.tab_var[IPR_TARGET_RECORD_OFFSET].val.pui4);
	  Descripteur->offsetVIADRSuivant = *( ipr.ipr.tab_var[IPR_TARGET_RECORD_OFFSET].val.pui4);
	  
	  /* on met a lour l'indicateur sur le premier VIADR */
	  first_viadr = CMN__FAUX;
        }
        
 // ANO?? : inclure if sur typerecord dans le 1er if ?
        /* si l'IPR pointele premier MDR */
	if( first_mdr == CMN__VRAI)
	{
	  /* on  initialise les offset dans la structure Descripteur */
	  Descripteur->offsetMDR = *( ipr.ipr.tab_var[IPR_TARGET_RECORD_OFFSET].val.pui4);
	  Descripteur->offsetMDRSuivant = *( ipr.ipr.tab_var[IPR_TARGET_RECORD_OFFSET].val.pui4);
	  
	  /* on met a lour l'indicateur sur le premier MDR */
	  if( type_pointe == TR_MDR)
	  {
	    first_mdr = CMN__FAUX;
	  }
	} /* fin si first_mdr == CMN__VRAI */
//..ANO ??
      } /* fin si type_ipr == TR_IPR */
      
      /* on passe a l'IPR suivant */
      i++;
      
      /* on incremente offset IPR */
      offset_ipr =  offset_ipr + SD_BAS_TAILLE_IPR;

      /* libere l' IPR precedent */
      DECOM__Liberer_Stockage_liste_var(&ipr.ipr);
      DECOM__Liberer_Stockage_liste_var(&ipr.grh);

    } /* fin while */
    // on libere en memoire le tableau de char
    SD_BAS_MacFREE(fic);
  }//fin si

 fin:

  // sortie de la fonction 
  if (cr_op != CMN__STATUT_OK)
   {
     SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                         SD_BAS_ERR_SET_Produits,
                         SD_BAS_ERR_NUM_JDB,
                         tcProcName_F,
                         num_lig,
                         msg_err);
   } 
   
 return(cr_op); 
}

/* FA 162 */
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LireIPRDump
//@
//@ Description : fonction de lecture des IPR d'un produit 
//@  cette operation generique s'applique aux produits IASI de N0 N1 et AVHRR
//@  on lit ici le gru et l'IPR. Meme fonction que lireIPR sans mise a jour des offsets 
//@
//@
//@ Parameters :
//@			(ENTREE)  descripteur  : descripteur du produit
//@				  utilise par les fonctions d'acces
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LireIPRDump(
		t_SD_BAS_Fd * Descripteur,
		t_SD_BAS_DataThread *ctx
			)
{

  static const char tcProcName_F[] = "SD_BAS_LireIPRDump";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  char * fic = NULL;
  int ret = 0;
  size_t nbElement = 0;
  int lg = 0;
  t_SD_BAS_status_d status_d;
  int num_lig = 0;
  char *msg_err = NULL;
  t_SD_BAS_IPR ipr;
  int i = 0;
  e_BAS_TypeRec type_ipr = TR_IPR;
  e_BAS_TypeRec type_pointe;
  off_t offset_ipr = 0;
  off_t taille_fic = 0;


  // on verifie que l'etat est a INIT ou ENCOURS 
  // et que le mode est lecture ou append
  if ( (Descripteur->etat == INIT || Descripteur->etat == ENCOURS)
       &&( Descripteur->mode == LECTURE || Descripteur->mode == APPEND))
  {
    switch(Descripteur->nature)
    {
      /* si produit de niveau 0 */
      case NA_IASIL0:
        offset_ipr = SD_BAS_OFFSET_IPR_N0;
      break;
      /* si produit avhrr */
      case NA_AVHRR:
        offset_ipr = SD_BAS_OFFSET_IPR_AVHRR;
      break;
      /* sinon produit de niveeau 1 */
      default:
        offset_ipr = SD_BAS_OFFSET_IPR_N1;
      break;
    }

    // on alloue un tableau de char de taille SD_BAS_TAILLE_IPR
    if( SD_BAS_MacMALLOC(fic,char,SD_BAS_TAILLE_IPR) == FALSE)
    {
      cr_op = SD_BAS_ERR_BAS_NIVEAU;
      num_lig = __LINE__;
      msg_err = SD_BAS_MSG_ALLOC;
      goto fin;
    }

    /* on recupere la taille du fichier */
    ret = fseeko(Descripteur->fd, 0, SEEK_END);
    if(ret == -1)
    {
      cr_op = SD_BAS_ERR_BAS_NIVEAU;
      num_lig = __LINE__;
      msg_err = SD_BAS_MSG_SEEK;
      goto fin;
    }
 
    taille_fic = ftello(Descripteur->fd);

    i=0;
    while(type_ipr == TR_IPR && offset_ipr < taille_fic)
    {
      // on recupere un bloc correspondant a une taille de SD_BAS_TAILLE_IPR dans le
      // fichier prealablement ouvert a l'offset SD_BAS_OFFSET_IPR
      ret=fseeko(Descripteur->fd, offset_ipr, SEEK_SET);
      if(ret == -1)
      {
        cr_op = SD_BAS_ERR_BAS_NIVEAU;
	num_lig = __LINE__;
	msg_err = SD_BAS_MSG_SEEK;
	goto fin;
      }
      
      nbElement = fread(fic, SD_BAS_TAILLE_IPR, 1, Descripteur->fd);
      if (nbElement != 1)
      { 
	cr_op = SD_BAS_ERR_BAS_NIVEAU;
	num_lig = __LINE__;
	msg_err = SD_BAS_MSG_READ;
	goto fin;
      }
          
      // on appelle la couche de decodage afin de lire le contenu du GRH  (fonction interne)
      // DECOM__Decommuter ( bloc SD_BAS_BLOC_IPR fic SD_BAS_NOM_FIC_PROD)
      // on recopie l'adresse retournee dans iprviadr  
      status_d = SD_BAS_DecommuteGRH(&ipr.grh ,fic);
      SD_BAS_TEST_DECOM(status_d,SD_BAS_ERR_DECOMMUTER,__LINE__);   

      // on appelle la couche de decodage afin de lire le contenu du IPR 
      // DECOM__Decommuter ( bloc SD_BAS_BLOC_IPR fic SD_BAS_NOM_FIC_PROD)
      status_d=DECOM__Decommuter(fic,lg,SD_BAS_BLOC_IPR,"", &ipr.ipr);
      SD_BAS_TEST_DECOM(status_d,SD_BAS_ERR_DECOMMUTER,__LINE__);   
          
      type_ipr =  *(ipr.grh.tab_var[GRH_RECORD_CLASS].val.pui4);

      /* mettre a jour la structure IPR dans descripteur fichier */
      Descripteur->descripteurIPR[i].Rec = *( ipr.ipr.tab_var[IPR_TARGET_RECORD_CLASS].val.pui4);
      Descripteur->descripteurIPR[i].Instrument = *( ipr.ipr.tab_var[IPR_TARGET_INSTRUMENT_GROUP].val.pui4);
      Descripteur->descripteurIPR[i].subclass =  *( ipr.ipr.tab_var[IPR_TARGET_RECORD_SUBCLASS].val.pui4);     
      Descripteur->descripteurIPR[i].Offset = *( ipr.ipr.tab_var[IPR_TARGET_RECORD_OFFSET].val.pui4);
      
#ifdef OPS_DEBUG
      // FT-2765
      printf( "IPR lu dans LireIPRDump :\n" );
      printf( " descripteurIPR[%d].Rec = %d\n", i, Descripteur->descripteurIPR[i].Rec );
      printf( " descripteurIPR[%d].Instrument = %d\n", i, Descripteur->descripteurIPR[i].Instrument );
      printf( " descripteurIPR[%d].subclass = %d\n", i, Descripteur->descripteurIPR[i].subclass );
      printf( " descripteurIPR[%d].Offset = %d\n", i, Descripteur->descripteurIPR[i].Offset );
      // ...FT-2765
#endif
      /* on passe a l'IPR suivant */
      i++;
      
      /* on incremente offset IPR */
      offset_ipr =  offset_ipr + SD_BAS_TAILLE_IPR;

      /* libere l' IPR precedent */
      DECOM__Liberer_Stockage_liste_var(&ipr.ipr);
      DECOM__Liberer_Stockage_liste_var(&ipr.grh);

    } /* fin while */
    // on libere en memoire le tableau de char
    SD_BAS_MacFREE(fic);
  }//fin si

 fin:

  // sortie de la fonction 
  if (cr_op != CMN__STATUT_OK)
   {
     SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                         SD_BAS_ERR_SET_Produits,
                         SD_BAS_ERR_NUM_JDB,
                         tcProcName_F,
                         num_lig,
                         msg_err);
   } 
   
 return(cr_op); 
}

/* ..FA 162 */

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_EcrireIPR
//@
//@ Description : fonction d'ecriture de l'IPR d'un produit 
//@  cette operation generique s'applique aux produits IASI de N0 N1 et AVHRR
//@
//@
//@ Parameters :
//@			(ENTREE/SORTIE)  descripteur  : descripteur du produit
//@				  utilise par les fonctions d'acces
//@			(ENTREE)   type  : type de record pointe
//@   (ENTREE)   instrument_group : valeur du champ 
//@                                 instrument_group du record pointe
//@			(ENTREE)   subclass : subclass du record pointe
//@			(ENTREE)   offest   : offset du record pointe
//@			(ENTREE)   start  : date de debut du produit
//@			(ENTREE)   stop  : date de fin du produit
//@			(ENTREE)   num   : position de l'IPR
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_EcrireIPR(
		t_SD_BAS_Fd * Descripteur,
		unsigned int type,
  unsigned int instrument_group,
		unsigned int subclass,
		unsigned int offset,
		t_SD_BAS_SCDSTime start,
		t_SD_BAS_SCDSTime stop,
		int num,
		t_SD_BAS_DataThread *ctx)
  
{
  /* nom de la fonction */
  static const char tcProcName_F[] = "SD_BAS_EcrireIPR";
  /* retour de la fonction */
  t_CMN__Statut cr_op=CMN__STATUT_OK; 
  /* donnees a lire dans le fichier */
  char *bloc_ipr = NULL;
  /* donnees a lire dans le fichier */
  char *grh = NULL;
  /* longueur des donnees ecrites */
  int lg = 0;                    
  /* retour des fonctions de commutation */
  t_SD_BAS_status_d status_d;
  /* retour de fonction */
  int ret = 0;
  /* elements lus ou ecrits */
  size_t nbElement = 0;
  /* tableau des elements a ajouter aux listevars */
  DECOM__T_val_var   valeur[SD_BAS_NB_GRH_FIELD];
  DECOM__T_val_var   valeur_ipr[SD_BAS_NB_IPR_FIELD];
  DECOM__T_val_var   valeur_date[SD_BAS_NB_DATE];
  /* indice de boucle */
  int i = 0;          
  /* message d'erreur pour le contexte d'erreur */
  char *msg_err = NULL;  
  /* numero de ligne pour le contexte d'erreur */
  int num_lig = 0;  
  /* ipr a ecrire */
  t_SD_BAS_IPR ipr;
  


/*debut de la fonction*/



// on verifie que l'etat est a INIT ou ENCOURS 
// et que le mode est ecriture ou en append
 if ( (Descripteur->etat == INIT || Descripteur->etat == ENCOURS)
      &&( Descripteur->mode == ECRITURE || Descripteur->mode == APPEND|| Descripteur->mode == APPEND_INIT))
   {

     // on alloue les valeurs a ajouter a l'IPR
     
     for(i=0;i<SD_BAS_NB_GRH_FIELD;i++)
       {
	 if(SD_BAS_MacMALLOC(valeur[i].pui4,unsigned int,1)==FALSE)
	   {
	     cr_op = SD_BAS_ERR_BAS_NIVEAU;
	     num_lig = __LINE__;
	     msg_err = SD_BAS_MSG_ALLOC;
	     goto fin;
	   }
       }
     

     // on alloue les dates a ajouter au GRH
     
     for(i=0;i<SD_BAS_NB_DATE;i++)
       {
	 if(SD_BAS_MacMALLOC(valeur_date[i].pdate, t_SD_BAS_SCDSTime, 1)==FALSE)
	   {
	     cr_op = SD_BAS_ERR_BAS_NIVEAU;
	     num_lig = __LINE__;
	     msg_err = SD_BAS_MSG_ALLOC;
	     goto fin;
	   }
       }


     // on remplit les champ du GRH manquant ( par des info standard )


     ipr.grh.nb_var=0;
     ipr.grh.tab_var=NULL;

     

     // on initialise la classe du record (TR_IPR)   
     *(valeur[0].pui4)=TR_IPR;
     /* on l'ajoute a la listevar */
     status_d=DECOM__Ajouter_variable (&ipr.grh,"","GRH_RECORD_CLASS",
				       DECOM__C_uint4,1,valeur[0]);
     /* si l'ajout echoue on emet une erreur */
     SD_BAS_TEST_DECOM(status_d,SD_BAS_ERR_AJOUTER,__LINE__);

     // on initialise l'instrument dans le GRH(IN_IASI)  
     // correction IN_GENERIC
     * (valeur[1].pui4)=IN_GENERIC;
     /* on l'ajoute a la listevar */
     status_d=DECOM__Ajouter_variable (&ipr.grh,"","GRH_INSTRUMENT_GROUP",
				       DECOM__C_uint4,1,valeur[1]);
     /* si l'ajout echoue on emet une erreur */
     SD_BAS_TEST_DECOM(status_d,SD_BAS_ERR_AJOUTER,__LINE__);

     // on initialise la sous classe du record (0)
     *(valeur[2].pui4)=0;
     /* on l'ajoute a la listevar */
     status_d=DECOM__Ajouter_variable (&ipr.grh,"","GRH_RECORD_SUBCLASS",
				   DECOM__C_uint4,1,valeur[2]);
     /* si l'ajout echoue on emet une erreur */
     SD_BAS_TEST_DECOM(status_d,SD_BAS_ERR_AJOUTER,__LINE__);

     /* on initialise la version  du record */
     *(valeur[3].pui4) = SD_BAS_IPR_VERSION ;
     /* on l'ajoute a la listevar */
     status_d=DECOM__Ajouter_variable (&ipr.grh,"","GRH_RECORD_SUBCLASS_VERSION",
				       DECOM__C_uint4,1,valeur[3]);
     SD_BAS_TEST_DECOM(status_d,SD_BAS_ERR_AJOUTER,__LINE__);


// on initialise la taille du record dans le GRH (SD_BAS_TAILLE_IPR)
     *(valeur[4].pi4)=SD_BAS_TAILLE_IPR;
 /* on l'ajoute a la listevar */
     status_d=DECOM__Ajouter_variable (&ipr.grh,"","GRH_RECORD_SIZE",
				       DECOM__C_uint4,1,valeur[4]); 

// on inialise le start
     valeur_date[0].pdate->day = start.day;
     valeur_date[0].pdate->millis = start.millis;
     status_d=DECOM__Ajouter_variable (&ipr.grh,"","GRH_RECORD_START_TIME",
                                       DECOM__C_date,1,valeur_date[0]); 
     SD_BAS_TEST_DECOM(status_d,SD_BAS_ERR_AJOUTER,__LINE__);
     
     // on inialise le stop
     valeur_date[1].pdate->day = stop.day;
     valeur_date[1].pdate->millis = stop.millis;
      status_d=DECOM__Ajouter_variable (&ipr.grh,"","GRH_RECORD_STOP_TIME",
                                       DECOM__C_date,1,valeur_date[1]); 
     SD_BAS_TEST_DECOM(status_d,SD_BAS_ERR_AJOUTER,__LINE__);


  //on initialise les IPR
     ipr.ipr.nb_var=0;
     ipr.ipr.tab_var=NULL;


     for(i=0;i<SD_BAS_NB_IPR_FIELD;i++)
       {
	 if(SD_BAS_MacMALLOC(valeur_ipr[i].pui4,unsigned int,1)==FALSE)
           {
	     cr_op = SD_BAS_ERR_BAS_NIVEAU;
	     msg_err = SD_BAS_MSG_ALLOC;
	     num_lig = __LINE__;
	     goto fin;
           }
       }


 // TYPE pointe par l'IPR
     *(valeur_ipr[0].pui4) = type ;
     status_d=DECOM__Ajouter_variable (&ipr.ipr,"",
                                       "IPR_TARGET_RECORD_CLASS",
                                       DECOM__C_uint4,1,valeur_ipr[0]);
     SD_BAS_TEST_DECOM(status_d,SD_BAS_ERR_AJOUTER,__LINE__);

     // RECORD SUBCLASS de l'objet pointe 
     *(valeur_ipr[1].pui4) = subclass ;
     status_d=DECOM__Ajouter_variable (&ipr.ipr,"",
                                       "IPR_TARGET_RECORD_SUBCLASS",
                                       DECOM__C_uint4,1,valeur_ipr[1]);
     SD_BAS_TEST_DECOM(status_d,SD_BAS_ERR_AJOUTER,__LINE__);

     // instrument 
     *(valeur_ipr[2].pui4) = instrument_group ;
     status_d=DECOM__Ajouter_variable (&ipr.ipr,"",
                                       "IPR_TARGET_INSTRUMENT_GROUP",
                                       DECOM__C_uint4,1,valeur_ipr[2]);
     SD_BAS_TEST_DECOM(status_d,SD_BAS_ERR_AJOUTER,__LINE__);

     // offest du premier GAD
     *(valeur_ipr[3].pui4) = offset;
     status_d=DECOM__Ajouter_variable (&ipr.ipr,"",
                                       "IPR_TARGET_RECORD_OFFSET",
                                       DECOM__C_uint4,1,valeur_ipr[3]);
     SD_BAS_TEST_DECOM(status_d,SD_BAS_ERR_AJOUTER,__LINE__);




// on appelle la couche de decodage afin de commuter le contenu du  iprgiadr
// ( initialise a vide )
// DECOM__Commuter ( fic SD_BAS_NOM_FIC_PROD bloc IPR)
     status_d=DECOM__Recommuter (SD_BAS_BLOC_IPR,ipr.ipr ,&bloc_ipr,&lg);
     SD_BAS_TEST_DECOM(status_d,SD_BAS_ERR_RECOMMUTER,__LINE__); 


// on commute le GRH contenu dans le produit (fonction interne)
// DECOM__Commuter ( fic SD_BAS_NOM_FIC_PROD bloc SD_BAS_BLOC_GRH)


     status_d = SD_BAS_CommuteGRH(ipr.grh,&grh);
     /* si la commutation echoue on emet une erreur */
     SD_BAS_TEST_DECOM(status_d,SD_BAS_ERR_DECOMMUTER,__LINE__);   


     /* pour chaque element ajoutes a la listevar du GRH */
     for(i=0;i<SD_BAS_NB_GRH_FIELD;i++)
       /* on libere l'element */ 
       SD_BAS_MacFREE(valeur[i].pui4);
     
     //on libere les valeurs ajoutees via ecrire IPR
     for(i=0;i<SD_BAS_NB_IPR_FIELD;i++)
       SD_BAS_MacFREE(valeur_ipr[i].pui4);
     
     /*on libere les dates ajoutees dans le GRH */
     for(i=0;i<SD_BAS_NB_DATE;i++)
       SD_BAS_MacFREE(valeur_date[i].pdate);
     
     /* on libere les listes var */
     DECOM__Liberer_liste_var(&ipr.grh);
     DECOM__Liberer_liste_var(&ipr.ipr);


     // on ecrit dans le fichier le buffer
     
     // on se place a l'offset du premier IPR
     ret=fseeko(Descripteur->fd, SD_BAS_OFFSET_IPR_N1 + num * SD_BAS_TAILLE_IPR, SEEK_SET);
     if(ret == -1)
       {
	 cr_op = SD_BAS_ERR_BAS_NIVEAU;
	 num_lig = __LINE__;
	 msg_err = SD_BAS_MSG_SEEK;
	 goto fin;
       }
     

     // on ecrit l'IPR dans le fichier
     nbElement = fwrite(bloc_ipr, SD_BAS_TAILLE_IPR, 1, Descripteur->fd);
     if(nbElement != 1)
       {
	 cr_op = SD_BAS_ERR_BAS_NIVEAU;
	 num_lig = __LINE__;
	 msg_err = SD_BAS_MSG_WRITE;
	 goto fin;
       }
     
     // on se place a l'offset du premier IPR
     ret=fseeko(Descripteur->fd, SD_BAS_OFFSET_IPR_N1 + num * SD_BAS_TAILLE_IPR, SEEK_SET);
     if(ret == -1)
       {
	 cr_op = SD_BAS_ERR_BAS_NIVEAU;
	 num_lig = __LINE__;
	 msg_err = SD_BAS_MSG_SEEK;
	 goto fin;
       }
     
     // on ecrit le GRH au debut de L'IPR
     nbElement = fwrite(grh, SD_BAS_TAILLE_GRH, 1, Descripteur->fd);
     if (nbElement != 1)
       {
	 cr_op = SD_BAS_ERR_BAS_NIVEAU;
	 num_lig = __LINE__;
	 msg_err = SD_BAS_MSG_WRITE;
         goto fin;
       }
     
     Descripteur->offsetGAD = SD_BAS_OFFSET_IPR_N1 + (num+1) * SD_BAS_TAILLE_IPR;
     Descripteur->offsetGADSuivant = Descripteur->offsetGAD;

     /* on libere le buffer de l IPR ecrit */
     SD_BAS_MacFREE(bloc_ipr);
     /* on libere le buffer du GRH ecrit */
     SD_BAS_MacFREE(grh);

   }
//fin si


fin:

// sortie de la fonction 
//if (status != MO_OK) cr_op=CMN__STATUT_NOK;
 if ( cr_op != CMN__STATUT_OK )
   {
     
/* on enregistre l'erreur */
     SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                         SD_BAS_ERR_SET_Produits,
                         SD_BAS_ERR_NUM_JDB,
                         tcProcName_F,
                         num_lig,
                         msg_err);
     
     
   }
/* on retourne le status */
 return(cr_op); 

}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LibereListeVar
//@
//@ Description : fonction de liberation d'une liste de variables decommutees
//@ recommutees
//@  cette operation generique s'applique aux produits IASI de N0 N1 et AVHRR
//@
//@
//@ Parameters :
//@			(ENTREE/SORTIE)  mphr  : liste des variables lues dans le MPHR
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
 t_CMN__Statut SD_BAS_LibereListeVar(
				     e_BAS_TypeAcces mode,
				     t_SD_BAS_Liste_Var * listeVar
				    )
{
t_CMN__Statut cr_op=CMN__STATUT_OK;


// attention dans certains cas (ecriture ou append) il faut appeler 
//  DECOM__Liberer_liste_var pour liberer la liste en entree ( liberation
//							superficielle)
// if( mode == ECRITURE || mode == APPEND)
//   DECOM__Liberer_liste_var(listeVar);
 
// attention dans d'autres cas (lecture)  il faut appeler 
//  DECOM__Liberer_Stockage_liste_var pour liberer la liste en entree ( liberation
//							profonde)
// else if( mode == LECTURE )
   DECOM__Liberer_Stockage_liste_var(listeVar);


//if (status != MO_OK) cr_op=CMN__STATUT_NOK;
return(cr_op); 

}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_CommuteGRH
//@
//@ Description : cette fonction permet de commuter le GRH d'un record (20)
//@
//@
//@ Parameters : 
//@			(ENTREE)  grh : grh decode
//@			(ENTREE/SORTIE)  buffer : buffer utilise pour le commutation
//@                     
//@
//@ Return Status : 
//@		t_SD_BAS_status_d status	 
//@		
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_CommuteGRH(
                                t_SD_BAS_Liste_Var  grh,
                                char ** buffer
			)
   {
 t_SD_BAS_status_d status;
 int lg = 0;
// on appelle la couche de codage afin de commuter le contenu du GRH 
// DECOM__Commuter ( fic SD_BAS_NOM_FIC_PROD bloc SD_BAS_BLOC_GRH)
// dans le buffer 
 
 status=DECOM__Recommuter (SD_BAS_BLOC_GRH, grh, buffer,&lg);
 
   

FIN:
// sortie de la fonction 
return(status); 

}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_DecommuteGRH
//@
//@ Description : cette fonction permet de decommuter le GRH d'un record (20)
//@
//@
//@ Parameters : 
//@			(ENTREE)  buffer : buffer issu de la lecture
//@			(SORTIE)  grh : grh decode
//@                     
//@
//@ Return Status : 
//@			 
//@		t_SD_BAS_status_d status
//@
//@ ------------------------------------------------------------------*/
t_SD_BAS_status_d SD_BAS_DecommuteGRH( /* CR301 */
                                  t_SD_BAS_Liste_Var  *grh,
                                  char * buffer
                                  )
{
  int lg = 0;
  t_SD_BAS_status_d status;



// on appelle la couche de decodage afin de decommuter le contenu du GPH 
// DECOM__Decommuter ( fic SD_BAS_NOM_FIC_PROD bloc SD_BAS_BLOC_GRH)
// dans le buffer ( octets a a 20)

  status = DECOM__Decommuter(buffer,lg ,SD_BAS_BLOC_GRH,"",grh);
 

 FIN:
  // sortie de la fonction 
  return(status); 

}


//======================================================================
//					FONCTIONS INTERNES  
//======================================================================


