//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/BAS/src/SD_BAS_Produits.h,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Component : SD_BAS
//@ MODULE : SD_BAS_Produits
//@
//@ Comment :
//@    include file used by all product used generated by OPS
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 04/09/2003 : [reference :] Benjamin MASSART
//@ cette version prend en compte les evolutions du code pour une gestion
//@ dynamique de l'ecriture des IPR dans les produits de niveau 1 suite
//@ aux modifications apparues dans la version 6.4 du GPFS et la version
//@ 6.5 du IASI Level 1 Product Format Specification
//@ --------------------------------------------------------------------
//@ DM-ID : PM-3 : 04/09/2003 :
//@ -ajout de la structure t_SD_BAS_Produit
//@ -ajout de la variable descripteurIPR dans la structure t_SD_BAS_Fd
//@ -ajout de SD_BAS_OFFSET_IPR_N1 et SD_BAS_TAILLE_IPR
//@ -ajout du parametre instrument_group dans l'appel a la fonction 
//@  SD_BAS_EcrireIPR()
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-146-CN : 10/11/2004 : B.MASSART
//@ Mauvaise orthographe de quelques champs du MPHR. Voir IA-2100-FC-146-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-271-CN : 22/06/2005 : JL.PASCAL
//@ prise en compte du mode transition
//@ fiche de correction correspondante : IA-2100-CR-271-THA
//@ --------------------------------------------------------------------
//@ DM-ID : IA-2100-CR-292-CNE : 10/03/2006 : B.MASSART
//@ Function SD_ALG_AVHRR_GetSubSatLatLon is added to set up 
//@ MPHR_SUBSAT_LATITUDE/LONGITUDE_START/END fields in Level1 products.
//@ See IA-2100-CR-292-THA.
//@ --------------------------------------------------------------------
//@ PM-ID :IA-2100-CR-301-CN : 03/07/2006 : B. MASSART
//@ SD_BAS_DecommuteGRH return status corrected. (patch3-5-5). 
//@ See IA-2100-CR-301-THA.
//@ --------------------------------------------------------------------
//@ MODIFIED FOR PATCH3-5-5 : 05/Jul/06
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-162-CN : 09/2006 : C.MERIGUET
//@ Gestion des IPR en mode DUMP
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------



#ifndef SD_BAS_Produits_h
#define SD_BAS_Produits_h


/* librairies C de bases utilisees par les modules du projet */
#include <stdio.h>       /* entree/sortie */
#include <stdlib.h>      /* alloc memoire, ...*/
#include <string.h>      /* chaines de carateres */
#include <unistd.h>      /* gestion pipes, ... */
#include <time.h>        /* gestion de la date */
#include <sys/param.h>   /* constantes systeme */

// inserer ici les fichiers include 

#include "SD_BAS_Global.h"
#include "ERR__Erreur.h"

// a supprimer quant SD_BAS_DECOM est pret
#include "SD_BAS_DECOM_DECOMMUTATION.h"



//*====================================================================*/
//*   External define  Declarations
//*====================================================================*/
#define SD_BAS_NAMEPROD 1024     //* longueur  max des champs ASCII */
#define SD_BAS_LG_NOM_BLOC  30  //* longueur  max des noms de blocs*/

#define ERR_BASE_PRODUITS       SD_BAS_ERR_BASE_Produits 
#define SD_BAS_ERR_DECOMMUTER    (ERR_BASE_PRODUITS +1)
#define SD_BAS_ERR_RECOMMUTER    (ERR_BASE_PRODUITS +2)
#define SD_BAS_ERR_AJOUTER       (ERR_BASE_PRODUITS +3) 
#define SD_BAS_ERR_ALLOC         (ERR_BASE_PRODUITS +4)
#define SD_BAS_ERR_ACCES_FIC     (ERR_BASE_PRODUITS +5)
#define SD_BAS_ERR_CHARGER       (ERR_BASE_PRODUITS +6)
#define SD_BAS_ERR_REMOVE        (ERR_BASE_PRODUITS +7)
#define SD_BAS_ERR_MISSING_FILE  (ERR_BASE_PRODUITS +8)

// FT-2765
static const int SD_BAS_ERR_SET_Produits =  1;
static const int SD_BAS_TAILLE_GRH = 20;      		//* taille du GHR */
// ...FT-2765


/* Macro testant les codes de retour des fonctions de DECOM */
#define SD_BAS_TEST_DECOM(status,cr,lg)\
{if(status != DECOM__C_ok)\
{ cr_op =cr;\
  num_lig = lg;\
  msg_err= DECOM__T_message[status];\
  goto fin;\
} }
//====================================================================*/
// type definissant les tableaux retounes par les fonctions de
// decommutation
//====================================================================*/
//  
// a supprimer quant SD_BAS_Decommutation est pret

typedef DECOM__T_liste_var t_SD_BAS_Liste_Var;
typedef DECOM__T_status    t_SD_BAS_status_d;
//====================================================================*/
// Exported types Declaration  
//====================================================================*/

/* Nom complet (path+nom) d'un fichier produit*/
typedef char t_SD_BAS_NomProduit[SD_BAS_LGNOMFICHIER+1];

// type definissant le type de record
typedef enum {
	TR_UNDEFINED=0,
	TR_MPHR=1,
	TR_SPHR=2,
	TR_IPR=3,
	TR_GEADR=4,		
	TR_GIADR=5,		
	TR_VEADR=6,		
	TR_VIADR=7,		
	TR_MDR=8		
	} e_BAS_TypeRec; 

// type definissant la validite d'un record associee a une ligne 
typedef enum 
	{
	VALIDE=1,
	INVALIDE=2,
	TRANSITION=3, /* CR271*/
	INCOHERENT=4
	} e_BAS_ValidRec; 
	
// type definissant le contexte d'initialisation
typedef enum 
	{
	CTX_NORMAL=1,
	CTX_INIT=2
	} e_BAS_CtxInit; 


// type definissant les differents type d'instrument 
typedef enum 
	{
	IN_GENERIC=0,
	IN_IASI=8,
	IN_AVHRR=4,
	IN_DUMMY=13
	} e_BAS_INSTRUMENT; 

// type definissant les le nom des produits dans le MPHR
typedef char t_BAS_NAMEPROD[SD_BAS_NAMEPROD]; 

// type definissant les differentes natures 
typedef enum 
	{
	NA_IASIL0=0,
	NA_IASIL1A=1,
	NA_IASIL1B=2,
	NA_IASIL1C=3,
	NA_IASIL1ENG=4,
	NA_IASIL1VER=5,
	NA_AVHRR=6,
	NA_PRODMODEL=7
	} e_BAS_Nature; 
//--------------------------------------------------------------------*/
// type definissant le type de record 
//--------------------------------------------------------------------*/
typedef enum 
	{
	PT_ENG=0,
	PT_VER=1,
	PT_xxx=2
	} e_BAS_PRODTYPE; 

typedef enum 
	{
	PL_00=0,
	PL_1A=1,
	PL_1B=2,
	PL_1C=3,
	PL_02=4
	} e_BAS_PRODLEVEL; 

typedef enum 
	{
	PM_NRT=0,
	PM_BACKLOG=1,
	PM_REPROCESSING=2
	} e_BAS_PROCMODE; 

typedef enum 
	{
	OPERATIONNAL=0,
	VALIDATION=1,
	COMMISSIONING=2
	} e_BAS_DISPMODE; 


typedef enum
         {
	   CHARGE=1,
	   NON_CHARGE=2 
	 
         } e_BAS_Desc;

//--------------------------------------------------------------------*/
// types definissant le DMDR
//--------------------------------------------------------------------*/
// attention l'ordre suivant doit correspondre a l'ordre dans le fichier
// ASCII de decommutation
typedef enum {
  DMDR_DATEDEB,
  DMDR_DATEFIN,
  DMDR_LOST_DATA_FLAG
} e_BAS_IND_DMDR;


//--------------------------------------------------------------------*/
// types definissant le generic record header
//--------------------------------------------------------------------*/
// attention l'ordre suivant doit correspondre a l'ordre dans le fichier
// ASCII de decommutation
typedef enum {
		GRH_RECORD_CLASS=0,
		GRH_INSTRUMENT_GROUP=1,
		GRH_RECORD_SUBCLASS=2,
		GRH_RECORD_SUBCLASS_VERSION=3,
		GRH_RECORD_SIZE=4,
		GRH_RECORD_START_TIME=5,
		GRH_RECORD_STOP_TIME=6
		} e_SD_BAS_IND_GRH;
//typedef struct
//	{
//	e_BAS_TypeRec record_class;
//	e_BAS_INSTRUMENT instrument_group;
//	unsigned int record_subclass;
//	unsigned int record_subclass_version;
//	unsigned int record_size;
//	t_SD_BAS_SCDSTime record_start_time;
//	t_SD_BAS_SCDSTime record_stop_time;
//	} t_SD_BAS_GRH;

//--------------------------------------------------------------------*/
// structure contenant les informations d'un IPR
//--------------------------------------------------------------------*/
typedef struct
{  
  e_BAS_TypeRec    Rec;        /* champ TARGET_RECORD_CLASS     */
  e_BAS_INSTRUMENT Instrument; /* champ TARGET_INSTRUMENT_GROUP */
  unsigned int     subclass;   /* champ TARGET_RECORD_SUBCLASS  */
  unsigned int     Offset;     /* champ TARGET_RECORD_OFFSET    */
}t_SD_BAS_Produit;

//--------------------------------------------------------------------*/
// types definissant le generic record pointer
//--------------------------------------------------------------------*/
// attention l'ordre suivant doit correspondre a l'ordre dans le fichier
// ASCII de decommutation
typedef enum {
		GRP_TARGET_RECORD_CLASS=0,
		GRP_TARGET_INSTRUMENT_GROUP=1,
		GRP_TARGET_RECORD_SUBCLASS=2,
		GRP_TARGET_RECORD_OFFSET=3
		} e_SD_BAS_IND_GRP;
//typedef struct
//	{
//	e_BAS_TypeRec target_record_class;
//	e_BAS_INSTRUMENT target_instrument_group;
//	unsigned int target_record_subclass;
//	unsigned int target_record_offset;
//	} t_SD_BAS_GRP;
	
//--------------------------------------------------------------------*/
// types definissant les IPR
//--------------------------------------------------------------------*/
// attention l'ordre suivant doit correspondre a l'ordre dans le fichier
// ASCII de decommutation
typedef enum {
  IPR_TARGET_RECORD_CLASS=0,
  IPR_TARGET_INSTRUMENT_GROUP=1,
  IPR_TARGET_RECORD_SUBCLASS=2,
  IPR_TARGET_RECORD_OFFSET=3
  
} e_SD_BAS_IND_IPR;

typedef struct
	{
	t_SD_BAS_Liste_Var grh;
	t_SD_BAS_Liste_Var ipr;	
	} t_SD_BAS_IPR;
//====================================================================*/
// types definissant les informations generales du MPHR
// on considere que seuls les champs utiles sont decodes
//====================================================================*/
// attention l'ordre suivant doit correspondre a l'ordre dans le fichier
// ASCII de decommutation



typedef enum  {
		MPHR_PRODUCT_NAME,  //0
		MPHR_PARENT_PRODUCT_NAME_1,
		MPHR_PARENT_PRODUCT_NAME_2,
		MPHR_PARENT_PRODUCT_NAME_3,
		MPHR_PARENT_PRODUCT_NAME_4,
		MPHR_INSTRUMENT_ID,
		MPHR_INSTRUMENT_MODEL,
		MPHR_PRODUCT_TYPE,
		MPHR_PROCESSING_LEVEL,
		MPHR_SPACECRAFT_ID,
		MPHR_SENSING_START,//10
		MPHR_SENSING_END,
		MPHR_SENSING_START_THEORETICAL,
		MPHR_SENSING_END_THEORETICAL,
		MPHR_PROCESSING_CENTRE,
		MPHR_PROCESSOR_MAJOR_VERSION,
		MPHR_PROCESSOR_MINOR_VERSION,
		MPHR_FORMAT_MAJOR_VERSION,
		MPHR_FORMAT_MINOR_VERSION,
		MPHR_PROCESSING_TIME_START,
		MPHR_PROCESSING_TIME_END,//20
		MPHR_PROCESSING_MODE,
		MPHR_DISPOSITION_MODE,
		MPHR_RECEIVING_GROUND_STATION,
		MPHR_RECEIVE_TIME_START,
		MPHR_RECEIVE_TIME_END,
		MPHR_ORBIT_START,
		MPHR_ORBIT_END,
		MPHR_ACTUAL_PRODUCT_SIZE,
		MPHR_STATE_VECTOR_TIME,
		MPHR_SEMI_MAJOR_AXIS,//30
		MPHR_ECCENTRICITY,
		MPHR_INCLINATION,
		MPHR_PERIGEE_ARGUMENT,
		MPHR_RIGHT_ASCENSION,
		MPHR_MEAN_ANOMALY,
	 	MPHR_X_POSITION,
	 	MPHR_Y_POSITION,
	 	MPHR_Z_POSITION,
	 	MPHR_X_VELOCITY,
	 	MPHR_Y_VELOCITY,//40
	 	MPHR_Z_VELOCITY,
		MPHR_EARTH_SUN_DISTANCE_RATIO,
		MPHR_LOCATION_TOLERANCE_RADIAL,
		MPHR_LOCATION_TOLERANCE_CROSSTRACK,
		MPHR_LOCATION_TOLERANCE_ALONGTRACK,
	 	MPHR_YAW_ERROR,
		MPHR_ROLL_ERROR,
		MPHR_PITCH_ERROR,
		MPHR_SUBSAT_LATITUDE_START, /* CR292 */
		MPHR_SUBSAT_LONGITUDE_START,//50
		MPHR_SUBSAT_LATITUDE_END, /* CR292 */
		MPHR_SUBSAT_LONGITUDE_END,
		MPHR_LEAP_SECOND,
		MPHR_LEAP_SECOND_UTC,
		MPHR_TOTAL_RECORDS,
		MPHR_TOTAL_MPHR,
		MPHR_TOTAL_SPHR,
		MPHR_TOTAL_IPR,
		MPHR_TOTAL_GEADR,
		MPHR_TOTAL_GIADR,//60
		MPHR_TOTAL_VEADR,
		MPHR_TOTAL_VIADR,
		MPHR_TOTAL_MDR,
		MPHR_COUNT_DEGRADED_INST_MDR,
		MPHR_COUNT_DEGRADED_PROC_MDR,
		MPHR_COUNT_DEGRADED_INST_MDR_BLOCKS,
		MPHR_COUNT_DEGRADED_PROC_MDR_BLOCKS,
		MPHR_DURATION_OF_PRODUCT,
		MPHR_MILLISECONDS_OF_DATA_PRESENT,
		MPHR_MILLISECONDS_OF_DATA_MISSING,//70
		MPHR_SUBSETTED_PRODUCT

		} e_SD_BAS_IND_MPHR;
			





typedef struct
	{
	t_BAS_NAMEPROD product_name;
	t_BAS_NAMEPROD parent_product_name_1;
	t_BAS_NAMEPROD parent_product_name_2;
	t_BAS_NAMEPROD parent_product_name_3;
	t_BAS_NAMEPROD parent_product_name_4;
	char instrument_id[5]; // non decode
        char instrument_model[4]; // non decode
	e_BAS_PRODTYPE product_type; 
	e_BAS_PRODLEVEL processing_level;
        char spacecraft_id[4];  // non decode
	t_SD_BAS_GeneralTime sensing_start;
	t_SD_BAS_GeneralTime sensing_end;
	t_SD_BAS_GeneralTime sensing_start_theoretical;
	t_SD_BAS_GeneralTime sensing_end_theoretical;
	char  processing_centre[5]; // non decode
	char processor_major_version[6]; // non decode
	char processor_minor_version[6]; // non decode
	t_SD_BAS_VERSION format_version;
	t_SD_BAS_GeneralTime processing_time_start;
	t_SD_BAS_GeneralTime processing_time_end;
	e_BAS_PROCMODE  processing_mode; // utilise dans les MDR
	e_BAS_DISPMODE  disposition_mode; // utilise dans les MDR
	char  receiving_ground_station[3]; // non decode
	char receive_time_start[16]; // non decode
	char receive_time_end[16]; // non decode
	char orbit_start[6]; // non decode
	char orbit_end[6]; // non decode
	unsigned int actual_product_size;
	} t_SD_BAS_MPHR_ProdDetails;

typedef struct
	{
	t_SD_BAS_GeneralTime state_vector_time;
	unsigned int semi_major_axis;
	unsigned int eccentricity;
	unsigned int inclination;
	unsigned int perigee_argument;
	unsigned int right_ascension;
	unsigned int mean_anomaly;
	unsigned int x_position;
	unsigned int y_position;
	unsigned int z_position;
	unsigned int x_velocity;
	unsigned int y_velocity;
	unsigned int z_velocity;
	unsigned int earth_sun_distance_ratio;
	int location_tolerance_radial;
	int location_tolerance_crosstrack;
	int location_tolerance_alongtrack;
	int yaw_error;
	int roll_error;
	int pitch_error;
	} t_SD_BAS_MPHR_OrbParam;

typedef struct
	{
	char subsat_latitude_start[12]; // non decode
	char subsat_longitude_start[12]; // non decode
	char subsat_latitude_end[12]; // non decode
	char subsat_longitude_end[12]; // non decode
	} t_SD_BAS_MPHR_LocSum; // non decode

typedef struct
	{
	char leap_second[3]; // non decode
	char leap_second_utc[16]; // non decode
	} t_SD_BAS_MPHR_LeapSecInf;


typedef struct
	{
	char obt_utc_correction[6]; // non decode
	char obt_utc_correction_utc[16];  // non decode
	} t_SD_BAS_MPHR_obtInf;

typedef struct
	{
	unsigned int total_records;
	unsigned int total_mphr;
	unsigned int total_sphr;
	unsigned int total_ipr;
	unsigned int total_geadr;
	unsigned int total_giadr;
	unsigned int total_veadr;
	unsigned int total_viadr;
	unsigned int total_mdr;
	} t_SD_BAS_MPHR_RecCount;

typedef struct
	{
	unsigned int count_degraded_inst_mdr;
	unsigned int count_degraded_proc_mdr;
	unsigned int count_degraded_inst_mdr_blocks;
	unsigned int count_degraded_proc_mdr_blocks;
	} t_SD_BAS_MPHR_RecQualFlag;


typedef struct
	{
	unsigned int duration_of_product;
	unsigned int milliseconds_of_data_present;
	unsigned int milliseconds_of_data_missing;
	char roi_filtered[2]; // non decode
	char archive_retrieval[2]; // non decode
	} t_SD_BAS_MPHR_TimeQualFlag;

typedef struct
	{
//	t_SD_BAS_MPHR_ProdDetails ProdDetails; 
//	t_SD_BAS_MPHR_OrbParam OrbParam;
//	t_SD_BAS_MPHR_LocSum LocSum;
//	t_SD_BAS_MPHR_LeapSecInf LeapSecInf;
//	t_SD_BAS_MPHR_obtInf obtInf;
//	t_SD_BAS_MPHR_RecCount RecCount;
//	t_SD_BAS_MPHR_RecQualFlag RecQualFlag;
//	t_SD_BAS_MPHR_TimeQualFlag TimeQualFlag;
	t_SD_BAS_Liste_Var grh;
	t_SD_BAS_Liste_Var listevar;	
	} t_SD_BAS_MPHR;

//====================================================================*/
// fin du MPHR
//====================================================================*/


// type definissant les differents type d'acces au fichier 
typedef enum 
	{
	INIT=1,
	ENCOURS=2,
	FIN=3		
	} e_BAS_etat; 
	
//--------------------------------------------------------------------*/
// type definissant le type d'acces aux fichiers
//--------------------------------------------------------------------*/
typedef enum 
	{
	LECTURE=0,
	ECRITURE=1,
	APPEND=2,
	APPEND_INIT=3,
	APPEND_CLOSE=4
	} e_BAS_TypeAcces; 

//====================================================================*/
// type definissant les informations generales pour un produit ouvert
//====================================================================*/
//  
typedef struct
	{
	t_SD_BAS_NomProduit nom; // nom du produit
	t_SD_BAS_NomProduit nom_sec; // nom du produit secondaire
	FILE * fd ; 		 // file descripteur du fichier principal
	FILE * fd_sec ; 		 // file descripteur du fichier secondaire
/* FA 162 */
	t_SD_BAS_NomProduit nom_viadr_tmp; // nom du produit temporaire pour les viadr
	FILE * fd_viadr_tmp ; 		 // file descripteur du fichier temporaire
/* ..FA 162 */
	e_BAS_etat etat; 	 // etat ( INIT, ENCOURS, FIN)
	e_BAS_TypeAcces mode;    // mode ouverture (LECTURE,ECRITURE,APPEND)  
	e_BAS_Nature nature;     // nature du produit (IASI AVHRR PRODMOD ...)
	t_SD_BAS_VERSION version; // (version du produit  lue dans le MPHR)
	char *libelle; // libelle de l'erreur interne
//--- informations sur le nombre et la tailles des record 	
	unsigned long int offsetGAD; //offset du premier GAD ( issu des IPR en lecture 
	unsigned long int offsetGADSuivant; //offset du  GAD suivant( issu des IPR en lecture 
					//	     ( incremente en ecriture)
	 					     
	unsigned long int offsetVIADR; // offset du premier VAD ( issu des IPR en lecture 
	unsigned long int offsetVIADRSuivant; // offset du  MDR suivant ( incremente en lecture et
	            				//	       en ecriture )
	unsigned int nombreVIADR; // nombre de VIADR  ( initialise en lecture ( MPHR) et
	            				//	       incremente en ecriture a
	// partir d'une initialisation a 0 ( mode creation ) ou en fonction du MPHR ( mode append )
	
	unsigned long int offsetMDR; // offset du premier MDR ( issu des IPR en lecture 
						  //   ( incremente en ecriture)
	unsigned long int offsetMDRSuivant; // offset du  MDR suivant ( incremente en lecture et
	            				//	       en ecriture )
	// partir d'une initialisation a 0 ( mode creation ) ou en fonction du MPHR ( mode append )
	unsigned int nombreMDR; // nombre de MDR  ( initialise en lecture ( MPHR) et
	            					//       incremente en ecriture a
	// partir d'une initialisation a 0 ( mode creation ) ou en fonction du MPHR ( mode append )
 
 /* liste dynamique de descripteurs des IPR */
	t_SD_BAS_Produit descripteurIPR[SD_BAS_NB_MAX_IPR];
 
//--- informations pour la coherence temporelle 	
	t_SD_BAS_OBTTime dateOBTprec;
	  t_SD_BAS_SCDSTime dateSCDSprec;
	} t_SD_BAS_Fd;
	
//====================================================================*/
// External variables Declaration 
//====================================================================*/
// FT-2765
static const int SD_BAS_MPHR_VERSION=1;
static const int SD_BAS_MDR_VERSION=1;
static const int SD_BAS_IPR_VERSION=1;

/* offset des IPR */
static const int SD_BAS_OFFSET_IPR_N1 = 3307;
/* taille des IPR */
static const int SD_BAS_TAILLE_IPR    = 27;
// ...FT-2765

//====================================================================*/
// External fonctions Declaration 
//====================================================================*/


//					FONCTIONS EXTERNES
//======================================================================
#ifdef __cplusplus
extern "C" {
#endif

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_OuvrirProduit
//@
//@ Description : fonction d'ouverture du produit en lecture ou en ecriture
//@  cette operation generique s'applique aux produits IASI de N0 N1 et AVHRR
//@ remarque : cette fonction peut s'appliquer aux productmodel
//@
//@
//@ Parameters :
//@			(ENTREE)  mode  : mode d'ouverture (LECTURE, ECRITURE, APPEND)
//@			(ENTREE)  nature  : nature du produit (IASI, AVHRR, PRODMODEL)
//@			(ENTREE)  nom  : nom du produit
//@			(SORTIE)  descripteur  : descripteur du produit
//@				  utilise par les fonctions d'acces
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
/* FA 162 */
t_CMN__Statut SD_BAS_OuvrirProduit(
                                   e_BAS_TypeAcces mode, 
                                   e_BAS_Nature nature,
                                   t_BAS_NAMEPROD nom,
                                   t_SD_BAS_Fd * Descripteur,
                                   t_SD_BAS_DataThread *ctx,
				   const e_BAS_ModeTrt modeTraitement
                                   )

;
/* ..FA 162 */
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_FermerProduit
//@
//@ Description : fonction de fermeture du produit en lecture ou en ecriture
//@  cette operation generique s'applique aux produits IASI de N0 N1 et AVHRR
//@ remarque : cette fonction peut s'appliquer au productmodel
//@
//@
//@ Parameters :
//@			(ENTREE)  descripteur  : descripteur du produit
//@				  utilise par les fonctions d'acces
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
/* FA 162 */
t_CMN__Statut SD_BAS_FermerProduit(
                                   e_BAS_TypeAcces mode, 
				   t_SD_BAS_Fd * Descripteur,
                                   t_SD_BAS_DataThread *ctx,
				   const e_BAS_ModeTrt modeTraitement
                                   )
;
/* ..FA 162 */
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LireMPHR
//@
//@ Description : fonction de lecture du MPHR d'un produit 
//@  cette operation generique s'applique aux produits IASI de N0 N1 et AVHRR
//@ remarque : cette fonction peut s'appliquer au productmodel
//@
//@
//@ Parameters :
//@			(ENTREE)  descripteur  : descripteur du produit
//@				  utilise par les fonctions d'acces
//@			(SORTIE)  mphr  : liste des variables lues dans le MPHR
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LireMPHR(
		t_SD_BAS_Fd * Descripteur,
		t_SD_BAS_MPHR * mphr,
		t_SD_BAS_DataThread *ctx
			)
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_EcrireMPHR
//@
//@ Description : fonction d'ecriture du MPHR d'un produit 
//@  cette operation generique s'applique aux produits IASI de N0 N1 et AVHRR
//@
//@
//@ Parameters :
//@			(ENTREE)  descripteur  : descripteur du produit
//@				  utilise par les fonctions d'acces
//@			(ENTREE)  mphr  : liste des variables a ecrire dans le MPHR
//@			(ENTREE)  grh  : liste des variables a ecrire dans le MPHR
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_EcrireMPHR(
		t_SD_BAS_Fd * Descripteur,
		t_SD_BAS_MPHR * mphr,
		t_SD_BAS_DataThread *ctx
			)
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LireIPR
//@
//@ Description : fonction de lecture des IPR d'un produit 
//@  cette operation generique s'applique aux produits IASI de N0 N1 et AVHRR
//@  on lit ici le gru et l'IPR
//@
//@
//@ Parameters :
//@			(ENTREE)  descripteur  : descripteur du produit
//@				  utilise par les fonctions d'acces
//@		
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LireIPR(
		t_SD_BAS_Fd * Descripteur,
		t_SD_BAS_DataThread *ctx
			)
;
/* FA 162 */
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LireIPRDump
//@
//@ Description : fonction de lecture des IPR d'un produit 
//@  cette operation generique s'applique aux produits IASI de N0 N1 et AVHRR
//@  on lit ici le gru et l'IPR
//@
//@
//@ Parameters :
//@			(ENTREE)  descripteur  : descripteur du produit
//@				  utilise par les fonctions d'acces
//@		
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LireIPRDump(
		t_SD_BAS_Fd * Descripteur,
		t_SD_BAS_DataThread *ctx
			)
;

/* ..FA 162 */
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_EcrireIPR
//@
//@ Description : fonction d'ecriture de l'IPR d'un produit 
//@  cette operation generique s'applique aux produits IASI de N0 N1 et AVHRR
//@
//@
//@ Parameters :
//@			(ENTREE/SORTIE)  descripteur  : descripteur du produit
//@				  utilise par les fonctions d'acces
//@			(ENTREE)   type  : type de record pointe
//@   (ENTREE)   instrument_group : valeur du champ INSTRUMENT_GROUP 
//@                                 du record pointe
//@			(ENTREE)   subclass : subclass du record pointe
//@			(ENTREE)   offest   : offset du record pointe
//@			(ENTREE)   start  : date de debut du produit
//@			(ENTREE)   stop  : date de fin du produit
//@			(ENTREE)   num   : position de l'IPR
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_EcrireIPR(
		t_SD_BAS_Fd * Descripteur,
		unsigned int type,
		unsigned int instrument_group, 
		unsigned int subclass,
		unsigned int offset,
		t_SD_BAS_SCDSTime start,
		t_SD_BAS_SCDSTime stop,
		int num,
		
		t_SD_BAS_DataThread *ctx
		);

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LibereListeVar
//@
//@ Description : fonction de liberation d'une liste de variables decommutees
//@ recommutees
//@  cette operation generique s'applique aux produits IASI de N0 N1 et AVHRR
//@
//@
//@ Parameters :
//@			(ENTREE/SORTIE)  mphr  : liste des variables lues dans le MPHR
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LibereListeVar(
				      e_BAS_TypeAcces mode,
				    t_SD_BAS_Liste_Var * listevar
				    )
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_CommuteGRH
//@
//@ Description : cette fonction permet de commuter le GRH d'un record (20)
//@
//@
//@ Parameters : 
//@			(ENTREE)  grh : grh decode
//@			(ENTREE/SORTIE)  buffer : buffer utilise pour le commutation
//@                     
//@
//@ Return Status : 
//@			 
//@			CMN__STATUT_OK si la fonction doit etre tracee
//@			CMN__STATUT_NOK sonon
//@
//@ ------------------------------------------------------------------*/

 t_CMN__Statut SD_BAS_CommuteGRH(
		t_SD_BAS_Liste_Var  grh,
		char ** buffer
			)
;
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_DecommuteGRH
//@
//@ Description : cette fonction permet de decommuter le GRH d'un record (20)
//@
//@
//@ Parameters : 
//@			(ENTREE)  buffer : buffer issu de la lecture
//@			(SORTIE)  grh : grh decode
//@                     
//@
//@ Return Status : 
//@			 
//@			CMN__STATUT_OK si la fonction doit etre tracee
//@			CMN__STATUT_NOK sonon
//@
//@ ------------------------------------------------------------------*/
t_SD_BAS_status_d SD_BAS_DecommuteGRH( /* CR301 */
  t_SD_BAS_Liste_Var *grh,
  char * buffer);

t_CMN__Statut SD_BAS_Charger();

//======================================================================

#ifdef __cplusplus
}
#endif


#endif
