//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/BAS/src/SD_BAS_TraceOutils.c,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_BAS
//@ MODULE : SD_BAS_Trace
//@
//@ Comment :
//@   
//@   
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-169-CN : 27/02/2006 : B.MASSART
//@ Trace log added.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-184-CN : 18/10/2006 : C.MERIGUET
//@ Codage des fonctions reentrantes
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

/*---------------------------------------------------------------------------*/
/*                Inclusion des fichiers STANDARDS (.h)                      */
/*---------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <time.h>
#include <sys/time.h>
#include <sys/stat.h>   
#include <fcntl.h>
#include <malloc.h>
#include <limits.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <sys/uio.h>


#include "CMN__Commun.h"
#include "SD_BAS_TraceOutils.h"

// FT-2765
#include "SD_BAS_SwapEndian.h"
// ...FT-2765

/*====================================================================*/
/* External Constant Declaration                                      */
/*====================================================================*/

/* message d'erreur fixe par les fonctions de traces dans SD_BAS_Trace ou SD_BAS_TraceOutils */
char s_SD_BAS_ERREUR[200];
const int SD_BAS_ENTETELUM = 12;

/*====================================================================*/
/* Local Constant Declaration                                         */
/*====================================================================*/
static int s_autom1[5][5]={{ 1,11,11, 6, 0},
			 { 2,11,11,13, 1},
			 {11, 4, 3,13, 2},
			 {11, 4, 3,13, 3},
			 { 2, 5,11,13, 4}};

static int s_autom2[4][6]={{ 3, 2,11, 1, 1,11},
			 { 3,11, 4, 1,11,11},
			 { 2, 5, 2, 2, 2, 2},
			 { 3,11, 6, 3, 3, 3}}; 

static int s_autom3[2][4]={{  1, 0,  -1, 0},
			 { -25, 2, -24, 1}};

/*====================================================================*/
/* Local types Declaration                                            */
/*====================================================================*/
typedef enum { SD_BAS_DEBUT = 0 , SD_BAS_ETAT31 = 1 } e_SD_BAS_CpfEtat3;
typedef enum { SD_BAS_INFERIEUR = 0 , SD_BAS_SUPERIEUR = 1 ,
	       SD_BAS_FIN = 2 , SD_BAS_AUTRES = 3 } e_SD_BAS_CpfEvt3;
typedef enum { SD_BAS_DEBUTDEC = 0 , SD_BAS_ETAT11 = 1 , SD_BAS_ETAT12 = 2 ,
	       SD_BAS_ETAT13 = 3 , SD_BAS_ETAT14 = 4 , SD_BAS_FINDEC = 5 , 
	       SD_BAS_BLOCVIDE=6 , SD_BAS_ERR11 = 11 , SD_BAS_ERR13 =13} e_SD_BAS_CpfEtat1;
typedef enum { SD_BAS_PAR_OUV = 0 , SD_BAS_PAR_FERM = 1 , SD_BAS_DONNEE = 2 , SD_BAS_INF = 3 ,
		 SD_BAS_BLANC = 4 } e_SD_BAS_CpfEvt1;
typedef enum { SD_BAS_DEBUTYPE = 0 , SD_BAS_ETAT21 = 1 , SD_BAS_ETAT22 = 2 , SD_BAS_ETAT23 = 3 , 
	       SD_BAS_ENTIER = 4 , SD_BAS_ASC = 5 , SD_BAS_REEL = 6 , SD_BAS_ERR21 = 11 }e_SD_BAS_CpfEtat2;
typedef enum { SD_BAS_POINT = 0 , SD_BAS_GUILL = 1 , SD_BAS_ESPACE = 2 , SD_BAS_CHIFFRE = 3 ,
		 SD_BAS_SIGNE = 4 , SD_BAS_AUTRE = 5 } e_SD_BAS_CpfEvt2;


//======================================================================
// Internal fonctions Declaration
//======================================================================

 static t_SD_BAS_OK_KO SD_BAS_TraceLireCle(t_SD_BAS_CpfFile * fd,
			     int ad_deb,
			     char * mot_cle,
			     int * ad_suiv,
			     int * ad_deb_mcc);

 static t_SD_BAS_OK_KO SD_BAS_TraceLireFic( t_SD_BAS_CpfFile * fd,
			     int ad_deb,
			     char * buff);

 static t_SD_BAS_OK_KO SD_BAS_TraceEcrireFic(t_SD_BAS_CpfFile * fd,
			       int ad_deb,
			       int taille,
			       char ** buff);
 
 static t_SD_BAS_OK_KO SD_BAS_TraceRechercheCle(
				  char * buff,
				  int ad_deb,
				  char * mot_cle,
				  int * ad_suiv); 

 static t_SD_BAS_OK_KO SD_BAS_TraceEcritBuf( t_SD_BAS_CpfFile * fd,
				char * mot_cle,
				t_SD_BAS_CpfType type,
				int nb_lig,
				int nb_col,
				int taille_cell, 
				void * donnee,
				int indic_ecrit,
				char ** da_ascii);

 static t_SD_BAS_OK_KO SD_BAS_TraceMotCle( char car,
				      e_SD_BAS_CpfEvt3 * evt3 );
 static t_SD_BAS_OK_KO SD_BAS_TraceDecodedon( int taille_buf,
			       char buff[],
			       int ad_deb,
			       t_SD_BAS_CpfType * type_lu,
			       int * nb_lig_lu,
			       int * nb_elt_lu,
			       int * nb_elt_alloc,
			       int * taille_cell_lu,
			       void ** don_lue,
			       e_SD_BAS_CpfEtat1 * etat1, 
			       int * ad_fin_dec);

 static t_SD_BAS_OK_KO SD_BAS_TraceLireDonnee(
				t_SD_BAS_CpfFile * fd, 
				int ad_deb,
				void **don_lue,
				t_SD_BAS_CpfType * type_lu,
				int * nb_lig_lu, 
				int * nb_col_lu,
				int * taille_cell_lu);

 static t_SD_BAS_OK_KO SD_BAS_TraceAnalyse( char car, 
				       e_SD_BAS_CpfEvt1 * evt1);

 static t_SD_BAS_OK_KO SD_BAS_TraceType( char car, 
				    e_SD_BAS_CpfEvt2 * evt2);

 static t_SD_BAS_OK_KO SD_BAS_TraceDeplacefic(
					 t_SD_BAS_CpfFile * fd,
					 int debut,
					 int deplacement);
static t_SD_BAS_OK_KO SD_BAS_TraceEcritFichierLum(
		       char *nom_fichier,
		       void *donee,
		       int nblig,
		       int nbcol,
		       t_SD_BAS_Codage codage);

static t_SD_BAS_OK_KO SD_BAS_TraceLireFichierLum(char *nom_fichier, 
				     int nblig,
				     int nbcol,
				     t_SD_BAS_Codage codage,
					    void * donnee);
t_SD_BAS_OK_KO SD_BAS_TraiteLigne(
t_SD_BAS_CpfFile * fd,
			  int indic_ecrit,
			  int longueur,
			  char ** da_ascii,
			  char * pt_ascii 
);

t_SD_BAS_OK_KO SD_BAS_TraiteDecodedon(e_SD_BAS_CpfEtat2   etat2,
				      e_SD_BAS_CpfEvt2    evt2,
				      int indice,
				      int taille_buf,
				      char    car,
				      t_SD_BAS_CpfType	type,
				      int nb_elt,
				      char buff[],
				      int	taille_max,
				      int taille,
				      int ad_deb_do,
				      long int *entier,
				      double   *reel,  
				      char     *asc,
				      int    nb_elt_alloues
				      );
t_SD_BAS_OK_KO SD_BAS_SousTraiteDecodedon(e_SD_BAS_CpfEtat2   etat2,
					  t_SD_BAS_CpfType	type,
					  int nb_elt,
					  char buff[],
					  int ad_deb_do,
					  long int *entier,
					  double   *reel,  
					  int    nb_elt_alloues,
					  int INaN,
					  int INaNMoins
					  );
static t_SD_BAS_OK_KO SD_BAS_TraceVerifLum(
				 t_SD_BAS_CpfFile * fd,
				 t_SD_BAS_CpfType * type,
				 int * nb_lig,
				 int * nb_col, 
				 t_SD_BAS_CpfType *type_lu,
				 int nb_lig_lu,
				 int nb_col_lu,
				 void * donnee,
				 void	*don_lue);  

 static t_SD_BAS_OK_KO SD_BAS_TraceVerif(
				 t_SD_BAS_CpfType * type,
				 int * nb_lig,
				 int * nb_col, 
				 int * taille_cell,
				 t_SD_BAS_CpfType type_lu,
				 int nb_lig_lu,
				 int nb_col_lu,
				 int taille_cell_lu,
				 void * donnee,
				 void	*don_lue);    

t_SD_BAS_OK_KO SD_BAS_TraceAjouteFichierBinaire(t_SD_BAS_CpfFile *fd,
						char * mot_cle,
						t_SD_BAS_CpfType type,
						int nb_lig,   
						int nb_col,
						int taille_cell,
						void * donnee,
						char *da_ascii,
						 int  indic_ecrit);    
/*---------------------------------------------------------------------------*/
/*                Definition des CONSTANTES propres                          */
/*---------------------------------------------------------------------------*/
#define maxi(a,b) ((a) > (b) ? (a) : (b))
#define mini(a,b) ((a) > (b) ? (b) : (a))



//======================================================================
//					FONCTIONS EXTERNES
//======================================================================


/*****************************************************************************/
/*                                                                           */
/* OPERATION :    SD_BAS_ComficOpen                                          */
/* ----------                                                                */
/*                                                                           */
/* ROLE :                                                                    */
/* ------                                                                    */
/*                                                                           */
/*                                                                           */
/* CONTEXTE D'APPEL :                                                        */
/* -----------------                                                         */
/*                                                                           */
/* VERSION :  V0                                                             */
/* --------                                                                  */
/*                                                                           */
/* LANGAGE: C  ANSI                                                          */
/* --------                                                                  */
/*                                                                           */
/*****************************************************************************/
t_SD_BAS_CpfFile *SD_BAS_TraceOpen(
		       char * nom_fichier, 
		       e_SD_BAS_CpfModeOuvre  mode_ouv ) 


{
  int 	flags;            /* flag utilise pour la fonction open           */
  int	mode;             /* valeur du umask utilise pou la fonction open */
  int	fd;               /* descripteur du fichier ouvert                */
  t_SD_BAS_CpfFile *ficdon;       /* structure descriptive du fichier de donnees  */
   ficdon =  NULL;

/*..................................................*/
/* allocation dynamique de la structure de donnees  */
/*..................................................*/

  ficdon = (t_SD_BAS_CpfFile *)malloc(sizeof(t_SD_BAS_CpfFile));
  if (ficdon == NULL)
  {
    sprintf(s_SD_BAS_ERREUR,
	"Erreur d'allocation a l'ouverture du fichier %s",nom_fichier);
    goto ERREUR;
  }
/*........................................................*/
/* en fonction du mode d'ouverture , ouverture du fichier */
/*........................................................*/

  if (mode_ouv == SD_BAS_LECTURE) 
  {
	flags = O_RDONLY;
	fd = open(nom_fichier,flags);
  }
  else
  {
	flags = O_RDWR|O_CREAT;
	mode  = PERMS;
	fd = open(nom_fichier,flags,mode);	
  }

/*..........................................................*/
/*   si erreur on sort avec messsage " erreur a l'ouverture */
/*..........................................................*/
  if (fd == -1)
  {
    sprintf(s_SD_BAS_ERREUR,"Erreur a l'ouverture du fichier %s",nom_fichier);
    if (ficdon != NULL)
    {
	  free(ficdon);
          ficdon = NULL;
    }

   goto ERREUR;
  }
  else
  {
/*...........................................................*/
/* initialisations des pointeurs de la structure t_SD_BAS_CpfFile   */
/* ( buffers de lecture , liste de mots cles , longueurs ...)*/ 
/*...........................................................*/

    strcpy(ficdon->nom_fichier,nom_fichier);
    ficdon->descr = fd;
    ficdon->buffer=NULL;
    ficdon->taille_buffer = SD_BAS_TAILLE_BUFFER;
    ficdon->taille_buffer_lu=0;
    ficdon->num_buffer=-1;
    ficdon->liste_mot_cle=NULL;
    ficdon->ad_mot_cle=NULL;
    ficdon->ad_donnee=NULL;
    ficdon->nb_mot_cle=-1;
    ficdon->ad_dern_mot_cle=-1;

/*.................................*/
/* allocation dynamique du buffer */
/*.................................*/

    ficdon->buffer = (char *)malloc(SD_BAS_TAILLE_BUFFER + 1);
    if (ficdon->buffer == NULL)
    {
       sprintf(s_SD_BAS_ERREUR,
	"Erreur d'allocation a l'ouverture du fichier  %s",nom_fichier);
       if (ficdon != NULL)
       {
	 free(ficdon);
         ficdon = NULL;

	}

	goto ERREUR;
     }	
  }

/*...................................*/
/* fin de l'operation en cas nominal */
/*...................................*/

  return(ficdon);

/*...................................*/
/* fin de l'operation en cas nominal */
/*...................................*/

ERREUR:
  return(NULL);
}
/*
   Fin de l'operation SD_BAS_TraceOpen
*/

/************************************************************************/
/*                                                                      */
/* OPERATION:  SD_BAS_TraceAjoute                                     */
/*                                                                   	*/
/* ROLE:  Cette operation ajoute a la fin du fichier, une donnee 	*/
/*                                                                      */
/* CONTEXTE D'APPEL:                                                    */
/*	Cette operation doit etre appelee apres ouverture prealable 	*/
/*	du fichier de Donnees 						*/
/*                                                                      */
/* VERSION:                                                             */
/*									*/
/* CODE RETOUR : OK ou KO						*/
/*Si KO, l'erreur peut etre affichee via la variable globale s_SD_BAS_ERREUR	*/
/*	" fichier de donnees  %s non ouvert "			        */
/*	" erreur de positionnement en fin de fichier     		*/
/*	" erreur d'ecriture dans le fichier     			*/
/************************************************************************/
t_SD_BAS_OK_KO SD_BAS_TraceAjoute(t_SD_BAS_CpfFile *fd,
			     char * mot_cle,
			     t_SD_BAS_CpfType type,
			     int nb_lig,   
			     int nb_col,
			     int taille_cell,
			     void * donnee
			     )

/* [E] structure decrivant fichier donnees        */
/* [E] mot a rechercher ds fichier terminee par \0*/
/* [E] type de la donnee lue 			  */
/* [E] nombre de lignes du tableau lu 		  */
/* [E] nombre de colonnes du tableau lu 	  */  
/* [E] taille maxi en octets 1 element donnee lue */ 
/* [E] non significatif si le type diff SD_BAS_ASCII */ 
/* [E/S] pointeur sur zone memoire a ecrire       */

{
  t_SD_BAS_OK_KO	retour;		/* code retour des fonctions 			  */
 int	i;		/* indice de boucle 				  */
 int	indic_ecrit;	/* indic ecriture ds fich (=1 ecriture =0 sinon)  */
 char	* da_ascii;	/* buffer de la donnee en ascii (sous forme 	  */
			/* de x fois SD_BAS_TAILLE_MAX_LIG caracteres)	  */
 int	indic_mot_cle; 	/* indicateur de recherche du mot cle             */   
   
 
 /* FT-2766 */
 da_ascii = NULL;
 /* ...FT-2766 */

  /*-----------------------------------------------------------------*/
/* on verifie au prealable que le fichier est effectivement ouvert */
/* SINON on sort en erreur 					   */
/*-----------------------------------------------------------------*/

 if ( fd == NULL ) 
	{
	  retour = SD_BAS_ERREUR_OUV;
	  sprintf(s_SD_BAS_ERREUR,"Dans SD_BAS_TraceAjoute, fichier de donnees non ouvert");
	  goto ERREUR;
	}

/*---------------------------------------------------------*/
/* on verifie que le type fait partie des types reconnus   */
/*---------------------------------------------------------*/
 
 if ( type != SD_BAS_ENTIER4 && type != SD_BAS_REEL4 && type != SD_BAS_REEL8 && type  != SD_BAS_ASCII && type !=SD_BAS_BLOC && type!=SD_BAS_FIC_BIN_DOUBLE && type!=SD_BAS_FIC_BIN_ENTIER)
   {
     retour = SD_BAS_ERR_TYP;
     sprintf(s_SD_BAS_ERREUR,"type :%d non reconnu ( 1:SD_BAS_ENTIER4 2:SD_BAS_REEL4 3:SD_BAS_REEL8 4:SD_BAS_ASCII 5:SD_BAS_BLOC 6:SD_BAS_FIC_BIN_REEL 7:SD_BAS_FIC_BIN_ENTIER )",type);
     goto ERREUR;
   } 
/*-----------------------------------------------------------------*/
/* on verifie que la donnee n'existe pas deja (SD_BAS_TraceListe)      */
/*-----------------------------------------------------------------*/
if (fd->nb_mot_cle <= 0)
   {
    
     retour = SD_BAS_TraceListe(fd);
    
     if (retour != OK)  goto ERREUR;
   }

 indic_mot_cle = SD_BAS_NON_TROUVE;
 i = 0;
 while (i< fd->nb_mot_cle && indic_mot_cle == SD_BAS_NON_TROUVE )
   {
     if (strncmp(mot_cle,(char *)(fd->liste_mot_cle +i),SD_BAS_TAILLE_MAX_CLE) == 0)  
		indic_mot_cle = SD_BAS_TROUVE;
     i = i + 1;
   }

if (indic_mot_cle == SD_BAS_TROUVE)
  {
    retour = SD_BAS_MOT_DEJA_EXISTE;
    sprintf(s_SD_BAS_ERREUR,"Dans SD_BAS_TraceAjoute, mot cle %s deja existant ",mot_cle);
    goto ERREUR;
  }
 
/*--------------------------------------------------------------------*/
/* On ecrit la donnee dans un buffer intermediaire et dans le fichier */
/*--------------------------------------------------------------------*/

 indic_ecrit = 1;
 
 if ( type == SD_BAS_ENTIER4 || type == SD_BAS_REEL4 || type == SD_BAS_REEL8 || type  == SD_BAS_ASCII || type ==SD_BAS_BLOC )
   {
     
     retour = SD_BAS_TraceEcritBuf
       ( 
	fd,	  /* [E] file descriptor du fichier             */
	mot_cle,	  /* [E] mot cle associe aux donnees            */
	type,	  /* [E] type des donnees a ecrire	      */
	nb_lig,	  /* [E] nombre de lignes du tableau a ecrire   */
	nb_col,	  /* [E] nombre de colonnes du tableau a ecrire */
	taille_cell, /* [E] taille maxi en octets d'un element     */
	/*     (significatif si type = ascii)	      */
	donnee,	  /* [E] pointeur sur zone memoire a ecrire     */
	indic_ecrit,	/* [E] indicateur si ecriture du buffer dans  */
                        /*     le fichier (=1) 1sinon (=0)             */
	&da_ascii);	/* [S] buffer de  donnee en ascii (sous forme */
				/* de x fois SD_BAS_TAILLE_MAX_LIG caracteres)   */
     if (retour != OK) goto ERREUR;
     
   }

// Cas des fichiers binaires
 else
   {

     retour = SD_BAS_TraceAjouteFichierBinaire(fd,
					      mot_cle,
					      type,
					      nb_lig,   
					      nb_col,
					      taille_cell,
					      donnee,
					      da_ascii,
					      indic_ecrit);
     if (retour != OK) goto ERREUR;

   }



/*-------------------------------*/
/* Fin du traitement nominal     */
/*-------------------------------*/
 
 retour = OK;

 return (retour);
 
/*----------------------------*/
/* traitement degrade :      */
/*---------------------------*/
 ERREUR :
   if (da_ascii != NULL)
     {
       free(da_ascii);
       
     }
 return (retour);
 
}
/*
   Fin de l'operation SD_BAS_TraceAjoute
*/
/************************************************************************/
/*                                                                      */
/* OPERATION:  SD_BAS_TraceAjouteFichierBinaire                                     */
/*                                                                   	*/
/* ROLE:  Cette operation cree un fichier binaire associe a une donnee 	*/
/*        du fichier de race                                                              */
/*                                                                      */
/* CONTEXTE D'APPEL:                                                    */
/*	Interne a  SD_BAS_TraceAjoute                             	*/
/*                                                                      */
/* VERSION:                                                             */
/*									*/
/* CODE RETOUR : OK ou KO						*/
/*Si KO, l'erreur peut etre affichee via la variable globale s_SD_BAS_ERREUR	*/
/************************************************************************/
t_SD_BAS_OK_KO SD_BAS_TraceAjouteFichierBinaire(t_SD_BAS_CpfFile *fd,
						char * mot_cle,
						t_SD_BAS_CpfType type,
						int nb_lig,   
						int nb_col,
						int taille_cell,
						void * donnee,
						char *da_ascii,
						int  indic_ecrit)

{
  char mot[7];   //buffer temporaire pour 
  int i;
  char pt_message1[SD_BAS_TAILLE_MAX_ASCII]; //premier partie du message de trace
 char pt_message2[SD_BAS_TAILLE_MAX_ASCII]; //deuxieme partie du message de trace

 t_SD_BAS_CpfType type_temp;   
 int nb_ligTmp;
 int nb_colTmp;
 t_SD_BAS_Codage codage; //codage du fichier binaire
 char nom_fic[SD_BAS_TAILLE_MAX_ASCII] ;         // nom du dichier binaire
 char directory[SD_BAS_TAILLE_MAX_ASCII];      // repertoire du fichier de trace
 char directory_lum[SD_BAS_TAILLE_MAX_ASCII];      // repertoire des fichiers binaires
 t_SD_BAS_OK_KO	retour;
 char * ext;
 char * path;

// CREATION DU MESSAGE POUR LE FICHIER DE TRACE


   

 if ( type == SD_BAS_FIC_BIN_DOUBLE ) 
   strcpy(mot,FICDOUBLE);
 
 if ( type == SD_BAS_FIC_BIN_ENTIER ) 
   strcpy(mot,FICENTIER);
 
 // extraction du nom

/* suppresion du PATH s'il existe */
 path = strrchr(fd->nom_fichier, '/');
 if(path != NULL){
   strcpy(nom_fic,(path+1));

   strncpy(directory, fd->nom_fichier, path - fd->nom_fichier+1);
   directory[ path - fd->nom_fichier+1]='\0';
   
 }
 else{
   strcpy(nom_fic,fd->nom_fichier);
   strcpy(directory,"\0");
 }

   
/* suppresion de l'extension si elle existe */
 ext = strrchr(nom_fic, '.');
 if(ext != NULL)
   nom_fic[(int)(ext - nom_fic)]='\0';

 sprintf(directory_lum,"%s%s%s",directory,nom_fic,LUM_DIRECTORY );
  

 // mise en place du message
 sprintf(pt_message2,"FICHIER %s %d %d",mot,nb_lig,nb_col);
 
 sprintf(pt_message1,"%s%s%s.lum",nom_fic,LUM_DIRECTORY, mot_cle);
 
 


 taille_cell= maxi(strlen(pt_message1)+1,strlen(pt_message2)+1);
 
 for(i=0;i<taille_cell;i++)
   {
     pt_message1[i+taille_cell]=pt_message2[i];
   }

 // ECRITURE DU MESSAGE
 type_temp = SD_BAS_ASCII;
 nb_ligTmp = 1;
 nb_colTmp = 2;
  
  


 retour = SD_BAS_TraceEcritBuf
   (
    fd,
    mot_cle,
    type_temp,
    nb_ligTmp,
    nb_colTmp,
    taille_cell,
    pt_message1,
    indic_ecrit,
    &da_ascii);
 

 taille_cell= maxi(strlen(pt_message1)+1,strlen(pt_message2)+1);
   
 if (retour != OK) goto ERREUR;
 
 
 //ecriture du fichier au format binaire
 
 // detection du type de codage
 if ( type == SD_BAS_FIC_BIN_DOUBLE )
    { 
      strcpy(codage,CODAGE_DOUBLE);
    }

 if ( type == SD_BAS_FIC_BIN_ENTIER ) 
   {
     strcpy(codage,CODAGE_ENTIER);
   } 

 
 if(access(pt_message1,R_OK) == -1)
   mkdir(directory_lum,0777);
 
 sprintf(nom_fic, "%s%s", directory, pt_message1);

 retour = SD_BAS_TraceEcritFichierLum(nom_fic,donnee,nb_lig,nb_col,codage);
 if (retour != OK) goto ERREUR;
 
 
ERREUR :

return (retour);

}


/*****************************************************************************/
/*                                                                           */
/* OPERATION: SD_BAS_TraceClose                                            */
/*                                                                           */
/* ROLE: Cette operation externe est chargee de fermer le fichier de donnees */
/*  precedenment ouvert par l'operation SD_BAS_TraceOpen et libere	     */
/* la zone memoire reservee lors des operations precedentes dans la structure*/
/* t_SD_BAS_CpfFile 	       						     */
/*                                                                           */
/*                                                           
                */
/*                                                                           */
/* CONTEXTE D'APPEL:                                                         */
/*                                                                           */
/* VERSION:                                                                  */
/* CODES RETOUR: OK ou KO						     */
/*	" erreur a la fermeture du fichier %s "				     */
/*****************************************************************************/
t_SD_BAS_OK_KO SD_BAS_TraceClose(t_SD_BAS_CpfFile * fd)

 /* [E] structure decrivant le fichier de donnees  */

{
  t_SD_BAS_OK_KO	retour;		/* code retour appel fonction */

  retour = OK;

  if (fd ==NULL)
	{
	retour = SD_BAS_ERREUR_OUV;
	sprintf(s_SD_BAS_ERREUR,"Erreur a la fermeture d'un fichier de donnees  ");
		goto ERREUR;
	}
close(fd->descr);
/* fermeture du fichier 						     */
/* desallocation des buffer alloues */
/* si le pointeurs de la structure SD_BAS_FILE sont <> NULL		     */
/*	la memoire allouee est liberee 					     */

if (fd->buffer != NULL)
	{
	free(fd->buffer);
        fd->buffer = NULL;
	}
if (fd->liste_mot_cle != NULL)
	{
	free(fd->liste_mot_cle);
        fd->liste_mot_cle = NULL;
   	}
if (fd->ad_mot_cle != NULL)
	{
	free(fd->ad_mot_cle);
        fd->ad_mot_cle= NULL;
	}
if (fd->ad_donnee != NULL)
	{
	free(fd->ad_donnee);
        fd->ad_donnee = NULL;
	}
free(fd);
fd = NULL;
/*-------------------------------------------------------------------*/
/* fin de l'operation en cas nominal  				     */
/*-------------------------------------------------------------------*/
return(retour);
/*-------------------------------------------------------------------*/
/* fin de l'operation en cas degrade  				     */
/*-------------------------------------------------------------------*/
ERREUR:

return(retour);
}
/*
   Fin de l'operation SD_BAS_TraceClose
*/


//======================================================================
//					FONCTIONS INTERNES  
//======================================================================

/**************************************************************************/
/*                                                                        */
/* OPERATION: SD_BAS_TraceListe                                         */
/*                                                                        */
/* ROLE:  Cette operation recherche la liste des mots cles presents ds un */
/*	  fichier de Donnees                                              */
/*                                                                        */
/* ATTENTION : cette fonction s'alloue de l'espace memoire qu'il faudra   */
/* liberer a la fin du traitement                                         */
/*                                                                        */
/* CONTEXTE D'APPEL:                                                      */
/*                                                                        */
/* VERSION:                                                               */
/* CODE RETOUR : OK ou KO						  */
/*	" taille d'un mot cle lu superieure a la limite "	          */
/*	" erreur a la lecture du fichier de donnees  "		          */
/**************************************************************************/
t_SD_BAS_OK_KO SD_BAS_TraceListe(t_SD_BAS_CpfFile * fd)

		/*[E/S] file descriptor du fichier de donnees */

{
int	adr_deb;	/* adresse dans le fichier de donnees         */
int	ad_suiv;	/* adresse 1er caractere suivant un mot cle   */
int	ad_deb_mcc;	/* adresse 1er caractere des donnees  mot cle */
int	*ad;		/* adresse pour copie			      */
int	nb_mot_cle;	/* nombre de mots cle trouves dans le fichier */
size_t	taille;		/* taille du buffer a allouer		      */
t_SD_BAS_OK_KO	retour;		/* code retour appel fonctions	      */
t_SD_BAS_CpfMotCle mot_cle;	/* mot cle trouve		      */
char 	*adr_copie;	/* adresse pour copie de mots cle	      */
int	nb_bloc;	/* nombre de realloc realises +1              */
off_t   offset;	  	
off_t   offreel;

/*------------------------------------------------------*/
/* initialisation a zero du nombre de mots cle trouves  */
/*------------------------------------------------------*/
nb_mot_cle = 0;
nb_bloc = 1;

/*-----------------------------------------------------------------*/
/* on verifie au prealable que le fichier est effectivement ouvert */
/* SINON on sort en erreur 					   */
/*-----------------------------------------------------------------*/
if ( fd == NULL ) 
{
   retour = SD_BAS_ERREUR_OUV;
   sprintf(s_SD_BAS_ERREUR,"Dans SD_BAS_TraceListe fichier de donnees non ouvert");
   goto ERREUR;
}

/*-------------------------------------*/
/* Positionnement en debut de fichier  */
/*-------------------------------------*/
/* on se positionne sur le debut du fichier ( rewind  */
offset=0;
offreel=lseek(fd->descr,offset,SEEK_SET);
/* SI erreur on sort "erreur lors d'un lseek du fichier de donnees " */
if ( offreel == -1)
	{
	retour=SD_BAS_PB_LSEEK;
	sprintf(s_SD_BAS_ERREUR,"erreur lors du lseek sur fichier %s",fd->nom_fichier);
	goto ERREUR;
	}

/*----------------------------------------------------------------------*/
/* Allocation espace memoire contenant la liste des mots cles trouves 	*/
/* Allocation d'espaces memoire contenant les adresses des mots cles    */
/* et les adresses des donnees associees a chaque mot cle 		*/
/* (il faut essayer d'allouer un maximum de place representative afin 	*/
/* d'eviter un trop grand nombre de realloc)				*/
/*----------------------------------------------------------------------*/
taille = SD_BAS_TAILLE_MAX_CLE * SD_BAS_NB_MAX_CLE;
fd->liste_mot_cle = (t_SD_BAS_CpfMotCle *)malloc (taille);
if (fd->liste_mot_cle == NULL) 
	{
	retour = SD_BAS_ERR_ALLOCA;
	sprintf (s_SD_BAS_ERREUR," erreur allocation memoire pour liste mots cle ");
	goto ERREUR;
	}

taille = SD_BAS_NB_MAX_CLE * sizeof(int);
fd->ad_mot_cle = (int *)malloc (taille);
if (fd->ad_mot_cle == NULL) 
	{
	retour = SD_BAS_ERR_ALLOCA;
	sprintf (s_SD_BAS_ERREUR," erreur allocation memoire pour adresse mots cle ");
	goto ERREUR;
	}

taille = SD_BAS_NB_MAX_CLE * sizeof(int);
fd->ad_donnee = (int *)malloc (taille);
if (fd->ad_donnee == NULL) 
{
   retour = SD_BAS_ERR_ALLOCA;
   sprintf (s_SD_BAS_ERREUR," erreur allocation memoire pour adresse donnees d'un mots cle ");
   goto ERREUR;
}

/*------------------------------------*/
/* Adr_deb = adresse debut de fichier */
/*------------------------------------*/
adr_deb = 0;

/*-----------------------------------------------------------*/
/* Recherche du premier mot cle du fichier(SD_BAS_TraceLireCle)  */
/* Si erreur de recherche, sortie en erreur		     */
/*-----------------------------------------------------------*/
retour = SD_BAS_TraceLireCle (fd, 	/* structure du fichier a lire */
			adr_deb, 	/* adresse du debut de recherche dans le fichier */
			mot_cle,	/* mot cle trouve */
			&ad_suiv, 	/* adresse dans le fichier du premier caractere suivant le mot cle */
			&ad_deb_mcc) ;	/* adresse dans le fichier du premier caractere du mot cle */

if (retour != OK && retour != SD_BAS_EOF) 
	{
	retour = SD_BAS_ERREUR_RECHERCHE_MOT;
	sprintf (s_SD_BAS_ERREUR," erreur recherche du premier mot cle ");
	goto ERREUR;
	}

/*-------------------------------------------------------*/
/* TANT QUE il existe des mots cles dans le fichier      */
/*-------------------------------------------------------*/
while (retour != SD_BAS_EOF)
{
/*------------------------------------------------------------------*/
/* 	si la liste des mots cle est saturee alors reallocation du  */
/*	double de place pour les mots cle et les adresse de ces mots*/
/*------------------------------------------------------------------*/
   if (nb_mot_cle == (SD_BAS_NB_MAX_CLE * nb_bloc))
   {
	nb_bloc = nb_bloc + 1;

	taille = SD_BAS_TAILLE_MAX_CLE * SD_BAS_NB_MAX_CLE * nb_bloc;
	fd->liste_mot_cle = (t_SD_BAS_CpfMotCle *)realloc (fd->liste_mot_cle, taille);
	if (fd->liste_mot_cle == NULL) 
	{
	   retour = SD_BAS_ERR_ALLOCA;
	   sprintf (s_SD_BAS_ERREUR," erreur re-allocation memoire pour liste mots cle - bloc %d ", 
	   nb_bloc);
	   goto ERREUR;
	 }

	taille = SD_BAS_NB_MAX_CLE * sizeof(int) * nb_bloc;
	fd->ad_mot_cle = (int *)realloc (fd->ad_mot_cle, taille);
	if (fd->ad_mot_cle == NULL) 
	{
	   retour = SD_BAS_ERR_ALLOCA;
	   sprintf (s_SD_BAS_ERREUR," erreur re-allocation memoire pour adresse mots cle - bloc %d ", 
			nb_bloc);
	   goto ERREUR;
	}

	taille = SD_BAS_NB_MAX_CLE * sizeof(int) * nb_bloc;
	fd->ad_donnee = (int *)realloc (fd->ad_donnee, taille);
	if (fd->ad_donnee == NULL) 
	{
	   retour = SD_BAS_ERR_ALLOCA;
	   sprintf (s_SD_BAS_ERREUR,
		" erreur re-allocation memoire pour donnees d un mots cle - bloc %d", nb_bloc);
	   goto ERREUR;
	}
   }
		
/*-------------------------------------------------------*/
/*	sauvegarde du mot cle trouve dans liste_mot_cle	 */
/*-------------------------------------------------------*/
	adr_copie =(char *)(fd->liste_mot_cle + nb_mot_cle);
	strncpy (adr_copie, mot_cle, SD_BAS_TAILLE_MAX_CLE);

/*-----------------------------------------------------------------*/
/*	sauvegarde adresse ds fichier du mot cle trouve tab_adresse*/
/* 	et adresse 1er caractere des donnees associees au mot cle  */
/*-----------------------------------------------------------------*/
	ad = (int *)(fd->ad_mot_cle + nb_mot_cle);
	*ad = ad_deb_mcc;
	ad = (int *)(fd->ad_donnee + nb_mot_cle);
	*ad = ad_suiv;

/*-------------------------------------------------------*/
/*	incrementation du nombre de mots cle trouves     */
/*-------------------------------------------------------*/
	nb_mot_cle = nb_mot_cle + 1;

/*---------------------------------------------------------------------*/
/*	recherche du mot cle suivant dans le fichier (SD_BAS_TraceLireCle) */
/*---------------------------------------------------------------------*/
	adr_deb = ad_suiv;
	retour = SD_BAS_TraceLireCle (
		fd, 	      /* structure du fichier a lire              */
	        adr_deb,      /* adresse du debut de recherche ds fichier */
		mot_cle,      /* mot cle trouve			          */
		&ad_suiv,     /* adres ds fic 1er caractere apres mot cle */
	        &ad_deb_mcc); /* adres ds fic 1er caractere du mot cle    */
	if (retour != OK && retour != SD_BAS_EOF) 
		{
		retour = SD_BAS_ERREUR_RECHERCHE_MOT;
		sprintf (s_SD_BAS_ERREUR," erreur recherche d un mot cle ");
		goto ERREUR;
		}

	}	/* FIN TANT QUE (plus de mots cle)		*/	

/*----------------------------------------------*/
/*  Sauvegarde du nombre de mots cle trouves    */
/*----------------------------------------------*/
fd->nb_mot_cle = nb_mot_cle;


/*----------------------------*/
/* Fin du traitement nominal  */
/*----------------------------*/
retour =  OK;
return (retour);

/*------------------------*/
/* traitement degrade :   */
/*------------------------*/
ERREUR :

if (fd != NULL)
	{
if (fd->liste_mot_cle != NULL)
	{
	free(fd->liste_mot_cle);
        fd->liste_mot_cle=NULL;

	}
if (fd->ad_mot_cle != NULL)
	{
	free(fd->ad_mot_cle);
	fd->ad_mot_cle=NULL;

	}
if (fd->ad_donnee != NULL)
	{
	free(fd->ad_donnee);
	fd->ad_donnee=NULL;
        }


	fd->nb_mot_cle = -1;
	}
return (retour);
}
/*
   Fin de l'operation SD_BAS_TraceListe
*/
/*************************************************************************/
/*                                                                       */
/* OPERATION: SD_BAS_TraceLireCle()                                    */
/*                                                                       */
/* ROLE: cette operation interne recherche le mot cle suivant detecte    */
/* apres une "adresse dans le fichier ".				 */
/* cette operation retourne le mot cle suivant s'il existe et une adresse*/
/* sur le premier caractere suivant ce mot cle                           */
/*  NB : la notion d'adresse ici est relative au fichier :c.a.d          */
/* 	1er octet du fichier = adresse 0                                 */
/* 	2eme octet du fichier = adresse 1 ...                            */
/*                                                                       */
/*                                                                       */
/* CONTEXTE D'APPEL:                                                     */
/*                                                                       */
/* VERSION:                                                              */
/* CODE RETOUR : OK ou KO 					         */
/*   " mot cle non present dans le fichier de donnees  %s"    		 */
/*   " taille du mot cle lue %s superieure a la limite %d ou egale a 0"  */
/*   " erreur a la lecture du fichier de donnees  %s"                    */
/*************************************************************************/

 static t_SD_BAS_OK_KO SD_BAS_TraceLireCle(
			     t_SD_BAS_CpfFile * fd,
			     int ad_deb,
			     char * mot_cle,
			     int * ad_suiv,
			     int * ad_deb_mcc
			     )	
	 	
      /* [E] structure  fichier de donnees 		*/ 
   /* [E] adresse de debut de recherche 		*/
    /* [S] mot cle suivant lu (termine par \0)        */
    /* [S] adresse du 1er caractere suivant le mot cle*/ 
  /* [S] adresse du 1er caractere du mot cle        */

{
  t_SD_BAS_OK_KO 	cr;		      /* code retour				   */
  int	num;		      /* numero du buffer courant 		   */
  int 	ad_deb_buff;          /* adresse de debut du buffer	           */
  int     ad_deb_rel;           /* adres deb recherche relative buf courant  */
  int     ad_suivante;          /* adres 1er caractere suivant mot cle couran*/
  int     taille_buff_courant;  /* taille du buffer lu		           */
  char   *buff_courant;         /* buffer courant			           */ 
  t_SD_BAS_CpfMotCle mot_cle_courant;  /* mot cle courant 		           */
 off_t   offset;	  	
 off_t   offreel;
 int     nb_buffer;	      /* nombre de buffer dans le fichier 	   */

/* dans t_SD_BAS_CpfFile on prevoit un indicateur qui vaut -1 si aucun buffer 	  */
/* n'a pas deja ete lu et qui vaut sinon  numero du buffer lu ds fichier  */
/* on prevoit aussi taille buffer lu et l'allocation statique du buffer   */  
/* et le decalage entre 2 buffer consecutifs ( fixe a  taille_buff/2)     */ 
/* initialisations 						          */ 
 cr=OK;
 
/*--------------------------------------------------------*/
/* on recherche alors le nombre de buffer dans le fichier */
/*--------------------------------------------------------*/
/* on se positionne sur la fin du fichier ( rewind )      */
offset=0;
offreel=lseek(fd->descr,offset,SEEK_END);
/* SI erreur on sort "erreur lors d'un lseek du fichier de donnees " */
if ( offreel == -1)
	{
	cr=SD_BAS_PB_LSEEK;
	sprintf(s_SD_BAS_ERREUR,"erreur lors du lseek sur fichier %s",fd->nom_fichier);
	goto ERREUR;
	}
offreel=lseek(fd->descr,offset,SEEK_CUR);
/* SI erreur on sort "erreur lors d'un lseek du fichier de donnees " */
if ( offreel == -1)
	{
	cr=SD_BAS_PB_LSEEK;
	sprintf(s_SD_BAS_ERREUR,"erreur lors du lseek sur fichier %s",fd->nom_fichier);
	goto ERREUR;
	}
if ( offreel < ad_deb)
{
   cr=SD_BAS_ERR_ADRESSE;
   sprintf(s_SD_BAS_ERREUR,"erreur adresse de recherche en dehors du fichier (%d)",
		ad_deb);
   goto ERREUR;
}

nb_buffer = (offreel / (fd->taille_buffer/2) ) + 1; 

/*---------------------------------------------------------*/
/* on commence par calculer le premier buffer a lire par :*/
/* num = 2 * ad_deb/(fd->taille buffer)			  */
/*--------------------------------------------------------*/
num= (ad_deb/(fd->taille_buffer/2));
ad_deb_buff = num * (fd->taille_buffer /2);

/* SI indicateur != num 	*/
if ( num != fd->num_buffer)
{
/*--------------------------------------------------*/
/* 	lecture du buffer num a partir de l'adresse */
/*      num*fd->taille_buffer/2 et de taille        */
/*     fd->taille_buffer*2 (operation interne )	    */
/*--------------------------------------------------*/
   cr=SD_BAS_TraceLireFic(
		fd,	       /* [E] structure decrivant le fichier  */
	        ad_deb_buff,   /* [E] adresse du premier octet a lire */ 
        	fd->buffer     /* [E/S] buffer a rempli		      */
		   );
/*    SI erreur on sort 								*/
	if (cr != OK)
		{
		goto ERREUR;
		}
/* initialisation du numero de buffer */
	fd->num_buffer=num;
	buff_courant = fd->buffer;
	taille_buff_courant = fd->taille_buffer_lu;	
	}
else
	{
/* SINON tab = fd ->ad_buffer 							*/
	buff_courant = fd->buffer;
	taille_buff_courant = fd->taille_buffer_lu;	
	}

ad_deb_rel = ad_deb - ad_deb_buff;
/*----------------------------------------*/
/* TANT QUE mot_cle non trouve et non EOF */
/*----------------------------------------*/
cr=KO;
while ( cr !=OK && taille_buff_courant > 0 && 
        cr != SD_BAS_EOF && cr != SD_BAS_PAS_MC)
{
/*--------------------------------------------------------------*/
/* 	recherche mot cle ds buffer buff on retourne mot cle    */
/*    sa longueur ,adres ds tableau du caractere suivant mot cle*/ 
/* NB: attention au cas ou le mot cle peut etre coupe           */ 
/*--------------------------------------------------------------*/
	cr=SD_BAS_TraceRechercheCle( 
        	buff_courant,       /* [E] buffer a scanner                  */
		ad_deb_rel,	    /* [E] adresse debut scanner le buffer   */
	        mot_cle_courant,    /* [S] mot cle suivant lu                */
	        &ad_suivante	    /* [S] adres 1er caractere apres mot cle */
				);
/* si erreur on sort */
	if (cr != OK && cr != SD_BAS_EOF && cr != SD_BAS_PAS_MC)
		{
		goto ERREUR;
		}

/*  Si mot_cle non trouve et non EOF  on incremente numero buffer 		*/
	if ( (cr == SD_BAS_EOF||cr == SD_BAS_PAS_MC) && num+1< nb_buffer)
	{
/* calcul de l'adresse de debut de recherche */
	   ad_deb_rel=0;
/*------------------------------------------------------------*/
/* 		lecture du buffer num a partir de l'adresse   */
/*		num*fd->taille_buffer/2 et de taille          */
/*     	fd->taille_buffer*2 (operation interne )              */
/*-----------------------------------------------------------*/
	   num=num+1;
	   ad_deb_buff = num * (fd->taille_buffer /2);
	   cr=SD_BAS_TraceLireFic(
		  fd,          /* [E] structure du fichier de donnees */
	          ad_deb_buff, /* [E] adresse du premier octet a lire */ 
        	  fd->buffer   /* [E/S] buffer a remplir              */
		  	 );
/*    SI erreur on sort 								*/
	   if (cr != OK)
	   {
		goto ERREUR;
	   }
/* initialisation du numero de buffer */
	   fd->num_buffer=num;
/*       		on met a jour les variables locales	*/
	   buff_courant = fd->buffer;
	   taille_buff_courant = fd->taille_buffer_lu;
	   cr=KO;
	} 		
     }
/*--------------------------------------*/
/* FIN TANT QUE mot_cle non trouve 	*/			
/*--------------------------------------*/

/*----------------------*/
/* SI mot_cle trouve 	*/
/*----------------------*/
if (cr == OK)
{
/*   le mot cle est recopie dans l'argument de sortie	*/
	strcpy(mot_cle,mot_cle_courant);

/*   l'adresse ds tableau du caractere apres mot cle est convertie */
/*   en adresse absolue ds le fichier 				   */		
/*  ad_suiv = num*fd->taille_buff/2 + (ad_suiv-ad_deb)             */
	*ad_suiv = (num * fd->taille_buffer/2) + ad_suivante;

/*  l'adresse ds tableau 1er caractere du  mot cle est convertie en */
/*  adresse absolue dans le fichier ad_deb_mc = ad_suiv -long       */
	*ad_deb_mcc=*ad_suiv -strlen(mot_cle) - 2;
}
/*-------------------------*/
/* SI EOF plus de mot cle  */
/*-------------------------*/
if (cr == SD_BAS_PAS_MC) 
	{
	goto ERREUR;
	}
if (taille_buff_courant ==0 || cr == SD_BAS_EOF)
	{
	cr=SD_BAS_EOF;
	goto ERREUR;
	}
/*-------------------------------------*/
/* fin de l'operation en cas nominal   */
/*-------------------------------------*/
return(OK);
/*-------------------------------------*/
/* fin de l'operation en cas d'erreur  */
/*-------------------------------------*/
ERREUR:
return(cr);	
}
/*
   Fin de l'operation SD_BAS_TraceLireCle
*/

/*****************************************************************************/
/*                                                                           */
/* OPERATION:  SD_BAS_TraceEcritBuf                                             */
/*                                                                           */
/* ROLE:  Cette operation ecrit une donnee dans un buffer     	             */
/*                                                                           */
/*                                                                           */
/*                                                                           */
/* CONTEXTE D'APPEL:                                                         */
/*	Cette operation doit etre appelee apres ouverture prealable fichier  */
/*	de Donnees   						             */
/*                                                                           */
/* VERSION:                                                                  */
/*									     */
/* CODE RETOUR : OK ou KO					             */
/*  si KO, l'erreur peut etre affichee via la variable globale s_SD_BAS_ERREUR    */
/*	" fichier de donnees  %s non ouvert "		                     */
/*	" erreur de positionnement en fin de fichier "			     */
/*	" erreur d'ecriture dans le fichier "				     */
/*****************************************************************************/
static t_SD_BAS_OK_KO SD_BAS_TraceEcritBuf( t_SD_BAS_CpfFile * fd,
				char * mot_cle,
				t_SD_BAS_CpfType type,
				int nb_lig,
				int nb_col,
				int taille_cell, 
				void * donnee,
				int indic_ecrit,
				char ** da_ascii
				) 

     /* [E] file descriptor du fichier de donnees	*/
     /* [E] mot cle associe aux donnees      	*/
     /* [E] type des donnees a ecrire		*/
     /* [E] nombre de lignes du tableau a ecrire	*/
     /* [E] nombre de colonnes du tableau a ecrire	*/
     /* [E] taille maxi en octets d'un element	*/
     /*     (significatif si type = ascii)		*/
     /* [E] pointeur sur la zone memoire a ecrire	*/
     /* [E] indicateur si ecriture du buffer dans 	*/
     /*     le fichier (=1) sinon (=0)		*/
     /* [S] buffer de la donnee en ascii (sous forme	*/
     /* de x fois SD_BAS_TAILLE_MAX_LIG caracteres)	*/

{
t_SD_BAS_OK_KO	retour;			       /* code retour des fonctions        */


int	lig;				/* indice de boucle sur les lignes */
 int     col;

char	* pt_deb_lig;			/* pointe sur da_ascii a l'adres de*/
char 	* pt_ascii;			/* pointe da_ascii 1er caract libre*/					/* debut de derniere ligne         */
int	nb_bloc;			/* nomb blocs de SD_BAS_TAILLE_BUFFER */	
				        /* alloues pour da_ascii           */
 int	longueur;			/* longueur significative buffer   */
 long	*dbentier;			/* pointeur variable entiere double*/
 float	*reel;				/* point variable reel simpl precis*/
 double	*dbreel;			/* point sur variable double precis*/
 char	* pt_don;		       /* pointeur sur don_ascii           */
 char *    tempo;
 int	sauv_diff_deb;			/* diff d'adres derniere ligne du  */ 
					/* buffer et debut du buffer       */
 int	sauv_diff_cour;			/* diff d'adres 1er element libre  */
					/* du buffer et debut du buffer    */

 char	don_ascii[SD_BAS_TAILLE_MAX_LIG]; /* buffer contenant une donnee ascii*/
/*--------------------------*/
/* initialisation des types */
/*--------------------------*/

  dbentier = (long *) donnee;
  reel = (float *) donnee;
  dbreel = (double *) donnee;


/*----------------------------------------------------------------*/
/* Allocation d'un espace memoire buffer d'ecriture de la donnee  */
/* (il faut essayer d'allouer un max de place representative afin */
/* d'eviter un trop grand nombre de realloc)          	          */
/*----------------------------------------------------------------*/
  *da_ascii = NULL;
  *da_ascii = (char *)malloc (SD_BAS_TAILLE_BUFFER);
  if (*da_ascii== NULL) 
    {
        retour = SD_BAS_ERR_ALLOCA;
	sprintf (s_SD_BAS_ERREUR," erreur allocation memoire pour donnees ascii ");
	goto ERREUR;
    }
         nb_bloc = 1;

/*-------------------------------------------------------------------*/
/* On ecrit ds buffer, avec syntaxe desiree le mot cle suivi d'un \n */
/*-------------------------------------------------------------------*/
sprintf (*da_ascii, "<%s>\n", mot_cle);
pt_ascii = *da_ascii + strlen (*da_ascii);
pt_deb_lig = pt_ascii;

/*------------------------------------------------------------*/
/* Init d'une ligne de caracteres ascii, si donnees a ecrire  */
/*------------------------------------------------------------*/

if ((nb_lig != 0 && nb_col != 0 ) || type == SD_BAS_BLOC)
	{
	if (type == SD_BAS_BLOC)
		{
		nb_lig=0;
		}
	else
		{
		*pt_deb_lig = '(';
		pt_ascii = pt_ascii + 1;
		}
	}







/*--------------------------------*/
/* POUR chaque ligne des donnees  */
/*--------------------------------*/
for (lig=0; lig < nb_lig; lig++)
	{

/*------------------------------------------*/
/*	Initialisation ( pour chaque ligne  */
/*------------------------------------------*/
        if (lig==0)
        {
	 strcpy (pt_ascii ,"( ");
	 pt_ascii = pt_ascii + 2;
        }
        else
        {
	 strcpy (pt_ascii ," ( ");
	 pt_ascii = pt_ascii + 3;
        }


	for (col=0; col < nb_col; col++)
	{


	  
/*-----------------------------------------------------------*/
/* 	   Suivant le type des donnees (entier  double,reel  */
/*	  simple ou double, ascii), on ecrit dans le buffer  */
/*	  DON_ASCII dans un format donne (%ld, %f, %lf, %s)  */
/*	   la donnee suivie d'un blanc            	     */
/*-----------------------------------------------------------*/
		
		switch (type)
			{
			case SD_BAS_ENTIER4:
				sprintf(don_ascii, "%ld\t", *(dbentier+(nb_col*lig + col)) );
				break;
			case SD_BAS_REEL4:
				sprintf(don_ascii, "%#.6g\t", *(reel+(nb_col*lig + col)));
				break;
			case SD_BAS_REEL8:
  
				sprintf(don_ascii, "%#.8g\t", *(dbreel+(nb_col*lig + col)));
				break;
			case SD_BAS_ASCII:
				pt_don = don_ascii;
				sprintf(pt_don, "\"");
				pt_don = pt_don + 1;
				tempo=((char *)donnee+(nb_col*lig + col)*taille_cell);
				longueur = strlen (tempo);
				memcpy (pt_don, tempo, longueur);
				pt_don = pt_don + longueur;
				sprintf(pt_don,"\" \0");
				break;
			default:
				retour = SD_BAS_ERREUR_ENCODE;
				sprintf (s_SD_BAS_ERREUR," erreur ecriture en ascii de donnee ");
				goto ERREUR;
			}

/*--------------------------------------------------------------*/
/* determiner significative de ce buffer DON_ASCII (sans \0)	*/
/*--------------------------------------------------------------*/
		longueur = strlen (don_ascii);

/*-----------------------------------------------------------------*/
/* SI longueur + taille significative de da_ascii + \n))\n\0 est   */
/* superieure a la taille allouee pour da_ascii, on se realloue	   */
/*		de la place	  			           */
/*-----------------------------------------------------------------*/
		// FT-2765
		if ( (longueur+5 + (unsigned long int) pt_ascii - (unsigned long int)&(**da_ascii)) > (SD_BAS_TAILLE_BUFFER*nb_bloc) )
		// ...FT-2765
			{
			nb_bloc = nb_bloc + 1;

			// FT-2765
			sauv_diff_deb = (unsigned long int) pt_deb_lig - (unsigned long int)&(**da_ascii);
			sauv_diff_cour = (unsigned long int) pt_ascii - (unsigned long int)&(**da_ascii);
			// ...FT-2765

			*da_ascii = (char *)realloc (*da_ascii, (SD_BAS_TAILLE_BUFFER*nb_bloc));
			if (*da_ascii== NULL) 
				{
				retour = SD_BAS_ERR_ALLOCA;
				sprintf (s_SD_BAS_ERREUR," erreur re-allocation memoire pour donnees ascii-bloc %d ", nb_bloc);
				goto ERREUR;
				}

			pt_deb_lig = *da_ascii + sauv_diff_deb;
			pt_ascii =  *da_ascii + sauv_diff_cour;
			}

/*----------------------------------------------------------------------*/
/* SI longueur + taille de ligne est > a SD_BAS_TAILLE_MAX_LIG caracteres	*/
/* A l'adresse courante  on ecrit \n dans le buffer da_ascii            */
/*----------------------------------------------------------------------*/
		// FT-2765
		if ( (longueur + (unsigned long int)pt_ascii - (unsigned long int)pt_deb_lig) >= SD_BAS_TAILLE_MAX_LIG)
		// ...FT-2765
			{
			*pt_ascii = '\n';
			pt_ascii = pt_ascii + 1;
			pt_deb_lig = pt_ascii;

			}

/*-----------------------------------------------------------------*/
/*	On recopie les donnees significatives de DON_ASCII dans le */
/*	buffer da_ascii a l'adresse pointeur		           */
/*-----------------------------------------------------------------*/
		strcpy (pt_ascii, don_ascii);
		pt_ascii = pt_ascii + longueur;
		}	/* 	FIN POUR (col)		*/

/*---------------------------------------------------------------*/
/*	Initialisation ) apres les donnees de la ligne	         */
/*	SI longueur de la ligne < SD_BAS_TAILLE_MAX_LIG alors       */
/*		on ecrit \n					 */
/*	SINON 					  		 */
/*		on ecrit \n)\n					 */
/*---------------------------------------------------------------*/
	if (lig != (nb_lig-1))
		{
		if ((pt_ascii - pt_deb_lig + 2) < (SD_BAS_TAILLE_MAX_LIG) )
			{
			strcpy (pt_ascii ,")\n");
			pt_ascii = pt_ascii + 2;
			pt_deb_lig = pt_ascii;
			}
		else
			{
			strcpy (pt_ascii ,"\n)\n");
			pt_ascii = pt_ascii + 3;
			pt_deb_lig = pt_ascii;
			}
		}
	else		/* on traite le cas de la derniere ligne */
		{
		if ((pt_ascii - pt_deb_lig + 3) < (SD_BAS_TAILLE_MAX_LIG) )
			{
			strcpy (pt_ascii ,"))\n");
			pt_ascii = pt_ascii + 3;
			pt_deb_lig = pt_ascii;
			}
		else 
			{
			if ((pt_ascii - pt_deb_lig + 2) < (SD_BAS_TAILLE_MAX_LIG) )
				{
				strcpy (pt_ascii ,")\n)\n");
				pt_ascii = pt_ascii + 4;
				pt_deb_lig = pt_ascii;
				}
			else
				{
				strcpy (pt_ascii ,"\n))\n");
				pt_ascii = pt_ascii + 4;
				pt_deb_lig = pt_ascii;
				}
			}

		}

	    }	/* FIN POUR (lig) */


retour =  SD_BAS_TraiteLigne(
			     fd,
			     indic_ecrit,
			     longueur,
			     da_ascii,
			     pt_ascii);
 if(retour != OK)
   goto ERREUR;


 if (indic_ecrit == 1) {   
   if(*da_ascii  != NULL)
     {
       free(da_ascii[0]);
       da_ascii = NULL;
       
     }
 }
/*---------------------------*/
/* Fin du traitement nominal */
/*---------------------------*/

return (OK);

/*----------------------*/
/* traitement degrade : */
/*----------------------*/
ERREUR :
        fd->num_buffer=-1;
        fd->nb_mot_cle=-1;
	if (fd->liste_mot_cle != NULL)
	{
	   free(fd->liste_mot_cle);
          fd->liste_mot_cle=NULL;

	}
	if (fd->ad_mot_cle != NULL)
	{
	  free(fd->ad_mot_cle);
	  fd->ad_mot_cle=NULL;

	}
	 if(fd->ad_donnee != NULL)
	{
	   free(fd->ad_donnee);
	   fd->ad_donnee=NULL;
        }

	return (retour);

}
/*
   Fin de l'operation SD_BAS_TraceEcritBuf
*/

/*****************************************************************************/
/*                                                                           */
/* OPERATION:  SD_BAS_TraiteLigne                                            */
/*                                                                           */
/* ROLE:      Ecrit une donnee dans le buffer             	             */
/*                                                                           */
/*                                                                           */
/*                                                                           */
/* CONTEXTE D'APPEL:                                                         */
/*              Interne a SD_BAS_TraceEcritBuf                               */
/*                                                                           */
/* VERSION:                                                                  */
/*									     */
/* CODE RETOUR : OK ou KO					             */
/*****************************************************************************/
t_SD_BAS_OK_KO SD_BAS_TraiteLigne(
t_SD_BAS_CpfFile * fd,
			  int indic_ecrit,
			  int longueur,
			  char ** da_ascii,
			  char * pt_ascii 
  )

   


{
 
 
  t_SD_BAS_OK_KO retour =OK;

/*-------------------------------*/
/* On ecrit \0 en fin de buffer	 */
/*-------------------------------*/
*pt_ascii='\0';

/*-----------------------------------------------------------------*/
/* Si l'utilisateur le desire on ecrit en fin de fichier le buffer */ 		/*-----------------------------------------------------------------*/
if (indic_ecrit == 1)
	{
/* 	On se positionne a la fin du fichier de donnees.				*/
	retour = lseek(fd->descr, 0L, SEEK_END);
	if (retour == -1)
		{
		retour = SD_BAS_PB_LSEEK;
		sprintf(s_SD_BAS_ERREUR,
		 "Dans SD_BAS_TraceAjoute, erreur position en fin de fichier");
		goto ERREUR;
		}

/* 	On ecrit le buffer dans le fichier */
	longueur = (int) strlen(*da_ascii);
	write (fd->descr, *da_ascii, longueur);

	} 	/* fin ecriture fichier */

/*------------------------------------------------------------*/
/* re-initialisations des pointeurs de la structure t_SD_BAS_CpfFile  */
/* ( buffers de lecture , liste de mots cles , longueurs ...) */ 
/*------------------------------------------------------------*/
if (indic_ecrit == 1) {   
	fd->num_buffer=-1;
	fd->nb_mot_cle=-1;
	if (fd->liste_mot_cle != NULL)
	{
	   free(fd->liste_mot_cle);
          fd->liste_mot_cle=NULL;

	}
	if (fd->ad_mot_cle != NULL)
	{
	  free(fd->ad_mot_cle);
	  fd->ad_mot_cle=NULL;

	}
	if (fd->ad_donnee != NULL)
	{
	   free(fd->ad_donnee);
	   fd->ad_donnee=NULL;
        }

	
}			



	
 return(OK);


ERREUR :
  return(retour);


}
/*****************************************************************************/
/*                                                                           */
/* OPERATION: SD_BAS_TraceMotCle                                                  */
/*                                                                           */
/* ROLE: Cette operation interne est un automate qui teste les caracteres    */
/*       < > fin_buffer blanc (' ' '\n') autre				     */	
/* 									     */
/*      								     */
/*                                                                           */
/* CONTEXTE D'APPEL:                                                         */
/*                                                                           */
/* VERSION:                                                                  */
/* CODE RETOUR : OK ou KO 						     */
/*****************************************************************************/
static t_SD_BAS_OK_KO SD_BAS_TraceMotCle( 
			     char car, 
			     e_SD_BAS_CpfEvt3 * evt3
			     )

   /* [E] caractere a tester */
  /* [S] evenement detecte */ 

{
/*---------------------------------------------------------------------------*/
/*		 	DEBUT DE L'OPERATION	        		     */
/*---------------------------------------------------------------------------*/
/* si  char = ' ' ou \n  	evt3 = ESPACE					*/
/* si  char = '<' 		evt3 = INFERIEUR				*/
/* si char = '>' 		evt3 = SUPERIEUR				*/
/* si char = \0  		evt3 = FIN     					*/
/* sinon 			evt3 = AUTRES  		      			*/

switch (car)
	{
	case (' ') :
		*evt3 = SD_BAS_AUTRES;
		break;
	case  ('\n') :
		*evt3 = SD_BAS_AUTRES;
		break;

	case '<' :
		*evt3 = SD_BAS_INFERIEUR;
		break;
	case '>' :
		*evt3 = SD_BAS_SUPERIEUR;
		break;
	case '\0' :
		*evt3 = SD_BAS_FIN;
		break;

	default:
		*evt3 = SD_BAS_AUTRES;
		break;
	}
return (OK);
}
/*
   Fin de l'operation SD_BAS_TraceMotCle
*/
/*************************************************************************/
/*                                                               	 */
/* OPERATION: SD_BAS_TraceRechercheCle				         */
/*                                                                   	 */
/* ROLE: Cette operation interne recherche un mot cle ds le buffer buff. */
/*  (de taille specifiee ds t_SD_BAS_CpfFile). on retourne mot cle,sa longueur   */ 	/*  et l'adresse dans le tableau du caractere suivant le mot cle .       */ 
/* NB: attention au cas ou le mot cle est coupe on renvoie un statut     */
/*  "non trouve" comme si rien n'avait pu etre detecte                   */
/*                                                                 	 */
/* ETAT = ( DEBUT DECODAGE <=> ATTENTE <  0				 */
/*          ATTENTE >			  1				 */
/*          FIN DECODAGE		  2	|			 */
/*          ERREUR Syntaxe (<...<) 	  SD_BAS_ERR_CROCHET| status de	 */
/*          ERREUR Donnee non presente	  SD_BAS_EOF| retour  	         */
/*          ERREUR fin donnee prematuree  SD_BAS_PAS_MC	|		 */
/*								         */
/* CARACTERE = <  >  AUTRE  FIN_FICHIER				 	 */
/*									 */
/*	Etat : CARACTERE     	<	>	AUTRE	FIN_FICHIER      */
/*	0			 1	 0	0	-1	         */
/*	1			-25	 2	1	-24              */
/*                                                                 	 */
/* CONTEXTE D'APPEL:                                               	 */
/*                                                                  	 */
/* VERSION:                                                       	 */
/*							  	         */
/*************************************************************************/
static t_SD_BAS_OK_KO SD_BAS_TraceRechercheCle(
					       char * buff,
					       int ad_deb,
					       char * mot_cle,
					       int * ad_suiv
					       )

     /* [E] buffer a scanner				   */
     /* [E] adr a partir de laquelle faut scanner buffer*/
     /* [S] mot cle suivant lu 			   */
     /* [S] adresse du premier caractere suivant mot cle*/

{
int	retour;			/* code retour */
char *	indice;		     	/* indice de parcourt du buffer */
int	nb_car;			/* nombre de caracteres du mot cle */
char	*ad;			/* adresse d'un caractere */
int	adr_deb_mot;		/* adresse debut de mot cle */
e_SD_BAS_CpfEtat3	etat;		/* etat de niveau 3 */
e_SD_BAS_CpfEvt3	evt3;		/* evenement de niveau 3 */
char	c;			/* caractere courant traite */
/*-----------------------------------------------------------------------*/
/*		 	DEBUT DE L'OPERATION	        		 */
/*-----------------------------------------------------------------------*/
/* Initialisation : On se positionne a l'adresse a partir de laquelle il */
/*		      faut scanner le buffer 		 	         */
/* On recherche le premier caractere < (strchr)				 */
/*-----------------------------------------------------------------------*/

indice =(char *) strchr ((char *)(buff+ad_deb), '<');

/* Si code retour de strchr est NULL (< non trouve), erreur rencontree,	*/
if (indice == NULL)
	{
	retour = SD_BAS_EOF;
	sprintf(s_SD_BAS_ERREUR,"erreur: pas de mots cle a partir de l adresse %d", 
				ad_deb); 
	goto ERREUR;
	}
 
/*--------------------------------------------------------------*/
/* On sauvegarde l'adresse relative  de debut du mot cle	*/
/*--------------------------------------------------------------*/
adr_deb_mot = (int)(indice - buff) + 1;

/*----------------------------------------------------------------------*/
/* On utilise un automate pour analyser les caracteres (en les comptant)*/
/*	on sort de l'analyse avec : 2 fin decodage ok			*/
/*				    -1 erreur syntaxe ( <    < )	*/
/*				    -3 On n'a pas assez de donnees pour */
/*				       analyser integralement le mot cle*/
/*----------------------------------------------------------------------*/
etat = 0;
nb_car = 0;

while (etat != 2 && etat != SD_BAS_PAS_MC && etat != SD_BAS_ERR_CROCHET)
	{
	ad = (char *)(indice + nb_car);
	c = *ad;
	nb_car = nb_car + 1;
	retour =SD_BAS_TraceMotCle(c,       	/* [E] caractere a tester */
			      &evt3);         	/* [S] evenement detecte */ 
	etat = s_autom3 [etat] [evt3];
	}
nb_car = nb_car -2;

/* si etat erreur : envoi message */
if (etat == SD_BAS_PAS_MC)
	{
	retour = SD_BAS_PAS_MC;
	sprintf (s_SD_BAS_ERREUR,"le mot cle n'est pas inclu entierement dans le buffer");
	goto ERREUR;
	}
if (etat == SD_BAS_ERR_CROCHET )
	{
	retour = SD_BAS_ERR_CROCHET;
	sprintf (s_SD_BAS_ERREUR,"2 crochets ouvrant < < ont ete detecte dans le mot cle");
	goto ERREUR;
	}

/* si mot cle trop long, erreur */
if (nb_car > SD_BAS_TAILLE_MAX_CLE)
	{
	retour = SD_BAS_CLE_TROP_LONG;
	sprintf (s_SD_BAS_ERREUR,"le mot cle est trop long");
	goto ERREUR;
	}

/* si decodage ok et si le mot cle n'est pas trop long,			*/
if (etat ==2 )
	{
/* 	sauvegarde du mot cle, de sa longueur, et de l'adresse du 1er	*/
/*	caractere suivant le mot cle					*/
	memcpy (mot_cle,(indice + 1), nb_car);
	*(mot_cle + nb_car) = '\0';
	*ad_suiv = adr_deb_mot + nb_car + 1;
	}

/*---------------------------*/
/* Fin du traitement nominal */
/*---------------------------*/
retour = OK;
return (retour);

/*----------------------*/
/* traitement degrade : */
/*----------------------*/
ERREUR :
return (retour);
}
/*
   Fin de l'operation SD_BAS_TraceRechercheCle
*/


/*****************************************************************************/
/*                                                                           */
/* OPERATION: SD_BAS_TraceLireFic                                                */
/*                                                                           */
/* ROLE: Cette operation interne lit dans le fichier  un buffer a partir     */
/*    d'une adresse specifiee par l'utilisateur d'un nombre d'octets specifie*/
/*      par l'utilisateur .                      		             */
/*         ( utilisation de  lseek , read ) 	     	        	     */
/*         ( buffer probable de 4096 octets)                                 */
/*                                                                           */
/*                                                                           */
/* CONTEXTE D'APPEL:                                                         */
/*                                                                           */
/* VERSION:                                                                  */
/* CODE RETOUR : OK ou KO 				  	             */
/*   " erreur lors du rewind sur le fichier de donnees "   	             */
/*   " erreur lors du fseek sur le fichier de donnees ( ad_deb =%d) "        */ 	/*   " erreur a la lecture du fichier de donnees  %s"                        */
/*****************************************************************************/
static t_SD_BAS_OK_KO SD_BAS_TraceLireFic(
			     t_SD_BAS_CpfFile * fd,
			     int ad_deb,
			     char * buff
			     )

      /* [E] structure decrivant fichier de donnees  */
     /* [E] adresse du premier octet a lire         */ 
     /* [E/S] buffer a remplir			  */

{

off_t offset; 	 /* offset dans le fichier de donnees  */
off_t offreel;	 /* offset reel obtenu apres lseek     */
int   nb_lus;  	 /* nombres d'octets lus               */
t_SD_BAS_OK_KO cr;      	 /* code retour 		       */	
/*---------------------------------------------------------------------------*/
/*		 	DEBUT DE L'OPERATION	        		     */
/*---------------------------------------------------------------------------*/
cr=OK;
/* on se positionne sur le debut du fichier ( rewind ) 		     */
offset=0;
offreel=lseek(fd->descr,offset,SEEK_SET);
/* SI erreur on sort "erreur lors du rewind du fichier de donnees "  */
if ( offreel == -1)
	{
	cr=SD_BAS_PB_LSEEK;
	sprintf(s_SD_BAS_ERREUR,"erreur lors du rewind du fichier %s",fd->nom_fichier);
	goto ERREUR;
	}
/* on se deplace de ad_deb octets dans le fichier (fseek) 		     */
offset=ad_deb;
offreel=lseek(fd->descr,offset,SEEK_SET);
/* SI erreur on sort " erreur lors du lseek du fichier donnees (ad_deb=%d)"  */  
if ( offreel == -1)
	{
	cr=SD_BAS_PB_LSEEK;
	sprintf(s_SD_BAS_ERREUR,"erreur lors du lseek sur le fichier %s",fd->nom_fichier);
	goto ERREUR;
	}
/* SI erreur on sort "erreur lors du fseek du fichier de donnees " */
/* on lit le buffer ( read )			 	             */
nb_lus=read(fd->descr,buff,fd->taille_buffer);
if (nb_lus == -1)
	{
	sprintf(s_SD_BAS_ERREUR,"erreur lors du read sur le fichier %s",
				fd->nom_fichier);
	cr=SD_BAS_PB_READ;
	goto ERREUR;
	}
*(buff + nb_lus)= '\0';
/* SI taille_ecrite <> taille buff on sort " erreur a l'ecriture du fichier de donnee ..."       */
if ( nb_lus != fd->taille_buffer)
	{
	fd->taille_buffer_lu=nb_lus;
	}
else
	{
	fd->taille_buffer_lu=fd->taille_buffer;
	}
/*------------------------------------*/
/* fin de l'operation en cas nominal  */
/*------------------------------------*/
return(OK);
/*------------------------------------*/
/* fin de l'operation en cas d'erreur */
/*------------------------------------*/
ERREUR:
return(cr);	

}
/*
   Fin de l'operation SD_BAS_TraceLireFic
*/


/*****************************************************************************/
/*                                                                           */
/* OPERATION: SD_BAS_TraceInitDate                                               */
/*                                                                           */
/* ROLE:   Recuperation de la date syteme sous la forme dd/mm/yy xxHyy       */
/*                                                                           */
/*                                                                           */
/* CONTEXTE D'APPEL:                                                         */
/*                                                                           */
/* VERSION:                                                                  */
/* CODE RETOUR : OK ou KO 				  	             */
/*                                                                           */
/*****************************************************************************/
 t_SD_BAS_OK_KO SD_BAS_TraceInitDate (char * date)

  /* date du jour a inscrire ds fic resultats */

{
    struct tm *lt;		/* differentes representations de la date */
    struct timeval t;

    void    *tz; 
    time_t	*tt;
    int	cr;		       /* CR d'erreur */

/* FA 184 */
    struct tm * CurrentTime;
/* ..FA 184 */
 
   /*-------------*/
   /* Initialiser */
   /*-------------*/
   strcpy(date, "");
   /*--------------------*/
   /* Recuperer la date  */
   /*--------------------*/
   cr=gettimeofday(&t, tz);

   /*---------------------*/
   /* SI pb alors warning */
   /*---------------------*/
   if(cr!=0)
   {
      strcpy(s_SD_BAS_ERREUR,"Probleme a la recuperation de la date courante.");
      goto ERREUR;
    }
   /*----------------------------------*/
   /* SINON convertir la date en local */
   /*----------------------------------*/
   else
   {
	tt=(time_t *)calloc((unsigned)sizeof(time_t), (unsigned)1);

	if(tt==NULL)
        {
	  strcpy(s_SD_BAS_ERREUR,
   "Probleme d'allocation detecte lors de la recuperation de la date systeme.");
          goto ERREUR;
        }
	else
	{
		/* recuperer le nombre de seconde depuis 01/01/1970 */
		(*tt)=t.tv_sec;
	
/* FA 184 */
			lt=localtime_r((const time_t *)tt,CurrentTime);
/* ..FA 184 */

                        

		/*---------------------*/
		/* SI pb alors warning */
		/*---------------------*/
		if (lt==NULL)
                {
		     strcpy(s_SD_BAS_ERREUR,
			"Probleme a la conversion de la date courante.");
		     goto ERREUR;
                 }

		/*--------------------------*/
		/* SINON ecrire le resultat */
		/*--------------------------*/
		else
		{
			sprintf (date, "%02d/%02d/%2d %2dH%2d", lt->tm_mday, 
					lt->tm_mon+1, lt->tm_year,
					lt->tm_hour,lt->tm_min); 
			/*-------------------------*/
			/* Liberer la structure tm */
			/*-------------------------*/
			free((char *)lt);
			free((char *)tt);
		}
	}
   }
  return(OK);

ERREUR:
  return(KO);
}
/*
   Fin de l'operation SD_BAS_TraceInitDate 
*/

/*****************************************************************************/
/*                                                                           */
/* OPERATION:  SD_BAS_TraceLireCaract                                            */
/*                                                                           */
/* ROLE:  cette operation externe est chargee de trouver le type et la taille*/
/*   d'une donnee  lue dans un fichier precedenment ouvert.                  */
/*   Il s'agit donc apres localisation du mot cle de lire le type de donnee  */
/*    puis de compter le nombres de cellules de ce type dans la ligne.       */
/*   Dans le cas d'une donnee de type SD_BAS_ASCII la taille maxi de la         */
/*   cellule est aussi indiquee a l'appelant.                                */
/*                                                                           */
/*                                                                           */
/* CONTEXTE D'APPEL:                                                         */
/* cette operation doit etre appele apres ouverture prealable du fichier     */
/* de donnees .						   		     */ 
/*                                                                           */
/* VERSION:                                                                  */
/* CODES RETOUR: OK ou KO						     */
/*   si KO l'erreur peut etre affiche via la variable globale s_SD_BAS_ERREUR     */
/*   " fichier de donnees  %s non ouvert"				     */ 
/*   " mot cle non present dans le fichier de donnees  %s"      	     */
/*   " aucun mot cle dans le fichier %s"			             */
/*   " taille du mot cle lue %s superieure a la limite %d ou egale a 0"      */ 
/*   " erreur a la lecture du fichier de donnees  %s"                        */
/*   " parenthese ouvrante non detectee (mot cle %s )                        */ 
/*   " parenthese fermante  non detectee (mot cle %s )                       */
/*   " nombre de niveaux de parentheses superieur a 2 ( mot cle %s )         */ 
/*   " % dimension du tableau non constante (mot cle %s)                     */
/*   " % dimension du tableau = 0     (mot cle %s)                           */
/*   " type de la donnee lue %s non reconnue (mot cle %s)                    */
/*   " erreur lors du decodage de la donnee lue %s" (mot cle)                */
/*****************************************************************************/

t_SD_BAS_OK_KO SD_BAS_TraceLireCaract( 
			 t_SD_BAS_CpfFile * fd, 
			 char * mot_cle,
			 t_SD_BAS_CpfType *type,
			 int * nb_lig,
			 int * nb_col, 	
			 int * taille_cell
			 )

      /* [E] structure decrivant fichier de donnees         */
   /* [E] mot cle a rechercher ds fichier terminee par \0*/
   /* [S] type de la donnee lue 			   */
 /* [S] nombre de lignes du tableau lu 		   */
/* [S] nombre de colonnes du tableau lu 	  	   */  
/* [S] taille maxi en octets d'un element donnee lue  */ 
		     /* [S] non significatif si le type est <>SD_BAS_ASCII    */  

{
t_SD_BAS_CpfMotCle mot_cle_lu;    /* mot cle courant lu dans le fichier     */
int	indic_mot_cle;     /* indicateur de recherche du mot cle     */
int	ad_mot_cle;        /* adresse du debut du mot cle courant    */	
int     ad_donnee;         /* adresse de donnee apres mot cle trouve */ 	
int	adresse_deb;       /* adresse  debut de recherche du mot cle */
t_SD_BAS_OK_KO	cr;	           /* code retour des operations	     */		
void	*don_lue;          /* donnees lue en par SD_BAS_TraceLireDonnee  */
int	i;	       /* indice de boucle */	
char    *don_asc; 
 char * message; 
 char * temp;
/*---------------------------------------------------------------------------*/
/*		 	DEBUT DE L'OPERATION	        		     */
/*---------------------------------------------------------------------------*/
/* initialisations  */
indic_mot_cle = SD_BAS_NON_TROUVE;

i=0;
ad_donnee=0;
cr=OK;
don_lue=NULL;

/* on verifie au prealable que le fichier est effectivement ouvert */
/* SINON on sort en erreur */
if ( fd ==NULL ) 

	{
	cr = SD_BAS_ERREUR_OUV;
	sprintf(s_SD_BAS_ERREUR,"Erreur ouverture du fichier de donnees ");
	goto ERREUR;
	}

/* SI la liste des mots cles est en memoire 			*/
if ( fd->nb_mot_cle != -1 ) 
	{
/*---------------------------------------------------------------------------*/
/* ALORS on scrute cette liste jusqu'a trouver le mot cle */
/*---------------------------------------------------------------------------*/
	while (i< fd->nb_mot_cle && indic_mot_cle == SD_BAS_NON_TROUVE )
		{
		if ( strncmp(mot_cle,(char *)(fd->liste_mot_cle + i),SD_BAS_TAILLE_MAX_CLE) == 0 )
			{
			indic_mot_cle = SD_BAS_TROUVE;
		
/*	   on recupere alors le pointeur sur la zone fichier apres ce mot cle */
			ad_donnee=*(fd->ad_donnee + i);
			strcpy(mot_cle_lu,(char *)(fd->liste_mot_cle +i));
			ad_mot_cle=*(fd->ad_mot_cle + i);
			}
		i++;
		}
	}
else
	{
/* SINON							*/	
/*   on se positionne au debut du fichier ( adresse_deb = 0)  */
	adresse_deb = 0;	
/*---------------------------------------------------------------------------*/
/*   on cherche le premier mot cle a partir de cette adresse (operation interne basee sur un automate) */
/*---------------------------------------------------------------------------*/
	cr = SD_BAS_TraceLireCle(fd,  /* [E] descriptor du fichier a lire         */
        	      adresse_deb,/* [E] adresse de debut de recherche        */
	              mot_cle_lu, /* [S] mot cle suivant lu avec \0)          */
		      &ad_donnee, /* [S] adres 1er caractere apres le mot cle */
		      &ad_mot_cle /* [S] adresse 1er caractere du mot cle     */
 		   	  );
/*    si erreur on sort */
	if (cr != OK && cr != SD_BAS_EOF) {
			goto ERREUR;
			}	
/*   si aucun mot cle trouve  erreur "aucun mot cle dans le fichier" */  
	if (cr == SD_BAS_EOF)
		{
		sprintf(s_SD_BAS_ERREUR,"Erreur fichier sans mot cle( %s) ",fd->nom_fichier);
		goto ERREUR;
		}
/*   TANT QUE  mot_cle non trouve &  pas erreur */
	while ( cr == OK  && indic_mot_cle == SD_BAS_NON_TROUVE )
		{
/*   	  si le mot cle est egal au mot cle demande alors mot_cle trouve */
		if ( strcmp(mot_cle,mot_cle_lu) == 0)
			{
			indic_mot_cle = SD_BAS_TROUVE;
			}
		else
		{
		  adresse_deb = ad_donnee;
/*---------------------------------------------------------------------------*/
/*      si mot_cle non trouve,chercher le mot cle suivant (operation interne */
/*	basee sur un automate) 	cette operation necessite : un pointeur sur  */ /*	la zone fichier ou l'on recherche le mot cle suivant 		     */
/*      cette operation ramene : le mot cle suivant et le pointeur sur       */
/*					    la zone fichier apres ce mot cle */
/*---------------------------------------------------------------------------*/
		  cr = SD_BAS_TraceLireCle(
		     fd,            /* [E] descriptor du fichier a lire      */
        	     adresse_deb,   /* [E] adresse de debut de recherche     */
	             mot_cle_lu,    /* [S] mot cle suivant lu (termine \0)   */
		     &ad_donnee,    /* [S] adres 1er caractere apres mot cle */
		     &ad_mot_cle    /* [S] adresse 1er caractere du  mot cle */
 		   	  		   );
		}
/*   FIN TANT QUE pas erreur & mot_cle non trouve */
		}
	}
/* FINSI */
if (cr != OK && cr != SD_BAS_EOF) 
	{
	goto ERREUR;
	}
/* 	SI fin de fichier on sort en erreur "mot cle non present */
if ( indic_mot_cle == SD_BAS_NON_TROUVE ) 
	{
	sprintf(s_SD_BAS_ERREUR,"Mot cle :%s non trouve dans le fichier :%s",mot_cle,fd->nom_fichier);
        cr=indic_mot_cle;
	goto ERREUR;
	}
/* on sauvegarde alors dans la structure fd les info sur le dernier mot cle */
strcpy(fd->dernier_mot_cle,mot_cle_lu);
fd->ad_dern_mot_cle = ad_mot_cle;
fd->ad_dern_donnee  = ad_donnee;
/*---------------------------------------------------------------------------*/
/* a partir du mot cle trouve,analyse syntaxiquement de la ligne jusqu'au    */
/*							    prochain mot cle */
/* (operation interne effectuant le decodage et donnant : le type de donnee ,*/
/* le nombres de lignes,le nombre de colonnes , la taille maxi d'un element, */
/* un pointeur sur le tableau lu					     */
/* ( ce tableau a ete alloue dynamiquement par cette operation) 	     */
/*---------------------------------------------------------------------------*/
cr=SD_BAS_TraceLireDonnee(
		fd,        /* [E] file descriptor du fichier a lire  */
        	ad_donnee, /* [E] adresse de debut de recherche      */
	        &don_lue,  /* [S] donnee lue			     */
		type,      /* [S] type de la donnee lue              */
		nb_lig,    /* [S] nombre de lignes du tableau lu     */
		nb_col,    /* [S] nombre de colonnes du tableau lu   */  
	        taille_cell/* [S] taille max en octet 1 element tab  */	
	      		);

/* si erreur de decodage on sort en erreur */
if (cr != OK ) {
	if (cr == SD_BAS_EOF ){
		sprintf(s_SD_BAS_ERREUR,"erreur fichier donnees incomplet (mot cle %s)",mot_cle);
		}
	goto ERREUR;
	}
//DETECTION DE DONNEE SOUS FORME DE FICHIER BINAIRE

 if (*type ==  SD_BAS_ASCII && *nb_lig == 1 && *nb_col == 2)
   {

 	don_asc = (char *)don_lue;
	message=(char*)malloc(SD_BAS_TAILLE_MAX_ASCII*sizeof(char));
	temp=(char*)malloc(SD_BAS_TAILLE_MAX_ASCII*sizeof(char));

	strcpy(message,don_asc + SD_BAS_TAILLE_MAX_ASCII);
  
	if(strncmp(message,"FICHIER",7) == 0)
	  { sscanf(message,"%s %s %d %d",temp,temp,nb_lig,nb_col);
	 
	  if(strncmp(temp,FICDOUBLE,7) == 0)
	    *type =  SD_BAS_FIC_BIN_DOUBLE;


	  else if (strncmp(temp,FICENTIER,7) == 0)
	    *type =  SD_BAS_FIC_BIN_ENTIER;
	  }

   }



/* liberation place memoire alloue par l'operation interne precedente  */
if (don_lue != NULL)
	{
	free(don_lue);
	don_lue=NULL;
	}
/*-------------------------------------------------------------------*/
/* fin de l'operation en cas nominal  				     */
/*-------------------------------------------------------------------*/
return(OK);
/*-------------------------------------------------------------------*/
/* fin de l'operation en cas d'erreur 				     */
/*-------------------------------------------------------------------*/
ERREUR:
/* liberation place memoire alloue par l'operation interne precedente  */
if (don_lue != NULL)
	{
	free(don_lue);
	don_lue=NULL;
	}
return(cr);

}
/*
   Fin de l'operation SD_BAS_TraceLireCaract
*/

/*****************************************************************************/
/*                                                                           */
/* OPERATION: SD_BAS_TraceLireDonnee                                       */
/*                                                                           */
/* ROLE: Cette operation interne analyse syntaxiquement les donnees stockees */
/* apres un mot cle jusqu'au prochain mot_cle du fichier ou EOF.	     */
/* Elle permet donc de determiner le type , la taille  de la donnee .        */
/*       ( ENTIER REEL OU SD_BAS_ASCII)            ( 2 dim maxi )		     */
/* Elle s'alloue de plus la place necessaire au stockage de cette donnees.   */
/* pour les donnees SD_BAS_ASCII , le stockage se fait de maniere contigue en   */
/* memoire dans un tableau de cellules de taille fixe (taille_max).          */
/*                                                                           */
/*                                                                           */
/*                                                                           */
/* CONTEXTE D'APPEL:                                                         */
/*                                                                           */
/* VERSION:                                                                  */
/* CODE RETOUR : OK ou KO 					             */
/*   " erreur a la lecture du fichier de donnees  %s"            	     */
/*   " parenthese ouvrante non detectee (mot cle %s )                        */ 
/*   " parenthese fermante  non detectee (mot cle %s )                       */
/*   " nombre de niveaux de parentheses superieur a 2 ( mot cle %s )         */ 
/*   " % dimension du tableau non constante (mot cle %s)                     */
/*   " % dimension du tableau = 0     (mot cle %s)                           */
/*   " type de la donnee lue %s non reconnue (mot cle %s)                    */
/*   " erreur lors du decodage de la donnee lue %s" (mot cle %s)             */
/*****************************************************************************/
static t_SD_BAS_OK_KO SD_BAS_TraceLireDonnee(
				t_SD_BAS_CpfFile * fd, 
				int ad_deb,
				void **don_lue,
				t_SD_BAS_CpfType * type_lu,
				int * nb_lig_lu, 
				int * nb_col_lu,
				int * taille_cell_lu
				)

           /* [E] structure fichier donnees  		  */
   /* [E] adresse de debut de recherche		  */
  /* [S] donnee lue 				  */
   /* [S] (SD_BAS_ENTIER4,SD_BAS_REEL8,SD_BAS_ASCII,SD_BAS_BLOC)*/
   /* [S] nombre de lignes du tableau lu 		  */
      /* [S] nombre de colonnes du tableau lu	  */  
  /* [S] taille max en octet 1 element tableau lu  */

{
t_SD_BAS_OK_KO 	cr;		    /* code retour */
int	num;		    /* numero du buffer courant */
int 	ad_deb_buff;        /* adresse de debut du buffer */
int     ad_deb_rel;         /* adres debut recherche relative buffer courant */
int     taille_buff_courant;/* taille du buffer lu */
char   *buff_courant;       /* buffer courant */ 
off_t   offset;	  	
off_t   offreel;
int     nb_buffer;	    /* nombre de buffer dans le fichier */
int     nb_lig;             /* nombre de lignes */
int     nb_elt;		    /* nombre d'elements */
int  	taille_maxi;	    /* taille maxi d'une donnee ascii */
e_SD_BAS_CpfEtat1   etat1;	    /* etat de l'automate de decodage */
t_SD_BAS_CpfType    type;	    /* type de la donnee */
int	nb_elt_alloues;	    /* nombre d'elements alloues */
int	ad_fin_decodage;    /* adresse dfin du decodage effectif du buffer */
/*---------------------------------------------------------------------------*/
/*		 	DEBUT DE L'OPERATION	        		     */
/*---------------------------------------------------------------------------*/
/* dans t_SD_BAS_CpfFile on prevoit un indicateur qui vaut zero si aucun buffer   */
/* n'a pas deja ete lu  sinon = le numero du buffer lu dans le fichier 	  */ 
/* on prevoit aussi taille buffer lu et l'allocation statique du buffer   */  
/* et le decalage entre 2 buffer consecutifs ( fixe a  taille_buff/2)     */ 

/*initialisations*/ 
cr=OK;
nb_lig = 1;
nb_elt = 0;
taille_maxi = 0;
etat1  = SD_BAS_DEBUTDEC; 
type=SD_BAS_INCONNU; 
nb_elt_alloues=0;

/* recherche alors le nombre de buffer dans le fichier */
/* on se positionne sur la fin du fichier ( rewind )   */
offset=0;
offreel=lseek(fd->descr,offset,SEEK_END);
/* SI erreur on sort "erreur lors d'un lseek du fichier de donnees "*/
if ( offreel == -1)
	{
	cr=SD_BAS_PB_LSEEK;
	sprintf(s_SD_BAS_ERREUR,"erreur lors du lseek sur le  fichier %s",fd->nom_fichier);
	goto ERREUR;
	}
offreel=lseek(fd->descr,offset,SEEK_CUR);
/* SI erreur on sort "erreur lors d'un lseek du fichier de donnees " */
if ( offreel == -1)
	{
	cr=SD_BAS_PB_LSEEK;
	sprintf(s_SD_BAS_ERREUR,"erreur lors du lseek sur le  fichier %s",fd->nom_fichier);
	goto ERREUR;

	}
if ( offreel <= ad_deb)
	{
	cr=SD_BAS_ERR_ADRESSE;
	sprintf(s_SD_BAS_ERREUR,"erreur adresse de recherche en dehors du fichier (%d)",ad_deb);
	goto ERREUR;
	}

nb_buffer = (offreel / (fd->taille_buffer/2) ) + 1; 

/* on commence par calculer le premier buffer a lire par : num = 2 * ad_deb/(fd->taille buffer) */
num= (ad_deb/(fd->taille_buffer/2));
ad_deb_buff = num * (fd->taille_buffer /2);

/* SI indicateur != num */
if ( num != fd->num_buffer)
{
/* lecture buffer num a partir de adres num*fd->taille_buffer/2 et de taille*/
/*     fd->taille_buffer*2 (operation interne )			*/
	cr=SD_BAS_TraceLireFic(
		    fd,           /* [E] descripteur du fichier de donnees  */
	            ad_deb_buff,  /* [E] adresse du premier octet a lire    */ 
        	    fd->buffer    /* [E/S] buffer a remplir                 */
		   );
/*   SI erreur on sort 	*/
	if (cr != OK)
		{
		goto ERREUR;
		}
/* initialisation du numero de buffer */
	fd->num_buffer=num;
	buff_courant = fd->buffer;
	taille_buff_courant = fd->taille_buffer_lu;	
}
else
{
/* SINON tab = fd ->ad_buffer 	*/
	buff_courant = fd->buffer;
	taille_buff_courant = fd->taille_buffer_lu;	
}

/* calcul de l'adres relative du debut recherche dans le buffer  */
/* ad_deb_buff = ad_deb - num* fd->taille buffer/2  	*/
ad_deb_rel = ad_deb - ad_deb_buff;


/* TANT QUE statut <> lu et non EOF  	*/
while ( etat1 != SD_BAS_FINDEC && cr != SD_BAS_EOF && taille_buff_courant > 0)
{
/* lecture donnee ds buffer buff on retourne le tableau des donnees qui*/
/* ont ete lues (tableau eventuellement incomplet) , taille tableau lu,*/
/* le type ,dimension1 ,  nb elt , la taille maxi , si incomplet le    */
/*    statut de l'automate est retourne (operation interne)            */ 
/*   NB: attention au cas ou la donnee peut etre coupe                 */ 
   cr=SD_BAS_TraceDecodedon(
	 taille_buff_courant,/* [E] taille buffer a scanner en octets  */
         buff_courant,       /* [E] buffer a scanner 		         */
	 ad_deb_rel,	     /* [E] adres debut  scanner le buffer     */
	 &type,      	     /* [E/S] type donnee lue (SD_BAS_INCONNU 1er appel) */
         &nb_lig,    	     /* [E/S] nomb lig du tableau lu ( 0 du 1er appel) */
         &nb_elt,            /* [E/S] nb elements du tableau lu(0 1er appel) */	
         &nb_elt_alloues,    /* [E/S] nb elements du tableau lu (0 1er appel) */
         &taille_maxi,       /* [E/S] taill max en octet d'un elt du tab lu   */
			     /* significatif  donnees SD_BAS_ASCII */	
	 don_lue,            /* [E/S] donnee lue */
         &etat1,             /* [E/S] etat auto ( = DEBUT_DECODAGE 1er appel) */
         &ad_fin_decodage    /* [E/S] adres de fin decodage effectif buffer */
		     );
	
	
/* si erreur on sort */
	if (cr != OK && cr != SD_BAS_EOF)
		{
		goto ERREUR;
		}

/*  Si mot_cle non trouve et non EOF  on incremente le numero de buffer	*/
	if ( cr == SD_BAS_EOF && num+1< nb_buffer)
	{
/*------------------------------------------------------------*/
/* 		lecture du buffer num a partir de l'adresse   */ 
/*  num*fd->taille_buffer/2 et de taille 		      */
/*     	fd->taille_buffer*2 (operation interne )	      */
/*------------------------------------------------------------*/
	   num=num+1;
	   ad_deb_buff = num * (fd->taille_buffer /2);
	   cr=SD_BAS_TraceLireFic(
		  fd,               /* [E] descripteur le fichier de donnees  */
	          ad_deb_buff,      /* [E] adresse du premier octet a lire */ 
        	  fd->buffer        /* [E/S] buffer a remplir */
		  	 );
/*   SI erreur on sort 	*/
		if (cr != OK)
			{
			goto ERREUR;
			}
/* initialisation du numero de buffer */
		fd->num_buffer=num;
/*      		on met a jour les variables locales  	*/
		buff_courant = fd->buffer;
		taille_buff_courant = fd->taille_buffer_lu;
/* calcul de l'adresse relative du debut de la recherche dans le buffer    */
/* 	ad_deb_buff = ad_deb_buff(donnee par l'operation precedente) -     */
/* fd->taille buffer/2  ( decalage entre 2 buffers)	                   */
		ad_deb_rel = ad_fin_decodage - (fd->taille_buffer/2);			
		} 		
	}
/* FIN TANT QUE statut <> lu et non EOF	*/

/*	si statut <> lu et EOF "erreur fichier donnees incomplet (mot cle %s) */
if (  etat1 != SD_BAS_FINDEC && etat1 != SD_BAS_DEBUTDEC && (cr == SD_BAS_EOF || taille_buff_courant == 0) )
	{
	cr = SD_BAS_EOF;
	goto ERREUR;
	}
else
	{
	if (etat1 == SD_BAS_DEBUTDEC && (cr == SD_BAS_EOF || taille_buff_courant == 0) )
		{
		type=SD_BAS_BLOC;
		}
	}
/*-------------------------------------------------------------------*/
/* fin de l'operation en cas nominal  				     */
/*-------------------------------------------------------------------*/
*nb_lig_lu=nb_lig;
*nb_col_lu= nb_elt/nb_lig;
*type_lu = type;
*taille_cell_lu=taille_maxi;
return(OK);
/*-------------------------------------------------------------------*/
/* fin de l'operation en cas d'erreur 				     */
/*-------------------------------------------------------------------*/
ERREUR:
*nb_lig_lu=nb_lig;
*nb_col_lu= nb_elt/nb_lig;
*type_lu = type;
*taille_cell_lu=taille_maxi;

/* FA169 */
printf ("Erreur de SD_BAS_TraceLireDonnee sur %s.\n", fd->dernier_mot_cle);

return(cr);	
/*---------------------------------------------------------------------------*/
/*		 	FIN DE L'OPERATION	        		     */
/*---------------------------------------------------------------------------*/
}
/*
   Fin de l'operation SD_BAS_TraceLireDonnee
*/


/*****************************************************************************/
/*                                                                           */
/* OPERATION: SD_BAS_TraceAnalyse                                               */
/*                                                                           */
/* ROLE: Cette operation interne est un automate qui teste les caracteres    */
/*       ( , ) , AUTRE , <  , BLANC ou TAB ou  \N		             */
/* 									     */
/*      								     */
/*                                                                           */
/* CONTEXTE D'APPEL:                                                         */
/*                                                                           */
/* VERSION:                                                                  */
/* CODE RETOUR : OK ou KO 						     */
/*****************************************************************************/
static t_SD_BAS_OK_KO SD_BAS_TraceAnalyse( 
			     char car,
			     e_SD_BAS_CpfEvt1 * evt1
			       )

               /* [E] caractere a tester */
        /* [S] evenement detecte */ 
{
/*---------------------------------------------------------------------------*/
/*		 	DEBUT DE L'OPERATION	        		     */
/*---------------------------------------------------------------------------*/
/* si  char = '('  evt1 = 0 					*/
if (car == '(' )
	{
	*evt1 = SD_BAS_PAR_OUV;
	}
else
	{
/* sinon si  char = ')' evt1 = 1 				*/
	if (car == ')' )
		{
		*evt1 = SD_BAS_PAR_FERM;
		}
	else
		{
/* 	   sinon si char = '<' evt1 = 3 			*/
		if (car == '<' )
			{
			*evt1=SD_BAS_INF;
			}
		else
			{
/* 		 sinon si char = BLANC ou TAB ou n evt1= 4 	*/
			if (car==' ' || car=='	' || car=='\n')
				{
				*evt1=SD_BAS_BLANC;
				}
			else
				{		
/*		       sinon evt1 = 2 				*/
				*evt1=SD_BAS_DONNEE;
				}
/*			fin si					*/
			}
/*		 fin si						*/
		}
/*	   fin si						*/
	}
 /* fin si							*/		return(OK);
}
/*
   Fin de l'operation SD_BAS_TraceAnalyse
*/
/*****************************************************************************/
/*                                                                           */
/* OPERATION: SD_BAS_TraceDecodedon                                              */
/*                                                                           */
/* ROLE:  Cette operation interne  decode une donnee sous la forme           */
/* ENTIER: (( 1 2 24 ) ( 4 5 6 )) ou REEL: ((1. 2. 24.) (4. 5. 6.))          */
/* ASCII: (( "1" "2" "24" ) ( "4" "5" "6" ))                                 */
/* BLOC ( donnee vide )							     */
/* cette operation commence par rechercher la premiere donnee pour determiner*/
/* le type de donnee puis decode  chaque donnee , alloue la place necessaire */
/* et parcourt ainsi tout le buffer					     */
/*   NB: Si la donnee est coupee on rend a l'appelant le statut "en cours"   */
/* ainsi qu'un pointeur sur le caractere suivant la derniere donnee lus et   */
/* l'etat de l'automate .			                             */
/* la place est alloue par cette operation mais l'appelant se charge de la   */
/* liberer si besoin.							     */
/*                                                                           */
/*                                                                           */
/* ETAT = ( DEBUT DECODAGE <=> ATTENTE ((	0			     */
/*          ATTENTE (				1			     */
/*	    ATTENTE DONNEE			2			     */
/*	    ATTENTE DONNEE ou )			3			     */
/*	    ATTENTE ) ou (   			4			     */
/* 	    FIN decodage   			5			     */
/*	    BLOC				6                            */
/*	    ERREUR parenthese	                11			     */
/*	    ERREUR fin de donne prematuree      13			     */

/* CARACTERE = (							     */
/*		)							     */
/*		AUTRE							     */
/*		<       						     */
/*		BLANC ou TAB ou \N				             */
/*	etat: CARACTERE  (    )    AUTRE <     BLANC ou TAB ou \n   	     */
/*	0		 1   11   11     6        0			     */
/*	1		 2   11   11    13  	  1			     */
/*	2		11    4    3    13        2			     */
/*	3		11    4    3    13        3			     */
/*	4		 2    5   11    13 	  4 			     */
/*									     */
/* dans les etats 2 et 3 si l'evenement  = AUTRE on passe a un nouvel        */
/*   automate	( remarque :si 2 dimensions on passe de l'etat 4 a l'etat 2  */
/*									     */
/* ETAT1 =  Debut decodage donnee			0		     */
/*	    attente point(reel) ou blanc (entier)	1		     */
/*          attente " 					2		     */
/*          attente BLANC ou TAB ou \n			3		     */
/*	    ENTIER                	 		4	             */
/*	    chaine de caracteres			5		     */
/*          REEL 					6		     */
/*	    erreur donnee				11		     */
/* CARACTERE1 = .							     */
/*              "							     */
/*		BLANC ou TAB ou \n					     */
/*		CHIFFRE							     */
/*		SIGNE							     */
/*              AUTRE							     */
/*									     */
/*	etat :caractere1 . 	" BLANC ou TAB ou \n ou ) CHIFFRE SIGNE AUTRE*/
/*	0	         3 	2          11              1      1     11   */
/*      1		 3     11           4              1     11     11   */
/*	2                2      5           2 		   2      2      2   */
/*	3		 3     11           6              3      3      3   */
/* CONTEXTE D'APPEL:                                                         */
/*                                                                           */
/* VERSION:                                                                  */
/* CODE RETOUR : OK ou KO 						     */
/*	"ERREUR donnee non presente ( mot cle %s)"			     */
/*	"erreur parenthese ( mot cle %s)"				     */
/*	"erreur fin de donnee prematuree ( mot cle %s)			     */
/*      "erreur type de donne non reconnu"  				     */
/*	"erreur de decodage de la donne (mot cle %s) "			     */ 
/*****************************************************************************/
static t_SD_BAS_OK_KO SD_BAS_TraceDecodedon( 
			       int taille_buf,
			       char buff[],
			       int ad_deb,
			       t_SD_BAS_CpfType * type_lu,
			       int * nb_lig_lu,
			       int * nb_elt_lu,
			       int * nb_elt_alloc,
			       int * taille_cell_lu,
			       void ** don_lue,
			       e_SD_BAS_CpfEtat1 * etat1, 
			       int * ad_fin_dec)

     /* [E] taille du buffer a scanner en octets  */
      /* [E] buffer a scanner */
       /* [E] adr a partir de laquelle faut scanner buffer */
   /* [E/S] type donnee lue (=SD_BAS_INCONNU lors du 1appel) */
		       /* SD_BAS_ENTIER4 SD_BAS_REEL8 SD_BAS_ASCII SD_BAS_BLOC  EN SORTIE */
    /* [E/S] nb lignes du tableau lu ( = 0 lors 1er appel) */
    /* [E/S] nb elements tableau lu ( = 0 lors 1er appel) */
  /* [E/S] nb elements tableau lu ( = 0 lors 1er appel) */
/*[E/S] taill max en octet d'un elt du tab lu 1er appel*/
		       /* significatif exclusivement pour donnees SD_BAS_ASCII */	
  /* [E/S] donnee lue */
   /* [E/S] etat l'auto ( = DEBUT_DECODAGE  1er appel )*/
    /* [E/S] adresse fin du decodage effectif du  buffer */

{

int 	indice;          /* indice de parcourt du buffer */
t_SD_BAS_CpfType	type;    /* type de la donnee decodee */
int	nb_lig;          /* nombre de lignes de la donnee */
int 	nb_elt;          /* nombre d'elements de la donnee */
int	taille_max;      /* taille max d'une cellule SD_BAS_ASCII */
int	cr;	         /* code retour de la fonction */
t_SD_BAS_OK_KO   code;            /* code retour */  
char    car;	         /* caractere courant traite */
e_SD_BAS_CpfEtat1 etat1_prec;    /* etat precedent de niveau 1 */
e_SD_BAS_CpfEvt1    evt1;        /* evenement de niveau 1 detectee */
e_SD_BAS_CpfEtat2   etat2;       /* etat de niveau 2 ( determination du type ) */
e_SD_BAS_CpfEvt2    evt2;        /* evenement de niveau2			  */
int    ad_deb_do;        /* adresse de debut de la donnee courante */
long int *entier;        /* pointeur sur donnees entieres */
double   *reel;          /* pointeur sur donnees reelles */
char     *asc;           /* pointeur sur donnees ascii */ 
int    nb_elt_alloues;   /* nombres d'elements alloues */ 
int	taille;	         /* taille d'une donnee a decoder */		
int	nb_col;	         /* nombre de colonnes */
char NaNQ[5];
char MoinsNaNQ[6];
int INaN,INaNMoins;
/*---------------------------------------------------------------------------*/
/*		 	DEBUT  DE L'OPERATION	        		     */
/*---------------------------------------------------------------------------*/	
/* initialisations */
INaN=0;		
NaNQ[0]='\0';
strcpy(NaNQ,"NaNQ");
INaNMoins=0;							     
MoinsNaNQ[0]='\0';
strcpy(MoinsNaNQ,"-NaNQ");
cr=OK;
indice=ad_deb;
type= *type_lu;
nb_lig= *nb_lig_lu;
nb_elt= *nb_elt_lu;
taille_max = *taille_cell_lu;
nb_elt_alloues= *nb_elt_alloc;
taille=0;
if (type == SD_BAS_ASCII)
	{
	asc=(char *) *don_lue;
	}
else
	{
	if (type == SD_BAS_REEL8)
		{
		reel=(double *) *don_lue;
		}
	else
		{
		if (type == SD_BAS_ENTIER4)
			{
			entier=(long int *) *don_lue;
			}
		else
			{
			asc = (char *)malloc(SD_BAS_TAILLE_MAX_ASCII);
			if (asc == NULL )
				{
				sprintf(s_SD_BAS_ERREUR,"Erreur d'allocation lors de la lecture de la donnees");
				cr=SD_BAS_ERR_ALLOCA;
				goto ERREUR;
				}
			reel=(double *)asc;
			entier=(long int *)asc;
			}
		}
	}

/*--------------------------------------------------------------------*/
/* TANT QUE  non erreur et non fin decodage et indice < taille buffer */
/*--------------------------------------------------------------------*/
while( *etat1 >= 0 && *etat1 != SD_BAS_FINDEC && indice < taille_buf)
	{
/* 	car = buff[indice]						        */
/* 	etat1_prec=etat1							*/
	car = buff[indice];	
	etat1_prec=*etat1;
/*----------------------------------------------------------------------*/
/* 	fonction interne testant ( ) < BLANC ou TAB ou n AUTRE 		*/
/*----------------------------------------------------------------------*/
	code=SD_BAS_TraceAnalyse(
			car,          /* [E] caractere a tester */
			&evt1);         /* [S] evenement detecte */ 
		       

	*etat1 = s_autom1[*etat1][evt1];

/*      si erreur on affiche le messsage associe et on sort	*/	
	if (*etat1 > 10)
		{
		switch (*etat1) {
			case SD_BAS_ERR11:cr = SD_BAS_ERR_FORME_FIC;
				sprintf(s_SD_BAS_ERREUR,"erreur: forme donnee non coherente");  
				break;
			case SD_BAS_ERR13:cr = SD_BAS_ERR_FIN_DO;
				sprintf(s_SD_BAS_ERREUR,"erreur: fin de donnee prematuree"); 
				break;
			default:cr = KO;
				sprintf(s_SD_BAS_ERREUR,"erreur automate1 non reconnue");
				break; 
			}
		goto ERREUR;
		}
/*	si etat1= 2 et etat1_prec=4 			*/
/*			dim2 = dim2 + 1			*/
	if ( *etat1 == SD_BAS_ETAT12 && etat1_prec == SD_BAS_ETAT14)
		{
		nb_lig=nb_lig + 1;
		}
/* sauvegarde ad_deb_do */
	ad_deb_do=indice+1;
	if ( *etat1 == SD_BAS_BLOCVIDE )
		{
		type=SD_BAS_BLOC;
		nb_lig=1;
		nb_elt=0;
		*etat1 = SD_BAS_FINDEC;
		}
/*	SI (etat1=3)					*/
	if ( *etat1 == SD_BAS_ETAT13)
		{
/*		initialisation etat2					        */
		etat2= SD_BAS_DEBUTYPE;
/*		sauvegarde ad_deb_do						*/ 
		ad_deb_do=indice;
/*---------------------------------------------------------------------------*/
/*TANT QUE etat2 non REEL et non ENTIER et non ASCII et indice < taille buff */
/*---------------------------------------------------------------------------*/
	       INaN=0;
	       INaNMoins=0;
		while ( etat2 != SD_BAS_REEL && etat2 != SD_BAS_ENTIER && etat2 != SD_BAS_ASC && indice <taille_buf )
			{
/*-----------------------------------------------------------------------*/
/*		  fonction interne testant . " CHIFFRE SIGNE BLANC AUTRE */
/*-----------------------------------------------------------------------*/

			code=SD_BAS_TraceType(
			     car,       /* [E] caractere a tester */
			     &evt2         /* [S] evenement detecte */ 
		        	);
                        if ((evt2==SD_BAS_AUTRE)&&(car==NaNQ[INaN]))
                        {
			   evt2=SD_BAS_CHIFFRE; 

			   INaN=INaN+1;
			 }
                        if ((evt2==SD_BAS_AUTRE)&&(car==MoinsNaNQ[INaNMoins]))
                        {
			   evt2=SD_BAS_CHIFFRE; 
			   INaNMoins=INaNMoins+1;
			 }
			if (((INaN==4)||(INaNMoins==5))&& (evt2==SD_BAS_ESPACE))
			    etat2=SD_BAS_REEL;
			else                    
			   etat2=s_autom2[etat2][evt2];


/*      si erreur on affiche le messsage associe et on sort	*/	
			if (etat2 > 10)
				{
				switch (etat2) {
					case SD_BAS_ERR21:cr = SD_BAS_ERR_TYP;
					sprintf(s_SD_BAS_ERREUR,"erreur: type de donnee non coherente");  
					break;
				default:cr = KO;
					sprintf(s_SD_BAS_ERREUR,"erreur automate2 non reconnue");
					break; 
					}
				goto ERREUR;
				}
/*		  l'indice est incremente					*/
			indice ++;
			car = buff[indice];	
			}
/*----------------------------------------------------------------------*/
/*	FIN TANT QUE non erreur et non REEL et non ENTIER et non ASCII  */
/*----------------------------------------------------------------------*/
/* SI on arrive en fin de buffer sans avoir decode la donnee */
		if ( indice == taille_buf && (etat2 != SD_BAS_REEL || etat2 != SD_BAS_ENTIER || etat2 != SD_BAS_ASC))
		    {
			cr= SD_BAS_EOF;
		    }
		else
		   {
  		   if (etat2 == SD_BAS_ASC || type == SD_BAS_ASCII )
			{
			if (etat2 != SD_BAS_ASC || (type != SD_BAS_ASCII && type != SD_BAS_INCONNU))
				{
				sprintf(s_SD_BAS_ERREUR,"Erreur: melange de type dans une meme donnee");
				cr=SD_BAS_ERR_ASCII;
				goto ERREUR;
				}
			type=SD_BAS_ASCII;
/*-----------------------------------------------------------------------*/
/* si SD_BAS_ASCII  allocation et decodage entre ad_deb+1 et ad_fin -1	 */
/*( verification que la taille est inferieure a la taille max SD_BAS_ASCII) */
/*-----------------------------------------------------------------------*/
			taille= (indice) - (ad_deb_do) -2 + 1;
			if (taille  >= SD_BAS_TAILLE_MAX_ASCII)
				{
				sprintf(s_SD_BAS_ERREUR,"donnee ascii de taille sup a SD_BAS_TAILLE_MAX_ASCII (%d)",taille);
				cr=SD_BAS_ERR_ASCII;
				goto ERREUR;
				} 
/* 		allocation de SD_BAS_NB_ALLOC element reel entier et ASCII */
/*		par un realloc d'un ascii ( taille max) et sscanf(%s)	*/
			if (nb_elt_alloues <= nb_elt)
				{
				asc = (char *)realloc(asc,SD_BAS_TAILLE_MAX_ASCII*(SD_BAS_NB_ALLOC + nb_elt_alloues));
				if (asc != NULL )
					{
					nb_elt_alloues = SD_BAS_NB_ALLOC + nb_elt_alloues;
					}
				else
					{
					sprintf(s_SD_BAS_ERREUR,"Erreur d'allocation lors de la lecture de la donnees");
					cr=SD_BAS_ERR_ALLOCA;
					goto ERREUR;
					}
				}

				strncpy(asc + (nb_elt*SD_BAS_TAILLE_MAX_ASCII),buff+ad_deb_do+1,taille-1);
				*(asc + (nb_elt)*SD_BAS_TAILLE_MAX_ASCII +taille -1 )='\0';
/*			   taille_max = sup (taille_max,taille)     */
			taille_max=maxi(taille,taille_max);

/*		nbelt = nbelt + 1						*/
			nb_elt=nb_elt +1;
			}
		   else
			{
			 indice--;
/*----------------------------------------------------------------------*/
/*	si REEL allocation et decodage entre ad_deb et ad_fin-1   	*/
/*		par un realloc d'un reel et sscanf(%lf)			*/
/*----------------------------------------------------------------------*/
			if (etat2 == SD_BAS_REEL || type == SD_BAS_REEL8)
				{
				if (etat2 != SD_BAS_REEL || (type != SD_BAS_REEL8 && type != SD_BAS_INCONNU))
					{
					sprintf(s_SD_BAS_ERREUR,"Erreur: melange de types dans une meme donnee");
					cr=SD_BAS_ERR_REEL;
					goto ERREUR;
					}
				type = SD_BAS_REEL8;				
/* 			allocation de SD_BAS_NB_ALLOC element reel  */
				if (nb_elt_alloues <= nb_elt)
					{
					reel=(double *)realloc(reel,sizeof(double)*(SD_BAS_NB_ALLOC + nb_elt_alloues));
					if (reel != NULL )
						{
						nb_elt_alloues = SD_BAS_NB_ALLOC + nb_elt_alloues;
						}
					else
						{
						sprintf(s_SD_BAS_ERREUR,"Erreur d'allocation lors de la lecture de la donnees");
						cr=SD_BAS_ERR_ALLOCA;
						goto ERREUR;
						}
					}
                               if ((INaN==4)||(INaNMoins==5))
				 reel[nb_elt]=REELMAX;
                               else
                               {
				code=sscanf(buff+ad_deb_do," %lf",reel+nb_elt);
				if (code != 1)
					{
					sprintf(s_SD_BAS_ERREUR,"Erreur de decodage de la donnee reelle (element %d)",nb_elt);
					cr=SD_BAS_ERR_REEL;
					goto ERREUR;
					}
                                  }
				}
			else							
				{
				if (etat2 != SD_BAS_ENTIER || (type != SD_BAS_ENTIER4 && type != SD_BAS_INCONNU))
					{
					sprintf(s_SD_BAS_ERREUR,"Erreur: melange de type dans une meme donnee");
					cr=SD_BAS_ERR_ENTIER;
					goto ERREUR;
					}
				type = SD_BAS_ENTIER4;
/*----------------------------------------------------------------------*/
/*	si ENTIER allocation et decodage entre ad_deb et ad_fin -1	*/
/*		par un realloc d'un entier4 et sscanf(%ld)		*/
/* 			allocation de SD_BAS_NB_ALLOC element entier       */
/*----------------------------------------------------------------------*/
				if (nb_elt_alloues <= nb_elt)
					{
					entier=(long int *)realloc(entier,sizeof(long int)*(SD_BAS_NB_ALLOC+nb_elt_alloues));
					if (entier != NULL )
						{
						nb_elt_alloues = SD_BAS_NB_ALLOC + nb_elt_alloues;
						}
					else
						{
						sprintf(s_SD_BAS_ERREUR,"Erreur d'allocation lors de la lecture de la donnees");
						cr=SD_BAS_ERR_ALLOCA;
						goto ERREUR;
						}
					}
				code =sscanf(buff+ad_deb_do," %ld",entier+nb_elt);
				if (code != 1)
					{
					sprintf(s_SD_BAS_ERREUR,"Erreur de decodage de la donnee entier(element %d)",nb_elt);
					cr=SD_BAS_ERR_ENTIER;
					goto ERREUR;
					}
				}
/*		nbelt = nbelt + 1						*/			
			nb_elt=nb_elt +1;
			}
/* FIN SI on arrive en fin de buffer sans avoir decode la donnee */
/* on se positionne sur le dernier caractere de la donnee 	*/
		   indice--;
		   }
		*etat1 = SD_BAS_ETAT12;
/* FIN SI (etat1=3)					*/
		}
/*---------------------------------------------------------------------------*/
/*	FIN SI									*/
/*---------------------------------------------------------------------------*/
/*      l'indice est incremente						  	*/
	indice++;
	}
/*---------------------------------------------------------------------------*/
/* FIN TANT QUE  non erreur et non fin decodage et indice < taille buffer 	*/
/*---------------------------------------------------------------------------*/
if (indice == taille_buf)
	{
	cr=SD_BAS_EOF;
	}
if (*etat1 == SD_BAS_FINDEC)
	{ 
/* nb_col = nbelt  nb_lig						  	*/
	nb_col = (nb_elt/nb_lig);
/* SI nb_lig . nb_col <> nbelt alors erreur dans le nombre d'elements d'une ligne*/
	if ( (nb_lig * nb_col) != nb_elt)
		{
		sprintf(s_SD_BAS_ERREUR,"Nombre d'elements decodes incoherent (nb_elt:%d nb_lig:%d)",nb_elt,nb_lig);
		cr=SD_BAS_ERR_DIM;
		goto ERREUR;
		}
	}
/*---------------------------------------------------------------------------*/
/* nb_elt_lu = nb_elt							     */
/* nb_lig_lu = dim2 							     */
/* etat1 = etat1							     */
/* type_lu  = type							     */
/* taille_cell_lu= SD_BAS_REEL8 ou SD_BAS_ENTIER4 ou SD_BAS_ASCII( taille max )       */
/* ad_fin_dec= ad_fin  ( pour lecture du prochain buffer si besoin) 	     */  
/*---------------------------------------------------------------------------*/
*type_lu=type;
*nb_lig_lu=nb_lig;
*nb_elt_lu=nb_elt;
*taille_cell_lu=taille_max;
*ad_fin_dec=ad_deb_do;
*nb_elt_alloc=nb_elt_alloues;
if (type == SD_BAS_ASCII)
	{
	*don_lue=(void *)asc;
	}
else
	{
	if (type == SD_BAS_REEL8)
		{
		*don_lue=(void *)reel;
		}
	else
		{
		*don_lue=(void *)entier;
		}
	}
/*---------------------------------------------------------------------------*/
/* fin de l'operation en cas nominal  				     */
/*---------------------------------------------------------------------------*/
return(cr);
/*---------------------------------------------------------------------------*/
/* fin de l'operation en cas d'erreur 				     */
/*---------------------------------------------------------------------------*/
ERREUR:
/* affectation pour pouvoir liberer la place allouee ( par l'appelant )*/
if (type == SD_BAS_ASCII)
	{
	*don_lue=(void *)asc;
	}
else
	{
	if (type == SD_BAS_REEL8)
		{
		*don_lue=(void *)reel;
		}
	else
		{
		*don_lue=(void *)entier;
		}
	}
printf ("%s\n", s_SD_BAS_ERREUR); /* FA169 */
printf("cr %d\n",cr);
return(cr);	
/*---------------------------------------------------------------------------*/
/*		 	FIN DE L'OPERATION	        		     */
/*---------------------------------------------------------------------------*/
}
/*
   Fin de l'operation SD_BAS_TraceDecodedon
*/
#ifdef couper
static t_SD_BAS_OK_KO SD_BAS_TraceDecodedon( 
			       int taille_buf,
			       char buff[],
			       int ad_deb,
			       t_SD_BAS_CpfType * type_lu,
			       int * nb_lig_lu,
			       int * nb_elt_lu,
			       int * nb_elt_alloc,
			       int * taille_cell_lu,
			       void ** don_lue,
			       e_SD_BAS_CpfEtat1 * etat1, 
			       int * ad_fin_dec)

     /* [E] taille du buffer a scanner en octets  */
      /* [E] buffer a scanner */
       /* [E] adr a partir de laquelle faut scanner buffer */
   /* [E/S] type donnee lue (=SD_BAS_INCONNU lors du 1appel) */
		       /* SD_BAS_ENTIER4 SD_BAS_REEL8 SD_BAS_ASCII SD_BAS_BLOC  EN SORTIE */
    /* [E/S] nb lignes du tableau lu ( = 0 lors 1er appel) */
    /* [E/S] nb elements tableau lu ( = 0 lors 1er appel) */
  /* [E/S] nb elements tableau lu ( = 0 lors 1er appel) */
/*[E/S] taill max en octet d'un elt du tab lu 1er appel*/
		       /* significatif exclusivement pour donnees SD_BAS_ASCII */	
  /* [E/S] donnee lue */
   /* [E/S] etat l'auto ( = DEBUT_DECODAGE  1er appel )*/
    /* [E/S] adresse fin du decodage effectif du  buffer */

{

int 	indice;          /* indice de parcourt du buffer */
t_SD_BAS_CpfType	type;    /* type de la donnee decodee */
int	nb_lig;          /* nombre de lignes de la donnee */
int 	nb_elt;          /* nombre d'elements de la donnee */
int	taille_max;      /* taille max d'une cellule SD_BAS_ASCII */
int	cr;	         /* code retour de la fonction */
t_SD_BAS_OK_KO   code;            /* code retour */  
char    car;	         /* caractere courant traite */
e_SD_BAS_CpfEtat1 etat1_prec;    /* etat precedent de niveau 1 */
e_SD_BAS_CpfEvt1    evt1;        /* evenement de niveau 1 detectee */
e_SD_BAS_CpfEtat2   etat2;       /* etat de niveau 2 ( determination du type ) */
e_SD_BAS_CpfEvt2    evt2;        /* evenement de niveau2			  */
int    ad_deb_do;        /* adresse de debut de la donnee courante */
long int *entier;        /* pointeur sur donnees entieres */
double   *reel;          /* pointeur sur donnees reelles */
char     *asc;           /* pointeur sur donnees ascii */ 
int    nb_elt_alloues;   /* nombres d'elements alloues */ 
int	taille;	         /* taille d'une donnee a decoder */		
int	nb_col;	         /* nombre de colonnes */

/*---------------------------------------------------------------------------*/
/*		 	DEBUT  DE L'OPERATION	        		     */
/*---------------------------------------------------------------------------*/	

cr=OK;
indice=ad_deb;
type= *type_lu;
nb_lig= *nb_lig_lu;
nb_elt= *nb_elt_lu;
taille_max = *taille_cell_lu;
nb_elt_alloues= *nb_elt_alloc;
taille=0;
if (type == SD_BAS_ASCII)
	{
	asc=(char *) *don_lue;
	}
else
	{
	if (type == SD_BAS_REEL8)
		{
		reel=(double *) *don_lue;
		}
	else
		{
		if (type == SD_BAS_ENTIER4)
			{
			entier=(long int *) *don_lue;
			}
		else
			{
			asc = (char *)malloc(SD_BAS_TAILLE_MAX_ASCII);
			if (asc == NULL )
				{
				sprintf(s_SD_BAS_ERREUR,"Erreur d'allocation lors de la lecture de la donnees");
				cr=SD_BAS_ERR_ALLOCA;
				goto ERREUR;
				}
			reel=(double *)asc;
			entier=(long int *)asc;
			}
		}
	}

/*--------------------------------------------------------------------*/
/* TANT QUE  non erreur et non fin decodage et indice < taille buffer */
/*--------------------------------------------------------------------*/
while( *etat1 >= 0 && *etat1 != SD_BAS_FINDEC && indice < taille_buf)
	{
/* 	car = buff[indice]						        */
/* 	etat1_prec=etat1							*/
	car = buff[indice];	
	etat1_prec=*etat1;
/*----------------------------------------------------------------------*/
/* 	fonction interne testant ( ) < BLANC ou TAB ou n AUTRE 		*/
/*----------------------------------------------------------------------*/
	code=SD_BAS_TraceAnalyse(
			car,          /* [E] caractere a tester */
			&evt1);         /* [S] evenement detecte */ 
		       

	*etat1 = s_autom1[*etat1][evt1];

/*      si erreur on affiche le messsage associe et on sort	*/	
	if (*etat1 > 10)
		{
		switch (*etat1) {
			case SD_BAS_ERR11:cr = SD_BAS_ERR_FORME_FIC;
				sprintf(s_SD_BAS_ERREUR,"erreur: forme donnee non coherente");  
				break;
			case SD_BAS_ERR13:cr = SD_BAS_ERR_FIN_DO;
				sprintf(s_SD_BAS_ERREUR,"erreur: fin de donnee prematuree"); 
				break;
			default:cr = KO;
				sprintf(s_SD_BAS_ERREUR,"erreur automate1 non reconnue");
				break; 
			}
		goto ERREUR;
		}
/*	si etat1= 2 et etat1_prec=4 			*/
/*			dim2 = dim2 + 1			*/
	if ( *etat1 == SD_BAS_ETAT12 && etat1_prec == SD_BAS_ETAT14)
		{
		nb_lig=nb_lig + 1;
		}
/* sauvegarde ad_deb_do */
	ad_deb_do=indice+1;
	if ( *etat1 == SD_BAS_BLOCVIDE )
		{
		type=SD_BAS_BLOC;
		nb_lig=1;
		nb_elt=0;
		*etat1 = SD_BAS_FINDEC;
		}
/*	SI (etat1=3)					*/
	if ( *etat1 == SD_BAS_ETAT13)
		{
/*		initialisation etat2					        */
		etat2= SD_BAS_DEBUTYPE;
/*		sauvegarde ad_deb_do						*/ 
		ad_deb_do=indice;

		
		code = SD_BAS_TraiteDecodedon(etat2,
			      evt2,
			      indice,
			      taille_buf,
			      car,
			      type,
			      nb_elt,
			      buff,
			      taille_max,
			      taille,
			      ad_deb_do,
			      entier,
			      reel,  
			      asc,
			      nb_elt_alloues
			      );
 *etat1 = SD_BAS_ETAT12;
/* FIN SI (etat1=3)					*/

		}
/*---------------------------------------------------------------------------*/
/*	FIN SI									*/
/*---------------------------------------------------------------------------*/
/*      l'indice est incremente						  	*/
	indice++;
	}
/*---------------------------------------------------------------------------*/
/* FIN TANT QUE  non erreur et non fin decodage et indice < taille buffer 	*/
/*---------------------------------------------------------------------------*/
if (indice == taille_buf)
	{
	cr=SD_BAS_EOF;
	}
if (*etat1 == SD_BAS_FINDEC)
	{ 
/* nb_col = nbelt  nb_lig						  	*/
	nb_col = (nb_elt/nb_lig);
/* SI nb_lig . nb_col <> nbelt alors erreur dans le nombre d'elements d'une ligne*/
	if ( (nb_lig * nb_col) != nb_elt)
		{
		sprintf(s_SD_BAS_ERREUR,"Nombre d'elements decodes incoherent (nb_elt:%d nb_lig:%d)",nb_elt,nb_lig);
		cr=SD_BAS_ERR_DIM;
		goto ERREUR;
		}
	}
/*---------------------------------------------------------------------------*/
/* nb_elt_lu = nb_elt							     */
/* nb_lig_lu = dim2 							     */
/* etat1 = etat1							     */
/* type_lu  = type							     */
/* taille_cell_lu= SD_BAS_REEL8 ou SD_BAS_ENTIER4 ou SD_BAS_ASCII( taille max )       */
/* ad_fin_dec= ad_fin  ( pour lecture du prochain buffer si besoin) 	     */  
/*---------------------------------------------------------------------------*/
*type_lu=type;
*nb_lig_lu=nb_lig;
*nb_elt_lu=nb_elt;
*taille_cell_lu=taille_max;
*ad_fin_dec=ad_deb_do;
*nb_elt_alloc=nb_elt_alloues;
if (type == SD_BAS_ASCII)
	{
	*don_lue=(void *)asc;
	}
else
	{
	if (type == SD_BAS_REEL8)
		{
		*don_lue=(void *)reel;
		}
	else
		{
		*don_lue=(void *)entier;
		}
	}
/*---------------------------------------------------------------------------*/
/* fin de l'operation en cas nominal  				     */
/*---------------------------------------------------------------------------*/
return(cr);
/*---------------------------------------------------------------------------*/
/* fin de l'operation en cas d'erreur 				     */
/*---------------------------------------------------------------------------*/
ERREUR:
/* affectation pour pouvoir liberer la place allouee ( par l'appelant )*/
if (type == SD_BAS_ASCII)
	{
	*don_lue=(void *)asc;
	}
else
	{
	if (type == SD_BAS_REEL8)
		{
		*don_lue=(void *)reel;
		}
	else
		{
		*don_lue=(void *)entier;
		}
	}
printf("cr %d\n",cr);
return(cr);	
/*---------------------------------------------------------------------------*/
/*		 	FIN DE L'OPERATION	        		     */
/*---------------------------------------------------------------------------*/
}
/*
   Fin de l'operation SD_BAS_TraceDecodedon
*/

#endif
/*****************************************************************************/
/*                                                                           */
/* OPERATION: SD_BAS_TraiteDecodedon                                         */
/*                                                                           */
/* CONTEXTE D'APPEL:  Interne a SD_BAS_TraceDecodedon                        */
/*                                                                           */
/* VERSION:                                                                  */
/* CODE RETOUR : OK ou KO 						     */
/*****************************************************************************/
t_SD_BAS_OK_KO SD_BAS_TraiteDecodedon(e_SD_BAS_CpfEtat2   etat2,
				      e_SD_BAS_CpfEvt2    evt2,
				      int indice,
				      int taille_buf,
				      char    car,
				      t_SD_BAS_CpfType	type,
				      int nb_elt,
				      char buff[],
				      int	taille_max,
				      int taille,
				      int ad_deb_do,
				      long int *entier,
				      double   *reel,  
				      char     *asc,
				      int    nb_elt_alloues
				      )
{

char NaNQ[5];
char MoinsNaNQ[6];
int INaN,INaNMoins;
 int cr=OK;
/* initialisations */
INaN=0;		
NaNQ[0]='\0';
strcpy(NaNQ,"NaNQ");
INaNMoins=0;							     
MoinsNaNQ[0]='\0';
strcpy(MoinsNaNQ,"-NaNQ");

/*---------------------------------------------------------------------------*/
/*TANT QUE etat2 non REEL et non ENTIER et non ASCII et indice < taille buff */
/*---------------------------------------------------------------------------*/
	 
 while ( etat2 != SD_BAS_REEL && etat2 != SD_BAS_ENTIER && etat2 != SD_BAS_ASC && indice <taille_buf )
			{
/*-----------------------------------------------------------------------*/
/*		  fonction interne testant . " CHIFFRE SIGNE BLANC AUTRE */
/*-----------------------------------------------------------------------*/

			  SD_BAS_TraceType(
			     car,       /* [E] caractere a tester */
			     &evt2         /* [S] evenement detecte */ 
		        	);
                        if ((evt2==SD_BAS_AUTRE)&&(car==NaNQ[INaN]))
                        {
			   evt2=SD_BAS_CHIFFRE; 

			   INaN=INaN+1;
			 }
                        if ((evt2==SD_BAS_AUTRE)&&(car==MoinsNaNQ[INaNMoins]))
                        {
			   evt2=SD_BAS_CHIFFRE; 
			   INaNMoins=INaNMoins+1;
			 }
			if (((INaN==4)||(INaNMoins==5))&& (evt2==SD_BAS_ESPACE))
			    etat2=SD_BAS_REEL;
			else                    
			   etat2=s_autom2[etat2][evt2];


/*      si erreur on affiche le messsage associe et on sort	*/	
			if (etat2 > 10)
				{
				switch (etat2) {
					case SD_BAS_ERR21:cr = SD_BAS_ERR_TYP;
					sprintf(s_SD_BAS_ERREUR,"erreur: type de donnee non coherente");  
					break;
				default:cr = KO;
					sprintf(s_SD_BAS_ERREUR,"erreur automate2 non reconnue");
					break; 
					}
				goto ERREUR;
				}
/*		  l'indice est incremente					*/
			indice ++;
			car = buff[indice];	
			}
/*----------------------------------------------------------------------*/
/*	FIN TANT QUE non erreur et non REEL et non ENTIER et non ASCII  */
/*----------------------------------------------------------------------*/
/* SI on arrive en fin de buffer sans avoir decode la donnee */
		if ( indice == taille_buf && (etat2 != SD_BAS_REEL || etat2 != SD_BAS_ENTIER || etat2 != SD_BAS_ASC))
		    {
			cr= SD_BAS_EOF;
		    }
		else
		   {
  		   if (etat2 == SD_BAS_ASC || type == SD_BAS_ASCII )
			{
			if (etat2 != SD_BAS_ASC || (type != SD_BAS_ASCII && type != SD_BAS_INCONNU))
				{
				sprintf(s_SD_BAS_ERREUR,"Erreur: melange de type dans une meme donnee");
				cr=SD_BAS_ERR_ASCII;
				goto ERREUR;
				}
			type=SD_BAS_ASCII;
/*-----------------------------------------------------------------------*/
/* si SD_BAS_ASCII  allocation et decodage entre ad_deb+1 et ad_fin -1	 */
/*( verification que la taille est inferieure a la taille max SD_BAS_ASCII) */
/*-----------------------------------------------------------------------*/
			taille= (indice) - (ad_deb_do) -2 + 1;
			if (taille  >= SD_BAS_TAILLE_MAX_ASCII)
				{
				sprintf(s_SD_BAS_ERREUR,"donnee ascii de taille sup a SD_BAS_TAILLE_MAX_ASCII (%d)",taille);
				cr=SD_BAS_ERR_ASCII;
				goto ERREUR;
				} 
/* 		allocation de SD_BAS_NB_ALLOC element reel entier et ASCII */
/*		par un realloc d'un ascii ( taille max) et sscanf(%s)	*/
			if (nb_elt_alloues <= nb_elt)
				{
				asc = (char *)realloc(asc,SD_BAS_TAILLE_MAX_ASCII*(SD_BAS_NB_ALLOC + nb_elt_alloues));
				if (asc != NULL )
					{
					nb_elt_alloues = SD_BAS_NB_ALLOC + nb_elt_alloues;
					}
				else
					{
					sprintf(s_SD_BAS_ERREUR,"Erreur d'allocation lors de la lecture de la donnees");
					cr=SD_BAS_ERR_ALLOCA;
					goto ERREUR;
					}
				}

				strncpy(asc + (nb_elt*SD_BAS_TAILLE_MAX_ASCII),buff+ad_deb_do+1,taille-1);
				*(asc + (nb_elt)*SD_BAS_TAILLE_MAX_ASCII +taille -1 )='\0';
/*			   taille_max = sup (taille_max,taille)     */
			taille_max=maxi(taille,taille_max);

/*		nbelt = nbelt + 1						*/
			nb_elt=nb_elt +1;
			}
		   else
			{
			 indice--;
/*----------------------------------------------------------------------*/
/*	si REEL allocation et decodage entre ad_deb et ad_fin-1   	*/
/*		par un realloc d'un reel et sscanf(%lf)			*/
/*----------------------------------------------------------------------*/
			 cr= SD_BAS_SousTraiteDecodedon(etat2,
							type,
							nb_elt,
							buff,
							ad_deb_do,
							  entier,
							reel,  
							nb_elt_alloues,
							INaN,
							INaNMoins
							);

			 if(cr != OK)
			   goto ERREUR;
		
/*		nbelt = nbelt + 1						*/			
			nb_elt=nb_elt +1;
			}
/* FIN SI on arrive en fin de buffer sans avoir decode la donnee */
/* on se positionne sur le dernier caractere de la donnee 	*/
		   indice--;
		   }
	
        



ERREUR:
return(cr);


}
/*****************************************************************************/
/*                                                                           */
/* OPERATION: SD_BAS_SousTraiteDecodedon                                     */
/*                                                                           */
/* CONTEXTE D'APPEL:  Interne a SD_BAS_TraceDecodedon                        */
/*                                                                           */
/* VERSION:                                                                  */
/* CODE RETOUR : OK ou KO 						     */
/*****************************************************************************/
t_SD_BAS_OK_KO SD_BAS_SousTraiteDecodedon(e_SD_BAS_CpfEtat2   etat2,
					  t_SD_BAS_CpfType	type,
					  int nb_elt,
					  char buff[],
					  int ad_deb_do,
					  long int *entier,
					  double   *reel,  
					  int    nb_elt_alloues,
					  int INaN,
					  int INaNMoins
					  )
{

  int cr=OK;            /*retour de fonction */

 t_SD_BAS_OK_KO   code; /*retour de fonction */


/*----------------------------------------------------------------------*/
/*	si REEL allocation et decodage entre ad_deb et ad_fin-1   	*/
/*		par un realloc d'un reel et sscanf(%lf)			*/
/*----------------------------------------------------------------------*/

 if (etat2 == SD_BAS_REEL || type == SD_BAS_REEL8)
   {
     if (etat2 != SD_BAS_REEL || (type != SD_BAS_REEL8 && type != SD_BAS_INCONNU))
       {
	 sprintf(s_SD_BAS_ERREUR,"Erreur: melange de types dans une meme donnee");
	 cr=SD_BAS_ERR_REEL;
	 goto ERREUR;
       }
     type = SD_BAS_REEL8;				
     
     /*si  le nombre d'element alloue et < oa nombre d'elements */
     if (nb_elt_alloues <= nb_elt)
       {
	 /*allocation de SD_BAS_NB_ALLOC element reel  */
	 reel=(double *)realloc(reel,sizeof(double)*(SD_BAS_NB_ALLOC + nb_elt_alloues));
	 if (reel != NULL )
	   {
	     /* mise a jour du nombre d'elements alloues */
	     nb_elt_alloues = SD_BAS_NB_ALLOC + nb_elt_alloues;
	   }
	 else
	   {
	     sprintf(s_SD_BAS_ERREUR,"Erreur d'allocation lors de la lecture de la donnees");
	     cr=SD_BAS_ERR_ALLOCA;
	     goto ERREUR;
	   }
       }
     if ((INaN==4)||(INaNMoins==5))
       reel[nb_elt]=REELMAX;
     else
       {
	 code=sscanf(buff+ad_deb_do," %lf",reel+nb_elt);
	 if (code != 1)
	   {
	     sprintf(s_SD_BAS_ERREUR,"Erreur de decodage de la donnee reelle (element %d)",nb_elt);
	     cr=SD_BAS_ERR_REEL;
	     goto ERREUR;
	   }
       }
   }
 else							
   {
     if (etat2 != SD_BAS_ENTIER || (type != SD_BAS_ENTIER4 && type != SD_BAS_INCONNU))
       {
	 sprintf(s_SD_BAS_ERREUR,"Erreur: melange de type dans une meme donnee");
	 cr=SD_BAS_ERR_ENTIER;
	 goto ERREUR;
       }
     type = SD_BAS_ENTIER4;
/*----------------------------------------------------------------------*/
/*	si ENTIER allocation et decodage entre ad_deb et ad_fin -1	*/
/*		par un realloc d'un entier4 et sscanf(%ld)		*/
/* 			allocation de SD_BAS_NB_ALLOC element entier       */
/*----------------------------------------------------------------------*/

 /*si  le nombre d'element alloue et < oa nombre d'elements */
     if (nb_elt_alloues <= nb_elt)
       {
	 /*allocation de SD_BAS_NB_ALLOC element entier */
	 entier=(long int *)realloc(entier,sizeof(long int)*(SD_BAS_NB_ALLOC+nb_elt_alloues));
	 if (entier != NULL )
	   {
	     /* mise a jour du nombre d'elements alloues */
	     nb_elt_alloues = SD_BAS_NB_ALLOC + nb_elt_alloues;
	   }
	 else
	   {
	     sprintf(s_SD_BAS_ERREUR,"Erreur d'allocation lors de la lecture de la donnees");
	     cr=SD_BAS_ERR_ALLOCA;
	     goto ERREUR;
	   }
       }
     code =sscanf(buff+ad_deb_do," %ld",entier+nb_elt);
     if (code != 1)
       {
	 sprintf(s_SD_BAS_ERREUR,"Erreur de decodage de la donnee entier(element %d)",nb_elt);
	 cr=SD_BAS_ERR_ENTIER;
	 goto ERREUR;
       }
   } /* fin si */
 
 /* FIN de fonctio, */ 
 ERREUR:
 return(cr);


}
/*****************************************************************************/
/*                                                                           */
/* OPERATION:   SD_BAS_TraceType                                           */
/*                                                                           */
/* ROLE: Cette operation interne est un automate qui teste les caracteres    */
/*       . , " , BLANC ou TAB ou \n ou ), CHIFFRE	, SIGNE	, AUTRE	     */
/* 									     */
/*      								     */
/*                                                                           */
/* CONTEXTE D'APPEL:                                                         */
/*                                                                           */
/* VERSION:                                                                  */
/* CODE RETOUR : OK ou KO 						     */
/*****************************************************************************/
static t_SD_BAS_OK_KO SD_BAS_TraceType(
			  char car,
			  e_SD_BAS_CpfEvt2 * evt2
			  )
    /* [E] caractere a tester */
        /* [S] evenement detecte */ 
{

/*---------------------------------------------------------------------------*/
/*		 	DEBUT DE L'OPERATION	        		     */
/*---------------------------------------------------------------------------*/
/* si  char = '.'  evt1 = 0 					*/
if (car == '.' )
	{
	*evt2=SD_BAS_POINT;
	}
else
	{
/* sinon si  char = '"' evt1 = 1 				*/
	if (car=='"' )
		{
		*evt2=SD_BAS_GUILL;
		}
	else
		{
/* 	   sinon si char = BLANC ou TAB ou \n ou ) evt1 = 2 	*/
		if (car==' ' || car=='	' || car=='\n' || car==')' )
			{
			*evt2=SD_BAS_ESPACE;
			}
else
			{
/* 		 sinon si char = CHIFFRE  (isdigit) evt1= 3       	*/
			if ( isdigit(car) != 0)
				{
				*evt2=SD_BAS_CHIFFRE;
				}
			else
				{
/*		       sinon si char = SIGNE  evt1= 4	        */
				if ( car == '+' || car =='-')
					{
					*evt2=SD_BAS_SIGNE;
					}
				else
					{
/*			     sinon evt1= 5			*/	
					*evt2=SD_BAS_AUTRE;
					}
				}
/*		       fin si					*/
			}
/*		 fin si						*/
		}
/*	   fin si						*/
	}
 /* fin si							*/	      
 return(OK);
}
/*
   Fin de l'operation SD_BAS_TraceType
*/

/***************************************************************************/
/*                                                                         */
/* OPERATION:  TraceLire                                                 */
/*                                                                         */
/* ROLE:  Cette operation externe est chargee de lire une donne associee a */
/*	  un mot cle ds le fichier de donnees  precedenment ouvert.        */
/*        Il s'agit donc apres localisation du mot cle de lire la donnee et*/
/*    toutes les informations associees(nb_lig,nb_col,taille_cell).        */
/*    On verifie alors la coherence entre les informations lues et celles  */
/*    issues des arguments donnes par l'appelant.		           */
/*	  Si les 3 informations sont coherentes la donnee est recopiee ds  */
/*    la zone memoire reservee par l'appelant.				   */
/* 	                                                                   */
/*                                                                         */
/* CONTEXTE D'APPEL:                                                       */
/* cette operation doit etre appele apres ouverture prealable du fichier   */
/* de donnees .						                   */
/* pour une souplesse d'utilisation max, il est utile d'appeler auparavant */
/* l'operation  SD_BAS_TraceLireCaract afin de reserver la place necessaire*/
/*                                                                         */
/* VERSION:                                                                */
/* CODES RETOUR: OK ou KO				                   */
/*   si KO l'erreur peut etre affiche via la variable globale s_SD_BAS_ERREUR   */
/*   " fichier de donnees  %s non ouvert"			           */ 
/*   " mot cle non present dans le fichier de donnees  %s"    	           */
/*   " aucun mot cle dans le fichier %s"			           */
/*   " taille du mot cle lue %s superieure a la limite %d ou egale a 0"    */ 
/*   " erreur a la lecture du fichier de donnees  %s"                      */
/*   " parenthese ouvrante non detectee (mot cle %s )                      */ 
/*   " parenthese fermante  non detectee (mot cle %s )                     */
/*   " nombre de niveaux de parentheses superieur a 2 ( mot cle %s )       */
/*   " % dimension du tableau non constante (mot cle %s)                   */
/*   " % dimension du tableau = 0     (mot cle %s)                         */
/*   " type de la donnee lue %s non reconnue (mot cle %s)                  */
/*   " erreur lors du decodage de la donnee lue %s" (mot cle %s)           */
/* " incoherence entre nblig lue (%d) et nblig specifiee (%d) (mot cle %s) */
/* " incoherence entre nbcol lue (%d) et nbcol specifiee (%d) (mot cle %s) */ 
/* " incoherence entre taille_cell lue (%d) et taille_cell  specifiee (%d) */
/*            (mot cle %s) 						   */
/***************************************************************************/
t_SD_BAS_OK_KO SD_BAS_TraceLire( 
		   t_SD_BAS_CpfFile * fd,
		   char * mot_cle,
		   t_SD_BAS_CpfType * type,
		   int * nb_lig,
		   int * nb_col, 
		   int * taille_cell,
		   void * donnee
		   )

     /* [E] structure decrivant fichie donnees          */
     /* [E] mot cle recherche ds fichier terminee par \0*/
     /* [S] type de la donnee lue 			 */
     /* [S] nombre de lignes du tableau lu 		 */
     /* [S] nombre de colonnes du tableau lu 	         */  
     /* [S] taille maxi en octets d'1 element donnee lue*/
     /* [S] non significatif si le type est <>SD_BAS_ASCII */ 
     /* [E/S] pointeur sur zone mem alloue par appelant */

{
int	i;	           /* indice de boucle */

t_SD_BAS_CpfMotCle mot_cle_lu;    /* mot cle courant lu dans le fichier */
int	indic_mot_cle;     /* indicateur de recherche du mot cle */
int	ad_mot_cle;        /* adresse du debut du mot cle courant */	
int     ad_donnee;         /* adresse de la donnee apres le mot cle trouve */ 
int	adresse_deb;       /* adresse de debut de recherche du mot cle */
t_SD_BAS_OK_KO	cr;	           /* code retour des operations */		
void	*don_lue;          /* donnees lue en interne par SD_BAS_TraceLireDonnee*/
t_SD_BAS_CpfType type_lu;   /* type de la donnee lue SD_BAS_ENTIER4 ou */
                           /* SD_BAS_REEL8 ou SD_BAS_BLOC*/
int	nb_lig_lu;         /* nombre de lignes lues */
int     nb_col_lu;         /* nombre de colonnes lues */
int	taille_cell_lu;    /* taille de la cellule SD_BAS_ASCII maxi lue */


 
  
/* initialisations */
/*-----------------*/
indic_mot_cle = SD_BAS_NON_TROUVE;
i=0;
ad_donnee=0;

cr=OK;
don_lue=NULL;
nb_lig_lu=0;
nb_col_lu=0;
type_lu= SD_BAS_INCONNU;
taille_cell_lu=0;


/* on verifie que le type fait partie des types reconnus */
/*-------------------------------------------------------*/
if ( *type != SD_BAS_ENTIER4 && *type != SD_BAS_REEL4 && *type != SD_BAS_REEL8 && *type  != SD_BAS_ASCII && *type !=SD_BAS_BLOC  && *type !=SD_BAS_FIC_BIN_DOUBLE && *type !=SD_BAS_FIC_BIN_ENTIER )
	{
	cr = SD_BAS_ERR_TYP;
	sprintf(s_SD_BAS_ERREUR,"type :%d non reconnu ( 1:SD_BAS_ENTIER4 2:SD_BAS_REEL4 3:SD_BAS_REEL8 4:SD_BAS_ASCII 5:SD_BAS_BLOC 6:SD_BAS_FIC_BIN_DOUBLE 7:SD_BAS_FIC_BIN_ENTIER)",*type);
	goto ERREUR;
	} 
/* on verifie au prealable que le fichier est effectivement ouvert */
/*-----------------------------------------------------------------*/
/* SINON on sort en erreur */
/*-------------------------*/
if ( fd ==NULL ) 

	{
	cr = SD_BAS_ERREUR_OUV;
	sprintf(s_SD_BAS_ERREUR,"Erreur ouverture du fichier de donnees ");
	goto ERREUR;
	}

/*---------------------------------------------------------------------------*/
/* SI le dernier mot cle lu dans t_SD_BAS_CpfFile n'existe pas ou  n'est pas egal au mot cle recherche */ 
/*---------------------------------------------------------------------------*/
if (fd->ad_dern_mot_cle == -1 || strncmp(mot_cle,fd->dernier_mot_cle,SD_BAS_TAILLE_MAX_CLE) != 0 )
	{ 
/* SI la liste des mots cles est en memoire 			*/
	if ( fd->nb_mot_cle != -1 ) 
		{
/*---------------------------------------------------------------------------*/
/* ALORS on scrute cette liste jusqu'a trouver le mot cle */
/*---------------------------------------------------------------------------*/
		while (i< fd->nb_mot_cle && indic_mot_cle == SD_BAS_NON_TROUVE )
			{
			if ( strncmp(mot_cle,(char *)(fd->liste_mot_cle +i),SD_BAS_TAILLE_MAX_CLE) == 0 )
				{
				indic_mot_cle = SD_BAS_TROUVE;
/*	   on recupere alors le pointeur sur la zone fichier apres ce mot cle */
				ad_donnee=*(fd->ad_donnee + i);
				strcpy(mot_cle_lu,(char *)(fd->liste_mot_cle +i));
				ad_mot_cle=*(fd->ad_mot_cle + i);
				}
			i++;
			}
		}
	else
		{
/* SINON	*/
/*   on se positionne au debut du fichier ( adresse_deb = 0)*/
		adresse_deb = 0;	
/*---------------------------------------------------------------------------*/
/*   on cherche le premier mot cle a partir de cette adresse (operation interne basee sur un automate) */
/*---------------------------------------------------------------------------*/
		cr = SD_BAS_TraceLireCle(fd,     /* [E] file descriptor du fichier a lire  */
        	      	   adresse_deb,   /* [E] adresse de debut de recherche */
	        	   mot_cle_lu,  /* [S] mot cle suivant lu ( termine par \0)*/
		           &ad_donnee, /* [S] adresse du premier caractere suivant le mot cle */
		           &ad_mot_cle /* [S] adresse du premier caractere du mot cle */
 		   	  );
/*    si erreur on sort */
		if (cr != OK && cr != SD_BAS_EOF) {
			goto ERREUR;
			}	
/*   si aucun mot cle trouve on sort en erreur "aucun mot cle dans le fichier" */  
		if ( cr == SD_BAS_EOF)
			{
			sprintf(s_SD_BAS_ERREUR,"Erreur fichier sans mot cle( %s)",fd->nom_fichier);
			goto ERREUR;
			}
/*   TANT QUE  mot_cle non trouve &  pas erreur */
		while (cr == OK && indic_mot_cle == SD_BAS_NON_TROUVE )
			{
/*   	  si le mot cle est egal au mot cle demande alors mot_cle trouve */
			if ( strcmp(mot_cle,mot_cle_lu) == 0)
				{
				indic_mot_cle = SD_BAS_TROUVE;
				}
			else
				{
				adresse_deb = ad_donnee;
/*---------------------------------------------------------------------------*/
/*      si mot_cle non trouve on cherche le mot cle suivant (operation interne basee sur un automate) */
/*      cette operation necessite : un pointeur sur la zone fichier ou l'on recherche le mot cle suivant */
/*      cette operation ramene : le mot cle suivant et le pointeur sur la zone fichier apres ce mot cle */
/*---------------------------------------------------------------------------*/
				cr = SD_BAS_TraceLireCle(fd,     /* [E] file descriptor du fichier a lire  */
        	      	   		   adresse_deb,   /* [E] adresse de debut de recherche */
	        	   		   mot_cle_lu,  /* [S] mot cle suivant lu (termine par \0)*/
		           		   &ad_donnee, /* [S] adresse du premier caractere suivant le mot cle */
		           		   &ad_mot_cle/* [S] adresse du premier caractere du mot cle */
 		   	  		   );
				}
/*    FIN TANT QUE pas erreur & mot_cle non trouve */
			}
/*	FIN SI LISTE MOT CLE presente en memoire*/	
		}
/* FIN SI le dernier mot cle lu dans t_SD_BAS_CpfFile n'existe pas ou  n'est pas egal au mot cle recherche */ 
	if (cr != OK && cr != SD_BAS_EOF) 
		{
		goto ERREUR;
		}
/* 	SI fin de fichier on sort en erreur "mot cle non present */
	if ( indic_mot_cle == SD_BAS_NON_TROUVE ) 
		{
		sprintf(s_SD_BAS_ERREUR,"Mot cle :%s non trouve dans le fichier :%s",mot_cle,fd->nom_fichier);
                cr=SD_BAS_NON_TROUVE;
		goto ERREUR;
		}
/* on sauvegarde alors dans la structure fd les info sur le dernier mot cle */
	strcpy(fd->dernier_mot_cle,mot_cle_lu);
	fd->ad_dern_mot_cle = ad_mot_cle;
	fd->ad_dern_donnee  = ad_donnee;
	}

/* SINON	*/
else
	{
	indic_mot_cle = SD_BAS_TROUVE;
	ad_donnee = fd->ad_dern_donnee;	
	ad_mot_cle= fd->ad_dern_mot_cle; /* FA169 */
	strcpy(mot_cle_lu,fd->dernier_mot_cle);
	}


/*---------------------------------------------------------------------------------------*/
/* a partir du mot cle trouve on analyse syntaxiquement la ligne jusqu'au prochain mot cle */
/* (operation interne effectuant le decodage et donnant : le type de donnee (entier ou reel ou SD_BAS_ASCII),*/
/* le nombres de lignes */
/* le nombre de colonnes , la taille maxi d'un element , un pointeur sur le tableau lu */
/* ( ce tableau a ete alloue dynamiquement par cette operation) 	       		 */
/*---------------------------------------------------------------------------------------*/

 

  
cr=SD_BAS_TraceLireDonnee(
			fd,         /* [E] file descriptor du fichier a lire  */
        	      	ad_donnee,  /* [E] adresse de debut de recherche */
	        	&don_lue,    /* [S] donnee lue */
			&type_lu,    /* [S] type de la donnee lue (SD_BAS_ENTIER4,SD_BAS_REEL8,SD_BAS_ASCII,SD_BAS_BLOC)*/
			&nb_lig_lu,  /* [S] nombre de lignes du tableau lu  */
			&nb_col_lu,  /* [S] nombre de colonnes du tableau lu  */  
		        &taille_cell_lu/* [S] taille maxi en octet d'un element du tableau lu ( significatif
					 pourla donnee SD_BAS_ASCII )*/   
		      );



  
/* si erreur de decodage on sort en erreur */
if (cr != OK ) {
	if (cr == SD_BAS_EOF){
		sprintf(s_SD_BAS_ERREUR,"erreur fichier de donnees incomplet (mot cle %s)",mot_cle);
		}
	goto ERREUR;
	}

 
/*---------------------------------------------------------------------------------------*/
/* on effectue les controles de coherence : */
/* type_lu = type  ( entier reel ou SD_BAS_ASCII)*/
/* nblig_lu = nblig */
/* nbcol_lu = nb_col */
/* taille_cell_lu <= taille_cell */
/* SI difference on sort en erreur "incoherence entre " */  
/* NB : une lecture en reel4 de reel ecrit en reel8 n'est pas une erreur */
/*---------------------------------------------------------------------------------------*/

/* donnee de type fichier */


 cr = SD_BAS_TraceVerifLum(
			   fd,
			   type,
			   nb_lig,
			   nb_col, 
			   &type_lu,
			   nb_lig_lu,
			   nb_col_lu,
			   donnee,
			   don_lue); 
  
 if (cr != OK)
   goto ERREUR;			


 cr = SD_BAS_TraceVerif(
			type,
			nb_lig,
			nb_col, 
			taille_cell,
			type_lu,
			nb_lig_lu,
			nb_col_lu,
			taille_cell_lu,
			donnee,
			don_lue);  

 if (cr != OK)
   goto ERREUR;				 

/*---------------------------------------------------------------------------*/
/* fin de l'operation en cas nominal  				     */
/*---------------------------------------------------------------------------*/
return(OK);
/*---------------------------------------------------------------------------*/
/* fin de l'operation en cas d'erreur 				     */
/*---------------------------------------------------------------------------*/
ERREUR:
/* liberation place memoire alloue par l'operation interne precedente  */
if (don_lue != NULL)
	{
	free(don_lue);
	don_lue=NULL;
	}
return(cr);	  
} 
/*
   Fin de l'operation TraceLire
*/

/***************************************************************************/
/*                                                                         */
/* OPERATION:  TraceVerifLum                                                   */
/*                                                                         */
/* ROLE:                                                                   */
/*    On verifie alors la coherence entre les informations lues et celles  */
/*    issues des arguments donnes par l'appelant.		           */
/*	  Si les 3 informations sont coherentes la donnee est recopiee ds  */
/*    la zone memoire reservee par l'appelant.				   */
/* 	                                                                   */
/*                                                                         */
/* CONTEXTE D'APPEL:                                                       */
/*     Interne a TraceLire                                                 */
/*                                                                         */
/* VERSION:                                                                */
/* CODES RETOUR: OK ou KO				                   */
/***************************************************************************/
static t_SD_BAS_OK_KO SD_BAS_TraceVerifLum(
				 t_SD_BAS_CpfFile * fd,	   
				 t_SD_BAS_CpfType * type,
				 int * nb_lig,
				 int * nb_col, 
				 t_SD_BAS_CpfType *type_lu,
				 int nb_lig_lu,
				 int nb_col_lu,
				 void * donnee,
				 void	*don_lue)      
{

  char * temp;
  char * message;
  t_SD_BAS_Codage codage;
  char    *don_asc;           /* donnee ascii lue */
  long int *don_ent;         /* donnee entiere lue */
  long int  *donnee_li;      /* donnee entiere a renseigner*/	
  long int  entier;          /* variable de travail */
  int i,j;
  char * pt;
  char nom_fic[SD_BAS_TAILLE_MAX_ASCII] ;  
  t_SD_BAS_OK_KO	cr=OK;

 if (*type_lu ==  SD_BAS_ASCII && nb_lig_lu == 1 && nb_col_lu == 2)
   {
 
 	don_asc = (char *)don_lue;
	message=(char*)malloc(SD_BAS_TAILLE_MAX_ASCII*sizeof(char));
	temp=(char*)malloc(SD_BAS_TAILLE_MAX_ASCII*sizeof(char));

	strcpy(message,don_asc + SD_BAS_TAILLE_MAX_ASCII);

	if(strncmp(message,"FICHIER",7) == 0)
	  { 
	    sscanf(message,"%s %s %d %d",temp,temp,nb_lig,nb_col);
	    
	    nb_lig_lu=*nb_lig;
	    nb_col_lu=*nb_col;
	   
	  if(strncmp(temp,FICDOUBLE,7) == 0){
	    *type_lu =  SD_BAS_FIC_BIN_DOUBLE;
	    *type =  SD_BAS_FIC_BIN_DOUBLE;
	    strcpy(codage,CODAGE_DOUBLE);

	   
	  }
	  else if (strncmp(temp,FICENTIER,7) == 0){
	    *type_lu = SD_BAS_FIC_BIN_ENTIER;
	    *type = SD_BAS_FIC_BIN_ENTIER;
	    strcpy(codage,CODAGE_ENTIER);
	  }

	  /* creation du chemin complet */

	  /* on recupere le repertoire du fichier de trace */
	  pt = strrchr(fd->nom_fichier, '/');
	  if(pt!= NULL)
	    {
	      strncpy( nom_fic, fd->nom_fichier, (pt - fd->nom_fichier));
	      nom_fic[(pt - fd->nom_fichier)]='\0';
	      strcat(nom_fic, "/");
	      strcat(nom_fic, don_asc);
	    }
	  else
	    {
	      strcpy(nom_fic, don_asc);
	    }
	 
	  SD_BAS_TraceLireFichierLum(nom_fic,*nb_lig,*nb_col,codage,donnee);



	  free(temp);
	  free(message);
	  }

  }

  /*---------------------------------------------------------------------------------------*/
/* donnee de type entiere */
/*---------------------------------------------------------------------------------------*/
if ( *type_lu == SD_BAS_ENTIER4) 
	{
 	if (*type != SD_BAS_ENTIER4)
		{
		cr = KO;
		sprintf(s_SD_BAS_ERREUR,"type lu (%d) incoherent avec le type specifie ( %d) ",type_lu,*type);
		goto ERREUR;
		}
	if (nb_lig_lu != *nb_lig || nb_col_lu != *nb_col)
		{
		cr = KO;
		sprintf(s_SD_BAS_ERREUR,"dim lue incoherente avec la dimension specifiee");
		goto ERREUR;
		} 
/* le pointeur sur le tableau alloue par l'appelant est caste au type donne par l'appelant  */
/* le pointeur sur le tableau alloue par l'operation de decodage est caste au type_lu */
	don_ent   = (long int *)don_lue;
	donnee_li = (long int *)donnee;
	i=0;
/* TANT QUE lig <= nblig_lu et non erreur */ 
	while ( i < nb_lig_lu && cr ==OK)
		{
		j=0;
/* 	TANT QUE col <= nbcol_lu et non erreur */ 
		while ( j < nb_col_lu && cr ==OK)
			{
			entier = *(don_ent + i*nb_col_lu + j);
			*(donnee_li +i*nb_col_lu + j) = entier;
			j++;			
			}
/* 	FIN TANT QUE col <= nbcol_lu et non erreur */ 
		i++;
		}
/* 	FIN TANT QUE lig <= nblig_lu et non erreur */ 
	if (cr != OK )
		{
		goto ERREUR;
		}
	}

/* fin de l'operation en cas nominal  				     */
/*---------------------------------------------------------------------------*/
return(OK);
/*---------------------------------------------------------------------------*/
/* fin de l'operation en cas d'erreur 				     */
/*---------------------------------------------------------------------------*/
ERREUR:
/* liberation place memoire alloue par l'operation interne precedente  */
if (don_lue != NULL)
	{
	free(don_lue);
	don_lue=NULL;
	}
 return(cr);
	
}

/***************************************************************************/
/*                                                                         */
/* OPERATION:  TraceVerif                                                   */
/*                                                                         */
/* ROLE:                                                                   */
/*    On verifie alors la coherence entre les informations lues et celles  */
/*    issues des arguments donnes par l'appelant.		           */
/*	  Si les 3 informations sont coherentes la donnee est recopiee ds  */
/*    la zone memoire reservee par l'appelant.				   */
/* 	                                                                   */
/*                                                                         */
/* CONTEXTE D'APPEL:                                                       */
/*     Interne a TraceLire                                                 */
/*                                                                         */
/* VERSION:                                                                */
/* CODES RETOUR: OK ou KO				                   */
/***************************************************************************/
static t_SD_BAS_OK_KO SD_BAS_TraceVerif(
				 t_SD_BAS_CpfType * type,
				 int * nb_lig,
				 int * nb_col, 
				 int * taille_cell,
				 t_SD_BAS_CpfType type_lu,
				 int nb_lig_lu,
				 int nb_col_lu,
				 int taille_cell_lu,
				 void * donnee,
				 void	*don_lue)      
				 
{

  double    reel;	           /* variable de travail */	 
 
  t_SD_BAS_OK_KO	cr=OK;

  char    *donnee_a;         /* donnee ascii lue */
  int i,j;	
  double    *donnee_d;       /* donnee reelle  a renseigner*/
  float     *donnee_f;       /* donnee reelle  a renseigner*/
 
  double   *don_ree;         /* donnee reelle lue */
  char    *don_asc;          /* donnee ascii lue */


/*---------------------------------------------------------------------------------------*/
/* donnee de type reelle */
/*---------------------------------------------------------------------------------------*/
if ( type_lu == SD_BAS_REEL8) 
	{
	if (*type != SD_BAS_REEL4 && *type != SD_BAS_REEL8)
		{
		cr=KO;
		sprintf(s_SD_BAS_ERREUR,"type lu (%d) incoherent avec le type specifie ( %d)",type_lu,*type);
		goto ERREUR;
		}
	if (nb_lig_lu != *nb_lig || nb_col_lu != *nb_col)
		{
		cr=KO;
		sprintf(s_SD_BAS_ERREUR,"dim lue incoherente avec la dimension specifiee ");
		goto ERREUR;
		} 
/* le pointeur sur le tableau alloue par l'appelant est caste au type donne par l'appelant  */
/* le pointeur sur le tableau alloue par l'operation de decodage est caste au type_lu */
	don_ree = (double *)don_lue;
	donnee_f = (float *)donnee;
	donnee_d = (double *)donnee;
	i=0;
/* TANT QUE lig <= nblig_lu et non erreur */ 
	while ( i < nb_lig_lu)
		{
		j=0;
/* 	TANT QUE col <= nbcol_lu et non erreur */ 
		while ( j < nb_col_lu)
			{
			reel = *(don_ree + i*nb_col_lu + j);
			if (*type == SD_BAS_REEL4)
				{
				*(donnee_f + i*nb_col_lu + j) = reel;
				}
			else
				{
				*(donnee_d + i*nb_col_lu + j) = reel;
				}
			j++;			
			}
/* 	FIN TANT QUE col <= nbcol_lu et non erreur */
		i++; 
		}
/* 	FIN TANT QUE lig <= nblig_lu et non erreur */ 
	}


/*---------------------------------------------------------------------------------------*/
/* donnee de type SD_BAS_ASCII */
/*---------------------------------------------------------------------------------------*/
if (type_lu == SD_BAS_ASCII )
	{
	if (*type != SD_BAS_ASCII )
		{
		cr=KO;
		sprintf(s_SD_BAS_ERREUR,"type lu (%d) incoherent avec le type specifie ( %d)",type_lu,*type);
		goto ERREUR;
		}
	if (nb_lig_lu != *nb_lig || nb_col_lu != *nb_col)
		{
		cr=KO;
		sprintf(s_SD_BAS_ERREUR,"dim lue incoherente avec la dimension specifiee ");
		goto ERREUR;
		} 
	if (taille_cell_lu > *taille_cell)
		{
		cr=KO;
		sprintf(s_SD_BAS_ERREUR,"longueur max des donnees SD_BAS_ASCII lues (%d) incoherente avec la taille specifie (%d)"
			,taille_cell_lu,*taille_cell);
		goto ERREUR;
		}
/* le pointeur sur le tableau alloue par l'appelant est caste au type donne par l'appelant  */
/* le pointeur sur le tableau alloue par l'operation de decodage est caste au type_lu */
	don_asc = (char *)don_lue;
	donnee_a = (char *)donnee;
	i=0;
	j=0;
/* TANT QUE lig <= nblig_lu et non erreur */ 
	while ( i < nb_lig_lu)
		{
		j=0;
/* 	TANT QUE col <= nbcol_lu et non erreur */ 
		while ( j < nb_col_lu)
			{
			strcpy(donnee_a + (i*nb_col_lu + j)* (*taille_cell),don_asc + (i*nb_col_lu + j)*SD_BAS_TAILLE_MAX_ASCII);
			j++;
			}
/* 	FIN TANT QUE col <= nbcol_lu et non erreur */
		i++;
		}
/* 	FIN TANT QUE lig <= nblig_lu et non erreur */ 
	}
/* liberation place memoire alloue par l'operation interne precedente  */
if (don_lue != NULL)
	{
	free(don_lue);
	don_lue=NULL;
	}
/*---------------------------------------------------------------------------------------*/
/* donnee de type SD_BAS_BLOC */
/*---------------------------------------------------------------------------------------*/
if (type_lu == SD_BAS_BLOC)
	{
	if (*type != SD_BAS_BLOC )
		{
		sprintf(s_SD_BAS_ERREUR,"type lu (%d) incoherent avec le type specifie ( %d)",type_lu,*type);
		goto ERREUR;
		}
	}
/*---------------------------------------------------------------------------*/
/* fin de l'operation en cas nominal  				     */
/*---------------------------------------------------------------------------*/
return(OK);
/*---------------------------------------------------------------------------*/
/* fin de l'operation en cas d'erreur 				     */
/*---------------------------------------------------------------------------*/
ERREUR:
/* liberation place memoire alloue par l'operation interne precedente  */
if (don_lue != NULL)
	{
	free(don_lue);
	don_lue=NULL;
	}
return(cr);	  
} 




/*****************************************************************************/
/*                                                                           */
/* OPERATION:  SD_BAS_TraceModifie                                         */
/*                                                                           */
/* ROLE:  Cette operation modifie le contenu d'une donnee  dans un champ     */
/*	Toutes les autres donnees sont conservees dans le fichier .Si le mot */
/*	cle n'est pas trouve, une erreur est detectee.			     */
/*   NB : le fichier doit au prealable avoir ete ouvert en mode SD_BAS_ECRITURE */
/*                                                                           */
/*                                                                           */
/* CONTEXTE D'APPEL:                                                         */
/*                                                                           */
/* VERSION:                                                                  */
/* CODES RETOUR: OK ou KO						     */
/* "erreur fichier ouvert en mode lecture "				     */
/*****************************************************************************/
t_SD_BAS_OK_KO SD_BAS_TraceModifie( 
		      t_SD_BAS_CpfFile  * fd,
		      char * mot_cle,
		      t_SD_BAS_CpfType type,
		      int nb_lig,
		      int nb_col,
		      int taille_cell,
		      void * donnee
		      )

     /* [E] structure  fichier donnees	        */
     /* [E] mot cle associe aux donnees a modifier */
     /* [E] type des donnees a modifier		*/
     /* [E] nombre de lignes du tableau a ecrire 	*/
     /* [E] nombre de colonnes du tableau a ecrire	*/
     /* [E] taille maxi en octets d'un element */	
     /* [E] pointeur sur la zone memoire a ecrire	*/

{
int	i;	/* indice de boucle */
int	indic;  /* indicateur de recherche du mot cle a modifier */
int	p1;	/* pointeur sur le mot cle a modifier */
int	p2;	/* pointeur sur le mot cle suivant */
int	num_mot_cle; /* numero du mot cle a modifier */ 
t_SD_BAS_OK_KO   cr;	/* code retour */
off_t   offset;	  	
off_t   offreel;
int	taille_init; /* taille en octets de la donnee a modifier */
int	taille_fin; /* taille en octets de la donnee a modifier */
int	indic_ecrit; /* indicateur d'ecriture dans un fichier */
char  * da_ascii;  /* pointeur sur la donnee formatee a ecrire */
int	deplacement; /* valeur du trou a faire dans le fichier */
/*---------------------------------------------------------------------------*/
/*		 	DEBUT DE L'OPERATION	        		     */
/*---------------------------------------------------------------------------*/
da_ascii=NULL;
/* on verifie au prealable que le fichier est effectivement ouvert */
/* SINON on sort en erreur */
if ( fd ==NULL ) 
	{
	cr = SD_BAS_ERREUR_OUV;
	sprintf(s_SD_BAS_ERREUR,"Erreur ouverture du fichier de donnees ");
	goto ERREUR;
	}
/*---------------------------------------------------------------------------*/
/* on appelle alors l'operation de liste de tous les mots cles 	     */
/*---------------------------------------------------------------------------*/
/*	si la liste n'existe pas					     */
if (fd->nb_mot_cle == -1)
	{
	cr=SD_BAS_TraceListe(fd);
	if (cr != OK )
		{
		goto ERREUR;
		}		
	}

i=0;
indic=SD_BAS_NON_TROUVE;

/*---------------------------------------------------------------------------*/
/* recherche du mot cle */
/*---------------------------------------------------------------------------*/
while (indic == SD_BAS_NON_TROUVE && i < fd->nb_mot_cle)
	{
	if (strncmp(mot_cle,(char *)(fd->liste_mot_cle + i),SD_BAS_TAILLE_MAX_CLE) == 0 )
		{
		indic = SD_BAS_TROUVE; 
		num_mot_cle=i;
		}
	i++;
	}
/*---------------------------------------------------------------------------*/
/* SI le mot cle n'existe pas 					     */
/*---------------------------------------------------------------------------*/
if (indic == SD_BAS_NON_TROUVE)
	{
/*	ALORS on sort en erreur        					     */
	cr=SD_BAS_MC_INCONNU;
	sprintf(s_SD_BAS_ERREUR,"Erreur : le mot cle %s n'existe pas dans le fichier",mot_cle);
	goto ERREUR; 
	}
else
	{
/*---------------------------------------------------------------------------*/
/*	SINON on recupere le pointeur sur le mot cle dans le fichier (p1)    */	
/*---------------------------------------------------------------------------*/
	p1 = *(fd->ad_mot_cle + num_mot_cle);
/*---------------------------------------------------------------------------*/
/*		ainsi que le pointeur sur le mot cle suivant (p2)	     */
/*---------------------------------------------------------------------------*/
/*	SI le mot cle a modifier est le dernier mot cle			     */
	if ( num_mot_cle == fd->nb_mot_cle -1)
		{
/* on se positionne sur la fin du fichier ( rewind ) 		     */
		offset=0;
		offreel=lseek(fd->descr,offset,SEEK_END);
/* SI erreur on sort "erreur lors d'un lseek du fichier de donnees " 	     */
		if ( offreel == -1)
			{
			cr=SD_BAS_PB_LSEEK;
			sprintf(s_SD_BAS_ERREUR,"erreur lors du lseek sur le  fichier %s",fd->nom_fichier);
			goto ERREUR;
			}
		offreel=lseek(fd->descr,offset,SEEK_CUR);
/* SI erreur on sort "erreur lors d'un lseek du fichier de donnees" 	     */
		if ( offreel == -1)
			{
			cr=SD_BAS_PB_LSEEK;
			sprintf(s_SD_BAS_ERREUR,"erreur lors du lseek sur le  fichier %s",fd->nom_fichier);
			goto ERREUR;
			}
		p2=offreel;
		}
	else
/*	SINON */
		{
		p2= *(fd->ad_mot_cle + num_mot_cle+1);
		}
/*	FIN SI le mot cle a modifier est le dernier mot cle		     */
/*---------------------------------------------------------------------------*/
/*		la taille de la donnee a modifier est p2 - p1 octets ti      */
/*---------------------------------------------------------------------------*/
	taille_init = p2 - p1;
/*---------------------------------------------------------------------------*/
/*		on formate dans une chaine de caracteres  la donnee a ecrire */
/*		(operation interne)( terminee par un \0)		     */
/*---------------------------------------------------------------------------*/
	indic_ecrit=0;
	cr=SD_BAS_TraceEcritBuf(fd,		/* [E] file descriptor du fichier de donnees	*/
				mot_cle,	/* [E] mot cle associe aux donnees */
				type,		/* [E] type des donnees a ecrire		*/
				nb_lig,		/* [E] nombre de lignes du tableau a ecrire	*/
				nb_col,		/* [E] nombre de colonnes du tableau a ecrire	*/
				taille_cell,	/* [E] taille maxi en octets d'un element	*/
				/*     (significatif si type = ascii)		*/
				donnee,	        /* [E] pointeur sur la zone memoire a ecrire	*/
				indic_ecrit,	/* [E] indicateur si ecriture du buffer dans 	*/
					/*     le fichier (=1) sinon (=0)	        */
				&da_ascii	/* buffer  en ascii (sous forme 	*/
					/* de x fois SD_BAS_TAILLE_MAX_LIG caracteres)	*/
		  );
	if (cr != OK)
		{
		goto ERREUR;
		}
/*		la taille de la donnee a ecrire est la taille de la chaine tf*/
	taille_fin=strlen(da_ascii);	
/*---------------------------------------------------------------------------*/
/*		on cree alors un trou dans le fichier  de taille  tf-ti      */
/*		en commencant a l'adresse p2 ( operation interne )	     */
/*---------------------------------------------------------------------------*/
	deplacement= taille_fin -taille_init;
	cr=SD_BAS_TraceDeplacefic( 
				fd  ,	  /* [E] file descripteur du fichier ouvert en lecture ecriture  */
			        p2,     /* [E] adresse de debut de la zone a deplacer */
			        deplacement/* [E] valeur deplacement <0 on racourcit le fichier  */
					  /*     valeur deplacement >0 on allonge le fichier    */
			   );
	if (cr != OK)
		{
		goto ERREUR;
		}

/*---------------------------------------------------------------------------*/
/*		on ecrit alors la donnee a ecrire a partir de l'adresse p1   */
/*			(operation interne qui ecrit directement la chaine ) */
/*			de caracteres precedente)			     */
/*---------------------------------------------------------------------------*/
	cr=SD_BAS_TraceEcrireFic(
			     fd,      /* [E] structure decrivant le fichier de donnees   */
			     p1,      /* [E] adresse dans le fichier ou ecrire le buffer */ 
			     taille_fin,      /* [E] taille de la chaine ascii a ecrire */
         	      	     &da_ascii        /* [E] chaine a ecrire*/
		   );
	if (cr != OK)
		{
		goto ERREUR;
		}
/*		liberation de la chaine de caracteres allouee		     */
	if ( da_ascii != NULL)
		{
		free(da_ascii);
	        da_ascii = NULL;
		}
	}
/*    FINSI le mot cle existe						     */		
/*---------------------------------------------------------------------------*/
/* on met a jour la structure SD_BAS_FILE ( buffer , liste ...)		     */ 
/*---------------------------------------------------------------------------*/
fd->num_buffer=-1;
fd->nb_mot_cle=-1;
if (fd->liste_mot_cle != NULL)
	{
	free(fd->liste_mot_cle);
        fd->liste_mot_cle=NULL;
	}
if (fd->ad_mot_cle != NULL)
	{
	free(fd->ad_mot_cle);
	fd->ad_mot_cle= NULL;
	}
if (fd->ad_donnee != NULL)
	{
	free(fd->ad_donnee);
        fd->ad_donnee= NULL;
	}
fd->ad_dern_mot_cle=-1;
/*---------------------------------------------------------------------------*/
/* fin de l'operation en cas nominal  				     */
/*---------------------------------------------------------------------------*/
return(OK);
/*---------------------------------------------------------------------------*/
/* fin de l'operation en cas d'erreur 				     */
/*---------------------------------------------------------------------------*/
ERREUR:
if (da_ascii != NULL)
	{
	free(da_ascii);
         da_ascii=NULL;
	}
/*---------------------------------------------------------------------------*/
/* on met a jour la structure SD_BAS_FILE ( buffer , liste ...)		     */ 
/*---------------------------------------------------------------------------*/
fd->num_buffer=-1;
fd->nb_mot_cle=-1;
if (fd->liste_mot_cle != NULL)
	{
	free(fd->liste_mot_cle);
        fd->liste_mot_cle = NULL;
	}
if (fd->ad_mot_cle != NULL)
	{
	free(fd->ad_mot_cle);
	fd->ad_mot_cle=NULL;
	}
if (fd->ad_donnee != NULL)
	{
	free(fd->ad_donnee);
	fd->ad_donnee=NULL;
	}
fd->ad_dern_mot_cle=-1;
return(cr);
}
/*
   Fin de l'operation SD_BAS_TraceModifie
*/

/*****************************************************************************/
/*                                                                           */
/* OPERATION: SD_BAS_TraceEcrireFic                                        */
/*                                                                           */
/* ROLE: Cette operation interne ecrit ds un fichier  un buffer a partir     */
/* 	d'une adres specifiee par l'utilisateur d'un nomb d'octets specifie  */
/*      par l'utilisateur .                      		             */
/*         ( utilisation de rewind , fseek , write ) 			     */
/*         ( buffer probable de 4096 octets)                                 */
/*                                                                           */
/*                                                                           */
/* CONTEXTE D'APPEL:                                                         */
/*                                                                           */
/* VERSION:                                                                  */
/* CODE RETOUR : OK ou KO 						     */
/*   " erreur lors du rewind sur le fichier de donnees "		     */
/*   " erreur lors du fseek sur le fichier de donnees ( ad_deb =%d) "	     */
/*   " erreur a l'ecriture du fichier de donnees  %s"                        */
/*****************************************************************************/
static t_SD_BAS_OK_KO SD_BAS_TraceEcrireFic(
			       t_SD_BAS_CpfFile * fd,
			       int ad_deb,
			       int taille,
			       char ** buff
			       )

      /* [E] structure decrivant le fichier donnees   */
      /* [E] adresse dans le fichier ou ecrire le buffer */ 
       /* [E] taille de la chaine ascii a ecrire */
       /* [E] chaine a ecrire*/
{
off_t offset;  /* offset dans le fichier de donnees  */
off_t offreel; /* offset reel obtenu apres lseek */
int   nb_ecrits;   /* nombres d'octets ecrit */
t_SD_BAS_OK_KO cr;      /* code retour */	
/*---------------------------------------------------------------------------*/
/*		 	DEBUT DE L'OPERATION	        		     */
/*---------------------------------------------------------------------------*/
cr=OK;
/* on se positionne sur le debut  du fichier ( lseek ) 		     */
offset=0;
offreel=lseek(fd->descr,offset,SEEK_SET);
/* SI erreur on sort "erreur lors du rewind du fichier de donnees " 	     */
if ( offreel == -1)
	{
	cr=SD_BAS_PB_LSEEK;
	sprintf(s_SD_BAS_ERREUR,"erreur lors du lseek sur le fichier %s",fd->nom_fichier);
	goto ERREUR;
	}
/* on se deplace de ad_deb octets dans le fichier (fseek) 		     */
offset=ad_deb;
offreel=lseek(fd->descr,offset,SEEK_SET);
/* SI erreur on sort " erreur lors du lseek du fichier  (ad_deb=%d)"  */  
if ( offreel == -1)
	{
	cr=SD_BAS_PB_LSEEK;
	sprintf(s_SD_BAS_ERREUR,"erreur lors du lseek sur le fichier %s",fd->nom_fichier);
	goto ERREUR;
	}
/* on ecrit le buffer ( write )			 	             */
nb_ecrits=write(fd->descr,*buff,taille);
/* SI taille_ecrite <> taille buff on sort " erreur a l'ecriture du fichier de donnee ..."       */
if ( nb_ecrits != taille)
	{
	sprintf(s_SD_BAS_ERREUR,"erreur lors du write sur le fichier %s",fd->nom_fichier);
	cr=SD_BAS_PB_WRITE;
	goto ERREUR;
	}
/*-------------------------------------------------------------------*/
/* fin de l'operation en cas nominal  				     */
/*-------------------------------------------------------------------*/
return(OK);
/*-------------------------------------------------------------------*/
/* fin de l'operation en cas d'erreur 				     */
/*-------------------------------------------------------------------*/
ERREUR:
return(cr);	
}
/*
   Fin de l'operation SD_BAS_TraceEcrireFic
*/

/*****************************************************************************/
/*                                                                           */
/* OPERATION: SD_BAS_TraceDeplacefic                                       */
/*                                                                           */
/* ROLE: Cette operation interne cree un trou ou compacte un fichier .       */
/*	precedenment ouvert en lecture ecriture . 			     */
/*	ex1 : avant [+++++++++++++++++++++++++++]			     */
/*	                         |--> deplacement			     */
/*				 V					     */
/*                             ad_deb                                        */
/*	      apres [+++++++++++++   ++++++++++++++]			     */
/*	                         |--> deplacement			     */
/*				 V					     */
/*                             ad_deb                                        */
/*	ex2 : avant [+++++++++++++++++++++++++++]			     */
/*	           deplacement<--|					     */
/*				 V					     */
/*                             ad_deb                                        */
/*	      apres [++++++++++++++++++++++++]				     */
/*	                      |				     	   	     */
/*			      V						     */
/*                             ad_deb                                        */
/* NB: on lit et on ecrit des buffers de taille fd->taille_buffer	     */
/*                                                                           */
/*                                                                           */
/* CONTEXTE D'APPEL:                                                         */
/*                                                                           */
/* VERSION:                                                                  */
/*****************************************************************************/
static t_SD_BAS_OK_KO SD_BAS_TraceDeplacefic(
				t_SD_BAS_CpfFile * fd,
				int debut,
				int deplacement)

     /* [E] file descripteur fichier ouvert lec-ecr */
     /* [E] adresse de debut de la zone a deplacer */
     /* [E] valeur deplacement <0 on racourcit le fichier  */
     /*     valeur deplacement >0 on allonge le fichier    */
{
int fin_fichier_avant_transfert;	/* fin fichier origine */
int nb_octets_a_transferer;		/* nombre total d'octets a transferer */
int nb_octets_transferes;		/* nombre total d'octets transferes */
int nb_octets_a_lire;			/* longueur courante du buffer a lire */
int nb_octets_lus;			/* longueur du buffer lu */
int adresse_debut_buffer;		/* adresse debut courante a lire */
t_SD_BAS_OK_KO retour; 
off_t cr; /* code retour des fonctions appelees */
int   cr2;
/*---------------------------------------------------------------------------*/
/*		 	DEBUT DE L'OPERATION	        		     */
/*---------------------------------------------------------------------------*/
retour =lseek(fd->descr,0L,SEEK_END);
if ( retour == -1) 
	{
	 sprintf(s_SD_BAS_ERREUR,"erreur lors d'un lseek sur le fichier ");
	 retour=SD_BAS_PB_LSEEK ;
	 goto ERREUR; /* pb lseek */
	}
fin_fichier_avant_transfert=retour;
if ( debut < 0) {
	 sprintf(s_SD_BAS_ERREUR,"erreur debut de la zone a deplacer negative (%d)",debut);
	 retour=SD_BAS_DEB_NEG;
	 goto ERREUR; /* debut negatif */
	}
if ( debut > fin_fichier_avant_transfert) {
	 sprintf(s_SD_BAS_ERREUR,"erreur debut de la zone a deplacer en dehors du fichier ");
	 retour=SD_BAS_FIN_DEP;
	 goto ERREUR; /* pb debut en dehors du fichier */
	}
if ( debut + deplacement < 0) {
	 sprintf(s_SD_BAS_ERREUR,"pb de tassement: deplacement de la zone impossible ");
	 retour=SD_BAS_ECRASEMENT;
	 goto ERREUR; /* pb tassement du fichier en tete */
	}
nb_octets_transferes=0;
nb_octets_a_transferer=fin_fichier_avant_transfert - debut;

if ( deplacement >= 0)
	{
	adresse_debut_buffer=maxi(debut,fin_fichier_avant_transfert - fd->taille_buffer);
	}
else
	{
	adresse_debut_buffer=debut;
	}

/* TANT QUE tous les octets n'ont pas ete transferes */
while (nb_octets_transferes < nb_octets_a_transferer)
	{
	nb_octets_a_lire = mini(fd->taille_buffer, nb_octets_a_transferer - nb_octets_transferes);
	cr = lseek(fd->descr,adresse_debut_buffer,SEEK_SET);
	if(  cr != adresse_debut_buffer)
		{
	 	sprintf(s_SD_BAS_ERREUR,"erreur lors d'un lseek sur le fichier ");
		retour=SD_BAS_PB_LSEEK;
		goto ERREUR;
		} 
	nb_octets_lus = read(fd->descr,fd->buffer,nb_octets_a_lire);
	if (nb_octets_lus != nb_octets_a_lire)
		{
	 	sprintf(s_SD_BAS_ERREUR,"erreur de lecture dans le fichier ");
		retour=SD_BAS_PB_READ;
		goto ERREUR;
		}
	cr= lseek(fd->descr,adresse_debut_buffer+deplacement,SEEK_SET);

	if ( cr != (adresse_debut_buffer + deplacement))
		{
	 	sprintf(s_SD_BAS_ERREUR,"erreur lors d'un lseek sur le fichier ");
		retour=SD_BAS_PB_LSEEK;
		goto ERREUR;
		}
	cr2=write(fd->descr,fd->buffer,nb_octets_lus);
	if ( cr2 != nb_octets_lus )
		{
	 	sprintf(s_SD_BAS_ERREUR,"erreur d'ecriture dans le fichier ");
		retour=SD_BAS_PB_WRITE;
		goto ERREUR;		
		}
	nb_octets_transferes = nb_octets_transferes + nb_octets_lus;
	if ( deplacement >= 0)
		{
		adresse_debut_buffer=maxi( adresse_debut_buffer - fd->taille_buffer,debut);
		}
	else
		{
		adresse_debut_buffer=mini( adresse_debut_buffer + fd->taille_buffer,fin_fichier_avant_transfert);
		}
	}
/* FIN TANT QUE tous les octets n'ont pas ete transferes */
if (deplacement < 0 )
	{
	retour=ftruncate(fd->descr,(size_t)(fin_fichier_avant_transfert+deplacement));
	if (retour != 0)
		{
	 	sprintf(s_SD_BAS_ERREUR,"erreur lors d'un ftruncate sur le fichier");
		retour=SD_BAS_PB_FTRUNC;
		goto ERREUR;
		}
	}
/*-------------------------------------------------------------------*/
/* fin de l'operation en cas nominal  				     */
/*-------------------------------------------------------------------*/
return(SD_BAS_OK);
/*-------------------------------------------------------------------*/
/* fin de l'operation en cas d'erreur 				     */
/*-------------------------------------------------------------------*/
ERREUR:
return(retour);	
}
/*
   Fin de l'operation SD_BAS_TraceDeplacefic
*/


/*****************************************************************************/
/*                                                                           */
/* OPERATION:  SD_BAS_TraceEcritFichierLum                                        */
/*                                                                           */
/* ROLE:                                                                     */
/*                                                                           */
/*                                                                           */
/* CONTEXTE D'APPEL:                                                         */
/*                                                                           */
/* VERSION:                                                                  */
/* CODES RETOUR: 						             */
/* 		                                          		     */
/*****************************************************************************/

static t_SD_BAS_OK_KO SD_BAS_TraceEcritFichierLum(
		       char *nom_fichier,
		       void *donnee,
		       int nblig,
		       int nbcol,
		       t_SD_BAS_Codage codage
		       )
{


  int 	flags;            /* flag utilise pour la fonction open           */
  int	mode;             /* valeur du umask utilise pou la fonction open */
  int	fd;               // file descriptor du fichier lum
  unsigned char *header;   /* entete du fichier lum           */
  int    *donnee_it;       // donnee sous le format INTE
  double *donnee_fl;       // donnee sous le format FLOA

  int		nboct_pix;    /* nombre d'octets par pixel */
 
  int           ret;         // code de retour
t_SD_BAS_OK_KO	retour;	     /* code retour des fonctions        */
  int           taille;     // tille du buffer ecrit




// OUVERTURE DU FIHIER

flags = O_RDWR|O_CREAT;
mode  = PERMS;


fd = open(nom_fichier,flags,mode);	
  
if (fd == -1)
{
  sprintf(s_SD_BAS_ERREUR,"Erreur a l'ouverture du fichier %s",nom_fichier);
  retour = SD_BAS_KO;
}

// DETECTION DU CODAGE DU FICHIER
 if (strncmp(codage,CODAGE_DOUBLE,4) ==0)
   {
     nboct_pix=8;
   }
 else if( strncmp ( codage,CODAGE_ENTIER,4 ) == 0 )
   {
     nboct_pix=4;
   }
 else
   {
     sprintf(s_SD_BAS_ERREUR,"Type de codage LUM non reconnu : %s\n",codage);
	   
   }

// CREATION DE L'ENTETE
 header = (unsigned char*)calloc(MAX(nbcol*nboct_pix,SD_BAS_ENTETELUM),sizeof(unsigned char));

if (header == NULL)
   {
      sprintf(s_SD_BAS_ERREUR,"Erreur alloc: Creation fichier %s impossible\n",
				nom_fichier);
      retour = SD_BAS_KO;
      goto FIN;
     
   }
  
 
/*********************************************/
/* ecriture du nombre de colonnes de l'image */
/*********************************************/
   memcpy(header,&nbcol , 4 );
   // FT-2765
   SWAP_INT_BUFFER(header,sizeof(nbcol),1);
   // ...FT-2765          

/*******************************************/
/* ecriture du nombre de lignes de l'image */
/*******************************************/
   memcpy(&header[4],&nblig , 4 );
   // FT-2765
   SWAP_INT_BUFFER(&header[4],sizeof(nblig),1);
   // ...FT-2765          

/**************************************/
/* ecriture type de codage de l'image */
/**************************************/
   

    memcpy(&header[8],codage , 4 );
  
  ret =  write ( fd, header , MAX(nbcol*nboct_pix, SD_BAS_ENTETELUM));
  if ( ret == -1 )
    {
      sprintf(s_SD_BAS_ERREUR,"Erreur : ecriture donnees impossible dans fichier %s\n",
	      nom_fichier ); 
      retour =  SD_BAS_PB_WRITE;
      goto FIN;
    }

  if(header != NULL)
    free(header);

  
if ( ret == 0 )
   {
      sprintf(s_SD_BAS_ERREUR,"Erreur : ecriture entete impossible dans fichier %s\n",
		  nom_fichier ); 
      retour =  SD_BAS_PB_WRITE;
      goto FIN;
   }
 
// ECRITURE DES DONNEES
 taille = nbcol * nboct_pix;

 
 if ( strncmp(codage,CODAGE_DOUBLE,4)==0){
   donnee_fl=(double*)donnee;
	 // FT-2765
   SWAP_INT_BUFFER(donnee_fl,sizeof(double),nblig*nbcol);
   // ...FT-2765          
   ret= write ( fd,donnee_fl , taille*nblig);
   // FT-2765
   SWAP_INT_BUFFER(donnee_fl,sizeof(double),nblig*nbcol);
   // ...FT-2765          
    if ( ret != taille*nblig )
      {
	sprintf(s_SD_BAS_ERREUR,"Erreur : ecriture donnees impossible dans fichier %s\n",
		nom_fichier ); 
	retour =  SD_BAS_PB_WRITE;
	goto FIN;
      }

 }

if ( strncmp(codage,CODAGE_ENTIER,4)==0){

   donnee_it=(int*)donnee;
	 // FT-2765
   SWAP_INT_BUFFER(donnee_it,sizeof(int),nblig*nbcol);
   // ...FT-2765          
   ret = write ( fd,donnee_it , taille*nblig);
	 // FT-2765
   SWAP_INT_BUFFER(donnee_it,sizeof(int),nblig*nbcol);
   // ...FT-2765          
   if ( ret !=  taille*nblig )
      {
	sprintf(s_SD_BAS_ERREUR,"Erreur : ecriture donnees impossible dans fichier %s\n",
		nom_fichier ); 
	retour =  SD_BAS_PB_WRITE;
	goto FIN;
      }
 }

   
 close(fd);
 return(SD_BAS_OK);



 FIN :
   return(retour);
} 

/*****************************************************************************/
/*                                                                           */
/* OPERATION:  SD_BAS_TraceLireFichierLum                                    */
/*                                                                           */
/* ROLE:                                                                     */
/*                                                                           */
/*                                                                           */
/* CONTEXTE D'APPEL:                                                         */
/*                                                                           */
/* VERSION:                                                                  */
/* CODES RETOUR: 						             */
/* 		                                          		     */
/*****************************************************************************/
static t_SD_BAS_OK_KO SD_BAS_TraceLireFichierLum(char *nom_fichier, 
				     int nblig,
				     int nbcol,
				     t_SD_BAS_Codage codage,
				     void * donnee)

{
  int flags;             /* flag utilise pour la fonction open           */
 
  int fd;                // file descriptor du fichier lum
  int nbcol_lu;          //nombre de colonnes lu
  int nblig_lu;          //nombre de ligne lu
  t_SD_BAS_Codage codage_lu;    //type de codage lu
  int  nboct_pix;	/* nombre d'octets par pixel */

  int taille;           // taille du buffer lu
   int ret;             // code retour
   t_SD_BAS_OK_KO	retour;	 /* code retour des fonctions        */
 
 
 
  flags = O_RDONLY;

 
  fd = open(nom_fichier,flags);


if (fd == -1)
{
  sprintf(s_SD_BAS_ERREUR,"Erreur a l'ouverture du fichier %s",nom_fichier);
  retour = SD_BAS_KO;
  goto FIN;
}




/* Lecture de l'entete */
  ret=read(fd,&nbcol_lu,4);
  // FT-2765
  SWAP_INT_BUFFER(&nbcol_lu,sizeof(int),1);
  // ...FT-2765          
  ret=read(fd,&nblig_lu,4);
  // FT-2765
  SWAP_INT_BUFFER(&nblig_lu,sizeof(int),1);
  // ...FT-2765          
  ret=read(fd,codage_lu,4);
 

/* Test de coherence  */

  if( nbcol_lu != nbcol || nblig_lu != nblig || strncmp(codage_lu, codage, 4)!=0)
    {
      sprintf(s_SD_BAS_ERREUR,"Donnees du fichier trace et entete fichier %s incoherente\n",
	     nom_fichier);
      retour=SD_BAS_KO;
      goto FIN;
    }


/*  recuperation du type de codage des pixels */

 if (strncmp(codage,CODAGE_DOUBLE,4) ==0)
	{
    	       	   nboct_pix=8;

	}
 else  if (strncmp(codage,CODAGE_ENTIER,4) ==0)
	{
    	       	   nboct_pix=4;

	}


 
/* positionnement au debut des donnees */	
 lseek(fd,MAX(SD_BAS_ENTETELUM,nboct_pix*nbcol),0);


/* lecture des donnees */
 if(strncmp(codage,CODAGE_DOUBLE,4) == 0){
   
   taille = nbcol * nboct_pix* nblig;

 
   ret = read ( fd, (double *)donnee, taille);
     
  // FT-2765
  SWAP_INT_BUFFER(donnee,sizeof(double),nbcol*nblig);
  // ...FT-2765          

   if( ret <= 0 )
     {
       sprintf(s_SD_BAS_ERREUR,"Erreur : Lecture donnees impossible dans fichier %s\n"
	      ,nom_fichier);
       retour = SD_BAS_PB_READ;
       goto FIN;
     }

 }

 else  if (strncmp(codage,CODAGE_ENTIER,4) ==0){
   
   taille = nbcol * nboct_pix* nblig;
 
   ret = read ( fd,(int*)donnee , taille );
  // FT-2765
  SWAP_INT_BUFFER(donnee,sizeof(int),nbcol*nblig);
  // ...FT-2765          
   if( ret <= 0 )
     {
        sprintf(s_SD_BAS_ERREUR,"Erreur : Lecture donnees impossible dans fichier %s\n"
	      ,nom_fichier);
       retour = SD_BAS_PB_READ;
       goto FIN;
     }
  


 }


  close(fd);
  return(SD_BAS_OK);

  FIN :
    return(retour);

}

