//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/BAS/src/SD_BAS_Traces.c,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_BAS
//@ MODULE : SD_BAS_Traces
//@
//@ Comment :
//@    Code file for the  SD_BAS_Traces class which encapsulates the
//@    debug generation facility for IASI OPS  :
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ VERSION : 15/11/2002 : [reference :] Ott
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <stdio.h>
#include <sys/timeb.h>
#include "SD_BAS_TraceOutils.h"
#include "SD_BAS_Traces.h"
#include <pthread.h>

 /*declaration des fonctions importees*/

/*====================================================================*/
/* Local Constant Declaration                                         */
/*====================================================================*/
#define SD_BAS_TAILLE_MAX_VAR_ENV      1024
#define SD_BAS_MAX_TRIGRAMMES          100
#define SD_BAS_TRIGRAMME_SIZE          8
#define SD_BAS_TAILLE_CLE              60
#define SD_BAS_TAILLE_MAX_STRING       50
#define SD_BAS_TAILLE_MAX_LNSN         10

// definition des messages d'erreurs
char * SD_BAS_MSG_OUVERT = "Fichier de traces deja ouvert";
char * SD_BAS_MSG_ENV = "variable d'environnement REP_DEBUG_DIR introuvable";
char *SD_BAS_MSG_REP = "Creation repertoire impossible";
char *SD_BAS_MSG_TRIG = "Extraction des trigrammes impossible";
char *SD_BAS_MSG_TRACE_OPEN = "Impossible d'ouvrir le fichier de trace";
char *SD_BAS_MSG_AJOUTE = "ajout d'une trace impossible";
char *SD_BAS_MSG_TRACE_FERME = "Fichier de traces ferme";
char *SD_BAS_MSG_FERMETURE = "Fermeture du fichier de traces impossible";
char *SD_BAS_MSG_CHOIX = "Choix impossible";
char *SD_BAS_MSG_TAILLE_CLE = " SIZE OF KEYWORD IS TOO LARGE";
/*====================================================================*/
/* Local types Declaration                                            */
/*====================================================================*/
//======================================================================
// Internal fonctions Declaration
//======================================================================
static t_CMN__Statut SD_BAS_IsTrace(
		char *                fonction,
		t_SD_BAS_DataThread * ctx);
  
void SD_BAS_FormaterNomFonction(char *fonction, char *trigramme);
//======================================================================
// Global variables Definition
//======================================================================
/* variable decrivant si le fichier de trace et ouvert */
static int s_Init=0;
/* tableau des fonctions a tracer */
static char s_headings[SD_BAS_MAX_TRIGRAMMES][SD_BAS_TRIGRAMME_SIZE];
/* variable permetant de tout tracer si = 1 */
static int s_All = 0;
/* descripteur du fichier de trace */
static t_SD_BAS_CpfFile *s_Filedescr=NULL;
/* temps CPU en seconde */
static int s_Cpu_time_sec;
/* temps CPU en milli seconde */
static unsigned short s_Cpu_time_millis;
/* repertoire de debug */
const char * REP_BIN_DEBUG_DIR = "REP_DEBUG_DIR";
/* nom des fonctions a tracer */
//const char * OPS_DEBUG = "OPS_DEBUG";
/* taille des mots cles dans les fichiers de traces */
// Mutex pour les variables conditionnelles POSIX
static     pthread_mutex_t   s_MutexTravaille;

static int s_NbLigne = 0;
static int s_NbSn = 0;
static int s_nbheadings =0;
static int s_Listelignes[SD_BAS_TAILLE_MAX_LNSN];
static int s_ListeSN[SD_BAS_TAILLE_MAX_LNSN];

//======================================================================
//					FONCTIONS EXTERNES
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_InitTraces
//@
//@ Description : ouverture et initialisation du fichier de traces 
//@   ( cette fonction est thread safe )
//@
//@
//@ Parameters :
//@			(ENTREE)  validity_date : date de debut du granule 
//@			(ENTREE)  numorbite : numero d'orbite 
//@			(ENTREE)  numgranule : numero de granule 
//@			(ENTREE/SORTIE)  Ctx : contexte lu
//@
//@ Return Status : 	t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@		SD_BAS_ERR_ENVIRONNEMENT : lecture des variables d'environnement
//@                                        impossible
//@             SD_BAS_ERR_OUVERTURE : ouverture impossible
//@             SD_BAS_ERR_ALLOCATION    :  allocation impossible
//@             SD_BAS_ERR_REPERTOIRE  : creation repertoire impossible
//@             SD_BAS_ERR_AJOUTE  : ajout de donnees dans le fichier de traces
//@                                 impossible
//@             SD_BAS_ERR_TRIGRAMME : extraction des trigrammes impossible
//@             SD_BAS_ERR_FICHIER_OUVERT : fichier de traces deja ouvert
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_InitTraces(
  t_SD_BAS_SCDSTime     validity_date, // date de fin du granule
  int                   numgranule, // issu du contexte ( 0 en debut de dump)
  t_SD_BAS_Contexte *   Ctx,
  t_SD_BAS_DataThread * ctx)
  
{

  /* nom de la foncion */ 
  static const char tcProcName_F[] = "SD_BAS_InitTraces";
  /* retour de la fonction */
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  /* retour de fonction */
  t_SD_BAS_OK_KO cr;
  /* nom du fichier de trace */
  char nom_fichier[SD_BAS_LGNOMFICHIER];         
  /* mot cle a ecrire dans le fichier */
  char  mot_cle[SD_BAS_TAILLE_MAX_CLE];  
  /* type d ouverture */
  const e_SD_BAS_CpfModeOuvre mode_ouv = SD_BAS_ECRITURE;
  /* nombre de lignes des donnees a ecrire */
  const int nbligne =1;     
  /* nombre de colonnes des donnees a ecrire */
  const int nbcolonne =1;
  /* taille maxi en octets 1 element donnee */
  int taille_cell; 
  /* type de donnee */
  const t_SD_BAS_CpfType type = SD_BAS_ASCII;  
  /* nombre de traces prevus */
  int nbheadings;              
  /* buffer de travail */
  char * buffer;
  /* buffer de travail */
  char * tok;
  /* repertoire ou ecrire les fichiers de traces */
  char *DebugDir;
  /* pointeur rendant strtok thread safe */
  char *pointer;
  /* temps CPU */
  struct timeb Tmill;
  /* temps au format ascii */
  char * date;
  /* temps au format entier */
  struct tm *pt_time;
  /* pointeur rendant localtime thread safe */
  struct tm CurrentTime;
  /* pointeur rendant asctime thread safe */
  char buffertime[SD_BAS_TAILLE_MAX_STRING];
  int ret;
  char buffer2[10];
  
  // Entree dans la fonction 
   
  //@ appel fonction de creation du mutex
  pthread_mutex_init(&s_MutexTravaille,NULL);

  //@ test pour savoir si le fichier de trace est ouvert ( s_Init = 0)
  if(s_Init == 1)
  {
    SD_BAS_TEST_ERR_INT(
      ctx->cr.errinterne, 
      SD_BAS_ERR_SET_traces, 
      SD_BAS_ERR_NUM_JDB, 
      tcProcName_F, 
      __LINE__, 
      SD_BAS_MSG_OUVERT);
    cr_op = SD_BAS_ERR_FICHIER_OUVERT;
    goto FIN;
  }

  //@ extraction de la variable d'environnement REP_BIN_DEBUG_DIR
  /* on recupere le repertoire ou ecrire les fichiers de traces */
  /* s'il n'existe pas on emet une erreur */
  DebugDir = getenv(REP_BIN_DEBUG_DIR);

  if(DebugDir == NULL)
  {
    SD_BAS_TEST_ERR_INT( 
      ctx->cr.errinterne, 
      SD_BAS_ERR_SET_traces, 
      SD_BAS_ERR_NUM_JDB, 
      tcProcName_F, 
      __LINE__, 
      SD_BAS_MSG_ENV);
    cr_op = SD_BAS_ERR_ENVIRONNEMENT;
    goto FIN;
  }
   
  if( SD_BAS_MacMALLOC(DebugDir, char, SD_BAS_LGNOMFICHIER) == FALSE)
  {
    SD_BAS_TEST_ERR_INT(
      ctx->cr.errinterne, 
      SD_BAS_ERR_SET_traces, 
      SD_BAS_ERR_NUM_JDB, 
      tcProcName_F, 
      __LINE__, 
      SD_BAS_MSG_ALLOC);
    cr_op = SD_BAS_ERR_BAS_NIVEAU;
    goto FIN;
  }

  sprintf(DebugDir,"%s/",getenv(REP_BIN_DEBUG_DIR));

  //@ verification de l'existence du repertoire/
  if(access(DebugDir,R_OK) == -1)
  {
    SD_BAS_TEST_ERR_INT( 
      ctx->cr.errinterne, 
      SD_BAS_ERR_SET_traces, 
      SD_BAS_ERR_NUM_JDB, 
      tcProcName_F, 
      __LINE__, 
      SD_BAS_MSG_REP);
    cr_op = SD_BAS_ERR_REPERTOIRE;
    goto FIN;
  }
 
  //@ constitution du nom du fichier suivant la nomenclature
  /* avec le repertire, la version, la date et le numero de granule */
  sprintf(nom_fichier,"%sDEB_%d_%d_%d:%d_%d.tra",
    DebugDir, 
    Ctx->GCTXVersOPS.major, 
    Ctx->GCTXVersOPS.minor, 
    validity_date.day, 
    validity_date.millis, 
    numgranule);
 
  /* on libere le nom du repertoire */
  SD_BAS_MacFREE(DebugDir);

  // FT-2765/FT-2766
  if(access(nom_fichier,R_OK) == 0)
  // ...FT-2765/FT-2766
    unlink(nom_fichier);


  //@ appel fonction TraceOpen et stockage du resultat dans s_Filedescr
  //@ si tout s'est bien passe s_Init = 1

  s_Filedescr = SD_BAS_TraceOpen( nom_fichier,mode_ouv);

  /* si le fichier n'est pas ouvert on place l'indicateur a 1 */
  if( s_Filedescr != NULL )
    s_Init = 1;  
  /* sinon on emet une erreur */
  else
  {
    SD_BAS_TEST_ERR_INT(
      ctx->cr.errinterne, 
      SD_BAS_ERR_SET_traces,	
      SD_BAS_ERR_NUM_JDB, 
      tcProcName_F, 
      __LINE__,
      SD_BAS_MSG_TRACE_OPEN);
    cr_op = SD_BAS_ERR_OUVERTURE;
    goto FIN;
  }
 
  //@ ecriture de l'entete du fichier 
  //@ nom du fichier <NOM> ("xxxxxxxx")

  /* initialisation du mot-cle */
  sprintf(mot_cle,"NAME");

  taille_cell= sizeof (char)*strlen(nom_fichier)+1;

  /* ecriture dans le fichier de trace */
  cr = SD_BAS_TraceAjoute(
    s_Filedescr, 
    mot_cle, 
    type, 
    nbligne, 
    nbcolonne, 
    taille_cell, 
    nom_fichier);

  if(cr == KO)
  {
    SD_BAS_TEST_ERR_INT(
      ctx->cr.errinterne, 
      SD_BAS_ERR_SET_traces, 
      SD_BAS_ERR_NUM_JDB, 
      tcProcName_F, 
      __LINE__, 
      SD_BAS_MSG_AJOUTE);
    cr_op = SD_BAS_ERR_AJOUTE;
    goto FIN;
  }

  //@ on initialise le timer pour mesurer le temps CPU global
  //@ on initialise le timer pour mesurer le temps ellapsed global

  /* on recupere le temps CPU */
  ftime(&Tmill);
  /* on le convertie au format ASCII */
  pt_time = localtime_r(&Tmill.time,&CurrentTime);
  date= asctime_r(pt_time,buffertime);

  /* on initialise les variables globales **/
  s_Cpu_time_sec=Tmill.time;
  s_Cpu_time_millis=Tmill.millitm;
 
  //@ date de debut de traitement <DATE_DEB> ("xxxxxxxx")
  //@ appel Compfic_ajoute
 
  /* on initialise le mot cle */
  sprintf(mot_cle,"DATE_DEB" );

  taille_cell= sizeof (char)*strlen(date)+1;

  /* on ecrit dans le fichier de trace */
  cr = SD_BAS_TraceAjoute(
    s_Filedescr, 
    mot_cle, 
    type, 
    nbligne, 
    nbcolonne, 
    taille_cell, 
    date);

  if(cr == KO)
  {
    SD_BAS_TEST_ERR_INT(
      ctx->cr.errinterne, 
      SD_BAS_ERR_SET_traces, 
		    SD_BAS_ERR_NUM_JDB, 
      tcProcName_F, 
      __LINE__, 
      SD_BAS_MSG_AJOUTE);
    cr_op = SD_BAS_ERR_AJOUTE;
	   goto FIN;
  }

  //@ recuperation de la variable d'environnemeent 
  //@ extraction de la variable d'environnement OPS_DEBUG

  /* on recupere le variable d'environnement decrivant les fonctions a ecrire */

  //@ recuperation des chaines separees par des : dans la variable
  //@    si le mot vaut ALL : alors tout doit etre trace
  //@	 sinon stocker les trigramme dans un tableau interne statique 
  //@    (taille max =  100)
  buffer = getenv("OPS_DEBUG");

  if(buffer == NULL)
  {
    /* si OPS_DEBUG n'est pas definit on trace rien */
    s_All = 0;
  }
  else
  {
    /* on initialise le nombre de trigrammes a 0 */
    nbheadings=0;
     
    /* on extrait de la variable d'environnement le premier string separe par ':'*/
    if ( (tok=strtok_r(buffer,":",&pointer)) != NULL )
    {
    	 /*  Boucle sur chaque trigramme */ 
	     do
      {
    	   /* si le nom est trop long */
	       if ( strlen(tok) > SD_BAS_TRIGRAMME_SIZE )
	       {
	         SD_BAS_TEST_ERR_INT(
            ctx->cr.errinterne, 
            SD_BAS_ERR_SET_traces, 
            SD_BAS_ERR_NUM_JDB, 
            tcProcName_F, 
            __LINE__, 
            SD_BAS_MSG_TRIG);
	         cr_op = SD_BAS_ERR_TRIGRAMME;
	         goto FIN; 
  	     }
	       else
	       {
	         /* si le mot vaut ALL */
	         if ( !strcmp(tok,"ALL") )
		        {
	           /* on incremente le nombre de trigramme */
		          nbheadings = nbheadings + 1;
       		   s_nbheadings = nbheadings;
	       
       		   /* si le nombre de trigrammes est superieue au nombre max on emet */
            /* une erreur*/
       		   if ( nbheadings >= SD_BAS_MAX_TRIGRAMMES ) 
		          {
		            SD_BAS_TEST_ERR_INT(
                ctx->cr.errinterne, 
                SD_BAS_ERR_SET_traces, 
                SD_BAS_ERR_NUM_JDB, 
                tcProcName_F, 
                __LINE__, 
                SD_BAS_MSG_TRIG);
        		   cr_op = SD_BAS_ERR_TRIGRAMME;
		           goto FIN;
     		     }
		          /* sinon on initialise la variable globale s_All */
       		   else
		          {
		            s_All = 1;
		          }
		   
		        }
	         /*si le mot est different de ALL */
	         else
		        {
		          /* on incremente le nombre de trigramme */
		          nbheadings = nbheadings + 1;
       		   s_nbheadings = nbheadings;
		   
		          /* si le nombre de trigrammes est superieue au nombre max on emet */
            /* une erreur*/
       		   if ( (nbheadings >= SD_BAS_MAX_TRIGRAMMES) || 
                 (strlen(tok)>SD_BAS_TRIGRAMME_SIZE) )
		          {
		            SD_BAS_TEST_ERR_INT(
                ctx->cr.errinterne, 
					           SD_BAS_ERR_SET_traces, 
                SD_BAS_ERR_NUM_JDB, 
					           tcProcName_F, 
					           __LINE__, 
                SD_BAS_MSG_TRIG);
		            cr_op = SD_BAS_ERR_TRIGRAMME;
		            goto FIN;
     		     } 
      		    /* sinon on stocke le trigramme */
		          else
     		       /* on recopie le trigramme dan s le tableau */
		            strcpy(s_headings[nbheadings-1],tok);
		        } /* fin si le mot est different de ALL */
	       } /* fin si le nom n'est pas trop long */
    	 } /* fin do */
   	 /* tant qu'il y a des trigrammes on boule */
	     while( ( (tok=strtok_r(NULL,":",&pointer)) != NULL) &&
		           (cr_op ==CMN__STATUT_OK) );
    } /* fin lecture du 1er element separe par ':' */
  } /* fin si OPS_DEBUG defini */

  /* on recupere le variable d'environnement decrivant les lignes a ecrire */
  //@ recuperation des chaines separees par des : dans la variable
  //@    si la variable n'est pas initialisee : alors tout doit etre trace
  //@	 sinon stocker les trigramme dans un tableau interne statique 
  //@    (taille max =  10)

  //@ recuperation des chaines separees par des : dans la variable
  buffer = getenv("OPS_DEBUG_LN");

  if(buffer == NULL)
  {
    /* si OPS_DEBUG n'est pas definit on trace toutes les lignes */
    s_NbLigne = 100;
  }
  else
  {
    /* on initialise le nombre de trigrammes a 0 */
    s_NbLigne=0;
     
    /* on extrait de la variable d'environnement le premier string separe par ':'*/
    if ( (tok=strtok_r(buffer,":",&pointer)) != NULL )
    {
    	 /*  Boucle sur chaque trigramme */ 
	     do
      {
   		   /* on incremente le nombre de lignes */
   		   s_NbLigne = s_NbLigne + 1;
        
   		   /* si le nombre de lignes  est superieue au nombre max on emet une erreur*/
		      if ( s_NbLigne >= SD_BAS_TAILLE_MAX_LNSN  )
		      {
		        SD_BAS_TEST_ERR_INT(
            ctx->cr.errinterne, 
					       SD_BAS_ERR_SET_traces, 
					       SD_BAS_ERR_NUM_JDB, 
					       tcProcName_F, 
					       __LINE__, 
            SD_BAS_MSG_TRIG);
		        cr_op = SD_BAS_ERR_TRIGRAMME; 
		        goto FIN;
 		     }
   		   /* sinon on stocke le trigramme */		     
		      /* on recopie le trigramme dan s le tableau */
		      strcpy(buffer2,tok);
		      /* on decode la valeur */
		      ret=sscanf(buffer2,"%d",&s_Listelignes[s_NbLigne-1]);
		      
        /* on ignore les valeurs non decodees */
		      if (ret !=1) s_NbLigne--;
      } /* fin do */
   	  /* tant qu'il y a des trigrammes on boule */
	     while( ( (tok=strtok_r(NULL,":",&pointer)) != NULL) &&
            	(cr_op ==CMN__STATUT_OK) );
    } /* fin lecture 1er element separe par ':' */
  } /* fin si OPS_DEBUG_LN defini */

  /* on recupere le variable d'environnement decrivant les SN a ecrire */
  //@ recuperation des chaines separees par des : dans la variable
  //@    si la variable n'est pas initialisee : alors tout doit etre trace
  //@	 sinon stocker les trigramme dans un tableau interne statique 
  //@    (taille max =  10)

  //@ recuperation des chaines separees par des : dans la variable
  buffer = getenv("OPS_DEBUG_SN");

  if(buffer == NULL)
  {
    /* si OPS_DEBUG n'est pas definit on trace toutes les lignes */
    s_NbSn = 100;
  }
  else
  {
    /* on initialise le nombre de trigrammes a 0 */
    s_NbSn=0;
     
    /* on extrait de la variable d'environnement le premier string separe par ':'*/
    if ( (tok=strtok_r(buffer,":",&pointer)) != NULL )
    {
    	 /*  Boucle sur chaque trigramme */ 
	     do
      {
   		   /* on incremente le nombre de lignes */
		      s_NbSn = s_NbSn + 1;
        
		      /* si le nombre de lignes  est superieue au nombre max on emet une erreur*/
   		   if ( s_NbSn >= SD_BAS_TAILLE_MAX_LNSN  )
		      {
		        SD_BAS_TEST_ERR_INT(
            ctx->cr.errinterne, 
            SD_BAS_ERR_SET_traces, 
            SD_BAS_ERR_NUM_JDB, 
            tcProcName_F, 
            __LINE__, 
            SD_BAS_MSG_TRIG);
		        cr_op = SD_BAS_ERR_TRIGRAMME; 
		        goto FIN;
 		     }
		   
		      /* sinon on stocke le trigramme */		     
		      /* on recopie le trigramme dan s le tableau */
		      strcpy(buffer2,tok);
		      /* on decode la valeur */
		      ret=sscanf(buffer2,"%d",&s_ListeSN[s_NbSn-1]);
		      /* on ignore les valeurs non decodees */
		      if (ret !=1) s_NbSn--;
    	 } /* fin do */
   	  /* tant qu'il y a des trigrammes on boule */
	     while( ( (tok=strtok_r(NULL,":",&pointer)) != NULL) &&
           		(cr_op ==CMN__STATUT_OK) );
    } /* fin lecture 1er element separe par ':' */
  } /* fin si OPS_DEBUG_SN defini */

  // on trace ici si besoin toutes les variables en sortie
FIN:
// sortie de la fonction
  return(cr_op); 
} /* fin de SD_BAS_InitTraces */

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_FinTraces
//@
//@ Description : fermeture  du fichier de traces 
//@   ( cette fonction est thread safe )
//@
//@
//@ Parameters : aucun
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@		SD_BAS_ERR_AJOUTE  : ajout de donnees dans le fichier de traces
//@                                 impossible
//@            SD_BAS_ERR_OUVERT : fichier de traces pas  ouvert
//@            SD_BAS_ERR_FERMETURE : fermeture du fichier impossible
//@            SD_BAS_ERR_BAS_NIVEAU : allocation impossible
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_FinTraces(t_SD_BAS_DataThread *ctx)
{ 

/* nom de la foncion */ 
static const char tcProcName_F[] = "SD_BAS_FinTraces";
/* retour de la fonction */
t_CMN__Statut cr_op=CMN__STATUT_OK;
/* retour de fonction */
t_SD_BAS_OK_KO cr;
/* nombre de lignes des donnees a ecrire */
const int nbligne = 1;
/* nombre de colonnes des donnees a ecrire */
const int nbcolonne = 1;
/* type de donnee */
const t_SD_BAS_CpfType type = SD_BAS_ASCII; 
/* taille maxi en octets 1 element donnee */
int taille_cell;
/* mot cle a ecrire dans le fichier */
 char mot_cle[SD_BAS_TAILLE_MAX_CLE];
 /* temps CPU */
 struct timeb Tmill;
 /* temps au format ascii */
 char * date;
 /* pointeur rendant localtime thread safe */
 struct tm CurrentTime;
 /* temps au format entier */
 struct tm *pt_time;
 /* pointeur rendant asctime thread safe */
 char buffertime[50];
 /* temps ecoule depuis l'ouverture du fichier de traces */
 /* en seconde */
 int Elapsed_time_sec;
 /* milliseconde */
 int Elapsed_time_millis;
 /* message a ajouter a fichier de traces */
 char *message;



/* Entree dans la fonction */
/* test pour savoir si le fichier de trace est ouvert ( s_Init = 1) */

//@ appel fonction de suppression du mutex
pthread_mutex_destroy(&s_MutexTravaille);

  if( s_Init == 0){

    /* s'il est ouvert on emet une erreur */
   SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                       SD_BAS_ERR_SET_traces,
                       SD_BAS_ERR_NUM_JDB,
                       tcProcName_F, __LINE__,SD_BAS_MSG_TRACE_FERME);
   cr_op = SD_BAS_ERR_FICHIER_FERME;
   goto FIN;
 }

  /* on utilise  le timer CPU pour mesurer le temps CPU global */
  /* on utilier le timer ellapsed pour mesurer le temps ellapsed global */

  /* on recupere le temps CPU */
 ftime(&Tmill);
 /* on le convertie au format entier */
 pt_time = localtime_r(&Tmill.time,&CurrentTime);
 /* on le convertie au format ASCII */
 date= asctime_r(pt_time,buffertime);

 /* temps ellapsed de traitement <ELLAPSED> ("xxxxxxxx") */

 /* Calcul du temps ecoule depuis l'ouverture du fichier */
 Elapsed_time_sec =   Tmill.time -  s_Cpu_time_sec;
 Elapsed_time_millis =  Tmill.millitm - s_Cpu_time_millis;

 /* si le nmbre de milli secondes est negatif */
 if( Elapsed_time_millis < 0)
   {
     /* on calule la retenue */
     Elapsed_time_sec ++;
     Elapsed_time_millis = 1000 + Elapsed_time_millis;
    
   }

 /* allocation d'un string */
 if(SD_BAS_MacMALLOC(message,char,SD_BAS_TAILLE_MAX_STRING) == FALSE)
   {
     /* si l'allocation echoue on emet une erreur */
     SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                         SD_BAS_ERR_SET_traces,
                         SD_BAS_ERR_NUM_JDB,tcProcName_F, 
                         __LINE__, SD_BAS_MSG_ALLOC);
     cr_op = SD_BAS_ERR_BAS_NIVEAU;
     goto FIN;
   }

 /* creation de du message contenant la duree du traitement */
 sprintf(message," %d sec  %d msec",Elapsed_time_sec, Elapsed_time_millis );

 /* initialisation du mot cle */
 strcpy(mot_cle,"TIME ELAPSED");
 /* initialisation de la taille */
 taille_cell= sizeof (char)*strlen(message)+1;

 /* ajout du message au fichier de traces */
 cr = SD_BAS_TraceAjoute(s_Filedescr,mot_cle,type,nbligne,
			 nbcolonne,taille_cell,message);
 
 /* si l'ajout echoue on emet une erreur */
 if(cr == KO)
   {
     
     SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_traces,
			 SD_BAS_ERR_NUM_JDB, tcProcName_F, __LINE__,SD_BAS_MSG_AJOUTE);
        

       cr_op = SD_BAS_ERR_AJOUTE;
       goto FIN;
     }
   
    /* liberation du buffer */
   SD_BAS_MacFREE(message);


   /* ajout du temps cpu de traitement <CPU> ("xxxxxxxx") */

   /* initialisation du mot cle */
   strcpy(mot_cle,"DATE_FIN");
   /* initialisation de la taille */
   taille_cell= sizeof (char)*strlen(date)+1;
   
   // appel Compfic_ajoute
   /* ecriture dans le fichier de traces */
   cr = SD_BAS_TraceAjoute(s_Filedescr,mot_cle,type,nbligne,
			   nbcolonne,taille_cell,date);
 
   if(cr == KO)
     {
       SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_traces,
                         SD_BAS_ERR_NUM_JDB,tcProcName_F, __LINE__,SD_BAS_MSG_AJOUTE);
       cr_op = SD_BAS_ERR_AJOUTE;
       goto FIN;
   }
 

   //@ appel fonction Compfic_close et initialisation de  s_Filedescr
   /* fermeture du fichier de traces */
   cr = SD_BAS_TraceClose( s_Filedescr);
   /* si la fermeture echoue on emet une erreur */
   if(cr == KO)
     {
       
       SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_traces,
			   SD_BAS_ERR_NUM_JDB,tcProcName_F, __LINE__, SD_BAS_MSG_FERMETURE );
       cr_op = SD_BAS_ERR_FERMETURE;
       goto FIN;
   }
   //@ si tout s'est bien passe s_Init = 0 et s_All=0
   s_Init=0;
   s_All=0; 

   // on trace ici si besoin toutes les variables en sortie

FIN:
// sortie de la fonction 
/* on retourne le status */
return(cr_op); 

}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_TraceReel1D
//@
//@ Description : ecriture  du tableau 1D de reels
//@
//@
//@ Parameters : 
//@			(ENTREE)  fonction : nom de la fonction sous la forme
//@				d'un trigramme (XXX)
//@			(ENTREE)  num : numero de ligne et SN traitee
//@				  ( les champ sont ajoutees au nom de la variable 
//@				  ( si differents de 0)
//@			(ENTREE)  nomvar : nom de la variable a ecrire
//@			(ENTREE)  dimension : dimension du tableau de 
//@					   reels 
//@			(ENTREE)  stride : offset entre chaque element 
//@					   du tableau 
//@			
//@			(ENTREE)  tab : tableau 1D de reels 
//@
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@		SD_BAS_ERR_AJOUTE  : ajout de donnees dans le fichier de traces
//@                                 impossible
//@             SD_BAS_ERR_OUVERT : fichier de traces pas  ouvert
//@             SD_BAS_ERR_BAS_NIVEAU : allocation impossible7
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_TraceReel1D(
		char * fonction,
		t_SD_BAS_ORIGINE *num,
		char * nomvar,
		int dimension,
		int stride,
		  double *tab,
		  t_SD_BAS_DataThread *ctx
			)
{

  /* nom de la foncion */
  static const char tcProcName_F[] = "SD_BAS_TraceReel1D";
  /* retour de la fonction */
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  /* retour de fonction */
  t_SD_BAS_OK_KO cr;
  /* mot cle a ecrire dans le fichier */
  char mot_cle[SD_BAS_TAILLE_MAX_CLE];
  /* taille du sous-tableau de donnee */
  int taille;
  /* sous-tableau de donnee */
  double *tab_stride;
  /* indice de boucle */
  int i;
  /* nombre de lignes des donnees a ecrire */
  const int nbligne = 1;
  /* type de donnee */
  const t_SD_BAS_CpfType type = SD_BAS_REEL8 ;
  /* taille maxi en octets 1 element donnee */
  const int taille_cell = 0;
  /* buffer temporaire */
  char buffer[SD_BAS_TAILLE_MAX_CLE];
  /*nom du trigramme */
  char trigramme[SD_BAS_TAILLE_MAX_CLE];

  // Entree dans la fonction 
  // test pour savoir si le fichier de trace est ouvert ( s_Init = 1)
  if(s_Init == 0)
   {
     
     SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                         SD_BAS_ERR_SET_traces,
                         SD_BAS_ERR_NUM_JDB,
                         tcProcName_F, __LINE__,SD_BAS_MSG_TRACE_FERME);
     cr_op = SD_BAS_ERR_FICHIER_FERME;
     goto FIN;
   }


 // transformation du nom dde la fonction ( SD_ALG_11_XXX -> 1_XXX)
 SD_BAS_FormaterNomFonction(fonction, trigramme);


 //@ on parcourt les traces requises que l'on compare avec le nom de 
 //@ la fonction si = alors trace=1 
 //@ si trace = 1 ou s_All=1
 if(SD_BAS_IsTrace(trigramme, ctx) == CMN__STATUT_OK)
   {
   
  // creation du mot cle  avec le nom de la fonction et le nom de la variable
     
     sprintf(mot_cle,"%s_%s",trigramme, nomvar);

  // ajout du numero de ligne au mot cle
     if( num->LN != 0)   
       {    
	  sprintf(buffer,"_%d",num->LN);
	 strcat(mot_cle,buffer);
       }
  // ajout du numero de cycle au mot cle
     if( num->SN != 0)   
      {    
	  sprintf(buffer,"_%d",num->SN);
	 strcat(mot_cle,buffer);
       }
 //  ajout du numero de pixel au mot cle
     if( num->PN != 0)   
      {    
	  sprintf(buffer,"_%d",num->PN);
	 strcat(mot_cle,buffer);
       }
 //  ajout de la direction cu coin de cube au mot cle
     if( num->CCD != 0)   
      {    
	  sprintf(buffer,"_%d",num->CCD);
	 strcat(mot_cle,buffer);
       }
// ajout du numero de bande au mot cle
//     if( num->SB != 0)   
//      {    
//	  sprintf(buffer,"_%d",num->SB);
//	 strcat(mot_cle,buffer);
//       }
 
    
  
     
 
     if (strlen(mot_cle) > SD_BAS_TAILLE_CLE)
       {
	 SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_traces,	
			     SD_BAS_ERR_NUM_JDB,tcProcName_F, __LINE__,SD_BAS_MSG_TAILLE_CLE);
	 cr_op = SD_BAS_ERR_TAILLE_CLE;
	 goto FIN;
       }


  //@	si stride <> 1
 if(stride>1 && stride<dimension)
   {
     // calcul de la taille du nouveau tableau  
   taille=(int)(dimension/stride)+1;

   // on s'alloue un tableau pour recopier les valeurs
     if(SD_BAS_MacMALLOC(tab_stride,double,taille)==FALSE)
       {

	SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
			     SD_BAS_ERR_SET_traces,
			     SD_BAS_ERR_NUM_JDB,tcProcName_F, 
			     __LINE__, SD_BAS_MSG_ALLOC);
	cr_op = SD_BAS_ERR_BAS_NIVEAU;
	goto FIN;	      
       }

     // recopie des valeurs
     for(i=0;i<taille;i++)
       *(tab_stride+i)=*(tab+ i*stride);

     //@ appel fonction de reservation du mutex
     pthread_mutex_lock(&s_MutexTravaille);
     //   appel Compfic_ajoute
     cr = SD_BAS_TraceAjoute(s_Filedescr,mot_cle,type,
			  nbligne,taille,taille_cell,tab_stride);
     //@ appel fonction de liberation du mutex
     pthread_mutex_unlock(&s_MutexTravaille);
    if(cr == KO)
    {
      SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_traces,	
                          SD_BAS_ERR_NUM_JDB,tcProcName_F, __LINE__,SD_BAS_MSG_AJOUTE);
      cr_op = SD_BAS_ERR_AJOUTE;
      goto FIN;

   }
   
    //liberation du tableau temporaire
      SD_BAS_MacFREE(tab_stride);
   }
  
 // si stride = 1 
 else
   {
     //@ appel fonction de reservation du mutex
     pthread_mutex_lock(&s_MutexTravaille);
     // on recopie le tableau en entree
     cr =  SD_BAS_TraceAjoute(s_Filedescr,mot_cle,type,
			  nbligne,dimension,taille_cell,tab);
      //@ appel fonction de liberation du mutex
     pthread_mutex_unlock(&s_MutexTravaille);

   if(cr == KO)
     {
       SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_traces,
                           SD_BAS_ERR_NUM_JDB,tcProcName_F, __LINE__,SD_BAS_MSG_AJOUTE);
       cr_op = SD_BAS_ERR_AJOUTE;
       goto FIN;

     }
   }
  

 
   }// si all  

// on trace ici si besoin toutes les variables en sortie

FIN:
// sortie de la fonction 
return(cr_op); 

}

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_TraceReel2D
//@
//@ Description : ecriture  du tableau 2D de reels
//@
//@
//@ Parameters : 
//@			(ENTREE)  fonction : nom de la fonction sous la forme
//@				d'un trigramme (XXX)
//@			(ENTREE)  num : numero de ligne et SN traitee
//@				  ( les champ sont ajoutees au nom de la variable 
//@				  ( si differents de 0)
//@			(ENTREE)  nomvar : nom de la variable a ecrire
//@			(ENTREE)  nbligne : nombre de ligne du tableau
//@			(ENTREE)  nbcolonne : nombre de colonne du tableau
//@			
//@			(ENTREE)  tab : tableau 2D de reels (supposes contigus) 
//@
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@		SD_BAS_ERR_AJOUTE  : ajout de donnees dans le fichier de traces
//@                                 impossible
//@              SD_BAS_ERR_FICHIER_FERME : fichier de traces pas  ouvert
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_TraceReel2D(
		char * fonction,
		t_SD_BAS_ORIGINE *num,
	       	char * nomvar,
		int nbligne,
		int nbcolonne,
		double *tab,
		t_SD_BAS_DataThread *ctx
				 )
{
  /* nom de la foncion */ 
  static const char tcProcName_F[] = "SD_BAS_TraceReel2D";
  /* retour de la fonction */
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  /* retour de fonction */
  t_SD_BAS_OK_KO cr;
  /* mot cle a ecrire dans le fichier */
  char  mot_cle[SD_BAS_TAILLE_MAX_CLE]; 
  /* type de donnee */
  const t_SD_BAS_CpfType type = SD_BAS_REEL8 ;
  /* taille maxi en octets 1 element donnee */
  const int taille_cell = 0;
  /* buffer temporaire */
  char buffer[SD_BAS_TAILLE_MAX_CLE];
  /* nom du trigramme */
  char trigramme[SD_BAS_TAILLE_MAX_CLE];

 //@ test pour savoir si le fichier de trace est ouvert ( s_Init = 1)
 if(s_Init == 0)
   {
     SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                         SD_BAS_ERR_SET_traces,
                         SD_BAS_ERR_NUM_JDB,
                         tcProcName_F, __LINE__, SD_BAS_MSG_TRACE_FERME);
     cr_op = SD_BAS_ERR_FICHIER_FERME;
		   
     goto FIN;
   }


 // transformation du nom dde la fonction ( SD_ALG_11_XXX -> 1_XXX)
 SD_BAS_FormaterNomFonction(fonction, trigramme);


 //@ on parcourt les traces requises que l'on compare avec le nom de 
 //@ la fonction si = alors trace=1 ( fonction interne a creer)
 //@ si trace = 1 ou s_Alls=1
 if(SD_BAS_IsTrace(trigramme, ctx) == CMN__STATUT_OK)
   {


  // creation du mot cle  avec le nom de la fonction et le nom de la variable

     sprintf(mot_cle,"%s_%s",trigramme,nomvar);

  // ajout du numero de ligne au mot cle
     if( num->LN != 0)   
       {    
	  sprintf(buffer,"_%d",num->LN);
	 strcat(mot_cle,buffer);
       }
  // ajout du numero de cycle au mot cle
     if( num->SN != 0)   
      {    
	  sprintf(buffer,"_%d",num->SN);
	 strcat(mot_cle,buffer);
       }
 //  ajout du numero de pixel au mot cle
     if( num->PN != 0)   
      {    
	  sprintf(buffer,"_%d",num->PN);
	 strcat(mot_cle,buffer);
       }
 //  ajout de la direction cu coin de cube au mot cle
     if( num->CCD != 0)   
      {    
	  sprintf(buffer,"_%d",num->CCD);
	 strcat(mot_cle,buffer);
       }
// ajout du numero de bande au mot cle
     if( num->SB != 0)   
      {    
	  sprintf(buffer,"_%d",num->SB);
	 strcat(mot_cle,buffer);
       }
    
     if (strlen(mot_cle) > SD_BAS_TAILLE_CLE)
       {
	 SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_traces,	
			     SD_BAS_ERR_NUM_JDB,tcProcName_F, __LINE__,SD_BAS_MSG_TAILLE_CLE);
	 cr_op = SD_BAS_ERR_TAILLE_CLE;
	 goto FIN;
       }


  //@   appel Compfic_ajoute

     //@ appel fonction de reservation du mutex
     pthread_mutex_lock(&s_MutexTravaille);
     cr =  SD_BAS_TraceAjoute(s_Filedescr,mot_cle,type,
			      nbligne, nbcolonne,taille_cell,tab);
     //@ appel fonction de liberation du mutex
     pthread_mutex_unlock(&s_MutexTravaille);

  
  if(cr == KO)
    {
      SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_traces,
                          SD_BAS_ERR_NUM_JDB,tcProcName_F, __LINE__,SD_BAS_MSG_AJOUTE);
      cr_op = SD_BAS_ERR_AJOUTE;
      goto FIN;
        
    }

  //@ fin si
   }
FIN:
// sortie de la fonction 
return(cr_op); 

}

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_TraceEntier1D
//@
//@ Description : ecriture  du tableau 1D d'entier
//@
//@
//@ Parameters : 
//@			(ENTREE)  fonction : nom de la fonction sous la forme
//@				d'un trigramme (XXX)
//@			(ENTREE)  num : numero de ligne et SN traitee
//@				  ( les champ sont ajoutees au nom de la variable 
//@				  ( si differents de 0)
//@			(ENTREE)  nomvar : nom de la variable a ecrire
//@			(ENTREE)  dimension : dimension du tableau d'entiers 
//@				   
//@			(ENTREE)  stride : offset entre chaque element 
//@					   du tableau 
//@			
//@			(ENTREE)  tab : tableau 1D d'entier 
//@
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@		SD_BAS_ERR_AJOUTE  : ajout de donnees dans le fichier de traces
//@                                 impossible
//@             SD_BAS_ERR_FICHIER_FERME : fichier de traces pas  ouvert
//@             SD_BAS_ERR_BAS_NIVEAU : allocation impossible
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_TraceEntier1D(
		char * fonction,
		t_SD_BAS_ORIGINE *num,
		char * nomvar,
		int dimension,
		int stride,
		int *tab,
		t_SD_BAS_DataThread *ctx
			)
{
  /* nom de la foncion */
  static const char tcProcName_F[] = "SD_BAS_TraceEntier1D";
  /* retour de la fonction */
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  /* retour de fonction */
  t_SD_BAS_OK_KO cr;
  /* mot cle a ecrire dans le fichier */
  char  mot_cle[SD_BAS_TAILLE_MAX_CLE]; 
  /* taille du sous-tableau de donnee */
  int taille;
  /* sous-tableau de donnee */
  int *tab_stride;
  /* indice de boucle */
  int i;
  /* nombre de lignes des donnees a ecrire */
  const int nbligne = 1;
  /* type de donnee */
  const t_SD_BAS_CpfType type = SD_BAS_ENTIER4 ;
  /* taille maxi en octets 1 element donnee */
  const int taille_cell = 0;
  /* buffer temporaire */
  char buffer[SD_BAS_TAILLE_MAX_CLE];
  /* nom du trigramme */
  char trigramme[SD_BAS_TAILLE_MAX_CLE];


// Entree dans la fonction 
  //@ test pour savoir si le fichier de trace est ouvert ( s_Init = 1)

 if(s_Init == 0)
   {
     SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                         SD_BAS_ERR_SET_traces,
                         SD_BAS_ERR_NUM_JDB,
                         tcProcName_F, __LINE__, SD_BAS_MSG_TRACE_FERME);
     cr_op = SD_BAS_ERR_FICHIER_FERME;
     goto FIN;
   }


 // transformation du nom dde la fonction ( SD_ALG_11_XXX -> 1_XXX) 
 SD_BAS_FormaterNomFonction(fonction, trigramme);

 //@ on parcourt les traces requises que l'on compare avec le nom de 
  //@ la fonction si = alors trace=1 ( fonction interne a creer)
  //@ si trace = 1 ou s_All=1
 if(SD_BAS_IsTrace(trigramme, ctx) == CMN__STATUT_OK)
   {

  //@   on encode le nom de la variable avec les numero de SN PN ...
  //@   on encode le nom de la variable avec les numero de SN PN ...
  //@   en utilisant les arguments

  // creation du mot cle  avec le nom de la fonction et le nom de la variable
     sprintf(mot_cle,"%s_%s",trigramme,nomvar);


  // ajout du numero de ligne au mot cle
     if( num->LN != 0)   
       {    
	  sprintf(buffer,"_%d",num->LN);
	 strcat(mot_cle,buffer);
       }
  // ajout du numero de cycle au mot cle
     if( num->SN != 0)   
      {    
	  sprintf(buffer,"_%d",num->SN);
	 strcat(mot_cle,buffer);
       }
 //  ajout du numero de pixel au mot cle
     if( num->PN != 0)   
      {    
	  sprintf(buffer,"_%d",num->PN);
	 strcat(mot_cle,buffer);
       }
 //  ajout de la direction cu coin de cube au mot cle
     if( num->CCD != 0)   
      {    
	  sprintf(buffer,"_%d",num->CCD);
	 strcat(mot_cle,buffer);
       }
// ajout du numero de bande au mot cle
     if( num->SB != 0)   
      {    
	  sprintf(buffer,"_%d",num->SB);
	 strcat(mot_cle,buffer);
       }
  
     if (strlen(mot_cle) > SD_BAS_TAILLE_CLE)
       {
	 SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_traces,	
			     SD_BAS_ERR_NUM_JDB,tcProcName_F, __LINE__,SD_BAS_MSG_TAILLE_CLE);
	 cr_op = SD_BAS_ERR_TAILLE_CLE;
	 goto FIN;
       }


  //@	si stride <> 1 
 if(stride>1 && stride<dimension)
   {
 // calcul de la taille du nouveau tableau  
     taille=(int)(dimension/stride)+1;

  // on s'alloue un tableau pour recopier les valeurs
     if( SD_BAS_MacMALLOC(tab_stride,int,taille)==FALSE)
       {
         SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                             SD_BAS_ERR_SET_traces,
                             SD_BAS_ERR_NUM_JDB,tcProcName_F, 
                             __LINE__, SD_BAS_MSG_ALLOC);
	 cr_op = SD_BAS_ERR_BAS_NIVEAU;
	 goto FIN;	      
       }

 //recopie des valeurs
     for(i=0;i<taille;i++)
       *(tab_stride+i)=*(tab+ i*stride);

//@ appel fonction de reservation du mutex
     pthread_mutex_lock(&s_MutexTravaille);
     //  appel Compfic_ajoute
     cr = SD_BAS_TraceAjoute(s_Filedescr,mot_cle,type,
                             nbligne,taille,taille_cell,tab_stride);
     
     //@ appel fonction de liberation du mutex
     pthread_mutex_unlock(&s_MutexTravaille);
     if(cr == KO)
       {
         
         SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_traces,
                             SD_BAS_ERR_NUM_JDB,tcProcName_F, __LINE__,SD_BAS_MSG_AJOUTE);
	 cr_op = SD_BAS_ERR_AJOUTE;
	 goto FIN;

       }
     //liberation du tableau temporaire
       SD_BAS_MacFREE(tab_stride);
   }
 
 // si stride = 1  
 else
   {
     //@ appel fonction de reservation du mutex
     pthread_mutex_lock(&s_MutexTravaille);
     // on recopie le tableau en entree
     cr =  SD_BAS_TraceAjoute(s_Filedescr,mot_cle,type,
			      nbligne,dimension,taille_cell,tab);
     //@ appel fonction de liberation du mutex
     pthread_mutex_unlock(&s_MutexTravaille);
     if(cr ==KO)
       {
	 
	 SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
			     SD_BAS_ERR_SET_traces,
			     SD_BAS_ERR_NUM_JDB,tcProcName_F, __LINE__,SD_BAS_MSG_AJOUTE);
	 cr_op = SD_BAS_ERR_AJOUTE; 
	 goto FIN;

       }
   }
  

  //@ fin si
 
   }

// on trace ici si besoin toutes les variables en sortie

FIN:
// sortie de la fonction 
return(cr_op); 

}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_TraceEntier2D
//@
//@ Description : ecriture  du tableau 2D d'entiers
//@
//@
//@ Parameters : 
//@			(ENTREE)  fonction : nom de la fonction sous la forme
//@				d'un trigramme (XXX)
//@			(ENTREE)  num : numero de ligne et SN traitee
//@				  ( les champ sont ajoutees au nom de la variable 
//@				  ( si differents de 0)
//@			(ENTREE)  nomvar : nom de la variable a ecrire
//@			(ENTREE)  nbligne : nombre de ligne du tableau
//@			(ENTREE)  nbcolonne : nombr de colonne du tableau
//@				  
//@			
//@			(ENTREE)  tab : tableau 2D d'entiers (supposes contigus)
//@
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@		SD_BAS_ERR_AJOUTE  : ajout de donnees dans le fichier de traces
//@                                 impossible
//@              SD_BAS_ERR_FICHIER_FERME : fichier de traces pas  ouvert
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_TraceEntier2D(
		char * fonction,
		t_SD_BAS_ORIGINE *num,
		char * nomvar,
		int nbligne,
		int nbcolonne,
		int *tab,
		t_SD_BAS_DataThread *ctx)
{
  /* nom de la foncion */
  static const char tcProcName_F[] = "SD_BAS_TraceEntier2D";
  /* retour de la fonction */
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  /* retour de fonction */
  t_SD_BAS_OK_KO cr;
  /* mot cle a ecrire dans le fichier */
  char  mot_cle[SD_BAS_TAILLE_MAX_CLE]; 
  /* type de donnee */
  const t_SD_BAS_CpfType type = SD_BAS_ENTIER4 ;
  /* taille maxi en octets 1 element donnee */
  const int taille_cell = 0;
  /* buffer temporaire */
  char buffer[SD_BAS_TAILLE_MAX_CLE];
  /* nom du trigramme */
  char trigramme[SD_BAS_TAILLE_MAX_CLE];



// Entree dans la fonction 
  //@ test pour savoir si le fichier de trace est ouvert ( s_Init = 1)


 if(s_Init == 0)
   {
     SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                         SD_BAS_ERR_SET_traces,
                         SD_BAS_ERR_NUM_JDB,
                         tcProcName_F, __LINE__,SD_BAS_MSG_TRACE_FERME);
     cr_op = SD_BAS_ERR_FICHIER_FERME;
     
     goto FIN;
   }



 // transformation du nom dde la fonction ( SD_ALG_11_XXX -> 1_XXX)
 SD_BAS_FormaterNomFonction(fonction,trigramme );


  //@ on parcourt les traces requises que l'on compare avec le nom de 
  //@ la fonction si = alors trace=1 ( fonction interne a creer)
  //@ si trace = 1 ou s_All=1
if( SD_BAS_IsTrace(trigramme, ctx) == CMN__STATUT_OK)
   {
  //@   on encode le nom de la variable avec les numero de SN PN ...
  //@   on encode le nom de la variable avec les numero de SN PN ...
  //@   en utilisant les arguments
  //@

  // creation du mot cle  avec le nom de la fonction et le nom de la variable
     
     sprintf(mot_cle,"%s_%s",trigramme,nomvar);
    
  
  // ajout du numero de ligne au mot cle
     if( num->LN != 0)   
       {    
	  sprintf(buffer,"_%d",num->LN);
	 strcat(mot_cle,buffer);
       }
  // ajout du numero de cycle au mot cle
     if( num->SN != 0)   
      {    
	  sprintf(buffer,"_%d",num->SN);
	 strcat(mot_cle,buffer);
       }
 //  ajout du numero de pixel au mot cle
     if( num->PN != 0)   
      {    
	  sprintf(buffer,"_%d",num->PN);
	 strcat(mot_cle,buffer);
       }
 //  ajout de la direction cu coin de cube au mot cle
     if( num->CCD != 0)   
      {    
	  sprintf(buffer,"_%d",num->CCD);
	 strcat(mot_cle,buffer);
       }
// ajout du numero de bande au mot cle
     if( num->SB != 0)   
      {    
	  sprintf(buffer,"_%d",num->SB);
	 strcat(mot_cle,buffer);
       }
  
     if (strlen(mot_cle) > SD_BAS_TAILLE_CLE)
       {
	 SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_traces,	
			     SD_BAS_ERR_NUM_JDB,tcProcName_F, __LINE__,SD_BAS_MSG_TAILLE_CLE);
	 cr_op = SD_BAS_ERR_TAILLE_CLE;
	 goto FIN;
       } 

  //@   appel Trace_ajoute


     //@ appel fonction de reservation du mutex
     pthread_mutex_lock(&s_MutexTravaille);
     cr =  SD_BAS_TraceAjoute(s_Filedescr,mot_cle,type,
			      nbligne, nbcolonne,taille_cell,tab);
     //@ appel fonction de liberation du mutex
     pthread_mutex_unlock(&s_MutexTravaille);
     if(cr == KO)
       {
	 SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_traces,
			     SD_BAS_ERR_NUM_JDB,tcProcName_F, __LINE__,SD_BAS_MSG_AJOUTE);
	 cr_op = SD_BAS_ERR_AJOUTE;
	 goto FIN; 

   
   }
 //@ fin si
   }
  

// on trace ici si besoin toutes les variables en sortie

FIN:
// sortie de la fonction 
return(cr_op); 

}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_TraceChar
//@
//@ Description : ecriture  d'une chaine de caractere
//@
//@
//@ Parameters : 
//@			(ENTREE)  fonction : nom de la fonction sous la forme
//@				d'un trigramme (XXX)
//@			(ENTREE)  num : numero de ligne et SN traitee
//@				  ( les champ sont ajoutees au nom de la variable
//@				  ( si differents de 0)
//@			(ENTREE)  nomvar : nom de la variable a ecrire
//@			(ENTREE)  dimension : longueur de la chaine de caractere
//@			
//@			(ENTREE)  chaine :  chaine de caractere
//@
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@		SD_BAS_ERR_AJOUTE  : ajout de donnees dans le fichier de traces//@                                 impossible
//@              SD_BAS_ERR_FICHIER_FERME : fichier de traces pas  ouvert
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_TraceChar(
		char * fonction,
		t_SD_BAS_ORIGINE *num,
		char *nomvar,
		int dimension,
		char * chaine,
		t_SD_BAS_DataThread *ctx
		)
{
  /* nom de la foncion */
  static const char tcProcName_F[] = "SD_BAS_TraceChar";
  /* retour de la fonction */
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  /* retour de fonction */
  t_SD_BAS_OK_KO cr;
  /* mot cle a ecrire dans le fichier */
  char  mot_cle[SD_BAS_TAILLE_MAX_CLE];
  /* type de donnee */
  const t_SD_BAS_CpfType type = SD_BAS_ASCII ;
  /* nombre de lignes des donnees a ecrire */
  const int nbligne = 1;
  /* nombre de colonnes des donnees a ecrire */
  const int nbcolonne = 1;
  /* buffer temporaire */
  char buffer[SD_BAS_TAILLE_MAX_CLE];
  /* nom du trigramme */
  char trigramme[SD_BAS_TAILLE_MAX_CLE];



  // Entree dans la fonction 
  //@ test pour savoir si le fichier de trace est ouvert ( s_Init = 1)

 if(s_Init == 0)
  {
   
    SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
		     SD_BAS_ERR_SET_traces,
			SD_BAS_ERR_NUM_JDB,
		     tcProcName_F, __LINE__,SD_BAS_MSG_TRACE_FERME);
    cr_op = SD_BAS_ERR_FICHIER_FERME;
    goto FIN;	   

   }

 // transformation du nom dde la fonction ( SD_ALG_11_XXX -> 1_XXX)  
 SD_BAS_FormaterNomFonction(fonction, trigramme);

//@ on parcourt les traces requises que l'on compare avec le nom de 
  //@ la fonction si = alors trace=1 ( fonction interne a creer)
  //@ si trace = 1 ou s_All=1
 if( SD_BAS_IsTrace(trigramme, ctx) == CMN__STATUT_OK)


   {

 // creation du mot cle  avec le nom de la fonction et le nom de la variable
     
     sprintf(mot_cle,"%s_%s",trigramme,nomvar);

  // ajout du numero de ligne au mot cle
     if( num->LN != 0)   
       {    
	  sprintf(buffer,"_%d",num->LN);
	 strcat(mot_cle,buffer);
       }
  // ajout du numero de cycle au mot cle
     if( num->SN != 0)   
      {    
	  sprintf(buffer,"_%d",num->SN);
	 strcat(mot_cle,buffer);
       }
 //  ajout du numero de pixel au mot cle
     if( num->PN != 0)   
      {    
	  sprintf(buffer,"_%d",num->PN);
	 strcat(mot_cle,buffer);
       }
 //  ajout de la direction cu coin de cube au mot cle
     if( num->CCD != 0)   
      {    
	  sprintf(buffer,"_%d",num->CCD);
	 strcat(mot_cle,buffer);
       }
// ajout du numero de bande au mot cle
     if( num->SB != 0)   
      {    
	  sprintf(buffer,"_%d",num->SB);
	 strcat(mot_cle,buffer);
       }

    if (strlen(mot_cle) > SD_BAS_TAILLE_CLE)
       {
	 SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_traces,	
			     SD_BAS_ERR_NUM_JDB,tcProcName_F, __LINE__,SD_BAS_MSG_TAILLE_CLE);
	 cr_op = SD_BAS_ERR_TAILLE_CLE;
	 goto FIN;
       }


  //@   appel Trace_ajoute

//@ appel fonction de reservation du mutex
    pthread_mutex_lock(&s_MutexTravaille);
    cr = SD_BAS_TraceAjoute(s_Filedescr,mot_cle,type,
			    nbligne, nbcolonne,dimension,chaine);
    
    //@ appel fonction de liberation du mutex
    pthread_mutex_unlock(&s_MutexTravaille);
    
    if(cr == KO)
      {
	SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_traces,
			    SD_BAS_ERR_NUM_JDB,tcProcName_F, __LINE__,SD_BAS_MSG_AJOUTE);
	cr_op = SD_BAS_ERR_AJOUTE;
	goto FIN;  
	
      }
  

  //@ fin si
   }
  

// on trace ici si besoin toutes les variables en sortie

FIN:
// sortie de la fonction 
return(cr_op); 

}

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_TraceFichierEntier
//@
//@ Description : ecriture  d'un fichier binaire d'entier
//@
//@
//@ Parameters :
//@			(ENTREE)  fonction : nom de la fonction sous la forme
//@				d'un trigramme (XXX)
//@			(ENTREE)  num : numero de ligne et SN traitee
//@				  ( les champ sont ajoutees au nom de la variable 
//@				  ( si differents de 0)
//@			(ENTREE)  nomvar : nom de la variable a ecrire
//@			(ENTREE)  nbligne : nombre de ligne du tableau
//@			(ENTREE)  nbcolonne : nombre de colonne du tableau
//@			(ENTREE)  tab :  tableau d'entiers
//@                     
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@		SD_BAS_ERR_AJOUTE  : ajout de donnees dans le fichier de traces
//@                                 impossible
//@              SD_BAS_ERR_FICHIER_FERME : fichier de traces pas  ouvert
//@ ------------------------------------------------------------------*/

t_CMN__Statut SD_BAS_TraceFichierEntier(
		char * fonction,
		t_SD_BAS_ORIGINE *num,
		char *nomvar,
		int nbligne,
		int nbcolonne,
		int *tab,
		t_SD_BAS_DataThread *ctx
		)
{
  /* nom de la foncion */
  static const char tcProcName_F[] = "SD_BAS_TraceFichierEntier";
  /* retour de la fonction */
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  /* retour de fonction */
  t_SD_BAS_OK_KO cr;
  /* mot cle a ecrire dans le fichier */
  char  mot_cle[SD_BAS_TAILLE_MAX_CLE]; 
  /* type de donnee */
  const t_SD_BAS_CpfType type = SD_BAS_FIC_BIN_ENTIER ;
  /* taille maxi en octets 1 element donnee */
  const int taille_cell = 0;
  /* buffer temporaire */
  char buffer[SD_BAS_TAILLE_MAX_CLE];
  /* nom du trigramme */
  char trigramme[SD_BAS_TAILLE_MAX_CLE];




 // Entree dans la fonction 

 //@ test pour savoir si le fichier de trace est ouvert ( s_Init = 1)


 if(s_Init == 0)
  {
    
    SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                        SD_BAS_ERR_SET_traces,
                        SD_BAS_ERR_NUM_JDB,
                        tcProcName_F, __LINE__,SD_BAS_MSG_TRACE_FERME);
    cr_op = SD_BAS_ERR_FICHIER_FERME;
		   
     goto FIN;
   }
 
 // transformation du nom dde la fonction ( SD_ALG_11_XXX -> 1_XXX)
 SD_BAS_FormaterNomFonction(fonction, trigramme);

 //@ on parcourt les traces requises que l'on compare avec le nom de 
  //@ la fonction si = alors trace=1 ( fonction interne a creer)
  //@ si trace = 1 ou s_All=1
if(SD_BAS_IsTrace(trigramme, ctx) == CMN__STATUT_OK)
   {
  // creation du mot cle  avec le nom de la fonction et le nom de la variable
     
     sprintf(mot_cle,"%s_%s",trigramme, nomvar);

  // ajout du numero de ligne au mot cle
     if( num->LN != 0)   
       {    
	  sprintf(buffer,"_%d",num->LN);
	 strcat(mot_cle,buffer);
       }
  // ajout du numero de cycle au mot cle
     if( num->SN != 0)   
      {    
	  sprintf(buffer,"_%d",num->SN);
	 strcat(mot_cle,buffer);
       }
 //  ajout du numero de pixel au mot cle
     if( num->PN != 0)   
      {    
	  sprintf(buffer,"_%d",num->PN);
	 strcat(mot_cle,buffer);
       }
 //  ajout de la direction cu coin de cube au mot cle
     if( num->CCD != 0)   
      {    
	  sprintf(buffer,"_%d",num->CCD);
	 strcat(mot_cle,buffer);
       }
// ajout du numero de bande au mot cle
     if( num->SB != 0)   
      {    
	  sprintf(buffer,"_%d",num->SB);
	 strcat(mot_cle,buffer);
       }
       
  
    if (strlen(mot_cle) > SD_BAS_TAILLE_CLE)
       {
	 SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_traces,	
			     SD_BAS_ERR_NUM_JDB,tcProcName_F, __LINE__,SD_BAS_MSG_TAILLE_CLE);
	 cr_op = SD_BAS_ERR_TAILLE_CLE;
	 goto FIN;
       }

    //@  appel de Trace_ajoute
    //@ appel fonction de reservation du mutex
    pthread_mutex_lock(&s_MutexTravaille);
    cr =SD_BAS_TraceAjoute(s_Filedescr,mot_cle,type,
			   nbligne, nbcolonne,taille_cell,tab);
    //@ appel fonction de liberation du mutex
    pthread_mutex_unlock(&s_MutexTravaille);
    
    if(cr == KO)
      {
	SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_traces,
			    SD_BAS_ERR_NUM_JDB,tcProcName_F, __LINE__,SD_BAS_MSG_AJOUTE);
	cr_op = SD_BAS_ERR_AJOUTE;
      goto FIN;

   }
  //@ fin si
   }
 


FIN:
// sortie de la fonction 
return(cr_op); 

}

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_TraceFichierReel
//@
//@ Description : ecriture  d'un fichier binaire de reels
//@
//@
//@ Parameters : 
//@			(ENTREE)  fonction : nom de la fonction sous la forme
//@				d'un trigramme (XXX)
//@			(ENTREE)  num : numero de ligne et SN traitee
//@				  ( les champ sont ajoutees au nom de la variable 
//@				  ( si differents de 0)
//@			(ENTREE)  nomvar : nom de la variable a ecrire
//@			(ENTREE)  nbligne : nombre de ligne du tableau
//@			(ENTREE)  nbcolonne : nombre de colonne du tableau			
//@			(ENTREE)  tab :  tableau de reels
//@                     
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@		SD_BAS_ERR_AJOUTE  : ajout de donnees dans le fichier de traces
//@                                 impossible
//@              SD_BAS_ERR_FICHIER_FERME : fichier de traces pas  ouvert
//@ ------------------------------------------------------------------*/

t_CMN__Statut SD_BAS_TraceFichierReel(
				      char * fonction,
				      t_SD_BAS_ORIGINE *num,
				      char* nomvar,
				      int nbligne,
				      int nbcolonne,
				      double *tab,
				      t_SD_BAS_DataThread *ctx
				      )
{
  /* nom de la foncion */
  static const char tcProcName_F[] = "SD_BAS_TraceFichierReel";
  /* retour de la fonction */
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  /* retour de fonction */
  t_SD_BAS_OK_KO cr;
  /* mot cle a ecrire dans le fichier */
  char  mot_cle[SD_BAS_TAILLE_MAX_CLE]; 
  /* type de donnee */
  const t_SD_BAS_CpfType type = SD_BAS_FIC_BIN_DOUBLE;
  /* taille maxi en octets 1 element donnee */
  const int taille_cell = 0;
  /* buffer temporaire */
  char buffer[SD_BAS_TAILLE_MAX_CLE];
  /* nom du trigramme */
  char trigramme[SD_BAS_TAILLE_MAX_CLE];



 // Entree dans la fonction 

 //@ test pour savoir si le fichier de trace est ouvert ( s_Init = 1)

 if(s_Init == 0)
   {
     SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                         SD_BAS_ERR_SET_traces,
                         SD_BAS_ERR_NUM_JDB,
                         tcProcName_F, __LINE__,SD_BAS_MSG_TRACE_FERME);
     cr_op = SD_BAS_ERR_FICHIER_FERME;
      goto FIN;
   }
 


 // transformation du nom dde la fonction ( SD_ALG_11_XXX -> 1_XXX)
 SD_BAS_FormaterNomFonction(fonction, trigramme);

  //@ on parcourt les traces requises que l'on compare avec le nom de 
  //@ la fonction si = alors trace=1 ( fonction interne a creer)
  //@ si trace = 1 ou s_All=1
 if( SD_BAS_IsTrace(trigramme, ctx) == CMN__STATUT_OK)
   {

 
  // creation du mot cle  avec le nom de la fonction et le nom de la variable
     
     sprintf(mot_cle,"%s_%s",trigramme,nomvar);

  // ajout du numero de ligne au mot cle
     if( num->LN != 0)   
       {    
	  sprintf(buffer,"_%d",num->LN);
	 strcat(mot_cle,buffer);
       }
  // ajout du numero de cycle au mot cle
     if( num->SN != 0)   
      {    
	  sprintf(buffer,"_%d",num->SN);
	 strcat(mot_cle,buffer);
       }
 //  ajout du numero de pixel au mot cle
     if( num->PN != 0)   
      {    
	  sprintf(buffer,"_%d",num->PN);
	 strcat(mot_cle,buffer);
       }
 //  ajout de la direction du coin de cube au mot cle
     if( num->CCD != 0)   
      {    
	  sprintf(buffer,"_%d",num->CCD);
	 strcat(mot_cle,buffer);
       }
// ajout du numero de bande au mot cle
//     if( num->SB != 0)   
//      {    
//	  sprintf(buffer,"_%d",num->SB);
//	 strcat(mot_cle,buffer);
//       }
 


     if (strlen(mot_cle) > SD_BAS_TAILLE_CLE)
       {
	 SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_traces,	
			     SD_BAS_ERR_NUM_JDB,tcProcName_F, __LINE__,SD_BAS_MSG_TAILLE_CLE);
	 cr_op = SD_BAS_ERR_TAILLE_CLE;
	 goto FIN;
       }

 
//@ appel fonction de reservation du mutex
     pthread_mutex_lock(&s_MutexTravaille);
     //@  appel de Trace_ajoute
     cr = SD_BAS_TraceAjoute(s_Filedescr,mot_cle,type,
			     nbligne, nbcolonne,taille_cell,tab);
     
     //@ appel fonction de liberation du mutex
     pthread_mutex_unlock(&s_MutexTravaille);
     if(cr == KO)
       {
     SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_traces,
                         SD_BAS_ERR_NUM_JDB,tcProcName_F, __LINE__,SD_BAS_MSG_AJOUTE);
	 cr_op = SD_BAS_ERR_AJOUTE;
	 goto FIN;

   }
  //@ fin si
   }
  //@ 
  //@ 



FIN:
// sortie de la fonction 
return(cr_op); 
}



//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_TracePrecision
//@
//@ Description : ecriture  de la precision attendue
//@
//@
//@ Parameters : 
//@			(ENTREE) precision
//@
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@     	SD_BAS_ERR_AJOUTE  : ajout de donnees dans le fichier de traces
//@                                 impossible
//@              SD_BAS_ERR_FICHIER_FERME : fichier de traces pas  ouvert
//@ cette fonction est rendu thread safe par ajout d'un mecanisme de mutex
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_TracePrecision(double precision,
				    t_SD_BAS_DataThread *ctx )
{
  /* nom de la foncion */
  static const char tcProcName_F[] = "SD_BAS_TracePrecision";
  /* retour de la fonction */
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  /* retour de fonction */
  t_SD_BAS_OK_KO cr;
  /* mot cle a ecrire dans le fichier */
  char mot_cle[SD_BAS_TAILLE_MAX_CLE]= "PRECISION";
  /* nombre de lignes des donnees a ecrire */
  const int nbligne = 1;
  /* nombre de colones des donnees a ecrire */
  const int nbcolonne =1;
  /* type de donnee */
  const t_SD_BAS_CpfType type = SD_BAS_REEL8 ;
  /* taille maxi en octets 1 element donnee */
  const int taille_cell = 0;



// Entree dans la fonction 
  //@ test pour savoir si le fichier de trace est ouvert ( s_Init = 1)
 if(s_Init == 0)
   {
     SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                         SD_BAS_ERR_SET_traces,
                         SD_BAS_ERR_NUM_JDB,
                         tcProcName_F, __LINE__,SD_BAS_MSG_TRACE_FERME);
     cr_op = SD_BAS_ERR_FICHIER_FERME;
     goto FIN;
   }

//@ ecriture de la precision attendue
 cr = SD_BAS_TraceAjoute(s_Filedescr,mot_cle,type,
			 nbligne,nbcolonne,taille_cell,&precision);
 if(cr == KO)
   {
     SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_traces,
                         SD_BAS_ERR_NUM_JDB,tcProcName_F, __LINE__,SD_BAS_MSG_AJOUTE);
     cr_op = SD_BAS_ERR_AJOUTE;
     goto FIN;

   }
 
 
 
FIN:
// sortie de la fonction 
return(cr_op); 

}


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_Trace
//@
//@ Description : ecriture  de traces de debugage
//@   ( cette fonction est thread safe )
//@
//@ Parameters : 
//@			(ENTREE)  fonction : nom de la fonction sous la forme
//@				d'un trigramme (XXX)
//@			(ENTREE)  num : numero de ligne et SN traitee
//@				  ( les champ sont ajoutees au nom de la variable 
//@				  ( si differents de 0)
//@			(ENTREE)  nomvar : nom de la variable a ecrire
//@			(ENTREE)  dimension : dimension du tableau de 
//@					   reels 
//@			(ENTREE)  stride : offset entre chaque element 
//@					   du tableau 
//@			
//@			(ENTREE)  tab : tableau 1D de reels 
//@ 
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@		SD_BAS_ERR_AJOUTE  : ajout de donnees dans le fichier de traces
//@                                 impossible
//@             SD_BAS_ERR_OUVERT : fichier de traces pas  ouvert
//@             SD_BAS_ERR_BAS_NIVEAU : allocation impossible7
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_Trace(
		char * fonction,
		char * nomvar,
		int nb_lig,
		int nb_col,
		int stride,
		e_SD_BAS_TypTrace type,
		e_SD_BAS_FormatTrace format,
		void *tab,
		t_SD_BAS_DataThread *ctx
)
		
{

  /* nom de la foncion */

  static const char tcProcName_F[] = "SD_BAS_TraceReel1D";
  /* retour de la fonction */
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  int num_lig;
  char * msg_err;
  char * vide="NULL";

  if(nb_lig == 0 || nb_col ==0)
    {

      cr_op = SD_BAS_TraceChar(fonction,
			       &ctx->ctxtrt,
			       nomvar,
			       1,
			       vide,
			       ctx);


    }
  else
    {
      /* suivant lt type */
      switch(type)
	{
	  
	  /* si type entier */
	case TRACE_INT4:
	case TRACE_UINT4:
	  /* si le format de sortie est ascii */
	  if(format == TRACE_ASC)
	    {
	      /* si la donnee est monodimensionelle */
	      if(nb_lig == 1) 
		/* appel a SD_BAS_TraceEntier1D */
		cr_op = SD_BAS_TraceEntier1D( fonction,
					      &ctx->ctxtrt,
					      nomvar,
					      nb_col,
					      stride,
					      tab,
					      ctx);
	      /* sinon */
	      else 
		/* appel a SD_BAS_TraceEntier2D */
		cr_op = SD_BAS_TraceEntier2D(fonction,		       
					     &ctx->ctxtrt,
					     nomvar,
					     nb_lig,
					     nb_col,
					     tab,
					     ctx);
	    }
	  /* sinon on cree un fichier *.LUM */
	  else
	    /* appel a SD_BAS_TraceFichierEntier */
	    cr_op = SD_BAS_TraceFichierEntier(fonction,		       
					      &ctx->ctxtrt,
					      nomvar,
					      nb_lig,
					      nb_col,
					      tab,
					      ctx);
	  break;
	  
	  
	  /* si les donnees sont reelles */
	case TRACE_REAL8:
	  /* si le format de sortie est ascii */
	  if(format == TRACE_ASC)
	    {
	      /* si la donnee est monodimensionelle */
	      if(nb_lig == 1) 
		/* appel a SD_BAS_TraceReel1D */
		cr_op = SD_BAS_TraceReel1D( fonction,
					    &ctx->ctxtrt,
					    nomvar,
					    nb_col,
					    stride,
					    tab,
					    ctx);
	      /* sinon */
	      else 
		/* appel a SD_BAS_TraceReel2D */
		cr_op = SD_BAS_TraceReel2D(fonction,		       
					   &ctx->ctxtrt,
					   nomvar,
					   nb_lig,
					   nb_col,
					   tab,
					   ctx);

	  
	    }
      
	  /* sinon on cree un fichier *.LUM */
	  else
	    /* appel a SD_BAS_TraceFichierReel */
	    cr_op = SD_BAS_TraceFichierReel(fonction,		       
					    &ctx->ctxtrt,
					    nomvar,
					    nb_lig,
					    nb_col,
					    tab,
					    ctx);
	  break;
	  
	case TRACE_STRING:
	  cr_op = SD_BAS_TraceChar(fonction,
				   &ctx->ctxtrt,
				   nomvar,
				   nb_col,
				   tab,
				   ctx);
			       



	  break;
	  
	default:
	  num_lig = __LINE__;
	  msg_err = SD_BAS_MSG_CHOIX;
	  cr_op = SD_BAS_ERR_CHOIX;
	  goto FIN;
	  
	}
   
      if(cr_op != CMN__STATUT_OK)
	goto FIN;

    }

 FIN:
 
  if (cr_op != CMN__STATUT_OK)
     
  {
    /* mise a jour de la variable ctx->cr.errinterne */
    SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
			SD_BAS_ERR_SET_traces,
			SD_BAS_ERR_NUM_JDB,
			tcProcName_F,
			num_lig,
			msg_err);

    /* on imprime le message d'erreur sur la sortie standard */

  }


    

// sortie de la fonction 
return(cr_op); 

}



//======================================================================
//					FONCTIONS INTERNES  
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_IsTrace
//@
//@ Description : determine si la fonction appealnte doit etre tracee
//@
//@
//@ Parameters : 
//@			(ENTREE)  fonction : nom de la fonction sous la forme
//@				d'un trigramme (XXX)
//@   (ENTREE)  ctx : structure de contexte pour remonter les messages d'erreur
//@                     
//@
//@ Return Status : 
//@			t_CMN__Statut:
//@			CMN__STATUT_OK si la fonction doit etre tracee
//@			CMN__STATUT_NOK sinon
//@      
//@ ------------------------------------------------------------------*/

static t_CMN__Statut SD_BAS_IsTrace(
		char * fonction, 
  t_SD_BAS_DataThread *ctx)

{
  t_CMN__Statut cr_op=CMN__STATUT_OK;

  int i; //indice de boucle
  int ret; //retour de fonction

  // Entree dans la fonction 
  //@ on parcourt les traces requises que l'on compare avec le nom de 
  //@ la fonction si = alors trace=1 ( fonction interne a creer)

  if( s_All == 1)
  {
    cr_op = CMN__STATUT_OK;
  }
  else
  {
    ret = 1;
    i = 0;
    while (ret != 0 && i< s_nbheadings )
    {
    	 ret = strncmp(fonction,s_headings[i],6);
    	 i++;
    }

    if ( ret == 0 )
      cr_op = CMN__STATUT_OK;
    else
      cr_op = CMN__STATUT_NOK;
  }

  /* determination si la ligne doit etre trace */
  if (cr_op == CMN__STATUT_OK)
  {
    if( s_NbLigne == 100)
    {
      cr_op = CMN__STATUT_OK;
    }
    else
    {
      ret = 1;
      i = 0;
      while(  i< SD_BAS_TAILLE_MAX_LNSN   )
      {
        if (s_Listelignes[i] == ctx->ctxtrt.LN || ctx->ctxtrt.LN== 0	) 
          ret = 0  ;
	       i++;
      } 

      if( ret == 0 )
        cr_op = CMN__STATUT_OK;
      else
        cr_op = CMN__STATUT_NOK;
    }
  }
    
  /* determination si le SN doit etre trace */
  if (cr_op == CMN__STATUT_OK)
  {
    if( s_NbSn == 100)
    {
      cr_op =CMN__STATUT_OK;
    }
    else
    {
      ret = 1;
      i = 0;
      while ( i< SD_BAS_TAILLE_MAX_LNSN )
      {
        if (s_ListeSN[i] == ctx->ctxtrt.SN || ctx->ctxtrt.SN== 0) 
          ret = 0;
       	i++;
      } 

      if( ret == 0 )
        cr_op=CMN__STATUT_OK;
      else
        cr_op=CMN__STATUT_NOK;
    }
  }

FIN:
// sortie de la fonction 
  return(cr_op); 
} /* fin de SD_BAS_IsTrace */

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_FormaterNomFonction
//@
//@ Description : Met la foction sous la NN_XXX si c'est possible
//@               sinon le nom de la fonction vaut ""
//@
//@
//@ Parameters : 
//@			(ENTREE/SORTIE)  fonction : nom de la fonction 
//@			 sous la forme	d'un trigramme (XXX)
//@                     
//@
//@ Return Status : 
//@			 
//@			CMN__STATUT_OK si la fonction doit etre tracee
//@			CMN__STATUT_NOK sinon
//@      
//@ ------------------------------------------------------------------*/ 
void SD_BAS_FormaterNomFonction(char *fonction, char *trigramme)
{
 char *pointer;
 char * tok1;
 char * tok2;
 char  buff[SD_BAS_TAILLE_MAX_CLE];


// si le nom de la fonction est trop long 
 if(strlen(fonction) > SD_BAS_TRIGRAMME_SIZE)
   {

     strcpy(buff, fonction);

     /* on recupere le trigramme */
     /* s'il exixte on extrait ALG_xxxxx_xx_xxx */
     if( strtok_r(buff,"_", &pointer)==NULL)
       /* on initialise le nom de la fonction a "" */
       strcpy(trigramme, "");

       /* s'il exixte on extrait xxx_nom_xx_xxx */
     if( strtok_r(NULL,"_", &pointer)==NULL)
       /* on initialise le nom de la fonction a "" */
       strcpy(trigramme, "");
     
       /* s'il exixte on extrait et on stocke le numero xxx_xxxx_NN_xxx */
     if( (tok1=strtok_r(NULL,"_", &pointer))!=NULL  )
       {
	 /* s'il exixte on extrait et on stocke le trigramme xxx_xxxx_xx_NNN */
	 if( (tok2=strtok_r(NULL,"_", &pointer))!=NULL)


	   /* on reconstruit le nom de la fonction */
	   sprintf(trigramme, "%s_%s",tok1,tok2);
	   
	 
	 else
	   /* si la fonction en entree n'est pas au bon format lle nom de la fonction vaut "" */
	   strcpy(trigramme, "");
       }
     else
       {
	 /* si la fonction en entree n'est pas au bon format lle nom de la fonction vaut "" */
	 strcpy(trigramme,"");
	 
       }
   }


}
