//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/BAS/src/SD_BAS_Traces.h,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_BAS
//@ MODULE : SD_BAS_Traces
//@
//@ Comment :
//@    Include file for the  SD_BAS_Traces class which encapsulates the
//@    debug generation facility for IASI OPS  :
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_BAS_Traces_h
#define SD_BAS_Traces_h


/* librairies C de bases utilisees par les modules du projet */
#include <stdio.h>       /* entree/sortie */
#include <stdlib.h>      /* alloc memoire, ...*/
#include <string.h>      /* chaines de carateres */
#include <unistd.h>      /* gestion pipes, ... */
#include <time.h>        /* gestion de la date */
#include <sys/param.h>   /* constantes systeme */
#include <fcntl.h>
#include <sys/timeb.h>
// inserer ici les fichiers include 
#include "SD_BAS_Global.h"
#include "SD_BAS_Contexte.h"
#include "SD_BAS_TraceOutils.h"

/*====================================================================*/
/*   External define  Declarations                                    */
/*====================================================================*/
/* Macro indiquant si des messages doivent etre traces ou non */
/* exemple d'utilisation */
/* if (!SD_BAS_TRACEREEL1D(validity_date, numorbite,numgranule,Ctx)) print(CPF_ERREUR); */
/* attention car la variable CPF_ERREUR n'est pas thread safe */


#define SD_BAS_INITTRACES(validity_date,numgranule,Ctx,err) \
  if(SD_BAS_InitTraces(validity_date,numgranule,Ctx,err) != CMN__STATUT_OK) printf("%s",s_SD_BAS_ERREUR);

#define SD_BAS_FINTRACES() SD_BAS_FinTraces()

#define SD_BAS_TRACEREEL1D(fonction,num,nomvar,dimension,stride,tab,err) \
  if(SD_BAS_TraceReel1D(fonction,num,nomvar,dimension,stride,tab,err) != CMN__STATUT_OK) printf("%s",s_SD_BAS_ERREUR);
   
#define SD_BAS_TRACEREEL2D(fonction,num,nomvar,nbligne,nbcolonne,tab,err) \
  if(SD_BAS_TraceReel2D(fonction,num,nomvar,nbligne,nbcolonne,tab,err) != CMN__STATUT_OK) printf("%s",s_SD_BAS_ERREUR);

#define SD_BAS_TRACEENTIER1D(fonction,num,nomvar,dimension,stride,tab,err) \
  if(SD_BAS_TraceEntier1D(fonction,num,nomvar,dimension,stride,tab,err) != CMN__STATUT_OK) printf("%s",s_SD_BAS_ERREUR);
   
#define SD_BAS_TRACEENTIER2D(fonction,num,nomvar,nbligne,nbcolonne,tab,err) \
  if(SD_BAS_TraceEntier2D(fonction,num,nomvar,nbligne,nbcolonne,tab,err) != CMN__STATUT_OK) printf("%s",s_SD_BAS_ERREUR);

#define SD_BAS_TRACECHAR(fonction,num,nomvar,dimension,chaine,err) \
  if(SD_BAS_TraceChar(fonction,num,nomvar,dimension,chaine,err) != CMN__STATUT_OK) printf("%s",s_SD_BAS_ERREUR);

#define SD_BAS_TRACEFICHIERENTIER(fonction,num,nomvar,nbligne,nbcolonne,tab,err) \
  if(SD_BAS_TraceFichierEntier(fonction,num,nomvar,nbligne,nbcolonne,tab,err) != CMN__STATUT_OK) printf("%s",s_SD_BAS_ERREUR);

#define SD_BAS_TRACEFICHIERREEL(fonction,num,nomvar,nbligne,nbcolonne,tab,err) \
  if(SD_BAS_TraceFichierReel(fonction,num,nomvar,nbligne,nbcolonne,tab,err) != CMN__STATUT_OK) printf("%s",s_SD_BAS_ERREUR);

#define SD_BAS_TRACE(fonction,nomvar,nb_lig,nb_col,stride,type,format,tab,ctx)\
if(SD_BAS_Trace(fonction,nomvar,nb_lig,nb_col,stride,type,format,tab,ctx)!= CMN__STATUT_OK) printf("%s",s_SD_BAS_ERREUR);

#define ERR_BASE_TRACES		 	SD_BAS_ERR_BASE_Traces

#define SD_BAS_ERR_OUVERTURE               (ERR_BASE_TRACES+0)
#define SD_BAS_ERR_CHOIX                   (ERR_BASE_TRACES+1)
#define SD_BAS_ERR_ENVIRONNEMENT           (ERR_BASE_TRACES+2)
#define SD_BAS_ERR_REPERTOIRE              (ERR_BASE_TRACES+3)
#define SD_BAS_ERR_FICHIER_OUVERT          (ERR_BASE_TRACES+4)
#define SD_BAS_ERR_AJOUTE                  (ERR_BASE_TRACES+5)
#define SD_BAS_ERR_TRIGRAMME               (ERR_BASE_TRACES+6)
#define SD_BAS_ERR_FERMETURE               (ERR_BASE_TRACES+7)
#define SD_BAS_ERR_FICHIER_FERME           (ERR_BASE_TRACES+8)
#define SD_BAS_ERR_TAILLE_CLE              (ERR_BASE_TRACES+9)

#define SD_BAS_ERR_SET_traces 1


//====================================================================*/
// Exported types Declaration  
//====================================================================*/
typedef enum
  {
    TRACE_INT4,
    TRACE_REAL8,
    TRACE_UINT4,
    TRACE_STRING
  }e_SD_BAS_TypTrace;

typedef enum
  {
    TRACE_ASC,
    TRACE_BIN
  }e_SD_BAS_FormatTrace;


//====================================================================*/
// External variables Declaration 
//====================================================================*/

#ifdef __cplusplus
extern "C" {
#endif

//====================================================================*/
//					FONCTIONS EXTERNES
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_InitTraces
//@
//@ Description : ouverture et initialisation du fichier de traces 
//@
//@
//@ Parameters :
//@			(ENTREE)  validity_date : date de fin du granule 
//@    			(ENTREE)  numgranule : numero de granule 
//@			(ENTREE/SORTIE)  Ctx : contexte lu
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_InitTraces(
		t_SD_BAS_SCDSTime validity_date, // date de fin du granule
       		int numgranule, // issu du contexte ( 0 en debut de dump)
		t_SD_BAS_Contexte * Ctx,
		t_SD_BAS_DataThread *err);

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_FinTraces
//@
//@ Description : fermeture  du fichier de traces 
//@
//@
//@ Parameters : aucun
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_FinTraces(	t_SD_BAS_DataThread *err
			);

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_TraceReel1D
//@
//@ Description : ecriture  du tableau 1D de reels
//@
//@
//@ Parameters : 
//@			(ENTREE)  fonction : nom de la fonction sous la forme
//@				d'un trigramme (XXX)
//@			(ENTREE)  num : numero de ligne et SN traitee
//@				  ( les champ sont ajoutees au nom de la variable 
//@				  ( si differents de 0)
//@			(ENTREE)  nomvar : nom de la variable a ecrire
//@			(ENTREE)  dimension : dimension du tableau de 
//@					   reels 
//@			(ENTREE)  stride : offset entre chaque element 
//@					   du tableau 
//@			
//@			(ENTREE)  tab : tableau 1D de reels 
//@
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_TraceReel1D(
		char * fonction,
		t_SD_BAS_ORIGINE *num,
		char * nomvar,
		int dimension,
		int stride,
		double *tab,
		t_SD_BAS_DataThread *err
		);


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_TraceReel2D
//@
//@ Description : ecriture  du tableau 2D de reels
//@
//@
//@ Parameters : 
//@			(ENTREE)  fonction : nom de la fonction sous la forme
//@				d'un trigramme (XXX)
//@			(ENTREE)  num : numero de ligne et SN traitee
//@				  ( les champ sont ajoutees au nom de la variable 
//@				  ( si differents de 0)
//@			(ENTREE)  nomvar : nom de la variable a ecrire
//@			(ENTREE)  nbligne : nombre de ligne du tableau
//@			(ENTREE)  nbcolonne : nombre de colonne du tableau
//@			
//@			(ENTREE)  tab : tableau 2D de reels (supposes contigus) 
//@
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_TraceReel2D(
		char * fonction,
		t_SD_BAS_ORIGINE *num,
		char * nomvar,
		int nbligne,
		int nbcolonne,
		double *tab,
		t_SD_BAS_DataThread *err
		);


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_TraceEntier1D
//@
//@ Description : ecriture  du tableau 1D d'entier
//@
//@
//@ Parameters : 
//@			(ENTREE)  fonction : nom de la fonction sous la forme
//@				d'un trigramme (XXX)
//@			(ENTREE)  num : numero de ligne et SN traitee
//@				  ( les champ sont ajoutees au nom de la variable 
//@				  ( si differents de 0)
//@			(ENTREE)  nomvar : nom de la variable a ecrire
//@			(ENTREE)  dimension : dimension du tableau d'entiers 
//@				   
//@			(ENTREE)  stride : offset entre chaque element 
//@					   du tableau 
//@			
//@			(ENTREE)  tab : tableau 1D d'entier 
//@
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_TraceEntier1D(
		char * fonction,
		t_SD_BAS_ORIGINE *num,
		char * nomvar,
		int dimension,
		int stride,
		int *tab,
		t_SD_BAS_DataThread *err
		);

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_TraceEntier2D
//@
//@ Description : ecriture  du tableau 2D d'entiers
//@
//@
//@ Parameters : 
//@			(ENTREE)  fonction : nom de la fonction sous la forme
//@				d'un trigramme (XXX)
//@			(ENTREE)  num : numero de ligne et SN traitee
//@				  ( les champ sont ajoutees au nom de la variable 
//@				  ( si differents de 0)
//@			(ENTREE)  nomvar : nom de la variable a ecrire
//@			(ENTREE)  nbligne : nombre de ligne du tableau
//@			(ENTREE)  nbcolonne : nombr de colonne du tableau
//@				  
//@			
//@			(ENTREE)  tab : tableau 2D d'entiers (supposes contigus)
//@
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_TraceEntier2D(
		char * fonction,
		t_SD_BAS_ORIGINE *num,
		char * nomvar,
		int nbligne,
		int nbcolonne,
		int *tab,
		t_SD_BAS_DataThread *err);

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_TraceChar
//@
//@ Description : ecriture  d'une chaine de caractere
//@
//@
//@ Parameters : 
//@			(ENTREE)  fonction : nom de la fonction sous la forme
//@				d'un trigramme (XXX)
//@			(ENTREE)  num : numero de ligne et SN traitee
//@				  ( les champ sont ajoutees au nom de la variable
//@				  ( si differents de 0)
//@			(ENTREE)  nomvar : nom de la variable a ecrire
//@			(ENTREE)  dimension : longueur de la chaine de caractere
//@			
//@			(ENTREE)  chaine :  chaine de caractere
//@
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_TraceChar(
		char * fonction,
		t_SD_BAS_ORIGINE *num,
		char *nomvar,
		int dimension,
		char * chaine,
		t_SD_BAS_DataThread *err
		);


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_TraceFichierEntier
//@
//@ Description : ecriture  d'un fichier binaire d'entier
//@
//@
//@ Parameters : 
//@			(ENTREE)  fonction : nom de la fonction sous la forme
//@				d'un trigramme (XXX)
//@			(ENTREE)  num : numero de ligne et SN traitee
//@				  ( les champ sont ajoutees au nom de la variable 
//@				  ( si differents de 0)
//@			(ENTREE)  nomvar : nom de la variable a ecrire
//@			(ENTREE)  nbligne : nombre de ligne du tableau
//@			(ENTREE)  nbcolonne : nombre de colonne du tableau
//@			(ENTREE)  tab :  tableau d'entiers
//@                     
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/

t_CMN__Statut SD_BAS_TraceFichierEntier(
		char * fonction,
		t_SD_BAS_ORIGINE *num,
		char *nomvar,
		int nbligne,
		int nbcolonne,
		int *tab,
		t_SD_BAS_DataThread *err
		);


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_TraceFichierReel
//@
//@ Description : ecriture  d'un fichier binaire de reels
//@
//@
//@ Parameters : 
//@			(ENTREE)  fonction : nom de la fonction sous la forme
//@				d'un trigramme (XXX)
//@			(ENTREE)  num : numero de ligne et SN traitee
//@				  ( les champ sont ajoutees au nom de la variable 
//@				  ( si differents de 0)
//@			(ENTREE)  nomvar : nom de la variable a ecrire
//@			(ENTREE)  nbligne : nombre de ligne du tableau
//@			(ENTREE)  nbcolonne : nombre de colonne du tableau			
//@			(ENTREE)  tab :  tableau de reels
//@                     
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/

t_CMN__Statut SD_BAS_TraceFichierReel(
		char * fonction,
		t_SD_BAS_ORIGINE *num,
		char *nomvar,
		int nbligne,
		int colonne,
		double *tab,
		t_SD_BAS_DataThread *err
		);


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_Trace
//@
//@ Description : ecriture  du tableau 1D ou 2D de reels ou entier
//@
//@
//@ Parameters : 
//@			(ENTREE)  fonction : nom de la fonction sous la forme
//@				d'un trigramme (XXX)
//@			(ENTREE)  num : numero de ligne et SN traitee
//@				  ( les champ sont ajoutees au nom de la variable 
//@				  ( si differents de 0)
//@			(ENTREE)  nomvar : nom de la variable a ecrire
//@			(ENTREE)  nb_lig : nombre de ligne du tableau
//@			(ENTREE)  nb_col : nombre de colonne du tableau
//@			(ENTREE)  stride : offset entre chaque element 
//@					   du tableau 
//@			
//@			(ENTREE)  tab : pointeur sur un tableau
//@
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@		SD_BAS_ERR_AJOUTE  : ajout de donnees dans le fichier de traces
//@                                 impossible
//@             SD_BAS_ERR_OUVERT : fichier de traces pas  ouvert
//@             SD_BAS_ERR_BAS_NIVEAU : allocation impossible7
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_Trace(
		char * fonction,
		char * nomvar,
		int nb_lig,
		int nb_col,
		int stride,
		e_SD_BAS_TypTrace type,
		e_SD_BAS_FormatTrace format,
		void *tab,
		t_SD_BAS_DataThread *ctx
		);

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_TracePrecision
//@
//@ Description : ecriture  de la precision attendue
//@
//@
//@ Parameters : 
//@			(ENTREE) precision
//@
//@
//@ Return Status : 
//@			t_CMN__Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_TracePrecision(double precision,
				    t_SD_BAS_DataThread *err);


//======================================================================

#ifdef __cplusplus
}
#endif


#endif
