static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_BanqueSpectrale.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_BanqueSpectrale
//@ 
//@ Comment :
//@    this module implement the SD_DON_BanqueSpectrale class
//@    used to reference the Data Base COnfiguration FIle
//@           
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 30/11/2002 : P.GASSIE
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <iostream>

#include "SD_DON_BanqueSpectrale.h"
#include "CMN__OPSFile.h"
#include "JDB__Client.h"

//
// Definition des constantes
//
const int SD_DON_BanqueSpectrale::POS_VAL_DATE = 5;
const int SD_DON_BanqueSpectrale::POS_IDENT    = 10;
const int SD_DON_BanqueSpectrale::NBRE_MIN     = 10;
const t_Chaine SD_DON_BanqueSpectrale::JDB_PARAM_1 = "$PRM1";
const t_Chaine SD_DON_BanqueSpectrale::DATA_AUX_BS_NOT_FOUND = "Spectral database file not found";
const t_Chaine SD_DON_BanqueSpectrale::DATA_AUX_BS_READ_ERROR = "Spectral database file read error";


//======================================================================
//           METHODES PUBLIQUES 
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Methode : SD_DON_BanqueSpectrale
//@
//@ Description : Constructeur
//@
//@ Parameters :
//@     Aucun
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_BanqueSpectrale::SD_DON_BanqueSpectrale()
{
  //initialisation des attributs
  m_DataBS = NULL;
  m_ident = -1;
  m_Charge = false;
}
//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_DON_BanqueSpectrale
//@
//@ Description : Destructeur
//@
//@
//@ Parameters :
//@     Aucun
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_BanqueSpectrale::~SD_DON_BanqueSpectrale()
{
  //@ SI il y a une valeur dans le champ m_DataBS
  if (m_DataBS)
  {
    //@ On la libere
    delete m_DataBS;  
  }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : Lire
//@
//@ Description : Cette methode permet de lire les informations
//@ stockees dans le fichier banque spectrale
//@  de donnees auxiliaires
//@
//@
//@ Parameters :
//@     (ENTREE)  Ident : Identificateur de la BS a lire
//@     (ENTREE)  Repertoire : Repertoire contenant le fichier
//@
//@ Return Status : 
//@
//@ ------------------------------------------------------------------*/
void SD_DON_BanqueSpectrale::Lire(
    const int& Ident,
    const t_Chaine & repertoire,
    t_Chaine& nomfichier
    ) throw(ERR__Exception)

{
  static const char tcProcName_F[] = "SD_DON_BanqueSpectrale::Lire";
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  t_SD_BAS_DataThread* ctx;
  t_SD_BAS_NomFichier nom_fichier;
  CMN__ListeInfoFichier ListeInfoFichier;
  CMN__ListeInfoFichier::iterator It;
  t_Chaine pattern("ODB");
  t_Chaine fichierTmp("");
  int IdentTmp = -1;
  
  // Entree dans la fonction 

  //@ SI la BS n'a pas deja ete chargee
  if(m_Charge == false)
  {
    //@ on recupere dans une liste l'ensemble des fichier de type BS presents
    cr_op = CMN__TrouverFichiersEx(repertoire, pattern, ListeInfoFichier, NULL, NULL, NULL, 1, 0);
    //@ SI erreur
    if(cr_op != CMN__STATUT_OK)
    {
      //@ message JDB
      JDB__Client MessageJDB;
      MessageJDB.SetParametre(JDB_PARAM_1, pattern);
      MessageJDB.EcrireMessage(SD_CONF_SET, SD__Message::MSG_CONF_NO_FILE);
      MessageJDB.Liberer();
      //@ exception
      ERR__Exception exception;
    
      //@ Emission exception SD_DON_TMT_DATA_AUX
      exception.SetParametre(DATA_AUX_BS_NOT_FOUND);
      exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_AUX);
      THROW(exception);
    }
    //@ FIN SI
    //@ POUR chaque fichier
    for (It = ListeInfoFichier.begin();(It != ListeInfoFichier.end()) && (m_ident == -1);It ++)
    {
      //@  on cree un objet t_OPSFile
      t_OPSFile fichierTmp(It->Name());
      //@ enregistrement de l'identificateur 
      IdentTmp = fichierTmp.GetIdent();
      //@ SI le IdentTmp est = Ident
      if (Ident == IdentTmp)
      {
        //@ on reference le nouveau fichier
        strcpy(nom_fichier, (repertoire + "/" + fichierTmp.GetNom()).c_str());
        m_ident = IdentTmp;
        nomfichier = repertoire + "/" + It->Name();
        //@ on enregistre la date de validite
        t_Date dateValiditeTmp(fichierTmp.GetValidityStartDate());
        m_DateValid = dateValiditeTmp;
      }
      //@ FIN SI
    }    
    //@ FIN POUR
    //@ SI aucun fichier correct n'a ete trouve
    if(m_ident == -1)
    {
      //@ message JDB
      JDB__Client MessageJDB;
      MessageJDB.SetParametre(JDB_PARAM_1, pattern);
      MessageJDB.EcrireMessage(SD_CONF_SET, SD__Message::MSG_CONF_NO_FILE);
      MessageJDB.Liberer();
      //@ exception
      ERR__Exception exception;
    
      //@ Emission exception SD_DON_TMT_DATA_AUX
      exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_AUX);
      exception.SetParametre(DATA_AUX_BS_NOT_FOUND);
      THROW(exception);
    }
    //@ FIN SI
    //@ on alloue la donnee m_DataBS
    m_DataBS = new t_SD_BAS_BanqueSpectrale;
    //@ on lit le fichier via la fonction de SD_BAS_BanqueSpectrale
    ctx = new t_SD_BAS_DataThread;
    InitialiserCompteRendu (&(ctx->cr));

    cr_op = SD_BAS_ChargerBS(nom_fichier,
                m_DataBS,
                ctx);
    //@ SI erreur
    if(cr_op != CMN__STATUT_OK)
    {
      //@ emission des messages du ctx
			EnvoyerMessagesStockes(&(ctx->cr.msgJDB));
      //@ emission d'un message vers le JdBS
      JDB__Client MessageJDB;
      MessageJDB.SetParametre(JDB_PARAM_1, nom_fichier);
      MessageJDB.EcrireMessage(SD_CONF_SET, SD__Message::MSG_CONF_LOAD_ERR);
      MessageJDB.Liberer();
      LibererCompteRendu(&(ctx->cr.msgJDB));
      delete ctx;
      //@ exception
      ERR__Exception exception;
    
      //@ Emission exception SD_DON_TMT_DATA_AUX
      exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_AUX);
      exception.SetParametre(DATA_AUX_BS_READ_ERROR);
      THROW(exception);
    }
    //@ SINON
    {
      //@ l'indicateur de chargement est initialise
      JDB__Client MessageJDB;
		  MessageJDB.SetParametre(JDB_PARAM_1, nom_fichier);
			MessageJDB.EcrireMessage(SD_CONF_SET, SD__Message::MSG_CONF_LOAD_OK);
			MessageJDB.Liberer();
      LibererCompteRendu(&(ctx->cr.msgJDB));
      delete ctx;
      m_Charge = TRUE;
    }
    //@ FIN SI
  }
  //@ FIN SI BS pas deja chargee
// on trace ici si besoin toutes les variables en sortie
#ifdef OPS_DEBUG
cout << endl << "SD_DON_BanqueSpectrale::Lire" << endl;
cout << "Nom du fichier a lire : " << nom_fichier << endl;
cout << "IdBS : " << m_ident << endl;
cout << "Date Validite : " << m_DateValid.Chaine() << endl;
#endif
// sortie de la fonction 


}

//@ ------------------------------------------------------------------*/
//@ Methode : Verifie
//@
//@ Description : Cette methode permet de verifier la coherence du fichier
//@ en debut de chaque granule par rapport a l'identificateur issu du fichier
//@ de conf autre courant
//@
//@
//@ Parameters :
//@     (ENTREE)  Ident : Identificateur de la banque spectrale 
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_DON_BanqueSpectrale::Verifie(
    const int& Ident
    )

{
static const char tcProcName_F[] = "SD_DON_BanqueSpectrale::Verifie";
t_CMN__Statut cr_op=CMN__STATUT_OK;
// Entree dans la fonction 

  //@ SI la BS a pas deja ete chargee
  if(m_Charge)
  {
    //@ SI Ident == m_ident
    if(Ident == m_ident)
    {
      //@ retour CMN__STATUT_OK
      cr_op = CMN__STATUT_OK;
    }
    //@ SINON
    else
    {
      //@ retour CMN__STATUT_NOK
      cr_op = CMN__STATUT_NOK;
    }
    //@ FIN SI
  }
  //@ FIN SI

// on trace ici si besoin toutes les variables en sortie

fin:
// sortie de la fonction 
return(cr_op); 

}


//@ ------------------------------------------------------------------*/
//@ Methode : GetData
//@
//@ Description : Cette methode permet de recuperer l'adresse de la zone
//@    memoire ou est stockee le fichier banque spectrale 
//@
//@
//@ Parameters :   aucun
//@     
//@
//@ Return Status :   t_SD_BAS_BanqueSpectrale pointeur sur la donnee
//@
//@ ------------------------------------------------------------------*/
t_SD_BAS_BanqueSpectrale* SD_DON_BanqueSpectrale::GetData(
    )

{
// Entree dans la fonction 

  //@ on verifie que la BS a  deja ete chargee
  //@  return m_DataBS
  return(m_DataBS);
// sortie de la fonction 

}

//======================================================================
//          METHODES PRIVEES 
//======================================================================
