static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_CasDegLigne.cc,v 1.4 2011/12/20 15:42:01 lcaspar Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_CasDegLigne
//@ 
//@ Comment :
//@    this module implement the SD_DON_CasDegLigne class
//@    used to administrate the degraded cases in a line of OPS product
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 20/12/2002 : P.GASSIE
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-067-CN : 29/10/2003 : JL.PASCAL
//@ lors de lignes incompletes le cas degrade CG13 n'est pas leve 
//@ poue les SN 9 et 10 (PX4 absent)
//@ le test portait sur les pixel de 1 a 3 !
//@ modification dans le fonction valide
//@ fiche de correction correspondante : IA-2100-FC-067-THA
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-070-CN : 10/11/2003 : JL.PASCAL
//@ dans le cas de cas degrade permanent l'OPS ne s'arrete pas
//@ car une exception n'est pas levee
//@ (cf IA-2100-FC-070-THA)
//@ --------------------------------------------------------------------
//@ DM-ID : PM4 : 31/03/2004 : B. MASSART
//@ SD_DON_CasDegLigne::Valide:
//@ -ajout SD_BAS_SN_IP_SUP_x
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-139-CN : 21/09/2004 : B.MASSART
//@ Probleme d'initialisation de GDatFlagIsrfemOff quand on entre en mode
//@ calibration externe ne visant pas la Terre. On introduit le mode
//@ de prise de vue dans la gestion des cas degrades. Voir IA-2100-FC-139-THA.
//@ --------------------------------------------------------------------
//@ DM-ID : IA-2100-CR-288-CN : 20/03/2006 : B. GUENDE
//@ Function SyntheseCasDegrades added to print a summary of all degraded cases encountered.
//@ Counter added instead of print command. See IA-2100-CR-288-THA.
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-177-CN : 21/07/2006 : B. MASSART
//@ correction erreur allocation memoire en mode calibration
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-193-CN : 9/11/2006 : R.MORENO
//@ Levee de CG pour les images associees aux visees da calibration CS
//@ et BB
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2919(FA-2100-OPS) : 14/12/2011 : L. CASPAR
//@ GOPSFlagPacketIPMiss non leve sur SN32 et SN35
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <iostream>

#include "SD_DON_CasDegLigne.h"

//
// Definition des constantes
//
const t_Chaine SD_DON_CasDegLigne::JDB_PARAM_1 = "$PRM1";

//======================================================================
//           METHODES PUBLIQUES  
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Methode : SD_DON_CasDegLigne
//@
//@ Description : Constructeur
//@
//@ Parameters :
//@     aucun
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_CasDegLigne::SD_DON_CasDegLigne(
  int NumLigne, 
  e_BAS_ModePdv Mode) /* FA139 */
{
  //@ on initialise ici les attributs en utilisant la definition des
  //@ 15 cas degrades identifies dans la spec
  //@ on initialise m_NumLigne
  m_NumLigne = NumLigne;
  m_Mode = Mode; /* FA139 */
  
  t_Date dateZero;
  dateZero.DateInit();
  
  //@ on initialise m_presIP
  for(int i = 0; i < SD_BAS_SN ; i++)
    m_presIP[i] = false;
  //@ on initialise m_presPX
  for(int i = 0; i < SD_BAS_SNOT; i++)
  {
    for(int j = 0;j < SD_BAS_PN; j++)
    {
        m_presPX[i][j] = false;
    }
  }
  //@ on initialise m_presAP
  m_presAP = false;
  //@ on initialise m_presVP
  for(int i = 0; i < 5 ; i++)
    m_presVP[i] = false;
  //@ on initialise m_Flt
  InitmFlt();
  //@ POUR tous les CGxx
  for(int cgxx = 0; cgxx < SD_BAS_NB_CGXX; cgxx++)
  {
/* CR288 */
    //@ on initialise le tableau compteur des cas degrades
    m_nbCasDeg[cgxx]=0;
/* Fin CR288 */
    
    m_GOPSDATFirst[cgxx] = dateZero;
    m_GOPSDATLast[cgxx] = dateZero;
    for (int ccd = 0 ; ccd < SD_BAS_CCD ; ccd++)
    {
      m_GOPSDATFirstCG3[ccd] = dateZero;
      m_GOPSDATLastCG3[ccd] = dateZero;
    }
    for (int sb = 0 ; sb < SD_BAS_SB ; sb++)
    {
      m_GOPSDATFirstCG4[sb] = dateZero;
      m_GOPSDATLastCG4[sb] = dateZero;
    }
    
    //@ POUR tous les SN
    for(int sn = 0; sn < SD_BAS_SN; sn++)
    {
      //@ POUR tous les PN
      for(int pn = 0; pn < SD_BAS_PN; pn++)
      {
        //@ POUR tous les SB
        for(int sb = 0; sb < SD_BAS_SB; sb++)
        {
          //@ initialisation des flags
          m_Flt[cgxx].flag[sn][pn][sb] = false;
        }
        //@ FIN POUR
      }
      //@ FIN POUR
    }
    //@ FIN POUR
  }
  //@ FIN POUR
}

//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_DON_CasDegLigne
//@
//@ Description : Destructeur
//@
//@
//@ Parameters :
//@     Aucun
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_CasDegLigne::~SD_DON_CasDegLigne()
{
}

//@ ------------------------------------------------------------------*/
//@ Methode : Init
//@
//@ Description : Cette methode permet d'initialiser le 
//@  cas degrade  de la ligne avec le contexte prealablement charge
//@ et avec les information sur les durees issue du fichier de conf
//@
//@
//@ Parameters :
//@     (ENTREE)  Contexte : contexte pour initialisation
//@     (ENTREE)  durees : durees issues du fichier de conf
//@
//@ Return Status : 
//@     Aucune
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
void SD_DON_CasDegLigne::Init(
           SD_DON_Contexte  & Contexte,
           const t_SD_DON_IdefDur & durees
)
{
  static const char tcProcName_F[] = "SD_DON_CasDegLigne::Init";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  t_Date dateZero;
  dateZero.DateInit();
  
  // Entree dans la fonction 

  //@ on initialise l'attribut m_DureeCasDeg a partir des fichier de conf
  for(int i = 0; i < SD_BAS_NB_CGXX; i++)
  {
    t_Date dureeTmp(durees[i]);
    m_DureeCasDeg[i] = dureeTmp;
  }
  //@ initialisation cas deg permanent
  m_casDegPermanent = 0;
  m_casDegPermanentPrec = 0;
  //@ on initialise les dates de declenchement des derniers CGxx m_GOPSDAT
  //@ a partir du contexte ( les dates sont init si pas de duree continue)
  m_GOPSDATFirst[2].SCDSTime((Contexte.GetData())->GOPSDatIsrfemOff[0]);
  m_GOPSDATFirst[3].SCDSTime((Contexte.GetData())->GOPSDatBandMiss[0]);
  m_GOPSDATFirst[4].SCDSTime((Contexte.GetData())->GOPSDatBBTMiss[0]);
  m_GOPSDATFirst[5].SCDSTime((Contexte.GetData())->GOPSDatImgEWMiss[0]);
  m_GOPSDATFirst[6].SCDSTime((Contexte.GetData())->GOPSDatImgBBMiss[0]);
  m_GOPSDATFirst[7].SCDSTime((Contexte.GetData())->GOPSDatImgCSMiss[0]);
  m_GOPSDATFirst[8].SCDSTime((Contexte.GetData())->GOPSDatRadAvhrrMiss[0]);
  
  m_GOPSDATLast[2] = dateZero;
  m_GOPSDATLast[3] = dateZero;
  m_GOPSDATLast[4] = dateZero;
  m_GOPSDATLast[5] = dateZero;
  m_GOPSDATLast[6] = dateZero;
  m_GOPSDATLast[7] = dateZero;
  m_GOPSDATLast[8] = dateZero;
  
  //@ initialisation de m_GOPSDATFirstCG3 
  for(int i = 0; i < SD_BAS_CCD; i++)
  {
    m_GOPSDATFirstCG3[i].SCDSTime((Contexte.GetData())->GOPSDatIsrfemOff[0]);
    m_GOPSDATFirstCG3[i] = dateZero;
  }
  //@ initialisation de m_GOPSDATFirstCG4 
  for(int i = 0; i < SD_BAS_SB; i++)
  {
    m_GOPSDATFirstCG4[i].SCDSTime((Contexte.GetData())->GOPSDatBandMiss[0]);
    m_GOPSDATLastCG4[i] = dateZero;
  }
// sortie de la fonction 
}

//@ ------------------------------------------------------------------*/
//@ Methode : InitPrec
//@
//@ Description : Cette methode permet d'initialiser le 
//@  cas degrade  de la ligne avec la ligne precedente 
//@ et avec les information sur les durees issue du fichier de conf
//@
//@
//@ Parameters :
//@     (ENTREE)  CasDegLignePrec : cas degrade de la ligne precedente
//@     (ENTREE)  durees : durees issues du fichier de conf
//@
//@ Return Status : 
//@     Aucune
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
void SD_DON_CasDegLigne::InitPrec(
           SD_DON_CasDegLigne  & CasDegLignePrec,
           const t_SD_DON_IdefDur & durees
)
{
static const char tcProcName_F[] = "SD_DON_CasDegLigne::InitPrec";
t_CMN__Statut cr_op=CMN__STATUT_OK;
// Entree dans la fonction 
  //@ recopie du flag de cas permanents
  m_casDegPermanent = CasDegLignePrec.GetFlagPermanent();
  m_casDegPermanentPrec = m_casDegPermanent;
  //@ on initialise l'attribut m_DureeCasDeg a partir des fichier de conf
  for(int i = 0; i < SD_BAS_NB_CGXX; i++)
  {
    t_Date dateTmp(durees[i]);
    m_DureeCasDeg[i] = dateTmp;
  }
  //@ on initialise l'attribut m_GOPSDATFirst a partir de celui de l'objet precedent
  for(int i = 0; i < SD_BAS_NB_CGXX; i++)
  {
    t_Date dateTmp(CasDegLignePrec.GetDate(i + 1));
    m_GOPSDATFirst[i] = dateTmp;
  }
  //@ initialisation de m_GOPSDATFirstCG3 et m_GOPSDATLastCG3
  for(int i = 0; i < SD_BAS_CCD; i++)
  {
    t_Date dateTmp1(CasDegLignePrec.m_GOPSDATFirstCG3[i]);
    m_GOPSDATFirstCG3[i] = dateTmp1;
    t_Date dateTmp2(CasDegLignePrec.m_GOPSDATLastCG3[i]);
    m_GOPSDATLastCG3[i] = dateTmp2;
  }
  //@ initialisation de m_GOPSDATFirstCG4 et m_GOPSDATLastCG4
  for(int i = 0; i < SD_BAS_SB; i++)
  {
    t_Date dateTmp1(CasDegLignePrec.m_GOPSDATFirstCG4[i]);
    m_GOPSDATFirstCG4[i] = dateTmp1;
    t_Date dateTmp2(CasDegLignePrec.m_GOPSDATLastCG4[i]);
    m_GOPSDATLastCG4[i] = dateTmp2;
  }
// sortie de la fonction 
}
//@ ------------------------------------------------------------------*/
//@ Methode : InitFlt
//@
//@ Description : Cette methode permet d'initialiser le tableau des flags
//@  des cas degrade  de la ligne en utilisant les informations 
//@  issue du produit donnees technologique
//@
//@
//@ Parameters :
//@     (ENTREE)  Flags : flags pour initialisation
//@     (ENTREE)  Dates : dates pour initialisation
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_CasDegLigne::InitFlt(
          const t_SD_DON_Flt & Flags,
          const t_SD_DON_GOPSDAT & Dates
        )
{
static const char tcProcName_F[] = "SD_DON_CasDegLigne::InitFlt";
t_CMN__Statut cr_op=CMN__STATUT_OK;
// Entree dans la fonction 

  //@ initialisation du flag de cas degrades permanents
  m_casDegPermanent = 0;
  m_casDegPermanentPrec = 0;
  //@ on initialise l'attribut m_Flt a partir de Flags
  //@ POUR tous les numeros de cas degrades
  for(int i = 0; i < SD_BAS_NB_CGXX; i++)
  {
    //@ on recopie les elements de Flags dans m_Flt
    for(int j = 0; j < SD_BAS_SN; j++)
    {
      for(int k = 0; k < SD_BAS_PN; k++)
      {
        for(int l = 0; l < SD_BAS_SB; l++)
        {
          m_Flt[i].flag[j][k][l] = Flags[i].flag[j][k][l];
        }
      }
    }
  }
  //@ FIN POUR
  //@ on initialise l'attribut m_GOPSDAT a partir de Dates
  for(int i = 0; i < SD_BAS_NB_CGXX; i++)
  {
    t_Date dateTmp(Dates[i]);
    m_GOPSDATFirst[i] = dateTmp;
  }
  //@ on initialise m_GOPSDATFirstCG4 et m_GOPSDATFirstCG3
  for(int j = 0; j < SD_BAS_SN; j++)
  {
    for(int k = 0; k < SD_BAS_PN; k++)
    {
      for(int l = 0; l < SD_BAS_SB; l++)
      {
        if(m_Flt[3].flag[j][k][l])
        {
          m_GOPSDATFirstCG4[l] = m_GOPSDATFirst[2];
        }
        if(m_Flt[2].flag[j][k][l] && l < SD_BAS_CCD)
        {
          m_GOPSDATFirstCG3[l] = m_GOPSDATFirst[2];
        }
      }
    }
  }
// sortie de la fonction 
}
//@ ------------------------------------------------------------------*/
//@ Methode : GetDate
//@
//@ Description : Cette methode permet d'acceder a l'attribut m_GOPSDAT  
//@
//@
//@ Parameters :  
//@     (ENTREE)  NumCas : numero de cas degrade (entre 1 et 15)
//@
//@ Return Status : 
//@     aucun
//@
//@ ------------------------------------------------------------------*/
t_Date SD_DON_CasDegLigne::GetDate(
          int NumCas
        )
{
static const char tcProcName_F[] = "SD_DON_CasDegLigne::GetDate";
// Entree dans la fonction 
  //@ on recupere l'attribut m_GOPSDATFirst a partir de celui de l'objet 
  return(m_GOPSDATFirst[NumCas - 1]); 
// sortie de la fonction 
}
//@ ------------------------------------------------------------------*/
//@ Methode : GetFlt
//@
//@ Description : Cette methode permet d'acceder a l'attribut m_Flt  
//@
//@
//@ Parameters :  
//@     (ENTREE)  NumCas : numero de cas degrade (entre 1 et 15)    
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_CasDegLigne::t_SD_DON_UnFlag& SD_DON_CasDegLigne::GetFlt(
            int NumCas
        )
{
static const char tcProcName_F[] = "SD_DON_CasDegLigne::GetFlt";
// Entree dans la fonction 
  //@ on recupere l'attribut m_Flt[NumCG] a partir de celui de l'objet 
  return(m_Flt[NumCas - 1]);
// sortie de la fonction 
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetFlagPermanent
//@
//@ Description : Cette methode permet d'acceder a l'attribut m_casDegPermanent  
//@               Retourne 1 si un cas deg permanent a ete detecte dans la ligne
//@               Retourne 0 sinon
//@
//@ Parameters :  
//@     Aucun    
//@ 
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
int SD_DON_CasDegLigne::GetFlagPermanent(
        )
{
static const char tcProcName_F[] = "SD_DON_CasDegLigne::GetFlagPermanent";
// Entree dans la fonction 
  //@ on recupere l'attribut m_Flt[NumCG] a partir de celui de l'objet 
  return(m_casDegPermanent);
// sortie de la fonction 
}


//@ ------------------------------------------------------------------*/
//@ Methode : SetDetecte
//@
//@ Description : Cette methode permet de positionner un flag degrade CGXX  
//@
//@
//@ Parameters :  
//@     (ENTREE)  NumCas : numero de cas degrade (entre 1 et 15)
//@     (ENTREE)  date : date de levee de l'erreur UTC ??
//@     (ENTREE)  pos1 : position du flag dans dim1 SN
//@     (ENTREE)  pos2 : position du flag dans dim2 PN
//@     (ENTREE)  pos3 : position du flag dans dim3 CCD ou SB
//@         utile pour CG3 et CG4
//@     (ENTREE)  majDate : mise a jour ou non de la date First
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_DON_CasDegLigne::SetDetecte(
        const int numCas,
        const t_Date& date,
        int pos1,
        int pos2,
        int pos3,
        bool majDate   //true par defaut    
        )
{
static const char tcProcName_F[] = "SD_DON_CasDegLigne::SetDetecte";
t_CMN__Statut cr_op=CMN__STATUT_OK;
t_Date dateTmp(date);
JDB__Client MessageJDB;

  /* FA139 */
  int pos1min;
  int pos1max;
  
  //@ SI mode EXTERNAL_CALIBRATION
  if (m_Mode == SD_BAS_EXTERNAL_CALIBRATION)
  {
    pos1min = IND1(SD_BAS_CALIB_EXT_SN_MIN);
    pos1max = IND1(SD_BAS_CALIB_EXT_SN_MAX);
  }
  //@ SINON (mode AUXILIARY ou NORMAL)
  else
  {
    pos1min = IND1(1);
    pos1max = IND1(SD_BAS_SNOT);
  } //@ FIN SI mode AUXILIARY ou NORMAL  
  /* FIN FA139 */
  
// Entree dans la fonction 
  //@ SI le numero de cas est superieur a m_NbCasDeg
  if(numCas > SD_BAS_NB_CGXX)
  {
    //@ sortie en erreur
    cr_op = CMN__STATUT_NOK;
    goto fin;
  }
  //@ FIN SI
  //@ SI pos1 est superieur ou egal a m_Flt[numCas-1].dim1
  if(pos1 >= m_Flt[numCas-1].dim1)
  {
    //@ sortie en erreur
    cr_op = CMN__STATUT_NOK;
    goto fin;
  }
  //@ FIN SI
  //@ SI pos2 est superieur ou egal a m_Flt[numCas-1].dim2 
  if(pos2 >= m_Flt[numCas-1].dim2)
  {
    //@ sortie en erreur
    cr_op = CMN__STATUT_NOK;
    goto fin;
  }
  //@ FIN SI
  //@ SI pos3 est superieur ou egal a m_Flt[numCas-1].dim3 
  if(pos3 >= m_Flt[numCas-1].dim3)
  {
    //@ sortie en erreur
    cr_op = CMN__STATUT_NOK;
    goto fin;
  }

  //@ FIN SI
  //@ on positionne systematiquement m_GOPSDATLast[numCas-1]=date
  //@ pour faire suivre pour cas degrades induits
  if(majDate)
    m_GOPSDATLast[numCas-1] = dateTmp;
  //@ SI m_Flt[numCas-1].flaflt=false
  if(m_Flt[numCas-1].flaflt == false)
  {
    t_Date dateZero;
    dateZero.DateInit();
    //@ SI m_GOPSDATFirst[numCas-1] n'est pas initialise
    if(m_GOPSDATFirst[numCas-1] == dateZero)
    {
      //@ on positionne m_GOPSDATFirst[numCas-1]=date
      if(majDate)
        m_GOPSDATFirst[numCas-1] = dateTmp;
    }
    //@ SINON
    else
    {
      //@ SI cas CG6 ou CG9 et SN > 1 (SN = 1 vient de ligne precedente)
      if( (numCas == 6 || numCas == 9 ) && 
          (pos1 > pos1min) && 
	         (pos1 <= pos1max) ) /* FA139 */
      {
        t_Date tmpDate;
        //@ SI cas non leve pour SN precedent
        if(! this->IsDetecte(numCas, pos1 - 1, 0, 0, tmpDate))
        {
          //@ on positionne m_GOPSDATFirst[numCas-1]=date
          if(majDate)
            m_GOPSDATFirst[numCas-1] = dateTmp;
        }
      }
      // pour les cas 5, 7 et 8 on ne fait rien car cas par ligne
    }  
    //@ FIN SI
  }
  //@ FIN SI
  //@ on positionne m_Flt[numCas-1].flag[pos1][pos2] = true
  if ((pos1 >= pos1min) && (pos1 <= pos1max))
  {
    m_Flt[numCas-1].flag[pos1][pos2][pos3] = true;
  }

// FA 193
  //Gestion du cas CG14 sur les images de calibration externe
  if((14 == numCas) && (pos1 > IND1(SD_BAS_SNOT)))
    {
    m_Flt[numCas-1].flag[pos1][pos2][pos3] = true;
    }

// ..FA 193
  
/* FA 177 */
/* correction erreur allocation memoire en mode calibration */
  if((numCas == 9) && (pos1 >= IND1(1)) && (pos1 <= IND1(SD_BAS_SNOT)))
  {
    m_Flt[numCas-1].flag[pos1][pos2][pos3] = true;
  }
/* Fin FA 177 */

  //@ dans le cas de CG3 et CG4 utiliser m_GOPSDATFirstCG3 et m_GOPSDATFirstCG4
  //@ SI CG3
  if(numCas == 3)
  {
    //@ on positionne m_GOPSDATLastCG3[pos3]=date
    m_GOPSDATLastCG3[pos3] = dateTmp;
    t_Date dateZero;
    dateZero.DateInit();
    //@ SI m_GOPSDATFirstCG3[pos3] n'est pas initialise
    if(m_GOPSDATFirstCG3[pos3] == dateZero)
    {
      //@ on positionne m_GOPSDATFirstCG3[pos3]=date
      if(majDate)
      {
        m_GOPSDATFirstCG3[pos3] = dateTmp;
        m_GOPSDATFirst[2] = dateTmp;
      }
    }
    //@ SINON
    else
    {
      //@ SI pas dans les cas SN=1
      if( (pos1 > pos1min) && (pos1 <= pos1max) ) /* FA139 */
      {
        t_Date tmpDate;
        //@ SI cas degrade 3 pas positionne pour SN precedent
        if(! this->IsDetecte(numCas, pos1 - 1, 0, pos3, tmpDate))
        {
          //@ on positionne m_GOPSDATFirst[numCas-1]=date
          if(majDate)
          {
            m_GOPSDATFirstCG3[pos3] = dateTmp;
            m_GOPSDATFirst[2] = dateTmp;
          }
        }
        //@ FIN SI
      }
      //@ FIN SI
    }
    //@ FIN SI
  }
  //@ FIN SI CG3
  //@ SI CG4
  if(numCas == 4)
  {
    //@ on positionne m_GOPSDATLastCG3[pos3]=date
    m_GOPSDATLastCG4[pos3] = dateTmp;
    t_Date dateZero;
    dateZero.DateInit();
    //@ SI m_GOPSDATFirstCG4[pos3] n'est pas initialise
    if(m_GOPSDATFirstCG4[pos3] == dateZero)
    {
      //@ on positionne m_GOPSDATFirstCG4[pos3]=date
      if(majDate)
      {
        m_GOPSDATFirstCG4[pos3] = dateTmp;
        m_GOPSDATFirst[3] = dateTmp;
      }
    }
    //@ SINON
    else
    {
      //@ SI pas dans le cas SN=1 PN=1
      if ( (! (pos1 == pos1min && pos2 == 0)) && /* FA139 */
           (pos1 >= pos1min) && 
       	   (pos1 <= pos1max) )
      {
        //@ calcul du (SN,PN) precedent
        int SNprec;
        int PNprec;
        if(pos2 == 0)
        {
          SNprec = pos1 - 1;
          PNprec = 3;
        }
        else
        {
          SNprec = pos1;
          PNprec = pos2 - 1;
        }
        t_Date tmpDate;
        //@ SI cas degrade 4 pas positionne pour (SN,PN) precedent
        if(! this->IsDetecte(numCas, SNprec, PNprec, pos3, tmpDate))
        {
          //@ on positionne m_GOPSDATFirstCG4[pos3]=date
          if(majDate)
          {
            m_GOPSDATFirstCG4[pos3] = dateTmp;
            m_GOPSDATFirst[3] = dateTmp;
          }
        }
        //@ FIN SI
      }
      //@ FIN SI
    }   
    //@ FIN SI
  }
  //@ FIN SI CG4
/* CR288 */  
  //@ on increment le compteur des cas degrades rencontres
  m_nbCasDeg[numCas-1]++;
/* Fin CR288 */
  #ifdef OPS_DEBUG
/* Fin CR288 */
  cout << "Cas detecte ligne " << m_NumLigne << ": " << " Num = " << numCas << " Pos1 = " << pos1 << " Pos2 = " << pos2 << " Pos3 = " << pos3 << endl;
/* CR288 */
  #endif
fin:
// sortie de la fonction 
return(cr_op); 

}

//@ ------------------------------------------------------------------*/
//@ Methode : IsDetecte
//@
//@ Description : Cette methode permet de determiner si un flag  CGXX
//@   est positionne, la date de positionnement est alors retournee  
//@
//@
//@ Parameters :  
//@     (ENTREE)  NumCas : flags pour initialisation
//@     (ENTREE)  pos1 : position du flag dans dim1 SN
//@     (ENTREE)  pos2 : position du flag dans dim2 PN
//@     (ENTREE)  pos3 : position du flag dans dim3
//@     (SORTIE)  date : date de levee de l'erreur 
//@
//@ Return Status : 
//@     CMN__STATUT_OK si le cas degrade est leve
//@     CMN__STATUT_NOK sinon
//@
//@ ------------------------------------------------------------------*/
bool  SD_DON_CasDegLigne::IsDetecte(
        const int numCas,
        int pos1,
        int pos2,
        int pos3,
        t_Date& date
        )
{
static const char tcProcName_F[] = "SD_DON_CasDegLigne::IsDetecte";
bool cr_op = false;
// Entree dans la fonction 
  //@ SI le numero de cas est superieur a SD_BAS_NB_CGXX
  if(numCas > SD_BAS_NB_CGXX)
  {
    //@ sortie en false
    cr_op = false;
    goto fin;
  }
  //@ FIN SI
  //@ SI pos1 est superieur ou egal a m_Flt[numCas-1].dim1
  if(pos1 >= m_Flt[numCas-1].dim1)
  {
    //@ sortie en false
    cr_op = false;
    goto fin;
  }
  //@ FIN SI
  //@ SI pos2 est superieur ou egal a m_Flt[numCas-1].dim2
  if(pos2 >= m_Flt[numCas-1].dim2)
  {
    //@ sortie en false
    cr_op = false;
    goto fin;
  }
  //@ FIN SI
  //@ SI pos3 est superieur ou egal a m_Flt[numCas-1].dim3
  if(pos3 >= m_Flt[numCas-1].dim3)
  {
    //@ sortie en false
    cr_op = false;
    goto fin;
  }
  //@ FIN SI
  //@ on positionne la date a m_GOPSDAT[numCas-1]
  //@ on fait suivre date last (voir explication plus haut)
  date = m_GOPSDATLast[numCas-1];
  cr_op = m_Flt[numCas-1].flag[pos1][pos2][pos3];

fin:
// sortie de la fonction 
return(cr_op); 

}
//@ ------------------------------------------------------------------*/
//@ Methode : IsCasDeg
//@
//@ Description : Cette methode permet de determiner si une Ligne
//@   a donnee lieu a un cas degrade instrument  de plus 
//@             elle retourne un indicateur par cas degrade pour savoir
//@   si le cas en question est present dans la ligne
//@
//@
//@ Parameters :  
//@     (SORTIE)  Idconf : synthese par cas deg par ligne 
//@
//@ Return Status : 
//@     CMN__STATUT_OK si le cas degrade est leve
//@     CMN__STATUT_NOK sinon
//@
//@ ------------------------------------------------------------------*/
bool SD_DON_CasDegLigne::IsCasDeg(
    bool* Idconf)
{
static const char tcProcName_F[] = "SD_DON_CasDegLigne::IsCasDeg";
bool casDeg = false;
bool flagTmp;
// Entree dans la fonction 
  //@ POUR chaque element de t_SD_DON_Flt
  for(int i = 0; i < SD_BAS_NB_CGXX; i++)
  {
    flagTmp = false;
    //@ POUR chaque element de t_SD_DON_Flt[i].flag
    for(int j = 0; j < SD_BAS_SN; j++)
    {
      for(int k = 0; k < SD_BAS_PN; k++)
      {
        for(int l = 0; l < SD_BAS_SB; l++)
        {
          //@ SI m_Flt[i].flag[j][k] = true
          if(m_Flt[i].flag[j][k][l])
          {
            //@ casDeg = vrai
            casDeg = true;
            //@ flagTmp = vrai
            flagTmp = true;
          }
          //@ FIN SI
        }
      }
    }
    //@ FIN POUR
    //@ Idconf[i]=flagTmp
    Idconf[i] = flagTmp;
  }
  //@ FIN POUR 
fin :
// sortie de la fonction 
return(casDeg);
}


//@ ------------------------------------------------------------------*/
//@ Methode : SetPresence
//@
//@ Description : Cette methode permet de valider la presence d'un SN,PN  
//@
//@
//@ Parameters :  
//@     (ENTREE)  type : type du paquet
//@     (ENTREE)  sn : SN
//@     (ENTREE)  pn : PN
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_CasDegLigne::SetPresence(
        const e_SD_BAS_TypeN0& type,
        const int sn,
        const int pn    
        )
{
static const char tcProcName_F[] = "SD_DON_CasDegLigne::SetPresence";
t_CMN__Statut cr_op=CMN__STATUT_OK;
// Entree dans la fonction 

  //@ selon le type
  switch(type)
  {
    //@ IP :
    case IP :
      //@ SI sn est strt inferieur a SD_BAS_SN
      if(sn < SD_BAS_SN)
      {
        //@ m_presIP[sn]=true;
        m_presIP[sn] = true;
      };
      //@ FIN SI
      break;
    //@ PX :
    case PX :
      //@ SI sn est strt inferieur a SD_BAS_SNOT et pn a SD_BAS_PN
      if((sn < SD_BAS_SNOT) && (pn < SD_BAS_PN))
      {
        //@ m_presPX[sn][pn]=true;
        m_presPX[sn][pn] = true;
      };
      //@ FIN SI
      break;
    //@ VP :
    case VP :
      //@ SI sn compris entre 0 et 4
      if(sn < 5)
      {
        //@ m_presVP[sn]= true
        m_presVP[sn] = true;
      }
      //@ FIN SI
      break;
    //@ AP:
    case AP :
      //@ m_presAP=true
      m_presAP = true;
      break;
    }
    //@ m_presVP[InfoDetail.numVP-1]=true;
// sortie de la fonction 
}
//@ ------------------------------------------------------------------*/
//@ Methode : Valide
//@
//@ Description : Cette methode permet de valider l'analyse des cas degrade
//@ elle permet notamment de :
//@ - lever les cas d'absence de donnees
//@ - repercuter les CGxx dependant d'autre CGXX
//@
//@
//@ Parameters :  
//@     (ENTREE)  mode   : mode de prise de vue
//@     (ENTREE)  datePV : date de prise de vue
//@
//@ Return Status : 
//@     Aucune
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_DON_CasDegLigne::Valide(
        e_BAS_ModePdv mode,
        t_Date datePV
        )
{
static const char tcProcName_F[] = "SD_DON_CasDegLigne::Valide";
t_CMN__Statut cr_op=CMN__STATUT_OK;

  /* FA139 */
  int SNdeb;
  int SNfin;
  
  //@ SI mode EXTERNAL_CALIBRATION
  if (mode == SD_BAS_EXTERNAL_CALIBRATION)
  {
    SNdeb = 3;
    SNfin = SD_BAS_SNOT-1;
  }
  //@ SINON (mode AUXILIARY ou NORMAL)
  else
  {
    SNdeb = 1;
    SNfin = SD_BAS_SNOT;
  } //@ FIN SI mode AUXILIARY ou NORMAL  
  /* FIN FA139 */

// Entree dans la fonction 

  //@ SI m_presAP=  FALSE 
  if(m_presAP == false)
  {
    //@ positionner CG12 sur m_Flt[12-1].flag[0][0]
    this->SetDetecte(12, datePV, 0, 0, 0);
  }
  //@ FIN SI
    
//@ selon le mode :
//@   AUXILIARY ou NORMAL :
//@   les SNPX sont compris entre 0 et 29 
//@   les SNIP sont compris entre 0 et 29 31 et 32 et 34 35
//@ EXTERNAL_CALIBRATION :
//@   les SNPX sont compris entre 2 et 28 
//@   les SNIP sont compris entre 2 et 28 31 et 32 et 34 35

  //@ POUR tous les SN 
  for(int SN = 0; SN < SD_BAS_SN; SN++)
  {
    //@ SI m_presIP[SN] = FALSE et SN compris entre 0 et 29 + 31, 32, 34 et 35
// FA 193
    if((m_presIP[SN] == false) 
        // FT-2919
        && (  (SN < SD_BAS_SNOT)  
           || (SN == IND1(SD_BAS_SN_IP_SUP_1)) 
           || (SN == IND1(SD_BAS_SN_IP_SUP_2))
           || (SN == IND1(SD_BAS_SN_IP_SUP_3)) 
           || (SN == IND1(SD_BAS_SN_IP_SUP_4)) ) )
        // ..FT-2919
// .. FA 193

    {
      //@ SI mode EXTERNAL_CALIBRATION
      if(mode == SD_BAS_EXTERNAL_CALIBRATION)
      {
        //@ SI SN compris entre 2 et 28 
        if((SN !=0 ) && (SN !=1 ) && (SN != SD_BAS_SNOT - 1) )
        {
          //@ positionner CG14 m_Flt[14-1].flag[SN][0]
          this->SetDetecte(14, datePV, SN, 0, 0);
        }
        //@ FIN SI
      }
      //@ SINON (mode AUXILIARY ou NORMAL)
      else
      {
        //@ positionner CG14 sur m_Flt[14-1].flag[SN][0]
        this->SetDetecte(14, datePV, SN, 0, 0);
      }
      //@ FIN SI
    }
    //@ FIN SI m_presIP[SN] = FALSE
    //@ POUR tous les PN compris entre 0 et 3 et SN < SD_BAS_SNOT
    for(int PN = 0; (PN <= 3) && (SN < SD_BAS_SNOT); PN++)
    {
      //@ SI m_presPX[SN][PN]=  FALSE
      if(m_presPX[SN][PN] == false)
      {
        //@ SI mode EXTERNAL_CALIBRATION
        if(mode == SD_BAS_EXTERNAL_CALIBRATION)
        {
          //@ SI SN compris entre 2 et 28
          if((SN > 1) && (SN <  SD_BAS_SNOT - 1))
          {
            //@ positionner CG13 sur m_Flt[13-1].flag[SN][PN]
            this->SetDetecte(13, datePV, SN, PN, 0);
          }
        }
        //@ SINON (mode AUXILIARY ou NORMAL)
        else
        {
          //@ positionner CG13 sur m_Flt[13-1].flag[SN][PN]
          this->SetDetecte(13, datePV, SN, PN, 0);
        }
        //@ FIN SI
      }
      //@ FIN SI m_presPX[SN][PN]=  FALSE
    }
    //@ FIN POUR tous les PN compris entre 0 et 3
  }
  //@ FIN POUR tous les SN 
  //@ POUR les 5 cas de VP
  for(int i = 0; i < 5; i++)
  {
    //@ SI m_presVP[i]= FALSE
    if(m_presVP[i] == false)
    {
      //@ positionner CG11 sur m_Flt[11-1].flag[i][0]
      this->SetDetecte(11, datePV, i, 0, 0);
    }
    //@ FIN SI
  }
  //@ FIN POUR les 5 cas de VP
  
  //@ Logique sur les CGXX
  //@ SI CG12 alors CG5
  t_Date dateTmp1;
  if(this->IsDetecte(12, 0, 0, 0, dateTmp1))
  {
    this->SetDetecte(5, datePV, 0, 0, 0);
  }
  //@ FIN SI
  //@ POUR tous les SN (0-29)
  for(int SN = 0; SN < SD_BAS_SNOT ; SN++)
  {
    //@ POUR tous les PN
    for(int PN = 0; PN < SD_BAS_PN; PN++)
    {
      //@ SI CG13 alors CG1
      t_Date dateTmp2;
      if(this->IsDetecte(13, SN, PN, 0, dateTmp2))
      {
        this->SetDetecte(1, datePV, SN, PN, 0);
      }
      //@ FIN SI
    }
    //@ FIN POUR tous les PN
  }
  //@ FIN POUR tous les SN
  //@ POUR tous les SN (0-36)
  for(int SN = 0; SN < SD_BAS_SN; SN++)
  {
    t_Date dateTmp3;
    //@ SI CG14
    if(this->IsDetecte(14, SN, 0, 0, dateTmp3))
    {
      //@ SI SN sur 0 29 alors CG6
      if(SN < SD_BAS_SNOT)
      {
        this->SetDetecte(6, datePV, SN, 0, 0);
      }
      //@ FIN SI
      //@ SI SN sur 32 33 alors CG7 (premier SN=1)
      if((SN == IND1(SD_BAS_SN_IP_SUP_1)) || (SN == IND1(SD_BAS_SN_IP_SUP_2)))
      {
        this->SetDetecte(7, datePV, 0, 0, 0);
      }
      //@ FIN SI
      //@ SI SN sur 35 36 alors CG8 (premier SN=1)
      if((SN == IND1(SD_BAS_SN_IP_SUP_3)) || (SN == IND1(SD_BAS_SN_IP_SUP_4)))
      {
        this->SetDetecte(8, datePV, 0, 0, 0);
      }
      //@ FIN SI
    }
    //@ FIN SI CG14
  }
  //@ FIN POUR tous les SN
  //@ POUR tous les SN (0-29)
  for(int SN = SNdeb-1; SN < SNfin; SN++) /* FA139 */
  {
    t_Date dateTmp4;
    //@ SI CG6 alors CG3
    if(this->IsDetecte(6, SN, 0, 0, dateTmp4))
    {
      /* FA139 */
      bool MajDate;
      if (SN == SNdeb)
      {
        MajDate = false;
      }
      else
      {
        MajDate = true;
      }
      this->SetDetecte(3, datePV, SN, 0, 0, MajDate);
      this->SetDetecte(3, datePV, SN, 0, 1, MajDate);
      /* FIN FA139 */
    }
    //@ FIN SI
  }
  //@ FIN POUR tous les SN
  //@ SI CG4 sur 2 pixel d'un meme SN alors CG3
  
  //@ POUR tous les SN (0-29)
  for(int SN = SNdeb-1; SN < SNfin; SN++) /* FA139 */
  {
    int compteur = 0;
    //@ POUR tous les PN
    for(int PN = 0; PN < SD_BAS_PN; PN++)
    {
//SI CG4 alors CG1
      int compteur2 = 0;
      for(int SB = 0; SB < SD_BAS_SB; SB++)
      {
        t_Date dateTmp5;
        //@ SI CG4
        if(this->IsDetecte(4, SN, PN, SB, dateTmp5))
        {
          //@ incrementation du compteur
          compteur2++;
        }
        //@ FIN SI
      }     
    //@ FIN POUR tous les PN
      if(compteur2 == SD_BAS_SB)
      {
      //@ alors CG1
      this->SetDetecte(1, datePV, SN, PN, 0);
      this->SetDetecte(1, datePV, SN, PN, 1);
      }
      t_Date dateTmp4;
      //@ SI CG1
      if(this->IsDetecte(1, SN, PN, 0, dateTmp4))
      {
        //@ incrementation du compteur
        compteur++;
      }
      //@ FIN SI
    }
    //@ FIN POUR tous les PN
    //@ SI CG1 sur 2 pixel d'un meme SN 
    if(compteur > 1)
    {
      //@ alors CG3
      /* FA139 */
      bool MajDate;
      if (SN == SNdeb)
      {
        MajDate = false;
      }
      else
      {
        MajDate = true;
      }
      this->SetDetecte(3, datePV, SN, 0, 0, MajDate);
      this->SetDetecte(3, datePV, SN, 0, 1, MajDate);
      /* FIN FA139 */
    }
    //@ FIN SI
  }
  //@ FIN POUR tous les SN
  cr_op = this->FinValide(mode, datePV); /* FA139 */
return(cr_op);
}
//@ ------------------------------------------------------------------*/
//@ Methode : FinValide
//@
//@ Description : Cette methode permet de valider l'analyse des cas degrade
//@ elle permet notamment de :
//@ - detecter les cas degrades permanents
//@ - gerer la remise a 0 des dates de declenchement des cas deg
//@
//@
//@ Parameters :  
//@     (ENTREE)  datePV : date de prise de vue
//@
//@ Return Status : 
//@     Aucune
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_DON_CasDegLigne::FinValide(
        e_BAS_ModePdv mode, /* FA139 */
        t_Date datePV
        )
{
static const char tcProcName_F[] = "SD_DON_CasDegLigne::FinValide";
t_CMN__Statut cr_op=CMN__STATUT_OK;
// Entree dans la fonction   
  t_Date dateTmp5;
  t_Date dateZero;
  dateZero.DateInit();
  
  /* FA139 */
  int SNfin;
  
  //@ SI mode EXTERNAL_CALIBRATION
  if (mode == SD_BAS_EXTERNAL_CALIBRATION)
  {
    SNfin = SD_BAS_SNOT-1;
  }
  //@ SINON (mode AUXILIARY ou NORMAL)
  else
  {
    SNfin = SD_BAS_SNOT;
  } //@ FIN SI mode AUXILIARY ou NORMAL
  /* FIN FA139 */
  
  //@ Point sur les cas degrades permanents a la fin de l'analyse de la ligne
  //@ SI CG3 pas detecte
  for(int ccd = 0; ccd < SD_BAS_CCD; ccd ++)
  {
    if(! this->IsDetecte(3, SNfin - 1, 0, ccd, dateTmp5)) /* FA139 */
    {
      //@ DateFirst et DateLast reinitialises
      m_GOPSDATFirstCG3[ccd] = dateZero;
      m_GOPSDATLastCG3[ccd] = dateZero;
      m_GOPSDATFirst[2] = dateZero;
    }
  }
  //@ SI CG4 pas detecte
  for(int sb = 0; sb < SD_BAS_SB; sb ++)
  {
    if(! this->IsDetecte(4, SNfin - 1, SD_BAS_PN - 1, sb, dateTmp5))
    {
      //@ DateFirst et DateLast reinitialises
      m_GOPSDATFirstCG4[sb] = dateZero;
      m_GOPSDATLastCG4[sb] = dateZero;
      m_GOPSDATFirst[3] = dateZero;
    }
  }
  //@ FIN SI
  //@ SI CG5 pas detecte 
  if(! this->IsDetecte(5, 0, 0, 0, dateTmp5))
  {
    //@ DateFirst et DateLast reinitialises
    m_GOPSDATFirst[4] = dateZero;
    m_GOPSDATLast[4] = dateZero;
  }
  //@ FIN SI
  //@ SI CG6 pas detecte 
  if(! this->IsDetecte(6, SNfin - 1, 0, 0, dateTmp5))
  {
    //@ DateFirst et DateLast reinitialises
    m_GOPSDATFirst[5] = dateZero;
    m_GOPSDATLast[5] = dateZero;
  }
  //@ FIN SI
  //@ SI CG7 pas detecte 
  if(! this->IsDetecte(7, 0, 0, 0, dateTmp5))
  {
    //@ DateFirst et DateLast reinitialises
    m_GOPSDATFirst[6] = dateZero;
    m_GOPSDATLast[6] = dateZero;
  }
  //@ FIN SI
  //@ SI CG8 pas detecte 
  if(! this->IsDetecte(8, 0, 0, 0, dateTmp5))
  {
    //@ DateFirst et DateLast reinitialises
    m_GOPSDATFirst[7] = dateZero;
    m_GOPSDATLast[7] = dateZero;
  }
  //@ FIN SI
  //@ SI CG9 pas detecte 
  if(! this->IsDetecte(9, SNfin - 1, 0, 0, dateTmp5))
  {
    //@ DateFirst et DateLast reinitialises
    m_GOPSDATFirst[8] = dateZero;
    m_GOPSDATLast[8] = dateZero;
  }
  //@ FIN SI
  
  //@ POUR tous les CGXX permanents
  for(int cgxx = 0; cgxx < SD_BAS_NB_CGXX; cgxx++)
  {
    if(m_Flt[cgxx].flaflt == false)
    { 
      //@ on teste la duree du cas CGXX et on la compare a m_DureeCasDeg
      //@ SI pas CG3 ou CG4
      if(cgxx != 2 && cgxx !=3)
      {
        //@ SI m_GOPSDATLast[cgxx] - m_GOPSDATFirst[cgxx] > m_DureeCasDeg
        if(m_GOPSDATLast[cgxx].ConvertirDateEnTemps() - m_GOPSDATFirst[cgxx].ConvertirDateEnTemps() > m_DureeCasDeg[cgxx].ConvertirDateEnTemps())
        {
          //@  emission d'un message vers le JdBS
          if(m_casDegPermanentPrec == 0)
          {
            JDB__Client MessageJDB;
            MessageJDB.SetParametre(JDB_PARAM_1, cgxx+1);
            MessageJDB.EcrireMessage(SD_PRODUCT_SET, SD__Message::MSG_PROD_CAS_PERMANENT);
            MessageJDB.Liberer();
          }
          //@ positionnement du flag de cas deg permanent
          m_casDegPermanent = 1;
          //@  on retourne un cas degrade  pour arreter la chaine ( ou exception !)
//FA 70 on arrete la chaine en cas d'exception
          ERR__Exception exception;
          //@ Emission exception SD_DON_TMT_CASDEG
          exception.SetErrorCode(ERR__Exception::SD_DON_TMT_CASDEG);
            exception.SetDebParametre("Permanent Degraded Case raised");
          THROW(exception);
         }
        //@ FIN SI
      }
      //@ SI CG3
      else if(cgxx == 2)
      {
        //@ POUR tous les CCD
        for(int ccd = 0; ccd < SD_BAS_CCD; ccd++)
        {
          //@ SI m_GOPSDATLastCG3[ccd] - m_GOPSDATFirstCG3[ccd] > m_DureeCasDeg
          if(m_GOPSDATLastCG3[ccd].ConvertirDateEnTemps() - m_GOPSDATFirstCG3[ccd].ConvertirDateEnTemps() > m_DureeCasDeg[cgxx].ConvertirDateEnTemps())
          {
            //@   emission d'un message vers le JdBS
            if(m_casDegPermanentPrec == 0)
            {
              JDB__Client MessageJDB;
	      MessageJDB.SetParametre(JDB_PARAM_1, cgxx+1);
	      MessageJDB.EcrireMessage(SD_PRODUCT_SET, SD__Message::MSG_PROD_CAS_PERMANENT);
	      MessageJDB.Liberer();
            }
	    //@ positionnement du flag de cas deg permanent
	    m_casDegPermanent = 1;
//FA 70 on arrete la chaine en cas d'exception
             //@  on retourne un cas degrade  pour arreter la chaine ( ou exception !)
            ERR__Exception exception;
            //@ Emission exception SD_DON_TMT_CASDEG
            exception.SetErrorCode(ERR__Exception::SD_DON_TMT_CASDEG);
            exception.SetDebParametre("Permanent Degraded Case raised");
            THROW(exception);
           }
          //@ FIN SI
        }
        //@ FIN POUR
      }
      //@ SI CG4
      else if(cgxx == 3)
      {
        //@ POUR tous les SB
        for(int sb = 0; sb < SD_BAS_SB; sb++)
        {
          //@ SI m_GOPSDATLastCG4[sb] - m_GOPSDATFirstCG4[sb] > m_DureeCasDeg
          if(m_GOPSDATLastCG4[sb].ConvertirDateEnTemps() - m_GOPSDATFirstCG4[sb].ConvertirDateEnTemps() > m_DureeCasDeg[cgxx].ConvertirDateEnTemps())
          {
            //@   emission d'un message vers le JdBS
            if(m_casDegPermanentPrec == 0)
            {
              JDB__Client MessageJDB;
	      MessageJDB.SetParametre(JDB_PARAM_1, cgxx+1);
	      MessageJDB.EcrireMessage(SD_PRODUCT_SET, SD__Message::MSG_PROD_CAS_PERMANENT);
	      MessageJDB.Liberer();
            }
//FA 70 on arrete la chaine en cas d'exception
 	    //@ positionnement du flag de cas deg permanent
	    m_casDegPermanent = 1;
	    //@  on retourne un cas degrade  pour arreter la chaine ( ou exception !)
            ERR__Exception exception;
            //@ Emission exception SD_DON_TMT_CASDEG
            exception.SetErrorCode(ERR__Exception::SD_DON_TMT_CASDEG);
            exception.SetDebParametre("Permanent Degraded Case raised");
            THROW(exception);
          }
          //@ FIN SI
        }
        //@ FIN POUR
      }
      //@ FIN SI
    }
  }
  //@ FIN POUR tous les CGXX permanents
  //@ SI le flag de cas degrade permanent est leve
  if(m_casDegPermanent == 1)
  {
    //@ levee de CG2 sur toute la ligne
    //@ POUR tous les SN (0-29)
    for(int SN = 0; SN < SD_BAS_SNOT; SN++)
    {
      t_Date dateTmp4;
      //@ levee de CG2
      this->SetDetecte(2, datePV, SN, 0, 0);
    }
    //@ FIN POUR tous les SN
  }
  //@ FIN SI
  //@ logique a preciser cf FD 44 46  
return(cr_op);
}

/* CR288 */
//@ ------------------------------------------------------------------*/
//@ Methode : SyntheseCasDegrades
//@
//@ Description : Cette methode permet d'afficher dans le fichier Trace et les
//@  logs les cas degrades rencontres
//@
//@
//@ Parameters :  
//@
//@ Return :
//@
//@ ------------------------------------------------------------------*/
void SD_DON_CasDegLigne::SyntheseCasDegrades()
{
  JDB__Client MessageJDB;
  int i;
  
  for (i=0; i<SD_BAS_NB_CGXX; i++)
  {
    if (m_nbCasDeg[i]>0)
    {
      //@ emission d'un message vers le JdBS dans les cas principaux (CG11 12 13 14)
      //@ ceci doit etre precise par le cnes
  
      MessageJDB.SetParametre(JDB_PARAM_1, i+1);
      MessageJDB.EcrireMessage(SD_PRODUCT_SET, SD__Message::MSG_PROD_CAS_DEG);
      MessageJDB.Liberer();
      cout << "Line " << m_NumLigne << ": Degraded Case " << i+1 << " raised " << m_nbCasDeg[i] << " times." << endl;
    }
  }
}
/* Fin CR288 */

//======================================================================
//          METHODES PRIVEES 
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Methode : InitmFlt
//@
//@ Description : Initialise m_Flt
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_CasDegLigne::InitmFlt()
{
  //@ pour CG1
  m_Flt[0].flaflt = true;
  m_Flt[0].dim1 = SD_BAS_SNOT;
  m_Flt[0].dim2 = SD_BAS_PN;
  m_Flt[0].dim3 = 1;
  //@ pour CG2
  m_Flt[1].flaflt = true;
  m_Flt[1].dim1 = SD_BAS_SNOT;
  m_Flt[1].dim2 = 1;
  m_Flt[1].dim3 = 1;
  //@ pour CG3
  m_Flt[2].flaflt = false;
  m_Flt[2].dim1 = SD_BAS_SNOT;
  m_Flt[2].dim2 = 1;
  m_Flt[2].dim3 = SD_BAS_CCD;
  //@ pour CG4
  m_Flt[3].flaflt = false;
  m_Flt[3].dim1 = SD_BAS_SNOT;
  m_Flt[3].dim2 = SD_BAS_PN;
  m_Flt[3].dim3 = SD_BAS_SB;
  //@ pour CG5
  m_Flt[4].flaflt = false;
  m_Flt[4].dim1 = 1;
  m_Flt[4].dim2 = 1;
  m_Flt[4].dim3 = 1;
  //@ pour CG6
  m_Flt[5].flaflt = false;
  m_Flt[5].dim1 = SD_BAS_SNOT;
  m_Flt[5].dim2 = 1;
  m_Flt[5].dim3 = 1;
  //@ pour CG7
  m_Flt[6].flaflt = false;
  m_Flt[6].dim1 = 1;
  m_Flt[6].dim2 = 1;
  m_Flt[6].dim3 = 1;
  //@ pour CG8
  m_Flt[7].flaflt = false;
  m_Flt[7].dim1 = 1;
  m_Flt[7].dim2 = 1;
  m_Flt[7].dim3 = 1;
  //@ pour CG9
  m_Flt[8].flaflt = false;
  m_Flt[8].dim1 = SD_BAS_SNOT;
  m_Flt[8].dim2 = 1;
  m_Flt[8].dim3 = 1;
  //@ pour CG10
  m_Flt[9].flaflt = true;
  m_Flt[9].dim1 = 1;
  m_Flt[9].dim2 = 1;
  m_Flt[9].dim3 = 1;
  //@ pour CG11
  m_Flt[10].flaflt = true;
  m_Flt[10].dim1 = 5;
  m_Flt[10].dim2 = 1;
  m_Flt[10].dim3 = 1;
  //@ pour CG12
  m_Flt[11].flaflt = true;
  m_Flt[11].dim1 = 1;
  m_Flt[11].dim2 = 1;
  m_Flt[11].dim3 = 1;
  //@ pour CG13
  m_Flt[12].flaflt = true;
  m_Flt[12].dim1 = SD_BAS_SNOT;
  m_Flt[12].dim2 = SD_BAS_PN;
  m_Flt[12].dim3 = 1;
  //@ pour CG14
  m_Flt[13].flaflt = true;
  m_Flt[13].dim1 = SD_BAS_SN;
  m_Flt[13].dim2 = 1;
  m_Flt[13].dim3 = 1;
  //@ pour CG15
  m_Flt[14].flaflt = true;
  m_Flt[14].dim1 = SD_BAS_SNOT;
  m_Flt[14].dim2 = 1;
  m_Flt[14].dim3 = 1;
}


