//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_CasDegLigne.h,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_CasDegLigne
//@ 
//@ Comment :
//@    this module implement the SD_DON_CasDegLigne class
//@    used to administrate the degraded cases in a line of OPS product
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 20/12/2002 : P.GASSIE
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-139-CN : 21/09/2004 : B.MASSART
//@ Probleme d'initialisation de GDatFlagIsrfemOff quand on entre en mode
//@ calibration externe ne visant pas la Terre. On introduit le mode
//@ de prise de vue dans la gestion des cas degrades. Voir IA-2100-FC-139-THA.
//@ --------------------------------------------------------------------
//@ DM-ID : IA-2100-CR-288-CN : 20/03/2006 : B. GUENDE
//@ Function SyntheseCasDegrades added to print a summary of all degraded cases encountered.
//@ Counter added instead of print command. See IA-2100-CR-288-THA.
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_DON_CasDegLigne_h
#define SD_DON_CasDegLigne_h


// Inserer ici les fichiers include 
#include "SD_BAS_Global.h"
#include "SD_BAS_Niveau0.h"
#include "SD_DON_Contexte.h"
#include "CMN__Date.h"
#include "SD__Message.h"
#include "JDB__Client.h"


// Definition des exceptions

class SD_DON_CasDegLigne {

/*====================================================================*/

public:

  //
  // Type definissant les info de duree de cas sur la ligne 
  //	
  typedef t_Date t_SD_DON_IdefDur[SD_BAS_NB_CGXX]; // duree limite du cas degrade  

  //
  // Type definissant les date de declanchement de cas sur la ligne 
  //	
  typedef t_Date t_SD_DON_GOPSDAT[SD_BAS_NB_CGXX]; // declenchement du cas degrade  

  //
  
  // Type definissant les info pour initialiser les flags sur la ligne 
	// a mettre dans SD_DON_CasDegLigne.h
  typedef struct {
	 	bool flaflt; // si true on a un flag de type fla
	 	int dim1;
	 	int dim2;
	 	int dim3;
		bool flag[SD_BAS_SN][SD_BAS_PN][SD_BAS_SB]; // flags pour CG1
  }	
  t_SD_DON_UnFlag;

  typedef t_SD_DON_UnFlag	t_SD_DON_Flt[SD_BAS_NB_CGXX];
		
  //
  // Constant
  //
	//
	// Constructeur
	//  
  SD_DON_CasDegLigne(int NumLigne, e_BAS_ModePdv Mode); /* FA139 */
	
 //
	// Destructeur
	//
	virtual ~SD_DON_CasDegLigne();
	//
	// Accesseurs
	//
	t_Date GetDate(
	        int NumCas
	);
	t_SD_DON_UnFlag& GetFlt(
	        int NumCas
				);
	int GetFlagPermanent(
	);
	//
	// publics Methods 
	//
	void Init(
			     SD_DON_Contexte  & Contexte,
			     const t_SD_DON_IdefDur & durees
	);
		
	void InitPrec(
			     SD_DON_CasDegLigne  & CasDegLignePrec,
			     const t_SD_DON_IdefDur & durees
	);
	void InitFlt(
					const t_SD_DON_Flt & Flags,
					const t_SD_DON_GOPSDAT & Dates
				);

	bool IsDetecte(
				const int numCas,
				int pos1,
				int pos2,
				int pos3,	
				t_Date& date
				);
	bool IsCasDeg();
	
	void SetPresence(
				const e_SD_BAS_TypeN0& type,
				const int sn,
				const int pn		
				);
	t_CMN__Statut  Valide(
				e_BAS_ModePdv mode,
				t_Date datePV
				);
	t_CMN__Statut  FinValide(
    e_BAS_ModePdv mode, /* FA139 */
				t_Date datePV
				);
	t_CMN__Statut SetDetecte(
				const int numCas,
				const t_Date& date,
				int pos1,
				int pos2,
				int pos3,
				bool majDate = true		
				);
	bool IsCasDeg(
		bool* Idconf
		);

/* CR288 */
        void SyntheseCasDegrades();
/* Fin CR288 */


	/*====================================================================*/
protected:
	//
	// protected Methods 
	//

	//
	// protected Attributes 
	//

	/*====================================================================*/
private:
	//
	// Private Methods 
	//
	void InitmFlt();
	//
	// Private Attributes 
	//
	//
	// Constantes
	//
	static const t_Chaine JDB_PARAM_1;

	// durees issues du fichier de conf
	t_SD_DON_IdefDur  m_DureeCasDeg;
	
	// dates de declenchement
	t_SD_DON_GOPSDAT  m_GOPSDATFirst;
	t_SD_DON_GOPSDAT  m_GOPSDATLast;

	// dates de declenchement CG3
	t_Date  m_GOPSDATFirstCG3[SD_BAS_CCD];
	t_Date  m_GOPSDATLastCG3[SD_BAS_CCD];
	
	// dates de declenchement CG4
	t_Date  m_GOPSDATFirstCG4[SD_BAS_SB];
	t_Date  m_GOPSDATLastCG4[SD_BAS_SB];
	
	// flags de gestion du cas deg 
	t_SD_DON_Flt  m_Flt;

	// numero de ligne 
	int  m_NumLigne;

	// indicateur de presence des IP
	bool  m_presIP[SD_BAS_SN];
	
	// indicateur de presence des PX
	bool  m_presPX[SD_BAS_SNOT][SD_BAS_PN];
	
	// indicateur de presence du AP
	bool  m_presAP;
	
	// indicateur de presence des VP
	bool  m_presVP[5];
	
	// indicateur de presence d'un cas degrade permanent dans la ligne
	int m_casDegPermanent;
	int m_casDegPermanentPrec;

	// mode de prise de vue
 e_BAS_ModePdv m_Mode; /* FA139 */
 
/* CR288 */
        // tableau compteur des cas degrades rencontres
        int m_nbCasDeg[SD_BAS_NB_CGXX];
/* Fin CR288 */
};


#endif
