static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_ConfAutre.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_ConfAutre
//@
//@ Comment :
//@    this module implement the SD_DON_ConfAutre class
//@    used to reference the "other" Configuration FIle
//@           
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 30/11/2002 : P.GASSIE
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <iostream>

#include "SD_DON_ConfAutre.h"
#include "CMN__OPSFile.h"
#include "JDB__Client.h"

//
// Definition des constantes
//
const int SD_DON_ConfAutre::POS_VAL_DATE = 5;
const int SD_DON_ConfAutre::POS_CRE_DATE = 7;
const int SD_DON_ConfAutre::POS_PTSI     = 10;
const int SD_DON_ConfAutre::NBRE_MIN     = 10;
const t_Chaine SD_DON_ConfAutre::JDB_PARAM_1 = "$PRM1";
const t_Chaine SD_DON_ConfAutre::DATA_AUX_AUT_READ_ERROR = "Other parameters configuration file read error";

//======================================================================
//           METHODES PUBLIQUES 
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Methode : SD_DON_ConfAutre
//@
//@ Description : Constructeur
//@
//@ Parameters :
//@     Aucun
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_ConfAutre::SD_DON_ConfAutre()
{
  // initialisation des parametres
  m_PTSI = -1;
  m_DataCA = NULL;
  m_identCS = -1;
  m_identBS = -1;
  m_Charge = false;
}
//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_DON_ConfAutre
//@
//@ Description : Destructeur
//@
//@
//@ Parameters :
//@     Aucun
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_ConfAutre::~SD_DON_ConfAutre()
{
  //@ SI il y a une valeur dans le champ m_DataCA
  if (m_DataCA)
  {
    //@ On la libere
    delete m_DataCA;
  }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : Initialise
//@
//@ Description : Cette methode permet d'initialiser les informations
//@ generales sans charger le fichier de configuration autre
//@
//@
//@ Parameters :
//@     (ENTREE)  NomFichier : nom du fichier
//@
//@ Return Status :   aucun
//@
//@ ------------------------------------------------------------------*/
void SD_DON_ConfAutre::Initialise(
    const t_Chaine & NomRepertoire,
    const t_Chaine & NomFichier
    )

{
// Entree dans la fonction 
  //@ on verifie que la donnee n'a pas ete deja chargee
  if (m_Charge == false)
    {
      //@ on construit un objet t_OPSFile
      t_OPSFile fichierTmp(NomFichier);
      //@ sauvegarde du nom du fichier (+repertoire)
      strcpy(m_NomFichier, (NomRepertoire + "/" + NomFichier).c_str());
      //@ sauvegarde de la date de validite
      t_Date dateValidTmp(fichierTmp.GetValidityStartDate());
      m_DateValid = dateValidTmp;
      //@ sauvegarde de la date de creation
      t_Date dateCreationTmp(fichierTmp.GetCreationDate());
      m_DateCreation = dateCreationTmp;
      //@ sauvegarde de l'identificateur 
      m_PTSI = fichierTmp.GetIdent();
      //@ suavegarde du nom complet
      m_NomComplet = NomRepertoire + "/" + NomFichier;
    }
    //@ FIN SI
// on trace ici si besoin toutes les variables en sortie
#ifdef OPS_DEBUG
cout << endl << "SD_DON_ConfAutre::Initialise" << endl;
cout << "Nom du fichier a lire : " << m_NomFichier << endl;
cout << "PTSI : " << m_PTSI << endl;
cout << "Date Validite : " << m_DateValid.Chaine() << endl;
cout << "Date Creation : " << m_DateCreation.Chaine() << endl;
#endif
// sortie de la fonction 
}
//@ ------------------------------------------------------------------*/
//@ Methode : Lire
//@
//@ Description : Cette methode permet de lire les informations
//@ stockees dans le fichier de configuration autre
//a il faut au prealable avoir initialise l'objet par Initialise
//@
//@
//@ Parameters :  aucun
//@
//@ Return Status : 
//@
//@ ------------------------------------------------------------------*/
void SD_DON_ConfAutre::Lire(
    ) throw(ERR__Exception)
{
  static const char tcProcName_F[] = "SD_DON_ConfAutre::Lire";
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  t_SD_BAS_DataThread* ctx;

// Entree dans la fonction 

  //@ SI m_Charge == false
  if (m_Charge == false)
    {
      //@ on alloue la donnee m_DataCS 
      m_DataCA = new t_SD_BAS_ConfAutre;
      //@ on lit le fichier via la fonction de SD_BAS_ConfAutre
      //@ un controle de coherence sur la version du format est ici mene
      //@ par la couche SD_BAS_ConfAutre
      ctx = new t_SD_BAS_DataThread;
      InitialiserCompteRendu(&(ctx->cr));

      cr_op = SD_BAS_ChargerConfAutre(m_NomFichier ,m_DataCA ,ctx);
      //@ SI erreur
      if(cr_op != CMN__STATUT_OK)
      {
        //@ emission des messages du ctx
			  EnvoyerMessagesStockes(&(ctx->cr.msgJDB));
        // emission d'un message vers le JdBS
        JDB__Client MessageJDB;
        MessageJDB.SetParametre(JDB_PARAM_1, m_NomFichier);
        MessageJDB.EcrireMessage(SD_CONF_SET, SD__Message::MSG_CONF_LOAD_ERR);
        MessageJDB.Liberer();
        LibererCompteRendu(&(ctx->cr.msgJDB));
        delete ctx;
        // exception
        ERR__Exception exception;
        //@ Emission exception SD_DON_TMT_DATA_IN
        exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_AUX);
        exception.SetParametre(DATA_AUX_AUT_READ_ERROR);
        THROW(exception);
      }
      //@ FIN SI
      JDB__Client MessageJDB;
		  MessageJDB.SetParametre(JDB_PARAM_1, m_NomFichier);
			MessageJDB.EcrireMessage(SD_CONF_SET, SD__Message::MSG_CONF_LOAD_OK);
			MessageJDB.Liberer();
      LibererCompteRendu(&(ctx->cr.msgJDB));
      delete ctx;
      //@ m_VersionFormat
      
      //@ m_identCS
      m_identCS = m_DataCA->IDefStableparamID;
      //@ m_identBS
      m_identBS =  m_DataCA->IDefSdbID;
      //@ l'indicateur de chargement est initialise
      m_Charge = true;
    }
    //@ FIN SI m_Charge == false
// on trace ici si besoin toutes les variables en sortie
#ifdef OPS_DEBUG
cout << endl << "SD_DON_ConfAutre::Lire" << endl;
cout << "Nom du fichier lu : " << m_NomFichier << endl;
#endif  
// sortie de la fonction 
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetData
//@
//@ Description : Cette methode permet de recuperer l'adresse de la zone
//@    memoire ou est stockee le fichier de conf autre 
//@
//@
//@ Parameters :   aucun
//@     
//@
//@ Return Status :   t_SD_BAS_Autre pointeur sur la donnee
//@
//@ ------------------------------------------------------------------*/
t_SD_BAS_ConfAutre* SD_DON_ConfAutre::GetData(
    )
{
// Entree dans la fonction 
  //@ return m_DataCA
  return(m_DataCA);
// sortie de la fonction 
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetPTSI
//@
//@ Description : Cette methode permet de recuperer le PTSI
//@    du fichier 
//@
//@
//@ Parameters :   aucun
//@     
//@
//@ Return Status :   int PTSI du fichier de conf autre
//@
//@ ------------------------------------------------------------------*/
int  SD_DON_ConfAutre::GetPTSI(
    )
{
// Entree dans la fonction 
  //@  return m_PTSI
  return(m_PTSI);
// sortie de la fonction 
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetDateValid
//@
//@ Description : Cette methode permet de recuperer la date de validite
//@    du fichier 
//@
//@
//@ Parameters :   aucun
//@     
//@
//@ Return Status :   t_Date date de validite du fichier de conf autre
//@
//@ ------------------------------------------------------------------*/
t_Date  SD_DON_ConfAutre::GetDateValid(
    )
{
// Entree dans la fonction 
  //@  return m_DateValid
  return(m_DateValid);
// sortie de la fonction 
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetDateCreation
//@
//@ Description : Cette methode permet de recuperer la date de creation
//@    du fichier 
//@
//@
//@ Parameters :   aucun
//@     
//@
//@ Return Status :   t_Date date de creation du fichier de conf autre
//@
//@ ------------------------------------------------------------------*/
t_Date  SD_DON_ConfAutre::GetDateCreation(
    )
{
// Entree dans la fonction 
  //@  return m_DateValid
  return(m_DateCreation);
// sortie de la fonction 
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetNom
//@
//@ Description : Cette methode permet de recuperer le nom complet
//@    du fichier (reperoire + nom)
//@
//@
//@ Parameters :   aucun
//@     
//@
//@ Return Status :   t_Chaine nom complet du fichier de conf autre
//@
//@ ------------------------------------------------------------------*/
t_Chaine  SD_DON_ConfAutre::GetNom(
    )
{
// Entree dans la fonction 
  //@  return m_NomComplet
  return(m_NomComplet);
// sortie de la fonction 
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetIdConf
//@
//@ Description : Cette methode permet de recuperer les Identifiants
//@    des fichiers de conf autre et de la BS associes
//@
//@
//@ Parameters :   
//@     (SORTIE)  IdCA : Identificateur du fichier de conf autre 
//@     (SORTIE)  IdCS : Identificateur du fichier de conf stable 
//@     (SORTIE)  IdBS : Identificateur de la banque spectrale 
//@     
//@
//@ Return Status :   int PTSI du fichier de conf autre
//@
//@ ------------------------------------------------------------------*/
void  SD_DON_ConfAutre::GetIdConf(
    int& IdCA,
    int& IdCS,
    int& IdBS
    )
{
// Entree dans la fonction 
  //@ IdCA=m_DataCA->IDefConf
  IdCA = m_DataCA->IDefConf;
  //@ IdCS=m_identCS
  IdCS = m_identCS;
  //@ IdBS=m_identBS
  IdBS = m_identBS;
// sortie de la fonction 
}

//======================================================================
//          METHODES PRIVEES 
//======================================================================
