static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_ConfStable.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_ConfStable
//@
//@ Comment :
//@    this module implement the SD_DON_ConfStable class
//@    used to reference the stable Configuration FIle
//@                                     
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 30/11/2002 : P.GASSIE
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <iostream>

#include "SD_DON_ConfStable.h"
#include "CMN__OPSFile.h"
#include "JDB__Client.h"

//
// Definition des constantes
//
const int SD_DON_ConfStable::POS_VAL_DATE = 5;
const int SD_DON_ConfStable::POS_CRE_DATE = 7;
const int SD_DON_ConfStable::POS_ID       = 10;
const int SD_DON_ConfStable::NBRE_MIN     = 10;
const t_Chaine SD_DON_ConfStable::JDB_PARAM_1 = "$PRM1";
const t_Chaine SD_DON_ConfStable::DATA_AUX_STA_NOT_FOUND = "Stable parameters configuration file not found";
const t_Chaine SD_DON_ConfStable::DATA_AUX_STA_READ_ERROR = "Stable parameters configuration file read error";

//======================================================================
//                                       METHODES PUBLIQUES 
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Methode : SD_DON_ConfStable
//@
//@ Description : Constructeur
//@
//@ Parameters :
//@                     Aucun
//@
//@ Return Status : 
//@                     Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_ConfStable::SD_DON_ConfStable()
{
  // Initialisation des attributs
  m_DataCS = NULL;
  m_ident = -1;
  m_Charge = false;
}
//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_DON_ConfStable
//@
//@ Description : Destructeur
//@
//@
//@ Parameters :
//@                     Aucun
//@
//@ Return Status : 
//@                     Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_ConfStable::~SD_DON_ConfStable()
{

  //@ SI il y a une valeur dans le champ m_DataCS
  if (m_DataCS)
    {
      //@ On la libere
      delete m_DataCS;
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : Initialise
//@
//@ Description : Cette methode permet d'initialiser les informations
//@ generales sans charger le fichiers de configuration stable
//@
//@
//@ Parameters :
//@                     (ENTREE)  Repertoire : Repertoire contenant les fichiers
//@
//@ Return              t_Chaine nom fichier
//@ Status :       void
//@
//@ ------------------------------------------------------------------*/
void SD_DON_ConfStable::Initialise(
  const int &      IdStable, 
  const t_Date &   tcourante, 
  const t_Chaine & repertoire, 
  t_Chaine &       nom_fichier) 
  
throw(ERR__Exception)
{
  // Entree dans la fonction
  CMN__ListeInfoFichier ListeInfoFichier;
  CMN__ListeInfoFichier::iterator It;
  t_Chaine pattern("GRD");
  int IdStableTmp = -1;
  t_CMN__Statut cr_op = CMN__STATUT_OK; 
        
  //@ on recupere dans une liste l'ensemble des fichiers de type conf Stable
  cr_op = CMN__TrouverFichiersEx(repertoire, pattern, ListeInfoFichier, NULL, NULL, NULL, 1, 0);
  //@ SI erreur
  if(cr_op != CMN__STATUT_OK)
    {
      //@ message JDB
      JDB__Client MessageJDB;
      MessageJDB.SetParametre(JDB_PARAM_1, pattern);
      MessageJDB.EcrireMessage(SD_CONF_SET, SD__Message::MSG_CONF_NO_FILE);
      MessageJDB.Liberer();
      //@ exception
      ERR__Exception exception;
    
      //@ Emission exception SD_DON_TMT_DATA_AUX
      exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_AUX);
      exception.SetParametre(DATA_AUX_STA_NOT_FOUND);
      THROW(exception);
    }
  //@ FIN SI
  //@ POUR chaque element de la liste
  for (It = ListeInfoFichier.begin();
       It != ListeInfoFichier.end();
       It ++)
    {
      // on construit un objet t_OPSFile
      t_OPSFile fichierTmp(It->Name());
      // on lit la date de validite du fichier et le IdStable
      t_Date dateValiditeTmp(fichierTmp.GetValidityStartDate());
      IdStableTmp = fichierTmp.GetIdent();
      //@ SI le IdStable est = Iddemande et si la date de debut validite est anterieure
      //@ a la date courante
      t_Date dateTmp(tcourante);
      if ((IdStable == IdStableTmp) && (dateValiditeTmp <= dateTmp))
        {
          //@ SI le fichier est deja reference
          if (m_ident != -1)
            {
              //@ SI date de debut validite prec < date de debut validite
              if (m_DateValid < dateValiditeTmp)
                {
                  //@ on reference les nouveaux fichiers
                  strcpy(m_NomFichier, (repertoire + "/" + fichierTmp.GetNom()).c_str());
                  m_ident = IdStableTmp;
                  m_DateValid = dateValiditeTmp;
                  t_Date dateCreationTmp(fichierTmp.GetCreationDate());
                  m_DateCreation = dateCreationTmp;
                  nom_fichier = repertoire + "/" + It->Name();
                }
              //@ FIN SI
            }
          //@ SINON
          else
            {
              //@ on reference le nouveau fichier
              strcpy(m_NomFichier, (repertoire + "/" + fichierTmp.GetNom()).c_str());
              m_ident = IdStableTmp;
              m_DateValid = dateValiditeTmp;
              t_Date dateCreationTmp(fichierTmp.GetCreationDate());
              m_DateCreation = dateCreationTmp;
              nom_fichier = repertoire + "/" + It->Name();
            }
          //@ FIN SI le fichier est deja reference
        }
      //@ FIN SI
    }    
  //@ FIN POUR
  //@ SI aucun fichier n'est reference 
  if (m_ident == -1)
    {
      //@ emission d'un message vers le JdBS
      JDB__Client MessageJDB;
      MessageJDB.SetParametre(JDB_PARAM_1, pattern);
      MessageJDB.EcrireMessage(SD_CONF_SET, SD__Message::MSG_CONF_NO_FILE);
      MessageJDB.Liberer();
      //@ exception
      ERR__Exception exception;
    
      //@ Emission exception SD_DON_TMT_DATA_AUX
      exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_AUX);
      exception.SetParametre(DATA_AUX_STA_NOT_FOUND);
      THROW(exception);
    }
    
  // on trace ici si besoin toutes les variables en sortie
#ifdef OPS_DEBUG
  cout << endl << "SD_DON_ConfStable::Initialise" << endl;
  cout << "Nom du fichier a lire : " << m_NomFichier << endl;
  cout << "IdStable : " << m_ident << endl;
  cout << "Date Validite : " << m_DateValid.Chaine() << endl;
  cout << "Date Creation : " << m_DateCreation.Chaine() << endl;
#endif
  // sortie de la fonction 
}
//@ ------------------------------------------------------------------*/
//@ Methode : Lire
//@
//@ Description : Cette methode permet de lire les informations
//@ stockees dans le fichier de configuration stable
//a il faut au prealable avoir initialise l'objet par Initialise
//@
//@
//@ Parameters :  aucun
//@
//@ Return Status : 
//@
//@ ------------------------------------------------------------------*/
void SD_DON_ConfStable::Lire(
                ) throw(ERR__Exception)

{
  static const char tcProcName_F[] = "SD_DON_ConfStable::Lire";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  t_SD_BAS_DataThread* ctx;
  // Entree dans la fonction 

  //@ SI la donnee n'a pas deja ete chargee
  if (m_Charge == FALSE)
    {
      //@ on alloue la donnee m_DataCS
      m_DataCS = new t_SD_BAS_ConfStable;
      //@ on lit le fichier via la fonction de SD_BAS_ConfStable
      ctx = new t_SD_BAS_DataThread;
      InitialiserCompteRendu (&(ctx->cr));

      cr_op = SD_BAS_ChargerConfStable(m_NomFichier, m_DataCS, ctx);
      //@ SI erreur
      if(cr_op != CMN__STATUT_OK)
        {
          //@ emission des messages du ctx
          EnvoyerMessagesStockes(&(ctx->cr.msgJDB));
          //@ emission d'un message vers le JdBS
          JDB__Client MessageJDB;
          MessageJDB.SetParametre(JDB_PARAM_1, m_NomFichier);
          MessageJDB.EcrireMessage(SD_CONF_SET, SD__Message::MSG_CONF_LOAD_ERR);
          MessageJDB.Liberer();
          LibererCompteRendu(&(ctx->cr.msgJDB));
          delete ctx;
          //@ exception
          ERR__Exception exception;
    
          //@ Emission exception SD_DON_TMT_DATA_AUX
          exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_AUX);
          exception.SetParametre(DATA_AUX_STA_READ_ERROR);
          THROW(exception);                     
        }
      //@ SINON
      {
        //@ l'indicateur de chargement est initialise
        JDB__Client MessageJDB;
        MessageJDB.SetParametre(JDB_PARAM_1, m_NomFichier);
        MessageJDB.EcrireMessage(SD_CONF_SET, SD__Message::MSG_CONF_LOAD_OK);
        MessageJDB.Liberer();
        LibererCompteRendu(&(ctx->cr.msgJDB));
        delete ctx;
        m_Charge = TRUE;
      }
      //@ FIN SI
    }
  //@ FIN SI
  // on trace ici si besoin toutes les variables en sortie
#ifdef OPS_DEBUG
  cout << endl << "SD_DON_ConfStable::Lire" << endl;
  cout << "Nom du fichier lu : " << m_NomFichier << endl;
  cout << "Lecture conf stable : " << m_DataCS->IDefStableParamDate.day << endl;
#endif  
  // sortie de la fonction  
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetData
//@
//@ Description : Cette methode permet de recuperer l'adresse de la zone
//@              memoire ou est stockee le fichier de conf stable 
//@
//@
//@ Parameters :   aucun
//@                     
//@
//@ Return Status :     t_SD_BAS_ConfStable pointeur sur la donnee
//@
//@ ------------------------------------------------------------------*/
t_SD_BAS_ConfStable* SD_DON_ConfStable::GetData(
                )
{
  // Entree dans la fonction
  //@ return m_DataCS
  return(m_DataCS);
  // sortie de la fonction 
}

//@ ------------------------------------------------------------------*/
//@ Methode : Verifie
//@
//@ Description : Cette methode permet de verifier la coherence du fichier
//@ en debut de chaque granule par rapport a l'identificateur issu du fichier
//@ de conf autre courant
//@
//@
//@ Parameters :
//@                     (ENTREE)  Ident : Identificateur de la banque spectrale 
//@
//@ Return Status : 
//@                     t_CMN__Statut 
//@                     CMN__STATUT_OK si la fonction a reussi
//@                     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_DON_ConfStable::Verifie(
                                         const int& Ident
                                         )

{
  static const char tcProcName_F[] = "SD_DON_ConfStable::Verifie";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  // Entree dans la fonction 

  //@ SI le fichier a  deja ete charge
  if (m_Charge == TRUE)
    {
      //@ SI Ident == m_IndStable 
      if (Ident == m_ident)
        {
          //@ return CMN__STATUT_OK
          cr_op=CMN__STATUT_OK;
        }
      //@ SINON
      else
        {
          //@ return CMN__STATUT_NOK
          cr_op=CMN__STATUT_NOK;
        }
      //@ FIN SI
    }
  //@ FIN SI
  // on trace ici si besoin toutes les variables en sortie

  // sortie de la fonction 
  return(cr_op); 

}

//======================================================================
//                                      METHODES PRIVEES 
//======================================================================
