static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_Contexte.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_Contexte
//@ 
//@ Comment :
//@    this module implement the SD_DON_Contexte class
//@    used to reference the OPS Context File
//@           
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 12/12/2002 : P.GASSIE
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-42-CN : 01/07/2003 : B. MASSART 
//@ dans AjouteElemHist, le test DUMP_OPS_N_01 permet de realiser la condition
//@ m_DataContexte->GBBTNbElt == SD_BAS_NB_HIST_BBT = 400
//@ le nombre d'elements GBBTNbElt ecrit dans le contexte doit alors plafonner 
//@ a 400 or il continuait a etre incremente.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-169-CN : 27/02/2006 : B.MASSART
//@ Syntax correction for IisInHomPix. See IA-2100-FC-169-THA.
//@ --------------------------------------------------------------------
//@ DM-ID :IA-2100-CR-293-CNE : 08/03/2006 : B.GUENDE
//@ The organism that creates CTX files is defined by the environment variable
//@ CONTEXT_SOURCE stored in SD.env. See IA-2100-CR-293-THA.
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <iostream>

#include "CFG__Configuration.h"
#include "SD_DON_Contexte.h"
#include "CMN__OPSFile.h"
#include "JDB__Client.h"

//
// Definition des constantes
//
const t_Chaine SD_DON_Contexte::JDB_PARAM_1 = "$PRM1";
const t_Chaine SD_DON_Contexte::DATA_AUX_CTX_NOT_FOUND = "Context file not found";
const t_Chaine SD_DON_Contexte::DATA_AUX_CTX_READ_ERROR = "Context file read error";
const t_Chaine SD_DON_Contexte::DATA_AUX_CTX_VERSION_ERROR = "Context file version error";
const t_Chaine SD_DON_Contexte::DATA_AUX_CTX_WRITE_ERROR = "Context file write error";

/* CR293 */
const t_Chaine SD_DON_Contexte::s_NOM_SOURCE = "CONTEXT_SOURCE";
const t_Chaine SD_DON_Contexte::NOM_SOURCE_DEFAUT = "CGS1";
/* FIN CR293 */

// initialisation variable statique
pthread_mutex_t   SD_DON_Contexte::m_MutexTravaille;

//======================================================================
//           METHODES PUBLIQUES 
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Methode : SD_DON_Contexte
//@
//@ Description : Constructeur
//@
//@ Parameters :
//@     Aucun
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_Contexte::SD_DON_Contexte() 
  : m_NomRepertoire(""), m_NomFichier(""), m_NumGranule(-1), 
    m_Charge(false), m_DataContexte(0)
{
  //@ Initialisation des attributs
}


//@ ------------------------------------------------------------------*/
//@ Methode : InitialiserMutex
//@
//@ Description : 
//@
//@ Parameters :
//@     Aucun
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_Contexte::InitialiserMutex() 
{
  //@ Creer la variable conditionnelle POSIX correspondante et son mutex
  pthread_mutex_init(&m_MutexTravaille, NULL);
#ifdef OPS_DEBUG
          cout << "InitialiserMutex contexte" << endl;
#endif
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_DON_Contexte
//@
//@ Description : Destructeur
//@
//@ Parameters :
//@     Aucun
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_Contexte::~SD_DON_Contexte()
{
  // Liberation si besoin des donnees de type pointeurs
#ifdef OPS_DEBUG
          cout << "Detruire contexte" << endl;
#endif
}


//@ ------------------------------------------------------------------*/
//@ Methode : LibererMutex
//@
//@ Description : 
//@
//@ Parameters :
//@     Aucun
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_Contexte::LibererMutex()
{
  // Liberation si besoin des donnees de type pointeurs
  //@ Supprimer le mutex et la variable conditionnele POSIX
#ifdef OPS_DEBUG
          cout << "Liberer contexte" << endl;
#endif
  pthread_mutex_destroy(&m_MutexTravaille);
  //@ SI il y a une valeur dans le champ m_DataContexte
  if (m_DataContexte)
    {
      //@ On la libere
      delete m_DataContexte;
      m_DataContexte = 0;
    }
  //@ FIN SI

}


//@ ------------------------------------------------------------------*/
//@ Methode : Recopie
//@
//@ Description : Cette methode permet de recopier un objet en 
//@ dupliquant les tableaux alloues
//@     (ENTREE)  contexte : Contexte a recopier
//@
//@ Parameters :   
//@     
//@
//@ Return Status :   e_TypeContexte  type issu du fichier
//@
//@ ------------------------------------------------------------------*/
void SD_DON_Contexte::Recopie( const SD_DON_Contexte & contexte)
{
  //Creer la variable conditionnelle POSIX correspondante et son mutex
  //pthread_mutex_init(&m_MutexTravaille,NULL);

  //@ SI il y a une valeur dans le champ m_DataContexte
  if (m_DataContexte)
    {
      //@ On la libere
      delete m_DataContexte;
      m_DataContexte = 0;
    }
  //@ FIN SI  

  //@ on recopie ici les attribut de l'objet
  m_NomRepertoire = contexte.m_NomRepertoire;
  m_NomFichier = contexte.m_NomFichier;
  m_NumGranule = contexte.m_NumGranule;
  m_Charge = contexte.m_Charge; 

  //@ SI le contexte est charge
  if(m_Charge)
    {
      //@ allocation de m_DataContexte
      m_DataContexte = new t_SD_BAS_Contexte;
      //@ recopie des elements de la structure t_SD_BAS_Contexte
      m_DataContexte->GCTXVersOPS.major = contexte.m_DataContexte->GCTXVersOPS.major;
      m_DataContexte->GCTXVersOPS.minor = contexte.m_DataContexte->GCTXVersOPS.minor;
      m_DataContexte->GCTXVersCTX.major = contexte.m_DataContexte->GCTXVersCTX.major;
      m_DataContexte->GCTXVersCTX.minor = contexte.m_DataContexte->GCTXVersCTX.minor;
      m_DataContexte->GCTXNatCTX = contexte.m_DataContexte->GCTXNatCTX;
      for(int i = 0; i < SD_BAS_NB_VAL * SD_BAS_IMLI * SD_BAS_IMCO; i++)
        {
          m_DataContexte->GIccRadCalSlopeImag[i] = contexte.m_DataContexte->GIccRadCalSlopeImag[i];
          m_DataContexte->GIccRadCalOffsetImag[i] = contexte.m_DataContexte->GIccRadCalOffsetImag[i];
          m_DataContexte->MDptVarImag[i] = contexte.m_DataContexte->MDptVarImag[i];
          
          /* FA169 */
          m_DataContexte->MDptIisInHomPix[i] = contexte.m_DataContexte->MDptIisInHomPix[i];
          
          m_DataContexte->MDptIisBadHealthPix[i] = contexte.m_DataContexte->MDptIisBadHealthPix[i];
        }
      m_DataContexte->GIaxNbElt = contexte.m_DataContexte->GIaxNbElt;
      m_DataContexte->GBBTNbElt = contexte.m_DataContexte->GBBTNbElt;
      for(int i = 0; i < SD_BAS_NB_VAL; i++)
        {
          m_DataContexte->GEPSDatIASI[i].day = contexte.m_DataContexte->GEPSDatIASI[i].day;
          m_DataContexte->GEPSDatIASI[i].millis = contexte.m_DataContexte->GEPSDatIASI[i].millis;
          m_DataContexte->BIMSBBT[i] = contexte.m_DataContexte->BIMSBBT[i];
          m_DataContexte->GCTXNumDerLig[i] = contexte.m_DataContexte->GCTXNumDerLig[i];
          m_DataContexte->PTSI[i] = contexte.m_DataContexte->PTSI[i];
          m_DataContexte->BIISEqualCountOld[i] = contexte.m_DataContexte->BIISEqualCountOld[i];
          m_DataContexte->GIacOffsetIISAVHRRLin[i] = contexte.m_DataContexte->GIacOffsetIISAVHRRLin[i];
          m_DataContexte->GIacOffsetIISAVHRRCol[i] = contexte.m_DataContexte->GIacOffsetIISAVHRRCol[i];
          m_DataContexte->GOPSDatIsrfemOff[i].day = contexte.m_DataContexte->GOPSDatIsrfemOff[i].day;
          m_DataContexte->GOPSDatIsrfemOff[i].millis = contexte.m_DataContexte->GOPSDatIsrfemOff[i].millis;
          m_DataContexte->GOPSDatBandMiss[i].day = contexte.m_DataContexte->GOPSDatBandMiss[i].day;
          m_DataContexte->GOPSDatBandMiss[i].millis = contexte.m_DataContexte->GOPSDatBandMiss[i].millis;
          m_DataContexte->GOPSDatBBTMiss[i].day = contexte.m_DataContexte->GOPSDatBBTMiss[i].day;
          m_DataContexte->GOPSDatBBTMiss[i].millis = contexte.m_DataContexte->GOPSDatBBTMiss[i].millis;
          m_DataContexte->GOPSDatImgEWMiss[i].day = contexte.m_DataContexte->GOPSDatImgEWMiss[i].day;
          m_DataContexte->GOPSDatImgEWMiss[i].millis = contexte.m_DataContexte->GOPSDatImgEWMiss[i].millis;
          m_DataContexte->GOPSDatImgBBMiss[i].day = contexte.m_DataContexte->GOPSDatImgBBMiss[i].day;
          m_DataContexte->GOPSDatImgBBMiss[i].millis = contexte.m_DataContexte->GOPSDatImgBBMiss[i].millis;
          m_DataContexte->GOPSDatImgCSMiss[i].day = contexte.m_DataContexte->GOPSDatImgCSMiss[i].day;
          m_DataContexte->GOPSDatImgCSMiss[i].millis = contexte.m_DataContexte->GOPSDatImgCSMiss[i].millis;
          m_DataContexte->GOPSDatRadAvhrrMiss[i].day = contexte.m_DataContexte->GOPSDatRadAvhrrMiss[i].day;
          m_DataContexte->GOPSDatRadAvhrrMiss[i].millis = contexte.m_DataContexte->GOPSDatRadAvhrrMiss[i].millis;
          m_DataContexte->GCTXNumGranule[i] = contexte.m_DataContexte->GCTXNumGranule[i];
        }  
      // historiques axe interf
      for(int i = 0; i < m_DataContexte->GIaxNbElt; i++)
        {
          for(int j = 0; j < SD_BAS_CCD; j++)
            {
              m_DataContexte->GIaxHistAxeInterf[i*SD_BAS_CCD+j].GiaxDateElt.day = contexte.m_DataContexte->GIaxHistAxeInterf[i*SD_BAS_CCD+j].GiaxDateElt.day;
              m_DataContexte->GIaxHistAxeInterf[i*SD_BAS_CCD+j].GiaxDateElt.millis = contexte.m_DataContexte->GIaxHistAxeInterf[i*SD_BAS_CCD+j].GiaxDateElt.millis;
              m_DataContexte->GIaxHistAxeInterf[i*SD_BAS_CCD+j].GiaxAxeYElt = contexte.m_DataContexte->GIaxHistAxeInterf[i*SD_BAS_CCD+j].GiaxAxeYElt;
              m_DataContexte->GIaxHistAxeInterf[i*SD_BAS_CCD+j].GiaxAxeZElt = contexte.m_DataContexte->GIaxHistAxeInterf[i*SD_BAS_CCD+j].GiaxAxeZElt;
              m_DataContexte->GIaxHistAxeInterf[i*SD_BAS_CCD+j].GiaxAxeQualElt = contexte.m_DataContexte->GIaxHistAxeInterf[i*SD_BAS_CCD+j].GiaxAxeQualElt;
              m_DataContexte->GIaxHistAxeInterf[i*SD_BAS_CCD+j].GiaxFlagAxeNonQual = contexte.m_DataContexte->GIaxHistAxeInterf[i*SD_BAS_CCD+j].GiaxFlagAxeNonQual;
            }
        }
      // 100_EXS
      for(int i = 0; i<SD_BAS_SPCT_COD; i++)
        {
          m_DataContexte->MExsSmin[i] = contexte.m_DataContexte->MExsSmin[i];
          m_DataContexte->MExsSmax[i] = contexte.m_DataContexte->MExsSmax[i];
        }
      // historique temp CN
      for(int i = 0; i < m_DataContexte->GBBTNbElt; i++)
        {
          m_DataContexte->GBBTHistTempCN[i].GBBTDateElt.day = contexte.m_DataContexte->GBBTHistTempCN[i].GBBTDateElt.day;
          m_DataContexte->GBBTHistTempCN[i].GBBTDateElt.millis = contexte.m_DataContexte->GBBTHistTempCN[i].GBBTDateElt.millis;
          m_DataContexte->GBBTHistTempCN[i].BIMSBBT = contexte.m_DataContexte->GBBTHistTempCN[i].BIMSBBT;
          m_DataContexte->GBBTHistTempCN[i].GBBTMissing = contexte.m_DataContexte->GBBTHistTempCN[i].GBBTMissing;
        }
    }
  //@ FIN SI le contexte est charge
}
//@ ------------------------------------------------------------------*/
//@ Methode : MajContexte
//@
//@ Description : Cette methode permet de mettre a jour un contexte  
//@ (mis a part les tableaux historiques)
//@     (ENTREE)  contexte : Contexte a recopier
//@
//@ Parameters :   
//@     
//@
//@ Return Status :   e_TypeContexte  type issu du fichier
//@
//@ ------------------------------------------------------------------*/
void SD_DON_Contexte::MajContexte( const t_SD_BAS_Contexte * contextecourant)
{
  //@ SI le contexte est deja charge
  if(m_Charge)
    {
      //@ recopie des elements de la structure t_SD_BAS_Contexte
      m_DataContexte->GCTXVersOPS.major = contextecourant->GCTXVersOPS.major;
      m_DataContexte->GCTXVersOPS.minor = contextecourant->GCTXVersOPS.minor;
      m_DataContexte->GCTXVersCTX.major = contextecourant->GCTXVersCTX.major;
      m_DataContexte->GCTXVersCTX.minor = contextecourant->GCTXVersCTX.minor;
      m_DataContexte->GCTXNatCTX = contextecourant->GCTXNatCTX;
      for(int i = 0; i < SD_BAS_NB_VAL * SD_BAS_IMLI * SD_BAS_IMCO; i++)
        {
          m_DataContexte->GIccRadCalSlopeImag[i] = contextecourant->GIccRadCalSlopeImag[i];
          m_DataContexte->GIccRadCalOffsetImag[i] = contextecourant->GIccRadCalOffsetImag[i];
          m_DataContexte->MDptVarImag[i] = contextecourant->MDptVarImag[i];
          
          /* FA169 */
          m_DataContexte->MDptIisInHomPix[i] = contextecourant->MDptIisInHomPix[i];
          
          m_DataContexte->MDptIisBadHealthPix[i] = contextecourant->MDptIisBadHealthPix[i];
        }
      for(int i = 0; i < SD_BAS_NB_VAL; i++)
        {
          m_DataContexte->GEPSDatIASI[i].day = contextecourant->GEPSDatIASI[i].day;
          m_DataContexte->GEPSDatIASI[i].millis = contextecourant->GEPSDatIASI[i].millis;
          m_DataContexte->BIMSBBT[i] = contextecourant->BIMSBBT[i];
          m_DataContexte->GCTXNumDerLig[i] = contextecourant->GCTXNumDerLig[i];
          m_DataContexte->PTSI[i] = contextecourant->PTSI[i];
          m_DataContexte->BIISEqualCountOld[i] = contextecourant->BIISEqualCountOld[i];
          m_DataContexte->GIacOffsetIISAVHRRLin[i] = contextecourant->GIacOffsetIISAVHRRLin[i];
          m_DataContexte->GIacOffsetIISAVHRRCol[i] = contextecourant->GIacOffsetIISAVHRRCol[i];
          m_DataContexte->GOPSDatIsrfemOff[i].day = contextecourant->GOPSDatIsrfemOff[i].day;
          m_DataContexte->GOPSDatIsrfemOff[i].millis = contextecourant->GOPSDatIsrfemOff[i].millis;
          m_DataContexte->GOPSDatBandMiss[i].day = contextecourant->GOPSDatBandMiss[i].day;
          m_DataContexte->GOPSDatBandMiss[i].millis = contextecourant->GOPSDatBandMiss[i].millis;
          m_DataContexte->GOPSDatBBTMiss[i].day = contextecourant->GOPSDatBBTMiss[i].day;
          m_DataContexte->GOPSDatBBTMiss[i].millis = contextecourant->GOPSDatBBTMiss[i].millis;
          m_DataContexte->GOPSDatImgEWMiss[i].day = contextecourant->GOPSDatImgEWMiss[i].day;
          m_DataContexte->GOPSDatImgEWMiss[i].millis = contextecourant->GOPSDatImgEWMiss[i].millis;
          m_DataContexte->GOPSDatImgBBMiss[i].day = contextecourant->GOPSDatImgBBMiss[i].day;
          m_DataContexte->GOPSDatImgBBMiss[i].millis = contextecourant->GOPSDatImgBBMiss[i].millis;
          m_DataContexte->GOPSDatImgCSMiss[i].day = contextecourant->GOPSDatImgCSMiss[i].day;
          m_DataContexte->GOPSDatImgCSMiss[i].millis = contextecourant->GOPSDatImgCSMiss[i].millis;
          m_DataContexte->GOPSDatRadAvhrrMiss[i].day = contextecourant->GOPSDatRadAvhrrMiss[i].day;
          m_DataContexte->GOPSDatRadAvhrrMiss[i].millis = contextecourant->GOPSDatRadAvhrrMiss[i].millis;
          m_DataContexte->GCTXNumGranule[i] = contextecourant->GCTXNumGranule[i];
        }  
      // 100_EXS
      for(int i = 0; i<SD_BAS_SPCT_COD; i++)
        {
          m_DataContexte->MExsSmin[i] = contextecourant->MExsSmin[i];
          m_DataContexte->MExsSmax[i] = contextecourant->MExsSmax[i];
        }
    }
  //@ FIN SI le contexte est charge
}
//@ ------------------------------------------------------------------*/
//@ Methode : AjouteElemHist
//@
//@ Description : Cette methode permet d'ajouter un element dans l'historique
//@ ( axe interf ou temperature ), selon que les elements d'entrees sont nuls ou pas
//@  
//@
//@
//@ Parameters :
//@     (ENTREE)  AxeInterf  : position de l'axe interf (tableau dimension SD_BAS_CCD)
//@     (ENTREE)  TempCN : temperature du corps noir
//@
//@ Return Status : 
//@      
//@
//@ ------------------------------------------------------------------*/
void SD_DON_Contexte::AjouteElemHist(
                                     t_SD_BAS_HistAxeInterf** AxeInterf,
                                     t_SD_BAS_HistTempCN* TempCN

                                     )

{
  int seuil = 0;
  int indice_insert = 0;
  int sortir = 0;
  // Entree dans la fonction
#ifdef OPS_DEBUG
  cout << "SD_DON_Contexte::AjouteElemHist" << endl;
  cout << "Nombre d'elements interf : " << m_DataContexte->GIaxNbElt << endl;
  cout << "Numero d'elements temper : " << m_DataContexte->GBBTNbElt << endl;
#endif  
  //@ SI le contexte a ete charge
  if(m_Charge)
    {
      //@ SI AxeInterf != NULL
      if(AxeInterf != NULL)
        {
#ifdef OPS_DEBUG
          cout << "Ajout element axe interf" << endl;
#endif
          //@ SI le nombre d'elements du tableau histo interf est egal 
          //@ a la capacite max du tableau
          if(m_DataContexte->GIaxNbElt == SD_BAS_NB_HIST_INTERF)
            {
              //@ seuil = m_DataContexte->GIaxNbElt - 1
              seuil = m_DataContexte->GIaxNbElt - 1;
            }
          //@ SINON
          else
            {
              //@ seuil = m_DataContexte->GIaxNbElt
              seuil= m_DataContexte->GIaxNbElt;
            }
          //@ FIN SI
          //@ On recopie le tableau  histo interf du contexte
          //@ en decallant d'un element jusqu'a une date plus recente
          //@ que la date de l'element a inserer
  
          for(int i = 0; i < seuil && sortir == 0; i++)
            {
              for(int j = 0; j < SD_BAS_CCD; j++)
                {
                  m_DataContexte->GIaxHistAxeInterf[(seuil - i)*SD_BAS_CCD+j].GiaxDateElt.day = m_DataContexte->GIaxHistAxeInterf[(seuil - i - 1)*SD_BAS_CCD+j].GiaxDateElt.day;
                  m_DataContexte->GIaxHistAxeInterf[(seuil - i)*SD_BAS_CCD+j].GiaxDateElt.millis = m_DataContexte->GIaxHistAxeInterf[(seuil - i - 1)*SD_BAS_CCD+j].GiaxDateElt.millis;
                  m_DataContexte->GIaxHistAxeInterf[(seuil - i)*SD_BAS_CCD+j].GiaxAxeYElt = m_DataContexte->GIaxHistAxeInterf[(seuil - i - 1)*SD_BAS_CCD+j].GiaxAxeYElt;
                  m_DataContexte->GIaxHistAxeInterf[(seuil - i)*SD_BAS_CCD+j].GiaxAxeZElt = m_DataContexte->GIaxHistAxeInterf[(seuil - i - 1)*SD_BAS_CCD+j].GiaxAxeZElt;
                  m_DataContexte->GIaxHistAxeInterf[(seuil - i)*SD_BAS_CCD+j].GiaxAxeQualElt = m_DataContexte->GIaxHistAxeInterf[(seuil - i - 1)*SD_BAS_CCD+j].GiaxAxeQualElt;
                  m_DataContexte->GIaxHistAxeInterf[(seuil - i)*SD_BAS_CCD+j].GiaxFlagAxeNonQual = m_DataContexte->GIaxHistAxeInterf[(seuil - i - 1)*SD_BAS_CCD+j].GiaxFlagAxeNonQual;
                }
              if((m_DataContexte->GIaxHistAxeInterf[(seuil - i - 1)*SD_BAS_CCD].GiaxDateElt.day > AxeInterf[0]->GiaxDateElt.day)
                 || ((m_DataContexte->GIaxHistAxeInterf[(seuil - i - 1)*SD_BAS_CCD].GiaxDateElt.day == AxeInterf[0]->GiaxDateElt.day)
                     &&(m_DataContexte->GIaxHistAxeInterf[(seuil - i - 1)*SD_BAS_CCD].GiaxDateElt.millis > AxeInterf[0]->GiaxDateElt.millis)))
                {
                  sortir = 1;
                  indice_insert = seuil - i;
                }
            }
          //@ on recopie l'argument a un emplacement fonction de la date de l'element
          //@ a inserer
          for(int i = 0; i < SD_BAS_CCD; i++)
            {
              m_DataContexte->GIaxHistAxeInterf[indice_insert*SD_BAS_CCD+i].GiaxDateElt.day = AxeInterf[i]->GiaxDateElt.day;
              m_DataContexte->GIaxHistAxeInterf[indice_insert*SD_BAS_CCD+i].GiaxDateElt.millis = AxeInterf[i]->GiaxDateElt.millis;
              m_DataContexte->GIaxHistAxeInterf[indice_insert*SD_BAS_CCD+i].GiaxAxeYElt = AxeInterf[i]->GiaxAxeYElt;
              m_DataContexte->GIaxHistAxeInterf[indice_insert*SD_BAS_CCD+i].GiaxAxeZElt = AxeInterf[i]->GiaxAxeZElt;
              m_DataContexte->GIaxHistAxeInterf[indice_insert*SD_BAS_CCD+i].GiaxAxeQualElt = AxeInterf[i]->GiaxAxeQualElt;
              m_DataContexte->GIaxHistAxeInterf[indice_insert*SD_BAS_CCD+i].GiaxFlagAxeNonQual= AxeInterf[i]->GiaxFlagAxeNonQual;
            }
          //@ on incremente le nombre d'elements du tableau
          m_DataContexte->GIaxNbElt = seuil + 1;
        }
      //@ FIN SI AxeInterf != NULL
      //@ SI TempCN != NULL
      if(TempCN != NULL)
        {
#ifdef OPS_DEBUG
          cout << "Ajout element temp CN" << endl;
#endif
          //@ SI le nombre d'elements du tableau temp CN est egal 
          //@ a la capacite max du tableau
          if(m_DataContexte->GBBTNbElt == SD_BAS_NB_HIST_BBT)
            {
              //@ seuil = m_DataContexte->GBBTNbElt - 1
              seuil = m_DataContexte->GBBTNbElt - 1;
            }
          //@ SINON
          else
            {
              //@ seuil = m_DataContexte->GBBTNbElt
              seuil= m_DataContexte->GBBTNbElt;
            }
          //@ FIN SI
          //@ On recopie le tableau temperature CN du contexte
          //@ en decallant d'un element jusqu'a une date plus recente
          //@ que la date de l'element a inserer
          indice_insert = 0;
          sortir = 0;
          for(int i = 0; i < seuil && sortir == 0; i++)
            {
              m_DataContexte->GBBTHistTempCN[seuil - i].GBBTDateElt.day = m_DataContexte->GBBTHistTempCN[seuil - i - 1].GBBTDateElt.day;
              m_DataContexte->GBBTHistTempCN[seuil - i].GBBTDateElt.millis = m_DataContexte->GBBTHistTempCN[seuil - i - 1].GBBTDateElt.millis;
              m_DataContexte->GBBTHistTempCN[seuil - i].BIMSBBT = m_DataContexte->GBBTHistTempCN[seuil - i - 1].BIMSBBT;
              m_DataContexte->GBBTHistTempCN[seuil - i].GBBTMissing = m_DataContexte->GBBTHistTempCN[seuil - i - 1].GBBTMissing;
              if((m_DataContexte->GBBTHistTempCN[seuil - i - 1].GBBTDateElt.day > TempCN->GBBTDateElt.day)
                 || ((m_DataContexte->GBBTHistTempCN[seuil - i - 1].GBBTDateElt.day == TempCN->GBBTDateElt.day)
                     &&(m_DataContexte->GBBTHistTempCN[seuil - i - 1].GBBTDateElt.millis > TempCN->GBBTDateElt.millis)))
                {
                  sortir = 1;
                  indice_insert = seuil - i;
                }
            }
          //@ on recopie l'argument a un emplacement fonction de la date de l'element
          //@ a inserer
          m_DataContexte->GBBTHistTempCN[indice_insert].GBBTDateElt.day = TempCN->GBBTDateElt.day;
          m_DataContexte->GBBTHistTempCN[indice_insert].GBBTDateElt.millis = TempCN->GBBTDateElt.millis;
          m_DataContexte->GBBTHistTempCN[indice_insert].BIMSBBT = TempCN->BIMSBBT;
          m_DataContexte->GBBTHistTempCN[indice_insert].GBBTMissing = TempCN->GBBTMissing;
          //@ on incremente le nombre d'elements du tableau
          m_DataContexte->GBBTNbElt = seuil + 1;
        }
      //@ FIN SI TempCN != NULL
    }
  //@ FIN SI
  // on trace ici si besoin toutes les variables en sortie
  // sortie de la fonction 
}
//@ ------------------------------------------------------------------*/
//@ Methode : Reserver
//@
//@ Description : Cette methode permet de reserver la donnee pour sa mise 
//@ a jour ( cette fonction est bloquante si une autre reservation est en cours
//@  
//@
//@
//@ Parameters : aucun
//@
//@ Return Status : aucun
//@
//@ ------------------------------------------------------------------*/
void SD_DON_Contexte::Reserver()

{
  // Entree dans la fonction 
  //@ reservation par mutex
  pthread_mutex_lock(&m_MutexTravaille);
  // on trace ici si besoin toutes les variables en sortie

  // sortie de la fonction 

}

//@ ------------------------------------------------------------------*/
//@ Methode : Liberer
//@
//@ Description : Cette methode permet de liberer un contexte precedemment
//@ reserve ( cette fonction libere une autre reservation  en cours )
//@  
//@
//@
//@ Parameters : aucun
//@
//@ Return Status : aucun
//@
//@ ------------------------------------------------------------------*/
void SD_DON_Contexte::Liberer()

{
  // Entree dans la fonction 
  //@ liberation du  mutex
  pthread_mutex_unlock(&m_MutexTravaille);
  // on trace ici si besoin toutes les variables en sortie

  // sortie de la fonction 

}

//@ ------------------------------------------------------------------*/
//@ Methode : Lire
//@
//@ Description : Cette methode permet de lire les informations
//@ stockees dans le fichier de contexte on realise ici egalement
//@ les controles de coherence suivants :
//@  - integrite du contexte
//@  - compatibilite de la version du format avec la couche d'acces
//@  - compatibilite de la version OPS courante avec la version utilisee
//@    pour la generation ( on verifie que VOPS >= Vfichier)
//@  
//@
//@
//@ Parameters :
//@     (ENTREE)  Repertoire : Repertoire contenant le fichier
//@
//@ Return Status : 
//@
//@ ------------------------------------------------------------------*/
void SD_DON_Contexte::Lire(
                           const t_Chaine & repertoire
                           ) throw(ERR__Exception)

{
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  t_Chaine pattern("CTX");
  t_SD_BAS_DataThread* ctx;
  t_Chaine nomFichier;
  t_SD_BAS_NomFichier cheminFichier;
  int granuleMax = -1;
  // Entree dans la fonction 
  
  //@ on verifie que le contexte n'a pas deja ete chargee
  if(m_Charge == false)
    {
      //@ Recherche des fichiers
      Initialiser(repertoire, nomFichier, granuleMax);
      strcpy(cheminFichier, (repertoire + "/" + nomFichier).c_str());
      //@ SI on a trouve au moins un fichier contexte
      if(granuleMax > -1)
        {
          //@ on alloue une structure t_SD_BAS_Contexte
          m_DataContexte = new t_SD_BAS_Contexte;
          //@ on lit le fichier via la fonction SD_BAS_LireCtx de SD_BAS_Contexte
          ctx = new t_SD_BAS_DataThread;
          InitialiserCompteRendu (&(ctx->cr));

          cr_op = SD_BAS_LireCtx(cheminFichier, m_DataContexte, ctx);
          //@ SI erreur
          if(cr_op != CMN__STATUT_OK)
            {
              //@ emission des messages du ctx
              EnvoyerMessagesStockes(&(ctx->cr.msgJDB));
              //@ emission d'un message vers le JdBS
              JDB__Client MessageJDB;
              MessageJDB.SetParametre(JDB_PARAM_1, nomFichier);
              MessageJDB.EcrireMessage(SD_CONF_SET, SD__Message::MSG_CONF_LOAD_ERR);
              MessageJDB.Liberer();
              LibererCompteRendu(&(ctx->cr.msgJDB));
              delete ctx;
              //@ exception
              ERR__Exception exception;
    
              //@ Emission exception SD_DON_TMT_DATA_AUX
              exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_AUX);
              exception.SetParametre(DATA_AUX_CTX_READ_ERROR);
              THROW(exception);
            }
          //@ SINON
          else
            {
              //@ message JDB
              JDB__Client MessageJDB;
              MessageJDB.SetParametre(JDB_PARAM_1, nomFichier);
              MessageJDB.EcrireMessage(SD_CONF_SET, SD__Message::MSG_CONF_LOAD_OK);
              MessageJDB.Liberer();
              LibererCompteRendu(&(ctx->cr.msgJDB));
              delete ctx;
              //@ l'ensemble des  info est stockee dans les attributs de l'objet
              //@  contenu, nom, repertoire, numero du granule
              //@  type
	      //@ on recupere le numero du granule dans le contexte
              m_NumGranule = m_DataContexte->GCTXNumGranule[0];
              m_NomFichier = nomFichier;
              m_NomRepertoire = repertoire;
              //@ on realise ici un controle sur la version de l'OPS :
              //@ SI la version de l'OPS courante est < a la version de l'OPS
              //@ ayant genere le fichier
              if((m_DataContexte->GCTXVersOPS.major < SD_BAS_OPSVERSION.major)
                 || ((m_DataContexte->GCTXVersOPS.major == SD_BAS_OPSVERSION.major)
                     && (m_DataContexte->GCTXVersOPS.minor < SD_BAS_OPSVERSION.minor)))
                {
                  //emission d'un message vers le JdBS 
                  JDB__Client MessageJDB;
                  MessageJDB.SetParametre(JDB_PARAM_1, nomFichier);
                  MessageJDB.EcrireMessage(SD_CONF_SET, SD__Message::MSG_CONF_VERSION_ERROR);
                  MessageJDB.Liberer();
                  //@exception
                  ERR__Exception exception;
    
                  //@ Emission exception SD_DON_TMT_DATA_AUX
                  exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_AUX);
                  exception.SetParametre(DATA_AUX_CTX_VERSION_ERROR);
                  THROW(exception);
                }
              //@ FIN SI erreur version
            }
          //@ FIN SI 
        }
      //@ SINON (pas de fichier trouve)
      else
        {
          //@ message JDB
          JDB__Client MessageJDB;
          MessageJDB.SetParametre(JDB_PARAM_1, pattern);
          MessageJDB.EcrireMessage(SD_CONF_SET, SD__Message::MSG_CONF_NO_FILE);
          MessageJDB.Liberer();
          //@ exception
          ERR__Exception exception;
    
          //@ Emission exception SD_DON_TMT_DATA_AUX
          exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_AUX);
          exception.SetParametre(DATA_AUX_CTX_NOT_FOUND);
          THROW(exception);
        }
      //@ FIN SI on a trouve au moins un fichier contexte
      m_Charge = true;
    }
  //@ FIN SI
  // on trace ici si besoin toutes les variables en sortie
#ifdef OPS_DEBUG
  cout << endl << "SD_DON_Contexte::Lire" << endl;
  cout << "Nom du fichier lu : " << m_NomFichier.c_str() << endl;
  cout << "Numero de granule : " << m_NumGranule << endl;
  cout << "***************** INFOS CONTEXTE **************" << endl;
  cout << "PTSI : " << m_DataContexte->PTSI[0] << endl;
  cout << "Nb valeurs historique interf : " << m_DataContexte->GIaxNbElt << endl;
  cout << "Nb valeurs historique temp CN: " << m_DataContexte->GBBTNbElt << endl;
  cout << "***********************************************" << endl;
#endif  
  // sortie de la fonction 
}
//@ ------------------------------------------------------------------*/
//@ Methode : GetData
//@
//@ Description : Cette methode permet de recuperer l'adresse de la zone
//@    memoire ou est stockee le fichier de contexte 
//@
//@
//@ Parameters :   aucun
//@     
//@
//@ Return Status :   t_SD_BAS_Contexte pointeur sur la donnee
//@
//@ ------------------------------------------------------------------*/
t_SD_BAS_Contexte * SD_DON_Contexte::GetData(
    )

{
  // Entree dans la fonction 
  //@ return m_DataContexte
  return(m_DataContexte);
  // sortie de la fonction 

}

//@ ------------------------------------------------------------------*/
//@ Methode : GetPTSI
//@
//@ Description : Cette methode permet de recuperer le PTSI stocke dans
//@ le contexte
//@
//@
//@ Parameters :   
//@     
//@
//@ Return Status :   int  PTSI issu du fichier
//@
//@ ------------------------------------------------------------------*/
int  SD_DON_Contexte::GetPTSI(
    )

{
  int PTSITmp = -1;
  // Entree dans la fonction 

  //@ SI le contexte a  deja ete charge
  if(m_Charge)
    {
      //@  on charge m_DataContexte.PTSI[0]
      PTSITmp = m_DataContexte->PTSI[0];
    }
 fin:
  // sortie de la fonction 
  return(PTSITmp);
}

//@ ------------------------------------------------------------------*/
//@ Methode : Ecrire
//@
//@ Description : Cette methode permet d'ecrire les informations
//@ en memoire  dans le fichier de contexte 
//@  
//@
//@
//@ Parameters :
//@
//@     (SORTIE)  nomFichier : Nom du fichier contexte ecrit
//@
//@ Return Status : 
//@
//@ ------------------------------------------------------------------*/
void SD_DON_Contexte::Ecrire(
                             const t_Date& datefin,
                             const int& orbitestart,
                             const e_BAS_PositionTrt& slideid,
                             t_Chaine& nomFichier
                             ) throw(ERR__Exception)

{
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  t_SD_BAS_DataThread* ctx;
  t_SD_BAS_NomFichier cheminFichier;
  // Entree dans la fonction 

  //@ SI le contexte a deja ete charge
  if(m_Charge)
    {
      //@ SI slideid = FIRST
      if(slideid == FIRST)
        {
          //@ NumGranule = 0 
	  // pour le mode backlog 
          m_NumGranule = m_NumGranule + 1;
        }
      //@ SINON
      else
        {
          //@ NumGranule = NumGranule + 1
          m_NumGranule = m_NumGranule + 1;
        }
      // initialisation avant ecriture
      m_DataContexte->GCTXNumGranule[0]=m_NumGranule;
      m_DataContexte->GCTXNatCTX = INIT_NORMAL;
      //@ FIN SI
      t_OPSFile ficOrigine(m_NomFichier);
      t_OPSFile ficTmp;
      //@ initialisation d'un nom de fichier contexte
      ficTmp.Init(CMN__OPSFile::FILE_CTX);
      //@ def du numero de granule
//CR 233
//      ficTmp.SetIdent(m_NumGranule);
      //@ def du nom du spacecraft
      ficTmp.SetSpacecraft(ficOrigine.GetSpacecraft());
      //@ date de fin
      ficTmp.SetValidityStartDate(datefin);
      
      /* CR293 */
      //@ source
      t_Chaine nomSource("");
      nomSource = getenv(s_NOM_SOURCE.c_str());
      if ( !nomSource.empty() )
      {
        ficTmp.SetSource(nomSource);
      }
      else
      {
        ficTmp.SetSource(NOM_SOURCE_DEFAUT);
      }
      /* FIN CR293 */
      
      //@ numero d'orbite
      ficTmp.SetOrbit(orbitestart);
      //@ date de creation
      t_Date dateTmp;
      dateTmp.DateCourante();
      ficTmp.SetProcDate(dateTmp);
  
      nomFichier = ficTmp.GetNom();

      //@ on met a jour la version de l'OPS dans m_DataContexte
      m_DataContexte->GCTXVersOPS.major = SD_BAS_OPSVERSION.major;
      m_DataContexte->GCTXVersOPS.minor = SD_BAS_OPSVERSION.minor;
    
      //@ construction du nom complet du nouveau fichier
      t_Chaine ficContexte = CFG__Configuration::
        GetRepertoire("DATA_TMP_DIR") + CMN__SEP_REPERTOIRE + 
        ficTmp.GetNom();

#ifdef OPS_DEBUG
      cout << "Fichier contexte: " << ficContexte << endl;
#endif

      strcpy(cheminFichier, ficContexte.c_str());
    
      //@ on ecrit alors le fichier via le fonction SD_BAS_EcrireCtx de SD_BAS_Contexte
      ctx = new t_SD_BAS_DataThread;
      InitialiserCompteRendu (&(ctx->cr));

      cr_op = SD_BAS_EcrireCtx( cheminFichier,
                                m_DataContexte,
                                ctx);
      //@ SI erreur
      if(cr_op != CMN__STATUT_OK)
        {
          //@ emission des messages du ctx
          EnvoyerMessagesStockes(&(ctx->cr.msgJDB));
          //@ emission d'un message vers le JdBS
          JDB__Client MessageJDB;
          MessageJDB.SetParametre(JDB_PARAM_1, cheminFichier);
          MessageJDB.EcrireMessage(SD_CONF_SET, SD__Message::MSG_CONF_WRITE_KO);
          MessageJDB.Liberer();
          LibererCompteRendu(&(ctx->cr.msgJDB));
          delete ctx;
          //@ exception
          ERR__Exception exception;
    
          //@ Emission exception SD_DON_TMT_DATA_AUX
          exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_AUX);
          exception.SetParametre(DATA_AUX_CTX_WRITE_ERROR);
          THROW(exception);     
        }
      //@ FIN SI
      //@ emission d'un message vers le JdBS
      JDB__Client MessageJDB;
      MessageJDB.SetParametre(JDB_PARAM_1, cheminFichier);
      MessageJDB.EcrireMessage(SD_CONF_SET, SD__Message::MSG_CONF_WRITE_OK);
      MessageJDB.Liberer();
      LibererCompteRendu(&(ctx->cr.msgJDB));
      delete ctx;
    }
  //@ FIN SI le contexte a deja ete charge
  // on trace ici si besoin toutes les variables en sortie
#ifdef OPS_DEBUG
  cout << endl << "SD_DON_Contexte::Ecrire" << endl;
  cout << "Nom du fichier ecrit : " << cheminFichier << endl;
  cout << "Numero de granule : " << m_NumGranule << endl;
#endif  
  // sortie de la fonction  

}

//======================================================================
//          METHODES PRIVEES 
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Methode : Initialise
//@
//@ Description : Cette methode permet d'initialiser l'acces aux fichiers
//@               de contexte
//@
//@
//@ Parameters :
//@     (ENTREE)  repertoire : repertoire de scrutation
//@     (ENTREE)  nomFichier : nom du fichier
//@     (SORTIE)  granuleMax : numero de granule du fichier charge
//@
//@ Return Status :    
//@     aucune 
//@
//@ ------------------------------------------------------------------*/
void SD_DON_Contexte::Initialiser(const t_Chaine & repertoire,
                                  t_Chaine &nomFichier,
                                  int& granuleMax
                                  ) throw(ERR__Exception)
{
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  int granuleTmp = -1;
  // CR233 modification de la nomenclature du contexte
  t_Date datemax;
  t_Chaine pattern("CTX");
  CMN__ListeInfoFichier ListeInfoFichier;
  CMN__ListeInfoFichier::iterator It;
  //@ on recupere dans une liste l'ensemble des fichiers de type CTX presents
  cr_op = CMN__TrouverFichiersEx(repertoire, pattern, ListeInfoFichier, NULL, NULL, NULL, 1, 0);
  //@ SI erreur
  if(cr_op != CMN__STATUT_OK)
    {
      //@ message JDB
      JDB__Client MessageJDB;
      MessageJDB.SetParametre(JDB_PARAM_1, pattern);
      MessageJDB.EcrireMessage(SD_CONF_SET, SD__Message::MSG_CONF_NO_FILE);
      MessageJDB.Liberer();
      //@ exception
      ERR__Exception exception;
  
      //@ Emission exception SD_DON_TMT_DATA_AUX
      exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_AUX);
      exception.SetParametre(DATA_AUX_CTX_NOT_FOUND);
      THROW(exception);
    }
  //@ FIN SI
  //@ POUR chaque element de la liste
  for (It = ListeInfoFichier.begin();
       It != ListeInfoFichier.end();
       It ++)
    {
      //@ on construit un objet t_OPSFile
      t_OPSFile fichierTmp(It->Name());
  // CR233 modification de la nomenclature du contexte
      t_Date datetmp;
      datetmp=fichierTmp.GetValidityStartDate();
       //@ on enregistre le numero de granule (fictif)
      granuleTmp = 1;
  // CR233 modification de la nomenclature du contexte
      //@ SI la date de fin de granule est > a la datemax
      if(datetmp > datemax)
        {
          //@ on remplace la valeur de la date max 
          datemax = datetmp;
          //@ on remplace la valeur du granule max
          granuleMax = granuleTmp;
          //@ on enregistre le nom du fichier
          nomFichier = It->Name();
        }
      //@ FIN SI
    }
  //@ FIN POUR
}
