//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_Contexte.h,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_Contexte
//@
//@ Comment :
//@    this module implement the SD_DON_Contexte class
//@    used to reference the OPS Context File
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 12/12/2002 : P.GASSIE
//@ --------------------------------------------------------------------
//@ DM-ID :IA-2100-CR-293-CNE : 08/03/2006 : B.GUENDE
//@ The organism that creates CTX files is defined by the environment variable
//@ CONTEXT_SOURCE stored in SD.env. See IA-2100-CR-293-THA.
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_DON_Contexte_h
#define SD_DON_Contexte_h


// Inserer ici les fichiers include 
#include <pthread.h>
#include "SD_BAS_Global.h"
#include "SD_BAS_CompteRendu.h"
#include "SD_BAS_Contexte.h"
#include "ERR__Exception.h"
#include "CMN__Chaine.h"
#include "CMN__Date.h"
#include "CMN__Fichier.h"
#include "SD__Message.h"


// Definition des exceptions

class SD_DON_Contexte {

/*====================================================================*/

public:

        //
        // Type
        //	
	typedef enum {
	  INIT_A_FROID =  1,
	  INIT_NORMALE = 2
	} e_TypeContexte;

        //
        // Constant
        //
	//
	// Constructeur
	//
	SD_DON_Contexte();
	//
	// Destructeur
	//
	virtual ~SD_DON_Contexte();

	//
	// Accesseurs
	//

	t_SD_BAS_Contexte * GetData();
	int  GetPTSI();
		
	//
	// publics Methods 
	//
	void InitialiserMutex();
	void LibererMutex();
	void Recopie( 
	  const SD_DON_Contexte &
	  );
	void MajContexte(
	  const t_SD_BAS_Contexte *
	  );
	void Lire(
		const t_Chaine & repertoire
		) throw(ERR__Exception);
	void Ecrire(
	const t_Date& datefin,
	const int& orbitestart,
	const e_BAS_PositionTrt& slideid,
        t_Chaine& nomFichier	
		) throw(ERR__Exception);
	void AjouteElemHist(
		  t_SD_BAS_HistAxeInterf** AxeInterf,
		  t_SD_BAS_HistTempCN* TempCN
		);
	void Reserver();
	void Liberer();

	/*====================================================================*/
protected:
	//
	// protected Methods 
	//

	//
	// protected Attributes 
	//

	/*====================================================================*/
private:
	//
	// Private Methods 
	//
	void Initialiser(const t_Chaine & repertoire,
                                  t_Chaine &nomFichier,
                                  int& granuleMax
                                 ) throw(ERR__Exception);
	//
	// Private Attributes 
	//
	//
	// Constantes
	//
	static const t_Chaine JDB_PARAM_1;
	static const t_Chaine DATA_AUX_CTX_NOT_FOUND;
	static const t_Chaine DATA_AUX_CTX_READ_ERROR;
	static const t_Chaine DATA_AUX_CTX_VERSION_ERROR;
	static const t_Chaine DATA_AUX_CTX_WRITE_ERROR;

 /* CR293 */
 static const t_Chaine s_NOM_SOURCE;
 static const t_Chaine NOM_SOURCE_DEFAUT;
 /* FIN CR293 */
 
	// nom du repertoire de stockage 
	t_Chaine m_NomRepertoire;
	
	// nom du contexte courant 
	t_Chaine m_NomFichier;

	// numero du granule courant 
	int m_NumGranule;
	
	// indicateur de chargement
	bool m_Charge;
		
	//
	// structure contenant les informations du contexte
	// 
	t_SD_BAS_Contexte* m_DataContexte;
		
  	// Mutex pour la mise a jour du contexte 
  	static pthread_mutex_t   m_MutexTravaille;
	
};


#endif
