static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_DonGenGranule.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_DonGenGranule
//@ 
//@ Comment :
//@    this module implement the SD_DON_DonGenGranule class
//@    used to reference the general data used by OPS L1 chain
//@           
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 11/02/2003 : P.GASSIE
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-169-CN : 27/02/2006 : B.MASSART
//@ Syntax correction for IisInHomPix. See IA-2100-FC-169-THA.
//@ --------------------------------------------------------------------
//@ PM-ID :IA-2100-CR-301-CN : 03/07/2006 : B. MASSART
//@ PTSI is now read in the first PX valid record in N0 product to select the 
//@ BRD file. Before CR301, PTSI was read in the first CTX file where its 
//@ value is 0. (patch3-5-5). See IA-2100-CR-301-THA.
//@ --------------------------------------------------------------------
//@ MODIFIED FOR PATCH3-5-5 : 05/Jul/06
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-184-CN : 20/09/2006 : C.MERIGUET
//@ Suite passage purify delete[] remplace delete pour les tableaux
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-162-CN : 09/2006 : C.MERIGUET
//@ Gestion des IPR en mode DUMP
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-208-CN : 02/03/2007 : C.MERIGUET
//@ La fenetre spectrale est selectionnee en convention C(N-1)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <iostream>
#include "SD_DON_DonGenGranule.h"

//
// Definition des constantes
//
const t_Chaine SD_DON_DonGenGranule::SD_DON_GEN_MUTEX_METOP_MSG = "Error on mutex creation for METOP library";

//======================================================================
//           METHODES PUBLIQUES 
//======================================================================


//@ ------------------------------------------------------------------*/
//@ Methode : SD_DON_DonGenGranule
//@
//@ Description : Constructeur
//@
//@ Parameters :
//@     Aucun
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/

SD_DON_DonGenGranule::SD_DON_DonGenGranule()
{
  m_Ptsi = -1;
  m_Init = false;
  m_DataGen = NULL;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_DON_DonGenGranule
//@
//@ Description : Destructeur
//@
//@
//@ Parameters :
//@     Aucun
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/

SD_DON_DonGenGranule::~SD_DON_DonGenGranule()
{

// Liberation si besoin des donnees ( pointeurs dans m_DataGen)

  //@ Liberation de la memoire allouee a la structure t_SD_BAS_DataGen
  if(m_DataGen)
  {
    delete [] m_DataGen->WnShift;
    delete [] m_DataGen->FlagNomQual;
    //delete [] m_DataGen->WnFcsTab;
    delete [] m_DataGen->CosDephasagePRODUIT;
    delete [] m_DataGen->SinDephasagePRODUIT;
    delete [] m_DataGen->CosDephasageISRFEM;
    delete [] m_DataGen->SinDephasageISRFEM;
    delete [] m_DataGen->GFtbFilteredBBT;
    delete m_DataGen;
    m_DataGen = 0;
  }
  //@ appel SD_BAS_DetruireOperateur pour detruire le mutex pour la librairie
  //@ metop
  SD_BAS_DetruireOperateur();
  //@ appel LibererMutexpour detruire le mutex de reservation du contexte
  m_ContexteCourant.LibererMutex(); 
}


//@ ------------------------------------------------------------------*/
//@ Methode : InitLecture
//@
//@ Description : Cette methode permet de lire les informations
//@ stockees dans le fichier de contexte et recupere le PTSI qui y est stocke
//@  
//@
//@
//@ Parameters :
//@     (ENTREE)  Trt : objet traitement
//@     (SORTIE)  Ptsi : PTSI contenu dans le fichier
//@
//@ Return Status : 
//@     aucune 
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_DON_DonGenGranule::InitLecture(
                                                const SD_DON_Traitement& Trt,
                                                int& Ptsi)
{

  static const char   tcProcName_F[] = "SD_DON_DonGenGranule::InitLecture";
  t_CMN__Statut       cr_op = CMN__STATUT_OK;
  t_Chaine            RepDataCtx("");
  int                 PtsiCTX = 0; /* CR301 */
  int                 FirstPtsiN0 = 0; /* CR301 */
  
  // Entree dans la fonction 

  //@ enregistrement de l'objet traitement
  m_Traitement = Trt;

  //@ on recupere ici la valeur de la variable d'environnement 
  //@ SD_DON_VAR_ENV_CTX_DATA
  RepDataCtx = CFG__Configuration::GetRepertoire(SD_DON_InventaireConf::SD_DON_VAR_ENV_CTX_DATA);

  //@ on initialise ici le contexte par SD_DON_Contexte::Lire
  m_ContexteInit.Lire(RepDataCtx);

  //@ on recupere  le PTSI par SD_DON_Contexte::GetPTSI et on le stocke
  Ptsi = m_ContexteInit.GetPTSI();
  
  /* CR301 */
  PtsiCTX = Ptsi;
  if (CMN__StageTM::L1a == m_Traitement.TypeTraitement())
  {
//    FirstPtsiN0 = m_Traitement.GetFirstPtsiN0();
/* FA 162 */
    FirstPtsiN0 =
    SD_BAS_GetFirstPtsiN0(m_Traitement.ListeProduitE().front().c_str(),
                          m_Traitement.ModeTraitement());
/* ..FA 162 */
    Ptsi = FirstPtsiN0;
  }
  /* FIN CR301 */
#ifdef OPS_DEBUG
  cout << "PtsiCTX     : " << PtsiCTX << endl;
  cout << "FirstPtsiN0 : " << FirstPtsiN0 << endl;
#endif

  //@ on recupere les donnees du contexte par SD_DON_Contexte::GetData
  //@ on recopie la donnee t_SD_BAS_Contexte dans l'attribut m_ContexteCourant
  //@ SD_DON_CONTEXTE::contructeur de copie  profonde 
  m_ContexteCourant.Recopie(m_ContexteInit);
  
  //@ appel InitialiserMutex pour initialiser le mutex de reservation du contexte
  m_ContexteCourant.InitialiserMutex(); 

  //@ on suppose ici que les autres donnees que les histogrammes stockees dans
  //@ m_DataGen sont allouees statiquement si possible
//FAJLP on ne realloue datagen que pour le premier granule (cas ou plusieurs first existent
  if (m_Init == false)
        {
         m_DataGen = new t_SD_BAS_DataGen;
	 }
//  m_DataGen = new t_SD_BAS_DataGen;
  
// CR 232 mise en place de l'environnement pour les METOP CFI
  t_SD_BAS_DataThread* ctx;
  //@ appel SD_BAS_InitialiserOperateur pour creer le mutex pour la librairie
  //@ metop et initialiser l'environnement d'utilisation
 //@ on reinitialise l'acces aux traces par une fonction de SD_BAS_Traces
  ctx = new t_SD_BAS_DataThread;
  InitialiserCompteRendu (&(ctx->cr));
  cr_op=SD_BAS_InitialiserOperateur(ctx);
    //@ SI erreur
  if(cr_op != CMN__STATUT_OK)
    {
      // Exception generee
      ERR__Exception       exception;
    
      // Message pour l'exception
      t_ChaineFixe         message;


      //@ Recuperation du message d'erreur genere par 
      //@ les couches basses
      RecupererDernierMessageStocke(&(ctx->cr.msgJDB), message);
    
      //@ Emission des messages du ctx
      EnvoyerMessagesStockes(&(ctx->cr.msgJDB));
      LibererCompteRendu(&(ctx->cr.msgJDB));
      delete ctx;
  
      //@ Emission exception SD_DON_TRACE_INIT_FAILED
      exception.SetErrorCode(ERR__Exception::SD_ENV_UNDEFINED);
      exception.SetParametre(t_Chaine(message));
      THROW(exception);     
    }
  //@ FIN SI

  LibererCompteRendu(&(ctx->cr.msgJDB));
  delete ctx;
 Fin:
  // sortie de la fonction 
  return(cr_op); 
}


//@ ------------------------------------------------------------------*/
//@ Methode : InitCalcul
//@
//@ Description : Cette methode permet d'initialiser les calculs necessaires
//@ realises globalement
//@  
//@
//@
//@ Parameters :
//@     (ENTREE)  Inventaire : Inventaire des donnees de configuration
//@     (ENTREE)  Ptsi : Ptsi issu de la ligne
//@     (ENTREE)  dateDebutGranule : date de debut de granule
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_DON_DonGenGranule::InitCalcul(
    SD_DON_InventaireConf&  inventaire,
    const int ptsi,
    const t_Date& dateDebutGranule
    ) throw(ERR__Exception)
{
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  t_SD_BAS_DataThread* ctx;
  t_Date dateTmp(dateDebutGranule);
  t_SD_BAS_Contexte * context;
  t_SD_BAS_DataGen* DataGen;


  // Entree dans la fonction 

  //@ on reinitialise l'acces aux traces par une fonction de SD_BAS_Traces
  ctx = new t_SD_BAS_DataThread;
  InitialiserCompteRendu (&(ctx->cr));

  cr_op = SD_BAS_InitTraces(dateTmp.SCDSTime(), 
                            (m_ContexteCourant.GetData())->GCTXNumGranule[0], 
                            m_ContexteCourant.GetData(), ctx); 
  //@ SI erreur
  if(cr_op != CMN__STATUT_OK)
    {
      // Exception generee
      ERR__Exception       exception;
    
      // Message pour l'exception
      t_ChaineFixe         message;


      //@ Recuperation du message d'erreur genere par 
      //@ les couches basses
      RecupererDernierMessageStocke(&(ctx->cr.msgJDB), message);
    
      //@ Emission des messages du ctx
      EnvoyerMessagesStockes(&(ctx->cr.msgJDB));
      LibererCompteRendu(&(ctx->cr.msgJDB));
      delete ctx;
  
      //@ Emission exception SD_DON_TRACE_INIT_FAILED
      exception.SetErrorCode(ERR__Exception::SD_DON_TRACE_INIT_FAILED);
      exception.SetParametre(t_Chaine(message));
      THROW(exception);     
    }
  //@ FIN SI

  LibererCompteRendu(&(ctx->cr.msgJDB));
  delete ctx;


// jlp3 ici le contexte doit toujours etre mis a jour
      //@ le contexte est ici deja charge on fait juste :
      //@ on recupere  les donnees du contexte courant dans l'attribut   
      //@ m_ContexteCourant.getData
      context = m_ContexteCourant.GetData();


      //@ SI mode complet on reprend les derniere valeurs globales ( PTSI..) 
      //@   et on les recopie dans les avant dernieres ( pour init du contexte)
      if (m_Traitement.TypeTraitement() == CMN__StageTM::L1a)
        {
      //@ on incremente le numero de granule de 1 
          context->GCTXNumGranule[1]=context->GCTXNumGranule[0];
          context->GEPSDatIASI[1] = context->GEPSDatIASI[0];
          context->BIMSBBT[1] = context->BIMSBBT[0];
          context->GCTXNumDerLig[1] = context->GCTXNumDerLig[0];
          context-> PTSI[1] = context-> PTSI[0];
          context->BIISEqualCountOld[1] = context->BIISEqualCountOld[0];
          context->GIacOffsetIISAVHRRLin[1] = context->GIacOffsetIISAVHRRLin[0];
          context->GIacOffsetIISAVHRRCol[1] = context->GIacOffsetIISAVHRRCol[0];
          context->GOPSDatIsrfemOff[1] = context->GOPSDatIsrfemOff[0];
          context->GOPSDatBandMiss[1] = context->GOPSDatBandMiss[0];
          context->GOPSDatBBTMiss[1] = context->GOPSDatBBTMiss[0];
          context->GOPSDatImgEWMiss[1] = context->GOPSDatImgEWMiss[0];
          context->GOPSDatImgBBMiss[1] = context->GOPSDatImgBBMiss[0];
          context->GOPSDatImgCSMiss[1] = context->GOPSDatImgCSMiss[0];
          context->GOPSDatRadAvhrrMiss[1] = context->GOPSDatRadAvhrrMiss[0];
          for(int i = 0; i < SD_BAS_IMLI * SD_BAS_IMCO; i++)
            {
              context->GIccRadCalSlopeImag[SD_BAS_IMLI * SD_BAS_IMCO + i] = context->GIccRadCalSlopeImag[i];
              context->GIccRadCalOffsetImag[SD_BAS_IMLI * SD_BAS_IMCO + i] = context->GIccRadCalOffsetImag[i];
              context->MDptVarImag[SD_BAS_IMLI * SD_BAS_IMCO + i] = context->MDptVarImag[i];
              
              /* FA169 */
              context->MDptIisInHomPix[SD_BAS_IMLI * SD_BAS_IMCO + i] = context->MDptIisInHomPix[i];
              
              context->MDptIisBadHealthPix[SD_BAS_IMLI * SD_BAS_IMCO + i] = context->MDptIisBadHealthPix[i];
            }
        }
      //@ SINON : mode intermediaires  on reprend les avant derniere valeurs globales 
      //@   ( PTSI..) et on les recopie dans les  dernieres ( de toute facon le 
      //@   contexte n'est pas ecrit en sortie)
      else
        {
          context->GCTXNumGranule[0]=context->GCTXNumGranule[1];
          context->GEPSDatIASI[0] = context->GEPSDatIASI[1];
          context->BIMSBBT[0] = context->BIMSBBT[1];
          context->GCTXNumDerLig[0] = context->GCTXNumDerLig[1];
          context-> PTSI[0] = context-> PTSI[1];
          context->BIISEqualCountOld[0] = context->BIISEqualCountOld[1];
          context->GIacOffsetIISAVHRRLin[0] = context->GIacOffsetIISAVHRRLin[1];
          context->GIacOffsetIISAVHRRCol[0] = context->GIacOffsetIISAVHRRCol[1];
          context->GOPSDatIsrfemOff[0] = context->GOPSDatIsrfemOff[1];
          context->GOPSDatBandMiss[0] = context->GOPSDatBandMiss[1];
          context->GOPSDatBBTMiss[0] = context->GOPSDatBBTMiss[1];
          context->GOPSDatImgEWMiss[0] = context->GOPSDatImgEWMiss[1];
          context->GOPSDatImgBBMiss[0] = context->GOPSDatImgBBMiss[1];
          context->GOPSDatImgCSMiss[0] = context->GOPSDatImgCSMiss[1];
          context->GOPSDatRadAvhrrMiss[0] = context->GOPSDatRadAvhrrMiss[1];
          
          for(int i = 0; i < SD_BAS_IMLI * SD_BAS_IMCO; i++)
            {
              context->GIccRadCalSlopeImag[i] = context->GIccRadCalSlopeImag[SD_BAS_IMLI * SD_BAS_IMCO + i];
              context->GIccRadCalOffsetImag[i] = context->GIccRadCalOffsetImag[SD_BAS_IMLI * SD_BAS_IMCO + i];
              context->MDptVarImag[i] = context->MDptVarImag[SD_BAS_IMLI * SD_BAS_IMCO + i];
              
              /* FA169 */
              context->MDptIisInHomPix[i] = context->MDptIisInHomPix[SD_BAS_IMLI * SD_BAS_IMCO + i];
              
              context->MDptIisBadHealthPix[i] = context->MDptIisBadHealthPix[SD_BAS_IMLI * SD_BAS_IMCO + i];
            }
        }
      //@ FIN SI
    

      //@ les donnees de m_DataGen hors contexte ne peuvent etre initialise qu'une fois par granule
      //@ SI m_Init=TRUE
      if (m_Init == true)
        {
          //@ prevoir ici d'envoyer un message de trace ou au JdB
          //@ normalement meme si le PTSI change ceci ne doit pas avoir
          //@ d'influence sur les calculs de YY_CONF
        }
      //@ SINON 
      else
        {
          //@ on recupere  les donnees de configuration
          t_SD_BAS_ConfStable** ConfStable;
          ConfStable = new t_SD_BAS_ConfStable*[1];
          t_SD_BAS_ConfAutre** ConfAutre;
          ConfAutre = new t_SD_BAS_ConfAutre*[1];
          t_SD_BAS_BanqueSpectrale** BanqueSpectrale;
          BanqueSpectrale = new t_SD_BAS_BanqueSpectrale*[1];
          t_Date dateCourante;
          
          dateCourante.DateCourante();
          inventaire.GetConf(ptsi, dateCourante, ConfAutre, ConfStable, BanqueSpectrale);
          
          //@  on appelle alors YY_CONF ( methode privee)
#ifdef OPS_DEBUG
          cout << "---------- Appel YY_CONF -------------" << endl;
#endif
          YY_CONF(ConfAutre[0], ConfStable[0], BanqueSpectrale[0]);
#ifdef OPS_DEBUG
          cout << "-------- Fin appel YY_CONF -----------" << endl;
#endif
          
          //@ delete pointeurs
/* FA 184 */
          delete [] ConfStable;
          delete [] ConfAutre;
          delete [] BanqueSpectrale;      
/* ..FA 184 */          
          //@  m_init=TRUE
          m_Init = true;

          //@  m_PTSI=PtSI
          m_Ptsi = ptsi;
        }
      //@ FIN SI m_Init=TRUE

  // on trace ici si besoin toutes les variables en sortie


  // sortie de la fonction 
  return(cr_op); 
}


//@ ------------------------------------------------------------------*/
//@ Methode : FinCalcul
//@
//@ Description : Cette methode permet de terminer les calculs
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_DonGenGranule::FinCalcul()
{
  // Donnees de contexte
  t_SD_BAS_DataThread* ctx = NULL;


  //@ Allocation memoire pour les donnees de contexte
  ctx = new t_SD_BAS_DataThread;
  InitialiserCompteRendu(&(ctx->cr));
  
  //@ Fermeture de l'acces aux traces via la 
  //@ fonction SD_BAS_FinTraces()
  SD_BAS_FinTraces(ctx);

  //@ Liberation memoire pour les donnees de contexte
  LibererCompteRendu(&(ctx->cr.msgJDB));
  delete ctx;
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetContexte
//@
//@ Description : Cette methode permet de recuperer le contexte courant
//@
//@
//@ Parameters : 
//@     (SORTIE)  contexte courant
//@
//@ Return Status : 
//@
//@ ------------------------------------------------------------------*/
SD_DON_Contexte& SD_DON_DonGenGranule::GetContexte()
{
  return(m_ContexteCourant);
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetData
//@
//@ Description : Cette methode permet de recuperer l'adresse des 
//@ donnees generales au granules. Elles sont de deux types
//@   - donnees variable associees au contexte qui peuvent etre lues ( historiques)
//@      pour les listes les modif passent  par AjouteElemHist 
//@   - donnees de taille fixe associees au contexte qui peuvent etre modifiees
//@     les modif passent par MajContexte
//@   - donnees precalculees qui ne peuvent etre que lues ( hors BBT)
//@    
//@  
//@
//@
//@ Parameters :
//@     (ENTREE)  contextecourant :  contexte courant
//@     (SORTIE)  datagen : pointeur sur les donnees generales
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/

void SD_DON_DonGenGranule::GetData(
    t_SD_BAS_Contexte**    contexte_courant,
    t_SD_BAS_DataGen**     datagen
    )

{
//Entree dans la fonction
  
  //@ on retourne l'objet m_ContexteCourant.getData apres recopie dans 
  //@ contexte_courant
  contexte_courant[0] = m_ContexteCourant.GetData();
  
  //@ on retourne l'adresse de m_DataGen dans datagen
  datagen[0] = m_DataGen;

  // on trace ici si besoin toutes les variables en sortie
// sortie de la fonction  
}

//@ ------------------------------------------------------------------*/
//@ Methode : MajContexte  
//@
//@ Description : Cette methode permet de mettre a jour le contexte 
//@    
//@  
//@
//@
//@ Parameters :
//@     (ENTREE)  contextecourant : pointeur sur le contexte
//@           courant modifie
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/

void SD_DON_DonGenGranule::MajContexte(
    const t_SD_BAS_Contexte * contextecourant
    )

{
// Entree dans la fonction 
  //@ on reserve la donnee m_ContexteCourant
  m_ContexteCourant.Reserver();
  //@ on recopie la donnee contextecourant dans l'objet m_ContexteCourant  
  //@ copie hors tableau dynamiques d'historique 
  m_ContexteCourant.MajContexte(contextecourant);
  //@ on libere la donnee m_ContexteCourant
  m_ContexteCourant.Liberer();

// on trace ici si besoin toutes les variables en sortie
// sortie de la fonction 
}

//@ ------------------------------------------------------------------*/
//@ Methode : AjouteElemHist
//@
//@ Description : Cette methode permet d'ajouter un element dans l'historique
//@ ( axe interf et temperature )
//@  
//@
//@
//@ Parameters :
//@     (ENTREE)  AxeInterf  : position de l'axe interf dimension SD_BAS_CCD)
//@     (ENTREE)  TempCN : temperature du corps noir
//@   si un des element est NULL rien n'est fait
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/

void SD_DON_DonGenGranule::AjouteElemHist(
    t_SD_BAS_HistAxeInterf** AxeInterf,
    t_SD_BAS_HistTempCN*    TempCN)

{
// Entree dans la fonction 
  //@ on reserve la donnee m_ContexteCourant
  m_ContexteCourant.Reserver();
  //@ On ajoute les elements
  m_ContexteCourant.AjouteElemHist(AxeInterf, TempCN);
  //@ on libere la donnee m_ContexteCourant
  m_ContexteCourant.Liberer();
// on trace ici si besoin toutes les variables en sortie

// sortie de la fonction  

}


//@ ------------------------------------------------------------------*/
//@ Methode : SauverContexte
//@
//@ Description : Cette methode permet de sauver dans un fichier 
//@ les informations stockees dans les donnee generales  
//@  
//@
//@
//@ Parameters :
//@     (ENTREE)  datefin : PTsi contenu dans le fichier
//@     (ENTREE)  source : Source du tt (MPHR)
//@     (ENTREE)  orbitestart : numero d'orbite issu du MPHR
//@     (ENTREE)  slideid : slide id issu du workorder
//@     (SORTIE)  nomFichier : Nom du fichier contexte ecrit
//@
//@ Return Status : 
//@     aucune 
//@
//@ ------------------------------------------------------------------*/

t_CMN__Statut SD_DON_DonGenGranule::SauverContexte(
  const t_Date&        datefin,
  const int&           orbitestart,
  const e_BAS_PositionTrt& slideid,
  t_Chaine& nomFichier)

{
t_CMN__Statut cr_op=CMN__STATUT_OK;
// Entree dans la fonction 
  //@ dans le cas du mode normal
  if(m_Traitement.TypeTraitement() == CMN__StageTM::L1a)
  {

    //@ on sauve la donnee m_ContexteCourant
    m_ContexteCourant.Ecrire(datefin, orbitestart, slideid, nomFichier);
  }
  //@ dans le cas du mode intermediaire on ne sauve rien 
// on trace ici si besoin toutes les variables en sortie

Fin:
// sortie de la fonction 
return(cr_op); 

}



//======================================================================
//          METHODES PRIVEES 
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : YY_CONF
//@
//@ Description : Cette methode permet d'initialiser les calculs necessaires
//@ realises globalement pour le granule
//@  
//@
//@
//@ Parameters :
//@     (ENTREE)  ConfAutre : pointeur sur le 
//@         fichier de conf autre
//@     (ENTREE)  ConfStable : pointeur sur le 
//@         fichier de conf stable
//@     (ENTREE)  BanqueSpectrale : pointeur sur le 
//@         fichier BanqueSpectrale
//@
//@ Return Status :    
//@     aucune 
//@
//@ ------------------------------------------------------------------*/
void SD_DON_DonGenGranule::YY_CONF (
    t_SD_BAS_ConfAutre *        ConfAutre,
    t_SD_BAS_ConfStable *       ConfStable,
    t_SD_BAS_BanqueSpectrale *  BanqueSpectrale)
{
  /* Variables locales */
  /*-------------------*/
  static const char tcProcName_F[] = "SD_DON_DonGenGranule::YY_CONF";
  int       NbScans = SD_BAS_SN;
  double *  DecalageSpectral = NULL;
  int *     FlagQualiteDecalage = NULL;

  int       NbEchantillonsPRODUIT = ConfAutre->IDefS1bNsFFT;
  int       EchantillonRefPRODUIT = NbEchantillonsPRODUIT / 2 + 1;
  int       SurechPRODUIT = ConfAutre->IDefS1bOSFftFactor;
  double *  DiffMarcheOptiquePRODUIT = NULL;
  double *  DephasagePRODUIT = NULL;
  double *  CosDephasagePRODUIT = NULL;
  double *  SinDephasagePRODUIT = NULL;

  int       IndiceFenetreSpectraleISRFEM = 
    ConfAutre->IDefSsdSpectralWindowOPS;
  //FA 208
  int       NbEchantillonsIRSFEM = 
    ConfAutre->IWnpNsFFT[IND1(IndiceFenetreSpectraleISRFEM)];
  int       SurechISRFEM = 
    ConfAutre->IDefSsdOSFftFactor[IND1(IndiceFenetreSpectraleISRFEM)];
  // FIN FA 208
  int       EchantillonRefISRFEM = NbEchantillonsIRSFEM / 2 + 1;
  double *  DiffMarcheOptiqueISRFEM = NULL;
  double *  DephasageISRFEM = NULL;
  double *  CosDephasageISRFEM = NULL;
  double *  SinDephasageISRFEM = NULL;

  double    PasSpectre = ConfAutre->IDefSpectrDWn;
  int       Echantillon = 0;
  int       IndiceSurech = 0;
  int       DimDecalageSpectral = 0;
  int       DimFlagQualiteDecalage = 0;
  int       DimDiffMarcheOptiquePRODUIT = 0;
  int       DimDephasagePRODUIT = 0;
  int       DimDiffMarcheOptiqueISRFEM = 0;
  int       DimDephasageISRFEM = 0;

  //Entree dans la fonction  
  //@ Allocation memoire
  m_DataGen->GFtbFilteredBBT=new double[SD_BAS_NB_HIST_BBT];

  DimDecalageSpectral = NbScans;
  DecalageSpectral = new double[DimDecalageSpectral];
  // initialisation effectuee plus bas

#ifdef OPS_DEBUG
  cout << "DimDecalageSpectral : "<< DimDecalageSpectral << endl;
#endif
    

  DimFlagQualiteDecalage = NbScans;
  FlagQualiteDecalage = new int[DimFlagQualiteDecalage];
  // initialisation effectuee plus bas

#ifdef OPS_DEBUG
  cout << "DimFlagQualiteDecalage : "<< DimFlagQualiteDecalage << endl;
#endif
  

  DimDiffMarcheOptiquePRODUIT = NbEchantillonsPRODUIT / 2 + 1;
  DiffMarcheOptiquePRODUIT = new double[DimDiffMarcheOptiquePRODUIT];
  memset(DiffMarcheOptiquePRODUIT, 0, sizeof(double)*DimDiffMarcheOptiquePRODUIT);

#ifdef OPS_DEBUG
  cout << "DimDiffMarcheOptiquePRODUIT : "<< DimDiffMarcheOptiquePRODUIT << endl;
#endif
  

  DimDephasagePRODUIT = SurechPRODUIT * NbEchantillonsPRODUIT;
  DephasagePRODUIT = new double[DimDephasagePRODUIT];
  CosDephasagePRODUIT = new double[DimDephasagePRODUIT];
  SinDephasagePRODUIT = new double[DimDephasagePRODUIT];
  memset(DephasagePRODUIT, 0, sizeof(double)*DimDephasagePRODUIT);

#ifdef OPS_DEBUG
  cout << "DimDephasagePRODUIT : "<< DimDephasagePRODUIT << endl;
#endif
  

  DimDiffMarcheOptiqueISRFEM = NbEchantillonsIRSFEM / 2 + 1;
  DiffMarcheOptiqueISRFEM = new double[DimDiffMarcheOptiqueISRFEM];
  memset (DiffMarcheOptiqueISRFEM, 0, sizeof(double)*DimDiffMarcheOptiqueISRFEM);

#ifdef OPS_DEBUG
  cout << "DimDiffMarcheOptiqueISRFEM : "<< DimDiffMarcheOptiqueISRFEM << endl;
#endif
  

  DimDephasageISRFEM = SurechISRFEM * NbEchantillonsIRSFEM;
  DephasageISRFEM = new double[DimDephasageISRFEM];
  CosDephasageISRFEM = new double[DimDephasageISRFEM];
  SinDephasageISRFEM = new double[DimDephasageISRFEM];
  memset (DephasageISRFEM, 0, sizeof(double)*DimDephasageISRFEM);

#ifdef OPS_DEBUG
  cout << "DimDephasageISRFEM : "<< DimDephasageISRFEM << endl;
#endif


  //@ Initialisation du decalage spectral pour la fenetre spectrale
  //@ Initialisation du flag de la qualite d'estimation du decalage spectral

  //@ on realise ici les calcul suivants ( cf p 61 du DA 3) et p 115 du DA2
  //@ 1 Initialise, pour tout le granule, WnShift a 0 et FlagNonQual a TRUE 
  //@   (de ce fait, si les algorithmes 22_SOS et 23_SSD ne sont pas 
  //@ executes, les flags seront positionnes pour l'execution de 21_SSS
  
  //@ Decalage spectral et flag de qualite de determination du decalage 
  //@ spectral
  for (int IndiceScan = 0; 
       IndiceScan < NbScans; 
       IndiceScan++)
    {
      DecalageSpectral[IndiceScan] = 0;
      FlagQualiteDecalage[IndiceScan] = CMN__VRAI;
    }
  
  //@************************************************************************
  
  //@ 3 Calcul de la base d'echantillonnage de la fonction de calibration 
  //@   spectrale pour n variant de NsFirstFcsTab a NsLastFcsTab  
  //@   WnFcsTab(n) = (n-1) . DWnFcsTab
  
  // deja mentionne dans SD_ALG_XX_Initprod !!!!!!!!  que choisir???
  // chercher l'algo qui contient l'equation et reflechir a la pertinence
  // de maintenir le calcul dans YY_CONF
  // Rq: Mathieu a code dans 22_SOS la base des echantillons spectraux

  //@************************************************************************
  //@ Calcul du dephasage des spectres precalcules pour l'algorithme de 
  //@ surechantillonnage par TF d'un facteur F entier (22_SOS)
  //@ pour n = 1..F, dphi(n) = 2.Pi.(n-1).x(m).dnu/F
  //@ x(m) est la difference de marche optique de l'echantillon m 
  //@ (m E [1..M] ou M est le nombre d'echantillons du spectre) dans 
  //@ l'interferogramme calcule par TF du spectre.
  //@ dnu est le pas d'echantillonnage du spectre
  //@ 22_SOS calcule F-1 interferogrammes dephases par rapport au premier de 
  //@ dphi
  //@ !!! ATTENTION !!!
  //@ on fait l'hypothese que le nombre d'echantillons des spectres est pair
  //@ (il s'agit des variables IWnpNsFFT[IDefSsdSpectralWindowOPS] pour la 
  //@ chaine ISRFEM et IDefS1bNsFFT pour la chaine PRODUIT
  //@***********************************************************************
  
  //@************************************************************************
  //@ Cas de la chaine PRODUIT
  //@ l'algorithme 22_SOS est appele pour reechantillonner les spectres 1A
  //@ (algo 35_S1B)
  //@************************************************************************
  
  //@ 4 DphiDT : dephasage des spectres precalcules pour 22_SOS de la chaine 
  //@   PRODUIT
  //@   Calcul des differences de marche optiques associees aux echantillons
  //@   mo = NsFFT/2 + 1
  //@   x(m) = (m - mo) .  1/ ( cf 22_SOS)
  //@   Boucle sur le facteur de surechantillonnage ns varie de 1 a 
  //@   OSFftFactor
  //@   Calcul du dephasage dphi(m) = 2.Pi . (ns-1) . x(m) .  ( cf 22_SOS)
  
  //@ Calcul des differences de marche optique pour chaque echantillon
  //@ Par symetrie, on ne calcule que la premiere moitie des coefficients
  for (Echantillon = 1; 
       Echantillon <= DimDiffMarcheOptiquePRODUIT; 
       Echantillon++)
    {
      DiffMarcheOptiquePRODUIT[IND1(Echantillon)] = 
        (IND1(Echantillon) - IND1(EchantillonRefPRODUIT)) / 
        (NbEchantillonsPRODUIT * PasSpectre);
    }
  
  //@ Initialisation du dephasage pour chaque difference de chemin optique
  int IndiceDephasage;

  //@ POUR chaque valeur de surech comprise entre 1 et le surech FAIRE
  for (IndiceSurech = 1; 
       IndiceSurech <= SurechPRODUIT; 
       IndiceSurech++)
    {
      //@ POUR chaque echantillon du spectre FAIRE
      for (Echantillon = 1; 
           Echantillon <= NbEchantillonsPRODUIT; 
           Echantillon++)
        {
          // Convention FORTRAN
          IndiceDephasage = 
            IND1(IndiceSurech) * NbEchantillonsPRODUIT + IND1(Echantillon) + 1;
      
          //@ Calculer le dephasage des echantillons precedant m0 (m0 inclus)
          if (IND1(Echantillon) <= IND1(EchantillonRefPRODUIT) )
            {
              DephasagePRODUIT[IND1(IndiceDephasage)] = 
                2 * CMN__PI * IND1(IndiceSurech) * 
                DiffMarcheOptiquePRODUIT[IND1(Echantillon)] * 
                PasSpectre / SurechPRODUIT;
            }
      
          //@ Calculer le dephasage des echantillons suivant m0 (m0 exclus)
          else
            {
              DephasagePRODUIT[IND1(IndiceDephasage)] = 
                - 2 * CMN__PI * IND1(IndiceSurech) * 
                DiffMarcheOptiquePRODUIT[NbEchantillonsPRODUIT - IND1(Echantillon)] * 
                PasSpectre / SurechPRODUIT;
            }
          CosDephasagePRODUIT[IND1(IndiceDephasage)] = cos(DephasagePRODUIT[IND1(IndiceDephasage)]);
          SinDephasagePRODUIT[IND1(IndiceDephasage)] = sin(DephasagePRODUIT[IND1(IndiceDephasage)]);
        }
      //@ FIN POUR chaque echantillon
    }
  //@ FIN POUR chaque valeur de surech
  
  
  //@************************************************************************
  //@ Cas de la chaine ISRFEM
  //@ l'algorithme 22_SOS est appele pour la determination des decalages 
  //@ spectraux (algo 23_SSD)
  //@************************************************************************
   
  //@ 5 DphiDTisrfem : dephasage des spectres precalcules pour 22_SOS de la 
  //@   chaine ISRFEM
  //@   Calcul des differences de marche optiques associees aux echantillons
  //@   mo = NsFFT/2 + 1
  //@   x(m) = (m - mo) .  1/ ( cf 22_SOS)
  //@   Boucle sur le facteur de surechantillonnage ns varie de 1 a 
  //@   OSFftFactor
  //@   Calcul du dephasage dphi(m) = 2.Pi . (ns-1) . x(m) .  ( cf 22_SOS)
  
  //@ Calcul des differences de marche optique pour chaque echantillon
  //@ Par symetrie, on ne calcule que la premiere moitie des coefficients
  for (Echantillon = 1; 
       Echantillon <= DimDiffMarcheOptiqueISRFEM;  
       Echantillon++)
    {
      // FA 208
      DiffMarcheOptiqueISRFEM[IND1(Echantillon)] = 
        ((double)(IND1(Echantillon) - IND1(EchantillonRefISRFEM))) / 
        (((double)NbEchantillonsIRSFEM) * PasSpectre);
      // FIN FA 208
    }
  
  // Initialisation du dephasage pour chaque difference de chemin optique
  //@ POUR chaque valeur de surech comprise entre 1 et le surech FAIRE
  for (IndiceSurech = 1; 
       IndiceSurech <= SurechISRFEM; 
       IndiceSurech++)
    {
      //@ POUR chaque echantillon du spectre FAIRE
      for (Echantillon = 1; 
           Echantillon <= NbEchantillonsIRSFEM; 
           Echantillon++)
        {
          // Convention FORTRAN
          IndiceDephasage = 
            IND1(IndiceSurech) * NbEchantillonsIRSFEM + IND1(Echantillon) + 1;
      
          //@ Calculer le dephasage des echantillons precedant m0 (m0 inclus)
          if (IND1(Echantillon) <= IND1(EchantillonRefISRFEM))
            {
		// FA 208
              DephasageISRFEM[IND1(IndiceDephasage)] = 
                2.0 * CMN__PI * ((double)IND1(IndiceSurech)) * 
                DiffMarcheOptiqueISRFEM[IND1(Echantillon)] * 
                PasSpectre / ((double)SurechISRFEM);
            }
      
          //@ Calculer le dephasage des echantillons suivant m0 (m0 exclus)
          else
            {
              DephasageISRFEM[IND1(IndiceDephasage)] = 
                - 2.0 * CMN__PI * ((double)IND1(IndiceSurech)) * 
                DiffMarcheOptiqueISRFEM[NbEchantillonsIRSFEM - IND1(Echantillon)] * 
                PasSpectre / ((double)SurechISRFEM);
            }
          CosDephasageISRFEM[IND1(IndiceDephasage)] = cos(DephasageISRFEM[IND1(IndiceDephasage)]);
          SinDephasageISRFEM[IND1(IndiceDephasage)] = sin(DephasageISRFEM[IND1(IndiceDephasage)]);

//printf("DPHI_%d\t%d\t%lf\n",IndiceSurech,Echantillon,DephasageISRFEM[IND1(IndiceDephasage)]);
	 // FIN FA 208

        }
      //@ FIN POUR chaque echantillon du spectre
    }
  //@ FIN POUR chaque valeur de surech 

  //@ Allocation des resultats
  m_DataGen->WnShift = DecalageSpectral;
  m_DataGen->FlagNomQual = FlagQualiteDecalage;
  
  m_DataGen->CosDephasagePRODUIT = CosDephasagePRODUIT;
  m_DataGen->CosDephasageISRFEM = CosDephasageISRFEM;
  m_DataGen->SinDephasagePRODUIT = SinDephasagePRODUIT;
  m_DataGen->SinDephasageISRFEM = SinDephasageISRFEM;
  
  delete [] DephasageISRFEM;
  delete [] DephasagePRODUIT;
  delete [] DiffMarcheOptiqueISRFEM;
  delete [] DiffMarcheOptiquePRODUIT;
  
#ifdef OPS_DEBUG
  cout << "m_DataGen->CosDephasagePRODUIT[0] : " << m_DataGen->CosDephasagePRODUIT[0] << endl;
#endif

  // sortie de la fonction 
}
