static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_GestCasDeg.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_GestCasDeg
//@ 
//@ Comment :
//@    this module implement the SD_DON_GestCasDeg class
//@    used to administrate the degraded cases found in OPS L1 PPF
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 17/01/2003 : P.GASSIE
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-139-CN : 21/09/2004 : B.MASSART
//@ Probleme d'initialisation de GDatFlagIsrfemOff quand on entre en mode
//@ calibration externe ne visant pas la Terre. On introduit le mode
//@ de prise de vue dans la gestion des cas degrades. Voir IA-2100-FC-139-THA.
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <iostream>

#include "SD_DON_GestCasDeg.h"

//======================================================================
//					 METHODES PUBLIQUES 
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Methode : SD_DON_GestCasDeg
//@
//@ Description : Constructeur
//@
//@ Parameters :
//@			aucun
//@
//@ Return Status : 
//@			Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_GestCasDeg::SD_DON_GestCasDeg()
{
  //@ initialisation des attributs
  m_ListeCasDegLigne.clear();
}

//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_DON_GestCasDeg
//@
//@ Description : Destructeur
//@
//@
//@ Parameters :
//@			Aucun
//@
//@ Return Status : 
//@			Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_GestCasDeg::~SD_DON_GestCasDeg()
{
  //@liberation des listes allouee
  m_ListeCasDegLigne.clear();
}
//@ ------------------------------------------------------------------*/
//@ Methode : Liberer
//@
//@ Description : libere la liste
//@
//@
//@ Parameters :
//@			Aucun
//@
//@ Return Status : 
//@			Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestCasDeg::Liberer()
{
  //@liberation des listes allouee
  t_ListeCasDegLigne::iterator It;
  for(It = m_ListeCasDegLigne.begin();
      It != m_ListeCasDegLigne.end();
      It++)
  {
    delete *It;
  }
  m_ListeCasDegLigne.clear();
}

//@ ------------------------------------------------------------------*/
//@ Methode : Init
//@
//@ Description : Cette methode permet d'initialiser le gestionnaire
//@ de cas degrade avec le contexte prealablement charge
//@
//@
//@ Parameters :
//@			(ENTREE)  Contexte : contexte pour initialisation
//@			(ENTREE)  durees : durees issues du fichier de conf
//@
//@ Return Status : 
//@			Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestCasDeg::Init(
			     SD_DON_Contexte  & Contexte,
			     SD_DON_CasDegLigne::t_SD_DON_IdefDur & durees
)
{
static const char tcProcName_F[] = "SD_DON_GestCasDeg::Init";
// Entree dans la fonction 
  //@ on initialise l'attribut m_Contexte
  m_Contexte.Recopie(Contexte);
  //@ on initialise l'attribut m_Duree
  for(int i = 0; i < SD_BAS_NB_CGXX; i++)
  {
    m_Duree[i] = durees[i];
  }
// sortie de la fonction  
}


//@ ------------------------------------------------------------------*/
//@ Methode : AjouteCasDegLigne
//@
//@ Description : Cette methode permet de d'ajouter Un Cas degrade 
//@ associee a une ligne pour le traitement complet 0-1C. 
//@
//@
//@ Parameters :
//@			(ENTREE)  numero : numero de la ligne concernee
//@
//@ Return Status : 
//@			Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestCasDeg::AjouteCasDegLigne(
  int NumLigne, 
  e_BAS_ModePdv Mode)/* FA139 */
{
static const char tcProcName_F[] = "SD_DON_GestCasDeg::AjouteCasDegLigne";
t_CMN__Statut cr_op=CMN__STATUT_OK;
// Entree dans la fonction 

  SD_DON_CasDegLigne* CasDegLigneTmp;
  CasDegLigneTmp = new SD_DON_CasDegLigne(NumLigne, Mode); /* FA139 */
  //@ SI c'est la premiere ligne
  if(NumLigne == 1)
  {
    //@ on initialise l'objet CasDegLigne
    //@ precedent a partir du contexte (methode Init(m_Contexte,m_Duree)
    //@ ceci se traduit par l'initialisation des dates et des durees des CGxx
    CasDegLigneTmp->Init(m_Contexte,m_Duree);
  }
  //@ SINON
  else
  {
    //@ on initialise l'objet CasDegLign
    //@  a partir du precedent (methode InitPrec(CasDegLigne)
    //@ remarque : ici on initialise les cas degrade a neutre et les dates
    //@ de declenchement sont recopiees depuis la ligne precedentes
    //@ attention : ceci suppose que la ligne precedente a ete controle au prealable
    CasDegLigneTmp->InitPrec(*(m_ListeCasDegLigne.back()), m_Duree);
  }
  //@ FIN SI
  //@ on ajoute le Cas degrade  a la liste  m_ListeCasDegLigne (STL)
  m_ListeCasDegLigne.insert(m_ListeCasDegLigne.end(), CasDegLigneTmp);
  
  //a	 les dates sont initialisee a 0 (AD) dans le cas ou aucun cas n'est leve

  // on trace ici si besoin toutes les variables en sortie
#ifdef OPS_DEBUG
  cout << "-----------------------------" << endl;
  /* FA139 */
  cout << "SD_DON_GestCasDeg::AjouteCasDegLigne, numligne : " << NumLigne 
       << endl << ", mode : " << Mode << endl;
  //cout << "m_Flt[1].dim1 : " << CasDegLigneTmp->GetFlt(1).dim1 << endl;
  //t_ListeCasDegLigne::iterator pInfo;
  //int i = 0;
  //for(pInfo = m_ListeCasDegLigne.begin();
  //     pInfo != m_ListeCasDegLigne.end(); pInfo++)
  //{
  //  cout << "m_ListeCasDegLigne[" << i++ << "].dim1 : " << (*(pInfo))->GetFlt(i).dim1 << endl;
  //}
  cout << "-----------------------------" << endl;
#endif
// sortie de la fonction
}
//@ ------------------------------------------------------------------*/
//@ Methode : AjouteCasDegLignePart
//@
//@ Description : Cette methode permet de d'ajouter Un Cas degrade 
//@ associee a une ligne pour le traitement partiel ( et pour le produit 
//@ technologique) . 
//@
//@
//@ Parameters :
//@			(ENTREE)  numero : numero de la ligne concernee
//@			(ENTREE)  Flags : flags issus de la ligne concernee 
//@			(produit de verification)
//@			(ENTREE)  Dates : Dates issus de la ligne concernee
//@			(produit de verification)
//@
//@ Return Status : 
//@			Aucune
//@			t_GLB_Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestCasDeg::AjouteCasDegLignePart(
  int NumLigne,
  e_BAS_ModePdv Mode,  /* FA139 */
  SD_DON_CasDegLigne::t_SD_DON_Flt & Flags,
  SD_DON_CasDegLigne::t_SD_DON_GOPSDAT & Dates
		)
{
static const char tcProcName_F[] = "SD_DON_GestCasDeg::AjouteCasDegLignePart";
t_CMN__Statut cr_op=CMN__STATUT_OK;
// Entree dans la fonction 

  SD_DON_CasDegLigne* CasDegLigneTmp;
  CasDegLigneTmp = new SD_DON_CasDegLigne(NumLigne, Mode); /* FA139 */
  //@ SI c'est la premiere ligne
  if(NumLigne == 1)
  {
    //@ on initialise l'objet CasDegLign
    //@ precedent a partir du contexte (methode Init(Contexte)
    //@ ceci se traduit par l'initialisation des dates et des durees 
    CasDegLigneTmp->Init(m_Contexte, m_Duree);
  }
  //@ SINON
  else
  {
    //@on initialise l'objet CasDegLign
    //@  a partir du precedent (methode InitPrec(CasDegLigne,durees)
    //@ remarque : ici on initialise les cas degrade a neutre et les dates
    //@ de declanchement et les durees sont recopiees depuis la ligne precedentes
    //@ attention : ceci suppose que la ligne precedente a ete controle au prealable 
    //@ cela permet une propagation des durees
    CasDegLigneTmp->InitPrec(*(m_ListeCasDegLigne.back()), m_Duree);
  }
  
  //@ on initialise alors avec les donnees issue du produit 1A (methode
  //@ InitFlt(Flags)
  //@ remarque : ici on initialise les cas degrade a neutre et
  //a	 les dates sont initialisee a 0 (AD) dans le cas ou aucun cas n'est leve
  CasDegLigneTmp->InitFlt(Flags, Dates);
  //@ on ajoute le Cas degrade  a la liste  m_ListeCasDegLigne (STL)
  m_ListeCasDegLigne.insert(m_ListeCasDegLigne.end(), CasDegLigneTmp);
    
// on trace ici si besoin toutes les variables en sortie

// sortie de la fonction 
}
//@ ------------------------------------------------------------------*/
//@ Methode : GetCasDegLigne
//@
//@ Description : Cette methode permet de recupere l'objet CasDegLigne
//@ associe a un numero de ligne
//@
//@
//@ Parameters :
//@			(ENTREE)  Num ligne : numero de la ligne
//@
//@ Return Status : 
//@			Aucune
//@			t_GLB_Statut 
//@			CMN__STATUT_OK si la fonction a reussi
//@			CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
SD_DON_CasDegLigne* SD_DON_GestCasDeg::GetCasDegLigne(
				int NumLigne
				)
{
static const char tcProcName_F[] = "SD_DON_GestCasDeg::GetCasDegLigne";
t_CMN__Statut cr_op=CMN__STATUT_OK;
t_ListeCasDegLigne::iterator It;

// Entree dans la fonction 

//@ on  recupere dans la liste m_ListeCasDegLigne le cas degrade
//@ correspondant au numero de ligne
  It = m_ListeCasDegLigne.begin();
  for(int i = 1; i < NumLigne; i++)
  {
    It++;
  }
fin:
// sortie de la fonction 
return(*It); 

}

//======================================================================
//					METHODES PRIVEES 
//======================================================================

