static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_GestConfAutre.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_GestConfAutre
//@ 
//@ Comment :
//@    this module implement the SD_DON_GestConfAutre class
//@    used to reference the different "other" configuration files
//@      needed by OPS L1 PPF
//@                                     
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 30/11/2002 : P.GASSIE
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <iostream>

#include "SD_DON_GestConfAutre.h"
#include "JDB__Client.h"

//
// Definition des constantes
//
const t_Chaine SD_DON_GestConfAutre::JDB_PARAM_1 = "$PRM1";
const t_Chaine SD_DON_GestConfAutre::DATA_AUX_AUT_NOT_FOUND = "Other parameters configuration file not found";
// initialisation variable statique
pthread_mutex_t   SD_DON_GestConfAutre::s_MutexTravailleConf;

//======================================================================
//                                       METHODES PUBLIQUES 
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Methode : SD_DON_GestConfAutre
//@
//@ Description : Constructeur
//@
//@ Parameters :
//@                     Aucun
//@
//@ Return Status : 
//@                     Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_GestConfAutre::SD_DON_GestConfAutre()
{
  // initialisation des attributs
  m_PTSI = -1;
  m_DateCreation = -1;
}
//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_DON_GestConfAutre
//@
//@ Description : Destructeur
//@
//@
//@ Parameters :
//@                     Aucun
//@
//@ Return Status : 
//@                     Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_GestConfAutre::~SD_DON_GestConfAutre()
{

}
//@ ------------------------------------------------------------------*/
//@ Methode : InitialiserMutex
//@
//@ Description : 
//@
//@ Parameters :
//@     Aucun
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestConfAutre::InitialiserMutex() 
{
  //@ Creer la variable conditionnelle POSIX correspondante et son mutex
  pthread_mutex_init(&s_MutexTravailleConf, NULL);
//temporaire
  cout << "InitialiserMutex conf" << endl;
#ifdef OPS_DEBUG
          cout << "InitialiserMutex conf" << endl;
#endif
}
//jlpJ1
//@ ------------------------------------------------------------------*/
//@ Methode : LibererMutex
//@
//@ Description : 
//@
//@ Parameters :
//@     Aucun
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestConfAutre::LibererMutex()
{
  // Liberation si besoin des donnees de type pointeurs
  //@ Supprimer le mutex et la variable conditionnele POSIX
#ifdef OPS_DEBUG
          cout << "Liberer conf" << endl;
#endif
//temporaire
  cout << "Liberer conf" << endl;
  pthread_mutex_destroy(&s_MutexTravailleConf);
 
}
//@ ------------------------------------------------------------------*/
//@ Methode : Reserver
//@
//@ Description : Cette methode permet de reserver la donnee pour sa mise 
//@ a jour ( cette fonction est bloquante si une autre reservation est en cours
//@  
//@
//@
//@ Parameters : aucun
//@
//@ Return Status : aucun
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestConfAutre::Reserver()

{
  // Entree dans la fonction 
  //@ reservation par mutex
  pthread_mutex_lock(&s_MutexTravailleConf);
  // on trace ici si besoin toutes les variables en sortie
  cout << "reserver conf" << endl;

  // sortie de la fonction 

}

//@ ------------------------------------------------------------------*/
//@ Methode : Liberer
//@
//@ Description : Cette methode permet de liberer un contexte precedemment
//@ reserve ( cette fonction libere une autre reservation  en cours )
//@  
//@
//@
//@ Parameters : aucun
//@
//@ Return Status : aucun
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestConfAutre::Liberer()

{
  // Entree dans la fonction 
  //@ liberation du  mutex
  pthread_mutex_unlock(&s_MutexTravailleConf);
  // on trace ici si besoin toutes les variables en sortie
  cout << "liberer conf" << endl;

  // sortie de la fonction 

}

//@ ------------------------------------------------------------------*/
//@ Methode : CreerInventaire
//@
//@ Description : Cette methode permet d'initialiser l'acces aux donnees 
//@ de configuration autre . Ici les fichiers sont justes repertories 
//@ sans lire le contenu precis
//@
//@
//@ Parameters :
//@                     (ENTREE)  Repertoire : Repertoire contenant le fichier
//@
//@ Return Status : 
//@
//@ ------------------------------------------------------------------*/

void SD_DON_GestConfAutre::CreerInventaire(
                                           const t_Chaine & repertoire
                                           ) throw(ERR__Exception)

{
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  t_Chaine pattern("BRD");
  CMN__ListeInfoFichier ListeInfoFichier;
  CMN__ListeInfoFichier::iterator It;
  SD_DON_ConfAutre confTmp;
  // Entree dans la fonction
  //@ on recupere dans une liste l'ensemble des fichiers de type conf Autre

#ifdef OPS_DEBUG
  cout << "ConfAutre::CreerInventaire(): " << endl;
  cout << "\trepertoire: " << repertoire << endl;
#endif

  cr_op = CMN__TrouverFichiersEx(repertoire, pattern, ListeInfoFichier, NULL, NULL, NULL, 1, 0);  
  //@ SI erreur
  if(cr_op != CMN__STATUT_OK)
    {
      //@ message JDB
      JDB__Client MessageJDB;
      MessageJDB.SetParametre(JDB_PARAM_1, pattern);
      MessageJDB.EcrireMessage(SD_CONF_SET, SD__Message::MSG_CONF_NO_FILE);
      MessageJDB.Liberer();
      //@ exception
      ERR__Exception exception;
      //@ Emission exception SD_DON_TMT_DATA_AUX
      exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_AUX);
      exception.SetParametre(DATA_AUX_AUT_NOT_FOUND);
      THROW(exception);
    }
  //@ FIN SI
  //@ enregisterement du repertoire
  m_repertoire = repertoire;  
  //@ Vider la liste des infos
  m_ListeConfAutre.clear();
  //@ POUR chaque element de la liste
  for (It = ListeInfoFichier.begin(); It != ListeInfoFichier.end(); It ++)
    {
      //@ on initialise un objet ConfAutre a partir du nom de fichier
      confTmp.Initialise(repertoire, It->Name());
      //@       on ajoute cet objet dans la liste m_ListeConfAutre (add STL)
      m_ListeConfAutre.insert(m_ListeConfAutre.end(), confTmp);
    }
  //@ FIN POUR

  // on trace ici si besoin toutes les variables en sortie
#ifdef OPS_DEBUG
  cout << endl << "SD_DON_GestConfAutre::CreerInventaire" << endl;
#endif
  // sortie de la fonction 
}
//@ ------------------------------------------------------------------*/
//@ Methode : ChargerPTSIDateValid
//@
//@ Description : Cette methode permet de charger en memoire le fichier 
//@ de conf autre qui correspond au PTSI issu du contexte
//@
//@
//@ Parameters :        
//@                     (ENTREE)  Ptsi : Ptsi du fichier a charger
//@                     (ENTREE)  tcourante : date courante 
//@                     (SORTIE)  IdCA : Identificateur du fichier de conf autre 
//@                     (SORTIE)  IdCS : Identificateur du fichier de conf stable 
//@                     (SORTIE)  IdBS : Identificateur de la banque spectrale 
//@
//@ Return Status : 
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestConfAutre::ChargerPTSIDateValid(
                                                const int Ptsi,
                                                const t_Date & tcourante,
                                                t_Chaine& nom_fichier,
                                                int& IdCA,
                                                int& IdCS,
                                                int& IdBS
                                                ) throw(ERR__Exception)

{
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  t_ListeConfAutre::iterator It;
  int PTSITmp = -1;
  double dateCreationTmp = -1;
  m_DateCreation = -1;
  t_Chaine pattern("BRD");
#ifdef OPS_DEBUG
  cout << endl << "SD_DON_GestConfAutre::ChargerPTSIDateValid" << endl;
  cout << "PTSI demande: " << Ptsi << endl;
#endif
  

  // Entree dans la fonction 
  //@ SI le PTSI demande est different du PTSI courant
  if(m_PTSI != Ptsi)
    {
      //@ Reinitialisation de m_PTSI
      m_PTSI = -1;

      //@ POUR chaque element de la liste m_ListeConfAutre prealablement triee
      for (It = m_ListeConfAutre.begin();
           It != m_ListeConfAutre.end();
           It ++)
        {
          //@ on initialise le PTSI de l'element courant 
          PTSITmp = It->GetPTSI();
          //@ on charge la date de creation de l'element courant
          dateCreationTmp = (It->GetDateCreation()).ConvertirDateEnTemps();
          //@   SI le PTSI est = Ptsi et si la date de debut validite est anterieure
          //@   a la date courante et si la date de creation est superieure a celle sauvegardee
          t_Date dateTmp(tcourante);
          if ((PTSITmp == Ptsi) && (It->GetDateValid() <= dateTmp) && 
              (dateCreationTmp > m_DateCreation))
            {
              //@       on charge le fichier en memoire
              It->Lire();

              //@       on met a jour l'attribut m_PTSI
              m_PTSI = Ptsi;

              //@ on met a jour l'attribut m_DateCreation
              m_DateCreation = dateCreationTmp;

              //@ on recupere l'identifiant de la BS et du fichier de conf stable et autre
              It->GetIdConf(IdCA, IdCS, IdBS);
              nom_fichier = It->GetNom();
            }
          //@ FIN SI
        }
      //@ FIN POUR

      //@ SI m_PTSI est egal a -1
      if (m_PTSI == -1)
        {
          //@ message JDB
          JDB__Client MessageJDB;
          MessageJDB.SetParametre(JDB_PARAM_1, pattern);
          MessageJDB.EcrireMessage(SD_CONF_SET, SD__Message::MSG_CONF_NO_FILE);
          MessageJDB.Liberer();
          //@  exception
          ERR__Exception exception;
    
          //@ Emission exception SD_DON_TMT_DATA_AUX
          exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_AUX);
          exception.SetParametre(DATA_AUX_AUT_NOT_FOUND);
          THROW(exception);
        }
      //@ FIN SI
    }
  //@ FIN SI

  // on trace ici si besoin toutes les variables en sortie
#ifdef OPS_DEBUG
  cout << endl << "SD_DON_GestConfAutre::ChargerPTSIDateValid" << endl;
  cout << "Fichier Charge : " << nom_fichier << endl;
  cout << "PTSI : " << m_PTSI << endl;
  cout << "IdCA : " << IdCA << endl;
  cout << "IdCS : " << IdCS << endl;
  cout << "IdBS : " << IdBS << endl;
#endif
  // sortie de la fonction 


}
//@ ------------------------------------------------------------------*/
//@ Methode : GetData
//@
//@ Description : Cette methode permet de recuperer l'adresse de la zone
//@              memoire ou est stockee le fichier de conf autre courant
//@             a utiliser 
//@
//@
//@ Parameters :   aucun
//@                     
//@
//@ Return Status :     t_SD_BAS_Autre pointeur sur la donnee
//@                     null est reourne si aucun fichier est charge
//@
//@ ------------------------------------------------------------------*/
t_SD_BAS_ConfAutre* SD_DON_GestConfAutre::GetData(
                )
{
  // Entree dans la fonction
  t_ListeConfAutre::iterator It;
  int PTSITmp = -1;
  t_SD_BAS_ConfAutre* confTmp = NULL;
  double dateCreationTmp = -1;
  bool cherche = true;

  //@ SI m_PTSI different de -1
  if (m_PTSI != -1)
    {
    
      //@ pour chaque element de la liste m_ListeConfAutre prealablement triee
      for (It = m_ListeConfAutre.begin(); (It != m_ListeConfAutre.end()) && (cherche);It ++)
        {
          //@ on recupere le PTSI de l'element courant
          PTSITmp = It->GetPTSI();
          //@ on recupere la date de creation de l'element courant
          dateCreationTmp = (It->GetDateCreation()).ConvertirDateEnTemps();
          //@   SI le PTSI est = m_Ptsi et si la date de creation correspond
          if((PTSITmp == m_PTSI) && (dateCreationTmp == m_DateCreation))
            {
              //@       on recupere l'adresse en memoire le fichier en memoire
              confTmp = It->GetData();
              //@ on positionne la sortie de la boucle
              cherche = false;
            }
          //@ FIN SI
        }
      //@ FIN POUR
    }
  //@ FIN SI
  return(confTmp); 
  // sortie de la fonction 
}


//======================================================================
//                                      METHODES PRIVEES 
//======================================================================
