static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_GestDonnees.cc,v 1.5 2013/03/29 16:55:26 lcaspar Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_GestDonnees
//@ 
//@ Comment :
//@    this module is the unique entry point to the IASI AVHRR 
//@    persitent data used bas IASI L1 Processing chain
//@    this ensure the availability and the coherence of
//@    the data user/produced  among the multithreaded acces
//@           
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 23/01/2003 : P.GASSIE
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-150-CN : 28/01/2005 : JL.PASCAL
//@ bug aleatoire : correction delete incorrects
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-158-CN : 30/06/2005 : JL PASCAL
//@ L'OPS genere des resulta VIADRENG_MEXSSMIN et MAX errone et qui  
//@ dependent du nombre de thread 
//@ Voir IA-2100-FC-158-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-271-CN : 22/06/2005 : JL.PASCAL
//@ prise en compte du mode transition
//@ fiche de correction correspondante : IA-2100-CR-271-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-308-CN : 12/09/2006 : C. MERIGUET
//@ Lever le cas degrade 9 si le(s) fichier(s) AVHRR n'est
//@ pas accessible ou mauvais numero de version. Ne pas lever
//@ d'exception et poursuivre le traitement.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-184-CN : 20/09/2006 : C.MERIGUET
//@ Suite passage purify delete[] remplace delete pour les tableaux
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FT-2454-CN : 07/04/2008 : S.BRUNEL
//@ Critere de choix du fichier SVM erronne
//@ --------------------------------------------------------------------
//@ DM-ID :IA-FT-2449(DM-2100-OPS) : 11/04/2008 : S.BRUNEL
//@ Rendre la lecture du fichier SVM plus efficace
//@ --------------------------------------------------------------------
//@ FA-ID :IA-FT-2466(FA-2100-OPS) : 30/05/2008 : S. BRUNEL
//@ Probleme de reproductibilite: SUBSAT_LATITUDE_START et
//@ SUBSAT_LONGITUDE_START
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2512 (FA-2100-OPS) : 17/11/2008 : S. BRUNEL
//@ Fuites memoires signalees par EUMETSAT (EUM.EPS.NCR.3627.1)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2939(DM-2100-OPS) : 12/12/2011 : L. CASPAR
//@ Mise en place de conditions de levee du cas degarde 9 alternatives pour 
//@ correction de la NCR 12303
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <iostream>
#include "env.h"

#include "SD_DON_GestDonnees.h"

//
// Version Classe
//
const  t_Chaine SD_DON_GestDonnees::VERSION_CLASSE = "1.0";
const t_Chaine SD_DON_GestDonnees::PREFIXE_SVM            = "xxxx_SVM_xx_";
const t_Chaine SD_DON_GestDonnees::DATA_AUX_SVM_NOT_FOUND = "SVM aux data not found";
const t_Chaine SD_DON_GestDonnees::PREFIXE_STABLE_SCF = "IASI_GRD_xx";
const t_Chaine SD_DON_GestDonnees::PREFIXE_STABLE_OCF = "IASI_BRD_xx";
const t_Chaine SD_DON_GestDonnees::PREFIXE_BANQUE_SPECTRALE = "IASI_ODB_xx";
const t_Chaine SD_DON_GestDonnees::PREFIXE_CONTEXTE = "IASI_CTX_xx";
const t_Chaine SD_DON_GestDonnees::MISSING_AUX_FILE = "Missing auxiliary file of nature ";
const t_Chaine SD_DON_GestDonnees::ENV_UNDEFINED = "Undefined environment variable: ";
const t_Chaine SD_DON_GestDonnees::ERROR_END_TREATMENT = "Error while writing products at the end of the treatment";

// initialisation variable statique FA158
pthread_mutex_t   SD_DON_GestDonnees::m_MutexTravaille;
t_CMN__Booleen  SD_DON_GestDonnees::m_transition=CMN__FAUX; //CR271
//======================================================================
//           METHODES PUBLIQUES 
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD_DON_GestDonnees
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_GestDonnees::SD_DON_GestDonnees()
  : m_pGestGranule(0), m_pSVM(0)
{
}

//FA158
//@ ------------------------------------------------------------------*/
//@ Methode : InitialiserMutex
//@
//@ Description : 
//@
//@ Parameters :
//@     Aucun
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestDonnees::InitialiserMutex() 
{
  //@ Creer la variable conditionnelle POSIX correspondante et son mutex
  pthread_mutex_init(&m_MutexTravaille, NULL);
#ifdef OPS_DEBUG
          cout << "InitialiserMutex gest donnees" << endl;
#endif
}
//@ ------------------------------------------------------------------*/
//@ Methode : LibererMutex
//@
//@ Description : 
//@
//@ Parameters :
//@     Aucun
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestDonnees::LibererMutex()
{
  // Liberation si besoin des donnees de type pointeurs
  //@ Supprimer le mutex et la variable conditionnele POSIX
#ifdef OPS_DEBUG
          cout << "Liberer gest donnees" << endl;
#endif
  pthread_mutex_destroy(&m_MutexTravaille);

}
//@ ------------------------------------------------------------------*/
//@ Methode : Reserver
//@
//@ Description : Cette methode permet de reserver la donnee pour sa mise 
//@ a jour ( cette fonction est bloquante si une autre reservation est en cours
//@  
//@
//@
//@ Parameters : aucun
//@
//@ Return Status : aucun
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestDonnees::Reserver()

{
  // Entree dans la fonction 
  //@ reservation par mutex
#ifdef OPS_DEBUG
          cout << "reserver gest donnees" << endl;
#endif
  pthread_mutex_lock(&m_MutexTravaille);
  // on trace ici si besoin toutes les variables en sortie

  // sortie de la fonction 

}

//@ ------------------------------------------------------------------*/
//@ Methode : Liberer
//@
//@ Description : Cette methode permet de liberer le gestionnaire precedemment
//@ reserve ( cette fonction libere une autre reservation  en cours )
//@  
//@
//@
//@ Parameters : aucun
//@
//@ Return Status : aucun
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestDonnees::Liberer()

{
  // Entree dans la fonction 
  //@ liberation du  mutex
#ifdef OPS_DEBUG
          cout << "liberer gest donnees" << endl;
#endif
  pthread_mutex_unlock(&m_MutexTravaille);
  // on trace ici si besoin toutes les variables en sortie

  // sortie de la fonction 

}

//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_DON_GestDonnees
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_GestDonnees::~SD_DON_GestDonnees()
{
  //@ on libere ici les pointeurs eventuellement alloues  
  //@ et m_pGestGranule
  if (m_pGestGranule)
    {
      delete m_pGestGranule;
      m_pGestGranule = 0;
    }

  if (m_pSVM)
    {
      m_pSVM->LibererMutex();       
      delete m_pSVM;
      m_pSVM = 0;
    }

  m_GestCasDeg.Liberer();
//FA158
  LibererMutex();
}


//@ ------------------------------------------------------------------*/
//@ Methode : InitDump
//@
//@ Description : Cette methode permet d'initialiser les donnees
//@               statiques de IASI (configuration, contexte) 
//@               En cas d'erreur de lecture dans cette fonction, le
//@               traitement est interrompu et la chaine s'arrete
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestDonnees::InitDump()
{
  int Ptsi = 0;
//FA158
InitialiserMutex();
#ifdef OPS_DEBUG
  cout << "------------------------------------" << endl;
  cout << " Variables environnement initiales: " << endl;
  cout << "\tREP_AUX_DATA: " 
       << CFG__Configuration::GetRepertoire(SD_DON_InventaireConf::
                                            SD_DON_VAR_ENV_AUX_DATA)
       << endl;
  cout << "\tREP_CTX_DATA: " 
       << CFG__Configuration::GetRepertoire(SD_DON_InventaireConf::
                                            SD_DON_VAR_ENV_CTX_DATA)
       << endl;
#endif

  //@ Initialisation des variables d'environnement pour les donnees
  //@ auxiliaires
  //@   - mise a jour variable d'environnement REP_AUX_DATA
//fa 115 verification de l'existence
  InitEnvironnement(PREFIXE_STABLE_SCF, 
                    SD_DON_InventaireConf::SD_DON_VAR_ENV_AUX_DATA);
  InitEnvironnement(PREFIXE_STABLE_OCF, 
                    SD_DON_InventaireConf::SD_DON_VAR_ENV_AUX_DATA);
  InitEnvironnement(PREFIXE_BANQUE_SPECTRALE, 
                    SD_DON_InventaireConf::SD_DON_VAR_ENV_AUX_DATA);
  //@   - mise a jour variable d'environnement REP_CTX_DATA
  InitEnvironnement(PREFIXE_CONTEXTE, 
                    SD_DON_InventaireConf::SD_DON_VAR_ENV_CTX_DATA);

#ifdef OPS_DEBUG
  cout << " Variables environnement modifiees: " << endl;
  cout << "\tREP_AUX_DATA: " 
       << CFG__Configuration::GetRepertoire(SD_DON_InventaireConf::
                                            SD_DON_VAR_ENV_AUX_DATA)
       << endl;
  cout << "\tREP_CTX_DATA: " 
       << CFG__Configuration::GetRepertoire(SD_DON_InventaireConf::
                                            SD_DON_VAR_ENV_CTX_DATA)
       << endl;
  cout << "------------------------------------" << endl << endl;
#endif

  //@ Appel a la fonction SD_DON_InventaireConf::CreerInventaire()
  m_InventaireConf.CreerInventaire();

  //@ Appel a la fonction SD_DON_DonGenGranule::InitLecture() 
  m_DonGenGranule.InitLecture(m_Traitement, Ptsi);

  //@ Appel a la fonction SD_DON_InventaireConf::InitConf()
  t_Date dateCourante;
  dateCourante.DateCourante();
  m_InventaireConf.InitConf(Ptsi, m_Traitement, dateCourante, m_IdCA);

  //@ En cas de succes, on emet un message Log event indiquant
  //@ que la conf est chargee
  //@ Emission d'un message vers le JdBS
  JDB__Client MessageJDB;
  MessageJDB.EcrireMessage(SD_PRODUCT_SET, 
                           SD__Message::MSG_PROD_CONF_CHARGEE);
  MessageJDB.Liberer();
}


//@ ------------------------------------------------------------------*/
//@ Methode : InitGranule
//@
//@ Description : Cette methode permet d'initialiser les donnees
//@ dynamique de IASI 
//@ - produits
//@ - donnees auxiliaires
//@ les donnees de configuration sont ici reinitialises
//@ et les donnees generales sont mises a jour 
//@ en cas d'erreur de lecture des fichiers de conf dans cette fonction
//@ le traitement est interrompu et la chaine s'arrete
//@
//@
//@ Parametres :
//@               (SORTIE) startTime
//@               (SORTIE) startOrbitNumber
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestDonnees::InitGranule(t_TableauDate& startTime, 
                                     t_TableauEntier& startOrbitNumber) 
  throw (ERR__Exception)
{
  ERR__Exception exception;
  t_SD_BAS_DataGen** datagen;
  t_SD_BAS_Contexte** contextecourant;
  int nbLigneAbsolu = 0;
  int nbDataGap = 0;
  int valeurPTSI = 0;
  t_Date tcourante;
  t_ListeChaine listeDataAux;
  t_ListeChaine::iterator It;
  t_ListeChaine listeDonneesSVM;
  t_Chaine nomFichierSVM;
  double    elapsedTime     = 0.;
  double    userTime        = 0.;
  double    systemTime      = 0.;


  //@ En captant les exceptions
  try
    {
      //@ on initialise ici les donnees dynamiques suivantes (new):
      //@   - gestionnaire des granules
      //@   - SVM
      m_pGestGranule = new SD_DON_GestGranule();
      
      //@ enregistrement date debut traitement granule
      tcourante.DateCourante();
      m_Traitement.StartTime(tcourante);
      
      //@ Mesure de temps
      MesurerTemps(CMN__VRAI, &elapsedTime, &userTime, &systemTime);
      
      //@ appel a la fonction SD_DON_GestDonnees::GetDonGenGranule
      datagen = new t_SD_BAS_DataGen*[1];
      contextecourant = new t_SD_BAS_Contexte*[1];
      this->GetDonGenGranule(contextecourant, datagen);
      
      //@ pour recupere le nombre de ligne depuis le debut du dump
      nbLigneAbsolu = contextecourant[0]->GCTXNumDerLig[0];
 
#ifdef INVESTIGATION 
      cout << "SD_DON_GestDonnees::InitGranule :: nbLigneAbsolu = " << nbLigneAbsolu << endl;
#endif      
/* FA 184 */
      delete [] datagen;
      delete [] contextecourant;
/* .. FA 184 */

      //@ appel a la fonction SD_DON_InventaireConf::ChargerConf
      //@ on stocke ici l'idconf du fichier de conf autre m_IdCA
      //@ en cas d'erreur le traitement s'arrete 
      //@ ( le fichier n'existe pas pour le PTSI donne)
      tcourante.DateCourante();
      valeurPTSI = m_InventaireConf.GetPTSI();
      m_InventaireConf.ChargerConf(valeurPTSI, 
                                   m_Traitement, 
                                   tcourante, 
                                   m_IdCA);
      
      //@ initialisation du gestionnaire de cas degrades
      InitialiserGestCasDeg(valeurPTSI);
      
      //@ appel a la fonction SD_DON_GestGranule::InitGranuleIASI
      //@ en cas d'erreur le traitement continue sauf pb IO
      m_pGestGranule->InitGranuleIASI(m_Traitement, 
                                      m_GestCasDeg, 
                                      valeurPTSI, 
                                      nbLigneAbsolu);
  
      //@ Referencer les trous de donnees existants
      nbDataGap = m_pGestGranule->GetNbDataGap();
      if (nbDataGap > 0)
        {
          t_DataGap dataGap;
          dataGap.Reason(CMN_DAT_DataGap::GAP_REASON_ONE);
          dataGap.Count(nbDataGap);
          m_Traitement.GeneratedFile().AjouterDataGap(dataGap);
        }

      //@ appel a la methode SD_DON_GestGranule::GetInfoHKTM afin de 
      //@ recuperer les infos HKTM a renvoyer
      m_pGestGranule->GetInfoHKTM(startTime, startOrbitNumber);
      
      //@ appel a la fonction SD_DON_GestGranule::InitGranuleAVHRR
      //@ doit etre appelee apres GestGranule.InitGranuleIASI()
      //@ en cas d'erreur le traitement continue sauf pb IO
/* CR 308*/
      m_pGestGranule->InitBandeauAVHRR(m_Traitement,m_GestCasDeg);
/* ..CR 308*/
   
      //@ on recupere  la liste des donnees auxiliaires en sortie 
      //@ par l'objet SD_DON_Traitement
      listeDataAux = m_Traitement.ListeDonneesAux();
      
      //@ POUR tous les elements de la liste des donnees auxiliaires
      for (It = listeDataAux.begin(); 
           It != listeDataAux.end();
           It++)
        {
          //@ SI le fichier est de type SVM
          if (EstDeType(PREFIXE_SVM, *It) == CMN__STATUT_OK)
            {
              //@ L'ajouter dans la liste des fichiers SVM
              listeDonneesSVM.push_back(*It);
            }
          //@ FIN SI
        }
      //@ FIN POUR
      
      //@ SI la liste des fichiers SVM est vide
      if (listeDonneesSVM.size() == 0)
        {      

          //@ Emission d'un message de type JDB MSG_DATA_SVM_UNAVAILABLE
          JDB__Client messageJDB;
          messageJDB.EcrireMessage(SD_CONF_SET, SD__Message::
                                   MSG_CONF_SVM_UNAVAILABLE);
          messageJDB.Liberer();
          
          //@ Emission d'une exception SD_DON_TMT_DATA_AUX
          exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_AUX);
          exception.SetParametre(DATA_AUX_SVM_NOT_FOUND);
          THROW(exception);
      }
      //@ FIN SI la liste des fichiers SVM est vide

      // FT-2454
      //@ On recupere la date de debut de granule
      t_Date dateDebutGranule = m_pGestGranule->GetDateDebutIASI();
      //@ On recupere la date de fin de granule
      t_Date dateFinGranule = m_pGestGranule->GetDateFinIASI();
      // ..FT2454
      
      //@ Tri de la liste des  donnees SVM
      listeDonneesSVM.sort();

      // FT-2454
      bool trouve = CMN__FAUX;
      It = listeDonneesSVM.begin();

      do {
          //@ On construit un objet t_OPSFile afin de recuperer
          //@ les dates de validite pour le premier element de la
          //@ liste des donnees OBTUTC
          t_OPSFile svmOPS(*It);
          t_Date dateDebutFichierSVM = svmOPS.GetValidityStartDate();
          t_Date dateFinFichierSVM = svmOPS.GetValidityEndDate();

          if ((dateDebutFichierSVM <= dateDebutGranule) &&
              (dateFinFichierSVM >= dateFinGranule))
          {
            nomFichierSVM = *It;

            // On verifie le fichier SVM suivant, s'il obeit aux memes
            // criteres, on selectionne ce fichier (Recouvrement)
            It++;
            if (It != listeDonneesSVM.end())
            {
              t_Date dateDebutFichierSVMbis = svmOPS.GetValidityStartDate();
              t_Date dateFinFichierSVMbis = svmOPS.GetValidityEndDate();

              if ((dateDebutFichierSVMbis <= dateDebutGranule) &&
                  (dateFinFichierSVMbis >= dateFinGranule))
              {
                nomFichierSVM = *It;
              }
            }
            trouve = CMN__VRAI;
          } 
          else
          {
            It++;
          }
      } while (trouve == CMN__FAUX && It != listeDonneesSVM.end());
      // ..FT-2454
      
      //@ RAZ de la liste des donnees SVM
      listeDonneesSVM.clear();
  
      //@ SI le fichier SVM n'a pas ete trouve
      if (trouve == CMN__FAUX)
        {
          //@ Emission d'un message de type JDB MSG_CONF_SVM_UNAVAILABLE
          JDB__Client messageJDB;
          messageJDB.EcrireMessage(SD_CONF_SET, SD__Message::
                                   MSG_CONF_SVM_UNAVAILABLE);
          messageJDB.Liberer();
          
          //@ Emission d'une exception SD_DON_TMT_DATA_AUX
          exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_AUX);
          exception.SetParametre(DATA_AUX_SVM_NOT_FOUND);
          THROW(exception);
        }
      //@ FIN SI le fichier SVM n'a pas ete trouve
  
      // FT-2449
      // Charge la liste des dates de fin d'eclipse en memoire
      if (m_pSVM == NULL)  {
        m_pSVM = new SD_DON_SVM();
        m_pSVM->InitialiserMutex();
        m_pSVM->Reserver();
        m_pSVM->Lire(nomFichierSVM);
        m_pSVM->Liberer();
      }
      // ..FT-2449

      
      //@ on ajoute a la liste des fichiers utilises :
      //@      dans SD_DON_FinTraitement
      m_Traitement.AjouterProduitE(nomFichierSVM);
 
      //@ en cas d'erreur le traitement continue sauf pb IO

      
      //@ appel a la fonction SD_DON_DonGenGranule::InitCalcul
      t_Date dateDebut = m_pGestGranule->GetDateDebutIASI();
      m_DonGenGranule.InitCalcul(m_InventaireConf, valeurPTSI, dateDebut);
  
      //@ en cas d'erreur le traitement continue sauf pb IO
    }

  //@ Traitement des exceptions
  catch (ERR__Exception& exception)
    {
     //@ appel a la fonction LibererGranule
      LibererGranule();
      
      //@ Re-emission de l'exception
      throw(exception);
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : SetEntete
//@
//@ Description : Cette methode permet de mettre a jour le MPR et GIADR 
//@ en fin de traitement pour les produits en sortie
//@
//@
//@ Parameters :
//@     (ENTREE)  InfoGen :  info generales pour le
//@           MPHR
//@     (ENTREE)  ParamConfStable :  parametre de qualit pour le
//@           GIADR
//@
//@ Return Status : 
//@     Aucune
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
// FT-2466
t_CMN__Statut  SD_DON_GestDonnees::SetEnteteS1(
    t_SD_BAS_StatMphr  * StatMPHR,
    t_SD_BAS_ConfStable  * ParamConfStable,
    int numeroLigne)
{
  t_CMN__Statut cr_op=CMN__STATUT_OK;


  //@ appel de la fonction  SD_DON_GestGranule::SetEnteteS1
  m_pGestGranule->SetEnteteS1(StatMPHR, ParamConfStable, numeroLigne);

  //@ si PCD = 1 dans StatMPHR  appel a SD_DON_FintraitementSetPCD
  

  return(cr_op);
}
// ..FT-2466

//@ ------------------------------------------------------------------*/
//@ Methode : FinGranule
//@
//@ Description : Cette methode permet de terminer le traitement d'un 
//@               granule de IASI par l'ecriture des donnees
//@                 - produits
//@                 - contexte
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestDonnees::FinGranule() throw (ERR__Exception)
{
  t_SD_BAS_MPHR mphr;
  int           orbitestart = 0;
  t_Date        tcourante;
  t_ListeChaine listeFichier;
  t_Chaine      nomFichierContexte("");


  //@ En captant les exceptions
  try
    {
      //@ appel a la fonction SD_DON_DonGenGranule::FinCalcul
      m_DonGenGranule.FinCalcul();

      //@ appel a la fonction SD_DON_GestGranule::EcrireS1
      m_pGestGranule->EcrireS1(m_IdCA, mphr, listeFichier);

      m_Traitement.ListeProduitS(listeFichier);

      orbitestart = *(mphr.listevar.tab_var[MPHR_ORBIT_START].val.pi4);
      
      //@ appel a la fonction SD_DON_DonGenGranule::SauverContexte 
      tcourante.DateCourante();
      // CR233 modification de la nomenclature du contexte
      tcourante = m_pGestGranule->GetDateFin();
      m_DonGenGranule.SauverContexte(tcourante, 
                                     orbitestart, 
                                     m_Traitement.SlideId(),
                                     nomFichierContexte);
      
      if (m_Traitement.TypeTraitement() == CMN__StageTM::L1a)
      {
        m_Traitement.AjouterDonneesAux(nomFichierContexte);
      }
      
      //@ appel a la fonction LibererGranule
      LibererGranule();
    }

  //@ Traitement des exceptions
  catch (ERR__Exception& exception)
    {
      //@ appel a la fonction LibererGranule
      LibererGranule();

      //@ Re-emission de l'exception
      exception.SetDebParametre(ERROR_END_TREATMENT);
      throw(exception);
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : ErreurGranule
//@
//@ Description : Cette methode permet de terminer le traitement d'un 
//@               granule de IASI en erreur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestDonnees::ErreurGranule()
{
  //@ appel a la fonction SD_DON_DonGenGranule::FinCalcul
  m_DonGenGranule.FinCalcul();

  //@ appel a la fonction LibererGranule
  LibererGranule();
}


//@ ------------------------------------------------------------------*/
//@ Methode : LibererGranule
//@
//@ Description : Cette methode permet de cloturer le traitement
//@               d'un granule et de liberer les donnees dynamiques 
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestDonnees::LibererGranule()
{
  double elapsedTime = 0;
  double userTime    = 0;
  double systemTime  = 0;
  t_Date dateCourante;


  //@ L'objet fin traitement a ete mis a jour au fur et a mesure du 
  //@ traitement

  //@ Mesure de temps
  MesurerTemps(CMN__FAUX, &elapsedTime, &userTime, &systemTime);
      
#ifdef OPS_DEBUG
  cout << "FinGranule: " << endl;
  cout << "\telapsedTime: " << elapsedTime << endl;
  cout << "\tuserTime   : " << userTime << endl;
  cout << "\tsystemTime : " << systemTime << endl;
#endif
  
  //@ enregistrement elapsed et user time
  m_Traitement.ElapsedTime(elapsedTime);
  m_Traitement.UserTime(userTime);
  
  //@ enregistrement date fin traitement
  dateCourante.DateCourante();
  m_Traitement.StopTime(dateCourante);
      
  //@ on libere ici les donnees dynamiques suivantes (delete):
  //@   - gestionnaire des granules
  if (m_pGestGranule)
    {
      m_pGestGranule->Liberer();
      delete m_pGestGranule;
      m_pGestGranule = 0;
    }

  //@   - SVM
  if (m_pSVM)
    {
      m_pSVM->LibererMutex();       
      delete m_pSVM;
      m_pSVM = 0;
    }

  //@   - gestionnaire des cas degrades
  m_GestCasDeg.Liberer();
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetSensingTime
//@
//@ Description : Recupere les informations SensingTime
//@
//@ Parametres :
//@               (ENTREE) numLigne
//@               (SORTIE) sensingTime
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestDonnees::GetSensingTime(const int numLigne, 
                                        t_TableauDate& sensingTime)
{
  //@ Appel a la methode SD_DON_GestGranule::GetSensingTime()
  m_pGestGranule->GetSensingTime(numLigne, sensingTime);
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetTraitement
//@
//@ Description : Cette methode permet de recuperer l'objet traitement
//@ pour son initialisation
//@
//@ Parameters :
//@ Return Status : SD_DON_Traitement &
//@
//@ ------------------------------------------------------------------*/
SD_DON_Traitement & SD_DON_GestDonnees::GetTraitement()
{
  return(m_Traitement); 
}


//@ ------------------------------------------------------------------*/
//@ Methode : SetTraitement
//@
//@ Description : Cette methode permet de positionner l'objet traitement
//@
//@ Parameters :
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestDonnees::SetTraitement(
  const SD_DON_Traitement & traitement)
{
  m_Traitement = traitement; 
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetLigne0
//@
//@ Description : Cette methode permet de recuperer l'adresse d'une ligne
//@ de donnees de niveau 0 en entree
//@ pour son utilisation par les traitement
//@ on ne retourne ici que les paquet coherents (cf CG 13 14)
//@
//@ Parameters :
//@     (ENTREE)  NumLigne : numero de la ligne demandee
//@     (ENTREE)  Type : type des paquets choisi
//@     (SORTIE)  NbPaquets : Nombre de paquets retournes
//@     (ENTREE/SORTIE)  ListePaquetLigne : Liste des paquets retournes
//@     (liste de SD_DON_PaquetN0)
//@
//@ Return Status :   aucun
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestDonnees::GetLigne0(
    int NumLigne,
    e_SD_BAS_TypeN0 type ,
    int NbPaquets,
    t_SD_BAS_PaquetN0** ListePaquetLigne 
    )
{
  //@ appel a la methode SD_DON_GestGranule::GetLigne0
  m_pGestGranule->GetLigne0(NumLigne, 
                            type, 
                            NbPaquets, 
                            ListePaquetLigne);
}


//@ ------------------------------------------------------------------*/
//@ Methode : DecodeDataLigne0
//@
//@ Description : Cette methode permet de decoder un
//@ paquet de niveau 0 contenue dans une ligne ( le decodage se limite ici
//@ a l'extraction du champ de bit correspondant au spectre
//@
//@
//@ Parameters :
//@     (ENTREE)  Paramdecodage : parametres de decodage
//@     (ENTREE)  PaquetLigne : paquet ISP issu de GetLigne0
//@     (SORTIE)  DataDecode : donnees decodees allouee par
//@     l'appelant
//@
//@ Return Status : 
//@     Aucune
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestDonnees::DecodeDataLigne0(
    t_SD_BAS_PaquetN0* PaquetLigne,
    void * DataDecode // images ou spectres alloues par l'appelant
    )
{
  //@ decodage du paquet courant de la ligne
  m_pGestGranule->DecodeDataLigne0(PaquetLigne, DataDecode);
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetLigneS1
//@
//@ Description : Cette methode permet de recuperer l'adresse d'une ligne
//@ de donnees de niveau 1 en entree
//@ pour son utilisation par les traitement
//@
//@ Parameters :
//@     (ENTREE)  NumLigne : numero de la ligne demandee
//@     (ENTREE)  Type : type de la ligne demandee (ENG,1A,...)
//@     (SORTIE)  MPHRN1 :  MPHR produit N1
//@     (alloue par l'appelant)
//@     (SORTIE)  LigneMDR : ligne retournee
//@     (alloue par l'appelant)
//@     (SORTIE)  LigneVIADR : ligne retournee
//@     (pointeur alloue par l'appelant)
//@     ( remarque si = NULL alors rien n'est retourne )
//@     ( remarque si = NULL en sortie alors cela signifie  )
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestDonnees::GetLigneS1(
    int NumLigne,
    SD_DON_GranuleN1::e_SD_DON_GranuleN1 type ,
    t_SD_BAS_MPHR& Mphr,
    t_SD_BAS_LigneN1& LigneMDR,
    t_SD_BAS_LigneN1& LigneVIADR,
    t_CMN__Booleen&  transition
    )
//CR271
{
  //@ appel a la methode SD_DON_GestGranule::GetLigneS1
  m_pGestGranule->GetLigneS1(NumLigne, 
                             type, 
                             Mphr, 
                             LigneMDR, 
                             LigneVIADR);
//CR271
  transition=m_pGestGranule->EstTransitionLigneS1(NumLigne, 
                             type);
}


//@ ------------------------------------------------------------------*/
//@ Methode : EncodeDecodeLigneS1
//@
//@ Description : Cette methode permet de decoder les donnees detaillees 
//@ contenue dans une ligne de niveau 1 
//@
//@
//@ Parameters :
//@     (ENTREE)  numligneSNPN : numero  SN PN a traiter
//@     (ENTREE)  Ligne : ligne issue appel GetDataLigne
//@     (alloue par l'appelant)
//@     (ENTREE)  niveau : info a traiter (IMAGE/1A/1B/1C)
//@     (ENTREE)  Paramdecodage : parametres de decodage
//@     (ENTREE)  sens : sens decodage/encodage
//@     (ENTREE/SORTIE)  Data : donnees decodees
//@ Return Status : 
//@     Aucune
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestDonnees::EncodeDecodeLigneS1(
    t_SD_BAS_ORIGINE numlignesnpn, 
    t_SD_BAS_LigneN1& Ligne,
    e_SD_BAS_Atraiter niveau, 
    t_SD_BAS_ConfStable& Paramdecodage,
    const e_SD_BAS_Sens sens, // sens (decodage/encodage)
    double& Data // donnees decodees
    )
{
  //@ decodage encode de la ligne
//CR271 on encode pas dans le cas transition
  t_CMN__Booleen transition;
  transition=m_pGestGranule->EstTransitionLigneS1((numlignesnpn.LN)-1, 
                             SD_DON_GranuleN1::SD_DON_GRANULE_N1_DATA);
			   
  if (transition == CMN__FAUX)
     m_pGestGranule->EncodeDecodeLigneS1(numlignesnpn, 
                                      &Ligne, 
                                      niveau, 
                                      &Paramdecodage, 
                                      sens, 
                                      &Data);
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetNbLignesAVHRR
//@
//@ Description : Cette methode permet de recuperer le nombre de lignes 
//@ AVHRR utiles entre 2 dates
//@
//@ Parameters :
//@     
//@ Return Status : 
//@     NbLignes 
//@
//@ ------------------------------------------------------------------*/
int  SD_DON_GestDonnees::GetNbLignesAVHRR(const t_Date t0,
                                          const t_Date t1)
{
  //@ appel SD_DON_GestGranule::GetNbLigneAVHRR pour recuperer 
  //@ le nombre de lignes
  return(m_pGestGranule->GetNbLigneAVHRR(t0, t1));
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetNbLignesIASI
//@
//@ Description : Cette methode permet de recuperer le nombre de lignes 
//@ du granule IASI en entree
//@
//@
//@ Parameters :  aucun
//@
//@ Return Status : 
//@     Nblignes
//@
//@ ------------------------------------------------------------------*/
int  SD_DON_GestDonnees::GetNbLignesIASI()
{
  //@ recuperation du nbre de lignes donnee generales de la ligne
  return(m_pGestGranule->GetNbLigneIASI());
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetNbPaquetN0
//@
//@ Description : Cette methode permet de recuperer le nombre de paquets N0
//@
//@ Parameters :
//@       (ENTREE) noLigne : Numero de la ligne demandee
//@       (ENTREE)  type : Type des paquets choisi
//@ Return Status : 
//@      Nombre de paquets N0 
//@
//@ ------------------------------------------------------------------*/
int SD_DON_GestDonnees::GetNbPaquetsN0(int noLigne,
                                       e_SD_BAS_TypeN0 type)
{
  //@ appel SD_DON_GestGranule::GetNbPaquetsN0 pour recuperer 
  //@ le nombre de paquets
  return(m_pGestGranule->GetNbPaquetN0(noLigne, type));
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetBandeauAVHRR
//@
//@ Description : Cette methode permet de recuperer les lignes 
//@ AVHRR utiles entre 2 dates
//@
//@ Parameters :
//@     (ENTREE)  t0 : date de debut des  lignes 
//@     (ENTREE)  t1 : date de fin des  lignes
//@     (ENTREE)  numlig : numero de la ligne IASI pour
//@     laquelle la donnee est demandee
//@     (ENTREE)  numsn : numero du SN de l'image IASI pour
//@     laquelle la donnee est demandee
//@     (ENTREE)  GestCasDeg : Gestionnaire de cas degrade
//@     (ENTREE)  nbLigneMax : nb lignes AVHRR allouees
//@     (entrees/SORTIE)  NbLignes : Nombre de lignes allouees/retournes
//@     (ENTREE/SORTIE)  ListeLignes : Liste des lignes retournees
//@     (tableau de pointeurs alloues par l'appelant)
//@     ( remarque si = NULL alors rien n'est retourne )
//@     (alloue par l'appelant)
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_DON_GestDonnees::GetBandeauAVHRR(
    t_Date & t0,
    t_Date & t1,  
    const int  numlig,
    const int  numlsn,
    SD_DON_GestCasDeg & GestCasDeg,
    const int NbLignesmax,
    int& nbLigne,
    t_SD_BAS_LigneAVHRR ** ListeLignes
    )
{
  t_CMN__Statut cr_op=CMN__STATUT_OK;
   nbLigne = 0;


  //@ appel a la methode SD_DON_GestGranule::GetBandeauAVHRR
  // FT-2939
  m_pGestGranule->GetBandeauAVHRR(
    t0, 
    t1, 
    numlig, 
    numlsn, 
    GestCasDeg, 
    NbLignesmax,
    m_pSVM, 
    nbLigne, 
    ListeLignes);
  // ..FT-2939

  return(cr_op);
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetDonGenGranule
//@
//@ Description : Cette methode permet de recuperer l'adresse des 
//@ donnees generales au granules. Elles sont de deux types
//@   - donnees associees au contexte qui peuvent etre lues et modifiees
//@    pour les listes passer par AjouteElemHist
//@   - donnees precalculees qui ne peuvent etre que lues 
//@    
//@  
//@
//@
//@ Parameters :
//@     (ENTREE)  contextecourant :  contexte courant 
//@     alloue par l'appelant
//@     (SORTIE)  datagen : pointeur sur les donnees generales
//@
//@ Return Status :    aucun
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestDonnees::GetDonGenGranule(
    t_SD_BAS_Contexte ** contextecourant,
    t_SD_BAS_DataGen  ** datagen
    )
{
  //@
  //@ appel a SD_DON_DonGenGranule::GetData
  m_DonGenGranule.GetData(contextecourant, datagen);
}


//@ ------------------------------------------------------------------*/
//@ Methode : AjouteElemHist
//@
//@ Description : Cette methode permet d'ajouter un element dans l'historique
//@ du contexte( axe interf et temperature )
//@  
//@
//@
//@ Parameters :
//@     (ENTREE)  AxeInterf  : position de l'axe interf (dimension SD_BAS_CCD)
//@     (ENTREE)  TempCN : temperature du corps noir
//@   si un des element est NULL rien n'est fait
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestDonnees::AjouteElemHist(
    t_SD_BAS_HistAxeInterf** AxeInterf,
    t_SD_BAS_HistTempCN* TempCN
    )
{
  //@
  //@ appel a SD_DON_DonGenGranule::AjouteElemHist
  m_DonGenGranule.AjouteElemHist(AxeInterf, TempCN);
}


//@ ------------------------------------------------------------------*/
//@ Methode : MajContexte
//@
//@ Description : Cette methode permet de mettre a jour le contexte courant
//@    
//@  
//@
//@
//@ Parameters :
//@     (ENTREE)  contextecourant : pointeur sur le contexte
//@           courant modifie
//@
//@ Return Status : 
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestDonnees::MajContexte(
    const t_SD_BAS_Contexte * contextecourant
    )

{
// Entree dans la fonction 
  //@ appel SD_DON_DonGenGranule::MajContexte
  m_DonGenGranule.MajContexte(contextecourant);
// on trace ici si besoin toutes les variables en sortie

// sortie de la fonction 
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetConf
//@
//@ Description : Cette methode permet de recuperer dans les traitements 
//a l'adresse memoire des donnees de configuration a utiliser 
//@ . Ici les fichiers de conf sont deja charges et la zone memoire
//@ a ete prealablement allouee , les pointeurs sont alloue par l'appelant 
//@
//@
//@ Parameters :  
//@     (ENTREE)  Ptsi : Ptsi courante
//@     (ENTREE)  tcourante : date courante 
//@     (SORTIE)  ConfAutre : pointeur sur le 
//@         fichier de conf autre
//@     (SORTIE)  ConfStable : pointeur sur le 
//@         fichier de conf autre
//@     (SORTIE)  BanqueSpectrale : pointeur sur le 
//@         fichier BanqueSpectrale
//@
//@ Return Status :  aucun
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestDonnees::GetConf(
    const int Ptsi,
    const t_Date & tcourante,
    t_SD_BAS_ConfAutre **  ConfAutre,
    t_SD_BAS_ConfStable ** ConfStable,
    t_SD_BAS_BanqueSpectrale ** BanqueSpectrale
      )

{
// Entree dans la fonction 

  //@ on recupere la donnee de conf autre courante
  m_InventaireConf.GetConf(Ptsi,
                           tcourante,
                           ConfAutre,
                           ConfStable,
                           BanqueSpectrale
                           );

// on trace ici si besoin toutes les variables en sortie

// sortie de la fonction  

}
//@ ------------------------------------------------------------------*/
//@ Methode : GetGestCasDeg
//@
//@ Description : Cette methode permet de recuperer le gestionnaire 
//@ de cas degrades
//@
//@ Parameters :
//@ Return Status : SD_DON_GestCasDeg &
//@
//@ ------------------------------------------------------------------*/
SD_DON_GestCasDeg & SD_DON_GestDonnees::GetGestCasDeg()
{
  //@ return m_GestCasDeg
  return(m_GestCasDeg); 
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetDateEclipse
//@
//@ Description : Cette methode permet de recuperer la date de fin
//@ de la dernier eclipse 
//@
//@
//@ Parameters :
//@     (ENTREE)  dateUTC : date UTC corrigee
//@     si aucune date n'est trouvee on retourne une date
//@     initialisee
//@
//@ Return Status :   date UTC de l'eclipse precedente
//@
//@ ------------------------------------------------------------------*/
t_Date SD_DON_GestDonnees::GetDateEclipse(const t_Date & dateUTC)
{
  t_Date tprec;
  t_Date dateTmp(dateUTC);

  // FT-2449
  //@ appel a SD_DON_SVM::GetDateEclipse pour la date de fin d'eclipse
  m_pSVM->GetDateEclipse( dateTmp, tprec);
  // ..FT-2449

  return(tprec);
}


//======================================================================
//          METHODES PRIVEES 
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : InitEnvironnement
//@
//@ Description : Cette methode permet d'initialiser les variables
//@               d'environnement pour les donnees auxiliaires
//@
//@ Parametres :
//@               (ENTREE) prefixeDonneesAux
//@               (ENTREE) varEnvironnement
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestDonnees::InitEnvironnement(t_Chaine prefixeDonneesAux,
                                           t_Chaine varEnvironnement) 
  throw (ERR__Exception)
{
  // Exception generee
  ERR__Exception          exception;

  // Message pour l'exception
  t_Chaine                messageException("");

  // Iterateur pour la liste des fichiers
  t_ListeChaine::iterator pInfo;

  // Temoin de recherche
  t_CMN__Booleen          trouve = CMN__FAUX;

  // Nom du fichier
  t_Chaine                nomFichier("");

  // Nom du repertoire
  t_Chaine                nomRepertoire("");

  // Variable environnement
  t_Chaine                chaineEnv("");

//fa 115
// verification de l'existences de fichiers aux
  // Code retour des fonctions utilisees
  t_CMN__Booleen          status = CMN__FAUX;
  t_ListeChaine           listeFichier;
  pInfo = m_Traitement.ListeDonneesAux().begin();
  while (pInfo != m_Traitement.ListeDonneesAux().end() )
    {
      //@ Rechercher le fichier physique
      status = CMN__RechercherFichier(*pInfo,
                                      CMN__FAUX,
                                      listeFichier);
      //@ SI le fichier est de type recherche et n'existe pas 
      if (EstDeType(prefixeDonneesAux, *pInfo) == CMN__STATUT_OK && status == CMN__FAUX)
        {
          //@ Emission d'une exception SD_DON_TMT_DATA_IN
          exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_AUX);
          exception.SetParametre(MISSING_AUX_FILE + prefixeDonneesAux +" "+ *pInfo);
  //FA 115 : utilisation de attribut pour gerer les pb de donnees
          THROW(exception);
        }
        pInfo++;
      //@ FIN SI le fichier est de type recherche
    }
  //@ FIN TANT QUE


  //@ TANT QUE il existe des donnees auxiliaires ET QUE le
  //@ fichier recherche n'a pas ete trouve
  pInfo = m_Traitement.ListeDonneesAux().begin();
  while (pInfo != m_Traitement.ListeDonneesAux().end() && 
         (trouve == CMN__FAUX))
    {
      //@ SI le fichier est de type recherche
      if (EstDeType(prefixeDonneesAux, *pInfo) == CMN__STATUT_OK)
        {
          //@ Le fichier recherche a ete trouve
          //@ Mettre a jour le temoin de recherche
          trouve = CMN__VRAI;
        }
      //@ SINON
      else
        {
          //@ Passer a la donnee auxiliaire suivante
          pInfo++;
        }
      //@ FIN SI le fichier est de type recherche
    }
  //@ FIN TANT QUE

  //@ SI le fichier recherche n'a pas ete trouve
  if (trouve == CMN__FAUX)
    {
      //@ Emission d'une exception SD_DON_TMT_DATA_AUX
      exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_AUX);
      exception.SetParametre(MISSING_AUX_FILE + prefixeDonneesAux);
      THROW(exception);
    }
  //@ FIN SI le fichier recherche n'a pas ete trouve


  //@ Recuperation du nom du fichier
  nomFichier = *pInfo;
  
  //@ Recuperation du repertoire de la banque spectrale
  nomRepertoire = nomFichier.substr(0, nomFichier.
                                    rfind(CMN__SEP_REPERTOIRE));
  
  //@ Mise a jour variable d'environnement REP_AUX_DATA
  chaineEnv = CFG__Configuration::CFG__PREFIXE_REPERTOIRE +
    varEnvironnement;
  
  //@ SI la mise a jour n'a pas ete possible 
  if (setenv((char*)chaineEnv.c_str(), (char*)nomRepertoire.c_str(), 1) 
      != 0)
    {
      //@ Emission d'une exception SD_ENV_UNDEFINED
      exception.SetErrorCode(ERR__Exception::SD_ENV_UNDEFINED);
      exception.SetParametre(ENV_UNDEFINED+varEnvironnement);
      THROW(exception);
    }
  //@ FIN SI la mise a jour n'a pas ete possible 
}


//@ ------------------------------------------------------------------*/
//@ Methode : EstDeType
//@
//@ Description : Teste si un fichier est du type passe en parametre
//@
//@ Parametres :
//@               (ENTREE) type : Type de fichier
//@               (ENTREE) nomFichier : Nom du fichier
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK si le fichier est du type donne
//@               CMN__STATUT_NOK sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_DON_GestDonnees::EstDeType(const t_Chaine& type,
                                             const t_Chaine& nomFichier)
{
  // Racine du nom du fichier
  t_Chaine       racine("");

  // Valeur de retour de la fonction
  t_CMN__Statut cr_op = CMN__STATUT_NOK;


  //@ Recuperation du debut du nom court du fichier
  racine = nomFichier.substr(nomFichier.rfind(CMN__SEP_REPERTOIRE)+1, 
                             type.length());

  //@ Comparaison de la chaine recuperee avec le type
  //@ SI les deux chaines sont identiques
  if (racine == type)
    {
      //@ Positionner la valeur de retour a VRAI
      cr_op = CMN__STATUT_OK;
    }
  //@ FIN SI les deux chaines sont identiques


  //@ Retourner la valeur de retour de la fonction
  return cr_op;
}


//@ ------------------------------------------------------------------*/
//@ Methode : MesurerTemps
//@
//@ Description : Mesure le temps d'execution
//@
//@ Parametres :
//@               (ENTREE) init : Initialisation ou non du temps
//@               (SORTIE) elapsedTime
//@               (SORTIE) userTime
//@               (SORTIE) systemTime
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestDonnees::MesurerTemps(t_CMN__Booleen init,
                                     double *elapsedTime, 
                                     double *userTime, 
                                     double *systemTime)
{
  static long     zsec = 0;
  static long     zusec = 0;
  static double   user = 0;
  static double   system = 0;

  struct timeval  tpCourant;
  struct timezone tzpCourant;
  struct tms      tmsCourant;

  long   ticks;

  //@ Recuperation du temps CPU
  gettimeofday(&tpCourant, &tzpCourant);
  times(&tmsCourant);
  
  //@ recuperation ticks
  ticks = sysconf(_SC_CLK_TCK);

  //@ SI initialisation demandee
  if (init == CMN__VRAI)
    {
      //@ Initialisation des variables
      zsec         = tpCourant.tv_sec;
      zusec        = tpCourant.tv_usec;
      user         = tmsCourant.tms_utime;
      system       = tmsCourant.tms_stime;
      *elapsedTime = 0;
      *userTime    = 0;
      *systemTime  = 0;
    }
  //@ SINON
  else
    {
      //@ Calcul elapsed time
      *elapsedTime = (tpCourant.tv_sec - zsec) + 
        (tpCourant.tv_usec - zusec) * 0.000001; 

      //@ Calcul user time et system time
      *userTime    = (tmsCourant.tms_utime - user) / ticks; 
      *systemTime  = (tmsCourant.tms_stime - system) / ticks;
    }
  //@ FIN SI
}

//@ ------------------------------------------------------------------*/
//@ Methode : MesurerTemps
//@
//@ Description : Mesure le temps d'execution
//@
//@ Parametres :
//@               (ENTREE) init : Initialisation ou non du temps
//@               (SORTIE) elapsedTime
//@               (SORTIE) userTime
//@               (SORTIE) systemTime
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestDonnees::InitialiserGestCasDeg(const int& ptsi)
{
SD_DON_CasDegLigne::t_SD_DON_IdefDur tableauDuree;
t_SD_BAS_ConfStable** ConfStable;
ConfStable = new t_SD_BAS_ConfStable*[1];
t_SD_BAS_ConfAutre** ConfAutre;
ConfAutre = new t_SD_BAS_ConfAutre*[1];
t_SD_BAS_BanqueSpectrale** BanqueSpectrale;
BanqueSpectrale = new t_SD_BAS_BanqueSpectrale*[1];
  //@ Chargement du fichier de conf autre
  t_Date dateTmp;
  dateTmp.DateCourante();
  m_InventaireConf.GetConf(ptsi,
                           dateTmp,
                           ConfAutre,
                           ConfStable,
                           BanqueSpectrale);

  //@initialisation des durees a partir du fichier de conf autre
  tableauDuree[2].ConvertirTempsEnDate(ConfAutre[0]->IDefDurFltIsrfemOff);
  tableauDuree[3].ConvertirTempsEnDate(ConfAutre[0]->IDefDurBandMiss);
  tableauDuree[4].ConvertirTempsEnDate(ConfAutre[0]->IDefDurBBTMiss);
  tableauDuree[5].ConvertirTempsEnDate(ConfAutre[0]->IDefDurImgEWMiss);
  tableauDuree[6].ConvertirTempsEnDate(ConfAutre[0]->IDefDurImgBBMiss);
  tableauDuree[7].ConvertirTempsEnDate(ConfAutre[0]->IDefDurImgCSMiss);
  tableauDuree[8].ConvertirTempsEnDate(ConfAutre[0]->IDefDurRadAvhrrMiss);
  
  //@ initialisation du gestionnaire
  m_GestCasDeg.Init(m_DonGenGranule.GetContexte(), tableauDuree);
  // FT-2512
  delete []ConfStable;
  delete []ConfAutre;
  delete []BanqueSpectrale;
  // ..FT-2512
}


  
