// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_GestDonnees.h,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_GestDonnees
//@
//@ Comment :
//@    this module is the unique entry point to the IASI AVHRR 
//@    persitent data used bas IASI L1 Processing chain
//@    this ensure the availability and the coherence of
//@    the data user/produced  among the multithreaded acces
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 23/01/2003 : P.GASSIE
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-158-CN : 30/06/2005 : JL PASCAL
//@ L'OPS genere des resulta VIADRENG_MEXSSMIN et MAX errone et qui  
//@ dependent du nombre de thread 
//@ Voir IA-2100-FC-158-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-271-CN : 22/06/2005 : JL.PASCAL
//@ prise en compte du mode transition
//@ fiche de correction correspondante : IA-2100-CR-271-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-FT-2466(FA-2100-OPS) : 30/05/2008 : S. BRUNEL
//@ Probleme de reproductibilite: SUBSAT_LATITUDE_START et
//@ SUBSAT_LONGITUDE_START
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_DON_GestDonnees_h
#define SD_DON_GestDonnees_h


// Inserer ici les fichiers include 
//FA158
#include <pthread.h>
#include <unistd.h>
#include <time.h>
#include <sys/times.h>
#include <sys/time.h>
#include "SD_DON_GestCasDeg.h"
#include "SD_DON_GestGranule.h"
#include "SD_DON_DonGenGranule.h"
#include "SD_DON_InventaireConf.h"
#include "SD_BAS_Global.h"
#include "SD_DON_SVM.h"
#include "SD_DON_Traitement.h"
#include "CMN__Date.h"
#include "CMN__OPSFile.h"
#include "ERR__Exception.h"


// Definition des exceptions

class SD_FRW_ApplicationServeurDonnees;
class SD_DON_GestDonnees {

    /*================================================================*/

 public:

    //
    // Type
    //  

    //
    // Constant
    //
    //
    // Constructeur
    //
    SD_DON_GestDonnees();

    //
    // Destructeur
    //
    virtual ~SD_DON_GestDonnees();

    //
    // Accesseurs
    //
    SD_DON_Traitement & GetTraitement();
    void SetTraitement(const SD_DON_Traitement & traitement);
    SD_DON_GestCasDeg & GetGestCasDeg();


    //
    // Methodes publiques
    //
    void InitDump();
    void InitGranule(t_TableauDate& startTime, 
                     t_TableauEntier& startOrbitNumber) 
      throw (ERR__Exception);
    void FinGranule() throw (ERR__Exception);
    void ErreurGranule();
    void LibererGranule();

    void GetSensingTime(const int numLigne, t_TableauDate& sensingTime);
    void GetLigne0(
                   int NumLigne,
                   e_SD_BAS_TypeN0 type ,
                   int NbPaquets,
                   t_SD_BAS_PaquetN0** ListePaquetLigne
                   );
    void DecodeDataLigne0(
                          t_SD_BAS_PaquetN0* PaquetLigne,
                          void * DataDecode // images ou spectres alloues par l'appelant
                          );

    void GetLigneS1(
                    int NumLigne,
                    SD_DON_GranuleN1::e_SD_DON_GranuleN1 type ,
                    t_SD_BAS_MPHR& Mphr,
                    t_SD_BAS_LigneN1& LigneMDR,
                    t_SD_BAS_LigneN1& LigneVIADR,
		    t_CMN__Booleen& transition = m_transition // CR271
                    );
    void EncodeDecodeLigneS1(
                             t_SD_BAS_ORIGINE numlignesnpn, 
                             t_SD_BAS_LigneN1& Ligne,
                             e_SD_BAS_Atraiter niveau, 
                             t_SD_BAS_ConfStable& Paramdecodage,
                             const e_SD_BAS_Sens sens, // sens (decodage/encodage)
                             double& Data // donnees decodees
                             );
    int GetNbPaquetsN0(int noLigne,
                       e_SD_BAS_TypeN0 type
                       );
    int  GetNbLignesAVHRR(const t_Date t0,
                          const t_Date t1
                          );
    int  GetNbLignesIASI();

    t_CMN__Statut GetBandeauAVHRR(
                                  t_Date & t0,
                                  t_Date & t1,  
                                  const int  numlig,
                                  const int  numlsn,
                                  SD_DON_GestCasDeg & GestCasDeg,
                                  const int NbLignesmax,
                                  int& nbLigne,
                                  t_SD_BAS_LigneAVHRR ** ListeLignes
                                  );
    void GetDonGenGranule(
                          t_SD_BAS_Contexte ** contextecourant,
                          t_SD_BAS_DataGen  ** datagen
                          );
    void AjouteElemHist(
                        t_SD_BAS_HistAxeInterf** AxeInterf,
                        t_SD_BAS_HistTempCN* TempCN
                        );
    void MajContexte(
                     const t_SD_BAS_Contexte * contextecourant
                     );

    void GetConf(
                 const int Ptsi,
                 const t_Date & tcourante,
                 t_SD_BAS_ConfAutre **  ConfAutre,
                 t_SD_BAS_ConfStable ** ConfStable,
                 t_SD_BAS_BanqueSpectrale ** BanqueSpectrale
                 );
    t_Date GetDateEclipse(
                          const t_Date & dateUTC
                          );
	// FT-2466
	t_CMN__Statut  SetEnteteS1(
                               t_SD_BAS_StatMphr  * StatMPHR,
                               t_SD_BAS_ConfStable  * ParamConfStable,
                               int numeroLigne
                               );
    // ..FT-2466
//FA158
    void InitialiserMutex();
    void LibererMutex();
    void Reserver();
    void Liberer();

    /*================================================================*/
 protected:
    //
    // protected Methods 
    //

    //
    // protected Attributes 
    //

    /*====================================================================*/
 private:
    //
    // Constante
    //
    static const t_Chaine VERSION_CLASSE;
    static const t_Chaine PREFIXE_SVM;
    static const t_Chaine DATA_AUX_SVM_NOT_FOUND;
    static const t_Chaine PREFIXE_BANQUE_SPECTRALE;
    static const t_Chaine PREFIXE_STABLE_SCF;
    static const t_Chaine PREFIXE_STABLE_OCF;
    static const t_Chaine PREFIXE_CONTEXTE;
    static const t_Chaine MISSING_AUX_FILE;
    static const t_Chaine ENV_UNDEFINED;
    static const t_Chaine ERROR_END_TREATMENT;

    // Version de l'OPS 
    static const t_SD_BAS_VERSION  SD_BAS_OPSVERSION;

    //
    // Private Methods 
    //
    void InitEnvironnement(t_Chaine prefixeDonneesAux,
                           t_Chaine varEnvironnement) 
        throw (ERR__Exception);
    t_CMN__Statut EstDeType(const t_Chaine& type, 
                            const t_Chaine& nomFichier);
    void MesurerTemps(t_CMN__Booleen init,
                      double *elapsedTime, 
                      double *userTime, 
                      double *systemTime);
    void InitialiserGestCasDeg(const int& ptsi);

    //
    // Private Attributes 
    //
  
    // Objet Inventaire des donnes de conf 
    SD_DON_InventaireConf  m_InventaireConf;
    // identificateur du fichier de conf autre 
    int  m_IdCA;
  

    // Objet traitement ( issu worder) 
    SD_DON_Traitement  m_Traitement;
  
    // Objet gestionnaire granule 
    SD_DON_GestGranule*  m_pGestGranule;
  
    // Objet donnees generales granules 
    SD_DON_DonGenGranule  m_DonGenGranule;
  
    // Objet gestionnaire cas degrade 
    SD_DON_GestCasDeg  m_GestCasDeg;

    // Objet SVM permettant la determination date fin eclipse
    SD_DON_SVM*  m_pSVM;

    // date de traitement courante
    t_Date  m_datecourante;
    //FA158
    // Mutex pour la mise a jour du viadr 
    static pthread_mutex_t   m_MutexTravaille;
    static t_CMN__Booleen m_transition; //CR271
};

#endif
