static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_GestGranule.cc,v 1.5 2013/03/29 16:55:26 lcaspar Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_GestGranule
//@
//@ Commentaire :
//@    Ce module gere la classe SD_DON_GestGranule, qui pilote la 
//@    lecture et l'ecriture des produits IASI N0, N1 et AVHRR N1
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 06/01/2003 : P. CABANE
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-159-CN : 26/05/2005 : JL. PASCAL
//@ modif suite a portage linux
//@ fiche de correction correspondante : IA-2100-FC-159-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-271-CN : 22/06/2005 : JL.PASCAL
//@ prise en compte du mode transition
//@ fiche de correction correspondante : IA-2100-CR-271-THA
//@ --------------------------------------------------------------------
//@ DM-ID :IA-2100-CR-286-CNE : 21/02/2006 : B. MASSART
//@ The environment variable DATE_MARGE_LIGNE_AVHRR is added. 
//@ See IA-2100-CR-286-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-183-CN : 08/08/2006 : C. MERIGUET
//@ Ne pas lever d'exception en mode dump si aucune lignes n'a ete trouvee
//@ pour empecher le rejet du work order global et l'arret de l'OPS en cas 
//@ de trou de donnees.
//@ --------------------------------------------------------------------
//@ DM-ID :IA-2100-CR-308-CNE : 12/09/2006 : C. MERIGUET
//@ Lever le cas degrade 9 si le(s) fichier(s) AVHRR n'est
//@ pas accessible ou mauvais numero de version. Ne pas lever
//@ d'exception et poursuivre le traitement.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-162-CN : 09/2006 : C.MERIGUET
//@ Gestion des IPR en mode DUMP
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FT-2454-CN : 07/04/2008 : S.BRUNEL
//@ Critere de choix du fichier SVM erronne
//@ --------------------------------------------------------------------
//@ FA-ID :IA-FT-2466(FA-2100-OPS) : 30/05/2008 : S. BRUNEL
//@ Probleme de reproductibilite: SUBSAT_LATITUDE_START et
//@ SUBSAT_LONGITUDE_START
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2512 (FA-2100-OPS) : 17/11/2008 : S. BRUNEL
//@ Fuites memoires signalees par EUMETSAT (EUM.EPS.NCR.3627.1)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2939(DM-2100-OPS) : 12/12/2011 : L. CASPAR
//@ Mise en place de conditions de levee du cas degarde 9 alternatives pour 
//@ correction de la NCR 12303
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL
#include <list>
#include <algorithm>

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN__Date.h"
#include "CMN__OPSFile.h"
#include "ERR__Exception.h"

#include "SD_DON_GranuleN0.h"
#include "SD_DON_LigneN0.h"
#include "SD_DON_GranuleN1.h"
#include "SD_DON_GranuleAVHRR.h"
#include "SD_DON_GestGranule.h"


//
// Definition des constantes
//
const t_Chaine SD_DON_GestGranule::JDB_PARAM_1               = "$PRM1";
//FA159
const int      SD_DON_GestGranule::DATE_MARGE                = 0;
const int      SD_DON_GestGranule::DATE_MARGE_CHARGEMENT_AVHRR     = 10;

/* CR286 */
const t_Chaine SD_DON_GestGranule::s_DATE_MARGE_LIGNE_AVHRR = "DATE_MARGE_LIGNE_AVHRR";
const int      SD_DON_GestGranule::DATE_MARGE_LIGNE_AVHRR    = 1;

const t_Chaine SD_DON_GestGranule::PREFIXE_OBTUTC            = "xxxx_OBT_xx_";
const t_Chaine SD_DON_GestGranule::PREFIXE_OSV               = "xxxx_OSV_xx_";
const t_Chaine SD_DON_GestGranule::PREFIXE_1A                = "IASI_xxx_1A_";
const t_Chaine SD_DON_GestGranule::PREFIXE_1B                = "IASI_xxx_1B_";
const t_Chaine SD_DON_GestGranule::PREFIXE_1C                = "IASI_xxx_1C_";
const t_Chaine SD_DON_GestGranule::PREFIXE_ENG               = "IASI_ENG_";
const t_Chaine SD_DON_GestGranule::PREFIXE_VER               = "IASI_VER_";

const t_Chaine SD_DON_GestGranule::DATA_IN_NOT_FOUND         = "Data in not found";
const t_Chaine SD_DON_GestGranule::DATA_IN_NOT_ENOUGH        = "Not enough data in";
const int      SD_DON_GestGranule::NB_DATA_IN_INTERMEDIAIRE  = 2;
const t_Chaine SD_DON_GestGranule::DATA_PROD_NOT_FOUND       = "Product model not found: ";
const t_Chaine SD_DON_GestGranule::DATA_AUX_OBTUTC_NOT_FOUND = "Valid OBTUTC aux data not found";


//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD_DON_GestGranule
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_GestGranule::SD_DON_GestGranule()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_DON_GestGranule
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_GestGranule::~SD_DON_GestGranule()
{
  //@ On libere ici les listes de granules
  m_ListeGranuleN0E.clear();
  m_ListeGranuleN1E.clear();
  m_ListeGranuleN1S.clear();
  m_ListeGranuleAVHRR.clear();
}


//@ ------------------------------------------------------------------*/
//@ Methode : Liberer
//@
//@ Description : Liberation memoire des donnees de l'objet
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestGranule::Liberer()
{
  //@ POUR tous les granules N0 en entree
  for (int i=0; i<m_ListeGranuleN0E.size(); i++)
    {
      //@ Liberation memoire des donnees
      m_ListeGranuleN0E[i]->Liberer();
    }
  //@ FIN POUR tous les granules N0 en entree
  
  //@ POUR tous les granules N1 en entree
  for (int i=0; i<m_ListeGranuleN1E.size(); i++)
    {
      //@ Liberation memoire des donnees
      m_ListeGranuleN1E[i]->Liberer();
    }
  //@ FIN POUR tous les granules N1 en entree
  
  //@ POUR tous les granules en sorties
  for (int i=0; i<m_ListeGranuleN1S.size(); i++)
    {
      //@ Liberation memoire des donnees
      m_ListeGranuleN1S[i]->Liberer();
      // FT-2512
      delete m_ListeGranuleN1S[i];
      // ..FT-2512
    }
  //@ FIN POUR tous les granules en sorties
  
  //@ POUR tous les granules AVHRR
  for (int i=0; i<m_ListeGranuleAVHRR.size(); i++)
    {
      //@ Liberation memoire des donnees
      m_ListeGranuleAVHRR[i]->Liberer();
      // FT-2512
      delete m_ListeGranuleAVHRR[i];
      // ..FT-2512
    }
  //@ FIN POUR tous les granules AVHRR
  
  //@ RAZ des listes
  m_ListeGranuleN0E.clear();
  m_ListeGranuleN1E.clear();
  m_ListeGranuleN1S.clear();
  m_ListeGranuleAVHRR.clear();
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetDateDebutIASI
//@
//@ Description : Recupere la date reelle de debut du granule IASI en
//@               entree
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Date de debut du granule IASI en entree
//@
//@ ------------------------------------------------------------------*/
t_Date SD_DON_GestGranule::GetDateDebutIASI() const throw (ERR__Exception)
{
  //@ SI la liste des granules en sortie est vide
  if (m_ListeGranuleN1S.size() == 0)
    {
      // Exception generee
      ERR__Exception exception;
      
      //@ Emission d'une exception SD_DON_OPER_FAILED
      exception.SetErrorCode(ERR__Exception::SD_DON_OPER_FAILED);
      THROW(exception);
    }
  //@ FIN SI la liste des granules en sortie est vide
  
  //@ Recuperation de la date reelle du debut du granule
  //@ Appel a la methode SD_DON_Granule::GetDates() a partir
  //@ du premier element de la liste des granules en sortie
  //@ On retourne alors la date UTC corrigee de debut
  return m_ListeGranuleN1S[0]->GetDateDebut();
}


// FT-2454
//@ ------------------------------------------------------------------*/
//@ Methode : GetDateFinIASI
//@
//@ Description : Recupere la date reelle de fin du granule IASI en
//@               entree
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Date de fin du granule IASI en entree
//@
//@ ------------------------------------------------------------------*/
t_Date SD_DON_GestGranule::GetDateFinIASI() const throw (ERR__Exception)
{
  //@ SI la liste des granules en sortie est vide
  if (m_ListeGranuleN1S.size() == 0)
    {
      // Exception generee
      ERR__Exception exception;
      
      //@ Emission d'une exception SD_DON_OPER_FAILED
      exception.SetErrorCode(ERR__Exception::SD_DON_OPER_FAILED);
      THROW(exception);
    }
  //@ FIN SI la liste des granules en sortie est vide
  
  //@ Recuperation de la date reelle de fin du granule
  //@ Appel a la methode SD_DON_Granule::GetDates() a partir
  //@ du premier element de la liste des granules en sortie
  //@ On retourne alors la date UTC corrigee de fin
  return m_ListeGranuleN1S[0]->GetDateFin();
}
// ..FT-2454


//@ ------------------------------------------------------------------*/
//@ Methode : GetNbLigneIASI
//@
//@ Description : Recupere le nombre de lignes du granule IASI en 
//@               entree
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Nombre de lignes du granule IASI en entree
//@
//@ ------------------------------------------------------------------*/
int SD_DON_GestGranule::GetNbLigneIASI() const
{
  // Nombre de lignes du granule IASI en entree
  int nbLigne = 0;


  //@ SI la liste des granules N0 en entree n'est pas vide
  if (m_ListeGranuleN0E.size() > 0)
    {
      //@ Recuperation du nombre de lignes (donnee generale 
      //@ de la ligne)
      //@ Appel a la methode SD_DON_Granule::GetNbLigne() a partir 
      //@ du premier element de la liste des granules en entree
      nbLigne = m_ListeGranuleN0E[0]->GetNbLigne();
    }
  //@ SINON SI la liste des granules N1 en entree n'est pas vide
  else if (m_ListeGranuleN1E.size() > 0)
    {
      nbLigne = m_ListeGranuleN1E[0]->GetNbLigne();
    }
  //@ FIN SI la liste des granules N0 en entree n'est pas vide


  //@ Renvoyer le nombre de lignes du granule IASI en entree
  return (nbLigne);
}




//@ ------------------------------------------------------------------*/
//@ Methode : GetNbDataGap
//@
//@ Description : Recupere le nombre de trous de donnees du  
//@               granule IASI en entree
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Nombre de trous de donnees du granule IASI en entree
//@
//@ ------------------------------------------------------------------*/
int SD_DON_GestGranule::GetNbDataGap() const
{
  // Nombre de trous de donnees du granule IASI en entree
  int nbDataGap = 0;


  //@ SI la liste des granules N0 en entree n'est pas vide
  if (m_ListeGranuleN0E.size() > 0)
    {
      //@ Recuperation du nombre de trous de donnees
      //@ Appel a la methode SD_DON_Granule::GetNbDataGap() a partir 
      //@ du premier element de la liste des granules en entree
      nbDataGap = m_ListeGranuleN0E[0]->GetNbDataGap();
    }
  //@ SINON SI la liste des granules N1 en entree n'est pas vide
  else if (m_ListeGranuleN1E.size() > 0)
    {
      nbDataGap = m_ListeGranuleN1E[0]->GetNbDataGap();
    }
  //@ FIN SI la liste des granules N0 en entree n'est pas vide


  //@ Renvoyer le nombre de trous de donnees du granule IASI en entree
  return (nbDataGap);
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetNbLigneAVHRR
//@
//@ Description : Recupere le nombre de lignes AVHRR utiles
//@
//@ Parametres :
//@               (ENTREE) t0 : Date de debut des lignes
//@               (ENTREE) t1 : Date de fin des lignes
//@
//@ Valeur de retour : 
//@               Nombre de lignes AVHRR
//@
//@ ------------------------------------------------------------------*/
int SD_DON_GestGranule::GetNbLigneAVHRR(const t_Date& t0, 
                                        const t_Date& t1)
{
  // Iterateur pour la liste des granules
  t_ListeGranuleAVHRR::iterator granuleAVHRR;

  // Nombre de lignes AVHRR
  int                           nbLigne = 0;


  //@ POUR chaque element de la liste des granules AVHRR
  for (granuleAVHRR = m_ListeGranuleAVHRR.begin();
       granuleAVHRR != m_ListeGranuleAVHRR.end(); 
       granuleAVHRR++)
    {
      //@ Appel de la methode SD_DON_GranuleAVHRR::GetNbLigneUtile()
      //@ pour recuperer le nombre de lignes

      //@ nbLigne += SD_DON_GranuleAVHRR::GetNbLignesUtiles()
      nbLigne += (*granuleAVHRR)->GetNbLignesUtiles(t0, t1);
    }
  //@ FIN POUR chaque element de la liste des granules AVHRR


  //@ Renvoyer le nombre de lignes AVHRR
  return (nbLigne);
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetNbPaquetN0
//@
//@ Description : Recupere le nombre de paquets N0 pour une ligne et
//@               un type donnes
//@
//@ Parametres :
//@               (ENTREE) noLigne : Numero de la ligne demandee
//@               (ENTREE) type : Type des paquets choisi
//@
//@ Valeur de retour : 
//@               Nombre de paquets N0
//@
//@ ------------------------------------------------------------------*/
int SD_DON_GestGranule::GetNbPaquetN0(const int noLigne,
                                      const e_SD_BAS_TypeN0 type)
{
  // Nombre de paquets N0
  int nbPaquetN0 = 0;
  
  
  //@ SI la liste des granules N0 en entree n'est pas vide
  if (m_ListeGranuleN0E.size() > 0)
    {
      //@ Appel de la methode SD_DON_GranuleN0::GetNbPaquetN0()
      //@ On recupere ainsi le nombre de paquets associes a
      //@ la ligne pour un type precis
      nbPaquetN0 = m_ListeGranuleN0E[0]->GetNbPaquetN0(noLigne, type);
    }
  //@ FIN SI la liste des granules N0 en entree n'est pas vide

  
  //@ Renvoyer le nombre de paquets N0
  return (nbPaquetN0);
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetInfoHKTM
//@
//@ Description : Recupere les informations pour les HKTM status
//@
//@ Parametres :
//@               (SORTIE) startTime
//@               (SORTIE) startOrbitNumber
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestGranule::GetInfoHKTM(t_TableauDate& startTime, 
                                     t_TableauEntier& startOrbitNumber)
{
  int    startOrbitNumberE = 0;
  int    startOrbitNumberS = 0;
  t_Date startTimeE;
  t_Date startTimeS;


  startTimeE.DateInit();
  startTimeS.DateInit();

  //@ SI la liste des granules N0 en entree n'est pas vide
  if (m_ListeGranuleN0E.size() > 0)
    {
      //@ Appel de la methode SD_DON_Granule::GetInfoHKTM()
      m_ListeGranuleN0E[0]->GetInfoHKTM(startTimeE, 
                                        startOrbitNumberE);
    }
  //@ SINON SI la liste des granules N1 en entree n'est pas vide
  else if (m_ListeGranuleN1E.size() > 0)
    {
      m_ListeGranuleN1E[0]->GetInfoHKTM(startTimeE, 
                                        startOrbitNumberE);
    }
  //@ FIN SI la liste des granules N0 en entree n'est pas vide


  //@ POUR tous les granules en sorties
  for (int i=0; i<m_ListeGranuleN1S.size(); i++)
    {
      //@ Appel de la methode SD_DON_Granule::GetInfoHKTM()
      m_ListeGranuleN1S[i]->GetInfoHKTM(startTimeS, 
                                        startOrbitNumberS);

      //@ Ajout des infos dans les tableaux passes en parametre
      startTime.push_back(startTimeS);
      startOrbitNumber.push_back(startOrbitNumberE);

      //@ SI granule de donnees
      if (m_ListeGranuleN1S[i]->GetNatureGranule() == 
          SD_DON_GranuleN1::SD_DON_GRANULE_N1_DATA)
        {
          //@ On recopie deux fois de plus les infos 
          //@ (1 GRANULE DATA = 3 produits)
          startTime.push_back(startTimeS);
          startTime.push_back(startTimeS);
          startOrbitNumber.push_back(startOrbitNumberE);
          startOrbitNumber.push_back(startOrbitNumberE);
        }
      //@ FIN SI granule de donnees
    }
  //@ FIN POUR tous les granules en sorties
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetSensingTime
//@
//@ Description : Recupere les informations SensingTime
//@
//@ Parametres :
//@               (ENTREE) numLigne
//@               (SORTIE) sensingTime
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestGranule::GetSensingTime(const int numLigne, 
                                        t_TableauDate& sensingTime)
{
  t_InfoGenLigne dataGenLigne;
  t_Date         sensingTimeS;


  sensingTimeS.DateInit();

  //@ POUR tous les granules en sorties
  for (int i=0; i<m_ListeGranuleN1S.size(); i++)
    {
      //@ Appel de la methode SD_DON_GranuleN1::GetDataGenLigne()
      m_ListeGranuleN1S[i]->GetDataGenLigne(numLigne, dataGenLigne);

      //@ Recuperation de la date de debut de la ligne
      sensingTimeS = dataGenLigne.GetDateDebut();

      //@ Ajout des infos dans le tableau passe en parametre
      sensingTime.push_back(sensingTimeS);

      //@ SI granule de donnees
      if (m_ListeGranuleN1S[i]->GetNatureGranule() == 
          SD_DON_GranuleN1::SD_DON_GRANULE_N1_DATA)
        {
          //@ On recopie deux fois de plus les infos 
          //@ (1 GRANULE DATA = 3 produits)
          sensingTime.push_back(sensingTimeS);
          sensingTime.push_back(sensingTimeS);
        }
      //@ FIN SI granule de donnees
    }
  //@ FIN POUR tous les granules en sorties
}


//@ ------------------------------------------------------------------*/
//@ Methode : InitGranuleIASI
//@
//@ Description : Initialise les donnees IASI necessaires aux 
//@               traitements 0-1C 1A-1C 1B-1C
//@
//@ Parametres :
//@               (ENTREE/SORTIE) traitement : Objet contenant les 
//@                                            infos du traitement
//@               (ENTREE) gestionCasDegrade : Gestionnaire de cas
//@                                            degrade
//@               (SORTIE) valeurPTSI : PTSI de la premiere ligne du
//@                                     granule
//@               (ENTREE) nbLigneAbsolu : Nombre de ligne absolu 
//@                                        depuis le debut du dump
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestGranule::InitGranuleIASI(SD_DON_Traitement& traitement,
                                         SD_DON_GestCasDeg& 
                                         gestionCasDegrade,
                                         int& valeurPTSI,
                                         const int nbLigneAbsolu)
  throw (ERR__Exception)
{
  // Type de traitement
  CMN__StageTM::e_CMN__StageId typeTraitement     = CMN__StageTM::L1a;

  // Mode de traitement
  e_BAS_ModeTrt                modeTraitement     = GRANULE;

  // Position de traitement
  e_BAS_PositionTrt            positionTraitement = FIRST;

  // Date de debut de traitement
  t_Date                       t0;

  // Date de fin de traitement
  t_Date                       t1;

  // Liste des fichiers en entree
  t_ListeChaine                listeProduit;

  // Tableau des product model
  t_TableauChaine              listeModele;


  //@ Recuperer et mettre a jour les informations contenues dans 
  //@ le traitement via la methode ActualiserTraitement()
  ActualiserTraitement(traitement, 
                       typeTraitement, 
                       modeTraitement,
                       positionTraitement, 
                       t0, 
                       t1, 
                       listeProduit, 
                       listeModele);

  //@ SELON le type de traitement
  switch (typeTraitement)
    {
    //@ SI 0-1C
    case CMN__StageTM::L1a :
      //@ Appel de la methode InitComplet()
      InitComplet(listeProduit.front(), 
                  listeModele, 
                  t0, 
                  t1, 
                  modeTraitement,
                  positionTraitement,
                  nbLigneAbsolu,
                  gestionCasDegrade,
                  valeurPTSI);
      break;

    //@ SI 1A-1C
    case CMN__StageTM::L1b :
      //@ Appel de la methode InitIntermediaire()
      InitIntermediaire(SD_DON_PRODUIT_N1_1B1C,
                        listeProduit, 
                        listeModele, 
                        t0, 
                        t1, 
                        modeTraitement,
                        positionTraitement,
                        nbLigneAbsolu,
                        gestionCasDegrade,
                        valeurPTSI);
      break;

    //@ SI 1B-1C
    case CMN__StageTM::L1c :
      //@ Appel de la methode InitIntermediaire()
      InitIntermediaire(SD_DON_PRODUIT_N1_1C,
                        listeProduit, 
                        listeModele, 
                        t0, 
                        t1, 
                        modeTraitement,
                        positionTraitement,
                        nbLigneAbsolu,
                        gestionCasDegrade,
                        valeurPTSI);
      break;
      
      //@ Defaut:  
    default:
      // Exception generee
      ERR__Exception exception;

      //@ Par defaut, traitement non prevu
      //@ Emission d'une exception SD_TMT_NOT_FOUND
      exception.SetErrorCode(ERR__Exception::SD_TMT_NOT_FOUND);
      THROW(exception);
      break;
    }
  //@ FIN SELON
}


//@ ------------------------------------------------------------------*/
//@ Methode : InitBandeauAVHRR
//@
//@ Description : Initialise les donnees AVHRR de niveau 1B
//@               necessaires aux traitements 0-1C 1A-1C 1B-1C.
//@               Un bandeau est constitue de plusieurs granules AVHRR
//@
//@ Parametres :
//@               (ENTREE/SORTIE) traitement : Objet contenant les
//@                                            infos du traitement
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
/* CR 308*/
void SD_DON_GestGranule::InitBandeauAVHRR(
SD_DON_Traitement& traitement,
SD_DON_GestCasDeg & GestCasDeg
) 
/* ..CR 308*/
{
  // Date de debut de traitement
  t_Date                  t0;

  // Date de fin de traitement
  t_Date                  t1;

  // Liste des produits AVHRR
  t_ListeChaine           listeProduitAVHRR;

  // Iterateur pour la liste des produits AVHRR
  t_ListeChaine::iterator pInfo;


  //@ On recupere les infos fournies par l'objet SD_DON_Traitement:
  //@   - dates de debut et de fin du traitement
  t0                = traitement.T0();
  t1                = traitement.T1();

  //@ On ajoute ici une marge qui tiends compte de l'overlap:
  t0.ConvertirTempsEnDate((t0.ConvertirDateEnTemps() - SD_BAS_DUREE_LGN - (SD_BAS_NLI_MAX_UTIL_AVHRR*0.5*
  SD_BAS_AVHRR_LINE_PERIOD) / 1000.));
  //@ On ajoute ici une marge qui tiends compte de l'overlap:
  t1.ConvertirTempsEnDate((t1.ConvertirDateEnTemps()  + (SD_BAS_NLI_MAX_UTIL_AVHRR*0.5*
  SD_BAS_AVHRR_LINE_PERIOD) / 1000.));
  //@   - liste des produits AVHRR en entree
  //@ GetListeProduitAVHRR
  listeProduitAVHRR = traitement.ListeProduitAVHRR();

  //@ POUR chaque element de la liste des produits AVHRR
  for (pInfo = listeProduitAVHRR.begin();
       pInfo != listeProduitAVHRR.end(); 
       pInfo++)
    {
      //@ Extraire du nom du fichier la date UTC de debut (dateDebut)
      //@ et la date UTC de fin (dateFin)

      //@ On construit un objet t_OPSFile afin de recuperer
      //@ les dates 
      t_OPSFile fichierOPS(*pInfo);
      t_Date    dateDebut = fichierOPS.GetValidityStartDate();
      t_Date    dateFin   = fichierOPS.GetValidityEndDate();

#ifdef OPS_DEBUG
      cout << "Produit AVHRR: " << *pInfo << endl;
      cout << "\tdate debut: " << dateDebut.Chaine() << endl;
      cout << "\tdate fin  : " << dateFin.Chaine() << endl;
#endif

      //@ Comparaison de ces dates avec les dates de debut et
      //@ de fin issues du work order
      
      //@ SI dateFin < (t0 - marge) ou dateDebut > (t1 + marge)
      if ( (dateFin < (t0-DATE_MARGE)) || (dateDebut > (t1+DATE_MARGE)) )
        {
          //@ Le produit AVHRR n'est pas utilise

          //@ Emission d'un message de type JDB MSG_DATA_AVHRR_NOT_USED
          JDB__Client messageJDB;
          messageJDB.SetParametre(JDB_PARAM_1, *pInfo);
          messageJDB.EcrireMessage(SD_DATA_SET, SD__Message::
                                   MSG_DATA_AVHRR_NOT_USED);
          messageJDB.Liberer();
        }
      //@ SINON
      else
        {
          //@ Creation d'un granule AVHRR
          SD_DON_GranuleAVHRR* nouveauGranule = 
            new(SD_DON_GranuleAVHRR);

          //@ Initialisation du granule precedent via la methode
          //@ SD_DON_GranuleAVHRR::InitLecture()
/* CR 308 + FA 162 */
          nouveauGranule->InitLecture(*pInfo, 
	                              (t0-DATE_MARGE_CHARGEMENT_AVHRR), 
				      (t1+DATE_MARGE_CHARGEMENT_AVHRR),
				      GestCasDeg,
				      traitement.ModeTraitement());
/* ..CR 308 + FA 162 */

          //@ Insertion du granule dans la liste m_ListeGranuleAVHRR
          m_ListeGranuleAVHRR.push_back(nouveauGranule);

          //@ On ajoute a la liste des fichiers en entree dans le
          //@ traitement en sortie: 
          //@   - le nom du fichier AVHRR
          traitement.AjouterProduitE(*pInfo);
        }
      //@ FIN SI dateFin < (t0 - marge) ou dateDebut > (t1 + marge)
    }
  //@ FIN POUR chaque element de la liste des produits AVHRR

  //@ Tri de la liste des granules AVHRR en se basant sur la date
  //@ de debut (methode < implementee dans SD_DON_GranuleAVHRR)
  //@ On s'assure ainsi que les lignes seront recuperees 
  //@ chronologiquement dans GetBandeauAVHRR()
  sort(m_ListeGranuleAVHRR.begin(), m_ListeGranuleAVHRR.end(),inferieur<SD_DON_GranuleAVHRR*>());
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetBandeauAVHRR
//@
//@ Description : Recupere un ensemble de lignes AVHRR pour son
//@               utilisation dans les traitements. On se base sur
//@               une fourchette de dates specifiee en entree
//@
//@ Parametres :
//@               (ENTREE) t0 : Date de debut des lignes
//@               (ENTREE) t1 : Date de fin des lignes
//@               (ENTREE) numLigne : Numero de la ligne IASI pour
//@                                   laquelle la donnee est demandee
//@               (ENTREE) numSN : Numero du SN de l'image IASI pour
//@                                laquelle la donnee est demandee
//@               (ENTREE) gestionCasDegrade : Gestionnaire de cas 
//@                                            degrade
//@               (ENTREE) nbLigneMax : Nombre de lignes max
//@               (ENTREE) nbLigneMax : Nombre de lignes max
//@								(ENTREE)  pSVM : pointeur sur les donnees lues dans le fichier SVM
//@               (ENTREE/SORTIE) nbLigne : Nombre de lignes 
//@                                         allouees/retournees
//@               (ENTREE/SORTIE) listeLignes : Liste des lignes
//@                                             retournees
//@                     (tableau de pointeurs alloues par l'appelant)
//@                     Remarque: si = NULL alors rien n'est retourne
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
// FT-2939
void SD_DON_GestGranule::GetBandeauAVHRR(t_Date& t0,
                                         t_Date& t1,   
                                         const int numLigne,
                                         const int numLSN,
                                         SD_DON_GestCasDeg& 
                                         gestionCasDegrade,
                                         const int nbLigneMax,
                                         SD_DON_SVM*  pSVM,                              
                                         int& nbLigne,
                                         t_SD_BAS_LigneAVHRR** 
                                         listeLignes)
// ..FT-2939
{
  // Iterateur pour la liste des granules AVHRR
  t_ListeGranuleAVHRR::iterator granuleAVHRR;

  // Nombre de lignes pour un granule 
  int                           nbLigneGran = 0;

  // Dates AVHRR recuperees
  t_SCDSTime                    dateAVHRR;
  t_Date                        datePrecedente;
  t_Date                        dateCourante;
  double                        differenceEntreDate = 0.;

  // Temoin d'arret
  t_CMN__Booleen                continuer = CMN__VRAI;

  t_SD_BAS_LigneAVHRR**         listeLigneTmp;

  /* CR286 */
  int DateMargeLigneAVHRR;
  t_Chaine VariableEnv("");
  
  VariableEnv = getenv(s_DATE_MARGE_LIGNE_AVHRR.c_str());
  if (!VariableEnv.empty())
  {
    DateMargeLigneAVHRR = (int) atoi(VariableEnv.c_str());
  }
  else
  {
    DateMargeLigneAVHRR = DATE_MARGE_LIGNE_AVHRR;
  }
  /* FIN CR286 */
  
  //@ Initialisation du nombre de lignes retournees
  nbLigne = 0;
  dateCourante.DateCourante();


  granuleAVHRR = m_ListeGranuleAVHRR.begin();

  //@ TANT QUE il existe des granules ET QUE le nombre de lignes
  //@ max n'est pas atteint
  while (granuleAVHRR != m_ListeGranuleAVHRR.end() && 
         (continuer == CMN__VRAI))
    {
      //@ Appel SD_DON_GranuleAVHRR::GetNbLignesUtiles()
      //@ pour recuperer le nombre de lignes
      nbLigneGran = (*granuleAVHRR)->GetNbLignesUtiles(t0, t1);
                     
      //@ SI le nombre de lignes est superieur a 0
      if (nbLigneGran > 0)
        {
          //@ SI (nbLigne + nbLigneGranule) > nbLigneMax
          if ((nbLigne + nbLigneGran) > nbLigneMax)
            {
              //@ On ne peut pas ajouter tous les elements
              //@ dans la liste: on leve  CG9 via la methode
              //@ SD_DON_CasDegLigne::SetDetecte()
              gestionCasDegrade.GetCasDegLigne(numLigne+1)->
                SetDetecte(9, dateCourante, 0, 0, 0);

              //@ Mettre a jour le temoin d'arret
              continuer = CMN__FAUX;
            }
          //@ SINON
          else
            {
              //@ Allocation tableau de lignes AVHRR
              listeLigneTmp = new t_SD_BAS_LigneAVHRR*[nbLigneGran];
              
              //@ Appel de la methode 
              //@ SD_DON_GranuleAVHRR::GetDataLigne()
              //@ pour recuperer les lignes chronologiques
              // FT-2939
              (*granuleAVHRR)->GetDataLigne(t0,
                                            t1,
                                            numLigne,
                                            numLSN,
                                            gestionCasDegrade,
                                            pSVM,
                                            listeLigneTmp);
              // ..FT-2939
              
              //@ Initialisation de la date UTC ligne courante
              dateAVHRR.day = listeLigneTmp[0]->InfoGene.
                tab_var[AVHRR_DATEDEB].val.pdate->day;
              dateAVHRR.millis = listeLigneTmp[0]->InfoGene.
                tab_var[AVHRR_DATEDEB].val.pdate->millis;
              dateCourante.SCDSTime(dateAVHRR);
              
              if (nbLigne == 0)
                {
                  datePrecedente = dateCourante - 
                    (SD_BAS_AVHRR_LINE_PERIOD/1000.); 
                }   
              
              
              //@ POUR chaque ligne recuperee
              for (int i=0; i<nbLigneGran; i++)
                {
                  //@ Obtention de la date UTC ligne courante
                  dateAVHRR.day = listeLigneTmp[i]->InfoGene.
                    tab_var[AVHRR_DATEDEB].val.pdate->day;
                  dateAVHRR.millis = listeLigneTmp[i]->InfoGene.
                    tab_var[AVHRR_DATEDEB].val.pdate->millis;
                  dateCourante.SCDSTime(dateAVHRR); 
                  
                  // difference en milli-secondes
                  differenceEntreDate = (dateCourante - datePrecedente) 
                    * 1000;
                  
                  datePrecedente = dateCourante;
                  
                  //@ SI (date UTC Ligne - date UTC Ligne 
                  //@ precedente) < periode ligne AVHRR
                  if (differenceEntreDate < 
/* CR286 */           (SD_BAS_AVHRR_LINE_PERIOD - DateMargeLigneAVHRR))
                    {                      
                      //@ Il y a alors du recouvrement entre
                      //@ les produits
                      //@ On continue
                      
                      //@ Liberation memoire de la ligne recuperee
                      delete listeLigneTmp[i];
                    }
                  //@ SINON SI (date UTC Ligne - date UTC Ligne 
                  //@ precedente) > periode ligne
                  else if (differenceEntreDate > 
/* CR286 */                (SD_BAS_AVHRR_LINE_PERIOD + DateMargeLigneAVHRR))
                    {
                      //@ Il y a alors un trou de mesure: un
                      //@ cas degrade doit etre leve
                      
                      //@ SI le flag d'incoherence temporelle
                      //@ est leve dans l'attribut m_DataAVHRR
//                      if (*(listeLigneTmp[i]->InfoDet.
//                            tab_var[AVHRR_VALIDREC].val.pui4) == INVALIDE)
//                        {
                          //@ On leve CG9 via la methode
                          //@ SD_DON_CasDegLigne::SetDetecte()
                          gestionCasDegrade.GetCasDegLigne(numLigne+1)->
                            SetDetecte(9, dateCourante, 0, 0, 0);
                          
                          //@ On continue
//                        }
                      //@ FIN SI le flag d'incoherence temporelle
                      //@ est leve dans l'attribut m_DataAVHRR
                      //@
                      
                      //@ Liberation memoire de la ligne recuperee
                      delete listeLigneTmp[i];
                    }
                  //@ SINON (date UTC Ligne - date UTC Ligne 
                  //@ precedente) = periode ligne
                  else
                    {
                      //@ Affectation de la donnees dans le tableau 
                      //@ alloue par l'appelant
                      //@ nbLigne++
                      listeLignes[nbLigne++] = listeLigneTmp[i];
                    }
                  //@ FIN SI (date UTC Ligne - date UTC Ligne 
                  //@ precedente) < periode ligne
                  //@
                }
              //@ POUR chaque ligne recuperee
              // FT-2512
              delete []listeLigneTmp;	
              // ..FT-2512
            }
          //@ FIN SI (nbLigne + nbLigneGranule) > nbLigneMax
        }
      //@ FIN SI le nombre de lignes recuperees est superieur a 0
 
      //@ Passer au granule suivant
      granuleAVHRR++;
    }
  //@ FIN TANT QUE il existe des granules 
  //@ ET QUE le nombre de lignes max n'est pas atteint


  //@ SI aucune ligne n'est trouvee 
  if (nbLigne == 0)
    {
      // on leve le cas degrade CG9
      //@ on leve CG9
      gestionCasDegrade.GetCasDegLigne(numLigne+1)->
        SetDetecte(9, dateCourante, 0, 0, 0);
    }
  //@ FIN SI aucune ligne n'est trouvee 

  
  //@ Normalement ici, comme la liste a ete triee et que
  //@ dans chaque fichier la chronologie doit etre respectee,
  //@ les lignes doivent etre dans l'ordre chronologique
  //@ Si des "trous ou des recouvrements existent", ils
  //@ seront detectes par le traitement
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetLigne0
//@
//@ Description : Recupere une ligne de produit de N0 pour son 
//@               utilisation dans les traitements. On ne retourne
//@               ici que les paquets coherents (cf CG 13 14)
//@
//@ Parametres :
//@               (ENTREE) noLigne : Numero de la ligne demandee
//@               (ENTREE) type : Type des paquets choisi
//@               (SORTIE) nbPaquet : Nombre de paquets retournes
//@               (SORTIE) listePaquetLigne : Liste des paquets
//@                                           retournes
//@                     (tableau de pointeurs alloues par l'appelant)
//@                     Remarque: si = NULL alors rien n'est retourne
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestGranule::GetLigne0(const int noLigne,
                                   const e_SD_BAS_TypeN0 type,
                                   int& nbPaquetN0,
                                   t_SD_BAS_PaquetN0** listePaquetLigne)
{
  // Nombre de paquets N0
  nbPaquetN0 = 0;


  //@ SI la liste des granules N0 en entree n'est pas vide
  if (m_ListeGranuleN0E.size() > 0)
    {
      //@ Appel de la methode SD_DON_GranuleN0::GetDataLigne()
      //@ On recupere ainsi la liste des paquets associes a
      //@ la ligne pour un type precis
      m_ListeGranuleN0E[0]->GetDataLigne(noLigne,
                                         type, 
                                         nbPaquetN0, 
                                         listePaquetLigne);
    }
  //@ FIN SI la liste des granules N0 en entree n'est pas vide
}


//@ ------------------------------------------------------------------*/
//@ Methode : DecodeDataLigne0
//@
//@ Description : Decode un paquet de niveau 0 contenue dans une ligne.
//@               Le decodage se limite ici a l'extraction du champ de
//@               bit correspondant au spectre
//@
//@ Parametres :
//@               (ENTREE) paquetLigne : Paquet ISP issu de GetLigne0
//@               (SORTIE) dataDecode : Donnees decodees allouees par
//@                                     l'appelant
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestGranule::DecodeDataLigne0(t_SD_BAS_PaquetN0* paquetLigne,
                                          void* dataDecode) 
{
  //@ SI la liste des granules N0 en entree n'est pas vide
  if (m_ListeGranuleN0E.size() > 0)
    {
      //@ Decodage du paquet courant de la ligne via la methode 
      //@ SD_DON_GranuleN0::DecodeDataLigne()
      m_ListeGranuleN0E[0]->DecodeDataLigne(paquetLigne, 
                                            dataDecode);
    }
  //@ FIN SI la liste des granules N0 en entree n'est pas vide
}

//CR271
//@ ------------------------------------------------------------------*/
//@ Methode : EstTransitionLigneS1
//@
//@ Description : Recupere la validite d'une ligne de produit de N1
//@
//@ Parametres :
//@               (ENTREE) numLigne : Numero de la ligne demandee
//@               (ENTREE) natureGranule : Nature du granule recherche
//@                                      (DATA, ENG, VER)
//@
//@ Valeur de retour : 
//@               CMN__VRAI : Ligne valide
//@               CMN__FAUX : Ligne invalide
//@
//@ ------------------------------------------------------------------*/
t_CMN__Booleen SD_DON_GestGranule::EstTransitionLigneS1(const int numLigne,
                                                    const 
                                                    SD_DON_GranuleN1::
                                                    e_SD_DON_GranuleN1 
                                                    natureGranule)
  throw (ERR__Exception)
{
  // Exception generee
  ERR__Exception             exception;

  // Iterateur pour la liste des granules
  t_ListeGranuleN1::iterator granuleN1;

  // Temoin de recherche
  t_CMN__Booleen             trouve    = CMN__FAUX;


  //@ Recherche du premier granule en sortie
  granuleN1 = m_ListeGranuleN1S.begin();

  //@ TANT QUE il existe des granules en sortie ET QUE le
  //@ granule souhaite n'a pas ete trouve
  while (granuleN1 != m_ListeGranuleN1S.end() && (trouve == CMN__FAUX))
    {
      //@ SI GetNatureGranule() = natureGranule
      if ((*granuleN1)->GetNatureGranule() == natureGranule)
        {
          //@ Le granule souhaite a ete trouve
          //@ Mettre a jour le temoin de recherche
          trouve = CMN__VRAI;
        }
      //@ SINON
      else
        {
          //@ Passer au granule suivant
          granuleN1++;
        }
      //@ FIN SI GetTypeProduit() = typeProduit
    }
  //@ FIN TANT QUE
  
  //@ SI le granule n'a pas ete trouve
  if (trouve != CMN__VRAI)
    {
      //@ Emission exception SD_DON_OPER_FAILED
      exception.SetErrorCode(ERR__Exception::SD_DON_OPER_FAILED);
      THROW(exception);
    }
  //@ FIN SI le granule n'a pas ete trouve

  //@ Appel a la methode SD_DON_GranuleN1::EstValideLigne()
  return ((*granuleN1)->EstTransitionLigne(numLigne)); //CR271
}
  

//@ ------------------------------------------------------------------*/
//@ Methode : GetLigneS1
//@
//@ Description : Recupere une ligne de produit de N1 pour son
//@               utilisation dans les traitements. Ceci est realise
//@               pour tous les traitements afin d'acceder au produit
//@               en sortie
//@
//@ Parametres :
//@               (ENTREE) numLigne : Numero de la ligne demandee
//@               (ENTREE) natureGranule : Nature du granule recherche
//@                                      (DATA, ENG, VER)
//@               (SORTIE) mphr : MPHR produit N1 (alloue par 
//@                               l'appelant)
//@               (SORTIE) ligneMDR : Ligne MDR retournee (allouee par
//@                                   par l'appelant)
//@               (SORTIE) ligneVIADR : Ligne VIADR retournee (alloue
//@                                     par l'appelant)
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestGranule::GetLigneS1(const int numLigne,
                                    const SD_DON_GranuleN1::
                                    e_SD_DON_GranuleN1 natureGranule,
                                    t_SD_BAS_MPHR& mphr,
                                    t_SD_BAS_LigneN1& ligneMDR,
                                    t_SD_BAS_LigneN1& ligneVIADR)
  throw (ERR__Exception)
{
  // Exception generee
  ERR__Exception             exception;

  // Iterateur pour la liste des granules
  t_ListeGranuleN1::iterator granuleN1;

  // Temoin de recherche
  t_CMN__Booleen             trouve    = CMN__FAUX;


  //@ Recherche du premier granule en sortie
  granuleN1 = m_ListeGranuleN1S.begin();

  //@ TANT QUE il existe des granules en sortie ET QUE le
  //@ granule souhaite n'a pas ete trouve
  while (granuleN1 != m_ListeGranuleN1S.end() && (trouve == CMN__FAUX))
    {
      //@ SI GetNatureGranule() = natureGranule
      if ((*granuleN1)->GetNatureGranule() == natureGranule)
        {
          //@ Le granule souhaite a ete trouve
          //@ Mettre a jour le temoin de recherche
          trouve = CMN__VRAI;
        }
      //@ SINON
      else
        {
          //@ Passer au granule suivant
          granuleN1++;
        }
      //@ FIN SI GetTypeProduit() = typeProduit
    }
  //@ FIN TANT QUE
  
  //@ SI le granule n'a pas ete trouve
  if (trouve != CMN__VRAI)
    {
      //@ Emission exception SD_DON_OPER_FAILED
      exception.SetErrorCode(ERR__Exception::SD_DON_OPER_FAILED);
      THROW(exception);
    }
  //@ FIN SI le granule n'a pas ete trouve

  //@ Appel a la methode SD_DON_Granule::GetEnteteMPHR()
  mphr = (*granuleN1)->GetEnteteMPHR();

//jp8 pour les fonction metop on reinitialise avec les donnes en entree  (N0 ou N1)
  //@ Appel a la methode SD_DON_Granule::GetEnteteMPHR()
  //@ Appel a la methode SD_DON_Granule::GetEnteteMPHR()
  if (m_ListeGranuleN1E.size() > 0 )
    { 
      //@ Appel de la methode SD_DON_GranuleN0::GetEnteteMPHR()
      //@ On recupere ainsi le MPHR
      mphr = (m_ListeGranuleN1E[0])->GetEnteteMPHR();
    }
  if (m_ListeGranuleN0E.size() > 0 )
    {
      //@ Appel de la methode SD_DON_GranuleN0::GetEnteteMPHR()
      //@ On recupere ainsi le MPHR
      mphr = (m_ListeGranuleN0E[0])->GetEnteteMPHR();
    }
  //@ Appel a la methode SD_DON_GranuleN1::GetDataLigne()
  (*granuleN1)->GetDataLigne(numLigne, ligneMDR, ligneVIADR);
}
  

//@ ------------------------------------------------------------------*/
//@ Methode : EncodeDecodeLigneS1
//@
//@ Description : Decode les donnees detaillees contenues dans une
//@               ligne de niveau 1 
//@
//@ Parametres :
//@               (ENTREE) numLigneSNPN : Numero SN PN a traiter
//@               (ENTREE) ligne : Ligne issue de l'appel GetDataLigne
//@               (ENTREE) niveau : Info a traiter (IMAGE/1A/1B/1C)
//@               (ENTREE) paramDecodage : Parametres de decodage
//@               (ENTREE) sens : Sens decodage/encodage
//@               (ENTREE/SORTIE) data : Donnees  decodees allouees
//@                                      par l'appelant
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestGranule::EncodeDecodeLigneS1(const t_SD_BAS_ORIGINE 
                                             numLigneSNPN, 
                                             t_SD_BAS_LigneN1* ligne,
                                             const e_SD_BAS_Atraiter 
                                             niveau,
                                             t_SD_BAS_ConfStable* 
                                             paramDecodage,
                                             const e_SD_BAS_Sens sens,
                                             double* data)
{
  //@ SI la liste des granules en sortie n'est pas vide
  if (m_ListeGranuleN1S.size() > 0)
    {
      //@ Decodage encodage de la ligne via la methode 
      //@ SD_DON_GranuleN1::EncodeDecodeDataLigne()
      m_ListeGranuleN1S[0]->EncodeDecodeDataLigne(numLigneSNPN, 
                                                  ligne, 
                                                  niveau, 
                                                  paramDecodage,
                                                  sens,
                                                  data);
    }
  //@ FIN SI la liste des granules en sortie n'est pas vide
}


//@ ------------------------------------------------------------------*/
//@ Methode : SetEnteteS1
//@
//@ Description : Ecrit les entetes de granules produits de N1 en fin
//@               de traitement
//@
//@ Parametres :
//@               (ENTREE) statMPHR : MPHR a ecrire 
//@               (ENTREE) paramConfStable : Parametre de qualite pour
//@                                          le GIADR
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
// FT-2466
void SD_DON_GestGranule::SetEnteteS1(const t_SD_BAS_StatMphr* statMPHR,
                                     const t_SD_BAS_ConfStable*
                                     paramConfStable,
                                     int numeroLigne) 
{
  // Iterateur pour la liste des granules
  t_ListeGranuleN1::iterator granuleN1;


  //@ POUR chaque element de la liste des granules en sortie
  for (granuleN1 = m_ListeGranuleN1S.begin();
       granuleN1 != m_ListeGranuleN1S.end(); 
       granuleN1++)
    {
      //@ Appel a la methode SD_DON_GranuleN1::SetEntete()
      (*granuleN1)->SetEntete(statMPHR, paramConfStable, m_OBTUTC, numeroLigne);
    }
  //@ FIN POUR chaque element de la liste des granules en sortie
}
// ..FT-2466

//@ ------------------------------------------------------------------*/
//@ Methode : EcrireS1
//@
//@ Description : Ecrit les granules produits de N1 en fin de
//@               traitement
//@
//@ Parametres :
//@               (ENTREE) idCA : Id du fichier de configuration autre
//@               (SORTIE) mphr : Donnees MPH
//@               (SORTIE) listeFichier : Liste des fichiers ecrits
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestGranule::EcrireS1(const int idCA, 
                                  t_SD_BAS_MPHR& mphr,
                                  t_ListeChaine& listeFichier)
{
  // Iterateur pour la liste des granules
  t_ListeGranuleN1::iterator granuleN1;

  // Temoin d'ecriture
  t_CMN__Booleen             premierGranuleEcrit = CMN__FAUX;


  //@ POUR chaque element de la liste des granules en sortie
  for (granuleN1 = m_ListeGranuleN1S.begin();
       granuleN1 != m_ListeGranuleN1S.end(); 
       granuleN1++)
    {
      //@ Appel a la methode SD_DON_GranuleN1::Ecrire()
      
#ifdef OPS_DEBUG
      cout << "EcrireS1(): Affichage granule en sortie" << endl;
      (*granuleN1)->Afficher();
#endif
 

      (*granuleN1)->Ecrire(idCA, listeFichier);
      
      //@ En cas d'erreur d'ecriture pour un produit,
      //@ on sort en exception et on s'arrete
      
      //@ On recupere les donnees MPH du premier granule ecrit

      if (premierGranuleEcrit == CMN__FAUX)
        {
          mphr = (*granuleN1)->GetEnteteMPHR();
          premierGranuleEcrit = CMN__VRAI; 
        }
    }
  //@ FIN POUR chaque element de la liste des granules en sortie
  

  //@ L'encodage des donnees spectre et image doit avoir ete realise
  //@ au prealable par appel direct des fonctions de SD_BAS_Niveau1 
  //@ depuis les chaines de traitement
}



//======================================================================
//                                      METHODES PRIVEES 
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : ActualiserTraitement
//@
//@ Description : Recupere et met a jour les informations du traitement
//@
//@ Parametres :
//@               (ENTREE/SORTIE) traitement : Objet contenant les 
//@                                            infos du traitement
//@               (SORTIE) typeTraitement : Type du traitement
//@               (SORTIE) modeTraitement : Mode du traitement
//@               (SORTIE) positionTraitement : Position du traitement
//@               (SORTIE) t0 : Date de debut du traitement
//@               (SORTIE) t1 : Date de fin du traitement
//@               (SORTIE) listeProduit : Liste des produits en entree
//@               (SORTIE) listeModele : Liste des product model
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestGranule::ActualiserTraitement(SD_DON_Traitement& traitement, 
                                              CMN__StageTM::
                                              e_CMN__StageId& 
                                              typeTraitement, 
                                              e_BAS_ModeTrt& 
                                              modeTraitement,
                                              e_BAS_PositionTrt&
                                              positionTraitement,
                                              t_Date& t0, 
                                              t_Date& t1, 
                                              t_ListeChaine& 
                                              listeProduit, 
                                              t_TableauChaine& 
                                              listeModele)
  throw (ERR__Exception)
{
  // Exception generee
  ERR__Exception          exception;

  // Iterateur pour la liste des fichiers
  t_ListeChaine::iterator pInfo;

  // Nom du fichier OBTUTC
  t_Chaine                nomFichierOBTUTC("");

  // Nom du fichier OSV
  t_Chaine                nomFichierOSV("");

  // Liste des product model
  t_ListeChaine           listeProductModel;

  // Liste des donnees OBTUTC
  t_ListeChaine           listeDonneesOBTUTC;

  // Liste des donnees OSV
  t_ListeChaine           listeDonneesOSV;

  
  //@ On recupere les infos fournies par l'objet SD_DON_Traitement:
  //@   - dates de debut et de fin du traitement
  t0                 = traitement.T0();
  t1                 = traitement.T1();
  
  //@   - mode de traitement (DUMP ou GRANULE)
  modeTraitement     = traitement.ModeTraitement();

  //@   - position du traitement (FIRST, MIDDLE ou LAST)
  positionTraitement = traitement.SlideId();

  //@   - type de traitement (0-1C 1A-1C 1B-1C)
  typeTraitement     = traitement.TypeTraitement();

  //@   - liste des produits IASI en entree: 
  //@

  //@ SI la liste des produits IASI en entree est vide
  if (traitement.ListeProduitE().size() == 0)
  {
    //@ Emission d'une exception SD_DON_TMT_DATA_IN
    exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_IN);
    exception.SetParametre(DATA_IN_NOT_FOUND);
    THROW(exception);
  }
  //@ FIN SI la liste des produits IASI en entree est vide

  //@ On prepare la liste des produit en entree
  listeProduit.clear();

  //@ SI le traitement est un traitement complet (0-1C)
  if (typeTraitement == CMN__StageTM::L1a)
  {
    //@  recuperation du nom du produit N0 en entree
    listeProduit.push_back(traitement.ListeProduitE().front());
  }
  //@ SINON le traitement est un traitement intermediaire
  else
  {
    //@  recuperation des produits N1 en entree: on positionne
    //@  le produit donnees en debut de liste et le produit 
    //@  ENG en fin de liste

    //@ POUR tous les produits en entree
    for (pInfo = traitement.ListeProduitE().begin(); 
         pInfo != traitement.ListeProduitE().end();
         pInfo++)
    {
      //@ SI le fichier est de type 1A ou de type 1B
      if ((EstDeType(PREFIXE_1A, *pInfo) == CMN__VRAI) || 
         (EstDeType(PREFIXE_1B, *pInfo) == CMN__VRAI))
      {
        //@ L'ajouter en tete de liste
        listeProduit.push_front(*pInfo);
      }

      //@ SI le fichier est de type ENG
      if (EstDeType(PREFIXE_ENG, *pInfo) == CMN__VRAI)
      {
        //@ L'ajouter en fin de liste
        listeProduit.push_back(*pInfo);
      }
      //@ FIN SI le fichier est de type donnees intermediaire
    }
    //@ FIN POUR tous les produits en entree

    //@ SI la liste des produits IASI en entree triee ne 
    //@ contient pas deux elements
    if (listeProduit.size() != NB_DATA_IN_INTERMEDIAIRE)
    {
      //@ Emission d'une exception SD_DON_TMT_DATA_IN
      exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_IN);
      exception.SetParametre(DATA_IN_NOT_ENOUGH);
      THROW(exception);
    }
    //@ FIN SI la liste des produits IASI en entree triee ne 
    //@ contient pas deux elements
  }
  //@ FIN SI le traitement est un traitement complet (0-1C)


  //@   - liste des modeles de produit
  listeProductModel = traitement.ListeProductModel();

  //@   - liste des donnees auxiliaires

  //@
  //@ On extrait de la liste des donnees auxiliaires:
  //@   - les fichiers OBTUTC
  //@   - les fichiers OSV
     
  //@ POUR tous les elements de la liste des donnees auxiliaires
  for (pInfo = traitement.ListeDonneesAux().begin(); 
       pInfo != traitement.ListeDonneesAux().end();
       pInfo++)
    {
      //@ SI le fichier est de type OBTUTC
      if (EstDeType(PREFIXE_OBTUTC, *pInfo) == CMN__VRAI)
        {

#ifdef OPS_DEBUG
          cout << "Fichier OBTUTC: " << *pInfo << endl;
#endif

          //@ L'ajouter dans la liste des fichiers OBTUTC
          listeDonneesOBTUTC.push_back(*pInfo);
        }
      //@ SINON SI le fichier est de type OSV
      else if (EstDeType(PREFIXE_OSV, *pInfo) == CMN__VRAI)
        {

#ifdef OPS_DEBUG
          cout << "Fichier OSV: " << *pInfo << endl;
#endif

          //@ L'ajouter dans la liste des fichiers OSV
          listeDonneesOSV.push_back(*pInfo);
        }
      //@ FIN SI le fichier est de type OBTUTC
    }
  //@ FIN POUR tous les elements de la liste des donnees auxiliaires


  //@ RAZ des listes de produits du traitement
  traitement.ListeProductModel().clear();
  traitement.ListeProduitS().clear();

  //@ On extrait de la liste des product model:
  //@   - le product model 1C
  //@   - le product model ENG
  //@   - le product model VER en mode complet

  //@ Recherche des product model 1C et ENG
  listeModele.clear();
  RecupererProductModel(listeProductModel, PREFIXE_1C, listeModele);
  RecupererProductModel(listeProductModel, PREFIXE_ENG, listeModele);

  //@ SI le traitement est un traitement complet (0-1C)
  if (typeTraitement == CMN__StageTM::L1a)
  {
    //@ Recherche des product model VER
    RecupererProductModel(
      listeProductModel, 
      PREFIXE_VER, 
      listeModele);
    
    //@ On recupere le fichier OBTUTC a utiliser (en fonction de la
    //@ date de validite ecrite dans le nom du fichier). On choisit
    //@ celui dont la date de validite est la plus proche de la
    //@ date courante
    InitOBTUTC(listeDonneesOBTUTC, nomFichierOBTUTC);
  }
  //@ FIN SI le traitement est un traitement complet (0-1C)

  //@ On recupere le fichier OSV a utiliser (en fonction de la
  //@ date de validite ecrite dans le nom du fichier). On choisit
  //@ celui dont la date de validite est la plus proche de la
  //@ date courante
  InitOSV(listeDonneesOSV, nomFichierOSV);

  //@ On ajoute a la liste des fichiers en entree dans le
  //@ traitement en sortie: 
  //@   - le nom du produit en entree
  //@   - le nom du product model 1C
  //@   - le nom du product model ENG
  //@   - le nom du product model VER (si mode complet)
  //@   - le nom du fichier OBTUTC utilise
  //@   - le nom du fichier OSV utilise (s'il a ete defini)
  traitement.ListeProduitE(listeProduit);
  for (int iModele=0; iModele<listeModele.size(); iModele++)
    {
      traitement.AjouterProduitE(listeModele[iModele]);
    }
  
  //@ SI le traitement est un traitement complet (0-1C)
  if (typeTraitement == CMN__StageTM::L1a)
  {
    if ( !nomFichierOBTUTC.empty() )
    {
      traitement.AjouterProduitE(nomFichierOBTUTC);
    }
  }

  if ( !nomFichierOSV.empty() )
  {
    traitement.AjouterProduitE(nomFichierOSV);
  }


  //@ Remarque: on suppose ici que les listes de produits en E/S
  //@           sont coherentes avec le type de traitement et le
  //@           mode
}


//@ ------------------------------------------------------------------*/
//@ Methode : RecupererProductModel
//@
//@ Description : Recupere le product model a utiliser
//@
//@ Parametres :
//@               (ENTREE) listeProductModel : Liste des product model
//@               (ENTREE) typeModele : Type du product model recherche
//@               (SORTIE) listeModele : Liste des product model
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestGranule::RecupererProductModel(t_ListeChaine& 
                                               listeProductModel,
                                               const t_Chaine& 
                                               typeModele,
                                               t_TableauChaine& 
                                               listeModele)
  throw (ERR__Exception)
{
  // Iterateur pour la liste des fichiers
  t_ListeChaine::iterator pInfo  = listeProductModel.begin();

  // Temoin de recherche
  t_CMN__Booleen          trouve = CMN__FAUX;


  //@ TANT QUE il existe des product model ET QUE le
  //@ product model recherche n'a pas ete trouve
  while (pInfo != listeProductModel.end() && (trouve == CMN__FAUX))
    {
      //@ SI le product model est de type 1C
      if (EstDeType(typeModele, *pInfo) == CMN__VRAI)
        {
          //@ Le product model recherche a ete trouve
          //@ Mettre a jour le temoin de recherche
          trouve = CMN__VRAI;
        }
      //@ SINON ce n'est pas le traitement courant
      else
        {
          //@ Passer au product model suivant
          pInfo++;
        }
      //@ FIN SI
    }
  //@ FIN TANT QUE

  //@ SI le product model n'a pas ete trouve
  if (trouve == CMN__FAUX)
    {
      // Exception generee
      ERR__Exception exception;

      //@ Emission d'une exception SD_DON_TMT_DATA_PROD
      exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_PROD);
      exception.SetParametre(DATA_PROD_NOT_FOUND+typeModele);
      THROW(exception);
    }
  //@ FIN SI le product model n'a pas ete trouve

  //@ Recuperation du nom du product model
  listeModele.push_back(*pInfo);
}


//@ ------------------------------------------------------------------*/
//@ Methode : InitOBTUTC
//@
//@ Description : Recupere le fichier OBTUTC a utiliser et initialise 
//@               l'objet correction OBTUTC
//@
//@ Parametres :
//@               (ENTREE/SORTIE) listeDonneesOBTUTC : Liste des 
//@                                                    fichiers OBTUTC
//@               (SORTIE) nomFichierOBTUTC : Fichier OBTUTC a utiliser
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestGranule::InitOBTUTC(t_ListeChaine& listeDonneesOBTUTC,
                                    t_Chaine& nomFichierOBTUTC)
  throw (ERR__Exception)
{
  // Exception generee
  ERR__Exception exception;

  // Iterateur pour la liste des fichiers
  t_ListeChaine::iterator pInfo;

  // Date courante
  t_Date                  dateCourante;

  // Temoins de recherche
  t_CMN__Booleen          dateDepassee = CMN__FAUX;
  t_CMN__Booleen          trouve       = CMN__FAUX;


  //@ SI la liste des fichiers OBTUTC est vide
  if (listeDonneesOBTUTC.size() == 0)
    {
      //@ Emission d'un message de type JDB MSG_DATA_OBTUTC_UNAVAILABLE
      JDB__Client messageJDB;
      messageJDB.EcrireMessage(SD_DATA_SET, SD__Message::
                               MSG_DATA_OBTUTC_UNAVAILABLE);
      messageJDB.Liberer();

      //@ Emission d'une exception SD_DON_TMT_DATA_AUX
      exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_AUX);
      exception.SetParametre(DATA_AUX_OBTUTC_NOT_FOUND);
      THROW(exception);
   }
  //@ FIN SI la liste des fichiers OBTUTC est vide


  //@ On recupere la date courante
  dateCourante.DateCourante();

  //@ Tri de la liste des  donnees OBTUTC
  listeDonneesOBTUTC.sort();

  //@ Recherche du premier fichier OBTUTC
  pInfo            = listeDonneesOBTUTC.begin();
  nomFichierOBTUTC = *pInfo;

  //@ TANT QUE il existe des fichiers OBTUTC ET QUE la date de
  //@ validite n'a pas ete depassee
  while (pInfo != listeDonneesOBTUTC.end() && 
         (dateDepassee == CMN__FAUX))
    {
      //@ On construit un objet t_OPSFile afin de recuperer
      //@ la date de validite pour le premier element de la
      //@ liste des donnees OBTUTC
      t_OPSFile fichierOPS(*pInfo);
      t_Date    dateValiditeTmp = fichierOPS.GetValidityStartDate();

      //@ SI la date de validite est superieure a la date courante
      if (dateValiditeTmp > dateCourante)
        {
          //@ Mettre a jour le temoin de recherche
          dateDepassee = CMN__VRAI;
        }
      //@ SINON ce n'est pas le traitement courant
      else
        {
          //@ Passer au fichier OBTUTC suivant
          trouve = CMN__VRAI;
          nomFichierOBTUTC = *pInfo;
          pInfo++;
        }
      //@ FIN SI
    }
  //@ FIN TANT QUE

  //@ RAZ de la liste des donnees OBTUTC
  listeDonneesOBTUTC.clear();

  //@ SI un fichier OBTUTC a ete trouve
  if (trouve == CMN__VRAI)
    {
      //@ En captant les exceptions
      try
        {
          //@ On initialise l'objet correction OBTUTC par appel a la
          //@ methode SD_DON_OBTUTC::Lire()
          m_OBTUTC.Lire(nomFichierOBTUTC);
        }

      //@ Traitement des exceptions
      catch (ERR__Exception exception)
        {
#ifdef OPS_DEBUG
          cerr << "Erreur lecture OBTUTC" << endl;
#endif
          
          //@ Un fichier OBTUTC valide n'a pas ete trouve
          trouve = CMN__FAUX;
        }
      //@ FIN SI un fichier OSV a ete trouve
    }
  //@ FIN SI la liste des fichiers OSV n'est pas vide

  //@ SI le fichier OBTUTC n'a pas ete trouve
  if (trouve == CMN__FAUX)
    {
      //@ Emission d'un message de type JDB MSG_DATA_OBTUTC_UNAVAILABLE
      JDB__Client messageJDB;
      messageJDB.EcrireMessage(SD_DATA_SET, SD__Message::
                               MSG_DATA_OBTUTC_UNAVAILABLE);
      messageJDB.Liberer();
      
      //@ Emission d'une exception SD_DON_TMT_DATA_AUX
      exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_AUX);
      exception.SetParametre(DATA_AUX_OBTUTC_NOT_FOUND);
      THROW(exception);
    }
  //@ FIN SI le fichier OBTUTC n'a pas ete trouve
}


//@ ------------------------------------------------------------------*/
//@ Methode : InitOSV
//@
//@ Description : Recupere le fichier OSV a utiliser et initialise 
//@               l'objet mode plateforme OSV
//@
//@ Parametres :
//@               (ENTREE/SORTIE) listeDonneesOSV : Liste des fichiers
//@                                                 OSV
//@               (SORTIE) nomFichierOSV : Fichier OSV a utiliser
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestGranule::InitOSV(t_ListeChaine& listeDonneesOSV,
                                 t_Chaine& nomFichierOSV)
{
  // Iterateur pour la liste des fichiers
  t_ListeChaine::iterator pInfo;

  // Objet OSV
  SD_DON_OSV              objetOSV;

  // Date courante
  t_Date                  dateCourante;

  // Temoins de recherche
  t_CMN__Booleen          dateDepassee = CMN__FAUX;
  t_CMN__Booleen          trouve       = CMN__FAUX;


  //@ SI la liste des fichiers OSV n'est pas vide
  if (listeDonneesOSV.size() != 0)
    {
      //@ On recupere le fichier OSV a utiliser (en fonction de la
      //@ date de validite ecrite dans le nom du fichier). On choisit
      //@ celui dont la date de validite est la plus proche de la
      //@ date courante

      //@ On recupere la date courante
      dateCourante.DateCourante();
      
      //@ Tri de la liste des  donnees OSV
      listeDonneesOSV.sort();

      //@ Recherche du premier fichier OSV
      pInfo         = listeDonneesOSV.begin();
      nomFichierOSV = *pInfo;

      //@ TANT QUE il existe des fichiers OSV ET QUE la date de
      //@ validite n'a pas ete depassee
      while (pInfo != listeDonneesOSV.end() && 
             (dateDepassee == CMN__FAUX))
        {
          //@ On construit un objet t_OPSFile afin de recuperer
          //@ la date de validite pour le premier element de la
          //@ liste des donnees OSV
          t_OPSFile fichierOPS(*pInfo);
          t_Date    dateValiditeTmp = fichierOPS.GetValidityStartDate();
          
          //@ SI la date de validite est superieure a la date courante
          if (dateValiditeTmp > dateCourante)
            {
              //@ Mettre a jour le temoin de recherche
              dateDepassee = CMN__VRAI;
            }
          //@ SINON ce n'est pas le traitement courant
          else
            {
              //@ Passer au fichier OSV suivant
              trouve = CMN__VRAI;
              nomFichierOSV = *pInfo;
              pInfo++;
            }
          //@ FIN SI
        }
      //@ FIN TANT QUE

      //@ RAZ de la liste des donnees OSV
      listeDonneesOSV.clear();

      //@ SI un fichier OSV a ete trouve
      if (trouve == CMN__VRAI)
        {
          //@ En captant les exceptions
          try
            {
              //@ On initialise l'objet mode plateforme OSV par 
              //@ appel a la methode SD_DON_OSV::Lire()
              m_OSV.Lire(nomFichierOSV);
            }

          //@ Traitement des exceptions
          // FT-2765/FT-2766
          catch (ERR__Exception e)
          // ...FT-2765/FT-2766
            {
#ifdef OPS_DEBUG
              cerr << "Erreur lecture OSV" << endl;
#endif
              
              //@ Un fichier OSV valide n'a pas ete trouve
              trouve = CMN__FAUX;
            }
        }
      //@ FIN SI un fichier OSV a ete trouve
    }
  //@ FIN SI la liste des fichiers OSV n'est pas vide
  
  //@ SI le fichier OSV n'a pas ete trouve
  if (trouve == CMN__FAUX)
    {
      //@ Emission d'un message de type JDB MSG_DATA_OSV_DEFAULT
      JDB__Client messageJDB;
      messageJDB.EcrireMessage(SD_DATA_SET, 
                               SD__Message::MSG_DATA_OSV_DEFAULT);
      messageJDB.Liberer();
    }
  //@ FIN SI le fichier OSV n'a pas ete trouve
}


//@ ------------------------------------------------------------------*/
//@ Methode : InitComplet
//@
//@ Description : Initialise les granules d'un traitement complet 0-1C
//@
//@ Parametres :
//@               (ENTREE) nomProduit : Nom du produit en entree 
//@               (ENTREE) listeModele : Liste des product model
//@               (ENTREE) t0 : Date de debut du traitement
//@               (ENTREE) t1 : Date de fin du traitement
//@               (ENTREE) modeTraitement : Mode du traitement
//@               (ENTREE) positionTraitement : Position du traitement
//@               (ENTREE) nbLigneAbsolu : Nombre de ligne absolu 
//@                                        depuis le debut du dump
//@               (ENTREE) gestionCasDegrade : Gestionnaire de cas
//@                                            degrade
//@               (SORTIE) valeurPTSI : PTSI de la premiere ligne du
//@                                     granule
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestGranule::InitComplet(const t_Chaine& nomProduit, 
                                     const t_TableauChaine& listeModele,
                                     const t_Date& t0, 
                                     const t_Date& t1, 
                                     const e_BAS_ModeTrt& 
                                     modeTraitement, 
                                     const e_BAS_PositionTrt&
                                     positionTraitement,
                                     const int nbLigneAbsolu,
                                     SD_DON_GestCasDeg& 
                                     gestionCasDegrade,
                                     int& valeurPTSI)
{
  // Granule en entree
  SD_DON_GranuleN0*  granuleN0  = new SD_DON_GranuleN0();

  // Granules en sortie
  SD_DON_GranuleN1*  granuleN1  = new SD_DON_GranuleN1();
  SD_DON_GranuleN1*  granuleENG = new SD_DON_GranuleN1();
  SD_DON_GranuleN1*  granuleVER = new SD_DON_GranuleN1();

  // Type de produit a generer
  e_SD_DON_ProduitN1 typeProduit;

  // Infos ligne
  t_InfoGenLigne     dataGenLigne;

  
  //@ 1. Creation du granule de niveau 0 (produit en 
  //@ entree) par SD_DON_GranuleN0::InitLecture()
  //@ et lecture du fichier associe

/* FA 183 - recup du mode traitement pour gestion des
execeptions a l'appel de gestionOverlap */
  granuleN0->InitLecture(nomProduit,
                         t0,
                         t1,
                         m_OBTUTC,
                         gestionCasDegrade,
			 modeTraitement);			 
/* ..FA 183*/
  /* modif CNES R. MORENO */
  if(granuleN0->GetNbLigne() > 0)
  {
    //@ Recuperation du PTSI de la premiere ligne par 
    //@ SD_DON_GranuleN0::GetDataGenLigne()
    valeurPTSI = -1;
    granuleN0->GetDataGenLigne(0, dataGenLigne);
    valeurPTSI = (int) dataGenLigne.GetPTSI();
  }
  
  //@ Appel a la methode SD_DON_Granule::GetEnteteMPHR()
  t_SD_BAS_MPHR mphr;
  mphr = (granuleN0)->GetEnteteMPHR();

  //@ Ajout du granule a la liste m_ListeGranuleN0E
  m_ListeGranuleN0E.push_back(granuleN0);


  //@ 2. Creation du granule de niveau 1A/1B/1C (produit 
  //@ en sortie) par SD_DON_GranuleN1::InitEcritureN0N1()
  typeProduit = SD_DON_PRODUIT_N1_1A1B1C;
  granuleN1->SetNatureGranule(SD_DON_GranuleN1::SD_DON_GRANULE_N1_DATA);
  granuleN1->InitEcritureN0N1(gestionCasDegrade,
                              listeModele[0],
                              modeTraitement,
                              positionTraitement,
                              typeProduit,
                              m_OSV,
                              m_OBTUTC,
                              *granuleN0,
                              nbLigneAbsolu,
			      mphr);
        
  //CR 233 modification du de la nomenclatire du contexte
  m_dateFinN1=granuleN1->GetDateFin();       
  //@ Ajout du granule a la liste m_ListeGranuleN1S
  m_ListeGranuleN1S.push_back(granuleN1);

  //@ Remarque : on ne cree qu'un seul granule 1A/1B/1C
  //@ en sortie alors que 3 fichiers seront generes (on
  //@ utilise le product model 1C)


  //@ 3. Creation du granule ENG (produit en sortie) par
  //@ SD_DON_GranuleN1::InitEcritureN0N1()
  typeProduit = SD_DON_PRODUIT_N1_ENG;
  granuleENG->SetNatureGranule(SD_DON_GranuleN1::SD_DON_GRANULE_N1_ENG);
  granuleENG->InitEcritureN0N1(gestionCasDegrade,
                               listeModele[1],
                               modeTraitement,
                               positionTraitement,
                               typeProduit,
                               m_OSV,
                               m_OBTUTC,
                               *granuleN0,
                               nbLigneAbsolu,
			      mphr);
  
  //@ Ajout du granule a la liste m_ListeGranuleN1S
  m_ListeGranuleN1S.push_back(granuleENG);
        
 
  //@ 4. Creation du granule VER (produit en sortie) par 
  //@ SD_DON_GranuleN1::InitEcritureN0N1()
  typeProduit = SD_DON_PRODUIT_N1_VER;
  granuleVER->SetNatureGranule(SD_DON_GranuleN1::SD_DON_GRANULE_N1_VER);
  granuleVER->InitEcritureN0N1(gestionCasDegrade,
                               listeModele[2],
                               modeTraitement,
                               positionTraitement,
                               typeProduit,
                               m_OSV,
                               m_OBTUTC,
                               *granuleN0,
                               nbLigneAbsolu,
			       mphr);
  
  //@ Ajout du granule a la liste m_ListeGranuleN1S
  m_ListeGranuleN1S.push_back(granuleVER);
  
#ifdef OPS_DEBUG
  cout << "----------------------" << endl;
  cout << "Affichage des granules" << endl;
  granuleN0->Afficher();
  granuleN1->Afficher();
  granuleENG->Afficher();
  granuleVER->Afficher();
#endif

}
 

//@ ------------------------------------------------------------------*/
//@ Methode : InitIntermediaire
//@
//@ Description : Initialise les granules d'un traitement intermediaire
//@
//@ Parametres :
//@               (ENTREE) typeTraitement : Type du traitement 
//@                                         intermediaire
//@               (ENTREE) listeProduit : Liste des produits en entree 
//@               (ENTREE) listeModele : Liste des product model
//@               (ENTREE) t0 : Date de debut du traitement
//@               (ENTREE) t1 : Date de fin du traitement
//@               (ENTREE) modeTraitement : Mode du traitement
//@               (ENTREE) positionTraitement : Position du traitement
//@               (ENTREE) gestionCasDegrade : Gestionnaire de cas
//@                                            degrade
//@               (SORTIE) valeurPTSI : PTSI de la premiere ligne du
//@                                     granule
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GestGranule::InitIntermediaire(e_SD_DON_ProduitN1 typeTraitement,
                                           t_ListeChaine& listeProduit,
                                           const t_TableauChaine& 
                                           listeModele,
                                           const t_Date& t0, 
                                           const t_Date& t1, 
                                           const e_BAS_ModeTrt& 
                                           modeTraitement, 
                                           const e_BAS_PositionTrt&
                                           positionTraitement,
                                            const int nbLigneAbsolu,
                                           SD_DON_GestCasDeg& 
                                           gestionCasDegrade,
                                           int& valeurPTSI)
{
  // Fichier en entree
  t_Chaine           nomProduit("");

  // Granules en entree
  SD_DON_GranuleN1*  granuleEntreeN1  = new SD_DON_GranuleN1();
  SD_DON_GranuleN1*  granuleEntreeENG = new SD_DON_GranuleN1();
  
  // Granules en sortie
  SD_DON_GranuleN1*  granuleSortieN1  = new SD_DON_GranuleN1();
  SD_DON_GranuleN1*  granuleSortieENG = new SD_DON_GranuleN1();
  
  // Type de produit a generer
  e_SD_DON_ProduitN1 typeProduit;
  
  // Nature du produit en entree
  e_BAS_Nature       natureProduit;

  // Infos ligne
  t_InfoGenLigne     dataGenLigne;
 

  //@ 1. Creation du granule de niveau 1 (produit en 
  //@ entree) par SD_DON_GranuleN1::InitLecture()
  //@ et lecture du fichier associe
  nomProduit = listeProduit.front();

  //@ Identifier la nature du produit
  natureProduit = IdentifierProduitN1(nomProduit);
  granuleEntreeN1->SetNatureGranule(SD_DON_GranuleN1::
                                    SD_DON_GRANULE_N1_DATA);
  granuleEntreeN1->InitLecture(nomProduit,
                               natureProduit,
                               t0,
                               t1,
                               nbLigneAbsolu,
                               gestionCasDegrade);
    
  //@ Recuperation du PTSI de la premiere ligne par 
  //@ SD_DON_GranuleN1::GetDataGenLigne()
  valeurPTSI = -1;
  granuleEntreeN1->GetDataGenLigne(0, dataGenLigne);
  valeurPTSI = (int) dataGenLigne.GetPTSI();

  //@ Ajout du granule a la liste m_ListeGranuleN1E
  m_ListeGranuleN1E.push_back(granuleEntreeN1);

  
  //@ 2. Creation du granule de niveau ENG (produit en 
  //@ entree) par SD_DON_GranuleN1::InitLecture()
  //@ et lecture du fichier associe
  nomProduit = listeProduit.back();

  //@ Identifier la nature du produit
  natureProduit = IdentifierProduitN1(nomProduit);
  granuleEntreeENG->SetNatureGranule(SD_DON_GranuleN1::
                                     SD_DON_GRANULE_N1_ENG);
  granuleEntreeENG->InitLecture(nomProduit,
                                natureProduit,
                                t0,
                                t1,
                                nbLigneAbsolu,
                                gestionCasDegrade);
    
  //@ Appel a la methode SD_DON_Granule::GetEnteteMPHR()
  t_SD_BAS_MPHR mphr;
  mphr = (granuleEntreeENG)->GetEnteteMPHR();

  //@ Ajout du granule a la liste m_ListeGranuleN1E
  m_ListeGranuleN1E.push_back(granuleEntreeENG);


  //@ 3. Creation du granule de niveau 1 (produit en
  //@ sortie) par SD_DON_GranuleN1::InitEcritureN1N1()
  typeProduit = typeTraitement;

  granuleSortieN1->SetNatureGranule(SD_DON_GranuleN1::
                                    SD_DON_GRANULE_N1_DATA);
  granuleSortieN1->InitEcritureN1N1(gestionCasDegrade,
                                    listeModele[0],
                                    modeTraitement,
                                    positionTraitement,
                                    typeProduit,
                                    *granuleEntreeN1,
			      		mphr);
          
  //CR 233
  m_dateFinN1=granuleSortieN1->GetDateFin();       
  //@ Ajout du granule a la liste m_ListeGranuleN1S
  m_ListeGranuleN1S.push_back(granuleSortieN1);


  //@ 4. Creation du granule ENG (produit en sortie) par
  //@ SD_DON_GranuleN1::InitEcritureN1N1()
  typeProduit = SD_DON_PRODUIT_N1_ENG;
  
  granuleSortieENG->SetNatureGranule(SD_DON_GranuleN1::
                                     SD_DON_GRANULE_N1_ENG);
  granuleSortieENG->InitEcritureN1N1(gestionCasDegrade,
                                     listeModele[1],
                                     modeTraitement,
                                     positionTraitement,
                                     typeProduit,
                                     *granuleEntreeENG,
			      		mphr);
  
  //@ Ajout du granule a la liste m_ListeGranuleN1S
  m_ListeGranuleN1S.push_back(granuleSortieENG);
}


//@ ------------------------------------------------------------------*/
//@ Methode : IdentifierProduit
//@
//@ Description : Identifie la nature d'un produit de niveau 1
//@
//@ Parametres :
//@               (ENTREE) nomProduit : Nom du produit
//@
//@ Valeur de retour : 
//@               Nature d'un produit
//@
//@ ------------------------------------------------------------------*/
e_BAS_Nature SD_DON_GestGranule::IdentifierProduitN1(const t_Chaine& 
                                                     nomProduit)
{
  // Valeur de retour de la fonction
  e_BAS_Nature valRetour = NA_IASIL1C;


  //@ SI le fichier est de type 1A
  if (EstDeType(PREFIXE_1A, nomProduit) == CMN__VRAI)
    {
      valRetour = NA_IASIL1A;
    }
  //@ SINON SI le fichier est de type 1B
  else if (EstDeType(PREFIXE_1B, nomProduit) == CMN__VRAI)
    {
      valRetour = NA_IASIL1B;
    }
  //@ SINON SI le fichier est de type 1C
  else if (EstDeType(PREFIXE_1C, nomProduit) == CMN__VRAI)
    {
      valRetour = NA_IASIL1C;
    }
  //@ SINON SI le fichier est de type ENG
  else if (EstDeType(PREFIXE_ENG, nomProduit) == CMN__VRAI)
    {
      valRetour = NA_IASIL1ENG;
    }
  //@ SINON SI le fichier est de type VER
  else if (EstDeType(PREFIXE_VER, nomProduit) == CMN__VRAI)
    {
      valRetour = NA_IASIL1VER;
    }


  //@ Retourner la valeur de retour de la fonction
  return valRetour;
}


//@ ------------------------------------------------------------------*/
//@ Methode : EstDeType
//@
//@ Description : Teste si un fichier est du type passe en parametre
//@
//@ Parametres :
//@               (ENTREE) type : Type de fichier
//@               (ENTREE) nomFichier : Nom du fichier
//@
//@ Valeur de retour : 
//@               CMN__VRAI si le fichier est du type donne
//@               CMN__FAUX sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Booleen SD_DON_GestGranule::EstDeType(const t_Chaine& type,
                                             const t_Chaine& nomFichier)
{
  // Racine du nom du fichier
  t_Chaine       racine("");

  // Valeur de retour de la fonction
  t_CMN__Booleen valRetour = CMN__FAUX;


  //@ Recuperation du debut du nom court du fichier
  racine = nomFichier.substr(nomFichier.rfind(CMN__SEP_REPERTOIRE)+1, 
                             type.length());

  //@ Comparaison de la chaine recuperee avec le type
  //@ SI les deux chaines sont identiques
  if (racine == type)
    {
      //@ Positionner la valeur de retour a VRAI
      valRetour = CMN__VRAI;
    }
  //@ FIN SI les deux chaines sont identiques


  //@ Retourner la valeur de retour de la fonction
  return valRetour;
}
