// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_GestGranule.h,v 1.5 2013/03/29 16:55:26 lcaspar Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_GestGranule
//@
//@ Commentaire :
//@    Fichier d'entete de la classe SD_DON_GestGranule
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 06/01/2003 : P. CABANE
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-271-CN : 22/06/2005 : JL.PASCAL
//@ prise en compte du mode transition
//@ fiche de correction correspondante : IA-2100-CR-271-THA
//@ --------------------------------------------------------------------
//@ DM-ID :IA-2100-CR-286-CNE : 21/02/2006 : B. MASSART
//@ The environment variable DATE_MARGE_LIGNE_AVHRR is added. 
//@ See IA-2100-CR-286-THA.
//@ --------------------------------------------------------------------
//@ DM-ID :IA-2100-CR-308-CNE : 12/09/2006 : C. MERIGUET
//@ Lever le cas degrade 9 si le(s) fichier(s) AVHRR n'est
//@ pas accessible ou mauvais numero de version. Ne pas lever
//@ d'exception et poursuivre le traitement.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FT-2454-CN : 07/04/2008 : S.BRUNEL
//@ Critere de choix du fichier SVM erronne
//@ --------------------------------------------------------------------
//@ FA-ID :IA-FT-2466(FA-2100-OPS) : 30/05/2008 : S. BRUNEL
//@ Probleme de reproductibilite: SUBSAT_LATITUDE_START et
//@ SUBSAT_LONGITUDE_START
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2939(DM-2100-OPS) : 12/12/2011 : L. CASPAR
//@ Mise en place de conditions de levee du cas degarde 9 alternatives pour 
//@ correction de la NCR 12303
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_DON_GestGranule_h
#define SD_DON_GestGranule_h

//
// Fichiers include
//

// STL
#include <vector>

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN__Date.h"

#include "SD_BAS_Global.h"
#include "SD_BAS_Produits.h"
#include "SD_BAS_Niveau1.h"
#include "SD_BAS_AVHRR.h"
#include "SD_BAS_ConfStable.h"

#include "SD_DON_Granule.h"
#include "SD_DON_GranuleN1.h"
#include "SD_DON_GranuleAVHRR.h"
#include "SD_DON_PaquetN0.h"
#include "SD_DON_Traitement.h"
#include "SD_DON_GestCasDeg.h"
#include "SD_DON_OBTUTC.h"
#include "SD_DON_OSV.h"

#include "SD_DON_SVM.h"


class SD_DON_GestGranule {

    /*================================================================*/
    
 public:
    
    ////////////////////////////////
    // Type definitions
    ////////////////////////////////

    // Liste des granules 
    typedef vector<SD_DON_GranuleN0*>    t_ListeGranuleN0;
    typedef vector<SD_DON_GranuleN1*>    t_ListeGranuleN1;
    typedef vector<SD_DON_GranuleAVHRR*> t_ListeGranuleAVHRR;


    ////////////////////////////////
    // Constructeur
    ////////////////////////////////
    SD_DON_GestGranule();

    ////////////////////////////////
    // Destructeur
    ////////////////////////////////
    ~SD_DON_GestGranule();
    void Liberer();

    ////////////////////////////////
    // Accesseurs
    ////////////////////////////////
 
    t_Date               GetDateFin() const { return m_dateFinN1;};
    
    ////////////////////////////////
    // Methodes publiques
    ////////////////////////////////
    t_Date GetDateDebutIASI() const throw (ERR__Exception);
     // FA2454
    t_Date GetDateFinIASI() const throw (ERR__Exception);
    // ..FA2454
    int GetNbLigneIASI() const;
    int GetNbDataGap() const;
    int GetNbLigneAVHRR(const t_Date& t0, const t_Date& t1);
    int GetNbPaquetN0(const int noLigne, const e_SD_BAS_TypeN0 type);
    void GetInfoHKTM(t_TableauDate& startTime, 
                     t_TableauEntier& startOrbitNumber);
    void GetSensingTime(const int numLigne, 
                        t_TableauDate& sensingTime);
    void InitGranuleIASI(SD_DON_Traitement& traitement,
                         SD_DON_GestCasDeg& gestionCasDegrade,
                         int& valeurPTSI,
                         const int nbLigneAbsolu) 
        throw (ERR__Exception);
/* CR 308*/
    void InitBandeauAVHRR(SD_DON_Traitement& traitement,
			  SD_DON_GestCasDeg & GestCasDeg); 
/* Fin CR 308*/

    // FT-2939
    void GetBandeauAVHRR(t_Date& t0,
                         t_Date& t1,   
                         const int numLigne,
                         const int numSN,
                         SD_DON_GestCasDeg& gestionCasDegrade,
                         const int nbLigneMax,
                         SD_DON_SVM*  pSVM,
                         int& nbLigne,
                         t_SD_BAS_LigneAVHRR** listeLignes);
    // ..FT-2939
                         
    void GetLigne0(const int noLigne,
                   const e_SD_BAS_TypeN0 type,
                   int& nbPaquetN0,
                   t_SD_BAS_PaquetN0** listePaquetLigne);
    void DecodeDataLigne0(t_SD_BAS_PaquetN0* paquetLigne,
                          void* dataDecode);
    //CR271
    t_CMN__Booleen EstTransitionLigneS1(const int numLigne,
                                    const SD_DON_GranuleN1::
                                    e_SD_DON_GranuleN1 natureGranule)
        throw (ERR__Exception);
    void GetLigneS1(const int numLigne,
                    const SD_DON_GranuleN1::e_SD_DON_GranuleN1 
                    natureGranule,
                    t_SD_BAS_MPHR& mphr,
                    t_SD_BAS_LigneN1& ligneMDR,
                    t_SD_BAS_LigneN1& ligneVIADR) 
        throw (ERR__Exception);
    void EncodeDecodeLigneS1(const t_SD_BAS_ORIGINE numLigneSNPN, 
                             t_SD_BAS_LigneN1* ligne,
                             const e_SD_BAS_Atraiter niveau,
                             t_SD_BAS_ConfStable* paramDecodage,
                             const e_SD_BAS_Sens sens,
                             double* data);
	// FT-2466
    void SetEnteteS1(const t_SD_BAS_StatMphr* statMPHR,
                     const t_SD_BAS_ConfStable* paramConfStable,
                     int numeroLigne);
	// ..FT-2466
    void EcrireS1(const int idCA, 
                  t_SD_BAS_MPHR& mphr, 
                  t_ListeChaine& listeFichier);


    /*================================================================*/

 protected:

    ////////////////////////////////
    // Methodes protegees
    ////////////////////////////////

    ////////////////////////////////
    // Attributs proteges
    ////////////////////////////////


    /*================================================================*/

 private:
    
    //
    // Constante
    //
    static const t_Chaine JDB_PARAM_1;
    static const int      DATE_MARGE;
    static const int      DATE_MARGE_LIGNE_AVHRR;
    static const t_Chaine s_DATE_MARGE_LIGNE_AVHRR; /* CR286 */
    static const int      DATE_MARGE_CHARGEMENT_AVHRR;
    static const t_Chaine PREFIXE_OBTUTC;
    static const t_Chaine PREFIXE_OSV;
    static const t_Chaine PREFIXE_1A;
    static const t_Chaine PREFIXE_1B;
    static const t_Chaine PREFIXE_1C;
    static const t_Chaine PREFIXE_ENG;
    static const t_Chaine PREFIXE_VER;
    static const int      NB_DATA_IN_INTERMEDIAIRE;
    static const t_Chaine DATA_IN_NOT_FOUND;
    static const t_Chaine DATA_IN_NOT_ENOUGH;
    static const t_Chaine DATA_PROD_NOT_FOUND;
    static const t_Chaine DATA_AUX_OBTUTC_NOT_FOUND;

    ////////////////////////////////
    // Methodes privees
    ////////////////////////////////
    void ActualiserTraitement(SD_DON_Traitement& traitement, 
                              CMN__StageTM::e_CMN__StageId& 
                              typeTraitement, 
                              e_BAS_ModeTrt& modeTraitement, 
                              e_BAS_PositionTrt& positionTraitement,
                              t_Date& t0, 
                              t_Date& t1, 
                              t_ListeChaine& nomProduit, 
                              t_TableauChaine& listeModele) 
        throw (ERR__Exception);
    void RecupererProductModel(t_ListeChaine& listeProductModel,
                               const t_Chaine& typeModele,
                               t_TableauChaine& listeModele)
        throw (ERR__Exception);
    void InitOBTUTC(t_ListeChaine& listeDonneesOBTUTC,
                    t_Chaine& nomFichierOBTUTC)
        throw (ERR__Exception);
    void InitOSV(t_ListeChaine& listeDonneesOSV,
                 t_Chaine& nomFichierOSV);
    void InitComplet(const t_Chaine& nomProduit, 
                     const t_TableauChaine& nomModele,
                     const t_Date& t0, 
                     const t_Date& t1, 
                     const e_BAS_ModeTrt& modeTraitement, 
                     const e_BAS_PositionTrt& positionTraitement,
                     const int nbLigneAbsolu,
                     SD_DON_GestCasDeg& gestionCasDegrade,
                     int& valeurPTSI);
    void InitIntermediaire(e_SD_DON_ProduitN1 typeTraitement,
                           t_ListeChaine& listeProduit, 
                           const t_TableauChaine& nomModele,
                           const t_Date& t0, 
                           const t_Date& t1, 
                           const e_BAS_ModeTrt& modeTraitement,
                           const e_BAS_PositionTrt& positionTraitement,
                           const int nbLigneAbsolu,
                           SD_DON_GestCasDeg& gestionCasDegrade,
                           int& valeurPTSI);
    e_BAS_Nature IdentifierProduitN1(const t_Chaine& nomProduit);
    t_CMN__Booleen EstDeType(const t_Chaine& type, 
                             const t_Chaine& nomFichier);


    ////////////////////////////////
    // Attributs prives
    ////////////////////////////////

    // Liste des granules N0 en entree 
    t_ListeGranuleN0     m_ListeGranuleN0E;

    // Liste des granules N1 en entree 
    t_ListeGranuleN1     m_ListeGranuleN1E;
    
    // Liste des granules en sortie 
    t_ListeGranuleN1     m_ListeGranuleN1S;
    
    // Date de fin de la derniere ligne N1 
    t_Date              m_dateFinN1;

    // Liste des granules AVHRR 
    t_ListeGranuleAVHRR  m_ListeGranuleAVHRR;
    
    // Objet OBTUTC permettant la correction des dates 
    SD_DON_OBTUTC        m_OBTUTC;

    // Objet OSV permettant d'obtenir le mode plateforme
    SD_DON_OSV           m_OSV;
};

// objet fonction pour le sort de la STL
template <class gran>
struct inferieur : binary_function <gran, gran, bool>{

    /*================================================================*/
    
    bool operator () (const gran& gran1,const gran& gran2) 
   { return (*gran1 < *gran2); }
 
 };
#endif
