static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_Granule.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_Granule
//@ 
//@ Commentaire :
//@    Ce module gere la classe SD_DON_Granule, qui contient les
//@    informations communes a un granule de donnees
//@                                     
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 06/01/2003 : P. CABANE
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-146-CN : 10/11/2004 : B.MASSART
//@ Mauvaise orthographe de quelques champs du MPHR. Voir IA-2100-FC-146-THA.
//@ --------------------------------------------------------------------
//@ DM-ID : IA-2100-CR-292-CNE : 10/03/2006 : B.MASSART
//@ Function SD_ALG_AVHRR_GetSubSatLatLon is added to set up 
//@ MPHR_SUBSAT_LATITUDE/LONGITUDE_START/END fields in Level1 products.
//@ See IA-2100-CR-292-THA.
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL

// Standard

// Projet
#include "SD_DON_Granule.h"


//
// Definition des constantes
//


//======================================================================
//                                       METHODES PUBLIQUES 
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD_DON_Granule
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_Granule::SD_DON_Granule()
  : m_NomGranule(""), m_NbLigne(0), m_NbDataGap(0), m_TypeGran(INDETERMINE),
    m_IndiceDebut(0), m_IndiceFin(0)
{
  m_TP0.DateInit();
  m_TP1.DateInit();
  m_MPHR.grh.nb_var      = 0;
  m_MPHR.listevar.nb_var = 0;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_DON_Granule
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_Granule::~SD_DON_Granule()
{
  //@ On libere ici les listes eventuellement remplies
  m_ListeInfoGenLigne.clear();
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetNomGranule
//@
//@ Description : Recupere le nom du granule
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Nom du granule 
//@
//@ ------------------------------------------------------------------*/
t_Chaine SD_DON_Granule::GetNomGranule() const
{
  //@ Renvoie le nom du granule
  return m_NomGranule;
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetNbLigne
//@
//@ Description : Recupere le nombre de lignes du granule
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Nombre de lignes du granule 
//@
//@ ------------------------------------------------------------------*/
int SD_DON_Granule::GetNbLigne() const
{
  //@ Renvoie le nombre de lignes du granule
  return m_NbLigne;
}


//@ ------------------------------------------------------------------*/
//@ Methode : SetNbLigne
//@
//@ Description : Modifie le nombre de lignes du granule
//@
//@ Parametres :
//@               (ENTREE) nbLigne : Nombre de lignes
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_Granule::SetNbLigne(const int nbLigne)
{
  //@ Modification du nombre de lignes du granule
  m_NbLigne = nbLigne;
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetNbDataGap
//@
//@ Description : Recupere le nombre de trous de donnees du granule
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Nombre de trous de donnees du granule 
//@
//@ ------------------------------------------------------------------*/
int SD_DON_Granule::GetNbDataGap() const
{
  //@ Renvoie le nombre de trous de donnees du granule
  return m_NbDataGap;
}


//@ ------------------------------------------------------------------*/
//@ Methode : SetNbDataGap
//@
//@ Description : Modifie le nombre de trous de donnees du granule
//@
//@ Parametres :
//@               (ENTREE) nbDataGap : Nombre de trous de donnees
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_Granule::SetNbDataGap(const int nbDataGap)
{
  //@ Modification du nombre de trous de donnees du granule
  m_NbDataGap = nbDataGap;
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetTypeGran
//@
//@ Description : Recupere le type du granule
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Type du granule
//@
//@ ------------------------------------------------------------------*/
SD_DON_Granule::e_DON_TypeGran SD_DON_Granule::GetTypeGran() const
{
  //@ Renvoie le type du granule 
  return m_TypeGran;
}


//@ ------------------------------------------------------------------*/
//@ Methode : SetTypeGran
//@
//@ Description : Modifie le type du granule
//@
//@ Parametres :
//@               (ENTREE) typeGranule : Type du granule
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_Granule::SetTypeGran(const e_DON_TypeGran typeGranule)
{
  //@ Modification du type du granule
  m_TypeGran = typeGranule;
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetDateDebut
//@
//@ Description : Recupere la date de debut du granule
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Date de debut du granule
//@
//@ ------------------------------------------------------------------*/
t_Date SD_DON_Granule::GetDateDebut() const
{
  //@ Renvoie la date de debut du granule (m_TP0)
  return m_TP0;
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetDateFin
//@
//@ Description : Recupere la date de fin du granule
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Date de fin du granule
//@
//@ ------------------------------------------------------------------*/
t_Date SD_DON_Granule::GetDateFin() const
{
  //@ Renvoie la date de fin du granule (m_TP1)
  return m_TP1;
}


//@ ------------------------------------------------------------------*/
//@ Methode : SetDates
//@
//@ Description : Modifie les date de debut et de fin du granule
//@
//@ Parametres :
//@               (ENTREE) dateDebut : Date de debut
//@               (ENTREE) dateFin : Date de fin
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_Granule::SetDates(const t_Date& dateDebut,
                             const t_Date& dateFin)
{
  //@ Modification des dates du granule
  m_TP0 = dateDebut;
  m_TP1 = dateFin;
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetEnteteMPHR
//@
//@ Description : Recupere le MPH initial pour initialiser le MPH en
//@               sortie
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               MPH en sortie
//@
//@ ------------------------------------------------------------------*/
t_SD_BAS_MPHR& SD_DON_Granule::GetEnteteMPHR()
{
  //@ Renvoie du MPH en sortie
  return m_MPHR;
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetListeInfoGenLigne
//@
//@ Description : Recupere la liste des infos generales sur les lignes
//@               du granule
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Liste des infos generales
//@
//@ ------------------------------------------------------------------*/
t_ListeInfoGenLigne& SD_DON_Granule::GetListeInfoGenLigne()
{
  //@ Renvoie de la liste des infos generales
  return m_ListeInfoGenLigne;
}


//@ ------------------------------------------------------------------*/
//@ Methode : AjouterInfoGenLigne
//@
//@ Description : Ajoute un element dans la liste des infos generales
//@               sur les lignes du granule
//@
//@ Parametres :
//@               (ENTREE) element : Nouvel element
//@
//@ Valeur de retour : 
//@               Aucun
//@
//@ ------------------------------------------------------------------*/
void SD_DON_Granule::AjouterInfoGenLigne(t_InfoGenLigne& element)
{
  //@ Ajoute un element dans la liste des infos generales
  m_ListeInfoGenLigne.push_back(element);
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetInfoHKTM
//@
//@ Description : Recupere les informations pour les HKTM status
//@
//@ Parametres :
//@               (SORTIE) startTime
//@               (SORTIE) startOrbitNumber
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_Granule::GetInfoHKTM(t_Date& startTime, 
                                 int& startOrbitNumber)
{
  //@ Recuperation du t0 du granule
  startTime        = m_TP0;

  //@ Recuperation de MPHR_ORBIT_START
  startOrbitNumber = 
    m_MPHR.listevar.tab_var[MPHR_ORBIT_START].val.pi4[0];
}


//@ ------------------------------------------------------------------*/
//@ Methode : Afficher
//@
//@ Description : Afficher les infos d'un granule
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_Granule::Afficher()
{
  cout << "Granule: " << m_NomGranule << endl;
  cout << "Nombre de lignes: " << m_NbLigne << endl;
  cout << "Type granule    : " << m_TypeGran
       << " (0:?? 1:N0 2:N1 3:AVHRR)" << endl;
  cout << "Indice debut    : " << m_IndiceDebut << endl;
  cout << "Indice fin      : " << m_IndiceFin << endl;
  cout << "Date debut      : " << m_TP0.Chaine() << endl;
  cout << "Date fin        : " << m_TP1.Chaine() << endl;

  cout << "Nombre infos gen: " << m_ListeInfoGenLigne.size() << endl;
  for (int i=0; i<m_ListeInfoGenLigne.size(); i++)
    {
      cout << "\tInfos gen no" << i+1 << ":" << endl;
      m_ListeInfoGenLigne[i].Afficher();
    }
  cout << endl;
}


//@ ------------------------------------------------------------------*/
//@ Methode : AfficherMPHR
//@
//@ Description : Afficher les infos generales du MPHR d'un granule
//@
//@ Parametres :
//@               (ENTREE) complet : Afficher les infos detaillees
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_Granule::AfficherMPHR(t_CMN__Booleen complet)
{
  cout << "--------------------------------------------------" 
       << endl;
  cout << "----------------- Affichage MPHR -----------------" 
       << endl;
  cout << "Granule: " << m_NomGranule << endl << endl;

  cout << "PRODUCT_NAME                  : " << 
    m_MPHR.listevar.tab_var[MPHR_PRODUCT_NAME].val.s << endl;
  cout << "PARENT_PRODUCT_NAME_1         : " << 
    m_MPHR.listevar.tab_var[MPHR_PARENT_PRODUCT_NAME_1].val.s << endl;
  cout << "PARENT_PRODUCT_NAME_2         : " << 
    m_MPHR.listevar.tab_var[MPHR_PARENT_PRODUCT_NAME_2].val.s << endl;
  cout << "PARENT_PRODUCT_NAME_3         : " << 
    m_MPHR.listevar.tab_var[MPHR_PARENT_PRODUCT_NAME_3].val.s << endl;
  cout << "PARENT_PRODUCT_NAME_4         : " << 
    m_MPHR.listevar.tab_var[MPHR_PARENT_PRODUCT_NAME_4].val.s << endl;

  cout << "INSTRUMENT_ID                 : " << 
    m_MPHR.listevar.tab_var[MPHR_INSTRUMENT_ID].val.s << endl;
  cout << "INSTRUMENT_MODEL              : " << 
    m_MPHR.listevar.tab_var[MPHR_INSTRUMENT_MODEL].val.pui4[0] << endl;
  cout << "PRODUCT_TYPE                  : " << 
    m_MPHR.listevar.tab_var[MPHR_PRODUCT_TYPE].val.s << endl;
  cout << "PROCESSING_LEVEL              : " << 
    m_MPHR.listevar.tab_var[MPHR_PROCESSING_LEVEL].val.s << endl;
  cout << "SPACECRAFT_ID                 : " << 
    m_MPHR.listevar.tab_var[MPHR_SPACECRAFT_ID].val.s << endl;

  cout << "SENSING_START                 : " << 
    m_MPHR.listevar.tab_var[MPHR_SENSING_START].val.s << endl;
  cout << "SENSING_END                   : " << 
    m_MPHR.listevar.tab_var[MPHR_SENSING_END].val.s << endl;
  cout << "SENSING_START_THEORETICAL       : " << 
    m_MPHR.listevar.tab_var[MPHR_SENSING_START_THEORETICAL].val.s << endl;
  cout << "SENSING_END_THEORETICAL         : " << 
    m_MPHR.listevar.tab_var[MPHR_SENSING_END_THEORETICAL].val.s << endl;
  
  cout << "PROCESSING_CENTRE             : " << 
    m_MPHR.listevar.tab_var[MPHR_PROCESSING_CENTRE].val.s << endl;
  cout << "PROCESSOR_MAJOR_VERSION       : " << 
    m_MPHR.listevar.tab_var[MPHR_PROCESSOR_MAJOR_VERSION].val.pui4[0] 
       << endl;
  cout << "PROCESSOR_MINOR_VERSION       : " << 
    m_MPHR.listevar.tab_var[MPHR_PROCESSOR_MINOR_VERSION].val.pui4[0] 
       << endl;
  cout << "FORMAT_MAJOR_VERSION          : " << 
    m_MPHR.listevar.tab_var[MPHR_FORMAT_MAJOR_VERSION].val.pui4[0] 
       << endl;
  cout << "FORMAT_MINOR_VERSION          : " << 
    m_MPHR.listevar.tab_var[MPHR_FORMAT_MINOR_VERSION].val.pui4[0] 
       << endl;
  cout << "PROCESSING_TIME_START         : " << 
    m_MPHR.listevar.tab_var[MPHR_PROCESSING_TIME_START].val.s << endl;
  cout << "PROCESSING_TIME_END           : " << 
    m_MPHR.listevar.tab_var[MPHR_PROCESSING_TIME_END].val.s << endl;
  cout << "PROCESSING_MODE               : " << 
    m_MPHR.listevar.tab_var[MPHR_PROCESSING_MODE].val.s << endl;
  cout << "DISPOSITION_MODE              : " << 
    m_MPHR.listevar.tab_var[MPHR_DISPOSITION_MODE].val.s << endl;

  if (complet == CMN__VRAI)
    {
      AfficherInfosDetMPHR();
    }

  cout << "---------------------------------------------------" 
       << endl << endl;
}


//@ ------------------------------------------------------------------*/
//@ Methode : AfficherInfosDetMPHR
//@
//@ Description : Afficher les infos detaillees du MPHR d'un granule
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_Granule::AfficherInfosDetMPHR()
{
  cout << "RECEIVING_GROUND_STATION      : " << 
    m_MPHR.listevar.tab_var[MPHR_RECEIVING_GROUND_STATION].val.s 
       << endl;
  cout << "RECEIVE_TIME_START          : " << 
    m_MPHR.listevar.tab_var[MPHR_RECEIVE_TIME_START].val.s << endl;
  cout << "RECEIVE_TIME_END            : " << 
    m_MPHR.listevar.tab_var[MPHR_RECEIVE_TIME_END].val.s << endl;
  cout << "ORBIT_START                   : " << 
    m_MPHR.listevar.tab_var[MPHR_ORBIT_START].val.pui4[0] << endl;
  cout << "ORBIT_END                     : " << 
    m_MPHR.listevar.tab_var[MPHR_ORBIT_END].val.pui4[0] << endl;

  cout << "ACTUAL_PRODUCT_SIZE           : " << 
    m_MPHR.listevar.tab_var[MPHR_ACTUAL_PRODUCT_SIZE].val.pui4[0]
       << endl;

//   cout << "STATE_VECTOR_TIME             : " << 
//     m_MPHR.listevar.tab_var[MPHR_STATE_VECTOR_TIME].val.s << endl;
  cout << "SEMI_MAJOR_AXIS               : " << 
    m_MPHR.listevar.tab_var[MPHR_SEMI_MAJOR_AXIS].val.pi4[0] << endl;
  cout << "ECCENTRICITY                  : " << 
    m_MPHR.listevar.tab_var[MPHR_ECCENTRICITY].val.pi4[0] << endl;
  cout << "INCLINATION                   : " << 
    m_MPHR.listevar.tab_var[MPHR_INCLINATION].val.pr8[0] << endl;
  cout << "PERIGEE_ARGUMENT              : " << 
    m_MPHR.listevar.tab_var[MPHR_PERIGEE_ARGUMENT].val.pr8[0] << endl;
  cout << "RIGHT_ASCENSION               : " << 
    m_MPHR.listevar.tab_var[MPHR_RIGHT_ASCENSION].val.pr8[0] << endl;
  cout << "MEAN_ANOMALY                  : " << 
    m_MPHR.listevar.tab_var[MPHR_MEAN_ANOMALY].val.pr8[0] << endl;
  cout << "X_POSITION                    : " << 
    m_MPHR.listevar.tab_var[MPHR_X_POSITION].val.pr8[0] << endl;
  cout << "Y_POSITION                    : " << 
    m_MPHR.listevar.tab_var[MPHR_Y_POSITION].val.pr8[0] << endl;
  cout << "Z_POSITION                    : " << 
    m_MPHR.listevar.tab_var[MPHR_Z_POSITION].val.pr8[0] << endl;
  cout << "X_VELOCITY                    : " << 
    m_MPHR.listevar.tab_var[MPHR_X_VELOCITY].val.pr8[0] << endl;
  cout << "Y_VELOCITY                    : " << 
    m_MPHR.listevar.tab_var[MPHR_Y_VELOCITY].val.pr8[0] << endl;
  cout << "Z_VELOCITY                    : " << 
    m_MPHR.listevar.tab_var[MPHR_Z_VELOCITY].val.pr8[0] << endl;

  cout << "EARTH_SUN_DISTANCE_RATIO      : " << 
    m_MPHR.listevar.tab_var[MPHR_EARTH_SUN_DISTANCE_RATIO].val.pi4[0] 
       << endl;
  cout << "LOCATION_TOLERANCE_RADIAL     : " << 
    m_MPHR.listevar.tab_var[MPHR_LOCATION_TOLERANCE_RADIAL].val.pi4[0] 
       << endl;
  cout << "LOCATION_TOLERANCE_CROSSTRACK : " << 
    m_MPHR.listevar.tab_var[MPHR_LOCATION_TOLERANCE_CROSSTRACK].val.
    pi4[0] << endl;
  cout << "LOCATION_TOLERANCE_ALONGTRACK : " << 
    m_MPHR.listevar.tab_var[MPHR_LOCATION_TOLERANCE_ALONGTRACK].val.
    pi4[0] << endl;
  cout << "YAW_ERROR                     : " << 
    m_MPHR.listevar.tab_var[MPHR_YAW_ERROR].val.pr8[0] << endl;
  cout << "ROLL_ERROR                    : " << 
    m_MPHR.listevar.tab_var[MPHR_ROLL_ERROR].val.pr8[0] << endl;
  cout << "PITCH_ERROR                   : " << 
    m_MPHR.listevar.tab_var[MPHR_PITCH_ERROR].val.pr8[0] << endl;

  cout << "SUBSAT_LATITDE_START          : " << 
    m_MPHR.listevar.tab_var[MPHR_SUBSAT_LATITUDE_START].val.pr8[0]  /* CR292 */
       << endl;
  cout << "SUBSAT_LONGITUDE_START        : " << 
    m_MPHR.listevar.tab_var[MPHR_SUBSAT_LONGITUDE_START].val.pr8[0] 
       << endl;
  cout << "SUBSAT_LATITDE_END            : " << 
    m_MPHR.listevar.tab_var[MPHR_SUBSAT_LATITUDE_END].val.pr8[0] << endl; /* CR292 */
  cout << "SUBSAT_LONGITUDE_END          : " << 
    m_MPHR.listevar.tab_var[MPHR_SUBSAT_LONGITUDE_END].val.pr8[0] 
       << endl;
  cout << "LEAP_SECOND                   : " << 
    m_MPHR.listevar.tab_var[MPHR_LEAP_SECOND].val.pi4[0] << endl;
  cout << "LEAP_SECOND_UTC               : " << 
    m_MPHR.listevar.tab_var[MPHR_LEAP_SECOND_UTC].val.s << endl;

  cout << "TOTAL_RECORDS                 : " << 
    m_MPHR.listevar.tab_var[MPHR_TOTAL_RECORDS].val.pui4[0] << endl;
  cout << "TOTAL_MPHR                    : " << 
    m_MPHR.listevar.tab_var[MPHR_TOTAL_MPHR].val.pui4[0] << endl;
  cout << "TOTAL_SPHR                    : " << 
    m_MPHR.listevar.tab_var[MPHR_TOTAL_SPHR].val.pui4[0] << endl;
  cout << "TOTAL_IPR                     : " << 
    m_MPHR.listevar.tab_var[MPHR_TOTAL_IPR].val.pui4[0] << endl;
  cout << "TOTAL_GEADR                   : " << 
    m_MPHR.listevar.tab_var[MPHR_TOTAL_GEADR].val.pui4[0] << endl;
  cout << "TOTAL_GIADR                   : " << 
    m_MPHR.listevar.tab_var[MPHR_TOTAL_GIADR].val.pui4[0] << endl;
  cout << "TOTAL_VEADR                   : " << 
    m_MPHR.listevar.tab_var[MPHR_TOTAL_VEADR].val.pui4[0] << endl;
  cout << "TOTAL_VIADR                   : " << 
    m_MPHR.listevar.tab_var[MPHR_TOTAL_VIADR].val.pui4[0] << endl;
  cout << "TOTAL_MDR                     : " << 
    m_MPHR.listevar.tab_var[MPHR_TOTAL_MDR].val.pui4[0] << endl;
	
  cout << "COUNT_DEGRADED_INST_MDR       : " << 
    m_MPHR.listevar.tab_var[MPHR_COUNT_DEGRADED_INST_MDR].val.pui4[0] 
       << endl;
  cout << "COUNT_DEGRADED_PROC_MDR       : " << 
    m_MPHR.listevar.tab_var[MPHR_COUNT_DEGRADED_PROC_MDR].val.pui4[0] 
       << endl;
  cout << "COUNT_DEGRADED_INST_MDR_BLOCKS: " << 
    m_MPHR.listevar.tab_var[MPHR_COUNT_DEGRADED_INST_MDR_BLOCKS].val.
    pui4[0] << endl;
  cout << "COUNT_DEGRADED_PROC_MDR_BLOCKS: " << 
    m_MPHR.listevar.tab_var[MPHR_COUNT_DEGRADED_PROC_MDR_BLOCKS].val.
    pui4[0] << endl;
	
  cout << "DURATION_OF_PRODUCT           : " << 
    m_MPHR.listevar.tab_var[MPHR_DURATION_OF_PRODUCT].val.pui4[0] 
       << endl;
  cout << "MILLISECONDS_OF_DATA_PRESENT  : " << 
    m_MPHR.listevar.tab_var[MPHR_MILLISECONDS_OF_DATA_PRESENT].val.
    pui4[0] << endl;
  cout << "MILLISECONDS_OF_DATA_MISSING  : " << 
    m_MPHR.listevar.tab_var[MPHR_MILLISECONDS_OF_DATA_MISSING].val.
    pui4[0] << endl;
  cout << "SUBSETTED_PRODUCT             : " << 
    m_MPHR.listevar.tab_var[MPHR_SUBSETTED_PRODUCT].val.pui4[0] << endl;
}


//======================================================================
//                                      METHODES PRIVEES 
//======================================================================

