static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_GranuleAVHRR.cc,v 1.5 2013/03/29 16:55:26 lcaspar Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_GranuleAVHRR
//@ 
//@ Comment :
//@    include file for the SD_DON_GranuleAVHRR class
//@    used to reference the AVHRR PDU read OPS L1 PPF
//@                                     
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 27/01/2003 : P.GASSIE
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-65-CN : 30/09/2003 : B. MASSART
//@ on verifie la validite de la ligne AVHRR avant d'afficher des infos
//@ detaillees dans GetDataLigne. Corrige egalement IA-2100-FA-076-CN.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-308-CN : 12/09/2006 : C. MERIGUET
//@ Lever le cas degrade 9 si le(s) fichier(s) AVHRR n'est
//@ pas accessible ou maivais numero de version. Ne pas lever
//@ d'exception et poursuivre le traitement.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-162-CN : 09/2006 : C.MERIGUET
//@ Gestion des IPR en mode DUMP
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2512 (FA-2100-OPS) : 17/11/2008 : S. BRUNEL
//@ Fuites memoires signalees par EUMETSAT (EUM.EPS.NCR.3627.1)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2939(DM-2100-OPS) : 12/12/2011 : L. CASPAR
//@ Mise en place de conditions de levee du cas degarde 9 alternatives pour 
//@ correction de la NCR 12303
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <iostream>

#include "SD_DON_GranuleAVHRR.h"

//
// Definition des constantes
//
const t_Chaine SD_DON_GranuleAVHRR::JDB_PARAM_1 = "$PRM1";
const t_Chaine SD_DON_GranuleAVHRR::DATA_READ_ERROR = "AVHRR file read error";


//======================================================================
//                                       METHODES PUBLIQUES 
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Methode : SD_DON_GranuleAVHRR
//@
//@ Description : Constructeur
//@
//@ Parameters :
//@                     Aucun
//@
//@ Return Status : 
//@                     Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_GranuleAVHRR::SD_DON_GranuleAVHRR() 
  : m_IndiceDeb(-1), m_NbLignesUtiles(-1)
{
  //@ Mise a jour du type du granule
  m_TypeGran = SD_DON_Granule::AVHRRN1;

  //@ initialisation des listes
  m_ListedescLignes.clear();
  m_ListeLignesMDR.clear();
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_DON_GranuleAVHRR
//@
//@ Description : Destructeur
//@
//@
//@ Parameters :
//@                     Aucun
//@
//@ Return Status : 
//@                     Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_GranuleAVHRR::~SD_DON_GranuleAVHRR()
{
  //@ on libere ici les listes eventuellement remplies
  m_ListedescLignes.clear();
  m_ListeLignesMDR.clear();
}


//@ ------------------------------------------------------------------*/
//@ Methode : Liberer
//@
//@ Description : Liberation memoire des donnees de l'objet
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_GranuleAVHRR::Liberer()
{
  t_ListeLignes::iterator it;

  //@ POUR toutes les lignes du granule
  for (it = m_ListeLignesMDR.begin();
       it != m_ListeLignesMDR.end();
       it ++)
    {
      //@ Liberation memoire des donnees
      (*it)->Liberer();
      delete *it;
    }
  //@ FIN POUR toutes les lignes du granule
  
  //@ RAZ de la liste
  m_ListeLignesMDR.clear();

  // FT-2512
  SD_BAS_LibereListeVar(ECRITURE, &m_MPHR.listevar);		
  SD_BAS_LibereListeVar(ECRITURE, &m_MPHR.grh);		
  // ..FT-2512
}


//@ ------------------------------------------------------------------*/
//@ Methode : InitLecture
//@
//@ Description : Cette methode permet d'initialiser l'acces a un
//@ granule AVHRR de niveau 1B 
//@
//@
//@ Parameters :
//@                     (ENTREE)  NomProduit : nom du produit
//@                     (ENTREE)  t0 : date de debut a traiter ( issu wo )
//@                     (ENTREE)  t1 : date de fin a traiter ( issu wo)
//@                     ces dates sont utilisees en particulier dans le cas 
//@                     du mode dump ou de gros volumes sont dispo et ou il
//@                     ne faut charge que les donnees necessaires 
//@                     (ENTREE)  GestCasDeg : Gestionnaire de cas degrade
//@
//@ Return Status : 
//@                     Aucune
//@                     t_CMN__Statut 
//@                     CMN__STATUT_OK si la fonction a reussi
//@                     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
/* CR 308 + FA 162 */
t_CMN__Statut SD_DON_GranuleAVHRR::InitLecture(const t_Chaine& 
                                               nomProduitAVHRR,
                                               const t_Date& t0,
                                               const t_Date& t1,
					       SD_DON_GestCasDeg & GestCasDeg,
					       const e_BAS_ModeTrt ModeTraitement) throw(ERR__Exception)
/* ..CR 308 + FA 162 */
{
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  t_SD_BAS_DataThread ctx;
  t_SD_BAS_Fd  Descripteur;
  t_ListeDescLignes::iterator It;


  // Entree dans la fonction 
  InitialiserCompteRendu(&(ctx.cr));

  strcpy(m_NomGranuleAVHRR, nomProduitAVHRR.c_str());

/* CR 308 + FA 162*/
  //@ Chargement des donnees generales des lignes
  cr_op = ChargementDataGen(t0, t1, ModeTraitement);

  //@ SI erreur
  if(cr_op != CMN__STATUT_OK)
    {
      //@ lever le cas degrade 9
      GestCasDeg.GetCasDegLigne(0)->SetDetecte(9, t0, 0, 0, 0);      
      //@ emission d'un message vers le JdBS
      JDB__Client MessageJDB;
      MessageJDB.SetParametre(JDB_PARAM_1, m_NomGranuleAVHRR);
      MessageJDB.EcrireMessage(SD_DATA_SET, SD__Message::MSG_DATA_LOAD_AVHRR_ERR);
      MessageJDB.Liberer();

      //@ Ne pas generer d'exception
       cr_op = CMN__STATUT_OK;
      
      //@ goto fin
      goto fin;     
    }
  //@ FIN SI

/* ..CR 308 + FA 162 */

  //@ Ouverture en lecture du produit
/* FA 162 */
  cr_op = SD_BAS_OuvrirProduit(LECTURE, 
                               NA_AVHRR, 
                               m_NomGranuleAVHRR,
                               &Descripteur, 
                               &ctx,
			       ModeTraitement);
/*.. FA 162 */ 
  //@ SI erreur
  if(cr_op != CMN__STATUT_OK)
    {
      //@ goto fin
      goto fin;     
    }
  //@ FIN SI

  //@ lecture  et initialisation du MPHR du granule m_MPHR
  cr_op = SD_BAS_LireMPHRAVHRR(&Descripteur, &m_MPHR, &ctx);

  //@ SI erreur
  if(cr_op != CMN__STATUT_OK)
    {
      //@ goto fin
      goto fin;     
    }
  //@ FIN SI
  //@ initialisation de l'iterateur
  It = m_ListedescLignes.begin();

  //@ POUR tous les MDR
  for(int i = 0; i < Descripteur.nombreMDR; i++)
    {
      //@ SI la date de la ligne correspond a l'intervalle de temps
      if(i >= m_IndiceDeb && i <= m_IndiceDeb + m_NbLigne - 1)
        {
          //@ creation  d'une ligne AVHRR
          SD_DON_LigneAVHRR *ligneAVHRRTmp = new SD_DON_LigneAVHRR;

          //@ lire le MDR suivant
          cr_op = ligneAVHRRTmp->Init(&Descripteur, 
                                      It->date, 
                                      It->validite, 
                                      It->validate);
          //@ SI erreur
          if(cr_op != CMN__STATUT_OK)
            {
              //@ on continue le traitement, CG9 est leve pour la ligne
              cr_op = CMN__STATUT_OK;
            }
          //@ FIN SI

          //@ Ajout de la ligne a la liste
          m_ListeLignesMDR.insert(m_ListeLignesMDR.end(), 
                                  ligneAVHRRTmp);
        }
      //@ SINON
      else
        {
          t_SD_BAS_LigneAVHRR LigneTmp;

          //@ lire le MDR suivant pour faire suivre l'offset
          cr_op = SD_BAS_LireMDRGenAVHRR(INIT, 
                                         0, 
                                         &Descripteur, 
                                         &LigneTmp.InfoGene, 
                                         &ctx);
          SD_BAS_LibereListeVar(LECTURE, &LigneTmp.InfoGene);

          //@ SI erreur
          if(cr_op != CMN__STATUT_OK)
            {
              //@ goto fin
              goto fin;     
            }
          //@ FIN SI
        }
      //@ FIN SI
    
      It ++;   
    }
  //@ FIN POUR
  
  //@ fermeture du fichier  AVHRR en lecture  (SD_BAS_AVHRR)
/* FA 162 */
  cr_op = SD_BAS_FermerProduit(LECTURE, &Descripteur, &ctx,ModeTraitement);
/* ..FA 162 */
  //@ SI erreur 
  if(cr_op != CMN__STATUT_OK)
    {
      //@ goto fin
      goto fin;     
    }
  //@ FIN SI

  // on trace ici si besoin toutes les variables en sortie


 fin:
 
  LibererCompteRendu(&(ctx.cr.msgJDB));
  
  //@ SI erreur 
  if(cr_op != CMN__STATUT_OK)
    {
      //@ emission d'un message vers le JdBS
      JDB__Client MessageJDB;
      MessageJDB.SetParametre(JDB_PARAM_1, m_NomGranuleAVHRR);
      MessageJDB.EcrireMessage(SD_DATA_SET, SD__Message::MSG_DATA_LOAD_AVHRR_ERR);
      MessageJDB.Liberer();
  
      //@ exception
      ERR__Exception exception;
   
      //@ Emission exception SD_DON_TMT_DATA_AUX
      exception.SetErrorCode(ERR__Exception::SD_DON_TMT_DATA_AUX);
      exception.SetParametre(DATA_READ_ERROR);
      THROW(exception);     
    }
  //@ FIN SI

  // sortie de la fonction 
  return(cr_op); 
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetNbLignesUtiles
//@
//@ Description : Cette methode permet de recuperer le nombre de lignes 
//@ contenue dans le granule entre 2 dates t0 et t1
//@
//@
//@ Parameters :
//@                     (ENTREE)  t0 : date de debut des  lignes 
//@                     (ENTREE)  t1 : date de fin des  lignes
//@
//@ Return Status : 
//@                     int Nb 
//@                     ( -1 si aucunes lignes)
//@
//@ ------------------------------------------------------------------*/
int SD_DON_GranuleAVHRR::GetNbLignesUtiles(const t_Date t0,
                                           const t_Date t1)
{
int indice = 0;
t_ListeLignes::iterator It;
t_Date t0Tmp(t0);
t_Date t1Tmp(t1);
// Entree dans la fonction 
  //@ POUR toutes les lignes de la liste
  for(It = m_ListeLignesMDR.begin();
      It != m_ListeLignesMDR.end();
      It ++)
  {
    //@ SI la date de la ligne est comprise entre t0 et t1
    if((*It)->GetInfoGenLigne().Date >= t0Tmp
        && (*It)->GetInfoGenLigne().Date <= t1Tmp)
    {
      //@ indice++
      indice++;
    }
    //@ FIN SI
  }
  //@ FIN POUR
  m_NbLignesUtiles = indice;
fin:
  // sortie de la fonction 
return(m_NbLignesUtiles); 

}
//@ ------------------------------------------------------------------*/
//@ Methode : GetDataLigne
//@
//@ Description : Cette methode permet de recuperer les donnees detaillees 
//@ de date comprise entre les 2 dates t0 et t1 , c'est a ce niveau que se fait
//a la detection du cas degrade CG9 et CG15. Les donnees sont ici decodees
//@
//@
//@ Parameters :
//@                     (ENTREE)  t0 : date de debut des  lignes 
//@                     (ENTREE)  t1 : date de fin des  lignes
//@                     (ENTREE)  numlig : numero de la ligne IASI pour
//@                     laquelle la donnee est demandee
//@                     (ENTREE)  numsn : numero du SN de l'image IASI pour
//@                     laquelle la donnee est demandee
//@                     (ENTREE)  GestCasDeg : Gestionnaire de cas degrade
//@                     (ENTREE)  pSVM : pointeur sur les donnees lues dans le fichier SVM
//@                     (ENTREE/SORTIE)  ListeLignes : Liste des lignes retournees
//@                     (tableau de pointeurs alloues par l'appelant)
//@                     ( remarque si = NULL alors rien n'est retourne )
//@                     (alloue par l'appelant)
//@
//@ Return Status : 
//@                     Aucune
//@
//@ ------------------------------------------------------------------*/
// FT-2939
t_CMN__Statut SD_DON_GranuleAVHRR::GetDataLigne(const t_Date t0,
                                                const t_Date t1,
                                                const int  numLigne,
                                                const int  numlsn,
                                                SD_DON_GestCasDeg& 
                                                gestCasDeg,
                                                SD_DON_SVM*  pSVM,
                                                t_SD_BAS_LigneAVHRR**
                                                listeLignes
                                                )
// ..FT-2939
{
t_CMN__Statut cr_op=CMN__STATUT_OK;
t_ListeLignes::iterator It;
int indice = 0;
t_Date t0Tmp(t0);
t_Date t1Tmp(t1);

// listeLignes est utilise pour plusieurs granules donc il faut commencer
// a stocker les infos du granule apres celles du precedent
// Entree dans la fonction 

//@     cette methode est appelee apres GetNbLignesUtiles de facon a permettre 
//@     a l'appelant une allocation du tableaux de lignes

  //@ POUR tous les elements de la liste m_ListeLignesMDR
  for (It = m_ListeLignesMDR.begin();
       It != m_ListeLignesMDR.end();
       It ++)
  { 
    //@ SI la date de la ligne est comprise entre t0 et t1
    if((*It)->GetInfoGenLigne().Date >= t0Tmp
        && (*It)->GetInfoGenLigne().Date <= t1Tmp)
    {
      t_SD_BAS_LigneAVHRR* ligne = new t_SD_BAS_LigneAVHRR;

      //@ recuperation des donnees detaillees de la ligne
      //@ l'adresse est recopiee dans la listes des lignes
      //(*It)->GetData(numLigne, numlsn, gestCasDeg, listeLignes[indice]);
      
      // FT-2939
      (*It)->GetData(numLigne, numlsn, gestCasDeg, pSVM, (*ligne));
      // ..FT-2939
      listeLignes[indice] = ligne;

#ifdef DEBUG
      cout << "****************" << endl;
      cout << "SD_DON_GranuleAVHRR::GetDataLigne(): " << endl;
  
      cout << "Day : " << ligne->InfoGene.tab_var[AVHRR_DATEDEB].val.pdate->day << endl;
      cout << "Millis : " << ligne->InfoGene.tab_var[AVHRR_DATEDEB].val.pdate->millis << endl;
  
      /* SI la ligne AVHRR est valide ALORS */
      if (*(ligne->InfoGene.tab_var[AVHRR_VALIDREC].val.pui4) == VALIDE)
      {
        cout << "Quality : " << 
          ligne->InfoDet.tab_var[AVHRR_QUALITY_INDICATOR].val.pui4[0] << endl;
        cout << "Navigation  : " << 
          ligne->InfoDet.tab_var[AVHRR_NAVIGATION_STATUS].val.pui4[0] << endl;
        cout << "Scan line : " << 
          ligne->InfoDet.tab_var[AVHRR_SCAN_LINE_QUALITY].val.pui4[0] << endl;
        cout << "Calibration : " << 
          ligne->InfoDet.tab_var[AVHRR_CALIBRATION_QUALITY].val.pui4[0] << endl;
        cout << "Digital B : " << 
          ligne->InfoDet.tab_var[AVHRR_DIGITAL_B_DATA].val.pui4[0] << endl;
        cout << "****************" << endl;
      }
#endif

      //@ indice++
      indice++;
    }
    //@ FIN SI
  }
  //@FIN POUR chaque element de la liste
//@ remarque ici la chronologie est assuree par la coherence du produit
// on trace ici si besoin toutes les variables en sortie
fin:
// sortie de la fonction 
return(cr_op); 

}

//@ ------------------------------------------------------------------*/
//@ Methode : operator <
//@
//@ Description : Teste si le granule est anterieur au granule
//@               passe en parametre
//@
//@ Parametres :
//@               (ENTREE) unGranule : Granule de comparaison
//@
//@ Valeur de retour : 
//@               bool : Resultat du test
//@
//@ ------------------------------------------------------------------*/
bool SD_DON_GranuleAVHRR::operator <(const SD_DON_GranuleAVHRR& unGranule)
{
  
  return (m_DatePremiereLigne < unGranule.m_DatePremiereLigne);
}


//======================================================================
//                                      METHODES PRIVEES 
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : ChargementDataGen
//@
//@ Description : Cette methode permet de charger les donnees generales
//@               des lignes entre deux dates
//@
//@
//@ Parameters :
//@                     (ENTREE)  t0 : date de debut issu du wo
//@                     (ENTREE)  t1 : date de fin issu du wo
//@
//@ Return Status:
//@                     t_CMN__Statut 
//@                     CMN__STATUT_OK si la fonction a reussi
//@                     CMN__STATUT_NOK si la fonction a echoue 
//@
//@ ------------------------------------------------------------------*/
/* FA 162 */
t_CMN__Statut SD_DON_GranuleAVHRR::ChargementDataGen(const t_Date& t0,
                                                     const t_Date& t1,
						     const e_BAS_ModeTrt ModeTraitement)

/* ..FA 162 */
{
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  t_SD_BAS_DataThread ctx;
  t_SD_BAS_Fd Descripteur;

  //@ Ouverture en lecture du produit
  InitialiserCompteRendu (&(ctx.cr));

/* FA 162 */
  cr_op = SD_BAS_OuvrirProduit(LECTURE, 
                               NA_AVHRR, 
                               m_NomGranuleAVHRR,
                               &Descripteur, 
                               &ctx,
			       ModeTraitement);
/* ..FA 162 */ 
  //@ SI erreur
  if(cr_op != CMN__STATUT_OK)
    {
      //@ emission des messages du ctx
      EnvoyerMessagesStockes(&(ctx.cr.msgJDB));

      //@ goto fin
      goto fin;     
    }
  //@ FIN SI

  //@ lecture  et initialisation du MPHR du granule m_MPHR
  cr_op = SD_BAS_LireMPHRAVHRR(&Descripteur, &m_MPHR, &ctx);

  //@ SI erreur
  if(cr_op != CMN__STATUT_OK)
    {
      //@ emission des messages du ctx
      EnvoyerMessagesStockes(&(ctx.cr.msgJDB));

      //@ goto fin
      goto fin;     
    }
  //@ FIN SI

  //@ gestion de l'overlap ( pour determiner la premiere et 
  //@ la derniere ligne du granule a traiter )
  cr_op = GestionOverlap(&Descripteur, t0, t1);
 
  //@ SI erreur
  if(cr_op != CMN__STATUT_OK)
    {
      //@ goto fin
      goto fin;     
    }
  //@ FIN SI
  
  //@ Fermeture du fichier
/* FA 162 */
  cr_op = SD_BAS_FermerProduit(LECTURE, &Descripteur, &ctx,ModeTraitement);
/* ..FA 162 */  
  //@ SI erreur
  if(cr_op != CMN__STATUT_OK)
    {
      //@ goto fin
      goto fin;     
    }
  //@ FIN SI


 fin:

  //@ liberation du compte rendu
  LibererCompteRendu(&(ctx.cr.msgJDB));

  // sortie de la fonction 
  return(cr_op); 
}

//@ ------------------------------------------------------------------*/
//@ Methode : GestionOverlap
//@
//@ Description : Cette methode permet de determiner les dates de 
//@             debut et fin des lignes utilises dans le traitement
//@             de niveau 1.
//@             On prende en compte ici le cas des dump qui se traduit
//@             par le choix de date t0 et t1 inclue dans le produit
//@
//@
//@ Parameters :
//@                     (ENTREE)  Descripeur : descripteur du fichier
//@                     (ENTREE)  t0 : date de debut issu du wo
//@                     (ENTREE)  t1 : date de fin issu du wo
//@
//@ Return Status: 
//@                     t_GLB_Statut 
//@                     CMN__STATUT_OK si la fonction a reussi
//@                     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_DON_GranuleAVHRR::GestionOverlap(t_SD_BAS_Fd* Descripteur,
                                                  const t_Date& t0,
                                                  const t_Date& t1)
{
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  t_SD_BAS_LigneAVHRR LigneTmp;
  t_SD_BAS_DataThread* ctx = new t_SD_BAS_DataThread;
  t_Date tp0, tp1;
  t_ListeDescLignes::iterator It;
  bool bInit = false; // indicateur d'initialisation d'overlap
  int indicedeb = -1;
  int indicefin = -1;
  // Entree dans la fonction 


  //@ Ouverture du fichier AVHRR en lecture  (SD_BAS_AVHRR)
  //@ lire le premier ISP en ne decodant la date UTC
  //@ (MDR)(SD_BAS_AVHRR)
  //@ ( on ne stocke pas ici la donnee elle meme)
  InitialiserCompteRendu (&(ctx->cr));

  //@ POUR tous les MDR
  for(int i = 0; i < Descripteur->nombreMDR; i++)
    {
      t_DescLigneAVHRR descLigne;

      //@ lire le MDR suivant en ne decodant que le type, la date UTC
      //@ et le mode (SD_BAS_AVHRR)
      //@ ( on ne stocke pas ici la donnee elle meme)
      cr_op = SD_BAS_LireMDRGenAVHRR(INIT, 
                                     0, 
                                     Descripteur, 
                                     &LigneTmp.InfoGene, 
                                     ctx);

      //@ SI erreur
      if(cr_op != CMN__STATUT_OK)
        {
          //@ emission des messages du ctx
          EnvoyerMessagesStockes(&(ctx->cr.msgJDB));
          LibererCompteRendu(&(ctx->cr.msgJDB));
          
          //@ goto fin
          goto fin;     
        }
      //@ FIN SI

     

      //@  ajouter un element dans le tableau des  descripteur de lignes
      //@  initialiser date
      //@  initialiser num ligne et validite
      t_SCDSTime dateTmp;
      dateTmp.day = LigneTmp.InfoGene.tab_var[AVHRR_DATEDEB].val.pdate->day;
      dateTmp.millis = LigneTmp.InfoGene.tab_var[AVHRR_DATEDEB].val.pdate->millis;
      (descLigne.date).SCDSTime(dateTmp);
      descLigne.numLigne = i;
      descLigne.validite = *LigneTmp.InfoGene.tab_var[AVHRR_VALIDREC].val.pui4; 
      descLigne.validate = *LigneTmp.InfoGene.tab_var[AVHRR_FLAG].val.pui4; 


      //@ liberation des infos generales
      SD_BAS_LibereListeVar(LECTURE, &LigneTmp.InfoGene);

      //@ ajout de la description de ligne fdans la liste
      m_ListedescLignes.insert(m_ListedescLignes.end(), descLigne);
      //@ SI premiere ligne
      if(i == 0)
        {
          //@ enregistrement de la date
          m_DatePremiereLigne = descLigne.date;
        }
      //@ FIN SI
    }
  //@ FIN POUR


  //@ liberation du compte rendu
  LibererCompteRendu(&(ctx->cr.msgJDB));


  //@ POUR chaque element de la liste m_ListedescLignes
  for (It = m_ListedescLignes.begin();
       It != m_ListedescLignes.end();
       It ++)
    {
      //@ SI Date >= t0  et bInit=false (Date <= t0  et bInit1=false)
      if((It->date >= t0) && (bInit == false))
        {
          //@ m_TP0=Date
          tp0 = It->date;
          //@ m_IndiceDeb=NumISPDeb
          m_IndiceDeb = It->numLigne;
          //@ indicedeb=numligne
          indicedeb = It->numLigne;
          //@ bInit=true
          bInit = true;
        }
      //@ FIN SI
      //@ SI Date <= t1 
      if(It->date <= t1)
        {
          //@ m_TP1=Date
          tp1 = It->date;
          //@ indicefin=numligne
          indicefin = It->numLigne;
        }
      //@ FIN SI
      //@ SI Date <= t1 et Date <=t0
      if(It->date <=t0 && It->date <= t1)
        {
          //@ indicefin=indicedeb=-1
          indicedeb = -1;
          indicefin = -1;
        }
      //@ FIN SI
    } 
  //@ FIN POUR
  //@ m_NbLignes=indicefin - indicedeb +1 ( methode heritee)
  this->SetNbLigne(indicefin - indicedeb + 1);
  //@ SI  indicefin=indicedeb=-1 
  if(indicefin == -1 || indicedeb == -1)
    {
      //@ alors le fichier n'est pas utilise
      //@   m_NbLignes=-1
      this->SetNbLigne(-1);
      //@   m_indicedeb=-1
      m_IndiceDeb = -1;
      //@ cr_op=CMN__STATUT_NOK;
      cr_op = CMN__STATUT_NOK;
    }
  //@ mettre a jour les dates de debut et de fin par this.SetDates
  this->SetDates(tp0, tp1);
  // on trace ici si besoin toutes les variables en sortie
#ifdef OPS_DEBUG
  cout << "Nombre de lignes : " << m_NbLigne << endl;
  cout << "T0 : " << tp0.Chaine() << endl;
  cout << "T1 : " << tp1.Chaine() << endl;
#endif
 fin:
  // sortie de la fonction 
  delete ctx;
  return(cr_op); 

}
