//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_GranuleAVHRR.h,v 1.5 2013/03/29 16:55:26 lcaspar Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_GranuleAVHRR
//@
//@ Commentaire :
//@    include file for the SD_DON_GranuleAVHRR class
//@    used to reference the AVHRR PDU read OPS L1 PPF
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 27/01/2003 : P.GASSIE
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-308-CN : 12/09/2006 : C. MERIGUET
//@ Lever le cas degrade 9 si le(s) fichier(s) AVHRR n'est
//@ pas accessible ou mauvais numero de version. Ne pas lever
//@ d'exception et poursuivre le traitement.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-162-CN : 09/2006 : C.MERIGUET
//@ Gestion des IPR en mode DUMP
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2939(DM-2100-OPS) : 12/12/2011 : L. CASPAR
//@ Mise en place de conditions de levee du cas degarde 9 alternatives pour 
//@ correction de la NCR 12303
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_DON_GranuleAVHRR_h
#define SD_DON_GranuleAVHRR_h


// Inserer ici les fichiers include
#include <list> 
#include "SD_BAS_Global.h"
#include "CMN__Date.h"
#include "SD_BAS_AVHRR.h"
#include "SD_DON_LigneAVHRR.h"
#include "SD_DON_GestCasDeg.h"
#include "SD_BAS_Produits.h"
#include "SD_DON_Granule.h"
#include "SD_DON_SVM.h"


// Definition des exceptions

class SD_DON_GranuleAVHRR : public SD_DON_Granule {
    
    /*================================================================*/
    
 public:
    
    //
    // Type
    // Descripteur de lignes
    typedef struct
    {
        t_Date date;
        int    numLigne;
        int   validite; //DUMMY ou pas (VALIDE ou INVALIDE)
        int   validate; // validite temporelle 
    } t_DescLigneAVHRR;

    //
    // Type
    //      
    typedef list<t_DescLigneAVHRR> t_ListeDescLignes;
    typedef list<SD_DON_LigneAVHRR*> t_ListeLignes;

    //
    // Constructeur
    //
    SD_DON_GranuleAVHRR();
    
    //
    // Destructeur
    //
    ~SD_DON_GranuleAVHRR();
    virtual void Liberer();
    
    //
    // Accesseurs
    //
    int GetNbLignesUtiles(const t_Date t0,
                          const t_Date t1);

    // FT-2939
    t_CMN__Statut GetDataLigne(const t_Date t0,
                               const t_Date t1,
                               const int  numLigne,
                               const int  numlsn,
                               SD_DON_GestCasDeg& gestCasDeg,
                               SD_DON_SVM*  pSVM,
                               t_SD_BAS_LigneAVHRR** listeLignes);
    // ..FT-2939

    //
    // Methodes publiques
    //
/* CR 308 + FA 162*/
    t_CMN__Statut InitLecture(const t_Chaine& 
                              nomProduitAVHRR,
                              const t_Date& t0,
                              const t_Date& t1,
			      SD_DON_GestCasDeg & GestCasDeg,
			      const e_BAS_ModeTrt ModeTraitement) throw(ERR__Exception);
/* ..CR 308 + FA 162*/
                              
    bool operator <(const SD_DON_GranuleAVHRR& unGranule);


    /*================================================================*/

 protected:

    //
    // protected Methods 
    //
    
    //
    // protected Attributes 
    //

    
    /*================================================================*/

 private:
    //
    // Constantes
    //
    static const t_Chaine JDB_PARAM_1;
    static const t_Chaine DATA_READ_ERROR;
    //
    // Private Methods 
    //
/* FA 162 */
    t_CMN__Statut ChargementDataGen(const t_Date& t0,
                                    const t_Date& t1,
				    const e_BAS_ModeTrt ModeTraitement);

/* ..FA 162 */

    t_CMN__Statut GestionOverlap(t_SD_BAS_Fd * Descripteur,
                                 const t_Date& t0,
                                 const t_Date& t1);

    //
    // Private Attributes 
    //
    
    // date de la premiere ligne du granule
    t_Date m_DatePremiereLigne;
      
    // liste des record de type MDR utilisees dans le granule
    t_ListeLignes m_ListeLignesMDR;
    
    // tableau de descripteur des lignes a traiter
    t_ListeDescLignes  m_ListedescLignes; 
    
    // nom du granule 
    t_SD_BAS_NomProduit m_NomGranuleAVHRR;
    
    // indice du premier MDR ( pour acces rapide)    
    int m_IndiceDeb;   
    
    // nombre de ligne utiles du granule    
    int m_NbLignesUtiles;
};

#endif
