// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_GranuleN0.h,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_GranuleN0
//@
//@ Commentaire :
//@    Fichier d'entete de la classe SD_DON_GranuleN0
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 06/01/2003 : P. CABANE
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-118-CN : 06/02/2004 : JL.PASCAL
//@ l'OPS prend en compte  la derniere ligne meme si elle est incomplete
//@ fiche de correction correspondante : IA-2100-FC-118-THA
//@ --------------------------------------------------------------------
//@ DM-ID :IA-CR-2100-246-CNE : 06/07/2004 : B. MASSART
//@ word # 16 in level 0 PX packets is no more read as part of PTSI number
//@ Variable ONBOARD_SOFTWARE_VERSION added in ENG, 1A, 1B and 1C products
//@ --------------------------------------------------------------------
//@ DM-ID :IA-CR-2100-243-CNE : 02/08/2004 : B. MASSART
//@ Line identification uses N0_LN in ISP packets. See IA-2100-CR-243-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-168-CN : 21/02/2006 : B.MASSART
//@ an exception is added in method DeterminerBornes when time range specified
//@ in a workorder does not match the date of the last line of the granule.
//@ See IA-2100-FC-168-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-183-CN : 08/08/2006 : C.MERIGUET
//@ Ne pas lever d'exception en mode dump si aucune lignes n'a ete trouvee
//@ pour empecher le rejet du work order global et l'arret de l'OPS en cas 
//@ de trou de donnees.
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-3353(DM-2100-OPS) : 22/05/2017 : MPE
//@ Gestion Dummy MDR et MDR en overlap => Msg et sortie en erreur de l'OPS
//@ ---------------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_DON_GranuleN0_h
#define SD_DON_GranuleN0_h


//
// Fichiers include
//

// STL

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN__Date.h"

#include "SD_DON_Granule.h"
#include "SD_DON_LigneN0.h"
#include "SD_DON_PaquetN0.h"
#include "SD_DON_OBTUTC.h"
#include "SD_DON_GestCasDeg.h"

/* FA 183 */
#include "SD_BAS_Global.h"
/* Fin FA 183 */

class SD_DON_GranuleN0 : public SD_DON_Granule {

    /*================================================================*/

 public:

    ////////////////////////////////
    // Constructeur
    ////////////////////////////////
    SD_DON_GranuleN0();

    ////////////////////////////////
    // Destructeur
    ////////////////////////////////
    ~SD_DON_GranuleN0();
    virtual void Liberer();

    ////////////////////////////////
    // Accesseurs
    ////////////////////////////////
    
    
    ////////////////////////////////
    // Methodes publiques
    ////////////////////////////////
    int GetNbPaquetN0(const int numLigne, 
                      const e_SD_BAS_TypeN0 type);
    void GetDataGenLigne(const int numLigne,
                         t_InfoGenLigne& dataGenLigne);
    void GetDataLigne(const int numLigne,
                      const e_SD_BAS_TypeN0 type,
                      int& nbPaquets,
                      t_SD_BAS_PaquetN0** listePaquetLigne);
/* FA 183 */
    void InitLecture(const t_Chaine& nomProduit,
                     const t_Date& t0,
                     const t_Date& t1,
                     SD_DON_OBTUTC& OBTUTC,
                     SD_DON_GestCasDeg& gestionCasDegrade,
		     const e_BAS_ModeTrt& modeTraitement)
        throw (ERR__Exception);
/* Fin FA 183 */
    void DecodeDataLigne(t_SD_BAS_PaquetN0* paquetLigne,
                         void* dataDecode);
    void Afficher();


    /*================================================================*/

 protected:

    ////////////////////////////////
    // Methodes protegees
    ////////////////////////////////

    ////////////////////////////////
    // Attributs proteges
    ////////////////////////////////


    /*================================================================*/

 private:
     
    ////////////////////////////////
    // Type definitions
    ////////////////////////////////

    // Etat interne de l'automate gerant l'overlap
    typedef enum 
    {
        EVT_INDETERMINE,
        EVT_DEBUT_LIGNE,
        EVT_FIN_LIGNE
    } e_EtatGranule;

    ////////////////////////////////
    // Constante
    ////////////////////////////////
    static const t_Chaine JDB_PARAM_1;
    static const t_Chaine JDB_PARAM_2;
    static const t_Chaine EXCEPTION_SEPARATEUR;
    static const t_Chaine ERROR_INIT_READING;
    static const t_Chaine NO_LINE_IN_PRODUCT;
    static const t_Chaine ERROR_IN_TIME_RANGE; /* FA168 */
    static const int NB_ISP_LIGNE_NORM_COMPLETE;
    static const int NB_ISP_LIGNE_CAL_COMPLETE;
    static const t_Chaine ERROR_DUMMY_NEXT_MDR_INCONSISTENT; // DM 3353

    ////////////////////////////////
    // Methodes privees
    ////////////////////////////////
    SD_DON_LigneN0& GetLigne(const int numLigne) throw (ERR__Exception);
/* FA 183 */
    void            GestionOverlap(const t_Date& t0,
                                   const t_Date& t1,      
                                   SD_DON_OBTUTC& OBTUTC,
                                   t_SD_BAS_Fd& ficDesc,
                                   t_SD_BAS_DataThread* ctx,
				   const e_BAS_ModeTrt& modeTraitement)
        throw (ERR__Exception);
/* Fin FA 183 */
    void            IdentifierLignes(const int indiceMDR,
                                     const t_Date& t0, /* CR243 */
                                     const t_Date& t1,
                                     const t_SD_BAS_Liste_Var&
                                     donneesGenerales,
                                     const t_Date& dateMDR, 
                                     t_CMN__Booleen& dateDepassee);
    void            CreerLigneDummy(const int indiceMDR,
                                    const unsigned int offset,
                                    const t_Date& dateDebut,
                                    const t_Date& dateFin);
    
    /* CR246 ajout de version_logiciel_bord */
    void            CreerLigne(const int indiceMDR,
                               const unsigned int offset,
                               const unsigned int PTSI,
                               const unsigned int version_logiciel_bord,
                               const e_BAS_ModePdv mode,
                               const unsigned int LN, /* CR243 */
                               const t_Date& dateMDR);
    void            DeterminerBornes(const t_Date& t0,
                                     const t_Date& t1)
        throw (ERR__Exception);


    ////////////////////////////////
    // Attributs prives
    ////////////////////////////////
 
    // Etat interne de l'automate gerant l'overlap
    e_EtatGranule  m_EtatGranule;

    // Liste des lignes utilisees dans le granule
    t_ListeLigneN0 m_ListeLigneN0;
    
    // CR243: Indice de ligne absolu precedent
    static int m_N0_LN_prec;
};

#endif
