// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_GranuleN1.h,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_GranuleN1
//@
//@ Commentaire :
//@    Fichier d'entete de la classe SD_DON_GranuleN1
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 06/01/2003 : P. CABANE
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 04/09/2003 : [reference :] Benjamin MASSART
//@ cette version prend en compte les evolutions du code pour une gestion
//@ dynamique de l'ecriture des IPR dans les produits de niveau 1 suite
//@ aux modifications apparues dans la version 6.4 du GPFS et la version
//@ 6.5 du IASI Level 1 Product Format Specification
//@ --------------------------------------------------------------------
//@ DM-ID : PM-3 : 04/09/2003 :
//@ -ajout de la methode DecrireIPR
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-271-CN : 22/06/2005 : JL.PASCAL
//@ prise en compte du mode transition
//@ fiche de correction correspondante : IA-2100-CR-271-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-162-CN : 09/2006 : C.MERIGUET
//@ Gestion des IPR en mode DUMP
//@ --------------------------------------------------------------------
//@ FA-ID :IA-FT-2466(FA-2100-OPS) : 30/05/2008 : S. BRUNEL
//@ Probleme de reproductibilite: SUBSAT_LATITUDE_START et
//@ SUBSAT_LONGITUDE_START
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_DON_GranuleN1_h
#define SD_DON_GranuleN1_h


//
// Fichiers include
//

// STL

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN__Date.h"

#include "SD_BAS_Global.h"

#include "SD_DON_Granule.h"
#include "SD_DON_GranuleN0.h"
#include "SD_DON_LigneN1.h"
#include "SD_DON_OBTUTC.h"
#include "SD_DON_GestCasDeg.h"


class SD_DON_GranuleN1 : public SD_DON_Granule {

    /*================================================================*/

 public:
    
    ////////////////////////////////
    // Type definitions
    ////////////////////////////////

    // Type definissant les types de produits a generer
    typedef enum 
        {
            SD_DON_GRANULE_N1_DATA,
            SD_DON_GRANULE_N1_ENG,
            SD_DON_GRANULE_N1_VER
        } e_SD_DON_GranuleN1; 
    
    // Type liste de lignes N1
    typedef vector<SD_DON_LigneN1> t_ListeLigneN1;

    ////////////////////////////////
    // Constructeur
    ////////////////////////////////
    SD_DON_GranuleN1();

    ////////////////////////////////
    // Destructeur
    ////////////////////////////////
    ~SD_DON_GranuleN1();
    virtual void Liberer();

    ////////////////////////////////
    // Accesseurs
    ////////////////////////////////
    e_SD_DON_GranuleN1 GetNatureGranule() const;
    void SetNatureGranule(e_SD_DON_GranuleN1 nature);
    e_SD_DON_ProduitN1 GetTypeProduit() const;
    int GetNbLigneVIADR() const;
        
    ////////////////////////////////
    // Methodes publiques
    ////////////////////////////////
    void GetDataGenLigne(const int numLigne,
                         t_InfoGenLigne& dataGenLigne);
    t_CMN__Booleen EstTransitionLigne(const int numLigne); //CR271
    void GetDataLigne(const int numLigne,
                      t_SD_BAS_LigneN1& ligneMDR,
                      t_SD_BAS_LigneN1& ligneVIADR);
    void InitLecture(const t_Chaine& nomProduit,
                     const e_BAS_Nature natureProduit,
                     const t_Date& t0,
                     const t_Date& t1,
                     const int nbLigneAbsolu,
                     SD_DON_GestCasDeg& gestionCasDegrade) 
        throw (ERR__Exception);
    void InitEcritureN0N1(SD_DON_GestCasDeg& gestionCasDegrade,
                          const t_Chaine& nomModele,
                          const e_BAS_ModeTrt modeTraitement,
                          const e_BAS_PositionTrt positionTraitement,
                          const e_SD_DON_ProduitN1 typeProduit,
                          SD_DON_OSV& objetOSV,
                          SD_DON_OBTUTC& OBTUTC,
                          SD_DON_GranuleN0& granuleN0,
                          const int nbLigneAbsolu,
			  t_SD_BAS_MPHR& mphr);
    void InitEcritureN1N1(SD_DON_GestCasDeg& gestionCasDegrade,
                          const t_Chaine& nomModele,
                          const e_BAS_ModeTrt modeTraitement,
                          const e_BAS_PositionTrt positionTraitement,
                          const e_SD_DON_ProduitN1 typeProduit,
                           SD_DON_GranuleN1& granuleN1,
			  t_SD_BAS_MPHR& mphr);
   // FT-2466
   void SetEntete(const t_SD_BAS_StatMphr* statMPHR,
                   const t_SD_BAS_ConfStable* paramConfStable,
                   const SD_DON_OBTUTC& OBTUTC,
                   int numeroLigne);
   // ..FT-2466
    void Ecrire(const int idCA, t_ListeChaine& listeFichier);
    void EncodeDecodeDataLigne(const t_SD_BAS_ORIGINE numLignesNPN,
                               t_SD_BAS_LigneN1* ligne,
                               const e_SD_BAS_Atraiter niveau, 
                               t_SD_BAS_ConfStable* paramDecodage,
                               const e_SD_BAS_Sens sens,
                               double* data);
    void Afficher();


    /*================================================================*/

 protected:

    ////////////////////////////////
    // Methodes protegees
    ////////////////////////////////

    ////////////////////////////////
    // Attributs proteges
    ////////////////////////////////


    /*================================================================*/

 private:
    
    ////////////////////////////////
    // Constante
    ////////////////////////////////
    static const t_Chaine DIR_DATA_TMP;
    static const t_Chaine JDB_PARAM_1;
    static const t_Chaine JDB_PARAM_2;
    static const t_Chaine EXCEPTION_SEPARATEUR;
    static const t_Chaine ERROR_INIT_READING;
    static const t_Chaine ERROR_INIT_WRITING;
    static const t_Chaine ERROR_END_WRITING;
    static const t_Chaine NO_LINE_IN_PRODUCT;
    static const t_Chaine ERROR_DUMP_FILENAME;
    static const t_Chaine ASCII_PM_BACKLOG;
    static const t_Chaine ASCII_PM_REPROCESSING;
    static const t_Chaine ASCII_DM_OPERATIONNAL;
    static const t_Chaine ASCII_DM_COMMISSIONING;
    static const t_Chaine PROC_LEVEL_1A;
    static const t_Chaine PROC_LEVEL_1B;
    static const int      NB_MAX_PRODUIT;
    static const int      TAILLE_FICHIER_PRODUIT;
  
    ////////////////////////////////
    // Methodes privees
    ////////////////////////////////
    void GetDataLigneMDR(const int numLigne,
                         t_SD_BAS_LigneN1& ligneMDR);
    void GetDataLigneVIADR(const int numLigne,
                           t_SD_BAS_LigneN1& ligneVIADR);
    void GetDataGIADR(const e_BAS_TypeRecN1 typeRecord,
                      t_SD_BAS_LigneN1& dataGIADR);
    SD_DON_LigneN1& GetLigneMDR(const int numLigne) 
        throw (ERR__Exception);
    SD_DON_LigneN1& GetLigneVIADR(const int numLigne) 
        throw (ERR__Exception);
    void            GestionOverlap(const t_Date& t0,
                                   const t_Date& t1,    
                                   t_SD_BAS_Fd& ficDesc,
                                   t_SD_BAS_DataThread* ctx);
    void            DeterminerBornes(const t_Date& t0,
                                     const t_Date& t1);
    void            LireProductModel(const t_Chaine& nomModele,
			  t_SD_BAS_MPHR& mphr)
        throw (ERR__Exception);
    void            InitGIADR() throw (ERR__Exception);
    void            EcrireGIADR(t_SD_BAS_Fd& ficDesc,
                                  const e_BAS_Nature natureProduit)
        throw (ERR__Exception);
    void            RecupererGIADR(SD_DON_GranuleN1& granuleN1)
        throw (ERR__Exception);
    e_BAS_PROCMODE  GetProcessingMode();
    e_BAS_DISPMODE  GetDispositionMode();
    void            InitLigne(const int numLigne,
                              SD_DON_OSV& objetOSV,
                              SD_DON_GranuleN0& granuleN0,
                              SD_DON_GestCasDeg& gestionCasDegrade,
                              const e_BAS_PROCMODE processingMode,
                              const e_BAS_DISPMODE dispositionMode,
                              const int nbLigneAbsolu,
                              SD_DON_OBTUTC& OBTUTC);
    void            InitLigneVIADR(const int numLigneVIADR,
                                   const int numLigneMDR,
                                   SD_DON_OSV& objetOSV,
                                   SD_DON_GranuleN0& granuleN0,
                                   SD_DON_GestCasDeg& gestionCasDegrade,
                                   const e_BAS_PROCMODE processingMode,
                                   const e_BAS_DISPMODE dispositionMode,
                                   const int nbLigneAbsolu,
                                   SD_DON_OBTUTC& OBTUTC);
    void            IdentifierProduit(int& nbProduit, 
                                      e_BAS_Nature natureProduit[]);
    void            ModifierNomProduit(const e_BAS_Nature natureProduit,
                                       t_Chaine& nomProduit) 
      throw (ERR__Exception);
    void            EcrireProduit(const int idCA,
                                  const e_BAS_TypeAcces 
                                  typeAccesOuverture,
                                  const e_BAS_TypeAcces 
                                  typeAccesFermeture,
                                  const e_BAS_Nature natureProduit,
                                  t_ListeChaine& listeFichier)
        throw (ERR__Exception);
    void            EcrireDonnees(t_SD_BAS_Fd& ficDesc, 
                                  const int idCA, 
                                  const e_BAS_Nature natureProduit);
    void            FinaliserMPHR(const t_Chaine& nomProduitEntree,
                                  const t_Chaine& nomProduitSortie);
    t_Chaine        NomComplet(const t_Chaine& nomFichier) 
        throw (ERR__Exception);
    t_Chaine        NomCourt(const t_Chaine& nomFichier);
    void            TraiterErreurBAS(t_SD_BAS_DataThread* ctx,
                                     const ERR__Exception::e_Code 
                                     codeException,
                                     const t_Chaine paramMessage)
        throw (ERR__Exception);
 
    void            DecrireIPR(t_SD_BAS_Fd * descripteur_fichier);
/* FA 162*/
    void            DecrireIPRDump(t_SD_BAS_Fd * descripteur_fichier,
                                    t_SD_BAS_DataThread* ctx);
/* ..FA 162*/

    ////////////////////////////////
    // Attributs prives
    ////////////////////////////////
    
    // Donnees d'entree
    t_Chaine           m_NomOrigine;

    // Nombre de lignes absolu depuis le debut du dump
    // si m_NbLigneAbsolu + numLigne % 10 = 0 alors une ligne VIADR
    // a ete creee 
    int                m_NbLigneAbsolu; 
    
    // Nature du granule (DATA, ENG, VER)
    e_SD_DON_GranuleN1 m_NatureGranule;
    
    // Type des produits a generer pour le granule: 1A/1B/1C, 
    // 1B/1C, 1C, VER ou ENG
    e_SD_DON_ProduitN1 m_TypeProduit;
   
    // Mode de traitement: granule ou dump (issu de
    // SD_DON_traitement)
    e_BAS_ModeTrt      m_ModeTraitement;
 
    // Position du traitement: FIRST, MIDDLE ou LAST
    e_BAS_PositionTrt  m_PositionTraitement;

    // Type d'acces: LECTURE ou ECRITURE
    e_BAS_TypeAcces    m_TypeAcces;

    // Liste des records de type MDR utilises dans le granule
    t_ListeLigneN1     m_ListeLigneMDR;
    
    // Liste des records de type VIADR utilises dans le granule
    t_ListeLigneN1     m_ListeLigneVIADR;
    
    // Contenu des GIADR du produit de N1
    // Selon les produits on n'utilise que certains champs
    t_SD_BAS_LigneN1   m_GIADRENG;
    t_SD_BAS_LigneN1   m_GIADRQUA;
    t_SD_BAS_LigneN1   m_GIADRSCA;
     // Date de debut de generation du produit
    t_Date              m_Tdeb;

     // Date de fin de generation du produit
    t_Date              m_Tfin;

/* FA 162*/
    static unsigned int     m_NbIPR1A;      // nb d'ipr detectes 
    static unsigned int     m_NbIPR1B;      // nb d'ipr detectes 
    static unsigned int     m_NbIPR1C;      // nb d'ipr detectes 
    static unsigned int     m_NbIPRVER;      // nb d'ipr detectes 
    static unsigned int     m_NbIPRENG;      // nb d'ipr detectes 
    static e_BAS_TypeRecN1  m_TypeLastRecord_1A; // type du dernier record du granule precedent
    static e_BAS_TypeRecN1  m_TypeLastRecord_1B; // type du dernier record du granule precedent
    static e_BAS_TypeRecN1  m_TypeLastRecord_1C; // type du dernier record du granule precedent
    static e_BAS_TypeRecN1  m_TypeLastRecENG; // type du dernier record du granule precedent
    static e_BAS_TypeRecN1  m_TypeLastRecVER; // type du dernier record du granule precedent
/* ..FA 162*/
};

#endif
