static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_InventaireConf.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_InventaireConf
//@ 
//@ Comment :
//@    this module implement the SD_DON_InventaireConf class
//@    It is the central point for acces to configuration file
//@	coherence between them is also verified here
//@	 needed by OPS L1 PPF
//@ 					
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ VERSION : 12/12/2002 : P.GASSIE
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <iostream>

#include "SD_DON_InventaireConf.h"

// Definition des constantes
const char *SD_DON_InventaireConf::SD_DON_VAR_ENV_AUX_DATA = "AUX_DATA";
const char *SD_DON_InventaireConf::SD_DON_VAR_ENV_CTX_DATA = "CTX_DATA";

//======================================================================
//					 METHODES PUBLIQUES 
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Methode : SD_DON_InventaireConf
//@
//@ Description : Constructeur
//@
//@ Parameters :
//@			Aucun
//@
//@ Return Status : 
//@			Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_InventaireConf::SD_DON_InventaireConf()
{
  // initialisation des attributs
  m_PTSI = -1;
}
//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_DON_InventaireConf
//@
//@ Description : Destructeur
//@
//@
//@ Parameters :
//@			Aucun
//@
//@ Return Status : 
//@			Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_InventaireConf::~SD_DON_InventaireConf()
{
//  m_GestConfAutre.LibererMutex();  
}
//@ ------------------------------------------------------------------*/
//@ Methode : CreerInventaire
//@
//@ Description : Cette methode permet d'initialiser l'acces aux donnees 
//@ de configuration autre . Ici les fichiers sont justes repertories 
//@ sans lire le contenu precis
//@
//@
//@ Parameters :
//@			aucune
//@
//@ Return Status : 
//@
//@ ------------------------------------------------------------------*/
void SD_DON_InventaireConf::CreerInventaire(
			)

{
static const char tcProcName_F[] = "SD_DON_InventaireConf::CreerInventaire";
t_CMN__Statut cr_op = CMN__STATUT_OK;
// Entree dans la fonction 

  //@ on recupere ici la valeur de la variable d'environnement SD_DON_VAR_ENV_AUX_DATA
  //@ et on la stocke dans l'attribut m_RepAuxData
  m_RepAuxData = CFG__Configuration::GetRepertoire(SD_DON_VAR_ENV_AUX_DATA);
  //@ on initialise le gestionaire de configuration autre
  m_GestConfAutre.CreerInventaire(m_RepAuxData);
  // mise en place du mutex pour les acces
//  m_GestConfAutre.InitialiserMutex(); 

// on trace ici si besoin toutes les variables en sortie
#ifdef OPS_DEBUG
cout << endl << "SD_DON_InventaireConf::CreerInventaire" << endl;
cout << "Repertoire des aux data : " << m_RepAuxData.c_str() << endl;
#endif	
// sortie de la fonction 
}

//@ ------------------------------------------------------------------*/
//@ Methode : InitConf
//@
//@ Description : Cette methode permet d'initialiser l'acces aux donnees 
//@ de configuration pour le dump . Ici les fichiers sont charges  
//@ en fonction du PTSI issu du contexte 
//@
//@
//@ Parameters :
//@			(ENTREE)  Ptsi : Ptsi du fichier a charger
//@			(ENTREE)  tcourante : date courante 
//@			(Sortie)  IdCA : id conf autre 
//@
//@ Return Status : 
//@
//@ ------------------------------------------------------------------*/
void SD_DON_InventaireConf::InitConf(
		const int& Ptsi,
		SD_DON_Traitement& Trt,
		const t_Date & tcourante,
		int& IdCA
			)

{
static const char tcProcName_F[] = "SD_DON_InventaireConf::InitConf";
t_CMN__Statut cr_op=CMN__STATUT_OK;
t_Chaine nom_fichier("");
t_Chaine nom_fichier1("");
t_Chaine nom_fichier2("");
int IdCS = -1;
int IdBS = -1;
// Entree dans la fonction 
  //@ on charge le fichier de conf autre correspondant au PTSI et a la date  
  //@ courante
  m_GestConfAutre.ChargerPTSIDateValid(Ptsi,
                                       tcourante,
                                       nom_fichier,
                                       IdCA,
                                       IdCS,
                                       IdBS);
  #ifdef OPS_DEBUG
  cout << endl << "SD_DON_InventaireConf::InitConf" << endl;
  cout << "IdCA : " << IdCA << endl;
  cout << "IdCS : " << IdCS << endl;
  cout << "IdBS : " << IdBS << endl;
  #endif	
  //@ ajout du fichier conf autre a la liste des fichiers en entree
  //@ de SD_DON_Traitement
  if ( !nom_fichier.empty() )
  {
    Trt.AjouterProduitE(nom_fichier);
  }
  
  //@ on charge le fichier de conf stable correspondant a l'id et a la date  
  //@ courante et present dans le repertoire m_RepAuxData
  m_ConfStable.Initialise(IdCS, tcourante, m_RepAuxData,nom_fichier1);
  m_ConfStable.Lire();
  //@ ajout du fichier conf stable a la liste des fichiers en entree
  //@ de SD_DON_Traitement
  if ( !nom_fichier1.empty() )
  {
    Trt.AjouterProduitE(nom_fichier1);
  }
  
  //@ on charge la banque spectrale correspondant a l'id   
  //@ courant et present dans le repertoire m_RepAuxData
  m_BanqueSpectrale.Lire(IdBS, m_RepAuxData, nom_fichier2);
  //@ ajout du fichier banque spectrale a la liste des fichiers en entree
  //@ de SD_DON_Traitement
  if ( !nom_fichier2.empty() )
  {
    Trt.AjouterProduitE(nom_fichier2);
  }
  
  //@ on initialise ici m_PTSI
  m_PTSI = Ptsi;

// on trace ici si besoin toutes les variables en sortie
#ifdef OPS_DEBUG
cout << "************** SD_DON_InventaireConf::InitConf *****" << endl;
cout << "Lecture conf stable : " << m_ConfStable.GetData()->IDefStableParamDate.day << endl;
#endif	
// sortie de la fonction 
}

//@ ------------------------------------------------------------------*/
//@ Methode : ChargerConf
//@
//@ Description : Cette methode permet d'initialiser la configuration 
//@ en cours de traitement (pour chaque granule) 
//@ Ici les fichiers de conf autre sont eventuellement charges  
//@ les autres fichiers sont juste verifies 
//@
//@
//@ Parameters :
//@			(ENTREE)  Ptsi : Ptsi du fichier a charger
//@			(ENTREE)  tcourante : date courante 
//@			(Sortie)  IdCA : id conf autre 
//@
//@ Return Status : 
//@
//@ ------------------------------------------------------------------*/
void SD_DON_InventaireConf::ChargerConf(
		const int& Ptsi,
		SD_DON_Traitement& Trt,
		const t_Date & tcourante,
		int& IdCA
			)

{
static const char tcProcName_F[] = "SD_DON_InventaireConf::ChargerConf";
t_CMN__Statut cr_op=CMN__STATUT_OK;
t_Chaine nom_fichier("");
int IdCS = -1;
int IdBS = -1;
// Entree dans la fonction 

  //@ SI  Ptsi != m_PTSI
  if(Ptsi != m_PTSI)
  {
    //@   on charge le fichier de conf autre correspondant au PTSI et a la date  
    //@   courante
    m_GestConfAutre.ChargerPTSIDateValid(Ptsi,
                                         tcourante,
                                         nom_fichier,
                                         IdCA,
                                         IdCS,
                                         IdBS);
    //@ ajout du fichier conf autre a la liste des fichiers en entree
    //@ de SD_DON_Traitement
    Trt.AjouterProduitE(nom_fichier);

    //@ on verifie ici le fichier de conf stable prealablement charge  
    cr_op = m_ConfStable.Verifie(IdCS);
    //@ SI IdCS different
    if(cr_op != CMN__STATUT_OK)
    {
      //@ message JDB
      JDB__Client MessageJDB;
      MessageJDB.EcrireMessage(SD_CONF_SET, SD__Message::MSG_CONF_IDCS_DIFF);
      MessageJDB.Liberer();
      //@ traces
      #ifdef OPS_DEBUG
      cout << "SD_DON_InventaireConf::ChargerConf" << endl;
      cout << "IdCS different, nouvelle valeur dans fic conf autre : " << IdCS << endl;
      #endif
    }
    //@ FIN SI
    //@   on verifie ici la banque spectrale prealablement charge  
    cr_op = m_BanqueSpectrale.Verifie(IdBS);
    //@ SI IdBS different
    if(cr_op != CMN__STATUT_OK)
    {
      //@ message JDB
      JDB__Client MessageJDB;
      MessageJDB.EcrireMessage(SD_CONF_SET, SD__Message::MSG_CONF_IDBS_DIFF);
      MessageJDB.Liberer();
      //@ traces
      #ifdef OPS_DEBUG
      cout << "SD_DON_InventaireConf::ChargerConf" << endl;
      cout << "IdBS different, nouvelle valeur dans fic conf autre : " << IdBS << endl;
      #endif
    }
    //@ FIN SI
  }
  //@ FIN SI
  //@ m_PTSI=Ptsi
  m_PTSI = Ptsi;

// on trace ici si besoin toutes les variables en sortie
#ifdef OPS_DEBUG
cout << endl << "SD_DON_InventaireConf::ChargerConf" << endl;
cout << "Si pas bon Ptsi en parametres les 3 Id sont recharges, sinon -1" << endl;
cout << "IdCA : " << IdCA << endl;
cout << "IdCS : " << IdCS << endl;
cout << "IdBS : " << IdBS << endl;
#endif	
// sortie de la fonction 

}
//@ ------------------------------------------------------------------*/
//@ Methode : GetConf
//@
//@ Description : Cette methode permet de recuperer dans les traitements 
//a l'adresse memoire des donnees de configuration a utiliser 
//@ Ici les fichiers de conf sont deja charges et la zone memoire
//@ a ete prealablement allouee , les pointeurs sont alloue par l'appelant 
//@
//@
//@ Parameters :	
//@			(ENTREE)  Ptsi : Ptsi courante
//@			(ENTREE)  tcourante : date courante 
//@			(SORTIE)  ConfAutre : pointeur sur le 
//@					fichier de conf autre
//@			(SORTIE)  ConfStable : pointeur sur le 
//@					fichier de conf autre
//@			(SORTIE)  BanqueSpectrale : pointeur sur le 
//@					fichier BanqueSpectrale
//@
//@ Return Status :  aucun
//@
//@ ------------------------------------------------------------------*/
void SD_DON_InventaireConf::GetConf(
		const int& Ptsi,
		const t_Date & tcourante,
		t_SD_BAS_ConfAutre**  ConfAutre,
		t_SD_BAS_ConfStable** ConfStable,
		t_SD_BAS_BanqueSpectrale** BanqueSpectrale
			)

{
t_CMN__Statut cr_op=CMN__STATUT_OK;
int IdCA = -1;
// Entree dans la fonction 

//reservation pour un contexte multithread
//  m_GestConfAutre.Reserver();
   //@ SI  Ptsi != m_PTSI
  if(Ptsi != m_PTSI)
  {
    SD_DON_Traitement Trt;
    //@   on charge ici la configuration 
    this->ChargerConf(Ptsi, Trt, tcourante, IdCA);
    //@ on emmet un message au JdB pour mentionner le changement (AC)
    JDB__Client MessageJDB;
    MessageJDB.EcrireMessage(SD_CONF_SET, SD__Message::MSG_CONF_PTSI_DIFF);
    MessageJDB.Liberer();
  }
  //@ FIN SI
  //@ on recupere la donnee de conf autre courante
  *ConfAutre = m_GestConfAutre.GetData();
  //@ on recupere la donnee de conf stable courante
  *ConfStable = m_ConfStable.GetData();
  //@ on recupere la banque spectrale courante
  *BanqueSpectrale = m_BanqueSpectrale.GetData();  
// liberation de la reservation pour un contexte multithread
//  m_GestConfAutre.Liberer();

// on trace ici si besoin toutes les variables en sortie
#ifdef OPS_DEBUG
cout << endl << "SD_DON_InventaireConf::GetConf" << endl;
cout << "Si pas bon PTSI IdCA != -1" << endl;
cout << "IdCA : " << IdCA << endl;
cout << "Lecture conf stable dans init: " << m_ConfStable.GetData()->IDefStableParamDate.day << endl;
cout << "Lecture conf stable dans init: " << ConfStable[0]->IDefStableParamDate.day << endl;
cout << "Lecture conf autre  dans init: " << ConfAutre[0]->IDefParamDate.day << endl;
cout << "Lecture banque spec dans init: " << BanqueSpectrale[0]->IDefSdbDate.day << endl;
#endif	
// sortie de la fonction 
}

//======================================================================
//					METHODES PRIVEES 
//======================================================================
