static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_LigneAVHRR.cc,v 1.6 2013/03/29 16:55:26 lcaspar Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_LigneAVHRR
//@ 
//@ Comment :
//@    this module implement the SD_DON_LigneAVHRR class
//@    used to reference all L1B AVHRR line read by OPS L1 PPF
//@           
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ VERSION : 27/01/2003 : P.GASSIE
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2939(DM-2100-OPS) : 12/12/2011 : L. CASPAR
//@ Mise en place de conditions de levee du cas degarde 9 alternatives pour 
//@ correction de la NCR 12303
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2967(FA-2100-OPS) : 05/04/2012 : L. CASPAR
//@ Modification des bits a lire pour la variable Calibration_quality
//@ les bits 0 et 8 a 15 sont spares et ne doivent donc pas etre lus
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-3352 : 29/05/2017 : M. LANDIECH
//@ Ajout de la fonction charger seuil.
//@ Modification des bits a lire pour la variable Calibration_quality (seulement le bit 7)
//@ Modification de la valeur du bit 3 en fonction du seuil de la chaine
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <iostream>

#include "CFG__Configuration.h"
#include "CMN__Config.h"

#include "SD_DON_LigneAVHRR.h"

//======================================================================
// Declaration des variables globales
//======================================================================
static int SD_DON_LigneAVHRR_CHANNEL_3_THRESHOLD = -1;
static int SD_DON_LigneAVHRR_CHANNEL_4_THRESHOLD = -1;
static int SD_DON_LigneAVHRR_CHANNEL_5_THRESHOLD = -1;

//======================================================================
// Declaration des constantes
//======================================================================
const t_Chaine SD_DON_LigneAVHRR::FICHIER_CONFIGURATION_SEUILS = "NeDT_Threshold.cfg";
const t_Chaine SD_DON_LigneAVHRR::BLOC_SEUILS_NEDT             = "NeDTThreshold";
const t_Chaine SD_DON_LigneAVHRR::VAL_SEUIL_CHANNEL_3          = "AVHRR_CHANNEL_3_THRESHOLD";
const t_Chaine SD_DON_LigneAVHRR::VAL_SEUIL_CHANNEL_4          = "AVHRR_CHANNEL_4_THRESHOLD";
const t_Chaine SD_DON_LigneAVHRR::VAL_SEUIL_CHANNEL_5          = "AVHRR_CHANNEL_5_THRESHOLD";

const int SD_DON_LigneAVHRR::MAX_SEUIL_NEDT                    = 255;
const int SD_DON_LigneAVHRR::MIN_SEUIL_NEDT                    = 0;

const int SD_DON_LigneAVHRR::CHANNEL_3_THRESHOLD_DEFAULT       = 100;
const int SD_DON_LigneAVHRR::CHANNEL_4_THRESHOLD_DEFAULT       = 50;
const int SD_DON_LigneAVHRR::CHANNEL_5_THRESHOLD_DEFAULT       = 50;


//======================================================================
//           METHODES PUBLIQUES 
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Methode : SD_DON_LigneAVHRR
//@
//@ Description : Constructeur
//@
//@ Parameters :
//@     Aucun
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_LigneAVHRR::SD_DON_LigneAVHRR()
{
  //@ initialisation des attributs
  m_Charge = 0; 
  if(SD_DON_LigneAVHRR_CHANNEL_3_THRESHOLD < 0
      && SD_DON_LigneAVHRR_CHANNEL_4_THRESHOLD < 0
      && SD_DON_LigneAVHRR_CHANNEL_5_THRESHOLD < 0){
    ChargerSeuilsAVHRR();
  }
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_DON_LigneAVHRR
//@
//@ Description : Destructeur
//@
//@
//@ Parameters :
//@     Aucun
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_LigneAVHRR::~SD_DON_LigneAVHRR()
{
}

//@ ------------------------------------------------------------------*/
//@ Methode : ChargerContexte
//@
//@ Description : Charge le contexte de l'application
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_LigneAVHRR::ChargerSeuilsAVHRR()
{
  ERR__Exception     exception;
  t_Chaine           fichierConfiguration;
  t_Chaine           valeur;
  CMN__Configuration configuration;

  //@ Ouvrir le fichier de configuration de l'application
  fichierConfiguration = CFG__Configuration::GetRepertoire(CONF_DIR);

  //@ SI la variable d'environnement REP_CONF_DIR n'est pas definie
  if (fichierConfiguration.empty())
    {
      //@ Emission d'une exception SD_ENV_UNDEFINED
      exception.SetErrorCode(ERR__Exception::SD_ENV_UNDEFINED);
      exception.SetParametre(CONF_DIR);
      THROW(exception);
    }
  //@ FIN SI

  fichierConfiguration += CMN__SEP_REPERTOIRE;
  fichierConfiguration += FICHIER_CONFIGURATION_SEUILS;

  //@ Charger le fichier de configuration du serveur de donnees
  //@ SI le chargement a ete incorrect
  if (configuration.ChargerFichier(fichierConfiguration) !=
      CMN__STATUT_OK)
    {
      //@ Emission d'une exception SD_FILE_OPEN_ERROR
      exception.SetErrorCode(ERR__Exception::SD_FILE_OPEN_ERROR);
      exception.SetParametre(FICHIER_CONFIGURATION_SEUILS);
      THROW(exception);
    }
  //@ FIN SI

  //@ Recuperer le seuil NEDT de la channel 3
  if (configuration.LireValeur(BLOC_SEUILS_NEDT, VAL_SEUIL_CHANNEL_3, valeur)
      == CMN__STATUT_OK)
    {
      SD_DON_LigneAVHRR_CHANNEL_3_THRESHOLD = atoi(valeur.c_str());
    }

  //@ Vérification valeur
  if(SD_DON_LigneAVHRR_CHANNEL_3_THRESHOLD < MIN_SEUIL_NEDT
      || SD_DON_LigneAVHRR_CHANNEL_3_THRESHOLD > MAX_SEUIL_NEDT)
    {
      SD_DON_LigneAVHRR_CHANNEL_3_THRESHOLD = CHANNEL_3_THRESHOLD_DEFAULT;
    }

  //@ Recuperer le seuil NEDT de la channel 4
  if (configuration.LireValeur(BLOC_SEUILS_NEDT, VAL_SEUIL_CHANNEL_4, valeur)
      == CMN__STATUT_OK)
    {
      SD_DON_LigneAVHRR_CHANNEL_4_THRESHOLD = atoi(valeur.c_str());
    }

  //@ Vérification valeur
  if(SD_DON_LigneAVHRR_CHANNEL_4_THRESHOLD < MIN_SEUIL_NEDT
      || SD_DON_LigneAVHRR_CHANNEL_4_THRESHOLD > MAX_SEUIL_NEDT)
    {
      SD_DON_LigneAVHRR_CHANNEL_4_THRESHOLD = CHANNEL_4_THRESHOLD_DEFAULT;
    }

  //@ Recuperer le seuil NEDT de la channel 5
  if (configuration.LireValeur(BLOC_SEUILS_NEDT, VAL_SEUIL_CHANNEL_5, valeur)
      == CMN__STATUT_OK)
    {
      SD_DON_LigneAVHRR_CHANNEL_5_THRESHOLD = atoi(valeur.c_str());
    }

  //@ Vérification valeur
  if(SD_DON_LigneAVHRR_CHANNEL_5_THRESHOLD < MIN_SEUIL_NEDT
      || SD_DON_LigneAVHRR_CHANNEL_5_THRESHOLD > MAX_SEUIL_NEDT)
    {
      SD_DON_LigneAVHRR_CHANNEL_5_THRESHOLD = CHANNEL_5_THRESHOLD_DEFAULT;
    }

}


//@ ------------------------------------------------------------------*/
//@ Methode : Liberer
//@
//@ Description : Liberation memoire des donnees de l'objet
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_LigneAVHRR::Liberer()
{
  //@ SI les donnees sont chargees
  if (m_Charge == 1)
    {
      //@ On libere ici les donnees allouees
      //@ via un appel a SD_BAS_LibereMDRAVHRR()
      SD_BAS_LibereMDRAVHRR(&m_DataAVHRR);
    }
  //@ FIN SI les donnees sont chargees
}


//@ ------------------------------------------------------------------*/
//@ Methode : Init
//@
//@ Description : Initialisation de l'objet ligne AVHRR et lecture associee
//@     les cas degrades ne sont pas ici leves ils le seront
//@     dans le traitement (GetLigne)
//@     produits technologiques (cas deg)
//@
//@ Parameters :
//@     (ENTREE)  Date : date de la ligne a creer
//@     (ENTREE)  validite : validite de la ligne (DUMMY ...)
//@     (ENTREE)  validate : validite temporelle de la ligne
//@
//@ Return Status : 
//@     
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_DON_LigneAVHRR::Init(
         t_SD_BAS_Fd * Descripteur,
         t_Date Date,
         int validite,
         int validate
         )
{
t_CMN__Statut cr_op=CMN__STATUT_OK;
t_SD_BAS_DataThread* ctx;
// Entree dans la fonction 
#ifdef OPS_DEBUG
cout << "Lecture de la ligne AVHRR, data : " << (Date.Chaine()).c_str() << endl;
#endif
  //@ initialisation de la date de la ligne et de sa validite
  m_InfoGenLigne.Date = Date;
  m_InfoGenLigne.validite = validite;
  m_InfoGenLigne.validate = validate;
  //@ initialisation du compte rendu
  ctx = new t_SD_BAS_DataThread;
  InitialiserCompteRendu(&(ctx->cr));

  //@lire la ligne courante en  decodant les info detaillee
  //@    (SD_BAS_AVHRR::Lire1B)
  cr_op = SD_BAS_LireMDRAVHRR(Descripteur, &m_DataAVHRR, ctx);

  //@ SI erreur
  if(cr_op != CMN__STATUT_OK)
  {
   cout << "Erreur LireMDR" << endl;
   //@ on declare la ligne invalide (pour ensuite levee CG9)
    m_InfoGenLigne.validite = INVALIDE;
  }
  //@ FIN SI
  //@ liberation du compte rendu
  LibererCompteRendu(&(ctx->cr.msgJDB));
  delete ctx;
  m_Charge = 1;

fin:

return(cr_op);
// sortie de la fonction 
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetData
//@
//@ Description : Cette methode permet de recuperer les information detaillees
//@ d'une ligne AVHRR , le retour se fait via une  pointeur
//@ afin de faciliter l'utilisation en C , la donnees est alors decodee
//@ via un appel a SD_BAS_AVHRR
//@
//@
//@ Parameters :
//@     (ENTREE)  numlig : numero de la ligne IASI pour
//@     laquelle la donnee est demandee
//@     (ENTREE)  numsn : numero du SN de l'image IASI pour
//@     laquelle la donnee est demandee
//@     (ENTREE)  GestCasDeg : gestionnaire de cas degrade
//@			(ENTREE)  pSVM : pointeur sur les donnees lues dans le fichier SVM
//@     (SORTIE)  Ligne : ligne retournee
//@     (alloue par l'appelant)
//@
//@ Return Status : 
//@     
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
// FT-2939
void SD_DON_LigneAVHRR::GetData(
    const int  numlig,
    const int  numlsn,
    SD_DON_GestCasDeg  & GestCasDeg,
    SD_DON_SVM*  pSVM,
    t_SD_BAS_LigneAVHRR& Ligne
    )
// ..FT-2939
{
t_CMN__Statut cr_op=CMN__STATUT_OK;

// Entree dans la fonction 
  //@ on appelle la fonction interne verifie afin de valider la telemesure
// FT-2939
  this->Verifie(numlig, numlsn, GestCasDeg,pSVM); 
// ..FT-2939

  //@ return (m_DataAVHRR)
  Ligne = m_DataAVHRR;

// on trace ici si besoin toutes les variables en sortie
}

//======================================================================
//          METHODES PRIVEES 
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Methode : Verifie
//@
//@ Description : Cette methode permet d'initialiser la coherence 
//@ dans une ligne de Niveau 1 la coherence se fait au niveau:
//@   temporel et telemesure ( via les indicateurs de qualite du produit
//a   technologique)
//@
//@
//@ Parameters :
//@     (ENTREE)  numlig : numero de la ligne IASI pour
//@     laquelle la donnee est demandee
//@     (ENTREE)  numsn : numero du SN de l'image IASI pour
//@     laquelle la donnee est demandee
//@     (ENTREE)  GestCasDeg : gestionnaire de cas degrade
//@			(ENTREE)  pSVM : pointeur sur les donnees lues dans le fichier SVM
//@
//@ Return Status : 
//@     
//@     t_CMN__Statut 
//@     CMN__STATUT_OK si la fonction a reussi
//@     CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
// FT-2939
t_CMN__Statut SD_DON_LigneAVHRR::Verifie(
        const int  numlig,
        const int  numlsn,
        SD_DON_GestCasDeg & GestCasDeg,
        SD_DON_SVM*  pSVM
        )
// ..FT-2939
{
t_CMN__Statut cr_op=CMN__STATUT_OK;
// Entree dans la fonction 
  //@ SI le flag d'incoherence temporelle
  if(m_InfoGenLigne.validate == INVALIDE)
  {
      //@ on leve CG9 par SD_DON_CasDegLigne::SetDetecte(CG9,date,SN-1,0)
    (GestCasDeg.GetCasDegLigne(numlig+1))->SetDetecte(9, m_InfoGenLigne.Date, numlsn - 1, 0, 0);
  }
  //@ FIN SI
  //@ SI le flag DUMMY est leve
  if(m_InfoGenLigne.validite == INVALIDE)
  {
      //@ on leve CG9 par SD_DON_CasDegLigne::SetDetecte(CG9,date,SN-1,0)
    (GestCasDeg.GetCasDegLigne(numlig+1))->SetDetecte(9, m_InfoGenLigne.Date, numlsn - 1, 0, 0);
  }
  else
  {
  //@ sinon
  
  //@ Etat des canaux
  unsigned int valTmp;
  //@ controle du mot DIGITAL_B_DATA (13 a 7)
  SD_BAS_ExtraireValeur(*m_DataAVHRR.InfoDet.tab_var[AVHRR_DIGITAL_B_DATA].val.pui4, 7, 13, &valTmp);
  //@ SI valeur differente de 0
//ce test est inutile car il suppose que tous les canaux sont disponibles
//  if(valTmp != CHANNEL_3A && valTmp != CHANNEL_3B)
//  {
//      //@ on leve CG9
//    (GestCasDeg.GetCasDegLigne(numlig+1))->SetDetecte(9, m_InfoGenLigne.Date, numlsn - 1, 0, 0);
//  }
  //@ FIN SI
  //@ controle du mot QUALITY_INDICATOR (7 a 2)
  unsigned int valTmpQual;
  SD_BAS_ExtraireValeur(*m_DataAVHRR.InfoDet.tab_var[AVHRR_QUALITY_INDICATOR].val.pui4, 2, 7, &valTmpQual);  
  //@ SI valeur differente de 0
  if(valTmpQual != QUALITY_INDICATOR_NULL && valTmpQual != QUALITY_INDICATOR_NOM)
  {
    //@ on leve CG9
    (GestCasDeg.GetCasDegLigne(numlig+1))->SetDetecte(9, m_InfoGenLigne.Date, numlsn - 1, 0, 0);
  }
  //@ FIN SI
  //@ controle du mot CALIBRATION_QUALITY (canaux 3b, 4 et 5) (7 a 1)
  unsigned int channelCalibrated1, channelCalibrated2, channelCalibrated3;
  
  // FT 3352 seul le bit 7 est extrait
  SD_BAS_ExtraireValeur(m_DataAVHRR.InfoDet.tab_var[AVHRR_CALIBRATION_QUALITY].val.pui4[0], 7, 7, &channelCalibrated1);
  SD_BAS_ExtraireValeur(m_DataAVHRR.InfoDet.tab_var[AVHRR_CALIBRATION_QUALITY].val.pui4[1], 7, 7, &channelCalibrated2);
  SD_BAS_ExtraireValeur(m_DataAVHRR.InfoDet.tab_var[AVHRR_CALIBRATION_QUALITY].val.pui4[2], 7, 7, &channelCalibrated3);
  
  unsigned int nedt1, nedt2, nedt3;
  // Surcharge du bit 3 si les nouveaux seuils sont dépassés
  SD_BAS_ExtraireValeur(m_DataAVHRR.InfoDet.tab_var[AVHRR_CALIBRATION_QUALITY].val.pui4[0], 8, 15, &nedt1);
  SD_BAS_ExtraireValeur(m_DataAVHRR.InfoDet.tab_var[AVHRR_CALIBRATION_QUALITY].val.pui4[1], 8, 15, &nedt2);
  SD_BAS_ExtraireValeur(m_DataAVHRR.InfoDet.tab_var[AVHRR_CALIBRATION_QUALITY].val.pui4[2], 8, 15, &nedt3);

  //FT 3352 -- Mise à jour bit 3

  bool nedtExceed1, nedtExceed2, nedtExceed3;
  //@ si le le canal 3a est actif on ne controle pas le champ 3b de CALIBRATION_QUALITY
  unsigned int channel = *( m_DataAVHRR.InfoDet.tab_var[AVHRR_CHANNEL].val.pui4);

  if ( (nedt1 > (unsigned int)SD_DON_LigneAVHRR_CHANNEL_3_THRESHOLD) && (channel == SELECTED_CHANNEL_3B))
  {
    m_DataAVHRR.InfoDet.tab_var[AVHRR_CALIBRATION_QUALITY].val.pui4[0] |= 0x0008;
    nedtExceed1 = true;
  } else {
    m_DataAVHRR.InfoDet.tab_var[AVHRR_CALIBRATION_QUALITY].val.pui4[0] &= 0xFFF7;
    nedtExceed1 = false;
  }

  if (nedt2 > (unsigned int)SD_DON_LigneAVHRR_CHANNEL_4_THRESHOLD)
  {
    m_DataAVHRR.InfoDet.tab_var[AVHRR_CALIBRATION_QUALITY].val.pui4[1] |= 0x0008;
    nedtExceed2 = true;
  } else {
    m_DataAVHRR.InfoDet.tab_var[AVHRR_CALIBRATION_QUALITY].val.pui4[1] &= 0xFFF7;
    nedtExceed2 = false;
  }

  if (nedt3 > (unsigned int)SD_DON_LigneAVHRR_CHANNEL_5_THRESHOLD)
  {
    m_DataAVHRR.InfoDet.tab_var[AVHRR_CALIBRATION_QUALITY].val.pui4[2] |= 0x0008;
    nedtExceed3 = true;
  } else {
    m_DataAVHRR.InfoDet.tab_var[AVHRR_CALIBRATION_QUALITY].val.pui4[2] &= 0xFFF7;
    nedtExceed3 = false;
  }

  // -- FT 3352

/*
  //Supprimer par la FT-2939
  //@ SI valeur differente de 0
  if(((valTmp1 != 0) && channel == SELECTED_CHANNEL_3B) || (valTmp2 != 0) || (valTmp3 != 0))
  {
    //@ on leve CG9
    (GestCasDeg.GetCasDegLigne(numlig+1))->SetDetecte(9, m_InfoGenLigne.Date, numlsn - 1, 0, 0);
  }
*/

  //@ FIN SI
  //@ controle du mot NAVIGATION_STATUS (bits 15 a 12)
  unsigned int valTmpNav;
  SD_BAS_ExtraireValeur(*m_DataAVHRR.InfoDet.tab_var[AVHRR_NAVIGATION_STATUS].val.pui4, 12, 15, &valTmpNav); 
  //@ SI valeur differente de 0
  if(valTmpNav != 0)
  {
    //@ on leve CG9
    (GestCasDeg.GetCasDegLigne(numlig+1))->SetDetecte(9, m_InfoGenLigne.Date, numlsn - 1, 0, 0);
    //@ on leve CG15
    (GestCasDeg.GetCasDegLigne(numlig+1))->SetDetecte(15, m_InfoGenLigne.Date, numlsn - 1, 0, 0);
  }
  //@ FIN SI
  //@ controle du mot SCAN_LINE_QUALITY (7 a 4)
  unsigned int valTmpScan;
  SD_BAS_ExtraireValeur(*m_DataAVHRR.InfoDet.tab_var[AVHRR_SCAN_LINE_QUALITY].val.pui4, 4, 7, &valTmpScan);  
  //@ SI valeur differente de 0
  if(valTmpScan != 0)
  {
    //@ on leve CG9
    (GestCasDeg.GetCasDegLigne(numlig+1))->SetDetecte(9, m_InfoGenLigne.Date, numlsn - 1, 0, 0);
    //@ on leve CG15
    (GestCasDeg.GetCasDegLigne(numlig+1))->SetDetecte(15, m_InfoGenLigne.Date, numlsn - 1, 0, 0); 
  }
  //@ FIN SI 
  //@ controle du mot QUALITY_INDICATOR (27)
  unsigned int valTmpQual27;
  SD_BAS_ExtraireValeur(*m_DataAVHRR.InfoDet.tab_var[AVHRR_QUALITY_INDICATOR].val.pui4, 27, 27, &valTmpQual27);  
  //@ SI valeur differente de 0
  if(valTmpQual27 != 0)
  {
    //@ on leve CG9
    (GestCasDeg.GetCasDegLigne(numlig+1))->SetDetecte(9, m_InfoGenLigne.Date, numlsn - 1, 0, 0);
    //@ on leve CG15
    (GestCasDeg.GetCasDegLigne(numlig+1))->SetDetecte(15, m_InfoGenLigne.Date, numlsn - 1, 0, 0);
  }
  //@ FIN SI
  //@ controle du mot QUALITY_INDICATOR (31, 30, 28)
  unsigned int valTmpQual28;
  SD_BAS_ExtraireValeur(*m_DataAVHRR.InfoDet.tab_var[AVHRR_QUALITY_INDICATOR].val.pui4, 28, 28, &valTmpQual28);  
  unsigned int valTmpQual30;
  SD_BAS_ExtraireValeur(*m_DataAVHRR.InfoDet.tab_var[AVHRR_QUALITY_INDICATOR].val.pui4, 30, 31, &valTmpQual30);  
  //@ SI valeur differente de 0
  if(valTmpQual28 != 0 || valTmpQual30 != 0)
  { 
    //@ on leve CG9
    (GestCasDeg.GetCasDegLigne(numlig+1))->SetDetecte(9, m_InfoGenLigne.Date, numlsn - 1, 0, 0);
  }
  //@ FIN SI
  //@ extraction du mot SCAN_LINE_QUALITY (23 a 20, 15 a 11)
  unsigned int valTmpScan20;
  SD_BAS_ExtraireValeur(*m_DataAVHRR.InfoDet.tab_var[AVHRR_SCAN_LINE_QUALITY].val.pui4, 20, 23, &valTmpScan20);  
  unsigned int valTmpScan11;  
  // FT-2939
  bool dateTransitionTrouvee = false;  
  t_Date tDateTransition;
  pSVM->GetDateTransition(m_InfoGenLigne.Date, tDateTransition, &dateTransitionTrouvee);
  if ((dateTransitionTrouvee == true) && (fabs(m_InfoGenLigne.Date - tDateTransition) <= SD_BAS_DELTA_TIME_MAX_NIGHT_TRANSITION_CG9))
  {
    SD_BAS_ExtraireValeur(*m_DataAVHRR.InfoDet.tab_var[AVHRR_SCAN_LINE_QUALITY].val.pui4, 12, 15, &valTmpScan11);
  }else{
    // On est hors de la periode de transition
    // SD_BAS_ExtraireValeur(*m_DataAVHRR.InfoDet.tab_var[AVHRR_SCAN_LINE_QUALITY].val.pui4, 11, 15, &valTmpScan11);    
    SD_BAS_ExtraireValeur(*m_DataAVHRR.InfoDet.tab_var[AVHRR_SCAN_LINE_QUALITY].val.pui4, 12, 15, &valTmpScan11);    
    if (dateTransitionTrouvee != true)
    {
      std::cout << "Pas de date de transition trouvee dans le fichier SVM" << endl;
    }

    //@ SI valeur differente de 0
    if(((channelCalibrated1 != 0 || nedtExceed1) && channel == SELECTED_CHANNEL_3B) || (nedtExceed2 || channelCalibrated2 != 0) || (nedtExceed3 || channelCalibrated3 != 0))
    {
      //@ on leve CG9
      (GestCasDeg.GetCasDegLigne(numlig+1))->SetDetecte(9, m_InfoGenLigne.Date, numlsn - 1, 0, 0);
    }
  }
  // ..FT-2939

  //@ SI valeur differente de 0
  if(valTmpScan20 != 0 || valTmpScan11 != 0)
  {
    //@ on leve CG9
    (GestCasDeg.GetCasDegLigne(numlig+1))->SetDetecte(9, m_InfoGenLigne.Date, numlsn - 1, 0, 0);
  }
  //@ FIN SI
  
  //@   SCAN line quality
  //@   calibration quality
  //@   count error frame issu du MDR
  //@   intrument invalid word flag ( VIADR ???)
  //@ si un de ces flags est positionne on leve le cas degrade CG9 et ou CG15 (AC)
  }
  // fin si la ligne n'est pas dummy

// on trace ici si besoin toutes les variables en sortie
fin:
// sortie de la fonction 
return(cr_op); 

}
