//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_LigneAVHRR.h,v 1.5 2013/03/29 16:55:26 lcaspar Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_LigneAVHRR
//@ 
//@ Comment :
//@    this module implement the SD_DON_LigneAVHRR class
//@    used to reference the N1 line read/created by OPS L1 PPF
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 27/01/2003 : P.GASSIE
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2939(DM-2100-OPS) : 12/12/2011 : L. CASPAR
//@ Mise en place de conditions de levee du cas degarde 9 alternatives pour 
//@ correction de la NCR 12303
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-3352 : 29/05/2017 : M. LANDIECH
//@ Ajout de la fonction charger seuil
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_DON_LigneAVHRR_h
#define SD_DON_LigneAVHRR_h


// Inserer ici les fichiers include 
#include "SD_BAS_AVHRR.h"
#include "SD_BAS_Global.h"
#include "SD_DON_GestCasDeg.h"
#include "CMN__Date.h"
#include "SD_BAS_Conversion.h"
#include "SD_DON_SVM.h"


// Definition des exceptions

class SD_DON_LigneAVHRR {

/*====================================================================*/

public:

  //
  // Type definissant les info generales sur la ligne (AC)
  //  
  typedef struct
  {
    t_Date  Date; // date  de la ligne (corrected UTC )
    int validite; // validite de la ligne (DUMMY ou pas)
    int validate; // validite temporelle de la ligne
    // a completer avec les info necessaire pour le MDR N1
  } 
  t_SD_DON_InfoGenLigneAVHRR;
  
  //
  // Constant
  //
  //
  // Constructeur
  //
  SD_DON_LigneAVHRR();
  //
  // Destructeur
  //
  ~SD_DON_LigneAVHRR();
  void Liberer();

  //
  // Accesseurs
  //
  //
  // publics Methods 
  //
  t_CMN__Statut Init(
         t_SD_BAS_Fd * Descripteur,
         t_Date Date,
         int validite,
         int validate
         );

  // FT-2939
  void GetData(
    const int  numlig,
    const int  numlsn,
    SD_DON_GestCasDeg  & GestCasDeg,
    SD_DON_SVM*  pSVM,
    t_SD_BAS_LigneAVHRR& Ligne
  );
  // ..FT-2939

  t_SD_DON_InfoGenLigneAVHRR GetInfoGenLigne() {return m_InfoGenLigne;};
    
  /*====================================================================*/
protected:
  //
  // protected Methods 
  //
  //FT 3352
  void ChargerSeuilsAVHRR();
  // -- FT 3352
  //
  // protected Attributes 
  //

  /*====================================================================*/
private:

  //
  // Constantes
  //

  // Fichier de configuration ddes seuils AVHRR
  static const t_Chaine FICHIER_CONFIGURATION_SEUILS;
  static const t_Chaine BLOC_SEUILS_NEDT;
  static const t_Chaine VAL_SEUIL_CHANNEL_3;
  static const t_Chaine VAL_SEUIL_CHANNEL_4;
  static const t_Chaine VAL_SEUIL_CHANNEL_5;

  static const int MAX_SEUIL_NEDT;
  static const int MIN_SEUIL_NEDT;

  static const int CHANNEL_3_THRESHOLD_DEFAULT;
  static const int CHANNEL_4_THRESHOLD_DEFAULT;
  static const int CHANNEL_5_THRESHOLD_DEFAULT;

  //
  // Private Methods 
  //

  // FT-2939
  t_CMN__Statut Verifie(
        const int  numlig,
        const int  numlsn,
        SD_DON_GestCasDeg  & GestCasDeg,
        SD_DON_SVM*  pSVM
        );
  // ..FT-2939

  //
  // Private Attributes 
  //

  // information de chargement
  int m_Charge;

  // informations generales sur la ligne
  t_SD_DON_InfoGenLigneAVHRR m_InfoGenLigne;

      
  //
  // structure contenant les informations generales et detaillees
  // pour une ligne
  t_SD_BAS_LigneAVHRR m_DataAVHRR;
};


#endif
