static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_LigneN0.cc,v 1.4 2015/07/31 15:37:27 lcaspar Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_LigneN0
//@
//@ Commentaire :
//@    Ce module gere la classe SD_DON_LigneN0, qui contient les
//@    informations relatives a une ligne de donnees de niveau 0
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 06/01/2003 : P. CABANE
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-074-CN : 29/10/2003 : JL.PASCAL
//@ lors de lignes incompletes le cas degrade CG12 et CG1 est leve sur la ligne suivante 
//@ le pb est du a l'iddentification incorrecte des debut et fin de ligne
//@ un cas est ajoute dans la fonction IdentifierLignes en gerant l'historique
//@ des ISP de plus la MARGE de detection des incoherence temporelle est augmentee
//@ dans SD_DON_LigneN0.c ( + 1)
//@ fiche de correction correspondante : IA-2100-FC-074-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-080-CN : 18/11/2003 : JL.PASCAL
//@ lors du mode calibration externe en visee NADIR un cas CG3 est leve de maniere incorrecte
//@ le pb est du a l'iddentification de la visee nadir (SP =34)
//@ un test est ajoute dans la fonction VerifieTemp 
//@ fiche de correction correspondante : IA-2100-FC-080-THA
//@ --------------------------------------------------------------------
//@ DM-ID : PM4 : 31/03/2004 : B. MASSART
//@ SD_DON_LigneN0::VerifieTelemAP:
//@ -correction levee CG7 sur word 88 (B15)
//@ -correction levee CG8 sur word 103 et 118 (B15)
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-139-CN : 21/09/2004 : B.MASSART
//@ Probleme d'initialisation de GDatFlagIsrfemOff quand on entre en mode
//@ calibration externe ne visant pas la Terre. On introduit le mode
//@ de prise de vue dans la gestion des cas degrades. Voir IA-2100-FC-139-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-150-CN : 28/01/2005 : JL.PASCAL
//@ bug aleatoire : correction allocation inutile
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-270-CN : 24/06/2005 : JL.PASCAL
//@ prise en compte des SN manquant
//@ fiche de correction correspondante : IA-2100-CR-270-THA
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-193-CN : 9/11/2006 : R.MORENO
//@ Levee de CG pour les images associees aux visees da calibration CS
//@ et BB
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-3226(DM-2100-OPS) : 22/07/2015 : M. PENZES
//@ Le flag CSQ_NV ne declenche plus la levee du CG6, CG7 et CG8
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL
#include <cstring>

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN__Date.h"

#include "SD_BAS_Conversion.h"

#include "SD_DON_LigneN0.h"


//
// Definition des constantes
//
// la marge est augmentee pour tenir compte des lignes incompletes

const double SD_DON_LigneN0::SD_DON_MARGE_NORMAL      = 0.45 + 1.;
const double SD_DON_LigneN0::SD_DON_MARGE_CALIBRATION = 1.00 + 1.;
//CR270
const double SD_DON_LigneN0::SD_DON_DUREEE_SN_MIN  = 9.*SD_BAS_DUREE_ASE_SPECTN0/1000.;


//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD_DON_LigneN0
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_LigneN0::SD_DON_LigneN0()
  : m_NumLigne(0), m_NbPaquet(0), m_CasDegradeLigne(0)
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_DON_LigneN0
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_LigneN0::~SD_DON_LigneN0()
{
  //@ On libere ici les listes eventuellement remplies
  m_ListePaquets.clear();

  //@ La liberation memoire pour le cas degrade ligne
  //@ est effectuee par le gestionnaire de cas degrades
}


//@ ------------------------------------------------------------------*/
//@ Methode : Liberer
//@
//@ Description : Liberation memoire des donnees de l'objet
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_LigneN0::Liberer()
{
  int indiceFin = m_ListePaquets.size() - 1;

  //@ POUR tous les paquets de la ligne
  for (int i=indiceFin; i>-1; i--)
    {
      //@ Liberation memoire des donnees
      m_ListePaquets[i]->Liberer();

      //@ Suppression du paquet
      delete m_ListePaquets[i];
    }
  //@ FIN POUR tous les paquets de la ligne
  
  //@ RAZ de la liste
  m_ListePaquets.clear();
}


//@ ------------------------------------------------------------------*/
//@ Methode : operator =
//@
//@ Description : Operateur =
//@
//@ Parametres :
//@               (ENTREE) ligne : Ligne origine
//@
//@ Valeur de retour : 
//@               SD_DON_LigneN0 : Objet destination
//@
//@ ------------------------------------------------------------------*/
SD_DON_LigneN0& SD_DON_LigneN0::operator =(const SD_DON_LigneN0& ligne)
{
  if (&ligne == this)
    {
      return *this;
    }

  m_NumLigne               = ligne.m_NumLigne;
  m_NbPaquet               = ligne.m_NbPaquet;
  m_InfoGenLigne           = ligne.m_InfoGenLigne;
  m_ListePaquets           = ligne.m_ListePaquets;

  /* FA139 */
  m_CasDegradeLigne        = new SD_DON_CasDegLigne(m_NumLigne+1, m_InfoGenLigne.GetModePriseDeVue());
  memcpy(m_CasDegradeLigne, 
         ligne.m_CasDegradeLigne, 
         sizeof(SD_DON_CasDegLigne));

  return *this;
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetNbPaquetN0
//@
//@ Description : Recupere le nombre de paquets N0 pour un type donne
//@
//@ Parametres :
//@               (ENTREE) type : Type des paquets choisis
//@
//@ Valeur de retour : 
//@               Nombre de paquets retournes
//@
//@ ------------------------------------------------------------------*/
int SD_DON_LigneN0::GetNbPaquetN0(const e_SD_BAS_TypeN0 type)
{
  // Iterateur pour la liste des paquets
  t_ListePaquetN0::iterator pInfo;

  //@ Initialisation du nombre de paquets
  int nbPaquetN0 = 0;


  //@ POUR tous les elements de la liste m_ListePaquets
  for (pInfo = m_ListePaquets.begin();
       pInfo != m_ListePaquets.end();
       pInfo++)
    {
      //@ SI le paquet est du type recherche
      if ((*pInfo)->GetData()->type == type)
        {
          //@ On ne selectionne pas les donnees correspondant au cas 
          //@ degrades CG13 et CG14 (incoherence ou absence)
          //@ SI le paquet est valide
          if ((*pInfo)->EstValide())
            {
              //@ On met a jour le nombre de paquets retournes
              nbPaquetN0++;
            }
          //@ FIN SI le paquet est valide
        }
      //@ FIN SI le paquet est du type recherche
    }
  //@ FIN POUR tous les elements de la liste m_ListePaquets


  //@ Renvoyer le nombre de paquets N0
  return (nbPaquetN0);
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetDataGen
//@
//@ Description : Recupere les informations globales a une ligne de
//@               niveau 0
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Informations globales
//@
//@ ------------------------------------------------------------------*/
const t_InfoGenLigne& SD_DON_LigneN0::GetDataGen() const
{
  //@ On retourne l'attribut m_InfoGenLigne
  return (m_InfoGenLigne); 
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetPaquetLigne
//@
//@ Description : Recupere les informations detaillees d'une ligne de
//@               niveau 0. Le retour se fait via une liste de 
//@               pointeurs afin de faciliter l'utilisation en C. On
//@               ne retourne ici que les paquets coherents (cf CG 13
//@               14)
//@
//@ Parametres :
//@               (ENTREE) type : Type des paquets choisis
//@               (SORTIE) nbPaquets : Nombre de paquets retournes
//@               (SORTIE) listePaquetLigne : Liste des paquets 
//@                                           retournes
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_LigneN0::GetPaquetLigne(const e_SD_BAS_TypeN0 type,
                                    int& nbPaquets,
                                    t_SD_BAS_PaquetN0** 
                                    listePaquetLigne)
{
  // Iterateur pour la liste des paquets
  t_ListePaquetN0::iterator pInfo;


  //@ Initialisation du nombre de paquets
  nbPaquets = 0;

  //@ On recupere dans la liste listePaquetLigne les paquets de
  //@ la ligne de niveau 0, de type = type, 
  //@ tries par scan position (issu du produit)
  
  //@ POUR tous les elements de la liste m_ListePaquets
  for (pInfo = m_ListePaquets.begin();
       pInfo != m_ListePaquets.end();
       pInfo++)
    {
      //@ SI le paquet est du type recherche
      if ((*pInfo)->GetData()->type == type)
        {
          //@ On ne selectionne pas les donnees correspondant au cas 
          //@ degrades CG13 et CG14 (incoherence ou absence)
          //@ SI le paquet est valide
          if ((*pInfo)->EstValide())
            {
              //@ Ajout du paquet dans la liste listePaquetLigne
              //@ On met a jour le nombre de paquets retournes
              listePaquetLigne[nbPaquets++] = (*pInfo)->GetData(); /* FA150 */
//              t_SD_BAS_PaquetN0* paquet = new t_SD_BAS_PaquetN0;
//              paquet = (*pInfo)->GetData();
//              listePaquetLigne[nbPaquets++] = paquet;
              //@ Ajout du paquet dans la liste listePaquetLigne
              //@ On met a jour le nombre de paquets retournes
            }
          //@ FIN SI le paquet est valide
        }
      //@ FIN SI le paquet est du type recherche
    }
  //@ FIN POUR tous les elements de la liste m_ListePaquets
}


//@ ------------------------------------------------------------------*/
//@ Methode : Init
//@
//@ Description : Initialisation de l'objet ligne N0
//@
//@ Parametres :
//@               (ENTREE) numLigne : Numero de la ligne a creer
//@               (ENTREE) infoGenLigne : Infos generales de la ligne
//@               (ENTREE/SORTIE) gestionCasDegrade : Gestionnaire de
//@                                                   cas degrade
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_LigneN0::Init(const int numLigne,
                          const t_InfoGenLigne& infoGenLigne,
                          SD_DON_GestCasDeg& gestionCasDegrade)
{
  //@ Initialisation du numero de ligne
  m_NumLigne     = numLigne;

  // Initialisation des infos generales de la ligne
  m_InfoGenLigne = infoGenLigne;

  //@ On cree un gestionnaire de cas degrade ligne
  //@ et on le reference au niveau de la ligne
  //@ Celui ci est initialise avec tous les cas degrades associes
  //@ (initialisation a OK, de plus les indicateurs de presence sont
  //@ initialises a faux)
  /* FA139 */
  gestionCasDegrade.AjouteCasDegLigne(m_NumLigne+1, m_InfoGenLigne.GetModePriseDeVue());
  
  m_CasDegradeLigne = gestionCasDegrade.GetCasDegLigne(m_NumLigne+1);
}


//@ ------------------------------------------------------------------*/
//@ Methode : AddPaquetLigne
//@
//@ Description : Ajoute un ISP dans la ligne de niveau 0. Au prealable 
//@               l'ISP a ete cree via un appel a SD_DON_PaquetN0
//@
//@ Parametres :
//@               (ENTREE) paquetN0 : Paquet de niveau 0 a ajouter a
//@                                   la ligne 
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_LigneN0::AddPaquetLigne(SD_DON_PaquetN0* paquetN0)
{
  //@ SI le paquet est valide
  if (paquetN0->EstValide() == CMN__VRAI)
    {
      //@ On ajoute le paquet a la liste des paquets
      m_ListePaquets.push_back(paquetN0);
      
      //@ On appelle la methode de verification temporelle
      //@ SI la verification temporelle a reussi
      if (VerifieTemp(*paquetN0) == CMN__STATUT_OK)
        {
          //@ On appelle la methode de verification telemesure
	  VerifieTelem(*paquetN0);
             //@ SI le type du paquet est AP
              if (paquetN0->GetData()->type == AP)
                {
                  //@ On initialise BBT
                  m_InfoGenLigne.SetBBT(paquetN0->GetData()->InfoDet.
                                        tab_var[AP_BBT].val.pui4[0]);
                }
              //@ FIN SI le type du paquet est AP
        }
      //@ SI la verification temporelle a reussi
    }
  //@ FIN SI le paquet est valide
}


//@ ------------------------------------------------------------------*/
//@ Methode : ValideLigne
//@
//@ Description : Valide la creation d'une ligne et met a jour tous
//@               les cas degrades de presence des ISP a la fin de la
//@               ligne. Au prealable, les ISP ont ete ajoutes via un
//@               appel a AddPaquetLigne
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_LigneN0::ValideLigne()
{
  // Ecart entre les dates de debut et de fin
  double ecartDate = 0.;
//CR270
  double margeInf  = SD_DON_DUREEE_SN_MIN;
  double margeSup  = SD_BAS_DUREE_LGN;


  //@ SI mode vaut EXTERNAL_CALIBRATION
  if (m_InfoGenLigne.GetModePriseDeVue() == SD_BAS_EXTERNAL_CALIBRATION)
    {
//CR270
//        margeInf -= SD_DON_LigneN0::SD_DON_MARGE_CALIBRATION;  
      margeSup += SD_DON_LigneN0::SD_DON_MARGE_CALIBRATION;
    }
  //@ SINON
  else
    {
//CR270
 //       margeInf -= SD_DON_LigneN0::SD_DON_MARGE_NORMAL;    
      margeSup += SD_DON_LigneN0::SD_DON_MARGE_NORMAL;
    }
  //@ FIN SI mode vaut EXTERNAL_CALIBRATION


  //@ On verifie ici si l'ecart entre les dates de debut et de fin de
  //@ ligne est:
  //@   - superieur a la valeur nominale (SD_BAS_DUREE_LGN) + marge
  //@   - inferieur a la valeur nominale (SD_BAS_DUREE_LGN) - marge 
  ecartDate = m_InfoGenLigne.GetDateFin().ConvertirDateEnTemps() - 
    m_InfoGenLigne.GetDateDebut().ConvertirDateEnTemps();

  //@ SI l'ecart precedent est anormal
  if ( (ecartDate < margeInf) || (ecartDate > margeSup) )
    {
      //@ POUR tous les SN de la ligne
      for (int iSN = 0; iSN < SD_BAS_SNOT; iSN++)
        {
#ifdef OPS_DEBUG
          cout << "LigneN0 no" << m_NumLigne << ": levee de CG2" 
               << endl;  
#endif

          //@ On leve le cas degrade CG2
          //@ Appel m_CasDegradeLigne.SetDetecte(2, date, SN-1)
          m_CasDegradeLigne->SetDetecte(2, 
                                        m_InfoGenLigne.GetDateDebut(),
                                        iSN,
                                        0,
                                        0);
        }
      //@ FIN POUR tous les SN de la ligne

      //@ La validite de la ligne est alors mise a FAUX
      m_InfoGenLigne.SetValidite(INCOHERENT);
   }
  //@ SINON
  else
    {
      //@ On valide la presence des ISP par appel a la methode
      //@ m_CasDegLigne.Valide(mode) qui selon le cas leve les
      //@ cas CG11 CG12 CG13 CG14 puis gere de maniere interne
      //@ la levee des flags induits
      //@ ex CG13 -> CG1 ...
      //@ et qui gere les cas degrades permanents
      m_CasDegradeLigne->Valide(m_InfoGenLigne.GetModePriseDeVue(), 
                                m_InfoGenLigne.GetDateDebut());
    }
  //@ FIN SI l'ecart precedent est anormal
}



//======================================================================
//                                      METHODES PRIVEES 
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : VerifieTemp
//@
//@ Description : Verifie la coherence d'un ISP dans une ligne de
//@               niveau 0. La coherence se fait au niveau temporel
//@
//@ Parametres :
//@               (ENTREE) paquetN0 : Paquet de niveau 0 a verifier
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK si la verifiaction temporelle a reussi
//@               CMN__STATUT_NOK sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_DON_LigneN0::VerifieTemp(SD_DON_PaquetN0& paquetN0)
{
  // Code retour de la methode
  t_CMN__Statut            statut         = CMN__STATUT_OK;

  //@ On recupere le contenu de l'ISP via la methode 
  //@ SD_DON_PaquetN0::GetData()
  const t_SD_BAS_PaquetN0* donneesPaquet  = paquetN0.GetData();


  //@ Recuperation du numero de SN
  unsigned int noSN = donneesPaquet->InfoGen.tab_var[N0_SN].val.pui4[0];

  //@ Recuperation du numero de PN
  unsigned int noPN = donneesPaquet->InfoGen.tab_var[N0_PN].val.pui4[0];

  //@ Recuperation du numero de SP
  unsigned int noSP = donneesPaquet->InfoGen.tab_var[N0_SP].val.pui4[0];


  //@ Recuperation de la validite du paquet
  e_BAS_ValidRec validite = (e_BAS_ValidRec)
    donneesPaquet->InfoGen.tab_var[N0_VALIDREC].val.pui4[0];

  //@ Recuperation du mode de prise de vue
  e_BAS_ModePdv mode = (e_BAS_ModePdv) 
    donneesPaquet->InfoGen.tab_var[N0_MODE].val.pui4[0];


  //@ On positionne l'indicateur de presence a VRAI par appel a la
  //@ methode m_CasDegradeLigne.SetPresence(type, SN-1, PN-1) qui 
  //@ le positionne a VRAI selon une logique similaire a ci-dessus
  if (donneesPaquet->type == PX)
    {
      m_CasDegradeLigne->SetPresence(donneesPaquet->type, 
                                     noSN-1, 
                                     noPN-1);
    }
  else if (donneesPaquet->type == IP)
    {
      m_CasDegradeLigne->SetPresence(donneesPaquet->type, 
                                     noSN-1, 
                                     0);
    }
  else if (donneesPaquet->type == VP)
    {
      m_CasDegradeLigne->SetPresence(donneesPaquet->type, 
                                     noPN-1, 
                                     0);
    }
  else
    {
      m_CasDegradeLigne->SetPresence(donneesPaquet->type, 
                                     0, 
                                     0);
    }

  //@ On verifie tout d'abord s'il existe une incoherence temporelle
  //@ detectee par la couche SD_BAS

  //@ SI une incoherence temporelle a ete levee
  if ( (validite == INVALIDE) || (validite == INCOHERENT) )
    {
      //@ SELON paquetN0.type
      switch (donneesPaquet->type)
        {
          //@ SI PX: levee CG 13
        case PX:

#ifdef OPS_DEBUG
          cout << "LigneN0 no" << m_NumLigne << ": levee de CG13" 
               << endl;  
#endif

          //@ Appel m_CasDegradeLigne.SetDetecte(13, date, SN-1, PN-1)
          m_CasDegradeLigne->SetDetecte(13, 
                                        m_InfoGenLigne.GetDateDebut(), 
                                        noSN-1, 
                                        noPN-1, 
                                        0);
          break;
          
          //@ SI IP: levee CG 14
        case IP:

#ifdef OPS_DEBUG
          cout << "LigneN0 no" << m_NumLigne << ": levee de CG14" 
               << endl;  
#endif

          //@ Appel m_CasDegradeLigne.SetDetecte(14, date, SN-1)
          m_CasDegradeLigne->SetDetecte(14,
                                        m_InfoGenLigne.GetDateDebut(),
                                        noSN-1, 
                                        0, 
                                        0);
          break;

          //@ SI AP: levee CG 12
        case AP:

#ifdef OPS_DEBUG
          cout << "LigneN0 no" << m_NumLigne << ": levee de CG12" 
               << endl;  
#endif

          //@ Appel m_CasDegradeLigne.SetDetecte(12, date)
          m_CasDegradeLigne->SetDetecte(12, 
                                        m_InfoGenLigne.GetDateDebut(),
                                        0, 
                                        0, 
                                        0);
          break;

          //@ SI VP: levee CG 11
        case VP:

#ifdef OPS_DEBUG
          cout << "LigneN0 no" << m_NumLigne << ": levee de CG11" 
               << endl;  
#endif

          //@ Appel m_CasDegradeLigne.SetDetecte(11, date)
          m_CasDegradeLigne->SetDetecte(11,
                                        m_InfoGenLigne.GetDateDebut(),
                                        noPN-1, 
                                        0, 
                                        0);
          break;

          //@ Defaut:
        default:
          break;
        }
      //@ FIN SELON

      //@ Mettre a jour le code retour
      statut = CMN__STATUT_NOK;
  }
  //@ FIN SI une incoherence temporelle a ete levee

  
  //@ On verifie ensuite si on est en mode calibration externe 
  //@ pour les paquets PX dans la mesure ou on ne vise pas la terre
  //@ (SP n'appartient pas a [1,30])

  //@ SI on est en mode calibration externe et 
  //@ SI le paquet est de type PX et 
  //@ SI on ne vise pas la terre
  if ( (mode == SD_BAS_EXTERNAL_CALIBRATION) &&
       (donneesPaquet->type == PX) &&
        ((noSP < SD_BAS_SP_EARTH_MIN) || (noSP > SD_BAS_SP_EARTH_MAX && noSP != SD_BAS_SP_EARTH_NADIR)) )
    {

#ifdef OPS_DEBUG
      cout << "LigneN0 no" << m_NumLigne+1 
           << ": levee de CG3" << endl;  
#endif

      //@ POUR tous les CCD
      for (int i=0; i<SD_BAS_CCD; i++)
        {
          //@ On leve ici le cas CG3
          //@ Appel m_CasDegradeLigne.SetDetecte(3, date, SN-1, xx, i)
          m_CasDegradeLigne->SetDetecte(3, 
                                        m_InfoGenLigne.GetDateDebut(),
                                        noSN-1, 
                                        0, 
                                        i);
        }
      //@ FIN POUR tous les CCD

      //@ Mettre a jour le code retour
      statut = CMN__STATUT_NOK;
    }
  //@ FIN SI on est en mode calibration externe


  //@ Renvoyer le code retour de la methode
  return statut;
}


//@ ------------------------------------------------------------------*/
//@ Methode : VerifieTelem
//@
//@ Description : Verifie la telemesure d'un paquet dans une ligne de 
//@               niveau 0
//@
//@ Parametres :
//@               (ENTREE) paquetN0 : Paquet de niveau 0 a verifier
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK si la verification temporelle a reussi
//@               CMN__STATUT_NOK sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_DON_LigneN0::VerifieTelem(SD_DON_PaquetN0& paquetN0)
{
  // Code retour de la methode
  t_CMN__Statut            statut         = CMN__STATUT_OK;

  // Pointeur de parcours des mots
  unsigned short*          pWord          = NULL;

  //@ On recupere le contenu de l'ISP via la methode 
  //@ SD_DON_PaquetN0::GetData()
  const t_SD_BAS_PaquetN0* donneesPaquet  = paquetN0.GetData();


  //@ Recuperation du numero de SN
  unsigned int noSN = donneesPaquet->InfoGen.tab_var[N0_SN].val.pui4[0];

  //@ Recuperation du numero de PN
  unsigned int noPN = donneesPaquet->InfoGen.tab_var[N0_PN].val.pui4[0];

  pWord = (unsigned short*) donneesPaquet->InfoCod.data;

  
  //@ SELON PaquetN0.type
  switch (donneesPaquet->type)
    {  
      //@ SI PX: 
    case PX:
      //@ Appel de la methode VerifieTelemPX()
      statut = VerifieTelemPX(pWord, noSN, noPN);
      break;

      //@ SI IP:
    case IP:
      //@ Appel de la methode VerifieTelemIP()
      statut = VerifieTelemIP(pWord, noSN);
      break;
  
      //@ SI AP:
    case AP:
      //@ Appel de la methode VerifieTelemAP()
      statut = VerifieTelemAP(pWord);   
      break;
      
      //@ SI VP:
    case VP:
      //@ a priori, rien a verifier
      break;

      //@ Defaut:
    default:
      break;
    }
  //@ FIN SELON


  //@ Renvoyer le code retour de la methode
  return statut;
}


//@ ------------------------------------------------------------------*/
//@ Methode : VerifieTelemPX
//@
//@ Description : Verifie la telemesure d'un paquet PX dans une ligne
//@               de niveau 0
//@
//@ Parametres :
//@               (ENTREE) pWord : Pointeur sur les donnees
//@               (ENTREE) noSN : Numero SN
//@               (ENTREE) noPN : Numero PN
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK si la verification temporelle a reussi
//@               CMN__STATUT_NOK sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_DON_LigneN0::VerifieTelemPX(unsigned short* pWord,
                                             const unsigned int noSN,
                                             const unsigned int noPN)
{
  int           valeur = 0;

  // Code retour de la methode
  t_CMN__Statut statut = CMN__STATUT_OK;


  //@ word 30-31 (= -1)
  //@ word 150 (B0, B1)
  //@ word 151 (B3, B10)
  valeur = (int) (*(pWord+29));
      
  //@ SI un des bits est positionne (SD_BAS_Is_BitSet)
  /* Prise en compte de CR216 : CMN__ERREUR */  
  if ( (valeur == -1) ||
       (SD_BAS_Is_BitSet(*(pWord+IND1(150)), 0) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(150)), 1) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(151)), 3) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(151)), 10) == CMN__ERREUR) )
    {
      
#ifdef OPS_DEBUG
      cout << "LigneN0 no" << m_NumLigne << ": levee de CG1" << endl;  
#endif

      //@ On leve le cas CG1
      //@ Appel m_CasDegradeLigne.SetDetecte(1, date, SN-1, PN-1)
      m_CasDegradeLigne->SetDetecte(
        1, 
        m_InfoGenLigne.GetDateDebut(), 
        noSN-1, 
        noPN-1, 
        0);
      
      //@ Mise a jour du code retour
      statut = CMN__STATUT_NOK;
    }
  //@ FIN SI un des bits est positionne (SD_BAS_Is_BitSet)
  
  //@ word 13 (B8 a B15)
  //@ SI un des bits est positionne (SD_BAS_Is_BitSet)
  /* Prise en compte de CR216 : CMN__ERREUR */  
  if ( (SD_BAS_Is_BitSet(*(pWord+IND1(13)), 8) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(13)), 9) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(13)), 10) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(13)), 11) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(13)), 12) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(13)), 13) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(13)), 14) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(13)), 15) == CMN__ERREUR) )
    {
      
#ifdef OPS_DEBUG
      cout << "LigneN0 no" << m_NumLigne << ": levee de CG3" << endl;  
#endif
      
      //@ POUR tous les CCD
      for (int i=0; i<SD_BAS_CCD; i++)
        {
          //@ On leve le cas CG3
          //@ Appel m_CasDegradeLigne.SetDetecte(3, date, 
          //@                                 SN-1, xx, i)
          m_CasDegradeLigne->SetDetecte(
            3, 
            m_InfoGenLigne.GetDateDebut(), 
            noSN-1, 
            0, 
            i);
        }
      //@ FIN POUR tous les CCD
      
      //@ Mise a jour du code retour
      statut = CMN__STATUT_NOK;
    }
  //@ FIN SI un des bits est positionne (SD_BAS_Is_BitSet)
  
  //@ word 151 (B0)
  //@ word 151 (B4)
  //@ word 151 (B7)
  //@ word 151 (B11)
  //@ SI un des bits est positionne (SD_BAS_Is_BitSet)
  /* Prise en compte de CR216 : CMN__ERREUR */  
  if ( (SD_BAS_Is_BitSet(*(pWord+IND1(151)), 0) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(151)), 4) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(151)), 7) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(151)), 11) == CMN__ERREUR) )
    {
      
#ifdef OPS_DEBUG
      cout << "LigneN0 no" << m_NumLigne << ": levee de CG4" << endl;  
#endif
   
      //@ On leve le cas CG4
      //@ Appel m_CasDegradeLigne.SetDetecte(4, date, SN-1,
      //@                                    PN-1, 0)
      m_CasDegradeLigne->SetDetecte(
        4, 
        m_InfoGenLigne.GetDateDebut(), 
        noSN-1, 
        noPN-1, 
        0);
      
      //@ Mise a jour du code retour
      statut = CMN__STATUT_NOK;
    }
  //@ FIN SI un des bits est positionne (SD_BAS_Is_BitSet)
  
  //@ word 151 (B1)
  //@ word 151 (B5)
  //@ word 151 (B8)
  //@ word 151 (B12)
  //@ SI un des bits est positionne (SD_BAS_Is_BitSet)
  /* Prise en compte de CR216 : CMN__ERREUR */  
  if ( (SD_BAS_Is_BitSet(*(pWord+IND1(151)), 1) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(151)), 5) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(151)), 8) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(151)), 12) == CMN__ERREUR) )
    {
      
#ifdef OPS_DEBUG
      cout << "LigneN0 no" << m_NumLigne << ": levee de CG4" << endl;  
#endif

      //@ On leve le cas CG4
      //@ Appel m_CasDegradeLigne.SetDetecte(4, date, SN-1,
      //@                                    PN-1, 1)
      m_CasDegradeLigne->SetDetecte(
        4, 
        m_InfoGenLigne.GetDateDebut(), 
        noSN-1, 
        noPN-1, 
        1);
      
      //@ Mise a jour du code retour
      statut = CMN__STATUT_NOK;
    }
  //@ FIN SI un des bits est positionne (SD_BAS_Is_BitSet)
  
  //@ word 151 (B2)
  //@ word 151 (B6)
  //@ word 151 (B9)
  //@ word 151 (B13)
  //@ SI un des bits est positionne (SD_BAS_Is_BitSet)
  /* Prise en compte de CR216 : CMN__ERREUR */  
  if ( (SD_BAS_Is_BitSet(*(pWord+IND1(151)), 2) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(151)), 6) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(151)), 9) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(151)), 13) == CMN__ERREUR) )
    {
      
#ifdef OPS_DEBUG
      cout << "LigneN0 no" << m_NumLigne << ": levee de CG4" << endl;  
#endif
      
      //@ On leve le cas CG4
      //@ Appel m_CasDegradeLigne.SetDetecte(4, date, SN-1,
      //@                                    PN-1, 2)
      m_CasDegradeLigne->SetDetecte(
        4, 
        m_InfoGenLigne.GetDateDebut(), 
        noSN-1, 
        noPN-1, 
        2);
      
      //@ Mise a jour du code retour
      statut = CMN__STATUT_NOK;
    }
  //@ FIN SI un des bits est positionne (SD_BAS_Is_BitSet)
  

  //@ Renvoyer le code retour de la methode
  return statut;
}


//@ ------------------------------------------------------------------*/
//@ Methode : VerifieTelemIP
//@
//@ Description : Verifie la telemesure d'un paquet IP dans une ligne
//@               de niveau 0
//@
//@ Parametres :
//@               (ENTREE) pWord : Pointeur sur les donnees
//@               (ENTREE) noSN : Numero SN
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK si la verification temporelle a reussi
//@               CMN__STATUT_NOK sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_DON_LigneN0::VerifieTelemIP(unsigned short* pWord,
                                             unsigned int noSN)
{
  // Code retour de la methode
  t_CMN__Statut statut = CMN__STATUT_OK;


  //@ word 13 (B8 a B15) et word 21 (B9)
  //@ SI un des bits est positionne (SD_BAS_Is_BitSet)
  /* Prise en compte de CR216 : CMN__ERREUR */  
  if ( (SD_BAS_Is_BitSet(*(pWord+IND1(13)), 8) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(13)), 9) == CMN__ERREUR) || 
       // FT 3226 : Le flag CSQ_NV ne doit plus levé le CG6, CG7 et CG8
       //(SD_BAS_Is_BitSet(*(pWord+IND1(13)), 10) == CMN__ERREUR) || 
       // ... FT 3226
       (SD_BAS_Is_BitSet(*(pWord+IND1(13)), 11) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(13)), 12) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(13)), 13) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(13)), 14) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(13)), 15) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(21)), 9) == CMN__ERREUR) )
    {
      //@ SI l'image est une visee Terre ou calibration externe ALORS
      if ( (noSN >= 1) && (noSN <= SD_BAS_SNOT) )
      {
#ifdef OPS_DEBUG
      cout << "LigneN0 no" << m_NumLigne << ": levee de CG6" << endl;  
#endif
      
        //@ On leve le cas CG6
        //@ Appel m_CasDegradeLigne.SetDetecte(6, date, SN-1)
        m_CasDegradeLigne->SetDetecte(
          6, 
          m_InfoGenLigne.GetDateDebut(), 
          noSN-1, 
          0, 
          0);
      }
      
      //@ SI l'image est une calibration du corps noir chaud ALORS 
      if ( (noSN == SD_BAS_SN_IP_SUP_1) || (noSN == SD_BAS_SN_IP_SUP_2) )
      {
#ifdef OPS_DEBUG
      cout << "LigneN0 no" << m_NumLigne << ": levee de CG7" << endl;  
#endif
      
        //@ On leve le cas CG7
        //@ Appel m_CasDegradeLigne.SetDetecte(7, date, SN-1)
// FA 193
      //Le flag CG7 n'est qu'un bool, pas un tab de bool
        m_CasDegradeLigne->SetDetecte(7, 
          m_InfoGenLigne.GetDateDebut(), 
          0, 
          0, 
          0);

//.. FA 193

      }
      
      //@ SI l'image est une calibration du corps noir froid ALORS
      if ( (noSN == SD_BAS_SN_IP_SUP_3) || (noSN == SD_BAS_SN_IP_SUP_4) )
      {
#ifdef OPS_DEBUG
      cout << "LigneN0 no" << m_NumLigne << ": levee de CG8" << endl;  
#endif
      
        //@ On leve le cas CG8
        //@ Appel m_CasDegradeLigne.SetDetecte(8, date, SN-1)
// FA 193
      //Le flag CG8 n'est qu'un bool, pas un tab de bool
        m_CasDegradeLigne->SetDetecte(8, 
          m_InfoGenLigne.GetDateDebut(), 
          0, 
          0, 
          0);

// ..FA 193

      }
      //@ Mise a jour du code retour
      statut = CMN__STATUT_NOK;
    }
  //@ FIN SI un des bits est positionne (SD_BAS_Is_BitSet)


  //@ Renvoyer le code retour de la methode
  return statut;
}


//@ ------------------------------------------------------------------*/
//@ Methode : VerifieTelemAP
//@
//@ Description : Verifie la telemesure d'un paquet AP dans une ligne
//@               de niveau 0
//@
//@ Parametres :
//@               (ENTREE) pWord : Pointeur sur les donnees
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK si la verification temporelle a reussi
//@               CMN__STATUT_NOK sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_DON_LigneN0::VerifieTelemAP(unsigned short* pWord)
{
  // Code retour de la methode
  t_CMN__Statut statut = CMN__STATUT_OK;


  //@ word 28 (B0)
  //@ SI un des bits est positionne (SD_BAS_Is_BitSet)
  /* Prise en compte de CR216 : CMN__ERREUR */  
  if (SD_BAS_Is_BitSet(*(pWord+IND1(28)), 0) == CMN__ERREUR)
    {
      
#ifdef OPS_DEBUG
      cout << "LigneN0 no" << m_NumLigne << ": levee de CG5" << endl;  
#endif
      
      //@ On leve le cas CG5
      //@ Appel m_CasDegradeLigne.SetDetecte(5, date)
      m_CasDegradeLigne->SetDetecte(
        5, 
        m_InfoGenLigne.GetDateDebut(), 
        0, 
        0, 
        0);
      
      //@ Mise a jour du code retour
      statut = CMN__STATUT_NOK;
    }
  //@ FIN SI un des bits est positionne (SD_BAS_Is_BitSet)
  
  //@ word 73 (B15)
  //@ word 88 (B15)
  //@ SI un des bits est positionne (SD_BAS_Is_BitSet)
  /* Prise en compte de CR216 : CMN__ERREUR */  
  if ( (SD_BAS_Is_BitSet(*(pWord+IND1(73)), 15) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(88)), 15) == CMN__ERREUR) )
    {
      
#ifdef OPS_DEBUG
      cout << "LigneN0 no" << m_NumLigne << ": levee de CG7" << endl;  
#endif
      
      //@ On leve le cas CG7
      //@ Appel m_CasDegradeLigne.SetDetecte(7, date)
      if (SD_BAS_Is_BitSet(*(pWord+IND1(73)), 15) == CMN__ERREUR)
      {
        m_CasDegradeLigne->SetDetecte(
          7, 
          m_InfoGenLigne.GetDateDebut(), 
          IND1(SD_BAS_SN_IP_SUP_1), 
          0, 
          0);
      }
      
      if (SD_BAS_Is_BitSet(*(pWord+IND1(88)), 15) == CMN__ERREUR)
      {
        m_CasDegradeLigne->SetDetecte(
          7, 
          m_InfoGenLigne.GetDateDebut(), 
          IND1(SD_BAS_SN_IP_SUP_2), 
          0, 
          0);
      }
      
      //@ Mise a jour du code retour
      statut = CMN__STATUT_NOK;
    }
  //@ FIN SI un des bits est positionne (SD_BAS_Is_BitSet)
  
  //@ word 103 (B15)
  //@ word 118 (B15)
  //@ SI un des bits est positionne (SD_BAS_Is_BitSet)
  /* Prise en compte de CR216 : CMN__ERREUR */  
  if ( (SD_BAS_Is_BitSet(*(pWord+IND1(103)), 15) == CMN__ERREUR) || 
       (SD_BAS_Is_BitSet(*(pWord+IND1(118)), 15) == CMN__ERREUR) )
    {
      
#ifdef OPS_DEBUG
      cout << "LigneN0 no" << m_NumLigne << ": levee de CG8" << endl;  
#endif
      
      //@ On leve le cas CG8
      //@ Appel m_CasDegradeLigne.SetDetecte(8, date)
      if (SD_BAS_Is_BitSet(*(pWord+IND1(103)), 15) == CMN__ERREUR)
      {
        m_CasDegradeLigne->SetDetecte(
          8, 
          m_InfoGenLigne.GetDateDebut(), 
          IND1(SD_BAS_SN_IP_SUP_3), 
          0, 
          0);
      }
      
      if (SD_BAS_Is_BitSet(*(pWord+IND1(118)), 15) == CMN__ERREUR)
      {
        m_CasDegradeLigne->SetDetecte(
          8, 
          m_InfoGenLigne.GetDateDebut(), 
          IND1(SD_BAS_SN_IP_SUP_4), 
          0, 
          0);
      }
      
      //@ Mise a jour du code retour
      statut = CMN__STATUT_NOK;
    }
  //@ FIN SI un des bits est positionne (SD_BAS_Is_BitSet)
  
  
  //@ Renvoyer le code retour de la methode
  return statut;
}
