// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_LigneN1.h,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_LigneN1
//@
//@ Commentaire :
//@    Fichier d'entete de la classe SD_DON_LigneN1
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 06/01/2003 : P. CABANE
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 04/09/2003 : [reference :] Benjamin MASSART
//@ cette version prend en compte les evolutions du code pour une gestion
//@ dynamique de l'ecriture des IPR dans les produits de niveau 1 suite
//@ aux modifications apparues dans la version 6.4 du GPFS et la version
//@ 6.5 du IASI Level 1 Product Format Specification
//@ --------------------------------------------------------------------
//@ DM-ID : PM-3 : 04/09/2003 :
//@ -mise dans le domaine publique de la methode MettreAJourTypeRecord()
//@ --------------------------------------------------------------------
//@ DM-ID : PM4 : 31/03/2004 : B. MASSART
//@ -suppression SD_DON_LigneN1::AdapteSN rendue inutile
//@ --------------------------------------------------------------------
//@ DM-ID :IA-CR-2100-246-CNE : 06/07/2004 : B. MASSART
//@ word # 16 in level 0 PX packets is no more read as part of PTSI number
//@ Variable ONBOARD_SOFTWARE_VERSION added in ENG, 1A, 1B and 1C products
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-271-CN : 22/06/2005 : JL.PASCAL
//@ prise en compte du mode transition
//@ fiche de correction correspondante : IA-2100-CR-271-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-162-CN : 09/2006 : C.MERIGUET
//@ Gestion des IPR en mode DUMP
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-191-CN : 18/10/2006 : C.MERIGUET
//@ Recopie du spectre L0 dans les produits ENG (champ BCODSPECVERIFF)
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-196-CN : 15/01/2007 : C.MERIGUET
//@ FA-ID : IA-2100-FA-198-CN : 15/01/2007 : C.MERIGUET
//@ Ces 2 FA sont traitees conjointement. Recuperation du PTSI lu dans 
//@ le mot 17 (2 octets) mais ecriture dans le champ PTSI des N1 des mots 
//@ 16 (version logiciel bord) et 17 (ptsi) (4 octets).
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2364(FA-2100-OPS) : 20/09/2010 : S. BRUNEL
//@ Product without  manoeuvre flag raised 
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_DON_LigneN1_h
#define SD_DON_LigneN1_h


//
// Fichiers include
//

// STL

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN__Date.h"

#include "SD_BAS_Produits.h"
#include "SD_BAS_Niveau1.h"

#include "SD_DON_LigneN0.h"
#include "SD_DON_GestCasDeg.h"
#include "SD_DON_CasDegLigne.h"
#include "SD_DON_OBTUTC.h"
#include "SD_DON_OSV.h"

// Type definissant les types de produits a generer
typedef enum 
    {
        SD_DON_PRODUIT_N1_1C,
        SD_DON_PRODUIT_N1_1B1C,
        SD_DON_PRODUIT_N1_1A1B1C,
        SD_DON_PRODUIT_N1_VER,
        SD_DON_PRODUIT_N1_ENG
    } e_SD_DON_ProduitN1; 


class SD_DON_LigneN1 {

    /*================================================================*/

 public:
    
    ////////////////////////////////
    // Type definitions
    ////////////////////////////////

    
    ////////////////////////////////
    // Constructeur
    ////////////////////////////////
    SD_DON_LigneN1();

    ////////////////////////////////
    // Destructeur
    ////////////////////////////////
    ~SD_DON_LigneN1();
    void Liberer();

    ////////////////////////////////
    // Accesseurs
    ////////////////////////////////
    int                   GetNumLigne() const {return m_NumLigne;};
    t_CMN__Booleen EstValide()
    const {return ((m_InfoGenLigne.GetValidite() == VALIDE) || (m_InfoGenLigne.GetValidite() == TRANSITION));};
    e_SD_DON_ProduitN1    GetTypeLigne() const {return m_TypeLigne;};
    t_CMN__Booleen EstTransition() 
    const {return (m_InfoGenLigne.GetValidite() == TRANSITION);};
    const t_InfoGenLigne& GetDataGen() const;
    void                  GetData(t_SD_BAS_LigneN1& ligneN1);
    void                  SetData(t_SD_BAS_LigneN1& ligneN1);
   
    
    ////////////////////////////////
    // Methodes publiques
    ////////////////////////////////
    void InitLecture(const int numLigne,
                     const t_InfoGenLigne& infoGenLigne,
                     const e_SD_DON_ProduitN1 typeProduit,
                     const e_BAS_TypeRec typeRecord,  
                     SD_DON_GestCasDeg& gestionCasDegrade,
                     t_SD_BAS_Fd& ficDesc);
/* FA 191*/
    // FT-2364
    void InitEcritureN0(const int numLigne,
                        const e_SD_DON_ProduitN1 typeProduit,
                        const e_BAS_TypeRec typeRecord,       
                        const e_BAS_PROCMODE processingMode, 
                        const e_BAS_DISPMODE dispositionMode,
                        const e_BAS_ModePlateforme modePTF,
                        const e_BAS_ModeTrt modeTraitement,
                        const t_InfoGenLigne& infoGenLigne,
                        const int nbLigneAbsolu,
                        const int nbPaquet1,
                        const int nbPaquet2,
                        t_SD_BAS_PaquetN0** dataPaquet1,
                        t_SD_BAS_PaquetN0** dataPaquet2,
                        SD_DON_CasDegLigne& casDegradeLigne,
                        SD_DON_OBTUTC& OBTUTC,
                        const int InfosVds,
                        bool flagManoeuvre)
    // ...FT-2364

        throw (ERR__Exception);
/* ..FA 191*/

    void InitEcritureN1(const int numLigne,
                        const e_SD_DON_ProduitN1 typeProduit,
                        const e_BAS_TypeRec typeRecord,       
                        const e_BAS_ModeTrt modeTraitement,
                        const t_InfoGenLigne& infoGenLigne,
                        t_SD_BAS_LigneN1& dataN1,
                        SD_DON_CasDegLigne& casDegradeLigne) 
        throw (ERR__Exception);
/* FA 162 */
    void Ecrire(t_SD_BAS_Fd& ficDesc,
                            const int idCA,
                            const e_BAS_Nature natureProduit,
			    const e_BAS_ModeTrt mode_traitement) 
        throw (ERR__Exception);
/* ..FA 162 */
    void DecodeEncode(const t_SD_BAS_ORIGINE numLignesNPN,
                      const e_SD_BAS_Atraiter niveau,
                      t_SD_BAS_ConfStable* paramDecodage,
                      const e_SD_BAS_Sens sens,
                      double* data) 
        throw (ERR__Exception);
    void MettreAJourTypeRecord(const e_BAS_Nature natureProduit);


    /*================================================================*/

 protected:

    ////////////////////////////////
    // Methodes protegees
    ////////////////////////////////

    ////////////////////////////////
    // Attributs proteges
    ////////////////////////////////


    /*================================================================*/

 private:
    
    ////////////////////////////////
    // Constante
    ////////////////////////////////
    static const t_Chaine EXCEPTION_SEPARATEUR;
    static const t_Chaine STR_OPS_MODE;
    static const t_Chaine STR_OPS_MODE_NOMINAL;

    ////////////////////////////////
    // Methodes privees
    ////////////////////////////////
    void InitCasDegrade(SD_DON_CasDegLigne::t_SD_DON_Flt& flagsCG,
                        SD_DON_CasDegLigne::t_SD_DON_GOPSDAT& datesCG);
    void IdentifierCasTemporel(SD_DON_CasDegLigne::t_SD_DON_Flt& flagsCG,
                               SD_DON_CasDegLigne::t_SD_DON_GOPSDAT& 
                               datesCG);
    t_Date RecupererDate(e_SD_BAS_IND_DonDetMDRN1ENG champ);
    e_BAS_TypeRecN1 GetTypeRecordN1(const e_BAS_TypeRec typeRecord, 
                                    const e_SD_DON_ProduitN1 
                                    typeProduit,
                                    const t_CMN__Booleen estValide);
    void InfoDetInitDates(const e_BAS_TypeRecN1 typeRecordN1);
    
// FA 196 et 198
    // FT-2364
    void InfoDetInitGen(const int instrumentMode,
                        const int scanPosition,
                        const int mode, 
                        const int target,
                        const int level,
                        const int debugMode,
                        const int interfaceMode,
                        const int PTSI,
                        const int normalProcessingMode,
                        const int backlogProcessingMode,
                        const int reprocessingMode,
                        const int parallelValidation,
                        const int manoeuvre,
                        const e_BAS_PROCMODE processingMode, 
                        const e_BAS_DISPMODE dispositionMode,
                        const e_BAS_ModePlateforme modePTF,
                        const e_BAS_ModeTrt modeTraitement,
                        unsigned int noSP,
                        bool flagManoeuvre);
    // ...FT-2364
// ..FA 196 et 198

    // FT-2364
    void InfoDetInit1C(const e_BAS_PROCMODE processingMode,
                       const e_BAS_DISPMODE dispositionMode,
                       const e_BAS_ModePlateforme modePTF,
                       const e_BAS_ModeTrt modeTraitement,
                       const int nbPaquetPX,
                       t_SD_BAS_PaquetN0** dataPaquetPX,
                       SD_DON_OBTUTC& OBTUTC,
                       bool flagManoeuvre);
    // ...FT-2364
    void InfoDetCopie1C(t_SD_BAS_LigneN1&   dataN1, 
                        const e_BAS_ModeTrt modeTraitement, 
                        const e_SD_DON_ProduitN1 typeProduit);
/* FA 191*/
    // FT-2364
    void InfoDetInitENG(const e_BAS_PROCMODE processingMode,
                        const e_BAS_DISPMODE 
                                    dispositionMode,
                        const e_BAS_ModePlateforme modePTF,
                        const e_BAS_ModeTrt modeTraitement,
                        const int nbLigneAbsolu,
                        const int nbPaquetPX,
                        const int nbPaquetIP,
                        t_SD_BAS_PaquetN0** dataPaquetPX,
                        t_SD_BAS_PaquetN0** dataPaquetIP,
                        SD_DON_OBTUTC& OBTUTC,
                        const int InfosVds,
			bool flagManoeuvre);
    // ...FT-2364
/* ..FA 191*/
    void InfoDetCopieENG(t_SD_BAS_LigneN1& dataN1,
                         const e_BAS_ModeTrt modeTraitement);
    void InfoDetInitVER(const int nbPaquetVP,
                        const int nbPaquetAP,
                        t_SD_BAS_PaquetN0** dataPaquetVP,
                        t_SD_BAS_PaquetN0** dataPaquetAP);
    void InfoDetCopieVER(t_SD_BAS_LigneN1& dataN1);

    ////////////////////////////////
    // Attributs prives
    ////////////////////////////////

    // Numero de la ligne
    int                 m_NumLigne;

    // Type de la ligne (1A/1B/1C, 1B/1C, 1C, VER ou ENG) 
    e_SD_DON_ProduitN1  m_TypeLigne;

    // Type de record (MDR ou VIADR) pour la ligne
    e_BAS_TypeRec       m_TypeRecord;

    // Type d'acces: LECTURE ou ECRITURE
    e_BAS_TypeAcces     m_TypeAcces;

    // Informations generales sur la ligne
    t_InfoGenLigne      m_InfoGenLigne;
    
    // Gestionnaire de cas degrade de la ligne
    SD_DON_CasDegLigne* m_CasDegradeLigne;
        
    // Structure contenant les informations generales et detaillees
    // pour une ligne
    t_SD_BAS_LigneN1    m_DataN1;
};

#endif
