//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_OBTUTC.h,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD_DON
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_OBTUTC
//@
//@ Comment :
//@    this module implement the SD_DON_OBTUTC class
//@    used to offer the method to correct OBT Onboard time
//@    it contains also acces to OBT-UTC aux datafile
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 18/12/2002 : Olivier Duffez
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-136-CN : 01/09/2004 : P.CABANE
//@ Complete access for elements <UTC_LEAP_SECOND> and <B_OFFSET> has been added
//@ fiche de correction correspondante : IA-2100-FC-136-THA
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_DON_OBTUTC_h
#define SD_DON_OBTUTC_h


// Inserer ici les fichiers include 
#include "CMN__Commun.h"
#include "CMN__Date.h"
#include "CMN_PRS_Parser.h"
#include "SD_BAS_Global.h"

// Definition des exceptions

class SD_DON_OBTUTC {

/*====================================================================*/

public:

        //
        // Type
        //	

        //
        // Constant
        //
	//
	// Constructeur
	//
	SD_DON_OBTUTC();
	//
	// Destructeur
	//
	~SD_DON_OBTUTC();

	//
	// Accesseurs
	//
  
  /* FA136 */
  // mise a jour  de la date UTC_LEAP_SECOND
  void SetDateUTCLEAPSECOND (t_Date date) { m_DateUTCLEAPSECOND = date; }
  /* FIN FA136 */
  
  // mise a jour de la date OBT de reference
  void SetDateOBTREF (long long date)  { m_DateOBTREF = date; }

  // mise a jour de la date UTC de reference
  void SetDateUTCREF (t_Date date)            { m_DateUTCREF = date; }

  // mise a jour du parametre A
  void SetAStep      (const double a)         { m_aStep      = a;    }

  /* FA136 */
  // mise a jour du parametre B OFFSET
  void SetBOffset    (const double bOffset)   { m_bOffset    = bOffset; }

  // recuperation de la date UTC_LEAP_SECOND
  t_Date GetDateUTCLEAPSECOND (void) const { return m_DateUTCLEAPSECOND; }
  /* FIN FA136 */
  
  // recuperation de la date OBT de reference
  long long GetDateOBTREF (void) const { return m_DateOBTREF; }

  // recuperation de la date UTT de reference
  t_Date           GetDateUTCREF (void) const { return m_DateUTCREF; }

  // recuperation du parametre A
  double           GetAStep      (void) const { return m_aStep;      }

  // recuperation de l'offset b
  double           GetBOffset    (void) const { return m_bOffset;      }

  // indicateur de chargement des donnees
  t_CMN__Booleen   GetCharge    (void) const { return m_Charge; }

	//
	// public Methods 
	//
	void Lire(
		const t_Chaine & NomFichier
		) throw (ERR__Exception);

	void CorrigeOBT(
		t_SD_BAS_OBTTime & DateOBT,
		t_Date &           DateUTC
		) throw (ERR__Exception);

	/*====================================================================*/
protected:
	//
	// protected Methods 
	//

	//
	// protected Attributes 
	//

	/*====================================================================*/
private:
	//
	// Constantes
	//

  // constante multiplicative pour passer des 1/256 secondes aux secondes
  static const double OBT_TO_SEC;

  //
	// Private Methods 
	//
	
  //
	// Private Attributes 
	//
	
	// objet XML contenant les coefficients de correction des dates 
	CMN_PRS_Parser m_DataXml;

 /* FA136 */
 // date UTC, extraite du tag XML <UTC_LEAP_SECOND>
 t_Date m_DateUTCLEAPSECOND;
 /* FIN FA136 */
  
	// date OBT de reference, extraite du tag XML <CCU_OBT>, notee k0 dans l'algo
	long long m_DateOBTREF;
	
	// date UTC de reference, extraite du tag XML <UTC>, notee t0 dans l'algo
	t_Date m_DateUTCREF;
	
	// step a, extrait du tag XML <A_STEP>, note a dans l'algo
	double m_aStep;
	
	// offset b, extrait du tag XML <B_OFFSET>, inutile dans l'algo ? TODO
	double m_bOffset;

  // indicateur de chargement
	t_CMN__Booleen m_Charge;
	
};


#endif
