static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_PaquetN0.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_PaquetN0
//@
//@ Commentaire :
//@    Ce module gere la classe SD_DON_PaquetN0, qui represente un 
//@    paquet de donnees N0
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 06/01/2003 : P. CABANE
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN__Date.h"

#include "SD_DON_PaquetN0.h"


//
// Definition des constantes
//


//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD_DON_PaquetN0
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_PaquetN0::SD_DON_PaquetN0() : m_EstValide(CMN__VRAI)
{
  m_DataN0.type           = IP;
  m_DataN0.InfoGen.nb_var = 0;
  m_DataN0.InfoDet.nb_var = 0;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_DON_PaquetN0
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_PaquetN0::~SD_DON_PaquetN0()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Liberer
//@
//@ Description : Liberation memoire des donnees de l'objet
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_PaquetN0::Liberer()
{
  //@ SI le paquet est valide
  if (m_EstValide == CMN__VRAI)
    {
      // Donnees de contexte
      t_SD_BAS_DataThread* ctx = 0;
      
      
      //@ Allocation memoire pour les donnees de contexte
      ctx = new t_SD_BAS_DataThread;
      InitialiserCompteRendu(&(ctx->cr));
    
      //@ On libere ici les donnees eventuellement allouees
      //@ via un appel a SD_BAS_LibereMDRN0()
      SD_BAS_LibereMDRN0(ctx, &m_DataN0);
    
      //@ Liberation memoire pour les donnees de contexte
      LibererCompteRendu(&(ctx->cr.msgJDB));
      delete ctx;
    }
  //@ FIN SI le paquet est valide
}


//@ ------------------------------------------------------------------*/
//@ Methode : GetData
//@
//@ Description : Recupere les donnees fondamentales des paquets de
//@               niveau 0
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Donnees paquet N0
//@
//@ ------------------------------------------------------------------*/
t_SD_BAS_PaquetN0* SD_DON_PaquetN0::GetData()
{
  //@ Renvoie la donnee paquet N0
  return (&m_DataN0);
}


//@ ------------------------------------------------------------------*/
//@ Methode : SetData
//@
//@ Description : Met a jour les donnees fondamentales des paquets de
//@               niveau 0
//@
//@ Parametres :
//@               (ENTREE) data : Donnees paquet de niveau 0
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_PaquetN0::SetData(t_SD_BAS_PaquetN0* data)
{
  //@ Met a jour la donnee paquet N0
  m_DataN0 = *data;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Lire
//@
//@ Description : Lit un ISP dans le granule de niveau 0
//@
//@ Parametres :
//@               (ENTREE/SORTIE) ficDesc : Descripteur de fichier
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_PaquetN0::Lire(t_SD_BAS_Fd& ficDesc)
{
  // Donnees de contexte
  t_SD_BAS_DataThread* ctx    = 0;

  // Code retour des fonctions utilisees
  t_CMN__Statut        statut = CMN__STATUT_OK;
  

  //@ Allocation memoire pour les donnees de contexte
  ctx = new t_SD_BAS_DataThread;
  InitialiserCompteRendu(&(ctx->cr));

  //@ La correction UTC issue des fic aux OBTUTC (via SD_DON_OBTUTC)
  //@ a deja ete faite dans SD_DON_GranuleN0::GestionOverlap()

  //@ Appel de la fonction de lecture des donnees 
  //@ detaillees d'un MDR
  statut = SD_BAS_LireMDRN0(&ficDesc,
                            ficDesc.offsetMDRSuivant,
                            ctx,
                            &m_DataN0);

  //@ SI la lecture des donnees a genere une erreur
  if (statut != CMN__STATUT_OK)
    {

#ifdef OPS_DEBUG
      cerr << "SD_DON_PaquetN0::Lire: Erreur LireMDRN0: " << 
        ficDesc.nom << " (" << statut << ")" << endl;
#endif

      //@ Mise a jour de la validite des donnees
      m_EstValide = CMN__FAUX;
      
      //@ Envoi des messages generes au JDB
      EnvoyerMessagesStockes(&(ctx->cr.msgJDB));
    }
  //@ FIN SI la lecture des donnees a genere une erreur
 
  //@ Le decodage n'est pas fait ici pour les donnees image et 
  //@ spectre, il sera fait a la volee lors du traitement

  //@ Liberation memoire pour les donnees de contexte
  LibererCompteRendu(&(ctx->cr.msgJDB));
  delete ctx;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Decode
//@
//@ Description : Decode les donnees fondamentales des paquets de
//@               niveau 0 (images ou spectres). Le decodage se
//@               limite ici a l'extraction du champ de bit 
//@               correspondant
//@
//@ Parametres :
//@               (SORTIE) dataDecode : Donnees decodees
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_PaquetN0::Decode(void* dataDecode) throw (ERR__Exception)
{
  //@ SI les donnees ne sont pas valides
  if (m_EstValide == CMN__FAUX)
    {
      //@ La donnee est reinitialisee a NULL
      dataDecode = 0;
    }
  //@ SINON
  else 
    {
      // Code retour des fonctions utilisees
      t_CMN__Statut        statut = CMN__STATUT_OK;

      // Donnees de contexte
      t_SD_BAS_DataThread* ctx    = 0;


      //@ Allocation memoire pour les donnees de contexte
      ctx = new t_SD_BAS_DataThread;
      InitialiserCompteRendu(&(ctx->cr));

      //@ On appelle la fonction de decodage en fonction du 
      //@ type de donnees

      //@ SI les donnees sont de type PX
      if (m_DataN0.type == PX)
        {
          //@ Appel de la procedure SD_BAS_DecodeSpectreN0
          statut = SD_BAS_DecodeSpectreN0(&m_DataN0, 
                                          ctx, 
                                          (char*) dataDecode);
        }
      //@ SINON SI les donnees sont de type IP
      else if (m_DataN0.type == IP)
        {
          //@ Appel de la procedure SD_BAS_DecodeImageN0
          statut = SD_BAS_DecodeImageN0(&m_DataN0, 
                                        ctx, 
                                        (double *) dataDecode);
        }        
      //@ FIN SI les donnees sont de type PX


      //@ SI le decodage a echoue
      if (statut != CMN__STATUT_OK)
        {
          // Exception generee
          ERR__Exception exception;
          
          // Message pour l'exception
          t_ChaineFixe   message;
          t_Chaine       messageException("");

#ifdef OPS_DEBUG
          cerr << "SD_DON_PaquetN0::Decode: Erreur Decode (" << 
            statut << ")" << endl;
#endif

          //@ Recuperation du message d'erreur genere par 
          //@ les couches basses
          RecupererDernierMessageStocke(&(ctx->cr.msgJDB), message);
          messageException = t_Chaine(message);

          //@ Envoi des messages generes au JDB
          EnvoyerMessagesStockes(&(ctx->cr.msgJDB));
 
          //@ Liberation memoire pour les donnees de contexte
          LibererCompteRendu(&(ctx->cr.msgJDB));
          delete ctx;

          //@ Emission d'une exception SD_DON_OPER_FAILED
          exception.SetErrorCode(ERR__Exception::SD_DON_OPER_FAILED);
          exception.SetParametre(messageException);
          THROW(exception);
        }
      //@ FIN SI le decodage a echoue

      //@ Liberation memoire pour les donnees de contexte
      LibererCompteRendu(&(ctx->cr.msgJDB));
      delete ctx;
    }
  //@ FIN SI les donnees ne sont pas valides

  //@ Remarque : la fonction appelante est capable de memoriser
  //@ les parametres de decodage puis de les appliquer
}


//======================================================================
//                                      METHODES PRIVEES 
//======================================================================
