static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_SVM.cc,v 1.5 2013/03/29 16:55:26 lcaspar Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD_DON
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_SVM
//@ 
//@ Comment :
//@    this module implement the SD_DON_SVM class
//@    used to offer the method to determine the platform mode
//@    it contains also acces to SVM ASCII aux datafile
//@ 					
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 18/12/2002 : Olivier Duffez
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-166-CN : 13/02/2006 : B. MASSART
//@ An exception is added when string "UMBRA_END" is not found in SVM auxiliary
//@ file. See IA-2100-FC-166-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-176-CN : 19/09/2006 : R.MORENO
//@ Correction gestion du champ UMBRA_END dans le fichier SVM
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-217-CN : 12/06/2006 : S.BRUNEL
//@ Correction du calcul de la date de la derniere eclipse 
//@ par rapport a une heure UTC.
//@ --------------------------------------------------------------------
//@ DM-ID :IA-2100-FT-2449-CN : 11/04/2008 : S. BRUNEL
//@ Rendre la lecture du fichier SVM plus efficace 
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2939(DM-2100-OPS) : 12/12/2011 : L. CASPAR
//@ Mise en place de conditions de levee du cas degarde 9 alternatives pour 
//@ correction de la NCR 12303
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <fstream>
#include "SD_DON_SVM.h"

// constantes privees
const int SD_DON_SVM::TAILLE_MAX_LIGNE_SVM  = 512;// nb max de caract. d'une ligne
const int SD_DON_SVM::COLONNE_FROM_ANX      = 36;// no de colonne du champ FROM_ANX
const int SD_DON_SVM::COLONNE_UMBRA_END     = 63;// no de colonne du champ UMBRA_END
const int SD_DON_SVM::LONGUEUR_FROM_ANX     = 7; // nb de caract. du champ FROM_ANX
const int SD_DON_SVM::LONGUEUR_UMBRA_END    = 9; // nb de caract. du champ UMBRA_END
const t_Chaine SD_DON_SVM::CHAINE_UMBRA_END = "UMBRA_END";
const t_Chaine SD_DON_SVM::MISSING_AUX_FILE = "Missing SVM file :";

// FT-2939
const int SD_DON_SVM::COLONNE_NIGHT_STR     = 63;// no de colonne du champ NIGHT_STR
const int SD_DON_SVM::COLONNE_NIGHT_END     = 63;// no de colonne du champ NIGHT_END
const int SD_DON_SVM::LONGUEUR_NIGHT_STR    = 9; // nb de caract. du champ NIGHT_STR
const int SD_DON_SVM::LONGUEUR_NIGHT_END    = 9; // nb de caract. du champ NIGHT_END
const t_Chaine SD_DON_SVM::CHAINE_NIGHT_STR = "NIGHT_STR";
const t_Chaine SD_DON_SVM::CHAINE_NIGHT_END = "NIGHT_END";
// ..FT-2939

/* FA166 */
const t_Chaine SD_DON_SVM::DATA_AUX_UMBRAEND_NOT_FOUND = "No UMBRA_END in file :";



// initialisation variable statique
pthread_mutex_t   SD_DON_SVM::m_MutexTravaille;
//======================================================================
//					 METHODES PUBLIQUES 
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Methode : SD_DON_SVM
//@
//@ Description : Constructeur
//@
//@ Parameters :
//@			Aucun
//@
//@ Return Status : 
//@			Aucun
//@
//@ ------------------------------------------------------------------*/
SD_DON_SVM::SD_DON_SVM()
  : m_NomFichier(""),
    m_Charge(false)
{
}

//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_DON_SVM
//@
//@ Description : Destructeur
//@
//@
//@ Parameters :
//@			Aucun
//@
//@ Return Status : 
//@			Aucun
//@
//@ ------------------------------------------------------------------*/
SD_DON_SVM::~SD_DON_SVM()
{
}

//@ ------------------------------------------------------------------*/
//@ Methode : InitialiserMutex
//@
//@ Description : 
//@
//@ Parameters :
//@     Aucun
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_SVM::InitialiserMutex() 
{
  //@ Creer la variable conditionnelle POSIX correspondante et son mutex
  pthread_mutex_init(&m_MutexTravaille, NULL);
#ifdef OPS_DEBUG
          cout << "InitialiserMutex SVM" << endl;
#endif
}
//@ ------------------------------------------------------------------*/
//@ Methode : LibererMutex
//@
//@ Description : 
//@
//@ Parameters :
//@     Aucun
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_DON_SVM::LibererMutex()
{
  // Liberation si besoin des donnees de type pointeurs
  //@ Supprimer le mutex et la variable conditionnele POSIX
#ifdef OPS_DEBUG
          cout << "Liberermutex SVM " << endl;
#endif
  pthread_mutex_destroy(&m_MutexTravaille);

}
//@ ------------------------------------------------------------------*/
//@ Methode : Reserver
//@
//@ Description : Cette methode permet de reserver la donnee pour sa mise 
//@ a jour ( cette fonction est bloquante si une autre reservation est en cours
//@  
//@
//@
//@ Parameters : aucun
//@
//@ Return Status : aucun
//@
//@ ------------------------------------------------------------------*/
void SD_DON_SVM::Reserver()

{
  // Entree dans la fonction 
  //@ reservation par mutex
  pthread_mutex_lock(&m_MutexTravaille);
  // on trace ici si besoin toutes les variables en sortie

  // sortie de la fonction 

}

//@ ------------------------------------------------------------------*/
//@ Methode : Liberer
//@
//@ Description : Cette methode permet de liberer un contexte precedemment
//@ reserve ( cette fonction libere une autre reservation  en cours )
//@  
//@
//@
//@ Parameters : aucun
//@
//@ Return Status : aucun
//@
//@ ------------------------------------------------------------------*/
void SD_DON_SVM::Liberer()

{
  // Entree dans la fonction 
  //@ liberation du  mutex
  pthread_mutex_unlock(&m_MutexTravaille);
  // on trace ici si besoin toutes les variables en sortie

  // sortie de la fonction 

}
//@ ------------------------------------------------------------------*/
//@ Methode : Lire
//@
//@ Description : Cette methode permet de tester si le fichier donne en
//@ argument existe, et de stocker son nom dans l'attribut m_NomFichier
//@
//@ Parameters :
//@			(ENTREE)  NomFichier : Nom du fichier SVM (ASCII)
//@
//@ Return Status : aucun
//@
//@ ------------------------------------------------------------------*/
void SD_DON_SVM::Lire(
		const t_Chaine & NomFichier
		) throw (ERR__Exception)
{
  // FT-2449
  static const char tcProcName_F[] = "SD_DON_SVM::Lire";
  ERR__Exception exception;
  char       c_ligne[TAILLE_MAX_LIGNE_SVM+1] = "";
  t_Chaine   ligne, cible;
  bool       umbra_end_trouve = false; // indique si la chaine UMBRA_END est trouvee
  int 			nombreUmbraEnd = 0; // indique le nombre de dates d'eclipse lues
  t_Date     date_courante;// date UTC at ANX lue dans la ligne courante
  CMN__Heure heureFromANX; // nb de millisecondes depuis ANX
  int        millisFromANX;// nb de millisecondes depuis ANX

  // FT-2939
  bool       night_end_trouve = false; // indique si la chaine NIGHT_END est trouvee
  int 			nombreNightEnd = 0; // indique le nombre de dates de transition nuit=>jour lues
  bool       night_str_trouve = false; // indique si la chaine NIGHT_STR est trouvee
  int 			nombreNightStr = 0; // indique le nombre de dates de transition jour=>nuit lues
  t_Date     date_transition_tmp; // date de transition de la ligne en cours
  // ..FT-2939

  // FA 217
  t_Date     date_eclipse_tmp; // date de fin d'eclipse de la ligne en cours
  // ...FA 217 

  // si le nom du fichier n'est pas initialise, on lance une exception
  if (NomFichier.size() == 0)
  {
    //@ Emission exception
    exception.SetErrorCode(ERR__Exception::CANT_OPEN_FILE);
    THROW(exception);
  }
  
  m_tabDateFinEclipse.clear();
  // FT-2939
  m_tabDateTransitionJN.clear();
  m_tabDateTransitionNJ.clear();
  // ..FT-2939

  //@ on se positionne en debut de fichier
  ifstream   fichier(NomFichier.c_str());
  if (fichier.fail())
  {
    //@ Emission exception
    exception.SetErrorCode(ERR__Exception::SD_DON_FILE_OPEN );
    exception.SetParametre(MISSING_AUX_FILE+NomFichier.c_str());
    THROW(exception);
  }

  //@ deplacement dans le fichier  de maniere a se positionner sur le 
  //@ premier element de type : UMBRA_END (5eme champ)
  //@ FORMAT: A24, 1X, A10, 1X, A7, 1X, I7, 1X, A10, 1X, A40, 1X, A3, 
  //@ 1X, A10, 1X, A10, 1X, A40)

  // on memorise chaque date de fin d'eclipse dans une liste
  while (!fichier.eof())
  {
  
    // lecture de la ligne courante
    fichier.getline(c_ligne, TAILLE_MAX_LIGNE_SVM);
    c_ligne[TAILLE_MAX_LIGNE_SVM]='\0';
    ligne = (t_Chaine)c_ligne;

    // recherche de la chaine UMBRA_END dans la colonne COLONNE_UMBRA_END
    umbra_end_trouve = ligne.find(CHAINE_UMBRA_END, COLONNE_UMBRA_END) == COLONNE_UMBRA_END ;
 
    // FT-2939
    // recherche de la chaine NIGHT_STR dans la colonne COLONNE_NIGHT_STR
    night_str_trouve = ligne.find(CHAINE_NIGHT_STR, COLONNE_NIGHT_STR) == COLONNE_NIGHT_STR ;
    // recherche de la chaine NIGHT_END dans la colonne COLONNE_NIGHT_END
    night_end_trouve = ligne.find(CHAINE_NIGHT_END, COLONNE_NIGHT_END) == COLONNE_NIGHT_END ;
    // ..FT-2939
    
    if (umbra_end_trouve) 
    {
      //@ decoder le champ FromANX (3eme champ) et le transformer en t_Date
      millisFromANX = atoi(ligne.substr(COLONNE_FROM_ANX, LONGUEUR_FROM_ANX).c_str());
      heureFromANX.Seconde((double)millisFromANX/1000.0);

      //@ conversion de la chaine de caracteres (UTC at ANX) en t_Date
      date_courante.ChaineUTC(ligne);

      // calcule la date de fin d'eclipse
      date_eclipse_tmp = date_courante;
      date_eclipse_tmp.Heure(date_eclipse_tmp.Heure() + heureFromANX);

      // ajoute la date de fin d'eclipse
      m_tabDateFinEclipse.push_back(date_eclipse_tmp);
      nombreUmbraEnd++;
    // FT-2939
    } else if (night_str_trouve) 
    {
       //@ decoder le champ FromANX (3eme champ) et le transformer en t_Date
       millisFromANX = atoi(ligne.substr(COLONNE_FROM_ANX, LONGUEUR_FROM_ANX).c_str());
       heureFromANX.Seconde((double)millisFromANX/1000.0);

       //@ conversion de la chaine de caracteres (UTC at ANX) en t_Date
       date_courante.ChaineUTC(ligne);

       // calcule la date de transition
       date_transition_tmp = date_courante;
       date_transition_tmp.Heure(date_transition_tmp.Heure() + heureFromANX);

      // ajoute la date de transition
      m_tabDateTransitionJN.push_back(date_transition_tmp);
      nombreNightStr++;    
    } else if (night_end_trouve) 
    {
       //@ decoder le champ FromANX (3eme champ) et le transformer en t_Date
       millisFromANX = atoi(ligne.substr(COLONNE_FROM_ANX, LONGUEUR_FROM_ANX).c_str());
       heureFromANX.Seconde((double)millisFromANX/1000.0);

       //@ conversion de la chaine de caracteres (UTC at ANX) en t_Date
       date_courante.ChaineUTC(ligne);

       // calcule la date de transition
       date_transition_tmp = date_courante;
       date_transition_tmp.Heure(date_transition_tmp.Heure() + heureFromANX);

      // ajoute la date de transition
      m_tabDateTransitionNJ.push_back(date_transition_tmp);
      nombreNightEnd++;    
    } else if (nombreUmbraEnd > 0 && nombreNightStr > 0 && nombreNightEnd > 0) {
      // on considere que les lignes UMBRA_END, NIGHT_END, NIGHT_STR sont sequentielles
      break;
    }
    // ..FT-2939
  }

#ifdef OPS_DEBUG
  cout << "Nombre de date de fin d'eclipse lu : " << nombreUmbraEnd << endl;
  // FT-2939
  cout << "Nombre de date de transition jour=>nuit lu : " << nombreNightStr << endl;
  cout << "Nombre de date de transition nuit=>jour lu : " << nombreNightEnd << endl;
  // ..FT-2939
#endif
  
  // si on n'a pas trouve la chaine UMBRA_END on lance une exception
  if (nombreUmbraEnd == 0)
  {
    //@ Emission exception
    exception.SetErrorCode(ERR__Exception::SD_DON_SVM_NOUMBRAEND); /* FA166 */
    THROW(exception);
  }
  // FT-2939
  // si on n'a pas trouve la chaine NIGHT_STR on trace avec un message
  if (nombreNightStr == 0)
  {
     cout << "Pas de ligne NIGHT_STR trouvee dans le fichier SVM" << endl;
  }
  // si on n'a pas trouve la chaine NIGHT_END on trace avec un message
  if (nombreNightEnd == 0)
  {
     cout << "Pas de ligne NIGHT_END trouvee dans le fichier SVM" << endl;
  }
  // ..FT-2939
  
  m_NomFichier = NomFichier;
  fichier.close();
  // ..FT-2449
}

// FT-2939
//@ ------------------------------------------------------------------*/
//@ Methode : GetDateTransition
//@
//@ Description : Cette methode permet de recuperer la date de transition
//@ jour=>nuit ou nuit=>jour la plus proche de celle passee en parametre 
//@
//@ Parameters :
//@			(ENTREE)  date : date de reference               
//@     (SORTIE)  date de transition la plus proche de celle en entree
//@     (SORTIE)  indique si une date de transition a ete trouvee
//@
//@ Return Status : aucun
//@ ------------------------------------------------------------------*/
void SD_DON_SVM::GetDateTransition(
		t_Date & dateRef,
		t_Date & date_transition,
    bool * dateTransitionTrouvee 
    
		) throw (ERR__Exception)
{
    static const char tcProcName_F[] = "SD_DON_SVM::GetDateTransition";
    t_CMN__Statut  cr_op=CMN__STATUT_OK;
    ERR__Exception exception;
    t_Date datePrecJN;
    t_Date dateSuivJN;
    t_Date datePrecNJ;
    t_Date dateSuivNJ;
    t_Date datePrec;
    t_Date dateSuiv;
    t_Date dateLue;
    int 	nombreNightStr = (int)m_tabDateTransitionJN.size();
    int 	nombreNightEnd = (int)m_tabDateTransitionNJ.size();
    int 	indexTransition = 0;
    bool 	nightStrPrecTrouve = false;
    bool 	nightStrSuivTrouve = false;
    bool 	nightEndPrecTrouve = false;
    bool 	nightEndSuivTrouve = false;
    bool 	datePrecExistante = true;
    bool 	dateSuivExistante = true;
    
    // Parcours de la liste des transitions Jour => Nuit
    if (nombreNightStr > 0)
    {
      do  {
        dateLue = m_tabDateTransitionJN[indexTransition];
        if (dateLue <= dateRef)
        {
          datePrecJN = dateLue;
          nightStrPrecTrouve = true;
        } else {
          dateSuivJN = dateLue;
          nightStrSuivTrouve = true;
        }
        indexTransition++;
      } 
      while ((nightStrSuivTrouve == false) && (indexTransition < nombreNightStr));
    }
    
    // Parcours de la liste des transitions Nuit => Jour
    indexTransition=0;
    if (nombreNightEnd > 0)
    {
      do  {
        dateLue = m_tabDateTransitionNJ[indexTransition];
        if (dateLue <= dateRef)
        {
          datePrecNJ = dateLue;
          nightEndPrecTrouve = true;
        } else {
          dateSuivNJ = dateLue;
          nightEndSuivTrouve = true;
        }
        indexTransition++;
      } 
      while ((nightEndSuivTrouve == false) && (indexTransition < nombreNightEnd));
    }
    // Comparaison des dates "Precedentes" 
    if (nightStrPrecTrouve && nightEndPrecTrouve){
      // Comparaison
      if(datePrecJN <= datePrecNJ)
      {
        datePrec = datePrecNJ;
      }else{
        datePrec = datePrecJN;
      }
   }else if (nightStrPrecTrouve){
      datePrec = datePrecJN;
   }else if (nightEndPrecTrouve){
      datePrec = datePrecNJ;
    }else{
      datePrecExistante = false;
    }
    
    // Comparaison des dates "Suivantes" 
    if (nightStrSuivTrouve && nightEndSuivTrouve){
      // Comparaison
      if(dateSuivJN > dateSuivNJ)
      {
        dateSuiv = dateSuivNJ;
      }else{
        dateSuiv = dateSuivJN;
      }
    }else if (nightStrSuivTrouve){
      dateSuiv = dateSuivJN;
    }else if (nightEndSuivTrouve){
      dateSuiv = dateSuivNJ;
    }else{
      dateSuivExistante = false;
    }

    // Comparaison des dates finalistes 
    if (dateSuivExistante && datePrecExistante)
    {
      // Comparaison
      if(dateRef-datePrec <= dateSuiv-dateRef)
      {
        date_transition = datePrec;
        *dateTransitionTrouvee = true;
      }else{
        date_transition = dateSuiv;
        *dateTransitionTrouvee = true;
      }
    } else if(datePrecExistante){
      date_transition = datePrec;
        *dateTransitionTrouvee = true;
    } else if(dateSuivExistante){
      date_transition = dateSuiv;
        *dateTransitionTrouvee = true;
    } else {
        *dateTransitionTrouvee = false;
    }
}
// ..FT-2939

//@ ------------------------------------------------------------------*/
//@ Methode : GetDateEclipse
//@
//@ Description : Cette methode permet de recuperer la date de fin
//@ de la derniere eclipse 
//@
//@ Parameters :
//@			(ENTREE)  dateUTC : date UTC corrigee
//@			si aucune date n'est trouvee on retourne une date
//@			initialisee
//@     (SORTIE)  date UTC de fin de la derniere eclipse
//@
//@ Return Status : aucun
//@ ------------------------------------------------------------------*/
void SD_DON_SVM::GetDateEclipse(
		t_Date & dateUTC,
		t_Date & date_eclipse
		) throw (ERR__Exception)
{
    // FT-2449
    static const char tcProcName_F[] = "SD_DON_SVM::GetDateEclipse";
    t_CMN__Statut  cr_op=CMN__STATUT_OK;
    ERR__Exception exception;
    t_Date dateFinEclipse;
    int 	nombreUmbraEnd = (int)m_tabDateFinEclipse.size();
    int 	indexUmbraEnd = 0;
    bool 	umbraEndTrouve = false;
    
    do  {
      dateFinEclipse = m_tabDateFinEclipse[indexUmbraEnd];
      if (dateFinEclipse >= dateUTC)
      {
        umbraEndTrouve = true;
      }
      indexUmbraEnd++;
    } 
    while ((umbraEndTrouve == false) && (indexUmbraEnd < nombreUmbraEnd));

    if (umbraEndTrouve == false)
    {
      //@ Emission exception
      exception.SetErrorCode(ERR__Exception::SD_DON_SVM_NOUMBRAEND); /* FA166 */
      THROW(exception);
    }
    date_eclipse = dateFinEclipse;
    // ..FT-2449

#ifdef OPS_DEBUG
  cout << "Date UTC : " << dateUTC.Chaine() << " Date de fin d'eclipse : " << date_eclipse.Chaine() << endl;
#endif
}


//======================================================================
//					METHODES PRIVEES 
//======================================================================

