//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_SVM.h,v 1.5 2013/03/29 16:55:26 lcaspar Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD_DON
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_SVM
//@
//@ Comment :
//@    this module implement the SD_DON_SVM class
//@    used to offer the method to determine the platform mode
//@    it contains also acces to SVM ASCII aux datafile
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 18/12/2002 : Olivier Duffez
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-166-CN : 13/02/2006 : B. MASSART
//@ An exception is added when string "UMBRA_END" is not found in SVM auxiliary
//@ file. See IA-2100-FC-166-THA.
//@ --------------------------------------------------------------------
//@ DM-ID :IA-2100-FT-2449-CN : 11/04/2008 : S. BRUNEL
//@ Rendre la lecture du fichier SVM plus efficace 
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2939(DM-2100-OPS) : 12/12/2011 : L. CASPAR
//@ Mise en place de conditions de levee du cas degarde 9 alternatives pour 
//@ correction de la NCR 12303
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_DON_SVM_h
#define SD_DON_SVM_h


// Inserer ici les fichiers include 
#include <pthread.h>
#include "SD_BAS_Global.h"
#include "CMN__Commun.h"
#include "CMN__Date.h"
#include "ERR__Exception.h"


// Definition des exceptions

class SD_DON_SVM {

/*====================================================================*/

public:

  //
  // Type
  //	

  //
  // Constant
  //

  //
	// Constructeur
	//
	SD_DON_SVM();
	
  //
	// Destructeur
	//
	virtual ~SD_DON_SVM();

	//
	// Accesseurs
	//

	//
	// public Methods 
	//
	void InitialiserMutex();
	void LibererMutex();
	void Reserver();
	void Liberer();
	void Lire(
		const t_Chaine & NomFichier
		) throw (ERR__Exception);

	void GetDateEclipse(
		t_Date & dateUTC,
		t_Date & date_eclipse
		) throw (ERR__Exception);
    
	// FT-2939
	void GetDateTransition(
		t_Date & dateRef,
		t_Date & date_transition,
	bool * dateTransitionTrouvee
		) throw (ERR__Exception);
	// ..FT-2939
		
	/*====================================================================*/
protected:
	//
	// protected Methods 
	//

	//
	// protected Attributes 
	//

	/*====================================================================*/
private:
	//
	// Private constants 
	//
  static const int TAILLE_MAX_LIGNE_SVM;// nb max de caract. d'une ligne
  static const int COLONNE_FROM_ANX;    // no de colonne du champ FROM_ANX
  static const int COLONNE_UMBRA_END;   // no de colonne du champ UMBRA_END
  static const int LONGUEUR_FROM_ANX;   // nb de caract. du champ FROM_ANX
  static const int LONGUEUR_UMBRA_END;  // nb de caract. du champ UMBRA_END
  static const t_Chaine CHAINE_UMBRA_END;
  static const t_Chaine MISSING_AUX_FILE;
  static const t_Chaine DATA_AUX_UMBRAEND_NOT_FOUND; /* FA166 */

  // FT-2939
  static const int COLONNE_NIGHT_STR; // no de colonne du champ NIGHT_STR
  static const int COLONNE_NIGHT_END; // no de colonne du champ NIGHT_END
  static const int LONGUEUR_NIGHT_STR; // nb de caract. du champ NIGHT_STR
  static const int LONGUEUR_NIGHT_END; // nb de caract. du champ NIGHT_END
  static const t_Chaine CHAINE_NIGHT_STR;
  static const t_Chaine CHAINE_NIGHT_END;
  // ..FT-2939

	//
	// Private Methods 
	//

  //
	// Private Attributes 
	//
	
	// nom du fichier en entree 
	t_Chaine m_NomFichier;

	// FT-2449
	// table des dates de fin d'eclipse	
	vector<t_Date> m_tabDateFinEclipse;
	// ..FT-2449

	// FT-2939
	// table des dates de transition	
	vector<t_Date> m_tabDateTransitionJN;
	vector<t_Date> m_tabDateTransitionNJ;
	// ..FT-2939
	
	// indicateur de chargement
	bool m_Charge;
	  	
	// Mutex pour l'acces au fichier 
  	static pthread_mutex_t   m_MutexTravaille;

};


#endif
