static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/FRW/src/SD_FRW_ApplicationServeurDonnees.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_FRW
//@ CLASSE : SD_FRW_ApplicationServeurDonnees
//@ 
//@ Commentaire :
//@    Ce module gere la classe SD_FRW_ApplicationServeurDonnees qui
//@    est le point d'entree du Serveur de Donnees
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 09/08/1999 : SAS
//@ --------------------------------------------------------------------
//@ FAID-SYS-005            22/11/99   SAS      Journalisation du commentaire de reconfiguration
//@ --------------------------------------------------------------------
//@ FAID<026>               25/11/99   Agnetti  Ajout appel proc d'init signaux,core,path
//@ --------------------------------------------------------------------
//@ FAID<SMM-M4-FA-310-CN>  22/05/00   Robert   Recuperation de la liste des fichiers externes en import
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-???-CN        27/03/2001 Robert   On attend quelques secondes entre chaque tentative
//@                                             d'affectation d'une tache a un thread
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-2285-CN       22/07/2001 Robert   On genere une alarme pour les comptes rendu asynchrone en erreur
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-2321-CN       30/08/2001 Robert   Il faut verrouiller avant la creation de tout fichier par le SD
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-2324-CN       30/08/2001 Robert   Il faut verrouiller avant l'allocation d'un chrono
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-2351-CN       30/08/2001 Robert   m_thr_nb-- a ete deplace pour s'executer avant pthread_mutex_unlock
//@ --------------------------------------------------------------------
//@ SMM-DM-M4-GO-21329  27/11/01        GRT     Import export sur une periode
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-2531-CN       06/01/2002 Robert   On calcule la prochaine echeance avant de lancer la tache pour que la meme tache ne soit pas relancee plusieurs fois
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-2706-CLS      15/03/2002 Robert   On signale a l'admin l'activite du SD
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Adaptation a l'OPS : modification du mecanisme d'interface, ajout
//@                      de fonctionnalites supplementaires et de la
//@                      gestion des exceptions
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-159-CN : 26/05/2005 : JL. PASCAL
//@ modif suite a portage linux
//@ fiche de correction correspondante : IA-2100-FC-159-THA
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL
#include <iostream>

// Standard
#include <pthread.h>
// FT-2765
#include <signal.h>
// ...FT-2765

// Projet
#include "CMN__Commun.h"
#include "CMN__Version.h"
#include "CMN__Securite.h"
#include "CMN__Date.h"
#include "CMN__Fichier.h"
#include "CMN__OPSProcess.h"
#include "CMN__PGEServices.h"
#include "CFG__Configuration.h"
#include "MSG__Abonnement.h"
#include "TES__Evenement.h"
#include "MP__Message.h"
#include "MP__HKTMStatus.h"
#include "SD_DON_Traitement.h"
#include "SD_TAC_Tache.h"
#include "SD_TAC_TacheTraitement.h"
#include "SD_TAC_GestionnaireDeSession.h"
#include "SD_EVT_SujetTerminaisonTache.h"
#include "SD_FRW_ApplicationServeurDonnees.h"


//
// Definition des constantes
//

// Definition des timeouts
const int      SD_FRW_ApplicationServeurDonnees::DEFAULT_TIMEOUT          = 100000;
const int      SD_FRW_ApplicationServeurDonnees::DEFAULT_SCRUTATION       = 100000;

// Fichier de configuration du SD
const t_Chaine SD_FRW_ApplicationServeurDonnees::FICHIER_CONFIGURATION_SD = "OPS_SD.cfg";
const t_Chaine SD_FRW_ApplicationServeurDonnees::BLOC_THREAD              = "MultiThreading";
const t_Chaine SD_FRW_ApplicationServeurDonnees::VAL_NB_THREAD            = "NombreDeThreads";
const t_Chaine SD_FRW_ApplicationServeurDonnees::BLOC_ETAT                = "EtatServeur";
const t_Chaine SD_FRW_ApplicationServeurDonnees::VAL_TIMEOUT              = "Timeout";
const t_Chaine SD_FRW_ApplicationServeurDonnees::VAL_SCRUTATION           = "PeriodeDeScrutation";
const t_Chaine SD_FRW_ApplicationServeurDonnees::BLOC_SIMU                = "Simu";
const t_Chaine SD_FRW_ApplicationServeurDonnees::VAL_MAX_LIGNE            = "NombreLignes";
const t_Chaine SD_FRW_ApplicationServeurDonnees::VAL_NB_LIGNE             = "NbLigIn";
const t_Chaine SD_FRW_ApplicationServeurDonnees::VAL_NB_COLONNE           = "NbColIn";

// Communication
const t_Chaine SD_FRW_ApplicationServeurDonnees::MSG_SERVER_HOST          = "MSG_SERVER_HOST";
const t_Chaine SD_FRW_ApplicationServeurDonnees::MSG_SERVER_PORT          = "MSG_SERVER_PORT";
const t_Chaine SD_FRW_ApplicationServeurDonnees::TIMEOUT_SD               = "TIMEOUT_SD";
const t_Chaine SD_FRW_ApplicationServeurDonnees::LISTE_SD                 = "LISTE_SD";
const t_Chaine SD_FRW_ApplicationServeurDonnees::NOM_EVT_TIMER            = "STATUS_RES_SD";
const t_Chaine SD_FRW_ApplicationServeurDonnees::ADR_SD_INPUT             = "SD_INPUT";
const t_Chaine SD_FRW_ApplicationServeurDonnees::ADR_WOM_INPUT            = "WOM_INPUT";
const t_Chaine SD_FRW_ApplicationServeurDonnees::EVT_FONCTION_APPLICATION = "SD";

#ifdef OPS_DEBUG
const int      SD_FRW_ApplicationServeurDonnees::TIMEOUT_LANCEMENT_THREAD = 1;
const int      SD_FRW_ApplicationServeurDonnees::TIMEOUT_ARRET            = 3;
#else
const int      SD_FRW_ApplicationServeurDonnees::TIMEOUT_LANCEMENT_THREAD = 1;
const int      SD_FRW_ApplicationServeurDonnees::TIMEOUT_ARRET            = 5;
#endif

const double   SD_FRW_ApplicationServeurDonnees::PERIODE_MIN_ENVOI_CPU    = 10.;


//=====================================================================
//                        DONNEES STATIQUES
//=====================================================================

// Serveur de Donnees
static SD_FRW_ApplicationServeurDonnees S_Serveur;

// Acces aux services PGE
static CMN__PGEServices                 S_PGEServices(PROCESS_SD);

// Handlers pour la reception d'un signal
static void SD_FRW_Exit(int signal);
static void SD_FRW_Terminer(int signal);

// Handler pour la reception d'une exception non traitee
static void SD_FRW_GestionnaireException();


//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD_FRW_ApplicationServeurDonnees
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_FRW_ApplicationServeurDonnees::SD_FRW_ApplicationServeurDonnees() 
  : m_ObservateurTerminaisonTache(0), m_Echeancier(0), 
    m_Continuer(CMN__VRAI), m_NombreDeThreads(NB_THREADS), 
    m_GestionnaireThread(0), m_ServeurEvenements(0), 
    m_GestionnaireDonnees(0), m_ChronoTache(1), m_ChronoEvenement(1), 
    m_Timeout(DEFAULT_TIMEOUT), 
    m_PeriodeDeScrutation(DEFAULT_SCRUTATION), m_TacheSessionMgr(0), 
    m_thr_nb(0),
    // FT-2765/FT-2766
    m_thr_id(static_cast<pthread_t>(-1))
    // ...FT-2765/FT-2766
{
  //@ Initialiser les proprietes de la classe a des valeurs par defaut
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_FRW_ApplicationServeurDonnees
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_FRW_ApplicationServeurDonnees::~SD_FRW_ApplicationServeurDonnees()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Initialiser
//@
//@ Description : Initialise les ressources de l'application
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ApplicationServeurDonnees::Initialiser()
{
  //@ En captant les exceptions
  try 
    { 
      //@ Inhiber les signaux correspondant aux commandes
      GererLesSignaux(CMN__FAUX);
      
      //@ Creer le mutex d'acces aux ressources de l'application
      CreerLeVerrouDeLapplication();
      
      //@ Charger le contexte de l'application
      ChargerContexte();
       
      //@ Creer le serveur d'evenements
      CreerLeServeurEvenements();
 
      //@ Creer le gestionnaire de donnees
      CreerLeGestionnaireDeDonnees();

      //@ Creer le gestionnaire de threads
      m_GestionnaireThread = new 
        SD_FRW_GestionnaireThread(m_NombreDeThreads);
      m_GestionnaireThread->Application(this);

      //@ Laisser un temps de latence pour que les threads aient
      //@ le temps de se creer
      sleep(TIMEOUT_LANCEMENT_THREAD);
      
      //@ Creer un observateur de terminaison de tache
      m_ObservateurTerminaisonTache = 
        new SD_EVT_ObservateurTerminaisonTache(m_GestionnaireThread->
                                               SujetTerminaisonTache());
      m_ObservateurTerminaisonTache->Application(this);

      //@ Creer la session
      CreerLaSession();
     
      //@ Initialiser le serveur d'evenements
      m_ServeurEvenements->Initialiser();

      //@ Creer le gestionnaire de session et l'associer a un thread
      CreerLeGestionnaireDeSession();
      
      //@ Creer l'echeancier
      m_Echeancier = new SD_FRW_Echeancier();
      m_Echeancier->Application(this);
      m_Echeancier->Initialiser(m_PeriodeDeScrutation);
  
      //@ Activer les signaux correspondant aux commandes
      GererLesSignaux(CMN__VRAI);

      //@ Initialiser la ressource CPU
      m_RessourceCPU.GetResourceValue();
      m_DateEnvoiCPU.DateCourante();
      m_DateEnvoiCPU = m_DateEnvoiCPU - PERIODE_MIN_ENVOI_CPU - 1;

      //@ Initialiser les evenements timers utilises
      //@ par le SD
      InitialiserEvtTimer();

#ifdef INVESTIGATION
      //@ INVESTIGATION: Envoyer un message de type JDB MSG_SD_START
      EnvoyerMessageJDB(SD__Message::MSG_SD_START);
#endif

      //@ Signaler que l'application est disponible
      SignalerFinInit(SD__Init::SD_INIT_OK);
   }
  
  //@ Exploiter les exceptions
  catch (ERR__Exception &exception) 
    {
      //@ Signaler que l'application n'est pas disponible
      SignalerFinInit(SD__Init::SD_INIT_NOK);

      //@ Re-emission de l'exception
      throw(exception);
    }  
}


//@ ------------------------------------------------------------------*/
//@ Methode : Demarrer
//@
//@ Description : Active le fonctionnement nominal du serveur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ApplicationServeurDonnees::Demarrer()
{
  //@ TANT QUE l'on peut continuer FAIRE
  while (Continuer() == CMN__VRAI)
    {
      //@ Verrouiller l'acces aux ressources de l'application
      Verrouiller();
 
      //@ SI l'echeancier est actif
      if (Echeancier()->EstActif() == CMN__VRAI) 
        {
          int            nbTaches       = 0;
          int            i              = 0;
          long           chrono         = 0;
          SD_TAC_Tache*  tacheNormale   = NULL;
          t_CMN__Booleen tachesArretees = CMN__FAUX;
          
          
          //@ Deverrouiller l'application
          //@ Il faut liberer le plus tot possible pour que les 
          //@ taches en cours puissent acceder aux ressources et se 
          //@ terminer correctement sinon LancerTache ne trouve jamais
          //@ un thread libre et il peut se produire un interbloquage
          Deverrouiller();
          
          //@ Compter les taches a faire
          nbTaches = Echeancier()->CompterLesTachesAFaire();

#ifdef OPS_DEBUG
          cout << "SD: nb taches a faire: " << nbTaches << endl;
#endif

          //@ POUR chaque tache a faire FAIRE
          i = 0;
          while ((i< nbTaches) && (tachesArretees == CMN__FAUX))
            {
              //@ En captant les exceptions
              try
                {
                  //@ Recuperer la tache a faire
                  tacheNormale = Echeancier()->TacheAFaire();
                }

              //@ Traiter les exceptions
              catch (ERR__Exception& exception)
                {
                  //@ Il n'y a plus de taches a faire si elles 
                  //@ ont ete deprogrammees de l'echeancier
                  tachesArretees = CMN__VRAI;
                }

              //@ SI les taches n'ont pas ete deprogrammees
              if (tachesArretees == CMN__FAUX)
                {
                  //@ Allouer un chrono a la tache
                  chrono = AllouerChronoTache();
                 
                  //@ Affecter le chrono a la tache
                  tacheNormale->Id(chrono);
                 
                  //@ Affecter la tache a un thread
                  while (!m_GestionnaireThread->
                         LancerTache(tacheNormale))
                    {
                      usleep(m_Timeout);
                    }
                 
                  //@ Passer a la tache suivante
                  i++;
                }
              //@ FIN SI
            }
          //@ FIN POUR
        }
      //@ SINON
      else
        {
          //@ Deverrouiller l'application
          Deverrouiller();
        }
      //@ FIN SI

      //@ Temporiser
      Echeancier()->Temporiser();
    }
  //@ FIN TANT QUE

  //@ On laisse refroidir...
  sleep(TIMEOUT_ARRET);
}


//@ ------------------------------------------------------------------*/
//@ Methode : Arreter
//@
//@ Description : Demande d'arret du serveur de donnees
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ApplicationServeurDonnees::Arreter()
{
  //@ Positionner l'indicateur de continuation a faux
  m_Continuer = CMN__FAUX;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Terminer
//@
//@ Description : Libere les ressources de l'application
//@
//@ Parametres :
//@               (ENTREE) arretPropre : si false on n'execute pas
//@					 toute la procedure d'arret
//@					 = true par defaut
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ApplicationServeurDonnees::Terminer(MP__ArretProcess::
                                                e_EtatFinProcessus etat,
						bool arretPropre)
{
  MP__ArretProcess msgArret;
  MSG__Message     message;


  //@ Construction d'un message de type ARRET PROCESSUS
  msgArret.SetProcessus(PROCESS_SD);
  msgArret.SetEtatFin(etat);
  msgArret.Serialiser(message);

  //@ Envoyer le message directement
  m_ConnexionServeur->PublierMessage(message);

  //@ SI la fin du process est normale
  if (etat == MP__ArretProcess::FIN_OK)
    {
      //@ Envoyer un message de type JDB MSG_SD_STOP_OK
      EnvoyerMessageJDB(SD__Message::MSG_SD_STOP_OK);
    }
  //@ SINON le process s'arrete sur une erreur
  else
    {
      //@ Envoyer un message de type JDB MSG_SD_STOP_ON_ERROR
      EnvoyerMessageJDB(SD__Message::MSG_SD_STOP_ON_ERROR);
    }
  //@ FIN SI la fin du process est normale

  //@ Supprimer le timer
  SupprimerEvtTimer();

  //@ si pas arret propre demande on sort la
  if(!arretPropre)
     return;
 
  //@ Liberer les ressources allouees par l'application
  if (m_ObservateurTerminaisonTache != NULL) 
    {
      delete m_ObservateurTerminaisonTache;
      m_ObservateurTerminaisonTache = NULL;
    }

  if (m_GestionnaireThread != NULL) 
    {
      delete m_GestionnaireThread;
      m_GestionnaireThread = NULL;
    }

  if (m_Echeancier != NULL)  
    {
      delete m_Echeancier;
      m_Echeancier = NULL;
    }

  if (m_GestionnaireDonnees != NULL)  
    {
      delete m_GestionnaireDonnees;
      m_GestionnaireDonnees = NULL;
    }

  if (m_ServeurEvenements != NULL)  
    {
      delete m_ServeurEvenements;
      m_ServeurEvenements = NULL;
    }
  
  if (m_ConnexionServeur != NULL)  
    {
      delete m_ConnexionServeur;
      m_ConnexionServeur = NULL;
    }
  
  //@ Liberer le verrou d'acces aux ressources de l'application
  //@ Supprimer le mutex POSIX
  pthread_mutex_destroy(&m_mutex_Ressources);
}


//@ ------------------------------------------------------------------*/
//@ Methode : Verrouiller
//@
//@ Description : Verrouille et garantit l'acces exclusif aux 
//@               ressources de l'application pour le demandeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ApplicationServeurDonnees::Verrouiller()
{
  //@ Verrouille le mutex d'acces aux ressources de l'application
  
  //@ Si c'est le meme thread que celui qui possede le mutex 
  //@ qui verrouille alors on incremente un compteur
  //@ Les autres threads ne passent que lorsque le compteur est
  //@ a zero  
 
  if ( (m_thr_nb == 0) || (m_thr_id != pthread_self())) 
    {
      pthread_mutex_lock(&m_mutex_Ressources);
      m_thr_id = pthread_self();
    }

  m_thr_nb++;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Deverrouiller
//@
//@ Description : Deverrouille l'acces exclusif aux ressources
//@               de l'application pour le demandeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ApplicationServeurDonnees::Deverrouiller()
{
  //@ Deverrouille le mutex d'acces aux ressources de l'application

  //@ On ne libere le verrou que lorsque le nombre
  //@ de verrous poses par le meme thread est egal a zero

  m_thr_nb--;

  if (m_thr_nb == 0) 
    {
	  // FT-2765
      m_thr_id = static_cast<pthread_t>(-1);
      // ...FT-2765
      pthread_mutex_unlock(&m_mutex_Ressources);
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : AllouerChronoTache
//@
//@ Description : Accesseur a l'allocateur de numero chrono de tache
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Numero chrono de tache
//@
//@ ------------------------------------------------------------------*/
long SD_FRW_ApplicationServeurDonnees::AllouerChronoTache()
{
  long chronoTache = 0;


  //@ Verrouiller l'acces aux ressources de l'application
  Verrouiller();

  //@ Incrementer le numero chrono de tache
  chronoTache = m_ChronoTache;
  m_ChronoTache++;

  //@ Deverrouiller l'acces aux ressources de l'application
  Deverrouiller();


  //@ Retourner le chrono alloue
  return (chronoTache);
}


//@ ------------------------------------------------------------------*/
//@ Methode : AllouerChronoEvenement
//@
//@ Description : Accesseur a l'allocateur de numero chrono 
//@               d'evenement
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Numero chrono de l'evenement
//@
//@ ------------------------------------------------------------------*/
long SD_FRW_ApplicationServeurDonnees::AllouerChronoEvenement()
{
  long chronoEvenement = 0;


  //@ Verrouiller l'acces aux ressources de l'application
  Verrouiller();

  //@ Incrementer le numero chrono d'evenement
  chronoEvenement = m_ChronoEvenement;
  m_ChronoEvenement++;

  //@ Deverrouiller l'acces aux ressources de l'application
  Deverrouiller();


  //@ Retourner le chrono alloue
  return (chronoEvenement);
}


//@ ------------------------------------------------------------------*/
//@ Methode : EnvoyerMessageFinTraitement
//@
//@ Description : Envoie un message de fin de traitement
//@
//@ Parametres :
//@               (ENTREE) traitement : Traitement a envoyer
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ApplicationServeurDonnees::
EnvoyerMessageFinTraitement(SD_DON_Traitement& traitement)
{
  MSG__Message message;

  message = traitement.Serialiser();
  message.SetAdresse(CFG__Configuration::GetAdresse(ADR_WOM_INPUT));
  EnvoyerMessage(message);
}


//@ ------------------------------------------------------------------*/
//@ Methode : DiffuserMessage
//@
//@ Description : Diffuse un message pour traitement
//@
//@ Parametres :
//@               (ENTREE) message : Message a envoyer
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ApplicationServeurDonnees::DiffuserMessage(const MSG__Message& 
                                                       message)
{
  t_CMN__Statut statut = CMN__STATUT_NOK;


  //@ Verifier si l'application sait le traiter directement
  statut = AnalyserMessage(message);

  //@ SI l'application ne sait pas traiter directement ALORS
  if (statut == CMN__STATUT_NOK) 
    {
      //@ Verifier si le serveur d'evenement sait le traiter
      statut = Evenements()->AnalyserMessage(message);

      //@ SI le message n'a pas pu etre traite ALORS
      if (statut == CMN__STATUT_NOK) 
        {
          ERR__Exception exception;

          //@ Par defaut, on envoie une exception UNEXPECTED_MSG
          exception.SetErrorCode(ERR__Exception::UNEXPECTED_MSG);
          exception.SetParametre(t_Chaine(message.GetType()));
          THROW(exception);
        }
      //@ FIN SI
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : CompterLesTachesEnCours
//@
//@ Description : Compte les taches en cours
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Nombre de taches en cours
//@
//@ ------------------------------------------------------------------*/
int SD_FRW_ApplicationServeurDonnees::CompterLesTachesEnCours()
{ 
  int tachesEnCours = 0;


  //@ Verrouiller l'acces aux ressources de l'application
  Verrouiller();
  
  //@ Demander au gestionnaire de threads de compter les taches
  //@ affectees aux threads
  tachesEnCours = m_GestionnaireThread->CompterLesTachesEnCours();

  //@ Verrouiller l'acces aux ressources de l'application
  Deverrouiller();


  //@ Retourner le nombre de taches en cours
  return tachesEnCours;
}


//@ ------------------------------------------------------------------*/
//@ Methode : EnvoyerMessageJDB
//@
//@ Description : Permet d'envoyer un message JDB sur le bus MSG
//@
//@ Parametres :
//@               (ENTREE) msgId : Type de message JDB pour le SD
//@               (ENTREE) prm1 : Premier parametre pour le journal
//@                               de bord
//@               (ENTREE) prm2 : Deuxieme parametre pour le journal
//@                               de bord
//@               (ENTREE) prm3 : Troiseme parametre pour le journal
//@                               de bord
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void
SD_FRW_ApplicationServeurDonnees::EnvoyerMessageJDB(SD__Message::
                                                    e_TypeMessageJDB
                                                    msgId, 
                                                    t_Chaine prm1, 
                                                    t_Chaine prm2, 
                                                    t_Chaine prm3)
{
  CMN_DAT_Evenement evenement;


  //@ Creer un evenement

  //@  - chrono
  evenement.Identifiant(AllouerChronoEvenement());

  //@  - type = msgId
  evenement.TypeEvenement(msgId);

  //@  - fonction = EVT_FONCTION_APPLICATION
  evenement.Fonction(EVT_FONCTION_APPLICATION);

  //@  - libelle
  evenement.Libelle("");

  //@  - renseignement des parametres pour le journal de bord
  if (! prm1.empty())
    {
      evenement.Prm1(prm1);
      if (! prm2.empty())
        {
          evenement.Prm2(prm2);
          if (! prm3.empty())
            {
              evenement.Prm3(prm3);
            }
        }
    }

  //@ Envoyer l'evenement au serveur d'evenements       
  Evenements()->JournaliserEvenement(evenement);
}


//@ ------------------------------------------------------------------*/
//@ Methode : EnvoyerTM
//@
//@ Description : Permet d'envoyer une telemesure
//@
//@ Parametres :
//@               (ENTREE) status : Statut telemesure
//@               (ENTREE) state : Etat telemesure
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ApplicationServeurDonnees::EnvoyerTM(const MP__HKTMStatus::
                                                 e_ListeStatus status, 
                                                 const t_Chaine state)
{
  //@ En captant les exceptions
  try 
    { 
      MP__HKTMStatus telemesure(status);
      MSG__Message   message;

#ifdef OPS_DEBUG
      cout << "Telemesure envoyee: " << endl;
      cout << "\ttype: " << status << endl;
      cout << "\tetat: " << state << endl;
#endif

      //@ Construction du message HKTM Status
      telemesure.SetAvailability(MP__HKTMStatus::AVAILABLE);
      telemesure.SetValeurChaine(state);
      telemesure.Serialiser(message);

      //@ Publication du message
      EnvoyerMessage(message); 
    }
  
  //@ Exploiter les exceptions
  catch (ERR__Exception &exception) 
    {
      ManageException(exception);
    }  
}


//@ ------------------------------------------------------------------*/
//@ Methode : main
//@
//@ Description : Programme principal du serveur
//@
//@ Parametres :
//@               (ENTREE) argc: Nombre d'arguments   
//@               (ENTREE) argv: Liste des arguments   
//@
//@ Valeur de retour :
//@               Code retour du process
//@
//@ ------------------------------------------------------------------*/
int main (int argc, char *argv[])
{
  t_CMN__PGEStatus status;


  //@ Installation d'un gestionnaire d'exception
  set_terminate(&SD_FRW_GestionnaireException);
  set_unexpected(&SD_FRW_GestionnaireException);

  //@ Traitement des options d'execution
  CMN__Version_Trace(argc, argv);
  
  //@ Declaration du demarrage au MCS
  status = S_PGEServices.MLAInitialisation();  
   
  //@ SI le status de l'initialisation du MCS est OK
  if (status == CMN__STATUT_OK) 
    {
      //@ Capture des exceptions
      try 
        {
          //@ Installation du module de gestion de la securite
//FA159
//          CMN__Securite_Init(SD_FRW_Exit, SD_FRW_Terminer);
          CMN__Securite_Init(SD_FRW_Terminer,SD_FRW_Exit);
          
          //@ Initialisation du serveur de donnees
          S_Serveur.Initialiser();

#ifdef OPS_DEBUG
          cout << endl << "INFO: Serveur de donnees lance..." << endl;
#endif
          
          //@ Demarrer le fonctionnement operationnel
          S_Serveur.Demarrer();

          //@ Terminer proprement l'application si le fonctionnement
          //@ operationnel est termine

          S_Serveur.Terminer(MP__ArretProcess::FIN_OK);

#ifdef OPS_DEBUG
          cout << endl << "INFO: Serveur de donnees arrete." << endl;
#endif
         
          //@ On signale au MCS la fin du processus (fait exit)
          S_PGEServices.MLATerminate();  
        } 

      //@ Exploitation des exceptions  
      catch (ERR__Exception &exception) 
        {
          //@ Impression des exceptions
          cout << "Exception (SD): " << endl;
          cout << "\tFile : " << exception.GetFile() << endl;
          cout << "\tLine : " << exception.GetLine() << endl;
          cout << "\tError: " << exception.GetErrorCode() << endl;
          cout << "\tErrno: " << exception.GetErrno() << endl;
          for (int i=0; i<exception.GetNbParametre(); i++)
            {
              cout << "\tInfo : " << exception.GetParametre(i) << endl;
            }
            
          //@ On essaye de terminer
          S_Serveur.Terminer(MP__ArretProcess::FIN_NOK);

#ifdef OPS_DEBUG
          cout << endl 
               << "INFO: Serveur de donnees arrete sur exception." 
               << endl;
#endif
           
          //@ On signale au MCS la fin du processus (fait exit)
          S_PGEServices.MLATerminate();  
        }

      //@ Exploitation des exceptions  
      catch (exception &exception) 
        {
          //@ Impression des exceptions
          cout << "Exception: " << endl;
          cout << "\tInfo : " << exception.what() << endl;
            
          //@ On essaye de terminer
          S_Serveur.Terminer(MP__ArretProcess::FIN_NOK, false);

#ifdef OPS_DEBUG
          cout << endl 
               << "INFO: Serveur de donnees arrete sur exception." 
               << endl;
#endif
           
          //@ On signale au MCS la fin du processus (fait exit)
          S_PGEServices.MLATerminate();  
        }
      //@ Exploitation des exceptions  
      catch (...) 
        {
          //@ Impression des exceptions
          cout << "Exception inconnue" << endl;

          //@ On essaye de terminer
          S_Serveur.Terminer(MP__ArretProcess::FIN_NOK, false);

#ifdef OPS_DEBUG
          cout << endl 
               << "INFO: Serveur de donnees arrete sur exception." 
               << endl;
#endif
           
          //@ On signale au MCS la fin du processus (fait exit)
          S_PGEServices.MLATerminate();  
        }
    } 
  //@ FIN SI status ok


  //@ Erreur initialisation MCS exit EXIT_FAILURE
  return EXIT_FAILURE;
}


//======================================================================
//                                      METHODES PRIVEES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : ManageException
//@
//@ Description : Permet de traiter les differents types d'exception
//@
//@ Parametres :
//@               (ENTREE) exception : Exception recue
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ApplicationServeurDonnees::ManageException(ERR__Exception 
                                                       exception)
{
#ifdef OPS_DEBUG
  // Impression de l'exception
  cout << "Exception (SD::ManageException): " << endl;
  cout << "\tFile : " << exception.GetFile() << endl;
  cout << "\tLine : " << exception.GetLine() << endl;
  cout << "\tError: " << exception.GetErrorCode() << endl;
  cout << "\tErrno: " << exception.GetErrno() << endl;
  for (int i=0; i<exception.GetNbParametre(); i++)
    {
      cout << "\tInfo : " << exception.GetParametre(i) << endl;
    }
#endif

  //@ Traiter les exceptions
  //@ SELON le type d'erreur
  switch (exception.GetErrorCode())
    {
    //@ Si c'est une erreur dans un type de message       
    case ERR__Exception::UNEXPECTED_MSG :
      //@ Envoyer un message de type JDB UNEXPECTED_MSG
      EnvoyerMessageJDB(SD__Message::MSG_UNEXPECTED_MSG, 
                        exception.GetFile(), 
                        exception.GetLine(), 
                        exception.GetParametre(0));
      break;
      
    //@ Si c'est une erreur dans un type logique de message       
    case ERR__Exception::UNEXPECTED_TYPE :
      //@ Envoyer un message de type JDB UNEXPECTED_TYPE
      EnvoyerMessageJDB(SD__Message::MSG_UNEXPECTED_TYP, 
                        exception.GetFile(), 
                        exception.GetLine(), 
                        exception.GetParametre(0));
      break;

    //@ Si c'est un lancement de traitement refuse
    case ERR__Exception::SD_TMT_START_REFUSED :
      //@ Envoyer un message de type JDB MSG_SD_TMT_START_REFUSED
      EnvoyerMessageJDB(SD__Message::MSG_SD_TMT_START_REFUSED, 
                        exception.GetParametre(0));
      break;

    //@ Si c'est un arret de traitement refuse
    case ERR__Exception::SD_TMT_STOP_REFUSED :
      //@ Envoyer un message de type JDB MSG_SD_TMT_STOP_REFUSED
      EnvoyerMessageJDB(SD__Message::MSG_SD_TMT_STOP_REFUSED);
      break;

    //@ Si c'est une erreur de traitement non trouve
    case ERR__Exception::SD_TMT_NOT_FOUND :
      //@ Envoyer un message de type JDB MSG_SD_TMT_NOT_FOUND
      EnvoyerMessageJDB(SD__Message::MSG_SD_TMT_NOT_FOUND, 
                        exception.GetParametre(0));
      break;

    //@ Si c'est une erreur de tache non trouvee
    case ERR__Exception::SD_TAC_NOT_FOUND :
      //@ Envoyer un message de type JDB MSG_SD_TAC_NOT_FOUND
      EnvoyerMessageJDB(SD__Message::MSG_SD_TAC_NOT_FOUND, 
                        exception.GetParametre(0));
      break;

    //@ Si c'est une erreur d'action non trouvee
    case ERR__Exception::SD_ACT_NOT_FOUND :
      //@ Envoyer un message de type JDB MSG_SD_ACT_NOT_FOUND
      EnvoyerMessageJDB(SD__Message::MSG_SD_ACT_NOT_FOUND, 
                        exception.GetParametre(0));
      break;

    //@ Si c'est une erreur dans l'echeancier
    case ERR__Exception::SD_ECH_TAC_NOT_FOUND :
      //@ Envoyer un message de type JDB MSG_SD_ECH_TAC_NOT_FOUND
      EnvoyerMessageJDB(SD__Message::MSG_SD_ECH_TAC_NOT_FOUND);
      break;

    //@ Si c'est une erreur de variable d'environnement
    case ERR__Exception::SD_ENV_UNDEFINED :
      //@ Envoyer un message de type JDB MSG_SD_ENV_UNDEFINED
      EnvoyerMessageJDB(SD__Message::MSG_SD_ENV_UNDEFINED, 
                        exception.GetParametre(0));
      break;

    //@ Si c'est une erreur de lecture d'un fichier
    case ERR__Exception::SD_FILE_OPEN_ERROR :
      //@ Envoyer un message de type JDB MSG_SD_FILE_OPEN_ERROR
      EnvoyerMessageJDB(SD__Message::MSG_SD_FILE_OPEN_ERROR, 
                        exception.GetParametre(0));
      break;

    //@ Si c'est une erreur d'ecriture d'un fichier
    case ERR__Exception::SD_FILE_WRITE_ERROR :
      //@ Envoyer un message de type JDB MSG_SD_FILE_WRITE_ERROR
      EnvoyerMessageJDB(SD__Message::MSG_SD_FILE_WRITE_ERROR, 
                        exception.GetParametre(0));
      break;

    //@ Defaut:  
    default :
      //@ Rien a faire
      break;
    }
  //@ FIN SELON
}


//@ ------------------------------------------------------------------*/
//@ Methode : AnalyserMessage
//@
//@ Description : Teste si le message est pris en compte directement
//@               par l'application
//@                 Si oui, le message est traite
//@
//@ Parametres :
//@               (ENTREE) message : Message a traiter
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK si le message est destine a 
//@                    l'application
//@               CMN__STATUT_NOK si le message n'est pas destine a
//@                    l'application
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut 
SD_FRW_ApplicationServeurDonnees::AnalyserMessage(const MSG__Message& 
                                                  message)
{
  // Exception generee
  ERR__Exception exception;
  
  // Valeur de retour de la methode
  t_CMN__Statut  statut = CMN__STATUT_OK;
 
 
  //@ En captant les exceptions
  try
    {
      //@ SELON le type du message
      switch (message.GetType())
        {
        //@ Si c'est un evenement TIMER
        case MSG__Message::EVT_TIMER :
          {
            TES__Evenement evenementTimer;
            
    
            //@ Le message a ete pris en compte
            statut = CMN__STATUT_OK;

            //@ On construit un objet timer
            evenementTimer.Deserialiser(message);
            
            //@ Examem de l'evenements timer
            //@ SI le type logique est un evenement EVT_HKTM_STATUS
            if (evenementTimer.GetTypeLogique() == 
                MP__Message::EVT_HKTM_STATUS) 
              {
                t_Date dateCourante;
                dateCourante.DateCourante();

                if ((dateCourante - m_DateEnvoiCPU) > 
                    PERIODE_MIN_ENVOI_CPU)
                  {

#ifdef OPS_DEBUG
                    cout << "Envoi HKTM CPU a : " 
                         << dateCourante.Chaine()<< endl;
#endif

                    //@ Envoyer les status
                    EnvoyerTM(MP__HKTMStatus::CPUusage, RecupererCPU());
                    m_DateEnvoiCPU = dateCourante;
                  }
              }
            //@ SINON
            else
              {
                //@ Emission d'une exception UNEXPECTED_MSG
                exception.SetErrorCode(ERR__Exception::UNEXPECTED_MSG);
                exception.SetParametre(t_Chaine(message.GetType()));
                THROW(exception);
              }
            //@ FIN SI
            break;
          }
    
        //@ Si c'est un evenement objet SD_DON_Traitement
        case SD__Message::EVT_TMT_SD :
          {
            SD_DON_Traitement traitement;


            //@ Le message a ete pris en compte
            statut = CMN__STATUT_OK;
            
            //@ On construit un objet traitement
            traitement.Deserialiser(message);
            
            //@ SI l'echeancier est actif
            if (Echeancier()->EstActif() == CMN__VRAI)
              {
                
#ifdef OPS_DEBUG
                cout << "SD: Traitement refuse" << endl;
#endif
                //@ Emission d'une exception SD_TMT_START_REFUSED
                exception.SetErrorCode(ERR__Exception::
                                       SD_TMT_START_REFUSED);
                exception.SetParametre(traitement.TypeTraitementStr());
                THROW(exception);
              }
            //@ SINON
            else
              {
#ifdef OPS_DEBUG
                cout << "SD: Reception traitement: " << endl;
                traitement.Afficher();
#endif
                
                //@ Programmation du traitement
                Echeancier()->AjouterTraitement(traitement);
                
                //@ L'echeancier devient actif
                Echeancier()->EstActif(CMN__VRAI);
              }
            break;
          }
        
        //@ Si c'est un evenement arret du traitement en cours
        case SD__Message::EVT_ARRET_TMT_SD :
          {
            //@ Le message a ete pris en compte
            statut = CMN__STATUT_OK;
            
            //@ SI l'echeancier n'est pas actif
            if (Echeancier()->EstActif() == CMN__FAUX)
              {
                
#ifdef OPS_DEBUG
                cout << "SD: Arret de traitement refuse" << endl;
#endif

                //@ Emission d'une exception SD_TMT_STOP_REFUSED
                exception.SetErrorCode(ERR__Exception::
                                       SD_TMT_STOP_REFUSED);
                THROW(exception);
              }
            //@ SINON
            else
              {
                //@ Arreter les taches programmees
                //@ SI des taches s'execute
                if (Echeancier()->Deprogrammer() == CMN__VRAI)
                  {
                    //@ Arreter les taches en cours d'execution
                    m_GestionnaireThread->ArreterToutesLesTaches();
                  }
                //@ FIN SI

#ifdef OPS_DEBUG
                cout << "SD: Arret de toutes les taches effectue" 
                     << endl;
#endif
              }
            break;
          }
        
        //@ CAS defaut: le message n'est pas traite
        default :
          {
            statut = CMN__STATUT_NOK;
            break;
          }
        }
      //@ FIN SELON
    }
  
  //@ Exploiter les exceptions
  catch (ERR__Exception& exception)
    {
      ManageException(exception);
    }
  
  return statut;
}


//@ ------------------------------------------------------------------*/
//@ Methode : EnvoyerMessage
//@
//@ Description : Envoie un message via la session
//@
//@ Parametres :
//@               (ENTREE) message : Message a envoyer
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ApplicationServeurDonnees::EnvoyerMessage(MSG__Message& message)
{
  //@ En captant les exceptions
  try 
    { 
      //@ Verrouiller l'acces aux ressources de l'application
      Verrouiller();
      
      //@ Envoyer le message par la session
      Session()->PublierMessage(message);

      //@ Deverrouiller l'acces aux ressources de l'application
      Deverrouiller();
    }

  //@ Exploiter les exceptions
  catch (ERR__Exception &exception) 
    {
      //@ Deverrouiller l'acces aux ressources de l'application
      Deverrouiller();

      //@ Gestion des exceptions
      ManageException(exception);
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : SignalerFinInit
//@
//@ Description : Signale la fin de l'initialisation du SD
//@
//@ Parametres :
//@               (ENTREE) etat : Compte-rendu de l'initialisation 
//@                               du SD
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ApplicationServeurDonnees::SignalerFinInit(SD__Init::e_ListeStatus
                                                       etat)
{
  //@ Construire un objet SD__Init
  SD__Init init;

  //@ Transmettre l'etat a l'objet
  init.Etat(etat);
   
  //@ Serialisation de l'evenement 
  MSG__Message message = init.Serialiser();
  
  //@ Envoyer le message directement
  m_ConnexionServeur->PublierMessage(message);
}


//@ ------------------------------------------------------------------*/
//@ Methode : CreerLeServeurEvenements
//@
//@ Description : Cree le serveur d'evenements
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ApplicationServeurDonnees::CreerLeServeurEvenements()
{
  //@ Creer le serveur d'evenements en lui passant une reference sur
  //@ l'application
  m_ServeurEvenements = new SD_SRV_ServeurEvenements();
  m_ServeurEvenements->Application(this); 
}


//@ ------------------------------------------------------------------*/
//@ Methode : CreerLeGestionnaireDeDonnees
//@
//@ Description : Cree le gestionnaire de donnees
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ApplicationServeurDonnees::CreerLeGestionnaireDeDonnees()
{
  //@ Creer le gestionnaire de donnees en lui passant une reference 
  //@ sur l'application
  m_GestionnaireDonnees = new SD_DON_GestDonnees(); 
}


//@ ------------------------------------------------------------------*/
//@ Methode : CreerLaSession
//@
//@ Description : Cree la session qui assure l'interface externe du
//@               serveur de donnees
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ApplicationServeurDonnees::CreerLaSession()
{
  //@ Lecture du parametrage (nom Host et numero de service)
  t_Chaine host    = CFG__Configuration::GetAdresse(MSG_SERVER_HOST);
  t_Chaine service = CFG__Configuration::GetAdresse(MSG_SERVER_PORT);

  //@ Creer la connexion serveur en appelant le gestionnaire 
  //@ d'activite
  m_ConnexionServeur = new MSG__Connexion();
  m_ConnexionServeur->GererVerrou();
  
  //@ Ouvrir la connexion du type cliente
  m_ConnexionServeur->Ouvrir((char*) host.c_str(), 
                             (char*) service.c_str(), 
                             MSG__Connexion::CNX_CLIENTE);

  //@ Positionne le timeout sur la base de temps                     
  m_ConnexionServeur->SetTimeout(CFG__Configuration::
                                 GetNumerique(TIMEOUT_SD));
                                
  //@ S'abonner aux adresses du processus 
  MSG__Abonnement abonnement;
  MSG__Message message;
  abonnement.SetListeAdresse(CFG__Configuration::
                             GetListeAdresse(LISTE_SD));
  abonnement.SetAction(MSG__Abonnement::ABONNER);
  abonnement.Serialiser(message);
  m_ConnexionServeur->PublierMessage(message);
}


//@ ------------------------------------------------------------------*/
//@ Methode : CreerLeGestionnaireDeSession
//@
//@ Description : Cree le gestionnaire de session qui prend en charge
//@               le traitement des requetes et des reponses qui
//@               transitent par la session
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ApplicationServeurDonnees::CreerLeGestionnaireDeSession()
{
  //@ Creer le gestionnaire de session
  m_TacheSessionMgr = new SD_TAC_GestionnaireDeSession();
  m_TacheSessionMgr->Id(0);
  
  //@ Affecter un thread de traitement au gestionnaire de session
  m_GestionnaireThread->LancerTache(m_TacheSessionMgr);
}


//@ ------------------------------------------------------------------*/
//@ Methode : CreerLeVerrouDeLapplication
//@
//@ Description : Cree le verrou d'acces aux ressources de 
//@               l'application
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ApplicationServeurDonnees::CreerLeVerrouDeLapplication()
{
  //@ Creer le mutex POSIX correspondant
  pthread_mutex_init(&m_mutex_Ressources, NULL);
}


//@ ------------------------------------------------------------------*/
//@ Methode : ChargerContexte
//@
//@ Description : Charge le contexte de l'application
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ApplicationServeurDonnees::ChargerContexte()
{
  ERR__Exception     exception;
  t_Chaine           fichierConfiguration;
  t_Chaine           valeur;
  CMN__Configuration configuration;


  //@ Ouvrir le fichier de configuration de l'application
  fichierConfiguration = CFG__Configuration::GetRepertoire(CONF_DIR);

  //@ SI la variable d'environnement REP_CONF_DIR n'est pas definie
  if (fichierConfiguration.empty())
    {
      //@ Emission d'une exception SD_ENV_UNDEFINED
      exception.SetErrorCode(ERR__Exception::SD_ENV_UNDEFINED);
      exception.SetParametre(CONF_DIR);
      THROW(exception);
    }
  //@ FIN SI

  fichierConfiguration += CMN__SEP_REPERTOIRE;
  fichierConfiguration += FICHIER_CONFIGURATION_SD;
  
  //@ Verrouiller l'acces aux ressources de l'application
  Verrouiller();
  
  //@ Charger le fichier de configuration du serveur de donnees
  //@ SI le chargement a ete incorrect
  if (configuration.ChargerFichier(fichierConfiguration) != 
      CMN__STATUT_OK)
    {     
      //@ Deverrouiller l'acces aux ressources de l'application
      Deverrouiller();
      
      //@ Emission d'une exception SD_FILE_OPEN_ERROR
      exception.SetErrorCode(ERR__Exception::SD_FILE_OPEN_ERROR);
      exception.SetParametre(FICHIER_CONFIGURATION_SD);
      THROW(exception);
    }
  //@ FIN SI
  
  //@ Deverrouiller l'acces aux ressources de l'application
  Deverrouiller();
  
  //@ Recuperer le nombre de threads a lancer
  //@ Valeur par defaut = NB_THREADS
  //@ Ajout de 1 thread pour le gestionnaire de session
  if (configuration.LireValeur(BLOC_THREAD, VAL_NB_THREAD, valeur)
      == CMN__STATUT_OK)
    {
      m_NombreDeThreads = atoi(valeur.c_str()) + 1;
    } 
  else 
    {
      cerr << endl << "Bad number of threads !!" << endl;
    }

  if (m_NombreDeThreads == 1)
    {
      m_NombreDeThreads = NB_THREADS;
    }
  

  //@ Recuperer le timeout par defaut de temporisation lors du
  //@ lancement d'une tache
  if (configuration.LireValeur(BLOC_ETAT, VAL_TIMEOUT, valeur)
      == CMN__STATUT_OK)
    {
      m_Timeout = atoi(valeur.c_str());
    }

  //@ Recuperer le timeout par defaut le delai entre deux analyses
  //@ de l'echeancier
  if (configuration.LireValeur(BLOC_ETAT, VAL_SCRUTATION, valeur)
      == CMN__STATUT_OK)
    {
      m_PeriodeDeScrutation = atoi(valeur.c_str());
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : InitialiserEvtTimer
//@
//@ Description : Permet d'initialiser les evenements timers utilises:
//@                 - evenement timer qui cadence la scrutation des
//@                   statuts de ressource
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ApplicationServeurDonnees::InitialiserEvtTimer()
{
  TES__Evenement evenement;
  CMN__Date      date;
  t_DateNum      periode;
  t_DateNum      tempsCourant;

  
  //@ Recuperer la periode de scrutation
  periode = CFG__Configuration::GetNumerique(NOM_EVT_TIMER);
  date.DateCourante();
  tempsCourant = date.ConvertirDateEnTemps();

  //@ Definition d'un evenement timer de type ADD
  evenement.SetAction(TES__Evenement::ACTION_ADD);
  evenement.SetPeriode((int)periode);
  evenement.SetStartDate(tempsCourant);
  evenement.SetNomLogique(NOM_EVT_TIMER);
  evenement.SetTypeLogique(MP__Message::EVT_HKTM_STATUS);
  evenement.SetAdresse(CFG__Configuration::GetAdresse(ADR_SD_INPUT));
  
  //@ Serialisation de l'evemenent timer
  MSG__Message message;
  message = evenement.Serialiser();
  
  //@ Envoyer le message
  EnvoyerMessage(message);
}


//@ ------------------------------------------------------------------*/
//@ Methode : SupprimerEvtTimer
//@
//@ Description : Permet de supprimer les evenements timers utilises:
//@                 - evenement timer qui cadence la scrutation des
//@                   statuts de ressource
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ApplicationServeurDonnees::SupprimerEvtTimer()
{
  TES__Evenement evenement;
  

  //@ Definition d'un evenement timer de type DELETE
  evenement.SetAction(TES__Evenement::ACTION_DELETE);
  evenement.SetNomLogique(NOM_EVT_TIMER);
  
  //@ Serialisation de l'evemenent timer
  MSG__Message message;
  message = evenement.Serialiser();

  //@ Envoyer le message
  EnvoyerMessage(message);
}


//@ ------------------------------------------------------------------*/
//@ Methode : GererLesSignaux
//@
//@ Description : Permet de definir le mode de gestion des signaux 
//@               pour le Serveur de Donnees
//@
//@ Parametres :
//@               (ENTREE) autoriser : CMN__VRAI si il faut positionner
//@                                    la procedure sur les signaux 
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ApplicationServeurDonnees::GererLesSignaux(t_CMN__Booleen 
                                                       autoriser)   
{
  //@ SI l'autorisation de positionner les signaux a ete donnee
  if (autoriser == CMN__VRAI)
    {
      sigset_t sig_set;
      sigset_t old_set;
      struct sigaction saexit;

      //@ Positionner la procedure sur les signaux
    }
  //@ SINON
  else
    {
      struct sigaction siginfo;
  
      //@ Ignorer les signaux SIGSTOP et SIGCONT
      sigemptyset(&(siginfo.sa_mask));
      sigaddset(&(siginfo.sa_mask), SIGSTOP);
      sigaddset(&(siginfo.sa_mask), SIGCONT);
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : RecupererCPU
//@
//@ Description : Permet de recuperer le pourcentage de la consommation
//@               CPU instantanee pour le Serveur de Donnees
//@
//@ Parametres :
//@               Aucun 
//@
//@ Valeur de retour :
//@               Chaine de caracteres contenant le pourcentage de la
//@               consommation CPU instantanee
//@
//@ ------------------------------------------------------------------*/
t_Chaine SD_FRW_ApplicationServeurDonnees::RecupererCPU()   
{
  //@ Appel a la methode CMN__ResourceMonitor::GetResourceValue()
  m_RessourceCPU.GetResourceValue();

  //@ Appel a la methode CMN__ResourceMonitor::instantaneousCPU()
  int tauxCPU = m_RessourceCPU.instantaneousCPU();

#ifdef OPS_DEBUG
  cout << "Consommation CPU: " << tauxCPU << " %" << endl;
#endif


  //@ Retourner la chaine contenant le pourcentage de la
  //@ consommation CPU instantanee
  return t_Chaine(tauxCPU);
}


//@ ------------------------------------------------------------------*/
//@ Methode : SD_FRW_Exit
//@
//@ Description : Procedure notifiee sur le signal SIGTERM. Arrete le
//@               serveur de donnees
//@
//@ Parametres :
//@               (ENTREE) signal : Numero du signal
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void SD_FRW_Exit(int signal)
{
  //@ Arret du serveur de donnees
  //FA159
  exit (EXIT_FAILURE);
}


//@ ------------------------------------------------------------------*/
//@ Methode : SD_FRW_Terminer
//@
//@ Description : Procedure notifiee sur un signal autre que SIGTERM.
//@               Arrete proprement le serveur de donnees
//@
//@ Parametres :
//@               (ENTREE) signal : Numero du signal
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void SD_FRW_Terminer(int signal)
{
#ifdef OPS_DEBUG
  cerr << "SD: Signal recu de type: " << signal << endl;
#endif

  try
    {
      //@ Terminaison du serveur de donnees
      S_Serveur.Terminer(MP__ArretProcess::FIN_NOK, false);
  
      //@ On signale au MCS la fin du processus (fait exit)
      S_PGEServices.MLATerminate();
    }

  catch (...)
    {
    }

  //@ Arret du serveur de donnees
  //FA159
  exit (EXIT_FAILURE);
}


//@ ------------------------------------------------------------------*/
//@ Methode : SD_FRW_GestionnaireException
//@
//@ Description : Handler pour la reception d'une exception non 
//@               traitee
//@
//@ Parametres :
//@               Aucune
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void SD_FRW_GestionnaireException()
{
#ifdef OPS_DEBUG
  cerr << "SD: Exception non geree ou illegale recue !" << endl;
#endif

  //@ Terminaison du serveur de Work Order
  S_Serveur.Terminer(MP__ArretProcess::FIN_NOK, false);
  
  //@ On signale au MCS la fin du processus (fait exit)
  S_PGEServices.MLATerminate();
//FA159
  exit (EXIT_FAILURE);
}
