// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/FRW/src/SD_FRW_ApplicationServeurDonnees.h,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_FRW
//@ CLASSE : SD_FRW_ApplicationServeurDonnees
//@
//@ Commentaire :
//@    Fichier d'entete de la classe SD_FRW_ApplicationServeurDonnees
//@                                                                     
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 09/08/1999 : SAS
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FAID-SYS-005  22/11/99  SAS  Journalisation du commentaire de reconfiguration
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Adaptation a l'OPS : modification du mecanisme d'interface, ajout
//@                      de fonctionnalites supplementaires et de la
//@                      gestion des exceptions
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_FRW_ApplicationServeurDonnees_h
#define SD_FRW_ApplicationServeurDonnees_h


//
// Fichiers include
//

// STL
#include <iostream>

// Standard
#include <unistd.h>
#include <pthread.h>

// Projet
#include "CMN__Commun.h"
#include "CMN__ResourceMonitor.h"
#include "ERR__Exception.h"
#include "SD__Message.h"
#include "SD__Init.h"
#include "MP__ArretProcess.h"
#include "MP__HKTMStatus.h"
#include "MSG__Connexion.h"
#include "SD_DON_GestDonnees.h"
#include "SD_EVT_ObservateurTerminaisonTache.h"
#include "SD_FRW_GestionnaireThread.h"  
#include "SD_FRW_Echeancier.h"
#include "SD_SRV_ServeurEvenements.h"


//
// Definition des constantes
//
#define FRW_HANDLE_MUTEX pthread_mutex_t
#define NB_THREADS       10


class SD_TAC_GestionnaireDeSession;

class SD_FRW_ApplicationServeurDonnees 
{

    /*================================================================*/
    
 public:
    
    ////////////////////////////////
    // Constructeur
    ////////////////////////////////
    SD_FRW_ApplicationServeurDonnees();
        
    ////////////////////////////////
    // Destructeur
    ////////////////////////////////
    virtual ~SD_FRW_ApplicationServeurDonnees();

    ////////////////////////////////
    // Accesseurs
    ////////////////////////////////
    int                       NombreDeThreads() 
        {return m_NombreDeThreads;};
    int                       Timeout() {return m_Timeout;};
    int                       PeriodeDeScrutation() 
        {return m_PeriodeDeScrutation;};
    int                       Continuer() {return m_Continuer;};
    SD_FRW_Echeancier*        Echeancier() {return m_Echeancier;};
    
    SD_SRV_ServeurEvenements* Evenements() {return m_ServeurEvenements;};
    SD_DON_GestDonnees*       Donnees() {return m_GestionnaireDonnees;};
    MSG__Connexion*           Session() {return m_ConnexionServeur;};  


    ////////////////////////////////
    // Methodes publiques
    ////////////////////////////////
    void Initialiser();
    void Demarrer();
    void Arreter();
    void Terminer(MP__ArretProcess::e_EtatFinProcessus etat, bool arretPropre = true);
    void Verrouiller();
    void Deverrouiller();
    long AllouerChronoTache();
    long AllouerChronoEvenement();
    void EnvoyerMessageFinTraitement(SD_DON_Traitement& traitement);
    void DiffuserMessage(const MSG__Message& message);
    int  CompterLesTachesEnCours();
    void EnvoyerMessageJDB(SD__Message::e_TypeMessageJDB msgId,
                           t_Chaine prm1 = "", 
                           t_Chaine prm2 = "", 
                           t_Chaine prm3 = "");
    void EnvoyerTM(const MP__HKTMStatus::e_ListeStatus status, 
                   const t_Chaine state);

   
    /*================================================================*/

 protected:

    ////////////////////////////////
    // Methodes protegees
    ////////////////////////////////

    ////////////////////////////////
    // Attributs proteges
    ////////////////////////////////

    
    /*================================================================*/

 private:

    ////////////////////////////////
    // Constantes
    ////////////////////////////////

    // Timeout
    static const int      DEFAULT_TIMEOUT;
    static const int      DEFAULT_SCRUTATION;
    static const int      TIMEOUT_LANCEMENT_THREAD;
    static const int      TIMEOUT_ARRET;

    // Fichier de configuration du SD
    static const t_Chaine FICHIER_CONFIGURATION_SD;
    static const t_Chaine BLOC_THREAD;
    static const t_Chaine VAL_NB_THREAD;
    static const t_Chaine BLOC_ETAT;
    static const t_Chaine VAL_TIMEOUT;
    static const t_Chaine VAL_SCRUTATION;
    static const t_Chaine BLOC_SIMU;
    static const t_Chaine VAL_MAX_LIGNE;
    static const t_Chaine VAL_NB_LIGNE;
    static const t_Chaine VAL_NB_COLONNE;

    // Communication
    static const t_Chaine MSG_SERVER_HOST;
    static const t_Chaine MSG_SERVER_PORT;
    static const t_Chaine TIMEOUT_SD;
    static const t_Chaine LISTE_SD;
    static const t_Chaine NOM_EVT_TIMER;
    static const t_Chaine ADR_SD_INPUT;
    static const t_Chaine ADR_WOM_INPUT;
    static const t_Chaine EVT_FONCTION_APPLICATION;
    static const double   PERIODE_MIN_ENVOI_CPU;
  
    ////////////////////////////////
    // Methodes privees
    ////////////////////////////////
    void          ManageException(ERR__Exception exception);
    t_CMN__Statut AnalyserMessage(const MSG__Message& message);
    void          EnvoyerMessage(MSG__Message& message);
    void          SignalerFinInit(SD__Init::e_ListeStatus etat);
    void          CreerLeServeurEvenements();
    void          CreerLeGestionnaireDeDonnees();
    void          CreerLaSession();
    void          CreerLeGestionnaireDeSession();
    void          CreerLeVerrouDeLapplication();
    void          ChargerContexte();
        
    // Gestion des evenements timers
    void          InitialiserEvtTimer();
    void          SupprimerEvtTimer();

    // Gestion des signaux
    void          GererLesSignaux(t_CMN__Booleen autoriser);

    // Divers
    t_Chaine      RecupererCPU();

    ////////////////////////////////
    // Attributs prives
    ////////////////////////////////
    SD_EVT_ObservateurTerminaisonTache* m_ObservateurTerminaisonTache;
    SD_FRW_Echeancier*                  m_Echeancier;
    int                                 m_Continuer;
    int                                 m_NombreDeThreads;
    SD_FRW_GestionnaireThread*          m_GestionnaireThread;
    SD_SRV_ServeurEvenements*           m_ServeurEvenements;
    SD_DON_GestDonnees*                 m_GestionnaireDonnees;
    long                                m_ChronoTache;
    long                                m_ChronoEvenement;
    int                                 m_Timeout;
    int                                 m_PeriodeDeScrutation;
    FRW_HANDLE_MUTEX                    m_mutex_Ressources;
    SD_TAC_GestionnaireDeSession*       m_TacheSessionMgr;
    int                                 m_thr_nb;
    pthread_t                           m_thr_id;
    CMN__ResourceMonitor                m_RessourceCPU;
    t_Date                              m_DateEnvoiCPU;

    // Connexion avec le serveur
    MSG__Connexion*                     m_ConnexionServeur;
};

#endif
