static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/FRW/src/SD_FRW_ContexteThread.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_FRW
//@ CLASSE : SD_FRW_ContexteThread
//@ 
//@ Commentaire :
//@    Ce module gere la classe SD_FRW_ContexteThread qui definit le
//@    contexte d'execution des threads
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 10/08/1999 : SAS
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Adaptation a l'OPS
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-184-CN : 20/09/2006 : C.MERIGUET
//@ Suite passage purify, initialisation de m_Sujet a NULL
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL

// Standard
#include <pthread.h>

// Projet
#include "CMN__Commun.h"
#include "SD_FRW_ContexteThread.h"


//
// Definition des constantes
//
#define MUTEX  // on synchronise a l'aide de mutex et non a l'aide
               // de condition


//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD_FRW_ContexteThread
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
/* FA 184 */
SD_FRW_ContexteThread::SD_FRW_ContexteThread() : 
// FT-2765/FT-2766
m_hthread(static_cast<pthread_t>(NULL)), m_Tache(0),m_Sujet(NULL)
// ...FT-2765
/* ..FA 184 */
{
  //@ Creer la condition de travail
  CreerConditionDeTravail();

  //@ Creer la condition de disponibilite
  CreerConditionDeDisponibilite();
  
  //@ Creer la condition de terminaison
  CreerConditionDeTerminaison();
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_FRW_ContexteThread
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_FRW_ContexteThread::~SD_FRW_ContexteThread()
{
  //@ Signaler une condition de travail
  SignalerConditionDeTravail(CMN__VRAI);
  
  //@ Suivi d'une condition de terminaison
  SignalerConditionDeTerminaison();

  sleep(0);
  
  //@ Attendre la terminaison
  AttendreLaTerminaison();
  
  //@ Supprimer la condition de travail
  SupprimerConditionDeTravail();
  
  //@ Supprimer la condition de disponibilite
  SupprimerConditionDeDisponibilite();
  
  //@ Supprimer la condition de terminaison
  SupprimerConditionDeTerminaison();
}


//@ ------------------------------------------------------------------*/
//@ Methode : AttendreConditionDeTravail
//@
//@ Description : Attente du thread d'une condition de travail 
//@               sur un objet de synchronisation. 
//@               Methode bloquante
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ContexteThread::AttendreConditionDeTravail()
{
  //@ Attendre la condition de travail
#ifdef MUTEX
  SignalerConditionDeDisponibilite(CMN__VRAI);
  pthread_mutex_lock(&m_MutexTravaille);
#else
  pthread_mutex_lock(&m_MutexTravaille);
  SignalerConditionDeDisponibilite(CMN__VRAI);
  pthread_cond_wait(&m_Travaille,&m_MutexTravaille);
  pthread_mutex_unlock(&m_MutexTravaille);
#endif
}


//@ ------------------------------------------------------------------*/
//@ Methode : VerifierConditionDeDisponibilite
//@
//@ Description : Verifie qu'un thread est disponible pour effectuer
//@               une tache. 
//@               Methode non bloquante
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               CMN__VRAI si le thread est disponible
//@               CMN__FAUX sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Booleen SD_FRW_ContexteThread::VerifierConditionDeDisponibilite()
{
  t_CMN__Booleen rep = CMN__FAUX;
  

  //@ Lire en exclusion mutuelle la disponibilite
  pthread_mutex_lock(&m_MutexDisponible);
  rep = m_Disponible;
  pthread_mutex_unlock(&m_MutexDisponible);

  
  //@ Retourner l'etat de disponibilite
  return rep;
}


//@ ------------------------------------------------------------------*/
//@ Methode : VerifierConditionDeTerminaison
//@
//@ Description : Verifie qu'un thread doit terminer de facon 
//@               anticipee une tache. 
//@               Methode non bloquante
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               CMN__VRAI si le thread doit se terminer de facon
//@                 anticipee
//@               CMN__FAUX sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Booleen SD_FRW_ContexteThread::VerifierConditionDeTerminaison()
{
  t_CMN__Booleen rep = CMN__FAUX;

  
  //@ Lire en exclusion mutuelle la condition de terminaison
  pthread_mutex_lock(&m_MutexMeurt);
  rep = m_Meurt;
  pthread_mutex_unlock(&m_MutexMeurt);
        

  //@ Retourner le resultat
  return rep;
}


//@ ------------------------------------------------------------------*/
//@ Methode : SignalerConditionDeTravail
//@
//@ Description : Signale a un thread s'il a une tache a effectuer ou 
//@               non
//@
//@ Parametres :
//@               (ENTREE) disponibilite : CMN__VRAI si le thread
//@                                        doit effectuer une tache
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ContexteThread::SignalerConditionDeTravail(t_CMN__Booleen
                                                       disponibilite)
{
#ifdef MUTEX
  if (disponibilite == CMN__VRAI)
    {
      SignalerConditionDeDisponibilite(CMN__FAUX);
      pthread_mutex_unlock(&m_MutexTravaille);
    }
#else

  //@ SI la condition de travail est demandee ALORS
  if (disponibilite == CMN__VRAI) 
    {
      //@ Signaler qu'il faut bosser
      pthread_mutex_lock(&m_MutexTravaille);
      
      // DEBUT FA 743
      SignalerConditionDeDisponibilite(CMN__FAUX);
      
      // FIN FA 743
      pthread_cond_signal(&m_Travaille);
      
      pthread_mutex_unlock(&m_MutexTravaille);
    }
  //@ FIN SI
#endif
}


//@ ------------------------------------------------------------------*/
//@ Methode : SignalerConditionDeDisponibilite
//@
//@ Description : Signale a un thread si le thread est disponible ou
//@               non
//@
//@ Parametres :
//@               (ENTREE) disponibilite : CMN__VRAI si le thread
//@                                        est disponible
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void 
SD_FRW_ContexteThread::SignalerConditionDeDisponibilite(t_CMN__Booleen 
                                                        disponibilite)
{
  //@ Signaler l'etat de disponibilite
  pthread_mutex_lock(&m_MutexDisponible);
  m_Disponible = disponibilite;
  pthread_mutex_unlock(&m_MutexDisponible);
}


//@ ------------------------------------------------------------------*/
//@ Methode : SignalerConditionDeTerminaison
//@
//@ Description : Signale a un thread qu'il doit se terminer
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ContexteThread::SignalerConditionDeTerminaison()
{
  //@ Signaler la terminaison
  pthread_mutex_lock(&m_MutexMeurt);

#ifdef OPS_DEBUG
  cout << "[" << m_ID << "] dead requested" << endl;
#endif

  m_Meurt = CMN__VRAI;
  pthread_mutex_unlock(&m_MutexMeurt);
}


//======================================================================
//                                      METHODES PRIVEES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : AttendreLaTerminaison
//@
//@ Description : Attente que le thread se termine. Methode bloquante
//@               sur un timer, mort du thread si pas de reaction
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ContexteThread::AttendreLaTerminaison()
{
#ifdef OPS_DEBUG
  cout << "[" << m_ID << "] murdering thread" << endl;
#endif

  //@ Attendre une seconde
  //@ Tres important car la ThreadFn doit terminer de s'executer
  //@ et elle fait appel au contexte de thread que l'on est en
  //@ train de detruire
  //sleep(1); 
  usleep(500000); 

  //@ Tuer le thread
  pthread_cancel(m_hthread);
}


//@ ------------------------------------------------------------------*/
//@ Methode : CreerConditionDeTravail
//@
//@ Description : Cree une variable de synchronisation pour gerer la
//@               condition de travail d'un thread
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ContexteThread::CreerConditionDeTravail()
{
  //@ Creer la variable conditionnelle POSIX correspondante et son
  //@ mutex
  pthread_mutex_init(&m_MutexTravaille,NULL);
#ifdef MUTEX
  pthread_mutex_lock(&m_MutexTravaille);
#else
  pthread_cond_init(&m_Travaille,NULL);
#endif
}


//@ ------------------------------------------------------------------*/
//@ Methode : CreerConditionDeDisponibilite
//@
//@ Description : Cree une variable de synchronisation pour gerer la
//@               condition de disponibilite d'un thread
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ContexteThread::CreerConditionDeDisponibilite()
{
  //@ Creer le mutex POSIX correspondant
  pthread_mutex_init(&m_MutexDisponible,NULL);
  pthread_mutex_lock(&m_MutexDisponible);
  m_Disponible = CMN__VRAI;
  pthread_mutex_unlock(&m_MutexDisponible);
}


//@ ------------------------------------------------------------------*/
//@ Methode : CreerConditionDeTerminaison
//@
//@ Description : Cree une variable de synchronisation pour gerer la
//@               condition de terminaison d'un thread
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ContexteThread::CreerConditionDeTerminaison()
{
  //@ Creer le mutex POSIX correspondant
  pthread_mutex_init(&m_MutexMeurt,NULL);
  pthread_mutex_lock(&m_MutexMeurt);
  m_Meurt = CMN__FAUX;
  pthread_mutex_unlock(&m_MutexMeurt);
}


//@ ------------------------------------------------------------------*/
//@ Methode : SupprimerConditionDeTravail
//@
//@ Description : Supprime la variable de synchronisation pour gerer 
//@               la condition de travail d'un thread
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ContexteThread::SupprimerConditionDeTravail()
{
#ifdef MUTEX
  pthread_mutex_destroy(&m_MutexTravaille);
#else
  //@ Supprimer le mutex et la variable conditionnele POSIX
  pthread_mutex_destroy(&m_MutexTravaille);
  pthread_cond_destroy(&m_Travaille);
#endif
}


//@ ------------------------------------------------------------------*/
//@ Methode : SupprimerConditionDeDisponibilite
//@
//@ Description : Supprime la variable de synchronisation pour gerer
//@               la condition de disponibilite d'un thread
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ContexteThread::SupprimerConditionDeDisponibilite()
{
  //@ Supprimer le mutex POSIX
  pthread_mutex_destroy(&m_MutexDisponible);
}


//@ ------------------------------------------------------------------*/
//@ Methode : SupprimerConditionDeTerminaison
//@
//@ Description : Supprime la variable de synchronisation pour gerer
//@               la condition de terminaison d'un thread
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_ContexteThread::SupprimerConditionDeTerminaison()
{
  //@ Supprimer le mutex POSIX
  pthread_mutex_destroy(&m_MutexMeurt);
}
