// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/FRW/src/SD_FRW_ContexteThread.h,v 1.3 2011/11/25 10:17:45 dpauly Exp $
///@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_FRW
//@ CLASSE : SD_FRW_ContexteThread
//@
//@ Commentaire :
//@    Fichier d'entete de la classe SD_FRW_ContexteThread
//@                                                                     
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 10/08/1999 : SAS
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Adaptation a l'OPS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_FRW_ContexteThread_h
#define SD_FRW_ContexteThread_h


//
// Fichiers include
//

// STL
#include <iostream>

// Standard
#include <unistd.h>
#include <pthread.h>

// Projet


//
// Definition des constantes
//
#define FRW_HANDLE_COND    pthread_cond_t
#define FRW_HANDLE_THREAD  pthread_t
#define FRW_HANDLE         t_CMN__Booleen


class CMN_OBS_Sujet;
class SD_TAC_Tache;

class SD_FRW_ContexteThread {

    /*================================================================*/

 public:
    
    ////////////////////////////////
    // Constructeur
    ////////////////////////////////
    SD_FRW_ContexteThread();
    
    ////////////////////////////////
    // Destructeur
    ////////////////////////////////
    virtual ~SD_FRW_ContexteThread();

    ////////////////////////////////
    // Accesseurs
    ////////////////////////////////
    SD_TAC_Tache*   Tache() {return m_Tache;};
    void            Tache(SD_TAC_Tache* tache) {m_Tache = tache;};

    void            ThreadHandle(FRW_HANDLE_THREAD hthread) 
                        {m_hthread = hthread;};   
    FRW_HANDLE_THREAD ThreadHandle() {return m_hthread;};   

    long            Id() {return m_ID;};
    void            Id(long id) {m_ID = id;};

    CMN_OBS_Sujet*  Sujet() {return m_Sujet;}; 
    void            Sujet(CMN_OBS_Sujet* sujet) {m_Sujet = sujet;};
    

    ////////////////////////////////
    // Methodes publiques
    ////////////////////////////////
    void            SignalerConditionDeTravail(t_CMN__Booleen disponibilite);
    void            SignalerConditionDeDisponibilite(t_CMN__Booleen
                                                     disponibilite = 
                                                     CMN__VRAI);
    void            SignalerConditionDeTerminaison();
    void            AttendreConditionDeTravail();
    t_CMN__Booleen  VerifierConditionDeDisponibilite();
    t_CMN__Booleen  VerifierConditionDeTerminaison();
    

    /*================================================================*/
    
 protected:
    
    ////////////////////////////////
    // Methodes protegees
    ////////////////////////////////

    ////////////////////////////////
    // Attributs proteges
    ////////////////////////////////

    
    /*================================================================*/

 private:

    ////////////////////////////////
    // Methodes privees
    ////////////////////////////////
    void AttendreLaTerminaison();
    void CreerConditionDeTravail();
    void CreerConditionDeDisponibilite();
    void CreerConditionDeTerminaison();
    
    void SupprimerConditionDeTravail();
    void SupprimerConditionDeDisponibilite();
    void SupprimerConditionDeTerminaison();
    
    ////////////////////////////////
    // Attributs prives
    ////////////////////////////////

    // Tache associee au thread
    SD_TAC_Tache*     m_Tache;

    // Demande d'activation d'un travail pour le thread
    FRW_HANDLE_COND   m_Travaille;
    
    // Etat de disponibilite du thread
    FRW_HANDLE        m_Disponible;

    // Demande de terminaison du thread
    FRW_HANDLE        m_Meurt;

    // Identificateur du thread
    FRW_HANDLE_THREAD m_hthread;

    // Mutex pour les variables conditionnelles POSIX
    pthread_mutex_t   m_MutexTravaille;

    // Mutex pour les variables conditionnelles POSIX
    pthread_mutex_t   m_MutexDisponible;

    // Mutex pour les variables conditionnelles POSIX
    pthread_mutex_t   m_MutexMeurt; 

    // Numero du thread
    long              m_ID;

    // Evenement au sujet de la terminaison d'une tache
    CMN_OBS_Sujet*    m_Sujet;
};

#endif
