static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/FRW/src/SD_FRW_Echeancier.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_FRW
//@ CLASSE : SD_FRW_Echeancier
//@ 
//@ Commentaire :
//@    Ce module gere la classe SD_FRW_Echeancier
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 26/08/1999 : SAS
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-93-CN    21/01/00    GRT  Appel de verifier symptome lors
//@                                             d'une fin anormale de tache
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-90-CN    21/01/00    GRT  on positionne la periode a manuel pour
//@                             que l'echeance ne soit pas traitee par l'echeancier
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-61-CN    07/02/00    GRT  On positionne systematiquement le 
//@                             contenu du contexte dans l'echeancier
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-662-CN  et   
//@ SMM-M4-FA-676-CN    17/10/00    GRT Lors de la mise a jour de l'echeancier on recalcule
//@                                     l'echeance uniquement si des parametres ont ete modifies
//@                                     ou si c'est une nouvelle echeance
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-795-CN    27/11/00    GRT Creation d'une liste temporaire des contextes manuels
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-2321-CN   30/08/2001  GRT Il faut verrouiller avant la creation de tout fichier par le SD
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-2366-CN   2/10/01 GRT On gere la liste des derniers fichiers importes
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-997-CN    2/10/01 GRT On verifie que la date du fichier importe est superieure
//@                                 a la date contenue dans l'echeancier
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-997-CN    19/10/01    GRT on affiche le nom du fichier importe Contexte->Fichier(i).c_str()
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-2413-CN   29/10/01    GRT on trie la liste des echeances
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-2490-CN   11/12/01    GRT on gere la liste des fichiers origine dans le cas d'un export zippe
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-2531-CN   06/01/2002 Robert   On calcule la prochaine echeance avant de lancer 
//@                                     la tache pour que la meme tache ne soit pas relancee plusieurs fois
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-2707-CN   13/03/2002 Robert   Le delai de surveillance du SA est en configuration
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Adaptation a l'OPS : modification du contenu et de la gestion de
//@                      l'echeancier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-115-CN : 10/02/2004 : JL.PASCAL
//@ no report were generated in cas of missing data
//@ fiche de correction correspondante : IA-2100-FC-115-THA
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2512 (FA-2100-OPS) : 17/11/2008 : S. BRUNEL
//@ Fuites memoires signalees par EUMETSAT (EUM.EPS.NCR.3627.1)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-3305 : 29/05/2017 : M. LANDIECH
//@ Different output files with mutli-threading
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL
#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <algorithm>
#include <functional>

// Standard
#include <unistd.h>
#include <time.h>
#include <sys/times.h>
#include <sys/time.h>

// Projet
#include "CMN__Commun.h"
#include "CMN__Date.h"
#include "CMN__Fichier.h"
#include "CFG__Configuration.h"
#include "SD_TAC_TacheTraitement.h"
#include "SD_TAC_ContexteDeTache.h"
#include "SD_FRW_ApplicationServeurDonnees.h"
#include "SD_FRW_Echeancier.h"


// #define OK exists in the whole application
#undef OK


//
// Definition des constantes
//
const t_Chaine SD_FRW_Echeancier::FICHIER_MODELES_TRAITEMENTS  = "ModelesTraitements.cfg";
const int      SD_FRW_Echeancier::DEFAUT_PERIODE_ENTRE_ANALYSE = 100000;
const int      SD_FRW_Echeancier::DEFAUT_NB_LIGNES             = 22;
const t_Chaine SD_FRW_Echeancier::LIBELLE_DEBUT_TRAITEMENT     = "Begin of treatment: ";
const t_Chaine SD_FRW_Echeancier::LIBELLE_FIN_TRAITEMENT       = "End of treatment: ";
const t_Chaine SD_FRW_Echeancier::EVT_FONCTION_TRAITEMENT      = "Traitement";
const int      SD_FRW_Echeancier::NB_DIGIT_TOTAL               = 10;
const int      SD_FRW_Echeancier::NB_DIGIT_AFTER_COMA          = 3;

const t_Chaine SD_FRW_Echeancier::NOM_TACHE_SEPARATEUR         = "_";
const t_Chaine SD_FRW_Echeancier::LOG_INFO_SEPARATEUR          = " - ";

const int      SD_FRW_Echeancier::NB_TYPE_TM                   = 5;


//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD_FRW_Echeancier
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_FRW_Echeancier::SD_FRW_Echeancier()
  : m_Application(0), m_EstActif(CMN__FAUX), m_FabriqueDeTache(0),
    m_PeriodeDeScrutation(DEFAUT_PERIODE_ENTRE_ANALYSE),
    m_NbLignes(DEFAUT_NB_LIGNES), m_NbRessourcesMax(1), 
    m_NbRessourcesDispos(1), m_Deprogramme(CMN__FAUX)
{
  //@ Initialiser les proprietes de la classe a leur valeur par defaut
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_FRW_Echeancier
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_FRW_Echeancier::~SD_FRW_Echeancier()
{
  //@ Liberer les ressources creees par l'echeancier
  //@ Supprimer la fabrique de tache
  if (m_FabriqueDeTache!= NULL) 
    {
      delete m_FabriqueDeTache;
    }
  m_FabriqueDeTache = NULL;

  // FT-2512
  t_ListeTache::iterator it = m_Taches.begin();

  for (it = m_Taches.begin(); it!= m_Taches.end(); it++)
  {       
	delete m_Contextes[(*it)->NomTache()];
	delete (*it);
  }
  // ..FT-2512

  //@ Liberer les descriptifs de taches a lancer 
  m_Taches.clear();
  m_Contextes.clear();
}


//@ ------------------------------------------------------------------*/
//@ Methode : Initialiser
//@
//@ Description : Initialise les ressources de l'echeancier
//@
//@ Parametres :
//@               (ENTREE) periodeDeScrutation : Valeur du timeout
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_Echeancier::Initialiser(int periodeDeScrutation)
{  
  //@ Affecter la valeur du timeout
  m_PeriodeDeScrutation = periodeDeScrutation;

  //@ Creer la fabrique de tache
  m_FabriqueDeTache = new SD_FRW_FabriqueDeTache();

  //@ Charger la liste des modeles de traitements
  ChargerModelesTraitements();
}


//@ ------------------------------------------------------------------*/
//@ Methode : Temporiser
//@
//@ Description : Temporise l'echeancier
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_Echeancier::Temporiser()
{
  //@ Temporiser le delai entre deux analyses
  usleep(m_PeriodeDeScrutation);
}


//@ ------------------------------------------------------------------*/
//@ Methode : Deprogrammer
//@
//@ Description : Deprogramme les taches prevues de l'echeancier
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               CMN__VRAI si des taches sont en cours d'execution
//@               CMN__FAUX sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Booleen SD_FRW_Echeancier::Deprogrammer()
{
  // Tache en cours
  t_CMN__Booleen         tacheEnCours = CMN__FAUX;

  // Iterateur pour la liste des taches
  t_ListeTache::iterator it = m_Taches.begin();


  //@ Le traitement ne s'est pas correctement termine
  m_Deprogramme = CMN__VRAI;

  //@ TANT QUE non fin de la liste des taches FAIRE
  while (it != m_Taches.end())
    {   
      //@ SI la tache n'est pas deja en cours ou terminee
      if ( ((*it)->Id() == -1) && (!(*it)->EstTerminee()) ) 
        {

#ifdef OPS_DEBUG
          cout << "ECHEANCIER: deprogramme tache: " 
               << (*it)->NomTache() << endl;
#endif
          
          //@ Supprimer le contexte de la tache
          m_Contextes.erase((*it)->NomTache());

          //@ Supprimer la tache
          it = m_Taches.erase(it);
        }
      //@ SINON
      else
        {
          //@ Tester si la tache est en cours
          if (!(*it)->EstTerminee())
            {
              tacheEnCours = CMN__VRAI;
            }

          //@ Passer a la tache suivante
          it++;
        }
      //@ FIN SI
    }
  //@ FIN TANT QUE non fin de la liste des taches


  //@ SI aucune tache n'etait en cours
  if (tacheEnCours == CMN__FAUX)
    {
    // FT-3305 -- Eviter le double appel à TraiterFinTraitement
    if (m_EstActif == CMN__VRAI)
      {
    // -- FT-3305
        //@ Mettre a jour les variables
        m_EstActif           = CMN__FAUX;
        m_NbRessourcesDispos = 1;
        TraiterFinTraitement(CMN__VRAI);
      }
      else {
#ifdef OPS_DEBUG
        cout << "ECHEANCIER - Deprogrammer : TraiterFinTraitement deja appele " << endl;
#endif
      }
    }
    
  //@ FIN SI


  //@ Renvoyer la valeur de retour de la methode
  return tacheEnCours;
}


//@ ------------------------------------------------------------------*/
//@ Methode : CompterLesTachesAFaire
//@
//@ Description : Compte les taches a faire apres une analyse de
//@               l'etat de l'echeancier
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Nombre de taches a faire
//@
//@ ------------------------------------------------------------------*/
int SD_FRW_Echeancier::CompterLesTachesAFaire()
{    
  // Iterateur pour la liste des taches
  t_ListeTache::iterator it;

  SD_TAC_Tache*          tacheCourante = NULL;
  int                    nbTaches = 0;


#ifdef OPS_DEBUG 
  AfficherTaches();
#endif

  //@ SI l'echeancier est actif ALORS
  if (m_EstActif == CMN__VRAI) 
    {
      //@ POUR toutes les taches FAIRE
      for (it = m_Taches.begin(); it!= m_Taches.end(); it++)
        {       
          tacheCourante = *it;
          
          //@ SI elle n'est pas deja en cours
          if (tacheCourante->Id() == -1) 
            {

#ifdef OPS_DEBUG
              cout << "ECHEANCIER: TACHE: " 
                   << tacheCourante->NomTache()
                   << " - ressources necessaires: " 
                   << tacheCourante->Contexte()->NbRessources()
                   << endl;

              cout << "ECHEANCIER: nb ressources dispos: " 
                   << m_NbRessourcesDispos << endl << endl;
#endif

              //@ SI il n'y a pas suffisamment de ressources
              if (tacheCourante->Contexte()->NbRessources() >
                  m_NbRessourcesDispos)
                {
                  //@ On arrete le comptage
                  break;
                }
              //@ SINON
              else
                {
                  //@ On comptabilise la tache
                  m_NbRessourcesDispos -= 
                    tacheCourante->Contexte()->NbRessources();
                  nbTaches++;
                }
              //@ FIN SI
            }
          //@ FIN SI
        }
      //@ FIN POUR toutes les taches
    }
  //@ FIN SI  

#ifdef OPS_DEBUG
  cout << "ECHEANCIER::CompterLesTachesAFaire: " << nbTaches << endl;
#endif


  //@ Retourner le nombre de taches a faire
  return nbTaches;
}


//@ ------------------------------------------------------------------*/
//@ Methode : TacheAFaire
//@
//@ Description : Retourne la premiere tache a faire par rapport a la
//@               derniere analyse de l'echeancier realisee
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Tache a faire
//@
//@ ------------------------------------------------------------------*/
SD_TAC_Tache* SD_FRW_Echeancier::TacheAFaire() throw (ERR__Exception)
{
  // Iterateur pour la liste des taches
  t_ListeTache::iterator it;

  // Temoin de recherche
  t_CMN__Booleen         trouve = CMN__FAUX;


  //@ Recherche de la tache a faire
  it = m_Taches.begin();

  //@ TANT QUE il existe des taches ET QUE la tache a faire
  //@ n'a pas ete trouvee
  while (it != m_Taches.end() && (trouve == CMN__FAUX))
    {   
      //@ SI la tache n'est pas deja en cours
      if ((*it)->Id() == -1)
        {
          //@ La tache a faire a ete trouvee
          //@ Mettre a jour le temoin de recherche
          trouve = CMN__VRAI;
        }
      //@ SINON
      else
        {
          //@ Passer a la tache suivante
          it++;
        }
      //@ FIN SI
    }
  //@ FIN TANT QUE il existe des taches

  //@ SI la tache n'a pas ete trouvee
  if (trouve != CMN__VRAI)
    {
      ERR__Exception exception;
    
      //@ Emission exception SD_ECH_TAC_NOT_FOUND
      exception.SetErrorCode(ERR__Exception::SD_ECH_TAC_NOT_FOUND);
      THROW(exception);
    }
  //@ SINON
  else
    {
      //@ Retourner la tache trouvee
      return (*it);
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : AjouterTraitement
//@
//@ Description : Ajoute les taches correspondant a un traitement
//@
//@ Parametres :
//@               (ENTREE) traitement : Traitement demande
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_Echeancier::AjouterTraitement(SD_DON_Traitement& traitement)
{
 //FA 115 : attribut pour gerer les pb de donnees
      int                      init = 0;
  //@ En captant les exceptions
  try
    {
      CMN_DAT_ModeleTraitement modele;
      t_Chaine                 nomTraitement("");
      int                      nbLigne = 1;
      t_TableauDate            startTime;
      t_TableauEntier          startOrbitNumber;
     

      //@ Initialiser les donnees
      m_Application->Donnees()->SetTraitement(traitement);

      //@ Initialisation
      nomTraitement = traitement.TypeTraitementStr();

      //@ Tracer le debut du traitement
      TracerDebutTraitement(nomTraitement);

      //@ Purge des taches de l'echeancier
      Purger();
      
      //@ Recuperer le modele du traitement
      RecupererModeleTraitement(nomTraitement, modele);

      //@ SI premier traitement du DUMP
      if (traitement.SlideId() == FIRST)
        {
          //@ Appeler la methode SD_DON_GestDonnees::InitDump()
          m_Application->Donnees()->InitDump();
        }
 //FA 115 : attribut pour gerer les pb de donnees
      init = 1;
      //@ FIN SI premier traitement du DUMP

      //@ Appeler la methode SD_DON_GestDonnees::InitGranule()
      m_Application->Donnees()->InitGranule(startTime, 
                                            startOrbitNumber);

 //FA 115 : attribut pour gerer les pb de donnees
      init = 2;

      //@ Envoi des HKTM status StartTime et StartOrbitNumber
      EnvoyerStartHKTM(startTime, startOrbitNumber);

      //@ Appeler la methode SD_DON_GestDonnees::GetNbLignesIASI()
      nbLigne = m_Application->Donnees()->GetNbLignesIASI();

      m_NbLignes           = nbLigne;
      m_NbRessourcesDispos = nbLigne;
      m_NbRessourcesMax    = nbLigne;

      //@ Definir les taches de l'echeancier
      RemplirEcheancier(modele);
    }

  //@ Exploiter les exceptions
  catch (ERR__Exception &exception)
    {
      // Infos sur l'erreur apparue
      t_Chaine infos("");

      //@ Traiter le cas de la fin d'un traitement en erreur
      for (int i=0; i<exception.GetNbParametre(); i++)
        {
          infos += LOG_INFO_SEPARATEUR + exception.GetParametre(i);
        }
	//jlp si un cas degrade permanent est leve on s'arrete
      t_Chaine tmp("Permanent Degraded Case raised");
      if (exception.GetNbParametre() >=2 && exception.GetParametre(1)==tmp)
       {
       init = 0;
       }

      //FA 115 : attribut pour gerer les pb de donnees
      TraiterFinTraitement(CMN__FAUX, infos,init);


      //@ Re-emission de l'exception
      throw(exception);
    }

#ifdef OPS_DEBUG
  AfficherTaches();
  AfficherContextes();
#endif
}


//@ ------------------------------------------------------------------*/
//@ Methode : MettreAJour
//@
//@ Description : Met a jour l'etat de l'echeancier en fonction des
//@               proprietes de la tache qui vient de se terminer
//@
//@ Parametres :
//@               (ENTREE) tache : Tache dans l'etat termine
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_Echeancier::MettreAJour(SD_TAC_Tache * tache)
{
  // Date de fin de la tache
  CMN__Date      dateFin;

  // Envoi du message fin de traitement
  t_CMN__Booleen envoiMsg = CMN__FAUX;


  //@ Mettre a jour la date de fin de la tache
  dateFin.DateCourante();
  tache->DateFin(dateFin);

#ifdef OPS_DEBUG
  cout << endl << "ECHEANCIER: mise a jour tache: " 
       << tache->NomTache() << endl;
#endif

  //@ Verrouiller l'acces aux ressources de l'application
  m_Application->Verrouiller();

  //@ Mettre a jour les ressources
  m_NbRessourcesDispos += tache->Contexte()->NbRessources();

  //@ Mettre a jour l'etat de l'echeancier: tester si on est a la
  //@ fin du traitement
  envoiMsg = TesterFinTraitement();

  //@ Deverrouiller l'acces aux ressources de l'application
  m_Application->Deverrouiller();

  //@ SI fin du traitement
  if (envoiMsg == CMN__VRAI)
    {
      // FT-3305 -- Eviter le double appel à TraiterFinTraitement
      if (m_EstActif == CMN__VRAI)
      {
      // -- FT-3305
        //@ Mettre a jour les variables
        m_EstActif           = CMN__FAUX;
        m_NbRessourcesDispos = 1;
        //@ Alors effectuer les post-traitements et envoyer un
        //@ message de fin de traitement
        TraiterFinTraitement(CMN__VRAI);
      }
      else {
#ifdef OPS_DEBUG
        cout << "ECHEANCIER - MettreAJour : TraiterFinTraitement deja appele " << endl;
#endif
      }


    }
  //@ FIN SI
}


//======================================================================
//                                      METHODES PRIVEES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : AjouterTache
//@
//@ Description : Ajoute une tache pour traitement lors
//@               de la prochaine analyse
//@
//@ Parametres :
//@               (ENTREE) nomDeTache : Nom de la tache a lancer
//@               (ENTREE) typeTache : Type de la tache a lancer
//@               (ENTREE) pContexte : Contexte de la tache a lancer
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_Echeancier::AjouterTache(t_Chaine& nomTache,
                                     t_Chaine& typeTache,
                                     SD_TAC_ContexteDeTache* pContexte)
{
  SD_TAC_Tache* nouvelleTache = NULL;
  t_Date        date;


  //@ Creer la tache
  nouvelleTache = m_FabriqueDeTache->CreerTache(typeTache);

  //@ Affecter le nom a la tache
  nouvelleTache->NomTache(nomTache);

  //@ Definir le type de la tache
  nouvelleTache->TypeTache(typeTache);
 
  //@ Definir la date de creation de la tache
  date.DateCourante();
  nouvelleTache->DateCreation(date);
 
  //@ Affecter le contexte a la tache
  nouvelleTache->Contexte(pContexte);

  //@ Inserer la tache dans la liste des taches
  m_Taches.push_back(nouvelleTache);

  //@ Inserer le contexte dans la liste des contextes
  m_Contextes[nomTache] = pContexte;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Purger
//@
//@ Description : Supprimer les taches et les contextes de 
//@               l'echeancier
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_Echeancier::Purger()
{
  // Iterateur pour la liste des taches
  t_ListeTache::iterator it;


  //@ POUR toutes les taches FAIRE
  for (it = m_Taches.begin(); it!= m_Taches.end(); it++)
    {   
      //@ Suppression du contexte et de la tache
      m_Contextes.erase((*it)->NomTache());
      it = m_Taches.erase(it);      
    } 
  //@ FIN POUR toutes les taches
 
  //@ RAZ des structures des contextes et des taches
  m_Taches.clear();
  m_Contextes.clear();
  m_Deprogramme = CMN__FAUX;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ChargerModelesTraitements
//@
//@ Description : Charge les modeles de traitements connus
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_Echeancier::ChargerModelesTraitements() throw (ERR__Exception)
{
  // Exception generee
  ERR__Exception exception;
  
  // Fichier des modeles de traitements
  t_Chaine       fichierConfiguration("");


  //@ Ouvrir le fichier des modeles de traitements
  fichierConfiguration = CFG__Configuration::GetRepertoire(CONF_DIR);

  //@ SI la variable d'environnement REP_CONF_DIR n'est pas definie
  if (fichierConfiguration.empty())
    {
      //@ Emission d'une exception SD_ENV_UNDEFINED
      exception.SetErrorCode(ERR__Exception::SD_ENV_UNDEFINED);
      exception.SetParametre(CONF_DIR);
      THROW(exception);
    }
  //@ FIN SI

  fichierConfiguration += CMN__SEP_REPERTOIRE;
  fichierConfiguration += FICHIER_MODELES_TRAITEMENTS;

  //@ Charger les differents modeles de traitements
  //@ Chaque modele est compose des mnemoniques ordonnes des
  //@ taches qui le composent
  //@ SI le chargement a ete incorrect
  if (m_ListeModelesTraitements.ChargerListe(fichierConfiguration) != 
      CMN__STATUT_OK)
    {
      //@ Emission d'une exception SD_FILE_OPEN_ERROR
      exception.SetErrorCode(ERR__Exception::SD_FILE_OPEN_ERROR);
      exception.SetParametre(FICHIER_MODELES_TRAITEMENTS);
      THROW(exception);
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : RecupererModeleTraitement
//@
//@ Description : Recupere un modele de traitement donne
//@
//@ Parametres :
//@               (ENTREE) nomTraitement : Nom du traitement
//@               (SORTIE) modele : Modele associe
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void 
SD_FRW_Echeancier::RecupererModeleTraitement(t_Chaine nomTraitement,
                                             CMN_DAT_ModeleTraitement& 
                                             modele)
  throw (ERR__Exception)
{
  //@ Parcourir la liste des modeles jusqu'au modele dont le
  //@ mnemonique est fourni
  int i = m_ListeModelesTraitements.RechercherElement(nomTraitement);
  
  //@ SI le modele n'a pas ete trouve
  if (i == -1) 
    {
      // Exception generee
      ERR__Exception exception;
      
      //@ Emission d'une exception SD_TMT_NOT_FOUND
      exception.SetErrorCode(ERR__Exception::SD_TMT_NOT_FOUND);
      exception.SetParametre(nomTraitement);
      THROW(exception);
    }
  //@ FIN SI
  
  //@ On lit les taches decrites dans le modele
  m_ListeModelesTraitements.LireElement(i, modele);
}


//@ ------------------------------------------------------------------*/
//@ Methode : RemplirEcheancier
//@
//@ Description : Remplit l'echeancier en fonction du modele de taches
//@               passe en parametre
//@
//@ Parametres :
//@               (ENTREE) modele : Modele de taches
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_Echeancier::RemplirEcheancier(CMN_DAT_ModeleTraitement& modele)
{
  t_Chaine       typeTache("");
  t_Chaine       nomTache("");
  int            nbOccurrences   = 1;
  int            nbRessources    = 1;
  int            noTacheCourante = 1;
  t_CMN__Booleen signal          = CMN__FAUX;


  //@ POUR chaque tache contenue dans le modele FAIRE
  for (int i=0; i<modele.NombreTaches(); i++) 
    { 
      t_CMN__Booleen signalerTemps = CMN__FAUX;

      //@ Si la tache est du type LIGNE,
      //@ alors ajouter autant d'occurrences que de lignes
      nbOccurrences = 1;
      nbRessources  = 1;
      if (modele.TypeTache(i) == 
          CMN_DAT_ModeleTraitement::TYPE_TACHE_LIGNE)
        {
         nbOccurrences = m_NbLignes;
         if (signal == CMN__FAUX)
           {
             signalerTemps = CMN__VRAI;
             signal        = CMN__VRAI;
           }
         }
      else if (modele.TypeTache(i) == 
               CMN_DAT_ModeleTraitement::TYPE_TACHE_RDV)
        {
          nbRessources = m_NbRessourcesMax;
        }
          
      //@ POUR chaque occurrence FAIRE
      for (int j=0; j<nbOccurrences; j++)
        {
          //@ Ajouter la tache a l'echancier
          typeTache = modele.Tache(i).c_str();
          nomTache  = typeTache + NOM_TACHE_SEPARATEUR + 
            t_Chaine(j+1) + NOM_TACHE_SEPARATEUR + 
            t_Chaine(noTacheCourante++);
          
          //@ Ajouter le contexte de la tache
          SD_TAC_ContexteDeTache* pContexte = NULL;
          pContexte = new SD_TAC_ContexteDeTache;
          pContexte->NomTravail(nomTache);
          pContexte->IndiceLigne(j+1);
          pContexte->NbRessources(nbRessources);
          pContexte->SignalerTemps(signalerTemps);

          //@ Ajouter la tache et affecter son contexte
          AjouterTache(nomTache, typeTache, pContexte);
        }
      //@ FIN POUR chaque occurrence
    }
  //@ FIN POUR chaque tache contenue dans le modele
}


//@ ------------------------------------------------------------------*/
//@ Methode : EnvoyerStartHKTM
//@
//@ Description : Envoie les HKTM status StartTime et StartOrbitNumber
//@
//@ Parametres :
//@               (ENTREE) startTime : Start Time des produits 
//@               (ENTREE) startOrbitNumber : Start Orbit Number des
//@                                           produits
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_Echeancier::EnvoyerStartHKTM(t_TableauDate& startTime, 
                                         t_TableauEntier& 
                                         startOrbitNumber)
{
  const MP__HKTMStatus::e_ListeStatus typeSensingStartTime[] = 
    {MP__HKTMStatus::L1a_SensingStartTime,
     MP__HKTMStatus::L1b_SensingStartTime,
     MP__HKTMStatus::L1c_SensingStartTime,
     MP__HKTMStatus::ENG_SensingStartTime,
     MP__HKTMStatus::VER_SensingStartTime};
  
  const MP__HKTMStatus::e_ListeStatus typeStartOrbitNumber[] = 
    {MP__HKTMStatus::L1a_StartOrbit,
     MP__HKTMStatus::L1b_StartOrbit,
     MP__HKTMStatus::L1c_StartOrbit,
     MP__HKTMStatus::ENG_StartOrbit,
     MP__HKTMStatus::VER_StartOrbit};
  

  //@ SI le nombre d'elements du tableau startTime n'est pas 
  //@ superieur au nombre maximum de type de telemesures
  if (startTime.size() <= NB_TYPE_TM)
    {
      //@ POUR tous les elements du tableau startTime
      for (int i=0; i<startTime.size(); i++)
        {
          //@ Appel a la fonction 
          //@ SD_FRW_ApplicationServeurDonnees::EnvoyerTM()
          m_Application->EnvoyerTM(typeSensingStartTime[i], 
                                   startTime[i].DateUTC());
        }
      //@ FIN POUR tous les elements du tableau startTime
    }
  //@ FIN SI


  //@ SI le nombre d'elements du tableau startOrbitNumber n'est pas 
  //@ superieur au nombre maximum de type de telemesures
  if (startOrbitNumber.size() <= NB_TYPE_TM)
    {
      //@ POUR tous les elements du tableau startOrbitNumber
      for (int i=0; i<startOrbitNumber.size(); i++)
        {
          //@ Appel a la fonction 
          //@ SD_FRW_ApplicationServeurDonnees::EnvoyerTM()
          m_Application->EnvoyerTM(typeStartOrbitNumber[i], 
                                   t_Chaine(startOrbitNumber[i]));
        }
      //@ FIN POUR tous les elements du tableau startOrbitNumber
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : TesterFinTraitement
//@
//@ Description : Teste si le traitement est termine
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               CMN__VRAI si le traitement est termine
//@               CMN__FAUX sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Booleen SD_FRW_Echeancier::TesterFinTraitement()
{
  // Iterateur pour la liste des taches
  t_ListeTache::iterator it;

  // Temoin de recherche
  t_CMN__Booleen         traitementTermine = CMN__VRAI;


  //@ Parcours de l'echeancier afin de trouver au moins une tache 
  //@ non terminee
  it = m_Taches.begin();

  //@ TANT QUE la liste des taches n'a pas ete entierement parcourue
  //@ ET que le temoin de recherche est a FAUX
  while ( (it != m_Taches.end()) && (traitementTermine == CMN__VRAI) )
    {
      //@ SI la tache courante n'est pas terminee
      if ((*it)->EstTerminee() == CMN__FAUX) 
        {
          //@ Positionner le temoin de recherche a VRAI
          traitementTermine = CMN__FAUX;
        }
      //@ SINON
      else
        {
          //@ Passer a la tache suivante
          it++;
        }
      //@ FIN SI
    }
  //@ FIN TANT QUE


  //@ Retourner le code retour de la fonction
  return traitementTermine;
}


//@ ------------------------------------------------------------------*/
//@ Methode : TraiterFinTraitement
//@
//@ Description : Traite la fin d'un traitement
//@
//@ Parametres :
//@               (ENTREE) finNominale : Fin normale ou en erreur
//@               (ENTREE) infos : Infos en cas d'erreur
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_Echeancier::TraiterFinTraitement(const t_CMN__Booleen finNominale,
                                             const t_Chaine infos, const
					     int init)
 //FA 115 : attribut pour gerer les pb de donnees
{
  // Processing state
  MP__HKTMStatus::e_ProcessingState procState = MP__HKTMStatus::FAILURE;
  
  // Nom du traitement
  t_Chaine                          nomTraitement("");
  
  // Message de log
  t_Chaine                          log("");

  // Traitement en erreur ou non  
  t_CMN__Booleen                    traitementEnErreur = CMN__VRAI;

  //@ Traitement
  SD_DON_Traitement                 traitementCourant;
 

  nomTraitement = m_Application->Donnees()->GetTraitement().
    TypeTraitementStr();
  log           = LIBELLE_FIN_TRAITEMENT + nomTraitement + infos;
  m_Application->Donnees()->GetTraitement().ListeDonneesAux().clear();

  //@ SI la fin du traitement est une fin normale
  if ( (finNominale == CMN__VRAI) && (m_Deprogramme == CMN__FAUX) )
    {
      //@ En captant les exceptions
      try
        {
          //@ Appeler la methode SD_DON_GestDonnees::FinGranule()
          m_Application->Donnees()->FinGranule();
          
          //@ Le traitement se termine correctement
          traitementEnErreur = CMN__FAUX;
        }

      //@ Traiter les exceptions
      catch (ERR__Exception &exception)
        {

#ifdef OPS_DEBUG
          cout << "Exception sur fin traitement: " << endl;
          cout << "\tFile : " << exception.GetFile() << endl;
          cout << "\tLine : " << exception.GetLine() << endl;
          cout << "\tError: " << exception.GetErrorCode() << endl;
          cout << "\tErrno: " << exception.GetErrno() << endl;
          for (int i=0; i<exception.GetNbParametre(); i++)
            {
              cout << "\tInfo : " << exception.GetParametre(i) << endl;
            }
#endif

          for (int i=0; i<exception.GetNbParametre(); i++)
            {
              log += LOG_INFO_SEPARATEUR + exception.GetParametre(i);
            }
        } 
    }
  //@ SINON SI l'echeancier a ete deprogramme
  else if (m_Deprogramme == CMN__VRAI)
    {
      //@ Appeler la methode SD_DON_GestDonnees::ErreurGranule()
      m_Application->Donnees()->ErreurGranule();
    }
  //@ FIN SI la fin du traitement est une fin normale


  //@ Recuperation du traitement
  traitementCourant = m_Application->Donnees()->GetTraitement();
  
  //@ Preparer le compte-rendu du traitement:
  //@ SI le traitement se termine en erreur
  if (traitementEnErreur == CMN__VRAI)
    {
      //@ - etat du traitement : NOK
      if (traitementCourant.Etat() == 
          SD_DON_Traitement::SD_TMT_TO_BE_RUN)
        {
 //FA 115 : attribut pour gerer les pb de donnees
        if ( init == 2 )
	  {
          traitementCourant.Etat(SD_DON_Traitement::SD_TMT_NOK);
	  }
	  else if ( init == 1 )
	  {
          traitementCourant.Etat(SD_DON_Traitement::SD_TMT_PB_INITGRAN);
	  }
	  else
	  {
          traitementCourant.Etat(SD_DON_Traitement::SD_TMT_PB_INITDUMP);
	  }	  
        }
    }
  //@ SINON
  else
    {
      double   elapsedTime = traitementCourant.ElapsedTime();
      double   userTime    = traitementCourant.UserTime();
      t_Chaine data("");
                  
      data.FormaterValeurF(elapsedTime, NB_DIGIT_TOTAL, 
                           NB_DIGIT_AFTER_COMA);
      log += " which lasts: " + data;
      data.FormaterValeurF(userTime, NB_DIGIT_TOTAL, 
                           NB_DIGIT_AFTER_COMA);
      log += " (ELAPSED); " + data + " (USER)";         
      
      //@ - etat du traitement : OK
      if (traitementCourant.Etat() == 
          SD_DON_Traitement::SD_TMT_TO_BE_RUN)
        {
          traitementCourant.Etat(SD_DON_Traitement::SD_TMT_OK);
          procState = MP__HKTMStatus::OK;
        }  
    }
  //@ FIN SI
  
  //@    - message de log
  traitementCourant.LogMessage(log);


  //@ Vider les listes inutilisees en retour
  traitementCourant.ListeProduitAVHRR().clear();
  if ( (traitementCourant.ModeTraitement() == DUMP) && 
       (traitementCourant.SlideId() != LAST) &&
       (traitementEnErreur == CMN__FAUX) )
    {
      traitementCourant.ListeProduitS().clear();
      traitementCourant.ListeDonneesAux().clear();
    }

  //@ Tracer la fin du traitement
  TracerFinTraitement(nomTraitement,
                      traitementCourant.Etat(),
                      traitementCourant.ElapsedTime(),
                      traitementCourant.UserTime());
  
#ifdef OPS_DEBUG
  cout << "ECHEANCIER: envoi traitement: " << endl;
  traitementCourant.Afficher();
#endif

  //@ Envoyer le message FIN TRAITEMENT
  m_Application->EnvoyerMessageFinTraitement(traitementCourant);
  

  //@ SI traitement LAST et/ou GRANULE
  if ( (traitementCourant.SlideId() == LAST) || 
       (traitementCourant.ModeTraitement() == GRANULE) )
    {
      //@ Envoi d'un message HKTM L1a ProcessingState
      m_Application->EnvoyerTM(MP__HKTMStatus::L1a_ProcessingState,
                               CMN_HKTM_PROCESSING_STATE[procState]);
      
      //@ Envoi d'un message HKTM L1b ProcessingState
      m_Application->EnvoyerTM(MP__HKTMStatus::L1b_ProcessingState,
                               CMN_HKTM_PROCESSING_STATE[procState]);
      
      //@ Envoi d'un message HKTM L1c ProcessingState
      m_Application->EnvoyerTM(MP__HKTMStatus::L1c_ProcessingState,
                               CMN_HKTM_PROCESSING_STATE[procState]);
      
      //@ Envoi d'un message HKTM ENG ProcessingState
      m_Application->EnvoyerTM(MP__HKTMStatus::ENG_ProcessingState,
                               CMN_HKTM_PROCESSING_STATE[procState]);
      
      //@ SI traitement complet
      if (traitementCourant.TypeTraitement() == CMN__StageTM::L1a)
        {
          //@ Envoi d'un message HKTM VER ProcessingState
          m_Application->EnvoyerTM(MP__HKTMStatus::VER_ProcessingState,
                                   CMN_HKTM_PROCESSING_STATE[procState]);
        }
      //@ FIN SI traitement complet
    }
  //@ FIN SI traitement LAST et/ou GRANULE
}


//@ ------------------------------------------------------------------*/
//@ Methode : TracerDebutTraitement
//@
//@ Description : Journalise le debut d'un traitement
//@
//@ Parametres :
//@               (ENTREE) traitement : Nom du traitement
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_Echeancier::TracerDebutTraitement(const t_Chaine traitement)
{
  CMN_DAT_Evenement evenement;


  //@ Creer un evenement

  //@  - chrono
  evenement.Identifiant(m_Application->AllouerChronoEvenement());

  //@  - type = MSG_SD_TMT_BEGIN
  evenement.TypeEvenement(SD__Message::MSG_SD_TMT_BEGIN);

  //@  - fonction = EVT_FONCTION_TRAITEMENT
  evenement.Fonction(EVT_FONCTION_TRAITEMENT);

  //@  - libelle
  evenement.Libelle(LIBELLE_DEBUT_TRAITEMENT+traitement);

  //@  - renseignement des parametres pour le journal de bord
  evenement.Prm1(traitement);

  //@ L'envoyer au serveur d'evenements 
  m_Application->Evenements()->JournaliserEvenement(evenement);
}


//@ ------------------------------------------------------------------*/
//@ Methode : TracerFinTraitement
//@
//@ Description : Journalise la fin d'un traitement
//@
//@ Parametres :
//@               (ENTREE) traitement : Nom du traitement
//@               (ENTREE) etat : Etat du traitement
//@               (ENTREE) elapsedTime : Temps ecoule pour le traitement
//@               (ENTREE) userTime : Temps utilise pour le traitement
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_Echeancier::TracerFinTraitement(const t_Chaine traitement,
                                            const SD_DON_Traitement::
                                            e_ListeStatus etat,
                                            const double elapsedTime,
                                            const double userTime)
{
  CMN_DAT_Evenement evenement;


  //@ Creer un evenement

  //@  - chrono
  evenement.Identifiant(m_Application->AllouerChronoEvenement());

  //@  - fonction
  evenement.Fonction(EVT_FONCTION_TRAITEMENT);

  //@  - libelle
  evenement.Libelle(LIBELLE_FIN_TRAITEMENT+traitement);
  
  //@  - renseignement du premier parametre pour le journal de bord
  evenement.Prm1(traitement);

  //@ SI le traitement s'est correctement deroule
  if (etat == SD_DON_Traitement::SD_TMT_OK)
    {
      // Formatage des valeurs doubles
      t_Chaine data("");


      //@  - type = MSG_SD_TMT_END
      evenement.TypeEvenement(SD__Message::MSG_SD_TMT_END);

      //@  - renseignement des autres parametres 
      //@    pour le journal de bord
      data.FormaterValeurF(elapsedTime, 
                           NB_DIGIT_TOTAL, 
                           NB_DIGIT_AFTER_COMA);
      evenement.Prm2(data);
      data.FormaterValeurF(userTime, 
                           NB_DIGIT_TOTAL, 
                           NB_DIGIT_AFTER_COMA);
      evenement.Prm3(data);
    }
  //@ SINON
  else
    {
      //@  - type = MSG_SD_TMT_ENDED_ON_ERROR
      evenement.TypeEvenement(SD__Message::MSG_SD_TMT_ENDED_ON_ERROR);
    }
  //@ FIN SI

  //@ L'envoyer au serveur d'evenements 
  m_Application->Evenements()->JournaliserEvenement(evenement);
}


//@ ------------------------------------------------------------------*/
//@ Methode : AfficherTaches
//@
//@ Description : Affiche les taches contenues dans la liste des
//@               taches de l'echeancier
//@
//@ Parametres :
//@               Aucune
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_Echeancier::AfficherTaches()
{
  // Iterateur pour la liste des taches
  t_ListeTache::iterator it;

  // Tache courante
  SD_TAC_Tache*          tacheCourante = NULL;

  // Compteur d'iteration
  int                    i = 0;


  cout << endl 
       << "-------------------- Liste des taches --------------------";
  cout << endl << "Nombre taches   : " << m_Taches.size() << endl;
 
  for (it = m_Taches.begin(); it!= m_Taches.end(); it++)
    {   
      tacheCourante = *it;
      i++;

      cout << endl << "TACHE " << i << ": id (" 
           << tacheCourante->Id() << ")";
      cout << endl << "\tNom de la tache : " 
           << tacheCourante->NomTache();
      cout << endl << "\tType de la tache: " 
           << tacheCourante->TypeTache();
      cout << endl << "\tEst terminee    : " 
           << tacheCourante->EstTerminee();
      cout << endl << "\tSimulation      : " 
           << tacheCourante->Simulation();
      cout << endl << "\tDate creation   : " << 
        tacheCourante->DateCreation().Chaine();
      cout << endl << "\tDate de debut   : " << 
        tacheCourante->DateDebut().Chaine();
      cout << endl << "\tDate de fin     : " << 
        tacheCourante->DateFin().Chaine();
    }
    
  cout << endl 
       << "----------------------------------------------------------"
       << endl;  

  cout << "ECHEANCIER: nb ressources dispos: " <<
    m_NbRessourcesDispos << "/" << m_NbRessourcesMax << endl;
}


//@ ------------------------------------------------------------------*/
//@ Methode : AfficherContextes
//@
//@ Description : Affiche les contextes contenues dans la liste des
//@               contextes de l'echeancier
//@
//@ Parametres :
//@               Aucune
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_Echeancier::AfficherContextes()
{  
  // Iterateur pour la liste des contextes
  t_MapContexte::iterator  it;

  // Contexte courant
  SD_TAC_ContexteDeTache*  pContexte = NULL;

  // Compteur d'iteration
  int                      i = 0;
  

  cout << endl 
       << "------------------- Liste des contextes ------------------";
  cout << endl << "Nombre contextes: " << m_Contextes.size() << endl;

  for (it = m_Contextes.begin(); it!= m_Contextes.end(); it++)
    {   
      pContexte = m_Contextes[(*it).first];
      i++;
      
      cout << endl << "CONTEXTE " << i 
           << ": clef (" << (*it).first << ")";
      cout << endl << "\tTravail     : " << pContexte->NomTravail();
      cout << endl << "\tIndice lig  : " << pContexte->IndiceLigne();
      cout << endl << "\tNb threads  : " << pContexte->NbRessources();
      cout << endl << "\tSignal temps: " 
           << ((pContexte->SignalerTemps() == CMN__VRAI)?"OUI":"NON")
           << endl;
    }

  cout << endl 
       << "----------------------------------------------------------"
       << endl;  
}
