// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/FRW/src/SD_FRW_Echeancier.h,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_FRW
//@ CLASSE : SD_FRW_Echeancier
//@
//@ Commentaire :
//@    Fichier d'entete de la classe SD_FRW_Echeancier
//@                                                                     
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Adaptation a l'OPS : modification du contenu et de la gestion de
//@                      l'echeancier
//@ --------------------------------------------------------------------
//@ FAID<SMM-M4-FA-47-CN>    20/01/00      Robert  Modification de la periodicite par defaut
//@ --------------------------------------------------------------------
//@ FAID<SMM-M4-FA-795-CN>    27/11/00    GRT Creation d'une liste temporaire des contextes manuels
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-2531-CN   06/01/2002 Robert   On calcule la prochaine echeance avant de lancer
//@                                     la tache pour que la meme tache ne soit pas relancee plusieurs fois
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-2707-CN   13/03/2002 Robert   Le delai de surveillance du SA est en configuration
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-115-CN : 10/02/2004 : JL.PASCAL
//@ no report were generated in cas of missing data
//@ fiche de correction correspondante : IA-2100-FC-115-THA
//@ --------------------------------------------------------------------
//@ VERSION : 10/08/1999 : SAS
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_FRW_Echeancier_h
#define SD_FRW_Echeancier_h


//
// Fichiers include
//

// STL
#include <iostream>
#include <iomanip>
#include <map>

// Standard

// Projet
#include "ERR__Exception.h"
#include "CMN_DAT_ListeObjets.h"
#include "CMN_DAT_ModeleTraitement.h"
#include "SD_DON_Traitement.h"
#include "SD_TAC_Tache.h"
#include "SD_FRW_FabriqueDeTache.h"

extern "C" {
#include "SD_BAS_Niveau1.h"
#include "SD_BAS_Produits.h"
}

class SD_FRW_Echeancier {

    /*================================================================*/

 public:
    
    ////////////////////////////////
    // Constructeur
    ////////////////////////////////
    SD_FRW_Echeancier();
        
    ////////////////////////////////
    // Destructeur
    ////////////////////////////////
    virtual ~SD_FRW_Echeancier();

    ////////////////////////////////
    // Accesseurs
    ////////////////////////////////
    void           Application(SD_FRW_ApplicationServeurDonnees* 
                               application) 
        {m_Application = application;};
    
    void           EstActif(t_CMN__Booleen etat) {m_EstActif = etat;};
    t_CMN__Booleen EstActif() {return m_EstActif;};


    ////////////////////////////////
    // Methodes publiques
    ////////////////////////////////
    void           Initialiser(int periodeDeScrutation);
    void           Temporiser();
    t_CMN__Booleen Deprogrammer();
    int            CompterLesTachesAFaire();
    SD_TAC_Tache*  TacheAFaire() throw (ERR__Exception);
    void           AjouterTraitement(SD_DON_Traitement& traitement);
    void           MettreAJour(SD_TAC_Tache* tache);


    /*================================================================*/
    
 protected:

    ////////////////////////////////
    // Methodes protegees
    ////////////////////////////////
    
    ////////////////////////////////
    // Attributs proteges
    ////////////////////////////////
    

    /*================================================================*/

 private:

    ////////////////////////////////
    // Types
    ////////////////////////////////
    typedef map<t_Chaine, SD_TAC_ContexteDeTache*> t_MapContexte;
    typedef list<SD_TAC_Tache*>                    t_ListeTache;

    ////////////////////////////////
    // Constantes
    ////////////////////////////////
    static const t_Chaine FICHIER_MODELES_TRAITEMENTS;
    static const int      DEFAUT_PERIODE_ENTRE_ANALYSE;
    static const int      DEFAUT_NB_LIGNES;
    static const t_Chaine LIBELLE_DEBUT_TRAITEMENT;
    static const t_Chaine LIBELLE_FIN_TRAITEMENT;
    static const t_Chaine EVT_FONCTION_TRAITEMENT;
    static const int      NB_DIGIT_TOTAL;
    static const int      NB_DIGIT_AFTER_COMA;
    static const t_Chaine NOM_TACHE_SEPARATEUR;
    static const t_Chaine LOG_INFO_SEPARATEUR;
    static const int      NB_TYPE_TM;

    ////////////////////////////////
    // Methodes privees
    ////////////////////////////////
    void           AjouterTache(t_Chaine& nomTache,
                                t_Chaine& typeTache,
                                SD_TAC_ContexteDeTache* pContexte);
    void           Purger();
    void           ChargerModelesTraitements() throw (ERR__Exception);
    void           RecupererModeleTraitement(t_Chaine nomTraitement,
                                             CMN_DAT_ModeleTraitement&
                                             modele)
        throw (ERR__Exception);
    void           RemplirEcheancier(CMN_DAT_ModeleTraitement& modele);
    void           EnvoyerStartHKTM(t_TableauDate& startTime, 
                                    t_TableauEntier& startOrbitNumber);
    t_CMN__Booleen TesterFinTraitement();
    void           TraiterFinTraitement(const t_CMN__Booleen finNominale,
                                        const t_Chaine infos = "",
 //FA 115 : attribut pour gerer les pb de donnees
					const int init = 2);
    t_CMN__Statut  EcrireProduits();
    void           TracerDebutTraitement(const t_Chaine traitement);
    void           TracerFinTraitement(const t_Chaine traitement,
                                       const SD_DON_Traitement::
                                       e_ListeStatus etat,
                                       const double elapsedTime,
                                       const double userTime);
    void           AfficherTaches();
    void           AfficherContextes();

    ////////////////////////////////
    // Attributs prives
    ////////////////////////////////
    SD_FRW_ApplicationServeurDonnees*  m_Application;
    t_CMN__Booleen                     m_EstActif;
    SD_FRW_FabriqueDeTache*            m_FabriqueDeTache;
    CMN_DAT_ListeObjets                m_ListeModelesTraitements;
    t_ListeTache                       m_Taches;
    t_MapContexte                      m_Contextes;

    // Timeout entre deux analyses de l'echeancier
    int                                m_PeriodeDeScrutation;

    int                                m_NbLignes;
    int                                m_NbRessourcesDispos;
    int                                m_NbRessourcesMax;

    t_CMN__Booleen                     m_Deprogramme;
};


#endif

