static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/FRW/src/SD_FRW_FabriqueAction.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_FRW
//@ CLASSE : SD_FRW_FabriqueAction
//@ 
//@ Commentaire :
//@    Ce module gere la classe SD_FRW_FabriqueAction
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Adaptation a l'OPS : ajout de la gestion des exceptions
//@ --------------------------------------------------------------------
//@ FAID<SMM-M4-FA60-CN>    20/01/00    Robert  Rajout de l'action export dans la fabrique
//@ --------------------------------------------------------------------
//@ FAID<SMM-M4-DM-20935-CN>  30/10/00 GRT      Prise en compte des nouveaux fichiers meteo 
//@ --------------------------------------------------------------------
//@ VERSION : 20/08/1999 : SAS
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------
//
// Fichiers include
//

// STL
#include <iostream>

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN__Fichier.h"
#include "CFG__Configuration.h"
#include "CMN_DAT_ModeleAction.h"
#include "SD_FRW_FabriqueAction.h"

// Actions
#include "SD_ACT_Action.h"
#include "SD_ACT_ActionDeTest.h"
#include "SD_GES_ChaineImageDeb.h"
#include "SD_GES_ChaineImageISRFEM.h"
#include "SD_GES_FiltrageAxeInterf.h"
#include "SD_GES_ChaineProd0_1C.h"
#include "SD_GES_ChaineProd1A_1C.h"
#include "SD_GES_ChaineProd1B_1C.h"


//
// Definition des constantes
//
const t_Chaine SD_FRW_FabriqueAction::FICHIER_MODELES_ACTIONS = "ModelesActions.cfg";


//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD_FRW_FabriqueAction
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_FRW_FabriqueAction::SD_FRW_FabriqueAction()
{
  //@ Charger la liste des modeles d'actions
  ChargerModelesActions();


  m_Actions["ChaineImageDeb"]    = 1;
  m_Actions["ChaineImageISRFEM"] = 2;
  m_Actions["FiltrageAxeInterf"] = 3;
  m_Actions["ChaineProd0_1C"]    = 4;
  m_Actions["ChaineProd1A_1C"]   = 5;
  m_Actions["ChaineProd1B_1C"]   = 6;
  m_Actions["ActionDeTest"]      = 7;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_FRW_FabriqueAction
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_FRW_FabriqueAction::~SD_FRW_FabriqueAction()
{
  //@ Supprimer la liste des actions
  m_Actions.clear();
}


//@ ------------------------------------------------------------------*/
//@ Methode : ChargerModelesActions
//@
//@ Description : Charge les modeles d'action connus
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_FabriqueAction::ChargerModelesActions() throw (ERR__Exception)
{
  t_Chaine fichierConfiguration("");


  //@ Ouvrir le fichier des modeles d'actions
  fichierConfiguration = CFG__Configuration::GetRepertoire(CONF_DIR);

  //@ SI la variable d'environnement REP_CONF_DIR n'est pas definie
  if (fichierConfiguration.empty())
    {
      ERR__Exception exception;
      
      //@ Emission d'une exception SD_ENV_UNDEFINED
      exception.SetErrorCode(ERR__Exception::SD_ENV_UNDEFINED);
      exception.SetParametre(CONF_DIR);
      THROW(exception);
    }
  //@ FIN SI
  
  fichierConfiguration += CMN__SEP_REPERTOIRE;
  fichierConfiguration += FICHIER_MODELES_ACTIONS;
  
  //@ Charger les differents modeles d'action
  //@ SI le chargement a ete incorrect
  if (m_ListeModelesActions.ChargerListe(fichierConfiguration) != 
      CMN__STATUT_OK)
    {
      ERR__Exception exception;
      
      //@ Emission d'une exception SD_FILE_OPEN_ERROR
      exception.SetErrorCode(ERR__Exception::SD_FILE_OPEN_ERROR);
      exception.SetParametre(FICHIER_MODELES_ACTIONS);
      THROW(exception);
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : CreerAction
//@
//@ Description : Cree une action a partir du mnemonique de l'action
//@               a realiser
//@
//@ Parametres :
//@               (ENTREE) action : Mnemonique Action a realiser
//@               (ENTREE) tache : Tache d'appartenance de l'action
//@
//@ Valeur de retour : 
//@               SD_ACT_Action : Action a realiser
//@
//@ ------------------------------------------------------------------*/
SD_ACT_Action* SD_FRW_FabriqueAction::CreerAction(t_Chaine& action, 
                                                  SD_TAC_Tache* tache)
  throw (ERR__Exception)
{
  mapActions::iterator iterateurAction;
  SD_ACT_Action*       nouvelleAction = NULL;
  int                  numeroAction = -1;


  //@ Selon le mnemonique de l'action, creer l'instance de classe
  //@ correspondante
  //@ Retrouver l'action
  iterateurAction = m_Actions.find(action);

  //@ SI l'action n'a pas ete retrouvee
  if (iterateurAction == m_Actions.end())
    {
      ERR__Exception exception;
      
      //@ Emission d'une exception SD_ACT_NOT_FOUND
      exception.SetErrorCode(ERR__Exception::SD_ACT_NOT_FOUND);
      exception.SetParametre(action);
      THROW(exception);
    }
  //@ FIN SI

  //@ Recuperer le numero de l'action
  numeroAction = (*iterateurAction).second;
  
  //@ Creer l'action
  switch (numeroAction) 
    {
    case 1: // ChaineImageDeb
      nouvelleAction = new SD_GES_ChaineImageDeb(tache);
      break;   
    case 2: // ChaineImageISRFEM
      nouvelleAction = new SD_GES_ChaineImageISRFEM(tache);
      break;
    case 3: // FiltrageAxeInterf
      nouvelleAction = new SD_GES_FiltrageAxeInterf(tache);
      break;   
    case 4: // ChaineProd0_1C
      nouvelleAction = new SD_GES_ChaineProd0_1C(tache);
      break;   
    case 5: // ChaineProd1A_1C
      nouvelleAction = new SD_GES_ChaineProd1A_1C(tache);
      break;
    case 6: // ChaineProd1B_1C
      nouvelleAction = new SD_GES_ChaineProd1B_1C(tache);
      break;   
    case 7: // ActionDeTest
      nouvelleAction = new SD_ACT_ActionDeTest(tache);
      break;   
   
    default :
      nouvelleAction = new SD_ACT_Action(tache);  // Action vide
      break;
    }

  nouvelleAction->NomAction(action);
  nouvelleAction->Simulation(RechercherSimulationAction(action));


  //@ Retourner l'action nouvellement creee
  return nouvelleAction;
}


//======================================================================
//                                      METHODES PRIVEES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : RechercherSimulationAction
//@
//@ Description : Recherche le parametre de simulation d'une action
//@
//@ Parametres :
//@               (ENTREE) mnemo : Action
//@
//@ Valeur de retour : 
//@               Valeur du parametre de simulation
//@
//@ ------------------------------------------------------------------*/
int SD_FRW_FabriqueAction::RechercherSimulationAction(t_Chaine& mnemo)
{
  int                  simulation = -1;
  int                  i = 0;
  CMN_DAT_ModeleAction modele;


  //@ Parcourir la liste des modeles jusqu'au modele dont le
  //@ mnemonique est fourni
  i = m_ListeModelesActions.RechercherElement(mnemo);

  //@ SI le modele a ete trouve
  if (i != -1) 
    {
      //@ On lit l'action
      m_ListeModelesActions.LireElement(i, modele);

      //@ On recupere la donnee simulation
      simulation = modele.Simulation();
    }
  //@ SINON on renvoie la valeur par defaut
  //@ FIN SI


  return simulation;
}
