static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/FRW/src/SD_FRW_FabriqueDeTache.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_FRW
//@ CLASSE : SD_FRW_FabriqueDeTache
//@ 
//@ Commentaire :
//@    Ce module gere la classe SD_FRW_FabriqueDeTache
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 20/08/1999 : SAS
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Adaptation a l'OPS : ajout de la gestion des exceptions
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL
#include <iostream>

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN__Fichier.h"
#include "CFG__Configuration.h"
#include "CMN_DAT_ModeleTache.h"
#include "SD_TAC_TacheTraitement.h"
#include "SD_FRW_FabriqueDeTache.h"


//
// Definition des constantes
//
const t_Chaine SD_FRW_FabriqueDeTache::FICHIER_MODELES_TACHES = "ModelesTaches.cfg";


//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD_FRW_FabriqueDeTache
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_FRW_FabriqueDeTache::SD_FRW_FabriqueDeTache()
{
  //@ Charger la liste des modeles de tache
  ChargerModelesTaches();

  //@ Creer une fabrique d'action
  m_FabriqueAction = new SD_FRW_FabriqueAction();
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_FRW_FabriqueDeTache
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_FRW_FabriqueDeTache::~SD_FRW_FabriqueDeTache()
{
  //@ Liberer les ressources allouees par la fabrique de tache

  //@ Supprimer la fabrique d'action
  if (m_FabriqueAction!= NULL) 
    {
      delete m_FabriqueAction;
    }
  m_FabriqueAction = NULL;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ChargerModelesTaches
//@
//@ Description : Charge les modeles de tache connus
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_FabriqueDeTache::ChargerModelesTaches() throw (ERR__Exception)
{
  t_Chaine fichierConfiguration("");


  //@ Ouvrir le fichier des modeles de taches
  fichierConfiguration = CFG__Configuration::GetRepertoire(CONF_DIR);

  //@ SI la variable d'environnement REP_CONF_DIR n'est pas definie
  if (fichierConfiguration.empty())
    {
      ERR__Exception exception;
      
      //@ Emission d'une exception SD_ENV_UNDEFINED
      exception.SetErrorCode(ERR__Exception::SD_ENV_UNDEFINED);
      exception.SetParametre(CONF_DIR);
      THROW(exception);
    }
  //@ FIN SI

  fichierConfiguration += CMN__SEP_REPERTOIRE;
  fichierConfiguration += FICHIER_MODELES_TACHES;

  //@ Charger les differents modeles de tache
  //@ Chaque modele est compose des mnemoniques ordonnes des
  //@ actions qui la composent
  //@ SI le chargement a ete incorrect
  if (m_ListeModelesTaches.ChargerListe(fichierConfiguration) != 
      CMN__STATUT_OK)
    {
      ERR__Exception exception;
      
      //@ Emission d'une exception SD_FILE_OPEN_ERROR
      exception.SetErrorCode(ERR__Exception::SD_FILE_OPEN_ERROR);
      exception.SetParametre(FICHIER_MODELES_TACHES);
      THROW(exception);
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : CreerTache
//@
//@ Description : Cree une tache a partir du type de travail a
//@               realiser
//@
//@ Parametres :
//@               (ENTREE) travail : Travail a realiser
//@
//@ Valeur de retour : 
//@               SD_TAC_Tache : Tache a realiser
//@
//@ ------------------------------------------------------------------*/
SD_TAC_Tache* SD_FRW_FabriqueDeTache::CreerTache(t_Chaine& travail)
{
  SD_TAC_Tache* tache = NULL;


  //@ Rechercher le modele de tache
  tache = RechercherTache(travail);


  //@ Retourner la tache creee
  return tache;
}


//======================================================================
//                                      METHODES PRIVEES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : RechercherTache
//@
//@ Description : Recherche une tache a partir de son modele
//@
//@ Parametres :
//@               (ENTREE) mnemo : Modele de tache
//@
//@ Valeur de retour : 
//@               SD_TAC_Tache : Tache a realiser
//@
//@ ------------------------------------------------------------------*/
SD_TAC_Tache* SD_FRW_FabriqueDeTache::RechercherTache(t_Chaine& mnemo)
  throw (ERR__Exception)
{
  SD_TAC_Tache*       tache = NULL;
  SD_ACT_Action*      action = NULL;
  int                 i;
  CMN_DAT_ModeleTache modele;
  t_Chaine            mnemoAction;


  //@ Parcourir la liste des modeles jusqu'au modele dont le
  //@ mnemonique est fourni
  i = m_ListeModelesTaches.RechercherElement(mnemo);
  //@ SI un modele n'a pas ete trouve
  if (i == -1) 
    {
      ERR__Exception exception;
      
      //@ Emission d'une exception SD_TAC_NOT_FOUND
      exception.SetErrorCode(ERR__Exception::SD_TAC_NOT_FOUND);
      exception.SetParametre(mnemo);
      THROW(exception);
    }
  //@ FIN SI

  //@ On lit les actions decrites dans le modele
  m_ListeModelesTaches.LireElement(i, modele);

  //@ Creer une tache de traitement
  tache = new SD_TAC_TacheTraitement();

  //@ POUR chaque action contenue dans le modele FAIRE
  for (i = 0; i<modele.NombreActions(); i++) 
    { 
      //@ Demander a la fabrique d'action de construire l'action
      mnemoAction = modele.Action(i).c_str();
      action = m_FabriqueAction->CreerAction(mnemoAction, tache);
    
      //@ Ajouter l'action a la tache de traitement
      tache->AjouterAction(action);
    }
  //@ FIN POUR

  // Ajout du temps de simulation
  tache->Simulation(modele.Simulation());


  //@ Retourner la tache nouvellement creee
  return tache;
}
