static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/FRW/src/SD_FRW_GestionnaireThread.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_FRW
//@ CLASSE : SD_FRW_GestionnaireThread
//@ 
//@ Commentaire :
//@    Ce module gere la classe SD_FRW_GestionnaireThread
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 10/08/1999 : SAS
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Adaptation a l'OPS
//@ --------------------------------------------------------------------
//@ FAID<SMM-M4-FA-743-CN>  01/12/00 Robert Le booleen d'indisponibilite d'un process est maintenant positionne avant son activation
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-46 et 53-CN : 17/07/2003 : JL PASCAL 
//@ dans CreerThread, le thread etait cree avec une taille de 96ko par defaut
//@ ors, les librairie MEtop ont besoin de plus !! on prend 256 ko
//@ --------------------------------------------------------------------
//@ FA-ID : souche SS	SALTO : 19/08/2003 : JL PASCAL 
//@ dans ThreadFn, on utilise un verrou ( bug issu gille robert sur le SD SD SSALTO)
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-184-CN : 18/10/2006 : C.MERIGUET
//@ Codage des fonctions reentrantes
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL
#include <iostream>

// Standard
#include <pthread.h>

// Projet
#include "CMN__Commun.h"
#include "SD_FRW_ApplicationServeurDonnees.h"
#include "SD_FRW_GestionnaireThread.h"


//
// Definition des constantes
//


//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD_FRW_GestionnaireThread
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               (ENTREE) noThreads : Nombre de threads a creer
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_FRW_GestionnaireThread::SD_FRW_GestionnaireThread(int noThreads) : 
  m_NbThreadsReels(noThreads)
{
#ifdef OPS_DEBUG
  time_t ltime;
 // FA 184
  char buff[26];

  time( &ltime );

  cout << "Gestionnaire de threads; lancement a: " 
 // FA 184
       << ctime_r(&ltime,buff) << endl;
  cout << "\tcreation de " << noThreads << " threads" << endl << endl;
#endif

  //@ Creer le nombre de contexte de threads approprie
  m_Threads = new SD_FRW_ContexteThread[m_NbThreadsReels];

  //@ Definir le nombre de threads virtuels
  m_NbThreadsVirtuels = 1;
  m_NbThreadsVirtuelsDisponibles = 1;

  //@ Creer un sujet d'interet sur la terminaison de tache
  m_SujetTerminaisonTache = new SD_EVT_SujetTerminaisonTache();

  //@ Creer les threads
  for (int i = 0; i < m_NbThreadsReels; i++)
    {
      CreerThread(i);
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_FRW_GestionnaireThread
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_FRW_GestionnaireThread::~SD_FRW_GestionnaireThread()
{
  //@ Detruire les threads
  delete [] m_Threads;

  //@ Detruire le sujet d'interet
  delete m_SujetTerminaisonTache;
}


//@ ------------------------------------------------------------------*/
//@ Methode : LancerTache
//@
//@ Description : Affecte un thread a une tache a lancer
//@
//@ Parametres :
//@               (ENTREE) tache : Tache a lancer
//@
//@ Valeur de retour : 
//@               CMN__VRAI si la tache a pu etre lancee
//@               CMN__FAUX si aucun thread n'est disponible, la tache
//@                    n'a pas pu etre lancee
//@
//@ ------------------------------------------------------------------*/
t_CMN__Booleen SD_FRW_GestionnaireThread::LancerTache(SD_TAC_Tache* tache)
{
  //@ POUR chaque thread FAIRE
  for (int i = 0; i < m_NbThreadsReels; i++)
    {
      //@ SI le thread est disponible ALORS
      if (m_Threads[i].VerifierConditionDeDisponibilite() == CMN__VRAI)
        {
          t_Date dateDebut;


#ifdef OPS_DEBUG
          cout << "Tache " << tache->Id() 
               << " geree par le thread: " << i << endl;
          if (tache->Contexte() != 0) 
            {
              cout << "\tcontexte defini" << endl;
            } 
          else 
            {
              cout << "\tcontexte = NIL" << endl;
            }
#endif
              
          //@ Lier la tache a son application parente
          tache->Application(Application());
          
          //@ Mettre a jour la date de debut
          dateDebut.DateCourante();
          tache->DateDebut(dateDebut);
          
          //@ Affecter la tache au thread
          m_Threads[i].Tache(tache);
              
          //@ Signaler au thread qu'il doit bosser...
          m_Threads[i].SignalerConditionDeTravail(CMN__VRAI);
          
          //@ Attente
          usleep(500000);
          //sleep(1);
          
          //@ Quitter la boucle de scrutation Condition reussie
          return CMN__VRAI;
        }
      //@ FIN SI
    }
  //@ FIN POUR

  
  //@ Retourner le statut de lancement de tache
  return CMN__FAUX;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ThreadFn
//@
//@ Description : Template de fonction multi-thread selon le standard 
//@               POSIX
//@
//@ Parametres :
//@               (ENTREE) lparam : Parametres du thread
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
extern "C" {
  void *ThreadFn(void *lparam)
  {
    SD_FRW_ContexteThread*        pdata = (SD_FRW_ContexteThread*)
      lparam;
    SD_TAC_Tache*                 tache = NULL;
    SD_EVT_SujetTerminaisonTache* sujet = NULL;    
    
    //@ POUR toujours FAIRE
    while (CMN__VRAI)
      {
#ifdef OPS_DEBUG
        cout << "Thread en attente: " << pdata->Id() << endl;
#endif

        //@ Attendre une condition de travail
        pdata->AttendreConditionDeTravail();

#ifdef OPS_DEBUG
        cout << "Thread active    : " << pdata->Id() << endl;
#endif

        //@ Verifier si une condition de terminaison ne s'est pas 
        //@ produite
        //@ Si oui quitter la ronde infernale
        if (pdata->VerifierConditionDeTerminaison() == CMN__VRAI)
          break;

        //@ Recuperer la tache passee en parametre
        tache = (SD_TAC_Tache*) pdata->Tache();

        //@ Tracer le demarrage de la tache
        tache->TracerDemarrage();

        //@ Executer la tache
        tache->Executer();

        //@ Tracer le fin de la tache
        tache->TracerFin();

        //@ Notifier la terminaison de la tache associee au thread
        sujet = (SD_EVT_SujetTerminaisonTache *) pdata->Sujet();

// bug SD detecte sur le SD SSALTO 
        SD_FRW_ApplicationServeurDonnees *pApplication = tache->Application();
        pApplication ->Verrouiller();
        sujet->Tache(tache);
        sujet->NotifierFinDeTache();
        pApplication->Deverrouiller();

        //@ Verifier si une condition de terminaison ne s'est pas 
        //@ produite
        //@ Si oui quitter la ronde infernale
        if (pdata->VerifierConditionDeTerminaison() == CMN__VRAI) 
          break;

        //@ Signaler que le thread n'a plus de travail
        pdata->SignalerConditionDeTravail(CMN__FAUX);
        

        sleep(0);
      }
    //@ FIN POUR
    
#ifdef OPS_DEBUG
    cout << "Thread died: " << pdata->Id() << endl;
#endif
    
    
    //@ Retourner que le thread s'est termine avec succes
    return NULL;
  }
}


//@ ------------------------------------------------------------------*/
//@ Methode : CompterLesTachesEnCours
//@
//@ Description : Compte les taches en cours
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Nombre de taches en cours
//@
//@ ------------------------------------------------------------------*/
int SD_FRW_GestionnaireThread::CompterLesTachesEnCours()
{   
  int tachesEnCours = 0;


  //@ POUR tous les threads: cumuler ceux dans l'etat indisponible
  for (int i = 0; i < m_NbThreadsReels; i++)
    {
      //@ SI le thread est indisponible ALORS
      if (m_Threads[i].VerifierConditionDeDisponibilite() == CMN__FAUX)
        {
          //@ Incrementer le nombre de taches en cours
          tachesEnCours++;
        }
      //@ FIN SI
    }
  //@ FIN POUR tous les threads

    
  //@ Retourner le nombre de taches en cours
  return tachesEnCours;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ArreterTache
//@
//@ Description : Arrete une des taches en cours
//@
//@ Parametres :
//@               (ENTREE) id : Numero d'identification de la tache
//@
//@ Valeur de retour : 
//@               0 si demande d'arret a ete prise en compte
//@               -1 si la tache n'est pas en cours d'execution 
//@
//@ ------------------------------------------------------------------*/
int SD_FRW_GestionnaireThread::ArreterTache(int id)
{   
  // Valeur de retour de la methode
  int status = -1;


  //@ Rechercher la tache a partir de son numero d'identification
  //@ Le premier thread est le gestionnaire de session;
  //@ il n'est pas concerne

  //@ POUR tous les autres threads
  for (int i=1; i < m_NbThreadsReels; i++)
    {
      //@ SI le thread est occupe ALORS
      if (m_Threads[i].VerifierConditionDeDisponibilite() == CMN__FAUX)
        {
          //@ Verifier si l'ID de tache correspond
          //@ SI oui ALORS
          if (id == m_Threads[i].Tache()->Id()) 
            {
              //@ Indiquer que le tache doit s'arreter
              m_Threads[i].Tache()->Arreter();
              status = 0;
            }
          //@ FIN SI
        }
      //@ FIN SI
    }
  //@ FIN POUR tous les autres threads


  //@ Renvoyer la valeur de retour de la methode
  return status;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ArreterToutesLesTaches
//@
//@ Description : Arrete les taches en cours
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_GestionnaireThread::ArreterToutesLesTaches()
{   
  //@ Le premier thread est le gestionnaire de session;
  //@ il n'est pas concerne
  //@ POUR tous les autres threads
  for (int i=1; i < m_NbThreadsReels; i++)
    {
      //@ SI le thread est occupe ALORS
      if (m_Threads[i].VerifierConditionDeDisponibilite() == CMN__FAUX)
        {
          //@ Indiquer que le tache doit s'arreter
          m_Threads[i].Tache()->Arreter();
        }
      //@ FIN SI
    }
  //@ FIN POUR
}


//======================================================================
//                                      METHODES PRIVEES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : CreerThread
//@
//@ Description : Cree un thread POSIX
//@
//@ Parametres :
//@               (ENTREE) noThread : Numero du thread a creer
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_FRW_GestionnaireThread::CreerThread(int noThread)
{   
  pthread_t       tid;
  pthread_attr_t  attr;


  //@ Affecter un identificateur au thread
  m_Threads[noThread].Id(noThread);
  
  //@ Affecter un sujet d'interet au thread
  m_Threads[noThread].Sujet(m_SujetTerminaisonTache);

  //@ Creer un thread POSIX gere par le systeme
  pthread_attr_init(&attr);
  if (pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM) != 0)
    {
      cerr << "Error pthread_attr_setscope..." << endl;
    }

// modification de la taille de la pile pour les librairies METOP
// FT-2765
#ifdef AIX
  size_t stacksize=256*1024;
#else
#ifdef OPS_PTHREAD_STACK_SIZE
	size_t stacksize=OPS_PTHREAD_STACK_SIZE * 1024;
#else
  	size_t stacksize=10*1024*1024;
#endif
#endif
// ...FT-2765
  pthread_attr_setstacksize (&attr, stacksize);
  pthread_create(&tid, &attr, ThreadFn, &m_Threads[noThread]);
  m_Threads[noThread].ThreadHandle(tid);
}
