static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/GES/src/SD_GES_ChaineImageDeb.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_GES
//@ CLASSE : SD_GES_ChaineImageDeb
//@ 
//@ Comment :
//@    this module contains the first action used for the IASI OPS 
//@     L1 Processing chain level 0-1C
//@                                     
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 10/03/2003 : P.GASSIE
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-32-CN : 07/07/2003  : jl PASCAL.  initialisation BBT dans le contexte
//@ --------------------------------------------------------------------
//@ DM-ID : PM4 : 31/03/2004 : B. MASSART
//@ SD_GES_ChaineImageDeb::Executer:
//@ -date du SN=15 pour 30_FTB
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-143-CN : 21/09/2004 : B.MASSART
//@ L'OPS s'arrete sur l'initialisation de VIADRENG_MEXSSMIN avec le contexte 
//@ a froid dans SD_GES_ChaineProd0_1C::Corpstraitement lorsque la 10ieme ligne 
//@ est un DMDR. On opte pour une gestion des VIADR_DUMMY similaire a celle des 
//@ VIADR_ENG valides. Voir IA-2100-FC-143-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-169-CN : 27/02/2006 : B.MASSART
//@ Syntax correction for IisInHomPix. See IA-2100-FC-169-THA.
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-196-CN : 15/01/2007 : C.MERIGUET
//@ FA-ID : IA-2100-FA-198-CN : 15/01/2007 : C.MERIGUET
//@ Ces 2 FA sont traitees conjointement. Recuperation du PTSI lu dans 
//@ le mot 17 (2 octets) mais ecriture dans le champ PTSI des N1 des mots 
//@ 16 (version logiciel bord) et 17 (ptsi) (4 octets).
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL
#include <iostream>

// Standard
#include <limits.h>

// Projet
#include "CMN__Commun.h"
#include "CMN__Chaine.h"

#include "SD_TAC_Tache.h"
#include "SD_GES_ChaineImageDeb.h"
#include "SD_FRW_ApplicationServeurDonnees.h"
#include "SD_ALG_Image.h"
#include "SD_ALG_Spectre.h"
#include "SD_ALG_Monitoring.h"


//======================================================================
//                                       METHODES PUBLIQUES 
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD_GES_ChaineImageDeb
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               (ENTREE) tache : Tache pere de l'action
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_GES_ChaineImageDeb::SD_GES_ChaineImageDeb(SD_TAC_Tache* tache) 
  : SD_ACT_Action(tache)
{
  //@ Memoriser la tache parente de l'action
  m_Tache = tache;
  m_PeutEtreEnchainee = CMN__FAUX;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_GES_ChaineImageDeb
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_GES_ChaineImageDeb::~SD_GES_ChaineImageDeb()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Executer
//@
//@ Description : Traitement principal de l'action
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_GES_ChaineImageDeb::Executer()
{
t_CMN__Statut cr_op = CMN__STATUT_OK;
  //@ Methode virtuelle surchargee pour executer la premiere partie de 
  //@ la chaine  image
  cout << "SD_GES_ChaineImageDeb pour la tache " << Tache()->Id() 
       << endl;

  //@ principes generaux a appliquer :
  //@ 0 - on considere que l'initialisation des traces a ete faite au prealable
  //@     dans  InitGranule, la structure t_SD_BAS_DataGen est passe aux 
  //@ algorithmes qui emettent eux meme leur traces
  //@ 1 - c'est a ce niveau que sont emis les messages au JdBS (  
  //@    trace  applicatives a partir de la structure t_SD_BAS_DataThread )
  //@  via : la remontee d'erreurs des algo
  //@ 2 - la memoire utilisee est de 2 type :
  //@   - partagee entre thread elle a alors ete allouee au prealable 
  //@   - locale au traitement, elle est alors allouee en debut puis liberee
  //@     en fin de methode 

//@ recuperer le serveur de donnees reference dans application serveur de donnee/
  SD_FRW_ApplicationServeurDonnees* application;
  application = m_Tache->Application();
//@----------------------------------------------------------------------*/
//@ recuperer les donnees temporaire globales initialisees */
//@ appel GetDonGenGranule 
//@ contexte + donnes generale YY_CONF
//@----------------------------------------------------------------------*/
  SD_DON_GestDonnees* gestionnaire;
  t_SD_BAS_Contexte* contextecourant;
//  contextecourant = new t_SD_BAS_Contexte*[1];
  t_SD_BAS_DataGen  * datagen ;
//  t_SD_BAS_DataGen  ** datagen = new t_SD_BAS_DataGen*[1];
  gestionnaire = application->Donnees();
  gestionnaire->GetDonGenGranule(&contextecourant, &datagen);
  //@ Recuperation du gestionniare des cas degrades
  SD_DON_GestCasDeg gestCasDeg = gestionnaire->GetGestCasDeg();
  
  //@ Nombre de lignes du granule
  int NbLignesGranule = gestionnaire->GetNbLignesIASI();
  int PTSI0;
  // @ Nombre de lignes valides du granul
  int NbLignesValides = 0;
  t_Date datePremiereLigne;
  
  //@ Filtrage des temperatures du corps noir chaud
  //@POUR chaque ligne du granule ( appel GetNbLignesIASI)
  for(int i = 1; i <= NbLignesGranule; i++)
  {
    t_Date dateTmp;
    //@ recuperation ENG
    t_SD_BAS_MPHR * MphrENG;
    t_SD_BAS_LigneN1 * LigneMDRENG;
    t_SD_BAS_LigneN1 * LigneVIADRENG;
    MphrENG        = new t_SD_BAS_MPHR;
    LigneMDRENG    = new t_SD_BAS_LigneN1;
    LigneVIADRENG  = new t_SD_BAS_LigneN1;
    gestionnaire->GetLigneS1(
      i - 1, 
      SD_DON_GranuleN1::SD_DON_GRANULE_N1_ENG, 
      *MphrENG, 
      *LigneMDRENG, 
      *LigneVIADRENG);
    
    t_SD_BAS_HistTempCN TempCN;
    // cas des DMDR
    if (LigneMDRENG->TypeRec != MDR_DUMMY)
    {
      NbLignesValides++;
      //@ initialisation de la structure t_SD_BAS_HistTempCN
      if (i == 1)
      {
        cout << " pdate->day = " 
          << LigneMDRENG->InfoDet.tab_var[ENG_DATEDEB].val.pdate->day 
          << endl;
        cout << " pdate->millis= " 
          << LigneMDRENG->InfoDet.tab_var[ENG_DATEDEB].val.pdate->millis 
          << endl;
        cout << " BIMSBBT = " 
          << LigneMDRENG->InfoDet.tab_var[ENG_BIMSBBT].val.pr8[0]
          << endl;
      }
      TempCN.BIMSBBT = LigneMDRENG->InfoDet.tab_var[ENG_BIMSBBT].val.pr8[0];
      
      /* DM 11/03/2004 pt 1.17 pour 30_FTB */
      TempCN.GBBTDateElt.day = LigneMDRENG->InfoDet.tab_var[ENG_GEPSDATIASI].val.pdate[IND1(SD_BAS_SNREFDATELIGNE)].day;
      TempCN.GBBTDateElt.millis = LigneMDRENG->InfoDet.tab_var[ENG_GEPSDATIASI].val.pdate[IND1(SD_BAS_SNREFDATELIGNE)].millis;
      
// initialisation de la temperature du corps noir chaud dans le contexte
      contextecourant->BIMSBBT[0]=TempCN.BIMSBBT;
    
      //@ SI (CG5)
      if(gestCasDeg.GetCasDegLigne(i)->IsDetecte(5, 0, 0, 0, dateTmp))
      {
        //@ BBTMissing pour la temperature = oui
        TempCN.GBBTMissing = CMN__OUI;
      }
      //@ SINON
      else
      {
        //@ BBTMissing pour la temperature = non
        TempCN.GBBTMissing = CMN__NON;
      }
      //@ FIN SI
      //@ Recuperation du PTSI 
// FA 196 et 198
      // inversion des mots, formule incorrecte? 
      //PTSI0 = (int) (LigneMDRENG->InfoDet.tab_var[ENG_PTSI].val.pui4[0]/65536);
      int version_log = (int) (LigneMDRENG->InfoDet.tab_var[ENG_PTSI].val.pui4[0]/65536);
      PTSI0 = LigneMDRENG->InfoDet.tab_var[ENG_PTSI].val.pui4[0] - (version_log*65536);
// ..FA 196 et 198
      datePremiereLigne.SCDSTime(LigneMDRENG->InfoDet.tab_var[ENG_DATEDEB].val.pdate[0]);
      //@ Mise a jour contexte avec les temperatures de chaque ligne
      t_SD_BAS_HistAxeInterf** AxeInterf = NULL;
      gestionnaire->AjouteElemHist(AxeInterf, &TempCN);
    }
    // fin cas des DMDR

    delete MphrENG;
    delete LigneMDRENG;
    delete LigneVIADRENG;
  }
  //FIN POUR

  //@ initialisation ctx
  t_SD_BAS_DataThread *ctx;
  ctx = new t_SD_BAS_DataThread;
  ctx->ctxtrt.LN=0;
  ctx->ctxtrt.SN=0;
  ctx->ctxtrt.PN=0;
  ctx->ctxtrt.CCD=0;
  ctx->ctxtrt.SB=0;
  InitialiserCompteRendu(&(ctx->cr));

// recup PTSI0 dans la boucle precedente 

  //@ recuperer les donnees de configuration en fonction du PTSI
  t_SD_BAS_ConfAutre *  ConfAutre0 ;
  t_SD_BAS_ConfStable * ConfStable0 ;
  t_SD_BAS_BanqueSpectrale * BanqueSpectrale0 ;
  
  if(NbLignesValides > 0)
  {
    gestionnaire->GetConf(
      PTSI0, 
      datePremiereLigne, 
      &ConfAutre0, 
      &ConfStable0, 
      &BanqueSpectrale0);
      
    //@ initialisation t_SD_BAS_DataThreadProduit
    t_SD_BAS_DataThreadProduit* InfoCalcLig;
    InfoCalcLig = new t_SD_BAS_DataThreadProduit;
    InfoCalcLig->A = new double[(ConfAutre0->IDefFtbRegressDegre+1) * (NbLignesGranule + ConfAutre0->IDefFtbFLS)];
    InfoCalcLig->B = new double[NbLignesGranule + ConfAutre0->IDefFtbFLS];
    InfoCalcLig->tabTmpMC = new double[(ConfAutre0->IDefFtbRegressDegre+1) * (NbLignesGranule + ConfAutre0->IDefFtbFLS)];
    InfoCalcLig->X = new double[ConfAutre0->IDefFtbRegressDegre + 1];
    
    //@ Call 30_FTB
    cr_op = SD_ALG_30_FTB(
      ctx, 
      NbLignesValides, 
      contextecourant, 
      ConfAutre0, 
      InfoCalcLig, 
      datagen);
      
    //@ traitement du compte-rendu
    Traiter(cr_op, ctx);
    delete [] InfoCalcLig->A;
    delete [] InfoCalcLig->B;
    delete [] InfoCalcLig->tabTmpMC;
    delete [] InfoCalcLig->X;
    delete InfoCalcLig;
  }
  
  double* slope = new double[SD_BAS_IMLI * SD_BAS_IMCO];
  double* offset = new double[SD_BAS_IMLI * SD_BAS_IMCO];



  //@POUR chaque ligne du granule
  int nbnondummy=0;
  for(int ln = 0; ln < NbLignesGranule; ln++)
  {
    //@ recuperer les donnees de niveau 0 de type IP
    t_SD_BAS_PaquetN0** ListePaquetIP;
    int NbPaquetsIP = gestionnaire->GetNbPaquetsN0(ln, IP);
    ListePaquetIP = new t_SD_BAS_PaquetN0*[NbPaquetsIP];
    gestionnaire->GetLigne0(ln, IP, NbPaquetsIP, ListePaquetIP);
    //@ recuperer les donnees de niveau 1 initialisees
    //@ 1ABC
    t_SD_BAS_MPHR * Mphr1ABC;
    t_SD_BAS_LigneN1 * LigneMDR1ABC;
    t_SD_BAS_LigneN1 * LigneVIADR1ABC;
    Mphr1ABC        = new t_SD_BAS_MPHR;
    LigneMDR1ABC    = new t_SD_BAS_LigneN1;
    LigneVIADR1ABC  = new t_SD_BAS_LigneN1;
    gestionnaire->GetLigneS1(
      ln, 
      SD_DON_GranuleN1::SD_DON_GRANULE_N1_DATA, 
      *Mphr1ABC, 
      *LigneMDR1ABC, 
      *LigneVIADR1ABC);

    //@ ENG
    t_SD_BAS_MPHR * MphrENG;
    t_SD_BAS_LigneN1 * LigneMDRENG;
    t_SD_BAS_LigneN1 * LigneVIADRENG;
    MphrENG        = new t_SD_BAS_MPHR;
    LigneMDRENG    = new t_SD_BAS_LigneN1;
    LigneVIADRENG    = new t_SD_BAS_LigneN1;
    gestionnaire->GetLigneS1(
      ln, 
      SD_DON_GranuleN1::SD_DON_GRANULE_N1_ENG, 
      *MphrENG, 
      *LigneMDRENG, 
      *LigneVIADRENG); 
   // cas des DMDR
    if (LigneMDRENG->TypeRec != MDR_DUMMY)
    {


// initialisation de la temperature du corps noir chaud filtree et indicateurs associes
     LigneMDRENG->InfoDet.tab_var[ENG_GFTBFILTEREDBBT].val.pr8[0]=datagen->GFtbFilteredBBT[NbLignesValides - nbnondummy - 1];
     LigneMDRENG->InfoDet.tab_var[ENG_GFTBBBTRES].val.pr8[0]=datagen->GFtbBBTRes;
     LigneMDRENG->InfoDet.tab_var[ENG_GFTBFLAGBBTNONQUAL].val.pui4[0]=(unsigned int)datagen->GFfbFLagBBTNonQual;
    //@recuperation offset et slope
     //@  on recupere offset et slope a partir du contexte
      for(int i = 0; i < SD_BAS_IMLI * SD_BAS_IMCO; i++)
      {
          slope[i] = contextecourant->GIccRadCalSlopeImag[i];
          offset[i] = contextecourant->GIccRadCalOffsetImag[i];
      }

    //@ Recuperation du PTSI
// FA 196 et 198
    // inversion des mots 
    int version_log_bord = (int) (LigneMDRENG->InfoDet.tab_var[ENG_PTSI].val.pui4[0]/65536);
    int PTSI = LigneMDRENG->InfoDet.tab_var[ENG_PTSI].val.pui4[0] - (version_log_bord*65536);
// ..FA 196 et 198
    //@ recuperer les donnees de configuration en fonction du PTSI
    t_Date tcourante;
    t_SD_BAS_ConfAutre *  ConfAutre ;
    t_SD_BAS_ConfStable * ConfStable ;
    t_SD_BAS_BanqueSpectrale * BanqueSpectrale ;
    t_Date datePremiereLigne;
    datePremiereLigne.SCDSTime(LigneMDRENG->InfoDet.tab_var[ENG_DATEDEB].val.pdate[0]);
    gestionnaire->GetConf(PTSI, datePremiereLigne, &ConfAutre, &ConfStable, &BanqueSpectrale);
    
// realisation du corps du traitement image  pour une ligne dans une fonction specifique
     Corpstraitement(
		ln,
		&NbLignesValides,
		&nbnondummy,
		gestionnaire,
  		contextecourant,
 		datagen,
		gestCasDeg,
		NbPaquetsIP,
		ListePaquetIP,							
 		LigneMDRENG,
 		LigneVIADRENG,
 		ConfAutre ,
  		ConfStable ,
  		BanqueSpectrale ,
		slope,
		offset,
		ctx
    		);
    


    }
    
    
    delete [] ListePaquetIP;
    delete Mphr1ABC;
    delete LigneMDR1ABC;
    delete LigneVIADR1ABC;
    delete MphrENG;
    delete LigneMDRENG;
    delete LigneVIADRENG;
    //@ liberation pointeurs dans   t_SD_BAS_DataThreadImageDeb
  }
  //@ FIN POUR chaque ligne du granule

// decoupage


  delete [] slope;
  delete [] offset;
  
  //@ liberation tableaux temporaires image
  delete ctx;

  //@ numero de ligne = issu du contexte + numero ligne local
  //@ maj du contexte avec le numero derniere ligne
  //@ remarque dans le cas ou c'est le premier granule le numero de ligne est
  //@ mis a 1
  contextecourant->GCTXNumDerLig[0] = 
    contextecourant->GCTXNumDerLig[0] + NbLignesGranule;
  

  //@ Mise a jour contexte
  
  //@ Invoquer la verification des symptomes une fois l'action
  //@ executee
  VerifierSymptome();
}


//@ ------------------------------------------------------------------*/
//@ Methode : VerifierSymptome
//@
//@ Description : Verifie si les symptomes associes a l'action sont
//@               en erreur ou non
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_GES_ChaineImageDeb::VerifierSymptome()
{
  //@ Methode virtuelle a surcharger dans les classes derivees
  //@ Les symptomes controles sont specifiques a l'action
        
  m_PeutEtreEnchainee = CMN__VRAI;
}


//======================================================================
//                                      METHODES PRIVEES 
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Methode : Corpstraitement
//@
//@ Description : algorithme principal de l'action
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_GES_ChaineImageDeb::Corpstraitement(
				 const int ln,
				 int * NbLignesValides,
				 int * nbnondummy,
			         SD_DON_GestDonnees* gestionnaire,
  				 t_SD_BAS_Contexte* contextecourant,
 				 t_SD_BAS_DataGen  * datagen,
				 SD_DON_GestCasDeg & gestCasDeg,
				const int NbPaquetsIP,
				 t_SD_BAS_PaquetN0** ListePaquetIP,							
 				 t_SD_BAS_LigneN1 * LigneMDRENG,
 				 t_SD_BAS_LigneN1 * LigneVIADRENG,
 				 t_SD_BAS_ConfAutre *  ConfAutre ,
  				 t_SD_BAS_ConfStable * ConfStable ,
  				 t_SD_BAS_BanqueSpectrale * BanqueSpectrale ,
				 double* slope,
				 double* offset,
				 t_SD_BAS_DataThread *ctx
    			)
{
t_CMN__Statut cr_op = CMN__STATUT_OK;

    
     //@ Initialisation des donnees /ligne
    t_Date dateTmp;
    //@ allocation pointeurs dans   t_SD_BAS_DataThreadImageDeb
    t_SD_BAS_DataThreadImageDeb* ImageDeb = NULL;
    ImageDeb = new t_SD_BAS_DataThreadImageDeb;
    ImageDeb->Wn = new double[ConfStable->IDefIISSrfNslast - ConfStable->IDefIISSrfNsfirst + 1];
    ImageDeb->Slope = new double[ConfAutre->IDefIISNbCol * ConfAutre->IDefIISNbLin];
    ImageDeb->Offset = new double[ConfAutre->IDefIISNbCol * ConfAutre->IDefIISNbLin];
    double * tableauImage35 = new double[SD_BAS_IMLI * SD_BAS_IMCO + 1];
    double * tableauImage36 = new double[SD_BAS_IMLI * SD_BAS_IMCO + 1];
    double * tableauImage32 = new double[SD_BAS_IMLI * SD_BAS_IMCO + 1];
    double * tableauImage33 = new double[SD_BAS_IMLI * SD_BAS_IMCO + 1];
    //  initialisations
    for (int j=0 ; j<SD_BAS_IMCO *  SD_BAS_IMLI ;j++)
        {
        LigneMDRENG->InfoDet.tab_var[ENG_GICCRADCALSLOPEIMAG].val.pr8[j]=0.;
        LigneMDRENG->InfoDet.tab_var[ENG_GICCRADCALOFFSETIMAG].val.pr8[j]=0.;
        }
    //@ SI pas (CG2) et pas (CG12 )
    if(!gestCasDeg.GetCasDegLigne(ln + 1)->IsDetecte(12, 0, 0, 0, dateTmp)
        && !gestCasDeg.GetCasDegLigne(ln + 1)->IsDetecte(2, 0, 0, 0, dateTmp))
    {
      //@ Si pas ( CG7 ou CG8 )
      if(!gestCasDeg.GetCasDegLigne(ln + 1)->IsDetecte(7, 0, 0, 0, dateTmp)
         && !gestCasDeg.GetCasDegLigne(ln + 1)->IsDetecte(8, 0, 0, 0, dateTmp))
      {
        //@ decodage du train binaire  image dans des tableaux temporaires
        //@ appel DecodeDataLigne0 pour les images 35 36 et 32 33 
        //@ POUR tous les paquets IP
        for(int paquet = 0; paquet < NbPaquetsIP; paquet++)
        {
          int numeroSN;
          //@ on regarde le numero  de SN
          numeroSN = ListePaquetIP[paquet]->InfoGen.tab_var[N0_SN].val.pui4[0];

          //@ SI numeroSN = 35
          if(numeroSN == 35)
          {
            //@ DecodeDataLigne0 pour image 35
            //@ premier champ : equal count, ensuite image
            gestionnaire->DecodeDataLigne0(ListePaquetIP[paquet], tableauImage35);
          }
          else if(numeroSN == 36) 
          {
            //@ DecodeDataLigne0 pour image 36
            gestionnaire->DecodeDataLigne0(ListePaquetIP[paquet], tableauImage36);
          }
          else if(numeroSN == 32)
          {
            //@ DecodeDataLigne0 pour image 32
            gestionnaire->DecodeDataLigne0(ListePaquetIP[paquet], tableauImage32);
          } 
          else if(numeroSN == 33)
          {
            //@ DecodeDataLigne0 pour image 33
             gestionnaire->DecodeDataLigne0(ListePaquetIP[paquet], tableauImage33);
          }
          //@ FIN SI
        }
        //@ FIN POUR
        //@ calculer les coefficients de calibration des images (A1323)
        ctx->ctxtrt.LN = ln + 1;
         //@ Call 38_ICC
         SD_ALG_38_ICC (
           ctx, 
           ImageDeb, 
           ConfAutre,
           ConfStable,
           datagen->GFtbFilteredBBT[*NbLignesValides - (*nbnondummy+1)], 
           &tableauImage35[1], // car premier champ equal count
           &tableauImage36[1],
           &tableauImage32[1],
           &tableauImage33[1],
           (int)tableauImage36[0],
           contextecourant->BIISEqualCountOld[0],
           offset,
           slope,
           contextecourant,
           LigneMDRENG->InfoDet);
 
        //@ traitement du compte-rendu
         Traiter(cr_op, ctx);
         
        //@ calculer la table des pixels morts (A1322)
        //@ Call 110_DPT
         cr_op = SD_ALG_110_DPT(ctx,
                                contextecourant,
                                ConfAutre,
                                ConfStable,
                                &tableauImage36[1],
                                contextecourant->MDptVarImag,
                                LigneMDRENG->InfoDet); 
        //@ traitement du compte-rendu
         Traiter(cr_op, ctx);
      }
      //@ FIN SI pas ( CG7 ou CG8 )
    }
    //@ FIN SI pas (CG2) et pas (CG12 )
 
          //@ SI numero de ligne absolu modulo 10 = 1
    /* FA143 */
    if((contextecourant->GCTXNumDerLig[0] + ln +1 )%PERIODE_VIADR == 0)
    /* FIN FA143 */
     {

     //@ on sauvegarde le VIADR a partir du contexte et/ou on reinitialise
      for(int i = 0; i < (SD_BAS_IMLI * SD_BAS_IMCO); i++)
        {
          /* FA169 */
          LigneVIADRENG->InfoDet.tab_var[VIADRENG_MDPTIISINHOMPIX].val.pui4[i] = (unsigned int)contextecourant->MDptIisInHomPix[i]%256;
          LigneVIADRENG->InfoDet.tab_var[VIADRENG_MDPTIISBADHEALTHPIX].val.pui4[i] = (unsigned int)contextecourant->MDptIisBadHealthPix[i];
//          contextecourant->MDptIisInHomPix[i]=0;
          contextecourant->MDptIisBadHealthPix[i]=0;
        }
      }
      //@ FIN SI


     delete [] ImageDeb->Wn;
     delete [] ImageDeb->Slope;
     delete [] ImageDeb->Offset;
     delete ImageDeb;
     delete [] tableauImage35;
     delete [] tableauImage36;
     delete [] tableauImage32;
     delete [] tableauImage33;
    (*nbnondummy)++;

}
//@ ------------------------------------------------------------------*/
//@ Methode : Traiter
//@
//@ Description : Permet de traiter les ctx
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@
//@ ------------------------------------------------------------------*/
void SD_GES_ChaineImageDeb::Traiter(t_CMN__Statut& cr_op,
                                    t_SD_BAS_DataThread *ctx
                                   )
{
  //@ SI erreur
  if(cr_op != CMN__STATUT_OK)
  {
    //@ emission des messages du ctx
    EnvoyerMessagesStockes(&(ctx->cr.msgJDB));
    InitialiserListeErreurs(&(ctx->cr.errinterne));
  }
}
