static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/GES/src/SD_GES_ChaineImageISRFEM.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_GES
//@ CLASSE : SD_GES_ChaineImageISRFEM
//@ 
//@ Comment :
//@    this module contains the second action used for the IASI OPS 
//@     L1 Processing chain level 0-1C ( multi-threaded bu line)
//@                                     
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 10/03/2003 : P.GASSIE
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-078-CN : 02/10/2003 : B. MASSART
//@ on verifie que le MDR n'est pas DUMMY avant d'afficher des infos detaillees
//@ dans SD_GES_ChaineImageISRFEM::Executer.
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-072-CN : 15/10/2003 : B. MASSART
//@ si au moins une des 3 bandes spectrales contient au moins un over/underflow
//@ et qu'au moins une bande n'en contient pas, on permet le traitement 
//@ de cette derniere. Dans SD_GES_ChaineImageISRFEM::Executer, correction 
//@ d'une coquille et ajout du booleen FenetreSpectraleValide qui autorise 
//@ ou non le traitement par 22_SOS et 23_SSD de la fenetre spectrale si 
//@ celle-ci ne contient pas d'over/underflow. (cf IA-2100-FC-072-THA)
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-084-CN : 13/11/2003 : JL. PASCAL
//@ optimisation du code ( les init de fft  sont fait une fois par ligne );
//@ pour SOS et SSD
//@ fiche de correction correspondante : IA-2100-FC-084-THA
//@ --------------------------------------------------------------------
//@ DM-ID : PM4 : 31/03/2004 : B. MASSART
//@ SD_GES_ChaineImageISRFEM::Executer:
//@ -date de SN=15 pour 25_FAX et 21_SSS
//@ SD_GES_ChaineImageISRFEM::InitialiserNiveau1:
//@ -correction syntaxe N1C_IDEFCCSCHANNELID
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-129-CN : 22/09/2004 : B.MASSART
//@ Correction de l'initialisation de ENG_EQUALCOUNT. Voir IA-2100-FC-129-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-184-CN : 20/09/2006 : C.MERIGUET
//@ Suite passage purify delete[] remplace delete pour les tableaux
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-196-CN : 15/01/2007 : C.MERIGUET
//@ FA-ID : IA-2100-FA-198-CN : 15/01/2007 : C.MERIGUET
//@ Ces 2 FA sont traitees conjointement. Recuperation du PTSI lu dans 
//@ le mot 17 (2 octets) mais ecriture dans le champ PTSI des N1 des mots 
//@ 16 (version logiciel bord) et 17 (ptsi) (4 octets).
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2512 (FA-2100-OPS) : 17/11/2008 : S. BRUNEL
//@ Fuites memoires signalees par EUMETSAT (EUM.EPS.NCR.3627.1)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2527(DM-2100-OPS) : 06/03/2009 : S. BRUNEL
//@ Permettre le calcul des spectres dans les bandes B1 & B2 meme
//@ lorsque un spike affecte la bande B3 (Day 2,point 8)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2586(DM-2100-OPS) : 16/04/2009 : S. BRUNEL
//@ Supprimer la mise a 1 de GSsdSpectralShiftNonQual lorsqu'un spike
//@ est observe dans la bande de determination du Zpd (B3)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2522(DM-2100-OPS) : 30/04/2009 : S. BRUNEL
//@ Ajout de variables dans les produits L1C et ENG  (Day 2,point 3)
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2638(FA-2100-OPS) : 17/06/2009 : S. BRUNEL
//@ Valeurs erronees de MHipNZpdInterPixel pour les SN 32,33, 35 et 36
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2650(FA-2100-OPS) : 08/07/2009 : S. BRUNEL
//@ Valeur erronnee des flags internes de l'OPS (GOPSXxxxXxxx) 
//@ en entree de 45_QIS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2652(DM-2100-OPS) : 08/07/2009 : S. BRUNEL
//@ Suppression des verrous temporaires assurant la non regression
//@ entre V4-2-5 et V4-2
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL
#include <iostream>

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN__Chaine.h"

#include "SD_TAC_Tache.h"
#include "SD_GES_ChaineImageISRFEM.h"
#include "SD_FRW_ApplicationServeurDonnees.h"
#include "SD_ALG_Image.h"
#include "SD_ALG_Spectre.h"

// FT-2527
# include "SD_ALG_Monitoring.h"
// ..FT-2527

//======================================================================
//                                       METHODES PUBLIQUES 
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD_GES_ChaineImageISRFEM
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               (ENTREE) tache : Tache pere de l'action
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_GES_ChaineImageISRFEM::SD_GES_ChaineImageISRFEM(SD_TAC_Tache* tache)
  : SD_ACT_Action(tache)
{
  //@ Memoriser la tache parente de l'action
  m_Tache = tache;
  m_PeutEtreEnchainee = CMN__FAUX;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_GES_ChaineImageISRFEM
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_GES_ChaineImageISRFEM::~SD_GES_ChaineImageISRFEM()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Executer
//@
//@ Description : Traitement principal de l'action
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_GES_ChaineImageISRFEM::Executer()
{
  t_CMN__Statut cr_op = CMN__STATUT_OK;

  t_CMN__Booleen FenetreSpectraleValide;
  
  //@ Methode virtuelle surchargee pour executer la premiere partie de 
  //@ la chaine  image
  
  cout << "SD_GES_ChaineImageISRFEM pour la tache " << Tache()->Id() 
       << endl;
#ifdef OPS_DEBUG
  cout << "SD_GES_ChaineImageISRFEM pour la tache " << Tache()->Id() 
       << endl;
#endif

  //@ principes generaux a appliquer :
  //@ 0 - on considere que l'initialisation des traces a ete faite au prealable
  //@     dans  InitGranule, la structure t_SD_BAS_DataGen est passe aux 
  //@ algorithmes qui emettent eux meme leur traces
  //@ 1 - c'est a ce niveau que sont emis les messages au JdBS (  
  //@    trace  applicatives a partir de la structure t_SD_BAS_DataThread )
  //@  via : la remontee d'erreurs des algo
  //@ 2 - la memoire utilisee est de 2 type :
  //@   - partagee entre thread elle a alors ete allouee au prealable 
  //@   - locale au traitement, elle est alors allouee en debut puis liberee
  //@     en fin de methode 

  //@ recuperer le serveur de donnees reference dans application serveur de donnee
  SD_FRW_ApplicationServeurDonnees* application;
  application = m_Tache->Application();

  //@ recuperer le numero de ligne a traiter
  int numeroLigne;
  int numeroLigneGES; //dans DON les numeros commencent a 0, sauf pour GestCasDeg
  numeroLigne = m_Tache->Contexte()->IndiceLigne();
  numeroLigneGES = numeroLigne - 1;

  SD_DON_GestDonnees* gestionnaire = application->Donnees();


  //@ SI le contexte indique l'envoi de la telemesure SensingTime
  if (m_Tache->Contexte()->SignalerTemps() == CMN__VRAI)
    {
      //@ Appel de la methode SD_DON_GestDonnees::GetSensingTime()
      t_TableauDate sensingTime;
      gestionnaire->GetSensingTime(numeroLigneGES, sensingTime);

      //@ Envoyer la telemesure SensingTime
      EnvoyerSensingTime(sensingTime);
    }
  //@ FIN SI


  //@----------------------------------------------------------------------*/
  //@ recuperer les donnees temporaire globales initialisees */
  //@ appel GetDonGenGranule 
  //@ contexte + donnes generale YY_CONF
  //@----------------------------------------------------------------------*/

  t_SD_BAS_Contexte* contextecourant;
//  contextecourant = new t_SD_BAS_Contexte*[1];
  t_SD_BAS_DataGen  * datagen ;
//  t_SD_BAS_DataGen  ** datagen = new t_SD_BAS_DataGen*[1];

  gestionnaire->GetDonGenGranule(&contextecourant, &datagen);

  //@ recuperer le nombre de lignes du granule
  int nombreLignesIASI = gestionnaire->GetNbLignesIASI();
  
  //@ recuperer les donnees de niveau 0 de type IP
  t_SD_BAS_PaquetN0** ListePaquetIP;
  int NbPaquetsIP = gestionnaire->GetNbPaquetsN0(numeroLigneGES, IP);
  ListePaquetIP = new t_SD_BAS_PaquetN0*[NbPaquetsIP];
  gestionnaire->GetLigne0(numeroLigneGES, IP, NbPaquetsIP, ListePaquetIP);
  
  //@ recuperer les donnees de niveau 0 de type PX
  t_SD_BAS_PaquetN0** ListePaquetPX;
  int NbPaquetsPX = gestionnaire->GetNbPaquetsN0(numeroLigneGES, PX);
  ListePaquetPX = new t_SD_BAS_PaquetN0*[NbPaquetsPX];
  gestionnaire->GetLigne0(numeroLigneGES, PX, NbPaquetsPX, ListePaquetPX);
  
  //@ recuperer les donnees de niveau 1 initialisees pour la ligne N
  //@ 1ABC
  t_SD_BAS_MPHR * Mphr1ABC;
  t_SD_BAS_LigneN1 * LigneMDR1ABC;
  t_SD_BAS_LigneN1 * LigneVIADR1ABC;
  Mphr1ABC       = new t_SD_BAS_MPHR;
  LigneMDR1ABC   = new t_SD_BAS_LigneN1;
  LigneVIADR1ABC = new t_SD_BAS_LigneN1;
  gestionnaire->GetLigneS1(
    numeroLigneGES, 
    SD_DON_GranuleN1::SD_DON_GRANULE_N1_DATA, 
    *Mphr1ABC, 
    *LigneMDR1ABC, 
    *LigneVIADR1ABC);

#ifdef OPS_DEBUG
  cout << "LigneMDR1ABC - N1C_DATEDEB: " 
       << LigneMDR1ABC->InfoDet.tab_var[N1C_DATEDEB].val.pdate->day 
       << endl;
#endif

  // FT-2638
  //@ VER
  t_SD_BAS_MPHR * MphrVER;
  t_SD_BAS_LigneN1 * LigneMDRVER;
  t_SD_BAS_LigneN1 * LigneVIADRVER;
  MphrVER          = new t_SD_BAS_MPHR;
  LigneMDRVER      = new t_SD_BAS_LigneN1;
  LigneVIADRVER    = new t_SD_BAS_LigneN1;
  gestionnaire->GetLigneS1(
    numeroLigneGES,
    SD_DON_GranuleN1::SD_DON_GRANULE_N1_VER,
    *MphrVER,
    *LigneMDRVER,
    *LigneVIADRVER);
  // ...FT-2638

  //@ ENG
  t_SD_BAS_MPHR * MphrENG;
  t_SD_BAS_LigneN1 * LigneMDRENG;
  t_SD_BAS_LigneN1 * LigneVIADRENG;
  MphrENG        = new t_SD_BAS_MPHR;
  LigneMDRENG    = new t_SD_BAS_LigneN1;
  LigneVIADRENG  = new t_SD_BAS_LigneN1;
  gestionnaire->GetLigneS1(
    numeroLigneGES, 
    SD_DON_GranuleN1::SD_DON_GRANULE_N1_ENG, 
    *MphrENG, 
    *LigneMDRENG, 
    *LigneVIADRENG);

#ifdef OPS_DEBUG
  if (LigneMDRENG->TypeRec != MDR_DUMMY)
  {
    cout << "LigneMDRENG - ENG_DATEDEB: " 
         << LigneMDRENG->InfoDet.tab_var[ENG_DATEDEB].val.pdate->day 
         << endl;
    cout << "LigneMDRENG - ENG_IDEFIDCONF: " 
         << LigneMDRENG->InfoDet.tab_var[ENG_IDEFIDCONF].val.pui4[0] 
         << endl;
  }
#endif 
// SI MDR DUMMY
  if (LigneMDRENG->TypeRec != MDR_DUMMY)
  {
     LigneMDRENG->InfoDet.tab_var[ENG_IDEFIDCONF].val.pui4[0] = 3;
  
  //@ recuperer les donnees de configuration en fonction du PTSI
  //@ appel GetConf
//  t_Date tcourante;
  t_SD_BAS_ConfAutre *  ConfAutre ;
  t_SD_BAS_ConfStable * ConfStable ;
  t_SD_BAS_BanqueSpectrale * BanqueSpectrale ;
//  t_SD_BAS_ConfAutre **  ConfAutre = new t_SD_BAS_ConfAutre*[1];
//  t_SD_BAS_ConfStable ** ConfStable = new t_SD_BAS_ConfStable*[1];
//  t_SD_BAS_BanqueSpectrale ** BanqueSpectrale = new t_SD_BAS_BanqueSpectrale*[1];
//  tcourante.DateCourante();
  t_Date datePremiereLigne;
  datePremiereLigne.SCDSTime(LigneMDRENG->InfoDet.tab_var[ENG_DATEDEB].val.pdate[0]);

// FA 196 et 198 
  // inversion de mots
  int version_log = (int)(LigneMDRENG->InfoDet.tab_var[ENG_PTSI].val.pui4[0]/65536);
  int PTSI = LigneMDRENG->InfoDet.tab_var[ENG_PTSI].val.pui4[0] - (version_log * 65536);
  gestionnaire->GetConf(PTSI, 
                        datePremiereLigne, 
                        &ConfAutre, 
                        &ConfStable, 
                        &BanqueSpectrale);
// ..FA 196 et 198

  //@----------------------------------------------------------------------*/
  //@ recuperer les donnees de niveau 1 initialisees pour la ligne N-1
  double* slope = new double[SD_BAS_IMLI * SD_BAS_IMCO];
  double* offset = new double[SD_BAS_IMLI * SD_BAS_IMCO];
 
  //@ SI pas premiere ligne alors
  if (numeroLigne > 1)
    {
      t_SD_BAS_MPHR * MphrENGPrec;
      t_SD_BAS_LigneN1 * LigneMDRENGPrec;
      t_SD_BAS_LigneN1 * LigneVIADRPrec;
      MphrENGPrec        = new t_SD_BAS_MPHR;
      LigneMDRENGPrec    = new t_SD_BAS_LigneN1;
      LigneVIADRPrec     = new t_SD_BAS_LigneN1;
      gestionnaire->GetLigneS1(numeroLigneGES - 1, 
                               SD_DON_GranuleN1::SD_DON_GRANULE_N1_ENG, 
                               *MphrENGPrec, 
                               *LigneMDRENGPrec, 
                               *LigneVIADRPrec);

      if (LigneMDRENGPrec->TypeRec != MDR_DUMMY)
       {
  
      //@ initialiser offset et slope a partir de la liste var
      for (int i = 0; i < SD_BAS_IMLI * SD_BAS_IMCO; i++)
        {
          slope[i] = LigneMDRENGPrec->InfoDet.tab_var[ENG_GICCRADCALSLOPEIMAG].val.pr8[i];
          offset[i]= LigneMDRENGPrec->InfoDet.tab_var[ENG_GICCRADCALOFFSETIMAG].val.pr8[i];
        }
       }
      else
       {
      // on recupere la derniere ligne non dummy si elle existe
      int nb;
      nb=numeroLigneGES;
       while (LigneMDRENGPrec->TypeRec == MDR_DUMMY && nb > 1 )
        {
         nb --;
         gestionnaire->GetLigneS1(nb - 1, 
                               SD_DON_GranuleN1::SD_DON_GRANULE_N1_ENG, 
                               *MphrENGPrec, 
                               *LigneMDRENGPrec, 
                               *LigneVIADRPrec);
        }
       if ( LigneMDRENGPrec->TypeRec == MDR_DUMMY )
        {
       // on choisit ici la valeur issue du contexte
      // a valider avec le CNES
      //@  on recupere offset et slope a partir du contexte precedent
        for(int i = 0; i < SD_BAS_IMLI * SD_BAS_IMCO; i++)
         {
	    //jlp11
           slope[i] = contextecourant->GIccRadCalSlopeImag[SD_BAS_IMCO * SD_BAS_IMLI+ i];
           offset[i] = contextecourant->GIccRadCalOffsetImag[SD_BAS_IMCO * SD_BAS_IMLI+ i];
         }
        }
       else
        {
 
      //@ initialiser offset et slope a partir de la liste var
        for (int i = 0; i < SD_BAS_IMLI * SD_BAS_IMCO; i++)
          {
          slope[i] = LigneMDRENGPrec->InfoDet.tab_var[ENG_GICCRADCALSLOPEIMAG].val.pr8[i];
          offset[i]= LigneMDRENGPrec->InfoDet.tab_var[ENG_GICCRADCALOFFSETIMAG].val.pr8[i];
          }
        }		   
       
       }
      delete MphrENGPrec;
      delete LigneMDRENGPrec;
      delete LigneVIADRPrec;
    }
  //@ SINON
  else
    {
      //@ SI initialisation a froid
      if (contextecourant->GCTXNatCTX == INIT_FROID)
        {
          //@ initialiser offset et slope a partir de la liste var
          for (int i = 0; i < SD_BAS_IMLI * SD_BAS_IMCO; i++)
            {
              slope[i] = LigneMDRENG->InfoDet.tab_var[ENG_GICCRADCALSLOPEIMAG].val.pr8[i];
              offset[i]= LigneMDRENG->InfoDet.tab_var[ENG_GICCRADCALOFFSETIMAG].val.pr8[i];
            }
        }
      //@ SINON
      else
        {
          //@  on recupere offset et slope a partir du contexte precedent
          for(int i = 0; i < SD_BAS_IMLI * SD_BAS_IMCO; i++)
            {
              slope[i] = contextecourant->GIccRadCalSlopeImag[SD_BAS_IMCO * SD_BAS_IMLI+ i];
              offset[i] = contextecourant->GIccRadCalOffsetImag[SD_BAS_IMCO * SD_BAS_IMLI + i];
            }
        }
      //@ FIN SI
    }
  //@ FIN SI

  //@ Initialisation des donnees de niveau 1
  InitialiserNiveau1(LigneMDR1ABC, 
                     *LigneMDRENG, 
                     ConfAutre, 
                     contextecourant);
  
  
  //@ Recuperation du gestionniare des cas degrades
  SD_DON_GestCasDeg gestCasDeg = gestionnaire->GetGestCasDeg();
  
  t_Date dateTmp;
  
  //@ Si pas (CG2) et pas (CG12)
  if ( !(gestCasDeg.GetCasDegLigne(numeroLigne))->IsDetecte(12, 0, 0, 0, dateTmp)
       && !(gestCasDeg.GetCasDegLigne(numeroLigne))->IsDetecte(2, 0, 0, 0, dateTmp))
    {
      //@ pointeurs pour algos
      t_SD_BAS_DataThread22SOS* data22SOS = new t_SD_BAS_DataThread22SOS;
      t_SD_BAS_DataThreadImageISRFEM* dataImageISRFEM = 
        new t_SD_BAS_DataThreadImageISRFEM;
      dataImageISRFEM->GIrcImg = 
        new double [SD_BAS_IMCO*SD_BAS_IMLI*SD_BAS_SN];

      int NombreSNminValide = -1;
      t_SD_BAS_DataThread *ctx = new t_SD_BAS_DataThread;
      ctx->ctxtrt.LN = numeroLigne;
      ctx->ctxtrt.SN = 0;
      ctx->ctxtrt.PN = 0;
      ctx->ctxtrt.CCD = 0;
      ctx->ctxtrt.SB = 0;
      InitialiserCompteRendu(&(ctx->cr));
 

      //@ allocation pointeurs ImageISRFEM
      AllouerPointeursImageISRFEM(dataImageISRFEM,
                                  ConfAutre,
                                  BanqueSpectrale,
                                  contextecourant,
 				  nombreLignesIASI);


      //@ allocation pointeurs 22 SOS
      AllouerPointeurs22SOS(data22SOS, ConfAutre);

      // FT-2650
      // Mise a jour des flags internes GOPS en fonction des Cas degrades 
      // avant l'appel de 45_QIS. Le traitement effectues provient de 
      // FinTraitement1() en fin de chaine produit.
      for (int sn = 0; sn < SD_BAS_SNOT; sn++)
      {
        for (int pn = 0; pn < SD_BAS_PN; pn++)
        {
          //@ CG1
          if(gestCasDeg.GetCasDegLigne(numeroLigne)->GetFlt(1).flag[sn][pn][0])
          {
            LigneMDRENG->InfoDet.tab_var[ENG_GOPSFLAPIXMISS].val.pui4[sn*SD_BAS_PN+pn] = 1;
          }
          //@ CG4
          for (int sb = 0; sb < SD_BAS_SB; sb++)
          {
            if(gestCasDeg.GetCasDegLigne(numeroLigne)->GetFlt(4).flag[sn][pn][sb])
            {
              LigneMDRENG->InfoDet.tab_var[ENG_GOPSFLTBANDMISS].val.pui4[sn] = 1;
            }
          }
        }
        //@ CG3
        for (int ccd = 0; ccd < SD_BAS_CCD; ccd++)
        {
          if (gestCasDeg.GetCasDegLigne(numeroLigne)->GetFlt(3).flag[sn][0][ccd])
          {
            LigneMDRENG->InfoDet.tab_var[ENG_GOPSFLTISRFEMOFF].val.pui4[sn] = 1;
          }
        }
        //@ CG6
        if (gestCasDeg.GetCasDegLigne(numeroLigne)->GetFlt(6).flag[sn][0][0])
        {
          LigneMDRENG->InfoDet.tab_var[ENG_GOPSFLTIMGEWMISS].val.pui4[sn] = 1;
        }
      }
      // ...FT-2650

      // FT-2527
      //@ Calcul les differences de NZPD interpixel
      //@ Pour mettre a jour FlagInterPixNZpdNonQual
      // et pouvoir determiner s'il existe un spike
      // dans la bande de determination du NZPD
      cr_op = SD_ALG_46_HIP(
                          ctx,
                          ConfStable,
                          // FT-2638
                          LigneMDRVER->InfoDet2,
                          // ...FT-2638
                          &LigneMDRENG->InfoDet
                          );
      // ..FT-2527


// realisation du corps du traitement image  pour une ligne dans une fonction specifique
      Corpstraitement(
			 numeroLigne,
		         gestionnaire,
  			 contextecourant,
 			 datagen,
			 gestCasDeg,
			 NbPaquetsPX,
			 ListePaquetPX,							
			 NbPaquetsIP,
			 ListePaquetIP,							
  			 Mphr1ABC,
 			 LigneMDR1ABC,
 			 LigneMDRENG,
 			 ConfAutre ,
  			 ConfStable ,
  			 BanqueSpectrale ,
			 slope,
			 offset,
			 &NombreSNminValide,
 			 dataImageISRFEM,
  			 data22SOS,
			 ctx
    			);


      //@ initialisation des dates
      t_SCDSTime t ;
      /* DM 11/03/2004 pt 1.17 pour 25_FAX, 21_SSS */
      t.day = LigneMDRENG->InfoDet.tab_var[ENG_GEPSDATIASI].val.pdate[IND1(SD_BAS_SNREFDATELIGNE)].day;
      t.millis = LigneMDRENG->InfoDet.tab_var[ENG_GEPSDATIASI].val.pdate[IND1(SD_BAS_SNREFDATELIGNE)].millis;
      t_Date dateSCDS;
      dateSCDS.SCDSTime(t);
      t_SCDSTime t_FE = (gestionnaire->GetDateEclipse(dateSCDS)).SCDSTime();
      ctx->ctxtrt.SB = 0; 
      ctx->ctxtrt.SN = 0; 

      //@ POUR chaque direction de coin de cube ( de 1 ,2 )
      for (int ccd = 0; ccd < SD_BAS_CCD; ccd++)
        {
          ctx->ctxtrt.CCD = ccd;

          //@ POUR chaque pixel (1..4)
          for(int pn = 1; pn <= SD_BAS_PN; pn++)
            {
              ctx->ctxtrt.PN = pn;
              //@ Selection des decalages spectraux (A1333)
              //@ Call 21_SSS
              cr_op = SD_ALG_21_SSS(ctx,
                                    dataImageISRFEM,
                                    t,
                                    t_FE,
                                    dataImageISRFEM->GIrcImg,
                                    ConfAutre,
                                    ConfStable,
                                    ListePaquetPX, 
                                    NbPaquetsPX, 
                                    &LigneMDRENG->InfoDet);

              //@ traitement du compte-rendu
              Traiter(cr_op, ctx);
            }
          //@ FIN POUR chaque pixel

          //@ Determination de l?axe interferometrique (A1334)
          //@  Call 24_IAX
          ctx->ctxtrt.PN = 0;
          cr_op = SD_ALG_24_IAX(ctx,
                                dataImageISRFEM,
                                ConfAutre,
                                BanqueSpectrale,
                                &LigneMDRENG->InfoDet);

          //@ traitement du compte-rendu
          Traiter(cr_op, ctx);
        }
      //@ FIN POUR chaque direction de coin de cube 

      ctx->ctxtrt.CCD=0;

      LibererCompteRendu(&(ctx->cr.msgJDB));
      delete ctx;

      //@ Mise a jour contexte
      //@ appel AjouteElemHist
      t_SD_BAS_HistAxeInterf** HistAxeInterf = new t_SD_BAS_HistAxeInterf*[SD_BAS_CCD];
      HistAxeInterf[0] = new t_SD_BAS_HistAxeInterf;
      HistAxeInterf[0]->GiaxDateElt.day = t.day;
      HistAxeInterf[0]->GiaxDateElt.millis = t.millis;
      HistAxeInterf[0]->GiaxAxeYElt = LigneMDRENG->InfoDet.tab_var[ENG_GIAXAXEY].val.pr8[0];
      HistAxeInterf[0]->GiaxAxeZElt = LigneMDRENG->InfoDet.tab_var[ENG_GIAXAXEZ].val.pr8[0];
      HistAxeInterf[0]->GiaxAxeQualElt = LigneMDRENG->InfoDet.tab_var[ENG_GIAXAXEQUAL].val.pr8[0];
      HistAxeInterf[0]->GiaxFlagAxeNonQual = LigneMDRENG->InfoDet.tab_var[ENG_GIAXFLAGAXENONQUAL].val.pui4[0];
      HistAxeInterf[1] = new t_SD_BAS_HistAxeInterf;
      HistAxeInterf[1]->GiaxDateElt.day = t.day;
      HistAxeInterf[1]->GiaxDateElt.millis = t.millis;
      HistAxeInterf[1]->GiaxAxeYElt = LigneMDRENG->InfoDet.tab_var[ENG_GIAXAXEY].val.pr8[1];
      HistAxeInterf[1]->GiaxAxeZElt = LigneMDRENG->InfoDet.tab_var[ENG_GIAXAXEZ].val.pr8[1];
      HistAxeInterf[1]->GiaxAxeQualElt = LigneMDRENG->InfoDet.tab_var[ENG_GIAXAXEQUAL].val.pr8[1];
      HistAxeInterf[1]->GiaxFlagAxeNonQual = LigneMDRENG->InfoDet.tab_var[ENG_GIAXFLAGAXENONQUAL].val.pui4[1];
      t_SD_BAS_HistTempCN* TempCN = NULL;
      gestionnaire->AjouteElemHist(HistAxeInterf, TempCN);

      // FT-2512
      delete HistAxeInterf[0];
      delete HistAxeInterf[1];
      // ..FT-2512

/* FA 184 */
      delete [] HistAxeInterf;
/* ..FA 184 */      
      //@ appel MajContexte

      //@ liberation pointeurs dans t_SD_BAS_DataThreadImageISRFEM 
      //@ et t_SD_BAS_DataThread22SOS
      DesallouerPointeurs22SOS(data22SOS);
      DesallouerPointeursImageISRFEM(dataImageISRFEM);

      delete data22SOS;
      delete dataImageISRFEM;
    }
  //@ FIN SI pas (CG2) et pas (CG12)

 //fin:

//  delete [] contextecourant;
//  delete [] datagen;
  delete [] slope;
  delete [] offset;
  }
// FIN SI DUMMY
  delete [] ListePaquetIP;
  delete [] ListePaquetPX;
  delete Mphr1ABC;
  delete LigneMDR1ABC;
  delete LigneVIADR1ABC;
  delete MphrENG;
  delete LigneMDRENG;
  delete LigneVIADRENG;
//  delete [] ConfAutre;
//  delete [] ConfStable;
//  delete [] BanqueSpectrale;
  // FT-2638
  delete MphrVER;
  delete LigneMDRVER;
  delete LigneVIADRVER;
  // ...FT-2638

  //@ Invoquer la verification des symptomes une fois l'action
  //@ executee
  VerifierSymptome();
}


//@ ------------------------------------------------------------------*/
//@ Methode : VerifierSymptome
//@
//@ Description : Verifie si les symptomes associes a l'action sont
//@               en erreur ou non
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_GES_ChaineImageISRFEM::VerifierSymptome()
{
  //@ Methode virtuelle a surcharger dans les classes derivees
  //@ Les symptomes controles sont specifiques a l'action
        
  m_PeutEtreEnchainee = CMN__VRAI;
}


//======================================================================
//                                      METHODES PRIVEES 
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : Corpstraitement
//@
//@ Description : algorithme principal de l'action
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_GES_ChaineImageISRFEM::Corpstraitement(
  const  int                      numeroLigne,
  SD_DON_GestDonnees*             gestionnaire,
  t_SD_BAS_Contexte*              contextecourant,
  t_SD_BAS_DataGen*               datagen,
  SD_DON_GestCasDeg&              gestCasDeg,
  const int                       NbPaquetsPX,
  t_SD_BAS_PaquetN0**             ListePaquetPX,							
  const int                       NbPaquetsIP,
  t_SD_BAS_PaquetN0**             ListePaquetIP,							
  t_SD_BAS_MPHR*                  Mphr1ABC,
  t_SD_BAS_LigneN1*               LigneMDR1ABC,
  t_SD_BAS_LigneN1*               LigneMDRENG,
  t_SD_BAS_ConfAutre*             ConfAutre ,
  t_SD_BAS_ConfStable*            ConfStable ,
  t_SD_BAS_BanqueSpectrale*       BanqueSpectrale ,
  double*                         slope,
  double*                         offset,
  int*                            NombreSNminValide,
  t_SD_BAS_DataThreadImageISRFEM* dataImageISRFEM,
  t_SD_BAS_DataThread22SOS*       data22SOS,
  t_SD_BAS_DataThread*            ctx)

{
  t_CMN__Statut cr_op = CMN__STATUT_OK;

  t_CMN__Booleen FenetreSpectraleValide;
  int indSW;
  int NfirstSW;
  int NlastSW;
  int NsfirstSB;
  int NslastSB;
  int iSB;
  int SN, SNcourant;
  int PN, PNcourant;
  t_CMN__Booleen Trouve;
  // FT-2527
  unsigned int        GOPSFlagSpikNZpdBzpd;       //@ flag de spike dans la bande de determination du NZPD
  unsigned int        BSpkFlagSpikB1;             //@ flag BSpkFlagSpik for band B1
  unsigned int        BSpkFlagSpikB2;             //@ flag BSpkFlagSpik for band B2
  unsigned int        BSpkFlagSpikB3;             //@ flag BSpkFlagSpik for band B3
  unsigned int        BSpkFlagSpikZpdBzpd;        //@ flag BSpkFlagSpik for band ZpdBzpd
  unsigned int        BZpdFlagNZpdNonQual;    
  unsigned int        BArcFlagCalSpectNonQualB1;
  unsigned int        BArcFlagCalSpectNonQualB2;
  unsigned int        BArcFlagCalSpectNonQualB3;
  // ..FT-2527
  
  FenetreSpectraleValide = CMN__FAUX;
  indSW     = 0;
  NfirstSW  = 0;
  NlastSW   = 0;
  NsfirstSB = 0;
  NslastSB  = 0;
  
  // debut decoupage 
  int indFS   = ConfAutre->IDefSsdSpectralWindowOPS;
  int Nsfirst = ConfAutre->IDefSsdNsfirstSW[IND1(indFS)];
  int Nslast  = ConfAutre->IDefSsdNslastSW[IND1(indFS)];
  
  //@ allocations tableau temporaire
  t_SD_BAS_22_SOS_Sortie* InfoGen = new t_SD_BAS_22_SOS_Sortie;
  double* tableauImage  = new double[SD_BAS_IMLI * SD_BAS_IMCO + 1];
  double* imageCalibree = new double[SD_BAS_IMLI * SD_BAS_IMCO];


  t_Date dateTmp;
  //@ POUR chaque paquet IP (boucle sur SN)
  for(int paquet = 0; paquet < NbPaquetsIP; paquet++)
  {
    int sn = ListePaquetIP[paquet]->InfoGen.tab_var[N0_SN].val.pui4[0];
    
    //@ SI pas (CG2)
    if ( !(gestCasDeg.GetCasDegLigne(numeroLigne))->IsDetecte(2, sn - 1, 0, 0, dateTmp))
    {
      ctx->ctxtrt.SN = sn;
      
      //@ SI NombreSNminValide pas initialise
      if (*NombreSNminValide == -1)
      {
        //@ initialisation
        *NombreSNminValide = sn;
      }
      //@ FIN SI
      
      //@ on renseigne le equal counter dans le MDR ENG
      int isn;
      if (sn == 32)
      {
        isn = 31;
      }
      else if (sn == 33)
      {
        isn = 32;
      }
      else if (sn == 35) 
      {
        isn = 33;
      }
      else if (sn == 36) 
      {
        isn = 34;
      }
      else 
      {
        isn = sn;
      }
      
       /* FA129 */
      //@ SI pas ( CG14 )
      if ( !(gestCasDeg.GetCasDegLigne(numeroLigne))->IsDetecte(14, sn - 1, 0, 0, dateTmp))
      {
        //@ decodage du train binaire image dans un tableau temporaire
        gestionnaire->DecodeDataLigne0(ListePaquetIP[paquet], tableauImage); 

        //@ renseignement de l'equal counter dans le MDR ENG     
        for (int pn = 0; pn < SD_BAS_PN; pn++)
        {
          LigneMDRENG->InfoDet.tab_var[ENG_EQUALIZATION_COUNTER].val.pi4[IND1(isn) * SD_BAS_PN + pn] = (int) tableauImage[0];
        }
      }

      //@ SI pas ( CG6 )
      if ( !(gestCasDeg.GetCasDegLigne(numeroLigne))->IsDetecte(6, sn - 1, 0, 0, dateTmp) && (sn <= 30) )
      {
        //@ decodage du train binaire image dans un tableau temporaire
        gestionnaire->DecodeDataLigne0(ListePaquetIP[paquet], tableauImage);
      
      /* FIN FA129 */
        
        //@ decodage image + calibration
        //@ Call 39_IRC
        cr_op = SD_ALG_39_IRC(
          ctx,
          ConfAutre,
          ConfStable,
          &tableauImage[1],
          offset,
          slope,
	  &LigneMDRENG->InfoDet, 
          imageCalibree);
        
        //@ traitement du compte-rendu
        Traiter(cr_op, ctx);
       
        // FT-2522
        LigneMDR1ABC->InfoDet.tab_var[N1C_GIACVARIMAGIIS].val.pr8[IND1(sn)] =
                LigneMDRENG->InfoDet.tab_var[ENG_GIACVARIMAGIIS].val.pr8[IND1(sn)];
        LigneMDR1ABC->InfoDet.tab_var[N1C_GIACAVGIMAGIIS].val.pr8[IND1(sn)] =
                LigneMDRENG->InfoDet.tab_var[ENG_GIACAVGIMAGIIS].val.pr8[IND1(sn)];
        // ..FT-2522

        //@ Codage de l'image dans le produit 1A
        t_SD_BAS_ORIGINE numlignesnpn;
        numlignesnpn.LN = numeroLigne;
        numlignesnpn.SN = sn; 
        numlignesnpn.PN = 0;
        gestionnaire->EncodeDecodeLigneS1(
          numlignesnpn, 
          *LigneMDR1ABC,
          AT_IMAGE,
          *(ConfStable),
          ENCODE,
          *imageCalibree);
        
        //@ recopie de l'image calibree dans dataImageISRFEM
        for (int i = 0; i < SD_BAS_IMCO * SD_BAS_IMLI; i++)
        {
          dataImageISRFEM->GIrcImg[(sn - 1) * SD_BAS_IMCO * SD_BAS_IMLI + i] = imageCalibree[i];
        }
      }
      //@ FIN SI pas (CG6)
    }
    //@ FIN SI pas (CG2)
  }
  //@ FIN POUR chaque paquet IP
  
  //@ liberation tableaux temporaires image
  delete [] tableauImage;
  delete [] imageCalibree;
  //goto fin;
  
  //@ reservation tableaux temporaires spectre
  char* tableauSpectre = new char[SD_BAS_MAXLENGTH_SPECTN0];
  double * Spect = new double[Nslast - Nsfirst + 1];
  int surechFFT = ConfAutre->IDefSsdOSFftFactor[IND1(indFS)];
  int surechSpline = ConfAutre->IDefSsdOSSplineFactor[IND1(indFS)];
  double * GSpectReech = new double[(Nslast-Nsfirst) * surechFFT * surechSpline + 1];
  
  //@ POUR chaque paquet PX (boucle sur SN et PN)
  for (int paquet = 0; paquet < NbPaquetsPX; paquet++)
  {
    //@ initialisation ctx
    int sn = ListePaquetPX[paquet]->InfoGen.tab_var[N0_SN].val.pui4[0];

#ifdef OPS_DEBUG
    int ccd = LigneMDRENG->InfoDet.tab_var[ENG_GEPS_CCD].val.pui4[sn-1];
	   if (ccd != 0 && ccd != 1)
	   {
          cout << " ERREUR sur le Numero CCD      : " 
               << ccd 
               << endl;
          cout << "Numero SN      : " 
               << sn 
               << endl;
	   }
#endif
    //@ SI pas (CG2)
    if ( !(gestCasDeg.GetCasDegLigne(numeroLigne))->IsDetecte(2, sn - 1, 0, 0, dateTmp))
    {
      ctx->ctxtrt.SN = sn;
      ctx->ctxtrt.SB = 3; // pour differentier avec la chaine produit
      
      //@ Si pas ( CG3)
      if ( !(gestCasDeg.GetCasDegLigne(numeroLigne))->IsDetecte(3, sn - 1, 0, 0, dateTmp)
           && !(gestCasDeg.GetCasDegLigne(numeroLigne))->IsDetecte(3, sn - 1, 0, 1, dateTmp))
      {
        int pn = ListePaquetPX[paquet]->InfoGen.tab_var[N0_PN].val.pui4[0];
        ctx->ctxtrt.PN = pn;
        
        //@ decodage du train binaire  spectre dans un tableau temporaire
        //@ appel DecodeDataLigne0 pour le spectre
        gestionnaire->DecodeDataLigne0(ListePaquetPX[paquet], tableauSpectre); 
        
        t_CMN__Booleen ValiditeBandesSpectrales[SD_BAS_SB];
        
        //@ Call 20_DOC
        cr_op = SD_ALG_20_DOC(
          ctx,
          ConfAutre,
          SD_BAS_ISRFEM,
          tableauSpectre,
          Spect,
          &(LigneMDRENG->InfoDet), 
          ValiditeBandesSpectrales);
        
        //@ traitement du compte-rendu
        Traiter(cr_op, ctx);
        
        //@ SI Overflow ou underflow
        int nbcg4 = 0;
        
        FenetreSpectraleValide = CMN__VRAI;
        indSW    = ConfAutre->IDefSsdSpectralWindowOPS;
        NfirstSW = ConfAutre->IDefSsdNsfirstSW[IND1(indFS)];
        NlastSW  = ConfAutre->IDefSsdNslastSW[IND1(indFS)];
        
        if (LigneMDRENG->InfoDet.tab_var[ENG_GDOCFLAGUNDEROVERFLOW].val.pui4[IND2(pn,sn,SD_BAS_PN)] == CMN__VRAI)
        {
          //@ on leve CG4 et eventuellement CG1 et CG3 ??????????
          //@ on leve CG4 pour les bandes concernees (pas d'info de bande)
          for (int sb = 0; sb < SD_BAS_SB; sb++)
          {
            if (ValiditeBandesSpectrales[sb] == CMN__FAUX)
            {
              nbcg4++;
              gestCasDeg.GetCasDegLigne(numeroLigne)->
              SetDetecte(4, dateTmp, sn - 1, pn - 1, sb);
              
              //@ SI la fenetre spectrale de calibration est 
              //@ incluse dans une bande invalide ALORS
              NsfirstSB = ConfAutre->IDefNsfirst[sb];
              NslastSB = ConfAutre->IDefNslast[sb];
              if ( (NsfirstSB <= NfirstSW) && 
                   (NslastSB >= NlastSW) )
              {
                //@ on declare la fenetre spectrale invalide
                FenetreSpectraleValide = CMN__FAUX;
              }
              //@ FIN SI
            }
          }
        }
        //@ FIN SI
        
        //@ SI pas ( CG1 ou CG6 ou CG4 pour la fenetre utilisee)
        if ( !(gestCasDeg.GetCasDegLigne(numeroLigne))->IsDetecte(1, sn - 1, pn - 1, 0, dateTmp)
             && (FenetreSpectraleValide == CMN__VRAI) )
        {
          //@ Determination du decalage spectral (A1332)
          //@ Call 22_SOS
          cr_op = SD_ALG_22_SOS(
            ctx,
            data22SOS,
            ISRFEM,
            ConfAutre,
            datagen,
            Spect,
            InfoGen,
            GSpectReech);
            
          //@ traitement du compte-rendu
          Traiter(cr_op, ctx);
         
          //@ Call 23_SSD
          cr_op = SD_ALG_23_SSD(
            ctx,
            dataImageISRFEM,
            ConfAutre,
            ConfStable,
            InfoGen,
            GSpectReech,
            LigneMDRENG->InfoDet);
          
          //@ traitement du compte-rendu
          Traiter(cr_op, ctx);  
	
        }
        //@ FIN SI
      }
      //@ FIN SI pas (CG3)
    }
    //@ FIN SI pas (CG2)
  }
  //@ FIN POUR chaque paquet PX

  
  Trouve = CMN__FAUX;
  for (SN = 1 ; SN <= SD_BAS_SNOT; SN++)
  {
    for (PN = 1 ; PN <= SD_BAS_PN ; PN++)
    {
      SNcourant = 0;
      PNcourant = 0;
      
      for (int paquet = 0 ; paquet < NbPaquetsPX ; paquet++)
      {
        SNcourant = ListePaquetPX[paquet]->InfoGen.tab_var[N0_SN].val.pui4[0];
        PNcourant = ListePaquetPX[paquet]->InfoGen.tab_var[N0_PN].val.pui4[0];
        if ( (SNcourant == SN) && (PNcourant == PN) )
        {
          Trouve = CMN__VRAI;
        }
      } /* fin boucle sur NbPaquetsPX */ 

      if (Trouve == CMN__FAUX)
      {
        LigneMDRENG->InfoDet.tab_var[ENG_GSSDFLAGSPECTRALSHIFTNONQUAL].val.pui4[IND2(PN,SN,SD_BAS_PN)] = CMN__OUI;
      }
      else
      {
         // FT-2527
         cr_op = sub_45_QIS(
               ctx, 
               ConfStable, 
               ConfAutre, 
               &LigneMDRENG->InfoDet, 
               SN, 
               PN, 
               &GOPSFlagSpikNZpdBzpd,
               &BSpkFlagSpikB1,
               &BSpkFlagSpikB2,
               &BSpkFlagSpikB3,
               &BSpkFlagSpikZpdBzpd,
               &BZpdFlagNZpdNonQual,
               &BArcFlagCalSpectNonQualB1,
               &BArcFlagCalSpectNonQualB2,
               &BArcFlagCalSpectNonQualB3        
               );
        if (GOPSFlagSpikNZpdBzpd == CMN__OUI) 
        {
          // FT-2652
          // FT-2586
          LigneMDRENG->InfoDet.tab_var[ENG_GSSDFLAGSPECTRALSHIFTNONQUAL].val.pui4[IND2(PN,SN,SD_BAS_PN)] = CMN__OUI;
          // ...FT-2586
          // ...FT-2652
        }
        // ...FT-2527
        Trouve = CMN__FAUX;
      }
    } /* fin boucle sur SD_BAS_PN */
  } /* fin boucle sur SD_BAS_SNOT */
  
  //@ liberation tableaux temporaires spectre
  delete [] Spect;
  delete [] tableauSpectre;
  delete [] GSpectReech;
  delete InfoGen;
}


//@ ------------------------------------------------------------------*/
//@ Methode : InitialiserNiveau1
//@
//@ Description : Initialise donnees niveau 1 
//@               a partir des donnees de niveau 0 et fic conf
//@
//@ Parametres :
//@               (ENTREE) type : Type de fichier
//@               (ENTREE) nomFichier : Nom du fichier
//@
//@ Valeur de retour : 
//@
//@ ------------------------------------------------------------------*/
void SD_GES_ChaineImageISRFEM::InitialiserNiveau1(t_SD_BAS_LigneN1* LigneMDR1ABC,
                                                  t_SD_BAS_LigneN1& LigneMDRENG,
                                                  t_SD_BAS_ConfAutre* ConfAutre,
                                                  t_SD_BAS_Contexte* contexte)
{
//@ MDR 1ABC  
//@     idefidconf
  LigneMDR1ABC->InfoDet.tab_var[N1AB_IDEFIDCONF].val.pui4[0] = ConfAutre->IDefConf;
//@     IdefSpectDwn
  LigneMDR1ABC->InfoDet.tab_var[N1A_IDEFSPECTDWN].val.pr8[0] = ConfAutre->IDefSpectrDWn;
  LigneMDR1ABC->InfoDet.tab_var[N1C_IDEFSPECTDWN1B].val.pr8[0] = ConfAutre->IDefSpectDWn1b;  
//@     IDefNsFirst
  LigneMDR1ABC->InfoDet.tab_var[N1A_IDEFNSFIRST].val.pi4[0] = ConfAutre->IDefNsfirst[SD_BAS_B1];
  LigneMDR1ABC->InfoDet.tab_var[N1C_IDEFNSFIRST1B].val.pi4[0] = ConfAutre->IDefNsfirst1b;
//@     IDefCCsMode
  LigneMDR1ABC->InfoDet.tab_var[N1C_IDEFCCSMODE].val.pui4[0] = ConfAutre->IDefCcsMode;
//@     IdefNsLast
  LigneMDR1ABC->InfoDet.tab_var[N1A_IDEFNSLAST].val.pi4[0] = ConfAutre->IDefNslast[SD_BAS_B3];
  LigneMDR1ABC->InfoDet.tab_var[N1C_IDEFNSLAST1B].val.pi4[0] = ConfAutre->IDefNslast1b;
//@ IDefCCsChannelId
  for(int i = 0; i < SD_BAS_NBK; i++)
    LigneMDR1ABC->InfoDet.tab_var[N1C_IDEFCCSCHANNELID].val.pui4[i] = ConfAutre->IDefCcsChannelId[i];
//@     IdefCovarMatEingenVal1
  for(int i = 0; i < SD_BAS_COVAR_DIM * SD_BAS_CCD; i++)
   { LigneMDR1ABC->InfoDet.tab_var[N1C_IDEFCOVARMATEIGENVAL1C].val.pr8[i] =
       ConfAutre->IDefCovarMatEigenVal1c[IND2((i/SD_BAS_CCD)+1,(i%SD_BAS_CCD)+1,SD_BAS_COVAR_DIM)];
   //inversion du rangement pa rapport au fichier de conf
    LigneMDR1ABC->InfoDet.tab_var[N1B_IDEFCOVARMATEIGENVAL1B].val.pr8[i] =
       ConfAutre->IDefCovarMatEigenVal1c[IND2((i/SD_BAS_CCD)+1,(i%SD_BAS_CCD)+1,SD_BAS_COVAR_DIM)];
    }
//@ MDR ENG
//@     idefidconf
  LigneMDRENG.InfoDet.tab_var[ENG_IDEFIDCONF].val.pui4[0] = ConfAutre->IDefConf;
//@     GCcsConvAvhrrChannel
  //LigneMDRENG->InfoDet.tab_var[ENG_GEPSCONFAVHRRCHANNEL].val.pui4[0] = 
//@     GEPSGranulnumber (issu du contexte)
// incremente de 1 car mis a jour en fin de traitement
  LigneMDRENG.InfoDet.tab_var[ENG_GEPSGRANULNUMBER].val.pi4[0] = contexte->GCTXNumGranule[0]+1;
//@     CGeoSubSatellitePosition
  //for(int i = 0; i < SD_BAS_CCD; i++)
  //  LigneMDRENG->InfoDet.tab_var[ENG_CGEOSUBSATELLITEPOSITION].val.pr8[i] = 
}

//@ ------------------------------------------------------------------*/
//@ Methode : AllouerPointeurs22SOS
//@
//@ Description : Alloue les pointeurs de la structure t_SD_BAS_DataThread22SOS
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@
//@ ------------------------------------------------------------------*/
void SD_GES_ChaineImageISRFEM::AllouerPointeurs22SOS(t_SD_BAS_DataThread22SOS* data22SOS,
                                                     t_SD_BAS_ConfAutre *ConfAutre
                                                    )
{
  //@----------------------------------------------------------------------*/
  //@ allocation pointeurs dans t_SD_BAS_DataThread22SOS

  int indFS       = ConfAutre->IDefSsdSpectralWindowOPS;
  int nFFT        = MAX(ConfAutre->IWnpNsFFT[indFS] , ConfAutre->IDefS1bNsFFT);
  int surechFFT   = MAX(ConfAutre->IDefSsdOSFftFactor[indFS] ,ConfAutre->IDefS1bOSFftFactor);
  int surechSpline= MAX(ConfAutre->IDefSsdOSSplineFactor[indFS] , ConfAutre->IDefS1bOSSplineFactor);
  int sigmaSpectre= MAX(ConfAutre->IDefSsdSigS[indFS],ConfAutre->IDefS1bSigS);
  int sigmaInterf = MAX(ConfAutre->IDefSsdSigI[indFS],ConfAutre->IDefS1bSigI);
  int Nfirst      = MIN(ConfAutre->IDefSsdNsfirstSW[indFS],ConfAutre->IDefNsfirst1b);
  int Nlast       = MAX(ConfAutre->IDefSsdNslastSW[indFS],ConfAutre->IDefNslast1b);
  int NbEchantillons = Nlast - Nfirst + 1;

// initialisation de l'indicateur d'initialisation (optimisation de calcul des FFT )
  data22SOS->CalculFFTSOS =0;
  data22SOS->GSpectInit_cp = new double[NbEchantillons];
  data22SOS->GSpecArr = new double[nFFT];
  data22SOS->GSpecArrC = new dcmplx[nFFT];
  data22SOS->GSpecArr2 = new double[nFFT];
  data22SOS->fctAmortissement = new double[nFFT];
  data22SOS->fctAmortissementArr = new double[nFFT];
  data22SOS->noyAmortissementS = new double[8*sigmaSpectre + 1];
  data22SOS->noyAmortissementI = new double[8*sigmaInterf + 1];
  // Pour les FFT
  int naux1, naux2;
  if(nFFT <= 2048)
        {
            naux1 = 22000;                          // cf doc lib ESSL p. 738 730 (max dcft drcft)
            naux2 = 20000;
        }
   else
        {
            naux1 = (int)(20000 + 2.28*nFFT);       // cf doc lib ESSL p. 738 730 (max dcft drcft)
            naux2 = (int)(20000 + 2.28*nFFT);
        }
  // FT-2765
  data22SOS->aux1 = get_AUX_SD_BAS_FFT1DirDR2C();
  data22SOS->aux12 = get_AUX_SD_BAS_FFT1DirInvDC2C();
  data22SOS->aux2 = NULL;
  data22SOS->aux22 = NULL;
  // ...FT-2765
  // Pour le rearrangement des FFT
  data22SOS->sTF = new dcmplx[nFFT];
  data22SOS->sTFt = new dcmplx[nFFT];
  data22SOS->sTFt2 = new dcmplx[nFFT];
  data22SOS->sTFdup = new dcmplx[nFFT];
  data22SOS->GSpectReechFFT = new double[(Nlast-Nfirst + 1)*surechFFT+1]; 
  data22SOS->wn = new double[(Nlast-Nfirst + 1)*surechFFT+1];
  data22SOS->oswn = new double[(Nlast-Nfirst + 1)*surechFFT*surechSpline+1];
  // FT-2765
  data22SOS->y2 = get_AUX_SD_BAS_InterpSpline1D();
  // ...FT-2765
//@----------------------------------------------------------------------*/
}

//@ ------------------------------------------------------------------*/
//@ Methode : DesallouerPointeurs22SOS
//@
//@ Description : Desalloue les pointeurs de la structure t_SD_BAS_DataThread22SOS
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@
//@ ------------------------------------------------------------------*/
void SD_GES_ChaineImageISRFEM::DesallouerPointeurs22SOS(t_SD_BAS_DataThread22SOS* data22SOS
                                                       )
{
  //@----------------------------------------------------------------------*/
  //@ desallocation pointeurs dans t_SD_BAS_DataThread22SOS
  delete [] data22SOS->GSpectInit_cp;
  delete [] data22SOS->GSpecArr;
  delete [] data22SOS->GSpecArrC;
  delete [] data22SOS->GSpecArr2;
  delete [] data22SOS->fctAmortissement;
  delete [] data22SOS->fctAmortissementArr;
  delete [] data22SOS->noyAmortissementS;
  delete [] data22SOS->noyAmortissementI;
  // FT-2765
  free_AUX_SD_BAS_FFT1DirDR2C( data22SOS->aux1 );
  free_AUX_SD_BAS_FFT1DirInvDC2C( data22SOS->aux12 );
  // ...FT-2765
  delete [] data22SOS->sTF;
  delete [] data22SOS->sTFt;
  delete [] data22SOS->sTFt2;
  delete [] data22SOS->sTFdup;
  delete [] data22SOS->GSpectReechFFT;
  delete [] data22SOS->wn;
  delete [] data22SOS->oswn;
  // FT-2765
  free_AUX_SD_BAS_InterpSpline1D( data22SOS->y2 );
  // ...FT-2765
//@----------------------------------------------------------------------*/
}

//@ ------------------------------------------------------------------*/
//@ Methode : AllouerPointeursImageISRFEM
//@
//@ Description : Alloue les pointeurs de la structure t_SD_BAS_DataThreadImageISRFEM
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@
//@ ------------------------------------------------------------------*/
void SD_GES_ChaineImageISRFEM::AllouerPointeursImageISRFEM(t_SD_BAS_DataThreadImageISRFEM* dataImageISRFEM,
                                                           t_SD_BAS_ConfAutre *ConfAutre,
                                                           t_SD_BAS_BanqueSpectrale *BanqueSpectrale,
                                                           t_SD_BAS_Contexte* contextecourant,                                                          
							   int nombreLignes
                                                          )
{
  int indFS = ConfAutre->IDefSsdSpectralWindowOPS;
  int nFFT        = MAX(ConfAutre->IWnpNsFFT[indFS] , ConfAutre->IDefS1bNsFFT);
  int surechFFT   = MAX(ConfAutre->IDefSsdOSFftFactor[indFS] ,ConfAutre->IDefS1bOSFftFactor);
  int surechSpline= MAX(ConfAutre->IDefSsdOSSplineFactor[indFS] , ConfAutre->IDefS1bOSSplineFactor);
  int sigmaSpectre= MAX(ConfAutre->IDefSsdSigS[indFS],ConfAutre->IDefS1bSigS);
  int sigmaInterf = MAX(ConfAutre->IDefSsdSigI[indFS],ConfAutre->IDefS1bSigI);
  int Nfirst      = MIN(ConfAutre->IDefSsdNsfirstSW[indFS],ConfAutre->IDefNsfirst1b);
  int Nlast       = MAX(ConfAutre->IDefSsdNslastSW[indFS],ConfAutre->IDefNslast1b);

// initialisation de l'indicateur d'initialisation (optimisation de calcul des FFT )
  dataImageISRFEM->CalculFFTSSD =0;
     // Pour SD_ALG_23_SSD  
     //========================================================================
        // Infogen = structure de sortie de 22_SOS APRES execution de 22_SOS

        dataImageISRFEM->fctAmortissement = new double[(Nlast - Nfirst)*surechSpline*surechFFT + 1];
        dataImageISRFEM->noyAmortissementS = new double[8*sigmaSpectre + 1];

        // Pour les FFT
        dataImageISRFEM->sTF = new dcmplx[nFFT];
        dataImageISRFEM->sTFdup = new dcmplx[nFFT];
        int naux1;
        int naux2;
  if(nFFT <= 2048)
        {
            naux1 = 22000;                          // cf doc lib ESSL p. 738 730 (max dcft drcft)
            naux2 = 20000;
        }
   else
        {
            naux1 = (int)(20000 + 2.28*nFFT);       // cf doc lib ESSL p. 738 730 (max dcft drcft)
            naux2 = (int)(20000 + 2.28*nFFT);
        }
        // FT-2765
        dataImageISRFEM->aux1 = get_AUX_SD_BAS_FFT1DirDR2C ();
        dataImageISRFEM->aux2 = NULL;
        // ...FT-2765
        
        dataImageISRFEM->Wno = new double[(Nlast - Nfirst)*surechSpline*surechFFT + 1];
        dataImageISRFEM->Wn = new double[(Nlast - Nfirst)*surechSpline*surechFFT + 1];
        // FT-2765
        dataImageISRFEM->y2ssd = get_AUX_SD_BAS_InterpSpline1D();
        // ...FT-2765
        dataImageISRFEM->RdSW = new double[nFFT];
        dataImageISRFEM->RdSW2 = new double[nFFT];
        
        int ColDeb  = Maximum(ConfAutre->IDefIISColDeb, SD_BAS_PN);
        int ColFin  = Maximum(ConfAutre->IDefIISColFin, SD_BAS_PN);
        int LinDeb  = Maximum(ConfAutre->IDefIISLinDeb, SD_BAS_PN);
        int LinFin  = Maximum(ConfAutre->IDefIISLinFin, SD_BAS_PN);
        
        dataImageISRFEM->Y = new double[(LinFin-LinDeb+1)*(ColFin-ColDeb+1)];
        dataImageISRFEM->Z = new double[(LinFin-LinDeb+1)*(ColFin-ColDeb+1)];
        dataImageISRFEM->ImgY = new double[(LinFin-LinDeb+1)*(ColFin-ColDeb+1)];
        dataImageISRFEM->ImgZ = new double[(LinFin-LinDeb+1)*(ColFin-ColDeb+1)];
        dataImageISRFEM->IISWgtIpsf = new double[(LinFin-LinDeb+1)*(ColFin-ColDeb+1)];
        
        int NbLin = BanqueSpectrale->IDefSdbGridNbLin;
        int NbCol = BanqueSpectrale->IDefSdbGridNbCol;
        dataImageISRFEM->Dist = new double[NbLin*NbCol];

        dataImageISRFEM->A = new double[(ConfAutre->IDefFaxRegressDegre+1)*(nombreLignes+ConfAutre->IDefFaxFLS)];
        dataImageISRFEM->B = new double[(nombreLignes+ConfAutre->IDefFaxFLS)];
        dataImageISRFEM->C = new double[(nombreLignes+ConfAutre->IDefFaxFLS)];
        dataImageISRFEM->X = new double[(nombreLignes+ConfAutre->IDefFaxFLS)];
        dataImageISRFEM->w = new double[(nombreLignes+ConfAutre->IDefFaxFLS)];
        dataImageISRFEM->AxeYInstant = new double[(nombreLignes+ConfAutre->IDefFaxFLS)];
        dataImageISRFEM->AxeZInstant = new double[(nombreLignes+ConfAutre->IDefFaxFLS)];
        dataImageISRFEM->AxeQualInstant = new double[(nombreLignes+ConfAutre->IDefFaxFLS)];
        dataImageISRFEM->FlagAxeNonQualInstant = new unsigned short int[(nombreLignes+ConfAutre->IDefFaxFLS)];
        dataImageISRFEM->AxeYFiltered = new double[(nombreLignes+ConfAutre->IDefFaxFLS)];
        dataImageISRFEM->AxeZFiltered = new double[(nombreLignes+ConfAutre->IDefFaxFLS)];
        dataImageISRFEM->tabTmpMC = new double[MAX(ConfAutre->IDefFaxRegressDegre+1 , (nombreLignes+ConfAutre->IDefFaxFLS))];
}

//@ ------------------------------------------------------------------*/
//@ Methode : DesallouerPointeursImageISRFEM
//@
//@ Description : Desalloue les pointeurs de la structure t_SD_BAS_DataThreadImageISRFEM
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@
//@ ------------------------------------------------------------------*/
void SD_GES_ChaineImageISRFEM::DesallouerPointeursImageISRFEM(t_SD_BAS_DataThreadImageISRFEM* dataImageISRFEM
                                                             )
{
  delete [] dataImageISRFEM->GIrcImg;
  delete [] dataImageISRFEM->fctAmortissement;
  delete [] dataImageISRFEM->noyAmortissementS;
  delete [] dataImageISRFEM->sTF;
  delete [] dataImageISRFEM->sTFdup;
  // FT-2765
  free_AUX_SD_BAS_FFT1DirDR2C( dataImageISRFEM->aux1 );
  // ...FT-2765
  delete [] dataImageISRFEM->Wno;
  delete [] dataImageISRFEM->Wn;
  // FT-2765
    free_AUX_SD_BAS_InterpSpline1D( dataImageISRFEM->y2ssd );
    // ...FT-2765
  delete [] dataImageISRFEM->RdSW;
  delete [] dataImageISRFEM->RdSW2;
  delete [] dataImageISRFEM->Y;
  delete [] dataImageISRFEM->Z;
  delete [] dataImageISRFEM->ImgY;
  delete [] dataImageISRFEM->ImgZ;
  delete [] dataImageISRFEM->IISWgtIpsf;
  delete [] dataImageISRFEM->Dist;
  delete [] dataImageISRFEM->A;
  delete [] dataImageISRFEM->B;
  delete [] dataImageISRFEM->C;
  delete [] dataImageISRFEM->X;
  delete [] dataImageISRFEM->w;
  delete [] dataImageISRFEM->AxeYInstant;
  delete [] dataImageISRFEM->AxeZInstant;
  delete [] dataImageISRFEM->AxeQualInstant;
  delete [] dataImageISRFEM->FlagAxeNonQualInstant;
  delete [] dataImageISRFEM->AxeYFiltered;
  delete [] dataImageISRFEM->AxeZFiltered;
  delete [] dataImageISRFEM->tabTmpMC;
}

//@ ------------------------------------------------------------------*/
//@ Methode : Maximum
//@
//@ Description : Retourne la valeur max d'un tableau
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@
//@ ------------------------------------------------------------------*/
int SD_GES_ChaineImageISRFEM::Maximum(int *tableau,
                                      int dimension
                                      )
{
  int maxi = tableau[0];
  for(int i = 1; i < dimension; i++)
    maxi = MAX(maxi, tableau[i]);
  return(maxi);
}
//@ ------------------------------------------------------------------*/
//@ Methode : Traiter
//@
//@ Description : Permet de traiter les ctx
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@
//@ ------------------------------------------------------------------*/
void SD_GES_ChaineImageISRFEM::Traiter(t_CMN__Statut& cr_op,
                                       t_SD_BAS_DataThread *ctx
                                      )
{
  //@ SI erreur
  if(cr_op != CMN__STATUT_OK)
  {
    //@ emission des messages du ctx
    EnvoyerMessagesStockes(&(ctx->cr.msgJDB));
    InitialiserListeErreurs(&(ctx->cr.errinterne));
  }
}
