static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/GES/src/SD_GES_FiltrageAxeInterf.cc,v 1.4 2013/03/29 16:55:26 lcaspar Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_GES
//@ CLASSE : SD_GES_FiltrageAxeInterf
//@ 
//@ Comment :
//@    this module contains the first action used for the IASI OPS 
//@     L1 Processing chain level 0-1C
//@                                     
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 10/03/2003 : P.GASSIE
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-066-CN : 7/11/2003 : JL.PASCAL
//@ le spectre est vide dans le cas de lignes dummy a cause d'un mauvais indicage des lignes
//@ le nombre de MDR non dummy est calcule dans  SD_GES_FiltrageAxeInterf
//@ et utilise dans SD_GES_ChaineProdxx_xx
//@ (cf IA-2100-FC-066-THA)
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-163-CN : 7/11/2003 : JL.PASCAL
//@ core dans le cas du mode dump
//@ (cf IA-2100-FC-163-THA)
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-196-CN : 15/01/2007 : C.MERIGUET
//@ FA-ID : IA-2100-FA-198-CN : 15/01/2007 : C.MERIGUET
//@ Ces 2 FA sont traitees conjointement. Recuperation du PTSI lu dans 
//@ le mot 17 (2 octets) mais ecriture dans le champ PTSI des N1 des mots 
//@ 16 (version logiciel bord) et 17 (ptsi) (4 octets).
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-209-CN : 08/03/2007 : S. BRUNEL
//@ Extension du test sur NbLignesValides  a tout le bloc concerne.
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2529(DM-2100-OPS) : 29/04/2009 : S. BRUNEL
//@ Implementation de valeurs par defaut pour l'axe interferometrique
//@ dans le fichier de "contexte a froid" (Day 2,point 10)
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2651(FA-2100-OPS) : 08/07/2009 : S. BRUNEL
//@ Plantage aleatoire de l'OPS lors de l'utilisation d'un contexte a froid
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2668(FA-2100-OPS) : 14/10/2009 : S. BRUNEL
//@ Pas d'initialisation de numeroLigneGES dans SD_GES_FiltrageAxeInterf.cc 
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-3095(DM-2100-OPS) : 27/03/2013 : P. JULIAN
//@ Amelioration de la position de l axe filtre SD_ALG_25_FAX
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------


//
// Fichiers include
//

// FT-2529
#include <math.h>
// ...FT-2529
 
// STL
#include <iostream>

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN__Chaine.h"

#include "SD_TAC_Tache.h"
#include "SD_GES_FiltrageAxeInterf.h"
#include "SD_FRW_ApplicationServeurDonnees.h"
#include "SD_ALG_ISRFEM.h"

//======================================================================
//                                       METHODES PUBLIQUES 
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD_GES_FiltrageAxeInterf
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               (ENTREE) tache : Tache pere de l'action
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_GES_FiltrageAxeInterf::SD_GES_FiltrageAxeInterf(SD_TAC_Tache* tache)
  : SD_ACT_Action(tache)
{
  //@ Memoriser la tache parente de l'action
  m_Tache = tache;
  m_PeutEtreEnchainee = CMN__FAUX;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_GES_FiltrageAxeInterf
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_GES_FiltrageAxeInterf::~SD_GES_FiltrageAxeInterf()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Executer
//@
//@ Description : Traitement principal de l'action
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_GES_FiltrageAxeInterf::Executer()
{
t_CMN__Statut cr_op = CMN__STATUT_OK;
  //@ Methode virtuelle surchargee pour executer la premiere partie de 
  //@ la chaine  image
  // FT-2765
  cout << "SD_GES_FiltrageAxeInterf pour la tache " << Tache()->Id() << endl;
  // ...FT-2765

  //@ principes generaux a appliquer :
  //@ 0 - on considere que l'initialisation des traces a ete faite au prealable
  //@     dans  InitGranule, la structure t_SD_BAS_DataGen est passe aux 
  //@ algorithmes qui emettent eux meme leur traces
  //@ 1 - c'est a ce niveau que sont emis les messages au JdBS ( JDB 
  //@   + trace  applicatives ) via :
  //@  des blocs try catch (AC) et la remontee d'erreurs des algo
  //@   les erreurs de base sont emises depuis les couches inferieures
  //@   via une couche de base en C encapsule dans la tache
  //@  SD_ACT_ACtion.journaliser
  //@    si une erreur est rencontree en C - appel fonction C (pas de static
  //@    car multithreading !!)
  //@    si une erreur est rencontree en C++ - appel methode C++ 
  //@ 2 - la memoire utilisee est de 2 type :
  //@   - partagee entre thread elle a alors ete allouee au prealable 
  //@   - locale au traitement, elle est alors allouee en debut puis liberee
  //@     en fin de methode 

//@ recuperer le serveur de donnees reference dans application serveur de donnee
  SD_FRW_ApplicationServeurDonnees* application;
  application = m_Tache->Application();

//@----------------------------------------------------------------------*/
//@ recuperer les donnees temporaire globales initialisees */
//@ appel GetDonGenGranule 
//@ contexte + donnes generale YY_CONF
//@----------------------------------------------------------------------*/
  SD_DON_GestDonnees* gestionnaire;
  t_SD_BAS_Contexte* contextecourant;
  t_SD_BAS_DataGen  * datagen ;
  gestionnaire = application->Donnees();
  gestionnaire->GetDonGenGranule(&contextecourant, &datagen);
//@ recuperer le nombre de ligne du granule */
//@ appel GetNbLignesIASI
  int NbLignesIASI;
  NbLignesIASI = gestionnaire->GetNbLignesIASI();

  //@ mise a jour du nombre de MDR OK dans la partie GeneratedFile
  gestionnaire->GetTraitement().GeneratedFile().MdrOk(NbLignesIASI);

  //@ mise a jour des trous de donnees de type 05
  for (int noLigne = 1; noLigne < (NbLignesIASI + 1); noLigne++)
    {
      bool tableauCasDeg[SD_BAS_NB_CGXX];
      SD_DON_CasDegLigne* casDegLigneCourant = 
        gestionnaire->GetGestCasDeg().GetCasDegLigne(noLigne);

      casDegLigneCourant->IsCasDeg(tableauCasDeg);
      if (tableauCasDeg[1] == true)
        {
          t_DataGap dataGap;
          dataGap.Reason(CMN_DAT_DataGap::GAP_REASON_FIVE);
          dataGap.Count(1);
          gestionnaire->GetTraitement().GeneratedFile().AjouterDataGap(dataGap);
        }
    }

  //@ allocation tableau de pointeurs sur des lignes
  t_SD_BAS_LigneN1 ** tableauLigneMDRENG;
  tableauLigneMDRENG = new t_SD_BAS_LigneN1*[NbLignesIASI];
  t_SD_BAS_Liste_Var* tableauListeVar;
  tableauListeVar = new t_SD_BAS_Liste_Var[NbLignesIASI];
  //@ recuperer les donnees de niveau 1 initialisees ligne par ligne
  int nblig=0;
  int PTSI;
  t_Date datePremiereLigne;
  
  //@ Nombre de lignes valides ie non DMDR;
  int NbLignesValides = 0;
  
  for(int i = 0; i < NbLignesIASI; i++)
  {
    //@ initialisation ligne i
    tableauLigneMDRENG[i] = new t_SD_BAS_LigneN1;
    //@ appel GetLigneS1 (ENG)
    t_SD_BAS_MPHR * MphrENG;
    t_SD_BAS_LigneN1 * VIADRENG;
    MphrENG    = new t_SD_BAS_MPHR;
    VIADRENG   = new t_SD_BAS_LigneN1;
    gestionnaire->GetLigneS1(
      i, 
      SD_DON_GranuleN1::SD_DON_GRANULE_N1_ENG, 
      *MphrENG, 
      *(tableauLigneMDRENG[i]), 
      *VIADRENG);

// gestion des DMDR
    if (tableauLigneMDRENG[i]->TypeRec != MDR_DUMMY)
    {
    NbLignesValides++;
    tableauListeVar[nblig].nb_var = tableauLigneMDRENG[i]->InfoDet.nb_var;
    tableauListeVar[nblig].tab_var = tableauLigneMDRENG[i]->InfoDet.tab_var;
//@ Recuperation du PTSI et de la date
// FA 196 et 198
     //inversion de mots 
     int version_log = (int)(tableauLigneMDRENG[i]->InfoDet.tab_var[ENG_PTSI].val.pui4[0]/65536);
     PTSI = tableauLigneMDRENG[i]->InfoDet.tab_var[ENG_PTSI].val.pui4[0] - (version_log * 65536);
    //PTSI = (int)(tableauLigneMDRENG[i]->InfoDet.tab_var[ENG_PTSI].val.pui4[0]/65536);
// ..FA 196 et 198
    datePremiereLigne.SCDSTime(tableauLigneMDRENG[i]->InfoDet.tab_var[ENG_DATEDEB].val.pdate[0]);
    nblig++;
    }
    delete MphrENG;
    delete VIADRENG;
  }
  //  FA 66 on renseigne ici le nombre total de MDR non dummy  dans le produit 
  // ( pour (SD_GES_Prodxx_xx)
  datagen->NbLignesNonDummy=NbLignesValides;
  //@ recuperer les donnees de configuration en fonction du PTSI de la premiere ligne
  t_SD_BAS_ConfAutre *  ConfAutre ;
  t_SD_BAS_ConfStable * ConfStable ;
  t_SD_BAS_BanqueSpectrale * BanqueSpectrale ;

  // FA 209
  //@ appel GetConf
  if(NbLignesValides > 0)
  {
    gestionnaire->GetConf(
      PTSI, 
      datePremiereLigne, 
      &ConfAutre, 
      &ConfStable, 
      &BanqueSpectrale);
 
  //@ initialisation ctx
  t_SD_BAS_DataThread *ctx;
  ctx = new t_SD_BAS_DataThread;
  ctx->ctxtrt.LN=0;
  ctx->ctxtrt.SN=0;
  ctx->ctxtrt.PN=0;
  ctx->ctxtrt.CCD=0;
  ctx->ctxtrt.SB=0;
  InitialiserCompteRendu(&(ctx->cr));
  
  //@ allocation pointeurs dans t_SD_BAS_DataThreadImageISRFEM
  t_SD_BAS_DataThreadImageISRFEM* dataImageISRFEM;
  dataImageISRFEM = new t_SD_BAS_DataThreadImageISRFEM;
  dataImageISRFEM->A = new double[(ConfAutre->IDefFaxRegressDegre+1)*(NbLignesIASI+ConfAutre->IDefFaxFLS)];
  dataImageISRFEM->B = new double[NbLignesIASI+ConfAutre->IDefFaxFLS];
  dataImageISRFEM->C = new double[NbLignesIASI+ConfAutre->IDefFaxFLS];
  dataImageISRFEM->X = new double[NbLignesIASI+ConfAutre->IDefFaxFLS];
  dataImageISRFEM->w = new double[NbLignesIASI+ConfAutre->IDefFaxFLS];
  dataImageISRFEM->AxeYInstant = new double[NbLignesIASI+ConfAutre->IDefFaxFLS];
  dataImageISRFEM->AxeZInstant = new double[NbLignesIASI+ConfAutre->IDefFaxFLS];
  dataImageISRFEM->AxeQualInstant = new double[NbLignesIASI+ConfAutre->IDefFaxFLS];
  dataImageISRFEM->FlagAxeNonQualInstant = new unsigned short int[NbLignesIASI+ConfAutre->IDefFaxFLS];
  dataImageISRFEM->AxeYFiltered = new double[NbLignesIASI+ConfAutre->IDefFaxFLS];
  dataImageISRFEM->AxeZFiltered = new double[NbLignesIASI+ConfAutre->IDefFaxFLS];
  dataImageISRFEM->tabTmpMC = new double[(ConfAutre->IDefFaxRegressDegre+1)*(NbLignesIASI+ConfAutre->IDefFaxFLS)];
  //@ POUR chaque CCD
    for(int ccd = 0; ccd < SD_BAS_CCD; ccd++)
    {
      ctx->ctxtrt.CCD = ccd;

    // FT-2529
    if (contextecourant->GCTXNatCTX == INIT_FROID )
    {
      int numeroLigneGES;
      double *WnShiftMean;
      double *GridWnShift;
      int NbLin;
      int NbCol;
      double *GridY;
      double *GridZ;
      int indFS;              // Indice de la fenetre spectrale dans laquelle on travaille
      double *Dist;
      double DY;
      double DZ;
      double DistPix[SD_BAS_PN];
      double minDist;

      int ct;
      int il;
      int ic;
      int i;
      int nlin0;              // Ligne du point de la grille de distance minimum
      int ncol0;              // Colonne du point de la grille de distance minimum
      int npix;
      double PdsPix[SD_BAS_PN];
      int ILS;
      int FLS;

	  // FT-2668
      numeroLigneGES = m_Tache->Contexte()->IndiceLigne()-1;
	  // ...FT-2668

      // FT-2651
      unsigned int *FlagNonSel = NULL;
      FlagNonSel = tableauLigneMDRENG[numeroLigneGES]->InfoDet.tab_var[ENG_GSSSFLAGNONSELPX].val.pui4;
      // ...FT-2651
 
      WnShiftMean = tableauLigneMDRENG[numeroLigneGES]->InfoDet.tab_var[ENG_GSSSWNSHIFTMEAN].val.pr8;

      GridWnShift = BanqueSpectrale->ISdbGridWnShift;
      NbLin = BanqueSpectrale->IDefSdbGridNbLin;
      NbCol = BanqueSpectrale->IDefSdbGridNbCol;
      GridY = BanqueSpectrale->IDefSdbGridY;
      GridZ = BanqueSpectrale->IDefSdbGridZ;

      indFS = ConfAutre->IDefSsdSpectralWindowOPS;
      
      ILS = nblig;
      FLS = MIN(ConfAutre->IDefFaxFLS, abs(contextecourant->GIaxNbElt - ILS));

      Dist = new double[NbLin*NbCol];
      if (Dist == NULL)
      {

      }

      // FT-2651
       //@ Pour chacun des pixels npix = 1 a 4
      for(npix=1; npix<=SD_BAS_PN; npix++)
      {
        PdsPix[IND1(npix)] = ConfAutre->IDefPdsPix[IND1(npix)];

        //@   Si FlagNonSel(npix) = oui alors pdspix(npix) = 0
        if(FlagNonSel[IND2(npix,ccd+1,SD_BAS_PN)] == CMN__OUI)
        {
          PdsPix[IND1(npix)] = 0.0;
        }
      }
      // ...FT-2651

      //@ Pour chacun des pixels npix = 1 a 4
      for(npix=1; npix<=SD_BAS_PN; npix++)
      {
        PdsPix[IND1(npix)] = ConfAutre->IDefPdsPix[IND1(npix)];

        //@   Si FlagNonSel(npix) = oui alors pdspix(npix) = 0
        if(FlagNonSel[IND2(npix,ccd+1,SD_BAS_PN)] == CMN__OUI)
        {
          PdsPix[IND1(npix)] = 0.0;
        }
      }
      // ...FT-2651
      //@ Pour chaque point de la grille (nlin,ncol) on calcule le carre de 
      //@  la distance entre les decalages spectraux mesures et le calcul
      for(il=1, ct=0; il<=NbLin; il++)
      {
        for(ic=1; ic<=NbCol; ic++, ct++)
        {
            //@   Pour chaque pixel npix de 1 a Npix
            for(npix=1; npix<=SD_BAS_PN; npix++)
            {
                //@     DistPix(npix)=(WnShiftMean(npix)- GridWnShift(nlin,ncol,npix))**2
                // FT-2651
                DistPix[IND1(npix)] = (WnShiftMean[IND2(npix,ccd+1,SD_BAS_PN)] - 
                            GridWnShift[IND5(npix,ccd+1,indFS,ic,il,SD_BAS_PN,SD_BAS_CDX,SD_BAS_WNP_NB_SW,SD_BAS_SDB_NB_COL)]) *
                                      (WnShiftMean[IND2(npix,ccd+1,SD_BAS_PN)] - 
                            GridWnShift[IND5(npix,ccd+1,indFS,ic,il,SD_BAS_PN,SD_BAS_CDX,SD_BAS_WNP_NB_SW,SD_BAS_SDB_NB_COL)]);
                // ...FT-2651
            }
            //@   Fin de boucle sur les pixels

            //@   Dist(nlin,ncol) = Sigma pixels (PdsPix(npix).DistPix(npix))
            Dist[ct]=0.0;
            for(npix=1; npix<=SD_BAS_PN; npix++)
            {
                Dist[ct] += PdsPix[IND1(npix)] * DistPix[IND1(npix)];
            }
            Dist[ct] = Dist[ct] / (double) SD_BAS_PN;
        }
      }
      //@ Fin de boucle sur les points de grille
 
      //@ Recherche du point de grille de distance minimum nlin0 et ncol0
      //@  tels que Dist(nlin0,ncol0)=min(Dist(nlin,ncol))
      nlin0=1;
      ncol0=1;
      minDist=Dist[0];
      for(il=1, ct=0; il<=NbLin; il++)
      {
        for(ic=1; ic<=NbCol; ic++, ct++)
        {
            if(Dist[ct] < minDist)
            {
                nlin0=il;
                ncol0=ic;
                minDist = Dist[ct];
            }
        }
      }

      //@ Pas de la grille
      //@   DY=GridY(ncol0+1)-GridY(ncol0-1)/2
      //@   DZ=GridZ(nlin0-1)-GridZ(nlin0+1)/2
      // FT-2651
      DY=(GridY[IND2(ccd+1,ncol0+1,SD_BAS_CDX)]-GridY[IND2(ccd+1,ncol0-1,SD_BAS_CDX)]) / 2.0;
      DZ=(GridZ[IND2(ccd+1,nlin0+1,SD_BAS_CDX)]-GridZ[IND2(ccd+1,nlin0-1,SD_BAS_CDX)]) / 2.0;
      // ...FT-2651
 
      for(i=0; i<ILS+FLS; i++)
      {
        // FT-2651
        // Default value of interferometric axis Y
        contextecourant->GIaxHistAxeInterf[i*SD_BAS_CCD+ccd].GiaxAxeYElt = ConfStable->IDefFaxGuessY[ccd]; 

        // Default value of interferometric axis Z
        contextecourant->GIaxHistAxeInterf[i*SD_BAS_CCD+ccd].GiaxAxeZElt = ConfStable->IDefFaxGuessZ[ccd]; 

        // Default value of AxeQual  (cf 24_IAX)
        contextecourant->GIaxHistAxeInterf[i*SD_BAS_CCD+ccd].GiaxAxeQualElt = 0.25 * sqrt(fabs(DY*DZ));

        // Defaut value of FlagAxeNonQual
        contextecourant->GIaxHistAxeInterf[i*SD_BAS_CCD+ccd].GiaxFlagAxeNonQual = CMN__NON;
        // ...FT-2651
      }  

      if (Dist != NULL)
      {
        delete [] Dist; 
        Dist = NULL;
      }
    } // fin si context a froid 
    // ...FT-2529
 		
	 
  	//@ Filtrage de l axe interferometrique (A1335)
    //@ Call 25_FAX 
    // FT-3095
    cr_op = SD_ALG_25_FAX(ctx,
                          dataImageISRFEM,
                          contextecourant,
                          nblig,
                          datePremiereLigne.SCDSTime(),
                          ConfAutre,
                          ConfStable,
                          tableauListeVar
			);
  	// ... FT-3095
  	
    //@ traitement du compte-rendu
    Traiter(cr_op, ctx);
    }//@ FIN POUR

  LibererCompteRendu(&(ctx->cr.msgJDB));
  delete ctx;
      //@ liberation pointeurs dans   t_SD_BAS_DataThreadProduit
  delete [] dataImageISRFEM->A;
  delete [] dataImageISRFEM->B;
  delete [] dataImageISRFEM->C;
  delete [] dataImageISRFEM->X;
  delete [] dataImageISRFEM->w;
  delete [] dataImageISRFEM->AxeYInstant;
  delete [] dataImageISRFEM->AxeZInstant;
  delete [] dataImageISRFEM->AxeQualInstant;
  delete [] dataImageISRFEM->FlagAxeNonQualInstant;
  delete [] dataImageISRFEM->AxeYFiltered;
  delete [] dataImageISRFEM->AxeZFiltered;
  delete [] dataImageISRFEM->tabTmpMC;
  delete dataImageISRFEM;

  } // FIN IF NbLignesValides > 0
  // FIN FA 209
  
  //@ liberation tableau de pointeurs sur des lignes
  for(int i = 0; i < NbLignesIASI; i++)
    delete tableauLigneMDRENG[i];
  delete [] tableauLigneMDRENG;
  delete [] tableauListeVar;
  //@ Invoquer la verification des symptomes une fois l'action
  //@ executee
  VerifierSymptome();
}


//@ ------------------------------------------------------------------*/
//@ Methode : VerifierSymptome
//@
//@ Description : Verifie si les symptomes associes a l'action sont
//@               en erreur ou non
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_GES_FiltrageAxeInterf::VerifierSymptome()
{
  //@ Methode virtuelle a surcharger dans les classes derivees
  //@ Les symptomes controles sont specifiques a l'action
        
  m_PeutEtreEnchainee = CMN__VRAI;
}


//======================================================================
//                                      METHODES PRIVEES 
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Methode : Traiter
//@
//@ Description : Permet de traiter les ctx
//@
//@ Parametres :
//@
//@ Valeur de retour : 
//@
//@ ------------------------------------------------------------------*/
void SD_GES_FiltrageAxeInterf::Traiter(t_CMN__Statut& cr_op,
                                       t_SD_BAS_DataThread *ctx
                                      )
{
  //@ SI erreur
  if(cr_op != CMN__STATUT_OK)
  {
    //@ emission des messages du ctx
    EnvoyerMessagesStockes(&(ctx->cr.msgJDB));
    InitialiserListeErreurs(&(ctx->cr.errinterne));
  }
}
