static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/SRV/src/SD_SRV_ServeurEvenements.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_SRV
//@ CLASSE : SD_SRV_ServeurEvenements
//@ 
//@ Commentaire :
//@    Ce module gere la classe SD_SRV_ServeurEvenements
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Adaptation a l'OPS : modification du journal de bord
//@ --------------------------------------------------------------------
//@ FAID<19>    30/11/99    GRT      Creation d'un nouveau JDB a chaque changement de jour
//@ SMM-M4-FA-2321-CN       30/08/2001 Robert   Il faut verrouiller avant la modification de tout fichier
//@ --------------------------------------------------------------------
//@ VERSION : 24/08/1999 : SAS
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL
#include <iostream>

// Standard

// Projet
#include "CMN__Commun.h"
#include "JDB__Callback.h"
#include "JDB__Client.h"
#include "SD_FRW_ApplicationServeurDonnees.h"
#include "SD_SRV_ServeurEvenements.h"


//
// Definition des constantes
//
const t_Chaine SD_SRV_ServeurEvenements::JDB_PARAM_1 = "$PRM1";
const t_Chaine SD_SRV_ServeurEvenements::JDB_PARAM_2 = "$PRM2";
const t_Chaine SD_SRV_ServeurEvenements::JDB_PARAM_3 = "$PRM3";


//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD_SRV_ServeurEvenements
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_SRV_ServeurEvenements::SD_SRV_ServeurEvenements()
  : m_pApplication(0)
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_SRV_ServeurEvenements
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_SRV_ServeurEvenements::~SD_SRV_ServeurEvenements()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Initialiser
//@
//@ Description : Initialise le serveur d'evenements
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_SRV_ServeurEvenements::Initialiser()
{
  static JDB__Callback msgCallback;
  JDB__Client messageJDB;
  
  //@ Installer les callback qui permettent d'envoyer les msg JDB
  msgCallback.SetConnexion(m_pApplication->Session());
  
  //@ Initialisation du set de message
  messageJDB.InitSet(SD_SET,
                     SD__Message::MSG_SD_START,
                     SD__Message::MSG_FIN);
}


//@ ------------------------------------------------------------------*/
//@ Methode : AnalyserMessage
//@
//@ Description : Teste si le message est destine au serveur
//@               d'evenement. Si oui, le message est traite
//@
//@ Parametres :
//@               (ENTREE) message : Message a traiter
//@
//@ Valeur de retour : 
//@               CMN__STATUT_OK si le message est destine au 
//@                    serveur d'evenement
//@               CMN__STATUT_NOK si le message n'est pas destine 
//@                    au serveur d'evenement
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut 
SD_SRV_ServeurEvenements::AnalyserMessage(const MSG__Message& message)
{
  t_CMN__Statut statut = CMN__STATUT_NOK;


  //@ A l'heure actuelle, aucun message n'est traite par le 
  //@ serveur de message
  switch (message.GetType())
    {
    default :
      {
        statut = CMN__STATUT_NOK;
        break;
      }
    }
  
  return statut;
}


//@ ------------------------------------------------------------------*/
//@ Methode : JournaliserEvenement
//@
//@ Description : Enregistre dans le journal le nouvel evenement
//@
//@ Parametres :
//@               (ENTREE) evenement : Evenement a enregistrer
//@
//@ Valeur de retour : 
//@                Aucune
//@
//@ ------------------------------------------------------------------*/
void 
SD_SRV_ServeurEvenements::JournaliserEvenement(CMN_DAT_Evenement& evenement)
{
  JDB__Client message;

  //@ Renseignement des parametres du message
  if (! evenement.Prm1().empty())
    {
      message.SetParametre(JDB_PARAM_1, evenement.Prm1());
      if (! evenement.Prm2().empty())
        {
          message.SetParametre(JDB_PARAM_2, evenement.Prm2());
          if (! evenement.Prm3().empty())
            {
              message.SetParametre(JDB_PARAM_3, evenement.Prm3());
            }
        }
    }

  //@ Envoi du message de type JDB
  message.EcrireMessage(SD_SET, evenement.TypeEvenement());
  message.Liberer();
}


//======================================================================
//                                      METHODES PRIVEES
//======================================================================
