static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/TAC/src/SD_TAC_GestionnaireDeSession.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_TAC
//@ CLASSE : SD_TAC_GestionnaireDeSession
//@ 
//@ Commentaire :
//@    Ce module gere la classe SD_TAC_GestionnaireDeSession
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 01/09/1999 : SAS
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FAID<003>   18/11/99    Agnetti  modif nom du shell de lancement ADM
//@ --------------------------------------------------------------------
//@ FAID<020>   24/11/99    Agnetti  strcpy->strncpy
//@ --------------------------------------------------------------------
//@ FAID<042>   24/11/99    Bobin    ADM_Lancer.sh
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Adaptation a l'OPS : modification des messages geres par le
//@                      gestionnaire de session
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL

// Standard
#include <unistd.h>

// Projet
#include "CMN__Commun.h"
#include "CMN__Fichier.h"
#include "MSG__Message.h"
#include "SD_FRW_ApplicationServeurDonnees.h"
#include "SD_TAC_GestionnaireDeSession.h"


//
// Definition des constantes
//
const unsigned int SD_TAC_GestionnaireDeSession::TIMEOUT_ATTENTE_FIN = 300000;


//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD_TAC_GestionnaireDeSession
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_TAC_GestionnaireDeSession::SD_TAC_GestionnaireDeSession()
  : m_Continuer(CMN__VRAI)
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_TAC_GestionnaireDeSession
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_TAC_GestionnaireDeSession::~SD_TAC_GestionnaireDeSession()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Executer
//@
//@ Description : Routine de traitement du gestionnaire de session
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_TAC_GestionnaireDeSession::Executer()
{
  MSG__Message    message;
  t_CMN__Booleen  messagePresent = CMN__FAUX;
  t_CMN__Booleen  aEteTraite = CMN__FAUX;


  //@ TANT QUE l'on doit continuer FAIRE
  while (m_Continuer == CMN__VRAI) 
    {
      //@ Attendre un message
      messagePresent = AttendreMessage(message);

      if (messagePresent == CMN__VRAI)
        {
          //@ Interpreter le message
          Application()->Verrouiller();   
          aEteTraite = InterpreterMessage(message);
          Application()->Deverrouiller();  

          //@ SI le message n'a pas ete traite localement
          if (aEteTraite == CMN__FAUX) 
            {
              //@ Diffuser le message pour traitement
              Application()->Verrouiller();    // IMPORTANT
              DiffuserMessage(message);
              Application()->Deverrouiller(); 
            }
          //@ FIN SI
        }
    }
  //@ FIN TANT QUE

  //@ Demander l'arret des taches en cours
  //  Application()->Verrouiller();    // IMPORTANT
//  Application()->Echeancier()->Deactiver();
 // Application()->Deverrouiller();  // IMPORTANT

  //@ Attendre que toutes les taches se terminent
  while (Application()->CompterLesTachesEnCours() > 1) 
    {
      usleep(TIMEOUT_ATTENTE_FIN);
    } 

  //@ Arreter l'application
  Application()->Arreter();
}


//======================================================================
//                                      METHODES PRIVEES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : AttendreMessage
//@
//@ Description : Routine d'attente de message aupres du gestionnaire
//@               de session
//@
//@ Parametres :
//@               (SORTIE) message : Message recu
//@
//@ Valeur de retour : 
//@               t_CMN__Booleen : Un message a ete recu ou non
//@
//@ ------------------------------------------------------------------*/
t_CMN__Booleen SD_TAC_GestionnaireDeSession::AttendreMessage(MSG__Message&
                                                             message) 
{
  //@ Message recu
  t_CMN__Booleen   messagePresent = CMN__FAUX;

  //@ Liste des descripteurs actifs
  MSG__Connexion::t_ListeDescripteur descripteurActif;


  //@ TANT QUE aucun message present ET l'on peut continuer FAIRE
  while ((descripteurActif.size() != 1) && (m_Continuer == CMN__VRAI)) 
    {
      //@ Appel bloquant scruter sur la connexion serveur
      Application()->Session()->Scruter(descripteurActif);

      //@ Verifier si une condition de terminaison n'est pas arrivee
      //@ SI oui ALORS
      if (VerifierConditionArret() == CMN__VRAI) 
        {
          //@ Indiquer qu'il ne faut plus continuer
          m_Continuer = CMN__FAUX;
        }
      //@ FIN SI
    }
  //@ FIN TANT QUE
 
  //@ SI l'on peut continuer ALORS
  if ((m_Continuer == CMN__VRAI))
    {
      //@ Lire le message
      messagePresent = CMN__VRAI;
      Application()->Session()->LireMessage(message);
    }
  //@ FIN SI

  return messagePresent;
}


//@ ------------------------------------------------------------------*/
//@ Methode : InterpreterMessage
//@
//@ Description : Routine d'interpretation de message en provenance 
//@               du gestionnaire de session
//@
//@ Parametres :
//@               (ENTREE) message : Message a interpreter
//@
//@ Valeur de retour : 
//@               t_CMN__Booleen : Le message a ete traite ou non
//@
//@ ------------------------------------------------------------------*/
t_CMN__Booleen 
SD_TAC_GestionnaireDeSession::InterpreterMessage(const MSG__Message& message)
{
  t_CMN__Booleen estTraite = CMN__VRAI;


#ifdef INVESTIGATION
  //@ INVESTIGATION: Envoyer un message de type JDB MSG_SD_MESSAGE
  Application()->EnvoyerMessageJDB(SD__Message::MSG_SD_MESSAGE, 
                                   message.GetType());
#endif
  
  //@ Le traitement des requetes est delegue a l'application
  //@ Sauf l'arret du SD

  //@ SELON le type du message
  switch (message.GetType())
    {
    //@ CAS evenement FIN:
    case MSG__Message::FIN :
      {
        //@ Alors on positionne le flag de fin d'execution
        m_Continuer = CMN__FAUX;
        break;
      }
    
    //@ CAS defaut: le message n'est pas traite
    default :
      {
        estTraite = CMN__FAUX;
        break;
      }
    }
  //@ FIN SELON

  return estTraite;
}


//@ ------------------------------------------------------------------*/
//@ Methode : DiffuserMessage
//@
//@ Description : Routine de diffusion de message en provenance 
//@               du gestionnaire de session
//@
//@ Parametres :
//@               (ENTREE) message : Message a diffuser
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_TAC_GestionnaireDeSession::DiffuserMessage(const MSG__Message&
                                                   message)
{
  //@ Diffuser le message a l'application pour traitement
  Application()->DiffuserMessage(message);
}
