static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/TAC/src/SD_TAC_Tache.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_TAC
//@ CLASSE : SD_TAC_Tache
//@ 
//@ Commentaire :
//@    Ce module gere la classe SD_TAC_Tache
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 30/08/1999 : SAS
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Adaptation a l'OPS : modification des attributs d'une tache
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL

// Standard

// Projet
#include "CMN__Commun.h"
#include "SD_FRW_ApplicationServeurDonnees.h"
#include "SD_TAC_Tache.h"


//
// Definition des constantes
//


//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD_TAC_Tache
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_TAC_Tache::SD_TAC_Tache()
  : m_Id(-1), m_NomTache(""), m_TypeTache(""), m_Contexte(0),
    m_Application(0), m_Simulation(-1), m_ActionCourante(""),
    m_EstActive(CMN__FAUX), m_EstTerminee(CMN__FAUX), 
    m_Arreter(CMN__FAUX)
{
  //@ Initialiser la liste des actions a vide
  m_Actions.clear();

  //@ Initialiser les proprietes de l'objet aux valeurs par defaut
  m_DateCreation.DateInit();
  m_DateDebut.DateInit();
  m_DateFin.DateInit();
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_TAC_Tache
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_TAC_Tache::~SD_TAC_Tache()
{
  t_ListeAction::iterator i;

  //@ Liberer la liste des actions
  for (i = m_Actions.begin(); i!= m_Actions.end(); i++)
    {
      delete (*i);
    }

  m_Actions.clear();
}


//@ ------------------------------------------------------------------*/
//@ Methode : Executer
//@
//@ Description : Traitement principal de la tache
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_TAC_Tache::Executer()
{
  //@ Methode canevas a surimplementer dans les classes derivees
  sleep(2);
}


//@ ------------------------------------------------------------------*/
//@ Methode : TracerDemarrage
//@
//@ Description : Trace le demarrage de la tache
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_TAC_Tache::TracerDemarrage()
{
  //@ Verrouiller l'application
  Application()->Verrouiller();

  //@ Methode virtuelle a surimplementer

  //@ Deverrouiller l'application
  Application()->Deverrouiller();
}


//@ ------------------------------------------------------------------*/
//@ Methode : TracerFin
//@
//@ Description : Trace la terminaison de la tache
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_TAC_Tache::TracerFin()
{
  //@ Verrouiller l'application
  Application()->Verrouiller();

  //@ Methode virtuelle a surimplementer

  //@ Deverrouiller l'application
  Application()->Deverrouiller();
}


//@ ------------------------------------------------------------------*/
//@ Methode : EstTerminee
//@
//@ Description : Indique si la tache s'est correctement 
//@               terminee
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               CMN__VRAI si la tache s'est correctement terminee
//@               CMN__FAUX sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Booleen SD_TAC_Tache::EstTerminee()
{
  //@ Methode canevas a surimplementer dans les classes derivees
  return CMN__VRAI;
}


//@ ------------------------------------------------------------------*/
//@ Methode : AjouterAction
//@
//@ Description : Ajoute une action a la liste des actions a faire 
//@               pour la Tache
//@
//@ Parametres :
//@               (ENTREE) action : Action a realiser
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_TAC_Tache::AjouterAction(SD_ACT_Action* action)
{
  //@ Ajouter l'action a la fin des actions a realiser
  m_Actions.insert(m_Actions.end(), action);
}


//@ ------------------------------------------------------------------*/
//@ Methode : VerifierConditionArret
//@
//@ Description : Verifie si une demande d'arret de la tache a ete 
//@               levee
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               CMN__VRAI si terminaison demandee
//@               CMN__FAUX sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Booleen SD_TAC_Tache::VerifierConditionArret()
{
  //@ Regarder si l'indicateur de terminaison de tache est positionne
  //@ Retourner sa valeur
  return m_Arreter;
}


//======================================================================
//                                      METHODES PRIVEES
//======================================================================
