static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/UTL/src/SD_UTL_Facteur.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_UTL
//@ CLASSE : SD_UTL_Facteur
//@
//@ Commentaire :
//@    Ce fichier gere la classe SD_UTL_Facteur, pilote de test du
//@    Serveur de Donnees
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL

// Standard
#include <stdio.h>
#include <sys/signal.h>
#include <unistd.h>

// Projet
#include "CMN__Commun.h"
#include "CMN__CGSCst.h"
#include "CMN__Securite.h"
#include "CFG__Configuration.h"
#include "ERR__Exception.h"
#include "SD_UTL_Facteur.h"


//
// Definition des constantes
//
const int      SD_UTL_Facteur::BUFFER_SIZE     = 10;
const t_Chaine SD_UTL_Facteur::FICHIER_CONSOLE = "msgRecu.txt";
const t_Chaine SD_UTL_Facteur::MSG_SERVER_HOST = "MSG_SERVER_HOST";
const t_Chaine SD_UTL_Facteur::MSG_SERVER_PORT = "MSG_SERVER_PORT";


//=====================================================================
//                        DONNEES STATIQUES
//=====================================================================

// Serveur de tests
static SD_UTL_Facteur S_Facteur;

// Handler pour la reception d'un signal
static void SD_UTL_Terminer(int signal);

// Handler pour la reception d'une exception non traitee
static void SD_UTL_GestionnaireException();


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD_UTL_Facteur
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_UTL_Facteur::SD_UTL_Facteur() : m_ModeExecution(MODE_NORMAL), m_AttenteAUTO(1)
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_UTL_Facteur
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_UTL_Facteur::~SD_UTL_Facteur()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Initialiser
//@
//@ Description : Permet d'initialiser la connexion sur le bus de
//@               message
//@
//@ Parametres :
//@               (ENTREE) mode
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_UTL_Facteur::Initialiser(const t_Chaine mode)
{
  //@ SI mode AUTO
  if (mode == "AUTO")
    {
      //@ Positionner le mode d'execution a AUTO
      m_ModeExecution = MODE_AUTO;

      //@ Positionne le timeout d'attente                     
      m_AttenteAUTO   = int(CFG__Configuration::
                            GetNumerique("TIMEOUT_AUTO"));
    }
  //@ FIN SI mode AUTO
      
  //@ Lecture du parametrage (nom Host et numero de service)
  t_Chaine host    = CFG__Configuration::GetAdresse(MSG_SERVER_HOST);
  t_Chaine service = CFG__Configuration::GetAdresse(MSG_SERVER_PORT);
  
  //@ Creer la connexion serveur en appelant le gestionnaire 
  //@ d'activites
  m_GestionnaireActivite.ConnecterServeur((char*)host.c_str(),
                                          (char*)service.c_str());
}


//@ ------------------------------------------------------------------*/
//@ Methode : Lancer
//@
//@ Description : Permet de lancer le menu du pilote de test
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_UTL_Facteur::Lancer()
{
  char         rep[BUFFER_SIZE];
  char         tmpWorkOrder[MAXPATHLEN];
  t_Chaine     workOrder("");


  //@ TANT QUE pas fin
  do
    {
      //@ Afficher le menu
      printf("\n\
              * CLIENT DU SERVEUR SD *\n");

      printf("\n\
                * SERVICES GENERAUX *\n\
\n\
        1 => Arreter Serveur SD\n\
       ----------------------------------\n\
        2 => Requete Demarrer Traitement\n\
        3 => Requete Arreter Traitement\n\
        4 => \n\
        5 => \n\
        6 => \n\
        7 => \n\
        8 => \n\
       ----------------------------------\n\
        9 => Lire message recu\n\
\n\
        0 => retour\n\
\n\
          Choix => ");


 
      memset(rep, 0, BUFFER_SIZE);
      scanf("%s", rep);
      
      //@ SELON le choix utilisateur
      switch(rep[0])
        {
        //@ CAS '1' :
        case '1' :

          //@ Envoi d'un message d'arret
          m_GestionnaireActivite.StopperProcess("SD");
          break;
          
          /*----------------------------------------------------------*/
          
        //@ CAS '2' :
        case '2' :
          
          memset(tmpWorkOrder, 0, MAXPATHLEN);
          printf("     . nom work order: \n");
          scanf("%s", tmpWorkOrder);

          //@ Envoi d'un message Demarrer Traitement
          workOrder = tmpWorkOrder;
          m_GestionnaireActivite.DemarrerTraitement(workOrder, 
                                                    CMN__FAUX);

          //@ SI mode automatique, on ne rend la main qu'a 
          //@ la fin du work order
          if (m_ModeExecution == MODE_AUTO)
            {
              //@ TANT QUE il y a des traitements a lancer
              while (m_GestionnaireActivite.NbTraitement() > 0)
                {
                  //@ Lire les messages recus
                  m_GestionnaireActivite.Ecouter();

                  //@ Attente
                  sleep(m_AttenteAUTO);
                }
              //@ FIN TANT QUE
            }
          //@ FIN SI
          break;

          /*----------------------------------------------------------*/
              
        //@ CAS '3' :
        case '3' :

          //@ Envoi d'un message Arreter Traitement
          m_GestionnaireActivite.StopperTraitement();
          break;

          /*----------------------------------------------------------*/

        //@ CAS '4' :
        case '4' :
          break;

          /*----------------------------------------------------------*/

        //@ CAS '5' :
        case '5' :
          break;

          /*----------------------------------------------------------*/

        //@ CAS '6' :
        case '6' :
          break;

          /*----------------------------------------------------------*/
                      
        //@ CAS '7' :
        case '7' :
          break;
          
          /*----------------------------------------------------------*/
          
        //@ CAS '8' :
        case '8' :
          break;

          /*----------------------------------------------------------*/

         //@ CAS '9' :
        case '9' :

          //@ Lire les messages recus
          m_GestionnaireActivite.Ecouter();
          break;
          
          /*----------------------------------------------------------*/
              
        //@ CAS '0' :
        case '0' :

          //@ Arreter le TES
          m_GestionnaireActivite.StopperProcess("TES");
          sleep(1);

          //@ Arreter le JDBS
          m_GestionnaireActivite.StopperProcess("JDBS");
          sleep(1);

          //@ Arreter le MSGS
          m_GestionnaireActivite.StopperProcess("SERVER");
          break;
              
        //@ CAS Defaut : choix invalide
        default:
          cerr << "Choix invalide" << endl;
          break;
        }
      //@ FIN SELON
    }
  while ( (rep[0] != '0') && (!m_GestionnaireActivite.Terminer()) );
  //@ FIN TANT QUE
}


//@ ------------------------------------------------------------------*/
//@ Methode : Terminer
//@
//@ Description : Permet de terminer l'execution du serveur
//@
//@ Parametres :
//@               Aucun   
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_UTL_Facteur::Terminer()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : main
//@
//@ Description : Programme principal du pilote de tests du SD
//@
//@ Parametres :
//@               (ENTREE) argc : Nombre de parametres
//@               (ENTREE) argv : Tableau des parametres
//@
//@ Valeur de retour :
//@               Code retour du process
//@
//@ ------------------------------------------------------------------*/
int main (int argc, char *argv[])
{
  // Mode d'execution
  t_Chaine mode("");


  //@ Installation d'un gestionnaire d'exception
  set_terminate(&SD_UTL_GestionnaireException);
  set_unexpected(&SD_UTL_GestionnaireException);

  //@ Test des arguments
  if (argc == 2)
    {
      mode = argv[1];
    }

  //@ En captant les exceptions
  try 
    {
      //@ Positionner l'environnement pour la securite
      CMN__Securite_Init(SD_UTL_Terminer, SD_UTL_Terminer);
      
      //@ Initialisation du pilote
      S_Facteur.Initialiser(mode);
       
      //@ Lancer le menu du pilote
      S_Facteur.Lancer();                   
      
      //@ On est sorti du menu: on termine
      S_Facteur.Terminer();
     } 
  
  //@ Exploitation des exceptions
  catch (ERR__Exception &exception) 
    {
      //@ Impression des exceptions
      cout << "Exception SD_UTL: " << endl;
      cout << "\tFile : " << exception.GetFile() << endl;
      cout << "\tLine : " << exception.GetLine() << endl;
      cout << "\tError: " << exception.GetErrorCode() << endl;
      cout << "\tErrno: " << exception.GetErrno() << endl;
      for (int i=0; i<exception.GetNbParametre(); i++)
        {
          cout << "\tInfo : " << exception.GetParametre(i) << endl;
        }
      
      //@ On essaye de terminer
      S_Facteur.Terminer();  
    }
  
  return EXIT_SUCCESS;
}


//=====================================================================
//                        METHODES PRIVEES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD_UTL_Terminer
//@
//@ Description : Arret du process
//@
//@ Parametres :
//@               (ENTREE) signal: Numero du signal
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void SD_UTL_Terminer (int signal)
{
  //@ Appel de la methode Terminer du serveur
  S_Facteur.Terminer();
}


//@ ------------------------------------------------------------------*/
//@ Methode : SD_UTL_GestionnaireException
//@
//@ Description : Handler pour la reception d'une exception non 
//@               traitee
//@
//@ Parametres :
//@               Aucune
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void SD_UTL_GestionnaireException()
{
  cerr << "Exception non geree ou illegale recue !" << endl;

  //@ Terminaison du serveur
  S_Facteur.Terminer();
}
