static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/UTL/src/SD_UTL_GestionnaireActivite.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ -------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_UTL
//@ CLASSE : SD_UTL_GestionnaireActivite
//@
//@ Commentaire :
//@    Ce fichier gere la classe SD_UTL_GestionnaireActivite
//@
//@ -------------------------------------------------------------------
//@ HISTORIQUE
//@ -------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ -------------------------------------------------------------------

//
// Fichiers include
//

// STL
#include <iostream>

// Standard
#include <unistd.h>
#include <values.h>
#include <math.h>

// Projet
#include "CMN__Commun.h"
#include "CMN__StageTM.h"
#include "CFG__Configuration.h"
#include "ERR__Exception.h"
#include "MSG__Message.h"
#include "MSG__Connexion.h"
#include "MSG__Abonnement.h"
#include "MP__Message.h"
#include "MP__ArretProcess.h"
#include "MP__HKTMStatus.h"
#include "SD__Message.h"
#include "SD__Init.h"
#include "CMN_PRS_Parser.h"
#include "SD_UTL_GestionnaireActivite.h"


//
// Definition des constantes
//
const t_Chaine SD_UTL_GestionnaireActivite::ENV_DUREE_GRANULE    = "DUREE_GRANULE";
const int      SD_UTL_GestionnaireActivite::DEFAUT_DUREE_GRANULE = 22*8;


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD_UTL_GestionnaireActivite
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_UTL_GestionnaireActivite::SD_UTL_GestionnaireActivite() 
  : m_FlagFin(CMN__FAUX), m_DureeGranule(DEFAUT_DUREE_GRANULE)
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_UTL_GestionnaireActivite
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_UTL_GestionnaireActivite::~SD_UTL_GestionnaireActivite()
{
  m_ListeTraitement.clear();
}


//@ ------------------------------------------------------------------*/
//@ Methode : ConnecterServeur
//@
//@ Description : Permet de se connecter au bus MSG en etablissant une
//@               connexion avec le serveur
//@
//@ Parametres :
//@               (ENTREE) host : Nom du hote sur lequel attend le 
//@                               serveur
//@               (ENTREE) service : Numero de port de la connexion
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_UTL_GestionnaireActivite::ConnecterServeur(char *host, char *service)
{
  //@ Ouvrir la connexion du type cliente
  m_ConnexionServeur.Ouvrir(host, service, MSG__Connexion::CNX_CLIENTE);

  //@ Positionne le timeout sur la base de temps                     
  m_ConnexionServeur.SetTimeout(CFG__Configuration::GetNumerique
                                ("TIMEOUT_WOM"));
  
  //@ S'abonner aux adresses du processus 
  MSG__Abonnement abonnement;
  MSG__Message message;
  abonnement.SetListeAdresse(CFG__Configuration::GetListeAdresse
                             ("LISTE_WOM"));
  abonnement.SetAction(MSG__Abonnement::ABONNER);
  abonnement.Serialiser(message);
  m_ConnexionServeur.PublierMessage(message);                


  //@ Recuperer la variable d'environnement DUREE_GRANULE
  double duree = CFG__Configuration::GetNumerique(ENV_DUREE_GRANULE);

  //@ SI la variable n'est pas definie
  if (duree != MAXDOUBLE)
    {
      m_DureeGranule = int(duree);
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : Ecouter
//@
//@ Description : Permet d'ecouter la connexion avec le serveur et de 
//@               dispatcher les messages
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_UTL_GestionnaireActivite::Ecouter()
{
  //@ Liste des descripteurs actifs
  MSG__Connexion::t_ListeDescripteur descripteurActif;
  MSG__Message newMessage;
  
  //@ Appel bloquant scruter sur la connexion serveur
  m_ConnexionServeur.Scruter(descripteurActif);
   
  //@ SI il y a un descripteur actif
  if (descripteurActif.size() > 0) 
    {
      //@ Lire le message
      m_ConnexionServeur.LireMessage(newMessage);
       
      //@ Traiter le message
      TraiterMessage(newMessage);
    }
  //@ SINON aucun message recu
  else
    {
      cerr << "--------------------------------------------" << endl;
      cerr << "Aucun message recu." << endl;
      cerr << "--------------------------------------------" << endl 
           << endl;
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : StopperProcess
//@
//@ Description : Permet de stopper un processus
//@
//@ Parametres :
//@               (ENTREE) nomProcess : Nom du process a stopper
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_UTL_GestionnaireActivite::StopperProcess(t_Chaine nomProcess)
{
  //@ Envoi d'un message de fin
  MSG__Message message;
  message.SetType(MSG__Message::FIN);
  message.SetAdresse(CFG__Configuration::GetAdresse(nomProcess+"_FIN"));
  m_ConnexionServeur.PublierMessage(message);
}


//@ ------------------------------------------------------------------*/
//@ Methode : StopperTraitement
//@
//@ Description : Permet de stopper le traitement en cours
//@
//@ Parametres :
//@               Aucun 
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_UTL_GestionnaireActivite::StopperTraitement()
{
  //@ Envoi d'un message d'arret de traitement au SD
  MSG__Message message;
  message.SetType(SD__Message::EVT_ARRET_TMT_SD);
  message.SetAdresse(CFG__Configuration::GetAdresse("SD_INPUT"));
  m_ConnexionServeur.PublierMessage(message);
}


//@ ------------------------------------------------------------------*/
//@ Methode : DemarrerTraitement
//@
//@ Description : Permet de lancer un traitement
//@
//@ Parametres :
//@               (ENTREE) fichierWorkOrder : Work order a lancer
//@               (ENTREE) analyseRealisee : VRAI si les traitements
//@                                          ont deja ete extraits du
//@                                          work order
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_UTL_GestionnaireActivite::DemarrerTraitement(t_Chaine fichierWorkOrder,
                                                     t_CMN__Booleen 
                                                     analyseRealisee)
{
  //@ Capture des exceptions
  try 
    {
      //@ SI le fichier WorkOrder doit etre analyse
      if (analyseRealisee == CMN__FAUX)
        {
          if (m_ListeTraitement.size() > 0)
            {
              cerr << "Work order en cours: "
                   << "attendre la fin de son traitement !" << endl;
              
              return;
            }

          CMN_PRS_Parser    parser;
          CMN_DAT_WorkOrder workOrder;

  
          //@ Initialiser le parsing du fichier WorkOrder
          parser.ParseFile(fichierWorkOrder);
      
          //@ Charger l'arbre XML dans le Work Order
          parser.ParseWorkOrder(workOrder);
          cout << endl;
          cout << "--------------------------------------" << endl;
          cout << "- Affichage du contenu du Work Order -" << endl;
          //workOrder.Afficher();
          cout << "--------------------------------------" << endl 
               << endl;

          //@ Identifier les traitements
          IdentifierTraitement(workOrder);
        }
      //@ FIN SI le fichier WorkOrder doit etre analyse

      //@ Lancer le premier traitement de la liste
      SD_DON_Traitement* messageTraitement = m_ListeTraitement.front();

      cout << "--------------------------------------------" << endl;
      cout << "--   Pilote de tests : traitement envoye  --" << endl;
      messageTraitement->Afficher();
      cout << "--------------------------------------------" << endl 
           << endl;
      
      //@ Envoi du message
      MSG__Message message = messageTraitement->Serialiser();
      message.SetAdresse(CFG__Configuration::GetAdresse("SD_INPUT"));
      m_ConnexionServeur.PublierMessage(message);
    }

  //@ Exploitation des exception    
  catch (ERR__Exception &exception) 
    {
      //@ Impression des exceptions
      cout << "Exception in SD_UTL::DemarrerTraitement(): " << endl;
      cout << "\tFile : " << exception.GetFile() << endl;
      cout << "\tLine : " << exception.GetLine() << endl;
      cout << "\tError: " << exception.GetErrorCode() << endl;
      for (int i=0; i<exception.GetNbParametre(); i++)
        {
          cout << "\tInfo : " << exception.GetParametre(i) << endl;
        }
    }
}


//=====================================================================
//                        METHODES PRIVEES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : ManageException
//@
//@ Description : Permet de traiter les differents types d'exception
//@
//@ Parametres :
//@               (ENTREE) exception : Exception recue
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_UTL_GestionnaireActivite::ManageException(ERR__Exception exception)
{
  //@ Traiter les exceptions
  //@ SELON le type d'erreur
  switch (exception.GetErrorCode())
    {
    //@ Defaut:  
    default :
      //@ Rien a faire
      break; 
    }
  //@ FIN SELON
}


//@ ------------------------------------------------------------------*/
//@ Methode : TraiterMessage
//@
//@ Description : Permet de traiter les differents types de messages
//@
//@ Parametres :
//@               (ENTREE) message : Message recu 
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_UTL_GestionnaireActivite::TraiterMessage(const MSG__Message& message)
{
  SD__Init          init;
  SD_DON_Traitement traitement;
  MP__HKTMStatus    status;
  ERR__Exception    exception;
  

  cerr << "--------------------------------------------" << endl;

  //@ En captant les exceptions
  try 
    { 
      //@ SELON le type du message
      switch (message.GetType())
        {
        //@ Si c'est un message FIN
        case MSG__Message::FIN :
          {
            //@ Alors on positionne le flag de fin d'execution
            Terminer(CMN__VRAI);
            break;
          }
        
        //@ Si c'est un message de fin d'initialisation du SD
        case SD__Message::EVT_INIT_SD :
          {
            init.Deserialiser(message);
            SD__Init::e_ListeStatus etat = init.Etat();

            cerr << "Initialisation du SD terminee; statut = " 
                 << etat << endl;
            cerr << "\tSD_INIT_OK       = 0" << endl;
            cerr << "\tSD_INIT_NOK      = 1" << endl;
            cerr << "\tSD_INIT_ERR_BANK = 2" << endl;
            
            break;
          }
        
        //@ Si c'est un message de fin de traitement du SD
        case SD__Message::EVT_TMT_SD :
          {
            traitement.Deserialiser(message);
            
            cerr << "Traitement termine; statut = " 
                 << traitement.Etat() << endl;
            cerr << "\tSD_TMT_OK       = 0" << endl;
            cerr << "\tSD_TMT_NOK      = 1" << endl;
            cerr << "\tSD_TMT_ERR_ALGO = 2" << endl;

            cerr << "--------------------------------------------" 
                 << endl;
            cerr << "--    Pilote de tests : traitement recu   --" 
                 << endl;
            traitement.Afficher();
            cerr << "--------------------------------------------" 
                 << endl << endl;

            //@ Suppression du traitement
            m_ListeTraitement.pop_front();

            //@ SI il reste des traitements a lancer
            if (m_ListeTraitement.size() > 0)
              {
                //@ SI le traitement precedent s'est mal termine
                if (traitement.Etat() != SD_DON_Traitement::SD_TMT_OK)
                  {
                    //@ Suppression des autres traitements a lancer
                    m_ListeTraitement.clear();
                  }
                //@ SINON
                else
                  {
                    //@ Lancement du traitement suivant
                    DemarrerTraitement("", CMN__VRAI);
                  }
                //@ FIN SI le traitement precedent s'est mal termine
              }
            //@ FIN SI il reste des traitements a lancer
            break;
          }
        
        //@ Si c'est l'arret d'un processus
        case MP__Message::EVT_ARRET_PROCESSUS :
          {
            //@ Recuperation des informations contenues dans le message
            MP__ArretProcess                     arret;
            CMN__Chaine                          nomProcess;
            e_CMN__Processus                     process;
            MP__ArretProcess::e_EtatFinProcessus etat;


            arret.Deserialiser(message);

            process = arret.GetProcessus();
            switch (process)
              {
              case PROCESS_MP :
                nomProcess = "MP";
                break;
              case PROCESS_TES :
                nomProcess = "TES";
                break;
              case PROCESS_JDBS :
                nomProcess = "JDBS";
                break;
              case PROCESS_WOM :
                nomProcess = "WOM";
                break;
              case PROCESS_SD :
                nomProcess = "SD";
                break;
              case PROCESS_MSGS :
                nomProcess = "MSGS";
                break;
              default :
                nomProcess = "UNKNOWN";
                break;
              }

            etat = arret.GetEtatArret();

            cerr << "Arret du process: " << nomProcess 
                 << "; statut = " << etat << endl;
            cerr << "\tFIN_OK       = 0" << endl;
            cerr << "\tFIN_NOK      = 1" << endl;

            break;
          }

        //@ Si c'est un message de type pipeline/operational status
        case MP__Message::EVT_HKTM_STATUS :
          {
            status.Deserialiser(message);
            
            cerr << "HKTM status recu: " << endl;
            cerr << "\tid      : "<< status.GetIdStatus() << endl;
            cerr << "\ttype    : "<< status.GetType() << endl;
            cerr << "\tvalidite: "<< status.GetValidity() << endl;
            cerr << "\tdispo   : "<< status.GetAvailability() << endl;
            cerr << "\tservice : "<< status.GetService() << endl;
            cerr << "\tnom     : "<< status.GetParamName() << endl;
            cerr << "\tvaleur  : "<< status.GetValeurChaine() << endl;
            break;
          }

        //@ Defaut: c'est un message inconnu
        default :
          {
            cerr << "!!!!!!!!!!!!! MESSAGE INCONNU !!!!!!!!!!!!!" 
                 << "!!!!!!!!!!!!!!!! type = " << message.GetType() 
                 << "!!!!!!!!!!!!!!!!" << endl;       

            //@ Par defaut, on envoie une exception UNEXCEPTED_MSG
            exception.SetErrorCode(ERR__Exception::UNEXPECTED_MSG);
            exception.SetParametre(t_Chaine(message.GetType()));
            THROW(exception);   
            break;
          }
        }
      //@ FIN SELON
    }

  // Exploiter les exceptions
  catch (ERR__Exception &exception) 
    {
      ManageException(exception);
    }  

  cerr << "--------------------------------------------" << endl 
       << endl;

}


//@ ------------------------------------------------------------------*/
//@ Methode : IdentifierTraitement
//@
//@ Description : Decompose le Work Order en un ou plusieurs 
//@               traitements
//@
//@ Parametres :
//@               (ENTREE) workOrder : Donnees du work order
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_UTL_GestionnaireActivite::IdentifierTraitement(CMN_DAT_WorkOrder&
                                                       workOrder) 
  throw (ERR__Exception)
{
  //@ Nombre de traitements a inserer: par defaut, un seul
  int            nbTraitement = 1;

  //@ Temoin de creation de traitements de type DUMP
  t_CMN__Booleen EstUnDump    = CMN__FAUX;

  // Compteur iteration
  int            i            = 0;

 
  //@ Tester le type du Work Order
  //@ SI le type est FULL
  if (workOrder.TimeIntervalFlag() == 
      CMN_DAT_WorkOrder::WORK_ORDER_TYPE_FULL)
    {
      double dureeDump = 0.0;
      double nbTemp    = 0.0;
      t_Date dateStart;
      t_Date dateStop;


      //@ Cas d'un dump: 1 WorkOrder = n traitements
      EstUnDump = CMN__VRAI;

#ifdef OPS_DEBUG
      cout << "IdentifierTraitement(): Le traitement est un dump" 
           << endl;
#endif

      //@ Calculer le nombre de traitements
      dateStart    = workOrder.SensingStart();
      dateStop     = workOrder.SensingStop();
      dureeDump    = dateStop - dateStart;
      nbTemp       = dureeDump / double(m_DureeGranule);
      nbTraitement = floor(nbTemp);

      if (nbTemp != nbTraitement)
        {
          nbTraitement++;
        }

#ifdef OPS_DEBUG
      cout << "IdentifierTraitement(): duree du dump     : " 
           << dureeDump << endl;
      cout << "IdentifierTraitement(): duree d'un granule: " 
           << m_DureeGranule << endl;
      cout << "IdentifierTraitement(): nb traitements    : " 
           << nbTraitement << endl;
#endif

    }
  //@ FIN SI le type est FULL
  
  
  //@ En captant les exceptions
  try
    {
      //@ POUR le nombre de traitements a creer
      for (i=0; i<nbTraitement; i++)
        {
          //@ Creer un nouveau traitement a partir des
          //@ informations contenues dans le Work Order
          SD_DON_Traitement* traitement = 
            new SD_DON_Traitement(workOrder);

          //@ Traitements particuliers pour un DUMP
          //@ SI c'est un DUMP
          if (EstUnDump == CMN__VRAI)
            {
              //@ Modification des dates et du SlideId
              traitement->T0(workOrder.SensingStart() + 
                             i*m_DureeGranule);
              traitement->Counter(i+1);

              //@ SI le traitement n'est pas le dernier
              if (i < (nbTraitement-1))
                {
                  //@ Positionner la date de fin a la date de debut
                  //@ + la duree d'un granule
                  traitement->T1(traitement->T0() + m_DureeGranule);

                  //@ Si c'est le premier traitement
                  if (i == 0)
                    {
                      //@ Positionner le SlideId a FIRST
                      traitement->SlideId(FIRST);
                    }
                  //@ SINON
                  else
                    {
                      //@ Positionner le SlideId a MIDDLE
                      traitement->SlideId(MIDDLE);
                    }
                  //@ FIN SI
                }
              //@ SINON
              else
                {
                  //@ Positionner la date de fin a la date de fin
                  //@ du WorkOrder
                  traitement->T1(workOrder.SensingStop());

                  //@ Positionner le SlideId a LAST
                  traitement->SlideId(LAST);
                }
              //@ FIN SI le traitement n'est pas le dernier
            }
          //@ FIN SI c'est un DUMP
           
          //@ Insertion du traitement dans la liste des traitements
          m_ListeTraitement.push_back(traitement);
        } 
      //@ FIN POUR le nombre de traitements a creer
    }

  //@ Traiter les exceptions
  catch (ERR__Exception& exception)
  {
    //@ Effacer la liste de traitement
    t_ListeTraitement::iterator i = m_ListeTraitement.begin();
    while (i != m_ListeTraitement.end())
      {
        i = m_ListeTraitement.erase(i);
      }
    m_ListeTraitement.clear();

    //@ Re-emission exception
    THROW(exception);
  }
}
