// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/TES/src/TES__GestionnaireActivite.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : TES
//@ CLASSE : TES__GestionnaireActivite
//@
//@ Commentaire :
//@    Ce fichier gere la classe TES__GestionnaireActivite
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 21/03/2000 : A.MONTMORY
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL

// Standard

// Projet
#include "CMN__Commun.h"
#include "CFG__Configuration.h"
#include "ERR__Exception.h"
#include "MSG__Abonnement.h"
#include "TES__GestionnaireActivite.h"


//
// Definition des constantes 
//


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : TES__GestionnaireActivite
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
TES__GestionnaireActivite::TES__GestionnaireActivite()
{
  //@ Positionne le flag de fin
  m_FlagFin = false;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~TES__GestionnaireActivite
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
TES__GestionnaireActivite::~TES__GestionnaireActivite()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ConnecterServeur
//@
//@ Description : Permet de se connecter au bus MSG en etablissant une
//@               connexion avec le serveur
//@
//@ Parametres :
//@               (ENTREE) host : Nom du hote sur lequel attend le
//@                               serveur
//@               (ENTREE) service : Numero de port de la connexion
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void TES__GestionnaireActivite::ConnecterServeur(char *host, char *service)
{
  //@ Ouvrir la connexion du type cliente
  m_ConnexionServeur.Ouvrir(host,service, 
			    MSG__Connexion::CNX_CLIENTE);
   
  //@ Positionne le timeout sur la base de temps		     
  m_ConnexionServeur.SetTimeout (CFG__Configuration::GetNumerique("TIMEOUT_TES"));
  
  //@ Positionne la connexion dans le gestionnaire d'evenement
  m_Evenements.SetConnexion(&m_ConnexionServeur);
  m_Evenements.SetResolution(CFG__Configuration::GetNumerique("TIMEOUT_TES"));
  
  //@ S'abonner aux adresses du processus 
  MSG__Abonnement abonnement;
  MSG__Message message;
  abonnement.SetListeAdresse
    (CFG__Configuration::GetListeAdresse("LISTE_TES"));
  abonnement.SetAction (MSG__Abonnement::ABONNER);
  abonnement.Serialiser(message);
  m_ConnexionServeur.PublierMessage(message);		     
}

//@ ------------------------------------------------------------------*/
//@ Methode : EnvoyerMessage
//@
//@ Description : Permet d'envoyer un message sur le bus MSG
//@
//@ Parametres :
//@               (ENTREE) message : Message a envoyer
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void TES__GestionnaireActivite::EnvoyerMessage(MSG__Message& message)
{
   //@ Publication du message
   m_ConnexionServeur.PublierMessage(message); 
}

//@ ------------------------------------------------------------------*/
//@ Methode : Ecouter
//@
//@ Description : Permet d'ecouter la connexion avec le serveur et de 
//@               dispatcher les messages
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void TES__GestionnaireActivite::Ecouter()
{
  //@ Liste des descripteurs actifs
  MSG__Connexion::t_ListeDescripteur descripteurActif;
  MSG__Message newMessage;
  
  //@ Appel bloquant scruter sur la connexion serveur
  m_ConnexionServeur.Scruter (descripteurActif);
  
  //@ SI il y a un descripteur descripteur actif
  if (descripteurActif.size() == 1) 
    {
      //@ Lire le message
      m_ConnexionServeur.LireMessage (newMessage);
      
      //@ Traiter le message
      TraiterMessage (newMessage);
    }
  //@ SINON 
  else
    {
      //@ Il faut regarder si il y a des evenements a publier
      TirerEvenement(); 
    }
  //@ FIN SI
}


//=====================================================================
//                        METHODES PRIVEES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : TraiterMessage
//@
//@ Description : Permet de traiter les differents types de messages
//@
//@ Parametres :
//@               (ENTREE) message : Message recu
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void TES__GestionnaireActivite::TraiterMessage(const MSG__Message& message)
{
   TES__Evenement evenementTimer;
   ERR__Exception exception;
    
   
   //@ SELON le type du message
   switch (message.GetType()) 
     {
     //@ Si c'est un evenement TIMER
     case MSG__Message::EVT_TIMER :
     
       //@ Alors on construit un objet abonnement
       evenementTimer.Deserialiser(message);
       break;
       
     //@ Si c'est un evenement FIN
     case MSG__Message::FIN :
       
       //@ Alors on positionne le flag de fin d'execution
       Terminer(true);
       break;
       
     //@ Defaut:
     default :
       //@ Par defaut, on envoie une exception UNEXCEPTED_MSG
       exception.SetErrorCode (ERR__Exception::UNEXPECTED_MSG);
       THROW (exception);   
       break;
     }
   //@ FIN SELON

   //@ SELON pour examen des actions contenues dans l'evenement
   switch (evenementTimer.GetAction()) 
     {
     //@ Si c'est une action ADD
     case TES__Evenement::ACTION_ADD :
       //@ Alors il faut ajouter l'evenement dans le gestionnaire 
       AjouterEvenement(evenementTimer);
       break;
     
     //@ Si c'est une action DELETE
     case TES__Evenement::ACTION_DELETE :
       //@ Alors il faut supprimer l'evenement du gestionnaire 
       DetruireEvenement(evenementTimer);
       break;
       
     //@ Si type UNKNOWN on ne fait rien (evenement vide)
     case TES__Evenement::ACTION_UNKNOWN :
       break;
       
     //@ Defaut:
     default :
       //@ Par defaut, on envoie une exception UNEXCEPTED_ACTION
       exception.SetErrorCode (ERR__Exception::UNEXPECTED_ACTION);
       THROW (exception);   
       break;
     }
   //@ FIN SELON
}


//@ ------------------------------------------------------------------*/
//@ Methode : AjouterEvenement
//@
//@ Description : Permet d'ajouter un evenement dans le gestionnaire
//@
//@ Parametres :
//@               (ENTREE) evenementTimer : Evenement Timer recu
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void TES__GestionnaireActivite::AjouterEvenement(TES__Evenement& evenementTimer)
{
  //@ Ajoute un evenement dans le gestionnaire
  m_Evenements.AjouterEvenement(evenementTimer);
}


//@ ------------------------------------------------------------------*/
//@ Methode : DetruireEvenement
//@
//@ Description : Permet de detruire un evenement dans le gestionnaire
//@
//@ Parametres :
//@               (ENTREE) evenementTimer : Evenement Timer a 
//@                                         supprimer
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void TES__GestionnaireActivite::DetruireEvenement(const TES__Evenement& evenementTimer)
{
  //@ Detruire un evenement dans le gestionnaire
  m_Evenements.DetruireEvenement(evenementTimer);
}


//@ ------------------------------------------------------------------*/
//@ Methode : TirerEvenement
//@
//@ Description : Permet de lancer un evenement dans le gestionnaire
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void TES__GestionnaireActivite::TirerEvenement()
{
  //@ Tirer un evenement dans le gestionnaire
  m_Evenements.TirerEvenement();   
}
