// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/TES/src/TES__GestionnaireEvenement.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : TES
//@ CLASSE : TES__GestionnaireEvenement
//@
//@ Commentaire :
//@    Ce fichier gere la classe TES__GestionnaireEvenement
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 21/03/2000 : A.MONTMORY
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------


//
// Fichiers include
//

// STL
#include <cstdio>
#include <vector>

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN__Chaine.h"
#include "CMN__Date.h"
#include "ERR__Exception.h"
#include "TES__GestionnaireEvenement.h"


//
// Definition des constantes 
//


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : TES__GestionnaireEvenement
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
TES__GestionnaireEvenement::TES__GestionnaireEvenement()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~TES__GestionnaireEvenement
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
TES__GestionnaireEvenement::~TES__GestionnaireEvenement()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : AjouterEvenement
//@
//@ Description : Permet d'ajouter un evenement dans le gestionnaire
//@
//@ Parametres :
//@               (ENTREE) evenementTimer : Evenement Timer recu
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void TES__GestionnaireEvenement::AjouterEvenement(TES__Evenement& evenementTimer)
{
  CMN__Date date;
  double tempsGSP = 0.0;
  double temps    = 0.0;


  date.DateCourante();
  tempsGSP = date.ConvertirDateEnTemps();
  temps = evenementTimer.GetStartDate();

  //@ Si c'est un evenement periodique et que la date de debut
  //@ n'est pas renseignee alors on determine la date de debut
  //@ d'activation de l'evenement
  if( temps <= tempsGSP && evenementTimer.GetPeriode() > 0 )
    {
      int nb_fois = ( (int) (tempsGSP - temps) + evenementTimer.GetPeriode()
		      - 1 ) / evenementTimer.GetPeriode();
      temps += nb_fois * evenementTimer.GetPeriode();
      evenementTimer.SetStartDate(temps);
    }

#ifdef DEBUG
  cout << endl;
  cout << "---------------------- TES ------------------------" << endl;
  cout << "AJOUT EVENEMENT : " <<  evenementTimer.GetNomLogique() << endl;
  printf( "\tTEMPS   : %10.0lf\n", temps);
  cout << "\tPERIODE : " << evenementTimer.GetPeriode() << endl;
  cout << "---------------------------------------------------" << endl;
  cout << endl;
#endif

  //@ Ajouter l'evenement dans la liste en le triant par date 
  //@ de debut
  m_ListeEvenement.insert (t_ListeEvenement::value_type(temps,evenementTimer));
}


//@ ------------------------------------------------------------------*/
//@ Methode : DetruireEvenement
//@
//@ Description : Permet de detruire un evenement dans le gestionnaire
//@
//@ Parametres :
//@               (ENTREE) evenementTimer : Evenement Timer a
//@                                         supprimer
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void TES__GestionnaireEvenement::DetruireEvenement(const TES__Evenement& evenementTimer)
{
  //@ Pour detruire un evenement, il faut parcourir toute la 
  //@ collection pour retrouver le nom de l'evenement car les
  //@ evenements sont classes par date de debut
  
  t_ListeEvenement::iterator pInfo;
  bool supprime = true;
  
  //@ TANT QUE l'evenement n'a pas ete supprime
  while( supprime == true )
    {
      supprime = false;

      //@ POUR tous les evenements
      for ( pInfo = m_ListeEvenement.begin(); 
	    pInfo !=  m_ListeEvenement.end();
	    pInfo++) 
	{
	  //@ SI le NomLogique de l'evenement est le meme
	  TES__Evenement evenement = (*pInfo).second;
	  
	  if (evenement.GetNomLogique() == evenementTimer.GetNomLogique()) {
	    //@ alors il faut effacer l'evenement
	    m_ListeEvenement.erase(pInfo);        
	    supprime = true;
	    
#ifdef DEBUG
	    cout << endl;
	    cout << "---------------------- TES ------------------------" << endl;
	    cout << "TES SUPPRIME EVT :" << evenement.GetNomLogique() << endl;
	    cout << "---------------------------------------------------" << endl;
	    cout << endl;
#endif
	    
	    break;
	  }
	  //@ FIN SI
	}
      //@ FIN POUR
    }
  //@ FIN TANT QUE
}
   
   
//@ ------------------------------------------------------------------*/
//@ Methode : TirerEvenement
//@
//@ Description : Permet de publier les evenements candidats au temps 
//@               courant
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void TES__GestionnaireEvenement::TirerEvenement()
{
  CMN__Date                  date;
  t_DateNum                  tempsCourant;
  vector<TES__Evenement>     lstEvt;
  t_ListeEvenement::iterator pInfo;
  t_ListeEvenement::iterator pDebut;
  t_ListeEvenement::iterator pFin ;
  

  //@ Rechercher tous les evenements de la collection qui sont autour 
  //@ du temps courant +- delta_T (la resolution de la base de temps)
  
  date.DateCourante();
  tempsCourant = date.ConvertirDateEnTemps();

  // lower_bound: la valeur 0.0 et la valeur tempsCourant_precedent
  // correspondent a la meme borne de depart
  pDebut = m_ListeEvenement.lower_bound (0.0);
  pFin = m_ListeEvenement.upper_bound (tempsCourant);

  //@ POUR tous les evenements trouves
  for ( pInfo = pDebut; pInfo !=  pFin; pInfo++) 
    {
      //@ SI l'evenement est periodique
      TES__Evenement event = (*pInfo).second;
      
      if (event.GetPeriode()) 
	{
	  TES__Evenement newEvent = event;
	  
	  //@ alors il faut creer un nouvel evenement a StartEvenement
	  //@ + periode qui est memorise dans une liste temporaire
	  newEvent.SetStartDate(newEvent.GetStartDate()+newEvent.GetPeriode());
	  lstEvt.push_back (newEvent);       
	}
      //@ FIN SI

      //@ L'action sur l'evenement est publiee
      event.SetAction(TES__Evenement::ACTION_PUBLISH);
      
      //@ Il faut serialiser l'evenement
      MSG__Message message = event.Serialiser();    
      message.SetAdresse (event.GetAdresse ());
      
#ifdef DEBUG
      cout << endl;
      cout << "---------------------- TES ------------------------" << endl;
      cout << "PUBLISH EVENEMENT : " << event.GetNomLogique() << endl;
      cout << "\tADRESSE DIFFUSION : " << event.GetAdresse() << endl;
      cout << "---------------------------------------------------" << endl;
      cout << endl;
#endif

      //@ et publier l'evenement
      GetConnexion()->PublierMessage(message); 
      
    }
  //@ FIN POUR sur liste evenement		  
  
  //@ Suppression des evenements publies du gestionnaire
  m_ListeEvenement.erase(pDebut, pFin);
  
  //@ Ajout des evenements crees
  if( ! lstEvt.empty() )
    {
      
#ifdef DEBUG
      cout << endl;
      cout << "---------------------- TES ------------------------" << endl;
      cout << "INSERTION DES " << lstEvt.size() << "NOUVEAUX EVTS" << endl;
      cout << "---------------------------------------------------" << endl;
      cout << endl;
#endif
      
      for( vector<TES__Evenement>::iterator pEvt = lstEvt.begin();
	   pEvt != lstEvt.end(); pEvt++ )
	{
	  AjouterEvenement ( *pEvt );
	}
      lstEvt.clear ();
    }
}
