// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/WOM/src/WOM__GestionnaireActivite.cc,v 1.4 2015/07/22 12:13:52 lcaspar Exp $
//@ --------------------------------------------------------------------
//@ Composant : WOM
//@ CLASSE : WOM__GestionnaireActivite
//@
//@ Commentaire :
//@    Ce fichier contient la classe WOM__GestionnaireActivite, qui 
//@    gere l'etat et les interfaces externes du process WOM: lecture
//@    et traitement de messages recus, envoi de messages
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-099-CN : 21/10/2003 : JL.PASCAL
//@ no entry was found in the report for the stop command 
//@ a message is added in TraiterFinTraitement
//@ fiche de correction correspondante : IA-2100-FC-099-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-115-CN : 10/02/2004 : JL.PASCAL
//@ no report were generated in cas of missing data
//@ fiche de correction correspondante : IA-2100-FC-115-THA
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-130-CN : 16/09/2004 : P.CABANE
//@ If work order contains missing files, it's coded all the same
//@ (later the treatment will detect if missing file is mandatory).
//@ A warning message is sent
//@ fiche de correction correspondante : IA-2100-FC-130-THA
//@ --------------------------------------------------------------------
//@ FA-ID : IA-2100-FA-147-CN : 10/11/2004 : P.CABANE
//@ Le ACK "aborted" est effectue meme en cas de probleme sur les donnees
//@ d'entree. Voir IA-2100-FC-147-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-159-CN : 26/05/2005 : JL. PASCAL
//@ modif suite a portage linux
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-3189(FA-2100-OPS) : 01/10/2014 : L. CASPAR
//@ Ajout d une condition pour l appel a la fonction TraiterFinDegrade
//@ pour arreter correctement l OPS dans le cas de l absence de fichier SVM
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------


//
// Fichiers include
//

// STL
#include <iostream>

// Standard
#include <unistd.h>
#include <values.h>
//FA159
#include <signal.h>

// Projet
#include "CMN__Commun.h"
#include "CMN__Fichier.h"
#include "CMN__Cmd.h"
#include "CFG__Configuration.h"
#include "ERR__Exception.h"
#include "JDB__Callback.h"
#include "JDB__Client.h"
#include "MSG__Message.h"
#include "MSG__Abonnement.h"
#include "MP__StageTM.h"
#include "SD__Message.h"
#include "WOM__Message.h"
#include "WOM__GestionnaireActivite.h"


//
// Definition des constantes 
//
const t_Chaine WOM__GestionnaireActivite::MP_INPUT      = "MP_INPUT";
const t_Chaine WOM__GestionnaireActivite::WOM_INPUT     = "WOM_INPUT";
const t_Chaine WOM__GestionnaireActivite::SD_INPUT      = "SD_INPUT";
const t_Chaine WOM__GestionnaireActivite::TIMEOUT_WOM   = "TIMEOUT_WOM";
const t_Chaine WOM__GestionnaireActivite::LISTE_WOM     = "LISTE_WOM";

const t_Chaine WOM__GestionnaireActivite::JDB_PRM_1     = "$PRM1";
const t_Chaine WOM__GestionnaireActivite::JDB_PRM_2     = "$PRM2";
const t_Chaine WOM__GestionnaireActivite::JDB_PRM_3     = "$PRM3";

const t_Chaine WOM__GestionnaireActivite::INVALID_STR   = "INVALID";
const t_Chaine WOM__GestionnaireActivite::STATE_STR[]   = {"INITIALISATION",
                                                           "WITHOUT_STEP", 
                                                           "PROCESSING",
                                                           "SUSPENDED",
                                                           "POST_PROCESSING",
                                                           "BREAKING",
                                                           "STOPPING",
                                                           "END",
                                                           "ERROR",
                                                           "DEAD"};

const t_Chaine WOM__GestionnaireActivite::LIBELLE_FIN_TRAITEMENT_STOP       = "\n PPF terminated after  STOP command ";
const t_Chaine WOM__GestionnaireActivite::LIBELLE_FIN_TRAITEMENT_BREAK      = "\n STEP terminated after  BREAK command ";

//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : WOM__GestionnaireActivite
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
WOM__GestionnaireActivite::WOM__GestionnaireActivite()
{
  //@ Positionne le flag de fin
  m_FlagFin = CMN__FAUX;

  m_IdCdeInt = CMN__OPS_INTERNAL_CMD;  

  //@ Initialisation du tableau event/state
  EventStateInit();
  m_EtatServeur = WOM_STATE_INITIALISATION;
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~WOM__GestionnaireActivite
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
WOM__GestionnaireActivite::~WOM__GestionnaireActivite()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ConnecterServeur
//@
//@ Description : Permet de se connecter au bus MSG en etablissant une
//@               connexion avec le serveur
//@
//@ Parametres :
//@               (ENTREE) host : Nom du hote sur lequel attend le 
//@                               serveur
//@               (ENTREE) service : Numero de port de la connexion
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::ConnecterServeur(char *host, char *service)
{
  //@ Ouvrir la connexion du type cliente
  m_ConnexionServeur.Ouvrir(host, service, MSG__Connexion::CNX_CLIENTE);
  
  //@ Positionne le timeout sur la base de temps                     
  m_ConnexionServeur.SetTimeout(CFG__Configuration::
                                GetNumerique(TIMEOUT_WOM));

  //@ S'abonner aux adresses du processus 
  MSG__Abonnement abonnement;
  MSG__Message message;
  abonnement.SetListeAdresse(CFG__Configuration::
                             GetListeAdresse(LISTE_WOM));
  abonnement.SetAction(MSG__Abonnement::ABONNER);
  abonnement.Serialiser(message);

  EnvoyerMessage(message); 
  
  //@ Installer les callback qui permettent d'envoyer les msg JDB
  static JDB__Callback msgCallback;
  msgCallback.SetConnexion(&m_ConnexionServeur);
  
  //@ Initialisation du set de message
  JDB__Client messageJDB;
  messageJDB.InitSet(WOM__Message::WOM_SET,
                     WOM__Message::MSG_WOM_START,
                     WOM__Message::MSG_FIN);


#ifdef INVESTIGATION
  //@ INVESTIGATION: Envoyer un message de type JDB MSG_WOM_START
  EnvoyerMessageJDB(WOM__Message::MSG_WOM_START);
#endif
}


//@ ------------------------------------------------------------------*/
//@ Methode : Ecouter
//@
//@ Description : Permet d'ecouter la connexion avec le serveur et de 
//@               dispatcher les messages
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::Ecouter()
{
   //@ Liste des descripteurs actifs
   MSG__Connexion::t_ListeDescripteur descripteurActif;
   MSG__Message                       nouveauMessage;


   //@ Appel bloquant scruter sur la connexion serveur
   m_ConnexionServeur.Scruter(descripteurActif);
   
   //@ SI il y a un descripteur actif
   if (descripteurActif.size() == 1) 
     {
       //@ Lire le message
       m_ConnexionServeur.LireMessage(nouveauMessage);
       
       //@ Traiter le message
       TraiterMessage(nouveauMessage);
     }
   //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : InitialiserGestionnaireWO
//@
//@ Description : Permet d'initaliser le gestionnaire des Work Orders
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::InitialiserGestionnaireWO()
{
  //@ Initialisation des repertoires de travail de l'application
  m_GestionnaireWorkOrder.InitialiserEspaceTravail();
}


//@ ------------------------------------------------------------------*/
//@ Methode : EnvoyerMessage
//@
//@ Description : Permet d'envoyer un message sur le bus MSG
//@
//@ Parametres :
//@               (ENTREE) message : Message a envoyer
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::EnvoyerMessage(MSG__Message& message)
{
  //@ En captant les exceptions
  try 
    { 
      //@ Publication du message
      m_ConnexionServeur.PublierMessage(message); 
    }
  
  //@ Exploiter les exceptions
  catch (ERR__Exception &exception) 
    {
      //@ Appel de la routine de gestion des exceptions
      ManageException(exception);
    }  
}


//@ ------------------------------------------------------------------*/
//@ Methode : EnvoyerMessageJDB
//@
//@ Description : Permet d'envoyer un message JDB sur le bus MSG
//@
//@ Parametres :
//@               (ENTREE) msgId : Numero du message JDB dans le
//@                                catalogue du WOM
//@               (ENTREE) prm1 : Parametre du message
//@               (ENTREE) prm2 : Parametre du message
//@               (ENTREE) prm3 : Parametre du message
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::EnvoyerMessageJDB(WOM__Message::
                                                  e_TypeMessageJDB
                                                  msgId, 
                                                  t_Chaine prm1, 
                                                  t_Chaine prm2, 
                                                  t_Chaine prm3)
{
  //@ En captant les exceptions
  try 
    { 
      JDB__Client message;

      //@ Envoi du message de type JDB
      if (! prm1.empty())
        {
          message.SetParametre(JDB_PRM_1, prm1);
          if (! prm2.empty())
            {
              message.SetParametre(JDB_PRM_2, prm2);
              if (! prm3.empty())
                {
                  message.SetParametre(JDB_PRM_3, prm3);
                }
            }
        }
      message.EcrireMessage(WOM__Message::WOM_SET, msgId);
      message.Liberer();
    }
  
  //@ Exploiter les exceptions
  catch (ERR__Exception &exception) 
    {
      //@ Appel de la routine de gestion des exceptions
      ManageException(exception);
    }  
}


//=====================================================================
//                        METHODES PRIVEES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : ManageException
//@
//@ Description : Permet de traiter les differents types d'exception
//@
//@ Parametres :
//@               (ENTREE) exception : Exception recue
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::ManageException(ERR__Exception exception)
{
#ifdef OPS_DEBUG
  // Impression de l'exception
  cout << "Exception: " << endl;
  cout << "\tFile : " << exception.GetFile() << endl;
  cout << "\tLine : " << exception.GetLine() << endl;
  cout << "\tError: " << exception.GetErrorCode() << endl;
  cout << "\tErrno: " << exception.GetErrno() << endl;
  for (int i=0; i<exception.GetNbParametre(); i++)
    {
      cout << "\tInfo : " << exception.GetParametre(i) << endl;
    }
#endif

  //@ Traiter les exceptions
  //@ SELON le type d'erreur
  switch (exception.GetErrorCode())
    {
    //@ Si c'est une erreur dans un type de message       
    case ERR__Exception::UNEXPECTED_MSG :
      //@ Envoyer un message de type JDB UNEXPECTED_MSG
      EnvoyerMessageJDB(WOM__Message::MSG_UNEXPECTED_MSG, 
                        exception.GetFile(), 
                        exception.GetLine(), 
                        exception.GetParametre(0));
      break;
      
    //@ Si c'est une erreur dans un type logique de message       
    case ERR__Exception::UNEXPECTED_TYPE :
      //@ Envoyer un message de type JDB UNEXPECTED_TYPE
      EnvoyerMessageJDB(WOM__Message::MSG_UNEXPECTED_TYP, 
                        exception.GetFile(), 
                        exception.GetLine(), 
                        exception.GetParametre(0));
      break;

    //@ Si c'est une erreur dans le WorkOrder
    case ERR__Exception::WOM_INVALID_WO :
      //@ Envoyer un message de type JDB MSG_WOM_WO_INVALID
      EnvoyerMessageJDB(WOM__Message::MSG_WOM_WO_INVALID, 
                        exception.GetParametre(0),
                        exception.GetParametre(1));
      break;

    //@ Si c'est une erreur dans le traitement des signaux
    case ERR__Exception::WOM_SIGNAL_ERROR :
      //@ Envoyer un message de type JDB MSG_WOM_ERROR
      EnvoyerMessageJDB(WOM__Message::MSG_WOM_ERROR, 
                        exception.GetParametre(0));
      break;

    //@ Si c'est une erreur dans la recherche des traitements
    case ERR__Exception::WOM_TMT_NOT_FOUND :
      //@ Envoyer un message de type JDB MSG_WOM_ERROR
      EnvoyerMessageJDB(WOM__Message::MSG_WOM_ERROR, 
                        exception.GetParametre(0));
      break;

    //@ Si c'est une erreur de variable d'environnement
    case ERR__Exception::WOM_ENV_UNDEFINED :
      //@ Envoyer un message de type JDB MSG_WOM_ENV_UNDEFINED
      EnvoyerMessageJDB(WOM__Message::MSG_WOM_ENV_UNDEFINED, 
                        exception.GetParametre(0));
      break;

    //@ Si c'est une erreur d'ecriture d'un fichier
    case ERR__Exception::WOM_FILE_WRITE_ERROR :
      //@ Envoyer un message de type JDB MSG_WOM_FILE_WRITE_ERROR
      EnvoyerMessageJDB(WOM__Message::MSG_WOM_FILE_WRITE_ERROR, 
                        exception.GetParametre(0));
      break;

    //@ Defaut:  
    default :
      {
        t_Chaine param("");
        int      i = 0;
        
        //@ Par defaut, envoyer un message unexpected exception
        for (i=0; i<exception.GetNbParametre(); i++)
          {
            param += exception.GetParametre(i) + ";";
          }

        //@ Envoyer un message de type JDB MSG_UNEXPECTED_EXP
        EnvoyerMessageJDB(WOM__Message::MSG_UNEXPECTED_EXP, 
                          exception.GetFile(), exception.GetLine(),
                          param);
        break;
      } 
    }
  //@ FIN SELON
}


//@ ------------------------------------------------------------------*/
//@ Methode : TraiterMessage
//@
//@ Description : Permet de traiter les differents types de messages
//@
//@ Parametres :
//@               (ENTREE) message : Message recu
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::TraiterMessage(const MSG__Message& message)
{
  ERR__Exception exception;


#ifdef OPS_DEBUG
  DumperServeur("Avant traitement message");
#endif
  
#ifdef INVESTIGATION
  //@ INVESTIGATION: Envoyer un message de type JDB MSG_WOM_STATE
  EnvoyerMessageJDB(WOM__Message::MSG_WOM_STATE, 
                    Etat(), 
                    "before receiving a message");

  //@ INVESTIGATION: Envoyer un message de type JDB MSG_WOM_MESSAGE
  EnvoyerMessageJDB(WOM__Message::MSG_WOM_MESSAGE, 
                    message.GetType());
#endif
            

  //@ En captant les exceptions
  try 
    { 
      //@ SELON le type du message
      switch (message.GetType())
        {

        //@ Si c'est une commande envoyee par le MP
        case MP__Message::EVT_PRCSS_CTRL :
          {
            //@ Recuperation des informations contenues dans
            //@ le message
            MP__Command commande;

            commande.Deserialiser(message);

            //@ Traiter la commande
            TraiterCommande(commande);
            
            break;
          }

        //@ Si c'est un message de fin d'initialisation du SD
        case SD__Message::EVT_INIT_SD :
          {
            //@ Recuperation des informations contenues dans 
            //@ le message
            SD__Init init;

            init.Deserialiser(message);

            //@ Traiter la fin de l'initialisation
            TraiterFinInit(init);

            break;
          }
        
        //@ Si c'est un message de fin de traitement du SD
        case SD__Message::EVT_TMT_SD :
          {
            //@ Recuperation des informations contenues dans
            //@ le message
            SD_DON_Traitement traitement;
            
            traitement.Deserialiser(message);

            //@ Traiter la fin du traitement
            TraiterFinTraitement(traitement);

            break;
          }
        
        //@ Si c'est un evenement FIN
        case MSG__Message::FIN :
          {
            //@ On positionne le flag de fin d'execution
            Terminer(CMN__VRAI);

            //@ Mettre a jour l'etat du serveur
            EventUpdate(WOM_EVENT_END);
            
            break;
          }
        
        //@ Defaut: message non traite
        default :
          {
            //@ Par defaut, on envoie une exception UNEXPECTED_MSG
            exception.SetErrorCode(ERR__Exception::UNEXPECTED_MSG);
            exception.SetParametre(t_Chaine(message.GetType()));
            THROW(exception)

            break;
          }
        }
      //@ FIN SELON
    }
  
  //@ Exploiter les exceptions
  catch (ERR__Exception &exception) 
    {
      //@ Appel de la routine de gestion des exceptions
      ManageException(exception);
    }  
  
#ifdef OPS_DEBUG
  DumperServeur("Apres traitement message");
#endif

#ifdef INVESTIGATION
  //@ INVESTIGATION: Envoyer un message de type JDB MSG_WOM_STATE
  EnvoyerMessageJDB(WOM__Message::MSG_WOM_STATE, 
                    Etat(), 
                    "after receiving a message");
#endif
}


//@ ------------------------------------------------------------------*/
//@ Methode : AccepterCommande
//@
//@ Description : Verifie si la commande doit etre traitee ou rejetee
//@
//@ Parametres :
//@               (ENTREE) typeCommande : Type de la commande recue
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK si la commande est acceptee
//@               CMN__STATUT_NOK sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut 
WOM__GestionnaireActivite::AccepterCommande(CMN__CmdTM::e_CMN__CommandType
                                            typeCommande)
{
  // Valeur de retour de la methode
  t_CMN__Statut status = CMN__STATUT_NOK;


  //@ SELON le type de la commande
  switch (typeCommande)
    {
      //@ Si c'est une commande STEP
    case CMN__CmdTM::STEP :
      status = EventCheck(WOM_EVENT_STEP);
      break;
      
      //@ Si c'est une commande SUSPEND
    case CMN__CmdTM::SUSPEND :
      status = EventCheck(WOM_EVENT_SUSPEND);
      break;
    
      //@ Si c'est une commande RESUME
    case CMN__CmdTM::RESUME :
      status = EventCheck(WOM_EVENT_RESUME);
      break;
      
      //@ Si c'est une commande BREAK
    case CMN__CmdTM::BREAK :
      status = EventCheck(WOM_EVENT_BREAK);
      break;
      
      //@ Si c'est une commande STOP
    case CMN__CmdTM::STOP :
      status = EventCheck(WOM_EVENT_STOP);
      break;
      
      //@ Defaut: aucun traitement
    default :
      break;
    }
  //@ FIN SELON


  //@ Retourner la valeur de retour de la methode
  return status;
}


//@ ------------------------------------------------------------------*/
//@ Methode : TraiterCommande
//@
//@ Description : Permet de traiter les differents types de commandes
//@               recues
//@
//@ Parametres :
//@               (ENTREE) commande : Commande recue
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::TraiterCommande(MP__Command& commande)
{
  // Exception
  ERR__Exception                 exception;

  // Code retour des fonctions appelees
  t_CMN__Statut                  status = CMN__STATUT_NOK;

  //@ Type de la commande recue
  CMN__CmdTM::e_CMN__CommandType typeCommande;


#ifdef OPS_DEBUG   
  cout << "Commande TM: "      << commande.GetCommandTypeStr()  << endl;
  cout << "\tid            = " << commande.GetCommandId()       << endl;
  cout << "\tgroupe        = " << commande.GetCommandGroupStr() << endl;
  cout << "\tfichier       = " << commande.GetWOFilename()      << endl;
  cout << "\tid process SD = " << commande.GetProcessId()       << endl;
#endif

  
  //@ Preparation de l'acquittement de reception du message
  typeCommande = commande.GetCommandType();

  //@ Verifier si la commande doit etre traitee ou rejetee
  status = AccepterCommande(typeCommande);

  //@ Mise a jour de la valeur de l'acquittement de reception 
  //@ du message
  //@ SI la commande doit etre traitee
  // FA 115 : dans le cas des STEP on emet le rejected ou coded dans la fonction TraiterCommandeSTEP
  // ou apres analyse par le SD
  if (status == CMN__STATUT_OK && typeCommande != CMN__CmdTM::STEP)
    {
      //@ Acquittement = CODED
      commande.SetCommandStatus(CMN__CmdTM::CODED);
  // jlp 115
    //@ Envoi de l'acquittement de reception du message
    EnvoyerCommandeTM(commande);
    }
  //@ SINON
  else if (status != CMN__STATUT_OK )
    {
      //@ Acquittement = REJECTED
      commande.SetCommandStatus(CMN__CmdTM::REJECTED);
    //@ Envoi de l'acquittement de reception du message
     EnvoyerCommandeTM(commande);
    }
  //@ FIN SI

  

  //@ SI la commande doit etre traitee
  if (status == CMN__STATUT_OK)
    {
      //@ SELON le type de la commande
      switch (typeCommande)
        {
        //@ CAS commande STEP:
        case CMN__CmdTM::STEP :
          {  
            //@ Lancer le traitement du STEP
            TraiterCommandeSTEP(commande);
            
            //@ Mettre a jour l'etat du serveur
            EventUpdate(WOM_EVENT_STEP);
            break;
          }
            
        //@ CAS commande SUSPEND:
        case CMN__CmdTM::SUSPEND :
          {
            //@ Envoyer un signal SIGSTOP au SD
            //@ SI la commande a echoue
            if (kill(commande.GetProcessId(), SIGSTOP) != 0)
              {
                //@ Envoi d'un message SUSPEND, ABORTED
                commande.SetCommandStatus(CMN__CmdTM::ABORTED);
                EnvoyerCommandeTM(commande);
                
                //@ Generation d'une exception
                exception.SetErrorCode(ERR__Exception::
                                       WOM_SIGNAL_ERROR);
                exception.SetParametre("SIGSTOP ERROR");
                THROW(exception)
              }
            //@ FIN SI la commande a echoue
            
            //@ Mettre a jour l'etat du serveur
            EventUpdate(WOM_EVENT_SUSPEND);
            
            //@ Envoi d'un message SUSPEND, COMPLETED
            commande.SetCommandStatus(CMN__CmdTM::COMPLETED);
            EnvoyerCommandeTM(commande);
            
            //@ Envoi d'un message HKTM Readiness IDLE
            EnvoyerTMReadiness(MP__HKTMStatus::IDLE);
            
            //@ Envoyer un message de type JDB 
            //@ MSG_WOM_STEP_SUSPEND
            EnvoyerMessageJDB(WOM__Message::MSG_WOM_STEP_SUSPEND,
                              m_GestionnaireWorkOrder.
                              WorkOrderCourant()->Id());
          }
        break;
        
        //@ CAS commande RESUME:
        case CMN__CmdTM::RESUME :
          {
            //@ Envoyer un signal SIGCONT au SD
            //@ SI la commande a echoue
            if (kill(commande.GetProcessId(), SIGCONT) != 0)
              {
                //@ Envoi d'un message RESUME, ABORTED
                commande.SetCommandStatus(CMN__CmdTM::ABORTED);
                EnvoyerCommandeTM(commande);
                
                //@ Generation d'une exception
                exception.SetErrorCode(ERR__Exception::
                                       WOM_SIGNAL_ERROR);
                exception.SetParametre("SIGCONT ERROR");
                THROW(exception)
              }
             //@ FIN SI la commande a echoue

            //@ Mettre a jour l'etat du serveur
            EventUpdate(WOM_EVENT_RESUME);
            
            //@ Envoi d'un message RESUME, COMPLETED
            commande.SetCommandStatus(CMN__CmdTM::COMPLETED);
            EnvoyerCommandeTM(commande);
            
            //@ Envoi d'un message HKTM Readiness WORKING
            EnvoyerTMReadiness(MP__HKTMStatus::WORKING);
            
            //@ Envoyer un message de type JDB MSG_WOM_STEP_RESUME
            EnvoyerMessageJDB(WOM__Message::MSG_WOM_STEP_RESUME, 
                              m_GestionnaireWorkOrder.
                              WorkOrderCourant()->Id());
          }
        break;
        
        //@ CAS commande BREAK:
        case CMN__CmdTM::BREAK :
          {  
            m_IdCdeInt = commande.GetCommandId();

            //@ Envoyer un message d'arret de traitement au SD
            StopperTraitement();
            
            //@ Mettre a jour l'etat du serveur
            EventUpdate(WOM_EVENT_BREAK);
            break;
          }
        
        //@ CAS commande STOP:
        case CMN__CmdTM::STOP :
          {
            //@ Traiter la commande STOP : 
            //@   - si un traitement est en cours: envoyer un
            //@     message d'arret de traitement au SD et
            //@     lancer les post-traitements
            //@   - envoyer un message de fin de dump 
            TraiterCommandeSTOP(commande);
            
            //@ Mettre a jour l'etat du serveur
            EventUpdate(WOM_EVENT_STOP);
            break;
          }
        
        //@ Defaut: message non traite -- NOT REACHABLE --
        default :
          {
            //@ Par defaut, on envoie une exception UNEXCEPTED_MSG
            exception.SetErrorCode(ERR__Exception::UNEXPECTED_MSG);
            exception.SetParametre(commande.GetCommandTypeStr());
            THROW(exception) 
            break;
          }
        }
      //@ FIN SELON
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : TraiterFinInit
//@
//@ Description : Permet de traiter le compte-rendu d'initialisation
//@               envoye par le SD
//@
//@ Parametres :
//@               (ENTREE) init : Compte-rendu d'initialisation recu
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::TraiterFinInit(SD__Init& init)
{
  // Acquittement de commande envoyee au MP
  MP__Command             commande;

#ifdef OPS_DEBUG
  cerr << "Initialisation du SD terminee; statut = " 
       << init.Etat() << endl;
  cerr << "\tSD_INIT_OK       = 0" << endl;
  cerr << "\tSD_INIT_NOK      = 1" << endl;
  cerr << "\tSD_INIT_ERR_BANK = 2" << endl;
#endif
  
  //@ SI le message ne doit pas etre pris en compte
  if (EventCheck(WOM_EVENT_END_INIT) == CMN__STATUT_NOK)
    {
      ERR__Exception exception;
  
      //@ Emission d'une exception UNEXPECTED_MSG
      exception.SetErrorCode(ERR__Exception::UNEXPECTED_MSG);
      exception.SetParametre("SD__Init");
      THROW(exception)
    }
  //@ FIN SI le message ne doit pas etre pris en compte


  commande.SetCommandGroup(CMN__Cmd::ACK);
  commande.SetCommandType(CMN__CmdTM::START);

  //@ SI l'initialisation du SD ne s'est pas bien deroulee
  if (init.Etat() != SD__Init::SD_INIT_OK)
    {
      //@ Envoi d'un message START, REJECTED 
      commande.SetCommandStatus(CMN__CmdTM::REJECTED);
      EnvoyerCommandeTM(commande);
      
      //@ Envoi d'un message FIN DUMP, NOK
      EnvoyerFinDump(MP__Dump::FIN_NOK);
      
      //@ Mettre a jour l'etat du serveur: le serveur
      //@ se met en attente d'un arret
      EventUpdate(WOM_EVENT_STOP);
    }
  //@ SINON
  else
    {
      //@ Envoi d'un message START, CODED
      commande.SetCommandStatus(CMN__CmdTM::CODED);
      EnvoyerCommandeTM(commande);
      
      //@ Mettre a jour l'etat du serveur: le serveur
      //@ se met en attente de traitements
      EventUpdate(WOM_EVENT_END_INIT);
    }
  //@ FIN SI

  //@ Envoi d'un message HKTM Readiness IDLE
  EnvoyerTMReadiness(MP__HKTMStatus::IDLE);
}


//@ ------------------------------------------------------------------*/
//@ Methode : TraiterFinTraitement
//@
//@ Description : Permet de traiter le compte-rendu de traitement
//@               envoye par le SD
//@
//@ Parametres :
//@               (ENTREE) traitement : Compte-rendu de traitement
//@                                     recu 
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::TraiterFinTraitement(SD_DON_Traitement& 
                                                     traitement)
{
#ifdef OPS_DEBUG
  cerr << "Traitement termine; statut = " << traitement.Etat() << endl;
  cerr << "\tSD_TMT_OK       = 0" << endl;
  cerr << "\tSD_TMT_NOK      = 1" << endl;
  cerr << "\tSD_TMT_ERR_ALGO = 2" << endl;
  
  traitement.Afficher();
#endif


  //@ Tester si le message doit etre pris en compte
  
  //@ SI le message ne doit pas etre pris en compte
  if (EventCheck(WOM_EVENT_END_TMT) == CMN__STATUT_NOK)
    {
      ERR__Exception exception;

      //@ Emission d'une exception UNEXPECTED_MSG
      exception.SetErrorCode(ERR__Exception::UNEXPECTED_MSG);
      exception.SetParametre("SD_DON_Traitement");
      THROW(exception)
    }
  //@ FIN SI le message ne doit pas etre pris en compte


  //@ Recuperer le numero du STEP
  int idSTEP = m_GestionnaireWorkOrder.WorkOrderCourant()->Id();
  
  //@ Recuperer le StageId
  CMN__StageTM::e_CMN__StageId idStage = m_GestionnaireWorkOrder.
    WorkOrderCourant()->Data().ProcessingType();
  
  //@ Tester si le traitement est une fin de DUMP
  t_CMN__Booleen finDump = m_GestionnaireWorkOrder.
    WorkOrderCourant()->TraitementCourant()->EstDeTypeFinDump();
  
  //@ Recuperer l'etat du traitement
  SD_DON_Traitement::e_ListeStatus etat = traitement.Etat();
  
  //@ Recuperer le PCD
  int valeurPCD = traitement.PCD();
  
 
  //@ Envoi d'un message HKTM Readiness IDLE
  EnvoyerTMReadiness(MP__HKTMStatus::IDLE);
  
  // ici on met a jour le message de LOG pour indiquer la raison de l'arret
  // Message de log
  if  (m_EtatServeur == WOM_STATE_BREAKING) 
   {
    traitement.LogMessage(traitement.LogMessage() + LIBELLE_FIN_TRAITEMENT_BREAK);
   }
  else if   (m_EtatServeur == WOM_STATE_STOPPING && m_IdCdeInt != CMN__OPS_INTERNAL_CMD)
   {
    traitement.LogMessage(traitement.LogMessage() + LIBELLE_FIN_TRAITEMENT_STOP);
  }
  //@ Mettre a jour le Work Order courant
  m_GestionnaireWorkOrder.MettreAJour(traitement);
  
  //@ SI le traitement du SD ne s'est pas bien deroule ou SI
  //@ le serveur a recu un BREAK ou un STOP: on arrete le STEP
  if ((etat != SD_DON_Traitement::SD_TMT_OK) ||
     (m_EtatServeur == WOM_STATE_BREAKING) ||
      (m_EtatServeur == WOM_STATE_STOPPING))
    {
      //@ Lancer la procedure degradee pour la fin du traitement    
      // FT-3189
      if ("End of treatment: L1a - SVM aux data not found" == traitement.LogMessage())
        {
         TraiterFinDegrade(SD_DON_Traitement::SD_TMT_NOK, idSTEP, idStage, finDump);
        }
      else
        {
         TraiterFinDegrade(etat, idSTEP, idStage, finDump);
        }
      //TraiterFinDegrade(etat, idSTEP, idStage, finDump);
      // .. FT-3189
    }
  //@ SINON mode NOMINAL
  else
    {
      //@ SI le STEP est termine, effectuer les post-traitements
      if (m_GestionnaireWorkOrder.WorkOrderCourant()->EstTermine())
        {
          //@ Mettre a jour l'etat du serveur
          EventUpdate(WOM_EVENT_END_TMT);
          
          //@ Effectuer les post-traitements
          PostTraiter(idSTEP, idStage, valeurPCD);
          
          //@ Tester s'il faut continuer
          //@ SI le STEP etait un STEP de type fin de dump, alors
          //@   il faut fermer l'application: envoyer un message
          //@   FIN DUMP et attendre un message d'arret
          //@ SINON
          //@   SI il existe d'autres STEP a executer, lancer un
          //@     nouveau STEP
          //@   SINON revenir en attente de STEP
          
          //@ Supprimer le Work Order
          m_GestionnaireWorkOrder.SupprimerWorkOrderCourant();
          
          //@ SI le STEP etait un STEP de type fin de dump
          if (finDump == CMN__VRAI)
            {
              //@ Envoi du message FIN DUMP, OK
              EnvoyerFinDump(MP__Dump::FIN_OK);
              
              //@ Mettre a jour l'etat du serveur: 
              //@ le serveur se met en attente d'un arret 
              EventUpdate(WOM_EVENT_END_DUMP);
            }
          //@ SINON le serveur lance un autre STEP ou se
          //@ met en attente de nouveaux STEP
          else
            {
              //@ SI il existe d'autres STEP a executer
              if (m_GestionnaireWorkOrder.ExisteWorkOrder() == CMN__VRAI)
                {
                  //@ Lancer le STEP suivant
                  LancerTraitement();
                  
                  //@ Envoyer un message de type JDB
                  //@ MSG_WOM_STEP_BEGIN
                  EnvoyerMessageJDB(WOM__Message::MSG_WOM_STEP_BEGIN,
                                    m_GestionnaireWorkOrder.
                                    WorkOrderCourant()->Id());
                      
                  //@ Mettre a jour l'etat du serveur
                  EventUpdate(WOM_EVENT_STEP);
                }
              //@ SINON
              else
                {
                  //@ Mettre a jour l'etat du serveur: 
                  //@ le serveur se met en attente de nouveaux
                  //@ STEP
                  EventUpdate(WOM_EVENT_NO_STEP);
                }
              //@ FIN SI il existe d'autres STEP a executer
            }
          //@ FIN SI le STEP etait un STEP de type fin de dump
        }
      //@ SINON il reste des traitements a executer dans
      //@ le STEP courant
      else
        {
          //@ Lancer le traitement suivant du STEP non termine
          LancerTraitement();
          
          //@ Mettre a jour l'etat du serveur
          EventUpdate(WOM_EVENT_STEP);
        }
      //@ FIN SI le STEP est termine
    }
  //@ FIN SI le traitement du SD ne s'est pas bien deroule ou SI
  //@ le serveur a recu un BREAK ou un STOP
}


//@ ------------------------------------------------------------------*/
//@ Methode : TraiterFinDegrade
//@
//@ Description : Traite la fin d'un traitement en mode degrade:
//@                 - reception d'un BREAK
//@                 - reception d'un STOP
//@                 - traitement termine en erreur
//@
//@ Parametres :
//@               (ENTREE) etat : Etat du traitement termine
//@               (ENTREE) idSTEP : Identifiant du STEP du traitement
//@               (ENTREE) idStage : StageId du traitement
//@               (ENTREE) finDump : Traitement de type fin de DUMP
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::TraiterFinDegrade(SD_DON_Traitement::
                                                  e_ListeStatus etat,
                                                  int idSTEP, 
                                                  CMN__StageTM::
                                                  e_CMN__StageId 
                                                  idStage,
                                                  t_CMN__Booleen 
                                                  finDump)
{
  // Acquittement de commande envoyee au MP
  MP__Command    commande;

  // Temoin si BREAK recu sur traitement fin de DUMP
  t_CMN__Booleen arretSurBreak = CMN__FAUX;


  //@ Effectuer les post-traitements
  PostTraiterDegrade(idSTEP, idStage);

  
  //@ SI le serveur a recu un BREAK
  if (m_EtatServeur == WOM_STATE_BREAKING)
    {
      //@ Envoi d'un message BREAK, COMPLETED 
      commande.SetCommandId(m_IdCdeInt);
      m_IdCdeInt = CMN__OPS_INTERNAL_CMD;
      commande.SetCommandType(CMN__CmdTM::BREAK);
      commande.SetCommandStatus(CMN__CmdTM::COMPLETED);
      EnvoyerCommandeTM(commande); 
      
      //@ Envoyer un message de type JDB MSG_WOM_STEP_BREAK
      EnvoyerMessageJDB(WOM__Message::MSG_WOM_STEP_BREAK, idSTEP);
      
      //@ Tester si BREAK recu sur traitement fin de DUMP
      arretSurBreak = finDump;
    }
  //@ SINON SI le serveur a recu un STOP
  else if (m_EtatServeur == WOM_STATE_STOPPING)
    {
      //@ Envoi d'un message STOP, COMPLETED 
      commande.SetCommandId(m_IdCdeInt);
      m_IdCdeInt = CMN__OPS_INTERNAL_CMD;
      commande.SetCommandType(CMN__CmdTM::STOP);
      commande.SetCommandStatus(CMN__CmdTM::COMPLETED);
      EnvoyerCommandeTM(commande);
      
      //@ Envoyer un message de type JDB MSG_WOM_STEP_STOP
      EnvoyerMessageJDB(WOM__Message::MSG_WOM_STEP_STOP, idSTEP);
    }
  //@ SINON fin du traitement sur erreur
  else
    {
      //@ Envoyer un message de type JDB MSG_WOM_STEP_ERROR
      EnvoyerMessageJDB(WOM__Message::MSG_WOM_STEP_ERROR, idSTEP);
    }
  //@ FIN SI
  
  //@ Envoi d'un message STEP, ABORTED pour le STEP courant
   commande.SetCommandId(idSTEP);
   commande.SetCommandType(CMN__CmdTM::STEP);
  
  // FA 147
  commande.SetCommandStatus(CMN__CmdTM::ABORTED);
  EnvoyerCommandeTM(commande);

  //@ Supprimer le Work Order
  m_GestionnaireWorkOrder.SupprimerWorkOrderCourant();
  
  //@ SI le traitement s'est termine en erreur 
  //@ ou SI le serveur a recu un STOP
  //@ ou SI le serveur a recu un BREAK en mode DUMP ou pour 
  //@ un granule de type fin de DUMP
  // FA 115 : si pb majeur ou pb initialisation on arrete la chaine
  if ((etat != SD_DON_Traitement::SD_TMT_OK  && etat != SD_DON_Traitement::SD_TMT_PB_INITGRAN &&
       (m_EtatServeur != WOM_STATE_BREAKING)) || 
      (m_EtatServeur == WOM_STATE_STOPPING) || 
      arretSurBreak)
    {
      //@ POUR tous les autres STEP a traiter
      while (m_GestionnaireWorkOrder.ExisteWorkOrder() == CMN__VRAI)
        {
          //@ Envoi d'un message STEP, ABORTED
          commande.SetCommandId(m_GestionnaireWorkOrder.
                                WorkOrderCourant()->Id());
          commande.SetCommandType(CMN__CmdTM::STEP);
          commande.SetCommandStatus(CMN__CmdTM::ABORTED);
          EnvoyerCommandeTM(commande);
          
          //@ Envoyer un message de type JDB MSG_WOM_STEP_END
          EnvoyerMessageJDB(WOM__Message::MSG_WOM_STEP_END,
                            m_GestionnaireWorkOrder.
                            WorkOrderCourant()->Id());
          
          //@ Supprimer le STEP
          m_GestionnaireWorkOrder.SupprimerWorkOrderCourant();
        }
      //@ FIN POUR tous les autres STEP a traiter
      
      //@ Envoi du message FIN DUMP, NOK
      EnvoyerFinDump(MP__Dump::FIN_NOK);
      
      //@ Mettre a jour l'etat du serveur: 
      //@ le serveur se met en attente d'un arret
      EventUpdate(WOM_EVENT_STOP);
    }
  //@ SINON le serveur a recu un BREAK sur un traitement
  //@ non final
  else
    {
      //@ SI il existe d'autres STEP a executer
      if (m_GestionnaireWorkOrder.ExisteWorkOrder() == CMN__VRAI)
        {
          //@ Lancer le STEP suivant
          LancerTraitement();
                  
          //@ Envoyer un message de type JDB 
          //@ MSG_WOM_STEP_BEGIN
          EnvoyerMessageJDB(WOM__Message::MSG_WOM_STEP_BEGIN,
                            m_GestionnaireWorkOrder.
                            WorkOrderCourant()->Id());
          
          //@ Mettre a jour l'etat du serveur
          EventUpdate(WOM_EVENT_STEP);
        }
      //@ SINON
      else
        {
          //@ Mettre a jour l'etat du serveur: 
          //@ le serveur se met en attente de nouveaux STEP
          EventUpdate(WOM_EVENT_NO_STEP);
        }
    }
   //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : PostTraiter
//@
//@ Description : Effectue les post-traitements a la fin d'un STEP:
//@                 - creation du report
//@                 - deplacement des produits
//@                 - envoi des Stage TM
//@
//@ Parametres :
//@               (ENTREE) idSTEP : Identifiant du STEP
//@               (ENTREE) idStage : Identifiant du traitement
//@               (ENTREE) valeurPCD : Valeur du PCD
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::PostTraiter(int idSTEP, 
                                            CMN__StageTM::
                                            e_CMN__StageId idStage, 
                                            int valeurPCD)
{
  // Iterateur pour la liste des fichiers
  t_ListeChaine::iterator pInfo;

  // Fichier report
  t_Chaine                report("");

  // Liste des produits generes
  t_ListeChaine           listeProduit;

  // Liste des donnees auxiliaires generees
  t_ListeChaine           listeAux;

  // Acquittement de commande envoyee au MP
  MP__Command             commande;

  // Stage TM envoyee au MP
  MP__StageTM             stageTM;
  
  
  //@ En captant les exceptions
  try
    {
      //@ Envoyer un message de type JDB MSG_WOM_STEP_DATA_PCD
      EnvoyerMessageJDB(WOM__Message::MSG_WOM_STEP_DATA_PCD, valeurPCD);
 
      //@ 1. Creation du Report
      //@ 2. Deplacement des produits
      m_GestionnaireWorkOrder.GererProduits(WOM__GestionnaireWorkOrder::
                                            PRODUITS_DEPLACER,
                                            report, 
                                            listeProduit, 
                                            listeAux);

      //@ 3. Envoi d'un StageTM pour le Report
      stageTM.SetCommandId(idSTEP);
      stageTM.SetStageId(idStage);
      stageTM.SetDeliveredFilename(report);
      stageTM.SetStageCategory(CMN__StageTM::REPORT);
      EnvoyerStageTM(stageTM);
      
      //@ Envoyer un message de type JDB 
      //@ MSG_WOM_STEP_REPORT_PRODUCED
      EnvoyerMessageJDB(WOM__Message::MSG_WOM_STEP_REPORT_PRODUCED, 
                        report);
  
      //@ 4. Envoi d'un StageTM pour les produits et fichiers
      //@ auxilaires
      //@  POUR tous les produits generes
      for (pInfo = listeProduit.begin();
           pInfo != listeProduit.end(); 
           pInfo++)
        {
          //@ Envoi d'un StageTM
          stageTM.SetDeliveredFilename(*pInfo);
          stageTM.SetStageCategory(CMN__StageTM::PRODUCT);
          EnvoyerStageTM(stageTM);
          
          //@ Envoyer un message de type JDB 
          //@ MSG_WOM_STEP_DATA_PRODUCED
          EnvoyerMessageJDB(WOM__Message::MSG_WOM_STEP_DATA_PRODUCED, 
                            *pInfo);
        }
      //@ FIN POUR tous les produits generes
      
      //@ POUR tous les fichiers auxiliaires generes
      for (pInfo = listeAux.begin(); pInfo != listeAux.end(); pInfo++)
        {
          //@ Envoi d'un StageTM
          stageTM.SetDeliveredFilename(*pInfo);
          stageTM.SetStageCategory(CMN__StageTM::AUXFILE);
          EnvoyerStageTM(stageTM);
          
          //@ Envoyer un message de type JDB 
          //@ MSG_WOM_STEP_AUX_PRODUCED
          EnvoyerMessageJDB(WOM__Message::MSG_WOM_STEP_AUX_PRODUCED, 
                            *pInfo);
        }
      //@ FIN POUR tous fichiers auxiliaires generes

      //@ Envoyer un message de type JDB MSG_WOM_STEP_END
      EnvoyerMessageJDB(WOM__Message::MSG_WOM_STEP_END, idSTEP);
              
      //@ 5. Envoi d'un STEP, COMPLETED
      commande.SetCommandId(idSTEP);
      commande.SetCommandType(CMN__CmdTM::STEP);
      commande.SetCommandStatus(CMN__CmdTM::COMPLETED);
      EnvoyerCommandeTM(commande);
    }
  
  //@ Exploiter les exceptions
  catch (ERR__Exception &exception)
    {
      //@ Appel de la routine de gestion des exceptions
      ManageException(exception);
      
      //@ Envoyer un message de type JDB MSG_WOM_STEP_ERROR
      EnvoyerMessageJDB(WOM__Message::MSG_WOM_STEP_ERROR, idSTEP);

      //@ Envoi d'un STEP, ABORTED
      commande.SetCommandId(idSTEP);
      commande.SetCommandType(CMN__CmdTM::STEP);
      commande.SetCommandStatus(CMN__CmdTM::ABORTED);
      EnvoyerCommandeTM(commande);        
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : PostTraiterDegrade
//@
//@ Description : Effectue les post-traitements a la fin d'un STEP qui
//@               ne s'est pas correctement deroule:
//@                 - creation du report
//@                 - purge des produits autre que le report
//@                 - envoi des Stage TM
//@
//@ Parametres :
//@               (ENTREE) idSTEP : Identifiant du STEP
//@               (ENTREE) idStage : Identifiant du traitement
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::PostTraiterDegrade(int idSTEP, 
                                                   CMN__StageTM::
                                                   e_CMN__StageId 
                                                   idStage)
{
  // Fichier report
  t_Chaine                report("");

  // Liste des produits generes
  t_ListeChaine           listeProduit;

  // Liste des donnees auxiliaires generees
  t_ListeChaine           listeAux;

  // Acquittement de commande envoyee au MP
  MP__Command             commande;

  // Stage TM envoyee au MP
  MP__StageTM             stageTM;
  

  //@ En captant les exceptions
  try
    {
      //@ 1. Creation du Report
      //@ 2. Purge des produits
      m_GestionnaireWorkOrder.GererProduits(WOM__GestionnaireWorkOrder::
                                            PRODUITS_PURGER,
                                            report, 
                                            listeProduit,
                                            listeAux);

      //@ 3. Envoi d'un StageTM pour le Report
      stageTM.SetCommandId(idSTEP);
      stageTM.SetStageId(idStage);
      stageTM.SetDeliveredFilename(report);
      stageTM.SetStageCategory(CMN__StageTM::REPORT);
      EnvoyerStageTM(stageTM);
      
      //@ Envoyer un message de type JDB 
      //@ MSG_WOM_STEP_REPORT_PRODUCED
      EnvoyerMessageJDB(WOM__Message::MSG_WOM_STEP_REPORT_PRODUCED, 
                        report);
    }
  
  //@ Exploiter les exceptions
  catch (ERR__Exception &exception)
    {
      //@ Appel de la routine de gestion des exceptions
      ManageException(exception);
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode :  TraiterCommandeSTEP 
//@
//@ Description : Permet de prendre en compte un Work Order
//@
//@ Parametres :
//@               (ENTREE) commande : Commande envoyee par le MP
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::TraiterCommandeSTEP(MP__Command& commande)
{
  // Nom du fichier Work Order
  t_Chaine       nomFichier(commande.GetWOFilename());

      
  //@ En captant les exceptions
  try
  {
    //@ Lecture du fichier WorkOrder et ajout des traitements
    //@ associes
    m_GestionnaireWorkOrder.AjouterWorkOrder(
      nomFichier, 
      commande.GetCommandId());
  }

  //@ Traiter les exceptions
  catch (ERR__Exception &exception) 
  {
    //@ Envoi d'un message STEP, ABORTED
    // FA 115 : si exception on rejette le STEP (ceci ne concerne pas les pb de donnee
    commande.SetCommandStatus(CMN__CmdTM::REJECTED);
    EnvoyerCommandeTM(commande);
    
    //@ Re-emission de l'exception
    exception.SetErrorCode(ERR__Exception::WOM_INVALID_WO);
    exception.SetDebParametre(nomFichier);
    throw(exception);
  }
  
  //FA 130 : si probleme de donnees:
  //  - le work order n'est pas rejette, il est accepte
  //  - on trace le probleme en indiquant que le work order contient un fichier inexistant
  commande.SetCommandType(CMN__CmdTM::STEP);
  commande.SetCommandId(m_GestionnaireWorkOrder.DernierWorkOrder()->Id());
  if (m_GestionnaireWorkOrder.DernierWorkOrder()->TraitementCourant()->GetPbFichierE() == CMN__VRAI)   
  {
    //@ Acquittement = CODED
    commande.SetCommandStatus(CMN__CmdTM::CODED);
    EnvoyerCommandeTM(commande);
    
    //@ Envoyer un message de type JDB MSG_WOM_WO_MISSING_FILE
    EnvoyerMessageJDB(WOM__Message::MSG_WOM_WO_MISSING_FILE, nomFichier);
  /* FIN FA130 */
	 }
  else
  {
    //@ Acquittement = CODED
    commande.SetCommandStatus(CMN__CmdTM::CODED);
    EnvoyerCommandeTM(commande);
    //@ Envoyer un message de type JDB MSG_WOM_WO_VALID
    EnvoyerMessageJDB(WOM__Message::MSG_WOM_WO_VALID, nomFichier);
	 }

  //@ SI la liste etait vide, alors lancer le traitement
  if (m_EtatServeur == WOM_STATE_WITHOUT_STEP)
  {
    LancerTraitement();
    EventUpdate(WOM_EVENT_STEP);
  }
  //@ FIN SI la liste etait vide
}


//@ ------------------------------------------------------------------*/
//@ Methode : TraiterCommandeSTOP
//@
//@ Description : Permet de prendre en compte une commande STOP
//@
//@ Parametres :
//@               (ENTREE) commande : Commande STOP
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::TraiterCommandeSTOP(MP__Command& commande)
{
  // Identifiant de la commande STOP envoyee par le MP
  CMN__Cmd::t_CMN__CommandId idCommande = commande.GetCommandId();


  //@ 1. Pre-traitements

  //@ SI le serveur de donnees est actuellement suspendu
  if (m_EtatServeur == WOM_STATE_SUSPENDED)
    {   
      //@ SI la commande n'est pas une commande interne
      if (idCommande != CMN__OPS_INTERNAL_CMD)
        {
          //@ Envoyer un signal SIGCONT au SD
          //@ SI la commande a echoue
          if (kill(commande.GetProcessId(), SIGCONT) != 0)
            {
              ERR__Exception exception;
              
              //@ Envoi d'un message STOP, ABORTED        
              commande.SetCommandStatus(CMN__CmdTM::ABORTED);
              EnvoyerCommandeTM(commande); 
              
              //@ Generation d'une exception
              exception.SetErrorCode(ERR__Exception::WOM_SIGNAL_ERROR);
              exception.SetParametre("SIGCONT ERROR while STOPPING");
              THROW(exception)
            }
          //@ FIN SI la commande a echoue
        }
      //@ FIN SI la commande n'est pas une commande interne

      m_EtatServeur = WOM_STATE_PROCESSING;

      //@ Envoyer un message de type JDB MSG_WOM_STEP_RESUME
      EnvoyerMessageJDB(WOM__Message::MSG_WOM_STEP_RESUME, 
                        m_GestionnaireWorkOrder.
                        WorkOrderCourant()->Id());
      
      //@ Envoi d'un message HKTM Readiness WORKING
      EnvoyerTMReadiness(MP__HKTMStatus::WORKING);
    }
  //@ FIN SI
  //@ Fin des pre-traitements
  //@

  //@ 2. Traitements
  //@ SI le serveur de donnees est en traitement
  if (m_EtatServeur == WOM_STATE_PROCESSING)
    {   
      m_IdCdeInt = commande.GetCommandId();

      //@ SI la commande n'est pas une commande interne
      if (idCommande != CMN__OPS_INTERNAL_CMD)
        {
          //@ Lancer la procedure d'arret des traitements en cours
          StopperTraitement();
        }
      //@ SINON
      else
        {
          //@ Simuler l'envoi d'une fin de traitement par le SD
          SimulerFinTraitement();
        }
      //@ FIN SI la commande n'est pas une commande interne
    }
  //@ SINON
  else
    {
      //@ Envoi d'un message STOP, COMPLETED
      commande.SetCommandStatus(CMN__CmdTM::COMPLETED);
      EnvoyerCommandeTM(commande);

      //@ Envoi d'un message FIN DUMP, NOK
      EnvoyerFinDump(MP__Dump::FIN_NOK);
    }
  //@ FIN SI
  //@ Fin des traitements
  //@
}


//@ ------------------------------------------------------------------*/
//@ Methode : LancerTraitement
//@
//@ Description : Demande au SD de lancer un traitement
//@
//@ Parametres :
//@               Aucun 
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::LancerTraitement()
{
  // Traitement a lancer
  SD_DON_Traitement* messageTraitement = NULL;

  // Message a envoyer
  MSG__Message       message;

  //@ Recuperer le traitement courant du Work Order courant
  m_GestionnaireWorkOrder.RecupererTraitement(&messageTraitement);
  
  //@ Envoi d'un message de lancement d'un traitement au SD
  
#ifdef OPS_DEBUG
  cout << "WOM: affichage traitement a lancer:" << endl; 
  messageTraitement->Afficher();
#endif
              
  message = messageTraitement->Serialiser();
  message.SetAdresse(CFG__Configuration::GetAdresse(SD_INPUT));
  EnvoyerMessage(message);

  //@ Envoi d'un message HKTM Readiness WORKING
  EnvoyerTMReadiness(MP__HKTMStatus::WORKING);
}


//@ ------------------------------------------------------------------*/
//@ Methode : StopperTraitement
//@
//@ Description : Demande au SD d'arreter son traitement en cours
//@
//@ Parametres :
//@               Aucun 
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::StopperTraitement()
{
  // Message a envoyer
  MSG__Message message;


  //@ Envoi d'un message d'arret de traitement au SD
  message.SetType(SD__Message::EVT_ARRET_TMT_SD);
  message.SetAdresse(CFG__Configuration::GetAdresse(SD_INPUT));
  EnvoyerMessage(message);
}


//@ ------------------------------------------------------------------*/
//@ Methode : SimulerFinTraitement
//@
//@ Description : Simule l'envoi d'une fin de traitement par le SD
//@
//@ Parametres :
//@               Aucun 
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::SimulerFinTraitement()
{
  // Traitement termine
  SD_DON_Traitement* traitementEnvoye = NULL;
  SD_DON_Traitement  messageTraitement;

  // Message a envoyer
  MSG__Message       message;

  // Date de fin
  t_Date             date;

  // Liste vide temporaire
  t_ListeChaine      listeVide;
  
  // Message de log
  t_Chaine           log("");


  //@ Recuperer le traitement courant du Work Order courant
  m_GestionnaireWorkOrder.RecupererTraitement(&traitementEnvoye);
  messageTraitement = *traitementEnvoye;

  //@ Mettre a jour les informations de fin de traitement
  messageTraitement.Etat(SD_DON_Traitement::SD_TMT_NOK);
  messageTraitement.PCD(0);
  date.DateCourante();
  messageTraitement.StartTime(date);
  messageTraitement.StopTime(date);
  messageTraitement.ElapsedTime(0.0);
  messageTraitement.UserTime(0.0);
  log = "Treatment stopped due to an unrecoverable error on SD: ";
  messageTraitement.LogMessage(log + 
                                messageTraitement.TypeTraitementStr());
      
  //@ Vider les listes de fichiers
  listeVide.clear();
  messageTraitement.ListeProduitE(listeVide);
  messageTraitement.ListeProduitS(listeVide);
  messageTraitement.ListeDonneesAux(listeVide);
  messageTraitement.ListeProductModel(listeVide);
  messageTraitement.ListeProduitAVHRR(listeVide);

  //@ Envoi d'un message de fin de traitement
  message = messageTraitement.Serialiser();
  message.SetAdresse(CFG__Configuration::GetAdresse(WOM_INPUT));
  EnvoyerMessage(message);
}


//@ ------------------------------------------------------------------*/
//@ Methode : EnvoyerCommandeTM
//@
//@ Description : Permet d'envoyer un message d'acquittement a une
//@               commande envoyee par le MP
//@
//@ Parametres :
//@               (ENTREE) commande : Message a envoyer
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::EnvoyerCommandeTM(MP__Command& commande)
{
  //@ En captant les exceptions
  try 
    { 
      MSG__Message message;

      //@ Construction du message
      commande.SetCommandGroup(CMN__Cmd::ACK);
      commande.Serialiser(MP__Message::EVT_CMD_TM, message);
      message.SetAdresse(CFG__Configuration::GetAdresse(MP_INPUT));

      //@ Publication du message
      m_ConnexionServeur.PublierMessage(message); 
    }
  
  //@ Exploiter les exceptions
  catch (ERR__Exception &exception) 
    {
      //@ Appel de la routine de gestion des exceptions
      ManageException(exception);
    }  
}


//@ ------------------------------------------------------------------*/
//@ Methode : EnvoyerStageTM
//@
//@ Description : Permet d'envoyer un message d'acquittement de la
//@               generation d'un produit au MP
//@
//@ Parametres :
//@               (ENTREE) stageTM : Acquittement de produit a envoyer
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::EnvoyerStageTM(MP__StageTM& stageTM)
{
  //@ En captant les exceptions
  try 
    { 
      MSG__Message message;

      //@ Construction du message
      stageTM.Serialiser(message);

      //@ Publication du message
      m_ConnexionServeur.PublierMessage(message); 
    }
  
  //@ Exploiter les exceptions
  catch (ERR__Exception &exception) 
    {
      //@ Appel de la routine de gestion des exceptions
      ManageException(exception);
    }  
}


//@ ------------------------------------------------------------------*/
//@ Methode : EnvoyerTMReadiness
//@
//@ Description : Permet d'envoyer une telemesure de type Readiness
//@
//@ Parametres :
//@               (ENTREE) status : Statut Readiness
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::EnvoyerTMReadiness(MP__HKTMStatus::
                                                   e_ReadinessState
                                                   status)
{
  //@ En captant les exceptions
  try 
    { 
      MP__HKTMStatus telemesure(MP__HKTMStatus::OPSReadiness);
      MSG__Message   message;

      //@ Construction du message HKTM Status
      telemesure.SetAvailability(MP__HKTMStatus::AVAILABLE);
      telemesure.SetValeurChaine(CMN_HKTM_READINESS[status]);
      telemesure.Serialiser(message);

      //@ Publication du message
      m_ConnexionServeur.PublierMessage(message); 
    }
  
  //@ Exploiter les exceptions
  catch (ERR__Exception &exception) 
    {
      //@ Appel de la routine de gestion des exceptions
      ManageException(exception);
    }  
}


//@ ------------------------------------------------------------------*/
//@ Methode : EnvoyerFinDump
//@
//@ Description : Permet d'envoyer un message de fin de dump au MP
//@
//@ Parametres :
//@               (ENTREE) etat : Etat du dump
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::EnvoyerFinDump(MP__Dump::e_EtatFinDump etat)
{
  //@ En captant les exceptions
  try 
    { 
      MSG__Message message;
      MP__Dump     finDump;

      //@ Construction du message
      finDump.SetEtatFinDump(etat);
      message = finDump.Serialiser();

      //@ Publication du message
      m_ConnexionServeur.PublierMessage(message); 
    }
  
  //@ Exploiter les exceptions
  catch (ERR__Exception &exception) 
    {
      //@ Appel de la routine de gestion des exceptions
      ManageException(exception);
    }  
}


//@ ------------------------------------------------------------------*/
//@ Methode : DumperServeur
//@
//@ Description : Affiche la description du serveur
//@
//@ Parametres :
//@               (ENTREE) info
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::DumperServeur(CMN__Chaine info)
{
  //@ Afficher l'etat du serveur
  cout << endl << "Etat du Serveur WOM -> " << info << endl;
  cout << "------------------------------------" << endl;
  cout << "Etat courant : " << Etat() << endl;
}


//@ ------------------------------------------------------------------*/
//@ Methode : Etat
//@
//@ Description : Renvoie l'etat du serveur sous forme ASCII
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Etat du serveur sous forme ASCII
//@
//@ ------------------------------------------------------------------*/
t_Chaine WOM__GestionnaireActivite::Etat()
{
  //@ Par defaut, le nom de l'etat est: etat invalide
  t_Chaine nomEtat(INVALID_STR);

  //@ SI l'etat a une valeur comprise dans l'ensemble des valeurs
  //@ autorisees
  if ((m_EtatServeur >= 0) && (m_EtatServeur < WOM_STATE_NB))
    {
      //@ Affecter au nom de l'etat la valeur correspondante
      //@ dans le tableau des etats 
      nomEtat = STATE_STR[m_EtatServeur];
    }
  //@ FIN SI


  //@ Renvoyer le nom de l'etat du serveur
  return nomEtat;
}


//@ ------------------------------------------------------------------*/
//@ Methode : EventStateInit
//@
//@ Description : Permet d'initialiser le tableau des etats du serveur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::EventStateInit()
{
  int i = 0;
  int j = 0;


  //@ Initialisation de tous les etats a invalide
  for (i=0; i<WOM_EVENT_NB; i++)
    {
      for (j=0; j<WOM_STATE_NB; j++)
        {
          m_EventState[i][j] = WOM_STATE_INVALID;
        }
    }
  
  //@ Etat : WOM_STATE_INITIALISATION
  m_EventState [WOM_EVENT_END_INIT] [WOM_STATE_INITIALISATION ] = WOM_STATE_WITHOUT_STEP;
  m_EventState [WOM_EVENT_STOP    ] [WOM_STATE_INITIALISATION ] = WOM_STATE_ERROR;
  m_EventState [WOM_EVENT_END     ] [WOM_STATE_INITIALISATION ] = WOM_STATE_DEAD;

  //@ Etat : WOM_STATE_WITHOUT_STEP
  m_EventState [WOM_EVENT_STEP    ] [WOM_STATE_WITHOUT_STEP   ] = WOM_STATE_PROCESSING;
  m_EventState [WOM_EVENT_STOP    ] [WOM_STATE_WITHOUT_STEP   ] = WOM_STATE_ERROR;
  m_EventState [WOM_EVENT_END     ] [WOM_STATE_WITHOUT_STEP   ] = WOM_STATE_DEAD;

  //@ Etat : WOM_STATE_PROCESSING 
//FA 115 : on autorise le passage de l'etat WOM_STATE_PROCESSING a WOM_STATE_WITHOUT_STEP
  m_EventState [WOM_EVENT_NO_STEP ] [WOM_STATE_PROCESSING     ] = WOM_STATE_WITHOUT_STEP;
  m_EventState [WOM_EVENT_STEP    ] [WOM_STATE_PROCESSING     ] = WOM_STATE_PROCESSING;
  m_EventState [WOM_EVENT_END_TMT ] [WOM_STATE_PROCESSING     ] = WOM_STATE_POST_PROCESSING;
  m_EventState [WOM_EVENT_SUSPEND ] [WOM_STATE_PROCESSING     ] = WOM_STATE_SUSPENDED;
  m_EventState [WOM_EVENT_BREAK   ] [WOM_STATE_PROCESSING     ] = WOM_STATE_BREAKING;
  m_EventState [WOM_EVENT_STOP    ] [WOM_STATE_PROCESSING     ] = WOM_STATE_STOPPING;
  m_EventState [WOM_EVENT_END     ] [WOM_STATE_PROCESSING     ] = WOM_STATE_DEAD;

  //@ Etat : WOM_STATE_SUSPENDED
  m_EventState [WOM_EVENT_STEP    ] [WOM_STATE_SUSPENDED      ] = WOM_STATE_SUSPENDED;
  m_EventState [WOM_EVENT_RESUME  ] [WOM_STATE_SUSPENDED      ] = WOM_STATE_PROCESSING;
  m_EventState [WOM_EVENT_STOP    ] [WOM_STATE_SUSPENDED      ] = WOM_STATE_STOPPING;
  m_EventState [WOM_EVENT_END     ] [WOM_STATE_SUSPENDED      ] = WOM_STATE_DEAD;

  //@ Etat : WOM_STATE_POST_PROCESSING
  m_EventState [WOM_EVENT_NO_STEP ] [WOM_STATE_POST_PROCESSING] = WOM_STATE_WITHOUT_STEP;
  m_EventState [WOM_EVENT_STEP    ] [WOM_STATE_POST_PROCESSING] = WOM_STATE_PROCESSING;
  m_EventState [WOM_EVENT_END_DUMP] [WOM_STATE_POST_PROCESSING] = WOM_STATE_END;

  //@ Etat : WOM_STATE_BREAKING
  m_EventState [WOM_EVENT_END_TMT ] [WOM_STATE_BREAKING       ] = WOM_STATE_BREAKING;
  m_EventState [WOM_EVENT_NO_STEP ] [WOM_STATE_BREAKING       ] = WOM_STATE_WITHOUT_STEP;
  m_EventState [WOM_EVENT_STEP    ] [WOM_STATE_BREAKING       ] = WOM_STATE_PROCESSING;
  m_EventState [WOM_EVENT_STOP    ] [WOM_STATE_BREAKING       ] = WOM_STATE_ERROR;

  //@ Etat : WOM_STATE_STOPPING
  m_EventState [WOM_EVENT_END_TMT ] [WOM_STATE_STOPPING       ] = WOM_STATE_STOPPING;
  m_EventState [WOM_EVENT_STOP    ] [WOM_STATE_STOPPING       ] = WOM_STATE_ERROR;

  //@ Etat : WOM_STATE_END
  m_EventState [WOM_EVENT_END     ] [WOM_STATE_END            ] = WOM_STATE_DEAD;

  //@ Etat : WOM_STATE_ERROR
  m_EventState [WOM_EVENT_END     ] [WOM_STATE_ERROR          ] = WOM_STATE_DEAD;
}


//@ ------------------------------------------------------------------*/
//@ Methode : EventCheck
//@
//@ Description : Verifie si un evenement est compatible avec l'etat
//@               courant du serveur. Si c'est le cas, le serveur
//@               passe dans le nouvel etat
//@
//@ Parametres :
//@               (ENTREE) event : Evenement recu
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK si l'evenement est compatible
//@               CMN__STATUT_NOK si l'evenement n'est pas compatible
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut WOM__GestionnaireActivite::EventCheck(e_WOM__Event event)
{
  // Valeur de retour de la methode
  t_CMN__Statut status = CMN__STATUT_NOK;
  

  //@ SI l'evenement est autorise dans l'etat courant
  if (m_EventState [event] [m_EtatServeur] != WOM_STATE_INVALID)
    {
      //@ Mettre a jour la valeur de retour de la methode
      status = CMN__STATUT_OK;
    }
  //@ FIN SI


  //@ Renvoyer la valeur de retour de la methode
  return status;
}


//@ ------------------------------------------------------------------*/
//@ Methode : EventUpdate
//@
//@ Description : Met a jour l'etat du serveur en fonction d'un 
//@               evenement recu
//@
//@ Parametres :
//@               (ENTREE) event : Evenement recu
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireActivite::EventUpdate(e_WOM__Event event)
{
  //@ Mettre a jour l'etat du serveur en fonction de l'evenement recu
  m_EtatServeur = m_EventState [event] [m_EtatServeur];
}
