// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/WOM/src/WOM__GestionnaireActivite.h,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : WOM
//@ CLASSE : WOM__GestionnaireActivite
//@
//@ Commentaire :
//@    Fichier d'interface de la classe WOM__GestionnaireActivite, qui 
//@    gere l'etat et les interfaces externes du process WOM: lecture
//@    et traitement de messages recus, envoi de messages
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-099-CN : 21/10/2003 : JL.PASCAL
//@ no entry was found in the report for the stop command 
//@ a message is added in TraiterFinTraitement
//@ fiche de correction correspondante : IA-2100-FC-099-THA
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef WOM__GestionnaireActivite_h
#define WOM__GestionnaireActivite_h


//
// Fichiers include
//

// STL

// Standard

// Projet
#include "CMN__Commun.h"
#include "CMN__StageTM.h"
#include "MSG__Connexion.h"
#include "MP__Message.h"
#include "MP__Command.h"
#include "MP__Dump.h"
#include "MP__HKTMStatus.h"
#include "SD__Init.h"
#include "SD_DON_Traitement.h"
#include "WOM__GestionnaireWorkOrder.h"


class WOM__GestionnaireActivite 
{
    /*================================================================*/

 public:
    
    ////////////////////////////////
    // Constructeur
    ////////////////////////////////
    WOM__GestionnaireActivite();
    
    ////////////////////////////////
    // Destructeur
    ////////////////////////////////
    ~WOM__GestionnaireActivite();
    
    ////////////////////////////////
    // Accesseurs
    ////////////////////////////////

    ////////////////////////////////
    // Methodes publiques
    ////////////////////////////////
    void           ConnecterServeur(char *host, char *service);
    void           Ecouter();
    void           Terminer(t_CMN__Booleen flagFin)
        {m_FlagFin = flagFin;};
    t_CMN__Booleen Terminer() {return m_FlagFin;};
    void           InitialiserGestionnaireWO();
    void           EnvoyerMessage(MSG__Message& message);
    void           EnvoyerMessageJDB(WOM__Message::e_TypeMessageJDB
                                     msgId, 
                                     t_Chaine prm1 = "", 
                                     t_Chaine prm2 = "", 
                                     t_Chaine prm3 = "");

        
    /*================================================================*/

 protected:

    ////////////////////////////////
    // Methodes protegees
    ////////////////////////////////

    ////////////////////////////////
    // Attributs proteges
    ////////////////////////////////


    /*================================================================*/

 private:
    
    ////////////////////////////////
    // Type definitions
    ////////////////////////////////

    typedef enum {
        WOM_EVENT_END_INIT =  0,
        WOM_EVENT_NO_STEP  =  1,
        WOM_EVENT_STEP     =  2,
        WOM_EVENT_BREAK    =  3,
        WOM_EVENT_SUSPEND  =  4,
        WOM_EVENT_RESUME   =  5,
        WOM_EVENT_STOP     =  6,
        WOM_EVENT_END_TMT  =  7,
        WOM_EVENT_END_DUMP =  8,
        WOM_EVENT_END      =  9,
        WOM_EVENT_NB       = 10
    } e_WOM__Event;
    
    typedef enum {
        WOM_STATE_INVALID         = -1,
        WOM_STATE_INITIALISATION  =  0,
        WOM_STATE_WITHOUT_STEP    =  1,
        WOM_STATE_PROCESSING      =  2,
        WOM_STATE_SUSPENDED       =  3,
        WOM_STATE_POST_PROCESSING =  4,
        WOM_STATE_BREAKING        =  5,
        WOM_STATE_STOPPING        =  6,
        WOM_STATE_END             =  7,
        WOM_STATE_ERROR           =  8,
        WOM_STATE_DEAD            =  9,
        WOM_STATE_NB              = 10
    } e_WOM__State;
    
    ////////////////////////////////
    // Constantes
    ////////////////////////////////
    static const t_Chaine MP_INPUT;
    static const t_Chaine WOM_INPUT;
    static const t_Chaine SD_INPUT;
    static const t_Chaine TIMEOUT_WOM;
    static const t_Chaine LISTE_WOM;

    static const t_Chaine JDB_PRM_1;
    static const t_Chaine JDB_PRM_2;
    static const t_Chaine JDB_PRM_3;

    static const t_Chaine INVALID_STR;
    static const t_Chaine STATE_STR[];

    static const t_Chaine LIBELLE_FIN_TRAITEMENT_STOP;
    static const t_Chaine LIBELLE_FIN_TRAITEMENT_BREAK;

    ////////////////////////////////
    // Methodes privees
    ////////////////////////////////
    void          ManageException(ERR__Exception exception);

    // Gestion de la communication
    void          TraiterMessage(const MSG__Message& message);
    t_CMN__Statut AccepterCommande(CMN__CmdTM::e_CMN__CommandType
                                   typeCommande);
    void          TraiterCommande(MP__Command& commande);
    void          TraiterFinInit(SD__Init& init);
    void          TraiterFinTraitement(SD_DON_Traitement& traitement);
    void          TraiterFinDegrade(SD_DON_Traitement::e_ListeStatus etat,
                                    int idSTEP, 
                                    CMN__StageTM::e_CMN__StageId
                                    idStage,
                                    t_CMN__Booleen finDump);
    void          PostTraiter(int idSTEP,
                              CMN__StageTM::e_CMN__StageId idStage,
                              int valeurPCD);
    void          PostTraiterDegrade(int idSTEP, 
                                     CMN__StageTM::e_CMN__StageId
                                     idStage);
    void          TraiterCommandeSTEP(MP__Command& commande);
    void          TraiterCommandeSTOP(MP__Command& commande);
    void          LancerTraitement();
    void          StopperTraitement();
    void          SimulerFinTraitement();
    void          EnvoyerCommandeTM(MP__Command& commande);
    void          EnvoyerStageTM(MP__StageTM& stageTM);
    void          EnvoyerTMReadiness(MP__HKTMStatus::e_ReadinessState
                                     status);
    void          EnvoyerFinDump(MP__Dump::e_EtatFinDump etat);
    
    // Gestion de l'etat du serveur
    void          DumperServeur(CMN__Chaine info);
    t_Chaine      Etat();
    void          EventStateInit();
    int           EventCheck(e_WOM__Event event);
    void          EventUpdate(e_WOM__Event event);


    ////////////////////////////////
    // Attributs prives
    ////////////////////////////////

    // Flag de fin
    t_CMN__Booleen             m_FlagFin;
        
    // Commande d'interruption
    int                        m_IdCdeInt;

    // Gestion de l'etat du serveur
    e_WOM__State               m_EventState[WOM_EVENT_NB][WOM_STATE_NB];
    e_WOM__State               m_EtatServeur;

    // Gestionnaire de traitements
    WOM__GestionnaireWorkOrder m_GestionnaireWorkOrder;

    // Connexion avec le serveur
    MSG__Connexion             m_ConnexionServeur;
};

#endif
