// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/WOM/src/WOM__GestionnaireWorkOrder.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : WOM
//@ CLASSE : WOM__GestionnaireWorkOrder
//@
//@ Commentaire :
//@    Ce fichier contient la classe WOM__GestionnaireWorkOrder, qui
//@    gere la liste des STEP que le WOM doit piloter
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-115-CN : 10/02/2004 : JL.PASCAL
//@ no report were generated in cas of missing data
//@ fiche de correction correspondante : IA-2100-FC-115-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-159-CN : 26/05/2005 : JL. PASCAL
//@ modif suite a portage linux
//@ fiche de correction correspondante : IA-2100-FC-159-THA
//@ --------------------------------------------------------------------
//@ DM-ID :IA-2100-CR-287-CNE : 08/03/2006 : B. GUENDE
//@ OPS must run if the type of the work order is FIRST or MIDDLE.
//@ OPS still stops if the type is LAST. See IA-2100-CR-287-THA.
//@ --------------------------------------------------------------------
//@ PM-ID :IA-2100-CR-305-CN : 03/07/2006 : C. MERIGUET
//@ Output CTX files are copied in input auxiliary files directory. 
//@ (patch3-5-5). See IA-2100-CR-305-THA.
//@ --------------------------------------------------------------------
//@ MODIFIED FOR PATCH3-5-5 : 05/Jul/06
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL

// Standard
#include "errno.h"

// Projet
#include "CMN__Commun.h"
#include "CFG__Configuration.h"
#include "CMN__Fichier.h"
#include "CMN_DAT_WorkOrder.h"
#include "CMN_PRS_Parser.h"
#include "WOM__WorkOrder.h"
#include "WOM__GestionnaireWorkOrder.h"


//
// Definition des constantes 
//
const t_Chaine WOM__GestionnaireWorkOrder::DIR_ROOT        = "WORKING_ROOT";
const t_Chaine WOM__GestionnaireWorkOrder::DIR_PGF_OUT     = "PGF_OUT_DIR";
const t_Chaine WOM__GestionnaireWorkOrder::DIR_DATA_TMP    = "DATA_TMP_DIR";

// CR305
const t_Chaine WOM__GestionnaireWorkOrder::DIR_IN_AUX      = "IN_AUX_DIR";

//FA159
const t_Chaine WOM__GestionnaireWorkOrder::CDE_CLEAN_BEG   = "/bin/sh -c 'rm -rf ";
const t_Chaine WOM__GestionnaireWorkOrder::CDE_CLEAN_END   = "*'";

const t_Chaine WOM__GestionnaireWorkOrder::FILE_NOT_FOUND  = "File not found";
const t_Chaine WOM__GestionnaireWorkOrder::UNEXPECTED_STEP = "unexpected time interval flag: ";
const t_Chaine WOM__GestionnaireWorkOrder::ERROR_NEW_WO    = "Unable to add a new WorkOrder";

/* CR305 */
const t_Chaine WOM__GestionnaireWorkOrder::TYPE_CTX	   = "_CTX_";

//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : WOM__GestionnaireWorkOrder
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
WOM__GestionnaireWorkOrder::WOM__GestionnaireWorkOrder()
{
  //@ Initialiser les attributs
  m_TimeIntervalFlag = TIF_UNDEFINED;
  m_PrecedentTIF     = TIF_UNDEFINED;
  m_RepertoireRoot   = "";
  m_RepertoireOut    = "";
  m_RepertoireTmp    = "";
  m_RepInputAux      = ""; /* CR305 */
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~WOM__GestionnaireWorkOrder
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
WOM__GestionnaireWorkOrder::~WOM__GestionnaireWorkOrder()
{
  //@ RAZ de la liste des Work Orders
  m_ListeWorkOrder.clear();
}


//@ ------------------------------------------------------------------*/
//@ Methode : AjouterWorkOrder
//@
//@ Description : Lance le parsing d'un fichier Work Order, decompose
//@               le Work Order en un (mode GRANULE) ou plusieurs 
//@               (mode DUMP) traitements et ajoute le Work order dans
//@               la liste des Work Orders
//@
//@ Parametres :
//@               (ENTREE/SORTIE) nomWorkOrder : Nom du Work Order
//@               (ENTREE) noChrono : Chrono du STEP
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireWorkOrder::AjouterWorkOrder(CMN__Chaine &nomWorkOrder,
                                                  int noChrono) 
  throw (ERR__Exception)
{
  // Work Order a ajouter
  WOM__WorkOrder* workOrder = NULL;
  

  //@ En captant les exceptions
  try
    {
      // Exception generee
      ERR__Exception     exception;

      // Parseur XML
      CMN_PRS_Parser     parser;

      // Chemin complet du fichier Work Order
      t_Chaine           nomFichier("");

      // Liste de fichiers
      t_ListeChaine      listeFichier;

      // Time interval flag
      e_TimeIntervalFlag nouveauTIF = m_TimeIntervalFlag;

      // Code retour des fonctions utilisees
      t_CMN__Booleen     status     = CMN__FAUX;


      //@ Recuperation des noms complet et court du fichier Work Order
      nomFichier   = NomComplet(nomWorkOrder);
      nomWorkOrder = NomCourt(nomWorkOrder);

      //@ Tester l'existence du fichier Work Order
      status = CMN__RechercherFichier(nomFichier, 
                                      CMN__FAUX, 
                                      listeFichier);

      //@ SI le fichier n'existe pas
      if (status == CMN__FAUX)
        {
          //@ Emission d'une exception WOM_INVALID_WO
          exception.SetErrorCode(ERR__Exception::WOM_INVALID_WO);
          exception.SetParametre(FILE_NOT_FOUND);
          THROW(exception)
        }
      //@ FIN SI le fichier n'existe pas

      //@ Creer un nouveau Work Order
      workOrder = new WOM__WorkOrder(noChrono, 
                                     nomWorkOrder, 
                                     nomFichier);

      //@ Initialiser le parsing du fichier WorkOrder
      parser.ParseFile(nomFichier);
      
      //@ Charger l'arbre XML dans le Work Order
      parser.ParseWorkOrder(workOrder->Data());
      
      
#ifdef OPS_DEBUG
      workOrder->Data().Afficher();
#endif
      
      
      //@ Tester l'acceptation du traitement en fonction
      //@ de son TimeIntervalFlag
      t_Chaine nomTIF = workOrder->Data().TimeIntervalFlag();
      if (TimeIntervalFlagCheck(nomTIF, nouveauTIF) != CMN__STATUT_OK)
        {
          //@ Emission d'une exception WOM_WRONG_TIF
          exception.SetErrorCode(ERR__Exception::WOM_INVALID_WO);
          exception.SetParametre(UNEXPECTED_STEP+nomTIF);
          THROW(exception);
        }

      //@ Identifier les traitements
      workOrder->IdentifierTraitement();
      
      //@ Ajouter le Work Order dans la liste
      m_ListeWorkOrder.push_back(workOrder);

      //@ Mettre a jour le time interval flag
      m_PrecedentTIF     = m_TimeIntervalFlag;
      m_TimeIntervalFlag = nouveauTIF;
    }

  //@ Traiter les exceptions
  catch (ERR__Exception &exception) 
    {
      //@ SI un WorkOrder a ete cree
      if (workOrder)
        {
          //@ Effacer le WorkOrder
          delete workOrder;
        }
      //@ FIN SI un WorkOrder a ete cree

      //@ Re-emission de l'exception
      exception.SetParametre(ERROR_NEW_WO); 
      throw(exception);
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : RecupererTraitement
//@
//@ Description : Determine et renvoie le traitement a lancer 
//@
//@ Parametres :
//@               (SORTIE) traitement : Traitement a lancer
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireWorkOrder::RecupererTraitement(SD_DON_Traitement** 
                                                     traitement)
  throw (ERR__Exception)
{
  //@ SI la liste des Work Order est vide
  if (m_ListeWorkOrder.size() == 0)
    {
      ERR__Exception exception;

      //@ Le traitement sera vide
      traitement = NULL;

      //@ Emission exception WOM_TMT_NOT_FOUND
      exception.SetErrorCode(ERR__Exception::WOM_TMT_NOT_FOUND);
      exception.SetParametre("RecupererTraitement()");
      THROW(exception)
    }
  //@ FIN SI la liste des Work Order est vide

  //@ Recuperer le traitement courant du Work Order courant
  *traitement = WorkOrderCourant()->TraitementCourant();

#ifdef OPS_DEBUG
  cout << "WOM: Traitement a lancer:" << endl; 
  (*traitement)->Afficher();
#endif
}


//@ ------------------------------------------------------------------*/
//@ Methode : WorkOrderCourant
//@
//@ Description : Renvoie le Work Order courant de la liste des Work
//@               Orders
//@
//@ Parametres :
//@               Aucun 
//@
//@ Valeur de retour :
//@               Le Work Order courant
//@
//@ ------------------------------------------------------------------*/
WOM__WorkOrder* WOM__GestionnaireWorkOrder::WorkOrderCourant()
{
  //@ Retourner le Work Order courant
  //@ Le Work Order courant est le premier Work Order de la liste
  return m_ListeWorkOrder.front();
}
//FA 115 pour recuperer le dernier wo introduit
//@ ------------------------------------------------------------------*/
//@ Methode : DernierWorkOrder
//@
//@ Description : Renvoie le dernier Work Order insere dans la liste des Work
//@               Orders
//@
//@ Parametres :
//@               Aucun 
//@
//@ Valeur de retour :
//@               Le Work Order courant
//@
//@ ------------------------------------------------------------------*/
WOM__WorkOrder* WOM__GestionnaireWorkOrder::DernierWorkOrder()
{
  //@ Retourner le Work Order courant
  //@ Le Work Order courant est le premier Work Order de la liste
  return m_ListeWorkOrder.back();
}


//@ ------------------------------------------------------------------*/
//@ Methode : ExisteWorkOrder
//@
//@ Description : Teste l'existence d'au moins un Work Order dans la
//@               liste 
//@
//@ Parametres :
//@               Aucun 
//@
//@ Valeur de retour :
//@               CMN__VRAI si un Work Order existe dans la liste
//@               CMN__FAUX sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Booleen WOM__GestionnaireWorkOrder::ExisteWorkOrder()
{
  // Valeur de retour de la methode
  t_CMN__Booleen status = CMN__FAUX;


  //@ SI la liste n'est pas vide
  if (m_ListeWorkOrder.size() > 0)
    {
      //@ Mettre a jour la valeur de retour de la methode
      status = CMN__VRAI;
    }
  //@ FIN SI


  //@ Retourner la valeur de retour de la methode
  return status; 
}


//@ ------------------------------------------------------------------*/
//@ Methode : MettreAJour
//@
//@ Description : Met a jour le Work Order en fonction des 
//@               informations contenues dans le traitement
//@               passe en parametre
//@
//@ Parametres :
//@               (ENTREE) traitement : Traitement acheve
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireWorkOrder::MettreAJour(SD_DON_Traitement& traitement)
{
  //@ Mettre a jour le Work Order courant
  WorkOrderCourant()->MettreAJour(traitement);
}


//@ ------------------------------------------------------------------*/
//@ Methode : SupprimerWorkOrderCourant
//@
//@ Description : Supprime le Work Order courant de la liste des 
//@               Work Orders
//@
//@ Parametres :
//@               Aucun 
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireWorkOrder::SupprimerWorkOrderCourant()
{
  //@ Le Work Order courant est le premier Work Order de la liste:
  //@ supprimer ce premier element
  m_ListeWorkOrder.pop_front();
}


//@ ------------------------------------------------------------------*/
//@ Methode : InitialiserEspaceTravail
//@
//@ Description : Recupere les variables d'environnement correspondant
//@               aux differents repertoires de travail et nettoie
//@               l'espace de travail
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireWorkOrder::InitialiserEspaceTravail() throw (ERR__Exception)
{
  // Exception
  ERR__Exception exception;
  
  // Commande systeme
  t_Chaine       commande("");


  //@ Recuperation de la variable d'environnement REP_ROOT
  m_RepertoireRoot = CFG__Configuration::GetRepertoire(DIR_ROOT);

  //@ SI la variable d'environnement REP_ROOT n'est pas definie
  if (m_RepertoireRoot.empty())
    {
      //@ Emission d'une exception WOM_ENV_UNDEFINED
      exception.SetErrorCode(ERR__Exception::WOM_ENV_UNDEFINED);
      exception.SetParametre(DIR_ROOT);
      THROW(exception)
    }
  //@ FIN SI

  m_RepertoireRoot += CMN__SEP_REPERTOIRE;

  //@ Recuperation de la variable d'environnement REP_PGF_OUT_DIR
  m_RepertoireOut = CFG__Configuration::GetRepertoire(DIR_PGF_OUT);

  //@ SI la variable d'environnement REP_PGF_OUT_DIR n'est pas definie
  if (m_RepertoireOut.empty())
    {
      //@ Emission d'une exception WOM_ENV_UNDEFINED
      exception.SetErrorCode(ERR__Exception::WOM_ENV_UNDEFINED);
      exception.SetParametre(DIR_PGF_OUT);
      THROW(exception)
    }
  //@ FIN SI

  m_RepertoireOut += CMN__SEP_REPERTOIRE;

  //@ CR305 Recuperation de la variable d'environnement REP_IN_AUX_DIR
  m_RepInputAux = CFG__Configuration::GetRepertoire(DIR_IN_AUX);

  //@ SI la variable d'environnement REP_IN_AUX_DIR n'est pas definie
  if (m_RepInputAux.empty())
    {
      //@ Emission d'une exception WOM_ENV_UNDEFINED
      exception.SetErrorCode(ERR__Exception::WOM_ENV_UNDEFINED);
      exception.SetParametre(DIR_IN_AUX);
      THROW(exception)
    }
  //@ FIN SI

  m_RepInputAux += CMN__SEP_REPERTOIRE;
  
  //@ Recuperation de la variable d'environnement REP_DATA_TMP_DIR
  m_RepertoireTmp = CFG__Configuration::GetRepertoire(DIR_DATA_TMP);

  //@ SI la variable d'environnement REP_DATA_TMP_DIR n'est pas
  //@ definie
  if (m_RepertoireTmp.empty())
    {
      //@ Emission d'une exception WOM_ENV_UNDEFINED
      exception.SetErrorCode(ERR__Exception::WOM_ENV_UNDEFINED);
      exception.SetParametre(DIR_DATA_TMP);
      THROW(exception)
    }
  //@ FIN SI

  m_RepertoireTmp += CMN__SEP_REPERTOIRE;

  //@ Nettoyage du repertoire OUT (to_pgf)
  commande = CDE_CLEAN_BEG + m_RepertoireOut + CDE_CLEAN_END;
  system(commande.c_str());
  
  //@ Nettoyage du repertoire TMP (tmp)
  commande = CDE_CLEAN_BEG + m_RepertoireTmp + CDE_CLEAN_END;
  system(commande.c_str());
}


//@ ------------------------------------------------------------------*/
//@ Methode : GererProduits
//@
//@ Description : Assure la gestion des produits generes:
//@                 - cree un fichier de rapport pour un Work Order
//@                   termine
//@                 - deplace le rapport vers le repertoire PGF
//@                 - deplace ou supprime les autres produits generes
//@
//@ Parametres :
//@               (ENTREE) modeGestion : 
//@               (SORTIE) rapport : Nom du fichier rapport cree (en
//@                                  relatif par rapport au 
//@                                  WorkingRoot)
//@               (SORTIE) listeProduit : Liste des fichiers de
//@                                       produits generes par le
//@                                       Work Order (en relatif par
//@                                       rapport au WorkingRoot)
//@               (SORTIE) listeAux : Liste des fichiers de donnees
//@                                   auxilaires generes par le
//@                                   Work Order (en relatif par 
//@                                   rapport au WorkingRoot)
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireWorkOrder::GererProduits(e_Gestion_Produits 
                                               modeGestion,
                                               t_Chaine& rapport, 
                                               t_ListeChaine& 
                                               listeProduit,
                                               t_ListeChaine& listeAux)
{
  // Iterateur pour la liste des fichiers
  t_ListeChaine::iterator pInfo;


  //@ Demander au Work Order courant creer le rapport; le nom relatif
  //@ par rapport au repertoire temporaire des fichiers rapport, 
  //@ produits et aux sont recuperes
  rapport = m_RepertoireTmp;
  WorkOrderCourant()->CreerRapport(rapport, listeProduit, listeAux);

  //@ Gestion des produits:
  //@  SI le mode de gestion est DEPLACER, alors deplacer tous les 
  //@  produits (rapport et fichiers generes) dans le repertoire
  //@  approprie du PGF (to_pgf)

  //@  SI le mode de gestion est PURGER, alors deplacer le rapport
  //@  dans le repertoire approprie du PGF (to_pgf) et supprimer
  //@  tous les autres produits (fichiers generes)
  //@ Fin

  //@ Deplacement du REPORT
  DeplacerFichier(rapport);

  //@ Gestion des produits generes
      
  //@ SI le mode de gestion est DEPLACER
  if (modeGestion == PRODUITS_DEPLACER)
    {
      //@ POUR tous les produits
      for (pInfo = listeProduit.begin(); 
           pInfo != listeProduit.end(); 
           pInfo++)
        {
          //@ Deplacer  le fichier
          DeplacerFichier(*pInfo);
        }
      //@ FIN POUR tous les produits
      
      //@ POUR tous les fichiers auxiliaires
      for (pInfo = listeAux.begin(); pInfo != listeAux.end(); pInfo++)
        {
          //@ Deplacer  le fichier
          DeplacerFichier(*pInfo);
        }
      //@ FIN POUR tous les fichiers auxiliaires
   }
  //@ SINON SI le mode de gestion est PURGER
  else if (modeGestion == PRODUITS_PURGER)
    {
      //@ POUR tous les produits
      for (pInfo = listeProduit.begin(); pInfo != listeProduit.end(); 
           pInfo++)
        {
          //@ Supprimer le fichier
          SupprimerFichier(*pInfo);
        }
      //@ FIN POUR tous les produits

      //@ POUR tous les fichiers auxiliaires
      for (pInfo = listeAux.begin(); pInfo != listeAux.end(); pInfo++)
        {
          //@ Supprimer le fichier
          SupprimerFichier(*pInfo);
        }
      //@ FIN POUR tous les fichiers auxiliaires
    }
  //@ FIN SI mode de gestion
}


//=====================================================================
//                        METHODES PRIVEES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : NomComplet
//@
//@ Description : Transforme le nom de fichier en chemin complet. Le
//@               nom en entree est un chemin absolu ou un chemin
//@               relatif au Working Root
//@
//@ Parametres :
//@               (ENTREE) nomFichier : Nom du fichier
//@
//@ Valeur de retour :
//@               Nom complet du fichier
//@
//@ ------------------------------------------------------------------*/
t_Chaine WOM__GestionnaireWorkOrder::NomComplet(t_Chaine& nomFichier)
{
  //@ Affecter le nom de fichier a la variable du nom complet
  t_Chaine nomComplet(nomFichier);


  //@ SI le premier caractere n'est pas le separateur de repertoire
  if (nomFichier.at(0) != CMN__CAR_SEP_REPERTOIRE)
    {
      //@ Ajouter le repertoire Root en debut de nom complet
      nomComplet = m_RepertoireRoot + nomFichier;
    }
  //@ FIN SI


  //@ Retourner le nom complet du fichier
  return nomComplet;
}


//@ ------------------------------------------------------------------*/
//@ Methode : NomCourt
//@
//@ Description : Recupere le nom court d'un fichier
//@
//@ Parametres :
//@               (ENTREE) nomFichier : Nom du fichier
//@
//@ Valeur de retour :
//@               Nom court du fichier
//@
//@ ------------------------------------------------------------------*/
t_Chaine WOM__GestionnaireWorkOrder::NomCourt(t_Chaine& nomFichier)
{
  //@ Affecter le nom de fichier a la variable du nom court
  t_Chaine            nomCourt(nomFichier);
  t_Chaine::size_type posDebut = t_Chaine::npos;
  int                 longueur = 0;


  //@ Rechercher le caractere separateur de repertoire
  posDebut = nomFichier.rfind(CMN__SEP_REPERTOIRE);

  //@ SI le caractere a ete trouve
  if (posDebut != t_Chaine::npos)
    {
      posDebut += strlen(CMN__SEP_REPERTOIRE);
      longueur = nomFichier.length() - posDebut;

      //@ Affecter le nom court avec les caracteres suivants
      nomCourt = nomFichier.substr(posDebut, longueur);
    }
  //@ FIN SI


  //@ Retourner le nom court du fichier
  return nomCourt;
}


//@ ------------------------------------------------------------------*/
//@ Methode : NomRoot
//@
//@ Description : Recupere le chemin relatif d'un fichier par rapport
//@               au Working Root
//@
//@ Parametres :
//@               (ENTREE) nomFichier : Nom court du fichier
//@
//@ Valeur de retour :
//@               Nom relatif du fichier par rapport au Working Root
//@
//@ ------------------------------------------------------------------*/
t_Chaine WOM__GestionnaireWorkOrder::NomRoot(t_Chaine& nomFichier)
{
  //@ Affecter le nom de fichier a la variable du nom Root
  t_Chaine            nomRoot(nomFichier);
  t_Chaine::size_type posDebut = t_Chaine::npos;
  int                 longueur = 0;


  //@ Rechercher la position de la chaine du repertoire Root dans
  //@ la chaine du repertoire Out
  posDebut = m_RepertoireOut.find(m_RepertoireRoot);

  //@ SI la position est egale a 0
  if (posDebut == 0)
    {
      posDebut = m_RepertoireRoot.length();
      longueur = m_RepertoireOut.length() - posDebut;

      //@ Affecter au nom relatif du fichier par rapport au
      //@ Working Root :
      //@  - la difference entre le Working Root et le 
      //@    repertoire Out 
      //@  - le nom court du fichier
      nomRoot  = m_RepertoireOut.substr(posDebut, longueur) + 
        nomFichier;
    }
  //@ FIN SI


  //@ Retourner le nom relatif du fichier par rapport au Working Root
  return nomRoot;
}


//@ ------------------------------------------------------------------*/
//@ Methode : DeplacerFichier
//@
//@ Description : Deplace un fichier du repertoire de travail
//@               temporaire vers le repertoire de sortie et 
//@               input/aux si c'est un fichier de contexte
//@
//@ Parametres :
//@               (ENTREE/SORTIE) nomFichier : Nom du fichier
//@                                     - en entree: nom relatif par
//@                                       rapport au repertoire
//@                                       temporaire
//@                                     - en sortie: nom relatif par
//@                                       rapport au Working Root
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireWorkOrder::DeplacerFichier(t_Chaine& nomFichier)
{
  t_Chaine fichierTmp("");
  t_Chaine fichierOut("");
  t_Chaine fichierInAux(""); /* CR305 */
  t_Chaine type(TYPE_CTX);   /* CR305 */

  //@ Affecter a fichierTmp le nom du repertoire temporaire suivi du 
  //@ nom court du fichier passe en parametre
  fichierTmp = m_RepertoireTmp + nomFichier;

  //@ Affecter a fichierOut le nom du repertoire de sortie suivi du 
  //@ nom court du fichier passe en parametre
  fichierOut = m_RepertoireOut + nomFichier;

  //@ CR305 Affecter a fichierInAux le nom du repertoire d'entree suivi du 
  //@ nom court du fichier passe en parametre
  fichierInAux = m_RepInputAux + nomFichier;

  //@ Recuperer le chemin relatif du fichier par rapport au
  //@ Working Root
  nomFichier = NomRoot(nomFichier);

  //@ Renommer le fichier
  //@ SI le deplacement de fichierTmp vers fichierOut a genere
  //@ une erreur
  if (rename(fichierTmp.c_str(), fichierOut.c_str()) != 0)
    {
      ERR__Exception exception;
      
      //@ Emission exception WOM_FILE_WRITE_ERROR
      exception.SetErrorCode(ERR__Exception::WOM_FILE_WRITE_ERROR);
      exception.SetErrno(errno);
      exception.SetParametre(nomFichier);
      THROW(exception)
    }
  //@ FIN SI

  /* CR305 */
#ifdef OPS_DEBUG  
  cout << "Verification type du fichier"<<nomFichier.c_str() << endl;
  cout << "type trouve =" 
       << nomFichier.substr(nomFichier.rfind(CMN__SEP_REPERTOIRE)+5,type.length()) <<endl;
#endif
  //@ SI le fichier est de type contexte le copier dans input/aux egalement
  if (type == (nomFichier.substr(nomFichier.rfind(CMN__SEP_REPERTOIRE)+5, 
                                 type.length())))
  { 
#ifdef OPS_DEBUG
  cout << "fichier copier dans : "<< fichierInAux.c_str() <<endl;
#endif 
    //@ Copier le fichier (false => ecrasement du fichier si il existe deja
    t_CMN__Statut status = CMN__CopierFichier(
      fichierOut.c_str(), 
      fichierInAux.c_str(), 
      false);
#ifdef OPS_DEBUG
  cout << " CMN__CopierFichier : status = " << status << endl;
#endif
    //@ SI le deplacement du ctx dans input a genere
    //@ une erreur
    if (status != CMN__STATUT_OK)
    {
      ERR__Exception exception;

      //@ Emission exception WOM_FILE_WRITE_ERROR
      exception.SetErrorCode(ERR__Exception::WOM_FILE_WRITE_ERROR);
      exception.SetErrno(errno);
      exception.SetParametre(nomFichier);
      THROW(exception)
    } //@ FIN SI
  }  //@ FIN SI CR305
}


//@ ------------------------------------------------------------------*/
//@ Methode : SupprimerFichier
//@
//@ Description : Supprime un fichier du repertoire de travail
//@
//@ Parametres :
//@               (ENTREE) nomFichier : Nom court du fichier
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__GestionnaireWorkOrder::SupprimerFichier(t_Chaine& nomFichier)
{
  t_Chaine fichierTmp("");
  
  
  //@ Affecter au fichier le nom du repertoire temporaire suivi du 
  //@ nom court du fichier passe en parametre
  fichierTmp = m_RepertoireTmp + nomFichier;
  
  //@ Supprimer le fichier
  //@ SI la suppression du fichier a genere une erreur
  if (remove(fichierTmp.c_str()) != 0)
    {
      ERR__Exception exception;

      //@ Emission exception WOM_FILE_WRITE_ERROR
      exception.SetErrorCode(ERR__Exception::WOM_FILE_WRITE_ERROR);
      exception.SetParametre(fichierTmp);
      THROW(exception)
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : TimeIntervalFlagCheck
//@
//@ Description : Verifie si le time interval flag d'un work order est 
//@               compatible avec le time interval flag courant du
//@               serveur. Si c'est le cas, le time interval flag du
//@               serveur est mis a jour
//@
//@ Parametres :
//@               (ENTREE) nomTIF : Time interval flag recu
//@               (SORTIE) nouveauTIF : Time interval flag mis a jour
//@
//@ Valeur de retour :
//@               CMN__STATUT_OK si le time interval flag est compatible
//@               CMN__STATUT_NOK si le time interval flag n'est pas 
//@                 compatible
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut WOM__GestionnaireWorkOrder::TimeIntervalFlagCheck(const t_Chaine nomTIF, 
                                                                e_TimeIntervalFlag& nouveauTIF)
{
  // Valeur de retour de la methode
  t_CMN__Statut status = CMN__STATUT_NOK;
  

  //@ Tester l'acceptation du work order en fonction
  //@ du TimeIntervalFlag courant
  //@ Si accepte:
  //@   - mettre a jour la valeur de retour
  //@   - mettre a jour la valeur du TimeIntervalFlag


  //@ TIF_UNDEFINED: First et Full acceptes
  if (m_TimeIntervalFlag == TIF_UNDEFINED)
    {
      if (nomTIF == CMN_DAT_WorkOrder::WORK_ORDER_TYPE_FIRST)
        {
          status     = CMN__STATUT_OK;
          nouveauTIF = TIF_FIRST;
        }
      else if (nomTIF == CMN_DAT_WorkOrder::WORK_ORDER_TYPE_FULL)
        {
          status     = CMN__STATUT_OK;
          nouveauTIF = TIF_LAST;
        }
      
      /* CR287 */
      else if (nomTIF == CMN_DAT_WorkOrder::WORK_ORDER_TYPE_MIDDLE)
        {
          status     = CMN__STATUT_OK;
          nouveauTIF = TIF_FIRST;
        }
      /* FIN CR287 */
    }
  //@ TIF_FIRST: First, Middle et Last acceptes
  else if (m_TimeIntervalFlag == TIF_FIRST)
    {
      if (nomTIF == CMN_DAT_WorkOrder::WORK_ORDER_TYPE_FIRST)
        {
          status     = CMN__STATUT_OK;
          nouveauTIF = TIF_FIRST;
        }
      else if (nomTIF == CMN_DAT_WorkOrder::WORK_ORDER_TYPE_MIDDLE)
        {
          status     = CMN__STATUT_OK;
          nouveauTIF = TIF_MIDDLE;
        }
      else if (nomTIF == CMN_DAT_WorkOrder::WORK_ORDER_TYPE_LAST)
        {
          status     = CMN__STATUT_OK;
          nouveauTIF = TIF_LAST;
        }
    }
  //@ TIF_MIDDLE: Middle et Last acceptes
  else if (m_TimeIntervalFlag == TIF_MIDDLE)
    {
      if (nomTIF == CMN_DAT_WorkOrder::WORK_ORDER_TYPE_MIDDLE)
        {
          status     = CMN__STATUT_OK;
        }
      else if (nomTIF == CMN_DAT_WorkOrder::WORK_ORDER_TYPE_LAST)
        {
          status     = CMN__STATUT_OK;
          nouveauTIF = TIF_LAST;
        }
    }

  //@ TIF_LAST: work order refuse


  //@ Renvoyer la valeur de retour de la methode
  return status;
}
