// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/WOM/src/WOM__GestionnaireWorkOrder.h,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : WOM
//@ CLASSE : WOM__GestionnaireWorkOrder
//@
//@ Commentaire :
//@    Fichier d'interface de la classe WOM__GestionnaireWorkOrder, qui
//@    gere la liste des STEP que le WOM doit piloter
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-115-CN : 10/02/2004 : JL.PASCAL
//@ no report were generated in cas of missing data
//@ fiche de correction correspondante : IA-2100-FC-115-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-144-CN : 21/09/2004 : B. MASSART
//@ Problemes lors du portage d'OPS vers Solaris par Meteo France
//@ Voir IA-2100-FC-144-THA.
//@ --------------------------------------------------------------------
//@ PM-ID :IA-2100-CR-305-CN : 03/07/2006 : C. MERIGUET
//@ Output CTX files are copied in input auxiliary files directory. 
//@ (patch3-5-5). See IA-2100-CR-305-THA.
//@ --------------------------------------------------------------------
//@ MODIFIED FOR PATCH3-5-5 : 05/Jul/06
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef WOM__GestionnaireWorkOrder_h
#define WOM__GestionnaireWorkOrder_h

//
// Fichiers include
//

// STL

// Standard

// Projet
#include "CMN__Commun.h"
#include "ERR__Exception.h"
#include "SD_DON_Traitement.h"
#include "WOM__WorkOrder.h"


class WOM__GestionnaireWorkOrder {

    /*================================================================*/

 public:
    
    ////////////////////////////////
    // Type definitions
    ////////////////////////////////
      typedef enum {
        PRODUITS_DEPLACER = 0,
        PRODUITS_PURGER   = 1
    } e_Gestion_Produits;

   ////////////////////////////////
    // Constructeur
    ////////////////////////////////
    WOM__GestionnaireWorkOrder();
    
    ////////////////////////////////
    // Destructeur
    ////////////////////////////////
    ~WOM__GestionnaireWorkOrder();
    
    ////////////////////////////////
    // Accesseurs
    ////////////////////////////////

    ////////////////////////////////
    // Methodes publiques
    ////////////////////////////////

    // Gestion des Work Orders
    void            AjouterWorkOrder(CMN__Chaine &nomWorkOrder,
                                     int noChrono)
        throw (ERR__Exception);
    void            RecupererTraitement(SD_DON_Traitement** traitement)
        throw (ERR__Exception);
    WOM__WorkOrder* WorkOrderCourant();
//FA 115 : pour recuperer le dernier wo introduit
    WOM__WorkOrder* DernierWorkOrder();
    t_CMN__Booleen  ExisteWorkOrder();
    void            MettreAJour(SD_DON_Traitement& traitement);
    void            SupprimerWorkOrderCourant();

    // Gestion des donnees
    void            InitialiserEspaceTravail() throw (ERR__Exception);
    void            GererProduits(e_Gestion_Produits modeGestion,
                                  t_Chaine& report, 
                                  t_ListeChaine& listeProduit,
                                  t_ListeChaine& listeAux);


    /*================================================================*/

 protected:

    ////////////////////////////////
    // Methodes protegees
    ////////////////////////////////

    ////////////////////////////////
    // Attributs proteges
    ////////////////////////////////


    /*================================================================*/

 private:
    
    ////////////////////////////////
    // Type definitions
    ////////////////////////////////
    
    // Liste des Work Orders
    typedef list<WOM__WorkOrder*> t_ListeWorkOrder;

    // Valeur des TimeIntervalFlag geres
    typedef enum {
        TIF_UNDEFINED = -1,
        TIF_FIRST     =  0,
        TIF_MIDDLE    =  1,
        TIF_LAST      =  2 /* FA144 */
    } e_TimeIntervalFlag;

    ////////////////////////////////
    // Constantes
    ////////////////////////////////
    static const t_Chaine DIR_ROOT;
    static const t_Chaine DIR_PGF_OUT;
    static const t_Chaine DIR_DATA_TMP;
    static const t_Chaine DIR_IN_AUX; /* CR305 */
    static const t_Chaine TYPE_CTX;   /* CR305 */
   
    static const t_Chaine CDE_CLEAN_BEG;
    static const t_Chaine CDE_CLEAN_END;

    static const t_Chaine FILE_NOT_FOUND;
    static const t_Chaine UNEXPECTED_STEP;
    static const t_Chaine ERROR_NEW_WO;

    ////////////////////////////////
    // Methodes privees
    ////////////////////////////////
    t_Chaine      NomComplet(t_Chaine& nomFichier);
    t_Chaine      NomCourt(t_Chaine& nomFichier);
    t_Chaine      NomRoot(t_Chaine& nomFichier);
    void          DeplacerFichier(t_Chaine& nomFichier);
    void          SupprimerFichier(t_Chaine& nomFichier);
    t_CMN__Statut TimeIntervalFlagCheck(const t_Chaine nomTIF, 
                                        e_TimeIntervalFlag& nouveauTIF);

    ////////////////////////////////
    // Attributs prives
    ////////////////////////////////

    // Liste des Work Orders
    t_ListeWorkOrder   m_ListeWorkOrder;

    // TimeIntervalFlag en cours
    e_TimeIntervalFlag m_TimeIntervalFlag;

    // TimeIntervalFlag precedent
    e_TimeIntervalFlag m_PrecedentTIF;

    // Gestion des donnees
    t_Chaine           m_RepertoireRoot;
    t_Chaine           m_RepertoireOut;
    t_Chaine           m_RepertoireTmp;
    t_Chaine           m_RepInputAux;   /* CR 305*/  
};

#endif
