// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/WOM/src/WOM__ServeurWorkOrder.cc,v 1.3 2011/11/25 10:17:46 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : WOM
//@ CLASSE : WOM__ServeurWorkOrder
//@
//@ Commentaire :
//@    Ce fichier contient la classe WOM__ServeurWorkOrder, qui
//@    est le point d'entree du Work Order Manager
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-159-CN : 26/05/2005 : JL. PASCAL
//@ modif suite a portage linux
//@ fiche de correction correspondante : IA-2100-FC-159-THA
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------
//@ ------------------------------------------------------------------*/

//
// Fichiers include
//

// STL
#include <iostream>

// Standard
#include <sys/signal.h>

// Projet
#include "CMN__Version.h"
#include "CMN__Securite.h"
#include "CMN__OPSProcess.h"
#include "CFG__Configuration.h"
#include "ERR__Exception.h"
#include "MSG__Message.h"
#include "WOM__Message.h"
#include "CMN__PGEServices.h"
#include "CMN__OPSProcess.h"
#include "WOM__ServeurWorkOrder.h"


//
// Definition des constantes
//
const t_Chaine WOM__ServeurWorkOrder::MSG_SERVER_HOST = "MSG_SERVER_HOST";
const t_Chaine WOM__ServeurWorkOrder::MSG_SERVER_PORT = "MSG_SERVER_PORT";


//=====================================================================
//                        DONNEES STATIQUES
//=====================================================================

// Serveur de Work Order
static WOM__ServeurWorkOrder S_Serveur;

// Acces aux services PGE
static CMN__PGEServices      S_PGEServices(PROCESS_WOM);

// Handlers pour la reception d'un signal
static void WOM__Exit(int signal);
static void WOM__Terminer(int signal);

// Handler pour la reception d'une exception non traitee
static void WOM__GestionnaireException();


//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : WOM__ServeurWorkOrder
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
WOM__ServeurWorkOrder::WOM__ServeurWorkOrder()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~WOM__ServeurWorkOrder
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
WOM__ServeurWorkOrder::~WOM__ServeurWorkOrder()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Initialiser
//@
//@ Description : Permet d'initialiser les traitements du serveur de
//@               message
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__ServeurWorkOrder::Initialiser()
{
  //@ Lecture du parametrage (nom Host et numero de service)
  t_Chaine host = CFG__Configuration::GetAdresse(MSG_SERVER_HOST);
  t_Chaine service = CFG__Configuration::GetAdresse(MSG_SERVER_PORT);
  
  //@ Creer la connexion serveur en appelant le gestionnaire 
  //@ d'activite
  m_GestionnaireActivite.ConnecterServeur((char*)host.c_str(),
                                          (char*)service.c_str());
  
  //@ Initialiser le gestionnaire de Work Order
  m_GestionnaireActivite.InitialiserGestionnaireWO();
}


//@ ------------------------------------------------------------------*/
//@ Methode : Lancer
//@
//@ Description : Permet de lancer la boucle de scrutation des 
//@               evenements
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__ServeurWorkOrder::Lancer()
{
  //@ TANT QUE pas fin
  while (!m_GestionnaireActivite.Terminer()) 
    {
      //@ Scruter les evenements
      m_GestionnaireActivite.Ecouter();
    }
  //@ FIN TANT QUE
}


//@ ------------------------------------------------------------------*/
//@ Methode : Terminer
//@
//@ Description : Permet de terminer l'execution du serveur
//@
//@ Parametres :
//@               Aucun   
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__ServeurWorkOrder::Terminer(MP__ArretProcess::e_EtatFinProcessus
                                     etat)
{
  MP__ArretProcess msgArret;
  MSG__Message     message;


  //@ Construction d'un message de type ARRET PROCESSUS
  msgArret.SetProcessus(PROCESS_WOM);
  msgArret.SetEtatFin(etat);
  msgArret.Serialiser(message);

  //@ Envoi du message
  m_GestionnaireActivite.EnvoyerMessage(message);

  //@ SI la fin du process est normale
  if (etat == MP__ArretProcess::FIN_OK)
    {
      //@ Envoyer un message de type JDB MSG_WOM_STOP_OK
      m_GestionnaireActivite.EnvoyerMessageJDB(WOM__Message::
                                               MSG_WOM_STOP_OK);
    }
  //@ SINON le process s'arrete sur une erreur
  else
    {
      //@ Envoyer un message de type JDB MSG_WOM_STOP_ON_ERROR
      m_GestionnaireActivite.EnvoyerMessageJDB(WOM__Message::
                                               MSG_WOM_STOP_ON_ERROR);
    }
  //@ FIN SI la fin du process est normale
}


//@ ------------------------------------------------------------------*/
//@ Methode : main
//@
//@ Description : Programme principal du serveur
//@
//@ Parametres :
//@               (ENTREE) argc: Nombre d'arguments   
//@               (ENTREE) argv: Liste des arguments   
//@
//@ Valeur de retour :
//@               Code retour du process
//@
//@ ------------------------------------------------------------------*/
int main (int argc, char *argv[])
{
  t_CMN__PGEStatus status = CMN__STATUT_OK;
  

  //@ Installation d'un gestionnaire d'exception
  set_terminate(&WOM__GestionnaireException);
  set_unexpected(&WOM__GestionnaireException);

  //@ Traitement des options d'execution
  CMN__Version_Trace(argc, argv);
  
  //@ Declaration du demarrage au MCS
  status = S_PGEServices.MLAInitialisation();  

  //@ SI le status de l'initialisation du MCS est OK
  if (status == CMN__STATUT_OK) 
    {
      //@ Capture des exceptions
      try 
        {
          //@ Installation du module de gestion de la securite
          CMN__Securite_Init(WOM__Exit, WOM__Terminer);
          
          //@ Initialisation du serveur de Work Order
          S_Serveur.Initialiser();

#ifdef DEBUG
          cout << endl << "INFO: Serveur de Work Orders lance..." 
               << endl;
#endif
          
          //@ Lancer la boucle infinie de scrutation des evenements
          S_Serveur.Lancer();               
          
           //@ On est sorti de la boucle: on termine
          S_Serveur.Terminer(MP__ArretProcess::FIN_OK);

#ifdef DEBUG
          cout << endl << "INFO: Serveur de Work Orders arrete." 
               << endl;
#endif
           
          //@ On signale au MCS la fin du processus (fait exit)
          S_PGEServices.MLATerminate();  
        } 
           
      //@ Exploitation des exceptions   
      catch (ERR__Exception &exception) 
        {
          //@ Impression des exceptions
          cout << "Exception (WOM): " << endl;
          cout << "\tFile : " << exception.GetFile() << endl;
          cout << "\tLine : " << exception.GetLine() << endl;
          cout << "\tError: " << exception.GetErrorCode() << endl;
          cout << "\tErrno: " << exception.GetErrno() << endl;
          for (int i=0; i<exception.GetNbParametre(); i++)
            {
              cout << "\tInfo : " << exception.GetParametre(i) << endl;
            }
          
          //@ On essaye de terminer
          S_Serveur.Terminer(MP__ArretProcess::FIN_NOK);  

#ifdef DEBUG
          cout << endl 
               << "INFO: Serveur de Work Orders arrete sur exception." 
               << endl;
#endif
           
          //@ On signale au MCS la fin du processus (fait exit)
          S_PGEServices.MLATerminate();  
        }
    } 
  //@ FIN SI status ok 

  
  //@ Erreur initialisation MCS exit(-1)
  return EXIT_FAILURE;
}


//=====================================================================
//                        METHODES PRIVEES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : WOM__Exit
//@
//@ Description : Procedure notifiee sur le signal SIGTERM. Arrete le
//@               serveur de Work Order
//@
//@ Parametres :
//@               (ENTREE) signal: Numero du signal
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void WOM__Exit(int signal)
{
  //@ Arret du serveur de Work Order
  //FA159
  exit (EXIT_FAILURE);
}


//@ ------------------------------------------------------------------*/
//@ Methode : WOM__Terminer
//@
//@ Description :  Procedure notifiee sur un signal autre que SIGTERM.
//@               Arrete proprement le serveur de Work Order
//@
//@ Parametres :
//@               (ENTREE) signal: Numero du signal
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void WOM__Terminer(int signal)
{
  //@ Terminaison du serveur de Work Order
  S_Serveur.Terminer(MP__ArretProcess::FIN_NOK);
  
  //@ On signale au MCS la fin du processus (fait exit)
  S_PGEServices.MLATerminate();

  //@ Arret du serveur de Work Order
  //FA159
  exit (EXIT_FAILURE);
}


//@ ------------------------------------------------------------------*/
//@ Methode : WOM__GestionnaireException
//@
//@ Description : Handler pour la reception d'une exception non 
//@               traitee
//@
//@ Parametres :
//@               Aucune
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void WOM__GestionnaireException()
{
#ifdef DEBUG
  cerr << "WOM: Exception non geree ou illegale recue !" << endl;
#endif

  //@ Terminaison du serveur de Work Order
  S_Serveur.Terminer(MP__ArretProcess::FIN_NOK);
  
  //@ On signale au MCS la fin du processus (fait exit)
  S_PGEServices.MLATerminate();
//FA159
  exit (EXIT_FAILURE);
}
