#!/usr/bin/env python

"""Build the Metopizer Python interface only.
Command line tools should be built using autotools.
"""

import os
import sys
from distutils.core import setup, Extension

DEFAULT_METOPIZER_HOME = os.path.join(sys.prefix, 'share', 'metopizer')

if 'METOPIZER_HOME' in os.environ:
	DEFAULT_METOPIZER_HOME = os.environ['METOPIZER_HOME']

version = 'not found'
for line in open('configure.ac', 'r').readlines():
	if line.startswith('VERSION='):
		version = line.strip()[8:]

macros = [('VERSION', '\"'+version+'\"'),
		  ('PYMETOPIZER', '1'),
		  ('DEFAULT_METOPIZER_HOME', '\"'+DEFAULT_METOPIZER_HOME+'\"')]

if sys.byteorder == 'big':
	macros.append(('WORDS_BIGENDIAN', '1'))

extra_compile_args = []

#compiler = 'g++'  # this doesn't work

if sys.platform.startswith('aix'):
	macros.append(('_HAVE_STRFTIME', '1'))
	extra_compile_args.append('-qmaxmem=32786')

metopizer_module = Extension('metopizer',
							 #compiler=compiler,
							 define_macros=macros,
							 include_dirs=['/usr/local/include'],
							 #libraries = ['tcl83'],
							 library_dirs=['/usr/local/lib'],
							 extra_compile_args=extra_compile_args,
							 sources=['src/common.cpp',
									  'src/ccsds.cpp',
									  'src/eps.cpp',
									  'src/ccsdsreader.cpp',
									  'src/filereader.cpp',
									  'src/sband.cpp',
									  'src/pyamsuareader.cpp',
									  'src/pyavhrrreader.cpp',
									  'src/pyascatreader.cpp',
									  'src/pygrasreader.cpp',
									  'src/pyhirsreader.cpp',
									  'src/pyiasireader.cpp',
									  'src/pygomereader.cpp',
									  'src/pyhktmreader.cpp',
									  'src/pymhsreader.cpp',
									  'src/pyreader.cpp',
									  'src/pymetopizer.cpp',
									  'src/pysemreader.cpp'])

setup(name='Metopizer',
	  version=version,
	  description='CCSDS packet readers',
	  author='Mike Elson',
	  author_email='mike.elson@eumetsat.int',
	  url='http://tctrac/projects/metopizer',  # EUM internal only
	  ext_modules=[metopizer_module])
