////////////////////////////////////////////////////////////////////////////////
//
//	File: amsuabase.hpp
//
//	Provides: AMSUABase ABC
//
//	Purpose: Base class code to normalise and produce images from AMSU-A
//			 product samples
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef amsuabase_defined
#define amsuabase_defined

#include "image.hpp"

class AMSUABase
{
public:

	virtual ~AMSUABase(void)
		{
		}

	//
	// stretch_8
	//

	string stretch_8(unsigned int number)
		{
			stringstream str;
			for(int i=7; i>=0; i--)
			{
				str << ((number&(1<<i))==0?"0":"1") << "	";
			}
			return str.str();
		}

	//
	// output_digital_telemetry
	//

	void output_digital_telemetry(
		textoutstream_type& os,
		uint8* sync_sequence,
		uint8 serial_number,
		uint8* housekeeping)
		{
			os << "Sync Sequence  : "
			   << ntohs(sync_sequence[0])
			   << ntohs(sync_sequence[1])
			   << ntohs(sync_sequence[2])
			   << endl
			   << "Serial Number  : "
			   << static_cast<unsigned int>(serial_number) << endl
			   << "						|Cold Cal |Nadr|Cold|Warm|Full|\n"
			   << "					zero|  Pos	  |Mode|Cal |Cal |Scan|zero\n"
			   << "					fill|MSB  LSB |	   |Mode|Mode|Mode|fill\n"
			   << "Housekeeping 1 : "
			   << stretch_8(housekeeping[0]) << endl
			   << "								  |Surv|PLL |Scan|Scan|	   \n"
			   << "								  |Heat|pwr |A1-2|A1-1|zero\n"
			   << "					 zero fill	  |	   |Powr|	 |Powr|fill\n"
			   << "Housekeeping 2 : "
			   << stretch_8(housekeeping[1]) << endl
			   << endl
//	   << "					   (zero fill)\n"
			   << "Housekeeping 3 : " << ntobs(housekeeping[2],8) << endl
//	   << stretch_8(housekeeping[2]) << endl << endl
//	   << "					   (zero fille)\n"
			   << "Housekeeping 4 : " << ntobs(housekeeping[3],8) << endl
//	   << stretch_8(housekeeping[3]) << endl
			   << endl;
		}

	//
	// write_image
	//

	virtual void write_image(int number, Image* image)
		{
			spinner.reset(new Spinner(string("Assembling image ")+
									  ntos(number+1)));

			image->set_filename_suffix(string(".") + ntos(number+1));
			image->set_attributes(Image::IMAGE_MONO,
								  30,
								  amsua_scan_lines.size());
			image->begin_output();

			byte row[30];

			for(AMSUAScanLines::iterator yiter=amsua_scan_lines.begin();
				yiter!=amsua_scan_lines.end(); yiter++)
			{
				for(int x=0; x<30; x++)
				{
					double pixel = (static_cast<double>
									((*yiter).pixels[x].channels[number])-
									static_cast<double>(amsua_min[number])) /
						static_cast<double>(amsua_max[number]-
											amsua_min[number]) * 255.0;

					row[x] = static_cast<byte>(pixel);
				}
				image->write_line(row);
				spinner->tick();
			}

			image->end_output();
			spinner->end();
		}

	//
	// get_image_count
	//

	int get_image_count(void)
		{
			return 15;
		}

	//
	// AMSUAPixel
	//

	struct AMSUAPixel
	{
		uint16_be channels[15];
	};

	//
	// AMSUAScanLine

	struct AMSUAScanLine
	{
		AMSUAPixel pixels[30];
	};

protected:

	typedef deque<AMSUAScanLine> AMSUAScanLines;
	AMSUAScanLines amsua_scan_lines;

	uint16_be amsua_min[30];
	uint16_be amsua_max[30];

};

#endif
