////////////////////////////////////////////////////////////////////////////////
//
//	File: bufferreader.hpp
//
//	Provides: BufferReader filter
//
//	Purpose: Extracts buffers of a specified size from a plain binary stream
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef bufferreader_defined
#define bufferreader_defined

#include "bufferproducer.hpp"
#include "fileproducer.hpp"
#include "fileconsumer.hpp"

class BufferReader :
	public BufferProducer,
	public TextSummaryProducer,
	public FileProducer
{
public:

	BufferReader(FileProducer& _source, long _length)
		: source(_source)
		, length(_length)
		, byte_count(0)
		, packet_count(0)
		, is(NULL)
		{
		}

	virtual Buffer* get_next_buffer(void)
		{

			if (!spinner.get())
			{
				spinner.reset(new Spinner());
			}
			spinner->tick();


			if (!is)
			{
				DEBUG("BufferReader::get_next_buffer getting next file\n");
				is = source.get_next_file();
				if (!is)
				{
					DEBUG("BufferReader::get_next_buffer end of input"
						  << " stream\n");
					spinner->end();
					return NULL;
				}
			}

			byte* new_buffer = new byte[length];
			is->read(reinterpret_cast<char*>(new_buffer), length);

			int count = is->gcount();
			if (count==0)
			{
				// end of file
				delete[] new_buffer;
				spinner->end();
				return NULL;
			}
			if (length>count)
			{
				length = count;
			}

			DEBUG("BufferReader::get_next_buffer read " << count
				  << " bytes\n");
			byte_count += count;

			const bool TAKE_OWNERSHIP = true;
			buffer.reset(new Buffer(new_buffer, length, TAKE_OWNERSHIP));
			delete[] new_buffer;
			packet_count++;
			return buffer.get();
		}


	virtual void output_text_summary( textoutstream_type& os )
		{
			os << "BufferReader report:\n"
			   << "	   " << byte_count << " bytes read\n"
			   << "	   " << packet_count << " packets extracted\n";
			os << endl;
		}


	virtual bool produce_files(FileConsumer& dest)
		{
			while( Buffer* buff = get_next_buffer() )
			{
				packet_count++;
				ostream_type* os = dest.get_named_output_stream("buffer.out");
				byte_count -= os->tellp();
				os->write(reinterpret_cast<char*>(buff->get_data()), length);
				byte_count += os->tellp();
			}
			return true;
		}

	virtual istream_type* get_next_file(void)
		{
			return NULL;
		}

private:

	long length;
	FileProducer& source;

	int packet_count;
	uint64_be byte_count;

	auto_ptr<Buffer> buffer;
	istream_type* is;

};

#endif
