////////////////////////////////////////////////////////////////////////////////
//
//	File: cadu.hpp
//
//	Provides: CADU struct
//
//	Purpose: misc structs
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef cadu_defined
#define cadu_defined

#include "cvcdu.hpp"

const uint32_be MAGIC_NUMBER = 0x1ACFFC1D;

struct CADU
{
	enum { SIZE = 1024 };
	enum { PAYLOAD_SIZE = 1020 };

	byte sync_marker[4] PACKED;
	CVCDU payload PACKED;

	int get_sync_marker(void)
		{
			return static_cast<int>( (sync_marker[0]<<24)+(sync_marker[1]<<16)
									 +(sync_marker[2]<<8)+(sync_marker[0]) );
		}

	void set_sync_marker(int marker)
		{
			sync_marker[0] = static_cast<byte>((marker>>24)&255);
			sync_marker[1] = static_cast<byte>((marker>>16)&255);
			sync_marker[2] = static_cast<byte>((marker>>8)&255);
			sync_marker[3] = static_cast<byte>(marker&255);
		}

	byte* get_data()
		{
			return &(sync_marker[0]);
		}


	void set_byte(byte new_data, int offset)
		{
			byte* data = get_data();
			data[offset] = new_data;
		}

	bool set_bit(int bit, int offset)
		{
			bool result = false;
			byte* data = get_data();
			int byte_offset = offset/8;
			int bit_offset = offset - (byte_offset*8);

			if (bit_offset > 0 && bit_offset < 8)
			{
				int a = 0;
				int b = data[byte_offset];

				a |= ((b>>(8-bit_offset))&((1<<bit_offset)-1))<<(8-bit_offset);
				if ( ((b>>(7-bit_offset))&1) != (bit&1) )
				{
					result = true;
				}
				a |= (bit&1)<<(7-bit_offset);
				a |= b&((1<<(7-bit_offset))-1);
				data[byte_offset] = a;
			}
			else
			{
				int a = 0;
				int b = data[byte_offset];
				if ( ((b>>7)&1) != (bit&1) )
				{
					result = true;
				}
				a |= (bit&1)<<7;
				a |= b&((1<<7)-1);
				data[byte_offset] = a;
			}

			return result;
		}

	void flip_bit(int offset)
		{
			byte* data = get_data();
			int byte_offset = offset/8;
			int bit_offset = offset - (byte_offset*8);

			if (bit_offset > 0 && bit_offset < 8)
			{
				int a = 0;
				int b = data[byte_offset];

				a |= ((b>>(8-bit_offset))&((1<<bit_offset)-1))<<(8-bit_offset);
				if ( static_cast<int>((b>>(7-bit_offset))&1) == 0 )
				{
					a |= 1<<(7-bit_offset);
				}
				a |= b&((1<<(7-bit_offset))-1);
				data[byte_offset] = a;
			}
			else
			{
				int a = 0;
				int b = data[byte_offset];
				if ( static_cast<int>((b>>7)&1) == 0 )
				{
					a |= 1<<7;
				}
				a |= b&((1<<7)-1);
				data[byte_offset] = a;
			}
		}
};

#endif
