////////////////////////////////////////////////////////////////////////////////
//
//	File: caduextractor.cpp
//
//	Provides: CADUExtractor filter
//
//	Purpose: Write CADU packets to a FileConsumer
//
//	Copyright (c) 2006 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "caduextractor.hpp"
#include "fileconsumer.hpp"

CADUExtractor::CADUExtractor(CADUProducer* _source)
	: source(_source),
	  count(0)
{
}

void CADUExtractor::output_text_summary(ostream& os)
{
	os << "CADUExtractor report:\n"
	   << "	   " << count << " packets recieved\n"
	   << "	   1024 bytes per packet\n"
	   << endl;
}

bool CADUExtractor::produce_files(FileConsumer& consumer)
{
	ostream_type* os = consumer.get_output_stream();
	while(CADU* cadu = source->get_next_cadu())
	{
		os->write(reinterpret_cast<char*>(cadu), sizeof(CADU));
		count++;
	}
	return true;
}

istream_type* CADUExtractor::get_next_file(void)
{
	return NULL;
}
