////////////////////////////////////////////////////////////////////////////////
//
//	File: cadugenerator.cpp
//
//	Provides: CADUGenerator filter
//
//	Purpose: Synthesize a stream of Fill CADU packets
//
//	Copyright (c) 2006 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "cadugenerator.hpp"
#include "cvcdu.hpp"
#include "vcdu.hpp"
#include "rs.hpp"

CADUGenerator::CADUGenerator(int _count, int _initial_vccount, int _mod_vccount, int _scid)
	: count(_count),
	  initial_vccount(_initial_vccount),
	  mod_vccount(_mod_vccount),
	  scid(_scid),
	  remaining(_count),
	  vccount(_initial_vccount)
{
}

void CADUGenerator::output_text_summary(ostream& os)
{
	os << "CADUGenerator report:\n"
	   << "	   SCID				: " << scid << endl
	   << "	   Count			: " << count << endl
	   << "	   Initial VC count : " << initial_vccount << endl
	   << "	   Mod VC count		: " << mod_vccount << endl
	   << "	   Final VC count	: " << vccount << endl
	   << endl;
}

CADU* CADUGenerator::get_next_cadu(void)
{
	if (remaining == 0)
	{
		return NULL;
	}

	memcpy(&cadu.sync_marker, &MAGIC_NUMBER, 4);
	bzero(&cadu.payload, sizeof(CVCDU));

	CVCDU& cvcdu = cadu.payload;

	VCDU& vcdu = cvcdu.payload;
	vcdu.set_version_number(1);
	vcdu.set_scid(scid);
	vcdu.set_vcid(VCID_FILL);
	vcdu.set_vccount(vccount);

	vccount += mod_vccount;
	while (vccount<0)
	{
		vccount += MAX_VCCOUNT;
	}
	while(vccount>=MAX_VCCOUNT)
	{
		vccount -= MAX_VCCOUNT;
	}

	byte buffer[2550];
	for(int i=0; i<4; i++)
	{
		for(int j=0; j<(255-32); j++)
		{
			buffer[j] = reinterpret_cast<byte*>(&cvcdu.payload)[j*4+i];
		}
		rs_encode(buffer, buffer+255-32, 0);
		for(int j=0; j<32; j++)
		{
			cvcdu.checksum[j*4+i] = buffer[255-32+j];
		}
	}

	//for (int b=0; b<2000; b++) cout << b << " " << (unsigned int)a[b] << endl;

	randomizer.randomize(&cvcdu);

	remaining--;

	return &cadu;
}
