////////////////////////////////////////////////////////////////////////////////
//
//	File: cadurandomizer.hpp
//
//	Provides: CADU randomizer/derandomizer
//
//	Purpose:
//
//	Copyright (c) 2006 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "cadurandomizer.hpp"

CADURandomizer::CADURandomizer(void)
{
	// initialise decode block

	// first zero fill
	memset(decode, 0, CADU::PAYLOAD_SIZE);

	// prepare registers
	int regs[8];
	for(int i=0; i<8; i++)
	{
		regs[i] = 1;
	}

	// iterate through 1020*8 bits
	for(int i=0; i<(CADU::PAYLOAD_SIZE * 8); i++)
	{
		// copy register 0 into next bit of decode array
		if (regs[0] == 1)
		{
			decode[i/8] |= (1<<( (7-(i%8)) ));
		}

		// calculate new top bit
		int new_reg_7 = ((regs[0] + regs[3] + regs[5]+ regs[7]) % 2);

		// shift registers
		for(int r=0; r<=6; r++)
		{
			regs[r] = regs[r+1];
		}

		// set new top bit
		regs[7] = new_reg_7;
	}
}

void CADURandomizer::randomize(CVCDU* cvcdu)
{
	for(int i=0; i<sizeof(CVCDU); i++)
	{
		byte* ptr = reinterpret_cast<byte*>(cvcdu);
		ptr += i;
		(*ptr) ^= decode[i];

		// why doesn't this work?
//		(reinterpret_cast<byte*>(cvcdu))[i] ^= decode[i];
	}

}
