////////////////////////////////////////////////////////////////////////////////
//
//	File: cadureader.hpp
//
//	Provides: CADUReader filter
//
//	Purpose: Extract a stream of CADU packets from a raw binary input stream
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef cadureader_defined
#define cadureader_defined

#include "cadu.hpp"
#include "fileproducer.hpp"
#include "caduproducer.hpp"

class CADUReader :
	public CADUProducer
{
public:

	enum { WARNING_GAP_SIZE = 4*1024*1024 };

	CADUReader( FileProducer& _source);
	void output_text_summary( ostream& os = cout);
	CADU* get_next_cadu(void);

	// look for CADU packet in the buffer
	CADU* scan_buffer(void);

	// grab more input data and place into the buffer
	bool fill_buffer(void);

private:

	enum { BUFFER_SIZE = 128*1024 };
	byte buffer[BUFFER_SIZE];

	FileProducer& source;

	unsigned long long buffer_pos; // current read pos
	unsigned long long buffer_size; // buffer is declared statically but here we track the
	// actual buffer usage

	istream_type* in_stream;

	int good_packets; // count of proper CADUs found
	unsigned long long skipped_bytes; // total bytes of input stream that were not used
	unsigned long long gap_size; // inside an unrecognised part of the input, size of
	// current bad area
	unsigned long long file_pos;

	typedef map<long long,int> Gaps;
	Gaps gaps;

	long long initial_gap;
	long long final_gap;
};

#endif
