////////////////////////////////////////////////////////////////////////////////
//
//	File: cadusplitter.hpp
//
//	Provides: CADUSplitter class
//
//	Purpose:
//
//	Copyright (c) 2006 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef cadusplitter_defined
#define cadusplitter_defined

#include "cadu.hpp"
#include "caduproducer.hpp"
#include "fileproducer.hpp"
#include "vcduproducer.hpp"
#include "ccsds.hpp"

class CADUSplitter :
	public FileProducer,
	public CADUProducer
{
public:

	CADUSplitter(CADUProducer* _source, long long dur, long long overlap, bool shift_gras_obt, bool apid1_only);
	~CADUSplitter(void);
	void output_text_summary( ostream& os = cout );
	istream_type* get_next_file(void);
	bool produce_files( FileConsumer& writer );
	CADU* get_next_cadu(void);

private:

	void set_dump_boundaries(const OBTTime& start);
	void store_cadu(CADU* cadu);
	void flush_overlap(void);
	void new_file(void);
	void finalise_file(void);

	CADUProducer* source;
	auto_ptr<VCDUProducer> vcdu_chain;
	CADU* current_cadu;
	long long overlap;
	long long duration;
	auto_ptr<OBTTime> initial_obt;
	OBTTime end_dump;
	OBTTime begin_overlap;
	typedef queue<CADU> CADUQueue;
	CADUQueue overlap_queue;

	FileConsumer* writer;
	int file_counter;
	ostream_type* current_file;

	struct FileInfo
	{
		bool fresh;
		OBTTime initial_obt;
		OBTTime final_obt;
		int rollovers;
		int cadu_counter;
	};

	FileInfo current_fileinfo;
	typedef map<string, FileInfo> FileInfos;
	FileInfos fileinfos;

	int cadu_counter;
	bool shift_gras_obt;
	bool apid1_only;
};

#endif
