////////////////////////////////////////////////////////////////////////////////
//
//	File: cadutocvcdu.hpp
//
//	Provides: CADUToCVCDU filter
//
//	Purpose: Convert a stream of CADU packet to a CVCDU packet stream
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef cadu_to_cvcdu_defined
#define cadu_to_cvcdu_defined

#include "interface.hpp"
#include "caduproducer.hpp"
#include "cvcduproducer.hpp"
#include "cadurandomizer.hpp"

class CADUToCVCDU :
	virtual public Interface,
	public CVCDUProducer
{
public:

	//
	// constructor
	//

	CADUToCVCDU( CADUProducer* _source )
		: source(_source),
		  packet_count(0)
		{
		}

	//
	// output_text_summary
	//

	virtual void output_text_summary( ostream& os = cout)
		{
			os << "CADUToCVCDU report:\n	" << packet_count
			   << " packets processed\n\n";
		}

	//
	// get_next_cvcdu
	//

	virtual CVCDU* get_next_cvcdu(void)
		{
			// the CADUReader has already extracted individual packets and
			// checked their sync marker is ok.

			// we just need to grab the next packet ...
			CADU* cadu = source->get_next_cadu();
			if (!cadu)
			{
				return NULL;
			}

			// ... and decode the payload
			randomizer.randomize(&(cadu->payload));

			packet_count++;
			return &cadu->payload;
		}

private:

	CADUProducer* source;
	CADURandomizer randomizer;
	int packet_count;

};

#endif
