////////////////////////////////////////////////////////////////////////////////
//
//	File: caduwriter.cpp
//
//	Provides: CADUWriter class
//
//	Purpose:
//
//	Copyright (c) 2005 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "caduwriter.hpp"

const int PADDING_SIZE = 4;

CADUWriter::CADUWriter(CADUProducer& source, const string& _filename, bool _pad)
	: filename(_filename),
	  counter(0),
	  pad(_pad),
	  bytes_written(0)
{
	ofs.reset(new ofstream64(filename.c_str()));

	if (!spinner.get())
	{
		spinner.reset(new Spinner());
	}

	byte zeros[PADDING_SIZE];
	bzero(zeros, PADDING_SIZE);

	CADU* cadu;
	while((cadu=source.get_next_cadu())!=NULL)
	{
		ofs->write(reinterpret_cast<char*>(cadu), CADU::SIZE);
		bytes_written+=CADU::SIZE;
		if (pad)
		{
			ofs->write(reinterpret_cast<char*>(zeros), PADDING_SIZE);
			bytes_written+=PADDING_SIZE;
		}
		counter++;
		spinner->tick();
	}

	spinner->end();
}

void CADUWriter::output_text_summary( textoutstream_type& os )
{
	os << "CADUWriter report:\n"
	   << "	   Output filename " << filename << endl
	   << "	   Packets written " << counter << endl
	   << "	   Bytes written " << bytes_written << endl;

	if (pad)
	{
		os << "	   Padding of 4 bytes per packet selected\n";
	}
}
