////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsds_config.hpp
//
//	Provides: Misc code specific to reading CCSDS configuration files used in
//			  creating EPS-L0 products
//
//	Purpose: Parse and store configuration file settings
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ccsds_config_defined
#define ccsds_config_defined

#include "common.hpp"

const string CCSDS_COMMENT_CHAR = "#";


class ccsds_config
{
public:
	ccsds_config()
		{
		}


	bool read(const string& filename)
		{
			DEBUG("ccsds config reading " << filename << endl);
			ifstream ifs(filename.c_str());
			if (!ifs)
			{
				cout << "Error: CCSDS-APID Configuration file '" << filename << "' could "
					 << "not be opened. Please ensure that MetOpizer is installed correctly. "
					 << "This error can also be overcome by setting the environment variable "
					 << "METOPIZER_HOME or using the ccsds_to_l0 --config-apid option. "
					 << "See ccsds_to_l0 --help for more information\n\n";
				return false;
			}
			else
			{
				while(ifs)
				{
					char line[256];
					ifs.getline(line, 256);
					string s = line;

					//cout << "LINE: '" << s << "'\n";

					// remove comments
					string::size_type c = s.find(CCSDS_COMMENT_CHAR);
					if (c != string::npos)
					{
						s.erase(c);
					}

					//cout << "nocomment '" << s << "'\n";


					string::size_type kc;
					string::size_type vc;
					int vc_comma = -1;
					int vc_dash = -1;
					int vc_temp = -1;

					// only examine non empty lines
					if (!s.empty())
					{
						c = s.find(":");
						if (c != string::npos)
						{
							string keyword = s.substr(0,c);
							string value = s.substr(c+1);
							//string value_array[15];

							// remove whitespace
							while( (kc=keyword.find(" ")) != string::npos)
							{
								keyword.erase(kc,1);
							}

							//cout << "keyword		= '" << keyword << "'\n";
							//cout << "value before = '" << value << "'\n";

							//int ctr = 0;
							keyword = tolower(keyword);

							if ( (vc=value.find("-")) != string::npos)
							{
								vc_dash = vc;
							}
							if ( (vc=value.find(",")) != string::npos)
							{
								vc_comma = vc;
							}

							if (vc_dash < vc_comma)
							{
								int ctr = 0;
								while( (vc=value.find("-")) != string::npos)
								{
									if (ctr>0)
									{
										vc = value.find(",");
										for (int k=0; k<=vc; k++)
										{
											if (vc != string::npos)
											{
												value.erase(0,1);
											}
										}
										vc = value.find("-");
									}

									int min = 0;
									int max = 0;

									//cout << "vc = " << vc << endl;
									if (vc != string::npos)
									{
										value.erase(vc,1);
									}

									if (vc > 0)
									{
										min = atoi( (value.substr(0, vc)).c_str() );

										for (int k=0; k<vc; k++)
										{
											value.erase(0,1);
										}
										max = atoi(value.c_str());
									}

									for (int i=min; i<=max; i++)
									{
										apid_map[keyword].push_back(i);
									}
									ctr++;
								}

								while( (vc=value.find(",")) != string::npos)
								{
									//cout << "vc = " << vc << endl;

									if (ctr > 0)
									{
										for (int k=0; k<vc; k++)
										{
											if (vc != string::npos)
											{
												value.erase(0,1);
											}
										}
										vc = value.find(",");
										ctr = 0;
									}

									if (vc != string::npos)
									{
										value.erase(vc,1);
									}

									if (vc > 0)
									{
										apid_map[keyword].push_back(atoi( (value.substr(0, vc)).c_str() ) );

										for (int k=0; k<vc; k++)
										{
											value.erase(0,1);
										}
									}
								}

								if (ctr == 0)
								{
									apid_map[keyword].push_back(atoi(value.c_str()));
								}

							}
							else
							{

								bool DASH_FLAG = true;
								while( (vc=value.find(",")) != string::npos	 && DASH_FLAG)
								{
									//cout << "vc = " << vc << endl;
									vc_temp = value.find("-");
									if (vc_temp < vc)
									{
										DASH_FLAG = false;
									}
									else
									{
										value.erase(vc,1);
									}

									if (vc > 0 && DASH_FLAG)
									{
										apid_map[keyword].push_back(atoi( (value.substr(0, vc)).c_str() ) );

										for (int k=0; k<vc; k++)
										{
											value.erase(0,1);
										}
									}
								}


								int ctr = 0;
								while( (vc=value.find("-")) != string::npos)
								{
									if (ctr>0)
									{
										vc = value.find(",");
										for (int k=0; k<=vc; k++)
										{
											if (vc != string::npos)
											{
												value.erase(0,1);
											}
										}
										vc = value.find("-");
									}

									int min = 0;
									int max = 0;

									//cout << "vc = " << vc << endl;
									if (vc != string::npos)
									{
										value.erase(vc,1);
									}

									if (vc > 0)
									{
										min = atoi( (value.substr(0, vc)).c_str() );

										for (int k=0; k<vc; k++)
										{
											value.erase(0,1);
										}
										max = atoi(value.c_str());
									}

									for (int i=min; i<=max; i++)
									{
										apid_map[keyword].push_back(i);
									}
									ctr++;
								}


								if (ctr == 0)
								{
									apid_map[keyword].push_back(atoi(value.c_str()));
								}

							}
						}
						else
						{
							error << "Unrecognised line in config file "
								  << filename
								  << " '" << line << "' not understood\n";
						}
					}
				}

			}

			return true;

		}


	string apid_to_instrument(int apid)
		{
			string instrument;
			bool FLAG = true;

			APIDMAP::iterator iter=apid_map.begin();
			while (iter!=apid_map.end() && FLAG)
			{
				deque<int>::iterator iter2=find( ((*iter).second).begin(), ((*iter).second).end(), apid);

				if (iter2 != ((*iter).second).end())
				{
					//cout << (*iter).first << endl;
					instrument = (*iter).first;
					FLAG = false;
				}
				iter++;
			}

			return instrument;
		}


	void ccsds_config_summary(textoutstream_type& os)
		{
			os << "CCSDS APID Configuration:\n"
			   << "-------------------------\n";

			for(APIDMAP::iterator iter=apid_map.begin();
				iter!=apid_map.end(); iter++)
			{
				os << (*iter).first << ": ";

				for (deque<int>::iterator iter2=((*iter).second).begin();
					 iter2!=((*iter).second).end(); iter2++)
				{
					os << " " << *iter2;
				}
				os << endl;
			}
			os << endl;
		}


private:
	typedef deque<int> apid_list;
	typedef map<string, apid_list> APIDMAP;
	APIDMAP apid_map;

};


#endif
