////////////////////////////////////////////////////////////////////////////////
//
//	File: CCSDSCorruptFilter.hpp
//
//	Provides: CCSDSCorruptFilter filter
//
//	Purpose:  Enables the corruption of CCSDS packet common fields
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ccsdscorruptfilter_defined
#define ccsdscorruptfilter_defined

#include "ccsds.hpp"
#include "ccsdsproducer.hpp"
#include "ccsdsapidfilter.hpp"


#define MAX_DAY 65535
#define MAX_MILI_SEC 86399999
#define MAX_MICRO_SEC 999


class CCSDSCorruptFilter :
	public CCSDSProducer,
	public FileProducer
{
public:

	//
	// constructor
	//

	CCSDSCorruptFilter(CCSDSProducer& _source, int _skip, int _count, StringList& _selected_apid,
					   short_cds_time start_time, short_cds_time end_time, int _start_time_relative,
					   int _end_time_relative, int _sequence_counter_start, int _sequence_counter_end,
					   string _version_number_opt, string _type_opt,
					   string _secondary_header_flag_opt, string _apid_opt,
					   string _sequence_flag_opt, string _sequence_counter_opt, string _packet_length_opt,
					   string _utc_days_opt, string _utc_milliseconds_opt, string _utc_microseconds_opt,
					   string _obt_coarse_opt, string _obt_fine_opt, string _pec_opt, int _version_number_val,
					   int _type_val, int _secondary_header_flag_val, int _apid_val, int _sequence_flag_val,
					   int _sequence_counter_val, int _packet_length_val, int _utc_days_val,
					   int _utc_milliseconds_val, int _utc_microseconds_val, int _obt_coarse_val,
					   int _obt_fine_val, int _pec_val, string _byte_opt, int _byte_val1, int _byte_val2,
					   int _byte_offset_start, int _byte_offset_end, string _byte_increment, bool collapse_flag)
	: source(&_source),
		skip_val(_skip),
		count_val(_count),
		selected_apid(_selected_apid),
		sequence_counter_start(_sequence_counter_start),
		sequence_counter_end(_sequence_counter_end),
		start_time_relative(_start_time_relative),
		end_time_relative(_end_time_relative),
		version_number_opt(_version_number_opt),
		type_opt(_type_opt),
		secondary_header_flag_opt(_secondary_header_flag_opt),
		apid_opt(_apid_opt),
		sequence_flag_opt(_sequence_flag_opt),
		sequence_counter_opt(_sequence_counter_opt),
		packet_length_opt(_packet_length_opt),
		utc_days_opt(_utc_days_opt),
		utc_milliseconds_opt(_utc_milliseconds_opt),
		utc_microseconds_opt(_utc_microseconds_opt),
		obt_coarse_opt(_obt_coarse_opt),
		obt_fine_opt(_obt_fine_opt),
		pec_opt(_pec_opt),
		version_number_val(_version_number_val),
		type_val(_type_val),
		secondary_header_flag_val(_secondary_header_flag_val),
		apid_val(_apid_val),
		sequence_flag_val(_sequence_flag_val),
		sequence_counter_val(_sequence_counter_val),
		packet_length_val(_packet_length_val),
		utc_days_val(_utc_days_val),
		utc_milliseconds_val(_utc_milliseconds_val),
		utc_microseconds_val(_utc_microseconds_val),
		obt_coarse_val(_obt_coarse_val),
		obt_fine_val(_obt_fine_val),
		pec_val(_pec_val),
		byte_opt(_byte_opt),
		byte_val1(_byte_val1),
		byte_val2(_byte_val2),
		byte_offset_start(_byte_offset_start),
		byte_offset_end(_byte_offset_end),
		byte_increment(_byte_increment),
		length_flag(collapse_flag)
		{


			// seed the pseudo random number generator with the current time
			srand(static_cast<unsigned>(time(0)));

			valid = true;
			micro_sec_start = -1.0;
			micro_sec_end = -1.0;
			packet_count = 0;
			corrupt_count = 0;
			copied_count = 0;
			apid_match_ctr = 0;
			counter_match_ctr = 0;
			time_match_ctr = 0;
			affected_ctr = 0;

			version_number_tmp = 0;
			version_number_ctr = 0;
			type_tmp = 0;
			type_ctr = 0;
			secondary_header_flag_tmp = 0;
			secondary_header_flag_ctr = 0;
			apid_tmp = 0;
			apid_ctr = 0;
			sequence_flag_tmp = 0;
			sequence_flag_ctr = 0;
			sequence_counter_tmp = 0;
			sequence_counter_ctr = 0;
			packet_length_tmp = 0;
			packet_length_ctr = 0;
			utc_days_tmp = 0;
			utc_days_ctr = 0;
			utc_milliseconds_tmp = 0;
			utc_milliseconds_ctr = 0;
			utc_microseconds_tmp = 0;
			utc_microseconds_ctr = 0;
			obt_coarse_tmp = 0;
			obt_coarse_ctr = 0;
			obt_fine_tmp = 0;
			obt_fine_ctr = 0;
			pec_tmp = 0;
			pec_ctr = 0;
			byte_tmp = 0;
			byte_ctr = 0;


			// Error handling of input parameters


			apid_filter = new CCSDSAPIDFilter(selected_apid);

			if (!apid_filter->is_valid())
			{
				valid = false;
			}


			if (sequence_counter_start == -1)
			{
				sequence_counter_start = 0;
			}
			else if (sequence_counter_start < 0 || sequence_counter_start > 16383)
			{
				error << "\nThe value of the input field for-sequence-counter is out of range (0-16383).\n";
				valid = false;
			}

			if (sequence_counter_end == -1)
			{
				sequence_counter_end = 16383;
			}
			else if (sequence_counter_end < 0 || sequence_counter_end > 16383)
			{
				error << "\nThe value of the input field for-sequence-counter is out of range (0-16383).\n";
				valid = false;
			}

			if (sequence_counter_start > sequence_counter_end)
			{
				swap(sequence_counter_start, sequence_counter_end);
			}


			//short_cds_time start_time, short_cds_time end_time,
			//int start_time_relative, int end_time_relative,
			if (start_time == short_cds_time() && start_time_relative == -1)
			{
				micro_sec_start = 0;
			}
			else
			{
				if (start_time == short_cds_time())
				{
					//use start_time_relative combined with first packet time
					//see get_next_ccsds()
				}
				else
				{
					micro_sec_start = (	  (start_time.get_days()*24.0*60.0*60.0*1.0e6)
										  + (start_time.get_milliseconds()*1.0e3) );
					//+ start_time.get_microseconds() );
				}
			}


			//short_cds_time start_time, short_cds_time end_time,
			//int start_time_relative, int end_time_relative,
			if (end_time == short_cds_time() && end_time_relative == -1)
			{
				micro_sec_end = ( (65535*24.0*60.0*60.0*1.0e6) + (999*1.0e3) + 999 );
			}
			else
			{
				if (end_time == short_cds_time())
				{
					//use end_time_relative combined with first packet time and start_time_relative
					//see get_next_ccsds()
				}
				else
				{
					micro_sec_end = (	(end_time.get_days()*24.0*60.0*60.0*1.0e6)
										+ (end_time.get_milliseconds()*1.0e3) );
					//+ end_time.get_microseconds() );
				}
			}



			if (test_input(version_number_val, version_number_opt, 0, 7, "version-number"))
			{
				version_number_val = 0;
				version_number_opt = "no";
			}

			if (test_input(type_val, type_opt, 0, 1, "type"))
			{
				type_val = 0;
				type_opt = "no";
			}

			if (test_input(secondary_header_flag_val, secondary_header_flag_opt,
						   0, 1, "secondary_header_flag"))
			{
				secondary_header_flag_val = 0;
				secondary_header_flag_opt = "no";
			}

			if (test_input(apid_val, apid_opt, 0, 2047, "apid"))
			{
				apid_val = 0;
				apid_opt = "no";
			}

			if (test_input(sequence_flag_val, sequence_flag_opt, 0, 3, "sequence_flag"))
			{
				sequence_flag_val = 0;
				sequence_flag_opt = "no";
			}

			if (test_input(sequence_counter_val, sequence_counter_opt,
						   0, 16383, "sequence_counter"))
			{
				sequence_counter_val = 0;
				sequence_counter_opt = "no";
			}

			if (test_input(packet_length_val, packet_length_opt,
						   0, 65535, "packet_length"))
			{
				packet_length_val = 0;
				packet_length_opt = "no";
			}

			if (test_input(utc_days_val, utc_days_opt, 0, MAX_DAY, "utc_days"))
			{
				utc_days_val = 0;
				utc_days_opt = "no";
			}

			if (test_input(utc_milliseconds_val, utc_milliseconds_opt,
						   0, MAX_MILI_SEC, "utc_milliseconds"))
			{
				utc_milliseconds_val = 0;
				utc_milliseconds_opt = "no";
			}

			if (test_input(utc_microseconds_val, utc_microseconds_opt,
						   0, MAX_MICRO_SEC, "utc_microseconds"))
			{
				utc_microseconds_val = 0;
				utc_microseconds_opt = "no";
			}

			if (test_input(obt_coarse_val, obt_coarse_opt, 0, 16777215, "obt_coarse"))
			{
				obt_coarse_val = 0;
				obt_coarse_opt = "no";
			}

			if (test_input(obt_fine_val, obt_fine_opt, 0, 65535, "obt_fine"))
			{
				obt_fine_val = 0;
				obt_fine_opt = "no";
			}

			if (test_input(pec_val, pec_opt, 0, 65535, "pec"))
			{
				pec_val = 0;
				pec_opt = "no";
			}


		}



	//
	// test_input
	//

	bool test_input (int val, string opt, int min, int max, string field)
		{
			if (val == -1 && opt == "")
			{
				return true;
			}
			else
			{
				if ((val < min) || (val > max))
				{
					if (opt != "random" && opt != "off")
					{
						error << "\nThe value of the input field set-" << field
							  << " is out of range (" << min << "-" << max << ").\n";
						valid = false;
						return true;
					}
				}
				return false;
			}
		}



	//
	// combine_val_opt
	//

	string combine_val_opt (string opt, int val, int val2 = 0)
		{
			if (opt == "random" || opt == "off" || opt == "no")
			{
				return opt;
			}
			else if (opt == "yes" && val2 == 0)
			{
				return lexical_cast<string>(val);
			}
			else
			{
				if (val2 != 0)
				{
					return (lexical_cast<string>(val) + ", " + lexical_cast<string>(val2));
				}
				else
				{
					return (lexical_cast<string>(val) + ", " + opt);
				}
			}
		}



	//
	// random_range
	//

	int random_range(int lowest_number, int highest_number)
		{
			if (lowest_number > highest_number){
				swap(lowest_number, highest_number);
			}

			double range = highest_number - lowest_number + 1;
			return lowest_number + int(range * rand()/(RAND_MAX + 1.0));
		}


	//
	// output_text_summary
	//

	virtual void output_text_summary( ostream& os = cout )
		{

			double days =  (micro_sec_start / (24.0*60.0*60.0*1.0e6));
			unsigned int utc_days_first = (unsigned int)days;
			double milli_sec = (days - utc_days_first) * (24.0*60.0*60.0*1.0e3);
			unsigned int utc_milliseconds_first = (unsigned int)milli_sec;
			//unsigned int utc_microseconds_first = (unsigned int)((milli_sec - utc_milliseconds_first) * 1e3);

			days =	(micro_sec_end / (24.0*60.0*60.0*1.0e6));
			unsigned int utc_days_second = (unsigned int)days;
			milli_sec = (days - utc_days_second) * (24.0*60.0*60.0*1.0e3);
			unsigned int utc_milliseconds_second = (unsigned int)milli_sec;
			//unsigned int utc_microseconds_second = (unsigned int)((milli_sec - utc_milliseconds_second) * 1e3);

			if (selected_apid.empty())
			{
				selected_apid.push_back("ALL");
			}

			string firstTimeStr;
			string secondTimeStr;
			if (utc_days_first > 13880)
			{
				firstTimeStr = "Time has exceeded 1st January 2038";
			}
			else
			{
				firstTimeStr = TimeConv::utc_days_milliseconds_to_string(utc_days_first, utc_milliseconds_first);
			}

			if (utc_days_second > 13880)
			{
				secondTimeStr = "Time has exceeded 1st January 2038";
			}
			else
			{
				secondTimeStr = TimeConv::utc_days_milliseconds_to_string(utc_days_second, utc_milliseconds_second);
			}


			if (count_val == -1)
			{
				count_val = packet_count - skip_val;
			}
			if (count_val > packet_count)
			{
				count_val = packet_count;
			}
			if (skip_val >= packet_count)
			{
				skip_val = packet_count;
				count_val = 0 ;
			}
			if ((skip_val + count_val) > packet_count)
			{
				count_val = packet_count - skip_val;
			}

			string tmpOffset;
			if (byte_offset_start != -1 && byte_offset_end != -1)
			{
				tmpOffset = (lexical_cast<string>(byte_offset_start)
							 + ", " + lexical_cast<string>(byte_offset_end));
			}
			else if (byte_offset_start != -1)
			{
				tmpOffset = lexical_cast<string>(byte_offset_start);
			}
			else
			{
				tmpOffset = "none";
			}

			string tmpPEC = combine_val_opt(pec_opt, pec_val);
			if (tmpPEC == "no")
			{
				tmpPEC = "recalculate";
			}

			os << "CCSDS_Corrupt_Filter Report:\n"
			   << "	   " << "Corruption Statistics\n"
			   << "		   " << packet_count << " packets read in total\n"
			   << "		   " << copied_count << " packets copied\n"
			   << "		   " << corrupt_count << " packets corrupted\n"
			   << "		   " << corruption_ctr << " corruptions applied to each packet\n"
			   << "		   " << skip_val << " packets skipped\n"
			   << "		   " << count_val << " packets counted\n"
			   << "		   " << apid_match_ctr << " packets matching selected APID\n"
			   << "		   " << counter_match_ctr << " packets matching selected APID and Sequence Counter constraints\n"
			   << "		   " << time_match_ctr << " packets matching selected APID, Sequence Counter and UTC-time constraints\n"
			   << "	   " << "Corruption Constraints\n"
			   << "		   " << "Selected APID			 : " << selected_apid << endl
			   << "		   " << "Sequence Counter Start	 : " << sequence_counter_start << endl
			   << "		   " << "Sequence Counter End	 : " << sequence_counter_end << endl
			   << "		   " << "Time Start				 : " << firstTimeStr << endl
			   << "		   " << "UTC-Days Start			 : " << utc_days_first	<< endl
			   << "		   " << "UTC-Milliseconds Start	 : " << utc_milliseconds_first	<< endl
			   << "		   " << "Time End				 : " << secondTimeStr << endl
			   << "		   " << "UTC-Days End			 : " << utc_days_second	 << endl
			   << "		   " << "UTC-Milliseconds End	 : " << utc_milliseconds_second	 << endl
			   << "		   " << "Selected Byte Offset	 : " << tmpOffset << endl
			   << "		   " << "Byte Increment Mode	 : " << byte_increment << endl
			   << "	   " << "Corruptions Applied\n"
			   << "		   " << "set-version-number		 : " << combine_val_opt(version_number_opt, version_number_val) << endl
			   << "		   " << "set-type				 : " << combine_val_opt(type_opt, type_val) << endl
			   << "		   " << "set-secondary-header	 : " << combine_val_opt(secondary_header_flag_opt, secondary_header_flag_val) << endl
			   << "		   " << "set-apid				 : " << combine_val_opt(apid_opt, apid_val) << endl
			   << "		   " << "set-sequence-flag		 : " << combine_val_opt(sequence_flag_opt, sequence_flag_val) << endl
			   << "		   " << "set-sequence-counter	 : " << combine_val_opt(sequence_counter_opt, sequence_counter_val) << endl
			   << "		   " << "set-packet-length		 : " << combine_val_opt(packet_length_opt, packet_length_val) << endl
			   << "		   " << "set-utc-day			 : " << combine_val_opt(utc_days_opt, utc_days_val) << endl
			   << "		   " << "set-utc-millisecond	 : " << combine_val_opt(utc_milliseconds_opt, utc_milliseconds_val) << endl
			   << "		   " << "set-utc-microsecond	 : " << combine_val_opt(utc_microseconds_opt, utc_microseconds_val) << endl
			   << "		   " << "set-obt-coarse			 : " << combine_val_opt(obt_coarse_opt, obt_coarse_val) << endl
			   << "		   " << "set-obt-fine			 : " << combine_val_opt(obt_fine_opt, obt_fine_val) << endl
			   << "		   " << "set-pec				 : " << tmpPEC << endl
			   << "		   " << "set-byte				 : " << combine_val_opt(byte_opt, byte_val1, byte_val2) << endl
			   << "	   " << "Packets Affected\n";

			if (affected_ctr == 0)
			{
				os << "		   " << "None" << endl;
			}
			else
			{
				for (int i=0; i<affected_ctr; i++)
				{
					os << "		   " << "APID					 : " << affected_apids[i] << endl
					   << "		   " << "Time Start				 : " << affected_times_start[i] << endl
					   << "		   " << "Time End				 : " << affected_times_end[i] << endl;
				}
			}
			os << endl;

		}




	//
	// increment
	//

	int increment(int param, string param_name, int start, string stepStr, int max_val, int ctr)
		{
			int result = 0;
			int step = lexical_cast<int>(stepStr);

			if ( (step <= max_val) && (step >= (-1.0 * max_val)) )
			{
				if (ctr == 0)
				{
					result = start;
				}
				else
				{
					result = param + step;
				}

				if (result > max_val)
				{
					result = result - (max_val + 1);
				}

				if (result < 0)
				{
					result = result + (max_val + 1);
				}

				return result;
			}
			else
			{
				if (ctr == 0)
				{
					error << "\n*** WARNING: The value of the input field " << param_name << " is out of "
						  << "range and is thus being ignored. Note that the increment or step value "
						  << "must be set to a value with a magnitude less than or equal to the maximum "
						  << "value the field of interest can accept (which in this case is " << max_val << ").\n";
				}
				valid = false;
				return -1;
			}

		}



	//
	// get_next_ccsds
	//

	virtual CCSDS* get_next_ccsds(void)
		{
			CCSDS* result = source->get_next_ccsds();

			// Check to ensure result is meaningful
			if (!result)
			{
				return NULL;
			}


			original_length = result->get_length();


			days = result->get_utc_days();
			milliseconds = result->get_utc_milliseconds();
			microseconds = result->get_utc_microseconds();

			micro_sec = (days*24.0*60.0*60.0*1.0e6) + (milliseconds*1.0e3) + (microseconds);

			if (packet_count == 0)
			{
				if (start_time_relative != -1 && micro_sec_start != -1.0)
				{
					micro_sec_end = micro_sec_start + (start_time_relative * 1.0e3);
				}
				else if (start_time_relative != -1 && micro_sec_start == -1.0)
				{
					micro_sec_start =  micro_sec + (start_time_relative * 1.0e3);
				}
				if (end_time_relative != -1)
				{
					micro_sec_end =	 micro_sec_start + (end_time_relative * 1.0e3);
				}
			}

			if (skip_val == -1)
			{
				skip_val = 0;
			}
			if (packet_count < skip_val)
			{
				// return unmodified CCSDS packet
				packet_count++;
				copied_count++;
				return result;
			}
			if ((count_val != -1) && (packet_count >= (skip_val + count_val)))
			{
				// return unmodified CCSDS packet
				packet_count++;
				copied_count++;
				return result;
			}

			APID current_apid = result->get_apid();

			// Only proceed if APID conditions are met
			if (apid_filter->check_apid(current_apid))
			{
				apid_match_ctr++;

				sequence_counter = result->get_sequence_count();

				// Only proceed if the sequence counter conditions are met
				if ( (sequence_counter >= sequence_counter_start)
					 && (sequence_counter <= sequence_counter_end) )
				{
					counter_match_ctr++;

					// Only proceed if the UTC time conditions are met
					if ( (micro_sec >= micro_sec_start) && (micro_sec <= micro_sec_end) )
					{

						corruption_ctr = 0;
						time_match_ctr++;


						// Corrupting Version Number
						// For MetOp mission, this field sould be set to 000,
						// signifying the Version-1 CCSDS TM Packet
						if (version_number_opt == "yes")
						{
							result->set_version_number(version_number_val);
							corruption_ctr++;
						}
						else if (version_number_opt == "random")
						{
							result->set_version_number(random_range(0, 7));
							corruption_ctr++;
						}
						else if (version_number_opt != "no")
						{
							int tmp = increment (version_number_tmp, "set-version-number",
												 version_number_val, version_number_opt,
												 7, version_number_ctr);
							version_number_ctr++;
							if (tmp != -1)
							{
								version_number_tmp = tmp;
								result->set_version_number(version_number_tmp);
								corruption_ctr++;
							}
						}

						// Corrupting Packet Type
						// Reflects the type of data unit. Not used; shall be set to 0.
						if (type_opt == "yes")
						{
							result->set_type(type_val);
							corruption_ctr++;
						}
						else if (type_opt == "random")
						{
							result->set_type(random_range(0, 1));
							corruption_ctr++;
						}
						else if (type_opt != "no")
						{
							int tmp = increment (type_tmp, "set-type", type_val, type_opt, 1, type_ctr);
							type_ctr++;
							if (tmp != -1)
							{
								type_tmp = tmp;
								result->set_type(type_tmp);
								corruption_ctr++;
							}
						}

						// Corrupting Secondary Header Flag
						// Indicates the presence of secondary header within the TM packet
						// 0 for absence, 1 for presence.
						if (secondary_header_flag_opt == "yes")
						{
							result->set_secondary_header_flag(secondary_header_flag_val);
							corruption_ctr++;
						}
						else if (secondary_header_flag_opt == "random")
						{
							result->set_secondary_header_flag(random_range(0, 1));
							corruption_ctr++;
						}
						else if (secondary_header_flag_opt != "no")
						{
							int tmp = increment (secondary_header_flag_tmp, "set-secondary-header-flag",
												 secondary_header_flag_val, secondary_header_flag_opt,
												 1, secondary_header_flag_ctr);
							secondary_header_flag_ctr++;
							if (tmp != -1)
							{
								secondary_header_flag_tmp = tmp;
								result->set_secondary_header_flag(secondary_header_flag_tmp);
								corruption_ctr++;
							}
						}

						// Corrupting APID
						// The APID can take any value between 1 to 511 and its value indicates
						// the instrument and type of measurement contained within the packet.
						// 11 bits are dedicated to the APID and hence for corruption purposes
						// it can be set to any value between 0 to 2047
						if (apid_opt == "yes")
						{
							result->set_apid(apid_val);
							corruption_ctr++;
						}
						else if (apid_opt == "random")
						{
							result->set_apid(random_range(0, 2047));
							corruption_ctr++;
						}
						else if (apid_opt != "no")
						{
							int tmp = increment (apid_tmp, "set-apid", apid_val, apid_opt, 2047, apid_ctr);
							apid_ctr++;
							if (tmp != -1)
							{
								apid_tmp = tmp;
								result->set_apid(apid_tmp);
								corruption_ctr++;
							}
						}

						// Corrupting Sequence Flag
						// Indicates if the packet is the segment of a larger set of application data
						// If it is a continuation segment (00), a first segment (01),
						// a last segment (10), or an unsegmented set of user data (11)
						if (sequence_flag_opt == "yes")
						{
							result->set_sequence_flags(sequence_flag_val);
							corruption_ctr++;
						}
						else if (sequence_flag_opt == "random")
						{
							result->set_sequence_flags(random_range(0, 3));
							corruption_ctr++;
						}
						else if (sequence_flag_opt != "no")
						{
							int tmp = increment (sequence_flag_tmp, "set-sequence-flag", sequence_flag_val,
												 sequence_flag_opt, 3, sequence_flag_ctr);
							sequence_flag_ctr++;
							if (tmp != -1)
							{
								sequence_flag_tmp = tmp;
								result->set_sequence_flags(sequence_flag_tmp);
								corruption_ctr++;
							}
						}

						// Corrupting Sequence Counter
						// Provides the sequential binary continuous count (modulo 16383)
						if (sequence_counter_opt == "yes")
						{
							result->set_sequence_count(sequence_counter_val);
							corruption_ctr++;
						}
						else if (sequence_counter_opt == "random")
						{
							result->set_sequence_count(random_range(0, 16383));
							corruption_ctr++;
						}
						else if (sequence_counter_opt != "no")
						{
							int tmp = increment (sequence_counter_tmp, "set-sequence-counter", sequence_counter_val,
												 sequence_counter_opt, 16383, sequence_counter_ctr);
							sequence_counter_ctr++;
							if (tmp != -1)
							{
								sequence_counter_tmp = tmp;
								result->set_sequence_count(sequence_counter_tmp);
								corruption_ctr++;
							}
						}

						// Corrupting Packet Length
						// Length of Source Data (0-65535)
						if (packet_length_opt == "yes")
						{
							result->set_packet_length(packet_length_val);
							corruption_ctr++;
						}
						else if (packet_length_opt == "random")
						{
							result->set_packet_length(random_range(0, 65535));
							corruption_ctr++;
						}
						else if (packet_length_opt != "no")
						{
							int tmp = increment (packet_length_tmp, "set-packet-length", (packet_length_val),
												 packet_length_opt, 65535, packet_length_ctr);
							packet_length_ctr++;
							if (tmp != -1)
							{
								packet_length_tmp = tmp;
								result->set_packet_length(packet_length_tmp);
								corruption_ctr++;
							}
						}

						// Corrupting UTC-Days
						if (utc_days_opt == "yes")
						{
							result->set_utc_days(utc_days_val);
							corruption_ctr++;
						}
						else if (utc_days_opt == "random")
						{
							result->set_utc_days(random_range(0, 65535));
							corruption_ctr++;
						}
						else if (utc_days_opt != "no")
						{
							int tmp = increment (utc_days_tmp, "set-utc-days", utc_days_val,
												 utc_days_opt, 65535, utc_days_ctr);
							utc_days_ctr++;
							if (tmp != -1)
							{
								utc_days_tmp = tmp;
								result->set_utc_days(utc_days_tmp);
								corruption_ctr++;
							}
						}

						// Corrupting UTC-Milliseconds
						if (utc_milliseconds_opt == "yes")
						{
							result->set_utc_milliseconds(utc_milliseconds_val);
							corruption_ctr++;
						}
						else if (utc_milliseconds_opt == "random")
						{
							result->set_utc_milliseconds(random_range(0, 86399999));
							corruption_ctr++;
						}
						else if (utc_milliseconds_opt != "no")
						{
							int tmp = increment (utc_milliseconds_tmp, "set-utc-milliseconds", utc_milliseconds_val,
												 utc_milliseconds_opt, 86399999, utc_milliseconds_ctr);
							utc_milliseconds_ctr++;
							if (tmp != -1)
							{
								utc_milliseconds_tmp = tmp;
								result->set_utc_milliseconds(utc_milliseconds_tmp);
								corruption_ctr++;
							}
						}

						// Corrupting UTC-Microseconds
						if (utc_microseconds_opt == "yes")
						{
							result->set_utc_microseconds(utc_microseconds_val);
							corruption_ctr++;
						}
						else if (utc_microseconds_opt == "random")
						{
							result->set_utc_microseconds(random_range(0, 999));
							corruption_ctr++;
						}
						else if (utc_microseconds_opt != "no")
						{
							int tmp = increment (utc_microseconds_tmp, "set-utc-microseconds", utc_microseconds_val,
												 utc_microseconds_opt, 999, utc_microseconds_ctr);
							utc_microseconds_ctr++;
							if (tmp != -1)
							{
								utc_microseconds_tmp = tmp;
								result->set_utc_microseconds(utc_microseconds_tmp);
								corruption_ctr++;
							}
						}

						// Corrupting OBT-Coarse time
						if (obt_coarse_opt == "yes")
						{
							result->set_obt_coarse(obt_coarse_val);
							corruption_ctr++;
						}
						else if (obt_coarse_opt == "random")
						{
							result->set_obt_coarse(random_range(0, 16777215));
							corruption_ctr++;
						}
						else if (obt_coarse_opt != "no")
						{
							int tmp = increment (obt_coarse_tmp, "set-obt-coarse", obt_coarse_val,
												 obt_coarse_opt, 16777215, obt_coarse_ctr);
							obt_coarse_ctr++;
							if (tmp != -1)
							{
								obt_coarse_tmp = tmp;
								result->set_obt_coarse(obt_coarse_tmp);
								corruption_ctr++;
							}
						}

						// Corrupting OBT-Fine time
						if (obt_fine_opt == "yes")
						{
							result->set_obt_fine(obt_fine_val);
							corruption_ctr++;
						}
						else if (obt_fine_opt == "random")
						{
							result->set_obt_fine(random_range(0, 65535));
							corruption_ctr++;
						}
						else if (obt_fine_opt != "no")
						{
							int tmp = increment (obt_fine_tmp, "set-obt-fine", obt_fine_val,
												 obt_fine_opt, 65535, obt_fine_ctr);
							obt_fine_ctr++;
							if (tmp != -1)
							{
								obt_fine_tmp = tmp;
								result->set_obt_fine(obt_fine_tmp);
								corruption_ctr++;
							}
						}


						// Corrupting a range of bytes at a given offset
						if (byte_offset_start >= original_length)
						{
							error << "\nSpecified byte offset is equal to or greater than the total packet length ("
								  << original_length << ").\n";
							valid = false;
							return NULL;
						}
						if (byte_offset_end >= original_length)
						{
							byte_offset_end = original_length - 1;
						}
						if (byte_offset_start > byte_offset_end && byte_offset_end != -1)
						{
							int tmp_offset = byte_offset_end;
							byte_offset_end = byte_offset_start;
							byte_offset_start = tmp_offset;
						}

						if (byte_offset_start != -1 && byte_offset_end == -1)
						{
							uint8 tmp_byte_val;
							if (byte_opt == "random")
							{
								tmp_byte_val = (uint8)random_range(0, 255);
								result->set_byte(tmp_byte_val, byte_offset_start);
								corruption_ctr++;
							}
							else
							{
								if (byte_val1 != -1 && byte_val2 == 0)
								{
									tmp_byte_val = (uint8)byte_val1;
									result->set_byte(tmp_byte_val, byte_offset_start);
									corruption_ctr++;
								}
								else if (byte_val1 != -1 && byte_val2 != 0)
								{
									int tmp = increment (byte_tmp, "set-byte", byte_val1,
														 lexical_cast<string>(byte_val2), 255, byte_ctr);
									byte_ctr++;
									if (tmp != -1)
									{
										byte_tmp = tmp;
										tmp_byte_val = (uint8)tmp;
										result->set_byte(tmp_byte_val, byte_offset_start);
										corruption_ctr++;
									}
								}
							}
						}
						else if (byte_offset_start != -1 && byte_offset_end != -1)
						{
							uint8 tmp_byte_val;
							if (byte_opt == "random")
							{
								for (int i=byte_offset_start; i<=byte_offset_end; i++)
								{
									tmp_byte_val = (uint8)random_range(0, 255);
									result->set_byte(tmp_byte_val, i);
								}
								corruption_ctr++;
							}
							else
							{
								if (byte_val1 != -1 && byte_val2 == 0)
								{
									tmp_byte_val = (uint8)byte_val1;
									for (int i=byte_offset_start; i<=byte_offset_end; i++)
									{
										result->set_byte(tmp_byte_val, i);
									}
									corruption_ctr++;
								}
								else if (byte_val1 != -1 && byte_val2 != 0)
								{
									int tmp;
									if (byte_increment == "row" || byte_increment == "both")
									{
										tmp = increment (byte_tmp, "set-byte", byte_val1,
														 lexical_cast<string>(byte_val2), 255, byte_ctr);
										byte_ctr++;
									}
									else
									{
										tmp = byte_val1;
									}

									if (tmp != -1)
									{
										byte_tmp = tmp;
										tmp_byte_val = (uint8)tmp;
										int ctr = 0;
										for (int i=byte_offset_start; i<=byte_offset_end; i++)
										{
											if (byte_increment == "column" || byte_increment == "both")
											{
												tmp_byte_val = byte_tmp + (byte_val2 * ctr);
												ctr++;
											}
											result->set_byte(tmp_byte_val, i);
										}
										corruption_ctr++;
									}
								}
							}
						}



						// Corrupting PEC - Packet Error Control
						if (pec_opt == "off")
						{
							// Do nothing thus allowing original or corrupt (byte corrupt)
							// PEC value to be simply copied over
						}
						else if (pec_opt == "yes")
						{
							result->set_error_control(pec_val);
							corruption_ctr++;
						}
						else if (pec_opt == "random")
						{
							result->set_error_control(random_range(0, 65535));
							corruption_ctr++;
						}
						else if (pec_opt != "no")
						{
							int tmp = increment (pec_tmp, "set-pec", pec_val, pec_opt, 65535, pec_ctr);
							pec_ctr++;
							if (tmp != -1)
							{
								pec_tmp = tmp;
								result->set_error_control(pec_tmp);
								corruption_ctr++;
							}
							else
							{
								result->set_error_control(result->calculate_error_control());
							}
						}
						else
						{
							result->set_error_control(result->calculate_error_control());
						}


						// return modified CCSDS packet
						packet_count++;
						if (corruption_ctr > 0)
						{
							corrupt_count++;


							if (corrupt_count == 1)
							{
								affected_apids[affected_ctr] = current_apid;
								affected_times_start[affected_ctr] =
									TimeConv::utc_days_milliseconds_to_string(days, milliseconds);
								affected_times_end[affected_ctr] =
									TimeConv::utc_days_milliseconds_to_string(days, milliseconds);
								affected_ctr++;
							}
							else
							{
								bool new_apid = true;
								for (int i=0; i<affected_ctr; i++)
								{
									if (affected_apids[i] == current_apid)
									{
										affected_times_end[i] =
											TimeConv::utc_days_milliseconds_to_string(days, milliseconds);
										new_apid = false;
									}
								}
								if (new_apid)
								{
									affected_apids[affected_ctr] = current_apid;
									affected_times_start[affected_ctr] =
										TimeConv::utc_days_milliseconds_to_string(days, milliseconds);
									affected_times_end[affected_ctr] =
										TimeConv::utc_days_milliseconds_to_string(days, milliseconds);
									affected_ctr++;
								}
							}

						}
						else
						{
							copied_count++;
						}
						return result;

					}
					else  // if the UTC time conditions are not met
					{
						// return unmodified CCSDS packet
						packet_count++;
						copied_count++;
						return result;
					}
				}
				else // if the sequence counter conditions are not met
				{
					// return unmodified CCSDS packet
					packet_count++;
					copied_count++;
					return result;
				}
			}
			else // if APID conditions are not met
			{
				// return unmodified CCSDS packet
				packet_count++;
				copied_count++;
				return result;
			}

		}




	//
	// produce_files
	//

	virtual bool produce_files(FileConsumer& dest)
		{
			CCSDS* ccsds;
			while((ccsds=get_next_ccsds()))
			{
				long length = original_length;
				if (length_flag)
				{
					length = ccsds->get_length();
				}

				stringstream s;
				s << "." << ccsds->get_apid() << ".ccsds";
				ostream_type* os = dest.get_named_output_stream(s.str());
				os->write((char*)ccsds->get_data(),length);
			}
			return true;
		}



	//
	// get_next_file
	//

	virtual istream_type* get_next_file(void)
		{
			return NULL;
		}



	//
	// is_valid
	//

	bool is_valid(void) const
		{
			return valid;
		}

	EPSProducer* as_eps_product(void)
		{
			return NULL;
		}

private:



	CCSDSProducer* source;
	int skip_val;
	int count_val;
	CCSDSAPIDFilter* apid_filter;
	StringList selected_apid;

	int sequence_counter_start;
	int sequence_counter_end;

	int start_time_relative;
	int end_time_relative;

	string version_number_opt;
	string type_opt;
	string secondary_header_flag_opt;
	string apid_opt;
	string sequence_flag_opt;
	string sequence_counter_opt;
	string packet_length_opt;
	string utc_days_opt;
	string utc_milliseconds_opt;
	string utc_microseconds_opt;
	string obt_coarse_opt;
	string obt_fine_opt;
	string pec_opt;

	int version_number_val;
	int type_val;
	int secondary_header_flag_val;
	int apid_val;
	int sequence_flag_val;
	int sequence_counter_val;
	int packet_length_val;
	int utc_days_val;
	int utc_milliseconds_val;
	int utc_microseconds_val;
	int obt_coarse_val;
	int obt_fine_val;
	int pec_val;

	string byte_opt;
	int byte_val1;
	int byte_val2;
	int byte_offset_start;
	int byte_offset_end;
	string byte_increment;

	bool length_flag;
	long original_length;

	int corruption_ctr;
	int packet_count;
	int corrupt_count;
	int copied_count;
	bool valid;

	int affected_ctr;
	APID affected_apids[512];
	string affected_times_start[512];
	string affected_times_end[512];

	unsigned int sequence_counter;
	unsigned int days;
	unsigned int milliseconds;
	unsigned int microseconds;

	double micro_sec;
	double micro_sec_start;
	double micro_sec_end;

	int apid_match_ctr;
	int counter_match_ctr;
	int time_match_ctr;

	int version_number_tmp;
	int version_number_ctr;
	int type_tmp;
	int type_ctr;
	int secondary_header_flag_tmp;
	int secondary_header_flag_ctr;
	int apid_tmp;
	int apid_ctr;
	int sequence_flag_tmp;
	int sequence_flag_ctr;
	int sequence_counter_tmp;
	int sequence_counter_ctr;
	int packet_length_tmp;
	int packet_length_ctr;
	int utc_days_tmp;
	int utc_days_ctr;
	int utc_milliseconds_tmp;
	int utc_milliseconds_ctr;
	int utc_microseconds_tmp;
	int utc_microseconds_ctr;
	int obt_coarse_tmp;
	int obt_coarse_ctr;
	int obt_fine_tmp;
	int obt_fine_ctr;
	int pec_tmp;
	int pec_ctr;
	int byte_tmp;
	int byte_ctr;

};


#endif
