////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsdscountfilter.hpp
//
//	Provides: CCSDSCountFilter filter
//
//	Purpose: Allow through a certain number of packets from a source packet
//			 stream
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ccsdscountfilter_defined
#define ccsdscountfilter_defined

#include "ccsdsproducer.hpp"

class CCSDSCountFilter :
	public CCSDSProducer,
	public FileProducer
{
public:

	CCSDSCountFilter( CCSDSProducer& _source, int _skip, int _count)
		: source(_source)
		, skip(_skip)
		, count(_count)
		, initial(0)
		, allowed(0)
		{
		}

	virtual void output_text_summary( ostream& os = cout )
		{
			os << "CCSDSCountFilter report:\n"
			   << "	   " << initial << " initial packets were skipped\n"
			   << "	   " << allowed << " packets were allowed through\n\n";
		}

	virtual CCSDS* get_next_ccsds(void)
		{
			CCSDS* result = source.get_next_ccsds();
			//DEBUG("ccsdscountfilter::get_next_ccsds producer sent "
			//<< ((unsigned long long)result) << endl);
			DEBUG("skip " << skip << " count " << count << endl);

			if (!result)
			{
				if (spinner.get())
				{
					spinner->end();
				}
				return NULL;
			}

			DEBUG("continuing...\n");

			while(skip>0)
			{
				DEBUG("ccsdscountfilter::get_next_ccsds skipping packet, "
					  << "looking for more...\n");
				result = source.get_next_ccsds();
				if (result == NULL)
				{
					DEBUG("Second try, source sent NULL\n");
					if (spinner.get())
					{
						spinner->end();
					}
					return NULL;
				}
				skip--;
				initial++;
				DEBUG("skip is now " << skip << endl);
			}

			DEBUG("continuing further, skip is " << skip << endl);

			if (count != -1)
			{
				if (count > 0)
				{
					count--;
					allowed++;
					DEBUG("ccsdscountfilter::get_next_ccsds passing packet "
						  << "on\n");
					if (!result && spinner.get())
					{
						spinner->end();
					}
					return result;
				}
				else
				{
					DEBUG("ccsdscountfilter::get_next_ccsds thats all\n");
					if (spinner.get())
					{
						spinner->end();
					}
					return NULL;
				}
			}
			else
			{
				DEBUG("ccsdscountfilter::get_next_ccsds passing on this and "
					  << "all future packets\n");
				allowed++;
				if (!result && spinner.get())
				{
					spinner->end();
				}
				return result;
			}
		}

	virtual bool produce_files(FileConsumer& dest)
		{
			CCSDS* ccsds;
			while((ccsds=get_next_ccsds()))
			{
				stringstream s;
				s << "." << ccsds->get_apid() << ".ccsds";
				ostream_type* os = dest.get_named_output_stream(s.str());
				os->write((char*)ccsds->get_data(),ccsds->get_length());
			}
			return true;
		}

	virtual istream_type* get_next_file(void)
		{
			return NULL;
		}

	EPSProducer* as_eps_product(void)
		{
			return source.as_eps_product();
		}

private:

	CCSDSProducer& source;

	int skip;
	int count;

	int initial;
	int allowed;
};

#endif
