////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsdsdemultiplexer.cpp
//
//	Provides: CCSDSDemultiplexer
//
//	Purpose: Seperate a CCSDS packet stream by application id, sorting each
//			 branch by sequence count
//
//	Copyright (c) 2002, 2003, 2004, 2005 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "ccsdsdemultiplexer.hpp"
#include "ccsdsapidfilter.hpp"

const string FILE_EXT = ".ccsds";

//
//
// CCSDSDemultiplexer::Channel
//
//

//
// Constructor
//

CCSDSDemultiplexer::Channel::Channel(void)
	: last_sequence(-1)
{
}

//
//
// CCSDSDemultiplexer
//
//

//
// Constructor
//

CCSDSDemultiplexer::CCSDSDemultiplexer(CCSDSProducer& _source,
									   bool _split_by_inst)
	: source(_source),
	  split_by_inst(_split_by_inst)
{
	inst_names[1] = "hktm";
	inst_names[2] = "hktm";
	inst_names[3] = "hktm";
	inst_names[6] = "hktm";
	inst_names[34] = "mhs";
	inst_names[35] = "a-dcs";
	inst_names[37] = "sem";
	inst_names[38] = "hirs";
	inst_names[39] = "amsua";
	inst_names[40] = "amsua";
	for(int i=64; i<=70; i++)
	{
		inst_names[i] = "avhrr";
	}
	inst_names[103] = "avhrr";
	inst_names[104] = "avhrr";
	for(int i=130; i<=180; i++)
	{
		inst_names[i] = "iasi";
	}
	for(int i=192; i<=255; i++)
	{
		inst_names[i] = "ascat";
	}
	inst_names[384] = "gome";
	inst_names[448] = "gras";
	inst_names[450] = "gras";
	inst_names[480] = "gras";
}

//
// output_text_summary
//

void CCSDSDemultiplexer::output_text_summary(ostream& os)
{
	os << "CCSDSDemultiplexer report:\n";
	for(Counts::iterator iter=counts.begin();
		iter!=counts.end();
		iter++)
	{
		if (split_by_inst)
		{
			os << "	   " << ((*iter).first)
			   << " packets " << ((*iter).second).counts
			   << " bytes " << ((*iter).second).bytes
			   << endl;
		}
		else
		{
			stringstream str;
			str << (*iter).first;
			int apid_num;
			str >> apid_num;
			string apid_str = (*iter).first;
			if (apid_str=="192")
			{
				apid_str = "192-255";
			}
			os << "	   APID " << apid_str
			   << " (" << CCSDSAPIDFilter::apid_to_string(apid_num)
			   << ") packets " << ((*iter).second).counts
			   << " bytes " << ((*iter).second).bytes
			   << endl;
		}
	}
	os << endl;
}

//
// get_next_file
//

// if this function gets called it means theres a singlefilereader
// reading us

istream_type* CCSDSDemultiplexer::get_next_file(void)
{
	return NULL;
}

//
// produce_files
//

bool CCSDSDemultiplexer::produce_files(FileConsumer& dest)
{
	spinner.reset(new Spinner());
	while(CCSDS* ccsds = source.get_next_ccsds())
	{
		APID apid = ccsds->get_apid();
		write_packet(apid, *ccsds, dest);
	}
	return true;
}

void CCSDSDemultiplexer::write_packet(APID apid,
									  CCSDS& packet,
									  FileConsumer& dest)
{
	string ext;
	if (split_by_inst)
	{
		ext = inst_names[apid];
		if (ext == "")
		{
			ext = ntos(apid);
		}
		ext += FILE_EXT;
	}
	else
	{
		if (apid>=192 && apid<=255)
		{
			ext = "192-224" + FILE_EXT;
		}
		else
		{
			ext = ntos(apid);
		}
	}

	counts[ext].counts = counts[ext].counts + 1;
	counts[ext].bytes = counts[ext].bytes + packet.get_length();

	// see if we have encountered this apid before
	OStreams::iterator iter = ostreams.find(ext);
	if (iter==ostreams.end())
	{
		ostreams[ext] = dest.get_named_output_stream(string(".")+ext);
		iter = ostreams.find(ext);
	}
	(*iter).second->write((char*)packet.get_data(),packet.get_length());
}

CCSDSDemultiplexer::Count::Count(void)
	: counts(0),
	  bytes(0)
{
}
