////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsdsdemultiplexer.hpp
//
//	Provides: CCSDSDemultiplexer
//
//	Purpose: Seperate a CCSDS packet stream by application id, sorting each
//			 branch by sequence count
//
//	Copyright (c) 2002, 2003, 2004, 2005 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ccsdsdemultiplexer
#define ccsdsdemultiplexer

#include "ccsdsproducer.hpp"
#include "fileproducer.hpp"
#include "fileconsumer.hpp"

class CCSDSDemultiplexer :
	public FileProducer
{

public:

	typedef map<int, CCSDS> ChannelData;

	struct Channel
	{
		Channel(void);

		ChannelData data;
		int last_sequence;
	};

	typedef map<APID, Channel> Channels;

	CCSDSDemultiplexer(CCSDSProducer& _source, bool split_by_inst);

	virtual void output_text_summary(ostream& os = cout);
	virtual istream_type* get_next_file(void);
	virtual bool produce_files(FileConsumer& dest);
	void write_packet(APID apid, CCSDS& packet, FileConsumer& dest);

private:

	CCSDSProducer& source;
	auto_ptr<istream_type> fileprod_is;

	typedef map<string, ostream_type* > OStreams;
	OStreams ostreams;

	struct Count
	{
		Count(void);
		int counts;
		long long bytes;
	};
	typedef map<string,Count> Counts;
	Counts counts;

	typedef map<APID, string> INST_NAMES;
	INST_NAMES inst_names;

	bool split_by_inst;
};

#endif
