////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsdsextractor.hpp
//
//	Provides: CCSDSExtractor extractor
//
//	Purpose: Convert CCSDS packets into a binary stream for storing
//
//	Copyright (c) 2002, 2003, 2004, 2006 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ccsdsextractor_defined
#define ccsdsextractor_defined

#include "interface.hpp"
#include "ccsdsproducer.hpp"

class CCSDSExtractor :
	public FileProducer
{
public:

	CCSDSExtractor( CCSDSProducer& _source )
		: source(_source)
		, packet_count(0)
		{
		}

	void output_text_summary( ostream& os = cout )
		{
			os << "CCSDSExtractor report:\n"
			   << "	   " << packet_count << " packets read\n"
			   << endl;
		}

	bool produce_files(FileConsumer& dest)
		{
			DEBUG("ccsdsextractor begin\n");
			while(CCSDS* ccsds = source.get_next_ccsds())
			{
				ostream_type* os = dest.get_output_stream();
				os->write((char*)ccsds->get_data(), ccsds->get_length());
				packet_count++;
//		DEBUG("ccsdsextractor got packet\n");
			}
			DEBUG("ccsdsextractor end of stream\n");
			return true;
		}

	istream_type* get_next_file(void)
		{
			return NULL;
		}

private:

	CCSDSProducer& source;
	int packet_count;
};

#endif


