////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsdsmultiplexer.hpp
//
//	Provides: CCSDSMultiplexer filter
//
//	Purpose: Merge several source packets streams into one
//
//	Copyright (c) 2002, 2003, 2004, 2006 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ccsdsmultiplexer_defined
#define ccsdsmultiplexer_defined

#include "ccsdsproducer.hpp"
#include "textsummaryproducer.hpp"

class CCSDSMultiplexer :
	public CCSDSProducer
{
public:

	CCSDSMultiplexer(void)
		: input_count(0)
		{
		}

	void add_input(CCSDSProducer& source)
		{
			sources.push_back(&source);
			next_source = sources.begin();
			input_count++;
		}

	CCSDS* get_next_ccsds(void)
		{
			if (sources.empty())
			{
				return NULL;
			}

			CCSDS* result = (*next_source)->get_next_ccsds();

			if (result)
			{
				next_source++;
				if (next_source == sources.end())
				{
					next_source = sources.begin();
				}
				return result;
			}
			else
			{
				Sources::iterator i(next_source);
				i++;
				if (i==sources.end())
				{
					i=sources.begin();
				}
				CCSDSProducer* c = *i;
				sources.erase(next_source);
				next_source = find(sources.begin(),sources.end(),c);
				return get_next_ccsds();
			}
		}

	void output_text_summary( ostream& os = cout )
		{
			os << "CCSDSMultiplexer report:\n"
			   << "	   " << input_count << " inputs used\n"
			   << endl;
		}

	EPSProducer* as_eps_product(void)
		{
			return NULL;
		}

private:

	typedef deque<CCSDSProducer*> Sources;
	Sources sources;

	Sources::iterator next_source;

	int input_count;

};

#endif
