////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsdspacketdisplayer.hpp
//
//	Provides: CCSDSPacketDisplayer filter
//
//	Purpose: Show contents of CCSDS packets as text
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ccsdspacketdisplayer_defined
#define ccsdspacketdisplayer_defined

#include "ccsds.hpp"
#include "ccsdsproducer.hpp"
#include "ccsdsapidfilter.hpp"
#include "hirsbase.hpp"
#include "amsuabase.hpp"
#include "iasi_config.hpp"
#include "sband.hpp"

class CCSDSPacketDisplayer : public HIRSBase, public AMSUABase
{
public:

	typedef NOAA_L1b_AVHRR_Data::Sample AVHRR_Sample;
	typedef MHSPayload::Sample MHS_Sample;

	CCSDSPacketDisplayer(CCSDSProducer& _source,
						 const string& config_file = "",
						 const string& _iasi_config_file = "");

	void output_text_record(ostream& os,
							bool hex,
							bool header_only,
							bool show_gaps,
							bool show_seq_diffs,
							bool shift_gras_obt);

	void output_header(CCSDS* packet, ostream& os, bool show_gaps, bool shift_gras_obt, bool show_seq_diffs);
	void output_raw_hex_dump(CCSDS* packet, ostream& os);
	void output_sband_record(CCSDS* packet, ostream& os);
	void output_amsu_hex_dump(CCSDS* packet, ostream& os);

	void output_avhrr_record(CCSDS* packet, ostream& os);
	void output_amsu_a1_measure(ostream& os, AMSUA1MeasurementScienceData& data);
	void output_amsu_a1_unique(ostream& os, AMSUA1UniqueScienceData& data);
	void output_amsu_a2_measure(ostream& os, AMSUA2MeasurementScienceData& data);
	void output_amsu_a2_unique(ostream& os, AMSUA2UniqueScienceData& data);
	void output_mhs_record(ostream& os, CCSDS* packet);
	void output_hirs_record(ostream& os, CCSDS* packet);
	void output_iasi_spec_record(ostream& os, CCSDS* packet);
	void output_iasi_image_record(ostream& os, CCSDS* packet);
	void output_iasi_verif_record(ostream& os, CCSDS* packet);
	void output_iasi_anc_record(ostream& os, CCSDS* packet);
	void output_gome_record(ostream& os, CCSDS* packet);
	void output_ascat_nom_record(ostream& os, CCSDS* packet);
	void output_ascat_raw_record(ostream& os, CCSDS* packet);
	void output_gras_navigation_nominal_record(ostream& os, CCSDS* packet);
	void output_gras_navigation_reduced_record(ostream& os, CCSDS* packet);
	void output_gras_measurement_record(ostream& os, CCSDS* packet);

	void display_mhs_samples(ostream& os, MHS_Sample* samples, int count);

	int read_avhrr_bit(byte*& payload_byte, int& payload_bit);
	AVHRR_Sample read_avhrr_sample(byte*& payload_byte, int& payload_bit);

	void extract_iasi_spectrum_data(CCSDS* packet, deque<byte>& iasi_data);
	uint16_be extract_iasi_spectrum(uint16_be* data, int noBits, int* bitsCtr, int* wordCtr);
	void extract_iasi_verification_data(CCSDS* packet, deque<byte>& iasi_data);
	void extract_iasi_image_data(CCSDS* packet, deque<byte>& iasi_data);
	uint16_be extract_iasi_image(uint16_be* data, int* offsetPtr, int bpis, int* ctr);
	uint16_be extract_iasi_image_ctr(uint16_be* data, int* offsetPtr, int bpis, int* ctr);
	void extract_amsu_data(CCSDS* packet, deque<byte>& sci_data);
	void extract_ascat_data(CCSDS* packet, deque<byte>& sci_data);
	void extract_gras_data(CCSDS* packet, deque<byte>& tmp_data, int length);

private:

	double obct_to_temp(int input);

	double obct_to_res(int input);

	CCSDSProducer& source;
	string config_file;
	string iasi_config_file;
	int rollover;
	int anti_rollover;

	unsigned int packet_counter;
	auto_ptr<UTCTime> last_utc_time;
	auto_ptr<OBTTime> last_obt_time;
	auto_ptr<OBTTime> last_iasi_obt_time;
	auto_ptr<short_cds_time> last_grh_time;
	auto_ptr<int> last_seq_count;
};

#undef Config

#endif

