////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsdsreader.hpp
//
//	Provides: CCSDSReader filter
//
//	Purpose: Extract CCSDS packets from a plain binary stream
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ccsdsreader_defined
#define ccsdsreader_defined

#include "ccsds.hpp"
#include "ccsdsproducer.hpp"
#include "fileproducer.hpp"
#include "fileconsumer.hpp"
#include "eps.hpp"

class CCSDSReader :
	public CCSDSProducer,
	public FileProducer,
	public EPSProducer
{
public:

	/// constructor
	CCSDSReader( FileProducer& _source, int _prepacket_header=0, bool show_viadr=false );

	/// return next CCSDS packet
	CCSDS* get_next_ccsds(void);

	// get data type of given file name
	GROUPID get_data_type(void);

	// reset the CCSDS stream and return to first file
	void reset_stream(void);

	// delete source reference
	void delete_source(void);

	/// output_text_summary
	void output_text_summary( ostream& os = cout );

	/// produce_files
	bool produce_files(FileConsumer& dest);

	/// get_next_file
	istream_type* get_next_file(void);

	/// get_scid (CCSDSProducer)
	string get_scid(void);

	EPSProducer* as_eps_product(void);
	short_cds_time get_grh_start(void) const;
	short_cds_time get_grh_stop(void) const;
	const VIADR_OBT2UTC* get_viadr_l0_obt2utc(const short_cds_time& grh_start_time, int* viadr_matching) const;
	string get_mphr_field(const string& field) const;

private:

	bool rec_head_equal(const REC_HEAD& a, const REC_HEAD& b);
	REC_HEAD make_matching_rec_head(void);
	void ingest_mphr_values(istream_type*);

	FileProducer* source;
	int prepacket_header;

	unsigned long long byte_count;
	int packet_count;

	auto_ptr<CCSDS> packet;
	istream_type* is;

	REC_HEAD grh_mphr;
	REC_HEAD grh_ipr;
	REC_HEAD grh_viadr_l0_obt2utc;
	REC_HEAD grh_dummy;
	REC_HEAD grh_isp;

	// hold last CCSDS packet for later retrieval
	CCSDS* hold;

	bool eps;
	short_cds_time grh_start;
	short_cds_time grh_stop;
	VIADR_OBT2UTC viadr_l0_obt2utc;
	vector<VIADR_OBT2UTC> viadr_l0_obt2utc_arr;

	bool show_viadr;

	typedef map<string,string> StringLookup;
	StringLookup mphr_values;

	//typedef vector<DMDR> DMDRs;
	//DMDRs dmdrs;
};

#endif
