////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsdsshowgaps.cpp
//
//	Provides: CCSDSShowGaps filter
//
//	Purpose: Show packets that have one of a specified gap
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "ccsdsshowgaps.hpp"

CCSDSShowGaps::CCSDSShowGaps(CCSDSProducer& source,
							 deque<int> utc_gaps,
							 deque<int> grh_gaps,
							 deque<int> obt_gaps)
{
	UTCTime blank_utc_time;
	blank_utc_time.days = 0;
	blank_utc_time.milliseconds = 0;
	blank_utc_time.microseconds = 0;
	UTCTime last_utc_time = blank_utc_time;

	OBTTime blank_obt_time;
	blank_obt_time.obt_coarse = 0;
	blank_obt_time.obt_fine = 0;
	OBTTime last_obt_time = blank_obt_time;

	short_cds_time blank_grh_time;
	short_cds_time last_grh_time;

	CCSDS* ccsds;
	int cc = 0;

	long long utc_diff;
	long long grh_diff;
	long long obt_diff;

	while((ccsds=source.get_next_ccsds())!=NULL)
	{
		bool show_it = false;

		if (ccsds->has_secondary_header())
		{
			UTCTime this_utc_time = *ccsds->get_secondary_header();
			if (last_utc_time!=blank_utc_time)
			{
				utc_diff = this_utc_time - last_utc_time;
				if (find(utc_gaps.begin(), utc_gaps.end(), utc_diff) != utc_gaps.end())
				{
					show_it = true;
				}
			}
			last_utc_time = this_utc_time;
		}

		OBTTime this_obt_time = *ccsds->get_ancillary_data();
		if (last_obt_time!=blank_obt_time)
		{
			obt_diff = this_obt_time - last_obt_time;
			if (find(obt_gaps.begin(), obt_gaps.end(), obt_diff) != obt_gaps.end())
			{
				show_it = true;
			}
		}
		last_obt_time = this_obt_time;

		EPSProducer* eps = source.as_eps_product();

		if (eps)
		{
			short_cds_time this_grh_time = eps->get_grh_start();
			if (last_grh_time != blank_grh_time)
			{
				grh_diff = this_grh_time - last_grh_time;
				if (find(grh_gaps.begin(), grh_gaps.end(), grh_diff) != grh_gaps.end())
				{
					show_it = true;
				}
			}
			last_grh_time = this_grh_time;
		}

		if (show_it)
		{
			cout << "Packet " << cc << " APID " << ccsds->get_apid() << " Seq Count " << ccsds->get_sequence_count();

			if (ccsds->has_secondary_header())
			{
				cout << " UTC Diff " << utc_diff
					 << " UTC " << last_utc_time;
			}

			cout << " OBT Diff " << obt_diff
				 << " OBT " << last_obt_time;

			if (eps)
			{
				cout << " GRH Diff " << grh_diff
					 << " GRH " << last_grh_time;
			}

			cout << endl;
		}
		cc++;
	}
}
