////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsdsstreamdisplayer.hpp
//
//	Provides: CCSDSStreamDisplayer filter
//
//	Purpose: Provide some statistical information on a stream of CCSDS packets
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ccsdsstreamdisplayer_defined
#define ccsdsstreamdisplayer_defined

#include "interface.hpp"
#include "ccsdsproducer.hpp"

class CCSDSStreamDisplayer : public Interface
{
public:

	CCSDSStreamDisplayer(CCSDSProducer& _source, bool _shift_gras_obt);

	int get_total_packets(void) const;
	void output_text_summary(ostream& os = cout);

private:

	// diff (ms) , count
	typedef map<long long, int> Gaps;

	CCSDSProducer& source;

	struct Channel
	{
		struct Sequence
		{
			int initial_seq_count;
			int no_of_packets;
			UTCTime start_utc;
			UTCTime end_utc;
			bool has_utc_time;
			bool is_shifted_gras;
			int rollovers;
			Gaps utc_gaps;
			OBTTime start_obt;
			OBTTime end_obt;
			Gaps obt_gaps;
			short_cds_time start_grh;
			short_cds_time end_grh;
			Gaps grh_gaps;
		};
		typedef deque<Sequence> Counts;
		Counts counts;

		int total_packets;

		typedef set<APID> APIDS;
		APIDS apids;
	};
	typedef map<string, Channel> Channels;

	Channels channels;
	int total_packets;
	CCSDS* last_packet;
	bool shift_gras_obt;
};

#endif
