////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsdstol0.hpp
//
//	Provides: CCSDSToL0filter
//
//	Purpose:
//
//	Copyright (c) 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ccsdstol0_defined
#define ccsdstol0_defined

#include "ccsds_config.hpp"
#include "interface.hpp"
#include "osvs.hpp"



//forward declarations
class CCSDSProducer;
class EPSL0;
class EPSL0Segmenter;
class MMAMhandler;
class CCSDS;

class CCSDSToL0 :  virtual public Interface
{
public:

    CCSDSToL0(CCSDSProducer& _source,
              const string& _output_filename,
              const string& _output_dirname,
              const string& _config_filename,
              const string& _config_apid,
              const string& _subsat_dirname,
              const string& _osvs_filename,
              const string& _viadr_filename,
              const string& _utc_obt_time,
              const string& _obt_utc_time,
              const string& _clock_drift,
              int _rollovers,
              bool _generateMMAM,
              bool _generateTLE,
              bool _validateMMAM,
              bool _useMMAMdata,
              const string& _mmam_indir,
              const string& _mmam_outdir,
              const string& _tle_outdir,
              const string & _mmam_schema,
              unsigned int _segmentSize,
              const string& _out_suffix);

    ~CCSDSToL0();

public:

    bool create_epsl0(void);
    bool complete_epsl0(void);
    virtual void output_text_summary(ostream& os=cout);
    void output_text_ccsds_config(void);
    void output_text_mphr_config(void);
    void output_text_viadr(void);


public:

    /// Return closest OSV before given time
	OSVS::OSV getStateVectorAtANX(const short_cds_time& time) const;
	OSVS::OSV getStateVector(const short_cds_time& time) const;
    bool checkTimeValidity(const short_cds_time& aInitialTimestamp, const short_cds_time &aEndTimestamp);
    bool isMMAMdataAvailable();

public:

    string getUTC0();
    string getOBT0();
    string getClockDrift();
	string getRolloverTime();
    int getRollovers();
    bool useMMAMdata();

    string selectValidMMAM(vector<string>& aMMAMfiles, const short_cds_time& aTimestamp);


    void setL0fileAborted(bool aState);
    bool getL0fileAborted();

public:  //getters used by segmenter

    ccsds_config getCCSDSconfig() { return ccsdsConfig; }
    string getOutputFileName() { return output_filename; }
    string getOutputDirName() { return output_dirname; }
    string getConfigFileName() { return config_filename; }
    string getSubSatDirName() { return subsat_dirname; }
    string getOSVsFileName() { return osvs_filename; }
    string getVIADRFileName() { return viadr_filename; }
    string getOutputSuffix() { return out_suffix; }

private:

    void handleMMAMisp(CCSDS* ccsds);
    bool loadCachedMMAM(const short_cds_time &aTime);
    void saveMMAMdata();

private:

    ccsds_config ccsdsConfig;

    unsigned int segmentSize;
    EPSL0Segmenter* epsl0_segmenter;

    CCSDSProducer& source;

    string output_filename;
    string output_dirname;
    string config_filename;
    string config_apid;
    string subsat_dirname;
    string osvs_filename;
    string viadr_filename;

    string utc_obt_time;
    string obt_utc_time;
    string clock_drift;
	string rollover_time;
    int initial_rollovers;
    int anti_rollovers;
    int rollovers;

    bool validMMAMloaded;
    bool generateMMAM;
    bool generateTLE;
    bool validateMMAM;
    bool m_useMMAMdata;
    string mmam_indir;
    string mmam_outdir;
    string tle_outdir;
    string mmam_schema;
    string out_suffix;

    MMAMhandler* mmam;
    OSVS osvs;
	OSVS osvs_anx;
    bool firstTime;

    bool processAborted;
};

#endif
