////////////////////////////////////////////////////////////////////////////////
//
//	File: common.hpp
//
//	Provides: misc
//
//	Purpose: Standard libraries and common code
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef COMMON_H
#define COMMON_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#else
//#error HAVE_CONFIG_H not defined
#endif

#ifdef HAVE_MALLOC
#undef HAVE_MALLOC
#endif
#define HAVE_MALLOC 1

#ifdef malloc
#undef malloc
#endif

#ifdef PYMETOPIZER
#ifdef _POSIX_C_SOURCE
#undef _POSIX_C_SOURCE
#endif
#ifdef HAVE_PROTOTYPES
#undef HAVE_PROTOTYPES
#endif
#ifdef HAVE_STDLIB_H
#undef HAVE_STDLIB_H
#endif
#include "Python.h"
#endif

#define _USE_LARGE_FILES

#if defined(_USE_LARGE_FILES) && !defined(_LARGEFILE64_SOURCE)
#define _LARGEFILE64_SOURCE
#endif

#define _FILE_OFFSET_BITS 64

#if defined(__GNUC__) || defined(_AIX_GCC)
#define _HAVE_STRFTIME
#endif

#ifdef _CYGWIN
#define _GCC
#endif

#include <stdlib.h>
#include <iostream>
#include <ostream>
#include <fstream>
#include <string>
#include <algorithm>
#include <map>
#include <deque>
#include <set>
#include <vector>
#include <sstream>
#include <assert.h>
#include <time.h>
#include <sys/types.h>
#include <sys/timeb.h>
#include <iomanip>
#include <math.h>
#include <strings.h>
#include <queue>
#include <sys/types.h>
#include <dirent.h>
#include <cstdlib>
#include <ctime>
#include <unistd.h>
#include <sys/stat.h>
#include <errno.h>
#include <glob.h>
#include <limits>
#include <string.h>
#include <memory>

using namespace std;

#define istream_type ifstream64
#define ifstream_type ifstream64
#define ostream_type ofstream64
#define ofstream_type ofstream64
#define textoutstream_type ostream
#define streampos_type unsigned long long

template <class A, class B> void copy_n (
	A in,
	int length,
	B out)
{
	for(int i=0; i<length; i++)
	{
		*out = *in;
		out++;
		in++;
	}
}

string tolower(const string& input);

extern textoutstream_type* _debug;
extern textoutstream_type* _info;
extern textoutstream_type* _error;
extern textoutstream_type* _warning;

#define debug (*_debug)
#define info (*_info)
#define error (*_error)
#define warning (*_warning)

#ifdef _DEBUG
#define DEBUG(x) debug << x
#else
#define DEBUG(x)
#endif

#ifdef _GCC
#define PACKED __attribute__((packed))
#else
#pragma pack(1)
#define PACKED
#endif

//typedef unsigned char uint8;
//typedef unsigned char byte;
// L1b products
//typedef signed char int8;

typedef unsigned char byte;
typedef unsigned char uint8;

#ifndef WORDS_BIGENDIAN

typedef unsigned char int8;

struct int16_be
{
	int16_be(short v);
	operator short (void) const;

	byte data[2];
};

struct uint16_be
{
	uint16_be(void);
	uint16_be(unsigned short v);
	uint16_be& operator = (unsigned short v);
	void set(unsigned short v);
	operator unsigned short (void) const;
	unsigned short get(void) const;
	uint16_be& operator <<= (int count);
	uint16_be& operator += (int count);
	uint16_be& operator -= (int count);
	uint16_be& operator ^= (const uint16_be& rhs);
	uint16_be& operator |= (const uint16_be& rhs);
	uint16_be operator -- (int);
	uint16_be& operator %= (int count);

	byte a;
	byte b;
};

struct uint32_be
{
	uint32_be(void);
	uint32_be(unsigned int v);
	void set(unsigned int v);
	operator unsigned int (void) const;
	unsigned int get(void) const;
	uint32_be& operator += (int count);
	uint32_be& operator -= (int count);
	uint32_be operator ++ (int);
	uint32_be& operator %= (int count);

	byte a;
	byte b;
	byte c;
	byte d;
};

struct int32_be
{
	int32_be(void);
	int32_be(signed int v);
	void set(signed int v);
	operator signed int (void) const;
	signed int get(void) const;
	int32_be& operator += (int count);
	int32_be& operator -= (int count);
	int32_be operator ++ (int);
	int32_be& operator %= (int count);

	byte a;
	byte b;
	byte c;
	byte d;
};

struct uint64_be
{
	uint64_be(void);
	uint64_be(unsigned long long v);
	void set(unsigned long long v);
	operator unsigned long long (void) const;
	unsigned long long get(void) const;
	uint64_be& operator += (long long count);
	uint64_be& operator -= (long long count);
	uint64_be operator ++ (int);
	uint64_be& operator %= (long long count);
	uint64_be& operator |= (unsigned long long count);

	byte a;
	byte b;
	byte c;
	byte d;
	byte e;
	byte f;
	byte g;
	byte h;
};

struct float64_be
{
	operator double (void) const;
	byte data[8];
};

typedef long long int64;
typedef unsigned long long uint64;

#else

typedef long long int64_be;
typedef unsigned long long uint64_be;
typedef unsigned int uint32_be;
typedef signed int int32_be;
typedef unsigned short uint16_be;
typedef signed short int16_be;
typedef double float64_be;

#endif


struct uint24_be
{
	uint24_be(void);
	uint24_be(unsigned long v);
	uint24_be& operator = (unsigned long v);
	void set(unsigned long v);
	operator unsigned long (void) const;
	unsigned long get(void) const;

	byte a;
	byte b;
	byte c;
};

struct int24_be
{
	int24_be(void);
	int24_be(signed long v);
	int24_be& operator = (signed long v);
	void set(signed long v);
	operator signed long (void) const;
	signed long get(void) const;

	char a;
	byte b;
	byte c;
};

const int MIN_INT = -1000000000;
const int MAX_INT = 1000000000;

// non terminated string
string nts(int chars, char* str);

// number to string
string ntos(int num);
string ntos(unsigned int num);

// number to binary string ("00110011")
string ntobs(unsigned int num, int bits);

// number to hex string
string ntohs(uint8 num);

// number to fixed-width string
string ntofs(int num, unsigned int width=5, char fillchar='0', bool rpad = false);

// two dp floating point
string dto2dp(double num);

const int MILLISECONDS_PER_SECOND = 1000;
const int MICROSECONDS_PER_MILLISECOND = 1000;
const int SECONDS_PER_DAY = 60*60*24;
const unsigned int MILLISECONDS_PER_DAY = 1000*SECONDS_PER_DAY;
const unsigned int MICROSECONDS_PER_DAY = 1000*MILLISECONDS_PER_DAY;

const int BASE_YEAR = 1900;
const int DAYS_PER_YEAR = 365;
const int UTC_TO_JULIAN_YEARS = 2000-1970;
const int LEAP_YEARS_1970_2000 = 7;
const int UTC_TO_JULIAN_SECONDS = SECONDS_PER_DAY *
							(DAYS_PER_YEAR*UTC_TO_JULIAN_YEARS + LEAP_YEARS_1970_2000);

// Difference between Proleptic Greogrian Calendar days (based on Year 1)
// and UTC days (based on Year 2000)
const int PGC_DAYS_TO_UTC_DAYS = 730120;

namespace TimeConv
{
	const string TIME_FORMAT = "%d-%b-%Y,%H:%M:%S";

	// set timezone to UTC to avoid problems with the OS converting
	// times to the real local zone
	void set_time_zone(void);

	time_t utc_days_milliseconds_to_time_t(unsigned int days,
										   unsigned int milliseconds);

	struct tm utc_days_milliseconds_to_components(unsigned int days,
												  unsigned int milliseconds);

	string time_t_to_string(time_t t, string format = "");

	int abs_year_day_to_utc_day(int years, int days);

	unsigned int utc_days_milliseconds_to_seconds(unsigned int days,
												  unsigned int milliseconds);

	string utc_days_milliseconds_to_string(unsigned int days,
										   unsigned int milliseconds,
										   string format = "");

	string abs_year_day_millisecond_to_string(
		int years,
		int days,
		int milliseconds);

	bool scan_gen_time(const string& gen_time, int& days, int& milliseconds);

	double utc_days_milliseconds_to_matlab(unsigned int days, unsigned int milliseconds);

	long long duration_to_int(const string& duration);
}

// Only allow spinner functionality if Python support is disabled
#ifndef PYMETOPIZER
#include "spinner.hpp"
extern auto_ptr<Spinner> spinner;
#endif

extern int gerror; // global error

class ifstream64
{
public:

	ifstream64(const char* filename);
	ifstream64(FILE* f);
	~ifstream64(void);
	void read(char* buffer, streampos_type length);
	bool good(void) const;
	operator bool (void);
	bool eof(void) const;
	void seekg(streampos_type pos, ios::seekdir d = ios::beg);
	streampos_type tellg(void) const;
	streampos_type gcount(void) const;

private:

	FILE* file;
	streampos_type last_read;
	bool owner;
};

class ofstream64
{
public:

	ofstream64(const char* filename);
	ofstream64(FILE* f);
	~ofstream64(void);
	void write(const char* buffer, unsigned int length);
	streampos_type tellp(void) const;
	void seekg(streampos_type pos, ios::seekdir d = ios::beg);
	bool fail(void) const;

	streampos_type get_total_written(void) const;

private:

	FILE* file;
	bool owner;
	bool fail_bit;
	streampos_type total_written;
};

typedef deque<string> StringList;

class DelimitedList : public StringList
{
public:
	DelimitedList(char delimiter, string in);
};

ostream& operator << (ostream& os, const StringList& sl);

/// Convert POD values between string and binary representations

template <typename TargetT, typename SourceT> TargetT lexical_cast(
	const SourceT& arg)
{
	stringstream str;
	str << arg;
	TargetT result;
	str >> result;
	return result;
}

StringList make_glob(const string& spec);

//#define PY3K

#endif //COMMON_H
