////////////////////////////////////////////////////////////////////////////////
//
//	File: cvcdutovcdu.hpp
//
//	Provides: CVCDUToVCDU filter
//
//	Purpose: Convert a stream of CVCDU packets to VCDU packets
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef cvcdutovcdu_defined
#define cvcdutovcdu_defined

#include "cvcduproducer.hpp"
#include "vcduproducer.hpp"
#include "mpduproducer.hpp"

class CVCDUToVCDU :
	virtual public Interface,
	public VCDUProducer,
	public MPDUProducer
{
public:

	CVCDUToVCDU(CVCDUProducer* _source,
				bool _strip_filler = true,
				bool _strip_encrypt = true,
				bool _rs_checks = true);

	virtual void output_text_summary( ostream& os = cout);
	virtual VCDU* get_next_vcdu(void);
	VCDU* cvcdu_to_vcdu(CVCDU* cvcdu);
	virtual MPDU* get_next_mpdu(void);

private:

	CVCDUProducer* source;

	int packet_count;
	int filler_count;
	int encrypt_count;
	int initial_filler;
	int final_filler;

	bool strip_filler;
	bool strip_encrypt;
	bool rs_checks;

	typedef map<unsigned int, int> SCIDCount;
	SCIDCount scid_count;

	int rs_good_packets;
	int rs_corrected_packets;
	int rs_bad_packets;
	int rs_total_errors;

	int version_warnings_count;
};

#endif
