////////////////////////////////////////////////////////////////////////////////
//
//	File: epsproductextractor.hpp
//
//	Provides: EPSProductExtractor extractor
//
//	Purpose: Convert product objects into binary files for storage
//
//	Copyright (c) EUMETSAT 2004
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef epsproductextractor_defined
#define epsproductextractor_defined

#include "mphr.hpp"
#include "fileproducer.hpp"
#include "epsproductproducer.hpp"

class EPSProductExtractor :
	public FileProducer
{
public:

	EPSProductExtractor(EPSProductProducer& _source)
		: source(_source)
		{
		}

	virtual bool produce_files( FileConsumer& dest )
		{
			if (!spinner.get())
			{
				spinner.reset(new Spinner());
			}

			ostream_type* os = dest.get_output_stream();

			// reserve space for MPHR and IPRs
			off_t skip_length = MPHR_body::SIZE +
				source.get_sphr_size() +
				source.get_ipr_count()*IPR::SIZE;

			os->seekg(skip_length);

			byte* record;
			int length;

			// write GEADRs
			while(source.make_geadr(record,length))
			{
				os->write(reinterpret_cast<char*>(record), length);
			}

			// write GIADRs
			while(source.make_giadr(record,length))
			{
				os->write(reinterpret_cast<char*>(record), length);
			}

			// write GEADRs
			while(source.make_geadr(record,length))
			{
				os->write(reinterpret_cast<char*>(record), length);
			}

			// write GIADRs
			while(source.make_giadr(record,length))
			{
				os->write(reinterpret_cast<char*>(record), length);
				spinner->tick();
			}

			// write VEADRs
			while(source.make_veadr(record,length))
			{
				os->write(reinterpret_cast<char*>(record), length);
			}

			// write VIADRs
			while(source.make_viadr(record,length))
			{
				os->write(reinterpret_cast<char*>(record), length);
				spinner->tick();
				delete[] record;
			}

			// write MDRs
			while(source.make_mdr(record,length))
			{
				os->write(reinterpret_cast<char*>(record), length);
				spinner->tick();
				delete[] record;
			}

			off_t filesize = os->tellp();

			// write SPHR
			os->seekg(MPHR_body::SIZE);
			if (source.make_sphr(record,length))
			{
				os->write(reinterpret_cast<char*>(record), length);
				delete[] record;
			}

			// write IPRs
			while(source.make_ipr(record,length))
			{
				os->write(reinterpret_cast<char*>(record), length);
			}

			// write MPHR
			if (source.make_mphr(record,length))
			{
				os->seekg(0);
				os->write(reinterpret_cast<char*>(record), length);
				delete[] record;
			}

			// reset the write pointer so the SingleFileWriter
			// report is accurate
			os->seekg(filesize);

			spinner->end();

			return true;
		}

	virtual istream_type* get_next_file(void)
		{
			return NULL;
		}


private:

	EPSProductProducer& source;

};

#endif
