////////////////////////////////////////////////////////////////////////////////
//
//	File: filereader.cpp
//
//	Provides: FileReader filter
//
//	Purpose: Open one of more files and pass ostreams to the next filter
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "filereader.hpp"

FileReader::FileReader(const string& name)
	: good_files(0),
	  bad_files(0),
	  total_read(0),
	  file_valid(false)
{
	if (name=="")
	{
		return;
	}
	size_t sp = name.find_first_not_of(" ");
	if (sp != string::npos)
	{
		string name2 = name.substr(sp);
		filenames.push_back(name2);
	}
	else
	{
		filenames.push_back(name);
	}
	pos = filenames.begin();
}

FileReader::FileReader(const deque<string>& names)
	: filenames(names)
	, good_files(0)
	, bad_files(0)
	, total_read(0)
{
	pos = filenames.begin();
}

void FileReader::output_text_summary( textoutstream_type& os)
{
//    if (in_file.get() && in_file->good())
//    {
//        if (static_cast<int>(in_file->tellg()) == -1)
//        {
//            // we hit eof
//            total_read += current_file_size;
//        }
//        else
//        {
//            // client didn't read all of their file
//            total_read += in_file->tellg();
//        }
//    }

	os << "\nFileReader report:\n"
	   << "	   " << good_files << " files opened successfully\n";
	if (bad_files > 0)
	{
		os << "	   " << bad_files << " files could not be read\n";
	}
	os << endl;
    //os << "	 " << total_read << " bytes read in total\n"
    //	 << endl;
}

//
// get_next_file
//

istream_type* FileReader::get_next_file(void)
{
	if (filenames.empty() || pos == filenames.end())
	{
		debug << "filereader::get_next_file hit end of files\n";
		return NULL;
	}

	// record number of bytes read
    if (in_file.get())
    {
        if (in_file->good())
        {
            if (static_cast<int>(in_file->tellg()) == -1)
            {
                // we've hit eof
                total_read += current_file_size;
            }
            else
            {
                // the client didn't read all of the file
                total_read += in_file->tellg();
            }
        }
	}

	in_file.reset(new ifstream_type((*pos).c_str()));

//	  cout << "Reading " << (*pos) << endl;

	// if open failed try the next file
	if (!in_file->good())
	{
		bad_files++;
		file_valid = false;
#ifndef PYMETOPIZER
		error << "Could not open input file '" << (*pos) << "'\n";
#endif
		pos++;
		return get_next_file();
	}
	else
	{
		file_valid = true;
	}

	// record file size
	in_file->seekg(0,ios::end);
	current_file_size = in_file->tellg();
	in_file->seekg(0,ios::beg);

	// ok
	good_files++;

	//debug << "Filereader::get_next_file returning " 
//	  << ((unsigned int)&in_file) 
//	  << endl;

	pos++;
	return in_file.get();
}

bool FileReader::produce_files( FileConsumer& )
{
	return false;
}

// reset file pointer to beginning
void FileReader::reset_files(void)
{
	pos = filenames.begin();
	if ( in_file.get() )
	{
		// seek file back to the begining
		in_file.reset();
	}
}
