////////////////////////////////////////////////////////////////////////////////
//
//	File: filereader.hpp
//
//	Provides: FileReader filter
//
//	Purpose: Open one of more files and pass ostreams to the next filter
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef filereader_defined
#define filereader_defined

#include "fileproducer.hpp"
#include "textsummaryproducer.hpp"

/// Filter to handle all file reading

class FileReader : public FileProducer
{
public:

	/// Constructor
	FileReader(const string& names);
	FileReader(const deque<string>& names);

	/// Display summary of actions
	virtual void output_text_summary( ostream& os = cout );

	/// Make pointer to next file
	virtual istream_type* get_next_file(void);

	/// Send all input files to client
	virtual bool produce_files( FileConsumer& );

	// reset file pointer to beginning
	virtual void reset_files(void);

	// Check if last open file attempt was valid
	virtual bool is_valid(void) { return file_valid; }
private:

	/// List of files passed by the user
	deque<string> filenames;

	/// Current file index in 'filenames'
	deque<string>::iterator pos;

	/// Remember the last file created o we can free it
	auto_ptr<ifstream_type> in_file;

	/// Current filesize
	unsigned long long current_file_size;

	/// Reporting; count of files opened
	int good_files;

	/// Reporting; count of files not readable
	int bad_files;

	/// Reporting; count of bytes in input files
	unsigned long long total_read;

	// flag to check if last open file was valid
	bool file_valid;
};

#endif
