////////////////////////////////////////////////////////////////////////////////
//
//	File: imagewriter.hpp
//
//	Provides: General graphics output class.
//
//	Purpose:
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef imagewriter_defined
#define imagewriter_defined

#include "image.hpp"
#include "tiff.hpp"

#ifndef _NO_JPEG
#include "jpeg.hpp"
#endif

#include "interface.hpp"

class ImageWriter : public Interface
{
public:

	ImageWriter(ImageProducer& source,
				const string& filename,
				Image::Format format)
		{
			source.prepare_images();

			for(int i=0; i<source.get_image_count(); i++)
			{
				Image* image;

				switch(format)
				{
				case Image::IMAGE_TIFF:
					image = new TIFF;
					break;
				case Image::IMAGE_JPEG:
#ifndef _NO_JPEG
					image = new JPEG;
#endif
					break;
				}

				image->set_filename_prefix(filename);
				// should call set_attributes, begin, multiple write lines
				// then end
				source.write_image(i, image);
				files.push_back(image->get_full_filename());
			}
		}

	void output_text_summary(ostream& os = cout)
		{
			os << "ImageWriter report:\n"
			   << "	   Wrote " << files.size() << " files\n";
			for(StringList::iterator iter=files.begin();
				iter!=files.end();
				iter++)
			{
				os << "		   " << (*iter) << " (";
				FILE* file = fopen((*iter).c_str(),"r");
				fseek(file, 0, SEEK_END);
				os << ftell(file) << " bytes)\n";
				fclose(file);
			}
			os << endl;
		}

private:

	StringList files;
};

#endif
