////////////////////////////////////////////////////////////////////////////////
//
//	File: jpeg.hpp
//
//	Provides: write JPEG files using libjpeg
//
//	Purpose:
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef _NO_JPEG

#ifndef jpeg_defined
#define jpeg_defined

#pragma pack(8)
#include "jpeglib.h"
#include <dlfcn.h>
#pragma pack(1)

class JPEG : public Image
{
public:

	virtual string get_filename_extension(void) const
		{
			return ".jpeg";
		}

	virtual void begin_output(void)
		{
			// find library
			so = dlopen("libjpeg.so", RTLD_LAZY);
			if (!so)
			{
				error << "Libjpeg not found. Make sure the LD_LIBRARY_PATH"
					  << " variable includes the directory containing libjpeg.so*\n";
				return;
			}

			jpeg_CreateCompress = (fn_jpeg_CreateCompress)dlsym
				(so,"jpeg_CreateCompress");
			if (!jpeg_CreateCompress)
			{
				cerr << "Could not lock function jpeg_CreateCompress. libjpeg "
					 << "is not usable.\n";
				return;
			}

			jpeg_stdio_dest = (fn_jpeg_stdio_dest)dlsym
				(so, "jpeg_stdio_dest");
			if (!jpeg_stdio_dest)
			{
				cerr << "Could not lock function jpeg_stdio_dest\n";
				return;
			}

			jpeg_set_defaults = (fn_jpeg_set_defaults)
				dlsym(so, "jpeg_set_defaults");
			if (!jpeg_set_defaults)
			{
				cerr << "Could not lock function jpeg_set_defaults\n";
				return;
			}

			jpeg_start_compress = (fn_jpeg_start_compress)dlsym
				(so, "jpeg_start_compress");
			if (!jpeg_start_compress)
			{
				cerr << "Could not lock function jpeg_start_compress\n";
				return;
			}

			jpeg_write_scanlines = (fn_jpeg_write_scanlines)dlsym
				(so, "jpeg_write_scanlines");
			if (!jpeg_write_scanlines)
			{
				cerr << "Could not lock function jpeg_write_scanlines\n";
				return;
			}

			jpeg_finish_compress = (fn_jpeg_finish_compress)dlsym
				(so, "jpeg_finish_compress");
			if (!jpeg_finish_compress)
			{
				cerr << "Could not lock function jpeg_finish_compress\n";
				return;
			}

			jpeg_std_error = (fn_jpeg_std_error)dlsym
				(so, "jpeg_std_error");
			if (!jpeg_std_error)
			{
				cerr << "Could not lock function jpeg_std_error\n";
				return;
			}

			file = fopen(get_full_filename().c_str(), "wb");
			if (!file)
			{
				error << "JPEG cannot open file '" << get_full_filename()
					  << "'\n";
				return;
			}

			cinfo.err = jpeg_std_error(&jerr);
			jpeg_create_compress(&cinfo);
			jpeg_stdio_dest(&cinfo, file);

			cinfo.image_width = width;
			cinfo.image_height = height;
			cinfo.in_color_space = (colour_type==IMAGE_COLOUR) ?
				JCS_RGB : JCS_GRAYSCALE;
			cinfo.input_components = (colour_type==IMAGE_COLOUR) ? 3 : 1;

			jpeg_set_defaults(&cinfo);

			jpeg_start_compress(&cinfo, TRUE);
		}

	virtual void write_line(byte* ptr)
		{
			JSAMPROW row[1];
			row[0] = ptr;
			jpeg_write_scanlines(&cinfo, row, 1);
		}

	virtual void end_output(void)
		{
			jpeg_finish_compress(&cinfo);

			fclose(file);
			dlclose(so);
		}

private:

	FILE* file;
	struct jpeg_compress_struct cinfo;
	struct jpeg_error_mgr jerr;

	// exported functions
	void* so;
	typedef void (*fn_jpeg_CreateCompress)(j_compress_ptr,int,size_t);
	typedef void (*fn_jpeg_stdio_dest)(j_compress_ptr,FILE*);
	typedef void (*fn_jpeg_set_defaults)(j_compress_ptr);
	typedef void (*fn_jpeg_start_compress)(j_compress_ptr,boolean);
	typedef void (*fn_jpeg_write_scanlines)(j_compress_ptr,JSAMPARRAY,JDIMENSION);
	typedef void (*fn_jpeg_finish_compress)(j_compress_ptr);
	typedef struct jpeg_error_mgr* (*fn_jpeg_std_error)(struct jpeg_error_mgr*);

	fn_jpeg_CreateCompress jpeg_CreateCompress;
	fn_jpeg_stdio_dest jpeg_stdio_dest;
	fn_jpeg_set_defaults jpeg_set_defaults;
	fn_jpeg_start_compress jpeg_start_compress;
	fn_jpeg_write_scanlines jpeg_write_scanlines;
	fn_jpeg_finish_compress jpeg_finish_compress;
	fn_jpeg_std_error jpeg_std_error;

};

#endif

#endif
