#ifndef MMAM_HPP
#define MMAM_HPP

#include <string>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#include "osvs.hpp"


using std::string;


class MMAMhandler
{
public:
    MMAMhandler(bool aGenerateTLEfile=false);
    ~MMAMhandler();

public:


    bool parseFromBuf(char* aMmamBuffer, unsigned int aSourceLen);
    bool parseFromFile(const string& m_aFileName);
    void generateMMAMfileFromBuf(const string &outdir);
    void generateTLEfile(const string &outdir);

private:


    bool tagNameMatchesText(const xmlChar* aTagName, const char* aText);
    string getNodeAttribute(xmlNodePtr aNode, const std::string &aName);
    string getNodeValue(xmlNodePtr aNode);


public:

    bool parseMMAM();
    void parseMessage (xmlNodePtr aNode);
    void parseTLEs (xmlNodePtr cur);
	void parseStateVectorAtANX(xmlNodePtr cur);
	void parseStateVector(xmlNodePtr cur);
    void parseObtUtcCorrelation(xmlNodePtr cur);
    bool schemaValidation(const string &aSchema);

public:

    string getUTC0();
    string getOBT0();
    string getClockStep();
    string getWrapAroundTime();

    bool checkTimeValidity(const short_cds_time& aTimestamp);
	OSVS getStateVectors();
	OSVS getStateVectorsAtANX();

private:

    xmlDocPtr  m_xmlDoc;
    char* m_MMAMbuf;
    unsigned int m_MMAMbufSize;
    string m_issueNumber;
    string m_spacecraftID;
    bool m_generateTLE;
    string m_TLElines;
    OSVS m_osvs;
	OSVS m_osvs_anx;

    short_cds_time m_mmamValidityFrom;
    short_cds_time m_mmamValidityUntil;

    //OBT UCT correlation
    string m_UTC0;
    string m_CCU_OBT0;
    string m_ClockStep;
    string m_WrapAroundTime;

};

#endif // MMAM_HPP
