////////////////////////////////////////////////////////////////////////////////
//
//	File: mpdu.hpp
//
//	Provides: MPDU struct
//
//	Purpose:
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef mpdu_defined
#define mpdu_defined

struct MPDU
{
	enum { PAYLOAD_SIZE = 882 };

	byte header[2];

	unsigned int get_byte(int pos) const
		{
			return static_cast<unsigned int>(header[pos]);
		}

	void set_byte(int pos, int value)
		{
			header[pos] = value;
		}

	unsigned int get_spare(void) const
		{
			return (get_byte(0)>>3);
		}

	void set_spare(int spare)
		{
			set_byte(0, spare&0x31);
		}

	unsigned int get_first_header_pointer(void) const
		{
			return ( ((get_byte(0)&7)<<8) | (get_byte(1)) );
		}

	void set_first_header_pointer(int _header)
		{
			set_byte(0, (get_byte(0)&0xf8)|(_header>>8));
			set_byte(1, _header&0xff);
		}

	string generate_a_checksum(void) const
		{
			unsigned int c = 0;

			int shift = 0;

			for(int i=0; i<PAYLOAD_SIZE; i++)
			{
				unsigned int value = payload[i];
				for(int j=0; j<shift; j++)
				{
					unsigned int carry = value >> 31;
					value <<= 1;
					if (carry != 0)
					{
						value ++;
					}
				}

				c += value;
				shift = (shift+1)%32;
			}

			string res = ntohs(c&255) + ntohs((c>>8)&255) +
				ntohs((c>>16)&255) + ntohs((c>>24)&255);

			return ("0x" + res);
		}

	byte payload[PAYLOAD_SIZE];
};

#endif
