////////////////////////////////////////////////////////////////////////////////
//
//	File: mphr.hpp
//
//	Provides: misc structs
//
//	Purpose: Code for representing MPHR records
//
//	Copyright (c) 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef mphr_defined
#define mphr_defined

#include "eps.hpp"
#include "osvs.hpp"
#include "subsat.hpp"

// these can't be enums inside MPHR_body as some are > 2byte

namespace MPHR
{
    const int SEMI_MAJOR_AXIS_SF = 1;
	const int ECCENTRICITY_SF = 1000000;
	const int INCLINATION_SF = 1000;
	const int PERIGEE_ARGUMENT_SF = 1000;
	const int RIGHT_ASCENSION_SF = 1000;
	const int MEAN_ANOMOLY_SF = 1000;
    const int POSITION_SF = 1000;
    const int VELOCITY_SF = 1000;
	const int SUBSAT_SF = 1000;
	const int EARTH_SUN_SF = 1000000;
}

struct MPHR_body
{
	enum { SIZE = 3307 };

	//20
//	  REC_HEAD record_header;

	//100
	char product_name_label[32];
	char product_name[68];

	//100
	char parent_product_name_1_label[32];
	char parent_product_name_1[68];

	//100
	char parent_product_name_2_label[32];
	char parent_product_name_2[68];

	//100
	char parent_product_name_3_label[32];
	char parent_product_name_3[68];

	//100
	char parent_product_name_4_label[32];
	char parent_product_name_4[68];

	//37
	char instrument_id_label[32];
	char instrument_id[5];

	//36
	char instrument_model_label[32];
	char instrument_model[4];

	//36
	char product_type_label[32];
	char product_type[4];

	//35
	char processing_level_label[32];
	char processing_level[3];

	//36
	char spacecraft_id_label[32];
	char spacecraft_id[4];

	//48
	char sensing_start_label[32];
	char sensing_start[16];

	//48
	char sensing_end_label[32];
	char sensing_end[16];

	//48
	char sensing_start_theoretical_label[32];
	char sensing_start_theoretical[16];

	//48
	char sensing_end_theoretical_label[32];
	char sensing_end_theoretical[16];

	//37
	char processing_centre_label[32];
	char processing_centre[5];

	//38
	char processor_major_version_label[32];
	char processor_major_version[6];

	//38
	char processor_minor_version_label[32];
	char processor_minor_version[6];

	//38
	char format_major_version_label[32];
	char format_major_version[6];

	//38
	char format_minor_version_label[32];
	char format_minor_version[6];

	//48
	char processing_time_start_label[32];
	char processing_time_start[16];

	//48
	char processing_time_end_label[32];
	char processing_time_end[16];

	//34
	char processing_mode_label[32];
	char processing_mode[2];

	//34
	char disposition_mode_label[32];
	char disposition_mode[2];

	//36
	char receiving_ground_station_label[32];
	char receiving_ground_station[4];

	//48
	char receive_time_start_label[32];
	char receive_time_start[16];

	//48
	char receive_time_end_label[32];
	char receive_time_end[16];

	//38
	char orbit_start_label[32];
	char orbit_start[6];

	//38
	char orbit_end_label[32];
	char orbit_end[6];

	//44
	char actual_product_size_label[32];
	char actual_product_size[12];

	//51
	char state_vector_time_label[32];
	char state_vector_time[19];

	//44
	char semi_major_axis_label[32];
	char semi_major_axis[12];

	//44
	char eccentricity_label[32];
	char eccentricity[12];

	//44
	char inclination_label[32];
	char inclination[12];

	//44
	char perigee_argument_label[32];
	char perigee_argument[12];

	//44
	char right_ascension_label[32];
	char right_ascension[12];

	//44
	char mean_anomaly_label[32];
	char mean_anomaly[12];

	//44
	char x_position_label[32];
	char x_position[12];

	//44
	char y_position_label[32];
	char y_position[12];

	//44
	char z_position_label[32];
	char z_position[12];

	//44
	char x_velocity_label[32];
	char x_velocity[12];

	//44
	char y_velocity_label[32];
	char y_velocity[12];

	//44
	char z_velocity_label[32];
	char z_velocity[12];

	//44
	char earth_sun_distance_ratio_label[32];
	char earth_sun_distance_ratio[12];

	//44
	char location_tolerance_radial_label[32];
	char location_tolerance_radial[12];

	//44
	char location_tolerance_crosstrack_label[32];
	char location_tolerance_crosstrack[12];

	//44
	char location_tolerance_alongtrack_label[32];
	char location_tolerance_alongtrack[12];

	//44
	char yaw_error_label[32];
	char yaw_error[12];

	//44
	char roll_error_label[32];
	char roll_error[12];

	//44
	char pitch_error_label[32];
	char pitch_error[12];

	//44
	char subsat_latitude_start_label[32];
	char subsat_latitude_start[12];

	//44
	char subsat_longitude_start_label[32];
	char subsat_longitude_start[12];

	//44
	char subsat_latitude_end_label[32];
	char subsat_latitude_end[12];

	//44
	char subsat_longitude_end_label[32];
	char subsat_longitude_end[12];

	//35
	char leap_second_label[32];
	char leap_second[3];

	//48
	char leap_second_utc_label[32];
	char leap_second_utc[16];

	//39
	char total_records_label[32];
	char total_records[7];

	//39
	char total_mphr_label[32];
	char total_mphr[7];

	//39
	char total_sphr_label[32];
	char total_sphr[7];

	//39
	char total_ipr_label[32];
	char total_ipr[7];

	//39
	char total_geadr_label[32];
	char total_geadr[7];

	//39
	char total_giadr_label[32];
	char total_giadr[7];

	//39
	char total_veadr_label[32];
	char total_veadr[7];

	//39
	char total_viadr_label[32];
	char total_viadr[7];

	//39
	char total_mdr_label[32];
	char total_mdr[7];

	//39
	char count_degraded_inst_mdr_label[32];
	char count_degraded_inst_mdr[7];

	//39
	char count_degraded_proc_mdr_label[32];
	char count_degraded_proc_mdr[7];

	//39
	char count_degraded_inst_mdr_blocks_label[32];
	char count_degraded_inst_mdr_blocks[7];

	//39
	char count_degraded_proc_mdr_blocks_label[32];
	char count_degraded_proc_mdr_blocks[7];

	//41
	char duration_of_product_label[32];
	char duration_of_product[9];

	//41
	char milliseconds_of_data_present_label[32];
	char milliseconds_of_data_present[9];

	//41
	char milliseconds_of_data_missing_label[32];
	char milliseconds_of_data_missing[9];

	//34
	char subsetted_product_label[32];
	char subsetted_product[2];
};

string fill_mphr(REC_HEAD& rec_head,
			   MPHR_body& mphr,
			   const string& config_file,
			   const string& instrument_id,
			   const short_cds_time& sensing_start,
			   const short_cds_time& sensing_end,
			   int total_sphr,
			   int total_ipr,
			   int total_geadr,
			   int total_giadr,
			   int total_veadr,
			   int total_viadr,
			   int total_mdr,
			   int actual_product_size,
			   const Subsat::Pos& start_pos,
			   const Subsat::Pos& end_pos,
			   const OSVS::OSV &osv_start,
			   const OSVS::OSV &osv_end);


void display_mphr_config(const string& config_file);


#endif

