//	File: multifilewriter.cpp
//
//	Provides: MultiFileWriter filter
//
//	Purpose: Write a set of files to disk
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "multifilewriter.hpp"

MultiFileWriter::MultiFileWriter(
	FileProducer& source,
	const string& _filename_base)
	: filename_base(_filename_base)
	, file_count(0)
{
	debug << "multifilewriter constructor\n";

	if (!source.produce_files(*this))
	{
		debug << "multifilewriter trying get_next_file method\n";
		ofstream_type ofs(filename_base.c_str());
		const int BUFFER_SIZE = 128*1024;
		char buffer[BUFFER_SIZE];

		while(istream_type* is = source.get_next_file())
		{
//			debug << "singlefilewriter got file\n";
			while(!is->eof())
			{
				is->read(buffer, BUFFER_SIZE);

				streampos old_pos = ofs.tellp();
				ofs.write(buffer, is->gcount());
				if (static_cast<unsigned int>(old_pos) +
					static_cast<unsigned int>(is->gcount())
					!= static_cast<unsigned int>(ofs.tellp()) || ofs.fail())
				{
					error << "WRITE ERROR OCCURRED - output file "
						  << "may be corrupt\n";
				}
			}
		}
	}

}

struct deletor : public unary_function<map<string,ofstream_type*>::
	value_type, void>
{
	void operator()(const map<string,ofstream_type*>::value_type& x)
		{
			delete x.second;
		}
};

MultiFileWriter::~MultiFileWriter(void)
{
	for_each(ostreams.begin(), ostreams.end(), deletor());
}

void MultiFileWriter::output_text_summary( ostream& os )
{
	off_t byte_count = 0;
	for(OStreams::iterator iter=ostreams.begin();
		iter!=ostreams.end(); iter++)
	{
		//cout << "tellp " << (*iter).second->tellp();
		byte_count += (*iter).second->tellp();
	}

	os << "MultiFileWriter report:\n"
	   << "	   " << file_count << " files written\n"
	   << "	   " << byte_count << " bytes written\n\n";
}

ostream_type* MultiFileWriter::get_named_output_stream(
	const string& filename_suffix)
{
	debug << "multifilewriter:: get_named_output_stream '"
		  << filename_suffix << "'\n";
	OStreams::iterator iter = ostreams.find(
		string(string(string("x")+filename_suffix)));
	if (iter==ostreams.end())
	{
		debug << "multifilewriter new file\n";
		ofstream_type* result = new ofstream_type(
			string(filename_base+filename_suffix).c_str());
		ostreams[string("x")+filename_suffix] = result;
		file_count++;
		return result;
	}
	else
	{
		debug << "multifilewriter using existing file\n";
		return (*iter).second;
	}
}

ostream_type* MultiFileWriter::get_output_stream(void)
{
	if (!single_file_output.get())
	{
		single_file_output.reset(
			new ofstream_type(filename_base.c_str()));
		file_count++;
	}
	return single_file_output.get();
}

string MultiFileWriter::get_filename_base(void)
{
	return filename_base;
}
