////////////////////////////////////////////////////////////////////////////////
//
//	File: noaa-v3.hpp
//
//	Provides: Updated structures for the NOAA N v3 formats introduced
//			  January 2005.
//
//	Purpose:
//
//	Copyright (c) 2005 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef noaa_v3_defined
#define noaa_v3_defined

//
// Archive Header
//

struct NOAA_Archive_Header_v3
{
	enum { SIZE = 512 };

	struct // size 30
	{
		char cost_number[6];
		char class_number[8];
		char year[4];
		char day[3];
		char processing_site;
		char processing_software[8];
	} order_id;

	struct // 89
	{
		char dataset_name[42];
		char space[2];
		char select_flag; // T=all, S=subset
		char beginning_latitude[3];
		char ending_latitude[3];
		char beginning_longitude[4];
		char ending_longitude[4];
		char start_hour[2];
		char start_minute[2];
		char number_of_minutes[3];
		char appended_data_flag;
		char channel_select_flags[20]; // Y/N
		char sensor_data_word_size[2];
	} query;

	struct // 48
	{
		char space[27];
		char direction_flag; // Ascend/Descend/Both
		char first_latitude[3];
		char last_latitude[3];
		char first_longitude[4];
		char last_longitude[4];
		char data_format[20];
		char record_size[6];
		char number_of_records[6];
	} data;

	char filler[319];
};

//
// Generic
//

struct NOAA_L1b_GEN_Header_v3
{
	enum { SIZE = 116 };
	enum DataTypeCode {DTC_Invalid=0, DTC_LAC, DTC_GAC, DTC_HRPT, DTC_TIP,
					   DTC_HIRS, DTC_MSU, DTC_SSU, DTC_DCS, DTC_SEM, DTC_AMSUA,
					   DTC_AMSUB, DTC_MHS, DTC_FRAC};

	// 0
	char creation_site[3];
	char space;

	// 4
	uint16_be level1b_format_version_number;
	uint16_be level1b_format_version_year;
	uint16_be level1b_format_version_day;
	uint16_be logical_record_length;
	uint16_be block_size;
	uint16_be count_of_header_records;

	// 16
	int16_be zero_1[3];
	char dataset_name[42];
	char processing_block_identification[8];

	// 72
	uint16_be scid; // 7=NOAA-N, 8=NOAA-N' ?? 2=NOAA16 4=NOAA15 ??
	uint16_be instrument_id; // 0=engineering EM,
	//						 1=proto flight model PFM NOAA-N
	//						 2=flight model FM NOAA-N'

	uint16_be data_type_code; // 1=LAC, 2=GAC, 3=HRPT, 4=TIP, 5=HIRS, 6=MSU, 7=SSU,
	// 8=DCS, 9=SEM, 10=AMSU-A, 11=AMSU-B, 12=MHS
	uint16_be tip_source_code;

	// 80
	uint32_be start_day; // days since 00:00 01/01/1950
	uint16_be start_year;
	uint16_be start_day_of_year;
	uint32_be start_millisecond;

	// 92
	uint32_be end_day; // days since 00:00 01/01/1950
	uint16_be end_year;
	uint16_be end_day_of_year;
	uint32_be end_millisecond;

	// 104
	uint16_be year_of_last_cpids_update;
	uint16_be day_of_year_of_last_cpids_update;
	int16_be zero_2[4];

	// 116
};

struct NOAA_L1b_MHS_Header_v3
{
	enum { SIZE = 3072 };

	// 0
	NOAA_L1b_GEN_Header_v3 gen;

	// 116
	int32_be space;

	// 120
	uint32_be instrument_status;

	// 124
	int16_be zero_fill;

	// 126
	uint16_be record_number_of_status_change;
	uint32_be second_instrument_status;

	// offset 132
	uint16_be num_data_records;
	uint16_be num_good_scan_lines;
	uint16_be num_missing_scan_lines;
	uint16_be num_data_gaps;
	uint16_be num_lunar_contaminated_space_views;
	uint16_be num_frames_without_sync_errors;
	uint16_be num_pacs_detected_tip_parity_errors;

	// offset 146
	int8 end_space[SIZE-146];
};

struct NOAA_L1b_MHS_Data_v3
{
	enum { SIZE = 3072 };

	// 0
	NOAA_L1b_GEN_Data gen;

	// 16
	uint32_be coarse_time;
	uint16_be fine_time;
	uint8 mhs_mode_flag;
	int8 zero_fill_0;

	// 24
	uint32_be quality_indicator;
	uint8 time_problem_flags;
	uint8 calib_problem_flags[2];
	uint8 earth_loc_problem_flags;
	uint16_be calibration_quality_flags[5];

	// 42
	int16_be zero_fill_1[9];

	// 60
	int8 calib_coeffs[136]; // [196-60]

	// 196
	int8 navigation_data[1284]; //[1480-196]

	struct Sample
	{
		// size is 12 bytes
		uint16_be position;
		uint16_be H[5];
	};

	// 1480
	Sample sensor_data[90];
	int32_be zero_fill_2[2];

	// 2568
	Sample space_view[4];

	// 2616
	Sample target_view[4];
	int32_be zero_fill_3[2];

	// 2672
	int8 position_validity_flags[12];
	int8 space_view_validity_flags;
	int8 obct_view_validity_flags;

	// 2686
	uint8 mode;
	uint8 telecommand[5];
	uint8 switch_status[3];
	uint8 temperature[24];
	uint8 current[6];
	int8 zero_fill_4;

	// 2726
	uint8 status_word;
	int8 zero_fill_5[7];

	// 2734
	uint8 offset_words[5];
	uint8 valid_flags;
	uint8 channel_gain[3];
	int8 zero_fill_6[7];

	// 2750
	uint16_be prt[5];
	uint16_be prt_cal[3];
	uint16_be zero_fill_7;
	uint32_be prt_temp[5];

	// 2788
	int8 science_spare[45];
	int8 zero_fill_8;

	// 2834
	uint8 discrete_telemetry[34];

	// 2868
	int16_be zero_fill_9[102];
};

#endif
