////////////////////////////////////////////////////////////////////////////////
//
//	File: noaaproductreader.hpp
//
//	Provides: NOAAProductReader filter
//
//	Purpose: Interpret a binary file as a NOAA L1b product
//
//	Copyright (c) 2002, 2003, 2004, 2006 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef noaaproductreader_defined
#define noaaproductreader_defined

#include "interface.hpp"
#include "noaa.hpp"
#include "noaa_v3.hpp"
#include "productproducer.hpp"
#include "fileproducer.hpp"

class NOAAProductReader :
	public Interface,
	public ProductProducer
{
public:

	/// constructor
	NOAAProductReader( FileProducer& source,
					   int user_bps = -1,
					   string user_channels = "" );

	/// destructor
	~NOAAProductReader(void);

	/// get_record_size
	virtual int get_record_size(void) const;

	/// output_text_summary
	void output_text_summary(ostream& os = cout);

	/// output_text_archive_header
	void output_text_archive_header(ostream& os = cout);

	/// output_text_gen_header
	void output_text_gen_header(ostream& os = cout);

	/// output_text_gen_record
	void output_text_gen_record(int record_no, ostream& os = cout);

	/// get_archive_header
	virtual NOAA_Archive_Header* get_archive_header(void);

	/// get_product_header
	virtual NOAA_L1b_GEN_Header* get_product_header(void);

	/// get_current_record
	virtual NOAA_L1b_GEN_Data* get_current_record(void) const;

	/// skip_records
	virtual void skip_records(unsigned int offset);

private:

	istream_type* is;

	auto_ptr<NOAA_Archive_Header> archive_header;
	byte* header;
	byte* buffer_1;
	byte* buffer_2;

	int block_size;
	bool user_specified_bps;
	bool user_specified_channels;
};

#endif
