////////////////////////////////////////////////////////////////////////////////
//
//	File: osvs.hpp
//
//	Purpose: Parse an OSVS xml file
//
//	Copyright (c) 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef osv_defined
#define osv_defined

#include "eps.hpp"

/// Database of orbit state vectors

class OSVS
{
public:

	/// Single state vector

	struct OSV
	{
		/// Constructor
		OSV(void);

		short_cds_time state_vector_time;
		double semi_major_axis;
		double eccentricity;
		double inclination;
		double perigee_argument;
		double right_ascension;
		double mean_anomaly;
		double x;
		double y;
		double z;
		double vx;
		double vy;
		double vz;
		int orbit_number;

		bool operator != (const OSV& o) const;
	};

	/// Constructor
	OSVS(void);

	/// Constructor
	OSVS(const string& filename);

	/// Return closest OSV before given time
	OSV get_osv(const short_cds_time& time) const;

	/// Return interpolated OSV for the given time
	OSV get_interpolated_osv(const short_cds_time& time) const;

    /// add new OSV
    void add_osv(short_cds_time cds, OSV o);

    /// DEBUG
    int size() const { return  osvs.size(); };

private:

	/// Database of times against OSVs
    map<short_cds_time, OSV> osvs;

	string filename;

	mutable bool warned;
};

ostream& operator << (ostream& os, const OSVS::OSV& osv);

#endif
