////////////////////////////////////////////////////////////////////////////////
//
//	File: pyamsuareader.hpp
//
//	Provides: python EPS Product reader class
//
//	Purpose: Code for representing source packets as Python Objects
//
//	Copyright (c) 2006 EUMETSAT
//
//	This file is part of Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef pyamsuareader_defined
#define pyamsuareader_defined

#include "pyreader.hpp"

// Forward declare methods
class CCSDSProducer;

////////////////////////////////////////////////////////////////////////////////
/// Specialisation for MHS Instrument with APID of 34
class PyAMSUAReader : public PyReader
{
public:
	PyAMSUAReader(CCSDSProducer* source);

	///	 return a list of strings ["Channel1". "Channel2", "Channel3", "Channel4", "Channel5"]
	PyObject* get_channels(void) const;

	/// instrument name for this reader
	string get_instrument_name(void) const;

	PyObject* get_samples(int channel, int band, int modulus, int count, int skip, int modulus2);

private:
	/// Encapsulate single source packet data for AMSUA packets
	struct SceneData
	{
		typedef vector<byte> SciData;

		SceneData(CCSDS* packet);
		PyObject* get_samples( int channel,int modulus = 1 );

		union
		{
			AMSUA1MeasurementScienceData* a1m;
			AMSUA1UniqueScienceData* a1u;
			AMSUA2MeasurementScienceData* a2m;
			AMSUA2UniqueScienceData* a2u;
		} data;

		enum { BAD,A1M,A1U,A2M,A2U } type;
		SciData sci_data;
	};
};

#endif
