///////////////////////////////////////////////////////////////////////////////
//
//	File: pyavhrrreader.hpp
//
//	Provides: python EPS Product reader class
//
//	Purpose: Code for representing source packets as Python Objects
//
//	Copyright (c) 2006 EUMETSAT
//
//	This file is part of Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef pyavhrr_reader_defined
#define pyavhrr_reader_defined

#include "pyreader.hpp"
#include "noaa.hpp"

// Forward declare methods
class CCSDSProducer;
typedef NOAA_L1b_AVHRR_Data::Sample AVHRR_Sample;

////////////////////////////////////////////////////////////////////////////////
/// Specialisation for MHS Instrument with APID of 34

class PyAVHRRReader : public PyReader
{
public:
	PyAVHRRReader(CCSDSProducer* source);

	///	 return a list of strings ["Channel1". "Channel2", "Channel3", "Channel4", "Channel5"]
	PyObject* get_channels(void) const;

	/// return a array of 90-element long arrays. optionalal 2nd parameter gives modulus [only every 5th sample included])
	PyObject* get_samples(int channel, int band, int modulus, int count, int skip, int modulus2);

	/// instrument name for this reader
	string get_instrument_name(void) const;

private:

	typedef vector<uint16_be> Samples;

	AVHRR_Sample read_avhrr_sample(byte*& payload_byte, int& payload_bit);
	int read_avhrr_bit(byte*& payload_byte, int& payload_bit);
};

#endif
