////////////////////////////////////////////////////////////////////////////////
//
//	File: pyhirsreader.hpp
//
//	Provides: python EPS Product reader class
//
//	Purpose: Code for representing source packets as Python Objects
//
//	Copyright (c) 2006 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef pyhirsreader_defined
#define pyhirsreader_defined

#include "pyreader.hpp"

// Forward declare classes
class CCSDSProducer;


////////////////////////////////////////////////////////////////////////////////
/// Specialisation for MHS Instrument with APID of 34
class PyHIRSReader : public PyReader
{
public:
	PyHIRSReader(CCSDSProducer* source);

	///	 return a list of strings ["Channel1". "Channel2", "Channel3", "Channel4", "Channel5"]
	PyObject* get_channels(void) const;

	/// return a array of 90-element long arrays. optionalal 2nd parameter gives modulus [only every 5th sample included])
	PyObject* get_samples(int channel, int band, int modulus, int count, int skip, int modulus2);

	/// instrument name for this reader
	string get_instrument_name(void) const;

private:

	enum {CHANNEL_COUNT = 20};
	enum {NUM_SAMPLES = 56};

	/// Store only channels data
	struct ChannelData
	{
		short channels[CHANNEL_COUNT];
	};

	ChannelData elements[NUM_SAMPLES];

	/// get all channel data per sample from each element
	ChannelData get_hirs_element(CCSDS_HIRS* packet, int element);
};

#endif
