////////////////////////////////////////////////////////////////////////////////
//
//	File: pyhktmreader.cpp
//
//	Provides: python EPS Product reader class
//
//	Purpose:
//
//	Copyright (c) 2006 EUMETSAT
//
//	This file is part of Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"

#include "pymetopizer.hpp"
#include "pyhktmreader.hpp"
#include "ccsds.hpp"

// Specialisation for HKTM Instrument
PyHKTMReader::PyHKTMReader( CCSDSProducer* source)
	: PyReader(source)
{
}

// get number of samples produced for a single scan line
int PyHKTMReader::get_samples_count(void)
{
	return -1;
}

//	return a list of strings ["Channel1". "Channel2", "Channel3", "Channel4", "Channel5"]
PyObject* PyHKTMReader::get_channels(void) const
{
	return Py_BuildValue("[(s,[s])]", "Packets", "Reduced nav (pos_x, pos_y, pos_z)");
}

// return an array of tracking state tuples
PyObject* PyHKTMReader::get_samples(int channel, int bands, int modulus, int count, int skip, int modulus2)
{
	// reset back to start of file cache
	reset_files();

	PyObject* lines = PyList_New(0);

	while ( CCSDS* packet = get_next_ccsds() )
	{
		unsigned int apid = packet->get_apid();

		// select output_gras_navigation_reduced_record	 ( AP_ID = 3)
		//if (apid != APID_REDUCED)
		if (apid != APID_NAV)
		{
			continue;
		}

		CCSDS_GRAS_NAVIGATION_Reduced* gras = reinterpret_cast<CCSDS_GRAS_NAVIGATION_Reduced*>(packet->get_data());

		store_times(packet);

		int pos_x = gras->basic_block.POS_X;
		int pos_y = gras->basic_block.POS_Y;
		int pos_z = gras->basic_block.POS_Z;

		//cout << "basic_block: " << gras->basic_block.UTC << " sec header: " << *utc
		//	<< "\t" << pos_x << "\t" << pos_y << "\t" << pos_z << endl;

		PyObject* loc = Py_BuildValue("iii", pos_x, pos_y, pos_z);
		PyList_Append(lines, loc);
		Py_DECREF(loc);

	}

	return lines;
}

// instrument name for this reader
string PyHKTMReader::get_instrument_name(void) const
{
	return string("HKTM");
}
