///////////////////////////////////////////////////////////////////////////////
//
//	File: pyiasieader.hpp
//
//	Provides: python interface
//
//	Purpose:
//
//	Copyright (c) 2006 EUMETSAT
//
//	This file is part of Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef pyiasireader_defined
#define pyiasireader_defined

#include "pyreader.hpp"
#include "brd.hpp"

// Forward declare methods
class CCSDSProducer;

////////////////////////////////////////////////////////////////////////////////

class PyIASIReader : public PyReader
{
public:
	PyIASIReader(CCSDSProducer* source);

	///	 return a list of strings
	PyObject* get_channels(void) const;

	PyObject* get_samples(int channel, int band, int modulus, int count, int skip, int modulus2);
	PyObject* get_samples_imager(int modulus, int count, int skip, int modulus2);
	PyObject* get_samples_sounder(int channel, int modulus, int count, int skip, int modulus2);

	/// instrument name for this reader
	string get_instrument_name(void) const;

private:

	// scan all possible directories for BRD files
	void scan_brd_dirs(void);

	// scan a single directory for BRD files
	void scan_brd_dir(const string& dir);

	enum { IIS_CELL_WIDTH = 64 };
	enum { IIS_CELL_HEIGHT = 64 };
	enum { IIS_LINE_WIDTH = IIS_CELL_WIDTH*30 };

	typedef int Sample;

	struct Samples
	{
		Sample samples[IIS_LINE_WIDTH][IIS_CELL_HEIGHT];
	};

	void push_samples(const UTCTime& base_utc_time,
					  const OBTTime& base_obt_time,
					  const short_cds_time& base_grh_time,
					  PyObject* result,
					  Samples* line,
					  int line_width,
					  int cell_height,
					  int modulus,
					  int modulus2);

	// scid,ptsi against BRD
	typedef map<pair<int, int>, BRD*> BRDs;
	static BRDs brds;
};

#endif
