////////////////////////////////////////////////////////////////////////////////
//
//	File: pymetopizer.hpp
//
//	Provides: python EPS Product handler class
//
//	Purpose: Code for representing source packets as Python Objects
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef pymetopizer_defined
#define pymetopizer_defined

#include "pyreader.hpp"

/// Global error handler for python specific exceptions
extern PyObject* PyMetopizerError;

/// Accepted time formats by which values are returned as doubles or strings
enum PY_TIME_TYPE { PY_TIME_BAD, PY_TIME_GENTIME, PY_TIME_MATLAB, PY_TIME_DATETIME };
/// Specify the time type to use when retrieving time values
extern PY_TIME_TYPE py_time_type;
extern string pydebug;

class _PyException : public stringstream
{
public:

	_PyException(PyObject* _type)
		: type(_type)
		{
		}

	~_PyException()
		{
			PyErr_SetString(type, str().c_str());
		}

	PyObject* type;
};

#define PyException(type,mess) {_PyException p(type); p<<mess;}

PyObject* make_pyobject(const StringList& l);
PyObject* make_pyobject(const string& l);
PyObject* make_pyobject(int i);
PyObject* make_pyobject(double d);
PyObject* make_pyobject(const UTCTime& utc_time);

#endif
