////////////////////////////////////////////////////////////////////////////////
//
//	File: pysemreader.cpp
//
//	Provides: python EPS Product reader class
//
//	Purpose:
//
//	Copyright (c) 2006 EUMETSAT
//
//	This file is part of Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"

#include "pymetopizer.hpp"
#include "pysemreader.hpp"
#include "ccsds.hpp"

#include <intobject.h>

// Specialisation for SEM Instrument
PySEMReader::PySEMReader( CCSDSProducer* source)
	: PyReader(source)
{
}

PyObject* PySEMReader::get_channels(void) const
{
	Py_INCREF(Py_None);
	return Py_None;
}

// get number of samples produced for a single scan line
int PySEMReader::get_samples_count(void)
{
	return -1;
}

PyObject* PySEMReader::get_samples(int channel, int bands, int modulus, int count, int skip, int modulus2)
{
	// reset back to start of file cache
	reset_files();

	PyObject* lines = PyList_New(0);

	while ( CCSDS* packet = get_next_ccsds() )
	{
		store_times(packet);

		byte* payload = packet->get_payload();

		for(int i=0; i<640; i+=40)
		{
			PyObject* line = PyList_New(40);
			for(int j=0; j<40; j++)
			{
				int val = (payload[i+j] ^ 0xff);
				PyList_SetItem(line, j, PyInt_FromLong(val));
			}
			PyList_Append(lines, line);
			Py_DECREF(line);
		}
	}

	return lines;
}

// instrument name for this reader
string PySEMReader::get_instrument_name(void) const
{
	return string("SEM");
}
