////////////////////////////////////////////////////////////////////////////////
//
//	File: common.hpp
//
//	Provides: Interface to libfec
//
//	Purpose: Load libfec.so and call functions from it
//
//	Copyright (c) 2011 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include <dlfcn.h>

#include "common.hpp"

void* _rs_library = NULL;

typedef void (*EncodeRSCCSDS)(unsigned char *data,unsigned char *parity,int pad);
EncodeRSCCSDS encode_rs_ccsds;

typedef int (*DecodeRSCCSDS)(unsigned char *data,int *eras_pos,int no_eras,int pad);
DecodeRSCCSDS decode_rs_ccsds;

void* get_rs_library()
{
	if (!_rs_library)
	{
		_rs_library = dlopen("libfec.so", RTLD_LAZY);
		if (!_rs_library)
		{
			fprintf (stderr, "%s\n", dlerror());
			cerr << "Cannot open Reed-Solomon library libfec (" << dlerror() << ")\n";
			exit(-1);
		}
		encode_rs_ccsds = reinterpret_cast<EncodeRSCCSDS>(dlsym(_rs_library, "encode_rs_ccsds"));
		if (!encode_rs_ccsds)
		{
			cerr << "Cannot find function encode_rs_ccsds\n";
			exit(-1);
		}
		decode_rs_ccsds = reinterpret_cast<DecodeRSCCSDS>(dlsym(_rs_library, "decode_rs_ccsds"));
		if (!decode_rs_ccsds)
		{
			cerr << "Cannot find function decode_rs_ccsds\n";
			exit(-1);
		}
	}
	return _rs_library;
}

void rs_encode(unsigned char *data,unsigned char *parity, int pad)
{
	// cout << "rs_encode called\n";
	get_rs_library();
	encode_rs_ccsds(data, parity, pad);
}

int rs_decode(unsigned char *data,int *eras_pos,int no_eras,int pad)
{
	// cout << "rs_decode called\n";
	get_rs_library();
	decode_rs_ccsds(data, eras_pos, no_eras, pad);
}
