////////////////////////////////////////////////////////////////////////////////
//
//	File: sband.cpp
//
//	Provides: misc structs
//
//	Purpose: Code for representing source packets
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "sband.hpp"


unsigned short calculate_crc(unsigned short in_crc, int in_length, byte* data)
{
	const int CCITT_CRC_GEN = 0x1021;
	//const int CCITT_CRC_GEN = 0xffff;
	//unsigned short crc = invert_initial_value ? 0x0 : 0xffff;
	unsigned short crc = in_crc;

	for(int j=0; j<in_length; j++)
	{
		/*	unsigned short tmp = data[j];
			unsigned short a = (tmp>>8)&255;
			unsigned short b = tmp&255;
			unsigned short dataByte = 0;
			dataByte |= (b << 8);
			dataByte |= a;
			dataByte = dataByte << 8;*/
		unsigned short dataByte = static_cast<unsigned short>(data[j]) << 8;
		for (int i = 8; i > 0; i--)
		{
			if ((dataByte ^ crc) & 0x8000)
			{
				crc = (crc << 1) ^ CCITT_CRC_GEN;
			}
			else
			{
				crc <<= 1 ;
			}
			dataByte <<= 1;
		}
	}
	return crc;
}


unsigned short SBand::check_crc(void) const
{
	//byte sync_marker[4] PACKED;
	//transferFrameHeader transfer_frame_header PACKED;
	//SBandPacketHeader packet_header PACKED;
	//COMMON_DATA common;
	//byte packet_data[470] PACKED;
	//transferFrameTrailer transfer_frame_trailer PACKED;

	unsigned short crc = 0xffff;
	//crc = calculate_crc(crc, int in_lenght, unsigned short data*)

	crc = calculate_crc(crc, 506, transfer_frame_header.get_data());

//	   crc = calculate_crc(crc, 3, transfer_frame_header.get_data());
//	   crc = calculate_crc(crc, 3, packet_header.get_data());
//	   crc = calculate_crc(crc, 10, (unsigned short*)common_data);
//	   crc = calculate_crc(crc, 235, (unsigned short*)packet_data);
//	   crc = calculate_crc(crc, 2, transfer_frame_trailer.get_data());

	//4 and not 6 becuase FECW not included in crc calculation


	return crc;
}



//
// transferFrameHeader
//

unsigned int transferFrameHeader::get_byte(int pos) const
{
	return static_cast<unsigned int>(data[pos]);
}

byte* transferFrameHeader::get_data(void) const
{
	return (byte*)data;
}

unsigned int transferFrameHeader::get_frame_id_version_numb(void) const
{
	return (get_byte(0)>>6);
}

unsigned int transferFrameHeader::get_frame_id_sl_id(void) const
{
	return ( ((get_byte(0)&63)<<8) + (get_byte(1)>>4) );
}

unsigned int transferFrameHeader::get_frame_id_vc_id(void) const
{
	return ((get_byte(1)>>1)&7);
}

unsigned int transferFrameHeader::get_frame_id_ocff(void) const
{
	return (get_byte(1)&1);
}

unsigned int transferFrameHeader::get_mc_frame_count(void) const
{
	return get_byte(2);
}

unsigned int transferFrameHeader::get_vc_frame_count(void) const
{
	return get_byte(3);
}

unsigned int transferFrameHeader::get_fdfs_sec_head_flag(void) const
{
	return (get_byte(4)>>7);
}

unsigned int transferFrameHeader::get_fdfs_sync_flag(void) const
{
	return ((get_byte(4)>>6)&1);
}

unsigned int transferFrameHeader::get_fdfs_order_flag(void) const
{
	return ((get_byte(4)>>5)&1);
}

unsigned int transferFrameHeader::get_fdfs_seg_len_id(void) const
{
	return ((get_byte(4)>>3)&3);
}

unsigned int transferFrameHeader::get_fdfs_first_head_pointer(void) const
{
	return ( ((get_byte(4)&7)<<8) + get_byte(5) );
}




//
// transferFrameTrailer
//

unsigned int transferFrameTrailer::get_byte(int pos) const
{
	return static_cast<unsigned int>(data[pos]);
}

unsigned short* transferFrameTrailer::get_data(void) const
{
	return (unsigned short*)data;
}

unsigned int transferFrameTrailer::get_clcw(void) const
{
	return ( (get_byte(0)<<24) + (get_byte(1)<<16) + (get_byte(2)<<8) + get_byte(3) );
}

unsigned int transferFrameTrailer::get_fecw(void) const
{
	return ( (get_byte(4)<<8) + get_byte(5) );
}





//
// SBandPacketHeader
//

unsigned int SBandPacketHeader::get_byte(int pos) const
{
	return static_cast<unsigned int>(data[pos]);
}

byte* SBandPacketHeader::get_data(void) const
{
	return (byte*)data;
}

unsigned int SBandPacketHeader::get_version_number(void) const
{
	return (get_byte(0)>>5);
}

unsigned int SBandPacketHeader::get_type(void) const
{
	return ((get_byte(0)>>4)&1);
}

unsigned int SBandPacketHeader::get_data_field_header_flag(void) const
{
	return ((get_byte(0)>>3)&1);
}

unsigned int SBandPacketHeader::get_apid(void) const
{
	return (((get_byte(0)&7)<<8)+get_byte(1));
}

unsigned int SBandPacketHeader::get_segment_flag(void) const
{
	return (get_byte(2)>>6);
}

unsigned int SBandPacketHeader::get_sequence_count(void) const
{
	return (((get_byte(2)&63)<<8)+get_byte(3));
}

unsigned int SBandPacketHeader::get_packet_length(void) const
{
	return ((get_byte(4)<<8)+get_byte(5));
}



//
// COMMON_DATA
//

double COMMON_DATA::get_ccu_time(void) const
{
	double tmp;
	tmp = static_cast<double>( (ccu_time[0]<<16) + (ccu_time[1]<<8) + ccu_time[2] );
	tmp = tmp + ( static_cast<double>(ccu_time[3]) / 256.0 );
	return tmp;
}

// unsigned short* COMMON_DATA::get_data(void) const
// {
//	   byte data[20];

//	   for (int i=0; i<4; i++)
//	   {
//	data[i] = ccu_time[i];
//	   }

//	   for (int i=4; i<7; i++)
//	   {
//	data[i] = sl_mode[i-4];
//	   }

//	   for (int i=7; i<19; i++)
//	   {
//	data[i] = svm_hk_data[i-7];
//	   }

//	   data[19] = svm_phase_no;

//	   return (unsigned short*)data;
// }



//
// SBand
//


int SBand::get_sync_marker(void) const
{
	return static_cast<int>( (sync_marker[0]<<24)+(sync_marker[1]<<16)
							 +(sync_marker[2]<<8)+(sync_marker[0]) );
}


COMMON_DATA* SBand::get_common_data(void) const
{
	return (COMMON_DATA*)common_data;
}


RTF_DATA* SBand::get_rtf_data(void) const
{
	return (RTF_DATA*)packet_data;
}


TEF_DATA* SBand::get_tef_data(void) const
{
	return (TEF_DATA*)packet_data;
}


double SBand::get_ccu_time(void) const
{
	return ((COMMON_DATA*)common_data)->get_ccu_time();
}


void SBand::create_text_output(textoutstream_type& os) const
{


	os << "\nSync Marker (Hex)	:  "
	   <<  ntohs(sync_marker[0])
	   <<  ntohs(sync_marker[1])
	   <<  ntohs(sync_marker[2])
	   <<  ntohs(sync_marker[3]);

	int syncMarker = static_cast<int>( (sync_marker[0]<<24)
									   + (sync_marker[1]<<16)
									   + (sync_marker[2]<<8)
									   + sync_marker[3] );
	if (syncMarker != 449838109)
	{
		os << " (WARNING expected value 1ACFFC1D)" << endl;
	}
	else
	{
		os << " (expected value 1ACFFC1D)" << endl;
	}


	os << "\nTM Transfer Frame Header\n"
	   << "	  Frame Identification\n"
	   << "		Version Number				: "
	   << transfer_frame_header.get_frame_id_version_numb() << endl
	   << "		Satellite Identifier		: "
	   << transfer_frame_header.get_frame_id_sl_id() << endl
	   << "		Virtual Channel Identifier	: "
	   << transfer_frame_header.get_frame_id_vc_id() << endl
	   << "		Operational Control Flag	: "
	   << transfer_frame_header.get_frame_id_ocff() << endl
	   << "	  Master Channel Frame Count	: "
	   << transfer_frame_header.get_mc_frame_count() << endl
	   << "	  Virtual Channel Frame Count	: "
	   << transfer_frame_header.get_vc_frame_count() << endl
	   << "	  Frame Data Field Status\n"
	   << "		 Secondary Header Flag		: "
	   << transfer_frame_header.get_fdfs_sec_head_flag() << endl
	   << "		 Synchronisation Flag		: "
	   << transfer_frame_header.get_fdfs_sync_flag() << endl
	   << "		 Packet Order Flag			: "
	   << transfer_frame_header.get_fdfs_order_flag() << endl
	   << "		 Segment Length Identifier	: "
	   << transfer_frame_header.get_fdfs_seg_len_id() << endl
	   << "		 First Header Pointer		: "
	   << transfer_frame_header.get_fdfs_first_head_pointer() << endl
	   << endl;


	os << "Packet Header\n"
	   << "	  Packet Identification\n"
	   << "		 Version Number			: "
	   << packet_header.get_version_number() << endl
	   << "		 Type					: "
	   << packet_header.get_type() << endl
	   << "		 Data Field Header Flag : "
	   << packet_header.get_data_field_header_flag() << endl
	   << "		 Application Process ID : "
	   << packet_header.get_apid() << endl
	   << "	  Packet Sequence Control\n"
	   << "		 Segmentation Flag		: "
	   << packet_header.get_segment_flag() << endl
	   << "		 Source Sequence Count	: "
	   << packet_header.get_sequence_count() << endl
	   << "	  Packet Length				: "
	   << packet_header.get_packet_length()
	   << " (actual length " << packet_header.get_packet_length()+1 << ")" << endl
	   << endl;


	COMMON_DATA* common_data = get_common_data();

	os << "Packet Data\n"
	   << "	  CCU Time (seconds)		   : " << get_ccu_time() << endl
	   << "		 First 3 bytes (seconds)   : "
	   << static_cast<int>( (common_data->ccu_time[0]<<16)
							+ (common_data->ccu_time[1]<<8)
							+ common_data->ccu_time[2] ) << endl
	   << "		 4th byte (1/256 seconds)  : "
	   << static_cast<int>(common_data->ccu_time[3]) << endl
	   << "	  Satellite (SL) Mode\n"
	   << "		 TM (telemetry format mode)				  : "
	   << static_cast<int>((common_data->sl_mode[0]>>6)&1) << endl
	   << "		 FS (flight software mode)				  : "
	   << static_cast<int>((common_data->sl_mode[0]>>4)&3) << endl
	   << "		 R (memory recopy mode)					  : "
	   << static_cast<int>((common_data->sl_mode[0]>>3)&1) << endl
	   << "		 TC-lock (execution lockout status)		  : "
	   << static_cast<int>((common_data->sl_mode[0]>>2)&1) << endl
	   << "		 TC-I/A (inhibition/authorisation status) : "
	   << static_cast<int>((common_data->sl_mode[0]>>1)&1) << endl
	   << "		 TDTR (real time data table status)		  : "
	   << static_cast<int>((common_data->sl_mode[0])&1) << endl
	   << "		 SVM mode								  : "
	   << static_cast<int>((common_data->sl_mode[1])&63) << endl
	   << "			Field C								  : "
	   << static_cast<int>((common_data->sl_mode[1]>>5)&1) << endl
	   << "			PF Mode								  : "
	   << static_cast<int>((common_data->sl_mode[1])&31) << endl
	   << "		 PMC mode								  : "
	   << static_cast<int>((common_data->sl_mode[2])&3) << endl
	   << "	  SVM HK Data\n"
	   << "		 Data On-Request Identifier	   : "
	   << static_cast<int>(common_data->svm_hk_data[0]) << endl
	   << "		 Last Anomalies Identifier\n"
	   << "			With passivation action	   : "
	   << static_cast<int>((common_data->svm_hk_data[1]<<8)+common_data->svm_hk_data[2]) << endl
	   << "			Without passivation action : "
	   << static_cast<int>((common_data->svm_hk_data[3]<<8)+common_data->svm_hk_data[4]) << endl
	   << "		 Anomalies Counter			   : "
	   << static_cast<int>(common_data->svm_hk_data[5]) << endl
	   << "		 Correct TC Packet Counter	   : "
	   << static_cast<int>(common_data->svm_hk_data[6]) << endl
	   << "		 Last TC Packet Report\n"
	   << "			TAS						   : "
	   << static_cast<int>((common_data->svm_hk_data[7]>>4)&15) << endl
	   << "			APID					   : "
	   << static_cast<int>(((common_data->svm_hk_data[7]&7)<<8)+common_data->svm_hk_data[8]) << endl
	   << "			Sequence Control		   : "
	   << static_cast<int>((common_data->svm_hk_data[9]<<8)+common_data->svm_hk_data[10]) << endl
	   << "		 PAC Report\n"
	   << "			Lockout Status			   : "
	   << static_cast<int>((common_data->svm_hk_data[11]>>7)&1) << endl
	   << "			Reasseembly Status		   : "
	   << static_cast<int>((common_data->svm_hk_data[11]>>6)&1) << endl
	   << "			MAP Identifier			   : "
	   << static_cast<int>(common_data->svm_hk_data[11]&63) << endl
	   << "	  SVM Phase Number				   : "
	   << static_cast<int>(common_data->svm_phase_no) << endl;


	RTF_DATA* rtf_data = get_rtf_data();

	os << "	  SVM RT Data					   : (124 octets)\n" //rtf_data->svm_rt_data[124]
	   << "	  PLM Phase Number				   : "
	   << static_cast<int>(rtf_data->plm_phase_no) << endl
	   << "	  PLM RT Data					   : (88 octets)\n" //rtf_data->plm_rt_data[88]
	   << "	  Programmable Part Phase Number   : "
	   << static_cast<int>(rtf_data->pp_phase_no) << endl
	   << "	  On-Request Table ID\n"
	   << "		 Field Type					   : "
	   << static_cast<int>((rtf_data->pp_on_request_table_id[0]>>4)&15)	 << endl
	   << "		 PL Valid Flag				   : "
	   << static_cast<int>((rtf_data->pp_on_request_table_id[0]>>3)&1)	<< endl
	   << "		 F1 Valid Flag				   : "
	   << static_cast<int>((rtf_data->pp_on_request_table_id[0]>>2)&1)	<< endl
	   << "		 F2 Valid Flag				   : "
	   << static_cast<int>((rtf_data->pp_on_request_table_id[0]>>1)&1)	<< endl
	   << "		 PLM Frame Counter			   : "
	   << static_cast<int>((((rtf_data->pp_on_request_table_id[0])&1)<<8)
						   + (rtf_data->pp_on_request_table_id[1])) << endl
	   << "			ICU/NIU ICU Format Number  : "
	   << static_cast<int>(((rtf_data->pp_on_request_table_id[0]&1)<<4)
						   + ((rtf_data->pp_on_request_table_id[1]>>5)&7)) << endl
	   << "			ICU/NIU ICU Frame Counter  : "
	   << static_cast<int>(rtf_data->pp_on_request_table_id[1]&31)	<< endl
	   << "	  Programmable Part Data		   : (254 octets)\n" //rtf_data->pp_data[254]
	   << endl;


	os << "TM Transfer Frame Trailer\n"
	   << "	  Command Link Control Word : "
	   << transfer_frame_trailer.get_clcw() << endl
	   << "		  First two bytes		: "
	   << (transfer_frame_trailer.get_clcw()>>16) << endl
	   << "		  Second two bytes		: "
	   << (transfer_frame_trailer.get_clcw()&65535) << endl
	   << "	  Frame Error Control Word	: "
	   << transfer_frame_trailer.get_fecw();


	if (transfer_frame_trailer.get_fecw() !=  check_crc() )
	{
		os << " ((WARNING calculated value " << check_crc() << ")" << endl;
	}
	else
	{
		os << " (calculated value " << check_crc() << ")" << endl;
	}
	os << endl;

}
