////////////////////////////////////////////////////////////////////////////////
//
//	File: sband.hpp
//
//	Provides: misc structs
//
//	Purpose: Code for representing source packets
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef sband_defined
#define sband_defined


struct transferFrameHeader
{
	byte data[6];

	unsigned int get_byte(int pos) const;
	byte* get_data(void) const;

	//unsigned int get_frame_id(void) const;
	unsigned int get_frame_id_version_numb(void) const;
	unsigned int get_frame_id_sl_id(void) const;
	unsigned int get_frame_id_vc_id(void) const;
	unsigned int get_frame_id_ocff(void) const;

	unsigned int get_mc_frame_count(void) const;
	unsigned int get_vc_frame_count(void) const;

	//unsigned int get_frame_data_field_status(void) const;
	unsigned int get_fdfs_sec_head_flag(void) const;
	unsigned int get_fdfs_sync_flag(void) const;
	unsigned int get_fdfs_order_flag(void) const;
	unsigned int get_fdfs_seg_len_id(void) const;
	unsigned int get_fdfs_first_head_pointer(void) const;

};


struct transferFrameTrailer
{
	byte data[6];

	unsigned int get_byte(int pos) const;
	unsigned short* get_data() const;
	unsigned int get_clcw(void) const;
	unsigned int get_fecw(void) const;

};


struct SBandPacketHeader
{
	byte data[6];

	unsigned int get_byte(int pos) const;
	byte* get_data() const;
	unsigned int get_version_number(void) const;
	unsigned int get_type(void) const;
	unsigned int get_data_field_header_flag(void) const;
	unsigned int get_apid(void) const;
	unsigned int get_segment_flag(void) const;
	unsigned int get_sequence_count(void) const;
	unsigned int get_packet_length(void) const;
};


struct COMMON_DATA
{
	//CCU Time
	byte ccu_time[4] PACKED;

	//SL Mode
	byte sl_mode[3] PACKED;

	//SVM RT Part
	byte svm_hk_data[12] PACKED;
	byte svm_phase_no PACKED;

	double get_ccu_time(void) const;
	//unsigned short* get_data() const;
};



struct RTF_DATA
{
	//SVM RT Part
	byte svm_rt_data[124] PACKED;

	//PLM RT Part
	byte plm_phase_no PACKED;
	byte plm_rt_data[88] PACKED;

	//Programmable Part
	byte pp_phase_no PACKED;
	byte pp_on_request_table_id[2] PACKED;
	byte pp_data[254] PACKED;

	//unsigned short* get_data() const;
};


struct TEF_DATA
{
	//Programmable Part
	byte pp_phase_no PACKED;
	byte pp_on_request_table_id[2] PACKED;
	byte svm_table[467] PACKED;

	//unsigned short* get_data() const;
};



struct SBand
{
	enum { SIZE = 512 };

	byte sync_marker[4] PACKED;
	transferFrameHeader transfer_frame_header PACKED;
	SBandPacketHeader packet_header PACKED;
	byte common_data[20] PACKED;
	byte packet_data[470] PACKED;
	transferFrameTrailer transfer_frame_trailer PACKED;

	int get_sync_marker(void) const;
	COMMON_DATA* get_common_data(void) const;
	RTF_DATA* get_rtf_data(void) const;
	TEF_DATA* get_tef_data(void) const;
	double get_ccu_time(void) const;
	unsigned short check_crc(void) const;
	void create_text_output(textoutstream_type& os) const;
};



#endif
