////////////////////////////////////////////////////////////////////////////////
//
//	File: sbandreader.hpp
//
//	Provides: SBbandReader filter
//
//	Purpose: Extract S-Band telemetry packets from a plain binary stream
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef sbandreader_defined
#define sbandreader_defined

#include "fileproducer.hpp"
#include "sbandproducer.hpp"
#include "textsummaryproducer.hpp"


class SBandReader :
	public SBandProducer,
	public TextSummaryProducer
{
public:

	enum { WARNING_GAP_SIZE = 4*1024*1024 };

	/// constructor
	SBandReader(FileProducer& _source);

	/// return next D-Band packet
	virtual SBand* get_next_sband(void);

	/// output_text_summary
	virtual void output_text_summary( textoutstream_type& os );

private:

	enum { BUFFER_SIZE = 128*512*2 };
	byte buffer[BUFFER_SIZE];

	FileProducer& source;

	off64_t buffer_pos; // current read pos
	off64_t buffer_size; // buffer is declared statically but here we track the
	// actual buffer usage

	istream_type* in_stream;

	int good_packets; // count of proper CADUs found
	unsigned long long skipped_bytes; // total bytes of input stream that were not used
	unsigned long long gap_size; // inside an unrecognised part of the input, size of
	// current bad area
	off64_t file_pos;

	typedef map<long long,int> Gaps;
	Gaps gaps;

	long long initial_gap;
	long long final_gap;

	bool fill_buffer(void);
	SBand* scan_buffer(void);
};

#endif
