////////////////////////////////////////////////////////////////////////////////
//
//	File: sbandtmdisplayer.hpp
//
//	Provides: SBandTMDisplayer filter
//
//	Purpose: Show contents of S-Band Telemetry packets as text
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef sbandtmdisplayer_defined
#define sbandtmdisplayer_defined

#include "sband.hpp"
#include "sbandproducer.hpp"
#include "textrecordproducer.hpp"


class SBandTMDisplayer :
	public TextRecordProducer
{
public:

	//
	// constructor
	//

	SBandTMDisplayer(SBandProducer& _source, int _skip, int _count)
		: source(_source),
		  skip(_skip),
		  count(_count),
		  packet_ctr(0)
		{
		}


	//
	// output_text_record
	//
	virtual void output_text_record(textoutstream_type& os,
									int record = -1,
									bool show_record_header = false)
		{

			DEBUG("output_text_record - sbandtmdisplayer\n");

			if (skip > 1 && count > 0)
			{
				count = count + skip;
			}

			if (count < 1)
			{
				count = 2147483647;
			}

			SBand* packet;
			while((packet=source.get_next_sband()))
			{
				packet_ctr++;


				if ( (packet_ctr > skip) && (packet_ctr <= count) )
				{

					os << "------ Start of Packet : " << packet_ctr << " ------" << endl;
					packet->create_text_output(info);
					os << "------ End of Packet	  : " << packet_ctr << " ------" << endl;

				}
			}

		}

private:

	SBandProducer& source;
	int skip;
	int count;
	int packet_ctr;

};


#endif

