////////////////////////////////////////////////////////////////////////////////
//
//	File: singlefilewriter.cpp
//
//	Provides: SingleFileWriter filter
//
//	Purpose: Write data to disk as one file
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "singlefilewriter.hpp"

SingleFileWriter::SingleFileWriter( FileProducer* source,
									const string& _filename )
	: total_bytes(0),
	  filename(_filename)
{
	debug << "singlefilewriter constructor\n";

	ofs.reset(new ofstream_type(filename.c_str()));
	if (!source->produce_files(*this))
	{
		debug << "singlefilewriter trying get_next_file method\n";
		const int BUFFER_SIZE = 32*1024;
		char buffer[BUFFER_SIZE];

		while(istream_type* is = source->get_next_file())
		{
			debug << "singlefilewriter got file\n";
			while(!is->eof())
			{
				is->read(buffer, BUFFER_SIZE);

				streampos old_pos = ofs->tellp();
				ofs->write(buffer, is->gcount());
				if (static_cast<unsigned int>(old_pos) +
					static_cast<unsigned int>(is->gcount())
					!= static_cast<unsigned int>(ofs->tellp()) || ofs->fail())
				{
					error << "WRITE ERROR OCCURRED - output file "
						  << "may be corrupt\n";
				}

			}
		}
	}
}

ostream_type* SingleFileWriter::get_output_stream(void)
{
	return ofs.get();
}

ostream_type* SingleFileWriter::get_named_output_stream(
	const string& filename_suffix)
{
	return ofs.get();
}

void SingleFileWriter::output_text_summary( textoutstream_type& os )
{
	os << "SingleFileWriter report:\n"
	   << "	   " << ofs->tellp() << " bytes written\n";
}

string SingleFileWriter::get_filename_base(void)
{
	return filename;
}
