////////////////////////////////////////////////////////////////////////////////
//
//	File: subsat.hpp
//
//	Purpose: Handle database of subsatellite positions
//
//	Copyright (c) 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef subsat_defined
#define subsat_defined

#include "eps.hpp"

/// Represent a database of subsatellite positions.
/// An input directory is scanned for files matching subsat-*.txt and each
/// match is read as a list of time stamped lat/lon pairs. Entries are stored
/// with an accuracy of 1 second.

class Subsat
{
public:

	/// A single position pair

	struct Pos
	{
		Pos(void);

		double lat;
		double lon;

		bool operator != (const Pos& o) const;
	};

	/// Constructor
	Subsat(void);

	/// Constructor
	Subsat(const string& dirname);

	/// Database lookup.
	/// Returns a blank (0.0,0.0) value if no match
	Pos get_pos(short_cds_time t) const;

private:

	/// Read in a single file
	void scan_file(const string& filename);

	/// The database
	map<short_cds_time, Pos> pos;

	mutable bool warned;
};

ostream& operator << (ostream& os, const Subsat::Pos& pos);

#endif
