
////////////////////////////////////////////////////////////////////////////////
//
//	File: tvcdu.hpp
//
//	Provides: tVCDU struct
//
//	Purpose:
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef tvcdu_defined
#define tvcdu_defined

#include "mpdu.hpp"
#include "ccsds.hpp"

struct tVCDU
{
	enum { SIZE = 901 };
	enum { PAYLOAD_SIZE = 884 };

	byte header[6];
	byte insert_zone[2];
	MPDU mpdu;

	UTCTime utc;
	byte reed_solomon_status;

	byte* get_data()
		{
			return &(header[0]);
		}

	void set_byte(uint8 new_data, unsigned int offset)
		{
			byte* data = get_data();
			data[offset] = new_data;
		}

	int get_version_number()
		{
			return (header[0] >> 6);
		}

	void set_version_number(int vn)
		{
			int a = 0;
			int b = (int)header[0];
			a |= (vn&3)<<6;
			a |= b&63;
			header[0] = a;
		}

	int get_scid()
		{
			return ( (((header[0])&63)<<2) + (header[1] >> 6) );
		}

	void set_scid(int scid)
		{
			int a = 0;
			int b = (int)header[0];
			int c = (int)header[1];

			a |= ((b>>6)&3)<<6;
			a |= (scid>>2)&63;
			header[0] = a;

			a = 0;
			a |= (scid&3)<<6;
			a |= b=c&63;
			header[1] = a;
		}

	int get_vcid()
		{
			return (header[1] & 63);
		}

	void set_vcid(int vcid)
		{
			int a = 0;
			int b = (int)header[1];
			a |= ((b>>6)&3)<<6;
			a |= vcid&63;
			header[1] = a;
		}

	int get_counter()
		{
			return ( (header[2] << 16) +
					 (header[3] << 8) +
					 (header[4]) );
		}

	void set_counter(int counter)
		{
			header[2] = ((counter>>16)&255);
			header[3] = ((counter>>8)&255);
			header[4] = (counter&255);
		}

	int get_replay_flag()
		{
			return ((header[5]>>7)&1);
		}

	void set_replay_flag(int flag)
		{
			int a = 0;
			int b = (int)header[5];
			a |= (flag&1)<<7;
			a |= b&127;
			header[5] = a;
		}

	int get_spare()
		{
			return (header[5]&127);
		}

	void set_spare(int spare)
		{
			int a = 0;
			int b = (int)header[5];
			a |= ((b>>7)&1)<<7;
			a |= spare&127;
			header[5] = a;
		}

	int get_encrypt_flag()
		{
			return insert_zone[0];
		}

	void set_encrypt_flag(int flag)
		{
			insert_zone[0] = flag;
		}

	int get_encrypt_key()
		{
			return (insert_zone[1]);
		}

	void set_encrypt_key(int key)
		{
			insert_zone[1] = key;
		}

	int get_mpdu_spare()
		{
			return mpdu.get_spare();
		}

	void set_mpdu_spare(int spare)
		{
			mpdu.set_spare(spare);
		}

	int get_mpdu_header()
		{
			return mpdu.get_first_header_pointer();
		}

	void set_mpdu_header(int _header)
		{
			mpdu.set_first_header_pointer(_header);
		}

	UTCTime get_utc(void)
		{
			return (utc);
		}

	void set_utc(UTCTime& t)
		{
			utc = t;
		}

	void set_utc_day(int day)
		{
			utc.days = day;
		}

	void set_utc_millisec(int millisec)
		{
			utc.milliseconds = millisec;
		}

	void set_utc_microsec(int microsec)
		{
			utc.microseconds = microsec;
		}

	int get_reed_solomon_status()
		{
			return reed_solomon_status;
		}

	void set_reed_solomon_status(int rss)
		{
			reed_solomon_status = rss;
		}

	static string time_to_string(const UTCTime& t)
		{
			return (TimeConv::utc_days_milliseconds_to_string(
						t.days, t.milliseconds) + "." +
					ntofs(t.milliseconds%1000,3) +
					ntofs(t.microseconds,3));
		}

};

#endif
