////////////////////////////////////////////////////////////////////////////////
//
//	File: tvcdu_displayer.cpp
//
//	Provides: tvcdu_displayer executable
//
//	Purpose: Display information on tVCDU packet streams
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "filereader.hpp"
#include "readfromstdin.hpp"
#include "textrecorddisplayer.hpp"
#include "tvcdudisplayer.hpp"
#include "tvcdureader.hpp"
#include "args.hpp"
#include "tvcduwriter.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

int gerror = 0; // global error
const string APPNAME = "tvcdu_displayer";
const string MAINTAINER = "elson@eumetsat.de";

int main(int argc, char* argv[])
{
	ofstream onull("/dev/null");
	_debug = &onull;
	_info = &cout;
	_warning = &cerr;
	_error = &cerr;

	deque<string> in_files;
	bool compact = false;
	int show_vc = -1;
	string write_tvcdus;

	Args args;
	args.set_doc_head("View tVCDU packets.");
	args.set_doc_tail("");
	args.set_package(PACKAGE);
	args.set_app(APPNAME);
	args.set_maintainer(MAINTAINER);
	args.set_copyright("Copyright (c) 2005 Eumetsat.");
	args.set_license(Args::GPL);
	args.set_version(VERSION);

	args.add_related_prog("cadu_displayer");
	args.add_related_prog("tvcdu_to_ccsds");
	args.add_related_prog("ccsds_displayer");

	args.add_example("tvcdu_displayer file.tvcdu", "");

	args.add(&in_files, "in", 'i', Args::DEFAULT_OPTION|Args::FILENAME, "Select input file(s)", "If omitted packets are read from standard input.");
	args.add(&show_vc, "show-vc", 0, 0, "Display only one Virtual Channel");
	args.add(&write_tvcdus, "write-tvcdus", 0, 0, "Write t-VCDU packets to a file. This option can be combined with --show-vc");
	args.add(&compact, "compact", 0, 0, "Show compact output style");
	args.add(0, "debug", Args::USER_OPTION, Args::HIDDEN);

	args.set_args(argc, argv);

	int arg;
	while((arg=args.get_next()) != Args::DONE)
	{
		switch(arg)
		{
		case Args::USER_OPTION:
			_debug = &cout;
			break;
		}
	}

	spinner->restream(&onull);

	// set UTC time zone
	TimeConv::set_time_zone();

	FileProducer* first_source;
	if (in_files.empty())
	{
		first_source = new ReadFromStdin();
	}
	else
	{
		first_source = new FileReader(in_files);
	}

	tVCDUReader tvcdu_reader(*first_source, show_vc);

	if (write_tvcdus.empty())
	{
		tVCDUDisplayer tvcduer(tvcdu_reader, compact, true);
		TextRecordDisplayer displayer(tvcduer);
	}
	else
	{
		tVCDUWriter writer(tvcdu_reader, write_tvcdus);
		first_source->output_text_summary();
		tvcdu_reader.output_text_summary();
		writer.output_text_summary();
	}

	delete first_source;
	return 0;
}
