////////////////////////////////////////////////////////////////////////////////
//
//	File: tvcdudisplayer.hpp
//
//	Provides: tVCDUDisplayer filter
//
//	Purpose: Show brief contents of tVCDU packets as text
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef tvcdudisplayer_defined
#define tvcdudisplayer_defined

#include "mpdu.hpp"
#include "tvcduproducer.hpp"
#include "textrecordproducer.hpp"
#include "vcdu.hpp"
#include "ccsds.hpp"

class tVCDUDisplayer :
	public TextRecordProducer
{
public:

	tVCDUDisplayer(tVCDUProducer& _source, bool _compact, bool _show_rs_info)
		: source(_source),
		  show_rs_info(_show_rs_info)
		{
		}

	virtual void output_text_record(textoutstream_type& os, int, bool)
		{
			int cc=0;

			while(tVCDU* tvcdu = source.get_next_tvcdu())
			{
				int version_number = tvcdu->header[0] >> 6;
				int scid = (((tvcdu->header[0])&63)<<2) +
					(tvcdu->header[1] >> 6);
				int vcid = tvcdu->header[1] & 63;
				int counter = (tvcdu->header[2] << 16) +
					(tvcdu->header[3] << 8) +
					(tvcdu->header[4]);
				MPDU& mpdu = tvcdu->mpdu;
				int reed_solomon = tvcdu->reed_solomon_status;
				string ttime = tVCDU::time_to_string(tvcdu->utc);
				int ptr = mpdu.get_first_header_pointer();

				os << cc
				   << " VCDU: version " << version_number
				   << " SCID " << scid
				   << " VCID " << ntofs(vcid,2,' ')
				   << " Count " << ntofs(counter,8,' ')
				   << " replay " << (int)(tvcdu->header[5])
				   << " encrypt " << (int)(tvcdu->header[6])
				   << " key " << (int)(tvcdu->header[7])
				   << " MPDU: header pointer " << ntofs(ptr,4)
				   << " spare " << mpdu.get_spare()
				   << endl;

				if (vcid!=63 && ptr!=2047)
				{
					if (ptr>=882)
					{
						os << "WARNING bad packet - FIRST_HEADER_POINTER cannot be "
						   << ">882\n";
					}
					else
					{
						while(ptr<VCDU::PAYLOAD_SIZE)
						{
							if (ptr > 876)
							{
								os << "(only a partial ISP header available)\n";
							}

							CCSDSPrimaryHeader isp;
							bzero(&isp, 6);
							memcpy(&isp, ((byte*)tvcdu)+ptr+10,
								   min(6,882-ptr));
							os << cc << " **** ISP:"
							   << " Version " << isp.get_version_number()
							   << " Type " << isp.get_type()
							   << " 2nd hdr " << isp.get_secondary_header_flag()
							   << " APID " << isp.get_apid()
							   << " Seq Flags " << isp.get_sequence_flags()
							   << " Seq Count " << isp.get_sequence_count()
							   << " Length " << isp.get_packet_length()
							   << " (" << (isp.get_packet_length()+7)
							   << " bytes)";

							if (ptr < 868)
							{
								UTCTime utc_time;
								memcpy(&utc_time, ((byte*)tvcdu)+ptr+10+6, 8);
								os << " UTC " << utc_time;
							}

							if (ptr < 862)
							{
								OBTTime obt_time;
								memcpy(&obt_time, ((byte*)tvcdu)+ptr+10+6+8, 6);
								os << " OBT " << obt_time;
							}

							os << endl;

							if (ptr<876)
							{
								ptr += isp.get_packet_length()+7;
							}
							else
							{
								ptr = VCDU::PAYLOAD_SIZE;
							}
						}
					}
				}

				if (show_rs_info)
				{
					os << cc << " tVCDU: RS status "
					   << reed_solomon
					   << " time " << ttime
					   << " mpdu checksum " << mpdu.generate_a_checksum()
					   << endl;
				}

				cc++;
			}
		}

private:

	tVCDUProducer& source;
	bool show_rs_info;

};

#endif
