////////////////////////////////////////////////////////////////////////////////
//
//	File: tvcduextractor.hpp
//
//	Provides: tVCDUExtractor extractor
//
//	Purpose: Convert tVCDU packets into a binary stream for storing
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef tvcduextractor_defined
#define tvcduextractor_defined

#include "interface.hpp"
#include "tvcduproducer.hpp"

class tVCDUExtractor :
	virtual public Interface,
	public FileProducer
{
public:

	tVCDUExtractor( tVCDUProducer& _source )
		: source(_source)
		, packet_count(0)
		{
		}

	virtual void output_text_summary( ostream& os = cout )
		{
			os << "tVCDUExtractor report:\n"
			   << "	   " << packet_count << " packets read\n"
			   << "	   " << sizeof(tVCDU) << " bytes per tVCDU packet\n"
			   << endl;
		}

	virtual bool produce_files(FileConsumer& dest)
		{
			DEBUG("tvcduextractor begin\n");
			while(tVCDU* tvcdu = source.get_next_tvcdu())
			{
				ostream_type* os = dest.get_output_stream();
				os->write((char*)tvcdu, sizeof(tVCDU));
				packet_count++;
//		DEBUG("ccsdsextractor got packet\n");
			}
			DEBUG("tvcduextractor end of stream\n");
			return true;
		}

	virtual istream_type* get_next_file(void)
		{
			return NULL;
		}

private:

	tVCDUProducer& source;
	int packet_count;
};

#endif
