////////////////////////////////////////////////////////////////////////////////
//
//	File: tvcdureader.hpp
//
//	Provides: tVCDUReader filter
//
//	Purpose: Extract a stream of tVCDU packets from a raw binary input stream
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef tvcdureader_defined
#define tvcdureader_defined

#include "interface.hpp"
#include "tvcdu.hpp"
#include "tvcduproducer.hpp"

class tVCDUReader :
	virtual public Interface,
	public tVCDUProducer
{
public:

	//
	// constructor
	//

	tVCDUReader( FileProducer& _source, int _filter = -1)
		: source(_source),
		  in_stream(NULL),
		  packet_count(0),
		  filter(_filter),
		  filtered_count(0)
		{
			packet.reset(new tVCDU);
		}

	//
	// output_text_summary
	//

	virtual void output_text_summary( ostream& os = cout )
		{
			os << "tVCDUReader report:\n"
			   << "	   Found " << packet_count << " packets\n";

			if (filter!=-1)
			{
				cout << "	 Filtered " << filtered_count << " packets on VC " << filter << endl;
			}
			os << endl;
		}

	//
	// get_next_tvcdu
	//

	virtual tVCDU* get_next_tvcdu(void)
		{
			if (!in_stream)
			{
				in_stream = source.get_next_file();
			}

			if (!in_stream)
			{
				return NULL;
			}

			while(true)
			{
				in_stream->read((char*)(packet.get()),
								sizeof(tVCDU));

				if (in_stream->gcount() < sizeof(tVCDU) ||
					in_stream->eof())
				{
					in_stream = NULL;
					return get_next_tvcdu();
				}

				packet_count++;
				if (filter != -1)
				{
					int vc = packet->get_vcid();
					if (vc != filter)
					{
						filtered_count++;
						continue;
					}
				}

				return packet.get();
			}
		}

private:

	FileProducer& source;
	istream_type* in_stream;
	int packet_count;
	auto_ptr<tVCDU> packet;
	int filter;
	int filtered_count;

};

#endif
