////////////////////////////////////////////////////////////////////////////////
//
//	File: vcdu.hpp
//
//	Provides: VCDU struct
//
//	Purpose:
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef vcdu_defined
#define vcdu_defined

#include "mpdu.hpp"

typedef unsigned int VCID;
const VCID MAX_VCID = 0xffffffff;
const int MAX_VCCOUNT = 0x1000000;

const int VCID_FILL = 63;
const int VCDU_ENCRYPTED = 0xff;
const int VCDU_CURRECT_VERSION = 1;
const int VCID_METOP_1 = 11;
const int VCID_METOP_2 = 12;
const int VCID_METOP_3 = 13;
const int VCID_METOP_SIM = 14;

struct VCDU
{
	enum { PAYLOAD_SIZE = 884 };

	byte header[8];

	unsigned int get_byte(int pos) const
		{
			return static_cast<unsigned int>(header[pos]);
		}

	unsigned int get_version_number(void) const
		{
			return (get_byte(0)>>6);
		}

	void set_version_number(unsigned int v)
		{
			header[0] &= 191;
			header[0] |= (v&3) << 6;
		}

	unsigned int get_scid(void) const
		{
			return ( ((get_byte(0)&63)<<2) | (get_byte(1)>>6) );
		}

	void set_scid(unsigned int id)
		{
			header[0] &= 192;
			header[0] |= (id>>2);
			header[1] &= 63;
			header[1] |= ((id&3) << 6);
		}

	unsigned int get_vcid(void) const
		{
			return ( get_byte(1)&63 );
		}

	void set_vcid(unsigned int vcid)
		{
			header[1] &= 192;
			header[1] |= vcid&63;
		}

	unsigned int get_vccount(void) const
		{
			return ( (get_byte(2)<<16) | (get_byte(3)<<8) | get_byte(4) );
		}

	void set_vccount(unsigned int vccount)
		{
			header[2] = (vccount>>16)&255;
			header[3] = (vccount>>8)&255;
			header[4] = vccount&255;
		}

	unsigned int get_replay_flag(void) const
		{
			return ( get_byte(5)>>7 );
		}

	void set_replay_flag(unsigned int flag)
		{
			header[5] &= 127;
			header[5] |= flag&1;
		}

	unsigned int get_spare(void) const
		{
			return ( get_byte(5) & 127 );
		}

	void set_spare(int spare)
		{
			header[5] &= 128;
			header[5] |= spare&127;
		}

	unsigned int get_encrypt_flag(void) const
		{
			return ( get_byte(6) );
		}

	void set_encrypt_flag(unsigned int flag)
		{
			header[6] = flag&255;
		}

	unsigned int get_encrypt_key(void) const
		{
			return ( get_byte(7) );
		}

	void set_encrypt_key(unsigned int key)
		{
			header[7] = key&255;
		}

	MPDU mpdu;
};

#endif
