////////////////////////////////////////////////////////////////////////////////
//
//	File: vcduscidsetter.cpp
//
//	Provides: VCDUSCIDSetter class
//
//	Purpose:
//
//	Copyright (c) 2008 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "vcduscidsetter.hpp"

VCDUSCIDSetter::VCDUSCIDSetter(VCDUProducer& _source, int _new_scid)
	: source(_source),
	  new_scid(_new_scid)
{
}

VCDU* VCDUSCIDSetter::get_next_vcdu(void)
{
	if (!spinner.get())
	{
		spinner.reset(new Spinner("Changing SCIDs"));
	}
	spinner->tick();

	VCDU* vcdu = source.get_next_vcdu();
	if (!vcdu)
	{
		return NULL;
	}

	Scids::iterator iter = old_scids.find(vcdu->get_scid());
	if (iter==old_scids.end())
	{
		old_scids[vcdu->get_scid()] = 1;
	}
	else
	{
		(*iter).second++;
	}
	vcdu->set_scid(new_scid);
	return vcdu;
}

void VCDUSCIDSetter::output_text_summary( ostream& os )
{
	cout << "VCDUSCIDSetter report:\n"
		 << "	 SCID set to " << new_scid << endl
		 << "	 Old SCIDs: ";
	for(Scids::iterator iter=old_scids.begin(); iter!=old_scids.end(); iter++)
	{
		cout << (*iter).first << " (" << (*iter).second << ") ";
	}
	cout << endl << endl;
}
