////////////////////////////////////////////////////////////////////////////////
//
//	File: vcdutocadu.cpp
//
//	Provides: VCDUToCADU class
//
//	Purpose: Convert VCDU packets to CADU packets
//
//	Copyright (c) 2008 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "vcdutocadu.hpp"
#include "rs.hpp"

VCDUToCADU::VCDUToCADU(VCDUProducer* _source)
	: source(_source),
	  randomizer(CADURandomizer())
{
	assert(source);
}

CADU* VCDUToCADU::get_next_cadu(void)
{
	VCDU* vcdu = source->get_next_vcdu();
	if (!vcdu)
	{
		return NULL;
	}

	CVCDU& cvcdu = cadu.payload;
	memcpy(&(cvcdu.payload), vcdu, sizeof(VCDU));
	byte buffer[2230];
	byte rs[320];
	for(int i=0; i<4; i++)
	{
		for(int j=0; j<223; j++)
		{
			buffer[j] = reinterpret_cast<byte*>(vcdu)[i+j*4];
		}
		rs_encode(buffer, rs, 0);
		for(int j=0; j<32; j++)
		{
			cvcdu.checksum[i+j*4] = rs[j];
		}
	}
	randomizer.randomize(&cvcdu);
	cadu.set_sync_marker(MAGIC_NUMBER);
	return &cadu;
}

void VCDUToCADU::output_text_summary( ostream& os )
{
	cout << "VCDUToCADU report:\n";
}

