////////////////////////////////////////////////////////////////////////////////
//
//	File: vcdutoccsds.hpp
//
//	Provides: VCDUToCCSDS filter
//
//	Purpose: Convert a stream of VCDU packets to CCSDs packets
//
//	Copyright (c) 2002, 2003, 2004, 2005 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef vcdutoccsds_defined
#define vcdutoccsds_defined

#include "interface.hpp"
#include "mpdu.hpp"
#include "cvcduproducer.hpp"
#include "ccsdsproducer.hpp"
#include "vcduproducer.hpp"

class VCDUToCCSDS :
	virtual public Interface,
	public CCSDSProducer
{
public:

	VCDUToCCSDS(VCDUProducer& _source);
	~VCDUToCCSDS(void);

	void output_text_summary(ostream& os = cout);
	CCSDS* get_next_ccsds(void);
	EPSProducer* as_eps_product(void);

private:
	// Store enumerated values of possible VCDU states during each sequence count
	enum VCDU_STATE { VCDU_OK=0, VCDU_UNKNOWN, VCDU_BREAK, VCDU_DUPS };

	class VirtualBuffer
	{
	public:

		VirtualBuffer(void);
		bool check_vccount(int vccount);
		int reinit(void);
		int clear(void);
		void insert(byte* new_buffer, int len);
		CCSDS* get_ccsds(void);
		bool is_warm(void) const {return warm;}
		bool is_bad(void) const {return bad_state!=VCDU_OK;}
		void set_bad(VCDU_STATE badstate) {bad_state = badstate;}
		VCDU_STATE get_bad(void) const {return bad_state;}
		void clear_bad(void) {bad_state = VCDU_OK;}
		bool is_empty(void) const {return ptr<=0;}

	private:
		enum { MAX_CCSDS = 65536 };
		byte buffer[MAX_CCSDS];
		int ptr;
		int ptr_target;
		int last_vccount;
		bool warm;
		VCDU_STATE bad_state;
	};

	// Check if VCDU counter is incorrect
	bool check_counter(VirtualBuffer& vb, VCDU* vcdu, int last_count);

	string get_bad_text(VCDU_STATE bad_flag) const;

	typedef map<VCID, VirtualBuffer> VirtualBuffers;
	VirtualBuffers vbs;

	VCDUProducer& source;
	// maintain counters for EACH VCID
	typedef map<VCID, int> ChannelCounters;
	ChannelCounters vccounts;
	int total_vcdus;

	auto_ptr<CCSDS> current_source_packet;

	auto_ptr<stringbuf> fileprod_sb;
	auto_ptr<istream> fileprod_is;

	int good_ccsds;
	VCID current_vc; // performance optimisation

	int mpdu_spare_noheader;
	int mpdu_pointer_noheader;

	byte* hold;
	int hold_size;
	int hold_vc;

	int unmapped_bytes;
	int initial_unmapped;

	typedef vector<string> VCErrors;
	VCErrors vccount_errors;

	auto_ptr<CCSDS> result;
};

#endif
