////////////////////////////////////////////////////////////////////////////////
//
//	File: writetonull.cpp
//
//	Provides: WriteToNull filter
//
//	Purpose: Discard inputs but report on them
//
//	Copyright (c) 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "writetonull.hpp"

WriteToNull::WriteToNull(FileProducer& source)
	: file_count(0),
	  byte_count(0)
{
	if (!source.produce_files(*this))
	{
		const int BUFFER_SIZE = 128*1024;
		char buffer[BUFFER_SIZE];

		while(istream_type* is = source.get_next_file())
		{
			while(!is->eof())
			{
				is->read(buffer, BUFFER_SIZE);
				byte_count += is->gcount();
			}
		}
	}
}

struct deletor : public unary_function<map<string,ofstream_type*>::
	value_type, void>
{
	void operator()(const map<string,ofstream_type*>::value_type& x)
		{
			delete x.second;
		}
};

WriteToNull::~WriteToNull(void)
{
	for_each(ostreams.begin(), ostreams.end(), deletor());
}

void WriteToNull::output_text_summary(ostream& os )
{
	for(OStreams::iterator iter=ostreams.begin();
		iter!=ostreams.end(); iter++)
	{
		byte_count += (*iter).second->get_total_written();
	}

	os << "WriteToNull report:\n"
	   << "	   " << file_count << " files would have been written\n"
	   << "	   " << byte_count << " bytes would have been written\n\n";
}

ostream_type* WriteToNull::get_named_output_stream(
	const string& filename_suffix)
{
	OStreams::iterator iter = ostreams.find(
		string(string(string("x")+filename_suffix)));
	if (iter==ostreams.end())
	{
		ofstream_type* result = new ofstream_type(
//		string(filename_base+filename_suffix).c_str());
			"/dev/null");
		ostreams[string("x")+filename_suffix] = result;
		file_count++;
		return result;
	}
	else
	{
		return (*iter).second;
	}
}

ostream_type* WriteToNull::get_output_stream(void)
{
	if (!single_file_output.get())
	{
		single_file_output.reset(
//		new ofstream_type(filename_base.c_str()));
			new ofstream_type("/dev/null"));
		file_count++;
	}
	return single_file_output.get();
}

string WriteToNull::get_filename_base(void)
{
	return "";
}
