////////////////////////////////////////////////////////////////////////////////
//
//	File: writetostdout.cpp
//
//	Provides: WriteoStdout filter
//
//	Purpose: Spool data to standard output
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "writetostdout.hpp"

WriteToStdout::WriteToStdout(FileProducer* source, bool force)
{
	// sanity check
	if (isatty(1)==1 && !force)
	{
		error << "WriteToStdout cannot display to a terminal. Data "
			  << "should be piped a process or file instead\n";
		gerror = -1;
		return;
	}

	if (!source->produce_files(*this))
	{
		const int BUFFER_SIZE = 32*1024;
		char buffer[BUFFER_SIZE];

		while(istream_type* is = source->get_next_file())
		{
			while(is)
			{
				is->read(buffer, BUFFER_SIZE);
				cout.write(buffer, is->gcount());
			}
		}
	}
}

ostream_type* WriteToStdout::get_output_stream(void)
{
#ifdef _USE_LARGE_FILES
	if (!stdout_stream.get())
	{
		stdout_stream.reset(new ofstream_type(stdout));
	}
	return stdout_stream.get();
#else
	return &cout;
#endif
}

ostream_type* WriteToStdout::get_named_output_stream(
	const string& filename_suffix)
{
#ifdef _USE_LARGE_FILES
	if (!stdout_stream.get())
	{
		stdout_stream.reset(new ofstream_type(stdout));
	}
	return stdout_stream.get();
#else
	return &cout;
#endif
}

void WriteToStdout::output_text_summary(textoutstream_type& os)
{
}

string WriteToStdout::get_filename_base(void)
{
	return "";
}
