//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/ALG/src/SD_ALG_AVHRR.c,v 1.6 2013/03/29 16:55:26 lcaspar Exp $
//MODIFIED FOR PATCH3-5-5 : 05/Jul/06
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_ALG
//@ MODULE : SD_ALG_AVHRR
//@
//@ Comment :
//@    Code file for the  SD_ALG_AVHRR class which includes the
//@    algorithm used to compare IASI and AVHRR images in order to :
//@ - analyse IIS type
//@ - geolocalize IASI L1 products
//@
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 27/02/2003 : [reference :] M. Jung
//@ <texte libre permettant de commenter le pourquoi et le comment
//@ <version du fichier>
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-084-CN : 13/11/2003 : JL. PASCAL
//@ optimisation du code ( transform ipsf et sd_alg_statclass );
//@ fiche de correction correspondante : IA-2100-FC-084-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-128-CN : 03/08/2004 : B. MASSART
//@ Correction of SD_ALG_40_IAC after the one that followed insure++ bench
//@ test. GIacPosMaxQual concerned. IISDeadPix must be transposed.
//@ SD_ALG_PseudoCanauxIIS corrected too. See IA-2100-FC-128-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-134-CN : 04/08/2004 : B. MASSART
//@ Initialization of FlagCoregNonQual and FlagCoregNonValid corrected in 
//@ SD_ALG_40_IAC. See IA-2100-FC-134-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-135-CN : 29/07/2004 : B. MASSART
//@ EigVect must be transposed in SD_ALG_EigenAnalysis. See IA-2100-FA-135-THA.
//@ --------------------------------------------------------------------
//@ PM-ID : IA-CR-0000-242-CNE : 06/08/2004 : B. MASSART
//@ 44_GEO is now called if Cdegraded cases #9 and #15 are raised.
//@ See IA-2100-CR-242-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-142-CN : 05/08/2004 : B. MASSART
//@ Correction of SD_ALG_41_CCS to make it process external calibration mode 
//@ with Earth target (SP15). See IA-2100-FA-142-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-153-CN : 29/01/2005 : JL. PASCAL
//@ calcul du champ SUBSAT_LAT et SUBSAT_LON du MPHR
//@ fiche de correction correspondante : IA-2100-FC-153-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-CR-270-CN : 24/06/2005 : JL.PASCAL
//@ prise en compte des SN manquant
//@ fiche de correction correspondante : IA-2100-CR-270-THA
//@ --------------------------------------------------------------------
//@ PM-ID : IA-2100-CR-292-CNE : 10/03/2006 : B.MASSART
//@ Function SD_ALG_AVHRR_GetSubSatLatLon is added to set up 
//@ MPHR_SUBSAT_LATITUDE/LONGITUDE_START/END fields in Level1 products.
//@ See IA-2100-CR-292-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-173-CN : 03/07/2006 : B. MASSART
//@ SD_ALG_40_IAC: initialization of the max correlation point (dlmax,dcmax)
//@ at 1 instead of 0 (patch3-5-5). See IA-2100-FC-173-THA.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-189-CN : 20/10/2006 : C.MERIGUET
//@ Differebces sur les valeurs numeriques du champ RADANALSTD du produit 
//@ ENG selon mode execution nominal ou debug : cast des float en double.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-202-CN : 29/01/2007 : R.MORENO
//@ Pb de gestion des IDefDeltaAvhrr* 
//@ --------------------------------------------------------------------
//@ DM-ID :IA-2100-CR-330-CNE : 06/02/2007 : C.MERIGUET
//@ OffsetImagAvhrr prend la valeur de OffsetGuess.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-211-CNE : 24/05/2007 : S. BRUNEL
//@ SD_ALG_40_IAC: prise en compte de l'OffsetGuess alors que non
//@ requis par le DD et inversion ligne colonne sur l'OffsetGuess.
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-212-CNE : 24/05/2007 : S. BRUNEL
//@ SD_ALG_44_GEO: L'OffsetGuess n'est pas pris en compte en cas degrade.
//@ --------------------------------------------------------------------
//@ DM-ID :IA-2100-CR-338-CNE : 31/05/2007 : S.BRUNEL
//@ 44_GEO: Initialisation des angles de geolocalisation a la valeur initiale
//@ maximum negative des entiers (valeur par defaut des angles non calcules).
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-216-CNE : 12/06/2007 : S. BRUNEL
//@ SD_ALG_44_GEO: Utilisation incorrecte de l'offset, inversion ligne/colonne 
//@ dans le calcul des sous-grilles IIS pour etre conforme au DD.
//@ --------------------------------------------------------------------
//@ DM-ID :IA-FT-2376 : 11/09/2007 : S.BRUNEL
//@ Modification de 40_IAC afin d'eviter GIacCorrelQual 
//@ et GIacPosMaxQual aberrants
//@ --------------------------------------------------------------------
//@ FA-ID :IA-FT-2466(FA-2100-OPS) : 30/05/2008 : S. BRUNEL
//@ Probleme de reproductibilite: SUBSAT_LATITUDE_START et 
//@ SUBSAT_LONGITUDE_START
//@ --------------------------------------------------------------------
//@ FA-ID :IA-FT-2467(FA-2100-OPS) : 13/08/2008 : S. BRUNEL
//@ Probleme de reproductibilitee: GCcsImageClassified.
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2528(DM-2100-OPS) : 17/03/2009 : S. BRUNEL
//@ Ajouter les informations sur les nuages et la surface obtenue
//@ par AVHRR dans le produit L1C (Day 2,point 9)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2521(DM-2100-OPS) : 24/04/2009 : S. BRUNEL
//@ Calcul de 4 pseudo-canaux AVHRR (Day 2, point 2)
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2661(FA-2100-OPS) : 02/09/2009 : S. BRUNEL
//@ Erreur de lecture de IDefAvhrrWgt dans le GRD
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2562(FA-2100-OPS) : 02/09/2009 : S.BRUNEL
//@ Les valeurs par defaut de geolocalisation ne sont pas conformes
//@ a GPS (EUM.EPS.NCR.3742.2)
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2751(FA-2100-OPS) : 22/03/2010 : S.BRUNEL
//@ Probleme de reproductibilite sur les variables 
//@ SUBSAT_XXXX_START et SUBSAT_XXXX_END
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2773(FA-2100-OPS) : 18/05/2010 : S. BRUNEL
//@ Indice de fin de boucle incorrect dans SD-ALG_AVRR.c (EUM/EPS/NCR/12131.3)
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2894(DM-2100-OPS) : 28/06/2011 : S. BRUNEL
//@ Modifications des calculs de TestSign dans AnalRadIntoFov
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2980(DM-2100-OPS) : 05/04/2012 : L. CASPAR
//@ Gestion de la variable GCcsImageClassifiedFirstLin lorsque IDefCCSMode=1
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2997(DM-2100-OPS) : 18/06/2012 : L. CASPAR
//@ Modification des fonctions calcoffset, 41_CCS et 44_GEO pour calculer 
//@ la geolocalisation sans l'AVHRR
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <math.h>
#include <stdlib.h>
#include "SD_ALG_AVHRR.h"
#include "SD_BAS_OperateurBase.h"
#include "SD_BAS_OperateurLoc.h"
#include "SD_BAS_LibMath.h"
#include "SD_BAS_LibESSL.h"
#include "CMN__Date.h"
#include "SD_BAS_Traces.h"
// FT-2765
#ifdef AIX
#include <fp.h>
#endif
// ...FT-2765
#ifdef INVESTIGATION
#include "SD_BAS_Traces.h"
#endif

/*====================================================================*/
// Local Constant Declaration
/*====================================================================*/

/*====================================================================*/
// Local types Declaration
/*====================================================================*/
typedef struct __type_paire_i_val
{
  double val;
  int    i;
} type_paire_i_val;


typedef struct __type_sort_eigen
{
/* bma complexes->reels */
  double val;
  double vect[SD_BAS_NBK];
/* fin bma */
} type_sort_eigen;
typedef enum
{
  SD_ALG_AVHRR_41_CCS_ALRT_TRAIT_CANAUX = ERR_BASE_AVHRR + 1, 
  SD_ALG_AVHRR_41_CCS_ALRT_NB_PIXAV_NUL = ERR_BASE_AVHRR + 2,
  SD_ALG_AVHRR_41_CCS_ALRT            	= ERR_BASE_AVHRR + 3,
  SD_ALG_AVHRR_40_IAC_ALRT            	= ERR_BASE_AVHRR + 4,
  SD_ALG_AVHRR_44_GEO_ALRT            	= ERR_BASE_AVHRR + 5, /* CR292 */
  SD_ALG_AVHRR_GetSubSatLatLon_ALRT     = ERR_BASE_AVHRR + 6
} e_CodeRetour;

//======================================================================
// Internal fonctions Declaration
//======================================================================
/* bma prototype des fonctions */
static int SD_ALG_sort_paire_i_val( const void *a, const void *b);

static int SD_ALG_sort_eigen_val( const void *a, const void *b);

static t_CMN__Statut SD_ALG_GeomIasiMetop(
  // Entrees
  double *MatRot,      // double( SD_BAS_DIM_ROT_MATRICE * SD_BAS_DIM_ROT_MATRICE)
  double  ScanAngle,   // double(1)
  double  PosY,        // double(1)
  double  PosZ,        // double(1)
  // Sorties
  double *azim,        // double(1)
  double *elev         // double(1)
);


static t_CMN__Statut SD_ALG_GeomMetopIasi(
  // Entrees
  double *MatRot,     // double( SD_BAS_DIM_ROT_MATRICE * SD_BAS_DIM_ROT_MATRICE)
  double  ScanAngle,  // double(1)
  double *VectPos,    // double(2) 0:Azim ; 1:Elev
  // Sorties
  double *PosY,       // double(1)
  double *PosZ        // double(1)
);


static t_CMN__Statut SD_ALG_ImageUtile(
  // Entrees
  t_SD_BAS_DataThread  *Contexte_courant,      // t_SD_BAS_DataThread*
  t_SD_BAS_MPHR        *MphrN1,                // t_SD_BAS_MPHR*
  int                   IISNbLin,              // int(1)
  int                   IISNbCol,              // int(1)
  double               *IISY,                  // double( SD_BAS_IMLI)
  double               *IISZ,                  // double( SD_BAS_IMCO)
  double                IISDY,                 // double(1)
  double                IISDZ,                 // double(1)
  double               *IasiMetopMatRot,       // double( SD_BAS_DIM_ROT_MATRICE * SD_BAS_DIM_ROT_MATRICE)
  double                IISRot,                // double(1)
  double                IasiScanAngle,         // double(1)
  t_SD_BAS_SCDSTime     IasiTime,              // t_SD_BAS_SCDSTime*
  double               *OffsetIISAvhrr,        // double( 2 * SD_BAS_SNOT)
  t_SD_BAS_LigneAVHRR **AvhrrListeLin,         // t_SD_BAS_LigneAVHRR * ( AvhrrNbLin)
  int                   AvhrrNbLin,            // int(1)
  int                   AvhrrLinDeb,           // int(1)
  int                   AvhrrLinFin,           // int(1)
  int                   AvhrrColDeb,           // int(1)
  int                   AvhrrColFin,           // int(1)
  int                   AvhrrNbChannel,        // int(1)
  int                   AvhrrChanMax,          // int(1)
  unsigned int         *AvhrrChannelState,     // unsigned int( SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NBK) 
  // Sorties
  float                *AvhrrUtilImage,        // float( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NBK)
                                               //          AvhrrUtilNbCol,         AvhrrUtilNbLin,    AvhrrChanMax
  int                  *AvhrrUtilLinDeb,       // int(1)
  int                  *AvhrrUtilColDeb,       // int(1)
  int                  *AvhrrUtilNbLin,        // int(1)
  int                  *AvhrrUtilNbCol,        // int(1)
  // FT-2528
  unsigned int         *AvhrrUtilChannelState, // unsigned int( SD_BAS_NBK)
  unsigned int         *QualityIndicator,      // unsigned int(SD_BAS_NLI_MAX_UTIL_AVHRR)
  unsigned int         *ScanLineQuality,       // unsigned int(SD_BAS_NLI_MAX_UTIL_AVHRR)
  unsigned short int   *CountErrorFrame,       // unsigned short int(SD_BAS_NLI_MAX_UTIL_AVHRR)
  // FT-2765
  unsigned short int   *CloudInformation       // unsigned short int(SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NPI_AVHRR)
  // ...FT-2765
  // ...FT-2528

);


static t_CMN__Statut SD_ALG_TraitCanauxAvhrr(
  // Entrees
  t_SD_BAS_ConfAutre *ConfAutre,             // Pour les constantes de Planck
  float              *AvhrrUtilImage,        // float( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NBK)
  int                 AvhrrUtilNbLin,        // int(1)
  int                 AvhrrUtilNbCol,        // int(1)
  unsigned int       *AvhrrUtilChannelState, // unsigned int( SD_BAS_NBK)
  int                 CCSRadToTb,            // int(1)
  int                 CCSDimClassSpace,      // int(1)
  int                 CCSSortingChannel,     // int(1)
  double             *CCSChannelWn,          // double( SD_BAS_NBK)
  int                *CCSChannelId,          // int( SD_BAS_NBK)
  double             *CCSChannelNoise,       // double( SD_BAS_NBK)
  int                *CCSChannelWgt,         // int( SD_BAS_NBK)
  // Sorties
  float              *RadAv,    // float( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NBK)
  int                *NbPixAv,  // int(1)
  int                *Nch,      // int(1)
  int                *ChId,     // int( SD_BAS_NBK)
  int                *SortCh,   // int(1)
  double             *ChNoise,  // double( SD_BAS_NBK)
  int                *ChWgt     // int( SD_BAS_NBK)
);


static t_CMN__Statut SD_ALG_PseudoCanauxIIS(
  // Entrees
  t_SD_BAS_ConfAutre *ConfAutre,     // Pour les constantes de Planck
  // Entrees / Sorties
  double             *Image,         // double( SD_BAS_IMCO * SD_BAS_IMLI)
  // Entrees
  int                 Ml,            // int(1)
  int                 Mc,            // int(1)
  int                *IISDeadPix,    // uint( SD_BAS_IMCO * SD_BAS_IMLI)
  double             *CCSChannelWn,  // double( SD_BAS_NBK)
  // Sorties
  float  *PseudoImage,   // float( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NBK)
  float  *RadIIS,        // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
                         // Considere comme NBK vecteurs de NPI_AVHRR * NLI_MAX_UTIL_AVHRR valeurs
  int    *NbPixIIS,      // int(1)
  int    *LinDeb,        // int(1)
  int    *ColDeb,        // int(1)
  int    *NbLin,         // int(1)
  int    *NbCol,         // int(1)
  int    *Nch            // int(1)
);


static t_CMN__Statut SD_ALG_NueesDynamiques(
  // Entrees
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  t_SD_BAS_DataThread  *Contexte_courant,   // t_SD_BAS_DataThread*
  float        *RadAv,               // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
                                     // Considere comme NBK vecteurs de NPI_AVHRR * NLI_MAX_UTIL_AVHRR valeurs
  int           NbPixAv,             // int(1)
  int           Nch,                 // int(1)
  int           SortingChannel,      // int(1)
  double       *ChannelNoise,        // double( SD_BAS_NBK)
  int          *ChannelWgt,          // int( SD_BAS_NBK)
  int           NbClusMax,           // int(1)
  int           NbIterMax,           // int(1)
  double        IterConvCutoff,      // double(1)
  double        SplitCutoff,         // double(1)
  double        GlueCutoff,          // double(1)
  double        EigValCutoff,        // double(1)
  double        DistAgloInit,        // double(1)
  int           MinIterNCCutOff,     // int(1)
  int           NbNonClassifMax,     // int(1)
  double        DistMax,             // double(1)
  // Sorties
  int          *IdPix,               // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  unsigned int *NoExt,               // unsigned int( SD_BAS_NCL_MAX)
  int          *NbClass,             // int(1)
  unsigned int *FlagPostProcessing,  // unsigned int(1)
  double       *NonClassifRate,      // double(1)
  double       *VarianceRate         // double(1)
);

static t_CMN__Statut SD_ALG_IpsfToAvhrr(
  // Entrees pour MetopAVHRR
  t_SD_BAS_DataThread  *Contexte_courant,   // t_SD_BAS_DataThread*
  t_SD_BAS_MPHR        *MphrN1,             // t_SD_BAS_MPHR*
  t_SD_BAS_LigneAVHRR **AvhrrListeLin,      // t_SD_BAS_LigneAVHRR * ( AvhrrNbLin)
  int                   AvhrrNbLin,         // int(1)
  int                   AvhrrLinDeb,        // int(1)
  int                   AvhrrLinFin,        // int(1)
  int                   AvhrrColDeb,        // int(1)
  int                   AvhrrColFin,        // int(1)
  // Entrees
  t_SD_BAS_SCDSTime     IasiTime,           // t_SD_BAS_SCDSTime*
  double                ScanAngle,          // double(1)
  double               *MatRot,             // double( SD_BAS_DIM_ROT_MATRICE * SD_BAS_DIM_ROT_MATRICE)
  int                   NbCol,              // int(1)
  int                   NbLin,              // int(1)
  double               *YPsf,               // double( SD_BAS_NB_COL_SOND)
  double               *ZPsf,               // double( SD_BAS_NB_LIN_SOND)
  double               *WgtPsf,             // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  int                   Osf,                // int(1)
  double               *OffsetIISAvhrr,     // double( SD_BAS_DEUX)
  double               *OffsetSondIIS,      // double( SD_BAS_DEUX)
  double               *OffsetSondAvhrr,    // double( SD_BAS_DEUX)
  int                   dl,                 // int(1)
  int                   dc,                 // int(1)
  // Sorties
  int                  *ColAvFovFirst,      // int(1)
  int                  *ColAvFovLast,       // int(1)
  int                  *LinAvFovFirst,      // int(1)
  int                  *LinAvFovLast,       // int(1)
  int                  *NbPixAvFov,         // int(1)
  double               *AvhrrYpsf,          // double( SD_BAS_IMLI)
  double               *AvhrrZpsf,          // double( SD_BAS_IMCO)
  double               *AvhrrWgtpsf         // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
);

// FT-2528
static t_CMN__Statut SD_ALG_AvhrrCloudInfoIntoFov(
// Entrees
  t_SD_BAS_DataThread  *Contexte_courant,   // t_SD_BAS_DataThread*
  int LinImgFirst,                          // int(1)
  int ColImgFirst,                          // int(1)
  int NbLinImg,                             // int(1)
  int NbColImg,                             // int(1)
  int NbPixImg,                             // int(1)
  unsigned short int *CloudInfo,            // unsigned short int(SD_BAS_NLI_MAX_UTIL_AVHRR*SD_BAS_NPI_AVHRR)
  unsigned int *QualityIndicator,           // unsigned short int(SD_BAS_NLI_MAX_UTIL_AVHRR)
  unsigned int *ScanLineQuality,            // unsigned short int(SD_BAS_NLI_MAX_UTIL_AVHRR)
  unsigned short int *CountErrorFrame,      // unsigned short int(SD_BAS_NLI_MAX_UTIL_AVHRR)
  int ColImgFovFirst,                       // int(1)
  int ColImgFovLast,                        // int(1)
  int LinImgFovFirst,                       // int(1)
  int LinImgFovLast,                        // int(1)
  int NbPixImgFov,                          // int(1)
  double *ImgWgtpsf,                        // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)  > SD_ALG_IpsfToAvhrr(29)
  int EUMAvhrrCldErrorCutoff,               // int(1)
  // Sorties
  unsigned int *GEUMAvhrr1BCldFrac,         // unsigned int(1)
  unsigned int *GEUMAvhrr1BLandFrac,        // unsigned int(1)
  unsigned int *GEUMAvhrr1BQual             // unsigned int(1)
);

static t_CMN__Statut SD_ALG_FlagAvhrrIntoFov(
// Entrees
  t_SD_BAS_DataThread  *ctx,                // t_SD_BAS_DataThread*
  int LinImgFirst,                          // int(1)
  int ColImgFirst,                          // int(1)
  int NbColImg,                             // int(1)
  int NbPixImg,                             // int(1)
  int LinImgFovFirst,                       // int(1)
  int LinImgFovLast,                        // int(1)
  int ColImgFovFirst,                       // int(1)
  int ColImgFovLast,                        // int(1)
  int NbFlag,                               // int(1)
  int *FlagVect,                            // int(NbFlag * NbPixImg)
  int NbPixImgFov,                          // int(1)
  double *ImgWgtpsf,                        // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  // Sorties
  unsigned int *FlagFov                     // int(NbFlag)
);
// ...FT-2528


static t_CMN__Statut SD_ALG_IpsfToIIS(
  // Entrees
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  int                   NbCol,              // int(1)
  int                   NbLin,              // int(1)
  double               *YPsf,               // double( SD_BAS_NB_COL_SOND)
  double               *ZPsf,               // double( SD_BAS_NB_LIN_SOND)
  double               *WgtPsf,             // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  int                   Osf,                // int(1)
  double                IISRot,             // double(1)
  int                   Ml,                 // int(1)
  int                   Mc,                 // int(1)
  double               *IISY,               // double( SD_BAS_IMLI)
  double               *IISZ,               // double( SD_BAS_IMCO)
  double                DY,                 // double(1)
  double                DZ,                 // double(1)
  double               *OffsetSondIIS,      // double( SD_BAS_DEUX)
  // Sorties
  int                  *ColIISFirst,        // int(1)
  int                  *ColIISLast,         // int(1)
  int                  *LinIISFirst,        // int(1)
  int                  *LinIISLast,         // int(1)
  int                  *NbPixIISFov,        // int(1)
  double               *IISYpsf,            // double( SD_BAS_IMLI)
  double               *IISZpsf,            // double( SD_BAS_IMCO)
  double               *IISWgtpsf           // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
);


static t_CMN__Statut SD_ALG_AnaRadIntoFov(
  // Entrees
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  int                   LinImgFirst,     // int(1)
  int                   ColImgFirst,     // int(1)
  int                   NbColImg,        // int(1)
  int                   NbPixImg,        // int(1)
  int                   LinImgFovFirst,  // int(1)
  int                   LinImgFovLast,   // int(1)
  int                   ColImgFovFirst,  // int(1)
  int                   ColImgFovLast,   // int(1)
  float                *RadVect,         // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
  int                  *IdPix,           // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  int                   NbClass,         // int(1)
  int                   Nch,             // int(1)
  int                   ChanMax,         // int(1)
  int                  *ChId,            // int( SD_BAS_NBK)
  double               *ImgWgtpsf,       // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  double               *ImgYpsf,         // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  double               *ImgZpsf,         // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  // Sorties
  double               *RadAnalWgt,      // double( SD_BAS_NCL_MAX)
  double               *RadAnalY,        // double( SD_BAS_NCL_MAX)
  double               *RadAnalZ,        // double( SD_BAS_NCL_MAX)
  double               *RadAnalMean,     // double( SD_BAS_NCL_MAX * SD_BAS_NBK)
  double               *RadAnalStd,      // double( SD_BAS_NCL_MAX * SD_BAS_NBK)
  int                  *NbClassFov       // int(1)
);


static t_CMN__Statut SD_ALG_TestQual(
  // Entrees
  int     NbPixAv,          // int(1)
  int     Nch,              // int(1)
  int     NbClass,          // int(1)
/* bma complexes->reels */
  double *EigVal,           // double( SD_BAS_NBK)
/* fin bma */
  double  EigValCutoff,     // double(1)
  int    *NbPixClas,        // int( SD_BAS_NCL_MAX)
  double *SumDistClas,      // double( SD_BAS_NCL_MAX)
  // Sorties
  double *NonClassifRate,   // double(1)
  double *NormVarRate       // double(1)
);


static t_CMN__Statut SD_ALG_InitRad(
  // Entrees
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  t_SD_BAS_DataThread  *Contexte_courant,   // t_SD_BAS_DataThread*
  float                *RadAv,              // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
  int                   NbPixAv,            // int(1)
  int                   Nch,                // int(1)
  double               *ChannelNoise,       // int( SD_BAS_NBK)
  int                  *ChannelWgt,         // int( SD_BAS_NBK)
  // Sorties
  double                *RadMean,            // float( SD_BAS_NBK)
  double                *RadStd,             // float( SD_BAS_NBK)
  float                *RadNW,              // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
  double               *ProtoZero,          // double( SD_BAS_NBK)
/* bma complexes->reels */
  double               *EigVal,             // double( SD_BAS_NBK)
  double               *EigVect             // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
);


static t_CMN__Statut SD_ALG_InitProto(
  // Entrees
  t_SD_BAS_DataThread* Contexte_courant,      // t_SD_BAS_DataThread*
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  float                *RadN,               // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
  int                   Nch,                // int(1)
  int                   NbPixAv,            // int(1)
  double               *ProtoZero,          // double( SD_BAS_NBK)
  int                   NbClusMax,          // int(1)
  double                DistMax,            // double(1)
  int                   SortingChannel,     // int(1)
  // Sorties
  int                  *NbPro,              // int(1)
  double               *Proto,              // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
  double               *Covar,              // double( SD_BAS_NBK * SD_BAS_NBK * SD_BAS_NCL_MAX)
  double               *Dist,               // double( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NCL_MAX)
  int                  *IdPix,              // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  int                  *NbPixClas,          // int( SD_BAS_NCL_MAX)
  double               *SumDistClas         // double( SD_BAS_NCL_MAX)
);


static t_CMN__Statut SD_ALG_IterNuDyn(
  // Entree
   t_SD_BAS_DataThread*           Contexte_courant,      // t_SD_BAS_DataThread*
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  float                *RadN,                  // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
  int                   NbPixAv,               // int(1)
  int                   Nch,                   // int(1)
  int                   NbPro,                 // int(1)
  int                   NbClusMax,             // int(1)
  int                   NbIterMax,             // int(1)
  double               *EigVal,                // double( SD_BAS_NBK)
  double               *EigVect,               // double( SD_BAS_NBK * SD_BAS_NBK)
  double                DistAgloInit,          // double(1)
  double                SplitCutoff,           // double(1)
  double                GlueCutoff,            // double(1)
  double                EigValCutoff,          // double(1)
  int                   MinIterNCCutoff,       // int(1)
  int                   NbNonClassifMax,       // int(1)
  int                   SortingChannel,        // int(1)
  double                DistMax,               // double(1)
  double                IterConvCutoff,        // double(1)
  // Entrees/Sortie
  double               *Proto,                 // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
  int                  *IdPix,                 // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  int                  *NbPixClas,             // int( SD_BAS_NCL_MAX)
  double               *SumDistClas,           // double( SD_BAS_NCL_MAX)
  // Sortie
  int                  *NbClass,               // int(1)
  unsigned int         *NoExt,                 // unsigned int( SD_BAS_NCL_MAX)
  int                  *Iter                   // int(1)
);


static t_CMN__Statut SD_ALG_IterNuDynPost(
  // Entree
  t_SD_BAS_DataThreadProduit  *ProduitCourant,  // t_SD_BAS_DataThreadProduit*
  int                          IterLast,        // int(1)
  int                          NbPixAv,         // int(1)
  int                          Nch,             // int(1)
  int                          NbClusMax,       // int(1)
/* bma complexes->reels */
  double                      *EigVal,          // double( SD_BAS_NBK)
  double                      *EigVect,         // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
  double                       DistAgloInit,    // double(1)
  double                       EigValCutoff,    // double(1)
  int                          SortingChannel,  // int(1)
  // Entrees / Sortie
  double                      *Proto,           // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
  int                         *IdPix,           // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  int                         *NbPixClas,       // int( SD_BAS_NCL_MAX)
  double                      *SumDistClas,     // double( SD_BAS_NCL_MAX)
  unsigned int                *NoExt,           // unsigned int( SD_BAS_NCL_MAX)
  // Sortie
  int                         *NbClass,         // int(1)
  int                         *Iter             // int(1)
);


static t_CMN__Statut SD_ALG_GestionProto(
  // Entrees
  t_SD_BAS_DataThread* Contexte_courant,      // t_SD_BAS_DataThread*
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  float                       *RadN,               // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
  int                          NbPixAv,            // int(1)
  int                          Nch,                // int(1)
  unsigned int                 Aglo,               // unsigned int(1)
  unsigned int                 Separ,              // unsigned int(1)
/* bma complexes->reels */
  double                      *EigVal,             // double( SD_BAS_NBK)
  double                      *EigVect,            // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
  double                       SplitCutoff,        // double(1)
  double                       DistAgloInit,       // double(1)
  double                       GlueCutoff,         // double(1)
  double                       EigValCutoff,       // double(1)
  // Entrees / Sortie
  double                      *Proto,              // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
  int                         *NbPro,              // int(1)
  int                         *IdPix,              // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  int                         *NbPixClas,          // int( SD_BAS_NCL_MAX)
  double                      *SumDistClas,        // double( SD_BAS_NCL_MAX)
  // Sorties
  unsigned int                *NoExt               // unsigned int( SD_BAS_NCL_MAX)
);


static t_CMN__Statut SD_ALG_GestionProtoPost(
  // Entrees
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  int                          Nch,                // int(1)
/* bma ajout du nombre de pixels AVHRR dans la signature de la fonction GestionProtoPost*/
  int                          NbPixAv,            // int(1)
/* fin bma */
/* bma complexes->reels */
  double                      *EigVal,             // double( SD_BAS_NBK)
  double                      *EigVect,            // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
  double                       DistAgloInit,       // double(1)
  double                       EigValCutoff,       // double(1)
  // Entrees / Sortie
  double                      *Proto,              // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
  int                         *NbPro,              // int(1)
  int                         *IdPix,              // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  int                         *NbPixClas,          // int( SD_BAS_NCL_MAX)
  double                      *SumDistClas,        // double( SD_BAS_NCL_MAX)
  unsigned int                *NoExt               // unsigned int( SD_BAS_NCL_MAX)
);


static t_CMN__Statut SD_ALG_DistPixProto(
  // Entrees
  float  *ImgAv,      // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
  int     NbChannel,  // int(1)
  int     NbPixAv,    // int(1)
  double *Proto,      // double( SD_BAS_NBK)
  // Sorties
  double *Dist        // double( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NCL_MAX)
);


t_CMN__Statut SD_ALG_DistPixProtoClas(
  // Entrees
  float  *ImgAv,      // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
  int     NbChannel,  // int(1)
  int     NbPixAv,    // int(1)
  int    *IdPix,      // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  int     IdPro,      // int(1)
  double *Proto,      // double( SD_BAS_NBK)
  // Sorties
  double *DistClas    // double( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NCL_MAX)
);


static t_CMN__Statut SD_ALG_MinDistAgloLoc(
  // Entrees
  double *DistAglo,    // double( SD_BAS_NCL_MAX * SD_BAS_NCL_MAX)
  int     NbPro,       // int(1)
  // Sorties
  double *DistMin,     // double(1)
  int    *Ind1,        // int(1)
  int    *Ind2         // int(1)
);


static t_CMN__Statut SD_ALG_LinkPixProto(
  // Entrees
  double *Dist,         // double( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NCL_MAX)
  double  DistMax,      // double(1)
  int     NbPixAv,      // int(1)
  int     NbPro,        // int(1)
  // Sorties
  int    *IdPix,        // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  int    *NbPixClas,    // int( SD_BAS_NCL_MAX)
  double *SumDistClas,  // double( SD_BAS_NCL_MAX)
  double *SumDistTot    // double(1)
);


static t_CMN__Statut SD_ALG_SortClas(
  // Entrees
  int     SortingChannel,  // int(1)
  int     NbChannel,       // int(1)
  int     NbPixAv,         // int(1)
  int     NbPro,           // int(1)
  // Entrees / Sorties
  double *Proto,           // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
  int    *IdPix,           // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  int    *NbPixClas,       // int( SD_BAS_NCL_MAX)
  double *SumDistClas      // double( SD_BAS_NCL_MAX)
);


static t_CMN__Statut SD_ALG_StatClas(
  // Entrees
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  float                *ImgAv,              // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
  int                  *IdPix,              // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  int                   NbChannel,          // int(1)
  int                   NbPixAv,            // int(1)
  int                   IdPro,              // int(1)
  // Sorties
  double               *Proto,              // double( SD_BAS_NBK)
  double               *Covar               // double( SD_BAS_NBK * SD_BAS_NBK)
);


static t_CMN__Statut SD_ALG_TransformIpsf_CCS(
  // Entrees
  int       NbLin,             // int(1)
  int       NbCol,             // int(1)
  double   *YPsf,              // double( SD_BAS_NB_COL_SOND) Limite a NbCol
  double   *ZPsf,              // double( SD_BAS_NB_LIN_SOND) Limite a NbLin
  double   *Wgtpsf,            // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  int       LinDeb,            // int(1)
  int       ColDeb,            // int(1)
  int       LinFin,            // int(1)
  int       ColFin,            // int(1)
  int       Mc,		       // nb col ImgY et ImgZ,
  double   *ImgY,              // double( NbLinAvFov * NbColAvFov)
  double   *ImgZ,              // double( NbLinAvFov * NbColAvFov)
  double    DY,                // double(1)
  double    DZ,                // double(1)
  double   *OffsetSond,        // double( SD_BAS_DEUX)
  double    OverSampFactor,    // double(1)
  // Sorties
  double   *ImgWgtpsf,         // double( NbLinAvFov * NbColAvFov)
  double   *ImgYpsf,           // double( NbLinAvFov * NbColAvFov)  Alloue: double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  double   *ImgZpsf            // double( NbLinAvFov * NbColAvFov)  Alloue: double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
);


static t_CMN__Statut SD_ALG_Tri_ascendant(
  // Entrees
  t_SD_BAS_DataThread* ctx,      // t_SD_BAS_DataThread*
  double   *Vecteur,        // double( NbElem)
  int       NbElem,         // int(1)
  // Sorties
  int      *IndicesClasses  // int( NbElem)
);


static double SD_ALG_MoyenneLocale(
  double *ImageIIS,   // double( NbCol * NbLin)
  int     lin,        // int(1)
  int     col,        // int(1)
  int     NbLin,      // int(1)
  int     NbCol       // int(1)
);


static double SD_ALG_VarianceLocale(
  double   *ImageIIS,   // double( NbCol * NbLin)
  int       lin,        // int(1)
  int       col,        // int(1)
  int       NbLin,      // int(1)
  int       NbCol,      // int(1)
  double    Moy         // double(1)
);


static double SD_ALG_Moyenne(
  // Entrees
  float   *ImgAv,     // float( NbPixAv)
  int      NbPixAv,   // int(1)
  int     *IdPix,     // int( NbPixAv)
  int      IdPro      // int(1)
);


static t_CMN__Statut SD_ALG_EigenAnalysis(
  // Entrees
  t_SD_BAS_DataThread *Contexte_courant,   // t_SD_BAS_DataThread*
  int                  Taille,             // int(1) Nombre de points
  double              *Covar,              // double( Taille * Taille),
  unsigned int         Transpose,          // unsigned int(1)
  // Sorties
/* bma complexes->reels */
  double              *EigVal,             // double( Taille)
  double              *EigVect             // double( Taille * Taille)
/* fin bma */
);


static t_CMN__Statut SD_ALG_ConstruireAvhrrChannelState(
  // Entrees
  t_SD_BAS_LigneAVHRR  **AvhrrListeLin,      // t_SD_BAS_LigneAVHRR( AvhrrNbLin)
  int                    AvhrrNbLin,         // int(1)
  int                    AvhrrNbChannel,     // int(1)
  // Sorties
  unsigned int          *AvhrrChannelState   // int( SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NBK)
);

static  t_CMN__Statut SD_ALG_Index_De_Max3(
     // Entrees
  t_SD_BAS_DataThread* ctx,      // t_SD_BAS_DataThread*
     int     NbElem,         // int(1)
     double *Vecteur,        // double( NbElem)
     // sorties
     int     *i3           // int(1)
);
/* fin bma */

//======================================================================
// Global variables Definition
//======================================================================
static char *SD_BAS_MSG_ERR_AVHRR2METOP="Error during SD_BAS_AVHRRMetop";
static char *SD_ALG_MSG_ERR_IMAGEUTILE="Error during SD_ALG_IMAGEUTILE";
static char *SD_ALG_MSG_IPSFTOAVHRR="Error during SD_ALG_IPSTTOAVHRR";
static char *SD_ALG_MSG_CALCOFFSET="Error during SD_ALG_CALCOFFSET";
static char *SD_ALG_MSG_NUEESDYNAMIQUES="Error during SD_ALG_NueesDynamiques";


static char *SD_BAS_MSG_ERR_SAMPLING="sub sampling problem";
static char *SD_BAS_MSG_ERR_INTERP="interpolation problem";
static char *SD_BAS_MSG_ERR_METOP2AVHRR="Error during SD_BAS_MetopAVHRR";
static char *SD_BAS_MSG_ERR_DOMAIN="AVHRR domain error : not in bandeau";
static char *SD_ALG_MSG_ERR_MOINDRES_CARRES="Error during MoindreCarre1D";
static char *SD_ALG_MSG_ERR_CHANNEL_UNAVAILABLE="AVHRR channel not available";
static char *SD_ALG_MSG_ERR_NUL_DISTANCE="Nul prototype distance";
static char *SD_BAS_MSG_ERR_NUL_EIG_VAL="Nul Eigen value";
static char *SD_BAS_MSG_ERR_EIG_VAL_PROCESSING="Eigen value processing";
static char *SD_BAS_MSG_ERR_METOP2IASI="Normalization error SD_BAS_GeomMetopIasi";
//FA153
static int SubsatDebFait = FALSE;

/* CR242 */
static char *SD_BAS_MSG_ERR_METOP2GEOD="Error during SD_BAS_MetopGeod";

// FT-2765
static const double PRECISION_AJUSTEMENT_MC=0.0001;
// ...FT-2765

const int   SD_ALG_INIT_INTERP2D         = 1;
const int   SD_ALG_CALCUL_INTERP2D       = 0;

const int   SD_BAS_MARGE_IAC                  = 2;

//const int   CANAL_AVHRR                  = 4;

const double MAX_DOUBLE=   1.0E100;
const double MIN_DOUBLE=  -1.0E100;
const int    MAX_INT=      2147483646;
const int    MIN_INT=     -2147483646;


// Les deux constantes ci-dessous doivent etre supprimees des lors que les
//valeurs auront ete ajoutees au fichier confautre.
//const int      IDefAvhrrSubGridDl          = 2;    // Pas de sousechantillonnage ligne AVHRR
//const int      IDefAvhrrSubGridDc          = 2;    // Pas de sousechantillonnage colonne AVHRR

int             SD_ALG_AVHRR_MsgSet;
int             SD_ALG_AVHRR_MsgNum;

//======================================================================
//                  FONCTIONS EXTERNES
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_40_IAC
//@
//@ Description : algorithme de co-registration Imageur-AVHRR
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  ctp : variables temporaires
//@         (ENTREE)  Contexte : Contexte
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (ENTREE)  ConfStable : fichier de conf stable
//@         (ENTREE)  Mphr :  MPHR produit N1
//@         (ENTREE)  GIrcImg : image calibree
//@         (ENTREE)  BandeauAVHRR : bandeau AVHRR charge en memoire
//@         (ENTREE)  nbligbandeau : nombre de lignes total du bandeau
//@         (Entree/Sortie)  InfoDet1ENG : info det du produit ENG
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_40_IAC(
        t_SD_BAS_DataThread * ctx,
        t_SD_BAS_DataThreadProduit *ctp,
        t_SD_BAS_Contexte *Contexte,
        t_SD_BAS_ConfAutre *  ConfAutre,
        t_SD_BAS_ConfStable *  ConfStable,
        t_SD_BAS_MPHR * Mphr,
        double * GIrcImg,
        t_SD_BAS_LigneAVHRR** BandeauAVHRR,
        int nbligbandeau,
        t_SD_BAS_Liste_Var *InfoDet1ENG
            )
{
static char tcProcName_F[] = "SD_ALG_40_IAC";
t_CMN__Statut cr_op=CMN__STATUT_OK;
// Entree dans la fonction


    // Recuperation des infos contenues dans les structure
    int                 SN;             // Numero de sous cycle
    int                 SP;

    // Entrees
    t_SD_BAS_SCDSTime   IasiTime;
    double              IasiTimeSec;
    double              ScanAngle;
    double              *MatRot;
    double              *OffsetGuess;
    int                 Ml;
    int                 Mc;
    double              *IISY;
    double              *IISZ;
    double              IISRot;
    double              *Z;             // Image IIS
    float               *Y;             // Image AVHRR
    int                 AvhrrColLeft;
    int                 AvhrrColRight;
    int                 AvhrrLigFirst;
    int                 AvhrrLigLast;
    double              AvhrrDatBefore;
    double              AvhrrDatAfter;


    int                 SGdl;
    int                 SGdc;
    int                 Dl;
    int                 Dc;
    double              CorrelCutoff;
    double              PosMaxCutoff;
    int                *IISDeadPix;

    // Sorties
    double              CorrelQual;
    double              PosMaxQual;
    int                 FlagCoregNonQual;
    int                 FlagCoregNonValid;
    double              OffsetImagAvhrr[2];



    // Variables internes
    double              IISCornerY[4];
    double              IISCornerZ[4];
    double              CornerY[4];
    double              CornerZ[4];
    double              IISCornerMetop[8];          // Les 4 azims suivis des 4 elev
    double              moyPixIIS;                  // Pour interpoler les pixels IIS morts
    int                 nbPixMoy;                   // Nombre de pixels utilises pour l'interpolation
    double              *ls;                        // Lignes de la sousgrille de l'imagette AVHRR
    double              *cs;                        // Lignes de la sousgrille de l'imagette AVHRR
    int                 nls;                        // Nombre de lignes de la sousgrille de l'imagette AVHRR
    int                 ncs;                        // Nombre de colonnes de la sousgrille de l'imagette AVHRR
    double  *            ImgAvLinSubG;               // Coordonnees lignes des pixels de la sousgrille dans le repere AVHRR
    double  *            ImgAvColSubG;               // Coordonnees colonnes des pixels de la sousgrille dans le repere AVHRR
    double  *            ImgAvMetopSubGAzim;          // Azim et elev dans le repere Metop
    double  *            ImgAvMetopSubGElev;          // Azim et elev dans le repere Metop
    double              ImgAvMetopSubG[2];          // Azim et elev dans le repere Metop
    double              ImgAvYSubG;
    double              ImgAvZSubG;
    double              *IISImgAvYSubG;
    double              *IISImgAvZSubG;
    double              *IISImgAvY;
    double              *IISImgAvZ;
    double              *lin;
    double              *col;
    double              *X;                         // Pour le reech de l'imagette IIS dans le raster AVHRR
    double              *ptX;                       // Pointeur sur X
    int                 lin0;                       // ligne du pt precedent le debut de l'image IASI dans le raster AVHRR
    int                 col0;                       // colonne du pt precedent le debut de l'image IASI dans le raster AVHRR
    double              Xbar2;
    double              Xbar;
    double              X2bar;
    double              XYbar;
    double              deltaY;
    double              deltaY2;
    int                 NbLigR;                     // Nombre de lignes de R
    int                 AncColon;
    int                 NouvColon;
    int                 AncLigne;
    int                 NouvLigne;
    int                 NbLigAvhrr;                 // Nombre de lignes de l'image AVHRR
    int                 DebColon;
    int                 FinColon;
    int                 DebLigne;
    int                 FinLigne;
    int                 LinAvFirst;
    int                 LinAvEnd;
    int                 ColAvFirst;
    int                 ColAvEnd;
    double              Rmax;                       // Max de la nappe de correlation
    int                 dlmax;                      // Ligne du max de la nappe
    double              dlm;                        // Idem a resolution subpixellaire
    int                 dcmax;                      // Colonne du max de la nappe
    double              dcm;                        // Idem a resolution subpixellaire
    int                 nlR;                        // Nombre de lignes de la nappe
    int                 ncR;                        // Nombre de colonnes de la nappe
    double              datetmp;                    // Date temporaire
    int                 nl;
    int                 nc;
    int                 Nl;
    int                 Nc;

    // Pour les appels a SD_BAS_OperateurLoc
    double              IISCornerAvhrrLin[4];       // Lignes AVHRR dans la conversion Metop2AVHRR
    double              IISCornerAvhrrCol[4];       // Colonnes AVHRR dans la conversion Metop2AVHRR
    double              EarthSatDist[4];            // Distance Terre/Satellite dans la conversion Metop2AVHRR
    double              lat[4];
    double              lon[4];
    double              Ang_mesY[4];
    double              Ang_mesZ[4];
    double              Azim_sol[4];
    double              Elev_sol[4];

    // Pour l'estimation d'extremum 2D par moindres carres + paraboloide
    int     nbeq = 9;           // Nombre d'equations
    int     degre = 5;          // Degre du polynome d'ajustement
    double  a[54];              // Matrice A 6x9 de la resolution de AX = B (9 equations,6inconnues)
    double  b[9];               // Vecteur B 9 de la resolution de AX = B
    double  tabTmpMC[9];        // Tableau temporaire pour moindres carres
    double  x[6];               // Coeffs du paraboloide ajuste
    double  eps;                // Moyenne quadratique des residus d'ajustement
    double  res[4];             // Resultat
    int     nlinm;
    int     nlinp;
    int     ncolm;
    int     ncolp;
    double  *ratio;
    double *Ytemp;
    char LibelleTraces[256];
    

    // Statuts de sortie des fonctions appelees
    t_CMN__Statut irc;

    // Gestion de erreurs
/* bma intialisation */
/*
    char *msg_err;          // Message d'erreur
    int num_lig;            // Ligne d'erreur
*/
    char *msg_err = (char*) NULL;          // Message d'erreur
    int num_lig = 0;            // Ligne d'erreur
/* fin bma */

    // Compteurs
    int i;
    int l;
    int c;
    int il;
    int ic;
    int ct;

    // Variables temporaires
    double  dtmp,dtmp1,dtmp2;
    double  minimum;
    int indice,j;

    #ifdef OPS_DEBUG
        printf("Entree dans la fonction 40 IAC\n========================================\n");
    #endif


    // Initialisations
    SN              = ctx->ctxtrt.SN;
    SP              = InfoDet1ENG->tab_var[ENG_GEPS_SP].val.pui4[IND1(SN)];

    IasiTime        = InfoDet1ENG->tab_var[ENG_GEPSDATIASI].val.pdate[IND1(SN)];
    IasiTimeSec     = CMN__DATE_CONV_SCDS_2_SEC(IasiTime);
    ScanAngle       = ConfAutre->IDefIASIScanAngle[IND1(SP)];
    MatRot          = ConfAutre->IDefIASIMETOPMatRot;
    Ml              = ConfAutre->IDefIISNbLin;
    Mc              = ConfAutre->IDefIISNbCol;
    IISY            = ConfAutre->IDefIISY;
    IISZ            = ConfAutre->IDefIISZ;
    IISRot          = ConfAutre->IDefIISRot;
    Z               = GIrcImg;
// version 1.6 conf autre
//    SGdl            = IDefAvhrrSubGridDl;
    SGdl            = ConfStable->IDefAvhrrSubGridDl;
//    SGdc            = IDefAvhrrSubGridDc;
    SGdc            = ConfStable->IDefAvhrrSubGridDc;
    OffsetGuess     = ConfAutre->IDefIISAVHRROffsetGuess;      // Ordre (colonne, ligne)
    Dl              = ConfAutre->IDefIacDLin;
    Dc              = ConfAutre->IDefIacDCol;
    nlR             = 2*Dl+1;       // A verifier
    ncR             = 2*Dc+1;       // A verifier

    CorrelCutoff    = ConfAutre->IDefIacCorrelCutoff;
    PosMaxCutoff    = ConfAutre->IDefIacPosMaxCutoff;
    IISDeadPix      = ConfStable->IDefDptIISDeadPix;

  // FA 202
  if(34 != SP)
     {
     /* FA142 */
     AvhrrDatBefore  = IasiTimeSec - ConfStable->IDefDeltaAvhrrDTbefore[IND1(SP)];
     AvhrrDatAfter   = IasiTimeSec + ConfStable->IDefDeltaAvhrrDTafter[IND1(SP)];

     AvhrrColLeft    = ConfStable->IDefDeltaAvhrrColLeft[IND1(SP)];
     AvhrrColRight   = ConfStable->IDefDeltaAvhrrColRight[IND1(SP)];
     }
  else
     {
     /* FA142 */
    AvhrrDatBefore  = IasiTimeSec - ConfStable->IDefDeltaAvhrrDTbefore[IND1(15)];
    AvhrrDatAfter   = IasiTimeSec + ConfStable->IDefDeltaAvhrrDTafter[IND1(15)];

    AvhrrColLeft    = ConfStable->IDefDeltaAvhrrColLeft[IND1(16)];
    AvhrrColRight   = ConfStable->IDefDeltaAvhrrColRight[IND1(15)];
     }
     
  // FIN FA 202
    
//  initialisations 
    ratio=NULL;
    IISImgAvY=NULL;  
    IISImgAvZ=NULL;
     ImgAvLinSubG=NULL;
    ImgAvColSubG=NULL;    
    ImgAvMetopSubGAzim=NULL;
    ImgAvMetopSubGElev=NULL;    
    ls=NULL;
    cs=NULL;
    lin=NULL;
    col=NULL;
    IISImgAvYSubG=NULL;
    IISImgAvZSubG=NULL;
    Y=NULL;
    X=NULL;

    CorrelQual      = 0.0;
    PosMaxQual      = 0.0;
    dlm             = 0.0;
    dcm             = 0.0;

/*  initialisations des variables mises dans les produits */
    CorrelQual=0.;
    PosMaxQual=0.;
    
    /* FA134 */
    FlagCoregNonQual=CMN__NON;
    FlagCoregNonValid=CMN__NON;
    
    OffsetImagAvhrr[0]=0.;
    OffsetImagAvhrr[1]=0.;
    
    #ifdef OPS_DEBUG
        printf("Fin des initialisations des parametres\n");
            printf("   Bandeau AVHRR    : %d lignes\n",nbligbandeau);
            printf("   AVHRR DateBefore : %lf\n",AvhrrDatBefore);
            printf("   AVHRR DateAfter  : %lf\n",AvhrrDatAfter);
            printf("   AVHRR ColLeft    : %d\n",AvhrrColLeft);
            printf("   AVHRR ColRight   : %d\n",AvhrrColRight);
    #endif



    #ifdef OPS_DEBUG
        printf("Calcul des premieres et dernieres lignes de l'image AVHRR en fonction des dates\n");
    #endif


    // Calcul des numeros de premiere et derniere ligne de l'image AVHRR en fonction des dates
    //------------------------------------------------------------------------
    // Premiere ligne
    AvhrrLigFirst=0;
    il=0;
    datetmp = CMN__DATE_CONV_SCDS_2_SEC(*(BandeauAVHRR[il]->InfoGene.tab_var[AVHRR_DATEDEB].val.pdate));
    while(datetmp <= AvhrrDatBefore && il < nbligbandeau)
    {
        datetmp = (double)CMN__DATE_CONV_SCDS_2_SEC(*(BandeauAVHRR[il]->InfoGene.tab_var[AVHRR_DATEDEB].val.pdate));
        il++;
        #ifdef OPS_DEBUG
            printf("   DATEDEB de la ligne %d : %lf\n",il,datetmp);
        #endif
    }
    AvhrrLigFirst = il-1;
    if(AvhrrLigFirst<0) AvhrrLigFirst=0;

    #ifdef OPS_DEBUG
        printf("   Ligne AVHRR correspondant a  DATEDEB : %d\n",AvhrrLigFirst);
    #endif

    // Derniere ligne
    AvhrrLigLast=0;
    il=0;
    datetmp = CMN__DATE_CONV_SCDS_2_SEC(*(BandeauAVHRR[il]->InfoGene.tab_var[AVHRR_DATEFIN].val.pdate));
    while(datetmp < AvhrrDatAfter && il < nbligbandeau)
    {
        datetmp = CMN__DATE_CONV_SCDS_2_SEC(*(BandeauAVHRR[il]->InfoGene.tab_var[AVHRR_DATEFIN].val.pdate));
        il++;
    #ifdef OPS_DEBUG
        printf("   DATEFIN de la ligne %d : %lf\n",il,datetmp);
    #endif

    }
    AvhrrLigLast = il;
    if(AvhrrLigLast>nbligbandeau-1) AvhrrLigLast=nbligbandeau-1;

    #ifdef OPS_DEBUG
        printf("   Ligne AVHRR correspondant a  DATEFIN : %d\n",AvhrrLigLast);
    #endif


    // Initialisation de l'image AVHRR
    //------------------------------------------------------------------------
    NbLigAvhrr=AvhrrLigLast-AvhrrLigFirst+1;
    // Allocation de la memoire
    if(SD_BAS_MacMALLOC(Y,  float,(NbLigAvhrr)*(AvhrrColRight-AvhrrColLeft+1)) == FALSE)
    {
        cr_op   = SD_ALG_AVHRR_ERR_ALLOCATION;
        msg_err = (char *)SD_BAS_MSG_ALLOC;
        num_lig = __LINE__;

        goto fin;
    }

    ct=0;
    for(ic=AvhrrColLeft,ct=0; ic<=AvhrrColRight; ic++)
    {
        for(il=AvhrrLigFirst; il<=AvhrrLigLast; il++,ct++)
        {
             Y[ct] = BandeauAVHRR[IND1(il+1)]->Image.im[IND2(ic,ConfStable->IDefIacAvhrrChannel,SD_BAS_NPI_AVHRR)];
//            Ytemp[ct] = BandeauAVHRR[IND1(il+1)]->Image.im[IND2(ConfStable->IDefIacAvhrrChannel,ic,5)];
        }
    }

    #ifdef OPS_DEBUG
        printf("Fin de l'initialisation de l'image AVHRR\n");
    #endif

//    goto SUITE;
    //@ Calcul des coordonnees des 4 coins de l'imagette IIS dans le repere IIS
    IISCornerY[IND1(1)]   = IISY[IND1(1)];
    IISCornerY[IND1(2)]   = IISY[IND1(Mc)];
    IISCornerY[IND1(3)]   = IISY[IND1(Mc)];
    IISCornerY[IND1(4)]   = IISY[IND1(1)];
    IISCornerZ[IND1(1)]   = IISZ[IND1(1)];
    IISCornerZ[IND1(2)]   = IISZ[IND1(1)];
    IISCornerZ[IND1(3)]   = IISZ[IND1(Ml)];
    IISCornerZ[IND1(4)]   = IISZ[IND1(Ml)];


    #ifdef OPS_DEBUG
        printf("Calcul des coordonnees des 4 coins de l'imagette IIS dans le repere IIS\n");
        printf("   Nb lignes IIS : %d\n",Ml);
        printf("   Nb colonnes IIS : %d\n",Mc);
        printf("   IISCornerY 1 2 3 4 : %lf\t%lf\t%lf\t%lf\n",IISCornerY[0],IISCornerY[1],IISCornerY[2],IISCornerY[3]);
        printf("   IISCornerZ 1 2 3 4 : %lf\t%lf\t%lf\t%lf\n",IISCornerZ[0],IISCornerZ[1],IISCornerZ[2],IISCornerZ[3]);
    #endif

    //@ Boucle sur les pixels i=1,4
    #ifdef OPS_DEBUG
        printf("Calcul des coordonnees des 4 coins de l'imagette IIS dans les repere IASI et METOP\n");
        printf("Angle IISRot : %lf\n",IISRot);
        // Utilisation de la sous-matrice | A B C |
        // 2 x 2 relative a Y et Z dans   | D E F | => | E F |
        // la matrice 3 x 3 (MatRot)      | G H K |    | H K |
        printf("Matrice MatRot : %lf\t%lf\n",MatRot[IND2(2,2,SD_BAS_DIM_ROT_MATRICE)],
                                             MatRot[IND2(3,2,SD_BAS_DIM_ROT_MATRICE)]);
        printf("                 %lf\t%lf\n",MatRot[IND2(2,3,SD_BAS_DIM_ROT_MATRICE)],
                                             MatRot[IND2(3,3,SD_BAS_DIM_ROT_MATRICE)]);
        printf("ScanAngle : %lf\n",ScanAngle);

    #endif
    for(i=1; i<=4; i++)
    {
         //@ Calcul des coordonnees des 4 coins de l'imagette IIS dans le repere IASI
        CornerY[IND1(i)] = IISCornerY[IND1(i)]*cos(IISRot) - IISCornerZ[IND1(i)]*sin(IISRot);
        CornerZ[IND1(i)] = IISCornerY[IND1(i)]*sin(IISRot) + IISCornerZ[IND1(i)]*cos(IISRot);

        #ifdef OPS_DEBUG
            printf("Pixel no%d\n",i);
            printf("   IASI CornerY : %lf\n",CornerY[IND1(i)] );
            printf("   IASI CornerZ : %lf\n",CornerZ[IND1(i)] ); 
        #endif


        //@ Calcul des coordonnees des 4 coins dans le repere MetopIIS
        SD_ALG_GeomIasiMetop(MatRot,ScanAngle,CornerY[IND1(i)],CornerZ[IND1(i)],
                             &IISCornerMetop[i-1],  // Azim du pixel i
                             &IISCornerMetop[i+3]   // Elev du pixel i
        );
        #ifdef OPS_DEBUG
            printf("   METOP Azim : %lf\n",IISCornerMetop[i-1]);
            printf("   METOP Elev : %lf\n",IISCornerMetop[i+3]);
        #endif
     }
     //@ Calcul des coordonnees des 4 coins dans le repere Avhrr :
//     SD_BAS_ReserverOperateur();
     irc = SD_BAS_MetopAVHRR(ctx,&IasiTime,
                             Mphr,4 /* Pour les 4 coins */,
                             IISCornerMetop + 0, // tableau des 4 azims
                             IISCornerMetop + 4, // tableau des 4 elev
                             AvhrrLigFirst,AvhrrLigLast,BandeauAVHRR,
                             nbligbandeau,AvhrrColLeft,AvhrrColRight,
                             lat,lon,
                             Ang_mesY,Ang_mesZ,
                             Azim_sol,Elev_sol,
                             IISCornerAvhrrCol,IISCornerAvhrrLin,EarthSatDist);

//      SD_BAS_LibererOperateur();
     if(irc != CMN__STATUT_OK)
     {
         cr_op   = SD_ALG_AVHRR_40_IAC_ALRT;
         msg_err = (char *)SD_BAS_MSG_ERR_METOP2AVHRR;
         num_lig = __LINE__;
         goto fin;
     }

    #ifdef OPS_DEBUG
 /*       IISCornerAvhrrLin[0] = 3.0; // Pour pouvoir continuer -> a virer des que ca marche
        IISCornerAvhrrLin[1] = 10.0; // Pour pouvoir continuer -> a virer des que ca marche
        IISCornerAvhrrLin[2] = 11.0; // Pour pouvoir continuer -> a virer des que ca marche
        IISCornerAvhrrLin[3] = 4.5; // Pour pouvoir continuer -> a virer des que ca marche
        IISCornerAvhrrCol[0] = 4.0; // Pour pouvoir continuer -> a virer des que ca marche
        IISCornerAvhrrCol[1] = 5.0; // Pour pouvoir continuer -> a virer des que ca marche
        IISCornerAvhrrCol[2] = 9.0; // Pour pouvoir continuer -> a virer des que ca marche
        IISCornerAvhrrCol[3] = 9.5; // Pour pouvoir continuer -> a virer des que ca marche */

        printf("Calcul des coordonnees des 4 coins dans le repere AVHRR\n");
        printf("   Pixel 1 : (ligAVHRR,colAVHRR) = (%lf,%lf)\n",IISCornerAvhrrLin[0],IISCornerAvhrrCol[0]);
        printf("   Pixel 2 : (ligAVHRR,colAVHRR) = (%lf,%lf)\n",IISCornerAvhrrLin[1],IISCornerAvhrrCol[1]);
        printf("   Pixel 3 : (ligAVHRR,colAVHRR) = (%lf,%lf)\n",IISCornerAvhrrLin[2],IISCornerAvhrrCol[2]);
        printf("   Pixel 4 : (ligAVHRR,colAVHRR) = (%lf,%lf)\n",IISCornerAvhrrLin[3],IISCornerAvhrrCol[3]);
    #endif


    //@   Determination du domaine Avhrr utile contenu dans l'imagette IIS
    //@   en tenant compte de l'offset a priori definie par
    //@   LinAvFirst  < ligne < LinAvEnd
    //@   ColAvFirst < colonne < ColAvEnd ( ce domaine n'est surement pas un rectangle)
    // on range les valeurs par ordre croissant et on s'interesse aux valeurs intermediaires (inscrit)
    for (i=1;i<=4;i++)
     {
     minimum=IISCornerAvhrrLin[IND1(i)];
     indice = i;
     for (j=i+1;j<=4;j++)
      {
      if (IISCornerAvhrrLin[IND1(j)] < minimum)
        {
        minimum=IISCornerAvhrrLin[IND1(j)];
        indice = j;
        }
      }
      IISCornerAvhrrLin[IND1(indice)] = IISCornerAvhrrLin[IND1(i)];
      IISCornerAvhrrLin[IND1(i)] = minimum;
     }
     /* FA 211 */
     LinAvFirst  = (int) ( ceil(IISCornerAvhrrLin[IND1(2)]));   
     LinAvEnd    = (int) (floor(IISCornerAvhrrLin[IND1(3)])); 
     /* .. FA 211 */
    for (i=1;i<=4;i++)
     {
     minimum=IISCornerAvhrrCol[IND1(i)];
     indice = i;
     for (j=i+1;j<=4;j++)
      {
      if (IISCornerAvhrrCol[IND1(j)] < minimum)
        {
        minimum=IISCornerAvhrrCol[IND1(j)];
        indice = j;
        }
      }
      IISCornerAvhrrCol[IND1(indice)] = IISCornerAvhrrCol[IND1(i)];
      IISCornerAvhrrCol[IND1(i)] = minimum;
     }
     /* FA 211 */
     ColAvFirst  = (int) ( ceil(IISCornerAvhrrCol[IND1(2)]));   
     ColAvEnd    = (int) (floor(IISCornerAvhrrCol[IND1(3)]));
     /* .. FA 211 */
//    LinAvFirst  = (int)ceil(MAX(IISCornerAvhrrLin[IND1(1)],IISCornerAvhrrLin[IND1(4)]));   // Verifier le sens des reperes
//    LinAvEnd    = (int)floor(MIN(IISCornerAvhrrLin[IND1(2)],IISCornerAvhrrLin[IND1(3)]));   // Verifier le sens des reperes
//    ColAvFirst  = (int)ceil(MAX(IISCornerAvhrrCol[IND1(3)],IISCornerAvhrrCol[IND1(4)]));   // Verifier le sens des reperes
//    ColAvEnd    = (int)floor(MIN(IISCornerAvhrrCol[IND1(1)],IISCornerAvhrrCol[IND1(2)]));   // Verifier le sens des reperes
    LinAvFirst  =  LinAvFirst + SD_BAS_MARGE_IAC;   // marge due a la rotondite
    LinAvEnd    =  LinAvEnd - SD_BAS_MARGE_IAC;   // marge due a la rotondite
    ColAvFirst  =  ColAvFirst + SD_BAS_MARGE_IAC;   // marge due a la rotondite
    ColAvEnd    =  ColAvEnd - SD_BAS_MARGE_IAC;   //  marge due a la rotondite

    #ifdef OPS_DEBUG
        printf("Determination du domaine Avhrr utile contenu dans l'imagette IIS\n");
        printf("   LinAvFirst : %d\n",LinAvFirst);
        printf("   LinAvEnd : %d\n",LinAvEnd);
        printf("   ColAvFirst : %d\n",ColAvFirst);
        printf("   ColAvEnd : %d\n",ColAvEnd);
    #endif




    //@---------------------------------
    //@ ATTENTION : il faut prevoir ici le cas ou le domaine AVHRR n'est pas dans le bandeau
    //@ en entree dans ce cas on retourne une erreur de traitement afin de passer dans CG9
    //@ pour cela il faut extraire le domaine AVHRR precedent (verification de la
    //a presence de toutes les lignes)
    //@-----------------------------------
    // la colonne commence a 1 et la ligne a 0
    if(    LinAvFirst<0 || LinAvEnd>nbligbandeau-1 || LinAvFirst > LinAvEnd
        || ColAvFirst<1 || ColAvEnd>SD_BAS_NPI_AVHRR-1 || ColAvFirst > ColAvEnd)
    {
        printf("Calcul des coordonnees des 4 coins dans le repere AVHRR\n");
        printf("   Pixel 1 : (ligAVHRR,colAVHRR) = (%lf,%lf)\n",IISCornerAvhrrLin[0],IISCornerAvhrrCol[0]);
        printf("   Pixel 2 : (ligAVHRR,colAVHRR) = (%lf,%lf)\n",IISCornerAvhrrLin[1],IISCornerAvhrrCol[1]);
        printf("   Pixel 3 : (ligAVHRR,colAVHRR) = (%lf,%lf)\n",IISCornerAvhrrLin[2],IISCornerAvhrrCol[2]);
        printf("   Pixel 4 : (ligAVHRR,colAVHRR) = (%lf,%lf)\n",IISCornerAvhrrLin[3],IISCornerAvhrrCol[3]);

        printf("Determination du domaine Avhrr utile contenu dans l'imagette IIS\n");
        printf("   LinAvFirst : %d\n",LinAvFirst);
        printf("   LinAvEnd : %d\n",LinAvEnd);
        printf("   ColAvFirst : %d\n",ColAvFirst);
        printf("   ColAvEnd : %d\n",ColAvEnd);

        cr_op   = SD_ALG_AVHRR_ERR_DOMAIN;
        msg_err = (char *)SD_BAS_MSG_ERR_DOMAIN;
        num_lig = __LINE__;

        goto fin;
    }

    //@   Nl = LinAvEnd - LinAvFirst + 1 cela sort d'ou ?
    //@   Nc = ColAvEnd - ColAvFirst + 1
    //@   NbPixAv = Nl * Nc
    Nl = LinAvEnd - LinAvFirst + 1;
    Nc = ColAvEnd - ColAvFirst + 1;
    #ifdef OPS_DEBUG
        printf("   ->Nb Lignes  : %d\n",Nl);
        printf("   ->Nb Colons : %d\n",Nc);
    #endif


    //@ Interpolation des pixels morts de l'IIS
    //@ Boucle sur les pixels l?image IIS (il,ic)
    #ifdef OPS_DEBUG
        printf("Interpolation des pixels morts de l'IIS");
    #endif
    for(ic=1; ic<=SD_BAS_IMCO; ic++)
    {
        for(il=1; il<=SD_BAS_IMLI; il++)
        {
            //@   Si (IISDeadpix(il,ic)=oui) alors
            /* FA128 */
            if(IISDeadPix[IND2(il,ic,SD_BAS_IMLI)] == CMN__OUI)
             {
                #ifdef OPS_DEBUG
                    printf(" Le pixel (l=%d,c=%d) est mort",il,ic);
                #endif
                //@     remplacer Z(il,ic) par la moyenne des pixels dans l?image qui sont voisins
                //@     immediats de (il,ic) et tels que IISDeadPix = non
                //@     (la moyenne porte donc sur 0 a 8 pixels)
                moyPixIIS = 0.0;
                nbPixMoy  =0;

                //@ Moyenne sur les pixels non morts dont il et ic sont contenenus dans l'image
                for(l=il-1; l<=il+1; l++)
                {
                    for(c=ic-1; c<=ic+1; c++)
                    {
                       /* FA128 */
                       if(l>=1 && l<=SD_BAS_IMLI && c>=1 && c<=SD_BAS_IMCO && (IISDeadPix[IND2(l,c,SD_BAS_IMLI)] == CMN__NON))
                       {
                            nbPixMoy++;
                            moyPixIIS += Z[IND2(c,l,SD_BAS_IMCO)];
                        }
                    }
                }
                if (nbPixMoy > 0) 
		  {
		   moyPixIIS /= nbPixMoy;
                   Z[IND2(ic,il,SD_BAS_IMCO)] = moyPixIIS;
		   }
            }
        }
    }


    //@ 1/ Coordonnees dans le repere IIS des pixels d'une sous-grille de l'imagette Avhrr utile
    //@ Definition d'une sous-grille de l'imagette Avhrr utile
    //@ (1 point sur SGdl + dernier point en ligne, 1 point sur SGdc + dernier point en colonne)

    // Determination du nombre de lignes et de colonnes de la sousgrille : un passage pour rien
    nls = 1;
    ncs = 1;

    l = LinAvFirst;
    c = ColAvFirst;

    while(l<LinAvEnd)
    {
        l += SGdl;
        nls++;
    }

    while(c<ColAvEnd)
    {
        c += SGdc;
        ncs++;
    }

  if (nls < 2 || ncs < 2)
   {
    cr_op   = SD_ALG_AVHRR_40_IAC_ALRT;
    msg_err = (char *)SD_BAS_MSG_ERR_SAMPLING;
    num_lig = __LINE__;
    goto fin;
   }
   #ifdef OPS_DEBUG
        printf("Sousgrille AVHRR\n-------------------------------\n");
        printf("(LinAvFirst,LinAvEnd,SGdl) = (%i,%i,%i)\n",LinAvFirst,LinAvEnd,SGdl);
        printf("(ColAvFirst,ColAvEnd,SGdc) = (%i,%i,%i)\n",ColAvFirst,ColAvEnd,SGdc);
        printf("(nls,ncs) = (%i,%i)\n",nls,ncs);
    #endif

    // Allocation des tableaux de lignes et colonnes de la sousgrille
    if(   SD_BAS_MacMALLOC(ls,double,nls) == FALSE
       || SD_BAS_MacMALLOC(cs,double,ncs) == FALSE
      )
    {
        cr_op   = SD_ALG_AVHRR_ERR_ALLOCATION;
        msg_err = (char *)SD_BAS_MSG_ALLOC;
        num_lig = __LINE__;

        goto fin;
    }


    //@ Calcul des lignes de la sousgrille
    for(l=0; l<nls-1; l++)
    {
        ls[l] = LinAvFirst + l*SGdl;
    }
    ls[nls-1] = LinAvEnd;

    //@ Calcul des lignes de la sousgrille
    for(c=0; c<ncs-1; c++)
    {
        cs[c] = ColAvFirst + c*SGdc;
    }
    cs[ncs-1] = ColAvEnd;

    #ifdef OPS_DEBUG
        printf("Lignes de la sousgrille\n   ");
        for(l=0; l<nls; l++)
        {
            printf("%lf\t",ls[l]);
        }
        printf("\nColonnes de la sousgrille\n   ");
        for(l=0; l<ncs; l++)
        {
            printf("%lf\t",cs[l]);
        }
        printf("\n");
    #endif



    //@ Coordonnees des pixels de la sousgrille dans le repere Avhrr avec correction inverse de loffset
    // ----------------------------------------------------------------------------------------------------
    // Allocation de la memoire
    if(    SD_BAS_MacMALLOC(IISImgAvYSubG,  double,nls*ncs) == FALSE
        || SD_BAS_MacMALLOC(IISImgAvZSubG,  double,nls*ncs) == FALSE
        || SD_BAS_MacMALLOC(ImgAvLinSubG,  double,nls*ncs) == FALSE
        || SD_BAS_MacMALLOC(ImgAvColSubG,  double,nls*ncs) == FALSE
        || SD_BAS_MacMALLOC(ImgAvMetopSubGAzim,  double,nls*ncs) == FALSE
               || SD_BAS_MacMALLOC(ImgAvMetopSubGElev,  double,nls*ncs) == FALSE
      )
    {
        cr_op   = SD_ALG_AVHRR_ERR_ALLOCATION;
        msg_err = (char *)SD_BAS_MSG_ALLOC;
        num_lig = __LINE__;

        goto fin;
    }


    // Calcul des coordonnees
    #ifdef OPS_DEBUG
        printf("\nCoordonnees des pixels de la sousgrille dans le repere Avhrr avec correction inverse de loffset\n");
        printf("----------------------------------------------------------------------------------\n");
        // Utilisation de la sous-matrice | A B C |
        // 2 x 2 relative a Y et Z dans   | D E F | => | E F |
        // la matrice 3 x 3 (MatRot)      | G H K |    | H K |
        printf("   MatRot : %lf\t%lf\n",MatRot[IND2(2,2,SD_BAS_DIM_ROT_MATRICE)],
                                        MatRot[IND2(3,2,SD_BAS_DIM_ROT_MATRICE)]);
        printf("            %lf\t%lf\n",MatRot[IND2(2,3,SD_BAS_DIM_ROT_MATRICE)],
                                        MatRot[IND2(3,3,SD_BAS_DIM_ROT_MATRICE)]);
        printf("   ScanAngle : %lf\n",ScanAngle);
    #endif 
    for(il=1, ct=0; il<=nls; il++)
    {
        for(ic=1; ic<=ncs; ic++,ct++)
        {
                #ifdef OPS_DEBUG
                    printf("\n Point (l,c) = (%d,%d)\n------------------------",il,ic);
                #endif
	    // FA 212
	    ImgAvLinSubG[ct] = ls[IND1(il)] - OffsetGuess[IND2(2,SP,2)];
            ImgAvColSubG[ct] = cs[IND1(ic)] - OffsetGuess[IND2(1,SP,2)];
	    // .. FA 212
        }
    }

    //@ Calcul des coordonnees des NbPixAvSubG pixels dans le repere Metop
//  SD_BAS_ReserverOperateur();
  irc = SD_BAS_AVHRRMetop(ctx,&IasiTime,
                              Mphr,nls*ncs,                   // Appel pour tous les points 
                              BandeauAVHRR,
                              nbligbandeau,
                              ImgAvColSubG,
                              ImgAvLinSubG,
                              ImgAvMetopSubGAzim,       // Azim
                              ImgAvMetopSubGElev        // Elev 
                              );
//   SD_BAS_LibererOperateur();
           // ici la gestion des cas degrades est ajoutee
   if(irc != CMN__STATUT_OK)
     		{
         	cr_op   = SD_ALG_AVHRR_40_IAC_ALRT;
         	msg_err = (char *)SD_BAS_MSG_ERR_AVHRR2METOP;
         	num_lig = __LINE__;
         	goto fin;
     		}


    for(il=1, ct=0; il<=nls; il++)
    {
        for(ic=1; ic<=ncs; ic++,ct++)
        {
	    ImgAvMetopSubG[0]=ImgAvMetopSubGAzim[ct];
	    ImgAvMetopSubG[1]=ImgAvMetopSubGElev[ct];	    
            //@ Calcul des coordonnees des NbPixAvSubG pixels dans le repere IASI
            SD_ALG_GeomMetopIasi(MatRot,ScanAngle,ImgAvMetopSubG,&ImgAvYSubG,&ImgAvZSubG);

            //@ Calcul des coordonnees des NbPixAvSubG pixels dans le repere IIS
            // !!!!! Les tableaux IISImgAvYSubG et IISImgAvYSubG doivent etre ranges par lignes
            // l'interpolation 2D
            IISImgAvYSubG[ct] =  ImgAvYSubG*cos(IISRot) + ImgAvZSubG*sin(IISRot);
            IISImgAvZSubG[ct] = -ImgAvYSubG*sin(IISRot) + ImgAvZSubG*cos(IISRot);

            #ifdef OPS_DEBUG
                printf("\nCoordonnees AVHRR\n");
                printf("   (lin,col) = (%lf,%lf)\n",ImgAvLinSubG,ImgAvColSubG);
                printf("Coordonnees METOP\n");
                printf("   (azim,elev) = (%lf,%lf)\n",ImgAvMetopSubG[0],ImgAvMetopSubG[1]);
                printf("Coordonnees IASI\n");
                printf("   (Y,Z) = (%lf,%lf)\n",ImgAvYSubG,ImgAvZSubG);
                printf("Coordonnees IIS\n");
                printf("   (Y,Z) = (%lf,%lf)\n\n",IISImgAvYSubG[ct],IISImgAvZSubG[ct]);
            #endif
        }
    }


    //@ 2/ Coordonnees dans le repere IIS de tous les points de l'imagette Avhrr utile
    //@ (calcul par interpolation lineaire pour la rapidite)

    if(     SD_BAS_MacMALLOC(lin,double,Nl) == FALSE
        ||  SD_BAS_MacMALLOC(col,double,Nc) == FALSE
        ||  SD_BAS_MacMALLOC(IISImgAvY,double,Nc*Nl) == FALSE
        ||  SD_BAS_MacMALLOC(IISImgAvZ,double,Nc*Nl) == FALSE
        ||  SD_BAS_MacMALLOC(ratio,double,2*(Nc+Nl)) == FALSE)
    {
        cr_op   = SD_ALG_AVHRR_ERR_ALLOCATION;
        msg_err = (char *)SD_BAS_MSG_ALLOC;
        num_lig = __LINE__;

        goto fin;
    }

    for(il=1; il<=Nl; il++)
    {
        lin[IND1(il)] = (double)(il + LinAvFirst-1);
    }

    for(ic=1; ic<=Nc; ic++)
    {
        col[IND1(ic)] = (double)(ic + ColAvFirst-1);
    }



     //@ Initialisation du spline Y
    cr_op=SD_BAS_InterpLineaire2D(ctx,SD_ALG_INIT_INTERP2D,ncs,nls,cs,ls,IISImgAvYSubG,Nc,Nl,col,lin,IISImgAvY,ratio);
     if( cr_op != CMN__STATUT_OK)
     {
       cr_op   = SD_ALG_AVHRR_40_IAC_ALRT;
       msg_err = (char *)SD_BAS_MSG_ERR_INTERP;
       num_lig = __LINE__;
       goto fin;
     }

    //@ Calcul du spline Y
    cr_op=SD_BAS_InterpLineaire2D(ctx,SD_ALG_CALCUL_INTERP2D,ncs,nls,cs,ls,IISImgAvYSubG,Nc,Nl,col,lin,IISImgAvY,ratio);
     if( cr_op != CMN__STATUT_OK)
     {
       cr_op   = SD_ALG_AVHRR_40_IAC_ALRT;
       msg_err = (char *)SD_BAS_MSG_ERR_INTERP;
       num_lig = __LINE__;
       goto fin;
     }

     //@ Initialisation du spline Z
     cr_op=SD_BAS_InterpLineaire2D(ctx,SD_ALG_INIT_INTERP2D,ncs,nls,cs,ls,IISImgAvZSubG,Nc,Nl,col,lin,IISImgAvZ,ratio);
     if( cr_op != CMN__STATUT_OK)
     {
       cr_op   = SD_ALG_AVHRR_40_IAC_ALRT;
       msg_err = (char *)SD_BAS_MSG_ERR_INTERP;
       num_lig = __LINE__;
       goto fin;
     }

     //@ Calcul du spline Z
    cr_op=SD_BAS_InterpLineaire2D(ctx,SD_ALG_CALCUL_INTERP2D,ncs,nls,cs,ls,IISImgAvZSubG,Nc,Nl,col,lin,IISImgAvZ,ratio);
     if( cr_op != CMN__STATUT_OK)
     {
       cr_op   = SD_ALG_AVHRR_40_IAC_ALRT;
       msg_err = (char *)SD_BAS_MSG_ERR_INTERP;
       num_lig = __LINE__;
       goto fin;
     }

    #ifdef OPS_DEBUG
        printf("\nAvant interpolation lineaire\n-------------------\n");
        printf("   LS\n");
        for(il=0; il<nls; il++) printf("\t%lf",ls[il]);
        printf("\n");
        printf("   CS\n");
        for(ic=0; ic<ncs; ic++) printf("\t%lf",cs[ic]);
        printf("\n\n");
        printf("   IISImgAvYSubG\n");
        for(il=0,ct=0; il<nls; il++)
        {
            for(ic=0; ic<ncs; ic++,ct++) printf("\t%lf",IISImgAvYSubG[ct]);
            printf("\n");
        }
        printf("\n\n");
        printf("   IISImgAvZSubG\n");
        for(il=0,ct=0; il<nls; il++)
        {
            for(ic=0; ic<ncs; ic++,ct++) printf("\t%lf",IISImgAvZSubG[ct]);
            printf("\n");
        }
        printf("\n\n");


        printf("\nApres interpolation lineaire\n-------------------\n");
        printf("   LIN\n");
        for(il=0; il<Nl; il++) printf("\t%lf",lin[il]);
        printf("\n");
        printf("   COL\n");
        for(ic=0; ic<Nc; ic++) printf("\t%lf",col[ic]);
        printf("\n\n");
        printf("   IISImgAvYSubG\n");
        for(il=0,ct=0; il<Nl; il++)
        {
            for(ic=0; ic<Nc; ic++,ct++) printf("\t%lf",IISImgAvY[ct]);
            printf("\n");
        }
        printf("\n\n");
        printf("   IISImgAvZSubG\n");
        for(il=0,ct=0; il<Nl; il++)
        {
            for(ic=0; ic<Nc; ic++,ct++) printf("\t%lf",IISImgAvZ[ct]);
            printf("\n");
        }
        printf("\n\n");
    #endif

    // Liberation des tableaux temporaires
    SD_BAS_MacFREE(ls);
    SD_BAS_MacFREE(cs);
    SD_BAS_MacFREE(lin);
    SD_BAS_MacFREE(col);
    SD_BAS_MacFREE(IISImgAvYSubG);
    SD_BAS_MacFREE(IISImgAvZSubG);




    //@ 3/ Reechantillonnage de l'imagette IIS dans le raster AVHRR : interpolation bilineaire
    if(SD_BAS_MacMALLOC(X,double,Nl*Nc) == FALSE)
    {
        cr_op   = SD_ALG_AVHRR_ERR_ALLOCATION;
        msg_err = (char *)SD_BAS_MSG_ALLOC;
        num_lig = __LINE__;

        goto fin;
    }
// il faut ici trier les points avant interpolation
// sinon il faut recommencer pour chaque point
    for(il=0, ct=0; il<Nl; il++)
    {
        for(ic=0; ic<Nc; ic++,ct++)
        {
       cr_op=SD_BAS_InterpLineaire2D(ctx,SD_ALG_INIT_INTERP2D,Mc,Ml,IISY,IISZ,Z,1,1,&IISImgAvY[ct],&IISImgAvZ[ct],&X[ct],ratio);
        if( cr_op != CMN__STATUT_OK)
        {
          cr_op   = SD_ALG_AVHRR_40_IAC_ALRT;
          msg_err = (char *)SD_BAS_MSG_ERR_INTERP;
          num_lig = __LINE__;
          goto fin;
        }
        cr_op=SD_BAS_InterpLineaire2D(ctx,SD_ALG_CALCUL_INTERP2D,Mc,Ml,IISY,IISZ,Z,1,1,&IISImgAvY[ct],&IISImgAvZ[ct],&X[ct],ratio);
        if( cr_op != CMN__STATUT_OK)
        {
          cr_op   = SD_ALG_AVHRR_40_IAC_ALRT;
          msg_err = (char *)SD_BAS_MSG_ERR_INTERP;
          num_lig = __LINE__;
          goto fin;
        }
	}
    }

     //@ Initialisation du spline X
//     SD_BAS_InterpLineaire2D(SD_ALG_INIT_INTERP2D,Mc,Ml,IISY,IISZ,Z,Nc,Nl,IISImgAvY,IISImgAvZ,X,ratio);
     //@ Calcul du spline X
//     SD_BAS_InterpLineaire2D(SD_ALG_CALCUL_INTERP2D,Mc,Ml,IISY,IISZ,Z,Nc,Nl,IISImgAvY,IISImgAvZ,X,ratio);

#ifdef OPS_DEBUG
  sprintf (LibelleTraces, "Z");
  SD_BAS_TRACE (
    tcProcName_F, 
    LibelleTraces,
    Mc, 
    Ml, 
    1, 
    TRACE_REAL8, 
    TRACE_BIN, 
    Z, 
    ctx);
  sprintf (LibelleTraces, "X");
  SD_BAS_TRACE (
    tcProcName_F, 
    LibelleTraces,
    Nc, 
    Nl, 
    1, 
    TRACE_REAL8, 
    TRACE_BIN, 
    X, 
    ctx);
  sprintf (LibelleTraces, "bandeauavhrr");
  SD_BAS_TRACE (
    tcProcName_F, 
    LibelleTraces,
    NbLigAvhrr, 
    AvhrrColRight-AvhrrColLeft+1, 
    1, 
    TRACE_REAL8, 
    TRACE_BIN, 
    Ytemp, 
    ctx);
 #endif
// temporaire initialisation de l'image IIS en supposant qu"elle est = a l'image AVHRR rangee par ligne
// un decalage de 20 lignes et 30 colonne est introduit
//    SUITE:
//    LinAvFirst  = AvhrrLigFirst + 20;   // Verifier le sens des reperes
//    LinAvEnd    = AvhrrLigFirst + 70;   // Verifier le sens des reperes
//    ColAvFirst  = AvhrrColLeft + 25;   // Verifier le sens des reperes
//    ColAvEnd    = AvhrrColLeft + 75;   // Verifier le sens des reperes
//    Nl = LinAvEnd - LinAvFirst + 1;
//    Nc = ColAvEnd - ColAvFirst + 1;
//    if(SD_BAS_MacMALLOC(X,double,Nl*Nc) == FALSE)
//    {
//        cr_op   = SD_ALG_AVHRR_ERR_ALLOCATION;
//        msg_err = (char *)SD_BAS_MSG_ALLOC;
//        num_lig = __LINE__;

//        goto fin;
//    }
// temporaire initialisation de l'image IIS en supposant qu'elle est = a l'image AVHRR rangee par ligne

//    for(il=0, ct=0; il<Nl; il++)
//    {
//        for(ic=0; ic<Nc; ic++,ct++)
//        {
//	X[ct]=Y[IND2(il+LinAvFirst+1+(1),ic+ColAvFirst+(2),NbLigAvhrr)];
//	}
//    }
    #ifdef OPS_DEBUG
        printf("\nReechantillonnage de l'imagette IIS dans le raster AVHRR\n--------------------------------\n");
        printf("   IISZ\n");
        for(il=0; il<Ml; il++) printf("\t%lf",IISZ[il]);
        printf("\n");
        printf("   IISY\n");
        for(ic=0; ic<Mc; ic++) printf("\t%lf",IISY[ic]);
        printf("\n\n");
        printf("   Z\n");
        for(il=0,ct=0; il<Ml; il++)
        {
            for(ic=0; ic<Mc; ic++,ct++) printf("\t%lf",Z[ct]);
            printf("\n");
        }
        printf("\n\n");


        printf("\nApres interpolation lineaire\n-------------------\n");
        printf("   IISImgAvZ\n");
        for(il=0; il<Nl; il++) printf("\t%lf",IISImgAvZ[il]);
        printf("\n");
        printf("   IISImgAvY\n");
        for(ic=0; ic<Nc; ic++) printf("\t%lf",IISImgAvY[ic]);
        printf("\n\n");
        printf("   X\n");
        for(il=0,ct=0; il<Nl; il++)
        {
            for(ic=0; ic<Nc; ic++,ct++) printf("\t%lf",X[ct]);
            printf("\n");
        }

        printf("\n\n");
    #endif

    SD_BAS_MacFREE(ratio);
    SD_BAS_MacFREE(IISImgAvY);  // verifier que ce n'est pas utilise apres...
    SD_BAS_MacFREE(IISImgAvZ);



    //@ Position du point precedent le debut de l?image IASI dans le raster AVHRR
// la ligne doit commencer a 1 et on se positionne par rapport au premier point
// on se positionne en relatif par rapport au bandeau etudie
    lin0 = LinAvFirst+1-AvhrrLigFirst;
//  prise en compte des bornes en colonnes
//  col0 = ColAvFirst;
   col0 = ColAvFirst + 1 -AvhrrColLeft;

    //@---------------- cet algorithme doit etre optimise selon l'annexe 3 du DA2
    //@ Calcul de la valeur moyenne de l?image IASI
    //@ cf DA7
    //@ Boucle sur les points de la grille de calcul de correlation
    //@ (dl,dc) de (-Dl,-Dc) a (Dl,Dc)
    //@   Calcul de la valeur moyenne de l?image AVHRR correspondante au decalage (dl,dc)
    //@     cf DA7
    //@   Calcul du coefficient de correlation entre les deux images
    //@     cf DA7
    //@ Fin de boucle sur le voisinnage
    //@----------------


    // Les quatre etapes d'optimisation proposees dans l'annexe 3 du DA2 sont reprises

    //@ Etape 1
    //----------

    Xbar2 = 0.0;
    Xbar = 0.0;
    X2bar = 0.0;
    ptX = X;

    for(i=0; i<Nl*Nc; i++)
    {
        Xbar  += *ptX;
        X2bar += (*ptX) * (*ptX);
        ptX++;
    }

    Xbar2 = Xbar*Xbar;

    #ifdef OPS_DEBUG
        printf("Calcul de R : etape 1\n--------------------\n");
        printf(" Col0 = %i\n",col0);
        printf(" Lig0 = %i\n",lin0);
        printf(" Excursion en ligne Dl = %i\n",Dl);
        printf(" Excursion en colon Dc = %i\n",Dc);
        printf("   Xbar  : %lf\n",Xbar);
        printf("   Xbar2 : %lf\n",Xbar2);
        printf("   X2bar : %lf\n",X2bar);
    #endif


    //@ Etape 2
    //----------
 // on teste ici de maniere a ne pas deborder du bandeau
    DebColon = col0-Dc;
    if (DebColon < 1)
     {
        cr_op   = SD_ALG_AVHRR_ERR_DOMAIN;
        msg_err = (char *)SD_BAS_MSG_ERR_DOMAIN;
        num_lig = __LINE__;

        goto fin;
     }
         
    FinColon = DebColon + Nc - 1;
    if (FinColon + 2*Dc > (AvhrrColRight - AvhrrColLeft + 1) )
     {
        cr_op   = SD_ALG_AVHRR_ERR_DOMAIN;
        msg_err = (char *)SD_BAS_MSG_ERR_DOMAIN;
        num_lig = __LINE__;

        goto fin;
     }
    
    DebLigne = lin0-Dl;
    if (DebLigne < 1)
     {
        cr_op   = SD_ALG_AVHRR_ERR_DOMAIN;
        msg_err = (char *)SD_BAS_MSG_ERR_DOMAIN;
        num_lig = __LINE__;

        goto fin;
     }
    FinLigne = DebLigne + Nl - 1;
    if (FinLigne + 2*Dl > AvhrrLigLast - AvhrrLigFirst + 1)
     {
        cr_op   = SD_ALG_AVHRR_ERR_DOMAIN;
        msg_err = (char *)SD_BAS_MSG_ERR_DOMAIN;
        num_lig = __LINE__;

        goto fin;
     }

    ctp->Ybar[0] = 0.0;
    ctp->Y2bar[0] = 0.0;
    XYbar = 0.0;
    for(ic=DebColon; ic<=FinColon; ic++)
     {
       for(il=DebLigne; il<=FinLigne; il++)
       {
            dtmp = Y[IND2(il,ic,NbLigAvhrr)];
            ctp->Ybar[0]     += dtmp;
            ctp->Y2bar[0]    += dtmp*dtmp;
            XYbar            += X[IND2(ic-DebColon+1,il-DebLigne+1,Nc)] * Y[IND2(il,ic,NbLigAvhrr)]; /*Verifier l'ordre des indices de Y */
        }
    }

    ctp->R[0] = ((XYbar*Nl*Nc)-Xbar*ctp->Ybar[0]) / sqrt((X2bar*Nl*Nc-Xbar2)*(ctp->Y2bar[0]*Nl*Nc-ctp->Ybar[0]*ctp->Ybar[0]));

    #ifdef OPS_DEBUG
        printf("\nCalcul de R : etape 2\n--------------------\n");
        printf("   Ybar  : %lf\n",ctp->Ybar);
        printf("   Xbar2 : %lf\n",ctp->Y2bar);
        printf("   XYbar : %lf\n",XYbar);
        printf("   R[0]  : %lf\n",ctp->R[0]);
    #endif

    //@ Etape 3
    //----------
    #ifdef OPS_DEBUG
        printf("\nCalcul de R : etape 3\n--------------------\n");
    #endif

    for(il=1; il<=2*Dl; il++)
    {
        #ifdef OPS_DEBUG
            printf("\n  ->il = %d\n",il);
        #endif

        DebColon = col0-Dc;
        FinColon = DebColon + Nc - 1;
        DebLigne = lin0-Dl+il;
        FinLigne = DebLigne + Nl - 1;
        AncLigne = DebLigne - 1;
        NouvLigne = FinLigne;

        deltaY  = 0.0;
        deltaY2 = 0.0;
        for(ic=DebColon; ic<=FinColon; ic++)
        {
            dtmp1 = Y[IND2(NouvLigne,ic,NbLigAvhrr)];
            dtmp2 = Y[IND2(AncLigne,ic,NbLigAvhrr)];
            deltaY  += (dtmp1 - dtmp2);
            deltaY2 += (dtmp1*dtmp1 - dtmp2*dtmp2);
        }

        ctp->Ybar[il] = ctp->Ybar[il-1] + deltaY;
        ctp->Y2bar[il] = ctp->Y2bar[il-1] + deltaY2;

        XYbar = 0.0;
        for(c=DebColon; c<=FinColon; c++)
         {
           for(l=DebLigne; l<=FinLigne; l++)
           {

                XYbar += X[IND2(c-DebColon+1,l-DebLigne+1,Nc)] * Y[IND2(l,c,NbLigAvhrr)];
            }
        }

        ctp->R[IND2(il+1,1,2*Dl+1)] = ((XYbar*Nl*Nc)-Xbar*ctp->Ybar[il]) / sqrt((X2bar*Nl*Nc-Xbar2)*(ctp->Y2bar[il]*Nl*Nc-ctp->Ybar[il]*ctp->Ybar[il]));

        #ifdef OPS_DEBUG
            printf("   Ybar  : %lf\n",ctp->Ybar);
            printf("   Xbar2 : %lf\n",ctp->Y2bar);
            printf("   XYbar : %lf\n",XYbar);
            printf("   R[0]  : %lf\n",ctp->R[0]);
        #endif
    }


    //@ Etape 4
    //----------
    #ifdef OPS_DEBUG
        printf("\nCalcul de R : etape 4\n--------------------\n");
    #endif
    for(ic=1; ic<=2*Dc; ic++)
    {
/* bma ordre des valeurs */
/*
        for(il=0; il<=2*Dl+1; il++)
*/
        for(il=0; il<=2*Dl; il++)
/* fin bma */
        {
            #ifdef OPS_DEBUG
                printf("\n  ->il,ic = %d,%d\n",il,ic);
            #endif

            DebColon = col0-Dc+ic;
            FinColon = DebColon + Nc - 1;
            DebLigne = lin0-Dl+il;
            FinLigne = DebLigne + Nl - 1;
            AncColon = DebColon - 1;
            NouvColon = FinColon;

            deltaY  = 0.0;
            deltaY2 = 0.0;


            for(l=DebLigne; l<=FinLigne; l++)
             {
                  dtmp1 = Y[IND2(l,NouvColon,NbLigAvhrr)];
                  dtmp2 = Y[IND2(l,AncColon,NbLigAvhrr)];
                deltaY += (dtmp1 - dtmp2);
                deltaY2 += (dtmp1*dtmp1 - dtmp2*dtmp2);
            }

            ctp->Ybar[il]  = ctp->Ybar[il] + deltaY;
            ctp->Y2bar[il] = ctp->Y2bar[il] + deltaY2;

            XYbar = 0.0;
            for(c=DebColon; c<=FinColon; c++)
            {
               for(l=DebLigne; l<=FinLigne; l++)
               {
                    XYbar +=   X[IND2(c-DebColon+1,l-DebLigne+1,Nc)]
		             * Y[IND2(l,c,NbLigAvhrr)];
                }
            }

            ctp->R[IND2(il+1,ic+1,2*Dl+1)] = ((XYbar*Nl*Nc)-Xbar*ctp->Ybar[il]) / sqrt((X2bar*Nl*Nc-Xbar2)*(ctp->Y2bar[il]*Nl*Nc-ctp->Ybar[il]*ctp->Ybar[il]));

            #ifdef OPS_DEBUG
                printf("   Ybar  : %lf\n",ctp->Ybar[il]);
                printf("   Xbar2 : %lf\n",ctp->Y2bar[il]);
                printf("   XYbar : %lf\n",XYbar);
                printf("   R[%d,%d]  : %lf\n",il-1,ic-1,ctp->R[IND2(il,ic,2*Dl+1)]);
            #endif
        }
    }



    //@ Recherche du point de grille (dlmax,dcmax) presentant le maximum de correlation
    //@   R(dlmax,dcmax)=max(R(dl,dc))
    Rmax = ctp->R[0];

    dlmax = 1; /* FA173 */
    dcmax = 1; /* FA173 */
    for(ic=1,ct=0; ic<=2*Dc+1; ic++)
      {
       for(il=1; il<=2*Dl+1; il++,ct++)
       {
            if(ctp->R[ct] > Rmax)
            {
                Rmax = ctp->R[ct];
                dlmax = il;
                dcmax = ic;
            }
        }
    }

#ifdef OPS_DEBUG
    printf("\nPosition du maximum de R : (lmax,cmax)=(%d,%d)\n\n",dlmax,dcmax);
#endif

    //@ Si (dlmax, dcmax ) n'est pas sur un bord de la grille alors
    //@     [ dlm, dcm, CorrelQual, PosMaxQual, FlagError ] =
    //@      Recherche d'extremum ( R, dlmax, dcmax )
    //@ appel SD_BAS_Extremum2D ( recherche maximum)
    //@ Fin de Si
    if(dlmax !=1 && dlmax != nlR && dcmax !=1 && dcmax != ncR)
    {

        //@ Ajustement de paraboloide resolution d'un systeme sousresolu AX = B
        //@ Calcul de A (ordre FORTRAN)
        nlinm = dlmax - 1;
        nlinp = dlmax + 1;
        ncolm = dcmax - 1;
        ncolp = dcmax + 1;

        for(ic=0; ic<9; ic++)
        {
            if(ic==0){nl = nlinm; nc = ncolm;};
            if(ic==1){nl = dlmax; nc = ncolm;};
            if(ic==2){nl = nlinp; nc = ncolm;};
            if(ic==3){nl = nlinm; nc = dcmax;};
            if(ic==4){nl = dlmax; nc = dcmax;};
            if(ic==5){nl = nlinp; nc = dcmax;};
            if(ic==6){nl = nlinm; nc = ncolp;};
            if(ic==7){nl = dlmax; nc = ncolp;};
            if(ic==8){nl = nlinp; nc = ncolp;};

            a[0*9 + ic] = 1.0;
            a[1*9 + ic] = nl;
            a[2*9 + ic] = nc;
            a[3*9 + ic] = nl*nl;
            a[4*9 + ic] = nc*nc;
            a[5*9 + ic] = nl*nc;
       }


        //@ Calcul de B
        NbLigR = 2*Dl+1;
        b[0] = ctp->R[IND2(nlinm,ncolm,NbLigR)];
        b[1] = ctp->R[IND2(dlmax,ncolm,NbLigR)];
        b[2] = ctp->R[IND2(nlinp,ncolm,NbLigR)];
        b[3] = ctp->R[IND2(nlinm,dcmax,NbLigR)];
        b[4] = ctp->R[IND2(dlmax,dcmax,NbLigR)];
        b[5] = ctp->R[IND2(nlinp,dcmax,NbLigR)];
        b[6] = ctp->R[IND2(nlinm,ncolp,NbLigR)];
        b[7] = ctp->R[IND2(dlmax,ncolp,NbLigR)];
        b[8] = ctp->R[IND2(nlinp,ncolp,NbLigR)];

        #ifdef OPS_DEBUG
            printf("   Matrice A\n");
            for(il=0; il<6; il++)
            {
                for(ic=0; ic<9; ic++)
                {
                    printf("%4.2lf \t",a[il*9+ic]);
                }
                printf("\n");
            }

            printf("   \nVecteur B\n");
            for(ic=0; ic<9; ic++)
            {
                printf("%4.2lf \t",b[ic]);
            }
            printf("\n\n");
            printf("Ajustement du paraboloide par moindres carres\n");
        #endif


        //@ Calcul des moindres carres
        irc = SD_BAS_MoindresCarres1D(ctx,nbeq,degre+1,a,b,tabTmpMC,PRECISION_AJUSTEMENT_MC,x,&eps);
        if(irc != CMN__STATUT_OK)
        {
            #ifdef OPS_DEBUG
                printf("L'ajustement a echoue\n");
            #endif
            cr_op   = SD_ALG_AVHRR_ERR_MOINDRES_CARRES;
            msg_err = (char *)SD_ALG_MSG_ERR_MOINDRES_CARRES;
            num_lig = __LINE__;

            #ifndef OPS_DEBUG
            goto fin;
            #endif
        }

       //@ Recherche de l'extremum du paraboloide
        irc = SD_BAS_Extremum2D (x,eps,res,dcmax,dlmax);

        // Recuperation des valeurs
        dlm         = res[0];
        dcm         = res[1];
        CorrelQual  = res[2];
        PosMaxQual  = res[3];

        #ifdef OPS_DEBUG
            printf("\nExtremum approche par une paraboloide\n");
            printf("   Ligne Min : %d\n",res[0]);
            printf("   Colonne Min : %d\n",res[1]);
            printf("   Distance correspondante : %d\n",res[2]);
            printf("   Erreur d'approximation : %d\n",res[3]);
        #endif
    }
    else
    {
//    irc=CMN__STATUT_NOK;
//    CorrelQual=0.;
    // CR 236 DD 5.2
//    irc=CMN__STATUT_OK;
    CorrelQual=-10.;
    PosMaxQual  = 2*PosMaxCutoff;
    }


    //@ Si CorrelQual < CorrelCutoff ou PosMaxQual < PosMaxCutoff
    //@ alors
    //@    FlagCoregNonQual = NON
    if(CorrelQual > CorrelCutoff && PosMaxQual < PosMaxCutoff)
    {
        FlagCoregNonQual = CMN__NON;
    }
    //@ Sinon
    //@    FlagCoregNonQual = OUI
    else
    {
        FlagCoregNonQual = CMN__OUI;
    }


    #ifdef OPS_DEBUG
        printf("\nFlagCoregNonQual : %i\n",FlagCoregNonQual);
    #endif



    //@ Si FlagError=OUI alors
    //@    FlagCoregNonValid = OUI
    //@    OffsetImagAvhrr=(OffsetGuess(2)+dlmax,OffsetGuess(1)+dcmax)
    /* CR 2376 */
    //@    CorrelQual=-10
    //@    PosMaxQual  = 2*PosMaxCutoff
    /* ... CR 2376 */
    if(irc != CMN__STATUT_OK)
    {
        FlagCoregNonValid = CMN__OUI;
		// CR 330
        OffsetImagAvhrr[0] = OffsetGuess[IND2(1,SP,2)];
        OffsetImagAvhrr[1] = OffsetGuess[IND2(2,SP,2)];
		// FIN CR 330
        /* CR 2376 */
		CorrelQual=-10.;
    	PosMaxQual  = 2*PosMaxCutoff;
        /* ... CR 2376 */
    }
    //@ Sinon
    //@    FlagCoregNonValid = NON
    //@    OffsetImagAvhrr=( OffsetGuess(2)+dlmax+dlm,OffsetGuess(1)+dcmax+dcm)
    else
    {
        FlagCoregNonValid = CMN__NON;
	// modif suite a mail bernard du 10/02 CR 236
        if ( FlagCoregNonQual == CMN__NON )
	 {
          OffsetImagAvhrr[0] = OffsetGuess[IND2(1,SP,2)]+dcm-(Dc+1);
          OffsetImagAvhrr[1] = OffsetGuess[IND2(2,SP,2)]+dlm-(Dl+1);
	 }
	else
	 {
          OffsetImagAvhrr[0] = OffsetGuess[IND2(1,SP,2)];
          OffsetImagAvhrr[1] = OffsetGuess[IND2(2,SP,2)];
	 }
   } 

// dans le cas ou on a un NaN ceci peut avoir des consequences desastreuses sur 41_CCS
   if (!FINITE(OffsetImagAvhrr[0]) || !FINITE(OffsetImagAvhrr[1]))
              {
              FlagCoregNonValid = CMN__OUI;
              OffsetImagAvhrr[0] = 0;
              OffsetImagAvhrr[1] = 0;
	      }
    #ifdef OPS_DEBUG
        printf("FlagCoregNonValid : %i\n",FlagCoregNonValid);
        printf("OffsetImagAvhrr[1] : %i\n",OffsetImagAvhrr[0]);
        printf("OffsetImagAvhrr[1] : %i\n",OffsetImagAvhrr[1]);
    #endif


// var imag iis est maintenant calcule dans IRC




// on trace ici si besoin toutes les variables en sortie

    // Recopie des sorties dans la listvar InfoDet
    // Remarque ... les tab var en question sont des tableaux et je ne sais pas ou copier
    // les resultats !
    InfoDet1ENG->tab_var[ENG_GIACCORRELQUAL].val.pr8[IND1(SN)]                       = CorrelQual;
    InfoDet1ENG->tab_var[ENG_GIACPOSMAXQUAL].val.pr8[IND1(SN)]                       = PosMaxQual;
    InfoDet1ENG->tab_var[ENG_GIACFLAGCOREGNONQUAL].val.pui4[IND1(SN)]                = FlagCoregNonQual;
    InfoDet1ENG->tab_var[ENG_GIACFLAGCOREGNONVALID].val.pui4[IND1(SN)]               = FlagCoregNonValid;
    InfoDet1ENG->tab_var[ENG_GIACOFFSETIISAVHRR].val.pr8[2 * IND1(SN) + 0]           = OffsetImagAvhrr[0];
    InfoDet1ENG->tab_var[ENG_GIACOFFSETIISAVHRR].val.pr8[2 * IND1(SN) + 1]           = OffsetImagAvhrr[1];
    // Mise a jour du contexte
    Contexte->GIacOffsetIISAVHRRLin[0]                                               = OffsetImagAvhrr[0];
    Contexte->GIacOffsetIISAVHRRCol[0]                                               = OffsetImagAvhrr[1];


// On trace ici toutes les variables en sortie
#ifdef INVESTIGATION
    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"OffsetIISAvhrr",2,1,OffsetImagAvhrr,ctx);
    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"CorrelQual",1,1,&CorrelQual,ctx);
    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"PosMaxQual",1,1,&PosMaxQual,ctx);
    SD_BAS_TRACEENTIER1D(tcProcName_F,&ctx->ctxtrt,"GIacFlagCoregNonQual",1,1,&FlagCoregNonQual,ctx);
    SD_BAS_TRACEENTIER1D(tcProcName_F,&ctx->ctxtrt,"GIacFlagCoregNonValid",1,1,&FlagCoregNonValid,ctx);
#endif


fin:

// liberation memoire
    SD_BAS_MacFREE(ratio);
    SD_BAS_MacFREE(IISImgAvY);  // verifier que ce n'est pas utilise apres...
    SD_BAS_MacFREE(IISImgAvZ);
    SD_BAS_MacFREE(ls);
    SD_BAS_MacFREE(cs);
    SD_BAS_MacFREE(lin);
    SD_BAS_MacFREE(col);
    SD_BAS_MacFREE(IISImgAvYSubG);
    SD_BAS_MacFREE(IISImgAvZSubG);
    SD_BAS_MacFREE(ImgAvLinSubG);
    SD_BAS_MacFREE(ImgAvColSubG);
    SD_BAS_MacFREE(ImgAvMetopSubGAzim);
    SD_BAS_MacFREE(ImgAvMetopSubGElev);
    SD_BAS_MacFREE(Y);
    SD_BAS_MacFREE(X);
    // Traitement des erreurs
    //================================================================
    if ( cr_op != CMN__STATUT_OK )
    {
        // mise a jour de la variable ctx->cr.errinterne
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
        SD_BAS_ERR_SET_AVHRR,
        cr_op,
        tcProcName_F,
        num_lig,
        msg_err);
      CreerMessage (
        SD_ALG_AVHRR_SET, 
        MSG_ALG_AVHRR_40_IAC_ALRT, 
        &(ctx->cr.msgJDB));

        // on imprime le message d'erreur sur la sortie standard
        #ifdef OPS_DEBUG
            SD_BAS_PRINT_ERR_INT(ctx->cr.errinterne.nomfonction,
                                 ctx->cr.errinterne.numligne,
                                 ctx->cr.errinterne.libelle);
        #endif
    }

    #ifdef OPS_DEBUG
        printf("\n Fin de l'algorithme 40_IAC\n================================\n");
    #endif

// sortie de la fonction
return(cr_op);

}

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_41_CCS
//@
//@ Description : analyse des radiances dans les FOV sondeurs
//@
//@ Commentaire :
//@    Les flags de disponibilte des produits "ChannelState" sont
//@    generes a l'interieur de 41_CCS (pas passes en parametres)
//@    A la demande du CNES "EigValCutoff" a ete rajoute dans les
//@    parametres.
//@
//@ Parameters :
//@         (ENTREE)  Contexte_courant : contexte de l'etape (numero de ligne SN traite)
//@         (ENTREE)  ConfAutre ...... : fichier de conf autre
//@         (ENTREE)  ConfStable ..... : fichier de conf stable
//@         (ENTREE)  MPHRN1 ......... : MPHR produit N1
//@         (ENTREE)  GIrcImg ........ : image calibree
//@         (ENTREE)  AvhrrListeLin .. : Suite de lignes AVHRR chargees en memoire
//@         (ENTREE)  AvhrrNbLin ..... : Taille de la liste de lignes
//@
//@         (ENTSOR)  InfoDet1C ...... : info det du produit 1C
//@                   pour le SN (on choisira la date UTC
//@
//@         (SORTIE)  ProduitCourant . : pour pointer sur l'image utile
//@         (SORTIE)  InfoDet1ENG .... : info det du produit ENG
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_41_CCS(
  t_SD_BAS_DataThread         *Contexte_courant,   // t_SD_BAS_DataThread*
  t_SD_BAS_ConfAutre          *ConfAutre,          // t_SD_BAS_ConfAutre*
  t_SD_BAS_ConfStable         *ConfStable,         // t_SD_BAS_ConfStable*
  t_SD_BAS_MPHR               *MphrN1,             // t_SD_BAS_MPHR*
  double                      *GIrcImg,            // double( (SD_BAS_IMCO * SD_BAS_IMLI) )
  t_SD_BAS_LigneAVHRR        **AvhrrListeLinIN,    // t_SD_BAS_LigneAVHRR * ( AvhrrNbLin)
  int                          AvhrrNbLinIN,       // int(1)
  double                       EigValCutoff,       // double(1) (= 1.0E-2)
  t_SD_BAS_Liste_Var           InfoDet1C,          // t_SD_BAS_Liste_Var
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  t_SD_BAS_Liste_Var           InfoDet1ENG         // t_SD_BAS_Liste_Var
)
{
  static const char tcProcName_F[]= "SD_ALG_41_CCS";
  t_CMN__Statut cr_op= CMN__STATUT_OK;


  // on trace ici si besoin toutes les variables en sortie

  // Recuperation des infos contenues dans les structure
  int                   SN;                  // Numero de sous-cycle
  int                   CCD;                 // Orientation du coin de cube
  int                   SP;                  // Scan position

  // Entrees
  t_SD_BAS_SCDSTime     IasiTime;            // t_SD_BAS_SCDSTime*
  double                IasiScanAngle;       // double(1)
  double               *IasiMetopMatRot;     // double( SD_BAS_DIM_ROT_MATRICE * SD_BAS_DIM_ROT_MATRICE)
  int                   PsfNbLin;            // int(1)
  int                   PsfNbCol;            // int(1)
  int                   Osf;                 // int(1)
  double               *YPsf;                // double( SD_BAS_NB_COL_SOND * SD_BAS_PN)
  double               *ZPsf;                // double( SD_BAS_NB_LIN_SOND * SD_BAS_PN)
  double               *WgtPsf;              // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND * SD_BAS_PN)
  double               *OffsetSondIISModel;  // double( SD_BAS_MODEL * SD_BAS_CCD * SD_BAS_PN)
  t_SD_BAS_SCDSTime     t_FE;                // t_SD_BAS_SCDSTime*
  double                MetopPeriod;         // double(1)
  double               *OffsetIISAvhrr;      // double( SD_BAS_DEUX * SD_BAS_SNOT)
  int                   CCSMode;             // int(1)

  t_SD_BAS_LigneAVHRR **AvhrrListeLin;       // t_SD_BAS_LigneAVHRR * ( AvhrrNbLin)
  int                   AvhrrNbLin;          // int(1)
  int                   AvhrrLinDeb;         // int(1)
  int                   AvhrrLinFin;         // int(1)
  int                   AvhrrColDeb;         // int(1)
  int                   AvhrrColFin;         // int(1)
  int                   AvhrrSGDl;           // int(1)
  int                   AvhrrSGDc;           // int(1)
  double                AvhrrDatBefore;      // double(1)
  double                AvhrrDatAfter;       // double(1)

  double               *IISImage;            // double( (SD_BAS_IMCO * SD_BAS_IMLI) * SD_BAS_SN)
  int                   IISNbLin;            // int(1)
  int                   IISNbCol;            // int(1)
  double               *IISY;                // double( SD_BAS_IMLI)
  double               *IISZ;                // double( SD_BAS_IMCO)
  double                IISDY;               // double(1)
  double                IISDZ;               // double(1)
  int                  *IISDeadPix;          // int( SD_BAS_IMCO * SD_BAS_IMLI)
  double                IISRot;              // double(1)

  int                   CCSImgClassifiedLinMax;  // int(1)
  int                   CCSImgClassifiedColMax;  // int(1)
  int                   CCSNbClusMax;            // int(1)
  int                   CCSNbIterMax;            // int(1)
  double                CCSIterConvCutoff;       // double(1)
  double                CCSSplitCutoff;          // double(1)
  double                CCSGlueCutoff;           // double(1)
  double                CCSDistAgloInit;         // double(1)
  int                   CCSMinIterNCCutoff;      // int(1)
  int                   CCSNbNonClassifMax;      // int(1)
  double                CCSDistMax;              // double(1)
  int                   CCSDimClassSpace;        // int(1)
  int                  *CCSChannelId;            // int( SD_BAS_NBK)
  int                   CCSSortingChannel;       // int(1)
  int                   CCSRadToTb;              // int(1)
  double               *CCSChannelWn;            // double( SD_BAS_NBK)
  double               *CCSChannelNoise;         // double( SD_BAS_NBK)
  int                  *CCSChannelWgt;           // int( SD_BAS_NBK)


  // Sorties
  double                OffsetSondAvhrr[SD_BAS_DEUX*SD_BAS_PN*SD_BAS_SNOT]; // double( SD_BAS_DEUX * SD_BAS_PN * SD_BAS_SNOT)
  double                OffsetSondIIS[SD_BAS_DEUX*SD_BAS_PN*SD_BAS_SNOT];   // double( SD_BAS_DEUX * SD_BAS_PN * SD_BAS_SNOT)
  int                  * ImageClassified;           			    // int( SD_BAS_AMCO*SD_BAS_AMLI)
//  int                   ImageClassified[SD_BAS_AMCO*SD_BAS_AMLI];         // int( SD_BAS_AMCO*SD_BAS_AMLI)
  int                   ImageClassifiedNbLin;                               // int(1)
  int                   ImageClassifiedNbCol;                               // int(1)
  int                   ImageClassifiedFirstLin;                            // int(1)
  int                   ImageClassifiedFirstCol;                            // int(1)
  unsigned int          ConfAvhrrChannel[SD_BAS_NBK];                       // unsigned int( SD_BAS_NBK)
  unsigned int          RadAnalType[SD_BAS_NCL_MAX*SD_BAS_SNOT];            // unsigned int( SD_BAS_NCL_MAX * SD_BAS_SNOT)
  int                   RadAnalNbClass;                                     // int(1)
  double                RadAnalMean[SD_BAS_NBK*SD_BAS_NCL_MAX];             // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
  double                RadAnalStd[SD_BAS_NBK*SD_BAS_NCL_MAX];              // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
  double                RadAnalY[SD_BAS_NCL_MAX];                           // double( SD_BAS_NCL_MAX)
  double                RadAnalZ[SD_BAS_NCL_MAX];                           // double( SD_BAS_NCL_MAX)
  double                RadAnalWgt[SD_BAS_NCL_MAX];                         // double( SD_BAS_NCL_MAX)
  unsigned int          FlagPostProcessing;                                 // int(1)
  double                NonClassifRate;                                     // double(1)
  double                VarianceRate;                                       // double(1)
  int                   FlagDateNOK;                                        // int(1)


  // Variables internes
  float         *AvhrrUtilImage;         // float( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NBK)
  int            AvhrrUtilLinFirst;      // int(1)
  int            AvhrrUtilColFirst;      // int(1)
  int            AvhrrUtilNbLin;         // int(1)
  int            AvhrrUtilNbCol;         // int(1)
  int            AvhrrNch, AvhrrChanMax; // int(1), int(1)
/* bma Remarque : AvhrrClassified n'est pas utilise : redondant avec IdPix ou ImageClassified*/
//  int           *AvhrrClassified;        // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)

  int            ColImgFovFirst;  // int(1)
  int            ColImgFovLast;   // int(1)
  int            LinImgFovFirst;  // int(1)
  int            LinImgFovLast;   // int(1)
  int            NbPixImgFov;     // int(1)

  double        *ImgYpsf;      // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  double        *ImgZpsf;      // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  double        *ImgWgtpsf;    // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)

  float         *RadVect;      // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
  int            NbPixImg;     // int(1)
  int            Nch;          // int(1)
  int           *ChId;         // int( SD_BAS_NBK)
  int            SortCh;       // int(1)
  double        *ChNoise;      // double( SD_BAS_NBK)
  int           *ChWgt;        // int( SD_BAS_NBK)
  int            TailleAvhrr;  // int(1)
  // FT-2467
  int volatile   il;           // index lignes
  int volatile   ic;           // index colonnes
  // ..FT-2467
  int            k,c,i,j;          // index general
  int            pix_1_2;      // index pixels (1 et 2)
  int            Ich;          // index sur les canaux
  int            Ipix;         // index sur les pixels
  int            Pix;          // index sur les pixels
  int            Ipsf;         // index sur les sondeurs
  double         datetmp;      // Date temporaire

  unsigned int  *AvhrrChannelState;     // int( SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NBK)
  int           *IdPix;                 // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  int                  PosLinMillis;
  int                  Tdeb;
  int                  T0;
  int                   NbClass;                                     // int(1)

  // FT-2528
  unsigned int         *QualityIndicator;               // unsigned int(SD_BAS_NLI_MAX_UTIL_AVHRR)
  unsigned int         *ScanLineQuality;                // unsigned int(SD_BAS_NLI_MAX_UTIL_AVHRR)
  unsigned short int   *CountErrorFrame;                // unsigned short int(SD_BAS_NLI_MAX_UTIL_AVHRR)
  unsigned short int   *CloudInformation;               // unsigned short int(SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NPI_AVHRR)
  int                  EUMAvhrrCldErrorCutoff;          // Threshold for AVHRR error
  // ...FT-2528

  // Statuts de sortie des fonctions appelees
  t_CMN__Statut        irc;

  // Gestion de erreurs
/* bma intialisation */
/*
  char                *msg_err;          // Message d'erreur
  int                  num_lig;            // Ligne d'erreur
*/
  char                *msg_err = (char*) NULL;          // Message d'erreur
  int                  num_lig = 0;            // Ligne d'erreur
/* fin bma */
  t_SD_BAS_DataThread *ctx = Contexte_courant;
  int                  SNmin;             // Numero de sous cycle moni
  t_SD_BAS_SCDSTime    IasiTime0;


  // Entree dans la fonction
  #ifdef OPS_DEBUG
    printf( "Entree dans la fonction 41 CCS\n========================================\n");
  #endif

//  initialisation du tableau
  for( i= 1; i <= SD_BAS_SNOT; i++)
  {
    for( j= 1; j <= SD_BAS_NCL_MAX; j++)
    {
     RadAnalType[IND2(j,i,SD_BAS_NCL_MAX)]=  0;
    }
  }

  NbPixImg = 0;

  TailleAvhrr= SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR;

  // Initialisations
  AvhrrUtilImage= ProduitCourant->Y;  // Image de taille maximale deja allouee globalement

  // Pour pouvoir utiliser les macros
  SN=  Contexte_courant->ctxtrt.SN;                                       // Numero de sous-cycle
  CCD= Contexte_courant->ctxtrt.CCD + 1;                                  // Orientation du coin de cube
  SP=  InfoDet1ENG.tab_var[ENG_GEPS_SP].val.pui4[IND1(SN)];  // Scan position
  
	// FT-2997 : Modif non liee a la PM. Il s'agit seulement d'une amelioration de code
	//CR270 on se positionne toujours par rapport au SN1
	SNmin = 1;
    // ...FT-2997
    
  IasiTime0                = InfoDet1C.tab_var[N1C_GEPSDATIASI].val.pdate[IND1(SNmin)];
  // Entrees
  IasiTime=                InfoDet1C.tab_var[N1C_GEPSDATIASI].val.pdate[IND1(SN)]; // t_SD_BAS_SCDSTime*
  IasiScanAngle=           ConfAutre->IDefIASIScanAngle[IND1(SP)]; // double(1)
  IasiMetopMatRot=         ConfAutre->IDefIASIMETOPMatRot;         // double( SD_BAS_DIM_ROT_MATRICE * SD_BAS_DIM_ROT_MATRICE)
  PsfNbLin=                ConfStable->IDefPsfSondNbLin;           // int(1)
  PsfNbCol=                ConfStable->IDefPsfSondNbCol;           // int(1)
  YPsf=                    ConfStable->IDefPsfSondY;               // double( SD_BAS_NB_LIN_SOND * SD_BAS_PN)
  ZPsf=                    ConfStable->IDefPsfSondZ;               // double( SD_BAS_NB_LIN_SOND * SD_BAS_PN)
  WgtPsf=                  ConfStable->IDefPsfSondWgt;             // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND * SD_BAS_PN)
  Osf=                     ConfStable->IDefPsfSondOverSampFactor;  // int(1)
  OffsetSondIISModel=      ConfAutre->IDefOffsetSondIISModel;      // double( SD_BAS_MODEL * SD_BAS_CCD * SD_BAS_PN)
  t_FE=                   *(InfoDet1ENG.tab_var[ENG_GEPSENDECLIPSETIME].val.pdate);  // t_SD_BAS_SCDSTime*
  MetopPeriod=             ConfAutre->IDefMETOPOrbitPeriod;        // double(1)
  OffsetIISAvhrr=          InfoDet1ENG.tab_var[ENG_GIACOFFSETIISAVHRR].val.pr8;  // double( SD_BAS_DEUX * SD_BAS_SNOT)
  CCSMode=                 ConfAutre->IDefCcsMode;                 // int(1)
  
  // FT-2528
  EUMAvhrrCldErrorCutoff = ConfStable->IDefEUMAvhrrCldErrorCutoff; // Threshold for AVHRR error
  // ...FT-2528

  AvhrrListeLin=           AvhrrListeLinIN;                               // t_SD_BAS_LigneAVHRR * (AvhrrNbLinIN)
  AvhrrNbLin=              AvhrrNbLinIN;                                  // int(1)
  
   // FA 202
   if(34 != SP)
     {
     /* FA142 */
     AvhrrDatBefore=          CMN__DATE_CONV_SCDS_2_SEC(IasiTime) - ConfStable->IDefDeltaAvhrrDTbefore[IND1(SP)];  // double(1)
     AvhrrDatAfter=           CMN__DATE_CONV_SCDS_2_SEC(IasiTime) + ConfStable->IDefDeltaAvhrrDTafter[IND1(SP)];   // double(1)

     AvhrrColDeb=             ConfStable->IDefDeltaAvhrrColLeft[IND1(SP)];   // int(1)  
     AvhrrColFin=             ConfStable->IDefDeltaAvhrrColRight[IND1(SP)];  // int(1)
     }
  else
     {
     /* FA142 */
     AvhrrDatBefore=          CMN__DATE_CONV_SCDS_2_SEC(IasiTime) - ConfStable->IDefDeltaAvhrrDTbefore[IND1(15)];  // double(1)
     AvhrrDatAfter=           CMN__DATE_CONV_SCDS_2_SEC(IasiTime) + ConfStable->IDefDeltaAvhrrDTafter[IND1(15)];   // double(1)

     AvhrrColDeb=             ConfStable->IDefDeltaAvhrrColLeft[IND1(16)];   // int(1)  
     AvhrrColFin=             ConfStable->IDefDeltaAvhrrColRight[IND1(15)];  // int(1)
     }
     
   // FIN FA 202
  
  AvhrrChanMax=            SD_BAS_NBK;                                    // int(1)

  IISImage=               &(GIrcImg[IND3(1,1,1,SD_BAS_IMCO,SD_BAS_IMLI)]); // double( (SD_BAS_IMCO * SD_BAS_IMLI) * SD_BAS_SN)
  IISNbLin=                ConfAutre->IDefIISNbLin;           // int(1)
  IISNbCol=                ConfAutre->IDefIISNbCol;           // int(1)
  IISY=                    ConfAutre->IDefIISY;               // double( SD_BAS_IMLI)
  IISZ=                    ConfAutre->IDefIISZ;               // double( SD_BAS_IMCO)
  IISDY=                   ConfAutre->IDefIISDY;              // double(1)
  IISDZ=                   ConfAutre->IDefIISDZ;              // double(1)
  IISDeadPix=              ConfStable->IDefDptIISDeadPix;     // uint( SD_BAS_IMCO * SD_BAS_IMLI)
  IISRot=                  ConfAutre->IDefIISRot;             // double(1)
  CCSImgClassifiedLinMax=  ConfAutre->IDefCcsImgCILinMax;     // int(1)
  CCSImgClassifiedColMax=  ConfAutre->IDefCcsImgCIColMax;     // int(1)
  CCSNbClusMax=            ConfAutre->IDefCcsNbClusMax;       // int(1)
  CCSNbIterMax=            ConfAutre->IDefCcsNbIterMax;       // int(1)
  CCSIterConvCutoff=       ConfAutre->IDefCcsIterConvCutoff;  // double(1)
  CCSSplitCutoff=          ConfAutre->IDefCcsSplitCutoff;     // double(1)
  CCSGlueCutoff=           ConfAutre->IDefCcsGlueCutoff;      // double(1)
  CCSDistAgloInit=         ConfAutre->IDefCcsDistAgloInit;    // double(1)
  CCSMinIterNCCutoff=      ConfAutre->IDefCcsMinIterNCCutoff; // int(1)
  CCSNbNonClassifMax=      ConfAutre->IDefCcsNbNonClassifMax; // int(1)
  CCSDistMax=              ConfAutre->IDefCcsDistMax;         // double(1)
  CCSDimClassSpace=        ConfAutre->IDefCcsDimClassSpace;   // int(1)
  CCSChannelId=            ConfAutre->IDefCcsChannelId;       // int( SD_BAS_NBK)
  CCSSortingChannel=       ConfAutre->IDefCcsSortingChannel;  // int(1)
  CCSRadToTb=              ConfAutre->IDefCcsRadToTb;         // int(1)
  CCSChannelWn=            ConfAutre->IDefCcsChannelWn;       // double( SD_BAS_NBK)
//@ si AVHRR temperature de brillance 
//@ si IIS radiance
  CCSChannelNoise=         ConfAutre->IDefCcsChannelNoise;    // double( SD_BAS_NBK)
// 5.0 PM 3
// si IIS tout est deja dans la conf

  CCSChannelWgt=          &(ConfAutre->IDefCcsChannelWgt[IND2(1,CCSMode+1,SD_BAS_NBK)]);  // int( SD_BAS_NBK) Modulo SD_BAS_CCD
// version 1.6 conf autre
  AvhrrSGDl =              ConfStable->IDefAvhrrSubGridDl;
//  AvhrrSGDl =              10;
//  AvhrrSGDc =              10;
  AvhrrSGDc =              ConfStable->IDefAvhrrSubGridDc;
  
// initialisations
  ChId = NULL;
  ChNoise = NULL;
  ChWgt = NULL;
  RadVect = NULL;
  ImageClassified=NULL;
  
  ChId = (int *) calloc(SD_BAS_NBK , sizeof(int));
  ChNoise = (double *) calloc(SD_BAS_NBK , sizeof(double));
  ChWgt = (int *) calloc(SD_BAS_NBK , sizeof(int));
  RadVect = (float *) calloc((SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK , sizeof(float));

  ImageClassified = (int *) calloc(SD_BAS_AMCO*SD_BAS_AMLI , sizeof(int));
  // FT-2528
  SD_BAS_MacMALLOC(QualityIndicator,unsigned int,SD_BAS_NLI_MAX_UTIL_AVHRR);
  SD_BAS_MacMALLOC(ScanLineQuality,unsigned int,SD_BAS_NLI_MAX_UTIL_AVHRR);
  SD_BAS_MacMALLOC(CountErrorFrame,unsigned short int,SD_BAS_NLI_MAX_UTIL_AVHRR);
  SD_BAS_MacMALLOC(CloudInformation,unsigned short int,SD_BAS_NLI_MAX_UTIL_AVHRR*SD_BAS_NPI_AVHRR);

  if( (ChId == NULL) || (ChNoise == NULL) || (ChWgt == NULL) || (RadVect == NULL)
  || (ImageClassified == NULL) || (QualityIndicator == NULL) || ( ScanLineQuality == NULL)
  || (CountErrorFrame == NULL) || (CloudInformation == NULL))
  {
  // ...FT-2528
        cr_op   = SD_ALG_AVHRR_ERR_ALLOCATION;
       SD_BAS_THROW(
       cr_op, SD_ALG_AVHRR_41_CCS_ALRT, 
       cr_op, BAS_CR_NEQ, CMN__STATUT_OK, SD_BAS_MSG_ALLOC);
  } 
  
  // Sorties

  // Variables internes
//  AvhrrClassified= ProduitCourant->AvhrrClassified;
  AvhrrChannelState= ProduitCourant->AvhrrChannelState;
  ImgYpsf= ProduitCourant->ImgYpsf;
  ImgZpsf= ProduitCourant->ImgZpsf;
  ImgWgtpsf= ProduitCourant->ImgWgtpsf;
  IdPix= ProduitCourant->IdPix;
  
// Extraction des flags de validite des canaux
  cr_op= SD_ALG_ConstruireAvhrrChannelState(
           // Entrees
            AvhrrListeLin,        // t_SD_BAS_LigneAVHRR( AvhrrNbLin)
            AvhrrNbLin,           // int(1)
            AvhrrChanMax-1,         // int(1)
	    //jlp30 le nombre de canal utile est  toujours 6-1
           // Sorties
            AvhrrChannelState);   // unsigned int( SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NBK)

  #ifdef OPS_DEBUG
    printf( "Fin des initialisations des parametres\n");
  #endif

  // ========================================================================================
  // I) Traitement des images
  // ========================================================================================
  // Si (Mode = Avhrr) alors
  if( CCSMode == SD_ALG_AVHRR_MODE_AVHRR)
  { // Extraction de l'imagette Avhrr utile (domaine de l'IIS),

    #ifdef OPS_DEBUG
      printf( "AVHRR Nb Lignes ... : %d lignes\n", AvhrrNbLin);
      printf( "AVHRR Date debut .. : %lf\n",       AvhrrDatBefore);
      printf( "AVHRR Date fin .... : %lf\n",       AvhrrDatAfter);
      printf( "AVHRR Colonne debut : %d\n",        AvhrrColDeb);
      printf( "AVHRR Colonne fin . : %d\n",        AvhrrColFin);
    #endif

    #ifdef OPS_DEBUG
      printf("Calcul des premieres et dernieres lignes de l'image AVHRR en fonction des dates\n");
    #endif

    // Calcul des numeros de premiere et derniere ligne de l'image AVHRR en fonction des dates
    //------------------------------------------------------------------------
    // Premiere ligne
    AvhrrLinDeb= 0;
    il= 0;
    datetmp= CMN__DATE_CONV_SCDS_2_SEC( *(AvhrrListeLin[il]->InfoGene.tab_var[AVHRR_DATEDEB].val.pdate));

    while( (datetmp <= AvhrrDatBefore) && (il < AvhrrNbLin))
    {
/* bma depassement de tableau */
/*      il++; */
/* fin bma */
      datetmp= (double)CMN__DATE_CONV_SCDS_2_SEC( *(AvhrrListeLin[il]->InfoGene.tab_var[AVHRR_DATEDEB].val.pdate));
/* bma depassement de tableau */
      il++;
/* fin bma */
      #ifdef OPS_DEBUG
        printf( "Date de la ligne %d : %lf\n", il, datetmp);
      #endif
    }

    AvhrrLinDeb= il - 1;

    if( AvhrrLinDeb < 0)   AvhrrLinDeb= 0;

    #ifdef OPS_DEBUG
      printf( "Date debut a la ligne : %d\n", AvhrrLinDeb);
    #endif

    // Derniere ligne
    AvhrrLinFin= 0;
    il= 0;
    datetmp= CMN__DATE_CONV_SCDS_2_SEC( *(AvhrrListeLin[il]->InfoGene.tab_var[AVHRR_DATEFIN].val.pdate));
  
    while( (datetmp < AvhrrDatAfter) && (il < AvhrrNbLin))
    {
/* bma depassement de tableau */
/*      il++; */
/* fin bma */
      datetmp= CMN__DATE_CONV_SCDS_2_SEC( *(AvhrrListeLin[il]->InfoGene.tab_var[AVHRR_DATEFIN].val.pdate));
/* bma depassement de tableau */
      il++;
/* fin bma */
      #ifdef OPS_DEBUG
        printf("Date de la ligne %d : %lf\n", il, datetmp);
      #endif
    }
    AvhrrLinFin= il;
  
    if( AvhrrLinFin > (AvhrrNbLin - 1))   AvhrrLinFin= AvhrrNbLin - 1;

    #ifdef OPS_DEBUG
      printf( "Date fin   a la ligne : %d\n", AvhrrLinFin);
    #endif

// temporaire le code de 41_CCS est shunte
//   cr_op=CMN__STATUT_NOK;
//   SD_BAS_THROW(
//      cr_op, SD_ALG_AVHRR_41_CCS_ALRT, 
//      cr_op, BAS_CR_NEQ, CMN__STATUT_OK, SD_ALG_MSG_CALCOFFSET);

    // --------------------------------------------------------------------------------------
    // expansion a la dimension maximale (ChanMax=6=SD_BAS_NBK) des canaux Avhrr
    //   en utilisant ChannelState et detection des transitions
    //   (generation du flag ConfAvhrrChannel)
    // --------------------------------------------------------------------------------------
    AvhrrNch= SD_BAS_NBA_AVHRR;
    cr_op= SD_ALG_ImageUtile(
             // Entrees
              Contexte_courant,      // t_SD_BAS_DataThread*
              MphrN1,                // t_SD_BAS_MPHR*
              IISNbLin,              // int(1)
              IISNbCol,              // int(1)
              IISY,                  // double( SD_BAS_IMLI)
              IISZ,                  // double( SD_BAS_IMCO)
              IISDY,                 // double(1)
              IISDZ,                 // double(1)
              IasiMetopMatRot,       // double( SD_BAS_DIM_ROT_MATRICE * SD_BAS_DIM_ROT_MATRICE)
              IISRot,                // double(1)
              IasiScanAngle,         // double(1)
              IasiTime,              // t_SD_BAS_SCDSTime*
              OffsetIISAvhrr,        // double( 2 * SD_BAS_SNOT)
              AvhrrListeLin,         // t_SD_BAS_LigneAVHRR * ( AvhrrNbLin)
              AvhrrNbLin,            // int(1)
              AvhrrLinDeb,           // int(1)
              AvhrrLinFin,           // int(1)
              AvhrrColDeb,           // int(1)
              AvhrrColFin,           // int(1)
              AvhrrNch,              // int(1)
              AvhrrChanMax,          // int(1)
              AvhrrChannelState,     // unsigned int( SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NBK)
             // Sorties
              AvhrrUtilImage,         // float( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NBK)
                                      //          AvhrrUtilNbCol,         AvhrrUtilNbLin,    AvhrrChanMax
             &AvhrrUtilLinFirst,      // int(1)
             &AvhrrUtilColFirst,      // int(1)
             &AvhrrUtilNbLin,         // int(1)
             &AvhrrUtilNbCol,         // int(1)
             // FT-2528
             ConfAvhrrChannel,        // unsigned int( SD_BAS_NBK)
             QualityIndicator,
             ScanLineQuality,
             CountErrorFrame,
             CloudInformation);
             // ...FT-2528
    SD_BAS_THROW(
      cr_op, SD_ALG_AVHRR_41_CCS_ALRT, 
      cr_op, BAS_CR_NEQ, CMN__STATUT_OK, SD_ALG_MSG_ERR_IMAGEUTILE);
  
    // --------------------------------------------------------------------------------------

    // --------------------------------------------------------------------------------------
    // Traitement des canaux Avhrr: Test de conversion en Tb,
    // limitation aux canaux presents (Nch) et transformation en vecteur
    // --------------------------------------------------------------------------------------
    cr_op= SD_ALG_TraitCanauxAvhrr(
             // Entrees
              ConfAutre,
              AvhrrUtilImage,        // float( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NBK)
              AvhrrUtilNbLin,        // int(1)
              AvhrrUtilNbCol,        // int(1)
              ConfAvhrrChannel,      // unsigned int( SD_BAS_NBK)
              CCSRadToTb,            // int(1)
              CCSDimClassSpace,      // int(1)
              CCSSortingChannel,     // int(1)
              CCSChannelWn,          // double( SD_BAS_NBK)
              CCSChannelId,          // int( SD_BAS_NBK)
              CCSChannelNoise,       // double( SD_BAS_NBK)
              CCSChannelWgt,         // int( SD_BAS_NBK)
             // Sorties
              RadVect,  // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
             &NbPixImg, // int(1)
             &Nch,      // int(1)
              ChId,     // int( SD_BAS_NBK)
             &SortCh,   // int(1)
              ChNoise,  // double( SD_BAS_NBK)
              ChWgt);   // int( SD_BAS_NBK)

    SD_BAS_THROW(
      cr_op, SD_ALG_AVHRR_41_CCS_ALRT_TRAIT_CANAUX, 
      cr_op, BAS_CR_NEQ, CMN__STATUT_OK, "");
      
    SD_BAS_THROW(
      cr_op, SD_ALG_AVHRR_41_CCS_ALRT_NB_PIXAV_NUL, 
      NbPixImg, BAS_CR_EQ, 0, "");    
  
    // --------------------------------------------------------------------------------------
  } // Fin AVHRR ==============================================================================
  else
  { // Traitement de l'image IIS : Conversion en Tb,

    // --------------------------------------------------------------------------------------
    // construction des pseudo-canaux (Nch=2) et transformation en vecteur
    // --------------------------------------------------------------------------------------
    cr_op= SD_ALG_PseudoCanauxIIS(
             // Entrees
              ConfAutre,          // Pour les constantes de Planck
              IISImage,           // double( SD_BAS_IMCO * SD_BAS_IMLI)
              IISNbLin, IISNbCol, // int(1), int(1)
              IISDeadPix,         // int( SD_BAS_IMCO * SD_BAS_IMLI)
              CCSChannelWn,       // double( SD_BAS_NBK)
             // Sorties
              AvhrrUtilImage,     // float( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NBK)
              RadVect,            // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
                                  // Considere comme NBK vecteurs de NPI_AVHRR * NLI_MAX_UTIL_AVHRR valeurs
             &NbPixImg,           // int(1)
             &AvhrrUtilLinFirst,  // int(1)
             &AvhrrUtilColFirst,  // int(1)
             &AvhrrUtilNbLin,     // int(1)
             &AvhrrUtilNbCol,     // int(1)
             &Nch);               // int(1)
  
    SortCh= CCSSortingChannel;
    // --------------------------------------------------------------------------------------

    // --------------------------------------------------------------------------------------
    // Boucle sur les pseudo-canaux k=1 a ChanMax
    // --------------------------------------------------------------------------------------
    for( k= 1; k <= AvhrrChanMax; k++)
    {
      ChId[IND1(k)]=    CCSChannelId[IND1(k)];
      ChNoise[IND1(k)]= CCSChannelNoise[IND1(k)];
      ChWgt[IND1(k)]=   CCSChannelWgt[IND1(k)];
    }
    // --------------------------------------------------------------------------------------

  }
  // Fin IIS ================================================================================

  // ========================================================================================
  // II) Analyse des radiances sur l'image RadVect
  // ========================================================================================
  cr_op= SD_ALG_NueesDynamiques(
           // Entrees
            ProduitCourant,     // t_SD_BAS_DataThreadProduit*
            Contexte_courant,   // t_SD_BAS_DataThread*
            RadVect,            // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
            NbPixImg,           // int(1)
            Nch,                // int(1)
            SortCh,             // int(1)
            ChNoise,            // double( SD_BAS_NBK)
            ChWgt,              // int( SD_BAS_NBK)
            CCSNbClusMax,       // int(1)
            CCSNbIterMax,       // int(1)
            CCSIterConvCutoff,  // double(1)
            CCSSplitCutoff,     // double(1)
            CCSGlueCutoff,      // double(1)
            EigValCutoff,       // double(1)
            CCSDistAgloInit,    // double(1)
            CCSMinIterNCCutoff, // int(1)
            CCSNbNonClassifMax, // int(1)
            CCSDistMax,         // double(1)
           // Sorties
            IdPix,
           &(RadAnalType[IND2(1,SN,SD_BAS_NCL_MAX)]),  // unsigned int( SD_BAS_NCL_MAX)
           &NbClass,                        // int(1)
           &FlagPostProcessing,                    // unsigned int(1)
           &NonClassifRate,                        // double(1)
           &VarianceRate);                         // double(1)
   SD_BAS_THROW(
      cr_op, SD_ALG_AVHRR_41_CCS_ALRT, 
      cr_op, BAS_CR_NEQ, CMN__STATUT_OK, SD_ALG_MSG_NUEESDYNAMIQUES);
  
  // ========================================================================================
    // VERRUE 
  #ifdef OPS_DEBUG
    printf("image classifiee \n");
    for( i= 1; i <= AvhrrUtilNbLin; i++)
    {
      printf( "      IdPix(%1d)= : ", i);
      for( j= 1; j <= AvhrrUtilNbCol; j++)
      {
        printf( "%1d", IdPix[IND2(j,i,AvhrrUtilNbCol)]);
      }
      printf( "\n");
    }

  #endif


  // ========================================================================================
  // Boucle sur les IPSF ( Ipsf = 1 a 4 )
  // ========================================================================================

  for( Ipsf= 1; Ipsf <= SD_BAS_PN; Ipsf++)
  {

	// FT-2997 Calcoffset n'est plus appele dans 41_CCS
	    // --------------------------------------------------------------------------------------
	    // IIIa) Calcul des offsets Sond-IIS et Sond-Avhrr
	    // --------------------------------------------------------------------------------------
	// ...FT-2997
 
    // --------------------------------------------------------------------------------------
    // IIIb) Projection des IPSFs dans l'Avhrr
    // --------------------------------------------------------------------------------------


    // FT-2528
    InfoDet1ENG.tab_var[ENG_GEUMAVHRR1BCLDFRAC].val.pui4[IND2(Ipsf,SN,SD_BAS_PN)]  = 0xFF; // Bit7 = 1
    InfoDet1ENG.tab_var[ENG_GEUMAVHRR1BLANDFRAC].val.pui4[IND2(Ipsf,SN,SD_BAS_PN)] = 0xFF; // Bit7 = 1
    InfoDet1ENG.tab_var[ENG_GEUMAVHRR1BQUAL].val.pui4[IND2(Ipsf,SN,SD_BAS_PN)]     = 0xFF; // Bit7...0 = 1
    // ...FT-2528
  
    // Si Mode = Avhrr alors
    if( CCSMode == SD_ALG_AVHRR_MODE_AVHRR)
    {

      // Si RadToTb=1 alors reconversion en radiance : RadVect = Planck{RadVect, ChannelWn}
      if( CCSRadToTb == 1)
      {
  
        for( Ich= 1; Ich <= Nch; Ich++)
        {
          for( Ipix=1; Ipix <= NbPixImg; Ipix++)
          {
            RadVect[IND2(Ipix,Ich,TailleAvhrr)]=
              (float)SD_BAS_Planck(
                                    (double)RadVect[IND2(Ipix,Ich,TailleAvhrr)],  /* temperature */
                                    CCSChannelWn[IND1(Ich)],              /* nombre d'onde */
                                    ConfAutre->IDefPlkPK1,                /* Precalcul de 2*h*c*c */
                                    ConfAutre->IDefPlkPK2);               /* Precalcul de h*c/k */
          }
        }
      }

		// FT-2997
      		cr_op= SD_ALG_IpsfToAvhrr(
	        	// Entrees pour MetopAVHRR
	         	Contexte_courant,      // t_SD_BAS_DataThread*
	         	MphrN1,                // t_SD_BAS_MPHR*
	         	AvhrrListeLin,         // t_SD_BAS_LigneAVHRR * ( AvhrrNbLin)
	         	AvhrrNbLin,            // int(1)
	         	AvhrrLinDeb,           // int(1)
	         	AvhrrLinFin,           // int(1)
	         	AvhrrColDeb,           // int(1)
	         	AvhrrColFin,           // int(1)
	        	// Entrees
	         	IasiTime,                                                       // t_SD_BAS_SCDSTime*
	         	IasiScanAngle,                                                  // double(1)
	         	IasiMetopMatRot,                                                // double( SD_BAS_DIM_ROT_MATRICE * SD_BAS_DIM_ROT_MATRICE)
	         	PsfNbCol,                                                       // int(1)
	         	PsfNbLin,                                                       // int(1)
	        	&(YPsf[IND2(1,Ipsf,SD_BAS_NB_COL_SOND)]),                          // double( SD_BAS_NB_COL_SOND)
	        	&(ZPsf[IND2(1,Ipsf,SD_BAS_NB_LIN_SOND)]),                          // double( SD_BAS_NB_LIN_SOND)
	        	&(WgtPsf[IND3(1,1,Ipsf,SD_BAS_NB_COL_SOND,SD_BAS_NB_LIN_SOND)]),   // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
	         	Osf,                                                            // int(1)
	        	&(OffsetIISAvhrr[IND2(1,SN,SD_BAS_DEUX)]),                       // double( SD_BAS_DEUX)
		    	&(InfoDet1ENG.tab_var[ENG_GCCSOFFSETSONDIIS].val.pr8[IND3(1,Ipsf,SN,SD_BAS_DEUX,SD_BAS_PN)]),	// double( SD_BAS_DEUX)
				&(InfoDet1ENG.tab_var[ENG_GCCSOFFSETSONDAVHRR].val.pr8[IND3(1,Ipsf,SN,SD_BAS_DEUX,SD_BAS_PN)]),	// double( SD_BAS_DEUX)
	         	AvhrrSGDl,                                                      // int(1)
	         	AvhrrSGDc,                                                      // int(1)
	        	// Sorties
	        	&ColImgFovFirst,        // int(1)
	        	&ColImgFovLast,         // int(1)
	        	&LinImgFovFirst,        // int(1)
	        	&LinImgFovLast,         // int(1)
	        	&NbPixImgFov,           // int(1)
	         	ImgYpsf,               // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
	         	ImgZpsf,               // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
	         	ImgWgtpsf);            // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
		// ...FT-2997
    	SD_BAS_THROW(
      		cr_op, SD_ALG_AVHRR_41_CCS_ALRT, 
      		cr_op, BAS_CR_NEQ, CMN__STATUT_OK, SD_ALG_MSG_IPSFTOAVHRR);
        
        // FT-2528
        // Extraction de l'information nuageuse dans le FOV sondeur
        cr_op = SD_ALG_AvhrrCloudInfoIntoFov(
        // Entrees
        Contexte_courant,                // t_SD_BAS_DataThread*
        AvhrrUtilLinFirst,               // int(1)
        AvhrrUtilColFirst,               // int(1)
        AvhrrUtilNbLin,                  // int(1)
        AvhrrUtilNbCol,                  // int(1)
        NbPixImg,                        // int(1)
        CloudInformation,                // unsigned short int(SD_BAS_NLI_MAX_UTIL_AVHRR*SD_BAS_NPI_AVHRR)
        QualityIndicator,                // unsigned short int(SD_BAS_NLI_MAX_UTIL_AVHRR)
        ScanLineQuality,                 // unsigned short int(SD_BAS_NLI_MAX_UTIL_AVHRR)
        CountErrorFrame,                 // unsigned short int(SD_BAS_NLI_MAX_UTIL_AVHRR)
        ColImgFovFirst,                  // int(1)
        ColImgFovLast,                   // int(1)
        LinImgFovFirst,                  // int(1)
        LinImgFovLast,                   // int(1)
        NbPixImgFov,                     // int(1)
        ImgWgtpsf,                       // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)  > SD_ALG_IpsfToAvhrr(29)
        EUMAvhrrCldErrorCutoff,          // int(1)
        // Sorties
        &(InfoDet1ENG.tab_var[ENG_GEUMAVHRR1BCLDFRAC].val.pui4[IND2(Ipsf,SN,SD_BAS_PN)]),  // unsigned int(1)
        &(InfoDet1ENG.tab_var[ENG_GEUMAVHRR1BLANDFRAC].val.pui4[IND2(Ipsf,SN,SD_BAS_PN)]), // unsigned int(1)
        &(InfoDet1ENG.tab_var[ENG_GEUMAVHRR1BQUAL].val.pui4[IND2(Ipsf,SN,SD_BAS_PN)])      // unsigned int(1)
        );
        // ...FT-2528

    }
    else
    // Sinon
    {
  
		// FT-2997
	       	cr_op= SD_ALG_IpsfToIIS(
	        	// Entrees
	         	ProduitCourant,                                                 // t_SD_BAS_DataThreadProduit*
	         	PsfNbCol,                                                       // int(1)
	         	PsfNbLin,                                                       // int(1)
	        	&(YPsf[IND2(1,Ipsf,SD_BAS_NB_COL_SOND)]),                          // double( SD_BAS_NB_COL_SOND)
	        	&(ZPsf[IND2(1,Ipsf,SD_BAS_NB_LIN_SOND)]),                          // double( SD_BAS_NB_LIN_SOND)
	        	&(WgtPsf[IND3(1,1,Ipsf,SD_BAS_NB_COL_SOND,SD_BAS_NB_LIN_SOND)]),   // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
	         	Osf,                                                            // int(1)
	         	IISRot,                                                         // double(1)
	         	IISNbLin,                                                       // int(1)
	         	IISNbCol,                                                       // int(1)
	         	IISY,                                                           // double( SD_BAS_IMLI)
	         	IISZ,                                                           // double( SD_BAS_IMCO)
	         	IISDY,                                                          // double(1)
	         	IISDZ,                                                          // double(1)
		    	&(InfoDet1ENG.tab_var[ENG_GCCSOFFSETSONDIIS].val.pr8[IND3(1,Ipsf,SN,SD_BAS_DEUX,SD_BAS_PN)]),	// double( SD_BAS_DEUX)
	        	// Sorties
	        	&ColImgFovFirst,  // int(1)
	        	&ColImgFovLast,   // int(1)
	        	&LinImgFovFirst,  // int(1)
	        	&LinImgFovLast,   // int(1)
	        	&NbPixImgFov,     // int(1)
	         	ImgYpsf,         // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
	         	ImgZpsf,         // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
	         	ImgWgtpsf);      // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
		// ...FT-2997 

    }
    // FT-2528
    InfoDet1C.tab_var[N1C_GEUMAVHRR1BCLDFRAC].val.pui4[IND2(Ipsf,SN,SD_BAS_PN)] =
        InfoDet1ENG.tab_var[ENG_GEUMAVHRR1BCLDFRAC].val.pui4[IND2(Ipsf,SN,SD_BAS_PN)];
    InfoDet1C.tab_var[N1C_GEUMAVHRR1BLANDFRAC].val.pui4[IND2(Ipsf,SN,SD_BAS_PN)] =
        InfoDet1ENG.tab_var[ENG_GEUMAVHRR1BLANDFRAC].val.pui4[IND2(Ipsf,SN,SD_BAS_PN)];
    InfoDet1C.tab_var[N1C_GEUMAVHRR1BQUAL].val.pui4[IND2(Ipsf,SN,SD_BAS_PN)] =
        InfoDet1ENG.tab_var[ENG_GEUMAVHRR1BQUAL].val.pui4[IND2(Ipsf,SN,SD_BAS_PN)];
    // ...FT-2528
  
    // --------------------------------------------------------------------------------------

    // --------------------------------------------------------------------------------------
    // IV) Analyse des radiances dans les FOVs du sondeur
    // --------------------------------------------------------------------------------------
    cr_op= SD_ALG_AnaRadIntoFov( // Entrees
                    ProduitCourant,     // t_SD_BAS_DataThreadProduit*
                    AvhrrUtilLinFirst,  // int(1)
                    AvhrrUtilColFirst,  // int(1)
                    AvhrrUtilNbCol,     // int(1)
                    NbPixImg,           // int(1)
                    LinImgFovFirst,     // int(1)
                    LinImgFovLast,      // int(1)
                    ColImgFovFirst,     // int(1)
                    ColImgFovLast,      // int(1)
                    RadVect,            // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
                    IdPix,              // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
                    NbClass,     // int(1)
                    Nch,                // int(1)
                    AvhrrChanMax,       // int(1)
                    ChId,               // int( SD_BAS_NBK)
                    ImgWgtpsf,          // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
                    ImgYpsf,            // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
                    ImgZpsf,            // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
                   // Sorties
                    RadAnalWgt,         // double( SD_BAS_NCL_MAX)
                    RadAnalY,           // double( SD_BAS_NCL_MAX)
                    RadAnalZ,           // double( SD_BAS_NCL_MAX)
                    RadAnalMean,        // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
                    RadAnalStd,         // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
                   &RadAnalNbClass);    // int(1)
    // --------------------------------------------------------------------------------------
  
	    // ========================================================================================
	    // Copie dans les variables de sorties
	    // ========================================================================================
	
	    // ----------------------------------------------------------------------------------------
	// FT-2997
	    for( pix_1_2= 1; pix_1_2 <= SD_BAS_DEUX; pix_1_2++)
	    {
	      // Offset Sondeur/AVHRR
	      //  conversion en degres
	      InfoDet1ENG.tab_var[ENG_GCCSOFFSETSONDAVHRR].val.pr8[IND3(pix_1_2,Ipsf,SN,SD_BAS_DEUX,SD_BAS_PN)] = InfoDet1ENG.tab_var[ENG_GCCSOFFSETSONDAVHRR].val.pr8[IND3(pix_1_2,Ipsf,SN,SD_BAS_DEUX,SD_BAS_PN)]*180./M_PI;
	      // Offset Sondeur/IIS
	      //  conversion en degres
	      InfoDet1ENG.tab_var[ENG_GCCSOFFSETSONDIIS].val.pr8[IND3(pix_1_2,Ipsf,SN,SD_BAS_DEUX,SD_BAS_PN)] = InfoDet1ENG.tab_var[ENG_GCCSOFFSETSONDIIS].val.pr8[IND3(pix_1_2,Ipsf,SN,SD_BAS_DEUX,SD_BAS_PN)]*180./M_PI;
	    }
	// ...FT-2997
  
    // ----------------------------------------------------------------------------------------
    // Nombre de classes
    InfoDet1C.tab_var[N1C_GCCSRADANALNBCLASS].val.pi4[IND2(Ipsf,SN,SD_BAS_PN)]= RadAnalNbClass;
    InfoDet1ENG.tab_var[ENG_CGCCSRADANALNBCLASS].val.pi4[IND2(Ipsf,SN,SD_BAS_PN)]= RadAnalNbClass;
    #ifdef INVESTIGATION
      SD_BAS_TRACE( (char *)tcProcName_F, "RadAnalNbClass",  1, 1, 1,
                    TRACE_INT4,  TRACE_ASC, &RadAnalNbClass, Contexte_courant);
    #endif
    // ----------------------------------------------------------------------------------------
  
    // Caracteristiques de toutes les classes
//  les classes sont stockes dans un tableau de taille NbClass 
    for( c= 1; c <= NbClass; c++)
    {
      // ----------------------------------------------------------------------------------------
      // Moyennes et ecart types des classe
      for( k= 1; k <= SD_BAS_NBK; k++)
      {
        InfoDet1C.tab_var[N1C_GCCSRADANALMEAN].val.pr8[IND4(k,c,Ipsf,SN,SD_BAS_NBK,SD_BAS_NCL,SD_BAS_PN)]=
          RadAnalMean[IND2(k,c,SD_BAS_NBK)];
        InfoDet1C.tab_var[N1C_GCCSRADANALSTD].val.pr8[IND4(k,c,Ipsf,SN,SD_BAS_NBK,SD_BAS_NCL,SD_BAS_PN)]=
          RadAnalStd[IND2(k,c,SD_BAS_NBK)];
      }
      // ----------------------------------------------------------------------------------------

      // ----------------------------------------------------------------------------------------
      // Positions angulaires des centres de classes converties en degre
      InfoDet1C.tab_var[N1C_GCCSRADANALY].val.pr8[IND3(c,Ipsf,SN,SD_BAS_NCL,SD_BAS_PN)]=
        RadAnalY[IND1(c)]*180./M_PI;
      InfoDet1C.tab_var[N1C_GCCSRADANALZ].val.pr8[IND3(c,Ipsf,SN,SD_BAS_NCL,SD_BAS_PN)]=
        RadAnalZ[IND1(c)]*180./M_PI;
      // ----------------------------------------------------------------------------------------

      // ----------------------------------------------------------------------------------------
      // Poids des classes
      InfoDet1C.tab_var[N1C_GCCSRADANALWGT].val.pr8[IND3(c,Ipsf,SN,SD_BAS_NCL,SD_BAS_PN)]=
        RadAnalWgt[IND1(c)];
      // ----------------------------------------------------------------------------------------
    }
    // ----------------------------------------------------------------------------------------
  
    #ifdef INVESTIGATION
      SD_BAS_TRACE( (char *)tcProcName_F, "RadAnalMean",
                    SD_BAS_NCL_MAX, SD_BAS_NBK, 1,
                    TRACE_REAL8, TRACE_BIN,
                    RadAnalMean, Contexte_courant);
      SD_BAS_TRACE( (char *)tcProcName_F, "RadAnalStd",
                    SD_BAS_NCL_MAX, SD_BAS_NBK, 1,
                    TRACE_REAL8, TRACE_BIN,
                    RadAnalStd,  Contexte_courant);
      SD_BAS_TRACE( (char *)tcProcName_F, "RadAnalY",
                    SD_BAS_NCL_MAX, 1, 1,
                    TRACE_REAL8, TRACE_ASC,
                    RadAnalY,    Contexte_courant);
      SD_BAS_TRACE( (char *)tcProcName_F, "RadAnalZ",
                    SD_BAS_NCL_MAX, 1, 1,
                    TRACE_REAL8, TRACE_ASC,
                    RadAnalZ,    Contexte_courant);
      SD_BAS_TRACE( (char *)tcProcName_F, "RadAnalWgt",
                    SD_BAS_NCL_MAX, 1, 1,
                    TRACE_REAL8, TRACE_ASC,
                    RadAnalWgt,  Contexte_courant);
    #endif
  }
  // Fin boucle IPSF ========================================================================

  // ========================================================================================
  // V) Construction de l'image classifiee
  // ========================================================================================
  ImageClassifiedNbCol=    AvhrrUtilNbCol;
  ImageClassifiedNbLin=    AvhrrUtilNbLin;
  ImageClassifiedFirstCol= AvhrrUtilColFirst;
  ImageClassifiedFirstLin= AvhrrUtilLinFirst;

  // --------------------------------------------------------------------------------------
  // initialisation a 0
  // Boucle sur les lignes il=1 a ImgClassifiedLinMax
  // Boucle sur les colonnes ic=1 a ImgClassifiedColMax
  for( il=1; il <= SD_BAS_AMLI; il++)
  {
    for( ic=1; ic <= SD_BAS_AMCO; ic++)
    {
      ImageClassified[IND2(ic,il,SD_BAS_AMCO)]= 0;
    }
  }
  // --------------------------------------------------------------------------------------

  // --------------------------------------------------------------------------------------
  // Construction proprement dite
  Pix= 0;
  // Boucle sur les lignes il=1 a NbLinImg
  // Boucle sur les colonnes ic=1 a NbColImg
  for( il=1; il <= AvhrrUtilNbLin; il++)
  {
    for( ic=1; ic <= AvhrrUtilNbCol; ic++)
    {
      Pix= Pix + 1;
      ImageClassified[IND2(ic,il,SD_BAS_AMCO)]= IdPix[IND1(Pix)];
    }
  }
  // --------------------------------------------------------------------------------------
  
  // ----------------------------------------------------------------------------------------
  // Caracteristiques de toutes les classes
//  les classes sont stockes dans un tableau de taille NbClass 
  for( c= 1; c <= NbClass; c++)
  {
    // Flags indicateurs de classes extremes
      InfoDet1C.tab_var[N1C_GCCSRADANALTYPE].val.pui4[IND2(c,SN,SD_BAS_NCL)]= RadAnalType[IND2(c,SN,SD_BAS_NCL_MAX)];
  }
  // ----------------------------------------------------------------------------------------

  // ----------------------------------------------------------------------------------------
  // Image classifiee
  for( il= 1; il <= SD_BAS_AMLI; il++)
  {
    for( ic= 1; ic <= SD_BAS_AMCO; ic++)
    {
      InfoDet1C.tab_var[N1C_GCCSIMAGECLASSIFIED].val.pi4[IND3(ic,il,SN,SD_BAS_AMCO,SD_BAS_AMLI)]=
        ImageClassified[IND2(ic,il,SD_BAS_AMCO)];
    }
  }

  InfoDet1C.tab_var[N1C_GCCSIMAGECLASSIFIEDNBLIN].val.pi4[IND1(SN)]=    ImageClassifiedNbLin;
  InfoDet1C.tab_var[N1C_GCCSIMAGECLASSIFIEDNBCOL].val.pi4[IND1(SN)]=    ImageClassifiedNbCol;
  // FT-2980
    // Si (Mode = Avhrr) alors
    if( CCSMode == SD_ALG_AVHRR_MODE_AVHRR)
    {

  		//jlp70
  		//@ Conversion en millisecondes
   		Tdeb    = (int)(AvhrrListeLin[ImageClassifiedFirstLin]->InfoGene.tab_var[AVHRR_DATEDEB].val.pdate)->day * CMN__DATE_SEC_PAR_JOUR*1000 + 
                             (AvhrrListeLin[ImageClassifiedFirstLin]->InfoGene.tab_var[AVHRR_DATEDEB].val.pdate)->millis;
  		//@ cf PFS 2.5
  		T0    = (int)(IasiTime0.day * CMN__DATE_SEC_PAR_JOUR*1000 + IasiTime0.millis);
  		PosLinMillis = (int)(Tdeb - T0);
  		InfoDet1C.tab_var[N1C_GCCSIMAGECLASSIFIEDFIRSTLIN].val.pr8[IND1(SN)]= PosLinMillis;

    }
    // Sinon (Mode = IIS)
    else
    {  
  		InfoDet1C.tab_var[N1C_GCCSIMAGECLASSIFIEDFIRSTLIN].val.pr8[IND1(SN)]= ImageClassifiedFirstLin;  
    }
  // ...FT-2980
  InfoDet1C.tab_var[N1C_GCCSIMAGECLASSIFIEDFIRSTCOL].val.pr8[IND1(SN)]= ImageClassifiedFirstCol;
  // ----------------------------------------------------------------------------------------

  // ----------------------------------------------------------------------------------------
  // Flags indicateurs de l'etat des canaux AVHRR
  for( k= 1; k <= SD_BAS_NBK; k++)
  {
    InfoDet1ENG.tab_var[ENG_GCCSCONFAVHRRCHANNEL].val.pui4[IND2(k,SN,SD_BAS_NBK)]=
      ConfAvhrrChannel[IND1(k)];
  }
  // ----------------------------------------------------------------------------------------

  // ----------------------------------------------------------------------------------------
   InfoDet1ENG.tab_var[ENG_GCCSFLAGPOSTPROCESSING].val.pi4[IND1(SN)]=    FlagPostProcessing;
   InfoDet1ENG.tab_var[ENG_GCCSNONCLASSIFRATE].val.pr8[IND1(SN)]=        NonClassifRate;
   InfoDet1ENG.tab_var[ENG_GCCSVARIANCERATE].val.pr8[IND1(SN)]=          VarianceRate;
  // ----------------------------------------------------------------------------------------
  
  // On trace ici toutes les variables en sortie
  #ifdef INVESTIGATION
    SD_BAS_TRACE( (char *)tcProcName_F, "RadAnalType",
                  1, SD_BAS_NCL_MAX, 1,
                  TRACE_INT4,  TRACE_ASC,
                  RadAnalType,             Contexte_courant);

    SD_BAS_TRACE(  (char *)tcProcName_F, "ImageClassified",
                   SD_BAS_AMLI, SD_BAS_AMCO, 1,
                   TRACE_INT4,  TRACE_BIN,
                   ImageClassified,         Contexte_courant);
    SD_BAS_TRACE( (char *)tcProcName_F, "ImageClassifiedNbLin",    1, 1, 1,
                  TRACE_INT4,  TRACE_ASC, &ImageClassifiedNbLin,    Contexte_courant);
    SD_BAS_TRACE( (char *)tcProcName_F, "ImageClassifiedNbCol",    1, 1, 1,
                  TRACE_INT4,  TRACE_ASC, &ImageClassifiedNbCol,    Contexte_courant);
    SD_BAS_TRACE( (char *)tcProcName_F, "ImageClassifiedFirstLin", 1, 1, 1,
                  TRACE_INT4,  TRACE_ASC, &ImageClassifiedFirstLin, Contexte_courant);
    SD_BAS_TRACE( (char *)tcProcName_F, "ImageClassifiedFirstCol", 1, 1, 1,
                  TRACE_INT4,  TRACE_ASC, &ImageClassifiedFirstCol, Contexte_courant);

    SD_BAS_TRACE( (char *)tcProcName_F, "ConfAvhrrChannel",
                  1, SD_BAS_NBK, 1,
                  TRACE_INT4,  TRACE_ASC,
                  ConfAvhrrChannel,        Contexte_courant);

    SD_BAS_TRACE( (char *)tcProcName_F, "FlagPostProcessing",      1, 1, 1,
                  TRACE_INT4,  TRACE_ASC, &FlagPostProcessing,      Contexte_courant);
    SD_BAS_TRACE( (char *)tcProcName_F, "NonClassifRate",          1, 1, 1,
                  TRACE_REAL8, TRACE_ASC, &NonClassifRate,          Contexte_courant);
    SD_BAS_TRACE( (char *)tcProcName_F, "GCcsVarianceRate",        1, 1, 1,
                  TRACE_REAL8, TRACE_ASC, &VarianceRate,            Contexte_courant);
  #endif

fin:
  // FT-2528
  SD_BAS_MacFREE(QualityIndicator);
  SD_BAS_MacFREE(ScanLineQuality);
  SD_BAS_MacFREE(CountErrorFrame);
  SD_BAS_MacFREE(CloudInformation);
  // ...FT-2528
  SD_BAS_MacFREE(ChId );
  SD_BAS_MacFREE(ChNoise);
  SD_BAS_MacFREE(ChWgt);
  SD_BAS_MacFREE(RadVect);
  SD_BAS_MacFREE(ImageClassified);
  
  // sortie de la fonction
  switch (cr_op)
  {
    case SD_ALG_AVHRR_41_CCS_ALRT:
    {  
#ifdef OPS_DEBUG
  printf (" echec du calcul de la fonction SD_ALG_R1_CCS.\n");
#endif
      /* "ERROR ON AVHHR CHANNEL PROCESSING" defini dans SD.msg */
      SD_ALG_AVHRR_MsgSet = SD_ALG_AVHRR_SET; /* defini dans SD__Message.h */
      SD_ALG_AVHRR_MsgNum = MSG_ALG_AVHRR_41_CCS_ALRT; /* defini dans l'enumere e_SPECMessageJDB de SD__Message.h */
      CreerMessage (
        SD_ALG_AVHRR_MsgSet, 
        SD_ALG_AVHRR_MsgNum, 
        &(ctx->cr.msgJDB));
      cr_op = CMN__STATUT_NOK;
    }
    break;
    case SD_ALG_AVHRR_41_CCS_ALRT_TRAIT_CANAUX:
    {
#ifdef OPS_DEBUG
  printf (" echec du calcul de la fonction SD_ALG_TraitCanauxAvhrr.\n");
#endif
      /* "ERROR ON AVHHR CHANNEL PROCESSING" defini dans SD.msg */
      SD_ALG_AVHRR_MsgSet = SD_ALG_AVHRR_SET; /* defini dans SD__Message.h */
      SD_ALG_AVHRR_MsgNum = MSG_ALG_AVHRR_41_CCS_ALRT_TRAIT_CANAUX; /* defini dans l'enumere e_SPECMessageJDB de SD__Message.h */
      CreerMessage (
        SD_ALG_AVHRR_MsgSet, 
        SD_ALG_AVHRR_MsgNum, 
        &(ctx->cr.msgJDB));
      cr_op = CMN__STATUT_NOK;
    }
    break;
    
    case SD_ALG_AVHRR_41_CCS_ALRT_NB_PIXAV_NUL:
    {
#ifdef OPS_DEBUG
  printf (" echec du calcul de la fonction SD_ALG_TraitCanauxAvhrr.\n");
#endif
      /* "ERROR AVHRR PIXEL NUMBER IS ZERO" defini dans SD.msg */
      SD_ALG_AVHRR_MsgSet = SD_ALG_AVHRR_SET; /* defini dans SD__Message.h */
      SD_ALG_AVHRR_MsgNum = MSG_ALG_AVHRR_41_CCS_ALRT_NB_PIXAV_NUL; /* defini dans l'enumere e_SPECMessageJDB de SD__Message.h */
      CreerMessage (
        SD_ALG_AVHRR_MsgSet, 
        SD_ALG_AVHRR_MsgNum, 
        &(ctx->cr.msgJDB));
      cr_op = CMN__STATUT_NOK;
    }
    break;
    
    default:
    {
      if (cr_op != CMN__STATUT_OK)
      {
        cr_op = CMN__STATUT_NOK;
      }
    }
  }
  
  return( cr_op);
}
// FIN 41_CCS ====================================================================
// ===============================================================================

// FT-2521
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_42_PCH
//@
//@ Description : Calcul de 4 pseudo-canaux AVHRR
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx .......... : contexte de l'etape (numero de ligne SN traite)
//@         (ENTREE)  ConfAutre .... : fichier de conf autre
//@         (ENTREE)  ConfStable ... : fichier de conf stable
//@         (ENTREE)  spectre1C..... : spectre IASI de niveau 1C
//@         (ENTREE)  InfoDet1C .... : info det du produit 1C
//@         (SORTIE)  InfoDet1ENG .. : info det du produit ENG
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
#include <math.h>
t_CMN__Statut SD_ALG_42_PCH(
  // Entree
  t_SD_BAS_DataThread *ctx,
  t_SD_BAS_ConfAutre *ConfAutre,
  t_SD_BAS_ConfStable *ConfStable,
  double *spectre1C, // spectre IASI de niveau 1C
  t_SD_BAS_Liste_Var InfoDet1C,
  // Sortie
  t_SD_BAS_Liste_Var InfoDetENG
  )
{
static const char tcProcName_F[]= "SD_ALG_42_PCH";
t_CMN__Statut CodeRetour = CMN__STATUT_OK;

#ifdef OPS_DEBUG
  printf ("Debut de %s\n", tcProcName_F);
#endif

  // Entrees
  int SN;                                           // numero de sous-cycle
  int PN;                                           // numero de pixel
  int RadAnalNbClass;                               // nombre de  classes dans le pixel
  double *RadAnalMean;                              // Radiances moyennes AVHRR classC)es - double( SD_BAS_NBK * SD_BAS_NCL_MAX)
  double *RadAnalWgt;                               // Poids des classes dans le pixel
  int Nsfirst;                                      // numero du premier echantillon du spectre S1C
  int Nslast;                                       // numero du dernier echantillon du spectre S1C
  int *AvhrrNsfirst;                                // numero du premier echantillon des filtres AVHRR 4 et 5
  int *AvhrrNslast;                                 // numero du premier echantillon des filtres AVHRR 4 et 5
  double *AvhrrWgt;                                 // Filtres AVHRR 4 et 5
  double DWnSpect;                                  // largeur spectrale d'un echantillon spectral

  // sorties
  // MDR ENG : GPchAvhrrPseudoChn : AVHRR4 like IASI
  // MDR ENG : GPchAvhrrPseudoChn : AVHRR5 like IASI
  // MDR ENG : GPchAvhrrPseudoChn : IASI like AVHRR4
  // MDR ENG : GPchAvhrrPseudoChn : IASI like AVHRR5

  // Local
  int c;
  int Ns;
  int Ls;
  int Ms;
  double AvhrrPseudoChn;

  // Initialisations
  SN             = ctx->ctxtrt.SN;
  PN             = ctx->ctxtrt.PN;
  DWnSpect       = ConfAutre->IDefSpectDWn1b;
  Nsfirst        = ConfAutre->IDefNsfirst1b;
  Nslast         = ConfAutre->IDefNslast1b;
  AvhrrNsfirst   = ConfStable->IDefAvhrrNsfirst;
  AvhrrNslast    = ConfStable->IDefAvhrrNslast;
  AvhrrWgt       = ConfStable->IDefAvhrrWgt;

  RadAnalNbClass = InfoDet1C.tab_var[N1C_GCCSRADANALNBCLASS].val.pi4[IND2(PN,SN,SD_BAS_PN)];
  RadAnalMean    = InfoDet1C.tab_var[N1C_GCCSRADANALMEAN].val.pr8;
  RadAnalWgt     = InfoDet1C.tab_var[N1C_GCCSRADANALWGT].val.pr8;

  // i=1, AVHRR 4
  AvhrrPseudoChn = 0.0;
  for (c = 1; c <= SD_BAS_NCL; c++)
  {
    AvhrrPseudoChn += RadAnalMean[IND4(5,c,PN,SN,SD_BAS_NBK,SD_BAS_NCL,SD_BAS_PN)]*
                      RadAnalWgt[IND3(c,PN,SN,SD_BAS_NCL,SD_BAS_PN)];
  }
  InfoDetENG.tab_var[ENG_GPCHAVHRRPSEUDOCHN].val.pr8[IND3(1,PN,SN,SD_BAS_PS,SD_BAS_PN)] = AvhrrPseudoChn;

  // i=2, AVHRR 5
  AvhrrPseudoChn = 0.0;
  for (c = 1; c <= SD_BAS_NCL; c++)
  { 
    AvhrrPseudoChn += RadAnalMean[IND4(6,c,PN,SN,SD_BAS_NBK,SD_BAS_NCL,SD_BAS_PN)]*
                      RadAnalWgt[IND3(c,PN,SN,SD_BAS_NCL,SD_BAS_PN)];
  }
  InfoDetENG.tab_var[ENG_GPCHAVHRRPSEUDOCHN].val.pr8[IND3(2,PN,SN,SD_BAS_PS,SD_BAS_PN)] = AvhrrPseudoChn;

  // i=1
  AvhrrPseudoChn = 0.0;
  for (Ns = AvhrrNsfirst[IND1(1)]; Ns <= AvhrrNslast[IND1(1)]; Ns++)
  {
    Ms = Ns - Nsfirst + 1;
    Ls = Ns - AvhrrNsfirst[IND1(1)] + 1;
    // FT-2661
    AvhrrPseudoChn += spectre1C[IND1(Ms)]*AvhrrWgt[(IND1(Ls)*2)];
    // ...FT-2661
  }
  InfoDetENG.tab_var[ENG_GPCHAVHRRPSEUDOCHN].val.pr8[IND3(3,PN,SN,SD_BAS_PS,SD_BAS_PN)] = AvhrrPseudoChn;

  // i=2
  AvhrrPseudoChn = 0.0;
  for (Ns = AvhrrNsfirst[IND1(2)]; Ns <= AvhrrNslast[IND1(2)]; Ns++)
  {
    Ms = Ns - Nsfirst + 1;
    Ls = Ns - AvhrrNsfirst[IND1(2)] + 1;
    // FT-2661
    AvhrrPseudoChn += spectre1C[IND1(Ms)]*AvhrrWgt[(IND1(Ls)*2)+1]; //AVHRR5 
    // ...FT-2661 
  }
  InfoDetENG.tab_var[ENG_GPCHAVHRRPSEUDOCHN].val.pr8[IND3(4,PN,SN,SD_BAS_PS,SD_BAS_PN)] = AvhrrPseudoChn;
  
#ifdef OPS_DEBUG
  printf ("Fin de %s\n", tcProcName_F);
#endif
}
// FIN 42_PCH ====================================================================
// ===============================================================================
// ...FT-2521


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_44_GEO
//@
//@ Description : Geolocalisation des produits IASI
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  ctp : variables temporaires
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (ENTREE)  ConfStable : fichier de conf stable
//@         (ENTREE)  StatMphr : Infos globales MPHR
//@         (ENTREE)  Mphr :  MPHR produit N1
//@         (Entree)  InfoDet1ENG : info det du produit ENG
//@         (Sortie)  InfoDet1ABC : info det du produit 1ABC pour le SN (on choisira la date UTC)
//@         (ENTREE)  nbligbandeau : nombre de lignes total du bandeau
//@         (ENTREE)  BandeauAVHRR : bandeau AVHRR charge en memoire
//@         (ENTREE)  OffsetIISAvhrr : Offset IIS-AVHRR dans le repere AVHRR (ligne,colonne)
//@         (ENTREE)  flag_geo_avhrr_miss : vaut VRAI si CG9 ou CG15
//@
//@ Return Status : test
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/


t_CMN__Statut SD_ALG_44_GEO(
  	t_SD_BAS_DataThread         *ctx,
  	t_SD_BAS_DataThreadProduit  *ctp,
  	t_SD_BAS_ConfAutre          *ConfAutre,
  	t_SD_BAS_ConfStable         *ConfStable,
  	t_SD_BAS_StatMphr           *StatMphr,
	t_SD_BAS_MPHR               *Mphr,
  	t_SD_BAS_Liste_Var          *InfoDet1ENG,
  	t_SD_BAS_Liste_Var          *InfoDet1ABC ,
  	int                         nbligbandeau,
  	t_SD_BAS_LigneAVHRR         **BandeauAVHRR,
  	/* CR242 */
	// FT-2997
  	double                      *OffsetIISAvhrrRI,	//Sortie de calcOffset
	// ...FT-2997
  	t_CMN__Booleen              flag_geo_avhrr_miss 
	)
	{
    	static char tcProcName_F[] = "SD_ALG_44_GEO";
    	t_CMN__Statut cr_op=CMN__STATUT_OK;

	    // Entree dans la fonction
	
	    // Recuperation des infos contenues dans les structure
	    int                 SN;             // Numero de sous cycle
	    int                 LN;
	    int                 SP;
	
	    // Entrees
	    t_SD_BAS_SCDSTime   IasiTime;
	    t_SD_BAS_SCDSTime   IasiTime0;
	    
	    /* CR242 */
	    t_SD_BAS_SCDSTime   t_FE;
	    double              Period;
	    double              *OffsetSondIISModel;  // double( SD_BAS_MODEL * SD_BAS_CCD * SD_BAS_PN)
	    
	    int                 SNmin;             // Numero de sous cycle moni
	    double              IasiTimeSec;
	    double              ScanAngle;
	    double              *PsfBarycentreY;
	    double              *PsfBarycentreZ;
	    double              *MatRot;
	    double              *IISY;
	    double              *IISZ;
	    int                 *NY;
	    int                 *NZ;
	    int                 Ssize;
	    double              IISRot;
	    int                 AvhrrColLeft;
	    int                 AvhrrColRight;
	    int                 AvhrrLigFirst;
	    int                 AvhrrLigLast;
	    double              AvhrrDatBefore;
	    double              AvhrrDatAfter;
	
	
	    // Pour les appels a SD_BAS_OperateurLoc
	    double              IISCornerMetop[8];     // Les 4 azims suivis des 4 elev dans le repere METOP
	    double              IISCornerAvhrrLin[4];  // Lignes AVHRR dans la conversion Metop2AVHRR
	    double              IISCornerAvhrrCol[4];  // Colonnes AVHRR dans la conversion Metop2AVHRR
	    double              EarthSatDist[4];       // Colonnes AVHRR dans la conversion Metop2AVHRR
	    double		 		LocIasiAvhrrLin[4]; // position en ligne des IPSF
	    double		 		LocIasiAvhrrCol[4]; // position en ligne des IPSF
	    double              lat[4];
	    double              lon[4];
	    double              Ang_mesY[SD_BAS_PN];
	    double              Ang_mesZ[SD_BAS_PN];
	    double              Azim_sol[SD_BAS_PN];
	    double              Elev_sol[SD_BAS_PN];
	
	    // Internes
	    double              datetmp;
	    double              PixY[SD_BAS_PN];
	    double              PixZ[SD_BAS_PN];
	    int                 PosLinMillis;
	    int                 Tdeb;
	    int                 Tfin;
	    int                 T0;
	    int                 u;
	    double              x;
	
	    // Gestion de erreurs
	    t_CMN__Statut irc;
		/* bma intialisation */
		/*
    	char *msg_err;          // Message d'erreur
    	int num_lig;            // Ligne d'erreur
		*/
    	char *msg_err = (char*) NULL;          // Message d'erreur
    	int num_lig = 0;            // Ligne d'erreur
		/* fin bma */

    	// Compteurs
    	int i, il, ic, ct;

    	#ifdef OPS_DEBUG
        	printf("Entree dans la fonction 44 GEO\n========================================\n");
    	#endif

	    // Initialisations
	    SN                      = ctx->ctxtrt.SN;
	    SP                      = InfoDet1ENG->tab_var[ENG_GEPS_SP].val.pui4[IND1(SN)];
	    LN                      = ctx->ctxtrt.LN; 
	    
	    /* Definir SNmin en fonction du mode de fonctionnement */
	    /* SI modePdv = external_calibration */
	    if( InfoDet1ENG->tab_var[ENG_MODE].val.pui4[0] == 1)
	    {
	      	/* SNmin = 3  */
	     	//CR270 on se positionne toujours par rapport au SN1
	      	SNmin = 1;
	    }
	    /* SINON (mode normal ou auxiliary) */
	    else
	    {
	    	/* SNmin = 1 */
	      	SNmin = 1;
	    }
	    
	    IasiTime0               = InfoDet1ENG->tab_var[ENG_GEPSDATIASI].val.pdate[IND1(SNmin)];
	    IasiTime                = InfoDet1ENG->tab_var[ENG_GEPSDATIASI].val.pdate[IND1(SN)];
	    IasiTimeSec     	    = CMN__DATE_CONV_SCDS_2_SEC(IasiTime);
	    ScanAngle               = ConfAutre->IDefIASIScanAngle[IND1(SP)];
	    PsfBarycentreY          = ConfAutre->IDefPsfSondBaryCentreY;
	    PsfBarycentreZ          = ConfAutre->IDefPsfSondBaryCentreZ;
	    MatRot                  = ConfAutre->IDefIASIMETOPMatRot;
	    IISY                    = ConfAutre->IDefIISY;
	    IISZ                    = ConfAutre->IDefIISZ;
	    NY                      = ConfAutre->IDefGeoIISSubGridNY;
	    NZ                      = ConfAutre->IDefGeoIISSubGridNZ;
	    Ssize                   = ConfAutre->IDefGeoIISSubGridSize;
	    IISRot                  = ConfAutre->IDefIISRot;
	
	    /* FA142 */
	    t_FE                    = *(InfoDet1ENG->tab_var[ENG_GEPSENDECLIPSETIME].val.pdate);  // t_SD_BAS_SCDSTime*
	    Period                  = ConfAutre->IDefMETOPOrbitPeriod;
	    OffsetSondIISModel      = ConfAutre->IDefOffsetSondIISModel;
  
	  	// FA 202
	  	if(34 != SP)
	    {
	    	AvhrrDatBefore          = IasiTimeSec - ConfStable->IDefDeltaAvhrrDTbefore[IND1(SP)];
	      	AvhrrDatAfter           = IasiTimeSec + ConfStable->IDefDeltaAvhrrDTafter[IND1(SP)];
	
		    AvhrrColLeft            = ConfStable->IDefDeltaAvhrrColLeft[IND1(SP)];
	      	AvhrrColRight           = ConfStable->IDefDeltaAvhrrColRight[IND1(SP)];
	    }
	  	else
	    {
	    	AvhrrDatBefore          = IasiTimeSec - ConfStable->IDefDeltaAvhrrDTbefore[IND1(15)];
	    	AvhrrDatAfter           = IasiTimeSec + ConfStable->IDefDeltaAvhrrDTafter[IND1(15)];
	
	    	AvhrrColLeft            = ConfStable->IDefDeltaAvhrrColLeft[IND1(16)];
	    	AvhrrColRight           = ConfStable->IDefDeltaAvhrrColRight[IND1(15)];
	    }
	   	// FIN FA 202     
	    
	    /* Fin FA142 */
	
	    /* CR242 */
	    if (flag_geo_avhrr_miss == CMN__FAUX)
	    {
		    /* FIN CR242 */
		    
		    // Calcul des numeros de premiere et derniere ligne de l'image AVHRR en fonction des dates
		    //------------------------------------------------------------------------
		    // Premiere ligne
		    AvhrrLigFirst=0;
		    il=0;
		    datetmp = CMN__DATE_CONV_SCDS_2_SEC(*(BandeauAVHRR[il]->InfoGene.tab_var[AVHRR_DATEDEB].val.pdate));
		    printf("BANDEAU AVHRR dateligne= %lf \n",(double)IasiTimeSec);
		    while(datetmp <= AvhrrDatBefore && il < nbligbandeau)
		    {
		    	datetmp = (double)CMN__DATE_CONV_SCDS_2_SEC(*(BandeauAVHRR[il]->InfoGene.tab_var[AVHRR_DATEDEB].val.pdate));
		        il++;
		        #ifdef OPS_DEBUG
		        	printf("   DATEDEB de la ligne %d : %lf\n",il,datetmp);
		        #endif
		    }
		    AvhrrLigFirst = il-1;
		    if(AvhrrLigFirst<0) AvhrrLigFirst=0;
		
		    #ifdef OPS_DEBUG
		        printf("   Ligne AVHRR correspondant a  DATEDEB : %d\n",AvhrrLigFirst);
		    #endif
		
		    // Derniere ligne
		    AvhrrLigLast=0;
		    il=0;
		    datetmp = CMN__DATE_CONV_SCDS_2_SEC(*(BandeauAVHRR[il]->InfoGene.tab_var[AVHRR_DATEFIN].val.pdate));
		    while(datetmp < AvhrrDatAfter && il < nbligbandeau)
		    {
		        datetmp = CMN__DATE_CONV_SCDS_2_SEC(*(BandeauAVHRR[il]->InfoGene.tab_var[AVHRR_DATEFIN].val.pdate));
		        il++;
		    #ifdef OPS_DEBUG
		        printf("   DATEFIN de la ligne %d : %lf\n",il,datetmp);
		    #endif
		
		    }
		    AvhrrLigLast = il;
		    if(AvhrrLigLast>nbligbandeau-1) AvhrrLigLast=nbligbandeau-1;
		
		    #ifdef OPS_DEBUG
		        printf("   Ligne AVHRR correspondant a  DATEFIN : %d\n",AvhrrLigLast);
		    #endif		    
		   /* CR242 */
	    }

    //@ 1) Caracteristique du barycentre des 4 IPSF
    //=============================================================

    //@ Boucle sur les 4 IPSF du sondeur
    #ifdef OPS_DEBUG
        printf("Caracteristique du barycentre des 4 IPSF\n=====================================\n");
        // Utilisation de la sous-matrice | A B C |
        // 2 x 2 relative a Y et Z dans   | D E F | => | E F |
        // la matrice 3 x 3 (MatRot)      | G H K |    | H K |
        printf("Matrice MatRot : %lf\t%lf\n",MatRot[IND2(2,2,SD_BAS_DIM_ROT_MATRICE)],
                                             MatRot[IND2(3,2,SD_BAS_DIM_ROT_MATRICE)]);
        printf("                 %lf\t%lf\n",MatRot[IND2(2,3,SD_BAS_DIM_ROT_MATRICE)],
                                             MatRot[IND2(3,3,SD_BAS_DIM_ROT_MATRICE)]);
        printf("ScanAngle : %lf\n",ScanAngle);
    #endif

    for(i=1; i<=SD_BAS_PN; i++)
    {
        //@ Coordonnees dans le repere IASI
      	//  conversion en radian
        PixY[IND1(i)] = PsfBarycentreY[IND1(i)] 
        					+ InfoDet1ENG->tab_var[ENG_GCCSOFFSETSONDAVHRR].val.pr8[IND3(1,i,SN,SD_BAS_CCD,SD_BAS_PN)]*M_PI/180.;
        PixZ[IND1(i)] = PsfBarycentreZ[IND1(i)] 
        					+ InfoDet1ENG->tab_var[ENG_GCCSOFFSETSONDAVHRR].val.pr8[IND3(2,i,SN,SD_BAS_CCD,SD_BAS_PN)]*M_PI/180.;

        //@ Coordonnees dans le repere METOP
      	SD_ALG_GeomIasiMetop(MatRot,ScanAngle,PixY[IND1(i)],PixZ[IND1(i)],
                         		&IISCornerMetop[i-1],    // Azim du pixel i
                            	&IISCornerMetop[i+3]   // Elev du pixel i
        					);

        #ifdef OPS_DEBUG
            printf("\nPixel no%d : Coordonnees dans le repere IASI\n",i);
            printf("pb call SD_BAS_MetopGeod from ALG_AVHRR line %d\n",__LINE__);

            printf("   PsfBarycentreY = %lf\n",PsfBarycentreY[IND1(i)]);
            printf("   PsfBarycentreZ = %lf\n",PsfBarycentreZ[IND1(i)]);
            printf("   OffsetSondAvhrrY = %lf\n",InfoDet1ENG->tab_var[ENG_GCCSOFFSETSONDAVHRR].val.pr8[IND3(1,i,SN,SD_BAS_CCD,SD_BAS_PN)]);
            printf("   OffsetSondAvhrrZ = %lf\n",InfoDet1ENG->tab_var[ENG_GCCSOFFSETSONDAVHRR].val.pr8[IND3(2,i,SN,SD_BAS_CCD,SD_BAS_PN)]);
            printf("   PixY = %lf\n",PixY[IND1(i)]);
            printf("   PixZ = %lf\n",PixZ[IND1(i)]);
            printf("Pixel no%d : Coordonnees dans le repere METOP\n",i);
            printf("   Azim = %lf\n",IISCornerMetop[i]);
            printf("   Elev = %lf\n",IISCornerMetop[i+4]);
        #endif
    }
    //@ Fin de boucle sur les IPSF


    //@ Coordonnees dans le raster AVHRR : factorisees pour les 4 IPSF
    
    /* CR242 */
    if (flag_geo_avhrr_miss == CMN__FAUX)
    {
    	/* FIN CR242 */
		// SD_BAS_ReserverOperateur();
    	irc = SD_BAS_MetopAVHRR(
						      	ctx,
						      	&IasiTime,
						      	Mphr,SD_BAS_PN,             // Pour les 4 IPSF
						      	&IISCornerMetop[0], // tableau des 4 azims
						      	&IISCornerMetop[SD_BAS_PN], // tableau des 4 elev
						      	AvhrrLigFirst,
						      	AvhrrLigLast,
						      	BandeauAVHRR,
						      	nbligbandeau,
						      	AvhrrColLeft,
						      	AvhrrColRight,
						      	lat,
						      	lon,
						      	Ang_mesY,
						      	Ang_mesZ,
						      	Azim_sol,
						      	Elev_sol,
						      	IISCornerAvhrrCol,
						      	IISCornerAvhrrLin,
						      	EarthSatDist);
		// SD_BAS_LibererOperateur();
    	if(irc != CMN__STATUT_OK)
    	{
        	cr_op   = SD_ALG_AVHRR_44_GEO_ALRT;
        	msg_err = (char *)SD_BAS_MSG_ERR_METOP2AVHRR;
        	num_lig = __LINE__;
         	goto fin;
    	}

    	//@ Boucle sur les IPSF
    	for(i=0; i < SD_BAS_PN; i++)
    	{
      		LocIasiAvhrrLin[i] = IISCornerAvhrrLin[i] ;
      		LocIasiAvhrrCol[i] = IISCornerAvhrrCol[i] ;
    	}
    	//@ Fin de boucle sur les IPSF
    	/* CR242 */
	}
    //@ SI flag_geo_avhrr_miss == VRAI ALORS
    else
    {
    	for(i=0; i<SD_BAS_PN; i++)
      	{
        	LocIasiAvhrrLin[i] = 0.0;
        	LocIasiAvhrrCol[i] = 0.0;
      	}
      
      	SD_BAS_ReserverOperateur();
    #ifdef OPS_DEBUG
        printf("pb call SD_BAS_MetopGeod from ALG_AVHRR line %d\n",__LINE__);
    	for(i=0; i<SD_BAS_PN; i++)
        {
            printf("pb  Pixel no %d : Coordonnees dans le raster AVHRR\n",i);
            printf("pb   IISCornerAvhrrCol = %lf\n",IISCornerAvhrrCol[i]);
            printf("pb   IISCornerAvhrrLin = %lf\n",IISCornerAvhrrLin[i]);
            printf("pb   IISCornerMetop[0] = %lf\n",IISCornerMetop[i]);
            printf("pb   IISCornerMetop[4] = %lf\n",IISCornerMetop[i+4]);
        }
    #endif

      	irc = SD_BAS_MetopGeod(
						        ctx,
						        &IasiTime,
						        Mphr,
						        SD_BAS_PN,
						        &IISCornerMetop[0],
						        &IISCornerMetop[4],
						        lat,
						        lon,
						        Ang_mesY,
						        Ang_mesZ,
						        Azim_sol,
						        Elev_sol,
						        EarthSatDist);
      	SD_BAS_LibererOperateur();
    #ifdef OPS_DEBUG
    if(irc != CMN__STATUT_OK)
    {
        printf("pb  erreur SD_BAS_MetopGeod => Code retour %d\n",irc);
    }
    #endif

      	if(irc != CMN__STATUT_OK)
      	{
        	cr_op   = SD_ALG_AVHRR_44_GEO_ALRT;
        	msg_err = (char *)SD_BAS_MSG_ERR_METOP2GEOD;
        	num_lig = __LINE__;
        	goto fin;
      	}
	}
    /* Fin CR242 */
    
    #ifdef OPS_DEBUG
    	for(i=0; i<SD_BAS_PN; i++)
        {
            printf("\nPixel no%d : Coordonnees dans le raster AVHRR\n",i);
            printf("   IISCornerAvhrrCol = %lf\n",IISCornerAvhrrCol[i]);
            printf("   IISCornerAvhrrLin = %lf\n",IISCornerAvhrrLin[i]);
            printf("Coordonnees geographiques et angles de mesure\n",i);
            printf("   lat = %lf\n",lat[i]);
            printf("   lon = %lf\n",lon[i]);
            printf("   Ang_mesY = %lf\n",Ang_mesY[i]);
            printf("   Ang_mesZ = %lf\n",Ang_mesZ[i]);
            printf("   Azim_sol = %lf\n",Azim_sol[i]);
            printf("   Elev_sol = %lf\n",Elev_sol[i]);
            printf("   EarthSatDist = %lf\n",EarthSatDist[i]);
        }
    #endif

    //@ 2) Caracteristiques des pixels de la sousgrille de l'IIS
    //=============================================================
    #ifdef OPS_DEBUG
        printf("\nCaracteristiques des pixels de la sousgrille de l'IIS\n==================================\n",i);
        printf("Angle IISRot : %lf\n",IISRot);
        // Utilisation de la sous-matrice | A B C |
        // 2 x 2 relative a Y et Z dans   | D E F | => | E F |
        // la matrice 3 x 3 (MatRot)      | G H K |    | H K |
        printf("Matrice MatRot : %lf\t%lf\n",MatRot[IND2(2,2,SD_BAS_DIM_ROT_MATRICE)],
                                             MatRot[IND2(3,2,SD_BAS_DIM_ROT_MATRICE)]);
        printf("                 %lf\t%lf\n",MatRot[IND2(2,3,SD_BAS_DIM_ROT_MATRICE)],
                                             MatRot[IND2(3,3,SD_BAS_DIM_ROT_MATRICE)]);
        printf("ScanAngle : %lf\n",ScanAngle);
    #endif

    //@ Boucle sur les pixels de la sousgrille
    for(il=1,ct=0; il<=Ssize; il++)
    {
        for(ic=1; ic<=Ssize; ic++,ct++)
        {
            //@ Coordonnees dans le repere IASI
			// FT-2997
            ctp->SgIISY[ct] = IISY[NY[IND1(ic)]-1]*cos(IISRot) - IISZ[NZ[IND1(il)]-1]*sin(IISRot)
            						+ OffsetIISAvhrrRI[IND1(1)];

            ctp->SgIISZ[ct] = IISY[NY[IND1(ic)]-1]*sin(IISRot) + IISZ[NZ[IND1(il)]-1]*cos(IISRot)
            						+ OffsetIISAvhrrRI[IND1(2)];
			// ...FT-2997

           	//@ Coordonnees dans le repere METOP
            SD_ALG_GeomIasiMetop(MatRot,ScanAngle,
                                 ctp->SgIISY[ct],
                                 ctp->SgIISZ[ct],
                                 &ctp->SgIISRM[ct],                // Azim
                                 &ctp->SgIISRM[ct + Ssize*Ssize]   // Elev
            );
            
        	#ifdef OPS_DEBUG
            	printf("\nPixel(ic,il) = (%d,%d)\n",ic,il);
            	printf("Coordonnees dans le repere IASI\n",i);
            	printf("   SgIISY : %lf\n",ctp->SgIISY[ct]);
            	printf("   SgIISZ : %lf\n",ctp->SgIISZ[ct]);
            	printf("Coordonnees dans le repere METOP\n",i);
            	printf("   Azim : %lf\n",ctp->SgIISRM[ct]);
            	printf("   Elev : %lf\n",ctp->SgIISRM[ct + Ssize*Ssize]);
        	#endif
		}
    }
    //@ Fin de boucle sur les pixel de la sousgrille

    //@ Coordonnees dans le raster AVHRR : factorisees pour tous les pixels de la sousgrille
    /* CR242 */
    if (flag_geo_avhrr_miss == CMN__FAUX)
    {
    	/* FIN CR242 */
    
		// SD_BAS_ReserverOperateur();
    	irc = SD_BAS_MetopAVHRR(ctx,&IasiTime,
        	                    Mphr,Ssize*Ssize,              // Pour tous les pixels de la sousgrille
            	                &ctp->SgIISRM[0],              // tableau des azims
                	            &ctp->SgIISRM[Ssize*Ssize],    // tableau des elev
                    	        AvhrrLigFirst,AvhrrLigLast,BandeauAVHRR,
                        	    nbligbandeau,AvhrrColLeft,AvhrrColRight,
                            	ctp->lat,ctp->lon,
	                            ctp->Ang_mesY,ctp->Ang_mesZ,
    	                        ctp->Azim_sol,ctp->Elev_sol,
        	                    ctp->IISCornerAvhrrCol,ctp->IISCornerAvhrrLin,
            	                ctp->EarthSatDist);
		// SD_BAS_LibererOperateur();
	   	if(irc != CMN__STATUT_OK)
	    {
	    	cr_op   = SD_ALG_AVHRR_44_GEO_ALRT;
	        msg_err = (char *)SD_BAS_MSG_ERR_METOP2AVHRR;
	        num_lig = __LINE__;
	        goto fin;
	    }
	
		//@ Boucle sur les pixels de la sousgrille
		for(il=1,ct=0; il<=Ssize; il++)
		{
    		for(ic=1; ic<=Ssize; ic++,ct++)
    		{
        		// FT-2997
        		ctp->LocIasiAvhrrLin[ct] = ctp->IISCornerAvhrrLin[ct];
        		ctp->LocIasiAvhrrCol[ct] = ctp->IISCornerAvhrrCol[ct];
				// ...FT-2997
    		}
		}
		//@ Fin de boucle sur les pixel de la sousgrille
        // FT-2997
	    SD_BAS_ReserverOperateur();
        #ifdef OPS_DEBUG
        printf("pb call SD_BAS_MetopGeod from ALG_AVHRR line %d\n",__LINE__);
        printf("pb   nbpoints %d\n",Ssize*Ssize);
        #endif

	    irc = SD_BAS_MetopGeod(
	        					ctx,
	        					&IasiTime,
	        					Mphr,
	        					Ssize*Ssize,
	        					&ctp->SgIISRM[0],
	        					&ctp->SgIISRM[Ssize*Ssize],
	        					ctp->lat,
	        					ctp->lon,
	        					ctp->Ang_mesY,
	        					ctp->Ang_mesZ,
	        					ctp->Azim_sol,
	        					ctp->Elev_sol,
	        					ctp->EarthSatDist
	        					);
		
	    SD_BAS_LibererOperateur();
    #ifdef OPS_DEBUG
    if(irc != CMN__STATUT_OK)
    {
        printf("pb  erreur SD_BAS_MetopGeod => Code retour %d\n",irc);
    }
    #endif

	    if(irc != CMN__STATUT_OK)
	    {
	       	cr_op   = SD_ALG_AVHRR_44_GEO_ALRT;
	       	msg_err = (char *)SD_BAS_MSG_ERR_METOP2GEOD;
	       	num_lig = __LINE__;
	       	goto fin;
	    }		
	    // ...FT-2997
	/* CR242 */			
    }
    //@ SI flag_geo_avhrr_miss == VRAI ALORS
    else
    {
    	for(il=1,ct=0; il<=Ssize; il++)
      	{
        	for(ic=1; ic<=Ssize; ic++,ct++)
        	{
          	ctp->LocIasiAvhrrLin[ct] = 0.0;
          	ctp->LocIasiAvhrrCol[ct] = 0.0;
        	} /* fin boucle sur ic */
      	} /* fin boucle sur il */
      	SD_BAS_ReserverOperateur();
        #ifdef OPS_DEBUG
        printf("pb call SD_BAS_MetopGeod from ALG_AVHRR line %d\n",__LINE__);
        printf("pb   nbpoints %d\n",Ssize*Ssize);
        #endif

      	irc = SD_BAS_MetopGeod(
        						ctx,
        						&IasiTime,
        						Mphr,
        						Ssize*Ssize,
        						&ctp->SgIISRM[0],
        						&ctp->SgIISRM[Ssize*Ssize],
        						ctp->lat,
        						ctp->lon,
        						ctp->Ang_mesY,
        						ctp->Ang_mesZ,
        						ctp->Azim_sol,
        						ctp->Elev_sol,
        						ctp->EarthSatDist
        					  );
	
      	SD_BAS_LibererOperateur();
    #ifdef OPS_DEBUG
    if(irc != CMN__STATUT_OK)
    {
        printf("pb  erreur SD_BAS_MetopGeod => Code retour %d\n",irc);
    }
    #endif

      	if(irc != CMN__STATUT_OK)
      	{
        	cr_op   = SD_ALG_AVHRR_44_GEO_ALRT;
        	msg_err = (char *)SD_BAS_MSG_ERR_METOP2GEOD;
        	num_lig = __LINE__;
        	goto fin;
      	}		
	}
    /* fin CR242 */
    
    #ifdef OPS_DEBUG
        for(il=1,ct=0; il<=Ssize; il++)
        {
            for(ic=1; ic<=Ssize; ic++,ct++)
            {
                printf("\nPixel (ic,il) = (%d,%d) : Coordonnees dans le raster AVHRR\n",ic,il);
                printf("   IISCornerAvhrrCol = %lf\n",ctp->IISCornerAvhrrCol[ct]);
                printf("   IISCornerAvhrrLin = %lf\n",ctp->IISCornerAvhrrLin[ct]);
                printf("Coordonnees geographiques et angles de mesure\n",i);
                printf("   lat = %lf\n",ctp->lat[ct]);
                printf("   lon = %lf\n",ctp->lon[ct]);
                printf("   Ang_mesY = %lf\n",ctp->Ang_mesY[ct]);
                printf("   Ang_mesZ = %lf\n",ctp->Ang_mesZ[ct]);
                printf("   Azim_sol = %lf\n",ctp->Azim_sol[ct]);
                printf("   Elev_sol = %lf\n",ctp->Elev_sol[ct]);
                printf("   EarthSatDist = %lf\n",ctp->EarthSatDist[ct]);
            }
        }
    #endif


    // Recopie des sorties dans la listvar InfoDet (avec conversion en

	// conversion en degre  et correction de l'ordre des angles
	//FA153 on prend la visee centale pour le point sous sat
    if(SubsatDebFait == FALSE && SN==15)
    {
    	SubsatDebFait = TRUE;
        StatMphr->subsat_latitude_start =  ctp->lat[(Ssize*Ssize)/2]*180/M_PI;
        StatMphr->subsat_longitude_start = ctp->lon[(Ssize*Ssize)/2]*180/M_PI;
    }


	//FA153 on prend la visee centale pour le point sous sat
    if(SN==15)
    {
    	// On met a jour systematiquement les informations de fin avec la meme remarque que precedemment
    	StatMphr->subsat_latitude_end =  ctp->lat[(Ssize*Ssize)/2]*180/M_PI;
    	StatMphr->subsat_longitude_end = ctp->lon[(Ssize*Ssize)/2]*180/M_PI;
    }

    // Infos relatives aux pixels de l'IIS
    for(i=1; i<=SD_BAS_PN; i++)
    {
        InfoDet1ABC->tab_var[N1C_GGEOSONDLOC].val.pr8[IND3(1,i,SN,2,SD_BAS_PN)] = lon[IND1(i)]*180./M_PI;
        InfoDet1ABC->tab_var[N1C_GGEOSONDLOC].val.pr8[IND3(2,i,SN,2,SD_BAS_PN)] = lat[IND1(i)]*180./M_PI;
        InfoDet1ABC->tab_var[N1C_GGEOSONDANGLESSUN].val.pr8[IND3(1,i,SN,2,SD_BAS_PN)] = 90.- Elev_sol[IND1(i)]*180./M_PI;
		//conversion entre 0 et 360 
        InfoDet1ABC->tab_var[N1C_GGEOSONDANGLESSUN].val.pr8[IND3(2,i,SN,2,SD_BAS_PN)] = Azim_sol[IND1(i)]*180./M_PI;
        InfoDet1ABC->tab_var[N1C_GGEOSONDANGLESMETOP].val.pr8[IND3(1,i,SN,2,SD_BAS_PN)] = 90.- Ang_mesZ[IND1(i)]*180./M_PI;
		//conversion entre 0 et 360 
        InfoDet1ABC->tab_var[N1C_GGEOSONDANGLESMETOP].val.pr8[IND3(2,i,SN,2,SD_BAS_PN)] = Ang_mesY[IND1(i)]*180./M_PI;

        //@ Conversion de LocIasiAvhrrLin[IND1(i)] en millisecondes
        u = (int)LocIasiAvhrrLin[IND1(i)];      // numero de ligne
        x = LocIasiAvhrrLin[IND1(i)] - u;       // partie fractionnaire
	    // protection pour la sortie du bandeau
	    if ( u > 0 && u <nbligbandeau )
	    {
        	Tdeb = (int)(BandeauAVHRR[u]->InfoGene.tab_var[AVHRR_DATEDEB].val.pdate)->day * CMN__DATE_SEC_PAR_JOUR * 1000 
        			+ (BandeauAVHRR[u]->InfoGene.tab_var[AVHRR_DATEDEB].val.pdate)->millis;
            Tfin = (int)(BandeauAVHRR[u]->InfoGene.tab_var[AVHRR_DATEFIN].val.pdate)->day * CMN__DATE_SEC_PAR_JOUR * 1000 
            		+ (BandeauAVHRR[u]->InfoGene.tab_var[AVHRR_DATEFIN].val.pdate)->millis;
            //@ cf PFS 2.5
            T0 = (int)(IasiTime0.day * CMN__DATE_SEC_PAR_JOUR * 1000 + IasiTime0.millis);
            PosLinMillis = (int)(Tdeb + x * (Tfin - Tdeb) - T0);

            InfoDet1ABC->tab_var[N1C_GEPSLOCIASIAVHRR_IASI].val.pr8[IND3(1,i,SN,2,SD_BAS_PN)] = PosLinMillis;
  		}
        InfoDet1ABC->tab_var[N1C_GEPSLOCIASIAVHRR_IASI].val.pr8[IND3(2,i,SN,2,SD_BAS_PN)] = LocIasiAvhrrCol[IND1(i)];
    }
    *(InfoDet1ABC->tab_var[N1AB_EARTH_SATELLITE_DISTANCE].val.pui4) = (unsigned int)ctp->EarthSatDist[Ssize * Ssize / 2];


	// Infos relatives aux pixels de la sousgrille
    for(ic = 0, ct = 1 ; ic < Ssize ; ic++)
    {
        for(il = 0 ; il < Ssize ; il++, ct++)
        {
            InfoDet1ABC->tab_var[N1C_GGEOIISLOC].val.pr8[IND3(1,ct,SN,2,SD_BAS_SGI)] = ctp->lon[IND1(ct)] * 180. / M_PI;
            InfoDet1ABC->tab_var[N1C_GGEOIISLOC].val.pr8[IND3(2,ct,SN,2,SD_BAS_SGI)] = ctp->lat[IND1(ct)] * 180. / M_PI;
            InfoDet1ABC->tab_var[N1C_GGEOIISANGLESSUN].val.pr8[IND3(1,ct,SN,2,SD_BAS_SGI)] = 90. - ctp->Elev_sol[IND1(ct)] * 180. / M_PI;
			//conversion entre 0 et 360 
            InfoDet1ABC->tab_var[N1C_GGEOIISANGLESSUN].val.pr8[IND3(2,ct,SN,2,SD_BAS_SGI)] = ctp->Azim_sol[IND1(ct)] * 180. / M_PI;
            InfoDet1ABC->tab_var[N1C_GGEOIISANGLESMETOP].val.pr8[IND3(1,ct,SN,2,SD_BAS_SGI)] = 90.- ctp->Ang_mesZ[IND1(ct)] * 180 / M_PI;
			//conversion entre 0 et 360 
            InfoDet1ABC->tab_var[N1C_GGEOIISANGLESMETOP].val.pr8[IND3(2,ct,SN,2,SD_BAS_SGI)] = ctp->Ang_mesY[IND1(ct)] * 180. / M_PI;

            //@ Conversion de ctp->LocIasiAvhrrLin[IND1(ct)] en millisecondes
            u = (int)ctp->LocIasiAvhrrLin[IND1(ct)];      // numero de ligne
            x = ctp->LocIasiAvhrrLin[IND1(ct)] - u;       // partie fractionnaire
	     	// protection pour la sortie du bandeau
	    	if ( u > 0 && u < nbligbandeau )
	      	{
            	Tdeb = (int)(BandeauAVHRR[u]->InfoGene.tab_var[AVHRR_DATEDEB].val.pdate)->day * CMN__DATE_SEC_PAR_JOUR * 1000 
            				+ (BandeauAVHRR[u]->InfoGene.tab_var[AVHRR_DATEDEB].val.pdate)->millis;
              	Tfin = (int)(BandeauAVHRR[u]->InfoGene.tab_var[AVHRR_DATEFIN].val.pdate)->day * CMN__DATE_SEC_PAR_JOUR * 1000 
              				+ (BandeauAVHRR[u]->InfoGene.tab_var[AVHRR_DATEFIN].val.pdate)->millis;
            	//@ cf PFS 2.5
            	T0 = (int)(IasiTime0.day * CMN__DATE_SEC_PAR_JOUR * 1000 + IasiTime0.millis);
           		PosLinMillis = (int)(Tdeb + x*(Tfin-Tdeb) - T0);

              	InfoDet1ABC->tab_var[N1C_GEPSLOCIASIAVHRR_IIS].val.pr8[IND3(1,ct,SN,2,SD_BAS_SGI)] = PosLinMillis;
	      	}
            InfoDet1ABC->tab_var[N1C_GEPSLOCIASIAVHRR_IIS].val.pr8[IND3(2,ct,SN,2,SD_BAS_SGI)] = ctp->LocIasiAvhrrCol[IND1(ct)];
		}
	}

	// On trace ici toutes les variables en sortie
	#ifdef INVESTIGATION
	    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"EarthSatelliteDistance",1,1,&ctp->EarthSatDist[Ssize*Ssize/2],ctx);
	    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"subsat_latitude_start",1,1,&StatMphr->subsat_latitude_start,ctx);
	    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"subsat_longitude_start",1,1,&StatMphr->subsat_longitude_start,ctx);
	    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"subsat_latitude_end",1,1,&StatMphr->subsat_latitude_end,ctx);
	    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"subsat_longitude_end",1,1,&StatMphr->subsat_longitude_end,ctx);
	    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"GpixLon",4,1,lon,ctx);
	    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"GpixLat",4,1,lat,ctx);
	    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"SpixAz",4,1,Azim_sol,ctx);
	    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"SpixEl",4,1,Elev_sol,ctx);
	    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"MpixAz",4,1,Ang_mesY,ctx);
	    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"MpixEl",4,1,Ang_mesZ,ctx);
	    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"IISCornerAvhrrLin",4,1,IISCornerAvhrrLin,ctx);
	    SD_BAS_TRACEREEL1D(tcProcName_F,&ctx->ctxtrt,"IISCornerAvhrrCol",4,1,IISCornerAvhrrCol,ctx);
	
	    SD_BAS_TRACEREEL2D(tcProcName_F,&ctx->ctxtrt,"GIISLon",Ssize,Ssize,ctp->lon,ctx);
	    SD_BAS_TRACEREEL2D(tcProcName_F,&ctx->ctxtrt,"GIISLat",Ssize,Ssize,ctp->lat,ctx);
	    SD_BAS_TRACEREEL2D(tcProcName_F,&ctx->ctxtrt,"SIISAz",Ssize,Ssize,ctp->Azim_sol,ctx);
	    SD_BAS_TRACEREEL2D(tcProcName_F,&ctx->ctxtrt,"SIISEl",Ssize,Ssize,ctp->Elev_sol,ctx);
	    SD_BAS_TRACEREEL2D(tcProcName_F,&ctx->ctxtrt,"MIISAz",Ssize,Ssize,ctp->Ang_mesY,ctx);
	    SD_BAS_TRACEREEL2D(tcProcName_F,&ctx->ctxtrt,"MIISEl",Ssize,Ssize,ctp->Ang_mesZ,ctx);
	    SD_BAS_TRACEREEL2D(tcProcName_F,&ctx->ctxtrt,"IISLocIasiAvhrr",Ssize,Ssize,ctp->LocIasiAvhrrLin,ctx);
	    SD_BAS_TRACEREEL2D(tcProcName_F,&ctx->ctxtrt,"IISLocIasiAvhrr",Ssize,Ssize,ctp->LocIasiAvhrrCol,ctx);
	#endif


fin:
    // Traitement des erreurs
    //================================================================
    if ( cr_op != CMN__STATUT_OK )
    {
        // mise a jour de la variable ctx->cr.errinterne
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
        					SD_BAS_ERR_SET_AVHRR,
        					cr_op,
        					tcProcName_F,
        					num_lig,
        					msg_err);

      	CreerMessage(SD_ALG_AVHRR_SET, 
        			 MSG_ALG_AVHRR_44_GEO_ALRT, 
        			 &(ctx->cr.msgJDB));

        // on imprime le message d'erreur sur la sortie standard
        #ifdef OPS_DEBUG
            SD_BAS_PRINT_ERR_INT(ctx->cr.errinterne.nomfonction,
                                 ctx->cr.errinterne.numligne,
                                 ctx->cr.errinterne.libelle);
        #endif
    }

	// sortie de la fonction
	#ifdef OPS_DEBUG
    	printf("Fin de la fonction 44 GEO\n========================================\n");
	#endif
	return(cr_op);
}


//======================================================================
//                  FONCTIONS INTERNES
//======================================================================


//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_GeomIasiMetop
//@
//@ objet :  Passage de coordonnees du repere nominal instrument (IASI)
//@ vers le repere METOP
//@
//@
//@ Parametres :
//@         (ENTREE)  MatRot : Matrice de rotation du repere IASI/Metop
//@         (ENTREE)  ScanAngle : Angle de visee IASI
//@         (ENTREE)  CornerYi : Coordonnee Y dans le repere IASI
//@         (ENTREE)  CornerZi : Coordonnee Z dans le repere IASI
//@         (SORTIE)  azim : Azim dans le repere IASI (RADIAN))
//@         (SORTIE)  elev : elev dans le repere IASI  (RADIAN)
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_GeomIasiMetop
(
  // Entrees
  double *MatRot,     // double( SD_BAS_DIM_ROT_MATRICE * SD_BAS_DIM_ROT_MATRICE)
  double  ScanAngle,  // double(1)
  double  PosY,       // double(1)
  double  PosZ,       // double(1)
  // Sorties
  double *azim,       // double(1)
  double *elev        // double(1)
)
{
  t_CMN__Statut cr_op=CMN__STATUT_OK;

  double  VectPos[SD_BAS_DIM_ROT_MATRICE];
  double  MatRotLOS[SD_BAS_DIM_ROT_MATRICE*SD_BAS_DIM_ROT_MATRICE];  //Matrice de rotation liee a l'angle de visee Iasi
  double  VectPosRI[SD_BAS_DIM_ROT_MATRICE];
  double  norm;
  double  Y, Z;
/* bma */
  double  VectPosRM[SD_BAS_DIM_ROT_MATRICE];
/* fin bma */

  //@ Vecteur position du point dans le repere Iasi (sans rotation)
  // Y = sin(PosY);
  // Z = sin(PosZ);
  // VectPos(1) = -Z;
  // VectPos(2) = -Y;
  // VectPos(3) = -sqrt( 1 - Y*Y - Z*Z);
  Y= sin( PosY);
  Z= sin( PosZ);
  VectPos[IND1(1)]= -Z;
  VectPos[IND1(2)]= -Y;
  VectPos[IND1(3)]= -sqrt( 1 - Y*Y - Z*Z);

  //@ Normalisation de VectPos
  // VectPosN = Normalisation{ VectPos }
  norm= sqrt(   VectPos[IND1(1)] * VectPos[IND1(1)]
              + VectPos[IND1(2)] * VectPos[IND1(2)]
              + VectPos[IND1(3)] * VectPos[IND1(3)]);

  VectPos[IND1(1)]= VectPos[IND1(1)] / norm;
  VectPos[IND1(2)]= VectPos[IND1(2)] / norm;
  VectPos[IND1(3)]= VectPos[IND1(3)] / norm;


  //@ Matrice de rotation liee a l?angle de visee Iasi
  // MatRotLOS = Matrice de rotation{ ScanAngle }
// on inverse le sens de l'angle ScanAngle
  SD_BAS_MatriceRotation( -ScanAngle, MatRotLOS);

  //@ Vecteur position dans le repere IASI
  // VectPosRI = MatRotLOS * VectPosN
  VectPosRI[IND1(1)]=   MatRotLOS[IND2(1,1,SD_BAS_DIM_ROT_MATRICE)] * VectPos[IND1(1)]
                      + MatRotLOS[IND2(2,1,SD_BAS_DIM_ROT_MATRICE)] * VectPos[IND1(2)]
                      + MatRotLOS[IND2(3,1,SD_BAS_DIM_ROT_MATRICE)] * VectPos[IND1(3)];

  VectPosRI[IND1(2)]=   MatRotLOS[IND2(1,2,SD_BAS_DIM_ROT_MATRICE)] * VectPos[IND1(1)]
                      + MatRotLOS[IND2(2,2,SD_BAS_DIM_ROT_MATRICE)] * VectPos[IND1(2)]
                      + MatRotLOS[IND2(3,2,SD_BAS_DIM_ROT_MATRICE)] * VectPos[IND1(3)];

  VectPosRI[IND1(3)]=   MatRotLOS[IND2(1,3,SD_BAS_DIM_ROT_MATRICE)] * VectPos[IND1(1)]
                      + MatRotLOS[IND2(2,3,SD_BAS_DIM_ROT_MATRICE)] * VectPos[IND1(2)]
                      + MatRotLOS[IND2(3,3,SD_BAS_DIM_ROT_MATRICE)] * VectPos[IND1(3)];

  //@ Vecteur position dans le repere METOP
  // Utilisation de la sous-matrice | A B C | => | A B |
  // 2 x 2 relative a X et Y dans   | D E F |    | D E |
  // la matrice 3 x 3 (MatRot)      | G H K |
  // VectPosRM = MatRot * VectPosRI
//  *azim =   MatRot[IND2(1,1,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(1)]
//          + MatRot[IND2(2,1,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(2)];
//          + MatRot[IND2(3,1,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(3)]; Terme nul

//  *elev =   MatRot[IND2(1,2,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(1)]
//          + MatRot[IND2(2,2,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(2)];
//          + MatRot[IND2(3,2,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(3)]; Terme nul
  VectPosRM[IND1(1)] = 
    MatRot[IND2(1,1,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(1)] +
    MatRot[IND2(2,1,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(2)] +
    MatRot[IND2(3,1,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(3)];
  
  VectPosRM[IND1(2)] = 
    MatRot[IND2(1,2,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(1)] +
    MatRot[IND2(2,2,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(2)] +
    MatRot[IND2(3,2,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(3)];
  
  VectPosRM[IND1(3)] = 
    MatRot[IND2(1,3,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(1)] +
    MatRot[IND2(2,3,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(2)] +
    MatRot[IND2(3,3,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(3)];
  
  /* calcul de l'azimut dans [0;2Pi] */
  /* calcul de l'elevation dans [-Pi/2;Pi/2] */
  *azim = atan2(VectPosRM[IND1(2)], VectPosRM[IND1(1)]) ;
  //l'axe reference est la vitesse satellite
  *azim = 3.*M_PI/2.-*azim ;
  if ( *azim< 0.) *azim=*azim+2*M_PI;
  if ( *azim> 2*M_PI) *azim=*azim-2*M_PI;
  *elev = atan ( 
	 -VectPosRM[IND1(3)]/
    sqrt((VectPosRM[IND1(1)] * VectPosRM[IND1(1)]) + 
         (VectPosRM[IND1(2)] * VectPosRM[IND1(2)])) );

fin:
  // sortie de la fonction
  return( cr_op);
}


//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_GeomMetopIasi
//@
//@ objet :  Passage de coordonnees du repere Metop vers le repere
//@ nominal instrument (IASI)
//@
//@
//@ Parametres :
//@         (ENTREE)  MatRot : Matrice de rotation du repere IASI/Metop
//@         (ENTREE)  ScanAngle : Angle de visee IASI (RADIAN)
//@         (ENTREE)  VectPos : Cosinus directeurs dans le repere Metop (RADIAN)
//@         (SORTIE)  PosY : Coordonnee Y dans le repere IASI
//@         (SORTIE)  PosZ : Coordonnee Z dans le repere IASI
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_GeomMetopIasi
(
  // Entrees
  double *MatRot,      // double( SD_BAS_DIM_ROT_MATRICE * SD_BAS_DIM_ROT_MATRICE)
  double  ScanAngle,   // double(1)
  double *VectPos,     // double(2) 0:Azim ; 1:Elev
  // Sorties
  double *PosY,        // double(1)
  double *PosZ         // double(1)
)
{
  t_CMN__Statut cr_op=CMN__STATUT_OK;

  // Gestion de erreurs
/* bma intialisation */
/*
  char *msg_err;          // Message d'erreur
  int num_lig;            // Ligne d'erreur
*/
  char *msg_err = (char*) NULL;          // Message d'erreur
  int num_lig = 0;            // Ligne d'erreur
/* fin bma */

  double  VectPosRM[SD_BAS_DIM_ROT_MATRICE];
  double  VectPosRI[SD_BAS_DIM_ROT_MATRICE];
  double  VectPosI[SD_BAS_DIM_ROT_MATRICE];

  double  MatRotLOS[SD_BAS_DIM_ROT_MATRICE*SD_BAS_DIM_ROT_MATRICE];
  double  norm;
  double x,y,z,xy;

  #ifdef OPS_DEBUG
    printf("\nEntree dans la fonction SD_ALG_GeomMetopIasi\n-----------------------------------\n");
  #endif

   // conversion Angle -> cosinus directeur
  /*  l'azimut est dans [0;2Pi] */
  /* l'elevation est  dans [-Pi/2;Pi/2] */
  VectPos[IND1(1)] = 3.*M_PI/2.-VectPos[IND1(1)] ;  
  if ( VectPos[IND1(1)]< 0.) VectPos[IND1(1)]=VectPos[IND1(1)]+2*M_PI;
  if ( VectPos[IND1(1)]> 2*M_PI) VectPos[IND1(1)]=VectPos[IND1(1)]-2*M_PI;
   // conversion Angle -> cosinus directeur
   z  = -sin(VectPos[IND1(2)]);
   xy = cos(VectPos[IND1(2)]);
//jlp31
   x  = xy * cos(VectPos[IND1(1)]);
   y  = xy * sin(VectPos[IND1(1)]);
  //@ Normalisation
  // Utilisation de la sous-matrice | A B C | => | A B |
  // 2 x 2 relative a X et Y dans   | D E F |    | D E |
  // la matrice 3 x 3 (MatRot)      | G H K |
  // VectPosRM = Normalisation{ VectPos }
  //norm= 1 - (VectPos[IND1(1)]*VectPos[IND1(1)]) - (VectPos[IND1(2)]*VectPos[IND1(2)]);

  //if( norm < 0.0)
  //{
  //  cr_op=   SD_ALG_AVHRR_ERR_METOP2IASI;
  //  msg_err= (char *)SD_BAS_MSG_ERR_METOP2IASI;
  //  num_lig= __LINE__;
  //}

  //norm= sqrt( norm);

  //VectPosRM[IND1(1)]= VectPos[IND1(1)];
  //VectPosRM[IND1(2)]= VectPos[IND1(2)];
  //VectPosRM[IND1(3)]= norm;

  //@ Vecteur position dans le repere IASI
  // VectPosRI = Transpose{ MatRot }* VectPosRM
  VectPosRI[IND1(1)]=   MatRot[IND2(1,1,SD_BAS_DIM_ROT_MATRICE)] * x
                      + MatRot[IND2(1,2,SD_BAS_DIM_ROT_MATRICE)] * y
                      + MatRot[IND2(1,3,SD_BAS_DIM_ROT_MATRICE)] * z;

  VectPosRI[IND1(2)]=   MatRot[IND2(2,1,SD_BAS_DIM_ROT_MATRICE)] * x
                      + MatRot[IND2(2,2,SD_BAS_DIM_ROT_MATRICE)] * y
                      + MatRot[IND2(2,3,SD_BAS_DIM_ROT_MATRICE)] * z;

  VectPosRI[IND1(3)]=   MatRot[IND2(3,1,SD_BAS_DIM_ROT_MATRICE)] * x
                      + MatRot[IND2(3,2,SD_BAS_DIM_ROT_MATRICE)] * y
                      + MatRot[IND2(3,3,SD_BAS_DIM_ROT_MATRICE)] * z;

  //@ Matrice de rotation liee a l'angle de visee IASI (sens oppose)
  //@   MatRotLOS = Matrice de rotation{ - ScanAngle }
  SD_BAS_MatriceRotation( -(ScanAngle), MatRotLOS);

  //@ Vecteur position dans le repere IASI sans rotation
  //@   VectPos = MatRotLOS * VectPosRI
  VectPosI[IND1(1)]=   MatRotLOS[IND2(1,1,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(1)]
                     + MatRotLOS[IND2(1,2,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(2)]
                     + MatRotLOS[IND2(1,3,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(3)];

  VectPosI[IND1(2)]=   MatRotLOS[IND2(2,1,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(1)]
                     + MatRotLOS[IND2(2,2,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(2)]
                     + MatRotLOS[IND2(2,3,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(3)];

  VectPosI[IND1(3)]=   MatRotLOS[IND2(3,1,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(1)]
                     + MatRotLOS[IND2(3,2,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(2)]
                     + MatRotLOS[IND2(3,3,SD_BAS_DIM_ROT_MATRICE)] * VectPosRI[IND1(3)];

  //@ Coordonnees angulaires dans le repere IASI selon les axes Y et Z
  //@   PosY = -Asin{ VectPos(2) }
  //@   PosZ = -Asin{ VectPos(1) }
  *PosY= -asin( VectPosI[IND1(2)]);
  *PosZ= -asin( VectPosI[IND1(1)]);


  #ifdef OPS_DEBUG
    printf( "MatRot .... : %lf\t%lf\t%lf\n", MatRot[IND2(1,1,SD_BAS_DIM_ROT_MATRICE)],
                                             MatRot[IND2(2,1,SD_BAS_DIM_ROT_MATRICE)],
                                             MatRot[IND2(3,1,SD_BAS_DIM_ROT_MATRICE)]);
    printf( "              %lf\t%lf\t%lf\n", MatRot[IND2(1,2,SD_BAS_DIM_ROT_MATRICE)],
                                             MatRot[IND2(2,2,SD_BAS_DIM_ROT_MATRICE)],
                                             MatRot[IND2(3,2,SD_BAS_DIM_ROT_MATRICE)]);
    printf( "              %lf\t%lf\t%lf\n", MatRot[IND2(1,3,SD_BAS_DIM_ROT_MATRICE)],
                                             MatRot[IND2(2,3,SD_BAS_DIM_ROT_MATRICE)],
                                             MatRot[IND2(3,3,SD_BAS_DIM_ROT_MATRICE)]);
    printf( "ScanAngle . : %lf\n",           ScanAngle);
    printf( "MatRotLOS . : %lf\t%lf\t%lf\n", MatRotLOS[IND2(1,1,SD_BAS_DIM_ROT_MATRICE)],
                                             MatRotLOS[IND2(2,1,SD_BAS_DIM_ROT_MATRICE)],
                                             MatRotLOS[IND2(3,1,SD_BAS_DIM_ROT_MATRICE)]);
    printf( "              %lf\t%lf\t%lf\n", MatRotLOS[IND2(1,2,SD_BAS_DIM_ROT_MATRICE)],
                                             MatRotLOS[IND2(2,2,SD_BAS_DIM_ROT_MATRICE)],
                                             MatRotLOS[IND2(3,2,SD_BAS_DIM_ROT_MATRICE)]);
    printf( "              %lf\t%lf\t%lf\n", MatRotLOS[IND2(1,3,SD_BAS_DIM_ROT_MATRICE)],
                                             MatRotLOS[IND2(2,3,SD_BAS_DIM_ROT_MATRICE)],
                                             MatRotLOS[IND2(3,3,SD_BAS_DIM_ROT_MATRICE)]);
    printf( "VectPosRM . : %lf\t%lf\t%lf\n", VectPosRM[IND1(1)], VectPosRM[IND1(2)], VectPosRM[IND1(3)]);
    printf( "VectPosRI . : %lf\t%lf\t%lf\n", VectPosRI[IND1(1)], VectPosRI[IND1(2)], VectPosRI[IND1(3)]);
    printf( "VectPosI .. : %lf\t%lf\t%lf\n", VectPosI[IND1(1)],  VectPosI[IND1(2)],  VectPosI[IND1(3)]);
    printf( "(PosY,PosZ) : (%lf,%lf)\n",    *PosY, *PosZ);
    printf( "\nFin de la fonction SD_ALG_GeomMetopIasi\n-----------------------------------");
  #endif

  // on trace ici si besoin toutes les variables en sortie

fin:
  // sortie de la fonction
  return( cr_op);
}


//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_ImageUtile
//@
//@ objet :  Extraction de l'imagette Avhrr utile (domaine de l'IIS)
//@
//@
//@ Parametres :
//@         (ENTREE)  Contexte_courant .... : Contexte de l'etape
//@         (ENTREE)  MphrN1 .............. : MPHR produit N1
//@         (ENTREE)  IISNbLin ............ : Nombre de pixels IIS en lignes
//@         (ENTREE)  IISNbCol ............ : Nombre de pixels IIS en colonnes
//@         (ENTREE)  IISY ................ : Position des centres des pixels IIS dans repere IIS selon Y
//@         (ENTREE)  IISZ ................ : Position des centres des pixels IIS dans repere IIS selon Z
//@         (ENTREE)  IISDY ............... : Dimension des pixels IIS selon Y
//@         (ENTREE)  IISDZ ............... : Dimension des pixels IIS selon Z
//@         (ENTREE)  IasiMetopMatRot ..... : Matrice de rotation du repere IASI/Metop (2x2)
//@         (ENTREE)  IISRot .............. : Angle de rotation repere IASI/InstrumentRI
//@         (ENTREE)  IasiScanAngle ....... : Angle de visee IASI
//@         (ENTREE)  IasiTime ............ : Date de la visee courante
//@         (ENTREE)  OffsetIISAvhrr ...... : Decalage IIS/AVHRR dans repere AVHRR
//@         (ENTREE)  AvhrrListeLin ....... : Image AVHRR (Liste de lignes)
//@         (ENTREE)  AvhrrNbLin .......... : Nombre de lignes de l'image AVHRR
//@         (ENTREE)  AvhrrLinDeb ......... : Ligne de debut
//@         (ENTREE)  AvhrrLinFin ......... : Ligne de fin
//@         (ENTREE)  AvhrrColDeb ......... : Colonne de debut
//@         (ENTREE)  AvhrrColFin ......... : Colonne de fin
//@         (ENTREE)  AvhrrNbChannel ...... : Nombre de canaux AVHRR a traiter
//@         (ENTREE)  AvhrrChanMax ........ : Nombre maximal de canaux AVHRR (SD_BAS_NBK)
//@         (ENTREE)  AvhrrChannelState ... : Vecteur d'etat des canaux AVHRR (1 vecteur de 6 canaux par ligne)
//@
//@         (SORTIE)  AvhrrUtilImage ...... : Image AVHRR utile
//@         (SORTIE)  AvhrrUtilLinFirst ... : 1ere ligne AVHRR utile
//@         (SORTIE)  AvhrrUtilColFirst ... : 1ere colonne AVHRR utile
//@         (SORTIE)  AvhrrUtilNbLin ...... : Nombre de lignes AVHRR utiles
//@         (SORTIE)  AvhrrUtilNbCol ...... : Nombre de colonnes AVHRR utiles
//@         (SORTIE)  AvhrrUtilChannelState : Flags de disponibilite des canaux (1 par canal)
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_ImageUtile(
  // Entrees
  t_SD_BAS_DataThread  *ctx,      // t_SD_BAS_DataThread*
  t_SD_BAS_MPHR        *MphrN1,                // t_SD_BAS_MPHR*
  int                   IISNbLin,              // int(1)
  int                   IISNbCol,              // int(1)
  double               *IISY,                  // double( SD_BAS_IMLI)
  double               *IISZ,                  // double( SD_BAS_IMCO)
  double                IISDY,                 // double(1)
  double                IISDZ,                 // double(1)
  double               *IasiMetopMatRot,       // double( SD_BAS_DIM_ROT_MATRICE * SD_BAS_DIM_ROT_MATRICE)
  double                IISRot,                // double(1)
  double                IasiScanAngle,         // double(1)
  t_SD_BAS_SCDSTime     IasiTime,              // t_SD_BAS_SCDSTime*
  double               *OffsetIISAvhrr,        // double( 2 * SD_BAS_SNOT)
  t_SD_BAS_LigneAVHRR **AvhrrListeLin,         // t_SD_BAS_LigneAVHRR * ( AvhrrNbLin)
  int                   AvhrrNbLin,            // int(1)
  int                   AvhrrLinDeb,           // int(1)
  int                   AvhrrLinFin,           // int(1)
  int                   AvhrrColDeb,           // int(1)
  int                   AvhrrColFin,           // int(1)
  int                   AvhrrNbChannel,        // int(1)
  int                   AvhrrChanMax,          // int(1)
  unsigned int         *AvhrrChannelState,     // unsigned int( SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NBK) 
  // Sorties
  float                *AvhrrUtilImage,        // float( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NBK)
                                               //          AvhrrUtilNbCol,         AvhrrUtilNbLin,    AvhrrChanMax
  int                  *AvhrrUtilLinDeb,       // int(1)
  int                  *AvhrrUtilColDeb,       // int(1)
  int                  *AvhrrUtilNbLin,        // int(1)
  int                  *AvhrrUtilNbCol,        // int(1)
  // FT-2528
  unsigned int         *AvhrrUtilChannelState, // unsigned int( SD_BAS_NBK)
  unsigned int         *QualityIndicator,      // unsigned int(SD_BAS_NLI_MAX_UTIL_AVHRR)
  unsigned int         *ScanLineQuality,       // unsigned int(SD_BAS_NLI_MAX_UTIL_AVHRR)
  unsigned short int   *CountErrorFrame,       // unsigned short int(SD_BAS_NLI_MAX_UTIL_AVHRR)
  // FT-2765
  unsigned short int   *CloudInformation       // unsigned short int(SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NPI_AVHRR
  // ...FT-2765
  // ...FT-2528

)
{
  static const char tcProcName_F[] = "SD_ALG_ImageUtile";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  double IISCornerY[4];
  double IISCornerZ[4];
  double IISCornerMetop[2*4];        // Les 4 azims suivis des 4 elev
  double minval, maxval;
  int    i, y, z;
  int    SN;                         // Numero de sous-cycle (StepNumber)
  // Sorties de SD_BAS_MetopAVHRR
  double latitude[4], longitude[4];
  double Ang_mesY[4], Ang_mesZ[4];
  double Azim_sol[4], Elev_sol[4];
  double EarthSatDist[4];            // Distance Terre/Satellite dans la conversion Metop2AVHRR
  double IISCornerAvhrrLin[4];       // Lignes AVHRR dans la conversion Metop2AVHRR
  double IISCornerAvhrrCol[4];       // Colonnes AVHRR dans la conversion Metop2AVHRR
  double CornerY[4];
  double CornerZ[4];
  int    AvhrrUtilLinFin;       // Ligne de fin de l'image AVHRR utile
  int    AvhrrUtilColFin;       // Colonne de fin de l'image AVHRR utile
  int    Nch;                   // index sur les canaux
  int    Il, Jl, Ic, Jc, k;
  unsigned int Sum[SD_BAS_NBK];

  // Gestion de erreurs
/* bma intialisation */


  char *msg_err = (char*) NULL;          // Message d'erreur
  int   num_lig = 0;          // Ligne d'erreur
/* fin bma */

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction ImageUtile\n----------------------------------\n");
  #endif

  SN=  ctx->ctxtrt.SN;

  //-------------------------------------------------------------------------
  // Calcul des coordonnees des 4 coins de l'imagette IIS dans le repere IIS
  //-------------------------------------------------------------------------
  minval= MAX_DOUBLE;
  maxval= MIN_DOUBLE;
  for( i=1; i <= IISNbCol; i++)
  {
    minval= MIN( IISY[IND1(i)], minval);
    maxval= MAX( IISY[IND1(i)], maxval);
  }
  IISCornerY[IND1(1)]= IISCornerY[IND1(4)]= minval - IISDY/2;
  IISCornerY[IND1(2)]= IISCornerY[IND1(3)]= maxval + IISDY/2;

  minval= MAX_DOUBLE;
  maxval= MIN_DOUBLE;
  for( i=1; i <= IISNbLin; i++)
  {
    minval= MIN( IISZ[IND1(i)], minval);
    maxval= MAX( IISZ[IND1(i)], maxval);
  }
  IISCornerZ[IND1(1)]= IISCornerZ[IND1(2)]= minval - IISDZ/2;
  IISCornerZ[IND1(3)]= IISCornerZ[IND1(4)]= maxval + IISDZ/2;

  #ifdef OPS_DEBUG
    printf( "Calcul des coordonnees des 4 coins de l'imagette IIS dans le repere IIS\n");
    printf( "   Nb lignes IIS .... : %d\n", IISNbLin);
    printf( "   Nb colonnes IIS .. : %d\n", IISNbCol);
    printf( "   IISCornerY 1 2 3 4 : %lf\t%lf\t%lf\t%lf\n",
            IISCornerY[0], IISCornerY[1], IISCornerY[2], IISCornerY[3]);
    printf( "   IISCornerZ 1 2 3 4 : %lf\t%lf\t%lf\t%lf\n",
            IISCornerZ[0], IISCornerZ[1], IISCornerZ[2], IISCornerZ[3]);
  #endif
  //-------------------------------------------------------------------------


  //-------------------------------------------------------------------------
  // Boucle sur les 4 coins
  //-------------------------------------------------------------------------
  #ifdef OPS_DEBUG
    printf( "Calcul des coordonnees des 4 coins de l'imagette IIS dans les repere IASI et METOP\n");
    printf( "Angle   IISRot : %lf\n", IISRot);
    // Utilisation de la sous-matrice | A B C |
    // 2 x 2 relative a Y et Z dans   | D E F | => | E F |
    // la matrice 3 x 3 (MatRot)      | G H K |    | H K |
    printf( "MatRot .... : %lf\t%lf\t%lf\n", IasiMetopMatRot[IND2(1,1,SD_BAS_DIM_ROT_MATRICE)],
                                             IasiMetopMatRot[IND2(2,1,SD_BAS_DIM_ROT_MATRICE)],
                                             IasiMetopMatRot[IND2(3,1,SD_BAS_DIM_ROT_MATRICE)]);
    printf( "              %lf\t%lf\t%lf\n", IasiMetopMatRot[IND2(1,2,SD_BAS_DIM_ROT_MATRICE)],
                                             IasiMetopMatRot[IND2(2,2,SD_BAS_DIM_ROT_MATRICE)],
                                             IasiMetopMatRot[IND2(3,2,SD_BAS_DIM_ROT_MATRICE)]);
    printf( "              %lf\t%lf\t%lf\n", IasiMetopMatRot[IND2(1,3,SD_BAS_DIM_ROT_MATRICE)],
                                             IasiMetopMatRot[IND2(2,3,SD_BAS_DIM_ROT_MATRICE)],
                                             IasiMetopMatRot[IND2(3,3,SD_BAS_DIM_ROT_MATRICE)]);
    printf( "ScanAngle .... : %lf\n", IasiScanAngle);
  #endif

  for( i=1; i <= 4; i++)
  {
    // Calcul des coordonnees des 4 coins de l'imagette IIS dans le repere IASI i=1,4
    CornerY[IND1(i)]= IISCornerY[IND1(i)]*cos(IISRot) - IISCornerZ[IND1(i)]*sin(IISRot);
    CornerZ[IND1(i)]= IISCornerY[IND1(i)]*sin(IISRot) + IISCornerZ[IND1(i)]*cos(IISRot);

    #ifdef OPS_DEBUG
      printf( "Pixel no%d\n", i);
      printf( "   IASI CornerY : %lf\n", CornerY[IND1(i)]);
      printf( "   IASI CornerZ : %lf\n", CornerZ[IND1(i)]);
    #endif

    // Calcul des coordonnees des 4 coins dans le repere Metop
    cr_op= SD_ALG_GeomIasiMetop(
                                 // Entrees
                                  IasiMetopMatRot,    // double( SD_BAS_DIM_ROT_MATRICE * SD_BAS_DIM_ROT_MATRICE)
                                  IasiScanAngle,      // double(1)
                                  CornerY[IND1(i)],   // double(1)
                                  CornerZ[IND1(i)],   // double(1)
                                 // Sorties
                                 &IISCornerMetop[i-1],    // double(1) - Azim du pixel i
                                 &IISCornerMetop[i+3]);   // double(1) - Elev du pixel i
    #ifdef OPS_DEBUG
      printf( "   METOP Azim   : %lf\n", IISCornerMetop[i-1]);
      printf( "   METOP Elev   : %lf\n", IISCornerMetop[i+3]);
    #endif
  }
  //-------------------------------------------------------------------------

/*MOR VERRUE A ENLEVER*/
//goto SUITE;
  //-------------------------------------------------------------------------
  // Calcul des coordonnees des 4 coins dans le repere Avhrr
  //-------------------------------------------------------------------------
//  SD_BAS_ReserverOperateur();
  cr_op= SD_BAS_MetopAVHRR(
                            // Entree
                             ctx,          // t_SD_BAS_DataThread*
                            &IasiTime,                  // t_SD_BAS_SCDSTime*
                             MphrN1,                    // t_SD_BAS_MPHR*
                             4,                         // Pour les 4 coins
                             IISCornerMetop + 0,        // double( 4)
                             IISCornerMetop + 4,        // double( 4)
                             AvhrrLinDeb,               // int(1)
                             AvhrrLinFin,               // int(1)
                             AvhrrListeLin,             // t_SD_BAS_LigneAVHRR * ( AvhrrNbLin)
                             AvhrrNbLin,                // int(1)
                             AvhrrColDeb,               // int(1)
                             AvhrrColFin,               // int(1)
                            // Sortie
                             latitude,                  // double(1)
                             longitude,                 // double(1)
                             Ang_mesY,                  // double(1)
                             Ang_mesZ,                  // double(1)
                             Azim_sol,                  // double(1)
                             Elev_sol,                  // double(1)
                             IISCornerAvhrrCol,         // double(1)
                             IISCornerAvhrrLin,         // double(1)
                             EarthSatDist);             // double(1)

//       SD_BAS_LibererOperateur();
     if(cr_op != CMN__STATUT_OK)
     {
         cr_op   = SD_ALG_AVHRR_ERR_METOP2AVHRR;
         msg_err = (char *)SD_BAS_MSG_ERR_METOP2AVHRR;
         num_lig = __LINE__;
         goto fin;
     }

  #ifdef OPS_DEBUG
    cr_op= CMN__STATUT_OK;
//    IISCornerAvhrrLin[0]=  3.0; // Pour pouvoir continuer -> a virer des que ca marche
//    IISCornerAvhrrLin[1]= 10.0; // Pour pouvoir continuer -> a virer des que ca marche
//    IISCornerAvhrrLin[2]= 11.0; // Pour pouvoir continuer -> a virer des que ca marche
//    IISCornerAvhrrLin[3]=  4.5; // Pour pouvoir continuer -> a virer des que ca marche
//    IISCornerAvhrrCol[0]=  4.0; // Pour pouvoir continuer -> a virer des que ca marche
//    IISCornerAvhrrCol[1]=  5.0; // Pour pouvoir continuer -> a virer des que ca marche
//    IISCornerAvhrrCol[2]=  9.0; // Pour pouvoir continuer -> a virer des que ca marche
//    IISCornerAvhrrCol[3]=  9.5; // Pour pouvoir continuer -> a virer des que ca marche

    printf( "Calcul des coordonnees des 4 coins dans le repere AVHRR\n");
    printf( "   Pixel 1 : (ligAVHRR,colAVHRR) = (%lf,%lf)\n", IISCornerAvhrrLin[0], IISCornerAvhrrCol[0]);
    printf( "   Pixel 2 : (ligAVHRR,colAVHRR) = (%lf,%lf)\n", IISCornerAvhrrLin[1], IISCornerAvhrrCol[1]);
    printf( "   Pixel 3 : (ligAVHRR,colAVHRR) = (%lf,%lf)\n", IISCornerAvhrrLin[2], IISCornerAvhrrCol[2]);
    printf( "   Pixel 4 : (ligAVHRR,colAVHRR) = (%lf,%lf)\n", IISCornerAvhrrLin[3], IISCornerAvhrrCol[3]);
  #endif

  if( cr_op != CMN__STATUT_OK)
  {
    cr_op=   SD_ALG_AVHRR_ERR_METOP2AVHRR;
    msg_err= (char *)SD_BAS_MSG_ERR_METOP2AVHRR;
    num_lig= __LINE__;
    goto fin;
  }
  //-------------------------------------------------------------------------


  //-------------------------------------------------------------------------
  // Calcul du domaine Avhrr utile : determination des numeros de lignes/colonnes de debut et de fin
  //-------------------------------------------------------------------------
  minval= MAX_DOUBLE;
  maxval= MIN_DOUBLE;
  for( i=1; i <= 4; i++)
  {
    minval= MIN( IISCornerAvhrrLin[IND1(i)] + OffsetIISAvhrr[IND2(2,SN,SD_BAS_DEUX)], minval);
    maxval= MAX( IISCornerAvhrrLin[IND1(i)] + OffsetIISAvhrr[IND2(2,SN,SD_BAS_DEUX)], maxval);
  }
  *AvhrrUtilLinDeb= (int)( minval);
  AvhrrUtilLinFin= (int)( maxval) + 1;
  //jlp33
  if  (*AvhrrUtilLinDeb <0 || AvhrrUtilLinFin >= AvhrrNbLin )
   {
    cr_op=   SD_ALG_AVHRR_ERR_METOP2AVHRR;
    msg_err= (char *)SD_BAS_MSG_ERR_METOP2AVHRR;
    num_lig= __LINE__;
    goto fin;
    }

  minval= MAX_DOUBLE;
  maxval= MIN_DOUBLE;
  for( i=1; i <= 4; i++)
  {
    minval= MIN( IISCornerAvhrrCol[IND1(i)] + OffsetIISAvhrr[IND2(1,SN,SD_BAS_DEUX)], minval);
    maxval= MAX( IISCornerAvhrrCol[IND1(i)] + OffsetIISAvhrr[IND2(1,SN,SD_BAS_DEUX)], maxval);
  }
    *AvhrrUtilColDeb= (int)( minval);
  AvhrrUtilColFin= (int)( maxval) + 1;
 if  (*AvhrrUtilColDeb <1 || AvhrrUtilColFin >= SD_BAS_NPI_AVHRR )
   {
    cr_op=   SD_ALG_AVHRR_ERR_METOP2AVHRR;
    msg_err= (char *)SD_BAS_MSG_ERR_METOP2AVHRR;
    num_lig= __LINE__;
    goto fin;
    }

  *AvhrrUtilNbLin=  AvhrrUtilLinFin - *AvhrrUtilLinDeb + 1;
  *AvhrrUtilNbCol=  AvhrrUtilColFin - *AvhrrUtilColDeb + 1;

  #ifdef OPS_DEBUG
    printf( "Determination du domaine Avhrr utile contenu dans l'imagette IIS\n");
    printf( "   AvhrrUtilLinDeb : %d\n", *AvhrrUtilLinDeb);
    printf( "   AvhrrUtilLinFin : %d\n",  AvhrrUtilLinFin);
    printf( "   AvhrrUtilNbLin  : %d\n", *AvhrrUtilNbLin);
    printf( "   -----------------------------\n");
    printf( "   AvhrrUtilColDeb : %d\n", *AvhrrUtilColDeb);
    printf( "   AvhrrUtilColFin : %d\n",  AvhrrUtilColFin);
    printf( "   AvhrrUtilNbCol  : %d\n", *AvhrrUtilNbCol);
  #endif
  //-------------------------------------------------------------------------

/*MOR VERRUE A ENLEVER*/
//SUITE:
//*AvhrrUtilNbLin=61;
//*AvhrrUtilNbCol=66;
//*AvhrrUtilNbLin=59;
//*AvhrrUtilNbCol=67;
//*AvhrrUtilLinDeb=43-1;
//*AvhrrUtilColDeb=851;
//AvhrrColDeb=1;
//AvhrrLinDeb=1;

/*FIN MOR*/

  //-------------------------------------------------------------------------
  // Extraction de l'imagette Avhrr : AvUtil( NbChannel, NbLinAv, NbColAv)
  //-------------------------------------------------------------------------

// AvhrrLinDeb commence a 0
// et *AvhrrUtilLinDeb commence a 0 (AC)
// AvhrrColDeb commence a 1
//  *AvhrrUtilColDeb commence a 1 (AC)
// les indices sont calcules en relatif 
 
  for( Nch=1; Nch <= AvhrrNbChannel; Nch++)
  {
    // Boucle sur les lignes AvhrrUtil ( Il= 1 a AvhrrUtilNbLin )
    for( Il=1; Il <= *AvhrrUtilNbLin; Il++)
    {
//la ligne commence a 0
      Jl= *AvhrrUtilLinDeb  + Il;
//      Jl= *AvhrrUtilLinDeb - AvhrrLinDeb + Il;

      // Boucle sur les colonnes AvhrrUtil ( Ic= 1 a AvhrrUtilNbCol)
      for( Ic=1; Ic <= *AvhrrUtilNbCol; Ic++)
      {
        Jc= *AvhrrUtilColDeb  + Ic -1;
//        Jc= *AvhrrUtilColDeb - AvhrrColDeb + Ic;
//temporaire : les lignes sont inversees en fonction des lignes Noveltis
//        AvhrrUtilImage[IND3(Ic,Il,Nch,SD_BAS_NPI_AVHRR,SD_BAS_NLI_MAX_UTIL_AVHRR)]=
//          AvhrrListeLin[IND1(Jl)]->Image.im[IND2(Nch,Jc,5)];
        AvhrrUtilImage[IND3(Ic,Il,Nch,SD_BAS_NPI_AVHRR,SD_BAS_NLI_MAX_UTIL_AVHRR)]=
          AvhrrListeLin[IND1(Jl)]->Image.im[IND2(Jc,Nch,SD_BAS_NPI_AVHRR)];
        // FT-2528
        CloudInformation[IND2(Ic,Il,SD_BAS_NPI_AVHRR)] =
          (unsigned short int)AvhrrListeLin[IND1(Jl)]->InfoDet.tab_var[AVHRR_CLOUD_INFORMATION].val.pui4[IND1(Jc)];
        // ...FT-2528
      }
      // FT-2528
      QualityIndicator[IND1(Il)]
          = AvhrrListeLin[IND1(Jl)]->InfoDet.tab_var[AVHRR_QUALITY_INDICATOR].val.pui4[0];
      ScanLineQuality[IND1(Il)]
          = AvhrrListeLin[IND1(Jl)]->InfoDet.tab_var[AVHRR_SCAN_LINE_QUALITY].val.pui4[0];
      CountErrorFrame[IND1(Il)]
          = (unsigned short int)AvhrrListeLin[IND1(Jl)]->InfoDet.tab_var[AVHRR_COUNT_ERROR_FRAME].val.pui4[0];
      // ...FT-2528

    }
  }
  //-------------------------------------------------------------------------


  //-------------------------------------------------------------------------
  // Extension a la dimension maximale (ChanMax=6) des canaux Avhrr en utilisant ChannelState
  //   AvUtil( 5, AvUtilNbLin, AvUtilNbCol) ==> RadImgAv( 6, AvUtilNbLin, AvUtilNbCol)
  //-------------------------------------------------------------------------
  for( Il= 1; Il <= *AvhrrUtilNbLin; Il++)
  {
    if(AvhrrChannelState[IND2(Il,3,SD_BAS_NLI_MAX_UTIL_AVHRR)] == 1) //Test du ChannelState 3A
    { //le 3A est OK
      //On decale les lignes 5 et 6
      for(k=6 ; k>=5 ; k--)
      {
        for( Ic=1; Ic <= *AvhrrUtilNbCol; Ic++)
        {
          AvhrrUtilImage[IND3(Ic,Il,k,SD_BAS_NPI_AVHRR,SD_BAS_NLI_MAX_UTIL_AVHRR)]=
          AvhrrUtilImage[IND3(Ic,Il,k-1,SD_BAS_NPI_AVHRR,SD_BAS_NLI_MAX_UTIL_AVHRR)];
        }  //FIN for Ic
      } //FIN for k

      for( Ic=1; Ic <= *AvhrrUtilNbCol; Ic++)
      {
        AvhrrUtilImage[IND3(Ic,Il,4,SD_BAS_NPI_AVHRR,SD_BAS_NLI_MAX_UTIL_AVHRR)]=0.0f;
      }  //FIN for Ic

    } //FIN if 3A
    else
    { //C'est le 3B qui est actif
        //On decale les lignes 4, 5 et 6
      for(k=6 ; k>=4 ; k--)
      {
        for( Ic=1; Ic <= *AvhrrUtilNbCol; Ic++)
        {
          AvhrrUtilImage[IND3(Ic,Il,k,SD_BAS_NPI_AVHRR,SD_BAS_NLI_MAX_UTIL_AVHRR)]=
          AvhrrUtilImage[IND3(Ic,Il,k-1,SD_BAS_NPI_AVHRR,SD_BAS_NLI_MAX_UTIL_AVHRR)];
        }  //FIN for Ic
      } //FIN for k

      for( Ic=1; Ic <= *AvhrrUtilNbCol; Ic++)
      {
        AvhrrUtilImage[IND3(Ic,Il,3,SD_BAS_NPI_AVHRR,SD_BAS_NLI_MAX_UTIL_AVHRR)]=0.0f;
      }  //FIN for Ic
    } //FIN else : 3B
  }//FIN for Il


  // Etat des canaux Avhrr et detection des transitions
  //Boucle sur les canaux Avhrr ( k = 1, ChanMax)
  for( k=1; k <= SD_BAS_NBK; k++)
  {
    // initialisations 
    AvhrrUtilChannelState[IND1(k)]= 0;
    Sum[IND1(k)]= 0;
    // Boucle sur les lignes Avhrr ( il = 1, AvUtilNbLin )
    for( Il= 1; Il <= *AvhrrUtilNbLin; Il++)
    {
      Jl= *AvhrrUtilLinDeb - AvhrrLinDeb + Il;
      Sum[IND1(k)]=   Sum[IND1(k)]
                    + AvhrrChannelState[IND2(Jl,k,SD_BAS_NLI_MAX_UTIL_AVHRR)];
    }

    // Si Sum(k) = 0 alors AvhrrChannel(k) = 0
    if( Sum[IND1(k)] == 0)
      AvhrrUtilChannelState[IND1(k)]= 0;

    // Si Sum(k) = AvUtilNbLin alors AvhrrChannel(k) = 1
    if( Sum[IND1(k)] == *AvhrrUtilNbLin)
      AvhrrUtilChannelState[IND1(k)]= 1;

    // Si 0 < Sum(k) < AvUtilNbLin alors AvhrrChannel(k) = 2
    if( (0 < Sum[IND1(k)]) && (Sum[IND1(k)] < *AvhrrUtilNbLin))
      AvhrrUtilChannelState[IND1(k)]= 2;
  }

fin:
  // Fin de la fonction ImageUtile
    // Traitement des erreurs
    //================================================================
    if ( cr_op != CMN__STATUT_OK )
    {
        // mise a jour de la variable ctx->cr.errinterne
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
        SD_BAS_ERR_SET_AVHRR,
        cr_op,
        tcProcName_F,
        num_lig,
        msg_err);

        // on imprime le message d'erreur sur la sortie standard
        #ifdef OPS_DEBUG
            SD_BAS_PRINT_ERR_INT(ctx->cr.errinterne.nomfonction,
                                 ctx->cr.errinterne.numligne,
                                 ctx->cr.errinterne.libelle);
        #endif
    }

  #ifdef OPS_DEBUG
    printf( "Fin de la fonction ImageUtile\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN ImageUtile ================================================================
// ===============================================================================



//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_TraitCanauxAvhrr
//@
//@ objet :  Traitement des canaux Avhrr: Test de conversion en Tb,
//@          limitation aux canaux presents (Nch) et transformation en vecteur
//@
//@
//@ Parametres :
//@         (ENTREE) ConfAutre ........... : Pour les constantes de Planck
//@         (ENTREE) AvhrrUtilImage ...... : Parite de l'mage AVHRR Utile
//@         (ENTREE) AvhrrUtilNbLin ...... : Nombre de lignes de l'image utile
//@         (ENTREE) AvhrrUtilNbCol ...... : Nombre de colonnes de l'image utile
//@         (ENTREE) AvhrrUtilChannelState : Flags d'etat de chaque canal de l'image utile
//@         (ENTREE) CCSRadToTb .......... : Flags de transformation en temperature de brillance
//@         (ENTREE) CCSDimClassSpace .... : Nombre maximal de canaux ou de pseudo-canaux
//@         (ENTREE) CCSSortingChannel ... : Canal de rangement
//@         (ENTREE) CCSChannelWn ........ : Nombre d'onde des canaux
//@         (ENTREE) CCSChannelId ........ : Identification des canaux
//@         (ENTREE) CCSChannelNoise ..... : Bruit des canaux
//@
//@         (ENTREE) CCSChannelWgt ....... : Poids des canaux
//@
//@         (SORTIE) RadAv ............... : Vecteur de radiance de l'image AVHRR
//@         (SORTIE) NbPixAv ............. : Nombre de pixels du vecteur
//@         (SORTIE) Nch ................. : Nombre de canaux presents
//@         (SORTIE) ChId ................ : Identificateur des canaux
//@         (SORTIE) SortCh .............. : Canal de rangement
//@         (SORTIE) ChNoise ............. : Bruit des canaux
//@         (SORTIE) ChWgt ............... : Poids des canaux
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_TraitCanauxAvhrr(
  // Entrees
  t_SD_BAS_ConfAutre *ConfAutre,       // Pour les constantes de Planck
  float        *AvhrrUtilImage,        // float( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NBK)
  int           AvhrrUtilNbLin,        // int(1)
  int           AvhrrUtilNbCol,        // int(1)
  unsigned int *AvhrrUtilChannelState, // unsigned int( SD_BAS_NBK)
  int           CCSRadToTb,            // int(1)
  int           CCSDimClassSpace,      // int(1)
  int           CCSSortingChannel,     // int(1)
  double       *CCSChannelWn,          // double( SD_BAS_NBK)
  int          *CCSChannelId,          // int( SD_BAS_NBK)
  double       *CCSChannelNoise,       // double( SD_BAS_NBK)
  int          *CCSChannelWgt,         // int( SD_BAS_NBK)
  // Sorties
  float        *RadAv,    // float( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NBK)
  int          *NbPixAv,  // int(1)
  int          *Nch,      // int(1)
  int          *ChId,     // int( SD_BAS_NBK)
  int          *SortCh,   // int(1)
  double       *ChNoise,  // double( SD_BAS_NBK)
  int          *ChWgt     // int( SD_BAS_NBK)
)
{
  static const char tcProcName_F[] = "SD_ALG_TraitCanauxAvhrr";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  double Plank1, Plank2;     // Constantes de Planck
  int    Ibk, Nbk, Ich;      // Index sur les canaux
  int    Il, Ic;             // Index sur l'image
  int    Ilin, Icol;         // Index sur l'image
  int    index, index2;      // Index d'usage general
  int    TailleImage;        // Nombre de pixels de l'image Avhrr
// evolution SD_ALG_TraitCanauxAvhrr suite a PM 5
  double DPlk;     // derivee de planck
  double ChNoise_loc[SD_BAS_NBK];
  int  ChanAVHRR;
  #ifdef OPS_DEBUG
    printf( "Debut de la fonction TraitCanauxAvhrr\n----------------------------------\n");
  #endif

  Plank1= ConfAutre->IDefPlkPK1;
  Plank2= ConfAutre->IDefPlkPK2;

  TailleImage= SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR;

  // Definitions :
  // DimClassSpace = Nombre de canaux souhaites
  // Nch = nombre de canaux utilisables

  // Nbk = 6 : nombre de canaux Avhrr
  Nbk= SD_BAS_NBK;

  // Conversion en temperature de brillance
  // Si (RadtoTb = 1) alors
  if( CCSRadToTb == 1)
  {
    // Boucle sur les canaux Avhrr 
    for( Ibk= 1; Ibk <= CCSDimClassSpace;  Ibk++)
    {
      ChanAVHRR=CCSChannelId[IND1(Ibk)];
      ChNoise_loc[IND1(Ibk)]= CCSChannelNoise[IND1(ChanAVHRR)];
      // Si (ChannelState(Ibk) = 1 && canal avhrr) alors
      if( AvhrrUtilChannelState[IND1(ChanAVHRR)] == 1 &&
          ChanAVHRR >= 4)
      {
        // RadImg(Ibk,NbLinAv,NbColAv)= InvPlanck{ RadImg(Ibk,NbLinAv,NbColAv), ChannelWn(Ibk) }
        for( Ilin= 1; Ilin <= AvhrrUtilNbLin; Ilin++)
        {
          for( Icol= 1; Icol <= AvhrrUtilNbCol; Icol++)
          {
            index= IND3( Icol, Ilin, ChanAVHRR, SD_BAS_NPI_AVHRR, SD_BAS_NLI_MAX_UTIL_AVHRR);
            AvhrrUtilImage[index]= (float)SD_BAS_PlanckInv( (double)AvhrrUtilImage[index],
                                                            CCSChannelWn[IND1(ChanAVHRR)],
                                                            Plank1,
							    Plank2);
          }
        }
      }  // Fin de Si
    }    // Fin de boucle Ibk
  }      // Fin de Si
else
  {
    // Boucle sur les canaux Avhrr 
     for( Ibk= 1; Ibk <= CCSDimClassSpace;  Ibk++)
      {
      ChanAVHRR=CCSChannelId[IND1(Ibk)];
     //jlp irpour les canaux infra-rouge les radiometries sont exprimees en nedt 
    // la valeur du chnoise doit donc etre exprimee aussi dans cette unite
    // a 280 k
     // alors on multiplie par la derive dw/dt aa 280 K 
      if ( ChanAVHRR >= 4)
     	{
     	DPlk= SD_BAS_PlanckDeriv( (double)280, CCSChannelWn[IND1(ChanAVHRR)], Plank1, Plank2);
     	ChNoise_loc[IND1(Ibk)]= CCSChannelNoise[IND1(ChanAVHRR)]*DPlk;
     	}
       else
        {
        ChNoise_loc[IND1(Ibk)]= CCSChannelNoise[IND1(ChanAVHRR)];
	}
      }
  }
  // Gestion des canaux et passage de matrice en vecteur
  *Nch= 0;
  *SortCh=1;
  // Boucle sur les canaux souhaites ( Ich=1 a DimClassSpace )
  for( Ich= 1; Ich <= CCSDimClassSpace; Ich++)
  {
      ChanAVHRR=CCSChannelId[IND1(Ich)];
    // Si ( ChannelState(ChannelId(Ich)) = 1 ) alors
    if( AvhrrUtilChannelState[IND1(ChanAVHRR)] == 1 )
    {
      *(Nch)= *(Nch) + 1;
      ChId[IND1(*(Nch))]= ChanAVHRR;

      // Si ( ChannelId(Ich) = SortingChannel ) alors SortCh = Nch
      if( ChanAVHRR == CCSSortingChannel)   *SortCh= *Nch;

      ChNoise[IND1(*(Nch))]= ChNoise_loc[IND1(Ich)];
 
       ChWgt[IND1(*(Nch))]= CCSChannelWgt[IND1(ChanAVHRR)];
      *(NbPixAv)= 0;

      // Boucle sur les lignes Avhrr (Ilin=1 a NbLinAv)
     for( Ilin= 1; Ilin <= AvhrrUtilNbLin; Ilin++)
      {
        // Boucle sur les colonnes Avhrr (Icol=1 a NbColAv)
        for( Icol= 1; Icol <= AvhrrUtilNbCol; Icol++)
        {
          // NbPixAv= NbPixAv + 1;
          *(NbPixAv)= *(NbPixAv) + 1;

          // RadAv(Nch, NbPixAv) = AvUtil(ChannelId(Ich),Ilin,Icol)
          index= IND2( *(NbPixAv), *(Nch), TailleImage);
          index2= IND3( Icol, Ilin, ChanAVHRR, SD_BAS_NPI_AVHRR, SD_BAS_NLI_MAX_UTIL_AVHRR);
          RadAv[index]= AvhrrUtilImage[index2];
        } // Fin de boucle sur les colonnes
      }   // Fin de boucle sur les lignes
    }     // Fin de Si
  }       // Fin de boucle

fin:
  // Fin de la fonction TraitCanauxAvhrr
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction TraitCanauxAvhrr\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN TraitCanauxAvhrr ==========================================================
// ===============================================================================



//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_PseudoCanauxIIS
//@
//@ objet :  Traitement de l'image AVHRR
//@
//@
//@ Parametres :
//@         (ENTREE) ConfAutre ........... : Pour les constantes de Planck
//@         (ENTREE) Image ............... : Image IIS
//@         (ENTREE) Ml .................. : Nombre de lignes de l'image IIS
//@         (ENTREE) Mc .................. : Nombre de colonnes de l'image IIS
//@         (ENTREE) IISDeadPix .......... : Table des pixels morts de l'image IIS
//@         (ENTREE) CCSChannelWn ........ : Nombre d'onde des canaux
//@
//@         (SORTIE) PseudoImage ......... : Pseudo image AVHRR construite a partir de l'IIS
//@         (SORTIE) RadIIS .............. : Vecteurs de radiances de la pseudo image
//@                                          Considere comme NBK vecteurs de NPI_AVHRR * NLI_MAX_UTIL_AVHRR valeurs
//@         (SORTIE) NbPixIIS ............ : Nombre des pixels de l'image IIS
//@         (SORTIE) LinDeb .............. : 1ere ligne utile
//@         (SORTIE) ColDeb .............. : 1ere colonne utile
//@         (SORTIE) NbLin ............... : Nombre de lignes utiles
//@         (SORTIE) NbCol ............... : Nombre de colonnes utiles
//@         (SORTIE) Nch ................. : Nombre de canaux presents
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_PseudoCanauxIIS(
  // Entrees
  t_SD_BAS_ConfAutre *ConfAutre,     // Pour les constantes de Planck
  // Entrees / Sorties
  double *Image,         // double( SD_BAS_IMCO * SD_BAS_IMLI)
  // Entrees
  int     Ml,            // int(1)
  int     Mc,            // int(1)
  int    *IISDeadPix,    // uint( SD_BAS_IMCO * SD_BAS_IMLI)
  double *CCSChannelWn,  // double( SD_BAS_NBK)
  // Sorties
  float  *PseudoImage,   // float( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NBK)
  float  *RadIIS,        // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
                         // Considere comme NBK vecteurs de NPI_AVHRR * NLI_MAX_UTIL_AVHRR valeurs
  int    *NbPixIIS,      // int(1)
  int    *LinDeb,        // int(1)
  int    *ColDeb,        // int(1)
  int    *NbLin,         // int(1)
  int    *NbCol,         // int(1)
  int    *Nch            // int(1)
)
{
  static const char tcProcName_F[] = "SD_ALG_PseudoCanauxIIS";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  double Plank1, Plank2;     // Constantes de Planck
  int    Ilin, Icol;
  int    TailleImage;        // Nombre de pixels de l'image Avhrr
  double MoyLocale;          // Moyenne locale (fenetre 3x3)
  double Moyenne;
  int    NbPix;
  int    Ich;

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction PseudoCanauxIIS\n----------------------------------\n");
  #endif

  Plank1= ConfAutre->IDefPlkPK1;
  Plank2= ConfAutre->IDefPlkPK2;

  // Interpolation des pixels morts de l'IIS
  // Boucle sur les pixels l'image IIS (il,ic)
  for( Ilin= 1; Ilin <= Ml; Ilin++)
  {
    for( Icol= 1; Icol <= Mc; Icol++)
    {
      // Si (IISDeadpix(il,ic)=oui) alors
      /* FA128 */
      if( IISDeadPix[IND2(Ilin,Icol,SD_BAS_IMLI)] == CMN__OUI)
      {
        // remplacer Image(il,ic) par la moyenne des pixels dans l'image       123
        // qui sont voisins immediats de (il,ic) et tels que IISDeadPix = non  8x4
        // (la moyenne porte donc sur 0 a 8 pixels)                            765
        Moyenne= 0.0;
        NbPix= 0;
        
        /* FA128 */
        if( (Icol > 1) && (Ilin > 1) &&
	           (IISDeadPix[IND2(Ilin-1,Icol-1,SD_BAS_IMLI)] == CMN__NON)
	         )
        { // Pixel 1
          Moyenne= Moyenne + Image[IND2(Icol-1,Ilin-1,SD_BAS_IMCO)];
          NbPix++;
        }
        
        /* FA128 */
        if( (Ilin > 1) && (IISDeadPix[IND2(Ilin-1,Icol,SD_BAS_IMLI)] == CMN__NON))
        { // Pixel 2
          Moyenne= Moyenne + Image[IND2(Icol,Ilin-1,SD_BAS_IMCO)];
          NbPix++;
        }
        
        /* FA128 */
        if( (Icol < Mc) && (Ilin > 1) &&
       	    (IISDeadPix[IND2(Ilin-1,Icol+1,SD_BAS_IMLI)] == CMN__NON)
	         )
        { // Pixel 3
          Moyenne= Moyenne + Image[IND2(Icol+1,Ilin-1,SD_BAS_IMCO)];
          NbPix++;
        }
        
        /* FA128 */
        if( (Icol < Mc) && (IISDeadPix[IND2(Ilin,Icol+1,SD_BAS_IMLI)] == CMN__NON))
        { // Pixel 4
          Moyenne= Moyenne + Image[IND2(Icol+1,Ilin,SD_BAS_IMCO)];
          NbPix++;
        }
        
        /* FA128 */
        if( (Icol < Mc) && (Ilin < Ml) &&
       	    (IISDeadPix[IND2(Ilin+1,Icol+1,SD_BAS_IMLI)] == CMN__NON)
	         )
        { // Pixel 5
          Moyenne= Moyenne + Image[IND2(Icol+1,Ilin+1,SD_BAS_IMCO)];
          NbPix++;
        }
        
        /* FA128 */
        if( (Ilin < Ml) && (IISDeadPix[IND2(Ilin+1,Icol,SD_BAS_IMLI)] == CMN__NON))
        { // Pixel 6
          Moyenne= Moyenne + Image[IND2(Icol,Ilin+1,SD_BAS_IMCO)];
          NbPix++;
        }
        
        /* FA128 */
        if( (Icol > 1) && (Ilin < Ml) &&
       	    (IISDeadPix[IND2(Ilin+1,Icol-1,SD_BAS_IMLI)] == CMN__NON)
	         )
        { // Pixel 7
          Moyenne= Moyenne + Image[IND2(Icol-1,Ilin+1,SD_BAS_IMCO)];
          NbPix++;
        }
        
        /* FA128 */
        if( (Icol > 1) && (IISDeadPix[IND2(Ilin,Icol-1,SD_BAS_IMLI)] == CMN__NON))
        { // Pixel 8
          Moyenne= Moyenne + Image[IND2(Icol-1,Ilin,SD_BAS_IMCO)];
          NbPix++;
        }
        if( NbPix > 0)   Moyenne= Moyenne / (double)NbPix;
        Image[IND2(Icol,Ilin,SD_BAS_IMCO)]= Moyenne;
      }
    }
  }

  // Conversion en temperature de brillance
  // Boucle sur les pixels l'image IIS (il,ic) ==> Utilisation du nombre d'onde no1
  for( Ilin= 1; Ilin <= Ml; Ilin++)
  {
    for( Icol= 1; Icol <= Mc; Icol++)
    {
      // Image(Ml, Mc) = InvPlanck{ Image(Ml, Mc), ChannelWn }
      Image[IND2(Icol,Ilin,SD_BAS_IMCO)]= SD_BAS_PlanckInv( (double)Image[IND2(Icol,Ilin,SD_BAS_IMCO)],
                                                        CCSChannelWn[IND1(1)],
                                                        Plank1, Plank2);
    }
  }

  // Construction des pseudo canaux
  *LinDeb= 2;
  *ColDeb= 2;
  *NbLin= Ml-2;
  *NbCol= Mc-2;
  *Nch= 2;

  // Boucle sur les lignes Ilin = 1 a Ml-2
  for( Ilin= 1; Ilin <= *NbLin; Ilin++)
  {
    // Boucle sur les colonnes Icol = 1 a Mc-2
    for( Icol= 1; Icol <= *NbCol; Icol++)
    {
      MoyLocale= SD_ALG_MoyenneLocale( Image, Ilin+1, Icol+1, SD_BAS_IMLI, SD_BAS_IMCO); // {sur grille 3x3}
      PseudoImage[IND3(Icol,Ilin,1,SD_BAS_NPI_AVHRR,SD_BAS_NLI_MAX_UTIL_AVHRR)]= (float)MoyLocale;
      PseudoImage[IND3(Icol,Ilin,2,SD_BAS_NPI_AVHRR,SD_BAS_NLI_MAX_UTIL_AVHRR)]=
        (float)SD_ALG_VarianceLocale( Image, Ilin+1, Icol+1, SD_BAS_IMLI, SD_BAS_IMCO, MoyLocale); // {sur grille 3x3}
      PseudoImage[IND3(Icol,Ilin,3,SD_BAS_NPI_AVHRR,SD_BAS_NLI_MAX_UTIL_AVHRR)]= (float)0.0;
      PseudoImage[IND3(Icol,Ilin,4,SD_BAS_NPI_AVHRR,SD_BAS_NLI_MAX_UTIL_AVHRR)]= (float)0.0;
      PseudoImage[IND3(Icol,Ilin,5,SD_BAS_NPI_AVHRR,SD_BAS_NLI_MAX_UTIL_AVHRR)]= (float)0.0;
      PseudoImage[IND3(Icol,Ilin,6,SD_BAS_NPI_AVHRR,SD_BAS_NLI_MAX_UTIL_AVHRR)]= (float)0.0;
    }
  }

  TailleImage= SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR;
  // Conversion de matrice en vecteur
  *NbPixIIS= 0;
  // Boucle sur les lignes Ilin = 1 a Ml-2
  for( Ilin= 1; Ilin <= *NbLin; Ilin++)
  {
    // Boucle sur les colonnes Icol = 1 a Mc-2
    for( Icol= 1; Icol <= *NbCol; Icol++)
    {
      *NbPixIIS= *NbPixIIS + 1;
      // Boucle sur les pseudo-canaux (Ich=1 a 6)
      for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
      {
        // RadIIS(Ich,NbPixAv) = PseudoImage (Ich,Ilin,Icol)
        RadIIS[IND2(*NbPixIIS,Ich,TailleImage)]= 
          PseudoImage[IND3(Icol,Ilin,Ich,SD_BAS_NPI_AVHRR,SD_BAS_NLI_MAX_UTIL_AVHRR)];
      }
    }
  }

fin:
  // Fin de la fonction PseudoCanauxIIS
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction PseudoCanauxIIS\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN PseudoCanauxIIS ===========================================================
// ===============================================================================



//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_NueesDynamiques
//@
//@ objet :  Traitement de l'image AVHRR
//@
//@
//@ Parametres :
//@         (ENTREE) ProduitCourant ...... : Pour pointer sur les variables temporaires
//@         (ENTREE) Contexte_courant .... : contexte de l'etape (numero de ligne SN traite)
//@         (ENTREE) RadAv ............... : Vecteur de radiance de l'image AVHRR
//@         (ENTREE) NbPixAv ............. : Nombre de pixels du vecteur
//@         (ENTREE) Nch ................. : Nombre de canaux presents
//@         (ENTREE) SortingChannel ...... : Canal de rangement
//@         (ENTREE) ChannelNoise ........ : Bruit des canaux
//@         (ENTREE) ChannelWgt .......... : Poids des canaux
//@         (ENTREE) NbClusMax ........... : Nombre maximal de clusters
//@         (ENTREE) NbIterMax ........... : Nombre maximal d'iterations
//@         (ENTREE) IterConvCutoff ...... : Seuil de convergence
//@         (ENTREE) SplitCutoff ......... : Seuil d'eclatement
//@         (ENTREE) GlueCutoff .......... : Seuil de coalescence
//@         (ENTREE) EigValCutoff ........ : Seuil de valeur propre
//@         (ENTREE) DistAgloInit ........ : Valeur initiale de la distance d'aglomeration
//@         (ENTREE) MinIterNCCutOff ..... : Nombre d'iteration minimales pour le calcul du seuil des non classifies
//@         (ENTREE) NbNonClassifMax ..... : Nombre maximal de non classifies
//@         (ENTREE) DistMax ............. : Distance maximale interclasses
//@
//@         (SORTIE) IdPix ............... : Flags de pixels valides
//@         (SORTIE) NoExt ............... : Flags de classes non extremes
//@         (SORTIE) NbClass ............. : Nombre de classes
//@         (SORTIE) FlagPostProcessing .. : Indique si le traitement a utilise un post processing ou non
//@         (SORTIE) NonClassifRate ...... : Taux de pixels non classes
//@         (SORTIE) VarianceRate ........ : Rapport : variance image classifiee / image non classifiee
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_NueesDynamiques(
  // Entrees
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  t_SD_BAS_DataThread  *Contexte_courant,    // t_SD_BAS_DataThread*
  float                *RadAv,               // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
                                             // Considere comme NBK vecteurs de NPI_AVHRR * NLI_MAX_UTIL_AVHRR valeurs
  int                   NbPixAv,             // int(1)
  int                   Nch,                 // int(1)
  int                   SortingChannel,      // int(1)
  double               *ChannelNoise,        // double( SD_BAS_NBK)
  int                  *ChannelWgt,          // int( SD_BAS_NBK)
  int                   NbClusMax,           // int(1)
  int                   NbIterMax,           // int(1)
  double                IterConvCutoff,      // double(1)
  double                SplitCutoff,         // double(1)
  double                GlueCutoff,          // double(1)
  double                EigValCutoff,        // double(1)
  double                DistAgloInit,        // double(1)
  int                   MinIterNCCutOff,     // int(1)
  int                   NbNonClassifMax,     // int(1)
  double                DistMax,             // double(1)
  // Sorties
  int                  *IdPix,               // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  unsigned int         *NoExt,               // unsigned int( SD_BAS_NCL_MAX)
  int                  *NbClass,             // int(1)
  unsigned int         *FlagPostProcessing,  // unsigned int(1)
  double               *NonClassifRate,      // double(1)
  double               *VarianceRate         // double(1)
)
{
  static const char tcProcName_F[] = "SD_ALG_NueesDynamiques";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  // Definitions
  // (#) (1 a Nch) nombre de canaux/pseudo-canaux Avhrr utilises
  // (*) (1 a NbPixAv) nombre de pixels Avhrr
  // ($) (1 a NbPro) nombre de classes prototype
  double   RadMean[SD_BAS_NBK];                      // double( SD_BAS_NBK)
  double   RadStd[SD_BAS_NBK];                       // double( SD_BAS_NBK)
  float  *RadNW;                                    // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
  double  ProtoZero[SD_BAS_NBK];                    // double( SD_BAS_NBK)
/* bma complexes->reels */
  double  EigVal[SD_BAS_NBK];                       // double( SD_BAS_NBK)
  double  EigVect[SD_BAS_NBK*SD_BAS_NBK];           // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
  int     NbPro;                                    // int(1)
  double  Proto[SD_BAS_NBK*SD_BAS_NCL_MAX];             // double( SD_BAS_NCL_MAX * SD_BAS_NBK)
  double  Covar[SD_BAS_NBK*SD_BAS_NBK*SD_BAS_NCL_MAX];  // double( SD_BAS_NBK * SD_BAS_NBK * SD_BAS_NCL_MAX)
  int     NbPixClas[SD_BAS_NCL_MAX];                    // int( SD_BAS_NCL_MAX)
  double  SumDistClas[SD_BAS_NCL_MAX];                  // double( SD_BAS_NCL_MAX)
  double *Dist;                                     // double( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NCL_MAX)
  int     Iter, IterLast;                           // int(1)
//CR 236 modif pixels non classes CCS
  int 	Ipix; // boucle sur les pixels
  int  Ich,Ipro; // boucle d'initialisation


  #ifdef OPS_DEBUG
    printf( "Debut de la fonction NueesDynamiques\n----------------------------------\n");
  #endif

// initialisation du tableau
  for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
  {
    // (#) = (1 a NbChannel) nombre de canaux
    for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
    {
      // ProtoWork(#,1 a NbPro) = Proto(#,1 a NbPro)
      Proto[IND2(Ich,Ipro,SD_BAS_NBK)]= 0.;
    }
  }
  RadNW= ProduitCourant->RadNW;
  Dist= ProduitCourant->Dist;

  // 1) Preparation des radiances et du prototype zero
  cr_op= SD_ALG_InitRad(
           // Entrees
           ProduitCourant,    // t_SD_BAS_DataThreadProduit*
           Contexte_courant,  // t_SD_BAS_DataThread*
           RadAv,             // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
           NbPixAv,           // int(1)
           Nch,               // int(1)
           ChannelNoise,      // double( SD_BAS_NBK)
           ChannelWgt,        // int( SD_BAS_NBK)
           // Sorties
           RadMean,           // double( SD_BAS_NBK)
           RadStd,            // double( SD_BAS_NBK)
           RadNW,             // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
           ProtoZero,         // double( SD_BAS_NBK)
/* bma complexes->reels */
           EigVal,            // double( SD_BAS_NBK)
           EigVect);          // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */

  // 2) Recherche du premier echantillon de prototypes
  cr_op= SD_ALG_InitProto(
           // Entrees
            Contexte_courant,      // t_SD_BAS_DataThread*
            ProduitCourant,  // t_SD_BAS_DataThreadProduit*
            RadNW,           // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
            Nch,             // int(1)
            NbPixAv,         // int(1)
            ProtoZero,       // double( SD_BAS_NBK)
            NbClusMax,       // int(1)
            DistMax,         // double(1)
            SortingChannel,  // int(1)
           // Sorties
           &NbPro,           // int(1)
            Proto,           // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
            Covar,           // double( SD_BAS_NBK * SD_BAS_NBK * SD_BAS_NCL_MAX)
            Dist,            // double( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NCL_MAX)
            IdPix,           // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
            NbPixClas,       // int( SD_BAS_NCL_MAX)
            SumDistClas);    // double( SD_BAS_NCL_MAX)
     if(cr_op != CMN__STATUT_OK)
     {
         goto fin;
     }

  // 3) Processus iteratif de caracterisation des classes
  cr_op= SD_ALG_IterNuDyn(
           // Entree
              Contexte_courant,      // t_SD_BAS_DataThread*
            ProduitCourant,  // t_SD_BAS_DataThreadProduit*
            RadNW,           // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
            NbPixAv,         // int(1)
            Nch,             // int(1)
            NbPro,           // int(1)
            NbClusMax,       // int(1)
            NbIterMax,       // int(1)
/* bma complexes->reels */
            EigVal,          // double( SD_BAS_NBK)
            EigVect,         // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
            DistAgloInit,    // double(1)
            SplitCutoff,     // double(1)
            GlueCutoff,      // double(1)
            EigValCutoff,    // double(1)
            MinIterNCCutOff, // int(1)
            NbNonClassifMax, // int(1)
            SortingChannel,  // int(1)
            DistMax,         // double(1)
            IterConvCutoff,  // double(1)
           // Entrees / Sorties
            Proto,           // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
            IdPix,           // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
            NbPixClas,       // int( SD_BAS_NCL_MAX)
            SumDistClas,     // double( SD_BAS_NCL_MAX)
           // Sortie
            NbClass,         // int(1)
            NoExt,           // unsigned int( SD_BAS_NCL_MAX)
           &Iter);           // int(1)
     if(cr_op != CMN__STATUT_OK)
     {
         goto fin;
     }

  // 4) Post-processing
  *FlagPostProcessing = CMN__NON;
  
  // Si NbClass > NbClusMax alors
  if( *NbClass > NbClusMax)
  {
    IterLast= Iter;
    cr_op= SD_ALG_IterNuDynPost(
             // Entrees
              ProduitCourant,  // t_SD_BAS_DataThreadProduit*
              IterLast,        // int(1)
              NbPixAv,         // int(1)
              Nch,             // int(1)
              NbClusMax,       // int(1)
              EigVal,          // double( SD_BAS_NBK)
              EigVect,         // double( SD_BAS_NBK * SD_BAS_NBK)
              DistAgloInit,    // double(1)
              EigValCutoff,    // double(1)
              SortingChannel,  // int(1)
             // Entrees / Sorties
              Proto,           // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
              IdPix,           // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
              NbPixClas,       // int( SD_BAS_NCL_MAX)
              SumDistClas,     // double( SD_BAS_NCL_MAX)
              NoExt,           // unsigned int( SD_BAS_NCL_MAX)
             // Sortie
              NbClass,         // int(1)
             &Iter);           // int(1)

    *FlagPostProcessing= CMN__OUI;
  }

  // 5) Tests qualite :
  // Si Post-processing alors ... Passe au dessus
  cr_op= SD_ALG_TestQual(
           // Entrees
            NbPixAv,        // int(1)
            Nch,            // int(1)
           *NbClass,        // int(1)
/* bma complexes->reels */
            EigVal,         // double( SD_BAS_NBK)
/* fin bma */
            EigValCutoff,   // double(1)
            NbPixClas,      // int( SD_BAS_NCL_MAX)
            SumDistClas,    // double( SD_BAS_NCL_MAX)
           // Sortie
            NonClassifRate, // double(1)
            VarianceRate);  // double(1)

//CR 236 modif pixels non classes CCS
// on affecte ici tous les pixels non classes a la classe 7
// initialement ils sont a la classe NbClass+1 on les transforme en 7
// ceci est valable pour toutes les sorties de l'algo NueesDynamiques
// c.a.d IdPix 

 // Boucle sur les pixels (Ipix = 1 a NbPixAv)
for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
  {
  if ( IdPix[IND1(Ipix)] == *NbClass + 1 )
   {
    IdPix[IND1(Ipix)] = 7;
   }
  } // Fin de boucle sur les pixels
*NbClass = 7;
//CR 236 modif pixels non classes CCS

fin:

  // Fin de la fonction NueesDynamiques
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction NueesDynamiques\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN NueesDynamiques ===========================================================
// ===============================================================================


//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_CalcOffset
//@
//@ objet :  Traitement de l'image AVHRR
//@
//@
//@ Parametres :
//@         (ENTREE) Contexte_courant . : contexte de l'etape (numero de ligne SN traite)
//@         (ENTREE) MPHRN1 ........... :  MPHR produit N1
//@         (ENTREE) AvhrrListeLin .... : Suite de lignes AVHRR chargees en memoire
//@         (ENTREE) AvhrrNbLin ....... : Taille de la liste de lignes
//@         (ENTREE) AvhrrLinDeb ...... : Ligne de debut
//@         (ENTREE) AvhrrLinFin ...... : Ligne de fin
//@         (ENTREE) AvhrrColDeb ...... : Colonne de debut
//@         (ENTREE) AvhrrColFin ...... : Colonne de fin
//@
//@         (ENTREE) IasiTime ......... : Date de la visee courante
//@         (ENTREE) ScanAngle ........ : Angle de visee IASI
//@         (ENTREE) MatRot ........... : Matrice de rotation IASI-METOP
//@         (ENTREE) OffsetSondIISModel : Modele orbital de l'offset angulaire entre le sondeur et l'imageur
//@         (ENTREE) t_FE ............. : Date de fin de l'eclipse precedente
//@         (ENTREE) Period ........... : Periode (nodale) nominale de METOP
//@         (ENTREE) OffsetIISAvhrr ... : Offset Imageur-AVHRR dans le repere AVHRR
//@
//@         (SORTIE) FlagDateNOK ...... : Indication d'une incoherence des dates
//@         (SORTIE) OffsetSondIIS .... : Offset entre le sondeur IASI et l'imageur IIS
//@         (SORTIE) OffsetSondAvhrr .. : Offset entre le sondeur IASI et l'imageur AVHRR
//@         (SORTIE) OffsetIISAvhrrRI . : Offset entre l'imageur IIS et l'imageur AVHRR dans le repere IASI
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_CalcOffset(
  	// Entrees pour MetopAVHRR
  	int  CCSMode,               			  //int
  	t_SD_BAS_DataThread  *ctx,      		  // t_SD_BAS_DataThread*
  	t_SD_BAS_MPHR        *MphrN1,             // t_SD_BAS_MPHR*
  	t_SD_BAS_LigneAVHRR **AvhrrListeLin,      // t_SD_BAS_LigneAVHRR * ( AvhrrNbLin)
  	int                   AvhrrNbLin,         // int(1)
  	int                   AvhrrLinDeb,        // int(1)
  	int                   AvhrrLinFin,        // int(1)
  	int                   AvhrrColDeb,        // int(1)
  	int                   AvhrrColFin,        // int(1)
  	// Entrees
 	t_SD_BAS_SCDSTime     IasiTime,           // t_SD_BAS_SCDSTime*
  	double                ScanAngle,          // double(1)
  	double               *MatRot,             // double( SD_BAS_DIM_ROT_MATRICE * SD_BAS_DIM_ROT_MATRICE)
  	double               *OffsetSondIISModel, // double( SD_BAS_MODEL * SD_BAS_CCD)
  	t_SD_BAS_SCDSTime     t_FE,               // t_SD_BAS_SCDSTime*
  	double                Period,             // double(1)
  	double               *OffsetIISAvhrr,     // double( SD_BAS_DEUX)
  	// Sorties
  	int                  *FlagDateNOK,        // unsigned int(1)
  	double               *OffsetSondIIS,      // double( SD_BAS_DEUX)
  	double               *OffsetSondAvhrr,     // double( SD_BAS_DEUX)
	// FT-2997
  	double 				 *OffsetIISAvhrrRI    // double( SD_BAS_DEUX)
	// ...FT-2997
	)
	{
	  	static char tcProcName_F[] = "SD_ALG_CalcOffset";
	  	t_CMN__Statut cr_op = CMN__STATUT_OK;
	
	  	// Sorties de SD_BAS_MetopAVHRR
	  	double latitude;
	  	double longitude;
	  	double Ang_mesY;
	  	double Ang_mesZ;
	  	double Azim_sol;
	  	double Elev_sol;
	  	double EarthSatDist;
	  	double PsfCentreMetop[2];
	  	double PsfCentreAvMetop[2];
	  	// -------------------------------------
	
	  	double PsfCentreAvCol;
	  	double PsfCentreAvLin;
	  	double YPsfCentre;
	  	double ZPsfCentre;
	  	double YPsfCentreAv;
	  	double ZPsfCentreAv;
	  		
		// FT-2997
		double TimeFirst;
		double TimeLast;
	  	double DateFirstLineAvhrr;							
		// ...FT-2997

	  	// Gestion de erreurs
	  	char *msg_err = (char*) NULL;          	// Message d'erreur
	  	int num_lig = 0;            			// Ligne d'erreur
	
	  	#ifdef OPS_DEBUG
	    	printf( "Debut de la fonction CalcOffset\n----------------------------------\n");
	  	#endif
	  	
		//-----------------------
  		//@ initialisations liees au mode optimise
		//-----------------------
	  	OffsetIISAvhrrRI[0] = 0.;
	  	OffsetIISAvhrrRI[1] = 0.;
	  	OffsetSondIIS[0] = 0.;
	  	OffsetSondIIS[1] = 0.;
	  	PsfCentreMetop[0] = 0.;
	  	PsfCentreMetop[1] = 0.;
	  	PsfCentreAvMetop[0] = 0.;
	  	PsfCentreAvMetop[1] = 0.;
	  	PsfCentreAvCol = 0.;
	  	PsfCentreAvLin = 0.;
	  	YPsfCentre = 0.;
	  	ZPsfCentre = 0.;
	  	YPsfCentreAv = 0.;
	  	ZPsfCentreAv = 0.;

		//-----------------------
  		//@ Estimation de l'offset sondeur-IIS par le modele orbital
  		//@ OffsetSondIIS(1) = Modele-Orbital{ IasiTime,t_FE,Period,OffsetSondIISModel(1) }
		//-----------------------
  		OffsetSondIIS[IND1(1)]=
    			SD_BAS_EvolutionModele( IasiTime,                                    	// t_SD_BAS_SCDSTime*
                             			t_FE,                                        	// t_SD_BAS_SCDSTime*
                             			Period,                                      	// double(1)
                            			&OffsetSondIISModel[IND2(1,1,SD_BAS_MODEL)]); 	// double( SD_BAS_MODEL)

		//-----------------------
  		//@ OffsetSondIIS(2) = Modele-Orbital{ IasiTime,t_FE,Period,OffsetSondIISModel(2) }
		//-----------------------
  		OffsetSondIIS[IND1(2)]=
    			SD_BAS_EvolutionModele( IasiTime,                                    	// t_SD_BAS_SCDSTime*
                             			t_FE,                                        	// t_SD_BAS_SCDSTime*
                             			Period,                                      	// double(1)
                            			&OffsetSondIISModel[IND2(1,2,SD_BAS_MODEL)]); 	// double( SD_BAS_MODEL)

  		// Si ( abs(t-t_FE) < 8*Period ou OffsetSondIISModel(1,6)=0 ) alors
  		if( ( 
  			 fabs( ((double)IasiTime.day - (double)t_FE.day) * NB_MILLIS_JOUR
                   + (double)IasiTime.millis - (double)t_FE.millis)
           	 < (int)(8.0 * Period * NB_MILLIS_SEC) 
            )
      	 	|| 
      	 	( 
      	 	 SD_BAS_MacEGAUX_D (OffsetSondIISModel[IND2(6,1,SD_BAS_MODEL)], 0.0)
      	 	)
      	  )
  		{
    		// FlagDateNOK = NON
    		*FlagDateNOK= CMN__NON;
  		}
  		else
  		{
    		// FlagDateNOK = OUI
    		*FlagDateNOK= CMN__OUI;
  		} // Fin de Si

		//-----------------------
  		//@ Calcul des coordonnees du centre du champ dans le repere Metop
	  	//@ YPsfCentre = ZpsfCentre = 0
		//-----------------------
	  	YPsfCentre= 0.0;
	  	ZPsfCentre= 0.0;	
	
		//-----------------------
  		//@ PsfCentreMetop = GeomIasiMetop{ MatRot, ScanAngle, YPsfCentre, ZPsfCentre }
		//-----------------------
		cr_op= SD_ALG_GeomIasiMetop( 
                            // Entrees
                            MatRot,                       // double( SD_BAS_DIM_ROT_MATRICE * SD_BAS_DIM_ROT_MATRICE)
                            ScanAngle,                    // double(1)
                            YPsfCentre,                   // double(1)
                            ZPsfCentre,                   // double(1)
                            // Sorties
                        	&PsfCentreMetop[SD_ALG_AZIM],	// double(1)
                            &PsfCentreMetop[SD_ALG_ELEV]);// double(1)
		
		//-----------------------
  		//@ Calcul des coordonnees du centre du champ dans le repere Avhrr
  		//@ [PsfCentreAvLin, PsfCentreAvCol] = GeomMetopAvhrr{ IasiTime, PsfCentreMetop }
		//-----------------------

		// FT-2997
		// si on peut utiliser les donnees AVHRR
		
		if(CCSMode == SD_ALG_USE_AVHRR_DATA)
		{
  			// SD_BAS_ReserverOperateur();	  			
  			cr_op= SD_BAS_MetopAVHRR(
                     	// Entree
                        ctx,             // t_SD_BAS_DataThread*
                        &IasiTime,                     // t_SD_BAS_SCDSTime*
                        MphrN1,                       // t_SD_BAS_MPHR*
                        1,                            // 1 seul point
                        &PsfCentreMetop[SD_ALG_AZIM],  // double(1)
                        &PsfCentreMetop[SD_ALG_ELEV],  // double(1)
                        AvhrrLinDeb,                  // int(1)
                        AvhrrLinFin,                  // int(1)
                        AvhrrListeLin,                // t_SD_BAS_LigneAVHRR * ( AvhrrNbLin)
                        AvhrrNbLin,                   // int(1)
                        AvhrrColDeb,                  // int(1)
                        AvhrrColFin,                  // int(1)
                        // Sortie
                        &latitude,                     // double(1)
                        &longitude,                    // double(1)
                        &Ang_mesY,                     // double(1)
                        &Ang_mesZ,                     // double(1)
                        &Azim_sol,                     // double(1)
                        &Elev_sol,                     // double(1)
                       	&PsfCentreAvCol,               // double(1)
                        &PsfCentreAvLin,               // double(1)
                        &EarthSatDist);                // double(1)	
			// SD_BAS_LibererOperateur();
		}
		else
		{
  		 	DateFirstLineAvhrr = (double)IasiTime.day 
  		 				+ ( (double)(IasiTime).millis / (1000.*86400.) );							
			TimeFirst = (double)IasiTime.day 
						+ ( ((double)(IasiTime).millis - SD_ALG_DELTA_TIME) / (1000.*86400.) );
			TimeLast = (double)IasiTime.day
						+ ( ((double)(IasiTime).millis + SD_ALG_DELTA_TIME) / (1000.*86400.) );
              			
  			SD_BAS_ReserverOperateur();	  			
  			cr_op= SD_BAS_MetopAVHRR_SansAVHRR(
                    	// Entrees
                        ctx,             // t_SD_BAS_DataThread*
                        1,               // 1 seul point
                        &IasiTime,       // t_SD_BAS_SCDSTime*
                        MphrN1,          // t_SD_BAS_MPHR*
                        //PsfCentreMetop
                        &PsfCentreMetop[SD_ALG_AZIM],  // double(1)
                        &PsfCentreMetop[SD_ALG_ELEV],  // double(1)
                        TimeFirst,
                        TimeLast,  
                        DateFirstLineAvhrr,                           	
                        // Sorties
                        &PsfCentreAvLin,  // double(1)
                        &PsfCentreAvCol); // double(1)
			SD_BAS_LibererOperateur();

		}
		// ...FT-2997

  		if( cr_op != CMN__STATUT_OK)
  		{
	    	cr_op=   SD_ALG_AVHRR_ERR_METOP2AVHRR;
    		msg_err= (char *)SD_BAS_MSG_ERR_METOP2AVHRR;
    		num_lig= __LINE__;
     		goto fin;
  		}
  
  		//-------------------------------------------------------------------------

		//-----------------------
  		//@ Ajout de l'offset IIS-Avhrr
  		//@ PsfCentreAvLin = PsfCentreAvLin + OffsetIISAvhrr(2)
 		//@ PsfCentreAvCol = PsfCentreAvCol + OffsetIISAvhrr(1)
		//-----------------------
   		PsfCentreAvLin= PsfCentreAvLin + OffsetIISAvhrr[IND1(2)];
  		PsfCentreAvCol= PsfCentreAvCol + OffsetIISAvhrr[IND1(1)];

		//-----------------------
  		//@ Calcul des coordonnees du centre + offset dans le repere Metop
  		//@ PsfCentreAvMetop = GeomAvhrrMetop{ IasiTime, PsfCentreAvLin , PsfCentreAvCol }
		//-----------------------
		// FT-2997
		// si on peut utiliser les donnees AVHRR
		if (CCSMode == SD_ALG_USE_AVHRR_DATA)
		{
			// SD_BAS_ReserverOperateur();			  	
		  	cr_op= SD_BAS_AVHRRMetop(
		                            // Entrees
		                            ctx,               // t_SD_BAS_DataThread*
		                            &IasiTime,                       // t_SD_BAS_SCDSTime
		                            MphrN1,                         // t_SD_BAS_MPHR*
		                            1,                              // 1 seul point
		                            AvhrrListeLin,                  // t_SD_BAS_LigneAVHRR * ( AvhrrNbLin)
		                            AvhrrNbLin,                     // int(1)
		                            &PsfCentreAvCol,                 // double(1)
		                            &PsfCentreAvLin,                 // double(1)
		                            // Sorties
		                            &PsfCentreAvMetop[SD_ALG_AZIM],  // double(1)
		                            &PsfCentreAvMetop[SD_ALG_ELEV]); // double(1)			
			// SD_BAS_LibererOperateur();
		}
		else
		{
			SD_BAS_ReserverOperateur();			  	
		  	cr_op= SD_BAS_AVHRRMetop_SansAVHRR(
		                            			// Entrees
		                            			ctx,               	// t_SD_BAS_DataThread*
		                            			1,                 	// 1 seul point
		                            			&IasiTime,			// t_SD_BAS_SCDSTime
		                            			MphrN1,             // t_SD_BAS_MPHR*
		                            			&PsfCentreAvLin,	// double(1)
		                            			&PsfCentreAvCol,	// double(1)
		                            			DateFirstLineAvhrr,
		                            			// Sorties
		                            			&PsfCentreAvMetop[SD_ALG_AZIM],  // double(1)
		                            			&PsfCentreAvMetop[SD_ALG_ELEV]); // double(1)	
			SD_BAS_LibererOperateur();
		}  	
		// ...FT-2997

		if( cr_op != CMN__STATUT_OK)
	  	{
	    	cr_op=   SD_ALG_AVHRR_ERR_METOP2AVHRR;
	    	msg_err= (char *)SD_BAS_MSG_ERR_METOP2AVHRR;
	    	num_lig= __LINE__;
	    	goto fin;
	  	}

		//-----------------------
  		//@ Calcul des coordonnees du centre + offset dans le repere IASI
  		//@ [YPsfCentreAv, ZPsfCentreAv] = GeomMetopIasi{ MatRot, ScanAngle, PsfCentreAvMetop }
		//-----------------------
 		cr_op= SD_ALG_GeomMetopIasi(
        							// Entrees
                                	MatRot,             // double( SD_BAS_DIM_ROT_MATRICE * SD_BAS_DIM_ROT_MATRICE)
                                	ScanAngle,          // double(1)
                                	PsfCentreAvMetop,   // double(2) 0:Azim ; 1:Elev
                               		// Sorties
                               		&YPsfCentreAv,       // double(1)
                               		&ZPsfCentreAv);      // double(1)

		//-----------------------
  		//@ Calcul de l'offset IIS-Avhrr dans le repere IASI
  		//@ OffsetIISAvhrrRI(1) = YPsfCentreAv
  		//@ OffsetIISAvhrrRI(2) = ZPsfCentreAv  		
		//-----------------------
  		OffsetIISAvhrrRI[IND1(1)]= YPsfCentreAv;
  		OffsetIISAvhrrRI[IND1(2)]= ZPsfCentreAv;

		//-----------------------
  		//@ Calcul de l'offset sondeur-Avhrr dans le repere IASI
  		//@ OffsetSondAvhrr(1) = OffsetSondIIS(1) + OffsetIISAvhrrRI(1)
  		//@ OffsetSondAvhrr(2) = OffsetSondIIS(2) + OffsetIISAvhrrRI(2)
		//-----------------------
  		OffsetSondAvhrr[IND1(1)]= OffsetSondIIS[IND1(1)] + OffsetIISAvhrrRI[IND1(1)];
  		OffsetSondAvhrr[IND1(2)]= OffsetSondIIS[IND1(2)] + OffsetIISAvhrrRI[IND1(2)];

fin:
  		// Fin de la fonction CalcOffset

   		// Traitement des erreurs
    	//================================================================
    	if ( cr_op != CMN__STATUT_OK )
    	{
        	// mise a jour de la variable ctx->cr.errinterne
        	SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
        	SD_BAS_ERR_SET_AVHRR,
        	cr_op,
        	tcProcName_F,
        	num_lig,
        	msg_err);

        	// on imprime le message d'erreur sur la sortie standard
        	#ifdef OPS_DEBUG
	            SD_BAS_PRINT_ERR_INT(ctx->cr.errinterne.nomfonction,
    	                             ctx->cr.errinterne.numligne,
        	                         ctx->cr.errinterne.libelle);
        	#endif
    	}

  		#ifdef OPS_DEBUG
    		printf( "Fin de la fonction CalcOffset\n----------------------------------\n");
  		#endif

  		// sortie de la fonction
  		return( cr_op);
	}
	// FIN CalcOffset ================================================================
	// ===============================================================================



//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_IpsfToAvhrr
//@
//@ objet :  Traitement de l'image AVHRR
//@
//@
//@ Parametres :
//@         (ENTREE) Contexte_courant . : contexte de l'etape (numero de ligne SN traite)
//@         (ENTREE) MPHRN1 ........... :  MPHR produit N1
//@         (ENTREE) AvhrrListeLin .... : Suite de lignes AVHRR chargees en memoire
//@         (ENTREE) AvhrrNbLin ....... : Taille de la liste de lignes
//@         (ENTREE) AvhrrLinDeb ...... : Ligne de debut
//@         (ENTREE) AvhrrLinFin ...... : Ligne de fin
//@         (ENTREE) AvhrrColDeb ...... : Colonne de debut
//@         (ENTREE) AvhrrColFin ...... : Colonne de fin
//@
//@         (ENTREE) IasiTime ...... : Date de la visee courante
//@         (ENTREE) ScanAngle ..... : Angle de visee IASI
//@         (ENTREE) MatRot ........ : Matrice de rotation IASI-METOP
//@         (ENTREE) NbCol ......... : Nombre de colonnes des IPSF
//@         (ENTREE) NbLin ......... : Nombre de lignes des IPSF
//@         (ENTREE) YPsf .......... : Positions des points IPSF dans le repere IASI nominal instrument selon Y
//@         (ENTREE) ZPsf .......... : Positions des points IPSF dans le repere IASI nominal instrument selon Z
//@         (ENTREE) WgtPsf ........ : Poids associes aux points des IPSF
//@         (ENTREE) Osf ........... : Facteur de sur-echantillonnage de l'IPSF
//@         (ENTREE) OffsetIISAvhrr  : Offset Imageur-AVHRR dans le repere AVHRR
//@         (ENTREE) OffsetSondIIS . : Offset entre le sondeur IASI et l'imageur IIS
//@         (ENTREE) OffsetSondAvhrr : Offset entre le sondeur IASI et l'imageur AVHRR
//@         (ENTREE) dl ............ : Increment pour le sous-echantillonage de l'image AVHRR en ligne 
//@         (ENTREE) dc ............ : Increment pour le sous-echantillonage de l'image AVHRR en colonne
//@
//@         (SORTIE) ColAvFovFirst . : Premiere colonne de l'image IPSF/AVHRR
//@         (SORTIE) ColAvFovLast .. : Derniere colonne de l'image IPSF/AVHRR
//@         (SORTIE) LinAvFovFirst . : Premiere ligne de l'image IPSF/AVHRR
//@         (SORTIE) LinAvFovLast .. : Derniere ligne de l'image IPSF/AVHRR
//@         (SORTIE) NbPixAvFov .... : Nombre de pixels de l'image IPSF/AVHRR
//@         (SORTIE) AvhrrYpsf ..... : Positions des points IPSF/AVHRR dans le repere IASI nominal instrument selon Y
//@         (SORTIE) AvhrrZpsf ..... : Positions des points IPSF/AVHRR dans le repere IASI nominal instrument selon Z
//@         (SORTIE) AvhrrWgtpsf ... : Poids associes aux points des IPSF/AVHRR
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_IpsfToAvhrr(
  // Entrees pour MetopAVHRR
  t_SD_BAS_DataThread  *ctx,   // t_SD_BAS_DataThread*
  t_SD_BAS_MPHR        *MphrN1,             // t_SD_BAS_MPHR*
  t_SD_BAS_LigneAVHRR **AvhrrListeLin,      // t_SD_BAS_LigneAVHRR * ( AvhrrNbLin)
  int                   AvhrrNbLin,         // int(1)
  int                   AvhrrLinDeb,        // int(1)
  int                   AvhrrLinFin,        // int(1)
  int                   AvhrrColDeb,        // int(1)
  int                   AvhrrColFin,        // int(1)
  // Entrees
  t_SD_BAS_SCDSTime     IasiTime,           // t_SD_BAS_SCDSTime*
  double                ScanAngle,          // double(1)
  double               *MatRot,             // double( SD_BAS_DIM_ROT_MATRICE * SD_BAS_DIM_ROT_MATRICE)
  int                   NbCol,              // int(1)
  int                   NbLin,              // int(1)
  double               *YPsf,               // double( SD_BAS_NB_COL_SOND)
  double               *ZPsf,               // double( SD_BAS_NB_LIN_SOND)
  double               *WgtPsf,             // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  int                   Osf,                // int(1)
  double               *OffsetIISAvhrr,     // double( SD_BAS_DEUX)
  double               *OffsetSondIIS,      // double( SD_BAS_DEUX)
  double               *OffsetSondAvhrr,    // double( SD_BAS_DEUX)
  int                   dl,                 // int(1)
  int                   dc,                 // int(1)
  // Sorties
  int                  *ColAvFovFirst,      // int(1)
  int                  *ColAvFovLast,       // int(1)
  int                  *LinAvFovFirst,      // int(1)
  int                  *LinAvFovLast,       // int(1)
  int                  *NbPixAvFov,         // int(1)
  double               *AvhrrYpsf,          // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  double               *AvhrrZpsf,          // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  double               *AvhrrWgtpsf         // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
)
{
  // Transformation de l'IPSF dans le rasteur AVHRR
  static const char tcProcName_F[] = "SD_ALG_IpsfToAvhrr";
  t_CMN__Statut cr_op=CMN__STATUT_OK;

  // Gestion de erreurs
/* bma intialisation */
/*
  char *msg_err;                     // Message d'erreur
  int   num_lig;                     // Ligne d'erreur
*/
  char *msg_err = (char*) NULL;                     // Message d'erreur
  int   num_lig = 0;                     // Ligne d'erreur
/* fin bma */

  int     i;                         // Index general
  int     Ipsf;                      // Index Ipsf
  int     Ipix;                      // Index pixels
  int     p;                         // Index pixels
  int     PixAv;                     // Index pixels AVHRR
  int     il;                        // Index lignes
  int     ic;                        // Index colonnes
  int     Ilin;                      // Index lignes
  int     Icol;                      // Index colonnes
  int     LinAvFov;                  // Index lignes AVHRR
  int     ColAvFov;                  // Index colonnes AVHRR

  double  DYpsf, DZpsf;              // Variations de coordonnees
  double  IpsfCornerY[4];    // 4 coins selon Y
  double  IpsfCornerZ[4];    // 4 coins selon Z
  double  MinYPsf, MaxYPsf;          // Valeurs extremes des 4 coins selon Y
  double  MinZPsf, MaxZPsf;          // Valeurs extremes des 4 coins selon Z
  double  MinAvLin, MaxAvLin;        // Valeurs extremes selon les lignes AVHRR
  double  MinAvCol, MaxAvCol;        // Valeurs extremes selon les colonnes AVHRR

  int     nls;                       // Nombre de lignes de la sousgrille de l'imagette AVHRR
  int     ncs;                       // Nombre de colonnes de la sousgrille de l'imagette AVHRR
  double *ls;                        // Lignes de la sousgrille de l'imagette AVHRR (taille nls)
  double *cs;                        // Colonnes de la sousgrille de l'imagette AVHRR (taille ncs)
  int     TailleUtile;               // Colonnes x Lignes
  double *ImgAvLinSubG;              // Coordonnees lignes des pixels de la sousgrille dans le repere AVHRR
  double *ImgAvColSubG;              // Coordonnees colonnes des pixels de la sousgrille dans le repere AVHRR
  double *ImgAvMetopSubG;            // Azimuth et Elevation dans le repere Metop
  double *ImgYpsf;                   // Image temporaire selon Y
  double *ImgZpsf;                   // Image temporaire selon Z
  double *ImgWgtpsf;                 // Poids temporaires
  double *ImgAvYSubG;                // Image temporaire sous echantillonnee selon Y
  double *ImgAvZSubG;                // Image temporaire sous echantillonnee selon Z
  double *ImgAvY;                    // Image AVHRR temporaire selon Y
  double *ImgAvZ;                    // Image AVHRR temporaire selon Z
  double *ratio;                     // Rapports lignes/colonnes des pixels interpoles dans la grille
  double  ImgAvDY;                   // Sens de variation selon Y
  double  ImgAvDZ;                   // Sens de variation selon Z
  int     Nl;                        // Nombre de lignes de l'image a interpoler
  int     Nc;                        // Nombre de colonnes de l'image a interpoler
  double *lin = NULL;                       // Position des lignes a interpoler
  double *col = NULL;                       // Position des colonnes a interpoler
  double  AzimElev[2];               // Couple Azimuth/Elevation
  double  PsfCornerMetop[4*SD_BAS_DEUX]; // Position des coins dans le repere Metop
  double  PsfCornerAvCol[4]; // Position des coins selon les colonnes AVHRR
  double  PsfCornerAvLin[4]; // Position des coins selon les lignes AVHRR
  int     NbLinAvFov;                // Nombre de lignes AVHRR
  int     NbColAvFov;                // Nombre de colonnes AVHRR


  // Sorties de SD_BAS_MetopAVHRR
  double latitude[4];        // Latitudes
  double longitude[4];       // Longitudes
  double Ang_mesY[4];        // Angles de visees selon Y
  double Ang_mesZ[4];        // Angles de visees selon Z
  double Azim_sol[4];        // Azimuth par rapport au sol
  double Elev_sol[4];        // Elevation par rapport au sol
  double EarthSatDist[4];    // Distance Terre/Satellite dans la conversion Metop2AVHRR


  #ifdef OPS_DEBUG
    printf( "Debut de la fonction IpsfToAvhrr\n----------------------------------\n");
  #endif
  ImgAvLinSubG=NULL;
  ImgAvColSubG=NULL;
  ImgAvMetopSubG=NULL;
  ls=NULL;
  cs=NULL;
  ImgAvYSubG=NULL;
  ImgAvZSubG=NULL;
  ImgAvY=NULL;
  ImgAvZ=NULL;
  ratio=NULL;
  ImgYpsf=NULL;
  ImgZpsf=NULL;
  ImgWgtpsf=NULL;
  col=NULL;
  lin=NULL;

  // Bornes des Y et Z
  MinYPsf= MAX_DOUBLE;
  MaxYPsf= MIN_DOUBLE;
  MinZPsf= MAX_DOUBLE;
  MaxZPsf= MIN_DOUBLE;
  // FT-2773
  for( Ipsf= 1; Ipsf <= NbCol; Ipsf++)
  // ...FT-2773
  {
    if( YPsf[IND1(Ipsf)] > MaxYPsf)   MaxYPsf= YPsf[IND1(Ipsf)];
    if( YPsf[IND1(Ipsf)] < MinYPsf)   MinYPsf= YPsf[IND1(Ipsf)];
  }
//jlp52
  // FT-2773
  for( Ipsf= 1; Ipsf <= NbLin; Ipsf++)
  // ...FT-2773
  {
    if( ZPsf[IND1(Ipsf)] > MaxZPsf)   MaxZPsf= ZPsf[IND1(Ipsf)];
    if( ZPsf[IND1(Ipsf)] < MinZPsf)   MinZPsf= ZPsf[IND1(Ipsf)];
  }

  // Ecart entre 2 points de discretisation Ipsf
  // DYpsf = YPsf(2) - YPsf(1)
  // DZpsf = ZPsf(2) - ZPsf(1)
  DYpsf= YPsf[IND1(2)] - YPsf[IND1(1)];
  DZpsf= ZPsf[IND1(2)] - ZPsf[IND1(1)];

  // Calcul des coordonnees des 4 coins de l'Ipsf dans le repere IASI
  // corrigees de l'offset sondeur-imageur

  // IpsfCornerY[1] = IpsfCornerY[4] = Min{ YPsf } - DYpsf/2 + OffsetSondIIS(1)
  IpsfCornerY[IND1(1)]= MinYPsf - DYpsf / 2.0 + OffsetSondIIS[IND1(1)];
  IpsfCornerY[IND1(4)]= IpsfCornerY[IND1(1)];

  // IpsfCornerY[2] = IpsfCornerY[3] = Max{ YPsf } + DYpsf/2 + OffsetSondIIS(1)
  IpsfCornerY[IND1(2)]= MaxYPsf + DYpsf / 2.0 + OffsetSondIIS[IND1(1)];
  IpsfCornerY[IND1(3)]= IpsfCornerY[IND1(2)];

  // IpsfCornerZ[1] = IpsfCornerZ[2] = Min{ ZPsf } - DZpsf/2 + OffsetSondIIS(2)
  IpsfCornerZ[IND1(1)]= MinZPsf - DZpsf / 2.0 + OffsetSondIIS[IND1(2)];
  IpsfCornerZ[IND1(2)]= IpsfCornerZ[IND1(1)];

  // IpsfCornerZ[3] = IpsfCornerZ[4] = Max{ ZPsf } + DZpsf/2 + OffsetSondIIS(2)
  IpsfCornerZ[IND1(3)]= MaxZPsf + DZpsf / 2.0 + OffsetSondIIS[IND1(2)];
  IpsfCornerZ[IND1(4)]= IpsfCornerZ[IND1(3)];


  // Calcul des coordonnees des 4 coins dans le repere Metop
  // PsfCornerMetop = GeomIasiMetop{ MatRot, ScanAngle, IpsfCornerY, IpsfCornerZ }
//jlp52
  for( i= 1; i <= 4; i++)
  {
    cr_op= SD_ALG_GeomIasiMetop(
                                 // Entrees
                                  MatRot,                 // double( SD_BAS_DIM_ROT_MATRICE * SD_BAS_DIM_ROT_MATRICE)
                                  ScanAngle,              // double(1)
                                  IpsfCornerY[IND1(i)],   // double(1)
                                  IpsfCornerZ[IND1(i)],   // double(1)
                                 // Sorties
                                 &PsfCornerMetop[i-1],    // double(1)
                                 &PsfCornerMetop[i+3]);   // double(1)
  }

  // Calcul des coordonnees des 4 coins dans le repere Avhrr
  // [PsfCornerAvLin, PsfCornerAvCol] = GeomMetopAvhrr{ IasiTime, PsfCornerMetop }
//  SD_BAS_ReserverOperateur();
  cr_op= SD_BAS_MetopAVHRR(
                            // Entree
                             ctx,             // t_SD_BAS_DataThread*
                            &IasiTime,                     // t_SD_BAS_SCDSTime*
                             MphrN1,                       // t_SD_BAS_MPHR*
//jlp52
                               4,                    // 4 coins
                             PsfCornerMetop + 0,           // double( SD_BAS_PN)  4 azimuths
                             PsfCornerMetop + SD_BAS_PN,   // double( SD_BAS_PN)  4 elevations
                             AvhrrLinDeb,                  // int(1)
                             AvhrrLinFin,                  // int(1)
                             AvhrrListeLin,                // t_SD_BAS_LigneAVHRR * ( AvhrrNbLin)
                             AvhrrNbLin,                   // int(1)
                             AvhrrColDeb,                  // int(1)
                             AvhrrColFin,                  // int(1)
                            // Sortie
                             latitude,                     // double( SD_BAS_PN)
                             longitude,                    // double( SD_BAS_PN)
                             Ang_mesY,                     // double( SD_BAS_PN)
                             Ang_mesZ,                     // double( SD_BAS_PN)
                             Azim_sol,                     // double( SD_BAS_PN)
                             Elev_sol,                     // double( SD_BAS_PN)
                             PsfCornerAvCol,               // double( SD_BAS_PN)
                             PsfCornerAvLin,               // double( SD_BAS_PN)
                             EarthSatDist);                // double( SD_BAS_PN)

//     SD_BAS_LibererOperateur();
  if( cr_op != CMN__STATUT_OK)
  {
    cr_op=   SD_ALG_AVHRR_ERR_METOP2AVHRR;
    msg_err= (char *)SD_BAS_MSG_ERR_METOP2AVHRR;
    num_lig= __LINE__;
    goto fin;
  }
  //-------------------------------------------------------------------------


  // Bornes des lignes et colonnes AVHRR
  MinAvLin= MAX_DOUBLE;
  MaxAvLin= MIN_DOUBLE;
  MinAvCol= MAX_DOUBLE;
  MaxAvCol= MIN_DOUBLE;
//jlp52
    for( Ipsf= 1; Ipsf <= 4; Ipsf++)
  {
    if( PsfCornerAvLin[IND1(Ipsf)] > MaxAvLin)   MaxAvLin= PsfCornerAvLin[IND1(Ipsf)];
    if( PsfCornerAvLin[IND1(Ipsf)] < MinAvLin)   MinAvLin= PsfCornerAvLin[IND1(Ipsf)];
    if( PsfCornerAvCol[IND1(Ipsf)] > MaxAvCol)   MaxAvCol= PsfCornerAvCol[IND1(Ipsf)];
    if( PsfCornerAvCol[IND1(Ipsf)] < MinAvCol)   MinAvCol= PsfCornerAvCol[IND1(Ipsf)];
  }

  // Calcul du domaine Avhrr utile recouvrant le Fov (theorique + marges tenant compte de l'Offset IIS-Avhrr): determination des numeros de lignes et colonnes de debut et de fin
  // LinAvFovFirst = PartieEntiere{ Min{ PsfCornerAvLin }} 
    *LinAvFovFirst= (int)( MinAvLin);

  // LinAvFovEnd = PartieEntiere{ Max{ PsfCornerAvLin }} + 1
    *LinAvFovLast= (int)( MaxAvLin) + 1;

  // ColAvFovFirst = PartieEntiere{ Min{ PsfCornerAvCol }} 
    *ColAvFovFirst= (int)( MinAvCol);

  // ColAvFovEnd = PartieEntiere{ Max{ PsfCornerAvCol }} + 1
    *ColAvFovLast= (int)( MaxAvCol) + 1;


  // Optimisation des marges
  // Si (OffsetIISAvhrr(1) > 0) alors
  if( OffsetIISAvhrr[IND1(1)] > 0.0)
  {
    //   ColAvFovEnd = ColAvFovEnd + PartieEntiere{OffsetIISAvhrr(1)} +1
      *ColAvFovLast= *ColAvFovLast + (int)( OffsetIISAvhrr[IND1(1)]) + 1;
  }
  else
  {
    //   ColAvFovFirst = ColAvFovFirst - PartieEntiere{OffsetIISAvhrr(1)} -1
      *ColAvFovFirst= *ColAvFovFirst - (int)( OffsetIISAvhrr[IND1(1)]) - 1;
  } // Fin de si

  // Si (OffsetIISAvhrr(2) > 0) alors
  if( OffsetIISAvhrr[IND1(2)] > 0.0)
  {
    //   LinAvFovEnd = LinAvFovEnd + PartieEntiere{OffsetIISAvhrr(2)} +1
      *LinAvFovLast= *LinAvFovLast + (int)( OffsetIISAvhrr[IND1(2)]) + 1;
  }
  else
  {
    //   LinAvFovFirst = LinAvFovFirst - PartieEntiere{OffsetIISAvhrr(2)} -1
      *LinAvFovFirst= *LinAvFovFirst - (int)( OffsetIISAvhrr[IND1(2)]) - 1;
  } // Fin de si

  // NbLinAvFov = LinAvFovEnd - LinAvFovFirst + 1
  NbLinAvFov= *LinAvFovLast - *LinAvFovFirst + 1;

  // NbColAvFov = ColAvFovEnd - ColAvFovFirst + 1
  NbColAvFov= *ColAvFovLast - *ColAvFovFirst + 1;

  // NbPixAvFov = NbLinAvFov * NbColAvFov
  *NbPixAvFov= NbLinAvFov * NbColAvFov;

  // Coordonnees des pixels de l'imagette Avhrr utile dans le repere plan focal sondeur

  // Ipix = 0
  Ipix= 0;

  // 1/ Coordonnees dans le repere IIS des pixels d'une sous-grille de l'imagette Avhrr
  //    utile Definition d'une sous-grille de l'imagette Avhrr utile
  //    (1 point sur dl + dernier point en ligne, 1 point sur dc + dernier point en colonne)

  // nls = NbLinAvFov ; ncs = NbColAvFov
  nls= (ceil)(1. + (double)(NbLinAvFov-1)/(double)dl);
  ncs= (ceil)(1. + (double)(NbColAvFov-1)/(double)dc);
  TailleUtile= nls * ncs;
 if (nls < 2 || ncs < 2)
  {
    cr_op   = SD_ALG_AVHRR_41_CCS_ALRT;
    msg_err = (char *)SD_BAS_MSG_ERR_SAMPLING;
    num_lig = __LINE__;
    goto fin;
  }

  Nl = NbLinAvFov;
  Nc = NbColAvFov;
     //@ Coordonnees des pixels de la sousgrille dans le repere Avhrr avec correction inverse de l'offset
  // ----------------------------------------------------------------------------------------------------
  // Allocation de la memoire
 
  if(    (SD_BAS_MacMALLOC( ImgAvLinSubG,   double, TailleUtile)           == FALSE)
      || (SD_BAS_MacMALLOC( ImgAvColSubG,   double, TailleUtile)           == FALSE)
      || (SD_BAS_MacMALLOC( ImgAvMetopSubG, double, 2*TailleUtile)         == FALSE)
      || (SD_BAS_MacMALLOC( ls,             double, nls)                   == FALSE)
      || (SD_BAS_MacMALLOC( cs,             double, ncs)                   == FALSE)
      || (SD_BAS_MacMALLOC( ImgAvYSubG,     double, TailleUtile)           == FALSE)
      || (SD_BAS_MacMALLOC( ImgAvZSubG,     double, TailleUtile)           == FALSE)
      || (SD_BAS_MacMALLOC( ImgAvY,         double, Nl*Nc)                 == FALSE)
      || (SD_BAS_MacMALLOC( ImgAvZ,         double, Nl*Nc)                 == FALSE)
      || (SD_BAS_MacMALLOC( ratio,          double, 2*Nl*Nc)               == FALSE)
      || (SD_BAS_MacMALLOC( ImgYpsf,        double, NbLinAvFov*NbColAvFov) == FALSE)
      || (SD_BAS_MacMALLOC( ImgZpsf,        double, NbLinAvFov*NbColAvFov) == FALSE)
      || (SD_BAS_MacMALLOC( ImgWgtpsf,      double, NbLinAvFov*NbColAvFov) == FALSE)
      || (SD_BAS_MacMALLOC( col,            double, Nc)                    == FALSE)
      || (SD_BAS_MacMALLOC( lin,            double, Nl)                    == FALSE)
      )
  {
    cr_op=   SD_ALG_AVHRR_ERR_ALLOCATION;
    msg_err= (char *)SD_BAS_MSG_ALLOC;
    num_lig= __LINE__;
    goto fin;
  }

  LinAvFov= *LinAvFovFirst;
  for( il= 1; il <= Nl; il++)
  {
    lin[IND1(il)]= LinAvFov+il-1;
  }

  ColAvFov= *ColAvFovFirst;
  for( ic= 1; ic <= Nc; ic++)
  {
    col[IND1(ic)]= ColAvFov+ic-1;
  }

  // Coordonnees des pixels de la sous-grille dans le repere Avhrr avec correction inverse de l'offset
  // Lignes : ls (nls) = {LinAvFovFirst, LinAvFovFirst+dl, LinAvFovFirst+2*dl, ..., LinAvFovEnd}
  LinAvFov= *LinAvFovFirst;
  for( il= 1; il <= nls -1; il++)
  {
    ls[IND1(il)]= LinAvFov;
    LinAvFov= LinAvFov + dl;
  }
   ls[IND1(nls)]= *LinAvFovLast;
  
  // Colonnes : cs (ncs) = {ColAvFovFirst,ColAvFOvFirst+dc, ColAvFovFirst+2*dc,..., ColAvFovEnd}
  ColAvFov= *ColAvFovFirst;
  for( ic= 1; ic <= ncs -1; ic++)
  {
    cs[IND1(ic)]= ColAvFov;
    ColAvFov= ColAvFov + dc;
  }
    cs[IND1(ncs)]= *ColAvFovLast;
  
  // Boucle sur les lignes ( il = 1 a nls )
  for( il= 1; il <= nls; il++)
  {
    // Boucle sur les colonnes ( ic = 1 a ncs)
    for( ic= 1; ic <= ncs; ic++)
    {
      // ImgAvLinSubG(il,ic) = IntToReal{ ls(il) }
      ImgAvLinSubG[IND2(ic,il,ncs)]= (double)ls[IND1(il)];
      // ImgAvColSubG(il,ic) = IntToReal{ cs(ic) }
      ImgAvColSubG[IND2(ic,il,ncs)]= cs[IND1(ic)];
    } // Fin de boucle sur les colonnes 
  } // Fin de boucle sur les lignes 

  // Calcul des coordonnees des NbPixAvSubG pixels dans le repere Metop : p= (il, ic)
  // ImgAvMetopSubG(p) = GeomAvhrrMetop{ IasiTime, ImgAvLinSubG(p) , ImgAvColSubG(p) }
//  SD_BAS_ReserverOperateur();
  cr_op= SD_BAS_AVHRRMetop(
                            // Entrees
                             ctx,  // t_SD_BAS_DataThread*
                            &IasiTime,          // t_SD_BAS_SCDSTime
                             MphrN1,            // t_SD_BAS_MPHR*
                             TailleUtile,       // TailleUtile points
                             AvhrrListeLin,     // t_SD_BAS_LigneAVHRR * ( AvhrrNbLin)
                             AvhrrNbLin,        // int(1)
                             ImgAvColSubG,      // double( TailleUtile)
                             ImgAvLinSubG,      // double( TailleUtile)
                            // Sorties
                            &ImgAvMetopSubG[IND2(1,1,TailleUtile)],   // double( TailleUtile) Azimuths
                            &ImgAvMetopSubG[IND2(1,2,TailleUtile)]);  // double( TailleUtile) Elevations

//     SD_BAS_LibererOperateur();
  if( cr_op != CMN__STATUT_OK)
  {
    cr_op=   SD_ALG_AVHRR_ERR_METOP2AVHRR;
    msg_err= (char *)SD_BAS_MSG_ERR_METOP2AVHRR;
    num_lig= __LINE__;
    goto fin;
  }

  // Calcul des coordonnees des NbPixAvSubG pixels dans le repere IASI
  // [ ImgAvYSubG(p), ImgAvZSubG(p) ] = GeomMetopIasi{ MatRot, ScanAngle, ImgAvMetopSubG(p) }
  for( p= 1; p <= TailleUtile; p++)
  {
    AzimElev[SD_ALG_AZIM]= ImgAvMetopSubG[IND2(p,1,TailleUtile)];
    AzimElev[SD_ALG_ELEV]= ImgAvMetopSubG[IND2(p,2,TailleUtile)];

    cr_op= SD_ALG_GeomMetopIasi(
                                 // Entrees
                                  MatRot,       // double( SD_BAS_DIM_ROT_MATRICE * SD_BAS_DIM_ROT_MATRICE)
                                  ScanAngle,    // double(1)
                                  AzimElev,     // double(2) 0:Azim ; 1:Elev
                                 // Sorties
                                 &ImgAvYSubG[IND1(p)],   // double(1)
                                 &ImgAvZSubG[IND1(p)]);  // double(1)
  }

  // 2/ Coordonnees dans le repere IIS de tous les points de l'imagette Avhrr utile
  //    (calcul par interpolation lineaire pour la rapidite)

  // Boucle sur les lignes de l'imagette Avhrr utile( il = 1 a nls )
    // Boucle sur les colonnes de l'imagette Avhrr utile ( Icol = 1 a ncs)
      // ImgAvY(il,ic) = Intlin2D{ImgAvYSubG,ls,cs,nls,ncs,il,ic}
      // ImgAvZ(il,ic) = Intlin2D{ImgAvZSubG,ls,cs,nls,ncs,il,ic}
    // Fin de boucle sur les colonnes Avhrr
  // Fin de boucle sur les lignes Avhrr

  // Initialisation de l'interpolation en Y
  // Initialisation de l'interpolation en Y
  cr_op= SD_BAS_InterpLineaire2D(
                           // Entrees
			   ctx,
                            SD_ALG_INIT_INTERP2D,  // int(1)
                            ncs,                   // int(1)
                            nls,                   // int(1)
                            cs,                    // double( ncs)
                            ls,                    // double( nls)
                            ImgAvYSubG,            // double( ncs * nls)
                            Nc,                    // int(1)
                            Nl,                    // int(1)
                            col,                   // double( Nc)
                            lin,                   // double( Nl)
                           // Sorties
                            ImgAvY,                // double( Nc * Nl)
                           // Entrees / Sorties
                            ratio);                // double( 2 * Nc * Nl)
  if( cr_op != CMN__STATUT_OK)
  {
    cr_op   = SD_ALG_AVHRR_41_CCS_ALRT;
    msg_err = (char *)SD_BAS_MSG_ERR_INTERP;
    num_lig = __LINE__;
    goto fin;
  }
  // Calcul de l'interpolation en Y
  cr_op= SD_BAS_InterpLineaire2D(
                           // Entrees
			   ctx,
                            SD_ALG_CALCUL_INTERP2D,  // int(1)
                            ncs,                     // int(1)
                            nls,                     // int(1)
                            cs,                      // double( ncs)
                            ls,                      // double( nls)
                            ImgAvYSubG,              // double( ncs * nls)
                            Nc,                      // int(1)
                            Nl,                      // int(1)
                            col,                     // double( Nc)
                            lin,                     // double( Nl)
                           // Sorties
                            ImgAvY,                  // double( Nc * Nl)
                           // Entrees / Sorties
                            ratio);                  // double( 2 * Nc * Nl)

  if( cr_op != CMN__STATUT_OK)
  {
    cr_op   = SD_ALG_AVHRR_41_CCS_ALRT;
    msg_err = (char *)SD_BAS_MSG_ERR_INTERP;
    num_lig = __LINE__;
    goto fin;
  }
  // Initialisation de l'interpolation en Z
  cr_op= SD_BAS_InterpLineaire2D(
                           // Entrees
			   ctx,
                            SD_ALG_INIT_INTERP2D,  // int(1)
                            ncs,                   // int(1)
                            nls,                   // int(1)
                            cs,                    // double( ncs)
                            ls,                    // double( nls)
                            ImgAvZSubG,            // double( ncs * nls)
                            Nc,                    // int(1)
                            Nl,                    // int(1)
                            col,                   // double( Nc)
                            lin,                   // double( Nl)
                           // Sorties
                            ImgAvZ,                // double( Nc * Nl)
                           // Entrees / Sorties
                            ratio);                // double( 2 * Nc * Nl)
  if( cr_op != CMN__STATUT_OK)
  {
    cr_op   = SD_ALG_AVHRR_41_CCS_ALRT;
    msg_err = (char *)SD_BAS_MSG_ERR_INTERP;
    num_lig = __LINE__;
    goto fin;
  }

  // Calcul de l'interpolation en Z
  cr_op= SD_BAS_InterpLineaire2D(
                           // Entrees
			   ctx,
                            SD_ALG_CALCUL_INTERP2D,  // int(1)
                            ncs,                     // int(1)
                            nls,                     // int(1)
                            cs,                      // double( ncs)
                            ls,                      // double( nls)
                            ImgAvZSubG,              // double( ncs * nls)
                            Nc,                      // int(1)
                            Nl,                      // int(1)
                            col,                     // double( Nc)
                            lin,                     // double( Nl)
                           // Sorties
                            ImgAvZ,                  // double( Nc * Nl)
                           // Entrees / Sorties
                            ratio);                  // double( 2 * Nc * Nl)

  if( cr_op != CMN__STATUT_OK)
  {
    cr_op   = SD_ALG_AVHRR_41_CCS_ALRT;
    msg_err = (char *)SD_BAS_MSG_ERR_INTERP;
    num_lig = __LINE__;
    goto fin;
  }
  // Calcul de la dimension moyenne des pixels Avhrr 

  // Test sur les directions des lignes et colonnes Avhrr dans le repere Iasi plan focal [Y,Z]
  // Si abs{ ImgAvZ(1,2) - ImgAvZ(1,1)} < abs{ ImgAvY(1,2) - ImgAvY(1,1)} alors
  if(   fabs( ImgAvZ[IND2(2,1,Nc)] - ImgAvZ[IND2(1,1,Nc)])
      < fabs( ImgAvY[IND2(2,1,Nc)] - ImgAvY[IND2(1,1,Nc)]))
  {
    // ImgAvDY =   [ ImgAvY(1,NbColAvFov) - ImgAvY(1,1) 
    //           + ImgAvY(NbLinAvFov,NbColAvFov) - ImgAvY(NbLinAvFov,1) ]
    //           / [ 2 NbColAvFov ]
    ImgAvDY=   (   ImgAvY[IND2(NbColAvFov,1,Nc)]
                 - ImgAvY[IND2(1,1,Nc)]
                 + ImgAvY[IND2(NbColAvFov,NbLinAvFov,Nc)]
                 - ImgAvY[IND2(1,NbLinAvFov,Nc)])
//             / ( 2.0 * NbColAvFov);
// PM 3 version 5.0
             / ( 2.0 * (NbColAvFov-1));
	      
    // ImgAvDZ =   [ ImgAvZ(NbLinAvFov,1) - ImgAvZ(1,1) 
    //           + ImgAvZ(NbLinAvFov,NbColAvFov) - ImgAvZ(1,NbColAvFov) ]
    //           / [ 2 NbLinAvFov ]
    ImgAvDZ=   (   ImgAvZ[IND2(1,NbLinAvFov,Nc)]
                 - ImgAvZ[IND2(1,1,Nc)]
                 + ImgAvZ[IND2(NbColAvFov,NbLinAvFov,Nc)]
                 - ImgAvZ[IND2(NbColAvFov,1,Nc)])
//             / ( 2.0 * NbLinAvFov);
// PM 3 version 5.0
             / ( 2.0 * (NbLinAvFov-1));
  }
  else
  {
    // ImgAvDY =   [ ImgAvY(NbLinAvFov,1) - ImgAvY(1,1) 
    //           + ImgAvY(NbLinAvFov,NbColAvFov) - ImgAvY(1,NbColAvFov) ]
    //           / [ 2 NbLinAvFov ]
    ImgAvDY=   (   ImgAvY[IND2(1,NbLinAvFov,Nc)]
                 - ImgAvY[IND2(1,1,Nc)]
                 + ImgAvY[IND2(NbColAvFov,NbLinAvFov,Nc)]
                 - ImgAvY[IND2(NbColAvFov,1,Nc)])
//             / ( 2.0 * NbLinAvFov);
//
// PM 3 version 5.0
	    /	 ( 2.0 * (NbLinAvFov-1));
    // ImgAvDZ =   [ ImgAvZ(1,NbColAvFov) - ImgAvZ(1,1) 
    //           + ImgAvZ(NbLinAvFov,NbColAvFov) - ImgAvZ(NbLinAvFov,1) ]
    //           / [ 2 NbColAvFov ]
    ImgAvDZ=   (   ImgAvZ[IND2(NbColAvFov,1,Nc)]
                 - ImgAvZ[IND2(1,1,Nc)]
                 + ImgAvZ[IND2(NbColAvFov,NbLinAvFov,Nc)]
                 - ImgAvZ[IND2(1,NbLinAvFov,Nc)])
//             / ( 2.0 * NbColAvFov);
//
// PM 3 version 5.0
             / ( 2.0 * (NbColAvFov-1));
  } // Fin de si

  // Transformation de l'Ipsf dans le raster Avhrr avec prise en compte de l'offset sondeur imageur
  // [ ImgWgtpsf, ImgYpsf, ImgZpsf ] = 
  // TransformIpsf{ NbLin, NbCol, YPsf, ZPsf, Wgtpsf, 1, 1, NbLinAvFov, NbColAvFov, ImgAvY, ImgAvZ, ImgAvDY, ImgAvDz, OffsetSondAvhrr , OverSampFactor }
  cr_op= SD_ALG_TransformIpsf_CCS(
                               // Entrees
                                NbLin,             // int(1)
                                NbCol,             // int(1)
                                YPsf,              // double( SD_BAS_NB_COL_SOND) Limite a NbCol
                                ZPsf,              // double( SD_BAS_NB_LIN_SOND) Limite a NbLin
                                WgtPsf,            // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
                                1,                 // int(1)
                                1,                 // int(1)
                                NbLinAvFov,        // int(1)
                                NbColAvFov,        // int(1)
			        Nc,		   // nb col ImgY et ImgZ,
                                ImgAvY,            // double( NbLinAvFov * NbColAvFov)
                                ImgAvZ,            // double( NbLinAvFov * NbColAvFov)
                                ImgAvDY,           // double(1)
                                ImgAvDZ,           // double(1)
                                OffsetSondAvhrr,   // double(1)
                                (double)Osf,               // double(1)
                               // Sorties
                                ImgWgtpsf,         // double( NbLinAvFov * NbColAvFov)
                                ImgYpsf,           // double( NbLinAvFov * NbColAvFov)  Alloue: double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
                                ImgZpsf);          // double( NbLinAvFov * NbColAvFov)  Alloue: double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)

  // Transformation de matrice en vecteur
  // PixAv = 0
  PixAv= 0;

  // Boucle sur les lignes Avhrr ( Ilin =1 a NbLinAvFov)
  for( Ilin= 1; Ilin <= NbLinAvFov; Ilin++)
  {
    // Boucle sur les colonnes Avhrr ( Icol =1 a NbColAvFov)
    for( Icol= 1; Icol <= NbColAvFov; Icol++)
    {
      // PixAv = PixAv+1
      PixAv= PixAv + 1;

      // AvhrrYpsf(PixAv) = ImgYpsf(Ilin,Icol)
      AvhrrYpsf[IND1(PixAv)]= ImgYpsf[IND2(Icol,Ilin,NbColAvFov)];

      // AvhrrZpsf(PixAv) = ImgZpsf(Ilin,Icol)
      AvhrrZpsf[IND1(PixAv)]= ImgZpsf[IND2(Icol,Ilin,NbColAvFov)];

      // AvhrrWgtpsf(PixAv) = ImgWgtpsf(Ilin,Icol)
      AvhrrWgtpsf[IND1(PixAv)]= ImgWgtpsf[IND2(Icol,Ilin,NbColAvFov)];

    } //Fin de boucle sur les colonnes Avhrr
  } // Fin de boucle sur les lignes Avhrr



fin:

// liberation memoire
  SD_BAS_MacFREE(ImgAvLinSubG);
  SD_BAS_MacFREE(ImgAvColSubG);
  SD_BAS_MacFREE(ImgAvMetopSubG);
  SD_BAS_MacFREE(ls);
  SD_BAS_MacFREE(cs);
  SD_BAS_MacFREE(ImgAvYSubG);
  SD_BAS_MacFREE(ImgAvZSubG);
  SD_BAS_MacFREE(ImgAvY);
  SD_BAS_MacFREE(ImgAvZ);
  SD_BAS_MacFREE(ratio);
  SD_BAS_MacFREE(ImgYpsf);
  SD_BAS_MacFREE(ImgZpsf);
  SD_BAS_MacFREE(ImgWgtpsf);
  SD_BAS_MacFREE(col);
  SD_BAS_MacFREE(lin);
    // Traitement des erreurs
    //================================================================
    if ( cr_op != CMN__STATUT_OK )
    {
        // mise a jour de la variable ctx->cr.errinterne
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
        SD_BAS_ERR_SET_AVHRR,
        cr_op,
        tcProcName_F,
        num_lig,
        msg_err);

        // on imprime le message d'erreur sur la sortie standard
        #ifdef OPS_DEBUG
            SD_BAS_PRINT_ERR_INT(ctx->cr.errinterne.nomfonction,
                                 ctx->cr.errinterne.numligne,
                                 ctx->cr.errinterne.libelle);
        #endif
    }

  // Liberation de tous les tableaux temporaires
  // MacFREE se charge de tester si la variable est a NULL
  // Fin de la fonction IpsfToAvhrr
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction IpsfToAvhrr\n----------------------------------\n");
  #endif
  
  // sortie de la fonction
  return( cr_op);
}
// FIN IpsfToAvhrr ===============================================================
// ===============================================================================

// FT-2528
//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_AvhrrCloudInfoIntoFov
//@
//@ objet :  Extraction de l'information nuageuse dans le FOV sondeur
//@
//@
//@ Parametres :
//@         (ENTREE) ProduitCourant ....... : Pour pointer sur les variables temporaires
//@         (ENTREE) LinImgFirst .......... : 1ere ligne AVHRR utile
//@         (ENTREE) ColImgFirst .......... : 1ere colonne AVHRR utile
//@         (ENTREE) NbLinImg ............. : Nombre de lignes AVHRR utiles
//@         (ENTREE) NbColImg ............. : Nombre de colonnes AVHRR utiles
//@         (ENTREE) NbPixImg ............. : Nombre des pixels de l'image IIS
//@         (ENTREE) CloudeInformation .... :
//@         (ENTREE) QualityIndicator ..... :
//@         (ENTREE) ScanLineQuality ...... :
//@         (ENTREE) CountErrorFrame ...... :
//@         (ENTREE) ColImgFovFirst ....... : Premiere colonne de l'image IPSF/AVHRR
//@         (ENTREE) ColImgFovLast ........ : Derniere colonne de l'image IPSF/AVHRR
//@         (ENTREE) LinImgFovFirst ....... : Premiere ligne de l'image IPSF/AVHRR
//@         (ENTREE) LinImgFovLast ........ : Derniere ligne de l'image IPSF/AVHRR
//@         (ENTREE) NbPixImgFov .......... : Nombre de pixels de l'image IPSF/AVHRR
//@         (ENTREE) ImgWgtpsf ............ : Poids associes aux points des IPSF/AVHRR
//@         (ENTREE) EUMAvhrrCldErrorCutoff : Seuil de comptage des erreurs
//@         (SORTIE) GEUMAvhrr1BCldFrac ... :
//@         (SORTIE) GEUMAvhrr1BLandFrac .. :
//@         (SORTIE) GEUMAvhrr1BQual ...... :
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
static t_CMN__Statut SD_ALG_AvhrrCloudInfoIntoFov(
// Entrees
  t_SD_BAS_DataThread  *ctx,                 // t_SD_BAS_DataThread*
  int LinImgFirst,                          // int(1)
  int ColImgFirst,                          // int(1)
  int NbLinImg,                             // int(1)
  int NbColImg,                             // int(1)
  int NbPixImg,                             // int(1)
  unsigned short int *CloudInfo,            // unsigned short int(SD_BAS_NLI_MAX_UTIL_AVHRR*SD_BAS_NPI_AVHRR)
  unsigned int *QualityIndicator,           // unsigned short int(SD_BAS_NLI_MAX_UTIL_AVHRR)
  unsigned int *ScanLineQuality,            // unsigned short int(SD_BAS_NLI_MAX_UTIL_AVHRR)
  unsigned short int *CountErrorFrame,      // unsigned short int(SD_BAS_NLI_MAX_UTIL_AVHRR)
  int ColImgFovFirst,                       // int(1)
  int ColImgFovLast,                        // int(1)
  int LinImgFovFirst,                       // int(1)
  int LinImgFovLast,                        // int(1)
  int NbPixImgFov,                          // int(1)
  double *ImgWgtpsf,                        // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)  > SD_ALG_IpsfToAvhrr(29)
  int EUMAvhrrCldErrorCutoff,               // int(1)
  // Sorties
  unsigned int *GEUMAvhrr1BCldFrac,         // unsigned int(1)
  unsigned int *GEUMAvhrr1BLandFrac,        // unsigned int(1)
  unsigned int *GEUMAvhrr1BQual             // unsigned int(1)
)
{
  static const char tcProcName_F[] = "SD_ALG_AvhrrCloudInfoIntoFov";
  t_CMN__Statut cr_op=CMN__STATUT_OK;

  // FT-2528
  int *FlagVect;
  unsigned int *FlagFov;
  // ...FT-2528
  int NbPix;
  int NbFlag;
  int ErrorCounterCld;
  unsigned short int value;
  int TestNumber;

  int Ipix;
  int Ilin;
  int Icol;
  unsigned int Criteria[8];
  int i;

  FlagVect = (int *) malloc(3*SD_BAS_NLI_MAX_UTIL_AVHRR*SD_BAS_NPI_AVHRR*sizeof(int));
  FlagFov = (unsigned int *) malloc(3*sizeof(unsigned int));

  if ((FlagVect == NULL) || (FlagFov == NULL))
  {
       cr_op   = SD_ALG_AVHRR_ERR_ALLOCATION;
       SD_BAS_THROW(
       cr_op, SD_ALG_AVHRR_41_CCS_ALRT,
       cr_op, BAS_CR_NEQ, CMN__STATUT_OK, SD_BAS_MSG_ALLOC);
  }

  // Creation du vecteur Information nuageuse FlagVect
  NbPix=0;
  NbFlag=3;
  ErrorCounterCld=0;
  for (Ilin = 1; Ilin <= NbLinImg; Ilin++)
  {
        for (Icol = 1; Icol <= NbColImg; Icol++)
        {
                for (i=1; i <= 8; i++)
                {
                  Criteria[IND1(i)] = 0;
                }
                NbPix++;
                FlagVect[IND2(1,NbPix,3)] = 0;
                FlagVect[IND2(2,NbPix,3)] = 0;
                FlagVect[IND2(3,NbPix,3)] = 0;
                if (((QualityIndicator[IND1(Ilin)] & 0x80000000) == 0) &&
                     (ScanLineQuality[IND1(Ilin)] == 0) &&
                     (CountErrorFrame[IND1(Ilin)] == 0))
                {
                  value = CloudInfo[IND2(Icol,Ilin,SD_BAS_NPI_AVHRR)];
                  // Calcul des criteres nuageux
                  // Si bit 4 = 0 et bit 5 = 1
                  if ((value & 0x0030) == 0x0020)
                  {
                      Criteria[IND1(1)] = 1;
                  }
                  // Si bit 6 = 0 et bit 7 = 1
                  if ((value & 0x00C0) == 0x0080)
                  {
                      Criteria[IND1(2)] = 1;
                  }
                  // Si bit 8 = 0 et bit 9 = 1
                  if ((value & 0x0300) == 0x0200)
                  {
                      Criteria[IND1(3)] = 1;
                  }
                  // Si bit 10 = 0 et bit 11 = 1
                  if ((value & 0x0C00) == 0x0800)
                  {
                      Criteria[IND1(4)] = 1;
                  }
                  // Si bit 12 = 0 et bit 13 = 1
                  if ((value & 0x3000) == 0x2000)
                  {
                      Criteria[IND1(5)] = 1;
                  }
                  // Si bit 14 = 0 et bit 15 = 1
                  if ((value & 0xC000) == 0x8000)
                  {
                      Criteria[IND1(6)] = 1;
                  }
                  // Si bit 4 -> bit 15 = 0
                  if ((value & 0xFFF0) == 0x0000)
                  {
                      Criteria[IND1(7)] = 1;
                  }
                  // Calcul des criteres neige/glace
                  // Si bit 4 = 1 et bit 5 = 1 et bit 6 = 1 et bit 7 = 1
                  if ((value & 0x00F0) == 0x00F0)
                  {
                      Criteria[IND1(8)] = 1;
                  }
                  // Construction du vecteur nuageux
                  if (Criteria[IND1(7)] == 1)
                  {
                      FlagVect[IND2(1,NbPix,3)] = 1;
                      ErrorCounterCld++;
                  }
                  else if ((Criteria[IND1(1)] == 1) ||
                           (Criteria[IND1(2)] == 1) ||
                           (Criteria[IND1(3)] == 1) ||
                           (Criteria[IND1(4)] == 1) ||
                           (Criteria[IND1(5)] == 1) ||
                           (Criteria[IND1(6)] == 1))
                  {
                      FlagVect[IND2(1,NbPix,3)] = 1;
                  }
                  // Construction du vecteur terre/mer
                  // TestNumber = 8*CloudInfo[Bit3] +  4*CloudInfo[Bit2] +  2*CloudInfo[Bit1] +  1*CloudInfo[Bit0]
                  TestNumber = value & 0x000F;
                  if ((TestNumber == 2) ||
                      (TestNumber == 3) ||
                      (TestNumber == 5) ||
                      (TestNumber == 7) ||
                      (TestNumber == 8) ||
                      (TestNumber == 10) ||
                      (TestNumber == 11))
                  {
                          FlagVect[IND2(2,NbPix,3)] = 1;
                  }
                  // Construction du vecteur neige/glace
                  if (Criteria[IND1(8)] == 1)
                  {
                          FlagVect[IND2(3,NbPix,3)] = 1;
                  }
                }
                else
                {
                  FlagVect[IND2(1,NbPix,3)] = 1;
                  FlagVect[IND2(2,NbPix,3)] = 1;
                  ErrorCounterCld++;
                }
        } // Boucle sur les colonnes image
  } // Boucle sur les lignes image

  if (ErrorCounterCld <= EUMAvhrrCldErrorCutoff)
  {
          // Integration des flags AVHRR dans les FOVs sondeur
          cr_op = SD_ALG_FlagAvhrrIntoFov(
          ctx,
          LinImgFirst,
          ColImgFirst,
          NbColImg,
          NbPixImg,
          LinImgFovFirst,
          LinImgFovLast,
          ColImgFovFirst,
          ColImgFovLast,
          NbFlag,
          FlagVect,
          NbPixImgFov,
          ImgWgtpsf,
          FlagFov);

          // Affectation des champs
          *GEUMAvhrr1BCldFrac = FlagFov[IND1(1)];
          *GEUMAvhrr1BLandFrac = FlagFov[IND1(2)];
          if (ErrorCounterCld == 0)
          {
            // Bit7 = 0
            *GEUMAvhrr1BQual = FlagFov[IND1(3)] & 0x7F;
          }
          else
          {
            // Bit7 = 1 et Min(126,ErrorCounterCld)
            if (ErrorCounterCld > 126)
            {
              *GEUMAvhrr1BQual = 128 + 126;
            }
            else
            {
              *GEUMAvhrr1BQual = 128 + ErrorCounterCld;
            }
          }
  }
  else
  {
    *GEUMAvhrr1BCldFrac  = 0xFF;  // bit7 = 1
    *GEUMAvhrr1BLandFrac = 0xFF;  // Bit7 = 1
    *GEUMAvhrr1BQual     = 0xFF;  // Bit7...0 = 1
  }
fin:
  if (FlagVect != NULL)
  {
    free(FlagVect);
    FlagVect = NULL;
  }
  if (FlagFov != NULL)
  {
    free(FlagFov);
    FlagFov = NULL;
  } 

  // sortie de la fonction
  return( cr_op);
}
// FIN AvhrrCloudInfoIntoFov =====================================================
// ===============================================================================
// ...FT-2528

// FT-2528
//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_FlagAvhrrIntoFov
//@
//@ objet :  Integration des flags AVHRR dans le FOV sondeur
//@
//@
//@ Parametres :
//@         (ENTREE) ProduitCourant ....... : Pour pointer sur les variables temporaires
//@         (ENTREE) LinImgFirst .......... : 1ere ligne AVHRR utile
//@         (ENTREE) ColImgFirst .......... : 1ere colonne AVHRR utile
//@         (ENTREE) NbColImg ............. : Nombre de colonnes AVHRR utiles
//@         (ENTREE) NbPixImg ............. : Nombre des pixels de l'image IIS
//@         (ENTREE) LinImgFovFirst ....... : Premiere ligne de l'image IPSF/AVHRR
//@         (ENTREE) LinImgFovLast ........ : Derniere ligne de l'image IPSF/AVHRR
//@         (ENTREE) ColImgFovFirst ....... : Premiere colonne de l'image IPSF/AVHRR
//@         (ENTREE) ColImgFovLast ........ : Derniere colonne de l'image IPSF/AVHRR
//@         (ENTREE) NbFlag ............... : 
//@         (ENTREE) FlagVect ............. :
//@         (ENTREE) NbPixImgFov .......... : Nombre de pixels de l'image IPSF/AVHRR
//@         (ENTREE) ImgWgtpsf ............ : Poids associes aux points des IPSF/AVHRR
//@         (SORTIE) FlagFov .............. : Flags AVHRR dans le FOV sondeur
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la focntion a echoue
//@
//@ ------------------------------------------------------------------*/
static t_CMN__Statut SD_ALG_FlagAvhrrIntoFov(
  // Entrees
  t_SD_BAS_DataThread  *ctx,                // t_SD_BAS_DataThread*
  int LinImgFirst,                          // int(1)
  int ColImgFirst,                          // int(1)
  int NbColImg,                             // int(1)
  int NbPixImg,                             // int(1)
  int LinImgFovFirst,                       // int(1)
  int LinImgFovLast,                        // int(1)
  int ColImgFovFirst,                       // int(1)
  int ColImgFovLast,                        // int(1)
  int NbFlag,                               // int(1)
  int *FlagVect,                            // int(NbFlag * NbPixImg)
  int NbPixImgFov,                          // int(1)
  double *ImgWgtpsf,                        // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  // Sorties
  unsigned int *FlagFov                     // int(NbFlag)
)
{
  static const char tcProcName_F[] = "SD_ALG_FlagAvhrrIntoFov";
  t_CMN__Statut cr_op=CMN__STATUT_OK;

  int NbpixFov;    //
  int Ilin;        // index des lignes
  int Icol;        // index des colonnes
  int Ipix;        // index des pixels
  int Iflag;       // index des flags
  int I;
  double FovWgt;  // somme des poids dans le FOV sondeur
  double FlagFovReal;
  int *FlagVectFov;

  FlagVectFov = (int *)malloc(3*SD_BAS_NLI_MAX_UTIL_AVHRR*SD_BAS_NPI_AVHRR*sizeof(int));

  if (FlagVectFov == NULL)
  {
       cr_op   = SD_ALG_AVHRR_ERR_ALLOCATION;
       SD_BAS_THROW(
       cr_op, SD_ALG_AVHRR_41_CCS_ALRT,
       cr_op, BAS_CR_NEQ, CMN__STATUT_OK, SD_BAS_MSG_ALLOC);
  }

  // Boucle sur les flags  (Flag = 1 a NbFlag)
  for (Iflag = 1; Iflag <= NbFlag; Iflag++)
  {
    // Extraction de l'imagette utile associee au Fov
    NbpixFov = 0;
    for (Ilin = LinImgFovFirst; Ilin <= LinImgFovLast; Ilin++)
    {
      for (Icol = ColImgFovFirst; Icol <= ColImgFovLast; Icol++)
      {
        NbpixFov++;
        Ipix = (Ilin - LinImgFirst) * NbColImg + Icol - ColImgFirst + 1;
        FlagVectFov[IND2(Iflag,NbpixFov,NbFlag)] = FlagVect[IND2(Iflag,Ipix,NbFlag)];
      }
    }
  }
  // Somme des poids dans le FOV sondeur FovWgt
  FovWgt = 0.0;
  for (I = 1; I <= NbpixFov ; I++)
  {
    FovWgt += ImgWgtpsf[IND1(I)];
  }

  // Boucle sur les flags  (Flag = 1 a NbFlag)
  for (Iflag = 1; Iflag <= NbFlag; Iflag++)
  {
    FlagFovReal = 0.0;
    for (I = 1; I <= NbpixFov ; I++)
    {
      FlagFovReal += FlagVectFov[IND2(Iflag,I,NbFlag)] * ImgWgtpsf[IND1(I)];

    }
    FlagFov[IND1(Iflag)] = (int)((100 * (FlagFovReal / FovWgt)) + 0.5);
  }
fin:
  if (FlagVectFov != NULL)
  {
    free(FlagVectFov);
    FlagVectFov = NULL;
  }
  // sortie de la fonction
  return( cr_op);
}
// FIN FlagAvhrrIntoFov ==========================================================
// ===============================================================================
// ...FT-2528


//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_IpsfToIIS
//@
//@ objet :  Traitement de l'image AVHRR
//@
//@
//@ Parametres :
//@         (ENTREE) ProduitCourant . : Pour pointer sur les variables temporaires
//@         (ENTREE) NbCol .......... : Nombre de colonnes des IPSF
//@         (ENTREE) NbLin .......... : Nombre de lignes des IPSF
//@         (ENTREE) YPsf ........... : Positions des points IPSF dans le repere IASI nominal instrument selon Y
//@         (ENTREE) ZPsf ........... : Positions des points IPSF dans le repere IASI nominal instrument selon Z
//@         (ENTREE) WgtPsf ......... : Poids associes aux points des IPSF
//@         (ENTREE) Osf ............ : Facteur de sur-echantillonnage de l'IPSF
//@         (ENTREE) IISRot ......... : Angles de rotation entre repere IIS et repere nominal instrument Ri
//@         (ENTREE) Ml ............. : Nombre de lignes de l'image IIS
//@         (ENTREE) Mc ............. : Nombre de colonnes de l'image IIS
//@         (ENTREE) IISY ........... : Position des centres des pixels IIS dans le repere IIS selon Y
//@         (ENTREE) IISZ ........... : Position des centres des pixels IIS dans le repere IIS selon Y
//@         (ENTREE) DY ............. : Dimension des pixels IIS selon Y
//@         (ENTREE) DZ ............. : Dimension des pixels IIS selon Z
//@         (ENTREE) OffsetSondIIS .. : Offset entre le sondeur IASI et l'imageur IIS
//@
//@         (SORTIE) ColIISFirst .... : Premiere colonne de l'IIS
//@         (SORTIE) ColIISLast ..... : Derniere colonne de l'IIS
//@         (SORTIE) LinIISFirst .... : Premiere ligne de l'IIS
//@         (SORTIE) LinIISLast ..... : Derniere ligne de l'IIS
//@         (SORTIE) NbPixIISFov .... : Nombre de pixels de l'IIS
//@         (SORTIE) IISYpsf ........ : Angles IIS corriges de l'offset sondeur/imageur selon Y
//@         (SORTIE) IISZpsf ........ : Angles IIS corriges de l'offset sondeur/imageur selon Z
//@         (SORTIE) IISWgtpsf ...... : Poids de l'IPSF sondeur pour chaque pixel IIS
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_IpsfToIIS(
  // Entrees
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  int                   NbCol,              // int(1)
  int                   NbLin,              // int(1)
  double               *YPsf,               // double( SD_BAS_NB_COL_SOND)
  double               *ZPsf,               // double( SD_BAS_NB_LIN_SOND)
  double               *WgtPsf,             // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  int                   Osf,                // int(1)
  double                IISRot,             // double(1)
  int                   Ml,                 // int(1)
  int                   Mc,                 // int(1)
  double               *IISY,               // double( SD_BAS_IMLI)
  double               *IISZ,               // double( SD_BAS_IMCO)
  double                DY,                 // double(1)
  double                DZ,                 // double(1)
  double               *OffsetSondIIS,      // double( SD_BAS_DEUX)
  // Sorties
  int                  *ColIISFovFirst,     // int(1)
  int                  *ColIISFovLast,      // int(1)
  int                  *LinIISFovFirst,     // int(1)
  int                  *LinIISFovLast,      // int(1)
  int                  *NbPixIISFov,        // int(1)
  double               *IISYpsf,            // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  double               *IISZpsf,            // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  double               *IISWgtpsf           // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
)
{
  // Transformation de l'IPSF dans le rasteur IIS
  static const char tcProcName_F[] = "SD_ALG_IpsfToIIS";
  t_CMN__Statut cr_op=CMN__STATUT_OK;

  // Gestion de erreurs
/* bma intialisation */
/*
  char *msg_err;                     // Message d'erreur
  int   num_lig;                     // Ligne d'erreur
*/
  char *msg_err = (char*) NULL;                     // Message d'erreur
  int   num_lig = 0;                     // Ligne d'erreur
/* fin bma */

  double  MinYPsf, MaxYPsf;                 // Valeurs extremes des 4 coins selon Y
  double  MinZPsf, MaxZPsf;                 // Valeurs extremes des 4 coins selon Z
  double *ImgY0;                            // Image initiale de colonnes selon Y
  double *ImgZ0;                            // Image initiale de lignes selon Z
  double *ImgY;                             // Image de colonnes selon Y
  double *ImgZ;                             // Image de lignes selon Z

  double *ImgYpsf;
  double *ImgZpsf;
  double *ImgWgtpsf;

  int     Ipsf,i;                             // Index Ipsf
  int     PixIIS;                           // Index de pixels sur l'image IIS
  int     Ilin, Icol;                       // Index lignes et colonnes
  double  DYpsf, DZpsf;                     // Variations de coordonnees
  double  IpsfCornerY[4];           // 4 coins selon Y
  double  IpsfCornerZ[4];           // 4 coins selon Z
  double  IpsfCornerYIIS[4];           // 4 coins selon Y
  double  IpsfCornerZIIS[4];           // 4 coins selon Z
  int     ligne;                            // Index lignes
  int     colonne;                          // Index colonnes
  double  MinIpsfCornerY, MaxIpsfCornerY;   // Valeurs extremes des 4 coins selon Y
  double  MinIpsfCornerZ, MaxIpsfCornerZ;   // Valeurs extremes des 4 coins selon Z
  int     MinIndiceY, MaxIndiceY;           // Indices du rectangle englobant de l'image IIS
  int     MinIndiceZ, MaxIndiceZ;           // Indices du rectangle englobant de l'image IIS
  int     NbLinIISFov, NbColIISFov;         // Taille de l'image IIS


  #ifdef OPS_DEBUG
    printf( "Debut de la fonction IpsfToIIS\n----------------------------------\n");
  #endif

  ImgY0= ProduitCourant->ImgY0;
  ImgZ0= ProduitCourant->ImgZ0;
  ImgY= ProduitCourant->ImgY;
  ImgZ= ProduitCourant->ImgZ;
  ImgYpsf= ProduitCourant->ImgYpsf;
  ImgZpsf= ProduitCourant->ImgZpsf;
  ImgWgtpsf= ProduitCourant->ImgWgtpsf;
  ImgYpsf= ProduitCourant->ImgYpsf;
  ImgZpsf= ProduitCourant->ImgZpsf;
  ImgWgtpsf= ProduitCourant->ImgWgtpsf;


  // Bornes des Y et Z
  MinYPsf= MAX_DOUBLE;
  MaxYPsf= MIN_DOUBLE;
  MinZPsf= MAX_DOUBLE;
  MaxZPsf= MIN_DOUBLE;
  // FT-2773
  for( Ipsf= 1; Ipsf <= NbCol; Ipsf++)
  // ...FT-2773
  {
    if( YPsf[IND1(Ipsf)] > MaxYPsf)   MaxYPsf= YPsf[IND1(Ipsf)];
    if( YPsf[IND1(Ipsf)] < MinYPsf)   MinYPsf= YPsf[IND1(Ipsf)];
  }
  // FT-2773
  for( Ipsf= 1; Ipsf <= NbLin; Ipsf++)
  // ...FT-2773
  {
    if( ZPsf[IND1(Ipsf)] > MaxZPsf)   MaxZPsf= ZPsf[IND1(Ipsf)];
    if( ZPsf[IND1(Ipsf)] < MinZPsf)   MinZPsf= ZPsf[IND1(Ipsf)];
  }

  // Ecart entre 2 points de discretisation Ipsf
  // DYpsf = YPsf(2) - YPsf(1)
  // DZpsf = ZPsf(2) - ZPsf(1)
  DYpsf= YPsf[IND1(2)] - YPsf[IND1(1)];
  DZpsf= ZPsf[IND1(2)] - ZPsf[IND1(1)];

  // Calcul des coordonnees des 4 coins de l'Ipsf dans le repere IASI corrigees
  // de l'offset sondeur-imageur
  // IpsfCornerY[1] = IpsfCornerY[4] = Min{ YPsf } - DYpsf/2 + OffsetSondIIS(1)
  IpsfCornerY[IND1(1)]= MinYPsf - DYpsf / 2.0 + OffsetSondIIS[IND1(1)];
  IpsfCornerY[IND1(4)]= IpsfCornerY[IND1(1)];

  // IpsfCornerY[2] = IpsfCornerY[3] = Max{ YPsf } + DYpsf/2 + OffsetSondIIS(1)
  IpsfCornerY[IND1(2)]= MaxYPsf + DYpsf / 2.0 + OffsetSondIIS[IND1(1)];
  IpsfCornerY[IND1(3)]= IpsfCornerY[IND1(2)];

  // IpsfCornerZ[1] = IpsfCornerZ[2] = Min{ ZPsf } - DZpsf/2 + OffsetSondIIS(2)
  IpsfCornerZ[IND1(1)]= MinZPsf - DZpsf / 2.0 + OffsetSondIIS[IND1(2)];
  IpsfCornerZ[IND1(2)]= IpsfCornerZ[IND1(1)];

  // IpsfCornerZ[3] = IpsfCornerZ[4] = Max{ ZPsf } + DZpsf/2 + OffsetSondIIS(2)
  IpsfCornerZ[IND1(3)]= MaxZPsf + DZpsf / 2.0 + OffsetSondIIS[IND1(2)];
  IpsfCornerZ[IND1(4)]= IpsfCornerZ[IND1(3)];

  // Coordonnees de la grille IIS dans le reprere IASI plan focal
  for( ligne= 1; ligne <= Ml; ligne++)
  {
    for( colonne= 1; colonne <= Mc; colonne++)
    {
      // ImgY0(ligne,colonne) = IISY(colonne)
      ImgY0[IND2(colonne,ligne,Mc)]= IISY[IND1(colonne)];

      // ImgZ0(ligne,colonne) = IISZ(ligne)
      ImgZ0[IND2(colonne,ligne,Mc)]= IISZ[IND1(ligne)];
    }
  }

  // Image des coordonnees de la grille IIS
  for( ligne= 1; ligne <= Ml; ligne++)
  {
    for( colonne= 1; colonne <= Mc; colonne++)
    {
      // ImgY(ligne,colonne) = cos(IISRot)* ImgY0(ligne,colonne) - sin(IISRot)* ImgZ0(ligne,colonne) 
      ImgY[IND2(colonne,ligne,Mc)]=   cos( IISRot) * ImgY0[IND2(colonne,ligne,Mc)]
                                    - sin( IISRot) * ImgZ0[IND2(colonne,ligne,Mc)];

      // ImgZ(ligne,colonne) = sin(IISRot)* ImgY0(ligne,colonne) + cos(IISRot)* ImgZ0(ligne,colonne) 
      ImgZ[IND2(colonne,ligne,Mc)]=   sin( IISRot) * ImgY0[IND2(colonne,ligne,Mc)]
                                    + cos( IISRot) * ImgZ0[IND2(colonne,ligne,Mc)];
    }
  }
  // Image des 4 coins de l ipsf dans le repere IIS 
  for( i= 1; i <= 4; i++)
  {
      // ImgY(ligne,colonne) = cos(IISRot)* ImgY0(ligne,colonne) - sin(IISRot)* ImgZ0(ligne,colonne) 
      IpsfCornerYIIS[IND1(i)]=   cos( IISRot) * IpsfCornerY[IND1(i)]
                                    - sin( IISRot) * IpsfCornerZ[IND1(i)];

      // ImgZ(ligne,colonne) = sin(IISRot)* ImgY0(ligne,colonne) + cos(IISRot)* ImgZ0(ligne,colonne) 
      IpsfCornerZIIS[IND1(i)]=   sin( IISRot) * IpsfCornerY[IND1(i)]
                                    + cos( IISRot) * IpsfCornerZ[IND1(i)];
  }
  // Bornes des lignes et colonnes AVHRR
  MinIpsfCornerY= MAX_DOUBLE;
  MaxIpsfCornerY= MIN_DOUBLE;
  MinIpsfCornerZ= MAX_DOUBLE;
  MaxIpsfCornerZ= MIN_DOUBLE;
  for( i= 1; i <= 4; i++)
  {
    if( IpsfCornerYIIS[IND1(i)] > MaxIpsfCornerY)   MaxIpsfCornerY= IpsfCornerYIIS[IND1(i)];
    if( IpsfCornerYIIS[IND1(i)] < MinIpsfCornerY)   MinIpsfCornerY= IpsfCornerYIIS[IND1(i)];
    if( IpsfCornerZIIS[IND1(i)] > MaxIpsfCornerZ)   MaxIpsfCornerZ= IpsfCornerZIIS[IND1(i)];
    if( IpsfCornerZIIS[IND1(i)] < MinIpsfCornerZ)   MinIpsfCornerZ= IpsfCornerZIIS[IND1(i)];
  }

  // Recherches des pixels IIS correspondant au domaine de l'Ipsf
  // IndiceY = find{ min{IpsfCornerY} <= ImgY0 <= max{IpsfCornerY} }
  // IndiceZ = find{ min{IpsfCornerZ} <= ImgZ0 <= max{IpsfCornerZ} }
  MinIndiceY= MAX_INT;
  MaxIndiceY= MIN_INT;
  MinIndiceZ= MAX_INT;
  MaxIndiceZ= MIN_INT;
  for( ligne= 1; ligne <= Ml; ligne++)
  {
    for( colonne= 1; colonne <= Mc; colonne++)
    {
      if( ImgY0[IND2(colonne,ligne,Mc)] <= MinIpsfCornerY)
        MinIndiceY= colonne;

      if( ImgY0[IND2(colonne,ligne,Mc)] >= MaxIpsfCornerY)
        MaxIndiceY= colonne;

      if( ImgZ0[IND2(colonne,ligne,Mc)] <= MinIpsfCornerZ)
        MinIndiceZ= ligne;

      if( ImgZ0[IND2(colonne,ligne,Mc)] >= MaxIpsfCornerZ)
        MaxIndiceZ= ligne;
    }
  }

// les indices commencent a 1 
  // ColIISFovFirst = min{ IndiceY }-1
  *ColIISFovFirst= MinIndiceY ;

  // LinIISFovFirst = min{ IndiceZ }-1
  *LinIISFovFirst= MinIndiceZ ;

  // ColIISFovLast = max{ IndiceY }+1
  *ColIISFovLast= MaxIndiceY ;

  // LinIISFovLast = max{ IndiceZ }+1
  *LinIISFovLast= MaxIndiceZ ;

  // NbLinIISFov = LinIISFovLast - LinIISFovFirst + 1
  NbLinIISFov= *LinIISFovLast - *LinIISFovFirst + 1;

  // NbColIISFov = ColIISFovLast - ColIISFovFirst + 1
  NbColIISFov= *ColIISFovLast - *ColIISFovFirst + 1;

  // NbPixIISFov = NbLinIISFov * NbColIISFov
  *NbPixIISFov= NbLinIISFov * NbColIISFov;

  // [ ImgWgtpsf, ImgYpsf, ImgZpsf ] = 
  // TransformIpsf{NbLin, NbCol, YPsf, ZPsf, Wgtpsf, LinIISFovFirst, ColIISFovFirst, LinIISFovLast, ColIISFovLast, ImgY, ImgZ, DY, Dz, OffsetSondIIS , OverSampFactor}
  cr_op= SD_ALG_TransformIpsf_CCS(
                               // Entrees
                                NbLin,             // int(1)
                                NbCol,             // int(1)
                                YPsf,              // double( SD_BAS_NB_COL_SOND) Limite a NbCol
                                ZPsf,              // double( SD_BAS_NB_LIN_SOND) Limite a NbLin
                                WgtPsf,            // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
                               *LinIISFovFirst,    // int(1)
                               *ColIISFovFirst,    // int(1)
                               *LinIISFovLast,     // int(1)
                               *ColIISFovLast,     // int(1)
			        Mc,		   // nb col ImgY et ImgZ,
                                ImgY,              // double( SD_BAS_NB_COL_SOND)
                                ImgZ,              // double( SD_BAS_NB_LIN_SOND)
                                DY,                // double(1)
                                DZ,                // double(1)
                                OffsetSondIIS,     // double(1)
                                (double)Osf,               // double(1)
                               // Sorties
                                ImgWgtpsf,         // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
                                ImgYpsf,           // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
                                ImgZpsf);          // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)

  // Transformation de matrice en vecteur
  // PixIIS = 0
  PixIIS= 0;

  // Boucle sur les lignes IIS ( Ilin =1 a NbLinIISFov)
  for( Ilin= 1; Ilin <= NbLinIISFov; Ilin++)
  {
    // Boucle sur les colonnes IIS ( Icol =1 a NbColIISFov)
    for( Icol= 1; Icol <= NbColIISFov; Icol++)
    {
      // PixIIS = PixIIS+1
      PixIIS= PixIIS + 1;

      // IISYpsf(PixIIS) = ImgYpsf(Ilin,Icol)
      IISYpsf[IND1(PixIIS)]= ImgYpsf[IND2(Icol,Ilin,NbColIISFov)];

      // IISZpsf(PixIIS) = ImgZpsf(Ilin,Icol)
      IISZpsf[IND1(PixIIS)]= ImgZpsf[IND2(Icol,Ilin,NbColIISFov)];

      // IISWgtpsf(PixIIS) = ImgWgtpsf(Ilin,Icol)
      IISWgtpsf[IND1(PixIIS)]= ImgWgtpsf[IND2(Icol,Ilin,NbColIISFov)];

    } // Fin de boucle sur les colonnes IIS
  } // Fin de boucle sur les lignes IIS

fin:
  // Fin de la fonction IpsfToIIS
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction IpsfToIIS\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN IpsfToIIS ==============================================================
// ===============================================================================



//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_AnaRadIntoFov
//@
//@ objet :  Traitement de l'image AVHRR
//@
//@
//@ Parametres :
//@         (ENTREE) ProduitCourant . : Pour pointer sur les variables temporaires
//@         (ENTREE) LinImgFirst .... : 1ere ligne AVHRR utile
//@         (ENTREE) ColImgFirst .... : 1ere colonne AVHRR utile
//@         (ENTREE) NbColImg ....... : Nombre de colonnes AVHRR utiles
//@         (ENTREE) NbPixImg ....... : Nombre de pixels AVHRR utiles
//@         (ENTREE) LinImgFovFirst . : Premiere ligne de l'image IPSF/AVHRR
//@         (ENTREE) LinImgFovLast .. : Derniere ligne de l'image IPSF/AVHRR
//@         (ENTREE) ColImgFovFirst . : Premiere colonne de l'image IPSF/AVHRR
//@         (ENTREE) ColImgFovLast .. : Derniere colonne de l'image IPSF/AVHRR
//@         (ENTREE) RadAv .......... : Vecteur de radiance de l'image AVHRR
//@         (ENTREE) IdPix .......... : Flags de pixels valides
//@         (ENTREE) NbClass ........ : Nombre de classes a traiter
//@         (ENTREE) Nch ............ : Nombre de canaux presents
//@         (ENTREE) ChanMax ........ : Nombre maximal de canaux
//@         (ENTREE) ChId ........... : Identificateur des canaux
//@         (ENTREE) ImgWgtpsf ...... : Poids associes aux points des IPSF/AVHRR
//@         (ENTREE) ImgYpsf ........ : Positions des points IPSF/AVHRR dans le repere IASI nominal instrument selon Y
//@         (ENTREE) ImgZpsf ........ : Positions des points IPSF/AVHRR dans le repere IASI nominal instrument selon Z
//@
//@         (SORTIE) RadAnalWgt ..... : Poids des classes dans le FOV
//@         (SORTIE) RadAnalY ....... : Barycentres des classes selon Y
//@         (SORTIE) RadAnalZ ....... : Barycentres des classes selon Z
//@         (SORTIE) RadAnalMean .... : Moyenne des classes
//@         (SORTIE) RadAnalStd ..... : Ecart type des classes
//@         (SORTIE) NbClassFov ..... : Nombre de classes dans le FOV
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_AnaRadIntoFov(
  // Entrees
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  int                   LinImgFirst,     // int(1)
  int                   ColImgFirst,     // int(1)
  int                   NbColImg,        // int(1)
  int                   NbPixImg,        // int(1)
  int                   LinImgFovFirst,  // int(1)
  int                   LinImgFovLast,   // int(1)
  int                   ColImgFovFirst,  // int(1)
  int                   ColImgFovLast,   // int(1)
  float                *RadVect,         // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
  int                  *IdPix,           // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  int                   NbClass,         // int(1)
  int                   Nch,             // int(1)
  int                   ChanMax,         // int(1)
  int                  *ChId,            // int( SD_BAS_NBK)
  double               *ImgWgtpsf,       // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  double               *ImgYpsf,         // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  double               *ImgZpsf,         // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  // Sorties
  double               *RadAnalWgt,      // double( SD_BAS_NCL_MAX)
  double               *RadAnalY,        // double( SD_BAS_NCL_MAX)
  double               *RadAnalZ,        // double( SD_BAS_NCL_MAX)
  double               *RadAnalMean,     // double( SD_BAS_NCL_MAX * SD_BAS_NBK)
  double               *RadAnalStd,      // double( SD_BAS_NCL_MAX * SD_BAS_NBK)
  int                  *NbClassFov       // int(1)
)
{
 // Definitions :
  // $ = NbClass
  // # = Nch

  // Analyse des radiances dans le FOV sondeur
  static const char tcProcName_F[] = "SD_ALG_AnaRadIntoFov";
  t_CMN__Statut cr_op=CMN__STATUT_OK;

  // Gestion de erreurs
/* bma intialisation */
/*
  char *msg_err;                     // Message d'erreur
  int   num_lig;                     // Ligne d'erreur
*/
  char *msg_err = (char*) NULL;                     // Message d'erreur
  int   num_lig = 0;                     // Ligne d'erreur
/* fin bma */

  int     TailleAvhrr;               // Taille d'une image AVHRR
  int     TailleFOV;                 // Taille de l'imagette associee au FOV
  int     NbpixFov;                  // Nombre de pixels du FOV
  int     Ilin;                      // Index lignes
  int     Icol;                      // Index colonnes
  int     Ipix;                      // Index pixels
  int     Iclas;                     // Index class
  int     k;                         // Index canaux
  int     i;                         // Index general
  double  TestSign;                  // Signe
  float  *RadVectFov;
  int    *IdPixFov;
  int     NbPixClas[SD_BAS_NCL_MAX];     // Nombre de pixels par classe
  int    *Index;


  #ifdef OPS_DEBUG
    printf( "Debut de la fonction AnaRadIntoFov\n----------------------------------\n");
  #endif

  TailleAvhrr= SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR;
  TailleFOV= (LinImgFovLast - LinImgFovFirst + 1) * (ColImgFovLast - ColImgFovFirst + 1);  

  RadVectFov= ProduitCourant->RadVectFov;
  IdPixFov= ProduitCourant->IdPixFov;
  Index= ProduitCourant->Index;

  // Extraction de l'imagette utile associee au Fov
  // NbpixFov = 0
 
  for( k= 1; k <= ChanMax; k++)
  {
   NbpixFov= 0;
    // Boucle sur les lignes image ( Ilin = LinImgFovFirst a LinImgFovLast )
    for( Ilin= LinImgFovFirst; Ilin <= LinImgFovLast; Ilin++)
    {
      // Boucle sur les colonnes image ( Icol = ColImgFovFirst a ColImgFovLast )
      for( Icol= ColImgFovFirst; Icol <= ColImgFovLast; Icol++)
      {
        // NbpixFov = NbpixFov+1
        NbpixFov= NbpixFov + 1;

        // Ipix = (Ilin-LinImgFirst)*NbColImg + Icol - ColImgFirst+1
        Ipix= (Ilin - LinImgFirst) * NbColImg + Icol - ColImgFirst + 1;

        // RadVectFov(NbpixFov) = RadVect(Ipix)


        // IdPixFov(NbpixFov) = IdPix(Ipix)
        if( (Ipix <= NbPixImg) && (Ipix>0))
        {
          IdPixFov[IND1(NbpixFov)]= IdPix[IND1(Ipix)];
          RadVectFov[IND2(NbpixFov,k,TailleFOV)]= RadVect[IND2(Ipix,k,TailleAvhrr)];
        }
        else
        {
          IdPixFov[IND1(NbpixFov)]= 0;
          RadVectFov[IND2(NbpixFov,k,TailleFOV)]= 0.0f;
        }

      } // Fin de boucle sur les colonnes 
    } // Fin de boucle sur les lignes 
  }  // Fin de boucle sur les canaux

  // Boucle sur les classes (Iclas = 1 a NbClass)
  for( Iclas= 1; Iclas <= NbClass; Iclas++)
  {
    // Recherche des pixels appartenant a la classe Iclas

    // Index = Find{ IdPixFov(NbPixFov) = Iclas}
    // NbPixClas(Iclas) = Nombre-d'elements{Index}
      NbPixClas[IND1(Iclas)]= 0;
    for( Ipix= 1; Ipix <= NbpixFov; Ipix++)
    {
      if( IdPixFov[IND1(Ipix)] == Iclas)
      {
        NbPixClas[IND1(Iclas)]= NbPixClas[IND1(Iclas)] + 1;
        Index[IND1(NbPixClas[IND1(Iclas)])]= Ipix;
      }
    }

    // Poids de la classe dans le FOV sondeur RadAnalWgt
    // RadAnalWgt(Iclas) =  somme(...)
    RadAnalWgt[IND1(Iclas)]= 0.0;
    for( i= 1; i <= NbPixClas[IND1(Iclas)]; i++)
    {
      RadAnalWgt[IND1(Iclas)]= RadAnalWgt[IND1(Iclas)] + ImgWgtpsf[IND1(Index[IND1(i)])];
    }

    // Calcul de la position du barycentres (cosinus moyen) de la classes : RadAnalY et RadAnalZ
    // RadAnalY(Iclas) =  somme(...)
    RadAnalY[IND1(Iclas)]= 0.0;
    for( i= 1; i <= NbPixClas[IND1(Iclas)]; i++)
    {
      RadAnalY[IND1(Iclas)]=   RadAnalY[IND1(Iclas)]
                             + (   cos(ImgYpsf[IND1(Index[IND1(i)])])
                                 * ImgWgtpsf[IND1(Index[IND1(i)])]);
    }
    if (!SD_BAS_MacEGAUX_D( RadAnalWgt[IND1(Iclas)], 0.0)) 
      {RadAnalY[IND1(Iclas)]= RadAnalY[IND1(Iclas)] / RadAnalWgt[IND1(Iclas)];
      RadAnalY[IND1(Iclas)]= acos( RadAnalY[IND1(Iclas)]);}
 
    // TestSign =  somme(...)
    TestSign= 0.0;
    for( i= 1; i <= NbPixClas[IND1(Iclas)]; i++)
    {
      // FT-2894	
      TestSign= TestSign + (ImgYpsf[IND1(Index[IND1(i)])] * ImgWgtpsf[IND1(Index[IND1(i)])]);
      // ...FT-2894	
    }

    // Si TestSign < 0 alors RadAnalY(Iclas) = - RadanalY(Iclas)
    if( TestSign < 0.0)
      RadAnalY[IND1(Iclas)]= -RadAnalY[IND1(Iclas)];

    // RadAnalZ(Iclas) =  somme(...)
    RadAnalZ[IND1(Iclas)]= 0.0;
    for( i= 1; i <= NbPixClas[IND1(Iclas)]; i++)
    {
      RadAnalZ[IND1(Iclas)]=   RadAnalZ[IND1(Iclas)]
                             + (   cos(ImgZpsf[IND1(Index[IND1(i)])])
                                 * ImgWgtpsf[IND1(Index[IND1(i)])]);
    }
   if (!SD_BAS_MacEGAUX_D( RadAnalWgt[IND1(Iclas)], 0.0)) 
      {RadAnalZ[IND1(Iclas)]= RadAnalZ[IND1(Iclas)] / RadAnalWgt[IND1(Iclas)];
       RadAnalZ[IND1(Iclas)]= acos( RadAnalZ[IND1(Iclas)]);}

    // TestSign =  somme(...)
    TestSign= 0.0;
    for( i= 1; i <= NbPixClas[IND1(Iclas)]; i++)
    {
      // FT-2894	
      TestSign= TestSign + (ImgZpsf[IND1(Index[IND1(i)])] * ImgWgtpsf[IND1(Index[IND1(i)])]);
      // ...FT-2894	
    }

    // Si TestSign < 0 alors RadAnalZ(Iclas) = - RadanalZ(Iclas)
    if( TestSign < 0.0)
      RadAnalZ[IND1(Iclas)]= -RadAnalZ[IND1(Iclas)];


    // Pour k = 1 a ChanMax
    for( k= 1; k <= ChanMax; k++)
    {
// inversion ordre canal et classe
      // RadAnalMean(k,Iclas) = 0
      RadAnalMean[IND2(k,Iclas,SD_BAS_NBK)]= 0.0;

      // RadAnalStd(k,Iclas) = 0
      RadAnalStd[IND2(k,Iclas,SD_BAS_NBK)]= 0.0;
    } // Fin de boucle sur k

    // Boucle sur les canaux max de sortie : k=1,NCh
    for( k= 1; k <= Nch; k++)
    {
      // Radiances moyennes de la classe

      // RadAnalMean(ChId(k),Iclas)= somme(...)
      RadAnalMean[IND2(ChId[IND1(k)],Iclas,SD_BAS_NBK)]= 0.0;
      for( i= 1; i <= NbPixClas[IND1(Iclas)]; i++)
      {
        RadAnalMean[IND2(ChId[IND1(k)],Iclas,SD_BAS_NBK)]=
                        RadAnalMean[IND2(ChId[IND1(k)],Iclas,SD_BAS_NBK)]
                  + (   RadVectFov[IND2(Index[IND1(i)],k,NbpixFov)]
                      * ImgWgtpsf[IND1(Index[IND1(i)])]);
      }
     if (!SD_BAS_MacEGAUX_D( RadAnalWgt[IND1(Iclas)], 0.0))      RadAnalMean[IND2(ChId[IND1(k)],Iclas,SD_BAS_NBK)]= 
                  RadAnalMean[IND2(ChId[IND1(k)],Iclas,SD_BAS_NBK)]
                / RadAnalWgt[IND1(Iclas)];
        
      // Ecart type de la classe 
      // RadAnalStd(ChId(k),Iclas) =  somme(...)
      RadAnalStd[IND2(ChId[IND1(k)],Iclas,SD_BAS_NBK)]= 0.0;
      for( i= 1; i <= NbPixClas[IND1(Iclas)]; i++)
      {
/*FA 189*/
        RadAnalStd[IND2(ChId[IND1(k)],Iclas,SD_BAS_NBK)]=
           (  (double) RadVectFov[IND2(Index[IND1(i)],k,NbpixFov)]
                      * (double)RadVectFov[IND2(Index[IND1(i)],k,NbpixFov)]
                      * ImgWgtpsf[IND1(Index[IND1(i)])])
		  + RadAnalStd[IND2(ChId[IND1(k)],Iclas,SD_BAS_NBK)];
/*..FA 189*/

      }

      if (!SD_BAS_MacEGAUX_D( RadAnalWgt[IND1(Iclas)], 0.0))      
           RadAnalStd[IND2(ChId[IND1(k)],Iclas,SD_BAS_NBK)]= 
                  (RadAnalStd[IND2(ChId[IND1(k)],Iclas,SD_BAS_NBK)])
                / (RadAnalWgt[IND1(Iclas)]);

      // RadAnalStd(ChId(k),Iclas) =  sqrt(...)
      RadAnalStd[IND2(ChId[IND1(k)],Iclas,SD_BAS_NBK)]=
                  RadAnalStd[IND2(ChId[IND1(k)],Iclas,SD_BAS_NBK)]
                - (   RadAnalMean[IND2(ChId[IND1(k)],Iclas,SD_BAS_NBK)]
                    * RadAnalMean[IND2(ChId[IND1(k)],Iclas,SD_BAS_NBK)]);
      RadAnalStd[IND2(ChId[IND1(k)],Iclas,SD_BAS_NBK)]=
                sqrt( RadAnalStd[IND2(ChId[IND1(k)],Iclas,SD_BAS_NBK)]);

    } // Fin de boucle sur les canaux

  } // Fin de boucle sur les classes

  // Nombre de classes dans le Fov
  // NbClassFov = Nombre-d'elements{ Find{RadAnalWgt($)  0}}
  *NbClassFov= 0;
  for( Iclas= 1; Iclas <= NbClass; Iclas++)
  {
    if( RadAnalWgt[IND1(Iclas)] > 0.0)
      *NbClassFov= *NbClassFov + 1;
  }

fin:
  // Fin de la fonction AnaRadIntoFov
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction AnaRadIntoFov\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN AnaRadIntoFov =============================================================
// ===============================================================================


//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_TestQual
//@
//@ objet :   Evaluation de la qualite de la classification
//@
//@
//@ Parametres :
//@         (ENTREE) NbPixAv ............. : Nombre de pixels du vecteur
//@         (ENTREE) Nch ................. : Nombre de canaux presents
//@         (ENTREE) NbClass ............. : Nombre de classes a traiter
//@         (ENTREE) EigVal .............. : Valeurs propres
//@         (ENTREE) EigValCutoff ........ : Seuil de valeur propre
//@         (ENTREE) NbPixClas ........... : Nombre pixels par classe
//@         (ENTREE) SumDistClas ......... : Somme des distances intra-classes
//@
//@         (SORTIE) NonClassifRate ...... : Taux de pixels non classes
//@         (SORTIE) NormVarRate ......... : Rapport : variance image classifiee / image non classifiee normalise
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_TestQual(
  // Entrees
  int     NbPixAv,          // int(1)
  int     Nch,              // int(1)
  int     NbClass,          // int(1)
/* bma complexes->reels */
  double *EigVal,           // double( SD_BAS_NBK)
/* fin bma */
  double  EigValCutoff,     // double(1)
  int    *NbPixClas,        // int( SD_BAS_NCL_MAX)
  double *SumDistClas,      // double( SD_BAS_NCL_MAX)
  // Sorties
  double *NonClassifRate,   // double(1)
  double *NormVarRate       // double(1)
)
{
  // Calcul des parametres qualite de l'analyse des radiances
  static const char tcProcName_F[] = "SD_ALG_TestQual";
  t_CMN__Statut cr_op=CMN__STATUT_OK;

  // Gestion de erreurs
/* bma intialisation */
/*
  char *msg_err;                     // Message d'erreur
  int   num_lig;                     // Ligne d'erreur
*/
  char *msg_err = (char*) NULL;                     // Message d'erreur
  int   num_lig = 0;                     // Ligne d'erreur
/* fin bma */

/* bma complexes->reels */
  double NbEigVal;     // Nombre de valeurs propre > EigValCutoff
  double SumDist;      // Somme des distances de toutes les classes
  int    NbPix;        // Nombre de pixels de toutes les classes
  double VarRate;      // Variance
  int    Ich;          // Index canaux
  int    Iclas;        // Index sur les classes

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction TestQual\n----------------------------------\n");
  #endif

  // Nombre de valeurs hors bruit
  // NbEigVal = Nombre-d'elements{ Find{EigVal(1aNch)/EigVal(1) > EigValCutoff } }

  if( EigVal[IND1(1)] < 1.0E-6)
  {
    cr_op=   SD_ALG_AVHRR_ERR_NUL_EIG_VAL;
    msg_err= (char *)SD_BAS_MSG_ERR_NUL_EIG_VAL;
    num_lig= __LINE__;

    goto fin;
  }

  NbEigVal= 0.0;
  for( Ich= 1; Ich <= Nch; Ich++)
  {
    if( (EigVal[IND1(Ich)] / EigVal[IND1(1)]) > EigValCutoff)
      NbEigVal= NbEigVal + 1.0;
  }
/* fin bma */

  // Proportion du nombre de pixels non classes
  // NonClassifRate = NbPixClas(NbClass+1) / NbPixAv
  *NonClassifRate= ((double)NbPixClas[IND1(NbClass+1)]) / ((double)NbPixAv);

  // Rapport entre la variance de l'image classee et celle de l'image non classee
  // VarRate = somme(...)
  SumDist= 0.0;
  NbPix= 0;
  for( Iclas= 1; Iclas <= NbClass; Iclas++)
  {
    SumDist= SumDist + SumDistClas[IND1(Iclas)];
    NbPix=   NbPix   + NbPixClas[IND1(Iclas)];
  }
  VarRate= SumDist / (double)(Nch * NbPix);

  // Rapport des variances normalise
  // NormVarRate = VarRate.NbClass^(2/NbEigVal)
  *NormVarRate= VarRate * pow( (double)NbClass, 2.0 / NbEigVal);

fin:
  // Fin de la fonction TestQual
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction TestQual\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN TestQual ==================================================================
// ===============================================================================



//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_InitRad
//@
//@ objet :   Preparation des radiances et du prototype zero
//@
//@
//@ Parametres :
//@         (ENTREE) ProduitCourant ...... : Pour pointer sur les variables temporaires
//@         (ENTREE) Contexte_courant .... : contexte de l'etape (numero de ligne SN traite)
//@         (ENTREE) RadAv ............... : Vecteur de radiance de l'image AVHRR
//@         (ENTREE) NbPixAv ............. : Nombre de pixels du vecteur
//@         (ENTREE) Nch ................. : Nombre de canaux presents
//@         (ENTREE) ChannelNoise ........ : Bruit des canaux
//@         (ENTREE) ChannelWgt .......... : Poids des canaux
//@
//@         (SORTIE) RadMean ............. : Radiance moyenne
//@         (SORTIE) RadStd .............. : Ecart type de la radiance
//@         (SORTIE) RadNW ............... : Vecteur de radiance de l'image AVHRR
//@         (SORTIE) ProtoZero ........... : Prototype initial
//@         (SORTIE) EigVal .............. : Valeurs propres
//@         (SORTIE) EigVect ............. : Vecteurs propres
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_InitRad(
  // Entrees
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  t_SD_BAS_DataThread  *Contexte_courant,   // t_SD_BAS_DataThread*
  float                *RadAv,              // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
  int                   NbPixAv,            // int(1)
  int                   Nch,                // int(1)
  double               *ChannelNoise,       // int( SD_BAS_NBK)
  int                  *ChannelWgt,         // int( SD_BAS_NBK)
  // Sorties
  double                *RadMean,            // double( SD_BAS_NBK)
  double                *RadStd,             // double( SD_BAS_NBK)
  float                *RadNW,              // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
  double               *ProtoZero,          // double( SD_BAS_NBK)
/* bma complexes->reels */
  double               *EigVal,             // double( SD_BAS_NBK)
  double               *EigVect             // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
)
{
  static const char tcProcName_F[] = "SD_ALG_InitRad";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  int    Ich;                               // Index sur les canaux
  int    PixNum;                            // Numero de pixel
  float *RadN;                              // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
  int   *IdPix;                             // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  int    Ipro;                              // int(1) Numero de classe
  double CovarZero[SD_BAS_NBK*SD_BAS_NBK];  // double( SD_BAS_NBK * SD_BAS_NBK)
  int    TailleImage;
  double  diff;
  double auxv[2*SD_BAS_NBK];


  #ifdef OPS_DEBUG
    printf( "Debut de la fonction InitRad\n----------------------------------\n");
  #endif

  TailleImage= SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR;

  RadN= ProduitCourant->RadN;
  IdPix= ProduitCourant->IdPix;

  // Preparation des radiances et du prototype zero

  // Normalisation
  // Boucle sur les canaux ( Ich = 1 a Nch)
  for( Ich= 1; Ich <= Nch; Ich++)
  {
    // Calcul de la moyenne
    RadMean[IND1(Ich)]= (double)0.0;
    for( PixNum= 1; PixNum <= NbPixAv; PixNum++)
    {
      RadMean[IND1(Ich)]=   RadMean[IND1(Ich)]
                          + RadAv[IND2(PixNum,Ich,TailleImage)];
    }
    RadMean[IND1(Ich)]= RadMean[IND1(Ich)] / NbPixAv;

    // Calcul de l'ecart type
    RadStd[IND1(Ich)]= (double)0.0;
    for( PixNum= 1; PixNum <= NbPixAv; PixNum++)
    {
      diff= RadAv[IND2(PixNum,Ich,TailleImage)] - RadMean[IND1(Ich)];
      RadStd[IND1(Ich)]= RadStd[IND1(Ich)] + (diff * diff);
    }
    RadStd[IND1(Ich)]= (float)sqrt( RadStd[IND1(Ich)] / NbPixAv);

    // Si RadStd(Ich) < 3*ChannelNoise(Ich) alors RadStd(Ich) = 3*ChannelNoise(Ich)
    /*RIC chgt temporaire pour que le TU fonctionne*/
    // VERRUE a enlever  if( RadStd[IND1(Ich)] == ((float)3.0*ChannelNoise[IND1(Ich)]))
    if( RadStd[IND1(Ich)] < ((double)3.0*ChannelNoise[IND1(Ich)]))
    {
      RadStd[IND1(Ich)]= (double)(3.0 * ChannelNoise[IND1(Ich)]);
    }

    // Calcul des Radiances normalisees
    for( PixNum= 1; PixNum <= NbPixAv; PixNum++)
    {
      RadN[IND2(PixNum,Ich,TailleImage)]=
        ((double)RadAv[IND2(PixNum,Ich,TailleImage)] - RadMean[IND1(Ich)]) / RadStd[IND1(Ich)];
    }
  } //Fin de boucle sur les canaux (Ich)

  // Ponderation des canaux
  for( Ich= 1; Ich <= Nch; Ich++)
  {
    for( PixNum= 1; PixNum <= NbPixAv; PixNum++)
    {
      RadNW[IND2(PixNum,Ich,TailleImage)]=   RadN[IND2(PixNum,Ich,TailleImage)]
                                           * (float)ChannelWgt[IND1(Ich)];
    }
  }

  // Prototype zero (moyenne des radiances normalisees = 0.) et matrice de covariance
  Ipro= 0;
  for( PixNum= 1; PixNum <= NbPixAv; PixNum++)    IdPix[IND1(PixNum)]= Ipro;

  cr_op= SD_ALG_StatClas(
                          // Entrees
                           ProduitCourant,  // t_SD_BAS_DataThreadProduit*
                           RadN,            // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
                           IdPix,           // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
                           Nch,             // int(1)
                           NbPixAv,         // int(1)
                           Ipro,            // int(1)
                          // Sorties
                           ProtoZero,       // double( SD_BAS_NBK)
                           CovarZero);      // double( SD_BAS_NBK * SD_BAS_NBK)

  // Calcul des valeurs propres et vecteurs propres de la matrice de covariance
  // Note : il faut que EigVal(1) > EigVal(2) > ...
  // [ EigVal(#), EigVect(#,#) ] = EigenAnalysis{ CovarZero(#,#) } 
  cr_op= SD_ALG_EigenAnalysis(
        // Entrees
         Contexte_courant,  // t_SD_BAS_DataThread*
         SD_BAS_NBK,        // int(1) Nombre de points
         CovarZero,         // double( SD_BAS_NBK * SD_BAS_NBK),
         CMN__OUI,          // unsigned int(1) Transposition de CovarZero avant calcul et EigVect apres calcul
        // Sorties
/* bma complexes->reels */
         EigVal,            // double( SD_BAS_NBK)
         EigVect);          // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */

fin:
  // Fin de la fonction InitRad
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction InitRad\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN InitRad ===================================================================
// ===============================================================================



//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_InitProto
//@
//@ objet :   Initialisation des prototypes
//@
//@
//@ Parametres :
//@         (ENTREE) ProduitCourant .. : Pour pointer sur les variables temporaires
//@         (ENTREE) RadN ............ : Vecteur de radiance de l'image AVHRR
//@         (ENTREE) Nch ............. : Nombre de canaux presents
//@         (ENTREE) NbPixAv ......... : Nombre de pixels du vecteur
//@         (ENTREE) ProtoZero ....... : Prototype initial
//@         (ENTREE) NbClusMax ....... : Nombre maximal de clusters
//@         (ENTREE) DistMax ......... : Distance maximale interclasses
//@         (ENTREE) SortingChannel .. : Canal de rangement
//@
//@         (SORTIE) NbPro ........... : Nombre de prototypes
//@         (SORTIE) Proto ........... : Prototypes de classes
//@         (SORTIE) Covar ........... : Matrice de covariance
//@         (SORTIE) Dist ............ : Matrice des distances inter-prototypes
//@         (SORTIE) IdPix ........... : Flags de pixels valides
//@         (SORTIE) NbPixClas ....... : Nombre pixels par classe
//@         (SORTIE) SumDistClas ..... : Somme des distances intra-classes
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_InitProto(
  // Entrees
  t_SD_BAS_DataThread* ctx,      // t_SD_BAS_DataThread*
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  float                *RadN,               // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
  int                   Nch,                // int(1)
  int                   NbPixAv,            // int(1)
  double               *ProtoZero,          // double( SD_BAS_NBK)
  int                   NbClusMax,          // int(1)
  double                DistMax,            // double(1)
  int                   SortingChannel,     // int(1)
  // Sorties
  int                  *NbPro,              // int(1)
  double               *Proto,              // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
  double               *Covar,              // double( SD_BAS_NBK * SD_BAS_NBK * SD_BAS_NCL_MAX)
  double               *Dist,               // double( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NCL_MAX)
  int                  *IdPix,              // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  int                  *NbPixClas,          // int( SD_BAS_NCL_MAX)
  double               *SumDistClas         // double( SD_BAS_NCL_MAX)
)
{
  static const char tcProcName_F[] = "SD_ALG_InitProto";
  t_CMN__Statut cr_op=CMN__STATUT_OK;

  // Gestion de erreurs
/* bma intialisation */
/*
  char *msg_err;          // Message d'erreur
  int num_lig;            // Ligne d'erreur
*/
  char *msg_err = (char*) NULL;          // Message d'erreur
  int num_lig = 0;            // Ligne d'erreur
/* fin bma */

  // Definitions
  // (#) (1 a Nch) nombre de canaux/pseudo-canaux Avhrr utilises
  // (*) (1 a NbPixAv) nombre de pixels Avhrr
  // ($) (1 a NbPro) nombre de classes prototype
  unsigned int *UsedPixTest;    // unsigned int(SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  double       *DistZero;
  double       *Dist1;
  int           Ipix;           // Index de pixels
  int           Ich;            // Index sur les canaux
  int           Ipro;           // Index sur les prototypes
  double        MaxDist0;       // Distance maximale dans DistZero
  int           Imax;           // Index de distance maximale
  int           TailleAvhrr;    // Taille d'une image AVHRR
  double        SumDistTot;     // double(1) - Utilise en retour de fonction, mais pas utile
  double        *discu1, *discu;  //CF code Noveltis
//jlp30 pour la rapidite
  const int irk=3;
  int k;
  discu1=NULL;
  discu=NULL;
  discu1 = (double *) calloc( NbPixAv,sizeof(double) );
  discu = (double *) calloc( NbPixAv,sizeof(double) );
 if( (discu1 == NULL) || (discu == NULL) )
  {
    cr_op   = SD_ALG_AVHRR_ERR_ALLOCATION;
    msg_err= (char *)SD_BAS_MSG_ALLOC;
    num_lig= __LINE__;

    goto fin;
  } 

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction InitProto\n----------------------------------\n");

    for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
    {
      printf( "RadN[%4d] : %10.5f %10.5f %10.5f %10.5f %10.5f %10.5f\n",
              Ipix,
              RadN[IND2(Ipix,1,SD_BAS_NPI_AVHRR*SD_BAS_NLI_MAX_UTIL_AVHRR)],
              RadN[IND2(Ipix,2,SD_BAS_NPI_AVHRR*SD_BAS_NLI_MAX_UTIL_AVHRR)],
              RadN[IND2(Ipix,3,SD_BAS_NPI_AVHRR*SD_BAS_NLI_MAX_UTIL_AVHRR)],
              RadN[IND2(Ipix,4,SD_BAS_NPI_AVHRR*SD_BAS_NLI_MAX_UTIL_AVHRR)],
              RadN[IND2(Ipix,5,SD_BAS_NPI_AVHRR*SD_BAS_NLI_MAX_UTIL_AVHRR)],
              RadN[IND2(Ipix,6,SD_BAS_NPI_AVHRR*SD_BAS_NLI_MAX_UTIL_AVHRR)]);
    }
    printf( "=========> Appel SD_ALG_DistPixProto\n");
  #endif

  TailleAvhrr= SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR;

  UsedPixTest= ProduitCourant->UsedPixTest;
  DistZero= ProduitCourant->DistZero;
  Dist1= ProduitCourant->Dist1;

  // Nombre initial de prototype 
  // NbPro = Min{ NbClusMax - 2, 3 }
  *NbPro= MIN( NbClusMax - 2, 3);

  // Calcul de la distance quadratique au prototype zero
  // DistZero(*) = DistPixProto{ RadN(#,*), Nch, NbPixAv, ProtoZero(#) }
  cr_op= SD_ALG_DistPixProto(
                 // Entrees
                 RadN,         // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
                 Nch,          // int(1)
                 NbPixAv,      // int(1)
                 ProtoZero,    // double( SD_BAS_NBK)
                 // Sorties
                 DistZero);    // double(SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)

  #ifdef OPS_DEBUG
    for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
      printf( "DistZero[%d]= %f\n", Ipix, DistZero[IND1(Ipix)]); 
  #endif

  // Recherche du pixel le plus eloigne du prototype zero
  // UsedPixTest(*) = False
  for( Ipix= 1; Ipix <= NbPixAv; Ipix++)   UsedPixTest[IND1(Ipix)]= CMN__NON;

  for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
    discu1[IND1(Ipix)] = DistZero[IND1(Ipix)];

// version 5.0 
// Imax = Index de Max3{ Dist1(*) }
//jlp30 pour la rapidite
// cr_op = SD_ALG_Index_De_Max3(ctx,NbPixAv,discu1,&Imax );
  for( k=1 ; k<= irk ; k++)
  {
    MaxDist0= -100000.0;
    Imax= 0;
    for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
    {
      if( discu1[IND1(Ipix)] > MaxDist0)
      {
        MaxDist0= discu1[IND1(Ipix)];
        Imax= Ipix;
      }
    }
    discu1[IND1(Imax)] = -1000.0;
  }    

  #ifdef OPS_DEBUG
MaxDist0=discu1[IND1(Imax)];
    printf( "MaxDist0= %f      Imax= %d\n", MaxDist0, Imax);
  #endif

  // Probleme si l'index de DistMax est nul
  if( Imax == 0)
  {
    cr_op=   SD_ALG_AVHRR_ERR_NUL_DISTANCE;
    msg_err= (char *)SD_ALG_MSG_ERR_NUL_DISTANCE;
    goto fin;
  }

  // UsedPixTest(Imax) = True
  UsedPixTest[IND1(Imax)]= CMN__OUI;

  // Premier prototype
  // Proto(#,1) = RadN(#,Imax)
  for( Ich= 1; Ich <= Nch; Ich++)
    Proto[IND2(Ich,1,SD_BAS_NBK)]= (double)RadN[IND2(Imax,Ich,TailleAvhrr)];


  #ifdef OPS_DEBUG
    for( Ich= 1; Ich <= Nch; Ich++)
      printf( "Proto[%d,1]= %f\n", Ich, Proto[IND2(Ich,1,SD_BAS_NBK)]);
    printf( "NbPro Init: %d\n", *NbPro);
  #endif

  for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
    Dist1[IND1(Ipix)]= 0.0; 

  // Boucle sur les prototypes (Ipro = 2 a NbPro)
  for( Ipro= 2; Ipro <= *NbPro; Ipro++)
  {
    #ifdef OPS_DEBUG
      printf( "=========> Appel SD_ALG_DistPixProto\n");
    #endif

    // Calcul de la distance quadratique au prototype precedent
    // Dist(Ipro-1,*) = DistPixProto{ RadN(#,*), Nch, NbPixAv, Proto(#,Ipro-1) }
    cr_op= SD_ALG_DistPixProto(
                 // Entrees
                  RadN,         // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
                  Nch,                                // int(1)
                  NbPixAv,                            // int(1)
                 &Proto[IND2(1,Ipro-1,SD_BAS_NBK)],   // double( SD_BAS_NBK)
                 // Sorties
                 &Dist[IND2(1,Ipro-1,TailleAvhrr)]);  // double( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)

    // Recherche du pixel non utilise le plus eloigne du prototype precedent
    // Dist1(*) = Dist(Ipro-1,*)
    for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
    {
      Dist1[IND1(Ipix)]= Dist1[IND1(Ipix)]+Dist[IND2(Ipix,Ipro-1,TailleAvhrr)];
      discu1[IND1(Ipix)]=Dist1[IND1(Ipix)];
    }

    #ifdef OPS_DEBUG
      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
        printf( "Dist1[%d]= %f\n", Ipix, Dist1[IND1(Ipix)]); 
    #endif

    if(Ipro == 2) //Initialisation
      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
	discu[IND1(Ipix)] = 0.0;

      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
	{
	  discu[IND1(Ipix)] = discu[IND1(Ipix)] + Dist[IND2(Ipix,Ipro-1,TailleAvhrr)];
	  discu1[IND1(Ipix)] = discu[IND1(Ipix)];
	}
// version 5.0 
// Imax = Index de Max3{ Dist1(*) }
//jlp30 pour la rapidite
//    cr_op = SD_ALG_Index_De_Max3(ctx,NbPixAv,discu1 ,&Imax);
    for(k=1 ; k<=irk ; k++)
    {
    MaxDist0= -100000.0;
    Imax= 0;
      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
      {
        if(discu1[IND1(Ipix)] > MaxDist0)
        {
          MaxDist0= discu1[IND1(Ipix)];
          Imax= Ipix;
        }
      }
      discu1[IND1(Imax)] = -1000.0;
    }

    #ifdef OPS_DEBUG
MaxDist0=discu1[IND1(Imax)];
      printf( "MaxDist0= %f      Imax= %d\n", MaxDist0, Imax);
    #endif

    // Tant que UsedPixTest(Imax) = True
    while( UsedPixTest[IND1(Imax)] == CMN__OUI)
    {
      // Dist1(Imax) = 0.
      Dist1[IND1(Imax)]= 0.0;

// version 5.0 
//     Imax= Index_De_Max3{ Dist1(*) }
//jlp30 pour la rapidite
//     cr_op= SD_ALG_Index_De_Max3(ctx,NbPixAv,Dist1,&Imax );
     for( k=1 ; k<= irk ; k++)
     {
      MaxDist0= -10000000.0;
      Imax= 0;
      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
      {
        if( Dist1[IND1(Ipix)] > MaxDist0)
        {
          MaxDist0= Dist1[IND1(Ipix)];
          Imax= Ipix;
        }
      }
      Dist1[IND1(Imax)] = -1000.0;      
     }
      
    }  // Fin de Tant que

    // UsedPixTest(Imax) = True
    UsedPixTest[IND1(Imax)]= CMN__OUI;

    // Nouveau prototype
    // Proto(#,Ipro) = RadN(#,Imax)
    for( Ich= 1; Ich <= Nch; Ich++)
      Proto[IND2(Ich,Ipro,SD_BAS_NBK)]= (double)RadN[IND2(Imax,Ich,TailleAvhrr)];

    #ifdef OPS_DEBUG
      for( Ich= 1; Ich <= Nch; Ich++)
        printf( "Proto[%d,%d]= %f\n", Ich, Ipro, Proto[IND2(Ich,Ipro,SD_BAS_NBK)]);
    #endif
  } // Fin de boucle sur les prototypes

  #ifdef OPS_DEBUG
    printf( "=========> Appel SD_ALG_DistPixProto\n");
  #endif
  // Calcul de la distance au dernier prototype
  // Dist(NbPro,*) = DistPixProto{ RadN(#,*), Nch, NbPixAv, Proto(#,NbPro) }
  cr_op= SD_ALG_DistPixProto(
            // Entrees
             RadN,         // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
             Nch,                                // int(1)
             NbPixAv,                            // int(1)
            &Proto[IND2(1,*NbPro,SD_BAS_NBK)],   // double( SD_BAS_NBK)
            // Sorties
            &Dist[IND2(1,*NbPro,TailleAvhrr)]);  // double( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)

  // Addition d'un autre prototype egal a la moyenne
  // NbPro = NbPro + 1
  *NbPro= *NbPro + 1;

  // Proto(#,NbPro) = 0.
  for( Ich= 1; Ich <= Nch; Ich++)
    Proto[IND2(Ich,*NbPro,SD_BAS_NBK)]= 0.0;

  // Distance a la moyenne
  // Dist(NbPro,*) = DistZero(*)
  for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
    Dist[IND2(Ipix,*NbPro,TailleAvhrr)]= DistZero[IND1(Ipix)];

  #ifdef OPS_DEBUG
    for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
      printf( "Dist[%d,%d]= %f\n", *NbPro, Ipix, Dist[IND2(Ipix,*NbPro,TailleAvhrr)]); 
    printf( "=========> Appel SD_ALG_LinkPixProto\n");
  #endif

  // Attribution des pixels aux prototypes
  cr_op= SD_ALG_LinkPixProto(
           // Entrees
            Dist,            // double( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NCL_MAX)
// version 5.0 
            2.0*DistMax,         // double(1)
            NbPixAv,         // int(1)
           *NbPro,           // int(1)
           // Sorties
            IdPix,           // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
            NbPixClas,       // int( SD_BAS_NCL_MAX)
            SumDistClas,     // double( SD_BAS_NCL_MAX)
           &SumDistTot);     // double(1)

  #ifdef OPS_DEBUG
    for( Ipix= 1; Ipix <= 20 /*NbPixAv*/; Ipix++)
      printf( "IdPix[%d]= %d\n", Ipix, IdPix[IND1(Ipix)]); 

    for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
      printf( "NbPixClas[%d]= %d\n", Ipro, NbPixClas[IND1(Ipro)]); 

    for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
      printf( "SumDistClas[%d]= %f\n", Ipro, SumDistClas[IND1(Ipro)]); 

    printf( "SumDistTot= %f\n", SumDistTot); 
  #endif

  // Caracteristiques des classes
  // Boucle sur les prototypes (Ipro = $)
  for( Ipro= 1; Ipro <= *NbPro; Ipro++)
  {
    #ifdef OPS_DEBUG
      printf( "=========> Appel SD_ALG_StatClas\n");
    #endif
    cr_op= SD_ALG_StatClas(
              // Entrees
               ProduitCourant,  // t_SD_BAS_DataThreadProduit*
               RadN,            // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
               IdPix,           // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
               Nch,             // int(1)
               NbPixAv,         // int(1)
               Ipro,            // int(1)
              // Sorties
              &Proto[IND2(1,Ipro,SD_BAS_NBK)],               // double( SD_BAS_NBK)
              &Covar[IND3(1,1,Ipro,SD_BAS_NBK,SD_BAS_NBK)]); // double( SD_BAS_NBK * SD_BAS_NBK)
  } // Fin de boucle sur les prototypes

  #ifdef OPS_DEBUG
    printf( "        SortingChannel : %d\n",  SortingChannel);
    printf( "                   Nch : %d\n",  Nch);
    printf( "               NbPixAv : %d\n",  NbPixAv);
    printf( "                 NbPro : %d\n", *NbPro);
    for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
    {
      printf( "      Proto(Ich=%12d,$) : ", Ich);
      for( Ipro= 1; Ipro <= *NbPro; Ipro++)
      {
        printf( "%12.5f", Proto[IND2(Ich,Ipro,SD_BAS_NBK)]);
      }
      printf( "\n");
    }

    for( Ipix= 1; Ipix <= 20 /*NbPixAv*/; Ipix++)
      printf( "IdPix[%d]= %d\n", Ipix, IdPix[IND1(Ipix)]); 

    for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
      printf( "NbPixClas[%d]= %d\n", Ipro, NbPixClas[IND1(Ipro)]); 

    for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
      printf( "SumDistClas[%d]= %f\n", Ipro, SumDistClas[IND1(Ipro)]); 

    printf( "=========> Appel SD_ALG_SortClas\n");
  #endif
  // Reclassement sur les valeurs du canal de classement par ordre decroissant des radiances
  cr_op= SD_ALG_SortClas(
            // Entrees
             SortingChannel,  // int(1)
             Nch,             // int(1)
             NbPixAv,         // int(1)
            *NbPro,           // int(1)
            // Entrees / Sorties
             Proto,           // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
             IdPix,           // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
             NbPixClas,       // int( SD_BAS_NCL_MAX)
             SumDistClas);    // double( SD_BAS_NCL_MAX)

fin:
SD_BAS_MacFREE(discu1);
SD_BAS_MacFREE(discu);
    // Traitement des erreurs
    //================================================================
    if ( cr_op != CMN__STATUT_OK )
    {
        // mise a jour de la variable ctx->cr.errinterne
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
        SD_BAS_ERR_SET_AVHRR,
        cr_op,
        tcProcName_F,
        num_lig,
        msg_err);

        // on imprime le message d'erreur sur la sortie standard
        #ifdef OPS_DEBUG
            SD_BAS_PRINT_ERR_INT(ctx->cr.errinterne.nomfonction,
                                 ctx->cr.errinterne.numligne,
                                 ctx->cr.errinterne.libelle);
        #endif
    }

  // Fin de la fonction InitProto
  #ifdef OPS_DEBUG
    for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
    {
      printf( "      Proto(Ich=%d,$) : ", Ich);
      for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
      {
        printf( "%12.5f", Proto[IND2(Ich,Ipro,SD_BAS_NBK)]);
      }
      printf( "\n");
    }

    for( Ipix= 1; Ipix <= 20 /*NbPixAv*/; Ipix++)
      printf( "IdPix[%d]= %d\n", Ipix, IdPix[IND1(Ipix)]); 

    for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
      printf( "NbPixClas[%d]= %d\n", Ipro, NbPixClas[IND1(Ipro)]); 

    for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
      printf( "SumDistClas[%d]= %f\n", Ipro, SumDistClas[IND1(Ipro)]); 

    printf( "Fin de la fonction InitProto\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN InitProto =================================================================
// ===============================================================================



//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_IterNuDyn
//@
//@ objet :   Une iteration de classification
//@
//@
//@ Parametres :
//@         (ENTREE) ProduitCourant ... : Pour pointer sur les variables temporaires
//@         (ENTREE) RadN ............. : Vecteur de radiance de l'image AVHRR
//@         (ENTREE) NbPixAv .......... : Nombre de pixels du vecteur
//@         (ENTREE) Nch .............. : Nombre de canaux presents
//@         (ENTREE) NbPro ............ : Nombre de prototypes
//@         (ENTREE) NbClusMax ........ : Nombre maximal de clusters
//@         (ENTREE) NbIterMax ........ : Nombre maximal d'iterations
//@         (ENTREE) EigVal ........... : Valeurs propres
//@         (ENTREE) EigVect .......... : Vecteurs propres
//@         (ENTREE) DistAgloInit ..... : Valeur initiale de la distance d'aglomeration
//@         (ENTREE) SplitCutoff ...... : Seuil d'eclatement
//@         (ENTREE) GlueCutoff ....... : Seuil de coalescence
//@         (ENTREE) EigValCutoff ..... : Seuil de valeur propre
//@         (ENTREE) MinIterNCCutOff .. : Nombre d'iteration minimales pour le calcul du seuil des non classifies
//@         (ENTREE) NbNonClassifMax .. : Nombre maximal de non classifies
//@         (ENTREE) SortingChannel ... : Canal de rangement
//@         (ENTREE) DistMax .......... : Distance maximale interclasses
//@         (ENTREE) IterConvCutoff ... : Seuil de convergence
//@
//@         (ENTSOR) Proto ............ : Prototypes de classes
//@         (ENTSOR) IdPix ............ : Flags de pixels valides
//@         (ENTSOR) NbPixClas ........ : Nombre pixels par classe
//@         (ENTSOR) SumDistClas ...... : Somme des distances intra-classes
//@
//@         (SORTIE) NbClass .......... : Nombre de classes
//@         (SORTIE) NoExt ............ : Flags de classes non extremes
//@         (SORTIE) Iter ............. : Numero d'iteration
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_IterNuDyn(
     // Entree
   t_SD_BAS_DataThread*           Contexte_courant,      // t_SD_BAS_DataThread*
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
     float                *RadN,                  // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
     int                   NbPixAv,               // int(1)
     int                   Nch,                   // int(1)
     int                   NbPro,                 // int(1)
     int                   NbClusMax,             // int(1)
     int                   NbIterMax,             // int(1)
/* bma complexes->reels */
     double               *EigVal,                // double( SD_BAS_NBK)
     double               *EigVect,               // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
     double                DistAgloInit,          // double(1)
     double                SplitCutoff,           // double(1)
     double                GlueCutoff,            // double(1)
     double                EigValCutoff,          // double(1)
     int                   MinIterNCCutoff,       // int(1)
     int                   NbNonClassifMax,       // int(1)
     int                   SortingChannel,        // int(1)
     double                DistMax,               // double(1)
     double                IterConvCutoff,        // double(1)
     // Entrees/Sortie
     double               *Proto,                 // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
     int                  *IdPix,                 // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
     int                  *NbPixClas,             // int( SD_BAS_NCL_MAX)
     double               *SumDistClas,           // double( SD_BAS_NCL_MAX)
     // Sortie
     int                  *NbClass,               // int(1)
     unsigned int         *NoExt,                 // unsigned int( SD_BAS_NCL_MAX)
     int                  *Iter                   // int(1)
)
{
  static const char tcProcName_F[] = "SD_ALG_IterNuDyn";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  unsigned int FinIter;
  unsigned int Aglo;
  unsigned int Separ;
  int          TailleAvhrr;                    // Taille d'une image AVHRR
  int          Ipro;                           // Index sur les canaux
  int          Ich;                            // Index sur les canaux
  int          Ipix;                           // Index sur les pixels
  double      *Dist;                           // double( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NCL_MAX)
  int          NbNCMax;                        // Nombre max non classifies
  double       SumDistTot0;
  double       SumDistTot[SD_ALG_MAX_ITER];    // double( SD_ALG_MAX_ITER)
  int          IdProLast;
  double       Covar[SD_BAS_NBK*SD_BAS_NBK];   // double( SD_BAS_NBK * SD_BAS_NBK)


  #ifdef OPS_DEBUG
    printf( "Debut de la fonction IterNuDyn\n----------------------------------\n");
  #endif

  Dist= ProduitCourant->Dist;

  TailleAvhrr= SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR;

  // StopIter = False
  FinIter= CMN__NON;


  // SumDistTot0 = Somme (SumDistClass(i)) pour i = 1 NbPro
  SumDistTot0= 0.0;
  for( Ipro= 1; Ipro <= NbPro; Ipro++)
  {
    SumDistTot0= SumDistTot0 + SumDistClas[IND1(Ipro)];
  }

  // Boucle iterative si StopIter = False (Iter = 2 a NbIterMax)
  *Iter= 2;
  while( (FinIter == CMN__NON) && (*Iter <= NbIterMax))
  {
    // Gestion des mecanismes d'agglomeration et de separation
    // Aglo = True si NbPro > NbClusMax
    if( NbPro > NbClusMax)   Aglo= CMN__OUI;
    else Aglo=CMN__NON;

    // Separ = True si NbPro < NbClusMax + (NbIterMax - Iter - 1)
    //jlp30
    if( NbPro < NbClusMax + (NbIterMax - *Iter - 1))
       Separ= CMN__OUI; 
    else 
       Separ= CMN__NON;

    cr_op= SD_ALG_GestionProto(
             // Entrees
              Contexte_courant,      // t_SD_BAS_DataThread*
              ProduitCourant,  // t_SD_BAS_DataThreadProduit*
              RadN,            // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
              NbPixAv,         // int(1)
              Nch,             // int(1)
              Aglo,            // unsigned int(1)
              Separ,           // unsigned int(1)
/* bma complexes->reels */
              EigVal,          // double( SD_BAS_NBK)
              EigVect,         // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
              SplitCutoff,     // double(1)
              DistAgloInit,    // double(1)
              GlueCutoff,      // double(1)
              EigValCutoff,    // double(1)
             // Entrees / Sorties
              Proto,           // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
             &NbPro,           // int(1)
              IdPix,           // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
              NbPixClas,       // int( SD_BAS_NCL_MAX)
              SumDistClas,     // double( SD_BAS_NCL_MAX)
             // Sorties
              NoExt);          // unsigned int( SD_BAS_NCL_MAX)
     if(cr_op != CMN__STATUT_OK)
     {
         goto fin;
     }

    // Calcul des distances aux prototypes
    // Dist(#,$) = 0.  => modifie en Dist($,*)
    for( Ipro= 1; Ipro <= NbPro; Ipro++)
    {
      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
      {
        Dist[IND2(Ipix,Ipro,TailleAvhrr)]= 0.0;
      }
    }

    // Boucle sur les prototypes (Ipro = $)
    // Dist(Ipro,*) = DistPixProto{ RadN(#,*), Nch, NbPixAv, Proto(#,Ipro) }
    for( Ipro= 1; Ipro <= NbPro; Ipro++)
    {
      cr_op= SD_ALG_DistPixProto(
                 // Entrees
                  RadN,         // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
                  Nch,                              // int(1)
                  NbPixAv,                           // int(1)
                 &Proto[IND2(1,Ipro,SD_BAS_NBK)],   // double( SD_BAS_NBK)
                 // Sorties
                 &Dist[IND2(1,Ipro,TailleAvhrr)]);  // double(SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
    } // Fin de boucle sur les prototypes

    // Attribution des pixels aux classes
    cr_op= SD_ALG_LinkPixProto(
              // Entrees
               Dist,                // double( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NCL_MAX)
               DistMax,             // double(1)
               NbPixAv,             // int(1)
               NbPro,               // int(1)
              // Sorties
               IdPix,                     // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
               NbPixClas,                 // int( SD_BAS_NCL_MAX)
               SumDistClas,               // double( SD_BAS_NCL_MAX)
              &SumDistTot[IND1(*Iter)]);   // double(1)

    // Gestion des non classes
    // Nombre max non classes en fonction de l'iteration
    // NbNCMax = NbNonClassifMax / Min{ MinIterNCCutoff, Max{ (NbIterMax - Iter), 1 }}
    NbNCMax= NbNonClassifMax / MIN( MinIterNCCutoff, MAX( (NbIterMax - *Iter), 1));

    // Si NbPixClas(NbPro+1) >= NbNCMax et Iter =/= NbIterMax alors
    if( (NbPixClas[IND1(NbPro+1)] >= NbNCMax) && (*Iter != NbIterMax))
    {
      // Addition d'une nouvelle classe a partir des non classes

      // IdProLast = NbPro + 1
      IdProLast= NbPro + 1;

      // Dist(IdProLast,*) = 0.
      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
      {
        Dist[IND2(Ipix,IdProLast,TailleAvhrr)]= 0.0;
      }

      // Caracteristiques de la classe
      // [ Proto(#,IdProLast), Covar(#,#,IdProLast) ] = 
      //      StatClas{ RadN(#,*), IdPix(*), Nch, NbPixAv, IdProLast }
      cr_op= SD_ALG_StatClas(
                // Entrees
                 ProduitCourant,  // t_SD_BAS_DataThreadProduit*
                 RadN,            // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
                 IdPix,           // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
                 Nch,             // int(1)
                 NbPixAv,         // int(1)
                 IdProLast,       // int(1)
                // Sorties
                &Proto[IND2(1,IdProLast,SD_BAS_NBK)],  // double( SD_BAS_NBK)
                 Covar);                               // double( SD_BAS_NBK * SD_BAS_NBK)

      // Calcul de la taille de la classe
      // Dist(IdProLast,*) = DistPixProtoClas{ RadN(#,*), Nch, NbPixAv, IdPix(*), IdProLast, Proto(#,IdProLast) }
      cr_op= SD_ALG_DistPixProtoClas(
                // Entrees
                 RadN,         // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
                 Nch,                              // int(1)
                 NbPixAv,                          // int(1)
                 IdPix,                            // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
                 IdProLast,                        // int(1)
                &Proto[IND2(1,IdProLast,SD_BAS_NBK)],   // double( SD_BAS_NBK)
                // Sorties
                &Dist[IND2(1,IdProLast,TailleAvhrr)]);  // double( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)

      // SumDistClas(IdProLast) = somme(...)
      SumDistClas[IND1(IdProLast)]= 0.0;
      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
      {
        SumDistClas[IND1(IdProLast)] += Dist[IND2(Ipix,IdProLast,TailleAvhrr)];
      }

      // SumDistTot(Iter) = SumDistTot(Iter) + SumDistClas(IdProLast)
      SumDistTot[IND1(*Iter)]= SumDistTot[IND1(*Iter)] + SumDistClas[IND1(IdProLast)];

      // NbPro = IdProLast
      NbPro= IdProLast;

    } // Fin de si (NbPixClas(NbPro+1) NbNCMax et Iter NbIterMax)

    // Test d'iteration
    // FinIter = { Iter = NbIterMax} ou 
    // { {NbPro NbClusMax} et {SumDistTot(Iter}/SumDistTot0 < IterConvCutoff} }
    FinIter=    (*Iter == NbIterMax)
             || (    (NbPro <= NbClusMax)
                  && ( (SumDistTot[IND1(*Iter)]/SumDistTot0) < IterConvCutoff));

    // Mise a jour des proprietes des classes
    // Boucle sur les prototypes (Ipro = $)
    for( Ipro= 1; Ipro <= NbPro; Ipro++)
    {
      // StatClas{ RadN(#,*), IdPix(*), Nch, NbPixAv, Ipro }
      //   [ Proto(#,Ipro), Covar(#,#,Ipro) ] = 
      cr_op= SD_ALG_StatClas(
                // Entrees
                 ProduitCourant,  // t_SD_BAS_DataThreadProduit*
                 RadN,            // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
                 IdPix,           // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
                 Nch,             // int(1)
                 NbPixAv,         // int(1)
                 Ipro,            // int(1)
                // Sorties
                &Proto[IND2(1,Ipro,SD_BAS_NBK)],  // double( SD_BAS_NBK)
                 Covar);                          // double( SD_BAS_NBK * SD_BAS_NBK)

    } // Fin de boucle sur les prototypes

    // Reclassement sur les valeurs du canal de classement par ordre decroissant des radiances
    // [ Proto(#,$), IdPix(*), NbPixClas($), SumDistClas($) ] = SortClas{ Proto(#,$), IdPix(*), NbPixClas($), SumDistClas($), SortingChannel, Nch, NbPixAv, NbPro }
    cr_op= SD_ALG_SortClas(
              // Entrees
               SortingChannel,  // int(1)
               Nch,             // int(1)
               NbPixAv,         // int(1)
               NbPro,           // int(1)
              // Entrees / Sorties
               Proto,           // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
               IdPix,           // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
               NbPixClas,       // int( SD_BAS_NCL_MAX)
               SumDistClas);    // double( SD_BAS_NCL_MAX)
    // VERRUE MOR 21/08/2003
  #ifdef OPS_DEBUG
    printf("Nb Iter=%d\n", *Iter);
    for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
    {
      printf( "      Proto(Ich=%12d,$) : ", Ich);
      for( Ipro= 1; Ipro <= NbPro; Ipro++)
      {
        printf( "%12.5f", Proto[IND2(Ich,Ipro,SD_BAS_NBK)]);
      }
      printf( "\n");
    }
    printf("NbPixClass: ");
    for( Ipro= 1; Ipro <= NbPro; Ipro++)
      printf("%d  ", NbPixClas[IND1(Ipro)]);
    printf("\n");

  #endif
    //fin MOR 21/08/2003

    // Si FinIter =True alors
    if( FinIter == CMN__OUI)
    {
      // Caracterisation des non classes 
      // Si NbPixClas(NbPro+1) =/= 0 alors
      if( NbPixClas[IND1(NbPro+1)] != 0)
      {
        // IdProLast = NbPro +1
        IdProLast= NbPro + 1;

        // [ Proto(#,IdProLast), Covar(#,#,IdProLast) ] = 
        // StatClas{ RadN(#,*), IdPix(*), Nch, NbPixAv, IdProLast }
        cr_op= SD_ALG_StatClas(
                  // Entrees
                   ProduitCourant,  // t_SD_BAS_DataThreadProduit*
                   RadN,            // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
                   IdPix,           // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
                   Nch,             // int(1)
                   NbPixAv,         // int(1)
                   IdProLast,       // int(1)
                  // Sorties
                  &Proto[IND2(1,IdProLast,SD_BAS_NBK)],  // double( SD_BAS_NBK)
                   Covar);                               // double( SD_BAS_NBK * SD_BAS_NBK)

      } // Fin de si (NbPixClas(NbPro+1) =/= 0)
    } // Fin de si (FinIter =True)

    *Iter= *Iter + 1;

  } // Fin de boucle iterative si StopIter = False (Iter = 2 a NbIterMax)

  // Nombre de classes identifiees
  // NbClass = NbPro
  *NbClass= NbPro;

fin:
  // Fin de la fonction IterNuDyn
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction IterNuDyn\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN IterNuDyn =================================================================
// ===============================================================================





//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_IterNuDynPost
//@
//@ objet :   Une iteration de classification
//@
//@
//@ Parametres :
//@         (ENTREE) IterLast ......... : Derniere iteration
//@         (ENTREE) NbPixAv .......... : Nombre de pixels du vecteur
//@         (ENTREE) Nch .............. : Nombre de canaux presents
//@         (ENTREE) NbPro ............ : Nombre de prototypes
//@         (ENTREE) NbClusMax ........ : Nombre maximal de clusters
//@         (ENTREE) EigVal ........... : Valeurs propres
//@         (ENTREE) EigVect .......... : Vecteurs propres
//@         (ENTREE) DistAgloInit ..... : Valeur initiale de la distance d'aglomeration
//@         (ENTREE) EigValCutoff ..... : Seuil de valeur propre
//@         (ENTREE) SortingChannel ... : Canal de rangement
//@
//@         (ENTSOR) Proto ............ : Prototypes de classes
//@         (ENTSOR) IdPix ............ : Flags de pixels valides
//@         (ENTSOR) NbPixClas ........ : Nombre pixels par classe
//@         (ENTSOR) SumDistClas ...... : Somme des distances intra-classes
//@         (ENTSOR) NoExt ............ : Flags de classes non extremes
//@
//@         (SORTIE) NbClass .......... : Nombre de classes
//@         (SORTIE) Iter ............. : Numero d'iteration
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_IterNuDynPost(
  // Entree
  t_SD_BAS_DataThreadProduit  *ProduitCourant,  // t_SD_BAS_DataThreadProduit*
  int                          IterLast,        // int(1)
  int                          NbPixAv,         // int(1)
  int                          Nch,             // int(1)
  int                          NbClusMax,       // int(1)
/* bma complexes->reels */
  double                      *EigVal,          // double( SD_BAS_NBK)
  double                      *EigVect,         // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
  double                       DistAgloInit,    // double(1)
  double                       EigValCutoff,    // double(1)
  int                          SortingChannel,  // int(1)
  // Entrees / Sortie
  double                      *Proto,           // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
  int                         *IdPix,           // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  int                         *NbPixClas,       // int( SD_BAS_NCL_MAX)
  double                      *SumDistClas,     // double( SD_BAS_NCL_MAX)
  unsigned int                *NoExt,           // unsigned int( SD_BAS_NCL_MAX)
  // Sortie
  int                         *NbClass,         // int(1)
  int                         *Iter             // int(1)
)
{
  static const char tcProcName_F[] = "SD_ALG_IterNuDynPost";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  unsigned int Aglo;
  unsigned int Separ;
  int          IterMin, IterMax;
  int NbPro;
  NbPro=*NbClass;
  #ifdef OPS_DEBUG
    printf( "Debut de la fonction IterNuDynPost\n----------------------------------\n");
  #endif

  // IterMin = IterLast +1
  IterMin= IterLast + 1;

  // IterMax = IterMin +(NbPro -NbClusMax) -1
  IterMax= IterMin + (NbPro - NbClusMax) - 1;

  // Boucle iterative Iter = IterMin,IterMax
  for( *Iter= IterMin; *Iter <= IterMax; (*Iter)++)
  {
    // Agglomeration forcee
    cr_op= SD_ALG_GestionProtoPost(
             // Entrees
              ProduitCourant,  // t_SD_BAS_DataThreadProduit*
              Nch,             // int(1)
/* bma ajout du nombre de pixels AVHRR dans la signature de la fonction GestionProtoPost*/
              NbPixAv,            // int(1)
/* fin bma */
/* bma complexes->reels */
              EigVal,          // double( SD_BAS_NBK)
              EigVect,         // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
              DistAgloInit,    // double(1)
              EigValCutoff,    // double(1)
             // Entrees / Sortie
              Proto,           // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
             &NbPro,           // int(1)
              IdPix,           // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
              NbPixClas,       // int( SD_BAS_NCL_MAX)
              SumDistClas,     // double( SD_BAS_NCL_MAX)
              NoExt);          // unsigned int( SD_BAS_NCL_MAX)
    // Reclassement sur les valeurs du canal de classement par ordre decroissant des radiances
    cr_op= SD_ALG_SortClas(
            // Entrees
            SortingChannel,  // int(1)
            Nch,             // int(1)
            NbPixAv,         // int(1)
            NbPro,           // int(1)
            // Rnetrees / Sorties
            Proto,           // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
            IdPix,           // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
            NbPixClas,       // int( SD_BAS_NCL_MAX)
            SumDistClas);    // double( SD_BAS_NCL_MAX)

  } // Fin de boucle iterative si StopIter = False (Iter = 2 a NbIterMax)

  // Nombre de classes identifiees
  // NbClass = NbPro
  *NbClass= NbPro;

fin:
  // Fin de la fonction IterNuDynPost
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction IterNuDynPost\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
// FIN IterNuDynPost =============================================================
// ===============================================================================
}


//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_GestionProto
//@
//@ objet :  Construction des prototypes par eclatement et aglomeration
//@
//@
//@ Parametres :
//@         (ENTREE) ProduitCourant  : Pour pointer sur les variables temporaires
//@         (ENTREE) RadN .......... : Vecteur de radiance de l'image AVHRR
//@         (ENTREE) NbPixAv ....... : Nombre de pixels du vecteur
//@         (ENTREE) Nch ........... : Nombre de canaux presents
//@         (ENTREE) Aglo .......... : Flag d'agglomeration de classes
//@         (ENTREE) Separ ......... : Flag d'eclatement (split) de classes
//@         (ENTREE) EigVal ........ : Valeurs propres
//@         (ENTREE) EigVect ....... : Vecteurs propres
//@         (ENTREE) SplitCutoff ... : Seuil d'eclatement
//@         (ENTREE) GlueCutoff .... : Seuil de coalescence
//@         (ENTREE) EigValCutoff .. : Seuil de valeur propre
//@
//@         (ENTSOR) Proto ......... : Prototypes de classes
//@         (ENTSOR) NbPro ......... : Nombre de prototypes
//@         (ENTSOR) NbPixClas ..... : Nombre pixels par classe
//@         (ENTSOR) SumDistClas ... : Somme des distances intra-classes
//@
//@         (SORTIE) NoExt ......... : Flags de classes non extremes
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_GestionProto(
  // Entrees
  t_SD_BAS_DataThread* ctx,      // t_SD_BAS_DataThread*
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  float                *RadN,               // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
  int                   NbPixAv,            // int(1)
  int                   Nch,                // int(1)
  unsigned int          Aglo,               // unsigned int(1)
  unsigned int          Separ,              // unsigned int(1)
/* bma complexes->reels */
  double               *EigVal,             // double( SD_BAS_NBK)
  double               *EigVect,            // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
  double                SplitCutoff,        // double(1)
  double                DistAgloInit,       // double(1)
  double                GlueCutoff,         // double(1)
  double                EigValCutoff,       // double(1)
  // Entrees / Sortie
  double               *Proto,              // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
  int                  *NbPro,              // int(1)
  int                  *IdPix,              // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  int                  *NbPixClas,          // int( SD_BAS_NCL_MAX)
  double               *SumDistClas,        // double( SD_BAS_NCL_MAX)
  // Sorties
  unsigned int         *NoExt               // unsigned int( SD_BAS_NCL_MAX)
)
{
  static const char tcProcName_F[] = "SD_ALG_GestionProto";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  int     Ipro, Jpro;
  int     Ich, Jch;
  int     Ipix;
  int     Ivect;
  int     Id;
/* bma complexes->reels */
  double  ProtoEigVect[SD_BAS_NBK * SD_BAS_NCL_MAX];   // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
/* fin bma */
  double  DistAglo[SD_BAS_NCL_MAX * SD_BAS_NCL_MAX];       // double( SD_BAS_NCL_MAX * SD_BAS_NCL_MAX)
  double  ProtoAglo[SD_BAS_NBK];                   // double( SD_BAS_NBK)
  int    *IdPixAglo;                               // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  double *Dist;                                    // double( SD_BAS_NCL_MAX * SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  double  SumDistClasN[SD_BAS_NCL_MAX];                // double( SD_BAS_NCL_MAX)
  int     IdProSort[SD_BAS_NCL_MAX];                   // int( SD_BAS_NCL_MAX)
/* bma complexes->reels */
  int     NbNoExt;
  double  Minval, Maxval;
/* fin bma */
  int     NbVect;
  double  EigValCutoff2;
  double  DistMin;
  int     IdAglo1;
  int     IdAglo2;
  int     NbPixClasAglo;
  int     TailleAvhrr;
/* bma complexes->reels */
  double  DistIJ;
//  double  ModEigVal;
/* fin bma */
  double  DistAglo1;
  double  DistAglo2;
  double  Diff;
  double  SumDistClasAglo;
  int     IdProLast;
  int     NbPixLast;
  double  MaxDist0;
  int     IdMax;
  int k;
//jlp30 pour la rapidite
  const int irk=3;
  double *discu1;
  char *msg_err = (char*) NULL;          // Message d'erreur
  int num_lig = 0;            // Ligne d'erreur
  
  discu1=NULL;
  discu1 = (double *) calloc( NbPixAv,sizeof(double) );
 if( (discu1 == NULL)  )
  {
    cr_op   = SD_ALG_AVHRR_ERR_ALLOCATION;
    msg_err= (char *)SD_BAS_MSG_ALLOC;
    num_lig= __LINE__;

    goto fin;
  } 

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction GestionProto\n----------------------------------\n");
  #endif

  TailleAvhrr= SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR;

  IdPixAglo= ProduitCourant->IdPixAglo;
  Dist= ProduitCourant->Dist;

  // Definitions
  // (#) (1 a Nch) nombre de canaux/pseudo-canaux Avhrr utilises
  // (*) (1 a NbPixAv) nombre de pixels Avhrr
  // ($) (1 a NbPro) nombre de classes prototype

  // Si Aglo = True alors
  if( Aglo == CMN__OUI)
  {
    // Mecanisme d'agglomeration
    // Projection sur les vecteurs propres
    // ProtoEigVect(#,$) = EigVect(#,#) Proto(#,$)
    for( Ipro= 1; Ipro <= *NbPro; Ipro++)
    {
      for( Ich= 1; Ich <= Nch; Ich++)
      {
/* bma complexes->reels */
        ProtoEigVect[IND2(Ich,Ipro,SD_BAS_NBK)]= 0.0;
/* fin bma */
/* bma la boucle doit aller jusqu'a Nch et pas Nbpro*/
/*
        for( Jch= 1; Jch < *NbPro; Jch++)
*/
//jlp 2905 modif <= pour correction validation
        for( Jch= 1; Jch <= Nch; Jch++)
/* fin bma */
        {
/* bma complexes->reels */
	  //MOR 22/08/2003 pas de somme
// EigVect est stockee en ligne 
          ProtoEigVect[IND2(Ich,Ipro,SD_BAS_NBK)]+=   EigVect[IND2(Ich,Jch,SD_BAS_NBK)]
                                                   * Proto[IND2(Jch,Ipro,SD_BAS_NBK)];
//          ProtoEigVect[IND2(Ich,Ipro,SD_BAS_NBK)]+=   EigVect[IND2(Jch,Ich,SD_BAS_NBK)]
//                                                   * Proto[IND2(Ich,Ipro,SD_BAS_NBK)];
/* fin bma */
        }
      }
    }
  #ifdef OPS_DEBUG
//VERRUE
    printf("VERRUE EigVect\n");
    for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
    {
 
      for( Jch= 1; Jch <= SD_BAS_NBK; Jch++)
	{
	  printf( "%9.6f   ",  EigVect[IND2(Ich,Jch,SD_BAS_NBK)]);
	}
      printf( "|\n");
    }
  #endif

    // Recherche des classes non extremes sur les deux premiers vecteurs propres
    // NoExt($) = True
    for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)      NoExt[IND1(Ipro)]= CMN__OUI;

    // Boucle sur les vecteurs propres (Ivect = 1 a 2)
    for( Ivect= 1; Ivect <= 2; Ivect++)
    {
      // Id = index de Max{ ProtoEigVect(Ivect,$) }
      Id= 1;
      Maxval=  ProtoEigVect[IND2(Ivect,Id,SD_BAS_NBK)];
      for( Ipro= 1; Ipro <= *NbPro; Ipro++)
      {
        // Id= index de Max{ ProtoEigVect(Ivect,$) }
/* bma complexes->reels */
        if( ProtoEigVect[IND2(Ivect,Ipro,SD_BAS_NBK)] > Maxval)
        {
          Maxval= ProtoEigVect[IND2(Ivect,Ipro,SD_BAS_NBK)];
          Id= Ipro;
        }
/* fin bma */
      }

      // NoExt(Id) = False
      NoExt[IND1(Id)]= CMN__NON;

     // Id = index de Min{ ProtoEigVect(Ivect,$) }
      Id= 0;
      Minval= MAX_DOUBLE;
      for( Ipro= 1; Ipro <= *NbPro; Ipro++)
      {
        // Id= index de Max{ ProtoEigVect(Ivect,$) }
/* bma complexes->reels */
        if( ProtoEigVect[IND2(Ivect,Ipro,SD_BAS_NBK)] < Minval)
        {
          Minval= ProtoEigVect[IND2(Ivect,Ipro,SD_BAS_NBK)];
          Id= Ipro;
        }
/* fin bma */
      }

      // NoExt(Id) = False
      NoExt[IND1(Id)]= CMN__NON;

    } //Fin de boucle sur les vecteurs propres

    // NbNoExt = 0
    NbNoExt= 0;

    // Boucle sur les prototypes (Ipro = 1 a NbPro)
    for( Ipro= 1; Ipro <= *NbPro; Ipro++)
    {
      // Si NoExt(Ipro) = True alors NbNoExt = NbNoExt + 1 
      if( NoExt[IND1(Ipro)] == CMN__OUI)   NbNoExt= NbNoExt + 1;
    }

    // Si NbNoExt   2 alors
    if( NbNoExt >= 2)
    {
      // Limitation vecteurs propres hors bruit
      // NbVect= Max( index( EigVal(#)/EigVal(1) >= EigValCutoff ) }
      //MOR 01 sept 2003
      //NbVect= 0;
      //EigValCutoff2= EigValCutoff * EigValCutoff;
/* bma complexes->reels */

      for( Ich= 1; Ich <= Nch; Ich++)
      {
	//MOR 22/08/2003 pas de carre
        if( (EigVal[IND1(Ich)] / EigVal[IND1(1)]) >= EigValCutoff)     NbVect= Ich;
      }
/* fin bma */

      // Calcul des distances entre les agglomerables (non extremes)
      // DistAglo($,$) = DistAgloInit
      for( Ipro= 1; Ipro <= *NbPro; Ipro++)
      {
        for( Jpro= 1; Jpro <= *NbPro; Jpro++)
        {
          DistAglo[IND2(Ipro,Jpro,SD_BAS_NCL_MAX)]= DistAgloInit;
        }
      }
        
      // Boucle sur les Prototypes (Ipro = $) si NoExt(Ipro) = True
      for( Ipro= 2; Ipro <= *NbPro; Ipro++)
      {
        if( NoExt[IND1(Ipro)] == CMN__OUI)
        {
          // Boucle sur les prototypes (Jpro = Ipro - 1) si NoExt(Jpro) = True
          for( Jpro= 1; Jpro <= Ipro-1; Jpro++)
          {
            if( NoExt[IND1(Jpro)] == CMN__OUI)
            {
//jlp30 erreur de dimension
              DistAglo[IND2(Ipro,Jpro,SD_BAS_NCL_MAX)]= 0.0;
              for( Ich= 1; Ich <= NbVect; Ich++)
              {
/* bma complexes->reels */
                Diff=   ProtoEigVect[IND2(Ich,Ipro,SD_BAS_NBK)]
                      - ProtoEigVect[IND2(Ich,Jpro,SD_BAS_NBK)];
                DistIJ= Diff * Diff;
                DistAglo[IND2(Ipro,Jpro,SD_BAS_NCL_MAX)]=   DistAglo[IND2(Ipro,Jpro,SD_BAS_NCL_MAX)]
                                                      + DistIJ / EigVal[IND1(Ich)];
/* fin bma */
              }
            }
          }
        }
      }
        
      // Agglomeration des 2 classes les plus proches
      // [DistMin, IdAglo1, IdAglo2] = MinDistAgloLoc{ DistAglo($,$), NbPro }
      cr_op= SD_ALG_MinDistAgloLoc(
                                    // Entrees
                                     DistAglo,    // double( SD_BAS_NCL_MAX * SD_BAS_NCL_MAX)
                                    *NbPro,       // int(1)
                                    // Sorties
                                    &DistMin,     // double(1)
                                    &IdAglo1,     // int(1)
                                    &IdAglo2);    // int(1)

      // Si DistMin < GlueCutoff alors
      if( DistMin < GlueCutoff)
      {
        // Caracteristique de la classe resultante
        // NbPixClasAglo = NbPixClas(IdAglo1) + NbPixClas(IdAglo2)
        NbPixClasAglo= NbPixClas[IND1(IdAglo1)] + NbPixClas[IND1(IdAglo2)];

        DistAglo1= 0.0;
        DistAglo2= 0.0;
        
        // Canaux
        for( Ich= 1; Ich <= Nch; Ich++)
        {
          // ProtoAglo(#) =   [ Proto(#,IdAglo1)* NbPixClas(IdAglo1) 
          //                + Proto(#,IdAglo2)* NbPixClas(IdAglo2) ] / NbPixClasAglo
          ProtoAglo[IND1(Ich)]= (   Proto[IND2(Ich,IdAglo1,SD_BAS_NBK)]
                                  * NbPixClas[IND1(IdAglo1)]
                                  + Proto[IND2(Ich,IdAglo2,SD_BAS_NBK)]
                                  * NbPixClas[IND1(IdAglo2)] )
                                / (double)NbPixClasAglo;

          // DistAglo1= somme( ...)
          // DistAglo2= somme( ...)
          Diff= Proto[IND2(Ich,IdAglo1,SD_BAS_NBK)] - ProtoAglo[IND1(Ich)];
          DistAglo1= DistAglo1 + (Diff * Diff);
          Diff= Proto[IND2(Ich,IdAglo2,SD_BAS_NBK)] - ProtoAglo[IND1(Ich)];
          DistAglo2= DistAglo2 + (Diff * Diff);

        }

          // SumDistClasAglo = SumDistClas(IdAglo1) 
          //   + NbPixClas(IdAglo1)*DistAglo1 + NbPixClas(IdAglo2)*DistAglo2
          SumDistClasAglo=   SumDistClas[IND1(IdAglo1)]
                           + NbPixClas[IND1(IdAglo1)] * DistAglo1
                           + NbPixClas[IND1(IdAglo2)] * DistAglo2;

          // Reclassement des prototypes + non classes

          // IdPixAglo(*) = IdPix(*)
          for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
          {
            IdPixAglo[IND1(Ipix)]= IdPix[IND1(Ipix)];
          }

          // Boucle sur les prototypes (Ipro = 1 a NbPro +1)
          for( Ipro= 1; Ipro <= (*NbPro + 1); Ipro++)
          {
            // Si Ipro > IdAglo2 alors
            if( Ipro > IdAglo2)
            {
              // Proto(#,Ipro-1) = Proto(#,Ipro) et Proto(#,Ipro) = 0.
              for( Ich= 1; Ich <= Nch; Ich++)
              {
                Proto[IND2(Ich,Ipro-1,SD_BAS_NBK)]= Proto[IND2(Ich,Ipro,SD_BAS_NBK)];
                Proto[IND2(Ich,Ipro,SD_BAS_NBK)]= 0.0;
              } //FIN for Ich

              // SumDistClas(Ipro-1) = SumDistClas(Ipro) et SumDistClas(Ipro) =  0.
              SumDistClas[IND1(Ipro-1)]= SumDistClas[IND1(Ipro)];
              SumDistClas[IND1(Ipro)]= 0.0;

              // NbPixClas(Ipro-1) = NbPixClas(Ipro)
              NbPixClas[IND1(Ipro-1)]= NbPixClas[IND1(Ipro)];

              // NoExt(Ipro-1) = NoExt(Ipro)
              NoExt[IND1(Ipro-1)]= NoExt[IND1(Ipro)];
            } // Fin de si
//jlp30 
         } // Fin de boucle sur les prototypes
         // Boucle sur les pixels (Ipix = *)
         for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
         {
              // Si IdPix(Ipix) = IdAglo2 alors IdPixAglo(Ipix) = IdAglo1
              if( IdPix[IND1(Ipix)] == IdAglo2)
                IdPixAglo[IND1(Ipix)]= IdAglo1;

              // Si IdPix(Ipix) > IdAglo2 alors IdPixAglo(Ipix) = IdPix(Ipix) - 1
              if( IdPix[IND1(Ipix)] > IdAglo2)
                IdPixAglo[IND1(Ipix)]= IdPix[IND1(Ipix)] - 1;

         } // Fin de boucle sur les pixels

 
          // Proto(#,IdAglo1) = ProtoAglo(#)
            for( Ich= 1; Ich <= Nch; Ich++)
                Proto[IND2(Ich,IdAglo1,SD_BAS_NBK)]= ProtoAglo[IND1(Ich)];
          // SumDistClas(IdAglo1) = SumDistClasAglo
          SumDistClas[IND1(IdAglo1)]= SumDistClasAglo;

          // NbPixClas(IdAglo1) = NbPixClasAglo
          NbPixClas[IND1(IdAglo1)]= NbPixClasAglo;

          // NoExt(IdAglo1) = True
          NoExt[IND1(IdAglo1)]= CMN__OUI;

          // IdPix(*) = IdPixAglo(*)
          for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
          {
            IdPix[IND1(Ipix)]= IdPixAglo[IND1(Ipix)];
          }

          // NbPro = NbPro - 1
          *NbPro= *NbPro - 1;
      } // Fin de si (DistMin < GlueCutoff)
    } // Fin de si (NbNoExt   2)
  } // Fin de si (Aglo = True)

  // Si Separ = True
  if( Separ == CMN__OUI)
  {
    // Mecanisme de Separation
    // Classement en fonction des dispersions
    // Distance intra-classe normalisee
    // SumDistClasN($) = SumDistClas($) / Max{ NbPixClas($), SplitCutoff }
    for( Ipro= 1; Ipro <= *NbPro; Ipro++)
    {
      SumDistClasN[IND1(Ipro)]=   SumDistClas[IND1(Ipro)]
                                / MAX( NbPixClas[IND1(Ipro)], SplitCutoff);
    }
    
    // Tri ascendant
    // IdProSort($) = Tri ascendant{ SumDistClasN }
    cr_op= SD_ALG_Tri_ascendant( // Entrees
  				  ctx,      // t_SD_BAS_DataThread*
                                  SumDistClasN,   // double( SD_BAS_NCL_MAX)
                                 *NbPro,          // int(1)
                                 // Sorties
                                  IdProSort);     // int( SD_BAS_NCL_MAX)
    if ( cr_op != CMN__STATUT_OK )
    {
    goto fin;
    }

    // IdProLast = IdProSort(NbPro)
    IdProLast= IdProSort[IND1(*NbPro)];

    // NbPixLast = NbPixClas(IdProLast)
    NbPixLast= NbPixClas[IND1(IdProLast)];

    // Si NbPixLast > SplitCutoff alors
    if( NbPixLast > SplitCutoff)
    {
      // Recherche des extrema dans la classe la plus forte et separation
      // Dist(IdProLast,*) = DistPixProtoClas{ RadN(#,*), Nch, NbPixAv, IdPix(*), IdProLast, Proto(#,IdProLast) }
      cr_op= SD_ALG_DistPixProtoClas(
                // Entrees
                 RadN,                                  // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
                 Nch,                                   // int(1)
                 NbPixAv,                               // int(1)
                 IdPix,                                 // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
                 IdProLast,                             // int(1)
                &Proto[IND2(1,IdProLast,SD_BAS_NBK)],   // double( SD_BAS_NBK)
                // Sorties
                &Dist[IND2(1,IdProLast,TailleAvhrr)]);  // double( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)

      // IdMax= Index de Max{Dist(IdProLast,*)}
      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
        discu1[IND1(Ipix)] = Dist[IND2(Ipix,IdProLast,TailleAvhrr)];
// fonction index de max 3
// version 5.0 
//jlp30 pour la rapidite
// Imax = Index de Max3{ Dist1(Idprolast,*) }
//    cr_op= SD_ALG_Index_De_Max3(ctx,NbPixAv,discu1, &IdMax);
      for(k=1 ; k<=irk ; k++)
      {
          MaxDist0= -1000000.0;
        IdMax= 0;
        for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
        {
          if( discu1[IND1(Ipix)] > MaxDist0)
          {
            MaxDist0= discu1[IND1(Ipix)];
            IdMax= Ipix;
           }
        }
        discu1[IND1(IdMax)] = -1000.0;
      }

      // Proto(#,IdProLast) = RadN(#,IdMax)
      for( Ich= 1; Ich <= Nch; Ich++)
        Proto[IND2(Ich,IdProLast,SD_BAS_NBK)]= (double)RadN[IND2(IdMax,Ich,TailleAvhrr)];

      // Dist(IdProLast,*) = DistPixProtoClas{ RadN(#,*), Nch, NbPixAv, IdPix(*), IdProLast, Proto(#,IdProLast) }
      cr_op= SD_ALG_DistPixProtoClas(
                // Entrees
                 RadN,                                  // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
                 Nch,                                   // int(1)
                 NbPixAv,                               // int(1)
                 IdPix,                                 // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
                 IdProLast,                             // int(1)
                &Proto[IND2(1,IdProLast,SD_BAS_NBK)],   // double( SD_BAS_NBK)
                // Sorties
                &Dist[IND2(1,IdProLast,TailleAvhrr)]);  // double( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)

      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
        discu1[IND1(Ipix)] = Dist[IND2(Ipix,IdProLast,TailleAvhrr)];
// fonction index de max 3
// version 5.0 
//jlp30 pour la rapidite
// Imax = Index de Max3{ Dist1(Idprolast,*) }
//    cr_op= SD_ALG_Index_De_Max3(ctx,NbPixAv,discu1,&IdMax );
      for(k=1 ; k<=irk ; k++)
      {
       MaxDist0= -1000000.0;
         IdMax= 0;
        for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
        {
          if( discu1[IND1(Ipix)] > MaxDist0)
          {
            MaxDist0= discu1[IND1(Ipix)];
            IdMax= Ipix;
          }
        }
        discu1[IND1(IdMax)] = -1000.0;
      }

      // Proto(#,NbPro+1) = RadN(#,IdMax)
      for( Ich= 1; Ich <= Nch; Ich++)
        Proto[IND2(Ich,*NbPro+1,SD_BAS_NBK)]= (double)RadN[IND2(IdMax,Ich,TailleAvhrr)];
      
  #ifdef OPS_DEBUG
    for( Ich= 1; Ich <= Nch; Ich++)
      printf( "Proto[%d,%d]= %f\n", Ich, *NbPro+1, Proto[IND2(Ich,*NbPro+1,SD_BAS_NBK)]); 
  #endif

      // NbPro = NbPro + 1
      *NbPro= *NbPro + 1;

    } // Fin de si (NbPixLast > SplitCutoff)
  } // Fin de si (Separ = True)


fin:
  SD_BAS_MacFREE(discu1);
    // Traitement des erreurs
    //================================================================
    if ( cr_op != CMN__STATUT_OK )
    {
        // mise a jour de la variable ctx->cr.errinterne
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
        SD_BAS_ERR_SET_AVHRR,
        cr_op,
        tcProcName_F,
        num_lig,
        msg_err);

        // on imprime le message d'erreur sur la sortie standard
        #ifdef OPS_DEBUG
            SD_BAS_PRINT_ERR_INT(ctx->cr.errinterne.nomfonction,
                                 ctx->cr.errinterne.numligne,
                                 ctx->cr.errinterne.libelle);
        #endif
    }


  // Fin de la fonction GestionProto
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction GestionProto\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN GestionProto ==============================================================
// ===============================================================================




//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_GestionProtoPost
//@
//@ objet :  Agglomeration forcee des 2 classes non extremes les plus proches
//@
//@
//@ Parametres :
//@         (ENTREE) ProduitCourant  : Pour pointer sur les variables temporaires
//@         (ENTREE) Nch ........... : Nombre de canaux presents
//@         (ENTREE) NbPixAv........ : Nombre de Pixels Avhrr
//@         (ENTREE) EigVal ........ : Valeurs propres
//@         (ENTREE) EigVect ....... : Vecteurs propres
//@         (ENTREE) DistAgloInit .. : Valeur initiale de la distance d'aglomeration
//@         (ENTREE) EigValCutoff .. : Seuil de valeur propre
//@
//@         (ENTSOR) Proto ......... : Prototypes de classes
//@         (ENTSOR) NbPro ......... : Nombre de prototypes
//@         (ENTSOR) IdPix ......... : Flags de pixels valides
//@         (ENTSOR) NbPixClas ..... : Nombre pixels par classe
//@         (ENTSOR) SumDistClas ... : Somme des distances intra-classes
//@         (ENTSOR) NoExt ......... : Flags de classes non extremes
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_GestionProtoPost(
  // Entrees
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  int                   Nch,                // int(1)
/* bma ajout du nombre de pixels AVHRR dans la signature de la fonction GestionProtoPost*/
  int                          NbPixAv,            // int(1)
/* fin bma */
/* bma complexes->reels */
  double               *EigVal,             // double( SD_BAS_NBK)
  double               *EigVect,            // double( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */
  double                DistAgloInit,       // double(1)
  double                EigValCutoff,       // double(1)
  // Entrees / Sortie
  double               *Proto,              // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
  int                  *NbPro,              // int(1)
  int                  *IdPix,              // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  int                  *NbPixClas,          // int( SD_BAS_NCL_MAX)
  double               *SumDistClas,        // double( SD_BAS_NCL_MAX)
  unsigned int         *NoExt               // unsigned int( SD_BAS_NCL_MAX)
)
{
  static const char tcProcName_F[] = "SD_ALG_GestionProtoPost";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  int     Ipro, Jpro;
  int     Ich, Jch;
  int     Ipix;
  int     Ivect;
  int     Id;
/* bma complexes->reels */
  double  ProtoEigVect[SD_BAS_NBK * SD_BAS_NCL_MAX];   // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
/* fin bma */
  double  DistAglo[SD_BAS_NCL_MAX * SD_BAS_NCL_MAX];       // double( SD_BAS_NCL_MAX * SD_BAS_NCL_MAX)
  double  ProtoAglo[SD_BAS_NBK];                   // double( SD_BAS_NBK)
  int    *IdPixAglo;                               // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  double  SumDistClasN[SD_BAS_NCL_MAX];                // double( SD_BAS_NCL_MAX)
  double  IdProSort[SD_BAS_NCL_MAX];                   // double( SD_BAS_NCL_MAX)
/* bma complexes->reels */
  int     NbNoExt;
  double  Minval, Maxval;
/* fin bma */
  int     NbVect;
  double  EigValCutoff2;
  double  DistMin;
  int     IdAglo1;
  int     IdAglo2;
  int     NbPixClasAglo;
  int     TailleAvhrr;
/* bma complexes->reels */
  double  Diff;
  double  DistIJ;
//  double  ModEigVal;
/* fin bma */
  double  DistAglo1;
  double  DistAglo2;
  double  SumDistClasAglo;

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction GestionProtoPost\n----------------------------------\n");
  #endif

  TailleAvhrr= SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR;

  IdPixAglo= ProduitCourant->IdPixAglo;

  // Definitions
  // (#) (1 a Nch) nombre de canaux/pseudo-canaux Avhrr utilises
  // (*) (1 a NbPixAv) nombre de pixels Avhrr
  // ($) (1 a NbPro) nombre de classes prototype

  // Projection sur les vecteurs propres
  // ProtoEigVect(#,$) = EigVect(#,#) Proto(#,$)   (Produit scalaire)
  for( Ipro= 1; Ipro <= *NbPro; Ipro++)
  {
    for( Ich= 1; Ich <= Nch; Ich++)
    {
      ProtoEigVect[IND2(Ich,Ipro,SD_BAS_NBK)]= 0.0;
//jlp 2905 modif <= Nch pour correction validation
      for( Jch= 1; Jch <= Nch; Jch++)
      {
        // Sommation
// EigVect est stockee en ligne 
          ProtoEigVect[IND2(Ich,Ipro,SD_BAS_NBK)]+=   EigVect[IND2(Ich,Jch,SD_BAS_NBK)]
                                                   * Proto[IND2(Jch,Ipro,SD_BAS_NBK)];
      }
    }
  }

  // Limitation vecteurs propres hors bruit
  // NbVect= Max( index( EigVal(#)/EigVal(1) >= EigValCutoff ) }
  NbVect= 0;
//jlp  25/05 correction seuil
//  EigValCutoff2= EigValCutoff * EigValCutoff;

/* bma complexes->reels */
  for( Ich= 1; Ich <= Nch; Ich++)
  {
//jlp? 25/05 correction seuil
    if( (EigVal[IND1(Ich)] / EigVal[IND1(1)]) >= EigValCutoff)     NbVect= Ich;
  }
/* fin bma */

  // Calcul des distances entre les agglomerables (non extremes)
  for( Ipro= 1; Ipro <= *NbPro; Ipro++)
  {
    for( Jpro= 1; Jpro <= *NbPro; Jpro++)
    {
      DistAglo[IND2(Ipro,Jpro,SD_BAS_NCL_MAX)]= DistAgloInit;
    }
  }
        
  // Double boucle sur les Prototypes (Ipro, Jpro = $) si NoExt(Ipro) & NoExt(Jpro) = True
  for( Ipro= 2; Ipro <= *NbPro; Ipro++)
  {
    if( NoExt[IND1(Ipro)] == CMN__OUI)
    {
      for( Jpro= 1; Jpro <= Ipro-1; Jpro++)
     {
        if( NoExt[IND1(Ipro)] == CMN__OUI)
        {
          DistAglo[IND2(Ipro,Jpro,SD_BAS_NCL_MAX)]= 0.0;
/* bma :  a boucle ne doit pas aller a Nch mais NbVect*/
/*
          for( Ich= 1; Ich <= Nch; Ich++)
*/
          for( Ich= 1; Ich <= NbVect; Ich++)
/* fin bma */
          {
/* bma complexes->reels */
            Diff=   ProtoEigVect[IND2(Ich,Ipro,SD_BAS_NBK)]
                  - ProtoEigVect[IND2(Ich,Jpro,SD_BAS_NBK)];
            DistIJ= Diff * Diff;
            DistAglo[IND2(Ipro,Jpro,SD_BAS_NCL_MAX)]=   DistAglo[IND2(Ipro,Jpro,SD_BAS_NCL_MAX)]
                                                  + DistIJ / EigVal[IND1(Ich)];
/* fin bma */
          }
        }
      }
    }
  } // Fin de double boucle sur les prototypes

  // Agglomeration des 2 classes les plus proches
  // [DistMin, IdAglo1, IdAglo2] = MinDistAgloLoc{ DistAglo($,$), NbPro }
  cr_op= SD_ALG_MinDistAgloLoc( // Entrees
                                 DistAglo,    // double( SD_BAS_NCL_MAX * SD_BAS_NCL_MAX)
                                *NbPro,       // int(1)
                                // Sorties
                                &DistMin,     // double(1)
                                &IdAglo1,     // int(1)
                                &IdAglo2);    // int(1)

  // Caracteristique de la classe resultante
  // NbPixClasAglo = NbPixClas(IdAglo1) + NbPixClas(IdAglo2)
  NbPixClasAglo= NbPixClas[IND1(IdAglo1)] + NbPixClas[IND1(IdAglo2)];

  DistAglo1= 0.0;
  DistAglo2= 0.0;

  // Canaux
  for( Ich= 1; Ich <= Nch; Ich++)
  {
    // ProtoAglo(#) =   [ Proto(#,IdAglo1)* NbPixClas(IdAglo1) 
    //                + Proto(#,IdAglo2)* NbPixClas(IdAglo2) ] / NbPixClasAglo
    ProtoAglo[IND1(Ich)]= (   Proto[IND2(Ich,IdAglo1,SD_BAS_NBK)]
                            * NbPixClas[IND1(IdAglo1)]
                            + Proto[IND2(Ich,IdAglo2,SD_BAS_NBK)]
                            * NbPixClas[IND1(IdAglo2)] )
                          / NbPixClasAglo;

    Diff= Proto[IND2(Ich,IdAglo1,SD_BAS_NBK)] - ProtoAglo[IND1(Ich)];
    DistAglo1= DistAglo1 + (Diff * Diff);

    Diff= Proto[IND2(Ich,IdAglo2,SD_BAS_NBK)] - ProtoAglo[IND1(Ich)];
    DistAglo2= DistAglo2 + (Diff * Diff);

  } //FIN for Ich (mal place)

    // SumDistClasAglo = SumDistClas(IdAglo1) 
    //   + NbPixClas(IdAglo1)*DistAglo1 + NbPixClas(IdAglo2)*DistAglo2
    SumDistClasAglo=   SumDistClas[IND1(IdAglo1)]
                     + NbPixClas[IND1(IdAglo1)] * DistAglo1
                     + NbPixClas[IND1(IdAglo2)] * DistAglo2;

    // Reclassement des prototypes + non classes

    // IdPixAglo(*) = IdPix(*)
    for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
    {
      IdPixAglo[IND1(Ipix)]= IdPix[IND1(Ipix)];
    }

    // Boucle sur les prototypes (Ipro = 1 a NbPro +1)
    for( Ipro= 1; Ipro <= (*NbPro + 1); Ipro++)
    {
      // Si Ipro > IdAglo2 alors
      if( Ipro > IdAglo2)
      {
        for( Ich= 1; Ich <= Nch; Ich++)
        {
          // Proto(#,Ipro-1) = Proto(#,Ipro) et Proto(#,Ipro) = 0.
          Proto[IND2(Ich,Ipro-1,SD_BAS_NBK)]= Proto[IND2(Ich,Ipro,SD_BAS_NBK)];
          Proto[IND2(Ich,Ipro,SD_BAS_NBK)]= 0.0;
        } //FIN for Ich

        // SumDistClas(Ipro-1) = SumDistClas(Ipro) et SumDistClas(Ipro) =  0.
        SumDistClas[IND1(Ipro-1)]= SumDistClas[IND1(Ipro)];
        SumDistClas[IND1(Ipro)]= 0.0;

        // NbPixClas(Ipro-1) = NbPixClas(Ipro)
        NbPixClas[IND1(Ipro-1)]= NbPixClas[IND1(Ipro)];

        // NoExt(Ipro-1) = NoExt(Ipro)
        NoExt[IND1(Ipro-1)]= NoExt[IND1(Ipro)];
      } // Fin de si
    } // Fin de boucle sur les prototypes

      // Boucle sur les pixels (Ipix = *)
      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
      {
        // Si IdPix(Ipix) = IdAglo2 alors IdPixAglo(Ipix) = IdAglo1
        if( IdPix[IND1(Ipix)] == IdAglo2)
          IdPixAglo[IND1(Ipix)]= IdAglo1;

        // Si IdPix(Ipix) > IdAglo2 alors IdPixAglo(Ipix) = IdPix(Ipix) - 1
        if( IdPix[IND1(Ipix)] > IdAglo2)
          IdPixAglo[IND1(Ipix)]= IdPix[IND1(Ipix)] - 1;

      } // Fin de boucle sur les pixels


    // Proto(#,IdAglo1) = ProtoAglo(#)
    for( Ich= 1; Ich <= Nch; Ich++)
      Proto[IND2(Ich,IdAglo1,SD_BAS_NBK)]= ProtoAglo[IND1(Ich)];

    // SumDistClas(IdAglo1) = SumDistClasAglo
    SumDistClas[IND1(IdAglo1)]= SumDistClasAglo;

    // NbPixClas(IdAglo1) = NbPixClasAglo
    NbPixClas[IND1(IdAglo1)]= NbPixClasAglo;

    // NoExt(IdAglo1) = True
    NoExt[IND1(IdAglo1)]= CMN__OUI;

    // IdPix(*) = IdPixAglo(*)
    for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
    {
      IdPix[IND1(Ipix)]= IdPixAglo[IND1(Ipix)];
    }

    // NbPro = NbPro - 1
    *NbPro= *NbPro - 1;

fin:
  // Fin de la fonction GestionProtoPost
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction GestionProtoPost\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN GestionProtoPost ==========================================================
// ===============================================================================



//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_DistPixProto
//@
//@ objet :   Distances entre les valeurs des pixels de l'imagette AVHRR
//@           et les valeurs d'un prototype donne
//@
//@
//@ Parametres :
//@         (ENTREE) ImgAv ...... : Vecteur de radiance de l'image AVHRR
//@         (ENTREE) NbChannel .. : Nombre de canaux AVHRR a traiter
//@         (ENTREE) NbPixAv .... : Nombre de pixels du vecteur
//@         (ENTREE) Proto ...... : Prototypes de classes
//@
//@         (SORTIE) Dist ....... : Distances des pixels aux prototypes
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_DistPixProto(
  // Entrees
  float  *ImgAv,      // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
  int     NbChannel,  // int(1)
  int     NbPixAv,    // int(1)
  double *Proto,      // double( SD_BAS_NBK)
  // Sorties
  double *Dist        // double( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
)
{
  // Distance quadratiques entre les valeurs des pixels de
  // l'imagette AVHRR et la valeur d'un prototype donne
  static const char tcProcName_F[] = "SD_ALG_DistPixProto";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  int    Ipix;        // Index sur les pixels
  int    Ich;         // Index sur les canaux
  int    TailleAvhrr; // Taille image AVHRR
  double Diff;

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction DistPixProto\n----------------------------------\n");
  #endif

  TailleAvhrr= SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR;

  // Boucle sur les pixels (Ipix = 1 a NbPixAv)
  for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
  {
    Dist[IND1(Ipix)]= 0.0;
    for( Ich= 1; Ich <= NbChannel; Ich++)
    {
      Diff= ImgAv[IND2(Ipix,Ich,TailleAvhrr)] - Proto[IND1(Ich)];
      Dist[IND1(Ipix)]= Dist[IND1(Ipix)] + Diff*Diff;
    }
  }  // Fin de boucle sur les pixels AVHRR

fin:
  // Fin de la fonction DistPixProto
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction DistPixProto\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN DistPixProto ==============================================================
// ===============================================================================



//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_DistPixProtoClas
//@
//@ objet :   Distances entre les valeurs des pixels d'une classe et
//@           les valeurs du prototype de cette classe
//@
//@
//@ Parametres :
//@         (ENTREE) ImgAv ...... : Vecteur de radiance de l'image AVHRR
//@         (ENTREE) NbChannel .. : Nombre de canaux AVHRR a traiter
//@         (ENTREE) NbPixAv .... : Nombre de pixels du vecteur
//@         (ENTREE) IdPix ...... : Flags de pixels valides
//@         (ENTREE) IdPro ...... : Numero de prototype
//@         (ENTREE) Proto ...... : Prototypes de classes
//@
//@         (SORTIE) DistClass .. : Distances des pixels aux prototypes
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_DistPixProtoClas(
  // Entrees
  float  *ImgAv,      // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
  int     NbChannel,  // int(1)
  int     NbPixAv,    // int(1)
  int    *IdPix,      // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  int     IdPro,      // int(1)
  double *Proto,      // double( SD_BAS_NBK)
  // Sorties
  double *DistClas    // double( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NCL_MAX)
)
{
  // Distance quadratiques entre les valeurs des pixels de
  // l'imagette AVHRR et la valeur d'un prototype donne
  static const char tcProcName_F[] = "SD_ALG_DistPixProtoClas";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  int    Ipix;        // Index sur les pixels
  int    Ich;         // Index sur les canaux
  int    IndClas;     // Index sur les classes
  int    TailleAvhrr; // Taille image AVHRR
  double Diff;

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction DistPixProtoClas\n----------------------------------\n");
  #endif

  TailleAvhrr= SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR;

  //MOR 22/08/2003  DistClas a une dimension ici 

  // Boucle sur les pixels (Ipix = 1 a NbPixAv)
  for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
  {
    DistClas[IND1(Ipix)]= 0.0;
    if( IdPix[IND1(Ipix)] == IdPro)
    {
      for( Ich= 1; Ich <= NbChannel; Ich++)
      {
        Diff= ImgAv[IND2(Ipix,Ich,TailleAvhrr)] - Proto[IND1(Ich)];
        DistClas[IND1(Ipix)]=   DistClas[IND1(Ipix)] + Diff*Diff;
      }
    }
  }  // Fin de boucle sur les pixels AVHRR

fin:
  // Fin de la fonction DistPixProtoClas
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction DistPixProtoClas\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN DistPixProtoClas ==========================================================
// ===============================================================================



//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_MinDistAgloLoc
//@
//@ objet :  Recherche la valeur minimale de la matrice DistAglo
//@          ainsi que les 2 indices correspondants 
//@
//@
//@ Parametres :
//@         (ENTREE) DistAglo .. : Matrice des distances inter-prototypes
//@         (ENTREE) NbPro ..... : Nombre de prototypes
  // Sorties
//@         (SORTIE) DistMin ... : Distance minimale
//@         (SORTIE) Ind1 ...... : Indice colonne
//@         (SORTIE) Ind2 ...... : Indice ligne
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_MinDistAgloLoc(
  // Entrees
  double *DistAglo,    // double( SD_BAS_NCL_MAX * SD_BAS_NCL_MAX)
  int     NbPro,       // int(1)
  // Sorties
  double *DistMin,     // double(1)
  int    *Ind1,        // int(1)
  int    *Ind2         // int(1)
)
{
  // Recherche la valeur minimale de la matrice DistAglo
  // ainsi que les 2 indices correspondants 
  static const char tcProcName_F[] = "SD_ALG_MinDistAgloLoc";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  int Ipro, Jpro;
  int temp;  
  #ifdef OPS_DEBUG
    printf( "Debut de la fonction MinDistAgloLoc\n----------------------------------\n");
  #endif

  // ($) = (1 a NbPro) nombre de prototypes
  // DistMin = Min{ DistAglo($,$)}
  // [ Ind1, Ind2 ] = indices de Min{ DistAglo($,$)}

  *DistMin= MAX_DOUBLE;
  *Ind1= 0;
  *Ind2= 0;
  for( Jpro= 1; Jpro <= NbPro; Jpro++)
  {
    for( Ipro= 1; Ipro <= NbPro; Ipro++)
    {
      if( DistAglo[IND2(Ipro,Jpro,SD_BAS_NCL_MAX)] < *DistMin)
      {
        *DistMin= DistAglo[IND2(Ipro,Jpro,SD_BAS_NCL_MAX)];
        *Ind1= Ipro;
        *Ind2= Jpro;
      }
    }
  }
// 5.0 PM 3
// si ind 1 > ind2 on echange ind 1 et ind2
if (*Ind1 > *Ind2)
  {
	temp=*Ind1;
        *Ind1= *Ind2;
        *Ind2= temp;
  }

fin:
  // Fin de la fonction MinDistAgloLoc
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction MinDistAgloLoc\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN MinDistAgloLoc ============================================================
// ===============================================================================


//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_LinkPixProto
//@
//@ objet :  Attribution des pixels aux differentes classes definies.
//@          Calcul du nombre de pixels par classe, de la taille de
//@          chaque classe et de la taille globale 
//@
//@ Parametres :
//@         (ENTREE) Dist ...... : Matrice des distances inter-prototypes
//@         (ENTREE) DistMax ... : Seuil de validite en distance
//@         (ENTREE) NbPixAv ... : Nombre de pixels du vecteur AVHRR
//@         (ENTREE) NbPro ..... : Nombre de prototypes
//@
//@         (SORTIE) IdPix ..... : Flags de pixels valides
//@         (SORTIE) NbPixClas . : Nombre pixels par classe
//@         (SORTIE) SumDistClas : Somme des distances intra-classes (Taille des classes)
//@         (SORTIE) SumDistTot  : Somme des distances (Taille de l'ensemble des classes)
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_LinkPixProto(
  // Entrees
  double *Dist,         // double( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NCL_MAX)
  double  DistMax,      // double(1)
  int     NbPixAv,      // int(1)
  int     NbPro,        // int(1)
  // Sorties
  int    *IdPix,        // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  int    *NbPixClas,    // int( SD_BAS_NCL_MAX)
  double *SumDistClas,  // double( SD_BAS_NCL_MAX)
  double *SumDistTot    // double(1)
)
{
  // Attribution des pixels aux differentes classes definies.
  // Calcul de nombre de pixels par classe, de la taille de chaque
  // classe et de la taille globale 
  static const char tcProcName_F[] = "SD_ALG_LinkPixProto";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  int    Ipro;           // Index sur les prototypes
  int    Ipix;           // Index sur les pixels
  double MinDist;        // Distance minimale
  int    TailleAvhrr;    // Taille d'une image AVHRR
  int    Id;             // Index sur les pixels

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction LinkPixProto\n----------------------------------\n");
    printf( "DistMax= %f\n", DistMax);
    printf( "NbPixAv= %d\n", NbPixAv);
    printf( "NbPro= %d\n",   NbPro);
  #endif

  TailleAvhrr= SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR;

  // Initialisation
  // NbPixClas(1 a NbPro+1) = 0
  // SumDistClas(1 a NbPro+1) =  0
  for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
  {
    NbPixClas[IND1(Ipro)]= 0;
    SumDistClas[IND1(Ipro)]= 0.0;
  }


  // Boucle sur les pixels (Ipix = 1 a NbPixAv)
  for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
  {
    // Recherche du numero de prototype associe au pixel
    // Id = indice de Min{Dist(1 a NbPro, Ipix)}
    MinDist= MAX_DOUBLE;
    Id= 0;
    for( Ipro= 1; Ipro <= NbPro; Ipro++)
    {
      if( Dist[IND2(Ipix,Ipro,TailleAvhrr)] < MinDist)
      {
        MinDist= Dist[IND2(Ipix,Ipro,TailleAvhrr)];
        Id= Ipro;
      }
    }

    // Si Dist(Id,Ipix) <= DistMax alors (attribution OK)
    if( Dist[IND2(Ipix,Id,TailleAvhrr)] <= DistMax)
    {
      // IdPix(Ipix) = Id
      IdPix[IND1(Ipix)]= Id;

      // SumDistClas(Id) = SumDistClas(Id) + Dist(Id,Ipix)
      SumDistClas[IND1(Id)]= SumDistClas[IND1(Id)] + Dist[IND2(Ipix,Id,TailleAvhrr)];
    }
    else
    {
      // Sinon (attribution " non classe ")
      //IdPix(Ipix) = NbPro +1
      Id= NbPro + 1;

      IdPix[IND1(Ipix)]= NbPro + 1;
    } // Fin de si (Dist(Id,Ipix) DistMax)

    // NbPixClas(Id) = NbPixClas(Id) +1
    NbPixClas[IND1(Id)]= NbPixClas[IND1(Id)] + 1;

  } // Fin de boucle sur les pixels

  // Calcul de la somme des distances intra-classes
  // SumDistTot = somme(...)
  *SumDistTot= 0.0;
  for( Ipro= 1; Ipro <= NbPro; Ipro++)
    *SumDistTot= *SumDistTot + SumDistClas[IND1(Ipro)];


fin:
  // Fin de la fonction LinkPixProto
  #ifdef OPS_DEBUG
    printf( "      NbPixClas($) : ");
    for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
    {
      printf( "%7d", NbPixClas[IND1(Ipro)]);
    }
    printf( "\n");
    printf( "NbPixClas(NbPro+1) : %7d\n", NbPixClas[IND1(NbPro+1)]);
    printf( "    SumDistClas($) : ");
    for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
    {
      printf( "%12.5f", SumDistClas[IND1(Ipro)]);
    }
    printf( "\n");
    printf( "SumDistTot= %f\n", *SumDistTot);
    printf( "Fin de la fonction LinkPixProto\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN LinkPixProto ==============================================================
// ===============================================================================


//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_SortClas
//@
//@ objet :  Rangement des classes par ordre decroissant de la valeur
//@          du prototype sur le canal de classement
//@
//@ Parametres :
//@         (ENTREE) SortingChannel : Canal de rangement
//@         (ENTREE) NbChannel .... : Nombre de canaux a traiter
//@         (ENTREE) NbPixAv ...... : Nombre de pixels du vecteur
//@         (ENTREE) Proto ........ : Prototypes de classes
//@
//@         (SORTIE) IdPix ........ : Flags de pixels valides
//@         (SORTIE) NbPixClas .... : Nombre pixels par classe
//@         (SORTIE) SumDistClas .. : Somme des distances intra-classes (Taille des classes)
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_SortClas(
    // Entrees
    int     SortingChannel,  // int(1)
    int     NbChannel,       // int(1)
    int     NbPixAv,         // int(1)
    int     NbPro,           // int(1)
    // Entrees / Sorties
    double *Proto,           // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
    int    *IdPix,           // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
    int    *NbPixClas,       // int( SD_BAS_NCL_MAX)
    double *SumDistClas      // double( SD_BAS_NCL_MAX)
)
{
  static const char tcProcName_F[] = "SD_ALG_SortClas";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  double ProtoWork[SD_BAS_NBK*SD_BAS_NCL_MAX];      // double( SD_BAS_NBK * SD_BAS_NCL_MAX)
  int    Ich;                                   // Index sur les canaux
  int    Ipix;                                  // Index sur les pixels
  int    Ipro;                                  // Index sur les prototypes
  int    Jpro;                                  // Index sur les prototypes
  int    IndexSort[SD_BAS_NCL_MAX];                 // int( SD_BAS_NCL_MAX)
  int    IndexSortWork[SD_BAS_NCL_MAX];             // int( SD_BAS_NCL_MAX)
  int    NbPixClasWork[SD_BAS_NCL_MAX];             // int( SD_BAS_NCL_MAX)
  double SumDistClasWork[SD_BAS_NCL_MAX];           // double( SD_BAS_NCL_MAX)
  int    Id[SD_BAS_NCL_MAX];                        // int( SD_BAS_NCL_MAX)
  int    Ind;                                   // Index general


  #ifdef OPS_DEBUG
    int  IproTr;           // Index sur les prototypes
    int  JproTr;           // Index sur les prototypes
    printf( "Debut de la fonction SortClas\n----------------------------------\n");
    printf( "SortingChannel : %d\n", SortingChannel);
    printf( "     NbChannel : %d\n", NbChannel);
    printf( "       NbPixAv : %d\n", NbPixAv);
    printf( "         NbPro : %d\n", NbPro);
  #endif

  // Initialisation
  for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
  {
    // (#) = (1 a NbChannel) nombre de canaux
    for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
    {
      // ProtoWork(#,1 a NbPro) = Proto(#,1 a NbPro)
      ProtoWork[IND2(Ich,Ipro,SD_BAS_NBK)]= Proto[IND2(Ich,Ipro,SD_BAS_NBK)];
    }

    // IndexSort(1 a NbPro) = 0
    IndexSort[IND1(Ipro)]= 0;
    IndexSortWork[IND1(Ipro)]= 1;

    // NbPixClasWork(1 a NbPro) = NbPixClas(1 a NbPro)
    NbPixClasWork[IND1(Ipro)]= NbPixClas[IND1(Ipro)];

    // SumDistClasWork(1 a NbPro) = SumDistClas(1 a NbPro)
    SumDistClasWork[IND1(Ipro)]= SumDistClas[IND1(Ipro)];
  }

  #ifdef OPS_DEBUG
    printf( "Variables de travail : ----------------------------------\n");
    for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
    {
      printf( "  ProtoWork(Ich=%d,$) : ", Ich);
      for( Ipro= 1; Ipro <= SD_BAS_NCL_MAX; Ipro++)
      {
        printf( "%12.5f", ProtoWork[IND2(Ich,Ipro,SD_BAS_NBK)]);
      }
      printf( "\n");
    }

    printf( "      IndexSort : ");
    for( Ipro= 1; Ipro <= NbPro; Ipro++)
      printf( "%12d ", IndexSort[IND1(Ipro)]);
    printf( "\n");

    printf( "  NbPixClasWork : ");
    for( Ipro= 1; Ipro <= NbPro; Ipro++)
      printf( "%12d ", NbPixClasWork[IND1(Ipro)]);
    printf( "\n");

    printf( "SumDistClasWork : ");
    for( Ipro= 1; Ipro <= NbPro; Ipro++)
      printf( "%12.5f ", SumDistClasWork[IND1(Ipro)]);
    printf( "\n");
    printf( "---------------------------------------------------------\n");
  #endif

// OK JUSQU'ICI

  // Boucle sur les prototypes (Ipro = 1 a NbPro)
  for( Ipro= 1; Ipro <= NbPro; Ipro++)
  {
    #ifdef OPS_DEBUG
      printf( "Debut boucle Ipro ==================================\n");
      printf( "Ipro=%d => Id : ", Ipro);
      for( IproTr= 1; IproTr <= SD_BAS_NCL_MAX; IproTr++)
      {
        printf( "%4d ", Id[IND1(IproTr)]);
      }
      printf( "\n");
      printf( "   IndexSort : ");
      for( IproTr= 1; IproTr <= SD_BAS_NCL_MAX; IproTr++)
        printf( "%4d ", IndexSort[IND1(IproTr)]);
      printf( "\n");
    #endif

    // Id(Ipro) = 0
    Id[IND1(Ipro)]= 0;

    // Boucle sur les prototypes (Jpro = 1 a NbPro)
    for( Jpro= 1; Jpro <= NbPro; Jpro++)
    {
      // Si ProtoWork(SortingChannel,Ipro) >= ProtoWork(SortingChannel,Jpro) alors 
      if(    ProtoWork[IND2(SortingChannel,Ipro,SD_BAS_NBK)]
          >= ProtoWork[IND2(SortingChannel,Jpro,SD_BAS_NBK)])
      {
        // Id(Ipro) = Id(Ipro) +1
        Id[IND1(Ipro)]= Id[IND1(Ipro)] + 1;
      } // Fin de si
    }   // Fin de boucle sur les prototypes 

    #ifdef OPS_DEBUG
      printf( "Apres boucle Jpro ==================================\n");
      printf( "Ipro=%d => Id : ", Ipro);
      for( IproTr= 1; IproTr <= SD_BAS_NCL_MAX; IproTr++)
      {
        printf( "%4d ", Id[IND1(IproTr)]);
      }
      printf( "\n");
    #endif

    // Jpro = Id(Ipro)
    Jpro= Id[IND1(Ipro)];

    // Traitement des ex-aequo
    // Tant que IndexSort(Jpro) =/= 0 et Jpro < NbPro 
    while( (IndexSort[IND1(Jpro)] != 0) && (Jpro < NbPro))
    {
      // IndexSort(Jpro) = Ipro
      Jpro = Jpro+1;
    } // Fin de Tant que
    IndexSort[IND1(Jpro)]= Ipro;

    #ifdef OPS_DEBUG
      printf("Jpro:%d\n",Jpro);
      printf( "   IndexSort : ");
      for( IproTr= 1; IproTr <= SD_BAS_NCL_MAX; IproTr++)
        printf( "%4d ", IndexSort[IND1(IproTr)]);
      printf( "\n");
      printf( "Fin   boucle Ipro ==================================\n");
    #endif
  }   // Fin de boucle sur les prototypes

  // Boucle sur les prototypes (Ipro = 1 a NbPro)
  for( Ipro= 1; Ipro <= NbPro; Ipro++)
  {
    // Ind = IndexSort(NbPro -Ipro -1)
    Ind= IndexSort[IND1(NbPro-Ipro+1)];

    // IndexSortWork(Ind) = Ipro
    IndexSortWork[IND1(Ind)]= Ipro;

    // NbPixClas(Ipro) = NbPixClasWork(Ind)
    NbPixClas[IND1(Ipro)]= NbPixClasWork[IND1(Ind)];

    // SumDistClas(Ipro) = SumDistClasWork(Ind)
    SumDistClas[IND1(Ipro)]= SumDistClasWork[IND1(Ind)];

    // Proto(#,Ipro) = ProtoWork(#,Ind)
    //jlp 2905 correction : on travaille sur le nombre de canaux reels
    for( Ich= 1; Ich <= NbChannel; Ich++)
      Proto[IND2(Ich,Ipro,SD_BAS_NBK)]= ProtoWork[IND2(Ich,Ind,SD_BAS_NBK)];
  } // Fin de boucle sur les prototypes

  #ifdef OPS_DEBUG
    printf( "Apres arrangement ==================================\n");
    for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
    {
      printf( "      Proto(Ich=%d,$) : ", Ich);
      for( IproTr= 1; IproTr <= SD_BAS_NCL_MAX; IproTr++)
      {
        printf( "%12.5f", Proto[IND2(Ich,IproTr,SD_BAS_NBK)]);
      }
      printf( "\n");
    }
  #endif

  // IndexSortWork(NbPro+1) = NbPro+1
  IndexSortWork[IND1(NbPro+1)]= NbPro + 1;

  // Boucle sur les pixels (Ipix = 1 a NbPixAv)
  for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
  {
    IdPix[IND1(Ipix)]= IndexSortWork[IND1(IdPix[IND1(Ipix)])];
  }
// VERRUE
  #ifdef OPS_DEBUG
    printf( "Apres boucle sur les pixels ==================================\n");
    for( Ipix= 1; Ipix <= 20 /*NbPixAv*/; Ipix++)
    {
      printf( "IdPix[%d]= %d\n", Ipix, IdPix[IND1(Ipix)]);
    }
  #endif

fin:
  // Fin de la fonction SortClas
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction SortClas\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN SortClas ==================================================================
// ===============================================================================


//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_StatClas
//@
//@ objet :  Calcul du prototype (valeur moyenne intra-classe) et
//@          de la matrice de covariance d'une classe. 
//@
//@ Parametres :
//@         (ENTREE) ProduitCourant : Pour pointer sur les variables temporaires
//@         (ENTREE) ImgAv ........ : Vecteur de radiance de l'image AVHRR
//@         (ENTREE) IdPix ........ : Flags de pixels valides
//@         (ENTREE) NbChannel .... : Nombre de canaux AVHRR a traiter
//@         (ENTREE) NbPixAv ...... : Nombre de pixels du vecteur
//@         (ENTREE) IdPro ........ : Numero du prototype a traiter
//@
//@         (SORTIE) Proto ........ : Prototypes de classes
//@         (SORTIE) Covar ........ : Matrice de covariance
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_StatClas(
  // Entrees
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  float                *ImgAv,              // float( (SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR) * SD_BAS_NBK)
  int                  *IdPix,              // int( SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR)
  int                   NbChannel,          // int(1)
  int                   NbPixAv,            // int(1)
  int                   IdPro,              // int(1)
  // Sorties
  double               *Proto,              // double( SD_BAS_NBK)
  double               *Covar               // double( SD_BAS_NBK * SD_BAS_NBK)
)
{
  static const char tcProcName_F[] = "SD_ALG_StatClas";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  int     Ich, Jch;            // Index sur les canaux
  int     Ipix;                // Index sur les pixels
  double  MoyI, MoyJ, MoyIJ;   // Moyennes
  int     TailleAvhrr;         // Taille d'une image AVHRR
  float  *ImgIJ;               // Vecteur de produit de pixels

  // Gestion de erreurs
  char *msg_err = (char*) NULL;                     // Message d'erreur
  int   num_lig = 0;                     // Ligne d'erreur
  double MoyIJtemp[SD_BAS_NBK*SD_BAS_NBK];
  // Calcul du prototype (valeur moyenne intra-classe) et de la matrice de covariance d'une classe.

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction StatClas\n----------------------------------\n");
  #endif

  TailleAvhrr= SD_BAS_NPI_AVHRR * SD_BAS_NLI_MAX_UTIL_AVHRR;

  ImgIJ= ProduitCourant->ImgIJ;

  // Mise a zero de la matrice de covariance
  for( Ich= 1; Ich <= SD_BAS_NBK; Ich++)
  {
    for( Jch= 1; Jch <= SD_BAS_NBK; Jch++)
    {
      Covar[IND2(Jch,Ich,SD_BAS_NBK)]= 0.0;
    }
  }

  // Boucle sur les canaux (Ich = 1 a NbChannel)
  for( Ich= 1; Ich <= NbChannel; Ich++)
  {
    // Calcul de la valeur moyenne et des covariances intra-classe,
    // c'est a dire pour tous les pixels appartenant a la classe donnee 

    // Ind= indices pour lesquels IdPix(Ipix) = IdPro
    MoyI= SD_ALG_Moyenne( &ImgAv[IND2(1,Ich,TailleAvhrr)], NbPixAv, IdPix, IdPro);
    Proto[IND1(Ich)]= MoyI;
  }
  for( Ich= 1; Ich <= NbChannel; Ich++)
  {
     MoyI=Proto[IND1(Ich)];
    // Boucle sur les canaux (Jch = 1 a NbChannel)
    for( Jch= 1; Jch <= NbChannel; Jch++)
    {
//      MoyJ= SD_ALG_Moyenne( &ImgAv[IND2(1,Jch,TailleAvhrr)], NbPixAv, IdPix, IdPro);
//optimisation
      MoyJ=Proto[IND1(Jch)];

      // Produit des 2 vecteurs canaux Ich et Jch
      for( Ipix= 1; Ipix <= NbPixAv; Ipix++)
      {
        ImgIJ[IND1(Ipix)]=   ImgAv[IND2(Ipix,Ich,TailleAvhrr)]
                           * ImgAv[IND2(Ipix,Jch,TailleAvhrr)];
      }
     
//optimisation
      if (Jch>=Ich)
      MoyIJtemp[IND2(Ich,Jch,SD_BAS_NBK)]= SD_ALG_Moyenne( ImgIJ, NbPixAv, IdPix, IdPro);
      else
      MoyIJtemp[IND2(Ich,Jch,SD_BAS_NBK)]= MoyIJtemp[IND2(Jch,Ich,SD_BAS_NBK)];
      
      MoyIJ=MoyIJtemp[IND2(Ich,Jch,SD_BAS_NBK)];
     
      // Moyenne du produit / Produit des moyennes
      Covar[IND2(Jch,Ich,SD_BAS_NBK)]= MoyIJ - (MoyI * MoyJ);

    }   // Fin de boucle sur les canaux Jch
  }     // Fin de boucle sur les canaux Ich

fin:
  // Fin de la fonction StatClas
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction StatClas\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN StatClas ==================================================================
// ===============================================================================


//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_Tri_ascendant
//@
//@ objet :   Tri d'un vecteur (Tableau) de reels
//@
//@
//@ Parametres :
//@         (ENTREE) Vecteur ...... : Vecteur de reels
//@         (ENTREE) NbElem ....... : Nombre d'elements du vecteur
//@
//@         (SORTIE) IndicesClasses : Vecteur d'indices classes (selon reels)
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_Tri_ascendant(
     // Entrees
     t_SD_BAS_DataThread* ctx,      // t_SD_BAS_DataThread*
     double *Vecteur,        // double( NbElem)
     int     NbElem,         // int(1)
     // Sorties
     int    *IndicesClasses  // int( NbElem)
)
{
  static const char tcProcName_F[]= "SD_ALG_Tri_ascendant";
  t_CMN__Statut cr_op= CMN__STATUT_OK;

  // Gestion de erreurs
/* bma intialisation */
/*
  char *msg_err;          // Message d'erreur
  int num_lig;            // Ligne d'erreur
*/
  char *msg_err = (char*) NULL;          // Message d'erreur
  int num_lig = 0;            // Ligne d'erreur
/* fin bma */

  int ind;
  type_paire_i_val *paires;

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction Tri_ascendant\n----------------------------------\n");
  #endif

  paires=NULL;
  if( SD_BAS_MacMALLOC( paires, type_paire_i_val, NbElem) == FALSE)
  {
    cr_op=   SD_ALG_AVHRR_ERR_ALLOCATION;
    msg_err= (char *)SD_BAS_MSG_ALLOC;
    num_lig= __LINE__;

    goto fin;
  }

  for( ind= 0; ind < NbElem; ind++)
  {
    paires[ind].i=   ind;
    paires[ind].val= Vecteur[ind];
  }

  #ifdef OPS_DEBUG
    printf( "----------------------------------\n");
    printf( "Indices :\n");
    for( ind= 0; ind < NbElem; ind++)
    {
      printf( "%5d  ", paires[ind].i);
    }
    printf( "\n----------------------------------\n");
    printf( "Valeurs a classer :\n");
    for( ind= 0; ind < NbElem; ind++)
    {
      printf( "%5.1f  ", paires[ind].val);
    }
    printf( "\n----------------------------------\n");
  #endif

  qsort( (void *)paires, NbElem, sizeof( type_paire_i_val), SD_ALG_sort_paire_i_val);

  #ifdef OPS_DEBUG
    printf( "----------------------------------\n");
    printf( "Indices :\n");
    for( ind= 0; ind < NbElem; ind++)
    {
      printf( "%5d  ", paires[ind].i);
    }
    printf( "\n----------------------------------\n");
    printf( "Valeurs classees :\n");
    for( ind= 0; ind < NbElem; ind++)
    {
      printf( "%5.1f  ", paires[ind].val);
    }
    printf( "\n----------------------------------\n");
  #endif

  for( ind= 0; ind < NbElem; ind++)
  {
    IndicesClasses[ind]= paires[ind].i + 1;
  }


fin:
  SD_BAS_MacFREE( paires);
  if ( cr_op != CMN__STATUT_OK )
    {
        // mise a jour de la variable ctx->cr.errinterne
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
        SD_BAS_ERR_SET_AVHRR,
        cr_op,
        tcProcName_F,
        num_lig,
        msg_err);

        // on imprime le message d'erreur sur la sortie standard
        #ifdef OPS_DEBUG
            SD_BAS_PRINT_ERR_INT(ctx->cr.errinterne.nomfonction,
                                 ctx->cr.errinterne.numligne,
                                 ctx->cr.errinterne.libelle);
        #endif
    }
  // Fin de la fonction Tri_ascendant
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction Tri_ascendant\n----------------------------------\n");
  #endif

  return( cr_op);
}
// FIN Tri_ascendant =============================================================
// ===============================================================================


//@ ------------------------------------------------------------------*/
//@ Function : SD_ALG_MoyenneLocale
//@
//@ objet :  Valeur moyenne sur une grille 3x3 autour du pixel specifie
//@
//@
//@ Parametres :
//@         (ENTREE) ImageIIS : Image
//@         (ENTREE) lin .... : Coordonnee ligne du pixel
//@         (ENTREE) col .... : Coordonnee colonne du pixel
//@         (ENTREE) NbLin .. : Nombre de lignes
//@         (ENTREE) NbCol .. : Nombre de colonnes
//@
//@ ------------------------------------------------------------------*/
double SD_ALG_MoyenneLocale(
  double *ImageIIS,   // double( NbCol * NbLin)
  int     lin,        // int(1)
  int     col,        // int(1)
  int     NbLin,      // int(1)
  int     NbCol       // int(1)
)
{
  static const char tcProcName_F[] = "SD_ALG_MoyenneLocale";
  double Moyenne= 0.0;
  int    NbPix= 0;
  int    i;

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction MoyenneLocale\n----------------------------------\n");
  #endif

  if( (col > 1) && (lin > 1))
  { // Pixel 1
    Moyenne= Moyenne + ImageIIS[IND2(col-1,lin-1,NbCol)];
    NbPix++;
  }
  if( lin > 1)
  { // Pixel 2
    Moyenne= Moyenne + ImageIIS[IND2(col,lin-1,NbCol)];
    NbPix++;
  }
  if( (col < NbCol) && (lin > 1))
  { // Pixel 3
    Moyenne= Moyenne + ImageIIS[IND2(col+1,lin-1,NbCol)];
    NbPix++;
  }
  if( col < NbCol)
  { // Pixel 4
    Moyenne= Moyenne + ImageIIS[IND2(col+1,lin,NbCol)];
    NbPix++;
  }
  // Pixel 5 (Centre)
  Moyenne= Moyenne + ImageIIS[IND2(col,lin,NbCol)];
  NbPix++;
  if( (col < NbCol) && (lin < NbLin))
  { // Pixel 6
    Moyenne= Moyenne + ImageIIS[IND2(col+1,lin+1,NbCol)];
    NbPix++;
  }
  if( lin < NbCol)
  { // Pixel 7
    Moyenne= Moyenne + ImageIIS[IND2(col,lin+1,NbCol)];
    NbPix++;
  }
  if( (col > 1) && (lin < NbLin))
  { // Pixel 8
    Moyenne= Moyenne + ImageIIS[IND2(col-1,lin+1,NbCol)];
    NbPix++;
  }
  if( col > 1)
  { // Pixel 9
    Moyenne= Moyenne + ImageIIS[IND2(col-1,lin,NbCol)];
    NbPix++;
  }

  if( NbPix > 0)   Moyenne= Moyenne / (double)NbPix;

  // Fin de la fonction MoyenneLocale
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction MoyenneLocale\n----------------------------------\n");
  #endif

  return( Moyenne);
}
// FIN MoyenneLocale =============================================================
// ===============================================================================


//@ ------------------------------------------------------------------*/
//@ Function : SD_ALG_VarianceLocale
//@
//@ objet :  Variance sur une grille 3x3 autour du pixel specifie
//@          a moyenne donnee.
//@
//@
//@ Parametres :
//@         (ENTREE) ImageIIS : Image
//@         (ENTREE) lin .... : Coordonnee ligne du pixel
//@         (ENTREE) col .... : Coordonnee colonne du pixel
//@         (ENTREE) NbLin .. : Nombre de lignes
//@         (ENTREE) NbCol .. : Nombre de colonnes
//@         (ENTREE) Moy .... : Moyenne de la grille 3x3
//@
//@ ------------------------------------------------------------------*/
double SD_ALG_VarianceLocale(
  double   *ImageIIS,   // double( NbCol * NbLin)
  int       lin,        // int(1)
  int       col,        // int(1)
  int       NbLin,      // int(1)
  int       NbCol,      // int(1)
  double    Moy         // double(1)
)
{
  static const char tcProcName_F[] = "SD_ALG_VarianceLocale";
  double Variance= 0.0;
  double diff;
  int    NbPix= 0;
  int    i;

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction VarianceLocale\n----------------------------------\n");
  #endif

  if( (col > 1) && (lin > 1))
  { // Pixel 1
    diff= ImageIIS[IND2(col-1,lin-1,NbCol)] - Moy;
    Variance= Variance + diff * diff;
    NbPix++;
  }
  if( lin > 1)
  { // Pixel 2
    diff= ImageIIS[IND2(col,lin-1,NbCol)] - Moy;
    Variance= Variance + diff * diff;
    NbPix++;
  }
  if( (col < NbCol) && (lin > 1))
  { // Pixel 3
    diff= ImageIIS[IND2(col+1,lin-1,NbCol)] - Moy;
    Variance= Variance + diff * diff;
    NbPix++;
  }
  if( col < NbCol)
  { // Pixel 4
    diff= ImageIIS[IND2(col+1,lin,NbCol)] - Moy;
    Variance= Variance + diff * diff;
    NbPix++;
  }
  // Pixel 5
  diff= ImageIIS[IND2(col,lin,NbCol)] - Moy;
  Variance= Variance + diff * diff;
  NbPix++;
  if( (col < NbCol) && (lin < NbLin))
  { // Pixel 6
    diff= ImageIIS[IND2(col+1,lin+1,NbCol)] - Moy;
    Variance= Variance + diff * diff;
    NbPix++;
  }
  if( lin < NbCol)
  { // Pixel 7
    diff= ImageIIS[IND2(col,lin+1,NbCol)] - Moy;
    Variance= Variance + diff * diff;
    NbPix++;
  }
  if( (col > 1) && (lin < NbLin))
  { // Pixel 8
    diff= ImageIIS[IND2(col-1,lin+1,NbCol)] - Moy;
    Variance= Variance + diff * diff;
    NbPix++;
  }
  if( col > 1)
  { // Pixel 9
    diff= ImageIIS[IND2(col-1,lin,NbCol)] - Moy;
    Variance= Variance + diff * diff;
    NbPix++;
  }

  if( NbPix > 0)   Variance= Variance / (double)NbPix;

  // Fin de la fonction VarianceLocale
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction VarianceLocale\n----------------------------------\n");
  #endif

  return( Variance);
}
// FIN VarianceLocale ============================================================
// ===============================================================================


//@ ------------------------------------------------------------------*/
//@ Function : SD_ALG_Moyenne
//@
//@ objet :  Moyenne des pixels de ImgAv de classe IdPro
//@
//@
//@ Parametres :
//@         (ENTREE) ImgAv ...... : Vecteur de radiance de l'image AVHRR
//@         (ENTREE) NbPixAv .... : Nombre de pixels du vecteur
//@         (ENTREE) IdPix ...... : Flags de pixels valides
//@         (ENTREE) IdPro ...... : Numero du prototype a traiter
//@
//@ ------------------------------------------------------------------*/
double SD_ALG_Moyenne(
       // Entrees
       float *ImgAv,     // float( NbPixAv)
       int    NbPixAv,   // int(1)
       int   *IdPix,     // int( NbPixAv)
       int    IdPro      // int(1)
)
{
  static const char tcProcName_F[]= "SD_ALG_Moyenne";
  double Moy= 0.0;
  int    NbPix= 0;
  int    i;

  #ifdef OPS_DEBUG2
    printf( "Debut de la fonction Moyenne\n----------------------------------\n");
  #endif

  for( i= 1; i <= NbPixAv; i++)
  {
    if( IdPix[IND1(i)] == IdPro)
    {
      Moy= Moy + (double)ImgAv[IND1(i)];
      NbPix++;
    }
  }
  
  if( NbPix > 0)  Moy= Moy / (double)NbPix;

  // Fin de la fonction Moyenne
  #ifdef OPS_DEBUG2
    printf( "Fin de la fonction Moyenne\n----------------------------------\n");
  #endif

  return( Moy);
}
// FIN Moyenne ===================================================================
// ===============================================================================



//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_TransformIpsf_CCS
//@
//@ Description : calcule la valeur du poids de l'IPSF sondeur correspondant
//@ a chaque pixel d'imageur.
//@
//@
//@ Parameters :
//@         (ENTREE) NbLin ........ : Nombre de lignes des IPSF
//@         (ENTREE) NbCol ........ : Nombre de colonnes des IPSF
//@         (ENTREE) YPsf ......... : Positions des points IPSF dans le repere IASI nominal instrument selon Y
//@         (ENTREE) ZPsf ......... : Positions des points IPSF dans le repere IASI nominal instrument selon Z
//@         (ENTREE) WgtPsf ....... : Poids associes aux points des IPSF
//@         (ENTREE) LinDeb ....... : Premiere ligne de l'image IPSF/Imageur
//@         (ENTREE) ColDeb ....... : Premiere colonne de l'image IPSF/Imageur
//@         (ENTREE) LinFin ....... : Derniere ligne de l'image IPSF/Imageur
//@         (ENTREE) ColFin ....... : Derniere colonne de l'image IPSF/Imageur
//@         (ENTREE) ImgY ......... : Image de colonnes selon Y
//@         (ENTREE) ImgZ ......... : Image de colonnes selon Z
//@         (ENTREE) DY ........... : Dimension des pixels imageur selon Y
//@         (ENTREE) DZ ........... : Dimension des pixels imageur selon Z
//@         (ENTREE) OffsetSond ... : Offset entre le sondeur IASI et l'imageur
//@         (ENTREE) OverSampFactor : Facteur de sur-echantillonnage de l'IPSF
//@
//@         (SORTIE) ImgWgtpsf .... : Poids associes aux points des IPSF/Imageur
//@         (SORTIE) ImgYpsf ...... : Positions des points IPSF/Imageur dans le repere IASI nominal instrument selon Y
//@         (SORTIE) ImgZpsf ...... : Positions des points IPSF/Imageur dans le repere IASI nominal instrument selon Z
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_TransformIpsf_CCS(
  // Entrees
  int                  NbLin,             // int(1)
  int                  NbCol,             // int(1)
  double              *YPsf,              // double( SD_BAS_NB_COL_SOND) Limite a NbCol
  double              *ZPsf,              // double( SD_BAS_NB_LIN_SOND) Limite a NbLin
  double              *WgtPsf,            // double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  int                  LinDeb,            // int(1)
  int                  ColDeb,            // int(1)
  int                  LinFin,            // int(1)
  int                  ColFin,            // int(1)
  int          		Mc,		   // nb col ImgY et ImgZ,
  double              *ImgY,              // double( NbLinAvFov * NbColAvFov)
  double              *ImgZ,              // double( NbLinAvFov * NbColAvFov)
  double               DY,                // double(1)
  double               DZ,                // double(1)
  double              *OffsetSond,        // double( SD_BAS_DEUX)
  double               OverSampFactor,    // double(1)
  // Sorties
  double              *ImgWgtpsf,         // double( NbLinAvFov * NbColAvFov)
  double              *ImgYpsf,           // double( NbLinAvFov * NbColAvFov)  Alloue: double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
  double              *ImgZpsf            // double( NbLinAvFov * NbColAvFov)  Alloue: double( SD_BAS_NB_COL_SOND * SD_BAS_NB_LIN_SOND)
)
{
  t_CMN__Statut cr_op=CMN__STATUT_OK;

  int     CpsfDeb;
  int     CpsfFin;
  int     LpsfDeb;
  int     LpsfFin;
  int     NbColpsf;
  double  SurfPixImg;
  double  SumWgtpsf;
  double  SurfSamp;
  double  Zspl;
  double  Yspl;
  double  DYpsf;
  double  DZpsf;
  double  Norm;
  double Ymin,Ymax,Zmin,Zmax;
  double Csty,Cstz;
  double Invnorm;
  //jlp200
  double  Zsplinit;
  double  Ysplinit;
  double  InvDYpsf;
  double  InvDZpsf;

  // Compteurs de boucle
  int     il, ic;
  int     jl, jc;
  int     kl, kc;
  int     ct;


  #ifdef OPS_DEBUG
    printf( "Debut de la fonction transform IPSF\n----------------------------------\n");
  #endif

  // Calcul des dimensions (DYpsf, DZpsf) associee aux points de discretisation de l'Ipsf :
  // * Hypothese de grille Ipsf reguliere
  //DYpsf = Ypsf(2) - Ypsf(1)
  DYpsf= YPsf[IND1(2)] - YPsf[IND1(1)];
  Csty=DYpsf/OverSampFactor;
  // DZpsf = Zpsf(2) - Zpsf(1)
  DZpsf= ZPsf[IND1(2)] - ZPsf[IND1(1)];
  Cstz=DZpsf/OverSampFactor;

  // Initialisationd de la normalisation
  // Norm = 0
  Norm= 0.0;

  // Application de l'offset sondeur/imageur
  // ImgY(il,ic) = IISY(il,ic) + OffsetSondIIS(1)
  // ImgZ(il,ic) = IISZ(il,ic) + OffsetSondIIS(2)
  ct= 0;
  for( il= LinDeb; il <= LinFin; il++)
  {
    for( ic= ColDeb; ic <= ColFin; ic++)
    {
//      ImgYpsf[ct]= ImgY[ct] + OffsetSond[IND1(1)];
//      ImgZpsf[ct]= ImgZ[ct] + OffsetSond[IND1(2)];
// 5.0      
//  on se ramene a un vecteur commencent a 0 ( cas iis)
      ImgYpsf[ct]= ImgY[IND2(ic,il,Mc)] - OffsetSond[IND1(1)];
      ImgZpsf[ct]= ImgZ[IND2(ic,il,Mc)] - OffsetSond[IND1(2)];
      ct= ct + 1;
    }
  }

  // Boucle sur les lignes des pixels imageur : il = LinDeb a LinFin
  InvDYpsf=1/DYpsf;
  InvDZpsf=1/DZpsf;
  ct= 0;
  for( il= LinDeb; il <= LinFin; il++)
  {
    // Boucle sur les colonnes des pixels imageur : ic = ColDeb a ColFin
    for(ic=ColDeb; ic<=ColFin; ic++)
    {

      // Calcul du domaine Ipsf associe au pixel imageur (domaine utile)
      // * Hypothese pixel imageur rectangulaire
      // CpsfDeb = max{1 + floor[(ImgY(il,ic)-abs(ImgDY)/2-Ypsf(1)) / DYpsf], 1 }
      CpsfDeb= ( MAX( 1.0,
                          1.0 + floor( (ImgYpsf[ct] - fabs(DY)*0.5 - YPsf[0]) *InvDYpsf)));

      // CpsfFin  = min{1 + ceil[(ImgY(il,ic)+abs(ImgDY)/2-Ypsf(1)) / DYpsf], NbCol }
      CpsfFin= ( MIN( NbCol,
                          1.0 + ceil(  (ImgYpsf[ct] + fabs(DY)*0.5 - YPsf[0]) *InvDYpsf)));

      // LpsfDeb = max{1+ floor[(ImgZ(il,ic)-abs(ImgDZ)/2-Zpsf(1)) / DZpsf], 1 }
      LpsfDeb= ( MAX( 1.0,
                          1.0 + floor( (ImgZpsf[ct] - fabs(DZ)*0.5 - ZPsf[0]) *InvDZpsf)));

      // LpsfFin  = min{1+ ceil[(ImgZ(il,ic)+abs(ImgDZ)/2-Zpsf(1)) / DZpsf], NbLin }
      LpsfFin= ( MIN( NbLin,
                          1.0 + ceil(  (ImgZpsf[ct] + fabs(DZ)*0.5 - ZPsf[0]) *InvDZpsf)));

      NbColpsf= CpsfFin - CpsfDeb + 1;

      // Initialisation de l'integration sur le domaine du pixel imageur
      // SurfPixImg = 0 & SumWgtpsf = 0
      SurfPixImg= 0.0;
      SumWgtpsf= 0.0;
      //precalculs
      Ymin = (ImgYpsf[ct] - fabs(DY)*0.5);
      Ymax = (ImgYpsf[ct] + fabs(DY)*0.5);
      Zmin = (ImgZpsf[ct] - fabs(DZ)*0.5);
      Zmax = (ImgZpsf[ct] + fabs(DZ)*0.5);
      // Calcul de la surface et du poids associes aux points du domaine Ipsf utile
      // Boucle sur les lignes du domaine de l'Ipsf utile : jl = LpsfDeb a LpsfFin
      for( jl= LpsfDeb; jl <= LpsfFin; jl++)
      {
        Zsplinit= ZPsf[IND1(jl)]  - (OverSampFactor*0.5 + 0.5) * Cstz;
        // Boucle sur les colonnes du domaine de l'Ipsf utile : jc = CpsfDeb a CpsfFin
        for( jc= CpsfDeb; jc <= CpsfFin; jc++)
        {
          // Initialisation
          SurfSamp= 0.0;
          Ysplinit= YPsf[IND1(jc)]  - (OverSampFactor*0.5 + 0.5) * Csty;
          Zspl= Zsplinit;
          // Boucle sur les lignes du sur-echantillonnage de l'Ipsf (kl = 1 a OverSampFactor)
          for( kl= 1; kl <= OverSampFactor; kl++)
          {
            // Zspl = Zpsf(jl) + [ kl - (OverSampfactor/2+0.5) ] DZpsf / OverSampFactor
            Zspl=  Zspl + Cstz;

            // Si (ImgZ(il,ic) - abs(ImgDZ)/2 <= Zspl <= ImgZ(il,ic) + abs(ImgDZ)/2) alors
            if( (Zmin <= Zspl) && (Zspl <= Zmax))
             {
              Yspl= Ysplinit;
              // Boucle sur les colonnes du sur-echantillonnage de l'Ipsf (kc = 1 a OverSampFactor)
              for( kc= 1; kc <= OverSampFactor; kc++)
              {
                // Yspl = Ypsf(jc) + [ kc - (OverSampfactor/2+0.5) ] DYpsf / OverSampFactor
                Yspl= Yspl + Csty;

                // Si (ImgY(il,ic) - abs(ImgDY)/2 <= Yspl <= ImgY(il,ic) + abs(ImgDY)/2) alors
                if( (Ymin <= Yspl) && (Yspl <= Ymax) )
                {
                  // SurfSamp = SurfSamp +1  
                  SurfSamp= SurfSamp + 1.0;
                } // Fin de si
              } // Fin de boucle kc
            } // Fin de si
          } // Fin de boucle kl
          // SurfPixImg = SurfPixImg + SurfSamp
         SurfPixImg= SurfPixImg + SurfSamp;

          // SumWgtpsf = SumWgtpsf + Wgtpsf (jl,jc)* SurfSamp
         SumWgtpsf= SumWgtpsf + WgtPsf[IND2(jc,jl,SD_BAS_NB_COL_SOND)] * SurfSamp;
//         SumWgtpsf= SumWgtpsf + WgtPsf[IND2(jc,jl,NbColpsf)] * SurfSamp;
        } // Fin de boucle jc
      } // Fin de boucle jl

      // Calcul du poids Ipsf associe a chaque pixel de l'imageur
      // Si SurfPixImg |= 0 alors
      if( !SD_BAS_MacEGAUX_D( SurfPixImg, 0.0))
      {
     // ImgWgtpsf (il,ic) = SumWgtpsf / SurfPixImg
        ImgWgtpsf[ct]= SumWgtpsf / SurfPixImg;
      }
      else
      {
        // ImgWgtpsf (il,ic) =0
        ImgWgtpsf[ct]= 0.0;
      }

      // Norm = Norm + ImgWgtpsf (il,ic)
      Norm= Norm + ImgWgtpsf[ct];

      ct= ct + 1;
    }
  } // Fin des boucles ic et il sur les lignes et les colonnes de l'imageur

  #ifdef OPS_DEBUG
    printf( "   Coefficient de normalisation des poids : %lf\n", Norm);
  #endif

  // Calcul du poids normalise pour tous les il de LinDeb a LinFin et ic de ColDeb a ColFin
  ct= 0;
  Invnorm=1./Norm;
  for( il= LinDeb, ct=0; il<=LinFin; il++)
  {
    for( ic= ColDeb; ic <= ColFin; ic++)
    {
      // ImgWgtpsf(il,ic) = ImgWgtpsf (il,ic)/ Norm
      ImgWgtpsf[ct]= ImgWgtpsf[ct] * Invnorm;
      ct= ct + 1;
    }
  }

  // Fin de la fonction TransformIpsf_CCS
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction transform IPSF\n----------------------------------\n");
  #endif

  return( cr_op);
}
// FIN TransformIpsf_CCS =========================================================
// ===============================================================================




//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_EigenAnalysis
//@
//@ objet :   Calcul et classement des valeurs et vecteurs propres
//@
//@
//@ Parametres :
//@         (ENTREE) Contexte_courant .... : Contexte de l'etape (numero de ligne SN traite)
//@         (ENTREE) Taille .............. : Taille de la matrice carree
//@         (ENTREE) Covar ............... : Matrice de covariance
//@         (ENTREE) Transpose ........... : Flag de transposition de la covariance puis des vecteurs
//@
//@         (SORTIE) EigVal .............. : Valeurs propres ordonnees
//@         (SORTIE) EigVect ............. : Vecteurs propres ordonnes
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_EigenAnalysis(
  // Entrees
  t_SD_BAS_DataThread *Contexte_courant,  // t_SD_BAS_DataThread*
  int                  Taille,            // int(1) Nombre de points (Max = SD_BAS_NBK)
  double              *Covar,             // double( Taille * Taille),
  unsigned int         Transpose,         // unsigned int(1)                  
  // Sorties
/* bma complexes->reels */
  double              *EigVal,            // double( Taille)
  double              *EigVect            // double( Taille * Taille)
/* fin bma */
)
{
  static const char tcProcName_F[] = "SD_ALG_EigenAnalysis";
  t_CMN__Statut cr_op=CMN__STATUT_OK;

  // Gestion de erreurs
/* bma intialisation */
/*
  char            *msg_err;                     // Message d'erreur
  int              num_lig;                     // Ligne d'erreur
*/
  char            *msg_err = (char*) NULL;                     // Message d'erreur
  int              num_lig = 0;                     // Ligne d'erreur
/* fin bma */

  double           auxv[2*SD_BAS_NBK];          // 2*Taille (Cf. Doc ESSL)
  double           tmp;
/* bma complexes->reels */
  dcmplx           CEigVal[SD_BAS_NBK];         // dcmplx( Taille)
  dcmplx           CEigVect[SD_BAS_NBK*SD_BAS_NBK];                    // dcmplx( Taille * Taille)
/* fin bma */
  type_sort_eigen  ValVect[SD_BAS_NBK];         // Structures de tri
  int              i, j;                        // Index generaux

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction EigenAnalysis\n----------------------------------\n");

    // ================================================================================
    // Trace des entrees ==============================================================
    // ================================================================================
    printf( "Taille= %d\n", Taille);
    printf( "Matrice de covariance :\n");
    for( i= 0; i < Taille; i++)
    {
      printf( "|   ");
      for( j= 0; j < Taille; j++)
      {
        printf( "%5.1f   ", Covar[IND2(j+1,i+1,Taille)]);
      }
      printf( "|\n");
    }
  #endif

  if( Transpose == CMN__OUI)
  {
    // La matrice est donnee lig1,col1 ; lig1,col2 ; ... ; lig1,colN
    //                       lig2,col1 ; lig2,col2 ; ... ; lig2,colN
    //                          ...         ...      ...      ...
    //                       ligM,col1 ; ligM,col2 ; ... ; ligM,colN
    // => Transposition de la matrice pour etre en accord avec ESSL
    for( i= 1; i < Taille; i++)  // Lignes
    {
      for( j= 0; j < i; j++)     // Colonnes
      {
        tmp= Covar[i*Taille+j];
        Covar[i*Taille+j]= Covar[j*Taille+i];
        Covar[j*Taille+i]= tmp;
      }
    }
  }

  // Calcul des valeurs et vecteurs propres
  cr_op= SD_BAS_ValVectPropre(
        // Entrees
        Contexte_courant,   // t_SD_BAS_DataThread*
        Taille,             // int(1) Nombre de points
        Covar,              // double( SD_BAS_NBK * SD_BAS_NBK),
        // Sorties
        auxv,               // double( 2*SD_BAS_NBK)
/* bma complexes->reels */
        CEigVal,            // dcmplx( SD_BAS_NBK)
        CEigVect);          // dcmplx( SD_BAS_NBK * SD_BAS_NBK)
/* fin bma */

  if( cr_op != CMN__STATUT_OK)
  {
    cr_op=   SD_ALG_AVHRR_ERR_EIG_VAL_PROCESSING;
    msg_err= (char *)SD_BAS_MSG_ERR_EIG_VAL_PROCESSING;
    num_lig= __LINE__;
    goto fin;
  }

/* bma complexes->reels */
  for( i= 0; i < Taille; i++)
  {
    EigVal[i]= RE( CEigVal[i]);
    for( j= 0; j < Taille; j++)
    {
      /*MOR suppression fabs*/
      EigVect[i*Taille+j]=  RE( CEigVect[i*Taille+j]);
    }
  }

//jlp30 inutile ici de transposer on stocke la matrice en colonne
//  if( Transpose == CMN__OUI)
//  {
//    // Transposition des vecteurs propres
//    for( i= 1; i < Taille; i++)  // Lignes
//    {
//      for( j= 0; j < i; j++)     // Colonnes
//      {
//        tmp= EigVect[i*Taille+j];
//        EigVect[i*Taille+j]= EigVect[j*Taille+i];
//        EigVect[j*Taille+i]= tmp;
//      }
//   }
//  }
/* fin bma */

  #ifdef OPS_DEBUG
    // ================================================================================
    // Trace des sorties des valeurs propres ==========================================
    // ================================================================================
    printf( "Valeurs propres :\n");
    printf( "|   ");
    for( i= 0; i < Taille; i++)
    {
/* bma complexes->reels */
      printf( "%9.6f   ", EigVal[i]);
/* fin bma */
    }
    printf( "|\n");

    printf( "Vecteurs propres :\n");
    for( j= 0; j < Taille; j++)
    {
      printf( "|   ");
      for( i= 0; i < Taille; i++)
      {
/* bma complexes->reels */
        printf( "%9.6f   ", EigVect[IND2(j+1,i+1,Taille)]);
/* fin bma */
      }
      printf( "|\n");
    }
  #endif

  // Classement des valeurs et vecteurs propres

/* bma complexes->reels */
  // Initialisation de la structure (toutes les valeurs propres au max)
  for( i= 0; i < SD_BAS_NBK; i++)   ValVect[i].val= MAX_DOUBLE;

  // Chargement de la structure
//la matrice en sortie est stockee vecteur propre par vecteur propre
  for( j= 0; j < Taille; j++)
  {
    ValVect[j].val= EigVal[j];
    for( i= 0; i < Taille; i++)
    {
      ValVect[j].vect[i]= EigVect[j*Taille+i];
    }
  }
/* fin bma */

  // Classement
  qsort( (void *)ValVect, Taille, sizeof( type_sort_eigen), SD_ALG_sort_eigen_val);

  // Recuperation des valeurs classees
/* bma complexes->reels */
//la matrice en sortie doit  stocker les vecteurs propre en colonne
  for( j= 0; j < Taille; j++)
  {
    EigVal[j]= ValVect[j].val;
    for( i= 0; i < Taille; i++)
    {
      // rangement ( en colonne)
      /* FA135 : correction transposition de la matrice des valeurs propres */
      EigVect[i*Taille+j]= ValVect[j].vect[i];
    }
  }
/* fin bma */
//RMO VERRUE A ENLEVER ( elle sert a remettre les vecteurs propre avec le bon signe
//  for(j= 0; j < Taille; j++)
//    if(j != 2)
//      for( i= 0; i < Taille; i++)
//	EigVect[j*Taille+i]=EigVect[j*Taille+i]*(-1.0);

  #ifdef OPS_DEBUG
    // ================================================================================
    // Trace des sorties des valeurs propres ==========================================
    // ================================================================================
//RMO  verrue
   printf( "Valeurs propres clasees :\n");
    printf( "|   ");
    for( i= 0; i < Taille; i++)
    {
/* bma complexes->reels */
      printf( "%9.6f   ", EigVal[i]);
/* fin bma */
    }
    printf( "|\n");

    printf( "Vecteurs propres classes :\n");
    for( j= 0; j < Taille; j++)
    {
      printf( "|   ");
      for( i= 0; i < Taille; i++)
      {
/* bma complexes->reels */
        printf( "%9.6f   ", EigVect[IND2(j+1,i+1,Taille)]);
/* fin bma */
      }
      printf( "|\n");
    }
  #endif

fin:
  // Fin de la fonction EigenAnalysis
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction EigenAnalysis\n----------------------------------\n");
  #endif

  // sortie de la fonction
  return( cr_op);
}
// FIN EigenAnalysis =============================================================
// ===============================================================================


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_sort_paire_i_val
//@
//@ Description : Tri croissant de paires valeur/indice sur la valeur
//@ But : obtenir la liste des indices correspondant aux valeurs croissantes
//@ ------------------------------------------------------------------*/
int SD_ALG_sort_paire_i_val( const void *a, const void *b)
{ int res;

  res= ((const type_paire_i_val *)a)->val > ((const type_paire_i_val *)b)->val;
  if( res == 0)  res= -1;
  return( res);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_sort_eigen_val
//@
//@ Description : Tri croissant de valeur/vecteurs propres
//@               en fonction des valeurs propres
//@ ------------------------------------------------------------------*/
int SD_ALG_sort_eigen_val( const void *a, const void *b)
{ int res;

/* bma complexes->reels */
  res= ((const type_sort_eigen *)a)->val < ((const type_sort_eigen *)b)->val;
/* fin bma */
  if( res == 0)  res= -1;
  return( res);
}


//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_ConstruireAvhrrChannelState
//@
//@ objet :  Extraction de l'imagette Avhrr utile (domaine de l'IIS)
//@
//@
//@ Parametres :
//@         (ENTREE)  AvhrrListeLin ... : Image AVHRR (Liste de lignes)
//@         (ENTREE)  AvhrrNbLin ...... : Nombre de lignes de l'image AVHRR
//@         (ENTREE)  AvhrrNbChannel .. : Nombre de canaux AVHRR a traiter
//@
//@         (SORTIE)  AvhrrChannelState : Flags de disponibilite des canaux
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_ConstruireAvhrrChannelState(
  // Entrees
  t_SD_BAS_LigneAVHRR  **AvhrrListeLin,      // t_SD_BAS_LigneAVHRR( AvhrrNbLin)
  int                    AvhrrNbLin,         // int(1)
  int                    AvhrrNbChannel,     // int(1)
  // Sorties
  unsigned int          *AvhrrChannelState   // int( SD_BAS_NLI_MAX_UTIL_AVHRR * SD_BAS_NBK)
)
{
  static const char tcProcName_F[] = "SD_ALG_ConstruireAvhrrChannelState";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
/* bma intialisation */
/*
  char         *msg_err;  // Message d'erreur
*/
  char         *msg_err = (char*) NULL;  // Message d'erreur
/* fin bma */
  int           Il;       // Index sur les lignes Avhrr
  int           Nch,can;      // Index sur les canaux
  unsigned int  flags;    // Indicateurs binaires (dont bits de validite des canaux)

// initialisation du tableau
  // Boucle sur les lignes AvhrrUtil ( Il= 1 a AvhrrUtilNbLin )
  for( Il=1; Il <= AvhrrNbLin; Il++)
  {
    for( Nch=1; Nch <= SD_BAS_NBK; Nch++)
    {
      AvhrrChannelState[IND2(Il,Nch,SD_BAS_NLI_MAX_UTIL_AVHRR)]= 0;
    }
   }
  // Boucle sur les lignes AvhrrUtil ( Il= 1 a AvhrrUtilNbLin )
  for( Il=1; Il <= AvhrrNbLin; Il++)
  {
    for( Nch=1; Nch <= AvhrrNbChannel; Nch++)
    {
      // Extraction des flags (ChannelState) de validite pour les AvhrrNbChannel canaux
      flags= (unsigned int) (*(AvhrrListeLin[IND1(Il)]->InfoDet.tab_var[AVHRR_DIGITAL_B_DATA].val.pui4));
      #ifdef OPS_DEBUG
        printf( "Ligne %d => Flags= %8X\n", Il, flags);
      #endif
      switch( Nch)
      {
        case AVHRR1 : flags= (flags & MASK_AVHRR1) > 0; can=1;
                      break;
        case AVHRR2 : flags= (flags & MASK_AVHRR2) > 0;can=2;
                      break;
        case AVHRR3 : if( (flags & MASK_AVHRR3) == AVHRR3a)
                        {flags= (flags & MASK_AVHRR3a) > 0;can=3;}
                      else
                        {flags= (flags & MASK_AVHRR3b) > 0;can=4;}
                      break;
        case AVHRR4 : flags= (flags & MASK_AVHRR4) > 0;can=5;
                      break;
        case AVHRR5 : flags= (flags & MASK_AVHRR5) > 0;can=6;
                      break;
        default:      // Canal non disponible (par exemple 6)
                      cr_op=   SD_ALG_AVHRR_ERR_CHANNEL_UNAVAILABLE;
                      msg_err= (char *)SD_ALG_MSG_ERR_CHANNEL_UNAVAILABLE;
                      goto fin;
      }
      AvhrrChannelState[IND2(Il,can,SD_BAS_NLI_MAX_UTIL_AVHRR)]= flags;
    }
// VERRUE a enlever
/*RMO cet algo ne fonctionnant pas avec le TU, verrue qui force le mode 3A*/
//    AvhrrChannelState[IND2(Il,1,SD_BAS_NLI_MAX_UTIL_AVHRR)]= 1;
//    AvhrrChannelState[IND2(Il,2,SD_BAS_NLI_MAX_UTIL_AVHRR)]= 1;
//    AvhrrChannelState[IND2(Il,3,SD_BAS_NLI_MAX_UTIL_AVHRR)]= 1;
//    AvhrrChannelState[IND2(Il,4,SD_BAS_NLI_MAX_UTIL_AVHRR)]= 0;
//    AvhrrChannelState[IND2(Il,5,SD_BAS_NLI_MAX_UTIL_AVHRR)]= 1;
//    AvhrrChannelState[IND2(Il,6,SD_BAS_NLI_MAX_UTIL_AVHRR)]= 1;
   //FIN

  }
fin:
  return( cr_op);
}
// 5.0 PM 3
//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_Index_De_Max3
//@
//@ objet :  recherche des 3 plus grands elements d'un tableau de valeur
//@
//@
//@ Parametres :
//@         (ENTREE) NbElem .......  : Nombre d'elements du vecteur
//@         (ENTREE) Vecteur ....... : vecteur
//@
//@         (SORTIE) i3 : indice du troisieme element le plus grand
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_OK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_Index_De_Max3(
     // Entrees
     t_SD_BAS_DataThread* ctx,      // t_SD_BAS_DataThread*
     int     NbElem,         // int(1)
     double *Vecteur,        // double( NbElem)
     // sorties
     int     *i3           // int(1)
)
{
  static const char tcProcName_F[]= "SD_ALG_Index_De_Max3";
  t_CMN__Statut cr_op= CMN__STATUT_OK;
  // Gestion de erreurs
/* bma intialisation */
/*
  char *msg_err;          // Message d'erreur
  int num_lig;            // Ligne d'erreur
*/
  char *msg_err = (char*) NULL;          // Message d'erreur
  int num_lig = 0;            // Ligne d'erreur
/* fin bma */

  int ind;
  type_paire_i_val *paires;

  #ifdef OPS_DEBUG
    printf( "Debut de la fonction SD_ALG_Index_De_Max3\n----------------------------------\n");
  #endif
  paires=NULL;
  if( SD_BAS_MacMALLOC( paires, type_paire_i_val, NbElem) == FALSE)
  {
    cr_op=   SD_ALG_AVHRR_ERR_ALLOCATION;
    msg_err= (char *)SD_BAS_MSG_ALLOC;
    num_lig= __LINE__;

    goto fin;
  }
// recopie dans le tableau de valeurs temporaires
  for( ind= 0; ind < NbElem; ind++)
  {
    paires[ind].i=   ind;
    paires[ind].val= Vecteur[ind];
  }
// appel de la fonction de tri ascendant 
  qsort( (void *)paires, NbElem, sizeof( type_paire_i_val), SD_ALG_sort_paire_i_val);


//verification du nombre d'elements ( on suppose qu'il est different de 0 )
  if(NbElem < 3 )
  {
// on choisi alors le plus petit element
  if (NbElem == 1 || Vecteur[IND1(1)]< Vecteur[IND1(2)] )
   {
   *i3=1;
   }
  else
   {
   *i3=2;
   }
  }
  else
  {
   *i3= paires[IND1(NbElem-2)].i + 1;
  }
 
  #ifdef OPS_DEBUG
    printf( "----------------------------------\n");
    printf( "i3 =\t");
    printf( "%5d  ",*i3);
    printf( "\n----------------------------------\n");
  #endif



fin:
  SD_BAS_MacFREE( paires);
  if ( cr_op != CMN__STATUT_OK )
    {
        // mise a jour de la variable ctx->cr.errinterne
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
        SD_BAS_ERR_SET_AVHRR,
        cr_op,
        tcProcName_F,
        num_lig,
        msg_err);

        // on imprime le message d'erreur sur la sortie standard
        #ifdef OPS_DEBUG
            SD_BAS_PRINT_ERR_INT(ctx->cr.errinterne.nomfonction,
                                 ctx->cr.errinterne.numligne,
                                 ctx->cr.errinterne.libelle);
        #endif
    }
  // Fin de la fonction SD_ALG_Index_De_Max3
  #ifdef OPS_DEBUG
    printf( "Fin de la fonction SD_ALG_Index_De_Max3\n----------------------------------\n");
  #endif

  return( cr_op);
}
// FIN SD_ALG_Index_De_Max3 =============================================================

/* CR292 */
//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_AVHRR_GetSubSatLatLon
//@
//@ objet :  estime la position subsatellite a partir de la date ENG_GEPSDATIASI
//@
//@
//@ Parametres :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (ENTREE/SORTIE)  StatMphr : Infos globales MPHR
//@         (ENTREE)  Mphr :  MPHR produit N1
//@         (ENTREE)  InfoDet1ENG : info det du produit ENG
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK                    si la fonction a reussi
//@         SD_ALG_AVHRR_GetSubSatLatLon_ALRT si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
// FT-2751
t_CMN__Statut SD_ALG_AVHRR_GetSubSatLatLon(
  t_SD_BAS_DataThread         *ctx,
  t_SD_BAS_ConfAutre          *ConfAutre,
  t_SD_BAS_StatMphr           *StatMphr,
  t_SD_BAS_MPHR               *Mphr,
  t_SD_BAS_Liste_Var          *InfoDet1ENG,
  int                         nombreLignes)
// ...FT-2751
{
  static char        tcProcName_F[] = "SD_ALG_AVHRR_GetSubSatLatLon";
  t_CMN__Statut      cr_op;
  t_CMN__Statut      irc;
  char              *msg_err;
  int                num_lig; // Ligne d'erreur

  t_SD_BAS_SCDSTime  IasiTime;
  int                SN;             // Numero de sous cycle
  int                NbPoints;
  double             PixY;
  double             PixZ;
  double             ScanAngle;
  double             Azim;
  double             Elev;
  double             Lat;
  double             Lon;
  double             Ang_mesY;
  double             Ang_mesZ;
  double             Azim_sol;
  double             Elev_sol;
  double             EarthSatDist;
  double            *MatRot;
  int i;
  
  // Initialisation
  cr_op = CMN__STATUT_OK;
  msg_err = (char*) NULL;
  num_lig = 0;
  SN = ctx->ctxtrt.SN;
  IasiTime = InfoDet1ENG->tab_var[ENG_GEPSDATIASI].val.pdate[IND1(SN)];
  NbPoints = 1;
  PixY = 0.0;
  PixZ = 0.0;
//  PixY = InfoDet1ENG->tab_var[ENG_GCCSOFFSETSONDAVHRR].val.pr8[IND3(1,i,SN,SD_BAS_CCD,SD_BAS_PN)]*M_PI/180.;
//  PixZ = InfoDet1ENG->tab_var[ENG_GCCSOFFSETSONDAVHRR].val.pr8[IND3(2,i,SN,SD_BAS_CCD,SD_BAS_PN)]*M_PI/180.;
  ScanAngle = 0.0;
  Azim = 0.0;
  Elev = 0.0;
  Lat = 0.0;
  Lon = 0.0;
  Ang_mesY = 0.0;
  Ang_mesZ = 0.0;
  Azim_sol = 0.0;
  Elev_sol = 0.0;
  EarthSatDist = 0.0;
  MatRot = ConfAutre->IDefIASIMETOPMatRot;
  
  // Corps de la fonction
  SD_ALG_GeomIasiMetop(
    MatRot,
    ScanAngle,
    PixY,
    PixZ,
    &Azim, 
    &Elev); 
 
  // Appel a la librairie Metop pour calculer les coordonnees Lat/Lon du point
  // subsatellite
  SD_BAS_ReserverOperateur();
    #ifdef OPS_DEBUG
        printf("pb call SD_BAS_MetopGeod from ALG_AVHRR line %d  (un seul point)\n",__LINE__);
        printf("pb   azim = %lf\n",Azim);
        printf("pb   elev = %lf\n",Elev);
    #endif

  irc = SD_BAS_MetopGeod(
    ctx,
    &IasiTime,
    Mphr,
    NbPoints,
    &Azim,
    &Elev,
    &Lat,
    &Lon,
    &Ang_mesY,
    &Ang_mesZ,
    &Azim_sol,
    &Elev_sol,
    &EarthSatDist);

  SD_BAS_LibererOperateur();
    #ifdef OPS_DEBUG
    if(irc != CMN__STATUT_OK)
    {
        printf("pb  erreur SD_BAS_MetopGeod => Code retour %d\n",irc);
    }
    #endif

  if(irc != CMN__STATUT_OK)
  {
    cr_op   = SD_ALG_AVHRR_GetSubSatLatLon_ALRT;
    msg_err = (char *)SD_BAS_MSG_ERR_METOP2GEOD;
    num_lig = __LINE__;
    goto fin;
  }

  // FT-2751
  if (ctx->ctxtrt.LN == 1)
  // ...FT-2751
  {
    StatMphr->subsat_latitude_start = Lat*180/M_PI;
    StatMphr->subsat_longitude_start = Lon*180/M_PI;
  }
  
  // FT-2751
  if (ctx->ctxtrt.LN == nombreLignes)
  {
    StatMphr->subsat_latitude_end = Lat*180/M_PI;
    StatMphr->subsat_longitude_end = Lon*180/M_PI;
  }
  // ...FT-2751
  
fin:
  // Traitement des erreurs
  //================================================================
  if ( cr_op != CMN__STATUT_OK )
  {
    // mise a jour de la variable ctx->cr.errinterne
    SD_BAS_TEST_ERR_INT(
      ctx->cr.errinterne,
      SD_BAS_ERR_SET_AVHRR,
      cr_op,
      tcProcName_F,
      num_lig,
      msg_err);

    CreerMessage (
      SD_ALG_AVHRR_SET, 
      MSG_ALG_AVHRR_44_GEO_ALRT, 
      &(ctx->cr.msgJDB));

    // on imprime le message d'erreur sur la sortie standard
#ifdef OPS_DEBUG
    SD_BAS_PRINT_ERR_INT(
      ctx->cr.errinterne.nomfonction,
      ctx->cr.errinterne.numligne,
      ctx->cr.errinterne.libelle);
#endif
  }

// sortie de la fonction
#ifdef OPS_DEBUG
     printf("Fin de la fonction SD_ALG_AVHRR_GetSubSatLatLon\n========================================\n");
#endif

return(cr_op);
}
/* FIN CR292 */
// ===============================================================================

