static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/CMN__Trace.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : CMN
//@ MODULE : CMN__Trace
//@ 
//@ Commentaire :
//@    Ce module gere les fonctionnalites de trace dans un fichier
//@    Les procedures disponibles ici sont utilisees par les macros 
//@    de traces TRINIT,TR0,TR1,TR2,TR3,TR4,TRFIN
//@    Ce module gere un fichier de trace qui peut etre circulaire et
//@    qui comporte la datation de la trace, la trace elle-meme et des 
//@    informations de localisation de l'appel dans les sources.
//@      Les procedures disponibles sont:
//@         CMN__Trace_Init
//@         CMN__Trace_Tracer
//@         CMN__Trace_Fin
//@         CMN__Trace_Print (interne)
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 10/03/2000 : Pierre-Louis Agnetti
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-184-CN : 18/10/2006 : C.MERIGUET
//@ Codage des fonctions reentrantes
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2766(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS2TEC SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-3358(DM-2100-OPS) : 13/04/2017 : MLA
//@ Pour produits > 2Go, il faut utiliser des long long pour stocker la position du curseur 
//@ et les fonctions fseeko et ftello
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//
#include <stdio.h>
#include <time.h>
// FT-2765
#include <sys/time.h>
// ...FT-2765
#include <sys/timeb.h>
#include <stdarg.h>
#include <unistd.h>
#include <stdlib.h>
#include <pthread.h>

// Projet
#include "CMN__Commun.h"
#include "CMN__Trace.h"


//======================================================================
// Declaration des fonctions internes
//======================================================================

// Taille du buffer de sortie des traces
#define CMN__TRACE_LG_BUFFER 255


//======================================================================
// Declaration des variables globales
//======================================================================

// Niveau des traces
int CMN__Trace_NiveauTrace=0;


// Descripteur fichier des traces
static FILE* CMN__Trace_FichierTrace=NULL;

// Mode de trace circulaire(1) ou non(0)
static int CMN__Trace_ModeTrace=1;

// Taille du fichier circulaire
static int CMN__Trace_TailleCirculaire=500;

// Trace des information fichier et numero de ligne a faire (1) ou pas (0)
static int CMN__Trace_InformationSource=0;


//======================================================================
//					FONCTIONS EXTERNES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Trace_Init
//@
//@ Description : Initialise les parametres de gestion de traces
//@               avec les parametres de la ligne de lancement
//@               du processus.
//@               Si argc=0 les traces seront inactives.
//@               Exploite par getopt les parametres argv et traduit 
//@                -n x par niveau de trace = x (defaut=0)
//@                -t fichier_trace fichier de trace (defaut=
//@                   Trace<pid_process>)
//@                -c traces non circulaires (defaut=oui)
//@                -l ligne du fichier circulaire (defaut=500)
//@                -s trace du source et de la ligne (defaut=NON)
//@
//@ Parametres :
//@               (ENTREE) argc : nombre de parametres
//@               (ENTREE) argv : tableau des parametres
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__Trace_Init(int argc,char **argv)
{
  int    flag='d'; // caractere d'option
  char    fichier_trace[CMN__LG_PATH_UNIX]; // fichier trace
  
  //@ Positionner le nom du fichier de trace par defaut Trace<pid>
  sprintf(fichier_trace,"Trace%d",getpid());
  
  //@ Exploiter les parametres de lancement
  while (flag>=0)
    {
      opterr=0;
      flag = getopt(argc, argv, "n:t:cl:s");
      //@ SELON l'argument passe en parametre
      switch (flag)
	{
	case 'n':
	  //@ CAS 'n': mettre le niveau de trace
	  CMN__Trace_NiveauTrace=atoi(optarg);
	  break;
	case 't':
	  //@ CAS 't': mettre le nom du fichier de trace
	  sprintf(fichier_trace,optarg);
	  break;
	case 'c':
	  //@ CAS 'c': positionner le flag fichier non circulaire
	  CMN__Trace_ModeTrace=0;
	  break;
	case 'l':
	  //@ CAS 'l': mettre le nombre de ligne si on a un fichier
	  //@          circulaire
	  CMN__Trace_TailleCirculaire=atoi(optarg);
	  break;
	case 's':
	  //@ CAS 's': positionner le flag de trace des informations
	  //@          de source
	  CMN__Trace_InformationSource=1;
	  break;
	case '?':
	  optind++;
	  break;
	default:
	  optind--;
	  flag=-1;
	  break;
	}
      //@ FIN SELON
    }
  
  //@ SI le niveau de trace est different de 0
  if (CMN__Trace_NiveauTrace)
    {
      //@ ouvrir le fichier de trace
      CMN__Trace_FichierTrace=fopen(fichier_trace,"w+");
      
      //@ tracer dans le fichier de trace les options
      TR1(0,"Fichier de trace   :%s",fichier_trace);
      TR1(0,"Niveau de trace    :%d",CMN__Trace_NiveauTrace);
      TR1(0,"Fichier circulaire :%s",CMN__Trace_ModeTrace==1?"OUI":"NON");
      if (CMN__Trace_ModeTrace==1)
	{
	  TR1(0,"Lignes circulaires :%d",CMN__Trace_TailleCirculaire);
	}
      TR1(0,"Trace source       :%s",CMN__Trace_InformationSource==1?"OUI":"NON");
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Trace_Fin
//@
//@ Description : Termine la getion des traces en fermant le fichier 
//@               de trace
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__Trace_Fin()
{
   //@ Fermer le fichier de trace si il a ete ouvert
   if (CMN__Trace_FichierTrace)
   {
      fclose(CMN__Trace_FichierTrace);
      CMN__Trace_FichierTrace=NULL;
   }
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Trace_Tracer
//@
//@ Description : Trace dans le fichier de trace les informations 
//@               passees. Les parametres sont :
//@                - le premier est le nom du fichier source de la
//@                  trace,
//@                - le deuxieme est la ligne dans le fichier source,
//@                - le troisieme est la chaine a tracer ou le format 
//@                  de sortie et dans ce cas on a des parametres pour 
//@                  le format a la suite
//@                - le dernier parametre est 0 pour fermer les 
//@                  parametres variable.
//@
//@               La trace est faite dans le fichier avec le format
//@               suivant: 
//@                 YY/MM/JJ hh:mm:ss trace applicative
//@                 fichier:nom_du_fichier ligne:numero_de_ligne
//@                  (si l'option source est demandee)
//@
//@ Parametres :
//@               (ENTREE) fichier_source : fichier C ou C++ appelant 
//@                                         la trace
//@               (ENTREE) numero_ligne : ligne dans le fichier C ou
//@                                       C++ appelant la trace
//@               (ENTREE) ligne : ligne a tracer
//@               (ENTREE) n*parametres : parametres pour le format 
//@                                       (printf like)
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__Trace_Tracer(const char *fichier_source,const int numero_ligne,const char *format, ...)
{
   va_list ap; // parametres variables
   char format_avec_date[CMN__TRACE_LG_BUFFER]; // format pour date
   char buffer_de_trace[CMN__TRACE_LG_BUFFER]; // buffer de travail
   struct timeb	t;
   struct timeval time_val; // traitement de la datation
   struct timezone time_zone; // traitement de la datation
   char szDate[50];
   unsigned short milisecondes = 0;
   struct tm* ptmTemp = NULL;
   pid_t	ThreadId=0;

   format_avec_date[0]=0;


   //@ Recuperer le pid du process
   ThreadId = getpid();
   
   //@ Initialisation de la gestion des parametres variables
   va_start(ap,format);
   
   //@ SI le niveau de trace est positif et SI un fichier de trace
   //@ est ouvert
   if ((CMN__Trace_NiveauTrace>0)&&(CMN__Trace_FichierTrace))
     {
       //@ recuperer la date courante
       ftime(&t);      
       
       //@ recuperer la date courante en milisecondes
       gettimeofday(&time_val,&time_zone);
       milisecondes=time_val.tv_usec/1000;

       //@ convertir en date locale
/* FA 184 */
       struct tm * CurrentTime;
       ptmTemp = localtime_r(&t.time,CurrentTime);
/*.. FA 184 */
       //@ convertir la date en chaine de caractere
       strftime(szDate,sizeof(szDate),"%Y/%m/%d\t%H:%M:%S",ptmTemp);
       
       
       //@ SI l'information source et ligne est demandee, la mettre
       //@ dans le format avec la date
       //@ SINON ne mettre que la date
       //@ FIN SI
       
       if (CMN__Trace_InformationSource)
	 {
	   sprintf(format_avec_date,
		   "%s.%03d\tsrc:%-20s\tlig:%04d\tthread:%d\t%s",
		   szDate,milisecondes,
		   fichier_source,numero_ligne,ThreadId,format);
	 }
       else
	 {
	   sprintf(format_avec_date,
		   "%s.%03d\t%s",
		   szDate,milisecondes,
		   format);
	 }
       
       //@ Appliquer le format sur les parametres passes
       vsprintf(buffer_de_trace,
		format_avec_date,ap);
      
       //@ Imprimer dans le fichier de traces
       CMN__Trace_Print(buffer_de_trace);
     }
   //@ FIN SI
   
   //@ Fin de gestion des parametres variables
   va_end(ap);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Trace_malloc
//@
//@ Description : Permet de tracer et de faire un malloc
//@
//@ Parametres :
//@               (ENTREE) size : taille du malloc
//@               (ENTREE) source : source appellant
//@               (ENTREE) ligne : ligne dans le source
//@
//@ Valeur de retour :
//@               Retour du malloc
//@
//@ ------------------------------------------------------------------*/
void *CMN__Trace_malloc(size_t size,const char *source,int ligne)
{
   void *lemalloc=NULL; // pointeur de malloc
   
   lemalloc=malloc(size);
   
   TR4(9,"TRACE_MALLOC 0x%x size %d source %s ligne %d",lemalloc,size,source,ligne);
   
   return lemalloc;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Trace_realloc
//@
//@ Description : Permet de tracer et de faire un realloc
//@
//@ Parametres :
//@               (ENTREE) ptr : adresse existante
//@               (ENTREE) size : taille du malloc total
//@               (ENTREE) source : source appellant
//@               (ENTREE) ligne : ligne dans le source
//@
//@ Valeur de retour :
//@               Retour du realloc
//@
//@ ------------------------------------------------------------------*/
void *CMN__Trace_realloc(void *ptr,size_t size,const char *source,int ligne)
{
   void *lemalloc=NULL; // pointeur de malloc
   
   TR3(9,"TRACE_FREE 0x%x source %s ligne %d",ptr,source,ligne);
   lemalloc=realloc(ptr,size);
   TR4(9,"TRACE_MALLOC 0x%x size %d source %s ligne %d",lemalloc,size,source,ligne);
   
   return lemalloc;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Trace_free
//@
//@ Description : Permet de tracer et de faire un free
//@
//@ Parametres :
//@               (ENTREE) adresse a liberer
//@               (ENTREE) source appellant
//@               (ENTREE) ligne dans le source
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__Trace_free(void *lemalloc,const char *source,int ligne)
{
   
   TR3(9,"TRACE_FREE 0x%x source %s ligne %d",lemalloc,source,ligne);
   free(lemalloc);
   
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Trace_Print
//@
//@ Description : Gere l'interface avec le fichier de trace
//@               pour la gestion du fichier circulaire
//@               Si la gestion n'est pas circulaire, sauver la ligne
//@               et c'est tout
//@               Si s'est du circulaire gerer la position de la ligne
//@               du fichier pour faire un repositionnement en debut
//@               de fichier quand on arrive a la ligne
//@               CMN__Trace_TailleCirculaire
//@
//@ Parametres :
//@               (ENTREE) buffer : buffer a sauver dans le fichier
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__Trace_Print(const char *buffer)
{
  // Position dans le fichier pour gerer le circulaire
  static int PositionDansLeFichier=0; 

  // Format d'ecriture
  char format[30];
   
   
  //@ SI on fonctionne en circulaire voir si on arrive pas a la borne 
  //@ pour faire un retour au debut de fichier
  if (CMN__Trace_ModeTrace==1)
   {
      PositionDansLeFichier++;
      if (PositionDansLeFichier==CMN__Trace_TailleCirculaire) 
      {
         PositionDansLeFichier=0;
         fseeko(CMN__Trace_FichierTrace,0,0);
      }
   }
  //@ FIN SI
   
  //@ Formater la chaine a tracer a un maximun de 255 caracteres
  sprintf(format,"%%-%ds\n",CMN__TRACE_LG_BUFFER);

  //@ et ecrire dans le fichier
  fprintf(CMN__Trace_FichierTrace,format,buffer);
  
  //@ Flusher le fichier
  fflush(CMN__Trace_FichierTrace);
}


//======================================================================
//					FONCTIONS INTERNES 
//======================================================================
