static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/CMN__Version.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : CMN
//@ MODULE : CMN__Version
//@ 
//@ Commentaire :
//@    Ce module permet de gerer la version du logiciel :
//@       -  CMN__Version_Trace
//@       -  CMN__Version_Valeur
//@       -  CMN__Version_Majeur
//@       -  CMN__Version_Mineur
//@                                     
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 20/09/1999 : Pierre-Louis Agnetti
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Adaptation a l'OPS : ajout des fonctions CMN__Version_Majeur et
//@                      CMN__Version_Mineur
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-126-CN : 01/09/2004 : P.CABANE
//@ Reading of the software has been updated
//@ fiche de correction correspondante : IA-2100-FC-126-THA
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//
#include <stdio.h>

#include <unistd.h>
#include <stdlib.h>

#include "CMN__Version.h"
#include "CMN__VersionComment.h"


//======================================================================
// Declaration des fonctions internes
//======================================================================


//======================================================================
// Declaration des variables globales
//======================================================================

// Version projet du logiciel
//static char s_CMN__Version[] = "V1-0";

/* FA126 */
#define FORMAT_VERSION    "%c%d-%d"
static const int NB_FORMAT_VERSION = 3;
/* FIN FA126 */


//======================================================================
//                                      FONCTIONS EXTERNES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Version_Trace
//@
//@ Description : Exploite par getopt les parametres argv et sur -v
//@               affiche le numero de version et sort du process
//@
//@ Parametres :
//@               (ENTREE) argc : Nombre de parametres
//@               (ENTREE) argv : Tableau des parametres
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__Version_Trace(int argc,char **argv)
{
  // Caractere d'option
  int flag = 'd';
  
  
  //@ Exploiter les parametres de lancement
  while (flag >= 0)
    {
      opterr = 0;
      flag   = getopt(argc, argv, "v");

      switch (flag)
        {
        case 'v':
          //@ SI option 'v', tracer la version et sortir
          printf("Version de %s:%s\n", argv[0], CMN__Version_Valeur());
          exit(0);
          break;

        default:
          optind--;
          flag = -1;
          break;
        }
    }
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Version_Valeur
//@
//@ Description : Retourne la valeur de la version du logiciel
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Pointeur sur la chaine version
//@
//@ ------------------------------------------------------------------*/
char *CMN__Version_Valeur()
{
  //@ Retourner la chaine version
   return s_CMN__Version;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Version_Majeur
//@
//@ Description : Retourne l'indice majeur de la version du logiciel
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Indice majeur de la version
//@
//@ ------------------------------------------------------------------*/
int CMN__Version_Majeur()
{
  /* FA126 */
  char type   = '\0';
  int  majeur = 0;
  int  mineur = 0;
  int  status = 0;


  //@ Recuperer le type, les indices majeur et mineur de la version
  status = sscanf(s_CMN__Version, FORMAT_VERSION, &type, &majeur, &mineur);
  if (status != NB_FORMAT_VERSION)
  {
    majeur = 0;
  }
  /* FIN FA126 */

  //@ Retourner l'indice majeur de la version
  return majeur;
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Version_Mineur
//@
//@ Description : Retourne l'indice mineur de la version du logiciel
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Indice mineur de la version
//@
//@ ------------------------------------------------------------------*/
int CMN__Version_Mineur()
{
  /* FA126 */
  char type   = '\0';
  int  majeur = 0;
  int  mineur = 0;
  int  status = 0;

  //@ Recuperer le type, les indices majeur et mineur de la version
  status = sscanf(s_CMN__Version, FORMAT_VERSION, &type, &majeur, &mineur);
  if (status != NB_FORMAT_VERSION)
  {
    mineur = 0;
  }
  /* FIN FA126 */

  //@ Retourner l'indice mineur de la version
  return mineur;
}


//======================================================================
//                                      FONCTIONS INTERNES
//======================================================================
