static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/TAC/src/SD_TAC_TacheTraitement.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_TAC
//@ CLASSE : SD_TAC_TacheTraitement
//@ 
//@ Commentaire :
//@    Ce module gere la classe SD_TAC_TacheTraitement
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Adaptation a l'OPS : ajout des methodes de simulation
//@ --------------------------------------------------------------------
//@ FAID<SMM-M4-FA-84-CN>    21/01/00    GRT Modification du construction 
//@                                      du libelle a journaliser
//@ --------------------------------------------------------------------
//@ FAID<SMM-M4-FA-61-CN>    27/01/00    Exploitation du statut d'action
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-2366-CN 02/10/01 GRT L'attribut dernier fichier importe n'a plus d'utilite
//@                                Il est remplace par la liste des derniers fichiers importes
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-2334-CN 02/10/01 GRT Le NomDuFichierResultat doit etre reinitialise entre 
//@                                chaque fichier importe ou exporte
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-2555-CN 12/01/02 GRT On utilise des traces collectives
//@ --------------------------------------------------------------------
//@ SMM-M4-FA-2323-CN 01/02/02 GRT on attend une seconde entre deux fichiers pour
//@                                conserver la chronologie
//@ --------------------------------------------------------------------
//@ VERSION : 30/08/1999 : SAS
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL
#include <iostream>
#include <string>

// Standard
#include <math.h> 
#include <fcntl.h>
#include <time.h>
#include <sys/times.h>
#include <sys/time.h>

// Projet
#include "CMN__Commun.h"
#include "CMN_DAT_Evenement.h"
#include "SD_FRW_ApplicationServeurDonnees.h"
#include "SD_TAC_TacheTraitement.h"


//
// Definition des constantes
//
const t_Chaine SD_TAC_TacheTraitement::LIBELLE_DEBUT_TACHE = "Debut de la tache: ";
const t_Chaine SD_TAC_TacheTraitement::LIBELLE_FIN_TACHE   = "Fin de la tache: ";
const t_Chaine SD_TAC_TacheTraitement::LIBELLE_ARRET_TACHE = "### Terminaison (incomplete) de la tache: ";


/*--------------------------------------------------------------------*/
/* MAXIMUM : maximum entre 2 valeurs                                    */
/*--------------------------------------------------------------------*/
#define MAXIMUM(x,y) ((x) > (y) ? (x) : (y))
#define MINIMUM(x,y) ((x) < (y) ? (x) : (y))


//======================================================================
//                                      METHODES PUBLIQUES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : SD_TAC_TacheTraitement
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_TAC_TacheTraitement::SD_TAC_TacheTraitement()
{
}

//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_TAC_TacheTraitement
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
SD_TAC_TacheTraitement::~SD_TAC_TacheTraitement()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : Executer
//@
//@ Description : Traitement principal de la tache
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_TAC_TacheTraitement::Executer()
{
  int                            noActionCourante = 1;
  list<SD_ACT_Action*>::iterator i;
  t_CMN__Booleen                 arreter = CMN__FAUX;


#ifdef OPS_DEBUG
  cout << endl <<
    "------------------------------------------------------";
  cout << endl << "TACHE (" << m_NomTache << "): execution en cours...";
  cout << endl << "\tType         : " << m_TypeTache;
  cout << endl << "\tSimulation   : " << m_Simulation;
  cout << endl << "\tDate creation: " << m_DateCreation.Chaine();
  cout << endl << "\tDate debut   : " << m_DateDebut.Chaine();
  cout << endl << "\tDate fin     : " << m_DateFin.Chaine();

  cout << endl << "\tContexte     : " << m_Contexte->NomTravail(); 
  cout << endl << "\t\tNom travail  : " << m_Contexte->NomTravail(); 
  cout << endl << "\t\tIndice ligne : " << m_Contexte->IndiceLigne();  
  cout << endl << "\t\tNb ressources: " << m_Contexte->NbRessources();  
  cout << endl << "\t\tSignal temps : " 
       << (m_Contexte->SignalerTemps() == CMN__VRAI)?"OUI":"NON";
  cout << endl <<
    "------------------------------------------------------" << endl;
#endif


  //@ SI mode simulation
  if (m_Simulation != -1)
    {
      //@ SI le temps est superieur a zero
      if (m_Simulation > 0)
        {
          cout << "Tache " << m_NomTache << ": attente de " 
               << m_Simulation << " secondes." << endl;
          
          //@ Attente de: temps de simulation secondes
          sleep(m_Simulation);
        }
      //@ SINON
      else
        {
          cout << "Tache " << m_NomTache << ": attente de 1 seconde." 
               << endl;

          //@ Attente d'un temps par defaut: 1 seconde
          sleep(1);
        }
      //@ FIN SI

      m_EstTerminee = CMN__VRAI;
    }
  //@ SINON
  else
    {
      //@ Initialiser l'action courante dans le contexte de tache
      //@ L'action courante est la premiere action par defaut
      i = m_Actions.begin();
      if (Contexte() != NULL) 
        {
          Contexte()->ActionCourante(noActionCourante);
        }
      
      
      //@ TANT QUE il y a des actions a realiser ET que
      //@ l'action precedente s'est correctement deroulee ET
      //@ que la tache ne doit pas etre interrompue FAIRE
      do 
        {
          //@ Executer l'action courante
          (*i)->TracerDebut();
          (*i)->Executer();
          (*i)->TracerFin();
          
          //@ SI l'action s'est bien terminee
          if ((*i)->PeutEtreEnchainee() == CMN__VRAI )
            {
              //@ Mettre a jour le contexte de tache pour la 
              //@ prochaine action courante

#ifdef OPS_DEBUG
              cout << "Tache " << m_NomTache 
                   << ": action correctement terminee: " 
                   << (*i)->NomAction() << endl;
#endif

              noActionCourante++;
              Contexte()->ActionCourante(noActionCourante);
            } 
          //@ SINON
          else
            {
#ifdef OPS_DEBUG
              cout << "Tache " << m_NomTache << ": action en erreur: " 
                   << (*i)->NomAction() << endl;
#endif
              
              //@ Arret de la tache
              arreter = CMN__VRAI;
            }
          //@ FIN SI
          
          //@ Verifier si une condition de terminaison est demandee
          //@ SI une condition de terminaison est demandee ALORS
          if (VerifierConditionArret() == CMN__VRAI) 
            {
              //@ Arret de la tache
              arreter = CMN__VRAI;
            }
          //@ FIN SI
          
          // FA 2323
          sleep(1);
          
          //@ Incrementer l'iterateur d'action
          i++;
        } 
      while ((i != m_Actions.end()) && (arreter == CMN__FAUX));
      //@ FIN TANT QUE
      
      
      //@ Positionner l'etat de terminaison de la tache
      m_EstTerminee = CMN__VRAI;
    }
  //@ FIN SI

#ifdef OPS_DEBUG
  cout << endl << "Tache " << m_NomTache << ": execution terminee..." 
       << endl << endl;
#endif
}


//@ ------------------------------------------------------------------*/
//@ Methode : EstTerminee
//@
//@ Description : Indique si la tache est terminee
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               CMN__VRAI si la tache est terminee
//@               CMN__FAUX sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Booleen SD_TAC_TacheTraitement::EstTerminee()
{
  return m_EstTerminee;
}


//@ ------------------------------------------------------------------*/
//@ Methode : TracerDemarrage
//@
//@ Description : Trace le demarrage de la tache de traitement
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void SD_TAC_TacheTraitement::TracerDemarrage()
{
  CMN_DAT_Evenement evtDemarrage;
  t_Chaine          libelle(LIBELLE_DEBUT_TACHE);


  //@ Verrouiller l'application
  Application()->Verrouiller();

  //@ Acceder au journal de bord et journaliser le demarrage
  evtDemarrage.Identifiant(Application()->AllouerChronoEvenement());
  evtDemarrage.TypeEvenement(SD__Message::MSG_SD_TAC_BEGIN);
  evtDemarrage.Fonction(m_NomTache);
  libelle += m_NomTache;
  evtDemarrage.Libelle(libelle);
  evtDemarrage.Prm1(m_NomTache);

  Application()->Evenements()->JournaliserEvenement(evtDemarrage);

  //@ Deverrouiller l'application
  Application()->Deverrouiller();
}


//@ ------------------------------------------------------------------*/
//@ Methode : TracerFin
//@
//@ Description : Trace la terminaison de la tache de traitement
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour : 
//@               Aucune
//@ ------------------------------------------------------------------*/
void SD_TAC_TacheTraitement::TracerFin()
{
  CMN_DAT_Evenement evtTerminaison;
  t_Chaine          libelle("");  


  //@ Verrouiller l'application
  Application()->Verrouiller();

  //@ Acceder au journal de bord et journaliser la terminaison
  evtTerminaison.Identifiant(Application()->AllouerChronoEvenement());
  evtTerminaison.Fonction(m_NomTache);
  if (m_EstTerminee == CMN__VRAI) 
    {
      evtTerminaison.TypeEvenement(SD__Message::MSG_SD_TAC_END);
      libelle = LIBELLE_FIN_TACHE; 
    }
  else
    {
      evtTerminaison.TypeEvenement(SD__Message::MSG_SD_TAC_ERROR);
      libelle = LIBELLE_ARRET_TACHE;
    } 
  libelle += m_NomTache; 
  evtTerminaison.Libelle(libelle);
  evtTerminaison.Prm1(m_NomTache);

  Application()->Evenements()->JournaliserEvenement(evtTerminaison);

  //@ Deverrouiller l'application
  Application()->Deverrouiller();
}


//======================================================================
//                                      METHODES PRIVEES
//======================================================================
